//===-------------------------- test_demangle.cpp -------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

// The demangler does not pass all these tests with the system dylibs on macOS.
// XFAIL: use_system_cxx_lib && target={{.+}}-apple-macosx10.{{9|10|11|12|13|14|15}}

#include "support/timer.h"
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cxxabi.h>
#include <string>

// Is long double fp80?  (Only x87 extended double has 64-bit mantissa)
#define LDBL_FP80 (__LDBL_MANT_DIG__ == 64)
// Is long double fp128?
#define LDBL_FP128 (__LDBL_MANT_DIG__ == 113)

const char* cases[][2] =
{
    {"_Z1A", "A"},
    {"_Z1Av", "A()"},
    {"_Z1A1B1C", "A(B, C)"},
    {"_Z4testI1A1BE1Cv", "C test<A, B>()"},
    {"_Z4testI1A1BET0_T_S3_", "B test<A, B>(A, A)"},
    {"_ZN1SgtEi", "S::operator>(int)"},
    {"_ZrsI1QEiT_i", "int operator>><Q>(Q, int)"},
    {"_ZN13dyldbootstrap5startEPK12macho_headeriPPKcl", "dyldbootstrap::start(macho_header const*, int, char const**, long)"},
    {"_ZN4dyld17getExecutablePathEv", "dyld::getExecutablePath()"},
    {"_ZN4dyld22mainExecutablePreboundEv", "dyld::mainExecutablePrebound()"},
    {"_ZN4dyld14mainExecutableEv", "dyld::mainExecutable()"},
    {"_ZN4dyld21findImageByMachHeaderEPK11mach_header", "dyld::findImageByMachHeader(mach_header const*)"},
    {"_ZN4dyld26findImageContainingAddressEPKv", "dyld::findImageContainingAddress(void const*)"},
    {"_ZN4dyld17clearErrorMessageEv", "dyld::clearErrorMessage()"},
    {"_ZN4dyld15getErrorMessageEv", "dyld::getErrorMessage()"},
    {"_ZN4dyld24registerUndefinedHandlerEPFvPKcE", "dyld::registerUndefinedHandler(void (*)(char const*))"},
    {"_ZN4dyld19openSharedCacheFileEv", "dyld::openSharedCacheFile()"},
    {"_ZN4dyld15setErrorMessageEPKc", "dyld::setErrorMessage(char const*)"},
    {"_ZN4dyld22registerRemoveCallbackEPFvPK11mach_headerlE", "dyld::registerRemoveCallback(void (*)(mach_header const*, long))"},
    {"_ZN4dyld13inSharedCacheEPKc", "dyld::inSharedCache(char const*)"},
    {"_ZN4dyld15runInitializersEP11ImageLoader", "dyld::runInitializers(ImageLoader*)"},
    {"_ZN4dyld27findCoalescedExportedSymbolEPKcPPKN11ImageLoader6SymbolEPPKS2_", "dyld::findCoalescedExportedSymbol(char const*, ImageLoader::Symbol const**, ImageLoader const**)"},
    {"_ZN4dyld22flatFindExportedSymbolEPKcPPKN11ImageLoader6SymbolEPPKS2_", "dyld::flatFindExportedSymbol(char const*, ImageLoader::Symbol const**, ImageLoader const**)"},
    {"_ZN4dyld15findLoadedImageERK4stat", "dyld::findLoadedImage(stat const&)"},
    {"_ZN4dyld24initializeMainExecutableEv", "dyld::initializeMainExecutable()"},
    {"_ZN4dyld4warnEPKcz", "dyld::warn(char const*, ...)"},
    {"_ZN4dyld29processDyldEnvironmentVaribleEPKcS1_", "dyld::processDyldEnvironmentVarible(char const*, char const*)"},
    {"_ZN4dyld3logEPKcz", "dyld::log(char const*, ...)"},
    {"_ZN4dyld6throwfEPKcz", "dyld::throwf(char const*, ...)"},
    {"_ZN4dyld9mkstringfEPKcz", "dyld::mkstringf(char const*, ...)"},
    {"_ZN4dyld14addMappedRangeEP11ImageLoadermm", "dyld::addMappedRange(ImageLoader*, unsigned long, unsigned long)"},
    {"_Z28coresymbolication_load_imageP25CSCppDyldSharedMemoryPagePK11ImageLoadery", "coresymbolication_load_image(CSCppDyldSharedMemoryPage*, ImageLoader const*, unsigned long long)"},
    {"_Z30coresymbolication_unload_imageP25CSCppDyldSharedMemoryPagePK11ImageLoader", "coresymbolication_unload_image(CSCppDyldSharedMemoryPage*, ImageLoader const*)"},
    {"_ZN4dyld18getCoalescedImagesEPP11ImageLoader", "dyld::getCoalescedImages(ImageLoader**)"},
    {"_ZN4dyld25findImageContainingSymbolEPKv", "dyld::findImageContainingSymbol(void const*)"},
    {"_ZN4dyld19registerAddCallbackEPFvPK11mach_headerlE", "dyld::registerAddCallback(void (*)(mach_header const*, long))"},
    {"_ZN4dyld14forEachImageDoEPFvP11ImageLoaderPvES2_", "dyld::forEachImageDo(void (*)(ImageLoader*, void*), void*)"},
    {"_ZN4dyld15getIndexedImageEj", "dyld::getIndexedImage(unsigned int)"},
    {"_ZN4dyld13getImageCountEv", "dyld::getImageCount()"},
    {"_ZN4dyld10validImageEPK11ImageLoader", "dyld::validImage(ImageLoader const*)"},
    {"_ZN4dyld30flatFindExportedSymbolWithHintEPKcS1_PPKN11ImageLoader6SymbolEPPKS2_", "dyld::flatFindExportedSymbolWithHint(char const*, char const*, ImageLoader::Symbol const**, ImageLoader const**)"},
    {"_ZN4dyld14loadFromMemoryEPKhyPKc", "dyld::loadFromMemory(unsigned char const*, unsigned long long, char const*)"},
    {"_ZN4dyld36registerImageStateBatchChangeHandlerE17dyld_image_statesPFPKcS0_jPK15dyld_image_infoE", "dyld::registerImageStateBatchChangeHandler(dyld_image_states, char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*))"},
    {"_ZN4dyld37registerImageStateSingleChangeHandlerE17dyld_image_statesPFPKcS0_jPK15dyld_image_infoE", "dyld::registerImageStateSingleChangeHandler(dyld_image_states, char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*))"},
    {"_ZN4dyld4haltEPKc", "dyld::halt(char const*)"},
    {"_ZN4dyld18fastBindLazySymbolEPP11ImageLoaderm", "dyld::fastBindLazySymbol(ImageLoader**, unsigned long)"},
    {"_ZN4dyld14bindLazySymbolEPK11mach_headerPm", "dyld::bindLazySymbol(mach_header const*, unsigned long*)"},
    {"_ZN4dyld14runTerminatorsEPv", "dyld::runTerminators(void*)"},
    {"_ZN4dyld11removeImageEP11ImageLoader", "dyld::removeImage(ImageLoader*)"},
    {"_ZN4dyld20garbageCollectImagesEv", "dyld::garbageCollectImages()"},
    {"_ZN4dyld9preflightEP11ImageLoaderRKNS0_10RPathChainE", "dyld::preflight(ImageLoader*, ImageLoader::RPathChain const&)"},
    {"_ZN4dyld4linkEP11ImageLoaderbRKNS0_10RPathChainE", "dyld::link(ImageLoader*, bool, ImageLoader::RPathChain const&)"},
    {"_ZN4dyld10cloneImageEP11ImageLoader", "dyld::cloneImage(ImageLoader*)"},
    {"_ZN4dyld4loadEPKcRKNS_11LoadContextE", "dyld::load(char const*, dyld::LoadContext const&)"},
    {"_ZN4dyld5_mainEPK12macho_headermiPPKcS5_S5_", "dyld::_main(macho_header const*, unsigned long, int, char const**, char const**, char const**)"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EE5eraseEN9__gnu_cxx17__normal_iteratorIPS1_S3_EE", "std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::erase(__gnu_cxx::__normal_iterator<dyld::RegisteredDOF*, std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> > >)"},
    {"_ZNSt12_Vector_baseIPKcSaIS1_EED2Ev", "std::_Vector_base<char const*, std::allocator<char const*> >::~_Vector_base()"},
    {"_ZNSt6vectorIPKcSaIS1_EED2Ev", "std::vector<char const*, std::allocator<char const*> >::~vector()"},
    {"_ZNSt12_Vector_baseIPFPKc17dyld_image_statesjPK15dyld_image_infoESaIS7_EED2Ev", "std::_Vector_base<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> >::~_Vector_base()"},
    {"_ZNSt6vectorIPFPKc17dyld_image_statesjPK15dyld_image_infoESaIS7_EED2Ev", "std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> >::~vector()"},
    {"_ZNSt12_Vector_baseIPFvPK11mach_headerlESaIS4_EED2Ev", "std::_Vector_base<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::~_Vector_base()"},
    {"_ZNSt6vectorIPFvPK11mach_headerlESaIS4_EED2Ev", "std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::~vector()"},
    {"_ZNSt12_Vector_baseIN4dyld13RegisteredDOFESaIS1_EED2Ev", "std::_Vector_base<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::~_Vector_base()"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EED2Ev", "std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::~vector()"},
    {"_ZNSt12_Vector_baseIP11ImageLoaderSaIS1_EED2Ev", "std::_Vector_base<ImageLoader*, std::allocator<ImageLoader*> >::~_Vector_base()"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EED2Ev", "std::vector<ImageLoader*, std::allocator<ImageLoader*> >::~vector()"},
    {"_ZN9__gnu_cxx13new_allocatorIPFPKc17dyld_image_statesjPK15dyld_image_infoEE8allocateEmPKv", "__gnu_cxx::new_allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)>::allocate(unsigned long, void const*)"},
    {"_ZN9__gnu_cxx13new_allocatorIPFvPK11mach_headerlEE8allocateEmPKv", "__gnu_cxx::new_allocator<void (*)(mach_header const*, long)>::allocate(unsigned long, void const*)"},
    {"_ZN9__gnu_cxx13new_allocatorIPKcE8allocateEmPKv", "__gnu_cxx::new_allocator<char const*>::allocate(unsigned long, void const*)"},
    {"_ZN9__gnu_cxx13new_allocatorIP11ImageLoaderE8allocateEmPKv", "__gnu_cxx::new_allocator<ImageLoader*>::allocate(unsigned long, void const*)"},
    {"_ZN9__gnu_cxx13new_allocatorIN4dyld13RegisteredDOFEE8allocateEmPKv", "__gnu_cxx::new_allocator<dyld::RegisteredDOF>::allocate(unsigned long, void const*)"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::_M_insert_aux(__gnu_cxx::__normal_iterator<dyld::RegisteredDOF*, std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> > >, dyld::RegisteredDOF const&)"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EE9push_backERKS1_", "std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::push_back(dyld::RegisteredDOF const&)"},
    {"_ZSt18uninitialized_copyIPPFPKc17dyld_image_statesjPK15dyld_image_infoES8_ET0_T_SA_S9_", "char const* (**std::uninitialized_copy<char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*), char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*)>(char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*), char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*), char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*)))(dyld_image_states, unsigned int, dyld_image_info const*)"},
    {"_ZSt18uninitialized_copyIPPFvPK11mach_headerlES5_ET0_T_S7_S6_", "void (**std::uninitialized_copy<void (**)(mach_header const*, long), void (**)(mach_header const*, long)>(void (**)(mach_header const*, long), void (**)(mach_header const*, long), void (**)(mach_header const*, long)))(mach_header const*, long)"},
    {"_ZSt18uninitialized_copyIPPKcS2_ET0_T_S4_S3_", "char const** std::uninitialized_copy<char const**, char const**>(char const**, char const**, char const**)"},
    {"_ZNSt15__copy_backwardILb1ESt26random_access_iterator_tagE8__copy_bIPFPKc17dyld_image_statesjPK15dyld_image_infoEEEPT_PKSB_SE_SC_", "char const* (**std::__copy_backward<true, std::random_access_iterator_tag>::__copy_b<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)>(char const* (* const*)(dyld_image_states, unsigned int, dyld_image_info const*), char const* (* const*)(dyld_image_states, unsigned int, dyld_image_info const*), char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*)))(dyld_image_states, unsigned int, dyld_image_info const*)"},
    {"_ZNSt6vectorIPFPKc17dyld_image_statesjPK15dyld_image_infoESaIS7_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> >::_M_insert_aux(__gnu_cxx::__normal_iterator<char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*), std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> > >, char const* (* const&)(dyld_image_states, unsigned int, dyld_image_info const*))"},
    {"_ZNSt6vectorIPFPKc17dyld_image_statesjPK15dyld_image_infoESaIS7_EE6insertEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> >::insert(__gnu_cxx::__normal_iterator<char const* (**)(dyld_image_states, unsigned int, dyld_image_info const*), std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> > >, char const* (* const&)(dyld_image_states, unsigned int, dyld_image_info const*))"},
    {"_ZNSt6vectorIPFPKc17dyld_image_statesjPK15dyld_image_infoESaIS7_EE9push_backERKS7_", "std::vector<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*), std::allocator<char const* (*)(dyld_image_states, unsigned int, dyld_image_info const*)> >::push_back(char const* (* const&)(dyld_image_states, unsigned int, dyld_image_info const*))"},
    {"_ZNSt15__copy_backwardILb1ESt26random_access_iterator_tagE8__copy_bIPFvPK11mach_headerlEEEPT_PKS8_SB_S9_", "void (**std::__copy_backward<true, std::random_access_iterator_tag>::__copy_b<void (*)(mach_header const*, long)>(void (* const*)(mach_header const*, long), void (* const*)(mach_header const*, long), void (**)(mach_header const*, long)))(mach_header const*, long)"},
    {"_ZNSt6vectorIPFvPK11mach_headerlESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::_M_insert_aux(__gnu_cxx::__normal_iterator<void (**)(mach_header const*, long), std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> > >, void (* const&)(mach_header const*, long))"},
    {"_ZNSt6vectorIPFvPK11mach_headerlESaIS4_EE9push_backERKS4_", "std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::push_back(void (* const&)(mach_header const*, long))"},
    {"_ZNSt15__copy_backwardILb1ESt26random_access_iterator_tagE8__copy_bIPKcEEPT_PKS5_S8_S6_", "char const** std::__copy_backward<true, std::random_access_iterator_tag>::__copy_b<char const*>(char const* const*, char const* const*, char const**)"},
    {"_ZNSt6vectorIPKcSaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<char const*, std::allocator<char const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<char const**, std::vector<char const*, std::allocator<char const*> > >, char const* const&)"},
    {"_ZNSt6vectorIPKcSaIS1_EE9push_backERKS1_", "std::vector<char const*, std::allocator<char const*> >::push_back(char const* const&)"},
    {"_ZNSt15__copy_backwardILb1ESt26random_access_iterator_tagE8__copy_bIP11ImageLoaderEEPT_PKS5_S8_S6_", "ImageLoader** std::__copy_backward<true, std::random_access_iterator_tag>::__copy_b<ImageLoader*>(ImageLoader* const*, ImageLoader* const*, ImageLoader**)"},
    {"_ZNSt6vectorIPFvPK11mach_headerlESaIS4_EE20_M_allocate_and_copyIPS4_EES8_mT_S9_", "void (**std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::_M_allocate_and_copy<void (**)(mach_header const*, long)>(unsigned long, void (**)(mach_header const*, long), void (**)(mach_header const*, long)))(mach_header const*, long)"},
    {"_ZNSt6vectorIPFvPK11mach_headerlESaIS4_EE7reserveEm", "std::vector<void (*)(mach_header const*, long), std::allocator<void (*)(mach_header const*, long)> >::reserve(unsigned long)"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EE20_M_allocate_and_copyIPS1_EES5_mT_S6_", "dyld::RegisteredDOF* std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::_M_allocate_and_copy<dyld::RegisteredDOF*>(unsigned long, dyld::RegisteredDOF*, dyld::RegisteredDOF*)"},
    {"_ZNSt6vectorIN4dyld13RegisteredDOFESaIS1_EE7reserveEm", "std::vector<dyld::RegisteredDOF, std::allocator<dyld::RegisteredDOF> >::reserve(unsigned long)"},
    {"_ZSt18uninitialized_copyIPP11ImageLoaderS2_ET0_T_S4_S3_", "ImageLoader** std::uninitialized_copy<ImageLoader**, ImageLoader**>(ImageLoader**, ImageLoader**, ImageLoader**)"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EE20_M_allocate_and_copyIPS1_EES5_mT_S6_", "ImageLoader** std::vector<ImageLoader*, std::allocator<ImageLoader*> >::_M_allocate_and_copy<ImageLoader**>(unsigned long, ImageLoader**, ImageLoader**)"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EE7reserveEm", "std::vector<ImageLoader*, std::allocator<ImageLoader*> >::reserve(unsigned long)"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<ImageLoader*, std::allocator<ImageLoader*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<ImageLoader**, std::vector<ImageLoader*, std::allocator<ImageLoader*> > >, ImageLoader* const&)"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EE9push_backERKS1_", "std::vector<ImageLoader*, std::allocator<ImageLoader*> >::push_back(ImageLoader* const&)"},
    {"_ZNSt6vectorIP11ImageLoaderSaIS1_EE5eraseEN9__gnu_cxx17__normal_iteratorIPS1_S3_EE", "std::vector<ImageLoader*, std::allocator<ImageLoader*> >::erase(__gnu_cxx::__normal_iterator<ImageLoader**, std::vector<ImageLoader*, std::allocator<ImageLoader*> > >)"},
    {"_Z18lookupDyldFunctionPKcPm", "lookupDyldFunction(char const*, unsigned long*)"},
    {"_ZNSt12_Vector_baseIP19__NSObjectFileImageSaIS1_EED2Ev", "std::_Vector_base<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> >::~_Vector_base()"},
    {"_ZNSt6vectorIP19__NSObjectFileImageSaIS1_EED2Ev", "std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> >::~vector()"},
    {"_ZNSt15__copy_backwardILb1ESt26random_access_iterator_tagE8__copy_bIP19__NSObjectFileImageEEPT_PKS5_S8_S6_", "__NSObjectFileImage** std::__copy_backward<true, std::random_access_iterator_tag>::__copy_b<__NSObjectFileImage*>(__NSObjectFileImage* const*, __NSObjectFileImage* const*, __NSObjectFileImage**)"},
    {"_ZN9__gnu_cxx13new_allocatorIP19__NSObjectFileImageE8allocateEmPKv", "__gnu_cxx::new_allocator<__NSObjectFileImage*>::allocate(unsigned long, void const*)"},
    {"_ZNSt6vectorIP19__NSObjectFileImageSaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<__NSObjectFileImage**, std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> > >, __NSObjectFileImage* const&)"},
    {"_ZNSt6vectorIP19__NSObjectFileImageSaIS1_EE9push_backERKS1_", "std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> >::push_back(__NSObjectFileImage* const&)"},
    {"_ZNSt6vectorIP19__NSObjectFileImageSaIS1_EE5eraseEN9__gnu_cxx17__normal_iteratorIPS1_S3_EE", "std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> >::erase(__gnu_cxx::__normal_iterator<__NSObjectFileImage**, std::vector<__NSObjectFileImage*, std::allocator<__NSObjectFileImage*> > >)"},
    {"_Z19setAlImageInfosHaltPKcm", "setAlImageInfosHalt(char const*, unsigned long)"},
    {"_Z24removeImageFromAllImagesPK11mach_header", "removeImageFromAllImages(mach_header const*)"},
    {"_Z20addImagesToAllImagesjPK15dyld_image_info", "addImagesToAllImages(unsigned int, dyld_image_info const*)"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EE5eraseEN9__gnu_cxx17__normal_iteratorIPS0_S2_EE", "std::vector<dyld_image_info, std::allocator<dyld_image_info> >::erase(__gnu_cxx::__normal_iterator<dyld_image_info*, std::vector<dyld_image_info, std::allocator<dyld_image_info> > >)"},
    {"_ZNSt12_Vector_baseI15dyld_image_infoSaIS0_EED2Ev", "std::_Vector_base<dyld_image_info, std::allocator<dyld_image_info> >::~_Vector_base()"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EED2Ev", "std::vector<dyld_image_info, std::allocator<dyld_image_info> >::~vector()"},
    {"_ZN9__gnu_cxx13new_allocatorI15dyld_image_infoE8allocateEmPKv", "__gnu_cxx::new_allocator<dyld_image_info>::allocate(unsigned long, void const*)"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EE20_M_allocate_and_copyIPS0_EES4_mT_S5_", "dyld_image_info* std::vector<dyld_image_info, std::allocator<dyld_image_info> >::_M_allocate_and_copy<dyld_image_info*>(unsigned long, dyld_image_info*, dyld_image_info*)"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EE7reserveEm", "std::vector<dyld_image_info, std::allocator<dyld_image_info> >::reserve(unsigned long)"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS0_S2_EERKS0_", "std::vector<dyld_image_info, std::allocator<dyld_image_info> >::_M_insert_aux(__gnu_cxx::__normal_iterator<dyld_image_info*, std::vector<dyld_image_info, std::allocator<dyld_image_info> > >, dyld_image_info const&)"},
    {"_ZNSt6vectorI15dyld_image_infoSaIS0_EE9push_backERKS0_", "std::vector<dyld_image_info, std::allocator<dyld_image_info> >::push_back(dyld_image_info const&)"},
    {"_ZN10__cxxabiv112__unexpectedEPFvvE", "__cxxabiv1::__unexpected(void (*)())"},
    {"_ZN10__cxxabiv111__terminateEPFvvE", "__cxxabiv1::__terminate(void (*)())"},
    {"_ZSt10unexpectedv", "std::unexpected()"},
    {"_ZSt9terminatev", "std::terminate()"},
    {"_ZN11ImageLoaderC2EPKcj", "ImageLoader::ImageLoader(char const*, unsigned int)"},
    {"_ZN11ImageLoader11deleteImageEPS_", "ImageLoader::deleteImage(ImageLoader*)"},
    {"_ZN11ImageLoader11setFileInfoEiyl", "ImageLoader::setFileInfo(int, unsigned long long, long)"},
    {"_ZN11ImageLoader9setMappedERKNS_11LinkContextE", "ImageLoader::setMapped(ImageLoader::LinkContext const&)"},
    {"_ZNK11ImageLoader7compareEPKS_", "ImageLoader::compare(ImageLoader const*) const"},
    {"_ZN11ImageLoader4hashEPKc", "ImageLoader::hash(char const*)"},
    {"_ZNK11ImageLoader16matchInstallPathEv", "ImageLoader::matchInstallPath() const"},
    {"_ZN11ImageLoader19setMatchInstallPathEb", "ImageLoader::setMatchInstallPath(bool)"},
    {"_ZNK11ImageLoader9statMatchERK4stat", "ImageLoader::statMatch(stat const&) const"},
    {"_ZN11ImageLoader14setLeaveMappedEv", "ImageLoader::setLeaveMapped()"},
    {"_ZN11ImageLoader14setHideExportsEb", "ImageLoader::setHideExports(bool)"},
    {"_ZNK11ImageLoader16hasHiddenExportsEv", "ImageLoader::hasHiddenExports() const"},
    {"_ZNK11ImageLoader8isLinkedEv", "ImageLoader::isLinked() const"},
    {"_ZNK11ImageLoader12lastModifiedEv", "ImageLoader::lastModified() const"},
    {"_ZNK11ImageLoader24overlapsWithAddressRangeEPKvS1_", "ImageLoader::overlapsWithAddressRange(void const*, void const*) const"},
    {"_ZNK11ImageLoader16getMappedRegionsERPNS_12MappedRegionE", "ImageLoader::getMappedRegions(ImageLoader::MappedRegion*&) const"},
    {"_ZNK11ImageLoader41findExportedSymbolInDependentImagesExceptEPKcPPKS_RS4_S4_S4_", "ImageLoader::findExportedSymbolInDependentImagesExcept(char const*, ImageLoader const**, ImageLoader const**&, ImageLoader const**, ImageLoader const**) const"},
    {"_ZN11ImageLoader29decrementDlopenReferenceCountEv", "ImageLoader::decrementDlopenReferenceCount()"},
    {"_ZN11ImageLoader19bindAllLazyPointersERKNS_11LinkContextEb", "ImageLoader::bindAllLazyPointers(ImageLoader::LinkContext const&, bool)"},
    {"_ZNK11ImageLoader35allDependentLibrariesAsWhenPreBoundEv", "ImageLoader::allDependentLibrariesAsWhenPreBound() const"},
    {"_ZN11ImageLoader20recursiveUpdateDepthEj", "ImageLoader::recursiveUpdateDepth(unsigned int)"},
    {"_ZN11ImageLoader23recursiveGetDOFSectionsERKNS_11LinkContextERSt6vectorINS_7DOFInfoESaIS4_EE", "ImageLoader::recursiveGetDOFSections(ImageLoader::LinkContext const&, std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >&)"},
    {"_ZNK11ImageLoader12getShortNameEv", "ImageLoader::getShortName() const"},
    {"_ZN11ImageLoader9addSuffixEPKcS1_Pc", "ImageLoader::addSuffix(char const*, char const*, char*)"},
    {"_ZN11ImageLoader15printStatisticsEj", "ImageLoader::printStatistics(unsigned int)"},
    {"_ZN11ImageLoader13recursiveBindERKNS_11LinkContextEb", "ImageLoader::recursiveBind(ImageLoader::LinkContext const&, bool)"},
    {"_ZN11ImageLoader25recursiveApplyInterposingERKNS_11LinkContextE", "ImageLoader::recursiveApplyInterposing(ImageLoader::LinkContext const&)"},
    {"_ZN11ImageLoader16applyInterposingERKNS_11LinkContextE", "ImageLoader::applyInterposing(ImageLoader::LinkContext const&)"},
    {"_ZN11ImageLoader15recursiveRebaseERKNS_11LinkContextE", "ImageLoader::recursiveRebase(ImageLoader::LinkContext const&)"},
    {"_ZNK11ImageLoader42findExportedSymbolInImageOrDependentImagesEPKcRKNS_11LinkContextEPPKS_", "ImageLoader::findExportedSymbolInImageOrDependentImages(char const*, ImageLoader::LinkContext const&, ImageLoader const**) const"},
    {"_ZNK11ImageLoader35findExportedSymbolInDependentImagesEPKcRKNS_11LinkContextEPPKS_", "ImageLoader::findExportedSymbolInDependentImages(char const*, ImageLoader::LinkContext const&, ImageLoader const**) const"},
    {"_ZN11ImageLoader8reExportEPS_", "ImageLoader::reExport(ImageLoader*)"},
    {"_ZN11ImageLoader19addDynamicReferenceEPKS_", "ImageLoader::addDynamicReference(ImageLoader const*)"},
    {"_ZN11ImageLoader14setPathUnownedEPKc", "ImageLoader::setPathUnowned(char const*)"},
    {"_ZN11ImageLoader7setPathEPKc", "ImageLoader::setPath(char const*)"},
    {"_ZN11ImageLoaderD0Ev", "ImageLoader::~ImageLoader()"},
    {"_ZNK11ImageLoader15containsAddressEPKv", "ImageLoader::containsAddress(void const*) const"},
    {"_ZN11ImageLoaderD2Ev", "ImageLoader::~ImageLoader()"},
    {"_ZN11ImageLoaderD1Ev", "ImageLoader::~ImageLoader()"},
    {"_ZN11ImageLoader23recursiveInitializationERKNS_11LinkContextEj", "ImageLoader::recursiveInitialization(ImageLoader::LinkContext const&, unsigned int)"},
    {"_ZN11ImageLoader15runInitializersERKNS_11LinkContextE", "ImageLoader::runInitializers(ImageLoader::LinkContext const&)"},
    {"_ZN11ImageLoader8weakBindERKNS_11LinkContextE", "ImageLoader::weakBind(ImageLoader::LinkContext const&)"},
    {"_ZN11ImageLoader22recursiveLoadLibrariesERKNS_11LinkContextEbRKNS_10RPathChainE", "ImageLoader::recursiveLoadLibraries(ImageLoader::LinkContext const&, bool, ImageLoader::RPathChain const&)"},
    {"_ZN11ImageLoader4linkERKNS_11LinkContextEbbRKNS_10RPathChainE", "ImageLoader::link(ImageLoader::LinkContext const&, bool, bool, ImageLoader::RPathChain const&)"},
    {"_ZNSt12_Vector_baseIN11ImageLoader7DOFInfoESaIS1_EED2Ev", "std::_Vector_base<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >::~_Vector_base()"},
    {"_ZNSt6vectorIN11ImageLoader7DOFInfoESaIS1_EED2Ev", "std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >::~vector()"},
    {"_ZNSt8_Rb_treeIPK11ImageLoaderS2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<ImageLoader const*, ImageLoader const*, std::_Identity<ImageLoader const*>, std::less<ImageLoader const*>, std::allocator<ImageLoader const*> >::_M_erase(std::_Rb_tree_node<ImageLoader const*>*)"},
    {"_ZNSt12_Vector_baseIN11ImageLoader14InterposeTupleESaIS1_EED2Ev", "std::_Vector_base<ImageLoader::InterposeTuple, std::allocator<ImageLoader::InterposeTuple> >::~_Vector_base()"},
    {"_ZNSt6vectorIN11ImageLoader14InterposeTupleESaIS1_EED2Ev", "std::vector<ImageLoader::InterposeTuple, std::allocator<ImageLoader::InterposeTuple> >::~vector()"},
    {"_ZNSt12_Vector_baseISt4pairIP11ImageLoaderS2_ESaIS3_EED2Ev", "std::_Vector_base<std::pair<ImageLoader*, ImageLoader*>, std::allocator<std::pair<ImageLoader*, ImageLoader*> > >::~_Vector_base()"},
    {"_ZNSt6vectorISt4pairIP11ImageLoaderS2_ESaIS3_EED2Ev", "std::vector<std::pair<ImageLoader*, ImageLoader*>, std::allocator<std::pair<ImageLoader*, ImageLoader*> > >::~vector()"},
    {"_ZN9__gnu_cxx13new_allocatorISt13_Rb_tree_nodeIPK11ImageLoaderEE8allocateEmPKv", "__gnu_cxx::new_allocator<std::_Rb_tree_node<ImageLoader const*> >::allocate(unsigned long, void const*)"},
    {"_ZNSt8_Rb_treeIPK11ImageLoaderS2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE14_M_create_nodeERKS2_", "std::_Rb_tree<ImageLoader const*, ImageLoader const*, std::_Identity<ImageLoader const*>, std::less<ImageLoader const*>, std::allocator<ImageLoader const*> >::_M_create_node(ImageLoader const* const&)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIP11ImageLoaderS3_EE8allocateEmPKv", "__gnu_cxx::new_allocator<std::pair<ImageLoader*, ImageLoader*> >::allocate(unsigned long, void const*)"},
    {"_ZNSt6vectorISt4pairIP11ImageLoaderS2_ESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<ImageLoader*, ImageLoader*>, std::allocator<std::pair<ImageLoader*, ImageLoader*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<ImageLoader*, ImageLoader*>*, std::vector<std::pair<ImageLoader*, ImageLoader*>, std::allocator<std::pair<ImageLoader*, ImageLoader*> > > >, std::pair<ImageLoader*, ImageLoader*> const&)"},
    {"_ZNSt6vectorISt4pairIP11ImageLoaderS2_ESaIS3_EE9push_backERKS3_", "std::vector<std::pair<ImageLoader*, ImageLoader*>, std::allocator<std::pair<ImageLoader*, ImageLoader*> > >::push_back(std::pair<ImageLoader*, ImageLoader*> const&)"},
    {"_ZNSt8_Rb_treeIPK11ImageLoaderS2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE9_M_insertEPSt18_Rb_tree_node_baseSA_RKS2_", "std::_Rb_tree<ImageLoader const*, ImageLoader const*, std::_Identity<ImageLoader const*>, std::less<ImageLoader const*>, std::allocator<ImageLoader const*> >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, ImageLoader const* const&)"},
    {"_ZNSt8_Rb_treeIPK11ImageLoaderS2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<ImageLoader const*, ImageLoader const*, std::_Identity<ImageLoader const*>, std::less<ImageLoader const*>, std::allocator<ImageLoader const*> >::_M_insert_unique(ImageLoader const* const&)"},
    {"_ZNK16ImageLoaderMachO12segmentCountEv", "ImageLoaderMachO::segmentCount() const"},
    {"_ZN16ImageLoaderMachO22segHasTrailingZeroFillEj", "ImageLoaderMachO::segHasTrailingZeroFill(unsigned int)"},
    {"_ZNK16ImageLoaderMachO19segActualEndAddressEj", "ImageLoaderMachO::segActualEndAddress(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO25segmentsMustSlideTogetherEv", "ImageLoaderMachO::segmentsMustSlideTogether() const"},
    {"_ZNK16ImageLoaderMachO16segmentsCanSlideEv", "ImageLoaderMachO::segmentsCanSlide() const"},
    {"_ZNK16ImageLoaderMachO8isBundleEv", "ImageLoaderMachO::isBundle() const"},
    {"_ZNK16ImageLoaderMachO7isDylibEv", "ImageLoaderMachO::isDylib() const"},
    {"_ZNK16ImageLoaderMachO12isExecutableEv", "ImageLoaderMachO::isExecutable() const"},
    {"_ZNK16ImageLoaderMachO31isPositionIndependentExecutableEv", "ImageLoaderMachO::isPositionIndependentExecutable() const"},
    {"_ZNK16ImageLoaderMachO9forceFlatEv", "ImageLoaderMachO::forceFlat() const"},
    {"_ZNK16ImageLoaderMachO21usesTwoLevelNameSpaceEv", "ImageLoaderMachO::usesTwoLevelNameSpace() const"},
    {"_ZNK16ImageLoaderMachO13isPrebindableEv", "ImageLoaderMachO::isPrebindable() const"},
    {"_ZNK16ImageLoaderMachO19hasCoalescedExportsEv", "ImageLoaderMachO::hasCoalescedExports() const"},
    {"_ZN16ImageLoaderMachO8setSlideEl", "ImageLoaderMachO::setSlide(long)"},
    {"_ZNK16ImageLoaderMachO14getInstallPathEv", "ImageLoaderMachO::getInstallPath() const"},
    {"_ZNK16ImageLoaderMachO7getMainEv", "ImageLoaderMachO::getMain() const"},
    {"_ZN16ImageLoaderMachO16doGetLibraryInfoEv", "ImageLoaderMachO::doGetLibraryInfo()"},
    {"_ZNK16ImageLoaderMachO21getExportedSymbolInfoEPKN11ImageLoader6SymbolE", "ImageLoaderMachO::getExportedSymbolInfo(ImageLoader::Symbol const*) const"},
    {"_ZNK16ImageLoaderMachO21getExportedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachO::getExportedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZNK16ImageLoaderMachO22getExportedSymbolCountEv", "ImageLoaderMachO::getExportedSymbolCount() const"},
    {"_ZNK16ImageLoaderMachO24getIndexedExportedSymbolEj", "ImageLoaderMachO::getIndexedExportedSymbol(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO22getImportedSymbolCountEv", "ImageLoaderMachO::getImportedSymbolCount() const"},
    {"_ZNK16ImageLoaderMachO24getIndexedImportedSymbolEj", "ImageLoaderMachO::getIndexedImportedSymbol(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO21getImportedSymbolInfoEPKN11ImageLoader6SymbolE", "ImageLoaderMachO::getImportedSymbolInfo(ImageLoader::Symbol const*) const"},
    {"_ZNK16ImageLoaderMachO21getImportedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachO::getImportedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZN16ImageLoaderMachO13getUnwindInfoEP20dyld_unwind_sections", "ImageLoaderMachO::getUnwindInfo(dyld_unwind_sections*)"},
    {"_ZN16ImageLoaderMachO11findSectionEPKvPPKcS4_Pm", "ImageLoaderMachO::findSection(void const*, char const**, char const**, unsigned long*)"},
    {"_ZNK16ImageLoaderMachO10machHeaderEv", "ImageLoaderMachO::machHeader() const"},
    {"_ZNK16ImageLoaderMachO8getSlideEv", "ImageLoaderMachO::getSlide() const"},
    {"_ZNK16ImageLoaderMachO17lookupProgramVarsERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::lookupProgramVars(ImageLoader::LinkContext const&) const"},
    {"_ZN16ImageLoaderMachO19needsInitializationEv", "ImageLoaderMachO::needsInitialization()"},
    {"_ZN16ImageLoaderMachO16needsTerminationEv", "ImageLoaderMachO::needsTermination()"},
    {"_ZN16ImageLoaderMachO10segProtectEjRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::segProtect(unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO18doModInitFunctionsERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::doModInitFunctions(ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO11doImageInitERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::doImageInit(ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO16doInitializationERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::doInitialization(ImageLoader::LinkContext const&)"},
    {"_ZNK16ImageLoaderMachO16getSymbolAddressEPKN11ImageLoader6SymbolEPKS0_RKNS0_11LinkContextE", "ImageLoaderMachO::getSymbolAddress(ImageLoader::Symbol const*, ImageLoader const*, ImageLoader::LinkContext const&) const"},
    {"_ZNK16ImageLoaderMachO24getExportedSymbolAddressEPKN11ImageLoader6SymbolERKNS0_11LinkContextEPKS0_", "ImageLoaderMachO::getExportedSymbolAddress(ImageLoader::Symbol const*, ImageLoader::LinkContext const&, ImageLoader const*) const"},
    {"_ZN16ImageLoaderMachO19throwSymbolNotFoundEPKcS1_S1_", "ImageLoaderMachO::throwSymbolNotFound(char const*, char const*, char const*)"},
    {"_ZN16ImageLoaderMachO21reserveAnAddressRangeEmRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::reserveAnAddressRange(unsigned long, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO23setupLazyPointerHandlerERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::setupLazyPointerHandler(ImageLoader::LinkContext const&)"},
    {"_ZNK16ImageLoaderMachO6getEndEv", "ImageLoaderMachO::getEnd() const"},
    {"_ZN16ImageLoaderMachO17getSectionContentEPKcS1_PPvPm", "ImageLoaderMachO::getSectionContent(char const*, char const*, void**, unsigned long*)"},
    {"_ZN16ImageLoaderMachO13parseLoadCmdsEv", "ImageLoaderMachO::parseLoadCmds()"},
    {"_ZN16ImageLoaderMachO15printStatisticsEj", "ImageLoaderMachO::printStatistics(unsigned int)"},
    {"_ZN16ImageLoaderMachO12bindLocationERKN11ImageLoader11LinkContextEmmPKS0_hPKclS7_", "ImageLoaderMachO::bindLocation(ImageLoader::LinkContext const&, unsigned long, unsigned long, ImageLoader const*, unsigned char, char const*, long, char const*)"},
    {"_ZN16ImageLoaderMachO16doGetDOFSectionsERKN11ImageLoader11LinkContextERSt6vectorINS0_7DOFInfoESaIS5_EE", "ImageLoaderMachO::doGetDOFSections(ImageLoader::LinkContext const&, std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >&)"},
    {"_ZNK16ImageLoaderMachO16usablePrebindingERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::usablePrebinding(ImageLoader::LinkContext const&) const"},
    {"_ZNK16ImageLoaderMachO7getUUIDEPh", "ImageLoaderMachO::getUUID(unsigned char*) const"},
    {"_ZN16ImageLoaderMachO26needsAddedLibSystemDepencyEjPK12macho_header", "ImageLoaderMachO::needsAddedLibSystemDepency(unsigned int, macho_header const*)"},
    {"_ZN16ImageLoaderMachO23doGetDependentLibrariesEPN11ImageLoader20DependentLibraryInfoE", "ImageLoaderMachO::doGetDependentLibraries(ImageLoader::DependentLibraryInfo*)"},
    {"_ZN16ImageLoaderMachO17sniffLoadCommandsEPK12macho_headerPKcPbPjS6_", "ImageLoaderMachO::sniffLoadCommands(macho_header const*, char const*, bool*, unsigned int*, unsigned int*)"},
    {"_ZNK16ImageLoaderMachO24participatesInCoalescingEv", "ImageLoaderMachO::participatesInCoalescing() const"},
    {"_ZN16ImageLoaderMachO12preFetchDATAEiyRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::preFetchDATA(int, unsigned long long, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO13UnmapSegmentsEv", "ImageLoaderMachO::UnmapSegments()"},
    {"_ZN16ImageLoaderMachO7destroyEv", "ImageLoaderMachO::destroy()"},
    {"_ZN16ImageLoaderMachO21instantiateFromMemoryEPKcPK12macho_headeryRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::instantiateFromMemory(char const*, macho_header const*, unsigned long long, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO20instantiateFromCacheEPK12macho_headerPKcRK4statRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::instantiateFromCache(macho_header const*, char const*, stat const&, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO19instantiateFromFileEPKciPKhyyRK4statRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::instantiateFromFile(char const*, int, unsigned char const*, unsigned long long, unsigned long long, stat const&, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO25instantiateMainExecutableEPK12macho_headermPKcRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::instantiateMainExecutable(macho_header const*, unsigned long, char const*, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachOC2EPK12macho_headerPKcjPjj", "ImageLoaderMachO::ImageLoaderMachO(macho_header const*, char const*, unsigned int, unsigned int*, unsigned int)"},
    {"_ZN16ImageLoaderMachO13doTerminationERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::doTermination(ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO8doRebaseERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::doRebase(ImageLoader::LinkContext const&)"},
    {"_ZNK16ImageLoaderMachO9getRPathsERKN11ImageLoader11LinkContextERSt6vectorIPKcSaIS6_EE", "ImageLoaderMachO::getRPaths(ImageLoader::LinkContext const&, std::vector<char const*, std::allocator<char const*> >&) const"},
    {"_ZN16ImageLoaderMachO19registerInterposingEv", "ImageLoaderMachO::registerInterposing()"},
    {"_ZNK16ImageLoaderMachO18findExportedSymbolEPKcbPPK11ImageLoader", "ImageLoaderMachO::findExportedSymbol(char const*, bool, ImageLoader const**) const"},
    {"_ZN16ImageLoaderMachO22assignSegmentAddressesERKN11ImageLoader11LinkContextE", "ImageLoaderMachO::assignSegmentAddresses(ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO11mapSegmentsEPKvyRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::mapSegments(void const*, unsigned long long, ImageLoader::LinkContext const&)"},
    {"_ZN16ImageLoaderMachO11mapSegmentsEiyyyRKN11ImageLoader11LinkContextE", "ImageLoaderMachO::mapSegments(int, unsigned long long, unsigned long long, unsigned long long, ImageLoader::LinkContext const&)"},
    {"_ZNK16ImageLoaderMachO23segPreferredLoadAddressEj", "ImageLoaderMachO::segPreferredLoadAddress(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO26segHasPreferredLoadAddressEj", "ImageLoaderMachO::segHasPreferredLoadAddress(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO15segUnaccessibleEj", "ImageLoaderMachO::segUnaccessible(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO13segExecutableEj", "ImageLoaderMachO::segExecutable(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO20segActualLoadAddressEj", "ImageLoaderMachO::segActualLoadAddress(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO7segSizeEj", "ImageLoaderMachO::segSize(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO7segNameEj", "ImageLoaderMachO::segName(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO12segWriteableEj", "ImageLoaderMachO::segWriteable(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO11segReadableEj", "ImageLoaderMachO::segReadable(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO13segFileOffsetEj", "ImageLoaderMachO::segFileOffset(unsigned int) const"},
    {"_ZNK16ImageLoaderMachO11segFileSizeEj", "ImageLoaderMachO::segFileSize(unsigned int) const"},
    {"_ZN16ImageLoaderMachOD2Ev", "ImageLoaderMachO::~ImageLoaderMachO()"},
    {"_ZN16ImageLoaderMachOD1Ev", "ImageLoaderMachO::~ImageLoaderMachO()"},
    {"_ZN16ImageLoaderMachOD0Ev", "ImageLoaderMachO::~ImageLoaderMachO()"},
    {"_ZN9__gnu_cxx13new_allocatorIN11ImageLoader7DOFInfoEE8allocateEmPKv", "__gnu_cxx::new_allocator<ImageLoader::DOFInfo>::allocate(unsigned long, void const*)"},
    {"_ZN9__gnu_cxx13new_allocatorIN11ImageLoader14InterposeTupleEE8allocateEmPKv", "__gnu_cxx::new_allocator<ImageLoader::InterposeTuple>::allocate(unsigned long, void const*)"},
    {"_ZNSt6vectorIN11ImageLoader7DOFInfoESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<ImageLoader::DOFInfo*, std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> > >, ImageLoader::DOFInfo const&)"},
    {"_ZNSt6vectorIN11ImageLoader7DOFInfoESaIS1_EE9push_backERKS1_", "std::vector<ImageLoader::DOFInfo, std::allocator<ImageLoader::DOFInfo> >::push_back(ImageLoader::DOFInfo const&)"},
    {"_ZNSt6vectorIN11ImageLoader14InterposeTupleESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<ImageLoader::InterposeTuple, std::allocator<ImageLoader::InterposeTuple> >::_M_insert_aux(__gnu_cxx::__normal_iterator<ImageLoader::InterposeTuple*, std::vector<ImageLoader::InterposeTuple, std::allocator<ImageLoader::InterposeTuple> > >, ImageLoader::InterposeTuple const&)"},
    {"_ZNSt6vectorIN11ImageLoader14InterposeTupleESaIS1_EE9push_backERKS1_", "std::vector<ImageLoader::InterposeTuple, std::allocator<ImageLoader::InterposeTuple> >::push_back(ImageLoader::InterposeTuple const&)"},
    {"_ZNK23ImageLoaderMachOClassic21segmentCommandOffsetsEv", "ImageLoaderMachOClassic::segmentCommandOffsets() const"},
    {"_ZNK23ImageLoaderMachOClassic8libImageEj", "ImageLoaderMachOClassic::libImage(unsigned int) const"},
    {"_ZNK23ImageLoaderMachOClassic13libReExportedEj", "ImageLoaderMachOClassic::libReExported(unsigned int) const"},
    {"_ZN23ImageLoaderMachOClassic11setLibImageEjP11ImageLoaderb", "ImageLoaderMachOClassic::setLibImage(unsigned int, ImageLoader*, bool)"},
    {"_ZN23ImageLoaderMachOClassic18setSymbolTableInfoEPK11macho_nlistPKcPK16dysymtab_command", "ImageLoaderMachOClassic::setSymbolTableInfo(macho_nlist const*, char const*, dysymtab_command const*)"},
    {"_ZNK23ImageLoaderMachOClassic14containsSymbolEPKv", "ImageLoaderMachOClassic::containsSymbol(void const*) const"},
    {"_ZNK23ImageLoaderMachOClassic21exportedSymbolAddressEPKN11ImageLoader6SymbolE", "ImageLoaderMachOClassic::exportedSymbolAddress(ImageLoader::Symbol const*) const"},
    {"_ZNK23ImageLoaderMachOClassic29exportedSymbolIsWeakDefintionEPKN11ImageLoader6SymbolE", "ImageLoaderMachOClassic::exportedSymbolIsWeakDefintion(ImageLoader::Symbol const*) const"},
    {"_ZNK23ImageLoaderMachOClassic18exportedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachOClassic::exportedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZNK23ImageLoaderMachOClassic19exportedSymbolCountEv", "ImageLoaderMachOClassic::exportedSymbolCount() const"},
    {"_ZNK23ImageLoaderMachOClassic21exportedSymbolIndexedEj", "ImageLoaderMachOClassic::exportedSymbolIndexed(unsigned int) const"},
    {"_ZNK23ImageLoaderMachOClassic19importedSymbolCountEv", "ImageLoaderMachOClassic::importedSymbolCount() const"},
    {"_ZNK23ImageLoaderMachOClassic21importedSymbolIndexedEj", "ImageLoaderMachOClassic::importedSymbolIndexed(unsigned int) const"},
    {"_ZNK23ImageLoaderMachOClassic18importedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachOClassic::importedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZN23ImageLoaderMachOClassic22initializeCoalIteratorERN11ImageLoader12CoalIteratorEj", "ImageLoaderMachOClassic::initializeCoalIterator(ImageLoader::CoalIterator&, unsigned int)"},
    {"_ZN23ImageLoaderMachOClassic21incrementCoalIteratorERN11ImageLoader12CoalIteratorE", "ImageLoaderMachOClassic::incrementCoalIterator(ImageLoader::CoalIterator&)"},
    {"_ZN23ImageLoaderMachOClassic22getAddressCoalIteratorERN11ImageLoader12CoalIteratorERKNS0_11LinkContextE", "ImageLoaderMachOClassic::getAddressCoalIterator(ImageLoader::CoalIterator&, ImageLoader::LinkContext const&)"},
    {"_ZNK23ImageLoaderMachOClassic17findClosestSymbolEPKvPS1_", "ImageLoaderMachOClassic::findClosestSymbol(void const*, void const**) const"},
    {"_ZN23ImageLoaderMachOClassic20doBindFastLazySymbolEjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::doBindFastLazySymbol(unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic30getFirstWritableSegmentAddressEv", "ImageLoaderMachOClassic::getFirstWritableSegmentAddress()"},
    {"_ZN23ImageLoaderMachOClassic18bindIndirectSymbolEPmPK13macho_sectionPKcmPK11ImageLoaderRKNS6_11LinkContextE", "ImageLoaderMachOClassic::bindIndirectSymbol(unsigned long*, macho_section const*, char const*, unsigned long, ImageLoader const*, ImageLoader::LinkContext const&)"},
    {"_ZNK23ImageLoaderMachOClassic12binarySearchEPKcS1_PK11macho_nlistj", "ImageLoaderMachOClassic::binarySearch(char const*, char const*, macho_nlist const*, unsigned int) const"},
    {"_ZNK23ImageLoaderMachOClassic19binarySearchWithTocEPKcS1_PK11macho_nlistPK23dylib_table_of_contentsjj", "ImageLoaderMachOClassic::binarySearchWithToc(char const*, char const*, macho_nlist const*, dylib_table_of_contents const*, unsigned int, unsigned int) const"},
    {"_ZNK23ImageLoaderMachOClassic18findExportedSymbolEPKcPPK11ImageLoader", "ImageLoaderMachOClassic::findExportedSymbol(char const*, ImageLoader const**) const"},
    {"_ZNK23ImageLoaderMachOClassic13hasSubLibraryERKN11ImageLoader11LinkContextEPKS0_", "ImageLoaderMachOClassic::hasSubLibrary(ImageLoader::LinkContext const&, ImageLoader const*) const"},
    {"_ZNK23ImageLoaderMachOClassic16isSubframeworkOfERKN11ImageLoader11LinkContextEPKS0_", "ImageLoaderMachOClassic::isSubframeworkOf(ImageLoader::LinkContext const&, ImageLoader const*) const"},
    {"_ZN23ImageLoaderMachOClassic18mapSegmentsClassicEiyyyRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::mapSegmentsClassic(int, unsigned long long, unsigned long long, unsigned long long, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic16prefetchLINKEDITERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::prefetchLINKEDIT(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassicD2Ev", "ImageLoaderMachOClassic::~ImageLoaderMachOClassic()"},
    {"_ZN23ImageLoaderMachOClassicD1Ev", "ImageLoaderMachOClassic::~ImageLoaderMachOClassic()"},
    {"_ZN23ImageLoaderMachOClassicD0Ev", "ImageLoaderMachOClassic::~ImageLoaderMachOClassic()"},
    {"_ZN23ImageLoaderMachOClassic17instantiateFinishERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::instantiateFinish(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic6rebaseERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::rebase(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic11doInterposeERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::doInterpose(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic16resolveUndefinedERKN11ImageLoader11LinkContextEPK11macho_nlistbbPPKS0_", "ImageLoaderMachOClassic::resolveUndefined(ImageLoader::LinkContext const&, macho_nlist const*, bool, bool, ImageLoader const**)"},
    {"_ZN23ImageLoaderMachOClassic26bindIndirectSymbolPointersERKN11ImageLoader11LinkContextEbb", "ImageLoaderMachOClassic::bindIndirectSymbolPointers(ImageLoader::LinkContext const&, bool, bool)"},
    {"_ZN23ImageLoaderMachOClassic16doBindJustLaziesERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::doBindJustLazies(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic16doBindLazySymbolEPmRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::doBindLazySymbol(unsigned long*, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic16instantiateStartEPK12macho_headerPKcjj", "ImageLoaderMachOClassic::instantiateStart(macho_header const*, char const*, unsigned int, unsigned int)"},
    {"_ZN23ImageLoaderMachOClassic21instantiateFromMemoryEPKcPK12macho_headeryjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::instantiateFromMemory(char const*, macho_header const*, unsigned long long, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic20instantiateFromCacheEPK12macho_headerPKcRK4statjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::instantiateFromCache(macho_header const*, char const*, stat const&, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic19instantiateFromFileEPKciPKhyyRK4statjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::instantiateFromFile(char const*, int, unsigned char const*, unsigned long long, unsigned long long, stat const&, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic25instantiateMainExecutableEPK12macho_headermPKcjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::instantiateMainExecutable(macho_header const*, unsigned long, char const*, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic22updateUsesCoalIteratorERN11ImageLoader12CoalIteratorEmPS0_RKNS0_11LinkContextE", "ImageLoaderMachOClassic::updateUsesCoalIterator(ImageLoader::CoalIterator&, unsigned long, ImageLoader*, ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic25doBindExternalRelocationsERKN11ImageLoader11LinkContextE", "ImageLoaderMachOClassic::doBindExternalRelocations(ImageLoader::LinkContext const&)"},
    {"_ZN23ImageLoaderMachOClassic6doBindERKN11ImageLoader11LinkContextEb", "ImageLoaderMachOClassic::doBind(ImageLoader::LinkContext const&, bool)"},
    {"_ZN23ImageLoaderMachOClassic11setDyldInfoEPK17dyld_info_command", "ImageLoaderMachOClassic::setDyldInfo(dyld_info_command const*)"},
    {"_ZNK26ImageLoaderMachOCompressed21segmentCommandOffsetsEv", "ImageLoaderMachOCompressed::segmentCommandOffsets() const"},
    {"_ZNK26ImageLoaderMachOCompressed8libImageEj", "ImageLoaderMachOCompressed::libImage(unsigned int) const"},
    {"_ZNK26ImageLoaderMachOCompressed13libReExportedEj", "ImageLoaderMachOCompressed::libReExported(unsigned int) const"},
    {"_ZN26ImageLoaderMachOCompressed11setLibImageEjP11ImageLoaderb", "ImageLoaderMachOCompressed::setLibImage(unsigned int, ImageLoader*, bool)"},
    {"_ZNK26ImageLoaderMachOCompressed14containsSymbolEPKv", "ImageLoaderMachOCompressed::containsSymbol(void const*) const"},
    {"_ZN26ImageLoaderMachOCompressed22initializeCoalIteratorERN11ImageLoader12CoalIteratorEj", "ImageLoaderMachOCompressed::initializeCoalIterator(ImageLoader::CoalIterator&, unsigned int)"},
    {"_ZN26ImageLoaderMachOCompressed22getAddressCoalIteratorERN11ImageLoader12CoalIteratorERKNS0_11LinkContextE", "ImageLoaderMachOCompressed::getAddressCoalIterator(ImageLoader::CoalIterator&, ImageLoader::LinkContext const&)"},
    {"_ZNK26ImageLoaderMachOCompressed17findClosestSymbolEPKvPS1_", "ImageLoaderMachOCompressed::findClosestSymbol(void const*, void const**) const"},
    {"_ZNK26ImageLoaderMachOCompressed18importedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachOCompressed::importedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZNK26ImageLoaderMachOCompressed21importedSymbolIndexedEj", "ImageLoaderMachOCompressed::importedSymbolIndexed(unsigned int) const"},
    {"_ZNK26ImageLoaderMachOCompressed19importedSymbolCountEv", "ImageLoaderMachOCompressed::importedSymbolCount() const"},
    {"_ZNK26ImageLoaderMachOCompressed21exportedSymbolIndexedEj", "ImageLoaderMachOCompressed::exportedSymbolIndexed(unsigned int) const"},
    {"_ZNK26ImageLoaderMachOCompressed19exportedSymbolCountEv", "ImageLoaderMachOCompressed::exportedSymbolCount() const"},
    {"_ZNK26ImageLoaderMachOCompressed18exportedSymbolNameEPKN11ImageLoader6SymbolE", "ImageLoaderMachOCompressed::exportedSymbolName(ImageLoader::Symbol const*) const"},
    {"_ZN26ImageLoaderMachOCompressed15resolveTwolevelERKN11ImageLoader11LinkContextEPKS0_bPKcPS5_", "ImageLoaderMachOCompressed::resolveTwolevel(ImageLoader::LinkContext const&, ImageLoader const*, bool, char const*, ImageLoader const**)"},
    {"_ZN26ImageLoaderMachOCompressed11resolveFlatERKN11ImageLoader11LinkContextEPKcbPPKS0_", "ImageLoaderMachOCompressed::resolveFlat(ImageLoader::LinkContext const&, char const*, bool, ImageLoader const**)"},
    {"_ZN26ImageLoaderMachOCompressed7resolveERKN11ImageLoader11LinkContextEPKchiPPKS0_PNS_10LastLookupE", "ImageLoaderMachOCompressed::resolve(ImageLoader::LinkContext const&, char const*, unsigned char, int, ImageLoader const**, ImageLoaderMachOCompressed::LastLookup*)"},
    {"_ZN26ImageLoaderMachOCompressed6bindAtERKN11ImageLoader11LinkContextEmhPKchliS5_PNS_10LastLookupE", "ImageLoaderMachOCompressed::bindAt(ImageLoader::LinkContext const&, unsigned long, unsigned char, char const*, unsigned char, long, int, char const*, ImageLoaderMachOCompressed::LastLookup*)"},
    {"_ZN26ImageLoaderMachOCompressed11interposeAtERKN11ImageLoader11LinkContextEmhPKchliS5_PNS_10LastLookupE", "ImageLoaderMachOCompressed::interposeAt(ImageLoader::LinkContext const&, unsigned long, unsigned char, char const*, unsigned char, long, int, char const*, ImageLoaderMachOCompressed::LastLookup*)"},
    {"_ZN26ImageLoaderMachOCompressed16instantiateStartEPK12macho_headerPKcjj", "ImageLoaderMachOCompressed::instantiateStart(macho_header const*, char const*, unsigned int, unsigned int)"},
    {"_ZN26ImageLoaderMachOCompressed16doBindLazySymbolEPmRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::doBindLazySymbol(unsigned long*, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed20instantiateFromCacheEPK12macho_headerPKcRK4statjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::instantiateFromCache(macho_header const*, char const*, stat const&, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed21instantiateFromMemoryEPKcPK12macho_headeryjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::instantiateFromMemory(char const*, macho_header const*, unsigned long long, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed25instantiateMainExecutableEPK12macho_headermPKcjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::instantiateMainExecutable(macho_header const*, unsigned long, char const*, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressedD0Ev", "ImageLoaderMachOCompressed::~ImageLoaderMachOCompressed()"},
    {"_ZN26ImageLoaderMachOCompressedD1Ev", "ImageLoaderMachOCompressed::~ImageLoaderMachOCompressed()"},
    {"_ZN26ImageLoaderMachOCompressed19instantiateFromFileEPKciPKhyyRK4statjjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::instantiateFromFile(char const*, int, unsigned char const*, unsigned long long, unsigned long long, stat const&, unsigned int, unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZNK26ImageLoaderMachOCompressed29exportedSymbolIsWeakDefintionEPKN11ImageLoader6SymbolE", "ImageLoaderMachOCompressed::exportedSymbolIsWeakDefintion(ImageLoader::Symbol const*) const"},
    {"_ZNK26ImageLoaderMachOCompressed21exportedSymbolAddressEPKN11ImageLoader6SymbolE", "ImageLoaderMachOCompressed::exportedSymbolAddress(ImageLoader::Symbol const*) const"},
    {"_ZNK26ImageLoaderMachOCompressed18findExportedSymbolEPKcPPK11ImageLoader", "ImageLoaderMachOCompressed::findExportedSymbol(char const*, ImageLoader const**) const"},
    {"_ZN26ImageLoaderMachOCompressed6rebaseERKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::rebase(ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed22updateUsesCoalIteratorERN11ImageLoader12CoalIteratorEmPS0_RKNS0_11LinkContextE", "ImageLoaderMachOCompressed::updateUsesCoalIterator(ImageLoader::CoalIterator&, unsigned long, ImageLoader*, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed21incrementCoalIteratorERN11ImageLoader12CoalIteratorE", "ImageLoaderMachOCompressed::incrementCoalIterator(ImageLoader::CoalIterator&)"},
    {"_ZN26ImageLoaderMachOCompressed20doBindFastLazySymbolEjRKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::doBindFastLazySymbol(unsigned int, ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed12eachLazyBindERKN11ImageLoader11LinkContextEMS_FmS3_mhPKchliS5_PNS_10LastLookupEE", "ImageLoaderMachOCompressed::eachLazyBind(ImageLoader::LinkContext const&, unsigned long (ImageLoaderMachOCompressed::*)(ImageLoader::LinkContext const&, unsigned long, unsigned char, char const*, unsigned char, long, int, char const*, ImageLoaderMachOCompressed::LastLookup*))"},
    {"_ZN26ImageLoaderMachOCompressed16doBindJustLaziesERKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::doBindJustLazies(ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed8eachBindERKN11ImageLoader11LinkContextEMS_FmS3_mhPKchliS5_PNS_10LastLookupEE", "ImageLoaderMachOCompressed::eachBind(ImageLoader::LinkContext const&, unsigned long (ImageLoaderMachOCompressed::*)(ImageLoader::LinkContext const&, unsigned long, unsigned char, char const*, unsigned char, long, int, char const*, ImageLoaderMachOCompressed::LastLookup*))"},
    {"_ZN26ImageLoaderMachOCompressed11doInterposeERKN11ImageLoader11LinkContextE", "ImageLoaderMachOCompressed::doInterpose(ImageLoader::LinkContext const&)"},
    {"_ZN26ImageLoaderMachOCompressed6doBindERKN11ImageLoader11LinkContextEb", "ImageLoaderMachOCompressed::doBind(ImageLoader::LinkContext const&, bool)"},
    {"_ZN26ImageLoaderMachOCompressed11setDyldInfoEPK17dyld_info_command", "ImageLoaderMachOCompressed::setDyldInfo(dyld_info_command const*)"},
    {"_ZN26ImageLoaderMachOCompressed18setSymbolTableInfoEPK11macho_nlistPKcPK16dysymtab_command", "ImageLoaderMachOCompressed::setSymbolTableInfo(macho_nlist const*, char const*, dysymtab_command const*)"},
    {"_ZNK26ImageLoaderMachOCompressed16isSubframeworkOfERKN11ImageLoader11LinkContextEPKS0_", "ImageLoaderMachOCompressed::isSubframeworkOf(ImageLoader::LinkContext const&, ImageLoader const*) const"},
    {"_ZNK26ImageLoaderMachOCompressed13hasSubLibraryERKN11ImageLoader11LinkContextEPKS0_", "ImageLoaderMachOCompressed::hasSubLibrary(ImageLoader::LinkContext const&, ImageLoader const*) const"},
    {"_ZSt17__throw_bad_allocv", "std::__throw_bad_alloc()"},
    {"_ZSt20__throw_length_errorPKc", "std::__throw_length_error(char const*)"},
    {"_ZSt19__throw_logic_errorPKc", "std::__throw_logic_error(char const*)"},
    {"_ZNSt12length_errorD0Ev", "std::length_error::~length_error()"},
    {"_ZNSt12length_errorD2Ev", "std::length_error::~length_error()"},
    {"_ZNSt12length_errorD1Ev", "std::length_error::~length_error()"},
    {"_ZNKSt11logic_error4whatEv", "std::logic_error::what() const"},
    {"_ZNSt11logic_errorC2ERKSs", "std::logic_error::logic_error(std::string const&)"},
    {"_ZNSt11logic_errorC1ERKSs", "std::logic_error::logic_error(std::string const&)"},
    {"_ZNSt12length_errorC1ERKSs", "std::length_error::length_error(std::string const&)"},
    {"_ZNSt11logic_errorD0Ev", "std::logic_error::~logic_error()"},
    {"_ZNSt11logic_errorD2Ev", "std::logic_error::~logic_error()"},
    {"_ZNSt11logic_errorD1Ev", "std::logic_error::~logic_error()"},
    {"_ZSt18_Rb_tree_decrementPSt18_Rb_tree_node_base", "std::_Rb_tree_decrement(std::_Rb_tree_node_base*)"},
    {"_ZSt29_Rb_tree_insert_and_rebalancebPSt18_Rb_tree_node_baseS0_RS_", "std::_Rb_tree_insert_and_rebalance(bool, std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::_Rb_tree_node_base&)"},
    {"_ZSt18_Rb_tree_incrementPKSt18_Rb_tree_node_base", "std::_Rb_tree_increment(std::_Rb_tree_node_base const*)"},
    {"_ZNSs4_Rep10_M_disposeERKSaIcE", "std::string::_Rep::_M_dispose(std::allocator<char> const&)"},
    {"_ZNSs4_Rep9_S_createEmmRKSaIcE", "std::string::_Rep::_S_create(unsigned long, unsigned long, std::allocator<char> const&)"},
    {"_ZNSs4_Rep8_M_cloneERKSaIcEm", "std::string::_Rep::_M_clone(std::allocator<char> const&, unsigned long)"},
    {"_ZNSs4_Rep7_M_grabERKSaIcES2_", "std::string::_Rep::_M_grab(std::allocator<char> const&, std::allocator<char> const&)"},
    {"_ZNSsC2ERKSs", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(std::string const&)"},
    {"_ZNSsC1ERKSs", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(std::string const&)"},
    {"_ZNSs12_S_constructIPKcEEPcT_S3_RKSaIcESt20forward_iterator_tag", "char* std::string::_S_construct<char const*>(char const*, char const*, std::allocator<char> const&, std::forward_iterator_tag)"},
    {"_ZNSsC2EPKcRKSaIcE", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(char const*, std::allocator<char> const&)"},
    {"_ZNSsC1EPKcRKSaIcE", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(char const*, std::allocator<char> const&)"},
    {"_ZN9__gnu_cxx13new_allocatorIcE8allocateEmPKv", "__gnu_cxx::new_allocator<char>::allocate(unsigned long, void const*)"},
    {"_ZdlPv", "operator delete(void*)"},
    {"_ZdaPv", "operator delete[](void*)"},
    {"_ZNK9__gnu_cxx24__concurrence_lock_error4whatEv", "__gnu_cxx::__concurrence_lock_error::what() const"},
    {"_ZNK9__gnu_cxx26__concurrence_unlock_error4whatEv", "__gnu_cxx::__concurrence_unlock_error::what() const"},
    {"_ZN9__gnu_cxx32__throw_concurrence_unlock_errorEv", "__gnu_cxx::__throw_concurrence_unlock_error()"},
    {"_ZN9__gnu_cxx30__throw_concurrence_lock_errorEv", "__gnu_cxx::__throw_concurrence_lock_error()"},
    {"_ZN9__gnu_cxx7__mutex4lockEv", "__gnu_cxx::__mutex::lock()"},
    {"_ZN9__gnu_cxx26__concurrence_unlock_errorD0Ev", "__gnu_cxx::__concurrence_unlock_error::~__concurrence_unlock_error()"},
    {"_ZN9__gnu_cxx26__concurrence_unlock_errorD2Ev", "__gnu_cxx::__concurrence_unlock_error::~__concurrence_unlock_error()"},
    {"_ZN9__gnu_cxx26__concurrence_unlock_errorD1Ev", "__gnu_cxx::__concurrence_unlock_error::~__concurrence_unlock_error()"},
    {"_ZN9__gnu_cxx24__concurrence_lock_errorD0Ev", "__gnu_cxx::__concurrence_lock_error::~__concurrence_lock_error()"},
    {"_ZN9__gnu_cxx24__concurrence_lock_errorD2Ev", "__gnu_cxx::__concurrence_lock_error::~__concurrence_lock_error()"},
    {"_ZN9__gnu_cxx24__concurrence_lock_errorD1Ev", "__gnu_cxx::__concurrence_lock_error::~__concurrence_lock_error()"},
    {"_ZN9__gnu_cxx7__mutex6unlockEv", "__gnu_cxx::__mutex::unlock()"},
    {"_ZN9__gnu_cxx13__scoped_lockD2Ev", "__gnu_cxx::__scoped_lock::~__scoped_lock()"},
    {"_ZNKSt9exception4whatEv", "std::exception::what() const"},
    {"_ZNKSt13bad_exception4whatEv", "std::bad_exception::what() const"},
    {"_ZNSt13bad_exceptionD0Ev", "std::bad_exception::~bad_exception()"},
    {"_ZNSt9exceptionD0Ev", "std::exception::~exception()"},
    {"_ZNSt9exceptionD2Ev", "std::exception::~exception()"},
    {"_ZNSt13bad_exceptionD1Ev", "std::bad_exception::~bad_exception()"},
    {"_ZNSt9exceptionD1Ev", "std::exception::~exception()"},
    {"_ZNKSt9bad_alloc4whatEv", "std::bad_alloc::what() const"},
    {"_ZNSt9bad_allocD0Ev", "std::bad_alloc::~bad_alloc()"},
    {"_ZNSt9bad_allocD2Ev", "std::bad_alloc::~bad_alloc()"},
    {"_ZNSt9bad_allocD1Ev", "std::bad_alloc::~bad_alloc()"},
    {"_Znwm", "operator new(unsigned long)"},
    {"_Znam", "operator new[](unsigned long)"},
    {"_ZNKSt9type_info14__is_pointer_pEv", "std::type_info::__is_pointer_p() const"},
    {"_ZNKSt9type_info15__is_function_pEv", "std::type_info::__is_function_p() const"},
    {"_ZNKSt9type_info11__do_upcastEPKN10__cxxabiv117__class_type_infoEPPv", "std::type_info::__do_upcast(__cxxabiv1::__class_type_info const*, void**) const"},
    {"_ZNK10__cxxabiv117__class_type_info10__do_catchEPKSt9type_infoPPvj", "__cxxabiv1::__class_type_info::__do_catch(std::type_info const*, void**, unsigned int) const"},
    {"_ZNK10__cxxabiv117__class_type_info11__do_upcastEPKS0_PPv", "__cxxabiv1::__class_type_info::__do_upcast(__cxxabiv1::__class_type_info const*, void**) const"},
    {"_ZNK10__cxxabiv117__class_type_info20__do_find_public_srcElPKvPKS0_S2_", "__cxxabiv1::__class_type_info::__do_find_public_src(long, void const*, __cxxabiv1::__class_type_info const*, void const*) const"},
    {"_ZNK10__cxxabiv117__class_type_info11__do_upcastEPKS0_PKvRNS0_15__upcast_resultE", "__cxxabiv1::__class_type_info::__do_upcast(__cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info::__upcast_result&) const"},
    {"_ZN10__cxxabiv117__class_type_infoD0Ev", "__cxxabiv1::__class_type_info::~__class_type_info()"},
    {"_ZN10__cxxabiv117__class_type_infoD1Ev", "__cxxabiv1::__class_type_info::~__class_type_info()"},
    {"_ZNSt9type_infoD0Ev", "std::type_info::~type_info()"},
    {"_ZNSt9type_infoD1Ev", "std::type_info::~type_info()"},
    {"_ZNSt9type_infoD2Ev", "std::type_info::~type_info()"},
    {"_ZNKSt9type_info10__do_catchEPKS_PPvj", "std::type_info::__do_catch(std::type_info const*, void**, unsigned int) const"},
    {"_ZNK10__cxxabiv117__class_type_info12__do_dyncastElNS0_10__sub_kindEPKS0_PKvS3_S5_RNS0_16__dyncast_resultE", "__cxxabiv1::__class_type_info::__do_dyncast(long, __cxxabiv1::__class_type_info::__sub_kind, __cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info::__dyncast_result&) const"},
    {"_ZNK10__cxxabiv120__si_class_type_info20__do_find_public_srcElPKvPKNS_17__class_type_infoES2_", "__cxxabiv1::__si_class_type_info::__do_find_public_src(long, void const*, __cxxabiv1::__class_type_info const*, void const*) const"},
    {"_ZNK10__cxxabiv120__si_class_type_info12__do_dyncastElNS_17__class_type_info10__sub_kindEPKS1_PKvS4_S6_RNS1_16__dyncast_resultE", "__cxxabiv1::__si_class_type_info::__do_dyncast(long, __cxxabiv1::__class_type_info::__sub_kind, __cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info::__dyncast_result&) const"},
    {"_ZNK10__cxxabiv120__si_class_type_info11__do_upcastEPKNS_17__class_type_infoEPKvRNS1_15__upcast_resultE", "__cxxabiv1::__si_class_type_info::__do_upcast(__cxxabiv1::__class_type_info const*, void const*, __cxxabiv1::__class_type_info::__upcast_result&) const"},
    {"_ZN10__cxxabiv120__si_class_type_infoD0Ev", "__cxxabiv1::__si_class_type_info::~__si_class_type_info()"},
    {"_ZN10__cxxabiv120__si_class_type_infoD2Ev", "__cxxabiv1::__si_class_type_info::~__si_class_type_info()"},
    {"_ZN10__cxxabiv120__si_class_type_infoD1Ev", "__cxxabiv1::__si_class_type_info::~__si_class_type_info()"},
    {"_ZNK10__cxxabiv119__pointer_type_info14__is_pointer_pEv", "__cxxabiv1::__pointer_type_info::__is_pointer_p() const"},
    {"_ZN10__cxxabiv117__pbase_type_infoD0Ev", "__cxxabiv1::__pbase_type_info::~__pbase_type_info()"},
    {"_ZN10__cxxabiv117__pbase_type_infoD2Ev", "__cxxabiv1::__pbase_type_info::~__pbase_type_info()"},
    {"_ZN10__cxxabiv119__pointer_type_infoD0Ev", "__cxxabiv1::__pointer_type_info::~__pointer_type_info()"},
    {"_ZN10__cxxabiv119__pointer_type_infoD2Ev", "__cxxabiv1::__pointer_type_info::~__pointer_type_info()"},
    {"_ZN10__cxxabiv119__pointer_type_infoD1Ev", "__cxxabiv1::__pointer_type_info::~__pointer_type_info()"},
    {"_ZN10__cxxabiv117__pbase_type_infoD1Ev", "__cxxabiv1::__pbase_type_info::~__pbase_type_info()"},
    {"_ZN10__cxxabiv123__fundamental_type_infoD0Ev", "__cxxabiv1::__fundamental_type_info::~__fundamental_type_info()"},
    {"_ZN10__cxxabiv123__fundamental_type_infoD2Ev", "__cxxabiv1::__fundamental_type_info::~__fundamental_type_info()"},
    {"_ZN10__cxxabiv123__fundamental_type_infoD1Ev", "__cxxabiv1::__fundamental_type_info::~__fundamental_type_info()"},
    {"_ZNK10__cxxabiv117__pbase_type_info10__do_catchEPKSt9type_infoPPvj", "__cxxabiv1::__pbase_type_info::__do_catch(std::type_info const*, void**, unsigned int) const"},
    {"_ZNK10__cxxabiv119__pointer_type_info15__pointer_catchEPKNS_17__pbase_type_infoEPPvj", "__cxxabiv1::__pointer_type_info::__pointer_catch(__cxxabiv1::__pbase_type_info const*, void**, unsigned int) const"},
    {"_ZNK10__cxxabiv117__pbase_type_info15__pointer_catchEPKS0_PPvj", "__cxxabiv1::__pbase_type_info::__pointer_catch(__cxxabiv1::__pbase_type_info const*, void**, unsigned int) const"},
    {"_ZN16CSCppTimeoutLockD1Ev", "CSCppTimeoutLock::~CSCppTimeoutLock()"},
    {"_ZN16CSCppTimeoutLockC2EPjj", "CSCppTimeoutLock::CSCppTimeoutLock(unsigned int*, unsigned int)"},
    {"_ZN16CSCppTimeoutLockC1EPjj", "CSCppTimeoutLock::CSCppTimeoutLock(unsigned int*, unsigned int)"},
    {"_ZN25CSCppDyldSharedMemoryPage11send_noticeEjRjS0_", "CSCppDyldSharedMemoryPage::send_notice(unsigned int, unsigned int&, unsigned int&)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE8validRegEi", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::validReg(int)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE13validFloatRegEi", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::validFloatReg(int)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE7getInfoEP15unw_proc_info_t", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getInfo(unw_proc_info_t*)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE15getRegisterNameEi", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getRegisterName(int)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE13isSignalFrameEv", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::isSignalFrame()"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EED0Ev", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::~UnwindCursor()"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE33getInfoFromCompactEncodingSectionEyyy", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getInfoFromCompactEncodingSection(unsigned long long, unsigned long long, unsigned long long)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE24setInfoBasedOnIPRegisterEb", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::setInfoBasedOnIPRegister(bool)"},
    {"_ZN9libunwind17LocalAddressSpace16findFunctionNameEyPcmPy", "libunwind::LocalAddressSpace::findFunctionName(unsigned long long, char*, unsigned long, unsigned long long*)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE15getFunctionNameEPcmPy", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getFunctionName(char*, unsigned long, unsigned long long*)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE6jumptoEv", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::jumpto()"},
    {"_ZN9libunwind22CompactUnwinder_x86_64INS_17LocalAddressSpaceEE32stepWithCompactEncodingFramelessEjyRS1_RNS_16Registers_x86_64Eb", "libunwind::CompactUnwinder_x86_64<libunwind::LocalAddressSpace>::stepWithCompactEncodingFrameless(unsigned int, unsigned long long, libunwind::LocalAddressSpace&, libunwind::Registers_x86_64&, bool)"},
    {"_ZN9libunwind22CompactUnwinder_x86_64INS_17LocalAddressSpaceEE31stepWithCompactEncodingRBPFrameEjyRS1_RNS_16Registers_x86_64E", "libunwind::CompactUnwinder_x86_64<libunwind::LocalAddressSpace>::stepWithCompactEncodingRBPFrame(unsigned int, unsigned long long, libunwind::LocalAddressSpace&, libunwind::Registers_x86_64&)"},
    {"_ZN9libunwind22CompactUnwinder_x86_64INS_17LocalAddressSpaceEE23stepWithCompactEncodingEjyRS1_RNS_16Registers_x86_64E", "libunwind::CompactUnwinder_x86_64<libunwind::LocalAddressSpace>::stepWithCompactEncoding(unsigned int, unsigned long long, libunwind::LocalAddressSpace&, libunwind::Registers_x86_64&)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE23stepWithCompactEncodingEv", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::stepWithCompactEncoding()"},
    {"_ZN9libunwind16Registers_x86_6411setRegisterEiy", "libunwind::Registers_x86_64::setRegister(int, unsigned long long)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE6setRegEiy", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::setReg(int, unsigned long long)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE6getRegEi", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getReg(int)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EED1Ev", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::~UnwindCursor()"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE4stepEv", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::step()"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE11setFloatRegEid", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::setFloatReg(int, double)"},
    {"_ZN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EE11getFloatRegEi", "libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>::getFloatReg(int)"},
    {"_ZN9libunwind16Registers_x86_646jumptoEv", "libunwind::Registers_x86_64::jumpto()"},
    {"_Z41__static_initialization_and_destruction_0ii", "__static_initialization_and_destruction_0(int, int)"},
    {"_Z41__static_initialization_and_destruction_0ii", "__static_initialization_and_destruction_0(int, int)"},
    {"_Z41__static_initialization_and_destruction_0ii", "__static_initialization_and_destruction_0(int, int)"},
    {"_Z41__static_initialization_and_destruction_0ii", "__static_initialization_and_destruction_0(int, int)"},
    {"_Z41__static_initialization_and_destruction_0ii", "__static_initialization_and_destruction_0(int, int)"},
    {"_ZZ34dyld_image_path_containing_addressE8__func__", "dyld_image_path_containing_address::__func__"},
    {"_ZZ40dyld_register_image_state_change_handlerE8__func__", "dyld_register_image_state_change_handler::__func__"},
    {"_ZZ5dlsymE8__func__", "dlsym::__func__"},
    {"_ZZ7dlerrorE8__func__", "dlerror::__func__"},
    {"_ZZ6dladdrE8__func__", "dladdr::__func__"},
    {"_ZZ7dlcloseE8__func__", "dlclose::__func__"},
    {"_ZZ6dlopenE8__func__", "dlopen::__func__"},
    {"_ZZ16dlopen_preflightE8__func__", "dlopen_preflight::__func__"},
    {"_ZZ23_dyld_launched_preboundE8__func__", "_dyld_launched_prebound::__func__"},
    {"_ZZ14NSUnLinkModuleE8__func__", "NSUnLinkModule::__func__"},
    {"_ZZ22NSLibraryNameForModuleE8__func__", "NSLibraryNameForModule::__func__"},
    {"_ZZ14NSNameOfModuleE8__func__", "NSNameOfModule::__func__"},
    {"_ZZ22NSLookupSymbolInModuleE8__func__", "NSLookupSymbolInModule::__func__"},
    {"_ZZ12NSLinkModuleE8__func__", "NSLinkModule::__func__"},
    {"_ZZ34NSIsSymbolDefinedInObjectFileImageE8__func__", "NSIsSymbolDefinedInObjectFileImage::__func__"},
    {"_ZZ33NSGetSectionDataInObjectFileImageE8__func__", "NSGetSectionDataInObjectFileImage::__func__"},
    {"_ZZ38NSSymbolReferenceNameInObjectFileImageE8__func__", "NSSymbolReferenceNameInObjectFileImage::__func__"},
    {"_ZZ39NSSymbolReferenceCountInObjectFileImageE8__func__", "NSSymbolReferenceCountInObjectFileImage::__func__"},
    {"_ZZ39NSSymbolDefinitionNameInObjectFileImageE8__func__", "NSSymbolDefinitionNameInObjectFileImage::__func__"},
    {"_ZZ40NSSymbolDefinitionCountInObjectFileImageE8__func__", "NSSymbolDefinitionCountInObjectFileImage::__func__"},
    {"_ZZ27NSHasModInitObjectFileImageE8__func__", "NSHasModInitObjectFileImage::__func__"},
    {"_ZZ24NSDestroyObjectFileImageE8__func__", "NSDestroyObjectFileImage::__func__"},
    {"_ZZ33NSCreateObjectFileImageFromMemoryE8__func__", "NSCreateObjectFileImageFromMemory::__func__"},
    {"_ZZ31NSCreateObjectFileImageFromFileE8__func__", "NSCreateObjectFileImageFromFile::__func__"},
    {"_ZZ30_dyld_image_containing_addressE8__func__", "_dyld_image_containing_address::__func__"},
    {"_ZZ41_dyld_bind_fully_image_containing_addressE8__func__", "_dyld_bind_fully_image_containing_address::__func__"},
    {"_ZZ22_dyld_bind_objc_moduleE8__func__", "_dyld_bind_objc_module::__func__"},
    {"_ZZ35_dyld_all_twolevel_modules_preboundE8__func__", "_dyld_all_twolevel_modules_prebound::__func__"},
    {"_ZZ17NSModuleForSymbolE8__func__", "NSModuleForSymbol::__func__"},
    {"_ZZ17NSAddressOfSymbolE8__func__", "NSAddressOfSymbol::__func__"},
    {"_ZZ14NSNameOfSymbolE8__func__", "NSNameOfSymbol::__func__"},
    {"_ZZ29NSIsSymbolNameDefinedWithHintE8__func__", "NSIsSymbolNameDefinedWithHint::__func__"},
    {"_ZZ21NSLookupSymbolInImageE8__func__", "NSLookupSymbolInImage::__func__"},
    {"_ZZ28NSIsSymbolNameDefinedInImageE8__func__", "NSIsSymbolNameDefinedInImage::__func__"},
    {"_ZZ25NSAddLibraryWithSearchingE8__func__", "NSAddLibraryWithSearching::__func__"},
    {"_ZZ12NSAddLibraryE8__func__", "NSAddLibrary::__func__"},
    {"_ZZ10NSAddImageE8__func__", "NSAddImage::__func__"},
    {"_ZZ29NSLookupAndBindSymbolWithHintE8__func__", "NSLookupAndBindSymbolWithHint::__func__"},
    {"_ZZ21NSLookupAndBindSymbolE8__func__", "NSLookupAndBindSymbol::__func__"},
    {"_ZZ31_dyld_lookup_and_bind_with_hintE8__func__", "_dyld_lookup_and_bind_with_hint::__func__"},
    {"_ZZ27_dyld_lookup_and_bind_fullyE8__func__", "_dyld_lookup_and_bind_fully::__func__"},
    {"_ZZ36_dyld_register_func_for_remove_imageE8__func__", "_dyld_register_func_for_remove_image::__func__"},
    {"_ZZ33_dyld_register_func_for_add_imageE8__func__", "_dyld_register_func_for_add_image::__func__"},
    {"_ZZ41_dyld_get_image_header_containing_addressE8__func__", "_dyld_get_image_header_containing_address::__func__"},
    {"_ZZ20_dyld_get_image_nameE8__func__", "_dyld_get_image_name::__func__"},
    {"_ZZ21_dyld_get_image_slideE8__func__", "_dyld_get_image_slide::__func__"},
    {"_ZZ28_dyld_get_image_vmaddr_slideE8__func__", "_dyld_get_image_vmaddr_slide::__func__"},
    {"_ZZ22_dyld_get_image_headerE8__func__", "_dyld_get_image_header::__func__"},
    {"_ZZ17_dyld_image_countE8__func__", "_dyld_image_count::__func__"},
    {"_ZZ20_NSGetExecutablePathE8__func__", "_NSGetExecutablePath::__func__"},
    {"_ZTSSt11logic_error", "typeinfo name for std::logic_error"},
    {"_ZTSSt13bad_exception", "typeinfo name for std::bad_exception"},
    {"_ZTSSt9exception", "typeinfo name for std::exception"},
    {"_ZTSSt9bad_alloc", "typeinfo name for std::bad_alloc"},
    {"_ZTSN10__cxxabiv120__si_class_type_infoE", "typeinfo name for __cxxabiv1::__si_class_type_info"},
    {"_ZTSN10__cxxabiv117__class_type_infoE", "typeinfo name for __cxxabiv1::__class_type_info"},
    {"_ZTSSt9type_info", "typeinfo name for std::type_info"},
    {"_ZTSPKc", "typeinfo name for char const*"},
    {"_ZTSPc", "typeinfo name for char*"},
    {"_ZTSc", "typeinfo name for char"},
    {"_ZTSv", "typeinfo name for void"},
    {"_ZTSN10__cxxabiv119__pointer_type_infoE", "typeinfo name for __cxxabiv1::__pointer_type_info"},
    {"_ZTSN10__cxxabiv117__pbase_type_infoE", "typeinfo name for __cxxabiv1::__pbase_type_info"},
    {"_ZTSN10__cxxabiv123__fundamental_type_infoE", "typeinfo name for __cxxabiv1::__fundamental_type_info"},
    {"_ZTSSt12length_error", "typeinfo name for std::length_error"},
    {"_ZTSN9__gnu_cxx24__concurrence_lock_errorE", "typeinfo name for __gnu_cxx::__concurrence_lock_error"},
    {"_ZTSN9__gnu_cxx26__concurrence_unlock_errorE", "typeinfo name for __gnu_cxx::__concurrence_unlock_error"},
    {"_ZTV11ImageLoader", "vtable for ImageLoader"},
    {"_ZTV16ImageLoaderMachO", "vtable for ImageLoaderMachO"},
    {"_ZTV23ImageLoaderMachOClassic", "vtable for ImageLoaderMachOClassic"},
    {"_ZTV26ImageLoaderMachOCompressed", "vtable for ImageLoaderMachOCompressed"},
    {"_ZTISt12length_error", "typeinfo for std::length_error"},
    {"_ZTVSt12length_error", "vtable for std::length_error"},
    {"_ZTVSt11logic_error", "vtable for std::logic_error"},
    {"_ZTISt11logic_error", "typeinfo for std::logic_error"},
    {"_ZTIN9__gnu_cxx24__concurrence_lock_errorE", "typeinfo for __gnu_cxx::__concurrence_lock_error"},
    {"_ZTVN9__gnu_cxx24__concurrence_lock_errorE", "vtable for __gnu_cxx::__concurrence_lock_error"},
    {"_ZTIN9__gnu_cxx26__concurrence_unlock_errorE", "typeinfo for __gnu_cxx::__concurrence_unlock_error"},
    {"_ZTVN9__gnu_cxx26__concurrence_unlock_errorE", "vtable for __gnu_cxx::__concurrence_unlock_error"},
    {"_ZTVSt13bad_exception", "vtable for std::bad_exception"},
    {"_ZTVSt9exception", "vtable for std::exception"},
    {"_ZTISt13bad_exception", "typeinfo for std::bad_exception"},
    {"_ZTISt9exception", "typeinfo for std::exception"},
    {"_ZTVSt9bad_alloc", "vtable for std::bad_alloc"},
    {"_ZTISt9bad_alloc", "typeinfo for std::bad_alloc"},
    {"_ZTVN10__cxxabiv120__si_class_type_infoE", "vtable for __cxxabiv1::__si_class_type_info"},
    {"_ZTVN10__cxxabiv117__class_type_infoE", "vtable for __cxxabiv1::__class_type_info"},
    {"_ZTVSt9type_info", "vtable for std::type_info"},
    {"_ZTIN10__cxxabiv120__si_class_type_infoE", "typeinfo for __cxxabiv1::__si_class_type_info"},
    {"_ZTIN10__cxxabiv117__class_type_infoE", "typeinfo for __cxxabiv1::__class_type_info"},
    {"_ZTISt9type_info", "typeinfo for std::type_info"},
    {"_ZTVN10__cxxabiv119__pointer_type_infoE", "vtable for __cxxabiv1::__pointer_type_info"},
    {"_ZTVN10__cxxabiv117__pbase_type_infoE", "vtable for __cxxabiv1::__pbase_type_info"},
    {"_ZTVN10__cxxabiv123__fundamental_type_infoE", "vtable for __cxxabiv1::__fundamental_type_info"},
    {"_ZTIPKc", "typeinfo for char const*"},
    {"_ZTIPc", "typeinfo for char*"},
    {"_ZTIc", "typeinfo for char"},
    {"_ZTIv", "typeinfo for void"},
    {"_ZTIN10__cxxabiv119__pointer_type_infoE", "typeinfo for __cxxabiv1::__pointer_type_info"},
    {"_ZTIN10__cxxabiv117__pbase_type_infoE", "typeinfo for __cxxabiv1::__pbase_type_info"},
    {"_ZTIN10__cxxabiv123__fundamental_type_infoE", "typeinfo for __cxxabiv1::__fundamental_type_info"},
    {"_ZTVN9libunwind12UnwindCursorINS_17LocalAddressSpaceENS_16Registers_x86_64EEE", "vtable for libunwind::UnwindCursor<libunwind::LocalAddressSpace, libunwind::Registers_x86_64>"},
    {"_ZN4dyld8gLogAPIsE", "dyld::gLogAPIs"},
    {"_ZN4dyld17gLibSystemHelpersE", "dyld::gLibSystemHelpers"},
    {"_ZN10__cxxabiv119__terminate_handlerE", "__cxxabiv1::__terminate_handler"},
    {"_ZN10__cxxabiv120__unexpected_handlerE", "__cxxabiv1::__unexpected_handler"},
    {"_ZN11ImageLoader27fgImagesUsedFromSharedCacheE", "ImageLoader::fgImagesUsedFromSharedCache"},
    {"_ZN11ImageLoader26fgImagesWithUsedPrebindingE", "ImageLoader::fgImagesWithUsedPrebinding"},
    {"_ZN11ImageLoader25fgImagesRequiringNoFixupsE", "ImageLoader::fgImagesRequiringNoFixups"},
    {"_ZN11ImageLoader27fgImagesRequiringCoalescingE", "ImageLoader::fgImagesRequiringCoalescing"},
    {"_ZN11ImageLoader26fgImagesHasWeakDefinitionsE", "ImageLoader::fgImagesHasWeakDefinitions"},
    {"_ZN11ImageLoader19fgTotalRebaseFixupsE", "ImageLoader::fgTotalRebaseFixups"},
    {"_ZN11ImageLoader17fgTotalBindFixupsE", "ImageLoader::fgTotalBindFixups"},
    {"_ZN11ImageLoader26fgTotalBindSymbolsResolvedE", "ImageLoader::fgTotalBindSymbolsResolved"},
    {"_ZN11ImageLoader24fgTotalBindImageSearchesE", "ImageLoader::fgTotalBindImageSearches"},
    {"_ZN11ImageLoader21fgTotalLazyBindFixupsE", "ImageLoader::fgTotalLazyBindFixups"},
    {"_ZN11ImageLoader29fgTotalPossibleLazyBindFixupsE", "ImageLoader::fgTotalPossibleLazyBindFixups"},
    {"_ZN11ImageLoader21fgTotalSegmentsMappedE", "ImageLoader::fgTotalSegmentsMapped"},
    {"_ZN11ImageLoader18fgTotalBytesMappedE", "ImageLoader::fgTotalBytesMapped"},
    {"_ZN11ImageLoader22fgTotalBytesPreFetchedE", "ImageLoader::fgTotalBytesPreFetched"},
    {"_ZN11ImageLoader21fgNextPIEDylibAddressE", "ImageLoader::fgNextPIEDylibAddress"},
    {"_ZN16ImageLoaderMachO26fgSymbolTableBinarySearchsE", "ImageLoaderMachO::fgSymbolTableBinarySearchs"},
    {"_ZN16ImageLoaderMachO19fgSymbolTrieSearchsE", "ImageLoaderMachO::fgSymbolTrieSearchs"},
    {"_ZN4dyld12gLinkContextE", "dyld::gLinkContext"},
    {"_ZN11ImageLoader24fgTotalLoadLibrariesTimeE", "ImageLoader::fgTotalLoadLibrariesTime"},
    {"_ZN11ImageLoader17fgTotalRebaseTimeE", "ImageLoader::fgTotalRebaseTime"},
    {"_ZN11ImageLoader15fgTotalBindTimeE", "ImageLoader::fgTotalBindTime"},
    {"_ZN11ImageLoader19fgTotalWeakBindTimeE", "ImageLoader::fgTotalWeakBindTime"},
    {"_ZN11ImageLoader10fgTotalDOFE", "ImageLoader::fgTotalDOF"},
    {"_ZN11ImageLoader15fgTotalInitTimeE", "ImageLoader::fgTotalInitTime"},
    {"_ZN11ImageLoader23fgDynamicImageReExportsE", "ImageLoader::fgDynamicImageReExports"},
    {"_ZN11ImageLoader19fgInterposingTuplesE", "ImageLoader::fgInterposingTuples"},
    {"_ZNSs4_Rep20_S_empty_rep_storageE", "std::string::_Rep::_S_empty_rep_storage"},
    {"_ZZN23ImageLoaderMachOClassic26bindIndirectSymbolPointersERKN11ImageLoader11LinkContextEbbE13alreadyWarned", "ImageLoaderMachOClassic::bindIndirectSymbolPointers(ImageLoader::LinkContext const&, bool, bool)::alreadyWarned"},
    {"_ZN12_GLOBAL__N_115emergency_mutexE", "(anonymous namespace)::emergency_mutex"},

    // clang

    {"_ZN4llvm8DenseMapIjcNS_12DenseMapInfoIjEENS1_IcEEE6insertERKSt4pairIjcE", "llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> >::insert(std::pair<unsigned int, char> const&)"},
    {"_ZN4llvm8DenseMapIjcNS_12DenseMapInfoIjEENS1_IcEEE16InsertIntoBucketERKjRKcPSt4pairIjcE", "llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> >::InsertIntoBucket(unsigned int const&, char const&, std::pair<unsigned int, char>*)"},
    {"_ZN4llvm8DenseMapIjcNS_12DenseMapInfoIjEENS1_IcEEE4growEj", "llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm19MachinePassRegistry3AddEPNS_23MachinePassRegistryNodeE", "llvm::MachinePassRegistry::Add(llvm::MachinePassRegistryNode*)"},
    {"_ZN4llvm2cl6valuesIiEENS0_11ValuesClassIT_EEPKcS3_S6_z", "llvm::cl::ValuesClass<int> llvm::cl::values<int>(char const*, int, char const*, ...)"},
    {"_ZN4llvm2cl11ValuesClassIiEC2EPKciS4_P13__va_list_tag", "llvm::cl::ValuesClass<int>::ValuesClass(char const*, int, char const*, __va_list_tag*)"},
    {"_ZN4llvm24PassRegistrationListenerC2Ev", "llvm::PassRegistrationListener::PassRegistrationListener()"},
    {"_ZN4llvm24PassRegistrationListener15enumeratePassesEv", "llvm::PassRegistrationListener::enumeratePasses()"},
    {"_ZN4llvm12PassRegistry15getPassRegistryEv", "llvm::PassRegistry::getPassRegistry()"},
    {"_ZNK4llvm12PassRegistry7getImplEv", "llvm::PassRegistry::getImpl() const"},
    {"_ZN4llvm12PassRegistry13enumerateWithEPNS_24PassRegistrationListenerE", "llvm::PassRegistry::enumerateWith(llvm::PassRegistrationListener*)"},
    {"_ZN4llvm12PassRegistry23addRegistrationListenerEPNS_24PassRegistrationListenerE", "llvm::PassRegistry::addRegistrationListener(llvm::PassRegistrationListener*)"},
    {"_ZNSt6vectorIPN4llvm24PassRegistrationListenerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::PassRegistrationListener**, std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> > >, llvm::PassRegistrationListener* const&)"},
    {"_ZN4llvm3sys11MemoryFenceEv", "llvm::sys::MemoryFence()"},
    {"_ZN4llvm2cl6Option11addArgumentEv", "llvm::cl::Option::addArgument()"},
    {"_ZN4llvm2cl18MarkOptionsChangedEv", "llvm::cl::MarkOptionsChanged()"},
    {"_ZNK4llvm17ManagedStaticBase21RegisterManagedStaticEPFPvvEPFvS1_E", "llvm::ManagedStaticBase::RegisterManagedStatic(void* (*)(), void (*)(void*)) const"},
    {"_ZN4llvm3sys9MutexImplC2Eb", "llvm::sys::MutexImpl::MutexImpl(bool)"},
    {"_ZN4llvm3sys9MutexImpl7acquireEv", "llvm::sys::MutexImpl::acquire()"},
    {"_ZN4llvm3sys9MutexImpl7releaseEv", "llvm::sys::MutexImpl::release()"},
    {"_ZN4llvm15SmallVectorBase8grow_podEmm", "llvm::SmallVectorBase::grow_pod(unsigned long, unsigned long)"},
    {"_ZN4llvm3sys15ThreadLocalImplC2Ev", "llvm::sys::ThreadLocalImpl::ThreadLocalImpl()"},
    {"_ZN4llvm21llvm_is_multithreadedEv", "llvm::llvm_is_multithreaded()"},
    {"_ZN4llvm24llvm_acquire_global_lockEv", "llvm::llvm_acquire_global_lock()"},
    {"_ZN4llvm24llvm_release_global_lockEv", "llvm::llvm_release_global_lock()"},
    {"_ZN4llvm15system_categoryEv", "llvm::system_category()"},
    {"_Z8cc1_mainPPKcS1_S0_Pv", "cc1_main(char const**, char const**, char const*, void*)"},
    {"_Z16LLVMErrorHandlerPvRKSs", "LLVMErrorHandler(void*, std::string const&)"},
    {"_ZNK5clang17DiagnosticBuilder9AddStringEN4llvm9StringRefE", "clang::DiagnosticBuilder::AddString(llvm::StringRef) const"},
    {"_ZNK4llvm18IntrusiveRefCntPtrIN5clang10DiagnosticEE6getPtrEv", "llvm::IntrusiveRefCntPtr<clang::Diagnostic>::getPtr() const"},
    {"_ZN5clang16DiagnosticClient15BeginSourceFileERKNS_11LangOptionsEPKNS_12PreprocessorE", "clang::DiagnosticClient::BeginSourceFile(clang::LangOptions const&, clang::Preprocessor const*)"},
    {"_ZN5clang16DiagnosticClient13EndSourceFileEv", "clang::DiagnosticClient::EndSourceFile()"},
    {"_ZN5clang18CompilerInvocationD2Ev", "clang::CompilerInvocation::~CompilerInvocation()"},
    {"_ZN5clang14CodeGenOptionsD2Ev", "clang::CodeGenOptions::~CodeGenOptions()"},
    {"_ZN5clang23DependencyOutputOptionsD2Ev", "clang::DependencyOutputOptions::~DependencyOutputOptions()"},
    {"_ZN5clang15FrontendOptionsD2Ev", "clang::FrontendOptions::~FrontendOptions()"},
    {"_ZNSt6vectorIS_ISsSaISsEESaIS1_EED2Ev", "std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > >::~vector()"},
    {"_ZN5clang19HeaderSearchOptionsD2Ev", "clang::HeaderSearchOptions::~HeaderSearchOptions()"},
    {"_ZN5clang19PreprocessorOptionsD2Ev", "clang::PreprocessorOptions::~PreprocessorOptions()"},
    {"_ZNSt8_Rb_treeISsSsSt9_IdentityISsESt4lessISsESaISsEE8_M_eraseEPSt13_Rb_tree_nodeISsE", "std::_Rb_tree<std::string, std::string, std::_Identity<std::string>, std::less<std::string>, std::allocator<std::string> >::_M_erase(std::_Rb_tree_node<std::string>*)"},
    {"_ZNSt6vectorISt4pairISsSsESaIS1_EED2Ev", "std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > >::~vector()"},
    {"_ZN5clang13TargetOptionsD2Ev", "clang::TargetOptions::~TargetOptions()"},
    {"_ZN5clang18CompilerInvocationC2Ev", "clang::CompilerInvocation::CompilerInvocation()"},
    {"_Z10cc1as_mainPPKcS1_S0_Pv", "cc1as_main(char const**, char const**, char const*, void*)"},
    {"_Z16LLVMErrorHandlerPvRKSs", "LLVMErrorHandler(void*, std::string const&)"},
    {"_ZN4llvm23PrettyStackTraceProgramD1Ev", "llvm::PrettyStackTraceProgram::~PrettyStackTraceProgram()"},
    {"_ZN4llvm23PrettyStackTraceProgramD0Ev", "llvm::PrettyStackTraceProgram::~PrettyStackTraceProgram()"},
    {"_ZN4llvm21formatted_raw_ostream9setStreamERNS_11raw_ostreamEb", "llvm::formatted_raw_ostream::setStream(llvm::raw_ostream&, bool)"},
    {"_ZN4llvm21formatted_raw_ostreamD1Ev", "llvm::formatted_raw_ostream::~formatted_raw_ostream()"},
    {"_ZN4llvm21formatted_raw_ostreamD0Ev", "llvm::formatted_raw_ostream::~formatted_raw_ostream()"},
    {"_ZN4llvm11raw_ostream11changeColorENS0_6ColorsEbb", "llvm::raw_ostream::changeColor(llvm::raw_ostream::Colors, bool, bool)"},
    {"_ZN4llvm11raw_ostream10resetColorEv", "llvm::raw_ostream::resetColor()"},
    {"_ZNK4llvm11raw_ostream12is_displayedEv", "llvm::raw_ostream::is_displayed() const"},
    {"_ZNK4llvm21formatted_raw_ostream11current_posEv", "llvm::formatted_raw_ostream::current_pos() const"},
    {"_ZN4llvm21formatted_raw_ostream13releaseStreamEv", "llvm::formatted_raw_ostream::releaseStream()"},
    {"_ZNSt6vectorISsSaISsEEaSERKS1_", "std::vector<std::string, std::allocator<std::string> >::operator=(std::vector<std::string, std::allocator<std::string> > const&)"},
    {"_ZNSt6vectorIN4llvm9SourceMgr9SrcBufferESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SourceMgr::SrcBuffer, std::allocator<llvm::SourceMgr::SrcBuffer> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SourceMgr::SrcBuffer*, std::vector<llvm::SourceMgr::SrcBuffer, std::allocator<llvm::SourceMgr::SrcBuffer> > >, llvm::SourceMgr::SrcBuffer const&)"},
    {"_ZN4llvm10error_code21unspecified_bool_trueEv", "llvm::error_code::unspecified_bool_true()"},
    {"_Z17GetExecutablePathPKcb", "GetExecutablePath(char const*, bool)"},
    {"_Z17ExpandArgsFromBufPKcRN4llvm15SmallVectorImplIS0_EERSt3setISsSt4lessISsESaISsEE", "ExpandArgsFromBuf(char const*, llvm::SmallVectorImpl<char const*>&, std::set<std::string, std::less<std::string>, std::allocator<std::string> >&)"},
    {"_ZN4llvm15SmallVectorImplIPKcE6insertIN9__gnu_cxx17__normal_iteratorIPS2_St6vectorIS2_SaIS2_EEEEEES7_S7_T_SC_", "char const** llvm::SmallVectorImpl<char const*>::insert<__gnu_cxx::__normal_iterator<char const**, std::vector<char const*, std::allocator<char const*> > > >(char const**, __gnu_cxx::__normal_iterator<char const**, std::vector<char const*, std::allocator<char const*> > >, __gnu_cxx::__normal_iterator<char const**, std::vector<char const*, std::allocator<char const*> > >)"},
    {"_ZNSt6vectorIPKcSaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<char const*, std::allocator<char const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<char const**, std::vector<char const*, std::allocator<char const*> > >, char const* const&)"},
    {"_ZNSt8_Rb_treeISsSsSt9_IdentityISsESt4lessISsESaISsEE16_M_insert_uniqueERKSs", "std::_Rb_tree<std::string, std::string, std::_Identity<std::string>, std::less<std::string>, std::allocator<std::string> >::_M_insert_unique(std::string const&)"},
    {"_ZNSt8_Rb_treeISsSsSt9_IdentityISsESt4lessISsESaISsEE9_M_insertEPSt18_Rb_tree_node_baseS7_RKSs", "std::_Rb_tree<std::string, std::string, std::_Identity<std::string>, std::less<std::string>, std::allocator<std::string> >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::string const&)"},
    {"_ZN4llvm15SmallVectorImplIPKcE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<char const*>::insert(char const**, char const* const&)"},
    {"_ZN5clang25ExecuteCompilerInvocationEPNS_16CompilerInstanceE", "clang::ExecuteCompilerInvocation(clang::CompilerInstance*)"},
    {"_ZN5clang14FrontendAction21BeginSourceFileActionERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::FrontendAction::BeginSourceFileAction(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang14FrontendAction19EndSourceFileActionEv", "clang::FrontendAction::EndSourceFileAction()"},
    {"_ZNK5clang26PreprocessorFrontendAction20usesPreprocessorOnlyEv", "clang::PreprocessorFrontendAction::usesPreprocessorOnly() const"},
    {"_ZN5clang14FrontendAction27usesCompleteTranslationUnitEv", "clang::FrontendAction::usesCompleteTranslationUnit()"},
    {"_ZNK5clang14FrontendAction13hasPCHSupportEv", "clang::FrontendAction::hasPCHSupport() const"},
    {"_ZNK5clang14FrontendAction17hasASTFileSupportEv", "clang::FrontendAction::hasASTFileSupport() const"},
    {"_ZNK5clang14FrontendAction12hasIRSupportEv", "clang::FrontendAction::hasIRSupport() const"},
    {"_ZNK5clang14FrontendAction24hasCodeCompletionSupportEv", "clang::FrontendAction::hasCodeCompletionSupport() const"},
    {"_ZN5clang26PreprocessorFrontendActionD1Ev", "clang::PreprocessorFrontendAction::~PreprocessorFrontendAction()"},
    {"_ZN5clang26PreprocessorFrontendActionD0Ev", "clang::PreprocessorFrontendAction::~PreprocessorFrontendAction()"},
    {"_ZNK5clang17ASTFrontendAction20usesPreprocessorOnlyEv", "clang::ASTFrontendAction::usesPreprocessorOnly() const"},
    {"_ZN5clang17ASTFrontendActionD1Ev", "clang::ASTFrontendAction::~ASTFrontendAction()"},
    {"_ZN5clang17ASTFrontendActionD0Ev", "clang::ASTFrontendAction::~ASTFrontendAction()"},
    {"_ZNK5clang23PrintPreprocessedAction13hasPCHSupportEv", "clang::PrintPreprocessedAction::hasPCHSupport() const"},
    {"_ZN5clang19PrintPreambleAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::PrintPreambleAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZNK5clang19PrintPreambleAction20usesPreprocessorOnlyEv", "clang::PrintPreambleAction::usesPreprocessorOnly() const"},
    {"_ZN5clang16SyntaxOnlyActionD1Ev", "clang::SyntaxOnlyAction::~SyntaxOnlyAction()"},
    {"_ZN5clang16SyntaxOnlyActionD0Ev", "clang::SyntaxOnlyAction::~SyntaxOnlyAction()"},
    {"_ZNK5clang16SyntaxOnlyAction24hasCodeCompletionSupportEv", "clang::SyntaxOnlyAction::hasCodeCompletionSupport() const"},
    {"_ZNK5clang14InitOnlyAction20usesPreprocessorOnlyEv", "clang::InitOnlyAction::usesPreprocessorOnly() const"},
    {"_ZN5clang17GeneratePCHAction27usesCompleteTranslationUnitEv", "clang::GeneratePCHAction::usesCompleteTranslationUnit()"},
    {"_ZNK5clang17GeneratePCHAction17hasASTFileSupportEv", "clang::GeneratePCHAction::hasASTFileSupport() const"},
    {"_ZN5clang16CreateASTPrinterEPN4llvm11raw_ostreamE", "clang::CreateASTPrinter(llvm::raw_ostream*)"},
    {"_ZN5clang19CreateASTPrinterXMLEPN4llvm11raw_ostreamE", "clang::CreateASTPrinterXML(llvm::raw_ostream*)"},
    {"_ZN5clang15CreateASTDumperEv", "clang::CreateASTDumper()"},
    {"_ZN5clang15CreateASTViewerEv", "clang::CreateASTViewer()"},
    {"_ZN5clang24CreateDeclContextPrinterEv", "clang::CreateDeclContextPrinter()"},
    {"_ZN5clang23CreateInheritanceViewerERKSs", "clang::CreateInheritanceViewer(std::string const&)"},
    {"_ZN5clang18CreateASTDumperXMLERN4llvm11raw_ostreamE", "clang::CreateASTDumperXML(llvm::raw_ostream&)"},
    {"_ZN12_GLOBAL__N_110ASTDumpXMLD1Ev", "(anonymous namespace)::ASTDumpXML::~ASTDumpXML()"},
    {"_ZN12_GLOBAL__N_110ASTDumpXMLD0Ev", "(anonymous namespace)::ASTDumpXML::~ASTDumpXML()"},
    {"_ZN12_GLOBAL__N_110ASTDumpXML21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::ASTDumpXML::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_117InheritanceViewerD1Ev", "(anonymous namespace)::InheritanceViewer::~InheritanceViewer()"},
    {"_ZN12_GLOBAL__N_117InheritanceViewerD0Ev", "(anonymous namespace)::InheritanceViewer::~InheritanceViewer()"},
    {"_ZN12_GLOBAL__N_117InheritanceViewer21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::InheritanceViewer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_118DeclContextPrinterD1Ev", "(anonymous namespace)::DeclContextPrinter::~DeclContextPrinter()"},
    {"_ZN12_GLOBAL__N_118DeclContextPrinterD0Ev", "(anonymous namespace)::DeclContextPrinter::~DeclContextPrinter()"},
    {"_ZN12_GLOBAL__N_118DeclContextPrinter21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::DeclContextPrinter::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_118DeclContextPrinter16PrintDeclContextEPKN5clang11DeclContextEj", "(anonymous namespace)::DeclContextPrinter::PrintDeclContext(clang::DeclContext const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_19ASTViewerD1Ev", "(anonymous namespace)::ASTViewer::~ASTViewer()"},
    {"_ZN12_GLOBAL__N_19ASTViewerD0Ev", "(anonymous namespace)::ASTViewer::~ASTViewer()"},
    {"_ZN12_GLOBAL__N_19ASTViewer10InitializeERN5clang10ASTContextE", "(anonymous namespace)::ASTViewer::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_19ASTViewer18HandleTopLevelDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::ASTViewer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_113ASTPrinterXMLD1Ev", "(anonymous namespace)::ASTPrinterXML::~ASTPrinterXML()"},
    {"_ZN12_GLOBAL__N_113ASTPrinterXMLD0Ev", "(anonymous namespace)::ASTPrinterXML::~ASTPrinterXML()"},
    {"_ZN12_GLOBAL__N_113ASTPrinterXML10InitializeERN5clang10ASTContextE", "(anonymous namespace)::ASTPrinterXML::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_113ASTPrinterXML21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::ASTPrinterXML::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_110ASTPrinterD1Ev", "(anonymous namespace)::ASTPrinter::~ASTPrinter()"},
    {"_ZN12_GLOBAL__N_110ASTPrinterD0Ev", "(anonymous namespace)::ASTPrinter::~ASTPrinter()"},
    {"_ZN12_GLOBAL__N_110ASTPrinter21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::ASTPrinter::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZNSt5dequeISsSaISsEED2Ev", "std::deque<std::string, std::allocator<std::string> >::~deque()"},
    {"_ZNSt5dequeISsSaISsEE19_M_destroy_data_auxESt15_Deque_iteratorISsRSsPSsES5_", "std::deque<std::string, std::allocator<std::string> >::_M_destroy_data_aux(std::_Deque_iterator<std::string, std::string&, std::string*>, std::_Deque_iterator<std::string, std::string&, std::string*>)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_jESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, unsigned int>, std::_Select1st<std::pair<clang::QualType const, unsigned int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::QualType const, unsigned int> >*)"},
    {"_ZN5clang14ASTMergeAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTMergeAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang14ASTMergeAction21BeginSourceFileActionERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTMergeAction::BeginSourceFileAction(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang14ASTMergeAction13ExecuteActionEv", "clang::ASTMergeAction::ExecuteAction()"},
    {"_ZN5clang14ASTMergeAction19EndSourceFileActionEv", "clang::ASTMergeAction::EndSourceFileAction()"},
    {"_ZN5clang14ASTMergeActionC1EPNS_14FrontendActionEPSsj", "clang::ASTMergeAction::ASTMergeAction(clang::FrontendAction*, std::string*, unsigned int)"},
    {"_ZN5clang14ASTMergeActionC2EPNS_14FrontendActionEPSsj", "clang::ASTMergeAction::ASTMergeAction(clang::FrontendAction*, std::string*, unsigned int)"},
    {"_ZN5clang14ASTMergeActionD0Ev", "clang::ASTMergeAction::~ASTMergeAction()"},
    {"_ZN5clang14ASTMergeActionD1Ev", "clang::ASTMergeAction::~ASTMergeAction()"},
    {"_ZN5clang14ASTMergeActionD2Ev", "clang::ASTMergeAction::~ASTMergeAction()"},
    {"_ZNK5clang14ASTMergeAction20usesPreprocessorOnlyEv", "clang::ASTMergeAction::usesPreprocessorOnly() const"},
    {"_ZN5clang14ASTMergeAction27usesCompleteTranslationUnitEv", "clang::ASTMergeAction::usesCompleteTranslationUnit()"},
    {"_ZNK5clang14ASTMergeAction13hasPCHSupportEv", "clang::ASTMergeAction::hasPCHSupport() const"},
    {"_ZNK5clang14ASTMergeAction17hasASTFileSupportEv", "clang::ASTMergeAction::hasASTFileSupport() const"},
    {"_ZNK5clang14ASTMergeAction24hasCodeCompletionSupportEv", "clang::ASTMergeAction::hasCodeCompletionSupport() const"},
    {"_ZN5clang7ASTUnitC1Eb", "clang::ASTUnit::ASTUnit(bool)"},
    {"_ZN5clang7ASTUnitC2Eb", "clang::ASTUnit::ASTUnit(bool)"},
    {"_ZN5clang7ASTUnitC3Eb", "clang::ASTUnit::ASTUnit(bool)"},
    {"_ZN5clang7ASTUnitC4Eb", "clang::ASTUnit::ASTUnit(bool)"},
    {"_ZN5clang7ASTUnitC5Eb", "clang::ASTUnit::ASTUnit(bool)"},
    {"_ZN5clang7ASTUnitD0Ev", "clang::ASTUnit::~ASTUnit()"},
    {"_ZN5clang7ASTUnitD1Ev", "clang::ASTUnit::~ASTUnit()"},
    {"_ZN5clang7ASTUnitD2Ev", "clang::ASTUnit::~ASTUnit()"},
    {"_ZN5clang7ASTUnitD4Ev", "clang::ASTUnit::~ASTUnit()"},
    {"_ZN5clang7ASTUnitD5Ev", "clang::ASTUnit::~ASTUnit()"},
    {"_ZN5clang7ASTUnit19CleanTemporaryFilesEv", "clang::ASTUnit::CleanTemporaryFiles()"},
    {"_ZN5clang7ASTUnit28ClearCachedCompletionResultsEv", "clang::ASTUnit::ClearCachedCompletionResults()"},
    {"_ZN5clang7ASTUnit26CacheCodeCompletionResultsEv", "clang::ASTUnit::CacheCodeCompletionResults()"},
    {"_ZNK5clang7ASTUnit15getMainFileNameEv", "clang::ASTUnit::getMainFileName() const"},
    {"_ZN12_GLOBAL__N_111SimpleTimerD1Ev", "(anonymous namespace)::SimpleTimer::~SimpleTimer()"},
    {"_ZN5clang7ASTUnit25getOriginalSourceFileNameEv", "clang::ASTUnit::getOriginalSourceFileName()"},
    {"_ZN5clang7ASTUnit14getASTFileNameEv", "clang::ASTUnit::getASTFileName()"},
    {"_ZN5clang7ASTUnit16getBufferForFileEN4llvm9StringRefEPSs", "clang::ASTUnit::getBufferForFile(llvm::StringRef, std::string*)"},
    {"_ZN5clang7ASTUnit14ConfigureDiagsERN4llvm18IntrusiveRefCntPtrINS_10DiagnosticEEEPPKcS8_RS0_b", "clang::ASTUnit::ConfigureDiags(llvm::IntrusiveRefCntPtr<clang::Diagnostic>&, char const**, char const**, clang::ASTUnit&, bool)"},
    {"_ZN5clang7ASTUnit15LoadFromASTFileERKSsN4llvm18IntrusiveRefCntPtrINS_10DiagnosticEEERKNS_17FileSystemOptionsEbPSt4pairISsPKNS3_12MemoryBufferEEjb", "clang::ASTUnit::LoadFromASTFile(std::string const&, llvm::IntrusiveRefCntPtr<clang::Diagnostic>, clang::FileSystemOptions const&, bool, std::pair<std::string, llvm::MemoryBuffer const*>*, unsigned int, bool)"},
    {"_ZN5clang7ASTUnit5ParseEPN4llvm12MemoryBufferE", "clang::ASTUnit::Parse(llvm::MemoryBuffer*)"},
    {"_ZN5clang7ASTUnit15ComputePreambleERNS_18CompilerInvocationEjRb", "clang::ASTUnit::ComputePreamble(clang::CompilerInvocation&, unsigned int, bool&)"},
    {"_ZN5clang7ASTUnit36getMainBufferWithPrecompiledPreambleENS_18CompilerInvocationEbj", "clang::ASTUnit::getMainBufferWithPrecompiledPreamble(clang::CompilerInvocation, bool, unsigned int)"},
    {"_ZN5clang7ASTUnit32RealizeTopLevelDeclsFromPreambleEv", "clang::ASTUnit::RealizeTopLevelDeclsFromPreamble()"},
    {"_ZN5clang7ASTUnit39RealizePreprocessedEntitiesFromPreambleEv", "clang::ASTUnit::RealizePreprocessedEntitiesFromPreamble()"},
    {"_ZN5clang7ASTUnit15pp_entity_beginEv", "clang::ASTUnit::pp_entity_begin()"},
    {"_ZN5clang7ASTUnit13pp_entity_endEv", "clang::ASTUnit::pp_entity_end()"},
    {"_ZNK5clang7ASTUnit14getMaxPCHLevelEv", "clang::ASTUnit::getMaxPCHLevel() const"},
    {"_ZN5clang7ASTUnit26LoadFromCompilerInvocationEb", "clang::ASTUnit::LoadFromCompilerInvocation(bool)"},
    {"_ZN5clang7ASTUnit26LoadFromCompilerInvocationEPNS_18CompilerInvocationEN4llvm18IntrusiveRefCntPtrINS_10DiagnosticEEEbbbbb", "clang::ASTUnit::LoadFromCompilerInvocation(clang::CompilerInvocation*, llvm::IntrusiveRefCntPtr<clang::Diagnostic>, bool, bool, bool, bool, bool)"},
    {"_ZN5clang7ASTUnit19LoadFromCommandLineEPPKcS3_N4llvm18IntrusiveRefCntPtrINS_10DiagnosticEEENS4_9StringRefEbbPSt4pairISsPKNS4_12MemoryBufferEEjbbbbb", "clang::ASTUnit::LoadFromCommandLine(char const**, char const**, llvm::IntrusiveRefCntPtr<clang::Diagnostic>, llvm::StringRef, bool, bool, std::pair<std::string, llvm::MemoryBuffer const*>*, unsigned int, bool, bool, bool, bool, bool)"},
    {"_ZN5clang7ASTUnit7ReparseEPSt4pairISsPKN4llvm12MemoryBufferEEj", "clang::ASTUnit::Reparse(std::pair<std::string, llvm::MemoryBuffer const*>*, unsigned int)"},
    {"_ZN5clang7ASTUnit12CodeCompleteEN4llvm9StringRefEjjPSt4pairISsPKNS1_12MemoryBufferEEjbbRNS_20CodeCompleteConsumerERNS_10DiagnosticERNS_11LangOptionsERNS_13SourceManagerERNS_11FileManagerERNS1_15SmallVectorImplINS_16StoredDiagnosticEEERNSJ_IS6_EE", "clang::ASTUnit::CodeComplete(llvm::StringRef, unsigned int, unsigned int, std::pair<std::string, llvm::MemoryBuffer const*>*, unsigned int, bool, bool, clang::CodeCompleteConsumer&, clang::Diagnostic&, clang::LangOptions&, clang::SourceManager&, clang::FileManager&, llvm::SmallVectorImpl<clang::StoredDiagnostic>&, llvm::SmallVectorImpl<llvm::MemoryBuffer const*>&)"},
    {"_ZN5clang7ASTUnit4SaveEN4llvm9StringRefE", "clang::ASTUnit::Save(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_129AugmentedCodeCompleteConsumerD1Ev", "(anonymous namespace)::AugmentedCodeCompleteConsumer::~AugmentedCodeCompleteConsumer()"},
    {"_ZN12_GLOBAL__N_129AugmentedCodeCompleteConsumerD0Ev", "(anonymous namespace)::AugmentedCodeCompleteConsumer::~AugmentedCodeCompleteConsumer()"},
    {"_ZN12_GLOBAL__N_129AugmentedCodeCompleteConsumer26ProcessCodeCompleteResultsERN5clang4SemaENS1_21CodeCompletionContextEPNS1_20CodeCompletionResultEj", "(anonymous namespace)::AugmentedCodeCompleteConsumer::ProcessCodeCompleteResults(clang::Sema&, clang::CodeCompletionContext, clang::CodeCompletionResult*, unsigned int)"},
    {"_ZN12_GLOBAL__N_129AugmentedCodeCompleteConsumer25ProcessOverloadCandidatesERN5clang4SemaEjPNS1_20CodeCompleteConsumer17OverloadCandidateEj", "(anonymous namespace)::AugmentedCodeCompleteConsumer::ProcessOverloadCandidates(clang::Sema&, unsigned int, clang::CodeCompleteConsumer::OverloadCandidate*, unsigned int)"},
    {"_ZN12_GLOBAL__N_129AugmentedCodeCompleteConsumer12getAllocatorEv", "(anonymous namespace)::AugmentedCodeCompleteConsumer::getAllocator()"},
    {"_ZN12_GLOBAL__N_122StoredDiagnosticClientD1Ev", "(anonymous namespace)::StoredDiagnosticClient::~StoredDiagnosticClient()"},
    {"_ZN12_GLOBAL__N_122StoredDiagnosticClientD0Ev", "(anonymous namespace)::StoredDiagnosticClient::~StoredDiagnosticClient()"},
    {"_ZN12_GLOBAL__N_122StoredDiagnosticClient16HandleDiagnosticEN5clang10Diagnostic5LevelERKNS1_14DiagnosticInfoE", "(anonymous namespace)::StoredDiagnosticClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN12_GLOBAL__N_124PrecompilePreambleAction17CreateASTConsumerERN5clang16CompilerInstanceEN4llvm9StringRefE", "(anonymous namespace)::PrecompilePreambleAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_124PrecompilePreambleActionD1Ev", "(anonymous namespace)::PrecompilePreambleAction::~PrecompilePreambleAction()"},
    {"_ZN12_GLOBAL__N_124PrecompilePreambleActionD0Ev", "(anonymous namespace)::PrecompilePreambleAction::~PrecompilePreambleAction()"},
    {"_ZN12_GLOBAL__N_124PrecompilePreambleAction27usesCompleteTranslationUnitEv", "(anonymous namespace)::PrecompilePreambleAction::usesCompleteTranslationUnit()"},
    {"_ZNK12_GLOBAL__N_124PrecompilePreambleAction17hasASTFileSupportEv", "(anonymous namespace)::PrecompilePreambleAction::hasASTFileSupport() const"},
    {"_ZNK12_GLOBAL__N_124PrecompilePreambleAction24hasCodeCompletionSupportEv", "(anonymous namespace)::PrecompilePreambleAction::hasCodeCompletionSupport() const"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumerD1Ev", "(anonymous namespace)::PrecompilePreambleConsumer::~PrecompilePreambleConsumer()"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumerD0Ev", "(anonymous namespace)::PrecompilePreambleConsumer::~PrecompilePreambleConsumer()"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumer18HandleTopLevelDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::PrecompilePreambleConsumer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumer21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::PrecompilePreambleConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumer27GetASTSerializationListenerEv", "(anonymous namespace)::PrecompilePreambleConsumer::GetASTSerializationListener()"},
    {"_ZN12_GLOBAL__N_126PrecompilePreambleConsumer28SerializedPreprocessedEntityEPN5clang18PreprocessedEntityEy", "(anonymous namespace)::PrecompilePreambleConsumer::SerializedPreprocessedEntity(clang::PreprocessedEntity*, unsigned long long)"},
    {"_ZThn1912_N12_GLOBAL__N_126PrecompilePreambleConsumerD1Ev", "non-virtual thunk to (anonymous namespace)::PrecompilePreambleConsumer::~PrecompilePreambleConsumer()"},
    {"_ZThn1912_N12_GLOBAL__N_126PrecompilePreambleConsumerD0Ev", "non-virtual thunk to (anonymous namespace)::PrecompilePreambleConsumer::~PrecompilePreambleConsumer()"},
    {"_ZThn1912_N12_GLOBAL__N_126PrecompilePreambleConsumer28SerializedPreprocessedEntityEPN5clang18PreprocessedEntityEy", "non-virtual thunk to (anonymous namespace)::PrecompilePreambleConsumer::SerializedPreprocessedEntity(clang::PreprocessedEntity*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_125TopLevelDeclTrackerAction17CreateASTConsumerERN5clang16CompilerInstanceEN4llvm9StringRefE", "(anonymous namespace)::TopLevelDeclTrackerAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_125TopLevelDeclTrackerActionD1Ev", "(anonymous namespace)::TopLevelDeclTrackerAction::~TopLevelDeclTrackerAction()"},
    {"_ZN12_GLOBAL__N_125TopLevelDeclTrackerActionD0Ev", "(anonymous namespace)::TopLevelDeclTrackerAction::~TopLevelDeclTrackerAction()"},
    {"_ZN12_GLOBAL__N_125TopLevelDeclTrackerAction27usesCompleteTranslationUnitEv", "(anonymous namespace)::TopLevelDeclTrackerAction::usesCompleteTranslationUnit()"},
    {"_ZNK12_GLOBAL__N_125TopLevelDeclTrackerAction24hasCodeCompletionSupportEv", "(anonymous namespace)::TopLevelDeclTrackerAction::hasCodeCompletionSupport() const"},
    {"_ZN12_GLOBAL__N_127TopLevelDeclTrackerConsumerD1Ev", "(anonymous namespace)::TopLevelDeclTrackerConsumer::~TopLevelDeclTrackerConsumer()"},
    {"_ZN12_GLOBAL__N_127TopLevelDeclTrackerConsumerD0Ev", "(anonymous namespace)::TopLevelDeclTrackerConsumer::~TopLevelDeclTrackerConsumer()"},
    {"_ZN12_GLOBAL__N_127TopLevelDeclTrackerConsumer18HandleTopLevelDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::TopLevelDeclTrackerConsumer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_127TopLevelDeclTrackerConsumer21HandleInterestingDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::TopLevelDeclTrackerConsumer::HandleInterestingDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollectorD1Ev", "(anonymous namespace)::ASTInfoCollector::~ASTInfoCollector()"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollectorD0Ev", "(anonymous namespace)::ASTInfoCollector::~ASTInfoCollector()"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollector19ReadLanguageOptionsERKN5clang11LangOptionsE", "(anonymous namespace)::ASTInfoCollector::ReadLanguageOptions(clang::LangOptions const&)"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollector16ReadTargetTripleEN4llvm9StringRefE", "(anonymous namespace)::ASTInfoCollector::ReadTargetTriple(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollector20ReadPredefinesBufferERKN4llvm11SmallVectorIN5clang18PCHPredefinesBlockELj2EEENS1_9StringRefERSs", "(anonymous namespace)::ASTInfoCollector::ReadPredefinesBuffer(llvm::SmallVector<clang::PCHPredefinesBlock, 2u> const&, llvm::StringRef, std::string&)"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollector18ReadHeaderFileInfoERKN5clang14HeaderFileInfoEj", "(anonymous namespace)::ASTInfoCollector::ReadHeaderFileInfo(clang::HeaderFileInfo const&, unsigned int)"},
    {"_ZN12_GLOBAL__N_116ASTInfoCollector11ReadCounterEj", "(anonymous namespace)::ASTInfoCollector::ReadCounter(unsigned int)"},
    {"_ZN5clang19PreprocessorOptions15addRemappedFileEN4llvm9StringRefEPKNS1_12MemoryBufferE", "clang::PreprocessorOptions::addRemappedFile(llvm::StringRef, llvm::MemoryBuffer const*)"},
    {"_ZN4llvm15SmallVectorImplIPKcE6insertIPS2_EES5_S5_T_S6_", "char const** llvm::SmallVectorImpl<char const*>::insert<char const**>(char const**, char const**, char const**)"},
    {"_ZN4llvm15SmallVectorImplIN5clang16StoredDiagnosticEE4swapERS3_", "llvm::SmallVectorImpl<clang::StoredDiagnostic>::swap(llvm::SmallVectorImpl<clang::StoredDiagnostic>&)"},
    {"_ZN5clang11LangOptionsaSERKS0_", "clang::LangOptions::operator=(clang::LangOptions const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang16StoredDiagnosticEE6insertIPS2_EES5_S5_T_S6_", "clang::StoredDiagnostic* llvm::SmallVectorImpl<clang::StoredDiagnostic>::insert<clang::StoredDiagnostic*>(clang::StoredDiagnostic*, clang::StoredDiagnostic*, clang::StoredDiagnostic*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang16StoredDiagnosticEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::StoredDiagnostic>::push_back(clang::StoredDiagnostic const&)"},
    {"_ZN5clang9ASTWriterD1Ev", "clang::ASTWriter::~ASTWriter()"},
    {"_ZN4llvm15SmallVectorImplIN5clang16StoredDiagnosticEE6appendIPS2_EEvT_S6_", "void llvm::SmallVectorImpl<clang::StoredDiagnostic>::append<clang::StoredDiagnostic*>(clang::StoredDiagnostic*, clang::StoredDiagnostic*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang16StoredDiagnosticELb0EE18uninitialized_copyIPS2_S5_EEvT_S6_T0_", "void llvm::SmallVectorTemplateBase<clang::StoredDiagnostic, false>::uninitialized_copy<clang::StoredDiagnostic*, clang::StoredDiagnostic*>(clang::StoredDiagnostic*, clang::StoredDiagnostic*, clang::StoredDiagnostic*)"},
    {"_ZNSt6vectorIN5clang15CharSourceRangeESaIS1_EEaSERKS3_", "std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> >::operator=(std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> > const&)"},
    {"_ZNSt6vectorIN5clang9FixItHintESaIS1_EEaSERKS3_", "std::vector<clang::FixItHint, std::allocator<clang::FixItHint> >::operator=(std::vector<clang::FixItHint, std::allocator<clang::FixItHint> > const&)"},
    {"_ZNSt6vectorIN5clang9FixItHintESaIS1_EEC2ERKS3_", "std::vector<clang::FixItHint, std::allocator<clang::FixItHint> >::vector(std::vector<clang::FixItHint, std::allocator<clang::FixItHint> > const&)"},
    {"_ZNSt6vectorIN5clang15CharSourceRangeESaIS1_EEC2ERKS3_", "std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> >::vector(std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> > const&)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPN5clang16StoredDiagnosticES5_EET0_T_S7_S6_", "clang::StoredDiagnostic* std::__copy<false, std::random_access_iterator_tag>::copy<clang::StoredDiagnostic*, clang::StoredDiagnostic*>(clang::StoredDiagnostic*, clang::StoredDiagnostic*, clang::StoredDiagnostic*)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN5clang16StoredDiagnosticES5_EET0_T_S7_S6_", "clang::StoredDiagnostic* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<clang::StoredDiagnostic*, clang::StoredDiagnostic*>(clang::StoredDiagnostic*, clang::StoredDiagnostic*, clang::StoredDiagnostic*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang16StoredDiagnosticELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::StoredDiagnostic, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIPN5clang18PreprocessedEntityESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS2_S4_EEEEvS9_T_SA_St20forward_iterator_tag", "void std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> >::_M_range_insert<__gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > > >(__gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, __gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, __gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, std::forward_iterator_tag)"},
    {"_ZNSt6vectorIPN5clang18PreprocessedEntityESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, clang::PreprocessedEntity* const&)"},
    {"_ZNSt6vectorIPN5clang4DeclESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS2_S4_EEEEvS9_T_SA_St20forward_iterator_tag", "void std::vector<clang::Decl*, std::allocator<clang::Decl*> >::_M_range_insert<__gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > > >(__gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > >, __gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > >, __gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > >, std::forward_iterator_tag)"},
    {"_ZN4llvm9StringMapISt4pairIxlENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<std::pair<long long, long> >& llvm::StringMap<std::pair<long long, long>, llvm::MallocAllocator>::GetOrCreateValue<std::pair<long long, long> >(llvm::StringRef, std::pair<long long, long>)"},
    {"_ZNSt8_Rb_treeIN5clang6FileIDESt4pairIKS1_St6vectorIPNS0_18PreprocessedEntityESaIS6_EEESt10_Select1stIS9_ESt4lessIS1_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<clang::FileID, std::pair<clang::FileID const, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, std::_Select1st<std::pair<clang::FileID const, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > > >, std::less<clang::FileID>, std::allocator<std::pair<clang::FileID const, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::FileID const, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > > >*)"},
    {"_ZNSt6vectorIN5clang7ASTUnit26CachedCodeCompletionResultESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::ASTUnit::CachedCodeCompletionResult, std::allocator<clang::ASTUnit::CachedCodeCompletionResult> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ASTUnit::CachedCodeCompletionResult*, std::vector<clang::ASTUnit::CachedCodeCompletionResult, std::allocator<clang::ASTUnit::CachedCodeCompletionResult> > >, clang::ASTUnit::CachedCodeCompletionResult const&)"},
    {"_ZN4llvm8DenseMapIN5clang7CanQualINS1_4TypeEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::CanQual<clang::Type>, unsigned int, llvm::DenseMapInfo<clang::CanQual<clang::Type> >, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::CanQual<clang::Type> const&, unsigned int const&, std::pair<clang::CanQual<clang::Type>, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIN5clang7CanQualINS1_4TypeEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::CanQual<clang::Type>, unsigned int, llvm::DenseMapInfo<clang::CanQual<clang::Type> >, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN5clang4DeclESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::Decl*, std::allocator<clang::Decl*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > >, clang::Decl* const&)"},
    {"_ZN4llvm15BitstreamWriterD2Ev", "llvm::BitstreamWriter::~BitstreamWriter()"},
    {"_ZThn8_N5clang9ASTWriterD1Ev", "non-virtual thunk to clang::ASTWriter::~ASTWriter()"},
    {"_ZN5clang9ASTWriterD2Ev", "clang::ASTWriter::~ASTWriter()"},
    {"_ZN5clang9ASTWriterD0Ev", "clang::ASTWriter::~ASTWriter()"},
    {"_ZThn8_N5clang9ASTWriterD0Ev", "non-virtual thunk to clang::ASTWriter::~ASTWriter()"},
    {"_ZNSt8_Rb_treeIPN5clang10SwitchCaseESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::SwitchCase*, std::pair<clang::SwitchCase* const, unsigned int>, std::_Select1st<std::pair<clang::SwitchCase* const, unsigned int> >, std::less<clang::SwitchCase*>, std::allocator<std::pair<clang::SwitchCase* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::SwitchCase* const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeIPN5clang9LabelStmtESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::LabelStmt*, std::pair<clang::LabelStmt* const, unsigned int>, std::_Select1st<std::pair<clang::LabelStmt* const, unsigned int> >, std::less<clang::LabelStmt*>, std::allocator<std::pair<clang::LabelStmt* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::LabelStmt* const, unsigned int> >*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang20CodeCompletionResultEE6insertIPS2_EES5_S5_T_S6_", "clang::CodeCompletionResult* llvm::SmallVectorImpl<clang::CodeCompletionResult>::insert<clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang20CodeCompletionResultELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeCompletionResult, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang20CodeCompletionString5ChunkELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeCompletionString::Chunk, false>::grow(unsigned long)"},
    {"_ZN4llvm9StringSetINS_16BumpPtrAllocatorEE6insertENS_9StringRefE", "llvm::StringSet<llvm::BumpPtrAllocator>::insert(llvm::StringRef)"},
    {"_ZNSt6vectorISt4pairISsSsESaIS1_EE5clearEv", "std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > >::clear()"},
    {"_ZN5clang18CompilerInvocationC2ERKS0_", "clang::CompilerInvocation::CompilerInvocation(clang::CompilerInvocation const&)"},
    {"_ZN5clang19PreprocessorOptionsC2ERKS0_", "clang::PreprocessorOptions::PreprocessorOptions(clang::PreprocessorOptions const&)"},
    {"_ZNSt6vectorISt4pairISsPKN4llvm12MemoryBufferEESaIS5_EEC2ERKS7_", "std::vector<std::pair<std::string, llvm::MemoryBuffer const*>, std::allocator<std::pair<std::string, llvm::MemoryBuffer const*> > >::vector(std::vector<std::pair<std::string, llvm::MemoryBuffer const*>, std::allocator<std::pair<std::string, llvm::MemoryBuffer const*> > > const&)"},
    {"_ZNSt6vectorISt4pairISsSsESaIS1_EEC2ERKS3_", "std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > >::vector(std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > > const&)"},
    {"_ZNSt6vectorISt4pairISsbESaIS1_EEC2ERKS3_", "std::vector<std::pair<std::string, bool>, std::allocator<std::pair<std::string, bool> > >::vector(std::vector<std::pair<std::string, bool>, std::allocator<std::pair<std::string, bool> > > const&)"},
    {"_ZNSt6vectorIN5clang19HeaderSearchOptions5EntryESaIS2_EEC2ERKS4_", "std::vector<clang::HeaderSearchOptions::Entry, std::allocator<clang::HeaderSearchOptions::Entry> >::vector(std::vector<clang::HeaderSearchOptions::Entry, std::allocator<clang::HeaderSearchOptions::Entry> > const&)"},
    {"_ZN5clang15FrontendOptionsC2ERKS0_", "clang::FrontendOptions::FrontendOptions(clang::FrontendOptions const&)"},
    {"_ZNSt6vectorIS_ISsSaISsEESaIS1_EEC2ERKS3_", "std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > >::vector(std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > > const&)"},
    {"_ZNSt6vectorISt4pairIN5clang9InputKindESsESaIS3_EEC2ERKS5_", "std::vector<std::pair<clang::InputKind, std::string>, std::allocator<std::pair<clang::InputKind, std::string> > >::vector(std::vector<std::pair<clang::InputKind, std::string>, std::allocator<std::pair<clang::InputKind, std::string> > > const&)"},
    {"_ZN5clang17DiagnosticOptionsC2ERKS0_", "clang::DiagnosticOptions::DiagnosticOptions(clang::DiagnosticOptions const&)"},
    {"_ZN5clang14CodeGenOptionsC2ERKS0_", "clang::CodeGenOptions::CodeGenOptions(clang::CodeGenOptions const&)"},
    {"_ZN5clang15AnalyzerOptionsC2ERKS0_", "clang::AnalyzerOptions::AnalyzerOptions(clang::AnalyzerOptions const&)"},
    {"_ZNSt6vectorIN5clang8AnalysesESaIS1_EEC2ERKS3_", "std::vector<clang::Analyses, std::allocator<clang::Analyses> >::vector(std::vector<clang::Analyses, std::allocator<clang::Analyses> > const&)"},
    {"_ZN5clang12PCHGenerator14InitializeSemaERNS_4SemaE", "clang::PCHGenerator::InitializeSema(clang::Sema&)"},
    {"_ZNSt6vectorIySaIyEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPyS1_EERKy", "std::vector<unsigned long long, std::allocator<unsigned long long> >::_M_insert_aux(__gnu_cxx::__normal_iterator<unsigned long long*, std::vector<unsigned long long, std::allocator<unsigned long long> > >, unsigned long long const&)"},
    {"_ZN5clang12PCHGeneratorD1Ev", "clang::PCHGenerator::~PCHGenerator()"},
    {"_ZN5clang12PCHGeneratorD0Ev", "clang::PCHGenerator::~PCHGenerator()"},
    {"_ZNSt6vectorISt4pairISsPKN4llvm12MemoryBufferEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<std::string, llvm::MemoryBuffer const*>, std::allocator<std::pair<std::string, llvm::MemoryBuffer const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<std::string, llvm::MemoryBuffer const*>*, std::vector<std::pair<std::string, llvm::MemoryBuffer const*>, std::allocator<std::pair<std::string, llvm::MemoryBuffer const*> > > >, std::pair<std::string, llvm::MemoryBuffer const*> const&)"},
    {"_ZN5clang14BoostConAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::BoostConAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_119BoostConASTConsumerD1Ev", "(anonymous namespace)::BoostConASTConsumer::~BoostConASTConsumer()"},
    {"_ZN12_GLOBAL__N_119BoostConASTConsumerD0Ev", "(anonymous namespace)::BoostConASTConsumer::~BoostConASTConsumer()"},
    {"_ZN12_GLOBAL__N_119BoostConASTConsumer21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::BoostConASTConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE12TraverseDeclEPNS_4DeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseDecl(clang::Decl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE12TraverseStmtEPNS_4StmtE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseStmt(clang::Stmt*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE15TraverseTypeLocENS_7TypeLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseTypeLoc(clang::TypeLoc)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE27TraverseNestedNameSpecifierEPNS_19NestedNameSpecifierE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseNestedNameSpecifier(clang::NestedNameSpecifier*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE27TraverseTemplateArgumentLocERKNS_19TemplateArgumentLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseTemplateArgumentLoc(clang::TemplateArgumentLoc const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE24TraverseTemplateArgumentERKNS_16TemplateArgumentE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE12TraverseTypeENS_8QualTypeE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseType(clang::QualType)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE24TraverseDeclaratorHelperEPNS_14DeclaratorDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseDeclaratorHelper(clang::DeclaratorDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE22TraverseFunctionHelperEPNS_12FunctionDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseFunctionHelper(clang::FunctionDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE23TraverseCXXRecordHelperEPNS_13CXXRecordDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::TraverseCXXRecordHelper(clang::CXXRecordDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119BoostConASTConsumerEE23WalkUpFromCXXRecordDeclEPNS_13CXXRecordDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::BoostConASTConsumer>::WalkUpFromCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_ZN5clang14BoostConActionD1Ev", "clang::BoostConAction::~BoostConAction()"},
    {"_ZN5clang14BoostConActionD0Ev", "clang::BoostConAction::~BoostConAction()"},
    {"_ZNK5clang38DependentTemplateSpecializationTypeLoc9getArgLocEj", "clang::DependentTemplateSpecializationTypeLoc::getArgLoc(unsigned int) const"},
    {"_ZNK5clang29TemplateSpecializationTypeLoc9getArgLocEj", "clang::TemplateSpecializationTypeLoc::getArgLoc(unsigned int) const"},
    {"_ZN5clang11CacheTokensERNS_12PreprocessorEPN4llvm14raw_fd_ostreamE", "clang::CacheTokens(clang::Preprocessor&, llvm::raw_fd_ostream*)"},
    {"_ZN5clang31OnDiskChainedHashTableGeneratorIN12_GLOBAL__N_121FileEntryPTHEntryInfoEE6insertENS1_18PTHEntryKeyVariantERKNS1_8PTHEntryE", "clang::OnDiskChainedHashTableGenerator<(anonymous namespace)::FileEntryPTHEntryInfo>::insert((anonymous namespace)::PTHEntryKeyVariant, (anonymous namespace)::PTHEntry const&)"},
    {"_ZN12_GLOBAL__N_19PTHWriter9EmitTokenERKN5clang5TokenE", "(anonymous namespace)::PTHWriter::EmitToken(clang::Token const&)"},
    {"_ZN12_GLOBAL__N_112StatListenerD1Ev", "(anonymous namespace)::StatListener::~StatListener()"},
    {"_ZN12_GLOBAL__N_112StatListenerD0Ev", "(anonymous namespace)::StatListener::~StatListener()"},
    {"_ZN12_GLOBAL__N_112StatListener7getStatEPKcR4statPi", "(anonymous namespace)::StatListener::getStat(char const*, stat&, int*)"},
    {"_ZN5clang2io6Emit32ERN4llvm11raw_ostreamEj", "clang::io::Emit32(llvm::raw_ostream&, unsigned int)"},
    {"_ZN5clang2io6Emit64ERN4llvm11raw_ostreamEy", "clang::io::Emit64(llvm::raw_ostream&, unsigned long long)"},
    {"_ZN4llvm8DenseMapIPKN5clang14IdentifierInfoEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::IdentifierInfo const*, unsigned int, llvm::DenseMapInfo<clang::IdentifierInfo const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::IdentifierInfo const* const&, unsigned int const&, std::pair<clang::IdentifierInfo const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang14IdentifierInfoEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::IdentifierInfo const*, unsigned int, llvm::DenseMapInfo<clang::IdentifierInfo const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang16CompilerInstanceC1Ev", "clang::CompilerInstance::CompilerInstance()"},
    {"_ZN5clang16CompilerInstanceC2Ev", "clang::CompilerInstance::CompilerInstance()"},
    {"_ZN5clang16CompilerInstanceD1Ev", "clang::CompilerInstance::~CompilerInstance()"},
    {"_ZN5clang16CompilerInstanceD2Ev", "clang::CompilerInstance::~CompilerInstance()"},
    {"_ZN5clang16CompilerInstance14setLLVMContextEPN4llvm11LLVMContextE", "clang::CompilerInstance::setLLVMContext(llvm::LLVMContext*)"},
    {"_ZN5clang16CompilerInstance13setInvocationEPNS_18CompilerInvocationE", "clang::CompilerInstance::setInvocation(clang::CompilerInvocation*)"},
    {"_ZN5clang16CompilerInstance14setDiagnosticsEPNS_10DiagnosticE", "clang::CompilerInstance::setDiagnostics(clang::Diagnostic*)"},
    {"_ZN5clang16CompilerInstance9setTargetEPNS_10TargetInfoE", "clang::CompilerInstance::setTarget(clang::TargetInfo*)"},
    {"_ZN5clang16CompilerInstance14setFileManagerEPNS_11FileManagerE", "clang::CompilerInstance::setFileManager(clang::FileManager*)"},
    {"_ZN5clang16CompilerInstance16setSourceManagerEPNS_13SourceManagerE", "clang::CompilerInstance::setSourceManager(clang::SourceManager*)"},
    {"_ZN5clang16CompilerInstance15setPreprocessorEPNS_12PreprocessorE", "clang::CompilerInstance::setPreprocessor(clang::Preprocessor*)"},
    {"_ZN5clang16CompilerInstance13setASTContextEPNS_10ASTContextE", "clang::CompilerInstance::setASTContext(clang::ASTContext*)"},
    {"_ZN5clang16CompilerInstance7setSemaEPNS_4SemaE", "clang::CompilerInstance::setSema(clang::Sema*)"},
    {"_ZN5clang16CompilerInstance14setASTConsumerEPNS_11ASTConsumerE", "clang::CompilerInstance::setASTConsumer(clang::ASTConsumer*)"},
    {"_ZN5clang16CompilerInstance25setCodeCompletionConsumerEPNS_20CodeCompleteConsumerE", "clang::CompilerInstance::setCodeCompletionConsumer(clang::CodeCompleteConsumer*)"},
    {"_ZN5clang16CompilerInstance17createDiagnosticsEiPKPKcPNS_16DiagnosticClientE", "clang::CompilerInstance::createDiagnostics(int, char const* const*, clang::DiagnosticClient*)"},
    {"_ZN5clang16CompilerInstance17createDiagnosticsERKNS_17DiagnosticOptionsEiPKPKcPNS_16DiagnosticClientE", "clang::CompilerInstance::createDiagnostics(clang::DiagnosticOptions const&, int, char const* const*, clang::DiagnosticClient*)"},
    {"_ZN5clang16CompilerInstance17createFileManagerEv", "clang::CompilerInstance::createFileManager()"},
    {"_ZN5clang16CompilerInstance19createSourceManagerERNS_11FileManagerE", "clang::CompilerInstance::createSourceManager(clang::FileManager&)"},
    {"_ZN5clang16CompilerInstance18createPreprocessorEv", "clang::CompilerInstance::createPreprocessor()"},
    {"_ZN5clang16CompilerInstance18createPreprocessorERNS_10DiagnosticERKNS_11LangOptionsERKNS_19PreprocessorOptionsERKNS_19HeaderSearchOptionsERKNS_23DependencyOutputOptionsERKNS_10TargetInfoERKNS_15FrontendOptionsERNS_13SourceManagerERNS_11FileManagerE", "clang::CompilerInstance::createPreprocessor(clang::Diagnostic&, clang::LangOptions const&, clang::PreprocessorOptions const&, clang::HeaderSearchOptions const&, clang::DependencyOutputOptions const&, clang::TargetInfo const&, clang::FrontendOptions const&, clang::SourceManager&, clang::FileManager&)"},
    {"_ZN5clang16CompilerInstance16createASTContextEv", "clang::CompilerInstance::createASTContext()"},
    {"_ZN5clang16CompilerInstance26createPCHExternalASTSourceEN4llvm9StringRefEbbPv", "clang::CompilerInstance::createPCHExternalASTSource(llvm::StringRef, bool, bool, void*)"},
    {"_ZN5clang16CompilerInstance26createPCHExternalASTSourceEN4llvm9StringRefERKSsbbRNS_12PreprocessorERNS_10ASTContextEPvb", "clang::CompilerInstance::createPCHExternalASTSource(llvm::StringRef, std::string const&, bool, bool, clang::Preprocessor&, clang::ASTContext&, void*, bool)"},
    {"_ZN5clang16CompilerInstance28createCodeCompletionConsumerEv", "clang::CompilerInstance::createCodeCompletionConsumer()"},
    {"_ZN5clang16CompilerInstance28createCodeCompletionConsumerERNS_12PreprocessorERKSsjjbbbRN4llvm11raw_ostreamE", "clang::CompilerInstance::createCodeCompletionConsumer(clang::Preprocessor&, std::string const&, unsigned int, unsigned int, bool, bool, bool, llvm::raw_ostream&)"},
    {"_Z20EnableCodeCompletionRN5clang12PreprocessorERKSsjj", "EnableCodeCompletion(clang::Preprocessor&, std::string const&, unsigned int, unsigned int)"},
    {"_ZN5clang16CompilerInstance19createFrontendTimerEv", "clang::CompilerInstance::createFrontendTimer()"},
    {"_ZN5clang16CompilerInstance10createSemaEbPNS_20CodeCompleteConsumerE", "clang::CompilerInstance::createSema(bool, clang::CodeCompleteConsumer*)"},
    {"_ZN5clang16CompilerInstance13addOutputFileERKNS0_10OutputFileE", "clang::CompilerInstance::addOutputFile(clang::CompilerInstance::OutputFile const&)"},
    {"_ZN5clang16CompilerInstance16clearOutputFilesEb", "clang::CompilerInstance::clearOutputFiles(bool)"},
    {"_ZN5clang16CompilerInstance23createDefaultOutputFileEbN4llvm9StringRefES2_", "clang::CompilerInstance::createDefaultOutputFile(bool, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang16CompilerInstance16createOutputFileEN4llvm9StringRefEbbS2_S2_", "clang::CompilerInstance::createOutputFile(llvm::StringRef, bool, bool, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang16CompilerInstance16createOutputFileEN4llvm9StringRefERSsbbS2_S2_PSsS4_", "clang::CompilerInstance::createOutputFile(llvm::StringRef, std::string&, bool, bool, llvm::StringRef, llvm::StringRef, std::string*, std::string*)"},
    {"_ZN5clang16CompilerInstance23InitializeSourceManagerEN4llvm9StringRefE", "clang::CompilerInstance::InitializeSourceManager(llvm::StringRef)"},
    {"_ZN5clang16CompilerInstance23InitializeSourceManagerEN4llvm9StringRefERNS_10DiagnosticERNS_11FileManagerERNS_13SourceManagerERKNS_15FrontendOptionsE", "clang::CompilerInstance::InitializeSourceManager(llvm::StringRef, clang::Diagnostic&, clang::FileManager&, clang::SourceManager&, clang::FrontendOptions const&)"},
    {"_ZN5clang16CompilerInstance13ExecuteActionERNS_14FrontendActionE", "clang::CompilerInstance::ExecuteAction(clang::FrontendAction&)"},
    {"_ZNSt10_List_baseIN5clang16CompilerInstance10OutputFileESaIS2_EE8_M_clearEv", "std::_List_base<clang::CompilerInstance::OutputFile, std::allocator<clang::CompilerInstance::OutputFile> >::_M_clear()"},
    {"_ZN5clang28PrintingCodeCompleteConsumer12getAllocatorEv", "clang::PrintingCodeCompleteConsumer::getAllocator()"},
    {"_ZN5clang20CodeCompleteConsumer26ProcessCodeCompleteResultsERNS_4SemaENS_21CodeCompletionContextEPNS_20CodeCompletionResultEj", "clang::CodeCompleteConsumer::ProcessCodeCompleteResults(clang::Sema&, clang::CodeCompletionContext, clang::CodeCompletionResult*, unsigned int)"},
    {"_ZN5clang20CodeCompleteConsumer25ProcessOverloadCandidatesERNS_4SemaEjPNS0_17OverloadCandidateEj", "clang::CodeCompleteConsumer::ProcessOverloadCandidates(clang::Sema&, unsigned int, clang::CodeCompleteConsumer::OverloadCandidate*, unsigned int)"},
    {"_ZN5clang23ChainedDiagnosticClientD1Ev", "clang::ChainedDiagnosticClient::~ChainedDiagnosticClient()"},
    {"_ZN5clang23ChainedDiagnosticClientD0Ev", "clang::ChainedDiagnosticClient::~ChainedDiagnosticClient()"},
    {"_ZN5clang23ChainedDiagnosticClient15BeginSourceFileERKNS_11LangOptionsEPKNS_12PreprocessorE", "clang::ChainedDiagnosticClient::BeginSourceFile(clang::LangOptions const&, clang::Preprocessor const*)"},
    {"_ZN5clang23ChainedDiagnosticClient13EndSourceFileEv", "clang::ChainedDiagnosticClient::EndSourceFile()"},
    {"_ZNK5clang23ChainedDiagnosticClient25IncludeInDiagnosticCountsEv", "clang::ChainedDiagnosticClient::IncludeInDiagnosticCounts() const"},
    {"_ZN5clang23ChainedDiagnosticClient16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::ChainedDiagnosticClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN5clang18CompilerInvocation6toArgsERSt6vectorISsSaISsEE", "clang::CompilerInvocation::toArgs(std::vector<std::string, std::allocator<std::string> >&)"},
    {"_ZN5clang18CompilerInvocation16GetResourcesPathEPKcPv", "clang::CompilerInvocation::GetResourcesPath(char const*, void*)"},
    {"_ZN5clang18CompilerInvocation15setLangDefaultsERNS_11LangOptionsENS_9InputKindENS_12LangStandard4KindE", "clang::CompilerInvocation::setLangDefaults(clang::LangOptions&, clang::InputKind, clang::LangStandard::Kind)"},
    {"_ZN5clang18CompilerInvocation14CreateFromArgsERS0_PKPKcS5_RNS_10DiagnosticE", "clang::CompilerInvocation::CreateFromArgs(clang::CompilerInvocation&, char const* const*, char const* const*, clang::Diagnostic&)"},
    {"_ZN5clang19PreprocessorOptions11addMacroDefEN4llvm9StringRefE", "clang::PreprocessorOptions::addMacroDef(llvm::StringRef)"},
    {"_ZN5clang19PreprocessorOptions13addMacroUndefEN4llvm9StringRefE", "clang::PreprocessorOptions::addMacroUndef(llvm::StringRef)"},
    {"_ZN5clang19PreprocessorOptions15addRemappedFileEN4llvm9StringRefES2_", "clang::PreprocessorOptions::addRemappedFile(llvm::StringRef, llvm::StringRef)"},
    {"_ZNSt6vectorISt4pairISsSsESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<std::string, std::string>*, std::vector<std::pair<std::string, std::string>, std::allocator<std::pair<std::string, std::string> > > >, std::pair<std::string, std::string> const&)"},
    {"_ZNSt6vectorISsSaISsEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSsS1_EERKSs", "std::vector<std::string, std::allocator<std::string> >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, std::string const&)"},
    {"_ZNSt6vectorISt4pairISsbESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<std::string, bool>, std::allocator<std::pair<std::string, bool> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<std::string, bool>*, std::vector<std::pair<std::string, bool>, std::allocator<std::pair<std::string, bool> > > >, std::pair<std::string, bool> const&)"},
    {"_ZN5clang19HeaderSearchOptions7AddPathEN4llvm9StringRefENS_8frontend15IncludeDirGroupEbbb", "clang::HeaderSearchOptions::AddPath(llvm::StringRef, clang::frontend::IncludeDirGroup, bool, bool, bool)"},
    {"_ZNSt6vectorIN5clang19HeaderSearchOptions5EntryESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::HeaderSearchOptions::Entry, std::allocator<clang::HeaderSearchOptions::Entry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::HeaderSearchOptions::Entry*, std::vector<clang::HeaderSearchOptions::Entry, std::allocator<clang::HeaderSearchOptions::Entry> > >, clang::HeaderSearchOptions::Entry const&)"},
    {"_ZN5clang20ParsedSourceLocation10FromStringEN4llvm9StringRefE", "clang::ParsedSourceLocation::FromString(llvm::StringRef)"},
    {"_ZNSt6vectorISt4pairIN5clang9InputKindESsESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<clang::InputKind, std::string>, std::allocator<std::pair<clang::InputKind, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::InputKind, std::string>*, std::vector<std::pair<clang::InputKind, std::string>, std::allocator<std::pair<clang::InputKind, std::string> > > >, std::pair<clang::InputKind, std::string> const&)"},
    {"_ZNSt6vectorIS_ISsSaISsEESaIS1_EE15_M_erase_at_endEPS1_", "std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > >::_M_erase_at_end(std::vector<std::string, std::allocator<std::string> >*)"},
    {"_ZNSt6vectorIS_ISsSaISsEESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::vector<std::string, std::allocator<std::string> >*, std::vector<std::vector<std::string, std::allocator<std::string> >, std::allocator<std::vector<std::string, std::allocator<std::string> > > > >, unsigned long, std::vector<std::string, std::allocator<std::string> > const&)"},
    {"_ZNSt6vectorISsSaISsEEC2ERKS1_", "std::vector<std::string, std::allocator<std::string> >::vector(std::vector<std::string, std::allocator<std::string> > const&)"},
    {"_ZNSt6vectorIN5clang8AnalysesESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::Analyses, std::allocator<clang::Analyses> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Analyses*, std::vector<clang::Analyses, std::allocator<clang::Analyses> > >, clang::Analyses const&)"},
    {"_ZN5clang11DocumentXML14writeDeclToXMLEPNS_4DeclE", "clang::DocumentXML::writeDeclToXML(clang::Decl*)"},
    {"_ZN5clang11DeclVisitorINS_11DocumentXML11DeclPrinterEvE5VisitEPNS_4DeclE", "clang::DeclVisitor<clang::DocumentXML::DeclPrinter, void>::Visit(clang::Decl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter20VisitLinkageSpecDeclEPNS_15LinkageSpecDeclE", "clang::DocumentXML::DeclPrinter::VisitLinkageSpecDecl(clang::LinkageSpecDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter18VisitNamespaceDeclEPNS_13NamespaceDeclE", "clang::DocumentXML::DeclPrinter::VisitNamespaceDecl(clang::NamespaceDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter23VisitNamespaceAliasDeclEPNS_18NamespaceAliasDeclE", "clang::DocumentXML::DeclPrinter::VisitNamespaceAliasDecl(clang::NamespaceAliasDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter13VisitEnumDeclEPNS_8EnumDeclE", "clang::DocumentXML::DeclPrinter::VisitEnumDecl(clang::EnumDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter15VisitRecordDeclEPNS_10RecordDeclE", "clang::DocumentXML::DeclPrinter::VisitRecordDecl(clang::RecordDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter18VisitCXXRecordDeclEPNS_13CXXRecordDeclE", "clang::DocumentXML::DeclPrinter::VisitCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter25VisitTemplateTypeParmDeclEPNS_20TemplateTypeParmDeclE", "clang::DocumentXML::DeclPrinter::VisitTemplateTypeParmDecl(clang::TemplateTypeParmDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter16VisitTypedefDeclEPNS_11TypedefDeclE", "clang::DocumentXML::DeclPrinter::VisitTypedefDecl(clang::TypedefDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter14VisitUsingDeclEPNS_9UsingDeclE", "clang::DocumentXML::DeclPrinter::VisitUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter23VisitUsingDirectiveDeclEPNS_18UsingDirectiveDeclE", "clang::DocumentXML::DeclPrinter::VisitUsingDirectiveDecl(clang::UsingDirectiveDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter20VisitUsingShadowDeclEPNS_15UsingShadowDeclE", "clang::DocumentXML::DeclPrinter::VisitUsingShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter14VisitFieldDeclEPNS_9FieldDeclE", "clang::DocumentXML::DeclPrinter::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter17VisitFunctionDeclEPNS_12FunctionDeclE", "clang::DocumentXML::DeclPrinter::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter18VisitCXXMethodDeclEPNS_13CXXMethodDeclE", "clang::DocumentXML::DeclPrinter::VisitCXXMethodDecl(clang::CXXMethodDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter23VisitCXXConstructorDeclEPNS_18CXXConstructorDeclE", "clang::DocumentXML::DeclPrinter::VisitCXXConstructorDecl(clang::CXXConstructorDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter22VisitCXXConversionDeclEPNS_17CXXConversionDeclE", "clang::DocumentXML::DeclPrinter::VisitCXXConversionDecl(clang::CXXConversionDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter22VisitCXXDestructorDeclEPNS_17CXXDestructorDeclE", "clang::DocumentXML::DeclPrinter::VisitCXXDestructorDecl(clang::CXXDestructorDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter12VisitVarDeclEPNS_7VarDeclE", "clang::DocumentXML::DeclPrinter::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter16VisitParmVarDeclEPNS_11ParmVarDeclE", "clang::DocumentXML::DeclPrinter::VisitParmVarDecl(clang::ParmVarDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter21VisitEnumConstantDeclEPNS_16EnumConstantDeclE", "clang::DocumentXML::DeclPrinter::VisitEnumConstantDecl(clang::EnumConstantDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter9VisitDeclEPNS_4DeclE", "clang::DocumentXML::DeclPrinter::VisitDecl(clang::Decl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter15addFunctionBodyEPNS_12FunctionDeclE", "clang::DocumentXML::DeclPrinter::addFunctionBody(clang::FunctionDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter11addSubNodesEPNS_13CXXRecordDeclE", "clang::DocumentXML::DeclPrinter::addSubNodes(clang::CXXRecordDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter11addSubNodesEPNS_10RecordDeclE", "clang::DocumentXML::DeclPrinter::addSubNodes(clang::RecordDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter11addSubNodesEPNS_8EnumDeclE", "clang::DocumentXML::DeclPrinter::addSubNodes(clang::EnumDecl*)"},
    {"_ZN5clang11DocumentXML11DeclPrinter17VisitTemplateDeclEPNS_12TemplateDeclE", "clang::DocumentXML::DeclPrinter::VisitTemplateDecl(clang::TemplateDecl*)"},
    {"_ZN5clang23AttachDependencyFileGenERNS_12PreprocessorERKNS_23DependencyOutputOptionsE", "clang::AttachDependencyFileGen(clang::Preprocessor&, clang::DependencyOutputOptions const&)"},
    {"_ZN12_GLOBAL__N_122DependencyFileCallbackD1Ev", "(anonymous namespace)::DependencyFileCallback::~DependencyFileCallback()"},
    {"_ZN12_GLOBAL__N_122DependencyFileCallbackD0Ev", "(anonymous namespace)::DependencyFileCallback::~DependencyFileCallback()"},
    {"_ZN12_GLOBAL__N_122DependencyFileCallback11FileChangedEN5clang14SourceLocationENS1_11PPCallbacks16FileChangeReasonENS1_6SrcMgr18CharacteristicKindE", "(anonymous namespace)::DependencyFileCallback::FileChanged(clang::SourceLocation, clang::PPCallbacks::FileChangeReason, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN12_GLOBAL__N_122DependencyFileCallback13EndOfMainFileEv", "(anonymous namespace)::DependencyFileCallback::EndOfMainFile()"},
    {"_ZN5clang11DocumentXMLC1ERKSsRN4llvm11raw_ostreamE", "clang::DocumentXML::DocumentXML(std::string const&, llvm::raw_ostream&)"},
    {"_ZN5clang11DocumentXMLC2ERKSsRN4llvm11raw_ostreamE", "clang::DocumentXML::DocumentXML(std::string const&, llvm::raw_ostream&)"},
    {"_ZN5clang11DocumentXML10addSubNodeERKSs", "clang::DocumentXML::addSubNode(std::string const&)"},
    {"_ZN5clang11DocumentXML6IndentEv", "clang::DocumentXML::Indent()"},
    {"_ZN5clang11DocumentXML8toParentEv", "clang::DocumentXML::toParent()"},
    {"_ZN5clang11DocumentXML12escapeStringEPKcm", "clang::DocumentXML::escapeString(char const*, unsigned long)"},
    {"_ZN5clang11DocumentXML8finalizeEv", "clang::DocumentXML::finalize()"},
    {"_ZN5clang11DocumentXML12addAttributeEPKcRKNS_8QualTypeE", "clang::DocumentXML::addAttribute(char const*, clang::QualType const&)"},
    {"_ZN5clang11DocumentXML18addTypeRecursivelyERKNS_8QualTypeE", "clang::DocumentXML::addTypeRecursively(clang::QualType const&)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_4TypeE", "clang::DocumentXML::addPtrAttribute(char const*, clang::Type const*)"},
    {"_ZN5clang11DocumentXML18addTypeRecursivelyEPKNS_4TypeE", "clang::DocumentXML::addTypeRecursively(clang::Type const*)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_19NestedNameSpecifierE", "clang::DocumentXML::addPtrAttribute(char const*, clang::NestedNameSpecifier const*)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_13NamespaceDeclE", "clang::DocumentXML::addPtrAttribute(char const*, clang::NamespaceDecl const*)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_11DeclContextE", "clang::DocumentXML::addPtrAttribute(char const*, clang::DeclContext const*)"},
    {"_ZN5clang11DocumentXML22addContextsRecursivelyEPKNS_11DeclContextE", "clang::DocumentXML::addContextsRecursively(clang::DeclContext const*)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_9NamedDeclE", "clang::DocumentXML::addPtrAttribute(char const*, clang::NamedDecl const*)"},
    {"_ZN5clang11DocumentXML22addSourceFileAttributeERKSs", "clang::DocumentXML::addSourceFileAttribute(std::string const&)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcPKNS_9LabelStmtE", "clang::DocumentXML::addPtrAttribute(char const*, clang::LabelStmt const*)"},
    {"_ZN5clang11DocumentXML11addLocationERKNS_14SourceLocationE", "clang::DocumentXML::addLocation(clang::SourceLocation const&)"},
    {"_ZN5clang11DocumentXML16addLocationRangeERKNS_11SourceRangeE", "clang::DocumentXML::addLocationRange(clang::SourceRange const&)"},
    {"_ZN5clang11DocumentXML9PrintDeclEPNS_4DeclE", "clang::DocumentXML::PrintDecl(clang::Decl*)"},
    {"_ZN5clang11DocumentXML12addAttributeISsEEvPKcRKT_", "void clang::DocumentXML::addAttribute<std::string>(char const*, std::string const&)"},
    {"_ZN5clang11DocumentXML15addPtrAttributeEPKcS2_", "clang::DocumentXML::addPtrAttribute(char const*, char const*)"},
    {"_ZN5clang11DocumentXML12addAttributeIjEEvPKcRKT_", "void clang::DocumentXML::addAttribute<unsigned int>(char const*, unsigned int const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9LabelStmtEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::LabelStmt const*, unsigned int, llvm::DenseMapInfo<clang::LabelStmt const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::LabelStmt const* const&, unsigned int const&, std::pair<clang::LabelStmt const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9LabelStmtEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::LabelStmt const*, unsigned int, llvm::DenseMapInfo<clang::LabelStmt const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang9LabelStmtEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::LabelStmt const*, unsigned int, llvm::DenseMapInfo<clang::LabelStmt const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::LabelStmt const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::DeclContext const*, unsigned int, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::DeclContext const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::DeclContext const*, unsigned int, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::DeclContext const* const&, unsigned int const&, std::pair<clang::DeclContext const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::DeclContext const*, unsigned int, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang9NamedDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::NamedDecl const*, unsigned int, llvm::DenseMapInfo<clang::NamedDecl const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::NamedDecl const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::Type const*, unsigned int, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::Type const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::Type const*, unsigned int, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::Type const* const&, unsigned int const&, std::pair<clang::Type const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::Type const*, unsigned int, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_jESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE16_M_insert_uniqueERKS4_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, unsigned int>, std::_Select1st<std::pair<clang::QualType const, unsigned int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, unsigned int> > >::_M_insert_unique(std::pair<clang::QualType const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_jESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS4_ERKS4_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, unsigned int>, std::_Select1st<std::pair<clang::QualType const, unsigned int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::QualType const, unsigned int> >, std::pair<clang::QualType const, unsigned int> const&)"},
    {"_ZNSt5dequeISsSaISsEE8pop_backEv", "std::deque<std::string, std::allocator<std::string> >::pop_back()"},
    {"_ZNSt11_Deque_baseISsSaISsEE17_M_initialize_mapEm", "std::_Deque_base<std::string, std::allocator<std::string> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeISsSaISsEEC2ERKS1_", "std::deque<std::string, std::allocator<std::string> >::deque(std::deque<std::string, std::allocator<std::string> > const&)"},
    {"_ZN5clang14FrontendActionC1Ev", "clang::FrontendAction::FrontendAction()"},
    {"_ZN5clang14FrontendActionC2Ev", "clang::FrontendAction::FrontendAction()"},
    {"_ZN5clang14FrontendActionD0Ev", "clang::FrontendAction::~FrontendAction()"},
    {"_ZN5clang14FrontendActionD1Ev", "clang::FrontendAction::~FrontendAction()"},
    {"_ZN5clang14FrontendActionD2Ev", "clang::FrontendAction::~FrontendAction()"},
    {"_ZN5clang14FrontendAction14setCurrentFileEN4llvm9StringRefENS_9InputKindEPNS_7ASTUnitE", "clang::FrontendAction::setCurrentFile(llvm::StringRef, clang::InputKind, clang::ASTUnit*)"},
    {"_ZN5clang14FrontendAction24CreateWrappedASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::FrontendAction::CreateWrappedASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang14FrontendAction15BeginSourceFileERNS_16CompilerInstanceEN4llvm9StringRefENS_9InputKindE", "clang::FrontendAction::BeginSourceFile(clang::CompilerInstance&, llvm::StringRef, clang::InputKind)"},
    {"_ZN5clang14FrontendAction7ExecuteEv", "clang::FrontendAction::Execute()"},
    {"_ZN5clang14FrontendAction13EndSourceFileEv", "clang::FrontendAction::EndSourceFile()"},
    {"_ZN5clang17ASTFrontendAction13ExecuteActionEv", "clang::ASTFrontendAction::ExecuteAction()"},
    {"_ZN5clang26PreprocessorFrontendAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::PreprocessorFrontendAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_124DeserializedDeclsCheckerD1Ev", "(anonymous namespace)::DeserializedDeclsChecker::~DeserializedDeclsChecker()"},
    {"_ZN12_GLOBAL__N_124DeserializedDeclsCheckerD0Ev", "(anonymous namespace)::DeserializedDeclsChecker::~DeserializedDeclsChecker()"},
    {"_ZN12_GLOBAL__N_124DeserializedDeclsChecker8DeclReadEjPKN5clang4DeclE", "(anonymous namespace)::DeserializedDeclsChecker::DeclRead(unsigned int, clang::Decl const*)"},
    {"_ZN12_GLOBAL__N_123DeserializedDeclsDumperD1Ev", "(anonymous namespace)::DeserializedDeclsDumper::~DeserializedDeclsDumper()"},
    {"_ZN12_GLOBAL__N_123DeserializedDeclsDumperD0Ev", "(anonymous namespace)::DeserializedDeclsDumper::~DeserializedDeclsDumper()"},
    {"_ZN12_GLOBAL__N_123DeserializedDeclsDumper8DeclReadEjPKN5clang4DeclE", "(anonymous namespace)::DeserializedDeclsDumper::DeclRead(unsigned int, clang::Decl const*)"},
    {"_ZNSt6vectorIPN5clang11ASTConsumerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::ASTConsumer*, std::allocator<clang::ASTConsumer*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ASTConsumer**, std::vector<clang::ASTConsumer*, std::allocator<clang::ASTConsumer*> > >, clang::ASTConsumer* const&)"},
    {"_ZN5clang26ASTDeserializationListener17ReaderInitializedEPNS_9ASTReaderE", "clang::ASTDeserializationListener::ReaderInitialized(clang::ASTReader*)"},
    {"_ZN5clang26ASTDeserializationListener14IdentifierReadEjPNS_14IdentifierInfoE", "clang::ASTDeserializationListener::IdentifierRead(unsigned int, clang::IdentifierInfo*)"},
    {"_ZN5clang26ASTDeserializationListener8TypeReadENS_13serialization7TypeIdxENS_8QualTypeE", "clang::ASTDeserializationListener::TypeRead(clang::serialization::TypeIdx, clang::QualType)"},
    {"_ZN5clang26ASTDeserializationListener12SelectorReadEjNS_8SelectorE", "clang::ASTDeserializationListener::SelectorRead(unsigned int, clang::Selector)"},
    {"_ZN5clang26ASTDeserializationListener19MacroDefinitionReadEjPNS_15MacroDefinitionE", "clang::ASTDeserializationListener::MacroDefinitionRead(unsigned int, clang::MacroDefinition*)"},
    {"_ZNSt8_Rb_treeISsSsSt9_IdentityISsESt4lessISsESaISsEE4findERKSs", "std::_Rb_tree<std::string, std::string, std::_Identity<std::string>, std::less<std::string>, std::allocator<std::string> >::find(std::string const&)"},
    {"_ZNSt8_Rb_treeISsSsSt9_IdentityISsESt4lessISsESaISsEE7_M_copyEPKSt13_Rb_tree_nodeISsEPS7_", "std::_Rb_tree<std::string, std::string, std::_Identity<std::string>, std::less<std::string>, std::allocator<std::string> >::_M_copy(std::_Rb_tree_node<std::string> const*, std::_Rb_tree_node<std::string>*)"},
    {"_ZN5clang26ASTDeserializationListener8DeclReadEjPKNS_4DeclE", "clang::ASTDeserializationListener::DeclRead(unsigned int, clang::Decl const*)"},
    {"_ZN5clang14InitOnlyAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::InitOnlyAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang14InitOnlyAction13ExecuteActionEv", "clang::InitOnlyAction::ExecuteAction()"},
    {"_ZN5clang14ASTPrintAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTPrintAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang17ASTPrintXMLAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTPrintXMLAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang13ASTDumpAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTDumpAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang16ASTDumpXMLAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTDumpXMLAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang13ASTViewAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ASTViewAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang22DeclContextPrintAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::DeclContextPrintAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang17GeneratePCHAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::GeneratePCHAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang17GeneratePCHAction27ComputeASTConsumerArgumentsERNS_16CompilerInstanceEN4llvm9StringRefERSsRPNS3_11raw_ostreamERb", "clang::GeneratePCHAction::ComputeASTConsumerArguments(clang::CompilerInstance&, llvm::StringRef, std::string&, llvm::raw_ostream*&, bool&)"},
    {"_ZN5clang21InheritanceViewAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::InheritanceViewAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang16SyntaxOnlyAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::SyntaxOnlyAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang19DumpRawTokensAction13ExecuteActionEv", "clang::DumpRawTokensAction::ExecuteAction()"},
    {"_ZN5clang16DumpTokensAction13ExecuteActionEv", "clang::DumpTokensAction::ExecuteAction()"},
    {"_ZN5clang17GeneratePTHAction13ExecuteActionEv", "clang::GeneratePTHAction::ExecuteAction()"},
    {"_ZN5clang20PreprocessOnlyAction13ExecuteActionEv", "clang::PreprocessOnlyAction::ExecuteAction()"},
    {"_ZN5clang23PrintPreprocessedAction13ExecuteActionEv", "clang::PrintPreprocessedAction::ExecuteAction()"},
    {"_ZN5clang19PrintPreambleAction13ExecuteActionEv", "clang::PrintPreambleAction::ExecuteAction()"},
    {"_ZN5clang5LexerD1Ev", "clang::Lexer::~Lexer()"},
    {"_ZN5clang12Preprocessor3LexERNS_5TokenE", "clang::Preprocessor::Lex(clang::Token&)"},
    {"_ZN5clang14InitOnlyActionD1Ev", "clang::InitOnlyAction::~InitOnlyAction()"},
    {"_ZN5clang14InitOnlyActionD0Ev", "clang::InitOnlyAction::~InitOnlyAction()"},
    {"_ZN5clang14ASTPrintActionD1Ev", "clang::ASTPrintAction::~ASTPrintAction()"},
    {"_ZN5clang14ASTPrintActionD0Ev", "clang::ASTPrintAction::~ASTPrintAction()"},
    {"_ZN5clang17ASTPrintXMLActionD1Ev", "clang::ASTPrintXMLAction::~ASTPrintXMLAction()"},
    {"_ZN5clang17ASTPrintXMLActionD0Ev", "clang::ASTPrintXMLAction::~ASTPrintXMLAction()"},
    {"_ZN5clang13ASTDumpActionD1Ev", "clang::ASTDumpAction::~ASTDumpAction()"},
    {"_ZN5clang13ASTDumpActionD0Ev", "clang::ASTDumpAction::~ASTDumpAction()"},
    {"_ZN5clang16ASTDumpXMLActionD1Ev", "clang::ASTDumpXMLAction::~ASTDumpXMLAction()"},
    {"_ZN5clang16ASTDumpXMLActionD0Ev", "clang::ASTDumpXMLAction::~ASTDumpXMLAction()"},
    {"_ZN5clang13ASTViewActionD1Ev", "clang::ASTViewAction::~ASTViewAction()"},
    {"_ZN5clang13ASTViewActionD0Ev", "clang::ASTViewAction::~ASTViewAction()"},
    {"_ZN5clang22DeclContextPrintActionD1Ev", "clang::DeclContextPrintAction::~DeclContextPrintAction()"},
    {"_ZN5clang22DeclContextPrintActionD0Ev", "clang::DeclContextPrintAction::~DeclContextPrintAction()"},
    {"_ZN5clang17GeneratePCHActionD1Ev", "clang::GeneratePCHAction::~GeneratePCHAction()"},
    {"_ZN5clang17GeneratePCHActionD0Ev", "clang::GeneratePCHAction::~GeneratePCHAction()"},
    {"_ZN5clang21InheritanceViewActionD1Ev", "clang::InheritanceViewAction::~InheritanceViewAction()"},
    {"_ZN5clang21InheritanceViewActionD0Ev", "clang::InheritanceViewAction::~InheritanceViewAction()"},
    {"_ZN5clang19PrintPreambleActionD1Ev", "clang::PrintPreambleAction::~PrintPreambleAction()"},
    {"_ZN5clang19PrintPreambleActionD0Ev", "clang::PrintPreambleAction::~PrintPreambleAction()"},
    {"_ZN5clang19DumpRawTokensActionD1Ev", "clang::DumpRawTokensAction::~DumpRawTokensAction()"},
    {"_ZN5clang19DumpRawTokensActionD0Ev", "clang::DumpRawTokensAction::~DumpRawTokensAction()"},
    {"_ZN5clang16DumpTokensActionD1Ev", "clang::DumpTokensAction::~DumpTokensAction()"},
    {"_ZN5clang16DumpTokensActionD0Ev", "clang::DumpTokensAction::~DumpTokensAction()"},
    {"_ZN5clang17GeneratePTHActionD1Ev", "clang::GeneratePTHAction::~GeneratePTHAction()"},
    {"_ZN5clang17GeneratePTHActionD0Ev", "clang::GeneratePTHAction::~GeneratePTHAction()"},
    {"_ZN5clang20PreprocessOnlyActionD1Ev", "clang::PreprocessOnlyAction::~PreprocessOnlyAction()"},
    {"_ZN5clang20PreprocessOnlyActionD0Ev", "clang::PreprocessOnlyAction::~PreprocessOnlyAction()"},
    {"_ZN5clang23PrintPreprocessedActionD1Ev", "clang::PrintPreprocessedAction::~PrintPreprocessedAction()"},
    {"_ZN5clang23PrintPreprocessedActionD0Ev", "clang::PrintPreprocessedAction::~PrintPreprocessedAction()"},
    {"_ZN5clang5LexerD0Ev", "clang::Lexer::~Lexer()"},
    {"_ZN5clang5Lexer11IndirectLexERNS_5TokenE", "clang::Lexer::IndirectLex(clang::Token&)"},
    {"_ZN5clang5Lexer17getSourceLocationEv", "clang::Lexer::getSourceLocation()"},
    {"_ZN5clang17PreprocessorLexerD1Ev", "clang::PreprocessorLexer::~PreprocessorLexer()"},
    {"_ZN5clang17PreprocessorLexerD0Ev", "clang::PreprocessorLexer::~PreprocessorLexer()"},
    {"_ZN5clang11ASTConsumerD1Ev", "clang::ASTConsumer::~ASTConsumer()"},
    {"_ZN5clang11ASTConsumerD0Ev", "clang::ASTConsumer::~ASTConsumer()"},
    {"_ZN5clang11ASTConsumer10InitializeERNS_10ASTContextE", "clang::ASTConsumer::Initialize(clang::ASTContext&)"},
    {"_ZN5clang11ASTConsumer21HandleTranslationUnitERNS_10ASTContextE", "clang::ASTConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang11ASTConsumer23HandleTagDeclDefinitionEPNS_7TagDeclE", "clang::ASTConsumer::HandleTagDeclDefinition(clang::TagDecl*)"},
    {"_ZN5clang11ASTConsumer27CompleteTentativeDefinitionEPNS_7VarDeclE", "clang::ASTConsumer::CompleteTentativeDefinition(clang::VarDecl*)"},
    {"_ZN5clang11ASTConsumer12HandleVTableEPNS_13CXXRecordDeclEb", "clang::ASTConsumer::HandleVTable(clang::CXXRecordDecl*, bool)"},
    {"_ZN5clang11ASTConsumer22GetASTMutationListenerEv", "clang::ASTConsumer::GetASTMutationListener()"},
    {"_ZN5clang11ASTConsumer29GetASTDeserializationListenerEv", "clang::ASTConsumer::GetASTDeserializationListener()"},
    {"_ZN5clang11ASTConsumer10PrintStatsEv", "clang::ASTConsumer::PrintStats()"},
    {"_ZN5clang15FrontendOptions24getInputKindForExtensionEN4llvm9StringRefE", "clang::FrontendOptions::getInputKindForExtension(llvm::StringRef)"},
    {"_ZN4llvm12StringSwitchIN5clang9InputKindES2_E5CasesILj4ELj4ELj4ELj4ELj4EEERS3_RAT__KcRAT0__S6_RAT1__S6_RAT2__S6_RAT3__S6_RKS2_", "llvm::StringSwitch<clang::InputKind, clang::InputKind>& llvm::StringSwitch<clang::InputKind, clang::InputKind>::Cases<4u, 4u, 4u, 4u, 4u>(char const (&) [4u], char const (&) [4u], char const (&) [4u], char const (&) [4u], char const (&) [4u], clang::InputKind const&)"},
    {"_ZN5clang22AttachHeaderIncludeGenERNS_12PreprocessorEbN4llvm9StringRefE", "clang::AttachHeaderIncludeGen(clang::Preprocessor&, bool, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_122HeaderIncludesCallbackD1Ev", "(anonymous namespace)::HeaderIncludesCallback::~HeaderIncludesCallback()"},
    {"_ZN12_GLOBAL__N_122HeaderIncludesCallbackD0Ev", "(anonymous namespace)::HeaderIncludesCallback::~HeaderIncludesCallback()"},
    {"_ZN12_GLOBAL__N_122HeaderIncludesCallback11FileChangedEN5clang14SourceLocationENS1_11PPCallbacks16FileChangeReasonENS1_6SrcMgr18CharacteristicKindE", "(anonymous namespace)::HeaderIncludesCallback::FileChanged(clang::SourceLocation, clang::PPCallbacks::FileChangeReason, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang24ApplyHeaderSearchOptionsERNS_12HeaderSearchERKNS_19HeaderSearchOptionsERKNS_11LangOptionsERKN4llvm6TripleE", "clang::ApplyHeaderSearchOptions(clang::HeaderSearch&, clang::HeaderSearchOptions const&, clang::LangOptions const&, llvm::Triple const&)"},
    {"_ZN12_GLOBAL__N_116InitHeaderSearch7AddPathERKN4llvm5TwineEN5clang8frontend15IncludeDirGroupEbbbb", "(anonymous namespace)::InitHeaderSearch::AddPath(llvm::Twine const&, clang::frontend::IncludeDirGroup, bool, bool, bool, bool)"},
    {"_ZN12_GLOBAL__N_116InitHeaderSearch17AddDelimitedPathsEN4llvm9StringRefE", "(anonymous namespace)::InitHeaderSearch::AddDelimitedPaths(llvm::StringRef)"},
    {"_Z16RemoveDuplicatesRSt6vectorIN5clang15DirectoryLookupESaIS1_EEb", "RemoveDuplicates(std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> >&, bool)"},
    {"_ZN12_GLOBAL__N_116InitHeaderSearch27AddGnuCPlusPlusIncludePathsEN4llvm9StringRefES2_S2_S2_RKNS1_6TripleE", "(anonymous namespace)::InitHeaderSearch::AddGnuCPlusPlusIncludePaths(llvm::StringRef, llvm::StringRef, llvm::StringRef, llvm::StringRef, llvm::Triple const&)"},
    {"_ZN12_GLOBAL__N_116InitHeaderSearch29AddMinGWCPlusPlusIncludePathsEN4llvm9StringRefES2_S2_", "(anonymous namespace)::InitHeaderSearch::AddMinGWCPlusPlusIncludePaths(llvm::StringRef, llvm::StringRef, llvm::StringRef)"},
    {"_ZNSt6vectorIN5clang15DirectoryLookupESaIS1_EEaSERKS3_", "std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> >::operator=(std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > const&)"},
    {"_ZNSt6vectorIN5clang15DirectoryLookupESaIS1_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS1_S3_EEEEvS8_T_S9_St20forward_iterator_tag", "void std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> >::_M_range_insert<__gnu_cxx::__normal_iterator<clang::DirectoryLookup*, std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > > >(__gnu_cxx::__normal_iterator<clang::DirectoryLookup*, std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > >, __gnu_cxx::__normal_iterator<clang::DirectoryLookup*, std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > >, __gnu_cxx::__normal_iterator<clang::DirectoryLookup*, std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > >, std::forward_iterator_tag)"},
    {"_ZNSt6vectorIN5clang15DirectoryLookupESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::DirectoryLookup*, std::vector<clang::DirectoryLookup, std::allocator<clang::DirectoryLookup> > >, clang::DirectoryLookup const&)"},
    {"_ZN5clang24NormalizeDashIncludePathEN4llvm9StringRefE", "clang::NormalizeDashIncludePath(llvm::StringRef)"},
    {"_ZN5clang22InitializePreprocessorERNS_12PreprocessorERKNS_19PreprocessorOptionsERKNS_19HeaderSearchOptionsERKNS_15FrontendOptionsE", "clang::InitializePreprocessor(clang::Preprocessor&, clang::PreprocessorOptions const&, clang::HeaderSearchOptions const&, clang::FrontendOptions const&)"},
    {"_Z18AddImplicitIncludeRN5clang12MacroBuilderEN4llvm9StringRefE", "AddImplicitInclude(clang::MacroBuilder&, llvm::StringRef)"},
    {"_Z17DefineFloatMacrosRN5clang12MacroBuilderEN4llvm9StringRefEPKNS2_12fltSemanticsE", "DefineFloatMacros(clang::MacroBuilder&, llvm::StringRef, llvm::fltSemantics const*)"},
    {"_Z23DefineExactWidthIntTypeN5clang10TargetInfo7IntTypeERKS0_RNS_12MacroBuilderE", "DefineExactWidthIntType(clang::TargetInfo::IntType, clang::TargetInfo const&, clang::MacroBuilder&)"},
    {"_ZN5clang12LangStandard22getLangStandardForKindENS0_4KindE", "clang::LangStandard::getLangStandardForKind(clang::LangStandard::Kind)"},
    {"_ZN5clang12LangStandard22getLangStandardForNameEN4llvm9StringRefE", "clang::LangStandard::getLangStandardForName(llvm::StringRef)"},
    {"_ZN5clang35MultiplexASTDeserializationListenerC1ERKSt6vectorIPNS_26ASTDeserializationListenerESaIS3_EE", "clang::MultiplexASTDeserializationListener::MultiplexASTDeserializationListener(std::vector<clang::ASTDeserializationListener*, std::allocator<clang::ASTDeserializationListener*> > const&)"},
    {"_ZN5clang35MultiplexASTDeserializationListenerC2ERKSt6vectorIPNS_26ASTDeserializationListenerESaIS3_EE", "clang::MultiplexASTDeserializationListener::MultiplexASTDeserializationListener(std::vector<clang::ASTDeserializationListener*, std::allocator<clang::ASTDeserializationListener*> > const&)"},
    {"_ZN5clang35MultiplexASTDeserializationListener17ReaderInitializedEPNS_9ASTReaderE", "clang::MultiplexASTDeserializationListener::ReaderInitialized(clang::ASTReader*)"},
    {"_ZN5clang35MultiplexASTDeserializationListener14IdentifierReadEjPNS_14IdentifierInfoE", "clang::MultiplexASTDeserializationListener::IdentifierRead(unsigned int, clang::IdentifierInfo*)"},
    {"_ZN5clang35MultiplexASTDeserializationListener8TypeReadENS_13serialization7TypeIdxENS_8QualTypeE", "clang::MultiplexASTDeserializationListener::TypeRead(clang::serialization::TypeIdx, clang::QualType)"},
    {"_ZN5clang35MultiplexASTDeserializationListener8DeclReadEjPKNS_4DeclE", "clang::MultiplexASTDeserializationListener::DeclRead(unsigned int, clang::Decl const*)"},
    {"_ZN5clang35MultiplexASTDeserializationListener12SelectorReadEjNS_8SelectorE", "clang::MultiplexASTDeserializationListener::SelectorRead(unsigned int, clang::Selector)"},
    {"_ZN5clang35MultiplexASTDeserializationListener19MacroDefinitionReadEjPNS_15MacroDefinitionE", "clang::MultiplexASTDeserializationListener::MacroDefinitionRead(unsigned int, clang::MacroDefinition*)"},
    {"_ZN5clang28MultiplexASTMutationListenerC1ERKSt6vectorIPNS_19ASTMutationListenerESaIS3_EE", "clang::MultiplexASTMutationListener::MultiplexASTMutationListener(std::vector<clang::ASTMutationListener*, std::allocator<clang::ASTMutationListener*> > const&)"},
    {"_ZN5clang28MultiplexASTMutationListenerC2ERKSt6vectorIPNS_19ASTMutationListenerESaIS3_EE", "clang::MultiplexASTMutationListener::MultiplexASTMutationListener(std::vector<clang::ASTMutationListener*, std::allocator<clang::ASTMutationListener*> > const&)"},
    {"_ZN5clang28MultiplexASTMutationListener22CompletedTagDefinitionEPKNS_7TagDeclE", "clang::MultiplexASTMutationListener::CompletedTagDefinition(clang::TagDecl const*)"},
    {"_ZN5clang28MultiplexASTMutationListener16AddedVisibleDeclEPKNS_11DeclContextEPKNS_4DeclE", "clang::MultiplexASTMutationListener::AddedVisibleDecl(clang::DeclContext const*, clang::Decl const*)"},
    {"_ZN5clang28MultiplexASTMutationListener22AddedCXXImplicitMemberEPKNS_13CXXRecordDeclEPKNS_4DeclE", "clang::MultiplexASTMutationListener::AddedCXXImplicitMember(clang::CXXRecordDecl const*, clang::Decl const*)"},
    {"_ZN5clang28MultiplexASTMutationListener30AddedCXXTemplateSpecializationEPKNS_17ClassTemplateDeclEPKNS_31ClassTemplateSpecializationDeclE", "clang::MultiplexASTMutationListener::AddedCXXTemplateSpecialization(clang::ClassTemplateDecl const*, clang::ClassTemplateSpecializationDecl const*)"},
    {"_ZN5clang17MultiplexConsumerC1ERKSt6vectorIPNS_11ASTConsumerESaIS3_EE", "clang::MultiplexConsumer::MultiplexConsumer(std::vector<clang::ASTConsumer*, std::allocator<clang::ASTConsumer*> > const&)"},
    {"_ZN5clang17MultiplexConsumerC2ERKSt6vectorIPNS_11ASTConsumerESaIS3_EE", "clang::MultiplexConsumer::MultiplexConsumer(std::vector<clang::ASTConsumer*, std::allocator<clang::ASTConsumer*> > const&)"},
    {"_ZN5clang17MultiplexConsumerD0Ev", "clang::MultiplexConsumer::~MultiplexConsumer()"},
    {"_ZN5clang17MultiplexConsumerD1Ev", "clang::MultiplexConsumer::~MultiplexConsumer()"},
    {"_ZN5clang17MultiplexConsumerD2Ev", "clang::MultiplexConsumer::~MultiplexConsumer()"},
    {"_ZN5clang17MultiplexConsumer10InitializeERNS_10ASTContextE", "clang::MultiplexConsumer::Initialize(clang::ASTContext&)"},
    {"_ZN5clang17MultiplexConsumer18HandleTopLevelDeclENS_12DeclGroupRefE", "clang::MultiplexConsumer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN5clang17MultiplexConsumer21HandleInterestingDeclENS_12DeclGroupRefE", "clang::MultiplexConsumer::HandleInterestingDecl(clang::DeclGroupRef)"},
    {"_ZN5clang17MultiplexConsumer21HandleTranslationUnitERNS_10ASTContextE", "clang::MultiplexConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang17MultiplexConsumer23HandleTagDeclDefinitionEPNS_7TagDeclE", "clang::MultiplexConsumer::HandleTagDeclDefinition(clang::TagDecl*)"},
    {"_ZN5clang17MultiplexConsumer27CompleteTentativeDefinitionEPNS_7VarDeclE", "clang::MultiplexConsumer::CompleteTentativeDefinition(clang::VarDecl*)"},
    {"_ZN5clang17MultiplexConsumer12HandleVTableEPNS_13CXXRecordDeclEb", "clang::MultiplexConsumer::HandleVTable(clang::CXXRecordDecl*, bool)"},
    {"_ZN5clang17MultiplexConsumer22GetASTMutationListenerEv", "clang::MultiplexConsumer::GetASTMutationListener()"},
    {"_ZN5clang17MultiplexConsumer29GetASTDeserializationListenerEv", "clang::MultiplexConsumer::GetASTDeserializationListener()"},
    {"_ZN5clang17MultiplexConsumer10PrintStatsEv", "clang::MultiplexConsumer::PrintStats()"},
    {"_ZN5clang17MultiplexConsumer14InitializeSemaERNS_4SemaE", "clang::MultiplexConsumer::InitializeSema(clang::Sema&)"},
    {"_ZN5clang17MultiplexConsumer10ForgetSemaEv", "clang::MultiplexConsumer::ForgetSema()"},
    {"_ZN5clang35MultiplexASTDeserializationListenerD1Ev", "clang::MultiplexASTDeserializationListener::~MultiplexASTDeserializationListener()"},
    {"_ZN5clang35MultiplexASTDeserializationListenerD0Ev", "clang::MultiplexASTDeserializationListener::~MultiplexASTDeserializationListener()"},
    {"_ZN5clang28MultiplexASTMutationListenerD1Ev", "clang::MultiplexASTMutationListener::~MultiplexASTMutationListener()"},
    {"_ZN5clang28MultiplexASTMutationListenerD0Ev", "clang::MultiplexASTMutationListener::~MultiplexASTMutationListener()"},
    {"_ZNSt6vectorIPN5clang26ASTDeserializationListenerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::ASTDeserializationListener*, std::allocator<clang::ASTDeserializationListener*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ASTDeserializationListener**, std::vector<clang::ASTDeserializationListener*, std::allocator<clang::ASTDeserializationListener*> > >, clang::ASTDeserializationListener* const&)"},
    {"_ZNSt6vectorIPN5clang19ASTMutationListenerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::ASTMutationListener*, std::allocator<clang::ASTMutationListener*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ASTMutationListener**, std::vector<clang::ASTMutationListener*, std::allocator<clang::ASTMutationListener*> > >, clang::ASTMutationListener* const&)"},
    {"_ZN5clang12SemaConsumerD1Ev", "clang::SemaConsumer::~SemaConsumer()"},
    {"_ZN5clang12SemaConsumerD0Ev", "clang::SemaConsumer::~SemaConsumer()"},
    {"_ZN5clang12SemaConsumer14InitializeSemaERNS_4SemaE", "clang::SemaConsumer::InitializeSema(clang::Sema&)"},
    {"_ZN5clang12SemaConsumer10ForgetSemaEv", "clang::SemaConsumer::ForgetSema()"},
    {"_ZN5clang24DoPrintPreprocessedInputERNS_12PreprocessorEPN4llvm11raw_ostreamERKNS_25PreprocessorOutputOptionsE", "clang::DoPrintPreprocessedInput(clang::Preprocessor&, llvm::raw_ostream*, clang::PreprocessorOutputOptions const&)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks10MoveToLineEj", "(anonymous namespace)::PrintPPOutputPPCallbacks::MoveToLine(unsigned int)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks13WriteLineInfoEjPKcj", "(anonymous namespace)::PrintPPOutputPPCallbacks::WriteLineInfo(unsigned int, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_120UnknownPragmaHandlerD1Ev", "(anonymous namespace)::UnknownPragmaHandler::~UnknownPragmaHandler()"},
    {"_ZN12_GLOBAL__N_120UnknownPragmaHandlerD0Ev", "(anonymous namespace)::UnknownPragmaHandler::~UnknownPragmaHandler()"},
    {"_ZN12_GLOBAL__N_120UnknownPragmaHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::UnknownPragmaHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacksD1Ev", "(anonymous namespace)::PrintPPOutputPPCallbacks::~PrintPPOutputPPCallbacks()"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacksD0Ev", "(anonymous namespace)::PrintPPOutputPPCallbacks::~PrintPPOutputPPCallbacks()"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks11FileChangedEN5clang14SourceLocationENS1_11PPCallbacks16FileChangeReasonENS1_6SrcMgr18CharacteristicKindE", "(anonymous namespace)::PrintPPOutputPPCallbacks::FileChanged(clang::SourceLocation, clang::PPCallbacks::FileChangeReason, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks5IdentEN5clang14SourceLocationERKSs", "(anonymous namespace)::PrintPPOutputPPCallbacks::Ident(clang::SourceLocation, std::string const&)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks13PragmaCommentEN5clang14SourceLocationEPKNS1_14IdentifierInfoERKSs", "(anonymous namespace)::PrintPPOutputPPCallbacks::PragmaComment(clang::SourceLocation, clang::IdentifierInfo const*, std::string const&)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks13PragmaMessageEN5clang14SourceLocationEN4llvm9StringRefE", "(anonymous namespace)::PrintPPOutputPPCallbacks::PragmaMessage(clang::SourceLocation, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks12MacroDefinedERKN5clang5TokenEPKNS1_9MacroInfoE", "(anonymous namespace)::PrintPPOutputPPCallbacks::MacroDefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN12_GLOBAL__N_124PrintPPOutputPPCallbacks14MacroUndefinedERKN5clang5TokenEPKNS1_9MacroInfoE", "(anonymous namespace)::PrintPPOutputPPCallbacks::MacroUndefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_Z20PrintMacroDefinitionRKN5clang14IdentifierInfoERKNS_9MacroInfoERNS_12PreprocessorERN4llvm11raw_ostreamE", "PrintMacroDefinition(clang::IdentifierInfo const&, clang::MacroInfo const&, clang::Preprocessor&, llvm::raw_ostream&)"},
    {"_Z14MacroIDComparePKvS0_", "MacroIDCompare(void const*, void const*)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPN5clang14IdentifierInfoEPNS2_9MacroInfoEEE6appendINS_16DenseMapIteratorIS4_S6_NS_12DenseMapInfoIS4_EENSB_IS6_EELb1EEEEEvT_SF_", "void llvm::SmallVectorImpl<std::pair<clang::IdentifierInfo*, clang::MacroInfo*> >::append<llvm::DenseMapIterator<clang::IdentifierInfo*, clang::MacroInfo*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::MacroInfo*>, true> >(llvm::DenseMapIterator<clang::IdentifierInfo*, clang::MacroInfo*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::MacroInfo*>, true>, llvm::DenseMapIterator<clang::IdentifierInfo*, clang::MacroInfo*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::MacroInfo*>, true>)"},
    {"_ZN5clang11DocumentXML9PrintStmtEPKNS_4StmtE", "clang::DocumentXML::PrintStmt(clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_17StmtXML11DumpSubTreeEPN5clang4StmtE", "(anonymous namespace)::StmtXML::DumpSubTree(clang::Stmt*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_17StmtXMLEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::StmtXML, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_17StmtXML19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::StmtXML::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_17StmtXML21VisitCXXConstructExprEPN5clang16CXXConstructExprE", "(anonymous namespace)::StmtXML::VisitCXXConstructExpr(clang::CXXConstructExpr*)"},
    {"_ZN12_GLOBAL__N_17StmtXML13VisitCallExprEPN5clang8CallExprE", "(anonymous namespace)::StmtXML::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN12_GLOBAL__N_17StmtXML18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::StmtXML::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN12_GLOBAL__N_17StmtXML9VisitStmtEPN5clang4StmtE", "(anonymous namespace)::StmtXML::VisitStmt(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_17StmtXML21VisitCXXNamedCastExprEPN5clang16CXXNamedCastExprE", "(anonymous namespace)::StmtXML::VisitCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN5clang20TextDiagnosticBuffer16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::TextDiagnosticBuffer::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZNK5clang20TextDiagnosticBuffer16FlushDiagnosticsERNS_10DiagnosticE", "clang::TextDiagnosticBuffer::FlushDiagnostics(clang::Diagnostic&) const"},
    {"_ZN5clang20TextDiagnosticBufferD1Ev", "clang::TextDiagnosticBuffer::~TextDiagnosticBuffer()"},
    {"_ZN5clang20TextDiagnosticBufferD0Ev", "clang::TextDiagnosticBuffer::~TextDiagnosticBuffer()"},
    {"_ZNSt6vectorISt4pairIN5clang14SourceLocationESsESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string>*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, std::pair<clang::SourceLocation, std::string> const&)"},
    {"_ZN5clang20TextDiagnosticBufferD2Ev", "clang::TextDiagnosticBuffer::~TextDiagnosticBuffer()"},
    {"_ZN5clang21TextDiagnosticPrinterC1ERN4llvm11raw_ostreamERKNS_17DiagnosticOptionsEb", "clang::TextDiagnosticPrinter::TextDiagnosticPrinter(llvm::raw_ostream&, clang::DiagnosticOptions const&, bool)"},
    {"_ZN5clang21TextDiagnosticPrinterC2ERN4llvm11raw_ostreamERKNS_17DiagnosticOptionsEb", "clang::TextDiagnosticPrinter::TextDiagnosticPrinter(llvm::raw_ostream&, clang::DiagnosticOptions const&, bool)"},
    {"_ZN5clang21TextDiagnosticPrinterD0Ev", "clang::TextDiagnosticPrinter::~TextDiagnosticPrinter()"},
    {"_ZN5clang21TextDiagnosticPrinterD1Ev", "clang::TextDiagnosticPrinter::~TextDiagnosticPrinter()"},
    {"_ZN5clang21TextDiagnosticPrinterD2Ev", "clang::TextDiagnosticPrinter::~TextDiagnosticPrinter()"},
    {"_ZN5clang21TextDiagnosticPrinter17PrintIncludeStackENS_14SourceLocationERKNS_13SourceManagerE", "clang::TextDiagnosticPrinter::PrintIncludeStack(clang::SourceLocation, clang::SourceManager const&)"},
    {"_ZN5clang21TextDiagnosticPrinter14HighlightRangeERKNS_15CharSourceRangeERKNS_13SourceManagerEjNS_6FileIDERSsRKSs", "clang::TextDiagnosticPrinter::HighlightRange(clang::CharSourceRange const&, clang::SourceManager const&, unsigned int, clang::FileID, std::string&, std::string const&)"},
    {"_ZN5clang21TextDiagnosticPrinter19EmitCaretDiagnosticENS_14SourceLocationEPNS_15CharSourceRangeEjRKNS_13SourceManagerEPKNS_9FixItHintEjjjjj", "clang::TextDiagnosticPrinter::EmitCaretDiagnostic(clang::SourceLocation, clang::CharSourceRange*, unsigned int, clang::SourceManager const&, clang::FixItHint const*, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int)"},
    {"_ZN5clang21TextDiagnosticPrinter16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::TextDiagnosticPrinter::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_Z13findEndOfWordjRKN4llvm15SmallVectorImplIcEEjjj", "findEndOfWord(unsigned int, llvm::SmallVectorImpl<char> const&, unsigned int, unsigned int, unsigned int)"},
    {"_ZNK5clang13SourceManager16getDecomposedLocENS_14SourceLocationE", "clang::SourceManager::getDecomposedLoc(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager29getDecomposedInstantiationLocENS_14SourceLocationE", "clang::SourceManager::getDecomposedInstantiationLoc(clang::SourceLocation) const"},
    {"_ZN5clang21TextDiagnosticPrinter15BeginSourceFileERKNS_11LangOptionsEPKNS_12PreprocessorE", "clang::TextDiagnosticPrinter::BeginSourceFile(clang::LangOptions const&, clang::Preprocessor const*)"},
    {"_ZN5clang21TextDiagnosticPrinter13EndSourceFileEv", "clang::TextDiagnosticPrinter::EndSourceFile()"},
    {"_ZNK5clang13SourceManager16isOffsetInFileIDENS_6FileIDEj", "clang::SourceManager::isOffsetInFileID(clang::FileID, unsigned int) const"},
    {"_ZN5clang11DocumentXML14addParentTypesEPKNS_4TypeE", "clang::DocumentXML::addParentTypes(clang::Type const*)"},
    {"_ZN5clang11DocumentXML14writeTypeToXMLEPKNS_4TypeE", "clang::DocumentXML::writeTypeToXML(clang::Type const*)"},
    {"_ZN5clang11DocumentXML14writeTypeToXMLERKNS_8QualTypeE", "clang::DocumentXML::writeTypeToXML(clang::QualType const&)"},
    {"_ZN5clang3XML12_GLOBAL__N_110TypeWriter9VisitTypeEPKNS_4TypeE", "clang::XML::(anonymous namespace)::TypeWriter::VisitType(clang::Type const*)"},
    {"_ZN5clang11TypeVisitorINS_11DocumentXML9TypeAdderEvE5VisitEPKNS_4TypeE", "clang::TypeVisitor<clang::DocumentXML::TypeAdder, void>::Visit(clang::Type const*)"},
    {"_ZN5clang11DocumentXML12addAttributeIN4llvm5APIntEEEvPKcRKT_", "void clang::DocumentXML::addAttribute<llvm::APInt>(char const*, llvm::APInt const&)"},
    {"_ZN5clang11DocumentXML12addAttributeINS_10Qualifiers2GCEEEvPKcRKT_", "void clang::DocumentXML::addAttribute<clang::Qualifiers::GC>(char const*, clang::Qualifiers::GC const&)"},
    {"_ZN5clang23VerifyDiagnosticsClientC1ERNS_10DiagnosticEPNS_16DiagnosticClientE", "clang::VerifyDiagnosticsClient::VerifyDiagnosticsClient(clang::Diagnostic&, clang::DiagnosticClient*)"},
    {"_ZN5clang23VerifyDiagnosticsClientC2ERNS_10DiagnosticEPNS_16DiagnosticClientE", "clang::VerifyDiagnosticsClient::VerifyDiagnosticsClient(clang::Diagnostic&, clang::DiagnosticClient*)"},
    {"_ZN5clang23VerifyDiagnosticsClientD0Ev", "clang::VerifyDiagnosticsClient::~VerifyDiagnosticsClient()"},
    {"_ZN5clang23VerifyDiagnosticsClientD1Ev", "clang::VerifyDiagnosticsClient::~VerifyDiagnosticsClient()"},
    {"_ZN5clang23VerifyDiagnosticsClientD2Ev", "clang::VerifyDiagnosticsClient::~VerifyDiagnosticsClient()"},
    {"_ZN5clang23VerifyDiagnosticsClient16CheckDiagnosticsEv", "clang::VerifyDiagnosticsClient::CheckDiagnostics()"},
    {"_ZN5clang23VerifyDiagnosticsClient15BeginSourceFileERKNS_11LangOptionsEPKNS_12PreprocessorE", "clang::VerifyDiagnosticsClient::BeginSourceFile(clang::LangOptions const&, clang::Preprocessor const*)"},
    {"_ZN5clang23VerifyDiagnosticsClient13EndSourceFileEv", "clang::VerifyDiagnosticsClient::EndSourceFile()"},
    {"_ZN5clang23VerifyDiagnosticsClient16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::VerifyDiagnosticsClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_Z12PrintProblemRN5clang10DiagnosticEPNS_13SourceManagerEN9__gnu_cxx17__normal_iteratorIPKSt4pairINS_14SourceLocationESsESt6vectorIS8_SaIS8_EEEESE_PKcb", "PrintProblem(clang::Diagnostic&, clang::SourceManager*, __gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, char const*, bool)"},
    {"_Z10CheckListsRN5clang10DiagnosticERNS_13SourceManagerEPKcRSt6vectorIPN12_GLOBAL__N_19DirectiveESaIS9_EEN9__gnu_cxx17__normal_iteratorIPKSt4pairINS_14SourceLocationESsES6_ISH_SaISH_EEEESM_", "CheckLists(clang::Diagnostic&, clang::SourceManager&, char const*, std::vector<(anonymous namespace)::Directive*, std::allocator<(anonymous namespace)::Directive*> >&, __gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >)"},
    {"_ZNSt6vectorIPN12_GLOBAL__N_19DirectiveESaIS2_EE9push_backERKS2_", "std::vector<(anonymous namespace)::Directive*, std::allocator<(anonymous namespace)::Directive*> >::push_back((anonymous namespace)::Directive* const&)"},
    {"_ZN12_GLOBAL__N_117StandardDirectiveD1Ev", "(anonymous namespace)::StandardDirective::~StandardDirective()"},
    {"_ZN12_GLOBAL__N_117StandardDirectiveD0Ev", "(anonymous namespace)::StandardDirective::~StandardDirective()"},
    {"_ZN12_GLOBAL__N_117StandardDirective7isValidERSs", "(anonymous namespace)::StandardDirective::isValid(std::string&)"},
    {"_ZN12_GLOBAL__N_117StandardDirective5MatchERKSs", "(anonymous namespace)::StandardDirective::Match(std::string const&)"},
    {"_ZN12_GLOBAL__N_19DirectiveD1Ev", "(anonymous namespace)::Directive::~Directive()"},
    {"_ZN12_GLOBAL__N_19DirectiveD0Ev", "(anonymous namespace)::Directive::~Directive()"},
    {"_ZN12_GLOBAL__N_114RegexDirectiveD1Ev", "(anonymous namespace)::RegexDirective::~RegexDirective()"},
    {"_ZN12_GLOBAL__N_114RegexDirectiveD0Ev", "(anonymous namespace)::RegexDirective::~RegexDirective()"},
    {"_ZN12_GLOBAL__N_114RegexDirective7isValidERSs", "(anonymous namespace)::RegexDirective::isValid(std::string&)"},
    {"_ZN12_GLOBAL__N_114RegexDirective5MatchERKSs", "(anonymous namespace)::RegexDirective::Match(std::string const&)"},
    {"_ZNSt6vectorISt4pairIN5clang14SourceLocationESsESaIS3_EE19_M_range_initializeIN9__gnu_cxx17__normal_iteratorIPKS3_S5_EEEEvT_SC_St20forward_iterator_tag", "void std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > >::_M_range_initialize<__gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<clang::SourceLocation, std::string> const*, std::vector<std::pair<clang::SourceLocation, std::string>, std::allocator<std::pair<clang::SourceLocation, std::string> > > >, std::forward_iterator_tag)"},
    {"_ZSt6searchIPKcS1_ET_S2_S2_T0_S3_", "char const* std::search<char const*, char const*>(char const*, char const*, char const*, char const*)"},
    {"_ZN5clang21ProcessWarningOptionsERNS_10DiagnosticERKNS_17DiagnosticOptionsE", "clang::ProcessWarningOptions(clang::Diagnostic&, clang::DiagnosticOptions const&)"},
    {"_ZN5clang6driver6ActionD0Ev", "clang::driver::Action::~Action()"},
    {"_ZN5clang6driver6ActionD1Ev", "clang::driver::Action::~Action()"},
    {"_ZN5clang6driver6ActionD2Ev", "clang::driver::Action::~Action()"},
    {"_ZN5clang6driver6Action12getClassNameENS1_11ActionClassE", "clang::driver::Action::getClassName(clang::driver::Action::ActionClass)"},
    {"_ZN5clang6driver11InputActionC1ERKNS0_3ArgENS0_5types2IDE", "clang::driver::InputAction::InputAction(clang::driver::Arg const&, clang::driver::types::ID)"},
    {"_ZN5clang6driver11InputActionC2ERKNS0_3ArgENS0_5types2IDE", "clang::driver::InputAction::InputAction(clang::driver::Arg const&, clang::driver::types::ID)"},
    {"_ZN5clang6driver14BindArchActionC1EPNS0_6ActionEPKc", "clang::driver::BindArchAction::BindArchAction(clang::driver::Action*, char const*)"},
    {"_ZN5clang6driver14BindArchActionC2EPNS0_6ActionEPKc", "clang::driver::BindArchAction::BindArchAction(clang::driver::Action*, char const*)"},
    {"_ZN5clang6driver9JobActionC1ENS0_6Action11ActionClassEPS2_NS0_5types2IDE", "clang::driver::JobAction::JobAction(clang::driver::Action::ActionClass, clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver9JobActionC2ENS0_6Action11ActionClassEPS2_NS0_5types2IDE", "clang::driver::JobAction::JobAction(clang::driver::Action::ActionClass, clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver9JobActionC1ENS0_6Action11ActionClassERKN4llvm11SmallVectorIPS2_Lj3EEENS0_5types2IDE", "clang::driver::JobAction::JobAction(clang::driver::Action::ActionClass, llvm::SmallVector<clang::driver::Action*, 3u> const&, clang::driver::types::ID)"},
    {"_ZN5clang6driver9JobActionC2ENS0_6Action11ActionClassERKN4llvm11SmallVectorIPS2_Lj3EEENS0_5types2IDE", "clang::driver::JobAction::JobAction(clang::driver::Action::ActionClass, llvm::SmallVector<clang::driver::Action*, 3u> const&, clang::driver::types::ID)"},
    {"_ZN5clang6driver19PreprocessJobActionC1EPNS0_6ActionENS0_5types2IDE", "clang::driver::PreprocessJobAction::PreprocessJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver19PreprocessJobActionC2EPNS0_6ActionENS0_5types2IDE", "clang::driver::PreprocessJobAction::PreprocessJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver19PrecompileJobActionC1EPNS0_6ActionENS0_5types2IDE", "clang::driver::PrecompileJobAction::PrecompileJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver19PrecompileJobActionC2EPNS0_6ActionENS0_5types2IDE", "clang::driver::PrecompileJobAction::PrecompileJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver16AnalyzeJobActionC1EPNS0_6ActionENS0_5types2IDE", "clang::driver::AnalyzeJobAction::AnalyzeJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver16AnalyzeJobActionC2EPNS0_6ActionENS0_5types2IDE", "clang::driver::AnalyzeJobAction::AnalyzeJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver16CompileJobActionC1EPNS0_6ActionENS0_5types2IDE", "clang::driver::CompileJobAction::CompileJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver16CompileJobActionC2EPNS0_6ActionENS0_5types2IDE", "clang::driver::CompileJobAction::CompileJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver17AssembleJobActionC1EPNS0_6ActionENS0_5types2IDE", "clang::driver::AssembleJobAction::AssembleJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver17AssembleJobActionC2EPNS0_6ActionENS0_5types2IDE", "clang::driver::AssembleJobAction::AssembleJobAction(clang::driver::Action*, clang::driver::types::ID)"},
    {"_ZN5clang6driver13LinkJobActionC1ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::LinkJobAction::LinkJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver13LinkJobActionC2ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::LinkJobAction::LinkJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver13LipoJobActionC1ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::LipoJobAction::LipoJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver13LipoJobActionC2ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::LipoJobAction::LipoJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver17DsymutilJobActionC1ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::DsymutilJobAction::DsymutilJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver17DsymutilJobActionC2ERN4llvm11SmallVectorIPNS0_6ActionELj3EEENS0_5types2IDE", "clang::driver::DsymutilJobAction::DsymutilJobAction(llvm::SmallVector<clang::driver::Action*, 3u>&, clang::driver::types::ID)"},
    {"_ZN5clang6driver17DsymutilJobActionD1Ev", "clang::driver::DsymutilJobAction::~DsymutilJobAction()"},
    {"_ZN5clang6driver17DsymutilJobActionD0Ev", "clang::driver::DsymutilJobAction::~DsymutilJobAction()"},
    {"_ZN5clang6driver13LipoJobActionD1Ev", "clang::driver::LipoJobAction::~LipoJobAction()"},
    {"_ZN5clang6driver13LipoJobActionD0Ev", "clang::driver::LipoJobAction::~LipoJobAction()"},
    {"_ZN5clang6driver13LinkJobActionD1Ev", "clang::driver::LinkJobAction::~LinkJobAction()"},
    {"_ZN5clang6driver13LinkJobActionD0Ev", "clang::driver::LinkJobAction::~LinkJobAction()"},
    {"_ZN5clang6driver17AssembleJobActionD1Ev", "clang::driver::AssembleJobAction::~AssembleJobAction()"},
    {"_ZN5clang6driver17AssembleJobActionD0Ev", "clang::driver::AssembleJobAction::~AssembleJobAction()"},
    {"_ZN5clang6driver16CompileJobActionD1Ev", "clang::driver::CompileJobAction::~CompileJobAction()"},
    {"_ZN5clang6driver16CompileJobActionD0Ev", "clang::driver::CompileJobAction::~CompileJobAction()"},
    {"_ZN5clang6driver16AnalyzeJobActionD1Ev", "clang::driver::AnalyzeJobAction::~AnalyzeJobAction()"},
    {"_ZN5clang6driver16AnalyzeJobActionD0Ev", "clang::driver::AnalyzeJobAction::~AnalyzeJobAction()"},
    {"_ZN5clang6driver19PrecompileJobActionD1Ev", "clang::driver::PrecompileJobAction::~PrecompileJobAction()"},
    {"_ZN5clang6driver19PrecompileJobActionD0Ev", "clang::driver::PrecompileJobAction::~PrecompileJobAction()"},
    {"_ZN5clang6driver19PreprocessJobActionD1Ev", "clang::driver::PreprocessJobAction::~PreprocessJobAction()"},
    {"_ZN5clang6driver19PreprocessJobActionD0Ev", "clang::driver::PreprocessJobAction::~PreprocessJobAction()"},
    {"_ZN5clang6driver9JobActionD1Ev", "clang::driver::JobAction::~JobAction()"},
    {"_ZN5clang6driver9JobActionD0Ev", "clang::driver::JobAction::~JobAction()"},
    {"_ZN5clang6driver14BindArchActionD1Ev", "clang::driver::BindArchAction::~BindArchAction()"},
    {"_ZN5clang6driver14BindArchActionD0Ev", "clang::driver::BindArchAction::~BindArchAction()"},
    {"_ZN5clang6driver11InputActionD1Ev", "clang::driver::InputAction::~InputAction()"},
    {"_ZN5clang6driver11InputActionD0Ev", "clang::driver::InputAction::~InputAction()"},
    {"_ZN4llvm15SmallVectorImplIPN5clang6driver6ActionEEaSERKS5_", "llvm::SmallVectorImpl<clang::driver::Action*>::operator=(llvm::SmallVectorImpl<clang::driver::Action*> const&)"},
    {"_ZN5clang6driver3ArgC1EPKNS0_6OptionEjPKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgC2EPKNS0_6OptionEjPKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgC1EPKNS0_6OptionEjPKcPKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, char const*, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgC2EPKNS0_6OptionEjPKcPKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, char const*, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgC1EPKNS0_6OptionEjPKcS6_PKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, char const*, char const*, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgC2EPKNS0_6OptionEjPKcS6_PKS1_", "clang::driver::Arg::Arg(clang::driver::Option const*, unsigned int, char const*, char const*, clang::driver::Arg const*)"},
    {"_ZN5clang6driver3ArgD1Ev", "clang::driver::Arg::~Arg()"},
    {"_ZN5clang6driver3ArgD2Ev", "clang::driver::Arg::~Arg()"},
    {"_ZNK5clang6driver3Arg4dumpEv", "clang::driver::Arg::dump() const"},
    {"_ZNK5clang6driver3Arg11getAsStringERKNS0_7ArgListE", "clang::driver::Arg::getAsString(clang::driver::ArgList const&) const"},
    {"_ZNK5clang6driver3Arg6renderERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::Arg::render(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver3Arg13renderAsInputERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::Arg::renderAsInput(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZN5clang6driver12arg_iterator13SkipToNextArgEv", "clang::driver::arg_iterator::SkipToNextArg()"},
    {"_ZN5clang6driver7ArgListC1Ev", "clang::driver::ArgList::ArgList()"},
    {"_ZN5clang6driver7ArgListC2Ev", "clang::driver::ArgList::ArgList()"},
    {"_ZN5clang6driver7ArgListD0Ev", "clang::driver::ArgList::~ArgList()"},
    {"_ZN5clang6driver7ArgListD1Ev", "clang::driver::ArgList::~ArgList()"},
    {"_ZN5clang6driver7ArgListD2Ev", "clang::driver::ArgList::~ArgList()"},
    {"_ZN5clang6driver7ArgList6appendEPNS0_3ArgE", "clang::driver::ArgList::append(clang::driver::Arg*)"},
    {"_ZNK5clang6driver7ArgList17getLastArgNoClaimENS0_12OptSpecifierE", "clang::driver::ArgList::getLastArgNoClaim(clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10getLastArgENS0_12OptSpecifierE", "clang::driver::ArgList::getLastArg(clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10getLastArgENS0_12OptSpecifierES2_", "clang::driver::ArgList::getLastArg(clang::driver::OptSpecifier, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10getLastArgENS0_12OptSpecifierES2_S2_", "clang::driver::ArgList::getLastArg(clang::driver::OptSpecifier, clang::driver::OptSpecifier, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10getLastArgENS0_12OptSpecifierES2_S2_S2_", "clang::driver::ArgList::getLastArg(clang::driver::OptSpecifier, clang::driver::OptSpecifier, clang::driver::OptSpecifier, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList7hasFlagENS0_12OptSpecifierES2_b", "clang::driver::ArgList::hasFlag(clang::driver::OptSpecifier, clang::driver::OptSpecifier, bool) const"},
    {"_ZNK5clang6driver7ArgList15getLastArgValueENS0_12OptSpecifierEN4llvm9StringRefE", "clang::driver::ArgList::getLastArgValue(clang::driver::OptSpecifier, llvm::StringRef) const"},
    {"_ZNK5clang6driver7ArgList18getLastArgIntValueENS0_12OptSpecifierEiRNS_10DiagnosticE", "clang::driver::ArgList::getLastArgIntValue(clang::driver::OptSpecifier, int, clang::Diagnostic&) const"},
    {"_ZNK5clang6driver7ArgList15getAllArgValuesENS0_12OptSpecifierE", "clang::driver::ArgList::getAllArgValues(clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList15AddAllArgValuesERN4llvm11SmallVectorIPKcLj16EEENS0_12OptSpecifierES8_S8_", "clang::driver::ArgList::AddAllArgValues(llvm::SmallVector<char const*, 16u>&, clang::driver::OptSpecifier, clang::driver::OptSpecifier, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10AddLastArgERN4llvm11SmallVectorIPKcLj16EEENS0_12OptSpecifierE", "clang::driver::ArgList::AddLastArg(llvm::SmallVector<char const*, 16u>&, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList10AddAllArgsERN4llvm11SmallVectorIPKcLj16EEENS0_12OptSpecifierES8_S8_", "clang::driver::ArgList::AddAllArgs(llvm::SmallVector<char const*, 16u>&, clang::driver::OptSpecifier, clang::driver::OptSpecifier, clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList20AddAllArgsTranslatedERN4llvm11SmallVectorIPKcLj16EEENS0_12OptSpecifierES5_b", "clang::driver::ArgList::AddAllArgsTranslated(llvm::SmallVector<char const*, 16u>&, clang::driver::OptSpecifier, char const*, bool) const"},
    {"_ZNK5clang6driver7ArgList13MakeArgStringERKN4llvm5TwineE", "clang::driver::ArgList::MakeArgString(llvm::Twine const&) const"},
    {"_ZNK5clang6driver7ArgList12ClaimAllArgsENS0_12OptSpecifierE", "clang::driver::ArgList::ClaimAllArgs(clang::driver::OptSpecifier) const"},
    {"_ZNK5clang6driver7ArgList24GetOrMakeJoinedArgStringEjN4llvm9StringRefES3_", "clang::driver::ArgList::GetOrMakeJoinedArgString(unsigned int, llvm::StringRef, llvm::StringRef) const"},
    {"_ZN5clang6driver12InputArgListC1EPKPKcS5_", "clang::driver::InputArgList::InputArgList(char const* const*, char const* const*)"},
    {"_ZN5clang6driver12InputArgListC2EPKPKcS5_", "clang::driver::InputArgList::InputArgList(char const* const*, char const* const*)"},
    {"_ZN5clang6driver12InputArgListD0Ev", "clang::driver::InputArgList::~InputArgList()"},
    {"_ZN5clang6driver12InputArgListD1Ev", "clang::driver::InputArgList::~InputArgList()"},
    {"_ZN5clang6driver12InputArgListD2Ev", "clang::driver::InputArgList::~InputArgList()"},
    {"_ZNK5clang6driver12InputArgList9MakeIndexEN4llvm9StringRefE", "clang::driver::InputArgList::MakeIndex(llvm::StringRef) const"},
    {"_ZNK5clang6driver12InputArgList9MakeIndexEN4llvm9StringRefES3_", "clang::driver::InputArgList::MakeIndex(llvm::StringRef, llvm::StringRef) const"},
    {"_ZNK5clang6driver12InputArgList13MakeArgStringEN4llvm9StringRefE", "clang::driver::InputArgList::MakeArgString(llvm::StringRef) const"},
    {"_ZN5clang6driver14DerivedArgListC1ERKNS0_12InputArgListE", "clang::driver::DerivedArgList::DerivedArgList(clang::driver::InputArgList const&)"},
    {"_ZN5clang6driver14DerivedArgListC2ERKNS0_12InputArgListE", "clang::driver::DerivedArgList::DerivedArgList(clang::driver::InputArgList const&)"},
    {"_ZN5clang6driver14DerivedArgListD0Ev", "clang::driver::DerivedArgList::~DerivedArgList()"},
    {"_ZN5clang6driver14DerivedArgListD1Ev", "clang::driver::DerivedArgList::~DerivedArgList()"},
    {"_ZN5clang6driver14DerivedArgListD2Ev", "clang::driver::DerivedArgList::~DerivedArgList()"},
    {"_ZNK5clang6driver14DerivedArgList13MakeArgStringEN4llvm9StringRefE", "clang::driver::DerivedArgList::MakeArgString(llvm::StringRef) const"},
    {"_ZNK5clang6driver14DerivedArgList11MakeFlagArgEPKNS0_3ArgEPKNS0_6OptionE", "clang::driver::DerivedArgList::MakeFlagArg(clang::driver::Arg const*, clang::driver::Option const*) const"},
    {"_ZNK5clang6driver14DerivedArgList17MakePositionalArgEPKNS0_3ArgEPKNS0_6OptionEN4llvm9StringRefE", "clang::driver::DerivedArgList::MakePositionalArg(clang::driver::Arg const*, clang::driver::Option const*, llvm::StringRef) const"},
    {"_ZNK5clang6driver14DerivedArgList15MakeSeparateArgEPKNS0_3ArgEPKNS0_6OptionEN4llvm9StringRefE", "clang::driver::DerivedArgList::MakeSeparateArg(clang::driver::Arg const*, clang::driver::Option const*, llvm::StringRef) const"},
    {"_ZNK5clang6driver14DerivedArgList13MakeJoinedArgEPKNS0_3ArgEPKNS0_6OptionEN4llvm9StringRefE", "clang::driver::DerivedArgList::MakeJoinedArg(clang::driver::Arg const*, clang::driver::Option const*, llvm::StringRef) const"},
    {"_ZNK5clang6driver12InputArgList12getArgStringEj", "clang::driver::InputArgList::getArgString(unsigned int) const"},
    {"_ZNK5clang6driver12InputArgList21getNumInputArgStringsEv", "clang::driver::InputArgList::getNumInputArgStrings() const"},
    {"_ZNK5clang6driver14DerivedArgList12getArgStringEj", "clang::driver::DerivedArgList::getArgString(unsigned int) const"},
    {"_ZNK5clang6driver14DerivedArgList21getNumInputArgStringsEv", "clang::driver::DerivedArgList::getNumInputArgStrings() const"},
    {"_ZNSt6vectorISsSaISsEE22_M_initialize_dispatchIPPKcEEvT_S6_St12__false_type", "void std::vector<std::string, std::allocator<std::string> >::_M_initialize_dispatch<char const**>(char const**, char const**, std::__false_type)"},
    {"_ZN5clang6driver19createCC1AsOptTableEv", "clang::driver::createCC1AsOptTable()"},
    {"_ZN5clang6driver17createCC1OptTableEv", "clang::driver::createCC1OptTable()"},
    {"_ZN5clang6driver11CompilationC1ERKNS0_6DriverERKNS0_9ToolChainEPNS0_12InputArgListEPNS0_14DerivedArgListE", "clang::driver::Compilation::Compilation(clang::driver::Driver const&, clang::driver::ToolChain const&, clang::driver::InputArgList*, clang::driver::DerivedArgList*)"},
    {"_ZN5clang6driver11CompilationC2ERKNS0_6DriverERKNS0_9ToolChainEPNS0_12InputArgListEPNS0_14DerivedArgListE", "clang::driver::Compilation::Compilation(clang::driver::Driver const&, clang::driver::ToolChain const&, clang::driver::InputArgList*, clang::driver::DerivedArgList*)"},
    {"_ZN5clang6driver11CompilationD1Ev", "clang::driver::Compilation::~Compilation()"},
    {"_ZN5clang6driver11CompilationD2Ev", "clang::driver::Compilation::~Compilation()"},
    {"_ZN5clang6driver11Compilation19getArgsForToolChainEPKNS0_9ToolChainEPKc", "clang::driver::Compilation::getArgsForToolChain(clang::driver::ToolChain const*, char const*)"},
    {"_ZNK5clang6driver11Compilation8PrintJobERN4llvm11raw_ostreamERKNS0_3JobEPKcb", "clang::driver::Compilation::PrintJob(llvm::raw_ostream&, clang::driver::Job const&, char const*, bool) const"},
    {"_ZNK5clang6driver11Compilation15CleanupFileListERKN4llvm11SmallVectorIPKcLj16EEEb", "clang::driver::Compilation::CleanupFileList(llvm::SmallVector<char const*, 16u> const&, bool) const"},
    {"_ZNK5clang6driver11Compilation14ExecuteCommandERKNS0_7CommandERPS3_", "clang::driver::Compilation::ExecuteCommand(clang::driver::Command const&, clang::driver::Command const*&) const"},
    {"_ZNK5clang6driver11Compilation10ExecuteJobERKNS0_3JobERPKNS0_7CommandE", "clang::driver::Compilation::ExecuteJob(clang::driver::Job const&, clang::driver::Command const*&) const"},
    {"_ZNK4llvm8DenseMapISt4pairIPKN5clang6driver9ToolChainEPKcEPNS3_14DerivedArgListENS_12DenseMapInfoIS9_EENSC_ISB_EEE15LookupBucketForERKS9_RPS1_IS9_SB_E", "llvm::DenseMap<std::pair<clang::driver::ToolChain const*, char const*>, clang::driver::DerivedArgList*, llvm::DenseMapInfo<std::pair<clang::driver::ToolChain const*, char const*> >, llvm::DenseMapInfo<clang::driver::DerivedArgList*> >::LookupBucketFor(std::pair<clang::driver::ToolChain const*, char const*> const&, std::pair<std::pair<clang::driver::ToolChain const*, char const*>, clang::driver::DerivedArgList*>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang6driver9ToolChainEPKcEPNS3_14DerivedArgListENS_12DenseMapInfoIS9_EENSC_ISB_EEE16InsertIntoBucketERKS9_RKSB_PS1_IS9_SB_E", "llvm::DenseMap<std::pair<clang::driver::ToolChain const*, char const*>, clang::driver::DerivedArgList*, llvm::DenseMapInfo<std::pair<clang::driver::ToolChain const*, char const*> >, llvm::DenseMapInfo<clang::driver::DerivedArgList*> >::InsertIntoBucket(std::pair<clang::driver::ToolChain const*, char const*> const&, clang::driver::DerivedArgList* const&, std::pair<std::pair<clang::driver::ToolChain const*, char const*>, clang::driver::DerivedArgList*>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang6driver9ToolChainEPKcEPNS3_14DerivedArgListENS_12DenseMapInfoIS9_EENSC_ISB_EEE4growEj", "llvm::DenseMap<std::pair<clang::driver::ToolChain const*, char const*>, clang::driver::DerivedArgList*, llvm::DenseMapInfo<std::pair<clang::driver::ToolChain const*, char const*> >, llvm::DenseMapInfo<clang::driver::DerivedArgList*> >::grow(unsigned int)"},
    {"_ZN5clang6driver6DriverC1EN4llvm9StringRefES3_S3_bbRNS_10DiagnosticE", "clang::driver::Driver::Driver(llvm::StringRef, llvm::StringRef, llvm::StringRef, bool, bool, clang::Diagnostic&)"},
    {"_ZN5clang6driver6DriverC2EN4llvm9StringRefES3_S3_bbRNS_10DiagnosticE", "clang::driver::Driver::Driver(llvm::StringRef, llvm::StringRef, llvm::StringRef, bool, bool, clang::Diagnostic&)"},
    {"_ZN5clang6driver6DriverD1Ev", "clang::driver::Driver::~Driver()"},
    {"_ZN5clang6driver6DriverD2Ev", "clang::driver::Driver::~Driver()"},
    {"_ZN5clang6driver6Driver15ParseArgStringsEPPKcS4_", "clang::driver::Driver::ParseArgStrings(char const**, char const**)"},
    {"_ZNK5clang6driver6Driver18TranslateInputArgsERKNS0_12InputArgListE", "clang::driver::Driver::TranslateInputArgs(clang::driver::InputArgList const&) const"},
    {"_ZN5clang6driver6Driver16BuildCompilationEiPPKc", "clang::driver::Driver::BuildCompilation(int, char const**)"},
    {"_ZNK5clang6driver6Driver11GetHostInfoEPKc", "clang::driver::Driver::GetHostInfo(char const*) const"},
    {"_ZNK5clang6driver6Driver12PrintOptionsERKNS0_7ArgListE", "clang::driver::Driver::PrintOptions(clang::driver::ArgList const&) const"},
    {"_ZN5clang6driver6Driver19HandleImmediateArgsERKNS0_11CompilationE", "clang::driver::Driver::HandleImmediateArgs(clang::driver::Compilation const&)"},
    {"_ZNK5clang6driver6Driver21BuildUniversalActionsERKNS0_9ToolChainERKNS0_7ArgListERN4llvm11SmallVectorIPNS0_6ActionELj3EEE", "clang::driver::Driver::BuildUniversalActions(clang::driver::ToolChain const&, clang::driver::ArgList const&, llvm::SmallVector<clang::driver::Action*, 3u>&) const"},
    {"_ZNK5clang6driver6Driver12BuildActionsERKNS0_9ToolChainERKNS0_7ArgListERN4llvm11SmallVectorIPNS0_6ActionELj3EEE", "clang::driver::Driver::BuildActions(clang::driver::ToolChain const&, clang::driver::ArgList const&, llvm::SmallVector<clang::driver::Action*, 3u>&) const"},
    {"_ZNK5clang6driver6Driver12PrintActionsERKNS0_11CompilationE", "clang::driver::Driver::PrintActions(clang::driver::Compilation const&) const"},
    {"_ZNK5clang6driver6Driver9BuildJobsERNS0_11CompilationE", "clang::driver::Driver::BuildJobs(clang::driver::Compilation&) const"},
    {"_ZNK5clang6driver6Driver18ExecuteCompilationERKNS0_11CompilationE", "clang::driver::Driver::ExecuteCompilation(clang::driver::Compilation const&) const"},
    {"_ZNK5clang6driver6Driver9PrintHelpEb", "clang::driver::Driver::PrintHelp(bool) const"},
    {"_ZNK5clang6driver6Driver12PrintVersionERKNS0_11CompilationERN4llvm11raw_ostreamE", "clang::driver::Driver::PrintVersion(clang::driver::Compilation const&, llvm::raw_ostream&) const"},
    {"_ZNK5clang6driver6Driver11GetFilePathEPKcRKNS0_9ToolChainE", "clang::driver::Driver::GetFilePath(char const*, clang::driver::ToolChain const&) const"},
    {"_ZNK5clang6driver6Driver14GetProgramPathEPKcRKNS0_9ToolChainEb", "clang::driver::Driver::GetProgramPath(char const*, clang::driver::ToolChain const&, bool) const"},
    {"_Z13PrintActions1RKN5clang6driver11CompilationEPNS0_6ActionERSt3mapIS5_jSt4lessIS5_ESaISt4pairIKS5_jEEE", "PrintActions1(clang::driver::Compilation const&, clang::driver::Action*, std::map<clang::driver::Action*, unsigned int, std::less<clang::driver::Action*>, std::allocator<std::pair<clang::driver::Action* const, unsigned int> > >&)"},
    {"_Z21ContainsCompileActionPKN5clang6driver6ActionE", "ContainsCompileAction(clang::driver::Action const*)"},
    {"_ZNK5clang6driver6Driver20ConstructPhaseActionERKNS0_7ArgListENS0_6phases2IDEPNS0_6ActionE", "clang::driver::Driver::ConstructPhaseAction(clang::driver::ArgList const&, clang::driver::phases::ID, clang::driver::Action*) const"},
    {"_ZNK5clang6driver6Driver18BuildJobsForActionERNS0_11CompilationEPKNS0_6ActionEPKNS0_9ToolChainEPKcbSB_RNS0_9InputInfoE", "clang::driver::Driver::BuildJobsForAction(clang::driver::Compilation&, clang::driver::Action const*, clang::driver::ToolChain const*, char const*, bool, char const*, clang::driver::InputInfo&) const"},
    {"_ZNK5clang6driver6Driver18GetNamedOutputPathERNS0_11CompilationERKNS0_9JobActionEPKcb", "clang::driver::Driver::GetNamedOutputPath(clang::driver::Compilation&, clang::driver::JobAction const&, char const*, bool) const"},
    {"_ZNK5clang6driver6Driver16GetTemporaryPathEPKc", "clang::driver::Driver::GetTemporaryPath(char const*) const"},
    {"_ZNK5clang6driver6Driver22ShouldUseClangCompilerERKNS0_11CompilationERKNS0_9JobActionERKN4llvm6TripleE", "clang::driver::Driver::ShouldUseClangCompiler(clang::driver::Compilation const&, clang::driver::JobAction const&, llvm::Triple const&) const"},
    {"_ZN5clang6driver6Driver17GetReleaseVersionEPKcRjS4_S4_Rb", "clang::driver::Driver::GetReleaseVersion(char const*, unsigned int&, unsigned int&, unsigned int&, bool&)"},
    {"_ZN4llvm22PrettyStackTraceStringD1Ev", "llvm::PrettyStackTraceString::~PrettyStackTraceString()"},
    {"_ZN4llvm9StringSetINS_15MallocAllocatorEE6insertENS_9StringRefE", "llvm::StringSet<llvm::MallocAllocator>::insert(llvm::StringRef)"},
    {"_ZNK5clang6driver9InputInfo11getAsStringEv", "clang::driver::InputInfo::getAsString() const"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang6driver9InputInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::driver::InputInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISsLb0EE4growEm", "llvm::SmallVectorTemplateBase<std::string, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIN4llvm6Triple8ArchTypeES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::Triple::ArchType, llvm::Triple::ArchType, std::_Identity<llvm::Triple::ArchType>, std::less<llvm::Triple::ArchType>, std::allocator<llvm::Triple::ArchType> >::_M_erase(std::_Rb_tree_node<llvm::Triple::ArchType>*)"},
    {"_ZNSt8_Rb_treeIN4llvm6Triple8ArchTypeES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<llvm::Triple::ArchType, llvm::Triple::ArchType, std::_Identity<llvm::Triple::ArchType>, std::less<llvm::Triple::ArchType>, std::allocator<llvm::Triple::ArchType> >::_M_insert_unique(llvm::Triple::ArchType const&)"},
    {"_ZNSt8_Rb_treeIPN5clang6driver6ActionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<clang::driver::Action*, std::pair<clang::driver::Action* const, unsigned int>, std::_Select1st<std::pair<clang::driver::Action* const, unsigned int> >, std::less<clang::driver::Action*>, std::allocator<std::pair<clang::driver::Action* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::driver::Action* const, unsigned int> >*)"},
    {"_ZNK4llvm18raw_string_ostream11current_posEv", "llvm::raw_string_ostream::current_pos() const"},
    {"_ZNSt8_Rb_treeIPN5clang6driver6ActionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<clang::driver::Action*, std::pair<clang::driver::Action* const, unsigned int>, std::_Select1st<std::pair<clang::driver::Action* const, unsigned int> >, std::less<clang::driver::Action*>, std::allocator<std::pair<clang::driver::Action* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::driver::Action* const, unsigned int> >, std::pair<clang::driver::Action* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang6driver6ActionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<clang::driver::Action*, std::pair<clang::driver::Action* const, unsigned int>, std::_Select1st<std::pair<clang::driver::Action* const, unsigned int> >, std::less<clang::driver::Action*>, std::allocator<std::pair<clang::driver::Action* const, unsigned int> > >::_M_insert_unique(std::pair<clang::driver::Action* const, unsigned int> const&)"},
    {"_ZN4llvm6TripleC2ENS_9StringRefES1_S1_", "llvm::Triple::Triple(llvm::StringRef, llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm22PrettyStackTraceStringD0Ev", "llvm::PrettyStackTraceString::~PrettyStackTraceString()"},
    {"_ZN5clang6driver20createDriverOptTableEv", "clang::driver::createDriverOptTable()"},
    {"_ZN5clang6driver8HostInfoC1ERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::HostInfo::HostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver8HostInfoC2ERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::HostInfo::HostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver8HostInfoD0Ev", "clang::driver::HostInfo::~HostInfo()"},
    {"_ZN5clang6driver8HostInfoD1Ev", "clang::driver::HostInfo::~HostInfo()"},
    {"_ZN5clang6driver8HostInfoD2Ev", "clang::driver::HostInfo::~HostInfo()"},
    {"_ZN5clang6driver22createAuroraUXHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createAuroraUXHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver20createDarwinHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createDarwinHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver21createOpenBSDHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createOpenBSDHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver21createFreeBSDHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createFreeBSDHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver20createNetBSDHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createNetBSDHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver19createMinixHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createMinixHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver23createDragonFlyHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createDragonFlyHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver19createLinuxHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createLinuxHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver17createTCEHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createTCEHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver21createWindowsHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createWindowsHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver19createMinGWHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createMinGWHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN5clang6driver21createUnknownHostInfoERKNS0_6DriverERKN4llvm6TripleE", "clang::driver::createUnknownHostInfo(clang::driver::Driver const&, llvm::Triple const&)"},
    {"_ZN12_GLOBAL__N_115UnknownHostInfoD1Ev", "(anonymous namespace)::UnknownHostInfo::~UnknownHostInfo()"},
    {"_ZN12_GLOBAL__N_115UnknownHostInfoD0Ev", "(anonymous namespace)::UnknownHostInfo::~UnknownHostInfo()"},
    {"_ZNK12_GLOBAL__N_115UnknownHostInfo15useDriverDriverEv", "(anonymous namespace)::UnknownHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_115UnknownHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::UnknownHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_115UnknownHostInfoD2Ev", "(anonymous namespace)::UnknownHostInfo::~UnknownHostInfo()"},
    {"_ZN12_GLOBAL__N_113MinGWHostInfoD1Ev", "(anonymous namespace)::MinGWHostInfo::~MinGWHostInfo()"},
    {"_ZN12_GLOBAL__N_113MinGWHostInfoD0Ev", "(anonymous namespace)::MinGWHostInfo::~MinGWHostInfo()"},
    {"_ZN12_GLOBAL__N_115WindowsHostInfoD1Ev", "(anonymous namespace)::WindowsHostInfo::~WindowsHostInfo()"},
    {"_ZN12_GLOBAL__N_115WindowsHostInfoD0Ev", "(anonymous namespace)::WindowsHostInfo::~WindowsHostInfo()"},
    {"_ZNK12_GLOBAL__N_115WindowsHostInfo15useDriverDriverEv", "(anonymous namespace)::WindowsHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_115WindowsHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::WindowsHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZNK12_GLOBAL__N_115WindowsHostInfo22lookupTypeForExtensionEPKc", "(anonymous namespace)::WindowsHostInfo::lookupTypeForExtension(char const*) const"},
    {"_ZN12_GLOBAL__N_111TCEHostInfoD1Ev", "(anonymous namespace)::TCEHostInfo::~TCEHostInfo()"},
    {"_ZN12_GLOBAL__N_111TCEHostInfoD0Ev", "(anonymous namespace)::TCEHostInfo::~TCEHostInfo()"},
    {"_ZNK12_GLOBAL__N_111TCEHostInfo15useDriverDriverEv", "(anonymous namespace)::TCEHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_111TCEHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::TCEHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_113LinuxHostInfoD1Ev", "(anonymous namespace)::LinuxHostInfo::~LinuxHostInfo()"},
    {"_ZN12_GLOBAL__N_113LinuxHostInfoD0Ev", "(anonymous namespace)::LinuxHostInfo::~LinuxHostInfo()"},
    {"_ZNK12_GLOBAL__N_113LinuxHostInfo15useDriverDriverEv", "(anonymous namespace)::LinuxHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_113LinuxHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::LinuxHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_117DragonFlyHostInfoD1Ev", "(anonymous namespace)::DragonFlyHostInfo::~DragonFlyHostInfo()"},
    {"_ZN12_GLOBAL__N_117DragonFlyHostInfoD0Ev", "(anonymous namespace)::DragonFlyHostInfo::~DragonFlyHostInfo()"},
    {"_ZNK12_GLOBAL__N_117DragonFlyHostInfo15useDriverDriverEv", "(anonymous namespace)::DragonFlyHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_117DragonFlyHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::DragonFlyHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_113MinixHostInfoD1Ev", "(anonymous namespace)::MinixHostInfo::~MinixHostInfo()"},
    {"_ZN12_GLOBAL__N_113MinixHostInfoD0Ev", "(anonymous namespace)::MinixHostInfo::~MinixHostInfo()"},
    {"_ZNK12_GLOBAL__N_113MinixHostInfo15useDriverDriverEv", "(anonymous namespace)::MinixHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_113MinixHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::MinixHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_114NetBSDHostInfoD1Ev", "(anonymous namespace)::NetBSDHostInfo::~NetBSDHostInfo()"},
    {"_ZN12_GLOBAL__N_114NetBSDHostInfoD0Ev", "(anonymous namespace)::NetBSDHostInfo::~NetBSDHostInfo()"},
    {"_ZNK12_GLOBAL__N_114NetBSDHostInfo15useDriverDriverEv", "(anonymous namespace)::NetBSDHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_114NetBSDHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::NetBSDHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_115FreeBSDHostInfoD1Ev", "(anonymous namespace)::FreeBSDHostInfo::~FreeBSDHostInfo()"},
    {"_ZN12_GLOBAL__N_115FreeBSDHostInfoD0Ev", "(anonymous namespace)::FreeBSDHostInfo::~FreeBSDHostInfo()"},
    {"_ZNK12_GLOBAL__N_115FreeBSDHostInfo15useDriverDriverEv", "(anonymous namespace)::FreeBSDHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_115FreeBSDHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::FreeBSDHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_115OpenBSDHostInfoD1Ev", "(anonymous namespace)::OpenBSDHostInfo::~OpenBSDHostInfo()"},
    {"_ZN12_GLOBAL__N_115OpenBSDHostInfoD0Ev", "(anonymous namespace)::OpenBSDHostInfo::~OpenBSDHostInfo()"},
    {"_ZNK12_GLOBAL__N_115OpenBSDHostInfo15useDriverDriverEv", "(anonymous namespace)::OpenBSDHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_115OpenBSDHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::OpenBSDHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_114DarwinHostInfoD1Ev", "(anonymous namespace)::DarwinHostInfo::~DarwinHostInfo()"},
    {"_ZN12_GLOBAL__N_114DarwinHostInfoD0Ev", "(anonymous namespace)::DarwinHostInfo::~DarwinHostInfo()"},
    {"_ZNK12_GLOBAL__N_114DarwinHostInfo15useDriverDriverEv", "(anonymous namespace)::DarwinHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_114DarwinHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::DarwinHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN12_GLOBAL__N_116AuroraUXHostInfoD1Ev", "(anonymous namespace)::AuroraUXHostInfo::~AuroraUXHostInfo()"},
    {"_ZN12_GLOBAL__N_116AuroraUXHostInfoD0Ev", "(anonymous namespace)::AuroraUXHostInfo::~AuroraUXHostInfo()"},
    {"_ZNK12_GLOBAL__N_116AuroraUXHostInfo15useDriverDriverEv", "(anonymous namespace)::AuroraUXHostInfo::useDriverDriver() const"},
    {"_ZNK12_GLOBAL__N_116AuroraUXHostInfo15CreateToolChainERKN5clang6driver7ArgListEPKc", "(anonymous namespace)::AuroraUXHostInfo::CreateToolChain(clang::driver::ArgList const&, char const*) const"},
    {"_ZN4llvm9StringMapIPN5clang6driver9ToolChainENS_15MallocAllocatorEE16GetOrCreateValueIS4_EERNS_14StringMapEntryIS4_EENS_9StringRefET_", "llvm::StringMapEntry<clang::driver::ToolChain*>& llvm::StringMap<clang::driver::ToolChain*, llvm::MallocAllocator>::GetOrCreateValue<clang::driver::ToolChain*>(llvm::StringRef, clang::driver::ToolChain*)"},
    {"_ZNK5clang6driver9ToolChain13TranslateArgsERKNS0_14DerivedArgListEPKc", "clang::driver::ToolChain::TranslateArgs(clang::driver::DerivedArgList const&, char const*) const"},
    {"_ZNK5clang6driver9ToolChain15IsBlocksDefaultEv", "clang::driver::ToolChain::IsBlocksDefault() const"},
    {"_ZNK5clang6driver9ToolChain28IsIntegratedAssemblerDefaultEv", "clang::driver::ToolChain::IsIntegratedAssemblerDefault() const"},
    {"_ZNK5clang6driver9ToolChain23IsStrictAliasingDefaultEv", "clang::driver::ToolChain::IsStrictAliasingDefault() const"},
    {"_ZNK5clang6driver9ToolChain35IsObjCDefaultSynthPropertiesDefaultEv", "clang::driver::ToolChain::IsObjCDefaultSynthPropertiesDefault() const"},
    {"_ZNK5clang6driver9ToolChain26IsObjCNonFragileABIDefaultEv", "clang::driver::ToolChain::IsObjCNonFragileABIDefault() const"},
    {"_ZNK5clang6driver9ToolChain27IsObjCLegacyDispatchDefaultEv", "clang::driver::ToolChain::IsObjCLegacyDispatchDefault() const"},
    {"_ZNK5clang6driver9ToolChain20UseObjCMixedDispatchEv", "clang::driver::ToolChain::UseObjCMixedDispatch() const"},
    {"_ZNK5clang6driver9ToolChain29GetDefaultStackProtectorLevelEv", "clang::driver::ToolChain::GetDefaultStackProtectorLevel() const"},
    {"_ZNK5clang6driver10toolchains18Darwin_Generic_GCC25GetDefaultRelocationModelEv", "clang::driver::toolchains::Darwin_Generic_GCC::GetDefaultRelocationModel() const"},
    {"_ZNK5clang6driver9ToolChain14SupportsObjCGCEv", "clang::driver::ToolChain::SupportsObjCGC() const"},
    {"_ZNK5clang6driver9ToolChain18UseDwarfDebugFlagsEv", "clang::driver::ToolChain::UseDwarfDebugFlags() const"},
    {"_ZNK5clang6driver9ToolChain17UseSjLjExceptionsEv", "clang::driver::ToolChain::UseSjLjExceptions() const"},
    {"_ZN4llvm8DenseMapIjPN5clang6driver9ToolChainENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PSt4pairIjS4_E", "llvm::DenseMap<unsigned int, clang::driver::ToolChain*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::driver::ToolChain*> >::InsertIntoBucket(unsigned int const&, clang::driver::ToolChain* const&, std::pair<unsigned int, clang::driver::ToolChain*>*)"},
    {"_ZN4llvm8DenseMapIjPN5clang6driver9ToolChainENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, clang::driver::ToolChain*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::driver::ToolChain*> >::grow(unsigned int)"},
    {"_ZN5clang6driver3JobD0Ev", "clang::driver::Job::~Job()"},
    {"_ZN5clang6driver3JobD1Ev", "clang::driver::Job::~Job()"},
    {"_ZN5clang6driver3JobD2Ev", "clang::driver::Job::~Job()"},
    {"_ZN5clang6driver7CommandC1ERKNS0_6ActionERKNS0_4ToolEPKcRKN4llvm11SmallVectorIS9_Lj16EEE", "clang::driver::Command::Command(clang::driver::Action const&, clang::driver::Tool const&, char const*, llvm::SmallVector<char const*, 16u> const&)"},
    {"_ZN5clang6driver7CommandC2ERKNS0_6ActionERKNS0_4ToolEPKcRKN4llvm11SmallVectorIS9_Lj16EEE", "clang::driver::Command::Command(clang::driver::Action const&, clang::driver::Tool const&, char const*, llvm::SmallVector<char const*, 16u> const&)"},
    {"_ZN5clang6driver7JobListC1Ev", "clang::driver::JobList::JobList()"},
    {"_ZN5clang6driver7JobListC2Ev", "clang::driver::JobList::JobList()"},
    {"_ZN5clang6driver7JobListD0Ev", "clang::driver::JobList::~JobList()"},
    {"_ZN5clang6driver7JobListD1Ev", "clang::driver::JobList::~JobList()"},
    {"_ZN5clang6driver7JobListD2Ev", "clang::driver::JobList::~JobList()"},
    {"_ZN5clang6driver3Job10addCommandEPNS0_7CommandE", "clang::driver::Job::addCommand(clang::driver::Command*)"},
    {"_ZN5clang6driver7CommandD1Ev", "clang::driver::Command::~Command()"},
    {"_ZN5clang6driver7CommandD0Ev", "clang::driver::Command::~Command()"},
    {"_ZN4llvm15SmallVectorImplIPKcEaSERKS3_", "llvm::SmallVectorImpl<char const*>::operator=(llvm::SmallVectorImpl<char const*> const&)"},
    {"_ZN5clang6driver12OptSpecifierC1EPKNS0_6OptionE", "clang::driver::OptSpecifier::OptSpecifier(clang::driver::Option const*)"},
    {"_ZN5clang6driver12OptSpecifierC2EPKNS0_6OptionE", "clang::driver::OptSpecifier::OptSpecifier(clang::driver::Option const*)"},
    {"_ZN5clang6driver8OptTableC1EPKNS1_4InfoEj", "clang::driver::OptTable::OptTable(clang::driver::OptTable::Info const*, unsigned int)"},
    {"_ZN5clang6driver8OptTableC2EPKNS1_4InfoEj", "clang::driver::OptTable::OptTable(clang::driver::OptTable::Info const*, unsigned int)"},
    {"_ZN5clang6driver8OptTableD1Ev", "clang::driver::OptTable::~OptTable()"},
    {"_ZN5clang6driver8OptTableD2Ev", "clang::driver::OptTable::~OptTable()"},
    {"_ZNK5clang6driver8OptTable12CreateOptionEj", "clang::driver::OptTable::CreateOption(unsigned int) const"},
    {"_ZNK5clang6driver8OptTable11ParseOneArgERKNS0_7ArgListERj", "clang::driver::OptTable::ParseOneArg(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZNK5clang6driver8OptTable9ParseArgsEPKPKcS5_RjS6_", "clang::driver::OptTable::ParseArgs(char const* const*, char const* const*, unsigned int&, unsigned int&) const"},
    {"_ZNK5clang6driver8OptTable9PrintHelpERN4llvm11raw_ostreamEPKcS6_b", "clang::driver::OptTable::PrintHelp(llvm::raw_ostream&, char const*, char const*, bool) const"},
    {"_ZNSt3mapISsSt6vectorISt4pairISsPKcESaIS4_EESt4lessISsESaIS1_IKSsS6_EEEixERS9_", "std::map<std::string, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > > >::operator[](std::string const&)"},
    {"_ZNSt6vectorISt4pairISsPKcESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<std::string, char const*>*, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >, std::pair<std::string, char const*> const&)"},
    {"_ZNSt6vectorISt4pairISsPKcESaIS3_EEC2ERKS5_", "std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > >::vector(std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsSt6vectorIS0_ISsPKcESaIS5_EEESt10_Select1stIS8_ESt4lessISsESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<std::string, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >, std::_Select1st<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsSt6vectorIS0_ISsPKcESaIS5_EEESt10_Select1stIS8_ESt4lessISsESaIS8_EE9_M_insertEPSt18_Rb_tree_node_baseSG_RKS8_", "std::_Rb_tree<std::string, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >, std::_Select1st<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsSt6vectorIS0_ISsPKcESaIS5_EEESt10_Select1stIS8_ESt4lessISsESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<std::string, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >, std::_Select1st<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > > >::_M_insert_unique(std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsSt6vectorIS0_ISsPKcESaIS5_EEESt10_Select1stIS8_ESt4lessISsESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<std::string, std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >, std::_Select1st<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >*)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIKSsSt6vectorIS1_ISsPKcESaIS6_EEEE7destroyEPS9_", "__gnu_cxx::new_allocator<std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > > >::destroy(std::pair<std::string const, std::vector<std::pair<std::string, char const*>, std::allocator<std::pair<std::string, char const*> > > >*)"},
    {"_ZN5clang6driver6OptionC1ENS1_11OptionClassENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKS1_", "clang::driver::Option::Option(clang::driver::Option::OptionClass, clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver6OptionC2ENS1_11OptionClassENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKS1_", "clang::driver::Option::Option(clang::driver::Option::OptionClass, clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver6OptionD0Ev", "clang::driver::Option::~Option()"},
    {"_ZN5clang6driver6OptionD1Ev", "clang::driver::Option::~Option()"},
    {"_ZN5clang6driver6OptionD2Ev", "clang::driver::Option::~Option()"},
    {"_ZNK5clang6driver6Option4dumpEv", "clang::driver::Option::dump() const"},
    {"_ZNK5clang6driver6Option7matchesENS0_12OptSpecifierE", "clang::driver::Option::matches(clang::driver::OptSpecifier) const"},
    {"_ZN5clang6driver11OptionGroupC1ENS0_12OptSpecifierEPKcPKS1_", "clang::driver::OptionGroup::OptionGroup(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*)"},
    {"_ZN5clang6driver11OptionGroupC2ENS0_12OptSpecifierEPKcPKS1_", "clang::driver::OptionGroup::OptionGroup(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*)"},
    {"_ZNK5clang6driver11OptionGroup6acceptERKNS0_7ArgListERj", "clang::driver::OptionGroup::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver11InputOptionC1ENS0_12OptSpecifierE", "clang::driver::InputOption::InputOption(clang::driver::OptSpecifier)"},
    {"_ZN5clang6driver11InputOptionC2ENS0_12OptSpecifierE", "clang::driver::InputOption::InputOption(clang::driver::OptSpecifier)"},
    {"_ZNK5clang6driver11InputOption6acceptERKNS0_7ArgListERj", "clang::driver::InputOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver13UnknownOptionC1ENS0_12OptSpecifierE", "clang::driver::UnknownOption::UnknownOption(clang::driver::OptSpecifier)"},
    {"_ZN5clang6driver13UnknownOptionC2ENS0_12OptSpecifierE", "clang::driver::UnknownOption::UnknownOption(clang::driver::OptSpecifier)"},
    {"_ZNK5clang6driver13UnknownOption6acceptERKNS0_7ArgListERj", "clang::driver::UnknownOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver10FlagOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::FlagOption::FlagOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver10FlagOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::FlagOption::FlagOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver10FlagOption6acceptERKNS0_7ArgListERj", "clang::driver::FlagOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver12JoinedOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedOption::JoinedOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver12JoinedOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedOption::JoinedOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver12JoinedOption6acceptERKNS0_7ArgListERj", "clang::driver::JoinedOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver17CommaJoinedOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::CommaJoinedOption::CommaJoinedOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver17CommaJoinedOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::CommaJoinedOption::CommaJoinedOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver17CommaJoinedOption6acceptERKNS0_7ArgListERj", "clang::driver::CommaJoinedOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver14SeparateOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::SeparateOption::SeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver14SeparateOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::SeparateOption::SeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver14SeparateOption6acceptERKNS0_7ArgListERj", "clang::driver::SeparateOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver14MultiArgOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionEj", "clang::driver::MultiArgOption::MultiArgOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*, unsigned int)"},
    {"_ZN5clang6driver14MultiArgOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionEj", "clang::driver::MultiArgOption::MultiArgOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*, unsigned int)"},
    {"_ZNK5clang6driver14MultiArgOption6acceptERKNS0_7ArgListERj", "clang::driver::MultiArgOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver22JoinedOrSeparateOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedOrSeparateOption::JoinedOrSeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver22JoinedOrSeparateOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedOrSeparateOption::JoinedOrSeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver22JoinedOrSeparateOption6acceptERKNS0_7ArgListERj", "clang::driver::JoinedOrSeparateOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver23JoinedAndSeparateOptionC1ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedAndSeparateOption::JoinedAndSeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZN5clang6driver23JoinedAndSeparateOptionC2ENS0_12OptSpecifierEPKcPKNS0_11OptionGroupEPKNS0_6OptionE", "clang::driver::JoinedAndSeparateOption::JoinedAndSeparateOption(clang::driver::OptSpecifier, char const*, clang::driver::OptionGroup const*, clang::driver::Option const*)"},
    {"_ZNK5clang6driver23JoinedAndSeparateOption6acceptERKNS0_7ArgListERj", "clang::driver::JoinedAndSeparateOption::accept(clang::driver::ArgList const&, unsigned int&) const"},
    {"_ZN5clang6driver11OptionGroupD1Ev", "clang::driver::OptionGroup::~OptionGroup()"},
    {"_ZN5clang6driver11OptionGroupD0Ev", "clang::driver::OptionGroup::~OptionGroup()"},
    {"_ZN5clang6driver11InputOptionD1Ev", "clang::driver::InputOption::~InputOption()"},
    {"_ZN5clang6driver11InputOptionD0Ev", "clang::driver::InputOption::~InputOption()"},
    {"_ZN5clang6driver13UnknownOptionD1Ev", "clang::driver::UnknownOption::~UnknownOption()"},
    {"_ZN5clang6driver13UnknownOptionD0Ev", "clang::driver::UnknownOption::~UnknownOption()"},
    {"_ZN5clang6driver10FlagOptionD1Ev", "clang::driver::FlagOption::~FlagOption()"},
    {"_ZN5clang6driver10FlagOptionD0Ev", "clang::driver::FlagOption::~FlagOption()"},
    {"_ZN5clang6driver12JoinedOptionD1Ev", "clang::driver::JoinedOption::~JoinedOption()"},
    {"_ZN5clang6driver12JoinedOptionD0Ev", "clang::driver::JoinedOption::~JoinedOption()"},
    {"_ZN5clang6driver17CommaJoinedOptionD1Ev", "clang::driver::CommaJoinedOption::~CommaJoinedOption()"},
    {"_ZN5clang6driver17CommaJoinedOptionD0Ev", "clang::driver::CommaJoinedOption::~CommaJoinedOption()"},
    {"_ZN5clang6driver14SeparateOptionD1Ev", "clang::driver::SeparateOption::~SeparateOption()"},
    {"_ZN5clang6driver14SeparateOptionD0Ev", "clang::driver::SeparateOption::~SeparateOption()"},
    {"_ZN5clang6driver14MultiArgOptionD1Ev", "clang::driver::MultiArgOption::~MultiArgOption()"},
    {"_ZN5clang6driver14MultiArgOptionD0Ev", "clang::driver::MultiArgOption::~MultiArgOption()"},
    {"_ZN5clang6driver22JoinedOrSeparateOptionD1Ev", "clang::driver::JoinedOrSeparateOption::~JoinedOrSeparateOption()"},
    {"_ZN5clang6driver22JoinedOrSeparateOptionD0Ev", "clang::driver::JoinedOrSeparateOption::~JoinedOrSeparateOption()"},
    {"_ZN5clang6driver23JoinedAndSeparateOptionD1Ev", "clang::driver::JoinedAndSeparateOption::~JoinedAndSeparateOption()"},
    {"_ZN5clang6driver23JoinedAndSeparateOptionD0Ev", "clang::driver::JoinedAndSeparateOption::~JoinedAndSeparateOption()"},
    {"_ZN5clang6driver6phases12getPhaseNameENS1_2IDE", "clang::driver::phases::getPhaseName(clang::driver::phases::ID)"},
    {"_ZN5clang6driver4ToolC1EPKcS3_RKNS0_9ToolChainE", "clang::driver::Tool::Tool(char const*, char const*, clang::driver::ToolChain const&)"},
    {"_ZN5clang6driver4ToolC2EPKcS3_RKNS0_9ToolChainE", "clang::driver::Tool::Tool(char const*, char const*, clang::driver::ToolChain const&)"},
    {"_ZN5clang6driver4ToolD0Ev", "clang::driver::Tool::~Tool()"},
    {"_ZN5clang6driver4ToolD1Ev", "clang::driver::Tool::~Tool()"},
    {"_ZN5clang6driver4ToolD2Ev", "clang::driver::Tool::~Tool()"},
    {"_ZN5clang6driver9ToolChainC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::ToolChain::ToolChain(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver9ToolChainC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::ToolChain::ToolChain(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver9ToolChainD0Ev", "clang::driver::ToolChain::~ToolChain()"},
    {"_ZN5clang6driver9ToolChainD1Ev", "clang::driver::ToolChain::~ToolChain()"},
    {"_ZN5clang6driver9ToolChainD2Ev", "clang::driver::ToolChain::~ToolChain()"},
    {"_ZNK5clang6driver9ToolChain9getDriverEv", "clang::driver::ToolChain::getDriver() const"},
    {"_ZNK5clang6driver9ToolChain11GetFilePathEPKc", "clang::driver::ToolChain::GetFilePath(char const*) const"},
    {"_ZNK5clang6driver9ToolChain14GetProgramPathEPKcb", "clang::driver::ToolChain::GetProgramPath(char const*, bool) const"},
    {"_ZNK5clang6driver9ToolChain22LookupTypeForExtensionEPKc", "clang::driver::ToolChain::LookupTypeForExtension(char const*) const"},
    {"_ZNK5clang6driver9ToolChain20HasNativeLLVMSupportEv", "clang::driver::ToolChain::HasNativeLLVMSupport() const"},
    {"_ZNK5clang6driver9ToolChain17ComputeLLVMTripleERKNS0_7ArgListE", "clang::driver::ToolChain::ComputeLLVMTriple(clang::driver::ArgList const&) const"},
    {"_ZNK5clang6driver9ToolChain27ComputeEffectiveClangTripleERKNS0_7ArgListE", "clang::driver::ToolChain::ComputeEffectiveClangTriple(clang::driver::ArgList const&) const"},
    {"_ZNK5clang6driver9ToolChain16GetCXXStdlibTypeERKNS0_7ArgListE", "clang::driver::ToolChain::GetCXXStdlibType(clang::driver::ArgList const&) const"},
    {"_ZNK5clang6driver9ToolChain28AddClangCXXStdlibIncludeArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::ToolChain::AddClangCXXStdlibIncludeArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver9ToolChain19AddCXXStdlibLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::ToolChain::AddCXXStdlibLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver9ToolChain16AddCCKextLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::ToolChain::AddCCKextLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZN5clang6driver10toolchains6DarwinC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Darwin::Darwin(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains6DarwinC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Darwin::Darwin(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains6Darwin22LookupTypeForExtensionEPKc", "clang::driver::toolchains::Darwin::LookupTypeForExtension(char const*) const"},
    {"_ZNK5clang6driver10toolchains6Darwin20HasNativeLLVMSupportEv", "clang::driver::toolchains::Darwin::HasNativeLLVMSupport() const"},
    {"_ZNK5clang6driver10toolchains6Darwin17getDarwinArchNameERKNS0_7ArgListE", "clang::driver::toolchains::Darwin::getDarwinArchName(clang::driver::ArgList const&) const"},
    {"_ZN5clang6driver10toolchains9DarwinGCCC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DarwinGCC::DarwinGCC(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains9DarwinGCCC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DarwinGCC::DarwinGCC(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains6DarwinD0Ev", "clang::driver::toolchains::Darwin::~Darwin()"},
    {"_ZN5clang6driver10toolchains6DarwinD1Ev", "clang::driver::toolchains::Darwin::~Darwin()"},
    {"_ZN5clang6driver10toolchains6DarwinD2Ev", "clang::driver::toolchains::Darwin::~Darwin()"},
    {"_ZNK5clang6driver10toolchains6Darwin27ComputeEffectiveClangTripleERKNS0_7ArgListE", "clang::driver::toolchains::Darwin::ComputeEffectiveClangTriple(clang::driver::ArgList const&) const"},
    {"_ZNK5clang6driver10toolchains6Darwin10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::Darwin::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZNK5clang6driver10toolchains9DarwinGCC21AddLinkSearchPathArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinGCC::AddLinkSearchPathArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver10toolchains9DarwinGCC21AddLinkRuntimeLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinGCC::AddLinkRuntimeLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZN5clang6driver10toolchains11DarwinClangC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DarwinClang::DarwinClang(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains11DarwinClangC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DarwinClang::DarwinClang(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains11DarwinClang21AddLinkSearchPathArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinClang::AddLinkSearchPathArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver10toolchains11DarwinClang21AddLinkRuntimeLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinClang::AddLinkRuntimeLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver10toolchains6Darwin19AddDeploymentTargetERNS0_14DerivedArgListE", "clang::driver::toolchains::Darwin::AddDeploymentTarget(clang::driver::DerivedArgList&) const"},
    {"_ZNK5clang6driver10toolchains11DarwinClang19AddCXXStdlibLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinClang::AddCXXStdlibLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver10toolchains11DarwinClang16AddCCKextLibArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::toolchains::DarwinClang::AddCCKextLibArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver10toolchains6Darwin13TranslateArgsERKNS0_14DerivedArgListEPKc", "clang::driver::toolchains::Darwin::TranslateArgs(clang::driver::DerivedArgList const&, char const*) const"},
    {"_ZNK5clang6driver10toolchains6Darwin21IsUnwindTablesDefaultEv", "clang::driver::toolchains::Darwin::IsUnwindTablesDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin18UseDwarfDebugFlagsEv", "clang::driver::toolchains::Darwin::UseDwarfDebugFlags() const"},
    {"_ZNK5clang6driver10toolchains6Darwin17UseSjLjExceptionsEv", "clang::driver::toolchains::Darwin::UseSjLjExceptions() const"},
    {"_ZNK5clang6driver10toolchains6Darwin25GetDefaultRelocationModelEv", "clang::driver::toolchains::Darwin::GetDefaultRelocationModel() const"},
    {"_ZNK5clang6driver10toolchains6Darwin17GetForcedPicModelEv", "clang::driver::toolchains::Darwin::GetForcedPicModel() const"},
    {"_ZNK5clang6driver10toolchains6Darwin14SupportsObjCGCEv", "clang::driver::toolchains::Darwin::SupportsObjCGC() const"},
    {"_ZNK5clang6driver10toolchains18Darwin_Generic_GCC27ComputeEffectiveClangTripleERKNS0_7ArgListE", "clang::driver::toolchains::Darwin_Generic_GCC::ComputeEffectiveClangTriple(clang::driver::ArgList const&) const"},
    {"_ZN5clang6driver10toolchains11Generic_GCCC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Generic_GCC::Generic_GCC(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains11Generic_GCCC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Generic_GCC::Generic_GCC(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains11Generic_GCCD0Ev", "clang::driver::toolchains::Generic_GCC::~Generic_GCC()"},
    {"_ZN5clang6driver10toolchains11Generic_GCCD1Ev", "clang::driver::toolchains::Generic_GCC::~Generic_GCC()"},
    {"_ZN5clang6driver10toolchains11Generic_GCCD2Ev", "clang::driver::toolchains::Generic_GCC::~Generic_GCC()"},
    {"_ZNK5clang6driver10toolchains11Generic_GCC10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::Generic_GCC::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZNK5clang6driver10toolchains11Generic_GCC21IsUnwindTablesDefaultEv", "clang::driver::toolchains::Generic_GCC::IsUnwindTablesDefault() const"},
    {"_ZNK5clang6driver10toolchains11Generic_GCC25GetDefaultRelocationModelEv", "clang::driver::toolchains::Generic_GCC::GetDefaultRelocationModel() const"},
    {"_ZNK5clang6driver10toolchains11Generic_GCC17GetForcedPicModelEv", "clang::driver::toolchains::Generic_GCC::GetForcedPicModel() const"},
    {"_ZN5clang6driver10toolchains12TCEToolChainC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::TCEToolChain::TCEToolChain(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains12TCEToolChainC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::TCEToolChain::TCEToolChain(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains12TCEToolChainD0Ev", "clang::driver::toolchains::TCEToolChain::~TCEToolChain()"},
    {"_ZN5clang6driver10toolchains12TCEToolChainD1Ev", "clang::driver::toolchains::TCEToolChain::~TCEToolChain()"},
    {"_ZN5clang6driver10toolchains12TCEToolChainD2Ev", "clang::driver::toolchains::TCEToolChain::~TCEToolChain()"},
    {"_ZNK5clang6driver10toolchains12TCEToolChain18IsMathErrnoDefaultEv", "clang::driver::toolchains::TCEToolChain::IsMathErrnoDefault() const"},
    {"_ZNK5clang6driver10toolchains12TCEToolChain21IsUnwindTablesDefaultEv", "clang::driver::toolchains::TCEToolChain::IsUnwindTablesDefault() const"},
    {"_ZNK5clang6driver10toolchains12TCEToolChain25GetDefaultRelocationModelEv", "clang::driver::toolchains::TCEToolChain::GetDefaultRelocationModel() const"},
    {"_ZNK5clang6driver10toolchains12TCEToolChain17GetForcedPicModelEv", "clang::driver::toolchains::TCEToolChain::GetForcedPicModel() const"},
    {"_ZNK5clang6driver10toolchains12TCEToolChain10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::TCEToolChain::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains7OpenBSDC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::OpenBSD::OpenBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains7OpenBSDC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::OpenBSD::OpenBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains7OpenBSD10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::OpenBSD::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains7FreeBSDC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::FreeBSD::FreeBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains7FreeBSDC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::FreeBSD::FreeBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains7FreeBSD10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::FreeBSD::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains6NetBSDC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::NetBSD::NetBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains6NetBSDC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::NetBSD::NetBSD(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains6NetBSD10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::NetBSD::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains5MinixC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Minix::Minix(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains5MinixC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Minix::Minix(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains5Minix10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::Minix::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains8AuroraUXC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::AuroraUX::AuroraUX(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains8AuroraUXC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::AuroraUX::AuroraUX(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains8AuroraUX10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::AuroraUX::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains5LinuxC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Linux::Linux(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains5LinuxC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Linux::Linux(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains5Linux20HasNativeLLVMSupportEv", "clang::driver::toolchains::Linux::HasNativeLLVMSupport() const"},
    {"_ZNK5clang6driver10toolchains5Linux10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::Linux::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains9DragonFlyC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DragonFly::DragonFly(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains9DragonFlyC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::DragonFly::DragonFly(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains9DragonFly10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::DragonFly::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZN5clang6driver10toolchains7WindowsC1ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Windows::Windows(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZN5clang6driver10toolchains7WindowsC2ERKNS0_8HostInfoERKN4llvm6TripleE", "clang::driver::toolchains::Windows::Windows(clang::driver::HostInfo const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver10toolchains7Windows10SelectToolERKNS0_11CompilationERKNS0_9JobActionE", "clang::driver::toolchains::Windows::SelectTool(clang::driver::Compilation const&, clang::driver::JobAction const&) const"},
    {"_ZNK5clang6driver10toolchains7Windows28IsIntegratedAssemblerDefaultEv", "clang::driver::toolchains::Windows::IsIntegratedAssemblerDefault() const"},
    {"_ZNK5clang6driver10toolchains7Windows21IsUnwindTablesDefaultEv", "clang::driver::toolchains::Windows::IsUnwindTablesDefault() const"},
    {"_ZNK5clang6driver10toolchains7Windows25GetDefaultRelocationModelEv", "clang::driver::toolchains::Windows::GetDefaultRelocationModel() const"},
    {"_ZNK5clang6driver10toolchains7Windows17GetForcedPicModelEv", "clang::driver::toolchains::Windows::GetForcedPicModel() const"},
    {"_ZN5clang6driver10toolchains18Darwin_Generic_GCCD1Ev", "clang::driver::toolchains::Darwin_Generic_GCC::~Darwin_Generic_GCC()"},
    {"_ZN5clang6driver10toolchains18Darwin_Generic_GCCD0Ev", "clang::driver::toolchains::Darwin_Generic_GCC::~Darwin_Generic_GCC()"},
    {"_ZNK5clang6driver10toolchains6Darwin15IsBlocksDefaultEv", "clang::driver::toolchains::Darwin::IsBlocksDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin28IsIntegratedAssemblerDefaultEv", "clang::driver::toolchains::Darwin::IsIntegratedAssemblerDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin23IsStrictAliasingDefaultEv", "clang::driver::toolchains::Darwin::IsStrictAliasingDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin35IsObjCDefaultSynthPropertiesDefaultEv", "clang::driver::toolchains::Darwin::IsObjCDefaultSynthPropertiesDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin26IsObjCNonFragileABIDefaultEv", "clang::driver::toolchains::Darwin::IsObjCNonFragileABIDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin27IsObjCLegacyDispatchDefaultEv", "clang::driver::toolchains::Darwin::IsObjCLegacyDispatchDefault() const"},
    {"_ZNK5clang6driver10toolchains6Darwin20UseObjCMixedDispatchEv", "clang::driver::toolchains::Darwin::UseObjCMixedDispatch() const"},
    {"_ZNK5clang6driver10toolchains6Darwin29GetDefaultStackProtectorLevelEv", "clang::driver::toolchains::Darwin::GetDefaultStackProtectorLevel() const"},
    {"_ZN5clang6driver10toolchains9DarwinGCCD1Ev", "clang::driver::toolchains::DarwinGCC::~DarwinGCC()"},
    {"_ZN5clang6driver10toolchains9DarwinGCCD0Ev", "clang::driver::toolchains::DarwinGCC::~DarwinGCC()"},
    {"_ZN5clang6driver10toolchains11DarwinClangD1Ev", "clang::driver::toolchains::DarwinClang::~DarwinClang()"},
    {"_ZN5clang6driver10toolchains11DarwinClangD0Ev", "clang::driver::toolchains::DarwinClang::~DarwinClang()"},
    {"_ZN5clang6driver10toolchains7OpenBSDD1Ev", "clang::driver::toolchains::OpenBSD::~OpenBSD()"},
    {"_ZN5clang6driver10toolchains7OpenBSDD0Ev", "clang::driver::toolchains::OpenBSD::~OpenBSD()"},
    {"_ZNK5clang6driver10toolchains11Generic_ELF28IsIntegratedAssemblerDefaultEv", "clang::driver::toolchains::Generic_ELF::IsIntegratedAssemblerDefault() const"},
    {"_ZN5clang6driver10toolchains7FreeBSDD1Ev", "clang::driver::toolchains::FreeBSD::~FreeBSD()"},
    {"_ZN5clang6driver10toolchains7FreeBSDD0Ev", "clang::driver::toolchains::FreeBSD::~FreeBSD()"},
    {"_ZN5clang6driver10toolchains6NetBSDD1Ev", "clang::driver::toolchains::NetBSD::~NetBSD()"},
    {"_ZN5clang6driver10toolchains6NetBSDD0Ev", "clang::driver::toolchains::NetBSD::~NetBSD()"},
    {"_ZN5clang6driver10toolchains5MinixD1Ev", "clang::driver::toolchains::Minix::~Minix()"},
    {"_ZN5clang6driver10toolchains5MinixD0Ev", "clang::driver::toolchains::Minix::~Minix()"},
    {"_ZN5clang6driver10toolchains8AuroraUXD1Ev", "clang::driver::toolchains::AuroraUX::~AuroraUX()"},
    {"_ZN5clang6driver10toolchains8AuroraUXD0Ev", "clang::driver::toolchains::AuroraUX::~AuroraUX()"},
    {"_ZN5clang6driver10toolchains5LinuxD1Ev", "clang::driver::toolchains::Linux::~Linux()"},
    {"_ZN5clang6driver10toolchains5LinuxD0Ev", "clang::driver::toolchains::Linux::~Linux()"},
    {"_ZN5clang6driver10toolchains9DragonFlyD1Ev", "clang::driver::toolchains::DragonFly::~DragonFly()"},
    {"_ZN5clang6driver10toolchains9DragonFlyD0Ev", "clang::driver::toolchains::DragonFly::~DragonFly()"},
    {"_ZN5clang6driver10toolchains7WindowsD1Ev", "clang::driver::toolchains::Windows::~Windows()"},
    {"_ZN5clang6driver10toolchains7WindowsD0Ev", "clang::driver::toolchains::Windows::~Windows()"},
    {"_ZN4llvm8DenseMapIjPN5clang6driver4ToolENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PSt4pairIjS4_E", "llvm::DenseMap<unsigned int, clang::driver::Tool*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::driver::Tool*> >::InsertIntoBucket(unsigned int const&, clang::driver::Tool* const&, std::pair<unsigned int, clang::driver::Tool*>*)"},
    {"_ZN4llvm8DenseMapIjPN5clang6driver4ToolENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, clang::driver::Tool*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::driver::Tool*> >::grow(unsigned int)"},
    {"_ZN5clang6driver10toolchains11Generic_ELFD1Ev", "clang::driver::toolchains::Generic_ELF::~Generic_ELF()"},
    {"_ZN5clang6driver10toolchains11Generic_ELFD0Ev", "clang::driver::toolchains::Generic_ELF::~Generic_ELF()"},
    {"_ZN5clang6driver10toolchains5LinuxD2Ev", "clang::driver::toolchains::Linux::~Linux()"},
    {"_ZNK5clang6driver4Tool22hasIntegratedAssemblerEv", "clang::driver::Tool::hasIntegratedAssembler() const"},
    {"_ZNK5clang6driver5tools12visualstudio4Link16hasIntegratedCPPEv", "clang::driver::tools::visualstudio::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver4Tool18hasGoodDiagnosticsEv", "clang::driver::Tool::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools9dragonfly4Link16hasIntegratedCPPEv", "clang::driver::tools::dragonfly::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools9dragonfly8Assemble16hasIntegratedCPPEv", "clang::driver::tools::dragonfly::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools10linuxtools4Link16hasIntegratedCPPEv", "clang::driver::tools::linuxtools::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools10linuxtools8Assemble16hasIntegratedCPPEv", "clang::driver::tools::linuxtools::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools8auroraux4Link16hasIntegratedCPPEv", "clang::driver::tools::auroraux::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools8auroraux8Assemble16hasIntegratedCPPEv", "clang::driver::tools::auroraux::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools5minix4Link16hasIntegratedCPPEv", "clang::driver::tools::minix::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools5minix8Assemble16hasIntegratedCPPEv", "clang::driver::tools::minix::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools6netbsd4Link16hasIntegratedCPPEv", "clang::driver::tools::netbsd::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools6netbsd8Assemble16hasIntegratedCPPEv", "clang::driver::tools::netbsd::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7freebsd4Link16hasIntegratedCPPEv", "clang::driver::tools::freebsd::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7freebsd8Assemble16hasIntegratedCPPEv", "clang::driver::tools::freebsd::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7openbsd4Link16hasIntegratedCPPEv", "clang::driver::tools::openbsd::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7openbsd8Assemble16hasIntegratedCPPEv", "clang::driver::tools::openbsd::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools3gcc4Link16hasIntegratedCPPEv", "clang::driver::tools::gcc::Link::hasIntegratedCPP() const"},
    {"_ZN5clang6driver5tools3gcc6CommonD1Ev", "clang::driver::tools::gcc::Common::~Common()"},
    {"_ZN5clang6driver5tools3gcc6CommonD0Ev", "clang::driver::tools::gcc::Common::~Common()"},
    {"_ZNK5clang6driver5tools3gcc8Assemble16hasIntegratedCPPEv", "clang::driver::tools::gcc::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools3gcc7Compile16hasIntegratedCPPEv", "clang::driver::tools::gcc::Compile::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools3gcc7Compile18hasGoodDiagnosticsEv", "clang::driver::tools::gcc::Compile::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools3gcc10Precompile16hasIntegratedCPPEv", "clang::driver::tools::gcc::Precompile::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools3gcc10Precompile18hasGoodDiagnosticsEv", "clang::driver::tools::gcc::Precompile::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools3gcc10Preprocess16hasIntegratedCPPEv", "clang::driver::tools::gcc::Preprocess::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools3gcc10Preprocess18hasGoodDiagnosticsEv", "clang::driver::tools::gcc::Preprocess::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools6darwin8Dsymutil16hasIntegratedCPPEv", "clang::driver::tools::darwin::Dsymutil::hasIntegratedCPP() const"},
    {"_ZN5clang6driver5tools6darwin10DarwinToolD1Ev", "clang::driver::tools::darwin::DarwinTool::~DarwinTool()"},
    {"_ZN5clang6driver5tools6darwin10DarwinToolD0Ev", "clang::driver::tools::darwin::DarwinTool::~DarwinTool()"},
    {"_ZNK5clang6driver5tools6darwin4Lipo16hasIntegratedCPPEv", "clang::driver::tools::darwin::Lipo::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools6darwin4Link16hasIntegratedCPPEv", "clang::driver::tools::darwin::Link::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools6darwin8Assemble16hasIntegratedCPPEv", "clang::driver::tools::darwin::Assemble::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7ClangAs22hasIntegratedAssemblerEv", "clang::driver::tools::ClangAs::hasIntegratedAssembler() const"},
    {"_ZNK5clang6driver5tools7ClangAs16hasIntegratedCPPEv", "clang::driver::tools::ClangAs::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools7ClangAs18hasGoodDiagnosticsEv", "clang::driver::tools::ClangAs::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools6darwin3CC116hasIntegratedCPPEv", "clang::driver::tools::darwin::CC1::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools6darwin3CC118hasGoodDiagnosticsEv", "clang::driver::tools::darwin::CC1::hasGoodDiagnostics() const"},
    {"_ZN5clang6driver5tools6darwin3CC1D1Ev", "clang::driver::tools::darwin::CC1::~CC1()"},
    {"_ZN5clang6driver5tools6darwin3CC1D0Ev", "clang::driver::tools::darwin::CC1::~CC1()"},
    {"_ZNK5clang6driver5tools5Clang22hasIntegratedAssemblerEv", "clang::driver::tools::Clang::hasIntegratedAssembler() const"},
    {"_ZNK5clang6driver5tools5Clang16hasIntegratedCPPEv", "clang::driver::tools::Clang::hasIntegratedCPP() const"},
    {"_ZNK5clang6driver5tools5Clang18hasGoodDiagnosticsEv", "clang::driver::tools::Clang::hasGoodDiagnostics() const"},
    {"_ZNK5clang6driver5tools5Clang23AddPreprocessingOptionsERKNS0_6DriverERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEERKNS0_9InputInfoERKNSA_ISF_Lj4EEE", "clang::driver::tools::Clang::AddPreprocessingOptions(clang::driver::Driver const&, clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&) const"},
    {"_Z25CheckPreprocessingOptionsRKN5clang6driver6DriverERKNS0_7ArgListE", "CheckPreprocessingOptions(clang::driver::Driver const&, clang::driver::ArgList const&)"},
    {"_ZN5clang6driver5tools6darwin3CC121getDependencyFileNameERKNS0_7ArgListERKN4llvm11SmallVectorINS0_9InputInfoELj4EEE", "clang::driver::tools::darwin::CC1::getDependencyFileName(clang::driver::ArgList const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&)"},
    {"_Z11QuoteTargetN4llvm9StringRefERNS_15SmallVectorImplIcEE", "QuoteTarget(llvm::StringRef, llvm::SmallVectorImpl<char>&)"},
    {"_ZNK5clang6driver5tools5Clang16AddARMTargetArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::Clang::AddARMTargetArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_Z15getARMTargetCPURKN5clang6driver7ArgListERKN4llvm6TripleE", "getARMTargetCPU(clang::driver::ArgList const&, llvm::Triple const&)"},
    {"_ZNK5clang6driver5tools5Clang17AddMIPSTargetArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::Clang::AddMIPSTargetArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools5Clang18AddSparcTargetArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::Clang::AddSparcTargetArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools5Clang16AddX86TargetArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::Clang::AddX86TargetArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools5Clang12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS8_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::Clang::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZN5clang6driver5tools6darwin3CC116getBaseInputNameERKNS0_7ArgListERKN4llvm11SmallVectorINS0_9InputInfoELj4EEE", "clang::driver::tools::darwin::CC1::getBaseInputName(clang::driver::ArgList const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&)"},
    {"_Z26CheckCodeGenerationOptionsRKN5clang6driver6DriverERKNS0_7ArgListE", "CheckCodeGenerationOptions(clang::driver::Driver const&, clang::driver::ArgList const&)"},
    {"_ZNK5clang6driver5tools7ClangAs12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS8_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::ClangAs::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools3gcc6Common12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::gcc::Common::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools3gcc10Preprocess19RenderExtraToolArgsERKNS0_9JobActionERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::gcc::Preprocess::RenderExtraToolArgs(clang::driver::JobAction const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools3gcc10Precompile19RenderExtraToolArgsERKNS0_9JobActionERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::gcc::Precompile::RenderExtraToolArgs(clang::driver::JobAction const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools3gcc7Compile19RenderExtraToolArgsERKNS0_9JobActionERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::gcc::Compile::RenderExtraToolArgs(clang::driver::JobAction const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools3gcc8Assemble19RenderExtraToolArgsERKNS0_9JobActionERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::gcc::Assemble::RenderExtraToolArgs(clang::driver::JobAction const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools3gcc4Link19RenderExtraToolArgsERKNS0_9JobActionERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::gcc::Link::RenderExtraToolArgs(clang::driver::JobAction const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools6darwin3CC110getCC1NameENS0_5types2IDE", "clang::driver::tools::darwin::CC1::getCC1Name(clang::driver::types::ID) const"},
    {"_ZN5clang6driver5tools6darwin3CC116getBaseInputStemERKNS0_7ArgListERKN4llvm11SmallVectorINS0_9InputInfoELj4EEE", "clang::driver::tools::darwin::CC1::getBaseInputStem(clang::driver::ArgList const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&)"},
    {"_ZNK5clang6driver5tools6darwin3CC110AddCC1ArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::darwin::CC1::AddCC1Args(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools6darwin3CC117AddCC1OptionsArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEERKNS8_INS0_9InputInfoELj4EEERKSB_", "clang::driver::tools::darwin::CC1::AddCC1OptionsArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, llvm::SmallVector<char const*, 16u> const&) const"},
    {"_ZNK5clang6driver5tools6darwin3CC117AddCPPOptionsArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEERKNS8_INS0_9InputInfoELj4EEERKSB_", "clang::driver::tools::darwin::CC1::AddCPPOptionsArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, llvm::SmallVector<char const*, 16u> const&) const"},
    {"_ZNK5clang6driver5tools6darwin3CC123AddCPPUniqueOptionsArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEERKNS8_INS0_9InputInfoELj4EEE", "clang::driver::tools::darwin::CC1::AddCPPUniqueOptionsArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&) const"},
    {"_ZNK5clang6driver5tools6darwin3CC110AddCPPArgsERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::darwin::CC1::AddCPPArgs(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools6darwin10Preprocess12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Preprocess::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6darwin7Compile12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Compile::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6darwin8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6darwin10DarwinTool13AddDarwinArchERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::darwin::DarwinTool::AddDarwinArch(clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools6darwin4Link11AddLinkArgsERNS0_11CompilationERKNS0_7ArgListERN4llvm11SmallVectorIPKcLj16EEE", "clang::driver::tools::darwin::Link::AddLinkArgs(clang::driver::Compilation&, clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&) const"},
    {"_ZNK5clang6driver5tools6darwin4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_Z15AddLinkerInputsRKN5clang6driver9ToolChainERKN4llvm11SmallVectorINS0_9InputInfoELj4EEERKNS0_7ArgListERNS5_IPKcLj16EEE", "AddLinkerInputs(clang::driver::ToolChain const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, llvm::SmallVector<char const*, 16u>&)"},
    {"_ZNK5clang6driver5tools6darwin4Lipo12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Lipo::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6darwin8Dsymutil12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::darwin::Dsymutil::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools8auroraux8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::auroraux::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools8auroraux4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::auroraux::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools7openbsd8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::openbsd::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools7openbsd4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::openbsd::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools7freebsd8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::freebsd::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools7freebsd4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::freebsd::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6netbsd8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::netbsd::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools6netbsd4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::netbsd::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools10linuxtools8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::linuxtools::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools10linuxtools4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::linuxtools::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools5minix8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::minix::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools5minix4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::minix::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools9dragonfly8Assemble12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::dragonfly::Assemble::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools9dragonfly4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::dragonfly::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZNK5clang6driver5tools12visualstudio4Link12ConstructJobERNS0_11CompilationERKNS0_9JobActionERKNS0_9InputInfoERKN4llvm11SmallVectorIS9_Lj4EEERKNS0_7ArgListEPKc", "clang::driver::tools::visualstudio::Link::ConstructJob(clang::driver::Compilation&, clang::driver::JobAction const&, clang::driver::InputInfo const&, llvm::SmallVector<clang::driver::InputInfo, 4u> const&, clang::driver::ArgList const&, char const*) const"},
    {"_ZN5clang6driver5tools5ClangD1Ev", "clang::driver::tools::Clang::~Clang()"},
    {"_ZN5clang6driver5tools5ClangD0Ev", "clang::driver::tools::Clang::~Clang()"},
    {"_ZN5clang6driver5tools7ClangAsD1Ev", "clang::driver::tools::ClangAs::~ClangAs()"},
    {"_ZN5clang6driver5tools7ClangAsD0Ev", "clang::driver::tools::ClangAs::~ClangAs()"},
    {"_ZN5clang6driver5tools3gcc10PreprocessD1Ev", "clang::driver::tools::gcc::Preprocess::~Preprocess()"},
    {"_ZN5clang6driver5tools3gcc10PreprocessD0Ev", "clang::driver::tools::gcc::Preprocess::~Preprocess()"},
    {"_ZN5clang6driver5tools3gcc10PrecompileD1Ev", "clang::driver::tools::gcc::Precompile::~Precompile()"},
    {"_ZN5clang6driver5tools3gcc10PrecompileD0Ev", "clang::driver::tools::gcc::Precompile::~Precompile()"},
    {"_ZN5clang6driver5tools3gcc7CompileD1Ev", "clang::driver::tools::gcc::Compile::~Compile()"},
    {"_ZN5clang6driver5tools3gcc7CompileD0Ev", "clang::driver::tools::gcc::Compile::~Compile()"},
    {"_ZN5clang6driver5tools3gcc8AssembleD1Ev", "clang::driver::tools::gcc::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools3gcc8AssembleD0Ev", "clang::driver::tools::gcc::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools3gcc4LinkD1Ev", "clang::driver::tools::gcc::Link::~Link()"},
    {"_ZN5clang6driver5tools3gcc4LinkD0Ev", "clang::driver::tools::gcc::Link::~Link()"},
    {"_ZN5clang6driver5tools6darwin10PreprocessD1Ev", "clang::driver::tools::darwin::Preprocess::~Preprocess()"},
    {"_ZN5clang6driver5tools6darwin10PreprocessD0Ev", "clang::driver::tools::darwin::Preprocess::~Preprocess()"},
    {"_ZN5clang6driver5tools6darwin7CompileD1Ev", "clang::driver::tools::darwin::Compile::~Compile()"},
    {"_ZN5clang6driver5tools6darwin7CompileD0Ev", "clang::driver::tools::darwin::Compile::~Compile()"},
    {"_ZN5clang6driver5tools6darwin8AssembleD1Ev", "clang::driver::tools::darwin::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools6darwin8AssembleD0Ev", "clang::driver::tools::darwin::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools6darwin4LinkD1Ev", "clang::driver::tools::darwin::Link::~Link()"},
    {"_ZN5clang6driver5tools6darwin4LinkD0Ev", "clang::driver::tools::darwin::Link::~Link()"},
    {"_ZN5clang6driver5tools6darwin4LipoD1Ev", "clang::driver::tools::darwin::Lipo::~Lipo()"},
    {"_ZN5clang6driver5tools6darwin4LipoD0Ev", "clang::driver::tools::darwin::Lipo::~Lipo()"},
    {"_ZN5clang6driver5tools6darwin8DsymutilD1Ev", "clang::driver::tools::darwin::Dsymutil::~Dsymutil()"},
    {"_ZN5clang6driver5tools6darwin8DsymutilD0Ev", "clang::driver::tools::darwin::Dsymutil::~Dsymutil()"},
    {"_ZN5clang6driver5tools7openbsd8AssembleD1Ev", "clang::driver::tools::openbsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools7openbsd8AssembleD0Ev", "clang::driver::tools::openbsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools7openbsd4LinkD1Ev", "clang::driver::tools::openbsd::Link::~Link()"},
    {"_ZN5clang6driver5tools7openbsd4LinkD0Ev", "clang::driver::tools::openbsd::Link::~Link()"},
    {"_ZN5clang6driver5tools7freebsd8AssembleD1Ev", "clang::driver::tools::freebsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools7freebsd8AssembleD0Ev", "clang::driver::tools::freebsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools7freebsd4LinkD1Ev", "clang::driver::tools::freebsd::Link::~Link()"},
    {"_ZN5clang6driver5tools7freebsd4LinkD0Ev", "clang::driver::tools::freebsd::Link::~Link()"},
    {"_ZN5clang6driver5tools6netbsd8AssembleD1Ev", "clang::driver::tools::netbsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools6netbsd8AssembleD0Ev", "clang::driver::tools::netbsd::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools6netbsd4LinkD1Ev", "clang::driver::tools::netbsd::Link::~Link()"},
    {"_ZN5clang6driver5tools6netbsd4LinkD0Ev", "clang::driver::tools::netbsd::Link::~Link()"},
    {"_ZN5clang6driver5tools10linuxtools8AssembleD1Ev", "clang::driver::tools::linuxtools::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools10linuxtools8AssembleD0Ev", "clang::driver::tools::linuxtools::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools10linuxtools4LinkD1Ev", "clang::driver::tools::linuxtools::Link::~Link()"},
    {"_ZN5clang6driver5tools10linuxtools4LinkD0Ev", "clang::driver::tools::linuxtools::Link::~Link()"},
    {"_ZN5clang6driver5tools5minix8AssembleD1Ev", "clang::driver::tools::minix::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools5minix8AssembleD0Ev", "clang::driver::tools::minix::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools5minix4LinkD1Ev", "clang::driver::tools::minix::Link::~Link()"},
    {"_ZN5clang6driver5tools5minix4LinkD0Ev", "clang::driver::tools::minix::Link::~Link()"},
    {"_ZN5clang6driver5tools8auroraux8AssembleD1Ev", "clang::driver::tools::auroraux::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools8auroraux8AssembleD0Ev", "clang::driver::tools::auroraux::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools8auroraux4LinkD1Ev", "clang::driver::tools::auroraux::Link::~Link()"},
    {"_ZN5clang6driver5tools8auroraux4LinkD0Ev", "clang::driver::tools::auroraux::Link::~Link()"},
    {"_ZN5clang6driver5tools9dragonfly8AssembleD1Ev", "clang::driver::tools::dragonfly::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools9dragonfly8AssembleD0Ev", "clang::driver::tools::dragonfly::Assemble::~Assemble()"},
    {"_ZN5clang6driver5tools9dragonfly4LinkD1Ev", "clang::driver::tools::dragonfly::Link::~Link()"},
    {"_ZN5clang6driver5tools9dragonfly4LinkD0Ev", "clang::driver::tools::dragonfly::Link::~Link()"},
    {"_ZN5clang6driver5tools12visualstudio4LinkD1Ev", "clang::driver::tools::visualstudio::Link::~Link()"},
    {"_ZN5clang6driver5tools12visualstudio4LinkD0Ev", "clang::driver::tools::visualstudio::Link::~Link()"},
    {"_ZN4llvm15SmallVectorImplISsEaSERKS1_", "llvm::SmallVectorImpl<std::string>::operator=(llvm::SmallVectorImpl<std::string> const&)"},
    {"_ZN5clang6driver5types11getTypeNameENS1_2IDE", "clang::driver::types::getTypeName(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types19getPreprocessedTypeENS1_2IDE", "clang::driver::types::getPreprocessedType(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types17getTypeTempSuffixENS1_2IDE", "clang::driver::types::getTypeTempSuffix(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types16onlyAssembleTypeENS1_2IDE", "clang::driver::types::onlyAssembleType(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types18onlyPrecompileTypeENS1_2IDE", "clang::driver::types::onlyPrecompileType(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types22canTypeBeUserSpecifiedENS1_2IDE", "clang::driver::types::canTypeBeUserSpecified(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types19appendSuffixForTypeENS1_2IDE", "clang::driver::types::appendSuffixForType(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types11canLipoTypeENS1_2IDE", "clang::driver::types::canLipoType(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types17isAcceptedByClangENS1_2IDE", "clang::driver::types::isAcceptedByClang(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types21isOnlyAcceptedByClangENS1_2IDE", "clang::driver::types::isOnlyAcceptedByClang(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types6isObjCENS1_2IDE", "clang::driver::types::isObjC(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types5isCXXENS1_2IDE", "clang::driver::types::isCXX(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types22lookupTypeForExtensionEPKc", "clang::driver::types::lookupTypeForExtension(char const*)"},
    {"_ZN5clang6driver5types26lookupTypeForTypeSpecifierEPKc", "clang::driver::types::lookupTypeForTypeSpecifier(char const*)"},
    {"_ZN5clang6driver5types23getNumCompilationPhasesENS1_2IDE", "clang::driver::types::getNumCompilationPhases(clang::driver::types::ID)"},
    {"_ZN5clang6driver5types19getCompilationPhaseENS1_2IDEj", "clang::driver::types::getCompilationPhase(clang::driver::types::ID, unsigned int)"},
    {"_ZN5clang6driver5types21lookupCXXTypeForCTypeENS1_2IDE", "clang::driver::types::lookupCXXTypeForCType(clang::driver::types::ID)"},
    {"_ZN5clang26ASTDeserializationListenerD0Ev", "clang::ASTDeserializationListener::~ASTDeserializationListener()"},
    {"_ZN5clang26ASTDeserializationListenerD1Ev", "clang::ASTDeserializationListener::~ASTDeserializationListener()"},
    {"_ZN5clang26ASTDeserializationListenerD2Ev", "clang::ASTDeserializationListener::~ASTDeserializationListener()"},
    {"_ZN5clang13serialization18TypeIdxFromBuiltinEPKNS_11BuiltinTypeE", "clang::serialization::TypeIdxFromBuiltin(clang::BuiltinType const*)"},
    {"_ZN5clang13serialization11ComputeHashENS_8SelectorE", "clang::serialization::ComputeHash(clang::Selector)"},
    {"_ZN5clang17ASTReaderListenerD0Ev", "clang::ASTReaderListener::~ASTReaderListener()"},
    {"_ZN5clang17ASTReaderListenerD1Ev", "clang::ASTReaderListener::~ASTReaderListener()"},
    {"_ZN5clang17ASTReaderListenerD2Ev", "clang::ASTReaderListener::~ASTReaderListener()"},
    {"_ZN5clang12PCHValidator19ReadLanguageOptionsERKNS_11LangOptionsE", "clang::PCHValidator::ReadLanguageOptions(clang::LangOptions const&)"},
    {"_ZN5clang9ASTReader4DiagEj", "clang::ASTReader::Diag(unsigned int)"},
    {"_ZN5clang12PCHValidator16ReadTargetTripleEN4llvm9StringRefE", "clang::PCHValidator::ReadTargetTriple(llvm::StringRef)"},
    {"_ZN5clang12PCHValidator20ReadPredefinesBufferERKN4llvm11SmallVectorINS_18PCHPredefinesBlockELj2EEENS1_9StringRefERSs", "clang::PCHValidator::ReadPredefinesBuffer(llvm::SmallVector<clang::PCHPredefinesBlock, 2u> const&, llvm::StringRef, std::string&)"},
    {"_ZN5clang9ASTReader4DiagENS_14SourceLocationEj", "clang::ASTReader::Diag(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang12PCHValidator18ReadHeaderFileInfoERKNS_14HeaderFileInfoEj", "clang::PCHValidator::ReadHeaderFileInfo(clang::HeaderFileInfo const&, unsigned int)"},
    {"_ZN5clang12PCHValidator11ReadCounterEj", "clang::PCHValidator::ReadCounter(unsigned int)"},
    {"_ZN5clang9ASTReader26setDeserializationListenerEPNS_26ASTDeserializationListenerE", "clang::ASTReader::setDeserializationListener(clang::ASTDeserializationListener*)"},
    {"_ZN5clang9ASTReader22ReadDeclContextStorageERN4llvm15BitstreamCursorERKSt4pairIyyERNS0_15DeclContextInfoE", "clang::ASTReader::ReadDeclContextStorage(llvm::BitstreamCursor&, std::pair<unsigned long long, unsigned long long> const&, clang::ASTReader::DeclContextInfo&)"},
    {"_ZN5clang9ASTReader5ErrorEPKc", "clang::ASTReader::Error(char const*)"},
    {"_ZN5clang9ASTReader22CheckPredefinesBuffersEv", "clang::ASTReader::CheckPredefinesBuffers()"},
    {"_ZN5clang9ASTReader14ParseLineTableERNS0_11PerFileDataERN4llvm15SmallVectorImplIyEE", "clang::ASTReader::ParseLineTable(clang::ASTReader::PerFileData&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTReader28MaybeAddSystemRootToFilenameERSs", "clang::ASTReader::MaybeAddSystemRootToFilename(std::string&)"},
    {"_ZN5clang9ASTReader22ReadSourceManagerBlockERNS0_11PerFileDataE", "clang::ASTReader::ReadSourceManagerBlock(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang9ASTReader15SLocCursorForIDEj", "clang::ASTReader::SLocCursorForID(unsigned int)"},
    {"_ZN5clang9ASTReader19ReadSLocEntryRecordEj", "clang::ASTReader::ReadSLocEntryRecord(unsigned int)"},
    {"_ZN5clang9ASTReader16ReadBlockAbbrevsERN4llvm15BitstreamCursorEj", "clang::ASTReader::ReadBlockAbbrevs(llvm::BitstreamCursor&, unsigned int)"},
    {"_ZN5clang9ASTReader15ReadMacroRecordERNS0_11PerFileDataEy", "clang::ASTReader::ReadMacroRecord(clang::ASTReader::PerFileData&, unsigned long long)"},
    {"_ZN5clang9ASTReader20DecodeIdentifierInfoEj", "clang::ASTReader::DecodeIdentifierInfo(unsigned int)"},
    {"_ZN5clang9ASTReader18getMacroDefinitionEj", "clang::ASTReader::getMacroDefinition(unsigned int)"},
    {"_ZN5clang9ASTReader22LoadPreprocessedEntityERNS0_11PerFileDataE", "clang::ASTReader::LoadPreprocessedEntity(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang9ASTReader20SetIdentifierIsMacroEPNS_14IdentifierInfoERNS0_11PerFileDataEy", "clang::ASTReader::SetIdentifierIsMacro(clang::IdentifierInfo*, clang::ASTReader::PerFileData&, unsigned long long)"},
    {"_ZN5clang9ASTReader17ReadDefinedMacrosEv", "clang::ASTReader::ReadDefinedMacros()"},
    {"_ZN5clang9ASTReader19LoadMacroDefinitionEN4llvm16DenseMapIteratorIPNS_14IdentifierInfoEyNS1_12DenseMapInfoIS4_EENS5_IyEELb0EEE", "clang::ASTReader::LoadMacroDefinition(llvm::DenseMapIterator<clang::IdentifierInfo*, unsigned long long, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<unsigned long long>, false>)"},
    {"_ZN5clang9ASTReader19LoadMacroDefinitionEPNS_14IdentifierInfoE", "clang::ASTReader::LoadMacroDefinition(clang::IdentifierInfo*)"},
    {"_ZN5clang9ASTReader12ReadASTBlockERNS0_11PerFileDataE", "clang::ASTReader::ReadASTBlock(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang9ASTReader11ReadASTCoreEN4llvm9StringRefENS0_11ASTFileTypeE", "clang::ASTReader::ReadASTCore(llvm::StringRef, clang::ASTReader::ASTFileType)"},
    {"_ZN5clang9ASTReader20ParseLanguageOptionsERKN4llvm15SmallVectorImplIyEE", "clang::ASTReader::ParseLanguageOptions(llvm::SmallVectorImpl<unsigned long long> const&)"},
    {"_ZN5clang9ASTReader7ReadASTERKSsNS0_11ASTFileTypeE", "clang::ASTReader::ReadAST(std::string const&, clang::ASTReader::ASTFileType)"},
    {"_ZN5clang9ASTReader17InitializeContextERNS_10ASTContextE", "clang::ASTReader::InitializeContext(clang::ASTContext&)"},
    {"_ZN5clang9ASTReader11PerFileDataC1ENS0_11ASTFileTypeE", "clang::ASTReader::PerFileData::PerFileData(clang::ASTReader::ASTFileType)"},
    {"_ZN5clang9ASTReader15setPreprocessorERNS_12PreprocessorE", "clang::ASTReader::setPreprocessor(clang::Preprocessor&)"},
    {"_ZN5clang9ASTReader22GetTranslationUnitDeclEv", "clang::ASTReader::GetTranslationUnitDecl()"},
    {"_ZN5clang9ASTReader7GetTypeEj", "clang::ASTReader::GetType(unsigned int)"},
    {"_ZN5clang9ASTReader28ReadPragmaDiagnosticMappingsERNS_10DiagnosticE", "clang::ASTReader::ReadPragmaDiagnosticMappings(clang::Diagnostic&)"},
    {"_ZN5clang9ASTReader7GetDeclEj", "clang::ASTReader::GetDecl(unsigned int)"},
    {"_ZN5clang9ASTReader21getOriginalSourceFileERKSsRNS_11FileManagerERNS_10DiagnosticE", "clang::ASTReader::getOriginalSourceFile(std::string const&, clang::FileManager&, clang::Diagnostic&)"},
    {"_ZThn8_N5clang9ASTReader24ReadPreprocessedEntitiesEv", "non-virtual thunk to clang::ASTReader::ReadPreprocessedEntities()"},
    {"_ZN5clang9ASTReader24ReadPreprocessedEntitiesEv", "clang::ASTReader::ReadPreprocessedEntities()"},
    {"_ZThn8_N5clang9ASTReader22ReadPreprocessedEntityEy", "non-virtual thunk to clang::ASTReader::ReadPreprocessedEntity(unsigned long long)"},
    {"_ZN5clang9ASTReader22ReadPreprocessedEntityEy", "clang::ASTReader::ReadPreprocessedEntity(unsigned long long)"},
    {"_ZN5clang9ASTReader18TypeCursorForIndexEj", "clang::ASTReader::TypeCursorForIndex(unsigned int)"},
    {"_ZN5clang9ASTReader14ReadTypeRecordEj", "clang::ASTReader::ReadTypeRecord(unsigned int)"},
    {"_ZN5clang9ASTReader9ReadAPIntERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadAPInt(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader23ReadNestedNameSpecifierERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadNestedNameSpecifier(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader20ReadTemplateArgumentERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadTemplateArgument(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader15ReadSourceRangeERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadSourceRange(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader16ReadTemplateNameERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadTemplateName(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader24ReadTemplateArgumentListERN4llvm11SmallVectorINS_16TemplateArgumentELj8EEERNS0_11PerFileDataERKNS2_IyLj64EEERj", "clang::ASTReader::ReadTemplateArgumentList(llvm::SmallVector<clang::TemplateArgument, 8u>&, clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13TypeLocReader21VisitQualifiedTypeLocENS_16QualifiedTypeLocE", "clang::TypeLocReader::VisitQualifiedTypeLoc(clang::QualifiedTypeLoc)"},
    {"_ZN5clang13TypeLocReader19VisitBuiltinTypeLocENS_14BuiltinTypeLocE", "clang::TypeLocReader::VisitBuiltinTypeLoc(clang::BuiltinTypeLoc)"},
    {"_ZN5clang13TypeLocReader19VisitComplexTypeLocENS_14ComplexTypeLocE", "clang::TypeLocReader::VisitComplexTypeLoc(clang::ComplexTypeLoc)"},
    {"_ZN5clang13TypeLocReader19VisitPointerTypeLocENS_14PointerTypeLocE", "clang::TypeLocReader::VisitPointerTypeLoc(clang::PointerTypeLoc)"},
    {"_ZN5clang13TypeLocReader24VisitBlockPointerTypeLocENS_19BlockPointerTypeLocE", "clang::TypeLocReader::VisitBlockPointerTypeLoc(clang::BlockPointerTypeLoc)"},
    {"_ZN5clang13TypeLocReader27VisitLValueReferenceTypeLocENS_22LValueReferenceTypeLocE", "clang::TypeLocReader::VisitLValueReferenceTypeLoc(clang::LValueReferenceTypeLoc)"},
    {"_ZN5clang13TypeLocReader27VisitRValueReferenceTypeLocENS_22RValueReferenceTypeLocE", "clang::TypeLocReader::VisitRValueReferenceTypeLoc(clang::RValueReferenceTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitMemberPointerTypeLocENS_20MemberPointerTypeLocE", "clang::TypeLocReader::VisitMemberPointerTypeLoc(clang::MemberPointerTypeLoc)"},
    {"_ZN5clang13TypeLocReader17VisitArrayTypeLocENS_12ArrayTypeLocE", "clang::TypeLocReader::VisitArrayTypeLoc(clang::ArrayTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitConstantArrayTypeLocENS_20ConstantArrayTypeLocE", "clang::TypeLocReader::VisitConstantArrayTypeLoc(clang::ConstantArrayTypeLoc)"},
    {"_ZN5clang13TypeLocReader27VisitIncompleteArrayTypeLocENS_22IncompleteArrayTypeLocE", "clang::TypeLocReader::VisitIncompleteArrayTypeLoc(clang::IncompleteArrayTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitVariableArrayTypeLocENS_20VariableArrayTypeLocE", "clang::TypeLocReader::VisitVariableArrayTypeLoc(clang::VariableArrayTypeLoc)"},
    {"_ZN5clang13TypeLocReader31VisitDependentSizedArrayTypeLocENS_26DependentSizedArrayTypeLocE", "clang::TypeLocReader::VisitDependentSizedArrayTypeLoc(clang::DependentSizedArrayTypeLoc)"},
    {"_ZN5clang13TypeLocReader35VisitDependentSizedExtVectorTypeLocENS_30DependentSizedExtVectorTypeLocE", "clang::TypeLocReader::VisitDependentSizedExtVectorTypeLoc(clang::DependentSizedExtVectorTypeLoc)"},
    {"_ZN5clang13TypeLocReader18VisitVectorTypeLocENS_13VectorTypeLocE", "clang::TypeLocReader::VisitVectorTypeLoc(clang::VectorTypeLoc)"},
    {"_ZN5clang13TypeLocReader21VisitExtVectorTypeLocENS_16ExtVectorTypeLocE", "clang::TypeLocReader::VisitExtVectorTypeLoc(clang::ExtVectorTypeLoc)"},
    {"_ZN5clang13TypeLocReader20VisitFunctionTypeLocENS_15FunctionTypeLocE", "clang::TypeLocReader::VisitFunctionTypeLoc(clang::FunctionTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitFunctionProtoTypeLocENS_20FunctionProtoTypeLocE", "clang::TypeLocReader::VisitFunctionProtoTypeLoc(clang::FunctionProtoTypeLoc)"},
    {"_ZN5clang13TypeLocReader27VisitFunctionNoProtoTypeLocENS_22FunctionNoProtoTypeLocE", "clang::TypeLocReader::VisitFunctionNoProtoTypeLoc(clang::FunctionNoProtoTypeLoc)"},
    {"_ZN5clang13TypeLocReader27VisitUnresolvedUsingTypeLocENS_22UnresolvedUsingTypeLocE", "clang::TypeLocReader::VisitUnresolvedUsingTypeLoc(clang::UnresolvedUsingTypeLoc)"},
    {"_ZN5clang13TypeLocReader19VisitTypedefTypeLocENS_14TypedefTypeLocE", "clang::TypeLocReader::VisitTypedefTypeLoc(clang::TypedefTypeLoc)"},
    {"_ZN5clang13TypeLocReader22VisitTypeOfExprTypeLocENS_17TypeOfExprTypeLocE", "clang::TypeLocReader::VisitTypeOfExprTypeLoc(clang::TypeOfExprTypeLoc)"},
    {"_ZN5clang13TypeLocReader18VisitTypeOfTypeLocENS_13TypeOfTypeLocE", "clang::TypeLocReader::VisitTypeOfTypeLoc(clang::TypeOfTypeLoc)"},
    {"_ZN5clang9ASTReader17GetTypeSourceInfoERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::GetTypeSourceInfo(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13TypeLocReader20VisitDecltypeTypeLocENS_15DecltypeTypeLocE", "clang::TypeLocReader::VisitDecltypeTypeLoc(clang::DecltypeTypeLoc)"},
    {"_ZN5clang13TypeLocReader18VisitRecordTypeLocENS_13RecordTypeLocE", "clang::TypeLocReader::VisitRecordTypeLoc(clang::RecordTypeLoc)"},
    {"_ZN5clang13TypeLocReader16VisitEnumTypeLocENS_11EnumTypeLocE", "clang::TypeLocReader::VisitEnumTypeLoc(clang::EnumTypeLoc)"},
    {"_ZN5clang13TypeLocReader22VisitAttributedTypeLocENS_17AttributedTypeLocE", "clang::TypeLocReader::VisitAttributedTypeLoc(clang::AttributedTypeLoc)"},
    {"_ZN5clang13TypeLocReader28VisitTemplateTypeParmTypeLocENS_23TemplateTypeParmTypeLocE", "clang::TypeLocReader::VisitTemplateTypeParmTypeLoc(clang::TemplateTypeParmTypeLoc)"},
    {"_ZN5clang13TypeLocReader33VisitSubstTemplateTypeParmTypeLocENS_28SubstTemplateTypeParmTypeLocE", "clang::TypeLocReader::VisitSubstTemplateTypeParmTypeLoc(clang::SubstTemplateTypeParmTypeLoc)"},
    {"_ZN5clang13TypeLocReader37VisitSubstTemplateTypeParmPackTypeLocENS_32SubstTemplateTypeParmPackTypeLocE", "clang::TypeLocReader::VisitSubstTemplateTypeParmPackTypeLoc(clang::SubstTemplateTypeParmPackTypeLoc)"},
    {"_ZN5clang13TypeLocReader34VisitTemplateSpecializationTypeLocENS_29TemplateSpecializationTypeLocE", "clang::TypeLocReader::VisitTemplateSpecializationTypeLoc(clang::TemplateSpecializationTypeLoc)"},
    {"_ZN5clang9ASTReader26GetTemplateArgumentLocInfoERNS0_11PerFileDataENS_16TemplateArgument7ArgKindERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::GetTemplateArgumentLocInfo(clang::ASTReader::PerFileData&, clang::TemplateArgument::ArgKind, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13TypeLocReader17VisitParenTypeLocENS_12ParenTypeLocE", "clang::TypeLocReader::VisitParenTypeLoc(clang::ParenTypeLoc)"},
    {"_ZN5clang13TypeLocReader22VisitElaboratedTypeLocENS_17ElaboratedTypeLocE", "clang::TypeLocReader::VisitElaboratedTypeLoc(clang::ElaboratedTypeLoc)"},
    {"_ZN5clang13TypeLocReader29VisitInjectedClassNameTypeLocENS_24InjectedClassNameTypeLocE", "clang::TypeLocReader::VisitInjectedClassNameTypeLoc(clang::InjectedClassNameTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitDependentNameTypeLocENS_20DependentNameTypeLocE", "clang::TypeLocReader::VisitDependentNameTypeLoc(clang::DependentNameTypeLoc)"},
    {"_ZN5clang13TypeLocReader43VisitDependentTemplateSpecializationTypeLocENS_38DependentTemplateSpecializationTypeLocE", "clang::TypeLocReader::VisitDependentTemplateSpecializationTypeLoc(clang::DependentTemplateSpecializationTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitPackExpansionTypeLocENS_20PackExpansionTypeLocE", "clang::TypeLocReader::VisitPackExpansionTypeLoc(clang::PackExpansionTypeLoc)"},
    {"_ZN5clang13TypeLocReader25VisitObjCInterfaceTypeLocENS_20ObjCInterfaceTypeLocE", "clang::TypeLocReader::VisitObjCInterfaceTypeLoc(clang::ObjCInterfaceTypeLoc)"},
    {"_ZN5clang13TypeLocReader22VisitObjCObjectTypeLocENS_17ObjCObjectTypeLocE", "clang::TypeLocReader::VisitObjCObjectTypeLoc(clang::ObjCObjectTypeLoc)"},
    {"_ZN5clang13TypeLocReader29VisitObjCObjectPointerTypeLocENS_24ObjCObjectPointerTypeLocE", "clang::TypeLocReader::VisitObjCObjectPointerTypeLoc(clang::ObjCObjectPointerTypeLoc)"},
    {"_ZNK5clang9ASTReader9GetTypeIDENS_8QualTypeE", "clang::ASTReader::GetTypeID(clang::QualType) const"},
    {"_ZNK5clang9ASTReader10GetTypeIdxENS_8QualTypeE", "clang::ASTReader::GetTypeIdx(clang::QualType) const"},
    {"_ZNK5clang9ASTReader28getTotalNumCXXBaseSpecifiersEv", "clang::ASTReader::getTotalNumCXXBaseSpecifiers() const"},
    {"_ZN5clang9ASTReader23ReadTemplateArgumentLocERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadTemplateArgumentLoc(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZThn16_N5clang9ASTReader15GetExternalDeclEj", "non-virtual thunk to clang::ASTReader::GetExternalDecl(unsigned int)"},
    {"_ZN5clang9ASTReader15GetExternalDeclEj", "clang::ASTReader::GetExternalDecl(unsigned int)"},
    {"_ZN5clang9ASTReader26GetCXXBaseSpecifiersOffsetEj", "clang::ASTReader::GetCXXBaseSpecifiersOffset(unsigned int)"},
    {"_ZThn16_N5clang9ASTReader28GetExternalCXXBaseSpecifiersEy", "non-virtual thunk to clang::ASTReader::GetExternalCXXBaseSpecifiers(unsigned long long)"},
    {"_ZN5clang9ASTReader28GetExternalCXXBaseSpecifiersEy", "clang::ASTReader::GetExternalCXXBaseSpecifiers(unsigned long long)"},
    {"_ZN5clang9ASTReader20ReadCXXBaseSpecifierERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadCXXBaseSpecifier(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZThn16_N5clang9ASTReader19GetExternalDeclStmtEy", "non-virtual thunk to clang::ASTReader::GetExternalDeclStmt(unsigned long long)"},
    {"_ZN5clang9ASTReader19GetExternalDeclStmtEy", "clang::ASTReader::GetExternalDeclStmt(unsigned long long)"},
    {"_ZN5clang9ASTReader18ClearSwitchCaseIDsEv", "clang::ASTReader::ClearSwitchCaseIDs()"},
    {"_ZThn16_N5clang9ASTReader24FindExternalLexicalDeclsEPKNS_11DeclContextEPFbNS_4Decl4KindEERN4llvm15SmallVectorImplIPS4_EE", "non-virtual thunk to clang::ASTReader::FindExternalLexicalDecls(clang::DeclContext const*, bool (*)(clang::Decl::Kind), llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang9ASTReader24FindExternalLexicalDeclsEPKNS_11DeclContextEPFbNS_4Decl4KindEERN4llvm15SmallVectorImplIPS4_EE", "clang::ASTReader::FindExternalLexicalDecls(clang::DeclContext const*, bool (*)(clang::Decl::Kind), llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZThn16_N5clang9ASTReader30FindExternalVisibleDeclsByNameEPKNS_11DeclContextENS_15DeclarationNameE", "non-virtual thunk to clang::ASTReader::FindExternalVisibleDeclsByName(clang::DeclContext const*, clang::DeclarationName)"},
    {"_ZN5clang9ASTReader30FindExternalVisibleDeclsByNameEPKNS_11DeclContextENS_15DeclarationNameE", "clang::ASTReader::FindExternalVisibleDeclsByName(clang::DeclContext const*, clang::DeclarationName)"},
    {"_ZThn16_N5clang9ASTReader23MaterializeVisibleDeclsEPKNS_11DeclContextE", "non-virtual thunk to clang::ASTReader::MaterializeVisibleDecls(clang::DeclContext const*)"},
    {"_ZN5clang9ASTReader23MaterializeVisibleDeclsEPKNS_11DeclContextE", "clang::ASTReader::MaterializeVisibleDecls(clang::DeclContext const*)"},
    {"_ZN5clang9ASTReader30PassInterestingDeclsToConsumerEv", "clang::ASTReader::PassInterestingDeclsToConsumer()"},
    {"_ZThn16_N5clang9ASTReader20StartTranslationUnitEPNS_11ASTConsumerE", "non-virtual thunk to clang::ASTReader::StartTranslationUnit(clang::ASTConsumer*)"},
    {"_ZN5clang9ASTReader20StartTranslationUnitEPNS_11ASTConsumerE", "clang::ASTReader::StartTranslationUnit(clang::ASTConsumer*)"},
    {"_ZThn16_N5clang9ASTReader10PrintStatsEv", "non-virtual thunk to clang::ASTReader::PrintStats()"},
    {"_ZN5clang9ASTReader10PrintStatsEv", "clang::ASTReader::PrintStats()"},
    {"_ZThn16_N5clang9ASTReader14InitializeSemaERNS_4SemaE", "non-virtual thunk to clang::ASTReader::InitializeSema(clang::Sema&)"},
    {"_ZN5clang9ASTReader14InitializeSemaERNS_4SemaE", "clang::ASTReader::InitializeSema(clang::Sema&)"},
    {"_ZN5clang9ASTReader14DecodeSelectorEj", "clang::ASTReader::DecodeSelector(unsigned int)"},
    {"_ZN5clang9ASTReader3getEPKcS2_", "clang::ASTReader::get(char const*, char const*)"},
    {"_ZN5clang21ASTIdentifierIteratorC1ERKNS_9ASTReaderE", "clang::ASTIdentifierIterator::ASTIdentifierIterator(clang::ASTReader const&)"},
    {"_ZN5clang21ASTIdentifierIteratorC2ERKNS_9ASTReaderE", "clang::ASTIdentifierIterator::ASTIdentifierIterator(clang::ASTReader const&)"},
    {"_ZN5clang21ASTIdentifierIterator4NextEv", "clang::ASTIdentifierIterator::Next()"},
    {"_ZThn32_NK5clang9ASTReader14getIdentifiersEv", "non-virtual thunk to clang::ASTReader::getIdentifiers() const"},
    {"_ZNK5clang9ASTReader14getIdentifiersEv", "clang::ASTReader::getIdentifiers() const"},
    {"_ZThn16_N5clang9ASTReader14ReadMethodPoolENS_8SelectorE", "non-virtual thunk to clang::ASTReader::ReadMethodPool(clang::Selector)"},
    {"_ZN5clang9ASTReader14ReadMethodPoolENS_8SelectorE", "clang::ASTReader::ReadMethodPool(clang::Selector)"},
    {"_ZN5clang9ASTReader12LoadSelectorENS_8SelectorE", "clang::ASTReader::LoadSelector(clang::Selector)"},
    {"_ZN5clang9ASTReader17SetIdentifierInfoEjPNS_14IdentifierInfoE", "clang::ASTReader::SetIdentifierInfo(unsigned int, clang::IdentifierInfo*)"},
    {"_ZN5clang9ASTReader23SetGloballyVisibleDeclsEPNS_14IdentifierInfoERKN4llvm15SmallVectorImplIjEEb", "clang::ASTReader::SetGloballyVisibleDecls(clang::IdentifierInfo*, llvm::SmallVectorImpl<unsigned int> const&, bool)"},
    {"_ZThn48_N5clang9ASTReader13ReadSLocEntryEj", "non-virtual thunk to clang::ASTReader::ReadSLocEntry(unsigned int)"},
    {"_ZN5clang9ASTReader13ReadSLocEntryEj", "clang::ASTReader::ReadSLocEntry(unsigned int)"},
    {"_ZN12_GLOBAL__N_122ASTSelectorLookupTrait7ReadKeyEPKhj", "(anonymous namespace)::ASTSelectorLookupTrait::ReadKey(unsigned char const*, unsigned int)"},
    {"_ZThn16_N5clang9ASTReader19GetExternalSelectorEj", "non-virtual thunk to clang::ASTReader::GetExternalSelector(unsigned int)"},
    {"_ZN5clang9ASTReader19GetExternalSelectorEj", "clang::ASTReader::GetExternalSelector(unsigned int)"},
    {"_ZThn16_N5clang9ASTReader23GetNumExternalSelectorsEv", "non-virtual thunk to clang::ASTReader::GetNumExternalSelectors()"},
    {"_ZN5clang9ASTReader23GetNumExternalSelectorsEv", "clang::ASTReader::GetNumExternalSelectors()"},
    {"_ZN5clang9ASTReader19ReadDeclarationNameERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadDeclarationName(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader22ReadDeclarationNameLocERNS0_11PerFileDataERNS_18DeclarationNameLocENS_15DeclarationNameERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadDeclarationNameLoc(clang::ASTReader::PerFileData&, clang::DeclarationNameLoc&, clang::DeclarationName, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader23ReadDeclarationNameInfoERNS0_11PerFileDataERNS_19DeclarationNameInfoERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadDeclarationNameInfo(clang::ASTReader::PerFileData&, clang::DeclarationNameInfo&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader17ReadQualifierInfoERNS0_11PerFileDataERNS_13QualifierInfoERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadQualifierInfo(clang::ASTReader::PerFileData&, clang::QualifierInfo&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader25ReadTemplateParameterListERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadTemplateParameterList(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader10ReadAPSIntERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadAPSInt(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader17ReadUnresolvedSetERNS_17UnresolvedSetImplERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadUnresolvedSet(clang::UnresolvedSetImpl&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader23ReadCXXCtorInitializersERNS0_11PerFileDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadCXXCtorInitializers(clang::ASTReader::PerFileData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader11ReadAPFloatERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadAPFloat(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader10ReadStringERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadString(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader16ReadCXXTemporaryERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTReader::ReadCXXTemporary(llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang9ASTReader18getIdentifierTableEv", "clang::ASTReader::getIdentifierTable()"},
    {"_ZN5clang9ASTReader18RecordSwitchCaseIDEPNS_10SwitchCaseEj", "clang::ASTReader::RecordSwitchCaseID(clang::SwitchCase*, unsigned int)"},
    {"_ZN5clang9ASTReader19getSwitchCaseWithIDEj", "clang::ASTReader::getSwitchCaseWithID(unsigned int)"},
    {"_ZN5clang9ASTReader15RecordLabelStmtEPNS_9LabelStmtEj", "clang::ASTReader::RecordLabelStmt(clang::LabelStmt*, unsigned int)"},
    {"_ZN5clang9ASTReader10SetLabelOfEPNS_8GotoStmtEj", "clang::ASTReader::SetLabelOf(clang::GotoStmt*, unsigned int)"},
    {"_ZN5clang9ASTReader10SetLabelOfEPNS_13AddrLabelExprEj", "clang::ASTReader::SetLabelOf(clang::AddrLabelExpr*, unsigned int)"},
    {"_ZThn16_N5clang9ASTReader21FinishedDeserializingEv", "non-virtual thunk to clang::ASTReader::FinishedDeserializing()"},
    {"_ZN5clang9ASTReader21FinishedDeserializingEv", "clang::ASTReader::FinishedDeserializing()"},
    {"_ZN5clang9ASTReaderC1ERNS_12PreprocessorEPNS_10ASTContextEPKcbb", "clang::ASTReader::ASTReader(clang::Preprocessor&, clang::ASTContext*, char const*, bool, bool)"},
    {"_ZN5clang9ASTReaderC2ERNS_12PreprocessorEPNS_10ASTContextEPKcbb", "clang::ASTReader::ASTReader(clang::Preprocessor&, clang::ASTContext*, char const*, bool, bool)"},
    {"_ZN5clang9ASTReaderC1ERNS_13SourceManagerERNS_11FileManagerERNS_10DiagnosticEPKcbb", "clang::ASTReader::ASTReader(clang::SourceManager&, clang::FileManager&, clang::Diagnostic&, char const*, bool, bool)"},
    {"_ZN5clang9ASTReaderC2ERNS_13SourceManagerERNS_11FileManagerERNS_10DiagnosticEPKcbb", "clang::ASTReader::ASTReader(clang::SourceManager&, clang::FileManager&, clang::Diagnostic&, char const*, bool, bool)"},
    {"_ZThn8_N5clang9ASTReaderD0Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZN5clang9ASTReaderD0Ev", "clang::ASTReader::~ASTReader()"},
    {"_ZThn16_N5clang9ASTReaderD0Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn32_N5clang9ASTReaderD0Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn40_N5clang9ASTReaderD0Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn48_N5clang9ASTReaderD0Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZN5clang9ASTReaderD1Ev", "clang::ASTReader::~ASTReader()"},
    {"_ZThn8_N5clang9ASTReaderD1Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn16_N5clang9ASTReaderD1Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn32_N5clang9ASTReaderD1Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn40_N5clang9ASTReaderD1Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZThn48_N5clang9ASTReaderD1Ev", "non-virtual thunk to clang::ASTReader::~ASTReader()"},
    {"_ZN5clang9ASTReaderD2Ev", "clang::ASTReader::~ASTReader()"},
    {"_ZN5clang9ASTReader11PerFileDataD1Ev", "clang::ASTReader::PerFileData::~PerFileData()"},
    {"_ZN5clang9ASTReader11PerFileDataC2ENS0_11ASTFileTypeE", "clang::ASTReader::PerFileData::PerFileData(clang::ASTReader::ASTFileType)"},
    {"_ZN5clang9ASTReader11PerFileDataD2Ev", "clang::ASTReader::PerFileData::~PerFileData()"},
    {"_ZN12_GLOBAL__N_129ASTDeclContextNameLookupTrait7ReadKeyEPKhj", "(anonymous namespace)::ASTDeclContextNameLookupTrait::ReadKey(unsigned char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_112ASTStatCacheD1Ev", "(anonymous namespace)::ASTStatCache::~ASTStatCache()"},
    {"_ZN12_GLOBAL__N_112ASTStatCacheD0Ev", "(anonymous namespace)::ASTStatCache::~ASTStatCache()"},
    {"_ZN12_GLOBAL__N_112ASTStatCache7getStatEPKcR4statPi", "(anonymous namespace)::ASTStatCache::getStat(char const*, stat&, int*)"},
    {"_ZSt14set_differenceIPN4llvm9StringRefES2_St20back_insert_iteratorISt6vectorIS1_SaIS1_EEEET1_T_S9_T0_SA_S8_", "std::back_insert_iterator<std::vector<llvm::StringRef, std::allocator<llvm::StringRef> > > std::set_difference<llvm::StringRef*, llvm::StringRef*, std::back_insert_iterator<std::vector<llvm::StringRef, std::allocator<llvm::StringRef> > > >(llvm::StringRef*, llvm::StringRef*, llvm::StringRef*, llvm::StringRef*, std::back_insert_iterator<std::vector<llvm::StringRef, std::allocator<llvm::StringRef> > >)"},
    {"_ZN4llvm15BitstreamCursor10ReadRecordEjRNS_15SmallVectorImplIyEEPPKcPj", "llvm::BitstreamCursor::ReadRecord(unsigned int, llvm::SmallVectorImpl<unsigned long long>&, char const**, unsigned int*)"},
    {"_ZNSt6vectorIN5clang9LineEntryESaIS1_EE7reserveEm", "std::vector<clang::LineEntry, std::allocator<clang::LineEntry> >::reserve(unsigned long)"},
    {"_ZN4llvm15BitstreamCursoraSERKS0_", "llvm::BitstreamCursor::operator=(llvm::BitstreamCursor const&)"},
    {"_ZN4llvm15BitstreamCursor9SkipBlockEv", "llvm::BitstreamCursor::SkipBlock()"},
    {"_ZN4llvm15BitstreamCursor13EnterSubBlockEjPj", "llvm::BitstreamCursor::EnterSubBlock(unsigned int, unsigned int*)"},
    {"_ZN4llvm15BitstreamCursor14ReadSubBlockIDEv", "llvm::BitstreamCursor::ReadSubBlockID()"},
    {"_ZN4llvm15BitstreamCursor16ReadAbbrevRecordEv", "llvm::BitstreamCursor::ReadAbbrevRecord()"},
    {"_ZN4llvm15SmallVectorImplIyE4swapERS1_", "llvm::SmallVectorImpl<unsigned long long>::swap(llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN4llvm15SmallVectorImplIyE6insertIPyEES3_S3_T_S4_", "unsigned long long* llvm::SmallVectorImpl<unsigned long long>::insert<unsigned long long*>(unsigned long long*, unsigned long long*, unsigned long long*)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang15MacroDefinitionEE6resizeEj", "llvm::SmallVectorImpl<clang::MacroDefinition*>::resize(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang8SelectorEE6resizeEj", "llvm::SmallVectorImpl<clang::Selector>::resize(unsigned int)"},
    {"_ZN5clang22OnDiskChainedHashTableINS_24ASTIdentifierLookupTraitEE4findERKSt4pairIPKcjEPS1_", "clang::OnDiskChainedHashTable<clang::ASTIdentifierLookupTrait>::find(std::pair<char const*, unsigned int> const&, clang::ASTIdentifierLookupTrait*)"},
    {"_ZN4llvm15BitstreamCursor4ReadEj", "llvm::BitstreamCursor::Read(unsigned int)"},
    {"_ZN4llvm15BitstreamCursor18ReadBlockInfoBlockEv", "llvm::BitstreamCursor::ReadBlockInfoBlock()"},
    {"_ZN4llvm15SmallVectorImplIN5clang9ASTReader15DeclContextInfoEE4swapERS4_", "llvm::SmallVectorImpl<clang::ASTReader::DeclContextInfo>::swap(llvm::SmallVectorImpl<clang::ASTReader::DeclContextInfo>&)"},
    {"_ZN5clang14TypeLocVisitorINS_13TypeLocReaderEvE5VisitENS_7TypeLocE", "clang::TypeLocVisitor<clang::TypeLocReader, void>::Visit(clang::TypeLoc)"},
    {"_ZN5clang13serialization10MakeTypeIDISt9binder1stISt16const_mem_fun1_tINS0_7TypeIdxENS_9ASTReaderENS_8QualTypeEEEEEjS6_T_", "unsigned int clang::serialization::MakeTypeID<std::binder1st<std::const_mem_fun1_t<clang::serialization::TypeIdx, clang::ASTReader, clang::QualType> > >(clang::QualType, std::binder1st<std::const_mem_fun1_t<clang::serialization::TypeIdx, clang::ASTReader, clang::QualType> >)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorENS1_14SourceLocationENS_12DenseMapInfoIS2_EENS4_IS3_EEE6insertERKSt4pairIS2_S3_E", "llvm::DenseMap<clang::Selector, clang::SourceLocation, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::SourceLocation> >::insert(std::pair<clang::Selector, clang::SourceLocation> const&)"},
    {"_ZNSt5dequeISt4pairIPN5clang9ValueDeclENS1_14SourceLocationEESaIS5_EE9push_backERKS5_", "std::deque<std::pair<clang::ValueDecl*, clang::SourceLocation>, std::allocator<std::pair<clang::ValueDecl*, clang::SourceLocation> > >::push_back(std::pair<clang::ValueDecl*, clang::SourceLocation> const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoENS1_4Sema8WeakInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, clang::Sema::WeakInfo, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::Sema::WeakInfo> >::insert(std::pair<clang::IdentifierInfo*, clang::Sema::WeakInfo> const&)"},
    {"_ZN5clang17ASTReaderListener19ReadLanguageOptionsERKNS_11LangOptionsE", "clang::ASTReaderListener::ReadLanguageOptions(clang::LangOptions const&)"},
    {"_ZN5clang17ASTReaderListener16ReadTargetTripleEN4llvm9StringRefE", "clang::ASTReaderListener::ReadTargetTriple(llvm::StringRef)"},
    {"_ZN5clang17ASTReaderListener20ReadPredefinesBufferERKN4llvm11SmallVectorINS_18PCHPredefinesBlockELj2EEENS1_9StringRefERSs", "clang::ASTReaderListener::ReadPredefinesBuffer(llvm::SmallVector<clang::PCHPredefinesBlock, 2u> const&, llvm::StringRef, std::string&)"},
    {"_ZN5clang17ASTReaderListener18ReadHeaderFileInfoERKNS_14HeaderFileInfoEj", "clang::ASTReaderListener::ReadHeaderFileInfo(clang::HeaderFileInfo const&, unsigned int)"},
    {"_ZN5clang17ASTReaderListener11ReadCounterEj", "clang::ASTReaderListener::ReadCounter(unsigned int)"},
    {"_ZN5clang12PCHValidatorD1Ev", "clang::PCHValidator::~PCHValidator()"},
    {"_ZN5clang12PCHValidatorD0Ev", "clang::PCHValidator::~PCHValidator()"},
    {"_ZN5clang9ASTReader20StartedDeserializingEv", "clang::ASTReader::StartedDeserializing()"},
    {"_ZN5clang9ASTReader10ForgetSemaEv", "clang::ASTReader::ForgetSema()"},
    {"_ZN5clang9ASTReader3getEN4llvm9StringRefE", "clang::ASTReader::get(llvm::StringRef)"},
    {"_ZN5clang9ASTReader13GetIdentifierEj", "clang::ASTReader::GetIdentifier(unsigned int)"},
    {"_ZThn16_N5clang9ASTReader20StartedDeserializingEv", "non-virtual thunk to clang::ASTReader::StartedDeserializing()"},
    {"_ZThn16_N5clang9ASTReader10ForgetSemaEv", "non-virtual thunk to clang::ASTReader::ForgetSema()"},
    {"_ZThn32_N5clang9ASTReader3getEN4llvm9StringRefE", "non-virtual thunk to clang::ASTReader::get(llvm::StringRef)"},
    {"_ZThn40_N5clang9ASTReader13GetIdentifierEj", "non-virtual thunk to clang::ASTReader::GetIdentifier(unsigned int)"},
    {"_ZN5clang21ASTIdentifierIteratorD1Ev", "clang::ASTIdentifierIterator::~ASTIdentifierIterator()"},
    {"_ZN5clang21ASTIdentifierIteratorD0Ev", "clang::ASTIdentifierIterator::~ASTIdentifierIterator()"},
    {"_ZNSt5dequeIN5clang9ASTReader21PendingIdentifierInfoESaIS2_EED2Ev", "std::deque<clang::ASTReader::PendingIdentifierInfo, std::allocator<clang::ASTReader::PendingIdentifierInfo> >::~deque()"},
    {"_ZNSt5dequeIN5clang9ASTReader21PendingIdentifierInfoESaIS2_EE19_M_destroy_data_auxESt15_Deque_iteratorIS2_RS2_PS2_ES8_", "std::deque<clang::ASTReader::PendingIdentifierInfo, std::allocator<clang::ASTReader::PendingIdentifierInfo> >::_M_destroy_data_aux(std::_Deque_iterator<clang::ASTReader::PendingIdentifierInfo, clang::ASTReader::PendingIdentifierInfo&, clang::ASTReader::PendingIdentifierInfo*>, std::_Deque_iterator<clang::ASTReader::PendingIdentifierInfo, clang::ASTReader::PendingIdentifierInfo&, clang::ASTReader::PendingIdentifierInfo*>)"},
    {"_ZNSt11_Deque_baseIPN5clang4DeclESaIS2_EE17_M_initialize_mapEm", "std::_Deque_base<clang::Decl*, std::allocator<clang::Decl*> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt11_Deque_baseIN5clang9ASTReader21PendingIdentifierInfoESaIS2_EE17_M_initialize_mapEm", "std::_Deque_base<clang::ASTReader::PendingIdentifierInfo, std::allocator<clang::ASTReader::PendingIdentifierInfo> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang13AddrLabelExprEESt10_Select1stIS5_ESt4lessIjESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::AddrLabelExpr*>, std::_Select1st<std::pair<unsigned int const, clang::AddrLabelExpr*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::AddrLabelExpr*> > >::erase(std::_Rb_tree_iterator<std::pair<unsigned int const, clang::AddrLabelExpr*> >, std::_Rb_tree_iterator<std::pair<unsigned int const, clang::AddrLabelExpr*> >)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang13AddrLabelExprEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::AddrLabelExpr*>, std::_Select1st<std::pair<unsigned int const, clang::AddrLabelExpr*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::AddrLabelExpr*> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, clang::AddrLabelExpr*> >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang8GotoStmtEESt10_Select1stIS5_ESt4lessIjESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::GotoStmt*>, std::_Select1st<std::pair<unsigned int const, clang::GotoStmt*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::GotoStmt*> > >::erase(std::_Rb_tree_iterator<std::pair<unsigned int const, clang::GotoStmt*> >, std::_Rb_tree_iterator<std::pair<unsigned int const, clang::GotoStmt*> >)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang8GotoStmtEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::GotoStmt*>, std::_Select1st<std::pair<unsigned int const, clang::GotoStmt*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::GotoStmt*> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, clang::GotoStmt*> >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang9LabelStmtEESt10_Select1stIS5_ESt4lessIjESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::LabelStmt*>, std::_Select1st<std::pair<unsigned int const, clang::LabelStmt*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::LabelStmt*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, clang::LabelStmt*> >, std::pair<unsigned int const, clang::LabelStmt*> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang9LabelStmtEESt10_Select1stIS5_ESt4lessIjESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::LabelStmt*>, std::_Select1st<std::pair<unsigned int const, clang::LabelStmt*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::LabelStmt*> > >::_M_insert_unique(std::pair<unsigned int const, clang::LabelStmt*> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang10SwitchCaseEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::SwitchCase*>, std::_Select1st<std::pair<unsigned int const, clang::SwitchCase*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::SwitchCase*> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, clang::SwitchCase*> >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang10SwitchCaseEESt10_Select1stIS5_ESt4lessIjESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::SwitchCase*>, std::_Select1st<std::pair<unsigned int const, clang::SwitchCase*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::SwitchCase*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, clang::SwitchCase*> >, std::pair<unsigned int const, clang::SwitchCase*> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang10SwitchCaseEESt10_Select1stIS5_ESt4lessIjESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::SwitchCase*>, std::_Select1st<std::pair<unsigned int const, clang::SwitchCase*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::SwitchCase*> > >::_M_insert_unique(std::pair<unsigned int const, clang::SwitchCase*> const&)"},
    {"_ZNSt5dequeIN5clang9ASTReader21PendingIdentifierInfoESaIS2_EE16_M_push_back_auxERKS2_", "std::deque<clang::ASTReader::PendingIdentifierInfo, std::allocator<clang::ASTReader::PendingIdentifierInfo> >::_M_push_back_aux(clang::ASTReader::PendingIdentifierInfo const&)"},
    {"_ZNSt5dequeIN5clang9ASTReader21PendingIdentifierInfoESaIS2_EE17_M_reallocate_mapEmb", "std::deque<clang::ASTReader::PendingIdentifierInfo, std::allocator<clang::ASTReader::PendingIdentifierInfo> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm8DenseMapIPN5clang13CXXRecordDeclEbNS_12DenseMapInfoIS3_EENS4_IbEEE16InsertIntoBucketERKS3_RKbPSt4pairIS3_bE", "llvm::DenseMap<clang::CXXRecordDecl*, bool, llvm::DenseMapInfo<clang::CXXRecordDecl*>, llvm::DenseMapInfo<bool> >::InsertIntoBucket(clang::CXXRecordDecl* const&, bool const&, std::pair<clang::CXXRecordDecl*, bool>*)"},
    {"_ZN4llvm8DenseMapIPN5clang13CXXRecordDeclEbNS_12DenseMapInfoIS3_EENS4_IbEEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl*, bool, llvm::DenseMapInfo<clang::CXXRecordDecl*>, llvm::DenseMapInfo<bool> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoENS1_4Sema8WeakInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, clang::Sema::WeakInfo, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::Sema::WeakInfo> >::InsertIntoBucket(clang::IdentifierInfo* const&, clang::Sema::WeakInfo const&, std::pair<clang::IdentifierInfo*, clang::Sema::WeakInfo>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoENS1_4Sema8WeakInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, clang::Sema::WeakInfo, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::Sema::WeakInfo> >::grow(unsigned int)"},
    {"_ZNSt5dequeISt4pairIPN5clang9ValueDeclENS1_14SourceLocationEESaIS5_EE17_M_reallocate_mapEmb", "std::deque<std::pair<clang::ValueDecl*, clang::SourceLocation>, std::allocator<std::pair<clang::ValueDecl*, clang::SourceLocation> > >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorENS1_14SourceLocationENS_12DenseMapInfoIS2_EENS4_IS3_EEE16InsertIntoBucketERKS2_RKS3_PSt4pairIS2_S3_E", "llvm::DenseMap<clang::Selector, clang::SourceLocation, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::SourceLocation> >::InsertIntoBucket(clang::Selector const&, clang::SourceLocation const&, std::pair<clang::Selector, clang::SourceLocation>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorENS1_14SourceLocationENS_12DenseMapInfoIS2_EENS4_IS3_EEE4growEj", "llvm::DenseMap<clang::Selector, clang::SourceLocation, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::SourceLocation> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameEPNS1_9NamedDeclENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<clang::DeclarationName, clang::NamedDecl*, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::NamedDecl*> >::InsertIntoBucket(clang::DeclarationName const&, clang::NamedDecl* const&, std::pair<clang::DeclarationName, clang::NamedDecl*>*)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameEPNS1_9NamedDeclENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::DeclarationName, clang::NamedDecl*, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::NamedDecl*> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang9ASTReader15DeclContextInfoEEaSERKS4_", "llvm::SmallVectorImpl<clang::ASTReader::DeclContextInfo>::operator=(llvm::SmallVectorImpl<clang::ASTReader::DeclContextInfo> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang9ASTReader15DeclContextInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ASTReader::DeclContextInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIN5clang8QualTypeENS1_13serialization7TypeIdxENS3_26UnsafeQualTypeDenseMapInfoENS_12DenseMapInfoIS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<clang::QualType, clang::serialization::TypeIdx, clang::serialization::UnsafeQualTypeDenseMapInfo, llvm::DenseMapInfo<clang::serialization::TypeIdx> >::InsertIntoBucket(clang::QualType const&, clang::serialization::TypeIdx const&, std::pair<clang::QualType, clang::serialization::TypeIdx>*)"},
    {"_ZN4llvm8DenseMapIN5clang8QualTypeENS1_13serialization7TypeIdxENS3_26UnsafeQualTypeDenseMapInfoENS_12DenseMapInfoIS4_EEE4growEj", "llvm::DenseMap<clang::QualType, clang::serialization::TypeIdx, clang::serialization::UnsafeQualTypeDenseMapInfo, llvm::DenseMapInfo<clang::serialization::TypeIdx> >::grow(unsigned int)"},
    {"_ZN5clang24ASTIdentifierLookupTrait8ReadDataERKSt4pairIPKcjEPKhj", "clang::ASTIdentifierLookupTrait::ReadData(std::pair<char const*, unsigned int> const&, unsigned char const*, unsigned int)"},
    {"_ZNSt6vectorIPN5clang4DeclESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<clang::Decl*, std::allocator<clang::Decl*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::Decl**, std::vector<clang::Decl*, std::allocator<clang::Decl*> > >, unsigned long, clang::Decl* const&)"},
    {"_ZNSt6vectorIN5clang8QualTypeESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<clang::QualType, std::allocator<clang::QualType> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::QualType*, std::vector<clang::QualType, std::allocator<clang::QualType> > >, unsigned long, clang::QualType const&)"},
    {"_ZNSt6vectorIPN5clang14IdentifierInfoESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<clang::IdentifierInfo*, std::allocator<clang::IdentifierInfo*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::IdentifierInfo**, std::vector<clang::IdentifierInfo*, std::allocator<clang::IdentifierInfo*> > >, unsigned long, clang::IdentifierInfo* const&)"},
    {"_ZN4llvm8DenseMapIjSt4pairIPN5clang9ASTReader11PerFileDataEyENS_12DenseMapInfoIjEENS7_IS6_EEE16InsertIntoBucketERKjRKS6_PS1_IjS6_E", "llvm::DenseMap<unsigned int, std::pair<clang::ASTReader::PerFileData*, unsigned long long>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<clang::ASTReader::PerFileData*, unsigned long long> > >::InsertIntoBucket(unsigned int const&, std::pair<clang::ASTReader::PerFileData*, unsigned long long> const&, std::pair<unsigned int, std::pair<clang::ASTReader::PerFileData*, unsigned long long> >*)"},
    {"_ZN4llvm8DenseMapIjSt4pairIPN5clang9ASTReader11PerFileDataEyENS_12DenseMapInfoIjEENS7_IS6_EEE4growEj", "llvm::DenseMap<unsigned int, std::pair<clang::ASTReader::PerFileData*, unsigned long long>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<clang::ASTReader::PerFileData*, unsigned long long> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIPN5clang9ASTReader11PerFileDataEyELj2EEENS_12DenseMapInfoIjEENS9_IS8_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u> > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIPN5clang9ASTReader11PerFileDataEyELj2EEENS_12DenseMapInfoIjEENS9_IS8_EEE16InsertIntoBucketERKjRKS8_PS2_IjS8_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u> const&, std::pair<unsigned int, llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIPN5clang9ASTReader11PerFileDataEyELj2EEENS_12DenseMapInfoIjEENS9_IS8_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::ASTReader::PerFileData*, unsigned long long>, 2u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPN5clang9ASTReader11PerFileDataEyEEaSERKS7_", "llvm::SmallVectorImpl<std::pair<clang::ASTReader::PerFileData*, unsigned long long> >::operator=(llvm::SmallVectorImpl<std::pair<clang::ASTReader::PerFileData*, unsigned long long> > const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPvLj1EEENS_12DenseMapInfoIjEENS4_IS3_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, llvm::SmallVector<void*, 1u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<void*, 1u> > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPvLj1EEENS_12DenseMapInfoIjEENS4_IS3_EEE16InsertIntoBucketERKjRKS3_PSt4pairIjS3_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<void*, 1u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<void*, 1u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallVector<void*, 1u> const&, std::pair<unsigned int, llvm::SmallVector<void*, 1u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPvLj1EEENS_12DenseMapInfoIjEENS4_IS3_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallVector<void*, 1u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<void*, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPvEaSERKS2_", "llvm::SmallVectorImpl<void*>::operator=(llvm::SmallVectorImpl<void*> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextENS_11SmallVectorINS1_9ASTReader15DeclContextInfoELj1EEENS_12DenseMapInfoIS4_EENS9_IS8_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::DeclContext const*, llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u>, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u> > >::FindAndConstruct(clang::DeclContext const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextENS_11SmallVectorINS1_9ASTReader15DeclContextInfoELj1EEENS_12DenseMapInfoIS4_EENS9_IS8_EEE16InsertIntoBucketERKS4_RKS8_PSt4pairIS4_S8_E", "llvm::DenseMap<clang::DeclContext const*, llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u>, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u> > >::InsertIntoBucket(clang::DeclContext const* const&, llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u> const&, std::pair<clang::DeclContext const*, llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang11DeclContextENS_11SmallVectorINS1_9ASTReader15DeclContextInfoELj1EEENS_12DenseMapInfoIS4_EENS9_IS8_EEE4growEj", "llvm::DenseMap<clang::DeclContext const*, llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u>, llvm::DenseMapInfo<clang::DeclContext const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ASTReader::DeclContextInfo, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEyNS_12DenseMapInfoIS3_EENS4_IyEEE16InsertIntoBucketERKS3_RKyPSt4pairIS3_yE", "llvm::DenseMap<clang::IdentifierInfo*, unsigned long long, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(clang::IdentifierInfo* const&, unsigned long long const&, std::pair<clang::IdentifierInfo*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEyNS_12DenseMapInfoIS3_EENS4_IyEEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, unsigned long long, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang18PCHPredefinesBlockELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::PCHPredefinesBlock, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiiESt10_Select1stIS2_ESt4lessIiESaIS2_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS2_ERKS2_", "std::_Rb_tree<int, std::pair<int const, int>, std::_Select1st<std::pair<int const, int> >, std::less<int>, std::allocator<std::pair<int const, int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<int const, int> >, std::pair<int const, int> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiiESt10_Select1stIS2_ESt4lessIiESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<int, std::pair<int const, int>, std::_Select1st<std::pair<int const, int> >, std::less<int>, std::allocator<std::pair<int const, int> > >::_M_insert_unique(std::pair<int const, int> const&)"},
    {"_ZSt16__introsort_loopIPN4llvm9StringRefElEvT_S3_T0_", "void std::__introsort_loop<llvm::StringRef*, long>(llvm::StringRef*, llvm::StringRef*, long)"},
    {"_ZSt22__final_insertion_sortIPN4llvm9StringRefEEvT_S3_", "void std::__final_insertion_sort<llvm::StringRef*>(llvm::StringRef*, llvm::StringRef*)"},
    {"_ZSt16__insertion_sortIPN4llvm9StringRefEEvT_S3_", "void std::__insertion_sort<llvm::StringRef*>(llvm::StringRef*, llvm::StringRef*)"},
    {"_ZSt21__unguarded_partitionIPN4llvm9StringRefES1_ET_S3_S3_T0_", "llvm::StringRef* std::__unguarded_partition<llvm::StringRef*, llvm::StringRef>(llvm::StringRef*, llvm::StringRef*, llvm::StringRef)"},
    {"_ZSt8__medianIN4llvm9StringRefEERKT_S4_S4_S4_", "llvm::StringRef const& std::__median<llvm::StringRef>(llvm::StringRef const&, llvm::StringRef const&, llvm::StringRef const&)"},
    {"_ZSt13__heap_selectIPN4llvm9StringRefEEvT_S3_S3_", "void std::__heap_select<llvm::StringRef*>(llvm::StringRef*, llvm::StringRef*, llvm::StringRef*)"},
    {"_ZSt13__adjust_heapIPN4llvm9StringRefElS1_EvT_T0_S4_T1_", "void std::__adjust_heap<llvm::StringRef*, long, llvm::StringRef>(llvm::StringRef*, long, long, llvm::StringRef)"},
    {"_ZN4llvm15SmallVectorImplIN5clang18PCHPredefinesBlockEEaSERKS3_", "llvm::SmallVectorImpl<clang::PCHPredefinesBlock>::operator=(llvm::SmallVectorImpl<clang::PCHPredefinesBlock> const&)"},
    {"_ZN4llvm15SmallVectorImplINS_9StringRefEEaSERKS2_", "llvm::SmallVectorImpl<llvm::StringRef>::operator=(llvm::SmallVectorImpl<llvm::StringRef> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN5clang9LabelStmtEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, clang::LabelStmt*>, std::_Select1st<std::pair<unsigned int const, clang::LabelStmt*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, clang::LabelStmt*> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, clang::LabelStmt*> >*)"},
    {"_ZN5clang18ExternalSemaSource14InitializeSemaERNS_4SemaE", "clang::ExternalSemaSource::InitializeSema(clang::Sema&)"},
    {"_ZN5clang18ExternalSemaSource10ForgetSemaEv", "clang::ExternalSemaSource::ForgetSema()"},
    {"_ZN4llvm15BitstreamReaderD2Ev", "llvm::BitstreamReader::~BitstreamReader()"},
    {"_ZN4llvm15BitstreamReader9BlockInfoD2Ev", "llvm::BitstreamReader::BlockInfo::~BlockInfo()"},
    {"_ZN4llvm15BitstreamReader20getOrCreateBlockInfoEj", "llvm::BitstreamReader::getOrCreateBlockInfo(unsigned int)"},
    {"_ZNSt6vectorISt4pairIjSsESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, std::string>*, std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > > >, std::pair<unsigned int, std::string> const&)"},
    {"_ZN9__gnu_cxx13new_allocatorIN4llvm15BitstreamReader9BlockInfoEE9constructEPS3_RKS3_", "__gnu_cxx::new_allocator<llvm::BitstreamReader::BlockInfo>::construct(llvm::BitstreamReader::BlockInfo*, llvm::BitstreamReader::BlockInfo const&)"},
    {"_ZNSt6vectorIN4llvm15BitstreamReader9BlockInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BitstreamReader::BlockInfo, std::allocator<llvm::BitstreamReader::BlockInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BitstreamReader::BlockInfo*, std::vector<llvm::BitstreamReader::BlockInfo, std::allocator<llvm::BitstreamReader::BlockInfo> > >, llvm::BitstreamReader::BlockInfo const&)"},
    {"_ZSt24__uninitialized_copy_auxIPN4llvm15BitstreamReader9BlockInfoES3_ET0_T_S5_S4_St12__false_type", "llvm::BitstreamReader::BlockInfo* std::__uninitialized_copy_aux<llvm::BitstreamReader::BlockInfo*, llvm::BitstreamReader::BlockInfo*>(llvm::BitstreamReader::BlockInfo*, llvm::BitstreamReader::BlockInfo*, llvm::BitstreamReader::BlockInfo*, std::__false_type)"},
    {"_ZNSt6vectorIPN4llvm13BitCodeAbbrevESaIS2_EEaSERKS4_", "std::vector<llvm::BitCodeAbbrev*, std::allocator<llvm::BitCodeAbbrev*> >::operator=(std::vector<llvm::BitCodeAbbrev*, std::allocator<llvm::BitCodeAbbrev*> > const&)"},
    {"_ZNSt6vectorISt4pairIjSsESaIS1_EEaSERKS3_", "std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > >::operator=(std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > > const&)"},
    {"_ZNSt6vectorISt4pairIjSsESaIS1_EEC2ERKS3_", "std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > >::vector(std::vector<std::pair<unsigned int, std::string>, std::allocator<std::pair<unsigned int, std::string> > > const&)"},
    {"_ZNSt6vectorIPN4llvm13BitCodeAbbrevESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BitCodeAbbrev*, std::allocator<llvm::BitCodeAbbrev*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BitCodeAbbrev**, std::vector<llvm::BitCodeAbbrev*, std::allocator<llvm::BitCodeAbbrev*> > >, llvm::BitCodeAbbrev* const&)"},
    {"_ZN4llvm15BitstreamCursor9freeStateEv", "llvm::BitstreamCursor::freeState()"},
    {"_ZN5clang19FileSystemStatCacheD1Ev", "clang::FileSystemStatCache::~FileSystemStatCache()"},
    {"_ZN5clang19FileSystemStatCacheD0Ev", "clang::FileSystemStatCache::~FileSystemStatCache()"},
    {"_ZN4llvm15BitstreamCursorD2Ev", "llvm::BitstreamCursor::~BitstreamCursor()"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15BitCodeAbbrevOpELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::BitCodeAbbrevOp, false>::grow(unsigned long)"},
    {"_ZN4llvm15BitstreamCursor13PopBlockScopeEv", "llvm::BitstreamCursor::PopBlockScope()"},
    {"_ZN4llvm15SmallVectorImplINS_15BitstreamCursor5BlockEE9push_backERKS2_", "llvm::SmallVectorImpl<llvm::BitstreamCursor::Block>::push_back(llvm::BitstreamCursor::Block const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15BitstreamCursor5BlockELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::BitstreamCursor::Block, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15BitstreamCursor5BlockELb0EE18uninitialized_copyIPS2_S5_EEvT_S6_T0_", "void llvm::SmallVectorTemplateBase<llvm::BitstreamCursor::Block, false>::uninitialized_copy<llvm::BitstreamCursor::Block*, llvm::BitstreamCursor::Block*>(llvm::BitstreamCursor::Block*, llvm::BitstreamCursor::Block*, llvm::BitstreamCursor::Block*)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiiESt10_Select1stIS2_ESt4lessIiESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<int, std::pair<int const, int>, std::_Select1st<std::pair<int const, int> >, std::less<int>, std::allocator<std::pair<int const, int> > >::_M_erase(std::_Rb_tree_node<std::pair<int const, int> >*)"},
    {"_ZN4llvm15BitstreamCursor20ReadAbbreviatedFieldERKNS_15BitCodeAbbrevOpERNS_15SmallVectorImplIyEE", "llvm::BitstreamCursor::ReadAbbreviatedField(llvm::BitCodeAbbrevOp const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang13ASTDeclReader22GetCurrentCursorOffsetEv", "clang::ASTDeclReader::GetCurrentCursorOffset()"},
    {"_ZN5clang13ASTDeclReader5VisitEPNS_4DeclE", "clang::ASTDeclReader::Visit(clang::Decl*)"},
    {"_ZN5clang13ASTDeclReader9VisitDeclEPNS_4DeclE", "clang::ASTDeclReader::VisitDecl(clang::Decl*)"},
    {"_ZN5clang9ASTReader14ReadAttributesERNS0_11PerFileDataERN4llvm11SmallVectorIPNS_4AttrELj2EEERKNS4_IyLj64EEERj", "clang::ASTReader::ReadAttributes(clang::ASTReader::PerFileData&, llvm::SmallVector<clang::Attr*, 2u>&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13ASTDeclReader24VisitTranslationUnitDeclEPNS_19TranslationUnitDeclE", "clang::ASTDeclReader::VisitTranslationUnitDecl(clang::TranslationUnitDecl*)"},
    {"_ZN5clang13ASTDeclReader14VisitNamedDeclEPNS_9NamedDeclE", "clang::ASTDeclReader::VisitNamedDecl(clang::NamedDecl*)"},
    {"_ZN5clang13ASTDeclReader13VisitTypeDeclEPNS_8TypeDeclE", "clang::ASTDeclReader::VisitTypeDecl(clang::TypeDecl*)"},
    {"_ZN5clang13ASTDeclReader16VisitTypedefDeclEPNS_11TypedefDeclE", "clang::ASTDeclReader::VisitTypedefDecl(clang::TypedefDecl*)"},
    {"_ZN5clang13ASTDeclReader12VisitTagDeclEPNS_7TagDeclE", "clang::ASTDeclReader::VisitTagDecl(clang::TagDecl*)"},
    {"_ZN5clang13ASTDeclReader13VisitEnumDeclEPNS_8EnumDeclE", "clang::ASTDeclReader::VisitEnumDecl(clang::EnumDecl*)"},
    {"_ZN5clang13ASTDeclReader15VisitRecordDeclEPNS_10RecordDeclE", "clang::ASTDeclReader::VisitRecordDecl(clang::RecordDecl*)"},
    {"_ZN5clang13ASTDeclReader14VisitValueDeclEPNS_9ValueDeclE", "clang::ASTDeclReader::VisitValueDecl(clang::ValueDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitEnumConstantDeclEPNS_16EnumConstantDeclE", "clang::ASTDeclReader::VisitEnumConstantDecl(clang::EnumConstantDecl*)"},
    {"_ZN5clang13ASTDeclReader19VisitDeclaratorDeclEPNS_14DeclaratorDeclE", "clang::ASTDeclReader::VisitDeclaratorDecl(clang::DeclaratorDecl*)"},
    {"_ZN5clang13ASTDeclReader17VisitFunctionDeclEPNS_12FunctionDeclE", "clang::ASTDeclReader::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN5clang13ASTDeclReader19VisitObjCMethodDeclEPNS_14ObjCMethodDeclE", "clang::ASTDeclReader::VisitObjCMethodDecl(clang::ObjCMethodDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitObjCContainerDeclEPNS_17ObjCContainerDeclE", "clang::ASTDeclReader::VisitObjCContainerDecl(clang::ObjCContainerDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitObjCInterfaceDeclEPNS_17ObjCInterfaceDeclE", "clang::ASTDeclReader::VisitObjCInterfaceDecl(clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang13ASTDeclReader17VisitObjCIvarDeclEPNS_12ObjCIvarDeclE", "clang::ASTDeclReader::VisitObjCIvarDecl(clang::ObjCIvarDecl*)"},
    {"_ZN5clang13ASTDeclReader14VisitFieldDeclEPNS_9FieldDeclE", "clang::ASTDeclReader::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitObjCProtocolDeclEPNS_16ObjCProtocolDeclE", "clang::ASTDeclReader::VisitObjCProtocolDecl(clang::ObjCProtocolDecl*)"},
    {"_ZN5clang13ASTDeclReader24VisitObjCAtDefsFieldDeclEPNS_19ObjCAtDefsFieldDeclE", "clang::ASTDeclReader::VisitObjCAtDefsFieldDecl(clang::ObjCAtDefsFieldDecl*)"},
    {"_ZN5clang13ASTDeclReader18VisitObjCClassDeclEPNS_13ObjCClassDeclE", "clang::ASTDeclReader::VisitObjCClassDecl(clang::ObjCClassDecl*)"},
    {"_ZN5clang13ASTDeclReader28VisitObjCForwardProtocolDeclEPNS_23ObjCForwardProtocolDeclE", "clang::ASTDeclReader::VisitObjCForwardProtocolDecl(clang::ObjCForwardProtocolDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitObjCCategoryDeclEPNS_16ObjCCategoryDeclE", "clang::ASTDeclReader::VisitObjCCategoryDecl(clang::ObjCCategoryDecl*)"},
    {"_ZN5clang13ASTDeclReader28VisitObjCCompatibleAliasDeclEPNS_23ObjCCompatibleAliasDeclE", "clang::ASTDeclReader::VisitObjCCompatibleAliasDecl(clang::ObjCCompatibleAliasDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitObjCPropertyDeclEPNS_16ObjCPropertyDeclE", "clang::ASTDeclReader::VisitObjCPropertyDecl(clang::ObjCPropertyDecl*)"},
    {"_ZN5clang13ASTDeclReader17VisitObjCImplDeclEPNS_12ObjCImplDeclE", "clang::ASTDeclReader::VisitObjCImplDecl(clang::ObjCImplDecl*)"},
    {"_ZN5clang13ASTDeclReader25VisitObjCCategoryImplDeclEPNS_20ObjCCategoryImplDeclE", "clang::ASTDeclReader::VisitObjCCategoryImplDecl(clang::ObjCCategoryImplDecl*)"},
    {"_ZN5clang13ASTDeclReader27VisitObjCImplementationDeclEPNS_22ObjCImplementationDeclE", "clang::ASTDeclReader::VisitObjCImplementationDecl(clang::ObjCImplementationDecl*)"},
    {"_ZN5clang13ASTDeclReader25VisitObjCPropertyImplDeclEPNS_20ObjCPropertyImplDeclE", "clang::ASTDeclReader::VisitObjCPropertyImplDecl(clang::ObjCPropertyImplDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitIndirectFieldDeclEPNS_17IndirectFieldDeclE", "clang::ASTDeclReader::VisitIndirectFieldDecl(clang::IndirectFieldDecl*)"},
    {"_ZN5clang13ASTDeclReader12VisitVarDeclEPNS_7VarDeclE", "clang::ASTDeclReader::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitImplicitParamDeclEPNS_17ImplicitParamDeclE", "clang::ASTDeclReader::VisitImplicitParamDecl(clang::ImplicitParamDecl*)"},
    {"_ZN5clang13ASTDeclReader16VisitParmVarDeclEPNS_11ParmVarDeclE", "clang::ASTDeclReader::VisitParmVarDecl(clang::ParmVarDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitFileScopeAsmDeclEPNS_16FileScopeAsmDeclE", "clang::ASTDeclReader::VisitFileScopeAsmDecl(clang::FileScopeAsmDecl*)"},
    {"_ZN5clang13ASTDeclReader14VisitBlockDeclEPNS_9BlockDeclE", "clang::ASTDeclReader::VisitBlockDecl(clang::BlockDecl*)"},
    {"_ZN5clang13ASTDeclReader20VisitLinkageSpecDeclEPNS_15LinkageSpecDeclE", "clang::ASTDeclReader::VisitLinkageSpecDecl(clang::LinkageSpecDecl*)"},
    {"_ZN5clang13ASTDeclReader18VisitNamespaceDeclEPNS_13NamespaceDeclE", "clang::ASTDeclReader::VisitNamespaceDecl(clang::NamespaceDecl*)"},
    {"_ZN5clang13ASTDeclReader23VisitNamespaceAliasDeclEPNS_18NamespaceAliasDeclE", "clang::ASTDeclReader::VisitNamespaceAliasDecl(clang::NamespaceAliasDecl*)"},
    {"_ZN5clang13ASTDeclReader14VisitUsingDeclEPNS_9UsingDeclE", "clang::ASTDeclReader::VisitUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang13ASTDeclReader20VisitUsingShadowDeclEPNS_15UsingShadowDeclE", "clang::ASTDeclReader::VisitUsingShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang13ASTDeclReader23VisitUsingDirectiveDeclEPNS_18UsingDirectiveDeclE", "clang::ASTDeclReader::VisitUsingDirectiveDecl(clang::UsingDirectiveDecl*)"},
    {"_ZN5clang13ASTDeclReader29VisitUnresolvedUsingValueDeclEPNS_24UnresolvedUsingValueDeclE", "clang::ASTDeclReader::VisitUnresolvedUsingValueDecl(clang::UnresolvedUsingValueDecl*)"},
    {"_ZN5clang13ASTDeclReader32VisitUnresolvedUsingTypenameDeclEPNS_27UnresolvedUsingTypenameDeclE", "clang::ASTDeclReader::VisitUnresolvedUsingTypenameDecl(clang::UnresolvedUsingTypenameDecl*)"},
    {"_ZN5clang13ASTDeclReader21ReadCXXDefinitionDataERNS_13CXXRecordDecl14DefinitionDataERKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTDeclReader::ReadCXXDefinitionData(clang::CXXRecordDecl::DefinitionData&, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13ASTDeclReader27InitializeCXXDefinitionDataEPNS_13CXXRecordDeclES2_RKN4llvm11SmallVectorIyLj64EEERj", "clang::ASTDeclReader::InitializeCXXDefinitionData(clang::CXXRecordDecl*, clang::CXXRecordDecl*, llvm::SmallVector<unsigned long long, 64u> const&, unsigned int&)"},
    {"_ZN5clang13ASTDeclReader18VisitCXXRecordDeclEPNS_13CXXRecordDeclE", "clang::ASTDeclReader::VisitCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_ZN5clang13ASTDeclReader18VisitCXXMethodDeclEPNS_13CXXMethodDeclE", "clang::ASTDeclReader::VisitCXXMethodDecl(clang::CXXMethodDecl*)"},
    {"_ZN5clang13ASTDeclReader23VisitCXXConstructorDeclEPNS_18CXXConstructorDeclE", "clang::ASTDeclReader::VisitCXXConstructorDecl(clang::CXXConstructorDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitCXXDestructorDeclEPNS_17CXXDestructorDeclE", "clang::ASTDeclReader::VisitCXXDestructorDecl(clang::CXXDestructorDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitCXXConversionDeclEPNS_17CXXConversionDeclE", "clang::ASTDeclReader::VisitCXXConversionDecl(clang::CXXConversionDecl*)"},
    {"_ZN5clang13ASTDeclReader19VisitAccessSpecDeclEPNS_14AccessSpecDeclE", "clang::ASTDeclReader::VisitAccessSpecDecl(clang::AccessSpecDecl*)"},
    {"_ZN5clang13ASTDeclReader15VisitFriendDeclEPNS_10FriendDeclE", "clang::ASTDeclReader::VisitFriendDecl(clang::FriendDecl*)"},
    {"_ZN5clang13ASTDeclReader23VisitFriendTemplateDeclEPNS_18FriendTemplateDeclE", "clang::ASTDeclReader::VisitFriendTemplateDecl(clang::FriendTemplateDecl*)"},
    {"_ZN5clang13ASTDeclReader17VisitTemplateDeclEPNS_12TemplateDeclE", "clang::ASTDeclReader::VisitTemplateDecl(clang::TemplateDecl*)"},
    {"_ZN5clang13ASTDeclReader29VisitRedeclarableTemplateDeclEPNS_24RedeclarableTemplateDeclE", "clang::ASTDeclReader::VisitRedeclarableTemplateDecl(clang::RedeclarableTemplateDecl*)"},
    {"_ZN5clang13ASTDeclReader22VisitClassTemplateDeclEPNS_17ClassTemplateDeclE", "clang::ASTDeclReader::VisitClassTemplateDecl(clang::ClassTemplateDecl*)"},
    {"_ZN5clang13ASTDeclReader36VisitClassTemplateSpecializationDeclEPNS_31ClassTemplateSpecializationDeclE", "clang::ASTDeclReader::VisitClassTemplateSpecializationDecl(clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN5clang13ASTDeclReader43VisitClassTemplatePartialSpecializationDeclEPNS_38ClassTemplatePartialSpecializationDeclE", "clang::ASTDeclReader::VisitClassTemplatePartialSpecializationDecl(clang::ClassTemplatePartialSpecializationDecl*)"},
    {"_ZN5clang13ASTDeclReader25VisitFunctionTemplateDeclEPNS_20FunctionTemplateDeclE", "clang::ASTDeclReader::VisitFunctionTemplateDecl(clang::FunctionTemplateDecl*)"},
    {"_ZN5clang13ASTDeclReader25VisitTemplateTypeParmDeclEPNS_20TemplateTypeParmDeclE", "clang::ASTDeclReader::VisitTemplateTypeParmDecl(clang::TemplateTypeParmDecl*)"},
    {"_ZN5clang13ASTDeclReader28VisitNonTypeTemplateParmDeclEPNS_23NonTypeTemplateParmDeclE", "clang::ASTDeclReader::VisitNonTypeTemplateParmDecl(clang::NonTypeTemplateParmDecl*)"},
    {"_ZN5clang13ASTDeclReader29VisitTemplateTemplateParmDeclEPNS_24TemplateTemplateParmDeclE", "clang::ASTDeclReader::VisitTemplateTemplateParmDecl(clang::TemplateTemplateParmDecl*)"},
    {"_ZN5clang13ASTDeclReader21VisitStaticAssertDeclEPNS_16StaticAssertDeclE", "clang::ASTDeclReader::VisitStaticAssertDecl(clang::StaticAssertDecl*)"},
    {"_ZN5clang13ASTDeclReader16VisitDeclContextEPNS_11DeclContextE", "clang::ASTDeclReader::VisitDeclContext(clang::DeclContext*)"},
    {"_ZN5clang9ASTReader18DeclCursorForIndexEjj", "clang::ASTReader::DeclCursorForIndex(unsigned int, unsigned int)"},
    {"_ZN5clang9ASTReader14ReadDeclRecordEjj", "clang::ASTReader::ReadDeclRecord(unsigned int, unsigned int)"},
    {"_ZN5clang13ASTDeclReader10UpdateDeclEPNS_4DeclERKN4llvm11SmallVectorIyLj64EEE", "clang::ASTDeclReader::UpdateDecl(clang::Decl*, llvm::SmallVector<unsigned long long, 64u> const&)"},
    {"_ZN5clang11DeclVisitorINS_13ASTDeclReaderEvE5VisitEPNS_4DeclE", "clang::DeclVisitor<clang::ASTDeclReader, void>::Visit(clang::Decl*)"},
    {"_ZN5clang13ASTDeclReader17VisitRedeclarableINS_7TagDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclReader::VisitRedeclarable<clang::TagDecl>(clang::Redeclarable<clang::TagDecl>*)"},
    {"_ZN5clang13ASTDeclReader17VisitRedeclarableINS_12FunctionDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclReader::VisitRedeclarable<clang::FunctionDecl>(clang::Redeclarable<clang::FunctionDecl>*)"},
    {"_ZN5clang13ASTDeclReader17VisitRedeclarableINS_7VarDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclReader::VisitRedeclarable<clang::VarDecl>(clang::Redeclarable<clang::VarDecl>*)"},
    {"_ZN5clang14AccessSpecDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::AccessSpecDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN4llvm15SmallVectorImplIN5clang9ASTReader15DeclContextInfoEE6insertEPS3_RKS3_", "llvm::SmallVectorImpl<clang::ASTReader::DeclContextInfo>::insert(clang::ASTReader::DeclContextInfo*, clang::ASTReader::DeclContextInfo const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorIPS2_Lj4EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<clang::CXXRecordDecl*, 4u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl*, 4u> > >::FindAndConstruct(clang::CXXRecordDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorIPS2_Lj4EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<clang::CXXRecordDecl*, 4u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl*, 4u> > >::InsertIntoBucket(clang::CXXRecordDecl const* const&, llvm::SmallVector<clang::CXXRecordDecl*, 4u> const&, std::pair<clang::CXXRecordDecl const*, llvm::SmallVector<clang::CXXRecordDecl*, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorIPS2_Lj4EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<clang::CXXRecordDecl*, 4u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl*, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang13CXXRecordDeclEEaSERKS4_", "llvm::SmallVectorImpl<clang::CXXRecordDecl*>::operator=(llvm::SmallVectorImpl<clang::CXXRecordDecl*> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang9BlockDecl7CaptureELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::BlockDecl::Capture, false>::grow(unsigned long)"},
    {"_ZN5clang13ASTStmtReader32ReadExplicitTemplateArgumentListERNS_28ExplicitTemplateArgumentListEj", "clang::ASTStmtReader::ReadExplicitTemplateArgumentList(clang::ExplicitTemplateArgumentList&, unsigned int)"},
    {"_ZN5clang13ASTStmtReader9VisitStmtEPNS_4StmtE", "clang::ASTStmtReader::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang13ASTStmtReader13VisitNullStmtEPNS_8NullStmtE", "clang::ASTStmtReader::VisitNullStmt(clang::NullStmt*)"},
    {"_ZN5clang13ASTStmtReader17VisitCompoundStmtEPNS_12CompoundStmtE", "clang::ASTStmtReader::VisitCompoundStmt(clang::CompoundStmt*)"},
    {"_ZN5clang13ASTStmtReader15VisitSwitchCaseEPNS_10SwitchCaseE", "clang::ASTStmtReader::VisitSwitchCase(clang::SwitchCase*)"},
    {"_ZN5clang13ASTStmtReader13VisitCaseStmtEPNS_8CaseStmtE", "clang::ASTStmtReader::VisitCaseStmt(clang::CaseStmt*)"},
    {"_ZN5clang9ASTReader11ReadSubExprEv", "clang::ASTReader::ReadSubExpr()"},
    {"_ZN5clang13ASTStmtReader16VisitDefaultStmtEPNS_11DefaultStmtE", "clang::ASTStmtReader::VisitDefaultStmt(clang::DefaultStmt*)"},
    {"_ZN5clang13ASTStmtReader14VisitLabelStmtEPNS_9LabelStmtE", "clang::ASTStmtReader::VisitLabelStmt(clang::LabelStmt*)"},
    {"_ZN5clang13ASTStmtReader11VisitIfStmtEPNS_6IfStmtE", "clang::ASTStmtReader::VisitIfStmt(clang::IfStmt*)"},
    {"_ZN5clang13ASTStmtReader15VisitSwitchStmtEPNS_10SwitchStmtE", "clang::ASTStmtReader::VisitSwitchStmt(clang::SwitchStmt*)"},
    {"_ZN5clang13ASTStmtReader14VisitWhileStmtEPNS_9WhileStmtE", "clang::ASTStmtReader::VisitWhileStmt(clang::WhileStmt*)"},
    {"_ZN5clang13ASTStmtReader11VisitDoStmtEPNS_6DoStmtE", "clang::ASTStmtReader::VisitDoStmt(clang::DoStmt*)"},
    {"_ZN5clang13ASTStmtReader12VisitForStmtEPNS_7ForStmtE", "clang::ASTStmtReader::VisitForStmt(clang::ForStmt*)"},
    {"_ZN5clang13ASTStmtReader13VisitGotoStmtEPNS_8GotoStmtE", "clang::ASTStmtReader::VisitGotoStmt(clang::GotoStmt*)"},
    {"_ZN5clang13ASTStmtReader21VisitIndirectGotoStmtEPNS_16IndirectGotoStmtE", "clang::ASTStmtReader::VisitIndirectGotoStmt(clang::IndirectGotoStmt*)"},
    {"_ZN5clang13ASTStmtReader17VisitContinueStmtEPNS_12ContinueStmtE", "clang::ASTStmtReader::VisitContinueStmt(clang::ContinueStmt*)"},
    {"_ZN5clang13ASTStmtReader14VisitBreakStmtEPNS_9BreakStmtE", "clang::ASTStmtReader::VisitBreakStmt(clang::BreakStmt*)"},
    {"_ZN5clang13ASTStmtReader15VisitReturnStmtEPNS_10ReturnStmtE", "clang::ASTStmtReader::VisitReturnStmt(clang::ReturnStmt*)"},
    {"_ZN5clang13ASTStmtReader13VisitDeclStmtEPNS_8DeclStmtE", "clang::ASTStmtReader::VisitDeclStmt(clang::DeclStmt*)"},
    {"_ZN5clang13ASTStmtReader12VisitAsmStmtEPNS_7AsmStmtE", "clang::ASTStmtReader::VisitAsmStmt(clang::AsmStmt*)"},
    {"_ZN5clang13ASTStmtReader9VisitExprEPNS_4ExprE", "clang::ASTStmtReader::VisitExpr(clang::Expr*)"},
    {"_ZN5clang13ASTStmtReader19VisitPredefinedExprEPNS_14PredefinedExprE", "clang::ASTStmtReader::VisitPredefinedExpr(clang::PredefinedExpr*)"},
    {"_ZN5clang13ASTStmtReader16VisitDeclRefExprEPNS_11DeclRefExprE", "clang::ASTStmtReader::VisitDeclRefExpr(clang::DeclRefExpr*)"},
    {"_ZN5clang13ASTStmtReader19VisitIntegerLiteralEPNS_14IntegerLiteralE", "clang::ASTStmtReader::VisitIntegerLiteral(clang::IntegerLiteral*)"},
    {"_ZN5clang13ASTStmtReader20VisitFloatingLiteralEPNS_15FloatingLiteralE", "clang::ASTStmtReader::VisitFloatingLiteral(clang::FloatingLiteral*)"},
    {"_ZN5clang13ASTStmtReader21VisitImaginaryLiteralEPNS_16ImaginaryLiteralE", "clang::ASTStmtReader::VisitImaginaryLiteral(clang::ImaginaryLiteral*)"},
    {"_ZN5clang13ASTStmtReader18VisitStringLiteralEPNS_13StringLiteralE", "clang::ASTStmtReader::VisitStringLiteral(clang::StringLiteral*)"},
    {"_ZN5clang13ASTStmtReader21VisitCharacterLiteralEPNS_16CharacterLiteralE", "clang::ASTStmtReader::VisitCharacterLiteral(clang::CharacterLiteral*)"},
    {"_ZN5clang13ASTStmtReader14VisitParenExprEPNS_9ParenExprE", "clang::ASTStmtReader::VisitParenExpr(clang::ParenExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitParenListExprEPNS_13ParenListExprE", "clang::ASTStmtReader::VisitParenListExpr(clang::ParenListExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitUnaryOperatorEPNS_13UnaryOperatorE", "clang::ASTStmtReader::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN5clang13ASTStmtReader17VisitOffsetOfExprEPNS_12OffsetOfExprE", "clang::ASTStmtReader::VisitOffsetOfExpr(clang::OffsetOfExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitSizeOfAlignOfExprEPNS_17SizeOfAlignOfExprE", "clang::ASTStmtReader::VisitSizeOfAlignOfExpr(clang::SizeOfAlignOfExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitArraySubscriptExprEPNS_18ArraySubscriptExprE", "clang::ASTStmtReader::VisitArraySubscriptExpr(clang::ArraySubscriptExpr*)"},
    {"_ZN5clang13ASTStmtReader13VisitCallExprEPNS_8CallExprE", "clang::ASTStmtReader::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN5clang13ASTStmtReader15VisitMemberExprEPNS_10MemberExprE", "clang::ASTStmtReader::VisitMemberExpr(clang::MemberExpr*)"},
    {"_ZN5clang13ASTStmtReader16VisitObjCIsaExprEPNS_11ObjCIsaExprE", "clang::ASTStmtReader::VisitObjCIsaExpr(clang::ObjCIsaExpr*)"},
    {"_ZN5clang13ASTStmtReader13VisitCastExprEPNS_8CastExprE", "clang::ASTStmtReader::VisitCastExpr(clang::CastExpr*)"},
    {"_ZN5clang13ASTStmtReader19VisitBinaryOperatorEPNS_14BinaryOperatorE", "clang::ASTStmtReader::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN5clang13ASTStmtReader27VisitCompoundAssignOperatorEPNS_22CompoundAssignOperatorE", "clang::ASTStmtReader::VisitCompoundAssignOperator(clang::CompoundAssignOperator*)"},
    {"_ZN5clang13ASTStmtReader24VisitConditionalOperatorEPNS_19ConditionalOperatorE", "clang::ASTStmtReader::VisitConditionalOperator(clang::ConditionalOperator*)"},
    {"_ZN5clang13ASTStmtReader21VisitImplicitCastExprEPNS_16ImplicitCastExprE", "clang::ASTStmtReader::VisitImplicitCastExpr(clang::ImplicitCastExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitExplicitCastExprEPNS_16ExplicitCastExprE", "clang::ASTStmtReader::VisitExplicitCastExpr(clang::ExplicitCastExpr*)"},
    {"_ZN5clang13ASTStmtReader19VisitCStyleCastExprEPNS_14CStyleCastExprE", "clang::ASTStmtReader::VisitCStyleCastExpr(clang::CStyleCastExpr*)"},
    {"_ZN5clang13ASTStmtReader24VisitCompoundLiteralExprEPNS_19CompoundLiteralExprE", "clang::ASTStmtReader::VisitCompoundLiteralExpr(clang::CompoundLiteralExpr*)"},
    {"_ZN5clang13ASTStmtReader25VisitExtVectorElementExprEPNS_20ExtVectorElementExprE", "clang::ASTStmtReader::VisitExtVectorElementExpr(clang::ExtVectorElementExpr*)"},
    {"_ZN5clang13ASTStmtReader17VisitInitListExprEPNS_12InitListExprE", "clang::ASTStmtReader::VisitInitListExpr(clang::InitListExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitDesignatedInitExprEPNS_18DesignatedInitExprE", "clang::ASTStmtReader::VisitDesignatedInitExpr(clang::DesignatedInitExpr*)"},
    {"_ZN5clang13ASTStmtReader26VisitImplicitValueInitExprEPNS_21ImplicitValueInitExprE", "clang::ASTStmtReader::VisitImplicitValueInitExpr(clang::ImplicitValueInitExpr*)"},
    {"_ZN5clang13ASTStmtReader14VisitVAArgExprEPNS_9VAArgExprE", "clang::ASTStmtReader::VisitVAArgExpr(clang::VAArgExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitAddrLabelExprEPNS_13AddrLabelExprE", "clang::ASTStmtReader::VisitAddrLabelExpr(clang::AddrLabelExpr*)"},
    {"_ZN5clang13ASTStmtReader13VisitStmtExprEPNS_8StmtExprE", "clang::ASTStmtReader::VisitStmtExpr(clang::StmtExpr*)"},
    {"_ZN5clang13ASTStmtReader15VisitChooseExprEPNS_10ChooseExprE", "clang::ASTStmtReader::VisitChooseExpr(clang::ChooseExpr*)"},
    {"_ZN5clang13ASTStmtReader16VisitGNUNullExprEPNS_11GNUNullExprE", "clang::ASTStmtReader::VisitGNUNullExpr(clang::GNUNullExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitShuffleVectorExprEPNS_17ShuffleVectorExprE", "clang::ASTStmtReader::VisitShuffleVectorExpr(clang::ShuffleVectorExpr*)"},
    {"_ZN5clang13ASTStmtReader14VisitBlockExprEPNS_9BlockExprE", "clang::ASTStmtReader::VisitBlockExpr(clang::BlockExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitBlockDeclRefExprEPNS_16BlockDeclRefExprE", "clang::ASTStmtReader::VisitBlockDeclRefExpr(clang::BlockDeclRefExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitObjCStringLiteralEPNS_17ObjCStringLiteralE", "clang::ASTStmtReader::VisitObjCStringLiteral(clang::ObjCStringLiteral*)"},
    {"_ZN5clang13ASTStmtReader19VisitObjCEncodeExprEPNS_14ObjCEncodeExprE", "clang::ASTStmtReader::VisitObjCEncodeExpr(clang::ObjCEncodeExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitObjCSelectorExprEPNS_16ObjCSelectorExprE", "clang::ASTStmtReader::VisitObjCSelectorExpr(clang::ObjCSelectorExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitObjCProtocolExprEPNS_16ObjCProtocolExprE", "clang::ASTStmtReader::VisitObjCProtocolExpr(clang::ObjCProtocolExpr*)"},
    {"_ZN5clang13ASTStmtReader20VisitObjCIvarRefExprEPNS_15ObjCIvarRefExprE", "clang::ASTStmtReader::VisitObjCIvarRefExpr(clang::ObjCIvarRefExpr*)"},
    {"_ZN5clang13ASTStmtReader24VisitObjCPropertyRefExprEPNS_19ObjCPropertyRefExprE", "clang::ASTStmtReader::VisitObjCPropertyRefExpr(clang::ObjCPropertyRefExpr*)"},
    {"_ZN5clang13ASTStmtReader20VisitObjCMessageExprEPNS_15ObjCMessageExprE", "clang::ASTStmtReader::VisitObjCMessageExpr(clang::ObjCMessageExpr*)"},
    {"_ZN5clang13ASTStmtReader26VisitObjCForCollectionStmtEPNS_21ObjCForCollectionStmtE", "clang::ASTStmtReader::VisitObjCForCollectionStmt(clang::ObjCForCollectionStmt*)"},
    {"_ZN5clang13ASTStmtReader20VisitObjCAtCatchStmtEPNS_15ObjCAtCatchStmtE", "clang::ASTStmtReader::VisitObjCAtCatchStmt(clang::ObjCAtCatchStmt*)"},
    {"_ZN5clang13ASTStmtReader22VisitObjCAtFinallyStmtEPNS_17ObjCAtFinallyStmtE", "clang::ASTStmtReader::VisitObjCAtFinallyStmt(clang::ObjCAtFinallyStmt*)"},
    {"_ZN5clang13ASTStmtReader18VisitObjCAtTryStmtEPNS_13ObjCAtTryStmtE", "clang::ASTStmtReader::VisitObjCAtTryStmt(clang::ObjCAtTryStmt*)"},
    {"_ZN5clang13ASTStmtReader27VisitObjCAtSynchronizedStmtEPNS_22ObjCAtSynchronizedStmtE", "clang::ASTStmtReader::VisitObjCAtSynchronizedStmt(clang::ObjCAtSynchronizedStmt*)"},
    {"_ZN5clang13ASTStmtReader20VisitObjCAtThrowStmtEPNS_15ObjCAtThrowStmtE", "clang::ASTStmtReader::VisitObjCAtThrowStmt(clang::ObjCAtThrowStmt*)"},
    {"_ZN5clang13ASTStmtReader17VisitCXXCatchStmtEPNS_12CXXCatchStmtE", "clang::ASTStmtReader::VisitCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN5clang13ASTStmtReader15VisitCXXTryStmtEPNS_10CXXTryStmtE", "clang::ASTStmtReader::VisitCXXTryStmt(clang::CXXTryStmt*)"},
    {"_ZN5clang13ASTStmtReader24VisitCXXOperatorCallExprEPNS_19CXXOperatorCallExprE", "clang::ASTStmtReader::VisitCXXOperatorCallExpr(clang::CXXOperatorCallExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitCXXConstructExprEPNS_16CXXConstructExprE", "clang::ASTStmtReader::VisitCXXConstructExpr(clang::CXXConstructExpr*)"},
    {"_ZN5clang13ASTStmtReader27VisitCXXTemporaryObjectExprEPNS_22CXXTemporaryObjectExprE", "clang::ASTStmtReader::VisitCXXTemporaryObjectExpr(clang::CXXTemporaryObjectExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitCXXNamedCastExprEPNS_16CXXNamedCastExprE", "clang::ASTStmtReader::VisitCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitCXXStaticCastExprEPNS_17CXXStaticCastExprE", "clang::ASTStmtReader::VisitCXXStaticCastExpr(clang::CXXStaticCastExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitCXXDynamicCastExprEPNS_18CXXDynamicCastExprE", "clang::ASTStmtReader::VisitCXXDynamicCastExpr(clang::CXXDynamicCastExpr*)"},
    {"_ZN5clang13ASTStmtReader27VisitCXXReinterpretCastExprEPNS_22CXXReinterpretCastExprE", "clang::ASTStmtReader::VisitCXXReinterpretCastExpr(clang::CXXReinterpretCastExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitCXXConstCastExprEPNS_16CXXConstCastExprE", "clang::ASTStmtReader::VisitCXXConstCastExpr(clang::CXXConstCastExpr*)"},
    {"_ZN5clang13ASTStmtReader26VisitCXXFunctionalCastExprEPNS_21CXXFunctionalCastExprE", "clang::ASTStmtReader::VisitCXXFunctionalCastExpr(clang::CXXFunctionalCastExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitCXXBoolLiteralExprEPNS_18CXXBoolLiteralExprE", "clang::ASTStmtReader::VisitCXXBoolLiteralExpr(clang::CXXBoolLiteralExpr*)"},
    {"_ZN5clang13ASTStmtReader26VisitCXXNullPtrLiteralExprEPNS_21CXXNullPtrLiteralExprE", "clang::ASTStmtReader::VisitCXXNullPtrLiteralExpr(clang::CXXNullPtrLiteralExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitCXXTypeidExprEPNS_13CXXTypeidExprE", "clang::ASTStmtReader::VisitCXXTypeidExpr(clang::CXXTypeidExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitCXXUuidofExprEPNS_13CXXUuidofExprE", "clang::ASTStmtReader::VisitCXXUuidofExpr(clang::CXXUuidofExpr*)"},
    {"_ZN5clang13ASTStmtReader16VisitCXXThisExprEPNS_11CXXThisExprE", "clang::ASTStmtReader::VisitCXXThisExpr(clang::CXXThisExpr*)"},
    {"_ZN5clang13ASTStmtReader17VisitCXXThrowExprEPNS_12CXXThrowExprE", "clang::ASTStmtReader::VisitCXXThrowExpr(clang::CXXThrowExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitCXXDefaultArgExprEPNS_17CXXDefaultArgExprE", "clang::ASTStmtReader::VisitCXXDefaultArgExpr(clang::CXXDefaultArgExpr*)"},
    {"_ZN5clang13ASTStmtReader25VisitCXXBindTemporaryExprEPNS_20CXXBindTemporaryExprE", "clang::ASTStmtReader::VisitCXXBindTemporaryExpr(clang::CXXBindTemporaryExpr*)"},
    {"_ZN5clang13ASTStmtReader27VisitCXXScalarValueInitExprEPNS_22CXXScalarValueInitExprE", "clang::ASTStmtReader::VisitCXXScalarValueInitExpr(clang::CXXScalarValueInitExpr*)"},
    {"_ZN5clang13ASTStmtReader15VisitCXXNewExprEPNS_10CXXNewExprE", "clang::ASTStmtReader::VisitCXXNewExpr(clang::CXXNewExpr*)"},
    {"_ZN5clang13ASTStmtReader18VisitCXXDeleteExprEPNS_13CXXDeleteExprE", "clang::ASTStmtReader::VisitCXXDeleteExpr(clang::CXXDeleteExpr*)"},
    {"_ZN5clang13ASTStmtReader28VisitCXXPseudoDestructorExprEPNS_23CXXPseudoDestructorExprE", "clang::ASTStmtReader::VisitCXXPseudoDestructorExpr(clang::CXXPseudoDestructorExpr*)"},
    {"_ZN5clang13ASTStmtReader21VisitExprWithCleanupsEPNS_16ExprWithCleanupsE", "clang::ASTStmtReader::VisitExprWithCleanups(clang::ExprWithCleanups*)"},
    {"_ZN5clang13ASTStmtReader32VisitCXXDependentScopeMemberExprEPNS_27CXXDependentScopeMemberExprE", "clang::ASTStmtReader::VisitCXXDependentScopeMemberExpr(clang::CXXDependentScopeMemberExpr*)"},
    {"_ZN5clang13ASTStmtReader30VisitDependentScopeDeclRefExprEPNS_25DependentScopeDeclRefExprE", "clang::ASTStmtReader::VisitDependentScopeDeclRefExpr(clang::DependentScopeDeclRefExpr*)"},
    {"_ZN5clang13ASTStmtReader31VisitCXXUnresolvedConstructExprEPNS_26CXXUnresolvedConstructExprE", "clang::ASTStmtReader::VisitCXXUnresolvedConstructExpr(clang::CXXUnresolvedConstructExpr*)"},
    {"_ZN5clang13ASTStmtReader17VisitOverloadExprEPNS_12OverloadExprE", "clang::ASTStmtReader::VisitOverloadExpr(clang::OverloadExpr*)"},
    {"_ZN5clang13ASTStmtReader25VisitUnresolvedMemberExprEPNS_20UnresolvedMemberExprE", "clang::ASTStmtReader::VisitUnresolvedMemberExpr(clang::UnresolvedMemberExpr*)"},
    {"_ZN5clang13ASTStmtReader25VisitUnresolvedLookupExprEPNS_20UnresolvedLookupExprE", "clang::ASTStmtReader::VisitUnresolvedLookupExpr(clang::UnresolvedLookupExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitUnaryTypeTraitExprEPNS_18UnaryTypeTraitExprE", "clang::ASTStmtReader::VisitUnaryTypeTraitExpr(clang::UnaryTypeTraitExpr*)"},
    {"_ZN5clang13ASTStmtReader24VisitBinaryTypeTraitExprEPNS_19BinaryTypeTraitExprE", "clang::ASTStmtReader::VisitBinaryTypeTraitExpr(clang::BinaryTypeTraitExpr*)"},
    {"_ZN5clang13ASTStmtReader20VisitCXXNoexceptExprEPNS_15CXXNoexceptExprE", "clang::ASTStmtReader::VisitCXXNoexceptExpr(clang::CXXNoexceptExpr*)"},
    {"_ZN5clang13ASTStmtReader22VisitPackExpansionExprEPNS_17PackExpansionExprE", "clang::ASTStmtReader::VisitPackExpansionExpr(clang::PackExpansionExpr*)"},
    {"_ZN5clang13ASTStmtReader19VisitSizeOfPackExprEPNS_14SizeOfPackExprE", "clang::ASTStmtReader::VisitSizeOfPackExpr(clang::SizeOfPackExpr*)"},
    {"_ZN5clang13ASTStmtReader37VisitSubstNonTypeTemplateParmPackExprEPNS_32SubstNonTypeTemplateParmPackExprE", "clang::ASTStmtReader::VisitSubstNonTypeTemplateParmPackExpr(clang::SubstNonTypeTemplateParmPackExpr*)"},
    {"_ZN5clang13ASTStmtReader20VisitOpaqueValueExprEPNS_15OpaqueValueExprE", "clang::ASTStmtReader::VisitOpaqueValueExpr(clang::OpaqueValueExpr*)"},
    {"_ZN5clang13ASTStmtReader23VisitCUDAKernelCallExprEPNS_18CUDAKernelCallExprE", "clang::ASTStmtReader::VisitCUDAKernelCallExpr(clang::CUDAKernelCallExpr*)"},
    {"_ZN5clang9ASTReader8ReadStmtERNS0_11PerFileDataE", "clang::ASTReader::ReadStmt(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang9ASTReader18ReadStmtFromStreamERNS0_11PerFileDataE", "clang::ASTReader::ReadStmtFromStream(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang9ASTReader8ReadExprERNS0_11PerFileDataE", "clang::ASTReader::ReadExpr(clang::ASTReader::PerFileData&)"},
    {"_ZN5clang11StmtVisitorINS_13ASTStmtReaderEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<clang::ASTStmtReader, void>::Visit(clang::Stmt*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang18DesignatedInitExpr10DesignatorELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::DesignatedInitExpr::Designator, false>::grow(unsigned long)"},
    {"_ZN5clang9ASTWriter19WriteBlockInfoBlockEv", "clang::ASTWriter::WriteBlockInfoBlock()"},
    {"_Z11EmitBlockIDjPKcRN4llvm15BitstreamWriterERNS1_15SmallVectorImplIyEE", "EmitBlockID(unsigned int, char const*, llvm::BitstreamWriter&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_Z12EmitRecordIDjPKcRN4llvm15BitstreamWriterERNS1_15SmallVectorImplIyEE", "EmitRecordID(unsigned int, char const*, llvm::BitstreamWriter&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter13WriteMetadataERNS_10ASTContextEPKc", "clang::ASTWriter::WriteMetadata(clang::ASTContext&, char const*)"},
    {"_ZN5clang9ASTWriter20WriteLanguageOptionsERKNS_11LangOptionsE", "clang::ASTWriter::WriteLanguageOptions(clang::LangOptions const&)"},
    {"_ZN5clang9ASTWriter14WriteStatCacheERNS_17MemorizeStatCallsE", "clang::ASTWriter::WriteStatCache(clang::MemorizeStatCalls&)"},
    {"_ZN5clang9ASTWriter23WriteSourceManagerBlockERNS_13SourceManagerERKNS_12PreprocessorEPKc", "clang::ASTWriter::WriteSourceManagerBlock(clang::SourceManager&, clang::Preprocessor const&, char const*)"},
    {"_ZN5clang9ASTWriter16AddIdentifierRefEPKNS_14IdentifierInfoERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddIdentifierRef(clang::IdentifierInfo const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter17WritePreprocessorERKNS_12PreprocessorE", "clang::ASTWriter::WritePreprocessor(clang::Preprocessor const&)"},
    {"_ZN5clang9ASTWriter20getMacroDefinitionIDEPNS_15MacroDefinitionE", "clang::ASTWriter::getMacroDefinitionID(clang::MacroDefinition*)"},
    {"_ZN5clang9ASTWriter23WritePreprocessorDetailERNS_19PreprocessingRecordE", "clang::ASTWriter::WritePreprocessorDetail(clang::PreprocessingRecord&)"},
    {"_ZN5clang9ASTWriter17AddSourceLocationENS_14SourceLocationERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddSourceLocation(clang::SourceLocation, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter29WritePragmaDiagnosticMappingsERKNS_10DiagnosticE", "clang::ASTWriter::WritePragmaDiagnosticMappings(clang::Diagnostic const&)"},
    {"_ZN5clang9ASTWriter9WriteTypeENS_8QualTypeE", "clang::ASTWriter::WriteType(clang::QualType)"},
    {"_ZN5clang9ASTWriter10AddTypeRefENS_8QualTypeERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTypeRef(clang::QualType, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN12_GLOBAL__N_113ASTTypeWriter15VisitVectorTypeEPKN5clang10VectorTypeE", "(anonymous namespace)::ASTTypeWriter::VisitVectorType(clang::VectorType const*)"},
    {"_ZN5clang9ASTWriter28WriteDeclContextLexicalBlockERNS_10ASTContextEPNS_11DeclContextE", "clang::ASTWriter::WriteDeclContextLexicalBlock(clang::ASTContext&, clang::DeclContext*)"},
    {"_ZN5clang9ASTWriter10GetDeclRefEPKNS_4DeclE", "clang::ASTWriter::GetDeclRef(clang::Decl const*)"},
    {"_ZN5clang9ASTWriter20WriteTypeDeclOffsetsEv", "clang::ASTWriter::WriteTypeDeclOffsets()"},
    {"_ZN5clang9ASTWriter14WriteSelectorsERNS_4SemaE", "clang::ASTWriter::WriteSelectors(clang::Sema&)"},
    {"_ZN5clang9ASTWriter28WriteReferencedSelectorsPoolERNS_4SemaE", "clang::ASTWriter::WriteReferencedSelectorsPool(clang::Sema&)"},
    {"_ZN5clang9ASTWriter14AddSelectorRefENS_8SelectorERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddSelectorRef(clang::Selector, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter20WriteIdentifierTableERNS_12PreprocessorE", "clang::ASTWriter::WriteIdentifierTable(clang::Preprocessor&)"},
    {"_ZN5clang9ASTWriter16getIdentifierRefEPKNS_14IdentifierInfoE", "clang::ASTWriter::getIdentifierRef(clang::IdentifierInfo const*)"},
    {"_ZN5clang9ASTWriter28WriteDeclContextVisibleBlockERNS_10ASTContextEPNS_11DeclContextE", "clang::ASTWriter::WriteDeclContextVisibleBlock(clang::ASTContext&, clang::DeclContext*)"},
    {"_ZN5clang31OnDiskChainedHashTableGeneratorIN12_GLOBAL__N_129ASTDeclContextNameLookupTraitEE6insertENS_15DeclarationNameERKNS_23DeclContextLookupResultERS2_", "clang::OnDiskChainedHashTableGenerator<(anonymous namespace)::ASTDeclContextNameLookupTrait>::insert(clang::DeclarationName, clang::DeclContextLookupResult const&, (anonymous namespace)::ASTDeclContextNameLookupTrait&)"},
    {"_ZN5clang31OnDiskChainedHashTableGeneratorIN12_GLOBAL__N_129ASTDeclContextNameLookupTraitEE4EmitERN4llvm11raw_ostreamERS2_", "clang::OnDiskChainedHashTableGenerator<(anonymous namespace)::ASTDeclContextNameLookupTrait>::Emit(llvm::raw_ostream&, (anonymous namespace)::ASTDeclContextNameLookupTrait&)"},
    {"_ZN5clang9ASTWriter29WriteDeclContextVisibleUpdateEPKNS_11DeclContextE", "clang::ASTWriter::WriteDeclContextVisibleUpdate(clang::DeclContext const*)"},
    {"_ZN5clang9ASTWriter9getDeclIDEPKNS_4DeclE", "clang::ASTWriter::getDeclID(clang::Decl const*)"},
    {"_ZN5clang9ASTWriter15WriteAttributesERKN4llvm11SmallVectorIPNS_4AttrELj2EEERNS1_15SmallVectorImplIyEE", "clang::ASTWriter::WriteAttributes(llvm::SmallVector<clang::Attr*, 2u> const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter9AddStringEN4llvm9StringRefERNS1_15SmallVectorImplIyEE", "clang::ASTWriter::AddString(llvm::StringRef, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter17AddTypeSourceInfoEPNS_14TypeSourceInfoERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTypeSourceInfo(clang::TypeSourceInfo*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter10AddDeclRefEPKNS_4DeclERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddDeclRef(clang::Decl const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter19SetIdentifierOffsetEPKNS_14IdentifierInfoEj", "clang::ASTWriter::SetIdentifierOffset(clang::IdentifierInfo const*, unsigned int)"},
    {"_ZN5clang9ASTWriter17SetSelectorOffsetENS_8SelectorEj", "clang::ASTWriter::SetSelectorOffset(clang::Selector, unsigned int)"},
    {"_ZN5clang9ASTWriterC1ERN4llvm15BitstreamWriterE", "clang::ASTWriter::ASTWriter(llvm::BitstreamWriter&)"},
    {"_ZN5clang9ASTWriterC2ERN4llvm15BitstreamWriterE", "clang::ASTWriter::ASTWriter(llvm::BitstreamWriter&)"},
    {"_ZN5clang9ASTWriter8WriteASTERNS_4SemaEPNS_17MemorizeStatCallsEPKc", "clang::ASTWriter::WriteAST(clang::Sema&, clang::MemorizeStatCalls*, char const*)"},
    {"_ZN5clang9ASTWriter13WriteASTChainERNS_4SemaEPNS_17MemorizeStatCallsEPKc", "clang::ASTWriter::WriteASTChain(clang::Sema&, clang::MemorizeStatCalls*, char const*)"},
    {"_ZN5clang9ASTWriter12WriteASTCoreERNS_4SemaEPNS_17MemorizeStatCallsEPKc", "clang::ASTWriter::WriteASTCore(clang::Sema&, clang::MemorizeStatCalls*, char const*)"},
    {"_ZN5clang9ASTWriter22WriteDeclUpdatesBlocksEv", "clang::ASTWriter::WriteDeclUpdatesBlocks()"},
    {"_ZN5clang9ASTWriter26WriteDeclReplacementsBlockEv", "clang::ASTWriter::WriteDeclReplacementsBlock()"},
    {"_ZN5clang9ASTWriter14AddSourceRangeENS_11SourceRangeERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddSourceRange(clang::SourceRange, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter8AddAPIntERKN4llvm5APIntERNS1_15SmallVectorImplIyEE", "clang::ASTWriter::AddAPInt(llvm::APInt const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter9AddAPSIntERKN4llvm6APSIntERNS1_15SmallVectorImplIyEE", "clang::ASTWriter::AddAPSInt(llvm::APSInt const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter10AddAPFloatERKN4llvm7APFloatERNS1_15SmallVectorImplIyEE", "clang::ASTWriter::AddAPFloat(llvm::APFloat const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter14getSelectorRefENS_8SelectorE", "clang::ASTWriter::getSelectorRef(clang::Selector)"},
    {"_ZN5clang9ASTWriter15AddCXXTemporaryEPKNS_12CXXTemporaryERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddCXXTemporary(clang::CXXTemporary const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter23AddCXXBaseSpecifiersRefEPKNS_16CXXBaseSpecifierES3_RN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddCXXBaseSpecifiersRef(clang::CXXBaseSpecifier const*, clang::CXXBaseSpecifier const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter26AddTemplateArgumentLocInfoENS_16TemplateArgument7ArgKindERKNS_23TemplateArgumentLocInfoERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateArgumentLocInfo(clang::TemplateArgument::ArgKind, clang::TemplateArgumentLocInfo const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter22AddTemplateArgumentLocERKNS_19TemplateArgumentLocERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateArgumentLoc(clang::TemplateArgumentLoc const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter19AddTemplateArgumentERKNS_16TemplateArgumentERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateArgument(clang::TemplateArgument const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter17GetOrCreateTypeIDENS_8QualTypeE", "clang::ASTWriter::GetOrCreateTypeID(clang::QualType)"},
    {"_ZN5clang9ASTWriter18GetOrCreateTypeIdxENS_8QualTypeE", "clang::ASTWriter::GetOrCreateTypeIdx(clang::QualType)"},
    {"_ZNK5clang9ASTWriter9getTypeIDENS_8QualTypeE", "clang::ASTWriter::getTypeID(clang::QualType) const"},
    {"_ZNK5clang9ASTWriter10getTypeIdxENS_8QualTypeE", "clang::ASTWriter::getTypeIdx(clang::QualType) const"},
    {"_ZN5clang9ASTWriter18AddDeclarationNameENS_15DeclarationNameERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddDeclarationName(clang::DeclarationName, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter21AddDeclarationNameLocERKNS_18DeclarationNameLocENS_15DeclarationNameERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddDeclarationNameLoc(clang::DeclarationNameLoc const&, clang::DeclarationName, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter22AddDeclarationNameInfoERKNS_19DeclarationNameInfoERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddDeclarationNameInfo(clang::DeclarationNameInfo const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter16AddQualifierInfoERKNS_13QualifierInfoERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddQualifierInfo(clang::QualifierInfo const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter22AddNestedNameSpecifierEPNS_19NestedNameSpecifierERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddNestedNameSpecifier(clang::NestedNameSpecifier*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter24AddTemplateParameterListEPKNS_21TemplateParameterListERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateParameterList(clang::TemplateParameterList const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter15AddTemplateNameENS_12TemplateNameERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateName(clang::TemplateName, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter23AddTemplateArgumentListEPKNS_20TemplateArgumentListERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddTemplateArgumentList(clang::TemplateArgumentList const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter16AddUnresolvedSetERKNS_17UnresolvedSetImplERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddUnresolvedSet(clang::UnresolvedSetImpl const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter19AddCXXBaseSpecifierERKNS_16CXXBaseSpecifierERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddCXXBaseSpecifier(clang::CXXBaseSpecifier const&, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter22FlushCXXBaseSpecifiersEv", "clang::ASTWriter::FlushCXXBaseSpecifiers()"},
    {"_ZN5clang9ASTWriter22AddCXXCtorInitializersEPKPKNS_18CXXCtorInitializerEjRN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddCXXCtorInitializers(clang::CXXCtorInitializer const* const*, unsigned int, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter20AddCXXDefinitionDataEPKNS_13CXXRecordDeclERN4llvm15SmallVectorImplIyEE", "clang::ASTWriter::AddCXXDefinitionData(clang::CXXRecordDecl const*, llvm::SmallVectorImpl<unsigned long long>&)"},
    {"_ZN5clang9ASTWriter17ReaderInitializedEPNS_9ASTReaderE", "clang::ASTWriter::ReaderInitialized(clang::ASTReader*)"},
    {"_ZN5clang9ASTWriter14IdentifierReadEjPNS_14IdentifierInfoE", "clang::ASTWriter::IdentifierRead(unsigned int, clang::IdentifierInfo*)"},
    {"_ZN5clang9ASTWriter8TypeReadENS_13serialization7TypeIdxENS_8QualTypeE", "clang::ASTWriter::TypeRead(clang::serialization::TypeIdx, clang::QualType)"},
    {"_ZN5clang9ASTWriter8DeclReadEjPKNS_4DeclE", "clang::ASTWriter::DeclRead(unsigned int, clang::Decl const*)"},
    {"_ZN5clang9ASTWriter12SelectorReadEjNS_8SelectorE", "clang::ASTWriter::SelectorRead(unsigned int, clang::Selector)"},
    {"_ZN5clang9ASTWriter19MacroDefinitionReadEjPNS_15MacroDefinitionE", "clang::ASTWriter::MacroDefinitionRead(unsigned int, clang::MacroDefinition*)"},
    {"_ZThn8_N5clang9ASTWriter22CompletedTagDefinitionEPKNS_7TagDeclE", "non-virtual thunk to clang::ASTWriter::CompletedTagDefinition(clang::TagDecl const*)"},
    {"_ZN5clang9ASTWriter22CompletedTagDefinitionEPKNS_7TagDeclE", "clang::ASTWriter::CompletedTagDefinition(clang::TagDecl const*)"},
    {"_ZThn8_N5clang9ASTWriter16AddedVisibleDeclEPKNS_11DeclContextEPKNS_4DeclE", "non-virtual thunk to clang::ASTWriter::AddedVisibleDecl(clang::DeclContext const*, clang::Decl const*)"},
    {"_ZN5clang9ASTWriter16AddedVisibleDeclEPKNS_11DeclContextEPKNS_4DeclE", "clang::ASTWriter::AddedVisibleDecl(clang::DeclContext const*, clang::Decl const*)"},
    {"_ZThn8_N5clang9ASTWriter22AddedCXXImplicitMemberEPKNS_13CXXRecordDeclEPKNS_4DeclE", "non-virtual thunk to clang::ASTWriter::AddedCXXImplicitMember(clang::CXXRecordDecl const*, clang::Decl const*)"},
    {"_ZN5clang9ASTWriter22AddedCXXImplicitMemberEPKNS_13CXXRecordDeclEPKNS_4DeclE", "clang::ASTWriter::AddedCXXImplicitMember(clang::CXXRecordDecl const*, clang::Decl const*)"},
    {"_ZThn8_N5clang9ASTWriter30AddedCXXTemplateSpecializationEPKNS_17ClassTemplateDeclEPKNS_31ClassTemplateSpecializationDeclE", "non-virtual thunk to clang::ASTWriter::AddedCXXTemplateSpecialization(clang::ClassTemplateDecl const*, clang::ClassTemplateSpecializationDecl const*)"},
    {"_ZN5clang9ASTWriter30AddedCXXTemplateSpecializationEPKNS_17ClassTemplateDeclEPKNS_31ClassTemplateSpecializationDeclE", "clang::ASTWriter::AddedCXXTemplateSpecialization(clang::ClassTemplateDecl const*, clang::ClassTemplateSpecializationDecl const*)"},
    {"_ZN5clang24ASTSerializationListenerD0Ev", "clang::ASTSerializationListener::~ASTSerializationListener()"},
    {"_ZN5clang24ASTSerializationListenerD1Ev", "clang::ASTSerializationListener::~ASTSerializationListener()"},
    {"_ZN5clang24ASTSerializationListenerD2Ev", "clang::ASTSerializationListener::~ASTSerializationListener()"},
    {"_ZN12_GLOBAL__N_113TypeLocWriter20VisitFunctionTypeLocEN5clang15FunctionTypeLocE", "(anonymous namespace)::TypeLocWriter::VisitFunctionTypeLoc(clang::FunctionTypeLoc)"},
    {"_ZN12_GLOBAL__N_113TypeLocWriter17VisitArrayTypeLocEN5clang12ArrayTypeLocE", "(anonymous namespace)::TypeLocWriter::VisitArrayTypeLoc(clang::ArrayTypeLoc)"},
    {"_ZN12_GLOBAL__N_113ASTTypeWriter12VisitTagTypeEPKN5clang7TagTypeE", "(anonymous namespace)::ASTTypeWriter::VisitTagType(clang::TagType const*)"},
    {"_ZN12_GLOBAL__N_113ASTTypeWriter17VisitFunctionTypeEPKN5clang12FunctionTypeE", "(anonymous namespace)::ASTTypeWriter::VisitFunctionType(clang::FunctionType const*)"},
    {"_ZN12_GLOBAL__N_113ASTTypeWriter14VisitArrayTypeEPKN5clang9ArrayTypeE", "(anonymous namespace)::ASTTypeWriter::VisitArrayType(clang::ArrayType const*)"},
    {"_ZN4llvm15BitstreamWriter13EnterSubblockEjj", "llvm::BitstreamWriter::EnterSubblock(unsigned int, unsigned int)"},
    {"_ZN4llvm15BitstreamWriter9ExitBlockEv", "llvm::BitstreamWriter::ExitBlock()"},
    {"_ZN4llvm15BitstreamWriter10EmitRecordIyEEvjRNS_15SmallVectorImplIT_EEj", "void llvm::BitstreamWriter::EmitRecord<unsigned long long>(unsigned int, llvm::SmallVectorImpl<unsigned long long>&, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIyE6insertIPKcEEPyS5_T_S6_", "unsigned long long* llvm::SmallVectorImpl<unsigned long long>::insert<char const*>(unsigned long long*, char const*, char const*)"},
    {"_ZN4llvm15BitstreamWriter4EmitEjj", "llvm::BitstreamWriter::Emit(unsigned int, unsigned int)"},
    {"_ZN5clang13serialization10MakeTypeIDISt9binder1stISt10mem_fun1_tINS0_7TypeIdxENS_9ASTWriterENS_8QualTypeEEEEEjS6_T_", "unsigned int clang::serialization::MakeTypeID<std::binder1st<std::mem_fun1_t<clang::serialization::TypeIdx, clang::ASTWriter, clang::QualType> > >(clang::QualType, std::binder1st<std::mem_fun1_t<clang::serialization::TypeIdx, clang::ASTWriter, clang::QualType> >)"},
    {"_ZN5clang13serialization10MakeTypeIDISt9binder1stISt16const_mem_fun1_tINS0_7TypeIdxENS_9ASTWriterENS_8QualTypeEEEEEjS6_T_", "unsigned int clang::serialization::MakeTypeID<std::binder1st<std::const_mem_fun1_t<clang::serialization::TypeIdx, clang::ASTWriter, clang::QualType> > >(clang::QualType, std::binder1st<std::const_mem_fun1_t<clang::serialization::TypeIdx, clang::ASTWriter, clang::QualType> >)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_11SmallVectorIyLj2EEENS_12DenseMapInfoIS4_EENS7_IS6_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::Decl const*, llvm::SmallVector<unsigned long long, 2u>, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned long long, 2u> > >::FindAndConstruct(clang::Decl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_11SmallVectorIyLj2EEENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Decl const*, llvm::SmallVector<unsigned long long, 2u>, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned long long, 2u> > >::InsertIntoBucket(clang::Decl const* const&, llvm::SmallVector<unsigned long long, 2u> const&, std::pair<clang::Decl const*, llvm::SmallVector<unsigned long long, 2u> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_11SmallVectorIyLj2EEENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Decl const*, llvm::SmallVector<unsigned long long, 2u>, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned long long, 2u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIyEaSERKS1_", "llvm::SmallVectorImpl<unsigned long long>::operator=(llvm::SmallVectorImpl<unsigned long long> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang9ASTWriter23QueuedCXXBaseSpecifiersELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ASTWriter::QueuedCXXBaseSpecifiers, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPKN5clang15MacroDefinitionEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::MacroDefinition const*, unsigned int, llvm::DenseMapInfo<clang::MacroDefinition const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::MacroDefinition const* const&, unsigned int const&, std::pair<clang::MacroDefinition const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang15MacroDefinitionEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::MacroDefinition const*, unsigned int, llvm::DenseMapInfo<clang::MacroDefinition const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt5dequeIN5clang9ASTWriter10DeclOrTypeESaIS2_EE9push_backERKS2_", "std::deque<clang::ASTWriter::DeclOrType, std::allocator<clang::ASTWriter::DeclOrType> >::push_back(clang::ASTWriter::DeclOrType const&)"},
    {"_ZNSt5dequeIN5clang9ASTWriter10DeclOrTypeESaIS2_EE17_M_reallocate_mapEmb", "std::deque<clang::ASTWriter::DeclOrType, std::allocator<clang::ASTWriter::DeclOrType> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIN5clang9ASTWriter10DeclOrTypeESaIS2_EE17_M_initialize_mapEm", "std::_Deque_base<clang::ASTWriter::DeclOrType, std::allocator<clang::ASTWriter::DeclOrType> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeIN5clang9ASTWriter10DeclOrTypeESaIS2_EEC2ERKS4_", "std::deque<clang::ASTWriter::DeclOrType, std::allocator<clang::ASTWriter::DeclOrType> >::deque(std::deque<clang::ASTWriter::DeclOrType, std::allocator<clang::ASTWriter::DeclOrType> > const&)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<clang::Selector, unsigned int, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::Selector const&, unsigned int const&, std::pair<clang::Selector, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<clang::Selector, unsigned int, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang4DeclEE6appendINS1_18IdentifierResolver8iteratorEEEvT_S8_", "void llvm::SmallVectorImpl<clang::Decl*>::append<clang::IdentifierResolver::iterator>(clang::IdentifierResolver::iterator, clang::IdentifierResolver::iterator)"},
    {"_ZN4llvm15BitstreamWriter24EmitRecordWithAbbrevImplIyEEvjRNS_15SmallVectorImplIT_EENS_9StringRefE", "void llvm::BitstreamWriter::EmitRecordWithAbbrevImpl<unsigned long long>(unsigned int, llvm::SmallVectorImpl<unsigned long long>&, llvm::StringRef)"},
    {"_ZN4llvm15BitstreamWriter20EmitAbbreviatedFieldIhEEvRKNS_15BitCodeAbbrevOpET_", "void llvm::BitstreamWriter::EmitAbbreviatedField<unsigned char>(llvm::BitCodeAbbrevOp const&, unsigned char)"},
    {"_ZN4llvm15BitstreamWriter20EmitAbbreviatedFieldIyEEvRKNS_15BitCodeAbbrevOpET_", "void llvm::BitstreamWriter::EmitAbbreviatedField<unsigned long long>(llvm::BitCodeAbbrevOp const&, unsigned long long)"},
    {"_ZN4llvm15BitstreamWriter11FlushToWordEv", "llvm::BitstreamWriter::FlushToWord()"},
    {"_ZNSt6vectorIhSaIhEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPhS1_EERKh", "std::vector<unsigned char, std::allocator<unsigned char> >::_M_insert_aux(__gnu_cxx::__normal_iterator<unsigned char*, std::vector<unsigned char, std::allocator<unsigned char> > >, unsigned char const&)"},
    {"_ZN4llvm15BitstreamWriter9EmitVBR64Eyj", "llvm::BitstreamWriter::EmitVBR64(unsigned long long, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIyE6insertEPyRKy", "llvm::SmallVectorImpl<unsigned long long>::insert(unsigned long long*, unsigned long long const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang14IdentifierInfoEyNS_12DenseMapInfoIS4_EENS5_IyEEE16InsertIntoBucketERKS4_RKyPSt4pairIS4_yE", "llvm::DenseMap<clang::IdentifierInfo const*, unsigned long long, llvm::DenseMapInfo<clang::IdentifierInfo const*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(clang::IdentifierInfo const* const&, unsigned long long const&, std::pair<clang::IdentifierInfo const*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang14IdentifierInfoEyNS_12DenseMapInfoIS4_EENS5_IyEEE4growEj", "llvm::DenseMap<clang::IdentifierInfo const*, unsigned long long, llvm::DenseMapInfo<clang::IdentifierInfo const*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm15BitstreamWriter12EncodeAbbrevEPNS_13BitCodeAbbrevE", "llvm::BitstreamWriter::EncodeAbbrev(llvm::BitCodeAbbrev*)"},
    {"_ZNSt6vectorIN4llvm15BitstreamWriter5BlockESaIS2_EE9push_backERKS2_", "std::vector<llvm::BitstreamWriter::Block, std::allocator<llvm::BitstreamWriter::Block> >::push_back(llvm::BitstreamWriter::Block const&)"},
    {"_ZNSt6vectorIN4llvm15BitstreamWriter5BlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BitstreamWriter::Block, std::allocator<llvm::BitstreamWriter::Block> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BitstreamWriter::Block*, std::vector<llvm::BitstreamWriter::Block, std::allocator<llvm::BitstreamWriter::Block> > >, llvm::BitstreamWriter::Block const&)"},
    {"_ZN5clang13ASTDeclWriter5VisitEPNS_4DeclE", "clang::ASTDeclWriter::Visit(clang::Decl*)"},
    {"_ZN5clang13ASTDeclWriter9VisitDeclEPNS_4DeclE", "clang::ASTDeclWriter::VisitDecl(clang::Decl*)"},
    {"_ZN5clang13ASTDeclWriter24VisitTranslationUnitDeclEPNS_19TranslationUnitDeclE", "clang::ASTDeclWriter::VisitTranslationUnitDecl(clang::TranslationUnitDecl*)"},
    {"_ZN5clang13ASTDeclWriter14VisitNamedDeclEPNS_9NamedDeclE", "clang::ASTDeclWriter::VisitNamedDecl(clang::NamedDecl*)"},
    {"_ZN5clang13ASTDeclWriter13VisitTypeDeclEPNS_8TypeDeclE", "clang::ASTDeclWriter::VisitTypeDecl(clang::TypeDecl*)"},
    {"_ZN5clang13ASTDeclWriter16VisitTypedefDeclEPNS_11TypedefDeclE", "clang::ASTDeclWriter::VisitTypedefDecl(clang::TypedefDecl*)"},
    {"_ZN5clang13ASTDeclWriter12VisitTagDeclEPNS_7TagDeclE", "clang::ASTDeclWriter::VisitTagDecl(clang::TagDecl*)"},
    {"_ZN5clang13ASTDeclWriter13VisitEnumDeclEPNS_8EnumDeclE", "clang::ASTDeclWriter::VisitEnumDecl(clang::EnumDecl*)"},
    {"_ZN5clang13ASTDeclWriter15VisitRecordDeclEPNS_10RecordDeclE", "clang::ASTDeclWriter::VisitRecordDecl(clang::RecordDecl*)"},
    {"_ZN5clang13ASTDeclWriter14VisitValueDeclEPNS_9ValueDeclE", "clang::ASTDeclWriter::VisitValueDecl(clang::ValueDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitEnumConstantDeclEPNS_16EnumConstantDeclE", "clang::ASTDeclWriter::VisitEnumConstantDecl(clang::EnumConstantDecl*)"},
    {"_ZN5clang13ASTDeclWriter19VisitDeclaratorDeclEPNS_14DeclaratorDeclE", "clang::ASTDeclWriter::VisitDeclaratorDecl(clang::DeclaratorDecl*)"},
    {"_ZN5clang13ASTDeclWriter17VisitFunctionDeclEPNS_12FunctionDeclE", "clang::ASTDeclWriter::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN5clang13ASTDeclWriter19VisitObjCMethodDeclEPNS_14ObjCMethodDeclE", "clang::ASTDeclWriter::VisitObjCMethodDecl(clang::ObjCMethodDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitObjCContainerDeclEPNS_17ObjCContainerDeclE", "clang::ASTDeclWriter::VisitObjCContainerDecl(clang::ObjCContainerDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitObjCInterfaceDeclEPNS_17ObjCInterfaceDeclE", "clang::ASTDeclWriter::VisitObjCInterfaceDecl(clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang13ASTDeclWriter17VisitObjCIvarDeclEPNS_12ObjCIvarDeclE", "clang::ASTDeclWriter::VisitObjCIvarDecl(clang::ObjCIvarDecl*)"},
    {"_ZN5clang13ASTDeclWriter14VisitFieldDeclEPNS_9FieldDeclE", "clang::ASTDeclWriter::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitObjCProtocolDeclEPNS_16ObjCProtocolDeclE", "clang::ASTDeclWriter::VisitObjCProtocolDecl(clang::ObjCProtocolDecl*)"},
    {"_ZN5clang13ASTDeclWriter24VisitObjCAtDefsFieldDeclEPNS_19ObjCAtDefsFieldDeclE", "clang::ASTDeclWriter::VisitObjCAtDefsFieldDecl(clang::ObjCAtDefsFieldDecl*)"},
    {"_ZN5clang13ASTDeclWriter18VisitObjCClassDeclEPNS_13ObjCClassDeclE", "clang::ASTDeclWriter::VisitObjCClassDecl(clang::ObjCClassDecl*)"},
    {"_ZN5clang13ASTDeclWriter28VisitObjCForwardProtocolDeclEPNS_23ObjCForwardProtocolDeclE", "clang::ASTDeclWriter::VisitObjCForwardProtocolDecl(clang::ObjCForwardProtocolDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitObjCCategoryDeclEPNS_16ObjCCategoryDeclE", "clang::ASTDeclWriter::VisitObjCCategoryDecl(clang::ObjCCategoryDecl*)"},
    {"_ZN5clang13ASTDeclWriter28VisitObjCCompatibleAliasDeclEPNS_23ObjCCompatibleAliasDeclE", "clang::ASTDeclWriter::VisitObjCCompatibleAliasDecl(clang::ObjCCompatibleAliasDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitObjCPropertyDeclEPNS_16ObjCPropertyDeclE", "clang::ASTDeclWriter::VisitObjCPropertyDecl(clang::ObjCPropertyDecl*)"},
    {"_ZN5clang13ASTDeclWriter17VisitObjCImplDeclEPNS_12ObjCImplDeclE", "clang::ASTDeclWriter::VisitObjCImplDecl(clang::ObjCImplDecl*)"},
    {"_ZN5clang13ASTDeclWriter25VisitObjCCategoryImplDeclEPNS_20ObjCCategoryImplDeclE", "clang::ASTDeclWriter::VisitObjCCategoryImplDecl(clang::ObjCCategoryImplDecl*)"},
    {"_ZN5clang13ASTDeclWriter27VisitObjCImplementationDeclEPNS_22ObjCImplementationDeclE", "clang::ASTDeclWriter::VisitObjCImplementationDecl(clang::ObjCImplementationDecl*)"},
    {"_ZN5clang13ASTDeclWriter25VisitObjCPropertyImplDeclEPNS_20ObjCPropertyImplDeclE", "clang::ASTDeclWriter::VisitObjCPropertyImplDecl(clang::ObjCPropertyImplDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitIndirectFieldDeclEPNS_17IndirectFieldDeclE", "clang::ASTDeclWriter::VisitIndirectFieldDecl(clang::IndirectFieldDecl*)"},
    {"_ZN5clang13ASTDeclWriter12VisitVarDeclEPNS_7VarDeclE", "clang::ASTDeclWriter::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitImplicitParamDeclEPNS_17ImplicitParamDeclE", "clang::ASTDeclWriter::VisitImplicitParamDecl(clang::ImplicitParamDecl*)"},
    {"_ZN5clang13ASTDeclWriter16VisitParmVarDeclEPNS_11ParmVarDeclE", "clang::ASTDeclWriter::VisitParmVarDecl(clang::ParmVarDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitFileScopeAsmDeclEPNS_16FileScopeAsmDeclE", "clang::ASTDeclWriter::VisitFileScopeAsmDecl(clang::FileScopeAsmDecl*)"},
    {"_ZN5clang13ASTDeclWriter14VisitBlockDeclEPNS_9BlockDeclE", "clang::ASTDeclWriter::VisitBlockDecl(clang::BlockDecl*)"},
    {"_ZN5clang13ASTDeclWriter20VisitLinkageSpecDeclEPNS_15LinkageSpecDeclE", "clang::ASTDeclWriter::VisitLinkageSpecDecl(clang::LinkageSpecDecl*)"},
    {"_ZN5clang13ASTDeclWriter18VisitNamespaceDeclEPNS_13NamespaceDeclE", "clang::ASTDeclWriter::VisitNamespaceDecl(clang::NamespaceDecl*)"},
    {"_ZN5clang13ASTDeclWriter23VisitNamespaceAliasDeclEPNS_18NamespaceAliasDeclE", "clang::ASTDeclWriter::VisitNamespaceAliasDecl(clang::NamespaceAliasDecl*)"},
    {"_ZN5clang13ASTDeclWriter14VisitUsingDeclEPNS_9UsingDeclE", "clang::ASTDeclWriter::VisitUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang13ASTDeclWriter20VisitUsingShadowDeclEPNS_15UsingShadowDeclE", "clang::ASTDeclWriter::VisitUsingShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang13ASTDeclWriter23VisitUsingDirectiveDeclEPNS_18UsingDirectiveDeclE", "clang::ASTDeclWriter::VisitUsingDirectiveDecl(clang::UsingDirectiveDecl*)"},
    {"_ZN5clang13ASTDeclWriter29VisitUnresolvedUsingValueDeclEPNS_24UnresolvedUsingValueDeclE", "clang::ASTDeclWriter::VisitUnresolvedUsingValueDecl(clang::UnresolvedUsingValueDecl*)"},
    {"_ZN5clang13ASTDeclWriter32VisitUnresolvedUsingTypenameDeclEPNS_27UnresolvedUsingTypenameDeclE", "clang::ASTDeclWriter::VisitUnresolvedUsingTypenameDecl(clang::UnresolvedUsingTypenameDecl*)"},
    {"_ZN5clang13ASTDeclWriter18VisitCXXRecordDeclEPNS_13CXXRecordDeclE", "clang::ASTDeclWriter::VisitCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_ZN5clang13ASTDeclWriter18VisitCXXMethodDeclEPNS_13CXXMethodDeclE", "clang::ASTDeclWriter::VisitCXXMethodDecl(clang::CXXMethodDecl*)"},
    {"_ZN5clang13ASTDeclWriter23VisitCXXConstructorDeclEPNS_18CXXConstructorDeclE", "clang::ASTDeclWriter::VisitCXXConstructorDecl(clang::CXXConstructorDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitCXXDestructorDeclEPNS_17CXXDestructorDeclE", "clang::ASTDeclWriter::VisitCXXDestructorDecl(clang::CXXDestructorDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitCXXConversionDeclEPNS_17CXXConversionDeclE", "clang::ASTDeclWriter::VisitCXXConversionDecl(clang::CXXConversionDecl*)"},
    {"_ZN5clang13ASTDeclWriter19VisitAccessSpecDeclEPNS_14AccessSpecDeclE", "clang::ASTDeclWriter::VisitAccessSpecDecl(clang::AccessSpecDecl*)"},
    {"_ZN5clang13ASTDeclWriter15VisitFriendDeclEPNS_10FriendDeclE", "clang::ASTDeclWriter::VisitFriendDecl(clang::FriendDecl*)"},
    {"_ZN5clang13ASTDeclWriter23VisitFriendTemplateDeclEPNS_18FriendTemplateDeclE", "clang::ASTDeclWriter::VisitFriendTemplateDecl(clang::FriendTemplateDecl*)"},
    {"_ZN5clang13ASTDeclWriter17VisitTemplateDeclEPNS_12TemplateDeclE", "clang::ASTDeclWriter::VisitTemplateDecl(clang::TemplateDecl*)"},
    {"_ZN5clang13ASTDeclWriter29VisitRedeclarableTemplateDeclEPNS_24RedeclarableTemplateDeclE", "clang::ASTDeclWriter::VisitRedeclarableTemplateDecl(clang::RedeclarableTemplateDecl*)"},
    {"_ZN5clang13ASTDeclWriter22VisitClassTemplateDeclEPNS_17ClassTemplateDeclE", "clang::ASTDeclWriter::VisitClassTemplateDecl(clang::ClassTemplateDecl*)"},
    {"_ZN5clang13ASTDeclWriter36VisitClassTemplateSpecializationDeclEPNS_31ClassTemplateSpecializationDeclE", "clang::ASTDeclWriter::VisitClassTemplateSpecializationDecl(clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN5clang13ASTDeclWriter43VisitClassTemplatePartialSpecializationDeclEPNS_38ClassTemplatePartialSpecializationDeclE", "clang::ASTDeclWriter::VisitClassTemplatePartialSpecializationDecl(clang::ClassTemplatePartialSpecializationDecl*)"},
    {"_ZN5clang13ASTDeclWriter25VisitFunctionTemplateDeclEPNS_20FunctionTemplateDeclE", "clang::ASTDeclWriter::VisitFunctionTemplateDecl(clang::FunctionTemplateDecl*)"},
    {"_ZN5clang13ASTDeclWriter25VisitTemplateTypeParmDeclEPNS_20TemplateTypeParmDeclE", "clang::ASTDeclWriter::VisitTemplateTypeParmDecl(clang::TemplateTypeParmDecl*)"},
    {"_ZN5clang13ASTDeclWriter28VisitNonTypeTemplateParmDeclEPNS_23NonTypeTemplateParmDeclE", "clang::ASTDeclWriter::VisitNonTypeTemplateParmDecl(clang::NonTypeTemplateParmDecl*)"},
    {"_ZN5clang13ASTDeclWriter29VisitTemplateTemplateParmDeclEPNS_24TemplateTemplateParmDeclE", "clang::ASTDeclWriter::VisitTemplateTemplateParmDecl(clang::TemplateTemplateParmDecl*)"},
    {"_ZN5clang13ASTDeclWriter21VisitStaticAssertDeclEPNS_16StaticAssertDeclE", "clang::ASTDeclWriter::VisitStaticAssertDecl(clang::StaticAssertDecl*)"},
    {"_ZN5clang13ASTDeclWriter16VisitDeclContextEPNS_11DeclContextEyy", "clang::ASTDeclWriter::VisitDeclContext(clang::DeclContext*, unsigned long long, unsigned long long)"},
    {"_ZN5clang9ASTWriter22WriteDeclsBlockAbbrevsEv", "clang::ASTWriter::WriteDeclsBlockAbbrevs()"},
    {"_ZN5clang9ASTWriter9WriteDeclERNS_10ASTContextEPNS_4DeclE", "clang::ASTWriter::WriteDecl(clang::ASTContext&, clang::Decl*)"},
    {"_ZN5clang11DeclVisitorINS_13ASTDeclWriterEvE5VisitEPNS_4DeclE", "clang::DeclVisitor<clang::ASTDeclWriter, void>::Visit(clang::Decl*)"},
    {"_ZN5clang13ASTDeclWriter17VisitRedeclarableINS_7TagDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclWriter::VisitRedeclarable<clang::TagDecl>(clang::Redeclarable<clang::TagDecl>*)"},
    {"_ZN5clang13ASTDeclWriter17VisitRedeclarableINS_12FunctionDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclWriter::VisitRedeclarable<clang::FunctionDecl>(clang::Redeclarable<clang::FunctionDecl>*)"},
    {"_ZN5clang13ASTDeclWriter17VisitRedeclarableINS_7VarDeclEEEvPNS_12RedeclarableIT_EE", "void clang::ASTDeclWriter::VisitRedeclarable<clang::VarDecl>(clang::Redeclarable<clang::VarDecl>*)"},
    {"_ZN5clang13ASTStmtWriter31AddExplicitTemplateArgumentListERKNS_28ExplicitTemplateArgumentListE", "clang::ASTStmtWriter::AddExplicitTemplateArgumentList(clang::ExplicitTemplateArgumentList const&)"},
    {"_ZN5clang13ASTStmtWriter9VisitStmtEPNS_4StmtE", "clang::ASTStmtWriter::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang13ASTStmtWriter13VisitNullStmtEPNS_8NullStmtE", "clang::ASTStmtWriter::VisitNullStmt(clang::NullStmt*)"},
    {"_ZN5clang13ASTStmtWriter17VisitCompoundStmtEPNS_12CompoundStmtE", "clang::ASTStmtWriter::VisitCompoundStmt(clang::CompoundStmt*)"},
    {"_ZN5clang13ASTStmtWriter15VisitSwitchCaseEPNS_10SwitchCaseE", "clang::ASTStmtWriter::VisitSwitchCase(clang::SwitchCase*)"},
    {"_ZN5clang9ASTWriter15getSwitchCaseIDEPNS_10SwitchCaseE", "clang::ASTWriter::getSwitchCaseID(clang::SwitchCase*)"},
    {"_ZN5clang13ASTStmtWriter13VisitCaseStmtEPNS_8CaseStmtE", "clang::ASTStmtWriter::VisitCaseStmt(clang::CaseStmt*)"},
    {"_ZN5clang13ASTStmtWriter16VisitDefaultStmtEPNS_11DefaultStmtE", "clang::ASTStmtWriter::VisitDefaultStmt(clang::DefaultStmt*)"},
    {"_ZN5clang13ASTStmtWriter14VisitLabelStmtEPNS_9LabelStmtE", "clang::ASTStmtWriter::VisitLabelStmt(clang::LabelStmt*)"},
    {"_ZN5clang9ASTWriter10GetLabelIDEPNS_9LabelStmtE", "clang::ASTWriter::GetLabelID(clang::LabelStmt*)"},
    {"_ZN5clang13ASTStmtWriter11VisitIfStmtEPNS_6IfStmtE", "clang::ASTStmtWriter::VisitIfStmt(clang::IfStmt*)"},
    {"_ZN5clang13ASTStmtWriter15VisitSwitchStmtEPNS_10SwitchStmtE", "clang::ASTStmtWriter::VisitSwitchStmt(clang::SwitchStmt*)"},
    {"_ZN5clang9ASTWriter18RecordSwitchCaseIDEPNS_10SwitchCaseE", "clang::ASTWriter::RecordSwitchCaseID(clang::SwitchCase*)"},
    {"_ZN5clang13ASTStmtWriter14VisitWhileStmtEPNS_9WhileStmtE", "clang::ASTStmtWriter::VisitWhileStmt(clang::WhileStmt*)"},
    {"_ZN5clang13ASTStmtWriter11VisitDoStmtEPNS_6DoStmtE", "clang::ASTStmtWriter::VisitDoStmt(clang::DoStmt*)"},
    {"_ZN5clang13ASTStmtWriter12VisitForStmtEPNS_7ForStmtE", "clang::ASTStmtWriter::VisitForStmt(clang::ForStmt*)"},
    {"_ZN5clang13ASTStmtWriter13VisitGotoStmtEPNS_8GotoStmtE", "clang::ASTStmtWriter::VisitGotoStmt(clang::GotoStmt*)"},
    {"_ZN5clang13ASTStmtWriter21VisitIndirectGotoStmtEPNS_16IndirectGotoStmtE", "clang::ASTStmtWriter::VisitIndirectGotoStmt(clang::IndirectGotoStmt*)"},
    {"_ZN5clang13ASTStmtWriter17VisitContinueStmtEPNS_12ContinueStmtE", "clang::ASTStmtWriter::VisitContinueStmt(clang::ContinueStmt*)"},
    {"_ZN5clang13ASTStmtWriter14VisitBreakStmtEPNS_9BreakStmtE", "clang::ASTStmtWriter::VisitBreakStmt(clang::BreakStmt*)"},
    {"_ZN5clang13ASTStmtWriter15VisitReturnStmtEPNS_10ReturnStmtE", "clang::ASTStmtWriter::VisitReturnStmt(clang::ReturnStmt*)"},
    {"_ZN5clang13ASTStmtWriter13VisitDeclStmtEPNS_8DeclStmtE", "clang::ASTStmtWriter::VisitDeclStmt(clang::DeclStmt*)"},
    {"_ZN5clang13ASTStmtWriter12VisitAsmStmtEPNS_7AsmStmtE", "clang::ASTStmtWriter::VisitAsmStmt(clang::AsmStmt*)"},
    {"_ZN5clang13ASTStmtWriter9VisitExprEPNS_4ExprE", "clang::ASTStmtWriter::VisitExpr(clang::Expr*)"},
    {"_ZN5clang13ASTStmtWriter19VisitPredefinedExprEPNS_14PredefinedExprE", "clang::ASTStmtWriter::VisitPredefinedExpr(clang::PredefinedExpr*)"},
    {"_ZN5clang13ASTStmtWriter16VisitDeclRefExprEPNS_11DeclRefExprE", "clang::ASTStmtWriter::VisitDeclRefExpr(clang::DeclRefExpr*)"},
    {"_ZN5clang13ASTStmtWriter19VisitIntegerLiteralEPNS_14IntegerLiteralE", "clang::ASTStmtWriter::VisitIntegerLiteral(clang::IntegerLiteral*)"},
    {"_ZN5clang13ASTStmtWriter20VisitFloatingLiteralEPNS_15FloatingLiteralE", "clang::ASTStmtWriter::VisitFloatingLiteral(clang::FloatingLiteral*)"},
    {"_ZN5clang13ASTStmtWriter21VisitImaginaryLiteralEPNS_16ImaginaryLiteralE", "clang::ASTStmtWriter::VisitImaginaryLiteral(clang::ImaginaryLiteral*)"},
    {"_ZN5clang13ASTStmtWriter18VisitStringLiteralEPNS_13StringLiteralE", "clang::ASTStmtWriter::VisitStringLiteral(clang::StringLiteral*)"},
    {"_ZN5clang13ASTStmtWriter21VisitCharacterLiteralEPNS_16CharacterLiteralE", "clang::ASTStmtWriter::VisitCharacterLiteral(clang::CharacterLiteral*)"},
    {"_ZN5clang13ASTStmtWriter14VisitParenExprEPNS_9ParenExprE", "clang::ASTStmtWriter::VisitParenExpr(clang::ParenExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitParenListExprEPNS_13ParenListExprE", "clang::ASTStmtWriter::VisitParenListExpr(clang::ParenListExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitUnaryOperatorEPNS_13UnaryOperatorE", "clang::ASTStmtWriter::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN5clang13ASTStmtWriter17VisitOffsetOfExprEPNS_12OffsetOfExprE", "clang::ASTStmtWriter::VisitOffsetOfExpr(clang::OffsetOfExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitSizeOfAlignOfExprEPNS_17SizeOfAlignOfExprE", "clang::ASTStmtWriter::VisitSizeOfAlignOfExpr(clang::SizeOfAlignOfExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitArraySubscriptExprEPNS_18ArraySubscriptExprE", "clang::ASTStmtWriter::VisitArraySubscriptExpr(clang::ArraySubscriptExpr*)"},
    {"_ZN5clang13ASTStmtWriter13VisitCallExprEPNS_8CallExprE", "clang::ASTStmtWriter::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN5clang13ASTStmtWriter15VisitMemberExprEPNS_10MemberExprE", "clang::ASTStmtWriter::VisitMemberExpr(clang::MemberExpr*)"},
    {"_ZN5clang13ASTStmtWriter16VisitObjCIsaExprEPNS_11ObjCIsaExprE", "clang::ASTStmtWriter::VisitObjCIsaExpr(clang::ObjCIsaExpr*)"},
    {"_ZN5clang13ASTStmtWriter13VisitCastExprEPNS_8CastExprE", "clang::ASTStmtWriter::VisitCastExpr(clang::CastExpr*)"},
    {"_ZN5clang13ASTStmtWriter19VisitBinaryOperatorEPNS_14BinaryOperatorE", "clang::ASTStmtWriter::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN5clang13ASTStmtWriter27VisitCompoundAssignOperatorEPNS_22CompoundAssignOperatorE", "clang::ASTStmtWriter::VisitCompoundAssignOperator(clang::CompoundAssignOperator*)"},
    {"_ZN5clang13ASTStmtWriter24VisitConditionalOperatorEPNS_19ConditionalOperatorE", "clang::ASTStmtWriter::VisitConditionalOperator(clang::ConditionalOperator*)"},
    {"_ZN5clang13ASTStmtWriter21VisitImplicitCastExprEPNS_16ImplicitCastExprE", "clang::ASTStmtWriter::VisitImplicitCastExpr(clang::ImplicitCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitExplicitCastExprEPNS_16ExplicitCastExprE", "clang::ASTStmtWriter::VisitExplicitCastExpr(clang::ExplicitCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter19VisitCStyleCastExprEPNS_14CStyleCastExprE", "clang::ASTStmtWriter::VisitCStyleCastExpr(clang::CStyleCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter24VisitCompoundLiteralExprEPNS_19CompoundLiteralExprE", "clang::ASTStmtWriter::VisitCompoundLiteralExpr(clang::CompoundLiteralExpr*)"},
    {"_ZN5clang13ASTStmtWriter25VisitExtVectorElementExprEPNS_20ExtVectorElementExprE", "clang::ASTStmtWriter::VisitExtVectorElementExpr(clang::ExtVectorElementExpr*)"},
    {"_ZN5clang13ASTStmtWriter17VisitInitListExprEPNS_12InitListExprE", "clang::ASTStmtWriter::VisitInitListExpr(clang::InitListExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitDesignatedInitExprEPNS_18DesignatedInitExprE", "clang::ASTStmtWriter::VisitDesignatedInitExpr(clang::DesignatedInitExpr*)"},
    {"_ZN5clang13ASTStmtWriter26VisitImplicitValueInitExprEPNS_21ImplicitValueInitExprE", "clang::ASTStmtWriter::VisitImplicitValueInitExpr(clang::ImplicitValueInitExpr*)"},
    {"_ZN5clang13ASTStmtWriter14VisitVAArgExprEPNS_9VAArgExprE", "clang::ASTStmtWriter::VisitVAArgExpr(clang::VAArgExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitAddrLabelExprEPNS_13AddrLabelExprE", "clang::ASTStmtWriter::VisitAddrLabelExpr(clang::AddrLabelExpr*)"},
    {"_ZN5clang13ASTStmtWriter13VisitStmtExprEPNS_8StmtExprE", "clang::ASTStmtWriter::VisitStmtExpr(clang::StmtExpr*)"},
    {"_ZN5clang13ASTStmtWriter15VisitChooseExprEPNS_10ChooseExprE", "clang::ASTStmtWriter::VisitChooseExpr(clang::ChooseExpr*)"},
    {"_ZN5clang13ASTStmtWriter16VisitGNUNullExprEPNS_11GNUNullExprE", "clang::ASTStmtWriter::VisitGNUNullExpr(clang::GNUNullExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitShuffleVectorExprEPNS_17ShuffleVectorExprE", "clang::ASTStmtWriter::VisitShuffleVectorExpr(clang::ShuffleVectorExpr*)"},
    {"_ZN5clang13ASTStmtWriter14VisitBlockExprEPNS_9BlockExprE", "clang::ASTStmtWriter::VisitBlockExpr(clang::BlockExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitBlockDeclRefExprEPNS_16BlockDeclRefExprE", "clang::ASTStmtWriter::VisitBlockDeclRefExpr(clang::BlockDeclRefExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitObjCStringLiteralEPNS_17ObjCStringLiteralE", "clang::ASTStmtWriter::VisitObjCStringLiteral(clang::ObjCStringLiteral*)"},
    {"_ZN5clang13ASTStmtWriter19VisitObjCEncodeExprEPNS_14ObjCEncodeExprE", "clang::ASTStmtWriter::VisitObjCEncodeExpr(clang::ObjCEncodeExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitObjCSelectorExprEPNS_16ObjCSelectorExprE", "clang::ASTStmtWriter::VisitObjCSelectorExpr(clang::ObjCSelectorExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitObjCProtocolExprEPNS_16ObjCProtocolExprE", "clang::ASTStmtWriter::VisitObjCProtocolExpr(clang::ObjCProtocolExpr*)"},
    {"_ZN5clang13ASTStmtWriter20VisitObjCIvarRefExprEPNS_15ObjCIvarRefExprE", "clang::ASTStmtWriter::VisitObjCIvarRefExpr(clang::ObjCIvarRefExpr*)"},
    {"_ZN5clang13ASTStmtWriter24VisitObjCPropertyRefExprEPNS_19ObjCPropertyRefExprE", "clang::ASTStmtWriter::VisitObjCPropertyRefExpr(clang::ObjCPropertyRefExpr*)"},
    {"_ZN5clang13ASTStmtWriter20VisitObjCMessageExprEPNS_15ObjCMessageExprE", "clang::ASTStmtWriter::VisitObjCMessageExpr(clang::ObjCMessageExpr*)"},
    {"_ZN5clang13ASTStmtWriter26VisitObjCForCollectionStmtEPNS_21ObjCForCollectionStmtE", "clang::ASTStmtWriter::VisitObjCForCollectionStmt(clang::ObjCForCollectionStmt*)"},
    {"_ZN5clang13ASTStmtWriter20VisitObjCAtCatchStmtEPNS_15ObjCAtCatchStmtE", "clang::ASTStmtWriter::VisitObjCAtCatchStmt(clang::ObjCAtCatchStmt*)"},
    {"_ZN5clang13ASTStmtWriter22VisitObjCAtFinallyStmtEPNS_17ObjCAtFinallyStmtE", "clang::ASTStmtWriter::VisitObjCAtFinallyStmt(clang::ObjCAtFinallyStmt*)"},
    {"_ZN5clang13ASTStmtWriter18VisitObjCAtTryStmtEPNS_13ObjCAtTryStmtE", "clang::ASTStmtWriter::VisitObjCAtTryStmt(clang::ObjCAtTryStmt*)"},
    {"_ZN5clang13ASTStmtWriter27VisitObjCAtSynchronizedStmtEPNS_22ObjCAtSynchronizedStmtE", "clang::ASTStmtWriter::VisitObjCAtSynchronizedStmt(clang::ObjCAtSynchronizedStmt*)"},
    {"_ZN5clang13ASTStmtWriter20VisitObjCAtThrowStmtEPNS_15ObjCAtThrowStmtE", "clang::ASTStmtWriter::VisitObjCAtThrowStmt(clang::ObjCAtThrowStmt*)"},
    {"_ZN5clang13ASTStmtWriter17VisitCXXCatchStmtEPNS_12CXXCatchStmtE", "clang::ASTStmtWriter::VisitCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN5clang13ASTStmtWriter15VisitCXXTryStmtEPNS_10CXXTryStmtE", "clang::ASTStmtWriter::VisitCXXTryStmt(clang::CXXTryStmt*)"},
    {"_ZN5clang13ASTStmtWriter24VisitCXXOperatorCallExprEPNS_19CXXOperatorCallExprE", "clang::ASTStmtWriter::VisitCXXOperatorCallExpr(clang::CXXOperatorCallExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitCXXMemberCallExprEPNS_17CXXMemberCallExprE", "clang::ASTStmtWriter::VisitCXXMemberCallExpr(clang::CXXMemberCallExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitCXXConstructExprEPNS_16CXXConstructExprE", "clang::ASTStmtWriter::VisitCXXConstructExpr(clang::CXXConstructExpr*)"},
    {"_ZN5clang13ASTStmtWriter27VisitCXXTemporaryObjectExprEPNS_22CXXTemporaryObjectExprE", "clang::ASTStmtWriter::VisitCXXTemporaryObjectExpr(clang::CXXTemporaryObjectExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitCXXNamedCastExprEPNS_16CXXNamedCastExprE", "clang::ASTStmtWriter::VisitCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitCXXStaticCastExprEPNS_17CXXStaticCastExprE", "clang::ASTStmtWriter::VisitCXXStaticCastExpr(clang::CXXStaticCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitCXXDynamicCastExprEPNS_18CXXDynamicCastExprE", "clang::ASTStmtWriter::VisitCXXDynamicCastExpr(clang::CXXDynamicCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter27VisitCXXReinterpretCastExprEPNS_22CXXReinterpretCastExprE", "clang::ASTStmtWriter::VisitCXXReinterpretCastExpr(clang::CXXReinterpretCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitCXXConstCastExprEPNS_16CXXConstCastExprE", "clang::ASTStmtWriter::VisitCXXConstCastExpr(clang::CXXConstCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter26VisitCXXFunctionalCastExprEPNS_21CXXFunctionalCastExprE", "clang::ASTStmtWriter::VisitCXXFunctionalCastExpr(clang::CXXFunctionalCastExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitCXXBoolLiteralExprEPNS_18CXXBoolLiteralExprE", "clang::ASTStmtWriter::VisitCXXBoolLiteralExpr(clang::CXXBoolLiteralExpr*)"},
    {"_ZN5clang13ASTStmtWriter26VisitCXXNullPtrLiteralExprEPNS_21CXXNullPtrLiteralExprE", "clang::ASTStmtWriter::VisitCXXNullPtrLiteralExpr(clang::CXXNullPtrLiteralExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitCXXTypeidExprEPNS_13CXXTypeidExprE", "clang::ASTStmtWriter::VisitCXXTypeidExpr(clang::CXXTypeidExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitCXXUuidofExprEPNS_13CXXUuidofExprE", "clang::ASTStmtWriter::VisitCXXUuidofExpr(clang::CXXUuidofExpr*)"},
    {"_ZN5clang13ASTStmtWriter16VisitCXXThisExprEPNS_11CXXThisExprE", "clang::ASTStmtWriter::VisitCXXThisExpr(clang::CXXThisExpr*)"},
    {"_ZN5clang13ASTStmtWriter17VisitCXXThrowExprEPNS_12CXXThrowExprE", "clang::ASTStmtWriter::VisitCXXThrowExpr(clang::CXXThrowExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitCXXDefaultArgExprEPNS_17CXXDefaultArgExprE", "clang::ASTStmtWriter::VisitCXXDefaultArgExpr(clang::CXXDefaultArgExpr*)"},
    {"_ZN5clang13ASTStmtWriter25VisitCXXBindTemporaryExprEPNS_20CXXBindTemporaryExprE", "clang::ASTStmtWriter::VisitCXXBindTemporaryExpr(clang::CXXBindTemporaryExpr*)"},
    {"_ZN5clang13ASTStmtWriter27VisitCXXScalarValueInitExprEPNS_22CXXScalarValueInitExprE", "clang::ASTStmtWriter::VisitCXXScalarValueInitExpr(clang::CXXScalarValueInitExpr*)"},
    {"_ZN5clang13ASTStmtWriter15VisitCXXNewExprEPNS_10CXXNewExprE", "clang::ASTStmtWriter::VisitCXXNewExpr(clang::CXXNewExpr*)"},
    {"_ZN5clang13ASTStmtWriter18VisitCXXDeleteExprEPNS_13CXXDeleteExprE", "clang::ASTStmtWriter::VisitCXXDeleteExpr(clang::CXXDeleteExpr*)"},
    {"_ZN5clang13ASTStmtWriter28VisitCXXPseudoDestructorExprEPNS_23CXXPseudoDestructorExprE", "clang::ASTStmtWriter::VisitCXXPseudoDestructorExpr(clang::CXXPseudoDestructorExpr*)"},
    {"_ZN5clang13ASTStmtWriter21VisitExprWithCleanupsEPNS_16ExprWithCleanupsE", "clang::ASTStmtWriter::VisitExprWithCleanups(clang::ExprWithCleanups*)"},
    {"_ZN5clang13ASTStmtWriter32VisitCXXDependentScopeMemberExprEPNS_27CXXDependentScopeMemberExprE", "clang::ASTStmtWriter::VisitCXXDependentScopeMemberExpr(clang::CXXDependentScopeMemberExpr*)"},
    {"_ZN5clang13ASTStmtWriter30VisitDependentScopeDeclRefExprEPNS_25DependentScopeDeclRefExprE", "clang::ASTStmtWriter::VisitDependentScopeDeclRefExpr(clang::DependentScopeDeclRefExpr*)"},
    {"_ZN5clang13ASTStmtWriter31VisitCXXUnresolvedConstructExprEPNS_26CXXUnresolvedConstructExprE", "clang::ASTStmtWriter::VisitCXXUnresolvedConstructExpr(clang::CXXUnresolvedConstructExpr*)"},
    {"_ZN5clang13ASTStmtWriter17VisitOverloadExprEPNS_12OverloadExprE", "clang::ASTStmtWriter::VisitOverloadExpr(clang::OverloadExpr*)"},
    {"_ZN5clang13ASTStmtWriter25VisitUnresolvedMemberExprEPNS_20UnresolvedMemberExprE", "clang::ASTStmtWriter::VisitUnresolvedMemberExpr(clang::UnresolvedMemberExpr*)"},
    {"_ZN5clang13ASTStmtWriter25VisitUnresolvedLookupExprEPNS_20UnresolvedLookupExprE", "clang::ASTStmtWriter::VisitUnresolvedLookupExpr(clang::UnresolvedLookupExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitUnaryTypeTraitExprEPNS_18UnaryTypeTraitExprE", "clang::ASTStmtWriter::VisitUnaryTypeTraitExpr(clang::UnaryTypeTraitExpr*)"},
    {"_ZN5clang13ASTStmtWriter24VisitBinaryTypeTraitExprEPNS_19BinaryTypeTraitExprE", "clang::ASTStmtWriter::VisitBinaryTypeTraitExpr(clang::BinaryTypeTraitExpr*)"},
    {"_ZN5clang13ASTStmtWriter20VisitCXXNoexceptExprEPNS_15CXXNoexceptExprE", "clang::ASTStmtWriter::VisitCXXNoexceptExpr(clang::CXXNoexceptExpr*)"},
    {"_ZN5clang13ASTStmtWriter22VisitPackExpansionExprEPNS_17PackExpansionExprE", "clang::ASTStmtWriter::VisitPackExpansionExpr(clang::PackExpansionExpr*)"},
    {"_ZN5clang13ASTStmtWriter19VisitSizeOfPackExprEPNS_14SizeOfPackExprE", "clang::ASTStmtWriter::VisitSizeOfPackExpr(clang::SizeOfPackExpr*)"},
    {"_ZN5clang13ASTStmtWriter37VisitSubstNonTypeTemplateParmPackExprEPNS_32SubstNonTypeTemplateParmPackExprE", "clang::ASTStmtWriter::VisitSubstNonTypeTemplateParmPackExpr(clang::SubstNonTypeTemplateParmPackExpr*)"},
    {"_ZN5clang13ASTStmtWriter20VisitOpaqueValueExprEPNS_15OpaqueValueExprE", "clang::ASTStmtWriter::VisitOpaqueValueExpr(clang::OpaqueValueExpr*)"},
    {"_ZN5clang13ASTStmtWriter23VisitCUDAKernelCallExprEPNS_18CUDAKernelCallExprE", "clang::ASTStmtWriter::VisitCUDAKernelCallExpr(clang::CUDAKernelCallExpr*)"},
    {"_ZN5clang9ASTWriter18ClearSwitchCaseIDsEv", "clang::ASTWriter::ClearSwitchCaseIDs()"},
    {"_ZN5clang9ASTWriter12WriteSubStmtEPNS_4StmtE", "clang::ASTWriter::WriteSubStmt(clang::Stmt*)"},
    {"_ZN5clang9ASTWriter10FlushStmtsEv", "clang::ASTWriter::FlushStmts()"},
    {"_ZN5clang11StmtVisitorINS_13ASTStmtWriterEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<clang::ASTStmtWriter, void>::Visit(clang::Stmt*)"},
    {"_ZNSt8_Rb_treeIPN5clang9LabelStmtESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::LabelStmt*, std::pair<clang::LabelStmt* const, unsigned int>, std::_Select1st<std::pair<clang::LabelStmt* const, unsigned int> >, std::less<clang::LabelStmt*>, std::allocator<std::pair<clang::LabelStmt* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::LabelStmt* const, unsigned int> >, std::pair<clang::LabelStmt* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang9LabelStmtESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::LabelStmt*, std::pair<clang::LabelStmt* const, unsigned int>, std::_Select1st<std::pair<clang::LabelStmt* const, unsigned int> >, std::less<clang::LabelStmt*>, std::allocator<std::pair<clang::LabelStmt* const, unsigned int> > >::_M_insert_unique(std::pair<clang::LabelStmt* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang10SwitchCaseESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::SwitchCase*, std::pair<clang::SwitchCase* const, unsigned int>, std::_Select1st<std::pair<clang::SwitchCase* const, unsigned int> >, std::less<clang::SwitchCase*>, std::allocator<std::pair<clang::SwitchCase* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::SwitchCase* const, unsigned int> >, std::pair<clang::SwitchCase* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang10SwitchCaseESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::SwitchCase*, std::pair<clang::SwitchCase* const, unsigned int>, std::_Select1st<std::pair<clang::SwitchCase* const, unsigned int> >, std::less<clang::SwitchCase*>, std::allocator<std::pair<clang::SwitchCase* const, unsigned int> > >::_M_insert_unique(std::pair<clang::SwitchCase* const, unsigned int> const&)"},
    {"_ZN5clang12PCHGeneratorC1ERKNS_12PreprocessorEbPKcPN4llvm11raw_ostreamE", "clang::PCHGenerator::PCHGenerator(clang::Preprocessor const&, bool, char const*, llvm::raw_ostream*)"},
    {"_ZN5clang12PCHGeneratorC2ERKNS_12PreprocessorEbPKcPN4llvm11raw_ostreamE", "clang::PCHGenerator::PCHGenerator(clang::Preprocessor const&, bool, char const*, llvm::raw_ostream*)"},
    {"_ZN5clang12PCHGenerator21HandleTranslationUnitERNS_10ASTContextE", "clang::PCHGenerator::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang12PCHGenerator22GetASTMutationListenerEv", "clang::PCHGenerator::GetASTMutationListener()"},
    {"_ZN5clang12PCHGenerator27GetASTSerializationListenerEv", "clang::PCHGenerator::GetASTSerializationListener()"},
    {"_ZN5clang12PCHGenerator29GetASTDeserializationListenerEv", "clang::PCHGenerator::GetASTDeserializationListener()"},
    {"_ZN5clang17EmitBackendOutputERNS_10DiagnosticERKNS_14CodeGenOptionsERKNS_13TargetOptionsEPN4llvm6ModuleENS_13BackendActionEPNS8_11raw_ostreamE", "clang::EmitBackendOutput(clang::Diagnostic&, clang::CodeGenOptions const&, clang::TargetOptions const&, llvm::Module*, clang::BackendAction, llvm::raw_ostream*)"},
    {"_ZN5clang7CodeGen11CGBlockInfoC1EPKNS_9BlockExprEPKc", "clang::CodeGen::CGBlockInfo::CGBlockInfo(clang::BlockExpr const*, char const*)"},
    {"_ZN5clang7CodeGen11CGBlockInfoC2EPKNS_9BlockExprEPKc", "clang::CodeGen::CGBlockInfo::CGBlockInfo(clang::BlockExpr const*, char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitBlockLiteralEPKNS_9BlockExprE", "clang::CodeGen::CodeGenFunction::EmitBlockLiteral(clang::BlockExpr const*)"},
    {"_Z16computeBlockInfoRN5clang7CodeGen13CodeGenModuleERNS0_11CGBlockInfoE", "computeBlockInfo(clang::CodeGen::CodeGenModule&, clang::CodeGen::CGBlockInfo&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21GenerateBlockFunctionENS0_10GlobalDeclERKNS0_11CGBlockInfoEPKNS_4DeclERKN4llvm8DenseMapIS8_PNS9_5ValueENS9_12DenseMapInfoIS8_EENSD_ISC_EEEE", "clang::CodeGen::CodeGenFunction::GenerateBlockFunction(clang::CodeGen::GlobalDecl, clang::CodeGen::CGBlockInfo const&, clang::Decl const*, llvm::DenseMap<clang::Decl const*, llvm::Value*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::Value*> > const&)"},
    {"_Z16buildGlobalBlockRN5clang7CodeGen13CodeGenModuleERKNS0_11CGBlockInfoEPN4llvm8ConstantE", "buildGlobalBlock(clang::CodeGen::CodeGenModule&, clang::CodeGen::CGBlockInfo const&, llvm::Constant*)"},
    {"_Z20buildBlockDescriptorRN5clang7CodeGen13CodeGenModuleERKNS0_11CGBlockInfoE", "buildBlockDescriptor(clang::CodeGen::CodeGenModule&, clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule22getBlockDescriptorTypeEv", "clang::CodeGen::CodeGenModule::getBlockDescriptorType()"},
    {"_ZN5clang7CodeGen13CodeGenModule26getGenericBlockLiteralTypeEv", "clang::CodeGen::CodeGenModule::getGenericBlockLiteralType()"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitBlockCallExprEPKNS_8CallExprENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitBlockCallExpr(clang::CallExpr const*, clang::CodeGen::ReturnValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18GetAddrOfBlockDeclEPKNS_7VarDeclEb", "clang::CodeGen::CodeGenFunction::GetAddrOfBlockDecl(clang::VarDecl const*, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule20GetAddrOfGlobalBlockEPKNS_9BlockExprEPKc", "clang::CodeGen::CodeGenModule::GetAddrOfGlobalBlock(clang::BlockExpr const*, char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26GenerateCopyHelperFunctionERKNS0_11CGBlockInfoE", "clang::CodeGen::CodeGenFunction::GenerateCopyHelperFunction(clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29GenerateDestroyHelperFunctionERKNS0_11CGBlockInfoE", "clang::CodeGen::CodeGenFunction::GenerateDestroyHelperFunction(clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17BuildBlockReleaseEPN4llvm5ValueENS0_15BlockFieldFlagsE", "clang::CodeGen::CodeGenFunction::BuildBlockRelease(llvm::Value*, clang::CodeGen::BlockFieldFlags)"},
    {"_ZN5clang7CodeGen15CodeGenFunction31GeneratebyrefCopyHelperFunctionEPKN4llvm4TypeENS0_15BlockFieldFlagsEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::GeneratebyrefCopyHelperFunction(llvm::Type const*, clang::CodeGen::BlockFieldFlags, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction34GeneratebyrefDestroyHelperFunctionEPKN4llvm4TypeENS0_15BlockFieldFlagsEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::GeneratebyrefDestroyHelperFunction(llvm::Type const*, clang::CodeGen::BlockFieldFlags, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule20BuildbyrefCopyHelperEPKN4llvm4TypeENS0_15BlockFieldFlagsEjPKNS_7VarDeclE", "clang::CodeGen::CodeGenModule::BuildbyrefCopyHelper(llvm::Type const*, clang::CodeGen::BlockFieldFlags, unsigned int, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule23BuildbyrefDestroyHelperEPKN4llvm4TypeENS0_15BlockFieldFlagsEjPKNS_7VarDeclE", "clang::CodeGen::CodeGenModule::BuildbyrefDestroyHelper(llvm::Type const*, clang::CodeGen::BlockFieldFlags, unsigned int, clang::VarDecl const*)"},
    {"_ZSt21__inplace_stable_sortIPN12_GLOBAL__N_116BlockLayoutChunkEEvT_S3_", "void std::__inplace_stable_sort<(anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*)"},
    {"_ZSt22__stable_sort_adaptiveIPN12_GLOBAL__N_116BlockLayoutChunkES2_lEvT_S3_T0_T1_", "void std::__stable_sort_adaptive<(anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, long>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, long)"},
    {"_ZSt24__merge_sort_with_bufferIPN12_GLOBAL__N_116BlockLayoutChunkES2_EvT_S3_T0_", "void std::__merge_sort_with_buffer<(anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*)"},
    {"_ZSt16__merge_adaptiveIPN12_GLOBAL__N_116BlockLayoutChunkElS2_EvT_S3_S3_T0_S4_T1_S4_", "void std::__merge_adaptive<(anonymous namespace)::BlockLayoutChunk*, long, (anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, long, long, (anonymous namespace)::BlockLayoutChunk*, long)"},
    {"_ZSt13copy_backwardIPN12_GLOBAL__N_116BlockLayoutChunkES2_ET0_T_S4_S3_", "(anonymous namespace)::BlockLayoutChunk* std::copy_backward<(anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*)"},
    {"_ZSt6rotateIPN12_GLOBAL__N_116BlockLayoutChunkEEvT_S3_S3_", "void std::rotate<(anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*)"},
    {"_ZSt16__insertion_sortIPN12_GLOBAL__N_116BlockLayoutChunkEEvT_S3_", "void std::__insertion_sort<(anonymous namespace)::BlockLayoutChunk*>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*)"},
    {"_ZSt22__merge_without_bufferIPN12_GLOBAL__N_116BlockLayoutChunkElEvT_S3_S3_T0_S4_", "void std::__merge_without_buffer<(anonymous namespace)::BlockLayoutChunk*, long>((anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, (anonymous namespace)::BlockLayoutChunk*, long, long)"},
    {"_Z15hasMutableFieldPKN5clang13CXXRecordDeclE", "hasMutableField(clang::CXXRecordDecl const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_116BlockLayoutChunkELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::BlockLayoutChunk, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIyPNS_8ConstantENS_12DenseMapInfoIyEENS3_IS2_EEE16InsertIntoBucketERKyRKS2_PSt4pairIyS2_E", "llvm::DenseMap<unsigned long long, llvm::Constant*, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<llvm::Constant*> >::InsertIntoBucket(unsigned long long const&, llvm::Constant* const&, std::pair<unsigned long long, llvm::Constant*>*)"},
    {"_ZN4llvm8DenseMapIyPNS_8ConstantENS_12DenseMapInfoIyEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned long long, llvm::Constant*, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<llvm::Constant*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclENS1_7CodeGen11CGBlockInfo7CaptureENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::VarDecl const*, clang::CodeGen::CGBlockInfo::Capture, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::CodeGen::CGBlockInfo::Capture> >::InsertIntoBucket(clang::VarDecl const* const&, clang::CodeGen::CGBlockInfo::Capture const&, std::pair<clang::VarDecl const*, clang::CodeGen::CGBlockInfo::Capture>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclENS1_7CodeGen11CGBlockInfo7CaptureENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::VarDecl const*, clang::CodeGen::CGBlockInfo::Capture, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::CodeGen::CGBlockInfo::Capture> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitBuiltinExprEPKNS_12FunctionDeclEjPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitBuiltinExpr(clang::FunctionDecl const*, unsigned int, clang::CallExpr const*)"},
    {"_Z8EmitFAbsRN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueENS_8QualTypeE", "EmitFAbs(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::QualType)"},
    {"_Z16EmitBinaryAtomicRN5clang7CodeGen15CodeGenFunctionEN4llvm9Intrinsic2IDEPKNS_8CallExprE", "EmitBinaryAtomic(clang::CodeGen::CodeGenFunction&, llvm::Intrinsic::ID, clang::CallExpr const*)"},
    {"_Z20EmitBinaryAtomicPostRN5clang7CodeGen15CodeGenFunctionEN4llvm9Intrinsic2IDEPKNS_8CallExprENS3_11Instruction9BinaryOpsE", "EmitBinaryAtomicPost(clang::CodeGen::CodeGenFunction&, llvm::Intrinsic::ID, clang::CallExpr const*, llvm::Instruction::BinaryOps)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitTargetBuiltinExprEjPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitTargetBuiltinExpr(unsigned int, clang::CallExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitARMBuiltinExprEjPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitARMBuiltinExpr(unsigned int, clang::CallExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitX86BuiltinExprEjPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitX86BuiltinExpr(unsigned int, clang::CallExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitPPCBuiltinExprEjPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitPPCBuiltinExpr(unsigned int, clang::CallExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitNeonSplatEPN4llvm5ValueEPNS2_8ConstantE", "clang::CodeGen::CodeGenFunction::EmitNeonSplat(llvm::Value*, llvm::Constant*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitNeonCallEPN4llvm8FunctionERNS2_15SmallVectorImplIPNS2_5ValueEEEPKcjb", "clang::CodeGen::CodeGenFunction::EmitNeonCall(llvm::Function*, llvm::SmallVectorImpl<llvm::Value*>&, char const*, unsigned int, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitNeonShiftVectorEPN4llvm5ValueEPKNS2_4TypeEb", "clang::CodeGen::CodeGenFunction::EmitNeonShiftVector(llvm::Value*, llvm::Type const*, bool)"},
    {"_Z19GetPointeeAlignmentRN5clang7CodeGen15CodeGenFunctionEPKNS_4ExprE", "GetPointeeAlignment(clang::CodeGen::CodeGenFunction&, clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11BuildVectorERKN4llvm15SmallVectorImplIPNS2_5ValueEEE", "clang::CodeGen::CodeGenFunction::BuildVector(llvm::SmallVectorImpl<llvm::Value*> const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule28TryEmitBaseDestructorAsAliasEPKNS_17CXXDestructorDeclE", "clang::CodeGen::CodeGenModule::TryEmitBaseDestructorAsAlias(clang::CXXDestructorDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule24TryEmitDefinitionAsAliasENS0_10GlobalDeclES2_", "clang::CodeGen::CodeGenModule::TryEmitDefinitionAsAlias(clang::CodeGen::GlobalDecl, clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule19EmitCXXConstructorsEPKNS_18CXXConstructorDeclE", "clang::CodeGen::CodeGenModule::EmitCXXConstructors(clang::CXXConstructorDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule18EmitCXXConstructorEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeE", "clang::CodeGen::CodeGenModule::EmitCXXConstructor(clang::CXXConstructorDecl const*, clang::CXXCtorType)"},
    {"_ZN5clang7CodeGen13CodeGenModule23GetAddrOfCXXConstructorEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeE", "clang::CodeGen::CodeGenModule::GetAddrOfCXXConstructor(clang::CXXConstructorDecl const*, clang::CXXCtorType)"},
    {"_ZN5clang7CodeGen13CodeGenModule18EmitCXXDestructorsEPKNS_17CXXDestructorDeclE", "clang::CodeGen::CodeGenModule::EmitCXXDestructors(clang::CXXDestructorDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule17EmitCXXDestructorEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeE", "clang::CodeGen::CodeGenModule::EmitCXXDestructor(clang::CXXDestructorDecl const*, clang::CXXDtorType)"},
    {"_ZN5clang7CodeGen13CodeGenModule22GetAddrOfCXXDestructorEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeE", "clang::CodeGen::CodeGenModule::GetAddrOfCXXDestructor(clang::CXXDestructorDecl const*, clang::CXXDtorType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16BuildVirtualCallEPKNS_13CXXMethodDeclEPN4llvm5ValueEPKNS5_4TypeE", "clang::CodeGen::CodeGenFunction::BuildVirtualCall(clang::CXXMethodDecl const*, llvm::Value*, llvm::Type const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25BuildAppleKextVirtualCallEPKNS_13CXXMethodDeclEPNS_19NestedNameSpecifierEPKN4llvm4TypeE", "clang::CodeGen::CodeGenFunction::BuildAppleKextVirtualCall(clang::CXXMethodDecl const*, clang::NestedNameSpecifier*, llvm::Type const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction35BuildAppleKextVirtualDestructorCallEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenFunction::BuildAppleKextVirtualDestructorCall(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16BuildVirtualCallEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeEPN4llvm5ValueEPKNS6_4TypeE", "clang::CodeGen::CodeGenFunction::BuildVirtualCall(clang::CXXDestructorDecl const*, clang::CXXDtorType, llvm::Value*, llvm::Type const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule15GetAddrOfGlobalENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::GetAddrOfGlobal(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen8CGCXXABID0Ev", "clang::CodeGen::CGCXXABI::~CGCXXABI()"},
    {"_ZN5clang7CodeGen8CGCXXABID1Ev", "clang::CodeGen::CGCXXABI::~CGCXXABI()"},
    {"_ZN5clang7CodeGen8CGCXXABID2Ev", "clang::CodeGen::CGCXXABI::~CGCXXABI()"},
    {"_ZN5clang7CodeGen8CGCXXABI24ConvertMemberPointerTypeEPKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::ConvertMemberPointerType(clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI31EmitLoadOfMemberFunctionPointerERNS0_15CodeGenFunctionERPN4llvm5ValueES6_PKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::EmitLoadOfMemberFunctionPointer(clang::CodeGen::CodeGenFunction&, llvm::Value*&, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI28EmitMemberDataPointerAddressERNS0_15CodeGenFunctionEPN4llvm5ValueES6_PKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::EmitMemberDataPointerAddress(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI27EmitMemberPointerConversionERNS0_15CodeGenFunctionEPKNS_8CastExprEPN4llvm5ValueE", "clang::CodeGen::CGCXXABI::EmitMemberPointerConversion(clang::CodeGen::CodeGenFunction&, clang::CastExpr const*, llvm::Value*)"},
    {"_ZN5clang7CodeGen8CGCXXABI27EmitMemberPointerComparisonERNS0_15CodeGenFunctionEPN4llvm5ValueES6_PKNS_17MemberPointerTypeEb", "clang::CodeGen::CGCXXABI::EmitMemberPointerComparison(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::MemberPointerType const*, bool)"},
    {"_ZN5clang7CodeGen8CGCXXABI26EmitMemberPointerIsNotNullERNS0_15CodeGenFunctionEPN4llvm5ValueEPKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::EmitMemberPointerIsNotNull(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI27EmitMemberPointerConversionEPN4llvm8ConstantEPKNS_8CastExprE", "clang::CodeGen::CGCXXABI::EmitMemberPointerConversion(llvm::Constant*, clang::CastExpr const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI21EmitNullMemberPointerEPKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::EmitNullMemberPointer(clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI17EmitMemberPointerEPKNS_13CXXMethodDeclE", "clang::CodeGen::CGCXXABI::EmitMemberPointer(clang::CXXMethodDecl const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI21EmitMemberDataPointerEPKNS_17MemberPointerTypeENS_9CharUnitsE", "clang::CodeGen::CGCXXABI::EmitMemberDataPointer(clang::MemberPointerType const*, clang::CharUnits)"},
    {"_ZN5clang7CodeGen8CGCXXABI19isZeroInitializableEPKNS_17MemberPointerTypeE", "clang::CodeGen::CGCXXABI::isZeroInitializable(clang::MemberPointerType const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI14BuildThisParamERNS0_15CodeGenFunctionERN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CGCXXABI::BuildThisParam(clang::CodeGen::CodeGenFunction&, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN5clang7CodeGen8CGCXXABI13EmitThisParamERNS0_15CodeGenFunctionE", "clang::CodeGen::CGCXXABI::EmitThisParam(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen8CGCXXABI19EmitReturnFromThunkERNS0_15CodeGenFunctionENS0_6RValueENS_8QualTypeE", "clang::CodeGen::CGCXXABI::EmitReturnFromThunk(clang::CodeGen::CodeGenFunction&, clang::CodeGen::RValue, clang::QualType)"},
    {"_ZN5clang7CodeGen8CGCXXABI18GetArrayCookieSizeEPKNS_10CXXNewExprE", "clang::CodeGen::CGCXXABI::GetArrayCookieSize(clang::CXXNewExpr const*)"},
    {"_ZN5clang7CodeGen8CGCXXABI21InitializeArrayCookieERNS0_15CodeGenFunctionEPN4llvm5ValueES6_PKNS_10CXXNewExprENS_8QualTypeE", "clang::CodeGen::CGCXXABI::InitializeArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::CXXNewExpr const*, clang::QualType)"},
    {"_ZN5clang7CodeGen8CGCXXABI15ReadArrayCookieERNS0_15CodeGenFunctionEPN4llvm5ValueEPKNS_13CXXDeleteExprENS_8QualTypeERS6_SB_RNS_9CharUnitsE", "clang::CodeGen::CGCXXABI::ReadArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::CXXDeleteExpr const*, clang::QualType, llvm::Value*&, llvm::Value*&, clang::CharUnits&)"},
    {"_ZN5clang7CodeGen8CGCXXABI15EmitGuardedInitERNS0_15CodeGenFunctionERKNS_7VarDeclEPN4llvm14GlobalVariableE", "clang::CodeGen::CGCXXABI::EmitGuardedInit(clang::CodeGen::CodeGenFunction&, clang::VarDecl const&, llvm::GlobalVariable*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS_7CanQualINS_19FunctionNoProtoTypeEEEb", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CanQual<clang::FunctionNoProtoType>, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS_7CanQualINS_4TypeEEERKN4llvm15SmallVectorImplIS4_EERKNS_12FunctionType7ExtInfoEb", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CanQual<clang::Type>, llvm::SmallVectorImpl<clang::CanQual<clang::Type> > const&, clang::FunctionType::ExtInfo const&, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS_7CanQualINS_17FunctionProtoTypeEEEb", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CanQual<clang::FunctionProtoType>, bool)"},
    {"_Z15getFunctionInfoRN5clang7CodeGen12CodeGenTypesERN4llvm15SmallVectorImplINS_7CanQualINS_4TypeEEEEENS5_INS_17FunctionProtoTypeEEEb", "getFunctionInfo(clang::CodeGen::CodeGenTypes&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&, clang::CanQual<clang::FunctionProtoType>, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_13CXXRecordDeclEPKNS_17FunctionProtoTypeE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CXXRecordDecl const*, clang::FunctionProtoType const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_13CXXMethodDeclE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CXXMethodDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CXXConstructorDecl const*, clang::CXXCtorType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CXXDestructorDecl const*, clang::CXXDtorType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_12FunctionDeclE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::FunctionDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoEPKNS_14ObjCMethodDeclE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::ObjCMethodDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS0_10GlobalDeclE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS_8QualTypeERKN4llvm11SmallVectorISt4pairINS0_6RValueES2_ELj16EEERKNS_12FunctionType7ExtInfoE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::QualType, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::FunctionType::ExtInfo const&)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15getFunctionInfoENS_8QualTypeERKN4llvm11SmallVectorISt4pairIPKNS_7VarDeclES2_ELj16EEERKNS_12FunctionType7ExtInfoE", "clang::CodeGen::CodeGenTypes::getFunctionInfo(clang::QualType, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u> const&, clang::FunctionType::ExtInfo const&)"},
    {"_ZN5clang7CodeGen14CGFunctionInfoC1EjbjNS_7CanQualINS_4TypeEEEPKS4_j", "clang::CodeGen::CGFunctionInfo::CGFunctionInfo(unsigned int, bool, unsigned int, clang::CanQual<clang::Type>, clang::CanQual<clang::Type> const*, unsigned int)"},
    {"_ZN5clang7CodeGen14CGFunctionInfoC2EjbjNS_7CanQualINS_4TypeEEEPKS4_j", "clang::CodeGen::CGFunctionInfo::CGFunctionInfo(unsigned int, bool, unsigned int, clang::CanQual<clang::Type>, clang::CanQual<clang::Type> const*, unsigned int)"},
    {"_ZN5clang7CodeGen12CodeGenTypes16GetExpandedTypesENS_8QualTypeERSt6vectorIPKN4llvm4TypeESaIS7_EEb", "clang::CodeGen::CodeGenTypes::GetExpandedTypes(clang::QualType, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> >&, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18ExpandTypeFromArgsENS_8QualTypeENS0_6LValueEN4llvm14ilist_iteratorINS4_8ArgumentEEE", "clang::CodeGen::CodeGenFunction::ExpandTypeFromArgs(clang::QualType, clang::CodeGen::LValue, llvm::ilist_iterator<llvm::Argument>)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16ExpandTypeToArgsENS_8QualTypeENS0_6RValueERN4llvm11SmallVectorIPNS4_5ValueELj16EEE", "clang::CodeGen::CodeGenFunction::ExpandTypeToArgs(clang::QualType, clang::CodeGen::RValue, llvm::SmallVector<llvm::Value*, 16u>&)"},
    {"_ZN5clang7CodeGen13CodeGenModule18ReturnTypeUsesSRetERKNS0_14CGFunctionInfoE", "clang::CodeGen::CodeGenModule::ReturnTypeUsesSRet(clang::CodeGen::CGFunctionInfo const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule19ReturnTypeUsesFPRetENS_8QualTypeE", "clang::CodeGen::CodeGenModule::ReturnTypeUsesFPRet(clang::QualType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15GetFunctionTypeENS0_10GlobalDeclE", "clang::CodeGen::CodeGenTypes::GetFunctionType(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen12CodeGenTypes15GetFunctionTypeERKNS0_14CGFunctionInfoEbb", "clang::CodeGen::CodeGenTypes::GetFunctionType(clang::CodeGen::CGFunctionInfo const&, bool, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes24GetFunctionTypeForVTableENS0_10GlobalDeclE", "clang::CodeGen::CodeGenTypes::GetFunctionTypeForVTable(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule22ConstructAttributeListERKNS0_14CGFunctionInfoEPKNS_4DeclERN4llvm11SmallVectorINS8_18AttributeWithIndexELj8EEERj", "clang::CodeGen::CodeGenModule::ConstructAttributeList(clang::CodeGen::CGFunctionInfo const&, clang::Decl const*, llvm::SmallVector<llvm::AttributeWithIndex, 8u>&, unsigned int&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitFunctionPrologERKNS0_14CGFunctionInfoEPN4llvm8FunctionERKNS5_11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitFunctionProlog(clang::CodeGen::CGFunctionInfo const&, llvm::Function*, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u> const&)"},
    {"_Z18CreateCoercedStorePN4llvm5ValueES1_bRN5clang7CodeGen15CodeGenFunctionE", "CreateCoercedStore(llvm::Value*, llvm::Value*, bool, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitFunctionEpilogERKNS0_14CGFunctionInfoE", "clang::CodeGen::CodeGenFunction::EmitFunctionEpilog(clang::CodeGen::CGFunctionInfo const&)"},
    {"_Z17CreateCoercedLoadPN4llvm5ValueEPKNS_4TypeERN5clang7CodeGen15CodeGenFunctionE", "CreateCoercedLoad(llvm::Value*, llvm::Type const*, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitDelegateCallArgEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::EmitDelegateCallArg(clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitCallArgEPKNS_4ExprENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitCallArg(clang::Expr const*, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitCallOrInvokeEPN4llvm5ValueEPKS4_S6_RKNS2_5TwineE", "clang::CodeGen::CodeGenFunction::EmitCallOrInvoke(llvm::Value*, llvm::Value* const*, llvm::Value* const*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction8EmitCallERKNS0_14CGFunctionInfoEPN4llvm5ValueENS0_15ReturnValueSlotERKNS5_11SmallVectorISt4pairINS0_6RValueENS_8QualTypeEELj16EEEPKNS_4DeclEPPNS5_11InstructionE", "clang::CodeGen::CodeGenFunction::EmitCall(clang::CodeGen::CGFunctionInfo const&, llvm::Value*, clang::CodeGen::ReturnValueSlot, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::Decl const*, llvm::Instruction**)"},
    {"_ZN5clang7CodeGen15CodeGenFunction9EmitVAArgEPN4llvm5ValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitVAArg(llvm::Value*, clang::QualType)"},
    {"_Z34EnterStructPointerForCoercedAccessPN4llvm5ValueEPKNS_10StructTypeEyRN5clang7CodeGen15CodeGenFunctionE", "EnterStructPointerForCoercedAccess(llvm::Value*, llvm::StructType const*, unsigned long long, clang::CodeGen::CodeGenFunction&)"},
    {"_Z24CoerceIntOrPtrToIntOrPtrPN4llvm5ValueEPKNS_4TypeERN5clang7CodeGen15CodeGenFunctionE", "CoerceIntOrPtrToIntOrPtr(llvm::Value*, llvm::Type const*, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen14CGFunctionInfo7ProfileIPKNS_7CanQualINS_4TypeEEEEEvRN4llvm16FoldingSetNodeIDERKNS_12FunctionType7ExtInfoES5_T_SF_", "void clang::CodeGen::CGFunctionInfo::Profile<clang::CanQual<clang::Type> const*>(llvm::FoldingSetNodeID&, clang::FunctionType::ExtInfo const&, clang::CanQual<clang::Type>, clang::CanQual<clang::Type> const*, clang::CanQual<clang::Type> const*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE18CreateConstGEP2_32EPNS_5ValueEjjRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstGEP2_32(llvm::Value*, unsigned int, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE13CreateRetVoidEv", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateRetVoid()"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateRetEPNS_5ValueE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateRet(llvm::Value*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCallIPKPNS_5ValueEEEPNS_8CallInstES7_T_SC_RKNS_5TwineE", "llvm::CallInst* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall<llvm::Value* const*>(llvm::Value*, llvm::Value* const*, llvm::Value* const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateInvokeIPKPNS_5ValueEEEPNS_10InvokeInstES7_PNS_10BasicBlockESD_T_SE_RKNS_5TwineE", "llvm::InvokeInst* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInvoke<llvm::Value* const*>(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value* const*, llvm::Value* const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE18CreateExtractValueEPNS_5ValueEjRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateExtractValue(llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm10InvokeInstC2IPKPNS_5ValueEEES3_PNS_10BasicBlockES7_T_S8_jRKNS_5TwineEPNS_11InstructionE", "llvm::InvokeInst::InvokeInst<llvm::Value* const*>(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value* const*, llvm::Value* const*, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CallInstC2IPKPNS_5ValueEEES3_T_S6_RKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst<llvm::Value* const*>(llvm::Value*, llvm::Value* const*, llvm::Value* const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE13CreateIntCastEPNS_5ValueEPKNS_4TypeEbRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateIntCast(llvm::Value*, llvm::Type const*, bool, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule28GetNonVirtualBaseClassOffsetEPKNS_13CXXRecordDeclEPKPKNS_16CXXBaseSpecifierES9_", "clang::CodeGen::CodeGenModule::GetNonVirtualBaseClassOffset(clang::CXXRecordDecl const*, clang::CXXBaseSpecifier const* const*, clang::CXXBaseSpecifier const* const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction37GetAddressOfDirectBaseInCompleteClassEPN4llvm5ValueEPKNS_13CXXRecordDeclES7_b", "clang::CodeGen::CodeGenFunction::GetAddressOfDirectBaseInCompleteClass(llvm::Value*, clang::CXXRecordDecl const*, clang::CXXRecordDecl const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21GetAddressOfBaseClassEPN4llvm5ValueEPKNS_13CXXRecordDeclEPKPKNS_16CXXBaseSpecifierESC_b", "clang::CodeGen::CodeGenFunction::GetAddressOfBaseClass(llvm::Value*, clang::CXXRecordDecl const*, clang::CXXBaseSpecifier const* const*, clang::CXXBaseSpecifier const* const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25GetVirtualBaseClassOffsetEPN4llvm5ValueEPKNS_13CXXRecordDeclES7_", "clang::CodeGen::CodeGenFunction::GetVirtualBaseClassOffset(llvm::Value*, clang::CXXRecordDecl const*, clang::CXXRecordDecl const*)"},
    {"_Z31ApplyNonVirtualAndVirtualOffsetRN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueEyS5_", "ApplyNonVirtualAndVirtualOffset(clang::CodeGen::CodeGenFunction&, llvm::Value*, unsigned long long, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24GetAddressOfDerivedClassEPN4llvm5ValueEPKNS_13CXXRecordDeclEPKPKNS_16CXXBaseSpecifierESC_b", "clang::CodeGen::CodeGenFunction::GetAddressOfDerivedClass(llvm::Value*, clang::CXXRecordDecl const*, clang::CXXBaseSpecifier const* const*, clang::CXXBaseSpecifier const* const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitConstructorBodyERN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitConstructorBody(llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction30EmitDelegateCXXConstructorCallEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeERKN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitDelegateCXXConstructorCall(clang::CXXConstructorDecl const*, clang::CXXCtorType, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u> const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitCtorPrologueEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeERN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitCtorPrologue(clang::CXXConstructorDecl const*, clang::CXXCtorType, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24InitializeVTablePointersEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenFunction::InitializeVTablePointers(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitDestructorBodyERN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitDestructorBody(llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EnterDtorCleanupsEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeE", "clang::CodeGen::CodeGenFunction::EnterDtorCleanups(clang::CXXDestructorDecl const*, clang::CXXDtorType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitCXXDestructorCallEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeEbPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::EmitCXXDestructorCall(clang::CXXDestructorDecl const*, clang::CXXDtorType, bool, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitCXXAggrConstructorCallEPKNS_18CXXConstructorDeclEPKNS_17ConstantArrayTypeEPN4llvm5ValueENS_17ConstExprIteratorESB_b", "clang::CodeGen::CodeGenFunction::EmitCXXAggrConstructorCall(clang::CXXConstructorDecl const*, clang::ConstantArrayType const*, llvm::Value*, clang::ConstExprIterator, clang::ConstExprIterator, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitCXXAggrConstructorCallEPKNS_18CXXConstructorDeclEPN4llvm5ValueES7_NS_17ConstExprIteratorES8_b", "clang::CodeGen::CodeGenFunction::EmitCXXAggrConstructorCall(clang::CXXConstructorDecl const*, llvm::Value*, llvm::Value*, clang::ConstExprIterator, clang::ConstExprIterator, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitCXXConstructorCallEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeEbPN4llvm5ValueENS_17ConstExprIteratorES9_", "clang::CodeGen::CodeGenFunction::EmitCXXConstructorCall(clang::CXXConstructorDecl const*, clang::CXXCtorType, bool, llvm::Value*, clang::ConstExprIterator, clang::ConstExprIterator)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitCXXAggrDestructorCallEPKNS_17CXXDestructorDeclEPKNS_9ArrayTypeEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::EmitCXXAggrDestructorCall(clang::CXXDestructorDecl const*, clang::ArrayType const*, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitCXXAggrDestructorCallEPKNS_17CXXDestructorDeclEPN4llvm5ValueES7_", "clang::CodeGen::CodeGenFunction::EmitCXXAggrDestructorCall(clang::CXXDestructorDecl const*, llvm::Value*, llvm::Value*)"},
    {"_Z15GetVTTParameterRN5clang7CodeGen15CodeGenFunctionENS0_10GlobalDeclEb", "GetVTTParameter(clang::CodeGen::CodeGenFunction&, clang::CodeGen::GlobalDecl, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction30EmitSynthesizedCXXCopyCtorCallEPKNS_18CXXConstructorDeclEPN4llvm5ValueES7_NS_17ConstExprIteratorES8_", "clang::CodeGen::CodeGenFunction::EmitSynthesizedCXXCopyCtorCall(clang::CXXConstructorDecl const*, llvm::Value*, llvm::Value*, clang::ConstExprIterator, clang::ConstExprIterator)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21PushDestructorCleanupEPKNS_17CXXDestructorDeclEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::PushDestructorCleanup(clang::CXXDestructorDecl const*, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21PushDestructorCleanupENS_8QualTypeEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::PushDestructorCleanup(clang::QualType, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12GetVTablePtrEPN4llvm5ValueEPKNS2_4TypeE", "clang::CodeGen::CodeGenFunction::GetVTablePtr(llvm::Value*, llvm::Type const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23InitializeVTablePointerENS0_13BaseSubobjectEPKNS_13CXXRecordDeclEyPN4llvm8ConstantES5_", "clang::CodeGen::CodeGenFunction::InitializeVTablePointer(clang::CodeGen::BaseSubobject, clang::CXXRecordDecl const*, unsigned long long, llvm::Constant*, clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24InitializeVTablePointersENS0_13BaseSubobjectEPKNS_13CXXRecordDeclEybPN4llvm8ConstantES5_RNS6_11SmallPtrSetIS5_Lj4EEE", "clang::CodeGen::CodeGenFunction::InitializeVTablePointers(clang::CodeGen::BaseSubobject, clang::CXXRecordDecl const*, unsigned long long, bool, llvm::Constant*, clang::CXXRecordDecl const*, llvm::SmallPtrSet<clang::CXXRecordDecl const*, 4u>&)"},
    {"_ZN12_GLOBAL__N_113CallLocalDtorD1Ev", "(anonymous namespace)::CallLocalDtor::~CallLocalDtor()"},
    {"_ZN12_GLOBAL__N_113CallLocalDtorD0Ev", "(anonymous namespace)::CallLocalDtor::~CallLocalDtor()"},
    {"_ZN12_GLOBAL__N_113CallLocalDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallLocalDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_113CallFieldDtorD1Ev", "(anonymous namespace)::CallFieldDtor::~CallFieldDtor()"},
    {"_ZN12_GLOBAL__N_113CallFieldDtorD0Ev", "(anonymous namespace)::CallFieldDtor::~CallFieldDtor()"},
    {"_ZN12_GLOBAL__N_113CallFieldDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallFieldDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_118CallArrayFieldDtorD1Ev", "(anonymous namespace)::CallArrayFieldDtor::~CallArrayFieldDtor()"},
    {"_ZN12_GLOBAL__N_118CallArrayFieldDtorD0Ev", "(anonymous namespace)::CallArrayFieldDtor::~CallArrayFieldDtor()"},
    {"_ZN12_GLOBAL__N_118CallArrayFieldDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallArrayFieldDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_114CallDtorDeleteD1Ev", "(anonymous namespace)::CallDtorDelete::~CallDtorDelete()"},
    {"_ZN12_GLOBAL__N_114CallDtorDeleteD0Ev", "(anonymous namespace)::CallDtorDelete::~CallDtorDelete()"},
    {"_ZN12_GLOBAL__N_114CallDtorDelete4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallDtorDelete::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_112CallBaseDtorD1Ev", "(anonymous namespace)::CallBaseDtor::~CallBaseDtor()"},
    {"_ZN12_GLOBAL__N_112CallBaseDtorD0Ev", "(anonymous namespace)::CallBaseDtor::~CallBaseDtor()"},
    {"_ZN12_GLOBAL__N_112CallBaseDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallBaseDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_Z24EmitAggMemberInitializerRN5clang7CodeGen15CodeGenFunctionENS0_6LValueEPN4llvm5ValueEPNS_18CXXCtorInitializerENS_8QualTypeEj", "EmitAggMemberInitializer(clang::CodeGen::CodeGenFunction&, clang::CodeGen::LValue, llvm::Value*, clang::CXXCtorInitializer*, clang::QualType, unsigned int)"},
    {"_ZN12_GLOBAL__N_114CallMemberDtorD1Ev", "(anonymous namespace)::CallMemberDtor::~CallMemberDtor()"},
    {"_ZN12_GLOBAL__N_114CallMemberDtorD0Ev", "(anonymous namespace)::CallMemberDtor::~CallMemberDtor()"},
    {"_ZN12_GLOBAL__N_114CallMemberDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallMemberDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_121DynamicThisUseCheckerEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::DynamicThisUseChecker, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang20EvaluatedExprVisitorIN12_GLOBAL__N_121DynamicThisUseCheckerEE9VisitStmtEPNS_4StmtE", "clang::EvaluatedExprVisitor<(anonymous namespace)::DynamicThisUseChecker>::VisitStmt(clang::Stmt*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE18CreateConstGEP1_64EPNS_5ValueEyRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstGEP1_64(llvm::Value*, unsigned long long, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE26CreateConstInBoundsGEP2_64EPNS_5ValueEyyRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstInBoundsGEP2_64(llvm::Value*, unsigned long long, unsigned long long, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15DominatingValueINS0_6RValueEE10saved_type11needsSavingES2_", "clang::CodeGen::DominatingValue<clang::CodeGen::RValue>::saved_type::needsSaving(clang::CodeGen::RValue)"},
    {"_ZN5clang7CodeGen15DominatingValueINS0_6RValueEE10saved_type4saveERNS0_15CodeGenFunctionES2_", "clang::CodeGen::DominatingValue<clang::CodeGen::RValue>::saved_type::save(clang::CodeGen::CodeGenFunction&, clang::CodeGen::RValue)"},
    {"_ZN5clang7CodeGen15DominatingValueINS0_6RValueEE10saved_type7restoreERNS0_15CodeGenFunctionE", "clang::CodeGen::DominatingValue<clang::CodeGen::RValue>::saved_type::restore(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen12EHScopeStack8allocateEm", "clang::CodeGen::EHScopeStack::allocate(unsigned long)"},
    {"_ZNK5clang7CodeGen12EHScopeStack21getEnclosingEHCleanupENS1_8iteratorE", "clang::CodeGen::EHScopeStack::getEnclosingEHCleanup(clang::CodeGen::EHScopeStack::iterator) const"},
    {"_ZN5clang7CodeGen12EHScopeStack11pushCleanupENS0_11CleanupKindEm", "clang::CodeGen::EHScopeStack::pushCleanup(clang::CodeGen::CleanupKind, unsigned long)"},
    {"_ZN5clang7CodeGen12EHScopeStack10popCleanupEv", "clang::CodeGen::EHScopeStack::popCleanup()"},
    {"_ZN5clang7CodeGen12EHScopeStack13popNullFixupsEv", "clang::CodeGen::EHScopeStack::popNullFixups()"},
    {"_ZN5clang7CodeGen12EHScopeStack10pushFilterEj", "clang::CodeGen::EHScopeStack::pushFilter(unsigned int)"},
    {"_ZN5clang7CodeGen12EHScopeStack9popFilterEv", "clang::CodeGen::EHScopeStack::popFilter()"},
    {"_ZN5clang7CodeGen12EHScopeStack9pushCatchEj", "clang::CodeGen::EHScopeStack::pushCatch(unsigned int)"},
    {"_ZN5clang7CodeGen12EHScopeStack13pushTerminateEv", "clang::CodeGen::EHScopeStack::pushTerminate()"},
    {"_ZN5clang7CodeGen15CodeGenFunction19initFullExprCleanupEv", "clang::CodeGen::CodeGenFunction::initFullExprCleanup()"},
    {"_ZN5clang7CodeGen12EHScopeStack7CleanupD0Ev", "clang::CodeGen::EHScopeStack::Cleanup::~Cleanup()"},
    {"_ZN5clang7CodeGen12EHScopeStack7CleanupD1Ev", "clang::CodeGen::EHScopeStack::Cleanup::~Cleanup()"},
    {"_ZN5clang7CodeGen12EHScopeStack7CleanupD2Ev", "clang::CodeGen::EHScopeStack::Cleanup::~Cleanup()"},
    {"_ZN5clang7CodeGen15CodeGenFunction19ResolveBranchFixupsEPN4llvm10BasicBlockE", "clang::CodeGen::CodeGenFunction::ResolveBranchFixups(llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16PopCleanupBlocksENS0_12EHScopeStack15stable_iteratorE", "clang::CodeGen::CodeGenFunction::PopCleanupBlocks(clang::CodeGen::EHScopeStack::stable_iterator)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15PopCleanupBlockEb", "clang::CodeGen::CodeGenFunction::PopCleanupBlock(bool)"},
    {"_Z29ForwardPrebranchedFallthroughPN4llvm10BasicBlockES1_S1_", "ForwardPrebranchedFallthrough(llvm::BasicBlock*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_Z20SimplifyCleanupEntryRN5clang7CodeGen15CodeGenFunctionEPN4llvm10BasicBlockE", "SimplifyCleanupEntry(clang::CodeGen::CodeGenFunction&, llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20getEHCleanupDestSlotEv", "clang::CodeGen::CodeGenFunction::getEHCleanupDestSlot()"},
    {"_Z11EmitCleanupRN5clang7CodeGen15CodeGenFunctionEPNS0_12EHScopeStack7CleanupEbPN4llvm5ValueE", "EmitCleanup(clang::CodeGen::CodeGenFunction&, clang::CodeGen::EHScopeStack::Cleanup*, bool, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24getNormalCleanupDestSlotEv", "clang::CodeGen::CodeGenFunction::getNormalCleanupDestSlot()"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitBranchThroughCleanupENS1_8JumpDestE", "clang::CodeGen::CodeGenFunction::EmitBranchThroughCleanup(clang::CodeGen::CodeGenFunction::JumpDest)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitBranchThroughEHCleanupENS1_10UnwindDestE", "clang::CodeGen::CodeGenFunction::EmitBranchThroughEHCleanup(clang::CodeGen::CodeGenFunction::UnwindDest)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20ActivateCleanupBlockENS0_12EHScopeStack15stable_iteratorE", "clang::CodeGen::CodeGenFunction::ActivateCleanupBlock(clang::CodeGen::EHScopeStack::stable_iterator)"},
    {"_Z27SetupCleanupBlockActivationRN5clang7CodeGen15CodeGenFunctionENS0_12EHScopeStack15stable_iteratorE15ForActivation_t", "SetupCleanupBlockActivation(clang::CodeGen::CodeGenFunction&, clang::CodeGen::EHScopeStack::stable_iterator, ForActivation_t)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22DeactivateCleanupBlockENS0_12EHScopeStack15stable_iteratorE", "clang::CodeGen::CodeGenFunction::DeactivateCleanupBlock(clang::CodeGen::EHScopeStack::stable_iterator)"},
    {"_ZN5clang7CodeGen14EHCleanupScope14addBranchAfterEPN4llvm11ConstantIntEPNS2_10BasicBlockE", "clang::CodeGen::EHCleanupScope::addBranchAfter(llvm::ConstantInt*, llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen14EHCleanupScope16addEHBranchAfterEPN4llvm11ConstantIntEPNS2_10BasicBlockE", "clang::CodeGen::EHCleanupScope::addEHBranchAfter(llvm::ConstantInt*, llvm::BasicBlock*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7CodeGen11BranchFixupELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeGen::BranchFixup, false>::grow(unsigned long)"},
    {"_ZN5clang7CodeGen14EHCleanupScopeD2Ev", "clang::CodeGen::EHCleanupScope::~EHCleanupScope()"},
    {"_ZN5clang7CodeGen11CGDebugInfoC1ERNS0_13CodeGenModuleE", "clang::CodeGen::CGDebugInfo::CGDebugInfo(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen11CGDebugInfoC2ERNS0_13CodeGenModuleE", "clang::CodeGen::CGDebugInfo::CGDebugInfo(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17CreateCompileUnitEv", "clang::CodeGen::CGDebugInfo::CreateCompileUnit()"},
    {"_ZN5clang7CodeGen11CGDebugInfoD1Ev", "clang::CodeGen::CGDebugInfo::~CGDebugInfo()"},
    {"_ZN5clang7CodeGen11CGDebugInfoD2Ev", "clang::CodeGen::CGDebugInfo::~CGDebugInfo()"},
    {"_ZN5clang7CodeGen11CGDebugInfo11setLocationENS_14SourceLocationE", "clang::CodeGen::CGDebugInfo::setLocation(clang::SourceLocation)"},
    {"_ZN5clang7CodeGen11CGDebugInfo20getContextDescriptorEPKNS_4DeclE", "clang::CodeGen::CGDebugInfo::getContextDescriptor(clang::Decl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo20getOrCreateNameSpaceEPKNS_13NamespaceDeclE", "clang::CodeGen::CGDebugInfo::getOrCreateNameSpace(clang::NamespaceDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15getOrCreateTypeENS_8QualTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::getOrCreateType(clang::QualType, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo19getOrCreateMainFileEv", "clang::CodeGen::CGDebugInfo::getOrCreateMainFile()"},
    {"_ZN5clang7CodeGen11CGDebugInfo15getFunctionNameEPKNS_12FunctionDeclE", "clang::CodeGen::CGDebugInfo::getFunctionName(clang::FunctionDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17getObjCMethodNameEPKNS_14ObjCMethodDeclE", "clang::CodeGen::CGDebugInfo::getObjCMethodName(clang::ObjCMethodDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo12getClassNameEPNS_10RecordDeclE", "clang::CodeGen::CGDebugInfo::getClassName(clang::RecordDecl*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15getOrCreateFileENS_14SourceLocationE", "clang::CodeGen::CGDebugInfo::getOrCreateFile(clang::SourceLocation)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17getCurrentDirnameEv", "clang::CodeGen::CGDebugInfo::getCurrentDirname()"},
    {"_ZN5clang7CodeGen11CGDebugInfo13getLineNumberENS_14SourceLocationE", "clang::CodeGen::CGDebugInfo::getLineNumber(clang::SourceLocation)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15getColumnNumberENS_14SourceLocationE", "clang::CodeGen::CGDebugInfo::getColumnNumber(clang::SourceLocation)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_11BuiltinTypeE", "clang::CodeGen::CGDebugInfo::CreateType(clang::BuiltinType const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_11ComplexTypeE", "clang::CodeGen::CGDebugInfo::CreateType(clang::ComplexType const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo19CreateQualifiedTypeENS_8QualTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateQualifiedType(clang::QualType, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_21ObjCObjectPointerTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::ObjCObjectPointerType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo21CreatePointerLikeTypeEjPKNS_4TypeENS_8QualTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreatePointerLikeType(unsigned int, clang::Type const*, clang::QualType, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_11PointerTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::PointerType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17CreatePointeeTypeENS_8QualTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreatePointeeType(clang::QualType, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_16BlockPointerTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::BlockPointerType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo16CreateMemberTypeEN4llvm6DIFileENS_8QualTypeENS2_9StringRefEPy", "clang::CodeGen::CGDebugInfo::CreateMemberType(llvm::DIFile, clang::QualType, llvm::StringRef, unsigned long long*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_11TypedefTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::TypedefType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_12FunctionTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::FunctionType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo19CollectRecordFieldsEPKNS_10RecordDeclEN4llvm6DIFileERNS5_15SmallVectorImplIPNS5_5ValueEEE", "clang::CodeGen::CGDebugInfo::CollectRecordFields(clang::RecordDecl const*, llvm::DIFile, llvm::SmallVectorImpl<llvm::Value*>&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo21getOrCreateMethodTypeEPKNS_13CXXMethodDeclEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::getOrCreateMethodType(clang::CXXMethodDecl const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo23CreateCXXMemberFunctionEPKNS_13CXXMethodDeclEN4llvm6DIFileENS5_6DITypeE", "clang::CodeGen::CGDebugInfo::CreateCXXMemberFunction(clang::CXXMethodDecl const*, llvm::DIFile, llvm::DIType)"},
    {"_ZN5clang7CodeGen11CGDebugInfo25CollectCXXMemberFunctionsEPKNS_13CXXRecordDeclEN4llvm6DIFileERNS5_15SmallVectorImplIPNS5_5ValueEEENS5_6DITypeE", "clang::CodeGen::CGDebugInfo::CollectCXXMemberFunctions(clang::CXXRecordDecl const*, llvm::DIFile, llvm::SmallVectorImpl<llvm::Value*>&, llvm::DIType)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17CollectCXXFriendsEPKNS_13CXXRecordDeclEN4llvm6DIFileERNS5_15SmallVectorImplIPNS5_5ValueEEENS5_6DITypeE", "clang::CodeGen::CGDebugInfo::CollectCXXFriends(clang::CXXRecordDecl const*, llvm::DIFile, llvm::SmallVectorImpl<llvm::Value*>&, llvm::DIType)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15CollectCXXBasesEPKNS_13CXXRecordDeclEN4llvm6DIFileERNS5_15SmallVectorImplIPNS5_5ValueEEENS5_6DITypeE", "clang::CodeGen::CGDebugInfo::CollectCXXBases(clang::CXXRecordDecl const*, llvm::DIFile, llvm::SmallVectorImpl<llvm::Value*>&, llvm::DIType)"},
    {"_ZN5clang7CodeGen11CGDebugInfo24getOrCreateVTablePtrTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::getOrCreateVTablePtrType(llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo13getVTableNameEPKNS_13CXXRecordDeclE", "clang::CodeGen::CGDebugInfo::getVTableName(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17CollectVTableInfoEPKNS_13CXXRecordDeclEN4llvm6DIFileERNS5_15SmallVectorImplIPNS5_5ValueEEE", "clang::CodeGen::CGDebugInfo::CollectVTableInfo(clang::CXXRecordDecl const*, llvm::DIFile, llvm::SmallVectorImpl<llvm::Value*>&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo21getOrCreateRecordTypeENS_8QualTypeENS_14SourceLocationE", "clang::CodeGen::CGDebugInfo::getOrCreateRecordType(clang::QualType, clang::SourceLocation)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_10RecordTypeE", "clang::CodeGen::CGDebugInfo::CreateType(clang::RecordType const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_14ObjCObjectTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::ObjCObjectType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_17ObjCInterfaceTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::ObjCInterfaceType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_7TagTypeE", "clang::CodeGen::CGDebugInfo::CreateType(clang::TagType const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo14CreateEnumTypeEPKNS_8EnumDeclE", "clang::CodeGen::CGDebugInfo::CreateEnumType(clang::EnumDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_10VectorTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::VectorType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_9ArrayTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::ArrayType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_19LValueReferenceTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::LValueReferenceType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_19RValueReferenceTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::RValueReferenceType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo10CreateTypeEPKNS_17MemberPointerTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateType(clang::MemberPointerType const*, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo14CreateTypeNodeENS_8QualTypeEN4llvm6DIFileE", "clang::CodeGen::CGDebugInfo::CreateTypeNode(clang::QualType, llvm::DIFile)"},
    {"_ZN5clang7CodeGen11CGDebugInfo17EmitFunctionStartENS0_10GlobalDeclENS_8QualTypeEPN4llvm8FunctionERNS4_9IRBuilderILb0ENS4_14ConstantFolderENS4_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitFunctionStart(clang::CodeGen::GlobalDecl, clang::QualType, llvm::Function*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo13EmitStopPointERN4llvm9IRBuilderILb0ENS2_14ConstantFolderENS2_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitStopPoint(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo25UpdateLineDirectiveRegionERN4llvm9IRBuilderILb0ENS2_14ConstantFolderENS2_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::UpdateLineDirectiveRegion(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15EmitRegionStartERN4llvm9IRBuilderILb0ENS2_14ConstantFolderENS2_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitRegionStart(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo13EmitRegionEndERN4llvm9IRBuilderILb0ENS2_14ConstantFolderENS2_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitRegionEnd(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo15EmitFunctionEndERN4llvm9IRBuilderILb0ENS2_14ConstantFolderENS2_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitFunctionEnd(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo28EmitTypeForVarWithBlocksAttrEPKNS_9ValueDeclEPy", "clang::CodeGen::CGDebugInfo::EmitTypeForVarWithBlocksAttr(clang::ValueDecl const*, unsigned long long*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo11EmitDeclareEPKNS_7VarDeclEjPN4llvm5ValueERNS5_9IRBuilderILb0ENS5_14ConstantFolderENS5_24IRBuilderDefaultInserterILb0EEEEEb", "clang::CodeGen::CGDebugInfo::EmitDeclare(clang::VarDecl const*, unsigned int, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, bool)"},
    {"_ZN5clang7CodeGen11CGDebugInfo11EmitDeclareEPKNS_7VarDeclEjPN4llvm5ValueERNS5_9IRBuilderILb0ENS5_14ConstantFolderENS5_24IRBuilderDefaultInserterILb0EEEEERKNS0_11CGBlockInfoE", "clang::CodeGen::CGDebugInfo::EmitDeclare(clang::VarDecl const*, unsigned int, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo25EmitDeclareOfAutoVariableEPKNS_7VarDeclEPN4llvm5ValueERNS5_9IRBuilderILb0ENS5_14ConstantFolderENS5_24IRBuilderDefaultInserterILb0EEEEE", "clang::CodeGen::CGDebugInfo::EmitDeclareOfAutoVariable(clang::VarDecl const*, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo33EmitDeclareOfBlockDeclRefVariableEPKNS_7VarDeclEPN4llvm5ValueERNS5_9IRBuilderILb0ENS5_14ConstantFolderENS5_24IRBuilderDefaultInserterILb0EEEEERKNS0_11CGBlockInfoE", "clang::CodeGen::CGDebugInfo::EmitDeclareOfBlockDeclRefVariable(clang::VarDecl const*, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN5clang7CodeGen11CGDebugInfo24EmitDeclareOfArgVariableEPKNS_7VarDeclEPN4llvm5ValueERNS5_9IRBuilderILb0ENS5_14ConstantFolderENS5_24IRBuilderDefaultInserterILb0EEEEEb", "clang::CodeGen::CGDebugInfo::EmitDeclareOfArgVariable(clang::VarDecl const*, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, bool)"},
    {"_ZN5clang7CodeGen11CGDebugInfo18EmitGlobalVariableEPN4llvm14GlobalVariableEPKNS_7VarDeclE", "clang::CodeGen::CGDebugInfo::EmitGlobalVariable(llvm::GlobalVariable*, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo18EmitGlobalVariableEPN4llvm14GlobalVariableEPNS_17ObjCInterfaceDeclE", "clang::CodeGen::CGDebugInfo::EmitGlobalVariable(llvm::GlobalVariable*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang7CodeGen11CGDebugInfo18EmitGlobalVariableEPKNS_9ValueDeclEPN4llvm8ConstantE", "clang::CodeGen::CGDebugInfo::EmitGlobalVariable(clang::ValueDecl const*, llvm::Constant*)"},
    {"_ZNK5clang15ASTRecordLayout24getBaseClassOffsetInBitsEPKNS_13CXXRecordDeclE", "clang::ASTRecordLayout::getBaseClassOffsetInBits(clang::CXXRecordDecl const*) const"},
    {"_ZN4llvm8DenseMapIPKN5clang13NamespaceDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::NamespaceDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::NamespaceDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::FindAndConstruct(clang::NamespaceDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13NamespaceDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::NamespaceDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::NamespaceDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::InsertIntoBucket(clang::NamespaceDecl const* const&, llvm::WeakVH const&, std::pair<clang::NamespaceDecl const*, llvm::WeakVH>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13NamespaceDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::NamespaceDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::NamespaceDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::Decl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::FindAndConstruct(clang::Decl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::Decl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::InsertIntoBucket(clang::Decl const* const&, llvm::WeakVH const&, std::pair<clang::Decl const*, llvm::WeakVH>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Decl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm10TrackingVHINS0_6MDNodeEEESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::TrackingVH<llvm::MDNode>, std::allocator<llvm::TrackingVH<llvm::MDNode> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::TrackingVH<llvm::MDNode>*, std::vector<llvm::TrackingVH<llvm::MDNode>, std::allocator<llvm::TrackingVH<llvm::MDNode> > > >, llvm::TrackingVH<llvm::MDNode> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang12FunctionDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::FunctionDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::FunctionDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::FindAndConstruct(clang::FunctionDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang12FunctionDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::FunctionDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::FunctionDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::InsertIntoBucket(clang::FunctionDecl const* const&, llvm::WeakVH const&, std::pair<clang::FunctionDecl const*, llvm::WeakVH>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang12FunctionDeclENS_6WeakVHENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::FunctionDecl const*, llvm::WeakVH, llvm::DenseMapInfo<clang::FunctionDecl const*>, llvm::DenseMapInfo<llvm::WeakVH> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPvNS_6WeakVHENS_12DenseMapInfoIS1_EENS3_IS2_EEE16FindAndConstructERKS1_", "llvm::DenseMap<void*, llvm::WeakVH, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<llvm::WeakVH> >::FindAndConstruct(void* const&)"},
    {"_ZN4llvm8DenseMapIPvNS_6WeakVHENS_12DenseMapInfoIS1_EENS3_IS2_EEE16InsertIntoBucketERKS1_RKS2_PSt4pairIS1_S2_E", "llvm::DenseMap<void*, llvm::WeakVH, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<llvm::WeakVH> >::InsertIntoBucket(void* const&, llvm::WeakVH const&, std::pair<void*, llvm::WeakVH>*)"},
    {"_ZN4llvm8DenseMapIPvNS_6WeakVHENS_12DenseMapInfoIS1_EENS3_IS2_EEE4growEj", "llvm::DenseMap<void*, llvm::WeakVH, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<llvm::WeakVH> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKcNS_6WeakVHENS_12DenseMapInfoIS2_EENS4_IS3_EEE16FindAndConstructERKS2_", "llvm::DenseMap<char const*, llvm::WeakVH, llvm::DenseMapInfo<char const*>, llvm::DenseMapInfo<llvm::WeakVH> >::FindAndConstruct(char const* const&)"},
    {"_ZN4llvm8DenseMapIPKcNS_6WeakVHENS_12DenseMapInfoIS2_EENS4_IS3_EEE16InsertIntoBucketERKS2_RKS3_PSt4pairIS2_S3_E", "llvm::DenseMap<char const*, llvm::WeakVH, llvm::DenseMapInfo<char const*>, llvm::DenseMapInfo<llvm::WeakVH> >::InsertIntoBucket(char const* const&, llvm::WeakVH const&, std::pair<char const*, llvm::WeakVH>*)"},
    {"_ZN4llvm8DenseMapIPKcNS_6WeakVHENS_12DenseMapInfoIS2_EENS4_IS3_EEE4growEj", "llvm::DenseMap<char const*, llvm::WeakVH, llvm::DenseMapInfo<char const*>, llvm::DenseMapInfo<llvm::WeakVH> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction8EmitDeclERKNS_4DeclE", "clang::CodeGen::CodeGenFunction::EmitDecl(clang::Decl const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitVarDeclERKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::EmitVarDecl(clang::VarDecl const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitAutoVarDeclERKNS_7VarDeclEPFvRS1_S4_PN4llvm5ValueEE", "clang::CodeGen::CodeGenFunction::EmitAutoVarDecl(clang::VarDecl const&, void (*)(clang::CodeGen::CodeGenFunction&, clang::VarDecl const&, llvm::Value*))"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitStaticVarDeclERKNS_7VarDeclEN4llvm11GlobalValue12LinkageTypesE", "clang::CodeGen::CodeGenFunction::EmitStaticVarDecl(clang::VarDecl const&, llvm::GlobalValue::LinkageTypes)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19CreateStaticVarDeclERKNS_7VarDeclEPKcN4llvm11GlobalValue12LinkageTypesE", "clang::CodeGen::CodeGenFunction::CreateStaticVarDecl(clang::VarDecl const&, char const*, llvm::GlobalValue::LinkageTypes)"},
    {"_Z17GetStaticDeclNameRN5clang7CodeGen15CodeGenFunctionERKNS_7VarDeclEPKc", "GetStaticDeclName(clang::CodeGen::CodeGenFunction&, clang::VarDecl const&, char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29AddInitializerToStaticVarDeclERKNS_7VarDeclEPN4llvm14GlobalVariableE", "clang::CodeGen::CodeGenFunction::AddInitializerToStaticVarDecl(clang::VarDecl const&, llvm::GlobalVariable*)"},
    {"_ZNK5clang7CodeGen15CodeGenFunction22getByRefValueLLVMFieldEPKNS_9ValueDeclE", "clang::CodeGen::CodeGenFunction::getByRefValueLLVMField(clang::ValueDecl const*) const"},
    {"_ZN5clang7CodeGen15CodeGenFunction22BuildBlockByrefAddressEPN4llvm5ValueEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::BuildBlockByrefAddress(llvm::Value*, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14BuildByRefTypeEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::BuildByRefType(clang::VarDecl const*)"},
    {"_Z28emitStoresForInitAfterMemsetPN4llvm8ConstantEPNS_5ValueERNS_9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEEE", "emitStoresForInitAfterMemset(llvm::Constant*, llvm::Value*, llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitParmDeclERKNS_7VarDeclEPN4llvm5ValueEb", "clang::CodeGen::CodeGenFunction::EmitParmDecl(clang::VarDecl const&, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_116CallBlockReleaseD1Ev", "(anonymous namespace)::CallBlockRelease::~CallBlockRelease()"},
    {"_ZN12_GLOBAL__N_116CallBlockReleaseD0Ev", "(anonymous namespace)::CallBlockRelease::~CallBlockRelease()"},
    {"_ZN12_GLOBAL__N_116CallBlockRelease4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallBlockRelease::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_119CallCleanupFunctionD1Ev", "(anonymous namespace)::CallCleanupFunction::~CallCleanupFunction()"},
    {"_ZN12_GLOBAL__N_119CallCleanupFunctionD0Ev", "(anonymous namespace)::CallCleanupFunction::~CallCleanupFunction()"},
    {"_ZN12_GLOBAL__N_119CallCleanupFunction4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallCleanupFunction::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_111CallVarDtorD1Ev", "(anonymous namespace)::CallVarDtor::~CallVarDtor()"},
    {"_ZN12_GLOBAL__N_111CallVarDtorD0Ev", "(anonymous namespace)::CallVarDtor::~CallVarDtor()"},
    {"_ZN12_GLOBAL__N_111CallVarDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallVarDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_113CallArrayDtorD1Ev", "(anonymous namespace)::CallArrayDtor::~CallArrayDtor()"},
    {"_ZN12_GLOBAL__N_113CallArrayDtorD0Ev", "(anonymous namespace)::CallArrayDtor::~CallArrayDtor()"},
    {"_ZN12_GLOBAL__N_113CallArrayDtor4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallArrayDtor::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_116CallStackRestoreD1Ev", "(anonymous namespace)::CallStackRestore::~CallStackRestore()"},
    {"_ZN12_GLOBAL__N_116CallStackRestoreD0Ev", "(anonymous namespace)::CallStackRestore::~CallStackRestore()"},
    {"_ZN12_GLOBAL__N_116CallStackRestore4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallStackRestore::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_Z35canEmitInitWithFewStoresAfterMemsetPN4llvm8ConstantERj", "canEmitInitWithFewStoresAfterMemset(llvm::Constant*, unsigned int&)"},
    {"_ZNK5clang4Decl7getAttrINS_11CleanupAttrEEEPT_v", "clang::CleanupAttr* clang::Decl::getAttr<clang::CleanupAttr>() const"},
    {"_ZN4llvm8DenseMapIPKN5clang9ValueDeclESt4pairIPKNS_4TypeEjENS_12DenseMapInfoIS4_EENSA_IS9_EEE16InsertIntoBucketERKS4_RKS9_PS5_IS4_S9_E", "llvm::DenseMap<clang::ValueDecl const*, std::pair<llvm::Type const*, unsigned int>, llvm::DenseMapInfo<clang::ValueDecl const*>, llvm::DenseMapInfo<std::pair<llvm::Type const*, unsigned int> > >::InsertIntoBucket(clang::ValueDecl const* const&, std::pair<llvm::Type const*, unsigned int> const&, std::pair<clang::ValueDecl const*, std::pair<llvm::Type const*, unsigned int> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9ValueDeclESt4pairIPKNS_4TypeEjENS_12DenseMapInfoIS4_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::ValueDecl const*, std::pair<llvm::Type const*, unsigned int>, llvm::DenseMapInfo<clang::ValueDecl const*>, llvm::DenseMapInfo<std::pair<llvm::Type const*, unsigned int> > >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitCXXGlobalVarDeclInitERKNS_7VarDeclEPN4llvm8ConstantE", "clang::CodeGen::CodeGenFunction::EmitCXXGlobalVarDeclInit(clang::VarDecl const&, llvm::Constant*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29EmitCXXGlobalDtorRegistrationEPN4llvm8ConstantES4_", "clang::CodeGen::CodeGenFunction::EmitCXXGlobalDtorRegistration(llvm::Constant*, llvm::Constant*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitCXXGuardedInitERKNS_7VarDeclEPN4llvm14GlobalVariableE", "clang::CodeGen::CodeGenFunction::EmitCXXGuardedInit(clang::VarDecl const&, llvm::GlobalVariable*)"},
    {"_ZN5clang7CodeGen13CodeGenModule28EmitCXXGlobalVarDeclInitFuncEPKNS_7VarDeclEPN4llvm14GlobalVariableE", "clang::CodeGen::CodeGenModule::EmitCXXGlobalVarDeclInitFunc(clang::VarDecl const*, llvm::GlobalVariable*)"},
    {"_Z34CreateGlobalInitOrDestructFunctionRN5clang7CodeGen13CodeGenModuleEPKN4llvm12FunctionTypeENS3_9StringRefE", "CreateGlobalInitOrDestructFunction(clang::CodeGen::CodeGenModule&, llvm::FunctionType const*, llvm::StringRef)"},
    {"_ZN5clang7CodeGen15CodeGenFunction32GenerateCXXGlobalVarDeclInitFuncEPN4llvm8FunctionEPKNS_7VarDeclEPNS2_14GlobalVariableE", "clang::CodeGen::CodeGenFunction::GenerateCXXGlobalVarDeclInitFunc(llvm::Function*, clang::VarDecl const*, llvm::GlobalVariable*)"},
    {"_ZN5clang7CodeGen13CodeGenModule21EmitCXXGlobalInitFuncEv", "clang::CodeGen::CodeGenModule::EmitCXXGlobalInitFunc()"},
    {"_ZN5clang7CodeGen15CodeGenFunction25GenerateCXXGlobalInitFuncEPN4llvm8FunctionEPPNS2_8ConstantEj", "clang::CodeGen::CodeGenFunction::GenerateCXXGlobalInitFunc(llvm::Function*, llvm::Constant**, unsigned int)"},
    {"_ZN5clang7CodeGen13CodeGenModule21EmitCXXGlobalDtorFuncEv", "clang::CodeGen::CodeGenModule::EmitCXXGlobalDtorFunc()"},
    {"_ZN5clang7CodeGen15CodeGenFunction25GenerateCXXGlobalDtorFuncEPN4llvm8FunctionERKSt6vectorISt4pairINS2_6WeakVHEPNS2_8ConstantEESaISA_EE", "clang::CodeGen::CodeGenFunction::GenerateCXXGlobalDtorFunc(llvm::Function*, std::vector<std::pair<llvm::WeakVH, llvm::Constant*>, std::allocator<std::pair<llvm::WeakVH, llvm::Constant*> > > const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction31GenerateCXXAggrDestructorHelperEPKNS_17CXXDestructorDeclEPKNS_9ArrayTypeEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::GenerateCXXAggrDestructorHelper(clang::CXXDestructorDecl const*, clang::ArrayType const*, llvm::Value*)"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIN5clang7CodeGen16OrderGlobalInitsEPNS_8FunctionEEEEiPKvS9_", "int llvm::array_pod_sort_comparator<std::pair<clang::CodeGen::OrderGlobalInits, llvm::Function*> >(void const*, void const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule15AddCXXDtorEntryEPN4llvm8ConstantES4_", "clang::CodeGen::CodeGenModule::AddCXXDtorEntry(llvm::Constant*, llvm::Constant*)"},
    {"_ZNK5clang4Decl7getAttrINS_16InitPriorityAttrEEEPT_v", "clang::InitPriorityAttr* clang::Decl::getAttr<clang::InitPriorityAttr>() const"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCallEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIN5clang7CodeGen16OrderGlobalInitsEPNS_8FunctionEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::CodeGen::OrderGlobalInits, llvm::Function*>, false>::grow(unsigned long)"},
    {"_ZNSt6vectorISt4pairIN4llvm6WeakVHEPNS1_8ConstantEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::WeakVH, llvm::Constant*>, std::allocator<std::pair<llvm::WeakVH, llvm::Constant*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::Constant*>*, std::vector<std::pair<llvm::WeakVH, llvm::Constant*>, std::allocator<std::pair<llvm::WeakVH, llvm::Constant*> > > >, std::pair<llvm::WeakVH, llvm::Constant*> const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm6WeakVHEPNS4_8ConstantEES9_EET0_T_SB_SA_", "std::pair<llvm::WeakVH, llvm::Constant*>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::WeakVH, llvm::Constant*>*, std::pair<llvm::WeakVH, llvm::Constant*>*>(std::pair<llvm::WeakVH, llvm::Constant*>*, std::pair<llvm::WeakVH, llvm::Constant*>*, std::pair<llvm::WeakVH, llvm::Constant*>*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26getUnwindResumeOrRethrowFnEv", "clang::CodeGen::CodeGenFunction::getUnwindResumeOrRethrowFn()"},
    {"_ZN5clang7CodeGen13EHPersonality3getERKNS_11LangOptionsE", "clang::CodeGen::EHPersonality::get(clang::LangOptions const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule19SimplifyPersonalityEv", "clang::CodeGen::CodeGenModule::SimplifyPersonality()"},
    {"_Z25PersonalityHasOnlyCXXUsesPN4llvm8ConstantE", "PersonalityHasOnlyCXXUses(llvm::Constant*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16getExceptionSlotEv", "clang::CodeGen::CodeGenFunction::getExceptionSlot()"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitCXXThrowExprEPKNS_12CXXThrowExprE", "clang::CodeGen::CodeGenFunction::EmitCXXThrowExpr(clang::CXXThrowExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitStartEHSpecEPKNS_4DeclE", "clang::CodeGen::CodeGenFunction::EmitStartEHSpec(clang::Decl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitEndEHSpecEPKNS_4DeclE", "clang::CodeGen::CodeGenFunction::EmitEndEHSpec(clang::Decl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitCXXTryStmtERKNS_10CXXTryStmtE", "clang::CodeGen::CodeGenFunction::EmitCXXTryStmt(clang::CXXTryStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EnterCXXTryStmtERKNS_10CXXTryStmtEb", "clang::CodeGen::CodeGenFunction::EnterCXXTryStmt(clang::CXXTryStmt const&, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14ExitCXXTryStmtERKNS_10CXXTryStmtEb", "clang::CodeGen::CodeGenFunction::ExitCXXTryStmt(clang::CXXTryStmt const&, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17getInvokeDestImplEv", "clang::CodeGen::CodeGenFunction::getInvokeDestImpl()"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitLandingPadEv", "clang::CodeGen::CodeGenFunction::EmitLandingPad()"},
    {"_ZN5clang7CodeGen15CodeGenFunction22getTerminateLandingPadEv", "clang::CodeGen::CodeGenFunction::getTerminateLandingPad()"},
    {"_Z22getOpaquePersonalityFnRN5clang7CodeGen13CodeGenModuleERKNS0_13EHPersonalityE", "getOpaquePersonalityFn(clang::CodeGen::CodeGenModule&, clang::CodeGen::EHPersonality const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19getTerminateHandlerEv", "clang::CodeGen::CodeGenFunction::getTerminateHandler()"},
    {"_ZN5clang7CodeGen15CodeGenFunction14getRethrowDestEv", "clang::CodeGen::CodeGenFunction::getRethrowDest()"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EnterFinallyBlockEPKNS_4StmtEPN4llvm8ConstantES7_S7_", "clang::CodeGen::CodeGenFunction::EnterFinallyBlock(clang::Stmt const*, llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16ExitFinallyBlockERNS1_11FinallyInfoE", "clang::CodeGen::CodeGenFunction::ExitFinallyBlock(clang::CodeGen::CodeGenFunction::FinallyInfo&)"},
    {"_Z14getTerminateFnRN5clang7CodeGen15CodeGenFunctionE", "getTerminateFn(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN12_GLOBAL__N_114PerformFinallyD1Ev", "(anonymous namespace)::PerformFinally::~PerformFinally()"},
    {"_ZN12_GLOBAL__N_114PerformFinallyD0Ev", "(anonymous namespace)::PerformFinally::~PerformFinally()"},
    {"_ZN12_GLOBAL__N_114PerformFinally4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::PerformFinally::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_122CallEndCatchForFinallyD1Ev", "(anonymous namespace)::CallEndCatchForFinally::~CallEndCatchForFinally()"},
    {"_ZN12_GLOBAL__N_122CallEndCatchForFinallyD0Ev", "(anonymous namespace)::CallEndCatchForFinally::~CallEndCatchForFinally()"},
    {"_ZN12_GLOBAL__N_122CallEndCatchForFinally4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallEndCatchForFinally::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_111CallRethrowD1Ev", "(anonymous namespace)::CallRethrow::~CallRethrow()"},
    {"_ZN12_GLOBAL__N_111CallRethrowD0Ev", "(anonymous namespace)::CallRethrow::~CallRethrow()"},
    {"_ZN12_GLOBAL__N_111CallRethrow4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallRethrow::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_Z14CallBeginCatchRN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueEb", "CallBeginCatch(clang::CodeGen::CodeGenFunction&, llvm::Value*, bool)"},
    {"_Z14InitCatchParamRN5clang7CodeGen15CodeGenFunctionERKNS_7VarDeclEPN4llvm5ValueE", "InitCatchParam(clang::CodeGen::CodeGenFunction&, clang::VarDecl const&, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_112CallEndCatchD1Ev", "(anonymous namespace)::CallEndCatch::~CallEndCatch()"},
    {"_ZN12_GLOBAL__N_112CallEndCatchD0Ev", "(anonymous namespace)::CallEndCatch::~CallEndCatch()"},
    {"_ZN12_GLOBAL__N_112CallEndCatch4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallEndCatch::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEED1Ev", "clang::CodeGen::EHScopeStack::ConditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::~ConditionalCleanup1()"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEED0Ev", "clang::CodeGen::EHScopeStack::ConditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::~ConditionalCleanup1()"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEE4EmitERNS0_15CodeGenFunctionEb", "clang::CodeGen::EHScopeStack::ConditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_113FreeException4EmitERN5clang7CodeGen15CodeGenFunctionEbPN4llvm5ValueE", "(anonymous namespace)::FreeException::Emit(clang::CodeGen::CodeGenFunction&, bool, llvm::Value*)"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEED1Ev", "clang::CodeGen::EHScopeStack::UnconditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::~UnconditionalCleanup1()"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEED0Ev", "clang::CodeGen::EHScopeStack::UnconditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::~UnconditionalCleanup1()"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEE4EmitERNS0_15CodeGenFunctionEb", "clang::CodeGen::EHScopeStack::UnconditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateInvokeEPNS_5ValueEPNS_10BasicBlockES8_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInvoke(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19getUnreachableBlockEv", "clang::CodeGen::CodeGenFunction::getUnreachableBlock()"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE13CreateInvoke3EPNS_5ValueEPNS_10BasicBlockES8_S6_S6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInvoke3(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7CodeGen12EHCatchScope7HandlerELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeGen::EHCatchScope::Handler, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEN5clang7CodeGen15CodeGenFunction10UnwindDestENS_12DenseMapInfoIS2_EENS7_IS6_EEE16InsertIntoBucketERKS2_RKS6_PSt4pairIS2_S6_E", "llvm::DenseMap<llvm::Value*, clang::CodeGen::CodeGenFunction::UnwindDest, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<clang::CodeGen::CodeGenFunction::UnwindDest> >::InsertIntoBucket(llvm::Value* const&, clang::CodeGen::CodeGenFunction::UnwindDest const&, std::pair<llvm::Value*, clang::CodeGen::CodeGenFunction::UnwindDest>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEN5clang7CodeGen15CodeGenFunction10UnwindDestENS_12DenseMapInfoIS2_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::Value*, clang::CodeGen::CodeGenFunction::UnwindDest, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<clang::CodeGen::CodeGenFunction::UnwindDest> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen19DominatingLLVMValue4saveERNS0_15CodeGenFunctionEPN4llvm5ValueE", "clang::CodeGen::DominatingLLVMValue::save(clang::CodeGen::CodeGenFunction&, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCastToVoidPtrEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::EmitCastToVoidPtr(llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16CreateTempAllocaEPKN4llvm4TypeERKNS2_5TwineE", "clang::CodeGen::CodeGenFunction::CreateTempAlloca(llvm::Type const*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14InitTempAllocaEPN4llvm10AllocaInstEPNS2_5ValueE", "clang::CodeGen::CodeGenFunction::InitTempAlloca(llvm::AllocaInst*, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12CreateIRTempENS_8QualTypeERKN4llvm5TwineE", "clang::CodeGen::CodeGenFunction::CreateIRTemp(clang::QualType, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13CreateMemTempENS_8QualTypeERKN4llvm5TwineE", "clang::CodeGen::CodeGenFunction::CreateMemTemp(clang::QualType, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EvaluateExprAsBoolEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EvaluateExprAsBool(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitIgnoredExprEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitIgnoredExpr(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitAnyExprEPKNS_4ExprENS0_12AggValueSlotEb", "clang::CodeGen::CodeGenFunction::EmitAnyExpr(clang::Expr const*, clang::CodeGen::AggValueSlot, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction10EmitLValueEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitLValue(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitAnyExprToTempEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitAnyExprToTemp(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitAnyExprToMemEPKNS_4ExprEPN4llvm5ValueEbb", "clang::CodeGen::CodeGenFunction::EmitAnyExprToMem(clang::Expr const*, llvm::Value*, bool, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitStoreThroughLValueENS0_6RValueENS0_6LValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitStoreThroughLValue(clang::CodeGen::RValue, clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitReferenceBindingToExprEPKNS_4ExprEPKNS_9NamedDeclE", "clang::CodeGen::CodeGenFunction::EmitReferenceBindingToExpr(clang::Expr const*, clang::NamedDecl const*)"},
    {"_Z27EmitExprForReferenceBindingRN5clang7CodeGen15CodeGenFunctionEPKNS_4ExprERPN4llvm5ValueERPKNS_17CXXDestructorDeclEPKNS_9NamedDeclE", "EmitExprForReferenceBinding(clang::CodeGen::CodeGenFunction&, clang::Expr const*, llvm::Value*&, clang::CXXDestructorDecl const*&, clang::NamedDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18getAccessedFieldNoEjPKN4llvm8ConstantE", "clang::CodeGen::CodeGenFunction::getAccessedFieldNo(unsigned int, llvm::Constant const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction9EmitCheckEPN4llvm5ValueEj", "clang::CodeGen::CodeGenFunction::EmitCheck(llvm::Value*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction9getTrapBBEv", "clang::CodeGen::CodeGenFunction::getTrapBB()"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitComplexPrePostIncDecEPKNS_13UnaryOperatorENS0_6LValueEbb", "clang::CodeGen::CodeGenFunction::EmitComplexPrePostIncDec(clang::UnaryOperator const*, clang::CodeGen::LValue, bool, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14GetUndefRValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::GetUndefRValue(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitUnsupportedRValueEPKNS_4ExprEPKc", "clang::CodeGen::CodeGenFunction::EmitUnsupportedRValue(clang::Expr const*, char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitUnsupportedLValueEPKNS_4ExprEPKc", "clang::CodeGen::CodeGenFunction::EmitUnsupportedLValue(clang::Expr const*, char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCheckedLValueEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitCheckedLValue(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitObjCSelectorLValueEPKNS_16ObjCSelectorExprE", "clang::CodeGen::CodeGenFunction::EmitObjCSelectorLValue(clang::ObjCSelectorExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitBinaryOperatorLValueEPKNS_14BinaryOperatorE", "clang::CodeGen::CodeGenFunction::EmitBinaryOperatorLValue(clang::BinaryOperator const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitCallExprLValueEPKNS_8CallExprE", "clang::CodeGen::CodeGenFunction::EmitCallExprLValue(clang::CallExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitVAArgExprLValueEPKNS_9VAArgExprE", "clang::CodeGen::CodeGenFunction::EmitVAArgExprLValue(clang::VAArgExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitDeclRefLValueEPKNS_11DeclRefExprE", "clang::CodeGen::CodeGenFunction::EmitDeclRefLValue(clang::DeclRefExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitPredefinedLValueEPKNS_14PredefinedExprE", "clang::CodeGen::CodeGenFunction::EmitPredefinedLValue(clang::PredefinedExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23EmitStringLiteralLValueEPKNS_13StringLiteralE", "clang::CodeGen::CodeGenFunction::EmitStringLiteralLValue(clang::StringLiteral const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitObjCEncodeExprLValueEPKNS_14ObjCEncodeExprE", "clang::CodeGen::CodeGenFunction::EmitObjCEncodeExprLValue(clang::ObjCEncodeExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitBlockDeclRefLValueEPKNS_16BlockDeclRefExprE", "clang::CodeGen::CodeGenFunction::EmitBlockDeclRefLValue(clang::BlockDeclRefExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitCXXConstructLValueEPKNS_16CXXConstructExprE", "clang::CodeGen::CodeGenFunction::EmitCXXConstructLValue(clang::CXXConstructExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitCXXBindTemporaryLValueEPKNS_20CXXBindTemporaryExprE", "clang::CodeGen::CodeGenFunction::EmitCXXBindTemporaryLValue(clang::CXXBindTemporaryExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction28EmitNullInitializationLValueEPKNS_22CXXScalarValueInitExprE", "clang::CodeGen::CodeGenFunction::EmitNullInitializationLValue(clang::CXXScalarValueInitExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitCXXTypeidLValueEPKNS_13CXXTypeidExprE", "clang::CodeGen::CodeGenFunction::EmitCXXTypeidLValue(clang::CXXTypeidExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitObjCMessageExprLValueEPKNS_15ObjCMessageExprE", "clang::CodeGen::CodeGenFunction::EmitObjCMessageExprLValue(clang::ObjCMessageExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitObjCIvarRefLValueEPKNS_15ObjCIvarRefExprE", "clang::CodeGen::CodeGenFunction::EmitObjCIvarRefLValue(clang::ObjCIvarRefExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitStmtExprLValueEPKNS_8StmtExprE", "clang::CodeGen::CodeGenFunction::EmitStmtExprLValue(clang::StmtExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitUnaryOpLValueEPKNS_13UnaryOperatorE", "clang::CodeGen::CodeGenFunction::EmitUnaryOpLValue(clang::UnaryOperator const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitArraySubscriptExprEPKNS_18ArraySubscriptExprE", "clang::CodeGen::CodeGenFunction::EmitArraySubscriptExpr(clang::ArraySubscriptExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitExtVectorElementExprEPKNS_20ExtVectorElementExprE", "clang::CodeGen::CodeGenFunction::EmitExtVectorElementExpr(clang::ExtVectorElementExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitMemberExprEPKNS_10MemberExprE", "clang::CodeGen::CodeGenFunction::EmitMemberExpr(clang::MemberExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitCompoundLiteralLValueEPKNS_19CompoundLiteralExprE", "clang::CodeGen::CodeGenFunction::EmitCompoundLiteralLValue(clang::CompoundLiteralExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29EmitConditionalOperatorLValueEPKNS_19ConditionalOperatorE", "clang::CodeGen::CodeGenFunction::EmitConditionalOperatorLValue(clang::ConditionalOperator const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitCastLValueEPKNS_8CastExprE", "clang::CodeGen::CodeGenFunction::EmitCastLValue(clang::CastExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitLoadOfScalarEPN4llvm5ValueEbjNS_8QualTypeEPNS2_6MDNodeE", "clang::CodeGen::CodeGenFunction::EmitLoadOfScalar(llvm::Value*, bool, unsigned int, clang::QualType, llvm::MDNode*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitFromMemoryEPN4llvm5ValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitFromMemory(llvm::Value*, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitToMemoryEPN4llvm5ValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitToMemory(llvm::Value*, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitStoreOfScalarEPN4llvm5ValueES4_bjNS_8QualTypeEPNS2_6MDNodeE", "clang::CodeGen::CodeGenFunction::EmitStoreOfScalar(llvm::Value*, llvm::Value*, bool, unsigned int, clang::QualType, llvm::MDNode*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitLoadOfLValueENS0_6LValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitLoadOfLValue(clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction32EmitLoadOfExtVectorElementLValueENS0_6LValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitLoadOfExtVectorElementLValue(clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24EmitLoadOfBitfieldLValueENS0_6LValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitLoadOfBitfieldLValue(clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction40EmitStoreThroughExtVectorComponentLValueENS0_6RValueENS0_6LValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitStoreThroughExtVectorComponentLValue(clang::CodeGen::RValue, clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction30EmitStoreThroughBitfieldLValueENS0_6RValueENS0_6LValueENS_8QualTypeEPPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::EmitStoreThroughBitfieldLValue(clang::CodeGen::RValue, clang::CodeGen::LValue, clang::QualType, llvm::Value**)"},
    {"_Z23EmitGlobalVarDeclLValueRN5clang7CodeGen15CodeGenFunctionEPKNS_4ExprEPKNS_7VarDeclE", "EmitGlobalVarDeclLValue(clang::CodeGen::CodeGenFunction&, clang::Expr const*, clang::VarDecl const*)"},
    {"_Z20setObjCGCLValueClassRKN5clang10ASTContextEPKNS_4ExprERNS_7CodeGen6LValueE", "setObjCGCLValueClass(clang::ASTContext const&, clang::Expr const*, clang::CodeGen::LValue&)"},
    {"_Z22EmitFunctionDeclLValueRN5clang7CodeGen15CodeGenFunctionEPKNS_4ExprEPKNS_12FunctionDeclE", "EmitFunctionDeclLValue(clang::CodeGen::CodeGenFunction&, clang::Expr const*, clang::FunctionDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitLValueForFieldEPN4llvm5ValueEPKNS_9FieldDeclEj", "clang::CodeGen::CodeGenFunction::EmitLValueForField(llvm::Value*, clang::FieldDecl const*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitLValueForBitfieldEPN4llvm5ValueEPKNS_9FieldDeclEj", "clang::CodeGen::CodeGenFunction::EmitLValueForBitfield(llvm::Value*, clang::FieldDecl const*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction28EmitLValueForAnonRecordFieldEPN4llvm5ValueEPKNS_17IndirectFieldDeclEj", "clang::CodeGen::CodeGenFunction::EmitLValueForAnonRecordField(llvm::Value*, clang::IndirectFieldDecl const*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction32EmitLValueForFieldInitializationEPN4llvm5ValueEPKNS_9FieldDeclEj", "clang::CodeGen::CodeGenFunction::EmitLValueForFieldInitialization(llvm::Value*, clang::FieldDecl const*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitCallExprEPKNS_8CallExprENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCallExpr(clang::CallExpr const*, clang::CodeGen::ReturnValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction8EmitCallENS_8QualTypeEPN4llvm5ValueENS0_15ReturnValueSlotENS_17ConstExprIteratorES7_PKNS_4DeclE", "clang::CodeGen::CodeGenFunction::EmitCall(clang::QualType, llvm::Value*, clang::CodeGen::ReturnValueSlot, clang::ConstExprIterator, clang::ConstExprIterator, clang::Decl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction33EmitPointerToDataMemberBinaryExprEPKNS_14BinaryOperatorE", "clang::CodeGen::CodeGenFunction::EmitPointerToDataMemberBinaryExpr(clang::BinaryOperator const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitIvarOffsetEPKNS_17ObjCInterfaceDeclEPKNS_12ObjCIvarDeclE", "clang::CodeGen::CodeGenFunction::EmitIvarOffset(clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitLValueForIvarENS_8QualTypeEPN4llvm5ValueEPKNS_12ObjCIvarDeclEj", "clang::CodeGen::CodeGenFunction::EmitLValueForIvar(clang::QualType, llvm::Value*, clang::ObjCIvarDecl const*, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_119SubobjectAdjustmentEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::SubobjectAdjustment>::push_back((anonymous namespace)::SubobjectAdjustment const&)"},
    {"_Z24CreateReferenceTemporaryRN5clang7CodeGen15CodeGenFunctionENS_8QualTypeEPKNS_9NamedDeclE", "CreateReferenceTemporary(clang::CodeGen::CodeGenFunction&, clang::QualType, clang::NamedDecl const*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE17CreateInBoundsGEPIPPNS_5ValueEEES7_S7_T_S9_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInBoundsGEP<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitCallArgsINS_17FunctionProtoTypeEEEvRN4llvm11SmallVectorISt4pairINS0_6RValueENS_8QualTypeEELj16EEEPKT_NS_17ConstExprIteratorESF_", "void clang::CodeGen::CodeGenFunction::EmitCallArgs<clang::FunctionProtoType>(llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u>&, clang::FunctionProtoType const*, clang::ConstExprIterator, clang::ConstExprIterator)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprENS1_7CodeGen6LValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Expr const*, clang::CodeGen::LValue, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<clang::CodeGen::LValue> >::InsertIntoBucket(clang::Expr const* const&, clang::CodeGen::LValue const&, std::pair<clang::Expr const*, clang::CodeGen::LValue>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprENS1_7CodeGen6LValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Expr const*, clang::CodeGen::LValue, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<clang::CodeGen::LValue> >::grow(unsigned int)"},
    {"_ZN4llvm11SmallVectorIPNS_8ConstantELj4EEC2EjRKS2_", "llvm::SmallVector<llvm::Constant*, 4u>::SmallVector(unsigned int, llvm::Constant* const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitAggExprEPKNS_4ExprENS0_12AggValueSlotEb", "clang::CodeGen::CodeGenFunction::EmitAggExpr(clang::Expr const*, clang::CodeGen::AggValueSlot, bool)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_114AggExprEmitterEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::AggExprEmitter, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitAggExprToLValueEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitAggExprToLValue(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitAggregateCopyEPN4llvm5ValueES4_NS_8QualTypeEb", "clang::CodeGen::CodeGenFunction::EmitAggregateCopy(llvm::Value*, llvm::Value*, clang::QualType, bool)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter13VisitCallExprEPKN5clang8CallExprE", "(anonymous namespace)::AggExprEmitter::VisitCallExpr(clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter17EmitFinalDestCopyEPKN5clang4ExprENS1_7CodeGen6LValueEb", "(anonymous namespace)::AggExprEmitter::EmitFinalDestCopy(clang::Expr const*, clang::CodeGen::LValue, bool)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter10EmitGCMoveEPKN5clang4ExprENS1_7CodeGen6RValueE", "(anonymous namespace)::AggExprEmitter::EmitGCMove(clang::Expr const*, clang::CodeGen::RValue)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter26EmitInitializationToLValueEPN5clang4ExprENS1_7CodeGen6LValueENS1_8QualTypeE", "(anonymous namespace)::AggExprEmitter::EmitInitializationToLValue(clang::Expr*, clang::CodeGen::LValue, clang::QualType)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter30EmitNullInitializationToLValueEN5clang7CodeGen6LValueENS1_8QualTypeE", "(anonymous namespace)::AggExprEmitter::EmitNullInitializationToLValue(clang::CodeGen::LValue, clang::QualType)"},
    {"_Z12isSimpleZeroPKN5clang4ExprERNS_7CodeGen15CodeGenFunctionE", "isSimpleZero(clang::Expr const*, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN12_GLOBAL__N_114AggExprEmitter13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::AggExprEmitter::VisitCastExpr(clang::CastExpr*)"},
    {"_Z24GetNumNonZeroBytesInInitPKN5clang4ExprERNS_7CodeGen15CodeGenFunctionE", "GetNumNonZeroBytesInInit(clang::Expr const*, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCXXMemberCallEPKNS_13CXXMethodDeclEPN4llvm5ValueENS0_15ReturnValueSlotES7_S7_NS_17ConstExprIteratorES9_", "clang::CodeGen::CodeGenFunction::EmitCXXMemberCall(clang::CXXMethodDecl const*, llvm::Value*, clang::CodeGen::ReturnValueSlot, llvm::Value*, llvm::Value*, clang::ConstExprIterator, clang::ConstExprIterator)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitCXXMemberCallExprEPKNS_17CXXMemberCallExprENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCXXMemberCallExpr(clang::CXXMemberCallExpr const*, clang::CodeGen::ReturnValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction28EmitCXXMemberPointerCallExprEPKNS_17CXXMemberCallExprENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCXXMemberPointerCallExpr(clang::CXXMemberCallExpr const*, clang::CodeGen::ReturnValueSlot)"},
    {"_Z34canDevirtualizeMemberFunctionCallsRN5clang10ASTContextEPKNS_4ExprEPKNS_13CXXMethodDeclE", "canDevirtualizeMemberFunctionCalls(clang::ASTContext&, clang::Expr const*, clang::CXXMethodDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29EmitCXXOperatorMemberCallExprEPKNS_19CXXOperatorCallExprEPKNS_13CXXMethodDeclENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCXXOperatorMemberCallExpr(clang::CXXOperatorCallExpr const*, clang::CXXMethodDecl const*, clang::CodeGen::ReturnValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitCXXConstructExprEPKNS_16CXXConstructExprENS0_12AggValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCXXConstructExpr(clang::CXXConstructExpr const*, clang::CodeGen::AggValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitSynthesizedCXXCopyCtorEPN4llvm5ValueES4_PKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitSynthesizedCXXCopyCtor(llvm::Value*, llvm::Value*, clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23EmitNewArrayInitializerEPKNS_10CXXNewExprEPN4llvm5ValueES7_", "clang::CodeGen::CodeGenFunction::EmitNewArrayInitializer(clang::CXXNewExpr const*, llvm::Value*, llvm::Value*)"},
    {"_Z23StoreAnyExprIntoOneUnitRN5clang7CodeGen15CodeGenFunctionEPKNS_10CXXNewExprEPN4llvm5ValueE", "StoreAnyExprIntoOneUnit(clang::CodeGen::CodeGenFunction&, clang::CXXNewExpr const*, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitCXXNewExprEPKNS_10CXXNewExprE", "clang::CodeGen::CodeGenFunction::EmitCXXNewExpr(clang::CXXNewExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitDeleteCallEPKNS_12FunctionDeclEPN4llvm5ValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitDeleteCall(clang::FunctionDecl const*, llvm::Value*, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCXXDeleteExprEPKNS_13CXXDeleteExprE", "clang::CodeGen::CodeGenFunction::EmitCXXDeleteExpr(clang::CXXDeleteExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCXXTypeidExprEPKNS_13CXXTypeidExprE", "clang::CodeGen::CodeGenFunction::EmitCXXTypeidExpr(clang::CXXTypeidExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitDynamicCastEPN4llvm5ValueEPKNS_18CXXDynamicCastExprE", "clang::CodeGen::CodeGenFunction::EmitDynamicCast(llvm::Value*, clang::CXXDynamicCastExpr const*)"},
    {"_ZN12_GLOBAL__N_116CallObjectDeleteD1Ev", "(anonymous namespace)::CallObjectDelete::~CallObjectDelete()"},
    {"_ZN12_GLOBAL__N_116CallObjectDeleteD0Ev", "(anonymous namespace)::CallObjectDelete::~CallObjectDelete()"},
    {"_ZN12_GLOBAL__N_116CallObjectDelete4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallObjectDelete::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_115CallArrayDeleteD1Ev", "(anonymous namespace)::CallArrayDelete::~CallArrayDelete()"},
    {"_ZN12_GLOBAL__N_115CallArrayDeleteD0Ev", "(anonymous namespace)::CallArrayDelete::~CallArrayDelete()"},
    {"_ZN12_GLOBAL__N_115CallArrayDelete4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallArrayDelete::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_130CallDeleteDuringConditionalNewD1Ev", "(anonymous namespace)::CallDeleteDuringConditionalNew::~CallDeleteDuringConditionalNew()"},
    {"_ZN12_GLOBAL__N_130CallDeleteDuringConditionalNewD0Ev", "(anonymous namespace)::CallDeleteDuringConditionalNew::~CallDeleteDuringConditionalNew()"},
    {"_ZN12_GLOBAL__N_130CallDeleteDuringConditionalNew4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallDeleteDuringConditionalNew::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_119CallDeleteDuringNewD1Ev", "(anonymous namespace)::CallDeleteDuringNew::~CallDeleteDuringNew()"},
    {"_ZN12_GLOBAL__N_119CallDeleteDuringNewD0Ev", "(anonymous namespace)::CallDeleteDuringNew::~CallDeleteDuringNew()"},
    {"_ZN12_GLOBAL__N_119CallDeleteDuringNew4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallDeleteDuringNew::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitComplexExprEPKNS_4ExprEbb", "clang::CodeGen::CodeGenFunction::EmitComplexExpr(clang::Expr const*, bool, bool)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter5VisitEPN5clang4ExprE", "(anonymous namespace)::ComplexExprEmitter::Visit(clang::Expr*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23EmitComplexExprIntoAddrEPKNS_4ExprEPN4llvm5ValueEb", "clang::CodeGen::CodeGenFunction::EmitComplexExprIntoAddr(clang::Expr const*, llvm::Value*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18StoreComplexToAddrESt4pairIPN4llvm5ValueES5_ES5_b", "clang::CodeGen::CodeGenFunction::StoreComplexToAddr(std::pair<llvm::Value*, llvm::Value*>, llvm::Value*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19LoadComplexFromAddrEPN4llvm5ValueEb", "clang::CodeGen::CodeGenFunction::LoadComplexFromAddr(llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter17EmitLoadOfComplexEPN4llvm5ValueEb", "(anonymous namespace)::ComplexExprEmitter::EmitLoadOfComplex(llvm::Value*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction27EmitComplexAssignmentLValueEPKNS_14BinaryOperatorE", "clang::CodeGen::CodeGenFunction::EmitComplexAssignmentLValue(clang::BinaryOperator const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction35EmitComplexCompoundAssignmentLValueEPKNS_22CompoundAssignOperatorE", "clang::CodeGen::CodeGenFunction::EmitComplexCompoundAssignmentLValue(clang::CompoundAssignOperator const*)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter10EmitBinMulERKNS0_9BinOpInfoE", "(anonymous namespace)::ComplexExprEmitter::EmitBinMul((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter10EmitBinDivERKNS0_9BinOpInfoE", "(anonymous namespace)::ComplexExprEmitter::EmitBinDiv((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter10EmitBinSubERKNS0_9BinOpInfoE", "(anonymous namespace)::ComplexExprEmitter::EmitBinSub((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter10EmitBinAddERKNS0_9BinOpInfoE", "(anonymous namespace)::ComplexExprEmitter::EmitBinAdd((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter24EmitCompoundAssignLValueEPKN5clang22CompoundAssignOperatorEMS0_FSt4pairIPN4llvm5ValueES8_ERKNS0_9BinOpInfoEERS9_", "(anonymous namespace)::ComplexExprEmitter::EmitCompoundAssignLValue(clang::CompoundAssignOperator const*, std::pair<llvm::Value*, llvm::Value*> ((anonymous namespace)::ComplexExprEmitter::*)((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&), std::pair<llvm::Value*, llvm::Value*>&)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter24EmitComplexToComplexCastESt4pairIPN4llvm5ValueES4_EN5clang8QualTypeES7_", "(anonymous namespace)::ComplexExprEmitter::EmitComplexToComplexCast(std::pair<llvm::Value*, llvm::Value*>, clang::QualType, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter22EmitStoreThroughLValueESt4pairIPN4llvm5ValueES4_EN5clang7CodeGen6LValueE", "(anonymous namespace)::ComplexExprEmitter::EmitStoreThroughLValue(std::pair<llvm::Value*, llvm::Value*>, clang::CodeGen::LValue)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_118ComplexExprEmitterESt4pairIPN4llvm5ValueES6_EE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::ComplexExprEmitter, std::pair<llvm::Value*, llvm::Value*> >::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter9VisitExprEPN5clang4ExprE", "(anonymous namespace)::ComplexExprEmitter::VisitExpr(clang::Expr*)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter16EmitLoadOfLValueEPKN5clang4ExprE", "(anonymous namespace)::ComplexExprEmitter::EmitLoadOfLValue(clang::Expr const*)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter8EmitCastEN5clang8CastKindEPNS1_4ExprENS1_8QualTypeE", "(anonymous namespace)::ComplexExprEmitter::EmitCast(clang::CastKind, clang::Expr*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118ComplexExprEmitter18EmitCompoundAssignEPKN5clang22CompoundAssignOperatorEMS0_FSt4pairIPN4llvm5ValueES8_ERKNS0_9BinOpInfoEE", "(anonymous namespace)::ComplexExprEmitter::EmitCompoundAssign(clang::CompoundAssignOperator const*, std::pair<llvm::Value*, llvm::Value*> ((anonymous namespace)::ComplexExprEmitter::*)((anonymous namespace)::ComplexExprEmitter::BinOpInfo const&))"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprESt4pairIPNS_5ValueES7_ENS_12DenseMapInfoIS4_EENS9_IS8_EEE16InsertIntoBucketERKS4_RKS8_PS5_IS4_S8_E", "llvm::DenseMap<clang::Expr const*, std::pair<llvm::Value*, llvm::Value*>, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::Value*> > >::InsertIntoBucket(clang::Expr const* const&, std::pair<llvm::Value*, llvm::Value*> const&, std::pair<clang::Expr const*, std::pair<llvm::Value*, llvm::Value*> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprESt4pairIPNS_5ValueES7_ENS_12DenseMapInfoIS4_EENS9_IS8_EEE4growEj", "llvm::DenseMap<clang::Expr const*, std::pair<llvm::Value*, llvm::Value*>, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::Value*> > >::grow(unsigned int)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFNegEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFNeg(llvm::Value*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule16EmitConstantExprEPKNS_4ExprENS_8QualTypeEPNS0_15CodeGenFunctionE", "clang::CodeGen::CodeGenModule::EmitConstantExpr(clang::Expr const*, clang::QualType, clang::CodeGen::CodeGenFunction*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_116ConstExprEmitterEPN4llvm8ConstantEE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::ConstExprEmitter, llvm::Constant*>::Visit(clang::Stmt*)"},
    {"_ZN5clang7CodeGen13CodeGenModule24getMemberPointerConstantEPKNS_13UnaryOperatorE", "clang::CodeGen::CodeGenModule::getMemberPointerConstant(clang::UnaryOperator const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule16EmitNullConstantENS_8QualTypeE", "clang::CodeGen::CodeGenModule::EmitNullConstant(clang::QualType)"},
    {"_Z16EmitNullConstantRN5clang7CodeGen13CodeGenModuleEPKNS_13CXXRecordDeclE", "EmitNullConstant(clang::CodeGen::CodeGenModule&, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_116ConstExprEmitter21VisitCXXConstructExprEPN5clang16CXXConstructExprE", "(anonymous namespace)::ConstExprEmitter::VisitCXXConstructExpr(clang::CXXConstructExpr*)"},
    {"_ZN12_GLOBAL__N_116ConstExprEmitter23EmitArrayInitializationEPN5clang12InitListExprE", "(anonymous namespace)::ConstExprEmitter::EmitArrayInitialization(clang::InitListExpr*)"},
    {"_ZN12_GLOBAL__N_118ConstStructBuilder5BuildEPN5clang12InitListExprE", "(anonymous namespace)::ConstStructBuilder::Build(clang::InitListExpr*)"},
    {"_ZN12_GLOBAL__N_118ConstStructBuilder21ConvertStructToPackedEv", "(anonymous namespace)::ConstStructBuilder::ConvertStructToPacked()"},
    {"_ZN12_GLOBAL__N_118ConstStructBuilder13AppendPaddingEy", "(anonymous namespace)::ConstStructBuilder::AppendPadding(unsigned long long)"},
    {"_ZNK12_GLOBAL__N_118ConstStructBuilder14getSizeInBytesEPKN4llvm8ConstantE", "(anonymous namespace)::ConstStructBuilder::getSizeInBytes(llvm::Constant const*) const"},
    {"_ZN12_GLOBAL__N_116ConstExprEmitter13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::ConstExprEmitter::VisitCastExpr(clang::CastExpr*)"},
    {"_Z28FillInNullDataMemberPointersRN5clang7CodeGen13CodeGenModuleENS_8QualTypeERSt6vectorIPN4llvm8ConstantESaIS7_EEy", "FillInNullDataMemberPointers(clang::CodeGen::CodeGenModule&, clang::QualType, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >&, unsigned long long)"},
    {"_ZNSt6vectorIPN4llvm8ConstantESaIS2_EEaSERKS4_", "std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >::operator=(std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Decl const*, llvm::Value*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(clang::Decl const* const&, llvm::Value* const&, std::pair<clang::Decl const*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Decl const*, llvm::Value*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitScalarExprEPKNS_4ExprEb", "clang::CodeGen::CodeGenFunction::EmitScalarExpr(clang::Expr const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitScalarConversionEPN4llvm5ValueENS_8QualTypeES5_", "clang::CodeGen::CodeGenFunction::EmitScalarConversion(llvm::Value*, clang::QualType, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter20EmitScalarConversionEPN4llvm5ValueEN5clang8QualTypeES5_", "(anonymous namespace)::ScalarExprEmitter::EmitScalarConversion(llvm::Value*, clang::QualType, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction29EmitComplexToScalarConversionESt4pairIPN4llvm5ValueES5_ENS_8QualTypeES7_", "clang::CodeGen::CodeGenFunction::EmitComplexToScalarConversion(std::pair<llvm::Value*, llvm::Value*>, clang::QualType, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter29EmitComplexToScalarConversionESt4pairIPN4llvm5ValueES4_EN5clang8QualTypeES7_", "(anonymous namespace)::ScalarExprEmitter::EmitComplexToScalarConversion(std::pair<llvm::Value*, llvm::Value*>, clang::QualType, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23EmitScalarPrePostIncDecEPKNS_13UnaryOperatorENS0_6LValueEbb", "clang::CodeGen::CodeGenFunction::EmitScalarPrePostIncDec(clang::UnaryOperator const*, clang::CodeGen::LValue, bool, bool)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter23EmitScalarPrePostIncDecEPKN5clang13UnaryOperatorENS1_7CodeGen6LValueEbb", "(anonymous namespace)::ScalarExprEmitter::EmitScalarPrePostIncDec(clang::UnaryOperator const*, clang::CodeGen::LValue, bool, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitObjCIsaExprEPKNS_11ObjCIsaExprE", "clang::CodeGen::CodeGenFunction::EmitObjCIsaExpr(clang::ObjCIsaExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction28EmitCompoundAssignmentLValueEPKNS_22CompoundAssignOperatorE", "clang::CodeGen::CodeGenFunction::EmitCompoundAssignmentLValue(clang::CompoundAssignOperator const*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter24EmitCompoundAssignLValueEPKN5clang22CompoundAssignOperatorEMS0_FPN4llvm5ValueERKNS_9BinOpInfoEERS7_", "(anonymous namespace)::ScalarExprEmitter::EmitCompoundAssignLValue(clang::CompoundAssignOperator const*, llvm::Value* ((anonymous namespace)::ScalarExprEmitter::*)((anonymous namespace)::BinOpInfo const&), llvm::Value*&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitMulERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitMul((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitDivERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitDiv((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitRemERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitRem((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitAddERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitAdd((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitSubERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitSub((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitShlERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitShl((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitShrERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitShr((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitAndERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitAnd((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter7EmitXorERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitXor((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter6EmitOrERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitOr((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter24EmitOverflowCheckedBinOpERKNS_9BinOpInfoE", "(anonymous namespace)::ScalarExprEmitter::EmitOverflowCheckedBinOp((anonymous namespace)::BinOpInfo const&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter42EmitUndefinedBehaviorIntegerDivAndRemCheckERKNS_9BinOpInfoEPN4llvm5ValueEb", "(anonymous namespace)::ScalarExprEmitter::EmitUndefinedBehaviorIntegerDivAndRemCheck((anonymous namespace)::BinOpInfo const&, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter31EmitAddConsiderOverflowBehaviorEPKN5clang13UnaryOperatorEPN4llvm5ValueES7_b", "(anonymous namespace)::ScalarExprEmitter::EmitAddConsiderOverflowBehavior(clang::UnaryOperator const*, llvm::Value*, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter25EmitFloatToBoolConversionEPN4llvm5ValueE", "(anonymous namespace)::ScalarExprEmitter::EmitFloatToBoolConversion(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter23EmitIntToBoolConversionEPN4llvm5ValueE", "(anonymous namespace)::ScalarExprEmitter::EmitIntToBoolConversion(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter27EmitPointerToBoolConversionEPN4llvm5ValueE", "(anonymous namespace)::ScalarExprEmitter::EmitPointerToBoolConversion(llvm::Value*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_117ScalarExprEmitterEPN4llvm5ValueEE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::ScalarExprEmitter, llvm::Value*>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter13VisitCallExprEPKN5clang8CallExprE", "(anonymous namespace)::ScalarExprEmitter::VisitCallExpr(clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter13EmitNullValueEN5clang8QualTypeE", "(anonymous namespace)::ScalarExprEmitter::EmitNullValue(clang::QualType)"},
    {"_Z38isCheapEnoughToEvaluateUnconditionallyPKN5clang4ExprERNS_7CodeGen15CodeGenFunctionE", "isCheapEnoughToEvaluateUnconditionally(clang::Expr const*, clang::CodeGen::CodeGenFunction&)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter12EmitCastExprEPN5clang8CastExprE", "(anonymous namespace)::ScalarExprEmitter::EmitCastExpr(clang::CastExpr*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter18EmitCompoundAssignEPKN5clang22CompoundAssignOperatorEMS0_FPN4llvm5ValueERKNS_9BinOpInfoEE", "(anonymous namespace)::ScalarExprEmitter::EmitCompoundAssign(clang::CompoundAssignOperator const*, llvm::Value* ((anonymous namespace)::ScalarExprEmitter::*)((anonymous namespace)::BinOpInfo const&))"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter10EmitBinOpsEPKN5clang14BinaryOperatorE", "(anonymous namespace)::ScalarExprEmitter::EmitBinOps(clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_117ScalarExprEmitter11EmitCompareEPKN5clang14BinaryOperatorEjjj", "(anonymous namespace)::ScalarExprEmitter::EmitCompare(clang::BinaryOperator const*, unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateXorEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateXor(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateLShrEPNS_5ValueES6_RKNS_5TwineEb", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateLShr(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateShlEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateShl(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFSubEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateNegEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateNeg(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateSDivEPNS_5ValueES6_RKNS_5TwineEb", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateSDiv(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateCall4EPNS_5ValueES6_S6_S6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall4(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFAddEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateURemEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateURem(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateSRemEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateSRem(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFDivEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFDiv(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateUDivEPNS_5ValueES6_RKNS_5TwineEb", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateUDiv(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFCmpENS_7CmpInst9PredicateEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFCmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateNSWMulEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateNSWMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateFMulEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateFMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE19CreateInsertElementEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInsertElement(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE19CreateShuffleVectorEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateShuffleVector(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateVAArgEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateVAArg(llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE20CreateExtractElementEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateExtractElement(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateNotEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateNot(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE19CreateZExtOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateZExtOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21EmitObjCStringLiteralEPKNS_17ObjCStringLiteralE", "clang::CodeGen::CodeGenFunction::EmitObjCStringLiteral(clang::ObjCStringLiteral const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitObjCSelectorExprEPKNS_16ObjCSelectorExprE", "clang::CodeGen::CodeGenFunction::EmitObjCSelectorExpr(clang::ObjCSelectorExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitObjCProtocolExprEPKNS_16ObjCProtocolExprE", "clang::CodeGen::CodeGenFunction::EmitObjCProtocolExpr(clang::ObjCProtocolExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitObjCMessageExprEPKNS_15ObjCMessageExprENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitObjCMessageExpr(clang::ObjCMessageExpr const*, clang::CodeGen::ReturnValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12LoadObjCSelfEv", "clang::CodeGen::CodeGenFunction::LoadObjCSelf()"},
    {"_ZN5clang7CodeGen15CodeGenFunction15StartObjCMethodEPKNS_14ObjCMethodDeclEPKNS_17ObjCContainerDeclE", "clang::CodeGen::CodeGenFunction::StartObjCMethod(clang::ObjCMethodDecl const*, clang::ObjCContainerDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18GenerateObjCMethodEPKNS_14ObjCMethodDeclE", "clang::CodeGen::CodeGenFunction::GenerateObjCMethod(clang::ObjCMethodDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18GenerateObjCGetterEPNS_22ObjCImplementationDeclEPKNS_20ObjCPropertyImplDeclE", "clang::CodeGen::CodeGenFunction::GenerateObjCGetter(clang::ObjCImplementationDecl*, clang::ObjCPropertyImplDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16TypeOfSelfObjectEv", "clang::CodeGen::CodeGenFunction::TypeOfSelfObject()"},
    {"_ZN5clang7CodeGen15CodeGenFunction25IvarTypeWithAggrGCObjectsENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::IvarTypeWithAggrGCObjects(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18GenerateObjCSetterEPNS_22ObjCImplementationDeclEPKNS_20ObjCPropertyImplDeclE", "clang::CodeGen::CodeGenFunction::GenerateObjCSetter(clang::ObjCImplementationDecl*, clang::ObjCPropertyImplDecl const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction21IndirectObjCSetterArgERKNS0_14CGFunctionInfoE", "clang::CodeGen::CodeGenFunction::IndirectObjCSetterArg(clang::CodeGen::CGFunctionInfo const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26GenerateObjCCtorDtorMethodEPNS_22ObjCImplementationDeclEPNS_14ObjCMethodDeclEb", "clang::CodeGen::CodeGenFunction::GenerateObjCCtorDtorMethod(clang::ObjCImplementationDecl*, clang::ObjCMethodDecl*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitObjCPropertyRefLValueEPKNS_19ObjCPropertyRefExprE", "clang::CodeGen::CodeGenFunction::EmitObjCPropertyRefLValue(clang::ObjCPropertyRefExpr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction27EmitLoadOfPropertyRefLValueENS0_6LValueENS0_15ReturnValueSlotE", "clang::CodeGen::CodeGenFunction::EmitLoadOfPropertyRefLValue(clang::CodeGen::LValue, clang::CodeGen::ReturnValueSlot)"},
    {"_Z24GenerateMessageSendSuperRN5clang7CodeGen15CodeGenFunctionENS0_15ReturnValueSlotENS_8QualTypeENS_8SelectorEPN4llvm5ValueERKNS6_11SmallVectorISt4pairINS0_6RValueES4_ELj16EEE", "GenerateMessageSendSuper(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, llvm::Value*, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction33EmitStoreThroughPropertyRefLValueENS0_6RValueENS0_6LValueE", "clang::CodeGen::CodeGenFunction::EmitStoreThroughPropertyRefLValue(clang::CodeGen::RValue, clang::CodeGen::LValue)"},
    {"_ZN5clang7CodeGen15CodeGenFunction25EmitObjCForCollectionStmtERKNS_21ObjCForCollectionStmtE", "clang::CodeGen::CodeGenFunction::EmitObjCForCollectionStmt(clang::ObjCForCollectionStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitObjCAtTryStmtERKNS_13ObjCAtTryStmtE", "clang::CodeGen::CodeGenFunction::EmitObjCAtTryStmt(clang::ObjCAtTryStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19EmitObjCAtThrowStmtERKNS_15ObjCAtThrowStmtE", "clang::CodeGen::CodeGenFunction::EmitObjCAtThrowStmt(clang::ObjCAtThrowStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitObjCAtSynchronizedStmtERKNS_22ObjCAtSynchronizedStmtE", "clang::CodeGen::CodeGenFunction::EmitObjCAtSynchronizedStmt(clang::ObjCAtSynchronizedStmt const&)"},
    {"_ZN5clang7CodeGen13CGObjCRuntimeD0Ev", "clang::CodeGen::CGObjCRuntime::~CGObjCRuntime()"},
    {"_ZN5clang7CodeGen13CGObjCRuntimeD1Ev", "clang::CodeGen::CGObjCRuntime::~CGObjCRuntime()"},
    {"_ZN5clang7CodeGen13CGObjCRuntimeD2Ev", "clang::CodeGen::CGObjCRuntime::~CGObjCRuntime()"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitCallArgsINS_14ObjCMethodDeclEEEvRN4llvm11SmallVectorISt4pairINS0_6RValueENS_8QualTypeEELj16EEEPKT_NS_17ConstExprIteratorESF_", "void clang::CodeGen::CodeGenFunction::EmitCallArgs<clang::ObjCMethodDecl>(llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u>&, clang::ObjCMethodDecl const*, clang::ConstExprIterator, clang::ConstExprIterator)"},
    {"_ZNK5clang9ValueDecl7getTypeEv", "clang::ValueDecl::getType() const"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7CodeGen15CodeGenFunction13BreakContinueELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeGen::CodeGenFunction::BreakContinue, false>::grow(unsigned long)"},
    {"_ZN5clang7CodeGen20CreateGNUObjCRuntimeERNS0_13CodeGenModuleE", "clang::CodeGen::CreateGNUObjCRuntime(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNUD1Ev", "(anonymous namespace)::CGObjCGNU::~CGObjCGNU()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNUD0Ev", "(anonymous namespace)::CGObjCGNU::~CGObjCGNU()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18ModuleInitFunctionEv", "(anonymous namespace)::CGObjCGNU::ModuleInitFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEN5clang8SelectorEb", "(anonymous namespace)::CGObjCGNU::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::Selector, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCGNU::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU9GetEHTypeEN5clang8QualTypeE", "(anonymous namespace)::CGObjCGNU::GetEHType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU22GenerateConstantStringEPKN5clang13StringLiteralE", "(anonymous namespace)::CGObjCGNU::GenerateConstantString(clang::StringLiteral const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU16GenerateCategoryEPKN5clang20ObjCCategoryImplDeclE", "(anonymous namespace)::CGObjCGNU::GenerateCategory(clang::ObjCCategoryImplDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU13GenerateClassEPKN5clang22ObjCImplementationDeclE", "(anonymous namespace)::CGObjCGNU::GenerateClass(clang::ObjCImplementationDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU19GenerateMessageSendERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPN4llvm5ValueERKNS8_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_17ObjCInterfaceDeclEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCGNU::GenerateMessageSend(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, llvm::Value*, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU24GenerateMessageSendSuperERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPKNS1_17ObjCInterfaceDeclEbPN4llvm5ValueEbRKNSB_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCGNU::GenerateMessageSendSuper(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, clang::ObjCInterfaceDecl const*, bool, llvm::Value*, bool, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU19GenerateProtocolRefERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCGNU::GenerateProtocolRef(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU16GenerateProtocolEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCGNU::GenerateProtocol(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU14GenerateMethodEPKN5clang14ObjCMethodDeclEPKNS1_17ObjCContainerDeclE", "(anonymous namespace)::CGObjCGNU::GenerateMethod(clang::ObjCMethodDecl const*, clang::ObjCContainerDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU22GetPropertyGetFunctionEv", "(anonymous namespace)::CGObjCGNU::GetPropertyGetFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU22GetPropertySetFunctionEv", "(anonymous namespace)::CGObjCGNU::GetPropertySetFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20GetGetStructFunctionEv", "(anonymous namespace)::CGObjCGNU::GetGetStructFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20GetSetStructFunctionEv", "(anonymous namespace)::CGObjCGNU::GetSetStructFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU8GetClassERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::CGObjCGNU::GetClass(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU27EnumerationMutationFunctionEv", "(anonymous namespace)::CGObjCGNU::EnumerationMutationFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20EmitSynchronizedStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_22ObjCAtSynchronizedStmtE", "(anonymous namespace)::CGObjCGNU::EmitSynchronizedStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtSynchronizedStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU11EmitTryStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_13ObjCAtTryStmtE", "(anonymous namespace)::CGObjCGNU::EmitTryStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtTryStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU13EmitThrowStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_15ObjCAtThrowStmtE", "(anonymous namespace)::CGObjCGNU::EmitThrowStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtThrowStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU16EmitObjCWeakReadERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::CGObjCGNU::EmitObjCWeakRead(clang::CodeGen::CodeGenFunction&, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18EmitObjCWeakAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCGNU::EmitObjCWeakAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20EmitObjCGlobalAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_b", "(anonymous namespace)::CGObjCGNU::EmitObjCGlobalAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18EmitObjCIvarAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCGNU::EmitObjCIvarAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU24EmitObjCStrongCastAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCGNU::EmitObjCStrongCastAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20EmitObjCValueForIvarERN5clang7CodeGen15CodeGenFunctionENS1_8QualTypeEPN4llvm5ValueEPKNS1_12ObjCIvarDeclEj", "(anonymous namespace)::CGObjCGNU::EmitObjCValueForIvar(clang::CodeGen::CodeGenFunction&, clang::QualType, llvm::Value*, clang::ObjCIvarDecl const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU14EmitIvarOffsetERN5clang7CodeGen15CodeGenFunctionEPKNS1_17ObjCInterfaceDeclEPKNS1_12ObjCIvarDeclE", "(anonymous namespace)::CGObjCGNU::EmitIvarOffset(clang::CodeGen::CodeGenFunction&, clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU24EmitGCMemmoveCollectableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCGNU::EmitGCMemmoveCollectable(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18BuildGCBlockLayoutERN5clang7CodeGen13CodeGenModuleERKNS2_11CGBlockInfoE", "(anonymous namespace)::CGObjCGNU::BuildGCBlockLayout(clang::CodeGen::CodeGenModule&, clang::CodeGen::CGBlockInfo const&)"},
    {"_Z17FindIvarInterfaceRN5clang10ASTContextEPKNS_17ObjCInterfaceDeclEPKNS_12ObjCIvarDeclE", "FindIvarInterface(clang::ASTContext&, clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN12_GLOBAL__N_112CallSyncExitD1Ev", "(anonymous namespace)::CallSyncExit::~CallSyncExit()"},
    {"_ZN12_GLOBAL__N_112CallSyncExitD0Ev", "(anonymous namespace)::CallSyncExit::~CallSyncExit()"},
    {"_ZN12_GLOBAL__N_112CallSyncExit4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallSyncExit::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU12EmitClassRefERKSs", "(anonymous namespace)::CGObjCGNU::EmitClassRef(std::string const&)"},
    {"_Z19SymbolNameForMethodRKSsS0_S0_b", "SymbolNameForMethod(std::string const&, std::string const&, std::string const&, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU20GenerateProtocolListERKN4llvm15SmallVectorImplISsEE", "(anonymous namespace)::CGObjCGNU::GenerateProtocolList(llvm::SmallVectorImpl<std::string> const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU26GenerateProtocolMethodListERKN4llvm15SmallVectorImplIPNS1_8ConstantEEES7_", "(anonymous namespace)::CGObjCGNU::GenerateProtocolMethodList(llvm::SmallVectorImpl<llvm::Constant*> const&, llvm::SmallVectorImpl<llvm::Constant*> const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18GenerateMethodListERKSsS2_RKN4llvm15SmallVectorImplIN5clang8SelectorEEERKNS4_IPNS3_8ConstantEEEb", "(anonymous namespace)::CGObjCGNU::GenerateMethodList(std::string const&, std::string const&, llvm::SmallVectorImpl<clang::Selector> const&, llvm::SmallVectorImpl<llvm::Constant*> const&, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU16GenerateIvarListERKN4llvm15SmallVectorImplIPNS1_8ConstantEEES7_S7_", "(anonymous namespace)::CGObjCGNU::GenerateIvarList(llvm::SmallVectorImpl<llvm::Constant*> const&, llvm::SmallVectorImpl<llvm::Constant*> const&, llvm::SmallVectorImpl<llvm::Constant*> const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU22GenerateClassStructureEPN4llvm8ConstantES3_jPKcS3_S3_S3_S3_S3_S3_S3_b", "(anonymous namespace)::CGObjCGNU::GenerateClassStructure(llvm::Constant*, llvm::Constant*, unsigned int, char const*, llvm::Constant*, llvm::Constant*, llvm::Constant*, llvm::Constant*, llvm::Constant*, llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCGNU18ExportUniqueStringERKSsSs", "(anonymous namespace)::CGObjCGNU::ExportUniqueString(std::string const&, std::string)"},
    {"_Z19MangleSelectorTypesRKSs", "MangleSelectorTypes(std::string const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateCall3EPNS_5ValueES6_S6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall3(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateLoadEPNS_5ValueEbRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateLoad(llvm::Value*, bool, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateCall2EPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall2(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCallEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE17CreateUnreachableEv", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateUnreachable()"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateInvokeIPPNS_5ValueEEEPNS_10InvokeInstES7_PNS_10BasicBlockESC_T_SD_RKNS_5TwineE", "llvm::InvokeInst* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInvoke<llvm::Value**>(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7CanQualINS1_4TypeEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CanQual<clang::Type>, false>::grow(unsigned long)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIN5clang7CodeGen6RValueENS2_8QualTypeEEE6insertIPKS6_EEPS6_SB_T_SC_", "std::pair<clang::CodeGen::RValue, clang::QualType>* llvm::SmallVectorImpl<std::pair<clang::CodeGen::RValue, clang::QualType> >::insert<std::pair<clang::CodeGen::RValue, clang::QualType> const*>(std::pair<clang::CodeGen::RValue, clang::QualType>*, std::pair<clang::CodeGen::RValue, clang::QualType> const*, std::pair<clang::CodeGen::RValue, clang::QualType> const*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateAllocaEPKNS_4TypeEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateAlloca(llvm::Type const*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCallIPPNS_5ValueEEEPNS_8CallInstES7_T_SB_RKNS_5TwineE", "llvm::CallInst* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIN5clang7CodeGen6RValueENS2_8QualTypeEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::CodeGen::RValue, clang::QualType>, false>::grow(unsigned long)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE8CreateBrEPNS_10BasicBlockE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateBr(llvm::BasicBlock*)"},
    {"_ZNSt3mapISt4pairISsSsEPN4llvm11GlobalAliasESt4lessIS1_ESaIS0_IKS1_S4_EEEixERS7_", "std::map<std::pair<std::string, std::string>, llvm::GlobalAlias*, std::less<std::pair<std::string, std::string> >, std::allocator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> > >::operator[](std::pair<std::string, std::string> const&)"},
    {"_ZNKSt4lessISt4pairISsSsEEclERKS1_S4_", "std::less<std::pair<std::string, std::string> >::operator()(std::pair<std::string, std::string> const&, std::pair<std::string, std::string> const&) const"},
    {"_ZNSt8_Rb_treeISt4pairISsSsES0_IKS1_PN4llvm11GlobalAliasEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<std::pair<std::string, std::string>, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*>, std::_Select1st<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >, std::less<std::pair<std::string, std::string> >, std::allocator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairISsSsES0_IKS1_PN4llvm11GlobalAliasEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::pair<std::string, std::string>, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*>, std::_Select1st<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >, std::less<std::pair<std::string, std::string> >, std::allocator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairISsSsES0_IKS1_PN4llvm11GlobalAliasEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::pair<std::string, std::string>, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*>, std::_Select1st<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >, std::less<std::pair<std::string, std::string> >, std::allocator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> > >::_M_insert_unique(std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> const&)"},
    {"_ZN4llvm9StringMapIPNS_11GlobalAliasENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::GlobalAlias*>& llvm::StringMap<llvm::GlobalAlias*, llvm::MallocAllocator>::GetOrCreateValue<llvm::GlobalAlias*>(llvm::StringRef, llvm::GlobalAlias*)"},
    {"_ZNSt6vectorIPN4llvm8ConstantESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS2_S4_EEEEvS9_T_SA_St20forward_iterator_tag", "void std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >(__gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, __gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, __gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::forward_iterator_tag)"},
    {"_ZNSt8_Rb_treeISt4pairISsSsES0_IKS1_PN4llvm11GlobalAliasEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::pair<std::string, std::string>, std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*>, std::_Select1st<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >, std::less<std::pair<std::string, std::string> >, std::allocator<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<std::string, std::string> const, llvm::GlobalAlias*> >*)"},
    {"_ZN5clang7CodeGen13CGObjCRuntime21ComputeIvarBaseOffsetERNS0_13CodeGenModuleEPKNS_17ObjCInterfaceDeclEPKNS_12ObjCIvarDeclE", "clang::CodeGen::CGObjCRuntime::ComputeIvarBaseOffset(clang::CodeGen::CodeGenModule&, clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_Z20LookupFieldBitOffsetRN5clang7CodeGen13CodeGenModuleEPKNS_17ObjCInterfaceDeclEPKNS_22ObjCImplementationDeclEPKNS_12ObjCIvarDeclE", "LookupFieldBitOffset(clang::CodeGen::CodeGenModule&, clang::ObjCInterfaceDecl const*, clang::ObjCImplementationDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN5clang7CodeGen13CGObjCRuntime21ComputeIvarBaseOffsetERNS0_13CodeGenModuleEPKNS_22ObjCImplementationDeclEPKNS_12ObjCIvarDeclE", "clang::CodeGen::CGObjCRuntime::ComputeIvarBaseOffset(clang::CodeGen::CodeGenModule&, clang::ObjCImplementationDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN5clang7CodeGen13CGObjCRuntime24EmitValueForIvarAtOffsetERNS0_15CodeGenFunctionEPKNS_17ObjCInterfaceDeclEPN4llvm5ValueEPKNS_12ObjCIvarDeclEjS9_", "clang::CodeGen::CGObjCRuntime::EmitValueForIvarAtOffset(clang::CodeGen::CodeGenFunction&, clang::ObjCInterfaceDecl const*, llvm::Value*, clang::ObjCIvarDecl const*, unsigned int, llvm::Value*)"},
    {"_ZN5clang7CodeGen20CreateMacObjCRuntimeERNS0_13CodeGenModuleE", "clang::CodeGen::CreateMacObjCRuntime(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen33CreateMacNonFragileABIObjCRuntimeERNS0_13CodeGenModuleE", "clang::CodeGen::CreateMacNonFragileABIObjCRuntime(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMacC2ERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::CGObjCCommonMac::CGObjCCommonMac(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMacD1Ev", "(anonymous namespace)::CGObjCNonFragileABIMac::~CGObjCNonFragileABIMac()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMacD0Ev", "(anonymous namespace)::CGObjCNonFragileABIMac::~CGObjCNonFragileABIMac()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18ModuleInitFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::ModuleInitFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEN5clang8SelectorEb", "(anonymous namespace)::CGObjCNonFragileABIMac::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::Selector, bool)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac9GetEHTypeEN5clang8QualTypeE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetEHType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac22GenerateConstantStringEPKN5clang13StringLiteralE", "(anonymous namespace)::CGObjCCommonMac::GenerateConstantString(clang::StringLiteral const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac16GenerateCategoryEPKN5clang20ObjCCategoryImplDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GenerateCategory(clang::ObjCCategoryImplDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac13GenerateClassEPKN5clang22ObjCImplementationDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GenerateClass(clang::ObjCImplementationDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac19GenerateMessageSendERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPN4llvm5ValueERKNS8_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_17ObjCInterfaceDeclEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GenerateMessageSend(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, llvm::Value*, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac24GenerateMessageSendSuperERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPKNS1_17ObjCInterfaceDeclEbPN4llvm5ValueEbRKNSB_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GenerateMessageSendSuper(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, clang::ObjCInterfaceDecl const*, bool, llvm::Value*, bool, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac19GenerateProtocolRefERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GenerateProtocolRef(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac16GenerateProtocolEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCCommonMac::GenerateProtocol(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac14GenerateMethodEPKN5clang14ObjCMethodDeclEPKNS1_17ObjCContainerDeclE", "(anonymous namespace)::CGObjCCommonMac::GenerateMethod(clang::ObjCMethodDecl const*, clang::ObjCContainerDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac22GetPropertyGetFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::GetPropertyGetFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac22GetPropertySetFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::GetPropertySetFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20GetGetStructFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::GetGetStructFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20GetSetStructFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::GetSetStructFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac8GetClassERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetClass(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac27EnumerationMutationFunctionEv", "(anonymous namespace)::CGObjCNonFragileABIMac::EnumerationMutationFunction()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20EmitSynchronizedStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_22ObjCAtSynchronizedStmtE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitSynchronizedStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtSynchronizedStmt const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac11EmitTryStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_13ObjCAtTryStmtE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitTryStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtTryStmt const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac13EmitThrowStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_15ObjCAtThrowStmtE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitThrowStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtThrowStmt const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac16EmitObjCWeakReadERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCWeakRead(clang::CodeGen::CodeGenFunction&, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18EmitObjCWeakAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCWeakAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20EmitObjCGlobalAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_b", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCGlobalAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18EmitObjCIvarAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCIvarAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac24EmitObjCStrongCastAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCStrongCastAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20EmitObjCValueForIvarERN5clang7CodeGen15CodeGenFunctionENS1_8QualTypeEPN4llvm5ValueEPKNS1_12ObjCIvarDeclEj", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitObjCValueForIvar(clang::CodeGen::CodeGenFunction&, clang::QualType, llvm::Value*, clang::ObjCIvarDecl const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac14EmitIvarOffsetERN5clang7CodeGen15CodeGenFunctionEPKNS1_17ObjCInterfaceDeclEPKNS1_12ObjCIvarDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitIvarOffset(clang::CodeGen::CodeGenFunction&, clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac24EmitGCMemmoveCollectableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitGCMemmoveCollectable(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac18BuildGCBlockLayoutERN5clang7CodeGen13CodeGenModuleERKNS2_11CGBlockInfoE", "(anonymous namespace)::CGObjCCommonMac::BuildGCBlockLayout(clang::CodeGen::CodeGenModule&, clang::CodeGen::CGBlockInfo const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac17GetOrEmitProtocolEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetOrEmitProtocol(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac20GetOrEmitProtocolRefEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetOrEmitProtocolRef(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac28GetMethodDescriptionConstantEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetMethodDescriptionConstant(clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac12GetClassNameEPN5clang14IdentifierInfoE", "(anonymous namespace)::CGObjCCommonMac::GetClassName(clang::IdentifierInfo*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac16EmitProtocolListEN4llvm5TwineEPKPN5clang16ObjCProtocolDeclES7_", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitProtocolList(llvm::Twine, clang::ObjCProtocolDecl* const*, clang::ObjCProtocolDecl* const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac14EmitMethodListEN4llvm5TwineEPKcRKSt6vectorIPNS1_8ConstantESaIS7_EE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitMethodList(llvm::Twine, char const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac16EmitPropertyListEN4llvm5TwineEPKN5clang4DeclEPKNS3_17ObjCContainerDeclERKNS_21ObjCCommonTypesHelperE", "(anonymous namespace)::CGObjCCommonMac::EmitPropertyList(llvm::Twine, clang::Decl const*, clang::ObjCContainerDecl const*, (anonymous namespace)::ObjCCommonTypesHelper const&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac15GetPropertyNameEPN5clang14IdentifierInfoE", "(anonymous namespace)::CGObjCCommonMac::GetPropertyName(clang::IdentifierInfo*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac21GetPropertyTypeStringEPKN5clang16ObjCPropertyDeclEPKNS1_4DeclE", "(anonymous namespace)::CGObjCCommonMac::GetPropertyTypeString(clang::ObjCPropertyDecl const*, clang::Decl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac22PushProtocolPropertiesERN4llvm11SmallPtrSetIPKN5clang14IdentifierInfoELj16EEERSt6vectorIPNS1_8ConstantESaISB_EEPKNS3_4DeclEPKNS3_16ObjCProtocolDeclERKNS_21ObjCCommonTypesHelperE", "(anonymous namespace)::CGObjCCommonMac::PushProtocolProperties(llvm::SmallPtrSet<clang::IdentifierInfo const*, 16u>&, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >&, clang::Decl const*, clang::ObjCProtocolDecl const*, (anonymous namespace)::ObjCCommonTypesHelper const&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac17CreateMetadataVarEN4llvm5TwineEPNS1_8ConstantEPKcjb", "(anonymous namespace)::CGObjCCommonMac::CreateMetadataVar(llvm::Twine, llvm::Constant*, char const*, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac16GetMethodVarNameEN5clang8SelectorE", "(anonymous namespace)::CGObjCCommonMac::GetMethodVarName(clang::Selector)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac16GetMethodVarTypeEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCCommonMac::GetMethodVarType(clang::ObjCMethodDecl const*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_115CGObjCCommonMac7GC_IVAREE9push_backERKS3_", "llvm::SmallVectorImpl<(anonymous namespace)::CGObjCCommonMac::GC_IVAR>::push_back((anonymous namespace)::CGObjCCommonMac::GC_IVAR const&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac25BuildAggrIvarRecordLayoutEPKN5clang10RecordTypeEjbRb", "(anonymous namespace)::CGObjCCommonMac::BuildAggrIvarRecordLayout(clang::RecordType const*, unsigned int, bool, bool&)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac21BuildIvarLayoutBitmapERSs", "(anonymous namespace)::CGObjCCommonMac::BuildIvarLayoutBitmap(std::string&)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_115CGObjCCommonMac9SKIP_SCANEE9push_backERKS3_", "llvm::SmallVectorImpl<(anonymous namespace)::CGObjCCommonMac::SKIP_SCAN>::push_back((anonymous namespace)::CGObjCCommonMac::SKIP_SCAN const&)"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_115CGObjCCommonMac7GC_IVAREEEiPKvS5_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::CGObjCCommonMac::GC_IVAR>(void const*, void const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac19BuildAggrIvarLayoutEPKN5clang22ObjCImplementationDeclEPKN4llvm12StructLayoutEPKNS1_10RecordDeclERKNS5_15SmallVectorImplIPNS1_9FieldDeclEEEjbRb", "(anonymous namespace)::CGObjCCommonMac::BuildAggrIvarLayout(clang::ObjCImplementationDecl const*, llvm::StructLayout const*, clang::RecordDecl const*, llvm::SmallVectorImpl<clang::FieldDecl*> const&, unsigned int, bool, bool&)"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper22GcMemmoveCollectableFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::GcMemmoveCollectableFn()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac22ObjCIvarOffsetVariableEPKN5clang17ObjCInterfaceDeclEPKNS1_12ObjCIvarDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::ObjCIvarOffsetVariable(clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper23getGcAssignStrongCastFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcAssignStrongCastFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper17getGcAssignIvarFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcAssignIvarFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper19getGcAssignGlobalFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcAssignGlobalFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper24getGcAssignThreadLocalFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcAssignThreadLocalFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper17getGcAssignWeakFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcAssignWeakFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper15getGcReadWeakFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGcReadWeakFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper19getExceptionThrowFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getExceptionThrowFn()"},
    {"_ZN12_GLOBAL__N_116CallObjCEndCatchD1Ev", "(anonymous namespace)::CallObjCEndCatch::~CallObjCEndCatch()"},
    {"_ZN12_GLOBAL__N_116CallObjCEndCatchD0Ev", "(anonymous namespace)::CallObjCEndCatch::~CallObjCEndCatch()"},
    {"_ZN12_GLOBAL__N_116CallObjCEndCatch4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallObjCEndCatch::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper14getSyncEnterFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getSyncEnterFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper13getSyncExitFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getSyncExitFn()"},
    {"_ZN12_GLOBAL__N_112CallSyncExitD1Ev", "(anonymous namespace)::CallSyncExit::~CallSyncExit()"},
    {"_ZN12_GLOBAL__N_112CallSyncExitD0Ev", "(anonymous namespace)::CallSyncExit::~CallSyncExit()"},
    {"_ZN12_GLOBAL__N_112CallSyncExit4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallSyncExit::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper24getEnumerationMutationFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getEnumerationMutationFn()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac14GetClassGlobalERKSs", "(anonymous namespace)::CGObjCNonFragileABIMac::GetClassGlobal(std::string const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac12EmitClassRefERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitClassRef(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper15getCopyStructFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getCopyStructFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper16getSetPropertyFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getSetPropertyFn()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelper16getGetPropertyFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getGetPropertyFn()"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac24LegacyDispatchedSelectorEN5clang8SelectorE", "(anonymous namespace)::CGObjCNonFragileABIMac::LegacyDispatchedSelector(clang::Selector)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac21EmitLegacyMessageSendERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeEPN4llvm5ValueES9_S6_bRKNS7_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_14ObjCMethodDeclERKNS_21ObjCCommonTypesHelperE", "(anonymous namespace)::CGObjCCommonMac::EmitLegacyMessageSend(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, llvm::Value*, llvm::Value*, clang::QualType, bool, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCMethodDecl const*, (anonymous namespace)::ObjCCommonTypesHelper const&)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac12EmitSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEN5clang8SelectorEb", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::Selector, bool)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac15EmitMessageSendERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPN4llvm5ValueES6_bRKNS8_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEE", "(anonymous namespace)::CGObjCNonFragileABIMac::EmitMessageSend(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, llvm::Value*, clang::QualType, bool, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&)"},
    {"_ZNK12_GLOBAL__N_121ObjCCommonTypesHelper21getMessageSendSuperFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getMessageSendSuperFn() const"},
    {"_ZNK12_GLOBAL__N_121ObjCCommonTypesHelper16getMessageSendFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getMessageSendFn() const"},
    {"_ZNK12_GLOBAL__N_121ObjCCommonTypesHelper22getMessageSendSuperFn2Ev", "(anonymous namespace)::ObjCCommonTypesHelper::getMessageSendSuperFn2() const"},
    {"_ZNK12_GLOBAL__N_121ObjCCommonTypesHelper21getMessageSendFpretFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getMessageSendFpretFn() const"},
    {"_ZNK12_GLOBAL__N_121ObjCCommonTypesHelper21getMessageSendStretFnEv", "(anonymous namespace)::ObjCCommonTypesHelper::getMessageSendStretFn() const"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac24BuildClassRoTInitializerEjjjPKN5clang22ObjCImplementationDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::BuildClassRoTInitializer(unsigned int, unsigned int, unsigned int, clang::ObjCImplementationDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18BuildClassMetaDataERSsPN4llvm8ConstantES4_S4_b", "(anonymous namespace)::CGObjCNonFragileABIMac::BuildClassMetaData(std::string&, llvm::Constant*, llvm::Constant*, llvm::Constant*, bool)"},
    {"_Z25hasObjCExceptionAttributeRN5clang10ASTContextEPKNS_17ObjCInterfaceDeclE", "hasObjCExceptionAttribute(clang::ASTContext&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18GetInterfaceEHTypeEPKN5clang17ObjCInterfaceDeclEb", "(anonymous namespace)::CGObjCNonFragileABIMac::GetInterfaceEHType(clang::ObjCInterfaceDecl const*, bool)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac15BuildIvarLayoutEPKN5clang22ObjCImplementationDeclEb", "(anonymous namespace)::CGObjCCommonMac::BuildIvarLayout(clang::ObjCImplementationDecl const*, bool)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac17GetMethodConstantEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCNonFragileABIMac::GetMethodConstant(clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac16GetMethodVarTypeEPKN5clang9FieldDeclE", "(anonymous namespace)::CGObjCCommonMac::GetMethodVarType(clang::FieldDecl const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac19GetMethodDefinitionEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCCommonMac::GetMethodDefinition(clang::ObjCMethodDecl const*)"},
    {"_ZSt4sortIPN12_GLOBAL__N_115CGObjCCommonMac7GC_IVAREEvT_S4_", "void std::sort<(anonymous namespace)::CGObjCCommonMac::GC_IVAR*>((anonymous namespace)::CGObjCCommonMac::GC_IVAR*, (anonymous namespace)::CGObjCCommonMac::GC_IVAR*)"},
    {"_ZSt16__introsort_loopIPN12_GLOBAL__N_115CGObjCCommonMac7GC_IVARElEvT_S4_T0_", "void std::__introsort_loop<(anonymous namespace)::CGObjCCommonMac::GC_IVAR*, long>((anonymous namespace)::CGObjCCommonMac::GC_IVAR*, (anonymous namespace)::CGObjCCommonMac::GC_IVAR*, long)"},
    {"_ZSt16__insertion_sortIPN12_GLOBAL__N_115CGObjCCommonMac7GC_IVAREEvT_S4_", "void std::__insertion_sort<(anonymous namespace)::CGObjCCommonMac::GC_IVAR*>((anonymous namespace)::CGObjCCommonMac::GC_IVAR*, (anonymous namespace)::CGObjCCommonMac::GC_IVAR*)"},
    {"_ZN12_GLOBAL__N_122CGObjCNonFragileABIMac18AddModuleClassListERKSt6vectorIPN4llvm11GlobalValueESaIS4_EEPKcSA_", "(anonymous namespace)::CGObjCNonFragileABIMac::AddModuleClassList(std::vector<llvm::GlobalValue*, std::allocator<llvm::GlobalValue*> > const&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMac13EmitImageInfoEv", "(anonymous namespace)::CGObjCCommonMac::EmitImageInfo()"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMacD2Ev", "(anonymous namespace)::CGObjCCommonMac::~CGObjCCommonMac()"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMacD1Ev", "(anonymous namespace)::CGObjCCommonMac::~CGObjCCommonMac()"},
    {"_ZN12_GLOBAL__N_115CGObjCCommonMacD0Ev", "(anonymous namespace)::CGObjCCommonMac::~CGObjCCommonMac()"},
    {"_ZN12_GLOBAL__N_121ObjCCommonTypesHelperC2ERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::ObjCCommonTypesHelper::ObjCCommonTypesHelper(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_19CGObjCMacD1Ev", "(anonymous namespace)::CGObjCMac::~CGObjCMac()"},
    {"_ZN12_GLOBAL__N_19CGObjCMacD0Ev", "(anonymous namespace)::CGObjCMac::~CGObjCMac()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac18ModuleInitFunctionEv", "(anonymous namespace)::CGObjCMac::ModuleInitFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEN5clang8SelectorEb", "(anonymous namespace)::CGObjCMac::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::Selector, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac11GetSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCMac::GetSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac9GetEHTypeEN5clang8QualTypeE", "(anonymous namespace)::CGObjCMac::GetEHType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac16GenerateCategoryEPKN5clang20ObjCCategoryImplDeclE", "(anonymous namespace)::CGObjCMac::GenerateCategory(clang::ObjCCategoryImplDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac13GenerateClassEPKN5clang22ObjCImplementationDeclE", "(anonymous namespace)::CGObjCMac::GenerateClass(clang::ObjCImplementationDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac19GenerateMessageSendERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPN4llvm5ValueERKNS8_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_17ObjCInterfaceDeclEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCMac::GenerateMessageSend(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, llvm::Value*, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac24GenerateMessageSendSuperERN5clang7CodeGen15CodeGenFunctionENS2_15ReturnValueSlotENS1_8QualTypeENS1_8SelectorEPKNS1_17ObjCInterfaceDeclEbPN4llvm5ValueEbRKNSB_11SmallVectorISt4pairINS2_6RValueES6_ELj16EEEPKNS1_14ObjCMethodDeclE", "(anonymous namespace)::CGObjCMac::GenerateMessageSendSuper(clang::CodeGen::CodeGenFunction&, clang::CodeGen::ReturnValueSlot, clang::QualType, clang::Selector, clang::ObjCInterfaceDecl const*, bool, llvm::Value*, bool, llvm::SmallVector<std::pair<clang::CodeGen::RValue, clang::QualType>, 16u> const&, clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac19GenerateProtocolRefERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCMac::GenerateProtocolRef(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac22GetPropertyGetFunctionEv", "(anonymous namespace)::CGObjCMac::GetPropertyGetFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac22GetPropertySetFunctionEv", "(anonymous namespace)::CGObjCMac::GetPropertySetFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20GetGetStructFunctionEv", "(anonymous namespace)::CGObjCMac::GetGetStructFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20GetSetStructFunctionEv", "(anonymous namespace)::CGObjCMac::GetSetStructFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac8GetClassERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::CGObjCMac::GetClass(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac27EnumerationMutationFunctionEv", "(anonymous namespace)::CGObjCMac::EnumerationMutationFunction()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20EmitSynchronizedStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_22ObjCAtSynchronizedStmtE", "(anonymous namespace)::CGObjCMac::EmitSynchronizedStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtSynchronizedStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac11EmitTryStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_13ObjCAtTryStmtE", "(anonymous namespace)::CGObjCMac::EmitTryStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtTryStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac13EmitThrowStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_15ObjCAtThrowStmtE", "(anonymous namespace)::CGObjCMac::EmitThrowStmt(clang::CodeGen::CodeGenFunction&, clang::ObjCAtThrowStmt const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac16EmitObjCWeakReadERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::CGObjCMac::EmitObjCWeakRead(clang::CodeGen::CodeGenFunction&, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac18EmitObjCWeakAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCMac::EmitObjCWeakAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20EmitObjCGlobalAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_b", "(anonymous namespace)::CGObjCMac::EmitObjCGlobalAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac18EmitObjCIvarAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCMac::EmitObjCIvarAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac24EmitObjCStrongCastAssignERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_", "(anonymous namespace)::CGObjCMac::EmitObjCStrongCastAssign(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20EmitObjCValueForIvarERN5clang7CodeGen15CodeGenFunctionENS1_8QualTypeEPN4llvm5ValueEPKNS1_12ObjCIvarDeclEj", "(anonymous namespace)::CGObjCMac::EmitObjCValueForIvar(clang::CodeGen::CodeGenFunction&, clang::QualType, llvm::Value*, clang::ObjCIvarDecl const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac14EmitIvarOffsetERN5clang7CodeGen15CodeGenFunctionEPKNS1_17ObjCInterfaceDeclEPKNS1_12ObjCIvarDeclE", "(anonymous namespace)::CGObjCMac::EmitIvarOffset(clang::CodeGen::CodeGenFunction&, clang::ObjCInterfaceDecl const*, clang::ObjCIvarDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac24EmitGCMemmoveCollectableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_S7_", "(anonymous namespace)::CGObjCMac::EmitGCMemmoveCollectable(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac17GetOrEmitProtocolEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCMac::GetOrEmitProtocol(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac20GetOrEmitProtocolRefEPKN5clang16ObjCProtocolDeclE", "(anonymous namespace)::CGObjCMac::GetOrEmitProtocolRef(clang::ObjCProtocolDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac28GetMethodDescriptionConstantEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCMac::GetMethodDescriptionConstant(clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac16EmitProtocolListEN4llvm5TwineEPKPN5clang16ObjCProtocolDeclES7_", "(anonymous namespace)::CGObjCMac::EmitProtocolList(llvm::Twine, clang::ObjCProtocolDecl* const*, clang::ObjCProtocolDecl* const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac18EmitMethodDescListEN4llvm5TwineEPKcRKSt6vectorIPNS1_8ConstantESaIS7_EE", "(anonymous namespace)::CGObjCMac::EmitMethodDescList(llvm::Twine, char const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac25EmitTryOrSynchronizedStmtERN5clang7CodeGen15CodeGenFunctionERKNS1_4StmtE", "(anonymous namespace)::CGObjCMac::EmitTryOrSynchronizedStmt(clang::CodeGen::CodeGenFunction&, clang::Stmt const&)"},
    {"_ZN12_GLOBAL__N_115ObjCTypesHelper22getExceptionTryEnterFnEv", "(anonymous namespace)::ObjCTypesHelper::getExceptionTryEnterFn()"},
    {"_ZN12_GLOBAL__N_115ObjCTypesHelper11getSetJmpFnEv", "(anonymous namespace)::ObjCTypesHelper::getSetJmpFn()"},
    {"_ZN12_GLOBAL__N_115ObjCTypesHelper21getExceptionExtractFnEv", "(anonymous namespace)::ObjCTypesHelper::getExceptionExtractFn()"},
    {"_ZN12_GLOBAL__N_19CGObjCMac12EmitClassRefERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEPKN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::CGObjCMac::EmitClassRef(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN12_GLOBAL__N_121PerformFragileFinallyD1Ev", "(anonymous namespace)::PerformFragileFinally::~PerformFragileFinally()"},
    {"_ZN12_GLOBAL__N_121PerformFragileFinallyD0Ev", "(anonymous namespace)::PerformFragileFinally::~PerformFragileFinally()"},
    {"_ZN12_GLOBAL__N_121PerformFragileFinally4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::PerformFragileFinally::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac12EmitSelectorERN4llvm9IRBuilderILb0ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb0EEEEEN5clang8SelectorEb", "(anonymous namespace)::CGObjCMac::EmitSelector(llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >&, clang::Selector, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac17GetMethodConstantEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::CGObjCMac::GetMethodConstant(clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac12EmitIvarListEPKN5clang22ObjCImplementationDeclEb", "(anonymous namespace)::CGObjCMac::EmitIvarList(clang::ObjCImplementationDecl const*, bool)"},
    {"_ZN12_GLOBAL__N_19CGObjCMac14EmitMethodListEN4llvm5TwineEPKcRKSt6vectorIPNS1_8ConstantESaIS7_EE", "(anonymous namespace)::CGObjCMac::EmitMethodList(llvm::Twine, char const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS_14GlobalVariableENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::InsertIntoBucket(clang::IdentifierInfo* const&, llvm::GlobalVariable* const&, std::pair<clang::IdentifierInfo*, llvm::GlobalVariable*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS_14GlobalVariableENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::grow(unsigned int)"},
    {"_ZN4llvm9StringMapIPNS_14GlobalVariableENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::GlobalVariable*>& llvm::StringMap<llvm::GlobalVariable*, llvm::MallocAllocator>::GetOrCreateValue<llvm::GlobalVariable*>(llvm::StringRef, llvm::GlobalVariable*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPNS_14GlobalVariableENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<clang::Selector, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::InsertIntoBucket(clang::Selector const&, llvm::GlobalVariable* const&, std::pair<clang::Selector, llvm::GlobalVariable*>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPNS_14GlobalVariableENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::Selector, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang9FieldDeclEE6appendINS1_11DeclContext22specific_decl_iteratorIS2_EEEEvT_S9_", "void llvm::SmallVectorImpl<clang::FieldDecl*>::append<clang::DeclContext::specific_decl_iterator<clang::FieldDecl> >(clang::DeclContext::specific_decl_iterator<clang::FieldDecl>, clang::DeclContext::specific_decl_iterator<clang::FieldDecl>)"},
    {"_ZN4llvm8DenseMapIPKN5clang14ObjCMethodDeclEPNS_8FunctionENS_12DenseMapInfoIS4_EENS7_IS6_EEE6insertERKSt4pairIS4_S6_E", "llvm::DenseMap<clang::ObjCMethodDecl const*, llvm::Function*, llvm::DenseMapInfo<clang::ObjCMethodDecl const*>, llvm::DenseMapInfo<llvm::Function*> >::insert(std::pair<clang::ObjCMethodDecl const*, llvm::Function*> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang14ObjCMethodDeclEPNS_8FunctionENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::ObjCMethodDecl const*, llvm::Function*, llvm::DenseMapInfo<clang::ObjCMethodDecl const*>, llvm::DenseMapInfo<llvm::Function*> >::InsertIntoBucket(clang::ObjCMethodDecl const* const&, llvm::Function* const&, std::pair<clang::ObjCMethodDecl const*, llvm::Function*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang14ObjCMethodDeclEPNS_8FunctionENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::ObjCMethodDecl const*, llvm::Function*, llvm::DenseMapInfo<clang::ObjCMethodDecl const*>, llvm::DenseMapInfo<llvm::Function*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<clang::IdentifierInfo*, char, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::IdentifierInfo*, char> const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<clang::IdentifierInfo*, char, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::IdentifierInfo* const&, char const&, std::pair<clang::IdentifierInfo*, char>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, char, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm9SetVectorISsSt6vectorISsSaISsEENS_8SmallSetISsLj16EEEED2Ev", "llvm::SetVector<std::string, std::vector<std::string, std::allocator<std::string> >, llvm::SmallSet<std::string, 16u> >::~SetVector()"},
    {"_ZNSt6vectorIPN5clang14IdentifierInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::IdentifierInfo*, std::allocator<clang::IdentifierInfo*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::IdentifierInfo**, std::vector<clang::IdentifierInfo*, std::allocator<clang::IdentifierInfo*> > >, clang::IdentifierInfo* const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateGEPIPPNS_5ValueEEES7_S7_T_S9_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateGEP<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm8SmallSetISsLj16EE6insertERKSs", "llvm::SmallSet<std::string, 16u>::insert(std::string const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule23GetAddrOfRTTIDescriptorENS_8QualTypeEb", "clang::CodeGen::CodeGenModule::GetAddrOfRTTIDescriptor(clang::QualType, bool)"},
    {"_ZN12_GLOBAL__N_111RTTIBuilder13BuildTypeInfoEN5clang8QualTypeEb", "(anonymous namespace)::RTTIBuilder::BuildTypeInfo(clang::QualType, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule29EmitFundamentalRTTIDescriptorENS_8QualTypeE", "clang::CodeGen::CodeGenModule::EmitFundamentalRTTIDescriptor(clang::QualType)"},
    {"_ZN5clang7CodeGen13CodeGenModule30EmitFundamentalRTTIDescriptorsEv", "clang::CodeGen::CodeGenModule::EmitFundamentalRTTIDescriptors()"},
    {"_Z18getTypeInfoLinkageRN5clang7CodeGen13CodeGenModuleENS_8QualTypeE", "getTypeInfoLinkage(clang::CodeGen::CodeGenModule&, clang::QualType)"},
    {"_Z23CanUseSingleInheritancePKN5clang13CXXRecordDeclE", "CanUseSingleInheritance(clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_111RTTIBuilder20BuildPointerTypeInfoEN5clang8QualTypeE", "(anonymous namespace)::RTTIBuilder::BuildPointerTypeInfo(clang::QualType)"},
    {"_Z27ContainsIncompleteClassTypeN5clang8QualTypeE", "ContainsIncompleteClassType(clang::QualType)"},
    {"_Z28ComputeVMIClassTypeInfoFlagsPKN5clang16CXXBaseSpecifierERN12_GLOBAL__N_19SeenBasesE", "ComputeVMIClassTypeInfoFlags(clang::CXXBaseSpecifier const*, (anonymous namespace)::SeenBases&)"},
    {"_ZN5clang7CodeGen14CGBitFieldInfo8MakeInfoERNS0_12CodeGenTypesEPKNS_9FieldDeclEyyyj", "clang::CodeGen::CGBitFieldInfo::MakeInfo(clang::CodeGen::CodeGenTypes&, clang::FieldDecl const*, unsigned long long, unsigned long long, unsigned long long, unsigned int)"},
    {"_ZN5clang7CodeGen14CGBitFieldInfo8MakeInfoERNS0_12CodeGenTypesEPKNS_9FieldDeclEyy", "clang::CodeGen::CGBitFieldInfo::MakeInfo(clang::CodeGen::CodeGenTypes&, clang::FieldDecl const*, unsigned long long, unsigned long long)"},
    {"_ZN5clang7CodeGen12CodeGenTypes19ComputeRecordLayoutEPKNS_10RecordDeclE", "clang::CodeGen::CodeGenTypes::ComputeRecordLayout(clang::RecordDecl const*)"},
    {"_ZNK5clang7CodeGen14CGRecordLayout4dumpEv", "clang::CodeGen::CGRecordLayout::dump() const"},
    {"_ZNK5clang7CodeGen14CGRecordLayout5printERN4llvm11raw_ostreamE", "clang::CodeGen::CGRecordLayout::print(llvm::raw_ostream&) const"},
    {"_ZNK5clang7CodeGen14CGBitFieldInfo5printERN4llvm11raw_ostreamE", "clang::CodeGen::CGBitFieldInfo::print(llvm::raw_ostream&) const"},
    {"_ZNK5clang7CodeGen14CGBitFieldInfo4dumpEv", "clang::CodeGen::CGBitFieldInfo::dump() const"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIjPKN5clang7CodeGen14CGBitFieldInfoEEEEiPKvS9_", "int llvm::array_pod_sort_comparator<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*> >(void const*, void const*)"},
    {"_ZN12_GLOBAL__N_121CGRecordLayoutBuilder12LayoutFieldsEPKN5clang10RecordDeclE", "(anonymous namespace)::CGRecordLayoutBuilder::LayoutFields(clang::RecordDecl const*)"},
    {"_ZN12_GLOBAL__N_121CGRecordLayoutBuilder18LayoutVirtualBasesEPKN5clang13CXXRecordDeclERKNS1_15ASTRecordLayoutE", "(anonymous namespace)::CGRecordLayoutBuilder::LayoutVirtualBases(clang::CXXRecordDecl const*, clang::ASTRecordLayout const&)"},
    {"_ZN12_GLOBAL__N_121CGRecordLayoutBuilder17LayoutVirtualBaseEPKN5clang13CXXRecordDeclEy", "(anonymous namespace)::CGRecordLayoutBuilder::LayoutVirtualBase(clang::CXXRecordDecl const*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_121CGRecordLayoutBuilder20LayoutNonVirtualBaseEPKN5clang13CXXRecordDeclEy", "(anonymous namespace)::CGRecordLayoutBuilder::LayoutNonVirtualBase(clang::CXXRecordDecl const*, unsigned long long)"},
    {"_ZNSt6vectorISt4pairIjPKN5clang7CodeGen14CGBitFieldInfoEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*>, std::allocator<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*>*, std::vector<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*>, std::allocator<std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*> > > >, std::pair<unsigned int, clang::CodeGen::CGBitFieldInfo const*> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclENS1_7CodeGen14CGBitFieldInfoENS_12DenseMapInfoIS4_EENS7_IS6_EEE6insertERKSt4pairIS4_S6_E", "llvm::DenseMap<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<clang::CodeGen::CGBitFieldInfo> >::insert(std::pair<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclENS1_7CodeGen14CGBitFieldInfoENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<clang::CodeGen::CGBitFieldInfo> >::InsertIntoBucket(clang::FieldDecl const* const&, clang::CodeGen::CGBitFieldInfo const&, std::pair<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclENS1_7CodeGen14CGBitFieldInfoENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<clang::CodeGen::CGBitFieldInfo> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::FieldDecl const*, unsigned int, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::FieldDecl const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::FieldDecl const*, unsigned int, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::FieldDecl const* const&, unsigned int const&, std::pair<clang::FieldDecl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FieldDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::FieldDecl const*, unsigned int, llvm::DenseMapInfo<clang::FieldDecl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::CXXRecordDecl const*, unsigned int, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::CXXRecordDecl const*, unsigned int> const&)"},
    {"_ZN5clang7CodeGen14CGRecordLayoutC2EPKN4llvm10StructTypeES5_b", "clang::CodeGen::CGRecordLayout::CGRecordLayout(llvm::StructType const*, llvm::StructType const*, bool)"},
    {"_ZNSt6vectorIPKN4llvm4TypeESaIS3_EEaSERKS5_", "std::vector<llvm::Type const*, std::allocator<llvm::Type const*> >::operator=(std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPKN5clang9FieldDeclENS2_7CodeGen14CGBitFieldInfoEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::FieldDecl const*, clang::CodeGen::CGBitFieldInfo>, false>::grow(unsigned long)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitStopPointEPKNS_4StmtE", "clang::CodeGen::CodeGenFunction::EmitStopPoint(clang::Stmt const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction8EmitStmtEPKNS_4StmtE", "clang::CodeGen::CodeGenFunction::EmitStmt(clang::Stmt const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitSimpleStmtEPKNS_4StmtE", "clang::CodeGen::CodeGenFunction::EmitSimpleStmt(clang::Stmt const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitIndirectGotoStmtERKNS_16IndirectGotoStmtE", "clang::CodeGen::CodeGenFunction::EmitIndirectGotoStmt(clang::IndirectGotoStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction10EmitIfStmtERKNS_6IfStmtE", "clang::CodeGen::CodeGenFunction::EmitIfStmt(clang::IfStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitWhileStmtERKNS_9WhileStmtE", "clang::CodeGen::CodeGenFunction::EmitWhileStmt(clang::WhileStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction10EmitDoStmtERKNS_6DoStmtE", "clang::CodeGen::CodeGenFunction::EmitDoStmt(clang::DoStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitForStmtERKNS_7ForStmtE", "clang::CodeGen::CodeGenFunction::EmitForStmt(clang::ForStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitReturnStmtERKNS_10ReturnStmtE", "clang::CodeGen::CodeGenFunction::EmitReturnStmt(clang::ReturnStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14EmitSwitchStmtERKNS_10SwitchStmtE", "clang::CodeGen::CodeGenFunction::EmitSwitchStmt(clang::SwitchStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitAsmStmtERKNS_7AsmStmtE", "clang::CodeGen::CodeGenFunction::EmitAsmStmt(clang::AsmStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitCompoundStmtERKNS_12CompoundStmtEbNS0_12AggValueSlotE", "clang::CodeGen::CodeGenFunction::EmitCompoundStmt(clang::CompoundStmt const&, bool, clang::CodeGen::AggValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitDeclStmtERKNS_8DeclStmtE", "clang::CodeGen::CodeGenFunction::EmitDeclStmt(clang::DeclStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitLabelStmtERKNS_9LabelStmtE", "clang::CodeGen::CodeGenFunction::EmitLabelStmt(clang::LabelStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitGotoStmtERKNS_8GotoStmtE", "clang::CodeGen::CodeGenFunction::EmitGotoStmt(clang::GotoStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitBreakStmtERKNS_9BreakStmtE", "clang::CodeGen::CodeGenFunction::EmitBreakStmt(clang::BreakStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitContinueStmtERKNS_12ContinueStmtE", "clang::CodeGen::CodeGenFunction::EmitContinueStmt(clang::ContinueStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitDefaultStmtERKNS_11DefaultStmtE", "clang::CodeGen::CodeGenFunction::EmitDefaultStmt(clang::DefaultStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitCaseStmtERKNS_8CaseStmtE", "clang::CodeGen::CodeGenFunction::EmitCaseStmt(clang::CaseStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction9EmitLabelERKNS_9LabelStmtE", "clang::CodeGen::CodeGenFunction::EmitLabel(clang::LabelStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24SimplifyForwardingBlocksEPN4llvm10BasicBlockE", "clang::CodeGen::CodeGenFunction::SimplifyForwardingBlocks(llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction9EmitBlockEPN4llvm10BasicBlockEb", "clang::CodeGen::CodeGenFunction::EmitBlock(llvm::BasicBlock*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction10EmitBranchEPN4llvm10BasicBlockE", "clang::CodeGen::CodeGenFunction::EmitBranch(llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction19getJumpDestForLabelEPKNS_9LabelStmtE", "clang::CodeGen::CodeGenFunction::getJumpDestForLabel(clang::LabelStmt const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitReturnOfRValueENS0_6RValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitReturnOfRValue(clang::CodeGen::RValue, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17EmitCaseStmtRangeERKNS_8CaseStmtE", "clang::CodeGen::CodeGenFunction::EmitCaseStmtRange(clang::CaseStmt const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction18EmitAsmInputLValueERKNS_7AsmStmtERKNS_10TargetInfo14ConstraintInfoENS0_6LValueENS_8QualTypeERSs", "clang::CodeGen::CodeGenFunction::EmitAsmInputLValue(clang::AsmStmt const&, clang::TargetInfo::ConstraintInfo const&, clang::CodeGen::LValue, clang::QualType, std::string&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12EmitAsmInputERKNS_7AsmStmtERKNS_10TargetInfo14ConstraintInfoEPKNS_4ExprERSs", "clang::CodeGen::CodeGenFunction::EmitAsmInput(clang::AsmStmt const&, clang::TargetInfo::ConstraintInfo const&, clang::Expr const*, std::string&)"},
    {"_Z18SimplifyConstraintPKcRKN5clang10TargetInfoEPN4llvm15SmallVectorImplINS2_14ConstraintInfoEEE", "SimplifyConstraint(char const*, clang::TargetInfo const&, llvm::SmallVectorImpl<clang::TargetInfo::ConstraintInfo>*)"},
    {"_Z22AddVariableConstraintsRKSsRKN5clang4ExprERKNS1_10TargetInfoERNS1_7CodeGen13CodeGenModuleERKNS1_7AsmStmtE", "AddVariableConstraints(std::string const&, clang::Expr const&, clang::TargetInfo const&, clang::CodeGen::CodeGenModule&, clang::AsmStmt const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateSwitchEPNS_5ValueEPNS_10BasicBlockEj", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateSwitch(llvm::Value*, llvm::BasicBlock*, unsigned int)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCallIN9__gnu_cxx17__normal_iteratorIPPNS_5ValueESt6vectorIS9_SaIS9_EEEEEEPNS_8CallInstES9_T_SH_RKNS_5TwineE", "llvm::CallInst* llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCall<__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >(llvm::Value*, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, llvm::Twine const&)"},
    {"_ZNSt6vectorIN5clang7CodeGen6LValueESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::CodeGen::LValue, std::allocator<clang::CodeGen::LValue> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CodeGen::LValue*, std::vector<clang::CodeGen::LValue, std::allocator<clang::CodeGen::LValue> > >, clang::CodeGen::LValue const&)"},
    {"_ZNSt6vectorIN5clang8QualTypeESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::QualType, std::allocator<clang::QualType> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::QualType*, std::vector<clang::QualType, std::allocator<clang::QualType> > >, clang::QualType const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::VarDecl const*, llvm::Value*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(clang::VarDecl const* const&, llvm::Value* const&, std::pair<clang::VarDecl const*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::VarDecl const*, llvm::Value*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang9LabelStmtENS1_7CodeGen15CodeGenFunction8JumpDestENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::LabelStmt const*, clang::CodeGen::CodeGenFunction::JumpDest, llvm::DenseMapInfo<clang::LabelStmt const*>, llvm::DenseMapInfo<clang::CodeGen::CodeGenFunction::JumpDest> >::InsertIntoBucket(clang::LabelStmt const* const&, clang::CodeGen::CodeGenFunction::JumpDest const&, std::pair<clang::LabelStmt const*, clang::CodeGen::CodeGenFunction::JumpDest>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9LabelStmtENS1_7CodeGen15CodeGenFunction8JumpDestENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::LabelStmt const*, clang::CodeGen::CodeGenFunction::JumpDest, llvm::DenseMapInfo<clang::LabelStmt const*>, llvm::DenseMapInfo<clang::CodeGen::CodeGenFunction::JumpDest> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitCXXTemporaryEPKNS_12CXXTemporaryEPN4llvm5ValueE", "clang::CodeGen::CodeGenFunction::EmitCXXTemporary(clang::CXXTemporary const*, llvm::Value*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitExprWithCleanupsEPKNS_16ExprWithCleanupsENS0_12AggValueSlotE", "clang::CodeGen::CodeGenFunction::EmitExprWithCleanups(clang::ExprWithCleanups const*, clang::CodeGen::AggValueSlot)"},
    {"_ZN5clang7CodeGen15CodeGenFunction26EmitExprWithCleanupsLValueEPKNS_16ExprWithCleanupsE", "clang::CodeGen::CodeGenFunction::EmitExprWithCleanupsLValue(clang::ExprWithCleanups const*)"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEED1Ev", "clang::CodeGen::EHScopeStack::ConditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::~ConditionalCleanup2()"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEED0Ev", "clang::CodeGen::EHScopeStack::ConditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::~ConditionalCleanup2()"},
    {"_ZN5clang7CodeGen12EHScopeStack19ConditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEE4EmitERNS0_15CodeGenFunctionEb", "clang::CodeGen::EHScopeStack::ConditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEED1Ev", "clang::CodeGen::EHScopeStack::UnconditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::~UnconditionalCleanup2()"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEED0Ev", "clang::CodeGen::EHScopeStack::UnconditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::~UnconditionalCleanup2()"},
    {"_ZN5clang7CodeGen12EHScopeStack21UnconditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEE4EmitERNS0_15CodeGenFunctionEb", "clang::CodeGen::EHScopeStack::UnconditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN5clang7CodeGen14CodeGenVTables17EmitVTTDefinitionEPN4llvm14GlobalVariableENS2_11GlobalValue12LinkageTypesEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::EmitVTTDefinition(llvm::GlobalVariable*, llvm::GlobalValue::LinkageTypes, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_110VTTBuilderC1ERN5clang7CodeGen13CodeGenModuleEPKNS1_13CXXRecordDeclEb", "(anonymous namespace)::VTTBuilder::VTTBuilder(clang::CodeGen::CodeGenModule&, clang::CXXRecordDecl const*, bool)"},
    {"_ZN5clang7CodeGen14CodeGenVTables12GetAddrOfVTTEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::GetAddrOfVTT(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen14CodeGenVTables17needsVTTParameterENS0_10GlobalDeclE", "clang::CodeGen::CodeGenVTables::needsVTTParameter(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen14CodeGenVTables14getSubVTTIndexEPKNS_13CXXRecordDeclENS0_13BaseSubobjectE", "clang::CodeGen::CodeGenVTables::getSubVTTIndex(clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject)"},
    {"_ZN5clang7CodeGen14CodeGenVTables31getSecondaryVirtualPointerIndexEPKNS_13CXXRecordDeclENS0_13BaseSubobjectE", "clang::CodeGen::CodeGenVTables::getSecondaryVirtualPointerIndex(clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject)"},
    {"_ZN12_GLOBAL__N_110VTTBuilder9LayoutVTTEN5clang7CodeGen13BaseSubobjectEb", "(anonymous namespace)::VTTBuilder::LayoutVTT(clang::CodeGen::BaseSubobject, bool)"},
    {"_ZN12_GLOBAL__N_110VTTBuilder16AddVTablePointerEN5clang7CodeGen13BaseSubobjectEPN4llvm8ConstantEPKNS1_13CXXRecordDeclERKNS4_8DenseMapIS3_yNS4_12DenseMapInfoIS3_EENSB_IyEEEE", "(anonymous namespace)::VTTBuilder::AddVTablePointer(clang::CodeGen::BaseSubobject, llvm::Constant*, clang::CXXRecordDecl const*, llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> > const&)"},
    {"_ZN12_GLOBAL__N_110VTTBuilder17LayoutVirtualVTTsEPKN5clang13CXXRecordDeclERN4llvm11SmallPtrSetIS4_Lj4EEE", "(anonymous namespace)::VTTBuilder::LayoutVirtualVTTs(clang::CXXRecordDecl const*, llvm::SmallPtrSet<clang::CXXRecordDecl const*, 4u>&)"},
    {"_ZN12_GLOBAL__N_110VTTBuilder30LayoutSecondaryVirtualPointersEN5clang7CodeGen13BaseSubobjectEbPN4llvm8ConstantEPKNS1_13CXXRecordDeclERKNS4_8DenseMapIS3_yNS4_12DenseMapInfoIS3_EENSB_IyEEEERNS4_11SmallPtrSetIS9_Lj4EEE", "(anonymous namespace)::VTTBuilder::LayoutSecondaryVirtualPointers(clang::CodeGen::BaseSubobject, bool, llvm::Constant*, clang::CXXRecordDecl const*, llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> > const&, llvm::SmallPtrSet<clang::CXXRecordDecl const*, 4u>&)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclENS2_7CodeGen13BaseSubobjectEEyNS_12DenseMapInfoIS8_EENS9_IyEEE6insertERKS1_IS8_yE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> >, llvm::DenseMapInfo<unsigned long long> >::insert(std::pair<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long> const&)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen13BaseSubobjectEyNS_12DenseMapInfoIS3_EENS4_IyEEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> >::FindAndConstruct(clang::CodeGen::BaseSubobject const&)"},
    {"_ZN5clang7CodeGen14CodeGenVTables26ComputeMethodVTableIndicesEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::ComputeMethodVTableIndices(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen14CodeGenVTables29getNumVirtualFunctionPointersEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::getNumVirtualFunctionPointers(clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_127FindNearestOverriddenMethodEPKN5clang13CXXMethodDeclERN4llvm14SmallSetVectorIPKNS0_13CXXRecordDeclELj8EEE", "(anonymous namespace)::FindNearestOverriddenMethod(clang::CXXMethodDecl const*, llvm::SmallSetVector<clang::CXXRecordDecl const*, 8u>&)"},
    {"_ZN12_GLOBAL__N_133ComputeReturnAdjustmentBaseOffsetERN5clang10ASTContextEPKNS0_13CXXMethodDeclES5_", "(anonymous namespace)::ComputeReturnAdjustmentBaseOffset(clang::ASTContext&, clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZN5clang7CodeGen14CodeGenVTables20getMethodVTableIndexENS0_10GlobalDeclE", "clang::CodeGen::CodeGenVTables::getMethodVTableIndex(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen14CodeGenVTables24ShouldEmitVTableInThisTUEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::ShouldEmitVTableInThisTU(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen14CodeGenVTables26getVirtualBaseOffsetOffsetEPKNS_13CXXRecordDeclES4_", "clang::CodeGen::CodeGenVTables::getVirtualBaseOffsetOffset(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_126VCallAndVBaseOffsetBuilderC1EPKN5clang13CXXRecordDeclES4_PKNS_15FinalOverridersENS1_7CodeGen13BaseSubobjectEby", "(anonymous namespace)::VCallAndVBaseOffsetBuilder::VCallAndVBaseOffsetBuilder(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*, (anonymous namespace)::FinalOverriders const*, clang::CodeGen::BaseSubobject, bool, unsigned long long)"},
    {"_ZN5clang7CodeGen14CodeGenVTables15getAddressPointENS0_13BaseSubobjectEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::getAddressPoint(clang::CodeGen::BaseSubobject, clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule14GetAddrOfThunkENS0_10GlobalDeclERKNS_9ThunkInfoE", "clang::CodeGen::CodeGenModule::GetAddrOfThunk(clang::CodeGen::GlobalDecl, clang::ThunkInfo const&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13GenerateThunkEPN4llvm8FunctionENS0_10GlobalDeclERKNS_9ThunkInfoE", "clang::CodeGen::CodeGenFunction::GenerateThunk(llvm::Function*, clang::CodeGen::GlobalDecl, clang::ThunkInfo const&)"},
    {"_Z21PerformTypeAdjustmentRN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueExx", "PerformTypeAdjustment(clang::CodeGen::CodeGenFunction&, llvm::Value*, long long, long long)"},
    {"_ZN5clang7CodeGen14CodeGenVTables9EmitThunkENS0_10GlobalDeclERKNS_9ThunkInfoEb", "clang::CodeGen::CodeGenVTables::EmitThunk(clang::CodeGen::GlobalDecl, clang::ThunkInfo const&, bool)"},
    {"_ZN5clang7CodeGen14CodeGenVTables33MaybeEmitThunkAvailableExternallyENS0_10GlobalDeclERKNS_9ThunkInfoE", "clang::CodeGen::CodeGenVTables::MaybeEmitThunkAvailableExternally(clang::CodeGen::GlobalDecl, clang::ThunkInfo const&)"},
    {"_ZN5clang7CodeGen14CodeGenVTables10EmitThunksENS0_10GlobalDeclE", "clang::CodeGen::CodeGenVTables::EmitThunks(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen14CodeGenVTables31ComputeVTableRelatedInformationEPKNS_13CXXRecordDeclEb", "clang::CodeGen::CodeGenVTables::ComputeVTableRelatedInformation(clang::CXXRecordDecl const*, bool)"},
    {"_ZN12_GLOBAL__N_113VTableBuilderC1ERN5clang7CodeGen14CodeGenVTablesEPKNS1_13CXXRecordDeclEybS7_", "(anonymous namespace)::VTableBuilder::VTableBuilder(clang::CodeGen::CodeGenVTables&, clang::CXXRecordDecl const*, unsigned long long, bool, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_113VTableBuilderD1Ev", "(anonymous namespace)::VTableBuilder::~VTableBuilder()"},
    {"_ZN5clang7CodeGen14CodeGenVTables23CreateVTableInitializerEPKNS_13CXXRecordDeclEPKyjRKN4llvm11SmallVectorISt4pairIyNS_9ThunkInfoEELj1EEE", "clang::CodeGen::CodeGenVTables::CreateVTableInitializer(clang::CXXRecordDecl const*, unsigned long long const*, unsigned int, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> const&)"},
    {"_ZN5clang7CodeGen14CodeGenVTables15GetAddrOfVTableEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::GetAddrOfVTable(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen14CodeGenVTables20EmitVTableDefinitionEPN4llvm14GlobalVariableENS2_11GlobalValue12LinkageTypesEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::EmitVTableDefinition(llvm::GlobalVariable*, llvm::GlobalValue::LinkageTypes, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder10dumpLayoutERN4llvm11raw_ostreamE", "(anonymous namespace)::VTableBuilder::dumpLayout(llvm::raw_ostream&)"},
    {"_ZN5clang7CodeGen14CodeGenVTables26GenerateConstructionVTableEPKNS_13CXXRecordDeclERKNS0_13BaseSubobjectEbRN4llvm8DenseMapIS5_yNS8_12DenseMapInfoIS5_EENSA_IyEEEE", "clang::CodeGen::CodeGenVTables::GenerateConstructionVTable(clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject const&, bool, llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> >&)"},
    {"_ZN5clang7CodeGen14CodeGenVTables17GenerateClassDataEN4llvm11GlobalValue12LinkageTypesEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenVTables::GenerateClassData(llvm::GlobalValue::LinkageTypes, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder32LayoutPrimaryAndSecondaryVTablesEN5clang7CodeGen13BaseSubobjectEbby", "(anonymous namespace)::VTableBuilder::LayoutPrimaryAndSecondaryVTables(clang::CodeGen::BaseSubobject, bool, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder28DeterminePrimaryVirtualBasesEPKN5clang13CXXRecordDeclEyRN4llvm11SmallPtrSetIS4_Lj4EEE", "(anonymous namespace)::VTableBuilder::DeterminePrimaryVirtualBases(clang::CXXRecordDecl const*, unsigned long long, llvm::SmallPtrSet<clang::CXXRecordDecl const*, 4u>&)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder28LayoutVTablesForVirtualBasesEPKN5clang13CXXRecordDeclERN4llvm11SmallPtrSetIS4_Lj4EEE", "(anonymous namespace)::VTableBuilder::LayoutVTablesForVirtualBases(clang::CXXRecordDecl const*, llvm::SmallPtrSet<clang::CXXRecordDecl const*, 4u>&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEN12_GLOBAL__N_114VCallOffsetMapENS_12DenseMapInfoIS4_EENS7_IS6_EEEixERKS4_", "llvm::DenseMap<clang::CXXRecordDecl const*, (anonymous namespace)::VCallOffsetMap, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<(anonymous namespace)::VCallOffsetMap> >::operator[](clang::CXXRecordDecl const* const&)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder10AddMethodsEN5clang7CodeGen13BaseSubobjectEyPKNS1_13CXXRecordDeclEyRN4llvm14SmallSetVectorIS6_Lj8EEE", "(anonymous namespace)::VTableBuilder::AddMethods(clang::CodeGen::BaseSubobject, unsigned long long, clang::CXXRecordDecl const*, unsigned long long, llvm::SmallSetVector<clang::CXXRecordDecl const*, 8u>&)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder22LayoutSecondaryVTablesEN5clang7CodeGen13BaseSubobjectEby", "(anonymous namespace)::VTableBuilder::LayoutSecondaryVTables(clang::CodeGen::BaseSubobject, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_113VTableBuilder21ComputeThisAdjustmentEPKN5clang13CXXMethodDeclEyNS_15FinalOverriders13OverriderInfoE", "(anonymous namespace)::VTableBuilder::ComputeThisAdjustment(clang::CXXMethodDecl const*, unsigned long long, (anonymous namespace)::FinalOverriders::OverriderInfo)"},
    {"_ZN12_GLOBAL__N_114VCallOffsetMap26MethodsCanShareVCallOffsetEPKN5clang13CXXMethodDeclES4_", "(anonymous namespace)::VCallOffsetMap::MethodsCanShareVCallOffset(clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_117ComputeBaseOffsetERN5clang10ASTContextEPKNS0_13CXXRecordDeclERKNS0_11CXXBasePathE", "(anonymous namespace)::ComputeBaseOffset(clang::ASTContext&, clang::CXXRecordDecl const*, clang::CXXBasePath const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPKN5clang13CXXMethodDeclEyEN12_GLOBAL__N_115FinalOverriders13OverriderInfoENS_12DenseMapInfoIS6_EENSA_IS9_EEE15LookupBucketForERKS6_RPS1_IS6_S9_E", "llvm::DenseMap<std::pair<clang::CXXMethodDecl const*, unsigned long long>, (anonymous namespace)::FinalOverriders::OverriderInfo, llvm::DenseMapInfo<std::pair<clang::CXXMethodDecl const*, unsigned long long> >, llvm::DenseMapInfo<(anonymous namespace)::FinalOverriders::OverriderInfo> >::LookupBucketFor(std::pair<clang::CXXMethodDecl const*, unsigned long long> const&, std::pair<std::pair<clang::CXXMethodDecl const*, unsigned long long>, (anonymous namespace)::FinalOverriders::OverriderInfo>*&) const"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclEN12_GLOBAL__N_113VTableBuilder10MethodInfoENS_12DenseMapInfoIS4_EENS8_IS7_EEE6insertERKSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXMethodDecl const*, (anonymous namespace)::VTableBuilder::MethodInfo, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<(anonymous namespace)::VTableBuilder::MethodInfo> >::insert(std::pair<clang::CXXMethodDecl const*, (anonymous namespace)::VTableBuilder::MethodInfo> const&)"},
    {"_ZN12_GLOBAL__N_130OverridesIndirectMethodInBasesEPKN5clang13CXXMethodDeclERN4llvm14SmallSetVectorIPKNS0_13CXXRecordDeclELj8EEE", "(anonymous namespace)::OverridesIndirectMethodInBases(clang::CXXMethodDecl const*, llvm::SmallSetVector<clang::CXXRecordDecl const*, 8u>&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclEN12_GLOBAL__N_113VTableBuilder10MethodInfoENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXMethodDecl const*, (anonymous namespace)::VTableBuilder::MethodInfo, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<(anonymous namespace)::VTableBuilder::MethodInfo> >::InsertIntoBucket(clang::CXXMethodDecl const* const&, (anonymous namespace)::VTableBuilder::MethodInfo const&, std::pair<clang::CXXMethodDecl const*, (anonymous namespace)::VTableBuilder::MethodInfo>*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_115VTableComponentELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::VTableComponent, false>::grow(unsigned long)"},
    {"_ZN12_GLOBAL__N_115FinalOverriders18ComputeBaseOffsetsEN5clang7CodeGen13BaseSubobjectEbyRN4llvm8DenseMapISt4pairIPKNS1_13CXXRecordDeclEjEyNS4_12DenseMapInfoISA_EENSB_IyEEEESF_RNS5_IS9_jNSB_IS9_EENSB_IjEEEE", "(anonymous namespace)::FinalOverriders::ComputeBaseOffsets(clang::CodeGen::BaseSubobject, bool, unsigned long long, llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, unsigned int> >, llvm::DenseMapInfo<unsigned long long> >&, llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, unsigned int> >, llvm::DenseMapInfo<unsigned long long> >&, llvm::DenseMap<clang::CXXRecordDecl const*, unsigned int, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN12_GLOBAL__N_126VCallAndVBaseOffsetBuilder23AddVCallAndVBaseOffsetsEN5clang7CodeGen13BaseSubobjectEby", "(anonymous namespace)::VCallAndVBaseOffsetBuilder::AddVCallAndVBaseOffsets(clang::CodeGen::BaseSubobject, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_126VCallAndVBaseOffsetBuilder15AddVBaseOffsetsEPKN5clang13CXXRecordDeclEy", "(anonymous namespace)::VCallAndVBaseOffsetBuilder::AddVBaseOffsets(clang::CXXRecordDecl const*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_126VCallAndVBaseOffsetBuilder15AddVCallOffsetsEN5clang7CodeGen13BaseSubobjectEy", "(anonymous namespace)::VCallAndVBaseOffsetBuilder::AddVCallOffsets(clang::CodeGen::BaseSubobject, unsigned long long)"},
    {"_ZN12_GLOBAL__N_127ComputeAllOverriddenMethodsEPKN5clang13CXXMethodDeclERN4llvm11SmallPtrSetIS3_Lj8EEE", "(anonymous namespace)::ComputeAllOverriddenMethods(clang::CXXMethodDecl const*, llvm::SmallPtrSet<clang::CXXMethodDecl const*, 8u>&)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclES5_ExNS_12DenseMapInfoIS6_EENS7_IxEEE6insertERKS1_IS6_xE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> >, llvm::DenseMapInfo<long long> >::insert(std::pair<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long> const&)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIyN5clang9ThunkInfoEEE6appendINS_16DenseMapIteratorIyS3_NS_12DenseMapInfoIyEENS8_IS3_EELb1EEEEEvT_SC_", "void llvm::SmallVectorImpl<std::pair<unsigned long long, clang::ThunkInfo> >::append<llvm::DenseMapIterator<unsigned long long, clang::ThunkInfo, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<clang::ThunkInfo>, true> >(llvm::DenseMapIterator<unsigned long long, clang::ThunkInfo, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<clang::ThunkInfo>, true>, llvm::DenseMapIterator<unsigned long long, clang::ThunkInfo, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<clang::ThunkInfo>, true>)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPNS_14GlobalVariableENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, llvm::GlobalVariable* const&, std::pair<clang::CXXRecordDecl const*, llvm::GlobalVariable*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPNS_14GlobalVariableENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::GlobalVariable*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen13BaseSubobjectEyNS_12DenseMapInfoIS3_EENS4_IyEEE6insertERKSt4pairIS3_yE", "llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> >::insert(std::pair<clang::CodeGen::BaseSubobject, unsigned long long> const&)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen13BaseSubobjectEyNS_12DenseMapInfoIS3_EENS4_IyEEE16InsertIntoBucketERKS3_RKyPSt4pairIS3_yE", "llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(clang::CodeGen::BaseSubobject const&, unsigned long long const&, std::pair<clang::CodeGen::BaseSubobject, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen13BaseSubobjectEyNS_12DenseMapInfoIS3_EENS4_IyEEE4growEj", "llvm::DenseMap<clang::CodeGen::BaseSubobject, unsigned long long, llvm::DenseMapInfo<clang::CodeGen::BaseSubobject>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclES5_ExNS_12DenseMapInfoIS6_EENS7_IxEEE15LookupBucketForERKS6_RPS1_IS6_xE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> >, llvm::DenseMapInfo<long long> >::LookupBucketFor(std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> const&, std::pair<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long>*&) const"},
    {"_ZNK4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclENS2_7CodeGen13BaseSubobjectEEyNS_12DenseMapInfoIS8_EENS9_IyEEE15LookupBucketForERKS8_RPS1_IS8_yE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> >, llvm::DenseMapInfo<unsigned long long> >::LookupBucketFor(std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> const&, std::pair<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclENS2_7CodeGen13BaseSubobjectEEyNS_12DenseMapInfoIS8_EENS9_IyEEE16InsertIntoBucketERKS8_RKyPS1_IS8_yE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> >, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> const&, unsigned long long const&, std::pair<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclENS2_7CodeGen13BaseSubobjectEEyNS_12DenseMapInfoIS8_EENS9_IyEEE4growEj", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CodeGen::BaseSubobject> >, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIPSt4pairIyN5clang9ThunkInfoEElEvT_S5_T0_", "void std::__introsort_loop<std::pair<unsigned long long, clang::ThunkInfo>*, long>(std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*, long)"},
    {"_ZSt22__final_insertion_sortIPSt4pairIyN5clang9ThunkInfoEEEvT_S5_", "void std::__final_insertion_sort<std::pair<unsigned long long, clang::ThunkInfo>*>(std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*)"},
    {"_ZSt16__insertion_sortIPSt4pairIyN5clang9ThunkInfoEEEvT_S5_", "void std::__insertion_sort<std::pair<unsigned long long, clang::ThunkInfo>*>(std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*)"},
    {"_ZSt21__unguarded_partitionIPSt4pairIyN5clang9ThunkInfoEES3_ET_S5_S5_T0_", "std::pair<unsigned long long, clang::ThunkInfo>* std::__unguarded_partition<std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo> >(std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>)"},
    {"_ZSt8__medianISt4pairIyN5clang9ThunkInfoEEERKT_S6_S6_S6_", "std::pair<unsigned long long, clang::ThunkInfo> const& std::__median<std::pair<unsigned long long, clang::ThunkInfo> >(std::pair<unsigned long long, clang::ThunkInfo> const&, std::pair<unsigned long long, clang::ThunkInfo> const&, std::pair<unsigned long long, clang::ThunkInfo> const&)"},
    {"_ZSt13__heap_selectIPSt4pairIyN5clang9ThunkInfoEEEvT_S5_S5_", "void std::__heap_select<std::pair<unsigned long long, clang::ThunkInfo>*>(std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*, std::pair<unsigned long long, clang::ThunkInfo>*)"},
    {"_ZSt13__adjust_heapIPSt4pairIyN5clang9ThunkInfoEElS3_EvT_T0_S6_T1_", "void std::__adjust_heap<std::pair<unsigned long long, clang::ThunkInfo>*, long, std::pair<unsigned long long, clang::ThunkInfo> >(std::pair<unsigned long long, clang::ThunkInfo>*, long, long, std::pair<unsigned long long, clang::ThunkInfo>)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIyN5clang9ThunkInfoEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<unsigned long long, clang::ThunkInfo>, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorISt4pairIyNS1_9ThunkInfoEELj1EEENS_12DenseMapInfoIS4_EENSA_IS9_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> > >::FindAndConstruct(clang::CXXRecordDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorISt4pairIyNS1_9ThunkInfoEELj1EEENS_12DenseMapInfoIS4_EENSA_IS9_EEE16InsertIntoBucketERKS4_RKS9_PS6_IS4_S9_E", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> > >::InsertIntoBucket(clang::CXXRecordDecl const* const&, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> const&, std::pair<clang::CXXRecordDecl const*, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_11SmallVectorISt4pairIyNS1_9ThunkInfoEELj1EEENS_12DenseMapInfoIS4_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u>, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned long long, clang::ThunkInfo>, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIyN5clang9ThunkInfoEEEaSERKS5_", "llvm::SmallVectorImpl<std::pair<unsigned long long, clang::ThunkInfo> >::operator=(llvm::SmallVectorImpl<std::pair<unsigned long long, clang::ThunkInfo> > const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS_11SmallVectorINS1_9ThunkInfoELj1EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE6insertERKSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ThunkInfo, 1u> > >::insert(std::pair<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u> > const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS_11SmallVectorINS1_9ThunkInfoELj1EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ThunkInfo, 1u> > >::InsertIntoBucket(clang::CXXMethodDecl const* const&, llvm::SmallVector<clang::ThunkInfo, 1u> const&, std::pair<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS_11SmallVectorINS1_9ThunkInfoELj1EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ThunkInfo, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang9ThunkInfoEEaSERKS3_", "llvm::SmallVectorImpl<clang::ThunkInfo>::operator=(llvm::SmallVectorImpl<clang::ThunkInfo> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang9ThunkInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ThunkInfo, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIPKN5clang13CXXRecordDeclESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::CXXRecordDecl const*, std::allocator<clang::CXXRecordDecl const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CXXRecordDecl const**, std::vector<clang::CXXRecordDecl const*, std::allocator<clang::CXXRecordDecl const*> > >, clang::CXXRecordDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_14PointerIntPairIPyLj1EbNS_21PointerLikeTypeTraitsIS6_EEEENS_12DenseMapInfoIS4_EENSA_IS9_EEE16InsertIntoBucketERKS4_RKS9_PSt4pairIS4_S9_E", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> >, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> > > >::InsertIntoBucket(clang::CXXRecordDecl const* const&, llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> > const&, std::pair<clang::CXXRecordDecl const*, llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> > >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS_14PointerIntPairIPyLj1EbNS_21PointerLikeTypeTraitsIS6_EEEENS_12DenseMapInfoIS4_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> >, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<llvm::PointerIntPair<unsigned long long*, 1u, bool, llvm::PointerLikeTypeTraits<unsigned long long*> > > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclES5_ExNS_12DenseMapInfoIS6_EENS7_IxEEE16InsertIntoBucketERKS6_RKxPS1_IS6_xE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> >, llvm::DenseMapInfo<long long> >::InsertIntoBucket(std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> const&, long long const&, std::pair<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclES5_ExNS_12DenseMapInfoIS6_EENS7_IxEEE4growEj", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*>, long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, clang::CXXRecordDecl const*> >, llvm::DenseMapInfo<long long> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEyNS_12DenseMapInfoIS4_EENS5_IyEEE16InsertIntoBucketERKS4_RKyPSt4pairIS4_yE", "llvm::DenseMap<clang::CXXRecordDecl const*, unsigned long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, unsigned long long const&, std::pair<clang::CXXRecordDecl const*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEyNS_12DenseMapInfoIS4_EENS5_IyEEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, unsigned long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen10GlobalDeclExNS_12DenseMapInfoIS3_EENS4_IxEEE16InsertIntoBucketERKS3_RKxPSt4pairIS3_xE", "llvm::DenseMap<clang::CodeGen::GlobalDecl, long long, llvm::DenseMapInfo<clang::CodeGen::GlobalDecl>, llvm::DenseMapInfo<long long> >::InsertIntoBucket(clang::CodeGen::GlobalDecl const&, long long const&, std::pair<clang::CodeGen::GlobalDecl, long long>*)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen10GlobalDeclExNS_12DenseMapInfoIS3_EENS4_IxEEE4growEj", "llvm::DenseMap<clang::CodeGen::GlobalDecl, long long, llvm::DenseMapInfo<clang::CodeGen::GlobalDecl>, llvm::DenseMapInfo<long long> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIySt4pairIKyN5clang7CodeGen13BaseSubobjectEESt10_Select1stIS5_ESt4lessIyESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned long long, std::pair<unsigned long long const, clang::CodeGen::BaseSubobject>, std::_Select1st<std::pair<unsigned long long const, clang::CodeGen::BaseSubobject> >, std::less<unsigned long long>, std::allocator<std::pair<unsigned long long const, clang::CodeGen::BaseSubobject> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned long long const, clang::CodeGen::BaseSubobject> >*)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPKN5clang13CXXMethodDeclEESt10_Select1stIS6_ESt4lessISsESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::string, std::pair<std::string const, clang::CXXMethodDecl const*>, std::_Select1st<std::pair<std::string const, clang::CXXMethodDecl const*> >, std::less<std::string>, std::allocator<std::pair<std::string const, clang::CXXMethodDecl const*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, clang::CXXMethodDecl const*> >*)"},
    {"_ZSt16__introsort_loopIPN5clang9ThunkInfoElEvT_S3_T0_", "void std::__introsort_loop<clang::ThunkInfo*, long>(clang::ThunkInfo*, clang::ThunkInfo*, long)"},
    {"_ZSt22__final_insertion_sortIPN5clang9ThunkInfoEEvT_S3_", "void std::__final_insertion_sort<clang::ThunkInfo*>(clang::ThunkInfo*, clang::ThunkInfo*)"},
    {"_ZSt16__insertion_sortIPN5clang9ThunkInfoEEvT_S3_", "void std::__insertion_sort<clang::ThunkInfo*>(clang::ThunkInfo*, clang::ThunkInfo*)"},
    {"_ZSt21__unguarded_partitionIPN5clang9ThunkInfoES1_ET_S3_S3_T0_", "clang::ThunkInfo* std::__unguarded_partition<clang::ThunkInfo*, clang::ThunkInfo>(clang::ThunkInfo*, clang::ThunkInfo*, clang::ThunkInfo)"},
    {"_ZSt8__medianIN5clang9ThunkInfoEERKT_S4_S4_S4_", "clang::ThunkInfo const& std::__median<clang::ThunkInfo>(clang::ThunkInfo const&, clang::ThunkInfo const&, clang::ThunkInfo const&)"},
    {"_ZSt13__heap_selectIPN5clang9ThunkInfoEEvT_S3_S3_", "void std::__heap_select<clang::ThunkInfo*>(clang::ThunkInfo*, clang::ThunkInfo*, clang::ThunkInfo*)"},
    {"_ZSt13__adjust_heapIPN5clang9ThunkInfoElS1_EvT_T0_S4_T1_", "void std::__adjust_heap<clang::ThunkInfo*, long, clang::ThunkInfo>(clang::ThunkInfo*, long, long, clang::ThunkInfo)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS_11SmallVectorINS1_9ThunkInfoELj1EEENS_12DenseMapInfoIS4_EENS8_IS7_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::CXXMethodDecl const*, llvm::SmallVector<clang::ThunkInfo, 1u>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ThunkInfo, 1u> > >::FindAndConstruct(clang::CXXMethodDecl const* const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPKN5clang13CXXMethodDeclEESt10_Select1stIS6_ESt4lessISsESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::string, std::pair<std::string const, clang::CXXMethodDecl const*>, std::_Select1st<std::pair<std::string const, clang::CXXMethodDecl const*> >, std::less<std::string>, std::allocator<std::pair<std::string const, clang::CXXMethodDecl const*> > >::_M_insert_unique(std::pair<std::string const, clang::CXXMethodDecl const*> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPKN5clang13CXXMethodDeclEESt10_Select1stIS6_ESt4lessISsESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::string, std::pair<std::string const, clang::CXXMethodDecl const*>, std::_Select1st<std::pair<std::string const, clang::CXXMethodDecl const*> >, std::less<std::string>, std::allocator<std::pair<std::string const, clang::CXXMethodDecl const*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, clang::CXXMethodDecl const*> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsxESt10_Select1stIS2_ESt4lessISsESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<std::string, std::pair<std::string const, long long>, std::_Select1st<std::pair<std::string const, long long> >, std::less<std::string>, std::allocator<std::pair<std::string const, long long> > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, long long> >*)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsxESt10_Select1stIS2_ESt4lessISsESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<std::string, std::pair<std::string const, long long>, std::_Select1st<std::pair<std::string const, long long> >, std::less<std::string>, std::allocator<std::pair<std::string const, long long> > >::_M_insert_unique(std::pair<std::string const, long long> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsxESt10_Select1stIS2_ESt4lessISsESaIS2_EE9_M_insertEPSt18_Rb_tree_node_baseSA_RKS2_", "std::_Rb_tree<std::string, std::pair<std::string const, long long>, std::_Select1st<std::pair<std::string const, long long> >, std::less<std::string>, std::allocator<std::pair<std::string const, long long> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, long long> const&)"},
    {"_ZNK5clang15ASTRecordLayout25getVBaseClassOffsetInBitsEPKNS_13CXXRecordDeclE", "clang::ASTRecordLayout::getVBaseClassOffsetInBits(clang::CXXRecordDecl const*) const"},
    {"_ZSt6__findIPN5clang9ThunkInfoES1_ET_S3_S3_RKT0_St26random_access_iterator_tag", "clang::ThunkInfo* std::__find<clang::ThunkInfo*, clang::ThunkInfo>(clang::ThunkInfo*, clang::ThunkInfo*, clang::ThunkInfo const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIyN5clang9ThunkInfoENS_12DenseMapInfoIyEENS3_IS2_EEE16InsertIntoBucketERKyRKS2_PSt4pairIyS2_E", "llvm::DenseMap<unsigned long long, clang::ThunkInfo, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<clang::ThunkInfo> >::InsertIntoBucket(unsigned long long const&, clang::ThunkInfo const&, std::pair<unsigned long long, clang::ThunkInfo>*)"},
    {"_ZN4llvm8DenseMapIyN5clang9ThunkInfoENS_12DenseMapInfoIyEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned long long, clang::ThunkInfo, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<clang::ThunkInfo> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclExNS_12DenseMapInfoIS4_EENS5_IxEEE8CopyFromERKS8_", "llvm::DenseMap<clang::CXXRecordDecl const*, long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<long long> >::CopyFrom(llvm::DenseMap<clang::CXXRecordDecl const*, long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<long long> > const&)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPKN5clang13CXXMethodDeclExEEaSERKS7_", "llvm::SmallVectorImpl<std::pair<clang::CXXMethodDecl const*, long long> >::operator=(llvm::SmallVectorImpl<std::pair<clang::CXXMethodDecl const*, long long> > const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclEjEyNS_12DenseMapInfoIS6_EENS7_IyEEE15LookupBucketForERKS6_RPS1_IS6_yE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, unsigned int> >, llvm::DenseMapInfo<unsigned long long> >::LookupBucketFor(std::pair<clang::CXXRecordDecl const*, unsigned int> const&, std::pair<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclEjEyNS_12DenseMapInfoIS6_EENS7_IyEEE16InsertIntoBucketERKS6_RKyPS1_IS6_yE", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, unsigned int> >, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(std::pair<clang::CXXRecordDecl const*, unsigned int> const&, unsigned long long const&, std::pair<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPKN5clang13CXXRecordDeclEjEyNS_12DenseMapInfoIS6_EENS7_IyEEE4growEj", "llvm::DenseMap<std::pair<clang::CXXRecordDecl const*, unsigned int>, unsigned long long, llvm::DenseMapInfo<std::pair<clang::CXXRecordDecl const*, unsigned int> >, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE26CreateConstInBoundsGEP1_64EPNS_5ValueEyRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstInBoundsGEP1_64(llvm::Value*, unsigned long long, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE17CreateInBoundsGEPEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInBoundsGEP(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclExNS_12DenseMapInfoIS4_EENS5_IxEEE6insertERKSt4pairIS4_xE", "llvm::DenseMap<clang::CXXRecordDecl const*, long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<long long> >::insert(std::pair<clang::CXXRecordDecl const*, long long> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclExNS_12DenseMapInfoIS4_EENS5_IxEEE16InsertIntoBucketERKS4_RKxPSt4pairIS4_xE", "llvm::DenseMap<clang::CXXRecordDecl const*, long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<long long> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, long long const&, std::pair<clang::CXXRecordDecl const*, long long>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclExNS_12DenseMapInfoIS4_EENS5_IxEEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, long long, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<long long> >::grow(unsigned int)"},
    {"_ZN5clang15BackendConsumer21InlineAsmDiagHandler2ERKN4llvm12SMDiagnosticENS_14SourceLocationE", "clang::BackendConsumer::InlineAsmDiagHandler2(llvm::SMDiagnostic const&, clang::SourceLocation)"},
    {"_ZN5clang13CodeGenActionC1Ej", "clang::CodeGenAction::CodeGenAction(unsigned int)"},
    {"_ZN5clang13CodeGenActionC2Ej", "clang::CodeGenAction::CodeGenAction(unsigned int)"},
    {"_ZN5clang13CodeGenActionD0Ev", "clang::CodeGenAction::~CodeGenAction()"},
    {"_ZN5clang13CodeGenActionD1Ev", "clang::CodeGenAction::~CodeGenAction()"},
    {"_ZN5clang13CodeGenActionD2Ev", "clang::CodeGenAction::~CodeGenAction()"},
    {"_ZNK5clang13CodeGenAction12hasIRSupportEv", "clang::CodeGenAction::hasIRSupport() const"},
    {"_ZN5clang13CodeGenAction19EndSourceFileActionEv", "clang::CodeGenAction::EndSourceFileAction()"},
    {"_ZN5clang13CodeGenAction10takeModuleEv", "clang::CodeGenAction::takeModule()"},
    {"_ZN5clang13CodeGenAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::CodeGenAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_Z15GetOutputStreamRN5clang16CompilerInstanceEN4llvm9StringRefENS_13BackendActionE", "GetOutputStream(clang::CompilerInstance&, llvm::StringRef, clang::BackendAction)"},
    {"_ZN5clang13CodeGenAction13ExecuteActionEv", "clang::CodeGenAction::ExecuteAction()"},
    {"_ZN5clang18EmitAssemblyActionC1Ev", "clang::EmitAssemblyAction::EmitAssemblyAction()"},
    {"_ZN5clang18EmitAssemblyActionC2Ev", "clang::EmitAssemblyAction::EmitAssemblyAction()"},
    {"_ZN5clang12EmitBCActionC1Ev", "clang::EmitBCAction::EmitBCAction()"},
    {"_ZN5clang12EmitBCActionC2Ev", "clang::EmitBCAction::EmitBCAction()"},
    {"_ZN5clang14EmitLLVMActionC1Ev", "clang::EmitLLVMAction::EmitLLVMAction()"},
    {"_ZN5clang14EmitLLVMActionC2Ev", "clang::EmitLLVMAction::EmitLLVMAction()"},
    {"_ZN5clang18EmitLLVMOnlyActionC1Ev", "clang::EmitLLVMOnlyAction::EmitLLVMOnlyAction()"},
    {"_ZN5clang18EmitLLVMOnlyActionC2Ev", "clang::EmitLLVMOnlyAction::EmitLLVMOnlyAction()"},
    {"_ZN5clang21EmitCodeGenOnlyActionC1Ev", "clang::EmitCodeGenOnlyAction::EmitCodeGenOnlyAction()"},
    {"_ZN5clang21EmitCodeGenOnlyActionC2Ev", "clang::EmitCodeGenOnlyAction::EmitCodeGenOnlyAction()"},
    {"_ZN5clang13EmitObjActionC1Ev", "clang::EmitObjAction::EmitObjAction()"},
    {"_ZN5clang13EmitObjActionC2Ev", "clang::EmitObjAction::EmitObjAction()"},
    {"_ZN4llvm7ParseIREPNS_12MemoryBufferERNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::ParseIR(llvm::MemoryBuffer*, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZN5clang13EmitObjActionD1Ev", "clang::EmitObjAction::~EmitObjAction()"},
    {"_ZN5clang13EmitObjActionD0Ev", "clang::EmitObjAction::~EmitObjAction()"},
    {"_ZN5clang21EmitCodeGenOnlyActionD1Ev", "clang::EmitCodeGenOnlyAction::~EmitCodeGenOnlyAction()"},
    {"_ZN5clang21EmitCodeGenOnlyActionD0Ev", "clang::EmitCodeGenOnlyAction::~EmitCodeGenOnlyAction()"},
    {"_ZN5clang18EmitLLVMOnlyActionD1Ev", "clang::EmitLLVMOnlyAction::~EmitLLVMOnlyAction()"},
    {"_ZN5clang18EmitLLVMOnlyActionD0Ev", "clang::EmitLLVMOnlyAction::~EmitLLVMOnlyAction()"},
    {"_ZN5clang14EmitLLVMActionD1Ev", "clang::EmitLLVMAction::~EmitLLVMAction()"},
    {"_ZN5clang14EmitLLVMActionD0Ev", "clang::EmitLLVMAction::~EmitLLVMAction()"},
    {"_ZN5clang12EmitBCActionD1Ev", "clang::EmitBCAction::~EmitBCAction()"},
    {"_ZN5clang12EmitBCActionD0Ev", "clang::EmitBCAction::~EmitBCAction()"},
    {"_ZN5clang18EmitAssemblyActionD1Ev", "clang::EmitAssemblyAction::~EmitAssemblyAction()"},
    {"_ZN5clang18EmitAssemblyActionD0Ev", "clang::EmitAssemblyAction::~EmitAssemblyAction()"},
    {"_ZN5clang15BackendConsumerD1Ev", "clang::BackendConsumer::~BackendConsumer()"},
    {"_ZN5clang15BackendConsumerD0Ev", "clang::BackendConsumer::~BackendConsumer()"},
    {"_ZN5clang15BackendConsumer10InitializeERNS_10ASTContextE", "clang::BackendConsumer::Initialize(clang::ASTContext&)"},
    {"_ZN5clang15BackendConsumer18HandleTopLevelDeclENS_12DeclGroupRefE", "clang::BackendConsumer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN5clang15BackendConsumer21HandleTranslationUnitERNS_10ASTContextE", "clang::BackendConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang15BackendConsumer23HandleTagDeclDefinitionEPNS_7TagDeclE", "clang::BackendConsumer::HandleTagDeclDefinition(clang::TagDecl*)"},
    {"_ZN5clang15BackendConsumer27CompleteTentativeDefinitionEPNS_7VarDeclE", "clang::BackendConsumer::CompleteTentativeDefinition(clang::VarDecl*)"},
    {"_ZN5clang15BackendConsumer12HandleVTableEPNS_13CXXRecordDeclEb", "clang::BackendConsumer::HandleVTable(clang::CXXRecordDecl*, bool)"},
    {"_ZN5clang20PrettyStackTraceDeclD1Ev", "clang::PrettyStackTraceDecl::~PrettyStackTraceDecl()"},
    {"_ZN5clang20PrettyStackTraceDeclD0Ev", "clang::PrettyStackTraceDecl::~PrettyStackTraceDecl()"},
    {"_ZN5clang15BackendConsumer20InlineAsmDiagHandlerERKN4llvm12SMDiagnosticEPvj", "clang::BackendConsumer::InlineAsmDiagHandler(llvm::SMDiagnostic const&, void*, unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunctionC1ERNS0_13CodeGenModuleE", "clang::CodeGen::CodeGenFunction::CodeGenFunction(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen15CodeGenFunctionC2ERNS0_13CodeGenModuleE", "clang::CodeGen::CodeGenFunction::CodeGenFunction(clang::CodeGen::CodeGenModule&)"},
    {"_ZNK5clang7CodeGen15CodeGenFunction10getContextEv", "clang::CodeGen::CodeGenFunction::getContext() const"},
    {"_ZN5clang7CodeGen15CodeGenFunction17ConvertTypeForMemENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::ConvertTypeForMem(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11ConvertTypeENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::ConvertType(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20hasAggregateLLVMTypeENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::hasAggregateLLVMType(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction15EmitReturnBlockEv", "clang::CodeGen::CodeGenFunction::EmitReturnBlock()"},
    {"_ZN5clang7CodeGen15CodeGenFunction14FinishFunctionENS_14SourceLocationE", "clang::CodeGen::CodeGenFunction::FinishFunction(clang::SourceLocation)"},
    {"_ZN5clang7CodeGen15CodeGenFunction27EmitFunctionInstrumentationEPKc", "clang::CodeGen::CodeGenFunction::EmitFunctionInstrumentation(char const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction24ShouldInstrumentFunctionEv", "clang::CodeGen::CodeGenFunction::ShouldInstrumentFunction()"},
    {"_ZN5clang7CodeGen15CodeGenFunction13StartFunctionENS0_10GlobalDeclENS_8QualTypeEPN4llvm8FunctionERKNS4_11SmallVectorISt4pairIPKNS_7VarDeclES3_ELj16EEENS_14SourceLocationE", "clang::CodeGen::CodeGenFunction::StartFunction(clang::CodeGen::GlobalDecl, clang::QualType, llvm::Function*, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u> const&, clang::SourceLocation)"},
    {"_ZN5clang7CodeGen15CodeGenFunction11EmitVLASizeENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitVLASize(clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitFunctionBodyERN4llvm11SmallVectorISt4pairIPKNS_7VarDeclENS_8QualTypeEELj16EEE", "clang::CodeGen::CodeGenFunction::EmitFunctionBody(llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction12GenerateCodeENS0_10GlobalDeclEPN4llvm8FunctionE", "clang::CodeGen::CodeGenFunction::GenerateCode(clang::CodeGen::GlobalDecl, llvm::Function*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction13ContainsLabelEPKNS_4StmtEb", "clang::CodeGen::CodeGenFunction::ContainsLabel(clang::Stmt const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction28ConstantFoldsToSimpleIntegerEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::ConstantFoldsToSimpleInteger(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20EmitBranchOnBoolExprEPKNS_4ExprEPN4llvm10BasicBlockES7_", "clang::CodeGen::CodeGenFunction::EmitBranchOnBoolExpr(clang::Expr const*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16ErrorUnsupportedEPKNS_4StmtEPKcb", "clang::CodeGen::CodeGenFunction::ErrorUnsupported(clang::Stmt const*, char const*, bool)"},
    {"_ZN5clang7CodeGen15CodeGenFunction22EmitNullInitializationEPN4llvm5ValueENS_8QualTypeE", "clang::CodeGen::CodeGenFunction::EmitNullInitialization(llvm::Value*, clang::QualType)"},
    {"_ZN5clang7CodeGen15CodeGenFunction10GetVLASizeEPKNS_17VariableArrayTypeE", "clang::CodeGen::CodeGenFunction::GetVLASize(clang::VariableArrayType const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction14GetAddrOfLabelEPKNS_9LabelStmtE", "clang::CodeGen::CodeGenFunction::GetAddrOfLabel(clang::LabelStmt const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction20GetIndirectGotoBlockEv", "clang::CodeGen::CodeGenFunction::GetIndirectGotoBlock()"},
    {"_ZN5clang7CodeGen15CodeGenFunction13EmitVAListRefEPKNS_4ExprE", "clang::CodeGen::CodeGenFunction::EmitVAListRef(clang::Expr const*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction23EmitDeclRefExprDbgValueEPKNS_11DeclRefExprEPN4llvm8ConstantE", "clang::CodeGen::CodeGenFunction::EmitDeclRefExprDbgValue(clang::DeclRefExpr const*, llvm::Constant*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateMulEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Expr const*, llvm::Value*, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(clang::Expr const* const&, llvm::Value* const&, std::pair<clang::Expr const*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ExprEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Expr const*, llvm::Value*, llvm::DenseMapInfo<clang::Expr const*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen13CodeGenModuleC1ERNS_10ASTContextERKNS_14CodeGenOptionsERN4llvm6ModuleERKNS7_10DataLayoutERNS_10DiagnosticE", "clang::CodeGen::CodeGenModule::CodeGenModule(clang::ASTContext&, clang::CodeGenOptions const&, llvm::Module&, llvm::DataLayout const&, clang::Diagnostic&)"},
    {"_ZN5clang7CodeGen13CodeGenModuleC2ERNS_10ASTContextERKNS_14CodeGenOptionsERN4llvm6ModuleERKNS7_10DataLayoutERNS_10DiagnosticE", "clang::CodeGen::CodeGenModule::CodeGenModule(clang::ASTContext&, clang::CodeGenOptions const&, llvm::Module&, llvm::DataLayout const&, clang::Diagnostic&)"},
    {"_ZN5clang7CodeGen13CodeGenModuleD1Ev", "clang::CodeGen::CodeGenModule::~CodeGenModule()"},
    {"_ZN5clang7CodeGen13CodeGenModuleD2Ev", "clang::CodeGen::CodeGenModule::~CodeGenModule()"},
    {"_ZN5clang7CodeGen13CodeGenModule17createObjCRuntimeEv", "clang::CodeGen::CodeGenModule::createObjCRuntime()"},
    {"_ZN5clang7CodeGen13CodeGenModule7ReleaseEv", "clang::CodeGen::CodeGenModule::Release()"},
    {"_ZN5clang7CodeGen13CodeGenModule12EmitDeferredEv", "clang::CodeGen::CodeGenModule::EmitDeferred()"},
    {"_ZN5clang7CodeGen13CodeGenModule13AddGlobalCtorEPN4llvm8FunctionEi", "clang::CodeGen::CodeGenModule::AddGlobalCtor(llvm::Function*, int)"},
    {"_ZN5clang7CodeGen13CodeGenModule12EmitCtorListERKSt6vectorISt4pairIPN4llvm8ConstantEiESaIS7_EEPKc", "clang::CodeGen::CodeGenModule::EmitCtorList(std::vector<std::pair<llvm::Constant*, int>, std::allocator<std::pair<llvm::Constant*, int> > > const&, char const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule15EmitAnnotationsEv", "clang::CodeGen::CodeGenModule::EmitAnnotations()"},
    {"_ZN5clang7CodeGen13CodeGenModule12EmitLLVMUsedEv", "clang::CodeGen::CodeGenModule::EmitLLVMUsed()"},
    {"_ZN5clang7CodeGen13CodeGenModule16EmitDeclMetadataEv", "clang::CodeGen::CodeGenModule::EmitDeclMetadata()"},
    {"_ZN5clang7CodeGen13CodeGenModule11getTBAAInfoENS_8QualTypeE", "clang::CodeGen::CodeGenModule::getTBAAInfo(clang::QualType)"},
    {"_ZN5clang7CodeGen13CodeGenModule19DecorateInstructionEPN4llvm11InstructionEPNS2_6MDNodeE", "clang::CodeGen::CodeGenModule::DecorateInstruction(llvm::Instruction*, llvm::MDNode*)"},
    {"_ZNK5clang7CodeGen13CodeGenModule14isTargetDarwinEv", "clang::CodeGen::CodeGenModule::isTargetDarwin() const"},
    {"_ZN5clang7CodeGen13CodeGenModule16ErrorUnsupportedEPKNS_4StmtEPKcb", "clang::CodeGen::CodeGenModule::ErrorUnsupported(clang::Stmt const*, char const*, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule16ErrorUnsupportedEPKNS_4DeclEPKcb", "clang::CodeGen::CodeGenModule::ErrorUnsupported(clang::Decl const*, char const*, bool)"},
    {"_ZNK5clang7CodeGen13CodeGenModule19setGlobalVisibilityEPN4llvm11GlobalValueEPKNS_9NamedDeclE", "clang::CodeGen::CodeGenModule::setGlobalVisibility(llvm::GlobalValue*, clang::NamedDecl const*) const"},
    {"_ZNK5clang7CodeGen13CodeGenModule17setTypeVisibilityEPN4llvm11GlobalValueEPKNS_13CXXRecordDeclENS1_18TypeVisibilityKindE", "clang::CodeGen::CodeGenModule::setTypeVisibility(llvm::GlobalValue*, clang::CXXRecordDecl const*, clang::CodeGen::CodeGenModule::TypeVisibilityKind) const"},
    {"_ZN5clang7CodeGen13CodeGenModule14getMangledNameENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::getMangledName(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule19getBlockMangledNameENS0_10GlobalDeclERNS_12MangleBufferEPKNS_9BlockDeclE", "clang::CodeGen::CodeGenModule::getBlockMangledName(clang::CodeGen::GlobalDecl, clang::MangleBuffer&, clang::BlockDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule14GetGlobalValueEN4llvm9StringRefE", "clang::CodeGen::CodeGenModule::GetGlobalValue(llvm::StringRef)"},
    {"_ZN5clang7CodeGen13CodeGenModule13AddGlobalDtorEPN4llvm8FunctionEi", "clang::CodeGen::CodeGenModule::AddGlobalDtor(llvm::Function*, int)"},
    {"_ZN5clang7CodeGen13CodeGenModule18getFunctionLinkageEPKNS_12FunctionDeclE", "clang::CodeGen::CodeGenModule::getFunctionLinkage(clang::FunctionDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule31SetFunctionDefinitionAttributesEPKNS_12FunctionDeclEPN4llvm11GlobalValueE", "clang::CodeGen::CodeGenModule::SetFunctionDefinitionAttributes(clang::FunctionDecl const*, llvm::GlobalValue*)"},
    {"_ZN5clang7CodeGen13CodeGenModule19SetCommonAttributesEPKNS_4DeclEPN4llvm11GlobalValueE", "clang::CodeGen::CodeGenModule::SetCommonAttributes(clang::Decl const*, llvm::GlobalValue*)"},
    {"_ZN5clang7CodeGen13CodeGenModule25SetLLVMFunctionAttributesEPKNS_4DeclERKNS0_14CGFunctionInfoEPN4llvm8FunctionE", "clang::CodeGen::CodeGenModule::SetLLVMFunctionAttributes(clang::Decl const*, clang::CodeGen::CGFunctionInfo const&, llvm::Function*)"},
    {"_ZN5clang7CodeGen13CodeGenModule38SetLLVMFunctionAttributesForDefinitionEPKNS_4DeclEPN4llvm8FunctionE", "clang::CodeGen::CodeGenModule::SetLLVMFunctionAttributesForDefinition(clang::Decl const*, llvm::Function*)"},
    {"_ZN5clang7CodeGen13CodeGenModule13AddUsedGlobalEPN4llvm11GlobalValueE", "clang::CodeGen::CodeGenModule::AddUsedGlobal(llvm::GlobalValue*)"},
    {"_ZN5clang7CodeGen13CodeGenModule29SetInternalFunctionAttributesEPKNS_4DeclEPN4llvm8FunctionERKNS0_14CGFunctionInfoE", "clang::CodeGen::CodeGenModule::SetInternalFunctionAttributes(clang::Decl const*, llvm::Function*, clang::CodeGen::CGFunctionInfo const&)"},
    {"_ZN5clang7CodeGen13CodeGenModule21SetFunctionAttributesENS0_10GlobalDeclEPN4llvm8FunctionEb", "clang::CodeGen::CodeGenModule::SetFunctionAttributes(clang::CodeGen::GlobalDecl, llvm::Function*, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule16getVTableLinkageEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenModule::getVTableLinkage(clang::CXXRecordDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule20EmitGlobalDefinitionENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::EmitGlobalDefinition(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule16EmitAnnotateAttrEPN4llvm11GlobalValueEPKNS_12AnnotateAttrEj", "clang::CodeGen::CodeGenModule::EmitAnnotateAttr(llvm::GlobalValue*, clang::AnnotateAttr const*, unsigned int)"},
    {"_ZN5clang7CodeGen13CodeGenModule18MayDeferGenerationEPKNS_9ValueDeclE", "clang::CodeGen::CodeGenModule::MayDeferGeneration(clang::ValueDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule19GetWeakRefReferenceEPKNS_9ValueDeclE", "clang::CodeGen::CodeGenModule::GetWeakRefReference(clang::ValueDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule23GetOrCreateLLVMFunctionEN4llvm9StringRefEPKNS2_4TypeENS0_10GlobalDeclEb", "clang::CodeGen::CodeGenModule::GetOrCreateLLVMFunction(llvm::StringRef, llvm::Type const*, clang::CodeGen::GlobalDecl, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule21GetOrCreateLLVMGlobalEN4llvm9StringRefEPKNS2_11PointerTypeEPKNS_7VarDeclEb", "clang::CodeGen::CodeGenModule::GetOrCreateLLVMGlobal(llvm::StringRef, llvm::PointerType const*, clang::VarDecl const*, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule10EmitGlobalENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::EmitGlobal(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule19EmitAliasDefinitionENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::EmitAliasDefinition(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule28EmitGlobalFunctionDefinitionENS0_10GlobalDeclE", "clang::CodeGen::CodeGenModule::EmitGlobalFunctionDefinition(clang::CodeGen::GlobalDecl)"},
    {"_ZN5clang7CodeGen13CodeGenModule23EmitGlobalVarDefinitionEPKNS_7VarDeclE", "clang::CodeGen::CodeGenModule::EmitGlobalVarDefinition(clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule17GetAddrOfFunctionENS0_10GlobalDeclEPKN4llvm4TypeEb", "clang::CodeGen::CodeGenModule::GetAddrOfFunction(clang::CodeGen::GlobalDecl, llvm::Type const*, bool)"},
    {"_ZN5clang7CodeGen13CodeGenModule21CreateRuntimeFunctionEPKN4llvm12FunctionTypeENS2_9StringRefE", "clang::CodeGen::CodeGenModule::CreateRuntimeFunction(llvm::FunctionType const*, llvm::StringRef)"},
    {"_Z20DeclIsConstantGlobalRN5clang10ASTContextEPKNS_7VarDeclE", "DeclIsConstantGlobal(clang::ASTContext&, clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule33CreateOrReplaceCXXRuntimeVariableEN4llvm9StringRefEPKNS2_4TypeENS2_11GlobalValue12LinkageTypesE", "clang::CodeGen::CodeGenModule::CreateOrReplaceCXXRuntimeVariable(llvm::StringRef, llvm::Type const*, llvm::GlobalValue::LinkageTypes)"},
    {"_ZN5clang7CodeGen13CodeGenModule18GetAddrOfGlobalVarEPKNS_7VarDeclEPKN4llvm4TypeE", "clang::CodeGen::CodeGenModule::GetAddrOfGlobalVar(clang::VarDecl const*, llvm::Type const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule21CreateRuntimeVariableEPKN4llvm4TypeENS2_9StringRefE", "clang::CodeGen::CodeGenModule::CreateRuntimeVariable(llvm::Type const*, llvm::StringRef)"},
    {"_ZN5clang7CodeGen13CodeGenModule23EmitTentativeDefinitionEPKNS_7VarDeclE", "clang::CodeGen::CodeGenModule::EmitTentativeDefinition(clang::VarDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule10EmitVTableEPNS_13CXXRecordDeclEb", "clang::CodeGen::CodeGenModule::EmitVTable(clang::CXXRecordDecl*, bool)"},
    {"_ZNK5clang7CodeGen13CodeGenModule22GetTargetTypeStoreSizeEPKN4llvm4TypeE", "clang::CodeGen::CodeGenModule::GetTargetTypeStoreSize(llvm::Type const*) const"},
    {"_ZN5clang7CodeGen13CodeGenModule27GetLLVMLinkageVarDefinitionEPKNS_7VarDeclEPN4llvm14GlobalVariableE", "clang::CodeGen::CodeGenModule::GetLLVMLinkageVarDefinition(clang::VarDecl const*, llvm::GlobalVariable*)"},
    {"_ZN5clang7CodeGen13CodeGenModule21getBuiltinLibFunctionEPKNS_12FunctionDeclEj", "clang::CodeGen::CodeGenModule::getBuiltinLibFunction(clang::FunctionDecl const*, unsigned int)"},
    {"_ZN5clang7CodeGen13CodeGenModule12getIntrinsicEjPPKN4llvm4TypeEj", "clang::CodeGen::CodeGenModule::getIntrinsic(unsigned int, llvm::Type const**, unsigned int)"},
    {"_ZN5clang7CodeGen13CodeGenModule25GetAddrOfConstantCFStringEPKNS_13StringLiteralE", "clang::CodeGen::CodeGenModule::GetAddrOfConstantCFString(clang::StringLiteral const*)"},
    {"_Z24GetConstantCFStringEntryRN4llvm9StringMapIPNS_8ConstantENS_15MallocAllocatorEEEPKN5clang13StringLiteralEbRbRj", "GetConstantCFStringEntry(llvm::StringMap<llvm::Constant*, llvm::MallocAllocator>&, clang::StringLiteral const*, bool, bool&, unsigned int&)"},
    {"_ZN5clang7CodeGen13CodeGenModule23GetAddrOfConstantStringEPKNS_13StringLiteralE", "clang::CodeGen::CodeGenModule::GetAddrOfConstantString(clang::StringLiteral const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule25GetStringForStringLiteralEPKNS_13StringLiteralE", "clang::CodeGen::CodeGenModule::GetStringForStringLiteral(clang::StringLiteral const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule34GetAddrOfConstantStringFromLiteralEPKNS_13StringLiteralE", "clang::CodeGen::CodeGenModule::GetAddrOfConstantStringFromLiteral(clang::StringLiteral const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule23GetAddrOfConstantStringERKSsPKc", "clang::CodeGen::CodeGenModule::GetAddrOfConstantString(std::string const&, char const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule37GetAddrOfConstantStringFromObjCEncodeEPKNS_14ObjCEncodeExprE", "clang::CodeGen::CodeGenModule::GetAddrOfConstantStringFromObjCEncode(clang::ObjCEncodeExpr const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule24GetAddrOfConstantCStringERKSsPKc", "clang::CodeGen::CodeGenModule::GetAddrOfConstantCString(std::string const&, char const*)"},
    {"_Z21GenerateStringLiteralRKSsbRN5clang7CodeGen13CodeGenModuleEPKc", "GenerateStringLiteral(std::string const&, bool, clang::CodeGen::CodeGenModule&, char const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule31EmitObjCPropertyImplementationsEPKNS_22ObjCImplementationDeclE", "clang::CodeGen::CodeGenModule::EmitObjCPropertyImplementations(clang::ObjCImplementationDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule27EmitObjCIvarInitializationsEPNS_22ObjCImplementationDeclE", "clang::CodeGen::CodeGenModule::EmitObjCIvarInitializations(clang::ObjCImplementationDecl*)"},
    {"_ZN5clang7CodeGen13CodeGenModule13EmitNamespaceEPKNS_13NamespaceDeclE", "clang::CodeGen::CodeGenModule::EmitNamespace(clang::NamespaceDecl const*)"},
    {"_ZN5clang7CodeGen13CodeGenModule16EmitTopLevelDeclEPNS_4DeclE", "clang::CodeGen::CodeGenModule::EmitTopLevelDecl(clang::Decl*)"},
    {"_ZN5clang7CodeGen13CodeGenModule15EmitLinkageSpecEPKNS_15LinkageSpecDeclE", "clang::CodeGen::CodeGenModule::EmitLinkageSpec(clang::LinkageSpecDecl const*)"},
    {"_Z22EmitGlobalDeclMetadataRN5clang7CodeGen13CodeGenModuleERPN4llvm11NamedMDNodeENS0_10GlobalDeclEPNS3_11GlobalValueE", "EmitGlobalDeclMetadata(clang::CodeGen::CodeGenModule&, llvm::NamedMDNode*&, clang::CodeGen::GlobalDecl, llvm::GlobalValue*)"},
    {"_ZN5clang7CodeGen15CodeGenFunction16EmitDeclMetadataEv", "clang::CodeGen::CodeGenFunction::EmitDeclMetadata()"},
    {"_ZN5clang7CodeGen13CodeGenModule21getBlockObjectDisposeEv", "clang::CodeGen::CodeGenModule::getBlockObjectDispose()"},
    {"_ZN5clang7CodeGen13CodeGenModule20getBlockObjectAssignEv", "clang::CodeGen::CodeGenModule::getBlockObjectAssign()"},
    {"_ZN5clang7CodeGen13CodeGenModule24getNSConcreteGlobalBlockEv", "clang::CodeGen::CodeGenModule::getNSConcreteGlobalBlock()"},
    {"_ZN5clang7CodeGen13CodeGenModule23getNSConcreteStackBlockEv", "clang::CodeGen::CodeGenModule::getNSConcreteStackBlock()"},
    {"_ZNK5clang4Decl7getAttrINS_11SectionAttrEEEPT_v", "clang::SectionAttr* clang::Decl::getAttr<clang::SectionAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_9AliasAttrEEEPT_v", "clang::AliasAttr* clang::Decl::getAttr<clang::AliasAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_12AnnotateAttrEEEPT_v", "clang::AnnotateAttr* clang::Decl::getAttr<clang::AnnotateAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_12NoCommonAttrEEEPT_v", "clang::NoCommonAttr* clang::Decl::getAttr<clang::NoCommonAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_10CommonAttrEEEPT_v", "clang::CommonAttr* clang::Decl::getAttr<clang::CommonAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_15ConstructorAttrEEEPT_v", "clang::ConstructorAttr* clang::Decl::getAttr<clang::ConstructorAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_14DestructorAttrEEEPT_v", "clang::DestructorAttr* clang::Decl::getAttr<clang::DestructorAttr>() const"},
    {"_ZN4llvm9StringMapIPNS_8ConstantENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::Constant*>& llvm::StringMap<llvm::Constant*, llvm::MallocAllocator>::GetOrCreateValue<llvm::Constant*>(llvm::StringRef, llvm::Constant*)"},
    {"_ZN4llvm9StringMapIN5clang7CodeGen10GlobalDeclENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<clang::CodeGen::GlobalDecl>& llvm::StringMap<clang::CodeGen::GlobalDecl, llvm::MallocAllocator>::GetOrCreateValue<clang::CodeGen::GlobalDecl>(llvm::StringRef, clang::CodeGen::GlobalDecl)"},
    {"_ZNSt6vectorIN5clang7CodeGen10GlobalDeclESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::CodeGen::GlobalDecl, std::allocator<clang::CodeGen::GlobalDecl> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CodeGen::GlobalDecl*, std::vector<clang::CodeGen::GlobalDecl, std::allocator<clang::CodeGen::GlobalDecl> > >, clang::CodeGen::GlobalDecl const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::Decl const*, unsigned int, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::Decl const* const&, unsigned int const&, std::pair<clang::Decl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::Decl const*, unsigned int, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIPN4llvm8ConstantEiESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::Constant*, int>, std::allocator<std::pair<llvm::Constant*, int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, int>*, std::vector<std::pair<llvm::Constant*, int>, std::allocator<std::pair<llvm::Constant*, int> > > >, std::pair<llvm::Constant*, int> const&)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen10GlobalDeclENS_9StringRefENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<clang::CodeGen::GlobalDecl, llvm::StringRef, llvm::DenseMapInfo<clang::CodeGen::GlobalDecl>, llvm::DenseMapInfo<llvm::StringRef> >::InsertIntoBucket(clang::CodeGen::GlobalDecl const&, llvm::StringRef const&, std::pair<clang::CodeGen::GlobalDecl, llvm::StringRef>*)"},
    {"_ZN4llvm8DenseMapIN5clang7CodeGen10GlobalDeclENS_9StringRefENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::CodeGen::GlobalDecl, llvm::StringRef, llvm::DenseMapInfo<clang::CodeGen::GlobalDecl>, llvm::DenseMapInfo<llvm::StringRef> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen15CodeGenFunctionD2Ev", "clang::CodeGen::CodeGenFunction::~CodeGenFunction()"},
    {"_ZN5clang7CodeGen14CodeGenVTablesD2Ev", "clang::CodeGen::CodeGenVTables::~CodeGenVTables()"},
    {"_ZN5clang7CodeGen14CodeGenVTablesC2ERNS0_13CodeGenModuleE", "clang::CodeGen::CodeGenVTables::CodeGenVTables(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen11CodeGenTBAAC1ERNS_10ASTContextERN4llvm11LLVMContextERKNS_11LangOptionsERNS_13MangleContextE", "clang::CodeGen::CodeGenTBAA::CodeGenTBAA(clang::ASTContext&, llvm::LLVMContext&, clang::LangOptions const&, clang::MangleContext&)"},
    {"_ZN5clang7CodeGen11CodeGenTBAAC2ERNS_10ASTContextERN4llvm11LLVMContextERKNS_11LangOptionsERNS_13MangleContextE", "clang::CodeGen::CodeGenTBAA::CodeGenTBAA(clang::ASTContext&, llvm::LLVMContext&, clang::LangOptions const&, clang::MangleContext&)"},
    {"_ZN5clang7CodeGen11CodeGenTBAAD1Ev", "clang::CodeGen::CodeGenTBAA::~CodeGenTBAA()"},
    {"_ZN5clang7CodeGen11CodeGenTBAAD2Ev", "clang::CodeGen::CodeGenTBAA::~CodeGenTBAA()"},
    {"_ZN5clang7CodeGen11CodeGenTBAA7getRootEv", "clang::CodeGen::CodeGenTBAA::getRoot()"},
    {"_ZN5clang7CodeGen11CodeGenTBAA23getTBAAInfoForNamedTypeEN4llvm9StringRefEPNS2_6MDNodeEb", "clang::CodeGen::CodeGenTBAA::getTBAAInfoForNamedType(llvm::StringRef, llvm::MDNode*, bool)"},
    {"_ZN5clang7CodeGen11CodeGenTBAA7getCharEv", "clang::CodeGen::CodeGenTBAA::getChar()"},
    {"_ZN5clang7CodeGen11CodeGenTBAA11getTBAAInfoENS_8QualTypeE", "clang::CodeGen::CodeGenTBAA::getTBAAInfo(clang::QualType)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEPNS_6MDNodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Type const*, llvm::MDNode*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::MDNode*> >::InsertIntoBucket(clang::Type const* const&, llvm::MDNode* const&, std::pair<clang::Type const*, llvm::MDNode*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEPNS_6MDNodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Type const*, llvm::MDNode*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::MDNode*> >::grow(unsigned int)"},
    {"_ZN5clang7CodeGen12CodeGenTypesC1ERNS_10ASTContextERN4llvm6ModuleERKNS4_10DataLayoutERKNS_7ABIInfoERNS0_8CGCXXABIE", "clang::CodeGen::CodeGenTypes::CodeGenTypes(clang::ASTContext&, llvm::Module&, llvm::DataLayout const&, clang::ABIInfo const&, clang::CodeGen::CGCXXABI&)"},
    {"_ZN5clang7CodeGen12CodeGenTypesC2ERNS_10ASTContextERN4llvm6ModuleERKNS4_10DataLayoutERKNS_7ABIInfoERNS0_8CGCXXABIE", "clang::CodeGen::CodeGenTypes::CodeGenTypes(clang::ASTContext&, llvm::Module&, llvm::DataLayout const&, clang::ABIInfo const&, clang::CodeGen::CGCXXABI&)"},
    {"_ZN5clang7CodeGen12CodeGenTypesD1Ev", "clang::CodeGen::CodeGenTypes::~CodeGenTypes()"},
    {"_ZN5clang7CodeGen12CodeGenTypesD2Ev", "clang::CodeGen::CodeGenTypes::~CodeGenTypes()"},
    {"_ZN5clang7CodeGen12CodeGenTypes26HandleLateResolvedPointersEv", "clang::CodeGen::CodeGenTypes::HandleLateResolvedPointers()"},
    {"_ZN5clang7CodeGen12CodeGenTypes11ConvertTypeENS_8QualTypeEb", "clang::CodeGen::CodeGenTypes::ConvertType(clang::QualType, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes20ConvertTypeRecursiveENS_8QualTypeE", "clang::CodeGen::CodeGenTypes::ConvertTypeRecursive(clang::QualType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes14ConvertNewTypeENS_8QualTypeE", "clang::CodeGen::CodeGenTypes::ConvertNewType(clang::QualType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes17ConvertTypeForMemENS_8QualTypeEb", "clang::CodeGen::CodeGenTypes::ConvertTypeForMem(clang::QualType, bool)"},
    {"_ZN5clang7CodeGen12CodeGenTypes22VerifyFuncTypeCompleteEPKNS_4TypeE", "clang::CodeGen::CodeGenTypes::VerifyFuncTypeComplete(clang::Type const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes19UpdateCompletedTypeEPKNS_7TagDeclE", "clang::CodeGen::CodeGenTypes::UpdateCompletedType(clang::TagDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes18ConvertTagDeclTypeEPKNS_7TagDeclE", "clang::CodeGen::CodeGenTypes::ConvertTagDeclType(clang::TagDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes17getCGRecordLayoutEPKNS_10RecordDeclE", "clang::CodeGen::CodeGenTypes::getCGRecordLayout(clang::RecordDecl const*)"},
    {"_ZN5clang7CodeGen12CodeGenTypes19isZeroInitializableENS_8QualTypeE", "clang::CodeGen::CodeGenTypes::isZeroInitializable(clang::QualType)"},
    {"_ZN5clang7CodeGen12CodeGenTypes19isZeroInitializableEPKNS_13CXXRecordDeclE", "clang::CodeGen::CodeGenTypes::isZeroInitializable(clang::CXXRecordDecl const*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeENS_12PATypeHolderENS_12DenseMapInfoIS4_EENS6_IS5_EEED1Ev", "llvm::DenseMap<clang::Type const*, llvm::PATypeHolder, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::PATypeHolder> >::~DenseMap()"},
    {"_ZN4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEED1Ev", "llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>::~FoldingSet()"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeENS_12PATypeHolderENS_12DenseMapInfoIS4_EENS6_IS5_EEE6insertERKSt4pairIS4_S5_E", "llvm::DenseMap<clang::Type const*, llvm::PATypeHolder, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::PATypeHolder> >::insert(std::pair<clang::Type const*, llvm::PATypeHolder> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEPNS1_7CodeGen14CGRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::Type const*, clang::CodeGen::CGRecordLayout*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<clang::CodeGen::CGRecordLayout*> >::InsertIntoBucket(clang::Type const* const&, clang::CodeGen::CGRecordLayout* const&, std::pair<clang::Type const*, clang::CodeGen::CGRecordLayout*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeEPNS1_7CodeGen14CGRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::Type const*, clang::CodeGen::CGRecordLayout*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<clang::CodeGen::CGRecordLayout*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang17ObjCInterfaceTypeEPKNS_4TypeENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::ObjCInterfaceType const*, llvm::Type const*, llvm::DenseMapInfo<clang::ObjCInterfaceType const*>, llvm::DenseMapInfo<llvm::Type const*> >::InsertIntoBucket(clang::ObjCInterfaceType const* const&, llvm::Type const* const&, std::pair<clang::ObjCInterfaceType const*, llvm::Type const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang17ObjCInterfaceTypeEPKNS_4TypeENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::ObjCInterfaceType const*, llvm::Type const*, llvm::DenseMapInfo<clang::ObjCInterfaceType const*>, llvm::DenseMapInfo<llvm::Type const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeENS_12PATypeHolderENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::Type const*, llvm::PATypeHolder, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::PATypeHolder> >::InsertIntoBucket(clang::Type const* const&, llvm::PATypeHolder const&, std::pair<clang::Type const*, llvm::PATypeHolder>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeENS_12PATypeHolderENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Type const*, llvm::PATypeHolder, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<llvm::PATypeHolder> >::grow(unsigned int)"},
    {"_ZN4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEED0Ev", "llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang7CodeGen14CGFunctionInfo7ProfileERN4llvm16FoldingSetNodeIDE", "clang::CodeGen::CGFunctionInfo::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang7CodeGen14CGRecordLayoutD2Ev", "clang::CodeGen::CGRecordLayout::~CGRecordLayout()"},
    {"_ZN5clang7CodeGen19CreateItaniumCXXABIERNS0_13CodeGenModuleE", "clang::CodeGen::CreateItaniumCXXABI(clang::CodeGen::CodeGenModule&)"},
    {"_ZN5clang7CodeGen15CreateARMCXXABIERNS0_13CodeGenModuleE", "clang::CodeGen::CreateARMCXXABI(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABID1Ev", "(anonymous namespace)::ARMCXXABI::~ARMCXXABI()"},
    {"_ZN12_GLOBAL__N_19ARMCXXABID0Ev", "(anonymous namespace)::ARMCXXABI::~ARMCXXABI()"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI24ConvertMemberPointerTypeEPKN5clang17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::ConvertMemberPointerType(clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI31EmitLoadOfMemberFunctionPointerERN5clang7CodeGen15CodeGenFunctionERPN4llvm5ValueES7_PKNS1_17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::EmitLoadOfMemberFunctionPointer(clang::CodeGen::CodeGenFunction&, llvm::Value*&, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI28EmitMemberDataPointerAddressERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_PKNS1_17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberDataPointerAddress(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI27EmitMemberPointerConversionERN5clang7CodeGen15CodeGenFunctionEPKNS1_8CastExprEPN4llvm5ValueE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberPointerConversion(clang::CodeGen::CodeGenFunction&, clang::CastExpr const*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI27EmitMemberPointerConversionEPN4llvm8ConstantEPKN5clang8CastExprE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberPointerConversion(llvm::Constant*, clang::CastExpr const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI19isZeroInitializableEPKN5clang17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::isZeroInitializable(clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI21EmitNullMemberPointerEPKN5clang17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::EmitNullMemberPointer(clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI17EmitMemberPointerEPKN5clang13CXXMethodDeclE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberPointer(clang::CXXMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI21EmitMemberDataPointerEPKN5clang17MemberPointerTypeENS1_9CharUnitsE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberDataPointer(clang::MemberPointerType const*, clang::CharUnits)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI27EmitMemberPointerComparisonERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_PKNS1_17MemberPointerTypeEb", "(anonymous namespace)::ItaniumCXXABI::EmitMemberPointerComparison(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::MemberPointerType const*, bool)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI26EmitMemberPointerIsNotNullERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueEPKNS1_17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::EmitMemberPointerIsNotNull(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::MemberPointerType const*)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI25BuildConstructorSignatureEPKN5clang18CXXConstructorDeclENS1_11CXXCtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::ARMCXXABI::BuildConstructorSignature(clang::CXXConstructorDecl const*, clang::CXXCtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI24BuildDestructorSignatureEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::ARMCXXABI::BuildDestructorSignature(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI27BuildInstanceFunctionParamsERN5clang7CodeGen15CodeGenFunctionERNS1_8QualTypeERN4llvm11SmallVectorISt4pairIPKNS1_7VarDeclES5_ELj16EEE", "(anonymous namespace)::ARMCXXABI::BuildInstanceFunctionParams(clang::CodeGen::CodeGenFunction&, clang::QualType&, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI26EmitInstanceFunctionPrologERN5clang7CodeGen15CodeGenFunctionE", "(anonymous namespace)::ARMCXXABI::EmitInstanceFunctionProlog(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI19EmitReturnFromThunkERN5clang7CodeGen15CodeGenFunctionENS2_6RValueENS1_8QualTypeE", "(anonymous namespace)::ARMCXXABI::EmitReturnFromThunk(clang::CodeGen::CodeGenFunction&, clang::CodeGen::RValue, clang::QualType)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI18GetArrayCookieSizeEPKN5clang10CXXNewExprE", "(anonymous namespace)::ARMCXXABI::GetArrayCookieSize(clang::CXXNewExpr const*)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI21InitializeArrayCookieERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_PKNS1_10CXXNewExprENS1_8QualTypeE", "(anonymous namespace)::ARMCXXABI::InitializeArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::CXXNewExpr const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABI15ReadArrayCookieERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueEPKNS1_13CXXDeleteExprENS1_8QualTypeERS7_SC_RNS1_9CharUnitsE", "(anonymous namespace)::ARMCXXABI::ReadArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::CXXDeleteExpr const*, clang::QualType, llvm::Value*&, llvm::Value*&, clang::CharUnits&)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI15EmitGuardedInitERN5clang7CodeGen15CodeGenFunctionERKNS1_7VarDeclEPN4llvm14GlobalVariableE", "(anonymous namespace)::ItaniumCXXABI::EmitGuardedInit(clang::CodeGen::CodeGenFunction&, clang::VarDecl const&, llvm::GlobalVariable*)"},
    {"_ZN12_GLOBAL__N_114CallGuardAbortD1Ev", "(anonymous namespace)::CallGuardAbort::~CallGuardAbort()"},
    {"_ZN12_GLOBAL__N_114CallGuardAbortD0Ev", "(anonymous namespace)::CallGuardAbort::~CallGuardAbort()"},
    {"_ZN12_GLOBAL__N_114CallGuardAbort4EmitERN5clang7CodeGen15CodeGenFunctionEb", "(anonymous namespace)::CallGuardAbort::Emit(clang::CodeGen::CodeGenFunction&, bool)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI26EmitInstanceFunctionPrologERN5clang7CodeGen15CodeGenFunctionE", "(anonymous namespace)::ItaniumCXXABI::EmitInstanceFunctionProlog(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI27BuildInstanceFunctionParamsERN5clang7CodeGen15CodeGenFunctionERNS1_8QualTypeERN4llvm11SmallVectorISt4pairIPKNS1_7VarDeclES5_ELj16EEE", "(anonymous namespace)::ItaniumCXXABI::BuildInstanceFunctionParams(clang::CodeGen::CodeGenFunction&, clang::QualType&, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI24BuildDestructorSignatureEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::ItaniumCXXABI::BuildDestructorSignature(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI25BuildConstructorSignatureEPKN5clang18CXXConstructorDeclENS1_11CXXCtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::ItaniumCXXABI::BuildConstructorSignature(clang::CXXConstructorDecl const*, clang::CXXCtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABID1Ev", "(anonymous namespace)::ItaniumCXXABI::~ItaniumCXXABI()"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABID0Ev", "(anonymous namespace)::ItaniumCXXABI::~ItaniumCXXABI()"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI18GetArrayCookieSizeEPKN5clang10CXXNewExprE", "(anonymous namespace)::ItaniumCXXABI::GetArrayCookieSize(clang::CXXNewExpr const*)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI21InitializeArrayCookieERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueES7_PKNS1_10CXXNewExprENS1_8QualTypeE", "(anonymous namespace)::ItaniumCXXABI::InitializeArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, llvm::Value*, clang::CXXNewExpr const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABI15ReadArrayCookieERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueEPKNS1_13CXXDeleteExprENS1_8QualTypeERS7_SC_RNS1_9CharUnitsE", "(anonymous namespace)::ItaniumCXXABI::ReadArrayCookie(clang::CodeGen::CodeGenFunction&, llvm::Value*, clang::CXXDeleteExpr const*, clang::QualType, llvm::Value*&, llvm::Value*&, clang::CharUnits&)"},
    {"_ZN5clang7CodeGen15CodeGenFunction17GetAddrOfLocalVarEPKNS_7VarDeclE", "clang::CodeGen::CodeGenFunction::GetAddrOfLocalVar(clang::VarDecl const*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE8CreateOrEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateOr(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateBinOpENS_11Instruction9BinaryOpsEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateBinOp(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateNSWSubEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateNSWSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateNSWAddEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateNSWAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateSelectEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateSelect(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE17CreateInsertValueEPNS_5ValueES6_jRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateInsertValue(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateAShrEPNS_5ValueES6_RKNS_5TwineEb", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateAShr(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateSubEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN5clang7CodeGen21CreateMicrosoftCXXABIERNS0_13CodeGenModuleE", "clang::CodeGen::CreateMicrosoftCXXABI(clang::CodeGen::CodeGenModule&)"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABID1Ev", "(anonymous namespace)::MicrosoftCXXABI::~MicrosoftCXXABI()"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABID0Ev", "(anonymous namespace)::MicrosoftCXXABI::~MicrosoftCXXABI()"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABI25BuildConstructorSignatureEPKN5clang18CXXConstructorDeclENS1_11CXXCtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::MicrosoftCXXABI::BuildConstructorSignature(clang::CXXConstructorDecl const*, clang::CXXCtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABI24BuildDestructorSignatureEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERNS1_7CanQualINS1_4TypeEEERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::MicrosoftCXXABI::BuildDestructorSignature(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::CanQual<clang::Type>&, llvm::SmallVectorImpl<clang::CanQual<clang::Type> >&)"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABI27BuildInstanceFunctionParamsERN5clang7CodeGen15CodeGenFunctionERNS1_8QualTypeERN4llvm11SmallVectorISt4pairIPKNS1_7VarDeclES5_ELj16EEE", "(anonymous namespace)::MicrosoftCXXABI::BuildInstanceFunctionParams(clang::CodeGen::CodeGenFunction&, clang::QualType&, llvm::SmallVector<std::pair<clang::VarDecl const*, clang::QualType>, 16u>&)"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABI26EmitInstanceFunctionPrologERN5clang7CodeGen15CodeGenFunctionE", "(anonymous namespace)::MicrosoftCXXABI::EmitInstanceFunctionProlog(clang::CodeGen::CodeGenFunction&)"},
    {"_ZN5clang17CreateLLVMCodeGenERNS_10DiagnosticERKSsRKNS_14CodeGenOptionsERN4llvm11LLVMContextE", "clang::CreateLLVMCodeGen(clang::Diagnostic&, std::string const&, clang::CodeGenOptions const&, llvm::LLVMContext&)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImplD1Ev", "(anonymous namespace)::CodeGeneratorImpl::~CodeGeneratorImpl()"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImplD0Ev", "(anonymous namespace)::CodeGeneratorImpl::~CodeGeneratorImpl()"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl10InitializeERN5clang10ASTContextE", "(anonymous namespace)::CodeGeneratorImpl::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl18HandleTopLevelDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::CodeGeneratorImpl::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::CodeGeneratorImpl::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl23HandleTagDeclDefinitionEPN5clang7TagDeclE", "(anonymous namespace)::CodeGeneratorImpl::HandleTagDeclDefinition(clang::TagDecl*)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl27CompleteTentativeDefinitionEPN5clang7VarDeclE", "(anonymous namespace)::CodeGeneratorImpl::CompleteTentativeDefinition(clang::VarDecl*)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl12HandleVTableEPN5clang13CXXRecordDeclEb", "(anonymous namespace)::CodeGeneratorImpl::HandleVTable(clang::CXXRecordDecl*, bool)"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl9GetModuleEv", "(anonymous namespace)::CodeGeneratorImpl::GetModule()"},
    {"_ZN12_GLOBAL__N_117CodeGeneratorImpl13ReleaseModuleEv", "(anonymous namespace)::CodeGeneratorImpl::ReleaseModule()"},
    {"_ZN5clang13CodeGeneratorD1Ev", "clang::CodeGenerator::~CodeGenerator()"},
    {"_ZN5clang13CodeGeneratorD0Ev", "clang::CodeGenerator::~CodeGenerator()"},
    {"_ZN5clang7ABIInfoD0Ev", "clang::ABIInfo::~ABIInfo()"},
    {"_ZN5clang7ABIInfoD1Ev", "clang::ABIInfo::~ABIInfo()"},
    {"_ZN5clang7ABIInfoD2Ev", "clang::ABIInfo::~ABIInfo()"},
    {"_ZNK5clang7ABIInfo10getContextEv", "clang::ABIInfo::getContext() const"},
    {"_ZNK5clang7ABIInfo12getVMContextEv", "clang::ABIInfo::getVMContext() const"},
    {"_ZNK5clang7ABIInfo13getDataLayoutEv", "clang::ABIInfo::getDataLayout() const"},
    {"_ZNK5clang10ABIArgInfo4dumpEv", "clang::ABIArgInfo::dump() const"},
    {"_ZN5clang17TargetCodeGenInfoD0Ev", "clang::TargetCodeGenInfo::~TargetCodeGenInfo()"},
    {"_ZN5clang17TargetCodeGenInfoD1Ev", "clang::TargetCodeGenInfo::~TargetCodeGenInfo()"},
    {"_ZN5clang17TargetCodeGenInfoD2Ev", "clang::TargetCodeGenInfo::~TargetCodeGenInfo()"},
    {"_ZN5clang7CodeGen13CodeGenModule20getTargetCodeGenInfoEv", "clang::CodeGen::CodeGenModule::getTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_123X86_64TargetCodeGenInfoD1Ev", "(anonymous namespace)::X86_64TargetCodeGenInfo::~X86_64TargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_123X86_64TargetCodeGenInfoD0Ev", "(anonymous namespace)::X86_64TargetCodeGenInfo::~X86_64TargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_123X86_64TargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::X86_64TargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_123X86_64TargetCodeGenInfo23initDwarfEHRegSizeTableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::X86_64TargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZN12_GLOBAL__N_113X86_64ABIInfoD1Ev", "(anonymous namespace)::X86_64ABIInfo::~X86_64ABIInfo()"},
    {"_ZN12_GLOBAL__N_113X86_64ABIInfoD0Ev", "(anonymous namespace)::X86_64ABIInfo::~X86_64ABIInfo()"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::X86_64ABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::X86_64ABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo20classifyArgumentTypeEN5clang8QualTypeERjS3_", "(anonymous namespace)::X86_64ABIInfo::classifyArgumentType(clang::QualType, unsigned int&, unsigned int&) const"},
    {"_Z19EmitVAArgFromMemoryPN4llvm5ValueEN5clang8QualTypeERNS2_7CodeGen15CodeGenFunctionE", "EmitVAArgFromMemory(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&)"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo8classifyEN5clang8QualTypeEyRNS0_5ClassES4_", "(anonymous namespace)::X86_64ABIInfo::classify(clang::QualType, unsigned long long, (anonymous namespace)::X86_64ABIInfo::Class&, (anonymous namespace)::X86_64ABIInfo::Class&) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo17getIndirectResultEN5clang8QualTypeE", "(anonymous namespace)::X86_64ABIInfo::getIndirectResult(clang::QualType) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo22GetINTEGERTypeAtOffsetEPKN4llvm4TypeEjN5clang8QualTypeEj", "(anonymous namespace)::X86_64ABIInfo::GetINTEGERTypeAtOffset(llvm::Type const*, unsigned int, clang::QualType, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo18GetSSETypeAtOffsetEPKN4llvm4TypeEjN5clang8QualTypeEj", "(anonymous namespace)::X86_64ABIInfo::GetSSETypeAtOffset(llvm::Type const*, unsigned int, clang::QualType, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_113X86_64ABIInfo19Get16ByteVectorTypeEN5clang8QualTypeE", "(anonymous namespace)::X86_64ABIInfo::Get16ByteVectorType(clang::QualType) const"},
    {"_Z26GetX86_64ByValArgumentPairPKN4llvm4TypeES2_RKNS_10DataLayoutE", "GetX86_64ByValArgumentPair(llvm::Type const*, llvm::Type const*, llvm::DataLayout const&)"},
    {"_Z21BitsContainNoUserDataN5clang8QualTypeEjjRNS_10ASTContextE", "BitsContainNoUserData(clang::QualType, unsigned int, unsigned int, clang::ASTContext&)"},
    {"_Z21ContainsFloatAtOffsetPKN4llvm4TypeEjRKNS_10DataLayoutE", "ContainsFloatAtOffset(llvm::Type const*, unsigned int, llvm::DataLayout const&)"},
    {"_ZN12_GLOBAL__N_126WinX86_64TargetCodeGenInfoD1Ev", "(anonymous namespace)::WinX86_64TargetCodeGenInfo::~WinX86_64TargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_126WinX86_64TargetCodeGenInfoD0Ev", "(anonymous namespace)::WinX86_64TargetCodeGenInfo::~WinX86_64TargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_126WinX86_64TargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::WinX86_64TargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_126WinX86_64TargetCodeGenInfo23initDwarfEHRegSizeTableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::WinX86_64TargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZN12_GLOBAL__N_116WinX86_64ABIInfoD1Ev", "(anonymous namespace)::WinX86_64ABIInfo::~WinX86_64ABIInfo()"},
    {"_ZN12_GLOBAL__N_116WinX86_64ABIInfoD0Ev", "(anonymous namespace)::WinX86_64ABIInfo::~WinX86_64ABIInfo()"},
    {"_ZNK12_GLOBAL__N_116WinX86_64ABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::WinX86_64ABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_116WinX86_64ABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::WinX86_64ABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_ZNK12_GLOBAL__N_116WinX86_64ABIInfo8classifyEN5clang8QualTypeE", "(anonymous namespace)::WinX86_64ABIInfo::classify(clang::QualType) const"},
    {"_ZN12_GLOBAL__N_123X86_32TargetCodeGenInfoD1Ev", "(anonymous namespace)::X86_32TargetCodeGenInfo::~X86_32TargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_123X86_32TargetCodeGenInfoD0Ev", "(anonymous namespace)::X86_32TargetCodeGenInfo::~X86_32TargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_123X86_32TargetCodeGenInfo19SetTargetAttributesEPKN5clang4DeclEPN4llvm11GlobalValueERNS1_7CodeGen13CodeGenModuleE", "(anonymous namespace)::X86_32TargetCodeGenInfo::SetTargetAttributes(clang::Decl const*, llvm::GlobalValue*, clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_123X86_32TargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::X86_32TargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_123X86_32TargetCodeGenInfo23initDwarfEHRegSizeTableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::X86_32TargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZN12_GLOBAL__N_113X86_32ABIInfoD1Ev", "(anonymous namespace)::X86_32ABIInfo::~X86_32ABIInfo()"},
    {"_ZN12_GLOBAL__N_113X86_32ABIInfoD0Ev", "(anonymous namespace)::X86_32ABIInfo::~X86_32ABIInfo()"},
    {"_ZNK12_GLOBAL__N_113X86_32ABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::X86_32ABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_113X86_32ABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::X86_32ABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_Z25isRecordWithSSEVectorTypeRN5clang10ASTContextENS_8QualTypeE", "isRecordWithSSEVectorType(clang::ASTContext&, clang::QualType)"},
    {"_Z21isSingleElementStructN5clang8QualTypeERNS_10ASTContextE", "isSingleElementStruct(clang::QualType, clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_113X86_32ABIInfo26shouldReturnTypeInRegisterEN5clang8QualTypeERNS1_10ASTContextE", "(anonymous namespace)::X86_32ABIInfo::shouldReturnTypeInRegister(clang::QualType, clang::ASTContext&)"},
    {"_Z12isEmptyFieldRN5clang10ASTContextEPKNS_9FieldDeclEb", "isEmptyField(clang::ASTContext&, clang::FieldDecl const*, bool)"},
    {"_Z13isEmptyRecordRN5clang10ASTContextENS_8QualTypeEb", "isEmptyRecord(clang::ASTContext&, clang::QualType, bool)"},
    {"_ZN12_GLOBAL__N_123MSP430TargetCodeGenInfoD1Ev", "(anonymous namespace)::MSP430TargetCodeGenInfo::~MSP430TargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_123MSP430TargetCodeGenInfoD0Ev", "(anonymous namespace)::MSP430TargetCodeGenInfo::~MSP430TargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_123MSP430TargetCodeGenInfo19SetTargetAttributesEPKN5clang4DeclEPN4llvm11GlobalValueERNS1_7CodeGen13CodeGenModuleE", "(anonymous namespace)::MSP430TargetCodeGenInfo::SetTargetAttributes(clang::Decl const*, llvm::GlobalValue*, clang::CodeGen::CodeGenModule&) const"},
    {"_ZN12_GLOBAL__N_114DefaultABIInfoD1Ev", "(anonymous namespace)::DefaultABIInfo::~DefaultABIInfo()"},
    {"_ZN12_GLOBAL__N_114DefaultABIInfoD0Ev", "(anonymous namespace)::DefaultABIInfo::~DefaultABIInfo()"},
    {"_ZNK12_GLOBAL__N_114DefaultABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::DefaultABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_114DefaultABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::DefaultABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_ZN12_GLOBAL__N_123MBlazeTargetCodeGenInfoD1Ev", "(anonymous namespace)::MBlazeTargetCodeGenInfo::~MBlazeTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_123MBlazeTargetCodeGenInfoD0Ev", "(anonymous namespace)::MBlazeTargetCodeGenInfo::~MBlazeTargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_123MBlazeTargetCodeGenInfo19SetTargetAttributesEPKN5clang4DeclEPN4llvm11GlobalValueERNS1_7CodeGen13CodeGenModuleE", "(anonymous namespace)::MBlazeTargetCodeGenInfo::SetTargetAttributes(clang::Decl const*, llvm::GlobalValue*, clang::CodeGen::CodeGenModule&) const"},
    {"_ZN12_GLOBAL__N_113MBlazeABIInfoD1Ev", "(anonymous namespace)::MBlazeABIInfo::~MBlazeABIInfo()"},
    {"_ZN12_GLOBAL__N_113MBlazeABIInfoD0Ev", "(anonymous namespace)::MBlazeABIInfo::~MBlazeABIInfo()"},
    {"_ZNK12_GLOBAL__N_113MBlazeABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::MBlazeABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_113MBlazeABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::MBlazeABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_ZN12_GLOBAL__N_124SystemZTargetCodeGenInfoD1Ev", "(anonymous namespace)::SystemZTargetCodeGenInfo::~SystemZTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_124SystemZTargetCodeGenInfoD0Ev", "(anonymous namespace)::SystemZTargetCodeGenInfo::~SystemZTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_114SystemZABIInfoD1Ev", "(anonymous namespace)::SystemZABIInfo::~SystemZABIInfo()"},
    {"_ZN12_GLOBAL__N_114SystemZABIInfoD0Ev", "(anonymous namespace)::SystemZABIInfo::~SystemZABIInfo()"},
    {"_ZNK12_GLOBAL__N_114SystemZABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::SystemZABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_114SystemZABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::SystemZABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_ZN12_GLOBAL__N_122PPC32TargetCodeGenInfoD1Ev", "(anonymous namespace)::PPC32TargetCodeGenInfo::~PPC32TargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_122PPC32TargetCodeGenInfoD0Ev", "(anonymous namespace)::PPC32TargetCodeGenInfo::~PPC32TargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_122PPC32TargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::PPC32TargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_122PPC32TargetCodeGenInfo23initDwarfEHRegSizeTableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::PPC32TargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZN12_GLOBAL__N_124DefaultTargetCodeGenInfoD1Ev", "(anonymous namespace)::DefaultTargetCodeGenInfo::~DefaultTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_124DefaultTargetCodeGenInfoD0Ev", "(anonymous namespace)::DefaultTargetCodeGenInfo::~DefaultTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_120ARMTargetCodeGenInfoD1Ev", "(anonymous namespace)::ARMTargetCodeGenInfo::~ARMTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_120ARMTargetCodeGenInfoD0Ev", "(anonymous namespace)::ARMTargetCodeGenInfo::~ARMTargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_120ARMTargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::ARMTargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZN12_GLOBAL__N_110ARMABIInfoD1Ev", "(anonymous namespace)::ARMABIInfo::~ARMABIInfo()"},
    {"_ZN12_GLOBAL__N_110ARMABIInfoD0Ev", "(anonymous namespace)::ARMABIInfo::~ARMABIInfo()"},
    {"_ZNK12_GLOBAL__N_110ARMABIInfo11computeInfoERN5clang7CodeGen14CGFunctionInfoE", "(anonymous namespace)::ARMABIInfo::computeInfo(clang::CodeGen::CGFunctionInfo&) const"},
    {"_ZNK12_GLOBAL__N_110ARMABIInfo9EmitVAArgEPN4llvm5ValueEN5clang8QualTypeERNS4_7CodeGen15CodeGenFunctionE", "(anonymous namespace)::ARMABIInfo::EmitVAArg(llvm::Value*, clang::QualType, clang::CodeGen::CodeGenFunction&) const"},
    {"_Z17isIntegerLikeTypeN5clang8QualTypeERNS_10ASTContextERN4llvm11LLVMContextE", "isIntegerLikeType(clang::QualType, clang::ASTContext&, llvm::LLVMContext&)"},
    {"_ZN12_GLOBAL__N_121MIPSTargetCodeGenInfoD1Ev", "(anonymous namespace)::MIPSTargetCodeGenInfo::~MIPSTargetCodeGenInfo()"},
    {"_ZN12_GLOBAL__N_121MIPSTargetCodeGenInfoD0Ev", "(anonymous namespace)::MIPSTargetCodeGenInfo::~MIPSTargetCodeGenInfo()"},
    {"_ZNK12_GLOBAL__N_121MIPSTargetCodeGenInfo22getDwarfEHStackPointerERN5clang7CodeGen13CodeGenModuleE", "(anonymous namespace)::MIPSTargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK12_GLOBAL__N_121MIPSTargetCodeGenInfo23initDwarfEHRegSizeTableERN5clang7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "(anonymous namespace)::MIPSTargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZNK5clang17TargetCodeGenInfo19SetTargetAttributesEPKNS_4DeclEPN4llvm11GlobalValueERNS_7CodeGen13CodeGenModuleE", "clang::TargetCodeGenInfo::SetTargetAttributes(clang::Decl const*, llvm::GlobalValue*, clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK5clang17TargetCodeGenInfo21extendPointerWithSExtEv", "clang::TargetCodeGenInfo::extendPointerWithSExt() const"},
    {"_ZNK5clang17TargetCodeGenInfo22getDwarfEHStackPointerERNS_7CodeGen13CodeGenModuleE", "clang::TargetCodeGenInfo::getDwarfEHStackPointer(clang::CodeGen::CodeGenModule&) const"},
    {"_ZNK5clang17TargetCodeGenInfo23initDwarfEHRegSizeTableERNS_7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "clang::TargetCodeGenInfo::initDwarfEHRegSizeTable(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZNK5clang17TargetCodeGenInfo19decodeReturnAddressERNS_7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "clang::TargetCodeGenInfo::decodeReturnAddress(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZNK5clang17TargetCodeGenInfo19encodeReturnAddressERNS_7CodeGen15CodeGenFunctionEPN4llvm5ValueE", "clang::TargetCodeGenInfo::encodeReturnAddress(clang::CodeGen::CodeGenFunction&, llvm::Value*) const"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE26CreateConstInBoundsGEP1_32EPNS_5ValueEjRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstInBoundsGEP1_32(llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE11CreateStoreEPNS_5ValueES6_b", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateStore(llvm::Value*, llvm::Value*, bool)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateLoadEPNS_5ValueEPKc", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateLoad(llvm::Value*, char const*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateAndEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateAnd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE12CreateCondBrEPNS_5ValueEPNS_10BasicBlockES8_", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCondBr(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateGEPEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateGEP(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateLoadEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateLoad(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE18CreateConstGEP1_32EPNS_5ValueEjRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstGEP1_32(llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreateAddEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE9CreatePHIEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreatePHI(llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateCastENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateCast(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE10CreateICmpENS_7CmpInst9PredicateEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateICmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb0ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb0EEEE26CreateConstInBoundsGEP2_32EPNS_5ValueEjjRKNS_5TwineE", "llvm::IRBuilder<false, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<false> >::CreateConstInBoundsGEP2_32(llvm::Value*, unsigned int, unsigned int, llvm::Twine const&)"},
    {"_ZNK5clang4Decl7getAttrINS_19MSP430InterruptAttrEEEPT_v", "clang::MSP430InterruptAttr* clang::Decl::getAttr<clang::MSP430InterruptAttr>() const"},
    {"_ZN5clang8ParseASTERNS_12PreprocessorEPNS_11ASTConsumerERNS_10ASTContextEbbPNS_20CodeCompleteConsumerE", "clang::ParseAST(clang::Preprocessor&, clang::ASTConsumer*, clang::ASTContext&, bool, bool, clang::CodeCompleteConsumer*)"},
    {"_ZN5clang8ParseASTERNS_4SemaEb", "clang::ParseAST(clang::Sema&, bool)"},
    {"_ZN5clang6Parser23ParseCXXInlineMethodDefENS_15AccessSpecifierERNS_10DeclaratorERKNS0_18ParsedTemplateInfoERKNS_14VirtSpecifiersE", "clang::Parser::ParseCXXInlineMethodDef(clang::AccessSpecifier, clang::Declarator&, clang::Parser::ParsedTemplateInfo const&, clang::VirtSpecifiers const&)"},
    {"_ZN5clang6Parser21LateParsedDeclarationD0Ev", "clang::Parser::LateParsedDeclaration::~LateParsedDeclaration()"},
    {"_ZN5clang6Parser21LateParsedDeclarationD1Ev", "clang::Parser::LateParsedDeclaration::~LateParsedDeclaration()"},
    {"_ZN5clang6Parser21LateParsedDeclarationD2Ev", "clang::Parser::LateParsedDeclaration::~LateParsedDeclaration()"},
    {"_ZN5clang6Parser21LateParsedDeclaration28ParseLexedMethodDeclarationsEv", "clang::Parser::LateParsedDeclaration::ParseLexedMethodDeclarations()"},
    {"_ZN5clang6Parser21LateParsedDeclaration20ParseLexedMethodDefsEv", "clang::Parser::LateParsedDeclaration::ParseLexedMethodDefs()"},
    {"_ZN5clang6Parser15LateParsedClassC1EPS0_PNS0_12ParsingClassE", "clang::Parser::LateParsedClass::LateParsedClass(clang::Parser*, clang::Parser::ParsingClass*)"},
    {"_ZN5clang6Parser15LateParsedClassC2EPS0_PNS0_12ParsingClassE", "clang::Parser::LateParsedClass::LateParsedClass(clang::Parser*, clang::Parser::ParsingClass*)"},
    {"_ZN5clang6Parser15LateParsedClassD0Ev", "clang::Parser::LateParsedClass::~LateParsedClass()"},
    {"_ZN5clang6Parser15LateParsedClassD1Ev", "clang::Parser::LateParsedClass::~LateParsedClass()"},
    {"_ZN5clang6Parser15LateParsedClassD2Ev", "clang::Parser::LateParsedClass::~LateParsedClass()"},
    {"_ZN5clang6Parser15LateParsedClass28ParseLexedMethodDeclarationsEv", "clang::Parser::LateParsedClass::ParseLexedMethodDeclarations()"},
    {"_ZN5clang6Parser28ParseLexedMethodDeclarationsERNS0_12ParsingClassE", "clang::Parser::ParseLexedMethodDeclarations(clang::Parser::ParsingClass&)"},
    {"_ZN5clang6Parser15LateParsedClass20ParseLexedMethodDefsEv", "clang::Parser::LateParsedClass::ParseLexedMethodDefs()"},
    {"_ZN5clang6Parser20ParseLexedMethodDefsERNS0_12ParsingClassE", "clang::Parser::ParseLexedMethodDefs(clang::Parser::ParsingClass&)"},
    {"_ZN5clang6Parser27LateParsedMethodDeclaration28ParseLexedMethodDeclarationsEv", "clang::Parser::LateParsedMethodDeclaration::ParseLexedMethodDeclarations()"},
    {"_ZN5clang6Parser27ParseLexedMethodDeclarationERNS0_27LateParsedMethodDeclarationE", "clang::Parser::ParseLexedMethodDeclaration(clang::Parser::LateParsedMethodDeclaration&)"},
    {"_ZN5clang6Parser11LexedMethod20ParseLexedMethodDefsEv", "clang::Parser::LexedMethod::ParseLexedMethodDefs()"},
    {"_ZN5clang6Parser19ParseLexedMethodDefERNS0_11LexedMethodE", "clang::Parser::ParseLexedMethodDef(clang::Parser::LexedMethod&)"},
    {"_ZN5clang6Parser20ConsumeAndStoreUntilENS_3tok9TokenKindES2_RN4llvm11SmallVectorINS_5TokenELj4EEEbb", "clang::Parser::ConsumeAndStoreUntil(clang::tok::TokenKind, clang::tok::TokenKind, llvm::SmallVector<clang::Token, 4u>&, bool, bool)"},
    {"_ZN5clang6Parser11LexedMethodD1Ev", "clang::Parser::LexedMethod::~LexedMethod()"},
    {"_ZN5clang6Parser11LexedMethodD0Ev", "clang::Parser::LexedMethod::~LexedMethod()"},
    {"_ZN5clang6Parser27LateParsedMethodDeclarationD1Ev", "clang::Parser::LateParsedMethodDeclaration::~LateParsedMethodDeclaration()"},
    {"_ZN5clang6Parser27LateParsedMethodDeclarationD0Ev", "clang::Parser::LateParsedMethodDeclaration::~LateParsedMethodDeclaration()"},
    {"_ZN5clang6Parser13ParseTypeNameEPNS_11SourceRangeENS_10Declarator10TheContextE", "clang::Parser::ParseTypeName(clang::SourceRange*, clang::Declarator::TheContext)"},
    {"_ZN5clang6Parser27ParseSpecifierQualifierListERNS_8DeclSpecE", "clang::Parser::ParseSpecifierQualifierList(clang::DeclSpec&)"},
    {"_ZN5clang6Parser15ParseDeclaratorERNS_10DeclaratorE", "clang::Parser::ParseDeclarator(clang::Declarator&)"},
    {"_ZN5clang6Parser18ParseGNUAttributesERNS_16ParsedAttributesEPNS_14SourceLocationE", "clang::Parser::ParseGNUAttributes(clang::ParsedAttributes&, clang::SourceLocation*)"},
    {"_ZN5clang6Parser22isDeclarationSpecifierEb", "clang::Parser::isDeclarationSpecifier(bool)"},
    {"_ZN5clang6Parser22ParseMicrosoftDeclSpecERNS_16ParsedAttributesE", "clang::Parser::ParseMicrosoftDeclSpec(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser28ParseMicrosoftTypeAttributesERNS_16ParsedAttributesE", "clang::Parser::ParseMicrosoftTypeAttributes(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser26ParseBorlandTypeAttributesERNS_16ParsedAttributesE", "clang::Parser::ParseBorlandTypeAttributes(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser28DiagnoseProhibitedAttributesERNS0_25ParsedAttributesWithRangeE", "clang::Parser::DiagnoseProhibitedAttributes(clang::Parser::ParsedAttributesWithRange&)"},
    {"_ZN5clang6Parser16ParseDeclarationERNS_15ASTOwningVectorIPNS_4StmtELj32EEEjRNS_14SourceLocationERNS0_25ParsedAttributesWithRangeE", "clang::Parser::ParseDeclaration(clang::ASTOwningVector<clang::Stmt*, 32u>&, unsigned int, clang::SourceLocation&, clang::Parser::ParsedAttributesWithRange&)"},
    {"_ZN5clang6Parser22ParseSimpleDeclarationERNS_15ASTOwningVectorIPNS_4StmtELj32EEEjRNS_14SourceLocationERNS_16ParsedAttributesEb", "clang::Parser::ParseSimpleDeclaration(clang::ASTOwningVector<clang::Stmt*, 32u>&, unsigned int, clang::SourceLocation&, clang::ParsedAttributes&, bool)"},
    {"_ZN5clang6Parser26ParseDeclarationSpecifiersERNS_8DeclSpecERKNS0_18ParsedTemplateInfoENS_15AccessSpecifierENS0_15DeclSpecContextE", "clang::Parser::ParseDeclarationSpecifiers(clang::DeclSpec&, clang::Parser::ParsedTemplateInfo const&, clang::AccessSpecifier, clang::Parser::DeclSpecContext)"},
    {"_ZN5clang6Parser39getDeclSpecContextFromDeclaratorContextEj", "clang::Parser::getDeclSpecContextFromDeclaratorContext(unsigned int)"},
    {"_ZN5clang6Parser14ParseDeclGroupERNS0_15ParsingDeclSpecEjbPNS_14SourceLocationE", "clang::Parser::ParseDeclGroup(clang::Parser::ParsingDeclSpec&, unsigned int, bool, clang::SourceLocation*)"},
    {"_ZN5clang6Parser31ParseDeclarationAfterDeclaratorERNS_10DeclaratorERKNS0_18ParsedTemplateInfoE", "clang::Parser::ParseDeclarationAfterDeclarator(clang::Declarator&, clang::Parser::ParsedTemplateInfo const&)"},
    {"_ZN5clang6Parser16ParseImplicitIntERNS_8DeclSpecEPNS_12CXXScopeSpecERKNS0_18ParsedTemplateInfoENS_15AccessSpecifierE", "clang::Parser::ParseImplicitInt(clang::DeclSpec&, clang::CXXScopeSpec*, clang::Parser::ParsedTemplateInfo const&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser18ParseEnumSpecifierENS_14SourceLocationERNS_8DeclSpecERKNS0_18ParsedTemplateInfoENS_15AccessSpecifierE", "clang::Parser::ParseEnumSpecifier(clang::SourceLocation, clang::DeclSpec&, clang::Parser::ParsedTemplateInfo const&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser23isConstructorDeclaratorEv", "clang::Parser::isConstructorDeclarator()"},
    {"_ZN5clang6Parser20ParseTypeofSpecifierERNS_8DeclSpecE", "clang::Parser::ParseTypeofSpecifier(clang::DeclSpec&)"},
    {"_ZN5clang6Parser26ParseOptionalTypeSpecifierERNS_8DeclSpecERbRPKcRjRKNS0_18ParsedTemplateInfoEb", "clang::Parser::ParseOptionalTypeSpecifier(clang::DeclSpec&, bool&, char const*&, unsigned int&, clang::Parser::ParsedTemplateInfo const&, bool)"},
    {"_ZN5clang6Parser22ParseStructDeclarationERNS_8DeclSpecERNS0_13FieldCallbackE", "clang::Parser::ParseStructDeclaration(clang::DeclSpec&, clang::Parser::FieldCallback&)"},
    {"_ZN5clang6Parser20ParseStructUnionBodyENS_14SourceLocationEjPNS_4DeclE", "clang::Parser::ParseStructUnionBody(clang::SourceLocation, unsigned int, clang::Decl*)"},
    {"_ZZN5clang6Parser20ParseStructUnionBodyENS_14SourceLocationEjPNS_4DeclEEN14CFieldCallbackD1Ev", "clang::Parser::ParseStructUnionBody(clang::SourceLocation, unsigned int, clang::Decl*)::CFieldCallback::~CFieldCallback()"},
    {"_ZN5clang6Parser13ParseEnumBodyENS_14SourceLocationEPNS_4DeclE", "clang::Parser::ParseEnumBody(clang::SourceLocation, clang::Decl*)"},
    {"_ZNK5clang6Parser15isTypeQualifierEv", "clang::Parser::isTypeQualifier() const"},
    {"_ZNK5clang6Parser24isKnownToBeTypeSpecifierERKNS_5TokenE", "clang::Parser::isKnownToBeTypeSpecifier(clang::Token const&) const"},
    {"_ZN5clang6Parser24isTypeSpecifierQualifierEv", "clang::Parser::isTypeSpecifierQualifier()"},
    {"_ZN5clang6Parser25ParseTypeQualifierListOptERNS_8DeclSpecEbb", "clang::Parser::ParseTypeQualifierListOpt(clang::DeclSpec&, bool, bool)"},
    {"_ZN5clang6Parser23ParseDeclaratorInternalERNS_10DeclaratorEMS0_FvS2_E", "clang::Parser::ParseDeclaratorInternal(clang::Declarator&, void (clang::Parser::*)(clang::Declarator&))"},
    {"_ZN5clang6Parser21ParseDirectDeclaratorERNS_10DeclaratorE", "clang::Parser::ParseDirectDeclarator(clang::Declarator&)"},
    {"_ZN5clang6Parser20ParseParenDeclaratorERNS_10DeclaratorE", "clang::Parser::ParseParenDeclarator(clang::Declarator&)"},
    {"_ZN5clang6Parser23ParseFunctionDeclaratorENS_14SourceLocationERNS_10DeclaratorERNS_16ParsedAttributesEb", "clang::Parser::ParseFunctionDeclarator(clang::SourceLocation, clang::Declarator&, clang::ParsedAttributes&, bool)"},
    {"_ZN5clang6Parser22ParseBracketDeclaratorERNS_10DeclaratorE", "clang::Parser::ParseBracketDeclarator(clang::Declarator&)"},
    {"_ZN5clang6Parser37ParseFunctionDeclaratorIdentifierListENS_14SourceLocationEPNS_14IdentifierInfoES1_RNS_10DeclaratorE", "clang::Parser::ParseFunctionDeclaratorIdentifierList(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Declarator&)"},
    {"_ZN5clang6Parser30TryAltiVecVectorTokenOutOfLineEv", "clang::Parser::TryAltiVecVectorTokenOutOfLine()"},
    {"_ZN5clang6Parser24TryAltiVecTokenOutOfLineERNS_8DeclSpecENS_14SourceLocationERPKcRjRb", "clang::Parser::TryAltiVecTokenOutOfLine(clang::DeclSpec&, clang::SourceLocation, char const*&, unsigned int&, bool&)"},
    {"_ZZN5clang6Parser20ParseStructUnionBodyENS_14SourceLocationEjPNS_4DeclEEN14CFieldCallback6invokeERNS_15FieldDeclaratorE", "clang::Parser::ParseStructUnionBody(clang::SourceLocation, unsigned int, clang::Decl*)::CFieldCallback::invoke(clang::FieldDeclarator&)"},
    {"_ZZN5clang6Parser20ParseStructUnionBodyENS_14SourceLocationEjPNS_4DeclEEN14CFieldCallbackD0Ev", "clang::Parser::ParseStructUnionBody(clang::SourceLocation, unsigned int, clang::Decl*)::CFieldCallback::~CFieldCallback()"},
    {"_ZN4llvm15SmallVectorImplIPN5clang4DeclEE6insertIPS3_EES6_S6_T_S7_", "clang::Decl** llvm::SmallVectorImpl<clang::Decl*>::insert<clang::Decl**>(clang::Decl**, clang::Decl**, clang::Decl**)"},
    {"_ZN5clang6Parser14ParseNamespaceEjRNS_14SourceLocationES1_", "clang::Parser::ParseNamespace(unsigned int, clang::SourceLocation&, clang::SourceLocation)"},
    {"_ZN5clang6Parser19ParseNamespaceAliasENS_14SourceLocationES1_PNS_14IdentifierInfoERS1_", "clang::Parser::ParseNamespaceAlias(clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation&)"},
    {"_ZN5clang6Parser12ParseLinkageERNS0_15ParsingDeclSpecEj", "clang::Parser::ParseLinkage(clang::Parser::ParsingDeclSpec&, unsigned int)"},
    {"_ZN5clang6Parser32ParseUsingDirectiveOrDeclarationEjRKNS0_18ParsedTemplateInfoERNS_14SourceLocationERNS0_25ParsedAttributesWithRangeE", "clang::Parser::ParseUsingDirectiveOrDeclaration(unsigned int, clang::Parser::ParsedTemplateInfo const&, clang::SourceLocation&, clang::Parser::ParsedAttributesWithRange&)"},
    {"_ZN5clang6Parser19ParseUsingDirectiveEjNS_14SourceLocationERS1_RNS_16ParsedAttributesE", "clang::Parser::ParseUsingDirective(unsigned int, clang::SourceLocation, clang::SourceLocation&, clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser21ParseUsingDeclarationEjRKNS0_18ParsedTemplateInfoENS_14SourceLocationERS4_NS_15AccessSpecifierE", "clang::Parser::ParseUsingDeclaration(unsigned int, clang::Parser::ParsedTemplateInfo const&, clang::SourceLocation, clang::SourceLocation&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser28ParseStaticAssertDeclarationERNS_14SourceLocationE", "clang::Parser::ParseStaticAssertDeclaration(clang::SourceLocation&)"},
    {"_ZN5clang6Parser22ParseDecltypeSpecifierERNS_8DeclSpecE", "clang::Parser::ParseDecltypeSpecifier(clang::DeclSpec&)"},
    {"_ZN5clang6Parser14ParseClassNameERNS_14SourceLocationEPNS_12CXXScopeSpecE", "clang::Parser::ParseClassName(clang::SourceLocation&, clang::CXXScopeSpec*)"},
    {"_ZN5clang6Parser19ParseClassSpecifierENS_3tok9TokenKindENS_14SourceLocationERNS_8DeclSpecERKNS0_18ParsedTemplateInfoENS_15AccessSpecifierEb", "clang::Parser::ParseClassSpecifier(clang::tok::TokenKind, clang::SourceLocation, clang::DeclSpec&, clang::Parser::ParsedTemplateInfo const&, clang::AccessSpecifier, bool)"},
    {"_ZNK5clang6Parser25isCXX0XClassVirtSpecifierEv", "clang::Parser::isCXX0XClassVirtSpecifier() const"},
    {"_ZN5clang6Parser27ParseCXXMemberSpecificationENS_14SourceLocationEjPNS_4DeclE", "clang::Parser::ParseCXXMemberSpecification(clang::SourceLocation, unsigned int, clang::Decl*)"},
    {"_ZN5clang6Parser15ParseBaseClauseEPNS_4DeclE", "clang::Parser::ParseBaseClause(clang::Decl*)"},
    {"_ZN5clang6Parser18ParseBaseSpecifierEPNS_4DeclE", "clang::Parser::ParseBaseSpecifier(clang::Decl*)"},
    {"_ZNK5clang6Parser27getAccessSpecifierIfPresentEv", "clang::Parser::getAccessSpecifierIfPresent() const"},
    {"_ZN5clang6Parser31HandleMemberFunctionDefaultArgsERNS_10DeclaratorEPNS_4DeclE", "clang::Parser::HandleMemberFunctionDefaultArgs(clang::Declarator&, clang::Decl*)"},
    {"_ZNK5clang6Parser20isCXX0XVirtSpecifierEv", "clang::Parser::isCXX0XVirtSpecifier() const"},
    {"_ZN5clang6Parser34ParseOptionalCXX0XVirtSpecifierSeqERNS_14VirtSpecifiersE", "clang::Parser::ParseOptionalCXX0XVirtSpecifierSeq(clang::VirtSpecifiers&)"},
    {"_ZN5clang6Parser39ParseOptionalCXX0XClassVirtSpecifierSeqERNS_19ClassVirtSpecifiersE", "clang::Parser::ParseOptionalCXX0XClassVirtSpecifierSeq(clang::ClassVirtSpecifiers&)"},
    {"_ZN5clang6Parser30ParseCXXClassMemberDeclarationENS_15AccessSpecifierERKNS0_18ParsedTemplateInfoEPNS0_21ParsingDeclRAIIObjectE", "clang::Parser::ParseCXXClassMemberDeclaration(clang::AccessSpecifier, clang::Parser::ParsedTemplateInfo const&, clang::Parser::ParsingDeclRAIIObject*)"},
    {"_ZN5clang6Parser27ParseConstructorInitializerEPNS_4DeclE", "clang::Parser::ParseConstructorInitializer(clang::Decl*)"},
    {"_ZN5clang6Parser19ParseMemInitializerEPNS_4DeclE", "clang::Parser::ParseMemInitializer(clang::Decl*)"},
    {"_ZN5clang6Parser27ParseExceptionSpecificationERNS_14SourceLocationERN4llvm15SmallVectorImplINS_9OpaquePtrINS_8QualTypeEEEEERNS4_INS_11SourceRangeEEERb", "clang::Parser::ParseExceptionSpecification(clang::SourceLocation&, llvm::SmallVectorImpl<clang::OpaquePtr<clang::QualType> >&, llvm::SmallVectorImpl<clang::SourceRange>&, bool&)"},
    {"_ZN5clang6Parser23ParseTrailingReturnTypeEv", "clang::Parser::ParseTrailingReturnType()"},
    {"_ZN5clang6Parser16PushParsingClassEPNS_4DeclEb", "clang::Parser::PushParsingClass(clang::Decl*, bool)"},
    {"_ZN5clang6Parser23DeallocateParsedClassesEPNS0_12ParsingClassE", "clang::Parser::DeallocateParsedClasses(clang::Parser::ParsingClass*)"},
    {"_ZN5clang6Parser15PopParsingClassEv", "clang::Parser::PopParsingClass()"},
    {"_ZN5clang6Parser20ParseCXX0XAttributesERNS0_25ParsedAttributesWithRangeEPNS_14SourceLocationE", "clang::Parser::ParseCXX0XAttributes(clang::Parser::ParsedAttributesWithRange&, clang::SourceLocation*)"},
    {"_ZN5clang6Parser23ParseCXX0XAlignArgumentENS_14SourceLocationE", "clang::Parser::ParseCXX0XAlignArgument(clang::SourceLocation)"},
    {"_ZN5clang6Parser24ParseMicrosoftAttributesERNS_16ParsedAttributesEPNS_14SourceLocationE", "clang::Parser::ParseMicrosoftAttributes(clang::ParsedAttributes&, clang::SourceLocation*)"},
    {"_ZNSt5dequeIPN5clang6Parser12ParsingClassESaIS3_EE9push_backERKS3_", "std::deque<clang::Parser::ParsingClass*, std::allocator<clang::Parser::ParsingClass*> >::push_back(clang::Parser::ParsingClass* const&)"},
    {"_ZNSt5dequeIPN5clang6Parser12ParsingClassESaIS3_EE17_M_reallocate_mapEmb", "std::deque<clang::Parser::ParsingClass*, std::allocator<clang::Parser::ParsingClass*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang6Parser25LateParsedDefaultArgumentELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Parser::LateParsedDefaultArgument, false>::grow(unsigned long)"},
    {"_ZN5clang6Parser15ParseExpressionEv", "clang::Parser::ParseExpression()"},
    {"_ZN5clang6Parser25ParseAssignmentExpressionEv", "clang::Parser::ParseAssignmentExpression()"},
    {"_ZN5clang6Parser26ParseRHSOfBinaryExpressionENS_12ActionResultIPNS_4ExprELb1EEENS_4prec5LevelE", "clang::Parser::ParseRHSOfBinaryExpression(clang::ActionResult<clang::Expr*, true>, clang::prec::Level)"},
    {"_ZN5clang6Parser28ParseExpressionWithLeadingAtENS_14SourceLocationE", "clang::Parser::ParseExpressionWithLeadingAt(clang::SourceLocation)"},
    {"_ZN5clang6Parser35ParseExpressionWithLeadingExtensionENS_14SourceLocationE", "clang::Parser::ParseExpressionWithLeadingExtension(clang::SourceLocation)"},
    {"_ZN5clang6Parser19ParseCastExpressionEbbNS_9OpaquePtrINS_8QualTypeEEE", "clang::Parser::ParseCastExpression(bool, bool, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang6Parser43ParseAssignmentExprWithObjCMessageExprStartENS_14SourceLocationES1_NS_9OpaquePtrINS_8QualTypeEEEPNS_4ExprE", "clang::Parser::ParseAssignmentExprWithObjCMessageExprStart(clang::SourceLocation, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::Expr*)"},
    {"_ZN5clang6Parser28ParsePostfixExpressionSuffixENS_12ActionResultIPNS_4ExprELb1EEE", "clang::Parser::ParsePostfixExpressionSuffix(clang::ActionResult<clang::Expr*, true>)"},
    {"_ZN5clang6Parser23ParseConstantExpressionEv", "clang::Parser::ParseConstantExpression()"},
    {"_ZN5clang6Parser19ParseCastExpressionEbbRbNS_9OpaquePtrINS_8QualTypeEEE", "clang::Parser::ParseCastExpression(bool, bool, bool&, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang6Parser20ParseParenExpressionERNS0_16ParenParseOptionEbNS_9OpaquePtrINS_8QualTypeEEERS5_RNS_14SourceLocationE", "clang::Parser::ParseParenExpression(clang::Parser::ParenParseOption&, bool, clang::OpaquePtr<clang::QualType>, clang::OpaquePtr<clang::QualType>&, clang::SourceLocation&)"},
    {"_ZN5clang6Parser28ParseStringLiteralExpressionEv", "clang::Parser::ParseStringLiteralExpression()"},
    {"_ZN5clang6Parser29ParseBuiltinPrimaryExpressionEv", "clang::Parser::ParseBuiltinPrimaryExpression()"},
    {"_ZN5clang6Parser28ParseSizeofAlignofExpressionEv", "clang::Parser::ParseSizeofAlignofExpression()"},
    {"_ZN5clang6Parser27ParseBlockLiteralExpressionEv", "clang::Parser::ParseBlockLiteralExpression()"},
    {"_ZN5clang6Parser19ParseExpressionListERN4llvm15SmallVectorImplIPNS_4ExprEEERNS2_INS_14SourceLocationEEEMNS_4SemaEFvPNS_5ScopeES4_PS4_jES4_", "clang::Parser::ParseExpressionList(llvm::SmallVectorImpl<clang::Expr*>&, llvm::SmallVectorImpl<clang::SourceLocation>&, void (clang::Sema::*)(clang::Scope*, clang::Expr*, clang::Expr**, unsigned int), clang::Expr*)"},
    {"_ZN5clang6Parser33ParseExprAfterTypeofSizeofAlignofERKNS_5TokenERbRNS_9OpaquePtrINS_8QualTypeEEERNS_11SourceRangeE", "clang::Parser::ParseExprAfterTypeofSizeofAlignof(clang::Token const&, bool&, clang::OpaquePtr<clang::QualType>&, clang::SourceRange&)"},
    {"_ZN5clang6Parser30ParseCompoundLiteralExpressionENS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationES4_", "clang::Parser::ParseCompoundLiteralExpression(clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang6Parser12ParseBlockIdEv", "clang::Parser::ParseBlockId()"},
    {"_ZN5clang6Parser30ParseOptionalCXXScopeSpecifierERNS_12CXXScopeSpecENS_9OpaquePtrINS_8QualTypeEEEbPb", "clang::Parser::ParseOptionalCXXScopeSpecifier(clang::CXXScopeSpec&, clang::OpaquePtr<clang::QualType>, bool, bool*)"},
    {"_ZN5clang6Parser26ParseUnqualifiedIdOperatorERNS_12CXXScopeSpecEbNS_9OpaquePtrINS_8QualTypeEEERNS_13UnqualifiedIdE", "clang::Parser::ParseUnqualifiedIdOperator(clang::CXXScopeSpec&, bool, clang::OpaquePtr<clang::QualType>, clang::UnqualifiedId&)"},
    {"_ZN5clang6Parser20ParseCXXIdExpressionEb", "clang::Parser::ParseCXXIdExpression(bool)"},
    {"_ZN5clang6Parser18ParseUnqualifiedIdERNS_12CXXScopeSpecEbbbNS_9OpaquePtrINS_8QualTypeEEERNS_13UnqualifiedIdE", "clang::Parser::ParseUnqualifiedId(clang::CXXScopeSpec&, bool, bool, bool, clang::OpaquePtr<clang::QualType>, clang::UnqualifiedId&)"},
    {"_ZN5clang6Parser13ParseCXXCastsEv", "clang::Parser::ParseCXXCasts()"},
    {"_ZN5clang6Parser14ParseCXXTypeidEv", "clang::Parser::ParseCXXTypeid()"},
    {"_ZN5clang6Parser14ParseCXXUuidofEv", "clang::Parser::ParseCXXUuidof()"},
    {"_ZN5clang6Parser24ParseCXXPseudoDestructorEPNS_4ExprENS_14SourceLocationENS_3tok9TokenKindERNS_12CXXScopeSpecENS_9OpaquePtrINS_8QualTypeEEE", "clang::Parser::ParseCXXPseudoDestructor(clang::Expr*, clang::SourceLocation, clang::tok::TokenKind, clang::CXXScopeSpec&, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang6Parser28ParseUnqualifiedIdTemplateIdERNS_12CXXScopeSpecEPNS_14IdentifierInfoENS_14SourceLocationEbNS_9OpaquePtrINS_8QualTypeEEERNS_13UnqualifiedIdEbS5_", "clang::Parser::ParseUnqualifiedIdTemplateId(clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::SourceLocation, bool, clang::OpaquePtr<clang::QualType>, clang::UnqualifiedId&, bool, clang::SourceLocation)"},
    {"_ZN5clang6Parser19ParseCXXBoolLiteralEv", "clang::Parser::ParseCXXBoolLiteral()"},
    {"_ZN5clang6Parser20ParseThrowExpressionEv", "clang::Parser::ParseThrowExpression()"},
    {"_ZN5clang6Parser12ParseCXXThisEv", "clang::Parser::ParseCXXThis()"},
    {"_ZN5clang6Parser31ParseCXXTypeConstructExpressionERKNS_8DeclSpecE", "clang::Parser::ParseCXXTypeConstructExpression(clang::DeclSpec const&)"},
    {"_ZN5clang6Parser17ParseCXXConditionERNS_12ActionResultIPNS_4ExprELb1EEERPNS_4DeclENS_14SourceLocationEb", "clang::Parser::ParseCXXCondition(clang::ActionResult<clang::Expr*, true>&, clang::Decl*&, clang::SourceLocation, bool)"},
    {"_ZNK5clang6Parser24isCXXSimpleTypeSpecifierEv", "clang::Parser::isCXXSimpleTypeSpecifier() const"},
    {"_ZN5clang6Parser27ParseCXXSimpleTypeSpecifierERNS_8DeclSpecE", "clang::Parser::ParseCXXSimpleTypeSpecifier(clang::DeclSpec&)"},
    {"_ZN5clang6Parser24ParseCXXTypeSpecifierSeqERNS_8DeclSpecE", "clang::Parser::ParseCXXTypeSpecifierSeq(clang::DeclSpec&)"},
    {"_ZN5clang6Parser21ParseCXXNewExpressionEbNS_14SourceLocationE", "clang::Parser::ParseCXXNewExpression(bool, clang::SourceLocation)"},
    {"_ZN5clang6Parser27ParseExpressionListOrTypeIdERN4llvm15SmallVectorImplIPNS_4ExprEEERNS_10DeclaratorE", "clang::Parser::ParseExpressionListOrTypeId(llvm::SmallVectorImpl<clang::Expr*>&, clang::Declarator&)"},
    {"_ZN5clang6Parser24ParseDirectNewDeclaratorERNS_10DeclaratorE", "clang::Parser::ParseDirectNewDeclarator(clang::Declarator&)"},
    {"_ZN5clang6Parser24ParseCXXDeleteExpressionEbNS_14SourceLocationE", "clang::Parser::ParseCXXDeleteExpression(bool, clang::SourceLocation)"},
    {"_ZN5clang6Parser19ParseUnaryTypeTraitEv", "clang::Parser::ParseUnaryTypeTrait()"},
    {"_ZN5clang6Parser20ParseBinaryTypeTraitEv", "clang::Parser::ParseBinaryTypeTrait()"},
    {"_ZN5clang6Parser32ParseCXXAmbiguousParenExpressionERNS0_16ParenParseOptionERNS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationERS7_", "clang::Parser::ParseCXXAmbiguousParenExpression(clang::Parser::ParenParseOption&, clang::OpaquePtr<clang::QualType>&, clang::SourceLocation, clang::SourceLocation&)"},
    {"_ZN5clang6Parser39ParseInitializerWithPotentialDesignatorEv", "clang::Parser::ParseInitializerWithPotentialDesignator()"},
    {"_ZN5clang6Parser21ParseBraceInitializerEv", "clang::Parser::ParseBraceInitializer()"},
    {"_ZN5clang6Parser21ParseObjCAtDirectivesEv", "clang::Parser::ParseObjCAtDirectives()"},
    {"_ZN5clang6Parser27ParseObjCAtClassDeclarationENS_14SourceLocationE", "clang::Parser::ParseObjCAtClassDeclaration(clang::SourceLocation)"},
    {"_ZN5clang6Parser31ParseObjCAtInterfaceDeclarationENS_14SourceLocationERNS_16ParsedAttributesE", "clang::Parser::ParseObjCAtInterfaceDeclaration(clang::SourceLocation, clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser30ParseObjCAtProtocolDeclarationENS_14SourceLocationERNS_16ParsedAttributesE", "clang::Parser::ParseObjCAtProtocolDeclaration(clang::SourceLocation, clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser36ParseObjCAtImplementationDeclarationENS_14SourceLocationE", "clang::Parser::ParseObjCAtImplementationDeclaration(clang::SourceLocation)"},
    {"_ZN5clang6Parser25ParseObjCAtEndDeclarationENS_11SourceRangeE", "clang::Parser::ParseObjCAtEndDeclaration(clang::SourceRange)"},
    {"_ZN5clang6Parser27ParseObjCAtAliasDeclarationENS_14SourceLocationE", "clang::Parser::ParseObjCAtAliasDeclaration(clang::SourceLocation)"},
    {"_ZN5clang6Parser27ParseObjCPropertySynthesizeENS_14SourceLocationE", "clang::Parser::ParseObjCPropertySynthesize(clang::SourceLocation)"},
    {"_ZN5clang6Parser24ParseObjCPropertyDynamicENS_14SourceLocationE", "clang::Parser::ParseObjCPropertyDynamic(clang::SourceLocation)"},
    {"_ZN5clang6Parser27ParseObjCProtocolReferencesERN4llvm15SmallVectorImplIPNS_4DeclEEERNS2_INS_14SourceLocationEEEbRS7_SA_", "clang::Parser::ParseObjCProtocolReferences(llvm::SmallVectorImpl<clang::Decl*>&, llvm::SmallVectorImpl<clang::SourceLocation>&, bool, clang::SourceLocation&, clang::SourceLocation&)"},
    {"_ZN5clang6Parser31ParseObjCClassInstanceVariablesEPNS_4DeclENS_3tok15ObjCKeywordKindENS_14SourceLocationE", "clang::Parser::ParseObjCClassInstanceVariables(clang::Decl*, clang::tok::ObjCKeywordKind, clang::SourceLocation)"},
    {"_ZN5clang6Parser26ParseObjCInterfaceDeclListEPNS_4DeclENS_3tok15ObjCKeywordKindE", "clang::Parser::ParseObjCInterfaceDeclList(clang::Decl*, clang::tok::ObjCKeywordKind)"},
    {"_ZN5clang6Parser24ParseObjCMethodPrototypeEPNS_4DeclENS_3tok15ObjCKeywordKindE", "clang::Parser::ParseObjCMethodPrototype(clang::Decl*, clang::tok::ObjCKeywordKind)"},
    {"_ZN5clang6Parser19ParseObjCMethodDeclENS_14SourceLocationENS_3tok9TokenKindEPNS_4DeclENS2_15ObjCKeywordKindE", "clang::Parser::ParseObjCMethodDecl(clang::SourceLocation, clang::tok::TokenKind, clang::Decl*, clang::tok::ObjCKeywordKind)"},
    {"_ZN5clang6Parser26ParseObjCPropertyAttributeERNS_12ObjCDeclSpecEPNS_4DeclE", "clang::Parser::ParseObjCPropertyAttribute(clang::ObjCDeclSpec&, clang::Decl*)"},
    {"_ZN5clang6Parser22ParseObjCSelectorPieceERNS_14SourceLocationE", "clang::Parser::ParseObjCSelectorPiece(clang::SourceLocation&)"},
    {"_ZNK5clang6Parser18isTokIdentifier_inEv", "clang::Parser::isTokIdentifier_in() const"},
    {"_ZN5clang6Parser26ParseObjCTypeQualifierListERNS_12ObjCDeclSpecEb", "clang::Parser::ParseObjCTypeQualifierList(clang::ObjCDeclSpec&, bool)"},
    {"_ZN5clang6Parser17ParseObjCTypeNameERNS_12ObjCDeclSpecEb", "clang::Parser::ParseObjCTypeName(clang::ObjCDeclSpec&, bool)"},
    {"_ZN5clang6Parser27ParseObjCProtocolQualifiersERNS_8DeclSpecE", "clang::Parser::ParseObjCProtocolQualifiers(clang::DeclSpec&)"},
    {"_ZZN5clang6Parser31ParseObjCClassInstanceVariablesEPNS_4DeclENS_3tok15ObjCKeywordKindENS_14SourceLocationEEN16ObjCIvarCallbackD1Ev", "clang::Parser::ParseObjCClassInstanceVariables(clang::Decl*, clang::tok::ObjCKeywordKind, clang::SourceLocation)::ObjCIvarCallback::~ObjCIvarCallback()"},
    {"_ZN5clang6Parser24FinishPendingObjCActionsEv", "clang::Parser::FinishPendingObjCActions()"},
    {"_ZN5clang6Parser18ParseObjCThrowStmtENS_14SourceLocationE", "clang::Parser::ParseObjCThrowStmt(clang::SourceLocation)"},
    {"_ZN5clang6Parser25ParseObjCSynchronizedStmtENS_14SourceLocationE", "clang::Parser::ParseObjCSynchronizedStmt(clang::SourceLocation)"},
    {"_ZN5clang6Parser16ParseObjCTryStmtENS_14SourceLocationE", "clang::Parser::ParseObjCTryStmt(clang::SourceLocation)"},
    {"_ZN5clang6Parser25ParseObjCMethodDefinitionEv", "clang::Parser::ParseObjCMethodDefinition()"},
    {"_ZN5clang6Parser20ParseObjCAtStatementENS_14SourceLocationE", "clang::Parser::ParseObjCAtStatement(clang::SourceLocation)"},
    {"_ZN5clang6Parser21ParseObjCAtExpressionENS_14SourceLocationE", "clang::Parser::ParseObjCAtExpression(clang::SourceLocation)"},
    {"_ZN5clang6Parser22ParseObjCStringLiteralENS_14SourceLocationE", "clang::Parser::ParseObjCStringLiteral(clang::SourceLocation)"},
    {"_ZN5clang6Parser25ParseObjCEncodeExpressionENS_14SourceLocationE", "clang::Parser::ParseObjCEncodeExpression(clang::SourceLocation)"},
    {"_ZN5clang6Parser27ParseObjCProtocolExpressionENS_14SourceLocationE", "clang::Parser::ParseObjCProtocolExpression(clang::SourceLocation)"},
    {"_ZN5clang6Parser27ParseObjCSelectorExpressionENS_14SourceLocationE", "clang::Parser::ParseObjCSelectorExpression(clang::SourceLocation)"},
    {"_ZN5clang6Parser26ParseObjCXXMessageReceiverERbRPv", "clang::Parser::ParseObjCXXMessageReceiver(bool&, void*&)"},
    {"_ZN5clang6Parser29isSimpleObjCMessageExpressionEv", "clang::Parser::isSimpleObjCMessageExpression()"},
    {"_ZN5clang6Parser43isStartOfObjCClassMessageMissingOpenBracketEv", "clang::Parser::isStartOfObjCClassMessageMissingOpenBracket()"},
    {"_ZN5clang6Parser26ParseObjCMessageExpressionEv", "clang::Parser::ParseObjCMessageExpression()"},
    {"_ZN5clang6Parser30ParseObjCMessageExpressionBodyENS_14SourceLocationES1_NS_9OpaquePtrINS_8QualTypeEEEPNS_4ExprE", "clang::Parser::ParseObjCMessageExpressionBody(clang::SourceLocation, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::Expr*)"},
    {"_ZZN5clang6Parser31ParseObjCClassInstanceVariablesEPNS_4DeclENS_3tok15ObjCKeywordKindENS_14SourceLocationEEN16ObjCIvarCallback6invokeERNS_15FieldDeclaratorE", "clang::Parser::ParseObjCClassInstanceVariables(clang::Decl*, clang::tok::ObjCKeywordKind, clang::SourceLocation)::ObjCIvarCallback::invoke(clang::FieldDeclarator&)"},
    {"_ZZN5clang6Parser31ParseObjCClassInstanceVariablesEPNS_4DeclENS_3tok15ObjCKeywordKindENS_14SourceLocationEEN16ObjCIvarCallbackD0Ev", "clang::Parser::ParseObjCClassInstanceVariables(clang::Decl*, clang::tok::ObjCKeywordKind, clang::SourceLocation)::ObjCIvarCallback::~ObjCIvarCallback()"},
    {"_ZN5clang6Parser20ObjCPropertyCallbackD1Ev", "clang::Parser::ObjCPropertyCallback::~ObjCPropertyCallback()"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang15DeclaratorChunk9ParamInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::DeclaratorChunk::ParamInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4Sema11ObjCArgInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Sema::ObjCArgInfo, false>::grow(unsigned long)"},
    {"_ZN5clang6Parser20ObjCPropertyCallback6invokeERNS_15FieldDeclaratorE", "clang::Parser::ObjCPropertyCallback::invoke(clang::FieldDeclarator&)"},
    {"_ZN5clang6Parser20ObjCPropertyCallbackD0Ev", "clang::Parser::ObjCPropertyCallback::~ObjCPropertyCallback()"},
    {"_ZN5clang6Parser18HandlePragmaUnusedEv", "clang::Parser::HandlePragmaUnused()"},
    {"_ZN5clang26PragmaGCCVisibilityHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaGCCVisibilityHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang17PragmaPackHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaPackHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang18PragmaAlignHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaAlignHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_Z16ParseAlignPragmaRN5clang4SemaERNS_12PreprocessorERNS_5TokenEb", "ParseAlignPragma(clang::Sema&, clang::Preprocessor&, clang::Token&, bool)"},
    {"_ZN5clang20PragmaOptionsHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaOptionsHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang19PragmaUnusedHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaUnusedHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang17PragmaWeakHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaWeakHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang18PragmaAlignHandlerD1Ev", "clang::PragmaAlignHandler::~PragmaAlignHandler()"},
    {"_ZN5clang18PragmaAlignHandlerD0Ev", "clang::PragmaAlignHandler::~PragmaAlignHandler()"},
    {"_ZN5clang26PragmaGCCVisibilityHandlerD1Ev", "clang::PragmaGCCVisibilityHandler::~PragmaGCCVisibilityHandler()"},
    {"_ZN5clang26PragmaGCCVisibilityHandlerD0Ev", "clang::PragmaGCCVisibilityHandler::~PragmaGCCVisibilityHandler()"},
    {"_ZN5clang20PragmaOptionsHandlerD1Ev", "clang::PragmaOptionsHandler::~PragmaOptionsHandler()"},
    {"_ZN5clang20PragmaOptionsHandlerD0Ev", "clang::PragmaOptionsHandler::~PragmaOptionsHandler()"},
    {"_ZN5clang17PragmaPackHandlerD1Ev", "clang::PragmaPackHandler::~PragmaPackHandler()"},
    {"_ZN5clang17PragmaPackHandlerD0Ev", "clang::PragmaPackHandler::~PragmaPackHandler()"},
    {"_ZN5clang19PragmaUnusedHandlerD1Ev", "clang::PragmaUnusedHandler::~PragmaUnusedHandler()"},
    {"_ZN5clang19PragmaUnusedHandlerD0Ev", "clang::PragmaUnusedHandler::~PragmaUnusedHandler()"},
    {"_ZN5clang17PragmaWeakHandlerD1Ev", "clang::PragmaWeakHandler::~PragmaWeakHandler()"},
    {"_ZN5clang17PragmaWeakHandlerD0Ev", "clang::PragmaWeakHandler::~PragmaWeakHandler()"},
    {"_ZN5clang6Parser27ParseStatementOrDeclarationERNS_15ASTOwningVectorIPNS_4StmtELj32EEEb", "clang::Parser::ParseStatementOrDeclaration(clang::ASTOwningVector<clang::Stmt*, 32u>&, bool)"},
    {"_ZN5clang6Parser21ParseLabeledStatementERNS_16ParsedAttributesE", "clang::Parser::ParseLabeledStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser18ParseCaseStatementERNS_16ParsedAttributesE", "clang::Parser::ParseCaseStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser21ParseDefaultStatementERNS_16ParsedAttributesE", "clang::Parser::ParseDefaultStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser22ParseCompoundStatementERNS_16ParsedAttributesEb", "clang::Parser::ParseCompoundStatement(clang::ParsedAttributes&, bool)"},
    {"_ZN5clang6Parser16ParseIfStatementERNS_16ParsedAttributesE", "clang::Parser::ParseIfStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser20ParseSwitchStatementERNS_16ParsedAttributesE", "clang::Parser::ParseSwitchStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser19ParseWhileStatementERNS_16ParsedAttributesE", "clang::Parser::ParseWhileStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser16ParseDoStatementERNS_16ParsedAttributesE", "clang::Parser::ParseDoStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser17ParseForStatementERNS_16ParsedAttributesE", "clang::Parser::ParseForStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser18ParseGotoStatementERNS_16ParsedAttributesE", "clang::Parser::ParseGotoStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser22ParseContinueStatementERNS_16ParsedAttributesE", "clang::Parser::ParseContinueStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser19ParseBreakStatementERNS_16ParsedAttributesE", "clang::Parser::ParseBreakStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser20ParseReturnStatementERNS_16ParsedAttributesE", "clang::Parser::ParseReturnStatement(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser17ParseAsmStatementERb", "clang::Parser::ParseAsmStatement(bool&)"},
    {"_ZN5clang6Parser16ParseCXXTryBlockERNS_16ParsedAttributesE", "clang::Parser::ParseCXXTryBlock(clang::ParsedAttributes&)"},
    {"_ZN5clang6Parser26ParseCompoundStatementBodyEb", "clang::Parser::ParseCompoundStatementBody(bool)"},
    {"_ZN5clang6Parser25ParseParenExprOrConditionERNS_12ActionResultIPNS_4ExprELb1EEERPNS_4DeclENS_14SourceLocationEb", "clang::Parser::ParseParenExprOrCondition(clang::ActionResult<clang::Expr*, true>&, clang::Decl*&, clang::SourceLocation, bool)"},
    {"_ZN5clang6Parser31FuzzyParseMicrosoftAsmStatementENS_14SourceLocationE", "clang::Parser::FuzzyParseMicrosoftAsmStatement(clang::SourceLocation)"},
    {"_ZN5clang6Parser19ParseAsmOperandsOptERN4llvm15SmallVectorImplIPNS_14IdentifierInfoEEERNS2_IPNS_4ExprEEESA_", "clang::Parser::ParseAsmOperandsOpt(llvm::SmallVectorImpl<clang::IdentifierInfo*>&, llvm::SmallVectorImpl<clang::Expr*>&, llvm::SmallVectorImpl<clang::Expr*>&)"},
    {"_ZN5clang6Parser26ParseFunctionStatementBodyEPNS_4DeclE", "clang::Parser::ParseFunctionStatementBody(clang::Decl*)"},
    {"_ZN5clang6Parser40trySkippingFunctionBodyForCodeCompletionEv", "clang::Parser::trySkippingFunctionBodyForCodeCompletion()"},
    {"_ZN5clang6Parser21ParseFunctionTryBlockEPNS_4DeclE", "clang::Parser::ParseFunctionTryBlock(clang::Decl*)"},
    {"_ZN5clang6Parser22ParseCXXTryBlockCommonENS_14SourceLocationE", "clang::Parser::ParseCXXTryBlockCommon(clang::SourceLocation)"},
    {"_ZN5clang6Parser18ParseCXXCatchBlockEv", "clang::Parser::ParseCXXCatchBlock()"},
    {"_ZN5clang6Parser36ParseDeclarationStartingWithTemplateEjRNS_14SourceLocationENS_15AccessSpecifierE", "clang::Parser::ParseDeclarationStartingWithTemplate(unsigned int, clang::SourceLocation&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser26ParseExplicitInstantiationENS_14SourceLocationES1_RS1_", "clang::Parser::ParseExplicitInstantiation(clang::SourceLocation, clang::SourceLocation, clang::SourceLocation&)"},
    {"_ZN5clang6Parser40ParseTemplateDeclarationOrSpecializationEjRNS_14SourceLocationENS_15AccessSpecifierE", "clang::Parser::ParseTemplateDeclarationOrSpecialization(unsigned int, clang::SourceLocation&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser23ParseTemplateParametersEjRN4llvm15SmallVectorImplIPNS_4DeclEEERNS_14SourceLocationES8_", "clang::Parser::ParseTemplateParameters(unsigned int, llvm::SmallVectorImpl<clang::Decl*>&, clang::SourceLocation&, clang::SourceLocation&)"},
    {"_ZN5clang6Parser35ParseSingleDeclarationAfterTemplateEjRKNS0_18ParsedTemplateInfoERNS0_21ParsingDeclRAIIObjectERNS_14SourceLocationENS_15AccessSpecifierE", "clang::Parser::ParseSingleDeclarationAfterTemplate(unsigned int, clang::Parser::ParsedTemplateInfo const&, clang::Parser::ParsingDeclRAIIObject&, clang::SourceLocation&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser26ParseTemplateParameterListEjRN4llvm15SmallVectorImplIPNS_4DeclEEE", "clang::Parser::ParseTemplateParameterList(unsigned int, llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang6Parser22ParseTemplateParameterEjj", "clang::Parser::ParseTemplateParameter(unsigned int, unsigned int)"},
    {"_ZN5clang6Parser30isStartOfTemplateTypeParameterEv", "clang::Parser::isStartOfTemplateTypeParameter()"},
    {"_ZN5clang6Parser18ParseTypeParameterEjj", "clang::Parser::ParseTypeParameter(unsigned int, unsigned int)"},
    {"_ZN5clang6Parser30ParseTemplateTemplateParameterEjj", "clang::Parser::ParseTemplateTemplateParameter(unsigned int, unsigned int)"},
    {"_ZN5clang6Parser29ParseNonTypeTemplateParameterEjj", "clang::Parser::ParseNonTypeTemplateParameter(unsigned int, unsigned int)"},
    {"_ZN5clang6Parser29ParseTemplateTemplateArgumentEv", "clang::Parser::ParseTemplateTemplateArgument()"},
    {"_ZN5clang6Parser32ParseTemplateIdAfterTemplateNameENS_9OpaquePtrINS_12TemplateNameEEENS_14SourceLocationEPKNS_12CXXScopeSpecEbRS4_RN4llvm11SmallVectorINS_22ParsedTemplateArgumentELj16EEES8_", "clang::Parser::ParseTemplateIdAfterTemplateName(clang::OpaquePtr<clang::TemplateName>, clang::SourceLocation, clang::CXXScopeSpec const*, bool, clang::SourceLocation&, llvm::SmallVector<clang::ParsedTemplateArgument, 16u>&, clang::SourceLocation&)"},
    {"_ZN5clang6Parser25ParseTemplateArgumentListERN4llvm11SmallVectorINS_22ParsedTemplateArgumentELj16EEE", "clang::Parser::ParseTemplateArgumentList(llvm::SmallVector<clang::ParsedTemplateArgument, 16u>&)"},
    {"_ZN5clang6Parser23AnnotateTemplateIdTokenENS_9OpaquePtrINS_12TemplateNameEEENS_16TemplateNameKindEPKNS_12CXXScopeSpecERNS_13UnqualifiedIdENS_14SourceLocationEb", "clang::Parser::AnnotateTemplateIdToken(clang::OpaquePtr<clang::TemplateName>, clang::TemplateNameKind, clang::CXXScopeSpec const*, clang::UnqualifiedId&, clang::SourceLocation, bool)"},
    {"_ZN5clang6Parser29AnnotateTemplateIdTokenAsTypeEPKNS_12CXXScopeSpecE", "clang::Parser::AnnotateTemplateIdTokenAsType(clang::CXXScopeSpec const*)"},
    {"_ZN5clang6Parser21ParseTemplateArgumentEv", "clang::Parser::ParseTemplateArgument()"},
    {"_ZN5clang6Parser22IsTemplateArgumentListEj", "clang::Parser::IsTemplateArgumentList(unsigned int)"},
    {"_ZNK5clang6Parser18ParsedTemplateInfo14getSourceRangeEv", "clang::Parser::ParsedTemplateInfo::getSourceRange() const"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang22ParsedTemplateArgumentELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ParsedTemplateArgument, false>::grow(unsigned long)"},
    {"_ZN5clang6Parser25isCXXDeclarationStatementEv", "clang::Parser::isCXXDeclarationStatement()"},
    {"_ZN5clang6Parser22isCXXSimpleDeclarationEv", "clang::Parser::isCXXSimpleDeclaration()"},
    {"_ZN5clang6Parser25isCXXDeclarationSpecifierEv", "clang::Parser::isCXXDeclarationSpecifier()"},
    {"_ZN5clang6Parser25TryParseSimpleDeclarationEv", "clang::Parser::TryParseSimpleDeclaration()"},
    {"_ZN5clang6Parser23TryParseTypeofSpecifierEv", "clang::Parser::TryParseTypeofSpecifier()"},
    {"_ZN5clang6Parser26TryParseProtocolQualifiersEv", "clang::Parser::TryParseProtocolQualifiers()"},
    {"_ZN5clang6Parser26TryParseInitDeclaratorListEv", "clang::Parser::TryParseInitDeclaratorList()"},
    {"_ZN5clang6Parser18TryParseDeclaratorEbb", "clang::Parser::TryParseDeclarator(bool, bool)"},
    {"_ZN5clang6Parser25isCXXConditionDeclarationEv", "clang::Parser::isCXXConditionDeclaration()"},
    {"_ZN5clang6Parser11isCXXTypeIdENS0_25TentativeCXXTypeIdContextERb", "clang::Parser::isCXXTypeId(clang::Parser::TentativeCXXTypeIdContext, bool&)"},
    {"_ZN5clang6Parser25isCXX0XAttributeSpecifierEbPNS_3tok9TokenKindE", "clang::Parser::isCXX0XAttributeSpecifier(bool, clang::tok::TokenKind*)"},
    {"_ZN5clang6Parser26TryParseFunctionDeclaratorEv", "clang::Parser::TryParseFunctionDeclarator()"},
    {"_ZN5clang6Parser23isCXXFunctionDeclaratorEb", "clang::Parser::isCXXFunctionDeclarator(bool)"},
    {"_ZN5clang6Parser25TryParseBracketDeclaratorEv", "clang::Parser::TryParseBracketDeclarator()"},
    {"_ZN5clang6Parser33isExpressionOrTypeSpecifierSimpleENS_3tok9TokenKindE", "clang::Parser::isExpressionOrTypeSpecifierSimple(clang::tok::TokenKind)"},
    {"_ZN5clang6Parser28TryParseDeclarationSpecifierEv", "clang::Parser::TryParseDeclarationSpecifier()"},
    {"_ZN5clang6Parser34TryParseParameterDeclarationClauseEv", "clang::Parser::TryParseParameterDeclarationClause()"},
    {"_ZN5clang6ParserC1ERNS_12PreprocessorERNS_4SemaE", "clang::Parser::Parser(clang::Preprocessor&, clang::Sema&)"},
    {"_ZN5clang6ParserC2ERNS_12PreprocessorERNS_4SemaE", "clang::Parser::Parser(clang::Preprocessor&, clang::Sema&)"},
    {"_ZNK5clang27PrettyStackTraceParserEntry5printERN4llvm11raw_ostreamE", "clang::PrettyStackTraceParserEntry::print(llvm::raw_ostream&) const"},
    {"_ZN5clang6Parser4DiagENS_14SourceLocationEj", "clang::Parser::Diag(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang6Parser4DiagERKNS_5TokenEj", "clang::Parser::Diag(clang::Token const&, unsigned int)"},
    {"_ZN5clang6Parser18SuggestParenthesesENS_14SourceLocationEjNS_11SourceRangeE", "clang::Parser::SuggestParentheses(clang::SourceLocation, unsigned int, clang::SourceRange)"},
    {"_ZN5clang6Parser19MatchRHSPunctuationENS_3tok9TokenKindENS_14SourceLocationE", "clang::Parser::MatchRHSPunctuation(clang::tok::TokenKind, clang::SourceLocation)"},
    {"_ZN5clang6Parser16ExpectAndConsumeENS_3tok9TokenKindEjPKcS2_", "clang::Parser::ExpectAndConsume(clang::tok::TokenKind, unsigned int, char const*, clang::tok::TokenKind)"},
    {"_ZN5clang6Parser20ExpectAndConsumeSemiEj", "clang::Parser::ExpectAndConsumeSemi(unsigned int)"},
    {"_ZN5clang6Parser9SkipUntilEPKNS_3tok9TokenKindEjbbb", "clang::Parser::SkipUntil(clang::tok::TokenKind const*, unsigned int, bool, bool, bool)"},
    {"_ZN5clang6Parser10EnterScopeEj", "clang::Parser::EnterScope(unsigned int)"},
    {"_ZN5clang6Parser9ExitScopeEv", "clang::Parser::ExitScope()"},
    {"_ZN5clang6ParserD0Ev", "clang::Parser::~Parser()"},
    {"_ZN5clang6ParserD1Ev", "clang::Parser::~Parser()"},
    {"_ZN5clang6ParserD2Ev", "clang::Parser::~Parser()"},
    {"_ZN5clang6Parser10InitializeEv", "clang::Parser::Initialize()"},
    {"_ZN5clang6Parser17ParseTopLevelDeclERNS_9OpaquePtrINS_12DeclGroupRefEEE", "clang::Parser::ParseTopLevelDecl(clang::OpaquePtr<clang::DeclGroupRef>&)"},
    {"_ZN5clang6Parser24ParseExternalDeclarationERNS0_25ParsedAttributesWithRangeEPNS0_15ParsingDeclSpecE", "clang::Parser::ParseExternalDeclaration(clang::Parser::ParsedAttributesWithRange&, clang::Parser::ParsingDeclSpec*)"},
    {"_ZN5clang6Parser20ParseTranslationUnitEv", "clang::Parser::ParseTranslationUnit()"},
    {"_ZN5clang6Parser14ParseSimpleAsmEPNS_14SourceLocationE", "clang::Parser::ParseSimpleAsm(clang::SourceLocation*)"},
    {"_ZN5clang6Parser36ParseDeclarationOrFunctionDefinitionERNS0_15ParsingDeclSpecENS_15AccessSpecifierE", "clang::Parser::ParseDeclarationOrFunctionDefinition(clang::Parser::ParsingDeclSpec&, clang::AccessSpecifier)"},
    {"_ZN5clang6Parser36ParseDeclarationOrFunctionDefinitionERNS_16ParsedAttributesENS_15AccessSpecifierE", "clang::Parser::ParseDeclarationOrFunctionDefinition(clang::ParsedAttributes&, clang::AccessSpecifier)"},
    {"_ZNK5clang6Parser28isDeclarationAfterDeclaratorEv", "clang::Parser::isDeclarationAfterDeclarator() const"},
    {"_ZN5clang6Parser27isStartOfFunctionDefinitionERKNS0_17ParsingDeclaratorE", "clang::Parser::isStartOfFunctionDefinition(clang::Parser::ParsingDeclarator const&)"},
    {"_ZN5clang6Parser23ParseFunctionDefinitionERNS0_17ParsingDeclaratorERKNS0_18ParsedTemplateInfoE", "clang::Parser::ParseFunctionDefinition(clang::Parser::ParsingDeclarator&, clang::Parser::ParsedTemplateInfo const&)"},
    {"_ZN5clang6Parser25ParseKNRParamDeclarationsERNS_10DeclaratorE", "clang::Parser::ParseKNRParamDeclarations(clang::Declarator&)"},
    {"_ZN5clang6Parser21ParseAsmStringLiteralEv", "clang::Parser::ParseAsmStringLiteral()"},
    {"_ZN5clang6Parser27TryAnnotateTypeOrScopeTokenEb", "clang::Parser::TryAnnotateTypeOrScopeToken(bool)"},
    {"_ZN5clang6Parser24TryAnnotateCXXScopeTokenEb", "clang::Parser::TryAnnotateCXXScopeToken(bool)"},
    {"_ZN5clang6Parser32isTokenEqualOrMistypedEqualEqualEj", "clang::Parser::isTokenEqualOrMistypedEqualEqual(unsigned int)"},
    {"_ZN5clang6Parser22CodeCompletionRecoveryEv", "clang::Parser::CodeCompletionRecovery()"},
    {"_ZN5clang6Parser13FieldCallback7_anchorEv", "clang::Parser::FieldCallback::_anchor()"},
    {"_ZN5clang6Parser21CodeCompleteDirectiveEb", "clang::Parser::CodeCompleteDirective(bool)"},
    {"_ZN5clang6Parser34CodeCompleteInConditionalExclusionEv", "clang::Parser::CodeCompleteInConditionalExclusion()"},
    {"_ZN5clang6Parser21CodeCompleteMacroNameEb", "clang::Parser::CodeCompleteMacroName(bool)"},
    {"_ZN5clang6Parser34CodeCompletePreprocessorExpressionEv", "clang::Parser::CodeCompletePreprocessorExpression()"},
    {"_ZN5clang6Parser25CodeCompleteMacroArgumentEPNS_14IdentifierInfoEPNS_9MacroInfoEj", "clang::Parser::CodeCompleteMacroArgument(clang::IdentifierInfo*, clang::MacroInfo*, unsigned int)"},
    {"_ZN5clang6Parser27CodeCompleteNaturalLanguageEv", "clang::Parser::CodeCompleteNaturalLanguage()"},
    {"_ZN5clang6Parser15ConsumeAnyTokenEv", "clang::Parser::ConsumeAnyToken()"},
    {"_ZN5clang5Scope4InitEPS0_j", "clang::Scope::Init(clang::Scope*, unsigned int)"},
    {"_ZN5clang27PrettyStackTraceParserEntryD1Ev", "clang::PrettyStackTraceParserEntry::~PrettyStackTraceParserEntry()"},
    {"_ZN5clang27PrettyStackTraceParserEntryD0Ev", "clang::PrettyStackTraceParserEntry::~PrettyStackTraceParserEntry()"},
    {"_ZN5clang6Parser13FieldCallbackD1Ev", "clang::Parser::FieldCallback::~FieldCallback()"},
    {"_ZN5clang6Parser13FieldCallbackD0Ev", "clang::Parser::FieldCallback::~FieldCallback()"},
    {"_ZNSt11_Deque_baseIPN5clang6Parser12ParsingClassESaIS3_EE17_M_initialize_mapEm", "std::_Deque_base<clang::Parser::ParsingClass*, std::allocator<clang::Parser::ParsingClass*> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeIPN5clang6Parser12ParsingClassESaIS3_EEC2ERKS5_", "std::deque<clang::Parser::ParsingClass*, std::allocator<clang::Parser::ParsingClass*> >::deque(std::deque<clang::Parser::ParsingClass*, std::allocator<clang::Parser::ParsingClass*> > const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang5TokenEE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<clang::Token>::insert(clang::Token*, clang::Token const&)"},
    {"_ZN5clang4sema21AnalysisBasedWarnings6PolicyC1Ev", "clang::sema::AnalysisBasedWarnings::Policy::Policy()"},
    {"_ZN5clang4sema21AnalysisBasedWarnings6PolicyC2Ev", "clang::sema::AnalysisBasedWarnings::Policy::Policy()"},
    {"_ZN5clang4sema21AnalysisBasedWarningsC1ERNS_4SemaE", "clang::sema::AnalysisBasedWarnings::AnalysisBasedWarnings(clang::Sema&)"},
    {"_ZN5clang4sema21AnalysisBasedWarningsC2ERNS_4SemaE", "clang::sema::AnalysisBasedWarnings::AnalysisBasedWarnings(clang::Sema&)"},
    {"_ZN5clang4sema21AnalysisBasedWarnings13IssueWarningsENS1_6PolicyEPKNS_4DeclENS_8QualTypeE", "clang::sema::AnalysisBasedWarnings::IssueWarnings(clang::sema::AnalysisBasedWarnings::Policy, clang::Decl const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_122UninitValsDiagReporterD1Ev", "(anonymous namespace)::UninitValsDiagReporter::~UninitValsDiagReporter()"},
    {"_ZN5clang4sema21AnalysisBasedWarnings13IssueWarningsENS1_6PolicyEPKNS_9BlockExprE", "clang::sema::AnalysisBasedWarnings::IssueWarnings(clang::sema::AnalysisBasedWarnings::Policy, clang::BlockExpr const*)"},
    {"_ZN5clang4sema21AnalysisBasedWarnings13IssueWarningsENS1_6PolicyEPKNS_14ObjCMethodDeclE", "clang::sema::AnalysisBasedWarnings::IssueWarnings(clang::sema::AnalysisBasedWarnings::Policy, clang::ObjCMethodDecl const*)"},
    {"_ZN5clang4sema21AnalysisBasedWarnings13IssueWarningsENS1_6PolicyEPKNS_12FunctionDeclE", "clang::sema::AnalysisBasedWarnings::IssueWarnings(clang::sema::AnalysisBasedWarnings::Policy, clang::FunctionDecl const*)"},
    {"_ZN12_GLOBAL__N_122UninitValsDiagReporterD0Ev", "(anonymous namespace)::UninitValsDiagReporter::~UninitValsDiagReporter()"},
    {"_ZN12_GLOBAL__N_122UninitValsDiagReporter25handleUseOfUninitVariableEPKN5clang4ExprEPKNS1_7VarDeclE", "(anonymous namespace)::UninitValsDiagReporter::handleUseOfUninitVariable(clang::Expr const*, clang::VarDecl const*)"},
    {"_ZSt16__introsort_loopIPPKN5clang4ExprElN12_GLOBAL__N_18SLocSortEEvT_S7_T0_T1_", "void std::__introsort_loop<clang::Expr const**, long, (anonymous namespace)::SLocSort>(clang::Expr const**, clang::Expr const**, long, (anonymous namespace)::SLocSort)"},
    {"_ZSt16__insertion_sortIPPKN5clang4ExprEN12_GLOBAL__N_18SLocSortEEvT_S7_T0_", "void std::__insertion_sort<clang::Expr const**, (anonymous namespace)::SLocSort>(clang::Expr const**, clang::Expr const**, (anonymous namespace)::SLocSort)"},
    {"_ZSt25__unguarded_linear_insertIPPKN5clang4ExprES3_N12_GLOBAL__N_18SLocSortEEvT_T0_T1_", "void std::__unguarded_linear_insert<clang::Expr const**, clang::Expr const*, (anonymous namespace)::SLocSort>(clang::Expr const**, clang::Expr const*, (anonymous namespace)::SLocSort)"},
    {"_ZSt13__adjust_heapIPPKN5clang4ExprElS3_N12_GLOBAL__N_18SLocSortEEvT_T0_S8_T1_T2_", "void std::__adjust_heap<clang::Expr const**, long, clang::Expr const*, (anonymous namespace)::SLocSort>(clang::Expr const**, long, long, clang::Expr const*, (anonymous namespace)::SLocSort)"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeHandlerD1Ev", "(anonymous namespace)::UnreachableCodeHandler::~UnreachableCodeHandler()"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeHandlerD0Ev", "(anonymous namespace)::UnreachableCodeHandler::~UnreachableCodeHandler()"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeHandler17HandleUnreachableEN5clang14SourceLocationENS1_11SourceRangeES3_", "(anonymous namespace)::UnreachableCodeHandler::HandleUnreachable(clang::SourceLocation, clang::SourceRange, clang::SourceRange)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS_11SmallVectorIPKNS1_4ExprELj2EEENS_12DenseMapInfoIS4_EENSB_ISA_EEE16InsertIntoBucketERKS4_RKSA_PSt4pairIS4_SA_E", "llvm::DenseMap<clang::VarDecl const*, llvm::SmallVector<clang::Expr const*, 2u>*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::Expr const*, 2u>*> >::InsertIntoBucket(clang::VarDecl const* const&, llvm::SmallVector<clang::Expr const*, 2u>* const&, std::pair<clang::VarDecl const*, llvm::SmallVector<clang::Expr const*, 2u>*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS_11SmallVectorIPKNS1_4ExprELj2EEENS_12DenseMapInfoIS4_EENSB_ISA_EEE4growEj", "llvm::DenseMap<clang::VarDecl const*, llvm::SmallVector<clang::Expr const*, 2u>*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::Expr const*, 2u>*> >::grow(unsigned int)"},
    {"_ZN5clang22UninitVariablesHandler25handleUseOfUninitVariableEPKNS_4ExprEPKNS_7VarDeclE", "clang::UninitVariablesHandler::handleUseOfUninitVariable(clang::Expr const*, clang::VarDecl const*)"},
    {"_ZN5clang14reachable_code8CallbackD1Ev", "clang::reachable_code::Callback::~Callback()"},
    {"_ZN5clang14reachable_code8CallbackD0Ev", "clang::reachable_code::Callback::~Callback()"},
    {"_ZSt6__findIPKPN5clang8CFGBlockES2_ET_S5_S5_RKT0_St26random_access_iterator_tag", "clang::CFGBlock* const* std::__find<clang::CFGBlock* const*, clang::CFGBlock*>(clang::CFGBlock* const*, clang::CFGBlock* const*, clang::CFGBlock* const&, std::random_access_iterator_tag)"},
    {"_ZN5clang13AttributeListC1ERN4llvm16BumpPtrAllocatorEPNS_14IdentifierInfoENS_14SourceLocationES5_S6_S5_S6_PPNS_4ExprEjbb", "clang::AttributeList::AttributeList(llvm::BumpPtrAllocator&, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Expr**, unsigned int, bool, bool)"},
    {"_ZN5clang13AttributeListC2ERN4llvm16BumpPtrAllocatorEPNS_14IdentifierInfoENS_14SourceLocationES5_S6_S5_S6_PPNS_4ExprEjbb", "clang::AttributeList::AttributeList(llvm::BumpPtrAllocator&, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Expr**, unsigned int, bool, bool)"},
    {"_ZN5clang13AttributeList7getKindEPKNS_14IdentifierInfoE", "clang::AttributeList::getKind(clang::IdentifierInfo const*)"},
    {"_ZNK5clang21CodeCompletionContext22wantConstructorResultsEv", "clang::CodeCompletionContext::wantConstructorResults() const"},
    {"_ZN5clang20CodeCompletionString5ChunkC1ENS0_9ChunkKindEPKc", "clang::CodeCompletionString::Chunk::Chunk(clang::CodeCompletionString::ChunkKind, char const*)"},
    {"_ZN5clang20CodeCompletionString5ChunkC2ENS0_9ChunkKindEPKc", "clang::CodeCompletionString::Chunk::Chunk(clang::CodeCompletionString::ChunkKind, char const*)"},
    {"_ZN5clang20CodeCompletionString5Chunk10CreateTextEPKc", "clang::CodeCompletionString::Chunk::CreateText(char const*)"},
    {"_ZN5clang20CodeCompletionString5Chunk14CreateOptionalEPS0_", "clang::CodeCompletionString::Chunk::CreateOptional(clang::CodeCompletionString*)"},
    {"_ZN5clang20CodeCompletionString5Chunk17CreatePlaceholderEPKc", "clang::CodeCompletionString::Chunk::CreatePlaceholder(char const*)"},
    {"_ZN5clang20CodeCompletionString5Chunk17CreateInformativeEPKc", "clang::CodeCompletionString::Chunk::CreateInformative(char const*)"},
    {"_ZN5clang20CodeCompletionString5Chunk16CreateResultTypeEPKc", "clang::CodeCompletionString::Chunk::CreateResultType(char const*)"},
    {"_ZN5clang20CodeCompletionString5Chunk22CreateCurrentParameterEPKc", "clang::CodeCompletionString::Chunk::CreateCurrentParameter(char const*)"},
    {"_ZN5clang20CodeCompletionStringC1EPKNS0_5ChunkEjj18CXAvailabilityKind", "clang::CodeCompletionString::CodeCompletionString(clang::CodeCompletionString::Chunk const*, unsigned int, unsigned int, CXAvailabilityKind)"},
    {"_ZN5clang20CodeCompletionStringC2EPKNS0_5ChunkEjj18CXAvailabilityKind", "clang::CodeCompletionString::CodeCompletionString(clang::CodeCompletionString::Chunk const*, unsigned int, unsigned int, CXAvailabilityKind)"},
    {"_ZNK5clang20CodeCompletionString11getAsStringEv", "clang::CodeCompletionString::getAsString() const"},
    {"_ZNK5clang20CodeCompletionString12getTypedTextEv", "clang::CodeCompletionString::getTypedText() const"},
    {"_ZN5clang23CodeCompletionAllocator10CopyStringEN4llvm9StringRefE", "clang::CodeCompletionAllocator::CopyString(llvm::StringRef)"},
    {"_ZN5clang21CodeCompletionBuilder10TakeStringEv", "clang::CodeCompletionBuilder::TakeString()"},
    {"_ZN5clang20CodeCompletionResult19getPriorityFromDeclEPNS_9NamedDeclE", "clang::CodeCompletionResult::getPriorityFromDecl(clang::NamedDecl*)"},
    {"_ZNK5clang20CodeCompleteConsumer17OverloadCandidate11getFunctionEv", "clang::CodeCompleteConsumer::OverloadCandidate::getFunction() const"},
    {"_ZNK5clang20CodeCompleteConsumer17OverloadCandidate15getFunctionTypeEv", "clang::CodeCompleteConsumer::OverloadCandidate::getFunctionType() const"},
    {"_ZN5clang20CodeCompleteConsumerD0Ev", "clang::CodeCompleteConsumer::~CodeCompleteConsumer()"},
    {"_ZN5clang20CodeCompleteConsumerD1Ev", "clang::CodeCompleteConsumer::~CodeCompleteConsumer()"},
    {"_ZN5clang20CodeCompleteConsumerD2Ev", "clang::CodeCompleteConsumer::~CodeCompleteConsumer()"},
    {"_ZN5clang28PrintingCodeCompleteConsumer26ProcessCodeCompleteResultsERNS_4SemaENS_21CodeCompletionContextEPNS_20CodeCompletionResultEj", "clang::PrintingCodeCompleteConsumer::ProcessCodeCompleteResults(clang::Sema&, clang::CodeCompletionContext, clang::CodeCompletionResult*, unsigned int)"},
    {"_ZN5clang28PrintingCodeCompleteConsumer25ProcessOverloadCandidatesERNS_4SemaEjPNS_20CodeCompleteConsumer17OverloadCandidateEj", "clang::PrintingCodeCompleteConsumer::ProcessOverloadCandidates(clang::Sema&, unsigned int, clang::CodeCompleteConsumer::OverloadCandidate*, unsigned int)"},
    {"_ZN5clang20CodeCompletionResult32computeCursorKindAndAvailabilityEv", "clang::CodeCompletionResult::computeCursorKindAndAvailability()"},
    {"_ZN5clangltERKNS_20CodeCompletionResultES2_", "clang::operator<(clang::CodeCompletionResult const&, clang::CodeCompletionResult const&)"},
    {"_Z14getOrderedNameRKN5clang20CodeCompletionResultERSs", "getOrderedName(clang::CodeCompletionResult const&, std::string&)"},
    {"_ZNK5clang4Decl7getAttrINS_15UnavailableAttrEEEPT_v", "clang::UnavailableAttr* clang::Decl::getAttr<clang::UnavailableAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_14DeprecatedAttrEEEPT_v", "clang::DeprecatedAttr* clang::Decl::getAttr<clang::DeprecatedAttr>() const"},
    {"_ZN5clang28PrintingCodeCompleteConsumerD1Ev", "clang::PrintingCodeCompleteConsumer::~PrintingCodeCompleteConsumer()"},
    {"_ZN5clang28PrintingCodeCompleteConsumerD0Ev", "clang::PrintingCodeCompleteConsumer::~PrintingCodeCompleteConsumer()"},
    {"_ZSt21__inplace_stable_sortIPN5clang20CodeCompletionResultEEvT_S3_", "void std::__inplace_stable_sort<clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZSt22__stable_sort_adaptiveIPN5clang20CodeCompletionResultES2_lEvT_S3_T0_T1_", "void std::__stable_sort_adaptive<clang::CodeCompletionResult*, clang::CodeCompletionResult*, long>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, long)"},
    {"_ZSt24__merge_sort_with_bufferIPN5clang20CodeCompletionResultES2_EvT_S3_T0_", "void std::__merge_sort_with_buffer<clang::CodeCompletionResult*, clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZSt16__merge_adaptiveIPN5clang20CodeCompletionResultElS2_EvT_S3_S3_T0_S4_T1_S4_", "void std::__merge_adaptive<clang::CodeCompletionResult*, long, clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, long, long, clang::CodeCompletionResult*, long)"},
    {"_ZSt5mergeIPN5clang20CodeCompletionResultES2_S2_ET1_T_S4_T0_S5_S3_", "clang::CodeCompletionResult* std::merge<clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZSt16__merge_backwardIPN5clang20CodeCompletionResultES2_S2_ET1_T_S4_T0_S5_S3_", "clang::CodeCompletionResult* std::__merge_backward<clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZSt17__rotate_adaptiveIPN5clang20CodeCompletionResultES2_lET_S3_S3_S3_T1_S4_T0_S4_", "clang::CodeCompletionResult* std::__rotate_adaptive<clang::CodeCompletionResult*, clang::CodeCompletionResult*, long>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, long, long, clang::CodeCompletionResult*, long)"},
    {"_ZSt8__rotateIPN5clang20CodeCompletionResultEEvT_S3_S3_St26random_access_iterator_tag", "void std::__rotate<clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, std::random_access_iterator_tag)"},
    {"_ZSt16__insertion_sortIPN5clang20CodeCompletionResultEEvT_S3_", "void std::__insertion_sort<clang::CodeCompletionResult*>(clang::CodeCompletionResult*, clang::CodeCompletionResult*)"},
    {"_ZSt22__merge_without_bufferIPN5clang20CodeCompletionResultElEvT_S3_S3_T0_S4_", "void std::__merge_without_buffer<clang::CodeCompletionResult*, long>(clang::CodeCompletionResult*, clang::CodeCompletionResult*, clang::CodeCompletionResult*, long, long)"},
    {"_ZN5clang13UnqualifiedId13setTemplateIdEPNS_20TemplateIdAnnotationE", "clang::UnqualifiedId::setTemplateId(clang::TemplateIdAnnotation*)"},
    {"_ZN5clang13UnqualifiedId24setConstructorTemplateIdEPNS_20TemplateIdAnnotationE", "clang::UnqualifiedId::setConstructorTemplateId(clang::TemplateIdAnnotation*)"},
    {"_ZN5clang15DeclaratorChunk11getFunctionERKNS_16ParsedAttributesEbbNS_14SourceLocationEPNS0_9ParamInfoEjjbS4_bS4_bPNS_9OpaquePtrINS_8QualTypeEEEPNS_11SourceRangeEjS4_S4_RNS_10DeclaratorES9_", "clang::DeclaratorChunk::getFunction(clang::ParsedAttributes const&, bool, bool, clang::SourceLocation, clang::DeclaratorChunk::ParamInfo*, unsigned int, unsigned int, bool, clang::SourceLocation, bool, clang::SourceLocation, bool, clang::OpaquePtr<clang::QualType>*, clang::SourceRange*, unsigned int, clang::SourceLocation, clang::SourceLocation, clang::Declarator&, clang::OpaquePtr<clang::QualType>)"},
    {"_ZNK5clang8DeclSpec19getParsedSpecifiersEv", "clang::DeclSpec::getParsedSpecifiers() const"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS0_3SCSE", "clang::DeclSpec::getSpecifierName(clang::DeclSpec::SCS)"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS_18TypeSpecifierWidthE", "clang::DeclSpec::getSpecifierName(clang::TypeSpecifierWidth)"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS0_3TSCE", "clang::DeclSpec::getSpecifierName(clang::DeclSpec::TSC)"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS_17TypeSpecifierSignE", "clang::DeclSpec::getSpecifierName(clang::TypeSpecifierSign)"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS_17TypeSpecifierTypeE", "clang::DeclSpec::getSpecifierName(clang::TypeSpecifierType)"},
    {"_ZN5clang8DeclSpec16getSpecifierNameENS0_2TQE", "clang::DeclSpec::getSpecifierName(clang::DeclSpec::TQ)"},
    {"_ZN5clang8DeclSpec19SetStorageClassSpecENS0_3SCSENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetStorageClassSpec(clang::DeclSpec::SCS, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec25SetStorageClassSpecThreadENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetStorageClassSpecThread(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec16SetTypeSpecWidthENS_18TypeSpecifierWidthENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeSpecWidth(clang::TypeSpecifierWidth, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec18SetTypeSpecComplexENS0_3TSCENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeSpecComplex(clang::DeclSpec::TSC, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec15SetTypeSpecSignENS_17TypeSpecifierSignENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeSpecSign(clang::TypeSpecifierSign, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec15SetTypeSpecTypeENS_17TypeSpecifierTypeENS_14SourceLocationERPKcRjNS_9OpaquePtrINS_8QualTypeEEE", "clang::DeclSpec::SetTypeSpecType(clang::TypeSpecifierType, clang::SourceLocation, char const*&, unsigned int&, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang8DeclSpec15SetTypeSpecTypeENS_17TypeSpecifierTypeENS_14SourceLocationERPKcRjPNS_4ExprE", "clang::DeclSpec::SetTypeSpecType(clang::TypeSpecifierType, clang::SourceLocation, char const*&, unsigned int&, clang::Expr*)"},
    {"_ZN5clang8DeclSpec15SetTypeSpecTypeENS_17TypeSpecifierTypeENS_14SourceLocationERPKcRjPNS_4DeclEb", "clang::DeclSpec::SetTypeSpecType(clang::TypeSpecifierType, clang::SourceLocation, char const*&, unsigned int&, clang::Decl*, bool)"},
    {"_ZN5clang8DeclSpec15SetTypeSpecTypeENS_17TypeSpecifierTypeENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeSpecType(clang::TypeSpecifierType, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec20SetTypeAltiVecVectorEbNS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeAltiVecVector(bool, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec19SetTypeAltiVecPixelEbNS_14SourceLocationERPKcRj", "clang::DeclSpec::SetTypeAltiVecPixel(bool, clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec16SetTypeSpecErrorEv", "clang::DeclSpec::SetTypeSpecError()"},
    {"_ZN5clang8DeclSpec11SetTypeQualENS0_2TQENS_14SourceLocationERPKcRjRKNS_11LangOptionsE", "clang::DeclSpec::SetTypeQual(clang::DeclSpec::TQ, clang::SourceLocation, char const*&, unsigned int&, clang::LangOptions const&)"},
    {"_ZN5clang8DeclSpec21SetFunctionSpecInlineENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetFunctionSpecInline(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec22SetFunctionSpecVirtualENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetFunctionSpecVirtual(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec23SetFunctionSpecExplicitENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetFunctionSpecExplicit(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec13SetFriendSpecENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetFriendSpec(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec16SetConstexprSpecENS_14SourceLocationERPKcRj", "clang::DeclSpec::SetConstexprSpec(clang::SourceLocation, char const*&, unsigned int&)"},
    {"_ZN5clang8DeclSpec21setProtocolQualifiersEPKPNS_4DeclEjPNS_14SourceLocationES5_", "clang::DeclSpec::setProtocolQualifiers(clang::Decl* const*, unsigned int, clang::SourceLocation*, clang::SourceLocation)"},
    {"_ZN5clang8DeclSpec23SaveWrittenBuiltinSpecsEv", "clang::DeclSpec::SaveWrittenBuiltinSpecs()"},
    {"_ZN5clang8DeclSpec29SaveStorageSpecifierAsWrittenEv", "clang::DeclSpec::SaveStorageSpecifierAsWritten()"},
    {"_ZN5clang8DeclSpec6FinishERNS_10DiagnosticERNS_12PreprocessorE", "clang::DeclSpec::Finish(clang::Diagnostic&, clang::Preprocessor&)"},
    {"_ZN5clang8DeclSpec21isMissingDeclaratorOkEv", "clang::DeclSpec::isMissingDeclaratorOk()"},
    {"_ZN5clang13UnqualifiedId5clearEv", "clang::UnqualifiedId::clear()"},
    {"_ZN5clang13UnqualifiedId21setOperatorFunctionIdENS_14SourceLocationENS_22OverloadedOperatorKindEPS1_", "clang::UnqualifiedId::setOperatorFunctionId(clang::SourceLocation, clang::OverloadedOperatorKind, clang::SourceLocation*)"},
    {"_ZN5clang14VirtSpecifiers12SetSpecifierENS0_9SpecifierENS_14SourceLocationERPKc", "clang::VirtSpecifiers::SetSpecifier(clang::VirtSpecifiers::Specifier, clang::SourceLocation, char const*&)"},
    {"_ZN5clang14VirtSpecifiers16getSpecifierNameENS0_9SpecifierE", "clang::VirtSpecifiers::getSpecifierName(clang::VirtSpecifiers::Specifier)"},
    {"_ZN5clang19ClassVirtSpecifiers12SetSpecifierENS0_9SpecifierENS_14SourceLocationERPKc", "clang::ClassVirtSpecifiers::SetSpecifier(clang::ClassVirtSpecifiers::Specifier, clang::SourceLocation, char const*&)"},
    {"_ZN5clang19ClassVirtSpecifiers16getSpecifierNameENS0_9SpecifierE", "clang::ClassVirtSpecifiers::getSpecifierName(clang::ClassVirtSpecifiers::Specifier)"},
    {"_ZN5clang18IdentifierResolver10IdDeclInfo10RemoveDeclEPNS_9NamedDeclE", "clang::IdentifierResolver::IdDeclInfo::RemoveDecl(clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolver10IdDeclInfo11ReplaceDeclEPNS_9NamedDeclES3_", "clang::IdentifierResolver::IdDeclInfo::ReplaceDecl(clang::NamedDecl*, clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolverC1ERKNS_11LangOptionsE", "clang::IdentifierResolver::IdentifierResolver(clang::LangOptions const&)"},
    {"_ZN5clang18IdentifierResolverC2ERKNS_11LangOptionsE", "clang::IdentifierResolver::IdentifierResolver(clang::LangOptions const&)"},
    {"_ZN5clang18IdentifierResolverD1Ev", "clang::IdentifierResolver::~IdentifierResolver()"},
    {"_ZN5clang18IdentifierResolverD2Ev", "clang::IdentifierResolver::~IdentifierResolver()"},
    {"_ZNK5clang18IdentifierResolver13isDeclInScopeEPNS_4DeclEPNS_11DeclContextERNS_10ASTContextEPNS_5ScopeE", "clang::IdentifierResolver::isDeclInScope(clang::Decl*, clang::DeclContext*, clang::ASTContext&, clang::Scope*) const"},
    {"_ZN5clang18IdentifierResolver7AddDeclEPNS_9NamedDeclE", "clang::IdentifierResolver::AddDecl(clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolver13IdDeclInfoMapixENS_15DeclarationNameE", "clang::IdentifierResolver::IdDeclInfoMap::operator[](clang::DeclarationName)"},
    {"_ZN5clang18IdentifierResolver10RemoveDeclEPNS_9NamedDeclE", "clang::IdentifierResolver::RemoveDecl(clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolver11ReplaceDeclEPNS_9NamedDeclES2_", "clang::IdentifierResolver::ReplaceDecl(clang::NamedDecl*, clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolver5beginENS_15DeclarationNameE", "clang::IdentifierResolver::begin(clang::DeclarationName)"},
    {"_ZN5clang18IdentifierResolver24AddDeclToIdentifierChainEPNS_14IdentifierInfoEPNS_9NamedDeclE", "clang::IdentifierResolver::AddDeclToIdentifierChain(clang::IdentifierInfo*, clang::NamedDecl*)"},
    {"_ZN5clang18IdentifierResolver8iterator17incrementSlowCaseEv", "clang::IdentifierResolver::iterator::incrementSlowCase()"},
    {"_ZN5clang4Sema20DiagnoseInvalidJumpsEPNS_4StmtE", "clang::Sema::DiagnoseInvalidJumps(clang::Stmt*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_116JumpScopeChecker9GotoScopeEE9push_backERKS3_", "llvm::SmallVectorImpl<(anonymous namespace)::JumpScopeChecker::GotoScope>::push_back((anonymous namespace)::JumpScopeChecker::GotoScope const&)"},
    {"_ZN12_GLOBAL__N_116JumpScopeChecker21BuildScopeInformationEPN5clang4StmtEj", "(anonymous namespace)::JumpScopeChecker::BuildScopeInformation(clang::Stmt*, unsigned int)"},
    {"_ZN12_GLOBAL__N_116JumpScopeChecker9CheckJumpEPN5clang4StmtES3_NS1_14SourceLocationEj", "(anonymous namespace)::JumpScopeChecker::CheckJump(clang::Stmt*, clang::Stmt*, clang::SourceLocation, unsigned int)"},
    {"_ZN12_GLOBAL__N_116JumpScopeChecker21BuildScopeInformationEPN5clang4DeclERj", "(anonymous namespace)::JumpScopeChecker::BuildScopeInformation(clang::Decl*, unsigned int&)"},
    {"_ZN4llvm8DenseMapIjPN5clang9LabelStmtENS_12DenseMapInfoIjEENS4_IS3_EEE16InsertIntoBucketERKjRKS3_PSt4pairIjS3_E", "llvm::DenseMap<unsigned int, clang::LabelStmt*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::LabelStmt*> >::InsertIntoBucket(unsigned int const&, clang::LabelStmt* const&, std::pair<unsigned int, clang::LabelStmt*>*)"},
    {"_ZN4llvm8DenseMapIjPN5clang9LabelStmtENS_12DenseMapInfoIjEENS4_IS3_EEE4growEj", "llvm::DenseMap<unsigned int, clang::LabelStmt*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::LabelStmt*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjPN5clang16IndirectGotoStmtENS_12DenseMapInfoIjEENS4_IS3_EEE16InsertIntoBucketERKjRKS3_PSt4pairIjS3_E", "llvm::DenseMap<unsigned int, clang::IndirectGotoStmt*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::IndirectGotoStmt*> >::InsertIntoBucket(unsigned int const&, clang::IndirectGotoStmt* const&, std::pair<unsigned int, clang::IndirectGotoStmt*>*)"},
    {"_ZN4llvm8DenseMapIjPN5clang16IndirectGotoStmtENS_12DenseMapInfoIjEENS4_IS3_EEE4growEj", "llvm::DenseMap<unsigned int, clang::IndirectGotoStmt*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<clang::IndirectGotoStmt*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<clang::Stmt*, unsigned int, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::Stmt* const&, unsigned int const&, std::pair<clang::Stmt*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<clang::Stmt*, unsigned int, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang4sema17FunctionScopeInfoD0Ev", "clang::sema::FunctionScopeInfo::~FunctionScopeInfo()"},
    {"_ZN5clang4sema17FunctionScopeInfoD1Ev", "clang::sema::FunctionScopeInfo::~FunctionScopeInfo()"},
    {"_ZN5clang4sema17FunctionScopeInfoD2Ev", "clang::sema::FunctionScopeInfo::~FunctionScopeInfo()"},
    {"_ZN5clang4sema17FunctionScopeInfo5ClearEv", "clang::sema::FunctionScopeInfo::Clear()"},
    {"_ZN5clang4sema14BlockScopeInfoD0Ev", "clang::sema::BlockScopeInfo::~BlockScopeInfo()"},
    {"_ZN5clang4sema14BlockScopeInfoD1Ev", "clang::sema::BlockScopeInfo::~BlockScopeInfo()"},
    {"_ZN5clang4sema14BlockScopeInfoD2Ev", "clang::sema::BlockScopeInfo::~BlockScopeInfo()"},
    {"_ZN5clang4Sema25ActOnTranslationUnitScopeEPNS_5ScopeE", "clang::Sema::ActOnTranslationUnitScope(clang::Scope*)"},
    {"_ZN5clang4SemaC1ERNS_12PreprocessorERNS_10ASTContextERNS_11ASTConsumerEbPNS_20CodeCompleteConsumerE", "clang::Sema::Sema(clang::Preprocessor&, clang::ASTContext&, clang::ASTConsumer&, bool, clang::CodeCompleteConsumer*)"},
    {"_ZN5clang4SemaC2ERNS_12PreprocessorERNS_10ASTContextERNS_11ASTConsumerEbPNS_20CodeCompleteConsumerE", "clang::Sema::Sema(clang::Preprocessor&, clang::ASTContext&, clang::ASTConsumer&, bool, clang::CodeCompleteConsumer*)"},
    {"_ZN5clang4Sema10InitializeEv", "clang::Sema::Initialize()"},
    {"_ZN5clang4SemaD1Ev", "clang::Sema::~Sema()"},
    {"_ZN5clang4SemaD2Ev", "clang::Sema::~Sema()"},
    {"_ZN5clang4Sema17ImpCastExprToTypeERPNS_4ExprENS_8QualTypeENS_8CastKindENS_13ExprValueKindEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "clang::Sema::ImpCastExprToType(clang::Expr*&, clang::QualType, clang::CastKind, clang::ExprValueKind, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*)"},
    {"_ZN5clang4Sema12CastCategoryEPNS_4ExprE", "clang::Sema::CastCategory(clang::Expr*)"},
    {"_ZN5clang4Sema25ActOnEndOfTranslationUnitEv", "clang::Sema::ActOnEndOfTranslationUnit()"},
    {"_Z22ShouldRemoveFromUnusedPN5clang4SemaEPKNS_14DeclaratorDeclE", "ShouldRemoveFromUnused(clang::Sema*, clang::DeclaratorDecl const*)"},
    {"_ZN5clang4Sema4DiagENS_14SourceLocationEj", "clang::Sema::Diag(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang4Sema21SemaDiagnosticBuilderD1Ev", "clang::Sema::SemaDiagnosticBuilder::~SemaDiagnosticBuilder()"},
    {"_ZN5clang4Sema27getFunctionLevelDeclContextEv", "clang::Sema::getFunctionLevelDeclContext()"},
    {"_ZN5clang4Sema18getCurFunctionDeclEv", "clang::Sema::getCurFunctionDecl()"},
    {"_ZN5clang4Sema16getCurMethodDeclEv", "clang::Sema::getCurMethodDecl()"},
    {"_ZN5clang4Sema26getCurFunctionOrMethodDeclEv", "clang::Sema::getCurFunctionOrMethodDecl()"},
    {"_ZN5clang4Sema21SemaDiagnosticBuilderD2Ev", "clang::Sema::SemaDiagnosticBuilder::~SemaDiagnosticBuilder()"},
    {"_ZN5clang4Sema4DiagENS_14SourceLocationERKNS_17PartialDiagnosticE", "clang::Sema::Diag(clang::SourceLocation, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema18getScopeForContextEPNS_11DeclContextE", "clang::Sema::getScopeForContext(clang::DeclContext*)"},
    {"_ZN5clang4Sema17PushFunctionScopeEv", "clang::Sema::PushFunctionScope()"},
    {"_ZN5clang4Sema14PushBlockScopeEPNS_5ScopeEPNS_9BlockDeclE", "clang::Sema::PushBlockScope(clang::Scope*, clang::BlockDecl*)"},
    {"_ZN5clang4Sema23PopFunctionOrBlockScopeEv", "clang::Sema::PopFunctionOrBlockScope()"},
    {"_ZNK5clang4Sema26hasAnyErrorsInThisFunctionEv", "clang::Sema::hasAnyErrorsInThisFunction() const"},
    {"_ZN5clang4Sema11getCurBlockEv", "clang::Sema::getCurBlock()"},
    {"_ZN5clang18ExternalSemaSourceD0Ev", "clang::ExternalSemaSource::~ExternalSemaSource()"},
    {"_ZN5clang18ExternalSemaSourceD1Ev", "clang::ExternalSemaSource::~ExternalSemaSource()"},
    {"_ZN5clang18ExternalSemaSourceD2Ev", "clang::ExternalSemaSource::~ExternalSemaSource()"},
    {"_ZN5clang18ExternalSemaSource14ReadMethodPoolENS_8SelectorE", "clang::ExternalSemaSource::ReadMethodPool(clang::Selector)"},
    {"_ZNK5clang25PrettyDeclStackTraceEntry5printERN4llvm11raw_ostreamE", "clang::PrettyDeclStackTraceEntry::print(llvm::raw_ostream&) const"},
    {"_ZN5clang4sema21TemplateDeductionInfo23addSuppressedDiagnosticENS_14SourceLocationERKNS_17PartialDiagnosticE", "clang::sema::TemplateDeductionInfo::addSuppressedDiagnostic(clang::SourceLocation, clang::PartialDiagnostic const&)"},
    {"_ZNK5clang17PartialDiagnostic4EmitERKNS_17DiagnosticBuilderE", "clang::PartialDiagnostic::Emit(clang::DiagnosticBuilder const&) const"},
    {"_ZN5clang25PrettyDeclStackTraceEntryD1Ev", "clang::PrettyDeclStackTraceEntry::~PrettyDeclStackTraceEntry()"},
    {"_ZN5clang25PrettyDeclStackTraceEntryD0Ev", "clang::PrettyDeclStackTraceEntry::~PrettyDeclStackTraceEntry()"},
    {"_ZSt9__find_ifIPPKN5clang14DeclaratorDeclESt9binder1stISt26pointer_to_binary_functionIPNS0_4SemaES3_bEEET_SB_SB_T0_St26random_access_iterator_tag", "clang::DeclaratorDecl const** std::__find_if<clang::DeclaratorDecl const**, std::binder1st<std::pointer_to_binary_function<clang::Sema*, clang::DeclaratorDecl const*, bool> > >(clang::DeclaratorDecl const**, clang::DeclaratorDecl const**, std::binder1st<std::pointer_to_binary_function<clang::Sema*, clang::DeclaratorDecl const*, bool> >, std::random_access_iterator_tag)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4Sema33ExpressionEvaluationContextRecordELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Sema::ExpressionEvaluationContextRecord, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclENS_11SmallVectorISt4pairINS1_14SourceLocationENS1_17PartialDiagnosticEELj1EEENS_12DenseMapInfoIS3_EENSA_IS9_EEED2Ev", "llvm::DenseMap<clang::Decl*, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> > >::~DenseMap()"},
    {"_ZNSt11_Deque_baseISt4pairIPN5clang9ValueDeclENS1_14SourceLocationEESaIS5_EE17_M_initialize_mapEm", "std::_Deque_base<std::pair<clang::ValueDecl*, clang::SourceLocation>, std::allocator<std::pair<clang::ValueDecl*, clang::SourceLocation> > >::_M_initialize_map(unsigned long)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_9LabelStmtENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<clang::IdentifierInfo*, clang::LabelStmt*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::LabelStmt*> >::shrink_and_clear()"},
    {"_ZN5clang4sema14BlockScopeInfoC2ERNS_10DiagnosticEPNS_5ScopeEPNS_9BlockDeclE", "clang::sema::BlockScopeInfo::BlockScopeInfo(clang::Diagnostic&, clang::Scope*, clang::BlockDecl*)"},
    {"_ZN5clang17PartialDiagnostic11freeStorageEv", "clang::PartialDiagnostic::freeStorage()"},
    {"_ZN5clang17PartialDiagnosticC2ERKNS_14DiagnosticInfoERNS0_16StorageAllocatorE", "clang::PartialDiagnostic::PartialDiagnostic(clang::DiagnosticInfo const&, clang::PartialDiagnostic::StorageAllocator&)"},
    {"_ZNK5clang17PartialDiagnostic9AddStringEN4llvm9StringRefE", "clang::PartialDiagnostic::AddString(llvm::StringRef) const"},
    {"_ZNK5clang17PartialDiagnostic10getStorageEv", "clang::PartialDiagnostic::getStorage() const"},
    {"_ZN5clang17PartialDiagnostic16StorageAllocator8AllocateEv", "clang::PartialDiagnostic::StorageAllocator::Allocate()"},
    {"_ZN5clang17PartialDiagnostic7StorageaSERKS1_", "clang::PartialDiagnostic::Storage::operator=(clang::PartialDiagnostic::Storage const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIN5clang14SourceLocationENS2_17PartialDiagnosticEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, false>::grow(unsigned long)"},
    {"_ZN5clang4Sema24SetMemberAccessSpecifierEPNS_9NamedDeclES2_NS_15AccessSpecifierE", "clang::Sema::SetMemberAccessSpecifier(clang::NamedDecl*, clang::NamedDecl*, clang::AccessSpecifier)"},
    {"_ZN5clang4Sema24HandleDelayedAccessCheckERNS_4sema17DelayedDiagnosticEPNS_4DeclE", "clang::Sema::HandleDelayedAccessCheck(clang::sema::DelayedDiagnostic&, clang::Decl*)"},
    {"_ZN12_GLOBAL__N_116EffectiveContextC1EPN5clang11DeclContextE", "(anonymous namespace)::EffectiveContext::EffectiveContext(clang::DeclContext*)"},
    {"_Z20CheckEffectiveAccessRN5clang4SemaERKN12_GLOBAL__N_116EffectiveContextENS_14SourceLocationERNS2_12AccessTargetE", "CheckEffectiveAccess(clang::Sema&, (anonymous namespace)::EffectiveContext const&, clang::SourceLocation, (anonymous namespace)::AccessTarget&)"},
    {"_ZN5clang4Sema26HandleDependentAccessCheckERKNS_19DependentDiagnosticERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::HandleDependentAccessCheck(clang::DependentDiagnostic const&, clang::MultiLevelTemplateArgumentList const&)"},
    {"_Z11CheckAccessRN5clang4SemaENS_14SourceLocationERN12_GLOBAL__N_112AccessTargetE", "CheckAccess(clang::Sema&, clang::SourceLocation, (anonymous namespace)::AccessTarget&)"},
    {"_ZN5clang4Sema27CheckUnresolvedLookupAccessEPNS_20UnresolvedLookupExprENS_14DeclAccessPairE", "clang::Sema::CheckUnresolvedLookupAccess(clang::UnresolvedLookupExpr*, clang::DeclAccessPair)"},
    {"_ZN5clang4Sema27CheckUnresolvedMemberAccessEPNS_20UnresolvedMemberExprENS_14DeclAccessPairE", "clang::Sema::CheckUnresolvedMemberAccess(clang::UnresolvedMemberExpr*, clang::DeclAccessPair)"},
    {"_ZN5clang4Sema21CheckDestructorAccessENS_14SourceLocationEPNS_17CXXDestructorDeclERKNS_17PartialDiagnosticE", "clang::Sema::CheckDestructorAccess(clang::SourceLocation, clang::CXXDestructorDecl*, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema22CheckConstructorAccessENS_14SourceLocationEPNS_18CXXConstructorDeclERKNS_17InitializedEntityENS_15AccessSpecifierEb", "clang::Sema::CheckConstructorAccess(clang::SourceLocation, clang::CXXConstructorDecl*, clang::InitializedEntity const&, clang::AccessSpecifier, bool)"},
    {"_ZN5clang4Sema23CheckDirectMemberAccessENS_14SourceLocationEPNS_9NamedDeclERKNS_17PartialDiagnosticE", "clang::Sema::CheckDirectMemberAccess(clang::SourceLocation, clang::NamedDecl*, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema21CheckAllocationAccessENS_14SourceLocationENS_11SourceRangeEPNS_13CXXRecordDeclENS_14DeclAccessPairE", "clang::Sema::CheckAllocationAccess(clang::SourceLocation, clang::SourceRange, clang::CXXRecordDecl*, clang::DeclAccessPair)"},
    {"_ZN5clang4Sema25CheckMemberOperatorAccessENS_14SourceLocationEPNS_4ExprES3_NS_14DeclAccessPairE", "clang::Sema::CheckMemberOperatorAccess(clang::SourceLocation, clang::Expr*, clang::Expr*, clang::DeclAccessPair)"},
    {"_ZN5clang4Sema26CheckAddressOfMemberAccessEPNS_4ExprENS_14DeclAccessPairE", "clang::Sema::CheckAddressOfMemberAccess(clang::Expr*, clang::DeclAccessPair)"},
    {"_ZN5clang4Sema20CheckBaseClassAccessENS_14SourceLocationENS_8QualTypeES2_RKNS_11CXXBasePathEjbb", "clang::Sema::CheckBaseClassAccess(clang::SourceLocation, clang::QualType, clang::QualType, clang::CXXBasePath const&, unsigned int, bool, bool)"},
    {"_ZN5clang4Sema17CheckLookupAccessERKNS_12LookupResultE", "clang::Sema::CheckLookupAccess(clang::LookupResult const&)"},
    {"_ZN5clang4Sema33ActOnStartSuppressingAccessChecksEv", "clang::Sema::ActOnStartSuppressingAccessChecks()"},
    {"_ZN5clang4Sema32ActOnStopSuppressingAccessChecksEv", "clang::Sema::ActOnStopSuppressingAccessChecks()"},
    {"_Z18FindDeclaringClassPN5clang9NamedDeclE", "FindDeclaringClass(clang::NamedDecl*)"},
    {"_Z9HasAccessRN5clang4SemaERKN12_GLOBAL__N_116EffectiveContextEPKNS_13CXXRecordDeclENS_15AccessSpecifierERKNS2_12AccessTargetE", "HasAccess(clang::Sema&, (anonymous namespace)::EffectiveContext const&, clang::CXXRecordDecl const*, clang::AccessSpecifier, (anonymous namespace)::AccessTarget const&)"},
    {"_Z12FindBestPathRN5clang4SemaERKN12_GLOBAL__N_116EffectiveContextERNS2_12AccessTargetENS_15AccessSpecifierERNS_12CXXBasePathsE", "FindBestPath(clang::Sema&, (anonymous namespace)::EffectiveContext const&, (anonymous namespace)::AccessTarget&, clang::AccessSpecifier, clang::CXXBasePaths&)"},
    {"_Z13GetFriendKindRN5clang4SemaERKN12_GLOBAL__N_116EffectiveContextEPKNS_13CXXRecordDeclE", "GetFriendKind(clang::Sema&, (anonymous namespace)::EffectiveContext const&, clang::CXXRecordDecl const*)"},
    {"_Z13MatchesFriendRN5clang4SemaERKN12_GLOBAL__N_116EffectiveContextEPKNS_13CXXRecordDeclE", "MatchesFriend(clang::Sema&, (anonymous namespace)::EffectiveContext const&, clang::CXXRecordDecl const*)"},
    {"_Z18MightInstantiateToRN5clang4SemaEPNS_12FunctionDeclES3_", "MightInstantiateTo(clang::Sema&, clang::FunctionDecl*, clang::FunctionDecl*)"},
    {"_Z22IsDerivedFromInclusivePKN5clang13CXXRecordDeclES2_", "IsDerivedFromInclusive(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_122ProtectedFriendContext14findFriendshipEPKN5clang13CXXRecordDeclEj", "(anonymous namespace)::ProtectedFriendContext::findFriendship(clang::CXXRecordDecl const*, unsigned int)"},
    {"_ZSt6__findIPKPN5clang13CXXRecordDeclEPKS1_ET_S7_S7_RKT0_St26random_access_iterator_tag", "clang::CXXRecordDecl* const* std::__find<clang::CXXRecordDecl* const*, clang::CXXRecordDecl const*>(clang::CXXRecordDecl* const*, clang::CXXRecordDecl* const*, clang::CXXRecordDecl const* const&, std::random_access_iterator_tag)"},
    {"_ZN5clang4Sema17FreePackedContextEv", "clang::Sema::FreePackedContext()"},
    {"_ZN5clang4Sema31AddAlignmentAttributesForRecordEPNS_10RecordDeclE", "clang::Sema::AddAlignmentAttributesForRecord(clang::RecordDecl*)"},
    {"_ZN5clang4Sema23ActOnPragmaOptionsAlignENS0_22PragmaOptionsAlignKindENS_14SourceLocationES2_", "clang::Sema::ActOnPragmaOptionsAlign(clang::Sema::PragmaOptionsAlignKind, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN12_GLOBAL__N_115PragmaPackStack4pushEPN5clang14IdentifierInfoE", "(anonymous namespace)::PragmaPackStack::push(clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema15ActOnPragmaPackENS0_14PragmaPackKindEPNS_14IdentifierInfoEPNS_4ExprENS_14SourceLocationES6_S6_", "clang::Sema::ActOnPragmaPack(clang::Sema::PragmaPackKind, clang::IdentifierInfo*, clang::Expr*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema17ActOnPragmaUnusedERKNS_5TokenEPNS_5ScopeENS_14SourceLocationE", "clang::Sema::ActOnPragmaUnused(clang::Token const&, clang::Scope*, clang::SourceLocation)"},
    {"_ZN5clang4Sema28AddPushedVisibilityAttributeEPNS_4DeclE", "clang::Sema::AddPushedVisibilityAttribute(clang::Decl*)"},
    {"_ZN5clang4Sema14FreeVisContextEv", "clang::Sema::FreeVisContext()"},
    {"_ZN5clang4Sema21ActOnPragmaVisibilityEbPKNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::ActOnPragmaVisibility(bool, clang::IdentifierInfo const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema19PopPragmaVisibilityEv", "clang::Sema::PopPragmaVisibility()"},
    {"_ZN5clang4Sema27PushNamespaceVisibilityAttrEPKNS_14VisibilityAttrE", "clang::Sema::PushNamespaceVisibilityAttr(clang::VisibilityAttr const*)"},
    {"_ZN5clang4Decl7addAttrEPNS_4AttrE", "clang::Decl::addAttr(clang::Attr*)"},
    {"_ZNSt6vectorISt4pairIjN5clang14SourceLocationEESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<unsigned int, clang::SourceLocation>, std::allocator<std::pair<unsigned int, clang::SourceLocation> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, clang::SourceLocation>*, std::vector<std::pair<unsigned int, clang::SourceLocation>, std::allocator<std::pair<unsigned int, clang::SourceLocation> > > >, std::pair<unsigned int, clang::SourceLocation> const&)"},
    {"_ZN5clang15InheritableAttrD1Ev", "clang::InheritableAttr::~InheritableAttr()"},
    {"_ZN5clang15InheritableAttrD0Ev", "clang::InheritableAttr::~InheritableAttr()"},
    {"_ZN4llvm11SmallVectorIPN5clang4AttrELj2EEC2EjRKS3_", "llvm::SmallVector<clang::Attr*, 2u>::SmallVector(unsigned int, clang::Attr* const&)"},
    {"_ZN5clang4Sema17ActOnCXXNamedCastENS_14SourceLocationENS_3tok9TokenKindES1_NS_9OpaquePtrINS_8QualTypeEEES1_S1_PNS_4ExprES1_", "clang::Sema::ActOnCXXNamedCast(clang::SourceLocation, clang::tok::TokenKind, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema17BuildCXXNamedCastENS_14SourceLocationENS_3tok9TokenKindEPNS_14TypeSourceInfoEPNS_4ExprENS_11SourceRangeES8_", "clang::Sema::BuildCXXNamedCast(clang::SourceLocation, clang::tok::TokenKind, clang::TypeSourceInfo*, clang::Expr*, clang::SourceRange, clang::SourceRange)"},
    {"_ZN5clang4Sema18CXXCheckCStyleCastENS_11SourceRangeENS_8QualTypeERNS_13ExprValueKindERPNS_4ExprERNS_8CastKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEb", "clang::Sema::CXXCheckCStyleCast(clang::SourceRange, clang::QualType, clang::ExprValueKind&, clang::Expr*&, clang::CastKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&, bool)"},
    {"_Z12TryConstCastRN5clang4SemaEPNS_4ExprENS_8QualTypeEbRj", "TryConstCast(clang::Sema&, clang::Expr*, clang::QualType, bool, unsigned int&)"},
    {"_Z13TryStaticCastRN5clang4SemaERPNS_4ExprENS_8QualTypeEbRKNS_11SourceRangeERjRNS_8CastKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "TryStaticCast(clang::Sema&, clang::Expr*&, clang::QualType, bool, clang::SourceRange const&, unsigned int&, clang::CastKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&)"},
    {"_Z18TryReinterpretCastRN5clang4SemaEPNS_4ExprENS_8QualTypeEbRKNS_11SourceRangeERjRNS_8CastKindE", "TryReinterpretCast(clang::Sema&, clang::Expr*, clang::QualType, bool, clang::SourceRange const&, unsigned int&, clang::CastKind&)"},
    {"_Z18CastsAwayConstnessRN5clang4SemaENS_8QualTypeES2_", "CastsAwayConstness(clang::Sema&, clang::QualType, clang::QualType)"},
    {"_Z17TryStaticDowncastRN5clang4SemaENS_7CanQualINS_4TypeEEES4_bRKNS_11SourceRangeENS_8QualTypeES8_RjRNS_8CastKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "TryStaticDowncast(clang::Sema&, clang::CanQual<clang::Type>, clang::CanQual<clang::Type>, bool, clang::SourceRange const&, clang::QualType, clang::QualType, unsigned int&, clang::CastKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&)"},
    {"_Z25NoteAllOverloadCandidatesPN5clang4ExprERNS_4SemaE", "NoteAllOverloadCandidates(clang::Expr*, clang::Sema&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang10QualifiersELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Qualifiers, false>::grow(unsigned long)"},
    {"_ZN5clang4Sema18computeDeclContextENS_8QualTypeE", "clang::Sema::computeDeclContext(clang::QualType)"},
    {"_ZN5clang4Sema18computeDeclContextERKNS_12CXXScopeSpecEb", "clang::Sema::computeDeclContext(clang::CXXScopeSpec const&, bool)"},
    {"_ZN5clang4Sema25getCurrentInstantiationOfEPNS_19NestedNameSpecifierE", "clang::Sema::getCurrentInstantiationOf(clang::NestedNameSpecifier*)"},
    {"_ZN5clang4Sema25isDependentScopeSpecifierERKNS_12CXXScopeSpecE", "clang::Sema::isDependentScopeSpecifier(clang::CXXScopeSpec const&)"},
    {"_ZN5clang4Sema23isUnknownSpecializationERKNS_12CXXScopeSpecE", "clang::Sema::isUnknownSpecialization(clang::CXXScopeSpec const&)"},
    {"_ZN5clang4Sema26RequireCompleteDeclContextERNS_12CXXScopeSpecEPNS_11DeclContextE", "clang::Sema::RequireCompleteDeclContext(clang::CXXScopeSpec&, clang::DeclContext*)"},
    {"_ZN5clang4Sema28ActOnCXXGlobalScopeSpecifierEPNS_5ScopeENS_14SourceLocationE", "clang::Sema::ActOnCXXGlobalScopeSpecifier(clang::Scope*, clang::SourceLocation)"},
    {"_ZN5clang4Sema31isAcceptableNestedNameSpecifierEPNS_9NamedDeclE", "clang::Sema::isAcceptableNestedNameSpecifier(clang::NamedDecl*)"},
    {"_ZN5clang4Sema25FindFirstQualifierInScopeEPNS_5ScopeEPNS_19NestedNameSpecifierE", "clang::Sema::FindFirstQualifierInScope(clang::Scope*, clang::NestedNameSpecifier*)"},
    {"_ZN5clang4Sema28isNonTypeNestedNameSpecifierEPNS_5ScopeERNS_12CXXScopeSpecENS_14SourceLocationERNS_14IdentifierInfoENS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::isNonTypeNestedNameSpecifier(clang::Scope*, clang::CXXScopeSpec&, clang::SourceLocation, clang::IdentifierInfo&, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang4Sema27BuildCXXNestedNameSpecifierEPNS_5ScopeERNS_12CXXScopeSpecENS_14SourceLocationES5_RNS_14IdentifierInfoENS_8QualTypeEPNS_9NamedDeclEbb", "clang::Sema::BuildCXXNestedNameSpecifier(clang::Scope*, clang::CXXScopeSpec&, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo&, clang::QualType, clang::NamedDecl*, bool, bool)"},
    {"_ZN5clang4Sema27ActOnCXXNestedNameSpecifierEPNS_5ScopeERNS_12CXXScopeSpecENS_14SourceLocationES5_RNS_14IdentifierInfoENS_9OpaquePtrINS_8QualTypeEEEb", "clang::Sema::ActOnCXXNestedNameSpecifier(clang::Scope*, clang::CXXScopeSpec&, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo&, clang::OpaquePtr<clang::QualType>, bool)"},
    {"_ZN5clang4Sema25IsInvalidUnlessNestedNameEPNS_5ScopeERNS_12CXXScopeSpecERNS_14IdentifierInfoENS_9OpaquePtrINS_8QualTypeEEEb", "clang::Sema::IsInvalidUnlessNestedName(clang::Scope*, clang::CXXScopeSpec&, clang::IdentifierInfo&, clang::OpaquePtr<clang::QualType>, bool)"},
    {"_ZN5clang4Sema27ActOnCXXNestedNameSpecifierEPNS_5ScopeERKNS_12CXXScopeSpecENS_9OpaquePtrINS_8QualTypeEEENS_11SourceRangeENS_14SourceLocationE", "clang::Sema::ActOnCXXNestedNameSpecifier(clang::Scope*, clang::CXXScopeSpec const&, clang::OpaquePtr<clang::QualType>, clang::SourceRange, clang::SourceLocation)"},
    {"_ZN5clang4Sema26ShouldEnterDeclaratorScopeEPNS_5ScopeERKNS_12CXXScopeSpecE", "clang::Sema::ShouldEnterDeclaratorScope(clang::Scope*, clang::CXXScopeSpec const&)"},
    {"_ZN5clang4Sema28ActOnCXXEnterDeclaratorScopeEPNS_5ScopeERNS_12CXXScopeSpecE", "clang::Sema::ActOnCXXEnterDeclaratorScope(clang::Scope*, clang::CXXScopeSpec&)"},
    {"_ZN5clang4Sema27ActOnCXXExitDeclaratorScopeEPNS_5ScopeERKNS_12CXXScopeSpecE", "clang::Sema::ActOnCXXExitDeclaratorScope(clang::Scope*, clang::CXXScopeSpec const&)"},
    {"_ZNK5clang4Sema30getLocationOfStringLiteralByteEPKNS_13StringLiteralEj", "clang::Sema::getLocationOfStringLiteralByte(clang::StringLiteral const*, unsigned int) const"},
    {"_ZN5clang4Sema19CheckablePrintfAttrEPKNS_10FormatAttrEPNS_8CallExprE", "clang::Sema::CheckablePrintfAttr(clang::FormatAttr const*, clang::CallExpr*)"},
    {"_ZN5clang4Sema24CheckBuiltinFunctionCallEjPNS_8CallExprE", "clang::Sema::CheckBuiltinFunctionCall(unsigned int, clang::CallExpr*)"},
    {"_ZN5clang4Sema22SemaBuiltinConstantArgEPNS_8CallExprEiRN4llvm6APSIntE", "clang::Sema::SemaBuiltinConstantArg(clang::CallExpr*, int, llvm::APSInt&)"},
    {"_ZN5clang4Sema15CheckObjCStringEPNS_4ExprE", "clang::Sema::CheckObjCString(clang::Expr*)"},
    {"_ZN5clang4Sema18SemaBuiltinVAStartEPNS_8CallExprE", "clang::Sema::SemaBuiltinVAStart(clang::CallExpr*)"},
    {"_ZN5clang4Sema27SemaBuiltinUnorderedCompareEPNS_8CallExprE", "clang::Sema::SemaBuiltinUnorderedCompare(clang::CallExpr*)"},
    {"_ZN5clang4Sema27SemaBuiltinFPClassificationEPNS_8CallExprEj", "clang::Sema::SemaBuiltinFPClassification(clang::CallExpr*, unsigned int)"},
    {"_ZN5clang4Sema24SemaBuiltinShuffleVectorEPNS_8CallExprE", "clang::Sema::SemaBuiltinShuffleVector(clang::CallExpr*)"},
    {"_ZN5clang4Sema19SemaBuiltinPrefetchEPNS_8CallExprE", "clang::Sema::SemaBuiltinPrefetch(clang::CallExpr*)"},
    {"_ZN5clang4Sema21SemaBuiltinObjectSizeEPNS_8CallExprE", "clang::Sema::SemaBuiltinObjectSize(clang::CallExpr*)"},
    {"_ZN5clang4Sema18SemaBuiltinLongjmpEPNS_8CallExprE", "clang::Sema::SemaBuiltinLongjmp(clang::CallExpr*)"},
    {"_ZN5clang4Sema27SemaBuiltinAtomicOverloadedENS_12ActionResultIPNS_4ExprELb1EEE", "clang::Sema::SemaBuiltinAtomicOverloaded(clang::ActionResult<clang::Expr*, true>)"},
    {"_ZN5clang4Sema27CheckARMBuiltinFunctionCallEjPNS_8CallExprE", "clang::Sema::CheckARMBuiltinFunctionCall(unsigned int, clang::CallExpr*)"},
    {"_ZN5clang4Sema17CheckFunctionCallEPNS_12FunctionDeclEPNS_8CallExprE", "clang::Sema::CheckFunctionCall(clang::FunctionDecl*, clang::CallExpr*)"},
    {"_ZN5clang4Sema25CheckPrintfScanfArgumentsEPKNS_8CallExprEbjjb", "clang::Sema::CheckPrintfScanfArguments(clang::CallExpr const*, bool, unsigned int, unsigned int, bool)"},
    {"_ZN5clang4Sema21CheckNonNullArgumentsEPKNS_11NonNullAttrEPKNS_8CallExprE", "clang::Sema::CheckNonNullArguments(clang::NonNullAttr const*, clang::CallExpr const*)"},
    {"_ZN5clang4Sema14CheckBlockCallEPNS_9NamedDeclEPNS_8CallExprE", "clang::Sema::CheckBlockCall(clang::NamedDecl*, clang::CallExpr*)"},
    {"_ZN5clang4Sema22SemaCheckStringLiteralEPKNS_4ExprEPKNS_8CallExprEbjjb", "clang::Sema::SemaCheckStringLiteral(clang::Expr const*, clang::CallExpr const*, bool, unsigned int, unsigned int, bool)"},
    {"_ZN5clang4Sema17CheckFormatStringEPKNS_13StringLiteralEPKNS_4ExprEPKNS_8CallExprEbjjb", "clang::Sema::CheckFormatString(clang::StringLiteral const*, clang::Expr const*, clang::CallExpr const*, bool, unsigned int, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler14DoneProcessingEv", "(anonymous namespace)::CheckFormatHandler::DoneProcessing()"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandlerD1Ev", "(anonymous namespace)::CheckPrintfHandler::~CheckPrintfHandler()"},
    {"_ZN12_GLOBAL__N_117CheckScanfHandlerD1Ev", "(anonymous namespace)::CheckScanfHandler::~CheckScanfHandler()"},
    {"_ZN5clang4Sema20CheckReturnStackAddrEPNS_4ExprENS_8QualTypeENS_14SourceLocationE", "clang::Sema::CheckReturnStackAddr(clang::Expr*, clang::QualType, clang::SourceLocation)"},
    {"_Z8EvalAddrPN5clang4ExprERN4llvm15SmallVectorImplIPNS_11DeclRefExprEEE", "EvalAddr(clang::Expr*, llvm::SmallVectorImpl<clang::DeclRefExpr*>&)"},
    {"_Z7EvalValPN5clang4ExprERN4llvm15SmallVectorImplIPNS_11DeclRefExprEEE", "EvalVal(clang::Expr*, llvm::SmallVectorImpl<clang::DeclRefExpr*>&)"},
    {"_ZN5clang4Sema20CheckFloatComparisonENS_14SourceLocationEPNS_4ExprES3_", "clang::Sema::CheckFloatComparison(clang::SourceLocation, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema24CheckImplicitConversionsEPNS_4ExprENS_14SourceLocationE", "clang::Sema::CheckImplicitConversions(clang::Expr*, clang::SourceLocation)"},
    {"_ZN12_GLOBAL__N_126AnalyzeImplicitConversionsERN5clang4SemaEPNS0_4ExprENS0_14SourceLocationE", "(anonymous namespace)::AnalyzeImplicitConversions(clang::Sema&, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema27CheckBitFieldInitializationENS_14SourceLocationEPNS_9FieldDeclEPNS_4ExprE", "clang::Sema::CheckBitFieldInitialization(clang::SourceLocation, clang::FieldDecl*, clang::Expr*)"},
    {"_ZN12_GLOBAL__N_125AnalyzeBitFieldAssignmentERN5clang4SemaEPNS0_9FieldDeclEPNS0_4ExprENS0_14SourceLocationE", "(anonymous namespace)::AnalyzeBitFieldAssignment(clang::Sema&, clang::FieldDecl*, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24CheckParmsForFunctionDefEPPNS_11ParmVarDeclES3_b", "clang::Sema::CheckParmsForFunctionDef(clang::ParmVarDecl**, clang::ParmVarDecl**, bool)"},
    {"_ZN5clang4Sema14CheckCastAlignEPNS_4ExprENS_8QualTypeENS_11SourceRangeE", "clang::Sema::CheckCastAlign(clang::Expr*, clang::QualType, clang::SourceRange)"},
    {"_ZN12_GLOBAL__N_124CheckConditionalOperatorERN5clang4SemaEPNS0_19ConditionalOperatorENS0_8QualTypeE", "(anonymous namespace)::CheckConditionalOperator(clang::Sema&, clang::ConditionalOperator*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_123CheckImplicitConversionERN5clang4SemaEPNS0_4ExprENS0_8QualTypeENS0_14SourceLocationEPb", "(anonymous namespace)::CheckImplicitConversion(clang::Sema&, clang::Expr*, clang::QualType, clang::SourceLocation, bool*)"},
    {"_ZN12_GLOBAL__N_130CheckTrivialUnsignedComparisonERN5clang4SemaEPNS0_14BinaryOperatorE", "(anonymous namespace)::CheckTrivialUnsignedComparison(clang::Sema&, clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_112GetExprRangeERN5clang10ASTContextEPNS0_4ExprEj", "(anonymous namespace)::GetExprRange(clang::ASTContext&, clang::Expr*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113GetValueRangeERN5clang10ASTContextERNS0_7APValueENS0_8QualTypeEj", "(anonymous namespace)::GetValueRange(clang::ASTContext&, clang::APValue&, clang::QualType, unsigned int)"},
    {"_ZN12_GLOBAL__N_18IntRange14forValueOfTypeERN5clang10ASTContextENS1_8QualTypeE", "(anonymous namespace)::IntRange::forValueOfType(clang::ASTContext&, clang::QualType)"},
    {"_ZN12_GLOBAL__N_113GetValueRangeERN5clang10ASTContextERN4llvm6APSIntEj", "(anonymous namespace)::GetValueRange(clang::ASTContext&, llvm::APSInt&, unsigned int)"},
    {"_ZN12_GLOBAL__N_16IsZeroERN5clang4SemaEPNS0_4ExprE", "(anonymous namespace)::IsZero(clang::Sema&, clang::Expr*)"},
    {"_ZN12_GLOBAL__N_115DiagnoseImpCastERN5clang4SemaEPNS0_4ExprENS0_8QualTypeENS0_14SourceLocationEj", "(anonymous namespace)::DiagnoseImpCast(clang::Sema&, clang::Expr*, clang::QualType, clang::SourceLocation, unsigned int)"},
    {"_ZN12_GLOBAL__N_120IsSameFloatAfterCastERKN5clang7APValueERKN4llvm12fltSemanticsES7_", "(anonymous namespace)::IsSameFloatAfterCast(clang::APValue const&, llvm::fltSemantics const&, llvm::fltSemantics const&)"},
    {"_ZN12_GLOBAL__N_123CheckConditionalOperandERN5clang4SemaEPNS0_4ExprENS0_8QualTypeENS0_14SourceLocationERb", "(anonymous namespace)::CheckConditionalOperand(clang::Sema&, clang::Expr*, clang::QualType, clang::SourceLocation, bool&)"},
    {"_ZN12_GLOBAL__N_117CheckScanfHandlerD0Ev", "(anonymous namespace)::CheckScanfHandler::~CheckScanfHandler()"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler14HandleNullCharEPKc", "(anonymous namespace)::CheckFormatHandler::HandleNullChar(char const*)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler21HandleInvalidPositionEPKcjN5clang21analyze_format_string15PositionContextE", "(anonymous namespace)::CheckFormatHandler::HandleInvalidPosition(char const*, unsigned int, clang::analyze_format_string::PositionContext)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler18HandleZeroPositionEPKcj", "(anonymous namespace)::CheckFormatHandler::HandleZeroPosition(char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler25HandleIncompleteSpecifierEPKcj", "(anonymous namespace)::CheckFormatHandler::HandleIncompleteSpecifier(char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117CheckScanfHandler37HandleInvalidScanfConversionSpecifierERKN5clang13analyze_scanf14ScanfSpecifierEPKcj", "(anonymous namespace)::CheckScanfHandler::HandleInvalidScanfConversionSpecifier(clang::analyze_scanf::ScanfSpecifier const&, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117CheckScanfHandler20HandleScanfSpecifierERKN5clang13analyze_scanf14ScanfSpecifierEPKcj", "(anonymous namespace)::CheckScanfHandler::HandleScanfSpecifier(clang::analyze_scanf::ScanfSpecifier const&, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117CheckScanfHandler24HandleIncompleteScanListEPKcS2_", "(anonymous namespace)::CheckScanfHandler::HandleIncompleteScanList(char const*, char const*)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler12CheckNumArgsERKN5clang21analyze_format_string15FormatSpecifierERKNS2_19ConversionSpecifierEPKcjj", "(anonymous namespace)::CheckFormatHandler::CheckNumArgs(clang::analyze_format_string::FormatSpecifier const&, clang::analyze_format_string::ConversionSpecifier const&, char const*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandler32HandleInvalidConversionSpecifierEjN5clang14SourceLocationEPKcjS4_j", "(anonymous namespace)::CheckFormatHandler::HandleInvalidConversionSpecifier(unsigned int, clang::SourceLocation, char const*, unsigned int, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandlerD1Ev", "(anonymous namespace)::CheckFormatHandler::~CheckFormatHandler()"},
    {"_ZN12_GLOBAL__N_118CheckFormatHandlerD0Ev", "(anonymous namespace)::CheckFormatHandler::~CheckFormatHandler()"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandlerD0Ev", "(anonymous namespace)::CheckPrintfHandler::~CheckPrintfHandler()"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler38HandleInvalidPrintfConversionSpecifierERKN5clang14analyze_printf15PrintfSpecifierEPKcj", "(anonymous namespace)::CheckPrintfHandler::HandleInvalidPrintfConversionSpecifier(clang::analyze_printf::PrintfSpecifier const&, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler21HandlePrintfSpecifierERKN5clang14analyze_printf15PrintfSpecifierEPKcj", "(anonymous namespace)::CheckPrintfHandler::HandlePrintfSpecifier(clang::analyze_printf::PrintfSpecifier const&, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler12HandleAmountERKN5clang21analyze_format_string14OptionalAmountEjPKcj", "(anonymous namespace)::CheckPrintfHandler::HandleAmount(clang::analyze_format_string::OptionalAmount const&, unsigned int, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler19HandleInvalidAmountERKN5clang14analyze_printf15PrintfSpecifierERKNS1_21analyze_format_string14OptionalAmountEjPKcj", "(anonymous namespace)::CheckPrintfHandler::HandleInvalidAmount(clang::analyze_printf::PrintfSpecifier const&, clang::analyze_format_string::OptionalAmount const&, unsigned int, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler10HandleFlagERKN5clang14analyze_printf15PrintfSpecifierERKNS1_21analyze_format_string12OptionalFlagEPKcj", "(anonymous namespace)::CheckPrintfHandler::HandleFlag(clang::analyze_printf::PrintfSpecifier const&, clang::analyze_format_string::OptionalFlag const&, char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_118CheckPrintfHandler17HandleIgnoredFlagERKN5clang14analyze_printf15PrintfSpecifierERKNS1_21analyze_format_string12OptionalFlagES9_PKcj", "(anonymous namespace)::CheckPrintfHandler::HandleIgnoredFlag(clang::analyze_printf::PrintfSpecifier const&, clang::analyze_format_string::OptionalFlag const&, clang::analyze_format_string::OptionalFlag const&, char const*, unsigned int)"},
    {"_ZNK5clang4Decl7getAttrINS_13FormatArgAttrEEEPT_v", "clang::FormatArgAttr* clang::Decl::getAttr<clang::FormatArgAttr>() const"},
    {"_ZN4llvm11SmallVectorItLj128EEC2EjRKt", "llvm::SmallVector<unsigned short, 128u>::SmallVector(unsigned int, unsigned short const&)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler38HandleInvalidPrintfConversionSpecifierERKNS_14analyze_printf15PrintfSpecifierEPKcj", "clang::analyze_format_string::FormatStringHandler::HandleInvalidPrintfConversionSpecifier(clang::analyze_printf::PrintfSpecifier const&, char const*, unsigned int)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler21HandlePrintfSpecifierERKNS_14analyze_printf15PrintfSpecifierEPKcj", "clang::analyze_format_string::FormatStringHandler::HandlePrintfSpecifier(clang::analyze_printf::PrintfSpecifier const&, char const*, unsigned int)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler37HandleInvalidScanfConversionSpecifierERKNS_13analyze_scanf14ScanfSpecifierEPKcj", "clang::analyze_format_string::FormatStringHandler::HandleInvalidScanfConversionSpecifier(clang::analyze_scanf::ScanfSpecifier const&, char const*, unsigned int)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler20HandleScanfSpecifierERKNS_13analyze_scanf14ScanfSpecifierEPKcj", "clang::analyze_format_string::FormatStringHandler::HandleScanfSpecifier(clang::analyze_scanf::ScanfSpecifier const&, char const*, unsigned int)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler24HandleIncompleteScanListEPKcS3_", "clang::analyze_format_string::FormatStringHandler::HandleIncompleteScanList(char const*, char const*)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler14HandleNullCharEPKc", "clang::analyze_format_string::FormatStringHandler::HandleNullChar(char const*)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler21HandleInvalidPositionEPKcjNS0_15PositionContextE", "clang::analyze_format_string::FormatStringHandler::HandleInvalidPosition(char const*, unsigned int, clang::analyze_format_string::PositionContext)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler18HandleZeroPositionEPKcj", "clang::analyze_format_string::FormatStringHandler::HandleZeroPosition(char const*, unsigned int)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandler25HandleIncompleteSpecifierEPKcj", "clang::analyze_format_string::FormatStringHandler::HandleIncompleteSpecifier(char const*, unsigned int)"},
    {"_ZN5clang22getSimplifiedTypeClassENS_7CanQualINS_4TypeEEE", "clang::getSimplifiedTypeClass(clang::CanQual<clang::Type>)"},
    {"_ZN5clang16getDeclUsageTypeERNS_10ASTContextEPNS_9NamedDeclE", "clang::getDeclUsageType(clang::ASTContext&, clang::NamedDecl*)"},
    {"_Z23GetCompletionTypeStringN5clang8QualTypeERNS_10ASTContextERNS_23CodeCompletionAllocatorE", "GetCompletionTypeString(clang::QualType, clang::ASTContext&, clang::CodeCompletionAllocator&)"},
    {"_ZN5clang20CodeCompletionResult26CreateCodeCompletionStringERNS_4SemaERNS_23CodeCompletionAllocatorE", "clang::CodeCompletionResult::CreateCodeCompletionString(clang::Sema&, clang::CodeCompletionAllocator&)"},
    {"_Z18AddResultTypeChunkRN5clang10ASTContextEPNS_9NamedDeclERNS_21CodeCompletionBuilderE", "AddResultTypeChunk(clang::ASTContext&, clang::NamedDecl*, clang::CodeCompletionBuilder&)"},
    {"_Z30AddQualifierToCompletionStringRN5clang21CodeCompletionBuilderEPNS_19NestedNameSpecifierEbRNS_10ASTContextE", "AddQualifierToCompletionString(clang::CodeCompletionBuilder&, clang::NestedNameSpecifier*, bool, clang::ASTContext&)"},
    {"_Z17AddTypedNameChunkRN5clang10ASTContextEPNS_9NamedDeclERNS_21CodeCompletionBuilderE", "AddTypedNameChunk(clang::ASTContext&, clang::NamedDecl*, clang::CodeCompletionBuilder&)"},
    {"_Z26AddFunctionParameterChunksRN5clang10ASTContextEPNS_12FunctionDeclERNS_21CodeCompletionBuilderEjb", "AddFunctionParameterChunks(clang::ASTContext&, clang::FunctionDecl*, clang::CodeCompletionBuilder&, unsigned int, bool)"},
    {"_Z38AddFunctionTypeQualsToCompletionStringRN5clang21CodeCompletionBuilderEPNS_12FunctionDeclE", "AddFunctionTypeQualsToCompletionString(clang::CodeCompletionBuilder&, clang::FunctionDecl*)"},
    {"_Z26AddTemplateParameterChunksRN5clang10ASTContextEPNS_12TemplateDeclERNS_21CodeCompletionBuilderEjjb", "AddTemplateParameterChunks(clang::ASTContext&, clang::TemplateDecl*, clang::CodeCompletionBuilder&, unsigned int, unsigned int, bool)"},
    {"_Z23FormatFunctionParameterRN5clang10ASTContextEPNS_11ParmVarDeclEb", "FormatFunctionParameter(clang::ASTContext&, clang::ParmVarDecl*, bool)"},
    {"_Z16MaybeAddSentinelRN5clang10ASTContextEPNS_9NamedDeclERNS_21CodeCompletionBuilderE", "MaybeAddSentinel(clang::ASTContext&, clang::NamedDecl*, clang::CodeCompletionBuilder&)"},
    {"_ZNK5clang20CodeCompleteConsumer17OverloadCandidate21CreateSignatureStringEjRNS_4SemaERNS_23CodeCompletionAllocatorE", "clang::CodeCompleteConsumer::OverloadCandidate::CreateSignatureString(unsigned int, clang::Sema&, clang::CodeCompletionAllocator&) const"},
    {"_ZN5clang21getMacroUsagePriorityEN4llvm9StringRefERKNS_11LangOptionsEb", "clang::getMacroUsagePriority(llvm::StringRef, clang::LangOptions const&, bool)"},
    {"_ZN5clang20getCursorKindForDeclEPNS_4DeclE", "clang::getCursorKindForDecl(clang::Decl*)"},
    {"_ZN5clang4Sema24CodeCompleteOrdinaryNameEPNS_5ScopeENS0_23ParserCompletionContextE", "clang::Sema::CodeCompleteOrdinaryName(clang::Scope*, clang::Sema::ParserCompletionContext)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder13EnterNewScopeEv", "(anonymous namespace)::ResultBuilder::EnterNewScope()"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder22IsOrdinaryNonValueNameEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsOrdinaryNonValueName(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder14IsOrdinaryNameEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsOrdinaryName(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder21IsOrdinaryNonTypeNameEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsOrdinaryNonTypeName(clang::NamedDecl*) const"},
    {"_Z21MaybeAddOverrideCallsRN5clang4SemaEPNS_11DeclContextERN12_GLOBAL__N_113ResultBuilderE", "MaybeAddOverrideCalls(clang::Sema&, clang::DeclContext*, (anonymous namespace)::ResultBuilder&)"},
    {"_Z22AddOrdinaryNameResultsN5clang4Sema23ParserCompletionContextEPNS_5ScopeERS0_RN12_GLOBAL__N_113ResultBuilderE", "AddOrdinaryNameResults(clang::Sema::ParserCompletionContext, clang::Scope*, clang::Sema&, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder9ExitScopeEv", "(anonymous namespace)::ResultBuilder::ExitScope()"},
    {"_Z24AddPrettyFunctionResultsRKN5clang11LangOptionsERN12_GLOBAL__N_113ResultBuilderE", "AddPrettyFunctionResults(clang::LangOptions const&, (anonymous namespace)::ResultBuilder&)"},
    {"_Z15AddMacroResultsRN5clang12PreprocessorERN12_GLOBAL__N_113ResultBuilderEb", "AddMacroResults(clang::Preprocessor&, (anonymous namespace)::ResultBuilder&, bool)"},
    {"_ZN12_GLOBAL__N_126CodeCompletionDeclConsumerD1Ev", "(anonymous namespace)::CodeCompletionDeclConsumer::~CodeCompletionDeclConsumer()"},
    {"_ZN5clang4Sema20CodeCompleteDeclSpecEPNS_5ScopeERNS_8DeclSpecEbb", "clang::Sema::CodeCompleteDeclSpec(clang::Scope*, clang::DeclSpec&, bool, bool)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder21IsImpossibleToSatisfyEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsImpossibleToSatisfy(clang::NamedDecl*) const"},
    {"_Z26AddClassMessageCompletionsRN5clang4SemaEPNS_5ScopeENS_9OpaquePtrINS_8QualTypeEEEPPNS_14IdentifierInfoEjbbRN12_GLOBAL__N_113ResultBuilderE", "AddClassMessageCompletions(clang::Sema&, clang::Scope*, clang::OpaquePtr<clang::QualType>, clang::IdentifierInfo**, unsigned int, bool, bool, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN5clang4Sema22CodeCompleteExpressionEPNS_5ScopeERKNS0_26CodeCompleteExpressionDataE", "clang::Sema::CodeCompleteExpression(clang::Scope*, clang::Sema::CodeCompleteExpressionData const&)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder16IsObjCCollectionEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsObjCCollection(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder23IsIntegralConstantValueEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsIntegralConstantValue(clang::NamedDecl*) const"},
    {"_ZN5clang4Sema29CodeCompletePostfixExpressionEPNS_5ScopeENS_12ActionResultIPNS_4ExprELb1EEE", "clang::Sema::CodeCompletePostfixExpression(clang::Scope*, clang::ActionResult<clang::Expr*, true>)"},
    {"_ZN5clang4Sema31CodeCompleteObjCInstanceMessageEPNS_5ScopeEPNS_4ExprEPPNS_14IdentifierInfoEjbPNS_17ObjCInterfaceDeclE", "clang::Sema::CodeCompleteObjCInstanceMessage(clang::Scope*, clang::Expr*, clang::IdentifierInfo**, unsigned int, bool, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema31CodeCompleteMemberReferenceExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationEb", "clang::Sema::CodeCompleteMemberReferenceExpr(clang::Scope*, clang::Expr*, clang::SourceLocation, bool)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder8IsMemberEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsMember(clang::NamedDecl*) const"},
    {"_Z17AddObjCPropertiesPN5clang17ObjCContainerDeclEbPNS_11DeclContextERN4llvm11SmallPtrSetIPNS_14IdentifierInfoELj16EEERN12_GLOBAL__N_113ResultBuilderE", "AddObjCProperties(clang::ObjCContainerDecl*, bool, clang::DeclContext*, llvm::SmallPtrSet<clang::IdentifierInfo*, 16u>&, (anonymous namespace)::ResultBuilder&)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder10IsObjCIvarEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsObjCIvar(clang::NamedDecl*) const"},
    {"_ZN5clang4Sema15CodeCompleteTagEPNS_5ScopeEj", "clang::Sema::CodeCompleteTag(clang::Scope*, unsigned int)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder6IsEnumEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsEnum(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder7IsUnionEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsUnion(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder15IsClassOrStructEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsClassOrStruct(clang::NamedDecl*) const"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder21IsNestedNameSpecifierEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsNestedNameSpecifier(clang::NamedDecl*) const"},
    {"_ZN5clang4Sema26CodeCompleteTypeQualifiersERNS_8DeclSpecE", "clang::Sema::CodeCompleteTypeQualifiers(clang::DeclSpec&)"},
    {"_ZN5clang4Sema16CodeCompleteCaseEPNS_5ScopeE", "clang::Sema::CodeCompleteCase(clang::Scope*)"},
    {"_Z24getRequiredQualificationRN5clang10ASTContextEPNS_11DeclContextES3_", "getRequiredQualification(clang::ASTContext&, clang::DeclContext*, clang::DeclContext*)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder9AddResultEN5clang20CodeCompletionResultEPNS1_11DeclContextEPNS1_9NamedDeclEb", "(anonymous namespace)::ResultBuilder::AddResult(clang::CodeCompletionResult, clang::DeclContext*, clang::NamedDecl*, bool)"},
    {"_ZN5clang4Sema16CodeCompleteCallEPNS_5ScopeEPNS_4ExprEPS4_j", "clang::Sema::CodeCompleteCall(clang::Scope*, clang::Expr*, clang::Expr**, unsigned int)"},
    {"_ZN5clang4Sema23CodeCompleteInitializerEPNS_5ScopeEPNS_4DeclE", "clang::Sema::CodeCompleteInitializer(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema18CodeCompleteReturnEPNS_5ScopeE", "clang::Sema::CodeCompleteReturn(clang::Scope*)"},
    {"_ZN5clang4Sema25CodeCompleteAssignmentRHSEPNS_5ScopeEPNS_4ExprE", "clang::Sema::CodeCompleteAssignmentRHS(clang::Scope*, clang::Expr*)"},
    {"_ZN5clang4Sema23CodeCompleteQualifiedIdEPNS_5ScopeERNS_12CXXScopeSpecEb", "clang::Sema::CodeCompleteQualifiedId(clang::Scope*, clang::CXXScopeSpec&, bool)"},
    {"_ZN5clang4Sema17CodeCompleteUsingEPNS_5ScopeE", "clang::Sema::CodeCompleteUsing(clang::Scope*)"},
    {"_ZN5clang4Sema26CodeCompleteUsingDirectiveEPNS_5ScopeE", "clang::Sema::CodeCompleteUsingDirective(clang::Scope*)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder18IsNamespaceOrAliasEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsNamespaceOrAlias(clang::NamedDecl*) const"},
    {"_ZN5clang4Sema25CodeCompleteNamespaceDeclEPNS_5ScopeE", "clang::Sema::CodeCompleteNamespaceDecl(clang::Scope*)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder11IsNamespaceEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsNamespace(clang::NamedDecl*) const"},
    {"_ZN5clang4Sema30CodeCompleteNamespaceAliasDeclEPNS_5ScopeE", "clang::Sema::CodeCompleteNamespaceAliasDecl(clang::Scope*)"},
    {"_ZN5clang4Sema24CodeCompleteOperatorNameEPNS_5ScopeE", "clang::Sema::CodeCompleteOperatorName(clang::Scope*)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder6IsTypeEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsType(clang::NamedDecl*) const"},
    {"_Z23AddTypeSpecifierResultsRKN5clang11LangOptionsERN12_GLOBAL__N_113ResultBuilderE", "AddTypeSpecifierResults(clang::LangOptions const&, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN5clang4Sema34CodeCompleteConstructorInitializerEPNS_4DeclEPPNS_18CXXCtorInitializerEj", "clang::Sema::CodeCompleteConstructorInitializer(clang::Decl*, clang::CXXCtorInitializer**, unsigned int)"},
    {"_ZN5clang4Sema27CodeCompleteObjCAtDirectiveEPNS_5ScopeEPNS_4DeclEb", "clang::Sema::CodeCompleteObjCAtDirective(clang::Scope*, clang::Decl*, bool)"},
    {"_Z28AddObjCImplementationResultsRKN5clang11LangOptionsERN12_GLOBAL__N_113ResultBuilderEb", "AddObjCImplementationResults(clang::LangOptions const&, (anonymous namespace)::ResultBuilder&, bool)"},
    {"_Z23AddObjCInterfaceResultsRKN5clang11LangOptionsERN12_GLOBAL__N_113ResultBuilderEb", "AddObjCInterfaceResults(clang::LangOptions const&, (anonymous namespace)::ResultBuilder&, bool)"},
    {"_Z22AddObjCTopLevelResultsRN12_GLOBAL__N_113ResultBuilderEb", "AddObjCTopLevelResults((anonymous namespace)::ResultBuilder&, bool)"},
    {"_ZN5clang4Sema28CodeCompleteObjCAtVisibilityEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCAtVisibility(clang::Scope*)"},
    {"_Z24AddObjCVisibilityResultsRKN5clang11LangOptionsERN12_GLOBAL__N_113ResultBuilderEb", "AddObjCVisibilityResults(clang::LangOptions const&, (anonymous namespace)::ResultBuilder&, bool)"},
    {"_ZN5clang4Sema27CodeCompleteObjCAtStatementEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCAtStatement(clang::Scope*)"},
    {"_Z23AddObjCStatementResultsRN12_GLOBAL__N_113ResultBuilderEb", "AddObjCStatementResults((anonymous namespace)::ResultBuilder&, bool)"},
    {"_Z24AddObjCExpressionResultsRN12_GLOBAL__N_113ResultBuilderEb", "AddObjCExpressionResults((anonymous namespace)::ResultBuilder&, bool)"},
    {"_ZN5clang4Sema28CodeCompleteObjCAtExpressionEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCAtExpression(clang::Scope*)"},
    {"_ZN5clang4Sema29CodeCompleteObjCPropertyFlagsEPNS_5ScopeERNS_12ObjCDeclSpecE", "clang::Sema::CodeCompleteObjCPropertyFlags(clang::Scope*, clang::ObjCDeclSpec&)"},
    {"_ZN5clang4Sema30CodeCompleteObjCPropertyGetterEPNS_5ScopeEPNS_4DeclE", "clang::Sema::CodeCompleteObjCPropertyGetter(clang::Scope*, clang::Decl*)"},
    {"_Z14AddObjCMethodsPN5clang17ObjCContainerDeclEb14ObjCMethodKindPPNS_14IdentifierInfoEjPNS_11DeclContextERN4llvm11SmallPtrSetINS_8SelectorELj16EEEbRN12_GLOBAL__N_113ResultBuilderEb", "AddObjCMethods(clang::ObjCContainerDecl*, bool, ObjCMethodKind, clang::IdentifierInfo**, unsigned int, clang::DeclContext*, llvm::SmallPtrSet<clang::Selector, 16u>&, bool, (anonymous namespace)::ResultBuilder&, bool)"},
    {"_ZN5clang4Sema30CodeCompleteObjCPropertySetterEPNS_5ScopeEPNS_4DeclE", "clang::Sema::CodeCompleteObjCPropertySetter(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema27CodeCompleteObjCPassingTypeEPNS_5ScopeERNS_12ObjCDeclSpecE", "clang::Sema::CodeCompleteObjCPassingType(clang::Scope*, clang::ObjCDeclSpec&)"},
    {"_ZN5clang4Sema31CodeCompleteObjCMessageReceiverEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCMessageReceiver(clang::Scope*)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder21IsObjCMessageReceiverEPN5clang9NamedDeclE", "(anonymous namespace)::ResultBuilder::IsObjCMessageReceiver(clang::NamedDecl*) const"},
    {"_Z22AddSuperSendCompletionRN5clang4SemaEbPPNS_14IdentifierInfoEjRN12_GLOBAL__N_113ResultBuilderE", "AddSuperSendCompletion(clang::Sema&, bool, clang::IdentifierInfo**, unsigned int, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN5clang4Sema28CodeCompleteObjCSuperMessageEPNS_5ScopeENS_14SourceLocationEPPNS_14IdentifierInfoEjb", "clang::Sema::CodeCompleteObjCSuperMessage(clang::Scope*, clang::SourceLocation, clang::IdentifierInfo**, unsigned int, bool)"},
    {"_ZN5clang4Sema28CodeCompleteObjCClassMessageEPNS_5ScopeENS_9OpaquePtrINS_8QualTypeEEEPPNS_14IdentifierInfoEjbb", "clang::Sema::CodeCompleteObjCClassMessage(clang::Scope*, clang::OpaquePtr<clang::QualType>, clang::IdentifierInfo**, unsigned int, bool, bool)"},
    {"_Z38getPreferredArgumentTypeForMessageSendRN12_GLOBAL__N_113ResultBuilderEj", "getPreferredArgumentTypeForMessageSend((anonymous namespace)::ResultBuilder&, unsigned int)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder14MaybeAddResultEN5clang20CodeCompletionResultEPNS1_11DeclContextE", "(anonymous namespace)::ResultBuilder::MaybeAddResult(clang::CodeCompletionResult, clang::DeclContext*)"},
    {"_ZN5clang4Sema29CodeCompleteObjCForCollectionEPNS_5ScopeENS_9OpaquePtrINS_12DeclGroupRefEEE", "clang::Sema::CodeCompleteObjCForCollection(clang::Scope*, clang::OpaquePtr<clang::DeclGroupRef>)"},
    {"_ZN5clang4Sema24CodeCompleteObjCSelectorEPNS_5ScopeEPPNS_14IdentifierInfoEj", "clang::Sema::CodeCompleteObjCSelector(clang::Scope*, clang::IdentifierInfo**, unsigned int)"},
    {"_Z24isAcceptableObjCSelectorN5clang8SelectorE14ObjCMethodKindPPNS_14IdentifierInfoEjb", "isAcceptableObjCSelector(clang::Selector, ObjCMethodKind, clang::IdentifierInfo**, unsigned int, bool)"},
    {"_ZN5clang4Sema34CodeCompleteObjCProtocolReferencesEPSt4pairIPNS_14IdentifierInfoENS_14SourceLocationEEj", "clang::Sema::CodeCompleteObjCProtocolReferences(std::pair<clang::IdentifierInfo*, clang::SourceLocation>*, unsigned int)"},
    {"_Z18AddProtocolResultsPN5clang11DeclContextES1_bRN12_GLOBAL__N_113ResultBuilderE", "AddProtocolResults(clang::DeclContext*, clang::DeclContext*, bool, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN5clang4Sema28CodeCompleteObjCProtocolDeclEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCProtocolDecl(clang::Scope*)"},
    {"_ZN5clang4Sema29CodeCompleteObjCInterfaceDeclEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCInterfaceDecl(clang::Scope*)"},
    {"_Z19AddInterfaceResultsPN5clang11DeclContextES1_bbRN12_GLOBAL__N_113ResultBuilderE", "AddInterfaceResults(clang::DeclContext*, clang::DeclContext*, bool, bool, (anonymous namespace)::ResultBuilder&)"},
    {"_ZN5clang4Sema26CodeCompleteObjCSuperclassEPNS_5ScopeEPNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::CodeCompleteObjCSuperclass(clang::Scope*, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema34CodeCompleteObjCImplementationDeclEPNS_5ScopeE", "clang::Sema::CodeCompleteObjCImplementationDecl(clang::Scope*)"},
    {"_ZN5clang4Sema33CodeCompleteObjCInterfaceCategoryEPNS_5ScopeEPNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::CodeCompleteObjCInterfaceCategory(clang::Scope*, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema38CodeCompleteObjCImplementationCategoryEPNS_5ScopeEPNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::CodeCompleteObjCImplementationCategory(clang::Scope*, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema34CodeCompleteObjCPropertyDefinitionEPNS_5ScopeEPNS_4DeclE", "clang::Sema::CodeCompleteObjCPropertyDefinition(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema38CodeCompleteObjCPropertySynthesizeIvarEPNS_5ScopeEPNS_14IdentifierInfoEPNS_4DeclE", "clang::Sema::CodeCompleteObjCPropertySynthesizeIvar(clang::Scope*, clang::IdentifierInfo*, clang::Decl*)"},
    {"_ZN5clang4Sema26CodeCompleteObjCMethodDeclEPNS_5ScopeEbNS_9OpaquePtrINS_8QualTypeEEEPNS_4DeclE", "clang::Sema::CodeCompleteObjCMethodDecl(clang::Scope*, bool, clang::OpaquePtr<clang::QualType>, clang::Decl*)"},
    {"_Z24FindImplementableMethodsRN5clang10ASTContextEPNS_17ObjCContainerDeclEbNS_8QualTypeERN4llvm8DenseMapINS_8SelectorESt4pairIPNS_14ObjCMethodDeclEbENS5_12DenseMapInfoIS7_EENSC_ISB_EEEEb", "FindImplementableMethods(clang::ASTContext&, clang::ObjCContainerDecl*, bool, clang::QualType, llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodDecl*, bool>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodDecl*, bool> > >&, bool)"},
    {"_ZN5clang4Sema34CodeCompleteObjCMethodDeclSelectorEPNS_5ScopeEbbNS_9OpaquePtrINS_8QualTypeEEEPPNS_14IdentifierInfoEj", "clang::Sema::CodeCompleteObjCMethodDeclSelector(clang::Scope*, bool, bool, clang::OpaquePtr<clang::QualType>, clang::IdentifierInfo**, unsigned int)"},
    {"_ZN5clang4Sema33CodeCompletePreprocessorDirectiveEb", "clang::Sema::CodeCompletePreprocessorDirective(bool)"},
    {"_ZN5clang4Sema46CodeCompleteInPreprocessorConditionalExclusionEPNS_5ScopeE", "clang::Sema::CodeCompleteInPreprocessorConditionalExclusion(clang::Scope*)"},
    {"_ZN5clang4Sema33CodeCompletePreprocessorMacroNameEb", "clang::Sema::CodeCompletePreprocessorMacroName(bool)"},
    {"_ZN5clang4Sema34CodeCompletePreprocessorExpressionEv", "clang::Sema::CodeCompletePreprocessorExpression()"},
    {"_ZN5clang4Sema37CodeCompletePreprocessorMacroArgumentEPNS_5ScopeEPNS_14IdentifierInfoEPNS_9MacroInfoEj", "clang::Sema::CodeCompletePreprocessorMacroArgument(clang::Scope*, clang::IdentifierInfo*, clang::MacroInfo*, unsigned int)"},
    {"_ZN5clang4Sema27CodeCompleteNaturalLanguageEv", "clang::Sema::CodeCompleteNaturalLanguage()"},
    {"_ZN5clang4Sema27GatherGlobalCodeCompletionsERNS_23CodeCompletionAllocatorERN4llvm15SmallVectorImplINS_20CodeCompletionResultEEE", "clang::Sema::GatherGlobalCodeCompletions(clang::CodeCompletionAllocator&, llvm::SmallVectorImpl<clang::CodeCompletionResult>&)"},
    {"_ZSt21__inplace_stable_sortIPN5clang17OverloadCandidateEN12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_T0_", "void std::__inplace_stable_sort<clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt22__stable_sort_adaptiveIPN5clang17OverloadCandidateES2_lN12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_T0_T1_T2_", "void std::__stable_sort_adaptive<clang::OverloadCandidate*, clang::OverloadCandidate*, long, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, long, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt24__merge_sort_with_bufferIPN5clang17OverloadCandidateES2_N12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_T0_T1_", "void std::__merge_sort_with_buffer<clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt16__merge_adaptiveIPN5clang17OverloadCandidateElS2_N12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_S5_T0_S6_T1_S6_T2_", "void std::__merge_adaptive<clang::OverloadCandidate*, long, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, long, long, clang::OverloadCandidate*, long, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt5mergeIPN5clang17OverloadCandidateES2_S2_N12_GLOBAL__N_125IsBetterOverloadCandidateEET1_T_S6_T0_S7_S5_T2_", "clang::OverloadCandidate* std::merge<clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt16__insertion_sortIPN5clang17OverloadCandidateEN12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_T0_", "void std::__insertion_sort<clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZSt22__merge_without_bufferIPN5clang17OverloadCandidateElN12_GLOBAL__N_125IsBetterOverloadCandidateEEvT_S5_S5_T0_S6_T1_", "void std::__merge_without_buffer<clang::OverloadCandidate*, long, (anonymous namespace)::IsBetterOverloadCandidate>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, long, long, (anonymous namespace)::IsBetterOverloadCandidate)"},
    {"_ZNK12_GLOBAL__N_113ResultBuilder17isInterestingDeclEPN5clang9NamedDeclERb", "(anonymous namespace)::ResultBuilder::isInterestingDecl(clang::NamedDecl*, bool&) const"},
    {"_ZN12_GLOBAL__N_113ResultBuilder17CheckHiddenResultERN5clang20CodeCompletionResultEPNS1_11DeclContextEPNS1_9NamedDeclE", "(anonymous namespace)::ResultBuilder::CheckHiddenResult(clang::CodeCompletionResult&, clang::DeclContext*, clang::NamedDecl*)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder27AdjustResultPriorityForDeclERN5clang20CodeCompletionResultE", "(anonymous namespace)::ResultBuilder::AdjustResultPriorityForDecl(clang::CodeCompletionResult&)"},
    {"_ZN12_GLOBAL__N_113ResultBuilder26MaybeAddConstructorResultsEN5clang20CodeCompletionResultE", "(anonymous namespace)::ResultBuilder::MaybeAddConstructorResults(clang::CodeCompletionResult)"},
    {"_ZN12_GLOBAL__N_126CodeCompletionDeclConsumerD0Ev", "(anonymous namespace)::CodeCompletionDeclConsumer::~CodeCompletionDeclConsumer()"},
    {"_ZN12_GLOBAL__N_126CodeCompletionDeclConsumer9FoundDeclEPN5clang9NamedDeclES3_b", "(anonymous namespace)::CodeCompletionDeclConsumer::FoundDecl(clang::NamedDecl*, clang::NamedDecl*, bool)"},
    {"_Z16AddTypedefResultRN12_GLOBAL__N_113ResultBuilderE", "AddTypedefResult((anonymous namespace)::ResultBuilder&)"},
    {"_Z20AddStorageSpecifiersN5clang4Sema23ParserCompletionContextERKNS_11LangOptionsERN12_GLOBAL__N_113ResultBuilderE", "AddStorageSpecifiers(clang::Sema::ParserCompletionContext, clang::LangOptions const&, (anonymous namespace)::ResultBuilder&)"},
    {"_Z21AddFunctionSpecifiersN5clang4Sema23ParserCompletionContextERKNS_11LangOptionsERN12_GLOBAL__N_113ResultBuilderE", "AddFunctionSpecifiers(clang::Sema::ParserCompletionContext, clang::LangOptions const&, (anonymous namespace)::ResultBuilder&)"},
    {"_ZNSt8_Rb_treeIPN5clang13NamespaceDeclESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::NamespaceDecl*, std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*>, std::_Select1st<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> >, std::less<clang::NamespaceDecl*>, std::allocator<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> >, std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang13NamespaceDeclESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::NamespaceDecl*, std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*>, std::_Select1st<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> >, std::less<clang::NamespaceDecl*>, std::allocator<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> > >::_M_insert_unique(std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> const&)"},
    {"_ZNSt17_Temporary_bufferIPN5clang17OverloadCandidateES1_ED2Ev", "std::_Temporary_buffer<clang::OverloadCandidate*, clang::OverloadCandidate>::~_Temporary_buffer()"},
    {"_ZSt17__rotate_adaptiveIPN5clang17OverloadCandidateES2_lET_S3_S3_S3_T1_S4_T0_S4_", "clang::OverloadCandidate* std::__rotate_adaptive<clang::OverloadCandidate*, clang::OverloadCandidate*, long>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, long, long, clang::OverloadCandidate*, long)"},
    {"_ZSt8__rotateIPN5clang17OverloadCandidateEEvT_S3_S3_St26random_access_iterator_tag", "void std::__rotate<clang::OverloadCandidate*>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, std::random_access_iterator_tag)"},
    {"_ZN4llvm15SmallVectorImplIN5clang26ImplicitConversionSequenceEEaSERKS3_", "llvm::SmallVectorImpl<clang::ImplicitConversionSequence>::operator=(llvm::SmallVectorImpl<clang::ImplicitConversionSequence> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang26ImplicitConversionSequenceELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ImplicitConversionSequence, false>::grow(unsigned long)"},
    {"_ZSt4swapIN5clang17OverloadCandidateEEvRT_S3_", "void std::swap<clang::OverloadCandidate>(clang::OverloadCandidate&, clang::OverloadCandidate&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN5clang17OverloadCandidateES5_EET0_T_S7_S6_", "clang::OverloadCandidate* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<clang::OverloadCandidate*, clang::OverloadCandidate*>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPN5clang17OverloadCandidateES5_EET0_T_S7_S6_", "clang::OverloadCandidate* std::__copy<false, std::random_access_iterator_tag>::copy<clang::OverloadCandidate*, clang::OverloadCandidate*>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*)"},
    {"_ZSt26__uninitialized_fill_n_auxIPN5clang17OverloadCandidateElS1_EvT_T0_RKT1_St12__false_type", "void std::__uninitialized_fill_n_aux<clang::OverloadCandidate*, long, clang::OverloadCandidate>(clang::OverloadCandidate*, long, clang::OverloadCandidate const&, std::__false_type)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang20CodeCompleteConsumer17OverloadCandidateELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CodeCompleteConsumer::OverloadCandidate, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorESt4pairIPNS1_14ObjCMethodDeclEbENS_12DenseMapInfoIS2_EENS7_IS6_EEE16InsertIntoBucketERKS2_RKS6_PS3_IS2_S6_E", "llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodDecl*, bool>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodDecl*, bool> > >::InsertIntoBucket(clang::Selector const&, std::pair<clang::ObjCMethodDecl*, bool> const&, std::pair<clang::Selector, std::pair<clang::ObjCMethodDecl*, bool> >*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorESt4pairIPNS1_14ObjCMethodDeclEbENS_12DenseMapInfoIS2_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodDecl*, bool>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodDecl*, bool> > >::grow(unsigned int)"},
    {"_ZNSt6vectorIN5clang20CodeCompletionResultESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::CodeCompletionResult, std::allocator<clang::CodeCompletionResult> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CodeCompletionResult*, std::vector<clang::CodeCompletionResult, std::allocator<clang::CodeCompletionResult> > >, clang::CodeCompletionResult const&)"},
    {"_ZNSt8_Rb_treeIPN5clang13NamespaceDeclESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::NamespaceDecl*, std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*>, std::_Select1st<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> >, std::less<clang::NamespaceDecl*>, std::allocator<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::NamespaceDecl* const, clang::NamespaceDecl*> >*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang17OverloadCandidateEED2Ev", "llvm::SmallVectorImpl<clang::OverloadCandidate>::~SmallVectorImpl()"},
    {"_ZNK5clang4Decl7getAttrINS_12SentinelAttrEEEPT_v", "clang::SentinelAttr* clang::Decl::getAttr<clang::SentinelAttr>() const"},
    {"_ZN5clang4Sema22ConvertDeclToDeclGroupEPNS_4DeclE", "clang::Sema::ConvertDeclToDeclGroup(clang::Decl*)"},
    {"_ZN5clang4Sema11getTypeNameERNS_14IdentifierInfoENS_14SourceLocationEPNS_5ScopeEPNS_12CXXScopeSpecEbbNS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::getTypeName(clang::IdentifierInfo&, clang::SourceLocation, clang::Scope*, clang::CXXScopeSpec*, bool, bool, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang4Sema9isTagNameERNS_14IdentifierInfoEPNS_5ScopeE", "clang::Sema::isTagName(clang::IdentifierInfo&, clang::Scope*)"},
    {"_ZN5clang4Sema23DiagnoseUnknownTypeNameERKNS_14IdentifierInfoENS_14SourceLocationEPNS_5ScopeEPNS_12CXXScopeSpecERNS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::DiagnoseUnknownTypeName(clang::IdentifierInfo const&, clang::SourceLocation, clang::Scope*, clang::CXXScopeSpec*, clang::OpaquePtr<clang::QualType>&)"},
    {"_ZN5clang4Sema15getContainingDCEPNS_11DeclContextE", "clang::Sema::getContainingDC(clang::DeclContext*)"},
    {"_ZN5clang4Sema15PushDeclContextEPNS_5ScopeEPNS_11DeclContextE", "clang::Sema::PushDeclContext(clang::Scope*, clang::DeclContext*)"},
    {"_ZN5clang4Sema14PopDeclContextEv", "clang::Sema::PopDeclContext()"},
    {"_ZN5clang4Sema22EnterDeclaratorContextEPNS_5ScopeEPNS_11DeclContextE", "clang::Sema::EnterDeclaratorContext(clang::Scope*, clang::DeclContext*)"},
    {"_ZN5clang4Sema21ExitDeclaratorContextEPNS_5ScopeE", "clang::Sema::ExitDeclaratorContext(clang::Scope*)"},
    {"_ZN5clang4Sema17PushOnScopeChainsEPNS_9NamedDeclEPNS_5ScopeEb", "clang::Sema::PushOnScopeChains(clang::NamedDecl*, clang::Scope*, bool)"},
    {"_ZN5clang4Sema13isDeclInScopeERPNS_9NamedDeclEPNS_11DeclContextEPNS_5ScopeE", "clang::Sema::isDeclInScope(clang::NamedDecl*&, clang::DeclContext*, clang::Scope*)"},
    {"_ZN5clang4Sema22getScopeForDeclContextEPNS_5ScopeEPNS_11DeclContextE", "clang::Sema::getScopeForDeclContext(clang::Scope*, clang::DeclContext*)"},
    {"_ZNK5clang4Sema32ShouldWarnIfUnusedFileScopedDeclEPKNS_14DeclaratorDeclE", "clang::Sema::ShouldWarnIfUnusedFileScopedDecl(clang::DeclaratorDecl const*) const"},
    {"_ZN5clang4Sema24MarkUnusedFileScopedDeclEPKNS_14DeclaratorDeclE", "clang::Sema::MarkUnusedFileScopedDecl(clang::DeclaratorDecl const*)"},
    {"_ZN5clang4Sema18DiagnoseUnusedDeclEPKNS_9NamedDeclE", "clang::Sema::DiagnoseUnusedDecl(clang::NamedDecl const*)"},
    {"_ZN5clang4Sema13ActOnPopScopeENS_14SourceLocationEPNS_5ScopeE", "clang::Sema::ActOnPopScope(clang::SourceLocation, clang::Scope*)"},
    {"_ZN5clang4Sema20getObjCInterfaceDeclERPNS_14IdentifierInfoENS_14SourceLocationEb", "clang::Sema::getObjCInterfaceDecl(clang::IdentifierInfo*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema20getNonFieldDeclScopeEPNS_5ScopeE", "clang::Sema::getNonFieldDeclScope(clang::Scope*)"},
    {"_ZN5clang4Sema19LazilyCreateBuiltinEPNS_14IdentifierInfoEjPNS_5ScopeEbNS_14SourceLocationE", "clang::Sema::LazilyCreateBuiltin(clang::IdentifierInfo*, unsigned int, clang::Scope*, bool, clang::SourceLocation)"},
    {"_ZN5clang4Sema26AddKnownFunctionAttributesEPNS_12FunctionDeclE", "clang::Sema::AddKnownFunctionAttributes(clang::FunctionDecl*)"},
    {"_ZN5clang4Sema16MergeTypeDefDeclEPNS_11TypedefDeclERNS_12LookupResultE", "clang::Sema::MergeTypeDefDecl(clang::TypedefDecl*, clang::LookupResult&)"},
    {"_ZN5clang4Sema16getSpecialMemberEPKNS_13CXXMethodDeclE", "clang::Sema::getSpecialMember(clang::CXXMethodDecl const*)"},
    {"_ZN5clang4Sema17MergeFunctionDeclEPNS_12FunctionDeclEPNS_4DeclE", "clang::Sema::MergeFunctionDecl(clang::FunctionDecl*, clang::Decl*)"},
    {"_ZN5clang4Sema28MergeCompatibleFunctionDeclsEPNS_12FunctionDeclES2_", "clang::Sema::MergeCompatibleFunctionDecls(clang::FunctionDecl*, clang::FunctionDecl*)"},
    {"_Z19MergeDeclAttributesPN5clang4DeclES1_RNS_10ASTContextE", "MergeDeclAttributes(clang::Decl*, clang::Decl*, clang::ASTContext&)"},
    {"_ZN5clang4Sema12MergeVarDeclEPNS_7VarDeclERNS_12LookupResultE", "clang::Sema::MergeVarDecl(clang::VarDecl*, clang::LookupResult&)"},
    {"_ZN5clang4Sema26ParsedFreeStandingDeclSpecEPNS_5ScopeENS_15AccessSpecifierERNS_8DeclSpecE", "clang::Sema::ParsedFreeStandingDeclSpec(clang::Scope*, clang::AccessSpecifier, clang::DeclSpec&)"},
    {"_ZN5clang4Sema27BuildAnonymousStructOrUnionEPNS_5ScopeERNS_8DeclSpecENS_15AccessSpecifierEPNS_10RecordDeclE", "clang::Sema::BuildAnonymousStructOrUnion(clang::Scope*, clang::DeclSpec&, clang::AccessSpecifier, clang::RecordDecl*)"},
    {"_ZN5clang4Sema30BuildMicrosoftCAnonymousStructEPNS_5ScopeERNS_8DeclSpecEPNS_10RecordDeclE", "clang::Sema::BuildMicrosoftCAnonymousStruct(clang::Scope*, clang::DeclSpec&, clang::RecordDecl*)"},
    {"_ZN5clang4Sema12ActOnVlaStmtERKNS_8DeclSpecE", "clang::Sema::ActOnVlaStmt(clang::DeclSpec const&)"},
    {"_ZN5clang4Sema20CheckNontrivialFieldEPNS_9FieldDeclE", "clang::Sema::CheckNontrivialField(clang::FieldDecl*)"},
    {"_Z35InjectAnonymousStructOrUnionMembersRN5clang4SemaEPNS_5ScopeEPNS_11DeclContextEPNS_10RecordDeclENS_15AccessSpecifierERN4llvm11SmallVectorIPNS_9NamedDeclELj2EEEb", "InjectAnonymousStructOrUnionMembers(clang::Sema&, clang::Scope*, clang::DeclContext*, clang::RecordDecl*, clang::AccessSpecifier, llvm::SmallVector<clang::NamedDecl*, 2u>&, bool)"},
    {"_ZN5clang4Sema20GetNameForDeclaratorERNS_10DeclaratorE", "clang::Sema::GetNameForDeclarator(clang::Declarator&)"},
    {"_ZN5clang4Sema24GetNameFromUnqualifiedIdERKNS_13UnqualifiedIdE", "clang::Sema::GetNameFromUnqualifiedId(clang::UnqualifiedId const&)"},
    {"_ZN5clang4Sema15ActOnDeclaratorEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnDeclarator(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema16HandleDeclaratorEPNS_5ScopeERNS_10DeclaratorENS_11ASTMultiPtrIPNS_21TemplateParameterListEEEb", "clang::Sema::HandleDeclarator(clang::Scope*, clang::Declarator&, clang::ASTMultiPtr<clang::TemplateParameterList*>, bool)"},
    {"_ZN5clang4Sema22ActOnTypedefDeclaratorEPNS_5ScopeERNS_10DeclaratorEPNS_11DeclContextENS_8QualTypeEPNS_14TypeSourceInfoERNS_12LookupResultERb", "clang::Sema::ActOnTypedefDeclarator(clang::Scope*, clang::Declarator&, clang::DeclContext*, clang::QualType, clang::TypeSourceInfo*, clang::LookupResult&, bool&)"},
    {"_ZN5clang4Sema23ActOnFunctionDeclaratorEPNS_5ScopeERNS_10DeclaratorEPNS_11DeclContextENS_8QualTypeEPNS_14TypeSourceInfoERNS_12LookupResultENS_11ASTMultiPtrIPNS_21TemplateParameterListEEEbRb", "clang::Sema::ActOnFunctionDeclarator(clang::Scope*, clang::Declarator&, clang::DeclContext*, clang::QualType, clang::TypeSourceInfo*, clang::LookupResult&, clang::ASTMultiPtr<clang::TemplateParameterList*>, bool, bool&)"},
    {"_ZN5clang4Sema23ActOnVariableDeclaratorEPNS_5ScopeERNS_10DeclaratorEPNS_11DeclContextENS_8QualTypeEPNS_14TypeSourceInfoERNS_12LookupResultENS_11ASTMultiPtrIPNS_21TemplateParameterListEEERb", "clang::Sema::ActOnVariableDeclarator(clang::Scope*, clang::Declarator&, clang::DeclContext*, clang::QualType, clang::TypeSourceInfo*, clang::LookupResult&, clang::ASTMultiPtr<clang::TemplateParameterList*>, bool&)"},
    {"_ZN5clang4Sema32RegisterLocallyScopedExternCDeclEPNS_9NamedDeclERKNS_12LookupResultEPNS_5ScopeE", "clang::Sema::RegisterLocallyScopedExternCDecl(clang::NamedDecl*, clang::LookupResult const&, clang::Scope*)"},
    {"_ZN5clang4Sema26DiagnoseFunctionSpecifiersERNS_10DeclaratorE", "clang::Sema::DiagnoseFunctionSpecifiers(clang::Declarator&)"},
    {"_ZN5clang4Sema16ParseTypedefDeclEPNS_5ScopeERNS_10DeclaratorENS_8QualTypeEPNS_14TypeSourceInfoE", "clang::Sema::ParseTypedefDecl(clang::Scope*, clang::Declarator&, clang::QualType, clang::TypeSourceInfo*)"},
    {"_Z35TryToFixInvalidVariablyModifiedTypeN5clang8QualTypeERNS_10ASTContextERbRN4llvm6APSIntE", "TryToFixInvalidVariablyModifiedType(clang::QualType, clang::ASTContext&, bool&, llvm::APSInt&)"},
    {"_Z20FilterLookupForScopeRN5clang4SemaERNS_12LookupResultEPNS_11DeclContextEPNS_5ScopeEb", "FilterLookupForScope(clang::Sema&, clang::LookupResult&, clang::DeclContext*, clang::Scope*, bool)"},
    {"_ZN5clang4Sema11CheckShadowEPNS_5ScopeEPNS_7VarDeclERKNS_12LookupResultE", "clang::Sema::CheckShadow(clang::Scope*, clang::VarDecl*, clang::LookupResult const&)"},
    {"_ZN5clang4Sema24CheckVariableDeclarationEPNS_7VarDeclERNS_12LookupResultERb", "clang::Sema::CheckVariableDeclaration(clang::VarDecl*, clang::LookupResult&, bool&)"},
    {"_ZN5clang4Sema11CheckShadowEPNS_5ScopeEPNS_7VarDeclE", "clang::Sema::CheckShadow(clang::Scope*, clang::VarDecl*)"},
    {"_ZN5clang4Sema20AddOverriddenMethodsEPNS_13CXXRecordDeclEPNS_13CXXMethodDeclE", "clang::Sema::AddOverriddenMethods(clang::CXXRecordDecl*, clang::CXXMethodDecl*)"},
    {"_Z20FindOverriddenMethodPKN5clang16CXXBaseSpecifierERNS_11CXXBasePathEPv", "FindOverriddenMethod(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang4Sema26BuildParmVarDeclForTypedefEPNS_11DeclContextENS_14SourceLocationENS_8QualTypeE", "clang::Sema::BuildParmVarDeclForTypedef(clang::DeclContext*, clang::SourceLocation, clang::QualType)"},
    {"_ZN5clang4Sema24CheckFunctionDeclarationEPNS_5ScopeEPNS_12FunctionDeclERNS_12LookupResultEbRb", "clang::Sema::CheckFunctionDeclaration(clang::Scope*, clang::FunctionDecl*, clang::LookupResult&, bool, bool&)"},
    {"_Z28DiagnoseInvalidRedeclarationRN5clang4SemaEPNS_12FunctionDeclE", "DiagnoseInvalidRedeclaration(clang::Sema&, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema9CheckMainEPNS_12FunctionDeclE", "clang::Sema::CheckMain(clang::FunctionDecl*)"},
    {"_ZN5clang4Sema27CheckForConstantInitializerEPNS_4ExprENS_8QualTypeE", "clang::Sema::CheckForConstantInitializer(clang::Expr*, clang::QualType)"},
    {"_ZN5clang4Sema20AddInitializerToDeclEPNS_4DeclEPNS_4ExprE", "clang::Sema::AddInitializerToDecl(clang::Decl*, clang::Expr*)"},
    {"_ZN5clang4Sema20AddInitializerToDeclEPNS_4DeclEPNS_4ExprEb", "clang::Sema::AddInitializerToDecl(clang::Decl*, clang::Expr*, bool)"},
    {"_ZN5clang4Sema32CheckCompleteVariableDeclarationEPNS_7VarDeclE", "clang::Sema::CheckCompleteVariableDeclaration(clang::VarDecl*)"},
    {"_ZN5clang4Sema21ActOnInitializerErrorEPNS_4DeclE", "clang::Sema::ActOnInitializerError(clang::Decl*)"},
    {"_ZN5clang4Sema22ActOnUninitializedDeclEPNS_4DeclEb", "clang::Sema::ActOnUninitializedDecl(clang::Decl*, bool)"},
    {"_ZN5clang4Sema23FinalizeDeclaratorGroupEPNS_5ScopeERKNS_8DeclSpecEPPNS_4DeclEj", "clang::Sema::FinalizeDeclaratorGroup(clang::Scope*, clang::DeclSpec const&, clang::Decl**, unsigned int)"},
    {"_ZN5clang4Sema20ActOnParamDeclaratorEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnParamDeclarator(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema14CheckParameterEPNS_11DeclContextEPNS_14TypeSourceInfoENS_8QualTypeEPNS_14IdentifierInfoENS_14SourceLocationENS_12StorageClassES9_", "clang::Sema::CheckParameter(clang::DeclContext*, clang::TypeSourceInfo*, clang::QualType, clang::IdentifierInfo*, clang::SourceLocation, clang::StorageClass, clang::StorageClass)"},
    {"_ZN5clang4Sema24DiagnoseUnusedParametersEPKPNS_11ParmVarDeclES4_", "clang::Sema::DiagnoseUnusedParameters(clang::ParmVarDecl* const*, clang::ParmVarDecl* const*)"},
    {"_ZN5clang4Sema38DiagnoseSizeOfParametersAndReturnValueEPKPNS_11ParmVarDeclES4_NS_8QualTypeEPNS_9NamedDeclE", "clang::Sema::DiagnoseSizeOfParametersAndReturnValue(clang::ParmVarDecl* const*, clang::ParmVarDecl* const*, clang::QualType, clang::NamedDecl*)"},
    {"_ZN5clang4Sema31ActOnFinishKNRParamDeclarationsEPNS_5ScopeERNS_10DeclaratorENS_14SourceLocationE", "clang::Sema::ActOnFinishKNRParamDeclarations(clang::Scope*, clang::Declarator&, clang::SourceLocation)"},
    {"_ZN5clang4Sema23ActOnStartOfFunctionDefEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnStartOfFunctionDef(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema23ActOnStartOfFunctionDefEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnStartOfFunctionDef(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema23ActOnFinishFunctionBodyEPNS_4DeclEPNS_4StmtE", "clang::Sema::ActOnFinishFunctionBody(clang::Decl*, clang::Stmt*)"},
    {"_ZN5clang4Sema23ActOnFinishFunctionBodyEPNS_4DeclEPNS_4StmtEb", "clang::Sema::ActOnFinishFunctionBody(clang::Decl*, clang::Stmt*, bool)"},
    {"_ZN5clang4Sema24ImplicitlyDefineFunctionENS_14SourceLocationERNS_14IdentifierInfoEPNS_5ScopeE", "clang::Sema::ImplicitlyDefineFunction(clang::SourceLocation, clang::IdentifierInfo&, clang::Scope*)"},
    {"_ZN5clang4Sema28isAcceptableTagRedeclarationEPKNS_7TagDeclENS_11TagTypeKindENS_14SourceLocationERKNS_14IdentifierInfoE", "clang::Sema::isAcceptableTagRedeclaration(clang::TagDecl const*, clang::TagTypeKind, clang::SourceLocation, clang::IdentifierInfo const&)"},
    {"_ZN5clang4Sema8ActOnTagEPNS_5ScopeEjNS0_10TagUseKindENS_14SourceLocationERNS_12CXXScopeSpecEPNS_14IdentifierInfoES4_PNS_13AttributeListENS_15AccessSpecifierENS_11ASTMultiPtrIPNS_21TemplateParameterListEEERbSG_bbNS_12ActionResultINS_9OpaquePtrINS_8QualTypeEEELb0EEE", "clang::Sema::ActOnTag(clang::Scope*, unsigned int, clang::Sema::TagUseKind, clang::SourceLocation, clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::SourceLocation, clang::AttributeList*, clang::AccessSpecifier, clang::ASTMultiPtr<clang::TemplateParameterList*>, bool&, bool&, bool, bool, clang::ActionResult<clang::OpaquePtr<clang::QualType>, false>)"},
    {"_ZN5clang4Sema23ActOnTagStartDefinitionEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnTagStartDefinition(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema31ActOnStartCXXMemberDeclarationsEPNS_5ScopeEPNS_4DeclERNS_19ClassVirtSpecifiersENS_14SourceLocationE", "clang::Sema::ActOnStartCXXMemberDeclarations(clang::Scope*, clang::Decl*, clang::ClassVirtSpecifiers&, clang::SourceLocation)"},
    {"_ZN5clang4Sema24ActOnTagFinishDefinitionEPNS_5ScopeEPNS_4DeclENS_14SourceLocationE", "clang::Sema::ActOnTagFinishDefinition(clang::Scope*, clang::Decl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema23ActOnTagDefinitionErrorEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnTagDefinitionError(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema14VerifyBitFieldENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPKNS_4ExprEPb", "clang::Sema::VerifyBitField(clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::Expr const*, bool*)"},
    {"_ZN5clang4Sema10ActOnFieldEPNS_5ScopeEPNS_4DeclENS_14SourceLocationERNS_10DeclaratorEPNS_4ExprE", "clang::Sema::ActOnField(clang::Scope*, clang::Decl*, clang::SourceLocation, clang::Declarator&, clang::Expr*)"},
    {"_ZN5clang4Sema11HandleFieldEPNS_5ScopeEPNS_10RecordDeclENS_14SourceLocationERNS_10DeclaratorEPNS_4ExprENS_15AccessSpecifierE", "clang::Sema::HandleField(clang::Scope*, clang::RecordDecl*, clang::SourceLocation, clang::Declarator&, clang::Expr*, clang::AccessSpecifier)"},
    {"_ZN5clang4Sema14CheckFieldDeclENS_15DeclarationNameENS_8QualTypeEPNS_14TypeSourceInfoEPNS_10RecordDeclENS_14SourceLocationEbPNS_4ExprES7_NS_15AccessSpecifierEPNS_9NamedDeclEPNS_10DeclaratorE", "clang::Sema::CheckFieldDecl(clang::DeclarationName, clang::QualType, clang::TypeSourceInfo*, clang::RecordDecl*, clang::SourceLocation, bool, clang::Expr*, clang::SourceLocation, clang::AccessSpecifier, clang::NamedDecl*, clang::Declarator*)"},
    {"_ZN5clang4Sema18DiagnoseNontrivialEPKNS_10RecordTypeENS0_16CXXSpecialMemberE", "clang::Sema::DiagnoseNontrivial(clang::RecordType const*, clang::Sema::CXXSpecialMember)"},
    {"_ZN5clang4Sema9ActOnIvarEPNS_5ScopeENS_14SourceLocationEPNS_4DeclERNS_10DeclaratorEPNS_4ExprENS_3tok15ObjCKeywordKindE", "clang::Sema::ActOnIvar(clang::Scope*, clang::SourceLocation, clang::Decl*, clang::Declarator&, clang::Expr*, clang::tok::ObjCKeywordKind)"},
    {"_ZN5clang4Sema17ActOnLastBitfieldENS_14SourceLocationEPNS_4DeclERN4llvm15SmallVectorImplIS3_EE", "clang::Sema::ActOnLastBitfield(clang::SourceLocation, clang::Decl*, llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang4Sema11ActOnFieldsEPNS_5ScopeENS_14SourceLocationEPNS_4DeclEPS5_jS3_S3_PNS_13AttributeListE", "clang::Sema::ActOnFields(clang::Scope*, clang::SourceLocation, clang::Decl*, clang::Decl**, unsigned int, clang::SourceLocation, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZN5clang4Sema17CheckEnumConstantEPNS_8EnumDeclEPNS_16EnumConstantDeclENS_14SourceLocationEPNS_14IdentifierInfoEPNS_4ExprE", "clang::Sema::CheckEnumConstant(clang::EnumDecl*, clang::EnumConstantDecl*, clang::SourceLocation, clang::IdentifierInfo*, clang::Expr*)"},
    {"_Z27isRepresentableIntegerValueRN5clang10ASTContextERN4llvm6APSIntENS_8QualTypeE", "isRepresentableIntegerValue(clang::ASTContext&, llvm::APSInt&, clang::QualType)"},
    {"_ZN5clang4Sema17ActOnEnumConstantEPNS_5ScopeEPNS_4DeclES4_NS_14SourceLocationEPNS_14IdentifierInfoEPNS_13AttributeListES5_PNS_4ExprE", "clang::Sema::ActOnEnumConstant(clang::Scope*, clang::Decl*, clang::Decl*, clang::SourceLocation, clang::IdentifierInfo*, clang::AttributeList*, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema13ActOnEnumBodyENS_14SourceLocationES1_S1_PNS_4DeclEPS3_jPNS_5ScopeEPNS_13AttributeListE", "clang::Sema::ActOnEnumBody(clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::Decl*, clang::Decl**, unsigned int, clang::Scope*, clang::AttributeList*)"},
    {"_ZN5clang4Sema21ActOnFileScopeAsmDeclENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnFileScopeAsmDecl(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema17ActOnPragmaWeakIDEPNS_14IdentifierInfoENS_14SourceLocationES3_", "clang::Sema::ActOnPragmaWeakID(clang::IdentifierInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema20ActOnPragmaWeakAliasEPNS_14IdentifierInfoES2_NS_14SourceLocationES3_S3_", "clang::Sema::ActOnPragmaWeakAlias(clang::IdentifierInfo*, clang::IdentifierInfo*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang12RedeclarableINS_11TypedefDeclEE22setPreviousDeclarationEPS1_", "clang::Redeclarable<clang::TypedefDecl>::setPreviousDeclaration(clang::TypedefDecl*)"},
    {"_ZN5clang12RedeclarableINS_7VarDeclEE22setPreviousDeclarationEPS1_", "clang::Redeclarable<clang::VarDecl>::setPreviousDeclaration(clang::VarDecl*)"},
    {"_ZNK5clang4Decl7getAttrINS_13DLLImportAttrEEEPT_v", "clang::DLLImportAttr* clang::Decl::getAttr<clang::DLLImportAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_13DLLExportAttrEEEPT_v", "clang::DLLExportAttr* clang::Decl::getAttr<clang::DLLExportAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_10FormatAttrEEEPT_v", "clang::FormatAttr* clang::Decl::getAttr<clang::FormatAttr>() const"},
    {"_ZNK5clang13CXXRecordDecl25hasTrivialCopyConstructorEv", "clang::CXXRecordDecl::hasTrivialCopyConstructor() const"},
    {"_ZNK5clang13CXXRecordDecl21hasTrivialConstructorEv", "clang::CXXRecordDecl::hasTrivialConstructor() const"},
    {"_ZNK5clang13CXXRecordDecl24hasTrivialCopyAssignmentEv", "clang::CXXRecordDecl::hasTrivialCopyAssignment() const"},
    {"_ZNK5clang13CXXRecordDecl20hasTrivialDestructorEv", "clang::CXXRecordDecl::hasTrivialDestructor() const"},
    {"_ZNK5clang4Decl7getAttrINS_10PackedAttrEEEPT_v", "clang::PackedAttr* clang::Decl::getAttr<clang::PackedAttr>() const"},
    {"_ZNK4llvm6APSInt10extOrTruncEj", "llvm::APSInt::extOrTrunc(unsigned int) const"},
    {"_ZN4llvm15SmallVectorImplIPN5clang9FieldDeclEE6resizeEj", "llvm::SmallVectorImpl<clang::FieldDecl*>::resize(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang15DeclaratorChunkELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::DeclaratorChunk, false>::grow(unsigned long)"},
    {"_ZN5clang12CXXBasePathsD2Ev", "clang::CXXBasePaths::~CXXBasePaths()"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_S2_IbjEESt10_Select1stIS5_ENS0_16QualTypeOrderingESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, std::pair<bool, unsigned int> >, std::_Select1st<std::pair<clang::QualType const, std::pair<bool, unsigned int> > >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, std::pair<bool, unsigned int> > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::QualType const, std::pair<bool, unsigned int> > >*)"},
    {"_ZN5clang12LookupResult22resolveKindAfterFilterEv", "clang::LookupResult::resolveKindAfterFilter()"},
    {"_ZN5clang10Declarator5clearEv", "clang::Declarator::clear()"},
    {"_ZN4llvm15SmallVectorImplIPN5clang9NamedDeclEE6resizeEj", "llvm::SmallVectorImpl<clang::NamedDecl*>::resize(unsigned int)"},
    {"_ZN5clang4Sema17CheckNoReturnAttrERKNS_13AttributeListE", "clang::Sema::CheckNoReturnAttr(clang::AttributeList const&)"},
    {"_ZN5clang4Sema14AddAlignedAttrENS_14SourceLocationEPNS_4DeclEPNS_4ExprE", "clang::Sema::AddAlignedAttr(clang::SourceLocation, clang::Decl*, clang::Expr*)"},
    {"_ZN5clang4Sema14AddAlignedAttrENS_14SourceLocationEPNS_4DeclEPNS_14TypeSourceInfoE", "clang::Sema::AddAlignedAttr(clang::SourceLocation, clang::Decl*, clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema20CheckCallingConvAttrERKNS_13AttributeListERNS_11CallingConvE", "clang::Sema::CheckCallingConvAttr(clang::AttributeList const&, clang::CallingConv&)"},
    {"_ZN5clang4Sema16CheckRegparmAttrERKNS_13AttributeListERj", "clang::Sema::CheckRegparmAttr(clang::AttributeList const&, unsigned int&)"},
    {"_ZN5clang4Sema24ProcessDeclAttributeListEPNS_5ScopeEPNS_4DeclEPKNS_13AttributeListEbb", "clang::Sema::ProcessDeclAttributeList(clang::Scope*, clang::Decl*, clang::AttributeList const*, bool, bool)"},
    {"_ZN5clang4Sema19DeclClonePragmaWeakEPNS_9NamedDeclEPNS_14IdentifierInfoE", "clang::Sema::DeclClonePragmaWeak(clang::NamedDecl*, clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema19DeclApplyPragmaWeakEPNS_5ScopeEPNS_9NamedDeclERNS0_8WeakInfoE", "clang::Sema::DeclApplyPragmaWeak(clang::Scope*, clang::NamedDecl*, clang::Sema::WeakInfo&)"},
    {"_ZN5clang4Sema21ProcessDeclAttributesEPNS_5ScopeEPNS_4DeclERKNS_10DeclaratorEbb", "clang::Sema::ProcessDeclAttributes(clang::Scope*, clang::Decl*, clang::Declarator const&, bool, bool)"},
    {"_ZN5clang4Sema22PushParsingDeclarationEv", "clang::Sema::PushParsingDeclaration()"},
    {"_ZN5clang4Sema21PopParsingDeclarationEmPNS_4DeclE", "clang::Sema::PopParsingDeclaration(unsigned long, clang::Decl*)"},
    {"_ZN5clang4Sema29HandleDelayedDeprecationCheckERNS_4sema17DelayedDiagnosticEPNS_4DeclE", "clang::Sema::HandleDelayedDeprecationCheck(clang::sema::DelayedDiagnostic&, clang::Decl*)"},
    {"_ZN5clang4Sema22EmitDeprecationWarningEPNS_9NamedDeclEN4llvm9StringRefENS_14SourceLocationEb", "clang::Sema::EmitDeprecationWarning(clang::NamedDecl*, llvm::StringRef, clang::SourceLocation, bool)"},
    {"_Z26ProcessInheritableDeclAttrPN5clang5ScopeEPNS_4DeclERKNS_13AttributeListERNS_4SemaE", "ProcessInheritableDeclAttr(clang::Scope*, clang::Decl*, clang::AttributeList const&, clang::Sema&)"},
    {"_Z15getFunctionTypePKN5clang4DeclEb", "getFunctionType(clang::Decl const*, bool)"},
    {"_ZN4llvm25array_pod_sort_comparatorIjEEiPKvS2_", "int llvm::array_pod_sort_comparator<unsigned int>(void const*, void const*)"},
    {"_Z14isNSStringTypeN5clang8QualTypeERNS_10ASTContextE", "isNSStringType(clang::QualType, clang::ASTContext&)"},
    {"_Z14isCFStringTypeN5clang8QualTypeERNS_10ASTContextE", "isCFStringType(clang::QualType, clang::ASTContext&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4sema17DelayedDiagnosticELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::sema::DelayedDiagnostic, false>::grow(unsigned long)"},
    {"_ZNK5clang4Decl7getAttrINS_13VecReturnAttrEEEPT_v", "clang::VecReturnAttr* clang::Decl::getAttr<clang::VecReturnAttr>() const"},
    {"_ZN5clang4Sema23SetParamDefaultArgumentEPNS_11ParmVarDeclEPNS_4ExprENS_14SourceLocationE", "clang::Sema::SetParamDefaultArgument(clang::ParmVarDecl*, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema25ActOnParamDefaultArgumentEPNS_4DeclENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnParamDefaultArgument(clang::Decl*, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_127CheckDefaultArgumentVisitorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::CheckDefaultArgumentVisitor, bool>::Visit(clang::Stmt*)"},
    {"_ZN5clang4Sema33ActOnParamUnparsedDefaultArgumentEPNS_4DeclENS_14SourceLocationES3_", "clang::Sema::ActOnParamUnparsedDefaultArgument(clang::Decl*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema30ActOnParamDefaultArgumentErrorEPNS_4DeclE", "clang::Sema::ActOnParamDefaultArgumentError(clang::Decl*)"},
    {"_ZN5clang4Sema29CheckExtraCXXDefaultArgumentsERNS_10DeclaratorE", "clang::Sema::CheckExtraCXXDefaultArguments(clang::Declarator&)"},
    {"_ZN5clang4Sema20MergeCXXFunctionDeclEPNS_12FunctionDeclES2_", "clang::Sema::MergeCXXFunctionDecl(clang::FunctionDecl*, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema24CheckCXXDefaultArgumentsEPNS_12FunctionDeclE", "clang::Sema::CheckCXXDefaultArguments(clang::FunctionDecl*)"},
    {"_ZN5clang4Sema18isCurrentClassNameERKNS_14IdentifierInfoEPNS_5ScopeEPKNS_12CXXScopeSpecE", "clang::Sema::isCurrentClassName(clang::IdentifierInfo const&, clang::Scope*, clang::CXXScopeSpec const*)"},
    {"_ZN5clang4Sema18CheckBaseSpecifierEPNS_13CXXRecordDeclENS_11SourceRangeEbNS_15AccessSpecifierEPNS_14TypeSourceInfoENS_14SourceLocationE", "clang::Sema::CheckBaseSpecifier(clang::CXXRecordDecl*, clang::SourceRange, bool, clang::AccessSpecifier, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema18ActOnBaseSpecifierEPNS_4DeclENS_11SourceRangeEbNS_15AccessSpecifierENS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationES8_", "clang::Sema::ActOnBaseSpecifier(clang::Decl*, clang::SourceRange, bool, clang::AccessSpecifier, clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema20AttachBaseSpecifiersEPNS_13CXXRecordDeclEPPNS_16CXXBaseSpecifierEj", "clang::Sema::AttachBaseSpecifiers(clang::CXXRecordDecl*, clang::CXXBaseSpecifier**, unsigned int)"},
    {"_ZN5clang4Sema19ActOnBaseSpecifiersEPNS_4DeclEPPNS_16CXXBaseSpecifierEj", "clang::Sema::ActOnBaseSpecifiers(clang::Decl*, clang::CXXBaseSpecifier**, unsigned int)"},
    {"_ZN5clang4Sema13IsDerivedFromENS_8QualTypeES1_", "clang::Sema::IsDerivedFrom(clang::QualType, clang::QualType)"},
    {"_Z15GetClassForTypeN5clang8QualTypeE", "GetClassForType(clang::QualType)"},
    {"_ZN5clang4Sema13IsDerivedFromENS_8QualTypeES1_RNS_12CXXBasePathsE", "clang::Sema::IsDerivedFrom(clang::QualType, clang::QualType, clang::CXXBasePaths&)"},
    {"_ZN5clang4Sema18BuildBasePathArrayERKNS_12CXXBasePathsERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "clang::Sema::BuildBasePathArray(clang::CXXBasePaths const&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&)"},
    {"_ZN5clang4Sema27BasePathInvolvesVirtualBaseERKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "clang::Sema::BasePathInvolvesVirtualBase(llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const&)"},
    {"_ZN5clang4Sema28CheckDerivedToBaseConversionENS_8QualTypeES1_jjNS_14SourceLocationENS_11SourceRangeENS_15DeclarationNameEPN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "clang::Sema::CheckDerivedToBaseConversion(clang::QualType, clang::QualType, unsigned int, unsigned int, clang::SourceLocation, clang::SourceRange, clang::DeclarationName, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>*)"},
    {"_ZN5clang4Sema30getAmbiguousPathsDisplayStringERNS_12CXXBasePathsE", "clang::Sema::getAmbiguousPathsDisplayString(clang::CXXBasePaths&)"},
    {"_ZN5clang4Sema28CheckDerivedToBaseConversionENS_8QualTypeES1_NS_14SourceLocationENS_11SourceRangeEPN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEb", "clang::Sema::CheckDerivedToBaseConversion(clang::QualType, clang::QualType, clang::SourceLocation, clang::SourceRange, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>*, bool)"},
    {"_ZN5clang4Sema20ActOnAccessSpecifierENS_15AccessSpecifierENS_14SourceLocationES2_", "clang::Sema::ActOnAccessSpecifier(clang::AccessSpecifier, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema20CheckOverrideControlEPKNS_4DeclE", "clang::Sema::CheckOverrideControl(clang::Decl const*)"},
    {"_ZN5clang4Sema38CheckIfOverriddenFunctionIsMarkedFinalEPKNS_13CXXMethodDeclES3_", "clang::Sema::CheckIfOverriddenFunctionIsMarkedFinal(clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZN5clang4Sema24ActOnCXXMemberDeclaratorEPNS_5ScopeENS_15AccessSpecifierERNS_10DeclaratorENS_11ASTMultiPtrIPNS_21TemplateParameterListEEEPNS_4ExprERKNS_14VirtSpecifiersESB_bb", "clang::Sema::ActOnCXXMemberDeclarator(clang::Scope*, clang::AccessSpecifier, clang::Declarator&, clang::ASTMultiPtr<clang::TemplateParameterList*>, clang::Expr*, clang::VirtSpecifiers const&, clang::Expr*, bool, bool)"},
    {"_ZN5clang4Sema14SetDeclDeletedEPNS_4DeclENS_14SourceLocationE", "clang::Sema::SetDeclDeleted(clang::Decl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema19ActOnMemInitializerEPNS_4DeclEPNS_5ScopeERNS_12CXXScopeSpecEPNS_14IdentifierInfoENS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationESC_PPNS_4ExprEjSC_SC_", "clang::Sema::ActOnMemInitializer(clang::Decl*, clang::Scope*, clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema22BuildMemberInitializerEPNS_9ValueDeclEPPNS_4ExprEjNS_14SourceLocationES6_S6_", "clang::Sema::BuildMemberInitializer(clang::ValueDecl*, clang::Expr**, unsigned int, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_Z19FindBaseInitializerRN5clang4SemaEPNS_13CXXRecordDeclENS_8QualTypeERPKNS_16CXXBaseSpecifierES8_", "FindBaseInitializer(clang::Sema&, clang::CXXRecordDecl*, clang::QualType, clang::CXXBaseSpecifier const*&, clang::CXXBaseSpecifier const*&)"},
    {"_ZN5clang4Sema20BuildBaseInitializerENS_8QualTypeEPNS_14TypeSourceInfoEPPNS_4ExprEjNS_14SourceLocationES7_PNS_13CXXRecordDeclES7_", "clang::Sema::BuildBaseInitializer(clang::QualType, clang::TypeSourceInfo*, clang::Expr**, unsigned int, clang::SourceLocation, clang::SourceLocation, clang::CXXRecordDecl*, clang::SourceLocation)"},
    {"_Z35InitExprContainsUninitializedFieldsPKN5clang4StmtEPKNS_9ValueDeclEPNS_14SourceLocationE", "InitExprContainsUninitializedFields(clang::Stmt const*, clang::ValueDecl const*, clang::SourceLocation*)"},
    {"_ZN5clang4Sema26BuildDelegatingInitializerEPNS_14TypeSourceInfoEPPNS_4ExprEjNS_14SourceLocationES6_PNS_13CXXRecordDeclES6_", "clang::Sema::BuildDelegatingInitializer(clang::TypeSourceInfo*, clang::Expr**, unsigned int, clang::SourceLocation, clang::SourceLocation, clang::CXXRecordDecl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema19SetCtorInitializersEPNS_18CXXConstructorDeclEPPNS_18CXXCtorInitializerEjb", "clang::Sema::SetCtorInitializers(clang::CXXConstructorDecl*, clang::CXXCtorInitializer**, unsigned int, bool)"},
    {"_Z28BuildImplicitBaseInitializerRN5clang4SemaEPNS_18CXXConstructorDeclE23ImplicitInitializerKindPNS_16CXXBaseSpecifierEbRPNS_18CXXCtorInitializerE", "BuildImplicitBaseInitializer(clang::Sema&, clang::CXXConstructorDecl*, ImplicitInitializerKind, clang::CXXBaseSpecifier*, bool, clang::CXXCtorInitializer*&)"},
    {"_Z23CollectFieldInitializerRN12_GLOBAL__N_116BaseAndFieldInfoEPN5clang9FieldDeclES4_", "CollectFieldInitializer((anonymous namespace)::BaseAndFieldInfo&, clang::FieldDecl*, clang::FieldDecl*)"},
    {"_ZN5clang4Sema38MarkBaseAndMemberDestructorsReferencedENS_14SourceLocationEPNS_13CXXRecordDeclE", "clang::Sema::MarkBaseAndMemberDestructorsReferenced(clang::SourceLocation, clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema20ActOnMemInitializersEPNS_4DeclENS_14SourceLocationEPPNS_18CXXCtorInitializerEjb", "clang::Sema::ActOnMemInitializers(clang::Decl*, clang::SourceLocation, clang::CXXCtorInitializer**, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_118CheckRedundantInitERN5clang4SemaEPNS0_18CXXCtorInitializerERS4_", "(anonymous namespace)::CheckRedundantInit(clang::Sema&, clang::CXXCtorInitializer*, clang::CXXCtorInitializer*&)"},
    {"_ZN5clang4Sema28ActOnDefaultCtorInitializersEPNS_4DeclE", "clang::Sema::ActOnDefaultCtorInitializers(clang::Decl*)"},
    {"_ZN5clang4Sema22RequireNonAbstractTypeENS_14SourceLocationENS_8QualTypeEjNS0_17AbstractDiagSelIDE", "clang::Sema::RequireNonAbstractType(clang::SourceLocation, clang::QualType, unsigned int, clang::Sema::AbstractDiagSelID)"},
    {"_ZN5clang4Sema22RequireNonAbstractTypeENS_14SourceLocationENS_8QualTypeERKNS_17PartialDiagnosticE", "clang::Sema::RequireNonAbstractType(clang::SourceLocation, clang::QualType, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema20DiagnoseAbstractTypeEPKNS_13CXXRecordDeclE", "clang::Sema::DiagnoseAbstractType(clang::CXXRecordDecl const*)"},
    {"_ZN5clang4Sema22CheckCompletedCXXClassEPNS_13CXXRecordDeclE", "clang::Sema::CheckCompletedCXXClass(clang::CXXRecordDecl*)"},
    {"_Z23CheckAbstractClassUsageRN12_GLOBAL__N_117AbstractUsageInfoEPN5clang13CXXRecordDeclE", "CheckAbstractClassUsage((anonymous namespace)::AbstractUsageInfo&, clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema28DiagnoseHiddenVirtualMethodsEPNS_13CXXRecordDeclEPNS_13CXXMethodDeclE", "clang::Sema::DiagnoseHiddenVirtualMethods(clang::CXXRecordDecl*, clang::CXXMethodDecl*)"},
    {"_ZN5clang4Sema28DeclareInheritedConstructorsEPNS_13CXXRecordDeclE", "clang::Sema::DeclareInheritedConstructors(clang::CXXRecordDecl*)"},
    {"_Z23FindHiddenVirtualMethodPKN5clang16CXXBaseSpecifierERNS_11CXXBasePathEPv", "FindHiddenVirtualMethod(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang4Sema33ActOnFinishCXXMemberSpecificationEPNS_5ScopeENS_14SourceLocationEPNS_4DeclES3_S3_PNS_13AttributeListE", "clang::Sema::ActOnFinishCXXMemberSpecification(clang::Scope*, clang::SourceLocation, clang::Decl*, clang::SourceLocation, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZN5clang4Sema35AddImplicitlyDeclaredMembersToClassEPNS_13CXXRecordDeclE", "clang::Sema::AddImplicitlyDeclaredMembersToClass(clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema29DeclareImplicitCopyAssignmentEPNS_13CXXRecordDeclE", "clang::Sema::DeclareImplicitCopyAssignment(clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema25DeclareImplicitDestructorEPNS_13CXXRecordDeclE", "clang::Sema::DeclareImplicitDestructor(clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema25ActOnReenterTemplateScopeEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnReenterTemplateScope(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema35ActOnStartDelayedMemberDeclarationsEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnStartDelayedMemberDeclarations(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema36ActOnFinishDelayedMemberDeclarationsEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnFinishDelayedMemberDeclarations(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema37ActOnStartDelayedCXXMethodDeclarationEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnStartDelayedCXXMethodDeclaration(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema30ActOnDelayedCXXMethodParameterEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnDelayedCXXMethodParameter(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema38ActOnFinishDelayedCXXMethodDeclarationEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnFinishDelayedCXXMethodDeclaration(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema16CheckConstructorEPNS_18CXXConstructorDeclE", "clang::Sema::CheckConstructor(clang::CXXConstructorDecl*)"},
    {"_ZN5clang4Sema26CheckConstructorDeclaratorERNS_10DeclaratorENS_8QualTypeERNS_12StorageClassE", "clang::Sema::CheckConstructorDeclarator(clang::Declarator&, clang::QualType, clang::StorageClass&)"},
    {"_ZN5clang4Sema15CheckDestructorEPNS_17CXXDestructorDeclE", "clang::Sema::CheckDestructor(clang::CXXDestructorDecl*)"},
    {"_ZN5clang4Sema25CheckDestructorDeclaratorERNS_10DeclaratorENS_8QualTypeERNS_12StorageClassE", "clang::Sema::CheckDestructorDeclarator(clang::Declarator&, clang::QualType, clang::StorageClass&)"},
    {"_ZN5clang4Sema25CheckConversionDeclaratorERNS_10DeclaratorERNS_8QualTypeERNS_12StorageClassE", "clang::Sema::CheckConversionDeclarator(clang::Declarator&, clang::QualType&, clang::StorageClass&)"},
    {"_ZN5clang4Sema25ActOnConversionDeclaratorEPNS_17CXXConversionDeclE", "clang::Sema::ActOnConversionDeclarator(clang::CXXConversionDecl*)"},
    {"_ZN5clang4Sema22ActOnStartNamespaceDefEPNS_5ScopeENS_14SourceLocationES3_PNS_14IdentifierInfoES3_PNS_13AttributeListE", "clang::Sema::ActOnStartNamespaceDef(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZNK5clang4Sema15getStdNamespaceEv", "clang::Sema::getStdNamespace() const"},
    {"_ZN5clang4Sema23ActOnFinishNamespaceDefEPNS_4DeclENS_14SourceLocationE", "clang::Sema::ActOnFinishNamespaceDef(clang::Decl*, clang::SourceLocation)"},
    {"_ZNK5clang4Sema14getStdBadAllocEv", "clang::Sema::getStdBadAlloc() const"},
    {"_ZN5clang4Sema23getOrCreateStdNamespaceEv", "clang::Sema::getOrCreateStdNamespace()"},
    {"_ZN5clang4Sema19ActOnUsingDirectiveEPNS_5ScopeENS_14SourceLocationES3_RNS_12CXXScopeSpecES3_PNS_14IdentifierInfoEPNS_13AttributeListE", "clang::Sema::ActOnUsingDirective(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::CXXScopeSpec&, clang::SourceLocation, clang::IdentifierInfo*, clang::AttributeList*)"},
    {"_ZN5clang4Sema18PushUsingDirectiveEPNS_5ScopeEPNS_18UsingDirectiveDeclE", "clang::Sema::PushUsingDirective(clang::Scope*, clang::UsingDirectiveDecl*)"},
    {"_ZN5clang4Sema21ActOnUsingDeclarationEPNS_5ScopeENS_15AccessSpecifierEbNS_14SourceLocationERNS_12CXXScopeSpecERNS_13UnqualifiedIdEPNS_13AttributeListEbS4_", "clang::Sema::ActOnUsingDeclaration(clang::Scope*, clang::AccessSpecifier, bool, clang::SourceLocation, clang::CXXScopeSpec&, clang::UnqualifiedId&, clang::AttributeList*, bool, clang::SourceLocation)"},
    {"_ZN5clang4Sema21BuildUsingDeclarationEPNS_5ScopeENS_15AccessSpecifierENS_14SourceLocationERNS_12CXXScopeSpecERKNS_19DeclarationNameInfoEPNS_13AttributeListEbbS4_", "clang::Sema::BuildUsingDeclaration(clang::Scope*, clang::AccessSpecifier, clang::SourceLocation, clang::CXXScopeSpec&, clang::DeclarationNameInfo const&, clang::AttributeList*, bool, bool, clang::SourceLocation)"},
    {"_ZN5clang4Sema20CheckUsingShadowDeclEPNS_9UsingDeclEPNS_9NamedDeclERKNS_12LookupResultE", "clang::Sema::CheckUsingShadowDecl(clang::UsingDecl*, clang::NamedDecl*, clang::LookupResult const&)"},
    {"_ZN5clang4Sema20BuildUsingShadowDeclEPNS_5ScopeEPNS_9UsingDeclEPNS_9NamedDeclE", "clang::Sema::BuildUsingShadowDecl(clang::Scope*, clang::UsingDecl*, clang::NamedDecl*)"},
    {"_ZN5clang4Sema19HideUsingShadowDeclEPNS_5ScopeEPNS_15UsingShadowDeclE", "clang::Sema::HideUsingShadowDecl(clang::Scope*, clang::UsingShadowDecl*)"},
    {"_ZN5clang4Sema27CheckUsingDeclRedeclarationENS_14SourceLocationEbRKNS_12CXXScopeSpecES1_RKNS_12LookupResultE", "clang::Sema::CheckUsingDeclRedeclaration(clang::SourceLocation, bool, clang::CXXScopeSpec const&, clang::SourceLocation, clang::LookupResult const&)"},
    {"_ZN5clang4Sema23CheckUsingDeclQualifierENS_14SourceLocationERKNS_12CXXScopeSpecES1_", "clang::Sema::CheckUsingDeclQualifier(clang::SourceLocation, clang::CXXScopeSpec const&, clang::SourceLocation)"},
    {"_ZN5clang4Sema34CheckInheritedConstructorUsingDeclEPNS_9UsingDeclE", "clang::Sema::CheckInheritedConstructorUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang4Sema22ActOnNamespaceAliasDefEPNS_5ScopeENS_14SourceLocationES3_PNS_14IdentifierInfoERNS_12CXXScopeSpecES3_S5_", "clang::Sema::ActOnNamespaceAliasDef(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*, clang::CXXScopeSpec&, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema33DeclareImplicitDefaultConstructorEPNS_13CXXRecordDeclE", "clang::Sema::DeclareImplicitDefaultConstructor(clang::CXXRecordDecl*)"},
    {"_ZN12_GLOBAL__N_130ImplicitExceptionSpecification10CalledDeclEPN5clang13CXXMethodDeclE", "(anonymous namespace)::ImplicitExceptionSpecification::CalledDecl(clang::CXXMethodDecl*)"},
    {"_Z27getDefaultConstructorUnsafeRN5clang4SemaEPNS_13CXXRecordDeclE", "getDefaultConstructorUnsafe(clang::Sema&, clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema32DefineImplicitDefaultConstructorENS_14SourceLocationEPNS_18CXXConstructorDeclE", "clang::Sema::DefineImplicitDefaultConstructor(clang::SourceLocation, clang::CXXConstructorDecl*)"},
    {"_ZN5clang4Sema14MarkVTableUsedENS_14SourceLocationEPNS_13CXXRecordDeclEb", "clang::Sema::MarkVTableUsed(clang::SourceLocation, clang::CXXRecordDecl*, bool)"},
    {"_ZN5clang4Sema24DefineImplicitDestructorENS_14SourceLocationEPNS_17CXXDestructorDeclE", "clang::Sema::DefineImplicitDestructor(clang::SourceLocation, clang::CXXDestructorDecl*)"},
    {"_Z22hasConstCopyAssignmentRN5clang4SemaEPKNS_13CXXRecordDeclE", "hasConstCopyAssignment(clang::Sema&, clang::CXXRecordDecl const*)"},
    {"_ZN5clang4Sema28DefineImplicitCopyAssignmentENS_14SourceLocationEPNS_13CXXMethodDeclE", "clang::Sema::DefineImplicitCopyAssignment(clang::SourceLocation, clang::CXXMethodDecl*)"},
    {"_Z21BuildSingleCopyAssignRN5clang4SemaENS_14SourceLocationENS_8QualTypeEPNS_4ExprES5_bj", "BuildSingleCopyAssign(clang::Sema&, clang::SourceLocation, clang::QualType, clang::Expr*, clang::Expr*, bool, unsigned int)"},
    {"_ZN5clang4Sema30DeclareImplicitCopyConstructorEPNS_13CXXRecordDeclE", "clang::Sema::DeclareImplicitCopyConstructor(clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema29DefineImplicitCopyConstructorENS_14SourceLocationEPNS_18CXXConstructorDeclEj", "clang::Sema::DefineImplicitCopyConstructor(clang::SourceLocation, clang::CXXConstructorDecl*, unsigned int)"},
    {"_ZN5clang4Sema21BuildCXXConstructExprENS_14SourceLocationENS_8QualTypeEPNS_18CXXConstructorDeclENS_11ASTMultiPtrIPNS_4ExprEEEbjNS_11SourceRangeE", "clang::Sema::BuildCXXConstructExpr(clang::SourceLocation, clang::QualType, clang::CXXConstructorDecl*, clang::ASTMultiPtr<clang::Expr*>, bool, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21BuildCXXConstructExprENS_14SourceLocationENS_8QualTypeEPNS_18CXXConstructorDeclEbNS_11ASTMultiPtrIPNS_4ExprEEEbjNS_11SourceRangeE", "clang::Sema::BuildCXXConstructExpr(clang::SourceLocation, clang::QualType, clang::CXXConstructorDecl*, bool, clang::ASTMultiPtr<clang::Expr*>, bool, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema28InitializeVarWithConstructorEPNS_7VarDeclEPNS_18CXXConstructorDeclENS_11ASTMultiPtrIPNS_4ExprEEE", "clang::Sema::InitializeVarWithConstructor(clang::VarDecl*, clang::CXXConstructorDecl*, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema25FinalizeVarWithDestructorEPNS_7VarDeclEPKNS_10RecordTypeE", "clang::Sema::FinalizeVarWithDestructor(clang::VarDecl*, clang::RecordType const*)"},
    {"_ZN5clang4Sema29AddCXXDirectInitializerToDeclEPNS_4DeclENS_14SourceLocationENS_11ASTMultiPtrIPNS_4ExprEEES3_", "clang::Sema::AddCXXDirectInitializerToDecl(clang::Decl*, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema23CompleteConstructorCallEPNS_18CXXConstructorDeclENS_11ASTMultiPtrIPNS_4ExprEEENS_14SourceLocationERNS_15ASTOwningVectorIS5_Lj8EEE", "clang::Sema::CompleteConstructorCall(clang::CXXConstructorDecl*, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation, clang::ASTOwningVector<clang::Expr*, 8u>&)"},
    {"_ZN5clang4Sema34CheckOverloadedOperatorDeclarationEPNS_12FunctionDeclE", "clang::Sema::CheckOverloadedOperatorDeclaration(clang::FunctionDecl*)"},
    {"_ZN5clang4Sema31CheckLiteralOperatorDeclarationEPNS_12FunctionDeclE", "clang::Sema::CheckLiteralOperatorDeclaration(clang::FunctionDecl*)"},
    {"_ZN5clang4Sema30ActOnStartLinkageSpecificationEPNS_5ScopeENS_14SourceLocationES3_N4llvm9StringRefES3_", "clang::Sema::ActOnStartLinkageSpecification(clang::Scope*, clang::SourceLocation, clang::SourceLocation, llvm::StringRef, clang::SourceLocation)"},
    {"_ZN5clang4Sema31ActOnFinishLinkageSpecificationEPNS_5ScopeEPNS_4DeclENS_14SourceLocationE", "clang::Sema::ActOnFinishLinkageSpecification(clang::Scope*, clang::Decl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema25BuildExceptionDeclarationEPNS_5ScopeEPNS_14TypeSourceInfoEPNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::BuildExceptionDeclaration(clang::Scope*, clang::TypeSourceInfo*, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24ActOnExceptionDeclaratorEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnExceptionDeclarator(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema28ActOnStaticAssertDeclarationENS_14SourceLocationEPNS_4ExprES3_", "clang::Sema::ActOnStaticAssertDeclaration(clang::SourceLocation, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema19CheckFriendTypeDeclENS_14SourceLocationEPNS_14TypeSourceInfoE", "clang::Sema::CheckFriendTypeDecl(clang::SourceLocation, clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema23ActOnTemplatedFriendTagEPNS_5ScopeENS_14SourceLocationEjS3_RNS_12CXXScopeSpecEPNS_14IdentifierInfoES3_PNS_13AttributeListENS_11ASTMultiPtrIPNS_21TemplateParameterListEEE", "clang::Sema::ActOnTemplatedFriendTag(clang::Scope*, clang::SourceLocation, unsigned int, clang::SourceLocation, clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::SourceLocation, clang::AttributeList*, clang::ASTMultiPtr<clang::TemplateParameterList*>)"},
    {"_ZN5clang4Sema19ActOnFriendTypeDeclEPNS_5ScopeERKNS_8DeclSpecENS_11ASTMultiPtrIPNS_21TemplateParameterListEEE", "clang::Sema::ActOnFriendTypeDecl(clang::Scope*, clang::DeclSpec const&, clang::ASTMultiPtr<clang::TemplateParameterList*>)"},
    {"_ZN5clang4Sema23ActOnFriendFunctionDeclEPNS_5ScopeERNS_10DeclaratorEbNS_11ASTMultiPtrIPNS_21TemplateParameterListEEE", "clang::Sema::ActOnFriendFunctionDecl(clang::Scope*, clang::Declarator&, bool, clang::ASTMultiPtr<clang::TemplateParameterList*>)"},
    {"_ZN5clang4Sema43DiagnoseReturnInConstructorExceptionHandlerEPNS_10CXXTryStmtE", "clang::Sema::DiagnoseReturnInConstructorExceptionHandler(clang::CXXTryStmt*)"},
    {"_Z21SearchForReturnInStmtRN5clang4SemaEPNS_4StmtE", "SearchForReturnInStmt(clang::Sema&, clang::Stmt*)"},
    {"_ZN5clang4Sema33CheckOverridingFunctionReturnTypeEPKNS_13CXXMethodDeclES3_", "clang::Sema::CheckOverridingFunctionReturnType(clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZN5clang4Sema15CheckPureMethodEPNS_13CXXMethodDeclENS_11SourceRangeE", "clang::Sema::CheckPureMethod(clang::CXXMethodDecl*, clang::SourceRange)"},
    {"_ZN5clang4Sema28ActOnCXXEnterDeclInitializerEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnCXXEnterDeclInitializer(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema27ActOnCXXExitDeclInitializerEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnCXXExitDeclInitializer(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema28ActOnCXXConditionDeclarationEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnCXXConditionDeclaration(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema28MarkVirtualMembersReferencedENS_14SourceLocationEPKNS_13CXXRecordDeclE", "clang::Sema::MarkVirtualMembersReferenced(clang::SourceLocation, clang::CXXRecordDecl const*)"},
    {"_ZN5clang4Sema17DefineUsedVTablesEv", "clang::Sema::DefineUsedVTables()"},
    {"_ZN5clang4Sema19SetIvarInitializersEPNS_22ObjCImplementationDeclE", "clang::Sema::SetIvarInitializers(clang::ObjCImplementationDecl*)"},
    {"_ZN12_GLOBAL__N_127CheckDefaultArgumentVisitor9VisitExprEPN5clang4ExprE", "(anonymous namespace)::CheckDefaultArgumentVisitor::VisitExpr(clang::Expr*)"},
    {"_Z38CheckOperatorNewDeleteDeclarationScopeRN5clang4SemaEPKNS_12FunctionDeclE", "CheckOperatorNewDeleteDeclarationScope(clang::Sema&, clang::FunctionDecl const*)"},
    {"_Z27CheckOperatorNewDeleteTypesRN5clang4SemaEPKNS_12FunctionDeclENS_7CanQualINS_4TypeEEES7_jj", "CheckOperatorNewDeleteTypes(clang::Sema&, clang::FunctionDecl const*, clang::CanQual<clang::Type>, clang::CanQual<clang::Type>, unsigned int, unsigned int)"},
    {"_ZZN5clang4Sema23CheckUsingDeclQualifierENS_14SourceLocationERKNS_12CXXScopeSpecES1_EN8UserData14doesNotContainEPKNS_13CXXRecordDeclEPv", "clang::Sema::CheckUsingDeclQualifier(clang::SourceLocation, clang::CXXScopeSpec const&, clang::SourceLocation)::UserData::doesNotContain(clang::CXXRecordDecl const*, void*)"},
    {"_ZZN5clang4Sema23CheckUsingDeclQualifierENS_14SourceLocationERKNS_12CXXScopeSpecES1_EN8UserData7collectEPKNS_13CXXRecordDeclEPv", "clang::Sema::CheckUsingDeclQualifier(clang::SourceLocation, clang::CXXScopeSpec const&, clang::SourceLocation)::UserData::collect(clang::CXXRecordDecl const*, void*)"},
    {"_ZN12_GLOBAL__N_118CheckAbstractUsage5VisitEN5clang7TypeLocENS1_4Sema17AbstractDiagSelIDE", "(anonymous namespace)::CheckAbstractUsage::Visit(clang::TypeLoc, clang::Sema::AbstractDiagSelID)"},
    {"_ZN12_GLOBAL__N_118CheckAbstractUsage5CheckEN5clang7TypeLocENS1_4Sema17AbstractDiagSelIDE", "(anonymous namespace)::CheckAbstractUsage::Check(clang::TypeLoc, clang::Sema::AbstractDiagSelID)"},
    {"_ZN5clang14AccessSpecDecl6CreateERNS_10ASTContextENS_15AccessSpecifierEPNS_11DeclContextENS_14SourceLocationES6_", "clang::AccessSpecDecl::Create(clang::ASTContext&, clang::AccessSpecifier, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeESt4pairINS1_7CanQualIS2_EEPNS1_18CXXConstructorDeclEENS_12DenseMapInfoIS4_EENSB_ISA_EEE6insertERKS5_IS4_SA_E", "llvm::DenseMap<clang::Type const*, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*>, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> > >::insert(std::pair<clang::Type const*, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> > const&)"},
    {"_ZN4llvm8DenseMapIPN5clang13CXXRecordDeclEbNS_12DenseMapInfoIS3_EENS4_IbEEE6insertERKSt4pairIS3_bE", "llvm::DenseMap<clang::CXXRecordDecl*, bool, llvm::DenseMapInfo<clang::CXXRecordDecl*>, llvm::DenseMapInfo<bool> >::insert(std::pair<clang::CXXRecordDecl*, bool> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeESt4pairINS1_7CanQualIS2_EEPNS1_18CXXConstructorDeclEENS_12DenseMapInfoIS4_EENSB_ISA_EEE16InsertIntoBucketERKS4_RKSA_PS5_IS4_SA_E", "llvm::DenseMap<clang::Type const*, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*>, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> > >::InsertIntoBucket(clang::Type const* const&, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> const&, std::pair<clang::Type const*, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeESt4pairINS1_7CanQualIS2_EEPNS1_18CXXConstructorDeclEENS_12DenseMapInfoIS4_EENSB_ISA_EEE4growEj", "llvm::DenseMap<clang::Type const*, std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*>, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CXXConstructorDecl*> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPvPN5clang18CXXCtorInitializerENS_12DenseMapInfoIS1_EENS5_IS4_EEE16InsertIntoBucketERKS1_RKS4_PSt4pairIS1_S4_E", "llvm::DenseMap<void*, clang::CXXCtorInitializer*, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<clang::CXXCtorInitializer*> >::InsertIntoBucket(void* const&, clang::CXXCtorInitializer* const&, std::pair<void*, clang::CXXCtorInitializer*>*)"},
    {"_ZN4llvm8DenseMapIPvPN5clang18CXXCtorInitializerENS_12DenseMapInfoIS1_EENS5_IS4_EEE4growEj", "llvm::DenseMap<void*, clang::CXXCtorInitializer*, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<clang::CXXCtorInitializer*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKvPN5clang18CXXCtorInitializerENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<void const*, clang::CXXCtorInitializer*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<clang::CXXCtorInitializer*> >::InsertIntoBucket(void const* const&, clang::CXXCtorInitializer* const&, std::pair<void const*, clang::CXXCtorInitializer*>*)"},
    {"_ZN4llvm8DenseMapIPKvPN5clang18CXXCtorInitializerENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<void const*, clang::CXXCtorInitializer*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<clang::CXXCtorInitializer*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_PNS0_16CXXBaseSpecifierEESt10_Select1stIS6_ENS0_16QualTypeOrderingESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, clang::CXXBaseSpecifier*>, std::_Select1st<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> >, std::pair<clang::QualType const, clang::CXXBaseSpecifier*> const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_PNS0_16CXXBaseSpecifierEESt10_Select1stIS6_ENS0_16QualTypeOrderingESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, clang::CXXBaseSpecifier*>, std::_Select1st<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> > >::_M_insert_unique(std::pair<clang::QualType const, clang::CXXBaseSpecifier*> const&)"},
    {"_ZN4llvm8DenseMapIPN5clang11ParmVarDeclENS1_14SourceLocationENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<clang::ParmVarDecl*, clang::SourceLocation, llvm::DenseMapInfo<clang::ParmVarDecl*>, llvm::DenseMapInfo<clang::SourceLocation> >::InsertIntoBucket(clang::ParmVarDecl* const&, clang::SourceLocation const&, std::pair<clang::ParmVarDecl*, clang::SourceLocation>*)"},
    {"_ZN4llvm8DenseMapIPN5clang11ParmVarDeclENS1_14SourceLocationENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::ParmVarDecl*, clang::SourceLocation, llvm::DenseMapInfo<clang::ParmVarDecl*>, llvm::DenseMapInfo<clang::SourceLocation> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEcNS_12DenseMapInfoIS4_EENS5_IcEEE6insertERKSt4pairIS4_cE", "llvm::DenseMap<clang::CXXRecordDecl const*, char, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::CXXRecordDecl const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEcNS_12DenseMapInfoIS4_EENS5_IcEEE16InsertIntoBucketERKS4_RKcPSt4pairIS4_cE", "llvm::DenseMap<clang::CXXRecordDecl const*, char, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, char const&, std::pair<clang::CXXRecordDecl const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEcNS_12DenseMapInfoIS4_EENS5_IcEEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, char, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang15UsingShadowDeclC2EPNS_11DeclContextENS_14SourceLocationEPNS_9UsingDeclEPNS_9NamedDeclE", "clang::UsingShadowDecl::UsingShadowDecl(clang::DeclContext*, clang::SourceLocation, clang::UsingDecl*, clang::NamedDecl*)"},
    {"_ZN5clang15UsingShadowDeclD1Ev", "clang::UsingShadowDecl::~UsingShadowDecl()"},
    {"_ZN5clang15UsingShadowDeclD0Ev", "clang::UsingShadowDecl::~UsingShadowDecl()"},
    {"_ZN4llvm8DenseMapIPN5clang10RecordDeclESt4pairIPNS1_9NamedDeclEPNS1_18CXXCtorInitializerEENS_12DenseMapInfoIS3_EENSA_IS9_EEE16InsertIntoBucketERKS3_RKS9_PS4_IS3_S9_E", "llvm::DenseMap<clang::RecordDecl*, std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*>, llvm::DenseMapInfo<clang::RecordDecl*>, llvm::DenseMapInfo<std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*> > >::InsertIntoBucket(clang::RecordDecl* const&, std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*> const&, std::pair<clang::RecordDecl*, std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang10RecordDeclESt4pairIPNS1_9NamedDeclEPNS1_18CXXCtorInitializerEENS_12DenseMapInfoIS3_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::RecordDecl*, std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*>, llvm::DenseMapInfo<clang::RecordDecl*>, llvm::DenseMapInfo<std::pair<clang::NamedDecl*, clang::CXXCtorInitializer*> > >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang17InitializedEntityELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::InitializedEntity, false>::grow(unsigned long)"},
    {"_ZN5clang14AccessSpecDeclD1Ev", "clang::AccessSpecDecl::~AccessSpecDecl()"},
    {"_ZN5clang14AccessSpecDeclD0Ev", "clang::AccessSpecDecl::~AccessSpecDecl()"},
    {"_ZNK5clang14AccessSpecDecl14getSourceRangeEv", "clang::AccessSpecDecl::getSourceRange() const"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_PNS0_16CXXBaseSpecifierEESt10_Select1stIS6_ENS0_16QualTypeOrderingESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, clang::CXXBaseSpecifier*>, std::_Select1st<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::QualType const, clang::CXXBaseSpecifier*> >*)"},
    {"_ZN5clang4Sema25ActOnStartOfObjCMethodDefEPNS_5ScopeEPNS_4DeclE", "clang::Sema::ActOnStartOfObjCMethodDef(clang::Scope*, clang::Decl*)"},
    {"_ZN5clang4Sema24ActOnStartClassInterfaceENS_14SourceLocationEPNS_14IdentifierInfoES1_S3_S1_PKPNS_4DeclEjPKS1_S1_PNS_13AttributeListE", "clang::Sema::ActOnStartClassInterface(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Decl* const*, unsigned int, clang::SourceLocation const*, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZN5clang4Sema18CheckObjCDeclScopeEPNS_4DeclE", "clang::Sema::CheckObjCDeclScope(clang::Decl*)"},
    {"_ZN5clang4Sema22ActOnCompatiblityAliasENS_14SourceLocationEPNS_14IdentifierInfoES1_S3_S1_", "clang::Sema::ActOnCompatiblityAlias(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema52CheckForwardProtocolDeclarationForCircularDependencyEPNS_14IdentifierInfoERNS_14SourceLocationES3_RKNS_8ObjCListINS_16ObjCProtocolDeclEEE", "clang::Sema::CheckForwardProtocolDeclarationForCircularDependency(clang::IdentifierInfo*, clang::SourceLocation&, clang::SourceLocation, clang::ObjCList<clang::ObjCProtocolDecl> const&)"},
    {"_ZN5clang4Sema27ActOnStartProtocolInterfaceENS_14SourceLocationEPNS_14IdentifierInfoES1_PKPNS_4DeclEjPKS1_S1_PNS_13AttributeListE", "clang::Sema::ActOnStartProtocolInterface(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Decl* const*, unsigned int, clang::SourceLocation const*, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZN5clang4Sema23FindProtocolDeclarationEbPKSt4pairIPNS_14IdentifierInfoENS_14SourceLocationEEjRN4llvm15SmallVectorImplIPNS_4DeclEEE", "clang::Sema::FindProtocolDeclaration(bool, std::pair<clang::IdentifierInfo*, clang::SourceLocation> const*, unsigned int, llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang4Sema32DiagnoseClassExtensionDupMethodsEPNS_16ObjCCategoryDeclEPNS_17ObjCInterfaceDeclE", "clang::Sema::DiagnoseClassExtensionDupMethods(clang::ObjCCategoryDecl*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema26MatchTwoMethodDeclarationsEPKNS_14ObjCMethodDeclES3_bb", "clang::Sema::MatchTwoMethodDeclarations(clang::ObjCMethodDecl const*, clang::ObjCMethodDecl const*, bool, bool)"},
    {"_ZN5clang4Sema31ActOnForwardProtocolDeclarationENS_14SourceLocationEPKSt4pairIPNS_14IdentifierInfoES1_EjPNS_13AttributeListE", "clang::Sema::ActOnForwardProtocolDeclaration(clang::SourceLocation, std::pair<clang::IdentifierInfo*, clang::SourceLocation> const*, unsigned int, clang::AttributeList*)"},
    {"_ZN5clang4Sema27ActOnStartCategoryInterfaceENS_14SourceLocationEPNS_14IdentifierInfoES1_S3_S1_PKPNS_4DeclEjPKS1_S1_", "clang::Sema::ActOnStartCategoryInterface(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Decl* const*, unsigned int, clang::SourceLocation const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema32ActOnStartCategoryImplementationENS_14SourceLocationEPNS_14IdentifierInfoES1_S3_S1_", "clang::Sema::ActOnStartCategoryImplementation(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema29ActOnStartClassImplementationENS_14SourceLocationEPNS_14IdentifierInfoES1_S3_S1_", "clang::Sema::ActOnStartClassImplementation(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24CheckImplementationIvarsEPNS_22ObjCImplementationDeclEPPNS_12ObjCIvarDeclEjNS_14SourceLocationE", "clang::Sema::CheckImplementationIvars(clang::ObjCImplementationDecl*, clang::ObjCIvarDecl**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema19WarnUndefinedMethodENS_14SourceLocationEPNS_14ObjCMethodDeclERbj", "clang::Sema::WarnUndefinedMethod(clang::SourceLocation, clang::ObjCMethodDecl*, bool&, unsigned int)"},
    {"_ZN5clang4Sema27WarnConflictingTypedMethodsEPNS_14ObjCMethodDeclES2_", "clang::Sema::WarnConflictingTypedMethods(clang::ObjCMethodDecl*, clang::ObjCMethodDecl*)"},
    {"_ZN5clang4Sema23CheckProtocolMethodDefsENS_14SourceLocationEPNS_16ObjCProtocolDeclERbRKN4llvm8DenseSetINS_8SelectorENS5_12DenseMapInfoIS7_EEEESC_PNS_17ObjCContainerDeclE", "clang::Sema::CheckProtocolMethodDefs(clang::SourceLocation, clang::ObjCProtocolDecl*, bool&, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> > const&, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> > const&, clang::ObjCContainerDecl*)"},
    {"_ZN5clang4Sema26MatchAllMethodDeclarationsERKN4llvm8DenseSetINS_8SelectorENS1_12DenseMapInfoIS3_EEEES8_RS6_S9_PNS_12ObjCImplDeclEPNS_17ObjCContainerDeclERbb", "clang::Sema::MatchAllMethodDeclarations(llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> > const&, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> > const&, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> >&, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> >&, clang::ObjCImplDecl*, clang::ObjCContainerDecl*, bool&, bool)"},
    {"_ZN5clang4Sema25ImplMethodsVsClassMethodsEPNS_5ScopeEPNS_12ObjCImplDeclEPNS_17ObjCContainerDeclEb", "clang::Sema::ImplMethodsVsClassMethods(clang::Scope*, clang::ObjCImplDecl*, clang::ObjCContainerDecl*, bool)"},
    {"_ZN5clang4Sema28ActOnForwardClassDeclarationENS_14SourceLocationEPPNS_14IdentifierInfoEPS1_j", "clang::Sema::ActOnForwardClassDeclaration(clang::SourceLocation, clang::IdentifierInfo**, clang::SourceLocation*, unsigned int)"},
    {"_ZN5clang4Sema14ReadMethodPoolENS_8SelectorE", "clang::Sema::ReadMethodPool(clang::Selector)"},
    {"_ZN5clang4Sema21AddMethodToGlobalPoolEPNS_14ObjCMethodDeclEbb", "clang::Sema::AddMethodToGlobalPool(clang::ObjCMethodDecl*, bool, bool)"},
    {"_ZN5clang4Sema24LookupMethodInGlobalPoolENS_8SelectorENS_11SourceRangeEbbb", "clang::Sema::LookupMethodInGlobalPool(clang::Selector, clang::SourceRange, bool, bool, bool)"},
    {"_ZN5clang4Sema35LookupImplementedMethodInGlobalPoolENS_8SelectorE", "clang::Sema::LookupImplementedMethodInGlobalPool(clang::Selector)"},
    {"_ZN5clang4Sema33CompareMethodParamsInBaseAndSuperEPNS_4DeclEPNS_14ObjCMethodDeclEb", "clang::Sema::CompareMethodParamsInBaseAndSuper(clang::Decl*, clang::ObjCMethodDecl*, bool)"},
    {"_ZN5clang4Sema22DiagnoseDuplicateIvarsEPNS_17ObjCInterfaceDeclES2_", "clang::Sema::DiagnoseDuplicateIvars(clang::ObjCInterfaceDecl*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema10ActOnAtEndEPNS_5ScopeENS_11SourceRangeEPNS_4DeclEPS5_jS6_jPNS_9OpaquePtrINS_12DeclGroupRefEEEj", "clang::Sema::ActOnAtEnd(clang::Scope*, clang::SourceRange, clang::Decl*, clang::Decl**, unsigned int, clang::Decl**, unsigned int, clang::OpaquePtr<clang::DeclGroupRef>*, unsigned int)"},
    {"_ZN5clang4Sema22ActOnMethodDeclarationEPNS_5ScopeENS_14SourceLocationES3_NS_3tok9TokenKindEPNS_4DeclERNS_12ObjCDeclSpecENS_9OpaquePtrINS_8QualTypeEEENS_8SelectorEPNS0_11ObjCArgInfoEPNS_15DeclaratorChunk9ParamInfoEjPNS_13AttributeListENS4_15ObjCKeywordKindEb", "clang::Sema::ActOnMethodDeclaration(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::tok::TokenKind, clang::Decl*, clang::ObjCDeclSpec&, clang::OpaquePtr<clang::QualType>, clang::Selector, clang::Sema::ObjCArgInfo*, clang::DeclaratorChunk::ParamInfo*, unsigned int, clang::AttributeList*, clang::tok::ObjCKeywordKind, bool)"},
    {"_ZN5clang4Sema9ActOnDefsEPNS_5ScopeEPNS_4DeclENS_14SourceLocationEPNS_14IdentifierInfoERN4llvm15SmallVectorImplIS4_EE", "clang::Sema::ActOnDefs(clang::Scope*, clang::Decl*, clang::SourceLocation, clang::IdentifierInfo*, llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang4Sema22BuildObjCExceptionDeclEPNS_14TypeSourceInfoENS_8QualTypeEPNS_14IdentifierInfoENS_14SourceLocationEb", "clang::Sema::BuildObjCExceptionDecl(clang::TypeSourceInfo*, clang::QualType, clang::IdentifierInfo*, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema22ActOnObjCExceptionDeclEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnObjCExceptionDecl(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema33CollectIvarsToConstructOrDestructEPNS_17ObjCInterfaceDeclERN4llvm15SmallVectorImplIPNS_12ObjCIvarDeclEEE", "clang::Sema::CollectIvarsToConstructOrDestruct(clang::ObjCInterfaceDecl*, llvm::SmallVectorImpl<clang::ObjCIvarDecl*>&)"},
    {"_ZN5clang22ObjCImplementationDecl19setIvarInitializersERNS_10ASTContextEPPNS_18CXXCtorInitializerEj", "clang::ObjCImplementationDecl::setIvarInitializers(clang::ASTContext&, clang::CXXCtorInitializer**, unsigned int)"},
    {"_ZN5clang4Sema35DiagnoseUseOfUnimplementedSelectorsEv", "clang::Sema::DiagnoseUseOfUnimplementedSelectors()"},
    {"_Z23isObjCTypeSubstitutableRN5clang10ASTContextEPKNS_21ObjCObjectPointerTypeES4_b", "isObjCTypeSubstitutable(clang::ASTContext&, clang::ObjCObjectPointerType const*, clang::ObjCObjectPointerType const*, bool)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorESt4pairINS1_14ObjCMethodListES4_ENS_12DenseMapInfoIS2_EENS6_IS5_EEE6insertERKS3_IS2_S5_E", "llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodList, clang::ObjCMethodList>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodList, clang::ObjCMethodList> > >::insert(std::pair<clang::Selector, std::pair<clang::ObjCMethodList, clang::ObjCMethodList> > const&)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorESt4pairINS1_14ObjCMethodListES4_ENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PS3_IS2_S5_E", "llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodList, clang::ObjCMethodList>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodList, clang::ObjCMethodList> > >::InsertIntoBucket(clang::Selector const&, std::pair<clang::ObjCMethodList, clang::ObjCMethodList> const&, std::pair<clang::Selector, std::pair<clang::ObjCMethodList, clang::ObjCMethodList> >*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorESt4pairINS1_14ObjCMethodListES4_ENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Selector, std::pair<clang::ObjCMethodList, clang::ObjCMethodList>, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<std::pair<clang::ObjCMethodList, clang::ObjCMethodList> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEcNS_12DenseMapInfoIS2_EENS3_IcEEE6insertERKSt4pairIS2_cE", "llvm::DenseMap<clang::Selector, char, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::Selector, char> const&)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEcNS_12DenseMapInfoIS2_EENS3_IcEEE16InsertIntoBucketERKS2_RKcPSt4pairIS2_cE", "llvm::DenseMap<clang::Selector, char, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::Selector const&, char const&, std::pair<clang::Selector, char>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEcNS_12DenseMapInfoIS2_EENS3_IcEEE4growEj", "llvm::DenseMap<clang::Selector, char, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPKNS1_14ObjCMethodDeclENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<clang::Selector, clang::ObjCMethodDecl const*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::ObjCMethodDecl const*> >::InsertIntoBucket(clang::Selector const&, clang::ObjCMethodDecl const* const&, std::pair<clang::Selector, clang::ObjCMethodDecl const*>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPKNS1_14ObjCMethodDeclENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Selector, clang::ObjCMethodDecl const*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::ObjCMethodDecl const*> >::grow(unsigned int)"},
    {"_ZN5clang4Sema27CheckSpecifiedExceptionTypeENS_8QualTypeERKNS_11SourceRangeE", "clang::Sema::CheckSpecifiedExceptionType(clang::QualType, clang::SourceRange const&)"},
    {"_ZN5clang4Sema25CheckDistantExceptionSpecENS_8QualTypeE", "clang::Sema::CheckDistantExceptionSpec(clang::QualType)"},
    {"_ZN5clang4Sema28CheckEquivalentExceptionSpecEPNS_12FunctionDeclES2_", "clang::Sema::CheckEquivalentExceptionSpec(clang::FunctionDecl*, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema28CheckEquivalentExceptionSpecERKNS_17PartialDiagnosticES3_PKNS_17FunctionProtoTypeENS_14SourceLocationES6_S7_PbS8_", "clang::Sema::CheckEquivalentExceptionSpec(clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::FunctionProtoType const*, clang::SourceLocation, clang::FunctionProtoType const*, clang::SourceLocation, bool*, bool*)"},
    {"_ZN5clang4Sema28CheckEquivalentExceptionSpecEPKNS_17FunctionProtoTypeENS_14SourceLocationES3_S4_", "clang::Sema::CheckEquivalentExceptionSpec(clang::FunctionProtoType const*, clang::SourceLocation, clang::FunctionProtoType const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24CheckExceptionSpecSubsetERKNS_17PartialDiagnosticES3_PKNS_17FunctionProtoTypeENS_14SourceLocationES6_S7_", "clang::Sema::CheckExceptionSpecSubset(clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::FunctionProtoType const*, clang::SourceLocation, clang::FunctionProtoType const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema23CheckParamExceptionSpecERKNS_17PartialDiagnosticEPKNS_17FunctionProtoTypeENS_14SourceLocationES6_S7_", "clang::Sema::CheckParamExceptionSpec(clang::PartialDiagnostic const&, clang::FunctionProtoType const*, clang::SourceLocation, clang::FunctionProtoType const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema31CheckExceptionSpecCompatibilityEPNS_4ExprENS_8QualTypeE", "clang::Sema::CheckExceptionSpecCompatibility(clang::Expr*, clang::QualType)"},
    {"_ZN5clang21GetUnderlyingFunctionENS_8QualTypeE", "clang::GetUnderlyingFunction(clang::QualType)"},
    {"_ZN5clang4Sema36CheckOverridingFunctionExceptionSpecEPKNS_13CXXMethodDeclES3_", "clang::Sema::CheckOverridingFunctionExceptionSpec(clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZN5clang4Sema17DiagnoseUseOfDeclEPNS_9NamedDeclENS_14SourceLocationEb", "clang::Sema::DiagnoseUseOfDecl(clang::NamedDecl*, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema21DiagnoseSentinelCallsEPNS_9NamedDeclENS_14SourceLocationEPPNS_4ExprEj", "clang::Sema::DiagnoseSentinelCalls(clang::NamedDecl*, clang::SourceLocation, clang::Expr**, unsigned int)"},
    {"_ZNK5clang4Sema12getExprRangeEPNS_4ExprE", "clang::Sema::getExprRange(clang::Expr*) const"},
    {"_ZN5clang4Sema30DefaultFunctionArrayConversionERPNS_4ExprE", "clang::Sema::DefaultFunctionArrayConversion(clang::Expr*&)"},
    {"_ZN5clang4Sema23DefaultLvalueConversionERPNS_4ExprE", "clang::Sema::DefaultLvalueConversion(clang::Expr*&)"},
    {"_ZN5clang4Sema24ConvertPropertyForRValueERPNS_4ExprE", "clang::Sema::ConvertPropertyForRValue(clang::Expr*&)"},
    {"_ZN5clang4Sema36DefaultFunctionArrayLvalueConversionERPNS_4ExprE", "clang::Sema::DefaultFunctionArrayLvalueConversion(clang::Expr*&)"},
    {"_ZN5clang4Sema21UsualUnaryConversionsERPNS_4ExprE", "clang::Sema::UsualUnaryConversions(clang::Expr*&)"},
    {"_ZN5clang4Sema24DefaultArgumentPromotionERPNS_4ExprE", "clang::Sema::DefaultArgumentPromotion(clang::Expr*&)"},
    {"_ZN5clang4Sema32DefaultVariadicArgumentPromotionERPNS_4ExprENS0_16VariadicCallTypeEPNS_12FunctionDeclE", "clang::Sema::DefaultVariadicArgumentPromotion(clang::Expr*&, clang::Sema::VariadicCallType, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema19DiagRuntimeBehaviorENS_14SourceLocationERKNS_17PartialDiagnosticE", "clang::Sema::DiagRuntimeBehavior(clang::SourceLocation, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema26UsualArithmeticConversionsERPNS_4ExprES3_b", "clang::Sema::UsualArithmeticConversions(clang::Expr*&, clang::Expr*&, bool)"},
    {"_ZN5clang4Sema18ActOnStringLiteralEPKNS_5TokenEj", "clang::Sema::ActOnStringLiteral(clang::Token const*, unsigned int)"},
    {"_ZN5clang4Sema16BuildDeclRefExprEPNS_9ValueDeclENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPKNS_12CXXScopeSpecE", "clang::Sema::BuildDeclRefExpr(clang::ValueDecl*, clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::CXXScopeSpec const*)"},
    {"_ZN5clang4Sema16BuildDeclRefExprEPNS_9ValueDeclENS_8QualTypeENS_13ExprValueKindERKNS_19DeclarationNameInfoEPKNS_12CXXScopeSpecE", "clang::Sema::BuildDeclRefExpr(clang::ValueDecl*, clang::QualType, clang::ExprValueKind, clang::DeclarationNameInfo const&, clang::CXXScopeSpec const*)"},
    {"_ZN5clang4Sema25MarkDeclarationReferencedENS_14SourceLocationEPNS_4DeclE", "clang::Sema::MarkDeclarationReferenced(clang::SourceLocation, clang::Decl*)"},
    {"_ZN5clang4Sema40BuildAnonymousStructUnionMemberReferenceERKNS_12CXXScopeSpecENS_14SourceLocationEPNS_17IndirectFieldDeclEPNS_4ExprES4_", "clang::Sema::BuildAnonymousStructUnionMemberReference(clang::CXXScopeSpec const&, clang::SourceLocation, clang::IndirectFieldDecl*, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24BuildDeclarationNameExprERKNS_12CXXScopeSpecERKNS_19DeclarationNameInfoEPNS_9NamedDeclE", "clang::Sema::BuildDeclarationNameExpr(clang::CXXScopeSpec const&, clang::DeclarationNameInfo const&, clang::NamedDecl*)"},
    {"_Z23BuildFieldReferenceExprRN5clang4SemaEPNS_4ExprEbRKNS_12CXXScopeSpecEPNS_9FieldDeclENS_14DeclAccessPairERKNS_19DeclarationNameInfoE", "BuildFieldReferenceExpr(clang::Sema&, clang::Expr*, bool, clang::CXXScopeSpec const&, clang::FieldDecl*, clang::DeclAccessPair, clang::DeclarationNameInfo const&)"},
    {"_ZN5clang4Sema19DiagnoseEmptyLookupEPNS_5ScopeERNS_12CXXScopeSpecERNS_12LookupResultENS0_18CorrectTypoContextE", "clang::Sema::DiagnoseEmptyLookup(clang::Scope*, clang::CXXScopeSpec&, clang::LookupResult&, clang::Sema::CorrectTypoContext)"},
    {"_ZN5clang4Sema28canSynthesizeProvisionalIvarEPNS_14IdentifierInfoE", "clang::Sema::canSynthesizeProvisionalIvar(clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema28canSynthesizeProvisionalIvarEPNS_16ObjCPropertyDeclE", "clang::Sema::canSynthesizeProvisionalIvar(clang::ObjCPropertyDecl*)"},
    {"_ZN5clang4Sema17ActOnIdExpressionEPNS_5ScopeERNS_12CXXScopeSpecERNS_13UnqualifiedIdEbb", "clang::Sema::ActOnIdExpression(clang::Scope*, clang::CXXScopeSpec&, clang::UnqualifiedId&, bool, bool)"},
    {"_Z22DecomposeUnqualifiedIdRN5clang4SemaERKNS_13UnqualifiedIdERNS_24TemplateArgumentListInfoERNS_19DeclarationNameInfoERPKS5_", "DecomposeUnqualifiedId(clang::Sema&, clang::UnqualifiedId const&, clang::TemplateArgumentListInfo&, clang::DeclarationNameInfo&, clang::TemplateArgumentListInfo const*&)"},
    {"_ZN5clang4Sema18LookupInObjCMethodERNS_12LookupResultEPNS_5ScopeEPNS_14IdentifierInfoEb", "clang::Sema::LookupInObjCMethod(clang::LookupResult&, clang::Scope*, clang::IdentifierInfo*, bool)"},
    {"_ZN5clang4Sema26UseArgumentDependentLookupERKNS_12CXXScopeSpecERKNS_12LookupResultEb", "clang::Sema::UseArgumentDependentLookup(clang::CXXScopeSpec const&, clang::LookupResult const&, bool)"},
    {"_ZN5clang4Sema31BuildPossibleImplicitMemberExprERKNS_12CXXScopeSpecERNS_12LookupResultEPKNS_24TemplateArgumentListInfoE", "clang::Sema::BuildPossibleImplicitMemberExpr(clang::CXXScopeSpec const&, clang::LookupResult&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang4Sema24BuildDeclarationNameExprERKNS_12CXXScopeSpecERNS_12LookupResultEb", "clang::Sema::BuildDeclarationNameExpr(clang::CXXScopeSpec const&, clang::LookupResult&, bool)"},
    {"_ZN5clang4Sema23BuildImplicitMemberExprERKNS_12CXXScopeSpecERNS_12LookupResultEPKNS_24TemplateArgumentListInfoEb", "clang::Sema::BuildImplicitMemberExpr(clang::CXXScopeSpec const&, clang::LookupResult&, clang::TemplateArgumentListInfo const*, bool)"},
    {"_Z25DiagnoseInstanceReferenceRN5clang4SemaERKNS_12CXXScopeSpecEPNS_9NamedDeclERKNS_19DeclarationNameInfoE", "DiagnoseInstanceReference(clang::Sema&, clang::CXXScopeSpec const&, clang::NamedDecl*, clang::DeclarationNameInfo const&)"},
    {"_ZN5clang4Sema33BuildQualifiedDeclarationNameExprERNS_12CXXScopeSpecERKNS_19DeclarationNameInfoE", "clang::Sema::BuildQualifiedDeclarationNameExpr(clang::CXXScopeSpec&, clang::DeclarationNameInfo const&)"},
    {"_ZN5clang4Sema29PerformObjectMemberConversionERPNS_4ExprEPNS_19NestedNameSpecifierEPNS_9NamedDeclES7_", "clang::Sema::PerformObjectMemberConversion(clang::Expr*&, clang::NestedNameSpecifier*, clang::NamedDecl*, clang::NamedDecl*)"},
    {"_ZN5clang4Sema24BuildMemberReferenceExprEPNS_4ExprENS_8QualTypeENS_14SourceLocationEbRKNS_12CXXScopeSpecEPNS_9NamedDeclERNS_12LookupResultEPKNS_24TemplateArgumentListInfoEb", "clang::Sema::BuildMemberReferenceExpr(clang::Expr*, clang::QualType, clang::SourceLocation, bool, clang::CXXScopeSpec const&, clang::NamedDecl*, clang::LookupResult&, clang::TemplateArgumentListInfo const*, bool)"},
    {"_Z15CheckDeclInExprRN5clang4SemaENS_14SourceLocationEPNS_9NamedDeclE", "CheckDeclInExpr(clang::Sema&, clang::SourceLocation, clang::NamedDecl*)"},
    {"_Z21BuildBlockDeclRefExprRN5clang4SemaEPNS_9ValueDeclERKNS_19DeclarationNameInfoEb", "BuildBlockDeclRefExpr(clang::Sema&, clang::ValueDecl*, clang::DeclarationNameInfo const&, bool)"},
    {"_ZN5clang4Sema19ActOnPredefinedExprENS_14SourceLocationENS_3tok9TokenKindE", "clang::Sema::ActOnPredefinedExpr(clang::SourceLocation, clang::tok::TokenKind)"},
    {"_ZN5clang4Sema22ActOnCharacterConstantERKNS_5TokenE", "clang::Sema::ActOnCharacterConstant(clang::Token const&)"},
    {"_ZN5clang4Sema20ActOnNumericConstantERKNS_5TokenE", "clang::Sema::ActOnNumericConstant(clang::Token const&)"},
    {"_ZN5clang4Sema14ActOnParenExprENS_14SourceLocationES1_PNS_4ExprE", "clang::Sema::ActOnParenExpr(clang::SourceLocation, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema25CheckSizeOfAlignOfOperandENS_8QualTypeENS_14SourceLocationENS_11SourceRangeEb", "clang::Sema::CheckSizeOfAlignOfOperand(clang::QualType, clang::SourceLocation, clang::SourceRange, bool)"},
    {"_ZN5clang4Sema23CreateSizeOfAlignOfExprEPNS_14TypeSourceInfoENS_14SourceLocationEbNS_11SourceRangeE", "clang::Sema::CreateSizeOfAlignOfExpr(clang::TypeSourceInfo*, clang::SourceLocation, bool, clang::SourceRange)"},
    {"_ZN5clang4Sema23CreateSizeOfAlignOfExprEPNS_4ExprENS_14SourceLocationEbNS_11SourceRangeE", "clang::Sema::CreateSizeOfAlignOfExpr(clang::Expr*, clang::SourceLocation, bool, clang::SourceRange)"},
    {"_ZN5clang4Sema20CheckPlaceholderExprEPNS_4ExprENS_14SourceLocationE", "clang::Sema::CheckPlaceholderExpr(clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema22ActOnSizeOfAlignOfExprENS_14SourceLocationEbbPvRKNS_11SourceRangeE", "clang::Sema::ActOnSizeOfAlignOfExpr(clang::SourceLocation, bool, bool, void*, clang::SourceRange const&)"},
    {"_ZN5clang4Sema19ActOnPostfixUnaryOpEPNS_5ScopeENS_14SourceLocationENS_3tok9TokenKindEPNS_4ExprE", "clang::Sema::ActOnPostfixUnaryOp(clang::Scope*, clang::SourceLocation, clang::tok::TokenKind, clang::Expr*)"},
    {"_ZN5clang4Sema12BuildUnaryOpEPNS_5ScopeENS_14SourceLocationENS_17UnaryOperatorKindEPNS_4ExprE", "clang::Sema::BuildUnaryOp(clang::Scope*, clang::SourceLocation, clang::UnaryOperatorKind, clang::Expr*)"},
    {"_ZN5clang4Sema23ActOnArraySubscriptExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationES4_S5_", "clang::Sema::ActOnArraySubscriptExpr(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema36MaybeConvertParenListExprToParenExprEPNS_5ScopeEPNS_4ExprE", "clang::Sema::MaybeConvertParenListExprToParenExpr(clang::Scope*, clang::Expr*)"},
    {"_ZN5clang4Sema31CreateBuiltinArraySubscriptExprEPNS_4ExprENS_14SourceLocationES2_S3_", "clang::Sema::CreateBuiltinArraySubscriptExpr(clang::Expr*, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24ActOnDependentMemberExprEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationERKNS_12CXXScopeSpecEPNS_9NamedDeclERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::Sema::ActOnDependentMemberExpr(clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::CXXScopeSpec const&, clang::NamedDecl*, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang4Sema29CheckQualifiedMemberReferenceEPNS_4ExprENS_8QualTypeERKNS_12CXXScopeSpecERKNS_12LookupResultE", "clang::Sema::CheckQualifiedMemberReference(clang::Expr*, clang::QualType, clang::CXXScopeSpec const&, clang::LookupResult const&)"},
    {"_Z24IsProvablyNotDerivedFromRN5clang4SemaEPNS_13CXXRecordDeclERKN4llvm11SmallPtrSetIS3_Lj4EEE", "IsProvablyNotDerivedFrom(clang::Sema&, clang::CXXRecordDecl*, llvm::SmallPtrSet<clang::CXXRecordDecl*, 4u> const&)"},
    {"_ZN5clang4Sema24BuildMemberReferenceExprEPNS_4ExprENS_8QualTypeENS_14SourceLocationEbRNS_12CXXScopeSpecEPNS_9NamedDeclERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::Sema::BuildMemberReferenceExpr(clang::Expr*, clang::QualType, clang::SourceLocation, bool, clang::CXXScopeSpec&, clang::NamedDecl*, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_Z24LookupMemberExprInRecordRN5clang4SemaERNS_12LookupResultENS_11SourceRangeEPKNS_10RecordTypeENS_14SourceLocationERNS_12CXXScopeSpecEb", "LookupMemberExprInRecord(clang::Sema&, clang::LookupResult&, clang::SourceRange, clang::RecordType const*, clang::SourceLocation, clang::CXXScopeSpec&, bool)"},
    {"_ZN5clang4Sema16LookupMemberExprERNS_12LookupResultERPNS_4ExprERbNS_14SourceLocationERNS_12CXXScopeSpecEPNS_4DeclEb", "clang::Sema::LookupMemberExpr(clang::LookupResult&, clang::Expr*&, bool&, clang::SourceLocation, clang::CXXScopeSpec&, clang::Decl*, bool)"},
    {"_Z34ShouldTryAgainWithRedefinitionTypeRN5clang4SemaERPNS_4ExprE", "ShouldTryAgainWithRedefinitionType(clang::Sema&, clang::Expr*&)"},
    {"_Z24FindGetterSetterNameDeclPKN5clang21ObjCObjectPointerTypeEPNS_14IdentifierInfoERKNS_8SelectorERNS_10ASTContextE", "FindGetterSetterNameDecl(clang::ObjCObjectPointerType const*, clang::IdentifierInfo*, clang::Selector const&, clang::ASTContext&)"},
    {"_ZN5clang4Sema13ActOnCallExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationENS_11ASTMultiPtrIS4_EES5_S4_", "clang::Sema::ActOnCallExpr(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema21ActOnMemberAccessExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationENS_3tok9TokenKindERNS_12CXXScopeSpecERNS_13UnqualifiedIdEPNS_4DeclEb", "clang::Sema::ActOnMemberAccessExpr(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::tok::TokenKind, clang::CXXScopeSpec&, clang::UnqualifiedId&, clang::Decl*, bool)"},
    {"_ZN5clang4Sema22BuildCXXDefaultArgExprENS_14SourceLocationEPNS_12FunctionDeclEPNS_11ParmVarDeclE", "clang::Sema::BuildCXXDefaultArgExpr(clang::SourceLocation, clang::FunctionDecl*, clang::ParmVarDecl*)"},
    {"_ZN5clang4Sema32MarkDeclarationsReferencedInExprEPNS_4ExprE", "clang::Sema::MarkDeclarationsReferencedInExpr(clang::Expr*)"},
    {"_ZN5clang4Sema23ConvertArgumentsForCallEPNS_8CallExprEPNS_4ExprEPNS_12FunctionDeclEPKNS_17FunctionProtoTypeEPS4_jNS_14SourceLocationE", "clang::Sema::ConvertArgumentsForCall(clang::CallExpr*, clang::Expr*, clang::FunctionDecl*, clang::FunctionProtoType const*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema22GatherArgumentsForCallENS_14SourceLocationEPNS_12FunctionDeclEPKNS_17FunctionProtoTypeEjPPNS_4ExprEjRN4llvm11SmallVectorIS8_Lj8EEENS0_16VariadicCallTypeE", "clang::Sema::GatherArgumentsForCall(clang::SourceLocation, clang::FunctionDecl*, clang::FunctionProtoType const*, unsigned int, clang::Expr**, unsigned int, llvm::SmallVector<clang::Expr*, 8u>&, clang::Sema::VariadicCallType)"},
    {"_ZN5clang4Sema19CheckCallReturnTypeENS_8QualTypeENS_14SourceLocationEPNS_8CallExprEPNS_12FunctionDeclE", "clang::Sema::CheckCallReturnType(clang::QualType, clang::SourceLocation, clang::CallExpr*, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema21BuildResolvedCallExprEPNS_4ExprEPNS_9NamedDeclENS_14SourceLocationEPS2_jS5_S2_", "clang::Sema::BuildResolvedCallExpr(clang::Expr*, clang::NamedDecl*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema23ActOnCUDAExecConfigExprEPNS_5ScopeENS_14SourceLocationENS_11ASTMultiPtrIPNS_4ExprEEES3_", "clang::Sema::ActOnCUDAExecConfigExpr(clang::Scope*, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema20ActOnCompoundLiteralENS_14SourceLocationENS_9OpaquePtrINS_8QualTypeEEES1_PNS_4ExprE", "clang::Sema::ActOnCompoundLiteral(clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema24BuildCompoundLiteralExprENS_14SourceLocationEPNS_14TypeSourceInfoES1_PNS_4ExprE", "clang::Sema::BuildCompoundLiteralExpr(clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema13ActOnInitListENS_14SourceLocationENS_11ASTMultiPtrIPNS_4ExprEEES1_", "clang::Sema::ActOnInitList(clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema14CheckCastTypesENS_11SourceRangeENS_8QualTypeERPNS_4ExprERNS_8CastKindERNS_13ExprValueKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEb", "clang::Sema::CheckCastTypes(clang::SourceRange, clang::QualType, clang::Expr*&, clang::CastKind&, clang::ExprValueKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&, bool)"},
    {"_ZN5clang4Sema18CheckExtVectorCastENS_11SourceRangeENS_8QualTypeERPNS_4ExprERNS_8CastKindE", "clang::Sema::CheckExtVectorCast(clang::SourceRange, clang::QualType, clang::Expr*&, clang::CastKind&)"},
    {"_ZN5clang4Sema15CheckVectorCastENS_11SourceRangeENS_8QualTypeES2_RNS_8CastKindE", "clang::Sema::CheckVectorCast(clang::SourceRange, clang::QualType, clang::QualType, clang::CastKind&)"},
    {"_Z17PrepareScalarCastRN5clang4SemaERPNS_4ExprENS_8QualTypeE", "PrepareScalarCast(clang::Sema&, clang::Expr*&, clang::QualType)"},
    {"_ZN5clang4Sema13ActOnCastExprEPNS_5ScopeENS_14SourceLocationENS_9OpaquePtrINS_8QualTypeEEES3_PNS_4ExprE", "clang::Sema::ActOnCastExpr(clang::Scope*, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema24ActOnCastOfParenListExprEPNS_5ScopeENS_14SourceLocationES3_PNS_4ExprEPNS_14TypeSourceInfoE", "clang::Sema::ActOnCastOfParenListExpr(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema19BuildCStyleCastExprENS_14SourceLocationEPNS_14TypeSourceInfoES1_PNS_4ExprE", "clang::Sema::BuildCStyleCastExpr(clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema10ActOnBinOpEPNS_5ScopeENS_14SourceLocationENS_3tok9TokenKindEPNS_4ExprES7_", "clang::Sema::ActOnBinOp(clang::Scope*, clang::SourceLocation, clang::tok::TokenKind, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema25ActOnParenOrParenListExprENS_14SourceLocationES1_NS_11ASTMultiPtrIPNS_4ExprEEENS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::ActOnParenOrParenListExpr(clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang4Sema24CheckConditionalOperandsERPNS_4ExprES3_S3_S3_RNS_13ExprValueKindERNS_14ExprObjectKindENS_14SourceLocationE", "clang::Sema::CheckConditionalOperands(clang::Expr*&, clang::Expr*&, clang::Expr*&, clang::Expr*&, clang::ExprValueKind&, clang::ExprObjectKind&, clang::SourceLocation)"},
    {"_ZN5clang4Sema19CheckVectorOperandsENS_14SourceLocationERPNS_4ExprES4_", "clang::Sema::CheckVectorOperands(clang::SourceLocation, clang::Expr*&, clang::Expr*&)"},
    {"_ZN5clang4Sema28FindCompositeObjCPointerTypeERPNS_4ExprES3_NS_14SourceLocationE", "clang::Sema::FindCompositeObjCPointerType(clang::Expr*&, clang::Expr*&, clang::SourceLocation)"},
    {"_ZN5clang4Sema18ActOnConditionalOpENS_14SourceLocationES1_PNS_4ExprES3_S3_", "clang::Sema::ActOnConditionalOp(clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema26CheckAssignmentConstraintsENS_14SourceLocationENS_8QualTypeES2_", "clang::Sema::CheckAssignmentConstraints(clang::SourceLocation, clang::QualType, clang::QualType)"},
    {"_ZN5clang4Sema26CheckAssignmentConstraintsENS_8QualTypeERPNS_4ExprERNS_8CastKindE", "clang::Sema::CheckAssignmentConstraints(clang::QualType, clang::Expr*&, clang::CastKind&)"},
    {"_ZN5clang4Sema40CheckTransparentUnionArgumentConstraintsENS_8QualTypeERPNS_4ExprE", "clang::Sema::CheckTransparentUnionArgumentConstraints(clang::QualType, clang::Expr*&)"},
    {"_ZN5clang4Sema32CheckSingleAssignmentConstraintsENS_8QualTypeERPNS_4ExprE", "clang::Sema::CheckSingleAssignmentConstraints(clang::QualType, clang::Expr*&)"},
    {"_ZN5clang4Sema15InvalidOperandsENS_14SourceLocationERPNS_4ExprES4_", "clang::Sema::InvalidOperands(clang::SourceLocation, clang::Expr*&, clang::Expr*&)"},
    {"_ZN5clang4Sema27CheckMultiplyDivideOperandsERPNS_4ExprES3_NS_14SourceLocationEbb", "clang::Sema::CheckMultiplyDivideOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang4Sema22CheckRemainderOperandsERPNS_4ExprES3_NS_14SourceLocationEb", "clang::Sema::CheckRemainderOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema21CheckAdditionOperandsERPNS_4ExprES3_NS_14SourceLocationEPNS_8QualTypeE", "clang::Sema::CheckAdditionOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, clang::QualType*)"},
    {"_ZN5clang4Sema24CheckSubtractionOperandsERPNS_4ExprES3_NS_14SourceLocationEPNS_8QualTypeE", "clang::Sema::CheckSubtractionOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, clang::QualType*)"},
    {"_ZN5clang4Sema18CheckShiftOperandsERPNS_4ExprES3_NS_14SourceLocationEb", "clang::Sema::CheckShiftOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema20CheckCompareOperandsERPNS_4ExprES3_NS_14SourceLocationEjb", "clang::Sema::CheckCompareOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, unsigned int, bool)"},
    {"_ZN5clang4Sema26CheckVectorCompareOperandsERPNS_4ExprES3_NS_14SourceLocationEb", "clang::Sema::CheckVectorCompareOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema23CheckAssignmentOperandsEPNS_4ExprERS2_NS_14SourceLocationENS_8QualTypeE", "clang::Sema::CheckAssignmentOperands(clang::Expr*, clang::Expr*&, clang::SourceLocation, clang::QualType)"},
    {"_Z24CheckForModifiableLvaluePN5clang4ExprENS_14SourceLocationERNS_4SemaE", "CheckForModifiableLvalue(clang::Expr*, clang::SourceLocation, clang::Sema&)"},
    {"_ZN5clang4Sema24ConvertPropertyForLValueERPNS_4ExprES3_RNS_8QualTypeE", "clang::Sema::ConvertPropertyForLValue(clang::Expr*&, clang::Expr*&, clang::QualType&)"},
    {"_ZN5clang4Sema24DiagnoseAssignmentResultENS0_17AssignConvertTypeENS_14SourceLocationENS_8QualTypeES3_PNS_4ExprENS0_16AssignmentActionEPb", "clang::Sema::DiagnoseAssignmentResult(clang::Sema::AssignConvertType, clang::SourceLocation, clang::QualType, clang::QualType, clang::Expr*, clang::Sema::AssignmentAction, bool*)"},
    {"_ZN5clang4Sema18CreateBuiltinBinOpENS_14SourceLocationENS_18BinaryOperatorKindEPNS_4ExprES4_", "clang::Sema::CreateBuiltinBinOp(clang::SourceLocation, clang::BinaryOperatorKind, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema10BuildBinOpEPNS_5ScopeENS_14SourceLocationENS_18BinaryOperatorKindEPNS_4ExprES6_", "clang::Sema::BuildBinOp(clang::Scope*, clang::SourceLocation, clang::BinaryOperatorKind, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema20CreateBuiltinUnaryOpENS_14SourceLocationENS_17UnaryOperatorKindEPNS_4ExprE", "clang::Sema::CreateBuiltinUnaryOp(clang::SourceLocation, clang::UnaryOperatorKind, clang::Expr*)"},
    {"_Z30CheckIncrementDecrementOperandRN5clang4SemaEPNS_4ExprERNS_13ExprValueKindENS_14SourceLocationEbb", "CheckIncrementDecrementOperand(clang::Sema&, clang::Expr*, clang::ExprValueKind&, clang::SourceLocation, bool, bool)"},
    {"_Z23CheckIndirectionOperandRN5clang4SemaEPNS_4ExprERNS_13ExprValueKindENS_14SourceLocationE", "CheckIndirectionOperand(clang::Sema&, clang::Expr*, clang::ExprValueKind&, clang::SourceLocation)"},
    {"_Z20CheckRealImagOperandRN5clang4SemaERPNS_4ExprENS_14SourceLocationEb", "CheckRealImagOperand(clang::Sema&, clang::Expr*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema12ActOnUnaryOpEPNS_5ScopeENS_14SourceLocationENS_3tok9TokenKindEPNS_4ExprE", "clang::Sema::ActOnUnaryOp(clang::Scope*, clang::SourceLocation, clang::tok::TokenKind, clang::Expr*)"},
    {"_ZN5clang4Sema14ActOnAddrLabelENS_14SourceLocationES1_PNS_14IdentifierInfoE", "clang::Sema::ActOnAddrLabel(clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema13ActOnStmtExprENS_14SourceLocationEPNS_4StmtES1_", "clang::Sema::ActOnStmtExpr(clang::SourceLocation, clang::Stmt*, clang::SourceLocation)"},
    {"_ZN5clang4Sema20BuildBuiltinOffsetOfENS_14SourceLocationEPNS_14TypeSourceInfoEPNS0_17OffsetOfComponentEjS1_", "clang::Sema::BuildBuiltinOffsetOf(clang::SourceLocation, clang::TypeSourceInfo*, clang::Sema::OffsetOfComponent*, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema20ActOnBuiltinOffsetOfEPNS_5ScopeENS_14SourceLocationES3_NS_9OpaquePtrINS_8QualTypeEEEPNS0_17OffsetOfComponentEjS3_", "clang::Sema::ActOnBuiltinOffsetOf(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::Sema::OffsetOfComponent*, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema15ActOnChooseExprENS_14SourceLocationEPNS_4ExprES3_S3_S1_", "clang::Sema::ActOnChooseExpr(clang::SourceLocation, clang::Expr*, clang::Expr*, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema15ActOnBlockStartENS_14SourceLocationEPNS_5ScopeE", "clang::Sema::ActOnBlockStart(clang::SourceLocation, clang::Scope*)"},
    {"_ZN5clang4Sema19ActOnBlockArgumentsERNS_10DeclaratorEPNS_5ScopeE", "clang::Sema::ActOnBlockArguments(clang::Declarator&, clang::Scope*)"},
    {"_ZN5clang4Sema15ActOnBlockErrorENS_14SourceLocationEPNS_5ScopeE", "clang::Sema::ActOnBlockError(clang::SourceLocation, clang::Scope*)"},
    {"_ZN5clang4Sema18ActOnBlockStmtExprENS_14SourceLocationEPNS_4StmtEPNS_5ScopeE", "clang::Sema::ActOnBlockStmtExpr(clang::SourceLocation, clang::Stmt*, clang::Scope*)"},
    {"_ZN5clang4Sema10ActOnVAArgENS_14SourceLocationEPNS_4ExprENS_9OpaquePtrINS_8QualTypeEEES1_", "clang::Sema::ActOnVAArg(clang::SourceLocation, clang::Expr*, clang::OpaquePtr<clang::QualType>, clang::SourceLocation)"},
    {"_ZN5clang4Sema14BuildVAArgExprENS_14SourceLocationEPNS_4ExprEPNS_14TypeSourceInfoES1_", "clang::Sema::BuildVAArgExpr(clang::SourceLocation, clang::Expr*, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema16ActOnGNUNullExprENS_14SourceLocationE", "clang::Sema::ActOnGNUNullExpr(clang::SourceLocation)"},
    {"_ZN5clang4Sema31VerifyIntegerConstantExpressionEPKNS_4ExprEPN4llvm6APSIntE", "clang::Sema::VerifyIntegerConstantExpression(clang::Expr const*, llvm::APSInt*)"},
    {"_ZN5clang4Sema31PushExpressionEvaluationContextENS0_27ExpressionEvaluationContextE", "clang::Sema::PushExpressionEvaluationContext(clang::Sema::ExpressionEvaluationContext)"},
    {"_ZN5clang4Sema30PopExpressionEvaluationContextEv", "clang::Sema::PopExpressionEvaluationContext()"},
    {"_ZN5clang4Sema32MarkDeclarationsReferencedInTypeENS_14SourceLocationENS_8QualTypeE", "clang::Sema::MarkDeclarationsReferencedInType(clang::SourceLocation, clang::QualType)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE12TraverseTypeENS_8QualTypeE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseType(clang::QualType)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_119EvaluatedExprMarkerEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::EvaluatedExprMarker, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang4Sema29DiagnoseAssignmentAsConditionEPNS_4ExprE", "clang::Sema::DiagnoseAssignmentAsCondition(clang::Expr*)"},
    {"_ZN5clang4Sema31DiagnoseEqualityWithExtraParensEPNS_9ParenExprE", "clang::Sema::DiagnoseEqualityWithExtraParens(clang::ParenExpr*)"},
    {"_ZN5clang4Sema21CheckBooleanConditionERPNS_4ExprENS_14SourceLocationE", "clang::Sema::CheckBooleanCondition(clang::Expr*&, clang::SourceLocation)"},
    {"_ZN5clang4Sema21ActOnBooleanConditionEPNS_5ScopeENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnBooleanCondition(clang::Scope*, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang20EvaluatedExprVisitorIN12_GLOBAL__N_119EvaluatedExprMarkerEE9VisitStmtEPNS_4StmtE", "clang::EvaluatedExprVisitor<(anonymous namespace)::EvaluatedExprMarker>::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE27TraverseNestedNameSpecifierEPNS_19NestedNameSpecifierE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseNestedNameSpecifier(clang::NestedNameSpecifier*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE24TraverseTemplateArgumentERKNS_16TemplateArgumentE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE12TraverseStmtEPNS_4StmtE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseStmt(clang::Stmt*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE15TraverseTypeLocENS_7TypeLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseTypeLoc(clang::TypeLoc)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE27TraverseTemplateArgumentLocERKNS_19TemplateArgumentLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseTemplateArgumentLoc(clang::TemplateArgumentLoc const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE12TraverseDeclEPNS_4DeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseDecl(clang::Decl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE24TraverseDeclaratorHelperEPNS_14DeclaratorDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseDeclaratorHelper(clang::DeclaratorDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE22TraverseFunctionHelperEPNS_12FunctionDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseFunctionHelper(clang::FunctionDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_119MarkReferencedDeclsEE23TraverseCXXRecordHelperEPNS_13CXXRecordDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::MarkReferencedDecls>::TraverseCXXRecordHelper(clang::CXXRecordDecl*)"},
    {"_Z38EmitDiagnosticForLogicalAndInLogicalOrRN5clang4SemaENS_14SourceLocationEPNS_4ExprE", "EmitDiagnosticForLogicalAndInLogicalOr(clang::Sema&, clang::SourceLocation, clang::Expr*)"},
    {"_Z18SuggestParenthesesRN5clang4SemaENS_14SourceLocationERKNS_17PartialDiagnosticES5_NS_11SourceRangeES5_S6_", "SuggestParentheses(clang::Sema&, clang::SourceLocation, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::SourceRange, clang::PartialDiagnostic const&, clang::SourceRange)"},
    {"_Z40FindGetterSetterNameDeclFromProtocolListPKN5clang16ObjCProtocolDeclEPNS_14IdentifierInfoERKNS_8SelectorERNS_10ASTContextE", "FindGetterSetterNameDeclFromProtocolList(clang::ObjCProtocolDecl const*, clang::IdentifierInfo*, clang::Selector const&, clang::ASTContext&)"},
    {"_Z16propagateCaptureRN5clang4SemaEjRKNS_9BlockDecl7CaptureE", "propagateCapture(clang::Sema&, unsigned int, clang::BlockDecl::Capture const&)"},
    {"_ZN5clang13SelectorTable19constructSetterNameERNS_15IdentifierTableERS0_PKNS_14IdentifierInfoE", "clang::SelectorTable::constructSetterName(clang::IdentifierTable&, clang::SelectorTable&, clang::IdentifierInfo const*)"},
    {"_ZN5clang4Sema20CheckBitwiseOperandsERPNS_4ExprES3_NS_14SourceLocationEb", "clang::Sema::CheckBitwiseOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema20CheckLogicalOperandsERPNS_4ExprES3_NS_14SourceLocationEj", "clang::Sema::CheckLogicalOperands(clang::Expr*&, clang::Expr*&, clang::SourceLocation, unsigned int)"},
    {"_ZNK5clang4Decl7getAttrINS_12NoReturnAttrEEEPT_v", "clang::NoReturnAttr* clang::Decl::getAttr<clang::NoReturnAttr>() const"},
    {"_ZN5clang4Sema33ExpressionEvaluationContextRecord7DestroyEv", "clang::Sema::ExpressionEvaluationContextRecord::Destroy()"},
    {"_ZN5clang4Sema33ExpressionEvaluationContextRecord13addDiagnosticENS_14SourceLocationERKNS_17PartialDiagnosticE", "clang::Sema::ExpressionEvaluationContextRecord::addDiagnostic(clang::SourceLocation, clang::PartialDiagnostic const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang12OffsetOfExpr12OffsetOfNodeELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::OffsetOfExpr::OffsetOfNode, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_9LabelStmtENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, clang::LabelStmt*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::LabelStmt*> >::InsertIntoBucket(clang::IdentifierInfo* const&, clang::LabelStmt* const&, std::pair<clang::IdentifierInfo*, clang::LabelStmt*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_9LabelStmtENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, clang::LabelStmt*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::LabelStmt*> >::grow(unsigned int)"},
    {"_ZN5clang10ChooseExprC2ENS_14SourceLocationEPNS_4ExprES3_S3_NS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindES1_bb", "clang::ChooseExpr::ChooseExpr(clang::SourceLocation, clang::Expr*, clang::Expr*, clang::Expr*, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang17CXXDefaultArgExprC2ENS_4Stmt9StmtClassENS_14SourceLocationEPNS_11ParmVarDeclE", "clang::CXXDefaultArgExpr::CXXDefaultArgExpr(clang::Stmt::StmtClass, clang::SourceLocation, clang::ParmVarDecl*)"},
    {"_ZN5clang18ArraySubscriptExprC2EPNS_4ExprES2_NS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindENS_14SourceLocationE", "clang::ArraySubscriptExpr::ArraySubscriptExpr(clang::Expr*, clang::Expr*, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind, clang::SourceLocation)"},
    {"_ZN4llvm8DenseMapIPN5clang7VarDeclEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<clang::VarDecl*, unsigned int, llvm::DenseMapInfo<clang::VarDecl*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::VarDecl* const&, unsigned int const&, std::pair<clang::VarDecl*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPN5clang7VarDeclEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<clang::VarDecl*, unsigned int, llvm::DenseMapInfo<clang::VarDecl*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang4Sema17getDestructorNameENS_14SourceLocationERNS_14IdentifierInfoES1_PNS_5ScopeERNS_12CXXScopeSpecENS_9OpaquePtrINS_8QualTypeEEEb", "clang::Sema::getDestructorName(clang::SourceLocation, clang::IdentifierInfo&, clang::SourceLocation, clang::Scope*, clang::CXXScopeSpec&, clang::OpaquePtr<clang::QualType>, bool)"},
    {"_ZN5clang4Sema14BuildCXXTypeIdENS_8QualTypeENS_14SourceLocationEPNS_14TypeSourceInfoES2_", "clang::Sema::BuildCXXTypeId(clang::QualType, clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema14BuildCXXTypeIdENS_8QualTypeENS_14SourceLocationEPNS_4ExprES2_", "clang::Sema::BuildCXXTypeId(clang::QualType, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema14ActOnCXXTypeidENS_14SourceLocationES1_bPvS1_", "clang::Sema::ActOnCXXTypeid(clang::SourceLocation, clang::SourceLocation, bool, void*, clang::SourceLocation)"},
    {"_ZN5clang4Sema14BuildCXXUuidofENS_8QualTypeENS_14SourceLocationEPNS_14TypeSourceInfoES2_", "clang::Sema::BuildCXXUuidof(clang::QualType, clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_Z17GetUuidAttrOfTypeN5clang8QualTypeE", "GetUuidAttrOfType(clang::QualType)"},
    {"_ZN5clang4Sema14BuildCXXUuidofENS_8QualTypeENS_14SourceLocationEPNS_4ExprES2_", "clang::Sema::BuildCXXUuidof(clang::QualType, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema14ActOnCXXUuidofENS_14SourceLocationES1_bPvS1_", "clang::Sema::ActOnCXXUuidof(clang::SourceLocation, clang::SourceLocation, bool, void*, clang::SourceLocation)"},
    {"_ZN5clang4Sema19ActOnCXXBoolLiteralENS_14SourceLocationENS_3tok9TokenKindE", "clang::Sema::ActOnCXXBoolLiteral(clang::SourceLocation, clang::tok::TokenKind)"},
    {"_ZN5clang4Sema22ActOnCXXNullPtrLiteralENS_14SourceLocationE", "clang::Sema::ActOnCXXNullPtrLiteral(clang::SourceLocation)"},
    {"_ZN5clang4Sema13ActOnCXXThrowENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnCXXThrow(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema20CheckCXXThrowOperandENS_14SourceLocationERPNS_4ExprE", "clang::Sema::CheckCXXThrowOperand(clang::SourceLocation, clang::Expr*&)"},
    {"_ZN5clang4Sema17tryCaptureCXXThisEv", "clang::Sema::tryCaptureCXXThis()"},
    {"_ZN5clang4Sema12ActOnCXXThisENS_14SourceLocationE", "clang::Sema::ActOnCXXThis(clang::SourceLocation)"},
    {"_ZN5clang4Sema25ActOnCXXTypeConstructExprENS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationENS_11ASTMultiPtrIPNS_4ExprEEES4_", "clang::Sema::ActOnCXXTypeConstructExpr(clang::OpaquePtr<clang::QualType>, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema25BuildCXXTypeConstructExprEPNS_14TypeSourceInfoENS_14SourceLocationENS_11ASTMultiPtrIPNS_4ExprEEES3_", "clang::Sema::BuildCXXTypeConstructExpr(clang::TypeSourceInfo*, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema11ActOnCXXNewENS_14SourceLocationEbS1_NS_11ASTMultiPtrIPNS_4ExprEEES1_NS_11SourceRangeERNS_10DeclaratorES1_S5_S1_", "clang::Sema::ActOnCXXNew(clang::SourceLocation, bool, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation, clang::SourceRange, clang::Declarator&, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema11BuildCXXNewENS_14SourceLocationEbS1_NS_11ASTMultiPtrIPNS_4ExprEEES1_NS_11SourceRangeENS_8QualTypeEPNS_14TypeSourceInfoES4_S1_S5_S1_", "clang::Sema::BuildCXXNew(clang::SourceLocation, bool, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation, clang::SourceRange, clang::QualType, clang::TypeSourceInfo*, clang::Expr*, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation)"},
    {"_ZN5clang4Sema18CheckAllocatedTypeENS_8QualTypeENS_14SourceLocationENS_11SourceRangeE", "clang::Sema::CheckAllocatedType(clang::QualType, clang::SourceLocation, clang::SourceRange)"},
    {"_ZN5clang4Sema23FindAllocationFunctionsENS_14SourceLocationENS_11SourceRangeEbNS_8QualTypeEbPPNS_4ExprEjRPNS_12FunctionDeclES9_", "clang::Sema::FindAllocationFunctions(clang::SourceLocation, clang::SourceRange, bool, clang::QualType, bool, clang::Expr**, unsigned int, clang::FunctionDecl*&, clang::FunctionDecl*&)"},
    {"_Z28doesUsualArrayDeleteWantSizeRN5clang4SemaENS_14SourceLocationENS_8QualTypeE", "doesUsualArrayDeleteWantSize(clang::Sema&, clang::SourceLocation, clang::QualType)"},
    {"_ZN5clang4Sema22FindAllocationOverloadENS_14SourceLocationENS_11SourceRangeENS_15DeclarationNameEPPNS_4ExprEjPNS_11DeclContextEbRPNS_12FunctionDeclE", "clang::Sema::FindAllocationOverload(clang::SourceLocation, clang::SourceRange, clang::DeclarationName, clang::Expr**, unsigned int, clang::DeclContext*, bool, clang::FunctionDecl*&)"},
    {"_ZN5clang4Sema22DeclareGlobalNewDeleteEv", "clang::Sema::DeclareGlobalNewDelete()"},
    {"_ZN5clang4Sema31DeclareGlobalAllocationFunctionENS_15DeclarationNameENS_8QualTypeES2_b", "clang::Sema::DeclareGlobalAllocationFunction(clang::DeclarationName, clang::QualType, clang::QualType, bool)"},
    {"_ZN5clang4Sema24FindDeallocationFunctionENS_14SourceLocationEPNS_13CXXRecordDeclENS_15DeclarationNameERPNS_12FunctionDeclE", "clang::Sema::FindDeallocationFunction(clang::SourceLocation, clang::CXXRecordDecl*, clang::DeclarationName, clang::FunctionDecl*&)"},
    {"_ZN5clang4Sema14ActOnCXXDeleteENS_14SourceLocationEbbPNS_4ExprE", "clang::Sema::ActOnCXXDelete(clang::SourceLocation, bool, bool, clang::Expr*)"},
    {"_ZN5clang4Sema22CheckConditionVariableEPNS_7VarDeclENS_14SourceLocationEb", "clang::Sema::CheckConditionVariable(clang::VarDecl*, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema24CheckCXXBooleanConditionERPNS_4ExprE", "clang::Sema::CheckCXXBooleanCondition(clang::Expr*&)"},
    {"_ZN5clang4Sema42IsStringLiteralToNonConstPointerConversionEPNS_4ExprENS_8QualTypeE", "clang::Sema::IsStringLiteralToNonConstPointerConversion(clang::Expr*, clang::QualType)"},
    {"_ZN5clang4Sema25PerformImplicitConversionERPNS_4ExprENS_8QualTypeERKNS_26ImplicitConversionSequenceENS0_16AssignmentActionEb", "clang::Sema::PerformImplicitConversion(clang::Expr*&, clang::QualType, clang::ImplicitConversionSequence const&, clang::Sema::AssignmentAction, bool)"},
    {"_ZN5clang4Sema25PerformImplicitConversionERPNS_4ExprENS_8QualTypeERKNS_26StandardConversionSequenceENS0_16AssignmentActionEb", "clang::Sema::PerformImplicitConversion(clang::Expr*&, clang::QualType, clang::StandardConversionSequence const&, clang::Sema::AssignmentAction, bool)"},
    {"_ZN5clang4Sema19ActOnUnaryTypeTraitENS_14UnaryTypeTraitENS_14SourceLocationENS_9OpaquePtrINS_8QualTypeEEES2_", "clang::Sema::ActOnUnaryTypeTrait(clang::UnaryTypeTrait, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::SourceLocation)"},
    {"_ZN5clang4Sema19BuildUnaryTypeTraitENS_14UnaryTypeTraitENS_14SourceLocationEPNS_14TypeSourceInfoES2_", "clang::Sema::BuildUnaryTypeTrait(clang::UnaryTypeTrait, clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema20ActOnBinaryTypeTraitENS_15BinaryTypeTraitENS_14SourceLocationENS_9OpaquePtrINS_8QualTypeEEES5_S2_", "clang::Sema::ActOnBinaryTypeTrait(clang::BinaryTypeTrait, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::OpaquePtr<clang::QualType>, clang::SourceLocation)"},
    {"_ZN5clang4Sema20BuildBinaryTypeTraitENS_15BinaryTypeTraitENS_14SourceLocationEPNS_14TypeSourceInfoES4_S2_", "clang::Sema::BuildBinaryTypeTrait(clang::BinaryTypeTrait, clang::SourceLocation, clang::TypeSourceInfo*, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema28CheckPointerToMemberOperandsERPNS_4ExprES3_RNS_13ExprValueKindENS_14SourceLocationEb", "clang::Sema::CheckPointerToMemberOperands(clang::Expr*&, clang::Expr*&, clang::ExprValueKind&, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema27CXXCheckConditionalOperandsERPNS_4ExprES3_S3_S3_RNS_13ExprValueKindERNS_14ExprObjectKindENS_14SourceLocationE", "clang::Sema::CXXCheckConditionalOperands(clang::Expr*&, clang::Expr*&, clang::Expr*&, clang::Expr*&, clang::ExprValueKind&, clang::ExprObjectKind&, clang::SourceLocation)"},
    {"_Z19TryClassUnificationRN5clang4SemaEPNS_4ExprES3_NS_14SourceLocationERbRNS_8QualTypeE", "TryClassUnification(clang::Sema&, clang::Expr*, clang::Expr*, clang::SourceLocation, bool&, clang::QualType&)"},
    {"_Z21ConvertForConditionalRN5clang4SemaERPNS_4ExprENS_8QualTypeE", "ConvertForConditional(clang::Sema&, clang::Expr*&, clang::QualType)"},
    {"_ZN5clang4Sema24FindCompositePointerTypeENS_14SourceLocationERPNS_4ExprES4_Pb", "clang::Sema::FindCompositePointerType(clang::SourceLocation, clang::Expr*&, clang::Expr*&, bool*)"},
    {"_ZN5clang4Sema20MaybeBindToTemporaryEPNS_4ExprE", "clang::Sema::MaybeBindToTemporary(clang::Expr*)"},
    {"_ZN5clang4Sema27MaybeCreateExprWithCleanupsEPNS_4ExprE", "clang::Sema::MaybeCreateExprWithCleanups(clang::Expr*)"},
    {"_ZN5clang4Sema27MaybeCreateExprWithCleanupsENS_12ActionResultIPNS_4ExprELb1EEE", "clang::Sema::MaybeCreateExprWithCleanups(clang::ActionResult<clang::Expr*, true>)"},
    {"_ZN5clang4Sema27MaybeCreateStmtWithCleanupsEPNS_4StmtE", "clang::Sema::MaybeCreateStmtWithCleanups(clang::Stmt*)"},
    {"_ZN5clang4Sema28ActOnStartCXXMemberReferenceEPNS_5ScopeEPNS_4ExprENS_14SourceLocationENS_3tok9TokenKindERNS_9OpaquePtrINS_8QualTypeEEERb", "clang::Sema::ActOnStartCXXMemberReference(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::tok::TokenKind, clang::OpaquePtr<clang::QualType>&, bool&)"},
    {"_ZN5clang4Sema21DiagnoseDtorReferenceENS_14SourceLocationEPNS_4ExprE", "clang::Sema::DiagnoseDtorReference(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema25BuildPseudoDestructorExprEPNS_4ExprENS_14SourceLocationENS_3tok9TokenKindERKNS_12CXXScopeSpecEPNS_14TypeSourceInfoES3_S3_NS_27PseudoDestructorTypeStorageEb", "clang::Sema::BuildPseudoDestructorExpr(clang::Expr*, clang::SourceLocation, clang::tok::TokenKind, clang::CXXScopeSpec const&, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation, clang::PseudoDestructorTypeStorage, bool)"},
    {"_ZN5clang4Sema25ActOnPseudoDestructorExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationENS_3tok9TokenKindERNS_12CXXScopeSpecERNS_13UnqualifiedIdES5_S5_SB_b", "clang::Sema::ActOnPseudoDestructorExpr(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::tok::TokenKind, clang::CXXScopeSpec&, clang::UnqualifiedId&, clang::SourceLocation, clang::SourceLocation, clang::UnqualifiedId&, bool)"},
    {"_ZN5clang4Sema22BuildCXXMemberCallExprEPNS_4ExprEPNS_9NamedDeclEPNS_13CXXMethodDeclE", "clang::Sema::BuildCXXMemberCallExpr(clang::Expr*, clang::NamedDecl*, clang::CXXMethodDecl*)"},
    {"_ZN5clang4Sema20BuildCXXNoexceptExprENS_14SourceLocationEPNS_4ExprES1_", "clang::Sema::BuildCXXNoexceptExpr(clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema17ActOnNoexceptExprENS_14SourceLocationES1_PNS_4ExprES1_", "clang::Sema::ActOnNoexceptExpr(clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema23IgnoredValueConversionsERPNS_4ExprE", "clang::Sema::IgnoredValueConversions(clang::Expr*&)"},
    {"_ZN5clang4Sema19ActOnFinishFullExprEPNS_4ExprE", "clang::Sema::ActOnFinishFullExpr(clang::Expr*)"},
    {"_ZN5clang4Sema19ActOnFinishFullStmtEPNS_4StmtE", "clang::Sema::ActOnFinishFullStmt(clang::Stmt*)"},
    {"_ZN5clang10Declarator19DropFirstTypeObjectEv", "clang::Declarator::DropFirstTypeObject()"},
    {"_ZN4llvm11SmallVectorIPN5clang4ExprELj8EEC2EjRKS3_", "llvm::SmallVector<clang::Expr*, 8u>::SmallVector(unsigned int, clang::Expr* const&)"},
    {"_ZN5clang19BinaryTypeTraitExprC2ENS_14SourceLocationENS_15BinaryTypeTraitEPNS_14TypeSourceInfoES4_bS1_NS_8QualTypeE", "clang::BinaryTypeTraitExpr::BinaryTypeTraitExpr(clang::SourceLocation, clang::BinaryTypeTrait, clang::TypeSourceInfo*, clang::TypeSourceInfo*, bool, clang::SourceLocation, clang::QualType)"},
    {"_ZNK5clang4Decl7getAttrINS_8UuidAttrEEEPT_v", "clang::UuidAttr* clang::Decl::getAttr<clang::UuidAttr>() const"},
    {"_ZN5clang4Sema22ParseObjCStringLiteralEPNS_14SourceLocationEPPNS_4ExprEj", "clang::Sema::ParseObjCStringLiteral(clang::SourceLocation*, clang::Expr**, unsigned int)"},
    {"_ZN5clang4Sema25BuildObjCEncodeExpressionENS_14SourceLocationEPNS_14TypeSourceInfoES1_", "clang::Sema::BuildObjCEncodeExpression(clang::SourceLocation, clang::TypeSourceInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema25ParseObjCEncodeExpressionENS_14SourceLocationES1_S1_NS_9OpaquePtrINS_8QualTypeEEES1_", "clang::Sema::ParseObjCEncodeExpression(clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::OpaquePtr<clang::QualType>, clang::SourceLocation)"},
    {"_ZN5clang4Sema27ParseObjCSelectorExpressionENS_8SelectorENS_14SourceLocationES2_S2_S2_", "clang::Sema::ParseObjCSelectorExpression(clang::Selector, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema27ParseObjCProtocolExpressionEPNS_14IdentifierInfoENS_14SourceLocationES3_S3_S3_", "clang::Sema::ParseObjCProtocolExpression(clang::IdentifierInfo*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema18tryCaptureObjCSelfEv", "clang::Sema::tryCaptureObjCSelf()"},
    {"_ZN5clang4Sema25CheckMessageArgumentTypesEPPNS_4ExprEjNS_8SelectorEPNS_14ObjCMethodDeclEbNS_14SourceLocationES7_RNS_8QualTypeERNS_13ExprValueKindE", "clang::Sema::CheckMessageArgumentTypes(clang::Expr**, unsigned int, clang::Selector, clang::ObjCMethodDecl*, bool, clang::SourceLocation, clang::SourceLocation, clang::QualType&, clang::ExprValueKind&)"},
    {"_ZN5clang4Sema10isSelfExprEPNS_4ExprE", "clang::Sema::isSelfExpr(clang::Expr*)"},
    {"_ZN5clang4Sema24LookupPrivateClassMethodENS_8SelectorEPNS_17ObjCInterfaceDeclE", "clang::Sema::LookupPrivateClassMethod(clang::Selector, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema27LookupPrivateInstanceMethodENS_8SelectorEPNS_17ObjCInterfaceDeclE", "clang::Sema::LookupPrivateInstanceMethod(clang::Selector, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema25HandleExprPropertyRefExprEPKNS_21ObjCObjectPointerTypeEPNS_4ExprENS_15DeclarationNameENS_14SourceLocationES7_NS_8QualTypeEb", "clang::Sema::HandleExprPropertyRefExpr(clang::ObjCObjectPointerType const*, clang::Expr*, clang::DeclarationName, clang::SourceLocation, clang::SourceLocation, clang::QualType, bool)"},
    {"_ZN5clang4Sema25ActOnClassPropertyRefExprERNS_14IdentifierInfoES2_NS_14SourceLocationES3_", "clang::Sema::ActOnClassPropertyRefExpr(clang::IdentifierInfo&, clang::IdentifierInfo&, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema18getObjCMessageKindEPNS_5ScopeEPNS_14IdentifierInfoENS_14SourceLocationEbbRNS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::getObjCMessageKind(clang::Scope*, clang::IdentifierInfo*, clang::SourceLocation, bool, bool, clang::OpaquePtr<clang::QualType>&)"},
    {"_ZN5clang4Sema17ActOnSuperMessageEPNS_5ScopeENS_14SourceLocationENS_8SelectorES3_S3_S3_NS_11ASTMultiPtrIPNS_4ExprEEE", "clang::Sema::ActOnSuperMessage(clang::Scope*, clang::SourceLocation, clang::Selector, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema20BuildInstanceMessageEPNS_4ExprENS_8QualTypeENS_14SourceLocationENS_8SelectorEPNS_14ObjCMethodDeclES4_S4_S4_NS_11ASTMultiPtrIS2_EE", "clang::Sema::BuildInstanceMessage(clang::Expr*, clang::QualType, clang::SourceLocation, clang::Selector, clang::ObjCMethodDecl*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema17BuildClassMessageEPNS_14TypeSourceInfoENS_8QualTypeENS_14SourceLocationENS_8SelectorEPNS_14ObjCMethodDeclES4_S4_S4_NS_11ASTMultiPtrIPNS_4ExprEEE", "clang::Sema::BuildClassMessage(clang::TypeSourceInfo*, clang::QualType, clang::SourceLocation, clang::Selector, clang::ObjCMethodDecl*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema17ActOnClassMessageEPNS_5ScopeENS_9OpaquePtrINS_8QualTypeEEENS_8SelectorENS_14SourceLocationES7_S7_NS_11ASTMultiPtrIPNS_4ExprEEE", "clang::Sema::ActOnClassMessage(clang::Scope*, clang::OpaquePtr<clang::QualType>, clang::Selector, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema20ActOnInstanceMessageEPNS_5ScopeEPNS_4ExprENS_8SelectorENS_14SourceLocationES6_S6_NS_11ASTMultiPtrIS4_EE", "clang::Sema::ActOnInstanceMessage(clang::Scope*, clang::Expr*, clang::Selector, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Expr*>)"},
    {"_ZN5clang4Sema26ActOnDesignatedInitializerERNS_11DesignationENS_14SourceLocationEbNS_12ActionResultIPNS_4ExprELb1EEE", "clang::Sema::ActOnDesignatedInitializer(clang::Designation&, clang::SourceLocation, bool, clang::ActionResult<clang::Expr*, true>)"},
    {"_Z24CheckArrayDesignatorExprRN5clang4SemaEPNS_4ExprERN4llvm6APSIntE", "CheckArrayDesignatorExpr(clang::Sema&, clang::Expr*, llvm::APSInt&)"},
    {"_ZN5clang4Sema13CheckInitListERKNS_17InitializedEntityERPNS_12InitListExprERNS_8QualTypeE", "clang::Sema::CheckInitList(clang::InitializedEntity const&, clang::InitListExpr*&, clang::QualType&)"},
    {"_ZN12_GLOBAL__N_115InitListCheckerC1ERN5clang4SemaERKNS1_17InitializedEntityEPNS1_12InitListExprERNS1_8QualTypeE", "(anonymous namespace)::InitListChecker::InitListChecker(clang::Sema&, clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType&)"},
    {"_ZN5clang17InitializedEntityC1ERNS_10ASTContextEjRKS0_", "clang::InitializedEntity::InitializedEntity(clang::ASTContext&, unsigned int, clang::InitializedEntity const&)"},
    {"_ZN5clang17InitializedEntityC2ERNS_10ASTContextEjRKS0_", "clang::InitializedEntity::InitializedEntity(clang::ASTContext&, unsigned int, clang::InitializedEntity const&)"},
    {"_ZN5clang17InitializedEntity14InitializeBaseERNS_10ASTContextEPNS_16CXXBaseSpecifierEb", "clang::InitializedEntity::InitializeBase(clang::ASTContext&, clang::CXXBaseSpecifier*, bool)"},
    {"_ZNK5clang17InitializedEntity7getNameEv", "clang::InitializedEntity::getName() const"},
    {"_ZNK5clang17InitializedEntity7getDeclEv", "clang::InitializedEntity::getDecl() const"},
    {"_ZNK5clang17InitializedEntity10allowsNRVOEv", "clang::InitializedEntity::allowsNRVO() const"},
    {"_ZN5clang22InitializationSequence4Step7DestroyEv", "clang::InitializationSequence::Step::Destroy()"},
    {"_ZNK5clang22InitializationSequence24isDirectReferenceBindingEv", "clang::InitializationSequence::isDirectReferenceBinding() const"},
    {"_ZNK5clang22InitializationSequence11isAmbiguousEv", "clang::InitializationSequence::isAmbiguous() const"},
    {"_ZNK5clang22InitializationSequence27isConstructorInitializationEv", "clang::InitializationSequence::isConstructorInitialization() const"},
    {"_ZN5clang22InitializationSequence32AddAddressOverloadResolutionStepEPNS_12FunctionDeclENS_14DeclAccessPairE", "clang::InitializationSequence::AddAddressOverloadResolutionStep(clang::FunctionDecl*, clang::DeclAccessPair)"},
    {"_ZN5clang22InitializationSequence24AddDerivedToBaseCastStepENS_8QualTypeENS_13ExprValueKindE", "clang::InitializationSequence::AddDerivedToBaseCastStep(clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang22InitializationSequence23AddReferenceBindingStepENS_8QualTypeEb", "clang::InitializationSequence::AddReferenceBindingStep(clang::QualType, bool)"},
    {"_ZN5clang22InitializationSequence28AddExtraneousCopyToTemporaryENS_8QualTypeE", "clang::InitializationSequence::AddExtraneousCopyToTemporary(clang::QualType)"},
    {"_ZN5clang22InitializationSequence21AddUserConversionStepEPNS_12FunctionDeclENS_14DeclAccessPairENS_8QualTypeE", "clang::InitializationSequence::AddUserConversionStep(clang::FunctionDecl*, clang::DeclAccessPair, clang::QualType)"},
    {"_ZN5clang22InitializationSequence30AddQualificationConversionStepENS_8QualTypeENS_13ExprValueKindE", "clang::InitializationSequence::AddQualificationConversionStep(clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang22InitializationSequence25AddConversionSequenceStepERKNS_26ImplicitConversionSequenceENS_8QualTypeE", "clang::InitializationSequence::AddConversionSequenceStep(clang::ImplicitConversionSequence const&, clang::QualType)"},
    {"_ZN5clang22InitializationSequence25AddListInitializationStepENS_8QualTypeE", "clang::InitializationSequence::AddListInitializationStep(clang::QualType)"},
    {"_ZN5clang22InitializationSequence32AddConstructorInitializationStepEPNS_18CXXConstructorDeclENS_15AccessSpecifierENS_8QualTypeE", "clang::InitializationSequence::AddConstructorInitializationStep(clang::CXXConstructorDecl*, clang::AccessSpecifier, clang::QualType)"},
    {"_ZN5clang22InitializationSequence25AddZeroInitializationStepENS_8QualTypeE", "clang::InitializationSequence::AddZeroInitializationStep(clang::QualType)"},
    {"_ZN5clang22InitializationSequence18AddCAssignmentStepENS_8QualTypeE", "clang::InitializationSequence::AddCAssignmentStep(clang::QualType)"},
    {"_ZN5clang22InitializationSequence17AddStringInitStepENS_8QualTypeE", "clang::InitializationSequence::AddStringInitStep(clang::QualType)"},
    {"_ZN5clang22InitializationSequence27AddObjCObjectConversionStepENS_8QualTypeE", "clang::InitializationSequence::AddObjCObjectConversionStep(clang::QualType)"},
    {"_ZN5clang22InitializationSequence18SetOverloadFailureENS0_11FailureKindENS_17OverloadingResultE", "clang::InitializationSequence::SetOverloadFailure(clang::InitializationSequence::FailureKind, clang::OverloadingResult)"},
    {"_ZN5clang22InitializationSequenceC1ERNS_4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPPNS_4ExprEj", "clang::InitializationSequence::InitializationSequence(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr**, unsigned int)"},
    {"_ZN5clang22InitializationSequenceC2ERNS_4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPPNS_4ExprEj", "clang::InitializationSequence::InitializationSequence(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr**, unsigned int)"},
    {"_Z12IsStringInitPN5clang4ExprENS_8QualTypeERNS_10ASTContextE", "IsStringInit(clang::Expr*, clang::QualType, clang::ASTContext&)"},
    {"_Z28TryConstructorInitializationRN5clang4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPPNS_4ExprEjNS_8QualTypeERNS_22InitializationSequenceE", "TryConstructorInitialization(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr**, unsigned int, clang::QualType, clang::InitializationSequence&)"},
    {"_Z24TryUserDefinedConversionRN5clang4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPNS_4ExprERNS_22InitializationSequenceE", "TryUserDefinedConversion(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr*, clang::InitializationSequence&)"},
    {"_ZN5clang22InitializationSequenceD1Ev", "clang::InitializationSequence::~InitializationSequence()"},
    {"_ZN5clang22InitializationSequenceD2Ev", "clang::InitializationSequence::~InitializationSequence()"},
    {"_ZN5clang22InitializationSequence21PrintInitLocationNoteERNS_4SemaERKNS_17InitializedEntityE", "clang::InitializationSequence::PrintInitLocationNote(clang::Sema&, clang::InitializedEntity const&)"},
    {"_ZN5clang22InitializationSequence7PerformERNS_4SemaERKNS_17InitializedEntityERKNS_18InitializationKindENS_11ASTMultiPtrIPNS_4ExprEEEPNS_8QualTypeE", "clang::InitializationSequence::Perform(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::ASTMultiPtr<clang::Expr*>, clang::QualType*)"},
    {"_ZN5clang22InitializationSequence8DiagnoseERNS_4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPPNS_4ExprEj", "clang::InitializationSequence::Diagnose(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr**, unsigned int)"},
    {"_Z10CopyObjectRN5clang4SemaENS_8QualTypeERKNS_17InitializedEntityENS_12ActionResultIPNS_4ExprELb1EEEb", "CopyObject(clang::Sema&, clang::QualType, clang::InitializedEntity const&, clang::ActionResult<clang::Expr*, true>, bool)"},
    {"_Z15CheckStringInitPN5clang4ExprERNS_8QualTypeERNS_4SemaE", "CheckStringInit(clang::Expr*, clang::QualType&, clang::Sema&)"},
    {"_ZNK5clang22InitializationSequence4dumpERN4llvm11raw_ostreamE", "clang::InitializationSequence::dump(llvm::raw_ostream&) const"},
    {"_ZNK5clang22InitializationSequence4dumpEv", "clang::InitializationSequence::dump() const"},
    {"_ZN5clang4Sema25PerformCopyInitializationERKNS_17InitializedEntityENS_14SourceLocationENS_12ActionResultIPNS_4ExprELb1EEE", "clang::Sema::PerformCopyInitialization(clang::InitializedEntity const&, clang::SourceLocation, clang::ActionResult<clang::Expr*, true>)"},
    {"_Z32TryRefInitWithConversionFunctionRN5clang4SemaERKNS_17InitializedEntityERKNS_18InitializationKindEPNS_4ExprEbRNS_22InitializationSequenceE", "TryRefInitWithConversionFunction(clang::Sema&, clang::InitializedEntity const&, clang::InitializationKind const&, clang::Expr*, bool, clang::InitializationSequence&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker26getStructuredSubobjectInitEPN5clang12InitListExprEjNS1_8QualTypeES3_jNS1_11SourceRangeE", "(anonymous namespace)::InitListChecker::getStructuredSubobjectInit(clang::InitListExpr*, unsigned int, clang::QualType, clang::InitListExpr*, unsigned int, clang::SourceRange)"},
    {"_ZN12_GLOBAL__N_115InitListChecker21CheckExplicitInitListERKN5clang17InitializedEntityEPNS1_12InitListExprERNS1_8QualTypeERjS6_S9_b", "(anonymous namespace)::InitListChecker::CheckExplicitInitList(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType&, unsigned int&, clang::InitListExpr*, unsigned int&, bool)"},
    {"_ZN12_GLOBAL__N_115InitListChecker26FillInValueInitializationsERKN5clang17InitializedEntityEPNS1_12InitListExprERb", "(anonymous namespace)::InitListChecker::FillInValueInitializations(clang::InitializedEntity const&, clang::InitListExpr*, bool&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker23FillInValueInitForFieldEjPN5clang9FieldDeclERKNS1_17InitializedEntityEPNS1_12InitListExprERb", "(anonymous namespace)::InitListChecker::FillInValueInitForField(unsigned int, clang::FieldDecl*, clang::InitializedEntity const&, clang::InitListExpr*, bool&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker21CheckListElementTypesERKN5clang17InitializedEntityEPNS1_12InitListExprERNS1_8QualTypeEbRjS6_S9_b", "(anonymous namespace)::InitListChecker::CheckListElementTypes(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType&, bool, unsigned int&, clang::InitListExpr*, unsigned int&, bool)"},
    {"_ZN12_GLOBAL__N_115InitListChecker15CheckScalarTypeERKN5clang17InitializedEntityEPNS1_12InitListExprENS1_8QualTypeERjS6_S8_", "(anonymous namespace)::InitListChecker::CheckScalarType(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType, unsigned int&, clang::InitListExpr*, unsigned int&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker21CheckStructUnionTypesERKN5clang17InitializedEntityEPNS1_12InitListExprENS1_8QualTypeENS1_11DeclContext22specific_decl_iteratorINS1_9FieldDeclEEEbRjS6_SC_b", "(anonymous namespace)::InitListChecker::CheckStructUnionTypes(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType, clang::DeclContext::specific_decl_iterator<clang::FieldDecl>, bool, unsigned int&, clang::InitListExpr*, unsigned int&, bool)"},
    {"_ZN12_GLOBAL__N_115InitListChecker14CheckArrayTypeERKN5clang17InitializedEntityEPNS1_12InitListExprERNS1_8QualTypeEN4llvm6APSIntEbRjS6_SB_", "(anonymous namespace)::InitListChecker::CheckArrayType(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType&, llvm::APSInt, bool, unsigned int&, clang::InitListExpr*, unsigned int&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker18CheckReferenceTypeERKN5clang17InitializedEntityEPNS1_12InitListExprENS1_8QualTypeERjS6_S8_", "(anonymous namespace)::InitListChecker::CheckReferenceType(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType, unsigned int&, clang::InitListExpr*, unsigned int&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker27UpdateStructuredListElementEPN5clang12InitListExprERjPNS1_4ExprE", "(anonymous namespace)::InitListChecker::UpdateStructuredListElement(clang::InitListExpr*, unsigned int&, clang::Expr*)"},
    {"_ZN12_GLOBAL__N_115InitListChecker26CheckDesignatedInitializerERKN5clang17InitializedEntityEPNS1_12InitListExprEPNS1_18DesignatedInitExprEjRNS1_8QualTypeEPNS1_11DeclContext22specific_decl_iteratorINS1_9FieldDeclEEEPN4llvm6APSIntERjS6_SJ_bb", "(anonymous namespace)::InitListChecker::CheckDesignatedInitializer(clang::InitializedEntity const&, clang::InitListExpr*, clang::DesignatedInitExpr*, unsigned int, clang::QualType&, clang::DeclContext::specific_decl_iterator<clang::FieldDecl>*, llvm::APSInt*, unsigned int&, clang::InitListExpr*, unsigned int&, bool, bool)"},
    {"_ZN12_GLOBAL__N_115InitListChecker19CheckSubElementTypeERKN5clang17InitializedEntityEPNS1_12InitListExprENS1_8QualTypeERjS6_S8_", "(anonymous namespace)::InitListChecker::CheckSubElementType(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType, unsigned int&, clang::InitListExpr*, unsigned int&)"},
    {"_ZN12_GLOBAL__N_115InitListChecker21CheckImplicitInitListERKN5clang17InitializedEntityEPNS1_12InitListExprENS1_8QualTypeERjS6_S8_b", "(anonymous namespace)::InitListChecker::CheckImplicitInitList(clang::InitializedEntity const&, clang::InitListExpr*, clang::QualType, unsigned int&, clang::InitListExpr*, unsigned int&, bool)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang22InitializationSequence4StepELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::InitializationSequence::Step, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIPN5clang12InitListExprESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::InitListExpr*, std::pair<clang::InitListExpr* const, clang::InitListExpr*>, std::_Select1st<std::pair<clang::InitListExpr* const, clang::InitListExpr*> >, std::less<clang::InitListExpr*>, std::allocator<std::pair<clang::InitListExpr* const, clang::InitListExpr*> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::InitListExpr* const, clang::InitListExpr*> >*)"},
    {"_ZNSt8_Rb_treeIPN5clang12InitListExprESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::InitListExpr*, std::pair<clang::InitListExpr* const, clang::InitListExpr*>, std::_Select1st<std::pair<clang::InitListExpr* const, clang::InitListExpr*> >, std::less<clang::InitListExpr*>, std::allocator<std::pair<clang::InitListExpr* const, clang::InitListExpr*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::InitListExpr* const, clang::InitListExpr*> >, std::pair<clang::InitListExpr* const, clang::InitListExpr*> const&)"},
    {"_ZNSt8_Rb_treeIPN5clang12InitListExprESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::InitListExpr*, std::pair<clang::InitListExpr* const, clang::InitListExpr*>, std::_Select1st<std::pair<clang::InitListExpr* const, clang::InitListExpr*> >, std::less<clang::InitListExpr*>, std::allocator<std::pair<clang::InitListExpr* const, clang::InitListExpr*> > >::_M_insert_unique(std::pair<clang::InitListExpr* const, clang::InitListExpr*> const&)"},
    {"_ZN5clang12LookupResult9configureEv", "clang::LookupResult::configure()"},
    {"_ZNK5clang12LookupResult6sanityEv", "clang::LookupResult::sanity() const"},
    {"_ZN5clang12LookupResult11deletePathsEPNS_12CXXBasePathsE", "clang::LookupResult::deletePaths(clang::CXXBasePaths*)"},
    {"_ZN5clang12LookupResult11resolveKindEv", "clang::LookupResult::resolveKind()"},
    {"_ZN5clang12LookupResult21addDeclsFromBasePathsERKNS_12CXXBasePathsE", "clang::LookupResult::addDeclsFromBasePaths(clang::CXXBasePaths const&)"},
    {"_ZN5clang12LookupResult26setAmbiguousBaseSubobjectsERNS_12CXXBasePathsE", "clang::LookupResult::setAmbiguousBaseSubobjects(clang::CXXBasePaths&)"},
    {"_ZN5clang12LookupResult30setAmbiguousBaseSubobjectTypesERNS_12CXXBasePathsE", "clang::LookupResult::setAmbiguousBaseSubobjectTypes(clang::CXXBasePaths&)"},
    {"_ZN5clang12LookupResult5printERN4llvm11raw_ostreamE", "clang::LookupResult::print(llvm::raw_ostream&)"},
    {"_ZN5clang4Sema33ForceDeclarationOfImplicitMembersEPNS_13CXXRecordDeclE", "clang::Sema::ForceDeclarationOfImplicitMembers(clang::CXXRecordDecl*)"},
    {"_Z31CanDeclareSpecialMemberFunctionRN5clang10ASTContextEPKNS_13CXXRecordDeclE", "CanDeclareSpecialMemberFunction(clang::ASTContext&, clang::CXXRecordDecl const*)"},
    {"_ZN5clang4Sema13CppLookupNameERNS_12LookupResultEPNS_5ScopeE", "clang::Sema::CppLookupName(clang::LookupResult&, clang::Scope*)"},
    {"_Z38DeclareImplicitMemberFunctionsWithNameRN5clang4SemaENS_15DeclarationNameEPKNS_11DeclContextE", "DeclareImplicitMemberFunctionsWithName(clang::Sema&, clang::DeclarationName, clang::DeclContext const*)"},
    {"_Z16findOuterContextPN5clang5ScopeE", "findOuterContext(clang::Scope*)"},
    {"_ZN5clang4Sema19LookupQualifiedNameERNS_12LookupResultEPNS_11DeclContextEb", "clang::Sema::LookupQualifiedName(clang::LookupResult&, clang::DeclContext*, bool)"},
    {"_ZN12_GLOBAL__N_123UnqualUsingDirectiveSet15visitScopeChainEPN5clang5ScopeES3_", "(anonymous namespace)::UnqualUsingDirectiveSet::visitScopeChain(clang::Scope*, clang::Scope*)"},
    {"_ZN12_GLOBAL__N_123UnqualUsingDirectiveSet4doneEv", "(anonymous namespace)::UnqualUsingDirectiveSet::done()"},
    {"_ZN5clang4Sema10LookupNameERNS_12LookupResultEPNS_5ScopeEb", "clang::Sema::LookupName(clang::LookupResult&, clang::Scope*, bool)"},
    {"_Z13LookupBuiltinRN5clang4SemaERNS_12LookupResultE", "LookupBuiltin(clang::Sema&, clang::LookupResult&)"},
    {"_Z12LookupDirectRN5clang4SemaERNS_12LookupResultEPKNS_11DeclContextE", "LookupDirect(clang::Sema&, clang::LookupResult&, clang::DeclContext const*)"},
    {"_Z15LookupAnyMemberPKN5clang16CXXBaseSpecifierERNS_11CXXBasePathEPv", "LookupAnyMember(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang4Sema16LookupParsedNameERNS_12LookupResultEPNS_5ScopeEPNS_12CXXScopeSpecEbb", "clang::Sema::LookupParsedName(clang::LookupResult&, clang::Scope*, clang::CXXScopeSpec*, bool, bool)"},
    {"_ZN5clang4Sema23DiagnoseAmbiguousLookupERNS_12LookupResultE", "clang::Sema::DiagnoseAmbiguousLookup(clang::LookupResult&)"},
    {"_ZN5clang4Sema34FindAssociatedClassesAndNamespacesEPPNS_4ExprEjRN4llvm11SmallPtrSetIPNS_11DeclContextELj16EEERNS5_IPNS_13CXXRecordDeclELj16EEE", "clang::Sema::FindAssociatedClassesAndNamespaces(clang::Expr**, unsigned int, llvm::SmallPtrSet<clang::DeclContext*, 16u>&, llvm::SmallPtrSet<clang::CXXRecordDecl*, 16u>&)"},
    {"_Z33addAssociatedClassesAndNamespacesRN12_GLOBAL__N_116AssociatedLookupEN5clang8QualTypeE", "addAssociatedClassesAndNamespaces((anonymous namespace)::AssociatedLookup&, clang::QualType)"},
    {"_ZN5clang4Sema16LookupSingleNameEPNS_5ScopeENS_15DeclarationNameENS_14SourceLocationENS0_14LookupNameKindENS0_17RedeclarationKindE", "clang::Sema::LookupSingleName(clang::Scope*, clang::DeclarationName, clang::SourceLocation, clang::Sema::LookupNameKind, clang::Sema::RedeclarationKind)"},
    {"_ZN5clang4Sema14LookupProtocolEPNS_14IdentifierInfoENS_14SourceLocationE", "clang::Sema::LookupProtocol(clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema28LookupOverloadedOperatorNameENS_22OverloadedOperatorKindEPNS_5ScopeENS_8QualTypeES4_RNS_17UnresolvedSetImplE", "clang::Sema::LookupOverloadedOperatorName(clang::OverloadedOperatorKind, clang::Scope*, clang::QualType, clang::QualType, clang::UnresolvedSetImpl&)"},
    {"_Z38IsAcceptableNonMemberOperatorCandidatePN5clang12FunctionDeclENS_8QualTypeES2_RNS_10ASTContextE", "IsAcceptableNonMemberOperatorCandidate(clang::FunctionDecl*, clang::QualType, clang::QualType, clang::ASTContext&)"},
    {"_ZN5clang4Sema18LookupConstructorsEPNS_13CXXRecordDeclE", "clang::Sema::LookupConstructors(clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema16LookupDestructorEPNS_13CXXRecordDeclE", "clang::Sema::LookupDestructor(clang::CXXRecordDecl*)"},
    {"_ZN5clang9ADLResult6insertEPNS_9NamedDeclE", "clang::ADLResult::insert(clang::NamedDecl*)"},
    {"_ZN5clang4Sema23ArgumentDependentLookupENS_15DeclarationNameEbPPNS_4ExprEjRNS_9ADLResultE", "clang::Sema::ArgumentDependentLookup(clang::DeclarationName, bool, clang::Expr**, unsigned int, clang::ADLResult&)"},
    {"_ZN5clang19VisibleDeclConsumerD0Ev", "clang::VisibleDeclConsumer::~VisibleDeclConsumer()"},
    {"_ZN5clang19VisibleDeclConsumerD1Ev", "clang::VisibleDeclConsumer::~VisibleDeclConsumer()"},
    {"_ZN5clang19VisibleDeclConsumerD2Ev", "clang::VisibleDeclConsumer::~VisibleDeclConsumer()"},
    {"_ZN5clang4Sema18LookupVisibleDeclsEPNS_5ScopeENS0_14LookupNameKindERNS_19VisibleDeclConsumerEb", "clang::Sema::LookupVisibleDecls(clang::Scope*, clang::Sema::LookupNameKind, clang::VisibleDeclConsumer&, bool)"},
    {"_ZN12_GLOBAL__N_117ShadowContextRAIIC1ERNS_18VisibleDeclsRecordE", "(anonymous namespace)::ShadowContextRAII::ShadowContextRAII((anonymous namespace)::VisibleDeclsRecord&)"},
    {"_Z18LookupVisibleDeclsPN5clang5ScopeERNS_12LookupResultERN12_GLOBAL__N_123UnqualUsingDirectiveSetERNS_19VisibleDeclConsumerERNS4_18VisibleDeclsRecordE", "LookupVisibleDecls(clang::Scope*, clang::LookupResult&, (anonymous namespace)::UnqualUsingDirectiveSet&, clang::VisibleDeclConsumer&, (anonymous namespace)::VisibleDeclsRecord&)"},
    {"_ZN12_GLOBAL__N_117ShadowContextRAIID1Ev", "(anonymous namespace)::ShadowContextRAII::~ShadowContextRAII()"},
    {"_ZN5clang4Sema18LookupVisibleDeclsEPNS_11DeclContextENS0_14LookupNameKindERNS_19VisibleDeclConsumerEb", "clang::Sema::LookupVisibleDecls(clang::DeclContext*, clang::Sema::LookupNameKind, clang::VisibleDeclConsumer&, bool)"},
    {"_Z18LookupVisibleDeclsPN5clang11DeclContextERNS_12LookupResultEbbRNS_19VisibleDeclConsumerERN12_GLOBAL__N_118VisibleDeclsRecordE", "LookupVisibleDecls(clang::DeclContext*, clang::LookupResult&, bool, bool, clang::VisibleDeclConsumer&, (anonymous namespace)::VisibleDeclsRecord&)"},
    {"_ZN5clang4Sema11CorrectTypoERNS_12LookupResultEPNS_5ScopeEPNS_12CXXScopeSpecEPNS_11DeclContextEbNS0_18CorrectTypoContextEPKNS_21ObjCObjectPointerTypeE", "clang::Sema::CorrectTypo(clang::LookupResult&, clang::Scope*, clang::CXXScopeSpec*, clang::DeclContext*, bool, clang::Sema::CorrectTypoContext, clang::ObjCObjectPointerType const*)"},
    {"_ZN12_GLOBAL__N_122TypoCorrectionConsumer9FoundNameEN4llvm9StringRefE", "(anonymous namespace)::TypoCorrectionConsumer::FoundName(llvm::StringRef)"},
    {"_Z25LookupPotentialTypoResultRN5clang4SemaERNS_12LookupResultEPNS_14IdentifierInfoEPNS_5ScopeEPNS_12CXXScopeSpecEPNS_11DeclContextEbNS0_18CorrectTypoContextE", "LookupPotentialTypoResult(clang::Sema&, clang::LookupResult&, clang::IdentifierInfo*, clang::Scope*, clang::CXXScopeSpec*, clang::DeclContext*, bool, clang::Sema::CorrectTypoContext)"},
    {"_ZN12_GLOBAL__N_122TypoCorrectionConsumerD1Ev", "(anonymous namespace)::TypoCorrectionConsumer::~TypoCorrectionConsumer()"},
    {"_ZN12_GLOBAL__N_122TypoCorrectionConsumerD0Ev", "(anonymous namespace)::TypoCorrectionConsumer::~TypoCorrectionConsumer()"},
    {"_ZN12_GLOBAL__N_122TypoCorrectionConsumer9FoundDeclEPN5clang9NamedDeclES3_b", "(anonymous namespace)::TypoCorrectionConsumer::FoundDecl(clang::NamedDecl*, clang::NamedDecl*, bool)"},
    {"_ZN12_GLOBAL__N_118VisibleDeclsRecord11checkHiddenEPN5clang9NamedDeclE", "(anonymous namespace)::VisibleDeclsRecord::checkHidden(clang::NamedDecl*)"},
    {"_ZN12_GLOBAL__N_118VisibleDeclsRecord3addEPN5clang9NamedDeclE", "(anonymous namespace)::VisibleDeclsRecord::add(clang::NamedDecl*)"},
    {"_ZNK12_GLOBAL__N_123UnqualUsingDirectiveSet16getNamespacesForEPN5clang11DeclContextE", "(anonymous namespace)::UnqualUsingDirectiveSet::getNamespacesFor(clang::DeclContext*) const"},
    {"_Z25CollectEnclosingNamespaceRN4llvm11SmallPtrSetIPN5clang11DeclContextELj16EEES3_", "CollectEnclosingNamespace(llvm::SmallPtrSet<clang::DeclContext*, 16u>&, clang::DeclContext*)"},
    {"_Z33addAssociatedClassesAndNamespacesRN12_GLOBAL__N_116AssociatedLookupERKN5clang16TemplateArgumentE", "addAssociatedClassesAndNamespaces((anonymous namespace)::AssociatedLookup&, clang::TemplateArgument const&)"},
    {"_ZSt16__introsort_loopIPN12_GLOBAL__N_116UnqualUsingEntryElNS1_10ComparatorEEvT_S4_T0_T1_", "void std::__introsort_loop<(anonymous namespace)::UnqualUsingEntry*, long, (anonymous namespace)::UnqualUsingEntry::Comparator>((anonymous namespace)::UnqualUsingEntry*, (anonymous namespace)::UnqualUsingEntry*, long, (anonymous namespace)::UnqualUsingEntry::Comparator)"},
    {"_ZSt16__insertion_sortIPN12_GLOBAL__N_116UnqualUsingEntryENS1_10ComparatorEEvT_S4_T0_", "void std::__insertion_sort<(anonymous namespace)::UnqualUsingEntry*, (anonymous namespace)::UnqualUsingEntry::Comparator>((anonymous namespace)::UnqualUsingEntry*, (anonymous namespace)::UnqualUsingEntry*, (anonymous namespace)::UnqualUsingEntry::Comparator)"},
    {"_ZN12_GLOBAL__N_123UnqualUsingDirectiveSet17addUsingDirectiveEPN5clang18UsingDirectiveDeclEPNS1_11DeclContextE", "(anonymous namespace)::UnqualUsingDirectiveSet::addUsingDirective(clang::UsingDirectiveDecl*, clang::DeclContext*)"},
    {"_ZN12_GLOBAL__N_123UnqualUsingDirectiveSet18addUsingDirectivesEPN5clang11DeclContextES3_", "(anonymous namespace)::UnqualUsingDirectiveSet::addUsingDirectives(clang::DeclContext*, clang::DeclContext*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoESt4pairINS_9StringRefEbENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PS4_IS3_S6_E", "llvm::DenseMap<clang::IdentifierInfo*, std::pair<llvm::StringRef, bool>, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<std::pair<llvm::StringRef, bool> > >::InsertIntoBucket(clang::IdentifierInfo* const&, std::pair<llvm::StringRef, bool> const&, std::pair<clang::IdentifierInfo*, std::pair<llvm::StringRef, bool> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoESt4pairINS_9StringRefEbENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, std::pair<llvm::StringRef, bool>, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<std::pair<llvm::StringRef, bool> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang9NamedDeclES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<clang::NamedDecl*, clang::NamedDecl*, llvm::DenseMapInfo<clang::NamedDecl*>, llvm::DenseMapInfo<clang::NamedDecl*> >::InsertIntoBucket(clang::NamedDecl* const&, clang::NamedDecl* const&, std::pair<clang::NamedDecl*, clang::NamedDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang9NamedDeclES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<clang::NamedDecl*, clang::NamedDecl*, llvm::DenseMapInfo<clang::NamedDecl*>, llvm::DenseMapInfo<clang::NamedDecl*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIPN5clang4DeclES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<clang::Decl*, clang::Decl*, std::_Identity<clang::Decl*>, std::less<clang::Decl*>, std::allocator<clang::Decl*> >::_M_insert_unique(clang::Decl* const&)"},
    {"_ZN4llvm9StringMapIbNS_16BumpPtrAllocatorEE16GetOrCreateValueIbEERNS_14StringMapEntryIbEENS_9StringRefET_", "llvm::StringMapEntry<bool>& llvm::StringMap<bool, llvm::BumpPtrAllocator>::GetOrCreateValue<bool>(llvm::StringRef, bool)"},
    {"_ZNSt8_Rb_treeIPN5clang4DeclES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<clang::Decl*, clang::Decl*, std::_Identity<clang::Decl*>, std::less<clang::Decl*>, std::allocator<clang::Decl*> >::_M_erase(std::_Rb_tree_node<clang::Decl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang11DeclContextEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<clang::DeclContext*, char, llvm::DenseMapInfo<clang::DeclContext*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::DeclContext*, char> const&)"},
    {"_ZN4llvm8DenseMapIPN5clang11DeclContextEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<clang::DeclContext*, char, llvm::DenseMapInfo<clang::DeclContext*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::DeclContext* const&, char const&, std::pair<clang::DeclContext*, char>*)"},
    {"_ZN4llvm8DenseMapIPN5clang11DeclContextEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<clang::DeclContext*, char, llvm::DenseMapInfo<clang::DeclContext*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang4Sema13ActOnPropertyEPNS_5ScopeENS_14SourceLocationERNS_15FieldDeclaratorERNS_12ObjCDeclSpecENS_8SelectorES8_PNS_4DeclEPbNS_3tok15ObjCKeywordKindEPNS_11DeclContextE", "clang::Sema::ActOnProperty(clang::Scope*, clang::SourceLocation, clang::FieldDeclarator&, clang::ObjCDeclSpec&, clang::Selector, clang::Selector, clang::Decl*, bool*, clang::tok::ObjCKeywordKind, clang::DeclContext*)"},
    {"_ZN5clang4Sema30HandlePropertyInClassExtensionEPNS_5ScopeEPNS_16ObjCCategoryDeclENS_14SourceLocationERNS_15FieldDeclaratorENS_8SelectorES8_bbjPbPNS_14TypeSourceInfoENS_3tok15ObjCKeywordKindE", "clang::Sema::HandlePropertyInClassExtension(clang::Scope*, clang::ObjCCategoryDecl*, clang::SourceLocation, clang::FieldDeclarator&, clang::Selector, clang::Selector, bool, bool, unsigned int, bool*, clang::TypeSourceInfo*, clang::tok::ObjCKeywordKind)"},
    {"_ZN5clang4Sema27CheckObjCPropertyAttributesEPNS_4DeclENS_14SourceLocationERj", "clang::Sema::CheckObjCPropertyAttributes(clang::Decl*, clang::SourceLocation, unsigned int&)"},
    {"_ZN5clang4Sema18CreatePropertyDeclEPNS_5ScopeEPNS_17ObjCContainerDeclENS_14SourceLocationERNS_15FieldDeclaratorENS_8SelectorES8_bbjPNS_14TypeSourceInfoENS_3tok15ObjCKeywordKindEPNS_11DeclContextE", "clang::Sema::CreatePropertyDecl(clang::Scope*, clang::ObjCContainerDecl*, clang::SourceLocation, clang::FieldDeclarator&, clang::Selector, clang::Selector, bool, bool, unsigned int, clang::TypeSourceInfo*, clang::tok::ObjCKeywordKind, clang::DeclContext*)"},
    {"_ZN5clang4Sema19ProcessPropertyDeclEPNS_16ObjCPropertyDeclEPNS_17ObjCContainerDeclES2_S4_", "clang::Sema::ProcessPropertyDecl(clang::ObjCPropertyDecl*, clang::ObjCContainerDecl*, clang::ObjCPropertyDecl*, clang::ObjCContainerDecl*)"},
    {"_ZN5clang4Sema21ActOnPropertyImplDeclEPNS_5ScopeENS_14SourceLocationES3_bPNS_4DeclEPNS_14IdentifierInfoES7_S3_", "clang::Sema::ActOnPropertyImplDecl(clang::Scope*, clang::SourceLocation, clang::SourceLocation, bool, clang::Decl*, clang::IdentifierInfo*, clang::IdentifierInfo*, clang::SourceLocation)"},
    {"_ZN5clang4Sema24DiagnosePropertyMismatchEPNS_16ObjCPropertyDeclES2_PKNS_14IdentifierInfoE", "clang::Sema::DiagnosePropertyMismatch(clang::ObjCPropertyDecl*, clang::ObjCPropertyDecl*, clang::IdentifierInfo const*)"},
    {"_ZN5clang4Sema32DiagnosePropertyAccessorMismatchEPNS_16ObjCPropertyDeclEPNS_14ObjCMethodDeclENS_14SourceLocationE", "clang::Sema::DiagnosePropertyAccessorMismatch(clang::ObjCPropertyDecl*, clang::ObjCMethodDecl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema31ComparePropertiesInBaseAndSuperEPNS_17ObjCInterfaceDeclE", "clang::Sema::ComparePropertiesInBaseAndSuper(clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema33MatchOneProtocolPropertiesInClassEPNS_4DeclEPNS_16ObjCProtocolDeclE", "clang::Sema::MatchOneProtocolPropertiesInClass(clang::Decl*, clang::ObjCProtocolDecl*)"},
    {"_ZN5clang4Sema17ComparePropertiesEPNS_4DeclES2_", "clang::Sema::CompareProperties(clang::Decl*, clang::Decl*)"},
    {"_ZN5clang4Sema18isPropertyReadonlyEPNS_16ObjCPropertyDeclEPNS_17ObjCInterfaceDeclE", "clang::Sema::isPropertyReadonly(clang::ObjCPropertyDecl*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang4Sema26CollectImmediatePropertiesEPNS_17ObjCContainerDeclERN4llvm8DenseMapIPNS_14IdentifierInfoEPNS_16ObjCPropertyDeclENS3_12DenseMapInfoIS6_EENS9_IS8_EEEESD_", "clang::Sema::CollectImmediateProperties(clang::ObjCContainerDecl*, llvm::DenseMap<clang::IdentifierInfo*, clang::ObjCPropertyDecl*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::ObjCPropertyDecl*> >&, llvm::DenseMap<clang::IdentifierInfo*, clang::ObjCPropertyDecl*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::ObjCPropertyDecl*> >&)"},
    {"_ZN5clang4Sema18LookupPropertyDeclEPKNS_17ObjCContainerDeclEPNS_14IdentifierInfoE", "clang::Sema::LookupPropertyDecl(clang::ObjCContainerDecl const*, clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema27DefaultSynthesizePropertiesEPNS_5ScopeEPNS_12ObjCImplDeclEPNS_17ObjCInterfaceDeclE", "clang::Sema::DefaultSynthesizeProperties(clang::Scope*, clang::ObjCImplDecl*, clang::ObjCInterfaceDecl*)"},
    {"_Z35CollectClassPropertyImplementationsPN5clang17ObjCContainerDeclERN4llvm8DenseMapIPNS_14IdentifierInfoEPNS_16ObjCPropertyDeclENS2_12DenseMapInfoIS5_EENS8_IS7_EEEE", "CollectClassPropertyImplementations(clang::ObjCContainerDecl*, llvm::DenseMap<clang::IdentifierInfo*, clang::ObjCPropertyDecl*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::ObjCPropertyDecl*> >&)"},
    {"_ZN5clang4Sema31DiagnoseUnimplementedPropertiesEPNS_5ScopeEPNS_12ObjCImplDeclEPNS_17ObjCContainerDeclERKN4llvm8DenseSetINS_8SelectorENS7_12DenseMapInfoIS9_EEEE", "clang::Sema::DiagnoseUnimplementedProperties(clang::Scope*, clang::ObjCImplDecl*, clang::ObjCContainerDecl*, llvm::DenseSet<clang::Selector, llvm::DenseMapInfo<clang::Selector> > const&)"},
    {"_ZN5clang4Sema31AtomicPropertySetterGetterRulesEPNS_12ObjCImplDeclEPNS_17ObjCContainerDeclE", "clang::Sema::AtomicPropertySetterGetterRules(clang::ObjCImplDecl*, clang::ObjCContainerDecl*)"},
    {"_Z16AddPropertyAttrsRN5clang4SemaEPNS_14ObjCMethodDeclEPNS_16ObjCPropertyDeclE", "AddPropertyAttrs(clang::Sema&, clang::ObjCMethodDecl*, clang::ObjCPropertyDecl*)"},
    {"_ZN4llvm8DenseMapIPN5clang16ObjCPropertyDeclEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<clang::ObjCPropertyDecl*, char, llvm::DenseMapInfo<clang::ObjCPropertyDecl*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::ObjCPropertyDecl*, char> const&)"},
    {"_ZN4llvm8DenseMapIPN5clang16ObjCPropertyDeclEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<clang::ObjCPropertyDecl*, char, llvm::DenseMapInfo<clang::ObjCPropertyDecl*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::ObjCPropertyDecl* const&, char const&, std::pair<clang::ObjCPropertyDecl*, char>*)"},
    {"_ZN4llvm8DenseMapIPN5clang16ObjCPropertyDeclEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<clang::ObjCPropertyDecl*, char, llvm::DenseMapInfo<clang::ObjCPropertyDecl*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_16ObjCPropertyDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, clang::ObjCPropertyDecl*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::ObjCPropertyDecl*> >::InsertIntoBucket(clang::IdentifierInfo* const&, clang::ObjCPropertyDecl* const&, std::pair<clang::IdentifierInfo*, clang::ObjCPropertyDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_16ObjCPropertyDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, clang::ObjCPropertyDecl*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::ObjCPropertyDecl*> >::grow(unsigned int)"},
    {"_ZN5clang21GetConversionCategoryENS_22ImplicitConversionKindE", "clang::GetConversionCategory(clang::ImplicitConversionKind)"},
    {"_ZN5clang17GetConversionRankENS_22ImplicitConversionKindE", "clang::GetConversionRank(clang::ImplicitConversionKind)"},
    {"_ZN5clang25GetImplicitConversionNameENS_22ImplicitConversionKindE", "clang::GetImplicitConversionName(clang::ImplicitConversionKind)"},
    {"_ZN5clang26StandardConversionSequence23setAsIdentityConversionEv", "clang::StandardConversionSequence::setAsIdentityConversion()"},
    {"_ZNK5clang26StandardConversionSequence7getRankEv", "clang::StandardConversionSequence::getRank() const"},
    {"_ZNK5clang26StandardConversionSequence25isPointerConversionToBoolEv", "clang::StandardConversionSequence::isPointerConversionToBool() const"},
    {"_ZNK5clang26StandardConversionSequence32isPointerConversionToVoidPointerERNS_10ASTContextE", "clang::StandardConversionSequence::isPointerConversionToVoidPointer(clang::ASTContext&) const"},
    {"_ZNK5clang26StandardConversionSequence10DebugPrintEv", "clang::StandardConversionSequence::DebugPrint() const"},
    {"_ZNK5clang29UserDefinedConversionSequence10DebugPrintEv", "clang::UserDefinedConversionSequence::DebugPrint() const"},
    {"_ZNK5clang26ImplicitConversionSequence10DebugPrintEv", "clang::ImplicitConversionSequence::DebugPrint() const"},
    {"_ZN5clang27AmbiguousConversionSequence9constructEv", "clang::AmbiguousConversionSequence::construct()"},
    {"_ZN5clang27AmbiguousConversionSequence8destructEv", "clang::AmbiguousConversionSequence::destruct()"},
    {"_ZN5clang27AmbiguousConversionSequence8copyFromERKS0_", "clang::AmbiguousConversionSequence::copyFrom(clang::AmbiguousConversionSequence const&)"},
    {"_ZN5clang17OverloadCandidate20DeductionFailureInfo7DestroyEv", "clang::OverloadCandidate::DeductionFailureInfo::Destroy()"},
    {"_ZN5clang17OverloadCandidate20DeductionFailureInfo20getTemplateParameterEv", "clang::OverloadCandidate::DeductionFailureInfo::getTemplateParameter()"},
    {"_ZN5clang17OverloadCandidate20DeductionFailureInfo23getTemplateArgumentListEv", "clang::OverloadCandidate::DeductionFailureInfo::getTemplateArgumentList()"},
    {"_ZN5clang17OverloadCandidate20DeductionFailureInfo11getFirstArgEv", "clang::OverloadCandidate::DeductionFailureInfo::getFirstArg()"},
    {"_ZN5clang17OverloadCandidate20DeductionFailureInfo12getSecondArgEv", "clang::OverloadCandidate::DeductionFailureInfo::getSecondArg()"},
    {"_ZN5clang20OverloadCandidateSet5clearEv", "clang::OverloadCandidateSet::clear()"},
    {"_ZN5clang4Sema13CheckOverloadEPNS_5ScopeEPNS_12FunctionDeclERKNS_12LookupResultERPNS_9NamedDeclEb", "clang::Sema::CheckOverload(clang::Scope*, clang::FunctionDecl*, clang::LookupResult const&, clang::NamedDecl*&, bool)"},
    {"_ZN5clang4Sema10IsOverloadEPNS_12FunctionDeclES2_b", "clang::Sema::IsOverload(clang::FunctionDecl*, clang::FunctionDecl*, bool)"},
    {"_ZN5clang4Sema24FunctionArgTypesAreEqualEPKNS_17FunctionProtoTypeES3_", "clang::Sema::FunctionArgTypesAreEqual(clang::FunctionProtoType const*, clang::FunctionProtoType const*)"},
    {"_ZN5clang4Sema21TryImplicitConversionERNS_22InitializationSequenceERKNS_17InitializedEntityEPNS_4ExprEbbbb", "clang::Sema::TryImplicitConversion(clang::InitializationSequence&, clang::InitializedEntity const&, clang::Expr*, bool, bool, bool, bool)"},
    {"_ZN5clang21TryImplicitConversionERNS_4SemaEPNS_4ExprENS_8QualTypeEbbbb", "clang::TryImplicitConversion(clang::Sema&, clang::Expr*, clang::QualType, bool, bool, bool, bool)"},
    {"_ZN5clang4Sema25PerformImplicitConversionERPNS_4ExprENS_8QualTypeENS0_16AssignmentActionEb", "clang::Sema::PerformImplicitConversion(clang::Expr*&, clang::QualType, clang::Sema::AssignmentAction, bool)"},
    {"_ZN5clang4Sema25PerformImplicitConversionERPNS_4ExprENS_8QualTypeENS0_16AssignmentActionEbRNS_26ImplicitConversionSequenceE", "clang::Sema::PerformImplicitConversion(clang::Expr*&, clang::QualType, clang::Sema::AssignmentAction, bool, clang::ImplicitConversionSequence&)"},
    {"_ZN5clang4Sema19IsIntegralPromotionEPNS_4ExprENS_8QualTypeES3_", "clang::Sema::IsIntegralPromotion(clang::Expr*, clang::QualType, clang::QualType)"},
    {"_ZN5clang4Sema24IsFloatingPointPromotionENS_8QualTypeES1_", "clang::Sema::IsFloatingPointPromotion(clang::QualType, clang::QualType)"},
    {"_ZN5clang4Sema18IsComplexPromotionENS_8QualTypeES1_", "clang::Sema::IsComplexPromotion(clang::QualType, clang::QualType)"},
    {"_ZN5clang4Sema19IsPointerConversionEPNS_4ExprENS_8QualTypeES3_bRS3_Rb", "clang::Sema::IsPointerConversion(clang::Expr*, clang::QualType, clang::QualType, bool, clang::QualType&, bool&)"},
    {"_ZN5clang4Sema23isObjCPointerConversionENS_8QualTypeES1_RS1_Rb", "clang::Sema::isObjCPointerConversion(clang::QualType, clang::QualType, clang::QualType&, bool&)"},
    {"_ZN5clang34BuildSimilarlyQualifiedPointerTypeEPKNS_4TypeENS_8QualTypeES3_RNS_10ASTContextE", "clang::BuildSimilarlyQualifiedPointerType(clang::Type const*, clang::QualType, clang::QualType, clang::ASTContext&)"},
    {"_ZN5clang4Sema22CheckPointerConversionEPNS_4ExprENS_8QualTypeERNS_8CastKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEb", "clang::Sema::CheckPointerConversion(clang::Expr*, clang::QualType, clang::CastKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&, bool)"},
    {"_ZN5clang4Sema25IsMemberPointerConversionEPNS_4ExprENS_8QualTypeES3_bRS3_", "clang::Sema::IsMemberPointerConversion(clang::Expr*, clang::QualType, clang::QualType, bool, clang::QualType&)"},
    {"_ZN5clang4Sema28CheckMemberPointerConversionEPNS_4ExprENS_8QualTypeERNS_8CastKindERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEb", "clang::Sema::CheckMemberPointerConversion(clang::Expr*, clang::QualType, clang::CastKind&, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&, bool)"},
    {"_ZN5clang4Sema25IsQualificationConversionENS_8QualTypeES1_b", "clang::Sema::IsQualificationConversion(clang::QualType, clang::QualType, bool)"},
    {"_ZN5clang4Sema37DiagnoseMultipleUserDefinedConversionEPNS_4ExprENS_8QualTypeE", "clang::Sema::DiagnoseMultipleUserDefinedConversion(clang::Expr*, clang::QualType)"},
    {"_ZN5clang23IsUserDefinedConversionERNS_4SemaEPNS_4ExprENS_8QualTypeERNS_29UserDefinedConversionSequenceERNS_20OverloadCandidateSetEb", "clang::IsUserDefinedConversion(clang::Sema&, clang::Expr*, clang::QualType, clang::UserDefinedConversionSequence&, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang20OverloadCandidateSet14NoteCandidatesERNS_4SemaENS_28OverloadCandidateDisplayKindEPPNS_4ExprEjPKcNS_14SourceLocationE", "clang::OverloadCandidateSet::NoteCandidates(clang::Sema&, clang::OverloadCandidateDisplayKind, clang::Expr**, unsigned int, char const*, clang::SourceLocation)"},
    {"_ZN5clang4Sema28CompareReferenceRelationshipENS_14SourceLocationENS_8QualTypeES2_RbS3_", "clang::Sema::CompareReferenceRelationship(clang::SourceLocation, clang::QualType, clang::QualType, bool&, bool&)"},
    {"_ZN5clang4Sema35PerformObjectArgumentInitializationERPNS_4ExprEPNS_19NestedNameSpecifierEPNS_9NamedDeclEPNS_13CXXMethodDeclE", "clang::Sema::PerformObjectArgumentInitialization(clang::Expr*&, clang::NestedNameSpecifier*, clang::NamedDecl*, clang::CXXMethodDecl*)"},
    {"_ZN5clang31TryObjectArgumentInitializationERNS_4SemaENS_8QualTypeENS_4Expr14ClassificationEPNS_13CXXMethodDeclEPNS_13CXXRecordDeclE", "clang::TryObjectArgumentInitialization(clang::Sema&, clang::QualType, clang::Expr::Classification, clang::CXXMethodDecl*, clang::CXXRecordDecl*)"},
    {"_ZN5clang4Sema32PerformContextuallyConvertToBoolERPNS_4ExprE", "clang::Sema::PerformContextuallyConvertToBool(clang::Expr*&)"},
    {"_ZN5clang4Sema34PerformContextuallyConvertToObjCIdERPNS_4ExprE", "clang::Sema::PerformContextuallyConvertToObjCId(clang::Expr*&)"},
    {"_ZN5clang4Sema34ConvertToIntegralOrEnumerationTypeENS_14SourceLocationEPNS_4ExprERKNS_17PartialDiagnosticES6_S6_S6_S6_S6_S6_", "clang::Sema::ConvertToIntegralOrEnumerationType(clang::SourceLocation, clang::Expr*, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema20AddOverloadCandidateEPNS_12FunctionDeclENS_14DeclAccessPairEPPNS_4ExprEjRNS_20OverloadCandidateSetEbb", "clang::Sema::AddOverloadCandidate(clang::FunctionDecl*, clang::DeclAccessPair, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool, bool)"},
    {"_ZN5clang4Sema18AddMethodCandidateEPNS_13CXXMethodDeclENS_14DeclAccessPairEPNS_13CXXRecordDeclENS_8QualTypeENS_4Expr14ClassificationEPPS7_jRNS_20OverloadCandidateSetEb", "clang::Sema::AddMethodCandidate(clang::CXXMethodDecl*, clang::DeclAccessPair, clang::CXXRecordDecl*, clang::QualType, clang::Expr::Classification, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang21TryCopyInitializationERNS_4SemaEPNS_4ExprENS_8QualTypeEbb", "clang::TryCopyInitialization(clang::Sema&, clang::Expr*, clang::QualType, bool, bool)"},
    {"_ZN5clang4Sema21AddFunctionCandidatesERKNS_17UnresolvedSetImplEPPNS_4ExprEjRNS_20OverloadCandidateSetEb", "clang::Sema::AddFunctionCandidates(clang::UnresolvedSetImpl const&, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang4Sema26AddMethodTemplateCandidateEPNS_20FunctionTemplateDeclENS_14DeclAccessPairEPNS_13CXXRecordDeclEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_4Expr14ClassificationEPPSA_jRNS_20OverloadCandidateSetEb", "clang::Sema::AddMethodTemplateCandidate(clang::FunctionTemplateDecl*, clang::DeclAccessPair, clang::CXXRecordDecl*, clang::TemplateArgumentListInfo const*, clang::QualType, clang::Expr::Classification, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang4Sema28AddTemplateOverloadCandidateEPNS_20FunctionTemplateDeclENS_14DeclAccessPairEPKNS_24TemplateArgumentListInfoEPPNS_4ExprEjRNS_20OverloadCandidateSetEb", "clang::Sema::AddTemplateOverloadCandidate(clang::FunctionTemplateDecl*, clang::DeclAccessPair, clang::TemplateArgumentListInfo const*, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang4Sema18AddMethodCandidateENS_14DeclAccessPairENS_8QualTypeENS_4Expr14ClassificationEPPS3_jRNS_20OverloadCandidateSetEb", "clang::Sema::AddMethodCandidate(clang::DeclAccessPair, clang::QualType, clang::Expr::Classification, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang4Sema22AddConversionCandidateEPNS_17CXXConversionDeclENS_14DeclAccessPairEPNS_13CXXRecordDeclEPNS_4ExprENS_8QualTypeERNS_20OverloadCandidateSetE", "clang::Sema::AddConversionCandidate(clang::CXXConversionDecl*, clang::DeclAccessPair, clang::CXXRecordDecl*, clang::Expr*, clang::QualType, clang::OverloadCandidateSet&)"},
    {"_ZN5clang4Sema30AddTemplateConversionCandidateEPNS_20FunctionTemplateDeclENS_14DeclAccessPairEPNS_13CXXRecordDeclEPNS_4ExprENS_8QualTypeERNS_20OverloadCandidateSetE", "clang::Sema::AddTemplateConversionCandidate(clang::FunctionTemplateDecl*, clang::DeclAccessPair, clang::CXXRecordDecl*, clang::Expr*, clang::QualType, clang::OverloadCandidateSet&)"},
    {"_ZN5clang4Sema21AddSurrogateCandidateEPNS_17CXXConversionDeclENS_14DeclAccessPairEPNS_13CXXRecordDeclEPKNS_17FunctionProtoTypeEPNS_4ExprEPSA_jRNS_20OverloadCandidateSetE", "clang::Sema::AddSurrogateCandidate(clang::CXXConversionDecl*, clang::DeclAccessPair, clang::CXXRecordDecl*, clang::FunctionProtoType const*, clang::Expr*, clang::Expr**, unsigned int, clang::OverloadCandidateSet&)"},
    {"_ZN5clang4Sema27AddMemberOperatorCandidatesENS_22OverloadedOperatorKindENS_14SourceLocationEPPNS_4ExprEjRNS_20OverloadCandidateSetENS_11SourceRangeE", "clang::Sema::AddMemberOperatorCandidates(clang::OverloadedOperatorKind, clang::SourceLocation, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, clang::SourceRange)"},
    {"_ZN5clang4Sema19AddBuiltinCandidateENS_8QualTypeEPS1_PPNS_4ExprEjRNS_20OverloadCandidateSetEbj", "clang::Sema::AddBuiltinCandidate(clang::QualType, clang::QualType*, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool, unsigned int)"},
    {"_ZN5clang23BuiltinCandidateTypeSet39AddPointerWithMoreQualifiedTypeVariantsENS_8QualTypeERKNS_10QualifiersE", "clang::BuiltinCandidateTypeSet::AddPointerWithMoreQualifiedTypeVariants(clang::QualType, clang::Qualifiers const&)"},
    {"_ZN5clang23BuiltinCandidateTypeSet45AddMemberPointerWithMoreQualifiedTypeVariantsENS_8QualTypeE", "clang::BuiltinCandidateTypeSet::AddMemberPointerWithMoreQualifiedTypeVariants(clang::QualType)"},
    {"_ZN5clang23BuiltinCandidateTypeSet21AddTypesConvertedFromENS_8QualTypeENS_14SourceLocationEbbRKNS_10QualifiersE", "clang::BuiltinCandidateTypeSet::AddTypesConvertedFrom(clang::QualType, clang::SourceLocation, bool, bool, clang::Qualifiers const&)"},
    {"_ZN5clang4Sema28AddBuiltinOperatorCandidatesENS_22OverloadedOperatorKindENS_14SourceLocationEPPNS_4ExprEjRNS_20OverloadCandidateSetE", "clang::Sema::AddBuiltinOperatorCandidates(clang::OverloadedOperatorKind, clang::SourceLocation, clang::Expr**, unsigned int, clang::OverloadCandidateSet&)"},
    {"_ZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder35addGenericBinaryArithmeticOverloadsEb", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::addGenericBinaryArithmeticOverloads(bool)"},
    {"_ZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder35addBinaryBitwiseArithmeticOverloadsENS_22OverloadedOperatorKindE", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::addBinaryBitwiseArithmeticOverloads(clang::OverloadedOperatorKind)"},
    {"_ZN5clang4Sema36AddArgumentDependentLookupCandidatesENS_15DeclarationNameEbPPNS_4ExprEjPKNS_24TemplateArgumentListInfoERNS_20OverloadCandidateSetEb", "clang::Sema::AddArgumentDependentLookupCandidates(clang::DeclarationName, bool, clang::Expr**, unsigned int, clang::TemplateArgumentListInfo const*, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang25isBetterOverloadCandidateERNS_4SemaERKNS_17OverloadCandidateES4_NS_14SourceLocationEb", "clang::isBetterOverloadCandidate(clang::Sema&, clang::OverloadCandidate const&, clang::OverloadCandidate const&, clang::SourceLocation, bool)"},
    {"_ZN5clang34CompareImplicitConversionSequencesERNS_4SemaERKNS_26ImplicitConversionSequenceES4_", "clang::CompareImplicitConversionSequences(clang::Sema&, clang::ImplicitConversionSequence const&, clang::ImplicitConversionSequence const&)"},
    {"_ZN5clang34CompareStandardConversionSequencesERNS_4SemaERKNS_26StandardConversionSequenceES4_", "clang::CompareStandardConversionSequences(clang::Sema&, clang::StandardConversionSequence const&, clang::StandardConversionSequence const&)"},
    {"_ZN5clang20OverloadCandidateSet18BestViableFunctionERNS_4SemaENS_14SourceLocationERPNS_17OverloadCandidateEb", "clang::OverloadCandidateSet::BestViableFunction(clang::Sema&, clang::SourceLocation, clang::OverloadCandidate*&, bool)"},
    {"_ZN5clang4Sema21NoteOverloadCandidateEPNS_12FunctionDeclE", "clang::Sema::NoteOverloadCandidate(clang::FunctionDecl*)"},
    {"_ZN5clang12_GLOBAL__N_125ClassifyOverloadCandidateERNS_4SemaEPNS_12FunctionDeclERSs", "clang::(anonymous namespace)::ClassifyOverloadCandidate(clang::Sema&, clang::FunctionDecl*, std::string&)"},
    {"_ZNK5clang26ImplicitConversionSequence27DiagnoseAmbiguousConversionERNS_4SemaENS_14SourceLocationERKNS_17PartialDiagnosticE", "clang::ImplicitConversionSequence::DiagnoseAmbiguousConversion(clang::Sema&, clang::SourceLocation, clang::PartialDiagnostic const&) const"},
    {"_ZN5clang4Sema34ResolveAddressOfOverloadedFunctionEPNS_4ExprENS_8QualTypeEbRNS_14DeclAccessPairE", "clang::Sema::ResolveAddressOfOverloadedFunction(clang::Expr*, clang::QualType, bool, clang::DeclAccessPair&)"},
    {"_ZN5clang20IsNoReturnConversionERNS_10ASTContextENS_8QualTypeES2_RS2_", "clang::IsNoReturnConversion(clang::ASTContext&, clang::QualType, clang::QualType, clang::QualType&)"},
    {"_ZN5clang4Sema43ResolveSingleFunctionTemplateSpecializationEPNS_4ExprE", "clang::Sema::ResolveSingleFunctionTemplateSpecialization(clang::Expr*)"},
    {"_ZN5clang4Sema27AddOverloadedCallCandidatesEPNS_20UnresolvedLookupExprEPPNS_4ExprEjRNS_20OverloadCandidateSetEb", "clang::Sema::AddOverloadedCallCandidates(clang::UnresolvedLookupExpr*, clang::Expr**, unsigned int, clang::OverloadCandidateSet&, bool)"},
    {"_ZN5clang4Sema23BuildOverloadedCallExprEPNS_5ScopeEPNS_4ExprEPNS_20UnresolvedLookupExprENS_14SourceLocationEPS4_jS7_S4_", "clang::Sema::BuildOverloadedCallExpr(clang::Scope*, clang::Expr*, clang::UnresolvedLookupExpr*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema30FixOverloadedFunctionReferenceEPNS_4ExprENS_14DeclAccessPairEPNS_12FunctionDeclE", "clang::Sema::FixOverloadedFunctionReference(clang::Expr*, clang::DeclAccessPair, clang::FunctionDecl*)"},
    {"_ZN5clang4Sema23CreateOverloadedUnaryOpENS_14SourceLocationEjRKNS_17UnresolvedSetImplEPNS_4ExprE", "clang::Sema::CreateOverloadedUnaryOp(clang::SourceLocation, unsigned int, clang::UnresolvedSetImpl const&, clang::Expr*)"},
    {"_ZN5clang4Sema21CreateOverloadedBinOpENS_14SourceLocationEjRKNS_17UnresolvedSetImplEPNS_4ExprES6_", "clang::Sema::CreateOverloadedBinOp(clang::SourceLocation, unsigned int, clang::UnresolvedSetImpl const&, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema34CreateOverloadedArraySubscriptExprENS_14SourceLocationES1_PNS_4ExprES3_", "clang::Sema::CreateOverloadedArraySubscriptExpr(clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::Expr*)"},
    {"_ZN5clang4Sema25BuildCallToMemberFunctionEPNS_5ScopeEPNS_4ExprENS_14SourceLocationEPS4_jS5_", "clang::Sema::BuildCallToMemberFunction(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema28BuildCallToObjectOfClassTypeEPNS_5ScopeEPNS_4ExprENS_14SourceLocationEPS4_jS5_", "clang::Sema::BuildCallToObjectOfClassType(clang::Scope*, clang::Expr*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema24BuildOverloadedArrowExprEPNS_5ScopeEPNS_4ExprENS_14SourceLocationE", "clang::Sema::BuildOverloadedArrowExpr(clang::Scope*, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema30FixOverloadedFunctionReferenceENS_12ActionResultIPNS_4ExprELb1EEENS_14DeclAccessPairEPNS_12FunctionDeclE", "clang::Sema::FixOverloadedFunctionReference(clang::ActionResult<clang::Expr*, true>, clang::DeclAccessPair, clang::FunctionDecl*)"},
    {"_ZSt16__introsort_loopIPPN5clang17OverloadCandidateElNS0_12_GLOBAL__N_135CompareOverloadCandidatesForDisplayEEvT_S6_T0_T1_", "void std::__introsort_loop<clang::OverloadCandidate**, long, clang::(anonymous namespace)::CompareOverloadCandidatesForDisplay>(clang::OverloadCandidate**, clang::OverloadCandidate**, long, clang::(anonymous namespace)::CompareOverloadCandidatesForDisplay)"},
    {"_ZN5clang12_GLOBAL__N_135CompareOverloadCandidatesForDisplayclEPKNS_17OverloadCandidateES4_", "clang::(anonymous namespace)::CompareOverloadCandidatesForDisplay::operator()(clang::OverloadCandidate const*, clang::OverloadCandidate const*)"},
    {"_ZSt13__adjust_heapIPPN5clang17OverloadCandidateElS2_NS0_12_GLOBAL__N_135CompareOverloadCandidatesForDisplayEEvT_T0_S7_T1_T2_", "void std::__adjust_heap<clang::OverloadCandidate**, long, clang::OverloadCandidate*, clang::(anonymous namespace)::CompareOverloadCandidatesForDisplay>(clang::OverloadCandidate**, long, long, clang::OverloadCandidate*, clang::(anonymous namespace)::CompareOverloadCandidatesForDisplay)"},
    {"_ZN5clang12_GLOBAL__N_121DiagnoseArityMismatchERNS_4SemaEPNS_17OverloadCandidateEj", "clang::(anonymous namespace)::DiagnoseArityMismatch(clang::Sema&, clang::OverloadCandidate*, unsigned int)"},
    {"_ZN5clang38AddBuiltinAssignmentOperatorCandidatesERNS_4SemaENS_8QualTypeEPPNS_4ExprEjRNS_20OverloadCandidateSetE", "clang::AddBuiltinAssignmentOperatorCandidates(clang::Sema&, clang::QualType, clang::Expr**, unsigned int, clang::OverloadCandidateSet&)"},
    {"_ZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder29getUsualArithmeticConversionsEjj", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::getUsualArithmeticConversions(unsigned int, unsigned int)"},
    {"_ZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder35addPlusPlusMinusMinusStyleOverloadsENS_8QualTypeEb", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::addPlusPlusMinusMinusStyleOverloads(clang::QualType, bool)"},
    {"_ZN5clang24FindConversionForRefInitERNS_4SemaERNS_26ImplicitConversionSequenceENS_8QualTypeENS_14SourceLocationEPNS_4ExprES4_bb", "clang::FindConversionForRefInit(clang::Sema&, clang::ImplicitConversionSequence&, clang::QualType, clang::SourceLocation, clang::Expr*, clang::QualType, bool, bool)"},
    {"_ZN4llvm15SmallVectorImplIN5clang17OverloadCandidateEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::OverloadCandidate>::push_back(clang::OverloadCandidate const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang26ImplicitConversionSequenceEE6resizeEj", "llvm::SmallVectorImpl<clang::ImplicitConversionSequence>::resize(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23BuiltinCandidateTypeSetEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::BuiltinCandidateTypeSet>::push_back(clang::BuiltinCandidateTypeSet const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang23BuiltinCandidateTypeSetELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::BuiltinCandidateTypeSet, false>::grow(unsigned long)"},
    {"_ZSt24__uninitialized_copy_auxIPN5clang23BuiltinCandidateTypeSetES2_ET0_T_S4_S3_St12__false_type", "clang::BuiltinCandidateTypeSet* std::__uninitialized_copy_aux<clang::BuiltinCandidateTypeSet*, clang::BuiltinCandidateTypeSet*>(clang::BuiltinCandidateTypeSet*, clang::BuiltinCandidateTypeSet*, clang::BuiltinCandidateTypeSet*, std::__false_type)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang26ImplicitConversionSequenceELb0EE18uninitialized_copyIPS2_S5_EEvT_S6_T0_", "void llvm::SmallVectorTemplateBase<clang::ImplicitConversionSequence, false>::uninitialized_copy<clang::ImplicitConversionSequence*, clang::ImplicitConversionSequence*>(clang::ImplicitConversionSequence*, clang::ImplicitConversionSequence*, clang::ImplicitConversionSequence*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang17OverloadCandidateELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::OverloadCandidate, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang17OverloadCandidateELb0EE13destroy_rangeEPS2_S4_", "llvm::SmallVectorTemplateBase<clang::OverloadCandidate, false>::destroy_range(clang::OverloadCandidate*, clang::OverloadCandidate*)"},
    {"_ZSt24__uninitialized_copy_auxIPN5clang17OverloadCandidateES2_ET0_T_S4_S3_St12__false_type", "clang::OverloadCandidate* std::__uninitialized_copy_aux<clang::OverloadCandidate*, clang::OverloadCandidate*>(clang::OverloadCandidate*, clang::OverloadCandidate*, clang::OverloadCandidate*, std::__false_type)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang26ImplicitConversionSequenceELb0EE18uninitialized_copyIPKS2_PS2_EEvT_S8_T0_", "void llvm::SmallVectorTemplateBase<clang::ImplicitConversionSequence, false>::uninitialized_copy<clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence*>(clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence*)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPKN5clang26ImplicitConversionSequenceEPS4_EET0_T_S9_S8_", "clang::ImplicitConversionSequence* std::__copy<false, std::random_access_iterator_tag>::copy<clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence*>(clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence const*, clang::ImplicitConversionSequence*)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang12FunctionDeclEEaSERKS4_", "llvm::SmallVectorImpl<clang::FunctionDecl*>::operator=(llvm::SmallVectorImpl<clang::FunctionDecl*> const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23BuiltinCandidateTypeSetEED2Ev", "llvm::SmallVectorImpl<clang::BuiltinCandidateTypeSet>::~SmallVectorImpl()"},
    {"_ZNK4llvm8DenseMapISt4pairIN5clang7CanQualINS2_4TypeEEES5_EcNS_12DenseMapInfoIS6_EENS7_IcEEE15LookupBucketForERKS6_RPS1_IS6_cE", "llvm::DenseMap<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > >, llvm::DenseMapInfo<char> >::LookupBucketFor(std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > const&, std::pair<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIN5clang7CanQualINS2_4TypeEEES5_EcNS_12DenseMapInfoIS6_EENS7_IcEEE6insertERKS1_IS6_cE", "llvm::DenseMap<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char> const&)"},
    {"_ZN4llvm8DenseMapISt4pairIN5clang7CanQualINS2_4TypeEEES5_EcNS_12DenseMapInfoIS6_EENS7_IcEEE16InsertIntoBucketERKS6_RKcPS1_IS6_cE", "llvm::DenseMap<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > const&, char const&, std::pair<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairIN5clang7CanQualINS2_4TypeEEES5_EcNS_12DenseMapInfoIS6_EENS7_IcEEE4growEj", "llvm::DenseMap<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> >, char, llvm::DenseMapInfo<std::pair<clang::CanQual<clang::Type>, clang::CanQual<clang::Type> > >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang20OverloadCandidateSetD2Ev", "clang::OverloadCandidateSet::~OverloadCandidateSet()"},
    {"_ZN5clang4Sema13ActOnExprStmtENS0_11FullExprArgE", "clang::Sema::ActOnExprStmt(clang::Sema::FullExprArg)"},
    {"_ZN5clang4Sema13ActOnNullStmtENS_14SourceLocationEb", "clang::Sema::ActOnNullStmt(clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema13ActOnDeclStmtENS_9OpaquePtrINS_12DeclGroupRefEEENS_14SourceLocationES4_", "clang::Sema::ActOnDeclStmt(clang::OpaquePtr<clang::DeclGroupRef>, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema20ActOnForEachDeclStmtENS_9OpaquePtrINS_12DeclGroupRefEEE", "clang::Sema::ActOnForEachDeclStmt(clang::OpaquePtr<clang::DeclGroupRef>)"},
    {"_ZN5clang4Sema24DiagnoseUnusedExprResultEPKNS_4StmtE", "clang::Sema::DiagnoseUnusedExprResult(clang::Stmt const*)"},
    {"_ZN5clang4Sema17ActOnCompoundStmtENS_14SourceLocationES1_NS_11ASTMultiPtrIPNS_4StmtEEEb", "clang::Sema::ActOnCompoundStmt(clang::SourceLocation, clang::SourceLocation, clang::ASTMultiPtr<clang::Stmt*>, bool)"},
    {"_ZN5clang4Sema13ActOnCaseStmtENS_14SourceLocationEPNS_4ExprES1_S3_S1_", "clang::Sema::ActOnCaseStmt(clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema17ActOnCaseStmtBodyEPNS_4StmtES2_", "clang::Sema::ActOnCaseStmtBody(clang::Stmt*, clang::Stmt*)"},
    {"_ZN5clang4Sema16ActOnDefaultStmtENS_14SourceLocationES1_PNS_4StmtEPNS_5ScopeE", "clang::Sema::ActOnDefaultStmt(clang::SourceLocation, clang::SourceLocation, clang::Stmt*, clang::Scope*)"},
    {"_ZN5clang4Sema14ActOnLabelStmtENS_14SourceLocationEPNS_14IdentifierInfoES1_PNS_4StmtEPKNS_13AttributeListE", "clang::Sema::ActOnLabelStmt(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Stmt*, clang::AttributeList const*)"},
    {"_ZN5clang4Sema14ActOnLabelStmtENS_14SourceLocationEPNS_14IdentifierInfoES1_PNS_4StmtEb", "clang::Sema::ActOnLabelStmt(clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::Stmt*, bool)"},
    {"_ZN5clang4Sema11ActOnIfStmtENS_14SourceLocationENS0_11FullExprArgEPNS_4DeclEPNS_4StmtES1_S6_", "clang::Sema::ActOnIfStmt(clang::SourceLocation, clang::Sema::FullExprArg, clang::Decl*, clang::Stmt*, clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang4Sema34ConvertIntegerToTypeWarnOnOverflowERN4llvm6APSIntEjbNS_14SourceLocationEj", "clang::Sema::ConvertIntegerToTypeWarnOnOverflow(llvm::APSInt&, unsigned int, bool, clang::SourceLocation, unsigned int)"},
    {"_ZN5clang4Sema22ActOnStartOfSwitchStmtENS_14SourceLocationEPNS_4ExprEPNS_4DeclE", "clang::Sema::ActOnStartOfSwitchStmt(clang::SourceLocation, clang::Expr*, clang::Decl*)"},
    {"_ZN5clang4Sema21ActOnFinishSwitchStmtENS_14SourceLocationEPNS_4StmtES3_", "clang::Sema::ActOnFinishSwitchStmt(clang::SourceLocation, clang::Stmt*, clang::Stmt*)"},
    {"_Z11CmpCaseValsRKSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_", "CmpCaseVals(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)"},
    {"_Z12AdjustAPSIntRN4llvm6APSIntEjb", "AdjustAPSInt(llvm::APSInt&, unsigned int, bool)"},
    {"_Z11CmpEnumValsRKSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_", "CmpEnumVals(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)"},
    {"_Z10EqEnumValsRKSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_", "EqEnumVals(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)"},
    {"_ZN5clang4Sema14ActOnWhileStmtENS_14SourceLocationENS0_11FullExprArgEPNS_4DeclEPNS_4StmtE", "clang::Sema::ActOnWhileStmt(clang::SourceLocation, clang::Sema::FullExprArg, clang::Decl*, clang::Stmt*)"},
    {"_ZN5clang4Sema11ActOnDoStmtENS_14SourceLocationEPNS_4StmtES1_S1_PNS_4ExprES1_", "clang::Sema::ActOnDoStmt(clang::SourceLocation, clang::Stmt*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema12ActOnForStmtENS_14SourceLocationES1_PNS_4StmtENS0_11FullExprArgEPNS_4DeclES4_S1_S3_", "clang::Sema::ActOnForStmt(clang::SourceLocation, clang::SourceLocation, clang::Stmt*, clang::Sema::FullExprArg, clang::Decl*, clang::Sema::FullExprArg, clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang4Sema22ActOnForEachLValueExprEPNS_4ExprE", "clang::Sema::ActOnForEachLValueExpr(clang::Expr*)"},
    {"_ZN5clang4Sema26ActOnObjCForCollectionStmtENS_14SourceLocationES1_PNS_4StmtEPNS_4ExprES1_S3_", "clang::Sema::ActOnObjCForCollectionStmt(clang::SourceLocation, clang::SourceLocation, clang::Stmt*, clang::Expr*, clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang4Sema13ActOnGotoStmtENS_14SourceLocationES1_PNS_14IdentifierInfoE", "clang::Sema::ActOnGotoStmt(clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang4Sema21ActOnIndirectGotoStmtENS_14SourceLocationES1_PNS_4ExprE", "clang::Sema::ActOnIndirectGotoStmt(clang::SourceLocation, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema17ActOnContinueStmtENS_14SourceLocationEPNS_5ScopeE", "clang::Sema::ActOnContinueStmt(clang::SourceLocation, clang::Scope*)"},
    {"_ZN5clang4Sema14ActOnBreakStmtENS_14SourceLocationEPNS_5ScopeE", "clang::Sema::ActOnBreakStmt(clang::SourceLocation, clang::Scope*)"},
    {"_ZN5clang4Sema23getCopyElisionCandidateENS_8QualTypeEPNS_4ExprEb", "clang::Sema::getCopyElisionCandidate(clang::QualType, clang::Expr*, bool)"},
    {"_ZN5clang4Sema31PerformMoveOrCopyInitializationERKNS_17InitializedEntityEPKNS_7VarDeclENS_8QualTypeEPNS_4ExprE", "clang::Sema::PerformMoveOrCopyInitialization(clang::InitializedEntity const&, clang::VarDecl const*, clang::QualType, clang::Expr*)"},
    {"_ZN5clang4Sema20ActOnBlockReturnStmtENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnBlockReturnStmt(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema15ActOnReturnStmtENS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnReturnStmt(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema12ActOnAsmStmtENS_14SourceLocationEbbjjPPNS_14IdentifierInfoENS_11ASTMultiPtrIPNS_4ExprEEES8_S7_S8_S1_b", "clang::Sema::ActOnAsmStmt(clang::SourceLocation, bool, bool, unsigned int, unsigned int, clang::IdentifierInfo**, clang::ASTMultiPtr<clang::Expr*>, clang::ASTMultiPtr<clang::Expr*>, clang::Expr*, clang::ASTMultiPtr<clang::Expr*>, clang::SourceLocation, bool)"},
    {"_Z14CheckAsmLValuePKN5clang4ExprERNS_4SemaE", "CheckAsmLValue(clang::Expr const*, clang::Sema&)"},
    {"_ZN5clang4Sema20ActOnObjCAtCatchStmtENS_14SourceLocationES1_PNS_4DeclEPNS_4StmtE", "clang::Sema::ActOnObjCAtCatchStmt(clang::SourceLocation, clang::SourceLocation, clang::Decl*, clang::Stmt*)"},
    {"_ZN5clang4Sema22ActOnObjCAtFinallyStmtENS_14SourceLocationEPNS_4StmtE", "clang::Sema::ActOnObjCAtFinallyStmt(clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang4Sema18ActOnObjCAtTryStmtENS_14SourceLocationEPNS_4StmtENS_11ASTMultiPtrIS3_EES3_", "clang::Sema::ActOnObjCAtTryStmt(clang::SourceLocation, clang::Stmt*, clang::ASTMultiPtr<clang::Stmt*>, clang::Stmt*)"},
    {"_ZN5clang4Sema20BuildObjCAtThrowStmtENS_14SourceLocationEPNS_4ExprE", "clang::Sema::BuildObjCAtThrowStmt(clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema20ActOnObjCAtThrowStmtENS_14SourceLocationEPNS_4ExprEPNS_5ScopeE", "clang::Sema::ActOnObjCAtThrowStmt(clang::SourceLocation, clang::Expr*, clang::Scope*)"},
    {"_ZN5clang4Sema27ActOnObjCAtSynchronizedStmtENS_14SourceLocationEPNS_4ExprEPNS_4StmtE", "clang::Sema::ActOnObjCAtSynchronizedStmt(clang::SourceLocation, clang::Expr*, clang::Stmt*)"},
    {"_ZN5clang4Sema18ActOnCXXCatchBlockENS_14SourceLocationEPNS_4DeclEPNS_4StmtE", "clang::Sema::ActOnCXXCatchBlock(clang::SourceLocation, clang::Decl*, clang::Stmt*)"},
    {"_ZN5clang4Sema16ActOnCXXTryBlockENS_14SourceLocationEPNS_4StmtENS_11ASTMultiPtrIS3_EE", "clang::Sema::ActOnCXXTryBlock(clang::SourceLocation, clang::Stmt*, clang::ASTMultiPtr<clang::Stmt*>)"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_115TypeWithHandlerEEEiPKvS4_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::TypeWithHandler>(void const*, void const*)"},
    {"_ZSt6uniqueIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEEPFbRKS6_S9_EET_SC_SC_T0_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::unique<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang10TargetInfo14ConstraintInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::TargetInfo::ConstraintInfo, false>::grow(unsigned long)"},
    {"_ZSt21__inplace_stable_sortIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEEPFbRKS6_S9_EEvT_SC_T0_", "void std::__inplace_stable_sort<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt22__stable_sort_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_lPFbRKS6_S9_EEvT_SC_T0_T1_T2_", "void std::__stable_sort_adaptive<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt24__merge_sort_with_bufferIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_PFbRKS6_S9_EEvT_SC_T0_T1_", "void std::__merge_sort_with_buffer<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt16__merge_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEElS7_PFbRKS6_S9_EEvT_SC_SC_T0_SD_T1_SD_T2_", "void std::__merge_adaptive<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, long, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt5mergeIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_S7_PFbRKS6_S9_EET1_T_SD_T0_SE_SC_T2_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::merge<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt16__merge_backwardIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_S7_PFbRKS6_S9_EET1_T_SD_T0_SE_SC_T2_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::__merge_backward<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt17__rotate_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES7_lET_S8_S8_S8_T1_S9_T0_S9_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::__rotate_adaptive<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, long, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long)"},
    {"_ZSt8__rotateIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEEEvT_S8_S8_St26random_access_iterator_tag", "void std::__rotate<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::random_access_iterator_tag)"},
    {"_ZSt4swapISt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEEEvRT_S8_", "void std::swap<std::pair<llvm::APSInt, clang::EnumConstantDecl*> >(std::pair<llvm::APSInt, clang::EnumConstantDecl*>&, std::pair<llvm::APSInt, clang::EnumConstantDecl*>&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEESA_EET0_T_SC_SB_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEESA_EET0_T_SC_SB_", "std::pair<llvm::APSInt, clang::EnumConstantDecl*>* std::__copy<false, std::random_access_iterator_tag>::copy<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*)"},
    {"_ZSt16__insertion_sortIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEEPFbRKS6_S9_EEvT_SC_T0_", "void std::__insertion_sort<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt25__unguarded_linear_insertIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES6_PFbRKS6_S9_EEvT_T0_T1_", "void std::__unguarded_linear_insert<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZSt22__merge_without_bufferIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEElPFbRKS6_S9_EEvT_SC_SC_T0_SD_T1_", "void std::__merge_without_buffer<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&)>(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, long, long, bool (*)(std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&, std::pair<llvm::APSInt, clang::EnumConstantDecl*> const&))"},
    {"_ZNSt17_Temporary_bufferIPSt4pairIN4llvm6APSIntEPN5clang16EnumConstantDeclEES6_EC2ES7_S7_", "std::_Temporary_buffer<std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*> >::_Temporary_buffer(std::pair<llvm::APSInt, clang::EnumConstantDecl*>*, std::pair<llvm::APSInt, clang::EnumConstantDecl*>*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairINS_6APSIntEPN5clang16EnumConstantDeclEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::APSInt, clang::EnumConstantDecl*>, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIN4llvm6APSIntESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::APSInt, std::allocator<llvm::APSInt> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::APSInt*, std::vector<llvm::APSInt, std::allocator<llvm::APSInt> > >, llvm::APSInt const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm6APSIntES5_EET0_T_S7_S6_", "llvm::APSInt* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::APSInt*, llvm::APSInt*>(llvm::APSInt*, llvm::APSInt*, llvm::APSInt*)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESA_EET0_T_SC_SB_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::__copy<false, std::random_access_iterator_tag>::copy<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*)"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEEEvT_SE_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES9_lEvT_SE_T0_T1_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*, long)"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES9_EvT_SE_T0_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*)"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEElS9_EvT_SE_SE_T0_SF_T1_SF_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, long, std::pair<llvm::APSInt, clang::CaseStmt*>*>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, long, long, std::pair<llvm::APSInt, clang::CaseStmt*>*, long)"},
    {"_ZSt5mergeIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEEN9__gnu_cxx17__normal_iteratorIS7_St6vectorIS6_SaIS6_EEEESD_ET1_T_SF_T0_SG_SE_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::merge<std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt16__merge_backwardIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES9_SD_ET1_T_SF_T0_SG_SE_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::__merge_backward<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt11lower_boundIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES8_ET_SE_SE_RKT0_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::lower_bound<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> const&)"},
    {"_ZSt11upper_boundIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES8_ET_SE_SE_RKT0_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::upper_bound<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> const&)"},
    {"_ZSt17__rotate_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES9_lET_SE_SE_SE_T1_SF_T0_SF_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::__rotate_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, long, long, std::pair<llvm::APSInt, clang::CaseStmt*>*, long)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEEEvT_SE_SE_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::random_access_iterator_tag)"},
    {"_ZSt4swapISt4pairIN4llvm6APSIntEPN5clang8CaseStmtEEEvRT_S8_", "void std::swap<std::pair<llvm::APSInt, clang::CaseStmt*> >(std::pair<llvm::APSInt, clang::CaseStmt*>&, std::pair<llvm::APSInt, clang::CaseStmt*>&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESA_EET0_T_SC_SB_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*)"},
    {"_ZSt5mergeIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_N9__gnu_cxx17__normal_iteratorIS7_St6vectorIS6_SaIS6_EEEEET1_T_SF_T0_SG_SE_", "__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > std::merge<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt5mergeIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEESD_S9_ET1_T_SF_T0_SG_SE_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::merge<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>*)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEEEvT_SE_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt25__unguarded_linear_insertIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES8_EvT_T0_", "void std::__unguarded_linear_insert<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> >(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*>)"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEElEvT_SE_SE_T0_SF_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, long, long)"},
    {"_ZNSt17_Temporary_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESt6vectorIS8_SaIS8_EEEES8_EC2ESD_SD_", "std::_Temporary_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> >::_Temporary_buffer(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >)"},
    {"_ZSt21__inplace_stable_sortIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEEPFbRKS6_S9_EEvT_SC_T0_", "void std::__inplace_stable_sort<std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt22__stable_sort_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_lPFbRKS6_S9_EEvT_SC_T0_T1_T2_", "void std::__stable_sort_adaptive<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt24__merge_sort_with_bufferIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_PFbRKS6_S9_EEvT_SC_T0_T1_", "void std::__merge_sort_with_buffer<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt16__merge_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEElS7_PFbRKS6_S9_EEvT_SC_SC_T0_SD_T1_SD_T2_", "void std::__merge_adaptive<std::pair<llvm::APSInt, clang::CaseStmt*>*, long, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, long, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt5mergeIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_S7_PFbRKS6_S9_EET1_T_SD_T0_SE_SC_T2_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::merge<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt16__merge_backwardIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_S7_PFbRKS6_S9_EET1_T_SD_T0_SE_SC_T2_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::__merge_backward<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt17__rotate_adaptiveIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES7_lET_S8_S8_S8_T1_S9_T0_S9_", "std::pair<llvm::APSInt, clang::CaseStmt*>* std::__rotate_adaptive<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, long, std::pair<llvm::APSInt, clang::CaseStmt*>*, long)"},
    {"_ZSt8__rotateIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEEEvT_S8_S8_St26random_access_iterator_tag", "void std::__rotate<std::pair<llvm::APSInt, clang::CaseStmt*>*>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::random_access_iterator_tag)"},
    {"_ZSt16__insertion_sortIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEEPFbRKS6_S9_EEvT_SC_T0_", "void std::__insertion_sort<std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt25__unguarded_linear_insertIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES6_PFbRKS6_S9_EEvT_T0_T1_", "void std::__unguarded_linear_insert<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZSt22__merge_without_bufferIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEElPFbRKS6_S9_EEvT_SC_SC_T0_SD_T1_", "void std::__merge_without_buffer<std::pair<llvm::APSInt, clang::CaseStmt*>*, long, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&)>(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*, long, long, bool (*)(std::pair<llvm::APSInt, clang::CaseStmt*> const&, std::pair<llvm::APSInt, clang::CaseStmt*> const&))"},
    {"_ZNSt17_Temporary_bufferIPSt4pairIN4llvm6APSIntEPN5clang8CaseStmtEES6_EC2ES7_S7_", "std::_Temporary_buffer<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*> >::_Temporary_buffer(std::pair<llvm::APSInt, clang::CaseStmt*>*, std::pair<llvm::APSInt, clang::CaseStmt*>*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairINS_6APSIntEPN5clang8CaseStmtEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::APSInt, clang::CaseStmt*>, false>::grow(unsigned long)"},
    {"_ZNSt6vectorISt4pairIN4llvm6APSIntEPN5clang8CaseStmtEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::APSInt, clang::CaseStmt*>*, std::vector<std::pair<llvm::APSInt, clang::CaseStmt*>, std::allocator<std::pair<llvm::APSInt, clang::CaseStmt*> > > >, std::pair<llvm::APSInt, clang::CaseStmt*> const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang10TargetInfo14ConstraintInfoEED2Ev", "llvm::SmallVectorImpl<clang::TargetInfo::ConstraintInfo>::~SmallVectorImpl()"},
    {"_ZN5clang22getTemplateParamsRangeEPKPKNS_21TemplateParameterListEj", "clang::getTemplateParamsRange(clang::TemplateParameterList const* const*, unsigned int)"},
    {"_ZN5clang4Sema14isTemplateNameEPNS_5ScopeERNS_12CXXScopeSpecEbRNS_13UnqualifiedIdENS_9OpaquePtrINS_8QualTypeEEEbRNS7_INS_12TemplateNameEEERb", "clang::Sema::isTemplateName(clang::Scope*, clang::CXXScopeSpec&, bool, clang::UnqualifiedId&, clang::OpaquePtr<clang::QualType>, bool, clang::OpaquePtr<clang::TemplateName>&, bool&)"},
    {"_ZN5clang4Sema18LookupTemplateNameERNS_12LookupResultEPNS_5ScopeERNS_12CXXScopeSpecENS_8QualTypeEbRb", "clang::Sema::LookupTemplateName(clang::LookupResult&, clang::Scope*, clang::CXXScopeSpec&, clang::QualType, bool, bool&)"},
    {"_ZN5clang4Sema27DiagnoseUnknownTemplateNameERKNS_14IdentifierInfoENS_14SourceLocationEPNS_5ScopeEPKNS_12CXXScopeSpecERNS_9OpaquePtrINS_12TemplateNameEEERNS_16TemplateNameKindE", "clang::Sema::DiagnoseUnknownTemplateName(clang::IdentifierInfo const&, clang::SourceLocation, clang::Scope*, clang::CXXScopeSpec const*, clang::OpaquePtr<clang::TemplateName>&, clang::TemplateNameKind&)"},
    {"_Z29FilterAcceptableTemplateNamesRN5clang10ASTContextERNS_12LookupResultE", "FilterAcceptableTemplateNames(clang::ASTContext&, clang::LookupResult&)"},
    {"_ZN5clang4Sema26ActOnDependentIdExpressionERKNS_12CXXScopeSpecERKNS_19DeclarationNameInfoEbPKNS_24TemplateArgumentListInfoE", "clang::Sema::ActOnDependentIdExpression(clang::CXXScopeSpec const&, clang::DeclarationNameInfo const&, bool, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang4Sema25BuildDependentDeclRefExprERKNS_12CXXScopeSpecERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::Sema::BuildDependentDeclRefExpr(clang::CXXScopeSpec const&, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang4Sema31DiagnoseTemplateParameterShadowENS_14SourceLocationEPNS_4DeclE", "clang::Sema::DiagnoseTemplateParameterShadow(clang::SourceLocation, clang::Decl*)"},
    {"_ZN5clang4Sema20AdjustDeclIfTemplateERPNS_4DeclE", "clang::Sema::AdjustDeclIfTemplate(clang::Decl*&)"},
    {"_ZNK5clang22ParsedTemplateArgument24getTemplatePackExpansionENS_14SourceLocationE", "clang::ParsedTemplateArgument::getTemplatePackExpansion(clang::SourceLocation) const"},
    {"_ZN5clang4Sema26translateTemplateArgumentsERKNS_18ASTTemplateArgsPtrERNS_24TemplateArgumentListInfoE", "clang::Sema::translateTemplateArguments(clang::ASTTemplateArgsPtr const&, clang::TemplateArgumentListInfo&)"},
    {"_Z25translateTemplateArgumentRN5clang4SemaERKNS_22ParsedTemplateArgumentE", "translateTemplateArgument(clang::Sema&, clang::ParsedTemplateArgument const&)"},
    {"_ZN5clang4Sema18ActOnTypeParameterEPNS_5ScopeEbbNS_14SourceLocationES3_PNS_14IdentifierInfoES3_jjS3_NS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::ActOnTypeParameter(clang::Scope*, bool, bool, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, unsigned int, unsigned int, clang::SourceLocation, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang4Sema21CheckTemplateArgumentEPNS_20TemplateTypeParmDeclEPNS_14TypeSourceInfoE", "clang::Sema::CheckTemplateArgument(clang::TemplateTypeParmDecl*, clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema33CheckNonTypeTemplateParameterTypeENS_8QualTypeENS_14SourceLocationE", "clang::Sema::CheckNonTypeTemplateParameterType(clang::QualType, clang::SourceLocation)"},
    {"_ZN5clang4Sema29ActOnNonTypeTemplateParameterEPNS_5ScopeERNS_10DeclaratorEjjNS_14SourceLocationEPNS_4ExprE", "clang::Sema::ActOnNonTypeTemplateParameter(clang::Scope*, clang::Declarator&, unsigned int, unsigned int, clang::SourceLocation, clang::Expr*)"},
    {"_ZN5clang4Sema21CheckTemplateArgumentEPNS_23NonTypeTemplateParmDeclENS_8QualTypeERPNS_4ExprERNS_16TemplateArgumentENS0_25CheckTemplateArgumentKindE", "clang::Sema::CheckTemplateArgument(clang::NonTypeTemplateParmDecl*, clang::QualType, clang::Expr*&, clang::TemplateArgument&, clang::Sema::CheckTemplateArgumentKind)"},
    {"_ZN5clang4Sema30ActOnTemplateTemplateParameterEPNS_5ScopeENS_14SourceLocationEPNS_21TemplateParameterListES3_PNS_14IdentifierInfoES3_jjS3_NS_22ParsedTemplateArgumentE", "clang::Sema::ActOnTemplateTemplateParameter(clang::Scope*, clang::SourceLocation, clang::TemplateParameterList*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, unsigned int, unsigned int, clang::SourceLocation, clang::ParsedTemplateArgument)"},
    {"_ZN5clang4Sema26ActOnTemplateParameterListEjNS_14SourceLocationES1_S1_PPNS_4DeclEjS1_", "clang::Sema::ActOnTemplateParameterList(unsigned int, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::Decl**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang4Sema18CheckClassTemplateEPNS_5ScopeEjNS0_10TagUseKindENS_14SourceLocationERNS_12CXXScopeSpecEPNS_14IdentifierInfoES4_PNS_13AttributeListEPNS_21TemplateParameterListENS_15AccessSpecifierE", "clang::Sema::CheckClassTemplate(clang::Scope*, unsigned int, clang::Sema::TagUseKind, clang::SourceLocation, clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::SourceLocation, clang::AttributeList*, clang::TemplateParameterList*, clang::AccessSpecifier)"},
    {"_ZN5clang4Sema22CheckTemplateDeclScopeEPNS_5ScopeEPNS_21TemplateParameterListE", "clang::Sema::CheckTemplateDeclScope(clang::Scope*, clang::TemplateParameterList*)"},
    {"_ZN5clang4Sema30TemplateParameterListsAreEqualEPNS_21TemplateParameterListES2_bNS0_30TemplateParameterListEqualKindENS_14SourceLocationE", "clang::Sema::TemplateParameterListsAreEqual(clang::TemplateParameterList*, clang::TemplateParameterList*, bool, clang::Sema::TemplateParameterListEqualKind, clang::SourceLocation)"},
    {"_ZN5clang4Sema26CheckTemplateParameterListEPNS_21TemplateParameterListES2_NS0_24TemplateParamListContextE", "clang::Sema::CheckTemplateParameterList(clang::TemplateParameterList*, clang::TemplateParameterList*, clang::Sema::TemplateParamListContext)"},
    {"_Z32DiagnoseUnexpandedParameterPacksRN5clang4SemaEPNS_24TemplateTemplateParmDeclE", "DiagnoseUnexpandedParameterPacks(clang::Sema&, clang::TemplateTemplateParmDecl*)"},
    {"_Z31DiagnoseDefaultTemplateArgumentRN5clang4SemaENS0_24TemplateParamListContextENS_14SourceLocationENS_11SourceRangeE", "DiagnoseDefaultTemplateArgument(clang::Sema&, clang::Sema::TemplateParamListContext, clang::SourceLocation, clang::SourceRange)"},
    {"_ZN5clang4Sema39MatchTemplateParametersToScopeSpecifierENS_14SourceLocationERKNS_12CXXScopeSpecEPPNS_21TemplateParameterListEjbRbS8_", "clang::Sema::MatchTemplateParametersToScopeSpecifier(clang::SourceLocation, clang::CXXScopeSpec const&, clang::TemplateParameterList**, unsigned int, bool, bool&, bool&)"},
    {"_ZN5clang4Sema19CheckTemplateIdTypeENS_12TemplateNameENS_14SourceLocationERKNS_24TemplateArgumentListInfoE", "clang::Sema::CheckTemplateIdType(clang::TemplateName, clang::SourceLocation, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang4Sema25CheckTemplateArgumentListEPNS_12TemplateDeclENS_14SourceLocationERKNS_24TemplateArgumentListInfoEbRN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "clang::Sema::CheckTemplateArgumentList(clang::TemplateDecl*, clang::SourceLocation, clang::TemplateArgumentListInfo const&, bool, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_ZN5clang4Sema19ActOnTemplateIdTypeENS_9OpaquePtrINS_12TemplateNameEEENS_14SourceLocationES4_NS_18ASTTemplateArgsPtrES4_", "clang::Sema::ActOnTemplateIdType(clang::OpaquePtr<clang::TemplateName>, clang::SourceLocation, clang::SourceLocation, clang::ASTTemplateArgsPtr, clang::SourceLocation)"},
    {"_ZN5clang4Sema22ActOnTagTemplateIdTypeERNS_12CXXScopeSpecENS_12ActionResultINS_9OpaquePtrINS_8QualTypeEEELb0EEENS0_10TagUseKindENS_17TypeSpecifierTypeENS_14SourceLocationE", "clang::Sema::ActOnTagTemplateIdType(clang::CXXScopeSpec&, clang::ActionResult<clang::OpaquePtr<clang::QualType>, false>, clang::Sema::TagUseKind, clang::TypeSpecifierType, clang::SourceLocation)"},
    {"_ZN5clang4Sema19BuildTemplateIdExprERKNS_12CXXScopeSpecERNS_12LookupResultEbRKNS_24TemplateArgumentListInfoE", "clang::Sema::BuildTemplateIdExpr(clang::CXXScopeSpec const&, clang::LookupResult&, bool, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang4Sema28BuildQualifiedTemplateIdExprERNS_12CXXScopeSpecERKNS_19DeclarationNameInfoERKNS_24TemplateArgumentListInfoE", "clang::Sema::BuildQualifiedTemplateIdExpr(clang::CXXScopeSpec&, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang4Sema26ActOnDependentTemplateNameEPNS_5ScopeENS_14SourceLocationERNS_12CXXScopeSpecERNS_13UnqualifiedIdENS_9OpaquePtrINS_8QualTypeEEEbRNS8_INS_12TemplateNameEEE", "clang::Sema::ActOnDependentTemplateName(clang::Scope*, clang::SourceLocation, clang::CXXScopeSpec&, clang::UnqualifiedId&, clang::OpaquePtr<clang::QualType>, bool, clang::OpaquePtr<clang::TemplateName>&)"},
    {"_ZN5clang4Sema25CheckTemplateTypeArgumentEPNS_20TemplateTypeParmDeclERKNS_19TemplateArgumentLocERN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "clang::Sema::CheckTemplateTypeArgument(clang::TemplateTypeParmDecl*, clang::TemplateArgumentLoc const&, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_ZN5clang4Sema39SubstDefaultTemplateArgumentIfAvailableEPNS_12TemplateDeclENS_14SourceLocationES3_PNS_4DeclERN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "clang::Sema::SubstDefaultTemplateArgumentIfAvailable(clang::TemplateDecl*, clang::SourceLocation, clang::SourceLocation, clang::Decl*, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_Z28SubstDefaultTemplateArgumentRN5clang4SemaEPNS_12TemplateDeclENS_14SourceLocationES4_PNS_20TemplateTypeParmDeclERN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "SubstDefaultTemplateArgument(clang::Sema&, clang::TemplateDecl*, clang::SourceLocation, clang::SourceLocation, clang::TemplateTypeParmDecl*, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_Z28SubstDefaultTemplateArgumentRN5clang4SemaEPNS_12TemplateDeclENS_14SourceLocationES4_PNS_23NonTypeTemplateParmDeclERN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "SubstDefaultTemplateArgument(clang::Sema&, clang::TemplateDecl*, clang::SourceLocation, clang::SourceLocation, clang::NonTypeTemplateParmDecl*, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_Z28SubstDefaultTemplateArgumentRN5clang4SemaEPNS_12TemplateDeclENS_14SourceLocationES4_PNS_24TemplateTemplateParmDeclERN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "SubstDefaultTemplateArgument(clang::Sema&, clang::TemplateDecl*, clang::SourceLocation, clang::SourceLocation, clang::TemplateTemplateParmDecl*, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_ZN5clang4Sema21CheckTemplateArgumentEPNS_9NamedDeclERKNS_19TemplateArgumentLocES2_NS_14SourceLocationES6_jRN4llvm15SmallVectorImplINS_16TemplateArgumentEEENS0_25CheckTemplateArgumentKindE", "clang::Sema::CheckTemplateArgument(clang::NamedDecl*, clang::TemplateArgumentLoc const&, clang::NamedDecl*, clang::SourceLocation, clang::SourceLocation, unsigned int, llvm::SmallVectorImpl<clang::TemplateArgument>&, clang::Sema::CheckTemplateArgumentKind)"},
    {"_ZN5clang4Sema21CheckTemplateArgumentEPNS_24TemplateTemplateParmDeclERKNS_19TemplateArgumentLocE", "clang::Sema::CheckTemplateArgument(clang::TemplateTemplateParmDecl*, clang::TemplateArgumentLoc const&)"},
    {"_ZN5clang4Sema36CheckTemplateArgumentPointerToMemberEPNS_4ExprERNS_16TemplateArgumentE", "clang::Sema::CheckTemplateArgumentPointerToMember(clang::Expr*, clang::TemplateArgument&)"},
    {"_Z46CheckTemplateArgumentAddressOfObjectOrFunctionRN5clang4SemaEPNS_23NonTypeTemplateParmDeclENS_8QualTypeEPNS_4ExprERNS_16TemplateArgumentE", "CheckTemplateArgumentAddressOfObjectOrFunction(clang::Sema&, clang::NonTypeTemplateParmDecl*, clang::QualType, clang::Expr*, clang::TemplateArgument&)"},
    {"_ZN5clang4Sema39BuildExpressionFromDeclTemplateArgumentERKNS_16TemplateArgumentENS_8QualTypeENS_14SourceLocationE", "clang::Sema::BuildExpressionFromDeclTemplateArgument(clang::TemplateArgument const&, clang::QualType, clang::SourceLocation)"},
    {"_ZN5clang4Sema43BuildExpressionFromIntegralTemplateArgumentERKNS_16TemplateArgumentENS_14SourceLocationE", "clang::Sema::BuildExpressionFromIntegralTemplateArgument(clang::TemplateArgument const&, clang::SourceLocation)"},
    {"_Z42DiagnoseTemplateParameterListArityMismatchRN5clang4SemaEPNS_21TemplateParameterListES3_NS0_30TemplateParameterListEqualKindENS_14SourceLocationE", "DiagnoseTemplateParameterListArityMismatch(clang::Sema&, clang::TemplateParameterList*, clang::TemplateParameterList*, clang::Sema::TemplateParameterListEqualKind, clang::SourceLocation)"},
    {"_Z26MatchTemplateParameterKindRN5clang4SemaEPNS_9NamedDeclES3_bNS0_30TemplateParameterListEqualKindENS_14SourceLocationE", "MatchTemplateParameterKind(clang::Sema&, clang::NamedDecl*, clang::NamedDecl*, bool, clang::Sema::TemplateParameterListEqualKind, clang::SourceLocation)"},
    {"_ZN5clang4Sema32ActOnClassTemplateSpecializationEPNS_5ScopeEjNS0_10TagUseKindENS_14SourceLocationERNS_12CXXScopeSpecENS_9OpaquePtrINS_12TemplateNameEEES4_S4_NS_18ASTTemplateArgsPtrES4_PNS_13AttributeListENS_11ASTMultiPtrIPNS_21TemplateParameterListEEE", "clang::Sema::ActOnClassTemplateSpecialization(clang::Scope*, unsigned int, clang::Sema::TagUseKind, clang::SourceLocation, clang::CXXScopeSpec&, clang::OpaquePtr<clang::TemplateName>, clang::SourceLocation, clang::SourceLocation, clang::ASTTemplateArgsPtr, clang::SourceLocation, clang::AttributeList*, clang::ASTMultiPtr<clang::TemplateParameterList*>)"},
    {"_Z32CheckTemplateSpecializationScopeRN5clang4SemaEPNS_9NamedDeclES3_NS_14SourceLocationEb", "CheckTemplateSpecializationScope(clang::Sema&, clang::NamedDecl*, clang::NamedDecl*, clang::SourceLocation, bool)"},
    {"_Z15getPreviousDeclPN5clang9NamedDeclE", "getPreviousDecl(clang::NamedDecl*)"},
    {"_ZN5clang4Sema23ActOnTemplateDeclaratorEPNS_5ScopeENS_11ASTMultiPtrIPNS_21TemplateParameterListEEERNS_10DeclaratorE", "clang::Sema::ActOnTemplateDeclarator(clang::Scope*, clang::ASTMultiPtr<clang::TemplateParameterList*>, clang::Declarator&)"},
    {"_ZN5clang4Sema31ActOnStartOfFunctionTemplateDefEPNS_5ScopeENS_11ASTMultiPtrIPNS_21TemplateParameterListEEERNS_10DeclaratorE", "clang::Sema::ActOnStartOfFunctionTemplateDef(clang::Scope*, clang::ASTMultiPtr<clang::TemplateParameterList*>, clang::Declarator&)"},
    {"_ZN5clang4Sema38CheckSpecializationInstantiationRedeclENS_14SourceLocationENS_26TemplateSpecializationKindEPNS_9NamedDeclES2_S1_Rb", "clang::Sema::CheckSpecializationInstantiationRedecl(clang::SourceLocation, clang::TemplateSpecializationKind, clang::NamedDecl*, clang::TemplateSpecializationKind, clang::SourceLocation, bool&)"},
    {"_ZN5clang4Sema44CheckDependentFunctionTemplateSpecializationEPNS_12FunctionDeclERKNS_24TemplateArgumentListInfoERNS_12LookupResultE", "clang::Sema::CheckDependentFunctionTemplateSpecialization(clang::FunctionDecl*, clang::TemplateArgumentListInfo const&, clang::LookupResult&)"},
    {"_ZN5clang4Sema35CheckFunctionTemplateSpecializationEPNS_12FunctionDeclEPKNS_24TemplateArgumentListInfoERNS_12LookupResultE", "clang::Sema::CheckFunctionTemplateSpecialization(clang::FunctionDecl*, clang::TemplateArgumentListInfo const*, clang::LookupResult&)"},
    {"_ZN5clang4Sema25CheckMemberSpecializationEPNS_9NamedDeclERNS_12LookupResultE", "clang::Sema::CheckMemberSpecialization(clang::NamedDecl*, clang::LookupResult&)"},
    {"_ZN5clang4Sema26ActOnExplicitInstantiationEPNS_5ScopeENS_14SourceLocationES3_jS3_RKNS_12CXXScopeSpecENS_9OpaquePtrINS_12TemplateNameEEES3_S3_NS_18ASTTemplateArgsPtrES3_PNS_13AttributeListE", "clang::Sema::ActOnExplicitInstantiation(clang::Scope*, clang::SourceLocation, clang::SourceLocation, unsigned int, clang::SourceLocation, clang::CXXScopeSpec const&, clang::OpaquePtr<clang::TemplateName>, clang::SourceLocation, clang::SourceLocation, clang::ASTTemplateArgsPtr, clang::SourceLocation, clang::AttributeList*)"},
    {"_Z31CheckExplicitInstantiationScopeRN5clang4SemaEPNS_9NamedDeclENS_14SourceLocationEb", "CheckExplicitInstantiationScope(clang::Sema&, clang::NamedDecl*, clang::SourceLocation, bool)"},
    {"_ZN5clang4Sema26ActOnExplicitInstantiationEPNS_5ScopeENS_14SourceLocationES3_jS3_RNS_12CXXScopeSpecEPNS_14IdentifierInfoES3_PNS_13AttributeListE", "clang::Sema::ActOnExplicitInstantiation(clang::Scope*, clang::SourceLocation, clang::SourceLocation, unsigned int, clang::SourceLocation, clang::CXXScopeSpec&, clang::IdentifierInfo*, clang::SourceLocation, clang::AttributeList*)"},
    {"_ZN5clang4Sema26ActOnExplicitInstantiationEPNS_5ScopeENS_14SourceLocationES3_RNS_10DeclaratorE", "clang::Sema::ActOnExplicitInstantiation(clang::Scope*, clang::SourceLocation, clang::SourceLocation, clang::Declarator&)"},
    {"_ZN5clang4Sema17ActOnDependentTagEPNS_5ScopeEjNS0_10TagUseKindERKNS_12CXXScopeSpecEPNS_14IdentifierInfoENS_14SourceLocationES9_", "clang::Sema::ActOnDependentTag(clang::Scope*, unsigned int, clang::Sema::TagUseKind, clang::CXXScopeSpec const&, clang::IdentifierInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang4Sema17ActOnTypenameTypeEPNS_5ScopeENS_14SourceLocationERKNS_12CXXScopeSpecERKNS_14IdentifierInfoES3_", "clang::Sema::ActOnTypenameType(clang::Scope*, clang::SourceLocation, clang::CXXScopeSpec const&, clang::IdentifierInfo const&, clang::SourceLocation)"},
    {"_ZN5clang4Sema17CheckTypenameTypeENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierERKNS_14IdentifierInfoENS_14SourceLocationENS_11SourceRangeES7_", "clang::Sema::CheckTypenameType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const&, clang::SourceLocation, clang::SourceRange, clang::SourceLocation)"},
    {"_ZN5clang4Sema17ActOnTypenameTypeEPNS_5ScopeENS_14SourceLocationERKNS_12CXXScopeSpecES3_NS_9OpaquePtrINS_8QualTypeEEE", "clang::Sema::ActOnTypenameType(clang::Scope*, clang::SourceLocation, clang::CXXScopeSpec const&, clang::SourceLocation, clang::OpaquePtr<clang::QualType>)"},
    {"_ZN5clang4Sema33RebuildTypeInCurrentInstantiationEPNS_14TypeSourceInfoENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::RebuildTypeInCurrentInstantiation(clang::TypeSourceInfo*, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE13TransformTypeEPNS_14TypeSourceInfoE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformType(clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema33RebuildExprInCurrentInstantiationEPNS_4ExprE", "clang::Sema::RebuildExprInCurrentInstantiation(clang::Expr*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE13TransformExprEPNS_4ExprE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformExpr(clang::Expr*)"},
    {"_ZN5clang4Sema48RebuildNestedNameSpecifierInCurrentInstantiationERNS_12CXXScopeSpecE", "clang::Sema::RebuildNestedNameSpecifierInCurrentInstantiation(clang::CXXScopeSpec&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE28TransformNestedNameSpecifierEPNS_19NestedNameSpecifierENS_11SourceRangeENS_8QualTypeEPNS_9NamedDeclE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformNestedNameSpecifier(clang::NestedNameSpecifier*, clang::SourceRange, clang::QualType, clang::NamedDecl*)"},
    {"_ZN5clang4Sema31getTemplateArgumentBindingsTextEPKNS_21TemplateParameterListERKNS_20TemplateArgumentListE", "clang::Sema::getTemplateArgumentBindingsText(clang::TemplateParameterList const*, clang::TemplateArgumentList const&)"},
    {"_ZN5clang4Sema31getTemplateArgumentBindingsTextEPKNS_21TemplateParameterListEPKNS_16TemplateArgumentEj", "clang::Sema::getTemplateArgumentBindingsText(clang::TemplateParameterList const*, clang::TemplateArgument const*, unsigned int)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE26TransformTypeInObjectScopeEPNS_14TypeSourceInfoENS_8QualTypeEPNS_9NamedDeclEPNS_19NestedNameSpecifierE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTypeInObjectScope(clang::TypeSourceInfo*, clang::QualType, clang::NamedDecl*, clang::NestedNameSpecifier*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE21TransformTemplateNameENS_12TemplateNameENS_8QualTypeEPNS_9NamedDeclE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTemplateName(clang::TemplateName, clang::QualType, clang::NamedDecl*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE35TransformTemplateSpecializationTypeERNS_14TypeLocBuilderENS_29TemplateSpecializationTypeLocENS_12TemplateNameE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTemplateSpecializationType(clang::TypeLocBuilder&, clang::TemplateSpecializationTypeLoc, clang::TemplateName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE44TransformDependentTemplateSpecializationTypeERNS_14TypeLocBuilderENS_38DependentTemplateSpecializationTypeLocEPNS_19NestedNameSpecifierE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformDependentTemplateSpecializationType(clang::TypeLocBuilder&, clang::DependentTemplateSpecializationTypeLoc, clang::NestedNameSpecifier*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE13TransformTypeERNS_14TypeLocBuilderENS_7TypeLocE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformType(clang::TypeLocBuilder&, clang::TypeLoc)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE27TransformFunctionTypeParamsENS_14SourceLocationEPPNS_11ParmVarDeclEjPKNS_8QualTypeERN4llvm15SmallVectorImplIS8_EEPNSC_IS6_EE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformFunctionTypeParams(clang::SourceLocation, clang::ParmVarDecl**, unsigned int, clang::QualType const*, llvm::SmallVectorImpl<clang::QualType>&, llvm::SmallVectorImpl<clang::ParmVarDecl*>*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE26TransformFunctionTypeParamEPNS_11ParmVarDeclEN4llvm8OptionalIjEE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformFunctionTypeParam(clang::ParmVarDecl*, llvm::Optional<unsigned int>)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE16RebuildArrayTypeENS_8QualTypeENS_9ArrayType17ArraySizeModifierEPKN4llvm5APIntEPNS_4ExprEjNS_11SourceRangeE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::RebuildArrayType(clang::QualType, clang::ArrayType::ArraySizeModifier, llvm::APInt const*, clang::Expr*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE22TransformReferenceTypeERNS_14TypeLocBuilderENS_16ReferenceTypeLocE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformReferenceType(clang::TypeLocBuilder&, clang::ReferenceTypeLoc)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE26TransformTemplateArgumentsINS_33TemplateArgumentLocInventIteratorIS2_PKNS_16TemplateArgumentEEEEEbT_SA_RNS_24TemplateArgumentListInfoE", "bool clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTemplateArguments<clang::TemplateArgumentLocInventIterator<(anonymous namespace)::CurrentInstantiationRebuilder, clang::TemplateArgument const*> >(clang::TemplateArgumentLocInventIterator<(anonymous namespace)::CurrentInstantiationRebuilder, clang::TemplateArgument const*>, clang::TemplateArgumentLocInventIterator<(anonymous namespace)::CurrentInstantiationRebuilder, clang::TemplateArgument const*>, clang::TemplateArgumentListInfo&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE25TransformTemplateArgumentERKNS_19TemplateArgumentLocERS4_", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTemplateArgument(clang::TemplateArgumentLoc const&, clang::TemplateArgumentLoc&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE20RebuildPackExpansionENS_19TemplateArgumentLocENS_14SourceLocationEN4llvm8OptionalIjEE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::RebuildPackExpansion(clang::TemplateArgumentLoc, clang::SourceLocation, llvm::Optional<unsigned int>)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE25InventTemplateArgumentLocERKNS_16TemplateArgumentERNS_19TemplateArgumentLocE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::InventTemplateArgumentLoc(clang::TemplateArgument const&, clang::TemplateArgumentLoc&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE23TransformBinaryOperatorEPNS_14BinaryOperatorE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE17TransformCallExprEPNS_8CallExprE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformCallExpr(clang::CallExpr*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE21TransformCompoundStmtEPNS_12CompoundStmtEb", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformCompoundStmt(clang::CompoundStmt*, bool)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE13TransformStmtEPNS_4StmtE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformStmt(clang::Stmt*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE21TransformCXXCatchStmtEPNS_12CXXCatchStmtE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE14TransformExprsEPPNS_4ExprEjbRN4llvm15SmallVectorImplIS5_EEPb", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformExprs(clang::Expr**, unsigned int, bool, llvm::SmallVectorImpl<clang::Expr*>&, bool*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE26TransformTemplateArgumentsIPKNS_19TemplateArgumentLocEEEbT_S8_RNS_24TemplateArgumentListInfoE", "bool clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformTemplateArguments<clang::TemplateArgumentLoc const*>(clang::TemplateArgumentLoc const*, clang::TemplateArgumentLoc const*, clang::TemplateArgumentListInfo&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE28TransformDeclarationNameInfoERKNS_19DeclarationNameInfoE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformDeclarationNameInfo(clang::DeclarationNameInfo const&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_129CurrentInstantiationRebuilderEE25TransformCXXNamedCastExprEPNS_16CXXNamedCastExprE", "clang::TreeTransform<(anonymous namespace)::CurrentInstantiationRebuilder>::TransformCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_Z50CheckNonTypeClassTemplatePartialSpecializationArgsRN5clang4SemaEPNS_23NonTypeTemplateParmDeclEPKNS_16TemplateArgumentEj", "CheckNonTypeClassTemplatePartialSpecializationArgs(clang::Sema&, clang::NonTypeTemplateParmDecl*, clang::TemplateArgument const*, unsigned int)"},
    {"_ZN5clang11TypeVisitorIN12_GLOBAL__N_127UnnamedLocalNoLinkageFinderEbE5VisitEPKNS_4TypeE", "clang::TypeVisitor<(anonymous namespace)::UnnamedLocalNoLinkageFinder, bool>::Visit(clang::Type const*)"},
    {"_ZN12_GLOBAL__N_127UnnamedLocalNoLinkageFinder24VisitNestedNameSpecifierEPN5clang19NestedNameSpecifierE", "(anonymous namespace)::UnnamedLocalNoLinkageFinder::VisitNestedNameSpecifier(clang::NestedNameSpecifier*)"},
    {"_ZN12_GLOBAL__N_127UnnamedLocalNoLinkageFinder12VisitTagDeclEPKN5clang7TagDeclE", "(anonymous namespace)::UnnamedLocalNoLinkageFinder::VisitTagDecl(clang::TagDecl const*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE12TraverseTypeENS_8QualTypeE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseType(clang::QualType)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE27TraverseNestedNameSpecifierEPNS_19NestedNameSpecifierE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseNestedNameSpecifier(clang::NestedNameSpecifier*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE24TraverseTemplateArgumentERKNS_16TemplateArgumentE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE12TraverseStmtEPNS_4StmtE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseStmt(clang::Stmt*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE15TraverseTypeLocENS_7TypeLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseTypeLoc(clang::TypeLoc)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE27TraverseTemplateArgumentLocERKNS_19TemplateArgumentLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseTemplateArgumentLoc(clang::TemplateArgumentLoc const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE12TraverseDeclEPNS_4DeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseDecl(clang::Decl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE24TraverseDeclaratorHelperEPNS_14DeclaratorDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseDeclaratorHelper(clang::DeclaratorDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE22TraverseFunctionHelperEPNS_12FunctionDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseFunctionHelper(clang::FunctionDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_117DependencyCheckerEE23TraverseCXXRecordHelperEPNS_13CXXRecordDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::DependencyChecker>::TraverseCXXRecordHelper(clang::CXXRecordDecl*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang19TemplateArgumentLocEEaSERKS3_", "llvm::SmallVectorImpl<clang::TemplateArgumentLoc>::operator=(llvm::SmallVectorImpl<clang::TemplateArgumentLoc> const&)"},
    {"_ZSt24__uninitialized_copy_auxIPKN5clang19TemplateArgumentLocEPS1_ET0_T_S6_S5_St12__false_type", "clang::TemplateArgumentLoc* std::__uninitialized_copy_aux<clang::TemplateArgumentLoc const*, clang::TemplateArgumentLoc*>(clang::TemplateArgumentLoc const*, clang::TemplateArgumentLoc const*, clang::TemplateArgumentLoc*, std::__false_type)"},
    {"_ZN5clang4Sema23DeduceTemplateArgumentsEPNS_38ClassTemplatePartialSpecializationDeclERKNS_20TemplateArgumentListERNS_4sema21TemplateDeductionInfoE", "clang::Sema::DeduceTemplateArguments(clang::ClassTemplatePartialSpecializationDecl*, clang::TemplateArgumentList const&, clang::sema::TemplateDeductionInfo&)"},
    {"_Z31FinishTemplateArgumentDeductionRN5clang4SemaEPNS_38ClassTemplatePartialSpecializationDeclERKNS_20TemplateArgumentListERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEERNS_4sema21TemplateDeductionInfoE", "FinishTemplateArgumentDeduction(clang::Sema&, clang::ClassTemplatePartialSpecializationDecl*, clang::TemplateArgumentList const&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema35SubstituteExplicitTemplateArgumentsEPNS_20FunctionTemplateDeclERKNS_24TemplateArgumentListInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEERNS7_INS_8QualTypeEEEPSB_RNS_4sema21TemplateDeductionInfoE", "clang::Sema::SubstituteExplicitTemplateArguments(clang::FunctionTemplateDecl*, clang::TemplateArgumentListInfo const&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, llvm::SmallVectorImpl<clang::QualType>&, clang::QualType*, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema31FinishTemplateArgumentDeductionEPNS_20FunctionTemplateDeclERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEEjRPNS_12FunctionDeclERNS_4sema21TemplateDeductionInfoE", "clang::Sema::FinishTemplateArgumentDeduction(clang::FunctionTemplateDecl*, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, unsigned int, clang::FunctionDecl*&, clang::sema::TemplateDeductionInfo&)"},
    {"_Z30ConvertDeducedTemplateArgumentRN5clang4SemaEPNS_9NamedDeclENS_23DeducedTemplateArgumentES3_NS_8QualTypeEjRNS_4sema21TemplateDeductionInfoEbRN4llvm15SmallVectorImplINS_16TemplateArgumentEEE", "ConvertDeducedTemplateArgument(clang::Sema&, clang::NamedDecl*, clang::DeducedTemplateArgument, clang::NamedDecl*, clang::QualType, unsigned int, clang::sema::TemplateDeductionInfo&, bool, llvm::SmallVectorImpl<clang::TemplateArgument>&)"},
    {"_ZN5clang4Sema23DeduceTemplateArgumentsEPNS_20FunctionTemplateDeclEPKNS_24TemplateArgumentListInfoEPPNS_4ExprEjRPNS_12FunctionDeclERNS_4sema21TemplateDeductionInfoE", "clang::Sema::DeduceTemplateArguments(clang::FunctionTemplateDecl*, clang::TemplateArgumentListInfo const*, clang::Expr**, unsigned int, clang::FunctionDecl*&, clang::sema::TemplateDeductionInfo&)"},
    {"_Z41AdjustFunctionParmAndArgTypesForDeductionRN5clang4SemaEPNS_21TemplateParameterListERNS_8QualTypeES5_PNS_4ExprERj", "AdjustFunctionParmAndArgTypesForDeduction(clang::Sema&, clang::TemplateParameterList*, clang::QualType&, clang::QualType&, clang::Expr*, unsigned int&)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListENS_8QualTypeES4_RNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEEjbPNS9_I33RefParamPartialOrderingComparisonEE", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::QualType, clang::QualType, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, unsigned int, bool, llvm::SmallVectorImpl<RefParamPartialOrderingComparison>*)"},
    {"_Z28PrepareArgumentPackDeductionRN5clang4SemaERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEERKNS3_IjEES6_RNS3_INS2_11SmallVectorIS4_Lj4EEEEE", "PrepareArgumentPackDeduction(clang::Sema&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, llvm::SmallVectorImpl<unsigned int> const&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, llvm::SmallVectorImpl<llvm::SmallVector<clang::DeducedTemplateArgument, 4u> >&)"},
    {"_Z27FinishArgumentPackDeductionRN5clang4SemaEPNS_21TemplateParameterListEbRN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEERKNS5_IjEES8_RNS5_INS4_11SmallVectorIS6_Lj4EEEEERNS_4sema21TemplateDeductionInfoE", "FinishArgumentPackDeduction(clang::Sema&, clang::TemplateParameterList*, bool, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, llvm::SmallVectorImpl<unsigned int> const&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, llvm::SmallVectorImpl<llvm::SmallVector<clang::DeducedTemplateArgument, 4u> >&, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema23DeduceTemplateArgumentsEPNS_20FunctionTemplateDeclEPKNS_24TemplateArgumentListInfoENS_8QualTypeERPNS_12FunctionDeclERNS_4sema21TemplateDeductionInfoE", "clang::Sema::DeduceTemplateArguments(clang::FunctionTemplateDecl*, clang::TemplateArgumentListInfo const*, clang::QualType, clang::FunctionDecl*&, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema23DeduceTemplateArgumentsEPNS_20FunctionTemplateDeclENS_8QualTypeERPNS_17CXXConversionDeclERNS_4sema21TemplateDeductionInfoE", "clang::Sema::DeduceTemplateArguments(clang::FunctionTemplateDecl*, clang::QualType, clang::CXXConversionDecl*&, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema23DeduceTemplateArgumentsEPNS_20FunctionTemplateDeclEPKNS_24TemplateArgumentListInfoERPNS_12FunctionDeclERNS_4sema21TemplateDeductionInfoE", "clang::Sema::DeduceTemplateArguments(clang::FunctionTemplateDecl*, clang::TemplateArgumentListInfo const*, clang::FunctionDecl*&, clang::sema::TemplateDeductionInfo&)"},
    {"_ZN5clang4Sema26getMoreSpecializedTemplateEPNS_20FunctionTemplateDeclES2_NS_14SourceLocationENS_30TemplatePartialOrderingContextEj", "clang::Sema::getMoreSpecializedTemplate(clang::FunctionTemplateDecl*, clang::FunctionTemplateDecl*, clang::SourceLocation, clang::TemplatePartialOrderingContext, unsigned int)"},
    {"_Z24isAtLeastAsSpecializedAsRN5clang4SemaENS_14SourceLocationEPNS_20FunctionTemplateDeclES4_NS_30TemplatePartialOrderingContextEjPN4llvm15SmallVectorImplI33RefParamPartialOrderingComparisonEE", "isAtLeastAsSpecializedAs(clang::Sema&, clang::SourceLocation, clang::FunctionTemplateDecl*, clang::FunctionTemplateDecl*, clang::TemplatePartialOrderingContext, unsigned int, llvm::SmallVectorImpl<RefParamPartialOrderingComparison>*)"},
    {"_ZN5clang4Sema18getMostSpecializedENS_21UnresolvedSetIteratorES1_NS_30TemplatePartialOrderingContextEjNS_14SourceLocationERKNS_17PartialDiagnosticES6_S6_", "clang::Sema::getMostSpecialized(clang::UnresolvedSetIterator, clang::UnresolvedSetIterator, clang::TemplatePartialOrderingContext, unsigned int, clang::SourceLocation, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema39getMoreSpecializedPartialSpecializationEPNS_38ClassTemplatePartialSpecializationDeclES2_NS_14SourceLocationE", "clang::Sema::getMoreSpecializedPartialSpecialization(clang::ClassTemplatePartialSpecializationDecl*, clang::ClassTemplatePartialSpecializationDecl*, clang::SourceLocation)"},
    {"_ZN5clang4Sema26MarkUsedTemplateParametersERKNS_20TemplateArgumentListEbjRN4llvm15SmallVectorImplIbEE", "clang::Sema::MarkUsedTemplateParameters(clang::TemplateArgumentList const&, bool, unsigned int, llvm::SmallVectorImpl<bool>&)"},
    {"_Z26MarkUsedTemplateParametersRN5clang4SemaERKNS_16TemplateArgumentEbjRN4llvm15SmallVectorImplIbEE", "MarkUsedTemplateParameters(clang::Sema&, clang::TemplateArgument const&, bool, unsigned int, llvm::SmallVectorImpl<bool>&)"},
    {"_ZN5clang4Sema29MarkDeducedTemplateParametersEPNS_20FunctionTemplateDeclERN4llvm15SmallVectorImplIbEE", "clang::Sema::MarkDeducedTemplateParameters(clang::FunctionTemplateDecl*, llvm::SmallVectorImpl<bool>&)"},
    {"_Z26MarkUsedTemplateParametersRN5clang4SemaENS_8QualTypeEbjRN4llvm15SmallVectorImplIbEE", "MarkUsedTemplateParameters(clang::Sema&, clang::QualType, bool, unsigned int, llvm::SmallVectorImpl<bool>&)"},
    {"_Z26MarkUsedTemplateParametersRN5clang4SemaENS_12TemplateNameEbjRN4llvm15SmallVectorImplIbEE", "MarkUsedTemplateParameters(clang::Sema&, clang::TemplateName, bool, unsigned int, llvm::SmallVectorImpl<bool>&)"},
    {"_Z26MarkUsedTemplateParametersRN5clang4SemaEPNS_19NestedNameSpecifierEbjRN4llvm15SmallVectorImplIbEE", "MarkUsedTemplateParameters(clang::Sema&, clang::NestedNameSpecifier*, bool, unsigned int, llvm::SmallVectorImpl<bool>&)"},
    {"_Z35MaybeAddImplicitObjectParameterTypeRN5clang10ASTContextEPNS_13CXXMethodDeclERN4llvm15SmallVectorImplINS_8QualTypeEEE", "MaybeAddImplicitObjectParameterType(clang::ASTContext&, clang::CXXMethodDecl*, llvm::SmallVectorImpl<clang::QualType>&)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListEPKNS_8QualTypeEjS6_jRNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEEjbPNSB_I33RefParamPartialOrderingComparisonEE", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::QualType const*, unsigned int, clang::QualType const*, unsigned int, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, unsigned int, bool, llvm::SmallVectorImpl<RefParamPartialOrderingComparison>*)"},
    {"_Z29checkDeducedTemplateArgumentsRN5clang10ASTContextERKNS_23DeducedTemplateArgumentES4_", "checkDeducedTemplateArguments(clang::ASTContext&, clang::DeducedTemplateArgument const&, clang::DeducedTemplateArgument const&)"},
    {"_Z20hasSameExtendedValueN4llvm6APSIntES0_", "hasSameExtendedValue(llvm::APSInt, llvm::APSInt)"},
    {"_Z29DeduceNonTypeTemplateArgumentRN5clang4SemaEPNS_23NonTypeTemplateParmDeclEN4llvm6APSIntENS_8QualTypeEbRNS_4sema21TemplateDeductionInfoERNS4_15SmallVectorImplINS_23DeducedTemplateArgumentEEE", "DeduceNonTypeTemplateArgument(clang::Sema&, clang::NonTypeTemplateParmDecl*, llvm::APSInt, clang::QualType, bool, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&)"},
    {"_Z29DeduceNonTypeTemplateArgumentRN5clang4SemaEPNS_23NonTypeTemplateParmDeclEPNS_4ExprERNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEE", "DeduceNonTypeTemplateArgument(clang::Sema&, clang::NonTypeTemplateParmDecl*, clang::Expr*, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListEPKNS_26TemplateSpecializationTypeENS_8QualTypeERNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEE", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::TemplateSpecializationType const*, clang::QualType, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListENS_12TemplateNameES4_RNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEE", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::TemplateName, clang::TemplateName, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListEPKNS_16TemplateArgumentEjS6_jRNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEEb", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::TemplateArgument const*, unsigned int, clang::TemplateArgument const*, unsigned int, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&, bool)"},
    {"_Z23DeduceTemplateArgumentsRN5clang4SemaEPNS_21TemplateParameterListERKNS_16TemplateArgumentES4_RNS_4sema21TemplateDeductionInfoERN4llvm15SmallVectorImplINS_23DeducedTemplateArgumentEEE", "DeduceTemplateArguments(clang::Sema&, clang::TemplateParameterList*, clang::TemplateArgument const&, clang::TemplateArgument, clang::sema::TemplateDeductionInfo&, llvm::SmallVectorImpl<clang::DeducedTemplateArgument>&)"},
    {"_Z17GetTypeOfFunctionRN5clang10ASTContextERKNS_12OverloadExpr10FindResultEPNS_12FunctionDeclE", "GetTypeOfFunction(clang::ASTContext&, clang::OverloadExpr::FindResult const&, clang::FunctionDecl*)"},
    {"_Z17isSameTemplateArgRN5clang10ASTContextERKNS_16TemplateArgumentES4_", "isSameTemplateArg(clang::ASTContext&, clang::TemplateArgument const&, clang::TemplateArgument const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23DeducedTemplateArgumentEE6resizeEj", "llvm::SmallVectorImpl<clang::DeducedTemplateArgument>::resize(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23DeducedTemplateArgumentEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::DeducedTemplateArgument>::push_back(clang::DeducedTemplateArgument const&)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIN5clang14SourceLocationENS2_17PartialDiagnosticEEE6appendIPKS5_EEvT_SA_", "void llvm::SmallVectorImpl<std::pair<clang::SourceLocation, clang::PartialDiagnostic> >::append<std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*>(std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*, std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclENS_11SmallVectorISt4pairINS1_14SourceLocationENS1_17PartialDiagnosticEELj1EEENS_12DenseMapInfoIS3_EENSA_IS9_EEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::Decl*, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> > >::FindAndConstruct(clang::Decl* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclENS_11SmallVectorISt4pairINS1_14SourceLocationENS1_17PartialDiagnosticEELj1EEENS_12DenseMapInfoIS3_EENSA_IS9_EEE16InsertIntoBucketERKS3_RKS9_PS5_IS3_S9_E", "llvm::DenseMap<clang::Decl*, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> > >::InsertIntoBucket(clang::Decl* const&, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> const&, std::pair<clang::Decl*, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclENS_11SmallVectorISt4pairINS1_14SourceLocationENS1_17PartialDiagnosticEELj1EEENS_12DenseMapInfoIS3_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::Decl*, llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<clang::SourceLocation, clang::PartialDiagnostic>, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIN5clang14SourceLocationENS2_17PartialDiagnosticEEEaSERKS6_", "llvm::SmallVectorImpl<std::pair<clang::SourceLocation, clang::PartialDiagnostic> >::operator=(llvm::SmallVectorImpl<std::pair<clang::SourceLocation, clang::PartialDiagnostic> > const&)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPKSt4pairIN5clang14SourceLocationENS4_17PartialDiagnosticEEPS7_EET0_T_SC_SB_", "std::pair<clang::SourceLocation, clang::PartialDiagnostic>* std::__copy<false, std::random_access_iterator_tag>::copy<std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*, std::pair<clang::SourceLocation, clang::PartialDiagnostic>*>(std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*, std::pair<clang::SourceLocation, clang::PartialDiagnostic> const*, std::pair<clang::SourceLocation, clang::PartialDiagnostic>*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang23DeducedTemplateArgumentELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::DeducedTemplateArgument, false>::grow(unsigned long)"},
    {"_ZSt24__uninitialized_copy_auxIPN5clang23DeducedTemplateArgumentES2_ET0_T_S4_S3_St12__false_type", "clang::DeducedTemplateArgument* std::__uninitialized_copy_aux<clang::DeducedTemplateArgument*, clang::DeducedTemplateArgument*>(clang::DeducedTemplateArgument*, clang::DeducedTemplateArgument*, clang::DeducedTemplateArgument*, std::__false_type)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23DeducedTemplateArgumentEE15construct_rangeEPS2_S4_RKS2_", "llvm::SmallVectorImpl<clang::DeducedTemplateArgument>::construct_range(clang::DeducedTemplateArgument*, clang::DeducedTemplateArgument*, clang::DeducedTemplateArgument const&)"},
    {"_ZN4llvm11SmallVectorINS0_IN5clang23DeducedTemplateArgumentELj4EEELj2EEC2EjRKS3_", "llvm::SmallVector<llvm::SmallVector<clang::DeducedTemplateArgument, 4u>, 2u>::SmallVector(unsigned int, llvm::SmallVector<clang::DeducedTemplateArgument, 4u> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIN5clang23DeducedTemplateArgumentELj4EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<clang::DeducedTemplateArgument, 4u>, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIN5clang23DeducedTemplateArgumentELj4EEELb0EE13destroy_rangeEPS4_S6_", "llvm::SmallVectorTemplateBase<llvm::SmallVector<clang::DeducedTemplateArgument, 4u>, false>::destroy_range(llvm::SmallVector<clang::DeducedTemplateArgument, 4u>*, llvm::SmallVector<clang::DeducedTemplateArgument, 4u>*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23DeducedTemplateArgumentEEaSERKS3_", "llvm::SmallVectorImpl<clang::DeducedTemplateArgument>::operator=(llvm::SmallVectorImpl<clang::DeducedTemplateArgument> const&)"},
    {"_ZSt24__uninitialized_copy_auxIPKN5clang23DeducedTemplateArgumentEPS1_ET0_T_S6_S5_St12__false_type", "clang::DeducedTemplateArgument* std::__uninitialized_copy_aux<clang::DeducedTemplateArgument const*, clang::DeducedTemplateArgument*>(clang::DeducedTemplateArgument const*, clang::DeducedTemplateArgument const*, clang::DeducedTemplateArgument*, std::__false_type)"},
    {"_ZN4llvm15SmallVectorImplIN5clang8QualTypeEE6insertIPKS2_EEPS2_S7_T_S8_", "clang::QualType* llvm::SmallVectorImpl<clang::QualType>::insert<clang::QualType const*>(clang::QualType*, clang::QualType const*, clang::QualType const*)"},
    {"_ZN4llvm15SmallVectorImplIN5clang8QualTypeEE6resizeEj", "llvm::SmallVectorImpl<clang::QualType>::resize(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang23DeducedTemplateArgumentEE6appendIPKNS1_16TemplateArgumentEEEvT_S8_", "void llvm::SmallVectorImpl<clang::DeducedTemplateArgument>::append<clang::TemplateArgument const*>(clang::TemplateArgument const*, clang::TemplateArgument const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseI33RefParamPartialOrderingComparisonLb0EE4growEm", "llvm::SmallVectorTemplateBase<RefParamPartialOrderingComparison, false>::grow(unsigned long)"},
    {"_ZN5clang23DeducedTemplateArgumentC2ERKNS_16TemplateArgumentEb", "clang::DeducedTemplateArgument::DeducedTemplateArgument(clang::TemplateArgument const&, bool)"},
    {"_ZN5clang4Sema28getTemplateInstantiationArgsEPNS_9NamedDeclEPKNS_20TemplateArgumentListEbPKNS_12FunctionDeclE", "clang::Sema::getTemplateInstantiationArgs(clang::NamedDecl*, clang::TemplateArgumentList const*, bool, clang::FunctionDecl const*)"},
    {"_ZNK5clang4Sema27ActiveTemplateInstantiation21isInstantiationRecordEv", "clang::Sema::ActiveTemplateInstantiation::isInstantiationRecord() const"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_4DeclENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::Decl*, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_4DeclENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::Decl*, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplate23CheckInstantiationDepthENS_14SourceLocationENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::CheckInstantiationDepth(clang::SourceLocation, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_12TemplateDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::TemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_12TemplateDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::TemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_20FunctionTemplateDeclEPKNS_16TemplateArgumentEjNS0_27ActiveTemplateInstantiation17InstantiationKindERNS_4sema21TemplateDeductionInfoENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::FunctionTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::Sema::ActiveTemplateInstantiation::InstantiationKind, clang::sema::TemplateDeductionInfo&, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_20FunctionTemplateDeclEPKNS_16TemplateArgumentEjNS0_27ActiveTemplateInstantiation17InstantiationKindERNS_4sema21TemplateDeductionInfoENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::FunctionTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::Sema::ActiveTemplateInstantiation::InstantiationKind, clang::sema::TemplateDeductionInfo&, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_38ClassTemplatePartialSpecializationDeclEPKNS_16TemplateArgumentEjRNS_4sema21TemplateDeductionInfoENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::ClassTemplatePartialSpecializationDecl*, clang::TemplateArgument const*, unsigned int, clang::sema::TemplateDeductionInfo&, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_38ClassTemplatePartialSpecializationDeclEPKNS_16TemplateArgumentEjRNS_4sema21TemplateDeductionInfoENS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::ClassTemplatePartialSpecializationDecl*, clang::TemplateArgument const*, unsigned int, clang::sema::TemplateDeductionInfo&, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_11ParmVarDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::ParmVarDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_11ParmVarDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::ParmVarDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_9NamedDeclEPNS_23NonTypeTemplateParmDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::NamedDecl*, clang::NonTypeTemplateParmDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_9NamedDeclEPNS_23NonTypeTemplateParmDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::NamedDecl*, clang::NonTypeTemplateParmDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_9NamedDeclEPNS_24TemplateTemplateParmDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::NamedDecl*, clang::TemplateTemplateParmDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_9NamedDeclEPNS_24TemplateTemplateParmDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::NamedDecl*, clang::TemplateTemplateParmDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC1ERS0_NS_14SourceLocationEPNS_12TemplateDeclEPNS_9NamedDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::TemplateDecl*, clang::NamedDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplateC2ERS0_NS_14SourceLocationEPNS_12TemplateDeclEPNS_9NamedDeclEPKNS_16TemplateArgumentEjNS_11SourceRangeE", "clang::Sema::InstantiatingTemplate::InstantiatingTemplate(clang::Sema&, clang::SourceLocation, clang::TemplateDecl*, clang::NamedDecl*, clang::TemplateArgument const*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang4Sema21InstantiatingTemplate5ClearEv", "clang::Sema::InstantiatingTemplate::Clear()"},
    {"_ZN5clang4Sema23PrintInstantiationStackEv", "clang::Sema::PrintInstantiationStack()"},
    {"_ZNK5clang4Sema15isSFINAEContextEv", "clang::Sema::isSFINAEContext() const"},
    {"_ZN5clang4Sema9SubstTypeEPNS_14TypeSourceInfoERKNS_30MultiLevelTemplateArgumentListENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::SubstType(clang::TypeSourceInfo*, clang::MultiLevelTemplateArgumentList const&, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE13TransformTypeEPNS_14TypeSourceInfoE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformType(clang::TypeSourceInfo*)"},
    {"_ZN5clang4Sema9SubstTypeENS_7TypeLocERKNS_30MultiLevelTemplateArgumentListENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::SubstType(clang::TypeLoc, clang::MultiLevelTemplateArgumentList const&, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE13TransformTypeERNS_14TypeLocBuilderENS_7TypeLocE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformType(clang::TypeLocBuilder&, clang::TypeLoc)"},
    {"_ZN5clang4Sema9SubstTypeENS_8QualTypeERKNS_30MultiLevelTemplateArgumentListENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::SubstType(clang::QualType, clang::MultiLevelTemplateArgumentList const&, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE13TransformTypeENS_8QualTypeE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformType(clang::QualType)"},
    {"_ZN5clang4Sema21SubstFunctionDeclTypeEPNS_14TypeSourceInfoERKNS_30MultiLevelTemplateArgumentListENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::SubstFunctionDeclType(clang::TypeSourceInfo*, clang::MultiLevelTemplateArgumentList const&, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang4Sema16SubstParmVarDeclEPNS_11ParmVarDeclERKNS_30MultiLevelTemplateArgumentListEN4llvm8OptionalIjEE", "clang::Sema::SubstParmVarDecl(clang::ParmVarDecl*, clang::MultiLevelTemplateArgumentList const&, llvm::Optional<unsigned int>)"},
    {"_ZN5clang23LocalInstantiationScope24InstantiatedLocalPackArgEPKNS_4DeclEPS1_", "clang::LocalInstantiationScope::InstantiatedLocalPackArg(clang::Decl const*, clang::Decl*)"},
    {"_ZN5clang23LocalInstantiationScope17InstantiatedLocalEPKNS_4DeclEPS1_", "clang::LocalInstantiationScope::InstantiatedLocal(clang::Decl const*, clang::Decl*)"},
    {"_ZN5clang4Sema14SubstParmTypesENS_14SourceLocationEPPNS_11ParmVarDeclEjRKNS_30MultiLevelTemplateArgumentListERN4llvm15SmallVectorImplINS_8QualTypeEEEPNS9_IS3_EE", "clang::Sema::SubstParmTypes(clang::SourceLocation, clang::ParmVarDecl**, unsigned int, clang::MultiLevelTemplateArgumentList const&, llvm::SmallVectorImpl<clang::QualType>&, llvm::SmallVectorImpl<clang::ParmVarDecl*>*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE27TransformFunctionTypeParamsENS_14SourceLocationEPPNS_11ParmVarDeclEjPKNS_8QualTypeERN4llvm15SmallVectorImplIS8_EEPNSC_IS6_EE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformFunctionTypeParams(clang::SourceLocation, clang::ParmVarDecl**, unsigned int, clang::QualType const*, llvm::SmallVectorImpl<clang::QualType>&, llvm::SmallVectorImpl<clang::ParmVarDecl*>*)"},
    {"_ZN5clang4Sema19SubstBaseSpecifiersEPNS_13CXXRecordDeclES2_RKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstBaseSpecifiers(clang::CXXRecordDecl*, clang::CXXRecordDecl*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang4Sema16InstantiateClassENS_14SourceLocationEPNS_13CXXRecordDeclES3_RKNS_30MultiLevelTemplateArgumentListENS_26TemplateSpecializationKindEb", "clang::Sema::InstantiateClass(clang::SourceLocation, clang::CXXRecordDecl*, clang::CXXRecordDecl*, clang::MultiLevelTemplateArgumentList const&, clang::TemplateSpecializationKind, bool)"},
    {"_ZN5clang4Sema38InstantiateClassTemplateSpecializationENS_14SourceLocationEPNS_31ClassTemplateSpecializationDeclENS_26TemplateSpecializationKindEb", "clang::Sema::InstantiateClassTemplateSpecialization(clang::SourceLocation, clang::ClassTemplateSpecializationDecl*, clang::TemplateSpecializationKind, bool)"},
    {"_ZN5clang4Sema23InstantiateClassMembersENS_14SourceLocationEPNS_13CXXRecordDeclERKNS_30MultiLevelTemplateArgumentListENS_26TemplateSpecializationKindE", "clang::Sema::InstantiateClassMembers(clang::SourceLocation, clang::CXXRecordDecl*, clang::MultiLevelTemplateArgumentList const&, clang::TemplateSpecializationKind)"},
    {"_ZN5clang4Sema45InstantiateClassTemplateSpecializationMembersENS_14SourceLocationEPNS_31ClassTemplateSpecializationDeclENS_26TemplateSpecializationKindE", "clang::Sema::InstantiateClassTemplateSpecializationMembers(clang::SourceLocation, clang::ClassTemplateSpecializationDecl*, clang::TemplateSpecializationKind)"},
    {"_ZN5clang4Sema9SubstStmtEPNS_4StmtERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstStmt(clang::Stmt*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE13TransformStmtEPNS_4StmtE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformStmt(clang::Stmt*)"},
    {"_ZN5clang4Sema9SubstExprEPNS_4ExprERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstExpr(clang::Expr*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE13TransformExprEPNS_4ExprE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformExpr(clang::Expr*)"},
    {"_ZN5clang4Sema10SubstExprsEPPNS_4ExprEjbRKNS_30MultiLevelTemplateArgumentListERN4llvm15SmallVectorImplIS2_EE", "clang::Sema::SubstExprs(clang::Expr**, unsigned int, bool, clang::MultiLevelTemplateArgumentList const&, llvm::SmallVectorImpl<clang::Expr*>&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE14TransformExprsEPPNS_4ExprEjbRN4llvm15SmallVectorImplIS5_EEPb", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformExprs(clang::Expr**, unsigned int, bool, llvm::SmallVectorImpl<clang::Expr*>&, bool*)"},
    {"_ZN5clang4Sema24SubstNestedNameSpecifierEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstNestedNameSpecifier(clang::NestedNameSpecifier*, clang::SourceRange, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE28TransformNestedNameSpecifierEPNS_19NestedNameSpecifierENS_11SourceRangeENS_8QualTypeEPNS_9NamedDeclE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformNestedNameSpecifier(clang::NestedNameSpecifier*, clang::SourceRange, clang::QualType, clang::NamedDecl*)"},
    {"_ZN5clang4Sema24SubstDeclarationNameInfoERKNS_19DeclarationNameInfoERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstDeclarationNameInfo(clang::DeclarationNameInfo const&, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE28TransformDeclarationNameInfoERKNS_19DeclarationNameInfoE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformDeclarationNameInfo(clang::DeclarationNameInfo const&)"},
    {"_ZN5clang4Sema17SubstTemplateNameENS_12TemplateNameENS_14SourceLocationERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstTemplateName(clang::TemplateName, clang::SourceLocation, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN12_GLOBAL__N_120TemplateInstantiator21TransformTemplateNameEN5clang12TemplateNameENS1_8QualTypeEPNS1_9NamedDeclE", "(anonymous namespace)::TemplateInstantiator::TransformTemplateName(clang::TemplateName, clang::QualType, clang::NamedDecl*)"},
    {"_ZN5clang4Sema5SubstEPKNS_19TemplateArgumentLocEjRNS_24TemplateArgumentListInfoERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::Subst(clang::TemplateArgumentLoc const*, unsigned int, clang::TemplateArgumentListInfo&, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang23LocalInstantiationScope18getInstantiationOfEPKNS_4DeclE", "clang::LocalInstantiationScope::getInstantiationOf(clang::Decl const*)"},
    {"_ZN5clang23LocalInstantiationScope19findInstantiationOfEPKNS_4DeclE", "clang::LocalInstantiationScope::findInstantiationOf(clang::Decl const*)"},
    {"_ZN5clang23LocalInstantiationScope28MakeInstantiatedLocalArgPackEPKNS_4DeclE", "clang::LocalInstantiationScope::MakeInstantiatedLocalArgPack(clang::Decl const*)"},
    {"_ZN5clang23LocalInstantiationScope27SetPartiallySubstitutedPackEPNS_9NamedDeclEPKNS_16TemplateArgumentEj", "clang::LocalInstantiationScope::SetPartiallySubstitutedPack(clang::NamedDecl*, clang::TemplateArgument const*, unsigned int)"},
    {"_ZNK5clang23LocalInstantiationScope27getPartiallySubstitutedPackEPPKNS_16TemplateArgumentEPj", "clang::LocalInstantiationScope::getPartiallySubstitutedPack(clang::TemplateArgument const**, unsigned int*) const"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE26TransformTemplateArgumentsIPKNS_19TemplateArgumentLocEEEbT_S8_RNS_24TemplateArgumentListInfoE", "bool clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformTemplateArguments<clang::TemplateArgumentLoc const*>(clang::TemplateArgumentLoc const*, clang::TemplateArgumentLoc const*, clang::TemplateArgumentListInfo&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE26TransformTemplateArgumentsINS_33TemplateArgumentLocInventIteratorIS2_PKNS_16TemplateArgumentEEEEEbT_SA_RNS_24TemplateArgumentListInfoE", "bool clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformTemplateArguments<clang::TemplateArgumentLocInventIterator<(anonymous namespace)::TemplateInstantiator, clang::TemplateArgument const*> >(clang::TemplateArgumentLocInventIterator<(anonymous namespace)::TemplateInstantiator, clang::TemplateArgument const*>, clang::TemplateArgumentLocInventIterator<(anonymous namespace)::TemplateInstantiator, clang::TemplateArgument const*>, clang::TemplateArgumentListInfo&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE25TransformTemplateArgumentERKNS_19TemplateArgumentLocERS4_", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformTemplateArgument(clang::TemplateArgumentLoc const&, clang::TemplateArgumentLoc&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE20RebuildPackExpansionENS_19TemplateArgumentLocENS_14SourceLocationEN4llvm8OptionalIjEE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::RebuildPackExpansion(clang::TemplateArgumentLoc, clang::SourceLocation, llvm::Optional<unsigned int>)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE34ForgetPartiallySubstitutedPackRAIIC1ERS2_", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::ForgetPartiallySubstitutedPackRAII::ForgetPartiallySubstitutedPackRAII((anonymous namespace)::TemplateInstantiator&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE34ForgetPartiallySubstitutedPackRAIID1Ev", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::ForgetPartiallySubstitutedPackRAII::~ForgetPartiallySubstitutedPackRAII()"},
    {"_ZN12_GLOBAL__N_120TemplateInstantiator13TransformDeclEN5clang14SourceLocationEPNS1_4DeclE", "(anonymous namespace)::TemplateInstantiator::TransformDecl(clang::SourceLocation, clang::Decl*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE25InventTemplateArgumentLocERKNS_16TemplateArgumentERNS_19TemplateArgumentLocE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::InventTemplateArgumentLoc(clang::TemplateArgument const&, clang::TemplateArgumentLoc&)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE26TransformTypeInObjectScopeEPNS_14TypeSourceInfoENS_8QualTypeEPNS_9NamedDeclEPNS_19NestedNameSpecifierE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformTypeInObjectScope(clang::TypeSourceInfo*, clang::QualType, clang::NamedDecl*, clang::NestedNameSpecifier*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE35TransformTemplateSpecializationTypeERNS_14TypeLocBuilderENS_29TemplateSpecializationTypeLocENS_12TemplateNameE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformTemplateSpecializationType(clang::TypeLocBuilder&, clang::TemplateSpecializationTypeLoc, clang::TemplateName)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE44TransformDependentTemplateSpecializationTypeERNS_14TypeLocBuilderENS_38DependentTemplateSpecializationTypeLocEPNS_19NestedNameSpecifierE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformDependentTemplateSpecializationType(clang::TypeLocBuilder&, clang::DependentTemplateSpecializationTypeLoc, clang::NestedNameSpecifier*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE23TransformBinaryOperatorEPNS_14BinaryOperatorE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE21TransformCompoundStmtEPNS_12CompoundStmtEb", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformCompoundStmt(clang::CompoundStmt*, bool)"},
    {"_ZN12_GLOBAL__N_120TemplateInstantiator28TransformTemplateParmRefExprEPN5clang11DeclRefExprEPNS1_23NonTypeTemplateParmDeclE", "(anonymous namespace)::TemplateInstantiator::TransformTemplateParmRefExpr(clang::DeclRefExpr*, clang::NonTypeTemplateParmDecl*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE20TransformDeclRefExprEPNS_11DeclRefExprE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformDeclRefExpr(clang::DeclRefExpr*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE25TransformCXXNamedCastExprEPNS_16CXXNamedCastExprE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE17TransformCallExprEPNS_8CallExprE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformCallExpr(clang::CallExpr*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE21TransformCXXCatchStmtEPNS_12CXXCatchStmtE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE26TransformFunctionProtoTypeERNS_14TypeLocBuilderENS_20FunctionProtoTypeLocE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformFunctionProtoType(clang::TypeLocBuilder&, clang::FunctionProtoTypeLoc)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE16RebuildArrayTypeENS_8QualTypeENS_9ArrayType17ArraySizeModifierEPKN4llvm5APIntEPNS_4ExprEjNS_11SourceRangeE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::RebuildArrayType(clang::QualType, clang::ArrayType::ArraySizeModifier, llvm::APInt const*, clang::Expr*, unsigned int, clang::SourceRange)"},
    {"_ZN5clang13TreeTransformIN12_GLOBAL__N_120TemplateInstantiatorEE22TransformReferenceTypeERNS_14TypeLocBuilderENS_16ReferenceTypeLocE", "clang::TreeTransform<(anonymous namespace)::TemplateInstantiator>::TransformReferenceType(clang::TypeLocBuilder&, clang::ReferenceTypeLoc)"},
    {"_ZN5clang14TypeLocBuilder12pushFullCopyENS_7TypeLocE", "clang::TypeLocBuilder::pushFullCopy(clang::TypeLoc)"},
    {"_ZN5clang11DeclVisitorINS_24TemplateDeclInstantiatorEPNS_4DeclEE5VisitES3_", "clang::DeclVisitor<clang::TemplateDeclInstantiator, clang::Decl*>::Visit(clang::Decl*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_12PointerUnionIPS2_PNS_11SmallVectorIS6_Lj4EEEEENS_12DenseMapInfoIS4_EENSB_ISA_EEE16InsertIntoBucketERKS4_RKSA_PSt4pairIS4_SA_E", "llvm::DenseMap<clang::Decl const*, llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*>, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*> > >::InsertIntoBucket(clang::Decl const* const&, llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*> const&, std::pair<clang::Decl const*, llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclENS_12PointerUnionIPS2_PNS_11SmallVectorIS6_Lj4EEEEENS_12DenseMapInfoIS4_EENSB_ISA_EEE4growEj", "llvm::DenseMap<clang::Decl const*, llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*>, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::PointerUnion<clang::Decl*, llvm::SmallVector<clang::Decl*, 4u>*> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang19TemplateArgumentLocEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::TemplateArgumentLoc>::push_back(clang::TemplateArgumentLoc const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang19TemplateArgumentLocELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::TemplateArgumentLoc, false>::grow(unsigned long)"},
    {"_ZSt24__uninitialized_copy_auxIPN5clang19TemplateArgumentLocES2_ET0_T_S4_S3_St12__false_type", "clang::TemplateArgumentLoc* std::__uninitialized_copy_aux<clang::TemplateArgumentLoc*, clang::TemplateArgumentLoc*>(clang::TemplateArgumentLoc*, clang::TemplateArgumentLoc*, clang::TemplateArgumentLoc*, std::__false_type)"},
    {"_ZN5clang19TemplateArgumentLocC2ERKNS_16TemplateArgumentENS_11SourceRangeENS_14SourceLocationES5_", "clang::TemplateArgumentLoc::TemplateArgumentLoc(clang::TemplateArgument const&, clang::SourceRange, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang14TypeLocBuilder8pushImplENS_8QualTypeEm", "clang::TypeLocBuilder::pushImpl(clang::QualType, unsigned long)"},
    {"_ZN5clang4Expr19getValueKindForTypeENS_8QualTypeE", "clang::Expr::getValueKindForType(clang::QualType)"},
    {"_ZN5clang12LookupResultD2Ev", "clang::LookupResult::~LookupResult()"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4Sema17OffsetOfComponentELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Sema::OffsetOfComponent, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang10DesignatorELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Designator, false>::grow(unsigned long)"},
    {"_ZN5clang24TemplateDeclInstantiator9VisitDeclEPNS_4DeclE", "clang::TemplateDeclInstantiator::VisitDecl(clang::Decl*)"},
    {"_ZN4llvm8DenseMapIPN5clang11ParmVarDeclENS_11SmallVectorIS3_Lj1EEENS_12DenseMapInfoIS3_EENS6_IS5_EEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::ParmVarDecl*, llvm::SmallVector<clang::ParmVarDecl*, 1u>, llvm::DenseMapInfo<clang::ParmVarDecl*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ParmVarDecl*, 1u> > >::FindAndConstruct(clang::ParmVarDecl* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang11ParmVarDeclENS_11SmallVectorIS3_Lj1EEENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::ParmVarDecl*, llvm::SmallVector<clang::ParmVarDecl*, 1u>, llvm::DenseMapInfo<clang::ParmVarDecl*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ParmVarDecl*, 1u> > >::InsertIntoBucket(clang::ParmVarDecl* const&, llvm::SmallVector<clang::ParmVarDecl*, 1u> const&, std::pair<clang::ParmVarDecl*, llvm::SmallVector<clang::ParmVarDecl*, 1u> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang11ParmVarDeclENS_11SmallVectorIS3_Lj1EEENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::ParmVarDecl*, llvm::SmallVector<clang::ParmVarDecl*, 1u>, llvm::DenseMapInfo<clang::ParmVarDecl*>, llvm::DenseMapInfo<llvm::SmallVector<clang::ParmVarDecl*, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang11ParmVarDeclEEaSERKS4_", "llvm::SmallVectorImpl<clang::ParmVarDecl*>::operator=(llvm::SmallVectorImpl<clang::ParmVarDecl*> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4Sema27ActiveTemplateInstantiationELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::Sema::ActiveTemplateInstantiation, false>::grow(unsigned long)"},
    {"_ZN5clang4sema21TemplateDeductionInfoD2Ev", "clang::sema::TemplateDeductionInfo::~TemplateDeductionInfo()"},
    {"_ZN5clang23LocalInstantiationScope4ExitEv", "clang::LocalInstantiationScope::Exit()"},
    {"_ZN5clang24TemplateDeclInstantiator14SubstQualifierEPKNS_14DeclaratorDeclEPS1_", "clang::TemplateDeclInstantiator::SubstQualifier(clang::DeclaratorDecl const*, clang::DeclaratorDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator14SubstQualifierEPKNS_7TagDeclEPS1_", "clang::TemplateDeclInstantiator::SubstQualifier(clang::TagDecl const*, clang::TagDecl*)"},
    {"_ZN5clang4Sema16InstantiateAttrsERKNS_30MultiLevelTemplateArgumentListEPNS_4DeclES5_", "clang::Sema::InstantiateAttrs(clang::MultiLevelTemplateArgumentList const&, clang::Decl*, clang::Decl*)"},
    {"_ZN5clang24TemplateDeclInstantiator24VisitTranslationUnitDeclEPNS_19TranslationUnitDeclE", "clang::TemplateDeclInstantiator::VisitTranslationUnitDecl(clang::TranslationUnitDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator18VisitNamespaceDeclEPNS_13NamespaceDeclE", "clang::TemplateDeclInstantiator::VisitNamespaceDecl(clang::NamespaceDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator23VisitNamespaceAliasDeclEPNS_18NamespaceAliasDeclE", "clang::TemplateDeclInstantiator::VisitNamespaceAliasDecl(clang::NamespaceAliasDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator16VisitTypedefDeclEPNS_11TypedefDeclE", "clang::TemplateDeclInstantiator::VisitTypedefDecl(clang::TypedefDecl*)"},
    {"_ZN5clang4Sema20FindInstantiatedDeclENS_14SourceLocationEPNS_9NamedDeclERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::FindInstantiatedDecl(clang::SourceLocation, clang::NamedDecl*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang24TemplateDeclInstantiator12VisitVarDeclEPNS_7VarDeclE", "clang::TemplateDeclInstantiator::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator18VisitCXXRecordDeclEPNS_13CXXRecordDeclE", "clang::TemplateDeclInstantiator::VisitCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_Z22InstantiateInitializerRN5clang4SemaEPNS_4ExprERKNS_30MultiLevelTemplateArgumentListERNS_14SourceLocationERNS_15ASTOwningVectorIS3_Lj8EEES8_", "InstantiateInitializer(clang::Sema&, clang::Expr*, clang::MultiLevelTemplateArgumentList const&, clang::SourceLocation&, clang::ASTOwningVector<clang::Expr*, 8u>&, clang::SourceLocation&)"},
    {"_ZN5clang24TemplateDeclInstantiator19VisitAccessSpecDeclEPNS_14AccessSpecDeclE", "clang::TemplateDeclInstantiator::VisitAccessSpecDecl(clang::AccessSpecDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator14VisitFieldDeclEPNS_9FieldDeclE", "clang::TemplateDeclInstantiator::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator22VisitIndirectFieldDeclEPNS_17IndirectFieldDeclE", "clang::TemplateDeclInstantiator::VisitIndirectFieldDecl(clang::IndirectFieldDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator15VisitFriendDeclEPNS_10FriendDeclE", "clang::TemplateDeclInstantiator::VisitFriendDecl(clang::FriendDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator21VisitStaticAssertDeclEPNS_16StaticAssertDeclE", "clang::TemplateDeclInstantiator::VisitStaticAssertDecl(clang::StaticAssertDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator13VisitEnumDeclEPNS_8EnumDeclE", "clang::TemplateDeclInstantiator::VisitEnumDecl(clang::EnumDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator21VisitEnumConstantDeclEPNS_16EnumConstantDeclE", "clang::TemplateDeclInstantiator::VisitEnumConstantDecl(clang::EnumConstantDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator22VisitClassTemplateDeclEPNS_17ClassTemplateDeclE", "clang::TemplateDeclInstantiator::VisitClassTemplateDecl(clang::ClassTemplateDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator19SubstTemplateParamsEPNS_21TemplateParameterListE", "clang::TemplateDeclInstantiator::SubstTemplateParams(clang::TemplateParameterList*)"},
    {"_ZN5clang4Sema23FindInstantiatedContextENS_14SourceLocationEPNS_11DeclContextERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::FindInstantiatedContext(clang::SourceLocation, clang::DeclContext*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang24TemplateDeclInstantiator43VisitClassTemplatePartialSpecializationDeclEPNS_38ClassTemplatePartialSpecializationDeclE", "clang::TemplateDeclInstantiator::VisitClassTemplatePartialSpecializationDecl(clang::ClassTemplatePartialSpecializationDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator45InstantiateClassTemplatePartialSpecializationEPNS_17ClassTemplateDeclEPNS_38ClassTemplatePartialSpecializationDeclE", "clang::TemplateDeclInstantiator::InstantiateClassTemplatePartialSpecialization(clang::ClassTemplateDecl*, clang::ClassTemplatePartialSpecializationDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator25VisitFunctionTemplateDeclEPNS_20FunctionTemplateDeclE", "clang::TemplateDeclInstantiator::VisitFunctionTemplateDecl(clang::FunctionTemplateDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator18VisitCXXMethodDeclEPNS_13CXXMethodDeclEPNS_21TemplateParameterListE", "clang::TemplateDeclInstantiator::VisitCXXMethodDecl(clang::CXXMethodDecl*, clang::TemplateParameterList*)"},
    {"_ZN5clang24TemplateDeclInstantiator17VisitFunctionDeclEPNS_12FunctionDeclEPNS_21TemplateParameterListE", "clang::TemplateDeclInstantiator::VisitFunctionDecl(clang::FunctionDecl*, clang::TemplateParameterList*)"},
    {"_ZN5clang24TemplateDeclInstantiator17SubstFunctionTypeEPNS_12FunctionDeclERN4llvm15SmallVectorImplIPNS_11ParmVarDeclEEE", "clang::TemplateDeclInstantiator::SubstFunctionType(clang::FunctionDecl*, llvm::SmallVectorImpl<clang::ParmVarDecl*>&)"},
    {"_ZN5clang24TemplateDeclInstantiator25InitFunctionInstantiationEPNS_12FunctionDeclES2_", "clang::TemplateDeclInstantiator::InitFunctionInstantiation(clang::FunctionDecl*, clang::FunctionDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator23InitMethodInstantiationEPNS_13CXXMethodDeclES2_", "clang::TemplateDeclInstantiator::InitMethodInstantiation(clang::CXXMethodDecl*, clang::CXXMethodDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator23VisitCXXConstructorDeclEPNS_18CXXConstructorDeclE", "clang::TemplateDeclInstantiator::VisitCXXConstructorDecl(clang::CXXConstructorDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator22VisitCXXDestructorDeclEPNS_17CXXDestructorDeclE", "clang::TemplateDeclInstantiator::VisitCXXDestructorDecl(clang::CXXDestructorDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator22VisitCXXConversionDeclEPNS_17CXXConversionDeclE", "clang::TemplateDeclInstantiator::VisitCXXConversionDecl(clang::CXXConversionDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator16VisitParmVarDeclEPNS_11ParmVarDeclE", "clang::TemplateDeclInstantiator::VisitParmVarDecl(clang::ParmVarDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator25VisitTemplateTypeParmDeclEPNS_20TemplateTypeParmDeclE", "clang::TemplateDeclInstantiator::VisitTemplateTypeParmDecl(clang::TemplateTypeParmDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator28VisitNonTypeTemplateParmDeclEPNS_23NonTypeTemplateParmDeclE", "clang::TemplateDeclInstantiator::VisitNonTypeTemplateParmDecl(clang::NonTypeTemplateParmDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator29VisitTemplateTemplateParmDeclEPNS_24TemplateTemplateParmDeclE", "clang::TemplateDeclInstantiator::VisitTemplateTemplateParmDecl(clang::TemplateTemplateParmDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator23VisitUsingDirectiveDeclEPNS_18UsingDirectiveDeclE", "clang::TemplateDeclInstantiator::VisitUsingDirectiveDecl(clang::UsingDirectiveDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator14VisitUsingDeclEPNS_9UsingDeclE", "clang::TemplateDeclInstantiator::VisitUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator20VisitUsingShadowDeclEPNS_15UsingShadowDeclE", "clang::TemplateDeclInstantiator::VisitUsingShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator32VisitUnresolvedUsingTypenameDeclEPNS_27UnresolvedUsingTypenameDeclE", "clang::TemplateDeclInstantiator::VisitUnresolvedUsingTypenameDecl(clang::UnresolvedUsingTypenameDecl*)"},
    {"_ZN5clang24TemplateDeclInstantiator29VisitUnresolvedUsingValueDeclEPNS_24UnresolvedUsingValueDeclE", "clang::TemplateDeclInstantiator::VisitUnresolvedUsingValueDecl(clang::UnresolvedUsingValueDecl*)"},
    {"_ZN5clang4Sema9SubstDeclEPNS_4DeclEPNS_11DeclContextERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::SubstDecl(clang::Decl*, clang::DeclContext*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang4Sema29InstantiateFunctionDefinitionENS_14SourceLocationEPNS_12FunctionDeclEbb", "clang::Sema::InstantiateFunctionDefinition(clang::SourceLocation, clang::FunctionDecl*, bool, bool)"},
    {"_ZN5clang4Sema26InstantiateMemInitializersEPNS_18CXXConstructorDeclEPKS1_RKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::InstantiateMemInitializers(clang::CXXConstructorDecl*, clang::CXXConstructorDecl const*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang4Sema27PerformDependentDiagnosticsEPKNS_11DeclContextERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::PerformDependentDiagnostics(clang::DeclContext const*, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang4Sema28PerformPendingInstantiationsEb", "clang::Sema::PerformPendingInstantiations(bool)"},
    {"_ZN5clang4Sema37InstantiateStaticDataMemberDefinitionENS_14SourceLocationEPNS_7VarDeclEbb", "clang::Sema::InstantiateStaticDataMemberDefinition(clang::SourceLocation, clang::VarDecl*, bool, bool)"},
    {"_Z17isInstantiationOfRN5clang10ASTContextEPNS_9NamedDeclEPNS_4DeclE", "isInstantiationOf(clang::ASTContext&, clang::NamedDecl*, clang::Decl*)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPN5clang13CXXRecordDeclENS2_14SourceLocationEEE4swapERS7_", "llvm::SmallVectorImpl<std::pair<clang::CXXRecordDecl*, clang::SourceLocation> >::swap(llvm::SmallVectorImpl<std::pair<clang::CXXRecordDecl*, clang::SourceLocation> >&)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackENS_14SourceLocationEPNS_14TypeSourceInfoENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::SourceLocation, clang::TypeSourceInfo*, clang::Sema::UnexpandedParameterPackContext)"},
    {"_Z32DiagnoseUnexpandedParameterPacksRN5clang4SemaENS_14SourceLocationENS0_30UnexpandedParameterPackContextERKN4llvm15SmallVectorImplISt4pairINS4_12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEES2_EEE", "DiagnoseUnexpandedParameterPacks(clang::Sema&, clang::SourceLocation, clang::Sema::UnexpandedParameterPackContext, llvm::SmallVectorImpl<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> > const&)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackEPNS_4ExprENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::Expr*, clang::Sema::UnexpandedParameterPackContext)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackERKNS_12CXXScopeSpecENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::CXXScopeSpec const&, clang::Sema::UnexpandedParameterPackContext)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE27TraverseNestedNameSpecifierEPNS_19NestedNameSpecifierE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseNestedNameSpecifier(clang::NestedNameSpecifier*)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackERKNS_19DeclarationNameInfoENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::DeclarationNameInfo const&, clang::Sema::UnexpandedParameterPackContext)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackENS_14SourceLocationENS_12TemplateNameENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::SourceLocation, clang::TemplateName, clang::Sema::UnexpandedParameterPackContext)"},
    {"_ZN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitor20TraverseTemplateNameEN5clang12TemplateNameE", "(anonymous namespace)::CollectUnexpandedParameterPacksVisitor::TraverseTemplateName(clang::TemplateName)"},
    {"_ZN5clang4Sema31DiagnoseUnexpandedParameterPackENS_19TemplateArgumentLocENS0_30UnexpandedParameterPackContextE", "clang::Sema::DiagnoseUnexpandedParameterPack(clang::TemplateArgumentLoc, clang::Sema::UnexpandedParameterPackContext)"},
    {"_ZN5clang4Sema31collectUnexpandedParameterPacksENS_16TemplateArgumentERN4llvm15SmallVectorImplISt4pairINS2_12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEENS_14SourceLocationEEEE", "clang::Sema::collectUnexpandedParameterPacks(clang::TemplateArgument, llvm::SmallVectorImpl<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> >&)"},
    {"_ZN5clang4Sema31collectUnexpandedParameterPacksENS_19TemplateArgumentLocERN4llvm15SmallVectorImplISt4pairINS2_12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEENS_14SourceLocationEEEE", "clang::Sema::collectUnexpandedParameterPacks(clang::TemplateArgumentLoc, llvm::SmallVectorImpl<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> >&)"},
    {"_ZN5clang4Sema31collectUnexpandedParameterPacksENS_8QualTypeERN4llvm15SmallVectorImplISt4pairINS2_12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEENS_14SourceLocationEEEE", "clang::Sema::collectUnexpandedParameterPacks(clang::QualType, llvm::SmallVectorImpl<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> >&)"},
    {"_ZN5clang4Sema31collectUnexpandedParameterPacksENS_7TypeLocERN4llvm15SmallVectorImplISt4pairINS2_12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEENS_14SourceLocationEEEE", "clang::Sema::collectUnexpandedParameterPacks(clang::TypeLoc, llvm::SmallVectorImpl<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> >&)"},
    {"_ZN5clang4Sema18ActOnPackExpansionERKNS_22ParsedTemplateArgumentENS_14SourceLocationE", "clang::Sema::ActOnPackExpansion(clang::ParsedTemplateArgument const&, clang::SourceLocation)"},
    {"_ZN5clang4Sema18ActOnPackExpansionENS_9OpaquePtrINS_8QualTypeEEENS_14SourceLocationE", "clang::Sema::ActOnPackExpansion(clang::OpaquePtr<clang::QualType>, clang::SourceLocation)"},
    {"_ZN5clang4Sema18ActOnPackExpansionEPNS_4ExprENS_14SourceLocationE", "clang::Sema::ActOnPackExpansion(clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema18CheckPackExpansionEPNS_14TypeSourceInfoENS_14SourceLocationEN4llvm8OptionalIjEE", "clang::Sema::CheckPackExpansion(clang::TypeSourceInfo*, clang::SourceLocation, llvm::Optional<unsigned int>)"},
    {"_ZN5clang4Sema18CheckPackExpansionENS_8QualTypeENS_11SourceRangeENS_14SourceLocationEN4llvm8OptionalIjEE", "clang::Sema::CheckPackExpansion(clang::QualType, clang::SourceRange, clang::SourceLocation, llvm::Optional<unsigned int>)"},
    {"_ZN5clang4Sema18CheckPackExpansionEPNS_4ExprENS_14SourceLocationEN4llvm8OptionalIjEE", "clang::Sema::CheckPackExpansion(clang::Expr*, clang::SourceLocation, llvm::Optional<unsigned int>)"},
    {"_ZN5clang4Sema31CheckParameterPacksForExpansionENS_14SourceLocationENS_11SourceRangeEPKSt4pairIN4llvm12PointerUnionIPKNS_20TemplateTypeParmTypeEPNS_9NamedDeclEEES1_EjRKNS_30MultiLevelTemplateArgumentListERbSI_RNS4_8OptionalIjEE", "clang::Sema::CheckParameterPacksForExpansion(clang::SourceLocation, clang::SourceRange, std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation> const*, unsigned int, clang::MultiLevelTemplateArgumentList const&, bool&, bool&, llvm::Optional<unsigned int>&)"},
    {"_ZN5clang4Sema26getNumArgumentsInExpansionENS_8QualTypeERKNS_30MultiLevelTemplateArgumentListE", "clang::Sema::getNumArgumentsInExpansion(clang::QualType, clang::MultiLevelTemplateArgumentList const&)"},
    {"_ZN5clang4Sema32containsUnexpandedParameterPacksERNS_10DeclaratorE", "clang::Sema::containsUnexpandedParameterPacks(clang::Declarator&)"},
    {"_ZN5clang4Sema28ActOnSizeofParameterPackExprEPNS_5ScopeENS_14SourceLocationERNS_14IdentifierInfoES3_S3_", "clang::Sema::ActOnSizeofParameterPackExpr(clang::Scope*, clang::SourceLocation, clang::IdentifierInfo&, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE24TraverseTemplateArgumentERKNS_16TemplateArgumentE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE27TraverseTemplateArgumentLocERKNS_19TemplateArgumentLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseTemplateArgumentLoc(clang::TemplateArgumentLoc const&)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE12TraverseTypeENS_8QualTypeE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseType(clang::QualType)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE12TraverseStmtEPNS_4StmtE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseStmt(clang::Stmt*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE12TraverseDeclEPNS_4DeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseDecl(clang::Decl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE24TraverseDeclaratorHelperEPNS_14DeclaratorDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseDeclaratorHelper(clang::DeclaratorDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE22TraverseFunctionHelperEPNS_12FunctionDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseFunctionHelper(clang::FunctionDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE23TraverseCXXRecordHelperEPNS_13CXXRecordDeclE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseCXXRecordHelper(clang::CXXRecordDecl*)"},
    {"_ZN5clang19RecursiveASTVisitorIN12_GLOBAL__N_138CollectUnexpandedParameterPacksVisitorEE15TraverseTypeLocENS_7TypeLocE", "clang::RecursiveASTVisitor<(anonymous namespace)::CollectUnexpandedParameterPacksVisitor>::TraverseTypeLoc(clang::TypeLoc)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairINS_12PointerUnionIPKN5clang20TemplateTypeParmTypeEPNS3_9NamedDeclEEENS3_14SourceLocationEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::PointerUnion<clang::TemplateTypeParmType const*, clang::NamedDecl*>, clang::SourceLocation>, false>::grow(unsigned long)"},
    {"_ZN5clang4Sema19adjustParameterTypeENS_8QualTypeE", "clang::Sema::adjustParameterType(clang::QualType)"},
    {"_ZN5clang4Sema18BuildQualifiedTypeENS_8QualTypeENS_14SourceLocationENS_10QualifiersE", "clang::Sema::BuildQualifiedType(clang::QualType, clang::SourceLocation, clang::Qualifiers)"},
    {"_ZN5clang4Sema14BuildParenTypeENS_8QualTypeE", "clang::Sema::BuildParenType(clang::QualType)"},
    {"_ZN5clang4Sema16BuildPointerTypeENS_8QualTypeENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::BuildPointerType(clang::QualType, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang4Sema18BuildReferenceTypeENS_8QualTypeEbNS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::BuildReferenceType(clang::QualType, bool, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang4Sema14BuildArrayTypeENS_8QualTypeENS_9ArrayType17ArraySizeModifierEPNS_4ExprEjNS_11SourceRangeENS_15DeclarationNameE", "clang::Sema::BuildArrayType(clang::QualType, clang::ArrayType::ArraySizeModifier, clang::Expr*, unsigned int, clang::SourceRange, clang::DeclarationName)"},
    {"_ZN5clang4Sema19RequireCompleteTypeENS_14SourceLocationENS_8QualTypeEj", "clang::Sema::RequireCompleteType(clang::SourceLocation, clang::QualType, unsigned int)"},
    {"_ZN5clang4Sema18BuildExtVectorTypeENS_8QualTypeEPNS_4ExprENS_14SourceLocationE", "clang::Sema::BuildExtVectorType(clang::QualType, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema17BuildFunctionTypeENS_8QualTypeEPS1_jbjNS_16RefQualifierKindENS_14SourceLocationENS_15DeclarationNameENS_12FunctionType7ExtInfoE", "clang::Sema::BuildFunctionType(clang::QualType, clang::QualType*, unsigned int, bool, unsigned int, clang::RefQualifierKind, clang::SourceLocation, clang::DeclarationName, clang::FunctionType::ExtInfo)"},
    {"_ZN5clang4Sema22BuildMemberPointerTypeENS_8QualTypeES1_NS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::BuildMemberPointerType(clang::QualType, clang::QualType, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang4Sema21BuildBlockPointerTypeENS_8QualTypeENS_14SourceLocationENS_15DeclarationNameE", "clang::Sema::BuildBlockPointerType(clang::QualType, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang4Sema17GetTypeFromParserENS_9OpaquePtrINS_8QualTypeEEEPPNS_14TypeSourceInfoE", "clang::Sema::GetTypeFromParser(clang::OpaquePtr<clang::QualType>, clang::TypeSourceInfo**)"},
    {"_ZN5clang4Sema20GetTypeForDeclaratorERNS_10DeclaratorEPNS_5ScopeEPPNS_7TagDeclE", "clang::Sema::GetTypeForDeclarator(clang::Declarator&, clang::Scope*, clang::TagDecl**)"},
    {"_Z16processTypeAttrsRN12_GLOBAL__N_119TypeProcessingStateERN5clang8QualTypeEbPNS2_13AttributeListE", "processTypeAttrs((anonymous namespace)::TypeProcessingState&, clang::QualType&, bool, clang::AttributeList*)"},
    {"_ZN5clang4Sema30GetTypeSourceInfoForDeclaratorERNS_10DeclaratorENS_8QualTypeEPNS_14TypeSourceInfoE", "clang::Sema::GetTypeSourceInfoForDeclarator(clang::Declarator&, clang::QualType, clang::TypeSourceInfo*)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_117TypeSpecLocFillerEvE5VisitENS_13UnqualTypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::TypeSpecLocFiller, void>::Visit(clang::UnqualTypeLoc)"},
    {"_ZN5clang4Sema16CreateParsedTypeENS_8QualTypeEPNS_14TypeSourceInfoE", "clang::Sema::CreateParsedType(clang::QualType, clang::TypeSourceInfo*)"},
    {"_ZNK5clang11LocInfoType19getAsStringInternalERSsRKNS_14PrintingPolicyE", "clang::LocInfoType::getAsStringInternal(std::string&, clang::PrintingPolicy const&) const"},
    {"_ZN5clang4Sema13ActOnTypeNameEPNS_5ScopeERNS_10DeclaratorE", "clang::Sema::ActOnTypeName(clang::Scope*, clang::Declarator&)"},
    {"_ZN5clang4Sema19RequireCompleteTypeENS_14SourceLocationENS_8QualTypeERKNS_17PartialDiagnosticESt4pairIS1_S3_E", "clang::Sema::RequireCompleteType(clang::SourceLocation, clang::QualType, clang::PartialDiagnostic const&, std::pair<clang::SourceLocation, clang::PartialDiagnostic>)"},
    {"_ZN5clang4Sema19RequireCompleteTypeENS_14SourceLocationENS_8QualTypeERKNS_17PartialDiagnosticE", "clang::Sema::RequireCompleteType(clang::SourceLocation, clang::QualType, clang::PartialDiagnostic const&)"},
    {"_ZN5clang4Sema17getElaboratedTypeENS_21ElaboratedTypeKeywordERKNS_12CXXScopeSpecENS_8QualTypeE", "clang::Sema::getElaboratedType(clang::ElaboratedTypeKeyword, clang::CXXScopeSpec const&, clang::QualType)"},
    {"_ZN5clang4Sema19BuildTypeofExprTypeEPNS_4ExprENS_14SourceLocationE", "clang::Sema::BuildTypeofExprType(clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang4Sema17BuildDecltypeTypeEPNS_4ExprENS_14SourceLocationE", "clang::Sema::BuildDecltypeType(clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_119DeclaratorLocFillerEvE18VisitVectorTypeLocENS_13VectorTypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::DeclaratorLocFiller, void>::VisitVectorTypeLoc(clang::VectorTypeLoc)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_119DeclaratorLocFillerEvE22VisitObjCObjectTypeLocENS_17ObjCObjectTypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::DeclaratorLocFiller, void>::VisitObjCObjectTypeLoc(clang::ObjCObjectTypeLoc)"},
    {"_ZN12_GLOBAL__N_119DeclaratorLocFiller12VisitTypeLocEN5clang7TypeLocE", "(anonymous namespace)::DeclaratorLocFiller::VisitTypeLoc(clang::TypeLoc)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_119DeclaratorLocFillerEvE15VisitTagTypeLocENS_10TagTypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::DeclaratorLocFiller, void>::VisitTagTypeLoc(clang::TagTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller19VisitBuiltinTypeLocEN5clang14BuiltinTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitBuiltinTypeLoc(clang::BuiltinTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller22VisitElaboratedTypeLocEN5clang17ElaboratedTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitElaboratedTypeLoc(clang::ElaboratedTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller34VisitTemplateSpecializationTypeLocEN5clang29TemplateSpecializationTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitTemplateSpecializationTypeLoc(clang::TemplateSpecializationTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller25VisitDependentNameTypeLocEN5clang20DependentNameTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitDependentNameTypeLoc(clang::DependentNameTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller43VisitDependentTemplateSpecializationTypeLocEN5clang38DependentTemplateSpecializationTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitDependentTemplateSpecializationTypeLoc(clang::DependentTemplateSpecializationTypeLoc)"},
    {"_ZN12_GLOBAL__N_117TypeSpecLocFiller22VisitObjCObjectTypeLocEN5clang17ObjCObjectTypeLocE", "(anonymous namespace)::TypeSpecLocFiller::VisitObjCObjectTypeLoc(clang::ObjCObjectTypeLoc)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_117TypeSpecLocFillerEvE5VisitENS_7TypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::TypeSpecLocFiller, void>::Visit(clang::TypeLoc)"},
    {"_Z25handleObjCPointerTypeAttrRN12_GLOBAL__N_119TypeProcessingStateERN5clang13AttributeListERNS2_8QualTypeE", "handleObjCPointerTypeAttr((anonymous namespace)::TypeProcessingState&, clang::AttributeList&, clang::QualType&)"},
    {"_Z24HandleNeonVectorTypeAttrRN5clang8QualTypeERKNS_13AttributeListERNS_4SemaENS_10VectorType10VectorKindEPKc", "HandleNeonVectorTypeAttr(clang::QualType&, clang::AttributeList const&, clang::Sema&, clang::VectorType::VectorKind, char const*)"},
    {"_Z22handleFunctionTypeAttrRN12_GLOBAL__N_119TypeProcessingStateERN5clang13AttributeListERNS2_8QualTypeE", "handleFunctionTypeAttr((anonymous namespace)::TypeProcessingState&, clang::AttributeList&, clang::QualType&)"},
    {"_ZN12_GLOBAL__N_121FunctionTypeUnwrapper4wrapERN5clang10ASTContextENS1_8QualTypeEj", "(anonymous namespace)::FunctionTypeUnwrapper::wrap(clang::ASTContext&, clang::QualType, unsigned int)"},
    {"_ZN12_GLOBAL__N_121FunctionTypeUnwrapper4wrapERN5clang10ASTContextEPKNS1_4TypeEj", "(anonymous namespace)::FunctionTypeUnwrapper::wrap(clang::ASTContext&, clang::Type const*, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang15DeclaratorChunkEE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<clang::DeclaratorChunk>::insert(clang::DeclaratorChunk*, clang::DeclaratorChunk const&)"},
    {"_ZN5clang20TargetAttributesSemaD0Ev", "clang::TargetAttributesSema::~TargetAttributesSema()"},
    {"_ZN5clang20TargetAttributesSemaD1Ev", "clang::TargetAttributesSema::~TargetAttributesSema()"},
    {"_ZN5clang20TargetAttributesSemaD2Ev", "clang::TargetAttributesSema::~TargetAttributesSema()"},
    {"_ZNK5clang20TargetAttributesSema20ProcessDeclAttributeEPNS_5ScopeEPNS_4DeclERKNS_13AttributeListERNS_4SemaE", "clang::TargetAttributesSema::ProcessDeclAttribute(clang::Scope*, clang::Decl*, clang::AttributeList const&, clang::Sema&) const"},
    {"_ZNK5clang4Sema23getTargetAttributesSemaEv", "clang::Sema::getTargetAttributesSema() const"},
    {"_ZN12_GLOBAL__N_117X86AttributesSemaD1Ev", "(anonymous namespace)::X86AttributesSema::~X86AttributesSema()"},
    {"_ZN12_GLOBAL__N_117X86AttributesSemaD0Ev", "(anonymous namespace)::X86AttributesSema::~X86AttributesSema()"},
    {"_ZNK12_GLOBAL__N_117X86AttributesSema20ProcessDeclAttributeEPN5clang5ScopeEPNS1_4DeclERKNS1_13AttributeListERNS1_4SemaE", "(anonymous namespace)::X86AttributesSema::ProcessDeclAttribute(clang::Scope*, clang::Decl*, clang::AttributeList const&, clang::Sema&) const"},
    {"_ZN12_GLOBAL__N_120MBlazeAttributesSemaD1Ev", "(anonymous namespace)::MBlazeAttributesSema::~MBlazeAttributesSema()"},
    {"_ZN12_GLOBAL__N_120MBlazeAttributesSemaD0Ev", "(anonymous namespace)::MBlazeAttributesSema::~MBlazeAttributesSema()"},
    {"_ZNK12_GLOBAL__N_120MBlazeAttributesSema20ProcessDeclAttributeEPN5clang5ScopeEPNS1_4DeclERKNS1_13AttributeListERNS1_4SemaE", "(anonymous namespace)::MBlazeAttributesSema::ProcessDeclAttribute(clang::Scope*, clang::Decl*, clang::AttributeList const&, clang::Sema&) const"},
    {"_ZN12_GLOBAL__N_120MSP430AttributesSemaD1Ev", "(anonymous namespace)::MSP430AttributesSema::~MSP430AttributesSema()"},
    {"_ZN12_GLOBAL__N_120MSP430AttributesSemaD0Ev", "(anonymous namespace)::MSP430AttributesSema::~MSP430AttributesSema()"},
    {"_ZNK12_GLOBAL__N_120MSP430AttributesSema20ProcessDeclAttributeEPN5clang5ScopeEPNS1_4DeclERKNS1_13AttributeListERNS1_4SemaE", "(anonymous namespace)::MSP430AttributesSema::ProcessDeclAttribute(clang::Scope*, clang::Decl*, clang::AttributeList const&, clang::Sema&) const"},
    {"_ZN5clang4ento34RegisterAdjustedReturnValueCheckerERNS0_10ExprEngineE", "clang::ento::RegisterAdjustedReturnValueChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_126AdjustedReturnValueCheckerD1Ev", "(anonymous namespace)::AdjustedReturnValueChecker::~AdjustedReturnValueChecker()"},
    {"_ZN12_GLOBAL__N_126AdjustedReturnValueCheckerD0Ev", "(anonymous namespace)::AdjustedReturnValueChecker::~AdjustedReturnValueChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126AdjustedReturnValueCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AdjustedReturnValueChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126AdjustedReturnValueCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AdjustedReturnValueChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126AdjustedReturnValueCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AdjustedReturnValueChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126AdjustedReturnValueCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AdjustedReturnValueChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento22CreateAnalysisConsumerERKNS_12PreprocessorERKSsRKNS_15AnalyzerOptionsE", "clang::ento::CreateAnalysisConsumer(clang::Preprocessor const&, std::string const&, clang::AnalyzerOptions const&)"},
    {"_Z13ActionCFGDumpRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionCFGDump((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z13ActionCFGViewRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionCFGView((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z26ActionDisplayLiveVariablesRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionDisplayLiveVariables((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z29ActionSecuritySyntacticChecksRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionSecuritySyntacticChecks((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z27ActionLLVMConventionCheckerRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerERNS2_19TranslationUnitDeclE", "ActionLLVMConventionChecker((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::TranslationUnitDecl&)"},
    {"_Z20ActionWarnDeadStoresRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnDeadStores((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z20ActionWarnUninitValsRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnUninitVals((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z22ActionWarnObjCMethSigsRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnObjCMethSigs((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z21ActionWarnObjCDeallocRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnObjCDealloc((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z25ActionWarnObjCUnusedIvarsRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnObjCUnusedIvars((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z20ActionObjCMemCheckerRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionObjCMemChecker((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z23ActionWarnSizeofPointerRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclE", "ActionWarnSizeofPointer((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)"},
    {"_Z23ActionObjCMemCheckerAuxRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS2_4DeclEb", "ActionObjCMemCheckerAux((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*, bool)"},
    {"_ZN12_GLOBAL__N_111UbigraphVizD1Ev", "(anonymous namespace)::UbigraphViz::~UbigraphViz()"},
    {"_ZN12_GLOBAL__N_111UbigraphVizD0Ev", "(anonymous namespace)::UbigraphViz::~UbigraphViz()"},
    {"_ZN12_GLOBAL__N_111UbigraphViz7AddEdgeEPN5clang4ento12ExplodedNodeES4_", "(anonymous namespace)::UbigraphViz::AddEdge(clang::ento::ExplodedNode*, clang::ento::ExplodedNode*)"},
    {"_ZNSt6vectorIPFvRN12_GLOBAL__N_116AnalysisConsumerERN5clang4ento15AnalysisManagerEPNS3_4DeclEESaISA_EE9push_backERKSA_", "std::vector<void (*)((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*), std::allocator<void (*)((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)> >::push_back(void (* const&)((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*))"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumerD1Ev", "(anonymous namespace)::AnalysisConsumer::~AnalysisConsumer()"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumerD0Ev", "(anonymous namespace)::AnalysisConsumer::~AnalysisConsumer()"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumer10InitializeERN5clang10ASTContextE", "(anonymous namespace)::AnalysisConsumer::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumer21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::AnalysisConsumer::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumer17HandleDeclContextERN5clang10ASTContextEPNS1_11DeclContextE", "(anonymous namespace)::AnalysisConsumer::HandleDeclContext(clang::ASTContext&, clang::DeclContext*)"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumer15DisplayFunctionEPKN5clang4DeclE", "(anonymous namespace)::AnalysisConsumer::DisplayFunction(clang::Decl const*)"},
    {"_ZN12_GLOBAL__N_116AnalysisConsumer10HandleCodeEPN5clang4DeclERSt6vectorIPFvRS0_RNS1_4ento15AnalysisManagerES3_ESaISA_EE", "(anonymous namespace)::AnalysisConsumer::HandleCode(clang::Decl*, std::vector<void (*)((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*), std::allocator<void (*)((anonymous namespace)::AnalysisConsumer&, clang::ento::AnalysisManager&, clang::Decl*)> >&)"},
    {"_Z10FindBlocksPN5clang11DeclContextERN4llvm15SmallVectorImplIPNS_4DeclEEE", "FindBlocks(clang::DeclContext*, llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang4ento11BugReporter22GeneratePathDiagnosticERNS0_14PathDiagnosticERN4llvm15SmallVectorImplIPNS0_9BugReportEEE", "clang::ento::BugReporter::GeneratePathDiagnostic(clang::ento::PathDiagnostic&, llvm::SmallVectorImpl<clang::ento::BugReport*>&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEEC2Ev", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::ImutAVLFactory()"},
    {"_ZN4llvm8DenseMapIPvjNS_12DenseMapInfoIS1_EENS2_IjEEE16InsertIntoBucketERKS1_RKjPSt4pairIS1_jE", "llvm::DenseMap<void*, unsigned int, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(void* const&, unsigned int const&, std::pair<void*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPvjNS_12DenseMapInfoIS1_EENS2_IjEEE4growEj", "llvm::DenseMap<void*, unsigned int, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang4ento15AnalysisManagerD1Ev", "clang::ento::AnalysisManager::~AnalysisManager()"},
    {"_ZN5clang4ento15AnalysisManagerD0Ev", "clang::ento::AnalysisManager::~AnalysisManager()"},
    {"_ZN5clang4ento15AnalysisManager13getDiagnosticEv", "clang::ento::AnalysisManager::getDiagnostic()"},
    {"_ZN5clang4ento15AnalysisManager23getPathDiagnosticClientEv", "clang::ento::AnalysisManager::getPathDiagnosticClient()"},
    {"_ZN5clang4ento15AnalysisManager13getASTContextEv", "clang::ento::AnalysisManager::getASTContext()"},
    {"_ZN5clang4ento15AnalysisManager16getSourceManagerEv", "clang::ento::AnalysisManager::getSourceManager()"},
    {"_ZN5clang4ento15AnalysisManagerD2Ev", "clang::ento::AnalysisManager::~AnalysisManager()"},
    {"_ZN4llvm10FoldingSetIN5clang15LocationContextEED1Ev", "llvm::FoldingSet<clang::LocationContext>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang15LocationContextEED0Ev", "llvm::FoldingSet<clang::LocationContext>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang15LocationContextEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::LocationContext>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang15LocationContextEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::LocationContext>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang15LocationContextEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::LocationContext>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento28RegisterAnalyzerStatsCheckerERNS0_10ExprEngineE", "clang::ento::RegisterAnalyzerStatsChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_120AnalyzerStatsCheckerD1Ev", "(anonymous namespace)::AnalyzerStatsChecker::~AnalyzerStatsChecker()"},
    {"_ZN12_GLOBAL__N_120AnalyzerStatsCheckerD0Ev", "(anonymous namespace)::AnalyzerStatsChecker::~AnalyzerStatsChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120AnalyzerStatsCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AnalyzerStatsChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120AnalyzerStatsCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AnalyzerStatsChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_120AnalyzerStatsChecker16VisitEndAnalysisERN5clang4ento13ExplodedGraphERNS2_11BugReporterERNS2_10ExprEngineE", "(anonymous namespace)::AnalyzerStatsChecker::VisitEndAnalysis(clang::ento::ExplodedGraph&, clang::ento::BugReporter&, clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120AnalyzerStatsCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AnalyzerStatsChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120AnalyzerStatsCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AnalyzerStatsChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento25RegisterArrayBoundCheckerERNS0_10ExprEngineE", "clang::ento::RegisterArrayBoundChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_117ArrayBoundCheckerD1Ev", "(anonymous namespace)::ArrayBoundChecker::~ArrayBoundChecker()"},
    {"_ZN12_GLOBAL__N_117ArrayBoundCheckerD0Ev", "(anonymous namespace)::ArrayBoundChecker::~ArrayBoundChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ArrayBoundCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ArrayBoundCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_117ArrayBoundChecker13visitLocationERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValEb", "(anonymous namespace)::ArrayBoundChecker::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ArrayBoundCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ArrayBoundCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento27RegisterArrayBoundCheckerV2ERNS0_10ExprEngineE", "clang::ento::RegisterArrayBoundCheckerV2(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_119ArrayBoundCheckerV2D1Ev", "(anonymous namespace)::ArrayBoundCheckerV2::~ArrayBoundCheckerV2()"},
    {"_ZN12_GLOBAL__N_119ArrayBoundCheckerV2D0Ev", "(anonymous namespace)::ArrayBoundCheckerV2::~ArrayBoundCheckerV2()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ArrayBoundCheckerV2EE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundCheckerV2>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ArrayBoundCheckerV2EE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundCheckerV2>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119ArrayBoundCheckerV213visitLocationERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValEb", "(anonymous namespace)::ArrayBoundCheckerV2::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN12_GLOBAL__N_119ArrayBoundCheckerV29reportOOBERN5clang4ento14CheckerContextEPKNS2_7GRStateENS0_8OOB_KindE", "(anonymous namespace)::ArrayBoundCheckerV2::reportOOB(clang::ento::CheckerContext&, clang::ento::GRState const*, (anonymous namespace)::ArrayBoundCheckerV2::OOB_Kind)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ArrayBoundCheckerV2EED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundCheckerV2>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ArrayBoundCheckerV2EED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundCheckerV2>::~CheckerVisitor()"},
    {"_ZN5clang4ento10BuiltinBugC2EPKc", "clang::ento::BuiltinBug::BuiltinBug(char const*)"},
    {"_ZN5clang4ento10BuiltinBugD1Ev", "clang::ento::BuiltinBug::~BuiltinBug()"},
    {"_ZN5clang4ento10BuiltinBugD0Ev", "clang::ento::BuiltinBug::~BuiltinBug()"},
    {"_ZN5clang4ento26RegisterAttrNonNullCheckerERNS0_10ExprEngineE", "clang::ento::RegisterAttrNonNullChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118AttrNonNullCheckerD1Ev", "(anonymous namespace)::AttrNonNullChecker::~AttrNonNullChecker()"},
    {"_ZN12_GLOBAL__N_118AttrNonNullCheckerD0Ev", "(anonymous namespace)::AttrNonNullChecker::~AttrNonNullChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118AttrNonNullCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AttrNonNullChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118AttrNonNullCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::AttrNonNullChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118AttrNonNullCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AttrNonNullChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118AttrNonNullCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::AttrNonNullChecker>::~CheckerVisitor()"},
    {"_ZNK5clang4Decl7getAttrINS_11NonNullAttrEEEPT_v", "clang::NonNullAttr* clang::Decl::getAttr<clang::NonNullAttr>() const"},
    {"_ZN5clang4ento19RegisterAppleChecksERNS0_10ExprEngineERKNS_4DeclE", "clang::ento::RegisterAppleChecks(clang::ento::ExprEngine&, clang::Decl const&)"},
    {"_ZN12_GLOBAL__N_119ClassReleaseCheckerD1Ev", "(anonymous namespace)::ClassReleaseChecker::~ClassReleaseChecker()"},
    {"_ZN12_GLOBAL__N_119ClassReleaseCheckerD0Ev", "(anonymous namespace)::ClassReleaseChecker::~ClassReleaseChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ClassReleaseCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ClassReleaseChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ClassReleaseCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ClassReleaseChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119ClassReleaseChecker19preVisitObjCMessageERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::ClassReleaseChecker::preVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN12_GLOBAL__N_19APIMisuseD1Ev", "(anonymous namespace)::APIMisuse::~APIMisuse()"},
    {"_ZN12_GLOBAL__N_19APIMisuseD0Ev", "(anonymous namespace)::APIMisuse::~APIMisuse()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ClassReleaseCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ClassReleaseChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ClassReleaseCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ClassReleaseChecker>::~CheckerVisitor()"},
    {"_ZN12_GLOBAL__N_122CFRetainReleaseCheckerD1Ev", "(anonymous namespace)::CFRetainReleaseChecker::~CFRetainReleaseChecker()"},
    {"_ZN12_GLOBAL__N_122CFRetainReleaseCheckerD0Ev", "(anonymous namespace)::CFRetainReleaseChecker::~CFRetainReleaseChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_122CFRetainReleaseCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CFRetainReleaseChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_122CFRetainReleaseCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CFRetainReleaseChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_122CFRetainReleaseCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CFRetainReleaseChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_122CFRetainReleaseCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CFRetainReleaseChecker>::~CheckerVisitor()"},
    {"_ZN12_GLOBAL__N_121CFNumberCreateCheckerD1Ev", "(anonymous namespace)::CFNumberCreateChecker::~CFNumberCreateChecker()"},
    {"_ZN12_GLOBAL__N_121CFNumberCreateCheckerD0Ev", "(anonymous namespace)::CFNumberCreateChecker::~CFNumberCreateChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CFNumberCreateCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CFNumberCreateChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CFNumberCreateCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CFNumberCreateChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CFNumberCreateCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CFNumberCreateChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CFNumberCreateCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CFNumberCreateChecker>::~CheckerVisitor()"},
    {"_ZN12_GLOBAL__N_113NilArgCheckerD1Ev", "(anonymous namespace)::NilArgChecker::~NilArgChecker()"},
    {"_ZN12_GLOBAL__N_113NilArgCheckerD0Ev", "(anonymous namespace)::NilArgChecker::~NilArgChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113NilArgCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NilArgChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113NilArgCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NilArgChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_113NilArgChecker19preVisitObjCMessageERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::NilArgChecker::preVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113NilArgCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NilArgChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113NilArgCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NilArgChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerContext12generateNodeEb", "clang::ento::CheckerContext::generateNode(bool)"},
    {"_ZN5clang4ento30RegisterBuiltinFunctionCheckerERNS0_10ExprEngineE", "clang::ento::RegisterBuiltinFunctionChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_122BuiltinFunctionCheckerD1Ev", "(anonymous namespace)::BuiltinFunctionChecker::~BuiltinFunctionChecker()"},
    {"_ZN12_GLOBAL__N_122BuiltinFunctionCheckerD0Ev", "(anonymous namespace)::BuiltinFunctionChecker::~BuiltinFunctionChecker()"},
    {"_ZN12_GLOBAL__N_122BuiltinFunctionChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::BuiltinFunctionChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento22RegisterCStringCheckerERNS0_10ExprEngineE", "clang::ento::RegisterCStringChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_114CStringCheckerD1Ev", "(anonymous namespace)::CStringChecker::~CStringChecker()"},
    {"_ZN12_GLOBAL__N_114CStringCheckerD0Ev", "(anonymous namespace)::CStringChecker::~CStringChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114CStringCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CStringChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114CStringCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CStringChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker15evalDeadSymbolsERN5clang4ento14CheckerContextERNS2_12SymbolReaperE", "(anonymous namespace)::CStringChecker::evalDeadSymbols(clang::ento::CheckerContext&, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_114CStringChecker15MarkLiveSymbolsEPKN5clang4ento7GRStateERNS2_12SymbolReaperE", "(anonymous namespace)::CStringChecker::MarkLiveSymbols(clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_114CStringChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker23wantsRegionChangeUpdateEPKN5clang4ento7GRStateE", "(anonymous namespace)::CStringChecker::wantsRegionChangeUpdate(clang::ento::GRState const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker17EvalRegionChangesEPKN5clang4ento7GRStateEPKPKNS2_9MemRegionESA_Pb", "(anonymous namespace)::CStringChecker::EvalRegionChanges(clang::ento::GRState const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, bool*)"},
    {"_ZNK5clang4ento7GRState3setIN12_GLOBAL__N_113CStringLengthEEEPKS1_NS0_12GRStateTraitIT_E9data_typeE", "clang::ento::GRState const* clang::ento::GRState::set<(anonymous namespace)::CStringLength>(clang::ento::GRStateTrait<(anonymous namespace)::CStringLength>::data_type) const"},
    {"_ZN12_GLOBAL__N_114CStringChecker10evalMemcpyERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalMemcpy(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker10evalMemcmpERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalMemcmp(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker11evalMemmoveERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalMemmove(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker10evalStrcpyERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalStrcpy(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker10evalStpcpyERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalStpcpy(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker13evalstrLengthERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalstrLength(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker9evalBcopyERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CStringChecker::evalBcopy(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker14evalCopyCommonERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4ExprESA_SA_b", "(anonymous namespace)::CStringChecker::evalCopyCommon(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Expr const*, clang::Expr const*, clang::Expr const*, bool)"},
    {"_ZN12_GLOBAL__N_114CStringChecker10assumeZeroERN5clang4ento14CheckerContextEPKNS2_7GRStateENS2_4SValENS1_8QualTypeE", "(anonymous namespace)::CStringChecker::assumeZero(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::ento::SVal, clang::QualType)"},
    {"_ZN12_GLOBAL__N_114CStringChecker17CheckBufferAccessERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4ExprESA_SA_b", "(anonymous namespace)::CStringChecker::CheckBufferAccess(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Expr const*, clang::Expr const*, clang::Expr const*, bool)"},
    {"_ZN12_GLOBAL__N_114CStringChecker16InvalidateBufferERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4ExprENS2_4SValE", "(anonymous namespace)::CStringChecker::InvalidateBuffer(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Expr const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_114CStringChecker14emitOverlapBugERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4StmtESA_", "(anonymous namespace)::CStringChecker::emitOverlapBug(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Stmt const*, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_114CStringChecker12checkNonNullERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4ExprENS2_4SValE", "(anonymous namespace)::CStringChecker::checkNonNull(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Expr const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_114CStringChecker13CheckLocationERN5clang4ento14CheckerContextEPKNS2_7GRStateEPKNS1_4ExprENS2_4SValEb", "(anonymous namespace)::CStringChecker::CheckLocation(clang::ento::CheckerContext&, clang::ento::GRState const*, clang::Expr const*, clang::ento::SVal, bool)"},
    {"_ZN12_GLOBAL__N_114CStringChecker16getCStringLengthERN5clang4ento14CheckerContextERPKNS2_7GRStateEPKNS1_4ExprENS2_4SValE", "(anonymous namespace)::CStringChecker::getCStringLength(clang::ento::CheckerContext&, clang::ento::GRState const*&, clang::Expr const*, clang::ento::SVal)"},
    {"_ZNK5clang4ento7GRState3setIN12_GLOBAL__N_113CStringLengthEEEPKS1_NS0_12GRStateTraitIT_E8key_typeENS9_10value_typeE", "clang::ento::GRState const* clang::ento::GRState::set<(anonymous namespace)::CStringLength>(clang::ento::GRStateTrait<(anonymous namespace)::CStringLength>::key_type, clang::ento::GRStateTrait<(anonymous namespace)::CStringLength>::value_type) const"},
    {"_ZN12_GLOBAL__N_114CStringChecker16evalStrcpyCommonERN5clang4ento14CheckerContextEPKNS1_8CallExprEb", "(anonymous namespace)::CStringChecker::evalStrcpyCommon(clang::ento::CheckerContext&, clang::CallExpr const*, bool)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114CStringCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CStringChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114CStringCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CStringChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_9MemRegionENS0_4SValENS2_16ImutKeyValueInfoIS6_S7_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_9MemRegionENS0_4SValENS2_16ImutKeyValueInfoIS6_S7_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> > >::DeleteContext(void*)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_9MemRegionENS0_4SValENS2_16ImutKeyValueInfoIS6_S7_EEEEE3SetESA_S6_S7_RNSA_7FactoryE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> > >::Set(llvm::ImmutableMap<clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >, clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImmutableMap<clang::ento::MemRegion const*, clang::ento::SVal, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::Factory&)"},
    {"_ZN5clang4ento29RegisterCallAndMessageCheckerERNS0_10ExprEngineE", "clang::ento::RegisterCallAndMessageChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageCheckerD1Ev", "(anonymous namespace)::CallAndMessageChecker::~CallAndMessageChecker()"},
    {"_ZN12_GLOBAL__N_121CallAndMessageCheckerD0Ev", "(anonymous namespace)::CallAndMessageChecker::~CallAndMessageChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CallAndMessageCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CallAndMessageChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CallAndMessageCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CallAndMessageChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageChecker19preVisitObjCMessageERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::CallAndMessageChecker::preVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageChecker15evalNilReceiverERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::CallAndMessageChecker::evalNilReceiver(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageChecker18emitNilReceiverBugERN5clang4ento14CheckerContextERKNS2_11ObjCMessageEPNS2_12ExplodedNodeE", "(anonymous namespace)::CallAndMessageChecker::emitNilReceiverBug(clang::ento::CheckerContext&, clang::ento::ObjCMessage const&, clang::ento::ExplodedNode*)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageChecker19PreVisitProcessArgsERN5clang4ento14CheckerContextENS2_17CallOrObjCMessageEPKcRPNS2_7BugTypeE", "(anonymous namespace)::CallAndMessageChecker::PreVisitProcessArgs(clang::ento::CheckerContext&, clang::ento::CallOrObjCMessage, char const*, clang::ento::BugType*&)"},
    {"_ZZN12_GLOBAL__N_121CallAndMessageChecker18PreVisitProcessArgERN5clang4ento14CheckerContextENS2_4SValENS1_11SourceRangeEPKNS1_4ExprEPKcRPNS2_7BugTypeEEN22FindUninitializedField4FindEPKNS2_11TypedRegionE", "(anonymous namespace)::CallAndMessageChecker::PreVisitProcessArg(clang::ento::CheckerContext&, clang::ento::SVal, clang::SourceRange, clang::Expr const*, char const*, clang::ento::BugType*&)::FindUninitializedField::Find(clang::ento::TypedRegion const*)"},
    {"_ZN12_GLOBAL__N_121CallAndMessageChecker11EmitBadCallEPN5clang4ento7BugTypeERNS2_14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::CallAndMessageChecker::EmitBadCall(clang::ento::BugType*, clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CallAndMessageCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CallAndMessageChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CallAndMessageCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CallAndMessageChecker>::~CheckerVisitor()"},
    {"_ZNK5clang4ento17CallOrObjCMessage10getArgSValEj", "clang::ento::CallOrObjCMessage::getArgSVal(unsigned int) const"},
    {"_ZN5clang4ento23RegisterCastSizeCheckerERNS0_10ExprEngineE", "clang::ento::RegisterCastSizeChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_115CastSizeCheckerD1Ev", "(anonymous namespace)::CastSizeChecker::~CastSizeChecker()"},
    {"_ZN12_GLOBAL__N_115CastSizeCheckerD0Ev", "(anonymous namespace)::CastSizeChecker::~CastSizeChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_115CastSizeCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CastSizeChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_115CastSizeCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CastSizeChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_115CastSizeChecker16PreVisitCastExprERN5clang4ento14CheckerContextEPKNS1_8CastExprE", "(anonymous namespace)::CastSizeChecker::PreVisitCastExpr(clang::ento::CheckerContext&, clang::CastExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_115CastSizeCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CastSizeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_115CastSizeCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CastSizeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento10BuiltinBugC2EPKcS3_", "clang::ento::BuiltinBug::BuiltinBug(char const*, char const*)"},
    {"_ZN5clang4ento27RegisterCastToStructCheckerERNS0_10ExprEngineE", "clang::ento::RegisterCastToStructChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_119CastToStructCheckerD1Ev", "(anonymous namespace)::CastToStructChecker::~CastToStructChecker()"},
    {"_ZN12_GLOBAL__N_119CastToStructCheckerD0Ev", "(anonymous namespace)::CastToStructChecker::~CastToStructChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119CastToStructCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CastToStructChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119CastToStructCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::CastToStructChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119CastToStructChecker16PreVisitCastExprERN5clang4ento14CheckerContextEPKNS1_8CastExprE", "(anonymous namespace)::CastToStructChecker::PreVisitCastExpr(clang::ento::CheckerContext&, clang::CastExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119CastToStructCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CastToStructChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119CastToStructCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::CastToStructChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento16CheckObjCDeallocEPKNS_22ObjCImplementationDeclERKNS_11LangOptionsERNS0_11BugReporterE", "clang::ento::CheckObjCDealloc(clang::ObjCImplementationDecl const*, clang::LangOptions const&, clang::ento::BugReporter&)"},
    {"_Z12scan_deallocPN5clang4StmtENS_8SelectorE", "scan_dealloc(clang::Stmt*, clang::Selector)"},
    {"_Z17scan_ivar_releasePN5clang4StmtEPNS_12ObjCIvarDeclEPKNS_16ObjCPropertyDeclENS_8SelectorEPNS_14IdentifierInfoERNS_10ASTContextE", "scan_ivar_release(clang::Stmt*, clang::ObjCIvarDecl*, clang::ObjCPropertyDecl const*, clang::Selector, clang::IdentifierInfo*, clang::ASTContext&)"},
    {"_ZN5clang4ento26CheckObjCInstMethSignatureEPKNS_22ObjCImplementationDeclERNS0_11BugReporterE", "clang::ento::CheckObjCInstMethSignature(clang::ObjCImplementationDecl const*, clang::ento::BugReporter&)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPNS1_14ObjCMethodDeclENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<clang::Selector, clang::ObjCMethodDecl*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::ObjCMethodDecl*> >::InsertIntoBucket(clang::Selector const&, clang::ObjCMethodDecl* const&, std::pair<clang::Selector, clang::ObjCMethodDecl*>*)"},
    {"_ZN4llvm8DenseMapIN5clang8SelectorEPNS1_14ObjCMethodDeclENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::Selector, clang::ObjCMethodDecl*, llvm::DenseMapInfo<clang::Selector>, llvm::DenseMapInfo<clang::ObjCMethodDecl*> >::grow(unsigned int)"},
    {"_ZN5clang4ento23CheckSecuritySyntaxOnlyEPKNS_4DeclERNS0_11BugReporterE", "clang::ento::CheckSecuritySyntaxOnly(clang::Decl const*, clang::ento::BugReporter&)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_17WalkASTEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::WalkAST, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_17WalkAST13VisitCallExprEPN5clang8CallExprE", "(anonymous namespace)::WalkAST::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN12_GLOBAL__N_17WalkAST13VisitChildrenEPN5clang4StmtE", "(anonymous namespace)::WalkAST::VisitChildren(clang::Stmt*)"},
    {"_Z17GetIncrementedVarPKN5clang4ExprEPKNS_7VarDeclES5_", "GetIncrementedVar(clang::Expr const*, clang::VarDecl const*, clang::VarDecl const*)"},
    {"_ZN5clang4ento18CheckSizeofPointerEPKNS_4DeclERNS0_11BugReporterE", "clang::ento::CheckSizeofPointer(clang::Decl const*, clang::ento::BugReporter&)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_17WalkASTEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::WalkAST, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_17WalkAST13VisitChildrenEPN5clang4StmtE", "(anonymous namespace)::WalkAST::VisitChildren(clang::Stmt*)"},
    {"_ZN5clang4ento21RegisterChrootCheckerERNS0_10ExprEngineE", "clang::ento::RegisterChrootChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_113ChrootCheckerD1Ev", "(anonymous namespace)::ChrootChecker::~ChrootChecker()"},
    {"_ZN12_GLOBAL__N_113ChrootCheckerD0Ev", "(anonymous namespace)::ChrootChecker::~ChrootChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113ChrootCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ChrootChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113ChrootCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ChrootChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_113ChrootChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::ChrootChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_113ChrootChecker16PreVisitCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::ChrootChecker::PreVisitCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113ChrootCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ChrootChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113ChrootCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ChrootChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento15CheckDeadStoresERNS_3CFGERNS_13LiveVariablesERNS_9ParentMapERNS0_11BugReporterE", "clang::ento::CheckDeadStores(clang::CFG&, clang::LiveVariables&, clang::ParentMap&, clang::ento::BugReporter&)"},
    {"_ZN12_GLOBAL__N_112DeadStoreObsD1Ev", "(anonymous namespace)::DeadStoreObs::~DeadStoreObs()"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_111FindEscapedEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::FindEscaped, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111FindEscaped18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::FindEscaped::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_111FindEscapedEvE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::FindEscaped, void>::VisitChildren(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_112DeadStoreObsD0Ev", "(anonymous namespace)::DeadStoreObs::~DeadStoreObs()"},
    {"_ZN12_GLOBAL__N_112DeadStoreObs11ObserveStmtEPN5clang4StmtERKNS1_24LiveVariables_ValueTypes14AnalysisDataTyERKNS1_23StmtDeclBitVector_Types5ValTyE", "(anonymous namespace)::DeadStoreObs::ObserveStmt(clang::Stmt*, clang::LiveVariables_ValueTypes::AnalysisDataTy const&, clang::StmtDeclBitVector_Types::ValTy const&)"},
    {"_ZN12_GLOBAL__N_112DeadStoreObs12CheckVarDeclEPN5clang7VarDeclEPNS1_4ExprES5_NS0_13DeadStoreKindERKNS1_24LiveVariables_ValueTypes14AnalysisDataTyERKNS1_23StmtDeclBitVector_Types5ValTyE", "(anonymous namespace)::DeadStoreObs::CheckVarDecl(clang::VarDecl*, clang::Expr*, clang::Expr*, (anonymous namespace)::DeadStoreObs::DeadStoreKind, clang::LiveVariables_ValueTypes::AnalysisDataTy const&, clang::StmtDeclBitVector_Types::ValTy const&)"},
    {"_ZN12_GLOBAL__N_112DeadStoreObs6ReportEPN5clang7VarDeclENS0_13DeadStoreKindENS1_14SourceLocationENS1_11SourceRangeE", "(anonymous namespace)::DeadStoreObs::Report(clang::VarDecl*, (anonymous namespace)::DeadStoreObs::DeadStoreKind, clang::SourceLocation, clang::SourceRange)"},
    {"_ZN5clang24LiveVariables_ValueTypes10ObserverTy12ObserverKillEPNS_11DeclRefExprE", "clang::LiveVariables_ValueTypes::ObserverTy::ObserverKill(clang::DeclRefExpr*)"},
    {"_ZN5clang24LiveVariables_ValueTypes10ObserverTyD1Ev", "clang::LiveVariables_ValueTypes::ObserverTy::~ObserverTy()"},
    {"_ZN5clang24LiveVariables_ValueTypes10ObserverTyD0Ev", "clang::LiveVariables_ValueTypes::ObserverTy::~ObserverTy()"},
    {"_ZN5clang24LiveVariables_ValueTypes10ObserverTy11ObserveStmtEPNS_4StmtERKNS0_14AnalysisDataTyERKNS_23StmtDeclBitVector_Types5ValTyE", "clang::LiveVariables_ValueTypes::ObserverTy::ObserveStmt(clang::Stmt*, clang::LiveVariables_ValueTypes::AnalysisDataTy const&, clang::StmtDeclBitVector_Types::ValTy const&)"},
    {"_ZN5clang4ento26RegisterDereferenceCheckerERNS0_10ExprEngineE", "clang::ento::RegisterDereferenceChecker(clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento27GetImplicitNullDereferencesERNS0_10ExprEngineE", "clang::ento::GetImplicitNullDereferences(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118DereferenceCheckerD1Ev", "(anonymous namespace)::DereferenceChecker::~DereferenceChecker()"},
    {"_ZN12_GLOBAL__N_118DereferenceCheckerD0Ev", "(anonymous namespace)::DereferenceChecker::~DereferenceChecker()"},
    {"_ZN12_GLOBAL__N_118DereferenceChecker13visitLocationERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValEb", "(anonymous namespace)::DereferenceChecker::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN12_GLOBAL__N_118DereferenceChecker14AddDerefSourceERN4llvm11raw_ostreamERNS1_15SmallVectorImplIN5clang11SourceRangeEEEPKNS5_4ExprEb", "(anonymous namespace)::DereferenceChecker::AddDerefSource(llvm::raw_ostream&, llvm::SmallVectorImpl<clang::SourceRange>&, clang::Expr const*, bool)"},
    {"_ZN5clang4ento22RegisterDivZeroCheckerERNS0_10ExprEngineE", "clang::ento::RegisterDivZeroChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_114DivZeroCheckerD1Ev", "(anonymous namespace)::DivZeroChecker::~DivZeroChecker()"},
    {"_ZN12_GLOBAL__N_114DivZeroCheckerD0Ev", "(anonymous namespace)::DivZeroChecker::~DivZeroChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114DivZeroCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::DivZeroChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114DivZeroCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::DivZeroChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_114DivZeroChecker22PreVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::DivZeroChecker::PreVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114DivZeroCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::DivZeroChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114DivZeroCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::DivZeroChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento26RegisterExperimentalChecksERNS0_10ExprEngineE", "clang::ento::RegisterExperimentalChecks(clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento34RegisterExperimentalInternalChecksERNS0_10ExprEngineE", "clang::ento::RegisterExperimentalInternalChecks(clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento10ExprEngine12CheckerVisitEPKNS_4StmtERNS0_15ExplodedNodeSetES6_NS1_12CallbackKindE", "clang::ento::ExprEngine::CheckerVisit(clang::Stmt const*, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine::CallbackKind)"},
    {"_ZN5clang4ento10ExprEngine23CheckerVisitObjCMessageERKNS0_11ObjCMessageERNS0_15ExplodedNodeSetES6_b", "clang::ento::ExprEngine::CheckerVisitObjCMessage(clang::ento::ObjCMessage const&, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&, bool)"},
    {"_ZN5clang4ento10ExprEngine22CheckerEvalNilReceiverERKNS0_11ObjCMessageERNS0_15ExplodedNodeSetEPKNS0_7GRStateEPNS0_12ExplodedNodeE", "clang::ento::ExprEngine::CheckerEvalNilReceiver(clang::ento::ObjCMessage const&, clang::ento::ExplodedNodeSet&, clang::ento::GRState const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10ExprEngine15CheckerEvalCallEPKNS_8CallExprERNS0_15ExplodedNodeSetEPNS0_12ExplodedNodeE", "clang::ento::ExprEngine::CheckerEvalCall(clang::CallExpr const*, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10ExprEngine16CheckerVisitBindEPKNS_4StmtERNS0_15ExplodedNodeSetES6_NS0_4SValES7_b", "clang::ento::ExprEngine::CheckerVisitBind(clang::Stmt const*, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&, clang::ento::SVal, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento10ExprEngineC1ERNS0_15AnalysisManagerEPNS0_13TransferFuncsE", "clang::ento::ExprEngine::ExprEngine(clang::ento::AnalysisManager&, clang::ento::TransferFuncs*)"},
    {"_ZN5clang4ento10ExprEngineC2ERNS0_15AnalysisManagerEPNS0_13TransferFuncsE", "clang::ento::ExprEngine::ExprEngine(clang::ento::AnalysisManager&, clang::ento::TransferFuncs*)"},
    {"_ZN5clang4ento10ExprEngineD0Ev", "clang::ento::ExprEngine::~ExprEngine()"},
    {"_ZN5clang4ento10ExprEngineD1Ev", "clang::ento::ExprEngine::~ExprEngine()"},
    {"_ZN5clang4ento10ExprEngineD2Ev", "clang::ento::ExprEngine::~ExprEngine()"},
    {"_ZN5clang4ento10ExprEngine15getInitialStateEPKNS_15LocationContextE", "clang::ento::ExprEngine::getInitialState(clang::LocationContext const*)"},
    {"_ZN5clang4ento10ExprEngine13processAssumeEPKNS0_7GRStateENS0_4SValEb", "clang::ento::ExprEngine::processAssume(clang::ento::GRState const*, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento10ExprEngine23wantsRegionChangeUpdateEPKNS0_7GRStateE", "clang::ento::ExprEngine::wantsRegionChangeUpdate(clang::ento::GRState const*)"},
    {"_ZN5clang4ento10ExprEngine20processRegionChangesEPKNS0_7GRStateEPKPKNS0_9MemRegionES9_", "clang::ento::ExprEngine::processRegionChanges(clang::ento::GRState const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*)"},
    {"_ZN5clang4ento10ExprEngine18processEndWorklistEb", "clang::ento::ExprEngine::processEndWorklist(bool)"},
    {"_ZN5clang4ento10ExprEngine17processCFGElementENS_10CFGElementERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::processCFGElement(clang::CFGElement, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine11ProcessStmtENS_7CFGStmtERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessStmt(clang::CFGStmt, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine18ProcessInitializerENS_14CFGInitializerERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessInitializer(clang::CFGInitializer, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine19ProcessImplicitDtorENS_15CFGImplicitDtorERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessImplicitDtor(clang::CFGImplicitDtor, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine5VisitEPKNS_4StmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::Visit(clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine23ProcessAutomaticObjDtorENS_19CFGAutomaticObjDtorERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessAutomaticObjDtor(clang::CFGAutomaticObjDtor, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine15ProcessBaseDtorENS_11CFGBaseDtorERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessBaseDtor(clang::CFGBaseDtor, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine17ProcessMemberDtorENS_13CFGMemberDtorERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessMemberDtor(clang::CFGMemberDtor, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine20ProcessTemporaryDtorENS_16CFGTemporaryDtorERNS0_15StmtNodeBuilderE", "clang::ento::ExprEngine::ProcessTemporaryDtor(clang::CFGTemporaryDtor, clang::ento::StmtNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine8MakeNodeERNS0_15ExplodedNodeSetEPKNS_4StmtEPNS0_12ExplodedNodeEPKNS0_7GRStateENS_12ProgramPoint4KindEPKv", "clang::ento::ExprEngine::MakeNode(clang::ento::ExplodedNodeSet&, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ProgramPoint::Kind, void const*)"},
    {"_ZN5clang4ento10ExprEngine27VisitObjCAtSynchronizedStmtEPKNS_22ObjCAtSynchronizedStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitObjCAtSynchronizedStmt(clang::ObjCAtSynchronizedStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine24VisitObjCPropertyRefExprEPKNS_19ObjCPropertyRefExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitObjCPropertyRefExpr(clang::ObjCPropertyRefExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine27VisitLvalArraySubscriptExprEPKNS_18ArraySubscriptExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitLvalArraySubscriptExpr(clang::ArraySubscriptExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine12VisitAsmStmtEPKNS_7AsmStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitAsmStmt(clang::AsmStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine22VisitCommonDeclRefExprEPKNS_4ExprEPKNS_9NamedDeclEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCommonDeclRefExpr(clang::Expr const*, clang::NamedDecl const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine14VisitBlockExprEPKNS_9BlockExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitBlockExpr(clang::BlockExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine16VisitLogicalExprEPKNS_14BinaryOperatorEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitLogicalExpr(clang::BinaryOperator const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine19VisitBinaryOperatorEPKNS_14BinaryOperatorEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitBinaryOperator(clang::BinaryOperator const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine17evalEagerlyAssumeERNS0_15ExplodedNodeSetES3_PKNS_4ExprE", "clang::ento::ExprEngine::evalEagerlyAssume(clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&, clang::Expr const*)"},
    {"_ZN5clang4ento10ExprEngine9VisitCallEPKNS_8CallExprEPNS0_12ExplodedNodeENS_17ConstExprIteratorES7_RNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCall(clang::CallExpr const*, clang::ento::ExplodedNode*, clang::ConstExprIterator, clang::ConstExprIterator, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine16VisitGuardedExprEPKNS_4ExprES4_S4_PNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitGuardedExpr(clang::Expr const*, clang::Expr const*, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine24VisitCompoundLiteralExprEPKNS_19CompoundLiteralExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCompoundLiteralExpr(clang::CompoundLiteralExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine13VisitDeclStmtEPKNS_8DeclStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitDeclStmt(clang::DeclStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine13VisitCondInitEPKNS_7VarDeclEPKNS_4StmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCondInit(clang::VarDecl const*, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine9VisitCastEPKNS_8CastExprEPKNS_4ExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCast(clang::CastExpr const*, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine17VisitInitListExprEPKNS_12InitListExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitInitListExpr(clang::InitListExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine15VisitMemberExprEPKNS_10MemberExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitMemberExpr(clang::MemberExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine24VisitLvalObjCIvarRefExprEPKNS_15ObjCIvarRefExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitLvalObjCIvarRefExpr(clang::ObjCIvarRefExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine26VisitObjCForCollectionStmtEPKNS_21ObjCForCollectionStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitObjCForCollectionStmt(clang::ObjCForCollectionStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine20VisitObjCMessageExprEPKNS_15ObjCMessageExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitObjCMessageExpr(clang::ObjCMessageExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine15VisitReturnStmtEPKNS_10ReturnStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitReturnStmt(clang::ReturnStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine17VisitOffsetOfExprEPKNS_12OffsetOfExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitOffsetOfExpr(clang::OffsetOfExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine22VisitSizeOfAlignOfExprEPKNS_17SizeOfAlignOfExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitSizeOfAlignOfExpr(clang::SizeOfAlignOfExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine18VisitUnaryOperatorEPKNS_13UnaryOperatorEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitUnaryOperator(clang::UnaryOperator const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine23processCFGBlockEntranceERNS0_15ExplodedNodeSetERNS0_18GenericNodeBuilderINS_13BlockEntranceEEE", "clang::ento::ExprEngine::processCFGBlockEntrance(clang::ento::ExplodedNodeSet&, clang::ento::GenericNodeBuilder<clang::BlockEntrance>&)"},
    {"_ZN5clang4ento10ExprEngine10MarkBranchEPKNS0_7GRStateEPKNS_4StmtEb", "clang::ento::ExprEngine::MarkBranch(clang::ento::GRState const*, clang::Stmt const*, bool)"},
    {"_ZN5clang4ento10ExprEngine13processBranchEPKNS_4StmtES4_RNS0_17BranchNodeBuilderE", "clang::ento::ExprEngine::processBranch(clang::Stmt const*, clang::Stmt const*, clang::ento::BranchNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine19processIndirectGotoERNS0_23IndirectGotoNodeBuilderE", "clang::ento::ExprEngine::processIndirectGoto(clang::ento::IndirectGotoNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine20processEndOfFunctionERNS0_24EndOfFunctionNodeBuilderE", "clang::ento::ExprEngine::processEndOfFunction(clang::ento::EndOfFunctionNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine13processSwitchERNS0_17SwitchNodeBuilderE", "clang::ento::ExprEngine::processSwitch(clang::ento::SwitchNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine16processCallEnterERNS0_20CallEnterNodeBuilderE", "clang::ento::ExprEngine::processCallEnter(clang::ento::CallEnterNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine15processCallExitERNS0_19CallExitNodeBuilderE", "clang::ento::ExprEngine::processCallExit(clang::ento::CallExitNodeBuilder&)"},
    {"_ZN5clang4ento10ExprEngine8evalLoadERNS0_15ExplodedNodeSetEPKNS_4ExprEPNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValEPKvNS_8QualTypeE", "clang::ento::ExprEngine::evalLoad(clang::ento::ExplodedNodeSet&, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, void const*, clang::QualType)"},
    {"_ZN5clang4ento10ExprEngine8evalBindERNS0_15ExplodedNodeSetEPKNS_4StmtEPNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValESC_b", "clang::ento::ExprEngine::evalBind(clang::ento::ExplodedNodeSet&, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento10ExprEngine9evalStoreERNS0_15ExplodedNodeSetEPKNS_4ExprES6_PNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValESC_PKv", "clang::ento::ExprEngine::evalStore(clang::ento::ExplodedNodeSet&, clang::Expr const*, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, clang::ento::SVal, void const*)"},
    {"_ZN5clang4ento10ExprEngine16VisitObjCMessageERKNS0_11ObjCMessageERNS0_15ExplodedNodeSetES6_", "clang::ento::ExprEngine::VisitObjCMessage(clang::ento::ObjCMessage const&, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine12evalLocationERNS0_15ExplodedNodeSetEPKNS_4StmtEPNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValEPKvb", "clang::ento::ExprEngine::evalLocation(clang::ento::ExplodedNodeSet&, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, void const*, bool)"},
    {"_ZN5clang4ento10ExprEngine14evalLoadCommonERNS0_15ExplodedNodeSetEPKNS_4ExprEPNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValEPKvNS_8QualTypeE", "clang::ento::ExprEngine::evalLoadCommon(clang::ento::ExplodedNodeSet&, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, void const*, clang::QualType)"},
    {"_ZN5clang4ento10ExprEngine10InlineCallERNS0_15ExplodedNodeSetEPKNS_8CallExprEPNS0_12ExplodedNodeE", "clang::ento::ExprEngine::InlineCall(clang::ento::ExplodedNodeSet&, clang::CallExpr const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10ExprEngine29VisitObjCForCollectionStmtAuxEPKNS_21ObjCForCollectionStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetENS0_4SValE", "clang::ento::ExprEngine::VisitObjCForCollectionStmtAux(clang::ObjCForCollectionStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&, clang::ento::SVal)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_113ObjCMsgWLItemEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::ObjCMsgWLItem>::push_back((anonymous namespace)::ObjCMsgWLItem const&)"},
    {"_ZN5clang4ento10ExprEngine25VisitAsmStmtHelperOutputsEPKNS_7AsmStmtENS_17ConstExprIteratorES5_PNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitAsmStmtHelperOutputs(clang::AsmStmt const*, clang::ConstExprIterator, clang::ConstExprIterator, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine24VisitAsmStmtHelperInputsEPKNS_7AsmStmtENS_17ConstExprIteratorES5_PNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitAsmStmtHelperInputs(clang::AsmStmt const*, clang::ConstExprIterator, clang::ConstExprIterator, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZNK5clang4ento10ExprEngine13lookupCheckerEPv", "clang::ento::ExprEngine::lookupChecker(void*) const"},
    {"_ZN5clang4ento10ExprEngine9ViewGraphEb", "clang::ento::ExprEngine::ViewGraph(bool)"},
    {"_ZN5clang4ento10ExprEngine9ViewGraphEPPNS0_12ExplodedNodeES4_", "clang::ento::ExprEngine::ViewGraph(clang::ento::ExplodedNode**, clang::ento::ExplodedNode**)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_114InitListWLItemELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::InitListWLItem, false>::grow(unsigned long)"},
    {"_ZN5clang4ento15ExplodedNodeSet6insertERKS1_", "clang::ento::ExplodedNodeSet::insert(clang::ento::ExplodedNodeSet const&)"},
    {"_ZN5clang4ento7Checker8GR_VisitERNS0_15ExplodedNodeSetERNS0_15StmtNodeBuilderERNS0_10ExprEngineEPKNS_4StmtEPNS0_12ExplodedNodeEPvbRb", "clang::ento::Checker::GR_Visit(clang::ento::ExplodedNodeSet&, clang::ento::StmtNodeBuilder&, clang::ento::ExprEngine&, clang::Stmt const*, clang::ento::ExplodedNode*, void*, bool, bool&)"},
    {"_ZN5clang4ento7Checker19GR_visitObjCMessageERNS0_15ExplodedNodeSetERNS0_15StmtNodeBuilderERNS0_10ExprEngineERKNS0_11ObjCMessageEPNS0_12ExplodedNodeEPvb", "clang::ento::Checker::GR_visitObjCMessage(clang::ento::ExplodedNodeSet&, clang::ento::StmtNodeBuilder&, clang::ento::ExprEngine&, clang::ento::ObjCMessage const&, clang::ento::ExplodedNode*, void*, bool)"},
    {"_ZN5clang4ento10ExprEngine18getAnalysisManagerEv", "clang::ento::ExprEngine::getAnalysisManager()"},
    {"_ZN5clang4ento10ExprEngine15getStateManagerEv", "clang::ento::ExprEngine::getStateManager()"},
    {"_ZN4llvm8DenseMapIjPSt6vectorISt4pairIPvPN5clang4ento7CheckerEESaIS8_EENS_12DenseMapInfoIjEENSC_ISB_EEE16InsertIntoBucketERKjRKSB_PS2_IjSB_E", "llvm::DenseMap<unsigned int, std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >*> >::InsertIntoBucket(unsigned int const&, std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >* const&, std::pair<unsigned int, std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >*>*)"},
    {"_ZN4llvm8DenseMapIjPSt6vectorISt4pairIPvPN5clang4ento7CheckerEESaIS8_EENS_12DenseMapInfoIjEENSC_ISB_EEE4growEj", "llvm::DenseMap<unsigned int, std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >*> >::grow(unsigned int)"},
    {"_ZN4llvm20ImmutableListFactoryIN5clang4ento4SValEE6concatERKS3_NS_13ImmutableListIS3_EE", "llvm::ImmutableListFactory<clang::ento::SVal>::concat(clang::ento::SVal const&, llvm::ImmutableList<clang::ento::SVal>)"},
    {"_ZN5clang4ento18StmtNodeBuilderRefD2Ev", "clang::ento::StmtNodeBuilderRef::~StmtNodeBuilderRef()"},
    {"_ZN5clang4ento12SymbolReaperC2EPKNS_15LocationContextEPKNS_4StmtERNS0_13SymbolManagerE", "clang::ento::SymbolReaper::SymbolReaper(clang::LocationContext const*, clang::Stmt const*, clang::ento::SymbolManager&)"},
    {"_ZN5clang4ento9SubEngineD1Ev", "clang::ento::SubEngine::~SubEngine()"},
    {"_ZN5clang4ento9SubEngineD0Ev", "clang::ento::SubEngine::~SubEngine()"},
    {"_ZN5clang4ento10CoreEngineD2Ev", "clang::ento::CoreEngine::~CoreEngine()"},
    {"_ZN5clang4ento14GRStateManagerC2ERNS_10ASTContextEPFPNS0_12StoreManagerERS1_EPFPNS0_17ConstraintManagerES6_RNS0_9SubEngineEERN4llvm16BumpPtrAllocatorESC_", "clang::ento::GRStateManager::GRStateManager(clang::ASTContext&, clang::ento::StoreManager* (*)(clang::ento::GRStateManager&), clang::ento::ConstraintManager* (*)(clang::ento::GRStateManager&, clang::ento::SubEngine&), llvm::BumpPtrAllocator&, clang::ento::SubEngine&)"},
    {"_ZN4llvm10FoldingSetIN5clang4ento7GRStateEED1Ev", "llvm::FoldingSet<clang::ento::GRState>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento7GRStateEED0Ev", "llvm::FoldingSet<clang::ento::GRState>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7GRStateEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::GRState>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7GRStateEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::GRState>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7GRStateEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::GRState>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang4ento12ExplodedNodeEED1Ev", "llvm::FoldingSet<clang::ento::ExplodedNode>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento12ExplodedNodeEED0Ev", "llvm::FoldingSet<clang::ento::ExplodedNode>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento12ExplodedNodeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::ExplodedNode>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento12ExplodedNodeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::ExplodedNode>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento12ExplodedNodeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::ExplodedNode>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento27RegisterFixedAddressCheckerERNS0_10ExprEngineE", "clang::ento::RegisterFixedAddressChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_119FixedAddressCheckerD1Ev", "(anonymous namespace)::FixedAddressChecker::~FixedAddressChecker()"},
    {"_ZN12_GLOBAL__N_119FixedAddressCheckerD0Ev", "(anonymous namespace)::FixedAddressChecker::~FixedAddressChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119FixedAddressCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::FixedAddressChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119FixedAddressCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::FixedAddressChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119FixedAddressChecker22PreVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::FixedAddressChecker::PreVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119FixedAddressCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::FixedAddressChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119FixedAddressCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::FixedAddressChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14AnalysisAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::ento::AnalysisAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang4ento14AnalysisActionD1Ev", "clang::ento::AnalysisAction::~AnalysisAction()"},
    {"_ZN5clang4ento14AnalysisActionD0Ev", "clang::ento::AnalysisAction::~AnalysisAction()"},
    {"_ZN5clang4ento34RegisterIdempotentOperationCheckerERNS0_10ExprEngineE", "clang::ento::RegisterIdempotentOperationChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationCheckerD1Ev", "(anonymous namespace)::IdempotentOperationChecker::~IdempotentOperationChecker()"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationCheckerD0Ev", "(anonymous namespace)::IdempotentOperationChecker::~IdempotentOperationChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126IdempotentOperationCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::IdempotentOperationChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126IdempotentOperationCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::IdempotentOperationChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationChecker16VisitEndAnalysisERN5clang4ento13ExplodedGraphERNS2_11BugReporterERNS2_10ExprEngineE", "(anonymous namespace)::IdempotentOperationChecker::VisitEndAnalysis(clang::ento::ExplodedGraph&, clang::ento::BugReporter&, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationChecker23CFGReachabilityAnalysis11isReachableEPKN5clang8CFGBlockES5_", "(anonymous namespace)::IdempotentOperationChecker::CFGReachabilityAnalysis::isReachable(clang::CFGBlock const*, clang::CFGBlock const*)"},
    {"_ZZN12_GLOBAL__N_126IdempotentOperationChecker25PathWasCompletelyAnalyzedEPKN5clang3CFGEPKNS1_8CFGBlockEPKNS1_10CFGStmtMapERKNS1_4ento10CoreEngineEEN7VisitWLD1Ev", "(anonymous namespace)::IdempotentOperationChecker::PathWasCompletelyAnalyzed(clang::CFG const*, clang::CFGBlock const*, clang::CFGStmtMap const*, clang::ento::CoreEngine const&)::VisitWL::~VisitWL()"},
    {"_ZZN12_GLOBAL__N_126IdempotentOperationChecker25PathWasCompletelyAnalyzedEPKN5clang3CFGEPKNS1_8CFGBlockEPKNS1_10CFGStmtMapERKNS1_4ento10CoreEngineEEN7VisitWLD0Ev", "(anonymous namespace)::IdempotentOperationChecker::PathWasCompletelyAnalyzed(clang::CFG const*, clang::CFGBlock const*, clang::CFGStmtMap const*, clang::ento::CoreEngine const&)::VisitWL::~VisitWL()"},
    {"_ZZN12_GLOBAL__N_126IdempotentOperationChecker25PathWasCompletelyAnalyzedEPKN5clang3CFGEPKNS1_8CFGBlockEPKNS1_10CFGStmtMapERKNS1_4ento10CoreEngineEEN7VisitWL5visitERKNSB_12WorkListUnitE", "(anonymous namespace)::IdempotentOperationChecker::PathWasCompletelyAnalyzed(clang::CFG const*, clang::CFGBlock const*, clang::CFGStmtMap const*, clang::ento::CoreEngine const&)::VisitWL::visit(clang::ento::WorkListUnit const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang14BinaryOperatorEN12_GLOBAL__N_126IdempotentOperationChecker18BinaryOperatorDataENS_12DenseMapInfoIS4_EENS8_IS7_EEEixERKS4_", "llvm::DenseMap<clang::BinaryOperator const*, (anonymous namespace)::IdempotentOperationChecker::BinaryOperatorData, llvm::DenseMapInfo<clang::BinaryOperator const*>, llvm::DenseMapInfo<(anonymous namespace)::IdempotentOperationChecker::BinaryOperatorData> >::operator[](clang::BinaryOperator const* const&)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationChecker22PreVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::IdempotentOperationChecker::PreVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationChecker7CanVaryEPKN5clang4ExprEPNS1_15AnalysisContextE", "(anonymous namespace)::IdempotentOperationChecker::CanVary(clang::Expr const*, clang::AnalysisContext*)"},
    {"_ZN12_GLOBAL__N_126IdempotentOperationChecker23containsNonLocalVarDeclEPKN5clang4StmtE", "(anonymous namespace)::IdempotentOperationChecker::containsNonLocalVarDecl(clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126IdempotentOperationCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::IdempotentOperationChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126IdempotentOperationCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::IdempotentOperationChecker>::~CheckerVisitor()"},
    {"_ZNSt6vectorISt4pairIPFvRN5clang4ento18BugReporterContextEPKvPKNS2_12ExplodedNodeEES6_ESaISC_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSC_SE_EERKSC_", "std::vector<std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*>, std::allocator<std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*>*, std::vector<std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*>, std::allocator<std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*> > > >, std::pair<void (*)(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*), void const*> const&)"},
    {"_ZN4llvm8DenseMapIjNS_8SmallSetIjLj32EEENS_12DenseMapInfoIjEENS3_IS2_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, llvm::SmallSet<unsigned int, 32u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallSet<unsigned int, 32u> > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjNS_8SmallSetIjLj32EEENS_12DenseMapInfoIjEENS3_IS2_EEE16InsertIntoBucketERKjRKS2_PSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::SmallSet<unsigned int, 32u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallSet<unsigned int, 32u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallSet<unsigned int, 32u> const&, std::pair<unsigned int, llvm::SmallSet<unsigned int, 32u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_8SmallSetIjLj32EEENS_12DenseMapInfoIjEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallSet<unsigned int, 32u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallSet<unsigned int, 32u> > >::grow(unsigned int)"},
    {"_ZNSt5dequeIPKN5clang8CFGBlockESaIS3_EE9push_backERKS3_", "std::deque<clang::CFGBlock const*, std::allocator<clang::CFGBlock const*> >::push_back(clang::CFGBlock const* const&)"},
    {"_ZNSt5dequeIPKN5clang8CFGBlockESaIS3_EE17_M_reallocate_mapEmb", "std::deque<clang::CFGBlock const*, std::allocator<clang::CFGBlock const*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIPKN5clang8CFGBlockESaIS3_EE17_M_initialize_mapEm", "std::_Deque_base<clang::CFGBlock const*, std::allocator<clang::CFGBlock const*> >::_M_initialize_map(unsigned long)"},
    {"_ZN4llvm8DenseMapIjNS_8SmallSetIjLj32EEENS_12DenseMapInfoIjEENS3_IS2_EEED1Ev", "llvm::DenseMap<unsigned int, llvm::SmallSet<unsigned int, 32u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallSet<unsigned int, 32u> > >::~DenseMap()"},
    {"_ZN5clang4ento20CheckLLVMConventionsERNS_19TranslationUnitDeclERNS0_11BugReporterE", "clang::ento::CheckLLVMConventions(clang::TranslationUnitDecl&, clang::ento::BugReporter&)"},
    {"_Z13ScanCodeDeclsPN5clang11DeclContextERNS_4ento11BugReporterE", "ScanCodeDecls(clang::DeclContext*, clang::ento::BugReporter&)"},
    {"_Z11IsPartOfASTPKN5clang13CXXRecordDeclE", "IsPartOfAST(clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_115ASTFieldVisitor5VisitEPN5clang9FieldDeclE", "(anonymous namespace)::ASTFieldVisitor::Visit(clang::FieldDecl*)"},
    {"_Z11IsStdStringN5clang8QualTypeE", "IsStdString(clang::QualType)"},
    {"_Z11InNamespacePKN5clang4DeclEN4llvm9StringRefE", "InNamespace(clang::Decl const*, llvm::StringRef)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_123StringRefCheckerVisitorEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::StringRefCheckerVisitor, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_123StringRefCheckerVisitor13VisitChildrenEPN5clang4StmtE", "(anonymous namespace)::StringRefCheckerVisitor::VisitChildren(clang::Stmt*)"},
    {"_ZN5clang4ento24RegisterMacOSXAPICheckerERNS0_10ExprEngineE", "clang::ento::RegisterMacOSXAPIChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_116MacOSXAPICheckerD1Ev", "(anonymous namespace)::MacOSXAPIChecker::~MacOSXAPIChecker()"},
    {"_ZN12_GLOBAL__N_116MacOSXAPICheckerD0Ev", "(anonymous namespace)::MacOSXAPIChecker::~MacOSXAPIChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_116MacOSXAPICheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::MacOSXAPIChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_116MacOSXAPICheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::MacOSXAPIChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_Z17CheckDispatchOnceRN5clang4ento14CheckerContextEPKNS_8CallExprERPNS0_7BugTypeEPKNS_14IdentifierInfoE", "CheckDispatchOnce(clang::ento::CheckerContext&, clang::CallExpr const*, clang::ento::BugType*&, clang::IdentifierInfo const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_116MacOSXAPICheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::MacOSXAPIChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_116MacOSXAPICheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::MacOSXAPIChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento21RegisterMallocCheckerERNS0_10ExprEngineE", "clang::ento::RegisterMallocChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_113MallocCheckerD1Ev", "(anonymous namespace)::MallocChecker::~MallocChecker()"},
    {"_ZN12_GLOBAL__N_113MallocCheckerD0Ev", "(anonymous namespace)::MallocChecker::~MallocChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113MallocCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::MallocChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113MallocCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::MallocChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_113MallocChecker13visitLocationERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValEb", "(anonymous namespace)::MallocChecker::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN12_GLOBAL__N_113MallocChecker12PreVisitBindERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValES8_", "(anonymous namespace)::MallocChecker::PreVisitBind(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_113MallocChecker15evalDeadSymbolsERN5clang4ento14CheckerContextERNS2_12SymbolReaperE", "(anonymous namespace)::MallocChecker::evalDeadSymbols(clang::ento::CheckerContext&, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_113MallocChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineE", "(anonymous namespace)::MallocChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_113MallocChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::MallocChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_113MallocChecker10evalAssumeEPKN5clang4ento7GRStateENS2_4SValEbPb", "(anonymous namespace)::MallocChecker::evalAssume(clang::ento::GRState const*, clang::ento::SVal, bool, bool*)"},
    {"_ZNK5clang4ento7GRState3setIN12_GLOBAL__N_111RegionStateEEEPKS1_NS0_12GRStateTraitIT_E8key_typeENS9_10value_typeE", "clang::ento::GRState const* clang::ento::GRState::set<(anonymous namespace)::RegionState>(clang::ento::GRStateTrait<(anonymous namespace)::RegionState>::key_type, clang::ento::GRStateTrait<(anonymous namespace)::RegionState>::value_type) const"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEEENS_12DenseMapInfoIjEENSD_ISC_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEEESaISC_EE9push_backERKSC_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >* const&)"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::operator++()"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_18RefStateENS2_16ImutKeyValueInfoIS6_S8_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefState, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_18RefStateENS2_16ImutKeyValueInfoIS6_S8_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefState, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE12add_internalERKSt4pairIS6_S8_EPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::add_internal(std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE13markImmutableEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE10createNodeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE11balanceTreeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN12_GLOBAL__N_113MallocChecker10FreeMemAuxERN5clang4ento14CheckerContextEPKNS1_8CallExprEPKNS2_7GRStateEjb", "(anonymous namespace)::MallocChecker::FreeMemAux(clang::ento::CheckerContext&, clang::CallExpr const*, clang::ento::GRState const*, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_113MallocChecker13ReportBadFreeERN5clang4ento14CheckerContextENS2_4SValENS1_11SourceRangeE", "(anonymous namespace)::MallocChecker::ReportBadFree(clang::ento::CheckerContext&, clang::ento::SVal, clang::SourceRange)"},
    {"_ZN12_GLOBAL__N_113MallocChecker12MallocMemAuxERN5clang4ento14CheckerContextEPKNS1_8CallExprENS2_4SValES8_PKNS2_7GRStateE", "(anonymous namespace)::MallocChecker::MallocMemAux(clang::ento::CheckerContext&, clang::CallExpr const*, clang::ento::SVal, clang::ento::SVal, clang::ento::GRState const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE15remove_internalERKS6_PNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::remove_internal(clang::ento::SymbolData const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RefStateEEEE16removeMinBindingEPNS_11ImutAVLTreeIS9_EERSD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefState> >*&)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113MallocCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::MallocChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113MallocCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::MallocChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento31RegisterNSAutoreleasePoolChecksERNS0_10ExprEngineE", "clang::ento::RegisterNSAutoreleasePoolChecks(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_124NSAutoreleasePoolCheckerD1Ev", "(anonymous namespace)::NSAutoreleasePoolChecker::~NSAutoreleasePoolChecker()"},
    {"_ZN12_GLOBAL__N_124NSAutoreleasePoolCheckerD0Ev", "(anonymous namespace)::NSAutoreleasePoolChecker::~NSAutoreleasePoolChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_124NSAutoreleasePoolCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NSAutoreleasePoolChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_124NSAutoreleasePoolCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NSAutoreleasePoolChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_124NSAutoreleasePoolChecker19preVisitObjCMessageERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::NSAutoreleasePoolChecker::preVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_124NSAutoreleasePoolCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NSAutoreleasePoolChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_124NSAutoreleasePoolCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NSAutoreleasePoolChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento21RegisterNSErrorChecksERNS0_11BugReporterERNS0_10ExprEngineERKNS_4DeclE", "clang::ento::RegisterNSErrorChecks(clang::ento::BugReporter&, clang::ento::ExprEngine&, clang::Decl const&)"},
    {"_ZN12_GLOBAL__N_114NSErrorCheckerC1ERKN5clang4DeclEbRNS1_4ento10ExprEngineE", "(anonymous namespace)::NSErrorChecker::NSErrorChecker(clang::Decl const&, bool, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_114NSErrorCheckerD1Ev", "(anonymous namespace)::NSErrorChecker::~NSErrorChecker()"},
    {"_ZN12_GLOBAL__N_114NSErrorCheckerD0Ev", "(anonymous namespace)::NSErrorChecker::~NSErrorChecker()"},
    {"_ZN12_GLOBAL__N_114NSErrorChecker12FlushReportsERN5clang4ento11BugReporterE", "(anonymous namespace)::NSErrorChecker::FlushReports(clang::ento::BugReporter&)"},
    {"_ZN12_GLOBAL__N_114NSErrorChecker20CheckNSErrorArgumentEN5clang8QualTypeE", "(anonymous namespace)::NSErrorChecker::CheckNSErrorArgument(clang::QualType)"},
    {"_ZN5clang4ento31RegisterNoReturnFunctionCheckerERNS0_10ExprEngineE", "clang::ento::RegisterNoReturnFunctionChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_123NoReturnFunctionCheckerD1Ev", "(anonymous namespace)::NoReturnFunctionChecker::~NoReturnFunctionChecker()"},
    {"_ZN12_GLOBAL__N_123NoReturnFunctionCheckerD0Ev", "(anonymous namespace)::NoReturnFunctionChecker::~NoReturnFunctionChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_123NoReturnFunctionCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NoReturnFunctionChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_123NoReturnFunctionCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::NoReturnFunctionChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_123NoReturnFunctionCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NoReturnFunctionChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_123NoReturnFunctionCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::NoReturnFunctionChecker>::~CheckerVisitor()"},
    {"_ZNK5clang4Decl7getAttrINS_20AnalyzerNoReturnAttrEEEPT_v", "clang::AnalyzerNoReturnAttr* clang::Decl::getAttr<clang::AnalyzerNoReturnAttr>() const"},
    {"_ZN5clang4ento23RegisterOSAtomicCheckerERNS0_10ExprEngineE", "clang::ento::RegisterOSAtomicChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_115OSAtomicCheckerD1Ev", "(anonymous namespace)::OSAtomicChecker::~OSAtomicChecker()"},
    {"_ZN12_GLOBAL__N_115OSAtomicCheckerD0Ev", "(anonymous namespace)::OSAtomicChecker::~OSAtomicChecker()"},
    {"_ZN12_GLOBAL__N_115OSAtomicChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::OSAtomicChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento25RegisterObjCAtSyncCheckerERNS0_10ExprEngineE", "clang::ento::RegisterObjCAtSyncChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_117ObjCAtSyncCheckerD1Ev", "(anonymous namespace)::ObjCAtSyncChecker::~ObjCAtSyncChecker()"},
    {"_ZN12_GLOBAL__N_117ObjCAtSyncCheckerD0Ev", "(anonymous namespace)::ObjCAtSyncChecker::~ObjCAtSyncChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ObjCAtSyncCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCAtSyncChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ObjCAtSyncCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCAtSyncChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ObjCAtSyncCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCAtSyncChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ObjCAtSyncCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCAtSyncChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento27registerObjCSelfInitCheckerERNS0_10ExprEngineE", "clang::ento::registerObjCSelfInitChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitCheckerD1Ev", "(anonymous namespace)::ObjCSelfInitChecker::~ObjCSelfInitChecker()"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitCheckerD0Ev", "(anonymous namespace)::ObjCSelfInitChecker::~ObjCSelfInitChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ObjCSelfInitCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCSelfInitChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ObjCSelfInitCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCSelfInitChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitChecker20postVisitObjCMessageERN5clang4ento14CheckerContextENS2_11ObjCMessageE", "(anonymous namespace)::ObjCSelfInitChecker::postVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitChecker13visitLocationERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValEb", "(anonymous namespace)::ObjCSelfInitChecker::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_Z11addSelfFlagN5clang4ento4SValEN12_GLOBAL__N_112SelfFlagEnumERNS0_14CheckerContextE", "addSelfFlag(clang::ento::SVal, (anonymous namespace)::SelfFlagEnum, clang::ento::CheckerContext&)"},
    {"_Z12getSelfFlagsN5clang4ento4SValERNS0_14CheckerContextE", "getSelfFlags(clang::ento::SVal, clang::ento::CheckerContext&)"},
    {"_Z27shouldRunOnFunctionOrMethodPKN5clang9NamedDeclE", "shouldRunOnFunctionOrMethod(clang::NamedDecl const*)"},
    {"_Z19checkForInvalidSelfPKN5clang4ExprERNS_4ento14CheckerContextEPKc", "checkForInvalidSelf(clang::Expr const*, clang::ento::CheckerContext&, char const*)"},
    {"_ZN12_GLOBAL__N_111InitSelfBugD1Ev", "(anonymous namespace)::InitSelfBug::~InitSelfBug()"},
    {"_ZN12_GLOBAL__N_111InitSelfBugD0Ev", "(anonymous namespace)::InitSelfBug::~InitSelfBug()"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitChecker20PostVisitGenericCallERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::ObjCSelfInitChecker::PostVisitGenericCall(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_119ObjCSelfInitChecker19PreVisitGenericCallERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::ObjCSelfInitChecker::PreVisitGenericCall(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ObjCSelfInitCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCSelfInitChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ObjCSelfInitCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ObjCSelfInitChecker>::~CheckerVisitor()"},
    {"_ZNSt6vectorISt4pairIPvPN5clang4ento7CheckerEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<void*, clang::ento::Checker*>*, std::vector<std::pair<void*, clang::ento::Checker*>, std::allocator<std::pair<void*, clang::ento::Checker*> > > >, std::pair<void*, clang::ento::Checker*> const&)"},
    {"_ZN5clang4ento7Checker19preVisitObjCMessageERNS0_14CheckerContextENS0_11ObjCMessageE", "clang::ento::Checker::preVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN5clang4ento7Checker12PreVisitBindERNS0_14CheckerContextEPKNS_4StmtENS0_4SValES7_", "clang::ento::Checker::PreVisitBind(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN5clang4ento7Checker15evalDeadSymbolsERNS0_14CheckerContextERNS0_12SymbolReaperE", "clang::ento::Checker::evalDeadSymbols(clang::ento::CheckerContext&, clang::ento::SymbolReaper&)"},
    {"_ZN5clang4ento7Checker11evalEndPathERNS0_24EndOfFunctionNodeBuilderEPvRNS0_10ExprEngineE", "clang::ento::Checker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento7Checker15MarkLiveSymbolsEPKNS0_7GRStateERNS0_12SymbolReaperE", "clang::ento::Checker::MarkLiveSymbols(clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN5clang4ento7Checker20VisitBranchConditionERNS0_17BranchNodeBuilderERNS0_10ExprEngineEPKNS_4StmtEPv", "clang::ento::Checker::VisitBranchCondition(clang::ento::BranchNodeBuilder&, clang::ento::ExprEngine&, clang::Stmt const*, void*)"},
    {"_ZN5clang4ento7Checker15evalNilReceiverERNS0_14CheckerContextENS0_11ObjCMessageE", "clang::ento::Checker::evalNilReceiver(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN5clang4ento7Checker12evalCallExprERNS0_14CheckerContextEPKNS_8CallExprE", "clang::ento::Checker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento7Checker10evalAssumeEPKNS0_7GRStateENS0_4SValEbPb", "clang::ento::Checker::evalAssume(clang::ento::GRState const*, clang::ento::SVal, bool, bool*)"},
    {"_ZN5clang4ento7Checker23wantsRegionChangeUpdateEPKNS0_7GRStateE", "clang::ento::Checker::wantsRegionChangeUpdate(clang::ento::GRState const*)"},
    {"_ZN5clang4ento7Checker17EvalRegionChangesEPKNS0_7GRStateEPKPKNS0_9MemRegionES9_Pb", "clang::ento::Checker::EvalRegionChanges(clang::ento::GRState const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, bool*)"},
    {"_ZN5clang4ento7Checker16VisitEndAnalysisERNS0_13ExplodedGraphERNS0_11BugReporterERNS0_10ExprEngineE", "clang::ento::Checker::VisitEndAnalysis(clang::ento::ExplodedGraph&, clang::ento::BugReporter&, clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento14CheckerContext13addTransitionEPKNS0_7GRStateEPKv", "clang::ento::CheckerContext::addTransition(clang::ento::GRState const*, void const*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEEESaISA_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSA_SC_EERKSA_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEEENS_12DenseMapInfoIjEENSB_ISA_EEE16InsertIntoBucketERKjRKSA_PSt4pairIjSA_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEEENS_12DenseMapInfoIjEENSB_ISA_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*> >::grow(unsigned int)"},
    {"_ZN5clang4ento14GRStateManager3setIN4llvm12ImmutableMapIPKNS0_10SymbolDataEjNS3_16ImutKeyValueInfoIS7_jEEEEEEPKNS0_7GRStateESD_NS0_12GRStateTraitIT_E8key_typeENSG_10value_typeENSG_12context_typeE", "clang::ento::GRState const* clang::ento::GRStateManager::set<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >(clang::ento::GRState const*, clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::key_type, clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::value_type, clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::context_type)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEjNS2_16ImutKeyValueInfoIS6_jEEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEjNS2_16ImutKeyValueInfoIS6_jEEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEjNS2_16ImutKeyValueInfoIS6_jEEEEE3SetES9_S6_jRNS9_7FactoryE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::Set(llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, clang::ento::SymbolData const*, unsigned int, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::Factory&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE3addEPNS_11ImutAVLTreeIS7_EERKSt4pairIS6_jE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, std::pair<clang::ento::SymbolData const*, unsigned int> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS7_EERNS_26ImutAVLTreeInOrderIteratorIS7_EESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE12add_internalERKSt4pairIS6_jEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::add_internal(std::pair<clang::ento::SymbolData const*, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE13markImmutableEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE10createNodeEPNS_11ImutAVLTreeIS7_EERKSt4pairIS6_jESB_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, std::pair<clang::ento::SymbolData const*, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE11balanceTreeEPNS_11ImutAVLTreeIS7_EERKSt4pairIS6_jESB_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*, std::pair<clang::ento::SymbolData const*, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >*)"},
    {"_ZN4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEED1Ev", "llvm::FoldingSet<clang::ento::BugReportEquivClass>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEED0Ev", "llvm::FoldingSet<clang::ento::BugReportEquivClass>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::BugReportEquivClass>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::BugReportEquivClass>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::BugReportEquivClass>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento17EnhancedBugReportD1Ev", "clang::ento::EnhancedBugReport::~EnhancedBugReport()"},
    {"_ZN5clang4ento17EnhancedBugReportD0Ev", "clang::ento::EnhancedBugReport::~EnhancedBugReport()"},
    {"_ZN5clang4ento9BugReport24isOwnedByReporterContextEv", "clang::ento::BugReport::isOwnedByReporterContext()"},
    {"_ZNK5clang4ento9BugReport7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::BugReport::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento9BugReport23getExtraDescriptiveTextEv", "clang::ento::BugReport::getExtraDescriptiveText()"},
    {"_ZNK5clang4ento15RangedBugReport9getRangesEv", "clang::ento::RangedBugReport::getRanges() const"},
    {"_ZN5clang4ento17EnhancedBugReport23registerInitialVisitorsERNS0_18BugReporterContextEPKNS0_12ExplodedNodeE", "clang::ento::EnhancedBugReport::registerInitialVisitors(clang::ento::BugReporterContext&, clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento9BugReport23registerInitialVisitorsERNS0_18BugReporterContextEPKNS0_12ExplodedNodeE", "clang::ento::BugReport::registerInitialVisitors(clang::ento::BugReporterContext&, clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento18BugReporterVisitor24isOwnedByReporterContextEv", "clang::ento::BugReporterVisitor::isOwnedByReporterContext()"},
    {"_ZN5clang4ento7Checker20postVisitObjCMessageERNS0_14CheckerContextENS0_11ObjCMessageE", "clang::ento::Checker::postVisitObjCMessage(clang::ento::CheckerContext&, clang::ento::ObjCMessage)"},
    {"_ZN5clang4ento7Checker13visitLocationERNS0_14CheckerContextEPKNS_4StmtENS0_4SValEb", "clang::ento::Checker::visitLocation(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento7Checker9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::Checker::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento7Checker10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::Checker::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento19CheckObjCUnusedIvarEPKNS_22ObjCImplementationDeclERNS0_11BugReporterE", "clang::ento::CheckObjCUnusedIvar(clang::ObjCImplementationDecl const*, clang::ento::BugReporter&)"},
    {"_Z4ScanRN4llvm8DenseMapIPKN5clang12ObjCIvarDeclE9IVarStateNS_12DenseMapInfoIS4_EENS6_IS5_EEEEPKNS1_17ObjCContainerDeclE", "Scan(llvm::DenseMap<clang::ObjCIvarDecl const*, IVarState, llvm::DenseMapInfo<clang::ObjCIvarDecl const*>, llvm::DenseMapInfo<IVarState> >&, clang::ObjCContainerDecl const*)"},
    {"_Z4ScanRN4llvm8DenseMapIPKN5clang12ObjCIvarDeclE9IVarStateNS_12DenseMapInfoIS4_EENS6_IS5_EEEEPKNS1_4StmtE", "Scan(llvm::DenseMap<clang::ObjCIvarDecl const*, IVarState, llvm::DenseMapInfo<clang::ObjCIvarDecl const*>, llvm::DenseMapInfo<IVarState> >&, clang::Stmt const*)"},
    {"_ZNK5clang4Decl7getAttrINS_10UnusedAttrEEEPT_v", "clang::UnusedAttr* clang::Decl::getAttr<clang::UnusedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_12IBOutletAttrEEEPT_v", "clang::IBOutletAttr* clang::Decl::getAttr<clang::IBOutletAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_22IBOutletCollectionAttrEEEPT_v", "clang::IBOutletCollectionAttr* clang::Decl::getAttr<clang::IBOutletCollectionAttr>() const"},
    {"_ZN4llvm8DenseMapIPKN5clang12ObjCIvarDeclE9IVarStateNS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::ObjCIvarDecl const*, IVarState, llvm::DenseMapInfo<clang::ObjCIvarDecl const*>, llvm::DenseMapInfo<IVarState> >::InsertIntoBucket(clang::ObjCIvarDecl const* const&, IVarState const&, std::pair<clang::ObjCIvarDecl const*, IVarState>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang12ObjCIvarDeclE9IVarStateNS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::ObjCIvarDecl const*, IVarState, llvm::DenseMapInfo<clang::ObjCIvarDecl const*>, llvm::DenseMapInfo<IVarState> >::grow(unsigned int)"},
    {"_ZN5clang4ento27RegisterPointerArithCheckerERNS0_10ExprEngineE", "clang::ento::RegisterPointerArithChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_119PointerArithCheckerD1Ev", "(anonymous namespace)::PointerArithChecker::~PointerArithChecker()"},
    {"_ZN12_GLOBAL__N_119PointerArithCheckerD0Ev", "(anonymous namespace)::PointerArithChecker::~PointerArithChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119PointerArithCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerArithChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119PointerArithCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerArithChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_119PointerArithChecker22PreVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::PointerArithChecker::PreVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119PointerArithCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerArithChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119PointerArithCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerArithChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento25RegisterPointerSubCheckerERNS0_10ExprEngineE", "clang::ento::RegisterPointerSubChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_117PointerSubCheckerD1Ev", "(anonymous namespace)::PointerSubChecker::~PointerSubChecker()"},
    {"_ZN12_GLOBAL__N_117PointerSubCheckerD0Ev", "(anonymous namespace)::PointerSubChecker::~PointerSubChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117PointerSubCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerSubChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117PointerSubCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerSubChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_117PointerSubChecker22PreVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::PointerSubChecker::PreVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117PointerSubCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerSubChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117PointerSubCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PointerSubChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento26RegisterPthreadLockCheckerERNS0_10ExprEngineE", "clang::ento::RegisterPthreadLockChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118PthreadLockCheckerD1Ev", "(anonymous namespace)::PthreadLockChecker::~PthreadLockChecker()"},
    {"_ZN12_GLOBAL__N_118PthreadLockCheckerD0Ev", "(anonymous namespace)::PthreadLockChecker::~PthreadLockChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118PthreadLockCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PthreadLockChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118PthreadLockCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::PthreadLockChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_118PthreadLockChecker11AcquireLockERN5clang4ento14CheckerContextEPKNS1_8CallExprENS2_4SValEb", "(anonymous namespace)::PthreadLockChecker::AcquireLock(clang::ento::CheckerContext&, clang::CallExpr const*, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118PthreadLockCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PthreadLockChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118PthreadLockCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::PthreadLockChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableSetIPKNS0_9MemRegionENS2_17ImutContainerInfoIS6_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableSetIPKNS0_9MemRegionENS2_17ImutContainerInfoIS6_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE6removeEPNS_11ImutAVLTreeIS7_EES6_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::remove(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, clang::ento::MemRegion const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE15remove_internalES6_PNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::remove_internal(clang::ento::MemRegion const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE16removeMinBindingEPNS_11ImutAVLTreeIS7_EERSB_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*&)"},
    {"_ZN5clang4ento33RegisterReturnPointerRangeCheckerERNS0_10ExprEngineE", "clang::ento::RegisterReturnPointerRangeChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_125ReturnPointerRangeCheckerD1Ev", "(anonymous namespace)::ReturnPointerRangeChecker::~ReturnPointerRangeChecker()"},
    {"_ZN12_GLOBAL__N_125ReturnPointerRangeCheckerD0Ev", "(anonymous namespace)::ReturnPointerRangeChecker::~ReturnPointerRangeChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_125ReturnPointerRangeCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnPointerRangeChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_125ReturnPointerRangeCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnPointerRangeChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_125ReturnPointerRangeCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnPointerRangeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_125ReturnPointerRangeCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnPointerRangeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento26RegisterReturnUndefCheckerERNS0_10ExprEngineE", "clang::ento::RegisterReturnUndefChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118ReturnUndefCheckerD1Ev", "(anonymous namespace)::ReturnUndefChecker::~ReturnUndefChecker()"},
    {"_ZN12_GLOBAL__N_118ReturnUndefCheckerD0Ev", "(anonymous namespace)::ReturnUndefChecker::~ReturnUndefChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118ReturnUndefCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnUndefChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118ReturnUndefCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnUndefChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118ReturnUndefCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnUndefChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118ReturnUndefCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::ReturnUndefChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento28RegisterStackAddrLeakCheckerERNS0_10ExprEngineE", "clang::ento::RegisterStackAddrLeakChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_120StackAddrLeakCheckerD1Ev", "(anonymous namespace)::StackAddrLeakChecker::~StackAddrLeakChecker()"},
    {"_ZN12_GLOBAL__N_120StackAddrLeakCheckerD0Ev", "(anonymous namespace)::StackAddrLeakChecker::~StackAddrLeakChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120StackAddrLeakCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::StackAddrLeakChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120StackAddrLeakCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::StackAddrLeakChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_120StackAddrLeakChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineE", "(anonymous namespace)::StackAddrLeakChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_120StackAddrLeakChecker7GenNameERN4llvm11raw_ostreamEPKN5clang4ento9MemRegionERNS4_13SourceManagerE", "(anonymous namespace)::StackAddrLeakChecker::GenName(llvm::raw_ostream&, clang::ento::MemRegion const*, clang::SourceManager&)"},
    {"_ZZN12_GLOBAL__N_120StackAddrLeakChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineEEN8CallBackD1Ev", "(anonymous namespace)::StackAddrLeakChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)::CallBack::~CallBack()"},
    {"_ZZN12_GLOBAL__N_120StackAddrLeakChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineEEN8CallBackD0Ev", "(anonymous namespace)::StackAddrLeakChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)::CallBack::~CallBack()"},
    {"_ZZN12_GLOBAL__N_120StackAddrLeakChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineEEN8CallBack13HandleBindingERNS2_12StoreManagerEPKvPKNS2_9MemRegionENS2_4SValE", "(anonymous namespace)::StackAddrLeakChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)::CallBack::HandleBinding(clang::ento::StoreManager&, void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120StackAddrLeakCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::StackAddrLeakChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120StackAddrLeakCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::StackAddrLeakChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento21RegisterStreamCheckerERNS0_10ExprEngineE", "clang::ento::RegisterStreamChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_113StreamCheckerD1Ev", "(anonymous namespace)::StreamChecker::~StreamChecker()"},
    {"_ZN12_GLOBAL__N_113StreamCheckerD0Ev", "(anonymous namespace)::StreamChecker::~StreamChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113StreamCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::StreamChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113StreamCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::StreamChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_113StreamChecker15evalDeadSymbolsERN5clang4ento14CheckerContextERNS2_12SymbolReaperE", "(anonymous namespace)::StreamChecker::evalDeadSymbols(clang::ento::CheckerContext&, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_113StreamChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineE", "(anonymous namespace)::StreamChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_113StreamChecker12evalCallExprERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::StreamChecker::evalCallExpr(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_113StreamChecker15CheckNullStreamEN5clang4ento4SValEPKNS2_7GRStateERNS2_14CheckerContextE", "(anonymous namespace)::StreamChecker::CheckNullStream(clang::ento::SVal, clang::ento::GRState const*, clang::ento::CheckerContext&)"},
    {"_ZNK5clang4ento7GRState3setIN12_GLOBAL__N_111StreamStateEEEPKS1_NS0_12GRStateTraitIT_E8key_typeENS9_10value_typeE", "clang::ento::GRState const* clang::ento::GRState::set<(anonymous namespace)::StreamState>(clang::ento::GRStateTrait<(anonymous namespace)::StreamState>::key_type, clang::ento::GRStateTrait<(anonymous namespace)::StreamState>::value_type) const"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_111StreamStateENS2_16ImutKeyValueInfoIS6_S8_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::StreamState, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_111StreamStateENS2_16ImutKeyValueInfoIS6_S8_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::StreamState, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> > >::DeleteContext(void*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEEENS_12DenseMapInfoIjEENSD_ISC_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEEESaISC_EE9push_backERKSC_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >* const&)"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE12add_internalERKSt4pairIS6_S8_EPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::add_internal(std::pair<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE13markImmutableEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE10createNodeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_111StreamStateEEEE11balanceTreeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::StreamState> >*)"},
    {"_ZN12_GLOBAL__N_113StreamChecker11OpenFileAuxERN5clang4ento14CheckerContextEPKNS1_8CallExprE", "(anonymous namespace)::StreamChecker::OpenFileAux(clang::ento::CheckerContext&, clang::CallExpr const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113StreamCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::StreamChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113StreamCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::StreamChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento26RegisterUndefBranchCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUndefBranchChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118UndefBranchCheckerD1Ev", "(anonymous namespace)::UndefBranchChecker::~UndefBranchChecker()"},
    {"_ZN12_GLOBAL__N_118UndefBranchCheckerD0Ev", "(anonymous namespace)::UndefBranchChecker::~UndefBranchChecker()"},
    {"_ZN12_GLOBAL__N_118UndefBranchChecker20VisitBranchConditionERN5clang4ento17BranchNodeBuilderERNS2_10ExprEngineEPKNS1_4StmtEPv", "(anonymous namespace)::UndefBranchChecker::VisitBranchCondition(clang::ento::BranchNodeBuilder&, clang::ento::ExprEngine&, clang::Stmt const*, void*)"},
    {"_ZN12_GLOBAL__N_118UndefBranchChecker13FindUndefExpr8FindExprEPKN5clang4ExprE", "(anonymous namespace)::UndefBranchChecker::FindUndefExpr::FindExpr(clang::Expr const*)"},
    {"_ZN5clang4ento36RegisterUndefCapturedBlockVarCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUndefCapturedBlockVarChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_128UndefCapturedBlockVarCheckerD1Ev", "(anonymous namespace)::UndefCapturedBlockVarChecker::~UndefCapturedBlockVarChecker()"},
    {"_ZN12_GLOBAL__N_128UndefCapturedBlockVarCheckerD0Ev", "(anonymous namespace)::UndefCapturedBlockVarChecker::~UndefCapturedBlockVarChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_128UndefCapturedBlockVarCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefCapturedBlockVarChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_128UndefCapturedBlockVarCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefCapturedBlockVarChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_Z20FindBlockDeclRefExprPKN5clang4StmtEPKNS_7VarDeclE", "FindBlockDeclRefExpr(clang::Stmt const*, clang::VarDecl const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_128UndefCapturedBlockVarCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefCapturedBlockVarChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_128UndefCapturedBlockVarCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefCapturedBlockVarChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento26RegisterUndefResultCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUndefResultChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_118UndefResultCheckerD1Ev", "(anonymous namespace)::UndefResultChecker::~UndefResultChecker()"},
    {"_ZN12_GLOBAL__N_118UndefResultCheckerD0Ev", "(anonymous namespace)::UndefResultChecker::~UndefResultChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118UndefResultCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefResultChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118UndefResultCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefResultChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_118UndefResultChecker23PostVisitBinaryOperatorERN5clang4ento14CheckerContextEPKNS1_14BinaryOperatorE", "(anonymous namespace)::UndefResultChecker::PostVisitBinaryOperator(clang::ento::CheckerContext&, clang::BinaryOperator const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118UndefResultCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefResultChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118UndefResultCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefResultChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento38RegisterUndefinedArraySubscriptCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUndefinedArraySubscriptChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_130UndefinedArraySubscriptCheckerD1Ev", "(anonymous namespace)::UndefinedArraySubscriptChecker::~UndefinedArraySubscriptChecker()"},
    {"_ZN12_GLOBAL__N_130UndefinedArraySubscriptCheckerD0Ev", "(anonymous namespace)::UndefinedArraySubscriptChecker::~UndefinedArraySubscriptChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_130UndefinedArraySubscriptCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedArraySubscriptChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_130UndefinedArraySubscriptCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedArraySubscriptChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_130UndefinedArraySubscriptCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedArraySubscriptChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_130UndefinedArraySubscriptCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedArraySubscriptChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento34RegisterUndefinedAssignmentCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUndefinedAssignmentChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_126UndefinedAssignmentCheckerD1Ev", "(anonymous namespace)::UndefinedAssignmentChecker::~UndefinedAssignmentChecker()"},
    {"_ZN12_GLOBAL__N_126UndefinedAssignmentCheckerD0Ev", "(anonymous namespace)::UndefinedAssignmentChecker::~UndefinedAssignmentChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126UndefinedAssignmentCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedAssignmentChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126UndefinedAssignmentCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedAssignmentChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_126UndefinedAssignmentChecker12PreVisitBindERN5clang4ento14CheckerContextEPKNS1_4StmtENS2_4SValES8_", "(anonymous namespace)::UndefinedAssignmentChecker::PreVisitBind(clang::ento::CheckerContext&, clang::Stmt const*, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126UndefinedAssignmentCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedAssignmentChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126UndefinedAssignmentCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedAssignmentChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento22RegisterUnixAPICheckerERNS0_10ExprEngineE", "clang::ento::RegisterUnixAPIChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_114UnixAPICheckerD1Ev", "(anonymous namespace)::UnixAPIChecker::~UnixAPIChecker()"},
    {"_ZN12_GLOBAL__N_114UnixAPICheckerD0Ev", "(anonymous namespace)::UnixAPIChecker::~UnixAPIChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114UnixAPICheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UnixAPIChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114UnixAPICheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::UnixAPIChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_Z9CheckOpenRN5clang4ento14CheckerContextERN12_GLOBAL__N_114UnixAPICheckerEPKNS_8CallExprERPNS0_7BugTypeE", "CheckOpen(clang::ento::CheckerContext&, (anonymous namespace)::UnixAPIChecker&, clang::CallExpr const*, clang::ento::BugType*&)"},
    {"_Z16CheckPthreadOnceRN5clang4ento14CheckerContextERN12_GLOBAL__N_114UnixAPICheckerEPKNS_8CallExprERPNS0_7BugTypeE", "CheckPthreadOnce(clang::ento::CheckerContext&, (anonymous namespace)::UnixAPIChecker&, clang::CallExpr const*, clang::ento::BugType*&)"},
    {"_Z15CheckMallocZeroRN5clang4ento14CheckerContextERN12_GLOBAL__N_114UnixAPICheckerEPKNS_8CallExprERPNS0_7BugTypeE", "CheckMallocZero(clang::ento::CheckerContext&, (anonymous namespace)::UnixAPIChecker&, clang::CallExpr const*, clang::ento::BugType*&)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114UnixAPICheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UnixAPIChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114UnixAPICheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::UnixAPIChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento30RegisterUnreachableCodeCheckerERNS0_10ExprEngineE", "clang::ento::RegisterUnreachableCodeChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeCheckerD1Ev", "(anonymous namespace)::UnreachableCodeChecker::~UnreachableCodeChecker()"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeCheckerD0Ev", "(anonymous namespace)::UnreachableCodeChecker::~UnreachableCodeChecker()"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeChecker16VisitEndAnalysisERN5clang4ento13ExplodedGraphERNS2_11BugReporterERNS2_10ExprEngineE", "(anonymous namespace)::UnreachableCodeChecker::VisitEndAnalysis(clang::ento::ExplodedGraph&, clang::ento::BugReporter&, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_122UnreachableCodeChecker26FindUnreachableEntryPointsEPKN5clang8CFGBlockE", "(anonymous namespace)::UnreachableCodeChecker::FindUnreachableEntryPoints(clang::CFGBlock const*)"},
    {"_ZN4llvm8SmallSetIjLj256EE6insertERKj", "llvm::SmallSet<unsigned int, 256u>::insert(unsigned int const&)"},
    {"_ZN5clang4ento12containsStmtINS_17SizeOfAlignOfExprEEEbPKNS_4StmtE", "bool clang::ento::containsStmt<clang::SizeOfAlignOfExpr>(clang::Stmt const*)"},
    {"_ZN5clang4ento22RegisterVLASizeCheckerERNS0_10ExprEngineE", "clang::ento::RegisterVLASizeChecker(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_114VLASizeCheckerD1Ev", "(anonymous namespace)::VLASizeChecker::~VLASizeChecker()"},
    {"_ZN12_GLOBAL__N_114VLASizeCheckerD0Ev", "(anonymous namespace)::VLASizeChecker::~VLASizeChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114VLASizeCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::VLASizeChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114VLASizeCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::VLASizeChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114VLASizeCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::VLASizeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114VLASizeCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::VLASizeChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento10ExprEngine12VisitAggExprEPKNS_4ExprEPKNS0_9MemRegionEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitAggExpr(clang::Expr const*, clang::ento::MemRegion const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_114AggExprVisitorEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::AggExprVisitor, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang4ento15AnalysisManager29getAnalysisContextInAnotherTUEPKNS_4DeclE", "clang::ento::AnalysisManager::getAnalysisContextInAnotherTU(clang::Decl const*)"},
    {"_ZN5clang4ento28CreateBasicConstraintManagerERNS0_14GRStateManagerERNS0_9SubEngineE", "clang::ento::CreateBasicConstraintManager(clang::ento::GRStateManager&, clang::ento::SubEngine&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManagerD1Ev", "(anonymous namespace)::BasicConstraintManager::~BasicConstraintManager()"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManagerD0Ev", "(anonymous namespace)::BasicConstraintManager::~BasicConstraintManager()"},
    {"_ZNK12_GLOBAL__N_122BasicConstraintManager9getSymValEPKN5clang4ento7GRStateEPKNS2_10SymbolDataE", "(anonymous namespace)::BasicConstraintManager::getSymVal(clang::ento::GRState const*, clang::ento::SymbolData const*) const"},
    {"_ZNK12_GLOBAL__N_122BasicConstraintManager7isEqualEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntE", "(anonymous namespace)::BasicConstraintManager::isEqual(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&) const"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager18removeDeadBindingsEPKN5clang4ento7GRStateERNS2_12SymbolReaperE", "(anonymous namespace)::BasicConstraintManager::removeDeadBindings(clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager5printEPKN5clang4ento7GRStateERN4llvm11raw_ostreamEPKcSA_", "(anonymous namespace)::BasicConstraintManager::print(clang::ento::GRState const*, llvm::raw_ostream&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymNEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymNE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymEQEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymEQ(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymLTEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymLT(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymGTEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymGT(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymLEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymLE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager11assumeSymGEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::BasicConstraintManager::assumeSymGE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122BasicConstraintManager5AddEQEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntE", "(anonymous namespace)::BasicConstraintManager::AddEQ(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&)"},
    {"_ZNK5clang4ento7GRState3getIN12_GLOBAL__N_110ConstNotEqEEENS0_12GRStateTraitIT_E11lookup_typeENS7_8key_typeE", "clang::ento::GRStateTrait<(anonymous namespace)::ConstNotEq>::lookup_type clang::ento::GRState::get<(anonymous namespace)::ConstNotEq>(clang::ento::GRStateTrait<(anonymous namespace)::ConstNotEq>::key_type) const"},
    {"_ZNK5clang4ento7GRState3getIN12_GLOBAL__N_17ConstEqEEENS0_12GRStateTraitIT_E11lookup_typeENS7_8key_typeE", "clang::ento::GRStateTrait<(anonymous namespace)::ConstEq>::lookup_type clang::ento::GRState::get<(anonymous namespace)::ConstEq>(clang::ento::GRStateTrait<(anonymous namespace)::ConstEq>::key_type) const"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEPKNS2_6APSIntENS2_16ImutKeyValueInfoIS6_S9_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEPKNS2_6APSIntENS2_16ImutKeyValueInfoIS6_S9_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEPKNS2_6APSIntENS2_16ImutKeyValueInfoIS6_S9_EEEEE3SetESC_S6_S9_RNSC_7FactoryE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> > >::Set(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >, clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::Factory&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS0_6APSIntEEEEESaISD_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSD_SF_EERKSD_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE16InsertIntoBucketERKjRKSD_PSt4pairIjSD_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE3addEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, std::pair<clang::ento::SymbolData const*, llvm::APSInt const*> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE16getCanonicalTreeEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeISA_EERNS_26ImutAVLTreeInOrderIteratorISA_EESH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE12add_internalERKSt4pairIS6_S9_EPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::add_internal(std::pair<clang::ento::SymbolData const*, llvm::APSInt const*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE13markImmutableEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE10createNodeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, std::pair<clang::ento::SymbolData const*, llvm::APSInt const*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE11balanceTreeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, std::pair<clang::ento::SymbolData const*, llvm::APSInt const*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS2_12ImmutableSetIPNS2_6APSIntENS2_17ImutContainerInfoIS9_EEEENS2_16ImutKeyValueInfoIS6_SC_EEEEE6LookupESF_S6_", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > > >::Lookup(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >, clang::ento::SymbolData const*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS0_12ImmutableSetIPNS0_6APSIntENS0_17ImutContainerInfoISA_EEEEEEEESaISG_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSG_SI_EERKSG_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoISA_EEEEEEEENS_12DenseMapInfoIjEENSH_ISG_EEE16InsertIntoBucketERKjRKSG_PSt4pairIjSG_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoISA_EEEEEEEENS_12DenseMapInfoIjEENSH_ISG_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*> >::grow(unsigned int)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS2_12ImmutableSetIPNS2_6APSIntENS2_17ImutContainerInfoIS9_EEEENS2_16ImutKeyValueInfoIS6_SC_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS2_12ImmutableSetIPNS2_6APSIntENS2_17ImutContainerInfoIS9_EEEENS2_16ImutKeyValueInfoIS6_SC_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::~ImutAVLFactory()"},
    {"_ZN4llvm12ImmutableMapIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS8_EEEENS_16ImutKeyValueInfoIS5_SB_EEE7Factory3addESE_RKS5_RKSB_", "llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::Factory::add(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >, clang::ento::SymbolData const* const&, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE3addEPNS_11ImutAVLTreeISD_EERKSt4pairIS6_SC_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE16getCanonicalTreeEPNS_11ImutAVLTreeISD_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeISD_EERNS_26ImutAVLTreeInOrderIteratorISD_EESK_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::operator++()"},
    {"_ZNK4llvm11ImutAVLTreeINS_17ImutContainerInfoIPNS_6APSIntEEEE7isEqualERKS5_", "llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >::isEqual(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> > const&) const"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_17ImutContainerInfoIPNS_6APSIntEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutContainerInfo<llvm::APSInt*> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE12add_internalERKSt4pairIS6_SC_EPNS_11ImutAVLTreeISD_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::add_internal(std::pair<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE13markImmutableEPNS_11ImutAVLTreeISD_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE10createNodeEPNS_11ImutAVLTreeISD_EERKSt4pairIS6_SC_ESH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE11balanceTreeEPNS_11ImutAVLTreeISD_EERKSt4pairIS6_SC_ESH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPNS_6APSIntEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPNS_6APSIntEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_17ImutContainerInfoIPNS0_6APSIntEEEEESaIS7_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >**, std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*> > >, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPNS_6APSIntEEEEENS_12DenseMapInfoIjEENS8_IS7_EEE16InsertIntoBucketERKjRKS7_PSt4pairIjS7_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPNS_6APSIntEEEEENS_12DenseMapInfoIjEENS8_IS7_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE3addEPNS_11ImutAVLTreeIS4_EEPKS2_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::add(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::APSInt const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS4_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS4_EERNS_26ImutAVLTreeInOrderIteratorIS4_EESB_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<llvm::APSInt*> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<llvm::APSInt*> >&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE12add_internalEPKS2_PNS_11ImutAVLTreeIS4_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::add_internal(llvm::APSInt const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE13markImmutableEPNS_11ImutAVLTreeIS4_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::markImmutable(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE10createNodeEPNS_11ImutAVLTreeIS4_EEPKS2_S8_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::createNode(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::APSInt const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEE11balanceTreeEPNS_11ImutAVLTreeIS4_EEPKS2_S8_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::balanceTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*, llvm::APSInt const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<llvm::APSInt*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE6removeEPNS_11ImutAVLTreeISD_EERKS6_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::remove(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, clang::ento::SymbolData const* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE15remove_internalERKS6_PNS_11ImutAVLTreeISD_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::remove_internal(clang::ento::SymbolData const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableSetIPNS_6APSIntENS_17ImutContainerInfoIS9_EEEEEEE16removeMinBindingEPNS_11ImutAVLTreeISD_EERSH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableSet<llvm::APSInt*, llvm::ImutContainerInfo<llvm::APSInt*> > > >*&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE6removeEPNS_11ImutAVLTreeISA_EERKS6_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::remove(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, clang::ento::SymbolData const* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE15remove_internalERKS6_PNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::remove_internal(clang::ento::SymbolData const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEPKNS_6APSIntEEEE16removeMinBindingEPNS_11ImutAVLTreeISA_EERSE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >*&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEPKNS2_6APSIntENS2_16ImutKeyValueInfoIS6_S9_EEEEE6LookupESC_S6_", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> > >::Lookup(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::APSInt const*, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::APSInt const*> >, clang::ento::SymbolData const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPNS_6APSIntEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<llvm::APSInt*> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento23CreateBasicStoreManagerERNS0_14GRStateManagerE", "clang::ento::CreateBasicStoreManager(clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento12StoreManager15BindingsHandlerD0Ev", "clang::ento::StoreManager::BindingsHandler::~BindingsHandler()"},
    {"_ZN5clang4ento12StoreManager15BindingsHandlerD1Ev", "clang::ento::StoreManager::BindingsHandler::~BindingsHandler()"},
    {"_ZN5clang4ento12StoreManager15BindingsHandlerD2Ev", "clang::ento::StoreManager::BindingsHandler::~BindingsHandler()"},
    {"_ZN12_GLOBAL__N_117BasicStoreManagerD1Ev", "(anonymous namespace)::BasicStoreManager::~BasicStoreManager()"},
    {"_ZN12_GLOBAL__N_117BasicStoreManagerD0Ev", "(anonymous namespace)::BasicStoreManager::~BasicStoreManager()"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager8RetrieveEPKvN5clang4ento3LocENS3_8QualTypeE", "(anonymous namespace)::BasicStoreManager::Retrieve(void const*, clang::ento::Loc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager4BindEPKvN5clang4ento3LocENS4_4SValE", "(anonymous namespace)::BasicStoreManager::Bind(void const*, clang::ento::Loc, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager6RemoveEPKvN5clang4ento3LocE", "(anonymous namespace)::BasicStoreManager::Remove(void const*, clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager19BindCompoundLiteralEPKvPKN5clang19CompoundLiteralExprEPKNS3_15LocationContextENS3_4ento4SValE", "(anonymous namespace)::BasicStoreManager::BindCompoundLiteral(void const*, clang::CompoundLiteralExpr const*, clang::LocationContext const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager15getInitialStoreEPKN5clang15LocationContextE", "(anonymous namespace)::BasicStoreManager::getInitialStore(clang::LocationContext const*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager15getSubRegionMapEPKv", "(anonymous namespace)::BasicStoreManager::getSubRegionMap(void const*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager14ArrayToPointerEN5clang4ento3LocE", "(anonymous namespace)::BasicStoreManager::ArrayToPointer(clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager18removeDeadBindingsEPKvPKN5clang17StackFrameContextERNS3_4ento12SymbolReaperERN4llvm15SmallVectorImplIPKNS7_9MemRegionEEE", "(anonymous namespace)::BasicStoreManager::removeDeadBindings(void const*, clang::StackFrameContext const*, clang::ento::SymbolReaper&, llvm::SmallVectorImpl<clang::ento::MemRegion const*>&)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager8BindDeclEPKvPKN5clang4ento9VarRegionENS4_4SValE", "(anonymous namespace)::BasicStoreManager::BindDecl(void const*, clang::ento::VarRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager18BindDeclWithNoInitEPKvPKN5clang4ento9VarRegionE", "(anonymous namespace)::BasicStoreManager::BindDeclWithNoInit(void const*, clang::ento::VarRegion const*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager17InvalidateRegionsEPKvPKPKN5clang4ento9MemRegionES9_PKNS3_4ExprEjPN4llvm8DenseSetIPKNS4_10SymbolDataENSD_12DenseMapInfoISH_EEEEbPNSD_11SmallVectorIS7_Lj8EEE", "(anonymous namespace)::BasicStoreManager::InvalidateRegions(void const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, clang::Expr const*, unsigned int, llvm::DenseSet<clang::ento::SymbolData const*, llvm::DenseMapInfo<clang::ento::SymbolData const*> >*, bool, llvm::SmallVector<clang::ento::MemRegion const*, 8u>*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager5printEPKvRN4llvm11raw_ostreamEPKcS7_", "(anonymous namespace)::BasicStoreManager::print(void const*, llvm::raw_ostream&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager12iterBindingsEPKvRN5clang4ento12StoreManager15BindingsHandlerE", "(anonymous namespace)::BasicStoreManager::iterBindings(void const*, clang::ento::StoreManager::BindingsHandler&)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager16InvalidateRegionEPKvPKN5clang4ento9MemRegionEPKNS3_4ExprEjPN4llvm8DenseSetIPKNS4_10SymbolDataENSB_12DenseMapInfoISF_EEEE", "(anonymous namespace)::BasicStoreManager::InvalidateRegion(void const*, clang::ento::MemRegion const*, clang::Expr const*, unsigned int, llvm::DenseSet<clang::ento::SymbolData const*, llvm::DenseMapInfo<clang::ento::SymbolData const*> >*)"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager16BindDeclInternalEPKvPKN5clang4ento9VarRegionEPNS4_4SValE", "(anonymous namespace)::BasicStoreManager::BindDeclInternal(void const*, clang::ento::VarRegion const*, clang::ento::SVal*)"},
    {"_ZN12_GLOBAL__N_122BasicStoreSubRegionMapD1Ev", "(anonymous namespace)::BasicStoreSubRegionMap::~BasicStoreSubRegionMap()"},
    {"_ZN12_GLOBAL__N_122BasicStoreSubRegionMapD0Ev", "(anonymous namespace)::BasicStoreSubRegionMap::~BasicStoreSubRegionMap()"},
    {"_ZNK12_GLOBAL__N_122BasicStoreSubRegionMap14iterSubRegionsEPKN5clang4ento9MemRegionERNS2_12SubRegionMap7VisitorE", "(anonymous namespace)::BasicStoreSubRegionMap::iterSubRegions(clang::ento::MemRegion const*, clang::ento::SubRegionMap::Visitor&) const"},
    {"_ZN12_GLOBAL__N_117BasicStoreManager12scanForIvarsEPN5clang4StmtEPKNS1_4DeclEPKNS1_4ento9MemRegionEPKv", "(anonymous namespace)::BasicStoreManager::scanForIvars(clang::Stmt*, clang::Decl const*, clang::ento::MemRegion const*, void const*)"},
    {"_ZN5clang4ento12StoreManager11BindDefaultEPKvPKNS0_9MemRegionENS0_4SValE", "clang::ento::StoreManager::BindDefault(void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN5clang4ento12StoreManager12getLValueVarEPKNS_7VarDeclEPKNS_15LocationContextE", "clang::ento::StoreManager::getLValueVar(clang::VarDecl const*, clang::LocationContext const*)"},
    {"_ZN5clang4ento12StoreManager15getLValueStringEPKNS_13StringLiteralE", "clang::ento::StoreManager::getLValueString(clang::StringLiteral const*)"},
    {"_ZN5clang4ento12StoreManager13getLValueIvarEPKNS_12ObjCIvarDeclENS0_4SValE", "clang::ento::StoreManager::getLValueIvar(clang::ObjCIvarDecl const*, clang::ento::SVal)"},
    {"_ZN5clang4ento12StoreManager14getLValueFieldEPKNS_9FieldDeclENS0_4SValE", "clang::ento::StoreManager::getLValueField(clang::FieldDecl const*, clang::ento::SVal)"},
    {"_ZN5clang4ento12StoreManager17getSizeInElementsEPKNS0_7GRStateEPKNS0_9MemRegionENS_8QualTypeE", "clang::ento::StoreManager::getSizeInElements(clang::ento::GRState const*, clang::ento::MemRegion const*, clang::QualType)"},
    {"_ZN5clang4ento12StoreManager17evalDerivedToBaseENS0_4SValENS_8QualTypeE", "clang::ento::StoreManager::evalDerivedToBase(clang::ento::SVal, clang::QualType)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS4_4SValEEEEESaISB_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSB_SD_EERKSB_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >* const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE13computeDigestEPS9_SA_RKSt4pairIS6_S7_E", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::computeDigest(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::pair<clang::ento::MemRegion const*, clang::ento::SVal> const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS4_4SValEEEEENS_12DenseMapInfoIjEENSC_ISB_EEE16InsertIntoBucketERKjRKSB_PSt4pairIjSB_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS4_4SValEEEEENS_12DenseMapInfoIjEENSC_ISB_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*> >::grow(unsigned int)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::operator++()"},
    {"_ZN5clang4ento12SubRegionMapD1Ev", "clang::ento::SubRegionMap::~SubRegionMap()"},
    {"_ZN5clang4ento12SubRegionMapD0Ev", "clang::ento::SubRegionMap::~SubRegionMap()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE6removeEPNS_11ImutAVLTreeIS8_EERKS6_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::remove(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, clang::ento::MemRegion const* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS8_EERNS_26ImutAVLTreeInOrderIteratorIS8_EESF_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE15remove_internalERKS6_PNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::remove_internal(clang::ento::MemRegion const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE13markImmutableEPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE11balanceTreeEPNS_11ImutAVLTreeIS8_EERKSt4pairIS6_S7_ESC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::pair<clang::ento::MemRegion const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE10createNodeEPNS_11ImutAVLTreeIS8_EERKSt4pairIS6_S7_ESC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::pair<clang::ento::MemRegion const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE16removeMinBindingEPNS_11ImutAVLTreeIS8_EERSC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE3addEPNS_11ImutAVLTreeIS8_EERKSt4pairIS6_S7_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*, std::pair<clang::ento::MemRegion const*, clang::ento::SVal> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEE12add_internalERKSt4pairIS6_S7_EPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::add_internal(std::pair<clang::ento::MemRegion const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >*)"},
    {"_ZN5clang4ento12StoreManagerD1Ev", "clang::ento::StoreManager::~StoreManager()"},
    {"_ZN5clang4ento12StoreManagerD0Ev", "clang::ento::StoreManager::~StoreManager()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS3_4SValEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, clang::ento::SVal> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento15CompoundValData7ProfileERN4llvm16FoldingSetNodeIDENS_8QualTypeENS2_13ImmutableListINS0_4SValEEE", "clang::ento::CompoundValData::Profile(llvm::FoldingSetNodeID&, clang::QualType, llvm::ImmutableList<clang::ento::SVal>)"},
    {"_ZN5clang4ento19LazyCompoundValData7ProfileERN4llvm16FoldingSetNodeIDEPKvPKNS0_11TypedRegionE", "clang::ento::LazyCompoundValData::Profile(llvm::FoldingSetNodeID&, void const*, clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento17BasicValueFactoryD1Ev", "clang::ento::BasicValueFactory::~BasicValueFactory()"},
    {"_ZN5clang4ento17BasicValueFactoryD2Ev", "clang::ento::BasicValueFactory::~BasicValueFactory()"},
    {"_ZN5clang4ento17BasicValueFactory8getValueERKN4llvm6APSIntE", "clang::ento::BasicValueFactory::getValue(llvm::APSInt const&)"},
    {"_ZN5clang4ento17BasicValueFactory8getValueERKN4llvm5APIntEb", "clang::ento::BasicValueFactory::getValue(llvm::APInt const&, bool)"},
    {"_ZN5clang4ento17BasicValueFactory8getValueEyjb", "clang::ento::BasicValueFactory::getValue(unsigned long long, unsigned int, bool)"},
    {"_ZN5clang4ento17BasicValueFactory8getValueEyNS_8QualTypeE", "clang::ento::BasicValueFactory::getValue(unsigned long long, clang::QualType)"},
    {"_ZN5clang4ento17BasicValueFactory18getCompoundValDataENS_8QualTypeEN4llvm13ImmutableListINS0_4SValEEE", "clang::ento::BasicValueFactory::getCompoundValData(clang::QualType, llvm::ImmutableList<clang::ento::SVal>)"},
    {"_ZN5clang4ento17BasicValueFactory22getLazyCompoundValDataEPKvPKNS0_11TypedRegionE", "clang::ento::BasicValueFactory::getLazyCompoundValData(void const*, clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento17BasicValueFactory10evalAPSIntENS_18BinaryOperatorKindERKN4llvm6APSIntES6_", "clang::ento::BasicValueFactory::evalAPSInt(clang::BinaryOperatorKind, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN5clang4ento17BasicValueFactory25getPersistentSValWithDataERKNS0_4SValEm", "clang::ento::BasicValueFactory::getPersistentSValWithData(clang::ento::SVal const&, unsigned long)"},
    {"_ZN5clang4ento17BasicValueFactory21getPersistentSValPairERKNS0_4SValES4_", "clang::ento::BasicValueFactory::getPersistentSValPair(clang::ento::SVal const&, clang::ento::SVal const&)"},
    {"_ZN5clang4ento17BasicValueFactory17getPersistentSValENS0_4SValE", "clang::ento::BasicValueFactory::getPersistentSVal(clang::ento::SVal)"},
    {"_ZN4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEED1Ev", "llvm::FoldingSet<clang::ento::LazyCompoundValData>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento15CompoundValDataEED1Ev", "llvm::FoldingSet<clang::ento::CompoundValData>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEED1Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >::~FoldingSet()"},
    {"_ZN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES4_EED1Ev", "llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> >::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES4_EED0Ev", "llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> >::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEED1Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEED0Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERSC_", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEED1Ev", "llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> >::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEED0Ev", "llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> >::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEED1Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEED0Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERSC_", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm21FoldingSetNodeWrapperINS_6APSIntEED1Ev", "llvm::FoldingSetNodeWrapper<llvm::APSInt>::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm21FoldingSetNodeWrapperINS_6APSIntEED0Ev", "llvm::FoldingSetNodeWrapper<llvm::APSInt>::~FoldingSetNodeWrapper()"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEED1Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEED0Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERSA_", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEED0Ev", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang4ento15CompoundValDataEED0Ev", "llvm::FoldingSet<clang::ento::CompoundValData>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento15CompoundValDataEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::CompoundValData>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento15CompoundValDataEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::CompoundValData>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento15CompoundValDataEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::CompoundValData>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEED0Ev", "llvm::FoldingSet<clang::ento::LazyCompoundValData>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::LazyCompoundValData>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::LazyCompoundValData>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::LazyCompoundValData>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento12BlockCounter13getNumVisitedEPKNS_17StackFrameContextEj", "clang::ento::BlockCounter::getNumVisited(clang::StackFrameContext const*, unsigned int) const"},
    {"_ZN5clang4ento12BlockCounter7FactoryC1ERN4llvm16BumpPtrAllocatorE", "clang::ento::BlockCounter::Factory::Factory(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento12BlockCounter7FactoryC2ERN4llvm16BumpPtrAllocatorE", "clang::ento::BlockCounter::Factory::Factory(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento12BlockCounter7FactoryD1Ev", "clang::ento::BlockCounter::Factory::~Factory()"},
    {"_ZN5clang4ento12BlockCounter7FactoryD2Ev", "clang::ento::BlockCounter::Factory::~Factory()"},
    {"_ZN5clang4ento12BlockCounter7Factory14IncrementCountES1_PKNS_17StackFrameContextEj", "clang::ento::BlockCounter::Factory::IncrementCount(clang::ento::BlockCounter, clang::StackFrameContext const*, unsigned int)"},
    {"_ZN5clang4ento12BlockCounter7Factory15GetEmptyCounterEv", "clang::ento::BlockCounter::Factory::GetEmptyCounter()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::computeDigest()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEEENS_12DenseMapInfoIjEENS8_IS7_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEEESaIS7_EE9push_backERKS7_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >* const&)"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE12add_internalERKSt4pairIS3_jEPNS_11ImutAVLTreeIS4_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::add_internal(std::pair<(anonymous namespace)::CountKey, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE13markImmutableEPNS_11ImutAVLTreeIS4_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE10createNodeEPNS_11ImutAVLTreeIS4_EERKSt4pairIS3_jES8_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*, std::pair<(anonymous namespace)::CountKey, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_18CountKeyEjEEE11balanceTreeEPNS_11ImutAVLTreeIS4_EERKSt4pairIS3_jES8_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*, std::pair<(anonymous namespace)::CountKey, unsigned int> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::CountKey, unsigned int> >*)"},
    {"_ZN5clang4ento18BugReporterVisitorD0Ev", "clang::ento::BugReporterVisitor::~BugReporterVisitor()"},
    {"_ZN5clang4ento18BugReporterVisitorD1Ev", "clang::ento::BugReporterVisitor::~BugReporterVisitor()"},
    {"_ZN5clang4ento18BugReporterVisitorD2Ev", "clang::ento::BugReporterVisitor::~BugReporterVisitor()"},
    {"_ZN5clang4ento18BugReporterContextD0Ev", "clang::ento::BugReporterContext::~BugReporterContext()"},
    {"_ZN5clang4ento18BugReporterContextD1Ev", "clang::ento::BugReporterContext::~BugReporterContext()"},
    {"_ZN5clang4ento18BugReporterContextD2Ev", "clang::ento::BugReporterContext::~BugReporterContext()"},
    {"_ZN5clang4ento18BugReporterContext10addVisitorEPNS0_18BugReporterVisitorE", "clang::ento::BugReporterContext::addVisitor(clang::ento::BugReporterVisitor*)"},
    {"_ZN5clang4ento7BugTypeD0Ev", "clang::ento::BugType::~BugType()"},
    {"_ZN5clang4ento7BugTypeD1Ev", "clang::ento::BugType::~BugType()"},
    {"_ZN5clang4ento7BugTypeD2Ev", "clang::ento::BugType::~BugType()"},
    {"_ZN5clang4ento19BugReportEquivClassD1Ev", "clang::ento::BugReportEquivClass::~BugReportEquivClass()"},
    {"_ZN5clang4ento7BugType12FlushReportsERNS0_11BugReporterE", "clang::ento::BugType::FlushReports(clang::ento::BugReporter&)"},
    {"_ZN5clang4ento9BugReportD0Ev", "clang::ento::BugReport::~BugReport()"},
    {"_ZN5clang4ento9BugReportD1Ev", "clang::ento::BugReport::~BugReport()"},
    {"_ZN5clang4ento9BugReportD2Ev", "clang::ento::BugReport::~BugReport()"},
    {"_ZN5clang4ento15RangedBugReportD0Ev", "clang::ento::RangedBugReport::~RangedBugReport()"},
    {"_ZN5clang4ento15RangedBugReportD1Ev", "clang::ento::RangedBugReport::~RangedBugReport()"},
    {"_ZN5clang4ento15RangedBugReportD2Ev", "clang::ento::RangedBugReport::~RangedBugReport()"},
    {"_ZNK5clang4ento9BugReport7getStmtEv", "clang::ento::BugReport::getStmt() const"},
    {"_ZN5clang4ento9BugReport10getEndPathERNS0_18BugReporterContextEPKNS0_12ExplodedNodeE", "clang::ento::BugReport::getEndPath(clang::ento::BugReporterContext&, clang::ento::ExplodedNode const*)"},
    {"_ZNK5clang4ento9BugReport9getRangesEv", "clang::ento::BugReport::getRanges() const"},
    {"_ZNK5clang4ento9BugReport11getLocationEv", "clang::ento::BugReport::getLocation() const"},
    {"_ZN5clang4ento9BugReport9VisitNodeEPKNS0_12ExplodedNodeES4_RNS0_18BugReporterContextE", "clang::ento::BugReport::VisitNode(clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, clang::ento::BugReporterContext&)"},
    {"_ZN5clang4ento19BugReportEquivClassD2Ev", "clang::ento::BugReportEquivClass::~BugReportEquivClass()"},
    {"_ZN5clang4ento13GRBugReporterD0Ev", "clang::ento::GRBugReporter::~GRBugReporter()"},
    {"_ZN5clang4ento13GRBugReporterD1Ev", "clang::ento::GRBugReporter::~GRBugReporter()"},
    {"_ZN5clang4ento13GRBugReporterD2Ev", "clang::ento::GRBugReporter::~GRBugReporter()"},
    {"_ZN5clang4ento11BugReporterD2Ev", "clang::ento::BugReporter::~BugReporter()"},
    {"_ZN5clang4ento15BugReporterDataD0Ev", "clang::ento::BugReporterData::~BugReporterData()"},
    {"_ZN5clang4ento15BugReporterDataD1Ev", "clang::ento::BugReporterData::~BugReporterData()"},
    {"_ZN5clang4ento15BugReporterDataD2Ev", "clang::ento::BugReporterData::~BugReporterData()"},
    {"_ZN5clang4ento13GRBugReporter8getGraphEv", "clang::ento::GRBugReporter::getGraph()"},
    {"_ZN5clang4ento13GRBugReporter15getStateManagerEv", "clang::ento::GRBugReporter::getStateManager()"},
    {"_ZN5clang4ento11BugReporterD0Ev", "clang::ento::BugReporter::~BugReporter()"},
    {"_ZN5clang4ento11BugReporterD1Ev", "clang::ento::BugReporter::~BugReporter()"},
    {"_ZN5clang4ento11BugReporter12FlushReportsEv", "clang::ento::BugReporter::FlushReports()"},
    {"_ZN5clang4ento11BugReporter11FlushReportERNS0_19BugReportEquivClassE", "clang::ento::BugReporter::FlushReport(clang::ento::BugReportEquivClass&)"},
    {"_ZN5clang4ento13GRBugReporter22GeneratePathDiagnosticERNS0_14PathDiagnosticERN4llvm15SmallVectorImplIPNS0_9BugReportEEE", "clang::ento::GRBugReporter::GeneratePathDiagnostic(clang::ento::PathDiagnostic&, llvm::SmallVectorImpl<clang::ento::BugReport*>&)"},
    {"_ZN12_GLOBAL__N_121PathDiagnosticBuilderD1Ev", "(anonymous namespace)::PathDiagnosticBuilder::~PathDiagnosticBuilder()"},
    {"_ZN5clang4ento11BugReporter8RegisterEPNS0_7BugTypeE", "clang::ento::BugReporter::Register(clang::ento::BugType*)"},
    {"_ZN5clang4ento11BugReporter10EmitReportEPNS0_9BugReportE", "clang::ento::BugReporter::EmitReport(clang::ento::BugReport*)"},
    {"_ZN5clang4ento11BugReporter15EmitBasicReportEN4llvm9StringRefES3_NS_14SourceLocationEPNS_11SourceRangeEj", "clang::ento::BugReporter::EmitBasicReport(llvm::StringRef, llvm::StringRef, clang::SourceLocation, clang::SourceRange*, unsigned int)"},
    {"_ZN5clang4ento11BugReporter15EmitBasicReportEN4llvm9StringRefES3_S3_NS_14SourceLocationEPNS_11SourceRangeEj", "clang::ento::BugReporter::EmitBasicReport(llvm::StringRef, llvm::StringRef, llvm::StringRef, clang::SourceLocation, clang::SourceRange*, unsigned int)"},
    {"_ZN4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEED1Ev", "llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEED0Ev", "llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_112FRIEC_WLItemEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::FRIEC_WLItem>::push_back((anonymous namespace)::FRIEC_WLItem const&)"},
    {"_ZN12_GLOBAL__N_114NodeMapClosureD1Ev", "(anonymous namespace)::NodeMapClosure::~NodeMapClosure()"},
    {"_ZN12_GLOBAL__N_121PathDiagnosticBuilderD0Ev", "(anonymous namespace)::PathDiagnosticBuilder::~PathDiagnosticBuilder()"},
    {"_ZN12_GLOBAL__N_121PathDiagnosticBuilder15getNodeResolverEv", "(anonymous namespace)::PathDiagnosticBuilder::getNodeResolver()"},
    {"_ZN12_GLOBAL__N_114NodeMapClosureD0Ev", "(anonymous namespace)::NodeMapClosure::~NodeMapClosure()"},
    {"_ZN12_GLOBAL__N_114NodeMapClosure15getOriginalNodeEPKN5clang4ento12ExplodedNodeE", "(anonymous namespace)::NodeMapClosure::getOriginalNode(clang::ento::ExplodedNode const*)"},
    {"_Z11GetNextStmtPKN5clang4ento12ExplodedNodeE", "GetNextStmt(clang::ento::ExplodedNode const*)"},
    {"_ZN12_GLOBAL__N_121PathDiagnosticBuilder24getEnclosingStmtLocationEPKN5clang4StmtE", "(anonymous namespace)::PathDiagnosticBuilder::getEnclosingStmtLocation(clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_121PathDiagnosticBuilder18ExecutionContinuesERN4llvm18raw_string_ostreamEPKN5clang4ento12ExplodedNodeE", "(anonymous namespace)::PathDiagnosticBuilder::ExecutionContinues(llvm::raw_string_ostream&, clang::ento::ExplodedNode const*)"},
    {"_ZN12_GLOBAL__N_118ScanNotableSymbolsD1Ev", "(anonymous namespace)::ScanNotableSymbols::~ScanNotableSymbols()"},
    {"_ZN12_GLOBAL__N_118ScanNotableSymbolsD0Ev", "(anonymous namespace)::ScanNotableSymbols::~ScanNotableSymbols()"},
    {"_ZN12_GLOBAL__N_118ScanNotableSymbols13HandleBindingERN5clang4ento12StoreManagerEPKvPKNS2_9MemRegionENS2_4SValE", "(anonymous namespace)::ScanNotableSymbols::HandleBinding(clang::ento::StoreManager&, void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_120NotableSymbolHandlerD1Ev", "(anonymous namespace)::NotableSymbolHandler::~NotableSymbolHandler()"},
    {"_ZN12_GLOBAL__N_120NotableSymbolHandlerD0Ev", "(anonymous namespace)::NotableSymbolHandler::~NotableSymbolHandler()"},
    {"_ZN12_GLOBAL__N_120NotableSymbolHandler13HandleBindingERN5clang4ento12StoreManagerEPKvPKNS2_9MemRegionENS2_4SValE", "(anonymous namespace)::NotableSymbolHandler::HandleBinding(clang::ento::StoreManager&, void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder7addEdgeEN5clang4ento22PathDiagnosticLocationEb", "(anonymous namespace)::EdgeBuilder::addEdge(clang::ento::PathDiagnosticLocation, bool)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder10addContextEPKN5clang4StmtE", "(anonymous namespace)::EdgeBuilder::addContext(clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder18addExtendedContextEPKN5clang4StmtE", "(anonymous namespace)::EdgeBuilder::addExtendedContext(clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder10rawAddEdgeEN5clang4ento22PathDiagnosticLocationE", "(anonymous namespace)::EdgeBuilder::rawAddEdge(clang::ento::PathDiagnosticLocation)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder15cleanUpLocationEN5clang4ento22PathDiagnosticLocationEb", "(anonymous namespace)::EdgeBuilder::cleanUpLocation(clang::ento::PathDiagnosticLocation, bool)"},
    {"_ZN12_GLOBAL__N_111EdgeBuilder16containsLocationERKN5clang4ento22PathDiagnosticLocationES5_", "(anonymous namespace)::EdgeBuilder::containsLocation(clang::ento::PathDiagnosticLocation const&, clang::ento::PathDiagnosticLocation const&)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_115ContextLocationESaIS1_EE9push_backERKS1_", "std::vector<(anonymous namespace)::ContextLocation, std::allocator<(anonymous namespace)::ContextLocation> >::push_back((anonymous namespace)::ContextLocation const&)"},
    {"_ZN4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEED1Ev", "llvm::FoldingSet<clang::ento::BugReporterVisitor>::~FoldingSet()"},
    {"_ZN5clang4ento14PathDiagnostic7addMetaEN4llvm9StringRefE", "clang::ento::PathDiagnostic::addMeta(llvm::StringRef)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE3addEPNS_11ImutAVLTreeIS6_EEPKS4_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::add(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, clang::ento::BugType const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS6_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*)"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >::computeDigest()"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS6_EERNS_26ImutAVLTreeInOrderIteratorIS6_EESD_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<clang::ento::BugType*> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<clang::ento::BugType*> >&)"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_17ImutContainerInfoIPN5clang4ento7BugTypeEEEEESaIS9_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS9_SB_EERKS9_", "std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >**, std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*> > >, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >* const&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutContainerInfo<clang::ento::BugType*> >::operator++()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEEENS_12DenseMapInfoIjEENSA_IS9_EEE16InsertIntoBucketERKjRKS9_PSt4pairIjS9_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEEENS_12DenseMapInfoIjEENSA_IS9_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE12add_internalEPKS4_PNS_11ImutAVLTreeIS6_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::add_internal(clang::ento::BugType const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE13markImmutableEPNS_11ImutAVLTreeIS6_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::markImmutable(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE10createNodeEPNS_11ImutAVLTreeIS6_EEPKS4_SA_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::createNode(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, clang::ento::BugType const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEE11balanceTreeEPNS_11ImutAVLTreeIS6_EEPKS4_SA_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::balanceTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*, clang::ento::BugType const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::BugType*> >*)"},
    {"_ZN4llvm20ImmutableListFactoryIPN5clang4ento18BugReporterVisitorEE6concatERKS4_NS_13ImmutableListIS4_EE", "llvm::ImmutableListFactory<clang::ento::BugReporterVisitor*>::concat(clang::ento::BugReporterVisitor* const&, llvm::ImmutableList<clang::ento::BugReporterVisitor*>)"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEED1Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEED0Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERSB_", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang11SourceRangeELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::SourceRange, false>::grow(unsigned long)"},
    {"_ZN5clang4ento13DiagBugReportD1Ev", "clang::ento::DiagBugReport::~DiagBugReport()"},
    {"_ZN5clang4ento13DiagBugReportD0Ev", "clang::ento::DiagBugReport::~DiagBugReport()"},
    {"_ZNK5clang4ento13DiagBugReport11getLocationEv", "clang::ento::DiagBugReport::getLocation() const"},
    {"_ZNSt5dequeISsSaISsEE16_M_push_back_auxERKSs", "std::deque<std::string, std::allocator<std::string> >::_M_push_back_aux(std::string const&)"},
    {"_ZNSt5dequeISsSaISsEE17_M_reallocate_mapEmb", "std::deque<std::string, std::allocator<std::string> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEjNS_12DenseMapInfoIS5_EENS6_IjEEE16InsertIntoBucketERKS5_RKjPSt4pairIS5_jE", "llvm::DenseMap<clang::ento::ExplodedNode const*, unsigned int, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::ento::ExplodedNode const* const&, unsigned int const&, std::pair<clang::ento::ExplodedNode const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEjNS_12DenseMapInfoIS5_EENS6_IjEEE4growEj", "llvm::DenseMap<clang::ento::ExplodedNode const*, unsigned int, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang4ento9BugReport12NodeResolverD1Ev", "clang::ento::BugReport::NodeResolver::~NodeResolver()"},
    {"_ZN5clang4ento9BugReport12NodeResolverD0Ev", "clang::ento::BugReport::NodeResolver::~NodeResolver()"},
    {"_ZN5clang4ento14PathDiagnostic10push_frontEPNS0_19PathDiagnosticPieceE", "clang::ento::PathDiagnostic::push_front(clang::ento::PathDiagnosticPiece*)"},
    {"_ZNSt6vectorISt4pairIPN5clang4ento24PathDiagnosticMacroPieceENS1_14SourceLocationEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation>, std::allocator<std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation>*, std::vector<std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation>, std::allocator<std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation> > > >, std::pair<clang::ento::PathDiagnosticMacroPiece*, clang::SourceLocation> const&)"},
    {"_ZN5clang4ento24PathDiagnosticMacroPieceC2ERKNS0_22PathDiagnosticLocationE", "clang::ento::PathDiagnosticMacroPiece::PathDiagnosticMacroPiece(clang::ento::PathDiagnosticLocation const&)"},
    {"_ZNK5clang4ento23PathDiagnosticSpotPiece11getLocationEv", "clang::ento::PathDiagnosticSpotPiece::getLocation() const"},
    {"_ZN5clang4ento24PathDiagnosticMacroPiece16flattenLocationsEv", "clang::ento::PathDiagnosticMacroPiece::flattenLocations()"},
    {"_ZN5clang4ento23PathDiagnosticSpotPiece16flattenLocationsEv", "clang::ento::PathDiagnosticSpotPiece::flattenLocations()"},
    {"_ZN5clang4ento23PathDiagnosticSpotPieceD1Ev", "clang::ento::PathDiagnosticSpotPiece::~PathDiagnosticSpotPiece()"},
    {"_ZN5clang4ento23PathDiagnosticSpotPieceD0Ev", "clang::ento::PathDiagnosticSpotPiece::~PathDiagnosticSpotPiece()"},
    {"_ZNSt6vectorIPN5clang4ento19PathDiagnosticPieceESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::ento::PathDiagnosticPiece*, std::allocator<clang::ento::PathDiagnosticPiece*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnosticPiece**, std::vector<clang::ento::PathDiagnosticPiece*, std::allocator<clang::ento::PathDiagnosticPiece*> > >, clang::ento::PathDiagnosticPiece* const&)"},
    {"_ZNK5clang4ento30PathDiagnosticControlFlowPiece11getLocationEv", "clang::ento::PathDiagnosticControlFlowPiece::getLocation() const"},
    {"_ZN5clang4ento30PathDiagnosticControlFlowPiece16flattenLocationsEv", "clang::ento::PathDiagnosticControlFlowPiece::flattenLocations()"},
    {"_ZNSt6vectorIN5clang4ento26PathDiagnosticLocationPairESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::ento::PathDiagnosticLocationPair, std::allocator<clang::ento::PathDiagnosticLocationPair> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnosticLocationPair*, std::vector<clang::ento::PathDiagnosticLocationPair, std::allocator<clang::ento::PathDiagnosticLocationPair> > >, clang::ento::PathDiagnosticLocationPair const&)"},
    {"_ZNSt5dequeIPN5clang4ento19PathDiagnosticPieceESaIS3_EE17_M_reallocate_mapEmb", "std::deque<clang::ento::PathDiagnosticPiece*, std::allocator<clang::ento::PathDiagnosticPiece*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt5dequeIPN5clang4ento19PathDiagnosticPieceESaIS3_EE9push_backERKS3_", "std::deque<clang::ento::PathDiagnosticPiece*, std::allocator<clang::ento::PathDiagnosticPiece*> >::push_back(clang::ento::PathDiagnosticPiece* const&)"},
    {"_ZN4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEED0Ev", "llvm::FoldingSet<clang::ento::BugReporterVisitor>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::BugReporterVisitor>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::BugReporterVisitor>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::BugReporterVisitor>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeES5_NS_12DenseMapInfoIS5_EES7_E16InsertIntoBucketERKS5_SA_PSt4pairIS5_S5_E", "llvm::DenseMap<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<clang::ento::ExplodedNode const*> >::InsertIntoBucket(clang::ento::ExplodedNode const* const&, clang::ento::ExplodedNode const* const&, std::pair<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeES5_NS_12DenseMapInfoIS5_EES7_E4growEj", "llvm::DenseMap<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<clang::ento::ExplodedNode const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKvjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<void const*, unsigned int, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(void const* const&, unsigned int const&, std::pair<void const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKvjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<void const*, unsigned int, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt5dequeIPKN5clang4ento12ExplodedNodeESaIS4_EE9push_backERKS4_", "std::deque<clang::ento::ExplodedNode const*, std::allocator<clang::ento::ExplodedNode const*> >::push_back(clang::ento::ExplodedNode const* const&)"},
    {"_ZNSt5dequeIPKN5clang4ento12ExplodedNodeESaIS4_EE17_M_reallocate_mapEmb", "std::deque<clang::ento::ExplodedNode const*, std::allocator<clang::ento::ExplodedNode const*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIPKN5clang4ento12ExplodedNodeESaIS4_EE17_M_initialize_mapEm", "std::_Deque_base<clang::ento::ExplodedNode const*, std::allocator<clang::ento::ExplodedNode const*> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeIPKN5clang4ento12ExplodedNodeESaIS4_EEC2ERKS6_", "std::deque<clang::ento::ExplodedNode const*, std::allocator<clang::ento::ExplodedNode const*> >::deque(std::deque<clang::ento::ExplodedNode const*, std::allocator<clang::ento::ExplodedNode const*> > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPN5clang4ento7BugTypeEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::BugType*> >::~ImutAVLFactory()"},
    {"_ZNSt6vectorIN5clang11SourceRangeESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::SourceRange, std::allocator<clang::SourceRange> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::SourceRange*, std::vector<clang::SourceRange, std::allocator<clang::SourceRange> > >, clang::SourceRange const&)"},
    {"_ZN5clang4ento11bugreporter12GetDerefExprEPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::GetDerefExpr(clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter12GetDenomExprEPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::GetDenomExpr(clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter13GetCalleeExprEPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::GetCalleeExpr(clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter13GetRetValExprEPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::GetRetValExpr(clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter29registerTrackNullOrUndefValueERNS0_18BugReporterContextEPKvPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::registerTrackNullOrUndefValue(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter21registerFindLastStoreERNS0_18BugReporterContextEPKvPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::registerFindLastStore(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*)"},
    {"_ZN5clang4ento11bugreporter26registerNilReceiverVisitorERNS0_18BugReporterContextE", "clang::ento::bugreporter::registerNilReceiverVisitor(clang::ento::BugReporterContext&)"},
    {"_ZN5clang4ento11bugreporter25registerVarDeclsLastStoreERNS0_18BugReporterContextEPKvPKNS0_12ExplodedNodeE", "clang::ento::bugreporter::registerVarDeclsLastStore(clang::ento::BugReporterContext&, void const*, clang::ento::ExplodedNode const*)"},
    {"_ZN12_GLOBAL__N_118NilReceiverVisitorD1Ev", "(anonymous namespace)::NilReceiverVisitor::~NilReceiverVisitor()"},
    {"_ZN12_GLOBAL__N_118NilReceiverVisitorD0Ev", "(anonymous namespace)::NilReceiverVisitor::~NilReceiverVisitor()"},
    {"_ZN12_GLOBAL__N_118NilReceiverVisitor9VisitNodeEPKN5clang4ento12ExplodedNodeES5_RNS2_18BugReporterContextE", "(anonymous namespace)::NilReceiverVisitor::VisitNode(clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, clang::ento::BugReporterContext&)"},
    {"_ZNK12_GLOBAL__N_118NilReceiverVisitor7ProfileERN4llvm16FoldingSetNodeIDE", "(anonymous namespace)::NilReceiverVisitor::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN12_GLOBAL__N_122FindLastStoreBRVisitorD1Ev", "(anonymous namespace)::FindLastStoreBRVisitor::~FindLastStoreBRVisitor()"},
    {"_ZN12_GLOBAL__N_122FindLastStoreBRVisitorD0Ev", "(anonymous namespace)::FindLastStoreBRVisitor::~FindLastStoreBRVisitor()"},
    {"_ZN12_GLOBAL__N_122FindLastStoreBRVisitor9VisitNodeEPKN5clang4ento12ExplodedNodeES5_RNS2_18BugReporterContextE", "(anonymous namespace)::FindLastStoreBRVisitor::VisitNode(clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, clang::ento::BugReporterContext&)"},
    {"_ZNK12_GLOBAL__N_122FindLastStoreBRVisitor7ProfileERN4llvm16FoldingSetNodeIDE", "(anonymous namespace)::FindLastStoreBRVisitor::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN12_GLOBAL__N_124TrackConstraintBRVisitorD1Ev", "(anonymous namespace)::TrackConstraintBRVisitor::~TrackConstraintBRVisitor()"},
    {"_ZN12_GLOBAL__N_124TrackConstraintBRVisitorD0Ev", "(anonymous namespace)::TrackConstraintBRVisitor::~TrackConstraintBRVisitor()"},
    {"_ZN12_GLOBAL__N_124TrackConstraintBRVisitor9VisitNodeEPKN5clang4ento12ExplodedNodeES5_RNS2_18BugReporterContextE", "(anonymous namespace)::TrackConstraintBRVisitor::VisitNode(clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, clang::ento::BugReporterContext&)"},
    {"_ZNK12_GLOBAL__N_124TrackConstraintBRVisitor7ProfileERN4llvm16FoldingSetNodeIDE", "(anonymous namespace)::TrackConstraintBRVisitor::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNSt5dequeIPKN5clang4StmtESaIS3_EE9push_backERKS3_", "std::deque<clang::Stmt const*, std::allocator<clang::Stmt const*> >::push_back(clang::Stmt const* const&)"},
    {"_ZNSt5dequeIPKN5clang4StmtESaIS3_EE17_M_reallocate_mapEmb", "std::deque<clang::Stmt const*, std::allocator<clang::Stmt const*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIPKN5clang4StmtESaIS3_EE17_M_initialize_mapEm", "std::_Deque_base<clang::Stmt const*, std::allocator<clang::Stmt const*> >::_M_initialize_map(unsigned long)"},
    {"_ZN5clang4ento16MakeCFRefCountTFERNS_10ASTContextEbRKNS_11LangOptionsE", "clang::ento::MakeCFRefCountTF(clang::ASTContext&, bool, clang::LangOptions const&)"},
    {"_ZN12_GLOBAL__N_110CFRefCountD1Ev", "(anonymous namespace)::CFRefCount::~CFRefCount()"},
    {"_ZN12_GLOBAL__N_110CFRefCountD0Ev", "(anonymous namespace)::CFRefCount::~CFRefCount()"},
    {"_ZN12_GLOBAL__N_110CFRefCount16RegisterPrintersERSt6vectorIPN5clang4ento7GRState7PrinterESaIS6_EE", "(anonymous namespace)::CFRefCount::RegisterPrinters(std::vector<clang::ento::GRState::Printer*, std::allocator<clang::ento::GRState::Printer*> >&)"},
    {"_ZN12_GLOBAL__N_110CFRefCount14RegisterChecksERN5clang4ento10ExprEngineE", "(anonymous namespace)::CFRefCount::RegisterChecks(clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_110CFRefCount8evalCallERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_8CallExprENS2_4SValEPNS2_12ExplodedNodeE", "(anonymous namespace)::CFRefCount::evalCall(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::CallExpr const*, clang::ento::SVal, clang::ento::ExplodedNode*)"},
    {"_ZN12_GLOBAL__N_110CFRefCount15evalObjCMessageERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderENS2_11ObjCMessageEPNS2_12ExplodedNodeEPKNS2_7GRStateE", "(anonymous namespace)::CFRefCount::evalObjCMessage(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ento::ObjCMessage, clang::ento::ExplodedNode*, clang::ento::GRState const*)"},
    {"_ZN12_GLOBAL__N_110CFRefCount8evalBindERN5clang4ento18StmtNodeBuilderRefENS2_4SValES5_", "(anonymous namespace)::CFRefCount::evalBind(clang::ento::StmtNodeBuilderRef&, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_110CFRefCount11evalEndPathERN5clang4ento10ExprEngineERNS2_24EndOfFunctionNodeBuilderE", "(anonymous namespace)::CFRefCount::evalEndPath(clang::ento::ExprEngine&, clang::ento::EndOfFunctionNodeBuilder&)"},
    {"_ZN12_GLOBAL__N_110CFRefCount15evalDeadSymbolsERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPNS2_12ExplodedNodeEPKNS2_7GRStateERNS2_12SymbolReaperE", "(anonymous namespace)::CFRefCount::evalDeadSymbols(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_110CFRefCount10evalReturnERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_10ReturnStmtEPNS2_12ExplodedNodeE", "(anonymous namespace)::CFRefCount::evalReturn(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ReturnStmt const*, clang::ento::ExplodedNode*)"},
    {"_ZN12_GLOBAL__N_110CFRefCount10evalAssumeEPKN5clang4ento7GRStateENS2_4SValEb", "(anonymous namespace)::CFRefCount::evalAssume(clang::ento::GRState const*, clang::ento::SVal, bool)"},
    {"_ZN4llvm12ImmutableMapIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValENS_16ImutKeyValueInfoIS5_S7_EEE7Factory6removeESA_RKS5_", "llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::Factory::remove(llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >, clang::ento::SymbolData const* const&)"},
    {"_ZNK5clang4ento7GRState3setIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS3_16ImutKeyValueInfoIS7_S9_EEEEEEPKS1_NS0_12GRStateTraitIT_E9data_typeE", "clang::ento::GRState const* clang::ento::GRState::set<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >(clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::data_type) const"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::operator++()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEEENS_12DenseMapInfoIjEENSD_ISC_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEEESaISC_EE9push_backERKSC_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE15remove_internalERKS6_PNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::remove_internal(clang::ento::SymbolData const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE13markImmutableEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE11balanceTreeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE10createNodeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE16removeMinBindingEPNS_11ImutAVLTreeIS9_EERSD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS2_16ImutKeyValueInfoIS6_S8_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS2_16ImutKeyValueInfoIS6_S8_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::DeleteContext(void*)"},
    {"_ZNK5clang4ento7GRState3setIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS3_16ImutKeyValueInfoIS7_S9_EEEEEEPKS1_NS0_12GRStateTraitIT_E8key_typeENSH_10value_typeE", "clang::ento::GRState const* clang::ento::GRState::set<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >(clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::key_type, clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::value_type) const"},
    {"_ZN12_GLOBAL__N_110CFRefCount23HandleAutoreleaseCountsEPKN5clang4ento7GRStateENS_26GenericNodeBuilderRefCountEPNS2_12ExplodedNodeERNS2_10ExprEngineEPKNS2_10SymbolDataENS_6RefValERb", "(anonymous namespace)::CFRefCount::HandleAutoreleaseCounts(clang::ento::GRState const*, (anonymous namespace)::GenericNodeBuilderRefCount, clang::ento::ExplodedNode*, clang::ento::ExprEngine&, clang::ento::SymbolData const*, (anonymous namespace)::RefVal, bool&)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager16getMethodSummaryEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::RetainSummaryManager::getMethodSummary(clang::ObjCMethodDecl const*)"},
    {"_ZN12_GLOBAL__N_115CFRefLeakReportC1ERNS_8CFRefBugERKNS_10CFRefCountEPN5clang4ento12ExplodedNodeEPKNS7_10SymbolDataERNS7_10ExprEngineE", "(anonymous namespace)::CFRefLeakReport::CFRefLeakReport((anonymous namespace)::CFRefBug&, (anonymous namespace)::CFRefCount const&, clang::ento::ExplodedNode*, clang::ento::SymbolData const*, clang::ento::ExprEngine&)"},
    {"_ZN12_GLOBAL__N_111CFRefReportD1Ev", "(anonymous namespace)::CFRefReport::~CFRefReport()"},
    {"_ZN12_GLOBAL__N_111CFRefReportD0Ev", "(anonymous namespace)::CFRefReport::~CFRefReport()"},
    {"_ZN12_GLOBAL__N_111CFRefReport9VisitNodeEPKN5clang4ento12ExplodedNodeES5_RNS2_18BugReporterContextE", "(anonymous namespace)::CFRefReport::VisitNode(clang::ento::ExplodedNode const*, clang::ento::ExplodedNode const*, clang::ento::BugReporterContext&)"},
    {"_ZN12_GLOBAL__N_111CFRefReport23getExtraDescriptiveTextEv", "(anonymous namespace)::CFRefReport::getExtraDescriptiveText()"},
    {"_ZN12_GLOBAL__N_111CFRefReport10getEndPathERN5clang4ento18BugReporterContextEPKNS2_12ExplodedNodeE", "(anonymous namespace)::CFRefReport::getEndPath(clang::ento::BugReporterContext&, clang::ento::ExplodedNode const*)"},
    {"_ZNK12_GLOBAL__N_111CFRefReport9getRangesEv", "(anonymous namespace)::CFRefReport::getRanges() const"},
    {"_Z17GetAllocationSiteRN5clang4ento14GRStateManagerEPKNS0_12ExplodedNodeEPKNS0_10SymbolDataE", "GetAllocationSite(clang::ento::GRStateManager&, clang::ento::ExplodedNode const*, clang::ento::SymbolData const*)"},
    {"_ZN12_GLOBAL__N_115CFRefLeakReportD1Ev", "(anonymous namespace)::CFRefLeakReport::~CFRefLeakReport()"},
    {"_ZN12_GLOBAL__N_115CFRefLeakReportD0Ev", "(anonymous namespace)::CFRefLeakReport::~CFRefLeakReport()"},
    {"_ZN12_GLOBAL__N_115CFRefLeakReport10getEndPathERN5clang4ento18BugReporterContextEPKNS2_12ExplodedNodeE", "(anonymous namespace)::CFRefLeakReport::getEndPath(clang::ento::BugReporterContext&, clang::ento::ExplodedNode const*)"},
    {"_ZNK12_GLOBAL__N_115CFRefLeakReport11getLocationEv", "(anonymous namespace)::CFRefLeakReport::getLocation() const"},
    {"_ZN12_GLOBAL__N_117FindUniqueBindingD1Ev", "(anonymous namespace)::FindUniqueBinding::~FindUniqueBinding()"},
    {"_ZN12_GLOBAL__N_117FindUniqueBindingD0Ev", "(anonymous namespace)::FindUniqueBinding::~FindUniqueBinding()"},
    {"_ZN12_GLOBAL__N_117FindUniqueBinding13HandleBindingERN5clang4ento12StoreManagerEPKvPKNS2_9MemRegionENS2_4SValE", "(anonymous namespace)::FindUniqueBinding::HandleBinding(clang::ento::StoreManager&, void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager24getInstanceMethodSummaryEN5clang8SelectorEPNS1_14IdentifierInfoEPKNS1_17ObjCInterfaceDeclEPKNS1_14ObjCMethodDeclENS1_8QualTypeE", "(anonymous namespace)::RetainSummaryManager::getInstanceMethodSummary(clang::Selector, clang::IdentifierInfo*, clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager21getClassMethodSummaryEN5clang8SelectorEPNS1_14IdentifierInfoEPKNS1_17ObjCInterfaceDeclEPKNS1_14ObjCMethodDeclENS1_8QualTypeE", "(anonymous namespace)::RetainSummaryManager::getClassMethodSummary(clang::Selector, clang::IdentifierInfo*, clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_116ObjCSummaryCache4findEPKN5clang17ObjCInterfaceDeclEPNS1_14IdentifierInfoENS1_8SelectorE", "(anonymous namespace)::ObjCSummaryCache::find(clang::ObjCInterfaceDecl const*, clang::IdentifierInfo*, clang::Selector)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager22getCommonMethodSummaryEPKN5clang14ObjCMethodDeclENS1_8SelectorENS1_8QualTypeE", "(anonymous namespace)::RetainSummaryManager::getCommonMethodSummary(clang::ObjCMethodDecl const*, clang::Selector, clang::QualType)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager28updateSummaryFromAnnotationsERNS_13RetainSummaryEPKN5clang14ObjCMethodDeclE", "(anonymous namespace)::RetainSummaryManager::updateSummaryFromAnnotations((anonymous namespace)::RetainSummary&, clang::ObjCMethodDecl const*)"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_114ObjCSummaryKeyEPNS1_13RetainSummaryENS_12DenseMapInfoIS2_EENS5_IS4_EEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::ObjCSummaryKey, (anonymous namespace)::RetainSummary*, llvm::DenseMapInfo<(anonymous namespace)::ObjCSummaryKey>, llvm::DenseMapInfo<(anonymous namespace)::RetainSummary*> >::operator[]((anonymous namespace)::ObjCSummaryKey const&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_114ObjCSummaryKeyEPNS1_13RetainSummaryENS_12DenseMapInfoIS2_EENS5_IS4_EEE15LookupBucketForERKS2_RPSt4pairIS2_S4_E", "llvm::DenseMap<(anonymous namespace)::ObjCSummaryKey, (anonymous namespace)::RetainSummary*, llvm::DenseMapInfo<(anonymous namespace)::ObjCSummaryKey>, llvm::DenseMapInfo<(anonymous namespace)::RetainSummary*> >::LookupBucketFor((anonymous namespace)::ObjCSummaryKey const&, std::pair<(anonymous namespace)::ObjCSummaryKey, (anonymous namespace)::RetainSummary*>*&) const"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_16RefValEEEE12add_internalERKSt4pairIS6_S8_EPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >::add_internal(std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> >*)"},
    {"_ZN12_GLOBAL__N_110CFRefCount17HandleSymbolDeathEPKN5clang4ento7GRStateEPKNS2_10SymbolDataENS_6RefValERN4llvm15SmallVectorImplIS8_EE", "(anonymous namespace)::CFRefCount::HandleSymbolDeath(clang::ento::GRState const*, clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::SmallVectorImpl<clang::ento::SymbolData const*>&)"},
    {"_ZN12_GLOBAL__N_110CFRefCount12ProcessLeaksEPKN5clang4ento7GRStateERN4llvm15SmallVectorImplIPKNS2_10SymbolDataEEERNS_26GenericNodeBuilderRefCountERNS2_10ExprEngineEPNS2_12ExplodedNodeE", "(anonymous namespace)::CFRefCount::ProcessLeaks(clang::ento::GRState const*, llvm::SmallVectorImpl<clang::ento::SymbolData const*>&, (anonymous namespace)::GenericNodeBuilderRefCount&, clang::ento::ExprEngine&, clang::ento::ExplodedNode*)"},
    {"_ZNK5clang4ento7GRState6removeIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS3_16ImutKeyValueInfoIS7_S9_EEEEEEPKS1_NS0_12GRStateTraitIT_E8key_typeE", "clang::ento::GRState const* clang::ento::GRState::remove<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >(clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::key_type) const"},
    {"_ZN12_GLOBAL__N_120StopTrackingCallbackD1Ev", "(anonymous namespace)::StopTrackingCallback::~StopTrackingCallback()"},
    {"_ZN12_GLOBAL__N_120StopTrackingCallback11VisitSymbolEPKN5clang4ento10SymbolDataE", "(anonymous namespace)::StopTrackingCallback::VisitSymbol(clang::ento::SymbolData const*)"},
    {"_ZN12_GLOBAL__N_120StopTrackingCallbackD0Ev", "(anonymous namespace)::StopTrackingCallback::~StopTrackingCallback()"},
    {"_ZN12_GLOBAL__N_110CFRefCount11evalSummaryERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_4ExprERKNS2_17CallOrObjCMessageENS_16InstanceReceiverERKNS_13RetainSummaryEPKNS2_9MemRegionEPNS2_12ExplodedNodeEPKNS2_7GRStateE", "(anonymous namespace)::CFRefCount::evalSummary(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::Expr const*, clang::ento::CallOrObjCMessage const&, (anonymous namespace)::InstanceReceiver, (anonymous namespace)::RetainSummary const&, clang::ento::MemRegion const*, clang::ento::ExplodedNode*, clang::ento::GRState const*)"},
    {"_ZN12_GLOBAL__N_110CFRefCount6UpdateEPKN5clang4ento7GRStateEPKNS2_10SymbolDataENS_6RefValE9ArgEffectRNS9_4KindE", "(anonymous namespace)::CFRefCount::Update(clang::ento::GRState const*, clang::ento::SymbolData const*, (anonymous namespace)::RefVal, ArgEffect, (anonymous namespace)::RefVal::Kind&)"},
    {"_ZN12_GLOBAL__N_116InstanceReceiver20getSValAsScalarOrLocEPKN5clang4ento7GRStateE", "(anonymous namespace)::InstanceReceiver::getSValAsScalarOrLoc(clang::ento::GRState const*)"},
    {"_Z13GetReturnTypePKN5clang4ExprERNS_10ASTContextE", "GetReturnType(clang::Expr const*, clang::ASTContext&)"},
    {"_ZNK5clang4ento7GRState3getIN12_GLOBAL__N_123AutoreleasePoolContentsEEENS0_12GRStateTraitIT_E11lookup_typeENS7_8key_typeE", "clang::ento::GRStateTrait<(anonymous namespace)::AutoreleasePoolContents>::lookup_type clang::ento::GRState::get<(anonymous namespace)::AutoreleasePoolContents>(clang::ento::GRStateTrait<(anonymous namespace)::AutoreleasePoolContents>::key_type) const"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager15getUnarySummaryEPKN5clang12FunctionTypeENS0_13UnaryFuncKindE", "(anonymous namespace)::RetainSummaryManager::getUnarySummary(clang::FunctionType const*, (anonymous namespace)::RetainSummaryManager::UnaryFuncKind)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager25getCFCreateGetRuleSummaryEPKN5clang12FunctionDeclEN4llvm9StringRefE", "(anonymous namespace)::RetainSummaryManager::getCFCreateGetRuleSummary(clang::FunctionDecl const*, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_120RetainReleaseCheckerD1Ev", "(anonymous namespace)::RetainReleaseChecker::~RetainReleaseChecker()"},
    {"_ZN12_GLOBAL__N_120RetainReleaseCheckerD0Ev", "(anonymous namespace)::RetainReleaseChecker::~RetainReleaseChecker()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120RetainReleaseCheckerEE9_PreVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::RetainReleaseChecker>::_PreVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120RetainReleaseCheckerEE10_PostVisitERNS0_14CheckerContextEPKNS_4StmtE", "clang::ento::CheckerVisitor<(anonymous namespace)::RetainReleaseChecker>::_PostVisit(clang::ento::CheckerContext&, clang::Stmt const*)"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120RetainReleaseCheckerEED1Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::RetainReleaseChecker>::~CheckerVisitor()"},
    {"_ZN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120RetainReleaseCheckerEED0Ev", "clang::ento::CheckerVisitor<(anonymous namespace)::RetainReleaseChecker>::~CheckerVisitor()"},
    {"_ZN12_GLOBAL__N_118LeakWithinFunctionD1Ev", "(anonymous namespace)::LeakWithinFunction::~LeakWithinFunction()"},
    {"_ZN12_GLOBAL__N_118LeakWithinFunctionD0Ev", "(anonymous namespace)::LeakWithinFunction::~LeakWithinFunction()"},
    {"_ZNK12_GLOBAL__N_14Leak14getDescriptionEv", "(anonymous namespace)::Leak::getDescription() const"},
    {"_ZNK12_GLOBAL__N_14Leak6isLeakEv", "(anonymous namespace)::Leak::isLeak() const"},
    {"_ZN12_GLOBAL__N_14LeakD1Ev", "(anonymous namespace)::Leak::~Leak()"},
    {"_ZN12_GLOBAL__N_14LeakD0Ev", "(anonymous namespace)::Leak::~Leak()"},
    {"_ZN12_GLOBAL__N_18CFRefBugD1Ev", "(anonymous namespace)::CFRefBug::~CFRefBug()"},
    {"_ZN12_GLOBAL__N_18CFRefBugD0Ev", "(anonymous namespace)::CFRefBug::~CFRefBug()"},
    {"_ZNK12_GLOBAL__N_18CFRefBug6isLeakEv", "(anonymous namespace)::CFRefBug::isLeak() const"},
    {"_ZN12_GLOBAL__N_112LeakAtReturnD1Ev", "(anonymous namespace)::LeakAtReturn::~LeakAtReturn()"},
    {"_ZN12_GLOBAL__N_112LeakAtReturnD0Ev", "(anonymous namespace)::LeakAtReturn::~LeakAtReturn()"},
    {"_ZN12_GLOBAL__N_124ReturnedNotOwnedForOwnedD1Ev", "(anonymous namespace)::ReturnedNotOwnedForOwned::~ReturnedNotOwnedForOwned()"},
    {"_ZN12_GLOBAL__N_124ReturnedNotOwnedForOwnedD0Ev", "(anonymous namespace)::ReturnedNotOwnedForOwned::~ReturnedNotOwnedForOwned()"},
    {"_ZNK12_GLOBAL__N_124ReturnedNotOwnedForOwned14getDescriptionEv", "(anonymous namespace)::ReturnedNotOwnedForOwned::getDescription() const"},
    {"_ZN12_GLOBAL__N_115OverAutoreleaseD1Ev", "(anonymous namespace)::OverAutorelease::~OverAutorelease()"},
    {"_ZN12_GLOBAL__N_115OverAutoreleaseD0Ev", "(anonymous namespace)::OverAutorelease::~OverAutorelease()"},
    {"_ZNK12_GLOBAL__N_115OverAutorelease14getDescriptionEv", "(anonymous namespace)::OverAutorelease::getDescription() const"},
    {"_ZN12_GLOBAL__N_115DeallocNotOwnedD1Ev", "(anonymous namespace)::DeallocNotOwned::~DeallocNotOwned()"},
    {"_ZN12_GLOBAL__N_115DeallocNotOwnedD0Ev", "(anonymous namespace)::DeallocNotOwned::~DeallocNotOwned()"},
    {"_ZNK12_GLOBAL__N_115DeallocNotOwned14getDescriptionEv", "(anonymous namespace)::DeallocNotOwned::getDescription() const"},
    {"_ZN12_GLOBAL__N_19DeallocGCD1Ev", "(anonymous namespace)::DeallocGC::~DeallocGC()"},
    {"_ZN12_GLOBAL__N_19DeallocGCD0Ev", "(anonymous namespace)::DeallocGC::~DeallocGC()"},
    {"_ZNK12_GLOBAL__N_19DeallocGC14getDescriptionEv", "(anonymous namespace)::DeallocGC::getDescription() const"},
    {"_ZN12_GLOBAL__N_110BadReleaseD1Ev", "(anonymous namespace)::BadRelease::~BadRelease()"},
    {"_ZN12_GLOBAL__N_110BadReleaseD0Ev", "(anonymous namespace)::BadRelease::~BadRelease()"},
    {"_ZNK12_GLOBAL__N_110BadRelease14getDescriptionEv", "(anonymous namespace)::BadRelease::getDescription() const"},
    {"_ZN12_GLOBAL__N_115UseAfterReleaseD1Ev", "(anonymous namespace)::UseAfterRelease::~UseAfterRelease()"},
    {"_ZN12_GLOBAL__N_115UseAfterReleaseD0Ev", "(anonymous namespace)::UseAfterRelease::~UseAfterRelease()"},
    {"_ZNK12_GLOBAL__N_115UseAfterRelease14getDescriptionEv", "(anonymous namespace)::UseAfterRelease::getDescription() const"},
    {"_ZN12_GLOBAL__N_110CFRefCount15BindingsPrinterD1Ev", "(anonymous namespace)::CFRefCount::BindingsPrinter::~BindingsPrinter()"},
    {"_ZN12_GLOBAL__N_110CFRefCount15BindingsPrinterD0Ev", "(anonymous namespace)::CFRefCount::BindingsPrinter::~BindingsPrinter()"},
    {"_ZN12_GLOBAL__N_110CFRefCount15BindingsPrinter5PrintERN4llvm11raw_ostreamEPKN5clang4ento7GRStateEPKcSB_", "(anonymous namespace)::CFRefCount::BindingsPrinter::Print(llvm::raw_ostream&, clang::ento::GRState const*, char const*, char const*)"},
    {"_Z9PrintPoolRN4llvm11raw_ostreamEPKN5clang4ento10SymbolDataEPKNS3_7GRStateE", "PrintPool(llvm::raw_ostream&, clang::ento::SymbolData const*, clang::ento::GRState const*)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager15addPanicSummaryEPKcz", "(anonymous namespace)::RetainSummaryManager::addPanicSummary(char const*, ...)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager18addInstMethSummaryEPKcPNS_13RetainSummaryEz", "(anonymous namespace)::RetainSummaryManager::addInstMethSummary(char const*, (anonymous namespace)::RetainSummary*, ...)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager16addMethodSummaryEPN5clang14IdentifierInfoERNS_16ObjCSummaryCacheEPNS_13RetainSummaryEP13__va_list_tag", "(anonymous namespace)::RetainSummaryManager::addMethodSummary(clang::IdentifierInfo*, (anonymous namespace)::ObjCSummaryCache&, (anonymous namespace)::RetainSummary*, __va_list_tag*)"},
    {"_ZN12_GLOBAL__N_120RetainSummaryManager17addClsMethSummaryEPN5clang14IdentifierInfoEPNS_13RetainSummaryEz", "(anonymous namespace)::RetainSummaryManager::addClsMethSummary(clang::IdentifierInfo*, (anonymous namespace)::RetainSummary*, ...)"},
    {"_ZNK5clang4Decl7getAttrINS_18NSConsumesSelfAttrEEEPT_v", "clang::NSConsumesSelfAttr* clang::Decl::getAttr<clang::NSConsumesSelfAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_14NSConsumedAttrEEEPT_v", "clang::NSConsumedAttr* clang::Decl::getAttr<clang::NSConsumedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_14CFConsumedAttrEEEPT_v", "clang::CFConsumedAttr* clang::Decl::getAttr<clang::CFConsumedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_21NSReturnsRetainedAttrEEEPT_v", "clang::NSReturnsRetainedAttr* clang::Decl::getAttr<clang::NSReturnsRetainedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_24NSReturnsNotRetainedAttrEEEPT_v", "clang::NSReturnsNotRetainedAttr* clang::Decl::getAttr<clang::NSReturnsNotRetainedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_21CFReturnsRetainedAttrEEEPT_v", "clang::CFReturnsRetainedAttr* clang::Decl::getAttr<clang::CFReturnsRetainedAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_24CFReturnsNotRetainedAttrEEEPT_v", "clang::CFReturnsNotRetainedAttr* clang::Decl::getAttr<clang::CFReturnsNotRetainedAttr>() const"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIj9ArgEffectEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIj9ArgEffectEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIj9ArgEffectEEEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIj9ArgEffectEEEENS_12DenseMapInfoIjEENS7_IS6_EEE16InsertIntoBucketERKjRKS6_PSt4pairIjS6_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIj9ArgEffectEEEENS_12DenseMapInfoIjEENS7_IS6_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE3addEPNS_11ImutAVLTreeIS3_EERKSt4pairIjS2_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, std::pair<unsigned int, ArgEffect> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS3_EERNS_26ImutAVLTreeInOrderIteratorIS3_EESA_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIj9ArgEffectEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE12add_internalERKSt4pairIjS2_EPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::add_internal(std::pair<unsigned int, ArgEffect> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE13markImmutableEPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE10createNodeEPNS_11ImutAVLTreeIS3_EERKSt4pairIjS2_ES7_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, std::pair<unsigned int, ArgEffect> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEE11balanceTreeEPNS_11ImutAVLTreeIS3_EERKSt4pairIjS2_ES7_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*, std::pair<unsigned int, ArgEffect> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >*)"},
    {"_ZNK5clang4ento17CallOrObjCMessage17getArgSourceRangeEj", "clang::ento::CallOrObjCMessage::getArgSourceRange(unsigned int) const"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS3_IS6_jNS2_16ImutKeyValueInfoIS6_jEEEENS7_IS6_S9_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS3_IS6_jNS2_16ImutKeyValueInfoIS6_jEEEENS7_IS6_S9_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > > >::DeleteContext(void*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::~ImutAVLFactory()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS0_12ImmutableMapIS7_jNS2_IS7_jEEEEEEEESaISD_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSD_SF_EERKSD_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS7_jNS2_IS7_jEEEEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE16InsertIntoBucketERKjRKSD_PSt4pairIjSD_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS7_jNS2_IS7_jEEEEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*> >::grow(unsigned int)"},
    {"_ZN4llvm12ImmutableMapIPKN5clang4ento10SymbolDataENS0_IS5_jNS_16ImutKeyValueInfoIS5_jEEEENS6_IS5_S8_EEE7Factory3addESA_RKS5_RKS8_", "llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::Factory::add(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >, clang::ento::SymbolData const* const&, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE3addEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE16getCanonicalTreeEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeISA_EERNS_26ImutAVLTreeInOrderIteratorISA_EESH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::operator++()"},
    {"_ZNK4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEE7isEqualERKS8_", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::isEqual(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > const&) const"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE12add_internalERKSt4pairIS6_S9_EPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::add_internal(std::pair<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE13markImmutableEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE10createNodeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataENS_12ImmutableMapIS6_jNS1_IS6_jEEEEEEE11balanceTreeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*, std::pair<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >*)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataENS3_IS6_jNS2_16ImutKeyValueInfoIS6_jEEEENS7_IS6_S9_EEEEE6LookupESB_S6_", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > > >::Lookup(llvm::ImmutableMap<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > > >, clang::ento::SymbolData const*)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm13ImmutableListIPKNS0_10SymbolDataEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableList<clang::ento::SymbolData const*> >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm13ImmutableListIPKNS0_10SymbolDataEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableList<clang::ento::SymbolData const*> >::DeleteContext(void*)"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEED1Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEED0Ev", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERSC_", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm20ImmutableListFactoryIPKN5clang4ento10SymbolDataEE6concatERKS5_NS_13ImmutableListIS5_EE", "llvm::ImmutableListFactory<clang::ento::SymbolData const*>::concat(clang::ento::SymbolData const* const&, llvm::ImmutableList<clang::ento::SymbolData const*>)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento10SymbolDataEcNS_12DenseMapInfoIS5_EENS6_IcEEE6insertERKSt4pairIS5_cE", "llvm::DenseMap<clang::ento::SymbolData const*, char, llvm::DenseMapInfo<clang::ento::SymbolData const*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::ento::SymbolData const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento10SymbolDataEcNS_12DenseMapInfoIS5_EENS6_IcEEE16InsertIntoBucketERKS5_RKcPSt4pairIS5_cE", "llvm::DenseMap<clang::ento::SymbolData const*, char, llvm::DenseMapInfo<clang::ento::SymbolData const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::ento::SymbolData const* const&, char const&, std::pair<clang::ento::SymbolData const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento10SymbolDataEcNS_12DenseMapInfoIS5_EENS6_IcEEE4growEj", "llvm::DenseMap<clang::ento::SymbolData const*, char, llvm::DenseMapInfo<clang::ento::SymbolData const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang4ento7GRState7PrinterD1Ev", "clang::ento::GRState::Printer::~Printer()"},
    {"_ZN5clang4ento7GRState7PrinterD0Ev", "clang::ento::GRState::Printer::~Printer()"},
    {"_ZNSt6vectorIPN5clang4ento7GRState7PrinterESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<clang::ento::GRState::Printer*, std::allocator<clang::ento::GRState::Printer*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::GRState::Printer**, std::vector<clang::ento::GRState::Printer*, std::allocator<clang::ento::GRState::Printer*> > >, clang::ento::GRState::Printer* const&)"},
    {"_ZN5clang4ento13TransferFuncsD1Ev", "clang::ento::TransferFuncs::~TransferFuncs()"},
    {"_ZN5clang4ento13TransferFuncsD0Ev", "clang::ento::TransferFuncs::~TransferFuncs()"},
    {"_ZN5clang4ento13TransferFuncs16RegisterPrintersERSt6vectorIPNS0_7GRState7PrinterESaIS5_EE", "clang::ento::TransferFuncs::RegisterPrinters(std::vector<clang::ento::GRState::Printer*, std::allocator<clang::ento::GRState::Printer*> >&)"},
    {"_ZN5clang4ento13TransferFuncs14RegisterChecksERNS0_10ExprEngineE", "clang::ento::TransferFuncs::RegisterChecks(clang::ento::ExprEngine&)"},
    {"_ZN5clang4ento13TransferFuncs8evalCallERNS0_15ExplodedNodeSetERNS0_10ExprEngineERNS0_15StmtNodeBuilderEPKNS_8CallExprENS0_4SValEPNS0_12ExplodedNodeE", "clang::ento::TransferFuncs::evalCall(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::CallExpr const*, clang::ento::SVal, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento13TransferFuncs15evalObjCMessageERNS0_15ExplodedNodeSetERNS0_10ExprEngineERNS0_15StmtNodeBuilderENS0_11ObjCMessageEPNS0_12ExplodedNodeEPKNS0_7GRStateE", "clang::ento::TransferFuncs::evalObjCMessage(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ento::ObjCMessage, clang::ento::ExplodedNode*, clang::ento::GRState const*)"},
    {"_ZN5clang4ento13TransferFuncs8evalBindERNS0_18StmtNodeBuilderRefENS0_4SValES4_", "clang::ento::TransferFuncs::evalBind(clang::ento::StmtNodeBuilderRef&, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN5clang4ento13TransferFuncs11evalEndPathERNS0_10ExprEngineERNS0_24EndOfFunctionNodeBuilderE", "clang::ento::TransferFuncs::evalEndPath(clang::ento::ExprEngine&, clang::ento::EndOfFunctionNodeBuilder&)"},
    {"_ZN5clang4ento13TransferFuncs15evalDeadSymbolsERNS0_15ExplodedNodeSetERNS0_10ExprEngineERNS0_15StmtNodeBuilderEPNS0_12ExplodedNodeEPKNS0_7GRStateERNS0_12SymbolReaperE", "clang::ento::TransferFuncs::evalDeadSymbols(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN5clang4ento13TransferFuncs10evalReturnERNS0_15ExplodedNodeSetERNS0_10ExprEngineERNS0_15StmtNodeBuilderEPKNS_10ReturnStmtEPNS0_12ExplodedNodeE", "clang::ento::TransferFuncs::evalReturn(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ReturnStmt const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento13TransferFuncs10evalAssumeEPKNS0_7GRStateENS0_4SValEb", "clang::ento::TransferFuncs::evalAssume(clang::ento::GRState const*, clang::ento::SVal, bool)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIj9ArgEffectEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<unsigned int, ArgEffect> >::~ImutAVLFactory()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEjEEEC2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> >::ImutAVLFactory()"},
    {"_ZN5clang4ento10ExprEngine13evalArgumentsENS_17ConstExprIteratorES2_PKNS_17FunctionProtoTypeEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetEb", "clang::ento::ExprEngine::evalArguments(clang::ConstExprIterator, clang::ConstExprIterator, clang::FunctionProtoType const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&, bool)"},
    {"_ZN5clang4ento10ExprEngine16getCXXThisRegionEPKNS_13CXXRecordDeclEPKNS_17StackFrameContextE", "clang::ento::ExprEngine::getCXXThisRegion(clang::CXXRecordDecl const*, clang::StackFrameContext const*)"},
    {"_ZN5clang4ento10ExprEngine16getCXXThisRegionEPKNS_13CXXMethodDeclEPKNS_17StackFrameContextE", "clang::ento::ExprEngine::getCXXThisRegion(clang::CXXMethodDecl const*, clang::StackFrameContext const*)"},
    {"_ZN5clang4ento10ExprEngine24CreateCXXTemporaryObjectEPKNS_4ExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::CreateCXXTemporaryObject(clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine21VisitCXXConstructExprEPKNS_16CXXConstructExprEPKNS0_9MemRegionEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXConstructExpr(clang::CXXConstructExpr const*, clang::ento::MemRegion const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine18VisitCXXDestructorEPKNS_17CXXDestructorDeclEPKNS0_9MemRegionEPKNS_4StmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXDestructor(clang::CXXDestructorDecl const*, clang::ento::MemRegion const*, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine22VisitCXXMemberCallExprEPKNS_17CXXMemberCallExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXMemberCallExpr(clang::CXXMemberCallExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine14evalMethodCallEPKNS_8CallExprEPKNS_13CXXMethodDeclEPKNS_4ExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetESE_", "clang::ento::ExprEngine::evalMethodCall(clang::CallExpr const*, clang::CXXMethodDecl const*, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine24VisitCXXOperatorCallExprEPKNS_19CXXOperatorCallExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXOperatorCallExpr(clang::CXXOperatorCallExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine15VisitCXXNewExprEPKNS_10CXXNewExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXNewExpr(clang::CXXNewExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine18VisitCXXDeleteExprEPKNS_13CXXDeleteExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXDeleteExpr(clang::CXXDeleteExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento10ExprEngine16VisitCXXThisExprEPKNS_11CXXThisExprEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetE", "clang::ento::ExprEngine::VisitCXXThisExpr(clang::CXXThisExpr const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_114CallExprWLItemELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::CallExprWLItem, false>::grow(unsigned long)"},
    {"_ZN5clang4ento7CheckerD0Ev", "clang::ento::Checker::~Checker()"},
    {"_ZN5clang4ento7CheckerD1Ev", "clang::ento::Checker::~Checker()"},
    {"_ZN5clang4ento7CheckerD2Ev", "clang::ento::Checker::~Checker()"},
    {"_ZN5clang4ento14CheckerContextD1Ev", "clang::ento::CheckerContext::~CheckerContext()"},
    {"_ZN5clang4ento14CheckerContextD2Ev", "clang::ento::CheckerContext::~CheckerContext()"},
    {"_ZN5clang4ento13containsMacroEPKNS_4StmtE", "clang::ento::containsMacro(clang::Stmt const*)"},
    {"_ZN5clang4ento12containsEnumEPKNS_4StmtE", "clang::ento::containsEnum(clang::Stmt const*)"},
    {"_ZN5clang4ento19containsStaticLocalEPKNS_4StmtE", "clang::ento::containsStaticLocal(clang::Stmt const*)"},
    {"_ZN5clang4ento23containsBuiltinOffsetOfEPKNS_4StmtE", "clang::ento::containsBuiltinOffsetOf(clang::Stmt const*)"},
    {"_ZN5clang4ento8WorkList7VisitorD0Ev", "clang::ento::WorkList::Visitor::~Visitor()"},
    {"_ZN5clang4ento8WorkList7VisitorD1Ev", "clang::ento::WorkList::Visitor::~Visitor()"},
    {"_ZN5clang4ento8WorkList7VisitorD2Ev", "clang::ento::WorkList::Visitor::~Visitor()"},
    {"_ZN5clang4ento8WorkListD0Ev", "clang::ento::WorkList::~WorkList()"},
    {"_ZN5clang4ento8WorkListD1Ev", "clang::ento::WorkList::~WorkList()"},
    {"_ZN5clang4ento8WorkListD2Ev", "clang::ento::WorkList::~WorkList()"},
    {"_ZN5clang4ento8WorkList7makeDFSEv", "clang::ento::WorkList::makeDFS()"},
    {"_ZN5clang4ento8WorkList7makeBFSEv", "clang::ento::WorkList::makeBFS()"},
    {"_ZN5clang4ento8WorkList23makeBFSBlockDFSContentsEv", "clang::ento::WorkList::makeBFSBlockDFSContents()"},
    {"_ZN5clang4ento10CoreEngine15ExecuteWorkListEPKNS_15LocationContextEjPKNS0_7GRStateE", "clang::ento::CoreEngine::ExecuteWorkList(clang::LocationContext const*, unsigned int, clang::ento::GRState const*)"},
    {"_ZN5clang4ento10CoreEngine12generateNodeERKNS_12ProgramPointEPKNS0_7GRStateEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::generateNode(clang::ProgramPoint const&, clang::ento::GRState const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine15HandleBlockEdgeERKNS_9BlockEdgeEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleBlockEdge(clang::BlockEdge const&, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine19HandleBlockEntranceERKNS_13BlockEntranceEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleBlockEntrance(clang::BlockEntrance const&, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine15HandleCallEnterERKNS_9CallEnterEPKNS_8CFGBlockEjPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleCallEnter(clang::CallEnter const&, clang::CFGBlock const*, unsigned int, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine14HandleCallExitERKNS_8CallExitEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleCallExit(clang::CallExit const&, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine14HandlePostStmtEPKNS_8CFGBlockEjPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandlePostStmt(clang::CFGBlock const*, unsigned int, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine31ExecuteWorkListWithInitialStateEPKNS_15LocationContextEjPKNS0_7GRStateERNS0_15ExplodedNodeSetE", "clang::ento::CoreEngine::ExecuteWorkListWithInitialState(clang::LocationContext const*, unsigned int, clang::ento::GRState const*, clang::ento::ExplodedNodeSet&)"},
    {"_ZN5clang4ento24EndOfFunctionNodeBuilderD1Ev", "clang::ento::EndOfFunctionNodeBuilder::~EndOfFunctionNodeBuilder()"},
    {"_ZN5clang4ento15StmtNodeBuilderC1EPKNS_8CFGBlockEjPNS0_12ExplodedNodeEPNS0_10CoreEngineERNS0_14GRStateManagerE", "clang::ento::StmtNodeBuilder::StmtNodeBuilder(clang::CFGBlock const*, unsigned int, clang::ento::ExplodedNode*, clang::ento::CoreEngine*, clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento15StmtNodeBuilderD1Ev", "clang::ento::StmtNodeBuilder::~StmtNodeBuilder()"},
    {"_ZN5clang4ento10CoreEngine15HandleBlockExitEPKNS_8CFGBlockEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleBlockExit(clang::CFGBlock const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento10CoreEngine12HandleBranchEPKNS_4StmtES4_PKNS_8CFGBlockEPNS0_12ExplodedNodeE", "clang::ento::CoreEngine::HandleBranch(clang::Stmt const*, clang::Stmt const*, clang::CFGBlock const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento17BranchNodeBuilderD1Ev", "clang::ento::BranchNodeBuilder::~BranchNodeBuilder()"},
    {"_ZN5clang4ento22GenericNodeBuilderImpl16generateNodeImplEPKNS0_7GRStateEPNS0_12ExplodedNodeENS_12ProgramPointEb", "clang::ento::GenericNodeBuilderImpl::generateNodeImpl(clang::ento::GRState const*, clang::ento::ExplodedNode*, clang::ProgramPoint, bool)"},
    {"_ZN5clang4ento15StmtNodeBuilderC2EPKNS_8CFGBlockEjPNS0_12ExplodedNodeEPNS0_10CoreEngineERNS0_14GRStateManagerE", "clang::ento::StmtNodeBuilder::StmtNodeBuilder(clang::CFGBlock const*, unsigned int, clang::ento::ExplodedNode*, clang::ento::CoreEngine*, clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento15StmtNodeBuilderD2Ev", "clang::ento::StmtNodeBuilder::~StmtNodeBuilder()"},
    {"_ZN5clang4ento15StmtNodeBuilder22GenerateAutoTransitionEPNS0_12ExplodedNodeE", "clang::ento::StmtNodeBuilder::GenerateAutoTransition(clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento15StmtNodeBuilder8MakeNodeERNS0_15ExplodedNodeSetEPKNS_4StmtEPNS0_12ExplodedNodeEPKNS0_7GRStateENS_12ProgramPoint4KindE", "clang::ento::StmtNodeBuilder::MakeNode(clang::ento::ExplodedNodeSet&, clang::Stmt const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ProgramPoint::Kind)"},
    {"_ZN5clang4ento15StmtNodeBuilder20generateNodeInternalEPKNS_4StmtEPKNS0_7GRStateEPNS0_12ExplodedNodeENS_12ProgramPoint4KindEPKv", "clang::ento::StmtNodeBuilder::generateNodeInternal(clang::Stmt const*, clang::ento::GRState const*, clang::ento::ExplodedNode*, clang::ProgramPoint::Kind, void const*)"},
    {"_ZN5clang4ento15StmtNodeBuilder20generateNodeInternalERKNS_12ProgramPointEPKNS0_7GRStateEPNS0_12ExplodedNodeE", "clang::ento::StmtNodeBuilder::generateNodeInternal(clang::ProgramPoint const&, clang::ento::GRState const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento17BranchNodeBuilder12generateNodeEPKNS0_7GRStateEb", "clang::ento::BranchNodeBuilder::generateNode(clang::ento::GRState const*, bool)"},
    {"_ZN5clang4ento17BranchNodeBuilderD2Ev", "clang::ento::BranchNodeBuilder::~BranchNodeBuilder()"},
    {"_ZN5clang4ento23IndirectGotoNodeBuilder12generateNodeERKNS1_8iteratorEPKNS0_7GRStateEb", "clang::ento::IndirectGotoNodeBuilder::generateNode(clang::ento::IndirectGotoNodeBuilder::iterator const&, clang::ento::GRState const*, bool)"},
    {"_ZN5clang4ento17SwitchNodeBuilder20generateCaseStmtNodeERKNS1_8iteratorEPKNS0_7GRStateE", "clang::ento::SwitchNodeBuilder::generateCaseStmtNode(clang::ento::SwitchNodeBuilder::iterator const&, clang::ento::GRState const*)"},
    {"_ZN5clang4ento17SwitchNodeBuilder23generateDefaultCaseNodeEPKNS0_7GRStateEb", "clang::ento::SwitchNodeBuilder::generateDefaultCaseNode(clang::ento::GRState const*, bool)"},
    {"_ZN5clang4ento24EndOfFunctionNodeBuilderD2Ev", "clang::ento::EndOfFunctionNodeBuilder::~EndOfFunctionNodeBuilder()"},
    {"_ZN5clang4ento24EndOfFunctionNodeBuilder20GenerateCallExitNodeEPKNS0_7GRStateE", "clang::ento::EndOfFunctionNodeBuilder::GenerateCallExitNode(clang::ento::GRState const*)"},
    {"_ZN5clang4ento24EndOfFunctionNodeBuilder12generateNodeEPKNS0_7GRStateEPKvPNS0_12ExplodedNodeE", "clang::ento::EndOfFunctionNodeBuilder::generateNode(clang::ento::GRState const*, void const*, clang::ento::ExplodedNode*)"},
    {"_ZN5clang4ento20CallEnterNodeBuilder12generateNodeEPKNS0_7GRStateE", "clang::ento::CallEnterNodeBuilder::generateNode(clang::ento::GRState const*)"},
    {"_ZN5clang4ento19CallExitNodeBuilder12generateNodeEPKNS0_7GRStateE", "clang::ento::CallExitNodeBuilder::generateNode(clang::ento::GRState const*)"},
    {"_ZN12_GLOBAL__N_119BFSBlockDFSContentsD1Ev", "(anonymous namespace)::BFSBlockDFSContents::~BFSBlockDFSContents()"},
    {"_ZN12_GLOBAL__N_119BFSBlockDFSContentsD0Ev", "(anonymous namespace)::BFSBlockDFSContents::~BFSBlockDFSContents()"},
    {"_ZNK12_GLOBAL__N_119BFSBlockDFSContents7hasWorkEv", "(anonymous namespace)::BFSBlockDFSContents::hasWork() const"},
    {"_ZN12_GLOBAL__N_119BFSBlockDFSContents7enqueueERKN5clang4ento12WorkListUnitE", "(anonymous namespace)::BFSBlockDFSContents::enqueue(clang::ento::WorkListUnit const&)"},
    {"_ZN12_GLOBAL__N_119BFSBlockDFSContents7dequeueEv", "(anonymous namespace)::BFSBlockDFSContents::dequeue()"},
    {"_ZN12_GLOBAL__N_119BFSBlockDFSContents20visitItemsInWorkListERN5clang4ento8WorkList7VisitorE", "(anonymous namespace)::BFSBlockDFSContents::visitItemsInWorkList(clang::ento::WorkList::Visitor&)"},
    {"_ZN12_GLOBAL__N_13BFSD1Ev", "(anonymous namespace)::BFS::~BFS()"},
    {"_ZN12_GLOBAL__N_13BFSD0Ev", "(anonymous namespace)::BFS::~BFS()"},
    {"_ZNK12_GLOBAL__N_13BFS7hasWorkEv", "(anonymous namespace)::BFS::hasWork() const"},
    {"_ZN12_GLOBAL__N_13BFS7enqueueERKN5clang4ento12WorkListUnitE", "(anonymous namespace)::BFS::enqueue(clang::ento::WorkListUnit const&)"},
    {"_ZN12_GLOBAL__N_13BFS7dequeueEv", "(anonymous namespace)::BFS::dequeue()"},
    {"_ZN12_GLOBAL__N_13BFS20visitItemsInWorkListERN5clang4ento8WorkList7VisitorE", "(anonymous namespace)::BFS::visitItemsInWorkList(clang::ento::WorkList::Visitor&)"},
    {"_ZN12_GLOBAL__N_13DFSD1Ev", "(anonymous namespace)::DFS::~DFS()"},
    {"_ZN12_GLOBAL__N_13DFSD0Ev", "(anonymous namespace)::DFS::~DFS()"},
    {"_ZNK12_GLOBAL__N_13DFS7hasWorkEv", "(anonymous namespace)::DFS::hasWork() const"},
    {"_ZN12_GLOBAL__N_13DFS7enqueueERKN5clang4ento12WorkListUnitE", "(anonymous namespace)::DFS::enqueue(clang::ento::WorkListUnit const&)"},
    {"_ZN12_GLOBAL__N_13DFS7dequeueEv", "(anonymous namespace)::DFS::dequeue()"},
    {"_ZN12_GLOBAL__N_13DFS20visitItemsInWorkListERN5clang4ento8WorkList7VisitorE", "(anonymous namespace)::DFS::visitItemsInWorkList(clang::ento::WorkList::Visitor&)"},
    {"_ZNSt6vectorISt4pairIN5clang9BlockEdgeEPKNS1_4ento12ExplodedNodeEESaIS7_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*>, std::allocator<std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*>*, std::vector<std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*>, std::allocator<std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*> > > >, std::pair<clang::BlockEdge, clang::ento::ExplodedNode const*> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang4ento12WorkListUnitELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::ento::WorkListUnit, false>::grow(unsigned long)"},
    {"_ZNSt5dequeIN5clang4ento12WorkListUnitESaIS2_EE17_M_reallocate_mapEmb", "std::deque<clang::ento::WorkListUnit, std::allocator<clang::ento::WorkListUnit> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIN5clang4ento12WorkListUnitESaIS2_EE17_M_initialize_mapEm", "std::_Deque_base<clang::ento::WorkListUnit, std::allocator<clang::ento::WorkListUnit> >::_M_initialize_map(unsigned long)"},
    {"_ZNK5clang4ento11Environment10lookupExprEPKNS_4StmtE", "clang::ento::Environment::lookupExpr(clang::Stmt const*) const"},
    {"_ZNK5clang4ento11Environment7getSValEPKNS_4StmtERNS0_11SValBuilderE", "clang::ento::Environment::getSVal(clang::Stmt const*, clang::ento::SValBuilder&) const"},
    {"_ZN5clang4ento18EnvironmentManager8bindExprENS0_11EnvironmentEPKNS_4StmtENS0_4SValEb", "clang::ento::EnvironmentManager::bindExpr(clang::ento::Environment, clang::Stmt const*, clang::ento::SVal, bool)"},
    {"_ZN5clang4ento18EnvironmentManager19bindExprAndLocationENS0_11EnvironmentEPKNS_4StmtENS0_4SValES6_", "clang::ento::EnvironmentManager::bindExprAndLocation(clang::ento::Environment, clang::Stmt const*, clang::ento::SVal, clang::ento::SVal)"},
    {"_ZN5clang4ento18EnvironmentManager18removeDeadBindingsENS0_11EnvironmentERNS0_12SymbolReaperEPKNS0_7GRStateERN4llvm15SmallVectorImplIPKNS0_9MemRegionEEE", "clang::ento::EnvironmentManager::removeDeadBindings(clang::ento::Environment, clang::ento::SymbolReaper&, clang::ento::GRState const*, llvm::SmallVectorImpl<clang::ento::MemRegion const*>&)"},
    {"_ZN12_GLOBAL__N_116MarkLiveCallbackD1Ev", "(anonymous namespace)::MarkLiveCallback::~MarkLiveCallback()"},
    {"_ZN12_GLOBAL__N_116MarkLiveCallback11VisitSymbolEPKN5clang4ento10SymbolDataE", "(anonymous namespace)::MarkLiveCallback::VisitSymbol(clang::ento::SymbolData const*)"},
    {"_ZN12_GLOBAL__N_116MarkLiveCallbackD0Ev", "(anonymous namespace)::MarkLiveCallback::~MarkLiveCallback()"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPKN5clang4StmtENS2_4ento4SValEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::Stmt const*, clang::ento::SVal>, false>::grow(unsigned long)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE3addEPNS_11ImutAVLTreeIS8_EERKSt4pairIS5_S7_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::pair<clang::Stmt const*, clang::ento::SVal> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS8_EERNS_26ImutAVLTreeInOrderIteratorIS8_EESF_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE12add_internalERKSt4pairIS5_S7_EPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::add_internal(std::pair<clang::Stmt const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE13markImmutableEPNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE10createNodeEPNS_11ImutAVLTreeIS8_EERKSt4pairIS5_S7_ESC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::pair<clang::Stmt const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE11balanceTreeEPNS_11ImutAVLTreeIS8_EERKSt4pairIS5_S7_ESC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::pair<clang::Stmt const*, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE6removeEPNS_11ImutAVLTreeIS8_EERKS5_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::remove(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, clang::Stmt const* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE15remove_internalERKS5_PNS_11ImutAVLTreeIS8_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::remove_internal(clang::Stmt const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE16removeMinBindingEPNS_11ImutAVLTreeIS8_EERSC_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*&)"},
    {"_ZN5clang4ento12ExplodedNode7AuditorD0Ev", "clang::ento::ExplodedNode::Auditor::~Auditor()"},
    {"_ZN5clang4ento12ExplodedNode7AuditorD1Ev", "clang::ento::ExplodedNode::Auditor::~Auditor()"},
    {"_ZN5clang4ento12ExplodedNode7AuditorD2Ev", "clang::ento::ExplodedNode::Auditor::~Auditor()"},
    {"_ZN5clang4ento12ExplodedNode10SetAuditorEPNS1_7AuditorE", "clang::ento::ExplodedNode::SetAuditor(clang::ento::ExplodedNode::Auditor*)"},
    {"_ZN5clang4ento13ExplodedGraphD1Ev", "clang::ento::ExplodedGraph::~ExplodedGraph()"},
    {"_ZN5clang4ento13ExplodedGraphD2Ev", "clang::ento::ExplodedGraph::~ExplodedGraph()"},
    {"_ZN5clang4ento13ExplodedGraph29reclaimRecentlyAllocatedNodesEv", "clang::ento::ExplodedGraph::reclaimRecentlyAllocatedNodes()"},
    {"_ZN5clang4ento12ExplodedNode14addPredecessorEPS1_RNS0_13ExplodedGraphE", "clang::ento::ExplodedNode::addPredecessor(clang::ento::ExplodedNode*, clang::ento::ExplodedGraph&)"},
    {"_ZN5clang4ento12ExplodedNode9NodeGroup7addNodeEPS1_RNS0_13ExplodedGraphE", "clang::ento::ExplodedNode::NodeGroup::addNode(clang::ento::ExplodedNode*, clang::ento::ExplodedGraph&)"},
    {"_ZN5clang4ento12ExplodedNode9NodeGroup11replaceNodeEPS1_", "clang::ento::ExplodedNode::NodeGroup::replaceNode(clang::ento::ExplodedNode*)"},
    {"_ZNK5clang4ento12ExplodedNode9NodeGroup4sizeEv", "clang::ento::ExplodedNode::NodeGroup::size() const"},
    {"_ZNK5clang4ento12ExplodedNode9NodeGroup5beginEv", "clang::ento::ExplodedNode::NodeGroup::begin() const"},
    {"_ZNK5clang4ento12ExplodedNode9NodeGroup3endEv", "clang::ento::ExplodedNode::NodeGroup::end() const"},
    {"_ZN5clang4ento13ExplodedGraph7getNodeERKNS_12ProgramPointEPKNS0_7GRStateEPb", "clang::ento::ExplodedGraph::getNode(clang::ProgramPoint const&, clang::ento::GRState const*, bool*)"},
    {"_ZNK5clang4ento13ExplodedGraph4TrimEPKPKNS0_12ExplodedNodeES6_PN4llvm8DenseMapIPKvSA_NS7_12DenseMapInfoISA_EESC_EE", "clang::ento::ExplodedGraph::Trim(clang::ento::ExplodedNode const* const*, clang::ento::ExplodedNode const* const*, llvm::DenseMap<void const*, void const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<void const*> >*) const"},
    {"_ZNK5clang4ento13ExplodedGraph12TrimInternalEPKPKNS0_12ExplodedNodeES6_PNS0_21InterExplodedGraphMapEPN4llvm8DenseMapIPKvSC_NS9_12DenseMapInfoISC_EESE_EE", "clang::ento::ExplodedGraph::TrimInternal(clang::ento::ExplodedNode const* const*, clang::ento::ExplodedNode const* const*, clang::ento::InterExplodedGraphMap*, llvm::DenseMap<void const*, void const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<void const*> >*) const"},
    {"_ZNK5clang4ento21InterExplodedGraphMap13getMappedNodeEPKNS0_12ExplodedNodeE", "clang::ento::InterExplodedGraphMap::getMappedNode(clang::ento::ExplodedNode const*) const"},
    {"_ZN4llvm8DenseMapIPKvS2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<void const*, void const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<void const*> >::InsertIntoBucket(void const* const&, void const* const&, std::pair<void const*, void const*>*)"},
    {"_ZN4llvm8DenseMapIPKvS2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<void const*, void const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<void const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEPS3_NS_12DenseMapInfoIS5_EENS7_IS6_EEE16InsertIntoBucketERKS5_RKS6_PSt4pairIS5_S6_E", "llvm::DenseMap<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode*, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<clang::ento::ExplodedNode*> >::InsertIntoBucket(clang::ento::ExplodedNode const* const&, clang::ento::ExplodedNode* const&, std::pair<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEPS3_NS_12DenseMapInfoIS5_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::ento::ExplodedNode const*, clang::ento::ExplodedNode*, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<clang::ento::ExplodedNode*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEcNS_12DenseMapInfoIS5_EENS6_IcEEE6insertERKSt4pairIS5_cE", "llvm::DenseMap<clang::ento::ExplodedNode const*, char, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::ento::ExplodedNode const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEcNS_12DenseMapInfoIS5_EENS6_IcEEE16InsertIntoBucketERKS5_RKcPSt4pairIS5_cE", "llvm::DenseMap<clang::ento::ExplodedNode const*, char, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::ento::ExplodedNode const* const&, char const&, std::pair<clang::ento::ExplodedNode const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento12ExplodedNodeEcNS_12DenseMapInfoIS5_EENS6_IcEEE4growEj", "llvm::DenseMap<clang::ento::ExplodedNode const*, char, llvm::DenseMapInfo<clang::ento::ExplodedNode const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN5clang4ento12ExplodedNodeESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::ento::ExplodedNode*, std::allocator<clang::ento::ExplodedNode*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::ExplodedNode**, std::vector<clang::ento::ExplodedNode*, std::allocator<clang::ento::ExplodedNode*> > >, clang::ento::ExplodedNode* const&)"},
    {"_ZNK4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPvS2_EEE7isEqualERKS4_", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >::isEqual(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> > const&) const"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPvS2_EEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<void*, void*> >::operator++()"},
    {"_ZN5clang4ento21InterExplodedGraphMapD1Ev", "clang::ento::InterExplodedGraphMap::~InterExplodedGraphMap()"},
    {"_ZN5clang4ento21InterExplodedGraphMapD0Ev", "clang::ento::InterExplodedGraphMap::~InterExplodedGraphMap()"},
    {"_ZN5clang4ento22CreateFlatStoreManagerERNS0_14GRStateManagerE", "clang::ento::CreateFlatStoreManager(clang::ento::GRStateManager&)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManagerD1Ev", "(anonymous namespace)::FlatStoreManager::~FlatStoreManager()"},
    {"_ZN12_GLOBAL__N_116FlatStoreManagerD0Ev", "(anonymous namespace)::FlatStoreManager::~FlatStoreManager()"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager8RetrieveEPKvN5clang4ento3LocENS3_8QualTypeE", "(anonymous namespace)::FlatStoreManager::Retrieve(void const*, clang::ento::Loc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager4BindEPKvN5clang4ento3LocENS4_4SValE", "(anonymous namespace)::FlatStoreManager::Bind(void const*, clang::ento::Loc, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager6RemoveEPKvN5clang4ento3LocE", "(anonymous namespace)::FlatStoreManager::Remove(void const*, clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager19BindCompoundLiteralEPKvPKN5clang19CompoundLiteralExprEPKNS3_15LocationContextENS3_4ento4SValE", "(anonymous namespace)::FlatStoreManager::BindCompoundLiteral(void const*, clang::CompoundLiteralExpr const*, clang::LocationContext const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager15getInitialStoreEPKN5clang15LocationContextE", "(anonymous namespace)::FlatStoreManager::getInitialStore(clang::LocationContext const*)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager15getSubRegionMapEPKv", "(anonymous namespace)::FlatStoreManager::getSubRegionMap(void const*)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager14ArrayToPointerEN5clang4ento3LocE", "(anonymous namespace)::FlatStoreManager::ArrayToPointer(clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager18removeDeadBindingsEPKvPKN5clang17StackFrameContextERNS3_4ento12SymbolReaperERN4llvm15SmallVectorImplIPKNS7_9MemRegionEEE", "(anonymous namespace)::FlatStoreManager::removeDeadBindings(void const*, clang::StackFrameContext const*, clang::ento::SymbolReaper&, llvm::SmallVectorImpl<clang::ento::MemRegion const*>&)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager8BindDeclEPKvPKN5clang4ento9VarRegionENS4_4SValE", "(anonymous namespace)::FlatStoreManager::BindDecl(void const*, clang::ento::VarRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager18BindDeclWithNoInitEPKvPKN5clang4ento9VarRegionE", "(anonymous namespace)::FlatStoreManager::BindDeclWithNoInit(void const*, clang::ento::VarRegion const*)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager17InvalidateRegionsEPKvPKPKN5clang4ento9MemRegionES9_PKNS3_4ExprEjPN4llvm8DenseSetIPKNS4_10SymbolDataENSD_12DenseMapInfoISH_EEEEbPNSD_11SmallVectorIS7_Lj8EEE", "(anonymous namespace)::FlatStoreManager::InvalidateRegions(void const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, clang::Expr const*, unsigned int, llvm::DenseSet<clang::ento::SymbolData const*, llvm::DenseMapInfo<clang::ento::SymbolData const*> >*, bool, llvm::SmallVector<clang::ento::MemRegion const*, 8u>*)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager5printEPKvRN4llvm11raw_ostreamEPKcS7_", "(anonymous namespace)::FlatStoreManager::print(void const*, llvm::raw_ostream&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager12iterBindingsEPKvRN5clang4ento12StoreManager15BindingsHandlerE", "(anonymous namespace)::FlatStoreManager::iterBindings(void const*, clang::ento::StoreManager::BindingsHandler&)"},
    {"_ZN12_GLOBAL__N_116FlatStoreManager16RegionToIntervalEPKN5clang4ento9MemRegionE", "(anonymous namespace)::FlatStoreManager::RegionToInterval(clang::ento::MemRegion const*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS0_20ImmutableIntervalMapINS4_4SValEEEEEEESaISD_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSD_SF_EERKSD_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS4_4SValEEEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE16InsertIntoBucketERKjRKSD_PSt4pairIjSD_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS4_4SValEEEEEEENS_12DenseMapInfoIjEENSE_ISD_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*> >::grow(unsigned int)"},
    {"_ZN4llvm12ImmutableMapIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS2_4SValEEENS_16ImutKeyValueInfoIS5_S8_EEE7Factory3addESB_RKS5_RKS8_", "llvm::ImmutableMap<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal>, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::Factory::add(llvm::ImmutableMap<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal>, llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >, clang::ento::MemRegion const* const&, llvm::ImmutableIntervalMap<clang::ento::SVal> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE3addEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, std::pair<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE16getCanonicalTreeEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeISA_EERNS_26ImutAVLTreeInOrderIteratorISA_EESH_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >&)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::operator++()"},
    {"_ZNK4llvm11ImutAVLTreeINS_16ImutIntervalInfoIN5clang4ento4SValEEEE7isEqualERKS6_", "llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >::isEqual(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> > const&) const"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutIntervalInfoIN5clang4ento4SValEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutIntervalInfo<clang::ento::SVal> >::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE12add_internalERKSt4pairIS6_S9_EPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::add_internal(std::pair<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE13markImmutableEPNS_11ImutAVLTreeISA_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE10createNodeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, std::pair<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEE11balanceTreeEPNS_11ImutAVLTreeISA_EERKSt4pairIS6_S9_ESE_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*, std::pair<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutIntervalInfoIN5clang4ento4SValEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutIntervalInfoIN5clang4ento4SValEEEEESaIS8_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS8_SA_EERKS8_", "std::vector<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >**, std::vector<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*> > >, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >* const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutIntervalInfoIN5clang4ento4SValEEEE13computeDigestEPS6_S7_RKSt4pairINS_8IntervalES4_E", "llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >::computeDigest(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::pair<llvm::Interval, clang::ento::SVal> const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutIntervalInfoIN5clang4ento4SValEEEEENS_12DenseMapInfoIjEENS9_IS8_EEE16InsertIntoBucketERKjRKS8_PSt4pairIjS8_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutIntervalInfoIN5clang4ento4SValEEEEENS_12DenseMapInfoIjEENS9_IS8_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE3addEPNS_11ImutAVLTreeIS5_EERKSt4pairINS_8IntervalES4_E", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::add(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::pair<llvm::Interval, clang::ento::SVal> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS5_EE", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS5_EERNS_26ImutAVLTreeInOrderIteratorIS5_EESC_", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutIntervalInfo<clang::ento::SVal> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutIntervalInfo<clang::ento::SVal> >&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE12add_internalERKSt4pairINS_8IntervalES4_EPNS_11ImutAVLTreeIS5_EE", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::add_internal(std::pair<llvm::Interval, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE13markImmutableEPNS_11ImutAVLTreeIS5_EE", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::markImmutable(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE10createNodeEPNS_11ImutAVLTreeIS5_EERKSt4pairINS_8IntervalES4_ES9_", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::createNode(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::pair<llvm::Interval, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEE11balanceTreeEPNS_11ImutAVLTreeIS5_EERKSt4pairINS_8IntervalES4_ES9_", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::balanceTree(llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*, std::pair<llvm::Interval, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutIntervalInfo<clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento9MemRegionENS_20ImmutableIntervalMapINS3_4SValEEEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::MemRegion const*, llvm::ImmutableIntervalMap<clang::ento::SVal> > >::~ImutAVLFactory()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutIntervalInfoIN5clang4ento4SValEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutIntervalInfo<clang::ento::SVal> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento17ConstraintManagerD0Ev", "clang::ento::ConstraintManager::~ConstraintManager()"},
    {"_ZN5clang4ento17ConstraintManagerD1Ev", "clang::ento::ConstraintManager::~ConstraintManager()"},
    {"_ZN5clang4ento17ConstraintManagerD2Ev", "clang::ento::ConstraintManager::~ConstraintManager()"},
    {"_ZN5clang4ento14GRStateManagerD1Ev", "clang::ento::GRStateManager::~GRStateManager()"},
    {"_ZN5clang4ento14GRStateManagerD2Ev", "clang::ento::GRStateManager::~GRStateManager()"},
    {"_ZN5clang4ento14GRStateManager18removeDeadBindingsEPKNS0_7GRStateEPKNS_17StackFrameContextERNS0_12SymbolReaperE", "clang::ento::GRStateManager::removeDeadBindings(clang::ento::GRState const*, clang::StackFrameContext const*, clang::ento::SymbolReaper&)"},
    {"_ZN5clang4ento14GRStateManager18getPersistentStateERNS0_7GRStateE", "clang::ento::GRStateManager::getPersistentState(clang::ento::GRState&)"},
    {"_ZN5clang4ento14GRStateManager12MarshalStateEPKNS0_7GRStateEPKNS_17StackFrameContextE", "clang::ento::GRStateManager::MarshalState(clang::ento::GRState const*, clang::StackFrameContext const*)"},
    {"_ZNK5clang4ento7GRState19bindCompoundLiteralEPKNS_19CompoundLiteralExprEPKNS_15LocationContextENS0_4SValE", "clang::ento::GRState::bindCompoundLiteral(clang::CompoundLiteralExpr const*, clang::LocationContext const*, clang::ento::SVal) const"},
    {"_ZNK5clang4ento7GRState13makeWithStoreEPKv", "clang::ento::GRState::makeWithStore(void const*) const"},
    {"_ZNK5clang4ento7GRState8bindDeclEPKNS0_9VarRegionENS0_4SValE", "clang::ento::GRState::bindDecl(clang::ento::VarRegion const*, clang::ento::SVal) const"},
    {"_ZNK5clang4ento7GRState18bindDeclWithNoInitEPKNS0_9VarRegionE", "clang::ento::GRState::bindDeclWithNoInit(clang::ento::VarRegion const*) const"},
    {"_ZNK5clang4ento7GRState7bindLocENS0_3LocENS0_4SValE", "clang::ento::GRState::bindLoc(clang::ento::Loc, clang::ento::SVal) const"},
    {"_ZNK5clang4ento7GRState11bindDefaultENS0_4SValES2_", "clang::ento::GRState::bindDefault(clang::ento::SVal, clang::ento::SVal) const"},
    {"_ZNK5clang4ento7GRState17InvalidateRegionsEPKPKNS0_9MemRegionES6_PKNS_4ExprEjPN4llvm8DenseSetIPKNS0_10SymbolDataENSA_12DenseMapInfoISE_EEEEb", "clang::ento::GRState::InvalidateRegions(clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, clang::Expr const*, unsigned int, llvm::DenseSet<clang::ento::SymbolData const*, llvm::DenseMapInfo<clang::ento::SymbolData const*> >*, bool) const"},
    {"_ZNK5clang4ento7GRState9unbindLocENS0_3LocE", "clang::ento::GRState::unbindLoc(clang::ento::Loc) const"},
    {"_ZNK5clang4ento7GRState15enterStackFrameEPKNS_17StackFrameContextE", "clang::ento::GRState::enterStackFrame(clang::StackFrameContext const*) const"},
    {"_ZNK5clang4ento7GRState20getSValAsScalarOrLocEPKNS0_9MemRegionE", "clang::ento::GRState::getSValAsScalarOrLoc(clang::ento::MemRegion const*) const"},
    {"_ZNK5clang4ento7GRState7getSValENS0_3LocENS_8QualTypeE", "clang::ento::GRState::getSVal(clang::ento::Loc, clang::QualType) const"},
    {"_ZNK5clang4ento7GRState8BindExprEPKNS_4StmtENS0_4SValEb", "clang::ento::GRState::BindExpr(clang::Stmt const*, clang::ento::SVal, bool) const"},
    {"_ZNK5clang4ento7GRState19bindExprAndLocationEPKNS_4StmtENS0_4SValES5_", "clang::ento::GRState::bindExprAndLocation(clang::Stmt const*, clang::ento::SVal, clang::ento::SVal) const"},
    {"_ZNK5clang4ento7GRState13assumeInBoundENS0_20DefinedOrUnknownSValES2_b", "clang::ento::GRState::assumeInBound(clang::ento::DefinedOrUnknownSVal, clang::ento::DefinedOrUnknownSVal, bool) const"},
    {"_ZN5clang4ento14GRStateManager15getInitialStateEPKNS_15LocationContextE", "clang::ento::GRStateManager::getInitialState(clang::LocationContext const*)"},
    {"_ZN5clang4ento14GRStateManager19recycleUnusedStatesEv", "clang::ento::GRStateManager::recycleUnusedStates()"},
    {"_ZNK5clang4ento7GRState5printERN4llvm11raw_ostreamERNS_3CFGEPKcS8_", "clang::ento::GRState::print(llvm::raw_ostream&, clang::CFG&, char const*, char const*) const"},
    {"_ZNK5clang4ento7GRState8printDOTERN4llvm11raw_ostreamERNS_3CFGE", "clang::ento::GRState::printDOT(llvm::raw_ostream&, clang::CFG&) const"},
    {"_ZNK5clang4ento7GRState11printStdErrERNS_3CFGE", "clang::ento::GRState::printStdErr(clang::CFG&) const"},
    {"_ZNK5clang4ento7GRState7FindGDMEPv", "clang::ento::GRState::FindGDM(void*) const"},
    {"_ZN5clang4ento14GRStateManager14FindGDMContextEPvPFS2_RN4llvm16BumpPtrAllocatorEEPFvS2_E", "clang::ento::GRStateManager::FindGDMContext(void*, void* (*)(llvm::BumpPtrAllocator&), void (*)(void*))"},
    {"_ZN5clang4ento14GRStateManager6addGDMEPKNS0_7GRStateEPvS5_", "clang::ento::GRStateManager::addGDM(clang::ento::GRState const*, void*, void*)"},
    {"_ZN5clang4ento14GRStateManager9removeGDMEPKNS0_7GRStateEPv", "clang::ento::GRStateManager::removeGDM(clang::ento::GRState const*, void*)"},
    {"_ZNK5clang4ento7GRState20scanReachableSymbolsENS0_4SValERNS0_13SymbolVisitorE", "clang::ento::GRState::scanReachableSymbols(clang::ento::SVal, clang::ento::SymbolVisitor&) const"},
    {"_ZN12_GLOBAL__N_120ScanReachableSymbols4scanEN5clang4ento4SValE", "(anonymous namespace)::ScanReachableSymbols::scan(clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_120ScanReachableSymbolsD1Ev", "(anonymous namespace)::ScanReachableSymbols::~ScanReachableSymbols()"},
    {"_ZNK5clang4ento7GRState20scanReachableSymbolsEPKNS0_4SValES4_RNS0_13SymbolVisitorE", "clang::ento::GRState::scanReachableSymbols(clang::ento::SVal const*, clang::ento::SVal const*, clang::ento::SymbolVisitor&) const"},
    {"_ZNK5clang4ento7GRState20scanReachableSymbolsEPKPKNS0_9MemRegionES6_RNS0_13SymbolVisitorE", "clang::ento::GRState::scanReachableSymbols(clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, clang::ento::SymbolVisitor&) const"},
    {"_ZN12_GLOBAL__N_120ScanReachableSymbols4scanEPKN5clang4ento9MemRegionE", "(anonymous namespace)::ScanReachableSymbols::scan(clang::ento::MemRegion const*)"},
    {"_ZN12_GLOBAL__N_120ScanReachableSymbolsD0Ev", "(anonymous namespace)::ScanReachableSymbols::~ScanReachableSymbols()"},
    {"_ZN12_GLOBAL__N_120ScanReachableSymbols5VisitEPKN5clang4ento9MemRegionES5_", "(anonymous namespace)::ScanReachableSymbols::Visit(clang::ento::MemRegion const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento17BasicValueFactory7ConvertENS_8QualTypeERKN4llvm6APSIntE", "clang::ento::BasicValueFactory::Convert(clang::QualType, llvm::APSInt const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE6removeEPNS_11ImutAVLTreeIS3_EERKS2_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::remove(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, void* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE16getCanonicalTreeEPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPvS2_EEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >::computeDigest()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS3_EERNS_26ImutAVLTreeInOrderIteratorIS3_EESA_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<void*, void*> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<void*, void*> >&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPvS2_EEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPvS3_EEEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPvS3_EEEENS_12DenseMapInfoIjEENS7_IS6_EEE16InsertIntoBucketERKjRKS6_PSt4pairIjS6_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPvS3_EEEENS_12DenseMapInfoIjEENS7_IS6_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE15remove_internalERKS2_PNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::remove_internal(void* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE13markImmutableEPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE11balanceTreeEPNS_11ImutAVLTreeIS3_EERKSt4pairIS2_S2_ES7_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, std::pair<void*, void*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE10createNodeEPNS_11ImutAVLTreeIS3_EERKSt4pairIS2_S2_ES7_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, std::pair<void*, void*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE16removeMinBindingEPNS_11ImutAVLTreeIS3_EERS7_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE3addEPNS_11ImutAVLTreeIS3_EERKSt4pairIS2_S2_E", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::add(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*, std::pair<void*, void*> const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEE12add_internalERKSt4pairIS2_S2_EPNS_11ImutAVLTreeIS3_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::add_internal(std::pair<void*, void*> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<void*, void*> >*)"},
    {"_ZN4llvm8DenseMapIPvSt4pairIS1_PFvS1_EENS_12DenseMapInfoIS1_EENS6_IS5_EEE16InsertIntoBucketERKS1_RKS5_PS2_IS1_S5_E", "llvm::DenseMap<void*, std::pair<void*, void (*)(void*)>, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<std::pair<void*, void (*)(void*)> > >::InsertIntoBucket(void* const&, std::pair<void*, void (*)(void*)> const&, std::pair<void*, std::pair<void*, void (*)(void*)> >*)"},
    {"_ZN4llvm8DenseMapIPvSt4pairIS1_PFvS1_EENS_12DenseMapInfoIS1_EENS6_IS5_EEE4growEj", "llvm::DenseMap<void*, std::pair<void*, void (*)(void*)>, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<std::pair<void*, void (*)(void*)> > >::grow(unsigned int)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::operator++()"},
    {"_ZNSt6vectorIPN5clang4ento7GRStateESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::ento::GRState*, std::allocator<clang::ento::GRState*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::GRState**, std::vector<clang::ento::GRState*, std::allocator<clang::ento::GRState*> > >, clang::ento::GRState* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionEcNS_12DenseMapInfoIS5_EENS6_IcEEE6insertERKSt4pairIS5_cE", "llvm::DenseMap<clang::ento::MemRegion const*, char, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<char> >::insert(std::pair<clang::ento::MemRegion const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionEcNS_12DenseMapInfoIS5_EENS6_IcEEE16InsertIntoBucketERKS5_RKcPSt4pairIS5_cE", "llvm::DenseMap<clang::ento::MemRegion const*, char, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(clang::ento::MemRegion const* const&, char const&, std::pair<clang::ento::MemRegion const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionEcNS_12DenseMapInfoIS5_EENS6_IcEEE4growEj", "llvm::DenseMap<clang::ento::MemRegion const*, char, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang4ento12SubRegionMap7VisitorD1Ev", "clang::ento::SubRegionMap::Visitor::~Visitor()"},
    {"_ZN5clang4ento12SubRegionMap7VisitorD0Ev", "clang::ento::SubRegionMap::Visitor::~Visitor()"},
    {"_ZNK4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE7isEqualERKS9_", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::isEqual(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> > const&) const"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::destroy()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4StmtENS3_4ento4SValEEEEESaISB_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSB_SD_EERKSB_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >**, std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*> > >, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >* const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEE13computeDigestEPS9_SA_RKSt4pairIS5_S7_E", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::computeDigest(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, std::pair<clang::Stmt const*, clang::ento::SVal> const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4StmtENS3_4ento4SValEEEEENS_12DenseMapInfoIjEENSC_ISB_EEE16InsertIntoBucketERKjRKSB_PSt4pairIjSB_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4StmtENS3_4ento4SValEEEEENS_12DenseMapInfoIjEENSC_ISB_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4StmtENS2_4ento4SValEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::Stmt const*, clang::ento::SVal> >::~ImutAVLFactory()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPvS2_EEED2Ev", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<void*, void*> >::~ImutAVLFactory()"},
    {"_ZN5clang4ento26createHTMLDiagnosticClientERKSsRKNS_12PreprocessorE", "clang::ento::createHTMLDiagnosticClient(std::string const&, clang::Preprocessor const&)"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnosticsD1Ev", "(anonymous namespace)::HTMLDiagnostics::~HTMLDiagnostics()"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnosticsD0Ev", "(anonymous namespace)::HTMLDiagnostics::~HTMLDiagnostics()"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnostics16FlushDiagnosticsEPN4llvm15SmallVectorImplISsEE", "(anonymous namespace)::HTMLDiagnostics::FlushDiagnostics(llvm::SmallVectorImpl<std::string>*)"},
    {"_ZNK12_GLOBAL__N_115HTMLDiagnostics7getNameEv", "(anonymous namespace)::HTMLDiagnostics::getName() const"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnostics20HandlePathDiagnosticEPKN5clang4ento14PathDiagnosticE", "(anonymous namespace)::HTMLDiagnostics::HandlePathDiagnostic(clang::ento::PathDiagnostic const*)"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnostics10ReportDiagERKN5clang4ento14PathDiagnosticEPN4llvm15SmallVectorImplISsEE", "(anonymous namespace)::HTMLDiagnostics::ReportDiag(clang::ento::PathDiagnostic const&, llvm::SmallVectorImpl<std::string>*)"},
    {"_ZN12_GLOBAL__N_115HTMLDiagnostics17ProcessMacroPieceERN4llvm11raw_ostreamERKN5clang4ento24PathDiagnosticMacroPieceEj", "(anonymous namespace)::HTMLDiagnostics::ProcessMacroPiece(llvm::raw_ostream&, clang::ento::PathDiagnosticMacroPiece const&, unsigned int)"},
    {"_Z16EmitAlphaCounterRN4llvm11raw_ostreamEj", "EmitAlphaCounter(llvm::raw_ostream&, unsigned int)"},
    {"_ZN5clang4ento9MemRegionD0Ev", "clang::ento::MemRegion::~MemRegion()"},
    {"_ZN5clang4ento9MemRegionD1Ev", "clang::ento::MemRegion::~MemRegion()"},
    {"_ZN5clang4ento9MemRegionD2Ev", "clang::ento::MemRegion::~MemRegion()"},
    {"_ZN5clang4ento16MemRegionManagerD1Ev", "clang::ento::MemRegionManager::~MemRegionManager()"},
    {"_ZN5clang4ento16MemRegionManagerD2Ev", "clang::ento::MemRegionManager::~MemRegionManager()"},
    {"_ZNK5clang4ento9SubRegion13isSubRegionOfEPKNS0_9MemRegionE", "clang::ento::SubRegion::isSubRegionOf(clang::ento::MemRegion const*) const"},
    {"_ZNK5clang4ento9SubRegion19getMemRegionManagerEv", "clang::ento::SubRegion::getMemRegionManager() const"},
    {"_ZNK5clang4ento9VarRegion13getStackFrameEv", "clang::ento::VarRegion::getStackFrame() const"},
    {"_ZNK5clang4ento9MemRegion14getMemorySpaceEv", "clang::ento::MemRegion::getMemorySpace() const"},
    {"_ZNK5clang4ento10DeclRegion9getExtentERNS0_11SValBuilderE", "clang::ento::DeclRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento11FieldRegion9getExtentERNS0_11SValBuilderE", "clang::ento::FieldRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento12AllocaRegion9getExtentERNS0_11SValBuilderE", "clang::ento::AllocaRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento14SymbolicRegion9getExtentERNS0_11SValBuilderE", "clang::ento::SymbolicRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento12StringRegion9getExtentERNS0_11SValBuilderE", "clang::ento::StringRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento19CXXBaseObjectRegion12getValueTypeEv", "clang::ento::CXXBaseObjectRegion::getValueType() const"},
    {"_ZNK5clang4ento14MemSpaceRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::MemSpaceRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento16StackSpaceRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::StackSpaceRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento23StaticGlobalSpaceRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::StaticGlobalSpaceRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento12StringRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_13StringLiteralEPKNS0_9MemRegionE", "clang::ento::StringRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::StringLiteral const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento12AllocaRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_4ExprEjPKNS0_9MemRegionE", "clang::ento::AllocaRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::Expr const*, unsigned int, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento12AllocaRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::AllocaRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento21CompoundLiteralRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::CompoundLiteralRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento21CompoundLiteralRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_19CompoundLiteralExprEPKNS0_9MemRegionE", "clang::ento::CompoundLiteralRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::CompoundLiteralExpr const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento13CXXThisRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_11PointerTypeEPKNS0_9MemRegionE", "clang::ento::CXXThisRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::PointerType const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento13CXXThisRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::CXXThisRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento10DeclRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_4DeclEPKNS0_9MemRegionENS8_4KindE", "clang::ento::DeclRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::Decl const*, clang::ento::MemRegion const*, clang::ento::MemRegion::Kind)"},
    {"_ZNK5clang4ento10DeclRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::DeclRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento9VarRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::VarRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento14SymbolicRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS0_10SymbolDataEPKNS0_9MemRegionE", "clang::ento::SymbolicRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::ento::SymbolData const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento14SymbolicRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolicRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento13ElementRegion13ProfileRegionERN4llvm16FoldingSetNodeIDENS_8QualTypeENS0_4SValEPKNS0_9MemRegionE", "clang::ento::ElementRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::QualType, clang::ento::SVal, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento13ElementRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::ElementRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento18FunctionTextRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_12FunctionDeclEPKNS0_9MemRegionE", "clang::ento::FunctionTextRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::FunctionDecl const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento18FunctionTextRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::FunctionTextRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento15BlockTextRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_9BlockDeclENS_7CanQualINS_4TypeEEEPKNS_15AnalysisContextEPKNS0_9MemRegionE", "clang::ento::BlockTextRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::BlockDecl const*, clang::CanQual<clang::Type>, clang::AnalysisContext const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento15BlockTextRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::BlockTextRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento15BlockDataRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS0_15BlockTextRegionEPKNS_15LocationContextEPKNS0_9MemRegionE", "clang::ento::BlockDataRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::ento::BlockTextRegion const*, clang::LocationContext const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento15BlockDataRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::BlockDataRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento19CXXTempObjectRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_4ExprEPKNS0_9MemRegionE", "clang::ento::CXXTempObjectRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::Expr const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento19CXXTempObjectRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::CXXTempObjectRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento19CXXBaseObjectRegion13ProfileRegionERN4llvm16FoldingSetNodeIDEPKNS_13CXXRecordDeclEPKNS0_9MemRegionE", "clang::ento::CXXBaseObjectRegion::ProfileRegion(llvm::FoldingSetNodeID&, clang::CXXRecordDecl const*, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento19CXXBaseObjectRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::CXXBaseObjectRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento9MemRegion4dumpEv", "clang::ento::MemRegion::dump() const"},
    {"_ZNK5clang4ento9MemRegion9getStringEv", "clang::ento::MemRegion::getString() const"},
    {"_ZNK5clang4ento9MemRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::MemRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento12AllocaRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::AllocaRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento18FunctionTextRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::FunctionTextRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento15BlockTextRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::BlockTextRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento15BlockDataRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::BlockDataRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento21CompoundLiteralRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::CompoundLiteralRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento19CXXTempObjectRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::CXXTempObjectRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento19CXXBaseObjectRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::CXXBaseObjectRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento13CXXThisRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::CXXThisRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento13ElementRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::ElementRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento11FieldRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::FieldRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento26NonStaticGlobalSpaceRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::NonStaticGlobalSpaceRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento14ObjCIvarRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::ObjCIvarRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento12StringRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::StringRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento14SymbolicRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolicRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento9VarRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::VarRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento15RegionRawOffset4dumpEv", "clang::ento::RegionRawOffset::dump() const"},
    {"_ZNK5clang4ento15RegionRawOffset12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::RegionRawOffset::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento23StaticGlobalSpaceRegion12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::StaticGlobalSpaceRegion::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZN5clang4ento16MemRegionManager20getStackLocalsRegionEPKNS_17StackFrameContextE", "clang::ento::MemRegionManager::getStackLocalsRegion(clang::StackFrameContext const*)"},
    {"_ZN5clang4ento16MemRegionManager23getStackArgumentsRegionEPKNS_17StackFrameContextE", "clang::ento::MemRegionManager::getStackArgumentsRegion(clang::StackFrameContext const*)"},
    {"_ZN5clang4ento16MemRegionManager16getGlobalsRegionEPKNS0_14CodeTextRegionE", "clang::ento::MemRegionManager::getGlobalsRegion(clang::ento::CodeTextRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager13getHeapRegionEv", "clang::ento::MemRegionManager::getHeapRegion()"},
    {"_ZN5clang4ento16MemRegionManager16getUnknownRegionEv", "clang::ento::MemRegionManager::getUnknownRegion()"},
    {"_ZN5clang4ento16MemRegionManager13getCodeRegionEv", "clang::ento::MemRegionManager::getCodeRegion()"},
    {"_ZN5clang4ento16MemRegionManager15getStringRegionEPKNS_13StringLiteralE", "clang::ento::MemRegionManager::getStringRegion(clang::StringLiteral const*)"},
    {"_ZN5clang4ento16MemRegionManager12getVarRegionEPKNS_7VarDeclEPKNS_15LocationContextE", "clang::ento::MemRegionManager::getVarRegion(clang::VarDecl const*, clang::LocationContext const*)"},
    {"_ZN5clang4ento16MemRegionManager21getFunctionTextRegionEPKNS_12FunctionDeclE", "clang::ento::MemRegionManager::getFunctionTextRegion(clang::FunctionDecl const*)"},
    {"_ZN5clang4ento16MemRegionManager18getBlockTextRegionEPKNS_9BlockDeclENS_7CanQualINS_4TypeEEEPNS_15AnalysisContextE", "clang::ento::MemRegionManager::getBlockTextRegion(clang::BlockDecl const*, clang::CanQual<clang::Type>, clang::AnalysisContext*)"},
    {"_ZN5clang4ento16MemRegionManager12getVarRegionEPKNS_7VarDeclEPKNS0_9MemRegionE", "clang::ento::MemRegionManager::getVarRegion(clang::VarDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager18getBlockDataRegionEPKNS0_15BlockTextRegionEPKNS_15LocationContextE", "clang::ento::MemRegionManager::getBlockDataRegion(clang::ento::BlockTextRegion const*, clang::LocationContext const*)"},
    {"_ZN5clang4ento16MemRegionManager24getCompoundLiteralRegionEPKNS_19CompoundLiteralExprEPKNS_15LocationContextE", "clang::ento::MemRegionManager::getCompoundLiteralRegion(clang::CompoundLiteralExpr const*, clang::LocationContext const*)"},
    {"_ZN5clang4ento16MemRegionManager16getElementRegionENS_8QualTypeENS0_6NonLocEPKNS0_9MemRegionERNS_10ASTContextE", "clang::ento::MemRegionManager::getElementRegion(clang::QualType, clang::ento::NonLoc, clang::ento::MemRegion const*, clang::ASTContext&)"},
    {"_ZN5clang4ento16MemRegionManager17getSymbolicRegionEPKNS0_10SymbolDataE", "clang::ento::MemRegionManager::getSymbolicRegion(clang::ento::SymbolData const*)"},
    {"_ZN5clang4ento16MemRegionManager14getFieldRegionEPKNS_9FieldDeclEPKNS0_9MemRegionE", "clang::ento::MemRegionManager::getFieldRegion(clang::FieldDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager17getObjCIvarRegionEPKNS_12ObjCIvarDeclEPKNS0_9MemRegionE", "clang::ento::MemRegionManager::getObjCIvarRegion(clang::ObjCIvarDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager22getCXXTempObjectRegionEPKNS_4ExprEPKNS_15LocationContextE", "clang::ento::MemRegionManager::getCXXTempObjectRegion(clang::Expr const*, clang::LocationContext const*)"},
    {"_ZN5clang4ento16MemRegionManager22getCXXBaseObjectRegionEPKNS_13CXXRecordDeclEPKNS0_9MemRegionE", "clang::ento::MemRegionManager::getCXXBaseObjectRegion(clang::CXXRecordDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager16getCXXThisRegionENS_8QualTypeEPKNS_15LocationContextE", "clang::ento::MemRegionManager::getCXXThisRegion(clang::QualType, clang::LocationContext const*)"},
    {"_ZN5clang4ento16MemRegionManager15getAllocaRegionEPKNS_4ExprEjPKNS_15LocationContextE", "clang::ento::MemRegionManager::getAllocaRegion(clang::Expr const*, unsigned int, clang::LocationContext const*)"},
    {"_ZNK5clang4ento9MemRegion15hasStackStorageEv", "clang::ento::MemRegion::hasStackStorage() const"},
    {"_ZNK5clang4ento9MemRegion28hasStackNonParametersStorageEv", "clang::ento::MemRegion::hasStackNonParametersStorage() const"},
    {"_ZNK5clang4ento9MemRegion25hasStackParametersStorageEv", "clang::ento::MemRegion::hasStackParametersStorage() const"},
    {"_ZNK5clang4ento9MemRegion29hasGlobalsOrParametersStorageEv", "clang::ento::MemRegion::hasGlobalsOrParametersStorage() const"},
    {"_ZNK5clang4ento9MemRegion13getBaseRegionEv", "clang::ento::MemRegion::getBaseRegion() const"},
    {"_ZNK5clang4ento9MemRegion10StripCastsEv", "clang::ento::MemRegion::StripCasts() const"},
    {"_ZNK5clang4ento13ElementRegion16getAsArrayOffsetEv", "clang::ento::ElementRegion::getAsArrayOffset() const"},
    {"_ZNK5clang4ento9MemRegion11getAsOffsetEv", "clang::ento::MemRegion::getAsOffset() const"},
    {"_ZN5clang4ento15BlockDataRegion28LazyInitializeReferencedVarsEv", "clang::ento::BlockDataRegion::LazyInitializeReferencedVars()"},
    {"_ZNK5clang4ento15BlockDataRegion21referenced_vars_beginEv", "clang::ento::BlockDataRegion::referenced_vars_begin() const"},
    {"_ZNK5clang4ento15BlockDataRegion19referenced_vars_endEv", "clang::ento::BlockDataRegion::referenced_vars_end() const"},
    {"_ZN4llvm10FoldingSetIN5clang4ento9MemRegionEED1Ev", "llvm::FoldingSet<clang::ento::MemRegion>::~FoldingSet()"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_12StringRegionEPKNS_13StringLiteralEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::StringRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::StringRegion, clang::StringLiteral const*>(clang::StringLiteral const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_9VarRegionEPKNS_7VarDeclEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::VarRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::VarRegion, clang::VarDecl const*>(clang::VarDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_15BlockDataRegionEPKNS0_15BlockTextRegionEPKNS_15LocationContextEEEPT_T0_T1_PKNS0_9MemRegionE", "clang::ento::BlockDataRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::BlockDataRegion, clang::ento::BlockTextRegion const*, clang::LocationContext const*>(clang::ento::BlockTextRegion const*, clang::LocationContext const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_21CompoundLiteralRegionEPKNS_19CompoundLiteralExprEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::CompoundLiteralRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::CompoundLiteralRegion, clang::CompoundLiteralExpr const*>(clang::CompoundLiteralExpr const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_18FunctionTextRegionEPKNS_12FunctionDeclEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::FunctionTextRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::FunctionTextRegion, clang::FunctionDecl const*>(clang::FunctionDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_15BlockTextRegionEPKNS_9BlockDeclENS_7CanQualINS_4TypeEEEPNS_15AnalysisContextEEEPT_T0_T1_T2_PKNS0_9MemRegionE", "clang::ento::BlockTextRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::BlockTextRegion, clang::BlockDecl const*, clang::CanQual<clang::Type>, clang::AnalysisContext*>(clang::BlockDecl const*, clang::CanQual<clang::Type>, clang::AnalysisContext*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_14SymbolicRegionEPKNS0_10SymbolDataEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::SymbolicRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::SymbolicRegion, clang::ento::SymbolData const*>(clang::ento::SymbolData const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_11FieldRegionEPKNS_9FieldDeclEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::FieldRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::FieldRegion, clang::FieldDecl const*>(clang::FieldDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_14ObjCIvarRegionEPKNS_12ObjCIvarDeclEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::ObjCIvarRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::ObjCIvarRegion, clang::ObjCIvarDecl const*>(clang::ObjCIvarDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_19CXXTempObjectRegionEPKNS_4ExprEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::CXXTempObjectRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::CXXTempObjectRegion, clang::Expr const*>(clang::Expr const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_19CXXBaseObjectRegionEPKNS_13CXXRecordDeclEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::CXXBaseObjectRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::CXXBaseObjectRegion, clang::CXXRecordDecl const*>(clang::CXXRecordDecl const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_13CXXThisRegionEPKNS_11PointerTypeEEEPT_T0_PKNS0_9MemRegionE", "clang::ento::CXXThisRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::CXXThisRegion, clang::PointerType const*>(clang::PointerType const*, clang::ento::MemRegion const*)"},
    {"_ZN5clang4ento16MemRegionManager12getSubRegionINS0_12AllocaRegionEPKNS_4ExprEjEEPT_T0_T1_PKNS0_9MemRegionE", "clang::ento::AllocaRegion* clang::ento::MemRegionManager::getSubRegion<clang::ento::AllocaRegion, clang::Expr const*, unsigned int>(clang::Expr const*, unsigned int, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento9MemRegion11isBoundableEv", "clang::ento::MemRegion::isBoundable() const"},
    {"_ZN5clang4ento14MemSpaceRegionD1Ev", "clang::ento::MemSpaceRegion::~MemSpaceRegion()"},
    {"_ZN5clang4ento14MemSpaceRegionD0Ev", "clang::ento::MemSpaceRegion::~MemSpaceRegion()"},
    {"_ZNK5clang4ento14MemSpaceRegion19getMemRegionManagerEv", "clang::ento::MemSpaceRegion::getMemRegionManager() const"},
    {"_ZNK5clang4ento14MemSpaceRegion11isBoundableEv", "clang::ento::MemSpaceRegion::isBoundable() const"},
    {"_ZN5clang4ento23StaticGlobalSpaceRegionD1Ev", "clang::ento::StaticGlobalSpaceRegion::~StaticGlobalSpaceRegion()"},
    {"_ZN5clang4ento23StaticGlobalSpaceRegionD0Ev", "clang::ento::StaticGlobalSpaceRegion::~StaticGlobalSpaceRegion()"},
    {"_ZN5clang4ento26NonStaticGlobalSpaceRegionD1Ev", "clang::ento::NonStaticGlobalSpaceRegion::~NonStaticGlobalSpaceRegion()"},
    {"_ZN5clang4ento26NonStaticGlobalSpaceRegionD0Ev", "clang::ento::NonStaticGlobalSpaceRegion::~NonStaticGlobalSpaceRegion()"},
    {"_ZN5clang4ento16StackSpaceRegionD1Ev", "clang::ento::StackSpaceRegion::~StackSpaceRegion()"},
    {"_ZN5clang4ento16StackSpaceRegionD0Ev", "clang::ento::StackSpaceRegion::~StackSpaceRegion()"},
    {"_ZN5clang4ento9SubRegionD1Ev", "clang::ento::SubRegion::~SubRegion()"},
    {"_ZN5clang4ento9SubRegionD0Ev", "clang::ento::SubRegion::~SubRegion()"},
    {"_ZNK5clang4ento9SubRegion9getExtentERNS0_11SValBuilderE", "clang::ento::SubRegion::getExtent(clang::ento::SValBuilder&) const"},
    {"_ZN5clang4ento12AllocaRegionD1Ev", "clang::ento::AllocaRegion::~AllocaRegion()"},
    {"_ZN5clang4ento12AllocaRegionD0Ev", "clang::ento::AllocaRegion::~AllocaRegion()"},
    {"_ZNK5clang4ento12AllocaRegion11isBoundableEv", "clang::ento::AllocaRegion::isBoundable() const"},
    {"_ZN5clang4ento18FunctionTextRegionD1Ev", "clang::ento::FunctionTextRegion::~FunctionTextRegion()"},
    {"_ZN5clang4ento18FunctionTextRegionD0Ev", "clang::ento::FunctionTextRegion::~FunctionTextRegion()"},
    {"_ZNK5clang4ento14CodeTextRegion11isBoundableEv", "clang::ento::CodeTextRegion::isBoundable() const"},
    {"_ZNK5clang4ento14CodeTextRegion12getValueTypeEv", "clang::ento::CodeTextRegion::getValueType() const"},
    {"_ZNK5clang4ento18FunctionTextRegion15getLocationTypeEv", "clang::ento::FunctionTextRegion::getLocationType() const"},
    {"_ZN5clang4ento15BlockTextRegionD1Ev", "clang::ento::BlockTextRegion::~BlockTextRegion()"},
    {"_ZN5clang4ento15BlockTextRegionD0Ev", "clang::ento::BlockTextRegion::~BlockTextRegion()"},
    {"_ZNK5clang4ento15BlockTextRegion15getLocationTypeEv", "clang::ento::BlockTextRegion::getLocationType() const"},
    {"_ZN5clang4ento15BlockDataRegionD1Ev", "clang::ento::BlockDataRegion::~BlockDataRegion()"},
    {"_ZN5clang4ento15BlockDataRegionD0Ev", "clang::ento::BlockDataRegion::~BlockDataRegion()"},
    {"_ZN5clang4ento14SymbolicRegionD1Ev", "clang::ento::SymbolicRegion::~SymbolicRegion()"},
    {"_ZN5clang4ento14SymbolicRegionD0Ev", "clang::ento::SymbolicRegion::~SymbolicRegion()"},
    {"_ZNK5clang4ento14SymbolicRegion11isBoundableEv", "clang::ento::SymbolicRegion::isBoundable() const"},
    {"_ZN5clang4ento12StringRegionD1Ev", "clang::ento::StringRegion::~StringRegion()"},
    {"_ZN5clang4ento12StringRegionD0Ev", "clang::ento::StringRegion::~StringRegion()"},
    {"_ZNK5clang4ento12StringRegion7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::StringRegion::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento12StringRegion11isBoundableEv", "clang::ento::StringRegion::isBoundable() const"},
    {"_ZNK5clang4ento12StringRegion12getValueTypeEv", "clang::ento::StringRegion::getValueType() const"},
    {"_ZNK5clang4ento11TypedRegion15getLocationTypeEv", "clang::ento::TypedRegion::getLocationType() const"},
    {"_ZN5clang4ento21CompoundLiteralRegionD1Ev", "clang::ento::CompoundLiteralRegion::~CompoundLiteralRegion()"},
    {"_ZN5clang4ento21CompoundLiteralRegionD0Ev", "clang::ento::CompoundLiteralRegion::~CompoundLiteralRegion()"},
    {"_ZNK5clang4ento21CompoundLiteralRegion11isBoundableEv", "clang::ento::CompoundLiteralRegion::isBoundable() const"},
    {"_ZNK5clang4ento21CompoundLiteralRegion12getValueTypeEv", "clang::ento::CompoundLiteralRegion::getValueType() const"},
    {"_ZN5clang4ento10DeclRegionD1Ev", "clang::ento::DeclRegion::~DeclRegion()"},
    {"_ZN5clang4ento10DeclRegionD0Ev", "clang::ento::DeclRegion::~DeclRegion()"},
    {"_ZNK5clang4ento11TypedRegion11isBoundableEv", "clang::ento::TypedRegion::isBoundable() const"},
    {"_ZN5clang4ento9VarRegionD1Ev", "clang::ento::VarRegion::~VarRegion()"},
    {"_ZN5clang4ento9VarRegionD0Ev", "clang::ento::VarRegion::~VarRegion()"},
    {"_ZNK5clang4ento9VarRegion12getValueTypeEv", "clang::ento::VarRegion::getValueType() const"},
    {"_ZN5clang4ento13CXXThisRegionD1Ev", "clang::ento::CXXThisRegion::~CXXThisRegion()"},
    {"_ZN5clang4ento13CXXThisRegionD0Ev", "clang::ento::CXXThisRegion::~CXXThisRegion()"},
    {"_ZNK5clang4ento13CXXThisRegion12getValueTypeEv", "clang::ento::CXXThisRegion::getValueType() const"},
    {"_ZN5clang4ento11FieldRegionD1Ev", "clang::ento::FieldRegion::~FieldRegion()"},
    {"_ZN5clang4ento11FieldRegionD0Ev", "clang::ento::FieldRegion::~FieldRegion()"},
    {"_ZNK5clang4ento11FieldRegion12getValueTypeEv", "clang::ento::FieldRegion::getValueType() const"},
    {"_ZN5clang4ento14ObjCIvarRegionD1Ev", "clang::ento::ObjCIvarRegion::~ObjCIvarRegion()"},
    {"_ZN5clang4ento14ObjCIvarRegionD0Ev", "clang::ento::ObjCIvarRegion::~ObjCIvarRegion()"},
    {"_ZNK5clang4ento14ObjCIvarRegion12getValueTypeEv", "clang::ento::ObjCIvarRegion::getValueType() const"},
    {"_ZN5clang4ento13ElementRegionD1Ev", "clang::ento::ElementRegion::~ElementRegion()"},
    {"_ZN5clang4ento13ElementRegionD0Ev", "clang::ento::ElementRegion::~ElementRegion()"},
    {"_ZNK5clang4ento13ElementRegion12getValueTypeEv", "clang::ento::ElementRegion::getValueType() const"},
    {"_ZN5clang4ento19CXXTempObjectRegionD1Ev", "clang::ento::CXXTempObjectRegion::~CXXTempObjectRegion()"},
    {"_ZN5clang4ento19CXXTempObjectRegionD0Ev", "clang::ento::CXXTempObjectRegion::~CXXTempObjectRegion()"},
    {"_ZNK5clang4ento19CXXTempObjectRegion12getValueTypeEv", "clang::ento::CXXTempObjectRegion::getValueType() const"},
    {"_ZN5clang4ento19CXXBaseObjectRegionD1Ev", "clang::ento::CXXBaseObjectRegion::~CXXBaseObjectRegion()"},
    {"_ZN5clang4ento19CXXBaseObjectRegionD0Ev", "clang::ento::CXXBaseObjectRegion::~CXXBaseObjectRegion()"},
    {"_ZN5clang4ento11TypedRegionD1Ev", "clang::ento::TypedRegion::~TypedRegion()"},
    {"_ZN5clang4ento11TypedRegionD0Ev", "clang::ento::TypedRegion::~TypedRegion()"},
    {"_ZN5clang4ento14CodeTextRegionD1Ev", "clang::ento::CodeTextRegion::~CodeTextRegion()"},
    {"_ZN5clang4ento14CodeTextRegionD0Ev", "clang::ento::CodeTextRegion::~CodeTextRegion()"},
    {"_ZN5clang4ento18UnknownSpaceRegionD1Ev", "clang::ento::UnknownSpaceRegion::~UnknownSpaceRegion()"},
    {"_ZN5clang4ento18UnknownSpaceRegionD0Ev", "clang::ento::UnknownSpaceRegion::~UnknownSpaceRegion()"},
    {"_ZN5clang4ento15HeapSpaceRegionD1Ev", "clang::ento::HeapSpaceRegion::~HeapSpaceRegion()"},
    {"_ZN5clang4ento15HeapSpaceRegionD0Ev", "clang::ento::HeapSpaceRegion::~HeapSpaceRegion()"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento14CodeTextRegionEPNS2_23StaticGlobalSpaceRegionENS_12DenseMapInfoIS5_EENS8_IS7_EEE16InsertIntoBucketERKS5_RKS7_PSt4pairIS5_S7_E", "llvm::DenseMap<clang::ento::CodeTextRegion const*, clang::ento::StaticGlobalSpaceRegion*, llvm::DenseMapInfo<clang::ento::CodeTextRegion const*>, llvm::DenseMapInfo<clang::ento::StaticGlobalSpaceRegion*> >::InsertIntoBucket(clang::ento::CodeTextRegion const* const&, clang::ento::StaticGlobalSpaceRegion* const&, std::pair<clang::ento::CodeTextRegion const*, clang::ento::StaticGlobalSpaceRegion*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento14CodeTextRegionEPNS2_23StaticGlobalSpaceRegionENS_12DenseMapInfoIS5_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::ento::CodeTextRegion const*, clang::ento::StaticGlobalSpaceRegion*, llvm::DenseMapInfo<clang::ento::CodeTextRegion const*>, llvm::DenseMapInfo<clang::ento::StaticGlobalSpaceRegion*> >::grow(unsigned int)"},
    {"_ZN5clang4ento18GlobalsSpaceRegionD1Ev", "clang::ento::GlobalsSpaceRegion::~GlobalsSpaceRegion()"},
    {"_ZN5clang4ento18GlobalsSpaceRegionD0Ev", "clang::ento::GlobalsSpaceRegion::~GlobalsSpaceRegion()"},
    {"_ZN4llvm8DenseMapIPKN5clang17StackFrameContextEPNS1_4ento25StackArgumentsSpaceRegionENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::StackFrameContext const*, clang::ento::StackArgumentsSpaceRegion*, llvm::DenseMapInfo<clang::StackFrameContext const*>, llvm::DenseMapInfo<clang::ento::StackArgumentsSpaceRegion*> >::InsertIntoBucket(clang::StackFrameContext const* const&, clang::ento::StackArgumentsSpaceRegion* const&, std::pair<clang::StackFrameContext const*, clang::ento::StackArgumentsSpaceRegion*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang17StackFrameContextEPNS1_4ento25StackArgumentsSpaceRegionENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::StackFrameContext const*, clang::ento::StackArgumentsSpaceRegion*, llvm::DenseMapInfo<clang::StackFrameContext const*>, llvm::DenseMapInfo<clang::ento::StackArgumentsSpaceRegion*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang17StackFrameContextEPNS1_4ento22StackLocalsSpaceRegionENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::StackFrameContext const*, clang::ento::StackLocalsSpaceRegion*, llvm::DenseMapInfo<clang::StackFrameContext const*>, llvm::DenseMapInfo<clang::ento::StackLocalsSpaceRegion*> >::InsertIntoBucket(clang::StackFrameContext const* const&, clang::ento::StackLocalsSpaceRegion* const&, std::pair<clang::StackFrameContext const*, clang::ento::StackLocalsSpaceRegion*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang17StackFrameContextEPNS1_4ento22StackLocalsSpaceRegionENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::StackFrameContext const*, clang::ento::StackLocalsSpaceRegion*, llvm::DenseMapInfo<clang::StackFrameContext const*>, llvm::DenseMapInfo<clang::ento::StackLocalsSpaceRegion*> >::grow(unsigned int)"},
    {"_ZN5clang4ento25StackArgumentsSpaceRegionD1Ev", "clang::ento::StackArgumentsSpaceRegion::~StackArgumentsSpaceRegion()"},
    {"_ZN5clang4ento25StackArgumentsSpaceRegionD0Ev", "clang::ento::StackArgumentsSpaceRegion::~StackArgumentsSpaceRegion()"},
    {"_ZN5clang4ento22StackLocalsSpaceRegionD1Ev", "clang::ento::StackLocalsSpaceRegion::~StackLocalsSpaceRegion()"},
    {"_ZN5clang4ento22StackLocalsSpaceRegionD0Ev", "clang::ento::StackLocalsSpaceRegion::~StackLocalsSpaceRegion()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento9MemRegionEED0Ev", "llvm::FoldingSet<clang::ento::MemRegion>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento9MemRegionEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::MemRegion>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento9MemRegionEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::MemRegion>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento9MemRegionEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::MemRegion>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento11ObjCMessage7getTypeERNS_10ASTContextE", "clang::ento::ObjCMessage::getType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento11ObjCMessage11getSelectorEv", "clang::ento::ObjCMessage::getSelector() const"},
    {"_ZNK5clang4ento11ObjCMessage13getMethodDeclEv", "clang::ento::ObjCMessage::getMethodDecl() const"},
    {"_ZNK5clang4ento11ObjCMessage20getReceiverInterfaceEv", "clang::ento::ObjCMessage::getReceiverInterface() const"},
    {"_ZNK5clang4ento11ObjCMessage10getArgExprEj", "clang::ento::ObjCMessage::getArgExpr(unsigned int) const"},
    {"_ZNK5clang4ento17CallOrObjCMessage13getResultTypeERNS_10ASTContextE", "clang::ento::CallOrObjCMessage::getResultType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento17CallOrObjCMessage23getArgSValAsScalarOrLocEj", "clang::ento::CallOrObjCMessage::getArgSValAsScalarOrLoc(unsigned int) const"},
    {"_ZNK5clang4ento24PathDiagnosticMacroPiece13containsEventEv", "clang::ento::PathDiagnosticMacroPiece::containsEvent() const"},
    {"_ZN5clang4ento19PathDiagnosticPieceC1EN4llvm9StringRefENS1_4KindENS1_11DisplayHintE", "clang::ento::PathDiagnosticPiece::PathDiagnosticPiece(llvm::StringRef, clang::ento::PathDiagnosticPiece::Kind, clang::ento::PathDiagnosticPiece::DisplayHint)"},
    {"_ZN5clang4ento19PathDiagnosticPieceC2EN4llvm9StringRefENS1_4KindENS1_11DisplayHintE", "clang::ento::PathDiagnosticPiece::PathDiagnosticPiece(llvm::StringRef, clang::ento::PathDiagnosticPiece::Kind, clang::ento::PathDiagnosticPiece::DisplayHint)"},
    {"_ZN5clang4ento19PathDiagnosticPieceC1ENS1_4KindENS1_11DisplayHintE", "clang::ento::PathDiagnosticPiece::PathDiagnosticPiece(clang::ento::PathDiagnosticPiece::Kind, clang::ento::PathDiagnosticPiece::DisplayHint)"},
    {"_ZN5clang4ento19PathDiagnosticPieceC2ENS1_4KindENS1_11DisplayHintE", "clang::ento::PathDiagnosticPiece::PathDiagnosticPiece(clang::ento::PathDiagnosticPiece::Kind, clang::ento::PathDiagnosticPiece::DisplayHint)"},
    {"_ZN5clang4ento19PathDiagnosticPieceD0Ev", "clang::ento::PathDiagnosticPiece::~PathDiagnosticPiece()"},
    {"_ZN5clang4ento19PathDiagnosticPieceD1Ev", "clang::ento::PathDiagnosticPiece::~PathDiagnosticPiece()"},
    {"_ZN5clang4ento19PathDiagnosticPieceD2Ev", "clang::ento::PathDiagnosticPiece::~PathDiagnosticPiece()"},
    {"_ZN5clang4ento24PathDiagnosticEventPieceD0Ev", "clang::ento::PathDiagnosticEventPiece::~PathDiagnosticEventPiece()"},
    {"_ZN5clang4ento24PathDiagnosticEventPieceD1Ev", "clang::ento::PathDiagnosticEventPiece::~PathDiagnosticEventPiece()"},
    {"_ZN5clang4ento24PathDiagnosticEventPieceD2Ev", "clang::ento::PathDiagnosticEventPiece::~PathDiagnosticEventPiece()"},
    {"_ZN5clang4ento30PathDiagnosticControlFlowPieceD0Ev", "clang::ento::PathDiagnosticControlFlowPiece::~PathDiagnosticControlFlowPiece()"},
    {"_ZN5clang4ento30PathDiagnosticControlFlowPieceD1Ev", "clang::ento::PathDiagnosticControlFlowPiece::~PathDiagnosticControlFlowPiece()"},
    {"_ZN5clang4ento30PathDiagnosticControlFlowPieceD2Ev", "clang::ento::PathDiagnosticControlFlowPiece::~PathDiagnosticControlFlowPiece()"},
    {"_ZN5clang4ento24PathDiagnosticMacroPieceD0Ev", "clang::ento::PathDiagnosticMacroPiece::~PathDiagnosticMacroPiece()"},
    {"_ZN5clang4ento24PathDiagnosticMacroPieceD1Ev", "clang::ento::PathDiagnosticMacroPiece::~PathDiagnosticMacroPiece()"},
    {"_ZN5clang4ento24PathDiagnosticMacroPieceD2Ev", "clang::ento::PathDiagnosticMacroPiece::~PathDiagnosticMacroPiece()"},
    {"_ZN5clang4ento14PathDiagnosticC1Ev", "clang::ento::PathDiagnostic::PathDiagnostic()"},
    {"_ZN5clang4ento14PathDiagnosticC2Ev", "clang::ento::PathDiagnostic::PathDiagnostic()"},
    {"_ZN5clang4ento14PathDiagnosticD1Ev", "clang::ento::PathDiagnostic::~PathDiagnostic()"},
    {"_ZN5clang4ento14PathDiagnosticD2Ev", "clang::ento::PathDiagnostic::~PathDiagnostic()"},
    {"_ZN5clang4ento14PathDiagnostic9resetPathEb", "clang::ento::PathDiagnostic::resetPath(bool)"},
    {"_ZN5clang4ento14PathDiagnosticC1EN4llvm9StringRefES3_S3_", "clang::ento::PathDiagnostic::PathDiagnostic(llvm::StringRef, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang4ento14PathDiagnosticC2EN4llvm9StringRefES3_S3_", "clang::ento::PathDiagnostic::PathDiagnostic(llvm::StringRef, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang4ento20PathDiagnosticClient16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::ento::PathDiagnosticClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZNK5clang4ento22PathDiagnosticLocation10asLocationEv", "clang::ento::PathDiagnosticLocation::asLocation() const"},
    {"_ZNK5clang4ento22PathDiagnosticLocation7asRangeEv", "clang::ento::PathDiagnosticLocation::asRange() const"},
    {"_ZN5clang4ento22PathDiagnosticLocation7flattenEv", "clang::ento::PathDiagnosticLocation::flatten()"},
    {"_ZNK5clang4ento22PathDiagnosticLocation7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnosticLocation::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento19PathDiagnosticPiece7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnosticPiece::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento23PathDiagnosticSpotPiece7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnosticSpotPiece::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento30PathDiagnosticControlFlowPiece7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnosticControlFlowPiece::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento24PathDiagnosticMacroPiece7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnosticMacroPiece::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4ento14PathDiagnostic7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::PathDiagnostic::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNSt11_Deque_baseIPN5clang4ento19PathDiagnosticPieceESaIS3_EE17_M_initialize_mapEm", "std::_Deque_base<clang::ento::PathDiagnosticPiece*, std::allocator<clang::ento::PathDiagnosticPiece*> >::_M_initialize_map(unsigned long)"},
    {"_ZN5clang4ento23PathDiagnosticSpotPieceC2ERKNS0_22PathDiagnosticLocationEN4llvm9StringRefENS0_19PathDiagnosticPiece4KindEb", "clang::ento::PathDiagnosticSpotPiece::PathDiagnosticSpotPiece(clang::ento::PathDiagnosticLocation const&, llvm::StringRef, clang::ento::PathDiagnosticPiece::Kind, bool)"},
    {"_ZN5clang4ento27createPlistDiagnosticClientERKSsRKNS_12PreprocessorEPNS0_20PathDiagnosticClientE", "clang::ento::createPlistDiagnosticClient(std::string const&, clang::Preprocessor const&, clang::ento::PathDiagnosticClient*)"},
    {"_ZN12_GLOBAL__N_116PlistDiagnosticsD1Ev", "(anonymous namespace)::PlistDiagnostics::~PlistDiagnostics()"},
    {"_ZN12_GLOBAL__N_116PlistDiagnosticsD0Ev", "(anonymous namespace)::PlistDiagnostics::~PlistDiagnostics()"},
    {"_ZN12_GLOBAL__N_116PlistDiagnostics16FlushDiagnosticsEPN4llvm15SmallVectorImplISsEE", "(anonymous namespace)::PlistDiagnostics::FlushDiagnostics(llvm::SmallVectorImpl<std::string>*)"},
    {"_ZNK12_GLOBAL__N_116PlistDiagnostics7getNameEv", "(anonymous namespace)::PlistDiagnostics::getName() const"},
    {"_ZN12_GLOBAL__N_116PlistDiagnostics20HandlePathDiagnosticEPKN5clang4ento14PathDiagnosticE", "(anonymous namespace)::PlistDiagnostics::HandlePathDiagnostic(clang::ento::PathDiagnostic const*)"},
    {"_ZNK12_GLOBAL__N_116PlistDiagnostics19getGenerationSchemeEv", "(anonymous namespace)::PlistDiagnostics::getGenerationScheme() const"},
    {"_ZNK12_GLOBAL__N_116PlistDiagnostics28supportsLogicalOpControlFlowEv", "(anonymous namespace)::PlistDiagnostics::supportsLogicalOpControlFlow() const"},
    {"_ZNK12_GLOBAL__N_116PlistDiagnostics21supportsAllBlockEdgesEv", "(anonymous namespace)::PlistDiagnostics::supportsAllBlockEdges() const"},
    {"_ZNK12_GLOBAL__N_116PlistDiagnostics21useVerboseDescriptionEv", "(anonymous namespace)::PlistDiagnostics::useVerboseDescription() const"},
    {"_Z6AddFIDRN4llvm8DenseMapIN5clang6FileIDEjNS_12DenseMapInfoIS2_EENS3_IjEEEERNS_15SmallVectorImplIS2_EEPKNS1_13SourceManagerENS1_14SourceLocationE", "AddFID(llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> >&, llvm::SmallVectorImpl<clang::FileID>&, clang::SourceManager const*, clang::SourceLocation)"},
    {"_Z10EmitStringRN4llvm11raw_ostreamERKSs", "EmitString(llvm::raw_ostream&, std::string const&)"},
    {"_Z12EmitLocationRN4llvm11raw_ostreamERKN5clang13SourceManagerERKNS2_11LangOptionsENS2_14SourceLocationERKNS_8DenseMapINS2_6FileIDEjNS_12DenseMapInfoISB_EENSC_IjEEEEjb", "EmitLocation(llvm::raw_ostream&, clang::SourceManager const&, clang::LangOptions const&, clang::SourceLocation, llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> > const&, unsigned int, bool)"},
    {"_Z11ReportEventRN4llvm11raw_ostreamERKN5clang4ento19PathDiagnosticPieceERKNS_8DenseMapINS2_6FileIDEjNS_12DenseMapInfoIS8_EENS9_IjEEEERKNS2_13SourceManagerERKNS2_11LangOptionsEj", "ReportEvent(llvm::raw_ostream&, clang::ento::PathDiagnosticPiece const&, llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> > const&, clang::SourceManager const&, clang::LangOptions const&, unsigned int)"},
    {"_Z11ReportMacroRN4llvm11raw_ostreamERKN5clang4ento24PathDiagnosticMacroPieceERKNS_8DenseMapINS2_6FileIDEjNS_12DenseMapInfoIS8_EENS9_IjEEEERKNS2_13SourceManagerERKNS2_11LangOptionsEj", "ReportMacro(llvm::raw_ostream&, clang::ento::PathDiagnosticMacroPiece const&, llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> > const&, clang::SourceManager const&, clang::LangOptions const&, unsigned int)"},
    {"_Z9EmitRangeRN4llvm11raw_ostreamERKN5clang13SourceManagerERKNS2_11LangOptionsENS2_4ento19PathDiagnosticRangeERKNS_8DenseMapINS2_6FileIDEjNS_12DenseMapInfoISC_EENSD_IjEEEEj", "EmitRange(llvm::raw_ostream&, clang::SourceManager const&, clang::LangOptions const&, clang::ento::PathDiagnosticRange, llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> > const&, unsigned int)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPPKN5clang4ento14PathDiagnosticESt6vectorIS6_SaIS6_EEEElN12_GLOBAL__N_118CompareDiagnosticsEEvT_SE_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, long, (anonymous namespace)::CompareDiagnostics>(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, __gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, long, (anonymous namespace)::CompareDiagnostics)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPPKN5clang4ento14PathDiagnosticESt6vectorIS6_SaIS6_EEEEN12_GLOBAL__N_118CompareDiagnosticsEEvT_SE_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, (anonymous namespace)::CompareDiagnostics>(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, __gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, (anonymous namespace)::CompareDiagnostics)"},
    {"_ZNK12_GLOBAL__N_118CompareDiagnosticsclEPKN5clang4ento14PathDiagnosticES5_", "(anonymous namespace)::CompareDiagnostics::operator()(clang::ento::PathDiagnostic const*, clang::ento::PathDiagnostic const*) const"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPPKN5clang4ento14PathDiagnosticESt6vectorIS6_SaIS6_EEEElS6_N12_GLOBAL__N_118CompareDiagnosticsEEvT_T0_SF_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, long, clang::ento::PathDiagnostic const*, (anonymous namespace)::CompareDiagnostics>(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, long, long, clang::ento::PathDiagnostic const*, (anonymous namespace)::CompareDiagnostics)"},
    {"_ZNSt6vectorIPKN5clang4ento14PathDiagnosticESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::ento::PathDiagnostic const**, std::vector<clang::ento::PathDiagnostic const*, std::allocator<clang::ento::PathDiagnostic const*> > >, clang::ento::PathDiagnostic const* const&)"},
    {"_ZN4llvm8DenseMapIN5clang6FileIDEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::FileID const&, unsigned int const&, std::pair<clang::FileID, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIN5clang6FileIDEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<clang::FileID, unsigned int, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang4ento28CreateRangeConstraintManagerERNS0_14GRStateManagerERNS0_9SubEngineE", "clang::ento::CreateRangeConstraintManager(clang::ento::GRStateManager&, clang::ento::SubEngine&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManagerD1Ev", "(anonymous namespace)::RangeConstraintManager::~RangeConstraintManager()"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManagerD0Ev", "(anonymous namespace)::RangeConstraintManager::~RangeConstraintManager()"},
    {"_ZNK12_GLOBAL__N_122RangeConstraintManager9getSymValEPKN5clang4ento7GRStateEPKNS2_10SymbolDataE", "(anonymous namespace)::RangeConstraintManager::getSymVal(clang::ento::GRState const*, clang::ento::SymbolData const*) const"},
    {"_ZNK12_GLOBAL__N_122RangeConstraintManager7isEqualEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntE", "(anonymous namespace)::RangeConstraintManager::isEqual(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&) const"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager18removeDeadBindingsEPKN5clang4ento7GRStateERNS2_12SymbolReaperE", "(anonymous namespace)::RangeConstraintManager::removeDeadBindings(clang::ento::GRState const*, clang::ento::SymbolReaper&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager5printEPKN5clang4ento7GRStateERN4llvm11raw_ostreamEPKcSA_", "(anonymous namespace)::RangeConstraintManager::print(clang::ento::GRState const*, llvm::raw_ostream&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymNEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymNE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymEQEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymEQ(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymLTEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymLT(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymGTEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymGT(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymLEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymLE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager11assumeSymGEEPKN5clang4ento7GRStateEPKNS2_10SymbolDataERKN4llvm6APSIntESC_", "(anonymous namespace)::RangeConstraintManager::assumeSymGE(clang::ento::GRState const*, clang::ento::SymbolData const*, llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_122RangeConstraintManager8GetRangeEPKN5clang4ento7GRStateEPKNS2_10SymbolDataE", "(anonymous namespace)::RangeConstraintManager::GetRange(clang::ento::GRState const*, clang::ento::SymbolData const*)"},
    {"_ZNK12_GLOBAL__N_18RangeSet9IntersectERN5clang4ento17BasicValueFactoryERN4llvm12ImmutableSetINS_5RangeENS_10RangeTraitEE7FactoryERKNS5_6APSIntESE_", "(anonymous namespace)::RangeSet::Intersect(clang::ento::BasicValueFactory&, llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>::Factory&, llvm::APSInt const&, llvm::APSInt const&) const"},
    {"_ZNK5clang4ento7GRState3setIN12_GLOBAL__N_115ConstraintRangeEEEPKS1_NS0_12GRStateTraitIT_E8key_typeENS9_10value_typeE", "clang::ento::GRState const* clang::ento::GRState::set<(anonymous namespace)::ConstraintRange>(clang::ento::GRStateTrait<(anonymous namespace)::ConstraintRange>::key_type, clang::ento::GRStateTrait<(anonymous namespace)::ConstraintRange>::value_type) const"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_18RangeSetENS2_16ImutKeyValueInfoIS6_S8_EEEEE13CreateContextERNS2_16BumpPtrAllocatorE", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> > >::CreateContext(llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento19GRStatePartialTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_18RangeSetENS2_16ImutKeyValueInfoIS6_S8_EEEEE13DeleteContextEPv", "clang::ento::GRStatePartialTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> > >::DeleteContext(void*)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEEENS_12DenseMapInfoIjEENSD_ISC_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEEESaISC_EE9push_backERKSC_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >* const&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::operator++()"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorIN12_GLOBAL__N_110RangeTraitEEppEv", "llvm::ImutAVLTreeGenericIterator<(anonymous namespace)::RangeTrait>::operator++()"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE12add_internalERKSt4pairIS6_S8_EPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::add_internal(std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE13markImmutableEPNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE10createNodeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE11balanceTreeEPNS_11ImutAVLTreeIS9_EERKSt4pairIS6_S8_ESD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*, std::pair<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm11ImutAVLTreeIN12_GLOBAL__N_110RangeTraitEE7destroyEv", "llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeIN12_GLOBAL__N_110RangeTraitEEENS_12DenseMapInfoIjEENS6_IS5_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeIN12_GLOBAL__N_110RangeTraitEE13computeDigestEv", "llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeIN12_GLOBAL__N_110RangeTraitEEESaIS5_EE9push_backERKS5_", "std::vector<llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*, std::allocator<llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*> >::push_back(llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>* const&)"},
    {"_ZNK12_GLOBAL__N_18RangeSet16IntersectInRangeERN5clang4ento17BasicValueFactoryERN4llvm12ImmutableSetINS_5RangeENS_10RangeTraitEE7FactoryERKNS5_6APSIntESE_RS9_RNS9_8iteratorESH_", "(anonymous namespace)::RangeSet::IntersectInRange(clang::ento::BasicValueFactory&, llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>::Factory&, llvm::APSInt const&, llvm::APSInt const&, llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>&, llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>::iterator&, llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>::iterator&) const"},
    {"_ZN4llvm12ImmutableSetIN12_GLOBAL__N_15RangeENS1_10RangeTraitEE7Factory3addES4_RKS2_", "llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>::Factory::add(llvm::ImmutableSet<(anonymous namespace)::Range, (anonymous namespace)::RangeTrait>, (anonymous namespace)::Range const&)"},
    {"_ZN4llvm14ImutAVLFactoryIN12_GLOBAL__N_110RangeTraitEE12add_internalERKNS1_5RangeEPNS_11ImutAVLTreeIS2_EE", "llvm::ImutAVLFactory<(anonymous namespace)::RangeTrait>::add_internal((anonymous namespace)::Range const&, llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*)"},
    {"_ZN4llvm14ImutAVLFactoryIN12_GLOBAL__N_110RangeTraitEE13markImmutableEPNS_11ImutAVLTreeIS2_EE", "llvm::ImutAVLFactory<(anonymous namespace)::RangeTrait>::markImmutable(llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*)"},
    {"_ZN4llvm14ImutAVLFactoryIN12_GLOBAL__N_110RangeTraitEE10createNodeEPNS_11ImutAVLTreeIS2_EERKNS1_5RangeES6_", "llvm::ImutAVLFactory<(anonymous namespace)::RangeTrait>::createNode(llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*, (anonymous namespace)::Range const&, llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*)"},
    {"_ZN4llvm14ImutAVLFactoryIN12_GLOBAL__N_110RangeTraitEE11balanceTreeEPNS_11ImutAVLTreeIS2_EERKNS1_5RangeES6_", "llvm::ImutAVLFactory<(anonymous namespace)::RangeTrait>::balanceTree(llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*, (anonymous namespace)::Range const&, llvm::ImutAVLTree<(anonymous namespace)::RangeTrait>*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE15remove_internalERKS6_PNS_11ImutAVLTreeIS9_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::remove_internal(clang::ento::SymbolData const* const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIPKN5clang4ento10SymbolDataEN12_GLOBAL__N_18RangeSetEEEE16removeMinBindingEPNS_11ImutAVLTreeIS9_EERSD_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RangeSet> >*&)"},
    {"_ZN5clang4ento17ConstraintManager7EndPathEPKNS0_7GRStateE", "clang::ento::ConstraintManager::EndPath(clang::ento::GRState const*)"},
    {"_ZN4llvm6APSInt11getMaxValueEjb", "llvm::APSInt::getMaxValue(unsigned int, bool)"},
    {"_ZN4llvm6APSInt11getMinValueEjb", "llvm::APSInt::getMinValue(unsigned int, bool)"},
    {"_ZN5clang4ento24CreateRegionStoreManagerERNS0_14GRStateManagerE", "clang::ento::CreateRegionStoreManager(clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento34CreateFieldsOnlyRegionStoreManagerERNS0_14GRStateManagerE", "clang::ento::CreateFieldsOnlyRegionStoreManager(clang::ento::GRStateManager&)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManagerD1Ev", "(anonymous namespace)::RegionStoreManager::~RegionStoreManager()"},
    {"_ZN12_GLOBAL__N_118RegionStoreManagerD0Ev", "(anonymous namespace)::RegionStoreManager::~RegionStoreManager()"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager8RetrieveEPKvN5clang4ento3LocENS3_8QualTypeE", "(anonymous namespace)::RegionStoreManager::Retrieve(void const*, clang::ento::Loc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager4BindEPKvN5clang4ento3LocENS4_4SValE", "(anonymous namespace)::RegionStoreManager::Bind(void const*, clang::ento::Loc, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager11BindDefaultEPKvPKN5clang4ento9MemRegionENS4_4SValE", "(anonymous namespace)::RegionStoreManager::BindDefault(void const*, clang::ento::MemRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager6RemoveEPKvN5clang4ento3LocE", "(anonymous namespace)::RegionStoreManager::Remove(void const*, clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager19BindCompoundLiteralEPKvPKN5clang19CompoundLiteralExprEPKNS3_15LocationContextENS3_4ento4SValE", "(anonymous namespace)::RegionStoreManager::BindCompoundLiteral(void const*, clang::CompoundLiteralExpr const*, clang::LocationContext const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager15getInitialStoreEPKN5clang15LocationContextE", "(anonymous namespace)::RegionStoreManager::getInitialStore(clang::LocationContext const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager15getSubRegionMapEPKv", "(anonymous namespace)::RegionStoreManager::getSubRegionMap(void const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager17getSizeInElementsEPKN5clang4ento7GRStateEPKNS2_9MemRegionENS1_8QualTypeE", "(anonymous namespace)::RegionStoreManager::getSizeInElements(clang::ento::GRState const*, clang::ento::MemRegion const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager14ArrayToPointerEN5clang4ento3LocE", "(anonymous namespace)::RegionStoreManager::ArrayToPointer(clang::ento::Loc)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager17evalDerivedToBaseEN5clang4ento4SValENS1_8QualTypeE", "(anonymous namespace)::RegionStoreManager::evalDerivedToBase(clang::ento::SVal, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager18removeDeadBindingsEPKvPKN5clang17StackFrameContextERNS3_4ento12SymbolReaperERN4llvm15SmallVectorImplIPKNS7_9MemRegionEEE", "(anonymous namespace)::RegionStoreManager::removeDeadBindings(void const*, clang::StackFrameContext const*, clang::ento::SymbolReaper&, llvm::SmallVectorImpl<clang::ento::MemRegion const*>&)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager8BindDeclEPKvPKN5clang4ento9VarRegionENS4_4SValE", "(anonymous namespace)::RegionStoreManager::BindDecl(void const*, clang::ento::VarRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager18BindDeclWithNoInitEPKvPKN5clang4ento9VarRegionE", "(anonymous namespace)::RegionStoreManager::BindDeclWithNoInit(void const*, clang::ento::VarRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager17InvalidateRegionsEPKvPKPKN5clang4ento9MemRegionES9_PKNS3_4ExprEjPN4llvm8DenseSetIPKNS4_10SymbolDataENSD_12DenseMapInfoISH_EEEEbPNSD_11SmallVectorIS7_Lj8EEE", "(anonymous namespace)::RegionStoreManager::InvalidateRegions(void const*, clang::ento::MemRegion const* const*, clang::ento::MemRegion const* const*, clang::Expr const*, unsigned int, llvm::DenseSet<clang::ento::SymbolData const*, llvm::DenseMapInfo<clang::ento::SymbolData const*> >*, bool, llvm::SmallVector<clang::ento::MemRegion const*, 8u>*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager15enterStackFrameEPKN5clang4ento7GRStateEPKNS1_17StackFrameContextE", "(anonymous namespace)::RegionStoreManager::enterStackFrame(clang::ento::GRState const*, clang::StackFrameContext const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager5printEPKvRN4llvm11raw_ostreamEPKcS7_", "(anonymous namespace)::RegionStoreManager::print(void const*, llvm::raw_ostream&, char const*, char const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager12iterBindingsEPKvRN5clang4ento12StoreManager15BindingsHandlerE", "(anonymous namespace)::RegionStoreManager::iterBindings(void const*, clang::ento::StoreManager::BindingsHandler&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::destroy()"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEEENS_12DenseMapInfoIjEENSB_ISA_EEEixERKj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*> >::operator[](unsigned int const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEEESaISA_EE9push_backERKSA_", "std::vector<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*> >::push_back(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >* const&)"},
    {"_ZN4llvm26ImutAVLTreeInOrderIteratorINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEEppEv", "llvm::ImutAVLTreeInOrderIterator<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::operator++()"},
    {"_ZN12_GLOBAL__N_115ClusterAnalysisINS_23InvalidateRegionsWorkerEE13AddToWorkListEPKN5clang4ento9MemRegionE", "(anonymous namespace)::ClusterAnalysis<(anonymous namespace)::InvalidateRegionsWorker>::AddToWorkList(clang::ento::MemRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager10addBindingEN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEES3_S6_", "(anonymous namespace)::RegionStoreManager::addBinding(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >, (anonymous namespace)::BindingKey, clang::ento::SVal)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE12add_internalERKSt4pairIS3_S6_EPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::add_internal(std::pair<(anonymous namespace)::BindingKey, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE13markImmutableEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::markImmutable(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE10createNodeEPNS_11ImutAVLTreeIS7_EERKSt4pairIS3_S6_ESB_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::createNode(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*, std::pair<(anonymous namespace)::BindingKey, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE11balanceTreeEPNS_11ImutAVLTreeIS7_EERKSt4pairIS3_S6_ESB_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::balanceTree(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*, std::pair<(anonymous namespace)::BindingKey, clang::ento::SVal> const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager13removeBindingEN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEES3_", "(anonymous namespace)::RegionStoreManager::removeBinding(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >, (anonymous namespace)::BindingKey)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE15remove_internalERKS3_PNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::remove_internal((anonymous namespace)::BindingKey const&, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_16ImutKeyValueInfoIN12_GLOBAL__N_110BindingKeyEN5clang4ento4SValEEEE16removeMinBindingEPNS_11ImutAVLTreeIS7_EERSB_", "llvm::ImutAVLFactory<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >::removeMinBinding(llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*, llvm::ImutAVLTree<llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >*&)"},
    {"_ZN12_GLOBAL__N_123InvalidateRegionsWorker12VisitBindingEN5clang4ento4SValE", "(anonymous namespace)::InvalidateRegionsWorker::VisitBinding(clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_115ClusterAnalysisINS_23InvalidateRegionsWorkerEE10getClusterEPKN5clang4ento9MemRegionE", "(anonymous namespace)::ClusterAnalysis<(anonymous namespace)::InvalidateRegionsWorker>::getCluster(clang::ento::MemRegion const*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionEPNS1_10BumpVectorIN12_GLOBAL__N_110BindingKeyEEENS_12DenseMapInfoIS5_EENSB_ISA_EEEixERKS5_", "llvm::DenseMap<clang::ento::MemRegion const*, clang::BumpVector<(anonymous namespace)::BindingKey>*, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<clang::BumpVector<(anonymous namespace)::BindingKey>*> >::operator[](clang::ento::MemRegion const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang10BumpVectorIN12_GLOBAL__N_110BindingKeyEEEjNS_12DenseMapInfoIS7_EENS8_IjEEEixERKS7_", "llvm::DenseMap<clang::BumpVector<(anonymous namespace)::BindingKey> const*, unsigned int, llvm::DenseMapInfo<clang::BumpVector<(anonymous namespace)::BindingKey> const*>, llvm::DenseMapInfo<unsigned int> >::operator[](clang::BumpVector<(anonymous namespace)::BindingKey> const* const&)"},
    {"_ZN5clang10BumpVectorIN12_GLOBAL__N_110BindingKeyEE9push_backERKS2_RNS_17BumpVectorContextE", "clang::BumpVector<(anonymous namespace)::BindingKey>::push_back((anonymous namespace)::BindingKey const&, clang::BumpVectorContext&)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager9BindArrayEPKvPKN5clang4ento11TypedRegionENS4_4SValE", "(anonymous namespace)::RegionStoreManager::BindArray(void const*, clang::ento::TypedRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager10BindStructEPKvPKN5clang4ento11TypedRegionENS4_4SValE", "(anonymous namespace)::RegionStoreManager::BindStruct(void const*, clang::ento::TypedRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager16CopyLazyBindingsEN5clang4ento6nonloc15LazyCompoundValEPKvPKNS2_11TypedRegionE", "(anonymous namespace)::RegionStoreManager::CopyLazyBindings(clang::ento::nonloc::LazyCompoundVal, void const*, clang::ento::TypedRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager10KillStructEPKvPKN5clang4ento11TypedRegionENS4_4SValE", "(anonymous namespace)::RegionStoreManager::KillStruct(void const*, clang::ento::TypedRegion const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager26getRegionStoreSubRegionMapEPKv", "(anonymous namespace)::RegionStoreManager::getRegionStoreSubRegionMap(void const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager23RemoveSubRegionBindingsERN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEEPKNS5_9MemRegionERNS_23RegionStoreSubRegionMapE", "(anonymous namespace)::RegionStoreManager::RemoveSubRegionBindings(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >&, clang::ento::MemRegion const*, (anonymous namespace)::RegionStoreSubRegionMap&)"},
    {"_ZN12_GLOBAL__N_123RegionStoreSubRegionMap7processERN4llvm15SmallVectorImplIPKN5clang4ento9SubRegionEEES7_", "(anonymous namespace)::RegionStoreSubRegionMap::process(llvm::SmallVectorImpl<clang::ento::SubRegion const*>&, clang::ento::SubRegion const*)"},
    {"_ZN12_GLOBAL__N_123RegionStoreSubRegionMapD1Ev", "(anonymous namespace)::RegionStoreSubRegionMap::~RegionStoreSubRegionMap()"},
    {"_ZN12_GLOBAL__N_123RegionStoreSubRegionMapD0Ev", "(anonymous namespace)::RegionStoreSubRegionMap::~RegionStoreSubRegionMap()"},
    {"_ZNK12_GLOBAL__N_123RegionStoreSubRegionMap14iterSubRegionsEPKN5clang4ento9MemRegionERNS2_12SubRegionMap7VisitorE", "(anonymous namespace)::RegionStoreSubRegionMap::iterSubRegions(clang::ento::MemRegion const*, clang::ento::SubRegionMap::Visitor&) const"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager23setImplicitDefaultValueEPKvPKN5clang4ento9MemRegionENS3_8QualTypeE", "(anonymous namespace)::RegionStoreManager::setImplicitDefaultValue(void const*, clang::ento::MemRegion const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_115ClusterAnalysisINS_24removeDeadBindingsWorkerEE13AddToWorkListEPKN5clang4ento9MemRegionE", "(anonymous namespace)::ClusterAnalysis<(anonymous namespace)::removeDeadBindingsWorker>::AddToWorkList(clang::ento::MemRegion const*)"},
    {"_ZN12_GLOBAL__N_124removeDeadBindingsWorker12VisitBindingEN5clang4ento4SValE", "(anonymous namespace)::removeDeadBindingsWorker::VisitBinding(clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager13RetrieveFieldEPKvPKN5clang4ento11FieldRegionE", "(anonymous namespace)::RegionStoreManager::RetrieveField(void const*, clang::ento::FieldRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager15RetrieveElementEPKvPKN5clang4ento13ElementRegionE", "(anonymous namespace)::RegionStoreManager::RetrieveElement(void const*, clang::ento::ElementRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager27RetrieveDerivedDefaultValueEN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEEPKNS5_9MemRegionEPKNS5_11TypedRegionENS4_8QualTypeE", "(anonymous namespace)::RegionStoreManager::RetrieveDerivedDefaultValue(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >, clang::ento::MemRegion const*, clang::ento::TypedRegion const*, clang::QualType)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager17getDefaultBindingEN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEEPKNS5_9MemRegionE", "(anonymous namespace)::RegionStoreManager::getDefaultBinding(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >, clang::ento::MemRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager28RetrieveFieldOrElementCommonEPKvPKN5clang4ento11TypedRegionENS3_8QualTypeEPKNS4_9MemRegionE", "(anonymous namespace)::RegionStoreManager::RetrieveFieldOrElementCommon(void const*, clang::ento::TypedRegion const*, clang::QualType, clang::ento::MemRegion const*)"},
    {"_ZN12_GLOBAL__N_118RegionStoreManager14GetLazyBindingEN4llvm12ImmutableMapINS_10BindingKeyEN5clang4ento4SValENS1_16ImutKeyValueInfoIS3_S6_EEEEPKNS5_9MemRegionE", "(anonymous namespace)::RegionStoreManager::GetLazyBinding(llvm::ImmutableMap<(anonymous namespace)::BindingKey, clang::ento::SVal, llvm::ImutKeyValueInfo<(anonymous namespace)::BindingKey, clang::ento::SVal> >, clang::ento::MemRegion const*)"},
    {"_ZN4llvm26ImutAVLTreeGenericIteratorINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEEppEv", "llvm::ImutAVLTreeGenericIterator<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::operator++()"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionENS_12ImmutableSetIS5_NS_17ImutContainerInfoIS5_EEEENS_12DenseMapInfoIS5_EENSA_IS9_EEE6insertERKSt4pairIS5_S9_E", "llvm::DenseMap<clang::ento::MemRegion const*, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> >, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > > >::insert(std::pair<clang::ento::MemRegion const*, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > > const&)"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE7destroyEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::destroy()"},
    {"_ZN4llvm11ImutAVLTreeINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE13computeDigestEv", "llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::computeDigest()"},
    {"_ZNSt6vectorIPN4llvm11ImutAVLTreeINS0_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEEESaISA_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSA_SC_EERKSA_", "std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >**, std::vector<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, std::allocator<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*> > >, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >* const&)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEEENS_12DenseMapInfoIjEENSB_ISA_EEE16InsertIntoBucketERKjRKSA_PSt4pairIjSA_E", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*> >::InsertIntoBucket(unsigned int const&, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >* const&, std::pair<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_11ImutAVLTreeINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEEENS_12DenseMapInfoIjEENSB_ISA_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*> >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE3addEPNS_11ImutAVLTreeIS7_EES6_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::add(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, clang::ento::MemRegion const*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE16getCanonicalTreeEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::getCanonicalTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE22compareTreeWithSectionEPNS_11ImutAVLTreeIS7_EERNS_26ImutAVLTreeInOrderIteratorIS7_EESE_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::compareTreeWithSection(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >&, llvm::ImutAVLTreeInOrderIterator<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >&)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE12add_internalES6_PNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::add_internal(clang::ento::MemRegion const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE13markImmutableEPNS_11ImutAVLTreeIS7_EE", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::markImmutable(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE10createNodeEPNS_11ImutAVLTreeIS7_EES6_SB_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::createNode(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, clang::ento::MemRegion const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEE11balanceTreeEPNS_11ImutAVLTreeIS7_EES6_SB_", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::balanceTree(llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*, clang::ento::MemRegion const*, llvm::ImutAVLTree<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionENS_12ImmutableSetIS5_NS_17ImutContainerInfoIS5_EEEENS_12DenseMapInfoIS5_EENSA_IS9_EEE16InsertIntoBucketERKS5_RKS9_PSt4pairIS5_S9_E", "llvm::DenseMap<clang::ento::MemRegion const*, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> >, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > > >::InsertIntoBucket(clang::ento::MemRegion const* const&, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > const&, std::pair<clang::ento::MemRegion const*, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4ento9MemRegionENS_12ImmutableSetIS5_NS_17ImutContainerInfoIS5_EEEENS_12DenseMapInfoIS5_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::ento::MemRegion const*, llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> >, llvm::DenseMapInfo<clang::ento::MemRegion const*>, llvm::DenseMapInfo<llvm::ImmutableSet<clang::ento::MemRegion const*, llvm::ImutContainerInfo<clang::ento::MemRegion const*> > > >::grow(unsigned int)"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEED2Ev", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::~ImutAVLFactory()"},
    {"_ZN4llvm14ImutAVLFactoryINS_17ImutContainerInfoIPKN5clang4ento9MemRegionEEEEC2Ev", "llvm::ImutAVLFactory<llvm::ImutContainerInfo<clang::ento::MemRegion const*> >::ImutAVLFactory()"},
    {"_ZN5clang4ento11SValBuilder11makeZeroValENS_8QualTypeE", "clang::ento::SValBuilder::makeZeroVal(clang::QualType)"},
    {"_ZN5clang4ento11SValBuilder10makeNonLocEPKNS0_7SymExprENS_18BinaryOperatorKindERKN4llvm6APSIntENS_8QualTypeE", "clang::ento::SValBuilder::makeNonLoc(clang::ento::SymExpr const*, clang::BinaryOperatorKind, llvm::APSInt const&, clang::QualType)"},
    {"_ZN5clang4ento11SValBuilder10makeNonLocEPKNS0_7SymExprENS_18BinaryOperatorKindES4_NS_8QualTypeE", "clang::ento::SValBuilder::makeNonLoc(clang::ento::SymExpr const*, clang::BinaryOperatorKind, clang::ento::SymExpr const*, clang::QualType)"},
    {"_ZN5clang4ento11SValBuilder19convertToArrayIndexENS0_4SValE", "clang::ento::SValBuilder::convertToArrayIndex(clang::ento::SVal)"},
    {"_ZN5clang4ento11SValBuilder23getRegionValueSymbolValEPKNS0_11TypedRegionE", "clang::ento::SValBuilder::getRegionValueSymbolVal(clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento11SValBuilder20getConjuredSymbolValEPKvPKNS_4ExprEj", "clang::ento::SValBuilder::getConjuredSymbolVal(void const*, clang::Expr const*, unsigned int)"},
    {"_ZN5clang4ento11SValBuilder20getConjuredSymbolValEPKvPKNS_4ExprENS_8QualTypeEj", "clang::ento::SValBuilder::getConjuredSymbolVal(void const*, clang::Expr const*, clang::QualType, unsigned int)"},
    {"_ZN5clang4ento11SValBuilder20getMetadataSymbolValEPKvPKNS0_9MemRegionEPKNS_4ExprENS_8QualTypeEj", "clang::ento::SValBuilder::getMetadataSymbolVal(void const*, clang::ento::MemRegion const*, clang::Expr const*, clang::QualType, unsigned int)"},
    {"_ZN5clang4ento11SValBuilder30getDerivedRegionValueSymbolValEPKNS0_10SymbolDataEPKNS0_11TypedRegionE", "clang::ento::SValBuilder::getDerivedRegionValueSymbolVal(clang::ento::SymbolData const*, clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento11SValBuilder18getFunctionPointerEPKNS_12FunctionDeclE", "clang::ento::SValBuilder::getFunctionPointer(clang::FunctionDecl const*)"},
    {"_ZN5clang4ento11SValBuilder15getBlockPointerEPKNS_9BlockDeclENS_7CanQualINS_4TypeEEEPKNS_15LocationContextE", "clang::ento::SValBuilder::getBlockPointer(clang::BlockDecl const*, clang::CanQual<clang::Type>, clang::LocationContext const*)"},
    {"_ZN5clang4ento11SValBuilder9evalBinOpEPKNS0_7GRStateENS_18BinaryOperatorKindENS0_4SValES6_NS_8QualTypeE", "clang::ento::SValBuilder::evalBinOp(clang::ento::GRState const*, clang::BinaryOperatorKind, clang::ento::SVal, clang::ento::SVal, clang::QualType)"},
    {"_ZN5clang4ento11SValBuilder6evalEQEPKNS0_7GRStateENS0_20DefinedOrUnknownSValES5_", "clang::ento::SValBuilder::evalEQ(clang::ento::GRState const*, clang::ento::DefinedOrUnknownSVal, clang::ento::DefinedOrUnknownSVal)"},
    {"_ZN5clang4ento11SValBuilder8evalCastENS0_4SValENS_8QualTypeES3_", "clang::ento::SValBuilder::evalCast(clang::ento::SVal, clang::QualType, clang::QualType)"},
    {"_ZNK5clang4ento4SVal17hasConjuredSymbolEv", "clang::ento::SVal::hasConjuredSymbol() const"},
    {"_ZNK5clang4ento4SVal17getAsFunctionDeclEv", "clang::ento::SVal::getAsFunctionDecl() const"},
    {"_ZNK5clang4ento4SVal14getAsLocSymbolEv", "clang::ento::SVal::getAsLocSymbol() const"},
    {"_ZNK5clang4ento3loc12MemRegionVal10StripCastsEv", "clang::ento::loc::MemRegionVal::StripCasts() const"},
    {"_ZNK5clang4ento4SVal18getLocSymbolInBaseEv", "clang::ento::SVal::getLocSymbolInBase() const"},
    {"_ZNK5clang4ento4SVal11getAsSymbolEv", "clang::ento::SVal::getAsSymbol() const"},
    {"_ZNK5clang4ento4SVal23getAsSymbolicExpressionEv", "clang::ento::SVal::getAsSymbolicExpression() const"},
    {"_ZNK5clang4ento4SVal11getAsRegionEv", "clang::ento::SVal::getAsRegion() const"},
    {"_ZNK5clang4ento4SVal15symbol_iteratoreqERKS2_", "clang::ento::SVal::symbol_iterator::operator==(clang::ento::SVal::symbol_iterator const&) const"},
    {"_ZNK5clang4ento4SVal15symbol_iteratorneERKS2_", "clang::ento::SVal::symbol_iterator::operator!=(clang::ento::SVal::symbol_iterator const&) const"},
    {"_ZN5clang4ento4SVal15symbol_iteratorC1EPKNS0_7SymExprE", "clang::ento::SVal::symbol_iterator::symbol_iterator(clang::ento::SymExpr const*)"},
    {"_ZN5clang4ento4SVal15symbol_iteratorC2EPKNS0_7SymExprE", "clang::ento::SVal::symbol_iterator::symbol_iterator(clang::ento::SymExpr const*)"},
    {"_ZN5clang4ento4SVal15symbol_iterator6expandEv", "clang::ento::SVal::symbol_iterator::expand()"},
    {"_ZN5clang4ento4SVal15symbol_iteratorppEv", "clang::ento::SVal::symbol_iterator::operator++()"},
    {"_ZN5clang4ento4SVal15symbol_iteratordeEv", "clang::ento::SVal::symbol_iterator::operator*()"},
    {"_ZNK5clang4ento6nonloc15LazyCompoundVal8getStoreEv", "clang::ento::nonloc::LazyCompoundVal::getStore() const"},
    {"_ZNK5clang4ento6nonloc15LazyCompoundVal9getRegionEv", "clang::ento::nonloc::LazyCompoundVal::getRegion() const"},
    {"_ZNK5clang4ento6nonloc11CompoundVal5beginEv", "clang::ento::nonloc::CompoundVal::begin() const"},
    {"_ZNK5clang4ento6nonloc11CompoundVal3endEv", "clang::ento::nonloc::CompoundVal::end() const"},
    {"_ZNK5clang4ento4SVal10isConstantEv", "clang::ento::SVal::isConstant() const"},
    {"_ZNK5clang4ento4SVal10isConstantEi", "clang::ento::SVal::isConstant(int) const"},
    {"_ZNK5clang4ento4SVal14isZeroConstantEv", "clang::ento::SVal::isZeroConstant() const"},
    {"_ZNK5clang4ento6nonloc11ConcreteInt9evalBinOpERNS0_11SValBuilderENS_18BinaryOperatorKindERKS2_", "clang::ento::nonloc::ConcreteInt::evalBinOp(clang::ento::SValBuilder&, clang::BinaryOperatorKind, clang::ento::nonloc::ConcreteInt const&) const"},
    {"_ZNK5clang4ento6nonloc11ConcreteInt14evalComplementERNS0_11SValBuilderE", "clang::ento::nonloc::ConcreteInt::evalComplement(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento6nonloc11ConcreteInt9evalMinusERNS0_11SValBuilderE", "clang::ento::nonloc::ConcreteInt::evalMinus(clang::ento::SValBuilder&) const"},
    {"_ZNK5clang4ento3loc11ConcreteInt9evalBinOpERNS0_17BasicValueFactoryENS_18BinaryOperatorKindERKS2_", "clang::ento::loc::ConcreteInt::evalBinOp(clang::ento::BasicValueFactory&, clang::BinaryOperatorKind, clang::ento::loc::ConcreteInt const&) const"},
    {"_ZNK5clang4ento4SVal4dumpEv", "clang::ento::SVal::dump() const"},
    {"_ZNK5clang4ento4SVal12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SVal::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento6NonLoc12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::NonLoc::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento3Loc12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::Loc::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZN5clang4ento23SimpleConstraintManagerD0Ev", "clang::ento::SimpleConstraintManager::~SimpleConstraintManager()"},
    {"_ZN5clang4ento23SimpleConstraintManagerD1Ev", "clang::ento::SimpleConstraintManager::~SimpleConstraintManager()"},
    {"_ZN5clang4ento23SimpleConstraintManagerD2Ev", "clang::ento::SimpleConstraintManager::~SimpleConstraintManager()"},
    {"_ZNK5clang4ento23SimpleConstraintManager14canReasonAboutENS0_4SValE", "clang::ento::SimpleConstraintManager::canReasonAbout(clang::ento::SVal) const"},
    {"_ZN5clang4ento23SimpleConstraintManager6assumeEPKNS0_7GRStateENS0_11DefinedSValEb", "clang::ento::SimpleConstraintManager::assume(clang::ento::GRState const*, clang::ento::DefinedSVal, bool)"},
    {"_ZN5clang4ento23SimpleConstraintManager6assumeEPKNS0_7GRStateENS0_6NonLocEb", "clang::ento::SimpleConstraintManager::assume(clang::ento::GRState const*, clang::ento::NonLoc, bool)"},
    {"_ZN5clang4ento23SimpleConstraintManager6assumeEPKNS0_7GRStateENS0_3LocEb", "clang::ento::SimpleConstraintManager::assume(clang::ento::GRState const*, clang::ento::Loc, bool)"},
    {"_ZN5clang4ento23SimpleConstraintManager9assumeAuxEPKNS0_7GRStateENS0_3LocEb", "clang::ento::SimpleConstraintManager::assumeAux(clang::ento::GRState const*, clang::ento::Loc, bool)"},
    {"_ZN5clang4ento23SimpleConstraintManager9assumeAuxEPKNS0_7GRStateENS0_6NonLocEb", "clang::ento::SimpleConstraintManager::assumeAux(clang::ento::GRState const*, clang::ento::NonLoc, bool)"},
    {"_ZN5clang4ento23SimpleConstraintManager12assumeSymRelEPKNS0_7GRStateEPKNS0_7SymExprENS_18BinaryOperatorKindERKN4llvm6APSIntE", "clang::ento::SimpleConstraintManager::assumeSymRel(clang::ento::GRState const*, clang::ento::SymExpr const*, clang::BinaryOperatorKind, llvm::APSInt const&)"},
    {"_ZN5clang4ento23createSimpleSValBuilderERN4llvm16BumpPtrAllocatorERNS_10ASTContextERNS0_14GRStateManagerE", "clang::ento::createSimpleSValBuilder(llvm::BumpPtrAllocator&, clang::ASTContext&, clang::ento::GRStateManager&)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder10evalCastNLEN5clang4ento6NonLocENS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::evalCastNL(clang::ento::NonLoc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder9evalCastLEN5clang4ento3LocENS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::evalCastL(clang::ento::Loc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilderD1Ev", "(anonymous namespace)::SimpleSValBuilder::~SimpleSValBuilder()"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilderD0Ev", "(anonymous namespace)::SimpleSValBuilder::~SimpleSValBuilder()"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder9evalMinusEN5clang4ento6NonLocE", "(anonymous namespace)::SimpleSValBuilder::evalMinus(clang::ento::NonLoc)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder14evalComplementEN5clang4ento6NonLocE", "(anonymous namespace)::SimpleSValBuilder::evalComplement(clang::ento::NonLoc)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder11evalBinOpNNEPKN5clang4ento7GRStateENS1_18BinaryOperatorKindENS2_6NonLocES7_NS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::evalBinOpNN(clang::ento::GRState const*, clang::BinaryOperatorKind, clang::ento::NonLoc, clang::ento::NonLoc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder11evalBinOpLLEPKN5clang4ento7GRStateENS1_18BinaryOperatorKindENS2_3LocES7_NS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::evalBinOpLL(clang::ento::GRState const*, clang::BinaryOperatorKind, clang::ento::Loc, clang::ento::Loc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder11evalBinOpLNEPKN5clang4ento7GRStateENS1_18BinaryOperatorKindENS2_3LocENS2_6NonLocENS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::evalBinOpLN(clang::ento::GRState const*, clang::BinaryOperatorKind, clang::ento::Loc, clang::ento::NonLoc, clang::QualType)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder13getKnownValueEPKN5clang4ento7GRStateENS2_4SValE", "(anonymous namespace)::SimpleSValBuilder::getKnownValue(clang::ento::GRState const*, clang::ento::SVal)"},
    {"_ZN12_GLOBAL__N_117SimpleSValBuilder13MakeSymIntValEPKN5clang4ento7SymExprENS1_18BinaryOperatorKindERKN4llvm6APSIntENS1_8QualTypeE", "(anonymous namespace)::SimpleSValBuilder::MakeSymIntVal(clang::ento::SymExpr const*, clang::BinaryOperatorKind, llvm::APSInt const&, clang::QualType)"},
    {"_ZN5clang4ento11SValBuilderC2ERN4llvm16BumpPtrAllocatorERNS_10ASTContextERNS0_14GRStateManagerE", "clang::ento::SValBuilder::SValBuilder(llvm::BumpPtrAllocator&, clang::ASTContext&, clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento11SValBuilderD1Ev", "clang::ento::SValBuilder::~SValBuilder()"},
    {"_ZN5clang4ento11SValBuilderD0Ev", "clang::ento::SValBuilder::~SValBuilder()"},
    {"_ZN5clang4ento16MemRegionManagerC2ERNS_10ASTContextERN4llvm16BumpPtrAllocatorE", "clang::ento::MemRegionManager::MemRegionManager(clang::ASTContext&, llvm::BumpPtrAllocator&)"},
    {"_ZN5clang4ento12StoreManagerC1ERNS0_14GRStateManagerE", "clang::ento::StoreManager::StoreManager(clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento12StoreManagerC2ERNS0_14GRStateManagerE", "clang::ento::StoreManager::StoreManager(clang::ento::GRStateManager&)"},
    {"_ZN5clang4ento12StoreManager15enterStackFrameEPKNS0_7GRStateEPKNS_17StackFrameContextE", "clang::ento::StoreManager::enterStackFrame(clang::ento::GRState const*, clang::StackFrameContext const*)"},
    {"_ZN5clang4ento12StoreManager17MakeElementRegionEPKNS0_9MemRegionENS_8QualTypeEy", "clang::ento::StoreManager::MakeElementRegion(clang::ento::MemRegion const*, clang::QualType, unsigned long long)"},
    {"_ZN5clang4ento12StoreManager20GetElementZeroRegionEPKNS0_9MemRegionENS_8QualTypeE", "clang::ento::StoreManager::GetElementZeroRegion(clang::ento::MemRegion const*, clang::QualType)"},
    {"_ZN5clang4ento12StoreManager10CastRegionEPKNS0_9MemRegionENS_8QualTypeE", "clang::ento::StoreManager::CastRegion(clang::ento::MemRegion const*, clang::QualType)"},
    {"_ZN5clang4ento12StoreManager16CastRetrievedValENS0_4SValEPKNS0_11TypedRegionENS_8QualTypeEb", "clang::ento::StoreManager::CastRetrievedVal(clang::ento::SVal, clang::ento::TypedRegion const*, clang::QualType, bool)"},
    {"_ZN5clang4ento12StoreManager20getLValueFieldOrIvarEPKNS_4DeclENS0_4SValE", "clang::ento::StoreManager::getLValueFieldOrIvar(clang::Decl const*, clang::ento::SVal)"},
    {"_ZN5clang4ento12StoreManager16getLValueElementENS_8QualTypeENS0_6NonLocENS0_4SValE", "clang::ento::StoreManager::getLValueElement(clang::QualType, clang::ento::NonLoc, clang::ento::SVal)"},
    {"_ZNK5clang4ento7SymExpr4dumpEv", "clang::ento::SymExpr::dump() const"},
    {"_ZNK5clang4ento10SymIntExpr12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymIntExpr::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento10SymSymExpr12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymSymExpr::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento14SymbolConjured12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolConjured::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento13SymbolDerived12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolDerived::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento12SymbolExtent12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolExtent::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento14SymbolMetadata12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolMetadata::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZNK5clang4ento17SymbolRegionValue12dumpToStreamERN4llvm11raw_ostreamE", "clang::ento::SymbolRegionValue::dumpToStream(llvm::raw_ostream&) const"},
    {"_ZN5clang4ento13SymbolManager20getRegionValueSymbolEPKNS0_11TypedRegionE", "clang::ento::SymbolManager::getRegionValueSymbol(clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento13SymbolManager17getConjuredSymbolEPKNS_4StmtENS_8QualTypeEjPKv", "clang::ento::SymbolManager::getConjuredSymbol(clang::Stmt const*, clang::QualType, unsigned int, void const*)"},
    {"_ZN5clang4ento13SymbolManager16getDerivedSymbolEPKNS0_10SymbolDataEPKNS0_11TypedRegionE", "clang::ento::SymbolManager::getDerivedSymbol(clang::ento::SymbolData const*, clang::ento::TypedRegion const*)"},
    {"_ZN5clang4ento13SymbolManager15getExtentSymbolEPKNS0_9SubRegionE", "clang::ento::SymbolManager::getExtentSymbol(clang::ento::SubRegion const*)"},
    {"_ZN5clang4ento13SymbolManager17getMetadataSymbolEPKNS0_9MemRegionEPKNS_4StmtENS_8QualTypeEjPKv", "clang::ento::SymbolManager::getMetadataSymbol(clang::ento::MemRegion const*, clang::Stmt const*, clang::QualType, unsigned int, void const*)"},
    {"_ZN5clang4ento13SymbolManager13getSymIntExprEPKNS0_7SymExprENS_18BinaryOperatorKindERKN4llvm6APSIntENS_8QualTypeE", "clang::ento::SymbolManager::getSymIntExpr(clang::ento::SymExpr const*, clang::BinaryOperatorKind, llvm::APSInt const&, clang::QualType)"},
    {"_ZN5clang4ento13SymbolManager13getSymSymExprEPKNS0_7SymExprENS_18BinaryOperatorKindES4_NS_8QualTypeE", "clang::ento::SymbolManager::getSymSymExpr(clang::ento::SymExpr const*, clang::BinaryOperatorKind, clang::ento::SymExpr const*, clang::QualType)"},
    {"_ZNK5clang4ento14SymbolConjured7getTypeERNS_10ASTContextE", "clang::ento::SymbolConjured::getType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento13SymbolDerived7getTypeERNS_10ASTContextE", "clang::ento::SymbolDerived::getType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento12SymbolExtent7getTypeERNS_10ASTContextE", "clang::ento::SymbolExtent::getType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento14SymbolMetadata7getTypeERNS_10ASTContextE", "clang::ento::SymbolMetadata::getType(clang::ASTContext&) const"},
    {"_ZNK5clang4ento17SymbolRegionValue7getTypeERNS_10ASTContextE", "clang::ento::SymbolRegionValue::getType(clang::ASTContext&) const"},
    {"_ZN5clang4ento13SymbolManagerD1Ev", "clang::ento::SymbolManager::~SymbolManager()"},
    {"_ZN5clang4ento13SymbolManagerD2Ev", "clang::ento::SymbolManager::~SymbolManager()"},
    {"_ZN5clang4ento13SymbolManager14canSymbolicateENS_8QualTypeE", "clang::ento::SymbolManager::canSymbolicate(clang::QualType)"},
    {"_ZN5clang4ento12SymbolReaper8markLiveEPKNS0_10SymbolDataE", "clang::ento::SymbolReaper::markLive(clang::ento::SymbolData const*)"},
    {"_ZN5clang4ento12SymbolReaper9markInUseEPKNS0_10SymbolDataE", "clang::ento::SymbolReaper::markInUse(clang::ento::SymbolData const*)"},
    {"_ZN5clang4ento12SymbolReaper9maybeDeadEPKNS0_10SymbolDataE", "clang::ento::SymbolReaper::maybeDead(clang::ento::SymbolData const*)"},
    {"_ZN5clang4ento12SymbolReaper6isLiveEPKNS0_10SymbolDataE", "clang::ento::SymbolReaper::isLive(clang::ento::SymbolData const*)"},
    {"_Z12IsLiveRegionRN5clang4ento12SymbolReaperEPKNS0_9MemRegionE", "IsLiveRegion(clang::ento::SymbolReaper&, clang::ento::MemRegion const*)"},
    {"_ZNK5clang4ento12SymbolReaper6isLiveEPKNS_4StmtE", "clang::ento::SymbolReaper::isLive(clang::Stmt const*) const"},
    {"_ZNK5clang4ento12SymbolReaper6isLiveEPKNS0_9VarRegionE", "clang::ento::SymbolReaper::isLive(clang::ento::VarRegion const*) const"},
    {"_ZN5clang4ento13SymbolVisitorD0Ev", "clang::ento::SymbolVisitor::~SymbolVisitor()"},
    {"_ZN5clang4ento13SymbolVisitorD1Ev", "clang::ento::SymbolVisitor::~SymbolVisitor()"},
    {"_ZN5clang4ento13SymbolVisitorD2Ev", "clang::ento::SymbolVisitor::~SymbolVisitor()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento7SymExprEED1Ev", "llvm::FoldingSet<clang::ento::SymExpr>::~FoldingSet()"},
    {"_ZN5clang4ento17SymbolRegionValueD1Ev", "clang::ento::SymbolRegionValue::~SymbolRegionValue()"},
    {"_ZN5clang4ento17SymbolRegionValueD0Ev", "clang::ento::SymbolRegionValue::~SymbolRegionValue()"},
    {"_ZN5clang4ento17SymbolRegionValue7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolRegionValue::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento14SymbolConjuredD1Ev", "clang::ento::SymbolConjured::~SymbolConjured()"},
    {"_ZN5clang4ento14SymbolConjuredD0Ev", "clang::ento::SymbolConjured::~SymbolConjured()"},
    {"_ZN5clang4ento14SymbolConjured7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolConjured::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento13SymbolDerivedD1Ev", "clang::ento::SymbolDerived::~SymbolDerived()"},
    {"_ZN5clang4ento13SymbolDerivedD0Ev", "clang::ento::SymbolDerived::~SymbolDerived()"},
    {"_ZN5clang4ento13SymbolDerived7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolDerived::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento12SymbolExtentD1Ev", "clang::ento::SymbolExtent::~SymbolExtent()"},
    {"_ZN5clang4ento12SymbolExtentD0Ev", "clang::ento::SymbolExtent::~SymbolExtent()"},
    {"_ZN5clang4ento12SymbolExtent7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolExtent::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento14SymbolMetadataD1Ev", "clang::ento::SymbolMetadata::~SymbolMetadata()"},
    {"_ZN5clang4ento14SymbolMetadataD0Ev", "clang::ento::SymbolMetadata::~SymbolMetadata()"},
    {"_ZN5clang4ento14SymbolMetadata7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymbolMetadata::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento10SymIntExprD1Ev", "clang::ento::SymIntExpr::~SymIntExpr()"},
    {"_ZN5clang4ento10SymIntExprD0Ev", "clang::ento::SymIntExpr::~SymIntExpr()"},
    {"_ZNK5clang4ento10SymIntExpr7getTypeERNS_10ASTContextE", "clang::ento::SymIntExpr::getType(clang::ASTContext&) const"},
    {"_ZN5clang4ento10SymIntExpr7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymIntExpr::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento10SymSymExprD1Ev", "clang::ento::SymSymExpr::~SymSymExpr()"},
    {"_ZN5clang4ento10SymSymExprD0Ev", "clang::ento::SymSymExpr::~SymSymExpr()"},
    {"_ZNK5clang4ento10SymSymExpr7getTypeERNS_10ASTContextE", "clang::ento::SymSymExpr::getType(clang::ASTContext&) const"},
    {"_ZN5clang4ento10SymSymExpr7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ento::SymSymExpr::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang4ento7SymExprD1Ev", "clang::ento::SymExpr::~SymExpr()"},
    {"_ZN5clang4ento7SymExprD0Ev", "clang::ento::SymExpr::~SymExpr()"},
    {"_ZN5clang4ento10SymbolDataD1Ev", "clang::ento::SymbolData::~SymbolData()"},
    {"_ZN5clang4ento10SymbolDataD0Ev", "clang::ento::SymbolData::~SymbolData()"},
    {"_ZN4llvm10FoldingSetIN5clang4ento7SymExprEED0Ev", "llvm::FoldingSet<clang::ento::SymExpr>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7SymExprEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::SymExpr>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7SymExprEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ento::SymExpr>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang4ento7SymExprEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ento::SymExpr>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang4ento30createTextPathDiagnosticClientERKSsRKNS_12PreprocessorE", "clang::ento::createTextPathDiagnosticClient(std::string const&, clang::Preprocessor const&)"},
    {"_ZN12_GLOBAL__N_119TextPathDiagnosticsD1Ev", "(anonymous namespace)::TextPathDiagnostics::~TextPathDiagnostics()"},
    {"_ZN12_GLOBAL__N_119TextPathDiagnosticsD0Ev", "(anonymous namespace)::TextPathDiagnostics::~TextPathDiagnostics()"},
    {"_ZN12_GLOBAL__N_119TextPathDiagnostics16FlushDiagnosticsEPN4llvm15SmallVectorImplISsEE", "(anonymous namespace)::TextPathDiagnostics::FlushDiagnostics(llvm::SmallVectorImpl<std::string>*)"},
    {"_ZNK12_GLOBAL__N_119TextPathDiagnostics7getNameEv", "(anonymous namespace)::TextPathDiagnostics::getName() const"},
    {"_ZN12_GLOBAL__N_119TextPathDiagnostics20HandlePathDiagnosticEPKN5clang4ento14PathDiagnosticE", "(anonymous namespace)::TextPathDiagnostics::HandlePathDiagnostic(clang::ento::PathDiagnostic const*)"},
    {"_ZNK12_GLOBAL__N_119TextPathDiagnostics19getGenerationSchemeEv", "(anonymous namespace)::TextPathDiagnostics::getGenerationScheme() const"},
    {"_ZNK12_GLOBAL__N_119TextPathDiagnostics28supportsLogicalOpControlFlowEv", "(anonymous namespace)::TextPathDiagnostics::supportsLogicalOpControlFlow() const"},
    {"_ZNK12_GLOBAL__N_119TextPathDiagnostics21supportsAllBlockEdgesEv", "(anonymous namespace)::TextPathDiagnostics::supportsAllBlockEdges() const"},
    {"_ZNK12_GLOBAL__N_119TextPathDiagnostics21useVerboseDescriptionEv", "(anonymous namespace)::TextPathDiagnostics::useVerboseDescription() const"},
    {"_ZN5clang4ento20PathDiagnosticClientD1Ev", "clang::ento::PathDiagnosticClient::~PathDiagnosticClient()"},
    {"_ZN5clang4ento20PathDiagnosticClientD0Ev", "clang::ento::PathDiagnosticClient::~PathDiagnosticClient()"},
    {"_ZNK5clang4ento20PathDiagnosticClient19getGenerationSchemeEv", "clang::ento::PathDiagnosticClient::getGenerationScheme() const"},
    {"_ZNK5clang4ento20PathDiagnosticClient28supportsLogicalOpControlFlowEv", "clang::ento::PathDiagnosticClient::supportsLogicalOpControlFlow() const"},
    {"_ZNK5clang4ento20PathDiagnosticClient21supportsAllBlockEdgesEv", "clang::ento::PathDiagnosticClient::supportsAllBlockEdges() const"},
    {"_ZNK5clang4ento20PathDiagnosticClient21useVerboseDescriptionEv", "clang::ento::PathDiagnosticClient::useVerboseDescription() const"},
    {"_ZN5clang22AnalysisContextManager5clearEv", "clang::AnalysisContextManager::clear()"},
    {"_ZN5clang15AnalysisContextD1Ev", "clang::AnalysisContext::~AnalysisContext()"},
    {"_ZN5clang15AnalysisContext7getBodyEv", "clang::AnalysisContext::getBody()"},
    {"_ZNK5clang15AnalysisContext11getSelfDeclEv", "clang::AnalysisContext::getSelfDecl() const"},
    {"_ZN5clang15AnalysisContext6getCFGEv", "clang::AnalysisContext::getCFG()"},
    {"_ZN5clang15AnalysisContext17getUnoptimizedCFGEv", "clang::AnalysisContext::getUnoptimizedCFG()"},
    {"_ZN5clang15AnalysisContext7dumpCFGEv", "clang::AnalysisContext::dumpCFG()"},
    {"_ZN5clang15AnalysisContext12getParentMapEv", "clang::AnalysisContext::getParentMap()"},
    {"_ZN5clang15AnalysisContext25getPseudoConstantAnalysisEv", "clang::AnalysisContext::getPseudoConstantAnalysis()"},
    {"_ZN5clang15AnalysisContext16getLiveVariablesEv", "clang::AnalysisContext::getLiveVariables()"},
    {"_ZN5clang15AnalysisContext23getRelaxedLiveVariablesEv", "clang::AnalysisContext::getRelaxedLiveVariables()"},
    {"_ZN5clang22AnalysisContextManager10getContextEPKNS_4DeclEPNS_3idx15TranslationUnitE", "clang::AnalysisContextManager::getContext(clang::Decl const*, clang::idx::TranslationUnit*)"},
    {"_ZN5clang15LocationContext13ProfileCommonERN4llvm16FoldingSetNodeIDENS0_11ContextKindEPNS_15AnalysisContextEPKS0_PKv", "clang::LocationContext::ProfileCommon(llvm::FoldingSetNodeID&, clang::LocationContext::ContextKind, clang::AnalysisContext*, clang::LocationContext const*, void const*)"},
    {"_ZN5clang17StackFrameContext7ProfileERN4llvm16FoldingSetNodeIDE", "clang::StackFrameContext::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang12ScopeContext7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ScopeContext::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang22BlockInvocationContext7ProfileERN4llvm16FoldingSetNodeIDE", "clang::BlockInvocationContext::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang22LocationContextManager13getStackFrameEPNS_15AnalysisContextEPKNS_15LocationContextEPKNS_4StmtEPKNS_8CFGBlockEj", "clang::LocationContextManager::getStackFrame(clang::AnalysisContext*, clang::LocationContext const*, clang::Stmt const*, clang::CFGBlock const*, unsigned int)"},
    {"_ZN5clang22LocationContextManager8getScopeEPNS_15AnalysisContextEPKNS_15LocationContextEPKNS_4StmtE", "clang::LocationContextManager::getScope(clang::AnalysisContext*, clang::LocationContext const*, clang::Stmt const*)"},
    {"_ZNK5clang15LocationContext20getCurrentStackFrameEv", "clang::LocationContext::getCurrentStackFrame() const"},
    {"_ZNK5clang15LocationContext27getStackFrameForDeclContextEPKNS_11DeclContextE", "clang::LocationContext::getStackFrameForDeclContext(clang::DeclContext const*) const"},
    {"_ZNK5clang15LocationContext10isParentOfEPKS0_", "clang::LocationContext::isParentOf(clang::LocationContext const*) const"},
    {"_ZN5clang15AnalysisContext22getReferencedBlockVarsEPKNS_9BlockDeclE", "clang::AnalysisContext::getReferencedBlockVars(clang::BlockDecl const*)"},
    {"_ZN5clang15AnalysisContextD2Ev", "clang::AnalysisContext::~AnalysisContext()"},
    {"_ZN5clang22AnalysisContextManagerD1Ev", "clang::AnalysisContextManager::~AnalysisContextManager()"},
    {"_ZN5clang22AnalysisContextManagerD2Ev", "clang::AnalysisContextManager::~AnalysisContextManager()"},
    {"_ZN5clang15LocationContextD0Ev", "clang::LocationContext::~LocationContext()"},
    {"_ZN5clang15LocationContextD1Ev", "clang::LocationContext::~LocationContext()"},
    {"_ZN5clang15LocationContextD2Ev", "clang::LocationContext::~LocationContext()"},
    {"_ZN5clang22LocationContextManagerD1Ev", "clang::LocationContextManager::~LocationContextManager()"},
    {"_ZN5clang22LocationContextManagerD2Ev", "clang::LocationContextManager::~LocationContextManager()"},
    {"_ZN5clang22LocationContextManager5clearEv", "clang::LocationContextManager::clear()"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_125FindBlockDeclRefExprsValsEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::FindBlockDeclRefExprsVals, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_125FindBlockDeclRefExprsVals9VisitStmtEPN5clang4StmtE", "(anonymous namespace)::FindBlockDeclRefExprsVals::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang22LocationContextManager18getLocationContextINS_12ScopeContextENS_4StmtEEEPKT_PNS_15AnalysisContextEPKNS_15LocationContextEPKT0_", "clang::ScopeContext const* clang::LocationContextManager::getLocationContext<clang::ScopeContext, clang::Stmt>(clang::AnalysisContext*, clang::LocationContext const*, clang::Stmt const*)"},
    {"_ZN5clang17StackFrameContextD1Ev", "clang::StackFrameContext::~StackFrameContext()"},
    {"_ZN5clang17StackFrameContextD0Ev", "clang::StackFrameContext::~StackFrameContext()"},
    {"_ZN5clang12ScopeContextD1Ev", "clang::ScopeContext::~ScopeContext()"},
    {"_ZN5clang12ScopeContextD0Ev", "clang::ScopeContext::~ScopeContext()"},
    {"_ZN5clang22BlockInvocationContextD1Ev", "clang::BlockInvocationContext::~BlockInvocationContext()"},
    {"_ZN5clang22BlockInvocationContextD0Ev", "clang::BlockInvocationContext::~BlockInvocationContext()"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEPvNS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::BlockDecl const*, void*, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<void*> >::InsertIntoBucket(clang::BlockDecl const* const&, void* const&, std::pair<clang::BlockDecl const*, void*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEPvNS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::BlockDecl const*, void*, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<void*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS1_15AnalysisContextENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Decl const*, clang::AnalysisContext*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<clang::AnalysisContext*> >::InsertIntoBucket(clang::Decl const* const&, clang::AnalysisContext* const&, std::pair<clang::Decl const*, clang::AnalysisContext*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS1_15AnalysisContextENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Decl const*, clang::AnalysisContext*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<clang::AnalysisContext*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS1_15AnalysisContextENS_12DenseMapInfoIS4_EENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<clang::Decl const*, clang::AnalysisContext*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<clang::AnalysisContext*> >::shrink_and_clear()"},
    {"_ZN5clang14DataflowValuesINS_24LiveVariables_ValueTypesENS_8dataflow21backward_analysis_tagEED2Ev", "clang::DataflowValues<clang::LiveVariables_ValueTypes, clang::dataflow::backward_analysis_tag>::~DataflowValues()"},
    {"_ZN5clang24LiveVariables_ValueTypes14AnalysisDataTyD1Ev", "clang::LiveVariables_ValueTypes::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEED2Ev", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::~DenseMap()"},
    {"_ZN5clang24LiveVariables_ValueTypes14AnalysisDataTyD0Ev", "clang::LiveVariables_ValueTypes::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang23StmtDeclBitVector_Types14AnalysisDataTyD1Ev", "clang::StmtDeclBitVector_Types::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang23StmtDeclBitVector_Types14AnalysisDataTyD0Ev", "clang::StmtDeclBitVector_Types::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang19DeclBitVector_Types14AnalysisDataTyD1Ev", "clang::DeclBitVector_Types::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang19DeclBitVector_Types14AnalysisDataTyD0Ev", "clang::DeclBitVector_Types::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::VarDecl const*, unsigned int, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::VarDecl const* const&, unsigned int const&, std::pair<clang::VarDecl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::VarDecl const*, unsigned int, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang3CFG11createBlockEv", "clang::CFG::createBlock()"},
    {"_ZN5clang3CFG8buildCFGEPKNS_4DeclEPNS_4StmtEPNS_10ASTContextENS0_12BuildOptionsE", "clang::CFG::buildCFG(clang::Decl const*, clang::Stmt*, clang::ASTContext*, clang::CFG::BuildOptions)"},
    {"_ZN5clang3CFG13getBlkExprNumEPKNS_4StmtE", "clang::CFG::getBlkExprNum(clang::Stmt const*)"},
    {"_Z18PopulateBlkExprMapRN5clang3CFGE", "PopulateBlkExprMap(clang::CFG&)"},
    {"_ZN5clang3CFG14getNumBlkExprsEv", "clang::CFG::getNumBlkExprs()"},
    {"_ZN5clang8CFGBlock10FilterEdgeERKNS0_13FilterOptionsEPKS0_S5_", "clang::CFGBlock::FilterEdge(clang::CFGBlock::FilterOptions const&, clang::CFGBlock const*, clang::CFGBlock const*)"},
    {"_ZN5clang3CFGD1Ev", "clang::CFG::~CFG()"},
    {"_ZN5clang3CFGD2Ev", "clang::CFG::~CFG()"},
    {"_ZNK5clang3CFG4dumpERKNS_11LangOptionsE", "clang::CFG::dump(clang::LangOptions const&) const"},
    {"_ZNK5clang3CFG5printERN4llvm11raw_ostreamERKNS_11LangOptionsE", "clang::CFG::print(llvm::raw_ostream&, clang::LangOptions const&) const"},
    {"_ZN12_GLOBAL__N_117StmtPrinterHelperC1EPKN5clang3CFGERKNS1_11LangOptionsE", "(anonymous namespace)::StmtPrinterHelper::StmtPrinterHelper(clang::CFG const*, clang::LangOptions const&)"},
    {"_Z11print_blockRN4llvm11raw_ostreamEPKN5clang3CFGERKNS2_8CFGBlockEPN12_GLOBAL__N_117StmtPrinterHelperEb", "print_block(llvm::raw_ostream&, clang::CFG const*, clang::CFGBlock const&, (anonymous namespace)::StmtPrinterHelper*, bool)"},
    {"_ZN12_GLOBAL__N_117StmtPrinterHelperD1Ev", "(anonymous namespace)::StmtPrinterHelper::~StmtPrinterHelper()"},
    {"_ZNK5clang8CFGBlock4dumpEPKNS_3CFGERKNS_11LangOptionsE", "clang::CFGBlock::dump(clang::CFG const*, clang::LangOptions const&) const"},
    {"_ZNK5clang8CFGBlock5printERN4llvm11raw_ostreamEPKNS_3CFGERKNS_11LangOptionsE", "clang::CFGBlock::print(llvm::raw_ostream&, clang::CFG const*, clang::LangOptions const&) const"},
    {"_ZNK5clang8CFGBlock15printTerminatorERN4llvm11raw_ostreamERKNS_11LangOptionsE", "clang::CFGBlock::printTerminator(llvm::raw_ostream&, clang::LangOptions const&) const"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_123CFGBlockTerminatorPrintEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::CFGBlockTerminatorPrint, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang8CFGBlock22getTerminatorConditionEv", "clang::CFGBlock::getTerminatorCondition()"},
    {"_ZNK5clang8CFGBlock25hasBinaryBranchTerminatorEv", "clang::CFGBlock::hasBinaryBranchTerminator() const"},
    {"_ZNK5clang3CFG7viewCFGERKNS_11LangOptionsE", "clang::CFG::viewCFG(clang::LangOptions const&) const"},
    {"_ZN12_GLOBAL__N_123CFGBlockTerminatorPrint19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::CFGBlockTerminatorPrint::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_117StmtPrinterHelperD0Ev", "(anonymous namespace)::StmtPrinterHelper::~StmtPrinterHelper()"},
    {"_ZN12_GLOBAL__N_117StmtPrinterHelper11handledStmtEPN5clang4StmtERN4llvm11raw_ostreamE", "(anonymous namespace)::StmtPrinterHelper::handledStmt(clang::Stmt*, llvm::raw_ostream&)"},
    {"_Z22FindSubExprAssignmentsPN5clang4StmtERN4llvm11SmallPtrSetIPNS_4ExprELj50EEE", "FindSubExprAssignments(clang::Stmt*, llvm::SmallPtrSet<clang::Expr*, 50u>&)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder22VisitForTemporaryDtorsEPN5clang4StmtEb", "(anonymous namespace)::CFGBuilder::VisitForTemporaryDtors(clang::Stmt*, bool)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder5VisitEPN5clang4StmtENS_13AddStmtChoiceE", "(anonymous namespace)::CFGBuilder::Visit(clang::Stmt*, (anonymous namespace)::AddStmtChoice)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder17VisitCompoundStmtEPN5clang12CompoundStmtE", "(anonymous namespace)::CFGBuilder::VisitCompoundStmt(clang::CompoundStmt*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder17VisitCXXCatchStmtEPN5clang12CXXCatchStmtE", "(anonymous namespace)::CFGBuilder::VisitCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder23addLocalScopeForVarDeclEPN5clang7VarDeclEPNS_10LocalScopeE", "(anonymous namespace)::CFGBuilder::addLocalScopeForVarDecl(clang::VarDecl*, (anonymous namespace)::LocalScope*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder20addAutomaticObjDtorsENS_10LocalScope14const_iteratorES2_PN5clang4StmtE", "(anonymous namespace)::CFGBuilder::addAutomaticObjDtors((anonymous namespace)::LocalScope::const_iterator, (anonymous namespace)::LocalScope::const_iterator, clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder20addLocalScopeForStmtEPN5clang4StmtE", "(anonymous namespace)::CFGBuilder::addLocalScopeForStmt(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder16VisitDeclSubExprEPN5clang8DeclStmtE", "(anonymous namespace)::CFGBuilder::VisitDeclSubExpr(clang::DeclStmt*)"},
    {"_ZN12_GLOBAL__N_110CFGBuilder13VisitChildrenEPN5clang4StmtE", "(anonymous namespace)::CFGBuilder::VisitChildren(clang::Stmt*)"},
    {"_ZN4llvm14format_object1IjED1Ev", "llvm::format_object1<unsigned int>::~format_object1()"},
    {"_ZNK4llvm14format_object1IjE7snprintEPcj", "llvm::format_object1<unsigned int>::snprint(char*, unsigned int) const"},
    {"_ZN4llvm14format_object1IjED0Ev", "llvm::format_object1<unsigned int>::~format_object1()"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclESt4pairIjjENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PS4_IS3_S5_E", "llvm::DenseMap<clang::Decl*, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::InsertIntoBucket(clang::Decl* const&, std::pair<unsigned int, unsigned int> const&, std::pair<clang::Decl*, std::pair<unsigned int, unsigned int> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclESt4pairIjjENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Decl*, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtESt4pairIjjENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PS4_IS3_S5_E", "llvm::DenseMap<clang::Stmt*, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::InsertIntoBucket(clang::Stmt* const&, std::pair<unsigned int, unsigned int> const&, std::pair<clang::Stmt*, std::pair<unsigned int, unsigned int> >*)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtESt4pairIjjENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Stmt*, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4StmtEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::Stmt const*, unsigned int, llvm::DenseMapInfo<clang::Stmt const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::Stmt const* const&, unsigned int const&, std::pair<clang::Stmt const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4StmtEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::Stmt const*, unsigned int, llvm::DenseMapInfo<clang::Stmt const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm9OwningPtrIN5clang3CFGEED2Ev", "llvm::OwningPtr<clang::CFG>::~OwningPtr()"},
    {"_ZN5clang8CFGBlock12addSuccessorEPS0_RNS_17BumpVectorContextE", "clang::CFGBlock::addSuccessor(clang::CFGBlock*, clang::BumpVectorContext&)"},
    {"_ZN5clang10BumpVectorINS_10CFGElementEE6insertEPS1_mRKS1_RNS_17BumpVectorContextE", "clang::BumpVector<clang::CFGElement>::insert(clang::CFGElement*, unsigned long, clang::CFGElement const&, clang::BumpVectorContext&)"},
    {"_ZN5clang10BumpVectorINS_10CFGElementEE9push_backERKS1_RNS_17BumpVectorContextE", "clang::BumpVector<clang::CFGElement>::push_back(clang::CFGElement const&, clang::BumpVectorContext&)"},
    {"_ZN5clang8CFGBlockC2EjRNS_17BumpVectorContextE", "clang::CFGBlock::CFGBlock(unsigned int, clang::BumpVectorContext&)"},
    {"_ZN5clang10BumpVectorINS_10CFGElementEE7reserveERNS_17BumpVectorContextEj", "clang::BumpVector<clang::CFGElement>::reserve(clang::BumpVectorContext&, unsigned int)"},
    {"_ZN5clang10CFGStmtMapD1Ev", "clang::CFGStmtMap::~CFGStmtMap()"},
    {"_ZN5clang10CFGStmtMapD2Ev", "clang::CFGStmtMap::~CFGStmtMap()"},
    {"_ZN5clang10CFGStmtMap8getBlockEPNS_4StmtE", "clang::CFGStmtMap::getBlock(clang::Stmt*)"},
    {"_ZN5clang10CFGStmtMap5BuildEPNS_3CFGEPNS_9ParentMapE", "clang::CFGStmtMap::Build(clang::CFG*, clang::ParentMap*)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtEPNS1_8CFGBlockENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::Stmt*, clang::CFGBlock*, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<clang::CFGBlock*> >::InsertIntoBucket(clang::Stmt* const&, clang::CFGBlock* const&, std::pair<clang::Stmt*, clang::CFGBlock*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtEPNS1_8CFGBlockENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Stmt*, clang::CFGBlock*, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<clang::CFGBlock*> >::grow(unsigned int)"},
    {"_ZN5clang4ento5cocoa22deriveNamingConventionENS_8SelectorEb", "clang::ento::cocoa::deriveNamingConvention(clang::Selector, bool)"},
    {"_Z9parseWordPKc", "parseWord(char const*)"},
    {"_ZN5clang4ento5cocoa9isRefTypeENS_8QualTypeEN4llvm9StringRefES4_", "clang::ento::cocoa::isRefType(clang::QualType, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang4ento5cocoa13isCFObjectRefENS_8QualTypeE", "clang::ento::cocoa::isCFObjectRef(clang::QualType)"},
    {"_ZN5clang4ento5cocoa16isCocoaObjectRefENS_8QualTypeE", "clang::ento::cocoa::isCocoaObjectRef(clang::QualType)"},
    {"_ZN5clang21analyze_format_string19FormatStringHandlerD0Ev", "clang::analyze_format_string::FormatStringHandler::~FormatStringHandler()"},
    {"_ZN5clang21analyze_format_string19FormatStringHandlerD1Ev", "clang::analyze_format_string::FormatStringHandler::~FormatStringHandler()"},
    {"_ZN5clang21analyze_format_string19FormatStringHandlerD2Ev", "clang::analyze_format_string::FormatStringHandler::~FormatStringHandler()"},
    {"_ZN5clang21analyze_format_string11ParseAmountERPKcS2_", "clang::analyze_format_string::ParseAmount(char const*&, char const*)"},
    {"_ZN5clang21analyze_format_string22ParseNonPositionAmountERPKcS2_Rj", "clang::analyze_format_string::ParseNonPositionAmount(char const*&, char const*, unsigned int&)"},
    {"_ZN5clang21analyze_format_string19ParsePositionAmountERNS0_19FormatStringHandlerEPKcRS4_S4_NS0_15PositionContextE", "clang::analyze_format_string::ParsePositionAmount(clang::analyze_format_string::FormatStringHandler&, char const*, char const*&, char const*, clang::analyze_format_string::PositionContext)"},
    {"_ZN5clang21analyze_format_string15ParseFieldWidthERNS0_19FormatStringHandlerERNS0_15FormatSpecifierEPKcRS6_S6_Pj", "clang::analyze_format_string::ParseFieldWidth(clang::analyze_format_string::FormatStringHandler&, clang::analyze_format_string::FormatSpecifier&, char const*, char const*&, char const*, unsigned int*)"},
    {"_ZN5clang21analyze_format_string16ParseArgPositionERNS0_19FormatStringHandlerERNS0_15FormatSpecifierEPKcRS6_S6_", "clang::analyze_format_string::ParseArgPosition(clang::analyze_format_string::FormatStringHandler&, clang::analyze_format_string::FormatSpecifier&, char const*, char const*&, char const*)"},
    {"_ZN5clang21analyze_format_string19ParseLengthModifierERNS0_15FormatSpecifierERPKcS4_", "clang::analyze_format_string::ParseLengthModifier(clang::analyze_format_string::FormatSpecifier&, char const*&, char const*)"},
    {"_ZNK5clang21analyze_format_string13ArgTypeResult11matchesTypeERNS_10ASTContextENS_8QualTypeE", "clang::analyze_format_string::ArgTypeResult::matchesType(clang::ASTContext&, clang::QualType) const"},
    {"_ZNK5clang21analyze_format_string13ArgTypeResult21getRepresentativeTypeERNS_10ASTContextE", "clang::analyze_format_string::ArgTypeResult::getRepresentativeType(clang::ASTContext&) const"},
    {"_ZNK5clang21analyze_format_string14OptionalAmount10getArgTypeERNS_10ASTContextE", "clang::analyze_format_string::OptionalAmount::getArgType(clang::ASTContext&) const"},
    {"_ZNK5clang21analyze_format_string14LengthModifier8toStringEv", "clang::analyze_format_string::LengthModifier::toString() const"},
    {"_ZNK5clang21analyze_format_string14OptionalAmount8toStringERN4llvm11raw_ostreamE", "clang::analyze_format_string::OptionalAmount::toString(llvm::raw_ostream&) const"},
    {"_ZNK5clang21analyze_format_string15FormatSpecifier22hasValidLengthModifierEv", "clang::analyze_format_string::FormatSpecifier::hasValidLengthModifier() const"},
    {"_ZN5clang13LiveVariablesC1ERNS_15AnalysisContextEb", "clang::LiveVariables::LiveVariables(clang::AnalysisContext&, bool)"},
    {"_ZN5clang13LiveVariablesC2ERNS_15AnalysisContextEb", "clang::LiveVariables::LiveVariables(clang::AnalysisContext&, bool)"},
    {"_ZN5clang13LiveVariables8runOnCFGERNS_3CFGE", "clang::LiveVariables::runOnCFG(clang::CFG&)"},
    {"_ZN5clang13LiveVariables14runOnAllBlocksERKNS_3CFGEPNS_24LiveVariables_ValueTypes10ObserverTyEb", "clang::LiveVariables::runOnAllBlocks(clang::CFG const&, clang::LiveVariables_ValueTypes::ObserverTy*, bool)"},
    {"_ZNK5clang13LiveVariables6isLiveEPKNS_8CFGBlockEPKNS_7VarDeclE", "clang::LiveVariables::isLive(clang::CFGBlock const*, clang::VarDecl const*) const"},
    {"_ZNK5clang13LiveVariables6isLiveERKNS_23StmtDeclBitVector_Types5ValTyEPKNS_7VarDeclE", "clang::LiveVariables::isLive(clang::StmtDeclBitVector_Types::ValTy const&, clang::VarDecl const*) const"},
    {"_ZNK5clang13LiveVariables6isLiveEPKNS_4StmtES3_", "clang::LiveVariables::isLive(clang::Stmt const*, clang::Stmt const*) const"},
    {"_ZNK5clang13LiveVariables6isLiveEPKNS_4StmtEPKNS_7VarDeclE", "clang::LiveVariables::isLive(clang::Stmt const*, clang::VarDecl const*) const"},
    {"_ZNK5clang13LiveVariables12dumpLivenessERKNS_23StmtDeclBitVector_Types5ValTyERKNS_13SourceManagerE", "clang::LiveVariables::dumpLiveness(clang::StmtDeclBitVector_Types::ValTy const&, clang::SourceManager const&) const"},
    {"_ZNK5clang13LiveVariables17dumpBlockLivenessERKNS_13SourceManagerE", "clang::LiveVariables::dumpBlockLiveness(clang::SourceManager const&) const"},
    {"_ZN5clang14DataflowSolverINS_13LiveVariablesEN12_GLOBAL__N_113TransferFuncsENS_23StmtDeclBitVector_Types5UnionESt8equal_toINS4_5ValTyEEE12ProcessBlockEPKNS_8CFGBlockEbNS_8dataflow21backward_analysis_tagE", "clang::DataflowSolver<clang::LiveVariables, (anonymous namespace)::TransferFuncs, clang::StmtDeclBitVector_Types::Union, std::equal_to<clang::StmtDeclBitVector_Types::ValTy> >::ProcessBlock(clang::CFGBlock const*, bool, clang::dataflow::backward_analysis_tag)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs5VisitEPN5clang4StmtE", "(anonymous namespace)::TransferFuncs::Visit(clang::Stmt*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_113TransferFuncsEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::TransferFuncs, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::TransferFuncs::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_113TransferFuncsEvE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::TransferFuncs, void>::VisitChildren(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs11VisitAssignEPN5clang14BinaryOperatorE", "(anonymous namespace)::TransferFuncs::VisitAssign(clang::BinaryOperator*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_113RegisterDeclsEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::RegisterDecls, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_113RegisterDeclsEvE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::RegisterDecls, void>::VisitChildren(clang::Stmt*)"},
    {"_ZN5clang21CFGRecStmtDeclVisitorIN12_GLOBAL__N_113RegisterDeclsEE9VisitDeclEPNS_4DeclE", "clang::CFGRecStmtDeclVisitor<(anonymous namespace)::RegisterDecls>::VisitDecl(clang::Decl*)"},
    {"_ZN5clang14DataflowValuesINS_24LiveVariables_ValueTypesENS_8dataflow21backward_analysis_tagEEC2Ev", "clang::DataflowValues<clang::LiveVariables_ValueTypes, clang::dataflow::backward_analysis_tag>::DataflowValues()"},
    {"_ZN5clang21analyze_format_string17ParsePrintfStringERNS0_19FormatStringHandlerEPKcS4_", "clang::analyze_format_string::ParsePrintfString(clang::analyze_format_string::FormatStringHandler&, char const*, char const*)"},
    {"_ZNK5clang21analyze_format_string19ConversionSpecifier8toStringEv", "clang::analyze_format_string::ConversionSpecifier::toString() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier10getArgTypeERNS_10ASTContextE", "clang::analyze_printf::PrintfSpecifier::getArgType(clang::ASTContext&) const"},
    {"_ZN5clang14analyze_printf15PrintfSpecifier7fixTypeENS_8QualTypeE", "clang::analyze_printf::PrintfSpecifier::fixType(clang::QualType)"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier8toStringERN4llvm11raw_ostreamE", "clang::analyze_printf::PrintfSpecifier::toString(llvm::raw_ostream&) const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier18hasValidPlusPrefixEv", "clang::analyze_printf::PrintfSpecifier::hasValidPlusPrefix() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier23hasValidAlternativeFormEv", "clang::analyze_printf::PrintfSpecifier::hasValidAlternativeForm() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier20hasValidLeadingZerosEv", "clang::analyze_printf::PrintfSpecifier::hasValidLeadingZeros() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier19hasValidSpacePrefixEv", "clang::analyze_printf::PrintfSpecifier::hasValidSpacePrefix() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier21hasValidLeftJustifiedEv", "clang::analyze_printf::PrintfSpecifier::hasValidLeftJustified() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier31hasValidThousandsGroupingPrefixEv", "clang::analyze_printf::PrintfSpecifier::hasValidThousandsGroupingPrefix() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier17hasValidPrecisionEv", "clang::analyze_printf::PrintfSpecifier::hasValidPrecision() const"},
    {"_ZNK5clang14analyze_printf15PrintfSpecifier18hasValidFieldWidthEv", "clang::analyze_printf::PrintfSpecifier::hasValidFieldWidth() const"},
    {"_ZN5clang22PseudoConstantAnalysisC1EPKNS_4StmtE", "clang::PseudoConstantAnalysis::PseudoConstantAnalysis(clang::Stmt const*)"},
    {"_ZN5clang22PseudoConstantAnalysisC2EPKNS_4StmtE", "clang::PseudoConstantAnalysis::PseudoConstantAnalysis(clang::Stmt const*)"},
    {"_ZN5clang22PseudoConstantAnalysisD1Ev", "clang::PseudoConstantAnalysis::~PseudoConstantAnalysis()"},
    {"_ZN5clang22PseudoConstantAnalysisD2Ev", "clang::PseudoConstantAnalysis::~PseudoConstantAnalysis()"},
    {"_ZN5clang22PseudoConstantAnalysis16isPseudoConstantEPKNS_7VarDeclE", "clang::PseudoConstantAnalysis::isPseudoConstant(clang::VarDecl const*)"},
    {"_ZN5clang22PseudoConstantAnalysis11RunAnalysisEv", "clang::PseudoConstantAnalysis::RunAnalysis()"},
    {"_ZN5clang22PseudoConstantAnalysis13wasReferencedEPKNS_7VarDeclE", "clang::PseudoConstantAnalysis::wasReferenced(clang::VarDecl const*)"},
    {"_ZN5clang14reachable_code22ScanReachableFromBlockERKNS_8CFGBlockERN4llvm9BitVectorE", "clang::reachable_code::ScanReachableFromBlock(clang::CFGBlock const&, llvm::BitVector&)"},
    {"_ZN5clang14reachable_code19FindUnreachableCodeERNS_15AnalysisContextERNS0_8CallbackE", "clang::reachable_code::FindUnreachableCode(clang::AnalysisContext&, clang::reachable_code::Callback&)"},
    {"_Z17GetUnreachableLocRKN5clang8CFGBlockERNS_11SourceRangeES4_", "GetUnreachableLoc(clang::CFGBlock const&, clang::SourceRange&, clang::SourceRange&)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_16ErrLocEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::ErrLoc>::push_back((anonymous namespace)::ErrLoc const&)"},
    {"_Z7LineCmpPKvS0_", "LineCmp(void const*, void const*)"},
    {"_ZN5clang21analyze_format_string16ParseScanfStringERNS0_19FormatStringHandlerEPKcS4_", "clang::analyze_format_string::ParseScanfString(clang::analyze_format_string::FormatStringHandler&, char const*, char const*)"},
    {"_ZN5clang19UninitializedValues16InitializeValuesERKNS_3CFGE", "clang::UninitializedValues::InitializeValues(clang::CFG const&)"},
    {"_ZN5clang30UninitializedValues_ValueTypes10ObserverTyD0Ev", "clang::UninitializedValues_ValueTypes::ObserverTy::~ObserverTy()"},
    {"_ZN5clang30UninitializedValues_ValueTypes10ObserverTyD1Ev", "clang::UninitializedValues_ValueTypes::ObserverTy::~ObserverTy()"},
    {"_ZN5clang30UninitializedValues_ValueTypes10ObserverTyD2Ev", "clang::UninitializedValues_ValueTypes::ObserverTy::~ObserverTy()"},
    {"_ZN5clang24CheckUninitializedValuesERNS_3CFGERNS_10ASTContextERNS_10DiagnosticEb", "clang::CheckUninitializedValues(clang::CFG&, clang::ASTContext&, clang::Diagnostic&, bool)"},
    {"_ZN12_GLOBAL__N_126UninitializedValuesCheckerD1Ev", "(anonymous namespace)::UninitializedValuesChecker::~UninitializedValuesChecker()"},
    {"_ZN5clang14DataflowSolverINS_19UninitializedValuesEN12_GLOBAL__N_113TransferFuncsENS_23StmtDeclBitVector_Types5UnionESt8equal_toINS4_5ValTyEEE12ProcessBlockEPKNS_8CFGBlockEbNS_8dataflow20forward_analysis_tagE", "clang::DataflowSolver<clang::UninitializedValues, (anonymous namespace)::TransferFuncs, clang::StmtDeclBitVector_Types::Union, std::equal_to<clang::StmtDeclBitVector_Types::ValTy> >::ProcessBlock(clang::CFGBlock const*, bool, clang::dataflow::forward_analysis_tag)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs19BlockStmt_VisitExprEPN5clang4ExprE", "(anonymous namespace)::TransferFuncs::BlockStmt_VisitExpr(clang::Expr*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_113TransferFuncsEbE5VisitEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::TransferFuncs, bool>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::TransferFuncs::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::TransferFuncs::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN12_GLOBAL__N_113TransferFuncs9VisitStmtEPN5clang4StmtE", "(anonymous namespace)::TransferFuncs::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_113TransferFuncsEbE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::TransferFuncs, bool>::VisitChildren(clang::Stmt*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_113RegisterDeclsEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::RegisterDecls, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_113RegisterDeclsEvE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::RegisterDecls, void>::VisitChildren(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_126UninitializedValuesCheckerD0Ev", "(anonymous namespace)::UninitializedValuesChecker::~UninitializedValuesChecker()"},
    {"_ZN12_GLOBAL__N_126UninitializedValuesChecker18ObserveDeclRefExprERN5clang23StmtDeclBitVector_Types5ValTyERNS1_30UninitializedValues_ValueTypes14AnalysisDataTyEPNS1_11DeclRefExprEPNS1_7VarDeclE", "(anonymous namespace)::UninitializedValuesChecker::ObserveDeclRefExpr(clang::StmtDeclBitVector_Types::ValTy&, clang::UninitializedValues_ValueTypes::AnalysisDataTy&, clang::DeclRefExpr*, clang::VarDecl*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4StmtENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::Stmt const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::Stmt const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::FindAndConstruct(clang::Stmt const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang4StmtENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::Stmt const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::Stmt const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::InsertIntoBucket(clang::Stmt const* const&, clang::StmtDeclBitVector_Types::ValTy const&, std::pair<clang::Stmt const*, clang::StmtDeclBitVector_Types::ValTy>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4StmtENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::Stmt const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::Stmt const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::grow(unsigned int)"},
    {"_ZN5clang18DataflowWorkListTy7dequeueEv", "clang::DataflowWorkListTy::dequeue()"},
    {"_ZN4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEE4findERKS2_", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::find(clang::ProgramPoint const&)"},
    {"_ZN5clang18DataflowWorkListTy7enqueueEPKNS_8CFGBlockE", "clang::DataflowWorkListTy::enqueue(clang::CFGBlock const*)"},
    {"_ZN4llvm8DenseMapIPKN5clang8CFGBlockEhNS_12DenseMapInfoIS4_EENS5_IhEEE16InsertIntoBucketERKS4_RKhPSt4pairIS4_hE", "llvm::DenseMap<clang::CFGBlock const*, unsigned char, llvm::DenseMapInfo<clang::CFGBlock const*>, llvm::DenseMapInfo<unsigned char> >::InsertIntoBucket(clang::CFGBlock const* const&, unsigned char const&, std::pair<clang::CFGBlock const*, unsigned char>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang8CFGBlockEhNS_12DenseMapInfoIS4_EENS5_IhEEE4growEj", "llvm::DenseMap<clang::CFGBlock const*, unsigned char, llvm::DenseMapInfo<clang::CFGBlock const*>, llvm::DenseMapInfo<unsigned char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEE16FindAndConstructERKS2_", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::FindAndConstruct(clang::ProgramPoint const&)"},
    {"_ZNK4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEE15LookupBucketForERKS2_RPSt4pairIS2_S4_E", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::LookupBucketFor(clang::ProgramPoint const&, std::pair<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy>*&) const"},
    {"_ZN4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::InsertIntoBucket(clang::ProgramPoint const&, clang::StmtDeclBitVector_Types::ValTy const&, std::pair<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy>*)"},
    {"_ZN4llvm8DenseMapIN5clang12ProgramPointENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<clang::ProgramPoint, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::ProgramPoint>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::grow(unsigned int)"},
    {"_ZNK5clang12ProgramPoint12getHashValueEv", "clang::ProgramPoint::getHashValue() const"},
    {"_ZN4llvm8DenseMapIPKN5clang8CFGBlockENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE6insertERKSt4pairIS4_S6_E", "llvm::DenseMap<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::CFGBlock const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::insert(std::pair<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy> const&)"},
    {"_ZNSt4pairIPKN5clang8CFGBlockENS0_23StmtDeclBitVector_Types5ValTyEEC2ERKS3_RKS5_", "std::pair<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy>::pair(clang::CFGBlock const* const&, clang::StmtDeclBitVector_Types::ValTy const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang8CFGBlockENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::CFGBlock const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::InsertIntoBucket(clang::CFGBlock const* const&, clang::StmtDeclBitVector_Types::ValTy const&, std::pair<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang8CFGBlockENS1_23StmtDeclBitVector_Types5ValTyENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::CFGBlock const*, clang::StmtDeclBitVector_Types::ValTy, llvm::DenseMapInfo<clang::CFGBlock const*>, llvm::DenseMapInfo<clang::StmtDeclBitVector_Types::ValTy> >::grow(unsigned int)"},
    {"_ZN5clang23StmtDeclBitVector_Types5ValTyoRERKS1_", "clang::StmtDeclBitVector_Types::ValTy::operator|=(clang::StmtDeclBitVector_Types::ValTy const&)"},
    {"_ZN5clang19DeclBitVector_Types14AnalysisDataTy8RegisterEPKNS_9NamedDeclE", "clang::DeclBitVector_Types::AnalysisDataTy::Register(clang::NamedDecl const*)"},
    {"_ZN5clang14DataflowValuesINS_30UninitializedValues_ValueTypesENS_8dataflow20forward_analysis_tagEED2Ev", "clang::DataflowValues<clang::UninitializedValues_ValueTypes, clang::dataflow::forward_analysis_tag>::~DataflowValues()"},
    {"_ZN5clang30UninitializedValues_ValueTypes14AnalysisDataTyD1Ev", "clang::UninitializedValues_ValueTypes::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang30UninitializedValues_ValueTypes14AnalysisDataTyD0Ev", "clang::UninitializedValues_ValueTypes::AnalysisDataTy::~AnalysisDataTy()"},
    {"_ZN5clang14DataflowValuesINS_30UninitializedValues_ValueTypesENS_8dataflow20forward_analysis_tagEEC2Ev", "clang::DataflowValues<clang::UninitializedValues_ValueTypes, clang::dataflow::forward_analysis_tag>::DataflowValues()"},
    {"_ZN5clang33runUninitializedVariablesAnalysisERKNS_11DeclContextERKNS_3CFGERNS_15AnalysisContextERNS_22UninitVariablesHandlerE", "clang::runUninitializedVariablesAnalysis(clang::DeclContext const&, clang::CFG const&, clang::AnalysisContext&, clang::UninitVariablesHandler&)"},
    {"_Z10runOnBlockPKN5clang8CFGBlockERKNS_3CFGERNS_15AnalysisContextERN12_GLOBAL__N_114CFGBlockValuesEPNS_22UninitVariablesHandlerEb", "runOnBlock(clang::CFGBlock const*, clang::CFG const&, clang::AnalysisContext&, (anonymous namespace)::CFGBlockValues&, clang::UninitVariablesHandler*, bool)"},
    {"_ZN5clang22UninitVariablesHandlerD0Ev", "clang::UninitVariablesHandler::~UninitVariablesHandler()"},
    {"_ZN5clang22UninitVariablesHandlerD1Ev", "clang::UninitVariablesHandler::~UninitVariablesHandler()"},
    {"_ZN5clang22UninitVariablesHandlerD2Ev", "clang::UninitVariablesHandler::~UninitVariablesHandler()"},
    {"_Z25getLogicalOperatorInChainPKN5clang8CFGBlockE", "getLogicalOperatorInChain(clang::CFGBlock const*)"},
    {"_ZN12_GLOBAL__N_114CFGBlockValues12getBitVectorEPKN5clang8CFGBlockES4_", "(anonymous namespace)::CFGBlockValues::getBitVector(clang::CFGBlock const*, clang::CFGBlock const*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_117TransferFunctionsEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::TransferFunctions, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_117TransferFunctions19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::TransferFunctions::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_117TransferFunctions18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::TransferFunctions::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN5clang14CFGStmtVisitorIN12_GLOBAL__N_117TransferFunctionsEvE13VisitChildrenEPNS_4StmtE", "clang::CFGStmtVisitor<(anonymous namespace)::TransferFunctions, void>::VisitChildren(clang::Stmt*)"},
    {"_Z12isTrackedVarPKN5clang7VarDeclEPKNS_11DeclContextE", "isTrackedVar(clang::VarDecl const*, clang::DeclContext const*)"},
    {"_ZN12_GLOBAL__N_117TransferFunctions13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::TransferFunctions::VisitCastExpr(clang::CastExpr*)"},
    {"_ZNK4llvm9BitVectoreqERKS0_", "llvm::BitVector::operator==(llvm::BitVector const&) const"},
    {"_ZNK5clang4Decl7getAttrINS_10BlocksAttrEEEPT_v", "clang::BlocksAttr* clang::Decl::getAttr<clang::BlocksAttr>() const"},
    {"_ZN5clang3idx12EntityGetter9getEntityENS0_6EntityENS_15DeclarationNameEjb", "clang::idx::EntityGetter::getEntity(clang::idx::Entity, clang::DeclarationName, unsigned int, bool)"},
    {"_ZN5clang3idx12EntityGetter15getGlobalEntityEN4llvm9StringRefE", "clang::idx::EntityGetter::getGlobalEntity(llvm::StringRef)"},
    {"_ZN5clang3idx12EntityGetter14VisitNamedDeclEPNS_9NamedDeclE", "clang::idx::EntityGetter::VisitNamedDecl(clang::NamedDecl*)"},
    {"_ZN5clang3idx6EntityC1EPNS_4DeclE", "clang::idx::Entity::Entity(clang::Decl*)"},
    {"_ZN5clang3idx12EntityGetter12VisitVarDeclEPNS_7VarDeclE", "clang::idx::EntityGetter::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN5clang3idx12EntityGetter17VisitFunctionDeclEPNS_12FunctionDeclE", "clang::idx::EntityGetter::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN5clang3idx12EntityGetter14VisitFieldDeclEPNS_9FieldDeclE", "clang::idx::EntityGetter::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang3idx12EntityGetter13VisitTypeDeclEPNS_8TypeDeclE", "clang::idx::EntityGetter::VisitTypeDecl(clang::TypeDecl*)"},
    {"_ZN5clang3idx10EntityImpl7getDeclERNS_10ASTContextE", "clang::idx::EntityImpl::getDecl(clang::ASTContext&)"},
    {"_ZNK5clang3idx6Entity7getDeclERNS_10ASTContextE", "clang::idx::Entity::getDecl(clang::ASTContext&) const"},
    {"_ZN5clang3idx10EntityImpl3getEPNS_4DeclERNS0_7ProgramERNS0_11ProgramImplE", "clang::idx::EntityImpl::get(clang::Decl*, clang::idx::Program&, clang::idx::ProgramImpl&)"},
    {"_ZN5clang3idx10EntityImpl3getEN4llvm9StringRefERNS0_7ProgramERNS0_11ProgramImplE", "clang::idx::EntityImpl::get(llvm::StringRef, clang::idx::Program&, clang::idx::ProgramImpl&)"},
    {"_ZN5clang3idx10EntityImpl16getPrintableNameEv", "clang::idx::EntityImpl::getPrintableName()"},
    {"_ZN5clang3idx6EntityC2EPNS_4DeclE", "clang::idx::Entity::Entity(clang::Decl*)"},
    {"_ZNK5clang3idx6Entity16getPrintableNameEv", "clang::idx::Entity::getPrintableName() const"},
    {"_ZN5clang3idx6Entity3getEPNS_4DeclERNS0_7ProgramE", "clang::idx::Entity::get(clang::Decl*, clang::idx::Program&)"},
    {"_ZN5clang3idx6Entity3getEN4llvm9StringRefERNS0_7ProgramE", "clang::idx::Entity::get(llvm::StringRef, clang::idx::Program&)"},
    {"_ZN4llvm12DenseMapInfoIN5clang3idx6EntityEE12getHashValueES3_", "llvm::DenseMapInfo<clang::idx::Entity>::getHashValue(clang::idx::Entity)"},
    {"_ZN5clang11DeclVisitorINS_3idx12EntityGetterENS1_6EntityEE5VisitEPNS_4DeclE", "clang::DeclVisitor<clang::idx::EntityGetter, clang::idx::Entity>::Visit(clang::Decl*)"},
    {"_ZNK5clang3idx14GlobalSelector11getSelectorERNS_10ASTContextE", "clang::idx::GlobalSelector::getSelector(clang::ASTContext&) const"},
    {"_ZNK5clang3idx14GlobalSelector16getPrintableNameEv", "clang::idx::GlobalSelector::getPrintableName() const"},
    {"_ZN5clang3idx14GlobalSelector3getENS_8SelectorERNS0_7ProgramE", "clang::idx::GlobalSelector::get(clang::Selector, clang::idx::Program&)"},
    {"_ZN4llvm12DenseMapInfoIN5clang3idx14GlobalSelectorEE12getHashValueES3_", "llvm::DenseMapInfo<clang::idx::GlobalSelector>::getHashValue(clang::idx::GlobalSelector)"},
    {"_ZN5clang3idx13EntityHandlerD0Ev", "clang::idx::EntityHandler::~EntityHandler()"},
    {"_ZN5clang3idx13EntityHandlerD1Ev", "clang::idx::EntityHandler::~EntityHandler()"},
    {"_ZN5clang3idx13EntityHandlerD2Ev", "clang::idx::EntityHandler::~EntityHandler()"},
    {"_ZN5clang3idx22TranslationUnitHandlerD0Ev", "clang::idx::TranslationUnitHandler::~TranslationUnitHandler()"},
    {"_ZN5clang3idx22TranslationUnitHandlerD1Ev", "clang::idx::TranslationUnitHandler::~TranslationUnitHandler()"},
    {"_ZN5clang3idx22TranslationUnitHandlerD2Ev", "clang::idx::TranslationUnitHandler::~TranslationUnitHandler()"},
    {"_ZN5clang3idx17TULocationHandlerD0Ev", "clang::idx::TULocationHandler::~TULocationHandler()"},
    {"_ZN5clang3idx17TULocationHandlerD1Ev", "clang::idx::TULocationHandler::~TULocationHandler()"},
    {"_ZN5clang3idx17TULocationHandlerD2Ev", "clang::idx::TULocationHandler::~TULocationHandler()"},
    {"_ZN5clang3idx13IndexProviderD0Ev", "clang::idx::IndexProvider::~IndexProvider()"},
    {"_ZN5clang3idx13IndexProviderD1Ev", "clang::idx::IndexProvider::~IndexProvider()"},
    {"_ZN5clang3idx13IndexProviderD2Ev", "clang::idx::IndexProvider::~IndexProvider()"},
    {"_ZN5clang3idx7Indexer8IndexASTEPNS0_15TranslationUnitE", "clang::idx::Indexer::IndexAST(clang::idx::TranslationUnit*)"},
    {"_ZN12_GLOBAL__N_113EntityIndexerD1Ev", "(anonymous namespace)::EntityIndexer::~EntityIndexer()"},
    {"_ZN5clang3idx7Indexer22GetTranslationUnitsForENS0_6EntityERNS0_22TranslationUnitHandlerE", "clang::idx::Indexer::GetTranslationUnitsFor(clang::idx::Entity, clang::idx::TranslationUnitHandler&)"},
    {"_ZN5clang3idx7Indexer22GetTranslationUnitsForENS0_14GlobalSelectorERNS0_22TranslationUnitHandlerE", "clang::idx::Indexer::GetTranslationUnitsFor(clang::idx::GlobalSelector, clang::idx::TranslationUnitHandler&)"},
    {"_ZN5clang3idx7Indexer16getDefinitionForENS0_6EntityE", "clang::idx::Indexer::getDefinitionFor(clang::idx::Entity)"},
    {"_ZN5clang11DeclVisitorIN12_GLOBAL__N_115SelectorIndexerEvE5VisitEPNS_4DeclE", "clang::DeclVisitor<(anonymous namespace)::SelectorIndexer, void>::Visit(clang::Decl*)"},
    {"_ZN5clang11DeclVisitorIN12_GLOBAL__N_115SelectorIndexerEvE14VisitNamedDeclEPNS_9NamedDeclE", "clang::DeclVisitor<(anonymous namespace)::SelectorIndexer, void>::VisitNamedDecl(clang::NamedDecl*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_115SelectorIndexerEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::SelectorIndexer, void>::Visit(clang::Stmt*)"},
    {"_ZN5clang3idx10ASTVisitorIN12_GLOBAL__N_115SelectorIndexerEE9VisitStmtEPNS_4StmtE", "clang::idx::ASTVisitor<(anonymous namespace)::SelectorIndexer>::VisitStmt(clang::Stmt*)"},
    {"_ZN5clang3idx10ASTVisitorIN12_GLOBAL__N_115SelectorIndexerEE19VisitDeclaratorDeclEPNS_14DeclaratorDeclE", "clang::idx::ASTVisitor<(anonymous namespace)::SelectorIndexer>::VisitDeclaratorDecl(clang::DeclaratorDecl*)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_115SelectorIndexerEvE5VisitENS_7TypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::SelectorIndexer, void>::Visit(clang::TypeLoc)"},
    {"_ZN12_GLOBAL__N_113EntityIndexerD0Ev", "(anonymous namespace)::EntityIndexer::~EntityIndexer()"},
    {"_ZN12_GLOBAL__N_113EntityIndexer6HandleEN5clang3idx6EntityE", "(anonymous namespace)::EntityIndexer::Handle(clang::idx::Entity)"},
    {"_ZN5clang3idx7IndexerD1Ev", "clang::idx::Indexer::~Indexer()"},
    {"_ZN5clang3idx7IndexerD0Ev", "clang::idx::Indexer::~Indexer()"},
    {"_ZNSt3mapIN5clang3idx14GlobalSelectorEN4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEESt4lessIS2_ESaISt4pairIKS2_S7_EEEixERSB_", "std::map<clang::idx::GlobalSelector, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u>, std::less<clang::idx::GlobalSelector>, std::allocator<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::operator[](clang::idx::GlobalSelector const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx14GlobalSelectorESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<clang::idx::GlobalSelector, std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::GlobalSelector>, std::allocator<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx14GlobalSelectorESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<clang::idx::GlobalSelector, std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::GlobalSelector>, std::allocator<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_insert_unique(std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > const&)"},
    {"_ZN4llvm8DenseMapIPN5clang10ASTContextEPNS1_3idx15TranslationUnitENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PSt4pairIS3_S6_E", "llvm::DenseMap<clang::ASTContext*, clang::idx::TranslationUnit*, llvm::DenseMapInfo<clang::ASTContext*>, llvm::DenseMapInfo<clang::idx::TranslationUnit*> >::InsertIntoBucket(clang::ASTContext* const&, clang::idx::TranslationUnit* const&, std::pair<clang::ASTContext*, clang::idx::TranslationUnit*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang10ASTContextEPNS1_3idx15TranslationUnitENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::ASTContext*, clang::idx::TranslationUnit*, llvm::DenseMapInfo<clang::ASTContext*>, llvm::DenseMapInfo<clang::idx::TranslationUnit*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >*)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_S3_IPNS0_12FunctionDeclEPNS1_15TranslationUnitEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> >, std::_Select1st<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > >*)"},
    {"_ZNSt8_Rb_treeIN5clang3idx14GlobalSelectorESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<clang::idx::GlobalSelector, std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::GlobalSelector>, std::allocator<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::idx::GlobalSelector const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >*)"},
    {"_ZNSt3mapIN5clang3idx6EntityEN4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEESt4lessIS2_ESaISt4pairIKS2_S7_EEEixERSB_", "std::map<clang::idx::Entity, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u>, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::operator[](clang::idx::Entity const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_S3_IPNS0_12FunctionDeclEPNS1_15TranslationUnitEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> >, std::_Select1st<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > >, std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_S3_IPNS0_12FunctionDeclEPNS1_15TranslationUnitEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> >, std::_Select1st<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > > >::_M_insert_unique(std::pair<clang::idx::Entity const, std::pair<clang::FunctionDecl*, clang::idx::TranslationUnit*> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang3idx6EntityESt4pairIKS2_N4llvm11SmallPtrSetIPNS1_15TranslationUnitELj4EEEESt10_Select1stISA_ESt4lessIS2_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<clang::idx::Entity, std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> >, std::_Select1st<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > >, std::less<clang::idx::Entity>, std::allocator<std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > > >::_M_insert_unique(std::pair<clang::idx::Entity const, llvm::SmallPtrSet<clang::idx::TranslationUnit*, 4u> > const&)"},
    {"_ZN5clang3idx15TranslationUnitD0Ev", "clang::idx::TranslationUnit::~TranslationUnit()"},
    {"_ZN5clang3idx15TranslationUnitD1Ev", "clang::idx::TranslationUnit::~TranslationUnit()"},
    {"_ZN5clang3idx15TranslationUnitD2Ev", "clang::idx::TranslationUnit::~TranslationUnit()"},
    {"_ZN5clang3idx7ProgramC1Ev", "clang::idx::Program::Program()"},
    {"_ZN5clang3idx7ProgramC2Ev", "clang::idx::Program::Program()"},
    {"_ZN5clang3idx7ProgramD1Ev", "clang::idx::Program::~Program()"},
    {"_ZN5clang3idx7ProgramD2Ev", "clang::idx::Program::~Program()"},
    {"_ZN5clang3idx7Program12FindEntitiesERNS_10ASTContextERNS0_13EntityHandlerE", "clang::idx::Program::FindEntities(clang::ASTContext&, clang::idx::EntityHandler&)"},
    {"_Z16FindEntitiesInDCPN5clang11DeclContextERNS_3idx7ProgramERNS2_13EntityHandlerE", "FindEntitiesInDC(clang::DeclContext*, clang::idx::Program&, clang::idx::EntityHandler&)"},
    {"_ZN4llvm10FoldingSetIN5clang3idx10EntityImplEED1Ev", "llvm::FoldingSet<clang::idx::EntityImpl>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang3idx10EntityImplEED0Ev", "llvm::FoldingSet<clang::idx::EntityImpl>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang3idx10EntityImplEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::idx::EntityImpl>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang3idx10EntityImplEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::idx::EntityImpl>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang3idx10EntityImplEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::idx::EntityImpl>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang3idx11ProgramImplC2Ev", "clang::idx::ProgramImpl::ProgramImpl()"},
    {"_ZN5clang9DeltaTreeC1Ev", "clang::DeltaTree::DeltaTree()"},
    {"_ZN5clang9DeltaTreeC2Ev", "clang::DeltaTree::DeltaTree()"},
    {"_ZN5clang9DeltaTreeC1ERKS0_", "clang::DeltaTree::DeltaTree(clang::DeltaTree const&)"},
    {"_ZN5clang9DeltaTreeC2ERKS0_", "clang::DeltaTree::DeltaTree(clang::DeltaTree const&)"},
    {"_ZN5clang9DeltaTreeD1Ev", "clang::DeltaTree::~DeltaTree()"},
    {"_ZN5clang9DeltaTreeD2Ev", "clang::DeltaTree::~DeltaTree()"},
    {"_ZN12_GLOBAL__N_113DeltaTreeNode7DestroyEv", "(anonymous namespace)::DeltaTreeNode::Destroy()"},
    {"_ZNK5clang9DeltaTree10getDeltaAtEj", "clang::DeltaTree::getDeltaAt(unsigned int) const"},
    {"_ZN5clang9DeltaTree8AddDeltaEji", "clang::DeltaTree::AddDelta(unsigned int, int)"},
    {"_ZN12_GLOBAL__N_113DeltaTreeNode11DoInsertionEjiPNS0_12InsertResultE", "(anonymous namespace)::DeltaTreeNode::DoInsertion(unsigned int, int, (anonymous namespace)::DeltaTreeNode::InsertResult*)"},
    {"_ZN12_GLOBAL__N_113DeltaTreeNode7DoSplitERNS0_12InsertResultE", "(anonymous namespace)::DeltaTreeNode::DoSplit((anonymous namespace)::DeltaTreeNode::InsertResult&)"},
    {"_ZN5clang13FixItRewriterC1ERNS_10DiagnosticERNS_13SourceManagerERKNS_11LangOptionsEPNS_12FixItOptionsE", "clang::FixItRewriter::FixItRewriter(clang::Diagnostic&, clang::SourceManager&, clang::LangOptions const&, clang::FixItOptions*)"},
    {"_ZN5clang13FixItRewriterC2ERNS_10DiagnosticERNS_13SourceManagerERKNS_11LangOptionsEPNS_12FixItOptionsE", "clang::FixItRewriter::FixItRewriter(clang::Diagnostic&, clang::SourceManager&, clang::LangOptions const&, clang::FixItOptions*)"},
    {"_ZN5clang13FixItRewriterD0Ev", "clang::FixItRewriter::~FixItRewriter()"},
    {"_ZN5clang13FixItRewriterD1Ev", "clang::FixItRewriter::~FixItRewriter()"},
    {"_ZN5clang13FixItRewriterD2Ev", "clang::FixItRewriter::~FixItRewriter()"},
    {"_ZN5clang13FixItRewriter14WriteFixedFileENS_6FileIDERN4llvm11raw_ostreamE", "clang::FixItRewriter::WriteFixedFile(clang::FileID, llvm::raw_ostream&)"},
    {"_ZN5clang13FixItRewriter15WriteFixedFilesEv", "clang::FixItRewriter::WriteFixedFiles()"},
    {"_ZN5clang13FixItRewriter4DiagENS_14SourceLocationEj", "clang::FixItRewriter::Diag(clang::SourceLocation, unsigned int)"},
    {"_ZNK5clang13FixItRewriter25IncludeInDiagnosticCountsEv", "clang::FixItRewriter::IncludeInDiagnosticCounts() const"},
    {"_ZN5clang13FixItRewriter16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::FixItRewriter::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN5clang12FixItOptionsD0Ev", "clang::FixItOptions::~FixItOptions()"},
    {"_ZN5clang12FixItOptionsD1Ev", "clang::FixItOptions::~FixItOptions()"},
    {"_ZN5clang12FixItOptionsD2Ev", "clang::FixItOptions::~FixItOptions()"},
    {"_ZN5clang15HTMLPrintAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::HTMLPrintAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang11FixItActionC1Ev", "clang::FixItAction::FixItAction()"},
    {"_ZN5clang11FixItActionC2Ev", "clang::FixItAction::FixItAction()"},
    {"_ZN5clang11FixItActionD0Ev", "clang::FixItAction::~FixItAction()"},
    {"_ZN5clang11FixItActionD1Ev", "clang::FixItAction::~FixItAction()"},
    {"_ZN5clang11FixItActionD2Ev", "clang::FixItAction::~FixItAction()"},
    {"_ZN5clang11FixItAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::FixItAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang11FixItAction21BeginSourceFileActionERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::FixItAction::BeginSourceFileAction(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang11FixItAction19EndSourceFileActionEv", "clang::FixItAction::EndSourceFileAction()"},
    {"_ZN5clang17RewriteObjCAction17CreateASTConsumerERNS_16CompilerInstanceEN4llvm9StringRefE", "clang::RewriteObjCAction::CreateASTConsumer(clang::CompilerInstance&, llvm::StringRef)"},
    {"_ZN5clang19RewriteMacrosAction13ExecuteActionEv", "clang::RewriteMacrosAction::ExecuteAction()"},
    {"_ZN5clang17RewriteTestAction13ExecuteActionEv", "clang::RewriteTestAction::ExecuteAction()"},
    {"_ZN12_GLOBAL__N_119FixItRewriteInPlaceD1Ev", "(anonymous namespace)::FixItRewriteInPlace::~FixItRewriteInPlace()"},
    {"_ZN12_GLOBAL__N_119FixItRewriteInPlaceD0Ev", "(anonymous namespace)::FixItRewriteInPlace::~FixItRewriteInPlace()"},
    {"_ZN12_GLOBAL__N_119FixItRewriteInPlace15RewriteFilenameERKSs", "(anonymous namespace)::FixItRewriteInPlace::RewriteFilename(std::string const&)"},
    {"_ZN12_GLOBAL__N_125FixItActionSuffixInserterD1Ev", "(anonymous namespace)::FixItActionSuffixInserter::~FixItActionSuffixInserter()"},
    {"_ZN12_GLOBAL__N_125FixItActionSuffixInserterD0Ev", "(anonymous namespace)::FixItActionSuffixInserter::~FixItActionSuffixInserter()"},
    {"_ZN12_GLOBAL__N_125FixItActionSuffixInserter15RewriteFilenameERKSs", "(anonymous namespace)::FixItActionSuffixInserter::RewriteFilename(std::string const&)"},
    {"_ZNK5clang11FixItAction17hasASTFileSupportEv", "clang::FixItAction::hasASTFileSupport() const"},
    {"_ZN5clang15HTMLPrintActionD1Ev", "clang::HTMLPrintAction::~HTMLPrintAction()"},
    {"_ZN5clang15HTMLPrintActionD0Ev", "clang::HTMLPrintAction::~HTMLPrintAction()"},
    {"_ZN5clang17RewriteObjCActionD1Ev", "clang::RewriteObjCAction::~RewriteObjCAction()"},
    {"_ZN5clang17RewriteObjCActionD0Ev", "clang::RewriteObjCAction::~RewriteObjCAction()"},
    {"_ZN5clang19RewriteMacrosActionD1Ev", "clang::RewriteMacrosAction::~RewriteMacrosAction()"},
    {"_ZN5clang19RewriteMacrosActionD0Ev", "clang::RewriteMacrosAction::~RewriteMacrosAction()"},
    {"_ZN5clang17RewriteTestActionD1Ev", "clang::RewriteTestAction::~RewriteTestAction()"},
    {"_ZN5clang17RewriteTestActionD0Ev", "clang::RewriteTestAction::~RewriteTestAction()"},
    {"_ZN5clang17CreateHTMLPrinterEPN4llvm11raw_ostreamERNS_12PreprocessorEbb", "clang::CreateHTMLPrinter(llvm::raw_ostream*, clang::Preprocessor&, bool, bool)"},
    {"_ZN12_GLOBAL__N_111HTMLPrinterD1Ev", "(anonymous namespace)::HTMLPrinter::~HTMLPrinter()"},
    {"_ZN12_GLOBAL__N_111HTMLPrinterD0Ev", "(anonymous namespace)::HTMLPrinter::~HTMLPrinter()"},
    {"_ZN12_GLOBAL__N_111HTMLPrinter10InitializeERN5clang10ASTContextE", "(anonymous namespace)::HTMLPrinter::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_111HTMLPrinter21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::HTMLPrinter::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN5clang4html14HighlightRangeERNS_8RewriterENS_14SourceLocationES3_PKcS5_", "clang::html::HighlightRange(clang::Rewriter&, clang::SourceLocation, clang::SourceLocation, char const*, char const*)"},
    {"_ZN5clang4html14HighlightRangeERNS_13RewriteBufferEjjPKcS4_S4_", "clang::html::HighlightRange(clang::RewriteBuffer&, unsigned int, unsigned int, char const*, char const*, char const*)"},
    {"_ZN5clang4html10EscapeTextERNS_8RewriterENS_6FileIDEbb", "clang::html::EscapeText(clang::Rewriter&, clang::FileID, bool, bool)"},
    {"_ZN5clang4html10EscapeTextERKSsbb", "clang::html::EscapeText(std::string const&, bool, bool)"},
    {"_ZN5clang4html14AddLineNumbersERNS_8RewriterENS_6FileIDE", "clang::html::AddLineNumbers(clang::Rewriter&, clang::FileID)"},
    {"_ZN5clang4html33AddHeaderFooterInternalBuiltinCSSERNS_8RewriterENS_6FileIDEPKc", "clang::html::AddHeaderFooterInternalBuiltinCSS(clang::Rewriter&, clang::FileID, char const*)"},
    {"_ZN5clang4html15SyntaxHighlightERNS_8RewriterENS_6FileIDERKNS_12PreprocessorE", "clang::html::SyntaxHighlight(clang::Rewriter&, clang::FileID, clang::Preprocessor const&)"},
    {"_ZN5clang4html15HighlightMacrosERNS_8RewriterENS_6FileIDERKNS_12PreprocessorE", "clang::html::HighlightMacros(clang::Rewriter&, clang::FileID, clang::Preprocessor const&)"},
    {"_ZN12_GLOBAL__N_118IgnoringDiagClientD1Ev", "(anonymous namespace)::IgnoringDiagClient::~IgnoringDiagClient()"},
    {"_ZN12_GLOBAL__N_118IgnoringDiagClientD0Ev", "(anonymous namespace)::IgnoringDiagClient::~IgnoringDiagClient()"},
    {"_ZN12_GLOBAL__N_118IgnoringDiagClient16HandleDiagnosticEN5clang10Diagnostic5LevelERKNS1_14DiagnosticInfoE", "(anonymous namespace)::IgnoringDiagClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN5clang20RewriteMacrosInInputERNS_12PreprocessorEPN4llvm11raw_ostreamE", "clang::RewriteMacrosInInput(clang::Preprocessor&, llvm::raw_ostream*)"},
    {"_ZN5clang18CreateObjCRewriterERKSsPN4llvm11raw_ostreamERNS_10DiagnosticERKNS_11LangOptionsEb", "clang::CreateObjCRewriter(std::string const&, llvm::raw_ostream*, clang::Diagnostic&, clang::LangOptions const&, bool)"},
    {"_ZN12_GLOBAL__N_111RewriteObjCD1Ev", "(anonymous namespace)::RewriteObjC::~RewriteObjC()"},
    {"_ZN12_GLOBAL__N_111RewriteObjCD0Ev", "(anonymous namespace)::RewriteObjC::~RewriteObjC()"},
    {"_ZN12_GLOBAL__N_111RewriteObjC10InitializeERN5clang10ASTContextE", "(anonymous namespace)::RewriteObjC::Initialize(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC18HandleTopLevelDeclEN5clang12DeclGroupRefE", "(anonymous namespace)::RewriteObjC::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC21HandleTranslationUnitERN5clang10ASTContextE", "(anonymous namespace)::RewriteObjC::HandleTranslationUnit(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC27RewriteObjCProtocolMetaDataEPN5clang16ObjCProtocolDeclEN4llvm9StringRefES5_RSs", "(anonymous namespace)::RewriteObjC::RewriteObjCProtocolMetaData(clang::ObjCProtocolDecl*, llvm::StringRef, llvm::StringRef, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC28SynthesizeMetaDataIntoBufferERSs", "(anonymous namespace)::RewriteObjC::SynthesizeMetaDataIntoBuffer(std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC26RewriteObjCMethodsMetaDataIPPN5clang14ObjCMethodDeclEEEvT_S6_bN4llvm9StringRefES8_RSs", "void (anonymous namespace)::RewriteObjC::RewriteObjCMethodsMetaData<clang::ObjCMethodDecl**>(clang::ObjCMethodDecl**, clang::ObjCMethodDecl**, bool, llvm::StringRef, llvm::StringRef, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC26RewriteObjCMethodsMetaDataIN5clang11DeclContext22filtered_decl_iteratorINS2_14ObjCMethodDeclEXadL_ZNKS5_13isClassMethodEvEEEEEEvT_S7_bN4llvm9StringRefES9_RSs", "void (anonymous namespace)::RewriteObjC::RewriteObjCMethodsMetaData<clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isClassMethod() const)> >(clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isClassMethod() const)>, clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isClassMethod() const)>, bool, llvm::StringRef, llvm::StringRef, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC31RewriteObjCProtocolListMetaDataERKN5clang8ObjCListINS1_16ObjCProtocolDeclEEEN4llvm9StringRefES8_RSs", "(anonymous namespace)::RewriteObjC::RewriteObjCProtocolListMetaData(clang::ObjCList<clang::ObjCProtocolDecl> const&, llvm::StringRef, llvm::StringRef, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC28SynthesizeObjCInternalStructEPN5clang17ObjCInterfaceDeclERSs", "(anonymous namespace)::RewriteObjC::SynthesizeObjCInternalStruct(clang::ObjCInterfaceDecl*, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC17QuoteDoublequotesERSsS1_", "(anonymous namespace)::RewriteObjC::QuoteDoublequotes(std::string&, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC31SynthesizeIvarOffsetComputationEPN5clang12ObjCIvarDeclERSs", "(anonymous namespace)::RewriteObjC::SynthesizeIvarOffsetComputation(clang::ObjCIvarDecl*, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC25RewriteImplementationDeclEPN5clang4DeclE", "(anonymous namespace)::RewriteObjC::RewriteImplementationDecl(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC21RewriteObjCMethodDeclEPKN5clang17ObjCInterfaceDeclEPNS1_14ObjCMethodDeclERSs", "(anonymous namespace)::RewriteObjC::RewriteObjCMethodDecl(clang::ObjCInterfaceDecl const*, clang::ObjCMethodDecl*, std::string&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC21RewriteTypeIntoStringEN5clang8QualTypeERSsRPKNS1_12FunctionTypeE", "(anonymous namespace)::RewriteObjC::RewriteTypeIntoString(clang::QualType, std::string&, clang::FunctionType const*&)"},
    {"_Z19getIvarAccessStringPN5clang12ObjCIvarDeclE", "getIvarAccessString(clang::ObjCIvarDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC24HandleTopLevelSingleDeclEPN5clang4DeclE", "(anonymous namespace)::RewriteObjC::HandleTopLevelSingleDecl(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC32RewriteBlocksInFunctionProtoTypeEN5clang8QualTypeEPNS1_9NamedDeclE", "(anonymous namespace)::RewriteObjC::RewriteBlocksInFunctionProtoType(clang::QualType, clang::NamedDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC22CollectPropertySettersEPN5clang4StmtE", "(anonymous namespace)::RewriteObjC::CollectPropertySetters(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC38RewriteFunctionBodyOrGlobalInitializerEPN5clang4StmtE", "(anonymous namespace)::RewriteObjC::RewriteFunctionBodyOrGlobalInitializer(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC34RewriteObjCQualifiedInterfaceTypesEPN5clang4DeclE", "(anonymous namespace)::RewriteObjC::RewriteObjCQualifiedInterfaceTypes(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23RewriteBlockPointerDeclEPN5clang9NamedDeclE", "(anonymous namespace)::RewriteObjC::RewriteBlockPointerDecl(clang::NamedDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC15RewriteCastExprEPN5clang14CStyleCastExprE", "(anonymous namespace)::RewriteObjC::RewriteCastExpr(clang::CStyleCastExpr*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC17RewriteRecordBodyEPN5clang10RecordDeclE", "(anonymous namespace)::RewriteObjC::RewriteRecordBody(clang::RecordDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23SynthesizeBlockLiteralsEN5clang14SourceLocationEN4llvm9StringRefE", "(anonymous namespace)::RewriteObjC::SynthesizeBlockLiterals(clang::SourceLocation, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC31RewriteBlockLiteralFunctionDeclEPN5clang12FunctionDeclE", "(anonymous namespace)::RewriteObjC::RewriteBlockLiteralFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23CollectBlockDeclRefInfoEPN5clang9BlockExprE", "(anonymous namespace)::RewriteObjC::CollectBlockDeclRefInfo(clang::BlockExpr*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC18RewriteByRefStringERSsRKSsPN5clang9ValueDeclEb", "(anonymous namespace)::RewriteObjC::RewriteByRefString(std::string&, std::string const&, clang::ValueDecl*, bool)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23RewriteBlockPointerTypeERSsN5clang8QualTypeE", "(anonymous namespace)::RewriteObjC::RewriteBlockPointerType(std::string&, clang::QualType)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC20GetBlockDeclRefExprsEPN5clang4StmtE", "(anonymous namespace)::RewriteObjC::GetBlockDeclRefExprs(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC33PointerTypeTakesAnyBlockArgumentsEN5clang8QualTypeE", "(anonymous namespace)::RewriteObjC::PointerTypeTakesAnyBlockArguments(clang::QualType)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23needToScanForQualifiersEN5clang8QualTypeE", "(anonymous namespace)::RewriteObjC::needToScanForQualifiers(clang::QualType)"},
    {"_Z21BuildUniqueMethodNameRSsPN5clang14ObjCMethodDeclE", "BuildUniqueMethodName(std::string&, clang::ObjCMethodDecl*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC28RewriteObjCNestedIvarRefExprEPN5clang4StmtERb", "(anonymous namespace)::RewriteObjC::RewriteObjCNestedIvarRefExpr(clang::Stmt*, bool&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC11ReplaceStmtEPN5clang4StmtES3_", "(anonymous namespace)::RewriteObjC::ReplaceStmt(clang::Stmt*, clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC25GetInnerBlockDeclRefExprsEPN5clang4StmtERN4llvm11SmallVectorIPNS1_16BlockDeclRefExprELj8EEERNS4_11SmallPtrSetIPKNS1_11DeclContextELj8EEE", "(anonymous namespace)::RewriteObjC::GetInnerBlockDeclRefExprs(clang::Stmt*, llvm::SmallVector<clang::BlockDeclRefExpr*, 8u>&, llvm::SmallPtrSet<clang::DeclContext const*, 8u>&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC23RewriteBlockDeclRefExprEPN5clang4ExprE", "(anonymous namespace)::RewriteObjC::RewriteBlockDeclRefExpr(clang::Expr*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC19SynthesizeBlockCallEPN5clang8CallExprEPKNS1_4ExprE", "(anonymous namespace)::RewriteObjC::SynthesizeBlockCall(clang::CallExpr*, clang::Expr const*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC15getProtocolTypeEv", "(anonymous namespace)::RewriteObjC::getProtocolType()"},
    {"_ZN12_GLOBAL__N_111RewriteObjC14HasReturnStmtsEPN5clang4StmtERb", "(anonymous namespace)::RewriteObjC::HasReturnStmts(clang::Stmt*, bool&)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC22RewriteSyncReturnStmtsEPN5clang4StmtESs", "(anonymous namespace)::RewriteObjC::RewriteSyncReturnStmts(clang::Stmt*, std::string)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC24WarnAboutReturnGotoStmtsEPN5clang4StmtE", "(anonymous namespace)::RewriteObjC::WarnAboutReturnGotoStmts(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC21RewriteTryReturnStmtsEPN5clang4StmtE", "(anonymous namespace)::RewriteObjC::RewriteTryReturnStmts(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC16SynthMessageExprEPN5clang15ObjCMessageExprENS1_14SourceLocationES4_", "(anonymous namespace)::RewriteObjC::SynthMessageExpr(clang::ObjCMessageExpr*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC26SynthSelGetUidFunctionDeclEv", "(anonymous namespace)::RewriteObjC::SynthSelGetUidFunctionDecl()"},
    {"_ZN12_GLOBAL__N_111RewriteObjC28SynthesizeCallToFunctionDeclEPN5clang12FunctionDeclEPPNS1_4ExprEjNS1_14SourceLocationES7_", "(anonymous namespace)::RewriteObjC::SynthesizeCallToFunctionDecl(clang::FunctionDecl*, clang::Expr**, unsigned int, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC18getSuperStructTypeEv", "(anonymous namespace)::RewriteObjC::getSuperStructType()"},
    {"_ZN12_GLOBAL__N_111RewriteObjC32SynthSuperContructorFunctionDeclEv", "(anonymous namespace)::RewriteObjC::SynthSuperContructorFunctionDecl()"},
    {"_ZN12_GLOBAL__N_111RewriteObjC26SynthBlockInitFunctionDeclEN4llvm9StringRefE", "(anonymous namespace)::RewriteObjC::SynthBlockInitFunctionDecl(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_111RewriteObjC24RewriteMethodDeclarationEPN5clang14ObjCMethodDeclE", "(anonymous namespace)::RewriteObjC::RewriteMethodDeclaration(clang::ObjCMethodDecl*)"},
    {"_ZN4llvm8DenseMapIPN5clang14ObjCMethodDeclESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::ObjCMethodDecl*, std::string, llvm::DenseMapInfo<clang::ObjCMethodDecl*>, llvm::DenseMapInfo<std::string> >::FindAndConstruct(clang::ObjCMethodDecl* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14ObjCMethodDeclESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16InsertIntoBucketERKS3_RKSsPSt4pairIS3_SsE", "llvm::DenseMap<clang::ObjCMethodDecl*, std::string, llvm::DenseMapInfo<clang::ObjCMethodDecl*>, llvm::DenseMapInfo<std::string> >::InsertIntoBucket(clang::ObjCMethodDecl* const&, std::string const&, std::pair<clang::ObjCMethodDecl*, std::string>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14ObjCMethodDeclESsNS_12DenseMapInfoIS3_EENS4_ISsEEE4growEj", "llvm::DenseMap<clang::ObjCMethodDecl*, std::string, llvm::DenseMapInfo<clang::ObjCMethodDecl*>, llvm::DenseMapInfo<std::string> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang14ObjCMethodDeclEE6appendINS1_11DeclContext22filtered_decl_iteratorIS2_XadL_ZNKS2_16isInstanceMethodEvEEEEEEvT_S9_", "void llvm::SmallVectorImpl<clang::ObjCMethodDecl*>::append<clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isInstanceMethod() const)> >(clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isInstanceMethod() const)>, clang::DeclContext::filtered_decl_iterator<clang::ObjCMethodDecl, &(clang::ObjCMethodDecl::isInstanceMethod() const)>)"},
    {"_ZNK5clang17ObjCInterfaceDecl9ivar_sizeEv", "clang::ObjCInterfaceDecl::ivar_size() const"},
    {"_ZNSs12_S_constructIN5clang22RopePieceBTreeIteratorEEEPcT_S3_RKSaIcESt20forward_iterator_tag", "char* std::string::_S_construct<clang::RopePieceBTreeIterator>(clang::RopePieceBTreeIterator, clang::RopePieceBTreeIterator, std::allocator<char> const&, std::forward_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPN5clang9BlockExprESsNS_12DenseMapInfoIS3_EENS4_ISsEEE5clearEv", "llvm::DenseMap<clang::BlockExpr*, std::string, llvm::DenseMapInfo<clang::BlockExpr*>, llvm::DenseMapInfo<std::string> >::clear()"},
    {"_ZN4llvm8DenseMapIPN5clang9BlockExprESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16shrink_and_clearEv", "llvm::DenseMap<clang::BlockExpr*, std::string, llvm::DenseMapInfo<clang::BlockExpr*>, llvm::DenseMapInfo<std::string> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPN5clang9BlockExprESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::BlockExpr*, std::string, llvm::DenseMapInfo<clang::BlockExpr*>, llvm::DenseMapInfo<std::string> >::FindAndConstruct(clang::BlockExpr* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang9BlockExprESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16InsertIntoBucketERKS3_RKSsPSt4pairIS3_SsE", "llvm::DenseMap<clang::BlockExpr*, std::string, llvm::DenseMapInfo<clang::BlockExpr*>, llvm::DenseMapInfo<std::string> >::InsertIntoBucket(clang::BlockExpr* const&, std::string const&, std::pair<clang::BlockExpr*, std::string>*)"},
    {"_ZN4llvm8DenseMapIPN5clang9BlockExprESsNS_12DenseMapInfoIS3_EENS4_ISsEEE4growEj", "llvm::DenseMap<clang::BlockExpr*, std::string, llvm::DenseMapInfo<clang::BlockExpr*>, llvm::DenseMapInfo<std::string> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang9ValueDeclEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<clang::ValueDecl*, unsigned int, llvm::DenseMapInfo<clang::ValueDecl*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::ValueDecl* const&, unsigned int const&, std::pair<clang::ValueDecl*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPN5clang9ValueDeclEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<clang::ValueDecl*, unsigned int, llvm::DenseMapInfo<clang::ValueDecl*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang19ConditionalOperatorC2EPNS_4ExprENS_14SourceLocationES2_S3_S2_S2_NS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindE", "clang::ConditionalOperator::ConditionalOperator(clang::Expr*, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::Expr*, clang::Expr*, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind)"},
    {"_ZN5clang13UnaryOperatorC2EPNS_4ExprENS_17UnaryOperatorKindENS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindENS_14SourceLocationE", "clang::UnaryOperator::UnaryOperator(clang::Expr*, clang::UnaryOperatorKind, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind, clang::SourceLocation)"},
    {"_ZN4llvm8DenseMapIycNS_12DenseMapInfoIyEENS1_IcEEE6insertERKSt4pairIycE", "llvm::DenseMap<unsigned long long, char, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<char> >::insert(std::pair<unsigned long long, char> const&)"},
    {"_ZN4llvm8DenseMapIycNS_12DenseMapInfoIyEENS1_IcEEE16InsertIntoBucketERKyRKcPSt4pairIycE", "llvm::DenseMap<unsigned long long, char, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<char> >::InsertIntoBucket(unsigned long long const&, char const&, std::pair<unsigned long long, char>*)"},
    {"_ZN4llvm8DenseMapIycNS_12DenseMapInfoIyEENS1_IcEEE4growEj", "llvm::DenseMap<unsigned long long, char, llvm::DenseMapInfo<unsigned long long>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN5clang14BinaryOperatorC2EPNS_4ExprES2_NS_18BinaryOperatorKindENS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindENS_14SourceLocationE", "clang::BinaryOperator::BinaryOperator(clang::Expr*, clang::Expr*, clang::BinaryOperatorKind, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind, clang::SourceLocation)"},
    {"_ZN4llvm8DenseMapIPN5clang4ExprEPNS1_4StmtENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::Expr*, clang::Stmt*, llvm::DenseMapInfo<clang::Expr*>, llvm::DenseMapInfo<clang::Stmt*> >::InsertIntoBucket(clang::Expr* const&, clang::Stmt* const&, std::pair<clang::Expr*, clang::Stmt*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4ExprEPNS1_4StmtENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Expr*, clang::Stmt*, llvm::DenseMapInfo<clang::Expr*>, llvm::DenseMapInfo<clang::Stmt*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<clang::Stmt*, clang::Stmt*, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<clang::Stmt*> >::InsertIntoBucket(clang::Stmt* const&, clang::Stmt* const&, std::pair<clang::Stmt*, clang::Stmt*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4StmtES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<clang::Stmt*, clang::Stmt*, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<clang::Stmt*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang4ExprEPNS1_14BinaryOperatorENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::Expr*, clang::BinaryOperator*, llvm::DenseMapInfo<clang::Expr*>, llvm::DenseMapInfo<clang::BinaryOperator*> >::InsertIntoBucket(clang::Expr* const&, clang::BinaryOperator* const&, std::pair<clang::Expr*, clang::BinaryOperator*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4ExprEPNS1_14BinaryOperatorENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::Expr*, clang::BinaryOperator*, llvm::DenseMapInfo<clang::Expr*>, llvm::DenseMapInfo<clang::BinaryOperator*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIN5clang6FileIDESt4pairIKS1_NS0_13RewriteBufferEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<clang::FileID, std::pair<clang::FileID const, clang::RewriteBuffer>, std::_Select1st<std::pair<clang::FileID const, clang::RewriteBuffer> >, std::less<clang::FileID>, std::allocator<std::pair<clang::FileID const, clang::RewriteBuffer> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::FileID const, clang::RewriteBuffer> >*)"},
    {"_ZN5clang22RopePieceBTreeIteratorC1EPKv", "clang::RopePieceBTreeIterator::RopePieceBTreeIterator(void const*)"},
    {"_ZN5clang22RopePieceBTreeIteratorC2EPKv", "clang::RopePieceBTreeIterator::RopePieceBTreeIterator(void const*)"},
    {"_ZN5clang22RopePieceBTreeIterator15MoveToNextPieceEv", "clang::RopePieceBTreeIterator::MoveToNextPiece()"},
    {"_ZN5clang14RopePieceBTreeC1Ev", "clang::RopePieceBTree::RopePieceBTree()"},
    {"_ZN5clang14RopePieceBTreeC2Ev", "clang::RopePieceBTree::RopePieceBTree()"},
    {"_ZN5clang14RopePieceBTreeC1ERKS0_", "clang::RopePieceBTree::RopePieceBTree(clang::RopePieceBTree const&)"},
    {"_ZN5clang14RopePieceBTreeC2ERKS0_", "clang::RopePieceBTree::RopePieceBTree(clang::RopePieceBTree const&)"},
    {"_ZN5clang14RopePieceBTreeD1Ev", "clang::RopePieceBTree::~RopePieceBTree()"},
    {"_ZN5clang14RopePieceBTreeD2Ev", "clang::RopePieceBTree::~RopePieceBTree()"},
    {"_ZN12_GLOBAL__N_118RopePieceBTreeNode7DestroyEv", "(anonymous namespace)::RopePieceBTreeNode::Destroy()"},
    {"_ZNK5clang14RopePieceBTree4sizeEv", "clang::RopePieceBTree::size() const"},
    {"_ZN5clang14RopePieceBTree5clearEv", "clang::RopePieceBTree::clear()"},
    {"_ZN5clang14RopePieceBTree6insertEjRKNS_9RopePieceE", "clang::RopePieceBTree::insert(unsigned int, clang::RopePiece const&)"},
    {"_ZN12_GLOBAL__N_118RopePieceBTreeNode5splitEj", "(anonymous namespace)::RopePieceBTreeNode::split(unsigned int)"},
    {"_ZN12_GLOBAL__N_118RopePieceBTreeNode6insertEjRKN5clang9RopePieceE", "(anonymous namespace)::RopePieceBTreeNode::insert(unsigned int, clang::RopePiece const&)"},
    {"_ZN5clang14RopePieceBTree5eraseEjj", "clang::RopePieceBTree::erase(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_118RopePieceBTreeNode5eraseEjj", "(anonymous namespace)::RopePieceBTreeNode::erase(unsigned int, unsigned int)"},
    {"_ZN5clang11RewriteRope14MakeRopeStringEPKcS2_", "clang::RewriteRope::MakeRopeString(char const*, char const*)"},
    {"_ZN12_GLOBAL__N_118RopePieceBTreeLeaf6insertEjRKN5clang9RopePieceE", "(anonymous namespace)::RopePieceBTreeLeaf::insert(unsigned int, clang::RopePiece const&)"},
    {"_ZN12_GLOBAL__N_122RopePieceBTreeInterior16HandleChildPieceEjPNS_18RopePieceBTreeNodeE", "(anonymous namespace)::RopePieceBTreeInterior::HandleChildPiece(unsigned int, (anonymous namespace)::RopePieceBTreeNode*)"},
    {"_ZN5clang13DoRewriteTestERNS_12PreprocessorEPN4llvm11raw_ostreamE", "clang::DoRewriteTest(clang::Preprocessor&, llvm::raw_ostream*)"},
    {"_ZNK5clang13RewriteBuffer5writeERN4llvm11raw_ostreamE", "clang::RewriteBuffer::write(llvm::raw_ostream&) const"},
    {"_ZN5clang13RewriteBuffer10RemoveTextEjj", "clang::RewriteBuffer::RemoveText(unsigned int, unsigned int)"},
    {"_ZN5clang13RewriteBuffer10InsertTextEjN4llvm9StringRefEb", "clang::RewriteBuffer::InsertText(unsigned int, llvm::StringRef, bool)"},
    {"_ZN5clang13RewriteBuffer11ReplaceTextEjjN4llvm9StringRefE", "clang::RewriteBuffer::ReplaceText(unsigned int, unsigned int, llvm::StringRef)"},
    {"_ZNK5clang8Rewriter12getRangeSizeERKNS_15CharSourceRangeE", "clang::Rewriter::getRangeSize(clang::CharSourceRange const&) const"},
    {"_ZNK5clang8Rewriter26getLocationOffsetAndFileIDENS_14SourceLocationERNS_6FileIDE", "clang::Rewriter::getLocationOffsetAndFileID(clang::SourceLocation, clang::FileID&) const"},
    {"_ZNK5clang8Rewriter12getRangeSizeENS_11SourceRangeE", "clang::Rewriter::getRangeSize(clang::SourceRange) const"},
    {"_ZNK5clang8Rewriter16getRewrittenTextENS_11SourceRangeE", "clang::Rewriter::getRewrittenText(clang::SourceRange) const"},
    {"_ZN5clang8Rewriter13getEditBufferENS_6FileIDE", "clang::Rewriter::getEditBuffer(clang::FileID)"},
    {"_ZN5clang8Rewriter10InsertTextENS_14SourceLocationEN4llvm9StringRefEb", "clang::Rewriter::InsertText(clang::SourceLocation, llvm::StringRef, bool)"},
    {"_ZN5clang8Rewriter10RemoveTextENS_14SourceLocationEj", "clang::Rewriter::RemoveText(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang8Rewriter11ReplaceTextENS_14SourceLocationEjN4llvm9StringRefE", "clang::Rewriter::ReplaceText(clang::SourceLocation, unsigned int, llvm::StringRef)"},
    {"_ZN5clang8Rewriter11ReplaceStmtEPNS_4StmtES2_", "clang::Rewriter::ReplaceStmt(clang::Stmt*, clang::Stmt*)"},
    {"_ZNSt8_Rb_treeIN5clang6FileIDESt4pairIKS1_NS0_13RewriteBufferEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::FileID, std::pair<clang::FileID const, clang::RewriteBuffer>, std::_Select1st<std::pair<clang::FileID const, clang::RewriteBuffer> >, std::less<clang::FileID>, std::allocator<std::pair<clang::FileID const, clang::RewriteBuffer> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::FileID const, clang::RewriteBuffer> >, std::pair<clang::FileID const, clang::RewriteBuffer> const&)"},
    {"_ZNSt8_Rb_treeIN5clang6FileIDESt4pairIKS1_NS0_13RewriteBufferEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<clang::FileID, std::pair<clang::FileID const, clang::RewriteBuffer>, std::_Select1st<std::pair<clang::FileID const, clang::RewriteBuffer> >, std::less<clang::FileID>, std::allocator<std::pair<clang::FileID const, clang::RewriteBuffer> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<clang::FileID const, clang::RewriteBuffer> const&)"},
    {"_ZNSt8_Rb_treeIN5clang6FileIDESt4pairIKS1_NS0_13RewriteBufferEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::FileID, std::pair<clang::FileID const, clang::RewriteBuffer>, std::_Select1st<std::pair<clang::FileID const, clang::RewriteBuffer> >, std::less<clang::FileID>, std::allocator<std::pair<clang::FileID const, clang::RewriteBuffer> > >::_M_insert_unique(std::pair<clang::FileID const, clang::RewriteBuffer> const&)"},
    {"_ZN5clang13TokenRewriterC1ENS_6FileIDERNS_13SourceManagerERKNS_11LangOptionsE", "clang::TokenRewriter::TokenRewriter(clang::FileID, clang::SourceManager&, clang::LangOptions const&)"},
    {"_ZN5clang13TokenRewriterC2ENS_6FileIDERNS_13SourceManagerERKNS_11LangOptionsE", "clang::TokenRewriter::TokenRewriter(clang::FileID, clang::SourceManager&, clang::LangOptions const&)"},
    {"_ZN5clang13TokenRewriter8AddTokenERKNS_5TokenESt14_List_iteratorIS1_E", "clang::TokenRewriter::AddToken(clang::Token const&, std::_List_iterator<clang::Token>)"},
    {"_ZN5clang13TokenRewriterD1Ev", "clang::TokenRewriter::~TokenRewriter()"},
    {"_ZN5clang13TokenRewriterD2Ev", "clang::TokenRewriter::~TokenRewriter()"},
    {"_ZN5clang13TokenRewriter13RemapIteratorESt20_List_const_iteratorINS_5TokenEE", "clang::TokenRewriter::RemapIterator(std::_List_const_iterator<clang::Token>)"},
    {"_ZN5clang13TokenRewriter14AddTokenBeforeESt20_List_const_iteratorINS_5TokenEEPKc", "clang::TokenRewriter::AddTokenBefore(std::_List_const_iterator<clang::Token>, char const*)"},
    {"_ZNSt8_Rb_treeIN5clang14SourceLocationESt4pairIKS1_St14_List_iteratorINS0_5TokenEEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<clang::SourceLocation, std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> >, std::_Select1st<std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > >, std::less<clang::SourceLocation>, std::allocator<std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > > >::_M_insert_unique(std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang14SourceLocationESt4pairIKS1_St14_List_iteratorINS0_5TokenEEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<clang::SourceLocation, std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> >, std::_Select1st<std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > >, std::less<clang::SourceLocation>, std::allocator<std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > > >::_M_erase(std::_Rb_tree_node<std::pair<clang::SourceLocation const, std::_List_iterator<clang::Token> > >*)"},
    {"_ZN5clang7APValueC1EPNS_4ExprE", "clang::APValue::APValue(clang::Expr*)"},
    {"_ZN5clang7APValueC2EPNS_4ExprE", "clang::APValue::APValue(clang::Expr*)"},
    {"_ZN5clang7APValue10MakeLValueEv", "clang::APValue::MakeLValue()"},
    {"_ZN5clang7APValue9setLValueEPNS_4ExprERKNS_9CharUnitsE", "clang::APValue::setLValue(clang::Expr*, clang::CharUnits const&)"},
    {"_ZN5clang7APValueaSERKS0_", "clang::APValue::operator=(clang::APValue const&)"},
    {"_ZN5clang7APValue10MakeUninitEv", "clang::APValue::MakeUninit()"},
    {"_ZNK5clang7APValue13getLValueBaseEv", "clang::APValue::getLValueBase() const"},
    {"_ZNK5clang7APValue15getLValueOffsetEv", "clang::APValue::getLValueOffset() const"},
    {"_ZNK5clang7APValue4dumpEv", "clang::APValue::dump() const"},
    {"_ZNK5clang7APValue5printERN4llvm11raw_ostreamE", "clang::APValue::print(llvm::raw_ostream&) const"},
    {"_ZN5clang11ASTConsumer18HandleTopLevelDeclENS_12DeclGroupRefE", "clang::ASTConsumer::HandleTopLevelDecl(clang::DeclGroupRef)"},
    {"_ZN5clang11ASTConsumer21HandleInterestingDeclENS_12DeclGroupRefE", "clang::ASTConsumer::HandleInterestingDecl(clang::DeclGroupRef)"},
    {"_ZN5clang10ASTContext29CanonicalTemplateTemplateParm7ProfileERN4llvm16FoldingSetNodeIDEPNS_24TemplateTemplateParmDeclE", "clang::ASTContext::CanonicalTemplateTemplateParm::Profile(llvm::FoldingSetNodeID&, clang::TemplateTemplateParmDecl*)"},
    {"_ZNK5clang10ASTContext36getCanonicalTemplateTemplateParmDeclEPNS_24TemplateTemplateParmDeclE", "clang::ASTContext::getCanonicalTemplateTemplateParmDecl(clang::TemplateTemplateParmDecl*) const"},
    {"_ZNK5clang10ASTContext24getTrivialTypeSourceInfoENS_8QualTypeENS_14SourceLocationE", "clang::ASTContext::getTrivialTypeSourceInfo(clang::QualType, clang::SourceLocation) const"},
    {"_ZN5clang10ASTContext12createCXXABIERKNS_10TargetInfoE", "clang::ASTContext::createCXXABI(clang::TargetInfo const&)"},
    {"_ZN5clang10ASTContextC1ERKNS_11LangOptionsERNS_13SourceManagerERKNS_10TargetInfoERNS_15IdentifierTableERNS_13SelectorTableERNS_7Builtin7ContextEj", "clang::ASTContext::ASTContext(clang::LangOptions const&, clang::SourceManager&, clang::TargetInfo const&, clang::IdentifierTable&, clang::SelectorTable&, clang::Builtin::Context&, unsigned int)"},
    {"_ZN5clang10ASTContextC2ERKNS_11LangOptionsERNS_13SourceManagerERKNS_10TargetInfoERNS_15IdentifierTableERNS_13SelectorTableERNS_7Builtin7ContextEj", "clang::ASTContext::ASTContext(clang::LangOptions const&, clang::SourceManager&, clang::TargetInfo const&, clang::IdentifierTable&, clang::SelectorTable&, clang::Builtin::Context&, unsigned int)"},
    {"_ZN5clang10ASTContext16InitBuiltinTypesEv", "clang::ASTContext::InitBuiltinTypes()"},
    {"_ZN5clang10ASTContextD1Ev", "clang::ASTContext::~ASTContext()"},
    {"_ZN5clang10ASTContextD2Ev", "clang::ASTContext::~ASTContext()"},
    {"_ZN5clang10ASTContext15AddDeallocationEPFvPvES1_", "clang::ASTContext::AddDeallocation(void (*)(void*), void*)"},
    {"_ZN5clang10ASTContext17setExternalSourceERN4llvm9OwningPtrINS_17ExternalASTSourceEEE", "clang::ASTContext::setExternalSource(llvm::OwningPtr<clang::ExternalASTSource>&)"},
    {"_ZNK5clang10ASTContext10PrintStatsEv", "clang::ASTContext::PrintStats() const"},
    {"_ZN5clang10ASTContext15InitBuiltinTypeERNS_7CanQualINS_4TypeEEENS_11BuiltinType4KindE", "clang::ASTContext::InitBuiltinType(clang::CanQual<clang::Type>&, clang::BuiltinType::Kind)"},
    {"_ZNK5clang10ASTContext17getFromTargetTypeEj", "clang::ASTContext::getFromTargetType(unsigned int) const"},
    {"_ZNK5clang10ASTContext14getDiagnosticsEv", "clang::ASTContext::getDiagnostics() const"},
    {"_ZN5clang10ASTContext12getDeclAttrsEPKNS_4DeclE", "clang::ASTContext::getDeclAttrs(clang::Decl const*)"},
    {"_ZN5clang10ASTContext14eraseDeclAttrsEPKNS_4DeclE", "clang::ASTContext::eraseDeclAttrs(clang::Decl const*)"},
    {"_ZN5clang10ASTContext35getInstantiatedFromStaticDataMemberEPKNS_7VarDeclE", "clang::ASTContext::getInstantiatedFromStaticDataMember(clang::VarDecl const*)"},
    {"_ZN5clang10ASTContext35setInstantiatedFromStaticDataMemberEPNS_7VarDeclES2_NS_26TemplateSpecializationKindENS_14SourceLocationE", "clang::ASTContext::setInstantiatedFromStaticDataMember(clang::VarDecl*, clang::VarDecl*, clang::TemplateSpecializationKind, clang::SourceLocation)"},
    {"_ZN5clang10ASTContext28getInstantiatedFromUsingDeclEPNS_9UsingDeclE", "clang::ASTContext::getInstantiatedFromUsingDecl(clang::UsingDecl*)"},
    {"_ZN5clang10ASTContext28setInstantiatedFromUsingDeclEPNS_9UsingDeclEPNS_9NamedDeclE", "clang::ASTContext::setInstantiatedFromUsingDecl(clang::UsingDecl*, clang::NamedDecl*)"},
    {"_ZN5clang10ASTContext34getInstantiatedFromUsingShadowDeclEPNS_15UsingShadowDeclE", "clang::ASTContext::getInstantiatedFromUsingShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang10ASTContext34setInstantiatedFromUsingShadowDeclEPNS_15UsingShadowDeclES2_", "clang::ASTContext::setInstantiatedFromUsingShadowDecl(clang::UsingShadowDecl*, clang::UsingShadowDecl*)"},
    {"_ZN5clang10ASTContext35getInstantiatedFromUnnamedFieldDeclEPNS_9FieldDeclE", "clang::ASTContext::getInstantiatedFromUnnamedFieldDecl(clang::FieldDecl*)"},
    {"_ZN5clang10ASTContext35setInstantiatedFromUnnamedFieldDeclEPNS_9FieldDeclES2_", "clang::ASTContext::setInstantiatedFromUnnamedFieldDecl(clang::FieldDecl*, clang::FieldDecl*)"},
    {"_ZNK5clang10ASTContext24overridden_methods_beginEPKNS_13CXXMethodDeclE", "clang::ASTContext::overridden_methods_begin(clang::CXXMethodDecl const*) const"},
    {"_ZNK5clang10ASTContext22overridden_methods_endEPKNS_13CXXMethodDeclE", "clang::ASTContext::overridden_methods_end(clang::CXXMethodDecl const*) const"},
    {"_ZNK5clang10ASTContext23overridden_methods_sizeEPKNS_13CXXMethodDeclE", "clang::ASTContext::overridden_methods_size(clang::CXXMethodDecl const*) const"},
    {"_ZN5clang10ASTContext19addOverriddenMethodEPKNS_13CXXMethodDeclES3_", "clang::ASTContext::addOverriddenMethod(clang::CXXMethodDecl const*, clang::CXXMethodDecl const*)"},
    {"_ZNK5clang10ASTContext21getFloatTypeSemanticsENS_8QualTypeE", "clang::ASTContext::getFloatTypeSemantics(clang::QualType) const"},
    {"_ZNK5clang10ASTContext12getDeclAlignEPKNS_4DeclEb", "clang::ASTContext::getDeclAlign(clang::Decl const*, bool) const"},
    {"_ZNK5clang10ASTContext14getPointerTypeENS_8QualTypeE", "clang::ASTContext::getPointerType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext14getAsArrayTypeENS_8QualTypeE", "clang::ASTContext::getAsArrayType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext18getBaseElementTypeEPKNS_9ArrayTypeE", "clang::ASTContext::getBaseElementType(clang::ArrayType const*) const"},
    {"_ZNK5clang10ASTContext21getPreferredTypeAlignEPKNS_4TypeE", "clang::ASTContext::getPreferredTypeAlign(clang::Type const*) const"},
    {"_ZNK5clang10ASTContext19toCharUnitsFromBitsEx", "clang::ASTContext::toCharUnitsFromBits(long long) const"},
    {"_ZN5clang10ASTContext18getTypeInfoInCharsEPKNS_4TypeE", "clang::ASTContext::getTypeInfoInChars(clang::Type const*)"},
    {"_ZNK5clang10ASTContext11getTypeInfoEPKNS_4TypeE", "clang::ASTContext::getTypeInfo(clang::Type const*) const"},
    {"_ZN5clang10ASTContext18getTypeInfoInCharsENS_8QualTypeE", "clang::ASTContext::getTypeInfoInChars(clang::QualType)"},
    {"_ZNK5clang10ASTContext18getPointerDiffTypeEv", "clang::ASTContext::getPointerDiffType() const"},
    {"_ZNK5clang10ASTContext25getASTObjCInterfaceLayoutEPKNS_17ObjCInterfaceDeclE", "clang::ASTContext::getASTObjCInterfaceLayout(clang::ObjCInterfaceDecl const*) const"},
    {"_ZNK5clang10ASTContext18getTypeSizeInCharsENS_8QualTypeE", "clang::ASTContext::getTypeSizeInChars(clang::QualType) const"},
    {"_ZNK5clang10ASTContext18getTypeSizeInCharsEPKNS_4TypeE", "clang::ASTContext::getTypeSizeInChars(clang::Type const*) const"},
    {"_ZNK5clang10ASTContext19getTypeAlignInCharsENS_8QualTypeE", "clang::ASTContext::getTypeAlignInChars(clang::QualType) const"},
    {"_ZNK5clang10ASTContext19getTypeAlignInCharsEPKNS_4TypeE", "clang::ASTContext::getTypeAlignInChars(clang::Type const*) const"},
    {"_ZNK5clang10ASTContext23ShallowCollectObjCIvarsEPKNS_17ObjCInterfaceDeclERN4llvm15SmallVectorImplIPNS_12ObjCIvarDeclEEE", "clang::ASTContext::ShallowCollectObjCIvars(clang::ObjCInterfaceDecl const*, llvm::SmallVectorImpl<clang::ObjCIvarDecl*>&) const"},
    {"_ZNK5clang10ASTContext20DeepCollectObjCIvarsEPKNS_17ObjCInterfaceDeclEbRN4llvm15SmallVectorImplIPNS_12ObjCIvarDeclEEE", "clang::ASTContext::DeepCollectObjCIvars(clang::ObjCInterfaceDecl const*, bool, llvm::SmallVectorImpl<clang::ObjCIvarDecl*>&) const"},
    {"_ZN5clang10ASTContext25CollectInheritedProtocolsEPKNS_4DeclERN4llvm11SmallPtrSetIPNS_16ObjCProtocolDeclELj8EEE", "clang::ASTContext::CollectInheritedProtocols(clang::Decl const*, llvm::SmallPtrSet<clang::ObjCProtocolDecl*, 8u>&)"},
    {"_ZNK5clang10ASTContext18CountNonClassIvarsEPKNS_17ObjCInterfaceDeclE", "clang::ASTContext::CountNonClassIvars(clang::ObjCInterfaceDecl const*) const"},
    {"_ZN5clang10ASTContext21getObjCImplementationEPNS_17ObjCInterfaceDeclE", "clang::ASTContext::getObjCImplementation(clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang10ASTContext21getObjCImplementationEPNS_16ObjCCategoryDeclE", "clang::ASTContext::getObjCImplementation(clang::ObjCCategoryDecl*)"},
    {"_ZN5clang10ASTContext21setObjCImplementationEPNS_17ObjCInterfaceDeclEPNS_22ObjCImplementationDeclE", "clang::ASTContext::setObjCImplementation(clang::ObjCInterfaceDecl*, clang::ObjCImplementationDecl*)"},
    {"_ZN5clang10ASTContext21setObjCImplementationEPNS_16ObjCCategoryDeclEPNS_20ObjCCategoryImplDeclE", "clang::ASTContext::setObjCImplementation(clang::ObjCCategoryDecl*, clang::ObjCCategoryImplDecl*)"},
    {"_ZN5clang10ASTContext20getBlockVarCopyInitsEPKNS_7VarDeclE", "clang::ASTContext::getBlockVarCopyInits(clang::VarDecl const*)"},
    {"_ZN5clang10ASTContext20setBlockVarCopyInitsEPNS_7VarDeclEPNS_4ExprE", "clang::ASTContext::setBlockVarCopyInits(clang::VarDecl*, clang::Expr*)"},
    {"_ZNK5clang10ASTContext20CreateTypeSourceInfoENS_8QualTypeEj", "clang::ASTContext::CreateTypeSourceInfo(clang::QualType, unsigned int) const"},
    {"_ZNK5clang10ASTContext30getASTObjCImplementationLayoutEPKNS_22ObjCImplementationDeclE", "clang::ASTContext::getASTObjCImplementationLayout(clang::ObjCImplementationDecl const*) const"},
    {"_ZNK5clang10ASTContext14getExtQualTypeEPKNS_4TypeENS_10QualifiersE", "clang::ASTContext::getExtQualType(clang::Type const*, clang::Qualifiers) const"},
    {"_ZNK5clang10ASTContext20getAddrSpaceQualTypeENS_8QualTypeEj", "clang::ASTContext::getAddrSpaceQualType(clang::QualType, unsigned int) const"},
    {"_ZNK5clang10ASTContext17getObjCGCQualTypeENS_8QualTypeENS_10Qualifiers2GCE", "clang::ASTContext::getObjCGCQualType(clang::QualType, clang::Qualifiers::GC) const"},
    {"_ZN5clang10ASTContext18adjustFunctionTypeEPKNS_12FunctionTypeENS1_7ExtInfoE", "clang::ASTContext::adjustFunctionType(clang::FunctionType const*, clang::FunctionType::ExtInfo)"},
    {"_ZNK5clang10ASTContext22getFunctionNoProtoTypeENS_8QualTypeERKNS_12FunctionType7ExtInfoE", "clang::ASTContext::getFunctionNoProtoType(clang::QualType, clang::FunctionType::ExtInfo const&) const"},
    {"_ZNK5clang10ASTContext15getFunctionTypeENS_8QualTypeEPKS1_jRKNS_17FunctionProtoType12ExtProtoInfoE", "clang::ASTContext::getFunctionType(clang::QualType, clang::QualType const*, unsigned int, clang::FunctionProtoType::ExtProtoInfo const&) const"},
    {"_ZNK5clang10ASTContext14getComplexTypeENS_8QualTypeE", "clang::ASTContext::getComplexType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext19getBlockPointerTypeENS_8QualTypeE", "clang::ASTContext::getBlockPointerType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext22getLValueReferenceTypeENS_8QualTypeEb", "clang::ASTContext::getLValueReferenceType(clang::QualType, bool) const"},
    {"_ZNK5clang10ASTContext22getRValueReferenceTypeENS_8QualTypeE", "clang::ASTContext::getRValueReferenceType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext20getMemberPointerTypeENS_8QualTypeEPKNS_4TypeE", "clang::ASTContext::getMemberPointerType(clang::QualType, clang::Type const*) const"},
    {"_ZNK5clang10ASTContext20getConstantArrayTypeENS_8QualTypeERKN4llvm5APIntENS_9ArrayType17ArraySizeModifierEj", "clang::ASTContext::getConstantArrayType(clang::QualType, llvm::APInt const&, clang::ArrayType::ArraySizeModifier, unsigned int) const"},
    {"_ZNK5clang10ASTContext27getVariableArrayDecayedTypeENS_8QualTypeE", "clang::ASTContext::getVariableArrayDecayedType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext26getDependentSizedArrayTypeENS_8QualTypeEPNS_4ExprENS_9ArrayType17ArraySizeModifierEjNS_11SourceRangeE", "clang::ASTContext::getDependentSizedArrayType(clang::QualType, clang::Expr*, clang::ArrayType::ArraySizeModifier, unsigned int, clang::SourceRange) const"},
    {"_ZNK5clang10ASTContext20getVariableArrayTypeENS_8QualTypeEPNS_4ExprENS_9ArrayType17ArraySizeModifierEjNS_11SourceRangeE", "clang::ASTContext::getVariableArrayType(clang::QualType, clang::Expr*, clang::ArrayType::ArraySizeModifier, unsigned int, clang::SourceRange) const"},
    {"_ZNK5clang10ASTContext22getIncompleteArrayTypeENS_8QualTypeENS_9ArrayType17ArraySizeModifierEj", "clang::ASTContext::getIncompleteArrayType(clang::QualType, clang::ArrayType::ArraySizeModifier, unsigned int) const"},
    {"_ZNK5clang10ASTContext13getVectorTypeENS_8QualTypeEjNS_10VectorType10VectorKindE", "clang::ASTContext::getVectorType(clang::QualType, unsigned int, clang::VectorType::VectorKind) const"},
    {"_ZNK5clang10ASTContext16getExtVectorTypeENS_8QualTypeEj", "clang::ASTContext::getExtVectorType(clang::QualType, unsigned int) const"},
    {"_ZNK5clang10ASTContext30getDependentSizedExtVectorTypeENS_8QualTypeEPNS_4ExprENS_14SourceLocationE", "clang::ASTContext::getDependentSizedExtVectorType(clang::QualType, clang::Expr*, clang::SourceLocation) const"},
    {"_ZNK5clang10ASTContext21getCanonicalParamTypeENS_8QualTypeE", "clang::ASTContext::getCanonicalParamType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext24getInjectedClassNameTypeEPNS_13CXXRecordDeclENS_8QualTypeE", "clang::ASTContext::getInjectedClassNameType(clang::CXXRecordDecl*, clang::QualType) const"},
    {"_ZNK5clang10ASTContext19getTypeDeclTypeSlowEPKNS_8TypeDeclE", "clang::ASTContext::getTypeDeclTypeSlow(clang::TypeDecl const*) const"},
    {"_ZNK5clang10ASTContext14getTypedefTypeEPKNS_11TypedefDeclENS_8QualTypeE", "clang::ASTContext::getTypedefType(clang::TypedefDecl const*, clang::QualType) const"},
    {"_ZNK5clang10ASTContext13getRecordTypeEPKNS_10RecordDeclE", "clang::ASTContext::getRecordType(clang::RecordDecl const*) const"},
    {"_ZNK5clang10ASTContext11getEnumTypeEPKNS_8EnumDeclE", "clang::ASTContext::getEnumType(clang::EnumDecl const*) const"},
    {"_ZN5clang10ASTContext17getAttributedTypeENS_14AttributedType4KindENS_8QualTypeES3_", "clang::ASTContext::getAttributedType(clang::AttributedType::Kind, clang::QualType, clang::QualType)"},
    {"_ZNK5clang10ASTContext28getSubstTemplateTypeParmTypeEPKNS_20TemplateTypeParmTypeENS_8QualTypeE", "clang::ASTContext::getSubstTemplateTypeParmType(clang::TemplateTypeParmType const*, clang::QualType) const"},
    {"_ZN5clang10ASTContext32getSubstTemplateTypeParmPackTypeEPKNS_20TemplateTypeParmTypeERKNS_16TemplateArgumentE", "clang::ASTContext::getSubstTemplateTypeParmPackType(clang::TemplateTypeParmType const*, clang::TemplateArgument const&)"},
    {"_ZNK5clang10ASTContext23getTemplateTypeParmTypeEjjbPNS_14IdentifierInfoE", "clang::ASTContext::getTemplateTypeParmType(unsigned int, unsigned int, bool, clang::IdentifierInfo*) const"},
    {"_ZNK5clang10ASTContext33getTemplateSpecializationTypeInfoENS_12TemplateNameENS_14SourceLocationERKNS_24TemplateArgumentListInfoENS_8QualTypeE", "clang::ASTContext::getTemplateSpecializationTypeInfo(clang::TemplateName, clang::SourceLocation, clang::TemplateArgumentListInfo const&, clang::QualType) const"},
    {"_ZNK5clang10ASTContext29getTemplateSpecializationTypeENS_12TemplateNameERKNS_24TemplateArgumentListInfoENS_8QualTypeE", "clang::ASTContext::getTemplateSpecializationType(clang::TemplateName, clang::TemplateArgumentListInfo const&, clang::QualType) const"},
    {"_ZNK5clang10ASTContext29getTemplateSpecializationTypeENS_12TemplateNameEPKNS_16TemplateArgumentEjNS_8QualTypeE", "clang::ASTContext::getTemplateSpecializationType(clang::TemplateName, clang::TemplateArgument const*, unsigned int, clang::QualType) const"},
    {"_ZNK5clang10ASTContext38getCanonicalTemplateSpecializationTypeENS_12TemplateNameEPKNS_16TemplateArgumentEj", "clang::ASTContext::getCanonicalTemplateSpecializationType(clang::TemplateName, clang::TemplateArgument const*, unsigned int) const"},
    {"_ZNK5clang10ASTContext24getCanonicalTemplateNameENS_12TemplateNameE", "clang::ASTContext::getCanonicalTemplateName(clang::TemplateName) const"},
    {"_ZNK5clang10ASTContext28getCanonicalTemplateArgumentERKNS_16TemplateArgumentE", "clang::ASTContext::getCanonicalTemplateArgument(clang::TemplateArgument const&) const"},
    {"_ZNK5clang10ASTContext17getElaboratedTypeENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierENS_8QualTypeE", "clang::ASTContext::getElaboratedType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::QualType) const"},
    {"_ZNK5clang10ASTContext12getParenTypeENS_8QualTypeE", "clang::ASTContext::getParenType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext20getDependentNameTypeENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoENS_8QualTypeE", "clang::ASTContext::getDependentNameType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, clang::QualType) const"},
    {"_ZNK5clang10ASTContext31getCanonicalNestedNameSpecifierEPNS_19NestedNameSpecifierE", "clang::ASTContext::getCanonicalNestedNameSpecifier(clang::NestedNameSpecifier*) const"},
    {"_ZNK5clang10ASTContext38getDependentTemplateSpecializationTypeENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoERKNS_24TemplateArgumentListInfoE", "clang::ASTContext::getDependentTemplateSpecializationType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, clang::TemplateArgumentListInfo const&) const"},
    {"_ZNK5clang10ASTContext38getDependentTemplateSpecializationTypeENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoEjPKNS_16TemplateArgumentE", "clang::ASTContext::getDependentTemplateSpecializationType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, unsigned int, clang::TemplateArgument const*) const"},
    {"_ZN5clang10ASTContext20getPackExpansionTypeENS_8QualTypeEN4llvm8OptionalIjEE", "clang::ASTContext::getPackExpansionType(clang::QualType, llvm::Optional<unsigned int>)"},
    {"_ZNK5clang10ASTContext17getObjCObjectTypeENS_8QualTypeEPKPNS_16ObjCProtocolDeclEj", "clang::ASTContext::getObjCObjectType(clang::QualType, clang::ObjCProtocolDecl* const*, unsigned int) const"},
    {"_ZNK5clang10ASTContext24getObjCObjectPointerTypeENS_8QualTypeE", "clang::ASTContext::getObjCObjectPointerType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext20getObjCInterfaceTypeEPKNS_17ObjCInterfaceDeclE", "clang::ASTContext::getObjCInterfaceType(clang::ObjCInterfaceDecl const*) const"},
    {"_ZNK5clang10ASTContext17getTypeOfExprTypeEPNS_4ExprE", "clang::ASTContext::getTypeOfExprType(clang::Expr*) const"},
    {"_ZNK5clang10ASTContext13getTypeOfTypeENS_8QualTypeE", "clang::ASTContext::getTypeOfType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext15getDecltypeTypeEPNS_4ExprE", "clang::ASTContext::getDecltypeType(clang::Expr*) const"},
    {"_ZNK5clang10ASTContext14getTagDeclTypeEPKNS_7TagDeclE", "clang::ASTContext::getTagDeclType(clang::TagDecl const*) const"},
    {"_ZNK5clang10ASTContext11getSizeTypeEv", "clang::ASTContext::getSizeType() const"},
    {"_ZNK5clang10ASTContext18getSignedWCharTypeEv", "clang::ASTContext::getSignedWCharType() const"},
    {"_ZNK5clang10ASTContext20getUnsignedWCharTypeEv", "clang::ASTContext::getUnsignedWCharType() const"},
    {"_ZNK5clang10ASTContext19getArrayDecayedTypeENS_8QualTypeE", "clang::ASTContext::getArrayDecayedType(clang::QualType) const"},
    {"_ZN5clang10ASTContext23getUnqualifiedArrayTypeENS_8QualTypeERNS_10QualifiersE", "clang::ASTContext::getUnqualifiedArrayType(clang::QualType, clang::Qualifiers&)"},
    {"_ZN5clang10ASTContext25UnwrapSimilarPointerTypesERNS_8QualTypeES2_", "clang::ASTContext::UnwrapSimilarPointerTypes(clang::QualType&, clang::QualType&)"},
    {"_ZNK5clang10ASTContext18getNameForTemplateENS_12TemplateNameENS_14SourceLocationE", "clang::ASTContext::getNameForTemplate(clang::TemplateName, clang::SourceLocation) const"},
    {"_ZNK5clang10ASTContext32getSubstTemplateTemplateParmPackEPNS_24TemplateTemplateParmDeclERKNS_16TemplateArgumentE", "clang::ASTContext::getSubstTemplateTemplateParmPack(clang::TemplateTemplateParmDecl*, clang::TemplateArgument const&) const"},
    {"_ZN5clang10ASTContext19hasSameTemplateNameENS_12TemplateNameES1_", "clang::ASTContext::hasSameTemplateName(clang::TemplateName, clang::TemplateName)"},
    {"_ZNK5clang10ASTContext24getDependentTemplateNameEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoE", "clang::ASTContext::getDependentTemplateName(clang::NestedNameSpecifier*, clang::IdentifierInfo const*) const"},
    {"_ZNK5clang10ASTContext18getBaseElementTypeENS_8QualTypeE", "clang::ASTContext::getBaseElementType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext28getConstantArrayElementCountEPKNS_17ConstantArrayTypeE", "clang::ASTContext::getConstantArrayElementCount(clang::ConstantArrayType const*) const"},
    {"_ZNK5clang10ASTContext33getFloatingTypeOfSizeWithinDomainENS_8QualTypeES1_", "clang::ASTContext::getFloatingTypeOfSizeWithinDomain(clang::QualType, clang::QualType) const"},
    {"_ZNK5clang10ASTContext20getFloatingTypeOrderENS_8QualTypeES1_", "clang::ASTContext::getFloatingTypeOrder(clang::QualType, clang::QualType) const"},
    {"_ZNK5clang10ASTContext14getIntegerRankEPKNS_4TypeE", "clang::ASTContext::getIntegerRank(clang::Type const*) const"},
    {"_ZNK5clang10ASTContext11getIntWidthENS_8QualTypeE", "clang::ASTContext::getIntWidth(clang::QualType) const"},
    {"_ZNK5clang10ASTContext20isPromotableBitFieldEPNS_4ExprE", "clang::ASTContext::isPromotableBitField(clang::Expr*) const"},
    {"_ZNK5clang10ASTContext22getPromotedIntegerTypeENS_8QualTypeE", "clang::ASTContext::getPromotedIntegerType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext19getIntegerTypeOrderENS_8QualTypeES1_", "clang::ASTContext::getIntegerTypeOrder(clang::QualType, clang::QualType) const"},
    {"_ZNK5clang10ASTContext23getCFConstantStringTypeEv", "clang::ASTContext::getCFConstantStringType() const"},
    {"_ZN5clang10ASTContext23setCFConstantStringTypeENS_8QualTypeE", "clang::ASTContext::setCFConstantStringType(clang::QualType)"},
    {"_ZNK5clang10ASTContext23getNSConstantStringTypeEv", "clang::ASTContext::getNSConstantStringType() const"},
    {"_ZN5clang10ASTContext23setNSConstantStringTypeENS_8QualTypeE", "clang::ASTContext::setNSConstantStringType(clang::QualType)"},
    {"_ZNK5clang10ASTContext31getObjCFastEnumerationStateTypeEv", "clang::ASTContext::getObjCFastEnumerationStateType() const"},
    {"_ZNK5clang10ASTContext22getBlockDescriptorTypeEv", "clang::ASTContext::getBlockDescriptorType() const"},
    {"_ZN5clang10ASTContext22setBlockDescriptorTypeENS_8QualTypeE", "clang::ASTContext::setBlockDescriptorType(clang::QualType)"},
    {"_ZNK5clang10ASTContext30getBlockDescriptorExtendedTypeEv", "clang::ASTContext::getBlockDescriptorExtendedType() const"},
    {"_ZN5clang10ASTContext30setBlockDescriptorExtendedTypeENS_8QualTypeE", "clang::ASTContext::setBlockDescriptorExtendedType(clang::QualType)"},
    {"_ZNK5clang10ASTContext20BlockRequiresCopyingENS_8QualTypeE", "clang::ASTContext::BlockRequiresCopying(clang::QualType) const"},
    {"_ZNK5clang10ASTContext18isObjCNSObjectTypeENS_8QualTypeE", "clang::ASTContext::isObjCNSObjectType(clang::QualType) const"},
    {"_ZNK5clang10ASTContext14BuildByRefTypeEN4llvm9StringRefENS_8QualTypeE", "clang::ASTContext::BuildByRefType(llvm::StringRef, clang::QualType) const"},
    {"_ZNK5clang10ASTContext16getBlockParmTypeEbRN4llvm15SmallVectorImplIPKNS_4ExprEEE", "clang::ASTContext::getBlockParmType(bool, llvm::SmallVectorImpl<clang::Expr const*>&) const"},
    {"_ZN5clang10ASTContext31setObjCFastEnumerationStateTypeENS_8QualTypeE", "clang::ASTContext::setObjCFastEnumerationStateType(clang::QualType)"},
    {"_ZNK5clang10ASTContext23getObjCEncodingTypeSizeENS_8QualTypeE", "clang::ASTContext::getObjCEncodingTypeSize(clang::QualType) const"},
    {"_ZNK5clang10ASTContext23getObjCEncodingForBlockEPKNS_9BlockExprE", "clang::ASTContext::getObjCEncodingForBlock(clang::BlockExpr const*) const"},
    {"_ZNK5clang10ASTContext22getObjCEncodingForTypeENS_8QualTypeERSsPKNS_9FieldDeclE", "clang::ASTContext::getObjCEncodingForType(clang::QualType, std::string&, clang::FieldDecl const*) const"},
    {"_Z17charUnitsToStringRKN5clang9CharUnitsE", "charUnitsToString(clang::CharUnits const&)"},
    {"_ZN5clang10ASTContext30getObjCEncodingForFunctionDeclEPKNS_12FunctionDeclERSs", "clang::ASTContext::getObjCEncodingForFunctionDecl(clang::FunctionDecl const*, std::string&)"},
    {"_ZNK5clang10ASTContext28getObjCEncodingForMethodDeclEPKNS_14ObjCMethodDeclERSs", "clang::ASTContext::getObjCEncodingForMethodDecl(clang::ObjCMethodDecl const*, std::string&) const"},
    {"_ZNK5clang10ASTContext31getObjCEncodingForTypeQualifierENS_4Decl17ObjCDeclQualifierERSs", "clang::ASTContext::getObjCEncodingForTypeQualifier(clang::Decl::ObjCDeclQualifier, std::string&) const"},
    {"_ZNK5clang10ASTContext30getObjCEncodingForPropertyDeclEPKNS_16ObjCPropertyDeclEPKNS_4DeclERSs", "clang::ASTContext::getObjCEncodingForPropertyDecl(clang::ObjCPropertyDecl const*, clang::Decl const*, std::string&) const"},
    {"_ZNK5clang10ASTContext26getObjCEncodingForTypeImplENS_8QualTypeERSsbbPKNS_9FieldDeclEbb", "clang::ASTContext::getObjCEncodingForTypeImpl(clang::QualType, std::string&, bool, bool, clang::FieldDecl const*, bool, bool) const"},
    {"_ZNK5clang10ASTContext29getLegacyIntegralTypeEncodingERNS_8QualTypeE", "clang::ASTContext::getLegacyIntegralTypeEncoding(clang::QualType&) const"},
    {"_Z14EncodeBitFieldPKN5clang10ASTContextERSsNS_8QualTypeEPKNS_9FieldDeclE", "EncodeBitField(clang::ASTContext const*, std::string&, clang::QualType, clang::FieldDecl const*)"},
    {"_Z28ObjCEncodingForPrimitiveKindPKN5clang10ASTContextENS_8QualTypeE", "ObjCEncodingForPrimitiveKind(clang::ASTContext const*, clang::QualType)"},
    {"_ZN5clang10ASTContext20setBuiltinVaListTypeENS_8QualTypeE", "clang::ASTContext::setBuiltinVaListType(clang::QualType)"},
    {"_ZN5clang10ASTContext13setObjCIdTypeENS_8QualTypeE", "clang::ASTContext::setObjCIdType(clang::QualType)"},
    {"_ZN5clang10ASTContext14setObjCSelTypeENS_8QualTypeE", "clang::ASTContext::setObjCSelType(clang::QualType)"},
    {"_ZN5clang10ASTContext16setObjCProtoTypeENS_8QualTypeE", "clang::ASTContext::setObjCProtoType(clang::QualType)"},
    {"_ZN5clang10ASTContext16setObjCClassTypeENS_8QualTypeE", "clang::ASTContext::setObjCClassType(clang::QualType)"},
    {"_ZN5clang10ASTContext30setObjCConstantStringInterfaceEPNS_17ObjCInterfaceDeclE", "clang::ASTContext::setObjCConstantStringInterface(clang::ObjCInterfaceDecl*)"},
    {"_ZNK5clang10ASTContext25getOverloadedTemplateNameENS_21UnresolvedSetIteratorES1_", "clang::ASTContext::getOverloadedTemplateName(clang::UnresolvedSetIterator, clang::UnresolvedSetIterator) const"},
    {"_ZNK5clang10ASTContext24getQualifiedTemplateNameEPNS_19NestedNameSpecifierEbPNS_12TemplateDeclE", "clang::ASTContext::getQualifiedTemplateName(clang::NestedNameSpecifier*, bool, clang::TemplateDecl*) const"},
    {"_ZNK5clang10ASTContext24getDependentTemplateNameEPNS_19NestedNameSpecifierENS_22OverloadedOperatorKindE", "clang::ASTContext::getDependentTemplateName(clang::NestedNameSpecifier*, clang::OverloadedOperatorKind) const"},
    {"_ZNK5clang10ASTContext17getObjCGCAttrKindENS_8QualTypeE", "clang::ASTContext::getObjCGCAttrKind(clang::QualType) const"},
    {"_ZN5clang10ASTContext24areCompatibleVectorTypesENS_8QualTypeES1_", "clang::ASTContext::areCompatibleVectorTypes(clang::QualType, clang::QualType)"},
    {"_ZNK5clang10ASTContext30ProtocolCompatibleWithProtocolEPNS_16ObjCProtocolDeclES2_", "clang::ASTContext::ProtocolCompatibleWithProtocol(clang::ObjCProtocolDecl*, clang::ObjCProtocolDecl*) const"},
    {"_ZN5clang10ASTContext30QualifiedIdConformsQualifiedIdENS_8QualTypeES1_", "clang::ASTContext::QualifiedIdConformsQualifiedId(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext33ObjCQualifiedIdTypesAreCompatibleENS_8QualTypeES1_b", "clang::ASTContext::ObjCQualifiedIdTypesAreCompatible(clang::QualType, clang::QualType, bool)"},
    {"_ZN5clang10ASTContext36ObjCQualifiedClassTypesAreCompatibleENS_8QualTypeES1_", "clang::ASTContext::ObjCQualifiedClassTypesAreCompatible(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext23canAssignObjCInterfacesEPKNS_21ObjCObjectPointerTypeES3_", "clang::ASTContext::canAssignObjCInterfaces(clang::ObjCObjectPointerType const*, clang::ObjCObjectPointerType const*)"},
    {"_ZN5clang10ASTContext23canAssignObjCInterfacesEPKNS_14ObjCObjectTypeES3_", "clang::ASTContext::canAssignObjCInterfaces(clang::ObjCObjectType const*, clang::ObjCObjectType const*)"},
    {"_ZN5clang10ASTContext37canAssignObjCInterfacesInBlockPointerEPKNS_21ObjCObjectPointerTypeES3_", "clang::ASTContext::canAssignObjCInterfacesInBlockPointer(clang::ObjCObjectPointerType const*, clang::ObjCObjectPointerType const*)"},
    {"_ZN5clang10ASTContext23areCommonBaseCompatibleEPKNS_21ObjCObjectPointerTypeES3_", "clang::ASTContext::areCommonBaseCompatible(clang::ObjCObjectPointerType const*, clang::ObjCObjectPointerType const*)"},
    {"_ZN5clang10ASTContext29areComparableObjCPointerTypesENS_8QualTypeES1_", "clang::ASTContext::areComparableObjCPointerTypes(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext21canBindObjCObjectTypeENS_8QualTypeES1_", "clang::ASTContext::canBindObjCObjectType(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext18typesAreCompatibleENS_8QualTypeES1_b", "clang::ASTContext::typesAreCompatible(clang::QualType, clang::QualType, bool)"},
    {"_ZN5clang10ASTContext10mergeTypesENS_8QualTypeES1_bb", "clang::ASTContext::mergeTypes(clang::QualType, clang::QualType, bool, bool)"},
    {"_ZN5clang10ASTContext30typesAreBlockPointerCompatibleENS_8QualTypeES1_", "clang::ASTContext::typesAreBlockPointerCompatible(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext25mergeTransparentUnionTypeENS_8QualTypeES1_bb", "clang::ASTContext::mergeTransparentUnionType(clang::QualType, clang::QualType, bool, bool)"},
    {"_ZN5clang10ASTContext26mergeFunctionArgumentTypesENS_8QualTypeES1_bb", "clang::ASTContext::mergeFunctionArgumentTypes(clang::QualType, clang::QualType, bool, bool)"},
    {"_ZN5clang10ASTContext18mergeFunctionTypesENS_8QualTypeES1_bb", "clang::ASTContext::mergeFunctionTypes(clang::QualType, clang::QualType, bool, bool)"},
    {"_ZN5clang10ASTContext21mergeObjCGCQualifiersENS_8QualTypeES1_", "clang::ASTContext::mergeObjCGCQualifiers(clang::QualType, clang::QualType)"},
    {"_ZN5clang10ASTContext28getCorrespondingUnsignedTypeENS_8QualTypeE", "clang::ASTContext::getCorrespondingUnsignedType(clang::QualType)"},
    {"_ZN5clang17ExternalASTSourceD0Ev", "clang::ExternalASTSource::~ExternalASTSource()"},
    {"_ZN5clang17ExternalASTSourceD1Ev", "clang::ExternalASTSource::~ExternalASTSource()"},
    {"_ZN5clang17ExternalASTSourceD2Ev", "clang::ExternalASTSource::~ExternalASTSource()"},
    {"_ZN5clang17ExternalASTSource10PrintStatsEv", "clang::ExternalASTSource::PrintStats()"},
    {"_ZN5clang19ASTMutationListenerD0Ev", "clang::ASTMutationListener::~ASTMutationListener()"},
    {"_ZN5clang19ASTMutationListenerD1Ev", "clang::ASTMutationListener::~ASTMutationListener()"},
    {"_ZN5clang19ASTMutationListenerD2Ev", "clang::ASTMutationListener::~ASTMutationListener()"},
    {"_ZNK5clang10ASTContext14GetBuiltinTypeEjRNS0_19GetBuiltinTypeErrorEPj", "clang::ASTContext::GetBuiltinType(unsigned int, clang::ASTContext::GetBuiltinTypeError&, unsigned int*) const"},
    {"_Z17DecodeTypeFromStrRPKcRKN5clang10ASTContextERNS3_19GetBuiltinTypeErrorERbb", "DecodeTypeFromStr(char const*&, clang::ASTContext const&, clang::ASTContext::GetBuiltinTypeError&, bool&, bool)"},
    {"_ZN5clang10ASTContext24GetGVALinkageForFunctionEPKNS_12FunctionDeclE", "clang::ASTContext::GetGVALinkageForFunction(clang::FunctionDecl const*)"},
    {"_ZN5clang10ASTContext24GetGVALinkageForVariableEPKNS_7VarDeclE", "clang::ASTContext::GetGVALinkageForVariable(clang::VarDecl const*)"},
    {"_ZN5clang10ASTContext17DeclMustBeEmittedEPKNS_4DeclE", "clang::ASTContext::DeclMustBeEmitted(clang::Decl const*)"},
    {"_ZN5clang10ASTContext24getDefaultMethodCallConvEv", "clang::ASTContext::getDefaultMethodCallConv()"},
    {"_ZNK5clang10ASTContext13isNearlyEmptyEPKNS_13CXXRecordDeclE", "clang::ASTContext::isNearlyEmpty(clang::CXXRecordDecl const*) const"},
    {"_ZN5clang10ASTContext19createMangleContextEv", "clang::ASTContext::createMangleContext()"},
    {"_ZN5clang6CXXABID0Ev", "clang::CXXABI::~CXXABI()"},
    {"_ZN5clang6CXXABID1Ev", "clang::CXXABI::~CXXABI()"},
    {"_ZN5clang6CXXABID2Ev", "clang::CXXABI::~CXXABI()"},
    {"_Z16CmpProtocolNamesPKN5clang16ObjCProtocolDeclES2_", "CmpProtocolNames(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)"},
    {"_ZN4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEED1Ev", "llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang19NestedNameSpecifierEED1Ev", "llvm::FoldingSet<clang::NestedNameSpecifier>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEED1Ev", "llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang21DependentTemplateNameEED1Ev", "llvm::FoldingSet<clang::DependentTemplateName>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang21QualifiedTemplateNameEED1Ev", "llvm::FoldingSet<clang::QualifiedTemplateName>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang14AttributedTypeEED1Ev", "llvm::FoldingSet<clang::AttributedType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEED1Ev", "llvm::FoldingSet<clang::ObjCObjectPointerType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEED1Ev", "llvm::FoldingSet<clang::ObjCObjectTypeImpl>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang17PackExpansionTypeEED1Ev", "llvm::FoldingSet<clang::PackExpansionType>::~FoldingSet()"},
    {"_ZN4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEED1Ev", "llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>::~ContextualFoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang17DependentNameTypeEED1Ev", "llvm::FoldingSet<clang::DependentNameType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang14ElaboratedTypeEED1Ev", "llvm::FoldingSet<clang::ElaboratedType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang9ParenTypeEED1Ev", "llvm::FoldingSet<clang::ParenType>::~FoldingSet()"},
    {"_ZN4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEED1Ev", "llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>::~ContextualFoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEED1Ev", "llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEED1Ev", "llvm::FoldingSet<clang::SubstTemplateTypeParmType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEED1Ev", "llvm::FoldingSet<clang::TemplateTypeParmType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang21DependentDecltypeTypeEED1Ev", "llvm::FoldingSet<clang::DependentDecltypeType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEED1Ev", "llvm::FoldingSet<clang::DependentTypeOfExprType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang17FunctionProtoTypeEED1Ev", "llvm::FoldingSet<clang::FunctionProtoType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEED1Ev", "llvm::FoldingSet<clang::FunctionNoProtoType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang10VectorTypeEED1Ev", "llvm::FoldingSet<clang::VectorType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEED1Ev", "llvm::FoldingSet<clang::DependentSizedExtVectorType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEED1Ev", "llvm::FoldingSet<clang::DependentSizedArrayType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang19IncompleteArrayTypeEED1Ev", "llvm::FoldingSet<clang::IncompleteArrayType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang17ConstantArrayTypeEED1Ev", "llvm::FoldingSet<clang::ConstantArrayType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang17MemberPointerTypeEED1Ev", "llvm::FoldingSet<clang::MemberPointerType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang19RValueReferenceTypeEED1Ev", "llvm::FoldingSet<clang::RValueReferenceType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang19LValueReferenceTypeEED1Ev", "llvm::FoldingSet<clang::LValueReferenceType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang16BlockPointerTypeEED1Ev", "llvm::FoldingSet<clang::BlockPointerType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang11PointerTypeEED1Ev", "llvm::FoldingSet<clang::PointerType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang11ComplexTypeEED1Ev", "llvm::FoldingSet<clang::ComplexType>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang8ExtQualsEED1Ev", "llvm::FoldingSet<clang::ExtQuals>::~FoldingSet()"},
    {"_ZN5clang20UsuallyTinyPtrVectorIKNS_13CXXMethodDeclEE9push_backEPS2_", "clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const>::push_back(clang::CXXMethodDecl const*)"},
    {"_ZNK5clang4Decl15getMaxAlignmentEv", "clang::Decl::getMaxAlignment() const"},
    {"_ZNK5clang16ObjCCategoryDecl9ivar_sizeEv", "clang::ObjCCategoryDecl::ivar_size() const"},
    {"_ZNK5clang22ObjCImplementationDecl9ivar_sizeEv", "clang::ObjCImplementationDecl::ivar_size() const"},
    {"_ZN4llvm15SmallVectorImplIN5clang16TemplateArgumentEE9push_backERKS2_", "llvm::SmallVectorImpl<clang::TemplateArgument>::push_back(clang::TemplateArgument const&)"},
    {"_ZN5clang16TemplateArgumentaSERKS0_", "clang::TemplateArgument::operator=(clang::TemplateArgument const&)"},
    {"_ZNK5clang4Decl7getAttrINS_16ObjCNSObjectAttrEEEPT_v", "clang::ObjCNSObjectAttr* clang::Decl::getAttr<clang::ObjCNSObjectAttr>() const"},
    {"_ZNK5clang4Type17isObjCBuiltinTypeEv", "clang::Type::isObjCBuiltinType() const"},
    {"_ZN5clang17ExternalASTSource12CompleteTypeEPNS_7TagDeclE", "clang::ExternalASTSource::CompleteType(clang::TagDecl*)"},
    {"_ZN5clang17ExternalASTSource12CompleteTypeEPNS_17ObjCInterfaceDeclE", "clang::ExternalASTSource::CompleteType(clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang17ExternalASTSource20StartedDeserializingEv", "clang::ExternalASTSource::StartedDeserializing()"},
    {"_ZN5clang17ExternalASTSource21FinishedDeserializingEv", "clang::ExternalASTSource::FinishedDeserializing()"},
    {"_ZN5clang17ExternalASTSource20StartTranslationUnitEPNS_11ASTConsumerE", "clang::ExternalASTSource::StartTranslationUnit(clang::ASTConsumer*)"},
    {"_ZN5clang19ASTMutationListener22CompletedTagDefinitionEPKNS_7TagDeclE", "clang::ASTMutationListener::CompletedTagDefinition(clang::TagDecl const*)"},
    {"_ZN5clang19ASTMutationListener16AddedVisibleDeclEPKNS_11DeclContextEPKNS_4DeclE", "clang::ASTMutationListener::AddedVisibleDecl(clang::DeclContext const*, clang::Decl const*)"},
    {"_ZN5clang19ASTMutationListener22AddedCXXImplicitMemberEPKNS_13CXXRecordDeclEPKNS_4DeclE", "clang::ASTMutationListener::AddedCXXImplicitMember(clang::CXXRecordDecl const*, clang::Decl const*)"},
    {"_ZN5clang19ASTMutationListener30AddedCXXTemplateSpecializationEPKNS_17ClassTemplateDeclEPKNS_31ClassTemplateSpecializationDeclE", "clang::ASTMutationListener::AddedCXXTemplateSpecialization(clang::ClassTemplateDecl const*, clang::ClassTemplateSpecializationDecl const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang16TemplateArgumentELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::TemplateArgument, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang16TemplateArgumentELb0EE18uninitialized_copyIPS2_S5_EEvT_S6_T0_", "void llvm::SmallVectorTemplateBase<clang::TemplateArgument, false>::uninitialized_copy<clang::TemplateArgument*, clang::TemplateArgument*>(clang::TemplateArgument*, clang::TemplateArgument*, clang::TemplateArgument*)"},
    {"_ZNSt6vectorIPN5clang17VariableArrayTypeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::VariableArrayType*, std::allocator<clang::VariableArrayType*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::VariableArrayType**, std::vector<clang::VariableArrayType*, std::allocator<clang::VariableArrayType*> > >, clang::VariableArrayType* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS1_4ExprENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::VarDecl const*, clang::Expr*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::Expr*> >::InsertIntoBucket(clang::VarDecl const* const&, clang::Expr* const&, std::pair<clang::VarDecl const*, clang::Expr*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS1_4ExprENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::VarDecl const*, clang::Expr*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::Expr*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang17ObjCContainerDeclEPNS1_12ObjCImplDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::ObjCContainerDecl*, clang::ObjCImplDecl*, llvm::DenseMapInfo<clang::ObjCContainerDecl*>, llvm::DenseMapInfo<clang::ObjCImplDecl*> >::InsertIntoBucket(clang::ObjCContainerDecl* const&, clang::ObjCImplDecl* const&, std::pair<clang::ObjCContainerDecl*, clang::ObjCImplDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang17ObjCContainerDeclEPNS1_12ObjCImplDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::ObjCContainerDecl*, clang::ObjCImplDecl*, llvm::DenseMapInfo<clang::ObjCContainerDecl*>, llvm::DenseMapInfo<clang::ObjCImplDecl*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPKN5clang13CXXMethodDeclESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::CXXMethodDecl const*, std::allocator<clang::CXXMethodDecl const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CXXMethodDecl const**, std::vector<clang::CXXMethodDecl const*, std::allocator<clang::CXXMethodDecl const*> > >, clang::CXXMethodDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_20UsuallyTinyPtrVectorIS3_EENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const> > >::InsertIntoBucket(clang::CXXMethodDecl const* const&, clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const> const&, std::pair<clang::CXXMethodDecl const*, clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const> >*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_20UsuallyTinyPtrVectorIS3_EENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const>, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::UsuallyTinyPtrVector<clang::CXXMethodDecl const> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang9FieldDeclES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<clang::FieldDecl*, clang::FieldDecl*, llvm::DenseMapInfo<clang::FieldDecl*>, llvm::DenseMapInfo<clang::FieldDecl*> >::InsertIntoBucket(clang::FieldDecl* const&, clang::FieldDecl* const&, std::pair<clang::FieldDecl*, clang::FieldDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang9FieldDeclES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<clang::FieldDecl*, clang::FieldDecl*, llvm::DenseMapInfo<clang::FieldDecl*>, llvm::DenseMapInfo<clang::FieldDecl*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang15UsingShadowDeclES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<clang::UsingShadowDecl*, clang::UsingShadowDecl*, llvm::DenseMapInfo<clang::UsingShadowDecl*>, llvm::DenseMapInfo<clang::UsingShadowDecl*> >::InsertIntoBucket(clang::UsingShadowDecl* const&, clang::UsingShadowDecl* const&, std::pair<clang::UsingShadowDecl*, clang::UsingShadowDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang15UsingShadowDeclES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<clang::UsingShadowDecl*, clang::UsingShadowDecl*, llvm::DenseMapInfo<clang::UsingShadowDecl*>, llvm::DenseMapInfo<clang::UsingShadowDecl*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang9UsingDeclEPNS1_9NamedDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::UsingDecl*, clang::NamedDecl*, llvm::DenseMapInfo<clang::UsingDecl*>, llvm::DenseMapInfo<clang::NamedDecl*> >::InsertIntoBucket(clang::UsingDecl* const&, clang::NamedDecl* const&, std::pair<clang::UsingDecl*, clang::NamedDecl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang9UsingDeclEPNS1_9NamedDeclENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::UsingDecl*, clang::NamedDecl*, llvm::DenseMapInfo<clang::UsingDecl*>, llvm::DenseMapInfo<clang::NamedDecl*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS1_24MemberSpecializationInfoENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::VarDecl const*, clang::MemberSpecializationInfo*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::MemberSpecializationInfo*> >::InsertIntoBucket(clang::VarDecl const* const&, clang::MemberSpecializationInfo* const&, std::pair<clang::VarDecl const*, clang::MemberSpecializationInfo*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7VarDeclEPNS1_24MemberSpecializationInfoENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::VarDecl const*, clang::MemberSpecializationInfo*, llvm::DenseMapInfo<clang::VarDecl const*>, llvm::DenseMapInfo<clang::MemberSpecializationInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS_11SmallVectorIPNS1_4AttrELj2EEENS_12DenseMapInfoIS4_EENSA_IS9_EEE16InsertIntoBucketERKS4_RKS9_PSt4pairIS4_S9_E", "llvm::DenseMap<clang::Decl const*, llvm::SmallVector<clang::Attr*, 2u>*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::Attr*, 2u>*> >::InsertIntoBucket(clang::Decl const* const&, llvm::SmallVector<clang::Attr*, 2u>* const&, std::pair<clang::Decl const*, llvm::SmallVector<clang::Attr*, 2u>*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4DeclEPNS_11SmallVectorIPNS1_4AttrELj2EEENS_12DenseMapInfoIS4_EENSA_IS9_EEE4growEj", "llvm::DenseMap<clang::Decl const*, llvm::SmallVector<clang::Attr*, 2u>*, llvm::DenseMapInfo<clang::Decl const*>, llvm::DenseMapInfo<llvm::SmallVector<clang::Attr*, 2u>*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN5clang4TypeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::Type*, std::allocator<clang::Type*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Type**, std::vector<clang::Type*, std::allocator<clang::Type*> > >, clang::Type* const&)"},
    {"_ZN4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEED0Ev", "llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS8_", "llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang19NestedNameSpecifierEED0Ev", "llvm::FoldingSet<clang::NestedNameSpecifier>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang19NestedNameSpecifierEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::NestedNameSpecifier>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19NestedNameSpecifierEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::NestedNameSpecifier>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19NestedNameSpecifierEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::NestedNameSpecifier>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEED0Ev", "llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang21DependentTemplateNameEED0Ev", "llvm::FoldingSet<clang::DependentTemplateName>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentTemplateNameEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentTemplateName>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentTemplateNameEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentTemplateName>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentTemplateNameEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentTemplateName>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang21DependentTemplateName7ProfileERN4llvm16FoldingSetNodeIDE", "clang::DependentTemplateName::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN4llvm10FoldingSetIN5clang21QualifiedTemplateNameEED0Ev", "llvm::FoldingSet<clang::QualifiedTemplateName>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang21QualifiedTemplateNameEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::QualifiedTemplateName>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21QualifiedTemplateNameEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::QualifiedTemplateName>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21QualifiedTemplateNameEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::QualifiedTemplateName>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang14AttributedTypeEED0Ev", "llvm::FoldingSet<clang::AttributedType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang14AttributedTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::AttributedType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14AttributedTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::AttributedType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14AttributedTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::AttributedType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEED0Ev", "llvm::FoldingSet<clang::ObjCObjectPointerType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ObjCObjectPointerType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ObjCObjectPointerType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ObjCObjectPointerType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEED0Ev", "llvm::FoldingSet<clang::ObjCObjectTypeImpl>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ObjCObjectTypeImpl>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ObjCObjectTypeImpl>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ObjCObjectTypeImpl>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang17PackExpansionTypeEED0Ev", "llvm::FoldingSet<clang::PackExpansionType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang17PackExpansionTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::PackExpansionType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17PackExpansionTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::PackExpansionType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17PackExpansionTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::PackExpansionType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang17DependentNameTypeEED0Ev", "llvm::FoldingSet<clang::DependentNameType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang17DependentNameTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentNameType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17DependentNameTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentNameType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17DependentNameTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentNameType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang14ElaboratedTypeEED0Ev", "llvm::FoldingSet<clang::ElaboratedType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang14ElaboratedTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ElaboratedType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14ElaboratedTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ElaboratedType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14ElaboratedTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ElaboratedType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang9ParenTypeEED0Ev", "llvm::FoldingSet<clang::ParenType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang9ParenTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ParenType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang9ParenTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ParenType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang9ParenTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ParenType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEED0Ev", "llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEED0Ev", "llvm::FoldingSet<clang::SubstTemplateTypeParmType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTypeParmType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::SubstTemplateTypeParmType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::SubstTemplateTypeParmType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEED0Ev", "llvm::FoldingSet<clang::TemplateTypeParmType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::TemplateTypeParmType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::TemplateTypeParmType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::TemplateTypeParmType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang21DependentDecltypeTypeEED0Ev", "llvm::FoldingSet<clang::DependentDecltypeType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentDecltypeTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentDecltypeType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentDecltypeTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentDecltypeType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang21DependentDecltypeTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentDecltypeType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEED0Ev", "llvm::FoldingSet<clang::DependentTypeOfExprType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentTypeOfExprType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentTypeOfExprType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentTypeOfExprType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang17FunctionProtoTypeEED0Ev", "llvm::FoldingSet<clang::FunctionProtoType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang17FunctionProtoTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionProtoType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17FunctionProtoTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::FunctionProtoType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17FunctionProtoTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionProtoType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEED0Ev", "llvm::FoldingSet<clang::FunctionNoProtoType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionNoProtoType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::FunctionNoProtoType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionNoProtoType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang10VectorTypeEED0Ev", "llvm::FoldingSet<clang::VectorType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang10VectorTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::VectorType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang10VectorTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::VectorType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang10VectorTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::VectorType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEED0Ev", "llvm::FoldingSet<clang::DependentSizedExtVectorType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentSizedExtVectorType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentSizedExtVectorType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentSizedExtVectorType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEED0Ev", "llvm::FoldingSet<clang::DependentSizedArrayType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentSizedArrayType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::DependentSizedArrayType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::DependentSizedArrayType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang19IncompleteArrayTypeEED0Ev", "llvm::FoldingSet<clang::IncompleteArrayType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang19IncompleteArrayTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::IncompleteArrayType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19IncompleteArrayTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::IncompleteArrayType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19IncompleteArrayTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::IncompleteArrayType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang17ConstantArrayTypeEED0Ev", "llvm::FoldingSet<clang::ConstantArrayType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang17ConstantArrayTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ConstantArrayType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17ConstantArrayTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ConstantArrayType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17ConstantArrayTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ConstantArrayType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang17MemberPointerTypeEED0Ev", "llvm::FoldingSet<clang::MemberPointerType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang17MemberPointerTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::MemberPointerType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17MemberPointerTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::MemberPointerType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang17MemberPointerTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::MemberPointerType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang19RValueReferenceTypeEED0Ev", "llvm::FoldingSet<clang::RValueReferenceType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang19RValueReferenceTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::RValueReferenceType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19RValueReferenceTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::RValueReferenceType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19RValueReferenceTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::RValueReferenceType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang19LValueReferenceTypeEED0Ev", "llvm::FoldingSet<clang::LValueReferenceType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang19LValueReferenceTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::LValueReferenceType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19LValueReferenceTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::LValueReferenceType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang19LValueReferenceTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::LValueReferenceType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang16BlockPointerTypeEED0Ev", "llvm::FoldingSet<clang::BlockPointerType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang16BlockPointerTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::BlockPointerType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang16BlockPointerTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::BlockPointerType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang16BlockPointerTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::BlockPointerType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang11PointerTypeEED0Ev", "llvm::FoldingSet<clang::PointerType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang11PointerTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::PointerType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang11PointerTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::PointerType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang11PointerTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::PointerType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang11ComplexTypeEED0Ev", "llvm::FoldingSet<clang::ComplexType>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang11ComplexTypeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ComplexType>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang11ComplexTypeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ComplexType>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang11ComplexTypeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ComplexType>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang8ExtQualsEED0Ev", "llvm::FoldingSet<clang::ExtQuals>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang8ExtQualsEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ExtQuals>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang8ExtQualsEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ExtQuals>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang8ExtQualsEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ExtQuals>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEED0Ev", "llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>::~ContextualFoldingSet()"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS9_", "llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEED0Ev", "llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>::~ContextualFoldingSet()"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS9_", "llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang18getFunctionExtInfoERKNS_4TypeE", "clang::getFunctionExtInfo(clang::Type const&)"},
    {"_ZSt16__introsort_loopIPPN5clang16ObjCProtocolDeclElPFbPKS1_S5_EEvT_S8_T0_T1_", "void std::__introsort_loop<clang::ObjCProtocolDecl**, long, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)>(clang::ObjCProtocolDecl**, clang::ObjCProtocolDecl**, long, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*))"},
    {"_ZSt22__final_insertion_sortIPPN5clang16ObjCProtocolDeclEPFbPKS1_S5_EEvT_S8_T0_", "void std::__final_insertion_sort<clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)>(clang::ObjCProtocolDecl**, clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*))"},
    {"_ZSt16__insertion_sortIPPN5clang16ObjCProtocolDeclEPFbPKS1_S5_EEvT_S8_T0_", "void std::__insertion_sort<clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)>(clang::ObjCProtocolDecl**, clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*))"},
    {"_ZSt13__heap_selectIPPN5clang16ObjCProtocolDeclEPFbPKS1_S5_EEvT_S8_S8_T0_", "void std::__heap_select<clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)>(clang::ObjCProtocolDecl**, clang::ObjCProtocolDecl**, clang::ObjCProtocolDecl**, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*))"},
    {"_ZSt13__adjust_heapIPPN5clang16ObjCProtocolDeclElS2_PFbPKS1_S5_EEvT_T0_S9_T1_T2_", "void std::__adjust_heap<clang::ObjCProtocolDecl**, long, clang::ObjCProtocolDecl*, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*)>(clang::ObjCProtocolDecl**, long, long, clang::ObjCProtocolDecl*, bool (*)(clang::ObjCProtocolDecl const*, clang::ObjCProtocolDecl const*))"},
    {"_ZN5clang31FormatASTNodeDiagnosticArgumentENS_10Diagnostic12ArgumentKindElPKcjS3_jPKSt4pairIS1_lEjRN4llvm15SmallVectorImplIcEEPv", "clang::FormatASTNodeDiagnosticArgument(clang::Diagnostic::ArgumentKind, long, char const*, unsigned int, char const*, unsigned int, std::pair<clang::Diagnostic::ArgumentKind, long> const*, unsigned int, llvm::SmallVectorImpl<char>&, void*)"},
    {"_Z29ConvertTypeToDiagnosticStringRN5clang10ASTContextENS_8QualTypeEPKSt4pairINS_10Diagnostic12ArgumentKindElEj", "ConvertTypeToDiagnosticString(clang::ASTContext&, clang::QualType, std::pair<clang::Diagnostic::ArgumentKind, long> const*, unsigned int)"},
    {"_Z7DesugarRN5clang10ASTContextENS_8QualTypeERb", "Desugar(clang::ASTContext&, clang::QualType, bool&)"},
    {"_Z14ImportCastPathPN5clang8CastExprERN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "ImportCastPath(clang::CastExpr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u>&)"},
    {"_ZN5clang11ASTImporterC1ERNS_10ASTContextERNS_11FileManagerES2_S4_b", "clang::ASTImporter::ASTImporter(clang::ASTContext&, clang::FileManager&, clang::ASTContext&, clang::FileManager&, bool)"},
    {"_ZN5clang11ASTImporterC2ERNS_10ASTContextERNS_11FileManagerES2_S4_b", "clang::ASTImporter::ASTImporter(clang::ASTContext&, clang::FileManager&, clang::ASTContext&, clang::FileManager&, bool)"},
    {"_ZN5clang11ASTImporterD0Ev", "clang::ASTImporter::~ASTImporter()"},
    {"_ZN5clang11ASTImporterD1Ev", "clang::ASTImporter::~ASTImporter()"},
    {"_ZN5clang11ASTImporterD2Ev", "clang::ASTImporter::~ASTImporter()"},
    {"_ZN5clang11ASTImporter6ImportENS_8QualTypeE", "clang::ASTImporter::Import(clang::QualType)"},
    {"_ZN5clang11ASTImporter6ImportEPNS_14TypeSourceInfoE", "clang::ASTImporter::Import(clang::TypeSourceInfo*)"},
    {"_ZN5clang11ASTImporter6ImportEPNS_4DeclE", "clang::ASTImporter::Import(clang::Decl*)"},
    {"_ZN5clang11ASTImporter13ImportContextEPNS_11DeclContextE", "clang::ASTImporter::ImportContext(clang::DeclContext*)"},
    {"_ZN5clang11ASTImporter6ImportEPNS_4ExprE", "clang::ASTImporter::Import(clang::Expr*)"},
    {"_ZN5clang11ASTImporter6ImportEPNS_4StmtE", "clang::ASTImporter::Import(clang::Stmt*)"},
    {"_ZN5clang11ASTImporter6ImportEPNS_19NestedNameSpecifierE", "clang::ASTImporter::Import(clang::NestedNameSpecifier*)"},
    {"_ZN5clang11ASTImporter6ImportENS_12TemplateNameE", "clang::ASTImporter::Import(clang::TemplateName)"},
    {"_ZN5clang11ASTImporter6ImportEPKNS_14IdentifierInfoE", "clang::ASTImporter::Import(clang::IdentifierInfo const*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter22ImportTemplateArgumentERKN5clang16TemplateArgumentE", "(anonymous namespace)::ASTNodeImporter::ImportTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZN5clang11ASTImporter6ImportENS_14SourceLocationE", "clang::ASTImporter::Import(clang::SourceLocation)"},
    {"_ZN5clang11ASTImporter6ImportENS_6FileIDE", "clang::ASTImporter::Import(clang::FileID)"},
    {"_ZN5clang11ASTImporter6ImportENS_11SourceRangeE", "clang::ASTImporter::Import(clang::SourceRange)"},
    {"_ZN5clang11ASTImporter16ImportDefinitionEPNS_4DeclE", "clang::ASTImporter::ImportDefinition(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter17ImportDeclContextEPN5clang11DeclContextEb", "(anonymous namespace)::ASTNodeImporter::ImportDeclContext(clang::DeclContext*, bool)"},
    {"_ZN5clang11ASTImporter6ImportENS_15DeclarationNameE", "clang::ASTImporter::Import(clang::DeclarationName)"},
    {"_ZN5clang11ASTImporter6ImportENS_8SelectorE", "clang::ASTImporter::Import(clang::Selector)"},
    {"_ZN5clang11ASTImporter18HandleNameConflictENS_15DeclarationNameEPNS_11DeclContextEjPPNS_9NamedDeclEj", "clang::ASTImporter::HandleNameConflict(clang::DeclarationName, clang::DeclContext*, unsigned int, clang::NamedDecl**, unsigned int)"},
    {"_ZN5clang11ASTImporter6ToDiagENS_14SourceLocationEj", "clang::ASTImporter::ToDiag(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang11ASTImporter8FromDiagENS_14SourceLocationEj", "clang::ASTImporter::FromDiag(clang::SourceLocation, unsigned int)"},
    {"_ZN5clang11ASTImporter8ImportedEPNS_4DeclES2_", "clang::ASTImporter::Imported(clang::Decl*, clang::Decl*)"},
    {"_ZN5clang11ASTImporter24IsStructurallyEquivalentENS_8QualTypeES1_", "clang::ASTImporter::IsStructurallyEquivalent(clang::QualType, clang::QualType)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::ASTNodeImporter::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter27VisitCompoundAssignOperatorEPN5clang22CompoundAssignOperatorE", "(anonymous namespace)::ASTNodeImporter::VisitCompoundAssignOperator(clang::CompoundAssignOperator*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::ASTNodeImporter::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter9VisitStmtEPN5clang4StmtE", "(anonymous namespace)::ASTNodeImporter::VisitStmt(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter9VisitExprEPN5clang4ExprE", "(anonymous namespace)::ASTNodeImporter::VisitExpr(clang::Expr*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter18VisitNamespaceDeclEPN5clang13NamespaceDeclE", "(anonymous namespace)::ASTNodeImporter::VisitNamespaceDecl(clang::NamespaceDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter21VisitObjCCategoryDeclEPN5clang16ObjCCategoryDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCCategoryDecl(clang::ObjCCategoryDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter25VisitObjCCategoryImplDeclEPN5clang20ObjCCategoryImplDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCCategoryImplDecl(clang::ObjCCategoryImplDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter27VisitObjCImplementationDeclEPN5clang22ObjCImplementationDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCImplementationDecl(clang::ObjCImplementationDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter22VisitObjCInterfaceDeclEPN5clang17ObjCInterfaceDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCInterfaceDecl(clang::ObjCInterfaceDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter21VisitObjCProtocolDeclEPN5clang16ObjCProtocolDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCProtocolDecl(clang::ObjCProtocolDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter19VisitObjCMethodDeclEPN5clang14ObjCMethodDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCMethodDecl(clang::ObjCMethodDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter21VisitObjCPropertyDeclEPN5clang16ObjCPropertyDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCPropertyDecl(clang::ObjCPropertyDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter22VisitClassTemplateDeclEPN5clang17ClassTemplateDeclE", "(anonymous namespace)::ASTNodeImporter::VisitClassTemplateDecl(clang::ClassTemplateDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter29VisitTemplateTemplateParmDeclEPN5clang24TemplateTemplateParmDeclE", "(anonymous namespace)::ASTNodeImporter::VisitTemplateTemplateParmDecl(clang::TemplateTemplateParmDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter13VisitEnumDeclEPN5clang8EnumDeclE", "(anonymous namespace)::ASTNodeImporter::VisitEnumDecl(clang::EnumDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter15VisitRecordDeclEPN5clang10RecordDeclE", "(anonymous namespace)::ASTNodeImporter::VisitRecordDecl(clang::RecordDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter36VisitClassTemplateSpecializationDeclEPN5clang31ClassTemplateSpecializationDeclE", "(anonymous namespace)::ASTNodeImporter::VisitClassTemplateSpecializationDecl(clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter16VisitTypedefDeclEPN5clang11TypedefDeclE", "(anonymous namespace)::ASTNodeImporter::VisitTypedefDecl(clang::TypedefDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter14VisitFieldDeclEPN5clang9FieldDeclE", "(anonymous namespace)::ASTNodeImporter::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter17VisitObjCIvarDeclEPN5clang12ObjCIvarDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCIvarDecl(clang::ObjCIvarDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter17VisitFunctionDeclEPN5clang12FunctionDeclE", "(anonymous namespace)::ASTNodeImporter::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter28VisitNonTypeTemplateParmDeclEPN5clang23NonTypeTemplateParmDeclE", "(anonymous namespace)::ASTNodeImporter::VisitNonTypeTemplateParmDecl(clang::NonTypeTemplateParmDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter12VisitVarDeclEPN5clang7VarDeclE", "(anonymous namespace)::ASTNodeImporter::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter22VisitImplicitParamDeclEPN5clang17ImplicitParamDeclE", "(anonymous namespace)::ASTNodeImporter::VisitImplicitParamDecl(clang::ImplicitParamDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter16VisitParmVarDeclEPN5clang11ParmVarDeclE", "(anonymous namespace)::ASTNodeImporter::VisitParmVarDecl(clang::ParmVarDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter21VisitEnumConstantDeclEPN5clang16EnumConstantDeclE", "(anonymous namespace)::ASTNodeImporter::VisitEnumConstantDecl(clang::EnumConstantDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter22VisitIndirectFieldDeclEPN5clang17IndirectFieldDeclE", "(anonymous namespace)::ASTNodeImporter::VisitIndirectFieldDecl(clang::IndirectFieldDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter18VisitObjCClassDeclEPN5clang13ObjCClassDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCClassDecl(clang::ObjCClassDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter28VisitObjCForwardProtocolDeclEPN5clang23ObjCForwardProtocolDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCForwardProtocolDecl(clang::ObjCForwardProtocolDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter25VisitObjCPropertyImplDeclEPN5clang20ObjCPropertyImplDeclE", "(anonymous namespace)::ASTNodeImporter::VisitObjCPropertyImplDecl(clang::ObjCPropertyImplDecl*)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter15ImportDeclPartsEPN5clang9NamedDeclERPNS1_11DeclContextES6_RNS1_15DeclarationNameERNS1_14SourceLocationE", "(anonymous namespace)::ASTNodeImporter::ImportDeclParts(clang::NamedDecl*, clang::DeclContext*&, clang::DeclContext*&, clang::DeclarationName&, clang::SourceLocation&)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter16ImportDefinitionEPN5clang10RecordDeclES3_", "(anonymous namespace)::ASTNodeImporter::ImportDefinition(clang::RecordDecl*, clang::RecordDecl*)"},
    {"_Z24IsStructurallyEquivalentRN12_GLOBAL__N_128StructuralEquivalenceContextEPN5clang4DeclES4_", "IsStructurallyEquivalent((anonymous namespace)::StructuralEquivalenceContext&, clang::Decl*, clang::Decl*)"},
    {"_ZN12_GLOBAL__N_128StructuralEquivalenceContext6FinishEv", "(anonymous namespace)::StructuralEquivalenceContext::Finish()"},
    {"_Z24IsStructurallyEquivalentRN12_GLOBAL__N_128StructuralEquivalenceContextEN5clang8QualTypeES3_", "IsStructurallyEquivalent((anonymous namespace)::StructuralEquivalenceContext&, clang::QualType, clang::QualType)"},
    {"_Z24IsStructurallyEquivalentRN12_GLOBAL__N_128StructuralEquivalenceContextEPN5clang21TemplateParameterListES4_", "IsStructurallyEquivalent((anonymous namespace)::StructuralEquivalenceContext&, clang::TemplateParameterList*, clang::TemplateParameterList*)"},
    {"_Z24IsStructurallyEquivalentRN12_GLOBAL__N_128StructuralEquivalenceContextERKN5clang16TemplateArgumentES5_", "IsStructurallyEquivalent((anonymous namespace)::StructuralEquivalenceContext&, clang::TemplateArgument const&, clang::TemplateArgument const&)"},
    {"_Z11IsSameValueRKN4llvm6APSIntES2_", "IsSameValue(llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_115ASTNodeImporter27ImportTemplateParameterListEPN5clang21TemplateParameterListE", "(anonymous namespace)::ASTNodeImporter::ImportTemplateParameterList(clang::TemplateParameterList*)"},
    {"_ZN4llvm8DenseMapIN5clang6FileIDES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<clang::FileID, clang::FileID, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<clang::FileID> >::InsertIntoBucket(clang::FileID const&, clang::FileID const&, std::pair<clang::FileID, clang::FileID>*)"},
    {"_ZN4llvm8DenseMapIN5clang6FileIDES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<clang::FileID, clang::FileID, llvm::DenseMapInfo<clang::FileID>, llvm::DenseMapInfo<clang::FileID> >::grow(unsigned int)"},
    {"_ZN5clang14BinaryOperatorC2EPNS_4ExprES2_NS_18BinaryOperatorKindENS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindENS_14SourceLocationEb", "clang::BinaryOperator::BinaryOperator(clang::Expr*, clang::Expr*, clang::BinaryOperatorKind, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind, clang::SourceLocation, bool)"},
    {"_ZN4llvm8DenseMapISt4pairIPN5clang4DeclES4_EcNS_12DenseMapInfoIS5_EENS6_IcEEE6insertERKS1_IS5_cE", "llvm::DenseMap<std::pair<clang::Decl*, clang::Decl*>, char, llvm::DenseMapInfo<std::pair<clang::Decl*, clang::Decl*> >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<clang::Decl*, clang::Decl*>, char> const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPN5clang4DeclES4_EcNS_12DenseMapInfoIS5_EENS6_IcEEE15LookupBucketForERKS5_RPS1_IS5_cE", "llvm::DenseMap<std::pair<clang::Decl*, clang::Decl*>, char, llvm::DenseMapInfo<std::pair<clang::Decl*, clang::Decl*> >, llvm::DenseMapInfo<char> >::LookupBucketFor(std::pair<clang::Decl*, clang::Decl*> const&, std::pair<std::pair<clang::Decl*, clang::Decl*>, char>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPN5clang4DeclES4_EcNS_12DenseMapInfoIS5_EENS6_IcEEE16InsertIntoBucketERKS5_RKcPS1_IS5_cE", "llvm::DenseMap<std::pair<clang::Decl*, clang::Decl*>, char, llvm::DenseMapInfo<std::pair<clang::Decl*, clang::Decl*> >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<clang::Decl*, clang::Decl*> const&, char const&, std::pair<std::pair<clang::Decl*, clang::Decl*>, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPN5clang4DeclES4_EcNS_12DenseMapInfoIS5_EENS6_IcEEE4growEj", "llvm::DenseMap<std::pair<clang::Decl*, clang::Decl*>, char, llvm::DenseMapInfo<std::pair<clang::Decl*, clang::Decl*> >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZNSt5dequeIPN5clang4DeclESaIS2_EE9push_backERKS2_", "std::deque<clang::Decl*, std::allocator<clang::Decl*> >::push_back(clang::Decl* const&)"},
    {"_ZNSt5dequeIPN5clang4DeclESaIS2_EE17_M_reallocate_mapEmb", "std::deque<clang::Decl*, std::allocator<clang::Decl*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeES4_NS_12DenseMapInfoIS4_EES6_E16InsertIntoBucketERKS4_S9_PSt4pairIS4_S4_E", "llvm::DenseMap<clang::Type const*, clang::Type const*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<clang::Type const*> >::InsertIntoBucket(clang::Type const* const&, clang::Type const* const&, std::pair<clang::Type const*, clang::Type const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang4TypeES4_NS_12DenseMapInfoIS4_EES6_E4growEj", "llvm::DenseMap<clang::Type const*, clang::Type const*, llvm::DenseMapInfo<clang::Type const*>, llvm::DenseMapInfo<clang::Type const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<clang::Decl*, clang::Decl*, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<clang::Decl*> >::InsertIntoBucket(clang::Decl* const&, clang::Decl* const&, std::pair<clang::Decl*, clang::Decl*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang4DeclES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<clang::Decl*, clang::Decl*, llvm::DenseMapInfo<clang::Decl*>, llvm::DenseMapInfo<clang::Decl*> >::grow(unsigned int)"},
    {"_ZN5clang4AttrD0Ev", "clang::Attr::~Attr()"},
    {"_ZN5clang4AttrD1Ev", "clang::Attr::~Attr()"},
    {"_ZN5clang4AttrD2Ev", "clang::Attr::~Attr()"},
    {"_ZNK5clang9AliasAttr5cloneERNS_10ASTContextE", "clang::AliasAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang15AlignMac68kAttr5cloneERNS_10ASTContextE", "clang::AlignMac68kAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11AlignedAttr20isAlignmentDependentEv", "clang::AlignedAttr::isAlignmentDependent() const"},
    {"_ZNK5clang11AlignedAttr12getAlignmentERNS_10ASTContextE", "clang::AlignedAttr::getAlignment(clang::ASTContext&) const"},
    {"_ZNK5clang11AlignedAttr5cloneERNS_10ASTContextE", "clang::AlignedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang16AlwaysInlineAttr5cloneERNS_10ASTContextE", "clang::AlwaysInlineAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang20AnalyzerNoReturnAttr5cloneERNS_10ASTContextE", "clang::AnalyzerNoReturnAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12AnnotateAttr5cloneERNS_10ASTContextE", "clang::AnnotateAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12AsmLabelAttr5cloneERNS_10ASTContextE", "clang::AsmLabelAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10BlocksAttr5cloneERNS_10ASTContextE", "clang::BlocksAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang9CDeclAttr5cloneERNS_10ASTContextE", "clang::CDeclAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14CFConsumedAttr5cloneERNS_10ASTContextE", "clang::CFConsumedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang24CFReturnsNotRetainedAttr5cloneERNS_10ASTContextE", "clang::CFReturnsNotRetainedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang21CFReturnsRetainedAttr5cloneERNS_10ASTContextE", "clang::CFReturnsRetainedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang16CUDAConstantAttr5cloneERNS_10ASTContextE", "clang::CUDAConstantAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14CUDADeviceAttr5cloneERNS_10ASTContextE", "clang::CUDADeviceAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14CUDAGlobalAttr5cloneERNS_10ASTContextE", "clang::CUDAGlobalAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12CUDAHostAttr5cloneERNS_10ASTContextE", "clang::CUDAHostAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang20CUDALaunchBoundsAttr5cloneERNS_10ASTContextE", "clang::CUDALaunchBoundsAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14CUDASharedAttr5cloneERNS_10ASTContextE", "clang::CUDASharedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang21CarriesDependencyAttr5cloneERNS_10ASTContextE", "clang::CarriesDependencyAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11CleanupAttr5cloneERNS_10ASTContextE", "clang::CleanupAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10CommonAttr5cloneERNS_10ASTContextE", "clang::CommonAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang9ConstAttr5cloneERNS_10ASTContextE", "clang::ConstAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang15ConstructorAttr5cloneERNS_10ASTContextE", "clang::ConstructorAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13DLLExportAttr5cloneERNS_10ASTContextE", "clang::DLLExportAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13DLLImportAttr5cloneERNS_10ASTContextE", "clang::DLLImportAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14DeprecatedAttr5cloneERNS_10ASTContextE", "clang::DeprecatedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14DestructorAttr5cloneERNS_10ASTContextE", "clang::DestructorAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12ExplicitAttr5cloneERNS_10ASTContextE", "clang::ExplicitAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12FastCallAttr5cloneERNS_10ASTContextE", "clang::FastCallAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang9FinalAttr5cloneERNS_10ASTContextE", "clang::FinalAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10FormatAttr5cloneERNS_10ASTContextE", "clang::FormatAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13FormatArgAttr5cloneERNS_10ASTContextE", "clang::FormatArgAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13GNUInlineAttr5cloneERNS_10ASTContextE", "clang::GNUInlineAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12IBActionAttr5cloneERNS_10ASTContextE", "clang::IBActionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12IBOutletAttr5cloneERNS_10ASTContextE", "clang::IBOutletAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang22IBOutletCollectionAttr5cloneERNS_10ASTContextE", "clang::IBOutletCollectionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang16InitPriorityAttr5cloneERNS_10ASTContextE", "clang::InitPriorityAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang26MBlazeInterruptHandlerAttr5cloneERNS_10ASTContextE", "clang::MBlazeInterruptHandlerAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang23MBlazeSaveVolatilesAttr5cloneERNS_10ASTContextE", "clang::MBlazeSaveVolatilesAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang19MSP430InterruptAttr5cloneERNS_10ASTContextE", "clang::MSP430InterruptAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10MallocAttr5cloneERNS_10ASTContextE", "clang::MallocAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang21MaxFieldAlignmentAttr5cloneERNS_10ASTContextE", "clang::MaxFieldAlignmentAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12MayAliasAttr5cloneERNS_10ASTContextE", "clang::MayAliasAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14NSConsumedAttr5cloneERNS_10ASTContextE", "clang::NSConsumedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang18NSConsumesSelfAttr5cloneERNS_10ASTContextE", "clang::NSConsumesSelfAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang25NSReturnsAutoreleasedAttr5cloneERNS_10ASTContextE", "clang::NSReturnsAutoreleasedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang24NSReturnsNotRetainedAttr5cloneERNS_10ASTContextE", "clang::NSReturnsNotRetainedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang21NSReturnsRetainedAttr5cloneERNS_10ASTContextE", "clang::NSReturnsRetainedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang9NakedAttr5cloneERNS_10ASTContextE", "clang::NakedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12NoCommonAttr5cloneERNS_10ASTContextE", "clang::NoCommonAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11NoDebugAttr5cloneERNS_10ASTContextE", "clang::NoDebugAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12NoInlineAttr5cloneERNS_10ASTContextE", "clang::NoInlineAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang24NoInstrumentFunctionAttr5cloneERNS_10ASTContextE", "clang::NoInstrumentFunctionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12NoReturnAttr5cloneERNS_10ASTContextE", "clang::NoReturnAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11NoThrowAttr5cloneERNS_10ASTContextE", "clang::NoThrowAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11NonNullAttr5cloneERNS_10ASTContextE", "clang::NonNullAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang17ObjCExceptionAttr5cloneERNS_10ASTContextE", "clang::ObjCExceptionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang16ObjCNSObjectAttr5cloneERNS_10ASTContextE", "clang::ObjCNSObjectAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang16OverloadableAttr5cloneERNS_10ASTContextE", "clang::OverloadableAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12OverrideAttr5cloneERNS_10ASTContextE", "clang::OverrideAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13OwnershipAttr5cloneERNS_10ASTContextE", "clang::OwnershipAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10PackedAttr5cloneERNS_10ASTContextE", "clang::PackedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10PascalAttr5cloneERNS_10ASTContextE", "clang::PascalAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang8PureAttr5cloneERNS_10ASTContextE", "clang::PureAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11RegparmAttr5cloneERNS_10ASTContextE", "clang::RegparmAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang21ReqdWorkGroupSizeAttr5cloneERNS_10ASTContextE", "clang::ReqdWorkGroupSizeAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11SectionAttr5cloneERNS_10ASTContextE", "clang::SectionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12SentinelAttr5cloneERNS_10ASTContextE", "clang::SentinelAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11StdCallAttr5cloneERNS_10ASTContextE", "clang::StdCallAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang12ThisCallAttr5cloneERNS_10ASTContextE", "clang::ThisCallAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang20TransparentUnionAttr5cloneERNS_10ASTContextE", "clang::TransparentUnionAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang15UnavailableAttr5cloneERNS_10ASTContextE", "clang::UnavailableAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang10UnusedAttr5cloneERNS_10ASTContextE", "clang::UnusedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang8UsedAttr5cloneERNS_10ASTContextE", "clang::UsedAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang8UuidAttr5cloneERNS_10ASTContextE", "clang::UuidAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang13VecReturnAttr5cloneERNS_10ASTContextE", "clang::VecReturnAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14VisibilityAttr5cloneERNS_10ASTContextE", "clang::VisibilityAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang20WarnUnusedResultAttr5cloneERNS_10ASTContextE", "clang::WarnUnusedResultAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang8WeakAttr5cloneERNS_10ASTContextE", "clang::WeakAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang14WeakImportAttr5cloneERNS_10ASTContextE", "clang::WeakImportAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang11WeakRefAttr5cloneERNS_10ASTContextE", "clang::WeakRefAttr::clone(clang::ASTContext&) const"},
    {"_ZNK5clang27X86ForceAlignArgPointerAttr5cloneERNS_10ASTContextE", "clang::X86ForceAlignArgPointerAttr::clone(clang::ASTContext&) const"},
    {"_ZN5clang9AliasAttrD1Ev", "clang::AliasAttr::~AliasAttr()"},
    {"_ZN5clang9AliasAttrD0Ev", "clang::AliasAttr::~AliasAttr()"},
    {"_ZN5clang15AlignMac68kAttrD1Ev", "clang::AlignMac68kAttr::~AlignMac68kAttr()"},
    {"_ZN5clang15AlignMac68kAttrD0Ev", "clang::AlignMac68kAttr::~AlignMac68kAttr()"},
    {"_ZN5clang11AlignedAttrD1Ev", "clang::AlignedAttr::~AlignedAttr()"},
    {"_ZN5clang11AlignedAttrD0Ev", "clang::AlignedAttr::~AlignedAttr()"},
    {"_ZN5clang16AlwaysInlineAttrD1Ev", "clang::AlwaysInlineAttr::~AlwaysInlineAttr()"},
    {"_ZN5clang16AlwaysInlineAttrD0Ev", "clang::AlwaysInlineAttr::~AlwaysInlineAttr()"},
    {"_ZN5clang20AnalyzerNoReturnAttrD1Ev", "clang::AnalyzerNoReturnAttr::~AnalyzerNoReturnAttr()"},
    {"_ZN5clang20AnalyzerNoReturnAttrD0Ev", "clang::AnalyzerNoReturnAttr::~AnalyzerNoReturnAttr()"},
    {"_ZN5clang12AnnotateAttrD1Ev", "clang::AnnotateAttr::~AnnotateAttr()"},
    {"_ZN5clang12AnnotateAttrD0Ev", "clang::AnnotateAttr::~AnnotateAttr()"},
    {"_ZN5clang12AsmLabelAttrD1Ev", "clang::AsmLabelAttr::~AsmLabelAttr()"},
    {"_ZN5clang12AsmLabelAttrD0Ev", "clang::AsmLabelAttr::~AsmLabelAttr()"},
    {"_ZN5clang10BlocksAttrD1Ev", "clang::BlocksAttr::~BlocksAttr()"},
    {"_ZN5clang10BlocksAttrD0Ev", "clang::BlocksAttr::~BlocksAttr()"},
    {"_ZN5clang9CDeclAttrD1Ev", "clang::CDeclAttr::~CDeclAttr()"},
    {"_ZN5clang9CDeclAttrD0Ev", "clang::CDeclAttr::~CDeclAttr()"},
    {"_ZN5clang14CFConsumedAttrD1Ev", "clang::CFConsumedAttr::~CFConsumedAttr()"},
    {"_ZN5clang14CFConsumedAttrD0Ev", "clang::CFConsumedAttr::~CFConsumedAttr()"},
    {"_ZN5clang24CFReturnsNotRetainedAttrD1Ev", "clang::CFReturnsNotRetainedAttr::~CFReturnsNotRetainedAttr()"},
    {"_ZN5clang24CFReturnsNotRetainedAttrD0Ev", "clang::CFReturnsNotRetainedAttr::~CFReturnsNotRetainedAttr()"},
    {"_ZN5clang21CFReturnsRetainedAttrD1Ev", "clang::CFReturnsRetainedAttr::~CFReturnsRetainedAttr()"},
    {"_ZN5clang21CFReturnsRetainedAttrD0Ev", "clang::CFReturnsRetainedAttr::~CFReturnsRetainedAttr()"},
    {"_ZN5clang16CUDAConstantAttrD1Ev", "clang::CUDAConstantAttr::~CUDAConstantAttr()"},
    {"_ZN5clang16CUDAConstantAttrD0Ev", "clang::CUDAConstantAttr::~CUDAConstantAttr()"},
    {"_ZN5clang14CUDADeviceAttrD1Ev", "clang::CUDADeviceAttr::~CUDADeviceAttr()"},
    {"_ZN5clang14CUDADeviceAttrD0Ev", "clang::CUDADeviceAttr::~CUDADeviceAttr()"},
    {"_ZN5clang14CUDAGlobalAttrD1Ev", "clang::CUDAGlobalAttr::~CUDAGlobalAttr()"},
    {"_ZN5clang14CUDAGlobalAttrD0Ev", "clang::CUDAGlobalAttr::~CUDAGlobalAttr()"},
    {"_ZN5clang12CUDAHostAttrD1Ev", "clang::CUDAHostAttr::~CUDAHostAttr()"},
    {"_ZN5clang12CUDAHostAttrD0Ev", "clang::CUDAHostAttr::~CUDAHostAttr()"},
    {"_ZN5clang20CUDALaunchBoundsAttrD1Ev", "clang::CUDALaunchBoundsAttr::~CUDALaunchBoundsAttr()"},
    {"_ZN5clang20CUDALaunchBoundsAttrD0Ev", "clang::CUDALaunchBoundsAttr::~CUDALaunchBoundsAttr()"},
    {"_ZN5clang14CUDASharedAttrD1Ev", "clang::CUDASharedAttr::~CUDASharedAttr()"},
    {"_ZN5clang14CUDASharedAttrD0Ev", "clang::CUDASharedAttr::~CUDASharedAttr()"},
    {"_ZN5clang21CarriesDependencyAttrD1Ev", "clang::CarriesDependencyAttr::~CarriesDependencyAttr()"},
    {"_ZN5clang21CarriesDependencyAttrD0Ev", "clang::CarriesDependencyAttr::~CarriesDependencyAttr()"},
    {"_ZN5clang11CleanupAttrD1Ev", "clang::CleanupAttr::~CleanupAttr()"},
    {"_ZN5clang11CleanupAttrD0Ev", "clang::CleanupAttr::~CleanupAttr()"},
    {"_ZN5clang10CommonAttrD1Ev", "clang::CommonAttr::~CommonAttr()"},
    {"_ZN5clang10CommonAttrD0Ev", "clang::CommonAttr::~CommonAttr()"},
    {"_ZN5clang9ConstAttrD1Ev", "clang::ConstAttr::~ConstAttr()"},
    {"_ZN5clang9ConstAttrD0Ev", "clang::ConstAttr::~ConstAttr()"},
    {"_ZN5clang15ConstructorAttrD1Ev", "clang::ConstructorAttr::~ConstructorAttr()"},
    {"_ZN5clang15ConstructorAttrD0Ev", "clang::ConstructorAttr::~ConstructorAttr()"},
    {"_ZN5clang13DLLExportAttrD1Ev", "clang::DLLExportAttr::~DLLExportAttr()"},
    {"_ZN5clang13DLLExportAttrD0Ev", "clang::DLLExportAttr::~DLLExportAttr()"},
    {"_ZN5clang13DLLImportAttrD1Ev", "clang::DLLImportAttr::~DLLImportAttr()"},
    {"_ZN5clang13DLLImportAttrD0Ev", "clang::DLLImportAttr::~DLLImportAttr()"},
    {"_ZN5clang14DeprecatedAttrD1Ev", "clang::DeprecatedAttr::~DeprecatedAttr()"},
    {"_ZN5clang14DeprecatedAttrD0Ev", "clang::DeprecatedAttr::~DeprecatedAttr()"},
    {"_ZN5clang14DestructorAttrD1Ev", "clang::DestructorAttr::~DestructorAttr()"},
    {"_ZN5clang14DestructorAttrD0Ev", "clang::DestructorAttr::~DestructorAttr()"},
    {"_ZN5clang12ExplicitAttrD1Ev", "clang::ExplicitAttr::~ExplicitAttr()"},
    {"_ZN5clang12ExplicitAttrD0Ev", "clang::ExplicitAttr::~ExplicitAttr()"},
    {"_ZN5clang12FastCallAttrD1Ev", "clang::FastCallAttr::~FastCallAttr()"},
    {"_ZN5clang12FastCallAttrD0Ev", "clang::FastCallAttr::~FastCallAttr()"},
    {"_ZN5clang9FinalAttrD1Ev", "clang::FinalAttr::~FinalAttr()"},
    {"_ZN5clang9FinalAttrD0Ev", "clang::FinalAttr::~FinalAttr()"},
    {"_ZN5clang10FormatAttrD1Ev", "clang::FormatAttr::~FormatAttr()"},
    {"_ZN5clang10FormatAttrD0Ev", "clang::FormatAttr::~FormatAttr()"},
    {"_ZN5clang13FormatArgAttrD1Ev", "clang::FormatArgAttr::~FormatArgAttr()"},
    {"_ZN5clang13FormatArgAttrD0Ev", "clang::FormatArgAttr::~FormatArgAttr()"},
    {"_ZN5clang13GNUInlineAttrD1Ev", "clang::GNUInlineAttr::~GNUInlineAttr()"},
    {"_ZN5clang13GNUInlineAttrD0Ev", "clang::GNUInlineAttr::~GNUInlineAttr()"},
    {"_ZN5clang12IBActionAttrD1Ev", "clang::IBActionAttr::~IBActionAttr()"},
    {"_ZN5clang12IBActionAttrD0Ev", "clang::IBActionAttr::~IBActionAttr()"},
    {"_ZN5clang12IBOutletAttrD1Ev", "clang::IBOutletAttr::~IBOutletAttr()"},
    {"_ZN5clang12IBOutletAttrD0Ev", "clang::IBOutletAttr::~IBOutletAttr()"},
    {"_ZN5clang22IBOutletCollectionAttrD1Ev", "clang::IBOutletCollectionAttr::~IBOutletCollectionAttr()"},
    {"_ZN5clang22IBOutletCollectionAttrD0Ev", "clang::IBOutletCollectionAttr::~IBOutletCollectionAttr()"},
    {"_ZN5clang16InitPriorityAttrD1Ev", "clang::InitPriorityAttr::~InitPriorityAttr()"},
    {"_ZN5clang16InitPriorityAttrD0Ev", "clang::InitPriorityAttr::~InitPriorityAttr()"},
    {"_ZN5clang26MBlazeInterruptHandlerAttrD1Ev", "clang::MBlazeInterruptHandlerAttr::~MBlazeInterruptHandlerAttr()"},
    {"_ZN5clang26MBlazeInterruptHandlerAttrD0Ev", "clang::MBlazeInterruptHandlerAttr::~MBlazeInterruptHandlerAttr()"},
    {"_ZN5clang23MBlazeSaveVolatilesAttrD1Ev", "clang::MBlazeSaveVolatilesAttr::~MBlazeSaveVolatilesAttr()"},
    {"_ZN5clang23MBlazeSaveVolatilesAttrD0Ev", "clang::MBlazeSaveVolatilesAttr::~MBlazeSaveVolatilesAttr()"},
    {"_ZN5clang19MSP430InterruptAttrD1Ev", "clang::MSP430InterruptAttr::~MSP430InterruptAttr()"},
    {"_ZN5clang19MSP430InterruptAttrD0Ev", "clang::MSP430InterruptAttr::~MSP430InterruptAttr()"},
    {"_ZN5clang10MallocAttrD1Ev", "clang::MallocAttr::~MallocAttr()"},
    {"_ZN5clang10MallocAttrD0Ev", "clang::MallocAttr::~MallocAttr()"},
    {"_ZN5clang21MaxFieldAlignmentAttrD1Ev", "clang::MaxFieldAlignmentAttr::~MaxFieldAlignmentAttr()"},
    {"_ZN5clang21MaxFieldAlignmentAttrD0Ev", "clang::MaxFieldAlignmentAttr::~MaxFieldAlignmentAttr()"},
    {"_ZN5clang12MayAliasAttrD1Ev", "clang::MayAliasAttr::~MayAliasAttr()"},
    {"_ZN5clang12MayAliasAttrD0Ev", "clang::MayAliasAttr::~MayAliasAttr()"},
    {"_ZN5clang14NSConsumedAttrD1Ev", "clang::NSConsumedAttr::~NSConsumedAttr()"},
    {"_ZN5clang14NSConsumedAttrD0Ev", "clang::NSConsumedAttr::~NSConsumedAttr()"},
    {"_ZN5clang18NSConsumesSelfAttrD1Ev", "clang::NSConsumesSelfAttr::~NSConsumesSelfAttr()"},
    {"_ZN5clang18NSConsumesSelfAttrD0Ev", "clang::NSConsumesSelfAttr::~NSConsumesSelfAttr()"},
    {"_ZN5clang25NSReturnsAutoreleasedAttrD1Ev", "clang::NSReturnsAutoreleasedAttr::~NSReturnsAutoreleasedAttr()"},
    {"_ZN5clang25NSReturnsAutoreleasedAttrD0Ev", "clang::NSReturnsAutoreleasedAttr::~NSReturnsAutoreleasedAttr()"},
    {"_ZN5clang24NSReturnsNotRetainedAttrD1Ev", "clang::NSReturnsNotRetainedAttr::~NSReturnsNotRetainedAttr()"},
    {"_ZN5clang24NSReturnsNotRetainedAttrD0Ev", "clang::NSReturnsNotRetainedAttr::~NSReturnsNotRetainedAttr()"},
    {"_ZN5clang21NSReturnsRetainedAttrD1Ev", "clang::NSReturnsRetainedAttr::~NSReturnsRetainedAttr()"},
    {"_ZN5clang21NSReturnsRetainedAttrD0Ev", "clang::NSReturnsRetainedAttr::~NSReturnsRetainedAttr()"},
    {"_ZN5clang9NakedAttrD1Ev", "clang::NakedAttr::~NakedAttr()"},
    {"_ZN5clang9NakedAttrD0Ev", "clang::NakedAttr::~NakedAttr()"},
    {"_ZN5clang12NoCommonAttrD1Ev", "clang::NoCommonAttr::~NoCommonAttr()"},
    {"_ZN5clang12NoCommonAttrD0Ev", "clang::NoCommonAttr::~NoCommonAttr()"},
    {"_ZN5clang11NoDebugAttrD1Ev", "clang::NoDebugAttr::~NoDebugAttr()"},
    {"_ZN5clang11NoDebugAttrD0Ev", "clang::NoDebugAttr::~NoDebugAttr()"},
    {"_ZN5clang12NoInlineAttrD1Ev", "clang::NoInlineAttr::~NoInlineAttr()"},
    {"_ZN5clang12NoInlineAttrD0Ev", "clang::NoInlineAttr::~NoInlineAttr()"},
    {"_ZN5clang24NoInstrumentFunctionAttrD1Ev", "clang::NoInstrumentFunctionAttr::~NoInstrumentFunctionAttr()"},
    {"_ZN5clang24NoInstrumentFunctionAttrD0Ev", "clang::NoInstrumentFunctionAttr::~NoInstrumentFunctionAttr()"},
    {"_ZN5clang12NoReturnAttrD1Ev", "clang::NoReturnAttr::~NoReturnAttr()"},
    {"_ZN5clang12NoReturnAttrD0Ev", "clang::NoReturnAttr::~NoReturnAttr()"},
    {"_ZN5clang11NoThrowAttrD1Ev", "clang::NoThrowAttr::~NoThrowAttr()"},
    {"_ZN5clang11NoThrowAttrD0Ev", "clang::NoThrowAttr::~NoThrowAttr()"},
    {"_ZN5clang11NonNullAttrD1Ev", "clang::NonNullAttr::~NonNullAttr()"},
    {"_ZN5clang11NonNullAttrD0Ev", "clang::NonNullAttr::~NonNullAttr()"},
    {"_ZN5clang17ObjCExceptionAttrD1Ev", "clang::ObjCExceptionAttr::~ObjCExceptionAttr()"},
    {"_ZN5clang17ObjCExceptionAttrD0Ev", "clang::ObjCExceptionAttr::~ObjCExceptionAttr()"},
    {"_ZN5clang16ObjCNSObjectAttrD1Ev", "clang::ObjCNSObjectAttr::~ObjCNSObjectAttr()"},
    {"_ZN5clang16ObjCNSObjectAttrD0Ev", "clang::ObjCNSObjectAttr::~ObjCNSObjectAttr()"},
    {"_ZN5clang16OverloadableAttrD1Ev", "clang::OverloadableAttr::~OverloadableAttr()"},
    {"_ZN5clang16OverloadableAttrD0Ev", "clang::OverloadableAttr::~OverloadableAttr()"},
    {"_ZN5clang12OverrideAttrD1Ev", "clang::OverrideAttr::~OverrideAttr()"},
    {"_ZN5clang12OverrideAttrD0Ev", "clang::OverrideAttr::~OverrideAttr()"},
    {"_ZN5clang13OwnershipAttrD1Ev", "clang::OwnershipAttr::~OwnershipAttr()"},
    {"_ZN5clang13OwnershipAttrD0Ev", "clang::OwnershipAttr::~OwnershipAttr()"},
    {"_ZN5clang10PackedAttrD1Ev", "clang::PackedAttr::~PackedAttr()"},
    {"_ZN5clang10PackedAttrD0Ev", "clang::PackedAttr::~PackedAttr()"},
    {"_ZN5clang10PascalAttrD1Ev", "clang::PascalAttr::~PascalAttr()"},
    {"_ZN5clang10PascalAttrD0Ev", "clang::PascalAttr::~PascalAttr()"},
    {"_ZN5clang8PureAttrD1Ev", "clang::PureAttr::~PureAttr()"},
    {"_ZN5clang8PureAttrD0Ev", "clang::PureAttr::~PureAttr()"},
    {"_ZN5clang11RegparmAttrD1Ev", "clang::RegparmAttr::~RegparmAttr()"},
    {"_ZN5clang11RegparmAttrD0Ev", "clang::RegparmAttr::~RegparmAttr()"},
    {"_ZN5clang21ReqdWorkGroupSizeAttrD1Ev", "clang::ReqdWorkGroupSizeAttr::~ReqdWorkGroupSizeAttr()"},
    {"_ZN5clang21ReqdWorkGroupSizeAttrD0Ev", "clang::ReqdWorkGroupSizeAttr::~ReqdWorkGroupSizeAttr()"},
    {"_ZN5clang11SectionAttrD1Ev", "clang::SectionAttr::~SectionAttr()"},
    {"_ZN5clang11SectionAttrD0Ev", "clang::SectionAttr::~SectionAttr()"},
    {"_ZN5clang12SentinelAttrD1Ev", "clang::SentinelAttr::~SentinelAttr()"},
    {"_ZN5clang12SentinelAttrD0Ev", "clang::SentinelAttr::~SentinelAttr()"},
    {"_ZN5clang11StdCallAttrD1Ev", "clang::StdCallAttr::~StdCallAttr()"},
    {"_ZN5clang11StdCallAttrD0Ev", "clang::StdCallAttr::~StdCallAttr()"},
    {"_ZN5clang12ThisCallAttrD1Ev", "clang::ThisCallAttr::~ThisCallAttr()"},
    {"_ZN5clang12ThisCallAttrD0Ev", "clang::ThisCallAttr::~ThisCallAttr()"},
    {"_ZN5clang20TransparentUnionAttrD1Ev", "clang::TransparentUnionAttr::~TransparentUnionAttr()"},
    {"_ZN5clang20TransparentUnionAttrD0Ev", "clang::TransparentUnionAttr::~TransparentUnionAttr()"},
    {"_ZN5clang15UnavailableAttrD1Ev", "clang::UnavailableAttr::~UnavailableAttr()"},
    {"_ZN5clang15UnavailableAttrD0Ev", "clang::UnavailableAttr::~UnavailableAttr()"},
    {"_ZN5clang10UnusedAttrD1Ev", "clang::UnusedAttr::~UnusedAttr()"},
    {"_ZN5clang10UnusedAttrD0Ev", "clang::UnusedAttr::~UnusedAttr()"},
    {"_ZN5clang8UsedAttrD1Ev", "clang::UsedAttr::~UsedAttr()"},
    {"_ZN5clang8UsedAttrD0Ev", "clang::UsedAttr::~UsedAttr()"},
    {"_ZN5clang8UuidAttrD1Ev", "clang::UuidAttr::~UuidAttr()"},
    {"_ZN5clang8UuidAttrD0Ev", "clang::UuidAttr::~UuidAttr()"},
    {"_ZN5clang13VecReturnAttrD1Ev", "clang::VecReturnAttr::~VecReturnAttr()"},
    {"_ZN5clang13VecReturnAttrD0Ev", "clang::VecReturnAttr::~VecReturnAttr()"},
    {"_ZN5clang14VisibilityAttrD1Ev", "clang::VisibilityAttr::~VisibilityAttr()"},
    {"_ZN5clang14VisibilityAttrD0Ev", "clang::VisibilityAttr::~VisibilityAttr()"},
    {"_ZN5clang20WarnUnusedResultAttrD1Ev", "clang::WarnUnusedResultAttr::~WarnUnusedResultAttr()"},
    {"_ZN5clang20WarnUnusedResultAttrD0Ev", "clang::WarnUnusedResultAttr::~WarnUnusedResultAttr()"},
    {"_ZN5clang8WeakAttrD1Ev", "clang::WeakAttr::~WeakAttr()"},
    {"_ZN5clang8WeakAttrD0Ev", "clang::WeakAttr::~WeakAttr()"},
    {"_ZN5clang14WeakImportAttrD1Ev", "clang::WeakImportAttr::~WeakImportAttr()"},
    {"_ZN5clang14WeakImportAttrD0Ev", "clang::WeakImportAttr::~WeakImportAttr()"},
    {"_ZN5clang11WeakRefAttrD1Ev", "clang::WeakRefAttr::~WeakRefAttr()"},
    {"_ZN5clang11WeakRefAttrD0Ev", "clang::WeakRefAttr::~WeakRefAttr()"},
    {"_ZN5clang27X86ForceAlignArgPointerAttrD1Ev", "clang::X86ForceAlignArgPointerAttr::~X86ForceAlignArgPointerAttr()"},
    {"_ZN5clang27X86ForceAlignArgPointerAttrD0Ev", "clang::X86ForceAlignArgPointerAttr::~X86ForceAlignArgPointerAttr()"},
    {"_ZN5clang12CXXBasePaths17ComputeDeclsFoundEv", "clang::CXXBasePaths::ComputeDeclsFound()"},
    {"_ZN5clang12CXXBasePaths17found_decls_beginEv", "clang::CXXBasePaths::found_decls_begin()"},
    {"_ZN5clang12CXXBasePaths15found_decls_endEv", "clang::CXXBasePaths::found_decls_end()"},
    {"_ZN5clang12CXXBasePaths11isAmbiguousENS_7CanQualINS_4TypeEEE", "clang::CXXBasePaths::isAmbiguous(clang::CanQual<clang::Type>)"},
    {"_ZN5clang12CXXBasePaths5clearEv", "clang::CXXBasePaths::clear()"},
    {"_ZN5clang12CXXBasePaths4swapERS0_", "clang::CXXBasePaths::swap(clang::CXXBasePaths&)"},
    {"_ZNK5clang13CXXRecordDecl13isDerivedFromEPKS0_", "clang::CXXRecordDecl::isDerivedFrom(clang::CXXRecordDecl const*) const"},
    {"_ZNK5clang13CXXRecordDecl13isDerivedFromEPKS0_RNS_12CXXBasePathsE", "clang::CXXRecordDecl::isDerivedFrom(clang::CXXRecordDecl const*, clang::CXXBasePaths&) const"},
    {"_ZNK5clang13CXXRecordDecl13lookupInBasesEPFbPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPvES6_RNS_12CXXBasePathsE", "clang::CXXRecordDecl::lookupInBases(bool (*)(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*), void*, clang::CXXBasePaths&) const"},
    {"_ZN5clang13CXXRecordDecl13FindBaseClassEPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPv", "clang::CXXRecordDecl::FindBaseClass(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZNK5clang13CXXRecordDecl22isVirtuallyDerivedFromEPS0_", "clang::CXXRecordDecl::isVirtuallyDerivedFrom(clang::CXXRecordDecl*) const"},
    {"_ZN5clang13CXXRecordDecl20FindVirtualBaseClassEPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPv", "clang::CXXRecordDecl::FindVirtualBaseClass(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZNK5clang13CXXRecordDecl24isProvablyNotDerivedFromEPKS0_", "clang::CXXRecordDecl::isProvablyNotDerivedFrom(clang::CXXRecordDecl const*) const"},
    {"_ZNK5clang13CXXRecordDecl11forallBasesEPFbPKS0_PvES3_b", "clang::CXXRecordDecl::forallBases(bool (*)(clang::CXXRecordDecl const*, void*), void*, bool) const"},
    {"_Z9BaseIsNotPKN5clang13CXXRecordDeclEPv", "BaseIsNot(clang::CXXRecordDecl const*, void*)"},
    {"_ZN5clang12CXXBasePaths13lookupInBasesERNS_10ASTContextEPKNS_13CXXRecordDeclEPFbPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPvESB_", "clang::CXXBasePaths::lookupInBases(clang::ASTContext&, clang::CXXRecordDecl const*, bool (*)(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*), void*)"},
    {"_ZN5clang13CXXRecordDecl13FindTagMemberEPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPv", "clang::CXXRecordDecl::FindTagMember(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang13CXXRecordDecl18FindOrdinaryMemberEPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPv", "clang::CXXRecordDecl::FindOrdinaryMember(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang13CXXRecordDecl29FindNestedNameSpecifierMemberEPKNS_16CXXBaseSpecifierERNS_11CXXBasePathEPv", "clang::CXXRecordDecl::FindNestedNameSpecifierMember(clang::CXXBaseSpecifier const*, clang::CXXBasePath&, void*)"},
    {"_ZN5clang17OverridingMethods3addEjNS_19UniqueVirtualMethodE", "clang::OverridingMethods::add(unsigned int, clang::UniqueVirtualMethod)"},
    {"_ZN5clang17OverridingMethods3addERKS0_", "clang::OverridingMethods::add(clang::OverridingMethods const&)"},
    {"_ZN5clang17OverridingMethods10replaceAllENS_19UniqueVirtualMethodE", "clang::OverridingMethods::replaceAll(clang::UniqueVirtualMethod)"},
    {"_ZNK5clang13CXXRecordDecl18getFinalOverridersERNS_20CXXFinalOverriderMapE", "clang::CXXRecordDecl::getFinalOverriders(clang::CXXFinalOverriderMap&) const"},
    {"_ZN12_GLOBAL__N_123FinalOverriderCollector7CollectEPKN5clang13CXXRecordDeclEbS4_RNS1_20CXXFinalOverriderMapE", "(anonymous namespace)::FinalOverriderCollector::Collect(clang::CXXRecordDecl const*, bool, clang::CXXRecordDecl const*, clang::CXXFinalOverriderMap&)"},
    {"_ZNK5clang13CXXRecordDecl23getIndirectPrimaryBasesERNS_25CXXIndirectPrimaryBaseSetE", "clang::CXXRecordDecl::getIndirectPrimaryBases(clang::CXXIndirectPrimaryBaseSet&) const"},
    {"_Z23AddIndirectPrimaryBasesPKN5clang13CXXRecordDeclERNS_10ASTContextERNS_25CXXIndirectPrimaryBaseSetE", "AddIndirectPrimaryBases(clang::CXXRecordDecl const*, clang::ASTContext&, clang::CXXIndirectPrimaryBaseSet&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang19UniqueVirtualMethodELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::UniqueVirtualMethod, false>::grow(unsigned long)"},
    {"_ZSt6__findIPN5clang19UniqueVirtualMethodES1_ET_S3_S3_RKT0_St26random_access_iterator_tag", "clang::UniqueVirtualMethod* std::__find<clang::UniqueVirtualMethod*, clang::UniqueVirtualMethod>(clang::UniqueVirtualMethod*, clang::UniqueVirtualMethod*, clang::UniqueVirtualMethod const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIN5clang19UniqueVirtualMethodELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<clang::UniqueVirtualMethod, 4u> > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIN5clang19UniqueVirtualMethodELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PSt4pairIjS4_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<clang::UniqueVirtualMethod, 4u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallVector<clang::UniqueVirtualMethod, 4u> const&, std::pair<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIN5clang19UniqueVirtualMethodELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<clang::UniqueVirtualMethod, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIN5clang19UniqueVirtualMethodEEaSERKS3_", "llvm::SmallVectorImpl<clang::UniqueVirtualMethod>::operator=(llvm::SmallVectorImpl<clang::UniqueVirtualMethod> const&)"},
    {"_ZN4llvm15SmallVectorImplIN5clang18CXXBasePathElementEEaSERKS3_", "llvm::SmallVectorImpl<clang::CXXBasePathElement>::operator=(llvm::SmallVectorImpl<clang::CXXBasePathElement> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang18CXXBasePathElementELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::CXXBasePathElement, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_S2_IbjEESt10_Select1stIS5_ENS0_16QualTypeOrderingESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, std::pair<bool, unsigned int> >, std::_Select1st<std::pair<clang::QualType const, std::pair<bool, unsigned int> > >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, std::pair<bool, unsigned int> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::QualType const, std::pair<bool, unsigned int> > >, std::pair<clang::QualType const, std::pair<bool, unsigned int> > const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_S2_IbjEESt10_Select1stIS5_ENS0_16QualTypeOrderingESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, std::pair<bool, unsigned int> >, std::_Select1st<std::pair<clang::QualType const, std::pair<bool, unsigned int> > >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, std::pair<bool, unsigned int> > > >::_M_insert_unique(std::pair<clang::QualType const, std::pair<bool, unsigned int> > const&)"},
    {"_ZNSt8_Rb_treeIPN5clang9NamedDeclES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<clang::NamedDecl*, clang::NamedDecl*, std::_Identity<clang::NamedDecl*>, std::less<clang::NamedDecl*>, std::allocator<clang::NamedDecl*> >::_M_insert_unique(clang::NamedDecl* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_17OverridingMethodsENS_12DenseMapInfoIS4_EENS6_IS5_EEE16FindAndConstructERKS4_", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::OverridingMethods, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::OverridingMethods> >::FindAndConstruct(clang::CXXMethodDecl const* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_17OverridingMethodsENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::OverridingMethods, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::OverridingMethods> >::InsertIntoBucket(clang::CXXMethodDecl const* const&, clang::OverridingMethods const&, std::pair<clang::CXXMethodDecl const*, clang::OverridingMethods>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_17OverridingMethodsENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::OverridingMethods, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::OverridingMethods> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIN5clang19UniqueVirtualMethodELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE8CopyFromERKS8_", "llvm::DenseMap<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<clang::UniqueVirtualMethod, 4u> > >::CopyFrom(llvm::DenseMap<unsigned int, llvm::SmallVector<clang::UniqueVirtualMethod, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<clang::UniqueVirtualMethod, 4u> > > const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPNS1_20CXXFinalOverriderMapENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CXXFinalOverriderMap*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CXXFinalOverriderMap*> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, clang::CXXFinalOverriderMap* const&, std::pair<clang::CXXRecordDecl const*, clang::CXXFinalOverriderMap*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPNS1_20CXXFinalOverriderMapENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CXXFinalOverriderMap*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CXXFinalOverriderMap*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::CXXRecordDecl const*, unsigned int, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, unsigned int const&, std::pair<clang::CXXRecordDecl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, unsigned int, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIPN5clang9NamedDeclES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<clang::NamedDecl*, clang::NamedDecl*, std::_Identity<clang::NamedDecl*>, std::less<clang::NamedDecl*>, std::allocator<clang::NamedDecl*> >::_M_erase(std::_Rb_tree_node<clang::NamedDecl*>*)"},
    {"_ZN5clang9NamedDecl17ClearLinkageCacheEv", "clang::NamedDecl::ClearLinkageCache()"},
    {"_ZNK5clang9NamedDecl10getLinkageEv", "clang::NamedDecl::getLinkage() const"},
    {"_Z12getLVForDeclPKN5clang9NamedDeclEN12_GLOBAL__N_17LVFlagsE", "getLVForDecl(clang::NamedDecl const*, (anonymous namespace)::LVFlags)"},
    {"_ZNK5clang9NamedDecl23getLinkageAndVisibilityEv", "clang::NamedDecl::getLinkageAndVisibility() const"},
    {"_ZNK5clang9NamedDecl24getQualifiedNameAsStringEv", "clang::NamedDecl::getQualifiedNameAsString() const"},
    {"_ZNK5clang9NamedDecl24getQualifiedNameAsStringERKNS_14PrintingPolicyE", "clang::NamedDecl::getQualifiedNameAsString(clang::PrintingPolicy const&) const"},
    {"_ZNK5clang12FunctionDecl12getNumParamsEv", "clang::FunctionDecl::getNumParams() const"},
    {"_ZNK5clang9NamedDecl19declarationReplacesEPS0_", "clang::NamedDecl::declarationReplaces(clang::NamedDecl*) const"},
    {"_ZNK5clang9NamedDecl10hasLinkageEv", "clang::NamedDecl::hasLinkage() const"},
    {"_ZN5clang9NamedDecl17getUnderlyingDeclEv", "clang::NamedDecl::getUnderlyingDecl()"},
    {"_ZNK5clang9NamedDecl19isCXXInstanceMemberEv", "clang::NamedDecl::isCXXInstanceMember() const"},
    {"_ZNK5clang14DeclaratorDecl19getTypeSpecStartLocEv", "clang::DeclaratorDecl::getTypeSpecStartLoc() const"},
    {"_ZN5clang14DeclaratorDecl16setQualifierInfoEPNS_19NestedNameSpecifierENS_11SourceRangeE", "clang::DeclaratorDecl::setQualifierInfo(clang::NestedNameSpecifier*, clang::SourceRange)"},
    {"_ZNK5clang14DeclaratorDecl16getOuterLocStartEv", "clang::DeclaratorDecl::getOuterLocStart() const"},
    {"_ZN5clang13QualifierInfo29setTemplateParameterListsInfoERNS_10ASTContextEjPPNS_21TemplateParameterListE", "clang::QualifierInfo::setTemplateParameterListsInfo(clang::ASTContext&, unsigned int, clang::TemplateParameterList**)"},
    {"_ZN5clang7VarDecl30getStorageClassSpecifierStringENS_12StorageClassE", "clang::VarDecl::getStorageClassSpecifierString(clang::StorageClass)"},
    {"_ZN5clang7VarDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoENS_12StorageClassESB_", "clang::VarDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::StorageClass, clang::StorageClass)"},
    {"_ZN5clang7VarDecl15setStorageClassENS_12StorageClassE", "clang::VarDecl::setStorageClass(clang::StorageClass)"},
    {"_ZNK5clang7VarDecl16getInnerLocStartEv", "clang::VarDecl::getInnerLocStart() const"},
    {"_ZNK5clang7VarDecl14getSourceRangeEv", "clang::VarDecl::getSourceRange() const"},
    {"_ZNK5clang7VarDecl9isExternCEv", "clang::VarDecl::isExternC() const"},
    {"_ZN5clang7VarDecl16getCanonicalDeclEv", "clang::VarDecl::getCanonicalDecl()"},
    {"_ZNK5clang7VarDecl28isThisDeclarationADefinitionEv", "clang::VarDecl::isThisDeclarationADefinition() const"},
    {"_ZNK5clang7VarDecl29getTemplateSpecializationKindEv", "clang::VarDecl::getTemplateSpecializationKind() const"},
    {"_ZN5clang7VarDecl19getActingDefinitionEv", "clang::VarDecl::getActingDefinition()"},
    {"_ZNK5clang7VarDecl24isTentativeDefinitionNowEv", "clang::VarDecl::isTentativeDefinitionNow() const"},
    {"_ZN5clang7VarDecl13getDefinitionEv", "clang::VarDecl::getDefinition()"},
    {"_ZNK5clang7VarDecl13hasDefinitionEv", "clang::VarDecl::hasDefinition() const"},
    {"_ZNK5clang7VarDecl17getAnyInitializerERPKS0_", "clang::VarDecl::getAnyInitializer(clang::VarDecl const*&) const"},
    {"_ZNK5clang7VarDecl11isOutOfLineEv", "clang::VarDecl::isOutOfLine() const"},
    {"_ZNK5clang7VarDecl35getInstantiatedFromStaticDataMemberEv", "clang::VarDecl::getInstantiatedFromStaticDataMember() const"},
    {"_ZN5clang7VarDecl22getOutOfLineDefinitionEv", "clang::VarDecl::getOutOfLineDefinition()"},
    {"_ZN5clang7VarDecl7setInitEPNS_4ExprE", "clang::VarDecl::setInit(clang::Expr*)"},
    {"_ZNK5clang7VarDecl27getMemberSpecializationInfoEv", "clang::VarDecl::getMemberSpecializationInfo() const"},
    {"_ZN5clang7VarDecl29setTemplateSpecializationKindENS_26TemplateSpecializationKindENS_14SourceLocationE", "clang::VarDecl::setTemplateSpecializationKind(clang::TemplateSpecializationKind, clang::SourceLocation)"},
    {"_ZN5clang11ParmVarDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoENS_12StorageClassESB_PNS_4ExprE", "clang::ParmVarDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::StorageClass, clang::StorageClass, clang::Expr*)"},
    {"_ZN5clang11ParmVarDecl13getDefaultArgEv", "clang::ParmVarDecl::getDefaultArg()"},
    {"_ZNK5clang11ParmVarDecl27getNumDefaultArgTemporariesEv", "clang::ParmVarDecl::getNumDefaultArgTemporaries() const"},
    {"_ZN5clang11ParmVarDecl22getDefaultArgTemporaryEj", "clang::ParmVarDecl::getDefaultArgTemporary(unsigned int)"},
    {"_ZNK5clang11ParmVarDecl18getDefaultArgRangeEv", "clang::ParmVarDecl::getDefaultArgRange() const"},
    {"_ZNK5clang11ParmVarDecl15isParameterPackEv", "clang::ParmVarDecl::isParameterPack() const"},
    {"_ZNK5clang12FunctionDecl20getNameForDiagnosticERSsRKNS_14PrintingPolicyEb", "clang::FunctionDecl::getNameForDiagnostic(std::string&, clang::PrintingPolicy const&, bool) const"},
    {"_ZNK5clang12FunctionDecl29getTemplateSpecializationArgsEv", "clang::FunctionDecl::getTemplateSpecializationArgs() const"},
    {"_ZNK5clang12FunctionDecl10isVariadicEv", "clang::FunctionDecl::isVariadic() const"},
    {"_ZNK5clang12FunctionDecl7hasBodyERPKS0_", "clang::FunctionDecl::hasBody(clang::FunctionDecl const*&) const"},
    {"_ZNK5clang12FunctionDecl7getBodyERPKS0_", "clang::FunctionDecl::getBody(clang::FunctionDecl const*&) const"},
    {"_ZN5clang12FunctionDecl7setBodyEPNS_4StmtE", "clang::FunctionDecl::setBody(clang::Stmt*)"},
    {"_ZN5clang12FunctionDecl7setPureEb", "clang::FunctionDecl::setPure(bool)"},
    {"_ZNK5clang12FunctionDecl6isMainEv", "clang::FunctionDecl::isMain() const"},
    {"_ZNK5clang12FunctionDecl9isExternCEv", "clang::FunctionDecl::isExternC() const"},
    {"_ZNK5clang12FunctionDecl8isGlobalEv", "clang::FunctionDecl::isGlobal() const"},
    {"_ZN5clang12FunctionDecl22setPreviousDeclarationEPS0_", "clang::FunctionDecl::setPreviousDeclaration(clang::FunctionDecl*)"},
    {"_ZNK5clang12FunctionDecl16getCanonicalDeclEv", "clang::FunctionDecl::getCanonicalDecl() const"},
    {"_ZN5clang12FunctionDecl16getCanonicalDeclEv", "clang::FunctionDecl::getCanonicalDecl()"},
    {"_ZN5clang12FunctionDecl15setStorageClassENS_12StorageClassE", "clang::FunctionDecl::setStorageClass(clang::StorageClass)"},
    {"_ZNK5clang12FunctionDecl12getBuiltinIDEv", "clang::FunctionDecl::getBuiltinID() const"},
    {"_ZN5clang12FunctionDecl9setParamsERNS_10ASTContextEPPNS_11ParmVarDeclEj", "clang::FunctionDecl::setParams(clang::ASTContext&, clang::ParmVarDecl**, unsigned int)"},
    {"_ZNK5clang12FunctionDecl23getMinRequiredArgumentsEv", "clang::FunctionDecl::getMinRequiredArguments() const"},
    {"_ZNK5clang12FunctionDecl9isInlinedEv", "clang::FunctionDecl::isInlined() const"},
    {"_ZNK5clang12FunctionDecl29getTemplateSpecializationKindEv", "clang::FunctionDecl::getTemplateSpecializationKind() const"},
    {"_ZNK5clang12FunctionDecl31getTemplateInstantiationPatternEv", "clang::FunctionDecl::getTemplateInstantiationPattern() const"},
    {"_ZNK5clang12FunctionDecl35isInlineDefinitionExternallyVisibleEv", "clang::FunctionDecl::isInlineDefinitionExternallyVisible() const"},
    {"_ZNK5clang12FunctionDecl21getOverloadedOperatorEv", "clang::FunctionDecl::getOverloadedOperator() const"},
    {"_ZNK5clang12FunctionDecl20getLiteralIdentifierEv", "clang::FunctionDecl::getLiteralIdentifier() const"},
    {"_ZNK5clang12FunctionDecl16getTemplatedKindEv", "clang::FunctionDecl::getTemplatedKind() const"},
    {"_ZNK5clang12FunctionDecl33getInstantiatedFromMemberFunctionEv", "clang::FunctionDecl::getInstantiatedFromMemberFunction() const"},
    {"_ZNK5clang12FunctionDecl27getMemberSpecializationInfoEv", "clang::FunctionDecl::getMemberSpecializationInfo() const"},
    {"_ZN5clang12FunctionDecl32setInstantiationOfMemberFunctionERNS_10ASTContextEPS0_NS_26TemplateSpecializationKindE", "clang::FunctionDecl::setInstantiationOfMemberFunction(clang::ASTContext&, clang::FunctionDecl*, clang::TemplateSpecializationKind)"},
    {"_ZNK5clang12FunctionDecl24isImplicitlyInstantiableEv", "clang::FunctionDecl::isImplicitlyInstantiable() const"},
    {"_ZNK5clang12FunctionDecl18getPrimaryTemplateEv", "clang::FunctionDecl::getPrimaryTemplate() const"},
    {"_ZNK5clang12FunctionDecl38getTemplateSpecializationArgsAsWrittenEv", "clang::FunctionDecl::getTemplateSpecializationArgsAsWritten() const"},
    {"_ZN5clang12FunctionDecl33setFunctionTemplateSpecializationERNS_10ASTContextEPNS_20FunctionTemplateDeclEPKNS_20TemplateArgumentListEPvNS_26TemplateSpecializationKindEPKNS_24TemplateArgumentListInfoENS_14SourceLocationE", "clang::FunctionDecl::setFunctionTemplateSpecialization(clang::ASTContext&, clang::FunctionTemplateDecl*, clang::TemplateArgumentList const*, void*, clang::TemplateSpecializationKind, clang::TemplateArgumentListInfo const*, clang::SourceLocation)"},
    {"_ZN5clang12FunctionDecl34setDependentTemplateSpecializationERNS_10ASTContextERKNS_17UnresolvedSetImplERKNS_24TemplateArgumentListInfoE", "clang::FunctionDecl::setDependentTemplateSpecialization(clang::ASTContext&, clang::UnresolvedSetImpl const&, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang43DependentFunctionTemplateSpecializationInfoC1ERKNS_17UnresolvedSetImplERKNS_24TemplateArgumentListInfoE", "clang::DependentFunctionTemplateSpecializationInfo::DependentFunctionTemplateSpecializationInfo(clang::UnresolvedSetImpl const&, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang43DependentFunctionTemplateSpecializationInfoC2ERKNS_17UnresolvedSetImplERKNS_24TemplateArgumentListInfoE", "clang::DependentFunctionTemplateSpecializationInfo::DependentFunctionTemplateSpecializationInfo(clang::UnresolvedSetImpl const&, clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang12FunctionDecl29setTemplateSpecializationKindENS_26TemplateSpecializationKindENS_14SourceLocationE", "clang::FunctionDecl::setTemplateSpecializationKind(clang::TemplateSpecializationKind, clang::SourceLocation)"},
    {"_ZNK5clang12FunctionDecl23getPointOfInstantiationEv", "clang::FunctionDecl::getPointOfInstantiation() const"},
    {"_ZNK5clang12FunctionDecl11isOutOfLineEv", "clang::FunctionDecl::isOutOfLine() const"},
    {"_ZN5clang9FieldDecl6CreateERKNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoEPNS_4ExprEb", "clang::FieldDecl::Create(clang::ASTContext const&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::Expr*, bool)"},
    {"_ZNK5clang9FieldDecl24isAnonymousStructOrUnionEv", "clang::FieldDecl::isAnonymousStructOrUnion() const"},
    {"_ZNK5clang9FieldDecl13getFieldIndexEv", "clang::FieldDecl::getFieldIndex() const"},
    {"_ZNK5clang10RecordDecl11field_beginEv", "clang::RecordDecl::field_begin() const"},
    {"_ZNK5clang7TagDecl16getOuterLocStartEv", "clang::TagDecl::getOuterLocStart() const"},
    {"_ZNK5clang7TagDecl14getSourceRangeEv", "clang::TagDecl::getSourceRange() const"},
    {"_ZN5clang7TagDecl16getCanonicalDeclEv", "clang::TagDecl::getCanonicalDecl()"},
    {"_ZN5clang7TagDecl21setTypedefForAnonDeclEPNS_11TypedefDeclE", "clang::TagDecl::setTypedefForAnonDecl(clang::TypedefDecl*)"},
    {"_ZN5clang7TagDecl15startDefinitionEv", "clang::TagDecl::startDefinition()"},
    {"_ZN5clang7TagDecl18completeDefinitionEv", "clang::TagDecl::completeDefinition()"},
    {"_ZNK5clang7TagDecl13getDefinitionEv", "clang::TagDecl::getDefinition() const"},
    {"_ZN5clang7TagDecl16setQualifierInfoEPNS_19NestedNameSpecifierENS_11SourceRangeE", "clang::TagDecl::setQualifierInfo(clang::NestedNameSpecifier*, clang::SourceRange)"},
    {"_ZN5clang8EnumDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES5_PS0_bbb", "clang::EnumDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::EnumDecl*, bool, bool, bool)"},
    {"_ZN5clang8EnumDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::EnumDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang8EnumDecl18completeDefinitionENS_8QualTypeES1_jj", "clang::EnumDecl::completeDefinition(clang::QualType, clang::QualType, unsigned int, unsigned int)"},
    {"_ZN5clang10RecordDeclC1ENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPS0_S6_", "clang::RecordDecl::RecordDecl(clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::RecordDecl*, clang::SourceLocation)"},
    {"_ZN5clang10RecordDeclC2ENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPS0_S6_", "clang::RecordDecl::RecordDecl(clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::RecordDecl*, clang::SourceLocation)"},
    {"_ZN5clang10RecordDecl6CreateERKNS_10ASTContextENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES7_PS0_", "clang::RecordDecl::Create(clang::ASTContext const&, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::RecordDecl*)"},
    {"_ZN5clang10RecordDecl6CreateERKNS_10ASTContextENS_4Decl10EmptyShellE", "clang::RecordDecl::Create(clang::ASTContext const&, clang::Decl::EmptyShell)"},
    {"_ZNK5clang10RecordDecl19isInjectedClassNameEv", "clang::RecordDecl::isInjectedClassName() const"},
    {"_ZNK5clang10RecordDecl29LoadFieldsFromExternalStorageEv", "clang::RecordDecl::LoadFieldsFromExternalStorage() const"},
    {"_ZN5clang10RecordDecl18completeDefinitionEv", "clang::RecordDecl::completeDefinition()"},
    {"_ZN5clang9BlockDecl9setParamsEPPNS_11ParmVarDeclEj", "clang::BlockDecl::setParams(clang::ParmVarDecl**, unsigned int)"},
    {"_ZN5clang9BlockDecl11setCapturesERNS_10ASTContextEPKNS0_7CaptureES5_b", "clang::BlockDecl::setCaptures(clang::ASTContext&, clang::BlockDecl::Capture const*, clang::BlockDecl::Capture const*, bool)"},
    {"_ZNK5clang9BlockDecl14getSourceRangeEv", "clang::BlockDecl::getSourceRange() const"},
    {"_ZN5clang19TranslationUnitDecl6CreateERNS_10ASTContextE", "clang::TranslationUnitDecl::Create(clang::ASTContext&)"},
    {"_ZN5clang13NamespaceDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoE", "clang::NamespaceDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang13NamespaceDecl16getNextNamespaceEv", "clang::NamespaceDecl::getNextNamespace()"},
    {"_ZN5clang17ImplicitParamDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeE", "clang::ImplicitParamDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType)"},
    {"_ZN5clang12FunctionDecl6CreateERNS_10ASTContextEPNS_11DeclContextERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoENS_12StorageClassESB_bb", "clang::FunctionDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, clang::StorageClass, clang::StorageClass, bool, bool)"},
    {"_ZN5clang9BlockDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationE", "clang::BlockDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation)"},
    {"_ZN5clang16EnumConstantDecl6CreateERNS_10ASTContextEPNS_8EnumDeclENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_4ExprERKN4llvm6APSIntE", "clang::EnumConstantDecl::Create(clang::ASTContext&, clang::EnumDecl*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::Expr*, llvm::APSInt const&)"},
    {"_ZN5clang17IndirectFieldDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPPNS_9NamedDeclEj", "clang::IndirectFieldDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::NamedDecl**, unsigned int)"},
    {"_ZNK5clang16EnumConstantDecl14getSourceRangeEv", "clang::EnumConstantDecl::getSourceRange() const"},
    {"_ZN5clang11TypedefDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPNS_14TypeSourceInfoE", "clang::TypedefDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::TypeSourceInfo*)"},
    {"_ZN5clang16FileScopeAsmDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_13StringLiteralE", "clang::FileScopeAsmDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::StringLiteral*)"},
    {"_Z21GetExplicitVisibilityPKN5clang4DeclE", "GetExplicitVisibility(clang::Decl const*)"},
    {"_Z29getLVForTemplateParameterListPKN5clang21TemplateParameterListE", "getLVForTemplateParameterList(clang::TemplateParameterList const*)"},
    {"_Z28getLVForTemplateArgumentListPKN5clang16TemplateArgumentEjRN12_GLOBAL__N_17LVFlagsE", "getLVForTemplateArgumentList(clang::TemplateArgument const*, unsigned int, (anonymous namespace)::LVFlags&)"},
    {"_ZNK5clang4Decl11isOutOfLineEv", "clang::Decl::isOutOfLine() const"},
    {"_ZNK5clang9NamedDecl20getNameForDiagnosticERSsRKNS_14PrintingPolicyEb", "clang::NamedDecl::getNameForDiagnostic(std::string&, clang::PrintingPolicy const&, bool) const"},
    {"_ZNK5clang4Decl7getAttrINS_16OverloadableAttrEEEPT_v", "clang::OverloadableAttr* clang::Decl::getAttr<clang::OverloadableAttr>() const"},
    {"_ZN5clang12RedeclarableINS_12FunctionDeclEE22setPreviousDeclarationEPS1_", "clang::Redeclarable<clang::FunctionDecl>::setPreviousDeclaration(clang::FunctionDecl*)"},
    {"_ZN5clang7TagDeclC2ENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPS0_S6_", "clang::TagDecl::TagDecl(clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::TagDecl*, clang::SourceLocation)"},
    {"_ZN5clang7VarDeclD1Ev", "clang::VarDecl::~VarDecl()"},
    {"_ZN5clang7VarDeclD0Ev", "clang::VarDecl::~VarDecl()"},
    {"_ZN5clang7VarDecl20getNextRedeclarationEv", "clang::VarDecl::getNextRedeclaration()"},
    {"_ZNK5clang4Decl7getBodyEv", "clang::Decl::getBody() const"},
    {"_ZNK5clang4Decl7hasBodyEv", "clang::Decl::hasBody() const"},
    {"_ZN5clang12FunctionDeclD1Ev", "clang::FunctionDecl::~FunctionDecl()"},
    {"_ZN5clang12FunctionDeclD0Ev", "clang::FunctionDecl::~FunctionDecl()"},
    {"_ZNK5clang12FunctionDecl14getSourceRangeEv", "clang::FunctionDecl::getSourceRange() const"},
    {"_ZN5clang12FunctionDecl20getNextRedeclarationEv", "clang::FunctionDecl::getNextRedeclaration()"},
    {"_ZNK5clang12FunctionDecl7getBodyEv", "clang::FunctionDecl::getBody() const"},
    {"_ZNK5clang12FunctionDecl7hasBodyEv", "clang::FunctionDecl::hasBody() const"},
    {"_ZNK5clang14DeclaratorDecl16getInnerLocStartEv", "clang::DeclaratorDecl::getInnerLocStart() const"},
    {"_ZN5clang16EnumConstantDeclD1Ev", "clang::EnumConstantDecl::~EnumConstantDecl()"},
    {"_ZN5clang16EnumConstantDeclD0Ev", "clang::EnumConstantDecl::~EnumConstantDecl()"},
    {"_ZN5clang4Decl16getCanonicalDeclEv", "clang::Decl::getCanonicalDecl()"},
    {"_ZN5clang4Decl20getNextRedeclarationEv", "clang::Decl::getNextRedeclaration()"},
    {"_ZN5clang7TagDeclD1Ev", "clang::TagDecl::~TagDecl()"},
    {"_ZN5clang7TagDeclD0Ev", "clang::TagDecl::~TagDecl()"},
    {"_ZN5clang7TagDecl20getNextRedeclarationEv", "clang::TagDecl::getNextRedeclaration()"},
    {"_ZNK5clang7TagDecl16getInnerLocStartEv", "clang::TagDecl::getInnerLocStart() const"},
    {"_ZN5clang9BlockDeclD1Ev", "clang::BlockDecl::~BlockDecl()"},
    {"_ZN5clang9BlockDeclD0Ev", "clang::BlockDecl::~BlockDecl()"},
    {"_ZNK5clang9BlockDecl7getBodyEv", "clang::BlockDecl::getBody() const"},
    {"_ZN5clang10RecordDeclD1Ev", "clang::RecordDecl::~RecordDecl()"},
    {"_ZN5clang10RecordDeclD0Ev", "clang::RecordDecl::~RecordDecl()"},
    {"_ZN5clang9FieldDecl11classofKindENS_4Decl4KindE", "clang::FieldDecl::classofKind(clang::Decl::Kind)"},
    {"_ZN5clang8TypeDeclD1Ev", "clang::TypeDecl::~TypeDecl()"},
    {"_ZN5clang8TypeDeclD0Ev", "clang::TypeDecl::~TypeDecl()"},
    {"_ZNK5clang4Decl14getSourceRangeEv", "clang::Decl::getSourceRange() const"},
    {"_ZN5clang9NamedDeclD1Ev", "clang::NamedDecl::~NamedDecl()"},
    {"_ZN5clang9NamedDeclD0Ev", "clang::NamedDecl::~NamedDecl()"},
    {"_ZN5clang9ValueDeclD1Ev", "clang::ValueDecl::~ValueDecl()"},
    {"_ZN5clang9ValueDeclD0Ev", "clang::ValueDecl::~ValueDecl()"},
    {"_ZN5clang14DeclaratorDeclD1Ev", "clang::DeclaratorDecl::~DeclaratorDecl()"},
    {"_ZN5clang14DeclaratorDeclD0Ev", "clang::DeclaratorDecl::~DeclaratorDecl()"},
    {"_ZNK5clang14DeclaratorDecl14getSourceRangeEv", "clang::DeclaratorDecl::getSourceRange() const"},
    {"_ZN5clang16FileScopeAsmDeclD1Ev", "clang::FileScopeAsmDecl::~FileScopeAsmDecl()"},
    {"_ZN5clang16FileScopeAsmDeclD0Ev", "clang::FileScopeAsmDecl::~FileScopeAsmDecl()"},
    {"_ZN5clang11TypedefDeclD1Ev", "clang::TypedefDecl::~TypedefDecl()"},
    {"_ZN5clang11TypedefDeclD0Ev", "clang::TypedefDecl::~TypedefDecl()"},
    {"_ZN5clang11TypedefDecl16getCanonicalDeclEv", "clang::TypedefDecl::getCanonicalDecl()"},
    {"_ZN5clang11TypedefDecl20getNextRedeclarationEv", "clang::TypedefDecl::getNextRedeclaration()"},
    {"_ZN5clang17IndirectFieldDeclD1Ev", "clang::IndirectFieldDecl::~IndirectFieldDecl()"},
    {"_ZN5clang17IndirectFieldDeclD0Ev", "clang::IndirectFieldDecl::~IndirectFieldDecl()"},
    {"_ZN5clang16EnumConstantDeclC2EPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_4ExprERKN4llvm6APSIntE", "clang::EnumConstantDecl::EnumConstantDecl(clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::Expr*, llvm::APSInt const&)"},
    {"_ZN5clang9BlockDeclC2EPNS_11DeclContextENS_14SourceLocationE", "clang::BlockDecl::BlockDecl(clang::DeclContext*, clang::SourceLocation)"},
    {"_ZN5clang12FunctionDeclC2ENS_4Decl4KindEPNS_11DeclContextERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoENS_12StorageClassESB_b", "clang::FunctionDecl::FunctionDecl(clang::Decl::Kind, clang::DeclContext*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, clang::StorageClass, clang::StorageClass, bool)"},
    {"_ZN5clang7VarDeclC2ENS_4Decl4KindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoENS_12StorageClassESB_", "clang::VarDecl::VarDecl(clang::Decl::Kind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::StorageClass, clang::StorageClass)"},
    {"_ZN5clang17ImplicitParamDeclD1Ev", "clang::ImplicitParamDecl::~ImplicitParamDecl()"},
    {"_ZN5clang17ImplicitParamDeclD0Ev", "clang::ImplicitParamDecl::~ImplicitParamDecl()"},
    {"_ZN5clang13NamespaceDeclC2EPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoE", "clang::NamespaceDecl::NamespaceDecl(clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang13NamespaceDeclD1Ev", "clang::NamespaceDecl::~NamespaceDecl()"},
    {"_ZN5clang13NamespaceDeclD0Ev", "clang::NamespaceDecl::~NamespaceDecl()"},
    {"_ZNK5clang13NamespaceDecl14getSourceRangeEv", "clang::NamespaceDecl::getSourceRange() const"},
    {"_ZN5clang13NamespaceDecl16getCanonicalDeclEv", "clang::NamespaceDecl::getCanonicalDecl()"},
    {"_ZN5clang19TranslationUnitDeclC2ERNS_10ASTContextE", "clang::TranslationUnitDecl::TranslationUnitDecl(clang::ASTContext&)"},
    {"_ZN5clang19TranslationUnitDeclD1Ev", "clang::TranslationUnitDecl::~TranslationUnitDecl()"},
    {"_ZN5clang19TranslationUnitDeclD0Ev", "clang::TranslationUnitDecl::~TranslationUnitDecl()"},
    {"_ZN5clang12RedeclarableINS_7TagDeclEE22setPreviousDeclarationEPS1_", "clang::Redeclarable<clang::TagDecl>::setPreviousDeclaration(clang::TagDecl*)"},
    {"_ZN5clang8EnumDeclD1Ev", "clang::EnumDecl::~EnumDecl()"},
    {"_ZN5clang8EnumDeclD0Ev", "clang::EnumDecl::~EnumDecl()"},
    {"_ZN5clang8EnumDecl16getCanonicalDeclEv", "clang::EnumDecl::getCanonicalDecl()"},
    {"_ZN5clang9FieldDeclC2ENS_4Decl4KindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoEPNS_4ExprEb", "clang::FieldDecl::FieldDecl(clang::Decl::Kind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::Expr*, bool)"},
    {"_ZN5clang9FieldDeclD1Ev", "clang::FieldDecl::~FieldDecl()"},
    {"_ZN5clang9FieldDeclD0Ev", "clang::FieldDecl::~FieldDecl()"},
    {"_ZN5clang11ParmVarDeclD1Ev", "clang::ParmVarDecl::~ParmVarDecl()"},
    {"_ZN5clang11ParmVarDeclD0Ev", "clang::ParmVarDecl::~ParmVarDecl()"},
    {"_ZNK5clang4Decl7getAttrINS_14VisibilityAttrEEEPT_v", "clang::VisibilityAttr* clang::Decl::getAttr<clang::VisibilityAttr>() const"},
    {"_ZNK5clang4Decl15getDeclKindNameEv", "clang::Decl::getDeclKindName() const"},
    {"_ZN5clang4Decl14setInvalidDeclEb", "clang::Decl::setInvalidDecl(bool)"},
    {"_ZNK5clang11DeclContext15getDeclKindNameEv", "clang::DeclContext::getDeclKindName() const"},
    {"_ZN5clang4Decl15CollectingStatsEb", "clang::Decl::CollectingStats(bool)"},
    {"_ZN5clang4Decl10PrintStatsEv", "clang::Decl::PrintStats()"},
    {"_ZN5clang4Decl3addENS0_4KindE", "clang::Decl::add(clang::Decl::Kind)"},
    {"_ZNK5clang4Decl23isTemplateParameterPackEv", "clang::Decl::isTemplateParameterPack() const"},
    {"_ZNK5clang4Decl15isParameterPackEv", "clang::Decl::isParameterPack() const"},
    {"_ZNK5clang4Decl28isFunctionOrFunctionTemplateEv", "clang::Decl::isFunctionOrFunctionTemplate() const"},
    {"_ZNK5clang4Decl32isDefinedOutsideFunctionOrMethodEv", "clang::Decl::isDefinedOutsideFunctionOrMethod() const"},
    {"_ZNK5clang20PrettyStackTraceDecl5printERN4llvm11raw_ostreamE", "clang::PrettyStackTraceDecl::print(llvm::raw_ostream&) const"},
    {"_ZN5clang4DeclD0Ev", "clang::Decl::~Decl()"},
    {"_ZN5clang4DeclD1Ev", "clang::Decl::~Decl()"},
    {"_ZN5clang4DeclD2Ev", "clang::Decl::~Decl()"},
    {"_ZN5clang4Decl14setDeclContextEPNS_11DeclContextE", "clang::Decl::setDeclContext(clang::DeclContext*)"},
    {"_ZN5clang4Decl21setLexicalDeclContextEPNS_11DeclContextE", "clang::Decl::setLexicalDeclContext(clang::DeclContext*)"},
    {"_ZNK5clang4Decl13getASTContextEv", "clang::Decl::getASTContext() const"},
    {"_ZNK5clang4Decl22isInAnonymousNamespaceEv", "clang::Decl::isInAnonymousNamespace() const"},
    {"_ZN5clang4Decl22getTranslationUnitDeclEv", "clang::Decl::getTranslationUnitDecl()"},
    {"_ZNK5clang4Decl22getASTMutationListenerEv", "clang::Decl::getASTMutationListener() const"},
    {"_ZNK5clang4Decl6isUsedEb", "clang::Decl::isUsed(bool) const"},
    {"_ZN5clang4Decl29getIdentifierNamespaceForKindENS0_4KindE", "clang::Decl::getIdentifierNamespaceForKind(clang::Decl::Kind)"},
    {"_ZN5clang4Decl8setAttrsERKN4llvm11SmallVectorIPNS_4AttrELj2EEE", "clang::Decl::setAttrs(llvm::SmallVector<clang::Attr*, 2u> const&)"},
    {"_ZN5clang4Decl9dropAttrsEv", "clang::Decl::dropAttrs()"},
    {"_ZNK5clang4Decl8getAttrsEv", "clang::Decl::getAttrs() const"},
    {"_ZN5clang4Decl9swapAttrsEPS0_", "clang::Decl::swapAttrs(clang::Decl*)"},
    {"_ZN5clang4Decl19castFromDeclContextEPKNS_11DeclContextE", "clang::Decl::castFromDeclContext(clang::DeclContext const*)"},
    {"_ZN5clang4Decl17castToDeclContextEPKS0_", "clang::Decl::castToDeclContext(clang::Decl const*)"},
    {"_ZNK5clang4Decl13getBodyRBraceEv", "clang::Decl::getBodyRBrace() const"},
    {"_ZNK5clang4Decl22CheckAccessDeclContextEv", "clang::Decl::CheckAccessDeclContext() const"},
    {"_ZN5clang11DeclContext7classofEPKNS_4DeclE", "clang::DeclContext::classof(clang::Decl const*)"},
    {"_ZN5clang11DeclContextD1Ev", "clang::DeclContext::~DeclContext()"},
    {"_ZN5clang11DeclContextD2Ev", "clang::DeclContext::~DeclContext()"},
    {"_ZN5clang11DeclContext15getLookupParentEv", "clang::DeclContext::getLookupParent()"},
    {"_ZN5clang11DeclContext16getRedeclContextEv", "clang::DeclContext::getRedeclContext()"},
    {"_ZNK5clang11DeclContext17isInlineNamespaceEv", "clang::DeclContext::isInlineNamespace() const"},
    {"_ZNK5clang11DeclContext18isDependentContextEv", "clang::DeclContext::isDependentContext() const"},
    {"_ZNK5clang11DeclContext20isTransparentContextEv", "clang::DeclContext::isTransparentContext() const"},
    {"_ZNK5clang11DeclContext16isExternCContextEv", "clang::DeclContext::isExternCContext() const"},
    {"_ZNK5clang11DeclContext8EnclosesEPKS0_", "clang::DeclContext::Encloses(clang::DeclContext const*) const"},
    {"_ZN5clang11DeclContext17getPrimaryContextEv", "clang::DeclContext::getPrimaryContext()"},
    {"_ZN5clang11DeclContext14getNextContextEv", "clang::DeclContext::getNextContext()"},
    {"_ZN5clang11DeclContext14BuildDeclChainERKN4llvm15SmallVectorImplIPNS_4DeclEEE", "clang::DeclContext::BuildDeclChain(llvm::SmallVectorImpl<clang::Decl*> const&)"},
    {"_ZNK5clang11DeclContext35LoadLexicalDeclsFromExternalStorageEv", "clang::DeclContext::LoadLexicalDeclsFromExternalStorage() const"},
    {"_ZN5clang17ExternalASTSource32SetNoExternalVisibleDeclsForNameEPKNS_11DeclContextENS_15DeclarationNameE", "clang::ExternalASTSource::SetNoExternalVisibleDeclsForName(clang::DeclContext const*, clang::DeclarationName)"},
    {"_ZNK5clang11DeclContext20CreateStoredDeclsMapERNS_10ASTContextE", "clang::DeclContext::CreateStoredDeclsMap(clang::ASTContext&) const"},
    {"_ZN5clang17ExternalASTSource30SetExternalVisibleDeclsForNameEPKNS_11DeclContextENS_15DeclarationNameERN4llvm15SmallVectorImplIPNS_9NamedDeclEEE", "clang::ExternalASTSource::SetExternalVisibleDeclsForName(clang::DeclContext const*, clang::DeclarationName, llvm::SmallVectorImpl<clang::NamedDecl*>&)"},
    {"_ZN5clang17ExternalASTSource30MaterializeVisibleDeclsForNameEPKNS_11DeclContextENS_15DeclarationNameERN4llvm15SmallVectorImplIPNS_9NamedDeclEEE", "clang::ExternalASTSource::MaterializeVisibleDeclsForName(clang::DeclContext const*, clang::DeclarationName, llvm::SmallVectorImpl<clang::NamedDecl*>&)"},
    {"_ZNK5clang11DeclContext18noload_decls_beginEv", "clang::DeclContext::noload_decls_begin() const"},
    {"_ZNK5clang11DeclContext16noload_decls_endEv", "clang::DeclContext::noload_decls_end() const"},
    {"_ZNK5clang11DeclContext11decls_beginEv", "clang::DeclContext::decls_begin() const"},
    {"_ZNK5clang11DeclContext9decls_endEv", "clang::DeclContext::decls_end() const"},
    {"_ZNK5clang11DeclContext11decls_emptyEv", "clang::DeclContext::decls_empty() const"},
    {"_ZN5clang11DeclContext10removeDeclEPNS_4DeclE", "clang::DeclContext::removeDecl(clang::Decl*)"},
    {"_ZN5clang11DeclContext13addHiddenDeclEPNS_4DeclE", "clang::DeclContext::addHiddenDecl(clang::Decl*)"},
    {"_ZN5clang11DeclContext7addDeclEPNS_4DeclE", "clang::DeclContext::addDecl(clang::Decl*)"},
    {"_ZN5clang11DeclContext24makeDeclVisibleInContextEPNS_9NamedDeclEb", "clang::DeclContext::makeDeclVisibleInContext(clang::NamedDecl*, bool)"},
    {"_ZN5clang11DeclContext11buildLookupEPS0_", "clang::DeclContext::buildLookup(clang::DeclContext*)"},
    {"_ZN5clang11DeclContext28makeDeclVisibleInContextImplEPNS_9NamedDeclE", "clang::DeclContext::makeDeclVisibleInContextImpl(clang::NamedDecl*)"},
    {"_ZN5clang11DeclContext6lookupENS_15DeclarationNameE", "clang::DeclContext::lookup(clang::DeclarationName)"},
    {"_ZNK5clang11DeclContext6lookupENS_15DeclarationNameE", "clang::DeclContext::lookup(clang::DeclarationName) const"},
    {"_ZN5clang11DeclContext28getEnclosingNamespaceContextEv", "clang::DeclContext::getEnclosingNamespaceContext()"},
    {"_ZNK5clang11DeclContext25InEnclosingNamespaceSetOfEPKS0_", "clang::DeclContext::InEnclosingNamespaceSetOf(clang::DeclContext const*) const"},
    {"_ZN5clang11DeclContext42MaterializeVisibleDeclsFromExternalStorageEv", "clang::DeclContext::MaterializeVisibleDeclsFromExternalStorage()"},
    {"_ZNK5clang11DeclContext18getUsingDirectivesEv", "clang::DeclContext::getUsingDirectives() const"},
    {"_ZN5clang10ASTContext22ReleaseDeclContextMapsEv", "clang::ASTContext::ReleaseDeclContextMaps()"},
    {"_ZN5clang14StoredDeclsMap10DestroyAllEPS0_b", "clang::StoredDeclsMap::DestroyAll(clang::StoredDeclsMap*, bool)"},
    {"_ZN5clang19DependentDiagnostic6CreateERNS_10ASTContextEPNS_11DeclContextERKNS_17PartialDiagnosticE", "clang::DependentDiagnostic::Create(clang::ASTContext&, clang::DeclContext*, clang::PartialDiagnostic const&)"},
    {"_ZNK5clang4Decl7hasAttrINS_8UsedAttrEEEbv", "bool clang::Decl::hasAttr<clang::UsedAttr>() const"},
    {"_ZN5clang15StoredDeclsList17AddSubsequentDeclEPNS_9NamedDeclE", "clang::StoredDeclsList::AddSubsequentDecl(clang::NamedDecl*)"},
    {"_ZN5clang15StoredDeclsList6removeEPNS_9NamedDeclE", "clang::StoredDeclsList::remove(clang::NamedDecl*)"},
    {"_ZN5clang15StoredDeclsList19HandleRedeclarationEPNS_9NamedDeclE", "clang::StoredDeclsList::HandleRedeclaration(clang::NamedDecl*)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameENS1_15StoredDeclsListENS_12DenseMapInfoIS2_EENS4_IS3_EEE16FindAndConstructERKS2_", "llvm::DenseMap<clang::DeclarationName, clang::StoredDeclsList, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::StoredDeclsList> >::FindAndConstruct(clang::DeclarationName const&)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameENS1_15StoredDeclsListENS_12DenseMapInfoIS2_EENS4_IS3_EEE16InsertIntoBucketERKS2_RKS3_PSt4pairIS2_S3_E", "llvm::DenseMap<clang::DeclarationName, clang::StoredDeclsList, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::StoredDeclsList> >::InsertIntoBucket(clang::DeclarationName const&, clang::StoredDeclsList const&, std::pair<clang::DeclarationName, clang::StoredDeclsList>*)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameENS1_15StoredDeclsListENS_12DenseMapInfoIS2_EENS4_IS3_EEE4growEj", "llvm::DenseMap<clang::DeclarationName, clang::StoredDeclsList, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::StoredDeclsList> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang9NamedDeclEEaSERKS4_", "llvm::SmallVectorImpl<clang::NamedDecl*>::operator=(llvm::SmallVectorImpl<clang::NamedDecl*> const&)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang4AttrEE4swapERS4_", "llvm::SmallVectorImpl<clang::Attr*>::swap(llvm::SmallVectorImpl<clang::Attr*>&)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang4AttrEEaSERKS4_", "llvm::SmallVectorImpl<clang::Attr*>::operator=(llvm::SmallVectorImpl<clang::Attr*> const&)"},
    {"_ZN4llvm8DenseMapIN5clang15DeclarationNameENS1_15StoredDeclsListENS_12DenseMapInfoIS2_EENS4_IS3_EEED2Ev", "llvm::DenseMap<clang::DeclarationName, clang::StoredDeclsList, llvm::DenseMapInfo<clang::DeclarationName>, llvm::DenseMapInfo<clang::StoredDeclsList> >::~DenseMap()"},
    {"_ZSt6__findIPPN5clang9NamedDeclES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "clang::NamedDecl** std::__find<clang::NamedDecl**, clang::NamedDecl*>(clang::NamedDecl**, clang::NamedDecl**, clang::NamedDecl* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang9NamedDeclEE6insertEPS3_RKS3_", "llvm::SmallVectorImpl<clang::NamedDecl*>::insert(clang::NamedDecl**, clang::NamedDecl* const&)"},
    {"_ZN5clang13CXXRecordDecl14DefinitionDataC1EPS0_", "clang::CXXRecordDecl::DefinitionData::DefinitionData(clang::CXXRecordDecl*)"},
    {"_ZN5clang13CXXRecordDecl14DefinitionDataC2EPS0_", "clang::CXXRecordDecl::DefinitionData::DefinitionData(clang::CXXRecordDecl*)"},
    {"_ZN5clang13CXXRecordDeclC1ENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPS0_S6_", "clang::CXXRecordDecl::CXXRecordDecl(clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::CXXRecordDecl*, clang::SourceLocation)"},
    {"_ZN5clang13CXXRecordDeclC2ENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPS0_S6_", "clang::CXXRecordDecl::CXXRecordDecl(clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::CXXRecordDecl*, clang::SourceLocation)"},
    {"_ZN5clang13CXXRecordDecl6CreateERKNS_10ASTContextENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES7_PS0_b", "clang::CXXRecordDecl::Create(clang::ASTContext const&, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::CXXRecordDecl*, bool)"},
    {"_ZN5clang13CXXRecordDecl6CreateERKNS_10ASTContextENS_4Decl10EmptyShellE", "clang::CXXRecordDecl::Create(clang::ASTContext const&, clang::Decl::EmptyShell)"},
    {"_ZN5clang13CXXRecordDecl8setBasesEPKPKNS_16CXXBaseSpecifierEj", "clang::CXXRecordDecl::setBases(clang::CXXBaseSpecifier const* const*, unsigned int)"},
    {"_ZNK5clang13CXXRecordDecl20hasAnyDependentBasesEv", "clang::CXXRecordDecl::hasAnyDependentBases() const"},
    {"_Z7SawBasePKN5clang13CXXRecordDeclEPv", "SawBase(clang::CXXRecordDecl const*, void*)"},
    {"_ZNK5clang13CXXRecordDecl23hasConstCopyConstructorERKNS_10ASTContextE", "clang::CXXRecordDecl::hasConstCopyConstructor(clang::ASTContext const&) const"},
    {"_ZNK5clang13CXXRecordDecl18getCopyConstructorERKNS_10ASTContextEj", "clang::CXXRecordDecl::getCopyConstructor(clang::ASTContext const&, unsigned int) const"},
    {"_ZNK5clang18CXXConstructorDecl17isCopyConstructorERj", "clang::CXXConstructorDecl::isCopyConstructor(unsigned int&) const"},
    {"_Z30GetBestOverloadCandidateSimpleRKN4llvm15SmallVectorImplISt4pairIPN5clang13CXXMethodDeclENS2_10QualifiersEEEE", "GetBestOverloadCandidateSimple(llvm::SmallVectorImpl<std::pair<clang::CXXMethodDecl*, clang::Qualifiers> > const&)"},
    {"_ZNK5clang13CXXRecordDecl25getCopyAssignmentOperatorEb", "clang::CXXRecordDecl::getCopyAssignmentOperator(bool) const"},
    {"_ZN5clang13CXXRecordDecl25markedVirtualFunctionPureEv", "clang::CXXRecordDecl::markedVirtualFunctionPure()"},
    {"_ZN5clang13CXXRecordDecl11addedMemberEPNS_4DeclE", "clang::CXXRecordDecl::addedMember(clang::Decl*)"},
    {"_ZNK5clang18CXXConstructorDecl20isDefaultConstructorEv", "clang::CXXConstructorDecl::isDefaultConstructor() const"},
    {"_ZN5clang13CXXRecordDecl29getVisibleConversionFunctionsEv", "clang::CXXRecordDecl::getVisibleConversionFunctions()"},
    {"_ZN5clang13CXXRecordDecl16removeConversionEPKNS_9NamedDeclE", "clang::CXXRecordDecl::removeConversion(clang::NamedDecl const*)"},
    {"_ZNK5clang13CXXRecordDecl30getInstantiatedFromMemberClassEv", "clang::CXXRecordDecl::getInstantiatedFromMemberClass() const"},
    {"_ZNK5clang13CXXRecordDecl27getMemberSpecializationInfoEv", "clang::CXXRecordDecl::getMemberSpecializationInfo() const"},
    {"_ZN5clang13CXXRecordDecl29setInstantiationOfMemberClassEPS0_NS_26TemplateSpecializationKindE", "clang::CXXRecordDecl::setInstantiationOfMemberClass(clang::CXXRecordDecl*, clang::TemplateSpecializationKind)"},
    {"_ZNK5clang13CXXRecordDecl29getTemplateSpecializationKindEv", "clang::CXXRecordDecl::getTemplateSpecializationKind() const"},
    {"_ZN5clang13CXXRecordDecl29setTemplateSpecializationKindENS_26TemplateSpecializationKindE", "clang::CXXRecordDecl::setTemplateSpecializationKind(clang::TemplateSpecializationKind)"},
    {"_ZNK5clang13CXXRecordDecl13getDestructorEv", "clang::CXXRecordDecl::getDestructor() const"},
    {"_ZN5clang13CXXRecordDecl18completeDefinitionEv", "clang::CXXRecordDecl::completeDefinition()"},
    {"_ZN5clang13CXXRecordDecl18completeDefinitionEPNS_20CXXFinalOverriderMapE", "clang::CXXRecordDecl::completeDefinition(clang::CXXFinalOverriderMap*)"},
    {"_ZNK5clang13CXXRecordDecl13mayBeAbstractEv", "clang::CXXRecordDecl::mayBeAbstract() const"},
    {"_ZN5clang13CXXMethodDecl6CreateERNS_10ASTContextEPNS_13CXXRecordDeclERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoEbNS_12StorageClassEb", "clang::CXXMethodDecl::Create(clang::ASTContext&, clang::CXXRecordDecl*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, bool, clang::StorageClass, bool)"},
    {"_ZNK5clang13CXXMethodDecl27isUsualDeallocationFunctionEv", "clang::CXXMethodDecl::isUsualDeallocationFunction() const"},
    {"_ZNK5clang13CXXMethodDecl24isCopyAssignmentOperatorEv", "clang::CXXMethodDecl::isCopyAssignmentOperator() const"},
    {"_ZN5clang13CXXMethodDecl19addOverriddenMethodEPKS0_", "clang::CXXMethodDecl::addOverriddenMethod(clang::CXXMethodDecl const*)"},
    {"_ZNK5clang13CXXMethodDecl24begin_overridden_methodsEv", "clang::CXXMethodDecl::begin_overridden_methods() const"},
    {"_ZNK5clang13CXXMethodDecl22end_overridden_methodsEv", "clang::CXXMethodDecl::end_overridden_methods() const"},
    {"_ZNK5clang13CXXMethodDecl23size_overridden_methodsEv", "clang::CXXMethodDecl::size_overridden_methods() const"},
    {"_ZNK5clang13CXXMethodDecl11getThisTypeERNS_10ASTContextE", "clang::CXXMethodDecl::getThisType(clang::ASTContext&) const"},
    {"_ZNK5clang13CXXMethodDecl13hasInlineBodyEv", "clang::CXXMethodDecl::hasInlineBody() const"},
    {"_ZN5clang18CXXCtorInitializerC1ERNS_10ASTContextEPNS_14TypeSourceInfoEbNS_14SourceLocationEPNS_4ExprES5_S5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::TypeSourceInfo*, bool, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC2ERNS_10ASTContextEPNS_14TypeSourceInfoEbNS_14SourceLocationEPNS_4ExprES5_S5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::TypeSourceInfo*, bool, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC1ERNS_10ASTContextEPNS_9FieldDeclENS_14SourceLocationES5_PNS_4ExprES5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::FieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC2ERNS_10ASTContextEPNS_9FieldDeclENS_14SourceLocationES5_PNS_4ExprES5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::FieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC1ERNS_10ASTContextEPNS_17IndirectFieldDeclENS_14SourceLocationES5_PNS_4ExprES5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::IndirectFieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC2ERNS_10ASTContextEPNS_17IndirectFieldDeclENS_14SourceLocationES5_PNS_4ExprES5_", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::IndirectFieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang18CXXCtorInitializerC1ERNS_10ASTContextEPNS_9FieldDeclENS_14SourceLocationES5_PNS_4ExprES5_PPNS_7VarDeclEj", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::FieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::VarDecl**, unsigned int)"},
    {"_ZN5clang18CXXCtorInitializerC2ERNS_10ASTContextEPNS_9FieldDeclENS_14SourceLocationES5_PNS_4ExprES5_PPNS_7VarDeclEj", "clang::CXXCtorInitializer::CXXCtorInitializer(clang::ASTContext&, clang::FieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::VarDecl**, unsigned int)"},
    {"_ZN5clang18CXXCtorInitializer6CreateERNS_10ASTContextEPNS_9FieldDeclENS_14SourceLocationES5_PNS_4ExprES5_PPNS_7VarDeclEj", "clang::CXXCtorInitializer::Create(clang::ASTContext&, clang::FieldDecl*, clang::SourceLocation, clang::SourceLocation, clang::Expr*, clang::SourceLocation, clang::VarDecl**, unsigned int)"},
    {"_ZNK5clang18CXXCtorInitializer15getBaseClassLocEv", "clang::CXXCtorInitializer::getBaseClassLoc() const"},
    {"_ZNK5clang18CXXCtorInitializer12getBaseClassEv", "clang::CXXCtorInitializer::getBaseClass() const"},
    {"_ZNK5clang18CXXCtorInitializer17getSourceLocationEv", "clang::CXXCtorInitializer::getSourceLocation() const"},
    {"_ZNK5clang18CXXCtorInitializer14getSourceRangeEv", "clang::CXXCtorInitializer::getSourceRange() const"},
    {"_ZN5clang18CXXConstructorDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::CXXConstructorDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang18CXXConstructorDecl6CreateERNS_10ASTContextEPNS_13CXXRecordDeclERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoEbbb", "clang::CXXConstructorDecl::Create(clang::ASTContext&, clang::CXXRecordDecl*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, bool, bool, bool)"},
    {"_ZNK5clang18CXXConstructorDecl23isCopyOrMoveConstructorERj", "clang::CXXConstructorDecl::isCopyOrMoveConstructor(unsigned int&) const"},
    {"_ZNK5clang18CXXConstructorDecl17isMoveConstructorERj", "clang::CXXConstructorDecl::isMoveConstructor(unsigned int&) const"},
    {"_ZNK5clang18CXXConstructorDecl23isConvertingConstructorEb", "clang::CXXConstructorDecl::isConvertingConstructor(bool) const"},
    {"_ZNK5clang18CXXConstructorDecl29isSpecializationCopyingObjectEv", "clang::CXXConstructorDecl::isSpecializationCopyingObject() const"},
    {"_ZNK5clang18CXXConstructorDecl23getInheritedConstructorEv", "clang::CXXConstructorDecl::getInheritedConstructor() const"},
    {"_ZN5clang18CXXConstructorDecl23setInheritedConstructorEPKS0_", "clang::CXXConstructorDecl::setInheritedConstructor(clang::CXXConstructorDecl const*)"},
    {"_ZN5clang17CXXDestructorDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::CXXDestructorDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang17CXXDestructorDecl6CreateERNS_10ASTContextEPNS_13CXXRecordDeclERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoEbb", "clang::CXXDestructorDecl::Create(clang::ASTContext&, clang::CXXRecordDecl*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, bool, bool)"},
    {"_ZN5clang17CXXConversionDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::CXXConversionDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang17CXXConversionDecl6CreateERNS_10ASTContextEPNS_13CXXRecordDeclERKNS_19DeclarationNameInfoENS_8QualTypeEPNS_14TypeSourceInfoEbb", "clang::CXXConversionDecl::Create(clang::ASTContext&, clang::CXXRecordDecl*, clang::DeclarationNameInfo const&, clang::QualType, clang::TypeSourceInfo*, bool, bool)"},
    {"_ZN5clang15LinkageSpecDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationENS0_11LanguageIDsEb", "clang::LinkageSpecDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::LinkageSpecDecl::LanguageIDs, bool)"},
    {"_ZN5clang18UsingDirectiveDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationES5_NS_11SourceRangeEPNS_19NestedNameSpecifierES5_PNS_9NamedDeclES4_", "clang::UsingDirectiveDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation, clang::SourceRange, clang::NestedNameSpecifier*, clang::SourceLocation, clang::NamedDecl*, clang::DeclContext*)"},
    {"_ZN5clang18UsingDirectiveDecl21getNominatedNamespaceEv", "clang::UsingDirectiveDecl::getNominatedNamespace()"},
    {"_ZN5clang18NamespaceAliasDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationES5_PNS_14IdentifierInfoENS_11SourceRangeEPNS_19NestedNameSpecifierES5_PNS_9NamedDeclE", "clang::NamespaceAliasDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceRange, clang::NestedNameSpecifier*, clang::SourceLocation, clang::NamedDecl*)"},
    {"_ZNK5clang15UsingShadowDecl12getUsingDeclEv", "clang::UsingShadowDecl::getUsingDecl() const"},
    {"_ZN5clang9UsingDecl13addShadowDeclEPNS_15UsingShadowDeclE", "clang::UsingDecl::addShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang9UsingDecl16removeShadowDeclEPNS_15UsingShadowDeclE", "clang::UsingDecl::removeShadowDecl(clang::UsingShadowDecl*)"},
    {"_ZN5clang9UsingDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_11SourceRangeENS_14SourceLocationEPNS_19NestedNameSpecifierERKNS_19DeclarationNameInfoEb", "clang::UsingDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceRange, clang::SourceLocation, clang::NestedNameSpecifier*, clang::DeclarationNameInfo const&, bool)"},
    {"_ZN5clang24UnresolvedUsingValueDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationENS_11SourceRangeEPNS_19NestedNameSpecifierERKNS_19DeclarationNameInfoE", "clang::UnresolvedUsingValueDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceRange, clang::NestedNameSpecifier*, clang::DeclarationNameInfo const&)"},
    {"_ZN5clang27UnresolvedUsingTypenameDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationES5_NS_11SourceRangeEPNS_19NestedNameSpecifierES5_NS_15DeclarationNameE", "clang::UnresolvedUsingTypenameDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation, clang::SourceRange, clang::NestedNameSpecifier*, clang::SourceLocation, clang::DeclarationName)"},
    {"_ZN5clang16StaticAssertDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_4ExprEPNS_13StringLiteralE", "clang::StaticAssertDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::Expr*, clang::StringLiteral*)"},
    {"_ZN5clanglsERKNS_17DiagnosticBuilderENS_15AccessSpecifierE", "clang::operator<<(clang::DiagnosticBuilder const&, clang::AccessSpecifier)"},
    {"_Z17GetConversionTypeRN5clang10ASTContextEPNS_9NamedDeclE", "GetConversionType(clang::ASTContext&, clang::NamedDecl*)"},
    {"_Z25CollectVisibleConversionsRN5clang10ASTContextEPNS_13CXXRecordDeclEbNS_15AccessSpecifierERKN4llvm11SmallPtrSetINS_7CanQualINS_4TypeEEELj8EEERNS_17UnresolvedSetImplESE_RNS6_IPNS_9NamedDeclELj8EEE", "CollectVisibleConversions(clang::ASTContext&, clang::CXXRecordDecl*, bool, clang::AccessSpecifier, llvm::SmallPtrSet<clang::CanQual<clang::Type>, 8u> const&, clang::UnresolvedSetImpl&, clang::UnresolvedSetImpl&, llvm::SmallPtrSet<clang::NamedDecl*, 8u>&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPN5clang13CXXMethodDeclENS2_10QualifiersEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<clang::CXXMethodDecl*, clang::Qualifiers>, false>::grow(unsigned long)"},
    {"_ZN5clang16StaticAssertDeclD1Ev", "clang::StaticAssertDecl::~StaticAssertDecl()"},
    {"_ZN5clang16StaticAssertDeclD0Ev", "clang::StaticAssertDecl::~StaticAssertDecl()"},
    {"_ZN5clang27UnresolvedUsingTypenameDeclD1Ev", "clang::UnresolvedUsingTypenameDecl::~UnresolvedUsingTypenameDecl()"},
    {"_ZN5clang27UnresolvedUsingTypenameDeclD0Ev", "clang::UnresolvedUsingTypenameDecl::~UnresolvedUsingTypenameDecl()"},
    {"_ZNK5clang27UnresolvedUsingTypenameDecl14getSourceRangeEv", "clang::UnresolvedUsingTypenameDecl::getSourceRange() const"},
    {"_ZN5clang24UnresolvedUsingValueDeclD1Ev", "clang::UnresolvedUsingValueDecl::~UnresolvedUsingValueDecl()"},
    {"_ZN5clang24UnresolvedUsingValueDeclD0Ev", "clang::UnresolvedUsingValueDecl::~UnresolvedUsingValueDecl()"},
    {"_ZNK5clang24UnresolvedUsingValueDecl14getSourceRangeEv", "clang::UnresolvedUsingValueDecl::getSourceRange() const"},
    {"_ZN5clang9UsingDeclD1Ev", "clang::UsingDecl::~UsingDecl()"},
    {"_ZN5clang9UsingDeclD0Ev", "clang::UsingDecl::~UsingDecl()"},
    {"_ZNK5clang9UsingDecl14getSourceRangeEv", "clang::UsingDecl::getSourceRange() const"},
    {"_ZN5clang18NamespaceAliasDeclD1Ev", "clang::NamespaceAliasDecl::~NamespaceAliasDecl()"},
    {"_ZN5clang18NamespaceAliasDeclD0Ev", "clang::NamespaceAliasDecl::~NamespaceAliasDecl()"},
    {"_ZNK5clang18NamespaceAliasDecl14getSourceRangeEv", "clang::NamespaceAliasDecl::getSourceRange() const"},
    {"_ZN5clang18UsingDirectiveDeclD1Ev", "clang::UsingDirectiveDecl::~UsingDirectiveDecl()"},
    {"_ZN5clang18UsingDirectiveDeclD0Ev", "clang::UsingDirectiveDecl::~UsingDirectiveDecl()"},
    {"_ZNK5clang18UsingDirectiveDecl14getSourceRangeEv", "clang::UsingDirectiveDecl::getSourceRange() const"},
    {"_ZN5clang15LinkageSpecDeclC2EPNS_11DeclContextENS_14SourceLocationENS0_11LanguageIDsEb", "clang::LinkageSpecDecl::LinkageSpecDecl(clang::DeclContext*, clang::SourceLocation, clang::LinkageSpecDecl::LanguageIDs, bool)"},
    {"_ZN5clang15LinkageSpecDeclD1Ev", "clang::LinkageSpecDecl::~LinkageSpecDecl()"},
    {"_ZN5clang15LinkageSpecDeclD0Ev", "clang::LinkageSpecDecl::~LinkageSpecDecl()"},
    {"_ZN5clang17CXXConversionDeclD1Ev", "clang::CXXConversionDecl::~CXXConversionDecl()"},
    {"_ZN5clang17CXXConversionDeclD0Ev", "clang::CXXConversionDecl::~CXXConversionDecl()"},
    {"_ZN5clang13CXXMethodDecl16getCanonicalDeclEv", "clang::CXXMethodDecl::getCanonicalDecl()"},
    {"_ZNK5clang13CXXMethodDecl16getCanonicalDeclEv", "clang::CXXMethodDecl::getCanonicalDecl() const"},
    {"_ZN5clang13CXXMethodDeclD1Ev", "clang::CXXMethodDecl::~CXXMethodDecl()"},
    {"_ZN5clang13CXXMethodDeclD0Ev", "clang::CXXMethodDecl::~CXXMethodDecl()"},
    {"_ZN5clang17CXXDestructorDeclD1Ev", "clang::CXXDestructorDecl::~CXXDestructorDecl()"},
    {"_ZN5clang17CXXDestructorDeclD0Ev", "clang::CXXDestructorDecl::~CXXDestructorDecl()"},
    {"_ZN5clang18CXXConstructorDeclD1Ev", "clang::CXXConstructorDecl::~CXXConstructorDecl()"},
    {"_ZN5clang18CXXConstructorDeclD0Ev", "clang::CXXConstructorDecl::~CXXConstructorDecl()"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXMethodDeclENS1_17OverridingMethodsENS_12DenseMapInfoIS4_EENS6_IS5_EEED2Ev", "llvm::DenseMap<clang::CXXMethodDecl const*, clang::OverridingMethods, llvm::DenseMapInfo<clang::CXXMethodDecl const*>, llvm::DenseMapInfo<clang::OverridingMethods> >::~DenseMap()"},
    {"_ZN5clang10FriendDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEN4llvm12PointerUnionIPNS_9NamedDeclEPNS_14TypeSourceInfoEEES5_", "clang::FriendDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, llvm::PointerUnion<clang::NamedDecl*, clang::TypeSourceInfo*>, clang::SourceLocation)"},
    {"_ZN5clang10FriendDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::FriendDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang10FriendDeclD1Ev", "clang::FriendDecl::~FriendDecl()"},
    {"_ZN5clang10FriendDeclD0Ev", "clang::FriendDecl::~FriendDecl()"},
    {"_ZN5clang9DeclGroup6CreateERNS_10ASTContextEPPNS_4DeclEj", "clang::DeclGroup::Create(clang::ASTContext&, clang::Decl**, unsigned int)"},
    {"_ZN5clang9DeclGroupC1EjPPNS_4DeclE", "clang::DeclGroup::DeclGroup(unsigned int, clang::Decl**)"},
    {"_ZN5clang9DeclGroupC2EjPPNS_4DeclE", "clang::DeclGroup::DeclGroup(unsigned int, clang::Decl**)"},
    {"_ZN5clang12ObjCListBase3setEPKPvjRNS_10ASTContextE", "clang::ObjCListBase::set(void* const*, unsigned int, clang::ASTContext&)"},
    {"_ZN5clang16ObjCProtocolList3setEPKPNS_16ObjCProtocolDeclEjPKNS_14SourceLocationERNS_10ASTContextE", "clang::ObjCProtocolList::set(clang::ObjCProtocolDecl* const*, unsigned int, clang::SourceLocation const*, clang::ASTContext&)"},
    {"_ZNK5clang17ObjCContainerDecl11getIvarDeclEPNS_14IdentifierInfoE", "clang::ObjCContainerDecl::getIvarDecl(clang::IdentifierInfo*) const"},
    {"_ZNK5clang17ObjCContainerDecl9getMethodENS_8SelectorEb", "clang::ObjCContainerDecl::getMethod(clang::Selector, bool) const"},
    {"_ZN5clang16ObjCPropertyDecl16findPropertyDeclEPKNS_11DeclContextEPNS_14IdentifierInfoE", "clang::ObjCPropertyDecl::findPropertyDecl(clang::DeclContext const*, clang::IdentifierInfo*)"},
    {"_ZNK5clang17ObjCContainerDecl23FindPropertyDeclarationEPNS_14IdentifierInfoE", "clang::ObjCContainerDecl::FindPropertyDeclaration(clang::IdentifierInfo*) const"},
    {"_ZNK5clang17ObjCInterfaceDecl33FindPropertyVisibleInPrimaryClassEPNS_14IdentifierInfoE", "clang::ObjCInterfaceDecl::FindPropertyVisibleInPrimaryClass(clang::IdentifierInfo*) const"},
    {"_ZNK5clang17ObjCInterfaceDecl22LoadExternalDefinitionEv", "clang::ObjCInterfaceDecl::LoadExternalDefinition() const"},
    {"_ZN5clang17ObjCInterfaceDecl31mergeClassExtensionProtocolListEPKPNS_16ObjCProtocolDeclEjRNS_10ASTContextE", "clang::ObjCInterfaceDecl::mergeClassExtensionProtocolList(clang::ObjCProtocolDecl* const*, unsigned int, clang::ASTContext&)"},
    {"_ZNK5clang17ObjCInterfaceDecl22getFirstClassExtensionEv", "clang::ObjCInterfaceDecl::getFirstClassExtension() const"},
    {"_ZNK5clang16ObjCCategoryDecl21getNextClassExtensionEv", "clang::ObjCCategoryDecl::getNextClassExtension() const"},
    {"_ZN5clang17ObjCInterfaceDecl22lookupInstanceVariableEPNS_14IdentifierInfoERPS0_", "clang::ObjCInterfaceDecl::lookupInstanceVariable(clang::IdentifierInfo*, clang::ObjCInterfaceDecl*&)"},
    {"_ZN5clang17ObjCInterfaceDecl20lookupInheritedClassEPKNS_14IdentifierInfoE", "clang::ObjCInterfaceDecl::lookupInheritedClass(clang::IdentifierInfo const*)"},
    {"_ZNK5clang17ObjCInterfaceDecl12lookupMethodENS_8SelectorEb", "clang::ObjCInterfaceDecl::lookupMethod(clang::Selector, bool) const"},
    {"_ZNK5clang16ObjCProtocolDecl12lookupMethodENS_8SelectorEb", "clang::ObjCProtocolDecl::lookupMethod(clang::Selector, bool) const"},
    {"_ZN5clang17ObjCInterfaceDecl19lookupPrivateMethodERKNS_8SelectorEb", "clang::ObjCInterfaceDecl::lookupPrivateMethod(clang::Selector const&, bool)"},
    {"_ZNK5clang17ObjCInterfaceDecl17getImplementationEv", "clang::ObjCInterfaceDecl::getImplementation() const"},
    {"_ZN5clang14ObjCMethodDecl6CreateERNS_10ASTContextENS_14SourceLocationES3_NS_8SelectorENS_8QualTypeEPNS_14TypeSourceInfoEPNS_11DeclContextEbbbbNS0_21ImplementationControlEj", "clang::ObjCMethodDecl::Create(clang::ASTContext&, clang::SourceLocation, clang::SourceLocation, clang::Selector, clang::QualType, clang::TypeSourceInfo*, clang::DeclContext*, bool, bool, bool, bool, clang::ObjCMethodDecl::ImplementationControl, unsigned int)"},
    {"_ZN5clang14ObjCMethodDecl20getNextRedeclarationEv", "clang::ObjCMethodDecl::getNextRedeclaration()"},
    {"_ZNK5clang20ObjCCategoryImplDecl15getCategoryDeclEv", "clang::ObjCCategoryImplDecl::getCategoryDecl() const"},
    {"_ZN5clang14ObjCMethodDecl16getCanonicalDeclEv", "clang::ObjCMethodDecl::getCanonicalDecl()"},
    {"_ZN5clang14ObjCMethodDecl20createImplicitParamsERNS_10ASTContextEPKNS_17ObjCInterfaceDeclE", "clang::ObjCMethodDecl::createImplicitParams(clang::ASTContext&, clang::ObjCInterfaceDecl const*)"},
    {"_ZN5clang14ObjCMethodDecl17getClassInterfaceEv", "clang::ObjCMethodDecl::getClassInterface()"},
    {"_ZN5clang17ObjCInterfaceDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES5_bb", "clang::ObjCInterfaceDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang17ObjCInterfaceDeclC1EPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES3_bb", "clang::ObjCInterfaceDecl::ObjCInterfaceDecl(clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang17ObjCInterfaceDeclC2EPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES3_bb", "clang::ObjCInterfaceDecl::ObjCInterfaceDecl(clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang17ObjCInterfaceDecl22setExternallyCompletedEv", "clang::ObjCInterfaceDecl::setExternallyCompleted()"},
    {"_ZN5clang17ObjCInterfaceDecl17setImplementationEPNS_22ObjCImplementationDeclE", "clang::ObjCInterfaceDecl::setImplementation(clang::ObjCImplementationDecl*)"},
    {"_ZN5clang17ObjCInterfaceDecl23all_declared_ivar_beginEv", "clang::ObjCInterfaceDecl::all_declared_ivar_begin()"},
    {"_ZNK5clang17ObjCInterfaceDecl23FindCategoryDeclarationEPNS_14IdentifierInfoE", "clang::ObjCInterfaceDecl::FindCategoryDeclaration(clang::IdentifierInfo*) const"},
    {"_ZNK5clang17ObjCInterfaceDecl25getCategoryInstanceMethodENS_8SelectorE", "clang::ObjCInterfaceDecl::getCategoryInstanceMethod(clang::Selector) const"},
    {"_ZNK5clang16ObjCCategoryDecl17getImplementationEv", "clang::ObjCCategoryDecl::getImplementation() const"},
    {"_ZNK5clang17ObjCInterfaceDecl22getCategoryClassMethodENS_8SelectorE", "clang::ObjCInterfaceDecl::getCategoryClassMethod(clang::Selector) const"},
    {"_ZN5clang17ObjCInterfaceDecl23ClassImplementsProtocolEPNS_16ObjCProtocolDeclEbb", "clang::ObjCInterfaceDecl::ClassImplementsProtocol(clang::ObjCProtocolDecl*, bool, bool)"},
    {"_ZN5clang12ObjCIvarDecl6CreateERNS_10ASTContextEPNS_17ObjCContainerDeclENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoENS0_13AccessControlEPNS_4ExprEb", "clang::ObjCIvarDecl::Create(clang::ASTContext&, clang::ObjCContainerDecl*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::ObjCIvarDecl::AccessControl, clang::Expr*, bool)"},
    {"_ZNK5clang12ObjCIvarDecl22getContainingInterfaceEv", "clang::ObjCIvarDecl::getContainingInterface() const"},
    {"_ZN5clang19ObjCAtDefsFieldDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_4ExprE", "clang::ObjCAtDefsFieldDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::Expr*)"},
    {"_ZN5clang16ObjCProtocolDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoE", "clang::ObjCProtocolDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang16ObjCProtocolDecl19lookupProtocolNamedEPNS_14IdentifierInfoE", "clang::ObjCProtocolDecl::lookupProtocolNamed(clang::IdentifierInfo*)"},
    {"_ZN5clang13ObjCClassDeclC1EPNS_11DeclContextENS_14SourceLocationEPKPNS_17ObjCInterfaceDeclEPKS3_jRNS_10ASTContextE", "clang::ObjCClassDecl::ObjCClassDecl(clang::DeclContext*, clang::SourceLocation, clang::ObjCInterfaceDecl* const*, clang::SourceLocation const*, unsigned int, clang::ASTContext&)"},
    {"_ZN5clang13ObjCClassDeclC2EPNS_11DeclContextENS_14SourceLocationEPKPNS_17ObjCInterfaceDeclEPKS3_jRNS_10ASTContextE", "clang::ObjCClassDecl::ObjCClassDecl(clang::DeclContext*, clang::SourceLocation, clang::ObjCInterfaceDecl* const*, clang::SourceLocation const*, unsigned int, clang::ASTContext&)"},
    {"_ZN5clang13ObjCClassDecl12setClassListERNS_10ASTContextEPKPNS_17ObjCInterfaceDeclEPKNS_14SourceLocationEj", "clang::ObjCClassDecl::setClassList(clang::ASTContext&, clang::ObjCInterfaceDecl* const*, clang::SourceLocation const*, unsigned int)"},
    {"_ZN5clang13ObjCClassDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPKPNS_17ObjCInterfaceDeclEPKS5_j", "clang::ObjCClassDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::ObjCInterfaceDecl* const*, clang::SourceLocation const*, unsigned int)"},
    {"_ZNK5clang13ObjCClassDecl14getSourceRangeEv", "clang::ObjCClassDecl::getSourceRange() const"},
    {"_ZN5clang23ObjCForwardProtocolDeclC1EPNS_11DeclContextENS_14SourceLocationEPKPNS_16ObjCProtocolDeclEjPKS3_RNS_10ASTContextE", "clang::ObjCForwardProtocolDecl::ObjCForwardProtocolDecl(clang::DeclContext*, clang::SourceLocation, clang::ObjCProtocolDecl* const*, unsigned int, clang::SourceLocation const*, clang::ASTContext&)"},
    {"_ZN5clang23ObjCForwardProtocolDeclC2EPNS_11DeclContextENS_14SourceLocationEPKPNS_16ObjCProtocolDeclEjPKS3_RNS_10ASTContextE", "clang::ObjCForwardProtocolDecl::ObjCForwardProtocolDecl(clang::DeclContext*, clang::SourceLocation, clang::ObjCProtocolDecl* const*, unsigned int, clang::SourceLocation const*, clang::ASTContext&)"},
    {"_ZN5clang23ObjCForwardProtocolDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPKPNS_16ObjCProtocolDeclEjPKS5_", "clang::ObjCForwardProtocolDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::ObjCProtocolDecl* const*, unsigned int, clang::SourceLocation const*)"},
    {"_ZN5clang16ObjCCategoryDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationES5_S5_PNS_14IdentifierInfoE", "clang::ObjCCategoryDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang16ObjCCategoryDecl17setImplementationEPNS_20ObjCCategoryImplDeclE", "clang::ObjCCategoryDecl::setImplementation(clang::ObjCCategoryImplDecl*)"},
    {"_ZN5clang20ObjCCategoryImplDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPNS_17ObjCInterfaceDeclE", "clang::ObjCCategoryImplDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang12ObjCImplDecl25addPropertyImplementationEPNS_20ObjCPropertyImplDeclE", "clang::ObjCImplDecl::addPropertyImplementation(clang::ObjCPropertyImplDecl*)"},
    {"_ZN5clang12ObjCImplDecl17setClassInterfaceEPNS_17ObjCInterfaceDeclE", "clang::ObjCImplDecl::setClassInterface(clang::ObjCInterfaceDecl*)"},
    {"_ZNK5clang12ObjCImplDecl24FindPropertyImplIvarDeclEPNS_14IdentifierInfoE", "clang::ObjCImplDecl::FindPropertyImplIvarDecl(clang::IdentifierInfo*) const"},
    {"_ZNK5clang12ObjCImplDecl20FindPropertyImplDeclEPNS_14IdentifierInfoE", "clang::ObjCImplDecl::FindPropertyImplDecl(clang::IdentifierInfo*) const"},
    {"_ZN5clanglsERN4llvm11raw_ostreamEPKNS_20ObjCCategoryImplDeclE", "clang::operator<<(llvm::raw_ostream&, clang::ObjCCategoryImplDecl const*)"},
    {"_ZN5clang22ObjCImplementationDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_17ObjCInterfaceDeclES7_", "clang::ObjCImplementationDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::ObjCInterfaceDecl*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clanglsERN4llvm11raw_ostreamEPKNS_22ObjCImplementationDeclE", "clang::operator<<(llvm::raw_ostream&, clang::ObjCImplementationDecl const*)"},
    {"_ZN5clang23ObjCCompatibleAliasDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEPNS_17ObjCInterfaceDeclE", "clang::ObjCCompatibleAliasDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::ObjCInterfaceDecl*)"},
    {"_ZN5clang16ObjCPropertyDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoES5_PNS_14TypeSourceInfoENS0_15PropertyControlE", "clang::ObjCPropertyDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, clang::SourceLocation, clang::TypeSourceInfo*, clang::ObjCPropertyDecl::PropertyControl)"},
    {"_ZN5clang20ObjCPropertyImplDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationES5_PNS_16ObjCPropertyDeclENS0_4KindEPNS_12ObjCIvarDeclES5_", "clang::ObjCPropertyImplDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::SourceLocation, clang::ObjCPropertyDecl*, clang::ObjCPropertyImplDecl::Kind, clang::ObjCIvarDecl*, clang::SourceLocation)"},
    {"_ZNK5clang20ObjCPropertyImplDecl14getSourceRangeEv", "clang::ObjCPropertyImplDecl::getSourceRange() const"},
    {"_ZNK5clang17ObjCInterfaceDecl29all_referenced_protocol_beginEv", "clang::ObjCInterfaceDecl::all_referenced_protocol_begin() const"},
    {"_ZNK5clang17ObjCInterfaceDecl27all_referenced_protocol_endEv", "clang::ObjCInterfaceDecl::all_referenced_protocol_end() const"},
    {"_ZN5clang17ObjCContainerDeclC2ENS_4Decl4KindEPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoE", "clang::ObjCContainerDecl::ObjCContainerDecl(clang::Decl::Kind, clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*)"},
    {"_ZN5clang14ObjCMethodDeclD1Ev", "clang::ObjCMethodDecl::~ObjCMethodDecl()"},
    {"_ZN5clang14ObjCMethodDeclD0Ev", "clang::ObjCMethodDecl::~ObjCMethodDecl()"},
    {"_ZNK5clang14ObjCMethodDecl14getSourceRangeEv", "clang::ObjCMethodDecl::getSourceRange() const"},
    {"_ZNK5clang14ObjCMethodDecl7getBodyEv", "clang::ObjCMethodDecl::getBody() const"},
    {"_ZN5clang20ObjCPropertyImplDeclD1Ev", "clang::ObjCPropertyImplDecl::~ObjCPropertyImplDecl()"},
    {"_ZN5clang20ObjCPropertyImplDeclD0Ev", "clang::ObjCPropertyImplDecl::~ObjCPropertyImplDecl()"},
    {"_ZN5clang13ObjCClassDeclD1Ev", "clang::ObjCClassDecl::~ObjCClassDecl()"},
    {"_ZN5clang13ObjCClassDeclD0Ev", "clang::ObjCClassDecl::~ObjCClassDecl()"},
    {"_ZN5clang23ObjCForwardProtocolDeclD1Ev", "clang::ObjCForwardProtocolDecl::~ObjCForwardProtocolDecl()"},
    {"_ZN5clang23ObjCForwardProtocolDeclD0Ev", "clang::ObjCForwardProtocolDecl::~ObjCForwardProtocolDecl()"},
    {"_ZN5clang17ObjCInterfaceDeclD1Ev", "clang::ObjCInterfaceDecl::~ObjCInterfaceDecl()"},
    {"_ZN5clang17ObjCInterfaceDeclD0Ev", "clang::ObjCInterfaceDecl::~ObjCInterfaceDecl()"},
    {"_ZNK5clang17ObjCContainerDecl14getSourceRangeEv", "clang::ObjCContainerDecl::getSourceRange() const"},
    {"_ZN5clang17ObjCContainerDeclD1Ev", "clang::ObjCContainerDecl::~ObjCContainerDecl()"},
    {"_ZN5clang17ObjCContainerDeclD0Ev", "clang::ObjCContainerDecl::~ObjCContainerDecl()"},
    {"_ZN5clang16ObjCPropertyDeclD1Ev", "clang::ObjCPropertyDecl::~ObjCPropertyDecl()"},
    {"_ZN5clang16ObjCPropertyDeclD0Ev", "clang::ObjCPropertyDecl::~ObjCPropertyDecl()"},
    {"_ZNK5clang16ObjCPropertyDecl14getSourceRangeEv", "clang::ObjCPropertyDecl::getSourceRange() const"},
    {"_ZN5clang23ObjCCompatibleAliasDeclD1Ev", "clang::ObjCCompatibleAliasDecl::~ObjCCompatibleAliasDecl()"},
    {"_ZN5clang23ObjCCompatibleAliasDeclD0Ev", "clang::ObjCCompatibleAliasDecl::~ObjCCompatibleAliasDecl()"},
    {"_ZN5clang22ObjCImplementationDeclD1Ev", "clang::ObjCImplementationDecl::~ObjCImplementationDecl()"},
    {"_ZN5clang22ObjCImplementationDeclD0Ev", "clang::ObjCImplementationDecl::~ObjCImplementationDecl()"},
    {"_ZN5clang12ObjCImplDeclD1Ev", "clang::ObjCImplDecl::~ObjCImplDecl()"},
    {"_ZN5clang12ObjCImplDeclD0Ev", "clang::ObjCImplDecl::~ObjCImplDecl()"},
    {"_ZN5clang20ObjCCategoryImplDeclD1Ev", "clang::ObjCCategoryImplDecl::~ObjCCategoryImplDecl()"},
    {"_ZN5clang20ObjCCategoryImplDeclD0Ev", "clang::ObjCCategoryImplDecl::~ObjCCategoryImplDecl()"},
    {"_ZN5clang16ObjCCategoryDeclD1Ev", "clang::ObjCCategoryDecl::~ObjCCategoryDecl()"},
    {"_ZN5clang16ObjCCategoryDeclD0Ev", "clang::ObjCCategoryDecl::~ObjCCategoryDecl()"},
    {"_ZNK5clang16ObjCCategoryDecl14getSourceRangeEv", "clang::ObjCCategoryDecl::getSourceRange() const"},
    {"_ZN5clang16ObjCProtocolDeclD1Ev", "clang::ObjCProtocolDecl::~ObjCProtocolDecl()"},
    {"_ZN5clang16ObjCProtocolDeclD0Ev", "clang::ObjCProtocolDecl::~ObjCProtocolDecl()"},
    {"_ZN5clang19ObjCAtDefsFieldDeclD1Ev", "clang::ObjCAtDefsFieldDecl::~ObjCAtDefsFieldDecl()"},
    {"_ZN5clang19ObjCAtDefsFieldDeclD0Ev", "clang::ObjCAtDefsFieldDecl::~ObjCAtDefsFieldDecl()"},
    {"_ZN5clang12ObjCIvarDeclC2EPNS_17ObjCContainerDeclENS_14SourceLocationEPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoENS0_13AccessControlEPNS_4ExprEb", "clang::ObjCIvarDecl::ObjCIvarDecl(clang::ObjCContainerDecl*, clang::SourceLocation, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::ObjCIvarDecl::AccessControl, clang::Expr*, bool)"},
    {"_ZN5clang12ObjCIvarDeclD1Ev", "clang::ObjCIvarDecl::~ObjCIvarDecl()"},
    {"_ZN5clang12ObjCIvarDeclD0Ev", "clang::ObjCIvarDecl::~ObjCIvarDecl()"},
    {"_ZN5clang14ObjCMethodDeclC2ENS_14SourceLocationES1_NS_8SelectorENS_8QualTypeEPNS_14TypeSourceInfoEPNS_11DeclContextEbbbbNS0_21ImplementationControlEj", "clang::ObjCMethodDecl::ObjCMethodDecl(clang::SourceLocation, clang::SourceLocation, clang::Selector, clang::QualType, clang::TypeSourceInfo*, clang::DeclContext*, bool, bool, bool, bool, clang::ObjCMethodDecl::ImplementationControl, unsigned int)"},
    {"_ZNK5clang4Decl5printERN4llvm11raw_ostreamEj", "clang::Decl::print(llvm::raw_ostream&, unsigned int) const"},
    {"_ZNK5clang4Decl5printERN4llvm11raw_ostreamERKNS_14PrintingPolicyEj", "clang::Decl::print(llvm::raw_ostream&, clang::PrintingPolicy const&, unsigned int) const"},
    {"_ZN5clang11DeclVisitorIN12_GLOBAL__N_111DeclPrinterEvE5VisitEPNS_4DeclE", "clang::DeclVisitor<(anonymous namespace)::DeclPrinter, void>::Visit(clang::Decl*)"},
    {"_ZN5clang4Decl10printGroupEPPS0_jRN4llvm11raw_ostreamERKNS_14PrintingPolicyEj", "clang::Decl::printGroup(clang::Decl**, unsigned int, llvm::raw_ostream&, clang::PrintingPolicy const&, unsigned int)"},
    {"_ZNK5clang11DeclContext15dumpDeclContextEv", "clang::DeclContext::dumpDeclContext() const"},
    {"_ZN12_GLOBAL__N_111DeclPrinter16VisitDeclContextEPN5clang11DeclContextEb", "(anonymous namespace)::DeclPrinter::VisitDeclContext(clang::DeclContext*, bool)"},
    {"_ZNK5clang4Decl4dumpEv", "clang::Decl::dump() const"},
    {"_ZN12_GLOBAL__N_111DeclPrinter18VisitCXXRecordDeclEPN5clang13CXXRecordDeclE", "(anonymous namespace)::DeclPrinter::VisitCXXRecordDecl(clang::CXXRecordDecl*)"},
    {"_ZN12_GLOBAL__N_111DeclPrinter14VisitFieldDeclEPN5clang9FieldDeclE", "(anonymous namespace)::DeclPrinter::VisitFieldDecl(clang::FieldDecl*)"},
    {"_ZN12_GLOBAL__N_111DeclPrinter17VisitFunctionDeclEPN5clang12FunctionDeclE", "(anonymous namespace)::DeclPrinter::VisitFunctionDecl(clang::FunctionDecl*)"},
    {"_ZN12_GLOBAL__N_111DeclPrinter12VisitVarDeclEPN5clang7VarDeclE", "(anonymous namespace)::DeclPrinter::VisitVarDecl(clang::VarDecl*)"},
    {"_ZN12_GLOBAL__N_111DeclPrinter17VisitTemplateDeclEPN5clang12TemplateDeclE", "(anonymous namespace)::DeclPrinter::VisitTemplateDecl(clang::TemplateDecl*)"},
    {"_ZN12_GLOBAL__N_111DeclPrinter16ProcessDeclGroupERN4llvm15SmallVectorImplIPN5clang4DeclEEE", "(anonymous namespace)::DeclPrinter::ProcessDeclGroup(llvm::SmallVectorImpl<clang::Decl*>&)"},
    {"_ZN5clang21TemplateParameterListC1ENS_14SourceLocationES1_PPNS_9NamedDeclEjS1_", "clang::TemplateParameterList::TemplateParameterList(clang::SourceLocation, clang::SourceLocation, clang::NamedDecl**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang21TemplateParameterListC2ENS_14SourceLocationES1_PPNS_9NamedDeclEjS1_", "clang::TemplateParameterList::TemplateParameterList(clang::SourceLocation, clang::SourceLocation, clang::NamedDecl**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang21TemplateParameterList6CreateERKNS_10ASTContextENS_14SourceLocationES4_PPNS_9NamedDeclEjS4_", "clang::TemplateParameterList::Create(clang::ASTContext const&, clang::SourceLocation, clang::SourceLocation, clang::NamedDecl**, unsigned int, clang::SourceLocation)"},
    {"_ZNK5clang21TemplateParameterList23getMinRequiredArgumentsEv", "clang::TemplateParameterList::getMinRequiredArguments() const"},
    {"_ZNK5clang21TemplateParameterList8getDepthEv", "clang::TemplateParameterList::getDepth() const"},
    {"_ZNK5clang20TemplateTypeParmDecl8getDepthEv", "clang::TemplateTypeParmDecl::getDepth() const"},
    {"_ZN5clang24RedeclarableTemplateDecl12getCommonPtrEv", "clang::RedeclarableTemplateDecl::getCommonPtr()"},
    {"_ZN5clang24RedeclarableTemplateDecl20getCanonicalDeclImplEv", "clang::RedeclarableTemplateDecl::getCanonicalDeclImpl()"},
    {"_ZN5clang24RedeclarableTemplateDecl26setPreviousDeclarationImplEPS0_", "clang::RedeclarableTemplateDecl::setPreviousDeclarationImpl(clang::RedeclarableTemplateDecl*)"},
    {"_ZN5clang24RedeclarableTemplateDecl20getNextRedeclarationEv", "clang::RedeclarableTemplateDecl::getNextRedeclaration()"},
    {"_ZN5clang20FunctionTemplateDecl16DeallocateCommonEPv", "clang::FunctionTemplateDecl::DeallocateCommon(void*)"},
    {"_ZN5clang20FunctionTemplateDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationENS_15DeclarationNameEPNS_21TemplateParameterListEPNS_9NamedDeclE", "clang::FunctionTemplateDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::DeclarationName, clang::TemplateParameterList*, clang::NamedDecl*)"},
    {"_ZN5clang20FunctionTemplateDecl9newCommonERNS_10ASTContextE", "clang::FunctionTemplateDecl::newCommon(clang::ASTContext&)"},
    {"_ZN5clang20FunctionTemplateDecl18findSpecializationEPKNS_16TemplateArgumentEjRPv", "clang::FunctionTemplateDecl::findSpecialization(clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN5clang17ClassTemplateDecl16DeallocateCommonEPv", "clang::ClassTemplateDecl::DeallocateCommon(void*)"},
    {"_ZN5clang17ClassTemplateDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationENS_15DeclarationNameEPNS_21TemplateParameterListEPNS_9NamedDeclEPS0_", "clang::ClassTemplateDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, clang::DeclarationName, clang::TemplateParameterList*, clang::NamedDecl*, clang::ClassTemplateDecl*)"},
    {"_ZN5clang17ClassTemplateDecl23LoadLazySpecializationsEv", "clang::ClassTemplateDecl::LoadLazySpecializations()"},
    {"_ZN5clang17ClassTemplateDecl18getSpecializationsEv", "clang::ClassTemplateDecl::getSpecializations()"},
    {"_ZN5clang17ClassTemplateDecl25getPartialSpecializationsEv", "clang::ClassTemplateDecl::getPartialSpecializations()"},
    {"_ZN5clang17ClassTemplateDecl9newCommonERNS_10ASTContextE", "clang::ClassTemplateDecl::newCommon(clang::ASTContext&)"},
    {"_ZN5clang17ClassTemplateDecl18findSpecializationEPKNS_16TemplateArgumentEjRPv", "clang::ClassTemplateDecl::findSpecialization(clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN5clang17ClassTemplateDecl17AddSpecializationEPNS_31ClassTemplateSpecializationDeclEPv", "clang::ClassTemplateDecl::AddSpecialization(clang::ClassTemplateSpecializationDecl*, void*)"},
    {"_ZN5clang17ClassTemplateDecl25findPartialSpecializationEPKNS_16TemplateArgumentEjRPv", "clang::ClassTemplateDecl::findPartialSpecialization(clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN5clang17ClassTemplateDecl24AddPartialSpecializationEPNS_38ClassTemplatePartialSpecializationDeclEPv", "clang::ClassTemplateDecl::AddPartialSpecialization(clang::ClassTemplatePartialSpecializationDecl*, void*)"},
    {"_ZN5clang17ClassTemplateDecl25getPartialSpecializationsERN4llvm15SmallVectorImplIPNS_38ClassTemplatePartialSpecializationDeclEEE", "clang::ClassTemplateDecl::getPartialSpecializations(llvm::SmallVectorImpl<clang::ClassTemplatePartialSpecializationDecl*>&)"},
    {"_ZN5clang17ClassTemplateDecl25findPartialSpecializationENS_8QualTypeE", "clang::ClassTemplateDecl::findPartialSpecialization(clang::QualType)"},
    {"_ZN5clang17ClassTemplateDecl37findPartialSpecInstantiatedFromMemberEPNS_38ClassTemplatePartialSpecializationDeclE", "clang::ClassTemplateDecl::findPartialSpecInstantiatedFromMember(clang::ClassTemplatePartialSpecializationDecl*)"},
    {"_ZN5clang17ClassTemplateDecl34getInjectedClassNameSpecializationEv", "clang::ClassTemplateDecl::getInjectedClassNameSpecialization()"},
    {"_ZN5clang20TemplateTypeParmDecl6CreateERKNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEjjPNS_14IdentifierInfoEbb", "clang::TemplateTypeParmDecl::Create(clang::ASTContext const&, clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, clang::IdentifierInfo*, bool, bool)"},
    {"_ZN5clang20TemplateTypeParmDecl6CreateERKNS_10ASTContextENS_4Decl10EmptyShellE", "clang::TemplateTypeParmDecl::Create(clang::ASTContext const&, clang::Decl::EmptyShell)"},
    {"_ZNK5clang20TemplateTypeParmDecl21getDefaultArgumentLocEv", "clang::TemplateTypeParmDecl::getDefaultArgumentLoc() const"},
    {"_ZNK5clang20TemplateTypeParmDecl8getIndexEv", "clang::TemplateTypeParmDecl::getIndex() const"},
    {"_ZN5clang23NonTypeTemplateParmDeclC1EPNS_11DeclContextENS_14SourceLocationEjjPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoEPKS6_jPS8_", "clang::NonTypeTemplateParmDecl::NonTypeTemplateParmDecl(clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::QualType const*, unsigned int, clang::TypeSourceInfo**)"},
    {"_ZN5clang23NonTypeTemplateParmDeclC2EPNS_11DeclContextENS_14SourceLocationEjjPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoEPKS6_jPS8_", "clang::NonTypeTemplateParmDecl::NonTypeTemplateParmDecl(clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::QualType const*, unsigned int, clang::TypeSourceInfo**)"},
    {"_ZN5clang23NonTypeTemplateParmDecl6CreateERKNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEjjPNS_14IdentifierInfoENS_8QualTypeEbPNS_14TypeSourceInfoE", "clang::NonTypeTemplateParmDecl::Create(clang::ASTContext const&, clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, clang::IdentifierInfo*, clang::QualType, bool, clang::TypeSourceInfo*)"},
    {"_ZN5clang23NonTypeTemplateParmDecl6CreateERKNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEjjPNS_14IdentifierInfoENS_8QualTypeEPNS_14TypeSourceInfoEPKS9_jPSB_", "clang::NonTypeTemplateParmDecl::Create(clang::ASTContext const&, clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, clang::IdentifierInfo*, clang::QualType, clang::TypeSourceInfo*, clang::QualType const*, unsigned int, clang::TypeSourceInfo**)"},
    {"_ZNK5clang23NonTypeTemplateParmDecl16getInnerLocStartEv", "clang::NonTypeTemplateParmDecl::getInnerLocStart() const"},
    {"_ZNK5clang23NonTypeTemplateParmDecl14getSourceRangeEv", "clang::NonTypeTemplateParmDecl::getSourceRange() const"},
    {"_ZNK5clang23NonTypeTemplateParmDecl21getDefaultArgumentLocEv", "clang::NonTypeTemplateParmDecl::getDefaultArgumentLoc() const"},
    {"_ZN5clang24TemplateTemplateParmDecl6CreateERKNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEjjbPNS_14IdentifierInfoEPNS_21TemplateParameterListE", "clang::TemplateTemplateParmDecl::Create(clang::ASTContext const&, clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, bool, clang::IdentifierInfo*, clang::TemplateParameterList*)"},
    {"_ZN5clang20TemplateArgumentList10CreateCopyERNS_10ASTContextEPKNS_16TemplateArgumentEj", "clang::TemplateArgumentList::CreateCopy(clang::ASTContext&, clang::TemplateArgument const*, unsigned int)"},
    {"_ZN5clang31ClassTemplateSpecializationDeclC1ERNS_10ASTContextENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_17ClassTemplateDeclEPKNS_16TemplateArgumentEjPS0_", "clang::ClassTemplateSpecializationDecl::ClassTemplateSpecializationDecl(clang::ASTContext&, clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::ClassTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN5clang31ClassTemplateSpecializationDeclC2ERNS_10ASTContextENS_4Decl4KindENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_17ClassTemplateDeclEPKNS_16TemplateArgumentEjPS0_", "clang::ClassTemplateSpecializationDecl::ClassTemplateSpecializationDecl(clang::ASTContext&, clang::Decl::Kind, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::ClassTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN5clang31ClassTemplateSpecializationDeclC1ENS_4Decl4KindE", "clang::ClassTemplateSpecializationDecl::ClassTemplateSpecializationDecl(clang::Decl::Kind)"},
    {"_ZN5clang31ClassTemplateSpecializationDeclC2ENS_4Decl4KindE", "clang::ClassTemplateSpecializationDecl::ClassTemplateSpecializationDecl(clang::Decl::Kind)"},
    {"_ZN5clang31ClassTemplateSpecializationDecl6CreateERNS_10ASTContextENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_17ClassTemplateDeclEPKNS_16TemplateArgumentEjPS0_", "clang::ClassTemplateSpecializationDecl::Create(clang::ASTContext&, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::ClassTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::ClassTemplateSpecializationDecl*)"},
    {"_ZN5clang31ClassTemplateSpecializationDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::ClassTemplateSpecializationDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZNK5clang31ClassTemplateSpecializationDecl20getNameForDiagnosticERSsRKNS_14PrintingPolicyEb", "clang::ClassTemplateSpecializationDecl::getNameForDiagnostic(std::string&, clang::PrintingPolicy const&, bool) const"},
    {"_ZNK5clang31ClassTemplateSpecializationDecl22getSpecializedTemplateEv", "clang::ClassTemplateSpecializationDecl::getSpecializedTemplate() const"},
    {"_ZN5clang38ClassTemplatePartialSpecializationDecl6CreateERNS_10ASTContextENS_11TagTypeKindEPNS_11DeclContextENS_14SourceLocationEPNS_21TemplateParameterListEPNS_17ClassTemplateDeclEPKNS_16TemplateArgumentEjRKNS_24TemplateArgumentListInfoENS_8QualTypeEPS0_j", "clang::ClassTemplatePartialSpecializationDecl::Create(clang::ASTContext&, clang::TagTypeKind, clang::DeclContext*, clang::SourceLocation, clang::TemplateParameterList*, clang::ClassTemplateDecl*, clang::TemplateArgument const*, unsigned int, clang::TemplateArgumentListInfo const&, clang::QualType, clang::ClassTemplatePartialSpecializationDecl*, unsigned int)"},
    {"_ZN5clang38ClassTemplatePartialSpecializationDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::ClassTemplatePartialSpecializationDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang18FriendTemplateDecl6CreateERNS_10ASTContextEPNS_11DeclContextENS_14SourceLocationEjPPNS_21TemplateParameterListEN4llvm12PointerUnionIPNS_9NamedDeclEPNS_14TypeSourceInfoEEES5_", "clang::FriendTemplateDecl::Create(clang::ASTContext&, clang::DeclContext*, clang::SourceLocation, unsigned int, clang::TemplateParameterList**, llvm::PointerUnion<clang::NamedDecl*, clang::TypeSourceInfo*>, clang::SourceLocation)"},
    {"_ZN5clang18FriendTemplateDecl6CreateERNS_10ASTContextENS_4Decl10EmptyShellE", "clang::FriendTemplateDecl::Create(clang::ASTContext&, clang::Decl::EmptyShell)"},
    {"_ZN5clang24RedeclarableTemplateDecl22findSpecializationImplINS_34FunctionTemplateSpecializationInfoEEEPNS0_15SpecEntryTraitsIT_E8DeclTypeERN4llvm10FoldingSetIS4_EEPKNS_16TemplateArgumentEjRPv", "clang::RedeclarableTemplateDecl::SpecEntryTraits<clang::FunctionTemplateSpecializationInfo>::DeclType* clang::RedeclarableTemplateDecl::findSpecializationImpl<clang::FunctionTemplateSpecializationInfo>(llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>&, clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN5clang24RedeclarableTemplateDecl22findSpecializationImplINS_31ClassTemplateSpecializationDeclEEEPNS0_15SpecEntryTraitsIT_E8DeclTypeERN4llvm10FoldingSetIS4_EEPKNS_16TemplateArgumentEjRPv", "clang::RedeclarableTemplateDecl::SpecEntryTraits<clang::ClassTemplateSpecializationDecl>::DeclType* clang::RedeclarableTemplateDecl::findSpecializationImpl<clang::ClassTemplateSpecializationDecl>(llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>&, clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN5clang24RedeclarableTemplateDecl22findSpecializationImplINS_38ClassTemplatePartialSpecializationDeclEEEPNS0_15SpecEntryTraitsIT_E8DeclTypeERN4llvm10FoldingSetIS4_EEPKNS_16TemplateArgumentEjRPv", "clang::RedeclarableTemplateDecl::SpecEntryTraits<clang::ClassTemplatePartialSpecializationDecl>::DeclType* clang::RedeclarableTemplateDecl::findSpecializationImpl<clang::ClassTemplatePartialSpecializationDecl>(llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>&, clang::TemplateArgument const*, unsigned int, void*&)"},
    {"_ZN4llvm15SmallVectorImplIPN5clang38ClassTemplatePartialSpecializationDeclEE6resizeEj", "llvm::SmallVectorImpl<clang::ClassTemplatePartialSpecializationDecl*>::resize(unsigned int)"},
    {"_ZN5clang24RedeclarableTemplateDeclD1Ev", "clang::RedeclarableTemplateDecl::~RedeclarableTemplateDecl()"},
    {"_ZN5clang24RedeclarableTemplateDeclD0Ev", "clang::RedeclarableTemplateDecl::~RedeclarableTemplateDecl()"},
    {"_ZNK5clang12TemplateDecl14getSourceRangeEv", "clang::TemplateDecl::getSourceRange() const"},
    {"_ZN5clang24RedeclarableTemplateDecl16getCanonicalDeclEv", "clang::RedeclarableTemplateDecl::getCanonicalDecl()"},
    {"_ZN5clang20FunctionTemplateDeclD1Ev", "clang::FunctionTemplateDecl::~FunctionTemplateDecl()"},
    {"_ZN5clang20FunctionTemplateDeclD0Ev", "clang::FunctionTemplateDecl::~FunctionTemplateDecl()"},
    {"_ZN5clang20FunctionTemplateDecl16getCanonicalDeclEv", "clang::FunctionTemplateDecl::getCanonicalDecl()"},
    {"_ZN5clang23NonTypeTemplateParmDeclD1Ev", "clang::NonTypeTemplateParmDecl::~NonTypeTemplateParmDecl()"},
    {"_ZN5clang23NonTypeTemplateParmDeclD0Ev", "clang::NonTypeTemplateParmDecl::~NonTypeTemplateParmDecl()"},
    {"_ZN5clang31ClassTemplateSpecializationDeclD1Ev", "clang::ClassTemplateSpecializationDecl::~ClassTemplateSpecializationDecl()"},
    {"_ZN5clang31ClassTemplateSpecializationDeclD0Ev", "clang::ClassTemplateSpecializationDecl::~ClassTemplateSpecializationDecl()"},
    {"_ZN5clang13CXXRecordDecl16getCanonicalDeclEv", "clang::CXXRecordDecl::getCanonicalDecl()"},
    {"_ZNK5clang31ClassTemplateSpecializationDecl16getInnerLocStartEv", "clang::ClassTemplateSpecializationDecl::getInnerLocStart() const"},
    {"_ZNK5clang13CXXRecordDecl16getCanonicalDeclEv", "clang::CXXRecordDecl::getCanonicalDecl() const"},
    {"_ZN5clang17ClassTemplateDeclD1Ev", "clang::ClassTemplateDecl::~ClassTemplateDecl()"},
    {"_ZN5clang17ClassTemplateDeclD0Ev", "clang::ClassTemplateDecl::~ClassTemplateDecl()"},
    {"_ZN5clang17ClassTemplateDecl16getCanonicalDeclEv", "clang::ClassTemplateDecl::getCanonicalDecl()"},
    {"_ZN5clang12TemplateDeclD1Ev", "clang::TemplateDecl::~TemplateDecl()"},
    {"_ZN5clang12TemplateDeclD0Ev", "clang::TemplateDecl::~TemplateDecl()"},
    {"_ZN5clang13CXXRecordDeclD1Ev", "clang::CXXRecordDecl::~CXXRecordDecl()"},
    {"_ZN5clang13CXXRecordDeclD0Ev", "clang::CXXRecordDecl::~CXXRecordDecl()"},
    {"_ZN5clang18FriendTemplateDeclD1Ev", "clang::FriendTemplateDecl::~FriendTemplateDecl()"},
    {"_ZN5clang18FriendTemplateDeclD0Ev", "clang::FriendTemplateDecl::~FriendTemplateDecl()"},
    {"_ZN5clang38ClassTemplatePartialSpecializationDeclD1Ev", "clang::ClassTemplatePartialSpecializationDecl::~ClassTemplatePartialSpecializationDecl()"},
    {"_ZN5clang38ClassTemplatePartialSpecializationDeclD0Ev", "clang::ClassTemplatePartialSpecializationDecl::~ClassTemplatePartialSpecializationDecl()"},
    {"_ZN5clang24TemplateTemplateParmDeclC2EPNS_11DeclContextENS_14SourceLocationEjjbPNS_14IdentifierInfoEPNS_21TemplateParameterListE", "clang::TemplateTemplateParmDecl::TemplateTemplateParmDecl(clang::DeclContext*, clang::SourceLocation, unsigned int, unsigned int, bool, clang::IdentifierInfo*, clang::TemplateParameterList*)"},
    {"_ZN5clang24TemplateTemplateParmDeclD1Ev", "clang::TemplateTemplateParmDecl::~TemplateTemplateParmDecl()"},
    {"_ZN5clang24TemplateTemplateParmDeclD0Ev", "clang::TemplateTemplateParmDecl::~TemplateTemplateParmDecl()"},
    {"_ZNK5clang24TemplateTemplateParmDecl14getSourceRangeEv", "clang::TemplateTemplateParmDecl::getSourceRange() const"},
    {"_ZN5clang20TemplateTypeParmDeclC2EPNS_11DeclContextENS_14SourceLocationEPNS_14IdentifierInfoEbNS_8QualTypeEb", "clang::TemplateTypeParmDecl::TemplateTypeParmDecl(clang::DeclContext*, clang::SourceLocation, clang::IdentifierInfo*, bool, clang::QualType, bool)"},
    {"_ZN5clang20TemplateTypeParmDeclD1Ev", "clang::TemplateTypeParmDecl::~TemplateTypeParmDecl()"},
    {"_ZN5clang20TemplateTypeParmDeclD0Ev", "clang::TemplateTypeParmDecl::~TemplateTypeParmDecl()"},
    {"_ZN4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEED1Ev", "llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEED0Ev", "llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEED1Ev", "llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEED0Ev", "llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEED1Ev", "llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEED0Ev", "llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN5clang15DeclarationName7compareES0_S0_", "clang::DeclarationName::compare(clang::DeclarationName, clang::DeclarationName)"},
    {"_ZNK5clang15DeclarationName11getNameKindEv", "clang::DeclarationName::getNameKind() const"},
    {"_ZNK5clang15DeclarationName15getObjCSelectorEv", "clang::DeclarationName::getObjCSelector() const"},
    {"_ZNK5clang15DeclarationName14getCXXNameTypeEv", "clang::DeclarationName::getCXXNameType() const"},
    {"_ZNK5clang15DeclarationName24getCXXOverloadedOperatorEv", "clang::DeclarationName::getCXXOverloadedOperator() const"},
    {"_ZNK5clang15DeclarationName23getCXXLiteralIdentifierEv", "clang::DeclarationName::getCXXLiteralIdentifier() const"},
    {"_ZN5clang15DeclarationNameC1ENS_8SelectorE", "clang::DeclarationName::DeclarationName(clang::Selector)"},
    {"_ZN5clang15DeclarationNameC2ENS_8SelectorE", "clang::DeclarationName::DeclarationName(clang::Selector)"},
    {"_ZNK5clang15DeclarationName15isDependentNameEv", "clang::DeclarationName::isDependentName() const"},
    {"_ZNK5clang15DeclarationName11getAsStringEv", "clang::DeclarationName::getAsString() const"},
    {"_ZNK5clang15DeclarationName9printNameERN4llvm11raw_ostreamE", "clang::DeclarationName::printName(llvm::raw_ostream&) const"},
    {"_ZNK5clang15DeclarationName20getFETokenInfoAsVoidEv", "clang::DeclarationName::getFETokenInfoAsVoid() const"},
    {"_ZN5clang15DeclarationName14setFETokenInfoEPv", "clang::DeclarationName::setFETokenInfo(void*)"},
    {"_ZN5clang15DeclarationName21getUsingDirectiveNameEv", "clang::DeclarationName::getUsingDirectiveName()"},
    {"_ZNK5clang15DeclarationName4dumpEv", "clang::DeclarationName::dump() const"},
    {"_ZN5clang20DeclarationNameTableC1ERKNS_10ASTContextE", "clang::DeclarationNameTable::DeclarationNameTable(clang::ASTContext const&)"},
    {"_ZN5clang20DeclarationNameTableC2ERKNS_10ASTContextE", "clang::DeclarationNameTable::DeclarationNameTable(clang::ASTContext const&)"},
    {"_ZN5clang20DeclarationNameTableD1Ev", "clang::DeclarationNameTable::~DeclarationNameTable()"},
    {"_ZN5clang20DeclarationNameTableD2Ev", "clang::DeclarationNameTable::~DeclarationNameTable()"},
    {"_ZN5clang20DeclarationNameTable17getCXXSpecialNameENS_15DeclarationName8NameKindENS_7CanQualINS_4TypeEEE", "clang::DeclarationNameTable::getCXXSpecialName(clang::DeclarationName::NameKind, clang::CanQual<clang::Type>)"},
    {"_ZN5clang20DeclarationNameTable18getCXXOperatorNameENS_22OverloadedOperatorKindE", "clang::DeclarationNameTable::getCXXOperatorName(clang::OverloadedOperatorKind)"},
    {"_ZN5clang20DeclarationNameTable25getCXXLiteralOperatorNameEPNS_14IdentifierInfoE", "clang::DeclarationNameTable::getCXXLiteralOperatorName(clang::IdentifierInfo*)"},
    {"_ZN4llvm12DenseMapInfoIN5clang15DeclarationNameEE12getHashValueES2_", "llvm::DenseMapInfo<clang::DeclarationName>::getHashValue(clang::DeclarationName)"},
    {"_ZN5clang18DeclarationNameLocC1ENS_15DeclarationNameE", "clang::DeclarationNameLoc::DeclarationNameLoc(clang::DeclarationName)"},
    {"_ZN5clang18DeclarationNameLocC2ENS_15DeclarationNameE", "clang::DeclarationNameLoc::DeclarationNameLoc(clang::DeclarationName)"},
    {"_ZNK5clang19DeclarationNameInfo31containsUnexpandedParameterPackEv", "clang::DeclarationNameInfo::containsUnexpandedParameterPack() const"},
    {"_ZNK5clang19DeclarationNameInfo11getAsStringEv", "clang::DeclarationNameInfo::getAsString() const"},
    {"_ZNK5clang19DeclarationNameInfo9printNameERN4llvm11raw_ostreamE", "clang::DeclarationNameInfo::printName(llvm::raw_ostream&) const"},
    {"_ZNK5clang19DeclarationNameInfo9getEndLocEv", "clang::DeclarationNameInfo::getEndLoc() const"},
    {"_ZN4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEED1Ev", "llvm::FoldingSet<clang::CXXLiteralOperatorIdName>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEED0Ev", "llvm::FoldingSet<clang::CXXLiteralOperatorIdName>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CXXLiteralOperatorIdName>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::CXXLiteralOperatorIdName>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CXXLiteralOperatorIdName>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm10FoldingSetIN5clang14CXXSpecialNameEED1Ev", "llvm::FoldingSet<clang::CXXSpecialName>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang14CXXSpecialNameEED0Ev", "llvm::FoldingSet<clang::CXXSpecialName>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang14CXXSpecialNameEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CXXSpecialName>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14CXXSpecialNameEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::CXXSpecialName>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang14CXXSpecialNameEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::CXXSpecialName>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang4Decl7dumpXMLEv", "clang::Decl::dumpXML() const"},
    {"_ZNK5clang4Decl7dumpXMLERN4llvm11raw_ostreamE", "clang::Decl::dumpXML(llvm::raw_ostream&) const"},
    {"_ZNK5clang4Expr25isKnownToHaveBooleanValueEv", "clang::Expr::isKnownToHaveBooleanValue() const"},
    {"_ZNK5clang4Expr10getExprLocEv", "clang::Expr::getExprLoc() const"},
    {"_ZN5clang28ExplicitTemplateArgumentList14initializeFromERKNS_24TemplateArgumentListInfoE", "clang::ExplicitTemplateArgumentList::initializeFrom(clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang28ExplicitTemplateArgumentList14initializeFromERKNS_24TemplateArgumentListInfoERbS4_", "clang::ExplicitTemplateArgumentList::initializeFrom(clang::TemplateArgumentListInfo const&, bool&, bool&)"},
    {"_ZNK5clang28ExplicitTemplateArgumentList8copyIntoERNS_24TemplateArgumentListInfoE", "clang::ExplicitTemplateArgumentList::copyInto(clang::TemplateArgumentListInfo&) const"},
    {"_ZN5clang28ExplicitTemplateArgumentList7sizeForEj", "clang::ExplicitTemplateArgumentList::sizeFor(unsigned int)"},
    {"_ZN5clang28ExplicitTemplateArgumentList7sizeForERKNS_24TemplateArgumentListInfoE", "clang::ExplicitTemplateArgumentList::sizeFor(clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang11DeclRefExpr17computeDependenceEv", "clang::DeclRefExpr::computeDependence()"},
    {"_Z24computeDeclRefDependencePN5clang9NamedDeclENS_8QualTypeERbS3_", "computeDeclRefDependence(clang::NamedDecl*, clang::QualType, bool&, bool&)"},
    {"_ZN5clang11DeclRefExprC1EPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclENS_14SourceLocationEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_13ExprValueKindE", "clang::DeclRefExpr::DeclRefExpr(clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::SourceLocation, clang::TemplateArgumentListInfo const*, clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang11DeclRefExprC2EPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclENS_14SourceLocationEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_13ExprValueKindE", "clang::DeclRefExpr::DeclRefExpr(clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::SourceLocation, clang::TemplateArgumentListInfo const*, clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang11DeclRefExprC1EPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_13ExprValueKindE", "clang::DeclRefExpr::DeclRefExpr(clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang11DeclRefExprC2EPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_13ExprValueKindE", "clang::DeclRefExpr::DeclRefExpr(clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::QualType, clang::ExprValueKind)"},
    {"_ZN5clang11DeclRefExpr6CreateERNS_10ASTContextEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclENS_14SourceLocationENS_8QualTypeENS_13ExprValueKindEPKNS_24TemplateArgumentListInfoE", "clang::DeclRefExpr::Create(clang::ASTContext&, clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::SourceLocation, clang::QualType, clang::ExprValueKind, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang11DeclRefExpr6CreateERNS_10ASTContextEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclERKNS_19DeclarationNameInfoENS_8QualTypeENS_13ExprValueKindEPKNS_24TemplateArgumentListInfoE", "clang::DeclRefExpr::Create(clang::ASTContext&, clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::DeclarationNameInfo const&, clang::QualType, clang::ExprValueKind, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang11DeclRefExpr11CreateEmptyERNS_10ASTContextEbbj", "clang::DeclRefExpr::CreateEmpty(clang::ASTContext&, bool, bool, unsigned int)"},
    {"_ZNK5clang11DeclRefExpr14getSourceRangeEv", "clang::DeclRefExpr::getSourceRange() const"},
    {"_ZN5clang14PredefinedExpr11ComputeNameENS0_9IdentTypeEPKNS_4DeclE", "clang::PredefinedExpr::ComputeName(clang::PredefinedExpr::IdentType, clang::Decl const*)"},
    {"_ZN5clang16APNumericStorage11setIntValueERNS_10ASTContextERKN4llvm5APIntE", "clang::APNumericStorage::setIntValue(clang::ASTContext&, llvm::APInt const&)"},
    {"_ZN5clang14IntegerLiteral6CreateERNS_10ASTContextERKN4llvm5APIntENS_8QualTypeENS_14SourceLocationE", "clang::IntegerLiteral::Create(clang::ASTContext&, llvm::APInt const&, clang::QualType, clang::SourceLocation)"},
    {"_ZN5clang14IntegerLiteral6CreateERNS_10ASTContextENS_4Stmt10EmptyShellE", "clang::IntegerLiteral::Create(clang::ASTContext&, clang::Stmt::EmptyShell)"},
    {"_ZN5clang15FloatingLiteral6CreateERNS_10ASTContextERKN4llvm7APFloatEbNS_8QualTypeENS_14SourceLocationE", "clang::FloatingLiteral::Create(clang::ASTContext&, llvm::APFloat const&, bool, clang::QualType, clang::SourceLocation)"},
    {"_ZN5clang15FloatingLiteral6CreateERNS_10ASTContextENS_4Stmt10EmptyShellE", "clang::FloatingLiteral::Create(clang::ASTContext&, clang::Stmt::EmptyShell)"},
    {"_ZNK5clang15FloatingLiteral27getValueAsApproximateDoubleEv", "clang::FloatingLiteral::getValueAsApproximateDouble() const"},
    {"_ZN5clang13StringLiteral6CreateERNS_10ASTContextEPKcjbNS_8QualTypeEPKNS_14SourceLocationEj", "clang::StringLiteral::Create(clang::ASTContext&, char const*, unsigned int, bool, clang::QualType, clang::SourceLocation const*, unsigned int)"},
    {"_ZN5clang13StringLiteral11CreateEmptyERNS_10ASTContextEj", "clang::StringLiteral::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang13StringLiteral9setStringERNS_10ASTContextEN4llvm9StringRefE", "clang::StringLiteral::setString(clang::ASTContext&, llvm::StringRef)"},
    {"_ZNK5clang13StringLiteral17getLocationOfByteEjRKNS_13SourceManagerERKNS_11LangOptionsERKNS_10TargetInfoE", "clang::StringLiteral::getLocationOfByte(unsigned int, clang::SourceManager const&, clang::LangOptions const&, clang::TargetInfo const&) const"},
    {"_ZN5clang13UnaryOperator12getOpcodeStrENS_17UnaryOperatorKindE", "clang::UnaryOperator::getOpcodeStr(clang::UnaryOperatorKind)"},
    {"_ZN5clang13UnaryOperator19getOverloadedOpcodeENS_22OverloadedOperatorKindEb", "clang::UnaryOperator::getOverloadedOpcode(clang::OverloadedOperatorKind, bool)"},
    {"_ZN5clang13UnaryOperator21getOverloadedOperatorENS_17UnaryOperatorKindE", "clang::UnaryOperator::getOverloadedOperator(clang::UnaryOperatorKind)"},
    {"_ZN5clang8CallExprC1ERNS_10ASTContextENS_4Stmt9StmtClassEPNS_4ExprEjPS6_jNS_8QualTypeENS_13ExprValueKindENS_14SourceLocationE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::Expr*, unsigned int, clang::Expr**, unsigned int, clang::QualType, clang::ExprValueKind, clang::SourceLocation)"},
    {"_ZN5clang8CallExprC2ERNS_10ASTContextENS_4Stmt9StmtClassEPNS_4ExprEjPS6_jNS_8QualTypeENS_13ExprValueKindENS_14SourceLocationE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::Expr*, unsigned int, clang::Expr**, unsigned int, clang::QualType, clang::ExprValueKind, clang::SourceLocation)"},
    {"_ZN5clang8CallExprC1ERNS_10ASTContextEPNS_4ExprEPS4_jNS_8QualTypeENS_13ExprValueKindENS_14SourceLocationE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Expr*, clang::Expr**, unsigned int, clang::QualType, clang::ExprValueKind, clang::SourceLocation)"},
    {"_ZN5clang8CallExprC2ERNS_10ASTContextEPNS_4ExprEPS4_jNS_8QualTypeENS_13ExprValueKindENS_14SourceLocationE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Expr*, clang::Expr**, unsigned int, clang::QualType, clang::ExprValueKind, clang::SourceLocation)"},
    {"_ZN5clang8CallExprC1ERNS_10ASTContextENS_4Stmt9StmtClassENS3_10EmptyShellE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::Stmt::EmptyShell)"},
    {"_ZN5clang8CallExprC2ERNS_10ASTContextENS_4Stmt9StmtClassENS3_10EmptyShellE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::Stmt::EmptyShell)"},
    {"_ZN5clang8CallExprC1ERNS_10ASTContextENS_4Stmt9StmtClassEjNS3_10EmptyShellE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, unsigned int, clang::Stmt::EmptyShell)"},
    {"_ZN5clang8CallExprC2ERNS_10ASTContextENS_4Stmt9StmtClassEjNS3_10EmptyShellE", "clang::CallExpr::CallExpr(clang::ASTContext&, clang::Stmt::StmtClass, unsigned int, clang::Stmt::EmptyShell)"},
    {"_ZN5clang8CallExpr13getCalleeDeclEv", "clang::CallExpr::getCalleeDecl()"},
    {"_ZN5clang4Expr16IgnoreParenCastsEv", "clang::Expr::IgnoreParenCasts()"},
    {"_ZN5clang8CallExpr15getDirectCalleeEv", "clang::CallExpr::getDirectCallee()"},
    {"_ZN5clang8CallExpr10setNumArgsERNS_10ASTContextEj", "clang::CallExpr::setNumArgs(clang::ASTContext&, unsigned int)"},
    {"_ZNK5clang8CallExpr13isBuiltinCallERKNS_10ASTContextE", "clang::CallExpr::isBuiltinCall(clang::ASTContext const&) const"},
    {"_ZNK5clang8CallExpr17getCallReturnTypeEv", "clang::CallExpr::getCallReturnType() const"},
    {"_ZN5clang12OffsetOfExpr6CreateERNS_10ASTContextENS_8QualTypeENS_14SourceLocationEPNS_14TypeSourceInfoEPNS0_12OffsetOfNodeEjPPNS_4ExprEjS4_", "clang::OffsetOfExpr::Create(clang::ASTContext&, clang::QualType, clang::SourceLocation, clang::TypeSourceInfo*, clang::OffsetOfExpr::OffsetOfNode*, unsigned int, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang12OffsetOfExprC1ERNS_10ASTContextENS_8QualTypeENS_14SourceLocationEPNS_14TypeSourceInfoEPNS0_12OffsetOfNodeEjPPNS_4ExprEjS4_", "clang::OffsetOfExpr::OffsetOfExpr(clang::ASTContext&, clang::QualType, clang::SourceLocation, clang::TypeSourceInfo*, clang::OffsetOfExpr::OffsetOfNode*, unsigned int, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang12OffsetOfExpr11CreateEmptyERNS_10ASTContextEjj", "clang::OffsetOfExpr::CreateEmpty(clang::ASTContext&, unsigned int, unsigned int)"},
    {"_ZN5clang12OffsetOfExprC2ERNS_10ASTContextENS_8QualTypeENS_14SourceLocationEPNS_14TypeSourceInfoEPNS0_12OffsetOfNodeEjPPNS_4ExprEjS4_", "clang::OffsetOfExpr::OffsetOfExpr(clang::ASTContext&, clang::QualType, clang::SourceLocation, clang::TypeSourceInfo*, clang::OffsetOfExpr::OffsetOfNode*, unsigned int, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZNK5clang12OffsetOfExpr12OffsetOfNode12getFieldNameEv", "clang::OffsetOfExpr::OffsetOfNode::getFieldName() const"},
    {"_ZN5clang10MemberExpr6CreateERNS_10ASTContextEPNS_4ExprEbPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9ValueDeclENS_14DeclAccessPairENS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_8QualTypeENS_13ExprValueKindENS_14ExprObjectKindE", "clang::MemberExpr::Create(clang::ASTContext&, clang::Expr*, bool, clang::NestedNameSpecifier*, clang::SourceRange, clang::ValueDecl*, clang::DeclAccessPair, clang::DeclarationNameInfo, clang::TemplateArgumentListInfo const*, clang::QualType, clang::ExprValueKind, clang::ExprObjectKind)"},
    {"_ZNK5clang8CastExpr15getCastKindNameEv", "clang::CastExpr::getCastKindName() const"},
    {"_ZN5clang8CastExpr19getSubExprAsWrittenEv", "clang::CastExpr::getSubExprAsWritten()"},
    {"_ZN5clang8CastExpr11path_bufferEv", "clang::CastExpr::path_buffer()"},
    {"_ZN5clang8CastExpr11setCastPathERKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEE", "clang::CastExpr::setCastPath(llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const&)"},
    {"_ZN5clang16ImplicitCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEENS_13ExprValueKindE", "clang::ImplicitCastExpr::Create(clang::ASTContext&, clang::QualType, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::ExprValueKind)"},
    {"_ZN5clang16ImplicitCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::ImplicitCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang14CStyleCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEPNS_14TypeSourceInfoENS_14SourceLocationESH_", "clang::CStyleCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang14CStyleCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::CStyleCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang14BinaryOperator12getOpcodeStrENS_18BinaryOperatorKindE", "clang::BinaryOperator::getOpcodeStr(clang::BinaryOperatorKind)"},
    {"_ZN5clang14BinaryOperator19getOverloadedOpcodeENS_22OverloadedOperatorKindE", "clang::BinaryOperator::getOverloadedOpcode(clang::OverloadedOperatorKind)"},
    {"_ZN5clang14BinaryOperator21getOverloadedOperatorENS_18BinaryOperatorKindE", "clang::BinaryOperator::getOverloadedOperator(clang::BinaryOperatorKind)"},
    {"_ZN5clang12InitListExprC1ERNS_10ASTContextENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::InitListExpr::InitListExpr(clang::ASTContext&, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang12InitListExprC2ERNS_10ASTContextENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::InitListExpr::InitListExpr(clang::ASTContext&, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang12InitListExpr12reserveInitsERNS_10ASTContextEj", "clang::InitListExpr::reserveInits(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang12InitListExpr11resizeInitsERNS_10ASTContextEj", "clang::InitListExpr::resizeInits(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang12InitListExpr10updateInitERNS_10ASTContextEjPNS_4ExprE", "clang::InitListExpr::updateInit(clang::ASTContext&, unsigned int, clang::Expr*)"},
    {"_ZNK5clang12InitListExpr14getSourceRangeEv", "clang::InitListExpr::getSourceRange() const"},
    {"_ZNK5clang9BlockExpr15getFunctionTypeEv", "clang::BlockExpr::getFunctionType() const"},
    {"_ZNK5clang9BlockExpr16getCaretLocationEv", "clang::BlockExpr::getCaretLocation() const"},
    {"_ZNK5clang9BlockExpr7getBodyEv", "clang::BlockExpr::getBody() const"},
    {"_ZN5clang9BlockExpr7getBodyEv", "clang::BlockExpr::getBody()"},
    {"_ZNK5clang4Expr22isUnusedResultAWarningERNS_14SourceLocationERNS_11SourceRangeES4_RNS_10ASTContextE", "clang::Expr::isUnusedResultAWarning(clang::SourceLocation&, clang::SourceRange&, clang::SourceRange&, clang::ASTContext&) const"},
    {"_ZN5clang4Expr12IgnoreParensEv", "clang::Expr::IgnoreParens()"},
    {"_ZNK5clang4Expr17isOBJCGCCandidateERNS_10ASTContextE", "clang::Expr::isOBJCGCCandidate(clang::ASTContext&) const"},
    {"_ZNK5clang4Expr21isBoundMemberFunctionERNS_10ASTContextE", "clang::Expr::isBoundMemberFunction(clang::ASTContext&) const"},
    {"_ZNK5clang4Expr8CanThrowERNS_10ASTContextE", "clang::Expr::CanThrow(clang::ASTContext&) const"},
    {"_Z16CanSubExprsThrowRN5clang10ASTContextEPKNS_4ExprE", "CanSubExprsThrow(clang::ASTContext&, clang::Expr const*)"},
    {"_Z14CanCalleeThrowPKN5clang4DeclEb", "CanCalleeThrow(clang::Decl const*, bool)"},
    {"_ZN5clang4Expr22IgnoreParenLValueCastsEv", "clang::Expr::IgnoreParenLValueCasts()"},
    {"_ZN5clang4Expr19IgnoreParenImpCastsEv", "clang::Expr::IgnoreParenImpCasts()"},
    {"_ZN5clang4Expr20IgnoreParenNoopCastsERNS_10ASTContextE", "clang::Expr::IgnoreParenNoopCasts(clang::ASTContext&)"},
    {"_ZNK5clang4Expr17isDefaultArgumentEv", "clang::Expr::isDefaultArgument() const"},
    {"_ZNK5clang4Expr17isTemporaryObjectERNS_10ASTContextEPKNS_13CXXRecordDeclE", "clang::Expr::isTemporaryObject(clang::ASTContext&, clang::CXXRecordDecl const*) const"},
    {"_ZN5clang4Expr28hasAnyTypeDependentArgumentsEPPS0_j", "clang::Expr::hasAnyTypeDependentArguments(clang::Expr**, unsigned int)"},
    {"_ZN5clang4Expr29hasAnyValueDependentArgumentsEPPS0_j", "clang::Expr::hasAnyValueDependentArguments(clang::Expr**, unsigned int)"},
    {"_ZNK5clang4Expr21isConstantInitializerERNS_10ASTContextEb", "clang::Expr::isConstantInitializer(clang::ASTContext&, bool) const"},
    {"_ZNK5clang4Expr21isNullPointerConstantERNS_10ASTContextENS0_34NullPointerConstantValueDependenceE", "clang::Expr::isNullPointerConstant(clang::ASTContext&, clang::Expr::NullPointerConstantValueDependence) const"},
    {"_ZNK5clang4Expr15getObjCPropertyEv", "clang::Expr::getObjCProperty() const"},
    {"_ZN5clang4Expr11getBitFieldEv", "clang::Expr::getBitField()"},
    {"_ZNK5clang4Expr21refersToVectorElementEv", "clang::Expr::refersToVectorElement() const"},
    {"_ZNK5clang20ExtVectorElementExpr7isArrowEv", "clang::ExtVectorElementExpr::isArrow() const"},
    {"_ZNK5clang20ExtVectorElementExpr14getNumElementsEv", "clang::ExtVectorElementExpr::getNumElements() const"},
    {"_ZNK5clang20ExtVectorElementExpr25containsDuplicateElementsEv", "clang::ExtVectorElementExpr::containsDuplicateElements() const"},
    {"_ZNK5clang20ExtVectorElementExpr23getEncodedElementAccessERN4llvm15SmallVectorImplIjEE", "clang::ExtVectorElementExpr::getEncodedElementAccess(llvm::SmallVectorImpl<unsigned int>&) const"},
    {"_ZN5clang15ObjCMessageExprC1ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationES3_bS1_NS_8SelectorES3_PNS_14ObjCMethodDeclEPPNS_4ExprEjS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::SourceLocation, bool, clang::QualType, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExprC2ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationES3_bS1_NS_8SelectorES3_PNS_14ObjCMethodDeclEPPNS_4ExprEjS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::SourceLocation, bool, clang::QualType, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExprC1ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_14TypeSourceInfoENS_8SelectorES3_PNS_14ObjCMethodDeclEPPNS_4ExprEjS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::TypeSourceInfo*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExprC2ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_14TypeSourceInfoENS_8SelectorES3_PNS_14ObjCMethodDeclEPPNS_4ExprEjS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::TypeSourceInfo*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExprC1ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_4ExprENS_8SelectorES3_PNS_14ObjCMethodDeclEPS5_jS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::Expr*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExprC2ENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_4ExprENS_8SelectorES3_PNS_14ObjCMethodDeclEPS5_jS3_", "clang::ObjCMessageExpr::ObjCMessageExpr(clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::Expr*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationES5_bS3_NS_8SelectorES5_PNS_14ObjCMethodDeclEPPNS_4ExprEjS5_", "clang::ObjCMessageExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::SourceLocation, bool, clang::QualType, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_14TypeSourceInfoENS_8SelectorES5_PNS_14ObjCMethodDeclEPPNS_4ExprEjS5_", "clang::ObjCMessageExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::TypeSourceInfo*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEPNS_4ExprENS_8SelectorES5_PNS_14ObjCMethodDeclEPS7_jS5_", "clang::ObjCMessageExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::SourceLocation, clang::Expr*, clang::Selector, clang::SourceLocation, clang::ObjCMethodDecl*, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang15ObjCMessageExpr11CreateEmptyERNS_10ASTContextEj", "clang::ObjCMessageExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZNK5clang15ObjCMessageExpr16getReceiverRangeEv", "clang::ObjCMessageExpr::getReceiverRange() const"},
    {"_ZNK5clang15ObjCMessageExpr11getSelectorEv", "clang::ObjCMessageExpr::getSelector() const"},
    {"_ZNK5clang15ObjCMessageExpr20getReceiverInterfaceEv", "clang::ObjCMessageExpr::getReceiverInterface() const"},
    {"_ZNK5clang10ChooseExpr15isConditionTrueERKNS_10ASTContextE", "clang::ChooseExpr::isConditionTrue(clang::ASTContext const&) const"},
    {"_ZN5clang17ShuffleVectorExprC1ERNS_10ASTContextEPPNS_4ExprEjNS_8QualTypeENS_14SourceLocationES7_", "clang::ShuffleVectorExpr::ShuffleVectorExpr(clang::ASTContext&, clang::Expr**, unsigned int, clang::QualType, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang17ShuffleVectorExprC2ERNS_10ASTContextEPPNS_4ExprEjNS_8QualTypeENS_14SourceLocationES7_", "clang::ShuffleVectorExpr::ShuffleVectorExpr(clang::ASTContext&, clang::Expr**, unsigned int, clang::QualType, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang17ShuffleVectorExpr8setExprsERNS_10ASTContextEPPNS_4ExprEj", "clang::ShuffleVectorExpr::setExprs(clang::ASTContext&, clang::Expr**, unsigned int)"},
    {"_ZN5clang18DesignatedInitExpr10Designator12getFieldNameEv", "clang::DesignatedInitExpr::Designator::getFieldName()"},
    {"_ZN5clang18DesignatedInitExprC1ERNS_10ASTContextENS_8QualTypeEjPKNS0_10DesignatorENS_14SourceLocationEbPPNS_4ExprEjS9_", "clang::DesignatedInitExpr::DesignatedInitExpr(clang::ASTContext&, clang::QualType, unsigned int, clang::DesignatedInitExpr::Designator const*, clang::SourceLocation, bool, clang::Expr**, unsigned int, clang::Expr*)"},
    {"_ZN5clang18DesignatedInitExprC2ERNS_10ASTContextENS_8QualTypeEjPKNS0_10DesignatorENS_14SourceLocationEbPPNS_4ExprEjS9_", "clang::DesignatedInitExpr::DesignatedInitExpr(clang::ASTContext&, clang::QualType, unsigned int, clang::DesignatedInitExpr::Designator const*, clang::SourceLocation, bool, clang::Expr**, unsigned int, clang::Expr*)"},
    {"_ZN5clang18DesignatedInitExpr6CreateERNS_10ASTContextEPNS0_10DesignatorEjPPNS_4ExprEjNS_14SourceLocationEbS6_", "clang::DesignatedInitExpr::Create(clang::ASTContext&, clang::DesignatedInitExpr::Designator*, unsigned int, clang::Expr**, unsigned int, clang::SourceLocation, bool, clang::Expr*)"},
    {"_ZN5clang18DesignatedInitExpr11CreateEmptyERNS_10ASTContextEj", "clang::DesignatedInitExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang18DesignatedInitExpr14setDesignatorsERNS_10ASTContextEPKNS0_10DesignatorEj", "clang::DesignatedInitExpr::setDesignators(clang::ASTContext&, clang::DesignatedInitExpr::Designator const*, unsigned int)"},
    {"_ZNK5clang18DesignatedInitExpr14getSourceRangeEv", "clang::DesignatedInitExpr::getSourceRange() const"},
    {"_ZN5clang18DesignatedInitExpr13getArrayIndexERKNS0_10DesignatorE", "clang::DesignatedInitExpr::getArrayIndex(clang::DesignatedInitExpr::Designator const&)"},
    {"_ZN5clang18DesignatedInitExpr18getArrayRangeStartERKNS0_10DesignatorE", "clang::DesignatedInitExpr::getArrayRangeStart(clang::DesignatedInitExpr::Designator const&)"},
    {"_ZN5clang18DesignatedInitExpr16getArrayRangeEndERKNS0_10DesignatorE", "clang::DesignatedInitExpr::getArrayRangeEnd(clang::DesignatedInitExpr::Designator const&)"},
    {"_ZN5clang18DesignatedInitExpr16ExpandDesignatorERNS_10ASTContextEjPKNS0_10DesignatorES5_", "clang::DesignatedInitExpr::ExpandDesignator(clang::ASTContext&, unsigned int, clang::DesignatedInitExpr::Designator const*, clang::DesignatedInitExpr::Designator const*)"},
    {"_ZN5clang13ParenListExprC1ERNS_10ASTContextENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::ParenListExpr::ParenListExpr(clang::ASTContext&, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang13ParenListExprC2ERNS_10ASTContextENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::ParenListExpr::ParenListExpr(clang::ASTContext&, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang12ExprIteratorixEm", "clang::ExprIterator::operator[](unsigned long)"},
    {"_ZNK5clang12ExprIteratordeEv", "clang::ExprIterator::operator*() const"},
    {"_ZNK5clang12ExprIteratorptEv", "clang::ExprIterator::operator->() const"},
    {"_ZNK5clang17ConstExprIteratorixEm", "clang::ConstExprIterator::operator[](unsigned long) const"},
    {"_ZNK5clang17ConstExprIteratordeEv", "clang::ConstExprIterator::operator*() const"},
    {"_ZNK5clang17ConstExprIteratorptEv", "clang::ConstExprIterator::operator->() const"},
    {"_ZN5clang17SizeOfAlignOfExpr8childrenEv", "clang::SizeOfAlignOfExpr::children()"},
    {"_ZN5clang15ObjCMessageExpr8childrenEv", "clang::ObjCMessageExpr::children()"},
    {"_ZN5clang16BlockDeclRefExprC1EPNS_7VarDeclENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEbb", "clang::BlockDeclRefExpr::BlockDeclRefExpr(clang::VarDecl*, clang::QualType, clang::ExprValueKind, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang16BlockDeclRefExprC2EPNS_7VarDeclENS_8QualTypeENS_13ExprValueKindENS_14SourceLocationEbb", "clang::BlockDeclRefExpr::BlockDeclRefExpr(clang::VarDecl*, clang::QualType, clang::ExprValueKind, clang::SourceLocation, bool, bool)"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6insertIPPNS_4ExprEEEPS2_RNS_10ASTContextES8_T_SB_", "clang::Stmt** clang::ASTVector<clang::Stmt*>::insert<clang::Expr**>(clang::ASTContext&, clang::Stmt**, clang::Expr**, clang::Expr**)"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6resizeERNS_10ASTContextEjRKS2_", "clang::ASTVector<clang::Stmt*>::resize(clang::ASTContext&, unsigned int, clang::Stmt* const&)"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6insertERNS_10ASTContextEPS2_mRKS2_", "clang::ASTVector<clang::Stmt*>::insert(clang::ASTContext&, clang::Stmt**, unsigned long, clang::Stmt* const&)"},
    {"_ZNK5clang4Decl7getAttrINS_20WarnUnusedResultAttrEEEPT_v", "clang::WarnUnusedResultAttr* clang::Decl::getAttr<clang::WarnUnusedResultAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_8PureAttrEEEPT_v", "clang::PureAttr* clang::Decl::getAttr<clang::PureAttr>() const"},
    {"_ZNK5clang4Decl7getAttrINS_9ConstAttrEEEPT_v", "clang::ConstAttr* clang::Decl::getAttr<clang::ConstAttr>() const"},
    {"_ZNK5clang7VarDecl16hasGlobalStorageEv", "clang::VarDecl::hasGlobalStorage() const"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6appendERNS_10ASTContextEmRKS2_", "clang::ASTVector<clang::Stmt*>::append(clang::ASTContext&, unsigned long, clang::Stmt* const&)"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6appendIPS2_EEvRNS_10ASTContextET_S8_", "void clang::ASTVector<clang::Stmt*>::append<clang::Stmt**>(clang::ASTContext&, clang::Stmt**, clang::Stmt**)"},
    {"_ZN5clang9ASTVectorIPNS_4StmtEE6appendIPPNS_4ExprEEEvRNS_10ASTContextET_SA_", "void clang::ASTVector<clang::Stmt*>::append<clang::Expr**>(clang::ASTContext&, clang::Expr**, clang::Expr**)"},
    {"_ZNK5clang16APNumericStorage11getIntValueEv", "clang::APNumericStorage::getIntValue() const"},
    {"_ZN5clang8CastExprC2ENS_4Stmt9StmtClassENS_8QualTypeENS_13ExprValueKindENS_8CastKindEPNS_4ExprEj", "clang::CastExpr::CastExpr(clang::Stmt::StmtClass, clang::QualType, clang::ExprValueKind, clang::CastKind, clang::Expr*, unsigned int)"},
    {"_ZN5clang14APFloatStorage8setValueERNS_10ASTContextERKN4llvm7APFloatE", "clang::APFloatStorage::setValue(clang::ASTContext&, llvm::APFloat const&)"},
    {"_ZN5clang14IntegerLiteralC2ERNS_10ASTContextERKN4llvm5APIntENS_8QualTypeENS_14SourceLocationE", "clang::IntegerLiteral::IntegerLiteral(clang::ASTContext&, llvm::APInt const&, clang::QualType, clang::SourceLocation)"},
    {"_ZNK5clang13CXXTypeidExpr14getTypeOperandEv", "clang::CXXTypeidExpr::getTypeOperand() const"},
    {"_ZNK5clang13CXXUuidofExpr14getTypeOperandEv", "clang::CXXUuidofExpr::getTypeOperand() const"},
    {"_ZNK5clang22CXXScalarValueInitExpr14getSourceRangeEv", "clang::CXXScalarValueInitExpr::getSourceRange() const"},
    {"_ZN5clang10CXXNewExprC1ERNS_10ASTContextEbPNS_12FunctionDeclEPPNS_4ExprEjNS_11SourceRangeES6_PNS_18CXXConstructorDeclEbS7_jS4_bNS_8QualTypeEPNS_14TypeSourceInfoENS_14SourceLocationESE_SE_SE_", "clang::CXXNewExpr::CXXNewExpr(clang::ASTContext&, bool, clang::FunctionDecl*, clang::Expr**, unsigned int, clang::SourceRange, clang::Expr*, clang::CXXConstructorDecl*, bool, clang::Expr**, unsigned int, clang::FunctionDecl*, bool, clang::QualType, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang10CXXNewExprC2ERNS_10ASTContextEbPNS_12FunctionDeclEPPNS_4ExprEjNS_11SourceRangeES6_PNS_18CXXConstructorDeclEbS7_jS4_bNS_8QualTypeEPNS_14TypeSourceInfoENS_14SourceLocationESE_SE_SE_", "clang::CXXNewExpr::CXXNewExpr(clang::ASTContext&, bool, clang::FunctionDecl*, clang::Expr**, unsigned int, clang::SourceRange, clang::Expr*, clang::CXXConstructorDecl*, bool, clang::Expr**, unsigned int, clang::FunctionDecl*, bool, clang::QualType, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang10CXXNewExpr17AllocateArgsArrayERNS_10ASTContextEbjj", "clang::CXXNewExpr::AllocateArgsArray(clang::ASTContext&, bool, unsigned int, unsigned int)"},
    {"_ZNK5clang13CXXDeleteExpr16getDestroyedTypeEv", "clang::CXXDeleteExpr::getDestroyedType() const"},
    {"_ZN5clang27PseudoDestructorTypeStorageC1EPNS_14TypeSourceInfoE", "clang::PseudoDestructorTypeStorage::PseudoDestructorTypeStorage(clang::TypeSourceInfo*)"},
    {"_ZN5clang27PseudoDestructorTypeStorageC2EPNS_14TypeSourceInfoE", "clang::PseudoDestructorTypeStorage::PseudoDestructorTypeStorage(clang::TypeSourceInfo*)"},
    {"_ZN5clang23CXXPseudoDestructorExprC1ERNS_10ASTContextEPNS_4ExprEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_14TypeSourceInfoES5_S5_NS_27PseudoDestructorTypeStorageE", "clang::CXXPseudoDestructorExpr::CXXPseudoDestructorExpr(clang::ASTContext&, clang::Expr*, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation, clang::PseudoDestructorTypeStorage)"},
    {"_ZN5clang23CXXPseudoDestructorExprC2ERNS_10ASTContextEPNS_4ExprEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_14TypeSourceInfoES5_S5_NS_27PseudoDestructorTypeStorageE", "clang::CXXPseudoDestructorExpr::CXXPseudoDestructorExpr(clang::ASTContext&, clang::Expr*, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation, clang::PseudoDestructorTypeStorage)"},
    {"_ZNK5clang23CXXPseudoDestructorExpr16getDestroyedTypeEv", "clang::CXXPseudoDestructorExpr::getDestroyedType() const"},
    {"_ZNK5clang23CXXPseudoDestructorExpr14getSourceRangeEv", "clang::CXXPseudoDestructorExpr::getSourceRange() const"},
    {"_ZN5clang20UnresolvedLookupExpr6CreateERNS_10ASTContextEPNS_13CXXRecordDeclEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEbRKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESE_", "clang::UnresolvedLookupExpr::Create(clang::ASTContext&, clang::CXXRecordDecl*, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, bool, clang::TemplateArgumentListInfo const&, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator)"},
    {"_ZN5clang20UnresolvedLookupExpr11CreateEmptyERNS_10ASTContextEbj", "clang::UnresolvedLookupExpr::CreateEmpty(clang::ASTContext&, bool, unsigned int)"},
    {"_ZN5clang12OverloadExprC1ENS_4Stmt9StmtClassERNS_10ASTContextEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESE_bb", "clang::OverloadExpr::OverloadExpr(clang::Stmt::StmtClass, clang::ASTContext&, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator, bool, bool)"},
    {"_ZN5clang12OverloadExprC2ENS_4Stmt9StmtClassERNS_10ASTContextEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESE_bb", "clang::OverloadExpr::OverloadExpr(clang::Stmt::StmtClass, clang::ASTContext&, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator, bool, bool)"},
    {"_ZN5clang12OverloadExpr17initializeResultsERNS_10ASTContextENS_21UnresolvedSetIteratorES3_", "clang::OverloadExpr::initializeResults(clang::ASTContext&, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator)"},
    {"_ZNK5clang12OverloadExpr14getNamingClassEv", "clang::OverloadExpr::getNamingClass() const"},
    {"_ZNK5clang20UnresolvedMemberExpr14getNamingClassEv", "clang::UnresolvedMemberExpr::getNamingClass() const"},
    {"_ZN5clang25DependentScopeDeclRefExprC1ENS_8QualTypeEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::DependentScopeDeclRefExpr::DependentScopeDeclRefExpr(clang::QualType, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang25DependentScopeDeclRefExprC2ENS_8QualTypeEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::DependentScopeDeclRefExpr::DependentScopeDeclRefExpr(clang::QualType, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang25DependentScopeDeclRefExpr6CreateERNS_10ASTContextEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::DependentScopeDeclRefExpr::Create(clang::ASTContext&, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang25DependentScopeDeclRefExpr11CreateEmptyERNS_10ASTContextEbj", "clang::DependentScopeDeclRefExpr::CreateEmpty(clang::ASTContext&, bool, unsigned int)"},
    {"_ZNK5clang16CXXConstructExpr14getSourceRangeEv", "clang::CXXConstructExpr::getSourceRange() const"},
    {"_ZNK5clang19CXXOperatorCallExpr14getSourceRangeEv", "clang::CXXOperatorCallExpr::getSourceRange() const"},
    {"_ZN5clang17CXXMemberCallExpr25getImplicitObjectArgumentEv", "clang::CXXMemberCallExpr::getImplicitObjectArgument()"},
    {"_ZN5clang17CXXMemberCallExpr13getRecordDeclEv", "clang::CXXMemberCallExpr::getRecordDecl()"},
    {"_ZNK5clang17CXXMemberCallExpr14getSourceRangeEv", "clang::CXXMemberCallExpr::getSourceRange() const"},
    {"_ZNK5clang16CXXNamedCastExpr11getCastNameEv", "clang::CXXNamedCastExpr::getCastName() const"},
    {"_ZN5clang17CXXStaticCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEPNS_14TypeSourceInfoENS_14SourceLocationESH_", "clang::CXXStaticCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang17CXXStaticCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::CXXStaticCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang18CXXDynamicCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEPNS_14TypeSourceInfoENS_14SourceLocationESH_", "clang::CXXDynamicCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang18CXXDynamicCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::CXXDynamicCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang22CXXReinterpretCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEEPNS_14TypeSourceInfoENS_14SourceLocationESH_", "clang::CXXReinterpretCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang22CXXReinterpretCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::CXXReinterpretCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang16CXXConstCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindEPNS_4ExprEPNS_14TypeSourceInfoENS_14SourceLocationES9_", "clang::CXXConstCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::Expr*, clang::TypeSourceInfo*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang16CXXConstCastExpr11CreateEmptyERNS_10ASTContextE", "clang::CXXConstCastExpr::CreateEmpty(clang::ASTContext&)"},
    {"_ZN5clang21CXXFunctionalCastExpr6CreateERNS_10ASTContextENS_8QualTypeENS_13ExprValueKindEPNS_14TypeSourceInfoENS_14SourceLocationENS_8CastKindEPNS_4ExprEPKN4llvm11SmallVectorIPNS_16CXXBaseSpecifierELj4EEES7_", "clang::CXXFunctionalCastExpr::Create(clang::ASTContext&, clang::QualType, clang::ExprValueKind, clang::TypeSourceInfo*, clang::SourceLocation, clang::CastKind, clang::Expr*, llvm::SmallVector<clang::CXXBaseSpecifier*, 4u> const*, clang::SourceLocation)"},
    {"_ZN5clang21CXXFunctionalCastExpr11CreateEmptyERNS_10ASTContextEj", "clang::CXXFunctionalCastExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang17CXXDefaultArgExpr6CreateERNS_10ASTContextENS_14SourceLocationEPNS_11ParmVarDeclEPNS_4ExprE", "clang::CXXDefaultArgExpr::Create(clang::ASTContext&, clang::SourceLocation, clang::ParmVarDecl*, clang::Expr*)"},
    {"_ZN5clang12CXXTemporary6CreateERNS_10ASTContextEPKNS_17CXXDestructorDeclE", "clang::CXXTemporary::Create(clang::ASTContext&, clang::CXXDestructorDecl const*)"},
    {"_ZN5clang20CXXBindTemporaryExpr6CreateERNS_10ASTContextEPNS_12CXXTemporaryEPNS_4ExprE", "clang::CXXBindTemporaryExpr::Create(clang::ASTContext&, clang::CXXTemporary*, clang::Expr*)"},
    {"_ZN5clang22CXXTemporaryObjectExprC1ERNS_10ASTContextEPNS_18CXXConstructorDeclEPNS_14TypeSourceInfoEPPNS_4ExprEjNS_11SourceRangeEb", "clang::CXXTemporaryObjectExpr::CXXTemporaryObjectExpr(clang::ASTContext&, clang::CXXConstructorDecl*, clang::TypeSourceInfo*, clang::Expr**, unsigned int, clang::SourceRange, bool)"},
    {"_ZN5clang22CXXTemporaryObjectExprC2ERNS_10ASTContextEPNS_18CXXConstructorDeclEPNS_14TypeSourceInfoEPPNS_4ExprEjNS_11SourceRangeEb", "clang::CXXTemporaryObjectExpr::CXXTemporaryObjectExpr(clang::ASTContext&, clang::CXXConstructorDecl*, clang::TypeSourceInfo*, clang::Expr**, unsigned int, clang::SourceRange, bool)"},
    {"_ZN5clang16CXXConstructExprC2ERNS_10ASTContextENS_4Stmt9StmtClassENS_8QualTypeENS_14SourceLocationEPNS_18CXXConstructorDeclEbPPNS_4ExprEjbNS0_16ConstructionKindENS_11SourceRangeE", "clang::CXXConstructExpr::CXXConstructExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::QualType, clang::SourceLocation, clang::CXXConstructorDecl*, bool, clang::Expr**, unsigned int, bool, clang::CXXConstructExpr::ConstructionKind, clang::SourceRange)"},
    {"_ZNK5clang22CXXTemporaryObjectExpr14getSourceRangeEv", "clang::CXXTemporaryObjectExpr::getSourceRange() const"},
    {"_ZN5clang16CXXConstructExpr6CreateERNS_10ASTContextENS_8QualTypeENS_14SourceLocationEPNS_18CXXConstructorDeclEbPPNS_4ExprEjbNS0_16ConstructionKindENS_11SourceRangeE", "clang::CXXConstructExpr::Create(clang::ASTContext&, clang::QualType, clang::SourceLocation, clang::CXXConstructorDecl*, bool, clang::Expr**, unsigned int, bool, clang::CXXConstructExpr::ConstructionKind, clang::SourceRange)"},
    {"_ZN5clang16CXXConstructExprC1ERNS_10ASTContextENS_4Stmt9StmtClassENS_8QualTypeENS_14SourceLocationEPNS_18CXXConstructorDeclEbPPNS_4ExprEjbNS0_16ConstructionKindENS_11SourceRangeE", "clang::CXXConstructExpr::CXXConstructExpr(clang::ASTContext&, clang::Stmt::StmtClass, clang::QualType, clang::SourceLocation, clang::CXXConstructorDecl*, bool, clang::Expr**, unsigned int, bool, clang::CXXConstructExpr::ConstructionKind, clang::SourceRange)"},
    {"_ZN5clang16ExprWithCleanupsC1ERNS_10ASTContextEPNS_4ExprEPPNS_12CXXTemporaryEj", "clang::ExprWithCleanups::ExprWithCleanups(clang::ASTContext&, clang::Expr*, clang::CXXTemporary**, unsigned int)"},
    {"_ZN5clang16ExprWithCleanupsC2ERNS_10ASTContextEPNS_4ExprEPPNS_12CXXTemporaryEj", "clang::ExprWithCleanups::ExprWithCleanups(clang::ASTContext&, clang::Expr*, clang::CXXTemporary**, unsigned int)"},
    {"_ZN5clang16ExprWithCleanups17setNumTemporariesERNS_10ASTContextEj", "clang::ExprWithCleanups::setNumTemporaries(clang::ASTContext&, unsigned int)"},
    {"_ZN5clang16ExprWithCleanups6CreateERNS_10ASTContextEPNS_4ExprEPPNS_12CXXTemporaryEj", "clang::ExprWithCleanups::Create(clang::ASTContext&, clang::Expr*, clang::CXXTemporary**, unsigned int)"},
    {"_ZN5clang26CXXUnresolvedConstructExprC1EPNS_14TypeSourceInfoENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::CXXUnresolvedConstructExpr::CXXUnresolvedConstructExpr(clang::TypeSourceInfo*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang26CXXUnresolvedConstructExprC2EPNS_14TypeSourceInfoENS_14SourceLocationEPPNS_4ExprEjS3_", "clang::CXXUnresolvedConstructExpr::CXXUnresolvedConstructExpr(clang::TypeSourceInfo*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang26CXXUnresolvedConstructExpr6CreateERNS_10ASTContextEPNS_14TypeSourceInfoENS_14SourceLocationEPPNS_4ExprEjS5_", "clang::CXXUnresolvedConstructExpr::Create(clang::ASTContext&, clang::TypeSourceInfo*, clang::SourceLocation, clang::Expr**, unsigned int, clang::SourceLocation)"},
    {"_ZN5clang26CXXUnresolvedConstructExpr11CreateEmptyERNS_10ASTContextEj", "clang::CXXUnresolvedConstructExpr::CreateEmpty(clang::ASTContext&, unsigned int)"},
    {"_ZNK5clang26CXXUnresolvedConstructExpr14getSourceRangeEv", "clang::CXXUnresolvedConstructExpr::getSourceRange() const"},
    {"_ZN5clang27CXXDependentScopeMemberExprC1ERNS_10ASTContextEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9NamedDeclENS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::CXXDependentScopeMemberExpr::CXXDependentScopeMemberExpr(clang::ASTContext&, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::NamedDecl*, clang::DeclarationNameInfo, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang27CXXDependentScopeMemberExprC2ERNS_10ASTContextEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9NamedDeclENS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::CXXDependentScopeMemberExpr::CXXDependentScopeMemberExpr(clang::ASTContext&, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::NamedDecl*, clang::DeclarationNameInfo, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang27CXXDependentScopeMemberExprC1ERNS_10ASTContextEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9NamedDeclENS_19DeclarationNameInfoE", "clang::CXXDependentScopeMemberExpr::CXXDependentScopeMemberExpr(clang::ASTContext&, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::NamedDecl*, clang::DeclarationNameInfo)"},
    {"_ZN5clang27CXXDependentScopeMemberExprC2ERNS_10ASTContextEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9NamedDeclENS_19DeclarationNameInfoE", "clang::CXXDependentScopeMemberExpr::CXXDependentScopeMemberExpr(clang::ASTContext&, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::NamedDecl*, clang::DeclarationNameInfo)"},
    {"_ZN5clang27CXXDependentScopeMemberExpr6CreateERNS_10ASTContextEPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeEPNS_9NamedDeclENS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoE", "clang::CXXDependentScopeMemberExpr::Create(clang::ASTContext&, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::NamedDecl*, clang::DeclarationNameInfo, clang::TemplateArgumentListInfo const*)"},
    {"_ZN5clang27CXXDependentScopeMemberExpr11CreateEmptyERNS_10ASTContextEbj", "clang::CXXDependentScopeMemberExpr::CreateEmpty(clang::ASTContext&, bool, unsigned int)"},
    {"_ZN5clang20UnresolvedMemberExprC1ERNS_10ASTContextEbPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESG_", "clang::UnresolvedMemberExpr::UnresolvedMemberExpr(clang::ASTContext&, bool, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator)"},
    {"_ZN5clang20UnresolvedMemberExprC2ERNS_10ASTContextEbPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESG_", "clang::UnresolvedMemberExpr::UnresolvedMemberExpr(clang::ASTContext&, bool, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator)"},
    {"_ZN5clang20UnresolvedMemberExpr6CreateERNS_10ASTContextEbPNS_4ExprENS_8QualTypeEbNS_14SourceLocationEPNS_19NestedNameSpecifierENS_11SourceRangeERKNS_19DeclarationNameInfoEPKNS_24TemplateArgumentListInfoENS_21UnresolvedSetIteratorESG_", "clang::UnresolvedMemberExpr::Create(clang::ASTContext&, bool, clang::Expr*, clang::QualType, bool, clang::SourceLocation, clang::NestedNameSpecifier*, clang::SourceRange, clang::DeclarationNameInfo const&, clang::TemplateArgumentListInfo const*, clang::UnresolvedSetIterator, clang::UnresolvedSetIterator)"},
    {"_ZN5clang20UnresolvedMemberExpr11CreateEmptyERNS_10ASTContextEbj", "clang::UnresolvedMemberExpr::CreateEmpty(clang::ASTContext&, bool, unsigned int)"},
    {"_ZN5clang32SubstNonTypeTemplateParmPackExprC1ENS_8QualTypeEPNS_23NonTypeTemplateParmDeclENS_14SourceLocationERKNS_16TemplateArgumentE", "clang::SubstNonTypeTemplateParmPackExpr::SubstNonTypeTemplateParmPackExpr(clang::QualType, clang::NonTypeTemplateParmDecl*, clang::SourceLocation, clang::TemplateArgument const&)"},
    {"_ZN5clang32SubstNonTypeTemplateParmPackExprC2ENS_8QualTypeEPNS_23NonTypeTemplateParmDeclENS_14SourceLocationERKNS_16TemplateArgumentE", "clang::SubstNonTypeTemplateParmPackExpr::SubstNonTypeTemplateParmPackExpr(clang::QualType, clang::NonTypeTemplateParmDecl*, clang::SourceLocation, clang::TemplateArgument const&)"},
    {"_ZNK5clang32SubstNonTypeTemplateParmPackExpr15getArgumentPackEv", "clang::SubstNonTypeTemplateParmPackExpr::getArgumentPack() const"},
    {"_ZNK5clang4Expr12ClassifyImplERNS_10ASTContextEPNS_14SourceLocationE", "clang::Expr::ClassifyImpl(clang::ASTContext&, clang::SourceLocation*) const"},
    {"_Z16ClassifyInternalRN5clang10ASTContextEPKNS_4ExprE", "ClassifyInternal(clang::ASTContext&, clang::Expr const*)"},
    {"_ZNK5clang4Expr14ClassifyLValueERNS_10ASTContextE", "clang::Expr::ClassifyLValue(clang::ASTContext&) const"},
    {"_ZNK5clang4Expr18isModifiableLvalueERNS_10ASTContextEPNS_14SourceLocationE", "clang::Expr::isModifiableLvalue(clang::ASTContext&, clang::SourceLocation*) const"},
    {"_Z15ClassifyUnnamedRN5clang10ASTContextENS_8QualTypeE", "ClassifyUnnamed(clang::ASTContext&, clang::QualType)"},
    {"_ZNK5clang4Expr8EvaluateERNS0_10EvalResultERKNS_10ASTContextE", "clang::Expr::Evaluate(clang::Expr::EvalResult&, clang::ASTContext const&) const"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_116IntExprEvaluatorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::IntExprEvaluator, bool>::Visit(clang::Stmt*)"},
    {"_ZNK5clang4Expr26EvaluateAsBooleanConditionERbRKNS_10ASTContextE", "clang::Expr::EvaluateAsBooleanCondition(bool&, clang::ASTContext const&) const"},
    {"_Z22HandleConversionToBoolPKN5clang4ExprERbR8EvalInfo", "HandleConversionToBool(clang::Expr const*, bool&, EvalInfo&)"},
    {"_ZNK5clang4Expr16EvaluateAsLValueERNS0_10EvalResultERKNS_10ASTContextE", "clang::Expr::EvaluateAsLValue(clang::Expr::EvalResult&, clang::ASTContext const&) const"},
    {"_ZNK5clang4Expr19EvaluateAsAnyLValueERNS0_10EvalResultERKNS_10ASTContextE", "clang::Expr::EvaluateAsAnyLValue(clang::Expr::EvalResult&, clang::ASTContext const&) const"},
    {"_ZNK5clang4Expr13isEvaluatableERKNS_10ASTContextE", "clang::Expr::isEvaluatable(clang::ASTContext const&) const"},
    {"_ZNK5clang4Expr14HasSideEffectsERKNS_10ASTContextE", "clang::Expr::HasSideEffects(clang::ASTContext const&) const"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_113HasSideEffectEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::HasSideEffect, bool>::Visit(clang::Stmt*)"},
    {"_ZNK5clang4Expr13EvaluateAsIntERKNS_10ASTContextE", "clang::Expr::EvaluateAsInt(clang::ASTContext const&) const"},
    {"_ZNK5clang4Expr10EvalResult14isGlobalLValueEv", "clang::Expr::EvalResult::isGlobalLValue() const"},
    {"_ZNK5clang4Expr21isIntegerConstantExprERN4llvm6APSIntERNS_10ASTContextEPNS_14SourceLocationEb", "clang::Expr::isIntegerConstantExpr(llvm::APSInt&, clang::ASTContext&, clang::SourceLocation*, bool) const"},
    {"_Z8CheckICEPKN5clang4ExprERNS_10ASTContextE", "CheckICE(clang::Expr const*, clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator19VisitBinaryOperatorEPKN5clang14BinaryOperatorE", "(anonymous namespace)::IntExprEvaluator::VisitBinaryOperator(clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator13VisitCallExprEPN5clang8CallExprE", "(anonymous namespace)::IntExprEvaluator::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator18VisitUnaryOperatorEPKN5clang13UnaryOperatorE", "(anonymous namespace)::IntExprEvaluator::VisitUnaryOperator(clang::UnaryOperator const*)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator7SuccessEyPKN5clang4ExprE", "(anonymous namespace)::IntExprEvaluator::Success(unsigned long long, clang::Expr const*)"},
    {"_Z15EvaluateIntegerPKN5clang4ExprERN4llvm6APSIntER8EvalInfo", "EvaluateInteger(clang::Expr const*, llvm::APSInt&, EvalInfo&)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator19CheckReferencedDeclEPKN5clang4ExprEPKNS1_4DeclE", "(anonymous namespace)::IntExprEvaluator::CheckReferencedDecl(clang::Expr const*, clang::Decl const*)"},
    {"_ZN12_GLOBAL__N_116IntExprEvaluator13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::IntExprEvaluator::VisitCastExpr(clang::CastExpr*)"},
    {"_Z18HandleIntToIntCastN5clang8QualTypeES0_RN4llvm6APSIntERKNS_10ASTContextE", "HandleIntToIntCast(clang::QualType, clang::QualType, llvm::APSInt&, clang::ASTContext const&)"},
    {"_Z20HandleFloatToIntCastN5clang8QualTypeES0_RN4llvm7APFloatERKNS_10ASTContextE", "HandleFloatToIntCast(clang::QualType, clang::QualType, llvm::APFloat&, clang::ASTContext const&)"},
    {"_Z22EvalPointerValueAsBoolRN12_GLOBAL__N_16LValueERb", "EvalPointerValueAsBool((anonymous namespace)::LValue&, bool&)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_119LValueExprEvaluatorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::LValueExprEvaluator, bool>::Visit(clang::Stmt*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_120ComplexExprEvaluatorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::ComplexExprEvaluator, bool>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_120ComplexExprEvaluator19VisitBinaryOperatorEPKN5clang14BinaryOperatorE", "(anonymous namespace)::ComplexExprEvaluator::VisitBinaryOperator(clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_120ComplexExprEvaluator18VisitUnaryOperatorEPKN5clang13UnaryOperatorE", "(anonymous namespace)::ComplexExprEvaluator::VisitUnaryOperator(clang::UnaryOperator const*)"},
    {"_ZN12_GLOBAL__N_120ComplexExprEvaluator13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::ComplexExprEvaluator::VisitCastExpr(clang::CastExpr*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_118FloatExprEvaluatorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::FloatExprEvaluator, bool>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_118FloatExprEvaluator19VisitBinaryOperatorEPKN5clang14BinaryOperatorE", "(anonymous namespace)::FloatExprEvaluator::VisitBinaryOperator(clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_118FloatExprEvaluator13VisitCallExprEPKN5clang8CallExprE", "(anonymous namespace)::FloatExprEvaluator::VisitCallExpr(clang::CallExpr const*)"},
    {"_ZN12_GLOBAL__N_118FloatExprEvaluator13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::FloatExprEvaluator::VisitCastExpr(clang::CastExpr*)"},
    {"_Z21TryEvaluateBuiltinNaNRKN5clang10ASTContextENS_8QualTypeEPKNS_4ExprEbRN4llvm7APFloatE", "TryEvaluateBuiltinNaN(clang::ASTContext const&, clang::QualType, clang::Expr const*, bool, llvm::APFloat&)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_120PointerExprEvaluatorEbE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::PointerExprEvaluator, bool>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_120PointerExprEvaluator19VisitBinaryOperatorEPKN5clang14BinaryOperatorE", "(anonymous namespace)::PointerExprEvaluator::VisitBinaryOperator(clang::BinaryOperator const*)"},
    {"_ZN12_GLOBAL__N_120PointerExprEvaluator13VisitCastExprEPN5clang8CastExprE", "(anonymous namespace)::PointerExprEvaluator::VisitCastExpr(clang::CastExpr*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_119VectorExprEvaluatorENS_7APValueEE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::VectorExprEvaluator, clang::APValue>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_119VectorExprEvaluator13GetZeroVectorEN5clang8QualTypeE", "(anonymous namespace)::VectorExprEvaluator::GetZeroVector(clang::QualType)"},
    {"_ZN12_GLOBAL__N_119VectorExprEvaluator13VisitCastExprEPKN5clang8CastExprE", "(anonymous namespace)::VectorExprEvaluator::VisitCastExpr(clang::CastExpr const*)"},
    {"_ZNK5clang10ASTContext12MakeIntValueEyNS_8QualTypeE", "clang::ASTContext::MakeIntValue(unsigned long long, clang::QualType) const"},
    {"_ZN5clang7APValueC2ERKN4llvm6APSIntE", "clang::APValue::APValue(llvm::APSInt const&)"},
    {"_ZNK4llvm5APIntcoEv", "llvm::APInt::operator~() const"},
    {"_ZNK5clang7VarDecl17setEvaluatedValueERKNS_7APValueE", "clang::VarDecl::setEvaluatedValue(clang::APValue const&) const"},
    {"_ZNK4llvm6APSIntanERKS0_", "llvm::APSInt::operator&(llvm::APSInt const&) const"},
    {"_ZNK4llvm6APSInteoERKS0_", "llvm::APSInt::operator^(llvm::APSInt const&) const"},
    {"_ZNK4llvm6APSIntorERKS0_", "llvm::APSInt::operator|(llvm::APSInt const&) const"},
    {"_ZNK4llvm6APSIntdvERKS0_", "llvm::APSInt::operator/(llvm::APSInt const&) const"},
    {"_ZNK4llvm6APSIntrmERKS0_", "llvm::APSInt::operator%(llvm::APSInt const&) const"},
    {"_ZNK4llvm6APSIntlsEj", "llvm::APSInt::operator<<(unsigned int) const"},
    {"_ZNK4llvm6APSIntrsEj", "llvm::APSInt::operator>>(unsigned int) const"},
    {"_ZN5clang7APValue13setComplexIntERKN4llvm6APSIntES4_", "clang::APValue::setComplexInt(llvm::APSInt const&, llvm::APSInt const&)"},
    {"_ZN5clang7APValue9setVectorEPKS0_j", "clang::APValue::setVector(clang::APValue const*, unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7APValueELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::APValue, false>::grow(unsigned long)"},
    {"_ZN4llvm11SmallVectorIN5clang7APValueELj4EEC2EjRKS2_", "llvm::SmallVector<clang::APValue, 4u>::SmallVector(unsigned int, clang::APValue const&)"},
    {"_ZN4llvm6APSIntrSEj", "llvm::APSInt::operator>>=(unsigned int)"},
    {"_ZN5clang26InheritanceHierarchyWriter9WriteNodeENS_8QualTypeEb", "clang::InheritanceHierarchyWriter::WriteNode(clang::QualType, bool)"},
    {"_ZN5clang26InheritanceHierarchyWriter18WriteNodeReferenceENS_8QualTypeEb", "clang::InheritanceHierarchyWriter::WriteNodeReference(clang::QualType, bool)"},
    {"_ZNK5clang13CXXRecordDecl15viewInheritanceERNS_10ASTContextE", "clang::CXXRecordDecl::viewInheritance(clang::ASTContext&) const"},
    {"_ZN5clang26InheritanceHierarchyWriter10WriteGraphENS_8QualTypeE", "clang::InheritanceHierarchyWriter::WriteGraph(clang::QualType)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_iESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS4_ERKS4_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, int>, std::_Select1st<std::pair<clang::QualType const, int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<clang::QualType const, int> >, std::pair<clang::QualType const, int> const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_iESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE16_M_insert_uniqueERKS4_", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, int>, std::_Select1st<std::pair<clang::QualType const, int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, int> > >::_M_insert_unique(std::pair<clang::QualType const, int> const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeES1_St9_IdentityIS1_ENS0_16QualTypeOrderingESaIS1_EE16_M_insert_uniqueERKS1_", "std::_Rb_tree<clang::QualType, clang::QualType, std::_Identity<clang::QualType>, clang::QualTypeOrdering, std::allocator<clang::QualType> >::_M_insert_unique(clang::QualType const&)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeESt4pairIKS1_iESt10_Select1stIS4_ENS0_16QualTypeOrderingESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<clang::QualType, std::pair<clang::QualType const, int>, std::_Select1st<std::pair<clang::QualType const, int> >, clang::QualTypeOrdering, std::allocator<std::pair<clang::QualType const, int> > >::_M_erase(std::_Rb_tree_node<std::pair<clang::QualType const, int> >*)"},
    {"_ZNSt8_Rb_treeIN5clang8QualTypeES1_St9_IdentityIS1_ENS0_16QualTypeOrderingESaIS1_EE8_M_eraseEPSt13_Rb_tree_nodeIS1_E", "std::_Rb_tree<clang::QualType, clang::QualType, std::_Identity<clang::QualType>, clang::QualTypeOrdering, std::allocator<clang::QualType> >::_M_erase(std::_Rb_tree_node<clang::QualType>*)"},
    {"_ZN5clang19CreateItaniumCXXABIERNS_10ASTContextE", "clang::CreateItaniumCXXABI(clang::ASTContext&)"},
    {"_ZN5clang15CreateARMCXXABIERNS_10ASTContextE", "clang::CreateARMCXXABI(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_19ARMCXXABID1Ev", "(anonymous namespace)::ARMCXXABI::~ARMCXXABI()"},
    {"_ZN12_GLOBAL__N_19ARMCXXABID0Ev", "(anonymous namespace)::ARMCXXABI::~ARMCXXABI()"},
    {"_ZNK12_GLOBAL__N_113ItaniumCXXABI20getMemberPointerSizeEPKN5clang17MemberPointerTypeE", "(anonymous namespace)::ItaniumCXXABI::getMemberPointerSize(clang::MemberPointerType const*) const"},
    {"_ZNK12_GLOBAL__N_113ItaniumCXXABI24getDefaultMethodCallConvEv", "(anonymous namespace)::ItaniumCXXABI::getDefaultMethodCallConv() const"},
    {"_ZNK12_GLOBAL__N_113ItaniumCXXABI13isNearlyEmptyEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::ItaniumCXXABI::isNearlyEmpty(clang::CXXRecordDecl const*) const"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABID1Ev", "(anonymous namespace)::ItaniumCXXABI::~ItaniumCXXABI()"},
    {"_ZN12_GLOBAL__N_113ItaniumCXXABID0Ev", "(anonymous namespace)::ItaniumCXXABI::~ItaniumCXXABI()"},
    {"_ZN5clang26createItaniumMangleContextERNS_10ASTContextERNS_10DiagnosticE", "clang::createItaniumMangleContext(clang::ASTContext&, clang::Diagnostic&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContextD1Ev", "(anonymous namespace)::ItaniumMangleContext::~ItaniumMangleContext()"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContextD0Ev", "(anonymous namespace)::ItaniumMangleContext::~ItaniumMangleContext()"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext16startNewFunctionEv", "(anonymous namespace)::ItaniumMangleContext::startNewFunction()"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext20shouldMangleDeclNameEPKN5clang9NamedDeclE", "(anonymous namespace)::ItaniumMangleContext::shouldMangleDeclName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext10mangleNameEPKN5clang9NamedDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleName(clang::NamedDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext11mangleThunkEPKN5clang13CXXMethodDeclERKNS1_9ThunkInfoERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleThunk(clang::CXXMethodDecl const*, clang::ThunkInfo const&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext18mangleCXXDtorThunkEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERKNS1_14ThisAdjustmentERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXDtorThunk(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::ThisAdjustment const&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext24mangleReferenceTemporaryEPKN5clang7VarDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleReferenceTemporary(clang::VarDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext15mangleCXXVTableEPKN5clang13CXXRecordDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXVTable(clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext12mangleCXXVTTEPKN5clang13CXXRecordDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXVTT(clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext19mangleCXXCtorVTableEPKN5clang13CXXRecordDeclExS4_RN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXCtorVTable(clang::CXXRecordDecl const*, long long, clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext13mangleCXXRTTIEN5clang8QualTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXRTTI(clang::QualType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext17mangleCXXRTTINameEN5clang8QualTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXRTTIName(clang::QualType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext13mangleCXXCtorEPKN5clang18CXXConstructorDeclENS1_11CXXCtorTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXCtor(clang::CXXConstructorDecl const*, clang::CXXCtorType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext13mangleCXXDtorEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleCXXDtor(clang::CXXDestructorDecl const*, clang::CXXDtorType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_120ItaniumMangleContext26mangleItaniumGuardVariableEPKN5clang7VarDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::ItaniumMangleContext::mangleItaniumGuardVariable(clang::VarDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler10mangleNameEPKN5clang9NamedDeclE", "(anonymous namespace)::CXXNameMangler::mangleName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_117GetLocalClassDeclEPKN5clang9NamedDeclE", "(anonymous namespace)::GetLocalClassDecl(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler15mangleLocalNameEPKN5clang9NamedDeclE", "(anonymous namespace)::CXXNameMangler::mangleLocalName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler26mangleUnscopedTemplateNameEPKN5clang12TemplateDeclE", "(anonymous namespace)::CXXNameMangler::mangleUnscopedTemplateName(clang::TemplateDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler16mangleNestedNameEPKN5clang9NamedDeclEPKNS1_11DeclContextEb", "(anonymous namespace)::CXXNameMangler::mangleNestedName(clang::NamedDecl const*, clang::DeclContext const*, bool)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler16mangleQualifiersEN5clang10QualifiersE", "(anonymous namespace)::CXXNameMangler::mangleQualifiers(clang::Qualifiers)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler20mangleTemplatePrefixEPKN5clang12TemplateDeclE", "(anonymous namespace)::CXXNameMangler::mangleTemplatePrefix(clang::TemplateDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler12manglePrefixEPKN5clang11DeclContextEb", "(anonymous namespace)::CXXNameMangler::manglePrefix(clang::DeclContext const*, bool)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler21mangleUnqualifiedNameEPKN5clang9NamedDeclENS1_15DeclarationNameEj", "(anonymous namespace)::CXXNameMangler::mangleUnqualifiedName(clang::NamedDecl const*, clang::DeclarationName, unsigned int)"},
    {"_Z24FindFirstNamedDataMemberPKN5clang10RecordDeclE", "FindFirstNamedDataMember(clang::RecordDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler10mangleTypeEN5clang8QualTypeE", "(anonymous namespace)::CXXNameMangler::mangleType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler18mangleOperatorNameEN5clang22OverloadedOperatorKindEj", "(anonymous namespace)::CXXNameMangler::mangleOperatorName(clang::OverloadedOperatorKind, unsigned int)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler18mangleSubstitutionEN5clang8QualTypeE", "(anonymous namespace)::CXXNameMangler::mangleSubstitution(clang::QualType)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler10mangleTypeEPKN5clang10VectorTypeE", "(anonymous namespace)::CXXNameMangler::mangleType(clang::VectorType const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler15addSubstitutionEN5clang8QualTypeE", "(anonymous namespace)::CXXNameMangler::addSubstitution(clang::QualType)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler15addSubstitutionEPKN5clang9NamedDeclE", "(anonymous namespace)::CXXNameMangler::addSubstitution(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler20mangleTemplatePrefixEN5clang12TemplateNameE", "(anonymous namespace)::CXXNameMangler::mangleTemplatePrefix(clang::TemplateName)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler18mangleTemplateArgsEN5clang12TemplateNameEPKNS1_16TemplateArgumentEj", "(anonymous namespace)::CXXNameMangler::mangleTemplateArgs(clang::TemplateName, clang::TemplateArgument const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler17mangleTemplateArgEPKN5clang9NamedDeclERKNS1_16TemplateArgumentE", "(anonymous namespace)::CXXNameMangler::mangleTemplateArg(clang::NamedDecl const*, clang::TemplateArgument const&)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler10mangleTypeEN5clang12TemplateNameE", "(anonymous namespace)::CXXNameMangler::mangleType(clang::TemplateName)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler16mangleExpressionEPKN5clang4ExprEj", "(anonymous namespace)::CXXNameMangler::mangleExpression(clang::Expr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler20mangleIntegerLiteralEN5clang8QualTypeERKN4llvm6APSIntE", "(anonymous namespace)::CXXNameMangler::mangleIntegerLiteral(clang::QualType, llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler6mangleEPKN5clang9NamedDeclEN4llvm9StringRefE", "(anonymous namespace)::CXXNameMangler::mangle(clang::NamedDecl const*, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler22mangleFunctionEncodingEPKN5clang12FunctionDeclE", "(anonymous namespace)::CXXNameMangler::mangleFunctionEncoding(clang::FunctionDecl const*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler22mangleBareFunctionTypeEPKN5clang12FunctionTypeEb", "(anonymous namespace)::CXXNameMangler::mangleBareFunctionType(clang::FunctionType const*, bool)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler12mangleNumberERKN4llvm6APSIntE", "(anonymous namespace)::CXXNameMangler::mangleNumber(llvm::APSInt const&)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler11mangleFloatERKN4llvm7APFloatE", "(anonymous namespace)::CXXNameMangler::mangleFloat(llvm::APFloat const&)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler21mangleUnresolvedScopeEPN5clang19NestedNameSpecifierE", "(anonymous namespace)::CXXNameMangler::mangleUnresolvedScope(clang::NestedNameSpecifier*)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler18mangleSubstitutionEN5clang12TemplateNameE", "(anonymous namespace)::CXXNameMangler::mangleSubstitution(clang::TemplateName)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler15addSubstitutionEN5clang12TemplateNameE", "(anonymous namespace)::CXXNameMangler::addSubstitution(clang::TemplateName)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler18mangleSubstitutionEm", "(anonymous namespace)::CXXNameMangler::mangleSubstitution(unsigned long)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler26mangleStandardSubstitutionEPKN5clang9NamedDeclE", "(anonymous namespace)::CXXNameMangler::mangleStandardSubstitution(clang::NamedDecl const*)"},
    {"_Z5isStdPKN5clang13NamespaceDeclE", "isStd(clang::NamespaceDecl const*)"},
    {"_Z20isCharSpecializationN5clang8QualTypeEPKc", "isCharSpecialization(clang::QualType, char const*)"},
    {"_Z26isStreamCharSpecializationILm14EEbPKN5clang31ClassTemplateSpecializationDeclERAT__Kc", "bool isStreamCharSpecialization<14ul>(clang::ClassTemplateSpecializationDecl const*, char const (&) [14ul])"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler10mangleNameEPKN5clang12TemplateDeclEPKNS1_16TemplateArgumentEj", "(anonymous namespace)::CXXNameMangler::mangleName(clang::TemplateDecl const*, clang::TemplateArgument const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114CXXNameMangler16mangleCallOffsetExx", "(anonymous namespace)::CXXNameMangler::mangleCallOffset(long long, long long)"},
    {"_ZN4llvm8DenseMapImjNS_12DenseMapInfoImEENS1_IjEEE16InsertIntoBucketERKmRKjPSt4pairImjE", "llvm::DenseMap<unsigned long, unsigned int, llvm::DenseMapInfo<unsigned long>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(unsigned long const&, unsigned int const&, std::pair<unsigned long, unsigned int>*)"},
    {"_ZN4llvm8DenseMapImjNS_12DenseMapInfoImEENS1_IjEEE4growEj", "llvm::DenseMap<unsigned long, unsigned int, llvm::DenseMapInfo<unsigned long>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNK5clang4Decl7getAttrINS_12AsmLabelAttrEEEPT_v", "clang::AsmLabelAttr* clang::Decl::getAttr<clang::AsmLabelAttr>() const"},
    {"_ZN4llvm8DenseMapIPKN5clang7TagDeclEyNS_12DenseMapInfoIS4_EENS5_IyEEE6insertERKSt4pairIS4_yE", "llvm::DenseMap<clang::TagDecl const*, unsigned long long, llvm::DenseMapInfo<clang::TagDecl const*>, llvm::DenseMapInfo<unsigned long long> >::insert(std::pair<clang::TagDecl const*, unsigned long long> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang7TagDeclEyNS_12DenseMapInfoIS4_EENS5_IyEEE16InsertIntoBucketERKS4_RKyPSt4pairIS4_yE", "llvm::DenseMap<clang::TagDecl const*, unsigned long long, llvm::DenseMapInfo<clang::TagDecl const*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(clang::TagDecl const* const&, unsigned long long const&, std::pair<clang::TagDecl const*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang7TagDeclEyNS_12DenseMapInfoIS4_EENS5_IyEEE4growEj", "llvm::DenseMap<clang::TagDecl const*, unsigned long long, llvm::DenseMapInfo<clang::TagDecl const*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang9NamedDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::NamedDecl const*, unsigned int, llvm::DenseMapInfo<clang::NamedDecl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::NamedDecl const* const&, unsigned int const&, std::pair<clang::NamedDecl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9NamedDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::NamedDecl const*, unsigned int, llvm::DenseMapInfo<clang::NamedDecl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang13MangleContext16startNewFunctionEv", "clang::MangleContext::startNewFunction()"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16shrink_and_clearEv", "llvm::DenseMap<clang::BlockDecl const*, unsigned int, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN5clang13MangleContextD1Ev", "clang::MangleContext::~MangleContext()"},
    {"_ZN5clang13MangleContextD0Ev", "clang::MangleContext::~MangleContext()"},
    {"_ZN5clang13MangleContext26mangleItaniumGuardVariableEPKNS_7VarDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleItaniumGuardVariable(clang::VarDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext17mangleGlobalBlockEPKNS_9BlockDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleGlobalBlock(clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext15mangleCtorBlockEPKNS_18CXXConstructorDeclENS_11CXXCtorTypeEPKNS_9BlockDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleCtorBlock(clang::CXXConstructorDecl const*, clang::CXXCtorType, clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_119mangleFunctionBlockERN5clang13MangleContextEN4llvm9StringRefEPKNS0_9BlockDeclERNS3_15SmallVectorImplIcEE", "(anonymous namespace)::mangleFunctionBlock(clang::MangleContext&, llvm::StringRef, clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext15mangleDtorBlockEPKNS_17CXXDestructorDeclENS_11CXXDtorTypeEPKNS_9BlockDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleDtorBlock(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext11mangleBlockEPKNS_11DeclContextEPKNS_9BlockDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleBlock(clang::DeclContext const*, clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext20mangleObjCMethodNameEPKNS_14ObjCMethodDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleObjCMethodName(clang::ObjCMethodDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang13MangleContext11mangleBlockEPKNS_9BlockDeclERN4llvm15SmallVectorImplIcEE", "clang::MangleContext::mangleBlock(clang::BlockDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE6insertERKSt4pairIS4_jE", "llvm::DenseMap<clang::BlockDecl const*, unsigned int, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<clang::BlockDecl const*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<clang::BlockDecl const*, unsigned int, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(clang::BlockDecl const* const&, unsigned int const&, std::pair<clang::BlockDecl const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9BlockDeclEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<clang::BlockDecl const*, unsigned int, llvm::DenseMapInfo<clang::BlockDecl const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN5clang21CreateMicrosoftCXXABIERNS_10ASTContextE", "clang::CreateMicrosoftCXXABI(clang::ASTContext&)"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABID1Ev", "(anonymous namespace)::MicrosoftCXXABI::~MicrosoftCXXABI()"},
    {"_ZN12_GLOBAL__N_115MicrosoftCXXABID0Ev", "(anonymous namespace)::MicrosoftCXXABI::~MicrosoftCXXABI()"},
    {"_ZNK12_GLOBAL__N_115MicrosoftCXXABI20getMemberPointerSizeEPKN5clang17MemberPointerTypeE", "(anonymous namespace)::MicrosoftCXXABI::getMemberPointerSize(clang::MemberPointerType const*) const"},
    {"_ZNK12_GLOBAL__N_115MicrosoftCXXABI24getDefaultMethodCallConvEv", "(anonymous namespace)::MicrosoftCXXABI::getDefaultMethodCallConv() const"},
    {"_ZNK12_GLOBAL__N_115MicrosoftCXXABI13isNearlyEmptyEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::MicrosoftCXXABI::isNearlyEmpty(clang::CXXRecordDecl const*) const"},
    {"_ZN5clang28createMicrosoftMangleContextERNS_10ASTContextERNS_10DiagnosticE", "clang::createMicrosoftMangleContext(clang::ASTContext&, clang::Diagnostic&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContextD1Ev", "(anonymous namespace)::MicrosoftMangleContext::~MicrosoftMangleContext()"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContextD0Ev", "(anonymous namespace)::MicrosoftMangleContext::~MicrosoftMangleContext()"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext20shouldMangleDeclNameEPKN5clang9NamedDeclE", "(anonymous namespace)::MicrosoftMangleContext::shouldMangleDeclName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext10mangleNameEPKN5clang9NamedDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleName(clang::NamedDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext11mangleThunkEPKN5clang13CXXMethodDeclERKNS1_9ThunkInfoERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleThunk(clang::CXXMethodDecl const*, clang::ThunkInfo const&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext18mangleCXXDtorThunkEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERKNS1_14ThisAdjustmentERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXDtorThunk(clang::CXXDestructorDecl const*, clang::CXXDtorType, clang::ThisAdjustment const&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext24mangleReferenceTemporaryEPKN5clang7VarDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleReferenceTemporary(clang::VarDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext15mangleCXXVTableEPKN5clang13CXXRecordDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXVTable(clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext12mangleCXXVTTEPKN5clang13CXXRecordDeclERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXVTT(clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext19mangleCXXCtorVTableEPKN5clang13CXXRecordDeclExS4_RN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXCtorVTable(clang::CXXRecordDecl const*, long long, clang::CXXRecordDecl const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext13mangleCXXRTTIEN5clang8QualTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXRTTI(clang::QualType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext17mangleCXXRTTINameEN5clang8QualTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXRTTIName(clang::QualType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext13mangleCXXCtorEPKN5clang18CXXConstructorDeclENS1_11CXXCtorTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXCtor(clang::CXXConstructorDecl const*, clang::CXXCtorType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_122MicrosoftMangleContext13mangleCXXDtorEPKN5clang17CXXDestructorDeclENS1_11CXXDtorTypeERN4llvm15SmallVectorImplIcEE", "(anonymous namespace)::MicrosoftMangleContext::mangleCXXDtor(clang::CXXDestructorDecl const*, clang::CXXDtorType, llvm::SmallVectorImpl<char>&)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler10mangleNameEPKN5clang9NamedDeclE", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler10mangleTypeEN5clang8QualTypeE", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler16mangleQualifiersEN5clang10QualifiersEb", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleQualifiers(clang::Qualifiers, bool)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler21mangleExtraDimensionsEN5clang8QualTypeE", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleExtraDimensions(clang::QualType)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler12mangleNumberEx", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleNumber(long long)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler10mangleTypeEPKN5clang7TagTypeE", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleType(clang::TagType const*)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler10mangleTypeEPKN5clang12FunctionTypeEPKNS1_12FunctionDeclEbb", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleType(clang::FunctionType const*, clang::FunctionDecl const*, bool, bool)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler21mangleUnqualifiedNameEPKN5clang9NamedDeclE", "(anonymous namespace)::MicrosoftCXXNameMangler::mangleUnqualifiedName(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_123MicrosoftCXXNameMangler13manglePostfixEPKN5clang11DeclContextEb", "(anonymous namespace)::MicrosoftCXXNameMangler::manglePostfix(clang::DeclContext const*, bool)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_5APIntELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::APInt, false>::grow(unsigned long)"},
    {"_ZN5clang19NestedNameSpecifier12FindOrInsertERKNS_10ASTContextERKS0_", "clang::NestedNameSpecifier::FindOrInsert(clang::ASTContext const&, clang::NestedNameSpecifier const&)"},
    {"_ZN5clang19NestedNameSpecifier6CreateERKNS_10ASTContextEPS0_PNS_14IdentifierInfoE", "clang::NestedNameSpecifier::Create(clang::ASTContext const&, clang::NestedNameSpecifier*, clang::IdentifierInfo*)"},
    {"_ZN5clang19NestedNameSpecifier6CreateERKNS_10ASTContextEPS0_PNS_13NamespaceDeclE", "clang::NestedNameSpecifier::Create(clang::ASTContext const&, clang::NestedNameSpecifier*, clang::NamespaceDecl*)"},
    {"_ZN5clang19NestedNameSpecifier6CreateERKNS_10ASTContextEPS0_bPKNS_4TypeE", "clang::NestedNameSpecifier::Create(clang::ASTContext const&, clang::NestedNameSpecifier*, bool, clang::Type const*)"},
    {"_ZN5clang19NestedNameSpecifier6CreateERKNS_10ASTContextEPNS_14IdentifierInfoE", "clang::NestedNameSpecifier::Create(clang::ASTContext const&, clang::IdentifierInfo*)"},
    {"_ZN5clang19NestedNameSpecifier15GlobalSpecifierERKNS_10ASTContextE", "clang::NestedNameSpecifier::GlobalSpecifier(clang::ASTContext const&)"},
    {"_ZNK5clang19NestedNameSpecifier11isDependentEv", "clang::NestedNameSpecifier::isDependent() const"},
    {"_ZNK5clang19NestedNameSpecifier31containsUnexpandedParameterPackEv", "clang::NestedNameSpecifier::containsUnexpandedParameterPack() const"},
    {"_ZNK5clang19NestedNameSpecifier5printERN4llvm11raw_ostreamERKNS_14PrintingPolicyE", "clang::NestedNameSpecifier::print(llvm::raw_ostream&, clang::PrintingPolicy const&) const"},
    {"_ZN5clang19NestedNameSpecifier4dumpERKNS_11LangOptionsE", "clang::NestedNameSpecifier::dump(clang::LangOptions const&)"},
    {"_ZN5clang9ParentMapC1EPNS_4StmtE", "clang::ParentMap::ParentMap(clang::Stmt*)"},
    {"_ZN5clang9ParentMapC2EPNS_4StmtE", "clang::ParentMap::ParentMap(clang::Stmt*)"},
    {"_Z14BuildParentMapRN4llvm8DenseMapIPN5clang4StmtES3_NS_12DenseMapInfoIS3_EES5_EES3_", "BuildParentMap(llvm::DenseMap<clang::Stmt*, clang::Stmt*, llvm::DenseMapInfo<clang::Stmt*>, llvm::DenseMapInfo<clang::Stmt*> >&, clang::Stmt*)"},
    {"_ZN5clang9ParentMapD1Ev", "clang::ParentMap::~ParentMap()"},
    {"_ZN5clang9ParentMapD2Ev", "clang::ParentMap::~ParentMap()"},
    {"_ZN5clang9ParentMap7addStmtEPNS_4StmtE", "clang::ParentMap::addStmt(clang::Stmt*)"},
    {"_ZNK5clang9ParentMap9getParentEPNS_4StmtE", "clang::ParentMap::getParent(clang::Stmt*) const"},
    {"_ZNK5clang9ParentMap21getParentIgnoreParensEPNS_4StmtE", "clang::ParentMap::getParentIgnoreParens(clang::Stmt*) const"},
    {"_ZNK5clang9ParentMap14isConsumedExprEPNS_4ExprE", "clang::ParentMap::isConsumedExpr(clang::Expr*) const"},
    {"_ZN5clang15ASTRecordLayout7DestroyERNS_10ASTContextE", "clang::ASTRecordLayout::Destroy(clang::ASTContext&)"},
    {"_ZN5clang15ASTRecordLayoutC1ERKNS_10ASTContextENS_9CharUnitsEjjPKyj", "clang::ASTRecordLayout::ASTRecordLayout(clang::ASTContext const&, clang::CharUnits, unsigned int, unsigned int, unsigned long long const*, unsigned int)"},
    {"_ZN5clang15ASTRecordLayoutC2ERKNS_10ASTContextENS_9CharUnitsEjjPKyj", "clang::ASTRecordLayout::ASTRecordLayout(clang::ASTContext const&, clang::CharUnits, unsigned int, unsigned int, unsigned long long const*, unsigned int)"},
    {"_ZN5clang15ASTRecordLayoutC1ERKNS_10ASTContextENS_9CharUnitsEjyPKyjS4_S4_S4_PKNS_13CXXRecordDeclEbRKN4llvm8DenseMapIS9_S4_NSA_12DenseMapInfoIS9_EENSC_IS4_EEEESH_", "clang::ASTRecordLayout::ASTRecordLayout(clang::ASTContext const&, clang::CharUnits, unsigned int, unsigned long long, unsigned long long const*, unsigned int, clang::CharUnits, clang::CharUnits, clang::CharUnits, clang::CXXRecordDecl const*, bool, llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> > const&, llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> > const&)"},
    {"_ZN5clang15ASTRecordLayoutC2ERKNS_10ASTContextENS_9CharUnitsEjyPKyjS4_S4_S4_PKNS_13CXXRecordDeclEbRKN4llvm8DenseMapIS9_S4_NSA_12DenseMapInfoIS9_EENSC_IS4_EEEESH_", "clang::ASTRecordLayout::ASTRecordLayout(clang::ASTContext const&, clang::CharUnits, unsigned int, unsigned long long, unsigned long long const*, unsigned int, clang::CharUnits, clang::CharUnits, clang::CharUnits, clang::CXXRecordDecl const*, bool, llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> > const&, llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> > const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS1_9CharUnitsENS_12DenseMapInfoIS4_EENS6_IS5_EEE8CopyFromERKS9_", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> >::CopyFrom(llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> > const&)"},
    {"_ZNK5clang10ASTContext18getASTRecordLayoutEPKNS_10RecordDeclE", "clang::ASTContext::getASTRecordLayout(clang::RecordDecl const*) const"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilderD1Ev", "(anonymous namespace)::RecordLayoutBuilder::~RecordLayoutBuilder()"},
    {"_ZNK5clang10ASTContext16DumpRecordLayoutEPKNS_10RecordDeclERN4llvm11raw_ostreamE", "clang::ASTContext::DumpRecordLayout(clang::RecordDecl const*, llvm::raw_ostream&) const"},
    {"_ZN5clang10ASTContext14getKeyFunctionEPKNS_13CXXRecordDeclE", "clang::ASTContext::getKeyFunction(clang::CXXRecordDecl const*)"},
    {"_ZNK5clang10ASTContext13getObjCLayoutEPKNS_17ObjCInterfaceDeclEPKNS_22ObjCImplementationDeclE", "clang::ASTContext::getObjCLayout(clang::ObjCInterfaceDecl const*, clang::ObjCImplementationDecl const*) const"},
    {"_Z19DumpCXXRecordLayoutRN4llvm11raw_ostreamEPKN5clang13CXXRecordDeclERKNS2_10ASTContextENS2_9CharUnitsEjPKcb", "DumpCXXRecordLayout(llvm::raw_ostream&, clang::CXXRecordDecl const*, clang::ASTContext const&, clang::CharUnits, unsigned int, char const*, bool)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder16InitializeLayoutEPKN5clang4DeclE", "(anonymous namespace)::RecordLayoutBuilder::InitializeLayout(clang::Decl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder11LayoutFieldEPKN5clang9FieldDeclE", "(anonymous namespace)::RecordLayoutBuilder::LayoutField(clang::FieldDecl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder12FinishLayoutEPKN5clang9NamedDeclE", "(anonymous namespace)::RecordLayoutBuilder::FinishLayout(clang::NamedDecl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder17CheckFieldPaddingEyyyjbPKN5clang9FieldDeclE", "(anonymous namespace)::RecordLayoutBuilder::CheckFieldPadding(unsigned long long, unsigned long long, unsigned long long, unsigned int, bool, clang::FieldDecl const*)"},
    {"_ZNK12_GLOBAL__N_117EmptySubobjectMap30CanPlaceFieldSubobjectAtOffsetEPKN5clang9FieldDeclENS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(clang::FieldDecl const*, clang::CharUnits) const"},
    {"_ZN12_GLOBAL__N_117EmptySubobjectMap26UpdateEmptyFieldSubobjectsEPKN5clang9FieldDeclENS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::UpdateEmptyFieldSubobjects(clang::FieldDecl const*, clang::CharUnits)"},
    {"_ZN12_GLOBAL__N_117EmptySubobjectMap26UpdateEmptyFieldSubobjectsEPKN5clang13CXXRecordDeclES4_NS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::UpdateEmptyFieldSubobjects(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*, clang::CharUnits)"},
    {"_ZN12_GLOBAL__N_117EmptySubobjectMap20AddSubobjectAtOffsetEPKN5clang13CXXRecordDeclENS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::AddSubobjectAtOffset(clang::CXXRecordDecl const*, clang::CharUnits)"},
    {"_ZNK12_GLOBAL__N_117EmptySubobjectMap30CanPlaceFieldSubobjectAtOffsetEPKN5clang13CXXRecordDeclES4_NS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*, clang::CharUnits) const"},
    {"_ZNK12_GLOBAL__N_117EmptySubobjectMap25CanPlaceSubobjectAtOffsetEPKN5clang13CXXRecordDeclENS1_9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::CanPlaceSubobjectAtOffset(clang::CXXRecordDecl const*, clang::CharUnits) const"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilderD2Ev", "(anonymous namespace)::RecordLayoutBuilder::~RecordLayoutBuilder()"},
    {"_ZNK12_GLOBAL__N_119RecordLayoutBuilder22GetVirtualPointersSizeEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::RecordLayoutBuilder::GetVirtualPointersSize(clang::CXXRecordDecl const*) const"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilderD0Ev", "(anonymous namespace)::RecordLayoutBuilder::~RecordLayoutBuilder()"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder18LayoutVirtualBasesEPKN5clang13CXXRecordDeclES4_", "(anonymous namespace)::RecordLayoutBuilder::LayoutVirtualBases(clang::CXXRecordDecl const*, clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder10LayoutBaseEPKNS_17BaseSubobjectInfoE", "(anonymous namespace)::RecordLayoutBuilder::LayoutBase((anonymous namespace)::BaseSubobjectInfo const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder28AddPrimaryVirtualBaseOffsetsEPKNS_17BaseSubobjectInfoEN5clang9CharUnitsE", "(anonymous namespace)::RecordLayoutBuilder::AddPrimaryVirtualBaseOffsets((anonymous namespace)::BaseSubobjectInfo const*, clang::CharUnits)"},
    {"_ZN12_GLOBAL__N_117EmptySubobjectMap29CanPlaceBaseSubobjectAtOffsetEPKNS_17BaseSubobjectInfoEN5clang9CharUnitsE", "(anonymous namespace)::EmptySubobjectMap::CanPlaceBaseSubobjectAtOffset((anonymous namespace)::BaseSubobjectInfo const*, clang::CharUnits)"},
    {"_ZN12_GLOBAL__N_117EmptySubobjectMap25UpdateEmptyBaseSubobjectsEPKNS_17BaseSubobjectInfoEN5clang9CharUnitsEb", "(anonymous namespace)::EmptySubobjectMap::UpdateEmptyBaseSubobjects((anonymous namespace)::BaseSubobjectInfo const*, clang::CharUnits, bool)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder24ComputeBaseSubobjectInfoEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::RecordLayoutBuilder::ComputeBaseSubobjectInfo(clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder24ComputeBaseSubobjectInfoEPKN5clang13CXXRecordDeclEbPNS_17BaseSubobjectInfoE", "(anonymous namespace)::RecordLayoutBuilder::ComputeBaseSubobjectInfo(clang::CXXRecordDecl const*, bool, (anonymous namespace)::BaseSubobjectInfo*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPN12_GLOBAL__N_117BaseSubobjectInfoENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXRecordDecl const*, (anonymous namespace)::BaseSubobjectInfo*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<(anonymous namespace)::BaseSubobjectInfo*> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, (anonymous namespace)::BaseSubobjectInfo* const&, std::pair<clang::CXXRecordDecl const*, (anonymous namespace)::BaseSubobjectInfo*>*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilder18SelectPrimaryVBaseEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::RecordLayoutBuilder::SelectPrimaryVBase(clang::CXXRecordDecl const*)"},
    {"_ZN12_GLOBAL__N_119RecordLayoutBuilderC2ERKN5clang10ASTContextEPNS_17EmptySubobjectMapE", "(anonymous namespace)::RecordLayoutBuilder::RecordLayoutBuilder(clang::ASTContext const&, (anonymous namespace)::EmptySubobjectMap*)"},
    {"_ZNK12_GLOBAL__N_121MSRecordLayoutBuilder22GetVirtualPointersSizeEPKN5clang13CXXRecordDeclE", "(anonymous namespace)::MSRecordLayoutBuilder::GetVirtualPointersSize(clang::CXXRecordDecl const*) const"},
    {"_ZN12_GLOBAL__N_121MSRecordLayoutBuilderD1Ev", "(anonymous namespace)::MSRecordLayoutBuilder::~MSRecordLayoutBuilder()"},
    {"_ZN12_GLOBAL__N_121MSRecordLayoutBuilderD0Ev", "(anonymous namespace)::MSRecordLayoutBuilder::~MSRecordLayoutBuilder()"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPKNS1_13CXXMethodDeclENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CXXMethodDecl const*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CXXMethodDecl const*> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, clang::CXXMethodDecl const* const&, std::pair<clang::CXXRecordDecl const*, clang::CXXMethodDecl const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclEPKNS1_13CXXMethodDeclENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CXXMethodDecl const*, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CXXMethodDecl const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang10RecordDeclEPKNS1_15ASTRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::RecordDecl const*, clang::ASTRecordLayout const*, llvm::DenseMapInfo<clang::RecordDecl const*>, llvm::DenseMapInfo<clang::ASTRecordLayout const*> >::InsertIntoBucket(clang::RecordDecl const* const&, clang::ASTRecordLayout const* const&, std::pair<clang::RecordDecl const*, clang::ASTRecordLayout const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang10RecordDeclEPKNS1_15ASTRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::RecordDecl const*, clang::ASTRecordLayout const*, llvm::DenseMapInfo<clang::RecordDecl const*>, llvm::DenseMapInfo<clang::ASTRecordLayout const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang17ObjCContainerDeclEPKNS1_15ASTRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::ObjCContainerDecl const*, clang::ASTRecordLayout const*, llvm::DenseMapInfo<clang::ObjCContainerDecl const*>, llvm::DenseMapInfo<clang::ASTRecordLayout const*> >::InsertIntoBucket(clang::ObjCContainerDecl const* const&, clang::ASTRecordLayout const* const&, std::pair<clang::ObjCContainerDecl const*, clang::ASTRecordLayout const*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang17ObjCContainerDeclEPKNS1_15ASTRecordLayoutENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::ObjCContainerDecl const*, clang::ASTRecordLayout const*, llvm::DenseMapInfo<clang::ObjCContainerDecl const*>, llvm::DenseMapInfo<clang::ASTRecordLayout const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS1_9CharUnitsENS_12DenseMapInfoIS4_EENS6_IS5_EEE16InsertIntoBucketERKS4_RKS5_PSt4pairIS4_S5_E", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> >::InsertIntoBucket(clang::CXXRecordDecl const* const&, clang::CharUnits const&, std::pair<clang::CXXRecordDecl const*, clang::CharUnits>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS1_9CharUnitsENS_12DenseMapInfoIS4_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> >::grow(unsigned int)"},
    {"_ZSt6__findIPPKN5clang13CXXRecordDeclES3_ET_S5_S5_RKT0_St26random_access_iterator_tag", "clang::CXXRecordDecl const** std::__find<clang::CXXRecordDecl const**, clang::CXXRecordDecl const*>(clang::CXXRecordDecl const**, clang::CXXRecordDecl const**, clang::CXXRecordDecl const* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIN5clang9CharUnitsENS_11SmallVectorIPKNS1_13CXXRecordDeclELj1EEENS_12DenseMapInfoIS2_EENS8_IS7_EEE16FindAndConstructERKS2_", "llvm::DenseMap<clang::CharUnits, llvm::SmallVector<clang::CXXRecordDecl const*, 1u>, llvm::DenseMapInfo<clang::CharUnits>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl const*, 1u> > >::FindAndConstruct(clang::CharUnits const&)"},
    {"_ZN4llvm8DenseMapIN5clang9CharUnitsENS_11SmallVectorIPKNS1_13CXXRecordDeclELj1EEENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<clang::CharUnits, llvm::SmallVector<clang::CXXRecordDecl const*, 1u>, llvm::DenseMapInfo<clang::CharUnits>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl const*, 1u> > >::InsertIntoBucket(clang::CharUnits const&, llvm::SmallVector<clang::CXXRecordDecl const*, 1u> const&, std::pair<clang::CharUnits, llvm::SmallVector<clang::CXXRecordDecl const*, 1u> >*)"},
    {"_ZN4llvm8DenseMapIN5clang9CharUnitsENS_11SmallVectorIPKNS1_13CXXRecordDeclELj1EEENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::CharUnits, llvm::SmallVector<clang::CXXRecordDecl const*, 1u>, llvm::DenseMapInfo<clang::CharUnits>, llvm::DenseMapInfo<llvm::SmallVector<clang::CXXRecordDecl const*, 1u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPKN5clang13CXXRecordDeclEEaSERKS5_", "llvm::SmallVectorImpl<clang::CXXRecordDecl const*>::operator=(llvm::SmallVectorImpl<clang::CXXRecordDecl const*> const&)"},
    {"_ZSt6__findIPKPKN5clang13CXXRecordDeclES3_ET_S6_S6_RKT0_St26random_access_iterator_tag", "clang::CXXRecordDecl const* const* std::__find<clang::CXXRecordDecl const* const*, clang::CXXRecordDecl const*>(clang::CXXRecordDecl const* const*, clang::CXXRecordDecl const* const*, clang::CXXRecordDecl const* const&, std::random_access_iterator_tag)"},
    {"_ZNK5clang4Decl7getAttrINS_21MaxFieldAlignmentAttrEEEPT_v", "clang::MaxFieldAlignmentAttr* clang::Decl::getAttr<clang::MaxFieldAlignmentAttr>() const"},
    {"_ZN4llvm8DenseMapIPKN5clang13CXXRecordDeclENS1_9CharUnitsENS_12DenseMapInfoIS4_EENS6_IS5_EEE6insertERKSt4pairIS4_S5_E", "llvm::DenseMap<clang::CXXRecordDecl const*, clang::CharUnits, llvm::DenseMapInfo<clang::CXXRecordDecl const*>, llvm::DenseMapInfo<clang::CharUnits> >::insert(std::pair<clang::CXXRecordDecl const*, clang::CharUnits> const&)"},
    {"_ZNK5clang4Stmt16getStmtClassNameEv", "clang::Stmt::getStmtClassName() const"},
    {"_Z21getStmtInfoTableEntryN5clang4Stmt9StmtClassE", "getStmtInfoTableEntry(clang::Stmt::StmtClass)"},
    {"_ZN5clang4Stmt10PrintStatsEv", "clang::Stmt::PrintStats()"},
    {"_ZN5clang4Stmt12addStmtClassENS0_9StmtClassE", "clang::Stmt::addStmtClass(clang::Stmt::StmtClass)"},
    {"_ZN5clang4Stmt15CollectingStatsEb", "clang::Stmt::CollectingStats(bool)"},
    {"_ZN5clang4Stmt8childrenEv", "clang::Stmt::children()"},
    {"_ZNK5clang4Stmt14getSourceRangeEv", "clang::Stmt::getSourceRange() const"},
    {"_ZNK5clang13ObjCAtTryStmt14getSourceRangeEv", "clang::ObjCAtTryStmt::getSourceRange() const"},
    {"_ZNK5clang10ReturnStmt14getSourceRangeEv", "clang::ReturnStmt::getSourceRange() const"},
    {"_ZN5clang12CompoundStmt8setStmtsERNS_10ASTContextEPPNS_4StmtEj", "clang::CompoundStmt::setStmts(clang::ASTContext&, clang::Stmt**, unsigned int)"},
    {"_ZNK5clang9LabelStmt7getNameEv", "clang::LabelStmt::getName() const"},
    {"_ZNK5clang4Stmt22hasImplicitControlFlowEv", "clang::Stmt::hasImplicitControlFlow() const"},
    {"_ZN5clang7AsmStmt13getOutputExprEj", "clang::AsmStmt::getOutputExpr(unsigned int)"},
    {"_ZNK5clang7AsmStmt19getOutputConstraintEj", "clang::AsmStmt::getOutputConstraint(unsigned int) const"},
    {"_ZNK5clang7AsmStmt18getNumPlusOperandsEv", "clang::AsmStmt::getNumPlusOperands() const"},
    {"_ZN5clang7AsmStmt12getInputExprEj", "clang::AsmStmt::getInputExpr(unsigned int)"},
    {"_ZNK5clang7AsmStmt18getInputConstraintEj", "clang::AsmStmt::getInputConstraint(unsigned int) const"},
    {"_ZN5clang7AsmStmt30setOutputsAndInputsAndClobbersERNS_10ASTContextEPPNS_14IdentifierInfoEPPNS_13StringLiteralEPPNS_4StmtEjjS8_j", "clang::AsmStmt::setOutputsAndInputsAndClobbers(clang::ASTContext&, clang::IdentifierInfo**, clang::StringLiteral**, clang::Stmt**, unsigned int, unsigned int, clang::StringLiteral**, unsigned int)"},
    {"_ZNK5clang7AsmStmt15getNamedOperandEN4llvm9StringRefE", "clang::AsmStmt::getNamedOperand(llvm::StringRef) const"},
    {"_ZNK5clang7AsmStmt16AnalyzeAsmStringERN4llvm15SmallVectorImplINS0_14AsmStringPieceEEERNS_10ASTContextERj", "clang::AsmStmt::AnalyzeAsmString(llvm::SmallVectorImpl<clang::AsmStmt::AsmStringPiece>&, clang::ASTContext&, unsigned int&) const"},
    {"_ZNK5clang12CXXCatchStmt13getCaughtTypeEv", "clang::CXXCatchStmt::getCaughtType() const"},
    {"_ZN5clang7AsmStmtC1ERNS_10ASTContextENS_14SourceLocationEbbbjjPPNS_14IdentifierInfoEPPNS_13StringLiteralEPPNS_4ExprES8_jS9_S3_", "clang::AsmStmt::AsmStmt(clang::ASTContext&, clang::SourceLocation, bool, bool, bool, unsigned int, unsigned int, clang::IdentifierInfo**, clang::StringLiteral**, clang::Expr**, clang::StringLiteral*, unsigned int, clang::StringLiteral**, clang::SourceLocation)"},
    {"_ZN5clang7AsmStmtC2ERNS_10ASTContextENS_14SourceLocationEbbbjjPPNS_14IdentifierInfoEPPNS_13StringLiteralEPPNS_4ExprES8_jS9_S3_", "clang::AsmStmt::AsmStmt(clang::ASTContext&, clang::SourceLocation, bool, bool, bool, unsigned int, unsigned int, clang::IdentifierInfo**, clang::StringLiteral**, clang::Expr**, clang::StringLiteral*, unsigned int, clang::StringLiteral**, clang::SourceLocation)"},
    {"_ZN5clang21ObjCForCollectionStmtC1EPNS_4StmtEPNS_4ExprES2_NS_14SourceLocationES5_", "clang::ObjCForCollectionStmt::ObjCForCollectionStmt(clang::Stmt*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang21ObjCForCollectionStmtC2EPNS_4StmtEPNS_4ExprES2_NS_14SourceLocationES5_", "clang::ObjCForCollectionStmt::ObjCForCollectionStmt(clang::Stmt*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang13ObjCAtTryStmtC1ENS_14SourceLocationEPNS_4StmtEPS3_jS3_", "clang::ObjCAtTryStmt::ObjCAtTryStmt(clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int, clang::Stmt*)"},
    {"_ZN5clang13ObjCAtTryStmtC2ENS_14SourceLocationEPNS_4StmtEPS3_jS3_", "clang::ObjCAtTryStmt::ObjCAtTryStmt(clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int, clang::Stmt*)"},
    {"_ZN5clang13ObjCAtTryStmt6CreateERNS_10ASTContextENS_14SourceLocationEPNS_4StmtEPS5_jS5_", "clang::ObjCAtTryStmt::Create(clang::ASTContext&, clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int, clang::Stmt*)"},
    {"_ZN5clang13ObjCAtTryStmt11CreateEmptyERNS_10ASTContextEjb", "clang::ObjCAtTryStmt::CreateEmpty(clang::ASTContext&, unsigned int, bool)"},
    {"_ZN5clang10CXXTryStmt6CreateERNS_10ASTContextENS_14SourceLocationEPNS_4StmtEPS5_j", "clang::CXXTryStmt::Create(clang::ASTContext&, clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int)"},
    {"_ZN5clang10CXXTryStmtC1ENS_14SourceLocationEPNS_4StmtEPS3_j", "clang::CXXTryStmt::CXXTryStmt(clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int)"},
    {"_ZN5clang10CXXTryStmt6CreateERNS_10ASTContextENS_4Stmt10EmptyShellEj", "clang::CXXTryStmt::Create(clang::ASTContext&, clang::Stmt::EmptyShell, unsigned int)"},
    {"_ZN5clang10CXXTryStmtC2ENS_14SourceLocationEPNS_4StmtEPS3_j", "clang::CXXTryStmt::CXXTryStmt(clang::SourceLocation, clang::Stmt*, clang::Stmt**, unsigned int)"},
    {"_ZN5clang6IfStmtC1ERNS_10ASTContextENS_14SourceLocationEPNS_7VarDeclEPNS_4ExprEPNS_4StmtES3_S9_", "clang::IfStmt::IfStmt(clang::ASTContext&, clang::SourceLocation, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang6IfStmtC2ERNS_10ASTContextENS_14SourceLocationEPNS_7VarDeclEPNS_4ExprEPNS_4StmtES3_S9_", "clang::IfStmt::IfStmt(clang::ASTContext&, clang::SourceLocation, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::Stmt*)"},
    {"_ZN5clang6IfStmt20setConditionVariableERNS_10ASTContextEPNS_7VarDeclE", "clang::IfStmt::setConditionVariable(clang::ASTContext&, clang::VarDecl*)"},
    {"_ZNK5clang6IfStmt20getConditionVariableEv", "clang::IfStmt::getConditionVariable() const"},
    {"_ZN5clang7ForStmtC1ERNS_10ASTContextEPNS_4StmtEPNS_4ExprEPNS_7VarDeclES6_S4_NS_14SourceLocationES9_S9_", "clang::ForStmt::ForStmt(clang::ASTContext&, clang::Stmt*, clang::Expr*, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang7ForStmtC2ERNS_10ASTContextEPNS_4StmtEPNS_4ExprEPNS_7VarDeclES6_S4_NS_14SourceLocationES9_S9_", "clang::ForStmt::ForStmt(clang::ASTContext&, clang::Stmt*, clang::Expr*, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang7ForStmt20setConditionVariableERNS_10ASTContextEPNS_7VarDeclE", "clang::ForStmt::setConditionVariable(clang::ASTContext&, clang::VarDecl*)"},
    {"_ZNK5clang7ForStmt20getConditionVariableEv", "clang::ForStmt::getConditionVariable() const"},
    {"_ZN5clang10SwitchStmtC1ERNS_10ASTContextEPNS_7VarDeclEPNS_4ExprE", "clang::SwitchStmt::SwitchStmt(clang::ASTContext&, clang::VarDecl*, clang::Expr*)"},
    {"_ZN5clang10SwitchStmtC2ERNS_10ASTContextEPNS_7VarDeclEPNS_4ExprE", "clang::SwitchStmt::SwitchStmt(clang::ASTContext&, clang::VarDecl*, clang::Expr*)"},
    {"_ZN5clang10SwitchStmt20setConditionVariableERNS_10ASTContextEPNS_7VarDeclE", "clang::SwitchStmt::setConditionVariable(clang::ASTContext&, clang::VarDecl*)"},
    {"_ZNK5clang10SwitchStmt20getConditionVariableEv", "clang::SwitchStmt::getConditionVariable() const"},
    {"_ZN5clang10SwitchCase10getSubStmtEv", "clang::SwitchCase::getSubStmt()"},
    {"_ZN5clang9WhileStmtC1ERNS_10ASTContextEPNS_7VarDeclEPNS_4ExprEPNS_4StmtENS_14SourceLocationE", "clang::WhileStmt::WhileStmt(clang::ASTContext&, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation)"},
    {"_ZN5clang9WhileStmtC2ERNS_10ASTContextEPNS_7VarDeclEPNS_4ExprEPNS_4StmtENS_14SourceLocationE", "clang::WhileStmt::WhileStmt(clang::ASTContext&, clang::VarDecl*, clang::Expr*, clang::Stmt*, clang::SourceLocation)"},
    {"_ZN5clang9WhileStmt20setConditionVariableERNS_10ASTContextEPNS_7VarDeclE", "clang::WhileStmt::setConditionVariable(clang::ASTContext&, clang::VarDecl*)"},
    {"_ZNK5clang9WhileStmt20getConditionVariableEv", "clang::WhileStmt::getConditionVariable() const"},
    {"_ZN5clang16IndirectGotoStmt17getConstantTargetEv", "clang::IndirectGotoStmt::getConstantTarget()"},
    {"_ZNK5clang10ReturnStmt11getRetValueEv", "clang::ReturnStmt::getRetValue() const"},
    {"_ZN5clang10ReturnStmt11getRetValueEv", "clang::ReturnStmt::getRetValue()"},
    {"_ZN5clang8DeclStmt8childrenEv", "clang::DeclStmt::children()"},
    {"_ZNK5clang27CXXDependentScopeMemberExpr14getSourceRangeEv", "clang::CXXDependentScopeMemberExpr::getSourceRange() const"},
    {"_ZNK5clang10MemberExpr14getSourceRangeEv", "clang::MemberExpr::getSourceRange() const"},
    {"_ZNK5clang20UnresolvedMemberExpr14getSourceRangeEv", "clang::UnresolvedMemberExpr::getSourceRange() const"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang7AsmStmt14AsmStringPieceELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::AsmStmt::AsmStringPiece, false>::grow(unsigned long)"},
    {"_ZNK5clang4Stmt4dumpERNS_13SourceManagerE", "clang::Stmt::dump(clang::SourceManager&) const"},
    {"_ZNK5clang4Stmt4dumpERN4llvm11raw_ostreamERNS_13SourceManagerE", "clang::Stmt::dump(llvm::raw_ostream&, clang::SourceManager&) const"},
    {"_ZN12_GLOBAL__N_110StmtDumper11DumpSubTreeEPN5clang4StmtE", "(anonymous namespace)::StmtDumper::DumpSubTree(clang::Stmt*)"},
    {"_ZNK5clang4Stmt4dumpEv", "clang::Stmt::dump() const"},
    {"_ZNK5clang4Stmt7dumpAllERNS_13SourceManagerE", "clang::Stmt::dumpAll(clang::SourceManager&) const"},
    {"_ZNK5clang4Stmt7dumpAllEv", "clang::Stmt::dumpAll() const"},
    {"_ZN12_GLOBAL__N_110StmtDumper13VisitDeclStmtEPN5clang8DeclStmtE", "(anonymous namespace)::StmtDumper::VisitDeclStmt(clang::DeclStmt*)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_110StmtDumperEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::StmtDumper, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper19VisitBinaryOperatorEPN5clang14BinaryOperatorE", "(anonymous namespace)::StmtDumper::VisitBinaryOperator(clang::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper27VisitCompoundAssignOperatorEPN5clang22CompoundAssignOperatorE", "(anonymous namespace)::StmtDumper::VisitCompoundAssignOperator(clang::CompoundAssignOperator*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper21VisitCXXConstructExprEPN5clang16CXXConstructExprE", "(anonymous namespace)::StmtDumper::VisitCXXConstructExpr(clang::CXXConstructExpr*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::StmtDumper::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper8DumpStmtEPKN5clang4StmtE", "(anonymous namespace)::StmtDumper::DumpStmt(clang::Stmt const*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper12DumpLocationEN5clang14SourceLocationE", "(anonymous namespace)::StmtDumper::DumpLocation(clang::SourceLocation)"},
    {"_ZN12_GLOBAL__N_110StmtDumper14DumpDeclaratorEPN5clang4DeclE", "(anonymous namespace)::StmtDumper::DumpDeclarator(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper8DumpExprEPKN5clang4ExprE", "(anonymous namespace)::StmtDumper::DumpExpr(clang::Expr const*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper8DumpTypeEN5clang8QualTypeE", "(anonymous namespace)::StmtDumper::DumpType(clang::QualType)"},
    {"_ZN12_GLOBAL__N_110StmtDumper11DumpDeclRefEPN5clang4DeclE", "(anonymous namespace)::StmtDumper::DumpDeclRef(clang::Decl*)"},
    {"_Z12DumpBasePathRN4llvm11raw_ostreamEPN5clang8CastExprE", "DumpBasePath(llvm::raw_ostream&, clang::CastExpr*)"},
    {"_ZN12_GLOBAL__N_110StmtDumper21VisitCXXNamedCastExprEPN5clang16CXXNamedCastExprE", "(anonymous namespace)::StmtDumper::VisitCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN5clang16StmtIteratorBase6NextVAEv", "clang::StmtIteratorBase::NextVA()"},
    {"_ZN5clang16StmtIteratorBase8NextDeclEb", "clang::StmtIteratorBase::NextDecl(bool)"},
    {"_ZN5clang16StmtIteratorBase10HandleDeclEPNS_4DeclE", "clang::StmtIteratorBase::HandleDecl(clang::Decl*)"},
    {"_ZN5clang16StmtIteratorBaseC1EPNS_4DeclEPPNS_4StmtE", "clang::StmtIteratorBase::StmtIteratorBase(clang::Decl*, clang::Stmt**)"},
    {"_ZN5clang16StmtIteratorBaseC2EPNS_4DeclEPPNS_4StmtE", "clang::StmtIteratorBase::StmtIteratorBase(clang::Decl*, clang::Stmt**)"},
    {"_ZN5clang16StmtIteratorBaseC1EPPNS_4DeclES3_", "clang::StmtIteratorBase::StmtIteratorBase(clang::Decl**, clang::Decl**)"},
    {"_ZN5clang16StmtIteratorBaseC2EPPNS_4DeclES3_", "clang::StmtIteratorBase::StmtIteratorBase(clang::Decl**, clang::Decl**)"},
    {"_ZN5clang16StmtIteratorBaseC1EPKNS_17VariableArrayTypeE", "clang::StmtIteratorBase::StmtIteratorBase(clang::VariableArrayType const*)"},
    {"_ZN5clang16StmtIteratorBaseC2EPKNS_17VariableArrayTypeE", "clang::StmtIteratorBase::StmtIteratorBase(clang::VariableArrayType const*)"},
    {"_ZNK5clang16StmtIteratorBase11GetDeclExprEv", "clang::StmtIteratorBase::GetDeclExpr() const"},
    {"_ZNK5clang4Stmt10dumpPrettyERNS_10ASTContextE", "clang::Stmt::dumpPretty(clang::ASTContext&) const"},
    {"_ZNK5clang4Stmt11printPrettyERN4llvm11raw_ostreamERNS_10ASTContextEPNS_13PrinterHelperERKNS_14PrintingPolicyEj", "clang::Stmt::printPretty(llvm::raw_ostream&, clang::ASTContext&, clang::PrinterHelper*, clang::PrintingPolicy const&, unsigned int) const"},
    {"_ZN5clang13PrinterHelperD0Ev", "clang::PrinterHelper::~PrinterHelper()"},
    {"_ZN5clang13PrinterHelperD1Ev", "clang::PrinterHelper::~PrinterHelper()"},
    {"_ZN5clang13PrinterHelperD2Ev", "clang::PrinterHelper::~PrinterHelper()"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_111StmtPrinterEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::StmtPrinter, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter13VisitCallExprEPN5clang8CallExprE", "(anonymous namespace)::StmtPrinter::VisitCallExpr(clang::CallExpr*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter18VisitStringLiteralEPN5clang13StringLiteralE", "(anonymous namespace)::StmtPrinter::VisitStringLiteral(clang::StringLiteral*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter18VisitUnaryOperatorEPN5clang13UnaryOperatorE", "(anonymous namespace)::StmtPrinter::VisitUnaryOperator(clang::UnaryOperator*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter9PrintExprEPN5clang4ExprE", "(anonymous namespace)::StmtPrinter::PrintExpr(clang::Expr*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter9PrintStmtEPN5clang4StmtEi", "(anonymous namespace)::StmtPrinter::PrintStmt(clang::Stmt*, int)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter20PrintRawCompoundStmtEPN5clang12CompoundStmtE", "(anonymous namespace)::StmtPrinter::PrintRawCompoundStmt(clang::CompoundStmt*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter16PrintRawDeclStmtEPN5clang8DeclStmtE", "(anonymous namespace)::StmtPrinter::PrintRawDeclStmt(clang::DeclStmt*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter14PrintRawIfStmtEPN5clang6IfStmtE", "(anonymous namespace)::StmtPrinter::PrintRawIfStmt(clang::IfStmt*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter21VisitCXXNamedCastExprEPN5clang16CXXNamedCastExprE", "(anonymous namespace)::StmtPrinter::VisitCXXNamedCastExpr(clang::CXXNamedCastExpr*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter13PrintCallArgsEPN5clang8CallExprE", "(anonymous namespace)::StmtPrinter::PrintCallArgs(clang::CallExpr*)"},
    {"_ZN12_GLOBAL__N_111StmtPrinter20PrintRawCXXCatchStmtEPN5clang12CXXCatchStmtE", "(anonymous namespace)::StmtPrinter::PrintRawCXXCatchStmt(clang::CXXCatchStmt*)"},
    {"_ZN5clang4Stmt7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextEb", "clang::Stmt::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, bool)"},
    {"_ZN5clang11StmtVisitorIN12_GLOBAL__N_112StmtProfilerEvE5VisitEPNS_4StmtE", "clang::StmtVisitor<(anonymous namespace)::StmtProfiler, void>::Visit(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_112StmtProfiler17VisitInitListExprEPN5clang12InitListExprE", "(anonymous namespace)::StmtProfiler::VisitInitListExpr(clang::InitListExpr*)"},
    {"_ZN12_GLOBAL__N_112StmtProfiler9VisitStmtEPN5clang4StmtE", "(anonymous namespace)::StmtProfiler::VisitStmt(clang::Stmt*)"},
    {"_ZN12_GLOBAL__N_112StmtProfiler9VisitDeclEPN5clang4DeclE", "(anonymous namespace)::StmtProfiler::VisitDecl(clang::Decl*)"},
    {"_ZN12_GLOBAL__N_112StmtProfiler21VisitTemplateArgumentERKN5clang16TemplateArgumentE", "(anonymous namespace)::StmtProfiler::VisitTemplateArgument(clang::TemplateArgument const&)"},
    {"_ZNK5clang4Stmt7viewASTEv", "clang::Stmt::viewAST() const"},
    {"_ZN5clang16TemplateArgument14CreatePackCopyERNS_10ASTContextEPKS0_j", "clang::TemplateArgument::CreatePackCopy(clang::ASTContext&, clang::TemplateArgument const*, unsigned int)"},
    {"_ZNK5clang16TemplateArgument11isDependentEv", "clang::TemplateArgument::isDependent() const"},
    {"_ZNK5clang16TemplateArgument15isPackExpansionEv", "clang::TemplateArgument::isPackExpansion() const"},
    {"_ZNK5clang16TemplateArgument31containsUnexpandedParameterPackEv", "clang::TemplateArgument::containsUnexpandedParameterPack() const"},
    {"_ZNK5clang16TemplateArgument24getNumTemplateExpansionsEv", "clang::TemplateArgument::getNumTemplateExpansions() const"},
    {"_ZNK5clang16TemplateArgument7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextE", "clang::TemplateArgument::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&) const"},
    {"_ZNK5clang16TemplateArgument18structurallyEqualsERKS0_", "clang::TemplateArgument::structurallyEquals(clang::TemplateArgument const&) const"},
    {"_ZNK5clang16TemplateArgument23getPackExpansionPatternEv", "clang::TemplateArgument::getPackExpansionPattern() const"},
    {"_ZNK5clang16TemplateArgument5printERKNS_14PrintingPolicyERN4llvm11raw_ostreamE", "clang::TemplateArgument::print(clang::PrintingPolicy const&, llvm::raw_ostream&) const"},
    {"_ZN5clang23TemplateArgumentLocInfoC1Ev", "clang::TemplateArgumentLocInfo::TemplateArgumentLocInfo()"},
    {"_ZN5clang23TemplateArgumentLocInfoC2Ev", "clang::TemplateArgumentLocInfo::TemplateArgumentLocInfo()"},
    {"_ZNK5clang19TemplateArgumentLoc14getSourceRangeEv", "clang::TemplateArgumentLoc::getSourceRange() const"},
    {"_ZNK5clang19TemplateArgumentLoc23getPackExpansionPatternERNS_14SourceLocationERN4llvm8OptionalIjEERNS_10ASTContextE", "clang::TemplateArgumentLoc::getPackExpansionPattern(clang::SourceLocation&, llvm::Optional<unsigned int>&, clang::ASTContext&) const"},
    {"_ZN5clanglsERKNS_17DiagnosticBuilderERKNS_16TemplateArgumentE", "clang::operator<<(clang::DiagnosticBuilder const&, clang::TemplateArgument const&)"},
    {"_ZNK5clang36SubstTemplateTemplateParmPackStorage15getArgumentPackEv", "clang::SubstTemplateTemplateParmPackStorage::getArgumentPack() const"},
    {"_ZN5clang36SubstTemplateTemplateParmPackStorage7ProfileERN4llvm16FoldingSetNodeIDE", "clang::SubstTemplateTemplateParmPackStorage::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang36SubstTemplateTemplateParmPackStorage7ProfileERN4llvm16FoldingSetNodeIDERNS_10ASTContextEPNS_24TemplateTemplateParmDeclERKNS_16TemplateArgumentE", "clang::SubstTemplateTemplateParmPackStorage::Profile(llvm::FoldingSetNodeID&, clang::ASTContext&, clang::TemplateTemplateParmDecl*, clang::TemplateArgument const&)"},
    {"_ZNK5clang12TemplateName7getKindEv", "clang::TemplateName::getKind() const"},
    {"_ZNK5clang12TemplateName17getAsTemplateDeclEv", "clang::TemplateName::getAsTemplateDecl() const"},
    {"_ZNK5clang12TemplateName11isDependentEv", "clang::TemplateName::isDependent() const"},
    {"_ZNK5clang12TemplateName31containsUnexpandedParameterPackEv", "clang::TemplateName::containsUnexpandedParameterPack() const"},
    {"_ZNK5clang12TemplateName5printERN4llvm11raw_ostreamERKNS_14PrintingPolicyEb", "clang::TemplateName::print(llvm::raw_ostream&, clang::PrintingPolicy const&, bool) const"},
    {"_ZN5clanglsERKNS_17DiagnosticBuilderENS_12TemplateNameE", "clang::operator<<(clang::DiagnosticBuilder const&, clang::TemplateName)"},
    {"_ZNK5clang12TemplateName4dumpEv", "clang::TemplateName::dump() const"},
    {"_ZN5clang8QualType10isConstantES0_RNS_10ASTContextE", "clang::QualType::isConstant(clang::QualType, clang::ASTContext&)"},
    {"_ZN5clang17ConstantArrayType20getNumAddressingBitsERNS_10ASTContextENS_8QualTypeERKN4llvm5APIntE", "clang::ConstantArrayType::getNumAddressingBits(clang::ASTContext&, clang::QualType, llvm::APInt const&)"},
    {"_ZN5clang17ConstantArrayType14getMaxSizeBitsERNS_10ASTContextE", "clang::ConstantArrayType::getMaxSizeBits(clang::ASTContext&)"},
    {"_ZN5clang23DependentSizedArrayTypeC1ERKNS_10ASTContextENS_8QualTypeES4_PNS_4ExprENS_9ArrayType17ArraySizeModifierEjNS_11SourceRangeE", "clang::DependentSizedArrayType::DependentSizedArrayType(clang::ASTContext const&, clang::QualType, clang::QualType, clang::Expr*, clang::ArrayType::ArraySizeModifier, unsigned int, clang::SourceRange)"},
    {"_ZN5clang23DependentSizedArrayTypeC2ERKNS_10ASTContextENS_8QualTypeES4_PNS_4ExprENS_9ArrayType17ArraySizeModifierEjNS_11SourceRangeE", "clang::DependentSizedArrayType::DependentSizedArrayType(clang::ASTContext const&, clang::QualType, clang::QualType, clang::Expr*, clang::ArrayType::ArraySizeModifier, unsigned int, clang::SourceRange)"},
    {"_ZN5clang23DependentSizedArrayType7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextENS_8QualTypeENS_9ArrayType17ArraySizeModifierEjPNS_4ExprE", "clang::DependentSizedArrayType::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, clang::QualType, clang::ArrayType::ArraySizeModifier, unsigned int, clang::Expr*)"},
    {"_ZN5clang27DependentSizedExtVectorTypeC1ERKNS_10ASTContextENS_8QualTypeES4_PNS_4ExprENS_14SourceLocationE", "clang::DependentSizedExtVectorType::DependentSizedExtVectorType(clang::ASTContext const&, clang::QualType, clang::QualType, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang27DependentSizedExtVectorTypeC2ERKNS_10ASTContextENS_8QualTypeES4_PNS_4ExprENS_14SourceLocationE", "clang::DependentSizedExtVectorType::DependentSizedExtVectorType(clang::ASTContext const&, clang::QualType, clang::QualType, clang::Expr*, clang::SourceLocation)"},
    {"_ZN5clang27DependentSizedExtVectorType7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextENS_8QualTypeEPNS_4ExprE", "clang::DependentSizedExtVectorType::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, clang::QualType, clang::Expr*)"},
    {"_ZN5clang10VectorTypeC1ENS_8QualTypeEjS1_NS0_10VectorKindE", "clang::VectorType::VectorType(clang::QualType, unsigned int, clang::QualType, clang::VectorType::VectorKind)"},
    {"_ZN5clang10VectorTypeC2ENS_8QualTypeEjS1_NS0_10VectorKindE", "clang::VectorType::VectorType(clang::QualType, unsigned int, clang::QualType, clang::VectorType::VectorKind)"},
    {"_ZN5clang10VectorTypeC1ENS_4Type9TypeClassENS_8QualTypeEjS3_NS0_10VectorKindE", "clang::VectorType::VectorType(clang::Type::TypeClass, clang::QualType, unsigned int, clang::QualType, clang::VectorType::VectorKind)"},
    {"_ZN5clang10VectorTypeC2ENS_4Type9TypeClassENS_8QualTypeEjS3_NS0_10VectorKindE", "clang::VectorType::VectorType(clang::Type::TypeClass, clang::QualType, unsigned int, clang::QualType, clang::VectorType::VectorKind)"},
    {"_ZNK5clang4Type29getArrayElementTypeNoTypeQualEv", "clang::Type::getArrayElementTypeNoTypeQual() const"},
    {"_ZNK5clang4Type27getUnqualifiedDesugaredTypeEv", "clang::Type::getUnqualifiedDesugaredType() const"},
    {"_ZN5clang8QualType16getDesugaredTypeES0_RKNS_10ASTContextE", "clang::QualType::getDesugaredType(clang::QualType, clang::ASTContext const&)"},
    {"_ZN5clang8QualType21getSplitDesugaredTypeES0_", "clang::QualType::getSplitDesugaredType(clang::QualType)"},
    {"_ZNK5clang11TypedefType7desugarEv", "clang::TypedefType::desugar() const"},
    {"_ZNK5clang14TypeOfExprType7desugarEv", "clang::TypeOfExprType::desugar() const"},
    {"_ZN5clang8QualType27getSplitUnqualifiedTypeImplES0_", "clang::QualType::getSplitUnqualifiedTypeImpl(clang::QualType)"},
    {"_ZN5clang8QualType12IgnoreParensES0_", "clang::QualType::IgnoreParens(clang::QualType)"},
    {"_ZNK5clang4Type10isVoidTypeEv", "clang::Type::isVoidType() const"},
    {"_ZNK5clang4Type13isDerivedTypeEv", "clang::Type::isDerivedType() const"},
    {"_ZNK5clang4Type11isClassTypeEv", "clang::Type::isClassType() const"},
    {"_ZNK5clang4Type15isStructureTypeEv", "clang::Type::isStructureType() const"},
    {"_ZNK5clang4Type22isStructureOrClassTypeEv", "clang::Type::isStructureOrClassType() const"},
    {"_ZNK5clang4Type17isVoidPointerTypeEv", "clang::Type::isVoidPointerType() const"},
    {"_ZNK5clang4Type11isUnionTypeEv", "clang::Type::isUnionType() const"},
    {"_ZNK5clang4Type13isComplexTypeEv", "clang::Type::isComplexType() const"},
    {"_ZNK5clang4Type14isFloatingTypeEv", "clang::Type::isFloatingType() const"},
    {"_ZNK5clang4Type20isComplexIntegerTypeEv", "clang::Type::isComplexIntegerType() const"},
    {"_ZNK5clang4Type23getAsComplexIntegerTypeEv", "clang::Type::getAsComplexIntegerType() const"},
    {"_ZNK5clang4Type13isIntegerTypeEv", "clang::Type::isIntegerType() const"},
    {"_ZNK5clang4Type14getPointeeTypeEv", "clang::Type::getPointeeType() const"},
    {"_ZNK5clang4Type18getAsStructureTypeEv", "clang::Type::getAsStructureType() const"},
    {"_ZNK5clang4Type14getAsUnionTypeEv", "clang::Type::getAsUnionType() const"},
    {"_ZN5clang14ObjCObjectTypeC1ENS_8QualTypeES1_PKPNS_16ObjCProtocolDeclEj", "clang::ObjCObjectType::ObjCObjectType(clang::QualType, clang::QualType, clang::ObjCProtocolDecl* const*, unsigned int)"},
    {"_ZN5clang14ObjCObjectTypeC2ENS_8QualTypeES1_PKPNS_16ObjCProtocolDeclEj", "clang::ObjCObjectType::ObjCObjectType(clang::QualType, clang::QualType, clang::ObjCProtocolDecl* const*, unsigned int)"},
    {"_ZNK5clang4Type31getAsObjCQualifiedInterfaceTypeEv", "clang::Type::getAsObjCQualifiedInterfaceType() const"},
    {"_ZNK5clang4Type28isObjCQualifiedInterfaceTypeEv", "clang::Type::isObjCQualifiedInterfaceType() const"},
    {"_ZNK5clang4Type24getAsObjCQualifiedIdTypeEv", "clang::Type::getAsObjCQualifiedIdType() const"},
    {"_ZNK5clang4Type29getAsObjCInterfacePointerTypeEv", "clang::Type::getAsObjCInterfacePointerType() const"},
    {"_ZNK5clang4Type30getCXXRecordDeclForPointerTypeEv", "clang::Type::getCXXRecordDeclForPointerType() const"},
    {"_ZNK5clang4Type18getAsCXXRecordDeclEv", "clang::Type::getAsCXXRecordDecl() const"},
    {"_ZNK5clang21InjectedClassNameType7getDeclEv", "clang::InjectedClassNameType::getDecl() const"},
    {"_ZNK5clang4Type24hasIntegerRepresentationEv", "clang::Type::hasIntegerRepresentation() const"},
    {"_ZNK5clang4Type14isIntegralTypeERNS_10ASTContextE", "clang::Type::isIntegralType(clang::ASTContext&) const"},
    {"_ZNK5clang4Type27isIntegralOrEnumerationTypeEv", "clang::Type::isIntegralOrEnumerationType() const"},
    {"_ZNK5clang4Type35isIntegralOrUnscopedEnumerationTypeEv", "clang::Type::isIntegralOrUnscopedEnumerationType() const"},
    {"_ZNK5clang4Type13isBooleanTypeEv", "clang::Type::isBooleanType() const"},
    {"_ZNK5clang4Type10isCharTypeEv", "clang::Type::isCharType() const"},
    {"_ZNK5clang4Type14isWideCharTypeEv", "clang::Type::isWideCharType() const"},
    {"_ZNK5clang4Type18isAnyCharacterTypeEv", "clang::Type::isAnyCharacterType() const"},
    {"_ZNK5clang4Type19isSignedIntegerTypeEv", "clang::Type::isSignedIntegerType() const"},
    {"_ZNK5clang4Type30hasSignedIntegerRepresentationEv", "clang::Type::hasSignedIntegerRepresentation() const"},
    {"_ZNK5clang4Type21isUnsignedIntegerTypeEv", "clang::Type::isUnsignedIntegerType() const"},
    {"_ZNK5clang4Type32hasUnsignedIntegerRepresentationEv", "clang::Type::hasUnsignedIntegerRepresentation() const"},
    {"_ZNK5clang4Type25hasFloatingRepresentationEv", "clang::Type::hasFloatingRepresentation() const"},
    {"_ZNK5clang4Type18isRealFloatingTypeEv", "clang::Type::isRealFloatingType() const"},
    {"_ZNK5clang4Type10isRealTypeEv", "clang::Type::isRealType() const"},
    {"_ZNK5clang4Type16isArithmeticTypeEv", "clang::Type::isArithmeticType() const"},
    {"_ZNK5clang4Type12isScalarTypeEv", "clang::Type::isScalarType() const"},
    {"_ZNK5clang4Type17getScalarTypeKindEv", "clang::Type::getScalarTypeKind() const"},
    {"_ZNK5clang4Type15isAggregateTypeEv", "clang::Type::isAggregateType() const"},
    {"_ZNK5clang4Type18isConstantSizeTypeEv", "clang::Type::isConstantSizeType() const"},
    {"_ZNK5clang4Type16isIncompleteTypeEv", "clang::Type::isIncompleteType() const"},
    {"_ZNK5clang7TagType7getDeclEv", "clang::TagType::getDecl() const"},
    {"_ZNK5clang4Type9isPODTypeEv", "clang::Type::isPODType() const"},
    {"_ZNK5clang4Type13isLiteralTypeEv", "clang::Type::isLiteralType() const"},
    {"_ZNK5clang4Type23isPromotableIntegerTypeEv", "clang::Type::isPromotableIntegerType() const"},
    {"_ZNK5clang4Type13isNullPtrTypeEv", "clang::Type::isNullPtrType() const"},
    {"_ZNK5clang4Type15isSpecifierTypeEv", "clang::Type::isSpecifierType() const"},
    {"_ZN5clang15TypeWithKeyword21getKeywordForTypeSpecEj", "clang::TypeWithKeyword::getKeywordForTypeSpec(unsigned int)"},
    {"_ZN5clang15TypeWithKeyword25getTagTypeKindForTypeSpecEj", "clang::TypeWithKeyword::getTagTypeKindForTypeSpec(unsigned int)"},
    {"_ZN5clang15TypeWithKeyword24getKeywordForTagTypeKindENS_11TagTypeKindE", "clang::TypeWithKeyword::getKeywordForTagTypeKind(clang::TagTypeKind)"},
    {"_ZN5clang15TypeWithKeyword24getTagTypeKindForKeywordENS_21ElaboratedTypeKeywordE", "clang::TypeWithKeyword::getTagTypeKindForKeyword(clang::ElaboratedTypeKeyword)"},
    {"_ZN5clang15TypeWithKeyword20KeywordIsTagTypeKindENS_21ElaboratedTypeKeywordE", "clang::TypeWithKeyword::KeywordIsTagTypeKind(clang::ElaboratedTypeKeyword)"},
    {"_ZN5clang15TypeWithKeyword14getKeywordNameENS_21ElaboratedTypeKeywordE", "clang::TypeWithKeyword::getKeywordName(clang::ElaboratedTypeKeyword)"},
    {"_ZN5clang35DependentTemplateSpecializationTypeC1ENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoEjPKNS_16TemplateArgumentENS_8QualTypeE", "clang::DependentTemplateSpecializationType::DependentTemplateSpecializationType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, unsigned int, clang::TemplateArgument const*, clang::QualType)"},
    {"_ZN5clang35DependentTemplateSpecializationTypeC2ENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoEjPKNS_16TemplateArgumentENS_8QualTypeE", "clang::DependentTemplateSpecializationType::DependentTemplateSpecializationType(clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, unsigned int, clang::TemplateArgument const*, clang::QualType)"},
    {"_ZN5clang35DependentTemplateSpecializationType7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextENS_21ElaboratedTypeKeywordEPNS_19NestedNameSpecifierEPKNS_14IdentifierInfoEjPKNS_16TemplateArgumentE", "clang::DependentTemplateSpecializationType::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, clang::ElaboratedTypeKeyword, clang::NestedNameSpecifier*, clang::IdentifierInfo const*, unsigned int, clang::TemplateArgument const*)"},
    {"_ZNK5clang4Type25isElaboratedTypeSpecifierEv", "clang::Type::isElaboratedTypeSpecifier() const"},
    {"_ZNK5clang4Type16getTypeClassNameEv", "clang::Type::getTypeClassName() const"},
    {"_ZNK5clang11BuiltinType7getNameERKNS_11LangOptionsE", "clang::BuiltinType::getName(clang::LangOptions const&) const"},
    {"_ZNK5clang8QualType20getNonLValueExprTypeERNS_10ASTContextE", "clang::QualType::getNonLValueExprType(clang::ASTContext&) const"},
    {"_ZN5clang12FunctionType18getNameForCallConvENS_11CallingConvE", "clang::FunctionType::getNameForCallConv(clang::CallingConv)"},
    {"_ZN5clang17FunctionProtoTypeC1ENS_8QualTypeEPKS1_jS1_RKNS0_12ExtProtoInfoE", "clang::FunctionProtoType::FunctionProtoType(clang::QualType, clang::QualType const*, unsigned int, clang::QualType, clang::FunctionProtoType::ExtProtoInfo const&)"},
    {"_ZN5clang17FunctionProtoTypeC2ENS_8QualTypeEPKS1_jS1_RKNS0_12ExtProtoInfoE", "clang::FunctionProtoType::FunctionProtoType(clang::QualType, clang::QualType const*, unsigned int, clang::QualType, clang::FunctionProtoType::ExtProtoInfo const&)"},
    {"_ZNK5clang17FunctionProtoType18isTemplateVariadicEv", "clang::FunctionProtoType::isTemplateVariadic() const"},
    {"_ZN5clang17FunctionProtoType7ProfileERN4llvm16FoldingSetNodeIDENS_8QualTypeEPKS4_jRKNS0_12ExtProtoInfoE", "clang::FunctionProtoType::Profile(llvm::FoldingSetNodeID&, clang::QualType, clang::QualType const*, unsigned int, clang::FunctionProtoType::ExtProtoInfo const&)"},
    {"_ZN5clang17FunctionProtoType7ProfileERN4llvm16FoldingSetNodeIDE", "clang::FunctionProtoType::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang14TypeOfExprTypeC1EPNS_4ExprENS_8QualTypeE", "clang::TypeOfExprType::TypeOfExprType(clang::Expr*, clang::QualType)"},
    {"_ZN5clang14TypeOfExprTypeC2EPNS_4ExprENS_8QualTypeE", "clang::TypeOfExprType::TypeOfExprType(clang::Expr*, clang::QualType)"},
    {"_ZN5clang23DependentTypeOfExprType7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextEPNS_4ExprE", "clang::DependentTypeOfExprType::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, clang::Expr*)"},
    {"_ZN5clang12DecltypeTypeC1EPNS_4ExprENS_8QualTypeES3_", "clang::DecltypeType::DecltypeType(clang::Expr*, clang::QualType, clang::QualType)"},
    {"_ZN5clang12DecltypeTypeC2EPNS_4ExprENS_8QualTypeES3_", "clang::DecltypeType::DecltypeType(clang::Expr*, clang::QualType, clang::QualType)"},
    {"_ZN5clang21DependentDecltypeTypeC1ERKNS_10ASTContextEPNS_4ExprE", "clang::DependentDecltypeType::DependentDecltypeType(clang::ASTContext const&, clang::Expr*)"},
    {"_ZN5clang21DependentDecltypeTypeC2ERKNS_10ASTContextEPNS_4ExprE", "clang::DependentDecltypeType::DependentDecltypeType(clang::ASTContext const&, clang::Expr*)"},
    {"_ZN5clang21DependentDecltypeType7ProfileERN4llvm16FoldingSetNodeIDERKNS_10ASTContextEPNS_4ExprE", "clang::DependentDecltypeType::Profile(llvm::FoldingSetNodeID&, clang::ASTContext const&, clang::Expr*)"},
    {"_ZN5clang7TagTypeC1ENS_4Type9TypeClassEPKNS_7TagDeclENS_8QualTypeE", "clang::TagType::TagType(clang::Type::TypeClass, clang::TagDecl const*, clang::QualType)"},
    {"_ZN5clang7TagTypeC2ENS_4Type9TypeClassEPKNS_7TagDeclENS_8QualTypeE", "clang::TagType::TagType(clang::Type::TypeClass, clang::TagDecl const*, clang::QualType)"},
    {"_ZNK5clang7TagType14isBeingDefinedEv", "clang::TagType::isBeingDefined() const"},
    {"_ZN5clang10RecordType7classofEPKNS_7TagTypeE", "clang::RecordType::classof(clang::TagType const*)"},
    {"_ZN5clang8EnumType7classofEPKNS_7TagTypeE", "clang::EnumType::classof(clang::TagType const*)"},
    {"_ZN5clang29SubstTemplateTypeParmPackTypeC1EPKNS_20TemplateTypeParmTypeENS_8QualTypeERKNS_16TemplateArgumentE", "clang::SubstTemplateTypeParmPackType::SubstTemplateTypeParmPackType(clang::TemplateTypeParmType const*, clang::QualType, clang::TemplateArgument const&)"},
    {"_ZN5clang29SubstTemplateTypeParmPackTypeC2EPKNS_20TemplateTypeParmTypeENS_8QualTypeERKNS_16TemplateArgumentE", "clang::SubstTemplateTypeParmPackType::SubstTemplateTypeParmPackType(clang::TemplateTypeParmType const*, clang::QualType, clang::TemplateArgument const&)"},
    {"_ZNK5clang29SubstTemplateTypeParmPackType15getArgumentPackEv", "clang::SubstTemplateTypeParmPackType::getArgumentPack() const"},
    {"_ZN5clang29SubstTemplateTypeParmPackType7ProfileERN4llvm16FoldingSetNodeIDE", "clang::SubstTemplateTypeParmPackType::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZN5clang29SubstTemplateTypeParmPackType7ProfileERN4llvm16FoldingSetNodeIDEPKNS_20TemplateTypeParmTypeERKNS_16TemplateArgumentE", "clang::SubstTemplateTypeParmPackType::Profile(llvm::FoldingSetNodeID&, clang::TemplateTypeParmType const*, clang::TemplateArgument const&)"},
    {"_ZN5clang26TemplateSpecializationType29anyDependentTemplateArgumentsERKNS_24TemplateArgumentListInfoE", "clang::TemplateSpecializationType::anyDependentTemplateArguments(clang::TemplateArgumentListInfo const&)"},
    {"_ZN5clang26TemplateSpecializationType29anyDependentTemplateArgumentsEPKNS_19TemplateArgumentLocEj", "clang::TemplateSpecializationType::anyDependentTemplateArguments(clang::TemplateArgumentLoc const*, unsigned int)"},
    {"_ZN5clang26TemplateSpecializationType29anyDependentTemplateArgumentsEPKNS_16TemplateArgumentEj", "clang::TemplateSpecializationType::anyDependentTemplateArguments(clang::TemplateArgument const*, unsigned int)"},
    {"_ZN5clang26TemplateSpecializationTypeC1ENS_12TemplateNameEPKNS_16TemplateArgumentEjNS_8QualTypeE", "clang::TemplateSpecializationType::TemplateSpecializationType(clang::TemplateName, clang::TemplateArgument const*, unsigned int, clang::QualType)"},
    {"_ZN5clang26TemplateSpecializationTypeC2ENS_12TemplateNameEPKNS_16TemplateArgumentEjNS_8QualTypeE", "clang::TemplateSpecializationType::TemplateSpecializationType(clang::TemplateName, clang::TemplateArgument const*, unsigned int, clang::QualType)"},
    {"_ZN5clang26TemplateSpecializationType7ProfileERN4llvm16FoldingSetNodeIDENS_12TemplateNameEPKNS_16TemplateArgumentEjRKNS_10ASTContextE", "clang::TemplateSpecializationType::Profile(llvm::FoldingSetNodeID&, clang::TemplateName, clang::TemplateArgument const*, unsigned int, clang::ASTContext const&)"},
    {"_ZNK5clang18QualifierCollector5applyERKNS_10ASTContextENS_8QualTypeE", "clang::QualifierCollector::apply(clang::ASTContext const&, clang::QualType) const"},
    {"_ZNK5clang18QualifierCollector5applyERKNS_10ASTContextEPKNS_4TypeE", "clang::QualifierCollector::apply(clang::ASTContext const&, clang::Type const*) const"},
    {"_ZN5clang18ObjCObjectTypeImpl7ProfileERN4llvm16FoldingSetNodeIDENS_8QualTypeEPKPNS_16ObjCProtocolDeclEj", "clang::ObjCObjectTypeImpl::Profile(llvm::FoldingSetNodeID&, clang::QualType, clang::ObjCProtocolDecl* const*, unsigned int)"},
    {"_ZN5clang18ObjCObjectTypeImpl7ProfileERN4llvm16FoldingSetNodeIDE", "clang::ObjCObjectTypeImpl::Profile(llvm::FoldingSetNodeID&)"},
    {"_ZNK5clang4Type10getLinkageEv", "clang::Type::getLinkage() const"},
    {"_ZN5clang17TypePropertyCacheIN12_GLOBAL__N_17PrivateEE6ensureEPKNS_4TypeE", "clang::TypePropertyCache<(anonymous namespace)::Private>::ensure(clang::Type const*)"},
    {"_ZNK5clang4Type13getVisibilityEv", "clang::Type::getVisibility() const"},
    {"_ZNK5clang4Type21hasUnnamedOrLocalTypeEv", "clang::Type::hasUnnamedOrLocalType() const"},
    {"_ZNK5clang4Type23getLinkageAndVisibilityEv", "clang::Type::getLinkageAndVisibility() const"},
    {"_ZN5clang4Type17ClearLinkageCacheEv", "clang::Type::ClearLinkageCache()"},
    {"_ZNK5clang4Type15hasSizedVLATypeEv", "clang::Type::hasSizedVLAType() const"},
    {"_ZNK4llvm6APSInt6extendEj", "llvm::APSInt::extend(unsigned int) const"},
    {"_ZN5clang7TypeLoc23getLocalSourceRangeImplES0_", "clang::TypeLoc::getLocalSourceRangeImpl(clang::TypeLoc)"},
    {"_ZN5clang7TypeLoc22getFullDataSizeForTypeENS_8QualTypeE", "clang::TypeLoc::getFullDataSizeForType(clang::QualType)"},
    {"_ZN5clang14TypeLocVisitorIN12_GLOBAL__N_19TypeSizerEjE5VisitENS_7TypeLocE", "clang::TypeLocVisitor<(anonymous namespace)::TypeSizer, unsigned int>::Visit(clang::TypeLoc)"},
    {"_ZN5clang7TypeLoc18getNextTypeLocImplES0_", "clang::TypeLoc::getNextTypeLocImpl(clang::TypeLoc)"},
    {"_ZN5clang7TypeLoc14initializeImplERNS_10ASTContextES0_NS_14SourceLocationE", "clang::TypeLoc::initializeImpl(clang::ASTContext&, clang::TypeLoc, clang::SourceLocation)"},
    {"_ZNK5clang7TypeLoc11getBeginLocEv", "clang::TypeLoc::getBeginLoc() const"},
    {"_ZNK5clang7TypeLoc9getEndLocEv", "clang::TypeLoc::getEndLoc() const"},
    {"_ZN5clang15TypeSpecTypeLoc7classofEPKNS_7TypeLocE", "clang::TypeSpecTypeLoc::classof(clang::TypeLoc const*)"},
    {"_ZNK5clang17TypeOfExprTypeLoc19getLocalSourceRangeEv", "clang::TypeOfExprTypeLoc::getLocalSourceRange() const"},
    {"_ZNK5clang14BuiltinTypeLoc18getWrittenTypeSpecEv", "clang::BuiltinTypeLoc::getWrittenTypeSpec() const"},
    {"_ZN5clang7TypeLoc16IgnoreParensImplES0_", "clang::TypeLoc::IgnoreParensImpl(clang::TypeLoc)"},
    {"_ZN5clang29TemplateSpecializationTypeLoc17initializeArgLocsERNS_10ASTContextEjPKNS_16TemplateArgumentEPNS_23TemplateArgumentLocInfoENS_14SourceLocationE", "clang::TemplateSpecializationTypeLoc::initializeArgLocs(clang::ASTContext&, unsigned int, clang::TemplateArgument const*, clang::TemplateArgumentLocInfo*, clang::SourceLocation)"},
    {"_ZN5clang29TemplateSpecializationTypeLoc15initializeLocalERNS_10ASTContextENS_14SourceLocationE", "clang::TemplateSpecializationTypeLoc::initializeLocal(clang::ASTContext&, clang::SourceLocation)"},
    {"_ZN5clang38DependentTemplateSpecializationTypeLoc15initializeLocalERNS_10ASTContextENS_14SourceLocationE", "clang::DependentTemplateSpecializationTypeLoc::initializeLocal(clang::ASTContext&, clang::SourceLocation)"},
    {"_ZN5clang26TemplateSpecializationType25PrintTemplateArgumentListERKNS_24TemplateArgumentListInfoERKNS_14PrintingPolicyE", "clang::TemplateSpecializationType::PrintTemplateArgumentList(clang::TemplateArgumentListInfo const&, clang::PrintingPolicy const&)"},
    {"_ZN5clang26TemplateSpecializationType25PrintTemplateArgumentListEPKNS_19TemplateArgumentLocEjRKNS_14PrintingPolicyE", "clang::TemplateSpecializationType::PrintTemplateArgumentList(clang::TemplateArgumentLoc const*, unsigned int, clang::PrintingPolicy const&)"},
    {"_ZN5clang26TemplateSpecializationType25PrintTemplateArgumentListEPKNS_16TemplateArgumentEjRKNS_14PrintingPolicyEb", "clang::TemplateSpecializationType::PrintTemplateArgumentList(clang::TemplateArgument const*, unsigned int, clang::PrintingPolicy const&, bool)"},
    {"_ZNK5clang8QualType4dumpEPKc", "clang::QualType::dump(char const*) const"},
    {"_ZNK5clang8QualType4dumpEv", "clang::QualType::dump() const"},
    {"_ZNK5clang4Type4dumpEv", "clang::Type::dump() const"},
    {"_ZNK5clang10Qualifiers11getAsStringEv", "clang::Qualifiers::getAsString() const"},
    {"_ZNK5clang10Qualifiers19getAsStringInternalERSsRKNS_14PrintingPolicyE", "clang::Qualifiers::getAsStringInternal(std::string&, clang::PrintingPolicy const&) const"},
    {"_Z18AppendTypeQualListRSsj", "AppendTypeQualList(std::string&, unsigned int)"},
    {"_ZN5clang8QualType11getAsStringEPKNS_4TypeENS_10QualifiersE", "clang::QualType::getAsString(clang::Type const*, clang::Qualifiers)"},
    {"_ZN5clang8QualType19getAsStringInternalEPKNS_4TypeENS_10QualifiersERSsRKNS_14PrintingPolicyE", "clang::QualType::getAsStringInternal(clang::Type const*, clang::Qualifiers, std::string&, clang::PrintingPolicy const&)"},
    {"_ZN12_GLOBAL__N_111TypePrinter5printEPKN5clang4TypeENS1_10QualifiersERSs", "(anonymous namespace)::TypePrinter::print(clang::Type const*, clang::Qualifiers, std::string&)"},
    {"_ZN12_GLOBAL__N_111TypePrinter21printTemplateTypeParmEPKN5clang20TemplateTypeParmTypeERSs", "(anonymous namespace)::TypePrinter::printTemplateTypeParm(clang::TemplateTypeParmType const*, std::string&)"},
    {"_ZN12_GLOBAL__N_111TypePrinter27printTemplateSpecializationEPKN5clang26TemplateSpecializationTypeERSs", "(anonymous namespace)::TypePrinter::printTemplateSpecialization(clang::TemplateSpecializationType const*, std::string&)"},
    {"_ZN12_GLOBAL__N_111TypePrinter8printTagEPN5clang7TagDeclERSs", "(anonymous namespace)::TypePrinter::printTag(clang::TagDecl*, std::string&)"},
    {"_ZN12_GLOBAL__N_111TypePrinter11AppendScopeEPN5clang11DeclContextERSs", "(anonymous namespace)::TypePrinter::AppendScope(clang::DeclContext*, std::string&)"},
    {"_Z13printTypeSpecPKN5clang9NamedDeclERSs", "printTypeSpec(clang::NamedDecl const*, std::string&)"},
    {"_ZN5clang9HeaderMap6CreateEPKNS_9FileEntryERNS_11FileManagerE", "clang::HeaderMap::Create(clang::FileEntry const*, clang::FileManager&)"},
    {"_ZN5clang9HeaderMapD1Ev", "clang::HeaderMap::~HeaderMap()"},
    {"_ZN5clang9HeaderMapD2Ev", "clang::HeaderMap::~HeaderMap()"},
    {"_ZNK5clang9HeaderMap11getFileNameEv", "clang::HeaderMap::getFileName() const"},
    {"_ZNK5clang9HeaderMap21getEndianAdjustedWordEj", "clang::HeaderMap::getEndianAdjustedWord(unsigned int) const"},
    {"_ZNK5clang9HeaderMap9getHeaderEv", "clang::HeaderMap::getHeader() const"},
    {"_ZNK5clang9HeaderMap9getBucketEj", "clang::HeaderMap::getBucket(unsigned int) const"},
    {"_ZNK5clang9HeaderMap9getStringEj", "clang::HeaderMap::getString(unsigned int) const"},
    {"_ZNK5clang9HeaderMap4dumpEv", "clang::HeaderMap::dump() const"},
    {"_ZNK5clang9HeaderMap10LookupFileEN4llvm9StringRefERNS_11FileManagerE", "clang::HeaderMap::LookupFile(llvm::StringRef, clang::FileManager&) const"},
    {"_ZN5clang14HeaderFileInfo19getControllingMacroEPNS_24ExternalIdentifierLookupE", "clang::HeaderFileInfo::getControllingMacro(clang::ExternalIdentifierLookup*)"},
    {"_ZN5clang12HeaderSearchC1ERNS_11FileManagerE", "clang::HeaderSearch::HeaderSearch(clang::FileManager&)"},
    {"_ZN5clang12HeaderSearchC2ERNS_11FileManagerE", "clang::HeaderSearch::HeaderSearch(clang::FileManager&)"},
    {"_ZN5clang12HeaderSearchD1Ev", "clang::HeaderSearch::~HeaderSearch()"},
    {"_ZN5clang12HeaderSearchD2Ev", "clang::HeaderSearch::~HeaderSearch()"},
    {"_ZN5clang12HeaderSearch10PrintStatsEv", "clang::HeaderSearch::PrintStats()"},
    {"_ZN5clang12HeaderSearch15CreateHeaderMapEPKNS_9FileEntryE", "clang::HeaderSearch::CreateHeaderMap(clang::FileEntry const*)"},
    {"_ZNK5clang15DirectoryLookup7getNameEv", "clang::DirectoryLookup::getName() const"},
    {"_ZNK5clang15DirectoryLookup10LookupFileEN4llvm9StringRefERNS_12HeaderSearchE", "clang::DirectoryLookup::LookupFile(llvm::StringRef, clang::HeaderSearch&) const"},
    {"_ZNK5clang15DirectoryLookup17DoFrameworkLookupEN4llvm9StringRefERNS_12HeaderSearchE", "clang::DirectoryLookup::DoFrameworkLookup(llvm::StringRef, clang::HeaderSearch&) const"},
    {"_ZN5clang12HeaderSearch10LookupFileEN4llvm9StringRefEbPKNS_15DirectoryLookupERS5_PKNS_9FileEntryE", "clang::HeaderSearch::LookupFile(llvm::StringRef, bool, clang::DirectoryLookup const*, clang::DirectoryLookup const*&, clang::FileEntry const*)"},
    {"_ZN5clang12HeaderSearch11getFileInfoEPKNS_9FileEntryE", "clang::HeaderSearch::getFileInfo(clang::FileEntry const*)"},
    {"_ZN5clang12HeaderSearch24LookupSubframeworkHeaderEN4llvm9StringRefEPKNS_9FileEntryE", "clang::HeaderSearch::LookupSubframeworkHeader(llvm::StringRef, clang::FileEntry const*)"},
    {"_ZN5clang12HeaderSearch23setHeaderFileInfoForUIDENS_14HeaderFileInfoEj", "clang::HeaderSearch::setHeaderFileInfoForUID(clang::HeaderFileInfo, unsigned int)"},
    {"_ZN5clang12HeaderSearch22ShouldEnterIncludeFileEPKNS_9FileEntryEb", "clang::HeaderSearch::ShouldEnterIncludeFile(clang::FileEntry const*, bool)"},
    {"_ZNSt6vectorIN5clang14HeaderFileInfoESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<clang::HeaderFileInfo, std::allocator<clang::HeaderFileInfo> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::HeaderFileInfo*, std::vector<clang::HeaderFileInfo, std::allocator<clang::HeaderFileInfo> > >, unsigned long, clang::HeaderFileInfo const&)"},
    {"_ZN4llvm15SmallVectorImplIcEaSERKS1_", "llvm::SmallVectorImpl<char>::operator=(llvm::SmallVectorImpl<char> const&)"},
    {"_ZN4llvm9StringMapISt4pairIjjENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<std::pair<unsigned int, unsigned int> >& llvm::StringMap<std::pair<unsigned int, unsigned int>, llvm::MallocAllocator>::GetOrCreateValue<std::pair<unsigned int, unsigned int> >(llvm::StringRef, std::pair<unsigned int, unsigned int>)"},
    {"_ZNSt6vectorISt4pairIPKN5clang9FileEntryEPKNS1_9HeaderMapEESaIS8_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS8_SA_EERKS8_", "std::vector<std::pair<clang::FileEntry const*, clang::HeaderMap const*>, std::allocator<std::pair<clang::FileEntry const*, clang::HeaderMap const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::FileEntry const*, clang::HeaderMap const*>*, std::vector<std::pair<clang::FileEntry const*, clang::HeaderMap const*>, std::allocator<std::pair<clang::FileEntry const*, clang::HeaderMap const*> > > >, std::pair<clang::FileEntry const*, clang::HeaderMap const*> const&)"},
    {"_ZN4llvm9StringMapIPKN5clang14DirectoryEntryENS_15MallocAllocatorEE16GetOrCreateValueIS4_EERNS_14StringMapEntryIS4_EENS_9StringRefET_", "llvm::StringMapEntry<clang::DirectoryEntry const*>& llvm::StringMap<clang::DirectoryEntry const*, llvm::MallocAllocator>::GetOrCreateValue<clang::DirectoryEntry const*>(llvm::StringRef, clang::DirectoryEntry const*)"},
    {"_ZNK5clang5Token15isObjCAtKeywordENS_3tok15ObjCKeywordKindE", "clang::Token::isObjCAtKeyword(clang::tok::ObjCKeywordKind) const"},
    {"_ZNK5clang5Token16getObjCKeywordIDEv", "clang::Token::getObjCKeywordID() const"},
    {"_ZN5clang5Lexer9InitLexerEPKcS2_S2_", "clang::Lexer::InitLexer(char const*, char const*, char const*)"},
    {"_ZN5clang5LexerC1ENS_6FileIDEPKN4llvm12MemoryBufferERNS_12PreprocessorE", "clang::Lexer::Lexer(clang::FileID, llvm::MemoryBuffer const*, clang::Preprocessor&)"},
    {"_ZN5clang5LexerC2ENS_6FileIDEPKN4llvm12MemoryBufferERNS_12PreprocessorE", "clang::Lexer::Lexer(clang::FileID, llvm::MemoryBuffer const*, clang::Preprocessor&)"},
    {"_ZN5clang5LexerC1ENS_14SourceLocationERKNS_11LangOptionsEPKcS6_S6_", "clang::Lexer::Lexer(clang::SourceLocation, clang::LangOptions const&, char const*, char const*, char const*)"},
    {"_ZN5clang5LexerC2ENS_14SourceLocationERKNS_11LangOptionsEPKcS6_S6_", "clang::Lexer::Lexer(clang::SourceLocation, clang::LangOptions const&, char const*, char const*, char const*)"},
    {"_ZN5clang5LexerC1ENS_6FileIDEPKN4llvm12MemoryBufferERKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::Lexer(clang::FileID, llvm::MemoryBuffer const*, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZN5clang5LexerC2ENS_6FileIDEPKN4llvm12MemoryBufferERKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::Lexer(clang::FileID, llvm::MemoryBuffer const*, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZN5clang5Lexer18Create_PragmaLexerENS_14SourceLocationES1_S1_jRNS_12PreprocessorE", "clang::Lexer::Create_PragmaLexer(clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, unsigned int, clang::Preprocessor&)"},
    {"_ZN5clang5Lexer9StringifyERKSsb", "clang::Lexer::Stringify(std::string const&, bool)"},
    {"_ZN5clang5Lexer9StringifyERN4llvm15SmallVectorImplIcEE", "clang::Lexer::Stringify(llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang5Lexer11getSpellingERKNS_5TokenERKNS_13SourceManagerERKNS_11LangOptionsEPb", "clang::Lexer::getSpelling(clang::Token const&, clang::SourceManager const&, clang::LangOptions const&, bool*)"},
    {"_ZN5clang5Lexer11getSpellingERKNS_5TokenERPKcRKNS_13SourceManagerERKNS_11LangOptionsEPb", "clang::Lexer::getSpelling(clang::Token const&, char const*&, clang::SourceManager const&, clang::LangOptions const&, bool*)"},
    {"_ZN5clang5Lexer18MeasureTokenLengthENS_14SourceLocationERKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::MeasureTokenLength(clang::SourceLocation, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZN5clang5Lexer19GetBeginningOfTokenENS_14SourceLocationERKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::GetBeginningOfToken(clang::SourceLocation, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZN5clang5Lexer15ComputePreambleEPKN4llvm12MemoryBufferEj", "clang::Lexer::ComputePreamble(llvm::MemoryBuffer const*, unsigned int)"},
    {"_ZN5clang5Lexer23AdvanceToTokenCharacterENS_14SourceLocationEjRKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::AdvanceToTokenCharacter(clang::SourceLocation, unsigned int, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZN5clang5Lexer19SkipEscapedNewLinesEPKc", "clang::Lexer::SkipEscapedNewLines(char const*)"},
    {"_ZN5clang5Lexer19getLocForEndOfTokenENS_14SourceLocationEjRKNS_13SourceManagerERKNS_11LangOptionsE", "clang::Lexer::getLocForEndOfToken(clang::SourceLocation, unsigned int, clang::SourceManager const&, clang::LangOptions const&)"},
    {"_ZNK5clang5Lexer17getSourceLocationEPKcj", "clang::Lexer::getSourceLocation(char const*, unsigned int) const"},
    {"_Z17GetMappedTokenLocRN5clang12PreprocessorENS_14SourceLocationEjj", "GetMappedTokenLoc(clang::Preprocessor&, clang::SourceLocation, unsigned int, unsigned int)"},
    {"_ZNK5clang5Lexer4DiagEPKcj", "clang::Lexer::Diag(char const*, unsigned int) const"},
    {"_ZN5clang5Lexer21getEscapedNewLineSizeEPKc", "clang::Lexer::getEscapedNewLineSize(char const*)"},
    {"_ZN5clang5Lexer18getCharAndSizeSlowEPKcRjPNS_5TokenE", "clang::Lexer::getCharAndSizeSlow(char const*, unsigned int&, clang::Token*)"},
    {"_ZN5clang5Lexer24getCharAndSizeSlowNoWarnEPKcRjRKNS_11LangOptionsE", "clang::Lexer::getCharAndSizeSlowNoWarn(char const*, unsigned int&, clang::LangOptions const&)"},
    {"_ZN5clang5Lexer9SkipBytesEjb", "clang::Lexer::SkipBytes(unsigned int, bool)"},
    {"_ZN5clang5Lexer13LexIdentifierERNS_5TokenEPKc", "clang::Lexer::LexIdentifier(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer18LexNumericConstantERNS_5TokenEPKc", "clang::Lexer::LexNumericConstant(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer16LexStringLiteralERNS_5TokenEPKcb", "clang::Lexer::LexStringLiteral(clang::Token&, char const*, bool)"},
    {"_ZN5clang5Lexer22LexAngledStringLiteralERNS_5TokenEPKc", "clang::Lexer::LexAngledStringLiteral(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer15LexCharConstantERNS_5TokenEPKc", "clang::Lexer::LexCharConstant(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer14SkipWhitespaceERNS_5TokenEPKc", "clang::Lexer::SkipWhitespace(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer15SkipBCPLCommentERNS_5TokenEPKc", "clang::Lexer::SkipBCPLComment(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer15SaveBCPLCommentERNS_5TokenEPKc", "clang::Lexer::SaveBCPLComment(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer16SkipBlockCommentERNS_5TokenEPKc", "clang::Lexer::SkipBlockComment(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer15ReadToEndOfLineEv", "clang::Lexer::ReadToEndOfLine()"},
    {"_ZN5clang5Lexer12LexEndOfFileERNS_5TokenEPKc", "clang::Lexer::LexEndOfFile(clang::Token&, char const*)"},
    {"_ZN5clang5Lexer19isNextPPTokenLParenEv", "clang::Lexer::isNextPPTokenLParen()"},
    {"_ZN5clang5Lexer16LexTokenInternalERNS_5TokenE", "clang::Lexer::LexTokenInternal(clang::Token&)"},
    {"_ZN5clang5Lexer23IsStartOfConflictMarkerEPKc", "clang::Lexer::IsStartOfConflictMarker(char const*)"},
    {"_ZN5clang5Lexer25HandleEndOfConflictMarkerEPKc", "clang::Lexer::HandleEndOfConflictMarker(char const*)"},
    {"_ZN5clang20NumericLiteralParserC1EPKcS2_NS_14SourceLocationERNS_12PreprocessorE", "clang::NumericLiteralParser::NumericLiteralParser(char const*, char const*, clang::SourceLocation, clang::Preprocessor&)"},
    {"_ZN5clang20NumericLiteralParserC2EPKcS2_NS_14SourceLocationERNS_12PreprocessorE", "clang::NumericLiteralParser::NumericLiteralParser(char const*, char const*, clang::SourceLocation, clang::Preprocessor&)"},
    {"_ZN5clang20NumericLiteralParser27ParseNumberStartingWithZeroENS_14SourceLocationE", "clang::NumericLiteralParser::ParseNumberStartingWithZero(clang::SourceLocation)"},
    {"_ZN5clang20NumericLiteralParser15GetIntegerValueERN4llvm5APIntE", "clang::NumericLiteralParser::GetIntegerValue(llvm::APInt&)"},
    {"_ZN5clang20NumericLiteralParser13GetFloatValueERN4llvm7APFloatE", "clang::NumericLiteralParser::GetFloatValue(llvm::APFloat&)"},
    {"_ZN5clang17CharLiteralParserC1EPKcS2_NS_14SourceLocationERNS_12PreprocessorE", "clang::CharLiteralParser::CharLiteralParser(char const*, char const*, clang::SourceLocation, clang::Preprocessor&)"},
    {"_ZN5clang17CharLiteralParserC2EPKcS2_NS_14SourceLocationERNS_12PreprocessorE", "clang::CharLiteralParser::CharLiteralParser(char const*, char const*, clang::SourceLocation, clang::Preprocessor&)"},
    {"_Z16ProcessUCNEscapeRPKcS0_RjRtN5clang13FullSourceLocEPNS4_10DiagnosticERKNS4_11LangOptionsE", "ProcessUCNEscape(char const*&, char const*, unsigned int&, unsigned short&, clang::FullSourceLoc, clang::Diagnostic*, clang::LangOptions const&)"},
    {"_Z17ProcessCharEscapeRPKcS0_RbN5clang13FullSourceLocEbPNS3_10DiagnosticERKNS3_10TargetInfoE", "ProcessCharEscape(char const*&, char const*, bool&, clang::FullSourceLoc, bool, clang::Diagnostic*, clang::TargetInfo const&)"},
    {"_ZN5clang19StringLiteralParserC1EPKNS_5TokenEjRNS_12PreprocessorEb", "clang::StringLiteralParser::StringLiteralParser(clang::Token const*, unsigned int, clang::Preprocessor&, bool)"},
    {"_ZN5clang19StringLiteralParserC2EPKNS_5TokenEjRNS_12PreprocessorEb", "clang::StringLiteralParser::StringLiteralParser(clang::Token const*, unsigned int, clang::Preprocessor&, bool)"},
    {"_ZN5clang19StringLiteralParser4initEPKNS_5TokenEj", "clang::StringLiteralParser::init(clang::Token const*, unsigned int)"},
    {"_ZNK5clang19StringLiteralParser21getOffsetOfStringByteERKNS_5TokenEj", "clang::StringLiteralParser::getOffsetOfStringByte(clang::Token const&, unsigned int) const"},
    {"_ZN5clang9MacroArgs6createEPKNS_9MacroInfoEPKNS_5TokenEjbRNS_12PreprocessorE", "clang::MacroArgs::create(clang::MacroInfo const*, clang::Token const*, unsigned int, bool, clang::Preprocessor&)"},
    {"_ZNK5clang9MacroArgs16getUnexpArgumentEj", "clang::MacroArgs::getUnexpArgument(unsigned int) const"},
    {"_ZN5clang9MacroArgs7destroyERNS_12PreprocessorE", "clang::MacroArgs::destroy(clang::Preprocessor&)"},
    {"_ZN5clang9MacroArgs10deallocateEv", "clang::MacroArgs::deallocate()"},
    {"_ZN5clang9MacroArgs12getArgLengthEPKNS_5TokenE", "clang::MacroArgs::getArgLength(clang::Token const*)"},
    {"_ZNK5clang9MacroArgs20ArgNeedsPreexpansionEPKNS_5TokenERNS_12PreprocessorE", "clang::MacroArgs::ArgNeedsPreexpansion(clang::Token const*, clang::Preprocessor&) const"},
    {"_ZN5clang9MacroArgs17getPreExpArgumentEjPKNS_9MacroInfoERNS_12PreprocessorE", "clang::MacroArgs::getPreExpArgument(unsigned int, clang::MacroInfo const*, clang::Preprocessor&)"},
    {"_ZN5clang9MacroArgs17StringifyArgumentEPKNS_5TokenERNS_12PreprocessorEb", "clang::MacroArgs::StringifyArgument(clang::Token const*, clang::Preprocessor&, bool)"},
    {"_ZN5clang9MacroArgs22getStringifiedArgumentEjRNS_12PreprocessorE", "clang::MacroArgs::getStringifiedArgument(unsigned int, clang::Preprocessor&)"},
    {"_ZNSt6vectorIN5clang5TokenESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<clang::Token, std::allocator<clang::Token> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::Token*, std::vector<clang::Token, std::allocator<clang::Token> > >, unsigned long, clang::Token const&)"},
    {"_ZNSt6vectorIN5clang5TokenESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::Token, std::allocator<clang::Token> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Token*, std::vector<clang::Token, std::allocator<clang::Token> > >, clang::Token const&)"},
    {"_ZNSt6vectorIS_IN5clang5TokenESaIS1_EESaIS3_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS3_S5_EEmRKS3_", "std::vector<std::vector<clang::Token, std::allocator<clang::Token> >, std::allocator<std::vector<clang::Token, std::allocator<clang::Token> > > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::vector<clang::Token, std::allocator<clang::Token> >*, std::vector<std::vector<clang::Token, std::allocator<clang::Token> >, std::allocator<std::vector<clang::Token, std::allocator<clang::Token> > > > >, unsigned long, std::vector<clang::Token, std::allocator<clang::Token> > const&)"},
    {"_ZNSt6vectorIN5clang5TokenESaIS1_EEaSERKS3_", "std::vector<clang::Token, std::allocator<clang::Token> >::operator=(std::vector<clang::Token, std::allocator<clang::Token> > const&)"},
    {"_ZNSt6vectorIN5clang5TokenESaIS1_EEC2ERKS3_", "std::vector<clang::Token, std::allocator<clang::Token> >::vector(std::vector<clang::Token, std::allocator<clang::Token> > const&)"},
    {"_ZN5clang9MacroInfoC1ENS_14SourceLocationE", "clang::MacroInfo::MacroInfo(clang::SourceLocation)"},
    {"_ZN5clang9MacroInfoC2ENS_14SourceLocationE", "clang::MacroInfo::MacroInfo(clang::SourceLocation)"},
    {"_ZN5clang9MacroInfoC1ERKS0_RN4llvm16BumpPtrAllocatorE", "clang::MacroInfo::MacroInfo(clang::MacroInfo const&, llvm::BumpPtrAllocator&)"},
    {"_ZN5clang9MacroInfoC2ERKS0_RN4llvm16BumpPtrAllocatorE", "clang::MacroInfo::MacroInfo(clang::MacroInfo const&, llvm::BumpPtrAllocator&)"},
    {"_ZNK5clang9MacroInfo13isIdenticalToERKS0_RNS_12PreprocessorE", "clang::MacroInfo::isIdenticalTo(clang::MacroInfo const&, clang::Preprocessor&) const"},
    {"_ZN4llvm15SmallVectorImplIN5clang5TokenEEaSERKS3_", "llvm::SmallVectorImpl<clang::Token>::operator=(llvm::SmallVectorImpl<clang::Token> const&)"},
    {"_ZN5clang12Preprocessor24EnableBacktrackAtThisPosEv", "clang::Preprocessor::EnableBacktrackAtThisPos()"},
    {"_ZN5clang12Preprocessor19EnterCachingLexModeEv", "clang::Preprocessor::EnterCachingLexMode()"},
    {"_ZN5clang12Preprocessor23CommitBacktrackedTokensEv", "clang::Preprocessor::CommitBacktrackedTokens()"},
    {"_ZN5clang12Preprocessor9BacktrackEv", "clang::Preprocessor::Backtrack()"},
    {"_ZN5clang12Preprocessor10CachingLexERNS_5TokenE", "clang::Preprocessor::CachingLex(clang::Token&)"},
    {"_ZN5clang12Preprocessor9PeekAheadEj", "clang::Preprocessor::PeekAhead(unsigned int)"},
    {"_ZN5clang12Preprocessor28AnnotatePreviousCachedTokensERKNS_5TokenE", "clang::Preprocessor::AnnotatePreviousCachedTokens(clang::Token const&)"},
    {"_ZNSt6vectorImSaImEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPmS1_EERKm", "std::vector<unsigned long, std::allocator<unsigned long> >::_M_insert_aux(__gnu_cxx::__normal_iterator<unsigned long*, std::vector<unsigned long, std::allocator<unsigned long> > >, unsigned long const&)"},
    {"_ZN5clang12Preprocessor17AllocateMacroInfoEv", "clang::Preprocessor::AllocateMacroInfo()"},
    {"_ZN5clang12Preprocessor17AllocateMacroInfoENS_14SourceLocationE", "clang::Preprocessor::AllocateMacroInfo(clang::SourceLocation)"},
    {"_ZN5clang12Preprocessor14CloneMacroInfoERKNS_9MacroInfoE", "clang::Preprocessor::CloneMacroInfo(clang::MacroInfo const&)"},
    {"_ZN5clang12Preprocessor16ReleaseMacroInfoEPNS_9MacroInfoE", "clang::Preprocessor::ReleaseMacroInfo(clang::MacroInfo*)"},
    {"_ZN5clang12Preprocessor26DiscardUntilEndOfDirectiveEv", "clang::Preprocessor::DiscardUntilEndOfDirective()"},
    {"_ZN5clang12Preprocessor13ReadMacroNameERNS_5TokenEc", "clang::Preprocessor::ReadMacroName(clang::Token&, char)"},
    {"_ZN5clang12Preprocessor19CheckEndOfDirectiveEPKcb", "clang::Preprocessor::CheckEndOfDirective(char const*, bool)"},
    {"_ZN5clang12Preprocessor28SkipExcludedConditionalBlockENS_14SourceLocationEbb", "clang::Preprocessor::SkipExcludedConditionalBlock(clang::SourceLocation, bool, bool)"},
    {"_ZN5clang12Preprocessor31PTHSkipExcludedConditionalBlockEv", "clang::Preprocessor::PTHSkipExcludedConditionalBlock()"},
    {"_ZN5clang12Preprocessor10LookupFileEN4llvm9StringRefEbPKNS_15DirectoryLookupERS5_", "clang::Preprocessor::LookupFile(llvm::StringRef, bool, clang::DirectoryLookup const*, clang::DirectoryLookup const*&)"},
    {"_ZN5clang12Preprocessor15HandleDirectiveERNS_5TokenE", "clang::Preprocessor::HandleDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor20HandleDigitDirectiveERNS_5TokenE", "clang::Preprocessor::HandleDigitDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor17HandleIfDirectiveERNS_5TokenEb", "clang::Preprocessor::HandleIfDirective(clang::Token&, bool)"},
    {"_ZN5clang12Preprocessor20HandleIfdefDirectiveERNS_5TokenEbb", "clang::Preprocessor::HandleIfdefDirective(clang::Token&, bool, bool)"},
    {"_ZN5clang12Preprocessor19HandleElifDirectiveERNS_5TokenE", "clang::Preprocessor::HandleElifDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor19HandleElseDirectiveERNS_5TokenE", "clang::Preprocessor::HandleElseDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor20HandleEndifDirectiveERNS_5TokenE", "clang::Preprocessor::HandleEndifDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor22HandleIncludeDirectiveENS_14SourceLocationERNS_5TokenEPKNS_15DirectoryLookupEb", "clang::Preprocessor::HandleIncludeDirective(clang::SourceLocation, clang::Token&, clang::DirectoryLookup const*, bool)"},
    {"_ZN5clang12Preprocessor28HandleIncludeMacrosDirectiveENS_14SourceLocationERNS_5TokenE", "clang::Preprocessor::HandleIncludeMacrosDirective(clang::SourceLocation, clang::Token&)"},
    {"_ZN5clang12Preprocessor21HandleDefineDirectiveERNS_5TokenE", "clang::Preprocessor::HandleDefineDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor20HandleUndefDirectiveERNS_5TokenE", "clang::Preprocessor::HandleUndefDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor19HandleLineDirectiveERNS_5TokenE", "clang::Preprocessor::HandleLineDirective(clang::Token&)"},
    {"_ZN5clang12Preprocessor29HandleUserDiagnosticDirectiveERNS_5TokenEb", "clang::Preprocessor::HandleUserDiagnosticDirective(clang::Token&, bool)"},
    {"_ZN5clang12Preprocessor21HandleImportDirectiveENS_14SourceLocationERNS_5TokenE", "clang::Preprocessor::HandleImportDirective(clang::SourceLocation, clang::Token&)"},
    {"_ZN5clang12Preprocessor26HandleIncludeNextDirectiveENS_14SourceLocationERNS_5TokenE", "clang::Preprocessor::HandleIncludeNextDirective(clang::SourceLocation, clang::Token&)"},
    {"_ZN5clang12Preprocessor24HandleIdentSCCSDirectiveERNS_5TokenE", "clang::Preprocessor::HandleIdentSCCSDirective(clang::Token&)"},
    {"_Z12GetLineValueRN5clang5TokenERjjRNS_12PreprocessorE", "GetLineValue(clang::Token&, unsigned int&, unsigned int, clang::Preprocessor&)"},
    {"_ZN5clang12Preprocessor26GetIncludeFilenameSpellingENS_14SourceLocationERN4llvm9StringRefE", "clang::Preprocessor::GetIncludeFilenameSpelling(clang::SourceLocation, llvm::StringRef&)"},
    {"_ZN5clang12Preprocessor22ConcatenateIncludeNameERN4llvm11SmallStringILj128EEERNS_14SourceLocationE", "clang::Preprocessor::ConcatenateIncludeName(llvm::SmallString<128u>&, clang::SourceLocation&)"},
    {"_ZN5clang12Preprocessor26ReadMacroDefinitionArgListEPNS_9MacroInfoE", "clang::Preprocessor::ReadMacroDefinitionArgList(clang::MacroInfo*)"},
    {"_ZSt6__findIPPN5clang14IdentifierInfoES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "clang::IdentifierInfo** std::__find<clang::IdentifierInfo**, clang::IdentifierInfo*>(clang::IdentifierInfo**, clang::IdentifierInfo**, clang::IdentifierInfo* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN5clang17PPConditionalInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<clang::PPConditionalInfo, false>::grow(unsigned long)"},
    {"_ZN5clang12Preprocessor27EvaluateDirectiveExpressionERPNS_14IdentifierInfoE", "clang::Preprocessor::EvaluateDirectiveExpression(clang::IdentifierInfo*&)"},
    {"_Z13EvaluateValueRN12_GLOBAL__N_17PPValueERN5clang5TokenER14DefinedTrackerbRNS2_12PreprocessorE", "EvaluateValue((anonymous namespace)::PPValue&, clang::Token&, DefinedTracker&, bool, clang::Preprocessor&)"},
    {"_Z24EvaluateDirectiveSubExprRN12_GLOBAL__N_17PPValueEjRN5clang5TokenEbRNS2_12PreprocessorE", "EvaluateDirectiveSubExpr((anonymous namespace)::PPValue&, unsigned int, clang::Token&, bool, clang::Preprocessor&)"},
    {"_ZN5clang11PPCallbacksD0Ev", "clang::PPCallbacks::~PPCallbacks()"},
    {"_ZN5clang11PPCallbacksD1Ev", "clang::PPCallbacks::~PPCallbacks()"},
    {"_ZN5clang11PPCallbacksD2Ev", "clang::PPCallbacks::~PPCallbacks()"},
    {"_ZNK5clang12Preprocessor15isInPrimaryFileEv", "clang::Preprocessor::isInPrimaryFile() const"},
    {"_ZNK5clang12Preprocessor19getCurrentFileLexerEv", "clang::Preprocessor::getCurrentFileLexer() const"},
    {"_ZN5clang12Preprocessor15EnterSourceFileENS_6FileIDEPKNS_15DirectoryLookupENS_14SourceLocationE", "clang::Preprocessor::EnterSourceFile(clang::FileID, clang::DirectoryLookup const*, clang::SourceLocation)"},
    {"_ZN5clang12Preprocessor22EnterSourceFileWithPTHEPNS_8PTHLexerEPKNS_15DirectoryLookupE", "clang::Preprocessor::EnterSourceFileWithPTH(clang::PTHLexer*, clang::DirectoryLookup const*)"},
    {"_ZN5clang12Preprocessor24EnterSourceFileWithLexerEPNS_5LexerEPKNS_15DirectoryLookupE", "clang::Preprocessor::EnterSourceFileWithLexer(clang::Lexer*, clang::DirectoryLookup const*)"},
    {"_ZN5clang12Preprocessor10EnterMacroERNS_5TokenENS_14SourceLocationEPNS_9MacroArgsE", "clang::Preprocessor::EnterMacro(clang::Token&, clang::SourceLocation, clang::MacroArgs*)"},
    {"_ZN5clang12Preprocessor16EnterTokenStreamEPKNS_5TokenEjbb", "clang::Preprocessor::EnterTokenStream(clang::Token const*, unsigned int, bool, bool)"},
    {"_ZN5clang12Preprocessor15HandleEndOfFileERNS_5TokenEb", "clang::Preprocessor::HandleEndOfFile(clang::Token&, bool)"},
    {"_ZN5clang12Preprocessor21RemoveTopOfLexerStackEv", "clang::Preprocessor::RemoveTopOfLexerStack()"},
    {"_ZN5clang12Preprocessor21HandleEndOfTokenLexerERNS_5TokenE", "clang::Preprocessor::HandleEndOfTokenLexer(clang::Token&)"},
    {"_ZN5clang12Preprocessor27HandleMicrosoftCommentPasteERNS_5TokenE", "clang::Preprocessor::HandleMicrosoftCommentPaste(clang::Token&)"},
    {"_ZNK5clang13SourceManager9getBufferENS_6FileIDENS_14SourceLocationEPb", "clang::SourceManager::getBuffer(clang::FileID, clang::SourceLocation, bool*) const"},
    {"_ZN5clang12Preprocessor20PopIncludeMacroStackEv", "clang::Preprocessor::PopIncludeMacroStack()"},
    {"_ZNSt6vectorIN5clang12Preprocessor16IncludeStackInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::Preprocessor::IncludeStackInfo, std::allocator<clang::Preprocessor::IncludeStackInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Preprocessor::IncludeStackInfo*, std::vector<clang::Preprocessor::IncludeStackInfo, std::allocator<clang::Preprocessor::IncludeStackInfo> > >, clang::Preprocessor::IncludeStackInfo const&)"},
    {"_ZNK5clang12Preprocessor15getInfoForMacroEPNS_14IdentifierInfoE", "clang::Preprocessor::getInfoForMacro(clang::IdentifierInfo*) const"},
    {"_ZN5clang12Preprocessor12setMacroInfoEPNS_14IdentifierInfoEPNS_9MacroInfoE", "clang::Preprocessor::setMacroInfo(clang::IdentifierInfo*, clang::MacroInfo*)"},
    {"_ZN5clang12Preprocessor21RegisterBuiltinMacrosEv", "clang::Preprocessor::RegisterBuiltinMacros()"},
    {"_ZN5clang12Preprocessor19isNextPPTokenLParenEv", "clang::Preprocessor::isNextPPTokenLParen()"},
    {"_ZN5clang12Preprocessor29HandleMacroExpandedIdentifierERNS_5TokenEPNS_9MacroInfoE", "clang::Preprocessor::HandleMacroExpandedIdentifier(clang::Token&, clang::MacroInfo*)"},
    {"_ZN5clang12Preprocessor18ExpandBuiltinMacroERNS_5TokenE", "clang::Preprocessor::ExpandBuiltinMacro(clang::Token&)"},
    {"_ZN5clang12Preprocessor25ReadFunctionLikeMacroArgsERNS_5TokenEPNS_9MacroInfoERNS_14SourceLocationE", "clang::Preprocessor::ReadFunctionLikeMacroArgs(clang::Token&, clang::MacroInfo*, clang::SourceLocation&)"},
    {"_ZN5clang12Preprocessor15markMacroAsUsedEPNS_9MacroInfoE", "clang::Preprocessor::markMacroAsUsed(clang::MacroInfo*)"},
    {"_Z16ComputeDATE_TIMERN5clang14SourceLocationES1_RNS_12PreprocessorE", "ComputeDATE_TIME(clang::SourceLocation&, clang::SourceLocation&, clang::Preprocessor&)"},
    {"_Z24EvaluateHasIncludeCommonRN5clang5TokenEPNS_14IdentifierInfoERNS_12PreprocessorEPKNS_15DirectoryLookupE", "EvaluateHasIncludeCommon(clang::Token&, clang::IdentifierInfo*, clang::Preprocessor&, clang::DirectoryLookup const*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_9MacroInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<clang::IdentifierInfo*, clang::MacroInfo*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::MacroInfo*> >::InsertIntoBucket(clang::IdentifierInfo* const&, clang::MacroInfo* const&, std::pair<clang::IdentifierInfo*, clang::MacroInfo*>*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoEPNS1_9MacroInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, clang::MacroInfo*, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<clang::MacroInfo*> >::grow(unsigned int)"},
    {"_ZN5clang8PTHLexerC1ERNS_12PreprocessorENS_6FileIDEPKhS5_RNS_10PTHManagerE", "clang::PTHLexer::PTHLexer(clang::Preprocessor&, clang::FileID, unsigned char const*, unsigned char const*, clang::PTHManager&)"},
    {"_ZN5clang8PTHLexerC2ERNS_12PreprocessorENS_6FileIDEPKhS5_RNS_10PTHManagerE", "clang::PTHLexer::PTHLexer(clang::Preprocessor&, clang::FileID, unsigned char const*, unsigned char const*, clang::PTHManager&)"},
    {"_ZN5clang8PTHLexer3LexERNS_5TokenE", "clang::PTHLexer::Lex(clang::Token&)"},
    {"_ZN5clang8PTHLexer12LexEndOfFileERNS_5TokenE", "clang::PTHLexer::LexEndOfFile(clang::Token&)"},
    {"_ZN5clang8PTHLexer6getEOFERNS_5TokenE", "clang::PTHLexer::getEOF(clang::Token&)"},
    {"_ZN5clang8PTHLexer18DiscardToEndOfLineEv", "clang::PTHLexer::DiscardToEndOfLine()"},
    {"_ZN5clang8PTHLexer9SkipBlockEv", "clang::PTHLexer::SkipBlock()"},
    {"_ZN5clang8PTHLexer17getSourceLocationEv", "clang::PTHLexer::getSourceLocation()"},
    {"_ZN5clang10PTHManagerC1EPKN4llvm12MemoryBufferEPvPKhPPNS_14IdentifierInfoES5_jS7_PKc", "clang::PTHManager::PTHManager(llvm::MemoryBuffer const*, void*, unsigned char const*, clang::IdentifierInfo**, void*, unsigned int, unsigned char const*, char const*)"},
    {"_ZN5clang10PTHManagerC2EPKN4llvm12MemoryBufferEPvPKhPPNS_14IdentifierInfoES5_jS7_PKc", "clang::PTHManager::PTHManager(llvm::MemoryBuffer const*, void*, unsigned char const*, clang::IdentifierInfo**, void*, unsigned int, unsigned char const*, char const*)"},
    {"_ZN5clang10PTHManagerD0Ev", "clang::PTHManager::~PTHManager()"},
    {"_ZN5clang10PTHManagerD1Ev", "clang::PTHManager::~PTHManager()"},
    {"_ZN5clang10PTHManagerD2Ev", "clang::PTHManager::~PTHManager()"},
    {"_ZN5clang10PTHManager6CreateERKSsRNS_10DiagnosticE", "clang::PTHManager::Create(std::string const&, clang::Diagnostic&)"},
    {"_ZN5clang10PTHManager26LazilyCreateIdentifierInfoEj", "clang::PTHManager::LazilyCreateIdentifierInfo(unsigned int)"},
    {"_ZN5clang10PTHManager3getEN4llvm9StringRefE", "clang::PTHManager::get(llvm::StringRef)"},
    {"_ZN5clang10PTHManager11CreateLexerENS_6FileIDE", "clang::PTHManager::CreateLexer(clang::FileID)"},
    {"_ZN5clang10PTHManager15createStatCacheEv", "clang::PTHManager::createStatCache()"},
    {"_ZN12_GLOBAL__N_112PTHStatCacheD1Ev", "(anonymous namespace)::PTHStatCache::~PTHStatCache()"},
    {"_ZN12_GLOBAL__N_112PTHStatCacheD0Ev", "(anonymous namespace)::PTHStatCache::~PTHStatCache()"},
    {"_ZN12_GLOBAL__N_112PTHStatCache7getStatEPKcR4statPi", "(anonymous namespace)::PTHStatCache::getStat(char const*, stat&, int*)"},
    {"_ZN5clang8PTHLexerD1Ev", "clang::PTHLexer::~PTHLexer()"},
    {"_ZN5clang8PTHLexerD0Ev", "clang::PTHLexer::~PTHLexer()"},
    {"_ZN5clang8PTHLexer11IndirectLexERNS_5TokenE", "clang::PTHLexer::IndirectLex(clang::Token&)"},
    {"_ZN5clang13PragmaHandlerD0Ev", "clang::PragmaHandler::~PragmaHandler()"},
    {"_ZN5clang13PragmaHandlerD1Ev", "clang::PragmaHandler::~PragmaHandler()"},
    {"_ZN5clang13PragmaHandlerD2Ev", "clang::PragmaHandler::~PragmaHandler()"},
    {"_ZN5clang18EmptyPragmaHandlerC1Ev", "clang::EmptyPragmaHandler::EmptyPragmaHandler()"},
    {"_ZN5clang18EmptyPragmaHandlerC2Ev", "clang::EmptyPragmaHandler::EmptyPragmaHandler()"},
    {"_ZN5clang18EmptyPragmaHandler12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::EmptyPragmaHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang15PragmaNamespaceD0Ev", "clang::PragmaNamespace::~PragmaNamespace()"},
    {"_ZN5clang15PragmaNamespaceD1Ev", "clang::PragmaNamespace::~PragmaNamespace()"},
    {"_ZN5clang15PragmaNamespaceD2Ev", "clang::PragmaNamespace::~PragmaNamespace()"},
    {"_ZNK5clang15PragmaNamespace11FindHandlerEN4llvm9StringRefEb", "clang::PragmaNamespace::FindHandler(llvm::StringRef, bool) const"},
    {"_ZN5clang15PragmaNamespace9AddPragmaEPNS_13PragmaHandlerE", "clang::PragmaNamespace::AddPragma(clang::PragmaHandler*)"},
    {"_ZN5clang15PragmaNamespace19RemovePragmaHandlerEPNS_13PragmaHandlerE", "clang::PragmaNamespace::RemovePragmaHandler(clang::PragmaHandler*)"},
    {"_ZN5clang15PragmaNamespace12HandlePragmaERNS_12PreprocessorENS_20PragmaIntroducerKindERNS_5TokenE", "clang::PragmaNamespace::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang12Preprocessor21HandlePragmaDirectiveEj", "clang::Preprocessor::HandlePragmaDirective(unsigned int)"},
    {"_ZN5clang12Preprocessor13Handle_PragmaERNS_5TokenE", "clang::Preprocessor::Handle_Pragma(clang::Token&)"},
    {"_ZN5clang12Preprocessor13Handle_PragmaEjRKSsNS_14SourceLocationES3_", "clang::Preprocessor::Handle_Pragma(unsigned int, std::string const&, clang::SourceLocation, clang::SourceLocation)"},
    {"_ZN5clang12Preprocessor23HandleMicrosoft__pragmaERNS_5TokenE", "clang::Preprocessor::HandleMicrosoft__pragma(clang::Token&)"},
    {"_ZN5clang12Preprocessor16HandlePragmaOnceERNS_5TokenE", "clang::Preprocessor::HandlePragmaOnce(clang::Token&)"},
    {"_ZN5clang12Preprocessor16HandlePragmaMarkEv", "clang::Preprocessor::HandlePragmaMark()"},
    {"_ZN5clang12Preprocessor18HandlePragmaPoisonERNS_5TokenE", "clang::Preprocessor::HandlePragmaPoison(clang::Token&)"},
    {"_ZN5clang12Preprocessor24HandlePragmaSystemHeaderERNS_5TokenE", "clang::Preprocessor::HandlePragmaSystemHeader(clang::Token&)"},
    {"_ZN5clang12Preprocessor22HandlePragmaDependencyERNS_5TokenE", "clang::Preprocessor::HandlePragmaDependency(clang::Token&)"},
    {"_ZN5clang12Preprocessor19HandlePragmaCommentERNS_5TokenE", "clang::Preprocessor::HandlePragmaComment(clang::Token&)"},
    {"_ZN5clang12Preprocessor19HandlePragmaMessageERNS_5TokenE", "clang::Preprocessor::HandlePragmaMessage(clang::Token&)"},
    {"_ZN5clang12Preprocessor25ParsePragmaPushOrPopMacroERNS_5TokenE", "clang::Preprocessor::ParsePragmaPushOrPopMacro(clang::Token&)"},
    {"_ZN5clang12Preprocessor21HandlePragmaPushMacroERNS_5TokenE", "clang::Preprocessor::HandlePragmaPushMacro(clang::Token&)"},
    {"_ZN5clang12Preprocessor20HandlePragmaPopMacroERNS_5TokenE", "clang::Preprocessor::HandlePragmaPopMacro(clang::Token&)"},
    {"_ZN5clang12Preprocessor16AddPragmaHandlerEN4llvm9StringRefEPNS_13PragmaHandlerE", "clang::Preprocessor::AddPragmaHandler(llvm::StringRef, clang::PragmaHandler*)"},
    {"_ZN5clang12Preprocessor19RemovePragmaHandlerEN4llvm9StringRefEPNS_13PragmaHandlerE", "clang::Preprocessor::RemovePragmaHandler(llvm::StringRef, clang::PragmaHandler*)"},
    {"_ZN5clang12Preprocessor22RegisterBuiltinPragmasEv", "clang::Preprocessor::RegisterBuiltinPragmas()"},
    {"_ZN12_GLOBAL__N_120PragmaCommentHandlerD1Ev", "(anonymous namespace)::PragmaCommentHandler::~PragmaCommentHandler()"},
    {"_ZN12_GLOBAL__N_120PragmaCommentHandlerD0Ev", "(anonymous namespace)::PragmaCommentHandler::~PragmaCommentHandler()"},
    {"_ZN12_GLOBAL__N_120PragmaCommentHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaCommentHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_125PragmaSTDC_UnknownHandlerD1Ev", "(anonymous namespace)::PragmaSTDC_UnknownHandler::~PragmaSTDC_UnknownHandler()"},
    {"_ZN12_GLOBAL__N_125PragmaSTDC_UnknownHandlerD0Ev", "(anonymous namespace)::PragmaSTDC_UnknownHandler::~PragmaSTDC_UnknownHandler()"},
    {"_ZN12_GLOBAL__N_125PragmaSTDC_UnknownHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaSTDC_UnknownHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_134PragmaSTDC_CX_LIMITED_RANGEHandlerD1Ev", "(anonymous namespace)::PragmaSTDC_CX_LIMITED_RANGEHandler::~PragmaSTDC_CX_LIMITED_RANGEHandler()"},
    {"_ZN12_GLOBAL__N_134PragmaSTDC_CX_LIMITED_RANGEHandlerD0Ev", "(anonymous namespace)::PragmaSTDC_CX_LIMITED_RANGEHandler::~PragmaSTDC_CX_LIMITED_RANGEHandler()"},
    {"_ZN12_GLOBAL__N_134PragmaSTDC_CX_LIMITED_RANGEHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaSTDC_CX_LIMITED_RANGEHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_114LexOnOffSwitchERN5clang12PreprocessorE", "(anonymous namespace)::LexOnOffSwitch(clang::Preprocessor&)"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FENV_ACCESSHandlerD1Ev", "(anonymous namespace)::PragmaSTDC_FENV_ACCESSHandler::~PragmaSTDC_FENV_ACCESSHandler()"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FENV_ACCESSHandlerD0Ev", "(anonymous namespace)::PragmaSTDC_FENV_ACCESSHandler::~PragmaSTDC_FENV_ACCESSHandler()"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FENV_ACCESSHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaSTDC_FENV_ACCESSHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FP_CONTRACTHandlerD1Ev", "(anonymous namespace)::PragmaSTDC_FP_CONTRACTHandler::~PragmaSTDC_FP_CONTRACTHandler()"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FP_CONTRACTHandlerD0Ev", "(anonymous namespace)::PragmaSTDC_FP_CONTRACTHandler::~PragmaSTDC_FP_CONTRACTHandler()"},
    {"_ZN12_GLOBAL__N_129PragmaSTDC_FP_CONTRACTHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaSTDC_FP_CONTRACTHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_118PragmaDebugHandlerD1Ev", "(anonymous namespace)::PragmaDebugHandler::~PragmaDebugHandler()"},
    {"_ZN12_GLOBAL__N_118PragmaDebugHandlerD0Ev", "(anonymous namespace)::PragmaDebugHandler::~PragmaDebugHandler()"},
    {"_ZN12_GLOBAL__N_118PragmaDebugHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaDebugHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_123PragmaDiagnosticHandlerD1Ev", "(anonymous namespace)::PragmaDiagnosticHandler::~PragmaDiagnosticHandler()"},
    {"_ZN12_GLOBAL__N_123PragmaDiagnosticHandlerD0Ev", "(anonymous namespace)::PragmaDiagnosticHandler::~PragmaDiagnosticHandler()"},
    {"_ZN12_GLOBAL__N_123PragmaDiagnosticHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaDiagnosticHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_123PragmaDependencyHandlerD1Ev", "(anonymous namespace)::PragmaDependencyHandler::~PragmaDependencyHandler()"},
    {"_ZN12_GLOBAL__N_123PragmaDependencyHandlerD0Ev", "(anonymous namespace)::PragmaDependencyHandler::~PragmaDependencyHandler()"},
    {"_ZN12_GLOBAL__N_123PragmaDependencyHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaDependencyHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_125PragmaSystemHeaderHandlerD1Ev", "(anonymous namespace)::PragmaSystemHeaderHandler::~PragmaSystemHeaderHandler()"},
    {"_ZN12_GLOBAL__N_125PragmaSystemHeaderHandlerD0Ev", "(anonymous namespace)::PragmaSystemHeaderHandler::~PragmaSystemHeaderHandler()"},
    {"_ZN12_GLOBAL__N_125PragmaSystemHeaderHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaSystemHeaderHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_119PragmaPoisonHandlerD1Ev", "(anonymous namespace)::PragmaPoisonHandler::~PragmaPoisonHandler()"},
    {"_ZN12_GLOBAL__N_119PragmaPoisonHandlerD0Ev", "(anonymous namespace)::PragmaPoisonHandler::~PragmaPoisonHandler()"},
    {"_ZN12_GLOBAL__N_119PragmaPoisonHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaPoisonHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_120PragmaMessageHandlerD1Ev", "(anonymous namespace)::PragmaMessageHandler::~PragmaMessageHandler()"},
    {"_ZN12_GLOBAL__N_120PragmaMessageHandlerD0Ev", "(anonymous namespace)::PragmaMessageHandler::~PragmaMessageHandler()"},
    {"_ZN12_GLOBAL__N_120PragmaMessageHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaMessageHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_121PragmaPopMacroHandlerD1Ev", "(anonymous namespace)::PragmaPopMacroHandler::~PragmaPopMacroHandler()"},
    {"_ZN12_GLOBAL__N_121PragmaPopMacroHandlerD0Ev", "(anonymous namespace)::PragmaPopMacroHandler::~PragmaPopMacroHandler()"},
    {"_ZN12_GLOBAL__N_121PragmaPopMacroHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaPopMacroHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_122PragmaPushMacroHandlerD1Ev", "(anonymous namespace)::PragmaPushMacroHandler::~PragmaPushMacroHandler()"},
    {"_ZN12_GLOBAL__N_122PragmaPushMacroHandlerD0Ev", "(anonymous namespace)::PragmaPushMacroHandler::~PragmaPushMacroHandler()"},
    {"_ZN12_GLOBAL__N_122PragmaPushMacroHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaPushMacroHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_117PragmaMarkHandlerD1Ev", "(anonymous namespace)::PragmaMarkHandler::~PragmaMarkHandler()"},
    {"_ZN12_GLOBAL__N_117PragmaMarkHandlerD0Ev", "(anonymous namespace)::PragmaMarkHandler::~PragmaMarkHandler()"},
    {"_ZN12_GLOBAL__N_117PragmaMarkHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaMarkHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN12_GLOBAL__N_117PragmaOnceHandlerD1Ev", "(anonymous namespace)::PragmaOnceHandler::~PragmaOnceHandler()"},
    {"_ZN12_GLOBAL__N_117PragmaOnceHandlerD0Ev", "(anonymous namespace)::PragmaOnceHandler::~PragmaOnceHandler()"},
    {"_ZN12_GLOBAL__N_117PragmaOnceHandler12HandlePragmaERN5clang12PreprocessorENS1_20PragmaIntroducerKindERNS1_5TokenE", "(anonymous namespace)::PragmaOnceHandler::HandlePragma(clang::Preprocessor&, clang::PragmaIntroducerKind, clang::Token&)"},
    {"_ZN5clang18EmptyPragmaHandlerD1Ev", "clang::EmptyPragmaHandler::~EmptyPragmaHandler()"},
    {"_ZN5clang18EmptyPragmaHandlerD0Ev", "clang::EmptyPragmaHandler::~EmptyPragmaHandler()"},
    {"_ZNSt6vectorIPN5clang9MacroInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::MacroInfo**, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > >, clang::MacroInfo* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoESt6vectorIPNS1_9MacroInfoESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16FindAndConstructERKS3_", "llvm::DenseMap<clang::IdentifierInfo*, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> >, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > > >::FindAndConstruct(clang::IdentifierInfo* const&)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoESt6vectorIPNS1_9MacroInfoESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16InsertIntoBucketERKS3_RKS8_PSt4pairIS3_S8_E", "llvm::DenseMap<clang::IdentifierInfo*, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> >, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > > >::InsertIntoBucket(clang::IdentifierInfo* const&, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > const&, std::pair<clang::IdentifierInfo*, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > >*)"},
    {"_ZN4llvm8DenseMapIPN5clang14IdentifierInfoESt6vectorIPNS1_9MacroInfoESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE4growEj", "llvm::DenseMap<clang::IdentifierInfo*, std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> >, llvm::DenseMapInfo<clang::IdentifierInfo*>, llvm::DenseMapInfo<std::vector<clang::MacroInfo*, std::allocator<clang::MacroInfo*> > > >::grow(unsigned int)"},
    {"_ZN4llvm9StringMapIPN5clang13PragmaHandlerENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<clang::PragmaHandler*>& llvm::StringMap<clang::PragmaHandler*, llvm::MallocAllocator>::GetOrCreateValue<clang::PragmaHandler*>(llvm::StringRef, clang::PragmaHandler*)"},
    {"_ZN5clang33ExternalPreprocessingRecordSourceD0Ev", "clang::ExternalPreprocessingRecordSource::~ExternalPreprocessingRecordSource()"},
    {"_ZN5clang33ExternalPreprocessingRecordSourceD1Ev", "clang::ExternalPreprocessingRecordSource::~ExternalPreprocessingRecordSource()"},
    {"_ZN5clang33ExternalPreprocessingRecordSourceD2Ev", "clang::ExternalPreprocessingRecordSource::~ExternalPreprocessingRecordSource()"},
    {"_ZN5clang18InclusionDirectiveC1ERNS_19PreprocessingRecordENS0_13InclusionKindEN4llvm9StringRefEbPKNS_9FileEntryENS_11SourceRangeE", "clang::InclusionDirective::InclusionDirective(clang::PreprocessingRecord&, clang::InclusionDirective::InclusionKind, llvm::StringRef, bool, clang::FileEntry const*, clang::SourceRange)"},
    {"_ZN5clang18InclusionDirectiveC2ERNS_19PreprocessingRecordENS0_13InclusionKindEN4llvm9StringRefEbPKNS_9FileEntryENS_11SourceRangeE", "clang::InclusionDirective::InclusionDirective(clang::PreprocessingRecord&, clang::InclusionDirective::InclusionKind, llvm::StringRef, bool, clang::FileEntry const*, clang::SourceRange)"},
    {"_ZNK5clang19PreprocessingRecord29MaybeLoadPreallocatedEntitiesEv", "clang::PreprocessingRecord::MaybeLoadPreallocatedEntities() const"},
    {"_ZN5clang19PreprocessingRecordC1Ev", "clang::PreprocessingRecord::PreprocessingRecord()"},
    {"_ZN5clang19PreprocessingRecordC2Ev", "clang::PreprocessingRecord::PreprocessingRecord()"},
    {"_ZN5clang19PreprocessingRecord5beginEb", "clang::PreprocessingRecord::begin(bool)"},
    {"_ZN5clang19PreprocessingRecord3endEb", "clang::PreprocessingRecord::end(bool)"},
    {"_ZNK5clang19PreprocessingRecord5beginEb", "clang::PreprocessingRecord::begin(bool) const"},
    {"_ZNK5clang19PreprocessingRecord3endEb", "clang::PreprocessingRecord::end(bool) const"},
    {"_ZN5clang19PreprocessingRecord21addPreprocessedEntityEPNS_18PreprocessedEntityE", "clang::PreprocessingRecord::addPreprocessedEntity(clang::PreprocessedEntity*)"},
    {"_ZN5clang19PreprocessingRecord17SetExternalSourceERNS_33ExternalPreprocessingRecordSourceEj", "clang::PreprocessingRecord::SetExternalSource(clang::ExternalPreprocessingRecordSource&, unsigned int)"},
    {"_ZN5clang19PreprocessingRecord21SetPreallocatedEntityEjPNS_18PreprocessedEntityE", "clang::PreprocessingRecord::SetPreallocatedEntity(unsigned int, clang::PreprocessedEntity*)"},
    {"_ZN5clang19PreprocessingRecord23RegisterMacroDefinitionEPNS_9MacroInfoEPNS_15MacroDefinitionE", "clang::PreprocessingRecord::RegisterMacroDefinition(clang::MacroInfo*, clang::MacroDefinition*)"},
    {"_ZN5clang19PreprocessingRecord19findMacroDefinitionEPKNS_9MacroInfoE", "clang::PreprocessingRecord::findMacroDefinition(clang::MacroInfo const*)"},
    {"_ZN5clang19PreprocessingRecord12MacroExpandsERKNS_5TokenEPKNS_9MacroInfoE", "clang::PreprocessingRecord::MacroExpands(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang19PreprocessingRecord12MacroDefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PreprocessingRecord::MacroDefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang19PreprocessingRecord14MacroUndefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PreprocessingRecord::MacroUndefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang19PreprocessingRecord18InclusionDirectiveENS_14SourceLocationERKNS_5TokenEN4llvm9StringRefEbPKNS_9FileEntryES1_", "clang::PreprocessingRecord::InclusionDirective(clang::SourceLocation, clang::Token const&, llvm::StringRef, bool, clang::FileEntry const*, clang::SourceLocation)"},
    {"_ZN5clang19PreprocessingRecordD1Ev", "clang::PreprocessingRecord::~PreprocessingRecord()"},
    {"_ZN5clang19PreprocessingRecordD0Ev", "clang::PreprocessingRecord::~PreprocessingRecord()"},
    {"_ZN4llvm8DenseMapIPKN5clang9MacroInfoEPNS1_15MacroDefinitionENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PSt4pairIS4_S6_E", "llvm::DenseMap<clang::MacroInfo const*, clang::MacroDefinition*, llvm::DenseMapInfo<clang::MacroInfo const*>, llvm::DenseMapInfo<clang::MacroDefinition*> >::InsertIntoBucket(clang::MacroInfo const* const&, clang::MacroDefinition* const&, std::pair<clang::MacroInfo const*, clang::MacroDefinition*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9MacroInfoEPNS1_15MacroDefinitionENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<clang::MacroInfo const*, clang::MacroDefinition*, llvm::DenseMapInfo<clang::MacroInfo const*>, llvm::DenseMapInfo<clang::MacroDefinition*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN5clang18PreprocessedEntityESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::PreprocessedEntity**, std::vector<clang::PreprocessedEntity*, std::allocator<clang::PreprocessedEntity*> > >, unsigned long, clang::PreprocessedEntity* const&)"},
    {"_ZN5clang26ExternalPreprocessorSourceD0Ev", "clang::ExternalPreprocessorSource::~ExternalPreprocessorSource()"},
    {"_ZN5clang26ExternalPreprocessorSourceD1Ev", "clang::ExternalPreprocessorSource::~ExternalPreprocessorSource()"},
    {"_ZN5clang26ExternalPreprocessorSourceD2Ev", "clang::ExternalPreprocessorSource::~ExternalPreprocessorSource()"},
    {"_ZN5clang12PreprocessorC1ERNS_10DiagnosticERKNS_11LangOptionsERKNS_10TargetInfoERNS_13SourceManagerERNS_12HeaderSearchEPNS_20IdentifierInfoLookupEb", "clang::Preprocessor::Preprocessor(clang::Diagnostic&, clang::LangOptions const&, clang::TargetInfo const&, clang::SourceManager&, clang::HeaderSearch&, clang::IdentifierInfoLookup*, bool)"},
    {"_ZN5clang12PreprocessorC2ERNS_10DiagnosticERKNS_11LangOptionsERKNS_10TargetInfoERNS_13SourceManagerERNS_12HeaderSearchEPNS_20IdentifierInfoLookupEb", "clang::Preprocessor::Preprocessor(clang::Diagnostic&, clang::LangOptions const&, clang::TargetInfo const&, clang::SourceManager&, clang::HeaderSearch&, clang::IdentifierInfoLookup*, bool)"},
    {"_ZN5clang12PreprocessorD1Ev", "clang::Preprocessor::~Preprocessor()"},
    {"_ZN5clang12PreprocessorD2Ev", "clang::Preprocessor::~Preprocessor()"},
    {"_ZN5clang12Preprocessor13setPTHManagerEPNS_10PTHManagerE", "clang::Preprocessor::setPTHManager(clang::PTHManager*)"},
    {"_ZNK5clang12Preprocessor9DumpTokenERKNS_5TokenEb", "clang::Preprocessor::DumpToken(clang::Token const&, bool) const"},
    {"_ZNK5clang12Preprocessor12DumpLocationENS_14SourceLocationE", "clang::Preprocessor::DumpLocation(clang::SourceLocation) const"},
    {"_ZNK5clang12Preprocessor9DumpMacroERKNS_9MacroInfoE", "clang::Preprocessor::DumpMacro(clang::MacroInfo const&) const"},
    {"_ZN5clang12Preprocessor10PrintStatsEv", "clang::Preprocessor::PrintStats()"},
    {"_ZNK5clang12Preprocessor11macro_beginEb", "clang::Preprocessor::macro_begin(bool) const"},
    {"_ZNK5clang12Preprocessor9macro_endEb", "clang::Preprocessor::macro_end(bool) const"},
    {"_ZN5clang12Preprocessor22SetCodeCompletionPointEPKNS_9FileEntryEjj", "clang::Preprocessor::SetCodeCompletionPoint(clang::FileEntry const*, unsigned int, unsigned int)"},
    {"_ZNK5clang12Preprocessor20isCodeCompletionFileENS_14SourceLocationE", "clang::Preprocessor::isCodeCompletionFile(clang::SourceLocation) const"},
    {"_ZN5clang12Preprocessor27CodeCompleteNaturalLanguageEv", "clang::Preprocessor::CodeCompleteNaturalLanguage()"},
    {"_ZNK5clang12Preprocessor11getSpellingERKNS_5TokenERN4llvm15SmallVectorImplIcEEPb", "clang::Preprocessor::getSpelling(clang::Token const&, llvm::SmallVectorImpl<char>&, bool*) const"},
    {"_ZN5clang12Preprocessor12CreateStringEPKcjRNS_5TokenENS_14SourceLocationE", "clang::Preprocessor::CreateString(char const*, unsigned int, clang::Token&, clang::SourceLocation)"},
    {"_ZN5clang12Preprocessor19EnterMainSourceFileEv", "clang::Preprocessor::EnterMainSourceFile()"},
    {"_ZN5clang12Preprocessor13EndSourceFileEv", "clang::Preprocessor::EndSourceFile()"},
    {"_ZNK5clang12Preprocessor20LookUpIdentifierInfoERNS_5TokenE", "clang::Preprocessor::LookUpIdentifierInfo(clang::Token&) const"},
    {"_ZN5clang12Preprocessor16HandleIdentifierERNS_5TokenE", "clang::Preprocessor::HandleIdentifier(clang::Token&)"},
    {"_ZN5clang12Preprocessor17AddCommentHandlerEPNS_14CommentHandlerE", "clang::Preprocessor::AddCommentHandler(clang::CommentHandler*)"},
    {"_ZN5clang12Preprocessor20RemoveCommentHandlerEPNS_14CommentHandlerE", "clang::Preprocessor::RemoveCommentHandler(clang::CommentHandler*)"},
    {"_ZN5clang12Preprocessor13HandleCommentERNS_5TokenENS_11SourceRangeE", "clang::Preprocessor::HandleComment(clang::Token&, clang::SourceRange)"},
    {"_ZN5clang14CommentHandlerD0Ev", "clang::CommentHandler::~CommentHandler()"},
    {"_ZN5clang14CommentHandlerD1Ev", "clang::CommentHandler::~CommentHandler()"},
    {"_ZN5clang14CommentHandlerD2Ev", "clang::CommentHandler::~CommentHandler()"},
    {"_ZN5clang21CodeCompletionHandlerD0Ev", "clang::CodeCompletionHandler::~CodeCompletionHandler()"},
    {"_ZN5clang21CodeCompletionHandlerD1Ev", "clang::CodeCompletionHandler::~CodeCompletionHandler()"},
    {"_ZN5clang21CodeCompletionHandlerD2Ev", "clang::CodeCompletionHandler::~CodeCompletionHandler()"},
    {"_ZN5clang12Preprocessor25createPreprocessingRecordEv", "clang::Preprocessor::createPreprocessingRecord()"},
    {"_ZN5clang21CodeCompletionHandler21CodeCompleteDirectiveEb", "clang::CodeCompletionHandler::CodeCompleteDirective(bool)"},
    {"_ZN5clang21CodeCompletionHandler34CodeCompleteInConditionalExclusionEv", "clang::CodeCompletionHandler::CodeCompleteInConditionalExclusion()"},
    {"_ZN5clang21CodeCompletionHandler21CodeCompleteMacroNameEb", "clang::CodeCompletionHandler::CodeCompleteMacroName(bool)"},
    {"_ZN5clang21CodeCompletionHandler34CodeCompletePreprocessorExpressionEv", "clang::CodeCompletionHandler::CodeCompletePreprocessorExpression()"},
    {"_ZN5clang21CodeCompletionHandler25CodeCompleteMacroArgumentEPNS_14IdentifierInfoEPNS_9MacroInfoEj", "clang::CodeCompletionHandler::CodeCompleteMacroArgument(clang::IdentifierInfo*, clang::MacroInfo*, unsigned int)"},
    {"_ZN5clang21CodeCompletionHandler27CodeCompleteNaturalLanguageEv", "clang::CodeCompletionHandler::CodeCompleteNaturalLanguage()"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN5clang14CommentHandlerESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<clang::CommentHandler**, std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> > > std::__find<__gnu_cxx::__normal_iterator<clang::CommentHandler**, std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> > >, clang::CommentHandler*>(__gnu_cxx::__normal_iterator<clang::CommentHandler**, std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> > >, __gnu_cxx::__normal_iterator<clang::CommentHandler**, std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> > >, clang::CommentHandler* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN5clang14CommentHandlerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CommentHandler**, std::vector<clang::CommentHandler*, std::allocator<clang::CommentHandler*> > >, clang::CommentHandler* const&)"},
    {"_ZN5clang18PPChainedCallbacksD1Ev", "clang::PPChainedCallbacks::~PPChainedCallbacks()"},
    {"_ZN5clang18PPChainedCallbacksD0Ev", "clang::PPChainedCallbacks::~PPChainedCallbacks()"},
    {"_ZN5clang18PPChainedCallbacks11FileChangedENS_14SourceLocationENS_11PPCallbacks16FileChangeReasonENS_6SrcMgr18CharacteristicKindE", "clang::PPChainedCallbacks::FileChanged(clang::SourceLocation, clang::PPCallbacks::FileChangeReason, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang18PPChainedCallbacks11FileSkippedERKNS_9FileEntryERKNS_5TokenENS_6SrcMgr18CharacteristicKindE", "clang::PPChainedCallbacks::FileSkipped(clang::FileEntry const&, clang::Token const&, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang11PPCallbacks18InclusionDirectiveENS_14SourceLocationERKNS_5TokenEN4llvm9StringRefEbPKNS_9FileEntryES1_", "clang::PPCallbacks::InclusionDirective(clang::SourceLocation, clang::Token const&, llvm::StringRef, bool, clang::FileEntry const*, clang::SourceLocation)"},
    {"_ZN5clang18PPChainedCallbacks13EndOfMainFileEv", "clang::PPChainedCallbacks::EndOfMainFile()"},
    {"_ZN5clang18PPChainedCallbacks5IdentENS_14SourceLocationERKSs", "clang::PPChainedCallbacks::Ident(clang::SourceLocation, std::string const&)"},
    {"_ZN5clang18PPChainedCallbacks13PragmaCommentENS_14SourceLocationEPKNS_14IdentifierInfoERKSs", "clang::PPChainedCallbacks::PragmaComment(clang::SourceLocation, clang::IdentifierInfo const*, std::string const&)"},
    {"_ZN5clang18PPChainedCallbacks13PragmaMessageENS_14SourceLocationEN4llvm9StringRefE", "clang::PPChainedCallbacks::PragmaMessage(clang::SourceLocation, llvm::StringRef)"},
    {"_ZN5clang18PPChainedCallbacks12MacroExpandsERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPChainedCallbacks::MacroExpands(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang18PPChainedCallbacks12MacroDefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPChainedCallbacks::MacroDefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang18PPChainedCallbacks14MacroUndefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPChainedCallbacks::MacroUndefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang18PPChainedCallbacks2IfENS_11SourceRangeE", "clang::PPChainedCallbacks::If(clang::SourceRange)"},
    {"_ZN5clang18PPChainedCallbacks4ElifENS_11SourceRangeE", "clang::PPChainedCallbacks::Elif(clang::SourceRange)"},
    {"_ZN5clang18PPChainedCallbacks5IfdefERKNS_5TokenE", "clang::PPChainedCallbacks::Ifdef(clang::Token const&)"},
    {"_ZN5clang18PPChainedCallbacks6IfndefERKNS_5TokenE", "clang::PPChainedCallbacks::Ifndef(clang::Token const&)"},
    {"_ZN5clang18PPChainedCallbacks4ElseEv", "clang::PPChainedCallbacks::Else()"},
    {"_ZN5clang18PPChainedCallbacks5EndifEv", "clang::PPChainedCallbacks::Endif()"},
    {"_ZN5clang11PPCallbacks11FileChangedENS_14SourceLocationENS0_16FileChangeReasonENS_6SrcMgr18CharacteristicKindE", "clang::PPCallbacks::FileChanged(clang::SourceLocation, clang::PPCallbacks::FileChangeReason, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang11PPCallbacks11FileSkippedERKNS_9FileEntryERKNS_5TokenENS_6SrcMgr18CharacteristicKindE", "clang::PPCallbacks::FileSkipped(clang::FileEntry const&, clang::Token const&, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang11PPCallbacks13EndOfMainFileEv", "clang::PPCallbacks::EndOfMainFile()"},
    {"_ZN5clang11PPCallbacks5IdentENS_14SourceLocationERKSs", "clang::PPCallbacks::Ident(clang::SourceLocation, std::string const&)"},
    {"_ZN5clang11PPCallbacks13PragmaCommentENS_14SourceLocationEPKNS_14IdentifierInfoERKSs", "clang::PPCallbacks::PragmaComment(clang::SourceLocation, clang::IdentifierInfo const*, std::string const&)"},
    {"_ZN5clang11PPCallbacks13PragmaMessageENS_14SourceLocationEN4llvm9StringRefE", "clang::PPCallbacks::PragmaMessage(clang::SourceLocation, llvm::StringRef)"},
    {"_ZN5clang11PPCallbacks12MacroExpandsERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPCallbacks::MacroExpands(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang11PPCallbacks12MacroDefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPCallbacks::MacroDefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang11PPCallbacks14MacroUndefinedERKNS_5TokenEPKNS_9MacroInfoE", "clang::PPCallbacks::MacroUndefined(clang::Token const&, clang::MacroInfo const*)"},
    {"_ZN5clang11PPCallbacks2IfENS_11SourceRangeE", "clang::PPCallbacks::If(clang::SourceRange)"},
    {"_ZN5clang11PPCallbacks4ElifENS_11SourceRangeE", "clang::PPCallbacks::Elif(clang::SourceRange)"},
    {"_ZN5clang11PPCallbacks5IfdefERKNS_5TokenE", "clang::PPCallbacks::Ifdef(clang::Token const&)"},
    {"_ZN5clang11PPCallbacks6IfndefERKNS_5TokenE", "clang::PPCallbacks::Ifndef(clang::Token const&)"},
    {"_ZN5clang11PPCallbacks4ElseEv", "clang::PPCallbacks::Else()"},
    {"_ZN5clang11PPCallbacks5EndifEv", "clang::PPCallbacks::Endif()"},
    {"_ZN5clang15PragmaNamespace14getIfNamespaceEv", "clang::PragmaNamespace::getIfNamespace()"},
    {"_ZN5clang13PragmaHandler14getIfNamespaceEv", "clang::PragmaHandler::getIfNamespace()"},
    {"_ZN5clang15IdentifierTable3getEN4llvm9StringRefE", "clang::IdentifierTable::get(llvm::StringRef)"},
    {"_ZN4llvm9StringMapIPN5clang14IdentifierInfoENS_16BumpPtrAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<clang::IdentifierInfo*>& llvm::StringMap<clang::IdentifierInfo*, llvm::BumpPtrAllocator>::GetOrCreateValue<clang::IdentifierInfo*>(llvm::StringRef, clang::IdentifierInfo*)"},
    {"_ZN5clang11LangOptionsC2ERKS0_", "clang::LangOptions::LangOptions(clang::LangOptions const&)"},
    {"_ZN5clang17PreprocessorLexer18LexIncludeFilenameERNS_5TokenE", "clang::PreprocessorLexer::LexIncludeFilename(clang::Token&)"},
    {"_ZNK5clang17PreprocessorLexer12getFileEntryEv", "clang::PreprocessorLexer::getFileEntry() const"},
    {"_ZN5clang13ScratchBufferC1ERNS_13SourceManagerE", "clang::ScratchBuffer::ScratchBuffer(clang::SourceManager&)"},
    {"_ZN5clang13ScratchBufferC2ERNS_13SourceManagerE", "clang::ScratchBuffer::ScratchBuffer(clang::SourceManager&)"},
    {"_ZN5clang13ScratchBuffer8getTokenEPKcjRS2_", "clang::ScratchBuffer::getToken(char const*, unsigned int, char const*&)"},
    {"_ZN5clang13ScratchBuffer18AllocScratchBufferEj", "clang::ScratchBuffer::AllocScratchBuffer(unsigned int)"},
    {"_ZNK5clang18TokenConcatenation11StartsWithLERKNS_5TokenE", "clang::TokenConcatenation::StartsWithL(clang::Token const&) const"},
    {"_ZNK5clang18TokenConcatenation13IsIdentifierLERKNS_5TokenE", "clang::TokenConcatenation::IsIdentifierL(clang::Token const&) const"},
    {"_ZN5clang18TokenConcatenationC1ERNS_12PreprocessorE", "clang::TokenConcatenation::TokenConcatenation(clang::Preprocessor&)"},
    {"_ZN5clang18TokenConcatenationC2ERNS_12PreprocessorE", "clang::TokenConcatenation::TokenConcatenation(clang::Preprocessor&)"},
    {"_ZNK5clang18TokenConcatenation11AvoidConcatERKNS_5TokenES3_S3_", "clang::TokenConcatenation::AvoidConcat(clang::Token const&, clang::Token const&, clang::Token const&) const"},
    {"_Z12GetFirstCharRN5clang12PreprocessorERKNS_5TokenE", "GetFirstChar(clang::Preprocessor&, clang::Token const&)"},
    {"_ZN5clang10TokenLexer4InitERNS_5TokenENS_14SourceLocationEPNS_9MacroArgsE", "clang::TokenLexer::Init(clang::Token&, clang::SourceLocation, clang::MacroArgs*)"},
    {"_ZN5clang10TokenLexer7destroyEv", "clang::TokenLexer::destroy()"},
    {"_ZN5clang10TokenLexer23ExpandFunctionArgumentsEv", "clang::TokenLexer::ExpandFunctionArguments()"},
    {"_ZN5clang10TokenLexer4InitEPKNS_5TokenEjbb", "clang::TokenLexer::Init(clang::Token const*, unsigned int, bool, bool)"},
    {"_ZN5clang10TokenLexer3LexERNS_5TokenE", "clang::TokenLexer::Lex(clang::Token&)"},
    {"_ZN5clang10TokenLexer11PasteTokensERNS_5TokenE", "clang::TokenLexer::PasteTokens(clang::Token&)"},
    {"_ZN5clang10TokenLexer27HandleMicrosoftCommentPasteERNS_5TokenE", "clang::TokenLexer::HandleMicrosoftCommentPaste(clang::Token&)"},
    {"_ZNK5clang10TokenLexer17isNextTokenLParenEv", "clang::TokenLexer::isNextTokenLParen() const"},
    {"_ZNK5clang7Builtin7Context9GetRecordEj", "clang::Builtin::Context::GetRecord(unsigned int) const"},
    {"_ZN5clang7Builtin7ContextC1ERKNS_10TargetInfoE", "clang::Builtin::Context::Context(clang::TargetInfo const&)"},
    {"_ZN5clang7Builtin7ContextC2ERKNS_10TargetInfoE", "clang::Builtin::Context::Context(clang::TargetInfo const&)"},
    {"_ZN5clang7Builtin7Context18InitializeBuiltinsERNS_15IdentifierTableERKNS_11LangOptionsE", "clang::Builtin::Context::InitializeBuiltins(clang::IdentifierTable&, clang::LangOptions const&)"},
    {"_ZN5clang7Builtin7Context15GetBuiltinNamesERN4llvm15SmallVectorImplIPKcEEb", "clang::Builtin::Context::GetBuiltinNames(llvm::SmallVectorImpl<char const*>&, bool)"},
    {"_ZN5clang7Builtin7Context13ForgetBuiltinEjRNS_15IdentifierTableE", "clang::Builtin::Context::ForgetBuiltin(unsigned int, clang::IdentifierTable&)"},
    {"_ZN5clang7Builtin7Context12isPrintfLikeEjRjRb", "clang::Builtin::Context::isPrintfLike(unsigned int, unsigned int&, bool&)"},
    {"_ZN5clang7Builtin7Context11isScanfLikeEjRjRb", "clang::Builtin::Context::isScanfLike(unsigned int, unsigned int&, bool&)"},
    {"_ZN5clang10DiagnosticC1ERKN4llvm18IntrusiveRefCntPtrINS_13DiagnosticIDsEEEPNS_16DiagnosticClientEb", "clang::Diagnostic::Diagnostic(llvm::IntrusiveRefCntPtr<clang::DiagnosticIDs> const&, clang::DiagnosticClient*, bool)"},
    {"_ZN5clang10DiagnosticC2ERKN4llvm18IntrusiveRefCntPtrINS_13DiagnosticIDsEEEPNS_16DiagnosticClientEb", "clang::Diagnostic::Diagnostic(llvm::IntrusiveRefCntPtr<clang::DiagnosticIDs> const&, clang::DiagnosticClient*, bool)"},
    {"_Z18DummyArgToStringFnN5clang10Diagnostic12ArgumentKindElPKcjS3_jPKSt4pairIS1_lEjRN4llvm15SmallVectorImplIcEEPv", "DummyArgToStringFn(clang::Diagnostic::ArgumentKind, long, char const*, unsigned int, char const*, unsigned int, std::pair<clang::Diagnostic::ArgumentKind, long> const*, unsigned int, llvm::SmallVectorImpl<char>&, void*)"},
    {"_ZN5clang10Diagnostic5ResetEv", "clang::Diagnostic::Reset()"},
    {"_ZN5clang10DiagnosticD1Ev", "clang::Diagnostic::~Diagnostic()"},
    {"_ZN5clang10DiagnosticD2Ev", "clang::Diagnostic::~Diagnostic()"},
    {"_ZN5clang10Diagnostic9setClientEPNS_16DiagnosticClientEb", "clang::Diagnostic::setClient(clang::DiagnosticClient*, bool)"},
    {"_ZN5clang10Diagnostic12pushMappingsENS_14SourceLocationE", "clang::Diagnostic::pushMappings(clang::SourceLocation)"},
    {"_ZN5clang10Diagnostic11popMappingsENS_14SourceLocationE", "clang::Diagnostic::popMappings(clang::SourceLocation)"},
    {"_ZN5clang10Diagnostic20SetDelayedDiagnosticEjN4llvm9StringRefES2_", "clang::Diagnostic::SetDelayedDiagnostic(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZN5clang10Diagnostic13ReportDelayedEv", "clang::Diagnostic::ReportDelayed()"},
    {"_ZNK5clang10Diagnostic23GetDiagStatePointForLocENS_14SourceLocationE", "clang::Diagnostic::GetDiagStatePointForLoc(clang::SourceLocation) const"},
    {"_ZN5clang10Diagnostic20setDiagnosticMappingEjNS_4diag7MappingENS_14SourceLocationE", "clang::Diagnostic::setDiagnosticMapping(unsigned int, clang::diag::Mapping, clang::SourceLocation)"},
    {"_ZN5clang17DiagnosticBuilder11FlushCountsEv", "clang::DiagnosticBuilder::FlushCounts()"},
    {"_ZN5clang17DiagnosticBuilder4EmitEv", "clang::DiagnosticBuilder::Emit()"},
    {"_ZN5clang16DiagnosticClientD0Ev", "clang::DiagnosticClient::~DiagnosticClient()"},
    {"_ZN5clang16DiagnosticClientD1Ev", "clang::DiagnosticClient::~DiagnosticClient()"},
    {"_ZN5clang16DiagnosticClientD2Ev", "clang::DiagnosticClient::~DiagnosticClient()"},
    {"_ZN5clang16DiagnosticClient16HandleDiagnosticENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::DiagnosticClient::HandleDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZNK5clang14DiagnosticInfo16FormatDiagnosticERN4llvm15SmallVectorImplIcEE", "clang::DiagnosticInfo::FormatDiagnostic(llvm::SmallVectorImpl<char>&) const"},
    {"_ZNK5clang14DiagnosticInfo16FormatDiagnosticEPKcS2_RN4llvm15SmallVectorImplIcEE", "clang::DiagnosticInfo::FormatDiagnostic(char const*, char const*, llvm::SmallVectorImpl<char>&) const"},
    {"_Z10ScanFormatPKcS0_c", "ScanFormat(char const*, char const*, char)"},
    {"_Z20HandlePluralModifierRKN5clang14DiagnosticInfoEjPKcjRN4llvm15SmallVectorImplIcEE", "HandlePluralModifier(clang::DiagnosticInfo const&, unsigned int, char const*, unsigned int, llvm::SmallVectorImpl<char>&)"},
    {"_Z21HandleOrdinalModifierjRN4llvm15SmallVectorImplIcEE", "HandleOrdinalModifier(unsigned int, llvm::SmallVectorImpl<char>&)"},
    {"_ZN5clang16StoredDiagnosticC1Ev", "clang::StoredDiagnostic::StoredDiagnostic()"},
    {"_ZN5clang16StoredDiagnosticC2Ev", "clang::StoredDiagnostic::StoredDiagnostic()"},
    {"_ZN5clang16StoredDiagnosticC1ENS_10Diagnostic5LevelEjN4llvm9StringRefE", "clang::StoredDiagnostic::StoredDiagnostic(clang::Diagnostic::Level, unsigned int, llvm::StringRef)"},
    {"_ZN5clang16StoredDiagnosticC2ENS_10Diagnostic5LevelEjN4llvm9StringRefE", "clang::StoredDiagnostic::StoredDiagnostic(clang::Diagnostic::Level, unsigned int, llvm::StringRef)"},
    {"_ZN5clang16StoredDiagnosticC1ENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::StoredDiagnostic::StoredDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN5clang16StoredDiagnosticC2ENS_10Diagnostic5LevelERKNS_14DiagnosticInfoE", "clang::StoredDiagnostic::StoredDiagnostic(clang::Diagnostic::Level, clang::DiagnosticInfo const&)"},
    {"_ZN5clang16StoredDiagnosticD1Ev", "clang::StoredDiagnostic::~StoredDiagnostic()"},
    {"_ZN5clang16StoredDiagnosticD2Ev", "clang::StoredDiagnostic::~StoredDiagnostic()"},
    {"_ZNK5clang16DiagnosticClient25IncludeInDiagnosticCountsEv", "clang::DiagnosticClient::IncludeInDiagnosticCounts() const"},
    {"_ZN5clang17PartialDiagnostic16StorageAllocatorC1Ev", "clang::PartialDiagnostic::StorageAllocator::StorageAllocator()"},
    {"_ZN5clang17PartialDiagnostic16StorageAllocatorC2Ev", "clang::PartialDiagnostic::StorageAllocator::StorageAllocator()"},
    {"_ZN5clang17PartialDiagnostic16StorageAllocatorD1Ev", "clang::PartialDiagnostic::StorageAllocator::~StorageAllocator()"},
    {"_ZN5clang17PartialDiagnostic16StorageAllocatorD2Ev", "clang::PartialDiagnostic::StorageAllocator::~StorageAllocator()"},
    {"_ZNSt6vectorIN5clang15CharSourceRangeESaIS1_EE7reserveEm", "std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> >::reserve(unsigned long)"},
    {"_ZNSt6vectorIN5clang9FixItHintESaIS1_EE7reserveEm", "std::vector<clang::FixItHint, std::allocator<clang::FixItHint> >::reserve(unsigned long)"},
    {"_ZNSt6vectorIN5clang9FixItHintESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::FixItHint, std::allocator<clang::FixItHint> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::FixItHint*, std::vector<clang::FixItHint, std::allocator<clang::FixItHint> > >, clang::FixItHint const&)"},
    {"_ZNSt6vectorIN5clang15CharSourceRangeESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::CharSourceRange*, std::vector<clang::CharSourceRange, std::allocator<clang::CharSourceRange> > >, clang::CharSourceRange const&)"},
    {"_ZNSt6vectorIN5clang10Diagnostic14DiagStatePointESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::Diagnostic::DiagStatePoint, std::allocator<clang::Diagnostic::DiagStatePoint> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Diagnostic::DiagStatePoint*, std::vector<clang::Diagnostic::DiagStatePoint, std::allocator<clang::Diagnostic::DiagStatePoint> > >, clang::Diagnostic::DiagStatePoint const&)"},
    {"_ZNSt6vectorIPN5clang10Diagnostic9DiagStateESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::Diagnostic::DiagState*, std::allocator<clang::Diagnostic::DiagState*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::Diagnostic::DiagState**, std::vector<clang::Diagnostic::DiagState*, std::allocator<clang::Diagnostic::DiagState*> > >, clang::Diagnostic::DiagState* const&)"},
    {"_ZN4llvm8DenseMapIjjNS_12DenseMapInfoIjEES2_E8CopyFromERKS3_", "llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >::CopyFrom(llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> > const&)"},
    {"_ZN5clang17PartialDiagnostic7StorageD2Ev", "clang::PartialDiagnostic::Storage::~Storage()"},
    {"_ZN5clang13DiagnosticIDs23getWarningOptionForDiagEj", "clang::DiagnosticIDs::getWarningOptionForDiag(unsigned int)"},
    {"_ZN5clang13DiagnosticIDs24getCategoryNumberForDiagEj", "clang::DiagnosticIDs::getCategoryNumberForDiag(unsigned int)"},
    {"_ZN5clang13DiagnosticIDs21getCategoryNameFromIDEj", "clang::DiagnosticIDs::getCategoryNameFromID(unsigned int)"},
    {"_ZN5clang13DiagnosticIDs27getDiagnosticSFINAEResponseEj", "clang::DiagnosticIDs::getDiagnosticSFINAEResponse(unsigned int)"},
    {"_ZN5clang13DiagnosticIDsC1Ev", "clang::DiagnosticIDs::DiagnosticIDs()"},
    {"_ZN5clang13DiagnosticIDsC2Ev", "clang::DiagnosticIDs::DiagnosticIDs()"},
    {"_ZN5clang13DiagnosticIDsD1Ev", "clang::DiagnosticIDs::~DiagnosticIDs()"},
    {"_ZN5clang13DiagnosticIDsD2Ev", "clang::DiagnosticIDs::~DiagnosticIDs()"},
    {"_ZN5clang13DiagnosticIDs15getCustomDiagIDENS0_5LevelEN4llvm9StringRefE", "clang::DiagnosticIDs::getCustomDiagID(clang::DiagnosticIDs::Level, llvm::StringRef)"},
    {"_ZN5clang13DiagnosticIDs27isBuiltinWarningOrExtensionEj", "clang::DiagnosticIDs::isBuiltinWarningOrExtension(unsigned int)"},
    {"_ZN5clang13DiagnosticIDs13isBuiltinNoteEj", "clang::DiagnosticIDs::isBuiltinNote(unsigned int)"},
    {"_ZN5clang13DiagnosticIDs22isBuiltinExtensionDiagEjRb", "clang::DiagnosticIDs::isBuiltinExtensionDiag(unsigned int, bool&)"},
    {"_ZNK5clang13DiagnosticIDs14getDescriptionEj", "clang::DiagnosticIDs::getDescription(unsigned int) const"},
    {"_ZNK5clang13DiagnosticIDs18getDiagnosticLevelEjNS_14SourceLocationERKNS_10DiagnosticE", "clang::DiagnosticIDs::getDiagnosticLevel(unsigned int, clang::SourceLocation, clang::Diagnostic const&) const"},
    {"_ZNK5clang13DiagnosticIDs18getDiagnosticLevelEjjNS_14SourceLocationERKNS_10DiagnosticE", "clang::DiagnosticIDs::getDiagnosticLevel(unsigned int, unsigned int, clang::SourceLocation, clang::Diagnostic const&) const"},
    {"_ZNK5clang13DiagnosticIDs25setDiagnosticGroupMappingEPKcNS_4diag7MappingENS_14SourceLocationERNS_10DiagnosticE", "clang::DiagnosticIDs::setDiagnosticGroupMapping(char const*, clang::diag::Mapping, clang::SourceLocation, clang::Diagnostic&) const"},
    {"_Z15MapGroupMembersPK13WarningOptionN5clang4diag7MappingENS2_14SourceLocationERNS2_10DiagnosticE", "MapGroupMembers(WarningOption const*, clang::diag::Mapping, clang::SourceLocation, clang::Diagnostic&)"},
    {"_ZNK5clang13DiagnosticIDs11ProcessDiagERNS_10DiagnosticE", "clang::DiagnosticIDs::ProcessDiag(clang::Diagnostic&) const"},
    {"_ZN5clang4diag14CustomDiagInfo17getOrCreateDiagIDENS_13DiagnosticIDs5LevelEN4llvm9StringRefERS2_", "clang::diag::CustomDiagInfo::getOrCreateDiagID(clang::DiagnosticIDs::Level, llvm::StringRef, clang::DiagnosticIDs&)"},
    {"_ZN4llvm8DenseMapIjjNS_12DenseMapInfoIjEES2_E16InsertIntoBucketERKjS5_PSt4pairIjjE", "llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(unsigned int const&, unsigned int const&, std::pair<unsigned int, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIjjNS_12DenseMapInfoIjEES2_E4growEj", "llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIN5clang13DiagnosticIDs5LevelESsESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<clang::DiagnosticIDs::Level, std::string>, std::allocator<std::pair<clang::DiagnosticIDs::Level, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<clang::DiagnosticIDs::Level, std::string>*, std::vector<std::pair<clang::DiagnosticIDs::Level, std::string>, std::allocator<std::pair<clang::DiagnosticIDs::Level, std::string> > > >, std::pair<clang::DiagnosticIDs::Level, std::string> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIN5clang13DiagnosticIDs5LevelESsES0_IKS4_jESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::pair<clang::DiagnosticIDs::Level, std::string>, std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int>, std::_Select1st<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> >, std::less<std::pair<clang::DiagnosticIDs::Level, std::string> >, std::allocator<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> > >::_M_insert_unique(std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIN5clang13DiagnosticIDs5LevelESsES0_IKS4_jESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::pair<clang::DiagnosticIDs::Level, std::string>, std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int>, std::_Select1st<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> >, std::less<std::pair<clang::DiagnosticIDs::Level, std::string> >, std::allocator<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIN5clang13DiagnosticIDs5LevelESsES0_IKS4_jESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::pair<clang::DiagnosticIDs::Level, std::string>, std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int>, std::_Select1st<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> >, std::less<std::pair<clang::DiagnosticIDs::Level, std::string> >, std::allocator<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<clang::DiagnosticIDs::Level, std::string> const, unsigned int> >*)"},
    {"_ZN5clang9FileEntryD1Ev", "clang::FileEntry::~FileEntry()"},
    {"_ZN5clang9FileEntryD2Ev", "clang::FileEntry::~FileEntry()"},
    {"_ZN5clang11FileManagerC1ERKNS_17FileSystemOptionsE", "clang::FileManager::FileManager(clang::FileSystemOptions const&)"},
    {"_ZN5clang11FileManagerC2ERKNS_17FileSystemOptionsE", "clang::FileManager::FileManager(clang::FileSystemOptions const&)"},
    {"_ZN5clang11FileManagerD1Ev", "clang::FileManager::~FileManager()"},
    {"_ZN5clang11FileManagerD2Ev", "clang::FileManager::~FileManager()"},
    {"_ZN5clang11FileManager12addStatCacheEPNS_19FileSystemStatCacheEb", "clang::FileManager::addStatCache(clang::FileSystemStatCache*, bool)"},
    {"_ZN5clang11FileManager15removeStatCacheEPNS_19FileSystemStatCacheE", "clang::FileManager::removeStatCache(clang::FileSystemStatCache*)"},
    {"_ZN5clang11FileManager12getDirectoryEN4llvm9StringRefE", "clang::FileManager::getDirectory(llvm::StringRef)"},
    {"_ZN5clang11FileManager12getStatValueEPKcR4statPi", "clang::FileManager::getStatValue(char const*, stat&, int*)"},
    {"_ZN5clang11FileManager7getFileEN4llvm9StringRefE", "clang::FileManager::getFile(llvm::StringRef)"},
    {"_ZN5clang11FileManager14getVirtualFileEN4llvm9StringRefExl", "clang::FileManager::getVirtualFile(llvm::StringRef, long long, long)"},
    {"_ZN5clang11FileManager17FixupRelativePathERN4llvm3sys4PathERKNS_17FileSystemOptionsE", "clang::FileManager::FixupRelativePath(llvm::sys::Path&, clang::FileSystemOptions const&)"},
    {"_ZN5clang11FileManager16getBufferForFileEPKNS_9FileEntryEPSs", "clang::FileManager::getBufferForFile(clang::FileEntry const*, std::string*)"},
    {"_ZN5clang11FileManager16getBufferForFileEN4llvm9StringRefEPSs", "clang::FileManager::getBufferForFile(llvm::StringRef, std::string*)"},
    {"_ZNK5clang11FileManager10PrintStatsEv", "clang::FileManager::PrintStats() const"},
    {"_ZN4llvm9StringMapIPN5clang9FileEntryENS_16BumpPtrAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<clang::FileEntry*>& llvm::StringMap<clang::FileEntry*, llvm::BumpPtrAllocator>::GetOrCreateValue<clang::FileEntry*>(llvm::StringRef, clang::FileEntry*)"},
    {"_ZN4llvm9StringMapIPN5clang14DirectoryEntryENS_16BumpPtrAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<clang::DirectoryEntry*>& llvm::StringMap<clang::DirectoryEntry*, llvm::BumpPtrAllocator>::GetOrCreateValue<clang::DirectoryEntry*>(llvm::StringRef, clang::DirectoryEntry*)"},
    {"_ZNSt8_Rb_treeIN5clang9FileEntryES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE16_M_insert_uniqueERKS1_", "std::_Rb_tree<clang::FileEntry, clang::FileEntry, std::_Identity<clang::FileEntry>, std::less<clang::FileEntry>, std::allocator<clang::FileEntry> >::_M_insert_unique(clang::FileEntry const&)"},
    {"_ZNSt8_Rb_treeISt4pairIiyES0_IKS1_N5clang14DirectoryEntryEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<std::pair<int, unsigned long long>, std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry>, std::_Select1st<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> >, std::less<std::pair<int, unsigned long long> >, std::allocator<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> >, std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIiyES0_IKS1_N5clang14DirectoryEntryEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<std::pair<int, unsigned long long>, std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry>, std::_Select1st<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> >, std::less<std::pair<int, unsigned long long> >, std::allocator<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> > >::_M_insert_unique(std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> const&)"},
    {"_ZNSt8_Rb_treeIN5clang9FileEntryES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE8_M_eraseEPSt13_Rb_tree_nodeIS1_E", "std::_Rb_tree<clang::FileEntry, clang::FileEntry, std::_Identity<clang::FileEntry>, std::less<clang::FileEntry>, std::allocator<clang::FileEntry> >::_M_erase(std::_Rb_tree_node<clang::FileEntry>*)"},
    {"_ZNSt8_Rb_treeISt4pairIiyES0_IKS1_N5clang14DirectoryEntryEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<std::pair<int, unsigned long long>, std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry>, std::_Select1st<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> >, std::less<std::pair<int, unsigned long long> >, std::allocator<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<int, unsigned long long> const, clang::DirectoryEntry> >*)"},
    {"_ZN5clang19FileSystemStatCache3getEPKcR4statPiPS0_", "clang::FileSystemStatCache::get(char const*, stat&, int*, clang::FileSystemStatCache*)"},
    {"_ZN5clang17MemorizeStatCalls7getStatEPKcR4statPi", "clang::MemorizeStatCalls::getStat(char const*, stat&, int*)"},
    {"_ZN5clang17MemorizeStatCallsD1Ev", "clang::MemorizeStatCalls::~MemorizeStatCalls()"},
    {"_ZN5clang17MemorizeStatCallsD0Ev", "clang::MemorizeStatCalls::~MemorizeStatCalls()"},
    {"_ZN4llvm9StringMapI4statNS_16BumpPtrAllocatorEE16GetOrCreateValueIS1_EERNS_14StringMapEntryIS1_EENS_9StringRefET_", "llvm::StringMapEntry<stat>& llvm::StringMap<stat, llvm::BumpPtrAllocator>::GetOrCreateValue<stat>(llvm::StringRef, stat)"},
    {"_ZN5clang14IdentifierInfoC1Ev", "clang::IdentifierInfo::IdentifierInfo()"},
    {"_ZN5clang14IdentifierInfoC2Ev", "clang::IdentifierInfo::IdentifierInfo()"},
    {"_ZN5clang18IdentifierIteratorD0Ev", "clang::IdentifierIterator::~IdentifierIterator()"},
    {"_ZN5clang18IdentifierIteratorD1Ev", "clang::IdentifierIterator::~IdentifierIterator()"},
    {"_ZN5clang18IdentifierIteratorD2Ev", "clang::IdentifierIterator::~IdentifierIterator()"},
    {"_ZN5clang20IdentifierInfoLookupD0Ev", "clang::IdentifierInfoLookup::~IdentifierInfoLookup()"},
    {"_ZN5clang20IdentifierInfoLookupD1Ev", "clang::IdentifierInfoLookup::~IdentifierInfoLookup()"},
    {"_ZN5clang20IdentifierInfoLookupD2Ev", "clang::IdentifierInfoLookup::~IdentifierInfoLookup()"},
    {"_ZNK5clang20IdentifierInfoLookup14getIdentifiersEv", "clang::IdentifierInfoLookup::getIdentifiers() const"},
    {"_ZN5clang24ExternalIdentifierLookupD0Ev", "clang::ExternalIdentifierLookup::~ExternalIdentifierLookup()"},
    {"_ZN5clang24ExternalIdentifierLookupD1Ev", "clang::ExternalIdentifierLookup::~ExternalIdentifierLookup()"},
    {"_ZN5clang24ExternalIdentifierLookupD2Ev", "clang::ExternalIdentifierLookup::~ExternalIdentifierLookup()"},
    {"_ZN5clang15IdentifierTableC1ERKNS_11LangOptionsEPNS_20IdentifierInfoLookupE", "clang::IdentifierTable::IdentifierTable(clang::LangOptions const&, clang::IdentifierInfoLookup*)"},
    {"_ZN5clang15IdentifierTableC2ERKNS_11LangOptionsEPNS_20IdentifierInfoLookupE", "clang::IdentifierTable::IdentifierTable(clang::LangOptions const&, clang::IdentifierInfoLookup*)"},
    {"_ZN5clang15IdentifierTable11AddKeywordsERKNS_11LangOptionsE", "clang::IdentifierTable::AddKeywords(clang::LangOptions const&)"},
    {"_ZNK5clang14IdentifierInfo14getPPKeywordIDEv", "clang::IdentifierInfo::getPPKeywordID() const"},
    {"_ZNK5clang15IdentifierTable10PrintStatsEv", "clang::IdentifierTable::PrintStats() const"},
    {"_ZN4llvm12DenseMapInfoIN5clang8SelectorEE12getHashValueES2_", "llvm::DenseMapInfo<clang::Selector>::getHashValue(clang::Selector)"},
    {"_ZNK5clang8Selector10getNumArgsEv", "clang::Selector::getNumArgs() const"},
    {"_ZNK5clang8Selector24getIdentifierInfoForSlotEj", "clang::Selector::getIdentifierInfoForSlot(unsigned int) const"},
    {"_ZNK5clang20MultiKeywordSelector7getNameEv", "clang::MultiKeywordSelector::getName() const"},
    {"_ZNK5clang8Selector11getAsStringEv", "clang::Selector::getAsString() const"},
    {"_ZN5clang13SelectorTable11getSelectorEjPPNS_14IdentifierInfoE", "clang::SelectorTable::getSelector(unsigned int, clang::IdentifierInfo**)"},
    {"_ZN5clang13SelectorTableC1Ev", "clang::SelectorTable::SelectorTable()"},
    {"_ZN5clang13SelectorTableC2Ev", "clang::SelectorTable::SelectorTable()"},
    {"_ZN5clang13SelectorTableD1Ev", "clang::SelectorTable::~SelectorTable()"},
    {"_ZN5clang13SelectorTableD2Ev", "clang::SelectorTable::~SelectorTable()"},
    {"_ZN5clang19getOperatorSpellingENS_22OverloadedOperatorKindE", "clang::getOperatorSpelling(clang::OverloadedOperatorKind)"},
    {"_ZN12_GLOBAL__N_119EmptyLookupIteratorD1Ev", "(anonymous namespace)::EmptyLookupIterator::~EmptyLookupIterator()"},
    {"_ZN12_GLOBAL__N_119EmptyLookupIteratorD0Ev", "(anonymous namespace)::EmptyLookupIterator::~EmptyLookupIterator()"},
    {"_ZN12_GLOBAL__N_119EmptyLookupIterator4NextEv", "(anonymous namespace)::EmptyLookupIterator::Next()"},
    {"_ZN4llvm10FoldingSetIN5clang20MultiKeywordSelectorEED1Ev", "llvm::FoldingSet<clang::MultiKeywordSelector>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetIN5clang20MultiKeywordSelectorEED0Ev", "llvm::FoldingSet<clang::MultiKeywordSelector>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetIN5clang20MultiKeywordSelectorEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::MultiKeywordSelector>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang20MultiKeywordSelectorEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS7_", "llvm::FoldingSet<clang::MultiKeywordSelector>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetIN5clang20MultiKeywordSelectorEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<clang::MultiKeywordSelector>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK5clang19PrettyStackTraceLoc5printERN4llvm11raw_ostreamE", "clang::PrettyStackTraceLoc::print(llvm::raw_ostream&) const"},
    {"_ZNK5clang14SourceLocation5printERN4llvm11raw_ostreamERKNS_13SourceManagerE", "clang::SourceLocation::print(llvm::raw_ostream&, clang::SourceManager const&) const"},
    {"_ZNK5clang14SourceLocation4dumpERKNS_13SourceManagerE", "clang::SourceLocation::dump(clang::SourceManager const&) const"},
    {"_ZNK5clang13FullSourceLoc9getFileIDEv", "clang::FullSourceLoc::getFileID() const"},
    {"_ZNK5clang13FullSourceLoc19getInstantiationLocEv", "clang::FullSourceLoc::getInstantiationLoc() const"},
    {"_ZNK5clang13FullSourceLoc14getSpellingLocEv", "clang::FullSourceLoc::getSpellingLoc() const"},
    {"_ZNK5clang13FullSourceLoc26getInstantiationLineNumberEPb", "clang::FullSourceLoc::getInstantiationLineNumber(bool*) const"},
    {"_ZNK5clang13FullSourceLoc28getInstantiationColumnNumberEPb", "clang::FullSourceLoc::getInstantiationColumnNumber(bool*) const"},
    {"_ZNK5clang13FullSourceLoc21getSpellingLineNumberEPb", "clang::FullSourceLoc::getSpellingLineNumber(bool*) const"},
    {"_ZNK5clang13FullSourceLoc23getSpellingColumnNumberEPb", "clang::FullSourceLoc::getSpellingColumnNumber(bool*) const"},
    {"_ZNK5clang13FullSourceLoc16isInSystemHeaderEv", "clang::FullSourceLoc::isInSystemHeader() const"},
    {"_ZNK5clang13FullSourceLoc29isBeforeInTranslationUnitThanENS_14SourceLocationE", "clang::FullSourceLoc::isBeforeInTranslationUnitThan(clang::SourceLocation) const"},
    {"_ZNK5clang13FullSourceLoc16getCharacterDataEPb", "clang::FullSourceLoc::getCharacterData(bool*) const"},
    {"_ZNK5clang13FullSourceLoc9getBufferEPb", "clang::FullSourceLoc::getBuffer(bool*) const"},
    {"_ZNK5clang13FullSourceLoc13getBufferDataEPb", "clang::FullSourceLoc::getBufferData(bool*) const"},
    {"_ZNK5clang13FullSourceLoc16getDecomposedLocEv", "clang::FullSourceLoc::getDecomposedLoc() const"},
    {"_ZN5clang19PrettyStackTraceLocD1Ev", "clang::PrettyStackTraceLoc::~PrettyStackTraceLoc()"},
    {"_ZN5clang19PrettyStackTraceLocD0Ev", "clang::PrettyStackTraceLoc::~PrettyStackTraceLoc()"},
    {"_ZN5clang6SrcMgr12ContentCacheD1Ev", "clang::SrcMgr::ContentCache::~ContentCache()"},
    {"_ZN5clang6SrcMgr12ContentCacheD2Ev", "clang::SrcMgr::ContentCache::~ContentCache()"},
    {"_ZNK5clang6SrcMgr12ContentCache18getSizeBytesMappedEv", "clang::SrcMgr::ContentCache::getSizeBytesMapped() const"},
    {"_ZNK5clang6SrcMgr12ContentCache7getSizeEv", "clang::SrcMgr::ContentCache::getSize() const"},
    {"_ZN5clang6SrcMgr12ContentCache13replaceBufferEPKN4llvm12MemoryBufferEb", "clang::SrcMgr::ContentCache::replaceBuffer(llvm::MemoryBuffer const*, bool)"},
    {"_ZNK5clang6SrcMgr12ContentCache9getBufferERNS_10DiagnosticERKNS_13SourceManagerENS_14SourceLocationEPb", "clang::SrcMgr::ContentCache::getBuffer(clang::Diagnostic&, clang::SourceManager const&, clang::SourceLocation, bool*) const"},
    {"_ZN5clang13LineTableInfo22getLineTableFilenameIDEPKcj", "clang::LineTableInfo::getLineTableFilenameID(char const*, unsigned int)"},
    {"_ZN5clang13LineTableInfo11AddLineNoteEjjji", "clang::LineTableInfo::AddLineNote(unsigned int, unsigned int, unsigned int, int)"},
    {"_ZN5clang13LineTableInfo11AddLineNoteEjjjijNS_6SrcMgr18CharacteristicKindE", "clang::LineTableInfo::AddLineNote(unsigned int, unsigned int, unsigned int, int, unsigned int, clang::SrcMgr::CharacteristicKind)"},
    {"_ZN5clang13LineTableInfo20FindNearestLineEntryEjj", "clang::LineTableInfo::FindNearestLineEntry(unsigned int, unsigned int)"},
    {"_ZN5clang13LineTableInfo8AddEntryEjRKSt6vectorINS_9LineEntryESaIS2_EE", "clang::LineTableInfo::AddEntry(unsigned int, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > const&)"},
    {"_ZN5clang13SourceManager22getLineTableFilenameIDEPKcj", "clang::SourceManager::getLineTableFilenameID(char const*, unsigned int)"},
    {"_ZN5clang13SourceManager11AddLineNoteENS_14SourceLocationEji", "clang::SourceManager::AddLineNote(clang::SourceLocation, unsigned int, int)"},
    {"_ZN5clang13SourceManager11AddLineNoteENS_14SourceLocationEjibbbb", "clang::SourceManager::AddLineNote(clang::SourceLocation, unsigned int, int, bool, bool, bool, bool)"},
    {"_ZN5clang13SourceManager12getLineTableEv", "clang::SourceManager::getLineTable()"},
    {"_ZN5clang13SourceManagerC1ERNS_10DiagnosticERNS_11FileManagerE", "clang::SourceManager::SourceManager(clang::Diagnostic&, clang::FileManager&)"},
    {"_ZN5clang13SourceManagerC2ERNS_10DiagnosticERNS_11FileManagerE", "clang::SourceManager::SourceManager(clang::Diagnostic&, clang::FileManager&)"},
    {"_ZN5clang13SourceManager13clearIDTablesEv", "clang::SourceManager::clearIDTables()"},
    {"_ZN5clang13SourceManagerD1Ev", "clang::SourceManager::~SourceManager()"},
    {"_ZN5clang13SourceManagerD2Ev", "clang::SourceManager::~SourceManager()"},
    {"_ZN5clang13SourceManager22createInstantiationLocENS_14SourceLocationES1_S1_jjj", "clang::SourceManager::createInstantiationLoc(clang::SourceLocation, clang::SourceLocation, clang::SourceLocation, unsigned int, unsigned int, unsigned int)"},
    {"_ZN5clang13SourceManager23getOrCreateContentCacheEPKNS_9FileEntryE", "clang::SourceManager::getOrCreateContentCache(clang::FileEntry const*)"},
    {"_ZN5clang13SourceManager27createMemBufferContentCacheEPKN4llvm12MemoryBufferE", "clang::SourceManager::createMemBufferContentCache(llvm::MemoryBuffer const*)"},
    {"_ZN5clang13SourceManager22PreallocateSLocEntriesEPNS_23ExternalSLocEntrySourceEjj", "clang::SourceManager::PreallocateSLocEntries(clang::ExternalSLocEntrySource*, unsigned int, unsigned int)"},
    {"_ZN5clang13SourceManager28ClearPreallocatedSLocEntriesEv", "clang::SourceManager::ClearPreallocatedSLocEntries()"},
    {"_ZN5clang13SourceManager12createFileIDEPKNS_6SrcMgr12ContentCacheENS_14SourceLocationENS1_18CharacteristicKindEjj", "clang::SourceManager::createFileID(clang::SrcMgr::ContentCache const*, clang::SourceLocation, clang::SrcMgr::CharacteristicKind, unsigned int, unsigned int)"},
    {"_ZN5clang13SourceManager22getMemoryBufferForFileEPKNS_9FileEntryEPb", "clang::SourceManager::getMemoryBufferForFile(clang::FileEntry const*, bool*)"},
    {"_ZN5clang13SourceManager20overrideFileContentsEPKNS_9FileEntryEPKN4llvm12MemoryBufferEb", "clang::SourceManager::overrideFileContents(clang::FileEntry const*, llvm::MemoryBuffer const*, bool)"},
    {"_ZNK5clang13SourceManager13getBufferDataENS_6FileIDEPb", "clang::SourceManager::getBufferData(clang::FileID, bool*) const"},
    {"_ZNK5clang13SourceManager13getFileIDSlowEj", "clang::SourceManager::getFileIDSlow(unsigned int) const"},
    {"_ZNK5clang13SourceManager27getInstantiationLocSlowCaseENS_14SourceLocationE", "clang::SourceManager::getInstantiationLocSlowCase(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager22getSpellingLocSlowCaseENS_14SourceLocationE", "clang::SourceManager::getSpellingLocSlowCase(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager37getDecomposedInstantiationLocSlowCaseEPKNS_6SrcMgr9SLocEntryEj", "clang::SourceManager::getDecomposedInstantiationLocSlowCase(clang::SrcMgr::SLocEntry const*, unsigned int) const"},
    {"_ZNK5clang13SourceManager32getDecomposedSpellingLocSlowCaseEPKNS_6SrcMgr9SLocEntryEj", "clang::SourceManager::getDecomposedSpellingLocSlowCase(clang::SrcMgr::SLocEntry const*, unsigned int) const"},
    {"_ZNK5clang13SourceManager23getImmediateSpellingLocENS_14SourceLocationE", "clang::SourceManager::getImmediateSpellingLoc(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager30getImmediateInstantiationRangeENS_14SourceLocationE", "clang::SourceManager::getImmediateInstantiationRange(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager21getInstantiationRangeENS_14SourceLocationE", "clang::SourceManager::getInstantiationRange(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager16getCharacterDataENS_14SourceLocationEPb", "clang::SourceManager::getCharacterData(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager15getColumnNumberENS_6FileIDEjPb", "clang::SourceManager::getColumnNumber(clang::FileID, unsigned int, bool*) const"},
    {"_ZNK5clang13SourceManager23getSpellingColumnNumberENS_14SourceLocationEPb", "clang::SourceManager::getSpellingColumnNumber(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager28getInstantiationColumnNumberENS_14SourceLocationEPb", "clang::SourceManager::getInstantiationColumnNumber(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager13getLineNumberENS_6FileIDEjPb", "clang::SourceManager::getLineNumber(clang::FileID, unsigned int, bool*) const"},
    {"_Z18ComputeLineNumbersRN5clang10DiagnosticEPNS_6SrcMgr12ContentCacheERN4llvm16BumpPtrAllocatorERKNS_13SourceManagerERb", "ComputeLineNumbers(clang::Diagnostic&, clang::SrcMgr::ContentCache*, llvm::BumpPtrAllocator&, clang::SourceManager const&, bool&)"},
    {"_ZNK5clang13SourceManager26getInstantiationLineNumberENS_14SourceLocationEPb", "clang::SourceManager::getInstantiationLineNumber(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager21getSpellingLineNumberENS_14SourceLocationEPb", "clang::SourceManager::getSpellingLineNumber(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager21getFileCharacteristicENS_14SourceLocationE", "clang::SourceManager::getFileCharacteristic(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager13getBufferNameENS_14SourceLocationEPb", "clang::SourceManager::getBufferName(clang::SourceLocation, bool*) const"},
    {"_ZNK5clang13SourceManager14getPresumedLocENS_14SourceLocationE", "clang::SourceManager::getPresumedLoc(clang::SourceLocation) const"},
    {"_ZN5clang13SourceManager11getLocationEPKNS_9FileEntryEjj", "clang::SourceManager::getLocation(clang::FileEntry const*, unsigned int, unsigned int)"},
    {"_ZNK5clang13SourceManager25isBeforeInTranslationUnitENS_14SourceLocationES1_", "clang::SourceManager::isBeforeInTranslationUnit(clang::SourceLocation, clang::SourceLocation) const"},
    {"_Z22MoveUpIncludeHierarchyRSt4pairIN5clang6FileIDEjERKNS0_13SourceManagerE", "MoveUpIncludeHierarchy(std::pair<clang::FileID, unsigned int>&, clang::SourceManager const&)"},
    {"_ZNK5clang13SourceManager10PrintStatsEv", "clang::SourceManager::PrintStats() const"},
    {"_ZN5clang23ExternalSLocEntrySourceD0Ev", "clang::ExternalSLocEntrySource::~ExternalSLocEntrySource()"},
    {"_ZN5clang23ExternalSLocEntrySourceD1Ev", "clang::ExternalSLocEntrySource::~ExternalSLocEntrySource()"},
    {"_ZN5clang23ExternalSLocEntrySourceD2Ev", "clang::ExternalSLocEntrySource::~ExternalSLocEntrySource()"},
    {"_ZNSt6vectorIN5clang9LineEntryESaIS1_EEaSERKS3_", "std::vector<clang::LineEntry, std::allocator<clang::LineEntry> >::operator=(std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > const&)"},
    {"_ZNSt6vectorIbSaIbEE6resizeEmb", "std::vector<bool, std::allocator<bool> >::resize(unsigned long, bool)"},
    {"_ZNK5clang13SourceManager24getDecomposedSpellingLocENS_14SourceLocationE", "clang::SourceManager::getDecomposedSpellingLoc(clang::SourceLocation) const"},
    {"_ZNK5clang13SourceManager9getBufferENS_6FileIDEPb", "clang::SourceManager::getBuffer(clang::FileID, bool*) const"},
    {"_ZNSt6vectorIN5clang6SrcMgr9SLocEntryESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<clang::SrcMgr::SLocEntry, std::allocator<clang::SrcMgr::SLocEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::SrcMgr::SLocEntry*, std::vector<clang::SrcMgr::SLocEntry, std::allocator<clang::SrcMgr::SLocEntry> > >, clang::SrcMgr::SLocEntry const&)"},
    {"_ZNSt6vectorIN5clang6SrcMgr9SLocEntryESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<clang::SrcMgr::SLocEntry, std::allocator<clang::SrcMgr::SLocEntry> >::_M_fill_insert(__gnu_cxx::__normal_iterator<clang::SrcMgr::SLocEntry*, std::vector<clang::SrcMgr::SLocEntry, std::allocator<clang::SrcMgr::SLocEntry> > >, unsigned long, clang::SrcMgr::SLocEntry const&)"},
    {"_ZNSt6vectorIbSaIbEE14_M_fill_insertESt13_Bit_iteratormb", "std::vector<bool, std::allocator<bool> >::_M_fill_insert(std::_Bit_iterator, unsigned long, bool)"},
    {"_ZSt4fillSt13_Bit_iteratorS_RKb", "std::fill(std::_Bit_iterator, std::_Bit_iterator, bool const&)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyISt13_Bit_iteratorS3_EET0_T_S5_S4_", "std::_Bit_iterator std::__copy<false, std::random_access_iterator_tag>::copy<std::_Bit_iterator, std::_Bit_iterator>(std::_Bit_iterator, std::_Bit_iterator, std::_Bit_iterator)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyISt19_Bit_const_iteratorSt13_Bit_iteratorEET0_T_S6_S5_", "std::_Bit_iterator std::__copy<false, std::random_access_iterator_tag>::copy<std::_Bit_const_iterator, std::_Bit_iterator>(std::_Bit_const_iterator, std::_Bit_const_iterator, std::_Bit_iterator)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bISt13_Bit_iteratorS3_EET0_T_S5_S4_", "std::_Bit_iterator std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::_Bit_iterator, std::_Bit_iterator>(std::_Bit_iterator, std::_Bit_iterator, std::_Bit_iterator)"},
    {"_ZNSt6vectorIPN5clang6SrcMgr12ContentCacheESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<clang::SrcMgr::ContentCache*, std::allocator<clang::SrcMgr::ContentCache*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::SrcMgr::ContentCache**, std::vector<clang::SrcMgr::ContentCache*, std::allocator<clang::SrcMgr::ContentCache*> > >, clang::SrcMgr::ContentCache* const&)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FileEntryEPNS1_6SrcMgr12ContentCacheENS_12DenseMapInfoIS4_EENS8_IS7_EEE16InsertIntoBucketERKS4_RKS7_PSt4pairIS4_S7_E", "llvm::DenseMap<clang::FileEntry const*, clang::SrcMgr::ContentCache*, llvm::DenseMapInfo<clang::FileEntry const*>, llvm::DenseMapInfo<clang::SrcMgr::ContentCache*> >::InsertIntoBucket(clang::FileEntry const* const&, clang::SrcMgr::ContentCache* const&, std::pair<clang::FileEntry const*, clang::SrcMgr::ContentCache*>*)"},
    {"_ZN4llvm8DenseMapIPKN5clang9FileEntryEPNS1_6SrcMgr12ContentCacheENS_12DenseMapInfoIS4_EENS8_IS7_EEE4growEj", "llvm::DenseMap<clang::FileEntry const*, clang::SrcMgr::ContentCache*, llvm::DenseMapInfo<clang::FileEntry const*>, llvm::DenseMapInfo<clang::SrcMgr::ContentCache*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN5clang9LineEntryESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<clang::LineEntry, std::allocator<clang::LineEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<clang::LineEntry*, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > >, clang::LineEntry const&)"},
    {"_ZNSt6vectorIN5clang9LineEntryESaIS1_EEC2ERKS3_", "std::vector<clang::LineEntry, std::allocator<clang::LineEntry> >::vector(std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjSt6vectorIN5clang9LineEntryESaIS4_EEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > >, std::_Select1st<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > >, std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjSt6vectorIN5clang9LineEntryESaIS4_EEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > >, std::_Select1st<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > > >::_M_insert_unique(std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > const&)"},
    {"_ZNSt6vectorIPN4llvm14StringMapEntryIjEESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::StringMapEntry<unsigned int>*, std::allocator<llvm::StringMapEntry<unsigned int>*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::StringMapEntry<unsigned int>**, std::vector<llvm::StringMapEntry<unsigned int>*, std::allocator<llvm::StringMapEntry<unsigned int>*> > >, llvm::StringMapEntry<unsigned int>* const&)"},
    {"_ZN4llvm9StringMapIjNS_16BumpPtrAllocatorEE16GetOrCreateValueIjEERNS_14StringMapEntryIjEENS_9StringRefET_", "llvm::StringMapEntry<unsigned int>& llvm::StringMap<unsigned int, llvm::BumpPtrAllocator>::GetOrCreateValue<unsigned int>(llvm::StringRef, unsigned int)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjSt6vectorIN5clang9LineEntryESaIS4_EEESt10_Select1stIS7_ESt4lessIjESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > >, std::_Select1st<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::vector<clang::LineEntry, std::allocator<clang::LineEntry> > > >*)"},
    {"_ZN5clang13LineTableInfoD2Ev", "clang::LineTableInfo::~LineTableInfo()"},
    {"_ZN5clang10TargetInfoC1ERKSs", "clang::TargetInfo::TargetInfo(std::string const&)"},
    {"_ZN5clang10TargetInfoC2ERKSs", "clang::TargetInfo::TargetInfo(std::string const&)"},
    {"_ZN5clang10TargetInfoD0Ev", "clang::TargetInfo::~TargetInfo()"},
    {"_ZN5clang10TargetInfoD1Ev", "clang::TargetInfo::~TargetInfo()"},
    {"_ZN5clang10TargetInfoD2Ev", "clang::TargetInfo::~TargetInfo()"},
    {"_ZN5clang10TargetInfo11getTypeNameENS0_7IntTypeE", "clang::TargetInfo::getTypeName(clang::TargetInfo::IntType)"},
    {"_ZN5clang10TargetInfo21getTypeConstantSuffixENS0_7IntTypeE", "clang::TargetInfo::getTypeConstantSuffix(clang::TargetInfo::IntType)"},
    {"_ZNK5clang10TargetInfo12getTypeWidthENS0_7IntTypeE", "clang::TargetInfo::getTypeWidth(clang::TargetInfo::IntType) const"},
    {"_ZNK5clang10TargetInfo12getTypeAlignENS0_7IntTypeE", "clang::TargetInfo::getTypeAlign(clang::TargetInfo::IntType) const"},
    {"_ZN5clang10TargetInfo12isTypeSignedENS0_7IntTypeE", "clang::TargetInfo::isTypeSigned(clang::TargetInfo::IntType)"},
    {"_ZN5clang10TargetInfo20setForcedLangOptionsERNS_11LangOptionsE", "clang::TargetInfo::setForcedLangOptions(clang::LangOptions&)"},
    {"_ZNK5clang10TargetInfo22isValidGCCRegisterNameEN4llvm9StringRefE", "clang::TargetInfo::isValidGCCRegisterName(llvm::StringRef) const"},
    {"_ZNK5clang10TargetInfo28getNormalizedGCCRegisterNameEN4llvm9StringRefE", "clang::TargetInfo::getNormalizedGCCRegisterName(llvm::StringRef) const"},
    {"_ZNK5clang10TargetInfo24validateOutputConstraintERNS0_14ConstraintInfoE", "clang::TargetInfo::validateOutputConstraint(clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZNK5clang10TargetInfo19resolveSymbolicNameERPKcPNS0_14ConstraintInfoEjRj", "clang::TargetInfo::resolveSymbolicName(char const*&, clang::TargetInfo::ConstraintInfo*, unsigned int, unsigned int&) const"},
    {"_ZNK5clang10TargetInfo23validateInputConstraintEPNS0_14ConstraintInfoEjRS1_", "clang::TargetInfo::validateInputConstraint(clang::TargetInfo::ConstraintInfo*, unsigned int, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN5clang10TargetInfo16CreateTargetInfoERNS_10DiagnosticERNS_13TargetOptionsE", "clang::TargetInfo::CreateTargetInfo(clang::Diagnostic&, clang::TargetOptions&)"},
    {"_ZN12_GLOBAL__N_122DarwinX86_64TargetInfoC1ERKSs", "(anonymous namespace)::DarwinX86_64TargetInfo::DarwinX86_64TargetInfo(std::string const&)"},
    {"_ZN12_GLOBAL__N_116X86_64TargetInfoD1Ev", "(anonymous namespace)::X86_64TargetInfo::~X86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_116X86_64TargetInfoD0Ev", "(anonymous namespace)::X86_64TargetInfo::~X86_64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::X86TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::X86TargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116X86_64TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::X86_64TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo17convertConstraintEc", "(anonymous namespace)::X86TargetInfo::convertConstraint(char) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo11getClobbersEv", "(anonymous namespace)::X86TargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo18getDefaultFeaturesERKSsRN4llvm9StringMapIbNS3_15MallocAllocatorEEE", "(anonymous namespace)::X86TargetInfo::getDefaultFeatures(std::string const&, llvm::StringMap<bool, llvm::MallocAllocator>&) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo17setFeatureEnabledERN4llvm9StringMapIbNS1_15MallocAllocatorEEERKSsb", "(anonymous namespace)::X86TargetInfo::setFeatureEnabled(llvm::StringMap<bool, llvm::MallocAllocator>&, std::string const&, bool) const"},
    {"_ZN12_GLOBAL__N_113X86TargetInfo20HandleTargetFeaturesERSt6vectorISsSaISsEE", "(anonymous namespace)::X86TargetInfo::HandleTargetFeatures(std::vector<std::string, std::allocator<std::string> >&)"},
    {"_ZNK12_GLOBAL__N_116X86_64TargetInfo23getEHDataRegisterNumberEj", "(anonymous namespace)::X86_64TargetInfo::getEHDataRegisterNumber(unsigned int) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo19adjustInlineAsmTypeERSsPKN4llvm4TypeERNS2_11LLVMContextE", "(anonymous namespace)::X86TargetInfo::adjustInlineAsmType(std::string&, llvm::Type const*, llvm::LLVMContext&) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::X86TargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::X86TargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113X86TargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::X86TargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_Z9DefineStdRN5clang12MacroBuilderEN4llvm9StringRefERKNS_11LangOptionsE", "DefineStd(clang::MacroBuilder&, llvm::StringRef, clang::LangOptions const&)"},
    {"_ZN12_GLOBAL__N_113X86TargetInfoD1Ev", "(anonymous namespace)::X86TargetInfo::~X86TargetInfo()"},
    {"_ZN12_GLOBAL__N_113X86TargetInfoD0Ev", "(anonymous namespace)::X86TargetInfo::~X86TargetInfo()"},
    {"_ZN12_GLOBAL__N_135VisualStudioWindowsX86_64TargetInfoD1Ev", "(anonymous namespace)::VisualStudioWindowsX86_64TargetInfo::~VisualStudioWindowsX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_135VisualStudioWindowsX86_64TargetInfoD0Ev", "(anonymous namespace)::VisualStudioWindowsX86_64TargetInfo::~VisualStudioWindowsX86_64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_135VisualStudioWindowsX86_64TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::VisualStudioWindowsX86_64TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_123WindowsX86_64TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::WindowsX86_64TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_117WindowsTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_123WindowsX86_64TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::WindowsX86_64TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_16X86_64TargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_123WindowsX86_64TargetInfoD1Ev", "(anonymous namespace)::WindowsX86_64TargetInfo::~WindowsX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_123WindowsX86_64TargetInfoD0Ev", "(anonymous namespace)::WindowsX86_64TargetInfo::~WindowsX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_117WindowsTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~WindowsTargetInfo()"},
    {"_ZN12_GLOBAL__N_117WindowsTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~WindowsTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_121MinGWX86_64TargetInfoD1Ev", "(anonymous namespace)::MinGWX86_64TargetInfo::~MinGWX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_121MinGWX86_64TargetInfoD0Ev", "(anonymous namespace)::MinGWX86_64TargetInfo::~MinGWX86_64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_121MinGWX86_64TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MinGWX86_64TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~SolarisTargetInfo()"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~SolarisTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117SolarisTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_123OpenBSDX86_64TargetInfoD1Ev", "(anonymous namespace)::OpenBSDX86_64TargetInfo::~OpenBSDX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_123OpenBSDX86_64TargetInfoD0Ev", "(anonymous namespace)::OpenBSDX86_64TargetInfo::~OpenBSDX86_64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~OpenBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~OpenBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~NetBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~NetBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~DragonFlyBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~DragonFlyBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~LinuxTargetInfo()"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~LinuxTargetInfo()"},
    {"_ZNK12_GLOBAL__N_115LinuxTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_122DarwinX86_64TargetInfoD1Ev", "(anonymous namespace)::DarwinX86_64TargetInfo::~DarwinX86_64TargetInfo()"},
    {"_ZN12_GLOBAL__N_122DarwinX86_64TargetInfoD0Ev", "(anonymous namespace)::DarwinX86_64TargetInfo::~DarwinX86_64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEE23isValidSectionSpecifierEN4llvm9StringRefE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEE29getStaticInitSectionSpecifierEv", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getStaticInitSectionSpecifier() const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_Z16getDarwinDefinesRN5clang12MacroBuilderERKNS_11LangOptionsERKN4llvm6TripleE", "getDarwinDefines(clang::MacroBuilder&, clang::LangOptions const&, llvm::Triple const&)"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_64TargetInfoEED1Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_64TargetInfoEED0Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_64TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZNK12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116X86_32TargetInfoD1Ev", "(anonymous namespace)::X86_32TargetInfo::~X86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_116X86_32TargetInfoD0Ev", "(anonymous namespace)::X86_32TargetInfo::~X86_32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_116X86_32TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::X86_32TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_116X86_32TargetInfo23getEHDataRegisterNumberEj", "(anonymous namespace)::X86_32TargetInfo::getEHDataRegisterNumber(unsigned int) const"},
    {"_ZN12_GLOBAL__N_121HaikuX86_32TargetInfoD1Ev", "(anonymous namespace)::HaikuX86_32TargetInfo::~HaikuX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_121HaikuX86_32TargetInfoD0Ev", "(anonymous namespace)::HaikuX86_32TargetInfo::~HaikuX86_32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_121HaikuX86_32TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::HaikuX86_32TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_135VisualStudioWindowsX86_32TargetInfoD1Ev", "(anonymous namespace)::VisualStudioWindowsX86_32TargetInfo::~VisualStudioWindowsX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_135VisualStudioWindowsX86_32TargetInfoD0Ev", "(anonymous namespace)::VisualStudioWindowsX86_32TargetInfo::~VisualStudioWindowsX86_32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_135VisualStudioWindowsX86_32TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::VisualStudioWindowsX86_32TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117WindowsTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_123WindowsX86_32TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::WindowsX86_32TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_16X86_32TargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_123WindowsX86_32TargetInfoD1Ev", "(anonymous namespace)::WindowsX86_32TargetInfo::~WindowsX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_123WindowsX86_32TargetInfoD0Ev", "(anonymous namespace)::WindowsX86_32TargetInfo::~WindowsX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_117WindowsTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~WindowsTargetInfo()"},
    {"_ZN12_GLOBAL__N_117WindowsTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::WindowsTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~WindowsTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_121MinGWX86_32TargetInfoD1Ev", "(anonymous namespace)::MinGWX86_32TargetInfo::~MinGWX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_121MinGWX86_32TargetInfoD0Ev", "(anonymous namespace)::MinGWX86_32TargetInfo::~MinGWX86_32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_121MinGWX86_32TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MinGWX86_32TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_122CygwinX86_32TargetInfoD1Ev", "(anonymous namespace)::CygwinX86_32TargetInfo::~CygwinX86_32TargetInfo()"},
    {"_ZN12_GLOBAL__N_122CygwinX86_32TargetInfoD0Ev", "(anonymous namespace)::CygwinX86_32TargetInfo::~CygwinX86_32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_122CygwinX86_32TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::CygwinX86_32TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~SolarisTargetInfo()"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~SolarisTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117SolarisTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115MinixTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::MinixTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~MinixTargetInfo()"},
    {"_ZN12_GLOBAL__N_115MinixTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::MinixTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~MinixTargetInfo()"},
    {"_ZNK12_GLOBAL__N_115MinixTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::MinixTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_121OpenBSDI386TargetInfoD1Ev", "(anonymous namespace)::OpenBSDI386TargetInfo::~OpenBSDI386TargetInfo()"},
    {"_ZN12_GLOBAL__N_121OpenBSDI386TargetInfoD0Ev", "(anonymous namespace)::OpenBSDI386TargetInfo::~OpenBSDI386TargetInfo()"},
    {"_ZNK12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~OpenBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117OpenBSDTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::OpenBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~OpenBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~NetBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~NetBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~DragonFlyBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~DragonFlyBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~LinuxTargetInfo()"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~LinuxTargetInfo()"},
    {"_ZNK12_GLOBAL__N_115LinuxTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_120DarwinI386TargetInfoD1Ev", "(anonymous namespace)::DarwinI386TargetInfo::~DarwinI386TargetInfo()"},
    {"_ZN12_GLOBAL__N_120DarwinI386TargetInfoD0Ev", "(anonymous namespace)::DarwinI386TargetInfo::~DarwinI386TargetInfo()"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEE23isValidSectionSpecifierEN4llvm9StringRefE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEE29getStaticInitSectionSpecifierEv", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getStaticInitSectionSpecifier() const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_32TargetInfoEED1Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_32TargetInfoEED0Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_32TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZNK12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_113TCETargetInfoD1Ev", "(anonymous namespace)::TCETargetInfo::~TCETargetInfo()"},
    {"_ZN12_GLOBAL__N_113TCETargetInfoD0Ev", "(anonymous namespace)::TCETargetInfo::~TCETargetInfo()"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::TCETargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::TCETargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo20getVAListDeclarationEv", "(anonymous namespace)::TCETargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo11getClobbersEv", "(anonymous namespace)::TCETargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::TCETargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::TCETargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113TCETargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::TCETargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_117SystemZTargetInfoD1Ev", "(anonymous namespace)::SystemZTargetInfo::~SystemZTargetInfo()"},
    {"_ZN12_GLOBAL__N_117SystemZTargetInfoD0Ev", "(anonymous namespace)::SystemZTargetInfo::~SystemZTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::SystemZTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::SystemZTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo20getVAListDeclarationEv", "(anonymous namespace)::SystemZTargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo11getClobbersEv", "(anonymous namespace)::SystemZTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::SystemZTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::SystemZTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SystemZTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::SystemZTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_116PS3SPUTargetInfoINS_15PPC64TargetInfoEED1Ev", "(anonymous namespace)::PS3SPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~PS3SPUTargetInfo()"},
    {"_ZN12_GLOBAL__N_116PS3SPUTargetInfoINS_15PPC64TargetInfoEED0Ev", "(anonymous namespace)::PS3SPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~PS3SPUTargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_15PPC64TargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::PPCTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_115PPC64TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::PPC64TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo11getClobbersEv", "(anonymous namespace)::PPCTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::PPCTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::PPCTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::PPCTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZNK12_GLOBAL__N_116PS3SPUTargetInfoINS_15PPC64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::PS3SPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_113PPCTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::PPCTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_15PPC64TargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_15PPC64TargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_115PPC64TargetInfoD1Ev", "(anonymous namespace)::PPC64TargetInfo::~PPC64TargetInfo()"},
    {"_ZN12_GLOBAL__N_115PPC64TargetInfoD0Ev", "(anonymous namespace)::PPC64TargetInfo::~PPC64TargetInfo()"},
    {"_ZN12_GLOBAL__N_113PPCTargetInfoD1Ev", "(anonymous namespace)::PPCTargetInfo::~PPCTargetInfo()"},
    {"_ZN12_GLOBAL__N_113PPCTargetInfoD0Ev", "(anonymous namespace)::PPCTargetInfo::~PPCTargetInfo()"},
    {"_ZN12_GLOBAL__N_117SparcV8TargetInfoD1Ev", "(anonymous namespace)::SparcV8TargetInfo::~SparcV8TargetInfo()"},
    {"_ZN12_GLOBAL__N_117SparcV8TargetInfoD0Ev", "(anonymous namespace)::SparcV8TargetInfo::~SparcV8TargetInfo()"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::SparcV8TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::SparcV8TargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::SparcV8TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo11getClobbersEv", "(anonymous namespace)::SparcV8TargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo17setFeatureEnabledERN4llvm9StringMapIbNS1_15MallocAllocatorEEERKSsb", "(anonymous namespace)::SparcV8TargetInfo::setFeatureEnabled(llvm::StringMap<bool, llvm::MallocAllocator>&, std::string const&, bool) const"},
    {"_ZN12_GLOBAL__N_117SparcV8TargetInfo20HandleTargetFeaturesERSt6vectorISsSaISsEE", "(anonymous namespace)::SparcV8TargetInfo::HandleTargetFeatures(std::vector<std::string, std::allocator<std::string> >&)"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::SparcV8TargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::SparcV8TargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_117SparcV8TargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::SparcV8TargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_124SolarisSparcV8TargetInfoD1Ev", "(anonymous namespace)::SolarisSparcV8TargetInfo::~SolarisSparcV8TargetInfo()"},
    {"_ZN12_GLOBAL__N_124SolarisSparcV8TargetInfoD0Ev", "(anonymous namespace)::SolarisSparcV8TargetInfo::~SolarisSparcV8TargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_17SparcV8TargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117SolarisTargetInfoINS_17SparcV8TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_17SparcV8TargetInfoEED1Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~SolarisTargetInfo()"},
    {"_ZN12_GLOBAL__N_117SolarisTargetInfoINS_17SparcV8TargetInfoEED0Ev", "(anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~SolarisTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_17SparcV8TargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_17SparcV8TargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_125AuroraUXSparcV8TargetInfoD1Ev", "(anonymous namespace)::AuroraUXSparcV8TargetInfo::~AuroraUXSparcV8TargetInfo()"},
    {"_ZN12_GLOBAL__N_125AuroraUXSparcV8TargetInfoD0Ev", "(anonymous namespace)::AuroraUXSparcV8TargetInfo::~AuroraUXSparcV8TargetInfo()"},
    {"_ZNK12_GLOBAL__N_118AuroraUXTargetInfoINS_17SparcV8TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_17SparcV8TargetInfoEED1Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZN12_GLOBAL__N_118AuroraUXTargetInfoINS_17SparcV8TargetInfoEED0Ev", "(anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::SparcV8TargetInfo>::~AuroraUXTargetInfo()"},
    {"_ZN12_GLOBAL__N_116MBlazeTargetInfoD1Ev", "(anonymous namespace)::MBlazeTargetInfo::~MBlazeTargetInfo()"},
    {"_ZN12_GLOBAL__N_116MBlazeTargetInfoD0Ev", "(anonymous namespace)::MBlazeTargetInfo::~MBlazeTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MBlazeTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::MBlazeTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo20getVAListDeclarationEv", "(anonymous namespace)::MBlazeTargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo11getClobbersEv", "(anonymous namespace)::MBlazeTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::MBlazeTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::MBlazeTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::MBlazeTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZNK12_GLOBAL__N_116MBlazeTargetInfo15getTargetPrefixEv", "(anonymous namespace)::MBlazeTargetInfo::getTargetPrefix() const"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC64TargetInfoEED1Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC64TargetInfoEED0Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116PS3PPUTargetInfoINS_15PPC64TargetInfoEED1Ev", "(anonymous namespace)::PS3PPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~PS3PPUTargetInfo()"},
    {"_ZN12_GLOBAL__N_116PS3PPUTargetInfoINS_15PPC64TargetInfoEED0Ev", "(anonymous namespace)::PS3PPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~PS3PPUTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116PS3PPUTargetInfoINS_15PPC64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::PS3PPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_121DarwinPPC64TargetInfoD1Ev", "(anonymous namespace)::DarwinPPC64TargetInfo::~DarwinPPC64TargetInfo()"},
    {"_ZN12_GLOBAL__N_121DarwinPPC64TargetInfoD0Ev", "(anonymous namespace)::DarwinPPC64TargetInfo::~DarwinPPC64TargetInfo()"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEE23isValidSectionSpecifierEN4llvm9StringRefE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEE29getStaticInitSectionSpecifierEv", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getStaticInitSectionSpecifier() const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEED1Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEED0Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_115PPC32TargetInfoD1Ev", "(anonymous namespace)::PPC32TargetInfo::~PPC32TargetInfo()"},
    {"_ZN12_GLOBAL__N_115PPC32TargetInfoD0Ev", "(anonymous namespace)::PPC32TargetInfo::~PPC32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_115PPC32TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::PPC32TargetInfo::getVAListDeclaration() const"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC32TargetInfoEED1Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC32TargetInfoEED0Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_15PPC32TargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC32TargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_15PPC32TargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_15PPC32TargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_121DarwinPPC32TargetInfoD1Ev", "(anonymous namespace)::DarwinPPC32TargetInfo::~DarwinPPC32TargetInfo()"},
    {"_ZN12_GLOBAL__N_121DarwinPPC32TargetInfoD0Ev", "(anonymous namespace)::DarwinPPC32TargetInfo::~DarwinPPC32TargetInfo()"},
    {"_ZNK12_GLOBAL__N_121DarwinPPC32TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::DarwinPPC32TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEE23isValidSectionSpecifierEN4llvm9StringRefE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEE29getStaticInitSectionSpecifierEv", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>::getStaticInitSectionSpecifier() const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEED1Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEED0Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116MipselTargetInfoD1Ev", "(anonymous namespace)::MipselTargetInfo::~MipselTargetInfo()"},
    {"_ZN12_GLOBAL__N_116MipselTargetInfoD0Ev", "(anonymous namespace)::MipselTargetInfo::~MipselTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116MipselTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MipselTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::MipsTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo20getVAListDeclarationEv", "(anonymous namespace)::MipsTargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo11getClobbersEv", "(anonymous namespace)::MipsTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo18getDefaultFeaturesERKSsRN4llvm9StringMapIbNS3_15MallocAllocatorEEE", "(anonymous namespace)::MipsTargetInfo::getDefaultFeatures(std::string const&, llvm::StringMap<bool, llvm::MallocAllocator>&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo6getABIEv", "(anonymous namespace)::MipsTargetInfo::getABI() const"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfo6setCPUERKSs", "(anonymous namespace)::MipsTargetInfo::setCPU(std::string const&)"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfo6setABIERKSs", "(anonymous namespace)::MipsTargetInfo::setABI(std::string const&)"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::MipsTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::MipsTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::MipsTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo14getArchDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MipsTargetInfo::getArchDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfoD1Ev", "(anonymous namespace)::MipsTargetInfo::~MipsTargetInfo()"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfoD0Ev", "(anonymous namespace)::MipsTargetInfo::~MipsTargetInfo()"},
    {"_ZNK12_GLOBAL__N_114MipsTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MipsTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16MipselTargetInfoEED1Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipselTargetInfo>::~LinuxTargetInfo()"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_16MipselTargetInfoEED0Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipselTargetInfo>::~LinuxTargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_16MipselTargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipselTargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_115LinuxTargetInfoINS_16MipselTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipselTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16MipselTargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipselTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_16MipselTargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipselTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_113PSPTargetInfoINS_16MipselTargetInfoEED1Ev", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipselTargetInfo>::~PSPTargetInfo()"},
    {"_ZN12_GLOBAL__N_113PSPTargetInfoINS_16MipselTargetInfoEED0Ev", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipselTargetInfo>::~PSPTargetInfo()"},
    {"_ZNK12_GLOBAL__N_113PSPTargetInfoINS_16MipselTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipselTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_14MipsTargetInfoEED1Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipsTargetInfo>::~LinuxTargetInfo()"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_14MipsTargetInfoEED0Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipsTargetInfo>::~LinuxTargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_14MipsTargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipsTargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_115LinuxTargetInfoINS_14MipsTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipsTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_14MipsTargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipsTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_14MipsTargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::MipsTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_113PSPTargetInfoINS_14MipsTargetInfoEED1Ev", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipsTargetInfo>::~PSPTargetInfo()"},
    {"_ZN12_GLOBAL__N_113PSPTargetInfoINS_14MipsTargetInfoEED0Ev", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipsTargetInfo>::~PSPTargetInfo()"},
    {"_ZNK12_GLOBAL__N_113PSPTargetInfoINS_14MipsTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipsTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116MSP430TargetInfoD1Ev", "(anonymous namespace)::MSP430TargetInfo::~MSP430TargetInfo()"},
    {"_ZN12_GLOBAL__N_116MSP430TargetInfoD0Ev", "(anonymous namespace)::MSP430TargetInfo::~MSP430TargetInfo()"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::MSP430TargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::MSP430TargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo20getVAListDeclarationEv", "(anonymous namespace)::MSP430TargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo11getClobbersEv", "(anonymous namespace)::MSP430TargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::MSP430TargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::MSP430TargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_116MSP430TargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::MSP430TargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_118BlackfinTargetInfoD1Ev", "(anonymous namespace)::BlackfinTargetInfo::~BlackfinTargetInfo()"},
    {"_ZN12_GLOBAL__N_118BlackfinTargetInfoD0Ev", "(anonymous namespace)::BlackfinTargetInfo::~BlackfinTargetInfo()"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::BlackfinTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::BlackfinTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo20getVAListDeclarationEv", "(anonymous namespace)::BlackfinTargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo11getClobbersEv", "(anonymous namespace)::BlackfinTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::BlackfinTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::BlackfinTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_118BlackfinTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::BlackfinTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfoC2ERKSs", "(anonymous namespace)::ARMTargetInfo::ARMTargetInfo(std::string const&)"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfoD1Ev", "(anonymous namespace)::ARMTargetInfo::~ARMTargetInfo()"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfoD0Ev", "(anonymous namespace)::ARMTargetInfo::~ARMTargetInfo()"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo16getTargetDefinesERKN5clang11LangOptionsERNS1_12MacroBuilderE", "(anonymous namespace)::ARMTargetInfo::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo17getTargetBuiltinsERPKN5clang7Builtin4InfoERj", "(anonymous namespace)::ARMTargetInfo::getTargetBuiltins(clang::Builtin::Info const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo20getVAListDeclarationEv", "(anonymous namespace)::ARMTargetInfo::getVAListDeclaration() const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo11getClobbersEv", "(anonymous namespace)::ARMTargetInfo::getClobbers() const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo18getDefaultFeaturesERKSsRN4llvm9StringMapIbNS3_15MallocAllocatorEEE", "(anonymous namespace)::ARMTargetInfo::getDefaultFeatures(std::string const&, llvm::StringMap<bool, llvm::MallocAllocator>&) const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo6getABIEv", "(anonymous namespace)::ARMTargetInfo::getABI() const"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo6setCPUERKSs", "(anonymous namespace)::ARMTargetInfo::setCPU(std::string const&)"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo6setABIERKSs", "(anonymous namespace)::ARMTargetInfo::setABI(std::string const&)"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo17setFeatureEnabledERN4llvm9StringMapIbNS1_15MallocAllocatorEEERKSsb", "(anonymous namespace)::ARMTargetInfo::setFeatureEnabled(llvm::StringMap<bool, llvm::MallocAllocator>&, std::string const&, bool) const"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo20HandleTargetFeaturesERSt6vectorISsSaISsEE", "(anonymous namespace)::ARMTargetInfo::HandleTargetFeatures(std::vector<std::string, std::allocator<std::string> >&)"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo14getGCCRegNamesERPKPKcRj", "(anonymous namespace)::ARMTargetInfo::getGCCRegNames(char const* const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo16getGCCRegAliasesERPKN5clang10TargetInfo11GCCRegAliasERj", "(anonymous namespace)::ARMTargetInfo::getGCCRegAliases(clang::TargetInfo::GCCRegAlias const*&, unsigned int&) const"},
    {"_ZNK12_GLOBAL__N_113ARMTargetInfo21validateAsmConstraintERPKcRN5clang10TargetInfo14ConstraintInfoE", "(anonymous namespace)::ARMTargetInfo::validateAsmConstraint(char const*&, clang::TargetInfo::ConstraintInfo&) const"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo18getCPUDefineSuffixEN4llvm9StringRefE", "(anonymous namespace)::ARMTargetInfo::getCPUDefineSuffix(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_13ARMTargetInfoEED1Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::ARMTargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZN12_GLOBAL__N_117FreeBSDTargetInfoINS_13ARMTargetInfoEED0Ev", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::ARMTargetInfo>::~FreeBSDTargetInfo()"},
    {"_ZNK12_GLOBAL__N_112OSTargetInfoINS_13ARMTargetInfoEE16getTargetDefinesERKN5clang11LangOptionsERNS3_12MacroBuilderE", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::ARMTargetInfo>::getTargetDefines(clang::LangOptions const&, clang::MacroBuilder&) const"},
    {"_ZNK12_GLOBAL__N_117FreeBSDTargetInfoINS_13ARMTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::ARMTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_13ARMTargetInfoEED1Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::ARMTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_112OSTargetInfoINS_13ARMTargetInfoEED0Ev", "(anonymous namespace)::OSTargetInfo<(anonymous namespace)::ARMTargetInfo>::~OSTargetInfo()"},
    {"_ZN12_GLOBAL__N_119DarwinARMTargetInfoD1Ev", "(anonymous namespace)::DarwinARMTargetInfo::~DarwinARMTargetInfo()"},
    {"_ZN12_GLOBAL__N_119DarwinARMTargetInfoD0Ev", "(anonymous namespace)::DarwinARMTargetInfo::~DarwinARMTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEE23isValidSectionSpecifierEN4llvm9StringRefE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEE29getStaticInitSectionSpecifierEv", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>::getStaticInitSectionSpecifier() const"},
    {"_ZNK12_GLOBAL__N_119DarwinARMTargetInfo12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS1_12MacroBuilderE", "(anonymous namespace)::DarwinARMTargetInfo::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEED1Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>::~DarwinTargetInfo()"},
    {"_ZN12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEED0Ev", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>::~DarwinTargetInfo()"},
    {"_ZNK12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_13ARMTargetInfoEED1Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::ARMTargetInfo>::~LinuxTargetInfo()"},
    {"_ZN12_GLOBAL__N_115LinuxTargetInfoINS_13ARMTargetInfoEED0Ev", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::ARMTargetInfo>::~LinuxTargetInfo()"},
    {"_ZNK12_GLOBAL__N_115LinuxTargetInfoINS_13ARMTargetInfoEE12getOSDefinesERKN5clang11LangOptionsERKN4llvm6TripleERNS3_12MacroBuilderE", "(anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::ARMTargetInfo>::getOSDefines(clang::LangOptions const&, llvm::Triple const&, clang::MacroBuilder&) const"},
    {"_ZN5clang10TargetInfo9setCXXABIERKSs", "clang::TargetInfo::setCXXABI(std::string const&)"},
    {"_ZNK5clang10TargetInfo31useGlobalsForAutomaticVariablesEv", "clang::TargetInfo::useGlobalsForAutomaticVariables() const"},
    {"_ZNK5clang10TargetInfo18getCFStringSectionEv", "clang::TargetInfo::getCFStringSection() const"},
    {"_ZNK5clang10TargetInfo18getNSStringSectionEv", "clang::TargetInfo::getNSStringSection() const"},
    {"_ZNK5clang10TargetInfo31getNSStringNonFragileABISectionEv", "clang::TargetInfo::getNSStringNonFragileABISection() const"},
    {"_ZNK5clang10TargetInfo23isValidSectionSpecifierEN4llvm9StringRefE", "clang::TargetInfo::isValidSectionSpecifier(llvm::StringRef) const"},
    {"_ZNK5clang10TargetInfo6getABIEv", "clang::TargetInfo::getABI() const"},
    {"_ZNK5clang10TargetInfo9getCXXABIEv", "clang::TargetInfo::getCXXABI() const"},
    {"_ZN5clang10TargetInfo6setCPUERKSs", "clang::TargetInfo::setCPU(std::string const&)"},
    {"_ZN5clang10TargetInfo6setABIERKSs", "clang::TargetInfo::setABI(std::string const&)"},
    {"_ZN5clang10TargetInfo9setCXXABIENS_12TargetCXXABIE", "clang::TargetInfo::setCXXABI(clang::TargetCXXABI)"},
    {"_ZNK5clang10TargetInfo29getStaticInitSectionSpecifierEv", "clang::TargetInfo::getStaticInitSectionSpecifier() const"},
    {"_ZNK5clang10TargetInfo16getPointerWidthVEj", "clang::TargetInfo::getPointerWidthV(unsigned int) const"},
    {"_ZNK5clang10TargetInfo16getPointerAlignVEj", "clang::TargetInfo::getPointerAlignV(unsigned int) const"},
    {"_ZNK5clang10TargetInfo15getPtrDiffTypeVEj", "clang::TargetInfo::getPtrDiffTypeV(unsigned int) const"},
    {"_ZN5clang12MacroBuilder11defineMacroERKN4llvm5TwineES4_", "clang::MacroBuilder::defineMacro(llvm::Twine const&, llvm::Twine const&)"},
    {"_ZNK5clang10TargetInfo23getEHDataRegisterNumberEj", "clang::TargetInfo::getEHDataRegisterNumber(unsigned int) const"},
    {"_ZNK5clang10TargetInfo17convertConstraintEc", "clang::TargetInfo::convertConstraint(char) const"},
    {"_ZNK5clang10TargetInfo18getDefaultFeaturesERKSsRN4llvm9StringMapIbNS3_15MallocAllocatorEEE", "clang::TargetInfo::getDefaultFeatures(std::string const&, llvm::StringMap<bool, llvm::MallocAllocator>&) const"},
    {"_ZNK5clang10TargetInfo17setFeatureEnabledERN4llvm9StringMapIbNS1_15MallocAllocatorEEERKSsb", "clang::TargetInfo::setFeatureEnabled(llvm::StringMap<bool, llvm::MallocAllocator>&, std::string const&, bool) const"},
    {"_ZN5clang10TargetInfo20HandleTargetFeaturesERSt6vectorISsSaISsEE", "clang::TargetInfo::HandleTargetFeatures(std::vector<std::string, std::allocator<std::string> >&)"},
    {"_ZNK5clang10TargetInfo19adjustInlineAsmTypeERSsPKN4llvm4TypeERNS2_11LLVMContextE", "clang::TargetInfo::adjustInlineAsmType(std::string&, llvm::Type const*, llvm::LLVMContext&) const"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPSsSt6vectorISsSaISsEEEEA16_cET_S8_S8_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > > std::__find<__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, char [16]>(__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, __gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, char const (&) [16], std::random_access_iterator_tag)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPSsSt6vectorISsSaISsEEEEA12_cET_S8_S8_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > > std::__find<__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, char [12]>(__gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, __gnu_cxx::__normal_iterator<std::string*, std::vector<std::string, std::allocator<std::string> > >, char const (&) [12], std::random_access_iterator_tag)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj10ELj13ELj14ELj14EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RAT2__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<10u, 13u, 14u, 14u>(char const (&) [10u], char const (&) [13u], char const (&) [14u], char const (&) [14u], char const* const&)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj9ELj11ELj8ELj8ELj5EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RAT2__S1_RAT3__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<9u, 11u, 8u, 8u, 5u>(char const (&) [9u], char const (&) [11u], char const (&) [8u], char const (&) [8u], char const (&) [5u], char const* const&)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj9ELj7ELj8ELj8ELj8EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RAT2__S1_RAT3__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<9u, 7u, 8u, 8u, 8u>(char const (&) [9u], char const (&) [7u], char const (&) [8u], char const (&) [8u], char const (&) [8u], char const* const&)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj6ELj10ELj10ELj10EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RAT2__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<6u, 10u, 10u, 10u>(char const (&) [6u], char const (&) [10u], char const (&) [10u], char const (&) [10u], char const* const&)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj7ELj9ELj9EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<7u, 9u, 9u>(char const (&) [7u], char const (&) [9u], char const (&) [9u], char const* const&)"},
    {"_ZN4llvm12StringSwitchIPKcS2_E5CasesILj12ELj12ELj7EEERS3_RAT__S1_RAT0__S1_RAT1__S1_RKS2_", "llvm::StringSwitch<char const*, char const*>& llvm::StringSwitch<char const*, char const*>::Cases<12u, 12u, 7u>(char const (&) [12u], char const (&) [12u], char const (&) [7u], char const* const&)"},
    {"_ZN5clang3tok12getTokenNameENS0_9TokenKindE", "clang::tok::getTokenName(clang::tok::TokenKind)"},
    {"_ZN5clang3tok22getTokenSimpleSpellingENS0_9TokenKindE", "clang::tok::getTokenSimpleSpelling(clang::tok::TokenKind)"},
    {"_ZN5clang22getClangRepositoryPathEv", "clang::getClangRepositoryPath()"},
    {"_ZN5clang16getClangRevisionEv", "clang::getClangRevision()"},
    {"_ZN5clang29getClangFullRepositoryVersionEv", "clang::getClangFullRepositoryVersion()"},
    {"_ZN5clang19getClangFullVersionEv", "clang::getClangFullVersion()"},
    {"_ZN4llvm26initializeArgPromotionPassERNS_12PassRegistryE", "llvm::initializeArgPromotionPass(llvm::PassRegistry&)"},
    {"_ZN4llvm27createArgumentPromotionPassEj", "llvm::createArgumentPromotionPass(unsigned int)"},
    {"_ZN12_GLOBAL__N_112ArgPromotionD1Ev", "(anonymous namespace)::ArgPromotion::~ArgPromotion()"},
    {"_ZN12_GLOBAL__N_112ArgPromotionD0Ev", "(anonymous namespace)::ArgPromotion::~ArgPromotion()"},
    {"_ZNK12_GLOBAL__N_112ArgPromotion16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::ArgPromotion::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_112ArgPromotion8runOnSCCERN4llvm12CallGraphSCCE", "(anonymous namespace)::ArgPromotion::runOnSCC(llvm::CallGraphSCC&)"},
    {"_ZNK12_GLOBAL__N_112ArgPromotion23isSafeToPromoteArgumentEPN4llvm8ArgumentEb", "(anonymous namespace)::ArgPromotion::isSafeToPromoteArgument(llvm::Argument*, bool) const"},
    {"_ZN12_GLOBAL__N_112ArgPromotion11DoPromotionEPN4llvm8FunctionERNS1_11SmallPtrSetIPNS1_8ArgumentELj8EEES8_", "(anonymous namespace)::ArgPromotion::DoPromotion(llvm::Function*, llvm::SmallPtrSet<llvm::Argument*, 8u>&, llvm::SmallPtrSet<llvm::Argument*, 8u>&)"},
    {"_Z15MarkIndicesSafeRKSt6vectorIySaIyEERSt3setIS1_St4lessIS1_ESaIS1_EE", "MarkIndicesSafe(std::vector<unsigned long long, std::allocator<unsigned long long> > const&, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112ArgPromotionEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::ArgPromotion>()"},
    {"_ZNK4llvm16CallGraphSCCPass27getPotentialPassManagerTypeEv", "llvm::CallGraphSCCPass::getPotentialPassManagerType() const"},
    {"_ZN4llvm16CallGraphSCCPass16doInitializationERNS_9CallGraphE", "llvm::CallGraphSCCPass::doInitialization(llvm::CallGraph&)"},
    {"_ZN4llvm16CallGraphSCCPass14doFinalizationERNS_9CallGraphE", "llvm::CallGraphSCCPass::doFinalization(llvm::CallGraph&)"},
    {"_ZNSt3mapIPN4llvm8ArgumentESt3setISt6vectorIySaIyEESt4lessIS6_ESaIS6_EES7_IS2_ESaISt4pairIKS2_SA_EEEixERSD_", "std::map<llvm::Argument*, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >, std::less<llvm::Argument*>, std::allocator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > > >::operator[](llvm::Argument* const&)"},
    {"_ZNSt3mapISt6vectorIySaIyEEPN4llvm8LoadInstESt4lessIS2_ESaISt4pairIKS2_S5_EEEixERS9_", "std::map<std::vector<unsigned long long, std::allocator<unsigned long long> >, llvm::LoadInst*, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> > >::operator[](std::vector<unsigned long long, std::allocator<unsigned long long> > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm8ArgumentESt4pairIKS2_St3setISt6vectorIySaIyEESt4lessIS8_ESaIS8_EEESt10_Select1stISD_ES9_IS2_ESaISD_EE8_M_eraseEPSt13_Rb_tree_nodeISD_E", "std::_Rb_tree<llvm::Argument*, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > >, std::_Select1st<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >, std::less<llvm::Argument*>, std::allocator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >*)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::_M_erase(std::_Rb_tree_node<std::vector<unsigned long long, std::allocator<unsigned long long> > >*)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEESt4pairIKS2_PN4llvm8LoadInstEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*>, std::_Select1st<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >*)"},
    {"_ZN4llvm17GetElementPtrInstC2IN9__gnu_cxx17__normal_iteratorIPPNS_5ValueESt6vectorIS5_SaIS5_EEEEEES5_T_SB_jRKNS_5TwineEPNS_11InstructionE", "llvm::GetElementPtrInst::GetElementPtrInst<__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >(llvm::Value*, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEESt4pairIKS2_PN4llvm8LoadInstEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*>, std::_Select1st<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >, std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEESt4pairIKS2_PN4llvm8LoadInstEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*>, std::_Select1st<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> > >::_M_insert_unique(std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEESt4pairIKS2_PN4llvm8LoadInstEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE14_M_create_nodeERKS8_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*>, std::_Select1st<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> > >::_M_create_node(std::pair<std::vector<unsigned long long, std::allocator<unsigned long long> > const, llvm::LoadInst*> const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::_M_insert_unique(std::vector<unsigned long long, std::allocator<unsigned long long> > const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE14_M_create_nodeERKS2_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::_M_create_node(std::vector<unsigned long long, std::allocator<unsigned long long> > const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE7_M_copyEPKSt13_Rb_tree_nodeIS2_EPSA_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::_M_copy(std::_Rb_tree_node<std::vector<unsigned long long, std::allocator<unsigned long long> > > const*, std::_Rb_tree_node<std::vector<unsigned long long, std::allocator<unsigned long long> > >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm8ArgumentESt4pairIKS2_St3setISt6vectorIySaIyEESt4lessIS8_ESaIS8_EEESt10_Select1stISD_ES9_IS2_ESaISD_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISD_ERKSD_", "std::_Rb_tree<llvm::Argument*, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > >, std::_Select1st<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >, std::less<llvm::Argument*>, std::allocator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm8ArgumentESt4pairIKS2_St3setISt6vectorIySaIyEESt4lessIS8_ESaIS8_EEESt10_Select1stISD_ES9_IS2_ESaISD_EE9_M_insertEPSt18_Rb_tree_node_baseSK_RKSD_", "std::_Rb_tree<llvm::Argument*, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > >, std::_Select1st<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >, std::less<llvm::Argument*>, std::allocator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm8ArgumentESt4pairIKS2_St3setISt6vectorIySaIyEESt4lessIS8_ESaIS8_EEESt10_Select1stISD_ES9_IS2_ESaISD_EE16_M_insert_uniqueERKSD_", "std::_Rb_tree<llvm::Argument*, std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > >, std::_Select1st<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > >, std::less<llvm::Argument*>, std::allocator<std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > > >::_M_insert_unique(std::pair<llvm::Argument* const, std::set<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > > > const&)"},
    {"_ZN4llvm11df_iteratorINS_7InverseIPNS_10BasicBlockEEENS_11SmallPtrSetIS3_Lj16EEELb1ENS_11GraphTraitsIS4_EEE6toNextEv", "llvm::df_iterator<llvm::Inverse<llvm::BasicBlock*>, llvm::SmallPtrSet<llvm::BasicBlock*, 16u>, true, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> > >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_10BasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEENS1_12PredIteratorIS3_NS1_18value_use_iteratorINS1_4UserEEEEEESaISD_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSD_SF_EERKSD_", "std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >*, std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > > > > >, std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > > const&)"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_10BasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEENS1_12PredIteratorIS3_NS1_18value_use_iteratorINS1_4UserEEEEEESaISD_EEC2ERKSF_", "std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > > > >::vector(std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > > > > const&)"},
    {"_ZN4llvm11df_iteratorINS_7InverseIPNS_10BasicBlockEEENS_11SmallPtrSetIS3_Lj16EEELb1ENS_11GraphTraitsIS4_EEEC2ES3_RS6_", "llvm::df_iterator<llvm::Inverse<llvm::BasicBlock*>, llvm::SmallPtrSet<llvm::BasicBlock*, 16u>, true, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> > >::df_iterator(llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 16u>&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE4findERKS2_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::find(std::vector<unsigned long long, std::allocator<unsigned long long> > const&)"},
    {"_ZNSt8_Rb_treeISt6vectorIySaIyEES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueESt23_Rb_tree_const_iteratorIS2_ERKS2_", "std::_Rb_tree<std::vector<unsigned long long, std::allocator<unsigned long long> >, std::vector<unsigned long long, std::allocator<unsigned long long> >, std::_Identity<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::less<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::allocator<std::vector<unsigned long long, std::allocator<unsigned long long> > > >::_M_insert_unique(std::_Rb_tree_const_iterator<std::vector<unsigned long long, std::allocator<unsigned long long> > >, std::vector<unsigned long long, std::allocator<unsigned long long> > const&)"},
    {"_ZN4llvm16CallGraphSCCPassD1Ev", "llvm::CallGraphSCCPass::~CallGraphSCCPass()"},
    {"_ZN4llvm16CallGraphSCCPassD0Ev", "llvm::CallGraphSCCPass::~CallGraphSCCPass()"},
    {"_ZN4llvm27initializeConstantMergePassERNS_12PassRegistryE", "llvm::initializeConstantMergePass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createConstantMergePassEv", "llvm::createConstantMergePass()"},
    {"_ZN12_GLOBAL__N_113ConstantMergeD1Ev", "(anonymous namespace)::ConstantMerge::~ConstantMerge()"},
    {"_ZN12_GLOBAL__N_113ConstantMergeD0Ev", "(anonymous namespace)::ConstantMerge::~ConstantMerge()"},
    {"_ZN12_GLOBAL__N_113ConstantMerge11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::ConstantMerge::runOnModule(llvm::Module&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113ConstantMergeEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::ConstantMerge>()"},
    {"_ZN4llvm8DenseMapIPNS_8ConstantEPNS_14GlobalVariableENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Constant*, llvm::GlobalVariable*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_8ConstantEPNS_14GlobalVariableENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Constant*, llvm::GlobalVariable*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::InsertIntoBucket(llvm::Constant* const&, llvm::GlobalVariable* const&, std::pair<llvm::Constant*, llvm::GlobalVariable*>*)"},
    {"_ZN4llvm8DenseMapIPNS_8ConstantEPNS_14GlobalVariableENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Constant*, llvm::GlobalVariable*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::GlobalVariable*> >::grow(unsigned int)"},
    {"_ZN4llvm17initializeDAEPassERNS_12PassRegistryE", "llvm::initializeDAEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm17initializeDAHPassERNS_12PassRegistryE", "llvm::initializeDAHPass(llvm::PassRegistry&)"},
    {"_ZN4llvm28createDeadArgEliminationPassEv", "llvm::createDeadArgEliminationPass()"},
    {"_ZN4llvm24createDeadArgHackingPassEv", "llvm::createDeadArgHackingPass()"},
    {"_ZN12_GLOBAL__N_13DAHD1Ev", "(anonymous namespace)::DAH::~DAH()"},
    {"_ZN12_GLOBAL__N_13DAHD0Ev", "(anonymous namespace)::DAH::~DAH()"},
    {"_ZN12_GLOBAL__N_13DAE11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::DAE::runOnModule(llvm::Module&)"},
    {"_ZNK12_GLOBAL__N_13DAH19ShouldHackArgumentsEv", "(anonymous namespace)::DAH::ShouldHackArguments() const"},
    {"_ZNSt3setIN12_GLOBAL__N_13DAE8RetOrArgESt4lessIS2_ESaIS2_EE5eraseERKS2_", "std::set<(anonymous namespace)::DAE::RetOrArg, std::less<(anonymous namespace)::DAE::RetOrArg>, std::allocator<(anonymous namespace)::DAE::RetOrArg> >::erase((anonymous namespace)::DAE::RetOrArg const&)"},
    {"_ZNSt8_Rb_treeIN12_GLOBAL__N_13DAE8RetOrArgES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<(anonymous namespace)::DAE::RetOrArg, (anonymous namespace)::DAE::RetOrArg, std::_Identity<(anonymous namespace)::DAE::RetOrArg>, std::less<(anonymous namespace)::DAE::RetOrArg>, std::allocator<(anonymous namespace)::DAE::RetOrArg> >::_M_erase(std::_Rb_tree_node<(anonymous namespace)::DAE::RetOrArg>*)"},
    {"_ZN12_GLOBAL__N_13DAE8MarkLiveERKN4llvm8FunctionE", "(anonymous namespace)::DAE::MarkLive(llvm::Function const&)"},
    {"_ZN12_GLOBAL__N_13DAE9MarkValueERKNS0_8RetOrArgENS0_8LivenessERKN4llvm11SmallVectorIS1_Lj5EEE", "(anonymous namespace)::DAE::MarkValue((anonymous namespace)::DAE::RetOrArg const&, (anonymous namespace)::DAE::Liveness, llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u> const&)"},
    {"_ZN12_GLOBAL__N_13DAE8MarkLiveERKNS0_8RetOrArgE", "(anonymous namespace)::DAE::MarkLive((anonymous namespace)::DAE::RetOrArg const&)"},
    {"_ZN12_GLOBAL__N_13DAE17PropagateLivenessERKNS0_8RetOrArgE", "(anonymous namespace)::DAE::PropagateLiveness((anonymous namespace)::DAE::RetOrArg const&)"},
    {"_ZNSt8_Rb_treeIN12_GLOBAL__N_13DAE8RetOrArgESt4pairIKS2_S2_ESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<(anonymous namespace)::DAE::RetOrArg, std::pair<(anonymous namespace)::DAE::RetOrArg const, (anonymous namespace)::DAE::RetOrArg>, std::_Select1st<std::pair<(anonymous namespace)::DAE::RetOrArg const, (anonymous namespace)::DAE::RetOrArg> >, std::less<(anonymous namespace)::DAE::RetOrArg>, std::allocator<std::pair<(anonymous namespace)::DAE::RetOrArg const, (anonymous namespace)::DAE::RetOrArg> > >::_M_erase(std::_Rb_tree_node<std::pair<(anonymous namespace)::DAE::RetOrArg const, (anonymous namespace)::DAE::RetOrArg> >*)"},
    {"_ZN12_GLOBAL__N_13DAE9SurveyUseEN4llvm18value_use_iteratorIKNS1_4UserEEERNS1_11SmallVectorINS0_8RetOrArgELj5EEEj", "(anonymous namespace)::DAE::SurveyUse(llvm::value_use_iterator<llvm::User const>, llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u>&, unsigned int)"},
    {"_ZN12_GLOBAL__N_13DAE13MarkIfNotLiveENS0_8RetOrArgERN4llvm11SmallVectorIS1_Lj5EEE", "(anonymous namespace)::DAE::MarkIfNotLive((anonymous namespace)::DAE::RetOrArg, llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u>&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_13DAE8RetOrArgELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::DAE::RetOrArg, false>::grow(unsigned long)"},
    {"_ZN4llvm11SmallVectorIN12_GLOBAL__N_13DAE8RetOrArgELj5EEC1ERKS4_", "llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u>::SmallVector(llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIN12_GLOBAL__N_13DAE8RetOrArgELj5EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<(anonymous namespace)::DAE::RetOrArg, 5u>, false>::grow(unsigned long)"},
    {"_ZN12_GLOBAL__N_13DAED1Ev", "(anonymous namespace)::DAE::~DAE()"},
    {"_ZN12_GLOBAL__N_13DAED0Ev", "(anonymous namespace)::DAE::~DAE()"},
    {"_ZNK12_GLOBAL__N_13DAE19ShouldHackArgumentsEv", "(anonymous namespace)::DAE::ShouldHackArguments() const"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_13DAHEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::DAH>()"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_13DAEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::DAE>()"},
    {"_ZN4llvm8CallInstC2IN9__gnu_cxx17__normal_iteratorIPPNS_5ValueESt6vectorIS5_SaIS5_EEEEEES5_T_SB_RKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst<__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >(llvm::Value*, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10InvokeInstC2IN9__gnu_cxx17__normal_iteratorIPPNS_5ValueESt6vectorIS5_SaIS5_EEEEEES5_PNS_10BasicBlockESC_T_SD_jRKNS_5TwineEPNS_11InstructionE", "llvm::InvokeInst::InvokeInst<__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, __gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm11SmallVectorIiLj5EEC2EjRKi", "llvm::SmallVector<int, 5u>::SmallVector(unsigned int, int const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE16_M_insert_uniqueERKS3_", "std::_Rb_tree<llvm::Function const*, llvm::Function const*, std::_Identity<llvm::Function const*>, std::less<llvm::Function const*>, std::allocator<llvm::Function const*> >::_M_insert_unique(llvm::Function const* const&)"},
    {"_ZNSt6vectorIPN4llvm5ValueESaIS2_EE13_M_assign_auxIPNS0_3UseEEEvT_S8_St20forward_iterator_tag", "void std::vector<llvm::Value*, std::allocator<llvm::Value*> >::_M_assign_aux<llvm::Use*>(llvm::Use*, llvm::Use*, std::forward_iterator_tag)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE8_M_eraseEPSt13_Rb_tree_nodeIS3_E", "std::_Rb_tree<llvm::Function const*, llvm::Function const*, std::_Identity<llvm::Function const*>, std::less<llvm::Function const*>, std::allocator<llvm::Function const*> >::_M_erase(std::_Rb_tree_node<llvm::Function const*>*)"},
    {"_ZN4llvm17initializeDTEPassERNS_12PassRegistryE", "llvm::initializeDTEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm29createDeadTypeEliminationPassEv", "llvm::createDeadTypeEliminationPass()"},
    {"_ZN12_GLOBAL__N_13DTED1Ev", "(anonymous namespace)::DTE::~DTE()"},
    {"_ZN12_GLOBAL__N_13DTED0Ev", "(anonymous namespace)::DTE::~DTE()"},
    {"_ZNK12_GLOBAL__N_13DTE16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::DTE::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_13DTE11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::DTE::runOnModule(llvm::Module&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_13DTEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::DTE>()"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE8_M_eraseEPSt13_Rb_tree_nodeIS3_E", "std::_Rb_tree<llvm::Type const*, llvm::Type const*, std::_Identity<llvm::Type const*>, std::less<llvm::Type const*>, std::allocator<llvm::Type const*> >::_M_erase(std::_Rb_tree_node<llvm::Type const*>*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE5eraseESt17_Rb_tree_iteratorIS3_ESB_", "std::_Rb_tree<llvm::Type const*, llvm::Type const*, std::_Identity<llvm::Type const*>, std::less<llvm::Type const*>, std::allocator<llvm::Type const*> >::erase(std::_Rb_tree_iterator<llvm::Type const*>, std::_Rb_tree_iterator<llvm::Type const*>)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE7_M_copyEPKSt13_Rb_tree_nodeIS3_EPSB_", "std::_Rb_tree<llvm::Type const*, llvm::Type const*, std::_Identity<llvm::Type const*>, std::less<llvm::Type const*>, std::allocator<llvm::Type const*> >::_M_copy(std::_Rb_tree_node<llvm::Type const*> const*, std::_Rb_tree_node<llvm::Type const*>*)"},
    {"_ZN4llvm27initializeFunctionAttrsPassERNS_12PassRegistryE", "llvm::initializeFunctionAttrsPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createFunctionAttrsPassEv", "llvm::createFunctionAttrsPass()"},
    {"_ZN12_GLOBAL__N_113FunctionAttrsD1Ev", "(anonymous namespace)::FunctionAttrs::~FunctionAttrs()"},
    {"_ZN12_GLOBAL__N_113FunctionAttrsD0Ev", "(anonymous namespace)::FunctionAttrs::~FunctionAttrs()"},
    {"_ZNK12_GLOBAL__N_113FunctionAttrs16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::FunctionAttrs::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_113FunctionAttrs8runOnSCCERN4llvm12CallGraphSCCE", "(anonymous namespace)::FunctionAttrs::runOnSCC(llvm::CallGraphSCC&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113FunctionAttrsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::FunctionAttrs>()"},
    {"_ZN4llvm12UniqueVectorIPNS_5ValueEE6insertERKS2_", "llvm::UniqueVector<llvm::Value*>::insert(llvm::Value* const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm5ValueESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::Value*, std::pair<llvm::Value* const, unsigned int>, std::_Select1st<std::pair<llvm::Value* const, unsigned int> >, std::less<llvm::Value*>, std::allocator<std::pair<llvm::Value* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Value* const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm5ValueESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<llvm::Value*, std::pair<llvm::Value* const, unsigned int>, std::_Select1st<std::pair<llvm::Value* const, unsigned int> >, std::less<llvm::Value*>, std::allocator<std::pair<llvm::Value* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Value* const, unsigned int> >, std::pair<llvm::Value* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm5ValueESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::Value*, std::pair<llvm::Value* const, unsigned int>, std::_Select1st<std::pair<llvm::Value* const, unsigned int> >, std::less<llvm::Value*>, std::allocator<std::pair<llvm::Value* const, unsigned int> > >::_M_insert_unique(std::pair<llvm::Value* const, unsigned int> const&)"},
    {"_ZN4llvm23initializeGlobalDCEPassERNS_12PassRegistryE", "llvm::initializeGlobalDCEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm19createGlobalDCEPassEv", "llvm::createGlobalDCEPass()"},
    {"_ZN12_GLOBAL__N_19GlobalDCED1Ev", "(anonymous namespace)::GlobalDCE::~GlobalDCE()"},
    {"_ZN12_GLOBAL__N_19GlobalDCED0Ev", "(anonymous namespace)::GlobalDCE::~GlobalDCE()"},
    {"_ZN12_GLOBAL__N_19GlobalDCE11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::GlobalDCE::runOnModule(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_19GlobalDCE14GlobalIsNeededEPN4llvm11GlobalValueE", "(anonymous namespace)::GlobalDCE::GlobalIsNeeded(llvm::GlobalValue*)"},
    {"_ZN12_GLOBAL__N_19GlobalDCE23MarkUsedGlobalsAsNeededEPN4llvm8ConstantE", "(anonymous namespace)::GlobalDCE::MarkUsedGlobalsAsNeeded(llvm::Constant*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_19GlobalDCEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::GlobalDCE>()"},
    {"_ZNSt6vectorIPN4llvm11GlobalAliasESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::GlobalAlias*, std::allocator<llvm::GlobalAlias*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GlobalAlias**, std::vector<llvm::GlobalAlias*, std::allocator<llvm::GlobalAlias*> > >, llvm::GlobalAlias* const&)"},
    {"_ZN4llvm23initializeGlobalOptPassERNS_12PassRegistryE", "llvm::initializeGlobalOptPass(llvm::PassRegistry&)"},
    {"_ZN4llvm25createGlobalOptimizerPassEv", "llvm::createGlobalOptimizerPass()"},
    {"_ZN12_GLOBAL__N_19GlobalOptD1Ev", "(anonymous namespace)::GlobalOpt::~GlobalOpt()"},
    {"_ZN12_GLOBAL__N_19GlobalOptD0Ev", "(anonymous namespace)::GlobalOpt::~GlobalOpt()"},
    {"_ZNK12_GLOBAL__N_19GlobalOpt16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::GlobalOpt::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_19GlobalOpt11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::GlobalOpt::runOnModule(llvm::Module&)"},
    {"_Z13AnalyzeGlobalPKN4llvm5ValueERN12_GLOBAL__N_112GlobalStatusERNS_11SmallPtrSetIPKNS_7PHINodeELj16EEE", "AnalyzeGlobal(llvm::Value const*, (anonymous namespace)::GlobalStatus&, llvm::SmallPtrSet<llvm::PHINode const*, 16u>&)"},
    {"_ZN12_GLOBAL__N_19GlobalOpt21ProcessInternalGlobalEPN4llvm14GlobalVariableERNS1_14ilist_iteratorIS2_EERKNS1_11SmallPtrSetIPKNS1_7PHINodeELj16EEERKNS_12GlobalStatusE", "(anonymous namespace)::GlobalOpt::ProcessInternalGlobal(llvm::GlobalVariable*, llvm::ilist_iterator<llvm::GlobalVariable>&, llvm::SmallPtrSet<llvm::PHINode const*, 16u> const&, (anonymous namespace)::GlobalStatus const&)"},
    {"_Z26CleanupConstantGlobalUsersPN4llvm5ValueEPNS_8ConstantE", "CleanupConstantGlobalUsers(llvm::Value*, llvm::Constant*)"},
    {"_Z41ValueIsOnlyUsedLocallyOrStoredToOneGlobalPKN4llvm11InstructionEPKNS_14GlobalVariableERNS_11SmallPtrSetIPKNS_7PHINodeELj8EEE", "ValueIsOnlyUsedLocallyOrStoredToOneGlobal(llvm::Instruction const*, llvm::GlobalVariable const*, llvm::SmallPtrSet<llvm::PHINode const*, 8u>&)"},
    {"_Z29ReplaceUsesOfMallocWithGlobalPN4llvm11InstructionEPNS_14GlobalVariableE", "ReplaceUsesOfMallocWithGlobal(llvm::Instruction*, llvm::GlobalVariable*)"},
    {"_Z16GetHeapSROAValuePN4llvm5ValueEjRNS_8DenseMapIS1_St6vectorIS1_SaIS1_EENS_12DenseMapInfoIS1_EENS6_IS5_EEEERS3_ISt4pairIPNS_7PHINodeEjESaISE_EE", "GetHeapSROAValue(llvm::Value*, unsigned int, llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >&, std::vector<std::pair<llvm::PHINode*, unsigned int>, std::allocator<std::pair<llvm::PHINode*, unsigned int> > >&)"},
    {"_Z23RewriteHeapSROALoadUserPN4llvm11InstructionERNS_8DenseMapIPNS_5ValueESt6vectorIS4_SaIS4_EENS_12DenseMapInfoIS4_EENS8_IS7_EEEERS5_ISt4pairIPNS_7PHINodeEjESaISG_EE", "RewriteHeapSROALoadUser(llvm::Instruction*, llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >&, std::vector<std::pair<llvm::PHINode*, unsigned int>, std::allocator<std::pair<llvm::PHINode*, unsigned int> > >&)"},
    {"_Z30LoadUsesSimpleEnoughForHeapSRAPKN4llvm5ValueERNS_11SmallPtrSetIPKNS_7PHINodeELj32EEES8_", "LoadUsesSimpleEnoughForHeapSRA(llvm::Value const*, llvm::SmallPtrSet<llvm::PHINode const*, 32u>&, llvm::SmallPtrSet<llvm::PHINode const*, 32u>&)"},
    {"_Z19ConstantPropUsersOfPN4llvm5ValueE", "ConstantPropUsersOf(llvm::Value*)"},
    {"_Z28AllUsesOfValueWillTrapIfNullPKN4llvm5ValueERNS_11SmallPtrSetIPKNS_7PHINodeELj8EEE", "AllUsesOfValueWillTrapIfNull(llvm::Value const*, llvm::SmallPtrSet<llvm::PHINode const*, 8u>&)"},
    {"_Z31OptimizeAwayTrappingUsesOfValuePN4llvm5ValueEPNS_8ConstantE", "OptimizeAwayTrappingUsesOfValue(llvm::Value*, llvm::Constant*)"},
    {"_Z27getAggregateConstantElementPN4llvm8ConstantES1_", "getAggregateConstantElement(llvm::Constant*, llvm::Constant*)"},
    {"_Z20isSafeSROAElementUsePN4llvm5ValueE", "isSafeSROAElementUse(llvm::Value*)"},
    {"_Z21SafeToDestroyConstantPKN4llvm8ConstantE", "SafeToDestroyConstant(llvm::Constant const*)"},
    {"_Z16EvaluateFunctionPN4llvm8FunctionERPNS_8ConstantERKNS_15SmallVectorImplIS3_EERSt6vectorIS1_SaIS1_EERNS_8DenseMapIS3_S3_NS_12DenseMapInfoIS3_EESF_EERS9_IPNS_14GlobalVariableESaISJ_EERNS_11SmallPtrSetIS3_Lj8EEEPKNS_10DataLayoutE", "EvaluateFunction(llvm::Function*, llvm::Constant*&, llvm::SmallVectorImpl<llvm::Constant*> const&, std::vector<llvm::Function*, std::allocator<llvm::Function*> >&, llvm::DenseMap<llvm::Constant*, llvm::Constant*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::Constant*> >&, std::vector<llvm::GlobalVariable*, std::allocator<llvm::GlobalVariable*> >&, llvm::SmallPtrSet<llvm::Constant*, 8u>&, llvm::DataLayout const*)"},
    {"_Z17EvaluateStoreIntoPN4llvm8ConstantES1_PNS_12ConstantExprEj", "EvaluateStoreInto(llvm::Constant*, llvm::Constant*, llvm::ConstantExpr*, unsigned int)"},
    {"_Z33isSimpleEnoughValueToCommitHelperPN4llvm8ConstantERNS_11SmallPtrSetIS1_Lj8EEE", "isSimpleEnoughValueToCommitHelper(llvm::Constant*, llvm::SmallPtrSet<llvm::Constant*, 8u>&)"},
    {"_Z9StripNestRKN4llvm11AttrListPtrE", "StripNest(llvm::AttrListPtr const&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_19GlobalOptEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::GlobalOpt>()"},
    {"_ZNSt6vectorIPN4llvm5ValueESaIS2_EEaSERKS4_", "std::vector<llvm::Value*, std::allocator<llvm::Value*> >::operator=(std::vector<llvm::Value*, std::allocator<llvm::Value*> > const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm7PHINodeEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::PHINode*, unsigned int>, std::allocator<std::pair<llvm::PHINode*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PHINode*, unsigned int>*, std::vector<std::pair<llvm::PHINode*, unsigned int>, std::allocator<std::pair<llvm::PHINode*, unsigned int> > > >, std::pair<llvm::PHINode*, unsigned int> const&)"},
    {"_ZNSt6vectorIPN4llvm5ValueESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::Value*, std::allocator<llvm::Value*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, unsigned long, llvm::Value* const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueESt6vectorIS2_SaIS2_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE6insertERKSt4pairIS2_S5_E", "llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >::insert(std::pair<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> > > const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueESt6vectorIS2_SaIS2_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >::InsertIntoBucket(llvm::Value* const&, std::vector<llvm::Value*, std::allocator<llvm::Value*> > const&, std::pair<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueESt6vectorIS2_SaIS2_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueESt6vectorIS2_SaIS2_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Value*, std::vector<llvm::Value*, std::allocator<llvm::Value*> >, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<std::vector<llvm::Value*, std::allocator<llvm::Value*> > > >::FindAndConstruct(llvm::Value* const&)"},
    {"_ZNSt6vectorIPN4llvm5ValueESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Value*, std::allocator<llvm::Value*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Value**, std::vector<llvm::Value*, std::allocator<llvm::Value*> > >, llvm::Value* const&)"},
    {"_ZNSt6vectorIPN4llvm14GlobalVariableESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::GlobalVariable*, std::allocator<llvm::GlobalVariable*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GlobalVariable**, std::vector<llvm::GlobalVariable*, std::allocator<llvm::GlobalVariable*> > >, llvm::GlobalVariable* const&)"},
    {"_ZNSt6vectorIPN4llvm8ConstantESaIS2_EE14_M_fill_assignEmRKS2_", "std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >::_M_fill_assign(unsigned long, llvm::Constant* const&)"},
    {"_ZN4llvm8DenseMapIPNS_8ConstantES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::Constant*, llvm::Constant*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::Constant*> >::InsertIntoBucket(llvm::Constant* const&, llvm::Constant* const&, std::pair<llvm::Constant*, llvm::Constant*>*)"},
    {"_ZN4llvm8DenseMapIPNS_8ConstantES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::Constant*, llvm::Constant*, llvm::DenseMapInfo<llvm::Constant*>, llvm::DenseMapInfo<llvm::Constant*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_8ConstantENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Value*, llvm::Constant*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::Constant*> >::InsertIntoBucket(llvm::Value* const&, llvm::Constant* const&, std::pair<llvm::Value*, llvm::Constant*>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_8ConstantENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Value*, llvm::Constant*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::Constant*> >::grow(unsigned int)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm8FunctionESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > >, llvm::Function*>(__gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > >, __gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > >, llvm::Function* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm8FunctionESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::Function*, std::allocator<llvm::Function*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > >, unsigned long, llvm::Function* const&)"},
    {"_ZN4llvm27initializeAlwaysInlinerPassERNS_12PassRegistryE", "llvm::initializeAlwaysInlinerPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createAlwaysInlinerPassEv", "llvm::createAlwaysInlinerPass()"},
    {"_ZN12_GLOBAL__N_113AlwaysInlinerD1Ev", "(anonymous namespace)::AlwaysInliner::~AlwaysInliner()"},
    {"_ZN12_GLOBAL__N_113AlwaysInlinerD0Ev", "(anonymous namespace)::AlwaysInliner::~AlwaysInliner()"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner13releaseMemoryEv", "(anonymous namespace)::AlwaysInliner::releaseMemory()"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner16doInitializationERN4llvm9CallGraphE", "(anonymous namespace)::AlwaysInliner::doInitialization(llvm::CallGraph&)"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner14doFinalizationERN4llvm9CallGraphE", "(anonymous namespace)::AlwaysInliner::doFinalization(llvm::CallGraph&)"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner13getInlineCostEN4llvm8CallSiteE", "(anonymous namespace)::AlwaysInliner::getInlineCost(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner20getInlineFudgeFactorEN4llvm8CallSiteE", "(anonymous namespace)::AlwaysInliner::getInlineFudgeFactor(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner19resetCachedCostInfoEPN4llvm8FunctionE", "(anonymous namespace)::AlwaysInliner::resetCachedCostInfo(llvm::Function*)"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner18growCachedCostInfoEPN4llvm8FunctionES3_", "(anonymous namespace)::AlwaysInliner::growCachedCostInfo(llvm::Function*, llvm::Function*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113AlwaysInlinerEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::AlwaysInliner>()"},
    {"_ZN4llvm18InlineCostAnalyzer19resetCachedCostInfoEPNS_8FunctionE", "llvm::InlineCostAnalyzer::resetCachedCostInfo(llvm::Function*)"},
    {"_ZNSt6vectorIN4llvm18InlineCostAnalyzer7ArgInfoESaIS2_EEaSERKS4_", "std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> >::operator=(std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> > const&)"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEjNS_12DenseMapInfoIS3_EENS4_IjEEE8CopyFromERKS7_", "llvm::DenseMap<llvm::BasicBlock const*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<unsigned int> >::CopyFrom(llvm::DenseMap<llvm::BasicBlock const*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<unsigned int> > const&)"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEED1Ev", "llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::~ValueMapCallbackVH()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEED0Ev", "llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::~ValueMapCallbackVH()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE7deletedEv", "llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::deleted()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE19allUsesReplacedWithEPNS_5ValueE", "llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4findERKSB_", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::find(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5eraseENS_16DenseMapIteratorISB_S6_SC_SA_Lb0EEE", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::erase(llvm::DenseMapIterator<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo>, false>)"},
    {"_ZN4llvm8ValueMapIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::ValueMap<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::insert(std::pair<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo> const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E6insertERKSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::insert(std::pair<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo> const&)"},
    {"_ZNSt4pairIN4llvm18ValueMapCallbackVHIPKNS0_8FunctionENS0_18InlineCostAnalyzer12FunctionInfoENS0_14ValueMapConfigIS4_EENS0_12DenseMapInfoIS6_EEEES6_EC2ERKSB_RKS6_", "std::pair<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo>::pair(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&, llvm::InlineCostAnalyzer::FunctionInfo const&)"},
    {"_ZNK4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E15LookupBucketForERKSB_RPSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::LookupBucketFor(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&, std::pair<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo>*&) const"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E16InsertIntoBucketERKSB_RKS6_PSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::InsertIntoBucket(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&, llvm::InlineCostAnalyzer::FunctionInfo const&, std::pair<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo>*)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4growEj", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm18InlineCostAnalyzer7ArgInfoESaIS2_EEC2ERKS4_", "std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> >::vector(std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5eraseERKSB_", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::erase(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E16FindAndConstructERKSB_", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::FindAndConstruct(llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > const&)"},
    {"_ZN4llvm7InlinerD1Ev", "llvm::Inliner::~Inliner()"},
    {"_ZN4llvm7InlinerD0Ev", "llvm::Inliner::~Inliner()"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_ED2Ev", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::~DenseMap()"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4initEj", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::init(unsigned int)"},
    {"_ZN4llvm27initializeSimpleInlinerPassERNS_12PassRegistryE", "llvm::initializeSimpleInlinerPass(llvm::PassRegistry&)"},
    {"_ZN4llvm26createFunctionInliningPassEv", "llvm::createFunctionInliningPass()"},
    {"_ZN4llvm26createFunctionInliningPassEi", "llvm::createFunctionInliningPass(int)"},
    {"_ZN12_GLOBAL__N_113SimpleInlinerD1Ev", "(anonymous namespace)::SimpleInliner::~SimpleInliner()"},
    {"_ZN12_GLOBAL__N_113SimpleInlinerD0Ev", "(anonymous namespace)::SimpleInliner::~SimpleInliner()"},
    {"_ZN12_GLOBAL__N_113SimpleInliner13releaseMemoryEv", "(anonymous namespace)::SimpleInliner::releaseMemory()"},
    {"_ZN12_GLOBAL__N_113SimpleInliner16doInitializationERN4llvm9CallGraphE", "(anonymous namespace)::SimpleInliner::doInitialization(llvm::CallGraph&)"},
    {"_ZN12_GLOBAL__N_113SimpleInliner13getInlineCostEN4llvm8CallSiteE", "(anonymous namespace)::SimpleInliner::getInlineCost(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_113SimpleInliner20getInlineFudgeFactorEN4llvm8CallSiteE", "(anonymous namespace)::SimpleInliner::getInlineFudgeFactor(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_113SimpleInliner19resetCachedCostInfoEPN4llvm8FunctionE", "(anonymous namespace)::SimpleInliner::resetCachedCostInfo(llvm::Function*)"},
    {"_ZN12_GLOBAL__N_113SimpleInliner18growCachedCostInfoEPN4llvm8FunctionES3_", "(anonymous namespace)::SimpleInliner::growCachedCostInfo(llvm::Function*, llvm::Function*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113SimpleInlinerEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::SimpleInliner>()"},
    {"_ZN4llvm7InlinerC1ERc", "llvm::Inliner::Inliner(char&)"},
    {"_ZN4llvm7InlinerC2ERc", "llvm::Inliner::Inliner(char&)"},
    {"_ZN4llvm7InlinerC1ERci", "llvm::Inliner::Inliner(char&, int)"},
    {"_ZN4llvm7InlinerC2ERci", "llvm::Inliner::Inliner(char&, int)"},
    {"_ZNK4llvm7Inliner16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::Inliner::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm7Inliner18getInlineThresholdENS_8CallSiteE", "llvm::Inliner::getInlineThreshold(llvm::CallSite) const"},
    {"_ZN4llvm7Inliner12shouldInlineENS_8CallSiteE", "llvm::Inliner::shouldInline(llvm::CallSite)"},
    {"_ZN4llvm7Inliner8runOnSCCERNS_12CallGraphSCCE", "llvm::Inliner::runOnSCC(llvm::CallGraphSCC&)"},
    {"_ZN4llvm7Inliner14doFinalizationERNS_9CallGraphE", "llvm::Inliner::doFinalization(llvm::CallGraph&)"},
    {"_ZN4llvm7Inliner19removeDeadFunctionsERNS_9CallGraphEPNS_11SmallPtrSetIPKNS_8FunctionELj16EEE", "llvm::Inliner::removeDeadFunctions(llvm::CallGraph&, llvm::SmallPtrSet<llvm::Function const*, 16u>*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairINS_8CallSiteEiELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::CallSite, int>, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPKNS_9ArrayTypeESt6vectorIPNS_10AllocaInstESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::ArrayType const*, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> >, llvm::DenseMapInfo<llvm::ArrayType const*>, llvm::DenseMapInfo<std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > > >::FindAndConstruct(llvm::ArrayType const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_9ArrayTypeESt6vectorIPNS_10AllocaInstESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16InsertIntoBucketERKS3_RKS8_PSt4pairIS3_S8_E", "llvm::DenseMap<llvm::ArrayType const*, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> >, llvm::DenseMapInfo<llvm::ArrayType const*>, llvm::DenseMapInfo<std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > > >::InsertIntoBucket(llvm::ArrayType const* const&, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > const&, std::pair<llvm::ArrayType const*, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > >*)"},
    {"_ZN4llvm8DenseMapIPKNS_9ArrayTypeESt6vectorIPNS_10AllocaInstESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE4growEj", "llvm::DenseMap<llvm::ArrayType const*, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> >, llvm::DenseMapInfo<llvm::ArrayType const*>, llvm::DenseMapInfo<std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > > >::grow(unsigned int)"},
    {"_ZN4llvm21initializePruneEHPassERNS_12PassRegistryE", "llvm::initializePruneEHPass(llvm::PassRegistry&)"},
    {"_ZN4llvm17createPruneEHPassEv", "llvm::createPruneEHPass()"},
    {"_ZN12_GLOBAL__N_17PruneEHD1Ev", "(anonymous namespace)::PruneEH::~PruneEH()"},
    {"_ZN12_GLOBAL__N_17PruneEHD0Ev", "(anonymous namespace)::PruneEH::~PruneEH()"},
    {"_ZN12_GLOBAL__N_17PruneEH8runOnSCCERN4llvm12CallGraphSCCE", "(anonymous namespace)::PruneEH::runOnSCC(llvm::CallGraphSCC&)"},
    {"_ZN12_GLOBAL__N_17PruneEH16SimplifyFunctionEPN4llvm8FunctionE", "(anonymous namespace)::PruneEH::SimplifyFunction(llvm::Function*)"},
    {"_ZN12_GLOBAL__N_17PruneEH16DeleteBasicBlockEPN4llvm10BasicBlockE", "(anonymous namespace)::PruneEH::DeleteBasicBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_17PruneEHEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PruneEH>()"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE22_M_initialize_dispatchINS0_12SuccIteratorIPNS0_14TerminatorInstES1_EEEEvT_SA_St12__false_type", "void std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_initialize_dispatch<llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >(llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock>, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock>, std::__false_type)"},
    {"_ZN4llvm37initializeStripDeadPrototypesPassPassERNS_12PassRegistryE", "llvm::initializeStripDeadPrototypesPassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm29createStripDeadPrototypesPassEv", "llvm::createStripDeadPrototypesPass()"},
    {"_ZN12_GLOBAL__N_123StripDeadPrototypesPassD1Ev", "(anonymous namespace)::StripDeadPrototypesPass::~StripDeadPrototypesPass()"},
    {"_ZN12_GLOBAL__N_123StripDeadPrototypesPassD0Ev", "(anonymous namespace)::StripDeadPrototypesPass::~StripDeadPrototypesPass()"},
    {"_ZN12_GLOBAL__N_123StripDeadPrototypesPass11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::StripDeadPrototypesPass::runOnModule(llvm::Module&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_123StripDeadPrototypesPassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::StripDeadPrototypesPass>()"},
    {"_ZN4llvm18WriteBitcodeToFileEPKNS_6ModuleERNS_11raw_ostreamE", "llvm::WriteBitcodeToFile(llvm::Module const*, llvm::raw_ostream&)"},
    {"_ZN4llvm20WriteBitcodeToStreamEPKNS_6ModuleERNS_15BitstreamWriterE", "llvm::WriteBitcodeToStream(llvm::Module const*, llvm::BitstreamWriter&)"},
    {"_Z11WriteModulePKN4llvm6ModuleERNS_15BitstreamWriterE", "WriteModule(llvm::Module const*, llvm::BitstreamWriter&)"},
    {"_Z13WriteFunctionRKN4llvm8FunctionERNS_15ValueEnumeratorERNS_15BitstreamWriterE", "WriteFunction(llvm::Function const&, llvm::ValueEnumerator&, llvm::BitstreamWriter&)"},
    {"_Z21WriteValueSymbolTableRKN4llvm16ValueSymbolTableERKNS_15ValueEnumeratorERNS_15BitstreamWriterE", "WriteValueSymbolTable(llvm::ValueSymbolTable const&, llvm::ValueEnumerator const&, llvm::BitstreamWriter&)"},
    {"_Z14WriteConstantsjjRKN4llvm15ValueEnumeratorERNS_15BitstreamWriterEb", "WriteConstants(unsigned int, unsigned int, llvm::ValueEnumerator const&, llvm::BitstreamWriter&, bool)"},
    {"_Z20GetOptimizationFlagsPKN4llvm5ValueE", "GetOptimizationFlags(llvm::Value const*)"},
    {"_Z11WriteMDNodePKN4llvm6MDNodeERKNS_15ValueEnumeratorERNS_15BitstreamWriterERNS_11SmallVectorIyLj64EEE", "WriteMDNode(llvm::MDNode const*, llvm::ValueEnumerator const&, llvm::BitstreamWriter&, llvm::SmallVector<unsigned long long, 64u>&)"},
    {"_Z17WriteStringRecordjRKSsjRN4llvm15BitstreamWriterE", "WriteStringRecord(unsigned int, std::string const&, unsigned int, llvm::BitstreamWriter&)"},
    {"_ZN4llvm15ValueEnumeratorD2Ev", "llvm::ValueEnumerator::~ValueEnumerator()"},
    {"_ZN4llvm15BitstreamWriter10EmitRecordIjEEvjRNS_15SmallVectorImplIT_EEj", "void llvm::BitstreamWriter::EmitRecord<unsigned int>(unsigned int, llvm::SmallVectorImpl<unsigned int>&, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIjE6insertEPjRKj", "llvm::SmallVectorImpl<unsigned int>::insert(unsigned int*, unsigned int const&)"},
    {"_ZN4llvm15BitstreamWriter24EmitRecordWithAbbrevImplIjEEvjRNS_15SmallVectorImplIT_EENS_9StringRefE", "void llvm::BitstreamWriter::EmitRecordWithAbbrevImpl<unsigned int>(unsigned int, llvm::SmallVectorImpl<unsigned int>&, llvm::StringRef)"},
    {"_ZN4llvm15BitstreamWriter20EmitAbbreviatedFieldIjEEvRKNS_15BitCodeAbbrevOpET_", "void llvm::BitstreamWriter::EmitAbbreviatedField<unsigned int>(llvm::BitCodeAbbrevOp const&, unsigned int)"},
    {"_ZN4llvm15BitstreamWriter19EmitBlockInfoAbbrevEjPNS_13BitCodeAbbrevE", "llvm::BitstreamWriter::EmitBlockInfoAbbrev(unsigned int, llvm::BitCodeAbbrev*)"},
    {"_ZN4llvm15BitstreamWriter15SwitchToBlockIDEj", "llvm::BitstreamWriter::SwitchToBlockID(unsigned int)"},
    {"_ZN4llvm15BitstreamWriter20getOrCreateBlockInfoEj", "llvm::BitstreamWriter::getOrCreateBlockInfo(unsigned int)"},
    {"_ZNSt6vectorIN4llvm15BitstreamWriter9BlockInfoESaIS2_EE9push_backERKS2_", "std::vector<llvm::BitstreamWriter::BlockInfo, std::allocator<llvm::BitstreamWriter::BlockInfo> >::push_back(llvm::BitstreamWriter::BlockInfo const&)"},
    {"_ZNSt6vectorIN4llvm15BitstreamWriter9BlockInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BitstreamWriter::BlockInfo, std::allocator<llvm::BitstreamWriter::BlockInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BitstreamWriter::BlockInfo*, std::vector<llvm::BitstreamWriter::BlockInfo, std::allocator<llvm::BitstreamWriter::BlockInfo> > >, llvm::BitstreamWriter::BlockInfo const&)"},
    {"_ZN4llvm23createBitcodeWriterPassERNS_11raw_ostreamE", "llvm::createBitcodeWriterPass(llvm::raw_ostream&)"},
    {"_ZN12_GLOBAL__N_116WriteBitcodePassD1Ev", "(anonymous namespace)::WriteBitcodePass::~WriteBitcodePass()"},
    {"_ZN12_GLOBAL__N_116WriteBitcodePassD0Ev", "(anonymous namespace)::WriteBitcodePass::~WriteBitcodePass()"},
    {"_ZNK12_GLOBAL__N_116WriteBitcodePass11getPassNameEv", "(anonymous namespace)::WriteBitcodePass::getPassName() const"},
    {"_ZN12_GLOBAL__N_116WriteBitcodePass11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::WriteBitcodePass::runOnModule(llvm::Module&)"},
    {"_ZN4llvm15ValueEnumeratorC1EPKNS_6ModuleE", "llvm::ValueEnumerator::ValueEnumerator(llvm::Module const*)"},
    {"_ZN4llvm15ValueEnumeratorC2EPKNS_6ModuleE", "llvm::ValueEnumerator::ValueEnumerator(llvm::Module const*)"},
    {"_ZN4llvm15ValueEnumerator14EnumerateValueEPKNS_5ValueE", "llvm::ValueEnumerator::EnumerateValue(llvm::Value const*)"},
    {"_ZN4llvm15ValueEnumerator19EnumerateAttributesERKNS_11AttrListPtrE", "llvm::ValueEnumerator::EnumerateAttributes(llvm::AttrListPtr const&)"},
    {"_ZN4llvm15ValueEnumerator24EnumerateTypeSymbolTableERKNS_15TypeSymbolTableE", "llvm::ValueEnumerator::EnumerateTypeSymbolTable(llvm::TypeSymbolTable const&)"},
    {"_ZN4llvm15ValueEnumerator25EnumerateValueSymbolTableERKNS_16ValueSymbolTableE", "llvm::ValueEnumerator::EnumerateValueSymbolTable(llvm::ValueSymbolTable const&)"},
    {"_ZN4llvm15ValueEnumerator22EnumerateNamedMetadataEPKNS_6ModuleE", "llvm::ValueEnumerator::EnumerateNamedMetadata(llvm::Module const*)"},
    {"_ZN4llvm15ValueEnumerator13EnumerateTypeEPKNS_4TypeE", "llvm::ValueEnumerator::EnumerateType(llvm::Type const*)"},
    {"_ZN4llvm15ValueEnumerator20EnumerateOperandTypeEPKNS_5ValueE", "llvm::ValueEnumerator::EnumerateOperandType(llvm::Value const*)"},
    {"_ZN4llvm15ValueEnumerator17EnumerateMetadataEPKNS_5ValueE", "llvm::ValueEnumerator::EnumerateMetadata(llvm::Value const*)"},
    {"_ZN4llvm15ValueEnumerator17OptimizeConstantsEjj", "llvm::ValueEnumerator::OptimizeConstants(unsigned int, unsigned int)"},
    {"_Z18CompareByFrequencyRKSt4pairIPKN4llvm4TypeEjES6_", "CompareByFrequency(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)"},
    {"_ZNK4llvm15ValueEnumerator16getInstructionIDEPKNS_11InstructionE", "llvm::ValueEnumerator::getInstructionID(llvm::Instruction const*) const"},
    {"_ZN4llvm15ValueEnumerator16setInstructionIDEPKNS_11InstructionE", "llvm::ValueEnumerator::setInstructionID(llvm::Instruction const*)"},
    {"_ZNK4llvm15ValueEnumerator10getValueIDEPKNS_5ValueE", "llvm::ValueEnumerator::getValueID(llvm::Value const*) const"},
    {"_ZN4llvm15ValueEnumerator20EnumerateNamedMDNodeEPKNS_11NamedMDNodeE", "llvm::ValueEnumerator::EnumerateNamedMDNode(llvm::NamedMDNode const*)"},
    {"_ZN4llvm15ValueEnumerator23EnumerateMDNodeOperandsEPKNS_6MDNodeE", "llvm::ValueEnumerator::EnumerateMDNodeOperands(llvm::MDNode const*)"},
    {"_ZN4llvm15ValueEnumerator30EnumerateFunctionLocalMetadataEPKNS_6MDNodeE", "llvm::ValueEnumerator::EnumerateFunctionLocalMetadata(llvm::MDNode const*)"},
    {"_ZN4llvm15ValueEnumerator19incorporateFunctionERKNS_8FunctionE", "llvm::ValueEnumerator::incorporateFunction(llvm::Function const&)"},
    {"_ZN4llvm15ValueEnumerator13purgeFunctionEv", "llvm::ValueEnumerator::purgeFunction()"},
    {"_ZNK4llvm15ValueEnumerator21getGlobalBasicBlockIDEPKNS_10BasicBlockE", "llvm::ValueEnumerator::getGlobalBasicBlockID(llvm::BasicBlock const*) const"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEEN12_GLOBAL__N_116CstSortPredicateEEvT_SF_T0_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, (anonymous namespace)::CstSortPredicate)"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEES8_lN12_GLOBAL__N_116CstSortPredicateEEvT_SF_T0_T1_T2_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, long, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, long, (anonymous namespace)::CstSortPredicate)"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEES8_N12_GLOBAL__N_116CstSortPredicateEEvT_SF_T0_T1_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, (anonymous namespace)::CstSortPredicate)"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEElS8_N12_GLOBAL__N_116CstSortPredicateEEvT_SF_SF_T0_SG_T1_SG_T2_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, long, std::pair<llvm::Value const*, unsigned int>*, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, long, long, std::pair<llvm::Value const*, unsigned int>*, long, (anonymous namespace)::CstSortPredicate)"},
    {"_ZN12_GLOBAL__N_116CstSortPredicateclERKSt4pairIPKN4llvm5ValueEjES8_", "(anonymous namespace)::CstSortPredicate::operator()(std::pair<llvm::Value const*, unsigned int> const&, std::pair<llvm::Value const*, unsigned int> const&)"},
    {"_ZSt5mergeIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEESC_S8_N12_GLOBAL__N_116CstSortPredicateEET1_T_SG_T0_SH_SF_T2_", "std::pair<llvm::Value const*, unsigned int>* std::merge<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, (anonymous namespace)::CstSortPredicate)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEEN12_GLOBAL__N_116CstSortPredicateEEvT_SF_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, (anonymous namespace)::CstSortPredicate)"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEElN12_GLOBAL__N_116CstSortPredicateEEvT_SF_SF_T0_SG_T1_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, long, (anonymous namespace)::CstSortPredicate>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, long, long, (anonymous namespace)::CstSortPredicate)"},
    {"_ZNSt6vectorISt4pairIPKN4llvm5ValueEjESaIS5_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS5_S7_EEmRKS5_", "std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, unsigned long, std::pair<llvm::Value const*, unsigned int> const&)"},
    {"_ZNSt6vectorISt4pairIPKN4llvm4TypeEjESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int> const&)"},
    {"_ZNSt6vectorISt4pairIPKN4llvm5ValueEjESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int> const&)"},
    {"_ZSt17__rotate_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEES8_lET_SD_SD_SD_T1_SE_T0_SE_", "__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > > std::__rotate_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::pair<llvm::Value const*, unsigned int>*, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, long, long, std::pair<llvm::Value const*, unsigned int>*, long)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm5ValueEjESt6vectorIS7_SaIS7_EEEEEvT_SD_SD_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Value const*, unsigned int>*, std::vector<std::pair<llvm::Value const*, unsigned int>, std::allocator<std::pair<llvm::Value const*, unsigned int> > > >, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::Type const*, unsigned int, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Type const* const&, unsigned int const&, std::pair<llvm::Type const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::Type const*, unsigned int, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEEPFbRKS7_SE_EEvT_SH_T0_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEES8_lPFbRKS7_SE_EEvT_SH_T0_T1_T2_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEES8_PFbRKS7_SE_EEvT_SH_T0_T1_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEElS8_PFbRKS7_SE_EEvT_SH_SH_T0_SI_T1_SI_T2_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, std::pair<llvm::Type const*, unsigned int>*, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, long, std::pair<llvm::Type const*, unsigned int>*, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt16__merge_backwardIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEES8_SC_PFbRKS7_SE_EET1_T_SI_T0_SJ_SH_T2_", "__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > > std::__merge_backward<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, std::pair<llvm::Type const*, unsigned int>*, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt17__rotate_adaptiveIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEES8_lET_SD_SD_SD_T1_SE_T0_SE_", "__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > > std::__rotate_adaptive<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, long, std::pair<llvm::Type const*, unsigned int>*, long)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEEEvT_SD_SD_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::random_access_iterator_tag)"},
    {"_ZSt17__merge_sort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEES8_lPFbRKS7_SE_EEvT_SH_T0_T1_T2_", "void std::__merge_sort_loop<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, std::pair<llvm::Type const*, unsigned int>*, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt17__merge_sort_loopIPSt4pairIPKN4llvm4TypeEjEN9__gnu_cxx17__normal_iteratorIS6_St6vectorIS5_SaIS5_EEEElPFbRKS5_SE_EEvT_SH_T0_T1_T2_", "void std::__merge_sort_loop<std::pair<llvm::Type const*, unsigned int>*, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(std::pair<llvm::Type const*, unsigned int>*, std::pair<llvm::Type const*, unsigned int>*, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEEPFbRKS7_SE_EEvT_SH_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPSt4pairIPKN4llvm4TypeEjESt6vectorIS7_SaIS7_EEEElPFbRKS7_SE_EEvT_SH_SH_T0_SI_T1_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&)>(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, unsigned int>*, std::vector<std::pair<llvm::Type const*, unsigned int>, std::allocator<std::pair<llvm::Type const*, unsigned int> > > >, long, long, bool (*)(std::pair<llvm::Type const*, unsigned int> const&, std::pair<llvm::Type const*, unsigned int> const&))"},
    {"_ZN4llvm13BitcodeReader9FreeStateEv", "llvm::BitcodeReader::FreeState()"},
    {"_ZN4llvm22BitcodeReaderValueList11AssignValueEPNS_5ValueEj", "llvm::BitcodeReaderValueList::AssignValue(llvm::Value*, unsigned int)"},
    {"_ZN4llvm22BitcodeReaderValueList17getConstantFwdRefEjPKNS_4TypeE", "llvm::BitcodeReaderValueList::getConstantFwdRef(unsigned int, llvm::Type const*)"},
    {"_ZN4llvm22BitcodeReaderValueList14getValueFwdRefEjPKNS_4TypeE", "llvm::BitcodeReaderValueList::getValueFwdRef(unsigned int, llvm::Type const*)"},
    {"_ZN4llvm22BitcodeReaderValueList26ResolveConstantForwardRefsEv", "llvm::BitcodeReaderValueList::ResolveConstantForwardRefs()"},
    {"_ZN4llvm24BitcodeReaderMDValueList11AssignValueEPNS_5ValueEj", "llvm::BitcodeReaderMDValueList::AssignValue(llvm::Value*, unsigned int)"},
    {"_ZN4llvm24BitcodeReaderMDValueList14getValueFwdRefEj", "llvm::BitcodeReaderMDValueList::getValueFwdRef(unsigned int)"},
    {"_ZN4llvm13BitcodeReader11getTypeByIDEjb", "llvm::BitcodeReader::getTypeByID(unsigned int, bool)"},
    {"_ZN4llvm13BitcodeReader19ParseAttributeBlockEv", "llvm::BitcodeReader::ParseAttributeBlock()"},
    {"_ZN4llvm13BitcodeReader14ParseTypeTableEv", "llvm::BitcodeReader::ParseTypeTable()"},
    {"_ZN4llvm13BitcodeReader20ParseTypeSymbolTableEv", "llvm::BitcodeReader::ParseTypeSymbolTable()"},
    {"_ZN4llvm13BitcodeReader21ParseValueSymbolTableEv", "llvm::BitcodeReader::ParseValueSymbolTable()"},
    {"_ZN4llvm13BitcodeReader13ParseMetadataEv", "llvm::BitcodeReader::ParseMetadata()"},
    {"_ZN4llvm13BitcodeReader26ResolveGlobalAndAliasInitsEv", "llvm::BitcodeReader::ResolveGlobalAndAliasInits()"},
    {"_ZN4llvm13BitcodeReader14ParseConstantsEv", "llvm::BitcodeReader::ParseConstants()"},
    {"_Z22GetDecodedBinaryOpcodejPKN4llvm4TypeE", "GetDecodedBinaryOpcode(unsigned int, llvm::Type const*)"},
    {"_ZN4llvm13BitcodeReader27RememberAndSkipFunctionBodyEv", "llvm::BitcodeReader::RememberAndSkipFunctionBody()"},
    {"_ZN4llvm13BitcodeReader11ParseModuleEv", "llvm::BitcodeReader::ParseModule()"},
    {"_ZN4llvm13BitcodeReader16ParseBitcodeIntoEPNS_6ModuleE", "llvm::BitcodeReader::ParseBitcodeInto(llvm::Module*)"},
    {"_ZN4llvm13BitcodeReader17ParseModuleTripleERSs", "llvm::BitcodeReader::ParseModuleTriple(std::string&)"},
    {"_ZN4llvm13BitcodeReader11ParseTripleERSs", "llvm::BitcodeReader::ParseTriple(std::string&)"},
    {"_ZN4llvm13BitcodeReader23ParseMetadataAttachmentEv", "llvm::BitcodeReader::ParseMetadataAttachment()"},
    {"_ZN4llvm13BitcodeReader17ParseFunctionBodyEPNS_8FunctionE", "llvm::BitcodeReader::ParseFunctionBody(llvm::Function*)"},
    {"_ZNK4llvm13BitcodeReader16isMaterializableEPKNS_11GlobalValueE", "llvm::BitcodeReader::isMaterializable(llvm::GlobalValue const*) const"},
    {"_ZN4llvm13BitcodeReader11MaterializeEPNS_11GlobalValueEPSs", "llvm::BitcodeReader::Materialize(llvm::GlobalValue*, std::string*)"},
    {"_ZNK4llvm13BitcodeReader18isDematerializableEPKNS_11GlobalValueE", "llvm::BitcodeReader::isDematerializable(llvm::GlobalValue const*) const"},
    {"_ZN4llvm13BitcodeReader13DematerializeEPNS_11GlobalValueE", "llvm::BitcodeReader::Dematerialize(llvm::GlobalValue*)"},
    {"_ZN4llvm13BitcodeReader17MaterializeModuleEPNS_6ModuleEPSs", "llvm::BitcodeReader::MaterializeModule(llvm::Module*, std::string*)"},
    {"_ZN4llvm20getLazyBitcodeModuleEPNS_12MemoryBufferERNS_11LLVMContextEPSs", "llvm::getLazyBitcodeModule(llvm::MemoryBuffer*, llvm::LLVMContext&, std::string*)"},
    {"_ZN4llvm16ParseBitcodeFileEPNS_12MemoryBufferERNS_11LLVMContextEPSs", "llvm::ParseBitcodeFile(llvm::MemoryBuffer*, llvm::LLVMContext&, std::string*)"},
    {"_ZN4llvm22getBitcodeTargetTripleEPNS_12MemoryBufferERNS_11LLVMContextEPSs", "llvm::getBitcodeTargetTriple(llvm::MemoryBuffer*, llvm::LLVMContext&, std::string*)"},
    {"_ZN4llvm12_GLOBAL__N_119ConstantPlaceHolderD1Ev", "llvm::(anonymous namespace)::ConstantPlaceHolder::~ConstantPlaceHolder()"},
    {"_ZN4llvm12_GLOBAL__N_119ConstantPlaceHolderD0Ev", "llvm::(anonymous namespace)::ConstantPlaceHolder::~ConstantPlaceHolder()"},
    {"_ZN4llvm22BitcodeReaderValueList9push_backEPNS_5ValueE", "llvm::BitcodeReaderValueList::push_back(llvm::Value*)"},
    {"_ZN4llvm22BitcodeReaderValueList6resizeEj", "llvm::BitcodeReaderValueList::resize(unsigned int)"},
    {"_ZN4llvm24BitcodeReaderMDValueList9push_backEPNS_5ValueE", "llvm::BitcodeReaderMDValueList::push_back(llvm::Value*)"},
    {"_ZN4llvm24BitcodeReaderMDValueList6resizeEj", "llvm::BitcodeReaderMDValueList::resize(unsigned int)"},
    {"_ZNSt6vectorIN4llvm12PATypeHolderESaIS1_EE7reserveEm", "std::vector<llvm::PATypeHolder, std::allocator<llvm::PATypeHolder> >::reserve(unsigned long)"},
    {"_ZN4llvm15SmallVectorImplIyE6resizeEj", "llvm::SmallVectorImpl<unsigned long long>::resize(unsigned int)"},
    {"_ZN4llvm22BitcodeReaderValueList8shrinkToEj", "llvm::BitcodeReaderValueList::shrinkTo(unsigned int)"},
    {"_ZN4llvm13BitcodeReader16getValueTypePairERNS_11SmallVectorIyLj64EEERjjRPNS_5ValueE", "llvm::BitcodeReader::getValueTypePair(llvm::SmallVector<unsigned long long, 64u>&, unsigned int&, unsigned int, llvm::Value*&)"},
    {"_ZN4llvm24BitcodeReaderMDValueList8shrinkToEj", "llvm::BitcodeReaderMDValueList::shrinkTo(unsigned int)"},
    {"_ZN4llvm13BitcodeReaderD1Ev", "llvm::BitcodeReader::~BitcodeReader()"},
    {"_ZN4llvm13BitcodeReaderD0Ev", "llvm::BitcodeReader::~BitcodeReader()"},
    {"_ZN4llvm10InvokeInstC2IPPNS_5ValueEEES3_PNS_10BasicBlockES6_T_S7_jRKNS_5TwineEPNS_11InstructionE", "llvm::InvokeInst::InvokeInst<llvm::Value**>(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value**, llvm::Value**, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm15InsertValueInstC2IPjEEPNS_5ValueES4_T_S5_RKNS_5TwineEPNS_11InstructionE", "llvm::InsertValueInst::InsertValueInst<unsigned int*>(llvm::Value*, llvm::Value*, unsigned int*, unsigned int*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, unsigned long, llvm::BasicBlock* const&)"},
    {"_ZNSt6vectorIPN4llvm8FunctionESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Function*, std::allocator<llvm::Function*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Function**, std::vector<llvm::Function*, std::allocator<llvm::Function*> > >, llvm::Function* const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm8FunctionES3_ESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::Function*, llvm::Function*>, std::allocator<std::pair<llvm::Function*, llvm::Function*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Function*, llvm::Function*>*, std::vector<std::pair<llvm::Function*, llvm::Function*>, std::allocator<std::pair<llvm::Function*, llvm::Function*> > > >, std::pair<llvm::Function*, llvm::Function*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionEyNS_12DenseMapInfoIS2_EENS3_IyEEE16InsertIntoBucketERKS2_RKyPSt4pairIS2_yE", "llvm::DenseMap<llvm::Function*, unsigned long long, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(llvm::Function* const&, unsigned long long const&, std::pair<llvm::Function*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionEyNS_12DenseMapInfoIS2_EENS3_IyEEE4growEj", "llvm::DenseMap<llvm::Function*, unsigned long long, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIjPN4llvm14GlobalVariableEESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::GlobalVariable*>*, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > >, std::pair<unsigned int, llvm::GlobalVariable*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionESt6vectorISt4pairIjPNS_14GlobalVariableEESaIS7_EENS_12DenseMapInfoIS2_EENSA_IS9_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Function*, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > >, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > > >::FindAndConstruct(llvm::Function* const&)"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionESt6vectorISt4pairIjPNS_14GlobalVariableEESaIS7_EENS_12DenseMapInfoIS2_EENSA_IS9_EEE16InsertIntoBucketERKS2_RKS9_PS4_IS2_S9_E", "llvm::DenseMap<llvm::Function*, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > >, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > > >::InsertIntoBucket(llvm::Function* const&, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > const&, std::pair<llvm::Function*, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > >*)"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionESt6vectorISt4pairIjPNS_14GlobalVariableEESaIS7_EENS_12DenseMapInfoIS2_EENSA_IS9_EEE4growEj", "llvm::DenseMap<llvm::Function*, std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > >, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > > >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIjPN4llvm14GlobalVariableEESaIS4_EEC2ERKS6_", "std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > >::vector(std::vector<std::pair<unsigned int, llvm::GlobalVariable*>, std::allocator<std::pair<unsigned int, llvm::GlobalVariable*> > > const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm11GlobalAliasEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::GlobalAlias*, unsigned int>, std::allocator<std::pair<llvm::GlobalAlias*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::GlobalAlias*, unsigned int>*, std::vector<std::pair<llvm::GlobalAlias*, unsigned int>, std::allocator<std::pair<llvm::GlobalAlias*, unsigned int> > > >, std::pair<llvm::GlobalAlias*, unsigned int> const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm14GlobalVariableEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::GlobalVariable*, unsigned int>, std::allocator<std::pair<llvm::GlobalVariable*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::GlobalVariable*, unsigned int>*, std::vector<std::pair<llvm::GlobalVariable*, unsigned int>, std::allocator<std::pair<llvm::GlobalVariable*, unsigned int> > > >, std::pair<llvm::GlobalVariable*, unsigned int> const&)"},
    {"_ZNSt6vectorIN4llvm11AttrListPtrESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::AttrListPtr, std::allocator<llvm::AttrListPtr> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AttrListPtr*, std::vector<llvm::AttrListPtr, std::allocator<llvm::AttrListPtr> > >, llvm::AttrListPtr const&)"},
    {"_ZNSt6vectorIN4llvm12PATypeHolderESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::PATypeHolder, std::allocator<llvm::PATypeHolder> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::PATypeHolder*, std::vector<llvm::PATypeHolder, std::allocator<llvm::PATypeHolder> > >, llvm::PATypeHolder const&)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIPN4llvm8ConstantEjESt6vectorIS6_SaIS6_EEEElEvT_SC_T0_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, long)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPN4llvm8ConstantEjESt6vectorIS6_SaIS6_EEEEEvT_SC_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIPN4llvm8ConstantEjESt6vectorIS6_SaIS6_EEEEEvT_SC_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPSt4pairIPN4llvm8ConstantEjESt6vectorIS6_SaIS6_EEEEEvT_SC_SC_", "void std::__heap_select<__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIPN4llvm8ConstantEjESt6vectorIS6_SaIS6_EEEElS6_EvT_T0_SD_T1_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, long, std::pair<llvm::Constant*, unsigned int> >(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, long, long, std::pair<llvm::Constant*, unsigned int>)"},
    {"_ZNSt6vectorISt4pairIPN4llvm8ConstantEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Constant*, unsigned int>*, std::vector<std::pair<llvm::Constant*, unsigned int>, std::allocator<std::pair<llvm::Constant*, unsigned int> > > >, std::pair<llvm::Constant*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIjjNS_12DenseMapInfoIjEES2_E16shrink_and_clearEv", "llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_8FunctionEyNS_12DenseMapInfoIS2_EENS3_IyEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Function*, unsigned long long, llvm::DenseMapInfo<llvm::Function*>, llvm::DenseMapInfo<unsigned long long> >::shrink_and_clear()"},
    {"_ZN4llvm13BitcodeReaderD2Ev", "llvm::BitcodeReader::~BitcodeReader()"},
    {"_ZN4llvm13BitcodeReaderC2EPNS_12MemoryBufferERNS_11LLVMContextE", "llvm::BitcodeReader::BitcodeReader(llvm::MemoryBuffer*, llvm::LLVMContext&)"},
    {"_ZN4llvm14GVMaterializer13DematerializeEPNS_11GlobalValueE", "llvm::GVMaterializer::Dematerialize(llvm::GlobalValue*)"},
    {"_ZNSt6vectorIN4llvm6WeakVHESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<llvm::WeakVH, std::allocator<llvm::WeakVH> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::WeakVH*, std::vector<llvm::WeakVH, std::allocator<llvm::WeakVH> > >, unsigned long, llvm::WeakVH const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm6WeakVHES5_EET0_T_S7_S6_", "llvm::WeakVH* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::WeakVH*, llvm::WeakVH*>(llvm::WeakVH*, llvm::WeakVH*, llvm::WeakVH*)"},
    {"_ZNSt6vectorIN4llvm6WeakVHESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::WeakVH, std::allocator<llvm::WeakVH> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::WeakVH*, std::vector<llvm::WeakVH, std::allocator<llvm::WeakVH> > >, llvm::WeakVH const&)"},
    {"_ZNK4llvm7LLLexer5ErrorENS_5SMLocERKNS_5TwineE", "llvm::LLLexer::Error(llvm::SMLoc, llvm::Twine const&) const"},
    {"_ZN4llvm7LLLexer6atoullEPKcS2_", "llvm::LLLexer::atoull(char const*, char const*)"},
    {"_ZN4llvm7LLLexer11HexIntToValEPKcS2_", "llvm::LLLexer::HexIntToVal(char const*, char const*)"},
    {"_ZN4llvm7LLLexer12HexToIntPairEPKcS2_Py", "llvm::LLLexer::HexToIntPair(char const*, char const*, unsigned long long*)"},
    {"_ZN4llvm7LLLexer16FP80HexToIntPairEPKcS2_Py", "llvm::LLLexer::FP80HexToIntPair(char const*, char const*, unsigned long long*)"},
    {"_ZN4llvm7LLLexerC1EPNS_12MemoryBufferERNS_9SourceMgrERNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::LLLexer::LLLexer(llvm::MemoryBuffer*, llvm::SourceMgr&, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZN4llvm7LLLexerC2EPNS_12MemoryBufferERNS_9SourceMgrERNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::LLLexer::LLLexer(llvm::MemoryBuffer*, llvm::SourceMgr&, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZNK4llvm7LLLexer11getFilenameEv", "llvm::LLLexer::getFilename() const"},
    {"_ZN4llvm7LLLexer11getNextCharEv", "llvm::LLLexer::getNextChar()"},
    {"_ZN4llvm7LLLexer8LexTokenEv", "llvm::LLLexer::LexToken()"},
    {"_ZN4llvm7LLLexer13LexIdentifierEv", "llvm::LLLexer::LexIdentifier()"},
    {"_ZN4llvm7LLLexer11LexPositiveEv", "llvm::LLLexer::LexPositive()"},
    {"_ZN4llvm7LLLexer5LexAtEv", "llvm::LLLexer::LexAt()"},
    {"_ZN4llvm7LLLexer10LexPercentEv", "llvm::LLLexer::LexPercent()"},
    {"_ZN4llvm7LLLexer8LexQuoteEv", "llvm::LLLexer::LexQuote()"},
    {"_ZN4llvm7LLLexer15SkipLineCommentEv", "llvm::LLLexer::SkipLineComment()"},
    {"_ZN4llvm7LLLexer10LexExclaimEv", "llvm::LLLexer::LexExclaim()"},
    {"_ZN4llvm7LLLexer18LexDigitOrNegativeEv", "llvm::LLLexer::LexDigitOrNegative()"},
    {"_Z13UnEscapeLexedRSs", "UnEscapeLexed(std::string&)"},
    {"_ZN4llvm7LLLexer5Lex0xEv", "llvm::LLLexer::Lex0x()"},
    {"_ZN4llvm8LLParser3RunEv", "llvm::LLParser::Run()"},
    {"_ZN4llvm8LLParser21ParseTopLevelEntitiesEv", "llvm::LLParser::ParseTopLevelEntities()"},
    {"_ZN4llvm8LLParser19ValidateEndOfModuleEv", "llvm::LLParser::ValidateEndOfModule()"},
    {"_ZN4llvm8LLParser31ResolveForwardRefBlockAddressesEPNS_8FunctionERSt6vectorISt4pairINS_5ValIDEPNS_11GlobalValueEESaIS8_EEPNS0_16PerFunctionStateE", "llvm::LLParser::ResolveForwardRefBlockAddresses(llvm::Function*, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > >&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser16PerFunctionState5GetBBERKSsNS_5SMLocE", "llvm::LLParser::PerFunctionState::GetBB(std::string const&, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser16PerFunctionState5GetBBEjNS_5SMLocE", "llvm::LLParser::PerFunctionState::GetBB(unsigned int, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser12ParseDeclareEv", "llvm::LLParser::ParseDeclare()"},
    {"_ZN4llvm8LLParser11ParseDefineEv", "llvm::LLParser::ParseDefine()"},
    {"_ZN4llvm8LLParser14ParseModuleAsmEv", "llvm::LLParser::ParseModuleAsm()"},
    {"_ZN4llvm8LLParser21ParseTargetDefinitionEv", "llvm::LLParser::ParseTargetDefinition()"},
    {"_ZN4llvm8LLParser12ParseDepLibsEv", "llvm::LLParser::ParseDepLibs()"},
    {"_ZN4llvm8LLParser16ParseUnnamedTypeEv", "llvm::LLParser::ParseUnnamedType()"},
    {"_ZN4llvm8LLParser14ParseNamedTypeEv", "llvm::LLParser::ParseNamedType()"},
    {"_ZN4llvm8LLParser18ParseUnnamedGlobalEv", "llvm::LLParser::ParseUnnamedGlobal()"},
    {"_ZN4llvm8LLParser16ParseNamedGlobalEv", "llvm::LLParser::ParseNamedGlobal()"},
    {"_ZN4llvm8LLParser23ParseStandaloneMetadataEv", "llvm::LLParser::ParseStandaloneMetadata()"},
    {"_ZN4llvm8LLParser18ParseNamedMetadataEv", "llvm::LLParser::ParseNamedMetadata()"},
    {"_ZN4llvm8LLParser23ParseOptionalVisibilityERj", "llvm::LLParser::ParseOptionalVisibility(unsigned int&)"},
    {"_ZN4llvm8LLParser11ParseGlobalERKSsNS_5SMLocEjbj", "llvm::LLParser::ParseGlobal(std::string const&, llvm::SMLoc, unsigned int, bool, unsigned int)"},
    {"_ZN4llvm8LLParser10ParseTokenENS_5lltok4KindEPKc", "llvm::LLParser::ParseToken(llvm::lltok::Kind, char const*)"},
    {"_ZN4llvm8LLParser19ParseStringConstantERSs", "llvm::LLParser::ParseStringConstant(std::string&)"},
    {"_ZN4llvm8LLParser9ParseTypeERNS_12PATypeHolderEb", "llvm::LLParser::ParseType(llvm::PATypeHolder&, bool)"},
    {"_ZN4llvm8LLParser19ParseFunctionHeaderERPNS_8FunctionEb", "llvm::LLParser::ParseFunctionHeader(llvm::Function*&, bool)"},
    {"_ZN4llvm8LLParser17ParseFunctionBodyERNS_8FunctionE", "llvm::LLParser::ParseFunctionBody(llvm::Function&)"},
    {"_ZN4llvm8LLParser15ParseGlobalTypeERb", "llvm::LLParser::ParseGlobalType(bool&)"},
    {"_ZN4llvm8LLParser20ParseOptionalLinkageERjRb", "llvm::LLParser::ParseOptionalLinkage(unsigned int&, bool&)"},
    {"_ZN4llvm8LLParser10ParseAliasERKSsNS_5SMLocEj", "llvm::LLParser::ParseAlias(std::string const&, llvm::SMLoc, unsigned int)"},
    {"_ZN4llvm8LLParser13ParseMDStringERPNS_8MDStringE", "llvm::LLParser::ParseMDString(llvm::MDString*&)"},
    {"_ZN4llvm8LLParser13ParseMDNodeIDERPNS_6MDNodeERj", "llvm::LLParser::ParseMDNodeID(llvm::MDNode*&, unsigned int&)"},
    {"_ZN4llvm8LLParser11ParseUInt32ERj", "llvm::LLParser::ParseUInt32(unsigned int&)"},
    {"_ZN4llvm8LLParser13ParseMDNodeIDERPNS_6MDNodeE", "llvm::LLParser::ParseMDNodeID(llvm::MDNode*&)"},
    {"_ZN4llvm8LLParser17ParseMDNodeVectorERNS_15SmallVectorImplIPNS_5ValueEEEPNS0_16PerFunctionStateE", "llvm::LLParser::ParseMDNodeVector(llvm::SmallVectorImpl<llvm::Value*>&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser23ParseGlobalTypeAndValueERPNS_8ConstantE", "llvm::LLParser::ParseGlobalTypeAndValue(llvm::Constant*&)"},
    {"_ZN4llvm8LLParser10ParseValIDERNS_5ValIDEPNS0_16PerFunctionStateE", "llvm::LLParser::ParseValID(llvm::ValID&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser22ParseOptionalAddrSpaceERj", "llvm::LLParser::ParseOptionalAddrSpace(unsigned int&)"},
    {"_ZN4llvm8LLParser16ParseGlobalValueEPKNS_4TypeERPNS_8ConstantE", "llvm::LLParser::ParseGlobalValue(llvm::Type const*, llvm::Constant*&)"},
    {"_ZN4llvm8LLParser22ParseOptionalAlignmentERj", "llvm::LLParser::ParseOptionalAlignment(unsigned int&)"},
    {"_ZN4llvm8LLParser12GetGlobalValERKSsPKNS_4TypeENS_5SMLocE", "llvm::LLParser::GetGlobalVal(std::string const&, llvm::Type const*, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser12GetGlobalValEjPKNS_4TypeENS_5SMLocE", "llvm::LLParser::GetGlobalVal(unsigned int, llvm::Type const*, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser18ParseOptionalAttrsERjj", "llvm::LLParser::ParseOptionalAttrs(unsigned int&, unsigned int)"},
    {"_ZN4llvm8LLParser27ParseOptionalStackAlignmentERj", "llvm::LLParser::ParseOptionalStackAlignment(unsigned int&)"},
    {"_ZN4llvm8LLParser24ParseOptionalCallingConvERNS_11CallingConv2IDE", "llvm::LLParser::ParseOptionalCallingConv(llvm::CallingConv::ID&)"},
    {"_ZN4llvm8LLParser24ParseInstructionMetadataEPNS_11InstructionEPNS0_16PerFunctionStateE", "llvm::LLParser::ParseInstructionMetadata(llvm::Instruction*, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser22ParseMetadataListValueERNS_5ValIDEPNS0_16PerFunctionStateE", "llvm::LLParser::ParseMetadataListValue(llvm::ValID&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser23ParseOptionalCommaAlignERjRb", "llvm::LLParser::ParseOptionalCommaAlign(unsigned int&, bool&)"},
    {"_ZN4llvm8LLParser14ParseIndexListERNS_15SmallVectorImplIjEERb", "llvm::LLParser::ParseIndexList(llvm::SmallVectorImpl<unsigned int>&, bool&)"},
    {"_ZN4llvm8LLParser12ParseTypeRecERNS_12PATypeHolderE", "llvm::LLParser::ParseTypeRec(llvm::PATypeHolder&)"},
    {"_ZN4llvm8LLParser12HandleUpRefsEPKNS_4TypeE", "llvm::LLParser::HandleUpRefs(llvm::Type const*)"},
    {"_ZN4llvm8LLParser15ParseStructTypeERNS_12PATypeHolderEb", "llvm::LLParser::ParseStructType(llvm::PATypeHolder&, bool)"},
    {"_ZN4llvm8LLParser20ParseArrayVectorTypeERNS_12PATypeHolderEb", "llvm::LLParser::ParseArrayVectorType(llvm::PATypeHolder&, bool)"},
    {"_ZN4llvm8LLParser17ParseFunctionTypeERNS_12PATypeHolderE", "llvm::LLParser::ParseFunctionType(llvm::PATypeHolder&)"},
    {"_ZN4llvm8LLParser18ParseParameterListERNS_15SmallVectorImplINS0_9ParamInfoEEERNS0_16PerFunctionStateE", "llvm::LLParser::ParseParameterList(llvm::SmallVectorImpl<llvm::LLParser::ParamInfo>&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser10ParseValueEPKNS_4TypeERPNS_5ValueERNS0_16PerFunctionStateE", "llvm::LLParser::ParseValue(llvm::Type const*, llvm::Value*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser17ParseArgumentListERSt6vectorINS0_7ArgInfoESaIS2_EERbb", "llvm::LLParser::ParseArgumentList(std::vector<llvm::LLParser::ArgInfo, std::allocator<llvm::LLParser::ArgInfo> >&, bool&, bool)"},
    {"_ZN4llvm8LLParser16PerFunctionStateC1ERS0_RNS_8FunctionEi", "llvm::LLParser::PerFunctionState::PerFunctionState(llvm::LLParser&, llvm::Function&, int)"},
    {"_ZN4llvm8LLParser16PerFunctionStateC2ERS0_RNS_8FunctionEi", "llvm::LLParser::PerFunctionState::PerFunctionState(llvm::LLParser&, llvm::Function&, int)"},
    {"_ZN4llvm8LLParser16PerFunctionStateD1Ev", "llvm::LLParser::PerFunctionState::~PerFunctionState()"},
    {"_ZN4llvm8LLParser16PerFunctionStateD2Ev", "llvm::LLParser::PerFunctionState::~PerFunctionState()"},
    {"_ZN4llvm8LLParser16PerFunctionState14FinishFunctionEv", "llvm::LLParser::PerFunctionState::FinishFunction()"},
    {"_ZN4llvm8LLParser16PerFunctionState6GetValERKSsPKNS_4TypeENS_5SMLocE", "llvm::LLParser::PerFunctionState::GetVal(std::string const&, llvm::Type const*, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser16PerFunctionState6GetValEjPKNS_4TypeENS_5SMLocE", "llvm::LLParser::PerFunctionState::GetVal(unsigned int, llvm::Type const*, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser16PerFunctionState11SetInstNameEiRKSsNS_5SMLocEPNS_11InstructionE", "llvm::LLParser::PerFunctionState::SetInstName(int, std::string const&, llvm::SMLoc, llvm::Instruction*)"},
    {"_ZN4llvm8LLParser16PerFunctionState8DefineBBERKSsNS_5SMLocE", "llvm::LLParser::PerFunctionState::DefineBB(std::string const&, llvm::SMLoc)"},
    {"_ZN4llvm8LLParser18ParseMetadataValueERNS_5ValIDEPNS0_16PerFunctionStateE", "llvm::LLParser::ParseMetadataValue(llvm::ValID&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser22ParseGlobalValueVectorERNS_15SmallVectorImplIPNS_8ConstantEEE", "llvm::LLParser::ParseGlobalValueVector(llvm::SmallVectorImpl<llvm::Constant*>&)"},
    {"_ZN4llvm8LLParser17ParseCmpPredicateERjj", "llvm::LLParser::ParseCmpPredicate(unsigned int&, unsigned int)"},
    {"_ZN4llvm8LLParser19ConvertValIDToValueEPKNS_4TypeERNS_5ValIDERPNS_5ValueEPNS0_16PerFunctionStateE", "llvm::LLParser::ConvertValIDToValue(llvm::Type const*, llvm::ValID&, llvm::Value*&, llvm::LLParser::PerFunctionState*)"},
    {"_ZN4llvm8LLParser17ParseTypeAndValueERPNS_5ValueERNS0_16PerFunctionStateE", "llvm::LLParser::ParseTypeAndValue(llvm::Value*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser22ParseTypeAndBasicBlockERPNS_10BasicBlockERNS_5SMLocERNS0_16PerFunctionStateE", "llvm::LLParser::ParseTypeAndBasicBlock(llvm::BasicBlock*&, llvm::SMLoc&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser15ParseBasicBlockERNS0_16PerFunctionStateE", "llvm::LLParser::ParseBasicBlock(llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser16ParseInstructionERPNS_11InstructionEPNS_10BasicBlockERNS0_16PerFunctionStateE", "llvm::LLParser::ParseInstruction(llvm::Instruction*&, llvm::BasicBlock*, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser8ParseRetERPNS_11InstructionEPNS_10BasicBlockERNS0_16PerFunctionStateE", "llvm::LLParser::ParseRet(llvm::Instruction*&, llvm::BasicBlock*, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser7ParseBrERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseBr(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser11ParseSwitchERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseSwitch(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser15ParseIndirectBrERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseIndirectBr(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser11ParseInvokeERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseInvoke(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser15ParseArithmeticERPNS_11InstructionERNS0_16PerFunctionStateEjj", "llvm::LLParser::ParseArithmetic(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, unsigned int, unsigned int)"},
    {"_ZN4llvm8LLParser12ParseLogicalERPNS_11InstructionERNS0_16PerFunctionStateEj", "llvm::LLParser::ParseLogical(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, unsigned int)"},
    {"_ZN4llvm8LLParser12ParseCompareERPNS_11InstructionERNS0_16PerFunctionStateEj", "llvm::LLParser::ParseCompare(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, unsigned int)"},
    {"_ZN4llvm8LLParser9ParseCastERPNS_11InstructionERNS0_16PerFunctionStateEj", "llvm::LLParser::ParseCast(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, unsigned int)"},
    {"_ZN4llvm8LLParser11ParseSelectERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseSelect(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser11ParseVA_ArgERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseVA_Arg(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser19ParseExtractElementERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseExtractElement(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser18ParseInsertElementERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseInsertElement(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser18ParseShuffleVectorERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseShuffleVector(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser8ParsePHIERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParsePHI(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser9ParseCallERPNS_11InstructionERNS0_16PerFunctionStateEb", "llvm::LLParser::ParseCall(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, bool)"},
    {"_ZN4llvm8LLParser10ParseAllocERPNS_11InstructionERNS0_16PerFunctionStateEPNS_10BasicBlockEb", "llvm::LLParser::ParseAlloc(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, llvm::BasicBlock*, bool)"},
    {"_ZN4llvm8LLParser9ParseFreeERPNS_11InstructionERNS0_16PerFunctionStateEPNS_10BasicBlockE", "llvm::LLParser::ParseFree(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, llvm::BasicBlock*)"},
    {"_ZN4llvm8LLParser9ParseLoadERPNS_11InstructionERNS0_16PerFunctionStateEb", "llvm::LLParser::ParseLoad(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, bool)"},
    {"_ZN4llvm8LLParser10ParseStoreERPNS_11InstructionERNS0_16PerFunctionStateEb", "llvm::LLParser::ParseStore(llvm::Instruction*&, llvm::LLParser::PerFunctionState&, bool)"},
    {"_ZN4llvm8LLParser14ParseGetResultERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseGetResult(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser18ParseGetElementPtrERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseGetElementPtr(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser17ParseExtractValueERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseExtractValue(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8LLParser16ParseInsertValueERPNS_11InstructionERNS0_16PerFunctionStateE", "llvm::LLParser::ParseInsertValue(llvm::Instruction*&, llvm::LLParser::PerFunctionState&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt6vectorINS_8LLParser5MDRefESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE5clearEv", "llvm::DenseMap<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > > >::clear()"},
    {"_ZNSt3mapIjSt4pairIN4llvm10TrackingVHINS1_6MDNodeEEENS1_5SMLocEESt4lessIjESaIS0_IKjS6_EEEixERS9_", "std::map<unsigned int, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc>, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > > >::operator[](unsigned int const&)"},
    {"_ZNSt3mapISsSt4pairIPN4llvm11GlobalValueENS1_5SMLocEESt4lessISsESaIS0_IKSsS5_EEEixERS8_", "std::map<std::string, std::pair<llvm::GlobalValue*, llvm::SMLoc>, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::operator[](std::string const&)"},
    {"_ZNSt3mapISsSt4pairIPN4llvm5ValueENS1_5SMLocEESt4lessISsESaIS0_IKSsS5_EEEixERS8_", "std::map<std::string, std::pair<llvm::Value*, llvm::SMLoc>, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::operator[](std::string const&)"},
    {"_ZNSt3mapIN4llvm5ValIDESt6vectorISt4pairIS1_PNS0_11GlobalValueEESaIS6_EESt4lessIS1_ESaIS3_IKS1_S8_EEEixERSB_", "std::map<llvm::ValID, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::operator[](llvm::ValID const&)"},
    {"_ZNSt4pairIN4llvm5ValIDEPNS0_11GlobalValueEEC2IS1_PNS0_14GlobalVariableEEERKS_IT_T0_E", "std::pair<llvm::ValID, llvm::GlobalValue*>::pair<llvm::ValID, llvm::GlobalVariable*>(std::pair<llvm::ValID, llvm::GlobalVariable*> const&)"},
    {"_ZNSt4pairIN4llvm5ValIDEPNS0_14GlobalVariableEEC2ERKS1_RKS3_", "std::pair<llvm::ValID, llvm::GlobalVariable*>::pair(llvm::ValID const&, llvm::GlobalVariable* const&)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIN4llvm5ValIDEPNS2_11GlobalValueEEE9constructEPS6_RKS6_", "__gnu_cxx::new_allocator<std::pair<llvm::ValID, llvm::GlobalValue*> >::construct(std::pair<llvm::ValID, llvm::GlobalValue*>*, std::pair<llvm::ValID, llvm::GlobalValue*> const&)"},
    {"_ZNSt6vectorISt4pairIN4llvm5ValIDEPNS1_11GlobalValueEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::ValID, llvm::GlobalValue*>*, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::pair<llvm::ValID, llvm::GlobalValue*> const&)"},
    {"_ZNSt4pairIN4llvm5ValIDEPNS0_11GlobalValueEEaSERKS4_", "std::pair<llvm::ValID, llvm::GlobalValue*>::operator=(std::pair<llvm::ValID, llvm::GlobalValue*> const&)"},
    {"_ZSt24__uninitialized_copy_auxIPSt4pairIN4llvm5ValIDEPNS1_11GlobalValueEES6_ET0_T_S8_S7_St12__false_type", "std::pair<llvm::ValID, llvm::GlobalValue*>* std::__uninitialized_copy_aux<std::pair<llvm::ValID, llvm::GlobalValue*>*, std::pair<llvm::ValID, llvm::GlobalValue*>*>(std::pair<llvm::ValID, llvm::GlobalValue*>*, std::pair<llvm::ValID, llvm::GlobalValue*>*, std::pair<llvm::ValID, llvm::GlobalValue*>*, std::__false_type)"},
    {"_ZNSt4pairIKN4llvm5ValIDESt6vectorIS_IS1_PNS0_11GlobalValueEESaIS6_EEEC2ERS2_RKS8_", "std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >::pair(llvm::ValID const&, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > const&)"},
    {"_ZSt24__uninitialized_copy_auxIN9__gnu_cxx17__normal_iteratorIPKSt4pairIN4llvm5ValIDEPNS3_11GlobalValueEESt6vectorIS7_SaIS7_EEEEPS7_ET0_T_SG_SF_St12__false_type", "std::pair<llvm::ValID, llvm::GlobalValue*>* std::__uninitialized_copy_aux<__gnu_cxx::__normal_iterator<std::pair<llvm::ValID, llvm::GlobalValue*> const*, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::pair<llvm::ValID, llvm::GlobalValue*>*>(__gnu_cxx::__normal_iterator<std::pair<llvm::ValID, llvm::GlobalValue*> const*, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::ValID, llvm::GlobalValue*> const*, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::pair<llvm::ValID, llvm::GlobalValue*>*, std::__false_type)"},
    {"_ZNSt8_Rb_treeIN4llvm5ValIDESt4pairIKS1_St6vectorIS2_IS1_PNS0_11GlobalValueEESaIS7_EEESt10_Select1stISA_ESt4lessIS1_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<llvm::ValID, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::_Select1st<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > const&)"},
    {"_ZNSt8_Rb_treeIN4llvm5ValIDESt4pairIKS1_St6vectorIS2_IS1_PNS0_11GlobalValueEESaIS7_EEESt10_Select1stISA_ESt4lessIS1_ESaISA_EE9_M_insertEPSt18_Rb_tree_node_baseSI_RKSA_", "std::_Rb_tree<llvm::ValID, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::_Select1st<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > const&)"},
    {"_ZNSt8_Rb_treeIN4llvm5ValIDESt4pairIKS1_St6vectorIS2_IS1_PNS0_11GlobalValueEESaIS7_EEESt10_Select1stISA_ESt4lessIS1_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<llvm::ValID, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::_Select1st<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::_M_insert_unique(std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > const&)"},
    {"_ZNSt4pairIKN4llvm5ValIDESt6vectorIS_IS1_PNS0_11GlobalValueEESaIS6_EEEC2ERKS9_", "std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >::pair(std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE11equal_rangeERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::equal_range(std::string const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::erase(std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > > >::erase(std::_Rb_tree_iterator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::_Rb_tree_iterator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_insert_unique(std::pair<unsigned int const, std::pair<llvm::Value*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE9_M_insertEPSt18_Rb_tree_node_baseSF_RKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::_M_insert_unique(std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm5ValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE4findERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::Value*, llvm::SMLoc> > > >::find(std::string const&)"},
    {"_ZNSt8_Rb_treeIN4llvm5ValIDESt4pairIKS1_St6vectorIS2_IS1_PNS0_11GlobalValueEESaIS7_EEESt10_Select1stISA_ESt4lessIS1_ESaISA_EE4findERS3_", "std::_Rb_tree<llvm::ValID, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::_Select1st<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::find(llvm::ValID const&)"},
    {"_ZNSt6vectorIN4llvm8LLParser7ArgInfoESaIS2_EED2Ev", "std::vector<llvm::LLParser::ArgInfo, std::allocator<llvm::LLParser::ArgInfo> >::~vector()"},
    {"_ZNSt6vectorIN4llvm8LLParser7ArgInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::LLParser::ArgInfo, std::allocator<llvm::LLParser::ArgInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LLParser::ArgInfo*, std::vector<llvm::LLParser::ArgInfo, std::allocator<llvm::LLParser::ArgInfo> > >, llvm::LLParser::ArgInfo const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_8LLParser9ParamInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::LLParser::ParamInfo, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIN4llvm8LLParser11UpRefRecordESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::LLParser::UpRefRecord, std::allocator<llvm::LLParser::UpRefRecord> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LLParser::UpRefRecord*, std::vector<llvm::LLParser::UpRefRecord, std::allocator<llvm::LLParser::UpRefRecord> > >, llvm::LLParser::UpRefRecord const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessIjESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_insert_unique(std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessISsESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_insert_unique(std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessISsESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > const&)"},
    {"_ZSt6__findIPN4llvm12PATypeHandleEPKNS0_4TypeEET_S6_S6_RKT0_St26random_access_iterator_tag", "llvm::PATypeHandle* std::__find<llvm::PATypeHandle*, llvm::Type const*>(llvm::PATypeHandle*, llvm::PATypeHandle*, llvm::Type const* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIN4llvm8LLParser5MDRefESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LLParser::MDRef*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > >, llvm::LLParser::MDRef const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt6vectorINS_8LLParser5MDRefESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > > >::FindAndConstruct(llvm::Instruction* const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt6vectorINS_8LLParser5MDRefESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > > >::InsertIntoBucket(llvm::Instruction* const&, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > const&, std::pair<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > >*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt6vectorINS_8LLParser5MDRefESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > > >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm8LLParser5MDRefESaIS2_EEC2ERKS4_", "std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >::vector(std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_insert_unique(std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE9_M_insertEPSt18_Rb_tree_node_baseSF_RKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_insert_unique(std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > const&)"},
    {"_ZNSt6vectorIPN4llvm11GlobalValueESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::GlobalValue*, std::allocator<llvm::GlobalValue*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GlobalValue**, std::vector<llvm::GlobalValue*, std::allocator<llvm::GlobalValue*> > >, llvm::GlobalValue* const&)"},
    {"_ZN4llvm6iplistINS_14GlobalVariableENS_12ilist_traitsIS1_EEE8transferENS_14ilist_iteratorIS1_EERS4_S6_S6_", "llvm::iplist<llvm::GlobalVariable, llvm::ilist_traits<llvm::GlobalVariable> >::transfer(llvm::ilist_iterator<llvm::GlobalVariable>, llvm::iplist<llvm::GlobalVariable, llvm::ilist_traits<llvm::GlobalVariable> >&, llvm::ilist_iterator<llvm::GlobalVariable>, llvm::ilist_iterator<llvm::GlobalVariable>)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE11equal_rangeERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::equal_range(std::string const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::erase(std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::_Rb_tree_iterator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE4findERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::find(std::string const&)"},
    {"_ZNSt6vectorIN4llvm10TrackingVHINS0_6MDNodeEEESaIS3_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS3_S5_EEmRKS3_", "std::vector<llvm::TrackingVH<llvm::MDNode>, std::allocator<llvm::TrackingVH<llvm::MDNode> > >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::TrackingVH<llvm::MDNode>*, std::vector<llvm::TrackingVH<llvm::MDNode>, std::allocator<llvm::TrackingVH<llvm::MDNode> > > >, unsigned long, llvm::TrackingVH<llvm::MDNode> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm10TrackingVHINS2_6MDNodeEEENS2_5SMLocEEESt10_Select1stIS8_ESt4lessIjESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm10TrackingVHINS2_6MDNodeEEENS2_5SMLocEEESt10_Select1stIS8_ESt4lessIjESaIS8_EE9_M_insertEPSt18_Rb_tree_node_baseSG_RKS8_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm10TrackingVHINS2_6MDNodeEEENS2_5SMLocEEESt10_Select1stIS8_ESt4lessIjESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > > >::_M_insert_unique(std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessISsESaIS6_EE4findERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::find(std::string const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt6vectorINS_8LLParser5MDRefESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> >, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::vector<llvm::LLParser::MDRef, std::allocator<llvm::LLParser::MDRef> > > >::shrink_and_clear()"},
    {"_ZN4llvm13ParseAssemblyEPNS_12MemoryBufferEPNS_6ModuleERNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::ParseAssembly(llvm::MemoryBuffer*, llvm::Module*, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZN4llvm17ParseAssemblyFileERKSsRNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::ParseAssemblyFile(std::string const&, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZN4llvm19ParseAssemblyStringEPKcPNS_6ModuleERNS_12SMDiagnosticERNS_11LLVMContextE", "llvm::ParseAssemblyString(char const*, llvm::Module*, llvm::SMDiagnostic&, llvm::LLVMContext&)"},
    {"_ZN4llvm8LLParserD2Ev", "llvm::LLParser::~LLParser()"},
    {"_ZN4llvm7LLLexerD2Ev", "llvm::LLLexer::~LLLexer()"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessISsESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessISsESaIS6_EE15_M_destroy_nodeEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_destroy_node(std::_Rb_tree_node<std::pair<std::string const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm12PATypeHolderENS2_5SMLocEEESt10_Select1stIS6_ESt4lessIjESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::pair<llvm::PATypeHolder, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IN4llvm10TrackingVHINS2_6MDNodeEEENS2_5SMLocEEESt10_Select1stIS8_ESt4lessIjESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::pair<llvm::TrackingVH<llvm::MDNode>, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessISsESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<std::string, std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<std::string>, std::allocator<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPN4llvm11GlobalValueENS2_5SMLocEEESt10_Select1stIS7_ESt4lessIjESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::pair<llvm::GlobalValue*, llvm::SMLoc> > >*)"},
    {"_ZNSt8_Rb_treeIN4llvm5ValIDESt4pairIKS1_St6vectorIS2_IS1_PNS0_11GlobalValueEESaIS7_EEESt10_Select1stISA_ESt4lessIS1_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<llvm::ValID, std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >, std::_Select1st<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >, std::less<llvm::ValID>, std::allocator<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > > >*)"},
    {"_ZNSt4pairIKN4llvm5ValIDESt6vectorIS_IS1_PNS0_11GlobalValueEESaIS6_EEED2Ev", "std::pair<llvm::ValID const, std::vector<std::pair<llvm::ValID, llvm::GlobalValue*>, std::allocator<std::pair<llvm::ValID, llvm::GlobalValue*> > > >::~pair()"},
    {"_ZSt13__destroy_auxIPSt4pairIN4llvm5ValIDEPNS1_11GlobalValueEEEvT_S7_St12__false_type", "void std::__destroy_aux<std::pair<llvm::ValID, llvm::GlobalValue*>*>(std::pair<llvm::ValID, llvm::GlobalValue*>*, std::pair<llvm::ValID, llvm::GlobalValue*>*, std::__false_type)"},
    {"_ZN4llvm8LLParserC2EPNS_12MemoryBufferERNS_9SourceMgrERNS_12SMDiagnosticEPNS_6ModuleE", "llvm::LLParser::LLParser(llvm::MemoryBuffer*, llvm::SourceMgr&, llvm::SMDiagnostic&, llvm::Module*)"},
    {"_ZN4llvm16RegisterAsmLexerIN12_GLOBAL__N_111X86AsmLexerEE9AllocatorERKNS_6TargetERKNS_9MCAsmInfoE", "llvm::RegisterAsmLexer<(anonymous namespace)::X86AsmLexer>::Allocator(llvm::Target const&, llvm::MCAsmInfo const&)"},
    {"_ZN12_GLOBAL__N_111X86AsmLexer8LexTokenEv", "(anonymous namespace)::X86AsmLexer::LexToken()"},
    {"_ZN12_GLOBAL__N_111X86AsmLexerD1Ev", "(anonymous namespace)::X86AsmLexer::~X86AsmLexer()"},
    {"_ZN12_GLOBAL__N_111X86AsmLexerD0Ev", "(anonymous namespace)::X86AsmLexer::~X86AsmLexer()"},
    {"_Z17MatchRegisterNameN4llvm9StringRefE", "MatchRegisterName(llvm::StringRef)"},
    {"_ZN4llvm17RegisterAsmParserIN12_GLOBAL__N_118X86_64ATTAsmParserEE9AllocatorERKNS_6TargetERNS_11MCAsmParserERNS_13TargetMachineE", "llvm::RegisterAsmParser<(anonymous namespace)::X86_64ATTAsmParser>::Allocator(llvm::Target const&, llvm::MCAsmParser&, llvm::TargetMachine&)"},
    {"_ZN12_GLOBAL__N_118X86_64ATTAsmParserD1Ev", "(anonymous namespace)::X86_64ATTAsmParser::~X86_64ATTAsmParser()"},
    {"_ZN12_GLOBAL__N_118X86_64ATTAsmParserD0Ev", "(anonymous namespace)::X86_64ATTAsmParser::~X86_64ATTAsmParser()"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser13ParseRegisterERjRN4llvm5SMLocES4_", "(anonymous namespace)::X86ATTAsmParser::ParseRegister(unsigned int&, llvm::SMLoc&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser16ParseInstructionEN4llvm9StringRefENS1_5SMLocERNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEE", "(anonymous namespace)::X86ATTAsmParser::ParseInstruction(llvm::StringRef, llvm::SMLoc, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>&)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser14ParseDirectiveEN4llvm8AsmTokenE", "(anonymous namespace)::X86ATTAsmParser::ParseDirective(llvm::AsmToken)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser23MatchAndEmitInstructionEN4llvm5SMLocERNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEERNS1_10MCStreamerE", "(anonymous namespace)::X86ATTAsmParser::MatchAndEmitInstruction(llvm::SMLoc, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>&, llvm::MCStreamer&)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser20MatchInstructionImplERKN4llvm15SmallVectorImplIPNS1_18MCParsedAsmOperandEEERNS1_6MCInstERj", "(anonymous namespace)::X86ATTAsmParser::MatchInstructionImpl(llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*> const&, llvm::MCInst&, unsigned int&)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser15ConvertToMCInstEjRN4llvm6MCInstEjRKNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEE", "(anonymous namespace)::X86ATTAsmParser::ConvertToMCInst(unsigned int, llvm::MCInst&, unsigned int, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*> const&)"},
    {"_ZNK12_GLOBAL__N_110X86Operand14addMemOperandsERN4llvm6MCInstEj", "(anonymous namespace)::X86Operand::addMemOperands(llvm::MCInst&, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_110X86Operand7addExprERN4llvm6MCInstEPKNS1_6MCExprE", "(anonymous namespace)::X86Operand::addExpr(llvm::MCInst&, llvm::MCExpr const*) const"},
    {"_ZN12_GLOBAL__N_110X86OperandD1Ev", "(anonymous namespace)::X86Operand::~X86Operand()"},
    {"_ZN12_GLOBAL__N_110X86OperandD0Ev", "(anonymous namespace)::X86Operand::~X86Operand()"},
    {"_ZNK12_GLOBAL__N_110X86Operand11getStartLocEv", "(anonymous namespace)::X86Operand::getStartLoc() const"},
    {"_ZNK12_GLOBAL__N_110X86Operand9getEndLocEv", "(anonymous namespace)::X86Operand::getEndLoc() const"},
    {"_ZNK12_GLOBAL__N_110X86Operand4dumpERN4llvm11raw_ostreamE", "(anonymous namespace)::X86Operand::dump(llvm::raw_ostream&) const"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser12ParseOperandEv", "(anonymous namespace)::X86ATTAsmParser::ParseOperand()"},
    {"_Z17MatchRegisterNameN4llvm9StringRefE", "MatchRegisterName(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParser15ParseMemOperandEjN4llvm5SMLocE", "(anonymous namespace)::X86ATTAsmParser::ParseMemOperand(unsigned int, llvm::SMLoc)"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParserD1Ev", "(anonymous namespace)::X86ATTAsmParser::~X86ATTAsmParser()"},
    {"_ZN12_GLOBAL__N_115X86ATTAsmParserD0Ev", "(anonymous namespace)::X86ATTAsmParser::~X86ATTAsmParser()"},
    {"_ZN4llvm17RegisterAsmParserIN12_GLOBAL__N_118X86_32ATTAsmParserEE9AllocatorERKNS_6TargetERNS_11MCAsmParserERNS_13TargetMachineE", "llvm::RegisterAsmParser<(anonymous namespace)::X86_32ATTAsmParser>::Allocator(llvm::Target const&, llvm::MCAsmParser&, llvm::TargetMachine&)"},
    {"_ZN12_GLOBAL__N_118X86_32ATTAsmParserD1Ev", "(anonymous namespace)::X86_32ATTAsmParser::~X86_32ATTAsmParser()"},
    {"_ZN12_GLOBAL__N_118X86_32ATTAsmParserD0Ev", "(anonymous namespace)::X86_32ATTAsmParser::~X86_32ATTAsmParser()"},
    {"_ZN4llvm22createSSEDomainFixPassEv", "llvm::createSSEDomainFixPass()"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPassD1Ev", "(anonymous namespace)::SSEDomainFixPass::~SSEDomainFixPass()"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPassD0Ev", "(anonymous namespace)::SSEDomainFixPass::~SSEDomainFixPass()"},
    {"_ZNK12_GLOBAL__N_116SSEDomainFixPass11getPassNameEv", "(anonymous namespace)::SSEDomainFixPass::getPassName() const"},
    {"_ZNK12_GLOBAL__N_116SSEDomainFixPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::SSEDomainFixPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::SSEDomainFixPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass14visitHardInstrEPN4llvm12MachineInstrEj", "(anonymous namespace)::SSEDomainFixPass::visitHardInstr(llvm::MachineInstr*, unsigned int)"},
    {"_ZN4llvm24SpecificBumpPtrAllocatorIN12_GLOBAL__N_111DomainValueEE10DestroyAllEv", "llvm::SpecificBumpPtrAllocator<(anonymous namespace)::DomainValue>::DestroyAll()"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass8CollapseEPNS_11DomainValueEj", "(anonymous namespace)::SSEDomainFixPass::Collapse((anonymous namespace)::DomainValue*, unsigned int)"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass10SetLiveRegEiPNS_11DomainValueE", "(anonymous namespace)::SSEDomainFixPass::SetLiveReg(int, (anonymous namespace)::DomainValue*)"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass5ForceEij", "(anonymous namespace)::SSEDomainFixPass::Force(int, unsigned int)"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass5MergeEPNS_11DomainValueES2_", "(anonymous namespace)::SSEDomainFixPass::Merge((anonymous namespace)::DomainValue*, (anonymous namespace)::DomainValue*)"},
    {"_ZN4llvm12df_ext_beginIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj16EEEEENS_15df_ext_iteratorIT_T0_EERKS6_RS7_", "llvm::df_ext_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u> > llvm::df_ext_begin<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u> >(llvm::MachineBasicBlock* const&, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u>&)"},
    {"_ZN4llvm11df_iteratorIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj16EEELb1ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u>, true, llvm::GraphTraits<llvm::MachineBasicBlock*> >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_17MachineBasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEEN9__gnu_cxx17__normal_iteratorIPS4_S_IS4_SaIS4_EEEEESaISE_EE13_M_insert_auxENS9_IPSE_SG_EERKSE_", "std::vector<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >*, std::vector<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > > > > >, std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > > const&)"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_17MachineBasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEEN9__gnu_cxx17__normal_iteratorIPS4_S_IS4_SaIS4_EEEEESaISE_EEC2ERKSG_", "std::vector<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > > > >::vector(std::vector<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::MachineBasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::MachineBasicBlock*> >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > > > > const&)"},
    {"_ZN4llvm11df_iteratorIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj16EEELb1ENS_11GraphTraitsIS2_EEEC2ES2_RS4_", "llvm::df_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u>, true, llvm::GraphTraits<llvm::MachineBasicBlock*> >::df_iterator(llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 16u>&)"},
    {"_ZN4llvm22createX86_32AsmBackendERKNS_6TargetERKSs", "llvm::createX86_32AsmBackend(llvm::Target const&, std::string const&)"},
    {"_ZN4llvm22createX86_64AsmBackendERKNS_6TargetERKSs", "llvm::createX86_64AsmBackend(llvm::Target const&, std::string const&)"},
    {"_ZN12_GLOBAL__N_119ELFX86_64AsmBackendD1Ev", "(anonymous namespace)::ELFX86_64AsmBackend::~ELFX86_64AsmBackend()"},
    {"_ZN12_GLOBAL__N_119ELFX86_64AsmBackendD0Ev", "(anonymous namespace)::ELFX86_64AsmBackend::~ELFX86_64AsmBackend()"},
    {"_ZNK12_GLOBAL__N_119ELFX86_64AsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::ELFX86_64AsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZNK12_GLOBAL__N_116ELFX86AsmBackend25doesSectionRequireSymbolsERKN4llvm9MCSectionE", "(anonymous namespace)::ELFX86AsmBackend::doesSectionRequireSymbols(llvm::MCSection const&) const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend16getNumFixupKindsEv", "(anonymous namespace)::X86AsmBackend::getNumFixupKinds() const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend16getFixupKindInfoEN4llvm11MCFixupKindE", "(anonymous namespace)::X86AsmBackend::getFixupKindInfo(llvm::MCFixupKind) const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend10ApplyFixupERKN4llvm7MCFixupEPcjy", "(anonymous namespace)::X86AsmBackend::ApplyFixup(llvm::MCFixup const&, char*, unsigned int, unsigned long long) const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend17MayNeedRelaxationERKN4llvm6MCInstE", "(anonymous namespace)::X86AsmBackend::MayNeedRelaxation(llvm::MCInst const&) const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend16RelaxInstructionERKN4llvm6MCInstERS2_", "(anonymous namespace)::X86AsmBackend::RelaxInstruction(llvm::MCInst const&, llvm::MCInst&) const"},
    {"_ZNK12_GLOBAL__N_113X86AsmBackend12WriteNopDataEyPN4llvm14MCObjectWriterE", "(anonymous namespace)::X86AsmBackend::WriteNopData(unsigned long long, llvm::MCObjectWriter*) const"},
    {"_ZN12_GLOBAL__N_118X86ELFObjectWriterD1Ev", "(anonymous namespace)::X86ELFObjectWriter::~X86ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_118X86ELFObjectWriterD0Ev", "(anonymous namespace)::X86ELFObjectWriter::~X86ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_116ELFX86AsmBackendD1Ev", "(anonymous namespace)::ELFX86AsmBackend::~ELFX86AsmBackend()"},
    {"_ZN12_GLOBAL__N_116ELFX86AsmBackendD0Ev", "(anonymous namespace)::ELFX86AsmBackend::~ELFX86AsmBackend()"},
    {"_ZN12_GLOBAL__N_113X86AsmBackendD1Ev", "(anonymous namespace)::X86AsmBackend::~X86AsmBackend()"},
    {"_ZN12_GLOBAL__N_113X86AsmBackendD0Ev", "(anonymous namespace)::X86AsmBackend::~X86AsmBackend()"},
    {"_ZN12_GLOBAL__N_122DarwinX86_64AsmBackendD1Ev", "(anonymous namespace)::DarwinX86_64AsmBackend::~DarwinX86_64AsmBackend()"},
    {"_ZN12_GLOBAL__N_122DarwinX86_64AsmBackendD0Ev", "(anonymous namespace)::DarwinX86_64AsmBackend::~DarwinX86_64AsmBackend()"},
    {"_ZNK12_GLOBAL__N_122DarwinX86_64AsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::DarwinX86_64AsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZNK12_GLOBAL__N_122DarwinX86_64AsmBackend25doesSectionRequireSymbolsERKN4llvm9MCSectionE", "(anonymous namespace)::DarwinX86_64AsmBackend::doesSectionRequireSymbols(llvm::MCSection const&) const"},
    {"_ZNK12_GLOBAL__N_122DarwinX86_64AsmBackend19isSectionAtomizableERKN4llvm9MCSectionE", "(anonymous namespace)::DarwinX86_64AsmBackend::isSectionAtomizable(llvm::MCSection const&) const"},
    {"_ZN12_GLOBAL__N_119DarwinX86AsmBackendD1Ev", "(anonymous namespace)::DarwinX86AsmBackend::~DarwinX86AsmBackend()"},
    {"_ZN12_GLOBAL__N_119DarwinX86AsmBackendD0Ev", "(anonymous namespace)::DarwinX86AsmBackend::~DarwinX86AsmBackend()"},
    {"_ZN12_GLOBAL__N_119ELFX86_32AsmBackendD1Ev", "(anonymous namespace)::ELFX86_32AsmBackend::~ELFX86_32AsmBackend()"},
    {"_ZN12_GLOBAL__N_119ELFX86_32AsmBackendD0Ev", "(anonymous namespace)::ELFX86_32AsmBackend::~ELFX86_32AsmBackend()"},
    {"_ZNK12_GLOBAL__N_119ELFX86_32AsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::ELFX86_32AsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZN12_GLOBAL__N_120WindowsX86AsmBackendD1Ev", "(anonymous namespace)::WindowsX86AsmBackend::~WindowsX86AsmBackend()"},
    {"_ZN12_GLOBAL__N_120WindowsX86AsmBackendD0Ev", "(anonymous namespace)::WindowsX86AsmBackend::~WindowsX86AsmBackend()"},
    {"_ZNK12_GLOBAL__N_120WindowsX86AsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::WindowsX86AsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZN12_GLOBAL__N_122DarwinX86_32AsmBackendD1Ev", "(anonymous namespace)::DarwinX86_32AsmBackend::~DarwinX86_32AsmBackend()"},
    {"_ZN12_GLOBAL__N_122DarwinX86_32AsmBackendD0Ev", "(anonymous namespace)::DarwinX86_32AsmBackend::~DarwinX86_32AsmBackend()"},
    {"_ZNK12_GLOBAL__N_122DarwinX86_32AsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::DarwinX86_32AsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZNK4llvm16TargetAsmBackend19isSectionAtomizableERKNS_9MCSectionE", "llvm::TargetAsmBackend::isSectionAtomizable(llvm::MCSection const&) const"},
    {"_ZN4llvm16TargetAsmBackend19HandleAssemblerFlagENS_15MCAssemblerFlagE", "llvm::TargetAsmBackend::HandleAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN4llvm15SmallVectorImplINS_9MCOperandEEaSERKS2_", "llvm::SmallVectorImpl<llvm::MCOperand>::operator=(llvm::SmallVectorImpl<llvm::MCOperand> const&)"},
    {"_ZNK4llvm16TargetAsmBackend25doesSectionRequireSymbolsERKNS_9MCSectionE", "llvm::TargetAsmBackend::doesSectionRequireSymbols(llvm::MCSection const&) const"},
    {"_ZN4llvm13X86AsmPrinter20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::X86AsmPrinter::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm13X86AsmPrinter18printSymbolOperandERKNS_14MachineOperandERNS_11raw_ostreamE", "llvm::X86AsmPrinter::printSymbolOperand(llvm::MachineOperand const&, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter15print_pcrel_immEPKNS_12MachineInstrEjRNS_11raw_ostreamE", "llvm::X86AsmPrinter::print_pcrel_imm(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter12printOperandEPKNS_12MachineInstrEjRNS_11raw_ostreamEPKc", "llvm::X86AsmPrinter::printOperand(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&, char const*)"},
    {"_ZN4llvm13X86AsmPrinter10printSSECCEPKNS_12MachineInstrEjRNS_11raw_ostreamE", "llvm::X86AsmPrinter::printSSECC(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter20printLeaMemReferenceEPKNS_12MachineInstrEjRNS_11raw_ostreamEPKc", "llvm::X86AsmPrinter::printLeaMemReference(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&, char const*)"},
    {"_ZN4llvm13X86AsmPrinter17printMemReferenceEPKNS_12MachineInstrEjRNS_11raw_ostreamEPKc", "llvm::X86AsmPrinter::printMemReference(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&, char const*)"},
    {"_ZN4llvm13X86AsmPrinter13printPICLabelEPKNS_12MachineInstrEjRNS_11raw_ostreamE", "llvm::X86AsmPrinter::printPICLabel(llvm::MachineInstr const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter17printAsmMRegisterERKNS_14MachineOperandEcRNS_11raw_ostreamE", "llvm::X86AsmPrinter::printAsmMRegister(llvm::MachineOperand const&, char, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter15PrintAsmOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::X86AsmPrinter::PrintAsmOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter21PrintAsmMemoryOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::X86AsmPrinter::PrintAsmMemoryOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZN4llvm13X86AsmPrinter18EmitStartOfAsmFileERNS_6ModuleE", "llvm::X86AsmPrinter::EmitStartOfAsmFile(llvm::Module&)"},
    {"_ZN4llvm13X86AsmPrinter16EmitEndOfAsmFileERNS_6ModuleE", "llvm::X86AsmPrinter::EmitEndOfAsmFile(llvm::Module&)"},
    {"_ZNK4llvm13X86AsmPrinter21getDebugValueLocationEPKNS_12MachineInstrE", "llvm::X86AsmPrinter::getDebugValueLocation(llvm::MachineInstr const*) const"},
    {"_ZN4llvm13X86AsmPrinter22PrintDebugValueCommentEPKNS_12MachineInstrERNS_11raw_ostreamE", "llvm::X86AsmPrinter::PrintDebugValueComment(llvm::MachineInstr const*, llvm::raw_ostream&)"},
    {"_Z22createX86MCInstPrinterRKN4llvm6TargetEjRKNS_9MCAsmInfoE", "createX86MCInstPrinter(llvm::Target const&, unsigned int, llvm::MCAsmInfo const&)"},
    {"_ZNK4llvm12X86Subtarget12isTargetCOFFEv", "llvm::X86Subtarget::isTargetCOFF() const"},
    {"_ZNK4llvm12X86Subtarget11isTargetELFEv", "llvm::X86Subtarget::isTargetELF() const"},
    {"_ZN4llvm7DIScopeD1Ev", "llvm::DIScope::~DIScope()"},
    {"_ZN4llvm12DISubprogramD1Ev", "llvm::DISubprogram::~DISubprogram()"},
    {"_ZN4llvm13X86AsmPrinterD1Ev", "llvm::X86AsmPrinter::~X86AsmPrinter()"},
    {"_ZN4llvm13X86AsmPrinterD0Ev", "llvm::X86AsmPrinter::~X86AsmPrinter()"},
    {"_ZNK4llvm13X86AsmPrinter11getPassNameEv", "llvm::X86AsmPrinter::getPassName() const"},
    {"_ZN4llvm10AsmPrinter14getISAEncodingEv", "llvm::AsmPrinter::getISAEncoding()"},
    {"_ZN4llvm18RegisterAsmPrinterINS_13X86AsmPrinterEE9AllocatorERNS_13TargetMachineERNS_10MCStreamerE", "llvm::RegisterAsmPrinter<llvm::X86AsmPrinter>::Allocator(llvm::TargetMachine&, llvm::MCStreamer&)"},
    {"_ZNSt6vectorIPKN4llvm8MCSymbolESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::MCSymbol const*, std::allocator<llvm::MCSymbol const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCSymbol const**, std::vector<llvm::MCSymbol const*, std::allocator<llvm::MCSymbol const*> > >, llvm::MCSymbol const* const&)"},
    {"_ZN4llvm12DISubprogramD0Ev", "llvm::DISubprogram::~DISubprogram()"},
    {"_ZN4llvm7DIScopeD0Ev", "llvm::DIScope::~DIScope()"},
    {"_ZN4llvm24X86COFFMachineModuleInfoD0Ev", "llvm::X86COFFMachineModuleInfo::~X86COFFMachineModuleInfo()"},
    {"_ZN4llvm24X86COFFMachineModuleInfoD1Ev", "llvm::X86COFFMachineModuleInfo::~X86COFFMachineModuleInfo()"},
    {"_ZN4llvm24X86COFFMachineModuleInfoD2Ev", "llvm::X86COFFMachineModuleInfo::~X86COFFMachineModuleInfo()"},
    {"_ZN4llvm27createX86JITCodeEmitterPassERNS_16X86TargetMachineERNS_14JITCodeEmitterE", "llvm::createX86JITCodeEmitterPass(llvm::X86TargetMachine&, llvm::JITCodeEmitter&)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEED1Ev", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::~Emitter()"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEED0Ev", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::~Emitter()"},
    {"_ZNK12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE11getPassNameEv", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::getPassName() const"},
    {"_ZNK12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE16getAnalysisUsageERNS1_13AnalysisUsageE", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE20runOnMachineFunctionERNS1_15MachineFunctionE", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE15emitInstructionERNS1_12MachineInstrEPKNS1_15TargetInstrDescE", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitInstruction(llvm::MachineInstr&, llvm::TargetInstrDesc const*)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE17emitGlobalAddressEPKNS1_11GlobalValueEjllb", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitGlobalAddress(llvm::GlobalValue const*, unsigned int, long, long, bool)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE25emitExternalSymbolAddressEPKcj", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitExternalSymbolAddress(char const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE20emitJumpTableAddressEjjl", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitJumpTableAddress(unsigned int, unsigned int, long)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE20emitConstPoolAddressEjjll", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitConstPoolAddress(unsigned int, unsigned int, long, long)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE16emitMemModRMByteERKNS1_12MachineInstrEjjl", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitMemModRMByte(llvm::MachineInstr const&, unsigned int, unsigned int, long)"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE21emitDisplacementFieldEPKNS1_14MachineOperandEilb", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitDisplacementField(llvm::MachineOperand const*, int, long, bool)"},
    {"_ZN4llvm16X86ELFWriterInfoC1Ebb", "llvm::X86ELFWriterInfo::X86ELFWriterInfo(bool, bool)"},
    {"_ZN4llvm16X86ELFWriterInfoC2Ebb", "llvm::X86ELFWriterInfo::X86ELFWriterInfo(bool, bool)"},
    {"_ZN4llvm16X86ELFWriterInfoD0Ev", "llvm::X86ELFWriterInfo::~X86ELFWriterInfo()"},
    {"_ZN4llvm16X86ELFWriterInfoD1Ev", "llvm::X86ELFWriterInfo::~X86ELFWriterInfo()"},
    {"_ZN4llvm16X86ELFWriterInfoD2Ev", "llvm::X86ELFWriterInfo::~X86ELFWriterInfo()"},
    {"_ZNK4llvm16X86ELFWriterInfo17getRelocationTypeEj", "llvm::X86ELFWriterInfo::getRelocationType(unsigned int) const"},
    {"_ZNK4llvm16X86ELFWriterInfo24getDefaultAddendForRelTyEjl", "llvm::X86ELFWriterInfo::getDefaultAddendForRelTy(unsigned int, long) const"},
    {"_ZNK4llvm16X86ELFWriterInfo19getRelocationTySizeEj", "llvm::X86ELFWriterInfo::getRelocationTySize(unsigned int) const"},
    {"_ZNK4llvm16X86ELFWriterInfo15isPCRelativeRelEj", "llvm::X86ELFWriterInfo::isPCRelativeRel(unsigned int) const"},
    {"_ZNK4llvm16X86ELFWriterInfo28getAbsoluteLabelMachineRelTyEv", "llvm::X86ELFWriterInfo::getAbsoluteLabelMachineRelTy() const"},
    {"_ZNK4llvm16X86ELFWriterInfo17computeRelocationEjjj", "llvm::X86ELFWriterInfo::computeRelocation(unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16X86ELFWriterInfo19hasRelocationAddendEv", "llvm::X86ELFWriterInfo::hasRelocationAddend() const"},
    {"_ZN4llvm3X8614createFastISelERNS_20FunctionLoweringInfoE", "llvm::X86::createFastISel(llvm::FunctionLoweringInfo&)"},
    {"_ZN12_GLOBAL__N_111X86FastISel13TryToFoldLoadEPN4llvm12MachineInstrEjPKNS1_8LoadInstE", "(anonymous namespace)::X86FastISel::TryToFoldLoad(llvm::MachineInstr*, unsigned int, llvm::LoadInst const*)"},
    {"_ZN12_GLOBAL__N_111X86FastISelD1Ev", "(anonymous namespace)::X86FastISel::~X86FastISel()"},
    {"_ZN12_GLOBAL__N_111X86FastISelD0Ev", "(anonymous namespace)::X86FastISel::~X86FastISel()"},
    {"_ZN12_GLOBAL__N_111X86FastISel23TargetSelectInstructionEPKN4llvm11InstructionE", "(anonymous namespace)::X86FastISel::TargetSelectInstruction(llvm::Instruction const*)"},
    {"_ZN12_GLOBAL__N_111X86FastISel10FastEmit_rEN4llvm3MVTES2_jjb", "(anonymous namespace)::X86FastISel::FastEmit_r(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111X86FastISel11FastEmit_rrEN4llvm3MVTES2_jjbjb", "(anonymous namespace)::X86FastISel::FastEmit_rr(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111X86FastISel11FastEmit_riEN4llvm3MVTES2_jjby", "(anonymous namespace)::X86FastISel::FastEmit_ri(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111X86FastISel10FastEmit_iEN4llvm3MVTES2_jy", "(anonymous namespace)::X86FastISel::FastEmit_i(llvm::MVT, llvm::MVT, unsigned int, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111X86FastISel25TargetMaterializeConstantEPKN4llvm8ConstantE", "(anonymous namespace)::X86FastISel::TargetMaterializeConstant(llvm::Constant const*)"},
    {"_ZN12_GLOBAL__N_111X86FastISel23TargetMaterializeAllocaEPKN4llvm10AllocaInstE", "(anonymous namespace)::X86FastISel::TargetMaterializeAlloca(llvm::AllocaInst const*)"},
    {"_ZN12_GLOBAL__N_111X86FastISel16X86SelectAddressEPKN4llvm5ValueERNS1_14X86AddressModeE", "(anonymous namespace)::X86FastISel::X86SelectAddress(llvm::Value const*, llvm::X86AddressMode&)"},
    {"_ZN4llvm14addFullAddressERKNS_19MachineInstrBuilderERKNS_14X86AddressModeE", "llvm::addFullAddress(llvm::MachineInstrBuilder const&, llvm::X86AddressMode const&)"},
    {"_ZN12_GLOBAL__N_111X86FastISel13X86SelectZExtEPKN4llvm11InstructionE", "(anonymous namespace)::X86FastISel::X86SelectZExt(llvm::Instruction const*)"},
    {"_ZN12_GLOBAL__N_111X86FastISel14X86SelectTruncEPKN4llvm11InstructionE", "(anonymous namespace)::X86FastISel::X86SelectTrunc(llvm::Instruction const*)"},
    {"_Z6CC_X86jN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_X86(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN12_GLOBAL__N_111X86FastISel16X86FastEmitStoreEN4llvm3EVTEPKNS1_5ValueERKNS1_14X86AddressModeE", "(anonymous namespace)::X86FastISel::X86FastEmitStore(llvm::EVT, llvm::Value const*, llvm::X86AddressMode const&)"},
    {"_ZN12_GLOBAL__N_111X86FastISel16X86FastEmitStoreEN4llvm3EVTEjRKNS1_14X86AddressModeE", "(anonymous namespace)::X86FastISel::X86FastEmitStore(llvm::EVT, unsigned int, llvm::X86AddressMode const&)"},
    {"_Z9RetCC_X86jN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm17addFrameReferenceERKNS_19MachineInstrBuilderEii", "llvm::addFrameReference(llvm::MachineInstrBuilder const&, int, int)"},
    {"_Z15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN12_GLOBAL__N_111X86FastISel18X86FastEmitCompareEPKN4llvm5ValueES4_NS1_3EVTE", "(anonymous namespace)::X86FastISel::X86FastEmitCompare(llvm::Value const*, llvm::Value const*, llvm::EVT)"},
    {"_ZN4llvm14X86AddressMode14getFullAddressERNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::X86AddressMode::getFullAddress(llvm::SmallVectorImpl<llvm::MachineOperand>&)"},
    {"_ZN4llvm36createX86FloatingPointStackifierPassEv", "llvm::createX86FloatingPointStackifierPass()"},
    {"_ZN12_GLOBAL__N_13FPSD1Ev", "(anonymous namespace)::FPS::~FPS()"},
    {"_ZN12_GLOBAL__N_13FPSD0Ev", "(anonymous namespace)::FPS::~FPS()"},
    {"_ZNK12_GLOBAL__N_13FPS11getPassNameEv", "(anonymous namespace)::FPS::getPassName() const"},
    {"_ZNK12_GLOBAL__N_13FPS16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::FPS::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_13FPS20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::FPS::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_13FPS17processBasicBlockERN4llvm15MachineFunctionERNS1_17MachineBasicBlockE", "(anonymous namespace)::FPS::processBasicBlock(llvm::MachineFunction&, llvm::MachineBasicBlock&)"},
    {"_ZN12_GLOBAL__N_13FPS14adjustLiveRegsEjN4llvm14ilist_iteratorINS1_12MachineInstrEEE", "(anonymous namespace)::FPS::adjustLiveRegs(unsigned int, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN12_GLOBAL__N_13FPS9moveToTopEjN4llvm14ilist_iteratorINS1_12MachineInstrEEE", "(anonymous namespace)::FPS::moveToTop(unsigned int, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN12_GLOBAL__N_13FPS13popStackAfterERN4llvm14ilist_iteratorINS1_12MachineInstrEEE", "(anonymous namespace)::FPS::popStackAfter(llvm::ilist_iterator<llvm::MachineInstr>&)"},
    {"_ZN12_GLOBAL__N_13FPS19freeStackSlotBeforeEN4llvm14ilist_iteratorINS1_12MachineInstrEEEj", "(anonymous namespace)::FPS::freeStackSlotBefore(llvm::ilist_iterator<llvm::MachineInstr>, unsigned int)"},
    {"_ZN12_GLOBAL__N_13FPS14duplicateToTopEjjPN4llvm12MachineInstrE", "(anonymous namespace)::FPS::duplicateToTop(unsigned int, unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm12df_ext_beginIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj8EEEEENS_15df_ext_iteratorIT_T0_EERKS6_RS7_", "llvm::df_ext_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u> > llvm::df_ext_begin<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u> >(llvm::MachineBasicBlock* const&, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>&)"},
    {"_ZN4llvm11df_iteratorIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj8EEELb1ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>, true, llvm::GraphTraits<llvm::MachineBasicBlock*> >::toNext()"},
    {"_ZN4llvm11df_iteratorIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj8EEELb1ENS_11GraphTraitsIS2_EEEC2ES2_RS4_", "llvm::df_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>, true, llvm::GraphTraits<llvm::MachineBasicBlock*> >::df_iterator(llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>&)"},
    {"_ZNK4llvm16X86FrameLowering20hasReservedCallFrameERKNS_15MachineFunctionE", "llvm::X86FrameLowering::hasReservedCallFrame(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16X86FrameLowering5hasFPERKNS_15MachineFunctionE", "llvm::X86FrameLowering::hasFP(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16X86FrameLowering25emitCalleeSavedFrameMovesERNS_15MachineFunctionEPNS_8MCSymbolEj", "llvm::X86FrameLowering::emitCalleeSavedFrameMoves(llvm::MachineFunction&, llvm::MCSymbol*, unsigned int) const"},
    {"_ZNK4llvm16X86FrameLowering12emitPrologueERNS_15MachineFunctionE", "llvm::X86FrameLowering::emitPrologue(llvm::MachineFunction&) const"},
    {"_Z14mergeSPUpdatesRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEjb", "mergeSPUpdates(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, bool)"},
    {"_Z12emitSPUpdateRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEjxbRKNS_15TargetInstrInfoERKNS_18TargetRegisterInfoE", "emitSPUpdate(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, long long, bool, llvm::TargetInstrInfo const&, llvm::TargetRegisterInfo const&)"},
    {"_ZNK4llvm16X86FrameLowering12emitEpilogueERNS_15MachineFunctionERNS_17MachineBasicBlockE", "llvm::X86FrameLowering::emitEpilogue(llvm::MachineFunction&, llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm16X86FrameLowering20getInitialFrameStateERSt6vectorINS_11MachineMoveESaIS2_EE", "llvm::X86FrameLowering::getInitialFrameState(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> >&) const"},
    {"_ZNK4llvm16X86FrameLowering19getFrameIndexOffsetERKNS_15MachineFunctionEi", "llvm::X86FrameLowering::getFrameIndexOffset(llvm::MachineFunction const&, int) const"},
    {"_ZNK4llvm16X86FrameLowering25spillCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::X86FrameLowering::spillCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm16X86FrameLowering27restoreCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::X86FrameLowering::restoreCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm16X86FrameLowering36processFunctionBeforeCalleeSavedScanERNS_15MachineFunctionEPNS_12RegScavengerE", "llvm::X86FrameLowering::processFunctionBeforeCalleeSavedScan(llvm::MachineFunction&, llvm::RegScavenger*) const"},
    {"_ZNK4llvm12X86Subtarget13isTargetWin32Ev", "llvm::X86Subtarget::isTargetWin32() const"},
    {"_ZNSt6vectorIN4llvm11MachineMoveESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineMove*, std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> > >, llvm::MachineMove const&)"},
    {"_ZN4llvm8SmallSetIjLj8EE6insertERKj", "llvm::SmallSet<unsigned int, 8u>::insert(unsigned int const&)"},
    {"_ZN4llvm16createX86ISelDagERNS_16X86TargetMachineENS_10CodeGenOpt5LevelE", "llvm::createX86ISelDag(llvm::X86TargetMachine&, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISelD1Ev", "(anonymous namespace)::X86DAGToDAGISel::~X86DAGToDAGISel()"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISelD0Ev", "(anonymous namespace)::X86DAGToDAGISel::~X86DAGToDAGISel()"},
    {"_ZNK12_GLOBAL__N_115X86DAGToDAGISel11getPassNameEv", "(anonymous namespace)::X86DAGToDAGISel::getPassName() const"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel21EmitFunctionEntryCodeEv", "(anonymous namespace)::X86DAGToDAGISel::EmitFunctionEntryCode()"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel17PreprocessISelDAGEv", "(anonymous namespace)::X86DAGToDAGISel::PreprocessISelDAG()"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel6SelectEPN4llvm6SDNodeE", "(anonymous namespace)::X86DAGToDAGISel::Select(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel28SelectInlineAsmMemoryOperandERKN4llvm7SDValueEcRSt6vectorIS2_SaIS2_EE", "(anonymous namespace)::X86DAGToDAGISel::SelectInlineAsmMemoryOperand(llvm::SDValue const&, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&)"},
    {"_ZNK12_GLOBAL__N_115X86DAGToDAGISel18IsProfitableToFoldEN4llvm7SDValueEPNS1_6SDNodeES4_", "(anonymous namespace)::X86DAGToDAGISel::IsProfitableToFold(llvm::SDValue, llvm::SDNode*, llvm::SDNode*) const"},
    {"_ZNK12_GLOBAL__N_115X86DAGToDAGISel21CheckPatternPredicateEj", "(anonymous namespace)::X86DAGToDAGISel::CheckPatternPredicate(unsigned int) const"},
    {"_ZNK12_GLOBAL__N_115X86DAGToDAGISel18CheckNodePredicateEPN4llvm6SDNodeEj", "(anonymous namespace)::X86DAGToDAGISel::CheckNodePredicate(llvm::SDNode*, unsigned int) const"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel19CheckComplexPatternEPN4llvm6SDNodeES3_NS1_7SDValueEjRNS1_15SmallVectorImplISt4pairIS4_S3_EEE", "(anonymous namespace)::X86DAGToDAGISel::CheckComplexPattern(llvm::SDNode*, llvm::SDNode*, llvm::SDValue, unsigned int, llvm::SmallVectorImpl<std::pair<llvm::SDValue, llvm::SDNode*> >&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel14RunSDNodeXFormEN4llvm7SDValueEj", "(anonymous namespace)::X86DAGToDAGISel::RunSDNodeXForm(llvm::SDValue, unsigned int)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel10SelectAddrEPN4llvm6SDNodeENS1_7SDValueERS4_S5_S5_S5_S5_", "(anonymous namespace)::X86DAGToDAGISel::SelectAddr(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel19SelectScalarSSELoadEPN4llvm6SDNodeENS1_7SDValueERS4_S5_S5_S5_S5_S5_", "(anonymous namespace)::X86DAGToDAGISel::SelectScalarSSELoad(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel13SelectLEAAddrEN4llvm7SDValueERS2_S3_S3_S3_S3_", "(anonymous namespace)::X86DAGToDAGISel::SelectLEAAddr(llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel17SelectTLSADDRAddrEN4llvm7SDValueERS2_S3_S3_S3_S3_", "(anonymous namespace)::X86DAGToDAGISel::SelectTLSADDRAddr(llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel18getAddressOperandsERNS_18X86ISelAddressModeERN4llvm7SDValueES5_S5_S5_S5_", "(anonymous namespace)::X86DAGToDAGISel::getAddressOperands((anonymous namespace)::X86ISelAddressMode&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel12MatchAddressEN4llvm7SDValueERNS_18X86ISelAddressModeE", "(anonymous namespace)::X86DAGToDAGISel::MatchAddress(llvm::SDValue, (anonymous namespace)::X86ISelAddressMode&)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel23MatchAddressRecursivelyEN4llvm7SDValueERNS_18X86ISelAddressModeEj", "(anonymous namespace)::X86DAGToDAGISel::MatchAddressRecursively(llvm::SDValue, (anonymous namespace)::X86ISelAddressMode&, unsigned int)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel14SelectAtomic64EPN4llvm6SDNodeEj", "(anonymous namespace)::X86DAGToDAGISel::SelectAtomic64(llvm::SDNode*, unsigned int)"},
    {"_ZN12_GLOBAL__N_115X86DAGToDAGISel11TryFoldLoadEPN4llvm6SDNodeENS1_7SDValueERS4_S5_S5_S5_S5_", "(anonymous namespace)::X86DAGToDAGISel::TryFoldLoad(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm17X86TargetLoweringC1ERNS_16X86TargetMachineE", "llvm::X86TargetLowering::X86TargetLowering(llvm::X86TargetMachine&)"},
    {"_ZN4llvm17X86TargetLoweringC2ERNS_16X86TargetMachineE", "llvm::X86TargetLowering::X86TargetLowering(llvm::X86TargetMachine&)"},
    {"_ZNK4llvm17X86TargetLowering18getSetCCResultTypeENS_3EVTE", "llvm::X86TargetLowering::getSetCCResultType(llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering21getByValTypeAlignmentEPKNS_4TypeE", "llvm::X86TargetLowering::getByValTypeAlignment(llvm::Type const*) const"},
    {"_Z16getMaxByValAlignPKN4llvm4TypeERj", "getMaxByValAlign(llvm::Type const*, unsigned int&)"},
    {"_ZNK4llvm17X86TargetLowering19getOptimalMemOpTypeEyjjbbRNS_15MachineFunctionE", "llvm::X86TargetLowering::getOptimalMemOpType(unsigned long long, unsigned int, unsigned int, bool, bool, llvm::MachineFunction&) const"},
    {"_ZNK4llvm17X86TargetLowering20getJumpTableEncodingEv", "llvm::X86TargetLowering::getJumpTableEncoding() const"},
    {"_ZNK4llvm17X86TargetLowering25LowerCustomJumpTableEntryEPKNS_20MachineJumpTableInfoEPKNS_17MachineBasicBlockEjRNS_9MCContextE", "llvm::X86TargetLowering::LowerCustomJumpTableEntry(llvm::MachineJumpTableInfo const*, llvm::MachineBasicBlock const*, unsigned int, llvm::MCContext&) const"},
    {"_ZNK4llvm17X86TargetLowering24getPICJumpTableRelocBaseENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::getPICJumpTableRelocBase(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering28getPICJumpTableRelocBaseExprEPKNS_15MachineFunctionEjRNS_9MCContextE", "llvm::X86TargetLowering::getPICJumpTableRelocBaseExpr(llvm::MachineFunction const*, unsigned int, llvm::MCContext&) const"},
    {"_ZNK4llvm17X86TargetLowering20getFunctionAlignmentEPKNS_8FunctionE", "llvm::X86TargetLowering::getFunctionAlignment(llvm::Function const*) const"},
    {"_ZNK4llvm17X86TargetLowering23findRepresentativeClassENS_3EVTE", "llvm::X86TargetLowering::findRepresentativeClass(llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering19getRegPressureLimitEPKNS_19TargetRegisterClassERNS_15MachineFunctionE", "llvm::X86TargetLowering::getRegPressureLimit(llvm::TargetRegisterClass const*, llvm::MachineFunction&) const"},
    {"_ZNK4llvm17X86TargetLowering22getStackCookieLocationERjS1_", "llvm::X86TargetLowering::getStackCookieLocation(unsigned int&, unsigned int&) const"},
    {"_ZNK4llvm17X86TargetLowering14CanLowerReturnENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERNS_11LLVMContextE", "llvm::X86TargetLowering::CanLowerReturn(llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::LLVMContext&) const"},
    {"_Z9RetCC_X86jN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZNK4llvm17X86TargetLowering11LowerReturnENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS4_IS1_EENS_8DebugLocERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerReturn(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::DebugLoc, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering18isUsedByReturnOnlyEPNS_6SDNodeE", "llvm::X86TargetLowering::isUsedByReturnOnly(llvm::SDNode*) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerCallResultENS_7SDValueES1_NS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EE", "llvm::X86TargetLowering::LowerCallResult(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm17X86TargetLowering16LowerMemArgumentENS_7SDValueENS_11CallingConv2IDERKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERKNS_11CCValAssignEPNS_16MachineFrameInfoEj", "llvm::X86TargetLowering::LowerMemArgument(llvm::SDValue, llvm::CallingConv::ID, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::CCValAssign const&, llvm::MachineFrameInfo*, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EE", "llvm::X86TargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_Z6CC_X86jN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_X86(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZNK4llvm17X86TargetLowering27GetAlignedArgumentStackSizeEjRNS_12SelectionDAGE", "llvm::X86TargetLowering::GetAlignedArgumentStackSize(unsigned int, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering16LowerMemOpCallToENS_7SDValueES1_S1_NS_8DebugLocERNS_12SelectionDAGERKNS_11CCValAssignENS_3ISD10ArgFlagsTyE", "llvm::X86TargetLowering::LowerMemOpCallTo(llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::DebugLoc, llvm::SelectionDAG&, llvm::CCValAssign const&, llvm::ISD::ArgFlagsTy) const"},
    {"_ZNK4llvm17X86TargetLowering23EmitTailCallLoadRetAddrERNS_12SelectionDAGERNS_7SDValueES3_bbiNS_8DebugLocE", "llvm::X86TargetLowering::EmitTailCallLoadRetAddr(llvm::SelectionDAG&, llvm::SDValue&, llvm::SDValue, bool, bool, int, llvm::DebugLoc) const"},
    {"_ZNK4llvm17X86TargetLowering26getReturnAddressFrameIndexERNS_12SelectionDAGE", "llvm::X86TargetLowering::getReturnAddressFrameIndex(llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerCallENS_7SDValueES1_NS_11CallingConv2IDEbRbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS5_IS1_EERKNS5_INS6_8InputArgEEENS_8DebugLocERNS_12SelectionDAGERSB_", "llvm::X86TargetLowering::LowerCall(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, bool&, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm17X86TargetLowering33IsEligibleForTailCallOptimizationENS_7SDValueENS_11CallingConv2IDEbbbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS4_IS1_EERKNS4_INS5_8InputArgEEERNS_12SelectionDAGE", "llvm::X86TargetLowering::IsEligibleForTailCallOptimization(llvm::SDValue, llvm::CallingConv::ID, bool, bool, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::SelectionDAG&) const"},
    {"_Z7getMOVLRN4llvm12SelectionDAGENS_8DebugLocENS_3EVTENS_7SDValueES4_", "getMOVL(llvm::SelectionDAG&, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZNK4llvm17X86TargetLowering18LowerGlobalAddressENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerGlobalAddress(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering19LowerExternalSymbolENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerExternalSymbol(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14createFastISelERNS_20FunctionLoweringInfoE", "llvm::X86TargetLowering::createFastISel(llvm::FunctionLoweringInfo&) const"},
    {"_ZN4llvm3X8628isOffsetSuitableForCodeModelExNS_9CodeModel5ModelEb", "llvm::X86::isOffsetSuitableForCodeModel(long long, llvm::CodeModel::Model, bool)"},
    {"_ZNK4llvm17X86TargetLowering12isFPImmLegalERKNS_7APFloatENS_3EVTE", "llvm::X86TargetLowering::isFPImmLegal(llvm::APFloat const&, llvm::EVT) const"},
    {"_ZN4llvm3X8612isPSHUFDMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isPSHUFDMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isPSHUFHWMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isPSHUFHWMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isPSHUFLWMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isPSHUFLWMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isPALIGNRMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isPALIGNRMask(llvm::ShuffleVectorSDNode*)"},
    {"_Z13isPALIGNRMaskRKN4llvm15SmallVectorImplIiEENS_3EVTEb", "isPALIGNRMask(llvm::SmallVectorImpl<int> const&, llvm::EVT, bool)"},
    {"_ZN4llvm3X8611isSHUFPMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isSHUFPMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isMOVHLPSMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVHLPSMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8622isMOVHLPS_v_undef_MaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVHLPS_v_undef_Mask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8611isMOVLPMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVLPMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isMOVLHPSMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVLHPSMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8612isUNPCKLMaskEPNS_19ShuffleVectorSDNodeEb", "llvm::X86::isUNPCKLMask(llvm::ShuffleVectorSDNode*, bool)"},
    {"_ZN4llvm3X8612isUNPCKHMaskEPNS_19ShuffleVectorSDNodeEb", "llvm::X86::isUNPCKHMask(llvm::ShuffleVectorSDNode*, bool)"},
    {"_ZN4llvm3X8621isUNPCKL_v_undef_MaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isUNPCKL_v_undef_Mask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8621isUNPCKH_v_undef_MaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isUNPCKH_v_undef_Mask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8610isMOVLMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVLMask(llvm::ShuffleVectorSDNode*)"},
    {"_Z10isMOVLMaskRKN4llvm15SmallVectorImplIiEENS_3EVTE", "isMOVLMask(llvm::SmallVectorImpl<int> const&, llvm::EVT)"},
    {"_ZN4llvm3X8614isMOVSHDUPMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVSHDUPMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8614isMOVSLDUPMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVSLDUPMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8613isMOVDDUPMaskEPNS_19ShuffleVectorSDNodeE", "llvm::X86::isMOVDDUPMask(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm3X8619isVEXTRACTF128IndexEPNS_6SDNodeE", "llvm::X86::isVEXTRACTF128Index(llvm::SDNode*)"},
    {"_ZN4llvm3X8618isVINSERTF128IndexEPNS_6SDNodeE", "llvm::X86::isVINSERTF128Index(llvm::SDNode*)"},
    {"_ZN4llvm3X8623getShuffleSHUFImmediateEPNS_6SDNodeE", "llvm::X86::getShuffleSHUFImmediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8626getShufflePSHUFHWImmediateEPNS_6SDNodeE", "llvm::X86::getShufflePSHUFHWImmediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8626getShufflePSHUFLWImmediateEPNS_6SDNodeE", "llvm::X86::getShufflePSHUFLWImmediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8626getShufflePALIGNRImmediateEPNS_6SDNodeE", "llvm::X86::getShufflePALIGNRImmediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8631getExtractVEXTRACTF128ImmediateEPNS_6SDNodeE", "llvm::X86::getExtractVEXTRACTF128Immediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8629getInsertVINSERTF128ImmediateEPNS_6SDNodeE", "llvm::X86::getInsertVINSERTF128Immediate(llvm::SDNode*)"},
    {"_ZN4llvm3X8610isZeroNodeENS_7SDValueE", "llvm::X86::isZeroNode(llvm::SDValue)"},
    {"_Z19getShuffleScalarEltPN4llvm6SDNodeEiRNS_12SelectionDAGEj", "getShuffleScalarElt(llvm::SDNode*, int, llvm::SelectionDAG&, unsigned int)"},
    {"_ZNK4llvm17X86TargetLowering22LowerAsSplatVectorLoadENS_7SDValueENS_3EVTENS_8DebugLocERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerAsSplatVectorLoad(llvm::SDValue, llvm::EVT, llvm::DebugLoc, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering17LowerBUILD_VECTORENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerBUILD_VECTOR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z13getZeroVectorN4llvm3EVTEbRNS_12SelectionDAGENS_8DebugLocE", "getZeroVector(llvm::EVT, bool, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_Z27getShuffleVectorZeroOrUndefN4llvm7SDValueEjbbRNS_12SelectionDAGE", "getShuffleVectorZeroOrUndef(llvm::SDValue, unsigned int, bool, bool, llvm::SelectionDAG&)"},
    {"_Z10getUnpacklRN4llvm12SelectionDAGENS_8DebugLocENS_3EVTENS_7SDValueES4_", "getUnpackl(llvm::SelectionDAG&, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_Z24EltsFromConsecutiveLoadsN4llvm3EVTERNS_15SmallVectorImplINS_7SDValueEEERNS_8DebugLocERNS_12SelectionDAGE", "EltsFromConsecutiveLoads(llvm::EVT, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::DebugLoc&, llvm::SelectionDAG&)"},
    {"_ZNK4llvm17X86TargetLowering19LowerCONCAT_VECTORSENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerCONCAT_VECTORS(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering24LowerVECTOR_SHUFFLEv8i16ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVECTOR_SHUFFLEv8i16(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering19LowerVECTOR_SHUFFLEENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVECTOR_SHUFFLE(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z12getVZextMovLN4llvm3EVTES0_NS_7SDValueERNS_12SelectionDAGEPKNS_12X86SubtargetENS_8DebugLocE", "getVZextMovL(llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SelectionDAG&, llvm::X86Subtarget const*, llvm::DebugLoc)"},
    {"_Z20CommuteVectorShufflePN4llvm19ShuffleVectorSDNodeERNS_12SelectionDAGE", "CommuteVectorShuffle(llvm::ShuffleVectorSDNode*, llvm::SelectionDAG&)"},
    {"_ZNK4llvm17X86TargetLowering28LowerEXTRACT_VECTOR_ELT_SSE4ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerEXTRACT_VECTOR_ELT_SSE4(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering23LowerEXTRACT_VECTOR_ELTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerEXTRACT_VECTOR_ELT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering27LowerINSERT_VECTOR_ELT_SSE4ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerINSERT_VECTOR_ELT_SSE4(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering22LowerINSERT_VECTOR_ELTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerINSERT_VECTOR_ELT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z19Extract128BitVectorN4llvm7SDValueES0_RNS_12SelectionDAGENS_8DebugLocE", "Extract128BitVector(llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_Z18Insert128BitVectorN4llvm7SDValueES0_S0_RNS_12SelectionDAGENS_8DebugLocE", "Insert128BitVector(llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_ZNK4llvm17X86TargetLowering21LowerSCALAR_TO_VECTORENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerSCALAR_TO_VECTOR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering22LowerEXTRACT_SUBVECTORENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerEXTRACT_SUBVECTOR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering21LowerINSERT_SUBVECTORENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerINSERT_SUBVECTOR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering17LowerConstantPoolENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerConstantPool(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerJumpTableENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerJumpTable(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering17LowerBlockAddressENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerBlockAddress(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering18LowerGlobalAddressEPKNS_11GlobalValueENS_8DebugLocExRNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerGlobalAddress(llvm::GlobalValue const*, llvm::DebugLoc, long long, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering21LowerGlobalTLSAddressENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerGlobalTLSAddress(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering10LowerShiftENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerShift(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerSINT_TO_FPENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerSINT_TO_FP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9BuildFILDENS_7SDValueENS_3EVTES1_S1_RNS_12SelectionDAGE", "llvm::X86TargetLowering::BuildFILD(llvm::SDValue, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering19LowerUINT_TO_FP_i64ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerUINT_TO_FP_i64(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering19LowerUINT_TO_FP_i32ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerUINT_TO_FP_i32(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerUINT_TO_FPENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerUINT_TO_FP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15FP_TO_INTHelperENS_7SDValueERNS_12SelectionDAGEb", "llvm::X86TargetLowering::FP_TO_INTHelper(llvm::SDValue, llvm::SelectionDAG&, bool) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerFP_TO_SINTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFP_TO_SINT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerFP_TO_UINTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFP_TO_UINT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerFABSENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFABS(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerFNEGENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFNEG(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerFCOPYSIGNENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFCOPYSIGN(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering8EmitTestENS_7SDValueEjRNS_12SelectionDAGE", "llvm::X86TargetLowering::EmitTest(llvm::SDValue, unsigned int, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering7EmitCmpENS_7SDValueES1_jRNS_12SelectionDAGE", "llvm::X86TargetLowering::EmitCmp(llvm::SDValue, llvm::SDValue, unsigned int, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerToBTENS_7SDValueENS_3ISD8CondCodeENS_8DebugLocERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerToBT(llvm::SDValue, llvm::ISD::CondCode, llvm::DebugLoc, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering10LowerSETCCENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerSETCC(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z14TranslateX86CCN4llvm3ISD8CondCodeEbRNS_7SDValueES3_RNS_12SelectionDAGE", "TranslateX86CC(llvm::ISD::CondCode, bool, llvm::SDValue&, llvm::SDValue&, llvm::SelectionDAG&)"},
    {"_ZNK4llvm17X86TargetLowering11LowerVSETCCENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVSETCC(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering11LowerSELECTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerSELECT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering11LowerBRCONDENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerBRCOND(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering23LowerDYNAMIC_STACKALLOCENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerDYNAMIC_STACKALLOC(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering12LowerVASTARTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVASTART(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering10LowerVAARGENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVAARG(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering11LowerVACOPYENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerVACOPY(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering23LowerINTRINSIC_WO_CHAINENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerINTRINSIC_WO_CHAIN(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerRETURNADDRENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerRETURNADDR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerFRAMEADDRENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFRAMEADDR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering25LowerFRAME_TO_ARGS_OFFSETENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFRAME_TO_ARGS_OFFSET(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerEH_RETURNENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerEH_RETURN(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerTRAMPOLINEENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerTRAMPOLINE(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering16LowerFLT_ROUNDS_ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerFLT_ROUNDS_(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerCTLZENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerCTLZ(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering9LowerCTTZENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerCTTZ(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerMUL_V2I64ENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerMUL_V2I64(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering8LowerSHLENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerSHL(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering10LowerXALUOENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerXALUO(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering15LowerMEMBARRIERENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerMEMBARRIER(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering13LowerCMP_SWAPENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerCMP_SWAP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering21LowerREADCYCLECOUNTERENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerREADCYCLECOUNTER(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering12LowerBITCASTENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerBITCAST(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering13LowerLOAD_SUBENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerLOAD_SUB(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering14LowerOperationENS_7SDValueERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerOperation(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering23ReplaceATOMIC_BINARY_64EPNS_6SDNodeERNS_15SmallVectorImplINS_7SDValueEEERNS_12SelectionDAGEj", "llvm::X86TargetLowering::ReplaceATOMIC_BINARY_64(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::SelectionDAG&, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering18ReplaceNodeResultsEPNS_6SDNodeERNS_15SmallVectorImplINS_7SDValueEEERNS_12SelectionDAGE", "llvm::X86TargetLowering::ReplaceNodeResults(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering17getTargetNodeNameEj", "llvm::X86TargetLowering::getTargetNodeName(unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering21isLegalAddressingModeERKNS_14TargetLowering8AddrModeEPKNS_4TypeE", "llvm::X86TargetLowering::isLegalAddressingMode(llvm::TargetLowering::AddrMode const&, llvm::Type const*) const"},
    {"_ZNK4llvm17X86TargetLowering14isTruncateFreeEPKNS_4TypeES3_", "llvm::X86TargetLowering::isTruncateFree(llvm::Type const*, llvm::Type const*) const"},
    {"_ZNK4llvm17X86TargetLowering14isTruncateFreeENS_3EVTES1_", "llvm::X86TargetLowering::isTruncateFree(llvm::EVT, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering10isZExtFreeEPKNS_4TypeES3_", "llvm::X86TargetLowering::isZExtFree(llvm::Type const*, llvm::Type const*) const"},
    {"_ZNK4llvm17X86TargetLowering10isZExtFreeENS_3EVTES1_", "llvm::X86TargetLowering::isZExtFree(llvm::EVT, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering21isNarrowingProfitableENS_3EVTES1_", "llvm::X86TargetLowering::isNarrowingProfitable(llvm::EVT, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering18isShuffleMaskLegalERKNS_15SmallVectorImplIiEENS_3EVTE", "llvm::X86TargetLowering::isShuffleMaskLegal(llvm::SmallVectorImpl<int> const&, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering22isVectorClearMaskLegalERKNS_15SmallVectorImplIiEENS_3EVTE", "llvm::X86TargetLowering::isVectorClearMaskLegal(llvm::SmallVectorImpl<int> const&, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering35EmitAtomicBitwiseWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockEjjjjjjPNS_19TargetRegisterClassEb", "llvm::X86TargetLowering::EmitAtomicBitwiseWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, llvm::TargetRegisterClass*, bool) const"},
    {"_ZNK4llvm17X86TargetLowering35EmitAtomicBit6432WithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockEjjjjb", "llvm::X86TargetLowering::EmitAtomicBit6432WithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int, unsigned int, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm17X86TargetLowering34EmitAtomicMinMaxWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockEj", "llvm::X86TargetLowering::EmitAtomicMinMaxWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering8EmitPCMPEPNS_12MachineInstrEPNS_17MachineBasicBlockEjb", "llvm::X86TargetLowering::EmitPCMP(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int, bool) const"},
    {"_ZNK4llvm17X86TargetLowering11EmitMonitorEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitMonitor(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering9EmitMwaitEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitMwait(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering29EmitVAARG64WithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitVAARG64WithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering40EmitVAStartSaveXMMRegsWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitVAStartSaveXMMRegsWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering17EmitLoweredSelectEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitLoweredSelect(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering20EmitLoweredWinAllocaEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitLoweredWinAlloca(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering18EmitLoweredTLSCallEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitLoweredTLSCall(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17X86TargetLowering27EmitInstrWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::X86TargetLowering::EmitInstrWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm17addFrameReferenceERKNS_19MachineInstrBuilderEii", "llvm::addFrameReference(llvm::MachineInstrBuilder const&, int, int)"},
    {"_ZNK4llvm17X86TargetLowering30computeMaskedBitsForTargetNodeENS_7SDValueERKNS_5APIntERS2_S5_RKNS_12SelectionDAGEj", "llvm::X86TargetLowering::computeMaskedBitsForTargetNode(llvm::SDValue, llvm::APInt const&, llvm::APInt&, llvm::APInt&, llvm::SelectionDAG const&, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering31ComputeNumSignBitsForTargetNodeENS_7SDValueEj", "llvm::X86TargetLowering::ComputeNumSignBitsForTargetNode(llvm::SDValue, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering14isGAPlusOffsetEPNS_6SDNodeERPKNS_11GlobalValueERx", "llvm::X86TargetLowering::isGAPlusOffset(llvm::SDNode*, llvm::GlobalValue const*&, long long&) const"},
    {"_ZNK4llvm17X86TargetLowering17PerformDAGCombineEPNS_6SDNodeERNS_14TargetLowering15DAGCombinerInfoE", "llvm::X86TargetLowering::PerformDAGCombine(llvm::SDNode*, llvm::TargetLowering::DAGCombinerInfo&) const"},
    {"_ZNK4llvm17X86TargetLowering20isTypeDesirableForOpEjNS_3EVTE", "llvm::X86TargetLowering::isTypeDesirableForOp(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering22IsDesirableToPromoteOpENS_7SDValueERNS_3EVTE", "llvm::X86TargetLowering::IsDesirableToPromoteOp(llvm::SDValue, llvm::EVT&) const"},
    {"_ZNK4llvm17X86TargetLowering15ExpandInlineAsmEPNS_8CallInstE", "llvm::X86TargetLowering::ExpandInlineAsm(llvm::CallInst*) const"},
    {"_ZNK4llvm17X86TargetLowering17getConstraintTypeERKSs", "llvm::X86TargetLowering::getConstraintType(std::string const&) const"},
    {"_ZNK4llvm17X86TargetLowering30getSingleConstraintMatchWeightERNS_14TargetLowering14AsmOperandInfoEPKc", "llvm::X86TargetLowering::getSingleConstraintMatchWeight(llvm::TargetLowering::AsmOperandInfo&, char const*) const"},
    {"_ZNK4llvm17X86TargetLowering16LowerXConstraintENS_3EVTE", "llvm::X86TargetLowering::LowerXConstraint(llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering28LowerAsmOperandForConstraintENS_7SDValueEcRSt6vectorIS1_SaIS1_EERNS_12SelectionDAGE", "llvm::X86TargetLowering::LowerAsmOperandForConstraint(llvm::SDValue, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17X86TargetLowering33getRegClassForInlineAsmConstraintERKSsNS_3EVTE", "llvm::X86TargetLowering::getRegClassForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering28getRegForInlineAsmConstraintERKSsNS_3EVTE", "llvm::X86TargetLowering::getRegForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_Z10GetTLSADDRRN4llvm12SelectionDAGENS_7SDValueEPNS_19GlobalAddressSDNodeEPS2_NS_3EVTEjh", "GetTLSADDR(llvm::SelectionDAG&, llvm::SDValue, llvm::GlobalAddressSDNode*, llvm::SDValue*, llvm::EVT, unsigned int, unsigned char)"},
    {"_Z17MayFoldVectorLoadN4llvm7SDValueE", "MayFoldVectorLoad(llvm::SDValue)"},
    {"_Z24RewriteAsNarrowerShufflePN4llvm19ShuffleVectorSDNodeERNS_12SelectionDAGENS_8DebugLocE", "RewriteAsNarrowerShuffle(llvm::ShuffleVectorSDNode*, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_Z16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm15SmallVectorImplINS_7SDValueEE6resizeEj", "llvm::SmallVectorImpl<llvm::SDValue>::resize(unsigned int)"},
    {"_ZN4llvm8SmallSetINS_7SDValueELj8EE6insertERKS1_", "llvm::SmallSet<llvm::SDValue, 8u>::insert(llvm::SDValue const&)"},
    {"_ZN4llvm5APInt10getBitsSetEjjj", "llvm::APInt::getBitsSet(unsigned int, unsigned int, unsigned int)"},
    {"_ZNK4llvm3EVT6bitsLTES0_", "llvm::EVT::bitsLT(llvm::EVT) const"},
    {"_ZNK4llvm19MachineInstrBuilder7addDispERKNS_14MachineOperandEx", "llvm::MachineInstrBuilder::addDisp(llvm::MachineOperand const&, long long) const"},
    {"_ZN4llvm5APInt14getHighBitsSetEjj", "llvm::APInt::getHighBitsSet(unsigned int, unsigned int)"},
    {"_ZN4llvm11SmallVectorIjLj4EEC2EjRKj", "llvm::SmallVector<unsigned int, 4u>::SmallVector(unsigned int, unsigned int const&)"},
    {"_ZNSt8_Rb_treeIN4llvm7SDValueES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE16_M_insert_uniqueERKS1_", "std::_Rb_tree<llvm::SDValue, llvm::SDValue, std::_Identity<llvm::SDValue>, std::less<llvm::SDValue>, std::allocator<llvm::SDValue> >::_M_insert_unique(llvm::SDValue const&)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm14ConstraintInfoESaIS2_EED2Ev", "std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >::~vector()"},
    {"_ZNSt6vectorIN4llvm9InlineAsm17SubConstraintInfoESaIS2_EED2Ev", "std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> >::~vector()"},
    {"_ZN4llvm5APInt13getLowBitsSetEjj", "llvm::APInt::getLowBitsSet(unsigned int, unsigned int)"},
    {"_ZNK4llvm5APIntplEy", "llvm::APInt::operator+(unsigned long long) const"},
    {"_ZN4llvm5APInt17getSignedMinValueEj", "llvm::APInt::getSignedMinValue(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIiiEE6resizeEj", "llvm::SmallVectorImpl<std::pair<int, int> >::resize(unsigned int)"},
    {"_ZN4llvm11SmallVectorIiLj8EEC2EjRKi", "llvm::SmallVector<int, 8u>::SmallVector(unsigned int, int const&)"},
    {"_ZNSt8_Rb_treeIN4llvm7SDValueES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE8_M_eraseEPSt13_Rb_tree_nodeIS1_E", "std::_Rb_tree<llvm::SDValue, llvm::SDValue, std::_Identity<llvm::SDValue>, std::less<llvm::SDValue>, std::allocator<llvm::SDValue> >::_M_erase(std::_Rb_tree_node<llvm::SDValue>*)"},
    {"_ZNSt6vectorIN4llvm7APFloatESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::APFloat, std::allocator<llvm::APFloat> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::APFloat*, std::vector<llvm::APFloat, std::allocator<llvm::APFloat> > >, llvm::APFloat const&)"},
    {"_ZN4llvm28TargetLoweringObjectFileCOFFD1Ev", "llvm::TargetLoweringObjectFileCOFF::~TargetLoweringObjectFileCOFF()"},
    {"_ZN4llvm28TargetLoweringObjectFileCOFFD0Ev", "llvm::TargetLoweringObjectFileCOFF::~TargetLoweringObjectFileCOFF()"},
    {"_ZNK4llvm28TargetLoweringObjectFileCOFF17getDrectveSectionEv", "llvm::TargetLoweringObjectFileCOFF::getDrectveSection() const"},
    {"_ZN4llvm12X86InstrInfoC1ERNS_16X86TargetMachineE", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)"},
    {"_ZN4llvm12X86InstrInfoC2ERNS_16X86TargetMachineE", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)"},
    {"_ZNK4llvm12X86InstrInfo21isCoalescableExtInstrERKNS_12MachineInstrERjS4_S4_", "llvm::X86InstrInfo::isCoalescableExtInstr(llvm::MachineInstr const&, unsigned int&, unsigned int&, unsigned int&) const"},
    {"_ZNK4llvm12X86InstrInfo14isFrameOperandEPKNS_12MachineInstrEjRi", "llvm::X86InstrInfo::isFrameOperand(llvm::MachineInstr const*, unsigned int, int&) const"},
    {"_ZNK4llvm12X86InstrInfo19isLoadFromStackSlotEPKNS_12MachineInstrERi", "llvm::X86InstrInfo::isLoadFromStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm12X86InstrInfo25isLoadFromStackSlotPostFEEPKNS_12MachineInstrERi", "llvm::X86InstrInfo::isLoadFromStackSlotPostFE(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm12X86InstrInfo20hasLoadFromStackSlotEPKNS_12MachineInstrERPKNS_17MachineMemOperandERi", "llvm::X86InstrInfo::hasLoadFromStackSlot(llvm::MachineInstr const*, llvm::MachineMemOperand const*&, int&) const"},
    {"_ZNK4llvm12X86InstrInfo18isStoreToStackSlotEPKNS_12MachineInstrERi", "llvm::X86InstrInfo::isStoreToStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm12X86InstrInfo24isStoreToStackSlotPostFEEPKNS_12MachineInstrERi", "llvm::X86InstrInfo::isStoreToStackSlotPostFE(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm12X86InstrInfo19hasStoreToStackSlotEPKNS_12MachineInstrERPKNS_17MachineMemOperandERi", "llvm::X86InstrInfo::hasStoreToStackSlot(llvm::MachineInstr const*, llvm::MachineMemOperand const*&, int&) const"},
    {"_ZNK4llvm12X86InstrInfo33isReallyTriviallyReMaterializableEPKNS_12MachineInstrEPNS_13AliasAnalysisE", "llvm::X86InstrInfo::isReallyTriviallyReMaterializable(llvm::MachineInstr const*, llvm::AliasAnalysis*) const"},
    {"_ZNK4llvm12X86InstrInfo13reMaterializeERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjjPKS4_RKNS_18TargetRegisterInfoE", "llvm::X86InstrInfo::reMaterialize(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, unsigned int, llvm::MachineInstr const*, llvm::TargetRegisterInfo const&) const"},
    {"_ZNK4llvm12X86InstrInfo28convertToThreeAddressWithLEAEjRNS_14ilist_iteratorINS_17MachineBasicBlockEEERNS1_INS_12MachineInstrEEEPNS_13LiveVariablesE", "llvm::X86InstrInfo::convertToThreeAddressWithLEA(unsigned int, llvm::ilist_iterator<llvm::MachineBasicBlock>&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::LiveVariables*) const"},
    {"_ZNK4llvm12X86InstrInfo21convertToThreeAddressERNS_14ilist_iteratorINS_17MachineBasicBlockEEERNS1_INS_12MachineInstrEEEPNS_13LiveVariablesE", "llvm::X86InstrInfo::convertToThreeAddress(llvm::ilist_iterator<llvm::MachineBasicBlock>&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::LiveVariables*) const"},
    {"_ZNK4llvm12X86InstrInfo18commuteInstructionEPNS_12MachineInstrEb", "llvm::X86InstrInfo::commuteInstruction(llvm::MachineInstr*, bool) const"},
    {"_ZN4llvm3X8621GetCondBranchFromCondENS0_8CondCodeE", "llvm::X86::GetCondBranchFromCond(llvm::X86::CondCode)"},
    {"_ZN4llvm3X8626GetOppositeBranchConditionENS0_8CondCodeE", "llvm::X86::GetOppositeBranchCondition(llvm::X86::CondCode)"},
    {"_ZNK4llvm12X86InstrInfo24isUnpredicatedTerminatorEPKNS_12MachineInstrE", "llvm::X86InstrInfo::isUnpredicatedTerminator(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm12X86InstrInfo13analyzeBranchERNS_17MachineBasicBlockERPS1_S4_RNS_15SmallVectorImplINS_14MachineOperandEEEb", "llvm::X86InstrInfo::analyzeBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*&, llvm::MachineBasicBlock*&, llvm::SmallVectorImpl<llvm::MachineOperand>&, bool) const"},
    {"_ZNK4llvm12X86InstrInfo12removeBranchERNS_17MachineBasicBlockE", "llvm::X86InstrInfo::removeBranch(llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm12X86InstrInfo12insertBranchERNS_17MachineBasicBlockEPS1_S3_RKNS_15SmallVectorImplINS_14MachineOperandEEENS_8DebugLocE", "llvm::X86InstrInfo::insertBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::DebugLoc) const"},
    {"_ZNK4llvm12X86InstrInfo11copyPhysRegERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjb", "llvm::X86InstrInfo::copyPhysReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm12X86InstrInfo19storeRegToStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjbiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::X86InstrInfo::storeRegToStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, bool, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm17addFrameReferenceERKNS_19MachineInstrBuilderEii", "llvm::addFrameReference(llvm::MachineInstrBuilder const&, int, int)"},
    {"_ZNK4llvm12X86InstrInfo14storeRegToAddrERNS_15MachineFunctionEjbRNS_15SmallVectorImplINS_14MachineOperandEEEPKNS_19TargetRegisterClassEPPNS_17MachineMemOperandESC_RNS3_IPNS_12MachineInstrEEE", "llvm::X86InstrInfo::storeRegToAddr(llvm::MachineFunction&, unsigned int, bool, llvm::SmallVectorImpl<llvm::MachineOperand>&, llvm::TargetRegisterClass const*, llvm::MachineMemOperand**, llvm::MachineMemOperand**, llvm::SmallVectorImpl<llvm::MachineInstr*>&) const"},
    {"_ZNK4llvm12X86InstrInfo20loadRegFromStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::X86InstrInfo::loadRegFromStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm12X86InstrInfo15loadRegFromAddrERNS_15MachineFunctionEjRNS_15SmallVectorImplINS_14MachineOperandEEEPKNS_19TargetRegisterClassEPPNS_17MachineMemOperandESC_RNS3_IPNS_12MachineInstrEEE", "llvm::X86InstrInfo::loadRegFromAddr(llvm::MachineFunction&, unsigned int, llvm::SmallVectorImpl<llvm::MachineOperand>&, llvm::TargetRegisterClass const*, llvm::MachineMemOperand**, llvm::MachineMemOperand**, llvm::SmallVectorImpl<llvm::MachineInstr*>&) const"},
    {"_ZNK4llvm12X86InstrInfo24emitFrameIndexDebugValueERNS_15MachineFunctionEiyPKNS_6MDNodeENS_8DebugLocE", "llvm::X86InstrInfo::emitFrameIndexDebugValue(llvm::MachineFunction&, int, unsigned long long, llvm::MDNode const*, llvm::DebugLoc) const"},
    {"_ZNK4llvm12X86InstrInfo21foldMemoryOperandImplERNS_15MachineFunctionEPNS_12MachineInstrEjRKNS_15SmallVectorImplINS_14MachineOperandEEEjj", "llvm::X86InstrInfo::foldMemoryOperandImpl(llvm::MachineFunction&, llvm::MachineInstr*, unsigned int, llvm::SmallVectorImpl<llvm::MachineOperand> const&, unsigned int, unsigned int) const"},
    {"_Z10MakeM0InstRKN4llvm15TargetInstrInfoEjRKNS_15SmallVectorImplINS_14MachineOperandEEEPNS_12MachineInstrE", "MakeM0Inst(llvm::TargetInstrInfo const&, unsigned int, llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::MachineInstr*)"},
    {"_ZNK4llvm12X86InstrInfo21foldMemoryOperandImplERNS_15MachineFunctionEPNS_12MachineInstrERKNS_15SmallVectorImplIjEEi", "llvm::X86InstrInfo::foldMemoryOperandImpl(llvm::MachineFunction&, llvm::MachineInstr*, llvm::SmallVectorImpl<unsigned int> const&, int) const"},
    {"_ZNK4llvm12X86InstrInfo21foldMemoryOperandImplERNS_15MachineFunctionEPNS_12MachineInstrERKNS_15SmallVectorImplIjEES4_", "llvm::X86InstrInfo::foldMemoryOperandImpl(llvm::MachineFunction&, llvm::MachineInstr*, llvm::SmallVectorImpl<unsigned int> const&, llvm::MachineInstr*) const"},
    {"_ZNK4llvm12X86InstrInfo20canFoldMemoryOperandEPKNS_12MachineInstrERKNS_15SmallVectorImplIjEE", "llvm::X86InstrInfo::canFoldMemoryOperand(llvm::MachineInstr const*, llvm::SmallVectorImpl<unsigned int> const&) const"},
    {"_ZNK4llvm12X86InstrInfo19unfoldMemoryOperandERNS_15MachineFunctionEPNS_12MachineInstrEjbbRNS_15SmallVectorImplIS4_EE", "llvm::X86InstrInfo::unfoldMemoryOperand(llvm::MachineFunction&, llvm::MachineInstr*, unsigned int, bool, bool, llvm::SmallVectorImpl<llvm::MachineInstr*>&) const"},
    {"_ZNK4llvm12X86InstrInfo19unfoldMemoryOperandERNS_12SelectionDAGEPNS_6SDNodeERNS_15SmallVectorImplIS4_EE", "llvm::X86InstrInfo::unfoldMemoryOperand(llvm::SelectionDAG&, llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDNode*>&) const"},
    {"_ZNK4llvm12X86InstrInfo26getOpcodeAfterMemoryUnfoldEjbbPj", "llvm::X86InstrInfo::getOpcodeAfterMemoryUnfold(unsigned int, bool, bool, unsigned int*) const"},
    {"_ZNK4llvm12X86InstrInfo23areLoadsFromSameBasePtrEPNS_6SDNodeES2_RxS3_", "llvm::X86InstrInfo::areLoadsFromSameBasePtr(llvm::SDNode*, llvm::SDNode*, long long&, long long&) const"},
    {"_ZNK4llvm12X86InstrInfo23shouldScheduleLoadsNearEPNS_6SDNodeES2_xxj", "llvm::X86InstrInfo::shouldScheduleLoadsNear(llvm::SDNode*, llvm::SDNode*, long long, long long, unsigned int) const"},
    {"_ZNK4llvm12X86InstrInfo22ReverseBranchConditionERNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::X86InstrInfo::ReverseBranchCondition(llvm::SmallVectorImpl<llvm::MachineOperand>&) const"},
    {"_ZNK4llvm12X86InstrInfo24isSafeToMoveRegClassDefsEPKNS_19TargetRegisterClassE", "llvm::X86InstrInfo::isSafeToMoveRegClassDefs(llvm::TargetRegisterClass const*) const"},
    {"_ZN4llvm12X86InstrInfo19isX86_64ExtendedRegEj", "llvm::X86InstrInfo::isX86_64ExtendedReg(unsigned int)"},
    {"_ZNK4llvm12X86InstrInfo16getGlobalBaseRegEPNS_15MachineFunctionE", "llvm::X86InstrInfo::getGlobalBaseReg(llvm::MachineFunction*) const"},
    {"_ZNK4llvm12X86InstrInfo12GetSSEDomainEPKNS_12MachineInstrE", "llvm::X86InstrInfo::GetSSEDomain(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm12X86InstrInfo12SetSSEDomainEPNS_12MachineInstrEj", "llvm::X86InstrInfo::SetSSEDomain(llvm::MachineInstr*, unsigned int) const"},
    {"_ZNK4llvm12X86InstrInfo21getNoopForMachoTargetERNS_6MCInstE", "llvm::X86InstrInfo::getNoopForMachoTarget(llvm::MCInst&) const"},
    {"_ZNK4llvm12X86InstrInfo21hasHighOperandLatencyEPKNS_18InstrItineraryDataEPKNS_19MachineRegisterInfoEPKNS_12MachineInstrEjS9_j", "llvm::X86InstrInfo::hasHighOperandLatency(llvm::InstrItineraryData const*, llvm::MachineRegisterInfo const*, llvm::MachineInstr const*, unsigned int, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZN4llvm23createGlobalBaseRegPassEv", "llvm::createGlobalBaseRegPass()"},
    {"_ZN12_GLOBAL__N_14CGBRD1Ev", "(anonymous namespace)::CGBR::~CGBR()"},
    {"_ZN12_GLOBAL__N_14CGBRD0Ev", "(anonymous namespace)::CGBR::~CGBR()"},
    {"_ZNK12_GLOBAL__N_14CGBR11getPassNameEv", "(anonymous namespace)::CGBR::getPassName() const"},
    {"_ZNK12_GLOBAL__N_14CGBR16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::CGBR::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14CGBR20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::CGBR::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_Z21getLoadStoreRegOpcodejPKN4llvm19TargetRegisterClassEbRKNS_13TargetMachineEb", "getLoadStoreRegOpcode(unsigned int, llvm::TargetRegisterClass const*, bool, llvm::TargetMachine const&, bool)"},
    {"_ZN4llvm22X86MachineFunctionInfoD1Ev", "llvm::X86MachineFunctionInfo::~X86MachineFunctionInfo()"},
    {"_ZN4llvm22X86MachineFunctionInfoD0Ev", "llvm::X86MachineFunctionInfo::~X86MachineFunctionInfo()"},
    {"_ZNSt6vectorIN4llvm3EVTESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::EVT, std::allocator<llvm::EVT> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::EVT*, std::vector<llvm::EVT, std::allocator<llvm::EVT> > >, llvm::EVT const&)"},
    {"_ZNSt6vectorIN4llvm7SDValueESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SDValue*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > >, llvm::SDValue const&)"},
    {"_ZN4llvm8DenseMapIjSt4pairIjjENS_12DenseMapInfoIjEENS3_IS2_EEE16InsertIntoBucketERKjRKS2_PS1_IjS2_E", "llvm::DenseMap<unsigned int, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::InsertIntoBucket(unsigned int const&, std::pair<unsigned int, unsigned int> const&, std::pair<unsigned int, std::pair<unsigned int, unsigned int> >*)"},
    {"_ZN4llvm8DenseMapIjSt4pairIjjENS_12DenseMapInfoIjEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned int, std::pair<unsigned int, unsigned int>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> > >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm12MachineInstrESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, llvm::MachineInstr* const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_14MachineOperandELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::MachineOperand, false>::grow(unsigned long)"},
    {"_ZN4llvm19MachineFunctionPassD1Ev", "llvm::MachineFunctionPass::~MachineFunctionPass()"},
    {"_ZN4llvm19MachineFunctionPassD0Ev", "llvm::MachineFunctionPass::~MachineFunctionPass()"},
    {"_ZN4llvm12FunctionPassD1Ev", "llvm::FunctionPass::~FunctionPass()"},
    {"_ZN4llvm12FunctionPassD0Ev", "llvm::FunctionPass::~FunctionPass()"},
    {"_ZN4llvm10X86JITInfo29replaceMachineCodeForFunctionEPvS1_", "llvm::X86JITInfo::replaceMachineCodeForFunction(void*, void*)"},
    {"_ZN4llvm10X86JITInfo23getLazyResolverFunctionEPFPvS1_E", "llvm::X86JITInfo::getLazyResolverFunction(void* (*)(void*))"},
    {"_ZN4llvm10X86JITInfoC1ERNS_16X86TargetMachineE", "llvm::X86JITInfo::X86JITInfo(llvm::X86TargetMachine&)"},
    {"_ZN4llvm10X86JITInfoC2ERNS_16X86TargetMachineE", "llvm::X86JITInfo::X86JITInfo(llvm::X86TargetMachine&)"},
    {"_ZN4llvm10X86JITInfo26emitGlobalValueIndirectSymEPKNS_11GlobalValueEPvRNS_14JITCodeEmitterE", "llvm::X86JITInfo::emitGlobalValueIndirectSym(llvm::GlobalValue const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm10X86JITInfo13getStubLayoutEv", "llvm::X86JITInfo::getStubLayout()"},
    {"_ZN4llvm10X86JITInfo16emitFunctionStubEPKNS_8FunctionEPvRNS_14JITCodeEmitterE", "llvm::X86JITInfo::emitFunctionStub(llvm::Function const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm10X86JITInfo20getPICJumpTableEntryEmm", "llvm::X86JITInfo::getPICJumpTableEntry(unsigned long, unsigned long)"},
    {"_ZN4llvm10X86JITInfo8relocateEPvPNS_17MachineRelocationEjPh", "llvm::X86JITInfo::relocate(void*, llvm::MachineRelocation*, unsigned int, unsigned char*)"},
    {"_ZN4llvm10X86JITInfo25allocateThreadLocalMemoryEm", "llvm::X86JITInfo::allocateThreadLocalMemory(unsigned long)"},
    {"_ZN4llvm18X86MCAsmInfoDarwinC1ERKNS_6TripleE", "llvm::X86MCAsmInfoDarwin::X86MCAsmInfoDarwin(llvm::Triple const&)"},
    {"_ZN4llvm18X86MCAsmInfoDarwinC2ERKNS_6TripleE", "llvm::X86MCAsmInfoDarwin::X86MCAsmInfoDarwin(llvm::Triple const&)"},
    {"_ZN4llvm15X86ELFMCAsmInfoC1ERKNS_6TripleE", "llvm::X86ELFMCAsmInfo::X86ELFMCAsmInfo(llvm::Triple const&)"},
    {"_ZN4llvm15X86ELFMCAsmInfoC2ERKNS_6TripleE", "llvm::X86ELFMCAsmInfo::X86ELFMCAsmInfo(llvm::Triple const&)"},
    {"_ZNK4llvm15X86ELFMCAsmInfo28getNonexecutableStackSectionERNS_9MCContextE", "llvm::X86ELFMCAsmInfo::getNonexecutableStackSection(llvm::MCContext&) const"},
    {"_ZN4llvm16X86MCAsmInfoCOFFC1ERKNS_6TripleE", "llvm::X86MCAsmInfoCOFF::X86MCAsmInfoCOFF(llvm::Triple const&)"},
    {"_ZN4llvm16X86MCAsmInfoCOFFC2ERKNS_6TripleE", "llvm::X86MCAsmInfoCOFF::X86MCAsmInfoCOFF(llvm::Triple const&)"},
    {"_ZN4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEED1Ev", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::~opt()"},
    {"_ZNK4llvm9MCAsmInfo18getDataASDirectiveEjj", "llvm::MCAsmInfo::getDataASDirective(unsigned int, unsigned int) const"},
    {"_ZN4llvm15X86ELFMCAsmInfoD1Ev", "llvm::X86ELFMCAsmInfo::~X86ELFMCAsmInfo()"},
    {"_ZN4llvm15X86ELFMCAsmInfoD0Ev", "llvm::X86ELFMCAsmInfo::~X86ELFMCAsmInfo()"},
    {"_ZN4llvm16X86MCAsmInfoCOFFD1Ev", "llvm::X86MCAsmInfoCOFF::~X86MCAsmInfoCOFF()"},
    {"_ZN4llvm16X86MCAsmInfoCOFFD0Ev", "llvm::X86MCAsmInfoCOFF::~X86MCAsmInfoCOFF()"},
    {"_ZNK4llvm9MCAsmInfo28getNonexecutableStackSectionERNS_9MCContextE", "llvm::MCAsmInfo::getNonexecutableStackSection(llvm::MCContext&) const"},
    {"_ZN4llvm18X86MCAsmInfoDarwinD1Ev", "llvm::X86MCAsmInfoDarwin::~X86MCAsmInfoDarwin()"},
    {"_ZN4llvm18X86MCAsmInfoDarwinD0Ev", "llvm::X86MCAsmInfoDarwin::~X86MCAsmInfoDarwin()"},
    {"_ZN4llvm15MCAsmInfoDarwinD1Ev", "llvm::MCAsmInfoDarwin::~MCAsmInfoDarwin()"},
    {"_ZN4llvm15MCAsmInfoDarwinD0Ev", "llvm::MCAsmInfoDarwin::~MCAsmInfoDarwin()"},
    {"_ZN4llvm13MCAsmInfoCOFFD1Ev", "llvm::MCAsmInfoCOFF::~MCAsmInfoCOFF()"},
    {"_ZN4llvm13MCAsmInfoCOFFD0Ev", "llvm::MCAsmInfoCOFF::~MCAsmInfoCOFF()"},
    {"_ZN4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEE16handleOccurrenceEjNS_9StringRefES6_", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEE14getOptionWidthEv", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEE15printOptionInfoEm", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEED0Ev", "llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >::~opt()"},
    {"_ZN4llvm2cl6parserI17AsmWriterFlavorTyE5parseERNS0_6OptionENS_9StringRefES6_RS2_", "llvm::cl::parser<AsmWriterFlavorTy>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, AsmWriterFlavorTy&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optI17AsmWriterFlavorTyLb0ENS0_6parserIS5_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> > >(llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserI17AsmWriterFlavorTyE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<AsmWriterFlavorTy>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserI17AsmWriterFlavorTyED1Ev", "llvm::cl::parser<AsmWriterFlavorTy>::~parser()"},
    {"_ZN4llvm2cl6parserI17AsmWriterFlavorTyED0Ev", "llvm::cl::parser<AsmWriterFlavorTy>::~parser()"},
    {"_ZNK4llvm2cl6parserI17AsmWriterFlavorTyE13getNumOptionsEv", "llvm::cl::parser<AsmWriterFlavorTy>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserI17AsmWriterFlavorTyE9getOptionEj", "llvm::cl::parser<AsmWriterFlavorTy>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserI17AsmWriterFlavorTyE14getDescriptionEj", "llvm::cl::parser<AsmWriterFlavorTy>::getDescription(unsigned int) const"},
    {"_ZN4llvm25createX86_32MCCodeEmitterERKNS_6TargetERNS_13TargetMachineERNS_9MCContextE", "llvm::createX86_32MCCodeEmitter(llvm::Target const&, llvm::TargetMachine&, llvm::MCContext&)"},
    {"_ZN4llvm25createX86_64MCCodeEmitterERKNS_6TargetERNS_13TargetMachineERNS_9MCContextE", "llvm::createX86_64MCCodeEmitter(llvm::Target const&, llvm::TargetMachine&, llvm::MCContext&)"},
    {"_ZN12_GLOBAL__N_116X86MCCodeEmitterD1Ev", "(anonymous namespace)::X86MCCodeEmitter::~X86MCCodeEmitter()"},
    {"_ZN12_GLOBAL__N_116X86MCCodeEmitterD0Ev", "(anonymous namespace)::X86MCCodeEmitter::~X86MCCodeEmitter()"},
    {"_ZNK12_GLOBAL__N_116X86MCCodeEmitter17EncodeInstructionERKN4llvm6MCInstERNS1_11raw_ostreamERNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::X86MCCodeEmitter::EncodeInstruction(llvm::MCInst const&, llvm::raw_ostream&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116X86MCCodeEmitter13EmitImmediateERKN4llvm9MCOperandEjNS1_11MCFixupKindERjRNS1_11raw_ostreamERNS1_15SmallVectorImplINS1_7MCFixupEEEi", "(anonymous namespace)::X86MCCodeEmitter::EmitImmediate(llvm::MCOperand const&, unsigned int, llvm::MCFixupKind, unsigned int&, llvm::raw_ostream&, llvm::SmallVectorImpl<llvm::MCFixup>&, int) const"},
    {"_ZNK12_GLOBAL__N_116X86MCCodeEmitter16EmitMemModRMByteERKN4llvm6MCInstEjjyRjRNS1_11raw_ostreamERNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::X86MCCodeEmitter::EmitMemModRMByte(llvm::MCInst const&, unsigned int, unsigned int, unsigned long long, unsigned int&, llvm::raw_ostream&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116X86MCCodeEmitter25EmitSegmentOverridePrefixEyRjiRKN4llvm6MCInstERNS2_11raw_ostreamE", "(anonymous namespace)::X86MCCodeEmitter::EmitSegmentOverridePrefix(unsigned long long, unsigned int&, int, llvm::MCInst const&, llvm::raw_ostream&) const"},
    {"_ZN4llvm14X86MCInstLowerC1EPNS_7ManglerERKNS_15MachineFunctionERNS_13X86AsmPrinterE", "llvm::X86MCInstLower::X86MCInstLower(llvm::Mangler*, llvm::MachineFunction const&, llvm::X86AsmPrinter&)"},
    {"_ZN4llvm14X86MCInstLowerC2EPNS_7ManglerERKNS_15MachineFunctionERNS_13X86AsmPrinterE", "llvm::X86MCInstLower::X86MCInstLower(llvm::Mangler*, llvm::MachineFunction const&, llvm::X86AsmPrinter&)"},
    {"_ZNK4llvm14X86MCInstLower11getMachOMMIEv", "llvm::X86MCInstLower::getMachOMMI() const"},
    {"_ZNK4llvm14X86MCInstLower20GetSymbolFromOperandERKNS_14MachineOperandE", "llvm::X86MCInstLower::GetSymbolFromOperand(llvm::MachineOperand const&) const"},
    {"_ZNK4llvm14X86MCInstLower18LowerSymbolOperandERKNS_14MachineOperandEPNS_8MCSymbolE", "llvm::X86MCInstLower::LowerSymbolOperand(llvm::MachineOperand const&, llvm::MCSymbol*) const"},
    {"_ZNK4llvm14X86MCInstLower5LowerEPKNS_12MachineInstrERNS_6MCInstE", "llvm::X86MCInstLower::Lower(llvm::MachineInstr const*, llvm::MCInst&) const"},
    {"_Z21SimplifyShortMoveFormRN4llvm13X86AsmPrinterERNS_6MCInstEj", "SimplifyShortMoveForm(llvm::X86AsmPrinter&, llvm::MCInst&, unsigned int)"},
    {"_Z20SimplifyShortImmFormRN4llvm6MCInstEj", "SimplifyShortImmForm(llvm::MCInst&, unsigned int)"},
    {"_ZN4llvm13X86AsmPrinter15EmitInstructionEPKNS_12MachineInstrE", "llvm::X86AsmPrinter::EmitInstruction(llvm::MachineInstr const*)"},
    {"_ZN4llvm15SmallVectorImplIcE6insertIPKcEEPcS5_T_S6_", "char* llvm::SmallVectorImpl<char>::insert<char const*>(char*, char const*, char const*)"},
    {"_ZN4llvm25createX86MachObjectWriterERNS_11raw_ostreamEbjj", "llvm::createX86MachObjectWriter(llvm::raw_ostream&, bool, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_119X86MachObjectWriterD1Ev", "(anonymous namespace)::X86MachObjectWriter::~X86MachObjectWriter()"},
    {"_ZN12_GLOBAL__N_119X86MachObjectWriterD0Ev", "(anonymous namespace)::X86MachObjectWriter::~X86MachObjectWriter()"},
    {"_ZN4llvm15X86RegisterInfoC1ERNS_16X86TargetMachineERKNS_15TargetInstrInfoE", "llvm::X86RegisterInfo::X86RegisterInfo(llvm::X86TargetMachine&, llvm::TargetInstrInfo const&)"},
    {"_ZN4llvm15X86RegisterInfoC2ERNS_16X86TargetMachineERKNS_15TargetInstrInfoE", "llvm::X86RegisterInfo::X86RegisterInfo(llvm::X86TargetMachine&, llvm::TargetInstrInfo const&)"},
    {"_ZN4llvm18X86GenRegisterInfoC2Eii", "llvm::X86GenRegisterInfo::X86GenRegisterInfo(int, int)"},
    {"_ZNK4llvm15X86RegisterInfo14getDwarfRegNumEjb", "llvm::X86RegisterInfo::getDwarfRegNum(unsigned int, bool) const"},
    {"_ZNK4llvm18X86GenRegisterInfo18getDwarfRegNumFullEjj", "llvm::X86GenRegisterInfo::getDwarfRegNumFull(unsigned int, unsigned int) const"},
    {"_ZN4llvm15X86RegisterInfo12getX86RegNumEj", "llvm::X86RegisterInfo::getX86RegNum(unsigned int)"},
    {"_ZNK4llvm15X86RegisterInfo24getMatchingSuperRegClassEPKNS_19TargetRegisterClassES3_j", "llvm::X86RegisterInfo::getMatchingSuperRegClass(llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*, unsigned int) const"},
    {"_ZNK4llvm15X86RegisterInfo18getPointerRegClassEj", "llvm::X86RegisterInfo::getPointerRegClass(unsigned int) const"},
    {"_ZNK4llvm15X86RegisterInfo20getCrossCopyRegClassEPKNS_19TargetRegisterClassE", "llvm::X86RegisterInfo::getCrossCopyRegClass(llvm::TargetRegisterClass const*) const"},
    {"_ZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionE", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const"},
    {"_ZNK4llvm15X86RegisterInfo15getReservedRegsERKNS_15MachineFunctionE", "llvm::X86RegisterInfo::getReservedRegs(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm15X86RegisterInfo15canRealignStackERKNS_15MachineFunctionE", "llvm::X86RegisterInfo::canRealignStack(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm15X86RegisterInfo21needsStackRealignmentERKNS_15MachineFunctionE", "llvm::X86RegisterInfo::needsStackRealignment(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm15X86RegisterInfo20hasReservedSpillSlotERKNS_15MachineFunctionEjRi", "llvm::X86RegisterInfo::hasReservedSpillSlot(llvm::MachineFunction const&, unsigned int, int&) const"},
    {"_ZNK4llvm15X86RegisterInfo29eliminateCallFramePseudoInstrERNS_15MachineFunctionERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::X86RegisterInfo::eliminateCallFramePseudoInstr(llvm::MachineFunction&, llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZNK4llvm15X86RegisterInfo19eliminateFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEiPNS_12RegScavengerE", "llvm::X86RegisterInfo::eliminateFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, int, llvm::RegScavenger*) const"},
    {"_ZNK4llvm15X86RegisterInfo13getRARegisterEv", "llvm::X86RegisterInfo::getRARegister() const"},
    {"_ZNK4llvm15X86RegisterInfo16getFrameRegisterERKNS_15MachineFunctionE", "llvm::X86RegisterInfo::getFrameRegister(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm15X86RegisterInfo22getEHExceptionRegisterEv", "llvm::X86RegisterInfo::getEHExceptionRegister() const"},
    {"_ZNK4llvm15X86RegisterInfo20getEHHandlerRegisterEv", "llvm::X86RegisterInfo::getEHHandlerRegister() const"},
    {"_ZN4llvm22getX86SubSuperRegisterEjNS_3EVTEb", "llvm::getX86SubSuperRegister(unsigned int, llvm::EVT, bool)"},
    {"_ZN4llvm3X868CCRClassC1Ev", "llvm::X86::CCRClass::CCRClass()"},
    {"_ZN4llvm3X8616CONTROL_REGClassC1Ev", "llvm::X86::CONTROL_REGClass::CONTROL_REGClass()"},
    {"_ZN4llvm3X8614DEBUG_REGClassC1Ev", "llvm::X86::DEBUG_REGClass::DEBUG_REGClass()"},
    {"_ZN4llvm3X869FR32ClassC1Ev", "llvm::X86::FR32Class::FR32Class()"},
    {"_ZN4llvm3X869FR64ClassC1Ev", "llvm::X86::FR64Class::FR64Class()"},
    {"_ZN4llvm3X869GR16ClassC1Ev", "llvm::X86::GR16Class::GR16Class()"},
    {"_ZN4llvm3X8614GR16_ABCDClassC1Ev", "llvm::X86::GR16_ABCDClass::GR16_ABCDClass()"},
    {"_ZN4llvm3X8615GR16_NOREXClassC1Ev", "llvm::X86::GR16_NOREXClass::GR16_NOREXClass()"},
    {"_ZN4llvm3X869GR32ClassC1Ev", "llvm::X86::GR32Class::GR32Class()"},
    {"_ZN4llvm3X8614GR32_ABCDClassC1Ev", "llvm::X86::GR32_ABCDClass::GR32_ABCDClass()"},
    {"_ZN4llvm3X8612GR32_ADClassC1Ev", "llvm::X86::GR32_ADClass::GR32_ADClass()"},
    {"_ZN4llvm3X8615GR32_NOREXClassC1Ev", "llvm::X86::GR32_NOREXClass::GR32_NOREXClass()"},
    {"_ZN4llvm3X8614GR32_NOSPClassC1Ev", "llvm::X86::GR32_NOSPClass::GR32_NOSPClass()"},
    {"_ZN4llvm3X8612GR32_TCClassC1Ev", "llvm::X86::GR32_TCClass::GR32_TCClass()"},
    {"_ZN4llvm3X869GR64ClassC1Ev", "llvm::X86::GR64Class::GR64Class()"},
    {"_ZN4llvm3X8614GR64_ABCDClassC1Ev", "llvm::X86::GR64_ABCDClass::GR64_ABCDClass()"},
    {"_ZN4llvm3X8615GR64_NOREXClassC1Ev", "llvm::X86::GR64_NOREXClass::GR64_NOREXClass()"},
    {"_ZN4llvm3X8620GR64_NOREX_NOSPClassC1Ev", "llvm::X86::GR64_NOREX_NOSPClass::GR64_NOREX_NOSPClass()"},
    {"_ZN4llvm3X8614GR64_NOSPClassC1Ev", "llvm::X86::GR64_NOSPClass::GR64_NOSPClass()"},
    {"_ZN4llvm3X8612GR64_TCClassC1Ev", "llvm::X86::GR64_TCClass::GR64_TCClass()"},
    {"_ZN4llvm3X8615GR64_TCW64ClassC1Ev", "llvm::X86::GR64_TCW64Class::GR64_TCW64Class()"},
    {"_ZN4llvm3X868GR8ClassC1Ev", "llvm::X86::GR8Class::GR8Class()"},
    {"_ZN4llvm3X8615GR8_ABCD_HClassC1Ev", "llvm::X86::GR8_ABCD_HClass::GR8_ABCD_HClass()"},
    {"_ZN4llvm3X8615GR8_ABCD_LClassC1Ev", "llvm::X86::GR8_ABCD_LClass::GR8_ABCD_LClass()"},
    {"_ZN4llvm3X8614GR8_NOREXClassC1Ev", "llvm::X86::GR8_NOREXClass::GR8_NOREXClass()"},
    {"_ZN4llvm3X8610RFP32ClassC1Ev", "llvm::X86::RFP32Class::RFP32Class()"},
    {"_ZN4llvm3X8610RFP64ClassC1Ev", "llvm::X86::RFP64Class::RFP64Class()"},
    {"_ZN4llvm3X8610RFP80ClassC1Ev", "llvm::X86::RFP80Class::RFP80Class()"},
    {"_ZN4llvm3X868RSTClassC1Ev", "llvm::X86::RSTClass::RSTClass()"},
    {"_ZN4llvm3X8616SEGMENT_REGClassC1Ev", "llvm::X86::SEGMENT_REGClass::SEGMENT_REGClass()"},
    {"_ZN4llvm3X8610VR128ClassC1Ev", "llvm::X86::VR128Class::VR128Class()"},
    {"_ZN4llvm3X8610VR256ClassC1Ev", "llvm::X86::VR256Class::VR256Class()"},
    {"_ZN4llvm3X869VR64ClassC1Ev", "llvm::X86::VR64Class::VR64Class()"},
    {"_ZNK4llvm3X868CCRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::CCRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X868CCRClassC2Ev", "llvm::X86::CCRClass::CCRClass()"},
    {"_ZN4llvm3X8616CONTROL_REGClassC2Ev", "llvm::X86::CONTROL_REGClass::CONTROL_REGClass()"},
    {"_ZN4llvm3X8614DEBUG_REGClassC2Ev", "llvm::X86::DEBUG_REGClass::DEBUG_REGClass()"},
    {"_ZNK4llvm3X869FR32Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::FR32Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X869FR32ClassC2Ev", "llvm::X86::FR32Class::FR32Class()"},
    {"_ZNK4llvm3X869FR64Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::FR64Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X869FR64ClassC2Ev", "llvm::X86::FR64Class::FR64Class()"},
    {"_ZNK4llvm3X869GR16Class22allocation_order_beginERKNS_15MachineFunctionE", "llvm::X86::GR16Class::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3X869GR16Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR16Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X869GR16ClassC2Ev", "llvm::X86::GR16Class::GR16Class()"},
    {"_ZN4llvm3X8614GR16_ABCDClassC2Ev", "llvm::X86::GR16_ABCDClass::GR16_ABCDClass()"},
    {"_ZNK4llvm3X8615GR16_NOREXClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR16_NOREXClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8615GR16_NOREXClassC2Ev", "llvm::X86::GR16_NOREXClass::GR16_NOREXClass()"},
    {"_ZNK4llvm3X869GR32Class22allocation_order_beginERKNS_15MachineFunctionE", "llvm::X86::GR32Class::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3X869GR32Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR32Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X869GR32ClassC2Ev", "llvm::X86::GR32Class::GR32Class()"},
    {"_ZN4llvm3X8614GR32_ABCDClassC2Ev", "llvm::X86::GR32_ABCDClass::GR32_ABCDClass()"},
    {"_ZN4llvm3X8612GR32_ADClassC2Ev", "llvm::X86::GR32_ADClass::GR32_ADClass()"},
    {"_ZNK4llvm3X8615GR32_NOREXClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR32_NOREXClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8615GR32_NOREXClassC2Ev", "llvm::X86::GR32_NOREXClass::GR32_NOREXClass()"},
    {"_ZNK4llvm3X8614GR32_NOSPClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::X86::GR32_NOSPClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3X8614GR32_NOSPClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR32_NOSPClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8614GR32_NOSPClassC2Ev", "llvm::X86::GR32_NOSPClass::GR32_NOSPClass()"},
    {"_ZN4llvm3X8612GR32_TCClassC2Ev", "llvm::X86::GR32_TCClass::GR32_TCClass()"},
    {"_ZNK4llvm3X869GR64Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR64Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X869GR64ClassC2Ev", "llvm::X86::GR64Class::GR64Class()"},
    {"_ZN4llvm3X8614GR64_ABCDClassC2Ev", "llvm::X86::GR64_ABCDClass::GR64_ABCDClass()"},
    {"_ZNK4llvm3X8615GR64_NOREXClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR64_NOREXClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8615GR64_NOREXClassC2Ev", "llvm::X86::GR64_NOREXClass::GR64_NOREXClass()"},
    {"_ZNK4llvm3X8620GR64_NOREX_NOSPClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR64_NOREX_NOSPClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8620GR64_NOREX_NOSPClassC2Ev", "llvm::X86::GR64_NOREX_NOSPClass::GR64_NOREX_NOSPClass()"},
    {"_ZNK4llvm3X8614GR64_NOSPClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR64_NOSPClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8614GR64_NOSPClassC2Ev", "llvm::X86::GR64_NOSPClass::GR64_NOSPClass()"},
    {"_ZN4llvm3X8612GR64_TCClassC2Ev", "llvm::X86::GR64_TCClass::GR64_TCClass()"},
    {"_ZN4llvm3X8615GR64_TCW64ClassC2Ev", "llvm::X86::GR64_TCW64Class::GR64_TCW64Class()"},
    {"_ZNK4llvm3X868GR8Class22allocation_order_beginERKNS_15MachineFunctionE", "llvm::X86::GR8Class::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3X868GR8Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR8Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X868GR8ClassC2Ev", "llvm::X86::GR8Class::GR8Class()"},
    {"_ZN4llvm3X8615GR8_ABCD_HClassC2Ev", "llvm::X86::GR8_ABCD_HClass::GR8_ABCD_HClass()"},
    {"_ZN4llvm3X8615GR8_ABCD_LClassC2Ev", "llvm::X86::GR8_ABCD_LClass::GR8_ABCD_LClass()"},
    {"_ZNK4llvm3X8614GR8_NOREXClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::X86::GR8_NOREXClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3X8614GR8_NOREXClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::GR8_NOREXClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8614GR8_NOREXClassC2Ev", "llvm::X86::GR8_NOREXClass::GR8_NOREXClass()"},
    {"_ZN4llvm3X8610RFP32ClassC2Ev", "llvm::X86::RFP32Class::RFP32Class()"},
    {"_ZN4llvm3X8610RFP64ClassC2Ev", "llvm::X86::RFP64Class::RFP64Class()"},
    {"_ZN4llvm3X8610RFP80ClassC2Ev", "llvm::X86::RFP80Class::RFP80Class()"},
    {"_ZNK4llvm3X868RSTClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::RSTClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X868RSTClassC2Ev", "llvm::X86::RSTClass::RSTClass()"},
    {"_ZN4llvm3X8616SEGMENT_REGClassC2Ev", "llvm::X86::SEGMENT_REGClass::SEGMENT_REGClass()"},
    {"_ZNK4llvm3X8610VR128Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::VR128Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8610VR128ClassC2Ev", "llvm::X86::VR128Class::VR128Class()"},
    {"_ZNK4llvm3X8610VR256Class20allocation_order_endERKNS_15MachineFunctionE", "llvm::X86::VR256Class::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3X8610VR256ClassC2Ev", "llvm::X86::VR256Class::VR256Class()"},
    {"_ZN4llvm3X869VR64ClassC2Ev", "llvm::X86::VR64Class::VR64Class()"},
    {"_ZNK4llvm18X86GenRegisterInfo9getSubRegEjj", "llvm::X86GenRegisterInfo::getSubReg(unsigned int, unsigned int) const"},
    {"_ZNK4llvm18X86GenRegisterInfo14getSubRegIndexEjj", "llvm::X86GenRegisterInfo::getSubRegIndex(unsigned int, unsigned int) const"},
    {"_ZNK4llvm18X86GenRegisterInfo20composeSubRegIndicesEjj", "llvm::X86GenRegisterInfo::composeSubRegIndices(unsigned int, unsigned int) const"},
    {"_ZN4llvm18X86GenRegisterInfoC1Eii", "llvm::X86GenRegisterInfo::X86GenRegisterInfo(int, int)"},
    {"_ZN4llvm39createX86MaxStackAlignmentHeuristicPassEv", "llvm::createX86MaxStackAlignmentHeuristicPass()"},
    {"_ZN12_GLOBAL__N_14MSAHD1Ev", "(anonymous namespace)::MSAH::~MSAH()"},
    {"_ZN12_GLOBAL__N_14MSAHD0Ev", "(anonymous namespace)::MSAH::~MSAH()"},
    {"_ZNK12_GLOBAL__N_14MSAH11getPassNameEv", "(anonymous namespace)::MSAH::getPassName() const"},
    {"_ZNK12_GLOBAL__N_14MSAH16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MSAH::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14MSAH20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MSAH::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm3X868CCRClassD1Ev", "llvm::X86::CCRClass::~CCRClass()"},
    {"_ZN4llvm3X8616CONTROL_REGClassD1Ev", "llvm::X86::CONTROL_REGClass::~CONTROL_REGClass()"},
    {"_ZN4llvm3X8614DEBUG_REGClassD1Ev", "llvm::X86::DEBUG_REGClass::~DEBUG_REGClass()"},
    {"_ZN4llvm3X869FR32ClassD1Ev", "llvm::X86::FR32Class::~FR32Class()"},
    {"_ZN4llvm3X869FR64ClassD1Ev", "llvm::X86::FR64Class::~FR64Class()"},
    {"_ZN4llvm3X869GR16ClassD1Ev", "llvm::X86::GR16Class::~GR16Class()"},
    {"_ZN4llvm3X8614GR16_ABCDClassD1Ev", "llvm::X86::GR16_ABCDClass::~GR16_ABCDClass()"},
    {"_ZN4llvm3X8615GR16_NOREXClassD1Ev", "llvm::X86::GR16_NOREXClass::~GR16_NOREXClass()"},
    {"_ZN4llvm3X869GR32ClassD1Ev", "llvm::X86::GR32Class::~GR32Class()"},
    {"_ZN4llvm3X8614GR32_ABCDClassD1Ev", "llvm::X86::GR32_ABCDClass::~GR32_ABCDClass()"},
    {"_ZN4llvm3X8612GR32_ADClassD1Ev", "llvm::X86::GR32_ADClass::~GR32_ADClass()"},
    {"_ZN4llvm3X8615GR32_NOREXClassD1Ev", "llvm::X86::GR32_NOREXClass::~GR32_NOREXClass()"},
    {"_ZN4llvm3X8614GR32_NOSPClassD1Ev", "llvm::X86::GR32_NOSPClass::~GR32_NOSPClass()"},
    {"_ZN4llvm3X8612GR32_TCClassD1Ev", "llvm::X86::GR32_TCClass::~GR32_TCClass()"},
    {"_ZN4llvm3X869GR64ClassD1Ev", "llvm::X86::GR64Class::~GR64Class()"},
    {"_ZN4llvm3X8614GR64_ABCDClassD1Ev", "llvm::X86::GR64_ABCDClass::~GR64_ABCDClass()"},
    {"_ZN4llvm3X8615GR64_NOREXClassD1Ev", "llvm::X86::GR64_NOREXClass::~GR64_NOREXClass()"},
    {"_ZN4llvm3X8620GR64_NOREX_NOSPClassD1Ev", "llvm::X86::GR64_NOREX_NOSPClass::~GR64_NOREX_NOSPClass()"},
    {"_ZN4llvm3X8614GR64_NOSPClassD1Ev", "llvm::X86::GR64_NOSPClass::~GR64_NOSPClass()"},
    {"_ZN4llvm3X8612GR64_TCClassD1Ev", "llvm::X86::GR64_TCClass::~GR64_TCClass()"},
    {"_ZN4llvm3X8615GR64_TCW64ClassD1Ev", "llvm::X86::GR64_TCW64Class::~GR64_TCW64Class()"},
    {"_ZN4llvm3X868GR8ClassD1Ev", "llvm::X86::GR8Class::~GR8Class()"},
    {"_ZN4llvm3X8615GR8_ABCD_HClassD1Ev", "llvm::X86::GR8_ABCD_HClass::~GR8_ABCD_HClass()"},
    {"_ZN4llvm3X8615GR8_ABCD_LClassD1Ev", "llvm::X86::GR8_ABCD_LClass::~GR8_ABCD_LClass()"},
    {"_ZN4llvm3X8614GR8_NOREXClassD1Ev", "llvm::X86::GR8_NOREXClass::~GR8_NOREXClass()"},
    {"_ZN4llvm3X8610RFP32ClassD1Ev", "llvm::X86::RFP32Class::~RFP32Class()"},
    {"_ZN4llvm3X8610RFP64ClassD1Ev", "llvm::X86::RFP64Class::~RFP64Class()"},
    {"_ZN4llvm3X8610RFP80ClassD1Ev", "llvm::X86::RFP80Class::~RFP80Class()"},
    {"_ZN4llvm3X868RSTClassD1Ev", "llvm::X86::RSTClass::~RSTClass()"},
    {"_ZN4llvm3X8616SEGMENT_REGClassD1Ev", "llvm::X86::SEGMENT_REGClass::~SEGMENT_REGClass()"},
    {"_ZN4llvm3X8610VR128ClassD1Ev", "llvm::X86::VR128Class::~VR128Class()"},
    {"_ZN4llvm3X8610VR256ClassD1Ev", "llvm::X86::VR256Class::~VR256Class()"},
    {"_ZN4llvm3X869VR64ClassD1Ev", "llvm::X86::VR64Class::~VR64Class()"},
    {"_ZN4llvm3X868CCRClassD0Ev", "llvm::X86::CCRClass::~CCRClass()"},
    {"_ZN4llvm3X869FR32ClassD0Ev", "llvm::X86::FR32Class::~FR32Class()"},
    {"_ZN4llvm3X869FR64ClassD0Ev", "llvm::X86::FR64Class::~FR64Class()"},
    {"_ZN4llvm3X869GR16ClassD0Ev", "llvm::X86::GR16Class::~GR16Class()"},
    {"_ZN4llvm3X8615GR16_NOREXClassD0Ev", "llvm::X86::GR16_NOREXClass::~GR16_NOREXClass()"},
    {"_ZN4llvm3X869GR32ClassD0Ev", "llvm::X86::GR32Class::~GR32Class()"},
    {"_ZN4llvm3X8615GR32_NOREXClassD0Ev", "llvm::X86::GR32_NOREXClass::~GR32_NOREXClass()"},
    {"_ZN4llvm3X8614GR32_NOSPClassD0Ev", "llvm::X86::GR32_NOSPClass::~GR32_NOSPClass()"},
    {"_ZN4llvm3X869GR64ClassD0Ev", "llvm::X86::GR64Class::~GR64Class()"},
    {"_ZN4llvm3X8615GR64_NOREXClassD0Ev", "llvm::X86::GR64_NOREXClass::~GR64_NOREXClass()"},
    {"_ZN4llvm3X8620GR64_NOREX_NOSPClassD0Ev", "llvm::X86::GR64_NOREX_NOSPClass::~GR64_NOREX_NOSPClass()"},
    {"_ZN4llvm3X8614GR64_NOSPClassD0Ev", "llvm::X86::GR64_NOSPClass::~GR64_NOSPClass()"},
    {"_ZN4llvm3X868GR8ClassD0Ev", "llvm::X86::GR8Class::~GR8Class()"},
    {"_ZN4llvm3X8614GR8_NOREXClassD0Ev", "llvm::X86::GR8_NOREXClass::~GR8_NOREXClass()"},
    {"_ZN4llvm3X868RSTClassD0Ev", "llvm::X86::RSTClass::~RSTClass()"},
    {"_ZN4llvm3X8610VR128ClassD0Ev", "llvm::X86::VR128Class::~VR128Class()"},
    {"_ZN4llvm3X8610VR256ClassD0Ev", "llvm::X86::VR256Class::~VR256Class()"},
    {"_ZN4llvm3X869VR64ClassD0Ev", "llvm::X86::VR64Class::~VR64Class()"},
    {"_ZN4llvm3X8616SEGMENT_REGClassD0Ev", "llvm::X86::SEGMENT_REGClass::~SEGMENT_REGClass()"},
    {"_ZN4llvm3X8610RFP80ClassD0Ev", "llvm::X86::RFP80Class::~RFP80Class()"},
    {"_ZN4llvm3X8610RFP64ClassD0Ev", "llvm::X86::RFP64Class::~RFP64Class()"},
    {"_ZN4llvm3X8610RFP32ClassD0Ev", "llvm::X86::RFP32Class::~RFP32Class()"},
    {"_ZN4llvm3X8615GR8_ABCD_LClassD0Ev", "llvm::X86::GR8_ABCD_LClass::~GR8_ABCD_LClass()"},
    {"_ZN4llvm3X8615GR8_ABCD_HClassD0Ev", "llvm::X86::GR8_ABCD_HClass::~GR8_ABCD_HClass()"},
    {"_ZN4llvm3X8615GR64_TCW64ClassD0Ev", "llvm::X86::GR64_TCW64Class::~GR64_TCW64Class()"},
    {"_ZN4llvm3X8612GR64_TCClassD0Ev", "llvm::X86::GR64_TCClass::~GR64_TCClass()"},
    {"_ZN4llvm3X8614GR64_ABCDClassD0Ev", "llvm::X86::GR64_ABCDClass::~GR64_ABCDClass()"},
    {"_ZN4llvm3X8612GR32_TCClassD0Ev", "llvm::X86::GR32_TCClass::~GR32_TCClass()"},
    {"_ZN4llvm3X8612GR32_ADClassD0Ev", "llvm::X86::GR32_ADClass::~GR32_ADClass()"},
    {"_ZN4llvm3X8614GR32_ABCDClassD0Ev", "llvm::X86::GR32_ABCDClass::~GR32_ABCDClass()"},
    {"_ZN4llvm3X8614GR16_ABCDClassD0Ev", "llvm::X86::GR16_ABCDClass::~GR16_ABCDClass()"},
    {"_ZN4llvm3X8614DEBUG_REGClassD0Ev", "llvm::X86::DEBUG_REGClass::~DEBUG_REGClass()"},
    {"_ZN4llvm3X8616CONTROL_REGClassD0Ev", "llvm::X86::CONTROL_REGClass::~CONTROL_REGClass()"},
    {"_ZN4llvm19X86SelectionDAGInfoC1ERKNS_16X86TargetMachineE", "llvm::X86SelectionDAGInfo::X86SelectionDAGInfo(llvm::X86TargetMachine const&)"},
    {"_ZN4llvm19X86SelectionDAGInfoC2ERKNS_16X86TargetMachineE", "llvm::X86SelectionDAGInfo::X86SelectionDAGInfo(llvm::X86TargetMachine const&)"},
    {"_ZN4llvm19X86SelectionDAGInfoD0Ev", "llvm::X86SelectionDAGInfo::~X86SelectionDAGInfo()"},
    {"_ZN4llvm19X86SelectionDAGInfoD1Ev", "llvm::X86SelectionDAGInfo::~X86SelectionDAGInfo()"},
    {"_ZN4llvm19X86SelectionDAGInfoD2Ev", "llvm::X86SelectionDAGInfo::~X86SelectionDAGInfo()"},
    {"_ZNK4llvm19X86SelectionDAGInfo23EmitTargetCodeForMemsetERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbNS_18MachinePointerInfoE", "llvm::X86SelectionDAGInfo::EmitTargetCodeForMemset(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, llvm::MachinePointerInfo) const"},
    {"_ZNK4llvm19X86SelectionDAGInfo23EmitTargetCodeForMemcpyERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbbNS_18MachinePointerInfoES5_", "llvm::X86SelectionDAGInfo::EmitTargetCodeForMemcpy(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo) const"},
    {"_ZNK4llvm3EVT6bitsGTES0_", "llvm::EVT::bitsGT(llvm::EVT) const"},
    {"_ZN4llvm12X86Subtarget22ParseSubtargetFeaturesERKSsS2_", "llvm::X86Subtarget::ParseSubtargetFeatures(std::string const&, std::string const&)"},
    {"_ZNK4llvm12X86Subtarget29ClassifyBlockAddressReferenceEv", "llvm::X86Subtarget::ClassifyBlockAddressReference() const"},
    {"_ZNK4llvm12X86Subtarget23ClassifyGlobalReferenceEPKNS_11GlobalValueERKNS_13TargetMachineE", "llvm::X86Subtarget::ClassifyGlobalReference(llvm::GlobalValue const*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm12X86Subtarget13getBZeroEntryEv", "llvm::X86Subtarget::getBZeroEntry() const"},
    {"_ZNK4llvm12X86Subtarget26IsLegalToCallImmediateAddrERKNS_13TargetMachineE", "llvm::X86Subtarget::IsLegalToCallImmediateAddr(llvm::TargetMachine const&) const"},
    {"_ZNK4llvm12X86Subtarget24getSpecialAddressLatencyEv", "llvm::X86Subtarget::getSpecialAddressLatency() const"},
    {"_ZN4llvm12X86Subtarget27AutoDetectSubtargetFeaturesEv", "llvm::X86Subtarget::AutoDetectSubtargetFeatures()"},
    {"_ZN4llvm12X86SubtargetC1ERKSsS2_b", "llvm::X86Subtarget::X86Subtarget(std::string const&, std::string const&, bool)"},
    {"_ZN4llvm12X86SubtargetC2ERKSsS2_b", "llvm::X86Subtarget::X86Subtarget(std::string const&, std::string const&, bool)"},
    {"_ZNK4llvm12X86Subtarget11IsCalleePopEbNS_11CallingConv2IDE", "llvm::X86Subtarget::IsCalleePop(bool, llvm::CallingConv::ID) const"},
    {"_Z15createMCAsmInfoRKN4llvm6TargetENS_9StringRefE", "createMCAsmInfo(llvm::Target const&, llvm::StringRef)"},
    {"_Z16createMCStreamerRKN4llvm6TargetERKSsRNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterEbb", "createMCStreamer(llvm::Target const&, std::string const&, llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*, bool, bool)"},
    {"_ZN4llvm19X86_32TargetMachineC1ERKNS_6TargetERKSsS5_", "llvm::X86_32TargetMachine::X86_32TargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm19X86_32TargetMachineC2ERKNS_6TargetERKSsS5_", "llvm::X86_32TargetMachine::X86_32TargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm16X86TargetMachineC2ERKNS_6TargetERKSsS5_b", "llvm::X86TargetMachine::X86TargetMachine(llvm::Target const&, std::string const&, std::string const&, bool)"},
    {"_ZN4llvm19X86_64TargetMachineC1ERKNS_6TargetERKSsS5_", "llvm::X86_64TargetMachine::X86_64TargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm19X86_64TargetMachineC2ERKNS_6TargetERKSsS5_", "llvm::X86_64TargetMachine::X86_64TargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm16X86TargetMachineC1ERKNS_6TargetERKSsS5_b", "llvm::X86TargetMachine::X86TargetMachine(llvm::Target const&, std::string const&, std::string const&, bool)"},
    {"_ZN4llvm16X86TargetMachine15addInstSelectorERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::X86TargetMachine::addInstSelector(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16X86TargetMachine14addPreRegAllocERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::X86TargetMachine::addPreRegAlloc(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16X86TargetMachine15addPostRegAllocERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::X86TargetMachine::addPostRegAlloc(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16X86TargetMachine14addPreEmitPassERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::X86TargetMachine::addPreEmitPass(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16X86TargetMachine14addCodeEmitterERNS_15PassManagerBaseENS_10CodeGenOpt5LevelERNS_14JITCodeEmitterE", "llvm::X86TargetMachine::addCodeEmitter(llvm::PassManagerBase&, llvm::CodeGenOpt::Level, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm16X86TargetMachine21setCodeModelForStaticEv", "llvm::X86TargetMachine::setCodeModelForStatic()"},
    {"_ZN4llvm16X86TargetMachine18setCodeModelForJITEv", "llvm::X86TargetMachine::setCodeModelForJIT()"},
    {"_ZNK4llvm12X86Subtarget13isTargetWin64Ev", "llvm::X86Subtarget::isTargetWin64() const"},
    {"_ZNK4llvm16X86TargetMachine16getSubtargetImplEv", "llvm::X86TargetMachine::getSubtargetImpl() const"},
    {"_ZN4llvm16X86TargetMachineD1Ev", "llvm::X86TargetMachine::~X86TargetMachine()"},
    {"_ZN4llvm16X86TargetMachineD0Ev", "llvm::X86TargetMachine::~X86TargetMachine()"},
    {"_ZNK4llvm16X86TargetMachine12getInstrInfoEv", "llvm::X86TargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm16X86TargetMachine16getFrameLoweringEv", "llvm::X86TargetMachine::getFrameLowering() const"},
    {"_ZNK4llvm16X86TargetMachine17getTargetLoweringEv", "llvm::X86TargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm16X86TargetMachine19getSelectionDAGInfoEv", "llvm::X86TargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm16X86TargetMachine15getRegisterInfoEv", "llvm::X86TargetMachine::getRegisterInfo() const"},
    {"_ZN4llvm16X86TargetMachine10getJITInfoEv", "llvm::X86TargetMachine::getJITInfo()"},
    {"_ZNK4llvm16X86TargetMachine16getELFWriterInfoEv", "llvm::X86TargetMachine::getELFWriterInfo() const"},
    {"_ZN4llvm19X86_64TargetMachineD1Ev", "llvm::X86_64TargetMachine::~X86_64TargetMachine()"},
    {"_ZN4llvm19X86_64TargetMachineD0Ev", "llvm::X86_64TargetMachine::~X86_64TargetMachine()"},
    {"_ZNK4llvm19X86_64TargetMachine12getInstrInfoEv", "llvm::X86_64TargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm19X86_64TargetMachine17getTargetLoweringEv", "llvm::X86_64TargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm19X86_64TargetMachine19getSelectionDAGInfoEv", "llvm::X86_64TargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm19X86_64TargetMachine13getDataLayoutEv", "llvm::X86_64TargetMachine::getDataLayout() const"},
    {"_ZN4llvm19X86_64TargetMachine10getJITInfoEv", "llvm::X86_64TargetMachine::getJITInfo()"},
    {"_ZN4llvm19X86_32TargetMachineD1Ev", "llvm::X86_32TargetMachine::~X86_32TargetMachine()"},
    {"_ZN4llvm19X86_32TargetMachineD0Ev", "llvm::X86_32TargetMachine::~X86_32TargetMachine()"},
    {"_ZNK4llvm19X86_32TargetMachine12getInstrInfoEv", "llvm::X86_32TargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm19X86_32TargetMachine17getTargetLoweringEv", "llvm::X86_32TargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm19X86_32TargetMachine19getSelectionDAGInfoEv", "llvm::X86_32TargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm19X86_32TargetMachine13getDataLayoutEv", "llvm::X86_32TargetMachine::getDataLayout() const"},
    {"_ZN4llvm19X86_32TargetMachine10getJITInfoEv", "llvm::X86_32TargetMachine::getJITInfo()"},
    {"_ZN4llvm19X86_32TargetMachineD2Ev", "llvm::X86_32TargetMachine::~X86_32TargetMachine()"},
    {"_ZN4llvm10X86JITInfoD1Ev", "llvm::X86JITInfo::~X86JITInfo()"},
    {"_ZN4llvm17X86TargetLoweringD1Ev", "llvm::X86TargetLowering::~X86TargetLowering()"},
    {"_ZN4llvm12X86InstrInfoD1Ev", "llvm::X86InstrInfo::~X86InstrInfo()"},
    {"_ZN4llvm16X86TargetMachineD2Ev", "llvm::X86TargetMachine::~X86TargetMachine()"},
    {"_ZN4llvm16X86FrameLoweringD1Ev", "llvm::X86FrameLowering::~X86FrameLowering()"},
    {"_ZN4llvm12X86SubtargetD1Ev", "llvm::X86Subtarget::~X86Subtarget()"},
    {"_ZN4llvm12X86SubtargetD0Ev", "llvm::X86Subtarget::~X86Subtarget()"},
    {"_ZN4llvm16X86FrameLoweringD0Ev", "llvm::X86FrameLowering::~X86FrameLowering()"},
    {"_ZN4llvm15X86RegisterInfoD1Ev", "llvm::X86RegisterInfo::~X86RegisterInfo()"},
    {"_ZN4llvm12X86InstrInfoD0Ev", "llvm::X86InstrInfo::~X86InstrInfo()"},
    {"_ZNK4llvm12X86InstrInfo15getRegisterInfoEv", "llvm::X86InstrInfo::getRegisterInfo() const"},
    {"_ZN4llvm15X86RegisterInfoD0Ev", "llvm::X86RegisterInfo::~X86RegisterInfo()"},
    {"_ZN4llvm18X86GenRegisterInfoD1Ev", "llvm::X86GenRegisterInfo::~X86GenRegisterInfo()"},
    {"_ZN4llvm18X86GenRegisterInfoD0Ev", "llvm::X86GenRegisterInfo::~X86GenRegisterInfo()"},
    {"_ZNK4llvm18X86GenRegisterInfo21needsStackRealignmentERKNS_15MachineFunctionE", "llvm::X86GenRegisterInfo::needsStackRealignment(llvm::MachineFunction const&) const"},
    {"_ZN4llvm17X86TargetLoweringD0Ev", "llvm::X86TargetLowering::~X86TargetLowering()"},
    {"_ZNK4llvm14TargetLowering23getSchedulingPreferenceEPNS_6SDNodeE", "llvm::TargetLowering::getSchedulingPreference(llvm::SDNode*) const"},
    {"_ZNK4llvm14TargetLowering14getRegClassForENS_3EVTE", "llvm::TargetLowering::getRegClassFor(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering18getTgtMemIntrinsicERNS0_13IntrinsicInfoERKNS_8CallInstEj", "llvm::TargetLowering::getTgtMemIntrinsic(llvm::TargetLowering::IntrinsicInfo&, llvm::CallInst const&, unsigned int) const"},
    {"_ZNK4llvm17X86TargetLowering22ShouldShrinkFPConstantENS_3EVTE", "llvm::X86TargetLowering::ShouldShrinkFPConstant(llvm::EVT) const"},
    {"_ZNK4llvm17X86TargetLowering29allowsUnalignedMemoryAccessesENS_3EVTE", "llvm::X86TargetLowering::allowsUnalignedMemoryAccesses(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering25getPreIndexedAddressPartsEPNS_6SDNodeERNS_7SDValueES4_RNS_3ISD14MemIndexedModeERNS_12SelectionDAGE", "llvm::TargetLowering::getPreIndexedAddressParts(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&, llvm::ISD::MemIndexedMode&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering26getPostIndexedAddressPartsEPNS_6SDNodeES2_RNS_7SDValueES4_RNS_3ISD14MemIndexedModeERNS_12SelectionDAGE", "llvm::TargetLowering::getPostIndexedAddressParts(llvm::SDNode*, llvm::SDNode*, llvm::SDValue&, llvm::SDValue&, llvm::ISD::MemIndexedMode&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering22getMaximalGlobalOffsetEv", "llvm::TargetLowering::getMaximalGlobalOffset() const"},
    {"_ZNK4llvm14TargetLowering33isDesirableToTransformToIntegerOpEjNS_3EVTE", "llvm::TargetLowering::isDesirableToTransformToIntegerOp(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering20isLegalICmpImmediateEx", "llvm::TargetLowering::isLegalICmpImmediate(long long) const"},
    {"_ZN4llvm10X86JITInfoD0Ev", "llvm::X86JITInfo::~X86JITInfo()"},
    {"_ZN4llvm19X86_64TargetMachineD2Ev", "llvm::X86_64TargetMachine::~X86_64TargetMachine()"},
    {"_ZN4llvm21RegisterTargetMachineINS_19X86_64TargetMachineEE9AllocatorERKNS_6TargetERKSsS7_", "llvm::RegisterTargetMachine<llvm::X86_64TargetMachine>::Allocator(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm21RegisterTargetMachineINS_19X86_32TargetMachineEE9AllocatorERKNS_6TargetERKSsS7_", "llvm::RegisterTargetMachine<llvm::X86_32TargetMachine>::Allocator(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZNK4llvm27X8664_MachoTargetObjectFile30getExprForDwarfGlobalReferenceEPKNS_11GlobalValueEPNS_7ManglerEPNS_17MachineModuleInfoEjRNS_10MCStreamerE", "llvm::X8664_MachoTargetObjectFile::getExprForDwarfGlobalReference(llvm::GlobalValue const*, llvm::Mangler*, llvm::MachineModuleInfo*, unsigned int, llvm::MCStreamer&) const"},
    {"_ZNK4llvm25X8632_ELFTargetObjectFile22getPersonalityEncodingEv", "llvm::X8632_ELFTargetObjectFile::getPersonalityEncoding() const"},
    {"_ZNK4llvm25X8632_ELFTargetObjectFile15getLSDAEncodingEv", "llvm::X8632_ELFTargetObjectFile::getLSDAEncoding() const"},
    {"_ZNK4llvm25X8632_ELFTargetObjectFile14getFDEEncodingEv", "llvm::X8632_ELFTargetObjectFile::getFDEEncoding() const"},
    {"_ZNK4llvm25X8632_ELFTargetObjectFile16getTTypeEncodingEv", "llvm::X8632_ELFTargetObjectFile::getTTypeEncoding() const"},
    {"_ZNK4llvm25X8664_ELFTargetObjectFile22getPersonalityEncodingEv", "llvm::X8664_ELFTargetObjectFile::getPersonalityEncoding() const"},
    {"_ZNK4llvm25X8664_ELFTargetObjectFile15getLSDAEncodingEv", "llvm::X8664_ELFTargetObjectFile::getLSDAEncoding() const"},
    {"_ZNK4llvm25X8664_ELFTargetObjectFile14getFDEEncodingEv", "llvm::X8664_ELFTargetObjectFile::getFDEEncoding() const"},
    {"_ZNK4llvm25X8664_ELFTargetObjectFile16getTTypeEncodingEv", "llvm::X8664_ELFTargetObjectFile::getTTypeEncoding() const"},
    {"_ZN4llvm25X8632_ELFTargetObjectFileD1Ev", "llvm::X8632_ELFTargetObjectFile::~X8632_ELFTargetObjectFile()"},
    {"_ZN4llvm25X8632_ELFTargetObjectFileD0Ev", "llvm::X8632_ELFTargetObjectFile::~X8632_ELFTargetObjectFile()"},
    {"_ZN4llvm25X8664_ELFTargetObjectFileD1Ev", "llvm::X8664_ELFTargetObjectFile::~X8664_ELFTargetObjectFile()"},
    {"_ZN4llvm25X8664_ELFTargetObjectFileD0Ev", "llvm::X8664_ELFTargetObjectFile::~X8664_ELFTargetObjectFile()"},
    {"_ZN4llvm27X8664_MachoTargetObjectFileD1Ev", "llvm::X8664_MachoTargetObjectFile::~X8664_MachoTargetObjectFile()"},
    {"_ZN4llvm27X8664_MachoTargetObjectFileD0Ev", "llvm::X8664_MachoTargetObjectFile::~X8664_MachoTargetObjectFile()"},
    {"_ZN4llvm17X86ATTInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZN4llvm17X86ATTInstPrinter12printOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::printOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm17X86ATTInstPrinter15print_pcrel_immEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::print_pcrel_imm(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm17X86ATTInstPrinter10printSSECCEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::printSSECC(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm17X86ATTInstPrinter15getRegisterNameEj", "llvm::X86ATTInstPrinter::getRegisterName(unsigned int)"},
    {"_ZN4llvm17X86ATTInstPrinter18getInstructionNameEj", "llvm::X86ATTInstPrinter::getInstructionName(unsigned int)"},
    {"_ZN4llvm17X86ATTInstPrinter9printInstEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::printInst(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZNK4llvm17X86ATTInstPrinter13getOpcodeNameEj", "llvm::X86ATTInstPrinter::getOpcodeName(unsigned int) const"},
    {"_ZN4llvm17X86ATTInstPrinter17printMemReferenceEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86ATTInstPrinter::printMemReference(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm17X86ATTInstPrinterD1Ev", "llvm::X86ATTInstPrinter::~X86ATTInstPrinter()"},
    {"_ZN4llvm17X86ATTInstPrinterD0Ev", "llvm::X86ATTInstPrinter::~X86ATTInstPrinter()"},
    {"_ZN4llvm22EmitAnyX86InstCommentsEPKNS_6MCInstERNS_11raw_ostreamEPFPKcjE", "llvm::EmitAnyX86InstComments(llvm::MCInst const*, llvm::raw_ostream&, char const* (*)(unsigned int))"},
    {"_Z16DecodeSHUFPSMaskjjRN4llvm15SmallVectorImplIjEE", "DecodeSHUFPSMask(unsigned int, unsigned int, llvm::SmallVectorImpl<unsigned int>&)"},
    {"_ZN4llvm19X86IntelInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZN4llvm19X86IntelInstPrinter12printOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::printOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm19X86IntelInstPrinter15print_pcrel_immEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::print_pcrel_imm(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm19X86IntelInstPrinter10printSSECCEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::printSSECC(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm19X86IntelInstPrinter15getRegisterNameEj", "llvm::X86IntelInstPrinter::getRegisterName(unsigned int)"},
    {"_ZN4llvm19X86IntelInstPrinter18getInstructionNameEj", "llvm::X86IntelInstPrinter::getInstructionName(unsigned int)"},
    {"_ZN4llvm19X86IntelInstPrinter9printInstEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::printInst(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZNK4llvm19X86IntelInstPrinter13getOpcodeNameEj", "llvm::X86IntelInstPrinter::getOpcodeName(unsigned int) const"},
    {"_ZN4llvm19X86IntelInstPrinter17printMemReferenceEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::X86IntelInstPrinter::printMemReference(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm19X86IntelInstPrinterD1Ev", "llvm::X86IntelInstPrinter::~X86IntelInstPrinter()"},
    {"_ZN4llvm19X86IntelInstPrinterD0Ev", "llvm::X86IntelInstPrinter::~X86IntelInstPrinter()"},
    {"_ZN4llvm14RegisterTargetILNS_6Triple8ArchTypeE16ELb1EE21getTripleMatchQualityERKSs", "llvm::RegisterTarget<(llvm::Triple::ArchType)16, true>::getTripleMatchQuality(std::string const&)"},
    {"_ZN4llvm14RegisterTargetILNS_6Triple8ArchTypeE15ELb1EE21getTripleMatchQualityERKSs", "llvm::RegisterTarget<(llvm::Triple::ArchType)15, true>::getTripleMatchQuality(std::string const&)"},
    {"_ZN4llvm16RegisterAsmLexerIN12_GLOBAL__N_113ThumbAsmLexerEE9AllocatorERKNS_6TargetERKNS_9MCAsmInfoE", "llvm::RegisterAsmLexer<(anonymous namespace)::ThumbAsmLexer>::Allocator(llvm::Target const&, llvm::MCAsmInfo const&)"},
    {"_ZN12_GLOBAL__N_115ARMBaseAsmLexer15InitRegisterMapEPKN4llvm18TargetRegisterInfoE", "(anonymous namespace)::ARMBaseAsmLexer::InitRegisterMap(llvm::TargetRegisterInfo const*)"},
    {"_ZN12_GLOBAL__N_115ARMBaseAsmLexer8LexTokenEv", "(anonymous namespace)::ARMBaseAsmLexer::LexToken()"},
    {"_ZN12_GLOBAL__N_113ThumbAsmLexerD1Ev", "(anonymous namespace)::ThumbAsmLexer::~ThumbAsmLexer()"},
    {"_ZN12_GLOBAL__N_113ThumbAsmLexerD0Ev", "(anonymous namespace)::ThumbAsmLexer::~ThumbAsmLexer()"},
    {"_ZN12_GLOBAL__N_115ARMBaseAsmLexerD1Ev", "(anonymous namespace)::ARMBaseAsmLexer::~ARMBaseAsmLexer()"},
    {"_ZN12_GLOBAL__N_115ARMBaseAsmLexerD0Ev", "(anonymous namespace)::ARMBaseAsmLexer::~ARMBaseAsmLexer()"},
    {"_ZN4llvm16RegisterAsmLexerIN12_GLOBAL__N_111ARMAsmLexerEE9AllocatorERKNS_6TargetERKNS_9MCAsmInfoE", "llvm::RegisterAsmLexer<(anonymous namespace)::ARMAsmLexer>::Allocator(llvm::Target const&, llvm::MCAsmInfo const&)"},
    {"_ZN12_GLOBAL__N_111ARMAsmLexerD1Ev", "(anonymous namespace)::ARMAsmLexer::~ARMAsmLexer()"},
    {"_ZN12_GLOBAL__N_111ARMAsmLexerD0Ev", "(anonymous namespace)::ARMAsmLexer::~ARMAsmLexer()"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsjESt10_Select1stIS2_ESt4lessISsESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<std::string, std::pair<std::string const, unsigned int>, std::_Select1st<std::pair<std::string const, unsigned int> >, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsjESt10_Select1stIS2_ESt4lessISsESaIS2_EE4findERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, unsigned int>, std::_Select1st<std::pair<std::string const, unsigned int> >, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::find(std::string const&)"},
    {"_ZNSt3mapISsjSt4lessISsESaISt4pairIKSsjEEEixERS3_", "std::map<std::string, unsigned int, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::operator[](std::string const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsjESt10_Select1stIS2_ESt4lessISsESaIS2_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS2_ERKS2_", "std::_Rb_tree<std::string, std::pair<std::string const, unsigned int>, std::_Select1st<std::pair<std::string const, unsigned int> >, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::string const, unsigned int> >, std::pair<std::string const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsjESt10_Select1stIS2_ESt4lessISsESaIS2_EE9_M_insertEPSt18_Rb_tree_node_baseSA_RKS2_", "std::_Rb_tree<std::string, std::pair<std::string const, unsigned int>, std::_Select1st<std::pair<std::string const, unsigned int> >, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsjESt10_Select1stIS2_ESt4lessISsESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<std::string, std::pair<std::string const, unsigned int>, std::_Select1st<std::pair<std::string const, unsigned int> >, std::less<std::string>, std::allocator<std::pair<std::string const, unsigned int> > >::_M_insert_unique(std::pair<std::string const, unsigned int> const&)"},
    {"_ZN4llvm17RegisterAsmParserIN12_GLOBAL__N_112ARMAsmParserEE9AllocatorERKNS_6TargetERNS_11MCAsmParserERNS_13TargetMachineE", "llvm::RegisterAsmParser<(anonymous namespace)::ARMAsmParser>::Allocator(llvm::Target const&, llvm::MCAsmParser&, llvm::TargetMachine&)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParserD1Ev", "(anonymous namespace)::ARMAsmParser::~ARMAsmParser()"},
    {"_ZN12_GLOBAL__N_112ARMAsmParserD0Ev", "(anonymous namespace)::ARMAsmParser::~ARMAsmParser()"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser13ParseRegisterERjRN4llvm5SMLocES4_", "(anonymous namespace)::ARMAsmParser::ParseRegister(unsigned int&, llvm::SMLoc&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser16ParseInstructionEN4llvm9StringRefENS1_5SMLocERNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEE", "(anonymous namespace)::ARMAsmParser::ParseInstruction(llvm::StringRef, llvm::SMLoc, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>&)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser14ParseDirectiveEN4llvm8AsmTokenE", "(anonymous namespace)::ARMAsmParser::ParseDirective(llvm::AsmToken)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser23MatchAndEmitInstructionEN4llvm5SMLocERNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEERNS1_10MCStreamerE", "(anonymous namespace)::ARMAsmParser::MatchAndEmitInstruction(llvm::SMLoc, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>&, llvm::MCStreamer&)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser20MatchInstructionImplERKN4llvm15SmallVectorImplIPNS1_18MCParsedAsmOperandEEERNS1_6MCInstERj", "(anonymous namespace)::ARMAsmParser::MatchInstructionImpl(llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*> const&, llvm::MCInst&, unsigned int&)"},
    {"_ZN12_GLOBAL__N_110ARMOperandD1Ev", "(anonymous namespace)::ARMOperand::~ARMOperand()"},
    {"_ZN12_GLOBAL__N_110ARMOperandD0Ev", "(anonymous namespace)::ARMOperand::~ARMOperand()"},
    {"_ZNK12_GLOBAL__N_110ARMOperand11getStartLocEv", "(anonymous namespace)::ARMOperand::getStartLoc() const"},
    {"_ZNK12_GLOBAL__N_110ARMOperand9getEndLocEv", "(anonymous namespace)::ARMOperand::getEndLoc() const"},
    {"_ZNK12_GLOBAL__N_110ARMOperand4dumpERN4llvm11raw_ostreamE", "(anonymous namespace)::ARMOperand::dump(llvm::raw_ostream&) const"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser15ConvertToMCInstEjRN4llvm6MCInstEjRKNS1_15SmallVectorImplIPNS1_18MCParsedAsmOperandEEE", "(anonymous namespace)::ARMAsmParser::ConvertToMCInst(unsigned int, llvm::MCInst&, unsigned int, llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*> const&)"},
    {"_ZNK12_GLOBAL__N_110ARMOperand19addCondCodeOperandsERN4llvm6MCInstEj", "(anonymous namespace)::ARMOperand::addCondCodeOperands(llvm::MCInst&, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_110ARMOperand14addImmOperandsERN4llvm6MCInstEj", "(anonymous namespace)::ARMOperand::addImmOperands(llvm::MCInst&, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_110ARMOperand19addMemMode5OperandsERN4llvm6MCInstEj", "(anonymous namespace)::ARMOperand::addMemMode5Operands(llvm::MCInst&, unsigned int) const"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser12ParseOperandERN4llvm15SmallVectorImplIPNS1_18MCParsedAsmOperandEEENS1_9StringRefE", "(anonymous namespace)::ARMAsmParser::ParseOperand(llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>&, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser16TryParseRegisterEv", "(anonymous namespace)::ARMAsmParser::TryParseRegister()"},
    {"_ZN4llvm25array_pod_sort_comparatorIjEEiPKvS2_", "int llvm::array_pod_sort_comparator<unsigned int>(void const*, void const*)"},
    {"_ZN12_GLOBAL__N_112ARMAsmParser20ParseMemoryOffsetRegERbS1_R9ShiftTypeRPKN4llvm6MCExprES8_S1_RiRNS4_5SMLocE", "(anonymous namespace)::ARMAsmParser::ParseMemoryOffsetReg(bool&, bool&, ShiftType&, llvm::MCExpr const*&, llvm::MCExpr const*&, bool&, int&, llvm::SMLoc&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_18MCParsedAsmOperandEE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<llvm::MCParsedAsmOperand*>::insert(llvm::MCParsedAsmOperand**, llvm::MCParsedAsmOperand* const&)"},
    {"_ZN4llvm18MCParsedAsmOperandD1Ev", "llvm::MCParsedAsmOperand::~MCParsedAsmOperand()"},
    {"_ZN4llvm18MCParsedAsmOperandD0Ev", "llvm::MCParsedAsmOperand::~MCParsedAsmOperand()"},
    {"_ZN4llvm8DenseMapIjbNS_12DenseMapInfoIjEENS1_IbEEE16InsertIntoBucketERKjRKbPSt4pairIjbE", "llvm::DenseMap<unsigned int, bool, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<bool> >::InsertIntoBucket(unsigned int const&, bool const&, std::pair<unsigned int, bool>*)"},
    {"_ZN4llvm8DenseMapIjbNS_12DenseMapInfoIjEENS1_IbEEE4growEj", "llvm::DenseMap<unsigned int, bool, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<bool> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIjNS_5SMLocEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<unsigned int, llvm::SMLoc>, false>::grow(unsigned long)"},
    {"_ZN4llvm19createARMAsmBackendERKNS_6TargetERKSs", "llvm::createARMAsmBackend(llvm::Target const&, std::string const&)"},
    {"_ZN12_GLOBAL__N_116ELFARMAsmBackendD1Ev", "(anonymous namespace)::ELFARMAsmBackend::~ELFARMAsmBackend()"},
    {"_ZN12_GLOBAL__N_116ELFARMAsmBackendD0Ev", "(anonymous namespace)::ELFARMAsmBackend::~ELFARMAsmBackend()"},
    {"_ZNK12_GLOBAL__N_116ELFARMAsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::ELFARMAsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZNK12_GLOBAL__N_113ARMAsmBackend16getNumFixupKindsEv", "(anonymous namespace)::ARMAsmBackend::getNumFixupKinds() const"},
    {"_ZNK12_GLOBAL__N_113ARMAsmBackend16getFixupKindInfoEN4llvm11MCFixupKindE", "(anonymous namespace)::ARMAsmBackend::getFixupKindInfo(llvm::MCFixupKind) const"},
    {"_ZNK12_GLOBAL__N_116ELFARMAsmBackend10ApplyFixupERKN4llvm7MCFixupEPcjy", "(anonymous namespace)::ELFARMAsmBackend::ApplyFixup(llvm::MCFixup const&, char*, unsigned int, unsigned long long) const"},
    {"_ZNK12_GLOBAL__N_113ARMAsmBackend17MayNeedRelaxationERKN4llvm6MCInstE", "(anonymous namespace)::ARMAsmBackend::MayNeedRelaxation(llvm::MCInst const&) const"},
    {"_ZNK12_GLOBAL__N_113ARMAsmBackend16RelaxInstructionERKN4llvm6MCInstERS2_", "(anonymous namespace)::ARMAsmBackend::RelaxInstruction(llvm::MCInst const&, llvm::MCInst&) const"},
    {"_ZNK12_GLOBAL__N_113ARMAsmBackend12WriteNopDataEyPN4llvm14MCObjectWriterE", "(anonymous namespace)::ARMAsmBackend::WriteNopData(unsigned long long, llvm::MCObjectWriter*) const"},
    {"_ZN12_GLOBAL__N_113ARMAsmBackend19HandleAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::ARMAsmBackend::HandleAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_Z16adjustFixupValuejy", "adjustFixupValue(unsigned int, unsigned long long)"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriterD1Ev", "(anonymous namespace)::ARMELFObjectWriter::~ARMELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriterD0Ev", "(anonymous namespace)::ARMELFObjectWriter::~ARMELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_113ARMAsmBackendD1Ev", "(anonymous namespace)::ARMAsmBackend::~ARMAsmBackend()"},
    {"_ZN12_GLOBAL__N_113ARMAsmBackendD0Ev", "(anonymous namespace)::ARMAsmBackend::~ARMAsmBackend()"},
    {"_ZN12_GLOBAL__N_119DarwinARMAsmBackendD1Ev", "(anonymous namespace)::DarwinARMAsmBackend::~DarwinARMAsmBackend()"},
    {"_ZN12_GLOBAL__N_119DarwinARMAsmBackendD0Ev", "(anonymous namespace)::DarwinARMAsmBackend::~DarwinARMAsmBackend()"},
    {"_ZNK12_GLOBAL__N_119DarwinARMAsmBackend18createObjectWriterERN4llvm11raw_ostreamE", "(anonymous namespace)::DarwinARMAsmBackend::createObjectWriter(llvm::raw_ostream&) const"},
    {"_ZNK12_GLOBAL__N_119DarwinARMAsmBackend25doesSectionRequireSymbolsERKN4llvm9MCSectionE", "(anonymous namespace)::DarwinARMAsmBackend::doesSectionRequireSymbols(llvm::MCSection const&) const"},
    {"_ZNK12_GLOBAL__N_119DarwinARMAsmBackend10ApplyFixupERKN4llvm7MCFixupEPcjy", "(anonymous namespace)::DarwinARMAsmBackend::ApplyFixup(llvm::MCFixup const&, char*, unsigned int, unsigned long long) const"},
    {"_ZN12_GLOBAL__N_119ARMMachObjectWriterD1Ev", "(anonymous namespace)::ARMMachObjectWriter::~ARMMachObjectWriter()"},
    {"_ZN12_GLOBAL__N_119ARMMachObjectWriterD0Ev", "(anonymous namespace)::ARMMachObjectWriter::~ARMMachObjectWriter()"},
    {"_ZN4llvm14MCObjectWriter7Write16Et", "llvm::MCObjectWriter::Write16(unsigned short)"},
    {"_ZN4llvm14MCObjectWriter9WriteLE32Ej", "llvm::MCObjectWriter::WriteLE32(unsigned int)"},
    {"_ZN4llvm14MCObjectWriter9WriteBE32Ej", "llvm::MCObjectWriter::WriteBE32(unsigned int)"},
    {"_ZNK4llvm13ARMAsmPrinter21getDebugValueLocationEPKNS_12MachineInstrE", "llvm::ARMAsmPrinter::getDebugValueLocation(llvm::MachineInstr const*) const"},
    {"_ZN4llvm13ARMAsmPrinter22EmitFunctionEntryLabelEv", "llvm::ARMAsmPrinter::EmitFunctionEntryLabel()"},
    {"_ZN4llvm13ARMAsmPrinter20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::ARMAsmPrinter::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm13ARMAsmPrinter12printOperandEPKNS_12MachineInstrEiRNS_11raw_ostreamEPKc", "llvm::ARMAsmPrinter::printOperand(llvm::MachineInstr const*, int, llvm::raw_ostream&, char const*)"},
    {"_ZNK4llvm13ARMAsmPrinter27GetARMSetPICJumpTableLabel2EjjPKNS_17MachineBasicBlockE", "llvm::ARMAsmPrinter::GetARMSetPICJumpTableLabel2(unsigned int, unsigned int, llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm13ARMAsmPrinter27GetARMJTIPICJumpTableLabel2Ejj", "llvm::ARMAsmPrinter::GetARMJTIPICJumpTableLabel2(unsigned int, unsigned int) const"},
    {"_ZNK4llvm13ARMAsmPrinter17GetARMSJLJEHLabelEv", "llvm::ARMAsmPrinter::GetARMSJLJEHLabel() const"},
    {"_ZN4llvm13ARMAsmPrinter15PrintAsmOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::ARMAsmPrinter::PrintAsmOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZN4llvm13ARMAsmPrinter21PrintAsmMemoryOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::ARMAsmPrinter::PrintAsmMemoryOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZN4llvm13ARMAsmPrinter18EmitStartOfAsmFileERNS_6ModuleE", "llvm::ARMAsmPrinter::EmitStartOfAsmFile(llvm::Module&)"},
    {"_ZN4llvm13ARMAsmPrinter14emitAttributesEv", "llvm::ARMAsmPrinter::emitAttributes()"},
    {"_ZN4llvm13ARMAsmPrinter16EmitEndOfAsmFileERNS_6ModuleE", "llvm::ARMAsmPrinter::EmitEndOfAsmFile(llvm::Module&)"},
    {"_ZN4llvm13ARMAsmPrinter23emitARMAttributeSectionEv", "llvm::ARMAsmPrinter::emitARMAttributeSection()"},
    {"_ZN4llvm13ARMAsmPrinter14GetARMGVSymbolEPKNS_11GlobalValueE", "llvm::ARMAsmPrinter::GetARMGVSymbol(llvm::GlobalValue const*)"},
    {"_ZN4llvm13ARMAsmPrinter28EmitMachineConstantPoolValueEPNS_24MachineConstantPoolValueE", "llvm::ARMAsmPrinter::EmitMachineConstantPoolValue(llvm::MachineConstantPoolValue*)"},
    {"_Z11getPICLabelPKcjjRN4llvm9MCContextE", "getPICLabel(char const*, unsigned int, unsigned int, llvm::MCContext&)"},
    {"_ZN4llvm13ARMAsmPrinter13EmitJumpTableEPKNS_12MachineInstrE", "llvm::ARMAsmPrinter::EmitJumpTable(llvm::MachineInstr const*)"},
    {"_ZN4llvm13ARMAsmPrinter14EmitJump2TableEPKNS_12MachineInstrE", "llvm::ARMAsmPrinter::EmitJump2Table(llvm::MachineInstr const*)"},
    {"_ZN4llvm13ARMAsmPrinter22PrintDebugValueCommentEPKNS_12MachineInstrERNS_11raw_ostreamE", "llvm::ARMAsmPrinter::PrintDebugValueComment(llvm::MachineInstr const*, llvm::raw_ostream&)"},
    {"_ZN4llvm13ARMAsmPrinter22EmitPatchedInstructionEPKNS_12MachineInstrEj", "llvm::ARMAsmPrinter::EmitPatchedInstruction(llvm::MachineInstr const*, unsigned int)"},
    {"_ZN4llvm13ARMAsmPrinter15EmitInstructionEPKNS_12MachineInstrE", "llvm::ARMAsmPrinter::EmitInstruction(llvm::MachineInstr const*)"},
    {"_Z19populateADROperandsRN4llvm6MCInstEjPKNS_8MCSymbolEjjRNS_9MCContextE", "populateADROperands(llvm::MCInst&, unsigned int, llvm::MCSymbol const*, unsigned int, unsigned int, llvm::MCContext&)"},
    {"_Z22createARMMCInstPrinterRKN4llvm6TargetEjRKNS_9MCAsmInfoE", "createARMMCInstPrinter(llvm::Target const&, unsigned int, llvm::MCAsmInfo const&)"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitter17MaybeSwitchVendorEN4llvm9StringRefE", "(anonymous namespace)::ObjectAttributeEmitter::MaybeSwitchVendor(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitter13EmitAttributeEjj", "(anonymous namespace)::ObjectAttributeEmitter::EmitAttribute(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitter17EmitTextAttributeEjN4llvm9StringRefE", "(anonymous namespace)::ObjectAttributeEmitter::EmitTextAttribute(unsigned int, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitter6FinishEv", "(anonymous namespace)::ObjectAttributeEmitter::Finish()"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitterD1Ev", "(anonymous namespace)::ObjectAttributeEmitter::~ObjectAttributeEmitter()"},
    {"_ZN12_GLOBAL__N_122ObjectAttributeEmitterD0Ev", "(anonymous namespace)::ObjectAttributeEmitter::~ObjectAttributeEmitter()"},
    {"_ZN12_GLOBAL__N_116AttributeEmitterD1Ev", "(anonymous namespace)::AttributeEmitter::~AttributeEmitter()"},
    {"_ZN12_GLOBAL__N_116AttributeEmitterD0Ev", "(anonymous namespace)::AttributeEmitter::~AttributeEmitter()"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitter17MaybeSwitchVendorEN4llvm9StringRefE", "(anonymous namespace)::AsmAttributeEmitter::MaybeSwitchVendor(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitter13EmitAttributeEjj", "(anonymous namespace)::AsmAttributeEmitter::EmitAttribute(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitter17EmitTextAttributeEjN4llvm9StringRefE", "(anonymous namespace)::AsmAttributeEmitter::EmitTextAttribute(unsigned int, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitter6FinishEv", "(anonymous namespace)::AsmAttributeEmitter::Finish()"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitterD1Ev", "(anonymous namespace)::AsmAttributeEmitter::~AsmAttributeEmitter()"},
    {"_ZN12_GLOBAL__N_119AsmAttributeEmitterD0Ev", "(anonymous namespace)::AsmAttributeEmitter::~AsmAttributeEmitter()"},
    {"_ZNSt6vectorISt4pairIPN4llvm8MCSymbolENS1_14PointerIntPairIS3_Lj1EbNS1_21PointerLikeTypeTraitsIS3_EEEEESaIS8_EEaSERKSA_", "std::vector<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, std::allocator<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > > >::operator=(std::vector<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, std::allocator<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > > > const&)"},
    {"_ZN4llvm13ARMAsmPrinterD1Ev", "llvm::ARMAsmPrinter::~ARMAsmPrinter()"},
    {"_ZN4llvm13ARMAsmPrinterD0Ev", "llvm::ARMAsmPrinter::~ARMAsmPrinter()"},
    {"_ZNK4llvm13ARMAsmPrinter11getPassNameEv", "llvm::ARMAsmPrinter::getPassName() const"},
    {"_ZNK4llvm4Pass9getPassIDEv", "llvm::Pass::getPassID() const"},
    {"_ZN4llvm13ARMAsmPrinter16EmitConstantPoolEv", "llvm::ARMAsmPrinter::EmitConstantPool()"},
    {"_ZN4llvm10AsmPrinter21EmitFunctionBodyStartEv", "llvm::AsmPrinter::EmitFunctionBodyStart()"},
    {"_ZN4llvm10AsmPrinter19EmitFunctionBodyEndEv", "llvm::AsmPrinter::EmitFunctionBodyEnd()"},
    {"_ZN4llvm13ARMAsmPrinter14getISAEncodingEv", "llvm::ARMAsmPrinter::getISAEncoding()"},
    {"_ZN4llvm18RegisterAsmPrinterINS_13ARMAsmPrinterEE9AllocatorERNS_13TargetMachineERNS_10MCStreamerE", "llvm::RegisterAsmPrinter<llvm::ARMAsmPrinter>::Allocator(llvm::TargetMachine&, llvm::MCStreamer&)"},
    {"_ZN4llvm22MachineModuleInfoMachOC2ERKNS_17MachineModuleInfoE", "llvm::MachineModuleInfoMachO::MachineModuleInfoMachO(llvm::MachineModuleInfo const&)"},
    {"_ZN4llvm15ARMFunctionInfoC2ERNS_15MachineFunctionE", "llvm::ARMFunctionInfo::ARMFunctionInfo(llvm::MachineFunction&)"},
    {"_ZN4llvm15ARMFunctionInfoD1Ev", "llvm::ARMFunctionInfo::~ARMFunctionInfo()"},
    {"_ZN4llvm15ARMFunctionInfoD0Ev", "llvm::ARMFunctionInfo::~ARMFunctionInfo()"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_9MCOperandELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::MCOperand, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolENS_14PointerIntPairIS2_Lj1EbNS_21PointerLikeTypeTraitsIS2_EEEENS_12DenseMapInfoIS2_EENS7_IS6_EEE16InsertIntoBucketERKS2_RKS6_PSt4pairIS2_S6_E", "llvm::DenseMap<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > >::InsertIntoBucket(llvm::MCSymbol* const&, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > const&, std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >*)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolENS_14PointerIntPairIS2_Lj1EbNS_21PointerLikeTypeTraitsIS2_EEEENS_12DenseMapInfoIS2_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > >::grow(unsigned int)"},
    {"_ZN4llvm16ARMBaseInstrInfoC1ERKNS_12ARMSubtargetE", "llvm::ARMBaseInstrInfo::ARMBaseInstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZN4llvm16ARMBaseInstrInfoC2ERKNS_12ARMSubtargetE", "llvm::ARMBaseInstrInfo::ARMBaseInstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm16ARMBaseInstrInfo28CreateTargetHazardRecognizerEPKNS_13TargetMachineEPKNS_11ScheduleDAGE", "llvm::ARMBaseInstrInfo::CreateTargetHazardRecognizer(llvm::TargetMachine const*, llvm::ScheduleDAG const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo34CreateTargetPostRAHazardRecognizerEPKNS_18InstrItineraryDataEPKNS_11ScheduleDAGE", "llvm::ARMBaseInstrInfo::CreateTargetPostRAHazardRecognizer(llvm::InstrItineraryData const*, llvm::ScheduleDAG const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo21convertToThreeAddressERNS_14ilist_iteratorINS_17MachineBasicBlockEEERNS1_INS_12MachineInstrEEEPNS_13LiveVariablesE", "llvm::ARMBaseInstrInfo::convertToThreeAddress(llvm::ilist_iterator<llvm::MachineBasicBlock>&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::LiveVariables*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo13analyzeBranchERNS_17MachineBasicBlockERPS1_S4_RNS_15SmallVectorImplINS_14MachineOperandEEEb", "llvm::ARMBaseInstrInfo::analyzeBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*&, llvm::MachineBasicBlock*&, llvm::SmallVectorImpl<llvm::MachineOperand>&, bool) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo12removeBranchERNS_17MachineBasicBlockE", "llvm::ARMBaseInstrInfo::removeBranch(llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo12insertBranchERNS_17MachineBasicBlockEPS1_S3_RKNS_15SmallVectorImplINS_14MachineOperandEEENS_8DebugLocE", "llvm::ARMBaseInstrInfo::insertBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::DebugLoc) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo22ReverseBranchConditionERNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::ARMBaseInstrInfo::ReverseBranchCondition(llvm::SmallVectorImpl<llvm::MachineOperand>&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo20PredicateInstructionEPNS_12MachineInstrERKNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::ARMBaseInstrInfo::PredicateInstruction(llvm::MachineInstr*, llvm::SmallVectorImpl<llvm::MachineOperand> const&) const"},
    {"_ZN4llvm27getMatchingCondBranchOpcodeEi", "llvm::getMatchingCondBranchOpcode(int)"},
    {"_ZNK4llvm16ARMBaseInstrInfo17SubsumesPredicateERKNS_15SmallVectorImplINS_14MachineOperandEEES5_", "llvm::ARMBaseInstrInfo::SubsumesPredicate(llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::SmallVectorImpl<llvm::MachineOperand> const&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo16DefinesPredicateEPNS_12MachineInstrERSt6vectorINS_14MachineOperandESaIS4_EE", "llvm::ARMBaseInstrInfo::DefinesPredicate(llvm::MachineInstr*, std::vector<llvm::MachineOperand, std::allocator<llvm::MachineOperand> >&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo12isPredicableEPNS_12MachineInstrE", "llvm::ARMBaseInstrInfo::isPredicable(llvm::MachineInstr*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo18GetInstSizeInBytesEPKNS_12MachineInstrE", "llvm::ARMBaseInstrInfo::GetInstSizeInBytes(llvm::MachineInstr const*) const"},
    {"_Z15getNumJTEntriesRKSt6vectorIN4llvm21MachineJumpTableEntryESaIS1_EEj", "getNumJTEntries(std::vector<llvm::MachineJumpTableEntry, std::allocator<llvm::MachineJumpTableEntry> > const&, unsigned int)"},
    {"_ZNK4llvm16ARMBaseInstrInfo11copyPhysRegERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjb", "llvm::ARMBaseInstrInfo::copyPhysReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo19storeRegToStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjbiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::ARMBaseInstrInfo::storeRegToStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, bool, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo18isStoreToStackSlotEPKNS_12MachineInstrERi", "llvm::ARMBaseInstrInfo::isStoreToStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo20loadRegFromStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::ARMBaseInstrInfo::loadRegFromStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo19isLoadFromStackSlotEPKNS_12MachineInstrERi", "llvm::ARMBaseInstrInfo::isLoadFromStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo24emitFrameIndexDebugValueERNS_15MachineFunctionEiyPKNS_6MDNodeENS_8DebugLocE", "llvm::ARMBaseInstrInfo::emitFrameIndexDebugValue(llvm::MachineFunction&, int, unsigned long long, llvm::MDNode const*, llvm::DebugLoc) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo13reMaterializeERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjjPKS4_RKNS_18TargetRegisterInfoE", "llvm::ARMBaseInstrInfo::reMaterialize(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, unsigned int, llvm::MachineInstr const*, llvm::TargetRegisterInfo const&) const"},
    {"_Z12duplicateCPVRN4llvm15MachineFunctionERj", "duplicateCPV(llvm::MachineFunction&, unsigned int&)"},
    {"_ZNK4llvm16ARMBaseInstrInfo9duplicateEPNS_12MachineInstrERNS_15MachineFunctionE", "llvm::ARMBaseInstrInfo::duplicate(llvm::MachineInstr*, llvm::MachineFunction&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo16produceSameValueEPKNS_12MachineInstrES3_PKNS_19MachineRegisterInfoE", "llvm::ARMBaseInstrInfo::produceSameValue(llvm::MachineInstr const*, llvm::MachineInstr const*, llvm::MachineRegisterInfo const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo23areLoadsFromSameBasePtrEPNS_6SDNodeES2_RxS3_", "llvm::ARMBaseInstrInfo::areLoadsFromSameBasePtr(llvm::SDNode*, llvm::SDNode*, long long&, long long&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo23shouldScheduleLoadsNearEPNS_6SDNodeES2_xxj", "llvm::ARMBaseInstrInfo::shouldScheduleLoadsNear(llvm::SDNode*, llvm::SDNode*, long long, long long, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo20isSchedulingBoundaryEPKNS_12MachineInstrEPKNS_17MachineBasicBlockERKNS_15MachineFunctionE", "llvm::ARMBaseInstrInfo::isSchedulingBoundary(llvm::MachineInstr const*, llvm::MachineBasicBlock const*, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo19isProfitableToIfCvtERNS_17MachineBasicBlockEjjff", "llvm::ARMBaseInstrInfo::isProfitableToIfCvt(llvm::MachineBasicBlock&, unsigned int, unsigned int, float, float) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo19isProfitableToIfCvtERNS_17MachineBasicBlockEjjS2_jjff", "llvm::ARMBaseInstrInfo::isProfitableToIfCvt(llvm::MachineBasicBlock&, unsigned int, unsigned int, llvm::MachineBasicBlock&, unsigned int, unsigned int, float, float) const"},
    {"_ZN4llvm17getInstrPredicateEPKNS_12MachineInstrERj", "llvm::getInstrPredicate(llvm::MachineInstr const*, unsigned int&)"},
    {"_ZN4llvm23emitARMRegPlusImmediateERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjiNS_5ARMCC9CondCodesEjRKNS_16ARMBaseInstrInfoE", "llvm::emitARMRegPlusImmediate(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::DebugLoc, unsigned int, unsigned int, int, llvm::ARMCC::CondCodes, unsigned int, llvm::ARMBaseInstrInfo const&)"},
    {"_ZN4llvm20rewriteARMFrameIndexERNS_12MachineInstrEjjRiRKNS_16ARMBaseInstrInfoE", "llvm::rewriteARMFrameIndex(llvm::MachineInstr&, unsigned int, unsigned int, int&, llvm::ARMBaseInstrInfo const&)"},
    {"_ZNK4llvm16ARMBaseInstrInfo14AnalyzeCompareEPKNS_12MachineInstrERjRiS5_", "llvm::ARMBaseInstrInfo::AnalyzeCompare(llvm::MachineInstr const*, unsigned int&, int&, int&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo20OptimizeCompareInstrEPNS_12MachineInstrEjiiPKNS_19MachineRegisterInfoE", "llvm::ARMBaseInstrInfo::OptimizeCompareInstr(llvm::MachineInstr*, unsigned int, int, int, llvm::MachineRegisterInfo const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo13FoldImmediateEPNS_12MachineInstrES2_jPNS_19MachineRegisterInfoE", "llvm::ARMBaseInstrInfo::FoldImmediate(llvm::MachineInstr*, llvm::MachineInstr*, unsigned int, llvm::MachineRegisterInfo*) const"},
    {"_ZN4llvm6ARM_AM22getT2SOImmTwoPartFirstEj", "llvm::ARM_AM::getT2SOImmTwoPartFirst(unsigned int)"},
    {"_ZNK4llvm16ARMBaseInstrInfo14getNumMicroOpsEPKNS_18InstrItineraryDataEPKNS_12MachineInstrE", "llvm::ARMBaseInstrInfo::getNumMicroOps(llvm::InstrItineraryData const*, llvm::MachineInstr const*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo15getVLDMDefCycleEPKNS_18InstrItineraryDataERKNS_15TargetInstrDescEjjj", "llvm::ARMBaseInstrInfo::getVLDMDefCycle(llvm::InstrItineraryData const*, llvm::TargetInstrDesc const&, unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo14getLDMDefCycleEPKNS_18InstrItineraryDataERKNS_15TargetInstrDescEjjj", "llvm::ARMBaseInstrInfo::getLDMDefCycle(llvm::InstrItineraryData const*, llvm::TargetInstrDesc const&, unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo15getVSTMUseCycleEPKNS_18InstrItineraryDataERKNS_15TargetInstrDescEjjj", "llvm::ARMBaseInstrInfo::getVSTMUseCycle(llvm::InstrItineraryData const*, llvm::TargetInstrDesc const&, unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo14getSTMUseCycleEPKNS_18InstrItineraryDataERKNS_15TargetInstrDescEjjj", "llvm::ARMBaseInstrInfo::getSTMUseCycle(llvm::InstrItineraryData const*, llvm::TargetInstrDesc const&, unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo17getOperandLatencyEPKNS_18InstrItineraryDataERKNS_15TargetInstrDescEjjS6_jj", "llvm::ARMBaseInstrInfo::getOperandLatency(llvm::InstrItineraryData const*, llvm::TargetInstrDesc const&, unsigned int, unsigned int, llvm::TargetInstrDesc const&, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo17getOperandLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEjS6_j", "llvm::ARMBaseInstrInfo::getOperandLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo17getOperandLatencyEPKNS_18InstrItineraryDataEPNS_6SDNodeEjS5_j", "llvm::ARMBaseInstrInfo::getOperandLatency(llvm::InstrItineraryData const*, llvm::SDNode*, unsigned int, llvm::SDNode*, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo15getInstrLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEPj", "llvm::ARMBaseInstrInfo::getInstrLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo15getInstrLatencyEPKNS_18InstrItineraryDataEPNS_6SDNodeE", "llvm::ARMBaseInstrInfo::getInstrLatency(llvm::InstrItineraryData const*, llvm::SDNode*) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo21hasHighOperandLatencyEPKNS_18InstrItineraryDataEPKNS_19MachineRegisterInfoEPKNS_12MachineInstrEjS9_j", "llvm::ARMBaseInstrInfo::hasHighOperandLatency(llvm::InstrItineraryData const*, llvm::MachineRegisterInfo const*, llvm::MachineInstr const*, unsigned int, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo16hasLowDefLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEj", "llvm::ARMBaseInstrInfo::hasLowDefLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo18isFpMLxInstructionEjRjS1_RbS2_", "llvm::ARMBaseInstrInfo::isFpMLxInstruction(unsigned int, unsigned int&, unsigned int&, bool&, bool&) const"},
    {"_ZN4llvm8DenseMapIjjNS_12DenseMapInfoIjEES2_E6insertERKSt4pairIjjE", "llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<unsigned int, unsigned int> const&)"},
    {"_ZN4llvm8SmallSetIjLj16EE6insertERKj", "llvm::SmallSet<unsigned int, 16u>::insert(unsigned int const&)"},
    {"_ZN4llvm13LiveVariables7VarInfo10removeKillEPNS_12MachineInstrE", "llvm::LiveVariables::VarInfo::removeKill(llvm::MachineInstr*)"},
    {"_ZNSt6vectorIN4llvm14MachineOperandESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MachineOperand, std::allocator<llvm::MachineOperand> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineOperand*, std::vector<llvm::MachineOperand, std::allocator<llvm::MachineOperand> > >, llvm::MachineOperand const&)"},
    {"_ZNSt8_Rb_treeIjjSt9_IdentityIjESt4lessIjESaIjEE16_M_insert_uniqueERKj", "std::_Rb_tree<unsigned int, unsigned int, std::_Identity<unsigned int>, std::less<unsigned int>, std::allocator<unsigned int> >::_M_insert_unique(unsigned int const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm12MachineInstrESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, llvm::MachineInstr*>(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, __gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, llvm::MachineInstr* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm24ScheduleHazardRecognizer8EmitNoopEv", "llvm::ScheduleHazardRecognizer::EmitNoop()"},
    {"_ZN4llvm19ARMBaseRegisterInfoC1ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::ARMBaseRegisterInfo::ARMBaseRegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm19ARMBaseRegisterInfoC2ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::ARMBaseRegisterInfo::ARMBaseRegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm18ARMGenRegisterInfoC2Eii", "llvm::ARMGenRegisterInfo::ARMGenRegisterInfo(int, int)"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo15getReservedRegsERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getReservedRegs(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo14hasBasePointerERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::hasBasePointer(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo13isReservedRegERKNS_15MachineFunctionEj", "llvm::ARMBaseRegisterInfo::isReservedReg(llvm::MachineFunction const&, unsigned int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo24getMatchingSuperRegClassEPKNS_19TargetRegisterClassES3_j", "llvm::ARMBaseRegisterInfo::getMatchingSuperRegClass(llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*, unsigned int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo23canCombineSubRegIndicesEPKNS_19TargetRegisterClassERNS_15SmallVectorImplIjEERj", "llvm::ARMBaseRegisterInfo::canCombineSubRegIndices(llvm::TargetRegisterClass const*, llvm::SmallVectorImpl<unsigned int>&, unsigned int&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18getPointerRegClassEj", "llvm::ARMBaseRegisterInfo::getPointerRegClass(unsigned int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo19getRegisterPairEvenEjRKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getRegisterPairEven(unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18getRegisterPairOddEjRKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getRegisterPairOdd(unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo19ResolveRegAllocHintEjjRKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::ResolveRegAllocHint(unsigned int, unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18UpdateRegAllocHintEjjRNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::UpdateRegAllocHint(unsigned int, unsigned int, llvm::MachineFunction&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo15canRealignStackERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::canRealignStack(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo21needsStackRealignmentERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::needsStackRealignment(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo20cannotEliminateFrameERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::cannotEliminateFrame(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo13getRARegisterEv", "llvm::ARMBaseRegisterInfo::getRARegister() const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo16getFrameRegisterERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::getFrameRegister(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo22getEHExceptionRegisterEv", "llvm::ARMBaseRegisterInfo::getEHExceptionRegister() const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo20getEHHandlerRegisterEv", "llvm::ARMBaseRegisterInfo::getEHHandlerRegister() const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo14getDwarfRegNumEjb", "llvm::ARMBaseRegisterInfo::getDwarfRegNum(unsigned int, bool) const"},
    {"_ZNK4llvm18ARMGenRegisterInfo18getDwarfRegNumFullEjj", "llvm::ARMGenRegisterInfo::getDwarfRegNumFull(unsigned int, unsigned int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo17emitLoadConstPoolERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjiNS_5ARMCC9CondCodesEj", "llvm::ARMBaseRegisterInfo::emitLoadConstPool(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::DebugLoc, unsigned int, unsigned int, int, llvm::ARMCC::CondCodes, unsigned int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo26requiresRegisterScavengingERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::requiresRegisterScavenging(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo28requiresFrameIndexScavengingERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::requiresFrameIndexScavenging(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo28requiresVirtualBaseRegistersERKNS_15MachineFunctionE", "llvm::ARMBaseRegisterInfo::requiresVirtualBaseRegisters(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo29eliminateCallFramePseudoInstrERNS_15MachineFunctionERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::ARMBaseRegisterInfo::eliminateCallFramePseudoInstr(llvm::MachineFunction&, llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo24getFrameIndexInstrOffsetEPKNS_12MachineInstrEi", "llvm::ARMBaseRegisterInfo::getFrameIndexInstrOffset(llvm::MachineInstr const*, int) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo17needsFrameBaseRegEPNS_12MachineInstrEx", "llvm::ARMBaseRegisterInfo::needsFrameBaseReg(llvm::MachineInstr*, long long) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo28materializeFrameBaseRegisterEPNS_17MachineBasicBlockEjix", "llvm::ARMBaseRegisterInfo::materializeFrameBaseRegister(llvm::MachineBasicBlock*, unsigned int, int, long long) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo17resolveFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEjx", "llvm::ARMBaseRegisterInfo::resolveFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, long long) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo18isFrameOffsetLegalEPKNS_12MachineInstrEx", "llvm::ARMBaseRegisterInfo::isFrameOffsetLegal(llvm::MachineInstr const*, long long) const"},
    {"_ZNK4llvm19ARMBaseRegisterInfo19eliminateFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEiPNS_12RegScavengerE", "llvm::ARMBaseRegisterInfo::eliminateFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, int, llvm::RegScavenger*) const"},
    {"_ZN4llvm3ARM8CCRClassC1Ev", "llvm::ARM::CCRClass::CCRClass()"},
    {"_ZN4llvm3ARM8DPRClassC1Ev", "llvm::ARM::DPRClass::DPRClass()"},
    {"_ZN4llvm3ARM10DPR_8ClassC1Ev", "llvm::ARM::DPR_8Class::DPR_8Class()"},
    {"_ZN4llvm3ARM13DPR_VFP2ClassC1Ev", "llvm::ARM::DPR_VFP2Class::DPR_VFP2Class()"},
    {"_ZN4llvm3ARM8GPRClassC1Ev", "llvm::ARM::GPRClass::GPRClass()"},
    {"_ZN4llvm3ARM8QPRClassC1Ev", "llvm::ARM::QPRClass::QPRClass()"},
    {"_ZN4llvm3ARM10QPR_8ClassC1Ev", "llvm::ARM::QPR_8Class::QPR_8Class()"},
    {"_ZN4llvm3ARM13QPR_VFP2ClassC1Ev", "llvm::ARM::QPR_VFP2Class::QPR_VFP2Class()"},
    {"_ZN4llvm3ARM9QQPRClassC1Ev", "llvm::ARM::QQPRClass::QQPRClass()"},
    {"_ZN4llvm3ARM14QQPR_VFP2ClassC1Ev", "llvm::ARM::QQPR_VFP2Class::QQPR_VFP2Class()"},
    {"_ZN4llvm3ARM11QQQQPRClassC1Ev", "llvm::ARM::QQQQPRClass::QQQQPRClass()"},
    {"_ZN4llvm3ARM8SPRClassC1Ev", "llvm::ARM::SPRClass::SPRClass()"},
    {"_ZN4llvm3ARM10SPR_8ClassC1Ev", "llvm::ARM::SPR_8Class::SPR_8Class()"},
    {"_ZN4llvm3ARM9rGPRClassC1Ev", "llvm::ARM::rGPRClass::rGPRClass()"},
    {"_ZN4llvm3ARM9tGPRClassC1Ev", "llvm::ARM::tGPRClass::tGPRClass()"},
    {"_ZN4llvm3ARM10tcGPRClassC1Ev", "llvm::ARM::tcGPRClass::tcGPRClass()"},
    {"_ZN4llvm3ARM8CCRClassC2Ev", "llvm::ARM::CCRClass::CCRClass()"},
    {"_ZNK4llvm3ARM8DPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::DPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM8DPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::DPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM8DPRClassC2Ev", "llvm::ARM::DPRClass::DPRClass()"},
    {"_ZN4llvm3ARM10DPR_8ClassC2Ev", "llvm::ARM::DPR_8Class::DPR_8Class()"},
    {"_ZN4llvm3ARM13DPR_VFP2ClassC2Ev", "llvm::ARM::DPR_VFP2Class::DPR_VFP2Class()"},
    {"_ZNK4llvm3ARM8GPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::GPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM8GPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::GPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM8GPRClassC2Ev", "llvm::ARM::GPRClass::GPRClass()"},
    {"_ZNK4llvm3ARM8QPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::QPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM8QPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::QPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM8QPRClassC2Ev", "llvm::ARM::QPRClass::QPRClass()"},
    {"_ZN4llvm3ARM10QPR_8ClassC2Ev", "llvm::ARM::QPR_8Class::QPR_8Class()"},
    {"_ZN4llvm3ARM13QPR_VFP2ClassC2Ev", "llvm::ARM::QPR_VFP2Class::QPR_VFP2Class()"},
    {"_ZNK4llvm3ARM9QQPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::QQPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM9QQPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::QQPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM9QQPRClassC2Ev", "llvm::ARM::QQPRClass::QQPRClass()"},
    {"_ZN4llvm3ARM14QQPR_VFP2ClassC2Ev", "llvm::ARM::QQPR_VFP2Class::QQPR_VFP2Class()"},
    {"_ZNK4llvm3ARM11QQQQPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::QQQQPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM11QQQQPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::QQQQPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM11QQQQPRClassC2Ev", "llvm::ARM::QQQQPRClass::QQQQPRClass()"},
    {"_ZN4llvm3ARM8SPRClassC2Ev", "llvm::ARM::SPRClass::SPRClass()"},
    {"_ZN4llvm3ARM10SPR_8ClassC2Ev", "llvm::ARM::SPR_8Class::SPR_8Class()"},
    {"_ZNK4llvm3ARM9rGPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::rGPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM9rGPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::rGPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM9rGPRClassC2Ev", "llvm::ARM::rGPRClass::rGPRClass()"},
    {"_ZN4llvm3ARM9tGPRClassC2Ev", "llvm::ARM::tGPRClass::tGPRClass()"},
    {"_ZNK4llvm3ARM10tcGPRClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::ARM::tcGPRClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm3ARM10tcGPRClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::ARM::tcGPRClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM10tcGPRClassC2Ev", "llvm::ARM::tcGPRClass::tcGPRClass()"},
    {"_ZNK4llvm18ARMGenRegisterInfo9getSubRegEjj", "llvm::ARMGenRegisterInfo::getSubReg(unsigned int, unsigned int) const"},
    {"_ZNK4llvm18ARMGenRegisterInfo14getSubRegIndexEjj", "llvm::ARMGenRegisterInfo::getSubRegIndex(unsigned int, unsigned int) const"},
    {"_ZNK4llvm18ARMGenRegisterInfo20composeSubRegIndicesEjj", "llvm::ARMGenRegisterInfo::composeSubRegIndices(unsigned int, unsigned int) const"},
    {"_ZN4llvm18ARMGenRegisterInfoC1Eii", "llvm::ARMGenRegisterInfo::ARMGenRegisterInfo(int, int)"},
    {"_ZN4llvm3ARM8CCRClassD1Ev", "llvm::ARM::CCRClass::~CCRClass()"},
    {"_ZN4llvm3ARM8DPRClassD1Ev", "llvm::ARM::DPRClass::~DPRClass()"},
    {"_ZN4llvm3ARM10DPR_8ClassD1Ev", "llvm::ARM::DPR_8Class::~DPR_8Class()"},
    {"_ZN4llvm3ARM13DPR_VFP2ClassD1Ev", "llvm::ARM::DPR_VFP2Class::~DPR_VFP2Class()"},
    {"_ZN4llvm3ARM8GPRClassD1Ev", "llvm::ARM::GPRClass::~GPRClass()"},
    {"_ZN4llvm3ARM8QPRClassD1Ev", "llvm::ARM::QPRClass::~QPRClass()"},
    {"_ZN4llvm3ARM10QPR_8ClassD1Ev", "llvm::ARM::QPR_8Class::~QPR_8Class()"},
    {"_ZN4llvm3ARM13QPR_VFP2ClassD1Ev", "llvm::ARM::QPR_VFP2Class::~QPR_VFP2Class()"},
    {"_ZN4llvm3ARM9QQPRClassD1Ev", "llvm::ARM::QQPRClass::~QQPRClass()"},
    {"_ZN4llvm3ARM14QQPR_VFP2ClassD1Ev", "llvm::ARM::QQPR_VFP2Class::~QQPR_VFP2Class()"},
    {"_ZN4llvm3ARM11QQQQPRClassD1Ev", "llvm::ARM::QQQQPRClass::~QQQQPRClass()"},
    {"_ZN4llvm3ARM8SPRClassD1Ev", "llvm::ARM::SPRClass::~SPRClass()"},
    {"_ZN4llvm3ARM10SPR_8ClassD1Ev", "llvm::ARM::SPR_8Class::~SPR_8Class()"},
    {"_ZN4llvm3ARM9rGPRClassD1Ev", "llvm::ARM::rGPRClass::~rGPRClass()"},
    {"_ZN4llvm3ARM9tGPRClassD1Ev", "llvm::ARM::tGPRClass::~tGPRClass()"},
    {"_ZN4llvm3ARM10tcGPRClassD1Ev", "llvm::ARM::tcGPRClass::~tcGPRClass()"},
    {"_ZN4llvm19TargetRegisterClassC2EjPKcPKNS_3EVTEPKPKS0_S9_S9_S9_jjiPKjSB_", "llvm::TargetRegisterClass::TargetRegisterClass(unsigned int, char const*, llvm::EVT const*, llvm::TargetRegisterClass const* const*, llvm::TargetRegisterClass const* const*, llvm::TargetRegisterClass const* const*, llvm::TargetRegisterClass const* const*, unsigned int, unsigned int, int, unsigned int const*, unsigned int const*)"},
    {"_ZN4llvm3ARM8DPRClassD0Ev", "llvm::ARM::DPRClass::~DPRClass()"},
    {"_ZN4llvm3ARM8GPRClassD0Ev", "llvm::ARM::GPRClass::~GPRClass()"},
    {"_ZN4llvm3ARM8QPRClassD0Ev", "llvm::ARM::QPRClass::~QPRClass()"},
    {"_ZN4llvm3ARM9QQPRClassD0Ev", "llvm::ARM::QQPRClass::~QQPRClass()"},
    {"_ZN4llvm3ARM11QQQQPRClassD0Ev", "llvm::ARM::QQQQPRClass::~QQQQPRClass()"},
    {"_ZN4llvm3ARM9rGPRClassD0Ev", "llvm::ARM::rGPRClass::~rGPRClass()"},
    {"_ZN4llvm3ARM10tcGPRClassD0Ev", "llvm::ARM::tcGPRClass::~tcGPRClass()"},
    {"_ZN4llvm3ARM9tGPRClassD0Ev", "llvm::ARM::tGPRClass::~tGPRClass()"},
    {"_ZNK4llvm19TargetRegisterClass22allocation_order_beginERKNS_15MachineFunctionE", "llvm::TargetRegisterClass::allocation_order_begin(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19TargetRegisterClass20allocation_order_endERKNS_15MachineFunctionE", "llvm::TargetRegisterClass::allocation_order_end(llvm::MachineFunction const&) const"},
    {"_ZN4llvm3ARM10SPR_8ClassD0Ev", "llvm::ARM::SPR_8Class::~SPR_8Class()"},
    {"_ZN4llvm3ARM8SPRClassD0Ev", "llvm::ARM::SPRClass::~SPRClass()"},
    {"_ZN4llvm3ARM14QQPR_VFP2ClassD0Ev", "llvm::ARM::QQPR_VFP2Class::~QQPR_VFP2Class()"},
    {"_ZN4llvm3ARM13QPR_VFP2ClassD0Ev", "llvm::ARM::QPR_VFP2Class::~QPR_VFP2Class()"},
    {"_ZN4llvm3ARM10QPR_8ClassD0Ev", "llvm::ARM::QPR_8Class::~QPR_8Class()"},
    {"_ZN4llvm3ARM13DPR_VFP2ClassD0Ev", "llvm::ARM::DPR_VFP2Class::~DPR_VFP2Class()"},
    {"_ZN4llvm3ARM10DPR_8ClassD0Ev", "llvm::ARM::DPR_8Class::~DPR_8Class()"},
    {"_ZN4llvm3ARM8CCRClassD0Ev", "llvm::ARM::CCRClass::~CCRClass()"},
    {"_ZN4llvm19TargetRegisterClassD1Ev", "llvm::TargetRegisterClass::~TargetRegisterClass()"},
    {"_ZN4llvm19TargetRegisterClassD0Ev", "llvm::TargetRegisterClass::~TargetRegisterClass()"},
    {"_ZN4llvm27createARMJITCodeEmitterPassERNS_20ARMBaseTargetMachineERNS_14JITCodeEmitterE", "llvm::createARMJITCodeEmitterPass(llvm::ARMBaseTargetMachine&, llvm::JITCodeEmitter&)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitterD1Ev", "(anonymous namespace)::ARMCodeEmitter::~ARMCodeEmitter()"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitterD0Ev", "(anonymous namespace)::ARMCodeEmitter::~ARMCodeEmitter()"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter11getPassNameEv", "(anonymous namespace)::ARMCodeEmitter::getPassName() const"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::ARMCodeEmitter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ARMCodeEmitter::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter15emitInstructionERKN4llvm12MachineInstrE", "(anonymous namespace)::ARMCodeEmitter::emitInstruction(llvm::MachineInstr const&)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter25emitLEApcrelJTInstructionERKN4llvm12MachineInstrE", "(anonymous namespace)::ARMCodeEmitter::emitLEApcrelJTInstruction(llvm::MachineInstr const&)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter29emitDataProcessingInstructionERKN4llvm12MachineInstrEjj", "(anonymous namespace)::ARMCodeEmitter::emitDataProcessingInstruction(llvm::MachineInstr const&, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter24emitLoadStoreInstructionERKN4llvm12MachineInstrEjj", "(anonymous namespace)::ARMCodeEmitter::emitLoadStoreInstruction(llvm::MachineInstr const&, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter28emitMiscLoadStoreInstructionERKN4llvm12MachineInstrEj", "(anonymous namespace)::ARMCodeEmitter::emitMiscLoadStoreInstruction(llvm::MachineInstr const&, unsigned int)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter25emitMiscBranchInstructionERKN4llvm12MachineInstrE", "(anonymous namespace)::ARMCodeEmitter::emitMiscBranchInstruction(llvm::MachineInstr const&)"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter21getBinaryCodeForInstrERKN4llvm12MachineInstrE", "(anonymous namespace)::ARMCodeEmitter::getBinaryCodeForInstr(llvm::MachineInstr const&) const"},
    {"_Z12encodeNEONRdRKN4llvm12MachineInstrEj", "encodeNEONRd(llvm::MachineInstr const&, unsigned int)"},
    {"_Z12encodeNEONRnRKN4llvm12MachineInstrEj", "encodeNEONRn(llvm::MachineInstr const&, unsigned int)"},
    {"_Z12encodeNEONRmRKN4llvm12MachineInstrEj", "encodeNEONRm(llvm::MachineInstr const&, unsigned int)"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter17getMachineOpValueERKN4llvm12MachineInstrERKNS1_14MachineOperandE", "(anonymous namespace)::ARMCodeEmitter::getMachineOpValue(llvm::MachineInstr const&, llvm::MachineOperand const&) const"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter23getAddrModeImm12OpValueERKN4llvm12MachineInstrEj", "(anonymous namespace)::ARMCodeEmitter::getAddrModeImm12OpValue(llvm::MachineInstr const&, unsigned int) const"},
    {"_ZNK12_GLOBAL__N_114ARMCodeEmitter19getAddrMode5OpValueERKN4llvm12MachineInstrEj", "(anonymous namespace)::ARMCodeEmitter::getAddrMode5OpValue(llvm::MachineInstr const&, unsigned int) const"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter19emitInlineJumpTableEj", "(anonymous namespace)::ARMCodeEmitter::emitInlineJumpTable(unsigned int)"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter14getMovi32ValueERKN4llvm12MachineInstrERKNS1_14MachineOperandEj", "(anonymous namespace)::ARMCodeEmitter::getMovi32Value(llvm::MachineInstr const&, llvm::MachineOperand const&, unsigned int)"},
    {"_ZN4llvm8DenseMapIjlNS_12DenseMapInfoIjEENS1_IlEEE6insertERKSt4pairIjlE", "llvm::DenseMap<unsigned int, long, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<long> >::insert(std::pair<unsigned int, long> const&)"},
    {"_ZN4llvm8DenseMapIjlNS_12DenseMapInfoIjEENS1_IlEEE16InsertIntoBucketERKjRKlPSt4pairIjlE", "llvm::DenseMap<unsigned int, long, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<long> >::InsertIntoBucket(unsigned int const&, long const&, std::pair<unsigned int, long>*)"},
    {"_ZN4llvm8DenseMapIjlNS_12DenseMapInfoIjEENS1_IlEEE4growEj", "llvm::DenseMap<unsigned int, long, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<long> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIlE6resizeEj", "llvm::SmallVectorImpl<long>::resize(unsigned int)"},
    {"_ZN4llvm27createARMConstantIslandPassEv", "llvm::createARMConstantIslandPass()"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslandsD1Ev", "(anonymous namespace)::ARMConstantIslands::~ARMConstantIslands()"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslandsD0Ev", "(anonymous namespace)::ARMConstantIslands::~ARMConstantIslands()"},
    {"_ZNK12_GLOBAL__N_118ARMConstantIslands11getPassNameEv", "(anonymous namespace)::ARMConstantIslands::getPassName() const"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ARMConstantIslands::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands12CPEIsInRangeEPN4llvm12MachineInstrEjS3_jbb", "(anonymous namespace)::ARMConstantIslands::CPEIsInRange(llvm::MachineInstr*, unsigned int, llvm::MachineInstr*, unsigned int, bool, bool)"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands20AdjustBBOffsetsAfterEPN4llvm17MachineBasicBlockEi", "(anonymous namespace)::ARMConstantIslands::AdjustBBOffsetsAfter(llvm::MachineBasicBlock*, int)"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands21SplitBlockBeforeInstrEPN4llvm12MachineInstrE", "(anonymous namespace)::ARMConstantIslands::SplitBlockBeforeInstr(llvm::MachineInstr*)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_118ARMConstantIslands9ImmBranchESaIS2_EE9push_backERKS2_", "std::vector<(anonymous namespace)::ARMConstantIslands::ImmBranch, std::allocator<(anonymous namespace)::ARMConstantIslands::ImmBranch> >::push_back((anonymous namespace)::ARMConstantIslands::ImmBranch const&)"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands17DecrementOldEntryEjPN4llvm12MachineInstrE", "(anonymous namespace)::ARMConstantIslands::DecrementOldEntry(unsigned int, llvm::MachineInstr*)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_118ARMConstantIslands7CPEntryESaIS2_EE9push_backERKS2_", "std::vector<(anonymous namespace)::ARMConstantIslands::CPEntry, std::allocator<(anonymous namespace)::ARMConstantIslands::CPEntry> >::push_back((anonymous namespace)::ARMConstantIslands::CPEntry const&)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_118ARMConstantIslands7CPEntryESaIS2_EEC1ERKS4_", "std::vector<(anonymous namespace)::ARMConstantIslands::CPEntry, std::allocator<(anonymous namespace)::ARMConstantIslands::CPEntry> >::vector(std::vector<(anonymous namespace)::ARMConstantIslands::CPEntry, std::allocator<(anonymous namespace)::ARMConstantIslands::CPEntry> > const&)"},
    {"_ZNSt6vectorIPN4llvm17MachineBasicBlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, llvm::MachineBasicBlock* const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, llvm::MachineBasicBlock*>(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, llvm::MachineBasicBlock* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm20ARMConstantPoolValueC1EPKNS_8ConstantEjNS_5ARMCP9ARMCPKindEhNS4_13ARMCPModifierEb", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::Constant const*, unsigned int, llvm::ARMCP::ARMCPKind, unsigned char, llvm::ARMCP::ARMCPModifier, bool)"},
    {"_ZN4llvm20ARMConstantPoolValueC2EPKNS_8ConstantEjNS_5ARMCP9ARMCPKindEhNS4_13ARMCPModifierEb", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::Constant const*, unsigned int, llvm::ARMCP::ARMCPKind, unsigned char, llvm::ARMCP::ARMCPModifier, bool)"},
    {"_ZN4llvm20ARMConstantPoolValueC1ERNS_11LLVMContextEPKcjhNS_5ARMCP13ARMCPModifierEb", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::LLVMContext&, char const*, unsigned int, unsigned char, llvm::ARMCP::ARMCPModifier, bool)"},
    {"_ZN4llvm20ARMConstantPoolValueC2ERNS_11LLVMContextEPKcjhNS_5ARMCP13ARMCPModifierEb", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::LLVMContext&, char const*, unsigned int, unsigned char, llvm::ARMCP::ARMCPModifier, bool)"},
    {"_ZN4llvm20ARMConstantPoolValueC1EPKNS_11GlobalValueENS_5ARMCP13ARMCPModifierE", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::GlobalValue const*, llvm::ARMCP::ARMCPModifier)"},
    {"_ZN4llvm20ARMConstantPoolValueC2EPKNS_11GlobalValueENS_5ARMCP13ARMCPModifierE", "llvm::ARMConstantPoolValue::ARMConstantPoolValue(llvm::GlobalValue const*, llvm::ARMCP::ARMCPModifier)"},
    {"_ZNK4llvm20ARMConstantPoolValue5getGVEv", "llvm::ARMConstantPoolValue::getGV() const"},
    {"_ZNK4llvm20ARMConstantPoolValue15getBlockAddressEv", "llvm::ARMConstantPoolValue::getBlockAddress() const"},
    {"_ZN4llvm20ARMConstantPoolValue25getExistingMachineCPValueEPNS_19MachineConstantPoolEj", "llvm::ARMConstantPoolValue::getExistingMachineCPValue(llvm::MachineConstantPool*, unsigned int)"},
    {"_ZN4llvm20ARMConstantPoolValueD0Ev", "llvm::ARMConstantPoolValue::~ARMConstantPoolValue()"},
    {"_ZN4llvm20ARMConstantPoolValueD1Ev", "llvm::ARMConstantPoolValue::~ARMConstantPoolValue()"},
    {"_ZN4llvm20ARMConstantPoolValueD2Ev", "llvm::ARMConstantPoolValue::~ARMConstantPoolValue()"},
    {"_ZN4llvm20ARMConstantPoolValue20AddSelectionDAGCSEIdERNS_16FoldingSetNodeIDE", "llvm::ARMConstantPoolValue::AddSelectionDAGCSEId(llvm::FoldingSetNodeID&)"},
    {"_ZN4llvm20ARMConstantPoolValue12hasSameValueEPS0_", "llvm::ARMConstantPoolValue::hasSameValue(llvm::ARMConstantPoolValue*)"},
    {"_ZNK4llvm20ARMConstantPoolValue4dumpEv", "llvm::ARMConstantPoolValue::dump() const"},
    {"_ZNK4llvm20ARMConstantPoolValue5printERNS_11raw_ostreamE", "llvm::ARMConstantPoolValue::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm20ARMConstantPoolValue17getRelocationInfoEv", "llvm::ARMConstantPoolValue::getRelocationInfo() const"},
    {"_ZNSt6vectorIN4llvm24MachineConstantPoolEntryESaIS1_EEC2ERKS3_", "std::vector<llvm::MachineConstantPoolEntry, std::allocator<llvm::MachineConstantPoolEntry> >::vector(std::vector<llvm::MachineConstantPoolEntry, std::allocator<llvm::MachineConstantPoolEntry> > const&)"},
    {"_ZN4llvm24MachineConstantPoolValueD1Ev", "llvm::MachineConstantPoolValue::~MachineConstantPoolValue()"},
    {"_ZN4llvm24MachineConstantPoolValueD0Ev", "llvm::MachineConstantPoolValue::~MachineConstantPoolValue()"},
    {"_ZN4llvm16ARMELFWriterInfoC1ERNS_13TargetMachineE", "llvm::ARMELFWriterInfo::ARMELFWriterInfo(llvm::TargetMachine&)"},
    {"_ZN4llvm16ARMELFWriterInfoC2ERNS_13TargetMachineE", "llvm::ARMELFWriterInfo::ARMELFWriterInfo(llvm::TargetMachine&)"},
    {"_ZN4llvm16ARMELFWriterInfoD0Ev", "llvm::ARMELFWriterInfo::~ARMELFWriterInfo()"},
    {"_ZN4llvm16ARMELFWriterInfoD1Ev", "llvm::ARMELFWriterInfo::~ARMELFWriterInfo()"},
    {"_ZN4llvm16ARMELFWriterInfoD2Ev", "llvm::ARMELFWriterInfo::~ARMELFWriterInfo()"},
    {"_ZNK4llvm16ARMELFWriterInfo17getRelocationTypeEj", "llvm::ARMELFWriterInfo::getRelocationType(unsigned int) const"},
    {"_ZNK4llvm16ARMELFWriterInfo24getDefaultAddendForRelTyEjl", "llvm::ARMELFWriterInfo::getDefaultAddendForRelTy(unsigned int, long) const"},
    {"_ZNK4llvm16ARMELFWriterInfo19getRelocationTySizeEj", "llvm::ARMELFWriterInfo::getRelocationTySize(unsigned int) const"},
    {"_ZNK4llvm16ARMELFWriterInfo15isPCRelativeRelEj", "llvm::ARMELFWriterInfo::isPCRelativeRel(unsigned int) const"},
    {"_ZNK4llvm16ARMELFWriterInfo28getAbsoluteLabelMachineRelTyEv", "llvm::ARMELFWriterInfo::getAbsoluteLabelMachineRelTy() const"},
    {"_ZNK4llvm16ARMELFWriterInfo17computeRelocationEjjj", "llvm::ARMELFWriterInfo::computeRelocation(unsigned int, unsigned int, unsigned int) const"},
    {"_ZNK4llvm16ARMELFWriterInfo19hasRelocationAddendEv", "llvm::ARMELFWriterInfo::hasRelocationAddend() const"},
    {"_ZN4llvm25createARMExpandPseudoPassEv", "llvm::createARMExpandPseudoPass()"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudoD1Ev", "(anonymous namespace)::ARMExpandPseudo::~ARMExpandPseudo()"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudoD0Ev", "(anonymous namespace)::ARMExpandPseudo::~ARMExpandPseudo()"},
    {"_ZNK12_GLOBAL__N_115ARMExpandPseudo11getPassNameEv", "(anonymous namespace)::ARMExpandPseudo::getPassName() const"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudo20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ARMExpandPseudo::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudo8ExpandMIERN4llvm17MachineBasicBlockENS1_14ilist_iteratorINS1_12MachineInstrEEE", "(anonymous namespace)::ARMExpandPseudo::ExpandMI(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudo10ExpandVTBLERN4llvm14ilist_iteratorINS1_12MachineInstrEEEjbj", "(anonymous namespace)::ARMExpandPseudo::ExpandVTBL(llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, bool, unsigned int)"},
    {"_Z11GetDSubRegsjN12_GLOBAL__N_114NEONRegSpacingEPKN4llvm18TargetRegisterInfoERjS5_S5_S5_", "GetDSubRegs(unsigned int, (anonymous namespace)::NEONRegSpacing, llvm::TargetRegisterInfo const*, unsigned int&, unsigned int&, unsigned int&, unsigned int&)"},
    {"_ZN4llvm3ARM14createFastISelERNS_20FunctionLoweringInfoE", "llvm::ARM::createFastISel(llvm::FunctionLoweringInfo&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISelD1Ev", "(anonymous namespace)::ARMFastISel::~ARMFastISel()"},
    {"_ZN12_GLOBAL__N_111ARMFastISelD0Ev", "(anonymous namespace)::ARMFastISel::~ARMFastISel()"},
    {"_ZN12_GLOBAL__N_111ARMFastISel23TargetSelectInstructionEPKN4llvm11InstructionE", "(anonymous namespace)::ARMFastISel::TargetSelectInstruction(llvm::Instruction const*)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel10FastEmit_rEN4llvm3MVTES2_jjb", "(anonymous namespace)::ARMFastISel::FastEmit_r(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel11FastEmit_rrEN4llvm3MVTES2_jjbjb", "(anonymous namespace)::ARMFastISel::FastEmit_rr(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel11FastEmit_riEN4llvm3MVTES2_jjby", "(anonymous namespace)::ARMFastISel::FastEmit_ri(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel10FastEmit_iEN4llvm3MVTES2_jy", "(anonymous namespace)::ARMFastISel::FastEmit_i(llvm::MVT, llvm::MVT, unsigned int, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel25TargetMaterializeConstantEPKN4llvm8ConstantE", "(anonymous namespace)::ARMFastISel::TargetMaterializeConstant(llvm::Constant const*)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel23TargetMaterializeAllocaEPKN4llvm10AllocaInstE", "(anonymous namespace)::ARMFastISel::TargetMaterializeAlloca(llvm::AllocaInst const*)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel13FastEmitInst_EjPKN4llvm19TargetRegisterClassE", "(anonymous namespace)::ARMFastISel::FastEmitInst_(unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel14FastEmitInst_rEjPKN4llvm19TargetRegisterClassEjb", "(anonymous namespace)::ARMFastISel::FastEmitInst_r(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel15FastEmitInst_rrEjPKN4llvm19TargetRegisterClassEjbjb", "(anonymous namespace)::ARMFastISel::FastEmitInst_rr(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel15FastEmitInst_riEjPKN4llvm19TargetRegisterClassEjby", "(anonymous namespace)::ARMFastISel::FastEmitInst_ri(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel15FastEmitInst_rfEjPKN4llvm19TargetRegisterClassEjbPKNS1_10ConstantFPE", "(anonymous namespace)::ARMFastISel::FastEmitInst_rf(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, llvm::ConstantFP const*)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel14FastEmitInst_iEjPKN4llvm19TargetRegisterClassEy", "(anonymous namespace)::ARMFastISel::FastEmitInst_i(unsigned int, llvm::TargetRegisterClass const*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel16FastEmitInst_rriEjPKN4llvm19TargetRegisterClassEjbjby", "(anonymous namespace)::ARMFastISel::FastEmitInst_rri(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned int, bool, unsigned long long)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel26FastEmitInst_extractsubregEN4llvm3MVTEjbj", "(anonymous namespace)::ARMFastISel::FastEmitInst_extractsubreg(llvm::MVT, unsigned int, bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel15AddOptionalDefsERKN4llvm19MachineInstrBuilderE", "(anonymous namespace)::ARMFastISel::AddOptionalDefs(llvm::MachineInstrBuilder const&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel16ARMMaterializeGVEPKN4llvm11GlobalValueENS1_3EVTE", "(anonymous namespace)::ARMFastISel::ARMMaterializeGV(llvm::GlobalValue const*, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel14SelectBinaryOpEPKN4llvm11InstructionEj", "(anonymous namespace)::ARMFastISel::SelectBinaryOp(llvm::Instruction const*, unsigned int)"},
    {"_Z19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z15RetCC_ARM_AAPCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z12CC_ARM_AAPCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z11CC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm13f64AssignAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEb", "llvm::f64AssignAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)"},
    {"_ZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateE", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)"},
    {"_Z19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEb", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)"},
    {"_Z22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel15ProcessCallArgsERN4llvm15SmallVectorImplIPNS1_5ValueEEERNS2_IjEERNS2_INS1_3MVTEEERNS2_INS1_3ISD10ArgFlagsTyEEES8_NS1_11CallingConv2IDERj", "(anonymous namespace)::ARMFastISel::ProcessCallArgs(llvm::SmallVectorImpl<llvm::Value*>&, llvm::SmallVectorImpl<unsigned int>&, llvm::SmallVectorImpl<llvm::MVT>&, llvm::SmallVectorImpl<llvm::ISD::ArgFlagsTy>&, llvm::SmallVectorImpl<unsigned int>&, llvm::CallingConv::ID, unsigned int&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel10FinishCallEN4llvm3MVTERNS1_15SmallVectorImplIjEEPKNS1_11InstructionENS1_11CallingConv2IDERj", "(anonymous namespace)::ARMFastISel::FinishCall(llvm::MVT, llvm::SmallVectorImpl<unsigned int>&, llvm::Instruction const*, llvm::CallingConv::ID, unsigned int&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel12ARMEmitStoreEN4llvm3EVTEjRNS_7AddressE", "(anonymous namespace)::ARMFastISel::ARMEmitStore(llvm::EVT, unsigned int, (anonymous namespace)::Address&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel18ARMSimplifyAddressERNS_7AddressEN4llvm3EVTE", "(anonymous namespace)::ARMFastISel::ARMSimplifyAddress((anonymous namespace)::Address&, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel20AddLoadStoreOperandsEN4llvm3EVTERNS_7AddressERKNS1_19MachineInstrBuilderE", "(anonymous namespace)::ARMFastISel::AddLoadStoreOperands(llvm::EVT, (anonymous namespace)::Address&, llvm::MachineInstrBuilder const&)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel14ARMEmitLibcallEPKN4llvm11InstructionENS1_5RTLIB7LibcallE", "(anonymous namespace)::ARMFastISel::ARMEmitLibcall(llvm::Instruction const*, llvm::RTLIB::Libcall)"},
    {"_ZN12_GLOBAL__N_111ARMFastISel17ARMComputeAddressEPKN4llvm5ValueERNS_7AddressE", "(anonymous namespace)::ARMFastISel::ARMComputeAddress(llvm::Value const*, (anonymous namespace)::Address&)"},
    {"_ZN4llvm8FastISel13TryToFoldLoadEPNS_12MachineInstrEjPKNS_8LoadInstE", "llvm::FastISel::TryToFoldLoad(llvm::MachineInstr*, unsigned int, llvm::LoadInst const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_3ISD10ArgFlagsTyELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::ISD::ArgFlagsTy, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_3MVTELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::MVT, false>::grow(unsigned long)"},
    {"_ZNK4llvm16ARMFrameLowering5hasFPERKNS_15MachineFunctionE", "llvm::ARMFrameLowering::hasFP(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16ARMFrameLowering20hasReservedCallFrameERKNS_15MachineFunctionE", "llvm::ARMFrameLowering::hasReservedCallFrame(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16ARMFrameLowering27canSimplifyCallFramePseudosERKNS_15MachineFunctionE", "llvm::ARMFrameLowering::canSimplifyCallFramePseudos(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm16ARMFrameLowering12emitPrologueERNS_15MachineFunctionE", "llvm::ARMFrameLowering::emitPrologue(llvm::MachineFunction&) const"},
    {"_ZNK4llvm16ARMFrameLowering12emitEpilogueERNS_15MachineFunctionERNS_17MachineBasicBlockE", "llvm::ARMFrameLowering::emitEpilogue(llvm::MachineFunction&, llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm16ARMFrameLowering22getFrameIndexReferenceERKNS_15MachineFunctionEiRj", "llvm::ARMFrameLowering::getFrameIndexReference(llvm::MachineFunction const&, int, unsigned int&) const"},
    {"_ZNK4llvm16ARMFrameLowering26ResolveFrameIndexReferenceERKNS_15MachineFunctionEiRji", "llvm::ARMFrameLowering::ResolveFrameIndexReference(llvm::MachineFunction const&, int, unsigned int&, int) const"},
    {"_ZNK4llvm16ARMFrameLowering19getFrameIndexOffsetERKNS_15MachineFunctionEi", "llvm::ARMFrameLowering::getFrameIndexOffset(llvm::MachineFunction const&, int) const"},
    {"_ZNK4llvm16ARMFrameLowering12emitPushInstERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEjjbPFbjbE", "llvm::ARMFrameLowering::emitPushInst(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, unsigned int, unsigned int, bool, bool (*)(unsigned int, bool)) const"},
    {"_ZNK4llvm16ARMFrameLowering11emitPopInstERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEjjbbPFbjbE", "llvm::ARMFrameLowering::emitPopInst(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, unsigned int, unsigned int, bool, bool, bool (*)(unsigned int, bool)) const"},
    {"_ZNK4llvm16ARMFrameLowering25spillCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::ARMFrameLowering::spillCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm18isARMArea1RegisterEjb", "llvm::isARMArea1Register(unsigned int, bool)"},
    {"_ZN4llvm18isARMArea2RegisterEjb", "llvm::isARMArea2Register(unsigned int, bool)"},
    {"_ZN4llvm18isARMArea3RegisterEjb", "llvm::isARMArea3Register(unsigned int, bool)"},
    {"_ZNK4llvm16ARMFrameLowering27restoreCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::ARMFrameLowering::restoreCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm16ARMFrameLowering36processFunctionBeforeCalleeSavedScanERNS_15MachineFunctionEPNS_12RegScavengerE", "llvm::ARMFrameLowering::processFunctionBeforeCalleeSavedScan(llvm::MachineFunction&, llvm::RegScavenger*) const"},
    {"_ZSt6__findIPjjET_S1_S1_RKT0_St26random_access_iterator_tag", "unsigned int* std::__find<unsigned int*, unsigned int>(unsigned int*, unsigned int*, unsigned int const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIN4llvm16MachineFrameInfo11StackObjectESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::MachineFrameInfo::StackObject, std::allocator<llvm::MachineFrameInfo::StackObject> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineFrameInfo::StackObject*, std::vector<llvm::MachineFrameInfo::StackObject, std::allocator<llvm::MachineFrameInfo::StackObject> > >, llvm::MachineFrameInfo::StackObject const&)"},
    {"_ZN4llvm24createARMGlobalMergePassEPKNS_14TargetLoweringE", "llvm::createARMGlobalMergePass(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_114ARMGlobalMergeD1Ev", "(anonymous namespace)::ARMGlobalMerge::~ARMGlobalMerge()"},
    {"_ZN12_GLOBAL__N_114ARMGlobalMergeD0Ev", "(anonymous namespace)::ARMGlobalMerge::~ARMGlobalMerge()"},
    {"_ZNK12_GLOBAL__N_114ARMGlobalMerge11getPassNameEv", "(anonymous namespace)::ARMGlobalMerge::getPassName() const"},
    {"_ZNK12_GLOBAL__N_114ARMGlobalMerge16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::ARMGlobalMerge::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114ARMGlobalMerge16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::ARMGlobalMerge::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_114ARMGlobalMerge13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::ARMGlobalMerge::runOnFunction(llvm::Function&)"},
    {"_ZNK12_GLOBAL__N_114ARMGlobalMerge7doMergeERN4llvm15SmallVectorImplIPNS1_14GlobalVariableEEERNS1_6ModuleEb", "(anonymous namespace)::ARMGlobalMerge::doMerge(llvm::SmallVectorImpl<llvm::GlobalVariable*>&, llvm::Module&, bool) const"},
    {"_ZSt21__inplace_stable_sortIPPN4llvm14GlobalVariableEN12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_T0_", "void std::__inplace_stable_sort<llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt22__stable_sort_adaptiveIPPN4llvm14GlobalVariableES3_lN12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_T0_T1_T2_", "void std::__stable_sort_adaptive<llvm::GlobalVariable**, llvm::GlobalVariable**, long, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, llvm::GlobalVariable**, long, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt24__merge_sort_with_bufferIPPN4llvm14GlobalVariableES3_N12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_T0_T1_", "void std::__merge_sort_with_buffer<llvm::GlobalVariable**, llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt16__merge_adaptiveIPPN4llvm14GlobalVariableElS3_N12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_S7_T0_S8_T1_S8_T2_", "void std::__merge_adaptive<llvm::GlobalVariable**, long, llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, llvm::GlobalVariable**, long, long, llvm::GlobalVariable**, long, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt16__insertion_sortIPPN4llvm14GlobalVariableEN12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_T0_", "void std::__insertion_sort<llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt22__merge_without_bufferIPPN4llvm14GlobalVariableElN12_GLOBAL__N_114ARMGlobalMerge9GlobalCmpEEvT_S7_S7_T0_S8_T1_", "void std::__merge_without_buffer<llvm::GlobalVariable**, long, (anonymous namespace)::ARMGlobalMerge::GlobalCmp>(llvm::GlobalVariable**, llvm::GlobalVariable**, llvm::GlobalVariable**, long, long, (anonymous namespace)::ARMGlobalMerge::GlobalCmp)"},
    {"_ZSt8__rotateIPPN4llvm14GlobalVariableEEvT_S4_S4_St26random_access_iterator_tag", "void std::__rotate<llvm::GlobalVariable**>(llvm::GlobalVariable**, llvm::GlobalVariable**, llvm::GlobalVariable**, std::random_access_iterator_tag)"},
    {"_ZN4llvm19ARMHazardRecognizer13getHazardTypeEPNS_5SUnitEi", "llvm::ARMHazardRecognizer::getHazardType(llvm::SUnit*, int)"},
    {"_ZN4llvm19ARMHazardRecognizer5ResetEv", "llvm::ARMHazardRecognizer::Reset()"},
    {"_ZN4llvm19ARMHazardRecognizer15EmitInstructionEPNS_5SUnitE", "llvm::ARMHazardRecognizer::EmitInstruction(llvm::SUnit*)"},
    {"_ZN4llvm19ARMHazardRecognizer12AdvanceCycleEv", "llvm::ARMHazardRecognizer::AdvanceCycle()"},
    {"_ZN4llvm19ARMHazardRecognizer11RecedeCycleEv", "llvm::ARMHazardRecognizer::RecedeCycle()"},
    {"_ZN4llvm19ARMHazardRecognizerD1Ev", "llvm::ARMHazardRecognizer::~ARMHazardRecognizer()"},
    {"_ZN4llvm19ARMHazardRecognizerD0Ev", "llvm::ARMHazardRecognizer::~ARMHazardRecognizer()"},
    {"_ZN4llvm16createARMISelDagERNS_20ARMBaseTargetMachineENS_10CodeGenOpt5LevelE", "llvm::createARMISelDag(llvm::ARMBaseTargetMachine&, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISelD1Ev", "(anonymous namespace)::ARMDAGToDAGISel::~ARMDAGToDAGISel()"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISelD0Ev", "(anonymous namespace)::ARMDAGToDAGISel::~ARMDAGToDAGISel()"},
    {"_ZNK12_GLOBAL__N_115ARMDAGToDAGISel11getPassNameEv", "(anonymous namespace)::ARMDAGToDAGISel::getPassName() const"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeE", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel28SelectInlineAsmMemoryOperandERKN4llvm7SDValueEcRSt6vectorIS2_SaIS2_EE", "(anonymous namespace)::ARMDAGToDAGISel::SelectInlineAsmMemoryOperand(llvm::SDValue const&, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&)"},
    {"_ZNK12_GLOBAL__N_115ARMDAGToDAGISel21CheckPatternPredicateEj", "(anonymous namespace)::ARMDAGToDAGISel::CheckPatternPredicate(unsigned int) const"},
    {"_ZNK12_GLOBAL__N_115ARMDAGToDAGISel18CheckNodePredicateEPN4llvm6SDNodeEj", "(anonymous namespace)::ARMDAGToDAGISel::CheckNodePredicate(llvm::SDNode*, unsigned int) const"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel19CheckComplexPatternEPN4llvm6SDNodeES3_NS1_7SDValueEjRNS1_15SmallVectorImplISt4pairIS4_S3_EEE", "(anonymous namespace)::ARMDAGToDAGISel::CheckComplexPattern(llvm::SDNode*, llvm::SDNode*, llvm::SDValue, unsigned int, llvm::SmallVectorImpl<std::pair<llvm::SDValue, llvm::SDNode*> >&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel14RunSDNodeXFormEN4llvm7SDValueEj", "(anonymous namespace)::ARMDAGToDAGISel::RunSDNodeXForm(llvm::SDValue, unsigned int)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel23SelectShifterOperandRegEN4llvm7SDValueERS2_S3_S3_", "(anonymous namespace)::ARMDAGToDAGISel::SelectShifterOperandReg(llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel20SelectT2AddrModeImm8EN4llvm7SDValueERS2_S3_", "(anonymous namespace)::ARMDAGToDAGISel::SelectT2AddrModeImm8(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel21SelectThumbAddrModeSPEN4llvm7SDValueERS2_S3_", "(anonymous namespace)::ARMDAGToDAGISel::SelectThumbAddrModeSP(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel15SelectAddrMode6EPN4llvm6SDNodeENS1_7SDValueERS4_S5_", "(anonymous namespace)::ARMDAGToDAGISel::SelectAddrMode6(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel21SelectAddrMode2OffsetEPN4llvm6SDNodeENS1_7SDValueERS4_S5_", "(anonymous namespace)::ARMDAGToDAGISel::SelectAddrMode2Offset(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel21SelectAddrMode3OffsetEPN4llvm6SDNodeENS1_7SDValueERS4_S5_", "(anonymous namespace)::ARMDAGToDAGISel::SelectAddrMode3Offset(llvm::SDNode*, llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel26SelectT2AddrModeImm8OffsetEPN4llvm6SDNodeENS1_7SDValueERS4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectT2AddrModeImm8Offset(llvm::SDNode*, llvm::SDValue, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel24SelectThumbAddrModeImm5SEN4llvm7SDValueEjRS2_S3_", "(anonymous namespace)::ARMDAGToDAGISel::SelectThumbAddrModeImm5S(llvm::SDValue, unsigned int, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel21SelectThumbAddrModeRIEN4llvm7SDValueERS2_S3_j", "(anonymous namespace)::ARMDAGToDAGISel::SelectThumbAddrModeRI(llvm::SDValue, llvm::SDValue&, llvm::SDValue&, unsigned int)"},
    {"_ZN4llvm6ARM_AM16decodeNEONModImmEjRj", "llvm::ARM_AM::decodeNEONModImm(unsigned int, unsigned int&)"},
    {"_ZNK12_GLOBAL__N_115ARMDAGToDAGISel18hasNoVMLxHazardUseEPN4llvm6SDNodeE", "(anonymous namespace)::ARMDAGToDAGISel::hasNoVMLxHazardUse(llvm::SDNode*) const"},
    {"_ZN4llvm6ARM_AM17isSOImmTwoPartValEj", "llvm::ARM_AM::isSOImmTwoPartVal(unsigned int)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel27SelectV6T2BitfieldExtractOpEPN4llvm6SDNodeEb", "(anonymous namespace)::ARMDAGToDAGISel::SelectV6T2BitfieldExtractOp(llvm::SDNode*, bool)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel12SelectVLDDupEPN4llvm6SDNodeEbjPj", "(anonymous namespace)::ARMDAGToDAGISel::SelectVLDDup(llvm::SDNode*, bool, unsigned int, unsigned int*)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel9SelectVLDEPN4llvm6SDNodeEbjPjS4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectVLD(llvm::SDNode*, bool, unsigned int, unsigned int*, unsigned int*, unsigned int*)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel15SelectVLDSTLaneEPN4llvm6SDNodeEbbjPjS4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectVLDSTLane(llvm::SDNode*, bool, bool, unsigned int, unsigned int*, unsigned int*)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel9SelectVSTEPN4llvm6SDNodeEbjPjS4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectVST(llvm::SDNode*, bool, unsigned int, unsigned int*, unsigned int*, unsigned int*)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel10SelectVTBLEPN4llvm6SDNodeEbjj", "(anonymous namespace)::ARMDAGToDAGISel::SelectVTBL(llvm::SDNode*, bool, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel9QuadDRegsEN4llvm3EVTENS1_7SDValueES3_S3_S3_", "(anonymous namespace)::ARMDAGToDAGISel::QuadDRegs(llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel9QuadQRegsEN4llvm3EVTENS1_7SDValueES3_S3_S3_", "(anonymous namespace)::ARMDAGToDAGISel::QuadQRegs(llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel19SelectT2CMOVShiftOpEPN4llvm6SDNodeENS1_7SDValueES4_NS1_5ARMCC9CondCodesES4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectT2CMOVShiftOp(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::ARMCC::CondCodes, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel20SelectARMCMOVShiftOpEPN4llvm6SDNodeENS1_7SDValueES4_NS1_5ARMCC9CondCodesES4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectARMCMOVShiftOp(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::ARMCC::CondCodes, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel17SelectT2CMOVImmOpEPN4llvm6SDNodeENS1_7SDValueES4_NS1_5ARMCC9CondCodesES4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectT2CMOVImmOp(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::ARMCC::CondCodes, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_115ARMDAGToDAGISel18SelectARMCMOVImmOpEPN4llvm6SDNodeENS1_7SDValueES4_NS1_5ARMCC9CondCodesES4_S4_", "(anonymous namespace)::ARMDAGToDAGISel::SelectARMCMOVImmOp(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::ARMCC::CondCodes, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16SelectionDAGISel21EmitFunctionEntryCodeEv", "llvm::SelectionDAGISel::EmitFunctionEntryCode()"},
    {"_ZN4llvm16SelectionDAGISel17PreprocessISelDAGEv", "llvm::SelectionDAGISel::PreprocessISelDAG()"},
    {"_ZN4llvm16SelectionDAGISel18PostprocessISelDAGEv", "llvm::SelectionDAGISel::PostprocessISelDAG()"},
    {"_ZN4llvm15SmallVectorImplISt4pairINS_7SDValueEPNS_6SDNodeEEE6resizeEj", "llvm::SmallVectorImpl<std::pair<llvm::SDValue, llvm::SDNode*> >::resize(unsigned int)"},
    {"_ZN4llvm16SelectionDAGISel11ISelUpdaterD1Ev", "llvm::SelectionDAGISel::ISelUpdater::~ISelUpdater()"},
    {"_ZN4llvm16SelectionDAGISel11ISelUpdaterD0Ev", "llvm::SelectionDAGISel::ISelUpdater::~ISelUpdater()"},
    {"_ZN4llvm16SelectionDAGISel11ISelUpdater11NodeDeletedEPNS_6SDNodeES3_", "llvm::SelectionDAGISel::ISelUpdater::NodeDeleted(llvm::SDNode*, llvm::SDNode*)"},
    {"_ZN4llvm16SelectionDAGISel11ISelUpdater11NodeUpdatedEPNS_6SDNodeE", "llvm::SelectionDAGISel::ISelUpdater::NodeUpdated(llvm::SDNode*)"},
    {"_ZN4llvm17ARMTargetLowering14addTypeForNEONENS_3EVTES1_S1_", "llvm::ARMTargetLowering::addTypeForNEON(llvm::EVT, llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm17ARMTargetLowering16addDRTypeForNEONENS_3EVTE", "llvm::ARMTargetLowering::addDRTypeForNEON(llvm::EVT)"},
    {"_ZN4llvm17ARMTargetLowering16addQRTypeForNEONENS_3EVTE", "llvm::ARMTargetLowering::addQRTypeForNEON(llvm::EVT)"},
    {"_ZN4llvm17ARMTargetLoweringC1ERNS_13TargetMachineE", "llvm::ARMTargetLowering::ARMTargetLowering(llvm::TargetMachine&)"},
    {"_ZN4llvm17ARMTargetLoweringC2ERNS_13TargetMachineE", "llvm::ARMTargetLowering::ARMTargetLowering(llvm::TargetMachine&)"},
    {"_ZNK4llvm17ARMTargetLowering23findRepresentativeClassENS_3EVTE", "llvm::ARMTargetLowering::findRepresentativeClass(llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering17getTargetNodeNameEj", "llvm::ARMTargetLowering::getTargetNodeName(unsigned int) const"},
    {"_ZNK4llvm17ARMTargetLowering14getRegClassForENS_3EVTE", "llvm::ARMTargetLowering::getRegClassFor(llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering14createFastISelERNS_20FunctionLoweringInfoE", "llvm::ARMTargetLowering::createFastISel(llvm::FunctionLoweringInfo&) const"},
    {"_ZNK4llvm17ARMTargetLowering20getFunctionAlignmentEPKNS_8FunctionE", "llvm::ARMTargetLowering::getFunctionAlignment(llvm::Function const*) const"},
    {"_ZNK4llvm17ARMTargetLowering22getMaximalGlobalOffsetEv", "llvm::ARMTargetLowering::getMaximalGlobalOffset() const"},
    {"_ZNK4llvm17ARMTargetLowering23getSchedulingPreferenceEPNS_6SDNodeE", "llvm::ARMTargetLowering::getSchedulingPreference(llvm::SDNode*) const"},
    {"_ZNK4llvm17ARMTargetLowering19getRegPressureLimitEPKNS_19TargetRegisterClassERNS_15MachineFunctionE", "llvm::ARMTargetLowering::getRegPressureLimit(llvm::TargetRegisterClass const*, llvm::MachineFunction&) const"},
    {"_ZNK4llvm17ARMTargetLowering17CCAssignFnForNodeENS_11CallingConv2IDEbb", "llvm::ARMTargetLowering::CCAssignFnForNode(llvm::CallingConv::ID, bool, bool) const"},
    {"_Z18RetFastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetFastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z15FastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "FastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z11CC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z15RetCC_ARM_AAPCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_Z12CC_ARM_AAPCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZNK4llvm17ARMTargetLowering15LowerCallResultENS_7SDValueES1_NS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EE", "llvm::ARMTargetLowering::LowerCallResult(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm17ARMTargetLowering16LowerMemOpCallToENS_7SDValueES1_S1_NS_8DebugLocERNS_12SelectionDAGERKNS_11CCValAssignENS_3ISD10ArgFlagsTyE", "llvm::ARMTargetLowering::LowerMemOpCallTo(llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::DebugLoc, llvm::SelectionDAG&, llvm::CCValAssign const&, llvm::ISD::ArgFlagsTy) const"},
    {"_ZNK4llvm17ARMTargetLowering16PassF64ArgInRegsENS_8DebugLocERNS_12SelectionDAGENS_7SDValueERS4_RNS_11SmallVectorISt4pairIjS4_ELj8EEERNS_11CCValAssignESC_S5_RNS6_IS4_Lj8EEENS_3ISD10ArgFlagsTyE", "llvm::ARMTargetLowering::PassF64ArgInRegs(llvm::DebugLoc, llvm::SelectionDAG&, llvm::SDValue, llvm::SDValue&, llvm::SmallVector<std::pair<unsigned int, llvm::SDValue>, 8u>&, llvm::CCValAssign&, llvm::CCValAssign&, llvm::SDValue&, llvm::SmallVector<llvm::SDValue, 8u>&, llvm::ISD::ArgFlagsTy) const"},
    {"_ZNK4llvm17ARMTargetLowering9LowerCallENS_7SDValueES1_NS_11CallingConv2IDEbRbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS5_IS1_EERKNS5_INS6_8InputArgEEENS_8DebugLocERNS_12SelectionDAGERSB_", "llvm::ARMTargetLowering::LowerCall(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, bool&, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm17ARMTargetLowering33IsEligibleForTailCallOptimizationENS_7SDValueENS_11CallingConv2IDEbbbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS4_IS1_EERKNS4_INS5_8InputArgEEERNS_12SelectionDAGE", "llvm::ARMTargetLowering::IsEligibleForTailCallOptimization(llvm::SDValue, llvm::CallingConv::ID, bool, bool, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering11LowerReturnENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS4_IS1_EENS_8DebugLocERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerReturn(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::DebugLoc, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering18isUsedByReturnOnlyEPNS_6SDNodeE", "llvm::ARMTargetLowering::isUsedByReturnOnly(llvm::SDNode*) const"},
    {"_ZNK4llvm17ARMTargetLowering20getJumpTableEncodingEv", "llvm::ARMTargetLowering::getJumpTableEncoding() const"},
    {"_ZNK4llvm17ARMTargetLowering17LowerBlockAddressENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerBlockAddress(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering29LowerToTLSGeneralDynamicModelEPNS_19GlobalAddressSDNodeERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerToTLSGeneralDynamicModel(llvm::GlobalAddressSDNode*, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering20LowerToTLSExecModelsEPNS_19GlobalAddressSDNodeERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerToTLSExecModels(llvm::GlobalAddressSDNode*, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering21LowerGlobalTLSAddressENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerGlobalTLSAddress(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering21LowerGlobalAddressELFENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerGlobalAddressELF(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering24LowerGlobalAddressDarwinENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerGlobalAddressDarwin(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering24LowerGLOBAL_OFFSET_TABLEENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerGLOBAL_OFFSET_TABLE(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering26LowerEH_SJLJ_DISPATCHSETUPENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerEH_SJLJ_DISPATCHSETUP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering19LowerEH_SJLJ_SETJMPENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerEH_SJLJ_SETJMP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering20LowerEH_SJLJ_LONGJMPENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerEH_SJLJ_LONGJMP(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering23LowerINTRINSIC_WO_CHAINENS_7SDValueERNS_12SelectionDAGEPKNS_12ARMSubtargetE", "llvm::ARMTargetLowering::LowerINTRINSIC_WO_CHAIN(llvm::SDValue, llvm::SelectionDAG&, llvm::ARMSubtarget const*) const"},
    {"_ZNK4llvm17ARMTargetLowering20GetF64FormalArgumentERNS_11CCValAssignES2_RNS_7SDValueERNS_12SelectionDAGENS_8DebugLocE", "llvm::ARMTargetLowering::GetF64FormalArgument(llvm::CCValAssign&, llvm::CCValAssign&, llvm::SDValue&, llvm::SelectionDAG&, llvm::DebugLoc) const"},
    {"_ZNK4llvm17ARMTargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EE", "llvm::ARMTargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm17ARMTargetLowering9getARMCmpENS_7SDValueES1_NS_3ISD8CondCodeERS1_RNS_12SelectionDAGENS_8DebugLocE", "llvm::ARMTargetLowering::getARMCmp(llvm::SDValue, llvm::SDValue, llvm::ISD::CondCode, llvm::SDValue&, llvm::SelectionDAG&, llvm::DebugLoc) const"},
    {"_ZNK4llvm17ARMTargetLowering9getVFPCmpENS_7SDValueES1_RNS_12SelectionDAGENS_8DebugLocE", "llvm::ARMTargetLowering::getVFPCmp(llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&, llvm::DebugLoc) const"},
    {"_ZNK4llvm17ARMTargetLowering11LowerSELECTENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerSELECT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering14LowerSELECT_CCENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerSELECT_CC(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering17OptimizeVFPBrcondENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::OptimizeVFPBrcond(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z14canChangeToIntN4llvm7SDValueERbPKNS_12ARMSubtargetE", "canChangeToInt(llvm::SDValue, bool&, llvm::ARMSubtarget const*)"},
    {"_Z15bitcastf32Toi32N4llvm7SDValueERNS_12SelectionDAGE", "bitcastf32Toi32(llvm::SDValue, llvm::SelectionDAG&)"},
    {"_Z14expandf64Toi32N4llvm7SDValueERNS_12SelectionDAGERS0_S3_", "expandf64Toi32(llvm::SDValue, llvm::SelectionDAG&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZNK4llvm17ARMTargetLowering10LowerBR_CCENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerBR_CC(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering10LowerBR_JTENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerBR_JT(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering14LowerFCOPYSIGNENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerFCOPYSIGN(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering15LowerRETURNADDRENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerRETURNADDR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering14LowerFRAMEADDRENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerFRAMEADDR(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering20LowerShiftRightPartsENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerShiftRightParts(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering19LowerShiftLeftPartsENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerShiftLeftParts(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering16LowerFLT_ROUNDS_ENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerFLT_ROUNDS_(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering17LowerBUILD_VECTORENS_7SDValueERNS_12SelectionDAGEPKNS_12ARMSubtargetE", "llvm::ARMTargetLowering::LowerBUILD_VECTOR(llvm::SDValue, llvm::SelectionDAG&, llvm::ARMSubtarget const*) const"},
    {"_Z17isNEONModifiedImmyyjRN4llvm12SelectionDAGERNS_3EVTEbNS_14NEONModImmTypeE", "isNEONModifiedImm(unsigned long long, unsigned long long, unsigned int, llvm::SelectionDAG&, llvm::EVT&, bool, llvm::NEONModImmType)"},
    {"_ZNK4llvm17ARMTargetLowering18ReconstructShuffleENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::ReconstructShuffle(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering18isShuffleMaskLegalERKNS_15SmallVectorImplIiEENS_3EVTE", "llvm::ARMTargetLowering::isShuffleMaskLegal(llvm::SmallVectorImpl<int> const&, llvm::EVT) const"},
    {"_Z10isVREVMaskRKN4llvm15SmallVectorImplIiEENS_3EVTEj", "isVREVMask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int)"},
    {"_Z10isVTRNMaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVTRNMask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_Z10isVUZPMaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVUZPMask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_Z10isVZIPMaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVZIPMask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_Z19isVTRN_v_undef_MaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVTRN_v_undef_Mask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_Z19isVUZP_v_undef_MaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVUZP_v_undef_Mask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_Z19isVZIP_v_undef_MaskRKN4llvm15SmallVectorImplIiEENS_3EVTERj", "isVZIP_v_undef_Mask(llvm::SmallVectorImpl<int> const&, llvm::EVT, unsigned int&)"},
    {"_ZNK4llvm17ARMTargetLowering14LowerOperationENS_7SDValueERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerOperation(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_Z13ExpandBITCASTPN4llvm6SDNodeERNS_12SelectionDAGE", "ExpandBITCAST(llvm::SDNode*, llvm::SelectionDAG&)"},
    {"_ZNK4llvm17ARMTargetLowering18ReplaceNodeResultsEPNS_6SDNodeERNS_15SmallVectorImplINS_7SDValueEEERNS_12SelectionDAGE", "llvm::ARMTargetLowering::ReplaceNodeResults(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering17EmitAtomicCmpSwapEPNS_12MachineInstrEPNS_17MachineBasicBlockEj", "llvm::ARMTargetLowering::EmitAtomicCmpSwap(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int) const"},
    {"_ZNK4llvm17ARMTargetLowering16EmitAtomicBinaryEPNS_12MachineInstrEPNS_17MachineBasicBlockEjj", "llvm::ARMTargetLowering::EmitAtomicBinary(llvm::MachineInstr*, llvm::MachineBasicBlock*, unsigned int, unsigned int) const"},
    {"_ZNK4llvm17ARMTargetLowering27EmitInstrWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::ARMTargetLowering::EmitInstrWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm17ARMTargetLowering17PerformDAGCombineEPNS_6SDNodeERNS_14TargetLowering15DAGCombinerInfoE", "llvm::ARMTargetLowering::PerformDAGCombine(llvm::SDNode*, llvm::TargetLowering::DAGCombinerInfo&) const"},
    {"_Z21PerformVMOVDRRCombinePN4llvm6SDNodeERNS_12SelectionDAGE", "PerformVMOVDRRCombine(llvm::SDNode*, llvm::SelectionDAG&)"},
    {"_Z17CombineBaseUpdatePN4llvm6SDNodeERNS_14TargetLowering15DAGCombinerInfoE", "CombineBaseUpdate(llvm::SDNode*, llvm::TargetLowering::DAGCombinerInfo&)"},
    {"_ZNK4llvm17ARMTargetLowering33isDesirableToTransformToIntegerOpEjNS_3EVTE", "llvm::ARMTargetLowering::isDesirableToTransformToIntegerOp(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering29allowsUnalignedMemoryAccessesENS_3EVTE", "llvm::ARMTargetLowering::allowsUnalignedMemoryAccesses(llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering29isLegalT2ScaledAddressingModeERKNS_14TargetLowering8AddrModeENS_3EVTE", "llvm::ARMTargetLowering::isLegalT2ScaledAddressingMode(llvm::TargetLowering::AddrMode const&, llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering21isLegalAddressingModeERKNS_14TargetLowering8AddrModeEPKNS_4TypeE", "llvm::ARMTargetLowering::isLegalAddressingMode(llvm::TargetLowering::AddrMode const&, llvm::Type const*) const"},
    {"_ZNK4llvm17ARMTargetLowering20isLegalICmpImmediateEx", "llvm::ARMTargetLowering::isLegalICmpImmediate(long long) const"},
    {"_ZNK4llvm17ARMTargetLowering25getPreIndexedAddressPartsEPNS_6SDNodeERNS_7SDValueES4_RNS_3ISD14MemIndexedModeERNS_12SelectionDAGE", "llvm::ARMTargetLowering::getPreIndexedAddressParts(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&, llvm::ISD::MemIndexedMode&, llvm::SelectionDAG&) const"},
    {"_Z24getT2IndexedAddressPartsPN4llvm6SDNodeENS_3EVTEbRNS_7SDValueES4_RbRNS_12SelectionDAGE", "getT2IndexedAddressParts(llvm::SDNode*, llvm::EVT, bool, llvm::SDValue&, llvm::SDValue&, bool&, llvm::SelectionDAG&)"},
    {"_Z25getARMIndexedAddressPartsPN4llvm6SDNodeENS_3EVTEbRNS_7SDValueES4_RbRNS_12SelectionDAGE", "getARMIndexedAddressParts(llvm::SDNode*, llvm::EVT, bool, llvm::SDValue&, llvm::SDValue&, bool&, llvm::SelectionDAG&)"},
    {"_ZNK4llvm17ARMTargetLowering26getPostIndexedAddressPartsEPNS_6SDNodeES2_RNS_7SDValueES4_RNS_3ISD14MemIndexedModeERNS_12SelectionDAGE", "llvm::ARMTargetLowering::getPostIndexedAddressParts(llvm::SDNode*, llvm::SDNode*, llvm::SDValue&, llvm::SDValue&, llvm::ISD::MemIndexedMode&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering30computeMaskedBitsForTargetNodeENS_7SDValueERKNS_5APIntERS2_S5_RKNS_12SelectionDAGEj", "llvm::ARMTargetLowering::computeMaskedBitsForTargetNode(llvm::SDValue, llvm::APInt const&, llvm::APInt&, llvm::APInt&, llvm::SelectionDAG const&, unsigned int) const"},
    {"_ZNK4llvm17ARMTargetLowering15ExpandInlineAsmEPNS_8CallInstE", "llvm::ARMTargetLowering::ExpandInlineAsm(llvm::CallInst*) const"},
    {"_ZNK4llvm17ARMTargetLowering17getConstraintTypeERKSs", "llvm::ARMTargetLowering::getConstraintType(std::string const&) const"},
    {"_ZNK4llvm17ARMTargetLowering30getSingleConstraintMatchWeightERNS_14TargetLowering14AsmOperandInfoEPKc", "llvm::ARMTargetLowering::getSingleConstraintMatchWeight(llvm::TargetLowering::AsmOperandInfo&, char const*) const"},
    {"_ZNK4llvm17ARMTargetLowering28getRegForInlineAsmConstraintERKSsNS_3EVTE", "llvm::ARMTargetLowering::getRegForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering33getRegClassForInlineAsmConstraintERKSsNS_3EVTE", "llvm::ARMTargetLowering::getRegClassForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering28LowerAsmOperandForConstraintENS_7SDValueEcRSt6vectorIS1_SaIS1_EERNS_12SelectionDAGE", "llvm::ARMTargetLowering::LowerAsmOperandForConstraint(llvm::SDValue, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm17ARMTargetLowering20isOffsetFoldingLegalEPKNS_19GlobalAddressSDNodeE", "llvm::ARMTargetLowering::isOffsetFoldingLegal(llvm::GlobalAddressSDNode const*) const"},
    {"_ZN4llvm3ARM12getVFPf32ImmERKNS_7APFloatE", "llvm::ARM::getVFPf32Imm(llvm::APFloat const&)"},
    {"_ZN4llvm3ARM12getVFPf64ImmERKNS_7APFloatE", "llvm::ARM::getVFPf64Imm(llvm::APFloat const&)"},
    {"_ZN4llvm3ARM22isBitFieldInvertedMaskEj", "llvm::ARM::isBitFieldInvertedMask(unsigned int)"},
    {"_ZNK4llvm17ARMTargetLowering12isFPImmLegalERKNS_7APFloatENS_3EVTE", "llvm::ARMTargetLowering::isFPImmLegal(llvm::APFloat const&, llvm::EVT) const"},
    {"_ZNK4llvm17ARMTargetLowering18getTgtMemIntrinsicERNS_14TargetLowering13IntrinsicInfoERKNS_8CallInstEj", "llvm::ARMTargetLowering::getTgtMemIntrinsic(llvm::TargetLowering::IntrinsicInfo&, llvm::CallInst const&, unsigned int) const"},
    {"_Z12getVShiftImmN4llvm7SDValueEjRx", "getVShiftImm(llvm::SDValue, unsigned int, long long&)"},
    {"_Z19combineSelectAndUsePN4llvm6SDNodeENS_7SDValueES2_RNS_14TargetLowering15DAGCombinerInfoE", "combineSelectAndUse(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::TargetLowering::DAGCombinerInfo&)"},
    {"_Z13SkipExtensionPN4llvm6SDNodeERNS_12SelectionDAGE", "SkipExtension(llvm::SDNode*, llvm::SelectionDAG&)"},
    {"_Z22isExtendedBUILD_VECTORPN4llvm6SDNodeERNS_12SelectionDAGEb", "isExtendedBUILD_VECTOR(llvm::SDNode*, llvm::SelectionDAG&, bool)"},
    {"_Z22GeneratePerfectShufflejN4llvm7SDValueES0_RNS_12SelectionDAGENS_8DebugLocE", "GeneratePerfectShuffle(unsigned int, llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_Z19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEb", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)"},
    {"_Z22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateE", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)"},
    {"_ZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateE", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)"},
    {"_ZN4llvm13f64AssignAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEb", "llvm::f64AssignAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)"},
    {"_ZN4llvm12SelectionDAG14getCALLSEQ_ENDENS_7SDValueES1_S1_S1_", "llvm::SelectionDAG::getCALLSEQ_END(llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm11make_vectorIjEESt6vectorIT_SaIS2_EES2_z", "std::vector<unsigned int, std::allocator<unsigned int> > llvm::make_vector<unsigned int>(unsigned int, ...)"},
    {"_ZNSt6vectorIN4llvm14TargetLowering12ArgListEntryESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::TargetLowering::ArgListEntry, std::allocator<llvm::TargetLowering::ArgListEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::TargetLowering::ArgListEntry*, std::vector<llvm::TargetLowering::ArgListEntry, std::allocator<llvm::TargetLowering::ArgListEntry> > >, llvm::TargetLowering::ArgListEntry const&)"},
    {"_ZN4llvm5APInt11flipAllBitsEv", "llvm::APInt::flipAllBits()"},
    {"_ZNK4llvm19ShuffleVectorSDNode7getMaskERNS_15SmallVectorImplIiEE", "llvm::ShuffleVectorSDNode::getMask(llvm::SmallVectorImpl<int>&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11CCValAssignELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::CCValAssign, false>::grow(unsigned long)"},
    {"_ZNK4llvm24TargetLoweringObjectFile26shouldEmitUsedDirectiveForEPKNS_11GlobalValueEPNS_7ManglerE", "llvm::TargetLoweringObjectFile::shouldEmitUsedDirectiveFor(llvm::GlobalValue const*, llvm::Mangler*) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile29getSpecialCasedSectionGlobalsEPKNS_11GlobalValueEPNS_7ManglerENS_11SectionKindE", "llvm::TargetLoweringObjectFile::getSpecialCasedSectionGlobals(llvm::GlobalValue const*, llvm::Mangler*, llvm::SectionKind) const"},
    {"_ZNK4llvm22ARMElfTargetObjectFile20getAttributesSectionEv", "llvm::ARMElfTargetObjectFile::getAttributesSection() const"},
    {"_ZN4llvm27TargetLoweringObjectFileELFD1Ev", "llvm::TargetLoweringObjectFileELF::~TargetLoweringObjectFileELF()"},
    {"_ZN4llvm27TargetLoweringObjectFileELFD0Ev", "llvm::TargetLoweringObjectFileELF::~TargetLoweringObjectFileELF()"},
    {"_ZN4llvm29TargetLoweringObjectFileMachOD1Ev", "llvm::TargetLoweringObjectFileMachO::~TargetLoweringObjectFileMachO()"},
    {"_ZN4llvm29TargetLoweringObjectFileMachOD0Ev", "llvm::TargetLoweringObjectFileMachO::~TargetLoweringObjectFileMachO()"},
    {"_ZNSt6vectorISt4pairIN4llvm3EVTEPNS1_19TargetRegisterClassEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::EVT, llvm::TargetRegisterClass*>, std::allocator<std::pair<llvm::EVT, llvm::TargetRegisterClass*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::EVT, llvm::TargetRegisterClass*>*, std::vector<std::pair<llvm::EVT, llvm::TargetRegisterClass*>, std::allocator<std::pair<llvm::EVT, llvm::TargetRegisterClass*> > > >, std::pair<llvm::EVT, llvm::TargetRegisterClass*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIjN4llvm3MVT15SimpleValueTypeEES0_IKS4_S3_ESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<std::pair<unsigned int, llvm::MVT::SimpleValueType>, std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType>, std::_Select1st<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> >, std::less<std::pair<unsigned int, llvm::MVT::SimpleValueType> >, std::allocator<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> >, std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIjN4llvm3MVT15SimpleValueTypeEES0_IKS4_S3_ESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::pair<unsigned int, llvm::MVT::SimpleValueType>, std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType>, std::_Select1st<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> >, std::less<std::pair<unsigned int, llvm::MVT::SimpleValueType> >, std::allocator<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> > >::_M_insert_unique(std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> const&)"},
    {"_ZN4llvm12ARMInstrInfoC1ERKNS_12ARMSubtargetE", "llvm::ARMInstrInfo::ARMInstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZN4llvm12ARMInstrInfoC2ERKNS_12ARMSubtargetE", "llvm::ARMInstrInfo::ARMInstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm12ARMInstrInfo18getUnindexedOpcodeEj", "llvm::ARMInstrInfo::getUnindexedOpcode(unsigned int) const"},
    {"_ZN4llvm10ARMJITInfo29replaceMachineCodeForFunctionEPvS1_", "llvm::ARMJITInfo::replaceMachineCodeForFunction(void*, void*)"},
    {"_ZN4llvm10ARMJITInfo23getLazyResolverFunctionEPFPvS1_E", "llvm::ARMJITInfo::getLazyResolverFunction(void* (*)(void*))"},
    {"_ZN4llvm10ARMJITInfo26emitGlobalValueIndirectSymEPKNS_11GlobalValueEPvRNS_14JITCodeEmitterE", "llvm::ARMJITInfo::emitGlobalValueIndirectSym(llvm::GlobalValue const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm10ARMJITInfo13getStubLayoutEv", "llvm::ARMJITInfo::getStubLayout()"},
    {"_ZN4llvm10ARMJITInfo16emitFunctionStubEPKNS_8FunctionEPvRNS_14JITCodeEmitterE", "llvm::ARMJITInfo::emitFunctionStub(llvm::Function const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZNK4llvm10ARMJITInfo20resolveRelocDestAddrEPNS_17MachineRelocationE", "llvm::ARMJITInfo::resolveRelocDestAddr(llvm::MachineRelocation*) const"},
    {"_ZN4llvm10ARMJITInfo8relocateEPvPNS_17MachineRelocationEjPh", "llvm::ARMJITInfo::relocate(void*, llvm::MachineRelocation*, unsigned int, unsigned char*)"},
    {"_ZN4llvm8DenseMapIPvlNS_12DenseMapInfoIS1_EENS2_IlEEE6insertERKSt4pairIS1_lE", "llvm::DenseMap<void*, long, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<long> >::insert(std::pair<void*, long> const&)"},
    {"_ZN4llvm8DenseMapIPvlNS_12DenseMapInfoIS1_EENS2_IlEEE16InsertIntoBucketERKS1_RKlPSt4pairIS1_lE", "llvm::DenseMap<void*, long, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<long> >::InsertIntoBucket(void* const&, long const&, std::pair<void*, long>*)"},
    {"_ZN4llvm8DenseMapIPvlNS_12DenseMapInfoIS1_EENS2_IlEEE4growEj", "llvm::DenseMap<void*, long, llvm::DenseMapInfo<void*>, llvm::DenseMapInfo<long> >::grow(unsigned int)"},
    {"_ZN4llvm6ARM_AM27getLoadStoreMultipleSubModeEi", "llvm::ARM_AM::getLoadStoreMultipleSubMode(int)"},
    {"_ZN4llvm34createARMLoadStoreOptimizationPassEb", "llvm::createARMLoadStoreOptimizationPass(bool)"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOptD1Ev", "(anonymous namespace)::ARMLoadStoreOpt::~ARMLoadStoreOpt()"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOptD0Ev", "(anonymous namespace)::ARMLoadStoreOpt::~ARMLoadStoreOpt()"},
    {"_ZNK12_GLOBAL__N_115ARMLoadStoreOpt11getPassNameEv", "(anonymous namespace)::ARMLoadStoreOpt::getPassName() const"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOpt20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ARMLoadStoreOpt::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_Z10isMemoryOpPKN4llvm12MachineInstrE", "isMemoryOp(llvm::MachineInstr const*)"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOpt12MergeLDR_STRERN4llvm17MachineBasicBlockEjjijNS1_5ARMCC9CondCodesEjjRNS1_11SmallVectorINS0_15MemOpQueueEntryELj8EEERNS6_INS1_14ilist_iteratorINS1_12MachineInstrEEELj4EEE", "(anonymous namespace)::ARMLoadStoreOpt::MergeLDR_STR(llvm::MachineBasicBlock&, unsigned int, unsigned int, int, unsigned int, llvm::ARMCC::CondCodes, unsigned int, unsigned int, llvm::SmallVector<(anonymous namespace)::ARMLoadStoreOpt::MemOpQueueEntry, 8u>&, llvm::SmallVector<llvm::ilist_iterator<llvm::MachineInstr>, 4u>&)"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOpt24MergeBaseUpdateLoadStoreERN4llvm17MachineBasicBlockENS1_14ilist_iteratorINS1_12MachineInstrEEEPKNS1_15TargetInstrInfoERbRS6_", "(anonymous namespace)::ARMLoadStoreOpt::MergeBaseUpdateLoadStore(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::TargetInstrInfo const*, bool&, llvm::ilist_iterator<llvm::MachineInstr>&)"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOpt14MergeOpsUpdateERN4llvm17MachineBasicBlockERNS1_11SmallVectorINS0_15MemOpQueueEntryELj8EEEjjjijbiNS1_5ARMCC9CondCodesEjjNS1_8DebugLocERNS4_INS1_14ilist_iteratorINS1_12MachineInstrEEELj4EEE", "(anonymous namespace)::ARMLoadStoreOpt::MergeOpsUpdate(llvm::MachineBasicBlock&, llvm::SmallVector<(anonymous namespace)::ARMLoadStoreOpt::MemOpQueueEntry, 8u>&, unsigned int, unsigned int, unsigned int, int, unsigned int, bool, int, llvm::ARMCC::CondCodes, unsigned int, unsigned int, llvm::DebugLoc, llvm::SmallVector<llvm::ilist_iterator<llvm::MachineInstr>, 4u>&)"},
    {"_Z13InsertLDR_STRRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEibNS_8DebugLocEjjbbjbbbbNS_5ARMCC9CondCodesEjPKNS_15TargetInstrInfoEb", "InsertLDR_STR(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, int, bool, llvm::DebugLoc, unsigned int, unsigned int, bool, bool, unsigned int, bool, bool, bool, bool, llvm::ARMCC::CondCodes, unsigned int, llvm::TargetInstrInfo const*, bool)"},
    {"_ZN12_GLOBAL__N_123ARMPreAllocLoadStoreOptD1Ev", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::~ARMPreAllocLoadStoreOpt()"},
    {"_ZN12_GLOBAL__N_123ARMPreAllocLoadStoreOptD0Ev", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::~ARMPreAllocLoadStoreOpt()"},
    {"_ZNK12_GLOBAL__N_123ARMPreAllocLoadStoreOpt11getPassNameEv", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::getPassName() const"},
    {"_ZN12_GLOBAL__N_123ARMPreAllocLoadStoreOpt20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_123ARMPreAllocLoadStoreOpt13RescheduleOpsEPN4llvm17MachineBasicBlockERNS1_11SmallVectorIPNS1_12MachineInstrELj4EEEjbRNS1_8DenseMapIS6_jNS1_12DenseMapInfoIS6_EENSA_IjEEEE", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::RescheduleOps(llvm::MachineBasicBlock*, llvm::SmallVector<llvm::MachineInstr*, 4u>&, unsigned int, bool, llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZSt16__introsort_loopIPPN4llvm12MachineInstrElN12_GLOBAL__N_113OffsetCompareEEvT_S6_T0_T1_", "void std::__introsort_loop<llvm::MachineInstr**, long, (anonymous namespace)::OffsetCompare>(llvm::MachineInstr**, llvm::MachineInstr**, long, (anonymous namespace)::OffsetCompare)"},
    {"_ZNK12_GLOBAL__N_113OffsetCompareclEPKN4llvm12MachineInstrES4_", "(anonymous namespace)::OffsetCompare::operator()(llvm::MachineInstr const*, llvm::MachineInstr const*) const"},
    {"_ZSt13__adjust_heapIPPN4llvm12MachineInstrElS2_N12_GLOBAL__N_113OffsetCompareEEvT_T0_S7_T1_T2_", "void std::__adjust_heap<llvm::MachineInstr**, long, llvm::MachineInstr*, (anonymous namespace)::OffsetCompare>(llvm::MachineInstr**, long, long, llvm::MachineInstr*, (anonymous namespace)::OffsetCompare)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_14ilist_iteratorINS_12MachineInstrEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::ilist_iterator<llvm::MachineInstr>, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE5clearEv", "llvm::DenseMap<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::clear()"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::shrink_and_clear()"},
    {"_ZN4llvm15SmallVectorImplIPNS_12MachineInstrEEaSERKS3_", "llvm::SmallVectorImpl<llvm::MachineInstr*>::operator=(llvm::SmallVectorImpl<llvm::MachineInstr*> const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PSt4pairIjS4_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallVector<llvm::MachineInstr*, 4u> const&, std::pair<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MachineInstr* const&, unsigned int const&, std::pair<llvm::MachineInstr*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm18ARMMCAsmInfoDarwinC1Ev", "llvm::ARMMCAsmInfoDarwin::ARMMCAsmInfoDarwin()"},
    {"_ZN4llvm18ARMMCAsmInfoDarwinC2Ev", "llvm::ARMMCAsmInfoDarwin::ARMMCAsmInfoDarwin()"},
    {"_ZN4llvm15ARMELFMCAsmInfoC1Ev", "llvm::ARMELFMCAsmInfo::ARMELFMCAsmInfo()"},
    {"_ZN4llvm15ARMELFMCAsmInfoC2Ev", "llvm::ARMELFMCAsmInfo::ARMELFMCAsmInfo()"},
    {"_ZN4llvm15ARMELFMCAsmInfoD1Ev", "llvm::ARMELFMCAsmInfo::~ARMELFMCAsmInfo()"},
    {"_ZN4llvm15ARMELFMCAsmInfoD0Ev", "llvm::ARMELFMCAsmInfo::~ARMELFMCAsmInfo()"},
    {"_ZN4llvm18ARMMCAsmInfoDarwinD1Ev", "llvm::ARMMCAsmInfoDarwin::~ARMMCAsmInfoDarwin()"},
    {"_ZN4llvm18ARMMCAsmInfoDarwinD0Ev", "llvm::ARMMCAsmInfoDarwin::~ARMMCAsmInfoDarwin()"},
    {"_ZN4llvm22createARMMCCodeEmitterERKNS_6TargetERNS_13TargetMachineERNS_9MCContextE", "llvm::createARMMCCodeEmitter(llvm::Target const&, llvm::TargetMachine&, llvm::MCContext&)"},
    {"_ZN12_GLOBAL__N_116ARMMCCodeEmitterD1Ev", "(anonymous namespace)::ARMMCCodeEmitter::~ARMMCCodeEmitter()"},
    {"_ZN12_GLOBAL__N_116ARMMCCodeEmitterD0Ev", "(anonymous namespace)::ARMMCCodeEmitter::~ARMMCCodeEmitter()"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter17EncodeInstructionERKN4llvm6MCInstERNS1_11raw_ostreamERNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::EncodeInstruction(llvm::MCInst const&, llvm::raw_ostream&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter21getBinaryCodeForInstrERKN4llvm6MCInstERNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getBinaryCodeForInstr(llvm::MCInst const&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter17getMachineOpValueERKN4llvm6MCInstERKNS1_9MCOperandERNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getMachineOpValue(llvm::MCInst const&, llvm::MCOperand const&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter15getSORegOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getSORegOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter17getT2SORegOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getT2SORegOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter17getT2SOImmOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getT2SOImmOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter19getHiLo16ImmOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getHiLo16ImmOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter22getRegisterListOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getRegisterListOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter23getAddrModeImm12OpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getAddrModeImm12OpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter25getT2AddrModeSORegOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getT2AddrModeSORegOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter25getARMBranchTargetOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getARMBranchTargetOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter19getAddrMode5OpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getAddrMode5OpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter19getLdStSORegOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getLdStSORegOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter19getAddrMode3OpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getAddrMode3OpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter25getAddrMode2OffsetOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getAddrMode2OffsetOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter25getAddrMode3OffsetOpValueERKN4llvm6MCInstEjRNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::getAddrMode3OffsetOpValue(llvm::MCInst const&, unsigned int, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZNK12_GLOBAL__N_116ARMMCCodeEmitter22EncodeAddrModeOpValuesERKN4llvm6MCInstEjRjS5_RNS1_15SmallVectorImplINS1_7MCFixupEEE", "(anonymous namespace)::ARMMCCodeEmitter::EncodeAddrModeOpValues(llvm::MCInst const&, unsigned int, unsigned int&, unsigned int&, llvm::SmallVectorImpl<llvm::MCFixup>&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_7MCFixupELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::MCFixup, false>::grow(unsigned long)"},
    {"_ZN4llvm9ARMMCExpr6CreateENS0_11VariantKindEPKNS_6MCExprERNS_9MCContextE", "llvm::ARMMCExpr::Create(llvm::ARMMCExpr::VariantKind, llvm::MCExpr const*, llvm::MCContext&)"},
    {"_ZNK4llvm9ARMMCExpr9PrintImplERNS_11raw_ostreamE", "llvm::ARMMCExpr::PrintImpl(llvm::raw_ostream&) const"},
    {"_ZNK4llvm9ARMMCExpr25EvaluateAsRelocatableImplERNS_7MCValueEPKNS_11MCAsmLayoutE", "llvm::ARMMCExpr::EvaluateAsRelocatableImpl(llvm::MCValue&, llvm::MCAsmLayout const*) const"},
    {"_ZNK4llvm9ARMMCExpr15AddValueSymbolsEPNS_11MCAssemblerE", "llvm::ARMMCExpr::AddValueSymbols(llvm::MCAssembler*) const"},
    {"_Z16AddValueSymbols_PKN4llvm6MCExprEPNS_11MCAssemblerE", "AddValueSymbols_(llvm::MCExpr const*, llvm::MCAssembler*)"},
    {"_ZN4llvm9ARMMCExprD1Ev", "llvm::ARMMCExpr::~ARMMCExpr()"},
    {"_ZN4llvm9ARMMCExprD0Ev", "llvm::ARMMCExpr::~ARMMCExpr()"},
    {"_ZN4llvm28LowerARMMachineInstrToMCInstEPKNS_12MachineInstrERNS_6MCInstERNS_13ARMAsmPrinterE", "llvm::LowerARMMachineInstrToMCInst(llvm::MachineInstr const*, llvm::MCInst&, llvm::ARMAsmPrinter&)"},
    {"_Z12GetSymbolRefRKN4llvm14MachineOperandEPKNS_8MCSymbolERNS_13ARMAsmPrinterE", "GetSymbolRef(llvm::MachineOperand const&, llvm::MCSymbol const*, llvm::ARMAsmPrinter&)"},
    {"_ZN4llvm15ARMRegisterInfoC1ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::ARMRegisterInfo::ARMRegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm15ARMRegisterInfoC2ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::ARMRegisterInfo::ARMRegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm19ARMSelectionDAGInfoC1ERKNS_13TargetMachineE", "llvm::ARMSelectionDAGInfo::ARMSelectionDAGInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm19ARMSelectionDAGInfoC2ERKNS_13TargetMachineE", "llvm::ARMSelectionDAGInfo::ARMSelectionDAGInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm19ARMSelectionDAGInfoD0Ev", "llvm::ARMSelectionDAGInfo::~ARMSelectionDAGInfo()"},
    {"_ZN4llvm19ARMSelectionDAGInfoD1Ev", "llvm::ARMSelectionDAGInfo::~ARMSelectionDAGInfo()"},
    {"_ZN4llvm19ARMSelectionDAGInfoD2Ev", "llvm::ARMSelectionDAGInfo::~ARMSelectionDAGInfo()"},
    {"_ZNK4llvm19ARMSelectionDAGInfo23EmitTargetCodeForMemcpyERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbbNS_18MachinePointerInfoES5_", "llvm::ARMSelectionDAGInfo::EmitTargetCodeForMemcpy(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo) const"},
    {"_ZNK4llvm22TargetSelectionDAGInfo24EmitTargetCodeForMemmoveERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbNS_18MachinePointerInfoES5_", "llvm::TargetSelectionDAGInfo::EmitTargetCodeForMemmove(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo) const"},
    {"_ZNK4llvm22TargetSelectionDAGInfo23EmitTargetCodeForMemsetERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbNS_18MachinePointerInfoE", "llvm::TargetSelectionDAGInfo::EmitTargetCodeForMemset(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, llvm::MachinePointerInfo) const"},
    {"_ZN4llvm12ARMSubtarget22ParseSubtargetFeaturesERKSsS2_", "llvm::ARMSubtarget::ParseSubtargetFeatures(std::string const&, std::string const&)"},
    {"_ZN4llvm12ARMSubtargetC1ERKSsS2_b", "llvm::ARMSubtarget::ARMSubtarget(std::string const&, std::string const&, bool)"},
    {"_ZN4llvm12ARMSubtargetC2ERKSsS2_b", "llvm::ARMSubtarget::ARMSubtarget(std::string const&, std::string const&, bool)"},
    {"_ZN4llvm12ARMSubtarget17computeIssueWidthEv", "llvm::ARMSubtarget::computeIssueWidth()"},
    {"_ZNK4llvm12ARMSubtarget18GVIsIndirectSymbolEPKNS_11GlobalValueENS_5Reloc5ModelE", "llvm::ARMSubtarget::GVIsIndirectSymbol(llvm::GlobalValue const*, llvm::Reloc::Model) const"},
    {"_ZNK4llvm12ARMSubtarget23getMispredictionPenaltyEv", "llvm::ARMSubtarget::getMispredictionPenalty() const"},
    {"_ZNK4llvm12ARMSubtarget21enablePostRASchedulerENS_10CodeGenOpt5LevelERNS_15TargetSubtarget16AntiDepBreakModeERNS_15SmallVectorImplIPNS_19TargetRegisterClassEEE", "llvm::ARMSubtarget::enablePostRAScheduler(llvm::CodeGenOpt::Level, llvm::TargetSubtarget::AntiDepBreakMode&, llvm::SmallVectorImpl<llvm::TargetRegisterClass*>&) const"},
    {"_Z15createMCAsmInfoRKN4llvm6TargetENS_9StringRefE", "createMCAsmInfo(llvm::Target const&, llvm::StringRef)"},
    {"_Z16createMCStreamerRKN4llvm6TargetERKSsRNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterEbb", "createMCStreamer(llvm::Target const&, std::string const&, llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*, bool, bool)"},
    {"_ZN4llvm20ARMBaseTargetMachineC1ERKNS_6TargetERKSsS5_b", "llvm::ARMBaseTargetMachine::ARMBaseTargetMachine(llvm::Target const&, std::string const&, std::string const&, bool)"},
    {"_ZN4llvm20ARMBaseTargetMachineC2ERKNS_6TargetERKSsS5_b", "llvm::ARMBaseTargetMachine::ARMBaseTargetMachine(llvm::Target const&, std::string const&, std::string const&, bool)"},
    {"_ZN4llvm16ARMTargetMachineC1ERKNS_6TargetERKSsS5_", "llvm::ARMTargetMachine::ARMTargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm16ARMTargetMachineC2ERKNS_6TargetERKSsS5_", "llvm::ARMTargetMachine::ARMTargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm18ThumbTargetMachineC1ERKNS_6TargetERKSsS5_", "llvm::ThumbTargetMachine::ThumbTargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm18ThumbTargetMachineC2ERKNS_6TargetERKSsS5_", "llvm::ThumbTargetMachine::ThumbTargetMachine(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm20ARMBaseTargetMachine10addPreISelERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::ARMBaseTargetMachine::addPreISel(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm20ARMBaseTargetMachine15addInstSelectorERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::ARMBaseTargetMachine::addInstSelector(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm20ARMBaseTargetMachine14addPreRegAllocERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::ARMBaseTargetMachine::addPreRegAlloc(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm20ARMBaseTargetMachine12addPreSched2ERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::ARMBaseTargetMachine::addPreSched2(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm20ARMBaseTargetMachine14addPreEmitPassERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::ARMBaseTargetMachine::addPreEmitPass(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm20ARMBaseTargetMachine14addCodeEmitterERNS_15PassManagerBaseENS_10CodeGenOpt5LevelERNS_14JITCodeEmitterE", "llvm::ARMBaseTargetMachine::addCodeEmitter(llvm::PassManagerBase&, llvm::CodeGenOpt::Level, llvm::JITCodeEmitter&)"},
    {"_ZNK4llvm20ARMBaseTargetMachine16getSubtargetImplEv", "llvm::ARMBaseTargetMachine::getSubtargetImpl() const"},
    {"_ZN4llvm20ARMBaseTargetMachineD1Ev", "llvm::ARMBaseTargetMachine::~ARMBaseTargetMachine()"},
    {"_ZN4llvm20ARMBaseTargetMachineD0Ev", "llvm::ARMBaseTargetMachine::~ARMBaseTargetMachine()"},
    {"_ZNK4llvm13TargetMachine12getInstrInfoEv", "llvm::TargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm13TargetMachine16getFrameLoweringEv", "llvm::TargetMachine::getFrameLowering() const"},
    {"_ZNK4llvm13TargetMachine17getTargetLoweringEv", "llvm::TargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm13TargetMachine19getSelectionDAGInfoEv", "llvm::TargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm13TargetMachine13getDataLayoutEv", "llvm::TargetMachine::getDataLayout() const"},
    {"_ZNK4llvm13TargetMachine15getRegisterInfoEv", "llvm::TargetMachine::getRegisterInfo() const"},
    {"_ZNK4llvm13TargetMachine16getIntrinsicInfoEv", "llvm::TargetMachine::getIntrinsicInfo() const"},
    {"_ZN4llvm20ARMBaseTargetMachine10getJITInfoEv", "llvm::ARMBaseTargetMachine::getJITInfo()"},
    {"_ZNK4llvm20ARMBaseTargetMachine21getInstrItineraryDataEv", "llvm::ARMBaseTargetMachine::getInstrItineraryData() const"},
    {"_ZNK4llvm13TargetMachine16getELFWriterInfoEv", "llvm::TargetMachine::getELFWriterInfo() const"},
    {"_ZNK4llvm17LLVMTargetMachine25getEnableTailMergeDefaultEv", "llvm::LLVMTargetMachine::getEnableTailMergeDefault() const"},
    {"_ZN4llvm17LLVMTargetMachine15addPostRegAllocERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addPostRegAlloc(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm18ThumbTargetMachineD1Ev", "llvm::ThumbTargetMachine::~ThumbTargetMachine()"},
    {"_ZN4llvm18ThumbTargetMachineD0Ev", "llvm::ThumbTargetMachine::~ThumbTargetMachine()"},
    {"_ZNK4llvm18ThumbTargetMachine12getInstrInfoEv", "llvm::ThumbTargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm18ThumbTargetMachine16getFrameLoweringEv", "llvm::ThumbTargetMachine::getFrameLowering() const"},
    {"_ZNK4llvm18ThumbTargetMachine17getTargetLoweringEv", "llvm::ThumbTargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm18ThumbTargetMachine19getSelectionDAGInfoEv", "llvm::ThumbTargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm18ThumbTargetMachine13getDataLayoutEv", "llvm::ThumbTargetMachine::getDataLayout() const"},
    {"_ZNK4llvm18ThumbTargetMachine15getRegisterInfoEv", "llvm::ThumbTargetMachine::getRegisterInfo() const"},
    {"_ZNK4llvm18ThumbTargetMachine16getELFWriterInfoEv", "llvm::ThumbTargetMachine::getELFWriterInfo() const"},
    {"_ZN4llvm16ARMTargetMachineD1Ev", "llvm::ARMTargetMachine::~ARMTargetMachine()"},
    {"_ZN4llvm16ARMTargetMachineD0Ev", "llvm::ARMTargetMachine::~ARMTargetMachine()"},
    {"_ZNK4llvm16ARMTargetMachine12getInstrInfoEv", "llvm::ARMTargetMachine::getInstrInfo() const"},
    {"_ZNK4llvm16ARMTargetMachine16getFrameLoweringEv", "llvm::ARMTargetMachine::getFrameLowering() const"},
    {"_ZNK4llvm16ARMTargetMachine17getTargetLoweringEv", "llvm::ARMTargetMachine::getTargetLowering() const"},
    {"_ZNK4llvm16ARMTargetMachine19getSelectionDAGInfoEv", "llvm::ARMTargetMachine::getSelectionDAGInfo() const"},
    {"_ZNK4llvm16ARMTargetMachine13getDataLayoutEv", "llvm::ARMTargetMachine::getDataLayout() const"},
    {"_ZNK4llvm16ARMTargetMachine15getRegisterInfoEv", "llvm::ARMTargetMachine::getRegisterInfo() const"},
    {"_ZNK4llvm16ARMTargetMachine16getELFWriterInfoEv", "llvm::ARMTargetMachine::getELFWriterInfo() const"},
    {"_ZN4llvm16ARMTargetMachineD2Ev", "llvm::ARMTargetMachine::~ARMTargetMachine()"},
    {"_ZN4llvm16ARMFrameLoweringD1Ev", "llvm::ARMFrameLowering::~ARMFrameLowering()"},
    {"_ZN4llvm17ARMTargetLoweringD1Ev", "llvm::ARMTargetLowering::~ARMTargetLowering()"},
    {"_ZN4llvm12ARMInstrInfoD1Ev", "llvm::ARMInstrInfo::~ARMInstrInfo()"},
    {"_ZN4llvm20ARMBaseTargetMachineD2Ev", "llvm::ARMBaseTargetMachine::~ARMBaseTargetMachine()"},
    {"_ZN4llvm10ARMJITInfoD1Ev", "llvm::ARMJITInfo::~ARMJITInfo()"},
    {"_ZN4llvm12ARMSubtargetD1Ev", "llvm::ARMSubtarget::~ARMSubtarget()"},
    {"_ZNK4llvm13TargetMachine16getSubtargetImplEv", "llvm::TargetMachine::getSubtargetImpl() const"},
    {"_ZN4llvm17LLVMTargetMachineD1Ev", "llvm::LLVMTargetMachine::~LLVMTargetMachine()"},
    {"_ZN4llvm17LLVMTargetMachineD0Ev", "llvm::LLVMTargetMachine::~LLVMTargetMachine()"},
    {"_ZN4llvm13TargetMachine10getJITInfoEv", "llvm::TargetMachine::getJITInfo()"},
    {"_ZNK4llvm13TargetMachine21getInstrItineraryDataEv", "llvm::TargetMachine::getInstrItineraryData() const"},
    {"_ZN4llvm17LLVMTargetMachine10addPreISelERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addPreISel(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm17LLVMTargetMachine15addInstSelectorERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addInstSelector(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm17LLVMTargetMachine14addPreRegAllocERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addPreRegAlloc(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm17LLVMTargetMachine12addPreSched2ERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addPreSched2(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm17LLVMTargetMachine14addPreEmitPassERNS_15PassManagerBaseENS_10CodeGenOpt5LevelE", "llvm::LLVMTargetMachine::addPreEmitPass(llvm::PassManagerBase&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm17LLVMTargetMachine14addCodeEmitterERNS_15PassManagerBaseENS_10CodeGenOpt5LevelERNS_14JITCodeEmitterE", "llvm::LLVMTargetMachine::addCodeEmitter(llvm::PassManagerBase&, llvm::CodeGenOpt::Level, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm12ARMSubtargetD0Ev", "llvm::ARMSubtarget::~ARMSubtarget()"},
    {"_ZNK4llvm15TargetSubtarget24getSpecialAddressLatencyEv", "llvm::TargetSubtarget::getSpecialAddressLatency() const"},
    {"_ZNK4llvm15TargetSubtarget21adjustSchedDependencyEPNS_5SUnitES2_RNS_4SDepE", "llvm::TargetSubtarget::adjustSchedDependency(llvm::SUnit*, llvm::SUnit*, llvm::SDep&) const"},
    {"_ZN4llvm10ARMJITInfoD0Ev", "llvm::ARMJITInfo::~ARMJITInfo()"},
    {"_ZN4llvm13TargetJITInfo20getPICJumpTableEntryEmm", "llvm::TargetJITInfo::getPICJumpTableEntry(unsigned long, unsigned long)"},
    {"_ZN4llvm13TargetJITInfo25allocateThreadLocalMemoryEm", "llvm::TargetJITInfo::allocateThreadLocalMemory(unsigned long)"},
    {"_ZNK4llvm10ARMJITInfo21hasCustomConstantPoolEv", "llvm::ARMJITInfo::hasCustomConstantPool() const"},
    {"_ZNK4llvm10ARMJITInfo19hasCustomJumpTablesEv", "llvm::ARMJITInfo::hasCustomJumpTables() const"},
    {"_ZNK4llvm10ARMJITInfo24allocateSeparateGVMemoryEv", "llvm::ARMJITInfo::allocateSeparateGVMemory() const"},
    {"_ZN4llvm13TargetJITInfoD1Ev", "llvm::TargetJITInfo::~TargetJITInfo()"},
    {"_ZN4llvm13TargetJITInfoD0Ev", "llvm::TargetJITInfo::~TargetJITInfo()"},
    {"_ZN4llvm13TargetJITInfo26emitGlobalValueIndirectSymEPKNS_11GlobalValueEPvRNS_14JITCodeEmitterE", "llvm::TargetJITInfo::emitGlobalValueIndirectSym(llvm::GlobalValue const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm13TargetJITInfo13getStubLayoutEv", "llvm::TargetJITInfo::getStubLayout()"},
    {"_ZN4llvm13TargetJITInfo16emitFunctionStubEPKNS_8FunctionEPvRNS_14JITCodeEmitterE", "llvm::TargetJITInfo::emitFunctionStub(llvm::Function const*, void*, llvm::JITCodeEmitter&)"},
    {"_ZN4llvm13TargetJITInfo23getLazyResolverFunctionEPFPvS1_E", "llvm::TargetJITInfo::getLazyResolverFunction(void* (*)(void*))"},
    {"_ZN4llvm13TargetJITInfo8relocateEPvPNS_17MachineRelocationEjPh", "llvm::TargetJITInfo::relocate(void*, llvm::MachineRelocation*, unsigned int, unsigned char*)"},
    {"_ZNK4llvm13TargetJITInfo21hasCustomConstantPoolEv", "llvm::TargetJITInfo::hasCustomConstantPool() const"},
    {"_ZNK4llvm13TargetJITInfo19hasCustomJumpTablesEv", "llvm::TargetJITInfo::hasCustomJumpTables() const"},
    {"_ZNK4llvm13TargetJITInfo24allocateSeparateGVMemoryEv", "llvm::TargetJITInfo::allocateSeparateGVMemory() const"},
    {"_ZN4llvm15ARMRegisterInfoD1Ev", "llvm::ARMRegisterInfo::~ARMRegisterInfo()"},
    {"_ZN4llvm12ARMInstrInfoD0Ev", "llvm::ARMInstrInfo::~ARMInstrInfo()"},
    {"_ZNK4llvm15TargetInstrInfo33isReallyTriviallyReMaterializableEPKNS_12MachineInstrEPNS_13AliasAnalysisE", "llvm::TargetInstrInfo::isReallyTriviallyReMaterializable(llvm::MachineInstr const*, llvm::AliasAnalysis*) const"},
    {"_ZNK4llvm15TargetInstrInfo21isCoalescableExtInstrERKNS_12MachineInstrERjS4_S4_", "llvm::TargetInstrInfo::isCoalescableExtInstr(llvm::MachineInstr const&, unsigned int&, unsigned int&, unsigned int&) const"},
    {"_ZNK4llvm15TargetInstrInfo25isLoadFromStackSlotPostFEEPKNS_12MachineInstrERi", "llvm::TargetInstrInfo::isLoadFromStackSlotPostFE(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo20hasLoadFromStackSlotEPKNS_12MachineInstrERPKNS_17MachineMemOperandERi", "llvm::TargetInstrInfo::hasLoadFromStackSlot(llvm::MachineInstr const*, llvm::MachineMemOperand const*&, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo24isStoreToStackSlotPostFEEPKNS_12MachineInstrERi", "llvm::TargetInstrInfo::isStoreToStackSlotPostFE(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo19hasStoreToStackSlotEPKNS_12MachineInstrERPKNS_17MachineMemOperandERi", "llvm::TargetInstrInfo::hasStoreToStackSlot(llvm::MachineInstr const*, llvm::MachineMemOperand const*&, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo21scheduleTwoAddrSourceEPNS_12MachineInstrES2_RKNS_18TargetRegisterInfoE", "llvm::TargetInstrInfo::scheduleTwoAddrSource(llvm::MachineInstr*, llvm::MachineInstr*, llvm::TargetRegisterInfo const&) const"},
    {"_ZNK4llvm15TargetInstrInfo19isLegalToSplitMBBAtERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::TargetInstrInfo::isLegalToSplitMBBAt(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo25isProfitableToDupForIfCvtERNS_17MachineBasicBlockEjff", "llvm::ARMBaseInstrInfo::isProfitableToDupForIfCvt(llvm::MachineBasicBlock&, unsigned int, float, float) const"},
    {"_ZNK4llvm15TargetInstrInfo21foldMemoryOperandImplERNS_15MachineFunctionEPNS_12MachineInstrERKNS_15SmallVectorImplIjEEi", "llvm::TargetInstrInfo::foldMemoryOperandImpl(llvm::MachineFunction&, llvm::MachineInstr*, llvm::SmallVectorImpl<unsigned int> const&, int) const"},
    {"_ZNK4llvm15TargetInstrInfo21foldMemoryOperandImplERNS_15MachineFunctionEPNS_12MachineInstrERKNS_15SmallVectorImplIjEES4_", "llvm::TargetInstrInfo::foldMemoryOperandImpl(llvm::MachineFunction&, llvm::MachineInstr*, llvm::SmallVectorImpl<unsigned int> const&, llvm::MachineInstr*) const"},
    {"_ZNK4llvm15TargetInstrInfo19unfoldMemoryOperandERNS_15MachineFunctionEPNS_12MachineInstrEjbbRNS_15SmallVectorImplIS4_EE", "llvm::TargetInstrInfo::unfoldMemoryOperand(llvm::MachineFunction&, llvm::MachineInstr*, unsigned int, bool, bool, llvm::SmallVectorImpl<llvm::MachineInstr*>&) const"},
    {"_ZNK4llvm15TargetInstrInfo19unfoldMemoryOperandERNS_12SelectionDAGEPNS_6SDNodeERNS_15SmallVectorImplIS4_EE", "llvm::TargetInstrInfo::unfoldMemoryOperand(llvm::SelectionDAG&, llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDNode*>&) const"},
    {"_ZNK4llvm15TargetInstrInfo26getOpcodeAfterMemoryUnfoldEjbbPj", "llvm::TargetInstrInfo::getOpcodeAfterMemoryUnfold(unsigned int, bool, bool, unsigned int*) const"},
    {"_ZNK4llvm15TargetInstrInfo21getNoopForMachoTargetERNS_6MCInstE", "llvm::TargetInstrInfo::getNoopForMachoTarget(llvm::MCInst&) const"},
    {"_ZNK4llvm16ARMBaseInstrInfo12isPredicatedEPKNS_12MachineInstrE", "llvm::ARMBaseInstrInfo::isPredicated(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm15TargetInstrInfo24isSafeToMoveRegClassDefsEPKNS_19TargetRegisterClassE", "llvm::TargetInstrInfo::isSafeToMoveRegClassDefs(llvm::TargetRegisterClass const*) const"},
    {"_ZNK4llvm12ARMInstrInfo15getRegisterInfoEv", "llvm::ARMInstrInfo::getRegisterInfo() const"},
    {"_ZN4llvm16ARMBaseInstrInfoD1Ev", "llvm::ARMBaseInstrInfo::~ARMBaseInstrInfo()"},
    {"_ZN4llvm16ARMBaseInstrInfoD0Ev", "llvm::ARMBaseInstrInfo::~ARMBaseInstrInfo()"},
    {"_ZN4llvm19TargetInstrInfoImplD1Ev", "llvm::TargetInstrInfoImpl::~TargetInstrInfoImpl()"},
    {"_ZN4llvm19TargetInstrInfoImplD0Ev", "llvm::TargetInstrInfoImpl::~TargetInstrInfoImpl()"},
    {"_ZNK4llvm15TargetInstrInfo19isLoadFromStackSlotEPKNS_12MachineInstrERi", "llvm::TargetInstrInfo::isLoadFromStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo18isStoreToStackSlotEPKNS_12MachineInstrERi", "llvm::TargetInstrInfo::isStoreToStackSlot(llvm::MachineInstr const*, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo21convertToThreeAddressERNS_14ilist_iteratorINS_17MachineBasicBlockEEERNS1_INS_12MachineInstrEEEPNS_13LiveVariablesE", "llvm::TargetInstrInfo::convertToThreeAddress(llvm::ilist_iterator<llvm::MachineBasicBlock>&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::LiveVariables*) const"},
    {"_ZNK4llvm15TargetInstrInfo13analyzeBranchERNS_17MachineBasicBlockERPS1_S4_RNS_15SmallVectorImplINS_14MachineOperandEEEb", "llvm::TargetInstrInfo::analyzeBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*&, llvm::MachineBasicBlock*&, llvm::SmallVectorImpl<llvm::MachineOperand>&, bool) const"},
    {"_ZNK4llvm15TargetInstrInfo12removeBranchERNS_17MachineBasicBlockE", "llvm::TargetInstrInfo::removeBranch(llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm15TargetInstrInfo12insertBranchERNS_17MachineBasicBlockEPS1_S3_RKNS_15SmallVectorImplINS_14MachineOperandEEENS_8DebugLocE", "llvm::TargetInstrInfo::insertBranch(llvm::MachineBasicBlock&, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::DebugLoc) const"},
    {"_ZNK4llvm15TargetInstrInfo19isProfitableToIfCvtERNS_17MachineBasicBlockEjjff", "llvm::TargetInstrInfo::isProfitableToIfCvt(llvm::MachineBasicBlock&, unsigned int, unsigned int, float, float) const"},
    {"_ZNK4llvm15TargetInstrInfo19isProfitableToIfCvtERNS_17MachineBasicBlockEjjS2_jjff", "llvm::TargetInstrInfo::isProfitableToIfCvt(llvm::MachineBasicBlock&, unsigned int, unsigned int, llvm::MachineBasicBlock&, unsigned int, unsigned int, float, float) const"},
    {"_ZNK4llvm15TargetInstrInfo25isProfitableToDupForIfCvtERNS_17MachineBasicBlockEjff", "llvm::TargetInstrInfo::isProfitableToDupForIfCvt(llvm::MachineBasicBlock&, unsigned int, float, float) const"},
    {"_ZNK4llvm15TargetInstrInfo11copyPhysRegERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjb", "llvm::TargetInstrInfo::copyPhysReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm15TargetInstrInfo19storeRegToStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjbiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::TargetInstrInfo::storeRegToStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, bool, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm15TargetInstrInfo20loadRegFromStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::TargetInstrInfo::loadRegFromStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm15TargetInstrInfo24emitFrameIndexDebugValueERNS_15MachineFunctionEiyPKNS_6MDNodeENS_8DebugLocE", "llvm::TargetInstrInfo::emitFrameIndexDebugValue(llvm::MachineFunction&, int, unsigned long long, llvm::MDNode const*, llvm::DebugLoc) const"},
    {"_ZNK4llvm15TargetInstrInfo23areLoadsFromSameBasePtrEPNS_6SDNodeES2_RxS3_", "llvm::TargetInstrInfo::areLoadsFromSameBasePtr(llvm::SDNode*, llvm::SDNode*, long long&, long long&) const"},
    {"_ZNK4llvm15TargetInstrInfo23shouldScheduleLoadsNearEPNS_6SDNodeES2_xxj", "llvm::TargetInstrInfo::shouldScheduleLoadsNear(llvm::SDNode*, llvm::SDNode*, long long, long long, unsigned int) const"},
    {"_ZNK4llvm15TargetInstrInfo22ReverseBranchConditionERNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::TargetInstrInfo::ReverseBranchCondition(llvm::SmallVectorImpl<llvm::MachineOperand>&) const"},
    {"_ZNK4llvm15TargetInstrInfo12isPredicatedEPKNS_12MachineInstrE", "llvm::TargetInstrInfo::isPredicated(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm15TargetInstrInfo17SubsumesPredicateERKNS_15SmallVectorImplINS_14MachineOperandEEES5_", "llvm::TargetInstrInfo::SubsumesPredicate(llvm::SmallVectorImpl<llvm::MachineOperand> const&, llvm::SmallVectorImpl<llvm::MachineOperand> const&) const"},
    {"_ZNK4llvm15TargetInstrInfo16DefinesPredicateEPNS_12MachineInstrERSt6vectorINS_14MachineOperandESaIS4_EE", "llvm::TargetInstrInfo::DefinesPredicate(llvm::MachineInstr*, std::vector<llvm::MachineOperand, std::allocator<llvm::MachineOperand> >&) const"},
    {"_ZNK4llvm15TargetInstrInfo12isPredicableEPNS_12MachineInstrE", "llvm::TargetInstrInfo::isPredicable(llvm::MachineInstr*) const"},
    {"_ZNK4llvm15TargetInstrInfo14AnalyzeCompareEPKNS_12MachineInstrERjRiS5_", "llvm::TargetInstrInfo::AnalyzeCompare(llvm::MachineInstr const*, unsigned int&, int&, int&) const"},
    {"_ZNK4llvm15TargetInstrInfo20OptimizeCompareInstrEPNS_12MachineInstrEjiiPKNS_19MachineRegisterInfoE", "llvm::TargetInstrInfo::OptimizeCompareInstr(llvm::MachineInstr*, unsigned int, int, int, llvm::MachineRegisterInfo const*) const"},
    {"_ZNK4llvm15TargetInstrInfo13FoldImmediateEPNS_12MachineInstrES2_jPNS_19MachineRegisterInfoE", "llvm::TargetInstrInfo::FoldImmediate(llvm::MachineInstr*, llvm::MachineInstr*, unsigned int, llvm::MachineRegisterInfo*) const"},
    {"_ZNK4llvm15TargetInstrInfo21hasHighOperandLatencyEPKNS_18InstrItineraryDataEPKNS_19MachineRegisterInfoEPKNS_12MachineInstrEjS9_j", "llvm::TargetInstrInfo::hasHighOperandLatency(llvm::InstrItineraryData const*, llvm::MachineRegisterInfo const*, llvm::MachineInstr const*, unsigned int, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNSt8_Rb_treeIjjSt9_IdentityIjESt4lessIjESaIjEE8_M_eraseEPSt13_Rb_tree_nodeIjE", "std::_Rb_tree<unsigned int, unsigned int, std::_Identity<unsigned int>, std::less<unsigned int>, std::allocator<unsigned int> >::_M_erase(std::_Rb_tree_node<unsigned int>*)"},
    {"_ZN4llvm15ARMRegisterInfoD0Ev", "llvm::ARMRegisterInfo::~ARMRegisterInfo()"},
    {"_ZNK4llvm18TargetRegisterInfo20getCrossCopyRegClassEPKNS_19TargetRegisterClassE", "llvm::TargetRegisterInfo::getCrossCopyRegClass(llvm::TargetRegisterClass const*) const"},
    {"_ZNK4llvm18TargetRegisterInfo20hasReservedSpillSlotERKNS_15MachineFunctionEjRi", "llvm::TargetRegisterInfo::hasReservedSpillSlot(llvm::MachineFunction const&, unsigned int, int&) const"},
    {"_ZNK4llvm18TargetRegisterInfo21saveScavengerRegisterERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERS5_PKNS_19TargetRegisterClassEj", "llvm::TargetRegisterInfo::saveScavengerRegister(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::TargetRegisterClass const*, unsigned int) const"},
    {"_ZN4llvm19ARMBaseRegisterInfoD1Ev", "llvm::ARMBaseRegisterInfo::~ARMBaseRegisterInfo()"},
    {"_ZN4llvm19ARMBaseRegisterInfoD0Ev", "llvm::ARMBaseRegisterInfo::~ARMBaseRegisterInfo()"},
    {"_ZN4llvm18ARMGenRegisterInfoD1Ev", "llvm::ARMGenRegisterInfo::~ARMGenRegisterInfo()"},
    {"_ZN4llvm18ARMGenRegisterInfoD0Ev", "llvm::ARMGenRegisterInfo::~ARMGenRegisterInfo()"},
    {"_ZNK4llvm18TargetRegisterInfo23canCombineSubRegIndicesEPKNS_19TargetRegisterClassERNS_15SmallVectorImplIjEERj", "llvm::TargetRegisterInfo::canCombineSubRegIndices(llvm::TargetRegisterClass const*, llvm::SmallVectorImpl<unsigned int>&, unsigned int&) const"},
    {"_ZNK4llvm18TargetRegisterInfo24getMatchingSuperRegClassEPKNS_19TargetRegisterClassES3_j", "llvm::TargetRegisterInfo::getMatchingSuperRegClass(llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*, unsigned int) const"},
    {"_ZNK4llvm18TargetRegisterInfo18getPointerRegClassEj", "llvm::TargetRegisterInfo::getPointerRegClass(unsigned int) const"},
    {"_ZNK4llvm18TargetRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18TargetRegisterInfo19ResolveRegAllocHintEjjRKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::ResolveRegAllocHint(unsigned int, unsigned int, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18TargetRegisterInfo18UpdateRegAllocHintEjjRNS_15MachineFunctionE", "llvm::TargetRegisterInfo::UpdateRegAllocHint(unsigned int, unsigned int, llvm::MachineFunction&) const"},
    {"_ZNK4llvm18TargetRegisterInfo26requiresRegisterScavengingERKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::requiresRegisterScavenging(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18TargetRegisterInfo28requiresFrameIndexScavengingERKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::requiresFrameIndexScavenging(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18TargetRegisterInfo28requiresVirtualBaseRegistersERKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::requiresVirtualBaseRegisters(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18ARMGenRegisterInfo21needsStackRealignmentERKNS_15MachineFunctionE", "llvm::ARMGenRegisterInfo::needsStackRealignment(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm18TargetRegisterInfo24getFrameIndexInstrOffsetEPKNS_12MachineInstrEi", "llvm::TargetRegisterInfo::getFrameIndexInstrOffset(llvm::MachineInstr const*, int) const"},
    {"_ZNK4llvm18TargetRegisterInfo17needsFrameBaseRegEPNS_12MachineInstrEx", "llvm::TargetRegisterInfo::needsFrameBaseReg(llvm::MachineInstr*, long long) const"},
    {"_ZNK4llvm18TargetRegisterInfo28materializeFrameBaseRegisterEPNS_17MachineBasicBlockEjix", "llvm::TargetRegisterInfo::materializeFrameBaseRegister(llvm::MachineBasicBlock*, unsigned int, int, long long) const"},
    {"_ZNK4llvm18TargetRegisterInfo17resolveFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEjx", "llvm::TargetRegisterInfo::resolveFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, long long) const"},
    {"_ZNK4llvm18TargetRegisterInfo18isFrameOffsetLegalEPKNS_12MachineInstrEx", "llvm::TargetRegisterInfo::isFrameOffsetLegal(llvm::MachineInstr const*, long long) const"},
    {"_ZNK4llvm18TargetRegisterInfo29eliminateCallFramePseudoInstrERNS_15MachineFunctionERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::TargetRegisterInfo::eliminateCallFramePseudoInstr(llvm::MachineFunction&, llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZN4llvm17ARMTargetLoweringD0Ev", "llvm::ARMTargetLowering::~ARMTargetLowering()"},
    {"_ZNK4llvm14TargetLowering17getRepRegClassForENS_3EVTE", "llvm::TargetLowering::getRepRegClassFor(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering21getRepRegClassCostForENS_3EVTE", "llvm::TargetLowering::getRepRegClassCostFor(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering22isVectorClearMaskLegalERKNS_15SmallVectorImplIiEENS_3EVTE", "llvm::TargetLowering::isVectorClearMaskLegal(llvm::SmallVectorImpl<int> const&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering22ShouldShrinkFPConstantENS_3EVTE", "llvm::TargetLowering::ShouldShrinkFPConstant(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering19getOptimalMemOpTypeEyjjbbRNS_15MachineFunctionE", "llvm::TargetLowering::getOptimalMemOpType(unsigned long long, unsigned int, unsigned int, bool, bool, llvm::MachineFunction&) const"},
    {"_ZNK4llvm14TargetLowering25LowerCustomJumpTableEntryEPKNS_20MachineJumpTableInfoEPKNS_17MachineBasicBlockEjRNS_9MCContextE", "llvm::TargetLowering::LowerCustomJumpTableEntry(llvm::MachineJumpTableInfo const*, llvm::MachineBasicBlock const*, unsigned int, llvm::MCContext&) const"},
    {"_ZNK4llvm14TargetLowering22getStackCookieLocationERjS1_", "llvm::TargetLowering::getStackCookieLocation(unsigned int&, unsigned int&) const"},
    {"_ZNK4llvm14TargetLowering20isTypeDesirableForOpEjNS_3EVTE", "llvm::TargetLowering::isTypeDesirableForOp(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering22IsDesirableToPromoteOpENS_7SDValueERNS_3EVTE", "llvm::TargetLowering::IsDesirableToPromoteOp(llvm::SDValue, llvm::EVT&) const"},
    {"_ZNK4llvm14TargetLowering14CanLowerReturnENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERNS_11LLVMContextE", "llvm::TargetLowering::CanLowerReturn(llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::LLVMContext&) const"},
    {"_ZNK4llvm14TargetLowering14isTruncateFreeEPKNS_4TypeES3_", "llvm::TargetLowering::isTruncateFree(llvm::Type const*, llvm::Type const*) const"},
    {"_ZNK4llvm14TargetLowering14isTruncateFreeENS_3EVTES1_", "llvm::TargetLowering::isTruncateFree(llvm::EVT, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering10isZExtFreeEPKNS_4TypeES3_", "llvm::TargetLowering::isZExtFree(llvm::Type const*, llvm::Type const*) const"},
    {"_ZNK4llvm14TargetLowering10isZExtFreeENS_3EVTES1_", "llvm::TargetLowering::isZExtFree(llvm::EVT, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering21isNarrowingProfitableENS_3EVTES1_", "llvm::TargetLowering::isNarrowingProfitable(llvm::EVT, llvm::EVT) const"},
    {"_ZN4llvm16ARMFrameLoweringD0Ev", "llvm::ARMFrameLowering::~ARMFrameLowering()"},
    {"_ZNK4llvm19TargetFrameLowering24getCalleeSavedSpillSlotsERj", "llvm::TargetFrameLowering::getCalleeSavedSpillSlots(unsigned int&) const"},
    {"_ZNK4llvm19TargetFrameLowering31targetHandlesStackFrameRoundingEv", "llvm::TargetFrameLowering::targetHandlesStackFrameRounding() const"},
    {"_ZNK4llvm19TargetFrameLowering35processFunctionBeforeFrameFinalizedERNS_15MachineFunctionE", "llvm::TargetFrameLowering::processFunctionBeforeFrameFinalized(llvm::MachineFunction&) const"},
    {"_ZN4llvm21RegisterTargetMachineINS_18ThumbTargetMachineEE9AllocatorERKNS_6TargetERKSsS7_", "llvm::RegisterTargetMachine<llvm::ThumbTargetMachine>::Allocator(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZN4llvm21RegisterTargetMachineINS_16ARMTargetMachineEE9AllocatorERKNS_6TargetERKSsS7_", "llvm::RegisterTargetMachine<llvm::ARMTargetMachine>::Allocator(llvm::Target const&, std::string const&, std::string const&)"},
    {"_ZNK4llvm19TargetFrameLowering25spillCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::TargetFrameLowering::spillCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm19TargetFrameLowering27restoreCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::TargetFrameLowering::restoreCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm19TargetFrameLowering20hasReservedCallFrameERKNS_15MachineFunctionE", "llvm::TargetFrameLowering::hasReservedCallFrame(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19TargetFrameLowering27canSimplifyCallFramePseudosERKNS_15MachineFunctionE", "llvm::TargetFrameLowering::canSimplifyCallFramePseudos(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19TargetFrameLowering36processFunctionBeforeCalleeSavedScanERNS_15MachineFunctionEPNS_12RegScavengerE", "llvm::TargetFrameLowering::processFunctionBeforeCalleeSavedScan(llvm::MachineFunction&, llvm::RegScavenger*) const"},
    {"_ZN4llvm13ImmutablePass18getAsImmutablePassEv", "llvm::ImmutablePass::getAsImmutablePass()"},
    {"_ZN4llvm13ImmutablePass11runOnModuleERNS_6ModuleE", "llvm::ImmutablePass::runOnModule(llvm::Module&)"},
    {"_ZN4llvm10ARMJITInfoC2Ev", "llvm::ARMJITInfo::ARMJITInfo()"},
    {"_ZN4llvm22ARMElfTargetObjectFile10InitializeERNS_9MCContextERKNS_13TargetMachineE", "llvm::ARMElfTargetObjectFile::Initialize(llvm::MCContext&, llvm::TargetMachine const&)"},
    {"_ZN4llvm22ARMElfTargetObjectFileD1Ev", "llvm::ARMElfTargetObjectFile::~ARMElfTargetObjectFile()"},
    {"_ZN4llvm22ARMElfTargetObjectFileD0Ev", "llvm::ARMElfTargetObjectFile::~ARMElfTargetObjectFile()"},
    {"_ZN4llvm22createMLxExpansionPassEv", "llvm::createMLxExpansionPass()"},
    {"_ZN12_GLOBAL__N_112MLxExpansionD1Ev", "(anonymous namespace)::MLxExpansion::~MLxExpansion()"},
    {"_ZN12_GLOBAL__N_112MLxExpansionD0Ev", "(anonymous namespace)::MLxExpansion::~MLxExpansion()"},
    {"_ZNK12_GLOBAL__N_112MLxExpansion11getPassNameEv", "(anonymous namespace)::MLxExpansion::getPassName() const"},
    {"_ZN12_GLOBAL__N_112MLxExpansion20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MLxExpansion::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm21createNEONMoveFixPassEv", "llvm::createNEONMoveFixPass()"},
    {"_ZN12_GLOBAL__N_115NEONMoveFixPassD1Ev", "(anonymous namespace)::NEONMoveFixPass::~NEONMoveFixPass()"},
    {"_ZN12_GLOBAL__N_115NEONMoveFixPassD0Ev", "(anonymous namespace)::NEONMoveFixPass::~NEONMoveFixPass()"},
    {"_ZNK12_GLOBAL__N_115NEONMoveFixPass11getPassNameEv", "(anonymous namespace)::NEONMoveFixPass::getPassName() const"},
    {"_ZN12_GLOBAL__N_115NEONMoveFixPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::NEONMoveFixPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm8DenseMapIjPKNS_12MachineInstrENS_12DenseMapInfoIjEENS4_IS3_EEE16InsertIntoBucketERKjRKS3_PSt4pairIjS3_E", "llvm::DenseMap<unsigned int, llvm::MachineInstr const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::InsertIntoBucket(unsigned int const&, llvm::MachineInstr const* const&, std::pair<unsigned int, llvm::MachineInstr const*>*)"},
    {"_ZN4llvm8DenseMapIjPKNS_12MachineInstrENS_12DenseMapInfoIjEENS4_IS3_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::MachineInstr const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::grow(unsigned int)"},
    {"_ZNK4llvm19Thumb1FrameLowering20hasReservedCallFrameERKNS_15MachineFunctionE", "llvm::Thumb1FrameLowering::hasReservedCallFrame(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19Thumb1FrameLowering12emitPrologueERNS_15MachineFunctionE", "llvm::Thumb1FrameLowering::emitPrologue(llvm::MachineFunction&) const"},
    {"_ZNK4llvm19Thumb1FrameLowering12emitEpilogueERNS_15MachineFunctionERNS_17MachineBasicBlockE", "llvm::Thumb1FrameLowering::emitEpilogue(llvm::MachineFunction&, llvm::MachineBasicBlock&) const"},
    {"_ZNK4llvm19Thumb1FrameLowering25spillCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::Thumb1FrameLowering::spillCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm19Thumb1FrameLowering27restoreCalleeSavedRegistersERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERKSt6vectorINS_15CalleeSavedInfoESaIS7_EEPKNS_18TargetRegisterInfoE", "llvm::Thumb1FrameLowering::restoreCalleeSavedRegisters(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&, llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm19Thumb1FrameLoweringD1Ev", "llvm::Thumb1FrameLowering::~Thumb1FrameLowering()"},
    {"_ZN4llvm19Thumb1FrameLoweringD0Ev", "llvm::Thumb1FrameLowering::~Thumb1FrameLowering()"},
    {"_ZNSt6vectorIjSaIjEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPjS1_EERKj", "std::vector<unsigned int, std::allocator<unsigned int> >::_M_insert_aux(__gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned int const&)"},
    {"_ZN4llvm9BitVector6resizeEjb", "llvm::BitVector::resize(unsigned int, bool)"},
    {"_ZN4llvm15Thumb1InstrInfoC1ERKNS_12ARMSubtargetE", "llvm::Thumb1InstrInfo::Thumb1InstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZN4llvm15Thumb1InstrInfoC2ERKNS_12ARMSubtargetE", "llvm::Thumb1InstrInfo::Thumb1InstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm15Thumb1InstrInfo18getUnindexedOpcodeEj", "llvm::Thumb1InstrInfo::getUnindexedOpcode(unsigned int) const"},
    {"_ZNK4llvm15Thumb1InstrInfo11copyPhysRegERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjb", "llvm::Thumb1InstrInfo::copyPhysReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm15Thumb1InstrInfo19storeRegToStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjbiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::Thumb1InstrInfo::storeRegToStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, bool, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm15Thumb1InstrInfo20loadRegFromStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::Thumb1InstrInfo::loadRegFromStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm15Thumb1InstrInfoD1Ev", "llvm::Thumb1InstrInfo::~Thumb1InstrInfo()"},
    {"_ZN4llvm15Thumb1InstrInfoD0Ev", "llvm::Thumb1InstrInfo::~Thumb1InstrInfo()"},
    {"_ZNK4llvm15Thumb1InstrInfo15getRegisterInfoEv", "llvm::Thumb1InstrInfo::getRegisterInfo() const"},
    {"_ZN4llvm18Thumb1RegisterInfoD1Ev", "llvm::Thumb1RegisterInfo::~Thumb1RegisterInfo()"},
    {"_ZN4llvm18Thumb1RegisterInfoD0Ev", "llvm::Thumb1RegisterInfo::~Thumb1RegisterInfo()"},
    {"_ZN4llvm18Thumb1RegisterInfoC1ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::Thumb1RegisterInfo::Thumb1RegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm18Thumb1RegisterInfoC2ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::Thumb1RegisterInfo::Thumb1RegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm18Thumb1RegisterInfo17emitLoadConstPoolERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjiNS_5ARMCC9CondCodesEj", "llvm::Thumb1RegisterInfo::emitLoadConstPool(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::DebugLoc, unsigned int, unsigned int, int, llvm::ARMCC::CondCodes, unsigned int) const"},
    {"_ZN4llvm25emitThumbRegPlusImmediateERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEjjiRKNS_15TargetInstrInfoERKNS_19ARMBaseRegisterInfoENS_8DebugLocE", "llvm::emitThumbRegPlusImmediate(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, unsigned int, int, llvm::TargetInstrInfo const&, llvm::ARMBaseRegisterInfo const&, llvm::DebugLoc)"},
    {"_Z24emitThumbRegPlusImmInRegRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEjjibRKNS_15TargetInstrInfoERKNS_19ARMBaseRegisterInfoENS_8DebugLocE", "emitThumbRegPlusImmInReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, unsigned int, int, bool, llvm::TargetInstrInfo const&, llvm::ARMBaseRegisterInfo const&, llvm::DebugLoc)"},
    {"_ZNK4llvm18Thumb1RegisterInfo29eliminateCallFramePseudoInstrERNS_15MachineFunctionERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::Thumb1RegisterInfo::eliminateCallFramePseudoInstr(llvm::MachineFunction&, llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZNK4llvm18Thumb1RegisterInfo17rewriteFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEjjRiRKNS_16ARMBaseInstrInfoE", "llvm::Thumb1RegisterInfo::rewriteFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, unsigned int, int&, llvm::ARMBaseInstrInfo const&) const"},
    {"_ZNK4llvm18Thumb1RegisterInfo17resolveFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEjx", "llvm::Thumb1RegisterInfo::resolveFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, long long) const"},
    {"_ZNK4llvm18Thumb1RegisterInfo21saveScavengerRegisterERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEERS5_PKNS_19TargetRegisterClassEj", "llvm::Thumb1RegisterInfo::saveScavengerRegister(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::TargetRegisterClass const*, unsigned int) const"},
    {"_ZNK4llvm18Thumb1RegisterInfo19eliminateFrameIndexENS_14ilist_iteratorINS_12MachineInstrEEEiPNS_12RegScavengerE", "llvm::Thumb1RegisterInfo::eliminateFrameIndex(llvm::ilist_iterator<llvm::MachineInstr>, int, llvm::RegScavenger*) const"},
    {"_ZN4llvm23createThumb2ITBlockPassEv", "llvm::createThumb2ITBlockPass()"},
    {"_ZN12_GLOBAL__N_117Thumb2ITBlockPassD1Ev", "(anonymous namespace)::Thumb2ITBlockPass::~Thumb2ITBlockPass()"},
    {"_ZN12_GLOBAL__N_117Thumb2ITBlockPassD0Ev", "(anonymous namespace)::Thumb2ITBlockPass::~Thumb2ITBlockPass()"},
    {"_ZNK12_GLOBAL__N_117Thumb2ITBlockPass11getPassNameEv", "(anonymous namespace)::Thumb2ITBlockPass::getPassName() const"},
    {"_ZN12_GLOBAL__N_117Thumb2ITBlockPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::Thumb2ITBlockPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_Z12TrackDefUsesPN4llvm12MachineInstrERNS_8SmallSetIjLj4EEES4_PKNS_18TargetRegisterInfoE", "TrackDefUses(llvm::MachineInstr*, llvm::SmallSet<unsigned int, 4u>&, llvm::SmallSet<unsigned int, 4u>&, llvm::TargetRegisterInfo const*)"},
    {"_ZN4llvm8SmallSetIjLj4EE6insertERKj", "llvm::SmallSet<unsigned int, 4u>::insert(unsigned int const&)"},
    {"_ZN4llvm15Thumb2InstrInfoC1ERKNS_12ARMSubtargetE", "llvm::Thumb2InstrInfo::Thumb2InstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZN4llvm15Thumb2InstrInfoC2ERKNS_12ARMSubtargetE", "llvm::Thumb2InstrInfo::Thumb2InstrInfo(llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm15Thumb2InstrInfo18getUnindexedOpcodeEj", "llvm::Thumb2InstrInfo::getUnindexedOpcode(unsigned int) const"},
    {"_ZNK4llvm15Thumb2InstrInfo23ReplaceTailWithBranchToENS_14ilist_iteratorINS_12MachineInstrEEEPNS_17MachineBasicBlockE", "llvm::Thumb2InstrInfo::ReplaceTailWithBranchTo(llvm::ilist_iterator<llvm::MachineInstr>, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm15Thumb2InstrInfo19isLegalToSplitMBBAtERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::Thumb2InstrInfo::isLegalToSplitMBBAt(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZN4llvm19getITInstrPredicateEPKNS_12MachineInstrERj", "llvm::getITInstrPredicate(llvm::MachineInstr const*, unsigned int&)"},
    {"_ZNK4llvm15Thumb2InstrInfo11copyPhysRegERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjb", "llvm::Thumb2InstrInfo::copyPhysReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc, unsigned int, unsigned int, bool) const"},
    {"_ZNK4llvm15Thumb2InstrInfo19storeRegToStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjbiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::Thumb2InstrInfo::storeRegToStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, bool, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm15Thumb2InstrInfo20loadRegFromStackSlotERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjiPKNS_19TargetRegisterClassEPKNS_18TargetRegisterInfoE", "llvm::Thumb2InstrInfo::loadRegFromStackSlot(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, int, llvm::TargetRegisterClass const*, llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm22emitT2RegPlusImmediateERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjiNS_5ARMCC9CondCodesEjRKNS_16ARMBaseInstrInfoE", "llvm::emitT2RegPlusImmediate(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::DebugLoc, unsigned int, unsigned int, int, llvm::ARMCC::CondCodes, unsigned int, llvm::ARMBaseInstrInfo const&)"},
    {"_ZN4llvm19rewriteT2FrameIndexERNS_12MachineInstrEjjRiRKNS_16ARMBaseInstrInfoE", "llvm::rewriteT2FrameIndex(llvm::MachineInstr&, unsigned int, unsigned int, int&, llvm::ARMBaseInstrInfo const&)"},
    {"_ZNK4llvm15Thumb2InstrInfo21scheduleTwoAddrSourceEPNS_12MachineInstrES2_RKNS_18TargetRegisterInfoE", "llvm::Thumb2InstrInfo::scheduleTwoAddrSource(llvm::MachineInstr*, llvm::MachineInstr*, llvm::TargetRegisterInfo const&) const"},
    {"_ZN4llvm15Thumb2InstrInfoD1Ev", "llvm::Thumb2InstrInfo::~Thumb2InstrInfo()"},
    {"_ZN4llvm15Thumb2InstrInfoD0Ev", "llvm::Thumb2InstrInfo::~Thumb2InstrInfo()"},
    {"_ZNK4llvm15Thumb2InstrInfo15getRegisterInfoEv", "llvm::Thumb2InstrInfo::getRegisterInfo() const"},
    {"_ZN4llvm18Thumb2RegisterInfoD1Ev", "llvm::Thumb2RegisterInfo::~Thumb2RegisterInfo()"},
    {"_ZN4llvm18Thumb2RegisterInfoD0Ev", "llvm::Thumb2RegisterInfo::~Thumb2RegisterInfo()"},
    {"_ZN4llvm18Thumb2RegisterInfoC1ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::Thumb2RegisterInfo::Thumb2RegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZN4llvm18Thumb2RegisterInfoC2ERKNS_16ARMBaseInstrInfoERKNS_12ARMSubtargetE", "llvm::Thumb2RegisterInfo::Thumb2RegisterInfo(llvm::ARMBaseInstrInfo const&, llvm::ARMSubtarget const&)"},
    {"_ZNK4llvm18Thumb2RegisterInfo17emitLoadConstPoolERNS_17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEENS_8DebugLocEjjiNS_5ARMCC9CondCodesEj", "llvm::Thumb2RegisterInfo::emitLoadConstPool(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::DebugLoc, unsigned int, unsigned int, int, llvm::ARMCC::CondCodes, unsigned int) const"},
    {"_ZN4llvm29createThumb2SizeReductionPassEv", "llvm::createThumb2SizeReductionPass()"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduceD1Ev", "(anonymous namespace)::Thumb2SizeReduce::~Thumb2SizeReduce()"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduceD0Ev", "(anonymous namespace)::Thumb2SizeReduce::~Thumb2SizeReduce()"},
    {"_ZNK12_GLOBAL__N_116Thumb2SizeReduce11getPassNameEv", "(anonymous namespace)::Thumb2SizeReduce::getPassName() const"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduce20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::Thumb2SizeReduce::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduce13ReduceTo2AddrERN4llvm17MachineBasicBlockEPNS1_12MachineInstrERKNS_11ReduceEntryEb", "(anonymous namespace)::Thumb2SizeReduce::ReduceTo2Addr(llvm::MachineBasicBlock&, llvm::MachineInstr*, (anonymous namespace)::ReduceEntry const&, bool)"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduce14ReduceToNarrowERN4llvm17MachineBasicBlockEPNS1_12MachineInstrERKNS_11ReduceEntryEb", "(anonymous namespace)::Thumb2SizeReduce::ReduceToNarrow(llvm::MachineBasicBlock&, llvm::MachineInstr*, (anonymous namespace)::ReduceEntry const&, bool)"},
    {"_ZN4llvm14TargetLowering15DAGCombinerInfo13AddToWorklistEPNS_6SDNodeE", "llvm::TargetLowering::DAGCombinerInfo::AddToWorklist(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner13AddToWorkListEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::AddToWorkList(llvm::SDNode*)"},
    {"_ZN4llvm14TargetLowering15DAGCombinerInfo9CombineToEPNS_6SDNodeERKSt6vectorINS_7SDValueESaIS5_EEb", "llvm::TargetLowering::DAGCombinerInfo::CombineTo(llvm::SDNode*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > const&, bool)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner9CombineToEPN4llvm6SDNodeEPKNS1_7SDValueEjb", "(anonymous namespace)::DAGCombiner::CombineTo(llvm::SDNode*, llvm::SDValue const*, unsigned int, bool)"},
    {"_ZN4llvm14TargetLowering15DAGCombinerInfo9CombineToEPNS_6SDNodeENS_7SDValueEb", "llvm::TargetLowering::DAGCombinerInfo::CombineTo(llvm::SDNode*, llvm::SDValue, bool)"},
    {"_ZN4llvm14TargetLowering15DAGCombinerInfo9CombineToEPNS_6SDNodeENS_7SDValueES4_b", "llvm::TargetLowering::DAGCombinerInfo::CombineTo(llvm::SDNode*, llvm::SDValue, llvm::SDValue, bool)"},
    {"_ZN4llvm14TargetLowering15DAGCombinerInfo23CommitTargetLoweringOptERKNS0_17TargetLoweringOptE", "llvm::TargetLowering::DAGCombinerInfo::CommitTargetLoweringOpt(llvm::TargetLowering::TargetLoweringOpt const&)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner23CommitTargetLoweringOptERKN4llvm14TargetLowering17TargetLoweringOptE", "(anonymous namespace)::DAGCombiner::CommitTargetLoweringOpt(llvm::TargetLowering::TargetLoweringOpt const&)"},
    {"_ZN4llvm12SelectionDAG7CombineENS_12CombineLevelERNS_13AliasAnalysisENS_10CodeGenOpt5LevelE", "llvm::SelectionDAG::Combine(llvm::CombineLevel, llvm::AliasAnalysis&, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner7combineEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::combine(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_115WorkListRemoverD1Ev", "(anonymous namespace)::WorkListRemover::~WorkListRemover()"},
    {"_ZN12_GLOBAL__N_115WorkListRemoverD0Ev", "(anonymous namespace)::WorkListRemover::~WorkListRemover()"},
    {"_ZN12_GLOBAL__N_115WorkListRemover11NodeDeletedEPN4llvm6SDNodeES3_", "(anonymous namespace)::WorkListRemover::NodeDeleted(llvm::SDNode*, llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_115WorkListRemover11NodeUpdatedEPN4llvm6SDNodeE", "(anonymous namespace)::WorkListRemover::NodeUpdated(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner5visitEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::visit(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner18SExtPromoteOperandEN4llvm7SDValueENS1_3EVTE", "(anonymous namespace)::DAGCombiner::SExtPromoteOperand(llvm::SDValue, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner18ZExtPromoteOperandEN4llvm7SDValueENS1_3EVTE", "(anonymous namespace)::DAGCombiner::ZExtPromoteOperand(llvm::SDValue, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner14PromoteOperandEN4llvm7SDValueENS1_3EVTERb", "(anonymous namespace)::DAGCombiner::PromoteOperand(llvm::SDValue, llvm::EVT, bool&)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner27ReplaceLoadWithPromotedLoadEPN4llvm6SDNodeES3_", "(anonymous namespace)::DAGCombiner::ReplaceLoadWithPromotedLoad(llvm::SDNode*, llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner8visitXOREPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::visitXOR(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner15FindBetterChainEPN4llvm6SDNodeENS1_7SDValueE", "(anonymous namespace)::DAGCombiner::FindBetterChain(llvm::SDNode*, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner28CombineToPreIndexedLoadStoreEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::CombineToPreIndexedLoadStore(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner29CombineToPostIndexedLoadStoreEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::CombineToPostIndexedLoadStore(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner15GetDemandedBitsEN4llvm7SDValueERKNS1_5APIntE", "(anonymous namespace)::DAGCombiner::GetDemandedBits(llvm::SDValue, llvm::APInt const&)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner20SimplifyDemandedBitsEN4llvm7SDValueERKNS1_5APIntE", "(anonymous namespace)::DAGCombiner::SimplifyDemandedBits(llvm::SDValue, llvm::APInt const&)"},
    {"_Z18CheckForMaskedLoadN4llvm7SDValueES0_S0_", "CheckForMaskedLoad(llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_Z31ShrinkLoadReplaceStoreWithStoreRKSt4pairIjjEN4llvm7SDValueEPNS3_11StoreSDNodeEPN12_GLOBAL__N_111DAGCombinerE", "ShrinkLoadReplaceStoreWithStore(std::pair<unsigned int, unsigned int> const&, llvm::SDValue, llvm::StoreSDNode*, (anonymous namespace)::DAGCombiner*)"},
    {"_ZNK12_GLOBAL__N_111DAGCombiner13FindAliasInfoEPN4llvm6SDNodeERNS1_7SDValueERxRPKNS1_5ValueERiRjRPKNS1_6MDNodeE", "(anonymous namespace)::DAGCombiner::FindAliasInfo(llvm::SDNode*, llvm::SDValue&, long long&, llvm::Value const*&, int&, unsigned int&, llvm::MDNode const*&) const"},
    {"_Z18isNegatibleForFreeN4llvm7SDValueEbj", "isNegatibleForFree(llvm::SDValue, bool, unsigned int)"},
    {"_Z20GetNegatedExpressionN4llvm7SDValueERNS_12SelectionDAGEbj", "GetNegatedExpression(llvm::SDValue, llvm::SelectionDAG&, bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner14SimplifyVBinOpEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::SimplifyVBinOp(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner23CombineConsecutiveLoadsEPN4llvm6SDNodeENS1_3EVTE", "(anonymous namespace)::DAGCombiner::CombineConsecutiveLoads(llvm::SDNode*, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner33ConstantFoldBITCASTofBUILD_VECTOREPN4llvm6SDNodeENS1_3EVTE", "(anonymous namespace)::DAGCombiner::ConstantFoldBITCASTofBUILD_VECTOR(llvm::SDNode*, llvm::EVT)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner15ReduceLoadWidthEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::ReduceLoadWidth(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner20SimplifyDemandedBitsEN4llvm7SDValueE", "(anonymous namespace)::DAGCombiner::SimplifyDemandedBits(llvm::SDValue)"},
    {"_Z23ExtendUsesToFormExtLoadPN4llvm6SDNodeENS_7SDValueEjRNS_11SmallVectorIS1_Lj4EEERKNS_14TargetLoweringE", "ExtendUsesToFormExtLoad(llvm::SDNode*, llvm::SDValue, unsigned int, llvm::SmallVector<llvm::SDNode*, 4u>&, llvm::TargetLowering const&)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner16SimplifySelectCCEN4llvm8DebugLocENS1_7SDValueES3_S3_S3_NS1_3ISD8CondCodeEb", "(anonymous namespace)::DAGCombiner::SimplifySelectCC(llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::ISD::CondCode, bool)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner17SimplifySelectOpsEPN4llvm6SDNodeENS1_7SDValueES4_", "(anonymous namespace)::DAGCombiner::SimplifySelectOps(llvm::SDNode*, llvm::SDValue, llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner20visitShiftByConstantEPN4llvm6SDNodeEj", "(anonymous namespace)::DAGCombiner::visitShiftByConstant(llvm::SDNode*, unsigned int)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner14ReassociateOpsEjN4llvm8DebugLocENS1_7SDValueES3_", "(anonymous namespace)::DAGCombiner::ReassociateOps(unsigned int, llvm::DebugLoc, llvm::SDValue, llvm::SDValue)"},
    {"_Z17isSetCCEquivalentN4llvm7SDValueERS0_S1_S1_", "isSetCCEquivalent(llvm::SDValue, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner32SimplifyBinOpWithSameOpcodeHandsEPN4llvm6SDNodeE", "(anonymous namespace)::DAGCombiner::SimplifyBinOpWithSameOpcodeHands(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_111DAGCombiner26SimplifyNodeWithTwoResultsEPN4llvm6SDNodeEjj", "(anonymous namespace)::DAGCombiner::SimplifyNodeWithTwoResults(llvm::SDNode*, unsigned int, unsigned int)"},
    {"_Z21combineShlAddConstantN4llvm8DebugLocENS_7SDValueES1_RNS_12SelectionDAGE", "combineShlAddConstant(llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SelectionDAG&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm6SDNodeESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > >, llvm::SDNode*>(__gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > >, __gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > >, llvm::SDNode* const&, std::random_access_iterator_tag)"},
    {"_ZNK4llvm3EVT6bitsGEES0_", "llvm::EVT::bitsGE(llvm::EVT) const"},
    {"_ZSt6__findIPPN4llvm6SDNodeES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "llvm::SDNode** std::__find<llvm::SDNode**, llvm::SDNode*>(llvm::SDNode**, llvm::SDNode**, llvm::SDNode* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8FastISel13startNewBlockEv", "llvm::FastISel::startNewBlock()"},
    {"_ZNK4llvm8FastISel14hasTrivialKillEPKNS_5ValueE", "llvm::FastISel::hasTrivialKill(llvm::Value const*) const"},
    {"_ZN4llvm8FastISel14getRegForValueEPKNS_5ValueE", "llvm::FastISel::getRegForValue(llvm::Value const*)"},
    {"_ZN4llvm8FastISel19enterLocalValueAreaEv", "llvm::FastISel::enterLocalValueArea()"},
    {"_ZN4llvm8FastISel22materializeRegForValueEPKNS_5ValueENS_3MVTE", "llvm::FastISel::materializeRegForValue(llvm::Value const*, llvm::MVT)"},
    {"_ZN4llvm8FastISel19leaveLocalValueAreaENS0_9SavePointE", "llvm::FastISel::leaveLocalValueArea(llvm::FastISel::SavePoint)"},
    {"_ZN4llvm8FastISel14SelectOperatorEPKNS_4UserEj", "llvm::FastISel::SelectOperator(llvm::User const*, unsigned int)"},
    {"_ZN4llvm8FastISel17lookUpRegForValueEPKNS_5ValueE", "llvm::FastISel::lookUpRegForValue(llvm::Value const*)"},
    {"_ZN4llvm8FastISel15createResultRegEPKNS_19TargetRegisterClassE", "llvm::FastISel::createResultReg(llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm8FastISel14UpdateValueMapEPKNS_5ValueEj", "llvm::FastISel::UpdateValueMap(llvm::Value const*, unsigned int)"},
    {"_ZN4llvm8FastISel17getRegForGEPIndexEPKNS_5ValueE", "llvm::FastISel::getRegForGEPIndex(llvm::Value const*)"},
    {"_ZN4llvm8FastISel17recomputeInsertPtEv", "llvm::FastISel::recomputeInsertPt()"},
    {"_ZN4llvm8FastISel14SelectBinaryOpEPKNS_4UserEj", "llvm::FastISel::SelectBinaryOp(llvm::User const*, unsigned int)"},
    {"_ZN4llvm8FastISel19SelectGetElementPtrEPKNS_4UserE", "llvm::FastISel::SelectGetElementPtr(llvm::User const*)"},
    {"_ZN4llvm8FastISel12FastEmit_ri_ENS_3MVTEjjbyS1_", "llvm::FastISel::FastEmit_ri_(llvm::MVT, unsigned int, unsigned int, bool, unsigned long long, llvm::MVT)"},
    {"_ZN4llvm8FastISel10SelectCallEPKNS_4UserE", "llvm::FastISel::SelectCall(llvm::User const*)"},
    {"_ZN4llvm8FastISel10SelectCastEPKNS_4UserEj", "llvm::FastISel::SelectCast(llvm::User const*, unsigned int)"},
    {"_ZN4llvm8FastISel18FastEmitZExtFromI1ENS_3MVTEjb", "llvm::FastISel::FastEmitZExtFromI1(llvm::MVT, unsigned int, bool)"},
    {"_ZN4llvm8FastISel13SelectBitCastEPKNS_4UserE", "llvm::FastISel::SelectBitCast(llvm::User const*)"},
    {"_ZN4llvm8FastISel17SelectInstructionEPKNS_11InstructionE", "llvm::FastISel::SelectInstruction(llvm::Instruction const*)"},
    {"_ZN4llvm8FastISel31HandlePHINodesInSuccessorBlocksEPKNS_10BasicBlockE", "llvm::FastISel::HandlePHINodesInSuccessorBlocks(llvm::BasicBlock const*)"},
    {"_ZN4llvm8FastISel14FastEmitBranchEPNS_17MachineBasicBlockENS_8DebugLocE", "llvm::FastISel::FastEmitBranch(llvm::MachineBasicBlock*, llvm::DebugLoc)"},
    {"_ZN4llvm8FastISel10SelectFNegEPKNS_4UserE", "llvm::FastISel::SelectFNeg(llvm::User const*)"},
    {"_ZN4llvm8FastISelC1ERNS_20FunctionLoweringInfoE", "llvm::FastISel::FastISel(llvm::FunctionLoweringInfo&)"},
    {"_ZN4llvm8FastISelC2ERNS_20FunctionLoweringInfoE", "llvm::FastISel::FastISel(llvm::FunctionLoweringInfo&)"},
    {"_ZN4llvm8FastISelD0Ev", "llvm::FastISel::~FastISel()"},
    {"_ZN4llvm8FastISelD1Ev", "llvm::FastISel::~FastISel()"},
    {"_ZN4llvm8FastISelD2Ev", "llvm::FastISel::~FastISel()"},
    {"_ZN4llvm8FastISel9FastEmit_ENS_3MVTES1_j", "llvm::FastISel::FastEmit_(llvm::MVT, llvm::MVT, unsigned int)"},
    {"_ZN4llvm8FastISel10FastEmit_rENS_3MVTES1_jjb", "llvm::FastISel::FastEmit_r(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool)"},
    {"_ZN4llvm8FastISel11FastEmit_rrENS_3MVTES1_jjbjb", "llvm::FastISel::FastEmit_rr(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned int, bool)"},
    {"_ZN4llvm8FastISel10FastEmit_iENS_3MVTES1_jy", "llvm::FastISel::FastEmit_i(llvm::MVT, llvm::MVT, unsigned int, unsigned long long)"},
    {"_ZN4llvm8FastISel10FastEmit_fENS_3MVTES1_jPKNS_10ConstantFPE", "llvm::FastISel::FastEmit_f(llvm::MVT, llvm::MVT, unsigned int, llvm::ConstantFP const*)"},
    {"_ZN4llvm8FastISel11FastEmit_riENS_3MVTES1_jjby", "llvm::FastISel::FastEmit_ri(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned long long)"},
    {"_ZN4llvm8FastISel11FastEmit_rfENS_3MVTES1_jjbPKNS_10ConstantFPE", "llvm::FastISel::FastEmit_rf(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, llvm::ConstantFP const*)"},
    {"_ZN4llvm8FastISel12FastEmit_rriENS_3MVTES1_jjbjby", "llvm::FastISel::FastEmit_rri(llvm::MVT, llvm::MVT, unsigned int, unsigned int, bool, unsigned int, bool, unsigned long long)"},
    {"_ZN4llvm8FastISel12FastEmit_rf_ENS_3MVTEjjbPKNS_10ConstantFPES1_", "llvm::FastISel::FastEmit_rf_(llvm::MVT, unsigned int, unsigned int, bool, llvm::ConstantFP const*, llvm::MVT)"},
    {"_ZN4llvm8FastISel13FastEmitInst_EjPKNS_19TargetRegisterClassE", "llvm::FastISel::FastEmitInst_(unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm8FastISel14FastEmitInst_rEjPKNS_19TargetRegisterClassEjb", "llvm::FastISel::FastEmitInst_r(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool)"},
    {"_ZN4llvm8FastISel15FastEmitInst_rrEjPKNS_19TargetRegisterClassEjbjb", "llvm::FastISel::FastEmitInst_rr(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned int, bool)"},
    {"_ZN4llvm8FastISel15FastEmitInst_riEjPKNS_19TargetRegisterClassEjby", "llvm::FastISel::FastEmitInst_ri(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned long long)"},
    {"_ZN4llvm8FastISel15FastEmitInst_rfEjPKNS_19TargetRegisterClassEjbPKNS_10ConstantFPE", "llvm::FastISel::FastEmitInst_rf(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, llvm::ConstantFP const*)"},
    {"_ZN4llvm8FastISel16FastEmitInst_rriEjPKNS_19TargetRegisterClassEjbjby", "llvm::FastISel::FastEmitInst_rri(unsigned int, llvm::TargetRegisterClass const*, unsigned int, bool, unsigned int, bool, unsigned long long)"},
    {"_ZN4llvm8FastISel14FastEmitInst_iEjPKNS_19TargetRegisterClassEy", "llvm::FastISel::FastEmitInst_i(unsigned int, llvm::TargetRegisterClass const*, unsigned long long)"},
    {"_ZN4llvm8FastISel26FastEmitInst_extractsubregENS_3MVTEjbj", "llvm::FastISel::FastEmitInst_extractsubreg(llvm::MVT, unsigned int, bool, unsigned int)"},
    {"_ZN4llvm8FastISel25TargetMaterializeConstantEPKNS_8ConstantE", "llvm::FastISel::TargetMaterializeConstant(llvm::Constant const*)"},
    {"_ZN4llvm8FastISel23TargetMaterializeAllocaEPKNS_10AllocaInstE", "llvm::FastISel::TargetMaterializeAlloca(llvm::AllocaInst const*)"},
    {"_ZNSt6vectorISt4pairIPN4llvm12MachineInstrEjESaIS4_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS4_S6_EEmRKS4_", "std::vector<std::pair<llvm::MachineInstr*, unsigned int>, std::allocator<std::pair<llvm::MachineInstr*, unsigned int> > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::pair<llvm::MachineInstr*, unsigned int>*, std::vector<std::pair<llvm::MachineInstr*, unsigned int>, std::allocator<std::pair<llvm::MachineInstr*, unsigned int> > > >, unsigned long, std::pair<llvm::MachineInstr*, unsigned int> const&)"},
    {"_ZN4llvm20FunctionLoweringInfoC1ERKNS_14TargetLoweringE", "llvm::FunctionLoweringInfo::FunctionLoweringInfo(llvm::TargetLowering const&)"},
    {"_ZN4llvm20FunctionLoweringInfoC2ERKNS_14TargetLoweringE", "llvm::FunctionLoweringInfo::FunctionLoweringInfo(llvm::TargetLowering const&)"},
    {"_ZN4llvm20FunctionLoweringInfo3setERKNS_8FunctionERNS_15MachineFunctionE", "llvm::FunctionLoweringInfo::set(llvm::Function const&, llvm::MachineFunction&)"},
    {"_ZN4llvm20FunctionLoweringInfo5clearEv", "llvm::FunctionLoweringInfo::clear()"},
    {"_ZN4llvm20FunctionLoweringInfo9CreateRegENS_3EVTE", "llvm::FunctionLoweringInfo::CreateReg(llvm::EVT)"},
    {"_ZN4llvm20FunctionLoweringInfo10CreateRegsEPKNS_4TypeE", "llvm::FunctionLoweringInfo::CreateRegs(llvm::Type const*)"},
    {"_ZN4llvm20FunctionLoweringInfo26setByValArgumentFrameIndexEPKNS_8ArgumentEi", "llvm::FunctionLoweringInfo::setByValArgumentFrameIndex(llvm::Argument const*, int)"},
    {"_ZN4llvm20FunctionLoweringInfo26getByValArgumentFrameIndexEPKNS_8ArgumentE", "llvm::FunctionLoweringInfo::getByValArgumentFrameIndex(llvm::Argument const*)"},
    {"_ZN4llvm12AddCatchInfoERKNS_8CallInstEPNS_17MachineModuleInfoEPNS_17MachineBasicBlockE", "llvm::AddCatchInfo(llvm::CallInst const&, llvm::MachineModuleInfo*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm13CopyCatchInfoEPKNS_10BasicBlockES2_PNS_17MachineModuleInfoERNS_20FunctionLoweringInfoE", "llvm::CopyCatchInfo(llvm::BasicBlock const*, llvm::BasicBlock const*, llvm::MachineModuleInfo*, llvm::FunctionLoweringInfo&)"},
    {"_ZN4llvm8DenseMapIPKNS_8ArgumentEiNS_12DenseMapInfoIS3_EENS4_IiEEE16InsertIntoBucketERKS3_RKiPSt4pairIS3_iE", "llvm::DenseMap<llvm::Argument const*, int, llvm::DenseMapInfo<llvm::Argument const*>, llvm::DenseMapInfo<int> >::InsertIntoBucket(llvm::Argument const* const&, int const&, std::pair<llvm::Argument const*, int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8ArgumentEiNS_12DenseMapInfoIS3_EENS4_IiEEE4growEj", "llvm::DenseMap<llvm::Argument const*, int, llvm::DenseMapInfo<llvm::Argument const*>, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_8ArgumentEiNS_12DenseMapInfoIS3_EENS4_IiEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Argument const*, int, llvm::DenseMapInfo<llvm::Argument const*>, llvm::DenseMapInfo<int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_10AllocaInstEiNS_12DenseMapInfoIS3_EENS4_IiEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::AllocaInst const*, int, llvm::DenseMapInfo<llvm::AllocaInst const*>, llvm::DenseMapInfo<int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEPNS_17MachineBasicBlockENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock const*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::shrink_and_clear()"},
    {"_ZN4llvm12InstrEmitter12CountResultsEPNS_6SDNodeE", "llvm::InstrEmitter::CountResults(llvm::SDNode*)"},
    {"_ZN4llvm12InstrEmitter13CountOperandsEPNS_6SDNodeE", "llvm::InstrEmitter::CountOperands(llvm::SDNode*)"},
    {"_ZN4llvm12InstrEmitter15EmitCopyFromRegEPNS_6SDNodeEjbbjRNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEE", "llvm::InstrEmitter::EmitCopyFromReg(llvm::SDNode*, unsigned int, bool, bool, unsigned int, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZNK4llvm12InstrEmitter24getDstOfOnlyCopyToRegUseEPNS_6SDNodeEj", "llvm::InstrEmitter::getDstOfOnlyCopyToRegUse(llvm::SDNode*, unsigned int) const"},
    {"_ZN4llvm12InstrEmitter22CreateVirtualRegistersEPNS_6SDNodeEPNS_12MachineInstrERKNS_15TargetInstrDescEbbRNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS9_EENSA_IjEEEE", "llvm::InstrEmitter::CreateVirtualRegisters(llvm::SDNode*, llvm::MachineInstr*, llvm::TargetInstrDesc const&, bool, bool, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitter5getVRENS_7SDValueERNS_8DenseMapIS1_jNS_12DenseMapInfoIS1_EENS3_IjEEEE", "llvm::InstrEmitter::getVR(llvm::SDValue, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitter18AddRegisterOperandEPNS_12MachineInstrENS_7SDValueEjPKNS_15TargetInstrDescERNS_8DenseMapIS3_jNS_12DenseMapInfoIS3_EENS8_IjEEEEbbb", "llvm::InstrEmitter::AddRegisterOperand(llvm::MachineInstr*, llvm::SDValue, unsigned int, llvm::TargetInstrDesc const*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, bool, bool, bool)"},
    {"_ZN4llvm12InstrEmitter10AddOperandEPNS_12MachineInstrENS_7SDValueEjPKNS_15TargetInstrDescERNS_8DenseMapIS3_jNS_12DenseMapInfoIS3_EENS8_IjEEEEbbb", "llvm::InstrEmitter::AddOperand(llvm::MachineInstr*, llvm::SDValue, unsigned int, llvm::TargetInstrDesc const*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, bool, bool, bool)"},
    {"_ZN4llvm12InstrEmitter14EmitSubregNodeEPNS_6SDNodeERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEEbb", "llvm::InstrEmitter::EmitSubregNode(llvm::SDNode*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, bool, bool)"},
    {"_ZN4llvm12InstrEmitter22EmitCopyToRegClassNodeEPNS_6SDNodeERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEE", "llvm::InstrEmitter::EmitCopyToRegClassNode(llvm::SDNode*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitter15EmitRegSequenceEPNS_6SDNodeERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEEbb", "llvm::InstrEmitter::EmitRegSequence(llvm::SDNode*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, bool, bool)"},
    {"_ZN4llvm12InstrEmitter12EmitDbgValueEPNS_10SDDbgValueERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEE", "llvm::InstrEmitter::EmitDbgValue(llvm::SDDbgValue*, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitter15EmitMachineNodeEPNS_6SDNodeEbbRNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEE", "llvm::InstrEmitter::EmitMachineNode(llvm::SDNode*, bool, bool, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitter15EmitSpecialNodeEPNS_6SDNodeEbbRNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS4_EENS5_IjEEEE", "llvm::InstrEmitter::EmitSpecialNode(llvm::SDNode*, bool, bool, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm12InstrEmitterC1EPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::InstrEmitter::InstrEmitter(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm12InstrEmitterC2EPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::InstrEmitter::InstrEmitter(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS1_EENS2_IjEEE6insertERKSt4pairIS1_jE", "llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<llvm::SDValue, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS1_EENS2_IjEEE4findERKS1_", "llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >::find(llvm::SDValue const&)"},
    {"_ZN4llvm8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS1_EENS2_IjEEE16InsertIntoBucketERKS1_RKjPSt4pairIS1_jE", "llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::SDValue const&, unsigned int const&, std::pair<llvm::SDValue, unsigned int>*)"},
    {"_ZN4llvm8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS1_EENS2_IjEEE4growEj", "llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm12SelectionDAG8LegalizeENS_10CodeGenOpt5LevelE", "llvm::SelectionDAG::Legalize(llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize10LegalizeOpEN4llvm7SDValueE", "(anonymous namespace)::SelectionDAGLegalize::LegalizeOp(llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize10ExpandNodeEPN4llvm6SDNodeERNS1_15SmallVectorImplINS1_7SDValueEEE", "(anonymous namespace)::SelectionDAGLegalize::ExpandNode(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&)"},
    {"_Z24FindCallEndFromCallStartPN4llvm6SDNodeEi", "FindCallEndFromCallStart(llvm::SDNode*, int)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize28LegalizeAllNodesNotLeadingToEPN4llvm6SDNodeES3_RNS1_11SmallPtrSetIS3_Lj32EEE", "(anonymous namespace)::SelectionDAGLegalize::LegalizeAllNodesNotLeadingTo(llvm::SDNode*, llvm::SDNode*, llvm::SmallPtrSet<llvm::SDNode*, 32u>&)"},
    {"_Z19ExpandUnalignedLoadPN4llvm10LoadSDNodeERNS_12SelectionDAGERKNS_14TargetLoweringE", "ExpandUnalignedLoad(llvm::LoadSDNode*, llvm::SelectionDAG&, llvm::TargetLowering const&)"},
    {"_Z20ExpandUnalignedStorePN4llvm11StoreSDNodeERNS_12SelectionDAGERKNS_14TargetLoweringE", "ExpandUnalignedStore(llvm::StoreSDNode*, llvm::SelectionDAG&, llvm::TargetLowering const&)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize29ExpandVectorBuildThroughStackEPN4llvm6SDNodeE", "(anonymous namespace)::SelectionDAGLegalize::ExpandVectorBuildThroughStack(llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize16EmitStackConvertEN4llvm7SDValueENS1_3EVTES3_NS1_8DebugLocE", "(anonymous namespace)::SelectionDAGLegalize::EmitStackConvert(llvm::SDValue, llvm::EVT, llvm::EVT, llvm::DebugLoc)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize35ExpandExtractFromVectorThroughStackEN4llvm7SDValueE", "(anonymous namespace)::SelectionDAGLegalize::ExpandExtractFromVectorThroughStack(llvm::SDValue)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize13ExpandLibCallEN4llvm5RTLIB7LibcallEPNS1_6SDNodeEb", "(anonymous namespace)::SelectionDAGLegalize::ExpandLibCall(llvm::RTLIB::Libcall, llvm::SDNode*, bool)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize21LegalizeSetCCCondCodeEN4llvm3EVTERNS1_7SDValueES4_S4_NS1_8DebugLocE", "(anonymous namespace)::SelectionDAGLegalize::LegalizeSetCCCondCode(llvm::EVT, llvm::SDValue&, llvm::SDValue&, llvm::SDValue&, llvm::DebugLoc)"},
    {"_ZN12_GLOBAL__N_120SelectionDAGLegalize18ExpandChainLibCallEN4llvm5RTLIB7LibcallEPNS1_6SDNodeEb", "(anonymous namespace)::SelectionDAGLegalize::ExpandChainLibCall(llvm::RTLIB::Libcall, llvm::SDNode*, bool)"},
    {"_Z9SplatBytejh", "SplatByte(unsigned int, unsigned char)"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E16shrink_and_clearEv", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::shrink_and_clear()"},
    {"_ZNK4llvm14TargetLowering18getTypeToPromoteToEjNS_3EVTE", "llvm::TargetLowering::getTypeToPromoteTo(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm3EVT21getHalfSizedIntegerVTERNS_11LLVMContextE", "llvm::EVT::getHalfSizedIntegerVT(llvm::LLVMContext&) const"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E6insertERKSt4pairIS1_S1_E", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::insert(std::pair<llvm::SDValue, llvm::SDValue> const&)"},
    {"_ZN4llvm16DAGTypeLegalizer17SoftenFloatResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::SoftenFloatResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer22SoftenFloatRes_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25SoftenFloatRes_BUILD_PAIREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_BUILD_PAIR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25SoftenFloatRes_ConstantFPEPNS_16ConstantFPSDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_ConstantFP(llvm::ConstantFPSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer33SoftenFloatRes_EXTRACT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_EXTRACT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FABSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FABS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FADDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FADD(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FCEILEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FCEIL(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24SoftenFloatRes_FCOPYSIGNEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FCOPYSIGN(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FCOSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FCOS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FDIVEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FDIV(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FEXPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FEXP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FEXP2EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FEXP2(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21SoftenFloatRes_FFLOOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FFLOOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FLOGEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FLOG(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FLOG2EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FLOG2(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21SoftenFloatRes_FLOG10EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FLOG10(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FMULEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FMUL(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25SoftenFloatRes_FNEARBYINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FNEARBYINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FNEGEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FNEG(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24SoftenFloatRes_FP_EXTENDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FP_EXTEND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23SoftenFloatRes_FP_ROUNDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FP_ROUND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer27SoftenFloatRes_FP16_TO_FP32EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FP16_TO_FP32(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FPOWEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FPOW(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FPOWIEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FPOWI(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FREMEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FREM(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FRINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FRINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FSINEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FSIN(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_FSQRTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FSQRT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_FSUBEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FSUB(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21SoftenFloatRes_FTRUNCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_FTRUNC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatRes_LOADEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_LOAD(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21SoftenFloatRes_SELECTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_SELECT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24SoftenFloatRes_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25SoftenFloatRes_XINT_TO_FPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_XINT_TO_FP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_UNDEFEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_UNDEF(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20SoftenFloatRes_VAARGEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatRes_VAARG(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18SoftenFloatOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::SoftenFloatOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer21SoftenFloatOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatOp_BR_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_BR_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22SoftenFloatOp_FP_ROUNDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_FP_ROUND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24SoftenFloatOp_FP_TO_SINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_FP_TO_SINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24SoftenFloatOp_FP_TO_UINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_FP_TO_UINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26SoftenFloatOp_FP32_TO_FP16EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_FP32_TO_FP16(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23SoftenFloatOp_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatOp_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SoftenFloatOp_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenFloatOp_STOREEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::SoftenFloatOp_STORE(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19SoftenSetCCOperandsERNS_7SDValueES2_RNS_3ISD8CondCodeENS_8DebugLocE", "llvm::DAGTypeLegalizer::SoftenSetCCOperands(llvm::SDValue&, llvm::SDValue&, llvm::ISD::CondCode&, llvm::DebugLoc)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandFloatResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandFloatResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer25ExpandFloatRes_ConstantFPEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_ConstantFP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FABSEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FABS(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FADDEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FADD(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FCEILEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FCEIL(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandFloatRes_FCOPYSIGNEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FCOPYSIGN(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FCOSEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FCOS(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FDIVEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FDIV(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FEXPEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FEXP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FEXP2EPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FEXP2(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandFloatRes_FFLOOREPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FFLOOR(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FLOGEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FLOG(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FLOG2EPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FLOG2(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandFloatRes_FLOG10EPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FLOG10(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FMULEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FMUL(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer25ExpandFloatRes_FNEARBYINTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FNEARBYINT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FNEGEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FNEG(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandFloatRes_FP_EXTENDEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FP_EXTEND(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FPOWEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FPOW(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FPOWIEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FPOWI(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FRINTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FRINT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FSINEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FSIN(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandFloatRes_FSQRTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FSQRT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_FSUBEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FSUB(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandFloatRes_FTRUNCEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_FTRUNC(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatRes_LOADEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_LOAD(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer25ExpandFloatRes_XINT_TO_FPEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandFloatRes_XINT_TO_FP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18ExpandFloatOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandFloatOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatOp_BR_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_BR_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ExpandFloatOp_FP_ROUNDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_FP_ROUND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandFloatOp_FP_TO_SINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_FP_TO_SINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandFloatOp_FP_TO_UINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_FP_TO_UINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandFloatOp_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatOp_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandFloatOp_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandFloatOp_STOREEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandFloatOp_STORE(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer24FloatExpandSetCCOperandsERNS_7SDValueES2_RNS_3ISD8CondCodeENS_8DebugLocE", "llvm::DAGTypeLegalizer::FloatExpandSetCCOperands(llvm::SDValue&, llvm::SDValue&, llvm::ISD::CondCode&, llvm::DebugLoc)"},
    {"_ZN4llvm16DAGTypeLegalizer20PromoteIntegerResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntegerResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntRes_AssertSextEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_AssertSext(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntRes_AssertZextEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_AssertZext(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21PromoteIntRes_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_BSWAPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_BSWAP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntRes_BUILD_PAIREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_BUILD_PAIR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntRes_ConstantEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_Constant(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer28PromoteIntRes_CONVERT_RNDSATEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_CONVERT_RNDSAT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntRes_CTLZEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_CTLZ(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_CTPOPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_CTPOP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntRes_CTTZEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_CTTZ(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer32PromoteIntRes_EXTRACT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_EXTRACT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntRes_LOADEPNS_10LoadSDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_LOAD(llvm::LoadSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20PromoteIntRes_SELECTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SELECT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntRes_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17PromoteIntRes_SHLEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SHL(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer31PromoteIntRes_SIGN_EXTEND_INREGEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SIGN_EXTEND_INREG(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17PromoteIntRes_SRAEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SRA(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17PromoteIntRes_SRLEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SRL(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntRes_TRUNCATEEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_TRUNCATE(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_UNDEFEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_UNDEF(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_VAARGEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_VAARG(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntRes_INT_EXTENDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_INT_EXTEND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntRes_FP_TO_XINTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_FP_TO_XINT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26PromoteIntRes_FP32_TO_FP16EPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_FP32_TO_FP16(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer28PromoteIntRes_SimpleIntBinOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SimpleIntBinOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntRes_SDIVEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_SDIV(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntRes_UDIVEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_UDIV(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntRes_SADDSUBOEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntRes_SADDSUBO(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntRes_UADDSUBOEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntRes_UADDSUBO(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntRes_XMULOEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntRes_XMULO(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer21PromoteIntRes_Atomic1EPNS_12AtomicSDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_Atomic1(llvm::AtomicSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21PromoteIntRes_Atomic2EPNS_12AtomicSDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_Atomic2(llvm::AtomicSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntRes_OverflowEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntRes_Overflow(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21PromoteIntegerOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntegerOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntOp_ANY_EXTENDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_ANY_EXTEND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20PromoteIntOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntOp_BR_CCEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_BR_CC(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntOp_BRCONDEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_BRCOND(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntOp_BUILD_PAIREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_BUILD_PAIR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25PromoteIntOp_BUILD_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_BUILD_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer27PromoteIntOp_CONVERT_RNDSATEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_CONVERT_RNDSAT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer30PromoteIntOp_INSERT_VECTOR_ELTEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_INSERT_VECTOR_ELT(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntOp_MEMBARRIEREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_MEMBARRIER(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29PromoteIntOp_SCALAR_TO_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_SCALAR_TO_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19PromoteIntOp_SELECTEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_SELECT(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer22PromoteIntOp_SELECT_CCEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_SELECT_CC(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntOp_SETCCEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_SETCC(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntOp_SIGN_EXTENDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_SIGN_EXTEND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntOp_SINT_TO_FPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_SINT_TO_FP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntOp_STOREEPNS_11StoreSDNodeEj", "llvm::DAGTypeLegalizer::PromoteIntOp_STORE(llvm::StoreSDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer21PromoteIntOp_TRUNCATEEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_TRUNCATE(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23PromoteIntOp_UINT_TO_FPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_UINT_TO_FP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24PromoteIntOp_ZERO_EXTENDEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_ZERO_EXTEND(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18PromoteIntOp_ShiftEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::PromoteIntOp_Shift(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20PromoteSetCCOperandsERNS_7SDValueES2_NS_3ISD8CondCodeE", "llvm::DAGTypeLegalizer::PromoteSetCCOperands(llvm::SDValue&, llvm::SDValue&, llvm::ISD::CondCode)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandIntegerResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandIntegerResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandIntRes_ANY_EXTENDEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_ANY_EXTEND(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandIntRes_AssertSextEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_AssertSext(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandIntRes_AssertZextEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_AssertZext(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18ExpandIntRes_BSWAPEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_BSWAP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandIntRes_ConstantEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_Constant(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_CTLZEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_CTLZ(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18ExpandIntRes_CTPOPEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_CTPOP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_CTTZEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_CTTZ(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandIntRes_FP_TO_SINTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_FP_TO_SINT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer23ExpandIntRes_FP_TO_UINTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_FP_TO_UINT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_LOADEPNS_10LoadSDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_LOAD(llvm::LoadSDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer16ExpandIntRes_MULEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_MUL(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_SDIVEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_SDIV(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandIntRes_SIGN_EXTENDEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_SIGN_EXTEND(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer30ExpandIntRes_SIGN_EXTEND_INREGEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_SIGN_EXTEND_INREG(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_SREMEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_SREM(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandIntRes_TRUNCATEEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_TRUNCATE(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_UDIVEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_UDIV(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntRes_UREMEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_UREM(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandIntRes_ZERO_EXTENDEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_ZERO_EXTEND(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer12ExpandAtomicEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandAtomic(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandIntRes_LogicalEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_Logical(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19ExpandIntRes_ADDSUBEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_ADDSUB(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandIntRes_ADDSUBCEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_ADDSUBC(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandIntRes_ADDSUBEEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_ADDSUBE(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18ExpandIntRes_ShiftEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_Shift(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandIntRes_SADDSUBOEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_SADDSUBO(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandIntRes_UADDSUBOEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_UADDSUBO(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer22ExpandIntRes_UMULSMULOEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandIntRes_UMULSMULO(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandShiftByConstantEPNS_6SDNodeEjRNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandShiftByConstant(llvm::SDNode*, unsigned int, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer29ExpandShiftWithKnownAmountBitEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandShiftWithKnownAmountBit(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer31ExpandShiftWithUnknownAmountBitEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandShiftWithUnknownAmountBit(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandIntegerOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandIntegerOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntOp_BR_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_BR_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandIntOp_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntOp_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ExpandIntOp_SINT_TO_FPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_SINT_TO_FP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntOp_STOREEPNS_11StoreSDNodeEj", "llvm::DAGTypeLegalizer::ExpandIntOp_STORE(llvm::StoreSDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandIntOp_TRUNCATEEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_TRUNCATE(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ExpandIntOp_UINT_TO_FPEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_UINT_TO_FP(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandIntOp_ShiftEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_Shift(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ExpandIntOp_RETURNADDREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandIntOp_RETURNADDR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26IntegerExpandSetCCOperandsERNS_7SDValueES2_RNS_3ISD8CondCodeENS_8DebugLocE", "llvm::DAGTypeLegalizer::IntegerExpandSetCCOperands(llvm::SDValue&, llvm::SDValue&, llvm::ISD::CondCode&, llvm::DebugLoc)"},
    {"_ZN4llvm16DAGTypeLegalizer22PerformExpensiveChecksEv", "llvm::DAGTypeLegalizer::PerformExpensiveChecks()"},
    {"_ZN4llvm16DAGTypeLegalizer3runEv", "llvm::DAGTypeLegalizer::run()"},
    {"_ZN4llvm16DAGTypeLegalizer14AnalyzeNewNodeEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::AnalyzeNewNode(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer16ReplaceValueWithENS_7SDValueES1_", "llvm::DAGTypeLegalizer::ReplaceValueWith(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer11ExpungeNodeEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpungeNode(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer15AnalyzeNewValueERNS_7SDValueE", "llvm::DAGTypeLegalizer::AnalyzeNewValue(llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer10RemapValueERNS_7SDValueE", "llvm::DAGTypeLegalizer::RemapValue(llvm::SDValue&)"},
    {"_ZN12_GLOBAL__N_118NodeUpdateListenerD1Ev", "(anonymous namespace)::NodeUpdateListener::~NodeUpdateListener()"},
    {"_ZN4llvm16DAGTypeLegalizer18SetPromotedIntegerENS_7SDValueES1_", "llvm::DAGTypeLegalizer::SetPromotedInteger(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer16SetSoftenedFloatENS_7SDValueES1_", "llvm::DAGTypeLegalizer::SetSoftenedFloat(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer19SetScalarizedVectorENS_7SDValueES1_", "llvm::DAGTypeLegalizer::SetScalarizedVector(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer18GetExpandedIntegerENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetExpandedInteger(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18SetExpandedIntegerENS_7SDValueES1_S1_", "llvm::DAGTypeLegalizer::SetExpandedInteger(llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer16GetExpandedFloatENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetExpandedFloat(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer16SetExpandedFloatENS_7SDValueES1_S1_", "llvm::DAGTypeLegalizer::SetExpandedFloat(llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer14GetSplitVectorENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetSplitVector(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer14SetSplitVectorENS_7SDValueES1_S1_", "llvm::DAGTypeLegalizer::SetSplitVector(llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer16SetWidenedVectorENS_7SDValueES1_", "llvm::DAGTypeLegalizer::SetWidenedVector(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer19BitConvertToIntegerENS_7SDValueE", "llvm::DAGTypeLegalizer::BitConvertToInteger(llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer31BitConvertVectorToIntegerVectorENS_7SDValueE", "llvm::DAGTypeLegalizer::BitConvertVectorToIntegerVector(llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer20CreateStackStoreLoadENS_7SDValueENS_3EVTE", "llvm::DAGTypeLegalizer::CreateStackStoreLoad(llvm::SDValue, llvm::EVT)"},
    {"_ZN4llvm16DAGTypeLegalizer15CustomLowerNodeEPNS_6SDNodeENS_3EVTEb", "llvm::DAGTypeLegalizer::CustomLowerNode(llvm::SDNode*, llvm::EVT, bool)"},
    {"_ZN4llvm16DAGTypeLegalizer20CustomWidenLowerNodeEPNS_6SDNodeENS_3EVTE", "llvm::DAGTypeLegalizer::CustomWidenLowerNode(llvm::SDNode*, llvm::EVT)"},
    {"_ZN4llvm16DAGTypeLegalizer15GetSplitDestVTsENS_3EVTERS1_S2_", "llvm::DAGTypeLegalizer::GetSplitDestVTs(llvm::EVT, llvm::EVT&, llvm::EVT&)"},
    {"_ZN4llvm16DAGTypeLegalizer15GetPairElementsENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetPairElements(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer23GetVectorElementPointerENS_7SDValueENS_3EVTES1_", "llvm::DAGTypeLegalizer::GetVectorElementPointer(llvm::SDValue, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer12JoinIntegersENS_7SDValueES1_", "llvm::DAGTypeLegalizer::JoinIntegers(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer10LibCallifyENS_5RTLIB7LibcallEPNS_6SDNodeEb", "llvm::DAGTypeLegalizer::LibCallify(llvm::RTLIB::Libcall, llvm::SDNode*, bool)"},
    {"_ZN4llvm16DAGTypeLegalizer11MakeLibCallENS_5RTLIB7LibcallENS_3EVTEPKNS_7SDValueEjbNS_8DebugLocE", "llvm::DAGTypeLegalizer::MakeLibCall(llvm::RTLIB::Libcall, llvm::EVT, llvm::SDValue const*, unsigned int, bool, llvm::DebugLoc)"},
    {"_ZN4llvm16DAGTypeLegalizer18ExpandChainLibCallENS_5RTLIB7LibcallEPNS_6SDNodeEb", "llvm::DAGTypeLegalizer::ExpandChainLibCall(llvm::RTLIB::Libcall, llvm::SDNode*, bool)"},
    {"_ZN4llvm16DAGTypeLegalizer20PromoteTargetBooleanENS_7SDValueENS_3EVTE", "llvm::DAGTypeLegalizer::PromoteTargetBoolean(llvm::SDValue, llvm::EVT)"},
    {"_ZN4llvm16DAGTypeLegalizer12SplitIntegerENS_7SDValueENS_3EVTES2_RS1_S3_", "llvm::DAGTypeLegalizer::SplitInteger(llvm::SDValue, llvm::EVT, llvm::EVT, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer12SplitIntegerENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::SplitInteger(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm12SelectionDAG13LegalizeTypesEv", "llvm::SelectionDAG::LegalizeTypes()"},
    {"_ZN12_GLOBAL__N_118NodeUpdateListenerD0Ev", "(anonymous namespace)::NodeUpdateListener::~NodeUpdateListener()"},
    {"_ZN12_GLOBAL__N_118NodeUpdateListener11NodeDeletedEPN4llvm6SDNodeES3_", "(anonymous namespace)::NodeUpdateListener::NodeDeleted(llvm::SDNode*, llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_118NodeUpdateListener11NodeUpdatedEPN4llvm6SDNodeE", "(anonymous namespace)::NodeUpdateListener::NodeUpdated(llvm::SDNode*)"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E4findERKS1_", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::find(llvm::SDValue const&)"},
    {"_ZN4llvm8DenseMapINS_7SDValueESt4pairIS1_S1_ENS_12DenseMapInfoIS1_EENS4_IS3_EEE4findERKS1_", "llvm::DenseMap<llvm::SDValue, std::pair<llvm::SDValue, llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<std::pair<llvm::SDValue, llvm::SDValue> > >::find(llvm::SDValue const&)"},
    {"_ZNK4llvm16DAGTypeLegalizer13getTypeActionENS_3EVTE", "llvm::DAGTypeLegalizer::getTypeAction(llvm::EVT) const"},
    {"_ZN4llvm8DenseMapINS_7SDValueESt4pairIS1_S1_ENS_12DenseMapInfoIS1_EENS4_IS3_EEE16FindAndConstructERKS1_", "llvm::DenseMap<llvm::SDValue, std::pair<llvm::SDValue, llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<std::pair<llvm::SDValue, llvm::SDValue> > >::FindAndConstruct(llvm::SDValue const&)"},
    {"_ZN4llvm8DenseMapINS_7SDValueESt4pairIS1_S1_ENS_12DenseMapInfoIS1_EENS4_IS3_EEE16InsertIntoBucketERKS1_RKS3_PS2_IS1_S3_E", "llvm::DenseMap<llvm::SDValue, std::pair<llvm::SDValue, llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<std::pair<llvm::SDValue, llvm::SDValue> > >::InsertIntoBucket(llvm::SDValue const&, std::pair<llvm::SDValue, llvm::SDValue> const&, std::pair<llvm::SDValue, std::pair<llvm::SDValue, llvm::SDValue> >*)"},
    {"_ZN4llvm8DenseMapINS_7SDValueESt4pairIS1_S1_ENS_12DenseMapInfoIS1_EENS4_IS3_EEE4growEj", "llvm::DenseMap<llvm::SDValue, std::pair<llvm::SDValue, llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<std::pair<llvm::SDValue, llvm::SDValue> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E16FindAndConstructERKS1_", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::FindAndConstruct(llvm::SDValue const&)"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E16InsertIntoBucketERKS1_S6_PSt4pairIS1_S1_E", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::InsertIntoBucket(llvm::SDValue const&, llvm::SDValue const&, std::pair<llvm::SDValue, llvm::SDValue>*)"},
    {"_ZN4llvm8DenseMapINS_7SDValueES1_NS_12DenseMapInfoIS1_EES3_E4growEj", "llvm::DenseMap<llvm::SDValue, llvm::SDValue, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<llvm::SDValue> >::grow(unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizerD2Ev", "llvm::DAGTypeLegalizer::~DAGTypeLegalizer()"},
    {"_ZN4llvm16DAGTypeLegalizerC2ERNS_12SelectionDAGE", "llvm::DAGTypeLegalizer::DAGTypeLegalizer(llvm::SelectionDAG&)"},
    {"_ZN4llvm16DAGTypeLegalizer17ExpandRes_BITCASTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_BITCAST(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandRes_BUILD_PAIREPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_BUILD_PAIR(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer25ExpandRes_EXTRACT_ELEMENTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_EXTRACT_ELEMENT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer28ExpandRes_EXTRACT_VECTOR_ELTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_EXTRACT_VECTOR_ELT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandRes_NormalLoadEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_NormalLoad(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer15ExpandRes_VAARGEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::ExpandRes_VAARG(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer16ExpandOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ExpandOp_BUILD_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandOp_BUILD_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24ExpandOp_EXTRACT_ELEMENTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandOp_EXTRACT_ELEMENT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26ExpandOp_INSERT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandOp_INSERT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25ExpandOp_SCALAR_TO_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ExpandOp_SCALAR_TO_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20ExpandOp_NormalStoreEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ExpandOp_NormalStore(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer21SplitRes_MERGE_VALUESEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitRes_MERGE_VALUES(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer15SplitRes_SELECTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitRes_SELECT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18SplitRes_SELECT_CCEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitRes_SELECT_CC(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer14SplitRes_UNDEFEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitRes_UNDEF(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZNSt6vectorIN4llvm7SDValueESaIS1_EE7reserveEm", "std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >::reserve(unsigned long)"},
    {"_ZN4llvm16DAGTypeLegalizer10GetSplitOpENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetSplitOp(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm12SelectionDAG15LegalizeVectorsEv", "llvm::SelectionDAG::LegalizeVectors()"},
    {"_ZN12_GLOBAL__N_115VectorLegalizer10LegalizeOpEN4llvm7SDValueE", "(anonymous namespace)::VectorLegalizer::LegalizeOp(llvm::SDValue)"},
    {"_ZN4llvm16DAGTypeLegalizer21ScalarizeVectorResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ScalarizeVectorResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer23ScalarizeVecRes_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer30ScalarizeVecRes_CONVERT_RNDSATEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_CONVERT_RNDSAT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer33ScalarizeVecRes_EXTRACT_SUBVECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_EXTRACT_SUBVECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23ScalarizeVecRes_InregOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_InregOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ScalarizeVecRes_FPOWIEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_FPOWI(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer33ScalarizeVecRes_INSERT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_INSERT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20ScalarizeVecRes_LOADEPNS_10LoadSDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_LOAD(llvm::LoadSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer32ScalarizeVecRes_SCALAR_TO_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_SCALAR_TO_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ScalarizeVecRes_SELECTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_SELECT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25ScalarizeVecRes_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ScalarizeVecRes_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ScalarizeVecRes_UNDEFEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_UNDEF(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer30ScalarizeVecRes_VECTOR_SHUFFLEEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_VECTOR_SHUFFLE(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ScalarizeVecRes_VSETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_VSETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer23ScalarizeVecRes_UnaryOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_UnaryOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21ScalarizeVecRes_BinOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecRes_BinOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer22ScalarizeVectorOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::ScalarizeVectorOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer22ScalarizeVecOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29ScalarizeVecOp_CONCAT_VECTORSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecOp_CONCAT_VECTORS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer33ScalarizeVecOp_EXTRACT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::ScalarizeVecOp_EXTRACT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20ScalarizeVecOp_STOREEPNS_11StoreSDNodeEj", "llvm::DAGTypeLegalizer::ScalarizeVecOp_STORE(llvm::StoreSDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer17SplitVectorResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::SplitVectorResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19SplitVecRes_BITCASTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_BITCAST(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer24SplitVecRes_BUILD_VECTOREPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_BUILD_VECTOR(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer26SplitVecRes_CONCAT_VECTORSEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_CONCAT_VECTORS(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer26SplitVecRes_CONVERT_RNDSATEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_CONVERT_RNDSAT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer29SplitVecRes_EXTRACT_SUBVECTOREPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_EXTRACT_SUBVECTOR(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19SplitVecRes_InregOpEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_InregOp(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17SplitVecRes_FPOWIEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_FPOWI(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer29SplitVecRes_INSERT_VECTOR_ELTEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_INSERT_VECTOR_ELT(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer28SplitVecRes_SCALAR_TO_VECTOREPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_SCALAR_TO_VECTOR(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer16SplitVecRes_LOADEPNS_10LoadSDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_LOAD(llvm::LoadSDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17SplitVecRes_SETCCEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_SETCC(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer26SplitVecRes_VECTOR_SHUFFLEEPNS_19ShuffleVectorSDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_VECTOR_SHUFFLE(llvm::ShuffleVectorSDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer19SplitVecRes_UnaryOpEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_UnaryOp(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer17SplitVecRes_BinOpEPNS_6SDNodeERNS_7SDValueES4_", "llvm::DAGTypeLegalizer::SplitVecRes_BinOp(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZN4llvm16DAGTypeLegalizer18SplitVectorOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::SplitVectorOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer18SplitVecOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SplitVecOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer28SplitVecOp_EXTRACT_SUBVECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SplitVecOp_EXTRACT_SUBVECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29SplitVecOp_EXTRACT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SplitVecOp_EXTRACT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25SplitVecOp_CONCAT_VECTORSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SplitVecOp_CONCAT_VECTORS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer16SplitVecOp_STOREEPNS_11StoreSDNodeEj", "llvm::DAGTypeLegalizer::SplitVecOp_STORE(llvm::StoreSDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer18SplitVecOp_UnaryOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::SplitVecOp_UnaryOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17WidenVectorResultEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::WidenVectorResult(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer19WidenVecRes_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer24WidenVecRes_BUILD_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_BUILD_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26WidenVecRes_CONCAT_VECTORSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_CONCAT_VECTORS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26WidenVecRes_CONVERT_RNDSATEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_CONVERT_RNDSAT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29WidenVecRes_EXTRACT_SUBVECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_EXTRACT_SUBVECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19WidenVecRes_InregOpEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_InregOp(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29WidenVecRes_INSERT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_INSERT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer16WidenVecRes_LOADEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_LOAD(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer28WidenVecRes_SCALAR_TO_VECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_SCALAR_TO_VECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVecRes_SELECTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_SELECT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer21WidenVecRes_SELECT_CCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_SELECT_CC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17WidenVecRes_SETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_SETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17WidenVecRes_UNDEFEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_UNDEF(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer26WidenVecRes_VECTOR_SHUFFLEEPNS_19ShuffleVectorSDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_VECTOR_SHUFFLE(llvm::ShuffleVectorSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVecRes_VSETCCEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_VSETCC(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVecRes_BinaryEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_Binary(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer16WidenVecRes_POWIEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_POWI(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17WidenVecRes_ShiftEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_Shift(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer19WidenVecRes_ConvertEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_Convert(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer17WidenVecRes_UnaryEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecRes_Unary(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer12ModifyToTypeENS_7SDValueENS_3EVTE", "llvm::DAGTypeLegalizer::ModifyToType(llvm::SDValue, llvm::EVT)"},
    {"_ZN4llvm16DAGTypeLegalizer22GenWidenVectorExtLoadsERNS_11SmallVectorINS_7SDValueELj16EEEPNS_10LoadSDNodeENS_3ISD11LoadExtTypeE", "llvm::DAGTypeLegalizer::GenWidenVectorExtLoads(llvm::SmallVector<llvm::SDValue, 16u>&, llvm::LoadSDNode*, llvm::ISD::LoadExtType)"},
    {"_ZN4llvm16DAGTypeLegalizer19GenWidenVectorLoadsERNS_11SmallVectorINS_7SDValueELj16EEEPNS_10LoadSDNodeE", "llvm::DAGTypeLegalizer::GenWidenVectorLoads(llvm::SmallVector<llvm::SDValue, 16u>&, llvm::LoadSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVectorOperandEPNS_6SDNodeEj", "llvm::DAGTypeLegalizer::WidenVectorOperand(llvm::SDNode*, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVecOp_BITCASTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_BITCAST(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25WidenVecOp_CONCAT_VECTORSEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_CONCAT_VECTORS(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer28WidenVecOp_EXTRACT_SUBVECTOREPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_EXTRACT_SUBVECTOR(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer29WidenVecOp_EXTRACT_VECTOR_ELTEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_EXTRACT_VECTOR_ELT(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer16WidenVecOp_STOREEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_STORE(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer18WidenVecOp_ConvertEPNS_6SDNodeE", "llvm::DAGTypeLegalizer::WidenVecOp_Convert(llvm::SDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer25GenWidenVectorTruncStoresERNS_11SmallVectorINS_7SDValueELj16EEEPNS_11StoreSDNodeE", "llvm::DAGTypeLegalizer::GenWidenVectorTruncStores(llvm::SmallVector<llvm::SDValue, 16u>&, llvm::StoreSDNode*)"},
    {"_ZN4llvm16DAGTypeLegalizer20GenWidenVectorStoresERNS_11SmallVectorINS_7SDValueELj16EEEPNS_11StoreSDNodeE", "llvm::DAGTypeLegalizer::GenWidenVectorStores(llvm::SmallVector<llvm::SDValue, 16u>&, llvm::StoreSDNode*)"},
    {"_Z11FindMemTypeRN4llvm12SelectionDAGERKNS_14TargetLoweringEjNS_3EVTEjj", "FindMemType(llvm::SelectionDAG&, llvm::TargetLowering const&, unsigned int, llvm::EVT, unsigned int, unsigned int)"},
    {"_Z21BuildVectorFromScalarRN4llvm12SelectionDAGENS_3EVTERNS_11SmallVectorINS_7SDValueELj16EEEjj", "BuildVectorFromScalar(llvm::SelectionDAG&, llvm::EVT, llvm::SmallVector<llvm::SDValue, 16u>&, unsigned int, unsigned int)"},
    {"_ZN4llvm16DAGTypeLegalizer13GetExpandedOpENS_7SDValueERS1_S2_", "llvm::DAGTypeLegalizer::GetExpandedOp(llvm::SDValue, llvm::SDValue&, llvm::SDValue&)"},
    {"_ZNK4llvm3EVT6bitsEqES0_", "llvm::EVT::bitsEq(llvm::EVT) const"},
    {"_ZN4llvm24createTDListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createTDListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_115ScheduleDAGListD1Ev", "(anonymous namespace)::ScheduleDAGList::~ScheduleDAGList()"},
    {"_ZN12_GLOBAL__N_115ScheduleDAGListD0Ev", "(anonymous namespace)::ScheduleDAGList::~ScheduleDAGList()"},
    {"_ZN12_GLOBAL__N_115ScheduleDAGList8ScheduleEv", "(anonymous namespace)::ScheduleDAGList::Schedule()"},
    {"_ZN12_GLOBAL__N_115ScheduleDAGList17ReleaseSuccessorsEPN4llvm5SUnitE", "(anonymous namespace)::ScheduleDAGList::ReleaseSuccessors(llvm::SUnit*)"},
    {"_ZN4llvm26createBURRListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createBURRListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm26createTDRRListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createTDRRListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm28createSourceListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createSourceListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm28createHybridListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createHybridListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm25createILPListDAGSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createILPListDAGScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRListC1ERN4llvm15MachineFunctionEbPNS1_23SchedulingPriorityQueueENS1_10CodeGenOpt5LevelE", "(anonymous namespace)::ScheduleDAGRRList::ScheduleDAGRRList(llvm::MachineFunction&, bool, llvm::SchedulingPriorityQueue*, llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBaseC2ERN4llvm15MachineFunctionEbbPKNS1_15TargetInstrInfoEPKNS1_18TargetRegisterInfoEPKNS1_14TargetLoweringE", "(anonymous namespace)::RegReductionPQBase::RegReductionPQBase(llvm::MachineFunction&, bool, bool, llvm::TargetInstrInfo const*, llvm::TargetRegisterInfo const*, llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEED1Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEED0Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEE10isBottomUpEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::isBottomUp() const"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase9initNodesERSt6vectorIN4llvm5SUnitESaIS3_EE", "(anonymous namespace)::RegReductionPQBase::initNodes(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >&)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase7addNodeEPKN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::addNode(llvm::SUnit const*)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase10updateNodeEPKN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::updateNode(llvm::SUnit const*)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase12releaseStateEv", "(anonymous namespace)::RegReductionPQBase::releaseState()"},
    {"_ZNK12_GLOBAL__N_118RegReductionPQBase5emptyEv", "(anonymous namespace)::RegReductionPQBase::empty() const"},
    {"_ZNK12_GLOBAL__N_118RegReductionPQBase17tracksRegPressureEv", "(anonymous namespace)::RegReductionPQBase::tracksRegPressure() const"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEE7isReadyEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::isReady(llvm::SUnit*) const"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase4pushEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::push(llvm::SUnit*)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEE3popEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::pop()"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase6removeEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::remove(llvm::SUnit*)"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEE4dumpEPN4llvm11ScheduleDAGE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::dump(llvm::ScheduleDAG*) const"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase13ScheduledNodeEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::ScheduledNode(llvm::SUnit*)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBase15UnscheduledNodeEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::UnscheduledNode(llvm::SUnit*)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEE12popFromQueueERSt6vectorIPN4llvm5SUnitESaIS6_EERS1_", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>::popFromQueue(std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >&, (anonymous namespace)::ilp_ls_rr_sort&)"},
    {"_Z8BURRSortPN4llvm5SUnitES1_PN12_GLOBAL__N_118RegReductionPQBaseE", "BURRSort(llvm::SUnit*, llvm::SUnit*, (anonymous namespace)::RegReductionPQBase*)"},
    {"_ZNK12_GLOBAL__N_118RegReductionPQBase15HighRegPressureEPKN4llvm5SUnitE", "(anonymous namespace)::RegReductionPQBase::HighRegPressure(llvm::SUnit const*) const"},
    {"_Z11closestSuccPKN4llvm5SUnitE", "closestSucc(llvm::SUnit const*)"},
    {"_Z16BUCompareLatencyPN4llvm5SUnitES1_bPN12_GLOBAL__N_118RegReductionPQBaseE", "BUCompareLatency(llvm::SUnit*, llvm::SUnit*, bool, (anonymous namespace)::RegReductionPQBase*)"},
    {"_Z25CalcNodeSethiUllmanNumberPKN4llvm5SUnitERSt6vectorIjSaIjEE", "CalcNodeSethiUllmanNumber(llvm::SUnit const*, std::vector<unsigned int, std::allocator<unsigned int> >&)"},
    {"_Z21canClobberPhysRegDefsPKN4llvm5SUnitES2_PKNS_15TargetInstrInfoEPKNS_18TargetRegisterInfoE", "canClobberPhysRegDefs(llvm::SUnit const*, llvm::SUnit const*, llvm::TargetInstrInfo const*, llvm::TargetRegisterInfo const*)"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBaseD1Ev", "(anonymous namespace)::RegReductionPQBase::~RegReductionPQBase()"},
    {"_ZN12_GLOBAL__N_118RegReductionPQBaseD0Ev", "(anonymous namespace)::RegReductionPQBase::~RegReductionPQBase()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEED1Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEED0Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEE10isBottomUpEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::isBottomUp() const"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEE7isReadyEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::isReady(llvm::SUnit*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEE3popEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::pop()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEE4dumpEPN4llvm11ScheduleDAGE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::dump(llvm::ScheduleDAG*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEE12popFromQueueERSt6vectorIPN4llvm5SUnitESaIS6_EERS1_", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>::popFromQueue(std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >&, (anonymous namespace)::hybrid_ls_rr_sort&)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEED1Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEED0Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEE10isBottomUpEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::isBottomUp() const"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEE7isReadyEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::isReady(llvm::SUnit*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEE3popEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::pop()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEE4dumpEPN4llvm11ScheduleDAGE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::dump(llvm::ScheduleDAG*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEE12popFromQueueERSt6vectorIPN4llvm5SUnitESaIS6_EERS1_", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>::popFromQueue(std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >&, (anonymous namespace)::src_ls_rr_sort&)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEED1Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEED0Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEE10isBottomUpEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::isBottomUp() const"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEE7isReadyEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::isReady(llvm::SUnit*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEE3popEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::pop()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEE4dumpEPN4llvm11ScheduleDAGE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::dump(llvm::ScheduleDAG*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEE12popFromQueueERSt6vectorIPN4llvm5SUnitESaIS6_EERS1_", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>::popFromQueue(std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >&, (anonymous namespace)::td_ls_rr_sort&)"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEED1Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEED0Ev", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::~RegReductionPriorityQueue()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEE10isBottomUpEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::isBottomUp() const"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEE7isReadyEPN4llvm5SUnitE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::isReady(llvm::SUnit*) const"},
    {"_ZN12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEE3popEv", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::pop()"},
    {"_ZNK12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEE4dumpEPN4llvm11ScheduleDAGE", "(anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>::dump(llvm::ScheduleDAG*) const"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRListD1Ev", "(anonymous namespace)::ScheduleDAGRRList::~ScheduleDAGRRList()"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRListD0Ev", "(anonymous namespace)::ScheduleDAGRRList::~ScheduleDAGRRList()"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList8ScheduleEv", "(anonymous namespace)::ScheduleDAGRRList::Schedule()"},
    {"_ZNK12_GLOBAL__N_117ScheduleDAGRRList18ForceUnitLatenciesEv", "(anonymous namespace)::ScheduleDAGRRList::ForceUnitLatencies() const"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList20ListScheduleBottomUpEv", "(anonymous namespace)::ScheduleDAGRRList::ListScheduleBottomUp()"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList19ReleasePredecessorsEPN4llvm5SUnitE", "(anonymous namespace)::ScheduleDAGRRList::ReleasePredecessors(llvm::SUnit*)"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList14AdvanceToCycleEj", "(anonymous namespace)::ScheduleDAGRRList::AdvanceToCycle(unsigned int)"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList14ReleasePendingEv", "(anonymous namespace)::ScheduleDAGRRList::ReleasePending()"},
    {"_ZN12_GLOBAL__N_117ScheduleDAGRRList8EmitNodeEPN4llvm5SUnitE", "(anonymous namespace)::ScheduleDAGRRList::EmitNode(llvm::SUnit*)"},
    {"_Z18CheckForLiveRegDefPN4llvm5SUnitEjRSt6vectorIS1_SaIS1_EERNS_8SmallSetIjLj4EEERNS_11SmallVectorIjLj4EEEPKNS_18TargetRegisterInfoE", "CheckForLiveRegDef(llvm::SUnit*, unsigned int, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >&, llvm::SmallSet<unsigned int, 4u>&, llvm::SmallVector<unsigned int, 4u>&, llvm::TargetRegisterInfo const*)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm5SUnitESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, llvm::SUnit*>(__gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, __gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, llvm::SUnit* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIjSaIjEE14_M_fill_assignEmRKj", "std::vector<unsigned int, std::allocator<unsigned int> >::_M_fill_assign(unsigned long, unsigned int const&)"},
    {"_ZNK4llvm18ScheduleDAGSDNodes21ComputeOperandLatencyEPNS_5SUnitES2_RNS_4SDepE", "llvm::ScheduleDAGSDNodes::ComputeOperandLatency(llvm::SUnit*, llvm::SUnit*, llvm::SDep&) const"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitENS_11SmallVectorIjLj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u>, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned int, 4u> > >::insert(std::pair<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitENS_11SmallVectorIjLj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u>, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned int, 4u> > >::FindAndConstruct(llvm::SUnit* const&)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitENS_11SmallVectorIjLj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u>, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned int, 4u> > >::InsertIntoBucket(llvm::SUnit* const&, llvm::SmallVector<unsigned int, 4u> const&, std::pair<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitENS_11SmallVectorIjLj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::SUnit*, llvm::SmallVector<unsigned int, 4u>, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<llvm::SmallVector<unsigned int, 4u> > >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm5SUnitESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, unsigned long, llvm::SUnit* const&)"},
    {"_ZN4llvm18ScheduleDAGSDNodesD1Ev", "llvm::ScheduleDAGSDNodes::~ScheduleDAGSDNodes()"},
    {"_ZN4llvm18ScheduleDAGSDNodesD0Ev", "llvm::ScheduleDAGSDNodes::~ScheduleDAGSDNodes()"},
    {"_ZN4llvm18ScheduleDAGSDNodesC1ERNS_15MachineFunctionE", "llvm::ScheduleDAGSDNodes::ScheduleDAGSDNodes(llvm::MachineFunction&)"},
    {"_ZN4llvm18ScheduleDAGSDNodesC2ERNS_15MachineFunctionE", "llvm::ScheduleDAGSDNodes::ScheduleDAGSDNodes(llvm::MachineFunction&)"},
    {"_ZN4llvm18ScheduleDAGSDNodes3RunEPNS_12SelectionDAGEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::ScheduleDAGSDNodes::Run(llvm::SelectionDAG*, llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm18ScheduleDAGSDNodes8NewSUnitEPNS_6SDNodeE", "llvm::ScheduleDAGSDNodes::NewSUnit(llvm::SDNode*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes5CloneEPNS_5SUnitE", "llvm::ScheduleDAGSDNodes::Clone(llvm::SUnit*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes23ClusterNeighboringLoadsEPNS_6SDNodeE", "llvm::ScheduleDAGSDNodes::ClusterNeighboringLoads(llvm::SDNode*)"},
    {"_Z7AddGluePN4llvm6SDNodeENS_7SDValueEbPNS_12SelectionDAGE", "AddGlue(llvm::SDNode*, llvm::SDValue, bool, llvm::SelectionDAG*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes12ClusterNodesEv", "llvm::ScheduleDAGSDNodes::ClusterNodes()"},
    {"_ZN4llvm18ScheduleDAGSDNodes15BuildSchedUnitsEv", "llvm::ScheduleDAGSDNodes::BuildSchedUnits()"},
    {"_ZN4llvm18ScheduleDAGSDNodes18InitNumRegDefsLeftEPNS_5SUnitE", "llvm::ScheduleDAGSDNodes::InitNumRegDefsLeft(llvm::SUnit*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes13AddSchedEdgesEv", "llvm::ScheduleDAGSDNodes::AddSchedEdges()"},
    {"_ZN4llvm18ScheduleDAGSDNodes15BuildSchedGraphEPNS_13AliasAnalysisE", "llvm::ScheduleDAGSDNodes::BuildSchedGraph(llvm::AliasAnalysis*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes10RegDefIter15InitNodeNumDefsEv", "llvm::ScheduleDAGSDNodes::RegDefIter::InitNodeNumDefs()"},
    {"_ZN4llvm18ScheduleDAGSDNodes10RegDefIterC1EPKNS_5SUnitEPKS0_", "llvm::ScheduleDAGSDNodes::RegDefIter::RegDefIter(llvm::SUnit const*, llvm::ScheduleDAGSDNodes const*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes10RegDefIterC2EPKNS_5SUnitEPKS0_", "llvm::ScheduleDAGSDNodes::RegDefIter::RegDefIter(llvm::SUnit const*, llvm::ScheduleDAGSDNodes const*)"},
    {"_ZN4llvm18ScheduleDAGSDNodes10RegDefIter7AdvanceEv", "llvm::ScheduleDAGSDNodes::RegDefIter::Advance()"},
    {"_ZN4llvm18ScheduleDAGSDNodes14ComputeLatencyEPNS_5SUnitE", "llvm::ScheduleDAGSDNodes::ComputeLatency(llvm::SUnit*)"},
    {"_ZNK4llvm18ScheduleDAGSDNodes21ComputeOperandLatencyEPNS_6SDNodeES2_jRNS_4SDepE", "llvm::ScheduleDAGSDNodes::ComputeOperandLatency(llvm::SDNode*, llvm::SDNode*, unsigned int, llvm::SDep&) const"},
    {"_ZNK4llvm18ScheduleDAGSDNodes8dumpNodeEPKNS_5SUnitE", "llvm::ScheduleDAGSDNodes::dumpNode(llvm::SUnit const*) const"},
    {"_ZN4llvm18ScheduleDAGSDNodes12EmitScheduleEv", "llvm::ScheduleDAGSDNodes::EmitSchedule()"},
    {"_Z17ProcessSourceNodePN4llvm6SDNodeEPNS_12SelectionDAGERNS_12InstrEmitterERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoIS7_EENS8_IjEEEERNS_11SmallVectorISt4pairIjPNS_12MachineInstrEELj32EEERNS_8SmallSetIjLj8EEE", "ProcessSourceNode(llvm::SDNode*, llvm::SelectionDAG*, llvm::InstrEmitter&, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, llvm::SmallVector<std::pair<unsigned int, llvm::MachineInstr*>, 32u>&, llvm::SmallSet<unsigned int, 8u>&)"},
    {"_ZSt16__introsort_loopIPSt4pairIjPN4llvm12MachineInstrEElN12_GLOBAL__N_111OrderSorterEEvT_S8_T0_T1_", "void std::__introsort_loop<std::pair<unsigned int, llvm::MachineInstr*>*, long, (anonymous namespace)::OrderSorter>(std::pair<unsigned int, llvm::MachineInstr*>*, std::pair<unsigned int, llvm::MachineInstr*>*, long, (anonymous namespace)::OrderSorter)"},
    {"_ZSt16__insertion_sortIPSt4pairIjPN4llvm12MachineInstrEEN12_GLOBAL__N_111OrderSorterEEvT_S8_T0_", "void std::__insertion_sort<std::pair<unsigned int, llvm::MachineInstr*>*, (anonymous namespace)::OrderSorter>(std::pair<unsigned int, llvm::MachineInstr*>*, std::pair<unsigned int, llvm::MachineInstr*>*, (anonymous namespace)::OrderSorter)"},
    {"_Z18ProcessSDDbgValuesPN4llvm6SDNodeEPNS_12SelectionDAGERNS_12InstrEmitterERNS_11SmallVectorISt4pairIjPNS_12MachineInstrEELj32EEERNS_8DenseMapINS_7SDValueEjNS_12DenseMapInfoISE_EENSF_IjEEEEj", "ProcessSDDbgValues(llvm::SDNode*, llvm::SelectionDAG*, llvm::InstrEmitter&, llvm::SmallVector<std::pair<unsigned int, llvm::MachineInstr*>, 32u>&, llvm::DenseMap<llvm::SDValue, unsigned int, llvm::DenseMapInfo<llvm::SDValue>, llvm::DenseMapInfo<unsigned int> >&, unsigned int)"},
    {"_ZN4llvm8DenseMapIxPNS_6SDNodeENS_12DenseMapInfoIxEENS3_IS2_EEE6insertERKSt4pairIxS2_E", "llvm::DenseMap<long long, llvm::SDNode*, llvm::DenseMapInfo<long long>, llvm::DenseMapInfo<llvm::SDNode*> >::insert(std::pair<long long, llvm::SDNode*> const&)"},
    {"_ZN4llvm8DenseMapIxPNS_6SDNodeENS_12DenseMapInfoIxEENS3_IS2_EEE16InsertIntoBucketERKxRKS2_PSt4pairIxS2_E", "llvm::DenseMap<long long, llvm::SDNode*, llvm::DenseMapInfo<long long>, llvm::DenseMapInfo<llvm::SDNode*> >::InsertIntoBucket(long long const&, llvm::SDNode* const&, std::pair<long long, llvm::SDNode*>*)"},
    {"_ZN4llvm8DenseMapIxPNS_6SDNodeENS_12DenseMapInfoIxEENS3_IS2_EEE4growEj", "llvm::DenseMap<long long, llvm::SDNode*, llvm::DenseMapInfo<long long>, llvm::DenseMapInfo<llvm::SDNode*> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIPxlEvT_S1_T0_", "void std::__introsort_loop<long long*, long>(long long*, long long*, long)"},
    {"_ZSt22__final_insertion_sortIPxEvT_S1_", "void std::__final_insertion_sort<long long*>(long long*, long long*)"},
    {"_ZSt13__heap_selectIPxEvT_S1_S1_", "void std::__heap_select<long long*>(long long*, long long*, long long*)"},
    {"_ZSt9sort_heapIPxEvT_S1_", "void std::sort_heap<long long*>(long long*, long long*)"},
    {"_ZSt9make_heapIPxEvT_S1_", "void std::make_heap<long long*>(long long*, long long*)"},
    {"_ZN4llvm12SelectionDAG17DAGUpdateListenerD0Ev", "llvm::SelectionDAG::DAGUpdateListener::~DAGUpdateListener()"},
    {"_ZN4llvm12SelectionDAG17DAGUpdateListenerD1Ev", "llvm::SelectionDAG::DAGUpdateListener::~DAGUpdateListener()"},
    {"_ZN4llvm12SelectionDAG17DAGUpdateListenerD2Ev", "llvm::SelectionDAG::DAGUpdateListener::~DAGUpdateListener()"},
    {"_ZNK4llvm16ConstantFPSDNode14isExactlyValueERKNS_7APFloatE", "llvm::ConstantFPSDNode::isExactlyValue(llvm::APFloat const&) const"},
    {"_ZN4llvm16ConstantFPSDNode19isValueValidForTypeENS_3EVTERKNS_7APFloatE", "llvm::ConstantFPSDNode::isValueValidForType(llvm::EVT, llvm::APFloat const&)"},
    {"_ZN4llvm3ISD20isBuildVectorAllOnesEPKNS_6SDNodeE", "llvm::ISD::isBuildVectorAllOnes(llvm::SDNode const*)"},
    {"_ZN4llvm3ISD21isBuildVectorAllZerosEPKNS_6SDNodeE", "llvm::ISD::isBuildVectorAllZeros(llvm::SDNode const*)"},
    {"_ZN4llvm3ISD16isScalarToVectorEPKNS_6SDNodeE", "llvm::ISD::isScalarToVector(llvm::SDNode const*)"},
    {"_ZN4llvm3ISD23getSetCCSwappedOperandsENS0_8CondCodeE", "llvm::ISD::getSetCCSwappedOperands(llvm::ISD::CondCode)"},
    {"_ZN4llvm3ISD15getSetCCInverseENS0_8CondCodeEb", "llvm::ISD::getSetCCInverse(llvm::ISD::CondCode, bool)"},
    {"_ZN4llvm3ISD19getSetCCOrOperationENS0_8CondCodeES1_b", "llvm::ISD::getSetCCOrOperation(llvm::ISD::CondCode, llvm::ISD::CondCode, bool)"},
    {"_ZN4llvm3ISD20getSetCCAndOperationENS0_8CondCodeES1_b", "llvm::ISD::getSetCCAndOperation(llvm::ISD::CondCode, llvm::ISD::CondCode, bool)"},
    {"_ZN4llvm12SelectionDAG15RemoveDeadNodesEv", "llvm::SelectionDAG::RemoveDeadNodes()"},
    {"_ZN4llvm12SelectionDAG15RemoveDeadNodesERNS_15SmallVectorImplIPNS_6SDNodeEEEPNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::RemoveDeadNodes(llvm::SmallVectorImpl<llvm::SDNode*>&, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12HandleSDNodeD1Ev", "llvm::HandleSDNode::~HandleSDNode()"},
    {"_ZN4llvm12SelectionDAG21RemoveNodeFromCSEMapsEPNS_6SDNodeE", "llvm::SelectionDAG::RemoveNodeFromCSEMaps(llvm::SDNode*)"},
    {"_ZN4llvm12SelectionDAG14DeallocateNodeEPNS_6SDNodeE", "llvm::SelectionDAG::DeallocateNode(llvm::SDNode*)"},
    {"_ZN4llvm12SelectionDAG14RemoveDeadNodeEPNS_6SDNodeEPNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::RemoveDeadNode(llvm::SDNode*, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG10DeleteNodeEPNS_6SDNodeE", "llvm::SelectionDAG::DeleteNode(llvm::SDNode*)"},
    {"_ZN4llvm12SelectionDAG22DeleteNodeNotInCSEMapsEPNS_6SDNodeE", "llvm::SelectionDAG::DeleteNodeNotInCSEMaps(llvm::SDNode*)"},
    {"_ZN4llvm6SDNode12DropOperandsEv", "llvm::SDNode::DropOperands()"},
    {"_ZN4llvm12SelectionDAG24AddModifiedNodeToCSEMapsEPNS_6SDNodeEPNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::AddModifiedNodeToCSEMaps(llvm::SDNode*, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG18ReplaceAllUsesWithEPNS_6SDNodeES2_PNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::ReplaceAllUsesWith(llvm::SDNode*, llvm::SDNode*, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG20FindModifiedNodeSlotEPNS_6SDNodeENS_7SDValueERPv", "llvm::SelectionDAG::FindModifiedNodeSlot(llvm::SDNode*, llvm::SDValue, void*&)"},
    {"_Z15AddNodeIDCustomRN4llvm16FoldingSetNodeIDEPKNS_6SDNodeE", "AddNodeIDCustom(llvm::FoldingSetNodeID&, llvm::SDNode const*)"},
    {"_ZN4llvm12SelectionDAG20FindModifiedNodeSlotEPNS_6SDNodeENS_7SDValueES3_RPv", "llvm::SelectionDAG::FindModifiedNodeSlot(llvm::SDNode*, llvm::SDValue, llvm::SDValue, void*&)"},
    {"_ZN4llvm12SelectionDAG20FindModifiedNodeSlotEPNS_6SDNodeEPKNS_7SDValueEjRPv", "llvm::SelectionDAG::FindModifiedNodeSlot(llvm::SDNode*, llvm::SDValue const*, unsigned int, void*&)"},
    {"_ZNK4llvm12SelectionDAG15getEVTAlignmentENS_3EVTE", "llvm::SelectionDAG::getEVTAlignment(llvm::EVT) const"},
    {"_ZN4llvm12SelectionDAGC1ERKNS_13TargetMachineE", "llvm::SelectionDAG::SelectionDAG(llvm::TargetMachine const&)"},
    {"_ZN4llvm12SelectionDAGC2ERKNS_13TargetMachineE", "llvm::SelectionDAG::SelectionDAG(llvm::TargetMachine const&)"},
    {"_ZN4llvm12SelectionDAG9getVTListENS_3EVTE", "llvm::SelectionDAG::getVTList(llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG4initERNS_15MachineFunctionE", "llvm::SelectionDAG::init(llvm::MachineFunction&)"},
    {"_ZN4llvm12SelectionDAGD1Ev", "llvm::SelectionDAG::~SelectionDAG()"},
    {"_ZN4llvm12SelectionDAGD2Ev", "llvm::SelectionDAG::~SelectionDAG()"},
    {"_ZN4llvm12SelectionDAG14allnodes_clearEv", "llvm::SelectionDAG::allnodes_clear()"},
    {"_ZN4llvm12SelectionDAG5clearEv", "llvm::SelectionDAG::clear()"},
    {"_ZN4llvm12SelectionDAG14getSExtOrTruncENS_7SDValueENS_8DebugLocENS_3EVTE", "llvm::SelectionDAG::getSExtOrTrunc(llvm::SDValue, llvm::DebugLoc, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTENS_7SDValueE", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getZExtOrTruncENS_7SDValueENS_8DebugLocENS_3EVTE", "llvm::SelectionDAG::getZExtOrTrunc(llvm::SDValue, llvm::DebugLoc, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG18getZeroExtendInRegENS_7SDValueENS_8DebugLocENS_3EVTE", "llvm::SelectionDAG::getZeroExtendInReg(llvm::SDValue, llvm::DebugLoc, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG11getConstantERKNS_5APIntENS_3EVTEb", "llvm::SelectionDAG::getConstant(llvm::APInt const&, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG6getNOTENS_8DebugLocENS_7SDValueENS_3EVTE", "llvm::SelectionDAG::getNOT(llvm::DebugLoc, llvm::SDValue, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG11getConstantEyNS_3EVTEb", "llvm::SelectionDAG::getConstant(unsigned long long, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG11getConstantERKNS_11ConstantIntENS_3EVTEb", "llvm::SelectionDAG::getConstant(llvm::ConstantInt const&, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTEPKNS_7SDValueEj", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG17getIntPtrConstantEyb", "llvm::SelectionDAG::getIntPtrConstant(unsigned long long, bool)"},
    {"_ZN4llvm12SelectionDAG13getConstantFPERKNS_7APFloatENS_3EVTEb", "llvm::SelectionDAG::getConstantFP(llvm::APFloat const&, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG13getConstantFPERKNS_10ConstantFPENS_3EVTEb", "llvm::SelectionDAG::getConstantFP(llvm::ConstantFP const&, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG13getConstantFPEdNS_3EVTEb", "llvm::SelectionDAG::getConstantFP(double, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG16getGlobalAddressEPKNS_11GlobalValueENS_8DebugLocENS_3EVTExbh", "llvm::SelectionDAG::getGlobalAddress(llvm::GlobalValue const*, llvm::DebugLoc, llvm::EVT, long long, bool, unsigned char)"},
    {"_ZN4llvm19GlobalAddressSDNodeC1EjNS_8DebugLocEPKNS_11GlobalValueENS_3EVTExh", "llvm::GlobalAddressSDNode::GlobalAddressSDNode(unsigned int, llvm::DebugLoc, llvm::GlobalValue const*, llvm::EVT, long long, unsigned char)"},
    {"_ZN4llvm12SelectionDAG13getFrameIndexEiNS_3EVTEb", "llvm::SelectionDAG::getFrameIndex(int, llvm::EVT, bool)"},
    {"_ZN4llvm12SelectionDAG12getJumpTableEiNS_3EVTEbh", "llvm::SelectionDAG::getJumpTable(int, llvm::EVT, bool, unsigned char)"},
    {"_ZN4llvm12SelectionDAG15getConstantPoolEPKNS_8ConstantENS_3EVTEjibh", "llvm::SelectionDAG::getConstantPool(llvm::Constant const*, llvm::EVT, unsigned int, int, bool, unsigned char)"},
    {"_ZN4llvm12SelectionDAG15getConstantPoolEPNS_24MachineConstantPoolValueENS_3EVTEjibh", "llvm::SelectionDAG::getConstantPool(llvm::MachineConstantPoolValue*, llvm::EVT, unsigned int, int, bool, unsigned char)"},
    {"_ZN4llvm12SelectionDAG13getBasicBlockEPNS_17MachineBasicBlockE", "llvm::SelectionDAG::getBasicBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12SelectionDAG12getValueTypeENS_3EVTE", "llvm::SelectionDAG::getValueType(llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG17getExternalSymbolEPKcNS_3EVTE", "llvm::SelectionDAG::getExternalSymbol(char const*, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG23getTargetExternalSymbolEPKcNS_3EVTEh", "llvm::SelectionDAG::getTargetExternalSymbol(char const*, llvm::EVT, unsigned char)"},
    {"_ZN4llvm12SelectionDAG11getCondCodeENS_3ISD8CondCodeE", "llvm::SelectionDAG::getCondCode(llvm::ISD::CondCode)"},
    {"_ZN4llvm12SelectionDAG16getVectorShuffleENS_3EVTENS_8DebugLocENS_7SDValueES3_PKi", "llvm::SelectionDAG::getVectorShuffle(llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, int const*)"},
    {"_ZN4llvm12SelectionDAG16getConvertRndSatENS_3EVTENS_8DebugLocENS_7SDValueES3_S3_S3_S3_NS_3ISD7CvtCodeE", "llvm::SelectionDAG::getConvertRndSat(llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::ISD::CvtCode)"},
    {"_ZN4llvm12SelectionDAG11getRegisterEjNS_3EVTE", "llvm::SelectionDAG::getRegister(unsigned int, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG10getEHLabelENS_8DebugLocENS_7SDValueEPNS_8MCSymbolE", "llvm::SelectionDAG::getEHLabel(llvm::DebugLoc, llvm::SDValue, llvm::MCSymbol*)"},
    {"_ZN4llvm12SelectionDAG15getBlockAddressEPKNS_12BlockAddressENS_3EVTEbh", "llvm::SelectionDAG::getBlockAddress(llvm::BlockAddress const*, llvm::EVT, bool, unsigned char)"},
    {"_ZN4llvm12SelectionDAG11getSrcValueEPKNS_5ValueE", "llvm::SelectionDAG::getSrcValue(llvm::Value const*)"},
    {"_ZN4llvm12SelectionDAG9getMDNodeEPKNS_6MDNodeE", "llvm::SelectionDAG::getMDNode(llvm::MDNode const*)"},
    {"_ZN4llvm12SelectionDAG21getShiftAmountOperandENS_7SDValueE", "llvm::SelectionDAG::getShiftAmountOperand(llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG20CreateStackTemporaryENS_3EVTEj", "llvm::SelectionDAG::CreateStackTemporary(llvm::EVT, unsigned int)"},
    {"_ZN4llvm12SelectionDAG20CreateStackTemporaryENS_3EVTES1_", "llvm::SelectionDAG::CreateStackTemporary(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG9FoldSetCCENS_3EVTENS_7SDValueES2_NS_3ISD8CondCodeENS_8DebugLocE", "llvm::SelectionDAG::FoldSetCC(llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::ISD::CondCode, llvm::DebugLoc)"},
    {"_ZNK4llvm12SelectionDAG13SignBitIsZeroENS_7SDValueEj", "llvm::SelectionDAG::SignBitIsZero(llvm::SDValue, unsigned int) const"},
    {"_ZNK4llvm12SelectionDAG17MaskedValueIsZeroENS_7SDValueERKNS_5APIntEj", "llvm::SelectionDAG::MaskedValueIsZero(llvm::SDValue, llvm::APInt const&, unsigned int) const"},
    {"_ZNK4llvm12SelectionDAG17ComputeMaskedBitsENS_7SDValueERKNS_5APIntERS2_S5_j", "llvm::SelectionDAG::ComputeMaskedBits(llvm::SDValue, llvm::APInt const&, llvm::APInt&, llvm::APInt&, unsigned int) const"},
    {"_ZNK4llvm12SelectionDAG18ComputeNumSignBitsENS_7SDValueEj", "llvm::SelectionDAG::ComputeNumSignBits(llvm::SDValue, unsigned int) const"},
    {"_ZNK4llvm12SelectionDAG15isKnownNeverNaNENS_7SDValueE", "llvm::SelectionDAG::isKnownNeverNaN(llvm::SDValue) const"},
    {"_ZNK4llvm12SelectionDAG16isKnownNeverZeroENS_7SDValueE", "llvm::SelectionDAG::isKnownNeverZero(llvm::SDValue) const"},
    {"_ZNK4llvm12SelectionDAG9isEqualToENS_7SDValueES1_", "llvm::SelectionDAG::isEqualTo(llvm::SDValue, llvm::SDValue) const"},
    {"_ZNK4llvm12SelectionDAG23isVerifiedDebugInfoDescENS_7SDValueE", "llvm::SelectionDAG::isVerifiedDebugInfoDesc(llvm::SDValue) const"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTE", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG22FoldConstantArithmeticEjNS_3EVTEPNS_14ConstantSDNodeES3_", "llvm::SelectionDAG::FoldConstantArithmetic(unsigned int, llvm::EVT, llvm::ConstantSDNode*, llvm::ConstantSDNode*)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_S3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG27getStackArgumentTokenFactorENS_7SDValueE", "llvm::SelectionDAG::getStackArgumentTokenFactor(llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG9getMemcpyENS_7SDValueENS_8DebugLocES1_S1_S1_jbbNS_18MachinePointerInfoES3_", "llvm::SelectionDAG::getMemcpy(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo)"},
    {"_Z23getMemcpyLoadsAndStoresRN4llvm12SelectionDAGENS_8DebugLocENS_7SDValueES3_S3_yjbbNS_18MachinePointerInfoES4_", "getMemcpyLoadsAndStores(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned long long, unsigned int, bool, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo)"},
    {"_ZN4llvm12SelectionDAG10getMemmoveENS_7SDValueENS_8DebugLocES1_S1_S1_jbNS_18MachinePointerInfoES3_", "llvm::SelectionDAG::getMemmove(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo)"},
    {"_ZN4llvm12SelectionDAG9getMemsetENS_7SDValueENS_8DebugLocES1_S1_S1_jbNS_18MachinePointerInfoE", "llvm::SelectionDAG::getMemset(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, llvm::MachinePointerInfo)"},
    {"_ZN4llvm12SelectionDAG9getAtomicEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_S3_NS_18MachinePointerInfoEj", "llvm::SelectionDAG::getAtomic(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, unsigned int)"},
    {"_ZN4llvm12SelectionDAG9getAtomicEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_S3_PNS_17MachineMemOperandE", "llvm::SelectionDAG::getAtomic(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG9getVTListENS_3EVTES1_", "llvm::SelectionDAG::getVTList(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG9getAtomicEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_PKNS_5ValueEj", "llvm::SelectionDAG::getAtomic(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::Value const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG9getAtomicEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_PNS_17MachineMemOperandE", "llvm::SelectionDAG::getAtomic(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG14getMergeValuesEPKNS_7SDValueEjNS_8DebugLocE", "llvm::SelectionDAG::getMergeValues(llvm::SDValue const*, unsigned int, llvm::DebugLoc)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEj", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG9getVTListEPKNS_3EVTEj", "llvm::SelectionDAG::getVTList(llvm::EVT const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG19getMemIntrinsicNodeEjNS_8DebugLocEPKNS_3EVTEjPKNS_7SDValueEjS2_NS_18MachinePointerInfoEjbbb", "llvm::SelectionDAG::getMemIntrinsicNode(unsigned int, llvm::DebugLoc, llvm::EVT const*, unsigned int, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::MachinePointerInfo, unsigned int, bool, bool, bool)"},
    {"_ZN4llvm12SelectionDAG19getMemIntrinsicNodeEjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEjNS_3EVTENS_18MachinePointerInfoEjbbb", "llvm::SelectionDAG::getMemIntrinsicNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::MachinePointerInfo, unsigned int, bool, bool, bool)"},
    {"_ZN4llvm12SelectionDAG19getMemIntrinsicNodeEjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEjNS_3EVTEPNS_17MachineMemOperandE", "llvm::SelectionDAG::getMemIntrinsicNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG7getLoadENS_3ISD14MemIndexedModeENS1_11LoadExtTypeENS_3EVTENS_8DebugLocENS_7SDValueES6_S6_NS_18MachinePointerInfoES4_bbjPKNS_6MDNodeE", "llvm::SelectionDAG::getLoad(llvm::ISD::MemIndexedMode, llvm::ISD::LoadExtType, llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, llvm::EVT, bool, bool, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm12SelectionDAG7getLoadENS_3ISD14MemIndexedModeENS1_11LoadExtTypeENS_3EVTENS_8DebugLocENS_7SDValueES6_S6_S4_PNS_17MachineMemOperandE", "llvm::SelectionDAG::getLoad(llvm::ISD::MemIndexedMode, llvm::ISD::LoadExtType, llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG9getVTListENS_3EVTES1_S1_", "llvm::SelectionDAG::getVTList(llvm::EVT, llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG7getLoadENS_3EVTENS_8DebugLocENS_7SDValueES3_NS_18MachinePointerInfoEbbjPKNS_6MDNodeE", "llvm::SelectionDAG::getLoad(llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, bool, bool, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm12SelectionDAG10getExtLoadENS_3ISD11LoadExtTypeENS_3EVTENS_8DebugLocENS_7SDValueES5_NS_18MachinePointerInfoES3_bbjPKNS_6MDNodeE", "llvm::SelectionDAG::getExtLoad(llvm::ISD::LoadExtType, llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, llvm::EVT, bool, bool, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm12SelectionDAG14getIndexedLoadENS_7SDValueENS_8DebugLocES1_S1_NS_3ISD14MemIndexedModeE", "llvm::SelectionDAG::getIndexedLoad(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::ISD::MemIndexedMode)"},
    {"_ZN4llvm12SelectionDAG8getStoreENS_7SDValueENS_8DebugLocES1_S1_NS_18MachinePointerInfoEbbjPKNS_6MDNodeE", "llvm::SelectionDAG::getStore(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, bool, bool, unsigned int, llvm::MDNode const*)"},
    {"_Z16InferPointerInfoN4llvm7SDValueEx", "InferPointerInfo(llvm::SDValue, long long)"},
    {"_ZN4llvm12SelectionDAG8getStoreENS_7SDValueENS_8DebugLocES1_S1_PNS_17MachineMemOperandE", "llvm::SelectionDAG::getStore(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG13getTruncStoreENS_7SDValueENS_8DebugLocES1_S1_NS_18MachinePointerInfoENS_3EVTEbbjPKNS_6MDNodeE", "llvm::SelectionDAG::getTruncStore(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::MachinePointerInfo, llvm::EVT, bool, bool, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm12SelectionDAG13getTruncStoreENS_7SDValueENS_8DebugLocES1_S1_NS_3EVTEPNS_17MachineMemOperandE", "llvm::SelectionDAG::getTruncStore(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12SelectionDAG15getIndexedStoreENS_7SDValueENS_8DebugLocES1_S1_NS_3ISD14MemIndexedModeE", "llvm::SelectionDAG::getIndexedStore(llvm::SDValue, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::ISD::MemIndexedMode)"},
    {"_ZN4llvm12SelectionDAG8getVAArgENS_3EVTENS_8DebugLocENS_7SDValueES3_S3_j", "llvm::SelectionDAG::getVAArg(llvm::EVT, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_3EVTEPKNS_5SDUseEj", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDUse const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocERKSt6vectorINS_3EVTESaIS3_EEPKNS_7SDValueEj", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, std::vector<llvm::EVT, std::allocator<llvm::EVT> > const&, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocEPKNS_3EVTEjPKNS_7SDValueEj", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::EVT const*, unsigned int, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListE", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListENS_7SDValueE", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListENS_7SDValueES3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListENS_7SDValueES3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListENS_7SDValueES3_S3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG7getNodeEjNS_8DebugLocENS_8SDVTListENS_7SDValueES3_S3_S3_S3_", "llvm::SelectionDAG::getNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm6SDNode16getValueTypeListENS_3EVTE", "llvm::SDNode::getValueTypeList(llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG9getVTListENS_3EVTES1_S1_S1_", "llvm::SelectionDAG::getVTList(llvm::EVT, llvm::EVT, llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeENS_7SDValueE", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeENS_7SDValueES3_", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeENS_7SDValueES3_S3_", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeEPKNS_7SDValueEj", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeENS_7SDValueES3_S3_S3_", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG18UpdateNodeOperandsEPNS_6SDNodeENS_7SDValueES3_S3_S3_S3_", "llvm::SelectionDAG::UpdateNodeOperands(llvm::SDNode*, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTE", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_8SDVTListEPKNS_7SDValueEj", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTENS_7SDValueE", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTENS_7SDValueES4_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTENS_7SDValueES4_S4_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTEPKNS_7SDValueEj", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_PKNS_7SDValueEj", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_S3_PKNS_7SDValueEj", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_S3_S3_PKNS_7SDValueEj", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_NS_7SDValueE", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_NS_7SDValueES4_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_NS_7SDValueES4_S4_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG12SelectNodeToEPNS_6SDNodeEjNS_3EVTES3_S3_NS_7SDValueES4_S4_", "llvm::SelectionDAG::SelectNodeTo(llvm::SDNode*, unsigned int, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG11MorphNodeToEPNS_6SDNodeEjNS_8SDVTListEPKNS_7SDValueEj", "llvm::SelectionDAG::MorphNodeTo(llvm::SDNode*, unsigned int, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTE", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTENS_7SDValueE", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTENS_7SDValueES3_S3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTEPKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_NS_7SDValueE", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_NS_7SDValueES3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_NS_7SDValueES3_S3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_PKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_S2_NS_7SDValueES3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_S2_NS_7SDValueES3_S3_", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_S2_PKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocENS_3EVTES2_S2_S2_PKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, llvm::EVT, llvm::EVT, llvm::EVT, llvm::EVT, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG14getMachineNodeEjNS_8DebugLocERKSt6vectorINS_3EVTESaIS3_EEPKNS_7SDValueEj", "llvm::SelectionDAG::getMachineNode(unsigned int, llvm::DebugLoc, std::vector<llvm::EVT, std::allocator<llvm::EVT> > const&, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG22getTargetExtractSubregEiNS_8DebugLocENS_3EVTENS_7SDValueE", "llvm::SelectionDAG::getTargetExtractSubreg(int, llvm::DebugLoc, llvm::EVT, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG21getTargetInsertSubregEiNS_8DebugLocENS_3EVTENS_7SDValueES3_", "llvm::SelectionDAG::getTargetInsertSubreg(int, llvm::DebugLoc, llvm::EVT, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12SelectionDAG15getNodeIfExistsEjNS_8SDVTListEPKNS_7SDValueEj", "llvm::SelectionDAG::getNodeIfExists(unsigned int, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZN4llvm12SelectionDAG11getDbgValueEPNS_6MDNodeEPNS_6SDNodeEjyNS_8DebugLocEj", "llvm::SelectionDAG::getDbgValue(llvm::MDNode*, llvm::SDNode*, unsigned int, unsigned long long, llvm::DebugLoc, unsigned int)"},
    {"_ZN4llvm12SelectionDAG11getDbgValueEPNS_6MDNodeEPKNS_5ValueEyNS_8DebugLocEj", "llvm::SelectionDAG::getDbgValue(llvm::MDNode*, llvm::Value const*, unsigned long long, llvm::DebugLoc, unsigned int)"},
    {"_ZN4llvm12SelectionDAG11getDbgValueEPNS_6MDNodeEjyNS_8DebugLocEj", "llvm::SelectionDAG::getDbgValue(llvm::MDNode*, unsigned int, unsigned long long, llvm::DebugLoc, unsigned int)"},
    {"_ZN4llvm12SelectionDAG18ReplaceAllUsesWithENS_7SDValueES1_PNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::ReplaceAllUsesWith(llvm::SDValue, llvm::SDValue, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN12_GLOBAL__N_118RAUWUpdateListenerD1Ev", "(anonymous namespace)::RAUWUpdateListener::~RAUWUpdateListener()"},
    {"_ZN4llvm12SelectionDAG18ReplaceAllUsesWithEPNS_6SDNodeEPKNS_7SDValueEPNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::ReplaceAllUsesWith(llvm::SDNode*, llvm::SDValue const*, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG25ReplaceAllUsesOfValueWithENS_7SDValueES1_PNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::ReplaceAllUsesOfValueWith(llvm::SDValue, llvm::SDValue, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG26ReplaceAllUsesOfValuesWithEPKNS_7SDValueES3_jPNS0_17DAGUpdateListenerE", "llvm::SelectionDAG::ReplaceAllUsesOfValuesWith(llvm::SDValue const*, llvm::SDValue const*, unsigned int, llvm::SelectionDAG::DAGUpdateListener*)"},
    {"_ZN4llvm12SelectionDAG22AssignTopologicalOrderEv", "llvm::SelectionDAG::AssignTopologicalOrder()"},
    {"_ZN4llvm14checkForCyclesEPKNS_6SDNodeE", "llvm::checkForCycles(llvm::SDNode const*)"},
    {"_ZN4llvm12SelectionDAG14AssignOrderingEPKNS_6SDNodeEj", "llvm::SelectionDAG::AssignOrdering(llvm::SDNode const*, unsigned int)"},
    {"_ZNK4llvm12SelectionDAG11GetOrderingEPKNS_6SDNodeE", "llvm::SelectionDAG::GetOrdering(llvm::SDNode const*) const"},
    {"_ZN4llvm12SelectionDAG11AddDbgValueEPNS_10SDDbgValueEPNS_6SDNodeEb", "llvm::SelectionDAG::AddDbgValue(llvm::SDDbgValue*, llvm::SDNode*, bool)"},
    {"_ZN4llvm12SelectionDAG17TransferDbgValuesENS_7SDValueES1_", "llvm::SelectionDAG::TransferDbgValues(llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm12HandleSDNodeD2Ev", "llvm::HandleSDNode::~HandleSDNode()"},
    {"_ZN4llvm19GlobalAddressSDNodeC2EjNS_8DebugLocEPKNS_11GlobalValueENS_3EVTExh", "llvm::GlobalAddressSDNode::GlobalAddressSDNode(unsigned int, llvm::DebugLoc, llvm::GlobalValue const*, llvm::EVT, long long, unsigned char)"},
    {"_ZN4llvm9MemSDNodeC1EjNS_8DebugLocENS_8SDVTListENS_3EVTEPNS_17MachineMemOperandE", "llvm::MemSDNode::MemSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm9MemSDNodeC2EjNS_8DebugLocENS_8SDVTListENS_3EVTEPNS_17MachineMemOperandE", "llvm::MemSDNode::MemSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm9MemSDNodeC1EjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEjNS_3EVTEPNS_17MachineMemOperandE", "llvm::MemSDNode::MemSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm9MemSDNodeC2EjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEjNS_3EVTEPNS_17MachineMemOperandE", "llvm::MemSDNode::MemSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZNK4llvm6SDNode7ProfileERNS_16FoldingSetNodeIDE", "llvm::SDNode::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm6SDNode15hasNUsesOfValueEjj", "llvm::SDNode::hasNUsesOfValue(unsigned int, unsigned int) const"},
    {"_ZNK4llvm6SDNode16hasAnyUseOfValueEj", "llvm::SDNode::hasAnyUseOfValue(unsigned int) const"},
    {"_ZNK4llvm6SDNode12isOnlyUserOfEPS0_", "llvm::SDNode::isOnlyUserOf(llvm::SDNode*) const"},
    {"_ZNK4llvm7SDValue11isOperandOfEPNS_6SDNodeE", "llvm::SDValue::isOperandOf(llvm::SDNode*) const"},
    {"_ZNK4llvm6SDNode11isOperandOfEPS0_", "llvm::SDNode::isOperandOf(llvm::SDNode*) const"},
    {"_ZNK4llvm7SDValue30reachesChainWithoutSideEffectsES0_j", "llvm::SDValue::reachesChainWithoutSideEffects(llvm::SDValue, unsigned int) const"},
    {"_ZNK4llvm6SDNode15isPredecessorOfEPS0_", "llvm::SDNode::isPredecessorOf(llvm::SDNode*) const"},
    {"_ZNK4llvm6SDNode21getConstantOperandValEj", "llvm::SDNode::getConstantOperandVal(unsigned int) const"},
    {"_ZNK4llvm6SDNode16getOperationNameEPKNS_12SelectionDAGE", "llvm::SDNode::getOperationName(llvm::SelectionDAG const*) const"},
    {"_ZN4llvm6SDNode18getIndexedModeNameENS_3ISD14MemIndexedModeE", "llvm::SDNode::getIndexedModeName(llvm::ISD::MemIndexedMode)"},
    {"_ZN4llvm3ISD10ArgFlagsTy17getArgFlagsStringEv", "llvm::ISD::ArgFlagsTy::getArgFlagsString()"},
    {"_ZNK4llvm6SDNode4dumpEv", "llvm::SDNode::dump() const"},
    {"_ZNK4llvm6SDNode4dumpEPKNS_12SelectionDAGE", "llvm::SDNode::dump(llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode5printERNS_11raw_ostreamEPKNS_12SelectionDAGE", "llvm::SDNode::print(llvm::raw_ostream&, llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode11print_typesERNS_11raw_ostreamEPKNS_12SelectionDAGE", "llvm::SDNode::print_types(llvm::raw_ostream&, llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode13print_detailsERNS_11raw_ostreamEPKNS_12SelectionDAGE", "llvm::SDNode::print_details(llvm::raw_ostream&, llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode15printrWithDepthERNS_11raw_ostreamEPKNS_12SelectionDAGEj", "llvm::SDNode::printrWithDepth(llvm::raw_ostream&, llvm::SelectionDAG const*, unsigned int) const"},
    {"_Z21printrWithDepthHelperRN4llvm11raw_ostreamEPKNS_6SDNodeEPKNS_12SelectionDAGEjj", "printrWithDepthHelper(llvm::raw_ostream&, llvm::SDNode const*, llvm::SelectionDAG const*, unsigned int, unsigned int)"},
    {"_ZNK4llvm6SDNode10printrFullERNS_11raw_ostreamEPKNS_12SelectionDAGE", "llvm::SDNode::printrFull(llvm::raw_ostream&, llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode14dumprWithDepthEPKNS_12SelectionDAGEj", "llvm::SDNode::dumprWithDepth(llvm::SelectionDAG const*, unsigned int) const"},
    {"_ZNK4llvm6SDNode9dumprFullEPKNS_12SelectionDAGE", "llvm::SDNode::dumprFull(llvm::SelectionDAG const*) const"},
    {"_ZN4llvm12SelectionDAG14UnrollVectorOpEPNS_6SDNodeEj", "llvm::SelectionDAG::UnrollVectorOp(llvm::SDNode*, unsigned int)"},
    {"_ZNK4llvm12SelectionDAG17isConsecutiveLoadEPNS_10LoadSDNodeES2_ji", "llvm::SelectionDAG::isConsecutiveLoad(llvm::LoadSDNode*, llvm::LoadSDNode*, unsigned int, int) const"},
    {"_ZNK4llvm12SelectionDAG17InferPtrAlignmentENS_7SDValueE", "llvm::SelectionDAG::InferPtrAlignment(llvm::SDValue) const"},
    {"_ZNK4llvm12SelectionDAG4dumpEv", "llvm::SelectionDAG::dump() const"},
    {"_Z9DumpNodesPKN4llvm6SDNodeEjPKNS_12SelectionDAGE", "DumpNodes(llvm::SDNode const*, unsigned int, llvm::SelectionDAG const*)"},
    {"_ZNK4llvm6SDNode6printrERNS_11raw_ostreamEPKNS_12SelectionDAGE", "llvm::SDNode::printr(llvm::raw_ostream&, llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm6SDNode5dumprEv", "llvm::SDNode::dumpr() const"},
    {"_Z10DumpNodesrRN4llvm11raw_ostreamEPKNS_6SDNodeEjPKNS_12SelectionDAGERNS_11SmallPtrSetIS4_Lj128EEE", "DumpNodesr(llvm::raw_ostream&, llvm::SDNode const*, unsigned int, llvm::SelectionDAG const*, llvm::SmallPtrSet<llvm::SDNode const*, 128u>&)"},
    {"_ZNK4llvm6SDNode5dumprEPKNS_12SelectionDAGE", "llvm::SDNode::dumpr(llvm::SelectionDAG const*) const"},
    {"_ZNK4llvm19GlobalAddressSDNode15getAddressSpaceEv", "llvm::GlobalAddressSDNode::getAddressSpace() const"},
    {"_ZNK4llvm18ConstantPoolSDNode7getTypeEv", "llvm::ConstantPoolSDNode::getType() const"},
    {"_ZN4llvm17BuildVectorSDNode15isConstantSplatERNS_5APIntES2_RjRbjb", "llvm::BuildVectorSDNode::isConstantSplat(llvm::APInt&, llvm::APInt&, unsigned int&, bool&, unsigned int, bool)"},
    {"_ZN4llvm19ShuffleVectorSDNode11isSplatMaskEPKiNS_3EVTE", "llvm::ShuffleVectorSDNode::isSplatMask(int const*, llvm::EVT)"},
    {"_ZN4llvm14checkForCyclesEPKNS_12SelectionDAGE", "llvm::checkForCycles(llvm::SelectionDAG const*)"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_18EVTArrayEEEPvv", "void* llvm::object_creator<(anonymous namespace)::EVTArray>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_18EVTArrayEE4callEPv", "llvm::object_deleter<(anonymous namespace)::EVTArray>::call(void*)"},
    {"_ZSt16__introsort_loopIPN12_GLOBAL__N_17UseMemoElEvT_S3_T0_", "void std::__introsort_loop<(anonymous namespace)::UseMemo*, long>((anonymous namespace)::UseMemo*, (anonymous namespace)::UseMemo*, long)"},
    {"_ZSt16__insertion_sortIPN12_GLOBAL__N_17UseMemoEEvT_S3_", "void std::__insertion_sort<(anonymous namespace)::UseMemo*>((anonymous namespace)::UseMemo*, (anonymous namespace)::UseMemo*)"},
    {"_ZN12_GLOBAL__N_118RAUWUpdateListenerD0Ev", "(anonymous namespace)::RAUWUpdateListener::~RAUWUpdateListener()"},
    {"_ZN12_GLOBAL__N_118RAUWUpdateListener11NodeDeletedEPN4llvm6SDNodeES3_", "(anonymous namespace)::RAUWUpdateListener::NodeDeleted(llvm::SDNode*, llvm::SDNode*)"},
    {"_ZN12_GLOBAL__N_118RAUWUpdateListener11NodeUpdatedEPN4llvm6SDNodeE", "(anonymous namespace)::RAUWUpdateListener::NodeUpdated(llvm::SDNode*)"},
    {"_Z24FindOptimalMemOpLoweringRSt6vectorIN4llvm3EVTESaIS1_EEjyjjbbRNS0_12SelectionDAGERKNS0_14TargetLoweringE", "FindOptimalMemOpLowering(std::vector<llvm::EVT, std::allocator<llvm::EVT> >&, unsigned int, unsigned long long, unsigned int, unsigned int, bool, bool, llvm::SelectionDAG&, llvm::TargetLowering const&)"},
    {"_Z14getMemsetValueN4llvm7SDValueENS_3EVTERNS_12SelectionDAGENS_8DebugLocE", "getMemsetValue(llvm::SDValue, llvm::EVT, llvm::SelectionDAG&, llvm::DebugLoc)"},
    {"_Z9SplatBytejh", "SplatByte(unsigned int, unsigned char)"},
    {"_ZN4llvm10FoldingSetINS_6SDNodeEED1Ev", "llvm::FoldingSet<llvm::SDNode>::~FoldingSet()"},
    {"_ZNSt3mapISt4pairISshEPN4llvm6SDNodeESt4lessIS1_ESaIS0_IKS1_S4_EEEixERS7_", "std::map<std::pair<std::string, unsigned char>, llvm::SDNode*, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::operator[](std::pair<std::string, unsigned char> const&)"},
    {"_ZNK4llvm5APInt4sremERKS0_", "llvm::APInt::srem(llvm::APInt const&) const"},
    {"_ZN4llvm9SDDbgInfo3addEPNS_10SDDbgValueEPKNS_6SDNodeEb", "llvm::SDDbgInfo::add(llvm::SDDbgValue*, llvm::SDNode const*, bool)"},
    {"_ZN4llvm6SDNodeC2EjNS_8DebugLocENS_8SDVTListEPKNS_7SDValueEj", "llvm::SDNode::SDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue const*, unsigned int)"},
    {"_ZNSt6vectorIN4llvm3EVTESaIS1_EE7reserveEm", "std::vector<llvm::EVT, std::allocator<llvm::EVT> >::reserve(unsigned long)"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTES1_St9_IdentityIS1_ENS1_14compareRawBitsESaIS1_EE16_M_insert_uniqueERKS1_", "std::_Rb_tree<llvm::EVT, llvm::EVT, std::_Identity<llvm::EVT>, llvm::EVT::compareRawBits, std::allocator<llvm::EVT> >::_M_insert_unique(llvm::EVT const&)"},
    {"_ZN4llvm14object_creatorISt3setINS_3EVTENS2_14compareRawBitsESaIS2_EEEEPvv", "void* llvm::object_creator<std::set<llvm::EVT, llvm::EVT::compareRawBits, std::allocator<llvm::EVT> > >()"},
    {"_ZN4llvm14object_deleterISt3setINS_3EVTENS2_14compareRawBitsESaIS2_EEE4callEPv", "llvm::object_deleter<std::set<llvm::EVT, llvm::EVT::compareRawBits, std::allocator<llvm::EVT> > >::call(void*)"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTES1_St9_IdentityIS1_ENS1_14compareRawBitsESaIS1_EE8_M_eraseEPSt13_Rb_tree_nodeIS1_E", "std::_Rb_tree<llvm::EVT, llvm::EVT, std::_Identity<llvm::EVT>, llvm::EVT::compareRawBits, std::allocator<llvm::EVT> >::_M_erase(std::_Rb_tree_node<llvm::EVT>*)"},
    {"_ZNSt6vectorIN4llvm8SDVTListESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::SDVTList, std::allocator<llvm::SDVTList> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SDVTList*, std::vector<llvm::SDVTList, std::allocator<llvm::SDVTList> > >, llvm::SDVTList const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_3EVTELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::EVT, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIPN4llvm14CondCodeSDNodeESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::CondCodeSDNode*, std::allocator<llvm::CondCodeSDNode*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::CondCodeSDNode**, std::vector<llvm::CondCodeSDNode*, std::allocator<llvm::CondCodeSDNode*> > >, unsigned long, llvm::CondCodeSDNode* const&)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::_M_insert_unique(std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE11lower_boundERS2_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::lower_bound(std::pair<std::string, unsigned char> const&)"},
    {"_ZN4llvm9StringMapIPNS_6SDNodeENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::SDNode*>& llvm::StringMap<llvm::SDNode*, llvm::MallocAllocator>::GetOrCreateValue<llvm::SDNode*>(llvm::StringRef, llvm::SDNode*)"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTESt4pairIKS1_PNS0_6SDNodeEESt10_Select1stIS6_ENS1_14compareRawBitsESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::EVT, std::pair<llvm::EVT const, llvm::SDNode*>, std::_Select1st<std::pair<llvm::EVT const, llvm::SDNode*> >, llvm::EVT::compareRawBits, std::allocator<std::pair<llvm::EVT const, llvm::SDNode*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::EVT const, llvm::SDNode*> >, std::pair<llvm::EVT const, llvm::SDNode*> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTESt4pairIKS1_PNS0_6SDNodeEESt10_Select1stIS6_ENS1_14compareRawBitsESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::EVT, std::pair<llvm::EVT const, llvm::SDNode*>, std::_Select1st<std::pair<llvm::EVT const, llvm::SDNode*> >, llvm::EVT::compareRawBits, std::allocator<std::pair<llvm::EVT const, llvm::SDNode*> > >::_M_insert_unique(std::pair<llvm::EVT const, llvm::SDNode*> const&)"},
    {"_ZNSt6vectorIPN4llvm6SDNodeESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > >, unsigned long, llvm::SDNode* const&)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTESt4pairIKS1_PNS0_6SDNodeEESt10_Select1stIS6_ENS1_14compareRawBitsESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::EVT, std::pair<llvm::EVT const, llvm::SDNode*>, std::_Select1st<std::pair<llvm::EVT const, llvm::SDNode*> >, llvm::EVT::compareRawBits, std::allocator<std::pair<llvm::EVT const, llvm::SDNode*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::EVT const, llvm::SDNode*> >*)"},
    {"_ZN4llvm10FoldingSetINS_6SDNodeEED0Ev", "llvm::FoldingSet<llvm::SDNode>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_6SDNodeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::SDNode>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_6SDNodeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS6_", "llvm::FoldingSet<llvm::SDNode>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_6SDNodeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::SDNode>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNSt8_Rb_treeIN4llvm3EVTESt4pairIKS1_PNS0_6SDNodeEESt10_Select1stIS6_ENS1_14compareRawBitsESaIS6_EE5eraseESt17_Rb_tree_iteratorIS6_ESD_", "std::_Rb_tree<llvm::EVT, std::pair<llvm::EVT const, llvm::SDNode*>, std::_Select1st<std::pair<llvm::EVT const, llvm::SDNode*> >, llvm::EVT::compareRawBits, std::allocator<std::pair<llvm::EVT const, llvm::SDNode*> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::EVT const, llvm::SDNode*> >, std::_Rb_tree_iterator<std::pair<llvm::EVT const, llvm::SDNode*> >)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE5eraseESt17_Rb_tree_iteratorIS6_ESE_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::erase(std::_Rb_tree_iterator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::_Rb_tree_iterator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >)"},
    {"_ZNSt8_Rb_treeISt4pairISshES0_IKS1_PN4llvm6SDNodeEESt10_Select1stIS6_ESt4lessIS1_ESaIS6_EE11upper_boundERS2_", "std::_Rb_tree<std::pair<std::string, unsigned char>, std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*>, std::_Select1st<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> >, std::less<std::pair<std::string, unsigned char> >, std::allocator<std::pair<std::pair<std::string, unsigned char> const, llvm::SDNode*> > >::upper_bound(std::pair<std::string, unsigned char> const&)"},
    {"_ZN4llvm11SmallVectorIPNS_6SDNodeELj16EEC2EjRKS2_", "llvm::SmallVector<llvm::SDNode*, 16u>::SmallVector(unsigned int, llvm::SDNode* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeENS_11SmallVectorIPNS_10SDDbgValueELj2EEENS_12DenseMapInfoIS3_EENS8_IS7_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u>, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<llvm::SDDbgValue*, 2u> > >::FindAndConstruct(llvm::SDNode const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeENS_11SmallVectorIPNS_10SDDbgValueELj2EEENS_12DenseMapInfoIS3_EENS8_IS7_EEE16InsertIntoBucketERKS3_RKS7_PSt4pairIS3_S7_E", "llvm::DenseMap<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u>, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<llvm::SDDbgValue*, 2u> > >::InsertIntoBucket(llvm::SDNode const* const&, llvm::SmallVector<llvm::SDDbgValue*, 2u> const&, std::pair<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeENS_11SmallVectorIPNS_10SDDbgValueELj2EEENS_12DenseMapInfoIS3_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u>, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<llvm::SDDbgValue*, 2u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPNS_10SDDbgValueEEaSERKS3_", "llvm::SmallVectorImpl<llvm::SDDbgValue*>::operator=(llvm::SmallVectorImpl<llvm::SDDbgValue*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::SDNode const*, unsigned int, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::SDNode const* const&, unsigned int const&, std::pair<llvm::SDNode const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::SDNode const*, unsigned int, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm12LSBaseSDNodeC2ENS_3ISD8NodeTypeENS_8DebugLocEPNS_7SDValueEjNS_8SDVTListENS1_14MemIndexedModeENS_3EVTEPNS_17MachineMemOperandE", "llvm::LSBaseSDNode::LSBaseSDNode(llvm::ISD::NodeType, llvm::DebugLoc, llvm::SDValue*, unsigned int, llvm::SDVTList, llvm::ISD::MemIndexedMode, llvm::EVT, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12AtomicSDNodeC2EjNS_8DebugLocENS_8SDVTListENS_3EVTENS_7SDValueES4_S4_PNS_17MachineMemOperandE", "llvm::AtomicSDNode::AtomicSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachineMemOperand*)"},
    {"_ZN4llvm12AtomicSDNodeC2EjNS_8DebugLocENS_8SDVTListENS_3EVTENS_7SDValueES4_S4_S4_PNS_17MachineMemOperandE", "llvm::AtomicSDNode::AtomicSDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::MachineMemOperand*)"},
    {"_ZN4llvm13TernarySDNodeC2EjNS_8DebugLocENS_8SDVTListENS_7SDValueES3_S3_", "llvm::TernarySDNode::TernarySDNode(unsigned int, llvm::DebugLoc, llvm::SDVTList, llvm::SDValue, llvm::SDValue, llvm::SDValue)"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeENS_11SmallVectorIPNS_10SDDbgValueELj2EEENS_12DenseMapInfoIS3_EENS8_IS7_EEE5clearEv", "llvm::DenseMap<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u>, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<llvm::SDDbgValue*, 2u> > >::clear()"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeENS_11SmallVectorIPNS_10SDDbgValueELj2EEENS_12DenseMapInfoIS3_EENS8_IS7_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::SDNode const*, llvm::SmallVector<llvm::SDDbgValue*, 2u>, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<llvm::SDDbgValue*, 2u> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_6SDNodeEjNS_12DenseMapInfoIS3_EENS4_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::SDNode const*, unsigned int, llvm::DenseMapInfo<llvm::SDNode const*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm19SelectionDAGBuilder4initEPNS_14GCFunctionInfoERNS_13AliasAnalysisE", "llvm::SelectionDAGBuilder::init(llvm::GCFunctionInfo*, llvm::AliasAnalysis&)"},
    {"_ZN4llvm19SelectionDAGBuilder5clearEv", "llvm::SelectionDAGBuilder::clear()"},
    {"_ZN4llvm19SelectionDAGBuilder7getRootEv", "llvm::SelectionDAGBuilder::getRoot()"},
    {"_ZN4llvm19SelectionDAGBuilder14getControlRootEv", "llvm::SelectionDAGBuilder::getControlRoot()"},
    {"_ZN4llvm19SelectionDAGBuilder20AssignOrderingToNodeEPKNS_6SDNodeE", "llvm::SelectionDAGBuilder::AssignOrderingToNode(llvm::SDNode const*)"},
    {"_ZN4llvm19SelectionDAGBuilder5visitERKNS_11InstructionE", "llvm::SelectionDAGBuilder::visit(llvm::Instruction const&)"},
    {"_ZN4llvm19SelectionDAGBuilder31HandlePHINodesInSuccessorBlocksEPKNS_10BasicBlockE", "llvm::SelectionDAGBuilder::HandlePHINodesInSuccessorBlocks(llvm::BasicBlock const*)"},
    {"_ZN4llvm19SelectionDAGBuilder5visitEjRKNS_4UserE", "llvm::SelectionDAGBuilder::visit(unsigned int, llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder24CopyToExportRegsIfNeededEPKNS_5ValueE", "llvm::SelectionDAGBuilder::CopyToExportRegsIfNeeded(llvm::Value const*)"},
    {"_ZN4llvm19SelectionDAGBuilder8visitPHIERKNS_7PHINodeE", "llvm::SelectionDAGBuilder::visitPHI(llvm::PHINode const&)"},
    {"_ZN4llvm19SelectionDAGBuilder8visitRetERKNS_10ReturnInstE", "llvm::SelectionDAGBuilder::visitRet(llvm::ReturnInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder7visitBrERKNS_10BranchInstE", "llvm::SelectionDAGBuilder::visitBr(llvm::BranchInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitSwitchERKNS_10SwitchInstE", "llvm::SelectionDAGBuilder::visitSwitch(llvm::SwitchInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder15visitIndirectBrERKNS_14IndirectBrInstE", "llvm::SelectionDAGBuilder::visitIndirectBr(llvm::IndirectBrInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitInvokeERKNS_10InvokeInstE", "llvm::SelectionDAGBuilder::visitInvoke(llvm::InvokeInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitUnwindERKNS_10UnwindInstE", "llvm::SelectionDAGBuilder::visitUnwind(llvm::UnwindInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitFSubERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFSub(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitAllocaERKNS_10AllocaInstE", "llvm::SelectionDAGBuilder::visitAlloca(llvm::AllocaInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitLoadERKNS_8LoadInstE", "llvm::SelectionDAGBuilder::visitLoad(llvm::LoadInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitStoreERKNS_9StoreInstE", "llvm::SelectionDAGBuilder::visitStore(llvm::StoreInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder18visitGetElementPtrERKNS_4UserE", "llvm::SelectionDAGBuilder::visitGetElementPtr(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitTruncERKNS_4UserE", "llvm::SelectionDAGBuilder::visitTrunc(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitZExtERKNS_4UserE", "llvm::SelectionDAGBuilder::visitZExt(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitSExtERKNS_4UserE", "llvm::SelectionDAGBuilder::visitSExt(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitFPToUIERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFPToUI(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitFPToSIERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFPToSI(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitUIToFPERKNS_4UserE", "llvm::SelectionDAGBuilder::visitUIToFP(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitSIToFPERKNS_4UserE", "llvm::SelectionDAGBuilder::visitSIToFP(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder12visitFPTruncERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFPTrunc(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitFPExtERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFPExt(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder13visitPtrToIntERKNS_4UserE", "llvm::SelectionDAGBuilder::visitPtrToInt(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder13visitIntToPtrERKNS_4UserE", "llvm::SelectionDAGBuilder::visitIntToPtr(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder12visitBitCastERKNS_4UserE", "llvm::SelectionDAGBuilder::visitBitCast(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitICmpERKNS_4UserE", "llvm::SelectionDAGBuilder::visitICmp(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitFCmpERKNS_4UserE", "llvm::SelectionDAGBuilder::visitFCmp(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitCallERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitCall(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitSelectERKNS_4UserE", "llvm::SelectionDAGBuilder::visitSelect(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitVAArgERKNS_9VAArgInstE", "llvm::SelectionDAGBuilder::visitVAArg(llvm::VAArgInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder19visitExtractElementERKNS_4UserE", "llvm::SelectionDAGBuilder::visitExtractElement(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder18visitInsertElementERKNS_4UserE", "llvm::SelectionDAGBuilder::visitInsertElement(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder18visitShuffleVectorERKNS_4UserE", "llvm::SelectionDAGBuilder::visitShuffleVector(llvm::User const&)"},
    {"_ZN4llvm19SelectionDAGBuilder17visitExtractValueERKNS_16ExtractValueInstE", "llvm::SelectionDAGBuilder::visitExtractValue(llvm::ExtractValueInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder16visitInsertValueERKNS_15InsertValueInstE", "llvm::SelectionDAGBuilder::visitInsertValue(llvm::InsertValueInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder8getValueEPKNS_5ValueE", "llvm::SelectionDAGBuilder::getValue(llvm::Value const*)"},
    {"_ZN4llvm19SelectionDAGBuilder24resolveDanglingDebugInfoEPKNS_5ValueENS_7SDValueE", "llvm::SelectionDAGBuilder::resolveDanglingDebugInfo(llvm::Value const*, llvm::SDValue)"},
    {"_ZN4llvm19SelectionDAGBuilder24EmitFuncArgumentDbgValueEPKNS_5ValueEPNS_6MDNodeExRKNS_7SDValueE", "llvm::SelectionDAGBuilder::EmitFuncArgumentDbgValue(llvm::Value const*, llvm::MDNode*, long long, llvm::SDValue const&)"},
    {"_ZN12_GLOBAL__N_112RegsForValueC1ERN4llvm11LLVMContextERKNS1_14TargetLoweringEjPKNS1_4TypeE", "(anonymous namespace)::RegsForValue::RegsForValue(llvm::LLVMContext&, llvm::TargetLowering const&, unsigned int, llvm::Type const*)"},
    {"_ZNK12_GLOBAL__N_112RegsForValue15getCopyFromRegsERN4llvm12SelectionDAGERNS1_20FunctionLoweringInfoENS1_8DebugLocERNS1_7SDValueEPS7_", "(anonymous namespace)::RegsForValue::getCopyFromRegs(llvm::SelectionDAG&, llvm::FunctionLoweringInfo&, llvm::DebugLoc, llvm::SDValue&, llvm::SDValue*) const"},
    {"_ZN4llvm19SelectionDAGBuilder12getValueImplEPKNS_5ValueE", "llvm::SelectionDAGBuilder::getValueImpl(llvm::Value const*)"},
    {"_ZN4llvm19SelectionDAGBuilder19getNonRegisterValueEPKNS_5ValueE", "llvm::SelectionDAGBuilder::getNonRegisterValue(llvm::Value const*)"},
    {"_Z14getCopyToPartsRN4llvm12SelectionDAGENS_8DebugLocENS_7SDValueEPS3_jNS_3EVTENS_3ISD8NodeTypeE", "getCopyToParts(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue*, unsigned int, llvm::EVT, llvm::ISD::NodeType)"},
    {"_ZN4llvm19SelectionDAGBuilder26CopyValueToVirtualRegisterEPKNS_5ValueEj", "llvm::SelectionDAGBuilder::CopyValueToVirtualRegister(llvm::Value const*, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder22ExportFromCurrentBlockEPKNS_5ValueE", "llvm::SelectionDAGBuilder::ExportFromCurrentBlock(llvm::Value const*)"},
    {"_ZN4llvm19SelectionDAGBuilder28isExportableFromCurrentBlockEPKNS_5ValueEPKNS_10BasicBlockE", "llvm::SelectionDAGBuilder::isExportableFromCurrentBlock(llvm::Value const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm19SelectionDAGBuilder28EmitBranchForMergedConditionEPKNS_5ValueEPNS_17MachineBasicBlockES5_S5_S5_", "llvm::SelectionDAGBuilder::EmitBranchForMergedCondition(llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder20FindMergedConditionsEPKNS_5ValueEPNS_17MachineBasicBlockES5_S5_S5_j", "llvm::SelectionDAGBuilder::FindMergedConditions(llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder20ShouldEmitAsBranchesERKSt6vectorINS0_9CaseBlockESaIS2_EE", "llvm::SelectionDAGBuilder::ShouldEmitAsBranches(std::vector<llvm::SelectionDAGBuilder::CaseBlock, std::allocator<llvm::SelectionDAGBuilder::CaseBlock> > const&)"},
    {"_ZN4llvm19SelectionDAGBuilder15visitSwitchCaseERNS0_9CaseBlockEPNS_17MachineBasicBlockE", "llvm::SelectionDAGBuilder::visitSwitchCase(llvm::SelectionDAGBuilder::CaseBlock&, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder14visitJumpTableERNS0_9JumpTableE", "llvm::SelectionDAGBuilder::visitJumpTable(llvm::SelectionDAGBuilder::JumpTable&)"},
    {"_ZN4llvm19SelectionDAGBuilder20visitJumpTableHeaderERNS0_9JumpTableERNS0_15JumpTableHeaderEPNS_17MachineBasicBlockE", "llvm::SelectionDAGBuilder::visitJumpTableHeader(llvm::SelectionDAGBuilder::JumpTable&, llvm::SelectionDAGBuilder::JumpTableHeader&, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder18visitBitTestHeaderERNS0_12BitTestBlockEPNS_17MachineBasicBlockE", "llvm::SelectionDAGBuilder::visitBitTestHeader(llvm::SelectionDAGBuilder::BitTestBlock&, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder16visitBitTestCaseERNS0_12BitTestBlockEPNS_17MachineBasicBlockEjRNS0_11BitTestCaseES4_", "llvm::SelectionDAGBuilder::visitBitTestCase(llvm::SelectionDAGBuilder::BitTestBlock&, llvm::MachineBasicBlock*, unsigned int, llvm::SelectionDAGBuilder::BitTestCase&, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder14visitInlineAsmENS_17ImmutableCallSiteE", "llvm::SelectionDAGBuilder::visitInlineAsm(llvm::ImmutableCallSite)"},
    {"_ZN4llvm19SelectionDAGBuilder11LowerCallToENS_17ImmutableCallSiteENS_7SDValueEbPNS_17MachineBasicBlockE", "llvm::SelectionDAGBuilder::LowerCallTo(llvm::ImmutableCallSite, llvm::SDValue, bool, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder22handleSmallSwitchRangeERNS0_7CaseRecERSt6vectorIS1_SaIS1_EEPKNS_5ValueEPNS_17MachineBasicBlockESB_", "llvm::SelectionDAGBuilder::handleSmallSwitchRange(llvm::SelectionDAGBuilder::CaseRec&, std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> >&, llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder18handleJTSwitchCaseERNS0_7CaseRecERSt6vectorIS1_SaIS1_EEPKNS_5ValueEPNS_17MachineBasicBlockESB_", "llvm::SelectionDAGBuilder::handleJTSwitchCase(llvm::SelectionDAGBuilder::CaseRec&, std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> >&, llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_Z12ComputeRangeRKN4llvm5APIntES2_", "ComputeRange(llvm::APInt const&, llvm::APInt const&)"},
    {"_ZN4llvm19SelectionDAGBuilder23handleBTSplitSwitchCaseERNS0_7CaseRecERSt6vectorIS1_SaIS1_EEPKNS_5ValueEPNS_17MachineBasicBlockESB_", "llvm::SelectionDAGBuilder::handleBTSplitSwitchCase(llvm::SelectionDAGBuilder::CaseRec&, std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> >&, llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder24handleBitTestsSwitchCaseERNS0_7CaseRecERSt6vectorIS1_SaIS1_EEPKNS_5ValueEPNS_17MachineBasicBlockESB_", "llvm::SelectionDAGBuilder::handleBitTestsSwitchCase(llvm::SelectionDAGBuilder::CaseRec&, std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> >&, llvm::Value const*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder10ClusterifyERSt6vectorINS0_4CaseESaIS2_EERKNS_10SwitchInstE", "llvm::SelectionDAGBuilder::Clusterify(std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> >&, llvm::SwitchInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder16UpdateSplitBlockEPNS_17MachineBasicBlockES2_", "llvm::SelectionDAGBuilder::UpdateSplitBlock(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitBinaryERKNS_4UserEj", "llvm::SelectionDAGBuilder::visitBinary(llvm::User const&, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitShiftERKNS_4UserEj", "llvm::SelectionDAGBuilder::visitShift(llvm::User const&, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder20visitTargetIntrinsicERKNS_8CallInstEj", "llvm::SelectionDAGBuilder::visitTargetIntrinsic(llvm::CallInst const&, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder21implVisitBinaryAtomicERKNS_8CallInstENS_3ISD8NodeTypeE", "llvm::SelectionDAGBuilder::implVisitBinaryAtomic(llvm::CallInst const&, llvm::ISD::NodeType)"},
    {"_ZN4llvm19SelectionDAGBuilder20implVisitAluOverflowERKNS_8CallInstENS_3ISD8NodeTypeE", "llvm::SelectionDAGBuilder::implVisitAluOverflow(llvm::CallInst const&, llvm::ISD::NodeType)"},
    {"_ZN4llvm19SelectionDAGBuilder8visitExpERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitExp(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder8visitLogERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitLog(llvm::CallInst const&)"},
    {"_Z11GetExponentRN4llvm12SelectionDAGENS_7SDValueERKNS_14TargetLoweringENS_8DebugLocE", "GetExponent(llvm::SelectionDAG&, llvm::SDValue, llvm::TargetLowering const&, llvm::DebugLoc)"},
    {"_Z14GetSignificandRN4llvm12SelectionDAGENS_7SDValueENS_8DebugLocE", "GetSignificand(llvm::SelectionDAG&, llvm::SDValue, llvm::DebugLoc)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitLog2ERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitLog2(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitLog10ERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitLog10(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder9visitExp2ERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitExp2(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder8visitPowERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitPow(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder18visitIntrinsicCallERKNS_8CallInstEj", "llvm::SelectionDAGBuilder::visitIntrinsicCall(llvm::CallInst const&, unsigned int)"},
    {"_ZN4llvm19SelectionDAGBuilder12visitVAStartERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitVAStart(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder10visitVAEndERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitVAEnd(llvm::CallInst const&)"},
    {"_ZN4llvm19SelectionDAGBuilder11visitVACopyERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitVACopy(llvm::CallInst const&)"},
    {"_ZNK4llvm14TargetLowering11LowerCallToENS_7SDValueEPKNS_4TypeEbbbbjNS_11CallingConv2IDEbbS1_RSt6vectorINS0_12ArgListEntryESaIS8_EERNS_12SelectionDAGENS_8DebugLocE", "llvm::TargetLowering::LowerCallTo(llvm::SDValue, llvm::Type const*, bool, bool, bool, bool, unsigned int, llvm::CallingConv::ID, bool, bool, llvm::SDValue, std::vector<llvm::TargetLowering::ArgListEntry, std::allocator<llvm::TargetLowering::ArgListEntry> >&, llvm::SelectionDAG&, llvm::DebugLoc) const"},
    {"_Z16getCopyFromPartsRN4llvm12SelectionDAGENS_8DebugLocEPKNS_7SDValueEjNS_3EVTES6_NS_3ISD8NodeTypeE", "getCopyFromParts(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue const*, unsigned int, llvm::EVT, llvm::EVT, llvm::ISD::NodeType)"},
    {"_ZN4llvm19SelectionDAGBuilder15visitMemCmpCallERKNS_8CallInstE", "llvm::SelectionDAGBuilder::visitMemCmpCall(llvm::CallInst const&)"},
    {"_Z13getMemCmpLoadPKN4llvm5ValueENS_3MVTEPKNS_4TypeERNS_19SelectionDAGBuilderE", "getMemCmpLoad(llvm::Value const*, llvm::MVT, llvm::Type const*, llvm::SelectionDAGBuilder&)"},
    {"_ZN4llvm19SelectionDAGBuilder20GetRegistersForValueERNS_20SDISelAsmOperandInfoERSt3setIjSt4lessIjESaIjEES8_", "llvm::SelectionDAGBuilder::GetRegistersForValue(llvm::SDISelAsmOperandInfo&, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&)"},
    {"_ZN12_GLOBAL__N_112RegsForValueC1ERKN4llvm11SmallVectorIjLj4EEENS1_3EVTES6_", "(anonymous namespace)::RegsForValue::RegsForValue(llvm::SmallVector<unsigned int, 4u> const&, llvm::EVT, llvm::EVT)"},
    {"_ZNK12_GLOBAL__N_112RegsForValue20AddInlineAsmOperandsEjbjRN4llvm12SelectionDAGERSt6vectorINS1_7SDValueESaIS5_EE", "(anonymous namespace)::RegsForValue::AddInlineAsmOperands(unsigned int, bool, unsigned int, llvm::SelectionDAG&, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&) const"},
    {"_ZNK12_GLOBAL__N_112RegsForValue13getCopyToRegsEN4llvm7SDValueERNS1_12SelectionDAGENS1_8DebugLocERS2_PS2_", "(anonymous namespace)::RegsForValue::getCopyToRegs(llvm::SDValue, llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue&, llvm::SDValue*) const"},
    {"_ZNK4llvm14TargetLowering21LowerOperationWrapperEPNS_6SDNodeERNS_15SmallVectorImplINS_7SDValueEEERNS_12SelectionDAGE", "llvm::TargetLowering::LowerOperationWrapper(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering14LowerOperationENS_7SDValueERNS_12SelectionDAGE", "llvm::TargetLowering::LowerOperation(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZN4llvm16SelectionDAGISel14LowerArgumentsEPKNS_10BasicBlockE", "llvm::SelectionDAGISel::LowerArguments(llvm::BasicBlock const*)"},
    {"_ZSt22__uninitialized_copy_aIPSt4pairIN12_GLOBAL__N_112RegsForValueEPN4llvm5ValueEES7_S6_ET0_T_S9_S8_SaIT1_E", "std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>* std::__uninitialized_copy_a<std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>*, std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>*, std::pair<(anonymous namespace)::RegsForValue, llvm::Value*> >(std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>*, std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>*, std::pair<(anonymous namespace)::RegsForValue, llvm::Value*>*, std::allocator<std::pair<(anonymous namespace)::RegsForValue, llvm::Value*> >)"},
    {"_ZN4llvm25array_pod_sort_comparatorIPNS_10BasicBlockEEEiPKvS4_", "int llvm::array_pod_sort_comparator<llvm::BasicBlock*>(void const*, void const*)"},
    {"_ZN4llvm19SelectionDAGBuilder12visitUserOp1ERKNS_11InstructionE", "llvm::SelectionDAGBuilder::visitUserOp1(llvm::Instruction const&)"},
    {"_ZN4llvm19SelectionDAGBuilder12visitUserOp2ERKNS_11InstructionE", "llvm::SelectionDAGBuilder::visitUserOp2(llvm::Instruction const&)"},
    {"_ZN4llvm20FunctionLoweringInfo21InitializeRegForValueEPKNS_5ValueE", "llvm::FunctionLoweringInfo::InitializeRegForValue(llvm::Value const*)"},
    {"_ZNSt6vectorISt4pairIN4llvm19SelectionDAGBuilder15JumpTableHeaderENS2_9JumpTableEESaIS5_EE9push_backERKS5_", "std::vector<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>, std::allocator<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> > >::push_back(std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> const&)"},
    {"_ZNK4llvm5APInt3ugeEy", "llvm::APInt::uge(unsigned long long) const"},
    {"_ZNK4llvm5APInt3sltEy", "llvm::APInt::slt(unsigned long long) const"},
    {"_ZN4llvm17MachineModuleInfo18setVariableDbgInfoEPNS_6MDNodeEjNS_8DebugLocE", "llvm::MachineModuleInfo::setVariableDbgInfo(llvm::MDNode*, unsigned int, llvm::DebugLoc)"},
    {"_ZNSt6vectorIN4llvm14TargetLowering12ArgListEntryESaIS2_EE7reserveEm", "std::vector<llvm::TargetLowering::ArgListEntry, std::allocator<llvm::TargetLowering::ArgListEntry> >::reserve(unsigned long)"},
    {"_ZN4llvm8po_beginIPKNS_4TypeEEENS_11po_iteratorIT_NS_11SmallPtrSetIPNS_11GraphTraitsIS5_E8NodeTypeELj8EEELb0ES8_EES5_", "llvm::po_iterator<llvm::Type const*, llvm::SmallPtrSet<llvm::GraphTraits<llvm::Type const*>::NodeType*, 8u>, false, llvm::GraphTraits<llvm::Type const*> > llvm::po_begin<llvm::Type const*>(llvm::Type const*)"},
    {"_ZNK4llvm20SDISelAsmOperandInfo17MarkAllocatedRegsEbbRSt3setIjSt4lessIjESaIjEES6_RKNS_18TargetRegisterInfoE", "llvm::SDISelAsmOperandInfo::MarkAllocatedRegs(bool, bool, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&, llvm::TargetRegisterInfo const&) const"},
    {"_ZNK4llvm20SDISelAsmOperandInfo20getCallOperandValEVTERNS_11LLVMContextERKNS_14TargetLoweringEPKNS_10DataLayoutE", "llvm::SDISelAsmOperandInfo::getCallOperandValEVT(llvm::LLVMContext&, llvm::TargetLowering const&, llvm::DataLayout const*) const"},
    {"_ZN4llvm8DenseMapIPKNS_8ConstantEjNS_12DenseMapInfoIS3_EENS4_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Constant const*, unsigned int, llvm::DenseMapInfo<llvm::Constant const*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZNSt6vectorISt4pairIPN4llvm12MachineInstrEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::MachineInstr*, unsigned int>, std::allocator<std::pair<llvm::MachineInstr*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::MachineInstr*, unsigned int>*, std::vector<std::pair<llvm::MachineInstr*, unsigned int>, std::allocator<std::pair<llvm::MachineInstr*, unsigned int> > > >, std::pair<llvm::MachineInstr*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_8ConstantEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::Constant const*, unsigned int, llvm::DenseMapInfo<llvm::Constant const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Constant const* const&, unsigned int const&, std::pair<llvm::Constant const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8ConstantEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::Constant const*, unsigned int, llvm::DenseMapInfo<llvm::Constant const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_3ISD8InputArgELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::ISD::InputArg, false>::grow(unsigned long)"},
    {"_ZNSt6vectorISt4pairIN4llvm7SDValueEPKNS1_5ValueEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<llvm::SDValue, llvm::Value const*>, std::allocator<std::pair<llvm::SDValue, llvm::Value const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::SDValue, llvm::Value const*>*, std::vector<std::pair<llvm::SDValue, llvm::Value const*>, std::allocator<std::pair<llvm::SDValue, llvm::Value const*> > > >, std::pair<llvm::SDValue, llvm::Value const*> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_20SDISelAsmOperandInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SDISelAsmOperandInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm20SDISelAsmOperandInfoC2ERKS0_", "llvm::SDISelAsmOperandInfo::SDISelAsmOperandInfo(llvm::SDISelAsmOperandInfo const&)"},
    {"_ZN4llvm11po_iteratorIPKNS_4TypeENS_11SmallPtrSetIS3_Lj8EEELb0ENS_11GraphTraitsIS3_EEE13traverseChildEv", "llvm::po_iterator<llvm::Type const*, llvm::SmallPtrSet<llvm::Type const*, 8u>, false, llvm::GraphTraits<llvm::Type const*> >::traverseChild()"},
    {"_ZNSt6vectorISt4pairIPKN4llvm4TypeEPNS1_12PATypeHandleEESaIS7_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<std::pair<llvm::Type const*, llvm::PATypeHandle*>, std::allocator<std::pair<llvm::Type const*, llvm::PATypeHandle*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Type const*, llvm::PATypeHandle*>*, std::vector<std::pair<llvm::Type const*, llvm::PATypeHandle*>, std::allocator<std::pair<llvm::Type const*, llvm::PATypeHandle*> > > >, std::pair<llvm::Type const*, llvm::PATypeHandle*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_10AllocaInstEiNS_12DenseMapInfoIS3_EENS4_IiEEE16InsertIntoBucketERKS3_RKiPSt4pairIS3_iE", "llvm::DenseMap<llvm::AllocaInst const*, int, llvm::DenseMapInfo<llvm::AllocaInst const*>, llvm::DenseMapInfo<int> >::InsertIntoBucket(llvm::AllocaInst const* const&, int const&, std::pair<llvm::AllocaInst const*, int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_10AllocaInstEiNS_12DenseMapInfoIS3_EENS4_IiEEE4growEj", "llvm::DenseMap<llvm::AllocaInst const*, int, llvm::DenseMapInfo<llvm::AllocaInst const*>, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEElNS3_7CaseCmpEEvT_SB_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, long, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, long, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEENS3_7CaseCmpEEvT_SB_T0_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEENS3_7CaseCmpEEvT_SB_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZSt21__unguarded_partitionIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEES4_NS3_7CaseCmpEET_SB_SB_T0_T1_", "__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > > std::__unguarded_partition<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::Case, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::Case, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEENS3_7CaseCmpEEvT_SB_SB_T0_", "void std::__heap_select<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder4CaseESt6vectorIS4_SaIS4_EEEElS4_NS3_7CaseCmpEEvT_T0_SC_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, long, llvm::SelectionDAGBuilder::Case, llvm::SelectionDAGBuilder::CaseCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, long, long, llvm::SelectionDAGBuilder::Case, llvm::SelectionDAGBuilder::CaseCmp)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder4CaseESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::Case*, std::vector<llvm::SelectionDAGBuilder::Case, std::allocator<llvm::SelectionDAGBuilder::Case> > >, llvm::SelectionDAGBuilder::Case const&)"},
    {"_ZN9__gnu_cxx13new_allocatorIN4llvm19SelectionDAGBuilder12BitTestBlockEE9constructEPS3_RKS3_", "__gnu_cxx::new_allocator<llvm::SelectionDAGBuilder::BitTestBlock>::construct(llvm::SelectionDAGBuilder::BitTestBlock*, llvm::SelectionDAGBuilder::BitTestBlock const&)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder12BitTestBlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SelectionDAGBuilder::BitTestBlock, std::allocator<llvm::SelectionDAGBuilder::BitTestBlock> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::BitTestBlock*, std::vector<llvm::SelectionDAGBuilder::BitTestBlock, std::allocator<llvm::SelectionDAGBuilder::BitTestBlock> > >, llvm::SelectionDAGBuilder::BitTestBlock const&)"},
    {"_ZN4llvm19SelectionDAGBuilder12BitTestBlockaSERKS1_", "llvm::SelectionDAGBuilder::BitTestBlock::operator=(llvm::SelectionDAGBuilder::BitTestBlock const&)"},
    {"_ZSt24__uninitialized_copy_auxIPN4llvm19SelectionDAGBuilder12BitTestBlockES3_ET0_T_S5_S4_St12__false_type", "llvm::SelectionDAGBuilder::BitTestBlock* std::__uninitialized_copy_aux<llvm::SelectionDAGBuilder::BitTestBlock*, llvm::SelectionDAGBuilder::BitTestBlock*>(llvm::SelectionDAGBuilder::BitTestBlock*, llvm::SelectionDAGBuilder::BitTestBlock*, llvm::SelectionDAGBuilder::BitTestBlock*, std::__false_type)"},
    {"_ZN4llvm15SmallVectorImplINS_19SelectionDAGBuilder11BitTestCaseEEaSERKS3_", "llvm::SmallVectorImpl<llvm::SelectionDAGBuilder::BitTestCase>::operator=(llvm::SmallVectorImpl<llvm::SelectionDAGBuilder::BitTestCase> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_19SelectionDAGBuilder11BitTestCaseELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SelectionDAGBuilder::BitTestCase, false>::grow(unsigned long)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder8CaseBitsESt6vectorIS4_SaIS4_EEEElNS3_11CaseBitsCmpEEvT_SB_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, long, llvm::SelectionDAGBuilder::CaseBitsCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, long, llvm::SelectionDAGBuilder::CaseBitsCmp)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder8CaseBitsESt6vectorIS4_SaIS4_EEEENS3_11CaseBitsCmpEEvT_SB_T0_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder8CaseBitsESt6vectorIS4_SaIS4_EEEENS3_11CaseBitsCmpEEvT_SB_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder8CaseBitsESt6vectorIS4_SaIS4_EEEENS3_11CaseBitsCmpEEvT_SB_SB_T0_", "void std::__heap_select<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp)"},
    {"_ZSt9sort_heapIN9__gnu_cxx17__normal_iteratorIPN4llvm19SelectionDAGBuilder8CaseBitsESt6vectorIS4_SaIS4_EEEENS3_11CaseBitsCmpEEvT_SB_T0_", "void std::sort_heap<__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp>(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, __gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBitsCmp)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder8CaseBitsESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBits*, std::vector<llvm::SelectionDAGBuilder::CaseBits, std::allocator<llvm::SelectionDAGBuilder::CaseBits> > >, llvm::SelectionDAGBuilder::CaseBits const&)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder7CaseRecESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseRec*, std::vector<llvm::SelectionDAGBuilder::CaseRec, std::allocator<llvm::SelectionDAGBuilder::CaseRec> > >, llvm::SelectionDAGBuilder::CaseRec const&)"},
    {"_ZNSt6vectorISt4pairIN4llvm19SelectionDAGBuilder15JumpTableHeaderENS2_9JumpTableEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>, std::allocator<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::vector<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>, std::allocator<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> > > >, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> const&)"},
    {"_ZSt24__uninitialized_copy_auxIPSt4pairIN4llvm19SelectionDAGBuilder15JumpTableHeaderENS2_9JumpTableEES6_ET0_T_S8_S7_St12__false_type", "std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>* std::__uninitialized_copy_aux<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*>(std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::__false_type)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm19SelectionDAGBuilder15JumpTableHeaderENS5_9JumpTableEES9_EET0_T_SB_SA_", "std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*>(std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*, std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>*)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder9CaseBlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SelectionDAGBuilder::CaseBlock, std::allocator<llvm::SelectionDAGBuilder::CaseBlock> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SelectionDAGBuilder::CaseBlock*, std::vector<llvm::SelectionDAGBuilder::CaseBlock, std::allocator<llvm::SelectionDAGBuilder::CaseBlock> > >, llvm::SelectionDAGBuilder::CaseBlock const&)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_19SelectionDAGBuilder17DanglingDebugInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::Value const*, llvm::SelectionDAGBuilder::DanglingDebugInfo, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SelectionDAGBuilder::DanglingDebugInfo> >::InsertIntoBucket(llvm::Value const* const&, llvm::SelectionDAGBuilder::DanglingDebugInfo const&, std::pair<llvm::Value const*, llvm::SelectionDAGBuilder::DanglingDebugInfo>*)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_19SelectionDAGBuilder17DanglingDebugInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::Value const*, llvm::SelectionDAGBuilder::DanglingDebugInfo, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SelectionDAGBuilder::DanglingDebugInfo> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_19SelectionDAGBuilder17DanglingDebugInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Value const*, llvm::SelectionDAGBuilder::DanglingDebugInfo, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SelectionDAGBuilder::DanglingDebugInfo> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_7SDValueENS_12DenseMapInfoIS3_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Value const*, llvm::SDValue, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SDValue> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_7SDValueENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::Value const*, llvm::SDValue, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SDValue> >::InsertIntoBucket(llvm::Value const* const&, llvm::SDValue const&, std::pair<llvm::Value const*, llvm::SDValue>*)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueENS_7SDValueENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Value const*, llvm::SDValue, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<llvm::SDValue> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplINS_3EVTEEaSERKS2_", "llvm::SmallVectorImpl<llvm::EVT>::operator=(llvm::SmallVectorImpl<llvm::EVT> const&)"},
    {"_ZN4llvm20SDISelAsmOperandInfoD2Ev", "llvm::SDISelAsmOperandInfo::~SDISelAsmOperandInfo()"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::MCSymbol*, unsigned int, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MCSymbol* const&, unsigned int const&, std::pair<llvm::MCSymbol*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::MCSymbol*, unsigned int, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm6GCRootESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::GCRoot, std::allocator<llvm::GCRoot> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GCRoot*, std::vector<llvm::GCRoot, std::allocator<llvm::GCRoot> > >, llvm::GCRoot const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairINS_10TrackingVHINS_6MDNodeEEES1_IjNS_8DebugLocEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::TrackingVH<llvm::MDNode>, std::pair<unsigned int, llvm::DebugLoc> >, false>::grow(unsigned long)"},
    {"_ZN4llvm19SelectionDAGBuilder12BitTestBlockC2ENS_5APIntES2_PKNS_5ValueEjNS_3EVTEbPNS_17MachineBasicBlockES8_RKNS_11SmallVectorINS0_11BitTestCaseELj3EEE", "llvm::SelectionDAGBuilder::BitTestBlock::BitTestBlock(llvm::APInt, llvm::APInt, llvm::Value const*, unsigned int, llvm::EVT, bool, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::SmallVector<llvm::SelectionDAGBuilder::BitTestCase, 3u> const&)"},
    {"_ZN4llvm22createDefaultSchedulerEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelE", "llvm::createDefaultScheduler(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)"},
    {"_ZNK4llvm14TargetLowering27EmitInstrWithCustomInserterEPNS_12MachineInstrEPNS_17MachineBasicBlockE", "llvm::TargetLowering::EmitInstrWithCustomInserter(llvm::MachineInstr*, llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm16SelectionDAGISelC1ERKNS_13TargetMachineENS_10CodeGenOpt5LevelE", "llvm::SelectionDAGISel::SelectionDAGISel(llvm::TargetMachine const&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16SelectionDAGISelC2ERKNS_13TargetMachineENS_10CodeGenOpt5LevelE", "llvm::SelectionDAGISel::SelectionDAGISel(llvm::TargetMachine const&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16SelectionDAGISelD0Ev", "llvm::SelectionDAGISel::~SelectionDAGISel()"},
    {"_ZN4llvm16SelectionDAGISelD1Ev", "llvm::SelectionDAGISel::~SelectionDAGISel()"},
    {"_ZN4llvm16SelectionDAGISelD2Ev", "llvm::SelectionDAGISel::~SelectionDAGISel()"},
    {"_ZNK4llvm16SelectionDAGISel16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::SelectionDAGISel::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm16SelectionDAGISel20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::SelectionDAGISel::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm16SelectionDAGISel20SelectAllBasicBlocksERKNS_8FunctionE", "llvm::SelectionDAGISel::SelectAllBasicBlocks(llvm::Function const&)"},
    {"_ZN4llvm16SelectionDAGISel16SelectBasicBlockENS_14ilist_iteratorIKNS_11InstructionEEES4_Rb", "llvm::SelectionDAGISel::SelectBasicBlock(llvm::ilist_iterator<llvm::Instruction const>, llvm::ilist_iterator<llvm::Instruction const>, bool&)"},
    {"_ZN4llvm16SelectionDAGISel17CodeGenAndEmitDAGEv", "llvm::SelectionDAGISel::CodeGenAndEmitDAG()"},
    {"_ZN4llvm16SelectionDAGISel22ComputeLiveOutVRegInfoEv", "llvm::SelectionDAGISel::ComputeLiveOutVRegInfo()"},
    {"_ZN4llvm16SelectionDAGISel22DoInstructionSelectionEv", "llvm::SelectionDAGISel::DoInstructionSelection()"},
    {"_ZN4llvm16SelectionDAGISel15CreateSchedulerEv", "llvm::SelectionDAGISel::CreateScheduler()"},
    {"_ZN4llvm16SelectionDAGISel19PrepareEHLandingPadEv", "llvm::SelectionDAGISel::PrepareEHLandingPad()"},
    {"_ZN4llvm16SelectionDAGISel21TryToFoldFastISelLoadEPKNS_8LoadInstEPNS_8FastISelE", "llvm::SelectionDAGISel::TryToFoldFastISelLoad(llvm::LoadInst const*, llvm::FastISel*)"},
    {"_ZN4llvm16SelectionDAGISel16FinishBasicBlockEv", "llvm::SelectionDAGISel::FinishBasicBlock()"},
    {"_ZNK4llvm16SelectionDAGISel12CheckAndMaskENS_7SDValueEPNS_14ConstantSDNodeEx", "llvm::SelectionDAGISel::CheckAndMask(llvm::SDValue, llvm::ConstantSDNode*, long long) const"},
    {"_ZNK4llvm16SelectionDAGISel11CheckOrMaskENS_7SDValueEPNS_14ConstantSDNodeEx", "llvm::SelectionDAGISel::CheckOrMask(llvm::SDValue, llvm::ConstantSDNode*, long long) const"},
    {"_ZN4llvm16SelectionDAGISel29SelectInlineAsmMemoryOperandsERSt6vectorINS_7SDValueESaIS2_EE", "llvm::SelectionDAGISel::SelectInlineAsmMemoryOperands(std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&)"},
    {"_ZNK4llvm16SelectionDAGISel18IsProfitableToFoldENS_7SDValueEPNS_6SDNodeES3_", "llvm::SelectionDAGISel::IsProfitableToFold(llvm::SDValue, llvm::SDNode*, llvm::SDNode*) const"},
    {"_ZN4llvm16SelectionDAGISel13IsLegalToFoldENS_7SDValueEPNS_6SDNodeES3_NS_10CodeGenOpt5LevelEb", "llvm::SelectionDAGISel::IsLegalToFold(llvm::SDValue, llvm::SDNode*, llvm::SDNode*, llvm::CodeGenOpt::Level, bool)"},
    {"_Z13findNonImmUsePN4llvm6SDNodeES1_S1_S1_RNS_11SmallPtrSetIS1_Lj16EEEb", "findNonImmUse(llvm::SDNode*, llvm::SDNode*, llvm::SDNode*, llvm::SDNode*, llvm::SmallPtrSet<llvm::SDNode*, 16u>&, bool)"},
    {"_ZN4llvm16SelectionDAGISel16Select_INLINEASMEPNS_6SDNodeE", "llvm::SelectionDAGISel::Select_INLINEASM(llvm::SDNode*)"},
    {"_ZN4llvm16SelectionDAGISel12Select_UNDEFEPNS_6SDNodeE", "llvm::SelectionDAGISel::Select_UNDEF(llvm::SDNode*)"},
    {"_ZN4llvm16SelectionDAGISel19UpdateChainsAndGlueEPNS_6SDNodeENS_7SDValueERKNS_15SmallVectorImplIS2_EES3_S7_b", "llvm::SelectionDAGISel::UpdateChainsAndGlue(llvm::SDNode*, llvm::SDValue, llvm::SmallVectorImpl<llvm::SDNode*> const&, llvm::SDValue, llvm::SmallVectorImpl<llvm::SDNode*> const&, bool)"},
    {"_ZN4llvm16SelectionDAGISel9MorphNodeEPNS_6SDNodeEjNS_8SDVTListEPKNS_7SDValueEjj", "llvm::SelectionDAGISel::MorphNode(llvm::SDNode*, unsigned int, llvm::SDVTList, llvm::SDValue const*, unsigned int, unsigned int)"},
    {"_ZN4llvm16SelectionDAGISel16SelectCodeCommonEPNS_6SDNodeEPKhj", "llvm::SelectionDAGISel::SelectCodeCommon(llvm::SDNode*, unsigned char const*, unsigned int)"},
    {"_Z22HandleMergeInputChainsRN4llvm15SmallVectorImplIPNS_6SDNodeEEEPNS_12SelectionDAGE", "HandleMergeInputChains(llvm::SmallVectorImpl<llvm::SDNode*>&, llvm::SelectionDAG*)"},
    {"_ZN4llvm16SelectionDAGISel15CannotYetSelectEPNS_6SDNodeE", "llvm::SelectionDAGISel::CannotYetSelect(llvm::SDNode*)"},
    {"_Z14WalkChainUsersPN4llvm6SDNodeERNS_15SmallVectorImplIS1_EES4_", "WalkChainUsers(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDNode*>&, llvm::SmallVectorImpl<llvm::SDNode*>&)"},
    {"_ZN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEED1Ev", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::~opt()"},
    {"_ZN4llvm17RegisterSchedulerD1Ev", "llvm::RegisterScheduler::~RegisterScheduler()"},
    {"_ZN4llvm15SmallVectorImplIPNS_17MachineMemOperandEE6resizeEj", "llvm::SmallVectorImpl<llvm::MachineMemOperand*>::resize(unsigned int)"},
    {"_ZN4llvm16SelectionDAGISel28SelectInlineAsmMemoryOperandERKNS_7SDValueEcRSt6vectorIS1_SaIS1_EE", "llvm::SelectionDAGISel::SelectInlineAsmMemoryOperand(llvm::SDValue const&, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&)"},
    {"_ZNK4llvm16SelectionDAGISel21CheckPatternPredicateEj", "llvm::SelectionDAGISel::CheckPatternPredicate(unsigned int) const"},
    {"_ZNK4llvm16SelectionDAGISel18CheckNodePredicateEPNS_6SDNodeEj", "llvm::SelectionDAGISel::CheckNodePredicate(llvm::SDNode*, unsigned int) const"},
    {"_ZN4llvm16SelectionDAGISel19CheckComplexPatternEPNS_6SDNodeES2_NS_7SDValueEjRNS_15SmallVectorImplISt4pairIS3_S2_EEE", "llvm::SelectionDAGISel::CheckComplexPattern(llvm::SDNode*, llvm::SDNode*, llvm::SDValue, unsigned int, llvm::SmallVectorImpl<std::pair<llvm::SDValue, llvm::SDNode*> >&)"},
    {"_ZN4llvm16SelectionDAGISel14RunSDNodeXFormENS_7SDValueEj", "llvm::SelectionDAGISel::RunSDNodeXForm(llvm::SDValue, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplINS_7SDValueEEaSERKS2_", "llvm::SmallVectorImpl<llvm::SDValue>::operator=(llvm::SmallVectorImpl<llvm::SDValue> const&)"},
    {"_ZNSt6vectorIjSaIjEE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPjS1_EEmRKj", "std::vector<unsigned int, std::allocator<unsigned int> >::_M_fill_insert(__gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned long, unsigned int const&)"},
    {"_ZNSt6vectorIN4llvm7SDValueESaIS1_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS1_S3_EEEEvS8_T_S9_St20forward_iterator_tag", "void std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::SDValue*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > > >(__gnu_cxx::__normal_iterator<llvm::SDValue*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > >, __gnu_cxx::__normal_iterator<llvm::SDValue*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > >, __gnu_cxx::__normal_iterator<llvm::SDValue*, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> > >, std::forward_iterator_tag)"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder12BitTestBlockESaIS2_EE15_M_erase_at_endEPS2_", "std::vector<llvm::SelectionDAGBuilder::BitTestBlock, std::allocator<llvm::SelectionDAGBuilder::BitTestBlock> >::_M_erase_at_end(llvm::SelectionDAGBuilder::BitTestBlock*)"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEPNS_17MachineBasicBlockENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::BasicBlock const*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::InsertIntoBucket(llvm::BasicBlock const* const&, llvm::MachineBasicBlock* const&, std::pair<llvm::BasicBlock const*, llvm::MachineBasicBlock*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEPNS_17MachineBasicBlockENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock const*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::grow(unsigned int)"},
    {"_ZN4llvm10IndexedMapINS_20FunctionLoweringInfo11LiveOutInfoENS_20VirtReg2IndexFunctorEE6resizeEm", "llvm::IndexedMap<llvm::FunctionLoweringInfo::LiveOutInfo, llvm::VirtReg2IndexFunctor>::resize(unsigned long)"},
    {"_ZNSt6vectorIN4llvm20FunctionLoweringInfo11LiveOutInfoESaIS2_EE6resizeEmS2_", "std::vector<llvm::FunctionLoweringInfo::LiveOutInfo, std::allocator<llvm::FunctionLoweringInfo::LiveOutInfo> >::resize(unsigned long, llvm::FunctionLoweringInfo::LiveOutInfo)"},
    {"_ZNSt6vectorIN4llvm20FunctionLoweringInfo11LiveOutInfoESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::FunctionLoweringInfo::LiveOutInfo, std::allocator<llvm::FunctionLoweringInfo::LiveOutInfo> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::FunctionLoweringInfo::LiveOutInfo*, std::vector<llvm::FunctionLoweringInfo::LiveOutInfo, std::allocator<llvm::FunctionLoweringInfo::LiveOutInfo> > >, unsigned long, llvm::FunctionLoweringInfo::LiveOutInfo const&)"},
    {"_ZSt26__uninitialized_fill_n_auxIPN4llvm20FunctionLoweringInfo11LiveOutInfoEmS2_EvT_T0_RKT1_St12__false_type", "void std::__uninitialized_fill_n_aux<llvm::FunctionLoweringInfo::LiveOutInfo*, unsigned long, llvm::FunctionLoweringInfo::LiveOutInfo>(llvm::FunctionLoweringInfo::LiveOutInfo*, unsigned long, llvm::FunctionLoweringInfo::LiveOutInfo const&, std::__false_type)"},
    {"_ZNSt6__fillILb0EE4fillIPN4llvm20FunctionLoweringInfo11LiveOutInfoES4_EEvT_S6_RKT0_", "void std::__fill<false>::fill<llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo>(llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm20FunctionLoweringInfo11LiveOutInfoES6_EET0_T_S8_S7_", "llvm::FunctionLoweringInfo::LiveOutInfo* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo*>(llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo*, llvm::FunctionLoweringInfo::LiveOutInfo*)"},
    {"_ZN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE4doneEv", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::done()"},
    {"_ZN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE16handleOccurrenceEjNS_9StringRefESE_", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE14getOptionWidthEv", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE15printOptionInfoEm", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEED0Ev", "llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >::~opt()"},
    {"_ZN4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEE5parseERNS0_6OptionENS_9StringRefESD_RS9_", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::ScheduleDAGSDNodes* (*&)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level))"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm18RegisterPassParserINS_17RegisterSchedulerEED1Ev", "llvm::RegisterPassParser<llvm::RegisterScheduler>::~RegisterPassParser()"},
    {"_ZN4llvm18RegisterPassParserINS_17RegisterSchedulerEED0Ev", "llvm::RegisterPassParser<llvm::RegisterScheduler>::~RegisterPassParser()"},
    {"_ZN4llvm18RegisterPassParserINS_17RegisterSchedulerEE9NotifyAddEPKcPFPvvES4_", "llvm::RegisterPassParser<llvm::RegisterScheduler>::NotifyAdd(char const*, void* (*)(), char const*)"},
    {"_ZN4llvm18RegisterPassParserINS_17RegisterSchedulerEE12NotifyRemoveEPKc", "llvm::RegisterPassParser<llvm::RegisterScheduler>::NotifyRemove(char const*)"},
    {"_ZThn16_N4llvm18RegisterPassParserINS_17RegisterSchedulerEED1Ev", "non-virtual thunk to llvm::RegisterPassParser<llvm::RegisterScheduler>::~RegisterPassParser()"},
    {"_ZThn16_N4llvm18RegisterPassParserINS_17RegisterSchedulerEED0Ev", "non-virtual thunk to llvm::RegisterPassParser<llvm::RegisterScheduler>::~RegisterPassParser()"},
    {"_ZNK4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEE13getNumOptionsEv", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEE9getOptionEj", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEE14getDescriptionEj", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::getDescription(unsigned int) const"},
    {"_ZN4llvm27MachinePassRegistryListenerD1Ev", "llvm::MachinePassRegistryListener::~MachinePassRegistryListener()"},
    {"_ZN4llvm27MachinePassRegistryListenerD0Ev", "llvm::MachinePassRegistryListener::~MachinePassRegistryListener()"},
    {"_ZN4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEED1Ev", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::~parser()"},
    {"_ZN4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEED0Ev", "llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>::~parser()"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::Value const*, unsigned int, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Value const* const&, unsigned int const&, std::pair<llvm::Value const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::Value const*, unsigned int, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm20FunctionLoweringInfoD2Ev", "llvm::FunctionLoweringInfo::~FunctionLoweringInfo()"},
    {"_ZNSt6vectorIN4llvm20FunctionLoweringInfo11LiveOutInfoESaIS2_EED2Ev", "std::vector<llvm::FunctionLoweringInfo::LiveOutInfo, std::allocator<llvm::FunctionLoweringInfo::LiveOutInfo> >::~vector()"},
    {"_ZN4llvm19SelectionDAGBuilderD2Ev", "llvm::SelectionDAGBuilder::~SelectionDAGBuilder()"},
    {"_ZNSt6vectorISt4pairIN4llvm19SelectionDAGBuilder15JumpTableHeaderENS2_9JumpTableEESaIS5_EED2Ev", "std::vector<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable>, std::allocator<std::pair<llvm::SelectionDAGBuilder::JumpTableHeader, llvm::SelectionDAGBuilder::JumpTable> > >::~vector()"},
    {"_ZNSt6vectorIN4llvm19SelectionDAGBuilder12BitTestBlockESaIS2_EED2Ev", "std::vector<llvm::SelectionDAGBuilder::BitTestBlock, std::allocator<llvm::SelectionDAGBuilder::BitTestBlock> >::~vector()"},
    {"_ZN4llvm19SelectionDAGBuilderC2ERNS_12SelectionDAGERNS_20FunctionLoweringInfoENS_10CodeGenOpt5LevelE", "llvm::SelectionDAGBuilder::SelectionDAGBuilder(llvm::SelectionDAG&, llvm::FunctionLoweringInfo&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm14DOTGraphTraitsIPNS_12SelectionDAGEE12getNodeLabelEPKNS_6SDNodeEPKS1_", "llvm::DOTGraphTraits<llvm::SelectionDAG*>::getNodeLabel(llvm::SDNode const*, llvm::SelectionDAG const*)"},
    {"_ZN4llvm12SelectionDAG9viewGraphERKSs", "llvm::SelectionDAG::viewGraph(std::string const&)"},
    {"_ZN4llvm12SelectionDAG9viewGraphEv", "llvm::SelectionDAG::viewGraph()"},
    {"_ZN4llvm12SelectionDAG15clearGraphAttrsEv", "llvm::SelectionDAG::clearGraphAttrs()"},
    {"_ZN4llvm12SelectionDAG13setGraphAttrsEPKNS_6SDNodeEPKc", "llvm::SelectionDAG::setGraphAttrs(llvm::SDNode const*, char const*)"},
    {"_ZNK4llvm12SelectionDAG13getGraphAttrsEPKNS_6SDNodeE", "llvm::SelectionDAG::getGraphAttrs(llvm::SDNode const*) const"},
    {"_ZN4llvm12SelectionDAG13setGraphColorEPKNS_6SDNodeEPKc", "llvm::SelectionDAG::setGraphColor(llvm::SDNode const*, char const*)"},
    {"_ZN4llvm12SelectionDAG22setSubgraphColorHelperEPNS_6SDNodeEPKcRNS_8DenseSetIS2_NS_12DenseMapInfoIS2_EEEEiRb", "llvm::SelectionDAG::setSubgraphColorHelper(llvm::SDNode*, char const*, llvm::DenseSet<llvm::SDNode*, llvm::DenseMapInfo<llvm::SDNode*> >&, int, bool&)"},
    {"_ZN4llvm12SelectionDAG16setSubgraphColorEPNS_6SDNodeEPKc", "llvm::SelectionDAG::setSubgraphColor(llvm::SDNode*, char const*)"},
    {"_ZNK4llvm18ScheduleDAGSDNodes17getGraphNodeLabelEPKNS_5SUnitE", "llvm::ScheduleDAGSDNodes::getGraphNodeLabel(llvm::SUnit const*) const"},
    {"_ZNK4llvm18ScheduleDAGSDNodes22getCustomGraphFeaturesERNS_11GraphWriterIPNS_11ScheduleDAGEEE", "llvm::ScheduleDAGSDNodes::getCustomGraphFeatures(llvm::GraphWriter<llvm::ScheduleDAG*>&) const"},
    {"_ZN4llvm11GraphWriterIPNS_11ScheduleDAGEE14emitSimpleNodeEPKvRKSsS7_jPKSt6vectorISsSaISsEE", "llvm::GraphWriter<llvm::ScheduleDAG*>::emitSimpleNode(void const*, std::string const&, std::string const&, unsigned int, std::vector<std::string, std::allocator<std::string> > const*)"},
    {"_ZN4llvm11GraphWriterIPNS_11ScheduleDAGEE8emitEdgeEPKviS5_iRKSs", "llvm::GraphWriter<llvm::ScheduleDAG*>::emitEdge(void const*, int, void const*, int, std::string const&)"},
    {"_ZN4llvm11getTLSModelEPKNS_11GlobalValueENS_5Reloc5ModelE", "llvm::getTLSModel(llvm::GlobalValue const*, llvm::Reloc::Model)"},
    {"_ZN4llvm5RTLIB8getFPEXTENS_3EVTES1_", "llvm::RTLIB::getFPEXT(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm5RTLIB10getFPROUNDENS_3EVTES1_", "llvm::RTLIB::getFPROUND(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm5RTLIB11getFPTOSINTENS_3EVTES1_", "llvm::RTLIB::getFPTOSINT(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm5RTLIB11getFPTOUINTENS_3EVTES1_", "llvm::RTLIB::getFPTOUINT(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm5RTLIB11getSINTTOFPENS_3EVTES1_", "llvm::RTLIB::getSINTTOFP(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm5RTLIB11getUINTTOFPENS_3EVTES1_", "llvm::RTLIB::getUINTTOFP(llvm::EVT, llvm::EVT)"},
    {"_ZN4llvm14TargetLoweringC1ERKNS_13TargetMachineEPKNS_24TargetLoweringObjectFileE", "llvm::TargetLowering::TargetLowering(llvm::TargetMachine const&, llvm::TargetLoweringObjectFile const*)"},
    {"_ZN4llvm14TargetLoweringC2ERKNS_13TargetMachineEPKNS_24TargetLoweringObjectFileE", "llvm::TargetLowering::TargetLowering(llvm::TargetMachine const&, llvm::TargetLoweringObjectFile const*)"},
    {"_ZN4llvm14TargetLoweringD0Ev", "llvm::TargetLowering::~TargetLowering()"},
    {"_ZN4llvm14TargetLoweringD1Ev", "llvm::TargetLowering::~TargetLowering()"},
    {"_ZN4llvm14TargetLoweringD2Ev", "llvm::TargetLowering::~TargetLowering()"},
    {"_ZNK4llvm14TargetLowering9canOpTrapEjNS_3EVTE", "llvm::TargetLowering::canOpTrap(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering9isLegalRCEPKNS_19TargetRegisterClassE", "llvm::TargetLowering::isLegalRC(llvm::TargetRegisterClass const*) const"},
    {"_ZNK4llvm14TargetLowering26hasLegalSuperRegRegClassesEPKNS_19TargetRegisterClassE", "llvm::TargetLowering::hasLegalSuperRegRegClasses(llvm::TargetRegisterClass const*) const"},
    {"_ZNK4llvm14TargetLowering23findRepresentativeClassENS_3EVTE", "llvm::TargetLowering::findRepresentativeClass(llvm::EVT) const"},
    {"_ZN4llvm14TargetLowering25computeRegisterPropertiesEv", "llvm::TargetLowering::computeRegisterProperties()"},
    {"_ZNK4llvm14TargetLowering17getTargetNodeNameEj", "llvm::TargetLowering::getTargetNodeName(unsigned int) const"},
    {"_ZNK4llvm14TargetLowering18getSetCCResultTypeENS_3EVTE", "llvm::TargetLowering::getSetCCResultType(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering23getCmpLibcallReturnTypeEv", "llvm::TargetLowering::getCmpLibcallReturnType() const"},
    {"_ZNK4llvm14TargetLowering22getVectorTypeBreakdownERNS_11LLVMContextENS_3EVTERS3_RjS4_", "llvm::TargetLowering::getVectorTypeBreakdown(llvm::LLVMContext&, llvm::EVT, llvm::EVT&, unsigned int&, llvm::EVT&) const"},
    {"_ZN4llvm13GetReturnInfoEPKNS_4TypeEjRNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS_14TargetLoweringEPNS3_IyEE", "llvm::GetReturnInfo(llvm::Type const*, unsigned int, llvm::SmallVectorImpl<llvm::ISD::OutputArg>&, llvm::TargetLowering const&, llvm::SmallVectorImpl<unsigned long long>*)"},
    {"_ZNK4llvm14TargetLowering21getByValTypeAlignmentEPKNS_4TypeE", "llvm::TargetLowering::getByValTypeAlignment(llvm::Type const*) const"},
    {"_ZNK4llvm14TargetLowering20getJumpTableEncodingEv", "llvm::TargetLowering::getJumpTableEncoding() const"},
    {"_ZNK4llvm14TargetLowering24getPICJumpTableRelocBaseENS_7SDValueERNS_12SelectionDAGE", "llvm::TargetLowering::getPICJumpTableRelocBase(llvm::SDValue, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering28getPICJumpTableRelocBaseExprEPKNS_15MachineFunctionEjRNS_9MCContextE", "llvm::TargetLowering::getPICJumpTableRelocBaseExpr(llvm::MachineFunction const*, unsigned int, llvm::MCContext&) const"},
    {"_ZNK4llvm14TargetLowering20isOffsetFoldingLegalEPKNS_19GlobalAddressSDNodeE", "llvm::TargetLowering::isOffsetFoldingLegal(llvm::GlobalAddressSDNode const*) const"},
    {"_ZN4llvm14TargetLowering17TargetLoweringOpt22ShrinkDemandedConstantENS_7SDValueERKNS_5APIntE", "llvm::TargetLowering::TargetLoweringOpt::ShrinkDemandedConstant(llvm::SDValue, llvm::APInt const&)"},
    {"_ZN4llvm14TargetLowering17TargetLoweringOpt16ShrinkDemandedOpENS_7SDValueEjRKNS_5APIntENS_8DebugLocE", "llvm::TargetLowering::TargetLoweringOpt::ShrinkDemandedOp(llvm::SDValue, unsigned int, llvm::APInt const&, llvm::DebugLoc)"},
    {"_ZNK4llvm14TargetLowering20SimplifyDemandedBitsENS_7SDValueERKNS_5APIntERS2_S5_RNS0_17TargetLoweringOptEj", "llvm::TargetLowering::SimplifyDemandedBits(llvm::SDValue, llvm::APInt const&, llvm::APInt&, llvm::APInt&, llvm::TargetLowering::TargetLoweringOpt&, unsigned int) const"},
    {"_ZNK4llvm14TargetLowering30computeMaskedBitsForTargetNodeENS_7SDValueERKNS_5APIntERS2_S5_RKNS_12SelectionDAGEj", "llvm::TargetLowering::computeMaskedBitsForTargetNode(llvm::SDValue, llvm::APInt const&, llvm::APInt&, llvm::APInt&, llvm::SelectionDAG const&, unsigned int) const"},
    {"_ZNK4llvm14TargetLowering31ComputeNumSignBitsForTargetNodeENS_7SDValueEj", "llvm::TargetLowering::ComputeNumSignBitsForTargetNode(llvm::SDValue, unsigned int) const"},
    {"_ZNK4llvm14TargetLowering13SimplifySetCCENS_3EVTENS_7SDValueES2_NS_3ISD8CondCodeEbRNS0_15DAGCombinerInfoENS_8DebugLocE", "llvm::TargetLowering::SimplifySetCC(llvm::EVT, llvm::SDValue, llvm::SDValue, llvm::ISD::CondCode, bool, llvm::TargetLowering::DAGCombinerInfo&, llvm::DebugLoc) const"},
    {"_Z24ValueHasExactlyOneBitSetN4llvm7SDValueERKNS_12SelectionDAGE", "ValueHasExactlyOneBitSet(llvm::SDValue, llvm::SelectionDAG const&)"},
    {"_ZNK4llvm14TargetLowering14isGAPlusOffsetEPNS_6SDNodeERPKNS_11GlobalValueERx", "llvm::TargetLowering::isGAPlusOffset(llvm::SDNode*, llvm::GlobalValue const*&, long long&) const"},
    {"_ZNK4llvm14TargetLowering17PerformDAGCombineEPNS_6SDNodeERNS0_15DAGCombinerInfoE", "llvm::TargetLowering::PerformDAGCombine(llvm::SDNode*, llvm::TargetLowering::DAGCombinerInfo&) const"},
    {"_ZNK4llvm14TargetLowering17getConstraintTypeERKSs", "llvm::TargetLowering::getConstraintType(std::string const&) const"},
    {"_ZNK4llvm14TargetLowering16LowerXConstraintENS_3EVTE", "llvm::TargetLowering::LowerXConstraint(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering28LowerAsmOperandForConstraintENS_7SDValueEcRSt6vectorIS1_SaIS1_EERNS_12SelectionDAGE", "llvm::TargetLowering::LowerAsmOperandForConstraint(llvm::SDValue, char, std::vector<llvm::SDValue, std::allocator<llvm::SDValue> >&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering33getRegClassForInlineAsmConstraintERKSsNS_3EVTE", "llvm::TargetLowering::getRegClassForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering28getRegForInlineAsmConstraintERKSsNS_3EVTE", "llvm::TargetLowering::getRegForInlineAsmConstraint(std::string const&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering14AsmOperandInfo25isMatchingInputConstraintEv", "llvm::TargetLowering::AsmOperandInfo::isMatchingInputConstraint() const"},
    {"_ZNK4llvm14TargetLowering14AsmOperandInfo17getMatchedOperandEv", "llvm::TargetLowering::AsmOperandInfo::getMatchedOperand() const"},
    {"_ZNK4llvm14TargetLowering16ParseConstraintsENS_17ImmutableCallSiteE", "llvm::TargetLowering::ParseConstraints(llvm::ImmutableCallSite) const"},
    {"_ZNK4llvm14TargetLowering32getMultipleConstraintMatchWeightERNS0_14AsmOperandInfoEi", "llvm::TargetLowering::getMultipleConstraintMatchWeight(llvm::TargetLowering::AsmOperandInfo&, int) const"},
    {"_ZNK4llvm14TargetLowering30getSingleConstraintMatchWeightERNS0_14AsmOperandInfoEPKc", "llvm::TargetLowering::getSingleConstraintMatchWeight(llvm::TargetLowering::AsmOperandInfo&, char const*) const"},
    {"_ZNK4llvm14TargetLowering22ComputeConstraintToUseERNS0_14AsmOperandInfoENS_7SDValueEPNS_12SelectionDAGE", "llvm::TargetLowering::ComputeConstraintToUse(llvm::TargetLowering::AsmOperandInfo&, llvm::SDValue, llvm::SelectionDAG*) const"},
    {"_ZNK4llvm14TargetLowering21isLegalAddressingModeERKNS0_8AddrModeEPKNS_4TypeE", "llvm::TargetLowering::isLegalAddressingMode(llvm::TargetLowering::AddrMode const&, llvm::Type const*) const"},
    {"_ZNK4llvm14TargetLowering9BuildSDIVEPNS_6SDNodeERNS_12SelectionDAGEPSt6vectorIS2_SaIS2_EE", "llvm::TargetLowering::BuildSDIV(llvm::SDNode*, llvm::SelectionDAG&, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> >*) const"},
    {"_ZNK4llvm14TargetLowering9BuildUDIVEPNS_6SDNodeERNS_12SelectionDAGEPSt6vectorIS2_SaIS2_EE", "llvm::TargetLowering::BuildUDIV(llvm::SDNode*, llvm::SelectionDAG&, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> >*) const"},
    {"_ZNK4llvm14TargetLowering20getTypeToTransformToERNS_11LLVMContextENS_3EVTE", "llvm::TargetLowering::getTypeToTransformTo(llvm::LLVMContext&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering15getRegisterTypeERNS_11LLVMContextENS_3EVTE", "llvm::TargetLowering::getRegisterType(llvm::LLVMContext&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering15getNumRegistersERNS_11LLVMContextENS_3EVTE", "llvm::TargetLowering::getNumRegisters(llvm::LLVMContext&, llvm::EVT) const"},
    {"_ZNK4llvm5APInt10intersectsERKS0_", "llvm::APInt::intersects(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt6isIntNEj", "llvm::APInt::isIntN(unsigned int) const"},
    {"_ZNK4llvm3EVT6bitsLEES0_", "llvm::EVT::bitsLE(llvm::EVT) const"},
    {"_ZN4llvm5APInt17getSignedMaxValueEj", "llvm::APInt::getSignedMaxValue(unsigned int)"},
    {"_ZNK4llvm5APIntmiEy", "llvm::APInt::operator-(unsigned long long) const"},
    {"_ZNK4llvm14TargetLowering19getRegPressureLimitEPKNS_19TargetRegisterClassERNS_15MachineFunctionE", "llvm::TargetLowering::getRegPressureLimit(llvm::TargetRegisterClass const*, llvm::MachineFunction&) const"},
    {"_ZNK4llvm14TargetLowering12isFPImmLegalERKNS_7APFloatENS_3EVTE", "llvm::TargetLowering::isFPImmLegal(llvm::APFloat const&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering18isShuffleMaskLegalERKNS_15SmallVectorImplIiEENS_3EVTE", "llvm::TargetLowering::isShuffleMaskLegal(llvm::SmallVectorImpl<int> const&, llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering29allowsUnalignedMemoryAccessesENS_3EVTE", "llvm::TargetLowering::allowsUnalignedMemoryAccesses(llvm::EVT) const"},
    {"_ZNK4llvm14TargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EE", "llvm::TargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm14TargetLowering9LowerCallENS_7SDValueES1_NS_11CallingConv2IDEbRbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS5_IS1_EERKNS5_INS6_8InputArgEEENS_8DebugLocERNS_12SelectionDAGERSB_", "llvm::TargetLowering::LowerCall(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, bool&, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const"},
    {"_ZNK4llvm14TargetLowering11LowerReturnENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS4_IS1_EENS_8DebugLocERNS_12SelectionDAGE", "llvm::TargetLowering::LowerReturn(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::DebugLoc, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering18isUsedByReturnOnlyEPNS_6SDNodeE", "llvm::TargetLowering::isUsedByReturnOnly(llvm::SDNode*) const"},
    {"_ZNK4llvm14TargetLowering18ReplaceNodeResultsEPNS_6SDNodeERNS_15SmallVectorImplINS_7SDValueEEERNS_12SelectionDAGE", "llvm::TargetLowering::ReplaceNodeResults(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&, llvm::SelectionDAG&) const"},
    {"_ZNK4llvm14TargetLowering14createFastISelERNS_20FunctionLoweringInfoE", "llvm::TargetLowering::createFastISel(llvm::FunctionLoweringInfo&) const"},
    {"_ZNK4llvm14TargetLowering15ExpandInlineAsmEPNS_8CallInstE", "llvm::TargetLowering::ExpandInlineAsm(llvm::CallInst*) const"},
    {"_ZNSt6vectorIPN4llvm6SDNodeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SDNode**, std::vector<llvm::SDNode*, std::allocator<llvm::SDNode*> > >, llvm::SDNode* const&)"},
    {"_ZNSt6vectorIN4llvm14TargetLowering14AsmOperandInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::TargetLowering::AsmOperandInfo, std::allocator<llvm::TargetLowering::AsmOperandInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::TargetLowering::AsmOperandInfo*, std::vector<llvm::TargetLowering::AsmOperandInfo, std::allocator<llvm::TargetLowering::AsmOperandInfo> > >, llvm::TargetLowering::AsmOperandInfo const&)"},
    {"_ZSt13__destroy_auxIPN4llvm14TargetLowering14AsmOperandInfoEEvT_S4_St12__false_type", "void std::__destroy_aux<llvm::TargetLowering::AsmOperandInfo*>(llvm::TargetLowering::AsmOperandInfo*, llvm::TargetLowering::AsmOperandInfo*, std::__false_type)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm17SubConstraintInfoESaIS2_EEaSERKS4_", "std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> >::operator=(std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> > const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm14TargetLowering14AsmOperandInfoES6_EET0_T_S8_S7_", "llvm::TargetLowering::AsmOperandInfo* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::TargetLowering::AsmOperandInfo*, llvm::TargetLowering::AsmOperandInfo*>(llvm::TargetLowering::AsmOperandInfo*, llvm::TargetLowering::AsmOperandInfo*, llvm::TargetLowering::AsmOperandInfo*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_3ISD9OutputArgELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::ISD::OutputArg, false>::grow(unsigned long)"},
    {"_ZNK4llvm3EVT17getPow2VectorTypeERNS_11LLVMContextE", "llvm::EVT::getPow2VectorType(llvm::LLVMContext&) const"},
    {"_ZNK4llvm14TargetLowering19ValueTypeActionImpl21getExtendedTypeActionENS_3EVTE", "llvm::TargetLowering::ValueTypeActionImpl::getExtendedTypeAction(llvm::EVT) const"},
    {"_ZNSt8_Rb_treeISt4pairIjN4llvm3MVT15SimpleValueTypeEES0_IKS4_S3_ESt10_Select1stIS6_ESt4lessIS4_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::pair<unsigned int, llvm::MVT::SimpleValueType>, std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType>, std::_Select1st<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> >, std::less<std::pair<unsigned int, llvm::MVT::SimpleValueType> >, std::allocator<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<unsigned int, llvm::MVT::SimpleValueType> const, llvm::MVT::SimpleValueType> >*)"},
    {"_ZN4llvm22TargetSelectionDAGInfoC1ERKNS_13TargetMachineE", "llvm::TargetSelectionDAGInfo::TargetSelectionDAGInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm22TargetSelectionDAGInfoC2ERKNS_13TargetMachineE", "llvm::TargetSelectionDAGInfo::TargetSelectionDAGInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm22TargetSelectionDAGInfoD0Ev", "llvm::TargetSelectionDAGInfo::~TargetSelectionDAGInfo()"},
    {"_ZN4llvm22TargetSelectionDAGInfoD1Ev", "llvm::TargetSelectionDAGInfo::~TargetSelectionDAGInfo()"},
    {"_ZN4llvm22TargetSelectionDAGInfoD2Ev", "llvm::TargetSelectionDAGInfo::~TargetSelectionDAGInfo()"},
    {"_ZNK4llvm22TargetSelectionDAGInfo23EmitTargetCodeForMemcpyERNS_12SelectionDAGENS_8DebugLocENS_7SDValueES4_S4_S4_jbbNS_18MachinePointerInfoES5_", "llvm::TargetSelectionDAGInfo::EmitTargetCodeForMemcpy(llvm::SelectionDAG&, llvm::DebugLoc, llvm::SDValue, llvm::SDValue, llvm::SDValue, llvm::SDValue, unsigned int, bool, bool, llvm::MachinePointerInfo, llvm::MachinePointerInfo) const"},
    {"_ZN4llvm10AsmPrinterC1ERNS_13TargetMachineERNS_10MCStreamerE", "llvm::AsmPrinter::AsmPrinter(llvm::TargetMachine&, llvm::MCStreamer&)"},
    {"_ZN4llvm10AsmPrinterC2ERNS_13TargetMachineERNS_10MCStreamerE", "llvm::AsmPrinter::AsmPrinter(llvm::TargetMachine&, llvm::MCStreamer&)"},
    {"_ZN4llvm10AsmPrinterD0Ev", "llvm::AsmPrinter::~AsmPrinter()"},
    {"_ZN4llvm10AsmPrinterD1Ev", "llvm::AsmPrinter::~AsmPrinter()"},
    {"_ZN4llvm10AsmPrinterD2Ev", "llvm::AsmPrinter::~AsmPrinter()"},
    {"_ZNK4llvm10AsmPrinter17getFunctionNumberEv", "llvm::AsmPrinter::getFunctionNumber() const"},
    {"_ZNK4llvm10AsmPrinter18getObjFileLoweringEv", "llvm::AsmPrinter::getObjFileLowering() const"},
    {"_ZNK4llvm10AsmPrinter13getDataLayoutEv", "llvm::AsmPrinter::getDataLayout() const"},
    {"_ZNK4llvm10AsmPrinter17getCurrentSectionEv", "llvm::AsmPrinter::getCurrentSection() const"},
    {"_ZNK4llvm10AsmPrinter16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::AsmPrinter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm10AsmPrinter16doInitializationERNS_6ModuleE", "llvm::AsmPrinter::doInitialization(llvm::Module&)"},
    {"_ZN4llvm10AsmPrinter20GetOrCreateGCPrinterEPNS_10GCStrategyE", "llvm::AsmPrinter::GetOrCreateGCPrinter(llvm::GCStrategy*)"},
    {"_ZNK4llvm10AsmPrinter11EmitLinkageEjPNS_8MCSymbolE", "llvm::AsmPrinter::EmitLinkage(unsigned int, llvm::MCSymbol*) const"},
    {"_ZN4llvm10AsmPrinter18EmitGlobalVariableEPKNS_14GlobalVariableE", "llvm::AsmPrinter::EmitGlobalVariable(llvm::GlobalVariable const*)"},
    {"_ZN4llvm10AsmPrinter21EmitSpecialLLVMGlobalEPKNS_14GlobalVariableE", "llvm::AsmPrinter::EmitSpecialLLVMGlobal(llvm::GlobalVariable const*)"},
    {"_ZNK4llvm10AsmPrinter14EmitVisibilityEPNS_8MCSymbolEj", "llvm::AsmPrinter::EmitVisibility(llvm::MCSymbol*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter13EmitAlignmentEjPKNS_11GlobalValueE", "llvm::AsmPrinter::EmitAlignment(unsigned int, llvm::GlobalValue const*) const"},
    {"_ZN4llvm10AsmPrinter18EmitGlobalConstantEPKNS_8ConstantEj", "llvm::AsmPrinter::EmitGlobalConstant(llvm::Constant const*, unsigned int)"},
    {"_ZNK4llvm10AsmPrinter23GetExternalSymbolSymbolENS_9StringRefE", "llvm::AsmPrinter::GetExternalSymbolSymbol(llvm::StringRef) const"},
    {"_ZN4llvm10AsmPrinter18EmitFunctionHeaderEv", "llvm::AsmPrinter::EmitFunctionHeader()"},
    {"_ZN4llvm10AsmPrinter22EmitFunctionEntryLabelEv", "llvm::AsmPrinter::EmitFunctionEntryLabel()"},
    {"_ZN4llvm10AsmPrinter16EmitFunctionBodyEv", "llvm::AsmPrinter::EmitFunctionBody()"},
    {"_ZNK4llvm10AsmPrinter19EmitBasicBlockStartEPKNS_17MachineBasicBlockE", "llvm::AsmPrinter::EmitBasicBlockStart(llvm::MachineBasicBlock const*) const"},
    {"_ZN4llvm10AsmPrinter17EmitJumpTableInfoEv", "llvm::AsmPrinter::EmitJumpTableInfo()"},
    {"_ZNK4llvm10AsmPrinter21getDebugValueLocationEPKNS_12MachineInstrE", "llvm::AsmPrinter::getDebugValueLocation(llvm::MachineInstr const*) const"},
    {"_ZN4llvm10AsmPrinter14doFinalizationERNS_6ModuleE", "llvm::AsmPrinter::doFinalization(llvm::Module&)"},
    {"_ZN4llvm10AsmPrinter20SetupMachineFunctionERNS_15MachineFunctionE", "llvm::AsmPrinter::SetupMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm10AsmPrinter16EmitConstantPoolEv", "llvm::AsmPrinter::EmitConstantPool()"},
    {"_ZNK4llvm10AsmPrinter12GetCPISymbolEj", "llvm::AsmPrinter::GetCPISymbol(unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter14GetJTSetSymbolEjj", "llvm::AsmPrinter::GetJTSetSymbol(unsigned int, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter12GetJTISymbolEjb", "llvm::AsmPrinter::GetJTISymbol(unsigned int, bool) const"},
    {"_ZNK4llvm10AsmPrinter18EmitJumpTableEntryEPKNS_20MachineJumpTableInfoEPKNS_17MachineBasicBlockEj", "llvm::AsmPrinter::EmitJumpTableEntry(llvm::MachineJumpTableInfo const*, llvm::MachineBasicBlock const*, unsigned int) const"},
    {"_ZN4llvm10AsmPrinter16EmitLLVMUsedListEPNS_8ConstantE", "llvm::AsmPrinter::EmitLLVMUsedList(llvm::Constant*)"},
    {"_ZN4llvm10AsmPrinter18EmitXXStructorListEPNS_8ConstantE", "llvm::AsmPrinter::EmitXXStructorList(llvm::Constant*)"},
    {"_ZNK4llvm10AsmPrinter8EmitInt8Ei", "llvm::AsmPrinter::EmitInt8(int) const"},
    {"_ZNK4llvm10AsmPrinter9EmitInt16Ei", "llvm::AsmPrinter::EmitInt16(int) const"},
    {"_ZNK4llvm10AsmPrinter9EmitInt32Ei", "llvm::AsmPrinter::EmitInt32(int) const"},
    {"_ZNK4llvm10AsmPrinter19EmitLabelDifferenceEPKNS_8MCSymbolES3_j", "llvm::AsmPrinter::EmitLabelDifference(llvm::MCSymbol const*, llvm::MCSymbol const*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter13GetTempSymbolENS_9StringRefEj", "llvm::AsmPrinter::GetTempSymbol(llvm::StringRef, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter25EmitLabelOffsetDifferenceEPKNS_8MCSymbolEyS3_j", "llvm::AsmPrinter::EmitLabelOffsetDifference(llvm::MCSymbol const*, unsigned long long, llvm::MCSymbol const*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter19EmitLabelPlusOffsetEPKNS_8MCSymbolEyj", "llvm::AsmPrinter::EmitLabelPlusOffset(llvm::MCSymbol const*, unsigned long long, unsigned int) const"},
    {"_Z22EmitGlobalConstantImplPKN4llvm8ConstantEjRNS_10AsmPrinterE", "EmitGlobalConstantImpl(llvm::Constant const*, unsigned int, llvm::AsmPrinter&)"},
    {"_ZN4llvm10AsmPrinter28EmitMachineConstantPoolValueEPNS_24MachineConstantPoolValueE", "llvm::AsmPrinter::EmitMachineConstantPoolValue(llvm::MachineConstantPoolValue*)"},
    {"_ZNK4llvm10AsmPrinter11printOffsetExRNS_11raw_ostreamE", "llvm::AsmPrinter::printOffset(long long, llvm::raw_ostream&) const"},
    {"_ZNK4llvm10AsmPrinter13GetTempSymbolENS_9StringRefE", "llvm::AsmPrinter::GetTempSymbol(llvm::StringRef) const"},
    {"_ZNK4llvm10AsmPrinter21GetBlockAddressSymbolEPKNS_12BlockAddressE", "llvm::AsmPrinter::GetBlockAddressSymbol(llvm::BlockAddress const*) const"},
    {"_ZNK4llvm10AsmPrinter21GetBlockAddressSymbolEPKNS_10BasicBlockE", "llvm::AsmPrinter::GetBlockAddressSymbol(llvm::BasicBlock const*) const"},
    {"_ZNK4llvm10AsmPrinter28GetSymbolWithGlobalValueBaseEPKNS_11GlobalValueENS_9StringRefEb", "llvm::AsmPrinter::GetSymbolWithGlobalValueBase(llvm::GlobalValue const*, llvm::StringRef, bool) const"},
    {"_Z26EmitBasicBlockLoopCommentsRKN4llvm17MachineBasicBlockEPKNS_15MachineLoopInfoERKNS_10AsmPrinterE", "EmitBasicBlockLoopComments(llvm::MachineBasicBlock const&, llvm::MachineLoopInfo const*, llvm::AsmPrinter const&)"},
    {"_ZNK4llvm10AsmPrinter33isBlockOnlyReachableByFallthroughEPKNS_17MachineBasicBlockE", "llvm::AsmPrinter::isBlockOnlyReachableByFallthrough(llvm::MachineBasicBlock const*) const"},
    {"_Z22PrintParentLoopCommentRN4llvm11raw_ostreamEPKNS_11MachineLoopEj", "PrintParentLoopComment(llvm::raw_ostream&, llvm::MachineLoop const*, unsigned int)"},
    {"_Z21PrintChildLoopCommentRN4llvm11raw_ostreamEPKNS_11MachineLoopEj", "PrintChildLoopComment(llvm::raw_ostream&, llvm::MachineLoop const*, unsigned int)"},
    {"_Z13LowerConstantPKN4llvm8ConstantERNS_10AsmPrinterE", "LowerConstant(llvm::Constant const*, llvm::AsmPrinter&)"},
    {"_Z12EmitDebugLocN4llvm8DebugLocEPKNS_15MachineFunctionERNS_11raw_ostreamE", "EmitDebugLoc(llvm::DebugLoc, llvm::MachineFunction const*, llvm::raw_ostream&)"},
    {"_ZN4llvm8DenseMapIPNS_10GCStrategyEPNS_17GCMetadataPrinterENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::GCStrategy*, llvm::GCMetadataPrinter*, llvm::DenseMapInfo<llvm::GCStrategy*>, llvm::DenseMapInfo<llvm::GCMetadataPrinter*> >::insert(std::pair<llvm::GCStrategy*, llvm::GCMetadataPrinter*> const&)"},
    {"_ZN4llvm10AsmPrinter20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::AsmPrinter::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm10AsmPrinter18EmitStartOfAsmFileERNS_6ModuleE", "llvm::AsmPrinter::EmitStartOfAsmFile(llvm::Module&)"},
    {"_ZN4llvm10AsmPrinter16EmitEndOfAsmFileERNS_6ModuleE", "llvm::AsmPrinter::EmitEndOfAsmFile(llvm::Module&)"},
    {"_ZN4llvm10AsmPrinter15EmitInstructionEPKNS_12MachineInstrE", "llvm::AsmPrinter::EmitInstruction(llvm::MachineInstr const*)"},
    {"_ZN4llvm8DenseMapIPNS_10GCStrategyEPNS_17GCMetadataPrinterENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::GCStrategy*, llvm::GCMetadataPrinter*, llvm::DenseMapInfo<llvm::GCStrategy*>, llvm::DenseMapInfo<llvm::GCMetadataPrinter*> >::InsertIntoBucket(llvm::GCStrategy* const&, llvm::GCMetadataPrinter* const&, std::pair<llvm::GCStrategy*, llvm::GCMetadataPrinter*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10GCStrategyEPNS_17GCMetadataPrinterENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::GCStrategy*, llvm::GCMetadataPrinter*, llvm::DenseMapInfo<llvm::GCStrategy*>, llvm::DenseMapInfo<llvm::GCMetadataPrinter*> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIjEaSERKS1_", "llvm::SmallVectorImpl<unsigned int>::operator=(llvm::SmallVectorImpl<unsigned int> const&)"},
    {"_ZN4llvm14format_object1IyED1Ev", "llvm::format_object1<unsigned long long>::~format_object1()"},
    {"_ZNK4llvm14format_object1IyE7snprintEPcj", "llvm::format_object1<unsigned long long>::snprint(char*, unsigned int) const"},
    {"_ZN4llvm14format_object1IyED0Ev", "llvm::format_object1<unsigned long long>::~format_object1()"},
    {"_ZNK4llvm10AsmPrinter11EmitSLEB128EiPKc", "llvm::AsmPrinter::EmitSLEB128(int, char const*) const"},
    {"_ZNK4llvm10AsmPrinter11EmitULEB128EjPKcj", "llvm::AsmPrinter::EmitULEB128(unsigned int, char const*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter11EmitCFAByteEj", "llvm::AsmPrinter::EmitCFAByte(unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter16EmitEncodingByteEjPKc", "llvm::AsmPrinter::EmitEncodingByte(unsigned int, char const*) const"},
    {"_ZNK4llvm10AsmPrinter21GetSizeOfEncodedValueEj", "llvm::AsmPrinter::GetSizeOfEncodedValue(unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter13EmitReferenceEPKNS_8MCSymbolEj", "llvm::AsmPrinter::EmitReference(llvm::MCSymbol const*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter13EmitReferenceEPKNS_11GlobalValueEj", "llvm::AsmPrinter::EmitReference(llvm::GlobalValue const*, unsigned int) const"},
    {"_ZNK4llvm10AsmPrinter17EmitSectionOffsetEPKNS_8MCSymbolES3_", "llvm::AsmPrinter::EmitSectionOffset(llvm::MCSymbol const*, llvm::MCSymbol const*) const"},
    {"_ZNK4llvm10AsmPrinter14EmitFrameMovesERKSt6vectorINS_11MachineMoveESaIS2_EEPNS_8MCSymbolEb", "llvm::AsmPrinter::EmitFrameMoves(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> > const&, llvm::MCSymbol*, bool) const"},
    {"_ZNK4llvm10AsmPrinter17EmitCFIFrameMovesERKSt6vectorINS_11MachineMoveESaIS2_EE", "llvm::AsmPrinter::EmitCFIFrameMoves(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> > const&) const"},
    {"_ZNK4llvm10AsmPrinter13EmitInlineAsmENS_9StringRefEPKNS_6MDNodeE", "llvm::AsmPrinter::EmitInlineAsm(llvm::StringRef, llvm::MDNode const*) const"},
    {"_Z17SrcMgrDiagHandlerRKN4llvm12SMDiagnosticEPv", "SrcMgrDiagHandler(llvm::SMDiagnostic const&, void*)"},
    {"_ZNK4llvm10AsmPrinter13EmitInlineAsmEPKNS_12MachineInstrE", "llvm::AsmPrinter::EmitInlineAsm(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm10AsmPrinter12PrintSpecialEPKNS_12MachineInstrERNS_11raw_ostreamEPKc", "llvm::AsmPrinter::PrintSpecial(llvm::MachineInstr const*, llvm::raw_ostream&, char const*) const"},
    {"_ZN4llvm10AsmPrinter15PrintAsmOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::AsmPrinter::PrintAsmOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZN4llvm10AsmPrinter21PrintAsmMemoryOperandEPKNS_12MachineInstrEjjPKcRNS_11raw_ostreamE", "llvm::AsmPrinter::PrintAsmMemoryOperand(llvm::MachineInstr const*, unsigned int, unsigned int, char const*, llvm::raw_ostream&)"},
    {"_ZNK4llvm13DIEAbbrevData7ProfileERNS_16FoldingSetNodeIDE", "llvm::DIEAbbrevData::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm9DIEAbbrev7ProfileERNS_16FoldingSetNodeIDE", "llvm::DIEAbbrev::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm9DIEAbbrev4EmitEPNS_10AsmPrinterE", "llvm::DIEAbbrev::Emit(llvm::AsmPrinter*) const"},
    {"_ZN4llvm3DIED0Ev", "llvm::DIE::~DIE()"},
    {"_ZN4llvm3DIED1Ev", "llvm::DIE::~DIE()"},
    {"_ZN4llvm3DIED2Ev", "llvm::DIE::~DIE()"},
    {"_ZN4llvm3DIE16addSiblingOffsetERNS_16BumpPtrAllocatorE", "llvm::DIE::addSiblingOffset(llvm::BumpPtrAllocator&)"},
    {"_ZNK4llvm10DIEInteger9EmitValueEPNS_10AsmPrinterEj", "llvm::DIEInteger::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm10DIEInteger6SizeOfEPNS_10AsmPrinterEj", "llvm::DIEInteger::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm9DIEString9EmitValueEPNS_10AsmPrinterEj", "llvm::DIEString::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIELabel9EmitValueEPNS_10AsmPrinterEj", "llvm::DIELabel::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIELabel6SizeOfEPNS_10AsmPrinterEj", "llvm::DIELabel::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIEDelta9EmitValueEPNS_10AsmPrinterEj", "llvm::DIEDelta::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIEDelta6SizeOfEPNS_10AsmPrinterEj", "llvm::DIEDelta::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIEEntry9EmitValueEPNS_10AsmPrinterEj", "llvm::DIEEntry::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZN4llvm8DIEBlock11ComputeSizeEPNS_10AsmPrinterE", "llvm::DIEBlock::ComputeSize(llvm::AsmPrinter*)"},
    {"_ZNK4llvm8DIEBlock9EmitValueEPNS_10AsmPrinterEj", "llvm::DIEBlock::EmitValue(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNK4llvm8DIEBlock6SizeOfEPNS_10AsmPrinterEj", "llvm::DIEBlock::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZN4llvm15SmallVectorImplIPNS_8DIEValueEE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<llvm::DIEValue*>::insert(llvm::DIEValue**, llvm::DIEValue* const&)"},
    {"_ZN4llvm10DIEIntegerD1Ev", "llvm::DIEInteger::~DIEInteger()"},
    {"_ZN4llvm10DIEIntegerD0Ev", "llvm::DIEInteger::~DIEInteger()"},
    {"_ZN4llvm9DIEStringD1Ev", "llvm::DIEString::~DIEString()"},
    {"_ZN4llvm9DIEStringD0Ev", "llvm::DIEString::~DIEString()"},
    {"_ZN4llvm8DIELabelD1Ev", "llvm::DIELabel::~DIELabel()"},
    {"_ZN4llvm8DIELabelD0Ev", "llvm::DIELabel::~DIELabel()"},
    {"_ZN4llvm8DIEDeltaD1Ev", "llvm::DIEDelta::~DIEDelta()"},
    {"_ZN4llvm8DIEDeltaD0Ev", "llvm::DIEDelta::~DIEDelta()"},
    {"_ZN4llvm8DIEEntryD1Ev", "llvm::DIEEntry::~DIEEntry()"},
    {"_ZN4llvm8DIEEntryD0Ev", "llvm::DIEEntry::~DIEEntry()"},
    {"_ZN4llvm15SmallVectorImplINS_13DIEAbbrevDataEE6insertEPS1_RKS1_", "llvm::SmallVectorImpl<llvm::DIEAbbrevData>::insert(llvm::DIEAbbrevData*, llvm::DIEAbbrevData const&)"},
    {"_ZN4llvm17DwarfCFIExceptionC1EPNS_10AsmPrinterE", "llvm::DwarfCFIException::DwarfCFIException(llvm::AsmPrinter*)"},
    {"_ZN4llvm17DwarfCFIExceptionC2EPNS_10AsmPrinterE", "llvm::DwarfCFIException::DwarfCFIException(llvm::AsmPrinter*)"},
    {"_ZN4llvm17DwarfCFIExceptionD0Ev", "llvm::DwarfCFIException::~DwarfCFIException()"},
    {"_ZN4llvm17DwarfCFIExceptionD1Ev", "llvm::DwarfCFIException::~DwarfCFIException()"},
    {"_ZN4llvm17DwarfCFIExceptionD2Ev", "llvm::DwarfCFIException::~DwarfCFIException()"},
    {"_ZN4llvm17DwarfCFIException9EndModuleEv", "llvm::DwarfCFIException::EndModule()"},
    {"_ZN4llvm17DwarfCFIException13BeginFunctionEPKNS_15MachineFunctionE", "llvm::DwarfCFIException::BeginFunction(llvm::MachineFunction const*)"},
    {"_ZN4llvm17DwarfCFIException11EndFunctionEv", "llvm::DwarfCFIException::EndFunction()"},
    {"_ZN4llvm8DbgScopeD0Ev", "llvm::DbgScope::~DbgScope()"},
    {"_ZN4llvm8DbgScopeD1Ev", "llvm::DbgScope::~DbgScope()"},
    {"_ZN4llvm8DbgScopeD2Ev", "llvm::DbgScope::~DbgScope()"},
    {"_ZN4llvm10DwarfDebugC1EPNS_10AsmPrinterEPNS_6ModuleE", "llvm::DwarfDebug::DwarfDebug(llvm::AsmPrinter*, llvm::Module*)"},
    {"_ZN4llvm10DwarfDebugC2EPNS_10AsmPrinterEPNS_6ModuleE", "llvm::DwarfDebug::DwarfDebug(llvm::AsmPrinter*, llvm::Module*)"},
    {"_ZN4llvm10DwarfDebug11beginModuleEPNS_6ModuleE", "llvm::DwarfDebug::beginModule(llvm::Module*)"},
    {"_ZN4llvm10DwarfDebugD1Ev", "llvm::DwarfDebug::~DwarfDebug()"},
    {"_ZN4llvm10DwarfDebugD2Ev", "llvm::DwarfDebug::~DwarfDebug()"},
    {"_ZN4llvm10DwarfDebug18getStringPoolEntryENS_9StringRefE", "llvm::DwarfDebug::getStringPoolEntry(llvm::StringRef)"},
    {"_ZN4llvm10DwarfDebug18assignAbbrevNumberERNS_9DIEAbbrevE", "llvm::DwarfDebug::assignAbbrevNumber(llvm::DIEAbbrev&)"},
    {"_ZN4llvm10DwarfDebug14createDIEEntryEPNS_3DIEE", "llvm::DwarfDebug::createDIEEntry(llvm::DIE*)"},
    {"_ZN4llvm10DwarfDebug7addUIntEPNS_3DIEEjjy", "llvm::DwarfDebug::addUInt(llvm::DIE*, unsigned int, unsigned int, unsigned long long)"},
    {"_ZN4llvm10DwarfDebug7addSIntEPNS_3DIEEjjx", "llvm::DwarfDebug::addSInt(llvm::DIE*, unsigned int, unsigned int, long long)"},
    {"_ZN4llvm10DwarfDebug9addStringEPNS_3DIEEjjNS_9StringRefE", "llvm::DwarfDebug::addString(llvm::DIE*, unsigned int, unsigned int, llvm::StringRef)"},
    {"_ZN4llvm10DwarfDebug8addLabelEPNS_3DIEEjjPKNS_8MCSymbolE", "llvm::DwarfDebug::addLabel(llvm::DIE*, unsigned int, unsigned int, llvm::MCSymbol const*)"},
    {"_ZN4llvm10DwarfDebug8addDeltaEPNS_3DIEEjjPKNS_8MCSymbolES5_", "llvm::DwarfDebug::addDelta(llvm::DIE*, unsigned int, unsigned int, llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN4llvm10DwarfDebug11addDIEEntryEPNS_3DIEEjjS2_", "llvm::DwarfDebug::addDIEEntry(llvm::DIE*, unsigned int, unsigned int, llvm::DIE*)"},
    {"_ZN4llvm10DwarfDebug8addBlockEPNS_3DIEEjjPNS_8DIEBlockE", "llvm::DwarfDebug::addBlock(llvm::DIE*, unsigned int, unsigned int, llvm::DIEBlock*)"},
    {"_ZN4llvm10DwarfDebug13addSourceLineEPNS_3DIEENS_10DIVariableE", "llvm::DwarfDebug::addSourceLine(llvm::DIE*, llvm::DIVariable)"},
    {"_ZN4llvm10DwarfDebug19GetOrCreateSourceIDENS_9StringRefE", "llvm::DwarfDebug::GetOrCreateSourceID(llvm::StringRef)"},
    {"_ZN4llvm10DwarfDebug13addSourceLineEPNS_3DIEENS_16DIGlobalVariableE", "llvm::DwarfDebug::addSourceLine(llvm::DIE*, llvm::DIGlobalVariable)"},
    {"_ZN4llvm10DwarfDebug13addSourceLineEPNS_3DIEENS_12DISubprogramE", "llvm::DwarfDebug::addSourceLine(llvm::DIE*, llvm::DISubprogram)"},
    {"_ZN4llvm10DwarfDebug13addSourceLineEPNS_3DIEENS_6DITypeE", "llvm::DwarfDebug::addSourceLine(llvm::DIE*, llvm::DIType)"},
    {"_ZN4llvm10DwarfDebug13addSourceLineEPNS_3DIEENS_11DINameSpaceE", "llvm::DwarfDebug::addSourceLine(llvm::DIE*, llvm::DINameSpace)"},
    {"_ZN4llvm10DwarfDebug18addVariableAddressERPNS_11DbgVariableEPNS_3DIEEx", "llvm::DwarfDebug::addVariableAddress(llvm::DbgVariable*&, llvm::DIE*, long long)"},
    {"_ZN4llvm10DwarfDebug17addComplexAddressERPNS_11DbgVariableEPNS_3DIEEjRKNS_15MachineLocationE", "llvm::DwarfDebug::addComplexAddress(llvm::DbgVariable*&, llvm::DIE*, unsigned int, llvm::MachineLocation const&)"},
    {"_ZN4llvm10DwarfDebug20addBlockByrefAddressERPNS_11DbgVariableEPNS_3DIEEjRKNS_15MachineLocationE", "llvm::DwarfDebug::addBlockByrefAddress(llvm::DbgVariable*&, llvm::DIE*, unsigned int, llvm::MachineLocation const&)"},
    {"_ZN4llvm10DwarfDebug10addAddressEPNS_3DIEEjRKNS_15MachineLocationE", "llvm::DwarfDebug::addAddress(llvm::DIE*, unsigned int, llvm::MachineLocation const&)"},
    {"_ZN4llvm10DwarfDebug18addRegisterAddressEPNS_3DIEERKNS_14MachineOperandE", "llvm::DwarfDebug::addRegisterAddress(llvm::DIE*, llvm::MachineOperand const&)"},
    {"_ZN4llvm10DwarfDebug16addConstantValueEPNS_3DIEERKNS_14MachineOperandE", "llvm::DwarfDebug::addConstantValue(llvm::DIE*, llvm::MachineOperand const&)"},
    {"_ZN4llvm10DwarfDebug18addConstantFPValueEPNS_3DIEERKNS_14MachineOperandE", "llvm::DwarfDebug::addConstantFPValue(llvm::DIE*, llvm::MachineOperand const&)"},
    {"_ZN4llvm10DwarfDebug16addConstantValueEPNS_3DIEEPNS_11ConstantIntEb", "llvm::DwarfDebug::addConstantValue(llvm::DIE*, llvm::ConstantInt*, bool)"},
    {"_ZN4llvm10DwarfDebug17addToContextOwnerEPNS_3DIEENS_12DIDescriptorE", "llvm::DwarfDebug::addToContextOwner(llvm::DIE*, llvm::DIDescriptor)"},
    {"_ZN4llvm10DwarfDebug18getOrCreateTypeDIEENS_6DITypeE", "llvm::DwarfDebug::getOrCreateTypeDIE(llvm::DIType)"},
    {"_ZN4llvm10DwarfDebug20getOrCreateNameSpaceENS_11DINameSpaceE", "llvm::DwarfDebug::getOrCreateNameSpace(llvm::DINameSpace)"},
    {"_ZN4llvm10DwarfDebug19createSubprogramDIEENS_12DISubprogramE", "llvm::DwarfDebug::createSubprogramDIE(llvm::DISubprogram)"},
    {"_ZNK4llvm10DwarfDebug14getCompileUnitEPKNS_6MDNodeE", "llvm::DwarfDebug::getCompileUnit(llvm::MDNode const*) const"},
    {"_ZN4llvm10DwarfDebug16constructTypeDIEERNS_3DIEENS_11DIBasicTypeE", "llvm::DwarfDebug::constructTypeDIE(llvm::DIE&, llvm::DIBasicType)"},
    {"_ZN4llvm10DwarfDebug16constructTypeDIEERNS_3DIEENS_15DICompositeTypeE", "llvm::DwarfDebug::constructTypeDIE(llvm::DIE&, llvm::DICompositeType)"},
    {"_ZN4llvm10DwarfDebug16constructTypeDIEERNS_3DIEENS_13DIDerivedTypeE", "llvm::DwarfDebug::constructTypeDIE(llvm::DIE&, llvm::DIDerivedType)"},
    {"_ZN4llvm10DwarfDebug7addTypeEPNS_3DIEENS_6DITypeE", "llvm::DwarfDebug::addType(llvm::DIE*, llvm::DIType)"},
    {"_ZN4llvm10DwarfDebug21constructArrayTypeDIEERNS_3DIEEPNS_15DICompositeTypeE", "llvm::DwarfDebug::constructArrayTypeDIE(llvm::DIE&, llvm::DICompositeType*)"},
    {"_ZN4llvm10DwarfDebug20constructEnumTypeDIEENS_12DIEnumeratorE", "llvm::DwarfDebug::constructEnumTypeDIE(llvm::DIEnumerator)"},
    {"_ZN4llvm10DwarfDebug15createMemberDIEENS_13DIDerivedTypeE", "llvm::DwarfDebug::createMemberDIE(llvm::DIDerivedType)"},
    {"_ZN4llvm10DwarfDebug35getOrCreateTemplateTypeParameterDIEENS_23DITemplateTypeParameterE", "llvm::DwarfDebug::getOrCreateTemplateTypeParameterDIE(llvm::DITemplateTypeParameter)"},
    {"_ZN4llvm10DwarfDebug36getOrCreateTemplateValueParameterDIEENS_24DITemplateValueParameterE", "llvm::DwarfDebug::getOrCreateTemplateValueParameterDIE(llvm::DITemplateValueParameter)"},
    {"_ZN4llvm10DwarfDebug20constructSubrangeDIEERNS_3DIEENS_10DISubrangeEPS1_", "llvm::DwarfDebug::constructSubrangeDIE(llvm::DIE&, llvm::DISubrange, llvm::DIE*)"},
    {"_ZN4llvm10DwarfDebug24getOrCreateAbstractScopeEPKNS_6MDNodeE", "llvm::DwarfDebug::getOrCreateAbstractScope(llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug24updateSubprogramScopeDIEEPKNS_6MDNodeE", "llvm::DwarfDebug::updateSubprogramScopeDIE(llvm::MDNode const*)"},
    {"_Z19isSubprogramContextPKN4llvm6MDNodeE", "isSubprogramContext(llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug24constructLexicalScopeDIEEPNS_8DbgScopeE", "llvm::DwarfDebug::constructLexicalScopeDIE(llvm::DbgScope*)"},
    {"_ZN4llvm10DwarfDebug18getLabelBeforeInsnEPKNS_12MachineInstrE", "llvm::DwarfDebug::getLabelBeforeInsn(llvm::MachineInstr const*)"},
    {"_ZN4llvm10DwarfDebug17getLabelAfterInsnEPKNS_12MachineInstrE", "llvm::DwarfDebug::getLabelAfterInsn(llvm::MachineInstr const*)"},
    {"_ZN4llvm10DwarfDebug24constructInlinedScopeDIEEPNS_8DbgScopeE", "llvm::DwarfDebug::constructInlinedScopeDIE(llvm::DbgScope*)"},
    {"_ZN4llvm10DwarfDebug20constructVariableDIEEPNS_11DbgVariableEPNS_8DbgScopeE", "llvm::DwarfDebug::constructVariableDIE(llvm::DbgVariable*, llvm::DbgScope*)"},
    {"_ZN4llvm10DwarfDebug22findVariableFrameIndexEPKNS_11DbgVariableEPi", "llvm::DwarfDebug::findVariableFrameIndex(llvm::DbgVariable const*, int*)"},
    {"_ZN4llvm10DwarfDebug11addPubTypesENS_12DISubprogramE", "llvm::DwarfDebug::addPubTypes(llvm::DISubprogram)"},
    {"_ZN4llvm10DwarfDebug17constructScopeDIEEPNS_8DbgScopeE", "llvm::DwarfDebug::constructScopeDIE(llvm::DbgScope*)"},
    {"_ZN4llvm10DwarfDebug20constructCompileUnitEPKNS_6MDNodeE", "llvm::DwarfDebug::constructCompileUnit(llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug26constructGlobalVariableDIEEPKNS_6MDNodeE", "llvm::DwarfDebug::constructGlobalVariableDIE(llvm::MDNode const*)"},
    {"_Z16isUnsignedDITypeN4llvm6DITypeE", "isUnsignedDIType(llvm::DIType)"},
    {"_ZN4llvm10DwarfDebug22constructSubprogramDIEEPKNS_6MDNodeE", "llvm::DwarfDebug::constructSubprogramDIE(llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug17EmitSectionLabelsEv", "llvm::DwarfDebug::EmitSectionLabels()"},
    {"_ZN4llvm10DwarfDebug9endModuleEv", "llvm::DwarfDebug::endModule()"},
    {"_ZN4llvm10DwarfDebug20emitCommonDebugFrameEv", "llvm::DwarfDebug::emitCommonDebugFrame()"},
    {"_ZN4llvm10DwarfDebug22emitFunctionDebugFrameERKNS0_22FunctionDebugFrameInfoE", "llvm::DwarfDebug::emitFunctionDebugFrame(llvm::DwarfDebug::FunctionDebugFrameInfo const&)"},
    {"_ZN4llvm10DwarfDebug21computeSizeAndOffsetsEv", "llvm::DwarfDebug::computeSizeAndOffsets()"},
    {"_ZN4llvm10DwarfDebug13emitDebugInfoEv", "llvm::DwarfDebug::emitDebugInfo()"},
    {"_ZNK4llvm10DwarfDebug17emitAbbreviationsEv", "llvm::DwarfDebug::emitAbbreviations() const"},
    {"_ZN4llvm10DwarfDebug17emitDebugPubNamesEv", "llvm::DwarfDebug::emitDebugPubNames()"},
    {"_ZN4llvm10DwarfDebug17emitDebugPubTypesEv", "llvm::DwarfDebug::emitDebugPubTypes()"},
    {"_ZN4llvm10DwarfDebug12emitDebugLocEv", "llvm::DwarfDebug::emitDebugLoc()"},
    {"_ZN4llvm10DwarfDebug16EmitDebugARangesEv", "llvm::DwarfDebug::EmitDebugARanges()"},
    {"_ZN4llvm10DwarfDebug15emitDebugRangesEv", "llvm::DwarfDebug::emitDebugRanges()"},
    {"_ZN4llvm10DwarfDebug16emitDebugMacInfoEv", "llvm::DwarfDebug::emitDebugMacInfo()"},
    {"_ZN4llvm10DwarfDebug19emitDebugInlineInfoEv", "llvm::DwarfDebug::emitDebugInlineInfo()"},
    {"_ZN4llvm10DwarfDebug12emitDebugStrEv", "llvm::DwarfDebug::emitDebugStr()"},
    {"_ZN4llvm10DwarfDebug20findAbstractVariableERNS_10DIVariableENS_8DebugLocE", "llvm::DwarfDebug::findAbstractVariable(llvm::DIVariable&, llvm::DebugLoc)"},
    {"_ZN4llvm10DwarfDebug31collectVariableInfoFromMMITableEPKNS_15MachineFunctionERNS_11SmallPtrSetIPKNS_6MDNodeELj16EEE", "llvm::DwarfDebug::collectVariableInfoFromMMITable(llvm::MachineFunction const*, llvm::SmallPtrSet<llvm::MDNode const*, 16u>&)"},
    {"_ZN4llvm10DwarfDebug24recordVariableFrameIndexEPKNS_11DbgVariableEi", "llvm::DwarfDebug::recordVariableFrameIndex(llvm::DbgVariable const*, int)"},
    {"_ZN4llvm10DwarfDebug19collectVariableInfoEPKNS_15MachineFunctionERNS_11SmallPtrSetIPKNS_6MDNodeELj16EEE", "llvm::DwarfDebug::collectVariableInfo(llvm::MachineFunction const*, llvm::SmallPtrSet<llvm::MDNode const*, 16u>&)"},
    {"_ZN4llvm10DwarfDebug12findDbgScopeEPKNS_12MachineInstrE", "llvm::DwarfDebug::findDbgScope(llvm::MachineInstr const*)"},
    {"_ZN4llvm10DwarfDebug16beginInstructionEPKNS_12MachineInstrE", "llvm::DwarfDebug::beginInstruction(llvm::MachineInstr const*)"},
    {"_ZN4llvm10DwarfDebug16recordSourceLineEjjPKNS_6MDNodeE", "llvm::DwarfDebug::recordSourceLine(unsigned int, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug14endInstructionEPKNS_12MachineInstrE", "llvm::DwarfDebug::endInstruction(llvm::MachineInstr const*)"},
    {"_ZN4llvm10DwarfDebug19getOrCreateDbgScopeEPKNS_6MDNodeES3_", "llvm::DwarfDebug::getOrCreateDbgScope(llvm::MDNode const*, llvm::MDNode const*)"},
    {"_ZN4llvm10DwarfDebug23extractScopeInformationEv", "llvm::DwarfDebug::extractScopeInformation()"},
    {"_ZN4llvm10DwarfDebug20identifyScopeMarkersEv", "llvm::DwarfDebug::identifyScopeMarkers()"},
    {"_ZN4llvm10DwarfDebug13beginFunctionEPKNS_15MachineFunctionE", "llvm::DwarfDebug::beginFunction(llvm::MachineFunction const*)"},
    {"_ZN4llvm10DwarfDebug11endFunctionEPKNS_15MachineFunctionE", "llvm::DwarfDebug::endFunction(llvm::MachineFunction const*)"},
    {"_ZN4llvm10DwarfDebug20computeSizeAndOffsetEPNS_3DIEEjb", "llvm::DwarfDebug::computeSizeAndOffset(llvm::DIE*, unsigned int, bool)"},
    {"_ZN4llvm10DwarfDebug7emitDIEEPNS_3DIEE", "llvm::DwarfDebug::emitDIE(llvm::DIE*)"},
    {"_ZN4llvm10DwarfDebug19emitEndOfLineMatrixEj", "llvm::DwarfDebug::emitEndOfLineMatrix(unsigned int)"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIjPNS_14StringMapEntryIS1_IPNS_8MCSymbolEjEEEEEEiPKvSA_", "int llvm::array_pod_sort_comparator<std::pair<unsigned int, llvm::StringMapEntry<std::pair<llvm::MCSymbol*, unsigned int> >*> >(void const*, void const*)"},
    {"_ZN4llvm10FoldingSetINS_9DIEAbbrevEED1Ev", "llvm::FoldingSet<llvm::DIEAbbrev>::~FoldingSet()"},
    {"_ZNK4llvm11DbgVariable7getTypeEv", "llvm::DbgVariable::getType() const"},
    {"_ZN4llvm8DenseMapIPNS_3DIEEPKNS_6MDNodeENS_12DenseMapInfoIS2_EENS6_IS5_EEE6insertERKSt4pairIS2_S5_E", "llvm::DenseMap<llvm::DIE*, llvm::MDNode const*, llvm::DenseMapInfo<llvm::DIE*>, llvm::DenseMapInfo<llvm::MDNode const*> >::insert(std::pair<llvm::DIE*, llvm::MDNode const*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_3DIEENS_12DenseMapInfoIS3_EENS6_IS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DIE*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIE*> >::insert(std::pair<llvm::MDNode const*, llvm::DIE*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11CompileUnitENS_12DenseMapInfoIS3_EENS6_IS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::CompileUnit*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::CompileUnit*> >::insert(std::pair<llvm::MDNode const*, llvm::CompileUnit*> const&)"},
    {"_ZN4llvm12UniqueVectorIPKNS_9MCSectionEE6insertERKS3_", "llvm::UniqueVector<llvm::MCSection const*>::insert(llvm::MCSection const* const&)"},
    {"_ZN4llvm22DeleteContainerSecondsINS_8DenseMapIPKNS_6MDNodeEPNS_8DbgScopeENS_12DenseMapInfoIS4_EENS7_IS6_EEEEEEvRT_", "void llvm::DeleteContainerSeconds<llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> > >(llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> >&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::insert(std::pair<llvm::MDNode const*, llvm::DbgScope*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEiNS_12DenseMapInfoIS3_EENS4_IiEEE16InsertIntoBucketERKS3_RKiPSt4pairIS3_iE", "llvm::DenseMap<llvm::DbgVariable const*, int, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<int> >::InsertIntoBucket(llvm::DbgVariable const* const&, int const&, std::pair<llvm::DbgVariable const*, int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEiNS_12DenseMapInfoIS3_EENS4_IiEEE4growEj", "llvm::DenseMap<llvm::DbgVariable const*, int, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrEPNS_8MCSymbolENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineInstr const*, llvm::MCSymbol*, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::MCSymbol*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11DbgVariableENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgVariable*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgVariable*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEPKNS_12MachineInstrENS_12DenseMapInfoIS3_EENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::DbgVariable const*, llvm::MachineInstr const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableES3_NS_12DenseMapInfoIS3_EES5_E16shrink_and_clearEv", "llvm::DenseMap<llvm::DbgVariable const*, llvm::DbgVariable const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::DbgVariable const*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEiNS_12DenseMapInfoIS3_EENS4_IiEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::DbgVariable const*, int, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<int> >::shrink_and_clear()"},
    {"_ZNSt6vectorIN4llvm10DwarfDebug22FunctionDebugFrameInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::DwarfDebug::FunctionDebugFrameInfo, std::allocator<llvm::DwarfDebug::FunctionDebugFrameInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DwarfDebug::FunctionDebugFrameInfo*, std::vector<llvm::DwarfDebug::FunctionDebugFrameInfo, std::allocator<llvm::DwarfDebug::FunctionDebugFrameInfo> > >, llvm::DwarfDebug::FunctionDebugFrameInfo const&)"},
    {"_ZNSt6vectorIN4llvm11MachineMoveESaIS1_EEaSERKS3_", "std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> >::operator=(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> > const&)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MachineInstr const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::InsertIntoBucket(llvm::MachineInstr const* const&, llvm::DbgScope* const&, std::pair<llvm::MachineInstr const*, llvm::DbgScope*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::InsertIntoBucket(llvm::MDNode const* const&, llvm::DbgScope* const&, std::pair<llvm::MDNode const*, llvm::DbgScope*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DbgScopeENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgScope*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgScope*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrEPNS_8MCSymbolENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MachineInstr const*, llvm::MCSymbol*, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::MCSymbol*> >::InsertIntoBucket(llvm::MachineInstr const* const&, llvm::MCSymbol* const&, std::pair<llvm::MachineInstr const*, llvm::MCSymbol*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrEPNS_8MCSymbolENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr const*, llvm::MCSymbol*, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::MCSymbol*> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_16DotDebugLocEntryELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::DotDebugLocEntry, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEPKNS_12MachineInstrENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PSt4pairIS3_S6_E", "llvm::DenseMap<llvm::DbgVariable const*, llvm::MachineInstr const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::InsertIntoBucket(llvm::DbgVariable const* const&, llvm::MachineInstr const* const&, std::pair<llvm::DbgVariable const*, llvm::MachineInstr const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableEPKNS_12MachineInstrENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::DbgVariable const*, llvm::MachineInstr const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<llvm::DbgVariable const*, llvm::DbgVariable const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::DbgVariable const*> >::InsertIntoBucket(llvm::DbgVariable const* const&, llvm::DbgVariable const* const&, std::pair<llvm::DbgVariable const*, llvm::DbgVariable const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_11DbgVariableES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<llvm::DbgVariable const*, llvm::DbgVariable const*, llvm::DenseMapInfo<llvm::DbgVariable const*>, llvm::DenseMapInfo<llvm::DbgVariable const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11DbgVariableENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgVariable*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgVariable*> >::InsertIntoBucket(llvm::MDNode const* const&, llvm::DbgVariable* const&, std::pair<llvm::MDNode const*, llvm::DbgVariable*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11DbgVariableENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::DbgVariable*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DbgVariable*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIPKN4llvm9MCSectionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::MCSection const*, std::pair<llvm::MCSection const* const, unsigned int>, std::_Select1st<std::pair<llvm::MCSection const* const, unsigned int> >, std::less<llvm::MCSection const*>, std::allocator<std::pair<llvm::MCSection const* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MCSection const* const, unsigned int> >, std::pair<llvm::MCSection const* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm9MCSectionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::MCSection const*, std::pair<llvm::MCSection const* const, unsigned int>, std::_Select1st<std::pair<llvm::MCSection const* const, unsigned int> >, std::less<llvm::MCSection const*>, std::allocator<std::pair<llvm::MCSection const* const, unsigned int> > >::_M_insert_unique(std::pair<llvm::MCSection const* const, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11CompileUnitENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::CompileUnit*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::CompileUnit*> >::InsertIntoBucket(llvm::MDNode const* const&, llvm::CompileUnit* const&, std::pair<llvm::MDNode const*, llvm::CompileUnit*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_11CompileUnitENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::CompileUnit*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::CompileUnit*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeENS_11SmallVectorISt4pairIPKNS_8MCSymbolEPNS_3DIEEELj4EEENS_12DenseMapInfoIS3_EENSD_ISC_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::MDNode const*, llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u>, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u> > >::FindAndConstruct(llvm::MDNode const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeENS_11SmallVectorISt4pairIPKNS_8MCSymbolEPNS_3DIEEELj4EEENS_12DenseMapInfoIS3_EENSD_ISC_EEE16InsertIntoBucketERKS3_RKSC_PS5_IS3_SC_E", "llvm::DenseMap<llvm::MDNode const*, llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u>, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u> > >::InsertIntoBucket(llvm::MDNode const* const&, llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u> const&, std::pair<llvm::MDNode const*, llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeENS_11SmallVectorISt4pairIPKNS_8MCSymbolEPNS_3DIEEELj4EEENS_12DenseMapInfoIS3_EENSD_ISC_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u>, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<llvm::MCSymbol const*, llvm::DIE*>, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPKNS_8MCSymbolEPNS_3DIEEEEaSERKS8_", "llvm::SmallVectorImpl<std::pair<llvm::MCSymbol const*, llvm::DIE*> >::operator=(llvm::SmallVectorImpl<std::pair<llvm::MCSymbol const*, llvm::DIE*> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_3DIEEPKNS_6MDNodeENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::DIE*, llvm::MDNode const*, llvm::DenseMapInfo<llvm::DIE*>, llvm::DenseMapInfo<llvm::MDNode const*> >::InsertIntoBucket(llvm::DIE* const&, llvm::MDNode const* const&, std::pair<llvm::DIE*, llvm::MDNode const*>*)"},
    {"_ZN4llvm8DenseMapIPNS_3DIEEPKNS_6MDNodeENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::DIE*, llvm::MDNode const*, llvm::DenseMapInfo<llvm::DIE*>, llvm::DenseMapInfo<llvm::MDNode const*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm8DIEBlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::DIEBlock*, std::allocator<llvm::DIEBlock*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DIEBlock**, std::vector<llvm::DIEBlock*, std::allocator<llvm::DIEBlock*> > >, llvm::DIEBlock* const&)"},
    {"_ZNSt6vectorIPN4llvm9DIEAbbrevESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::DIEAbbrev*, std::allocator<llvm::DIEAbbrev*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DIEAbbrev**, std::vector<llvm::DIEAbbrev*, std::allocator<llvm::DIEAbbrev*> > >, llvm::DIEAbbrev* const&)"},
    {"_ZN4llvm9StringMapISt4pairIPNS_8MCSymbolEjENS_15MallocAllocatorEE16GetOrCreateValueIS4_EERNS_14StringMapEntryIS4_EENS_9StringRefET_", "llvm::StringMapEntry<std::pair<llvm::MCSymbol*, unsigned int> >& llvm::StringMap<std::pair<llvm::MCSymbol*, unsigned int>, llvm::MallocAllocator>::GetOrCreateValue<std::pair<llvm::MCSymbol*, unsigned int> >(llvm::StringRef, std::pair<llvm::MCSymbol*, unsigned int>)"},
    {"_ZN4llvm10FoldingSetINS_9DIEAbbrevEED0Ev", "llvm::FoldingSet<llvm::DIEAbbrev>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_9DIEAbbrevEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::DIEAbbrev>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_9DIEAbbrevEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS6_", "llvm::FoldingSet<llvm::DIEAbbrev>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_9DIEAbbrevEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::DIEAbbrev>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_3DIEENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DIE*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIE*> >::InsertIntoBucket(llvm::MDNode const* const&, llvm::DIE* const&, std::pair<llvm::MDNode const*, llvm::DIE*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_3DIEENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::DIE*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIE*> >::grow(unsigned int)"},
    {"_ZN4llvm11CompileUnitD2Ev", "llvm::CompileUnit::~CompileUnit()"},
    {"_ZN4llvm9StringMapIPNS_3DIEENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::DIE*>& llvm::StringMap<llvm::DIE*, llvm::MallocAllocator>::GetOrCreateValue<llvm::DIE*>(llvm::StringRef, llvm::DIE*)"},
    {"_ZN4llvm11CompileUnitC2EjPNS_3DIEE", "llvm::CompileUnit::CompileUnit(unsigned int, llvm::DIE*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DIEEntryENS_12DenseMapInfoIS3_EENS6_IS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DIEEntry*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIEEntry*> >::insert(std::pair<llvm::MDNode const*, llvm::DIEEntry*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DIEEntryENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MDNode const*, llvm::DIEEntry*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIEEntry*> >::InsertIntoBucket(llvm::MDNode const* const&, llvm::DIEEntry* const&, std::pair<llvm::MDNode const*, llvm::DIEEntry*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEPNS_8DIEEntryENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MDNode const*, llvm::DIEEntry*, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<llvm::DIEEntry*> >::grow(unsigned int)"},
    {"_ZN4llvm9DIEAbbrevD1Ev", "llvm::DIEAbbrev::~DIEAbbrev()"},
    {"_ZN4llvm9DIEAbbrevD0Ev", "llvm::DIEAbbrev::~DIEAbbrev()"},
    {"_ZNSt6vectorIPN4llvm3DIEESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::DIE*, std::allocator<llvm::DIE*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DIE**, std::vector<llvm::DIE*, std::allocator<llvm::DIE*> > >, llvm::DIE* const&)"},
    {"_ZN4llvm8DIEBlockD1Ev", "llvm::DIEBlock::~DIEBlock()"},
    {"_ZN4llvm8DIEBlockD0Ev", "llvm::DIEBlock::~DIEBlock()"},
    {"_ZThn16_N4llvm8DIEBlockD1Ev", "non-virtual thunk to llvm::DIEBlock::~DIEBlock()"},
    {"_ZThn16_N4llvm8DIEBlockD0Ev", "non-virtual thunk to llvm::DIEBlock::~DIEBlock()"},
    {"_ZN4llvm8DIEValueD1Ev", "llvm::DIEValue::~DIEValue()"},
    {"_ZN4llvm8DIEValueD0Ev", "llvm::DIEValue::~DIEValue()"},
    {"_ZNK4llvm9DIEString6SizeOfEPNS_10AsmPrinterEj", "llvm::DIEString::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_13DIEAbbrevDataELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::DIEAbbrevData, false>::grow(unsigned long)"},
    {"_ZNK4llvm8DIEEntry6SizeOfEPNS_10AsmPrinterEj", "llvm::DIEEntry::SizeOf(llvm::AsmPrinter*, unsigned int) const"},
    {"_ZNSt8_Rb_treeIPKN4llvm9MCSectionESt4pairIKS3_jESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::MCSection const*, std::pair<llvm::MCSection const* const, unsigned int>, std::_Select1st<std::pair<llvm::MCSection const* const, unsigned int> >, std::less<llvm::MCSection const*>, std::allocator<std::pair<llvm::MCSection const* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MCSection const* const, unsigned int> >*)"},
    {"_ZN4llvm14DwarfExceptionC1EPNS_10AsmPrinterE", "llvm::DwarfException::DwarfException(llvm::AsmPrinter*)"},
    {"_ZN4llvm14DwarfExceptionC2EPNS_10AsmPrinterE", "llvm::DwarfException::DwarfException(llvm::AsmPrinter*)"},
    {"_ZN4llvm14DwarfExceptionD0Ev", "llvm::DwarfException::~DwarfException()"},
    {"_ZN4llvm14DwarfExceptionD1Ev", "llvm::DwarfException::~DwarfException()"},
    {"_ZN4llvm14DwarfExceptionD2Ev", "llvm::DwarfException::~DwarfException()"},
    {"_ZN4llvm14DwarfException13SharedTypeIdsEPKNS_14LandingPadInfoES3_", "llvm::DwarfException::SharedTypeIds(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)"},
    {"_ZN4llvm14DwarfException5PadLTEPKNS_14LandingPadInfoES3_", "llvm::DwarfException::PadLT(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)"},
    {"_ZN4llvm14DwarfException19ComputeActionsTableERKNS_15SmallVectorImplIPKNS_14LandingPadInfoEEERNS1_INS0_11ActionEntryEEERNS1_IjEE", "llvm::DwarfException::ComputeActionsTable(llvm::SmallVectorImpl<llvm::LandingPadInfo const*> const&, llvm::SmallVectorImpl<llvm::DwarfException::ActionEntry>&, llvm::SmallVectorImpl<unsigned int>&)"},
    {"_ZN4llvm14DwarfException22CallToNoUnwindFunctionEPKNS_12MachineInstrE", "llvm::DwarfException::CallToNoUnwindFunction(llvm::MachineInstr const*)"},
    {"_ZN4llvm14DwarfException20ComputeCallSiteTableERNS_15SmallVectorImplINS0_13CallSiteEntryEEERKNS_8DenseMapIPNS_8MCSymbolENS0_8PadRangeENS_12DenseMapInfoIS7_EENS9_IS8_EEEERKNS1_IPKNS_14LandingPadInfoEEERKNS1_IjEE", "llvm::DwarfException::ComputeCallSiteTable(llvm::SmallVectorImpl<llvm::DwarfException::CallSiteEntry>&, llvm::DenseMap<llvm::MCSymbol*, llvm::DwarfException::PadRange, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::DwarfException::PadRange> > const&, llvm::SmallVectorImpl<llvm::LandingPadInfo const*> const&, llvm::SmallVectorImpl<unsigned int> const&)"},
    {"_ZN4llvm14DwarfException18EmitExceptionTableEv", "llvm::DwarfException::EmitExceptionTable()"},
    {"_ZN4llvm14DwarfException9EndModuleEv", "llvm::DwarfException::EndModule()"},
    {"_ZN4llvm14DwarfException13BeginFunctionEPKNS_15MachineFunctionE", "llvm::DwarfException::BeginFunction(llvm::MachineFunction const*)"},
    {"_ZN4llvm14DwarfException11EndFunctionEv", "llvm::DwarfException::EndFunction()"},
    {"_ZN4llvm15SmallVectorImplINS_14DwarfException13CallSiteEntryEE6resizeEj", "llvm::SmallVectorImpl<llvm::DwarfException::CallSiteEntry>::resize(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolENS_14DwarfException8PadRangeENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MCSymbol*, llvm::DwarfException::PadRange, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::DwarfException::PadRange> >::InsertIntoBucket(llvm::MCSymbol* const&, llvm::DwarfException::PadRange const&, std::pair<llvm::MCSymbol*, llvm::DwarfException::PadRange>*)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolENS_14DwarfException8PadRangeENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MCSymbol*, llvm::DwarfException::PadRange, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::DwarfException::PadRange> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIPPKN4llvm14LandingPadInfoElPFbS3_S3_EEvT_S7_T0_T1_", "void std::__introsort_loop<llvm::LandingPadInfo const**, long, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)>(llvm::LandingPadInfo const**, llvm::LandingPadInfo const**, long, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*))"},
    {"_ZSt22__final_insertion_sortIPPKN4llvm14LandingPadInfoEPFbS3_S3_EEvT_S7_T0_", "void std::__final_insertion_sort<llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)>(llvm::LandingPadInfo const**, llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*))"},
    {"_ZSt16__insertion_sortIPPKN4llvm14LandingPadInfoEPFbS3_S3_EEvT_S7_T0_", "void std::__insertion_sort<llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)>(llvm::LandingPadInfo const**, llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*))"},
    {"_ZSt13__heap_selectIPPKN4llvm14LandingPadInfoEPFbS3_S3_EEvT_S7_S7_T0_", "void std::__heap_select<llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)>(llvm::LandingPadInfo const**, llvm::LandingPadInfo const**, llvm::LandingPadInfo const**, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*))"},
    {"_ZSt13__adjust_heapIPPKN4llvm14LandingPadInfoElS3_PFbS3_S3_EEvT_T0_S8_T1_T2_", "void std::__adjust_heap<llvm::LandingPadInfo const**, long, llvm::LandingPadInfo const*, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*)>(llvm::LandingPadInfo const**, long, long, llvm::LandingPadInfo const*, bool (*)(llvm::LandingPadInfo const*, llvm::LandingPadInfo const*))"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_14DwarfException13CallSiteEntryELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::DwarfException::CallSiteEntry, false>::grow(unsigned long)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_14DwarfException11ActionEntryELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::DwarfException::ActionEntry, false>::grow(unsigned long)"},
    {"_ZN4llvm19DwarfTableExceptionC1EPNS_10AsmPrinterE", "llvm::DwarfTableException::DwarfTableException(llvm::AsmPrinter*)"},
    {"_ZN4llvm19DwarfTableExceptionC2EPNS_10AsmPrinterE", "llvm::DwarfTableException::DwarfTableException(llvm::AsmPrinter*)"},
    {"_ZN4llvm19DwarfTableExceptionD0Ev", "llvm::DwarfTableException::~DwarfTableException()"},
    {"_ZN4llvm19DwarfTableExceptionD1Ev", "llvm::DwarfTableException::~DwarfTableException()"},
    {"_ZN4llvm19DwarfTableExceptionD2Ev", "llvm::DwarfTableException::~DwarfTableException()"},
    {"_ZN4llvm19DwarfTableException7EmitCIEEPKNS_8FunctionEj", "llvm::DwarfTableException::EmitCIE(llvm::Function const*, unsigned int)"},
    {"_ZN4llvm19DwarfTableException7EmitFDEERKNS0_19FunctionEHFrameInfoE", "llvm::DwarfTableException::EmitFDE(llvm::DwarfTableException::FunctionEHFrameInfo const&)"},
    {"_ZN4llvm19DwarfTableException9EndModuleEv", "llvm::DwarfTableException::EndModule()"},
    {"_ZN4llvm19DwarfTableException13BeginFunctionEPKNS_15MachineFunctionE", "llvm::DwarfTableException::BeginFunction(llvm::MachineFunction const*)"},
    {"_ZN4llvm19DwarfTableException11EndFunctionEv", "llvm::DwarfTableException::EndFunction()"},
    {"_ZNSt6vectorIN4llvm19DwarfTableException19FunctionEHFrameInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::DwarfTableException::FunctionEHFrameInfo, std::allocator<llvm::DwarfTableException::FunctionEHFrameInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DwarfTableException::FunctionEHFrameInfo*, std::vector<llvm::DwarfTableException::FunctionEHFrameInfo, std::allocator<llvm::DwarfTableException::FunctionEHFrameInfo> > >, llvm::DwarfTableException::FunctionEHFrameInfo const&)"},
    {"_ZN4llvm8AsmLexerC1ERKNS_9MCAsmInfoE", "llvm::AsmLexer::AsmLexer(llvm::MCAsmInfo const&)"},
    {"_ZN4llvm8AsmLexerC2ERKNS_9MCAsmInfoE", "llvm::AsmLexer::AsmLexer(llvm::MCAsmInfo const&)"},
    {"_ZN4llvm8AsmLexerD0Ev", "llvm::AsmLexer::~AsmLexer()"},
    {"_ZN4llvm8AsmLexerD1Ev", "llvm::AsmLexer::~AsmLexer()"},
    {"_ZN4llvm8AsmLexerD2Ev", "llvm::AsmLexer::~AsmLexer()"},
    {"_ZN4llvm8AsmLexer9setBufferEPKNS_12MemoryBufferEPKc", "llvm::AsmLexer::setBuffer(llvm::MemoryBuffer const*, char const*)"},
    {"_ZN4llvm8AsmLexer11ReturnErrorEPKcRKSs", "llvm::AsmLexer::ReturnError(char const*, std::string const&)"},
    {"_ZN4llvm8AsmLexer11getNextCharEv", "llvm::AsmLexer::getNextChar()"},
    {"_ZN4llvm8AsmLexer15LexFloatLiteralEv", "llvm::AsmLexer::LexFloatLiteral()"},
    {"_ZN4llvm8AsmLexer13LexIdentifierEv", "llvm::AsmLexer::LexIdentifier()"},
    {"_ZN4llvm8AsmLexer8LexSlashEv", "llvm::AsmLexer::LexSlash()"},
    {"_ZN4llvm8AsmLexer14LexLineCommentEv", "llvm::AsmLexer::LexLineComment()"},
    {"_ZN4llvm8AsmLexer8LexDigitEv", "llvm::AsmLexer::LexDigit()"},
    {"_ZN4llvm8AsmLexer14LexSingleQuoteEv", "llvm::AsmLexer::LexSingleQuote()"},
    {"_ZN4llvm8AsmLexer8LexQuoteEv", "llvm::AsmLexer::LexQuote()"},
    {"_ZN4llvm8AsmLexer22LexUntilEndOfStatementEv", "llvm::AsmLexer::LexUntilEndOfStatement()"},
    {"_ZN4llvm8AsmLexer18isAtStartOfCommentEc", "llvm::AsmLexer::isAtStartOfComment(char)"},
    {"_ZN4llvm8AsmLexer8LexTokenEv", "llvm::AsmLexer::LexToken()"},
    {"_ZN4llvm17createMCAsmParserERKNS_6TargetERNS_9SourceMgrERNS_9MCContextERNS_10MCStreamerERKNS_9MCAsmInfoE", "llvm::createMCAsmParser(llvm::Target const&, llvm::SourceMgr&, llvm::MCContext&, llvm::MCStreamer&, llvm::MCAsmInfo const&)"},
    {"_ZN12_GLOBAL__N_19AsmParserD1Ev", "(anonymous namespace)::AsmParser::~AsmParser()"},
    {"_ZN12_GLOBAL__N_19AsmParserD0Ev", "(anonymous namespace)::AsmParser::~AsmParser()"},
    {"_ZN12_GLOBAL__N_19AsmParser19AddDirectiveHandlerEPN4llvm20MCAsmParserExtensionENS1_9StringRefEPFbS3_S4_NS1_5SMLocEE", "(anonymous namespace)::AsmParser::AddDirectiveHandler(llvm::MCAsmParserExtension*, llvm::StringRef, bool (*)(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc))"},
    {"_ZN12_GLOBAL__N_19AsmParser16getSourceManagerEv", "(anonymous namespace)::AsmParser::getSourceManager()"},
    {"_ZN12_GLOBAL__N_19AsmParser8getLexerEv", "(anonymous namespace)::AsmParser::getLexer()"},
    {"_ZN12_GLOBAL__N_19AsmParser10getContextEv", "(anonymous namespace)::AsmParser::getContext()"},
    {"_ZN12_GLOBAL__N_19AsmParser11getStreamerEv", "(anonymous namespace)::AsmParser::getStreamer()"},
    {"_ZN12_GLOBAL__N_19AsmParser3RunEbb", "(anonymous namespace)::AsmParser::Run(bool, bool)"},
    {"_ZN12_GLOBAL__N_19AsmParser7WarningEN4llvm5SMLocERKNS1_5TwineE", "(anonymous namespace)::AsmParser::Warning(llvm::SMLoc, llvm::Twine const&)"},
    {"_ZN12_GLOBAL__N_19AsmParser5ErrorEN4llvm5SMLocERKNS1_5TwineE", "(anonymous namespace)::AsmParser::Error(llvm::SMLoc, llvm::Twine const&)"},
    {"_ZN12_GLOBAL__N_19AsmParser3LexEv", "(anonymous namespace)::AsmParser::Lex()"},
    {"_ZN12_GLOBAL__N_19AsmParser15ParseIdentifierERN4llvm9StringRefE", "(anonymous namespace)::AsmParser::ParseIdentifier(llvm::StringRef&)"},
    {"_ZN12_GLOBAL__N_19AsmParser27ParseStringToEndOfStatementEv", "(anonymous namespace)::AsmParser::ParseStringToEndOfStatement()"},
    {"_ZN12_GLOBAL__N_19AsmParser19EatToEndOfStatementEv", "(anonymous namespace)::AsmParser::EatToEndOfStatement()"},
    {"_ZN12_GLOBAL__N_19AsmParser15ParseExpressionERPKN4llvm6MCExprERNS1_5SMLocE", "(anonymous namespace)::AsmParser::ParseExpression(llvm::MCExpr const*&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_19AsmParser20ParseParenExpressionERPKN4llvm6MCExprERNS1_5SMLocE", "(anonymous namespace)::AsmParser::ParseParenExpression(llvm::MCExpr const*&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_19AsmParser23ParseAbsoluteExpressionERx", "(anonymous namespace)::AsmParser::ParseAbsoluteExpression(long long&)"},
    {"_ZN12_GLOBAL__N_19AsmParser14ParseParenExprERPKN4llvm6MCExprERNS1_5SMLocE", "(anonymous namespace)::AsmParser::ParseParenExpr(llvm::MCExpr const*&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_19AsmParser13ParseBinOpRHSEjRPKN4llvm6MCExprERNS1_5SMLocE", "(anonymous namespace)::AsmParser::ParseBinOpRHS(unsigned int, llvm::MCExpr const*&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_19AsmParser16ParsePrimaryExprERPKN4llvm6MCExprERNS1_5SMLocE", "(anonymous namespace)::AsmParser::ParsePrimaryExpr(llvm::MCExpr const*&, llvm::SMLoc&)"},
    {"_ZN12_GLOBAL__N_19AsmParser19ApplyModifierToExprEPKN4llvm6MCExprENS1_15MCSymbolRefExpr11VariantKindE", "(anonymous namespace)::AsmParser::ApplyModifierToExpr(llvm::MCExpr const*, llvm::MCSymbolRefExpr::VariantKind)"},
    {"_ZN12_GLOBAL__N_19AsmParser14ParseStatementEv", "(anonymous namespace)::AsmParser::ParseStatement()"},
    {"_ZN12_GLOBAL__N_19AsmParser19ParseDirectiveIfdefEN4llvm5SMLocEb", "(anonymous namespace)::AsmParser::ParseDirectiveIfdef(llvm::SMLoc, bool)"},
    {"_ZN12_GLOBAL__N_19AsmParser20CheckForValidSectionEv", "(anonymous namespace)::AsmParser::CheckForValidSection()"},
    {"_ZN12_GLOBAL__N_19AsmParser15ParseAssignmentEN4llvm9StringRefEb", "(anonymous namespace)::AsmParser::ParseAssignment(llvm::StringRef, bool)"},
    {"_ZN12_GLOBAL__N_19AsmParser17ParseDirectiveSetEN4llvm9StringRefEb", "(anonymous namespace)::AsmParser::ParseDirectiveSet(llvm::StringRef, bool)"},
    {"_ZN12_GLOBAL__N_19AsmParser19ParseDirectiveAsciiEN4llvm9StringRefEb", "(anonymous namespace)::AsmParser::ParseDirectiveAscii(llvm::StringRef, bool)"},
    {"_ZN12_GLOBAL__N_19AsmParser19ParseDirectiveValueEj", "(anonymous namespace)::AsmParser::ParseDirectiveValue(unsigned int)"},
    {"_ZN12_GLOBAL__N_19AsmParser23ParseDirectiveRealValueERKN4llvm12fltSemanticsE", "(anonymous namespace)::AsmParser::ParseDirectiveRealValue(llvm::fltSemantics const&)"},
    {"_ZN12_GLOBAL__N_19AsmParser19ParseDirectiveAlignEbj", "(anonymous namespace)::AsmParser::ParseDirectiveAlign(bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_19AsmParser29ParseDirectiveSymbolAttributeEN4llvm12MCSymbolAttrE", "(anonymous namespace)::AsmParser::ParseDirectiveSymbolAttribute(llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_19AsmParser18ParseDirectiveCommEb", "(anonymous namespace)::AsmParser::ParseDirectiveComm(bool)"},
    {"_Z8MarkUsedPKN4llvm6MCExprE", "MarkUsed(llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_116GenericAsmParserD1Ev", "(anonymous namespace)::GenericAsmParser::~GenericAsmParser()"},
    {"_ZN12_GLOBAL__N_116GenericAsmParserD0Ev", "(anonymous namespace)::GenericAsmParser::~GenericAsmParser()"},
    {"_ZN12_GLOBAL__N_116GenericAsmParser10InitializeERN4llvm11MCAsmParserE", "(anonymous namespace)::GenericAsmParser::Initialize(llvm::MCAsmParser&)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_20ParseDirectiveLEB128ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveLEB128(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_22ParseDirectiveEndMacroENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveEndMacro(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_19ParseDirectiveMacroENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveMacro(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_25ParseDirectiveMacrosOnOffENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveMacrosOnOff(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_29ParseDirectiveCFIRestoreStateENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIRestoreState(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_30ParseDirectiveCFIRememberStateENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIRememberState(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_34ParseDirectiveCFIPersonalityOrLsdaENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIPersonalityOrLsda(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_23ParseDirectiveCFIOffsetENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIOffset(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN12_GLOBAL__N_116GenericAsmParser29ParseRegisterOrRegisterNumberERxN4llvm5SMLocE", "(anonymous namespace)::GenericAsmParser::ParseRegisterOrRegisterNumber(long long&, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_31ParseDirectiveCFIDefCfaRegisterENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIDefCfaRegister(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_29ParseDirectiveCFIDefCfaOffsetENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIDefCfaOffset(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_23ParseDirectiveCFIDefCfaENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIDefCfa(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_24ParseDirectiveCFIEndProcENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIEndProc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_26ParseDirectiveCFIStartProcENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveCFIStartProc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_19ParseDirectiveStabsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveStabs(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_17ParseDirectiveLocENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveLoc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_18ParseDirectiveLineENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveLine(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_116GenericAsmParserEXadL_ZNS3_18ParseDirectiveFileENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::GenericAsmParser, &((anonymous namespace)::GenericAsmParser::ParseDirectiveFile(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZNSt6vectorIN4llvm8AsmTokenESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AsmToken*, std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > >, llvm::AsmToken const&)"},
    {"_ZNSt6vectorIS_IN4llvm8AsmTokenESaIS1_EESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >, std::allocator<std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >*, std::vector<std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >, std::allocator<std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > > > >, std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > const&)"},
    {"_ZNSt6vectorIN4llvm8AsmTokenESaIS1_EEaSERKS3_", "std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >::operator=(std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > const&)"},
    {"_ZNSt6vectorIN4llvm8AsmTokenESaIS1_EEC2ERKS3_", "std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> >::vector(std::vector<llvm::AsmToken, std::allocator<llvm::AsmToken> > const&)"},
    {"_ZNSt6vectorIN4llvm7AsmCondESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::AsmCond, std::allocator<llvm::AsmCond> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AsmCond*, std::vector<llvm::AsmCond, std::allocator<llvm::AsmCond> > >, llvm::AsmCond const&)"},
    {"_ZN4llvm9StringMapISt4pairIPNS_20MCAsmParserExtensionEPFbS3_NS_9StringRefENS_5SMLocEEENS_15MallocAllocatorEE16GetOrCreateValueIS8_EERNS_14StringMapEntryIS8_EES4_T_", "llvm::StringMapEntry<std::pair<llvm::MCAsmParserExtension*, bool (*)(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)> >& llvm::StringMap<std::pair<llvm::MCAsmParserExtension*, bool (*)(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)>, llvm::MallocAllocator>::GetOrCreateValue<std::pair<llvm::MCAsmParserExtension*, bool (*)(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)> >(llvm::StringRef, std::pair<llvm::MCAsmParserExtension*, bool (*)(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)>)"},
    {"_ZN4llvm19createCOFFAsmParserEv", "llvm::createCOFFAsmParser()"},
    {"_ZN12_GLOBAL__N_113COFFAsmParserD1Ev", "(anonymous namespace)::COFFAsmParser::~COFFAsmParser()"},
    {"_ZN12_GLOBAL__N_113COFFAsmParserD0Ev", "(anonymous namespace)::COFFAsmParser::~COFFAsmParser()"},
    {"_ZN12_GLOBAL__N_113COFFAsmParser10InitializeERN4llvm11MCAsmParserE", "(anonymous namespace)::COFFAsmParser::Initialize(llvm::MCAsmParser&)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_19ParseDirectiveEndefENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseDirectiveEndef(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_18ParseDirectiveTypeENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseDirectiveType(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_17ParseDirectiveSclENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseDirectiveScl(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_17ParseDirectiveDefENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseDirectiveDef(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_24ParseSectionDirectiveBSSENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseSectionDirectiveBSS(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN12_GLOBAL__N_113COFFAsmParser18ParseSectionSwitchEN4llvm9StringRefEjNS1_11SectionKindE", "(anonymous namespace)::COFFAsmParser::ParseSectionSwitch(llvm::StringRef, unsigned int, llvm::SectionKind)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_25ParseSectionDirectiveDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseSectionDirectiveData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_113COFFAsmParserEXadL_ZNS3_25ParseSectionDirectiveTextENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::COFFAsmParser, &((anonymous namespace)::COFFAsmParser::ParseSectionDirectiveText(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm21createDarwinAsmParserEv", "llvm::createDarwinAsmParser()"},
    {"_ZN12_GLOBAL__N_115DarwinAsmParserD1Ev", "(anonymous namespace)::DarwinAsmParser::~DarwinAsmParser()"},
    {"_ZN12_GLOBAL__N_115DarwinAsmParserD0Ev", "(anonymous namespace)::DarwinAsmParser::~DarwinAsmParser()"},
    {"_ZN12_GLOBAL__N_115DarwinAsmParser10InitializeERN4llvm11MCAsmParserE", "(anonymous namespace)::DarwinAsmParser::Initialize(llvm::MCAsmParser&)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_24ParseSectionDirectiveTLVENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveTLV(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN12_GLOBAL__N_115DarwinAsmParser18ParseSectionSwitchEPKcS2_jjj", "(anonymous namespace)::DarwinAsmParser::ParseSectionSwitch(char const*, char const*, unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_35ParseSectionDirectiveThreadInitFuncENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveThreadInitFunc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_25ParseSectionDirectiveTextENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveText(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_26ParseSectionDirectiveTDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveTData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_31ParseSectionDirectiveSymbolStubENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveSymbolStub(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_31ParseSectionDirectiveStaticDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveStaticData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveStaticConstENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveStaticConst(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_34ParseSectionDirectivePICSymbolStubENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectivePICSymbolStub(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveObjCSymbolsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCSymbols(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_37ParseSectionDirectiveObjCStringObjectENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCStringObject(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_37ParseSectionDirectiveObjCSelectorStrsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCSelectorStrs(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_33ParseSectionDirectiveObjCProtocolENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCProtocol(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_35ParseSectionDirectiveObjCModuleInfoENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCModuleInfo(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_37ParseSectionDirectiveObjCMethVarTypesENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCMethVarTypes(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_37ParseSectionDirectiveObjCMethVarNamesENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCMethVarNames(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_34ParseSectionDirectiveObjCMetaClassENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCMetaClass(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_36ParseSectionDirectiveObjCMessageRefsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCMessageRefs(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_37ParseSectionDirectiveObjCInstanceVarsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCInstanceVars(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_33ParseSectionDirectiveObjCInstMethENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCInstMeth(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveObjCClsRefsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCClsRefs(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveObjCClsMethENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCClsMeth(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_34ParseSectionDirectiveObjCClassVarsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCClassVars(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_35ParseSectionDirectiveObjCClassNamesENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCClassNames(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_30ParseSectionDirectiveObjCClassENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCClass(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_33ParseSectionDirectiveObjCCategoryENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCCategory(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_36ParseSectionDirectiveObjCCatInstMethENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCCatInstMeth(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_35ParseSectionDirectiveObjCCatClsMethENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveObjCCatClsMeth(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_42ParseSectionDirectiveNonLazySymbolPointersENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveNonLazySymbolPointers(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveModTermFuncENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveModTermFunc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveModInitFuncENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveModInitFunc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_29ParseSectionDirectiveLiteral8ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveLiteral8(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_29ParseSectionDirectiveLiteral4ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveLiteral4(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_30ParseSectionDirectiveLiteral16ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveLiteral16(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_39ParseSectionDirectiveLazySymbolPointersENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveLazySymbolPointers(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveFVMLibInit1ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveFVMLibInit1(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveFVMLibInit0ENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveFVMLibInit0(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_25ParseSectionDirectiveDyldENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveDyld(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_31ParseSectionDirectiveDestructorENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveDestructor(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_25ParseSectionDirectiveDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_28ParseSectionDirectiveCStringENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveCString(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_32ParseSectionDirectiveConstructorENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveConstructor(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_30ParseSectionDirectiveConstDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveConstData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_26ParseSectionDirectiveConstENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseSectionDirectiveConst(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_22ParseDirectiveZerofillENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveZerofill(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_18ParseDirectiveTBSSENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveTBSS(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_28ParseDirectiveSecureLogResetENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveSecureLogReset(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_29ParseDirectiveSecureLogUniqueENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveSecureLogUnique(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_21ParseDirectiveSectionENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveSection(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_24ParseDirectiveDumpOrLoadENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveDumpOrLoad(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_35ParseDirectiveSubsectionsViaSymbolsENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveSubsectionsViaSymbols(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_18ParseDirectiveLsymENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveLsym(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_115DarwinAsmParserEXadL_ZNS3_18ParseDirectiveDescENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::DarwinAsmParser, &((anonymous namespace)::DarwinAsmParser::ParseDirectiveDesc(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm18createELFAsmParserEv", "llvm::createELFAsmParser()"},
    {"_ZN12_GLOBAL__N_112ELFAsmParserD1Ev", "(anonymous namespace)::ELFAsmParser::~ELFAsmParser()"},
    {"_ZN12_GLOBAL__N_112ELFAsmParserD0Ev", "(anonymous namespace)::ELFAsmParser::~ELFAsmParser()"},
    {"_ZN12_GLOBAL__N_112ELFAsmParser10InitializeERN4llvm11MCAsmParserE", "(anonymous namespace)::ELFAsmParser::Initialize(llvm::MCAsmParser&)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_21ParseDirectiveWeakrefENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveWeakref(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_20ParseDirectiveSymverENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveSymver(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_19ParseDirectiveIdentENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveIdent(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_18ParseDirectiveTypeENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveType(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_22ParseDirectivePreviousENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectivePrevious(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_18ParseDirectiveSizeENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveSize(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_21ParseDirectiveSectionENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseDirectiveSection(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_28ParseSectionDirectiveEhFrameENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveEhFrame(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN12_GLOBAL__N_112ELFAsmParser18ParseSectionSwitchEN4llvm9StringRefEjjNS1_11SectionKindE", "(anonymous namespace)::ELFAsmParser::ParseSectionSwitch(llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_35ParseSectionDirectiveDataRelRoLocalENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveDataRelRoLocal(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_30ParseSectionDirectiveDataRelRoENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveDataRelRo(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_28ParseSectionDirectiveDataRelENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveDataRel(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_25ParseSectionDirectiveTBSSENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveTBSS(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_26ParseSectionDirectiveTDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveTData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_27ParseSectionDirectiveRoDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveRoData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_24ParseSectionDirectiveBSSENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveBSS(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_25ParseSectionDirectiveTextENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveText(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm20MCAsmParserExtension15HandleDirectiveIN12_GLOBAL__N_112ELFAsmParserEXadL_ZNS3_25ParseSectionDirectiveDataENS_9StringRefENS_5SMLocEEEEEbPS0_S4_S5_", "bool llvm::MCAsmParserExtension::HandleDirective<(anonymous namespace)::ELFAsmParser, &((anonymous namespace)::ELFAsmParser::ParseSectionDirectiveData(llvm::StringRef, llvm::SMLoc))>(llvm::MCAsmParserExtension*, llvm::StringRef, llvm::SMLoc)"},
    {"_ZN4llvm10MCAsmLexerC1Ev", "llvm::MCAsmLexer::MCAsmLexer()"},
    {"_ZN4llvm10MCAsmLexerC2Ev", "llvm::MCAsmLexer::MCAsmLexer()"},
    {"_ZN4llvm10MCAsmLexerD0Ev", "llvm::MCAsmLexer::~MCAsmLexer()"},
    {"_ZN4llvm10MCAsmLexerD1Ev", "llvm::MCAsmLexer::~MCAsmLexer()"},
    {"_ZN4llvm10MCAsmLexerD2Ev", "llvm::MCAsmLexer::~MCAsmLexer()"},
    {"_ZNK4llvm10MCAsmLexer6getLocEv", "llvm::MCAsmLexer::getLoc() const"},
    {"_ZNK4llvm8AsmToken6getLocEv", "llvm::AsmToken::getLoc() const"},
    {"_ZN4llvm11MCAsmParserC1Ev", "llvm::MCAsmParser::MCAsmParser()"},
    {"_ZN4llvm11MCAsmParserC2Ev", "llvm::MCAsmParser::MCAsmParser()"},
    {"_ZN4llvm11MCAsmParserD0Ev", "llvm::MCAsmParser::~MCAsmParser()"},
    {"_ZN4llvm11MCAsmParserD1Ev", "llvm::MCAsmParser::~MCAsmParser()"},
    {"_ZN4llvm11MCAsmParserD2Ev", "llvm::MCAsmParser::~MCAsmParser()"},
    {"_ZN4llvm11MCAsmParser15setTargetParserERNS_15TargetAsmParserE", "llvm::MCAsmParser::setTargetParser(llvm::TargetAsmParser&)"},
    {"_ZN4llvm11MCAsmParser6getTokEv", "llvm::MCAsmParser::getTok()"},
    {"_ZN4llvm11MCAsmParser8TokErrorERKNS_5TwineE", "llvm::MCAsmParser::TokError(llvm::Twine const&)"},
    {"_ZN4llvm11MCAsmParser15ParseExpressionERPKNS_6MCExprE", "llvm::MCAsmParser::ParseExpression(llvm::MCExpr const*&)"},
    {"_ZN4llvm20MCAsmParserExtensionC1Ev", "llvm::MCAsmParserExtension::MCAsmParserExtension()"},
    {"_ZN4llvm20MCAsmParserExtensionC2Ev", "llvm::MCAsmParserExtension::MCAsmParserExtension()"},
    {"_ZN4llvm20MCAsmParserExtensionD0Ev", "llvm::MCAsmParserExtension::~MCAsmParserExtension()"},
    {"_ZN4llvm20MCAsmParserExtensionD1Ev", "llvm::MCAsmParserExtension::~MCAsmParserExtension()"},
    {"_ZN4llvm20MCAsmParserExtensionD2Ev", "llvm::MCAsmParserExtension::~MCAsmParserExtension()"},
    {"_ZN4llvm20MCAsmParserExtension10InitializeERNS_11MCAsmParserE", "llvm::MCAsmParserExtension::Initialize(llvm::MCAsmParser&)"},
    {"_ZN4llvm15TargetAsmParserC1ERKNS_6TargetE", "llvm::TargetAsmParser::TargetAsmParser(llvm::Target const&)"},
    {"_ZN4llvm15TargetAsmParserC2ERKNS_6TargetE", "llvm::TargetAsmParser::TargetAsmParser(llvm::Target const&)"},
    {"_ZN4llvm15TargetAsmParserD0Ev", "llvm::TargetAsmParser::~TargetAsmParser()"},
    {"_ZN4llvm15TargetAsmParserD1Ev", "llvm::TargetAsmParser::~TargetAsmParser()"},
    {"_ZN4llvm15TargetAsmParserD2Ev", "llvm::TargetAsmParser::~TargetAsmParser()"},
    {"_ZN4llvm22AggressiveAntiDepStateC1EjPNS_17MachineBasicBlockE", "llvm::AggressiveAntiDepState::AggressiveAntiDepState(unsigned int, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm22AggressiveAntiDepStateC2EjPNS_17MachineBasicBlockE", "llvm::AggressiveAntiDepState::AggressiveAntiDepState(unsigned int, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm22AggressiveAntiDepState8GetGroupEj", "llvm::AggressiveAntiDepState::GetGroup(unsigned int)"},
    {"_ZN4llvm22AggressiveAntiDepState12GetGroupRegsEjRSt6vectorIjSaIjEEPSt8multimapIjNS0_17RegisterReferenceESt4lessIjESaISt4pairIKjS6_EEE", "llvm::AggressiveAntiDepState::GetGroupRegs(unsigned int, std::vector<unsigned int, std::allocator<unsigned int> >&, std::multimap<unsigned int, llvm::AggressiveAntiDepState::RegisterReference, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> > >*)"},
    {"_ZN4llvm22AggressiveAntiDepState11UnionGroupsEjj", "llvm::AggressiveAntiDepState::UnionGroups(unsigned int, unsigned int)"},
    {"_ZN4llvm22AggressiveAntiDepState10LeaveGroupEj", "llvm::AggressiveAntiDepState::LeaveGroup(unsigned int)"},
    {"_ZN4llvm22AggressiveAntiDepState6IsLiveEj", "llvm::AggressiveAntiDepState::IsLive(unsigned int)"},
    {"_ZN4llvm24AggressiveAntiDepBreakerC1ERNS_15MachineFunctionERNS_15SmallVectorImplIPNS_19TargetRegisterClassEEE", "llvm::AggressiveAntiDepBreaker::AggressiveAntiDepBreaker(llvm::MachineFunction&, llvm::SmallVectorImpl<llvm::TargetRegisterClass*>&)"},
    {"_ZN4llvm24AggressiveAntiDepBreakerC2ERNS_15MachineFunctionERNS_15SmallVectorImplIPNS_19TargetRegisterClassEEE", "llvm::AggressiveAntiDepBreaker::AggressiveAntiDepBreaker(llvm::MachineFunction&, llvm::SmallVectorImpl<llvm::TargetRegisterClass*>&)"},
    {"_ZN4llvm24AggressiveAntiDepBreakerD0Ev", "llvm::AggressiveAntiDepBreaker::~AggressiveAntiDepBreaker()"},
    {"_ZN4llvm24AggressiveAntiDepBreakerD1Ev", "llvm::AggressiveAntiDepBreaker::~AggressiveAntiDepBreaker()"},
    {"_ZN4llvm24AggressiveAntiDepBreakerD2Ev", "llvm::AggressiveAntiDepBreaker::~AggressiveAntiDepBreaker()"},
    {"_ZN4llvm24AggressiveAntiDepBreaker10StartBlockEPNS_17MachineBasicBlockE", "llvm::AggressiveAntiDepBreaker::StartBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker11FinishBlockEv", "llvm::AggressiveAntiDepBreaker::FinishBlock()"},
    {"_ZN4llvm24AggressiveAntiDepBreaker7ObserveEPNS_12MachineInstrEjj", "llvm::AggressiveAntiDepBreaker::Observe(llvm::MachineInstr*, unsigned int, unsigned int)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker15GetPassthruRegsEPNS_12MachineInstrERSt3setIjSt4lessIjESaIjEE", "llvm::AggressiveAntiDepBreaker::GetPassthruRegs(llvm::MachineInstr*, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker18PrescanInstructionEPNS_12MachineInstrEjRSt3setIjSt4lessIjESaIjEE", "llvm::AggressiveAntiDepBreaker::PrescanInstruction(llvm::MachineInstr*, unsigned int, std::set<unsigned int, std::less<unsigned int>, std::allocator<unsigned int> >&)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker15ScanInstructionEPNS_12MachineInstrEj", "llvm::AggressiveAntiDepBreaker::ScanInstruction(llvm::MachineInstr*, unsigned int)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker16IsImplicitDefUseEPNS_12MachineInstrERNS_14MachineOperandE", "llvm::AggressiveAntiDepBreaker::IsImplicitDefUse(llvm::MachineInstr*, llvm::MachineOperand&)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker13HandleLastUseEjjPKcS2_S2_", "llvm::AggressiveAntiDepBreaker::HandleLastUse(unsigned int, unsigned int, char const*, char const*, char const*)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker18GetRenameRegistersEj", "llvm::AggressiveAntiDepBreaker::GetRenameRegisters(unsigned int)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker25FindSuitableFreeRegistersEjRSt3mapIPKNS_19TargetRegisterClassEPKjSt4lessIS4_ESaISt4pairIKS4_S6_EEERS1_IjjS7_IjESaIS9_IS5_jEEE", "llvm::AggressiveAntiDepBreaker::FindSuitableFreeRegisters(unsigned int, std::map<llvm::TargetRegisterClass const*, unsigned int const*, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> > >&, std::map<unsigned int, unsigned int, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, unsigned int> > >&)"},
    {"_ZN4llvm24AggressiveAntiDepBreaker21BreakAntiDependenciesERKSt6vectorINS_5SUnitESaIS2_EENS_14ilist_iteratorINS_12MachineInstrEEES9_j", "llvm::AggressiveAntiDepBreaker::BreakAntiDependencies(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> > const&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_PKNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, llvm::SUnit const*>, std::_Select1st<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> >, std::pair<llvm::MachineInstr* const, llvm::SUnit const*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_PKNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, llvm::SUnit const*>, std::_Select1st<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> > >::_M_insert_unique(std::pair<llvm::MachineInstr* const, llvm::SUnit const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_PKjESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE5eraseESt17_Rb_tree_iteratorIS8_ESG_", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, unsigned int const*>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::_Rb_tree_iterator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_PKjESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, unsigned int const*>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjjESt10_Select1stIS2_ESt4lessIjESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, unsigned int>, std::_Select1st<std::pair<unsigned int const, unsigned int> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, unsigned int> > >::_M_insert_unique(std::pair<unsigned int const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm9BitVectorEESt10_Select1stIS4_ESt4lessIjESaIS4_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS4_ERKS4_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::BitVector>, std::_Select1st<std::pair<unsigned int const, llvm::BitVector> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::BitVector> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<unsigned int const, llvm::BitVector> >, std::pair<unsigned int const, llvm::BitVector> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm9BitVectorEESt10_Select1stIS4_ESt4lessIjESaIS4_EE9_M_insertEPSt18_Rb_tree_node_baseSC_RKS4_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::BitVector>, std::_Select1st<std::pair<unsigned int const, llvm::BitVector> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::BitVector> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<unsigned int const, llvm::BitVector> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm9BitVectorEESt10_Select1stIS4_ESt4lessIjESaIS4_EE16_M_insert_uniqueERKS4_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::BitVector>, std::_Select1st<std::pair<unsigned int const, llvm::BitVector> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::BitVector> > >::_M_insert_unique(std::pair<unsigned int const, llvm::BitVector> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjjESt10_Select1stIS2_ESt4lessIjESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, unsigned int>, std::_Select1st<std::pair<unsigned int const, unsigned int> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_PKjESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, unsigned int const*>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_PKjESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, unsigned int const*>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> > >::_M_insert_unique(std::pair<llvm::TargetRegisterClass const* const, unsigned int const*> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm22AggressiveAntiDepState17RegisterReferenceEESt10_Select1stIS5_ESt4lessIjESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference>, std::_Select1st<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> > >::erase(std::_Rb_tree_iterator<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> >, std::_Rb_tree_iterator<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> >)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm22AggressiveAntiDepState17RegisterReferenceEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference>, std::_Select1st<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, llvm::AggressiveAntiDepState::RegisterReference> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_PKNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, llvm::SUnit const*>, std::_Select1st<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineInstr* const, llvm::SUnit const*> >*)"},
    {"_ZNSt6vectorIPKN4llvm4SDepESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::SDep const*, std::allocator<llvm::SDep const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SDep const**, std::vector<llvm::SDep const*, std::allocator<llvm::SDep const*> > >, llvm::SDep const* const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm9BitVectorEESt10_Select1stIS4_ESt4lessIjESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::BitVector>, std::_Select1st<std::pair<unsigned int const, llvm::BitVector> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::BitVector> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, llvm::BitVector> >*)"},
    {"_ZN4llvm18ComputeLinearIndexEPKNS_4TypeEPKjS4_j", "llvm::ComputeLinearIndex(llvm::Type const*, unsigned int const*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm15ComputeValueVTsERKNS_14TargetLoweringEPKNS_4TypeERNS_15SmallVectorImplINS_3EVTEEEPNS6_IyEEy", "llvm::ComputeValueVTs(llvm::TargetLowering const&, llvm::Type const*, llvm::SmallVectorImpl<llvm::EVT>&, llvm::SmallVectorImpl<unsigned long long>*, unsigned long long)"},
    {"_ZN4llvm15ExtractTypeInfoEPNS_5ValueE", "llvm::ExtractTypeInfo(llvm::Value*)"},
    {"_ZN4llvm25hasInlineAsmMemConstraintERSt6vectorINS_9InlineAsm14ConstraintInfoESaIS2_EERKNS_14TargetLoweringE", "llvm::hasInlineAsmMemConstraint(std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >&, llvm::TargetLowering const&)"},
    {"_ZN4llvm15getFCmpCondCodeENS_7CmpInst9PredicateE", "llvm::getFCmpCondCode(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm15getICmpCondCodeENS_7CmpInst9PredicateE", "llvm::getICmpCondCode(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm20isInTailCallPositionENS_17ImmutableCallSiteEjRKNS_14TargetLoweringE", "llvm::isInTailCallPosition(llvm::ImmutableCallSite, unsigned int, llvm::TargetLowering const&)"},
    {"_ZN4llvm20isInTailCallPositionERNS_12SelectionDAGEPNS_6SDNodeERKNS_14TargetLoweringE", "llvm::isInTailCallPosition(llvm::SelectionDAG&, llvm::SDNode*, llvm::TargetLowering const&)"},
    {"_ZN4llvm23createBranchFoldingPassEb", "llvm::createBranchFoldingPass(bool)"},
    {"_ZN4llvm12BranchFolderC1Eb", "llvm::BranchFolder::BranchFolder(bool)"},
    {"_ZN4llvm12BranchFolderC2Eb", "llvm::BranchFolder::BranchFolder(bool)"},
    {"_ZN4llvm12BranchFolder15RemoveDeadBlockEPNS_17MachineBasicBlockE", "llvm::BranchFolder::RemoveDeadBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12BranchFolder20OptimizeImpDefsBlockEPNS_17MachineBasicBlockE", "llvm::BranchFolder::OptimizeImpDefsBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12BranchFolder16OptimizeFunctionERNS_15MachineFunctionEPKNS_15TargetInstrInfoEPKNS_18TargetRegisterInfoEPNS_17MachineModuleInfoE", "llvm::BranchFolder::OptimizeFunction(llvm::MachineFunction&, llvm::TargetInstrInfo const*, llvm::TargetRegisterInfo const*, llvm::MachineModuleInfo*)"},
    {"_ZN4llvm12BranchFolder15TailMergeBlocksERNS_15MachineFunctionE", "llvm::BranchFolder::TailMergeBlocks(llvm::MachineFunction&)"},
    {"_ZN4llvm12BranchFolder16OptimizeBranchesERNS_15MachineFunctionE", "llvm::BranchFolder::OptimizeBranches(llvm::MachineFunction&)"},
    {"_ZN4llvm12BranchFolder23ReplaceTailWithBranchToENS_14ilist_iteratorINS_12MachineInstrEEEPNS_17MachineBasicBlockE", "llvm::BranchFolder::ReplaceTailWithBranchTo(llvm::ilist_iterator<llvm::MachineInstr>, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12BranchFolder10SplitMBBAtERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::BranchFolder::SplitMBBAt(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZNK4llvm12BranchFolder18MergePotentialsEltltERKS1_", "llvm::BranchFolder::MergePotentialsElt::operator<(llvm::BranchFolder::MergePotentialsElt const&) const"},
    {"_ZN4llvm12BranchFolder16ComputeSameTailsEjjPNS_17MachineBasicBlockES2_", "llvm::BranchFolder::ComputeSameTails(unsigned int, unsigned int, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12BranchFolder20RemoveBlocksWithHashEjPNS_17MachineBasicBlockES2_", "llvm::BranchFolder::RemoveBlocksWithHash(unsigned int, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_Z7FixTailPN4llvm17MachineBasicBlockES1_PKNS_15TargetInstrInfoE", "FixTail(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::TargetInstrInfo const*)"},
    {"_ZN4llvm12BranchFolder25CreateCommonTailOnlyBlockERPNS_17MachineBasicBlockEjRj", "llvm::BranchFolder::CreateCommonTailOnlyBlock(llvm::MachineBasicBlock*&, unsigned int, unsigned int&)"},
    {"_ZN4llvm12BranchFolder18TryTailMergeBlocksEPNS_17MachineBasicBlockES2_", "llvm::BranchFolder::TryTailMergeBlocks(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_Z12HashEndOfMBBPKN4llvm17MachineBasicBlockE", "HashEndOfMBB(llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm12BranchFolder13OptimizeBlockEPNS_17MachineBasicBlockE", "llvm::BranchFolder::OptimizeBlock(llvm::MachineBasicBlock*)"},
    {"_ZN12_GLOBAL__N_116BranchFolderPassD1Ev", "(anonymous namespace)::BranchFolderPass::~BranchFolderPass()"},
    {"_ZN12_GLOBAL__N_116BranchFolderPassD0Ev", "(anonymous namespace)::BranchFolderPass::~BranchFolderPass()"},
    {"_ZNK12_GLOBAL__N_116BranchFolderPass11getPassNameEv", "(anonymous namespace)::BranchFolderPass::getPassName() const"},
    {"_ZN12_GLOBAL__N_116BranchFolderPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::BranchFolderPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNSt6vectorIN4llvm12BranchFolder18MergePotentialsEltESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt const&)"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >)"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEES5_lEvT_SA_T0_T1_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*, long>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*, long)"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEES5_EvT_SA_T0_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*)"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEElS5_EvT_SA_SA_T0_SB_T1_SB_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, long, llvm::BranchFolder::MergePotentialsElt*>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, long, long, llvm::BranchFolder::MergePotentialsElt*, long)"},
    {"_ZSt5mergeIPN4llvm12BranchFolder18MergePotentialsEltEN9__gnu_cxx17__normal_iteratorIS3_St6vectorIS2_SaIS2_EEEES9_ET1_T_SB_T0_SC_SA_", "__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > std::merge<llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >)"},
    {"_ZSt16__merge_backwardIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEES5_S9_ET1_T_SB_T0_SC_SA_", "__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > std::__merge_backward<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >)"},
    {"_ZSt17__rotate_adaptiveIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEES5_lET_SA_SA_SA_T1_SB_T0_SB_", "__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > std::__rotate_adaptive<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*, long>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, long, long, llvm::BranchFolder::MergePotentialsElt*, long)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEEEvT_SA_SA_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, std::random_access_iterator_tag)"},
    {"_ZSt5mergeIPN4llvm12BranchFolder18MergePotentialsEltES3_N9__gnu_cxx17__normal_iteratorIS3_St6vectorIS2_SaIS2_EEEEET1_T_SB_T0_SC_SA_", "__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > std::merge<llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, llvm::BranchFolder::MergePotentialsElt*, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >)"},
    {"_ZSt5mergeIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEES9_S5_ET1_T_SB_T0_SC_SA_", "llvm::BranchFolder::MergePotentialsElt* std::merge<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, llvm::BranchFolder::MergePotentialsElt*)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > > >(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >)"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPN4llvm12BranchFolder18MergePotentialsEltESt6vectorIS4_SaIS4_EEEElEvT_SA_SA_T0_SB_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, long>(__gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, __gnu_cxx::__normal_iterator<llvm::BranchFolder::MergePotentialsElt*, std::vector<llvm::BranchFolder::MergePotentialsElt, std::allocator<llvm::BranchFolder::MergePotentialsElt> > >, long, long)"},
    {"_ZNSt6vectorIN4llvm12BranchFolder11SameTailEltESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BranchFolder::SameTailElt, std::allocator<llvm::BranchFolder::SameTailElt> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BranchFolder::SameTailElt*, std::vector<llvm::BranchFolder::SameTailElt, std::allocator<llvm::BranchFolder::SameTailElt> > >, llvm::BranchFolder::SameTailElt const&)"},
    {"_ZN4llvm35initializeCalculateSpillWeightsPassERNS_12PassRegistryE", "llvm::initializeCalculateSpillWeightsPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm21CalculateSpillWeights16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::CalculateSpillWeights::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm21CalculateSpillWeights20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::CalculateSpillWeights::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm14VirtRegAuxInfo22CalculateWeightAndHintERNS_12LiveIntervalE", "llvm::VirtRegAuxInfo::CalculateWeightAndHint(llvm::LiveInterval&)"},
    {"_ZN4llvm14VirtRegAuxInfo17CalculateRegClassEj", "llvm::VirtRegAuxInfo::CalculateRegClass(unsigned int)"},
    {"_ZN4llvm21CalculateSpillWeightsD1Ev", "llvm::CalculateSpillWeights::~CalculateSpillWeights()"},
    {"_ZN4llvm21CalculateSpillWeightsD0Ev", "llvm::CalculateSpillWeights::~CalculateSpillWeights()"},
    {"_ZN4llvm8DenseMapIjfNS_12DenseMapInfoIjEENS1_IfEEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, float, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<float> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIjfNS_12DenseMapInfoIjEENS1_IfEEE16InsertIntoBucketERKjRKfPSt4pairIjfE", "llvm::DenseMap<unsigned int, float, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<float> >::InsertIntoBucket(unsigned int const&, float const&, std::pair<unsigned int, float>*)"},
    {"_ZN4llvm8DenseMapIjfNS_12DenseMapInfoIjEENS1_IfEEE4growEj", "llvm::DenseMap<unsigned int, float, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<float> >::grow(unsigned int)"},
    {"_ZN4llvm15callDefaultCtorINS_21CalculateSpillWeightsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::CalculateSpillWeights>()"},
    {"_ZN4llvm7CCStateC1ENS_11CallingConv2IDEbRKNS_13TargetMachineERNS_11SmallVectorINS_11CCValAssignELj16EEERNS_11LLVMContextE", "llvm::CCState::CCState(llvm::CallingConv::ID, bool, llvm::TargetMachine const&, llvm::SmallVector<llvm::CCValAssign, 16u>&, llvm::LLVMContext&)"},
    {"_ZN4llvm7CCStateC2ENS_11CallingConv2IDEbRKNS_13TargetMachineERNS_11SmallVectorINS_11CCValAssignELj16EEERNS_11LLVMContextE", "llvm::CCState::CCState(llvm::CallingConv::ID, bool, llvm::TargetMachine const&, llvm::SmallVector<llvm::CCValAssign, 16u>&, llvm::LLVMContext&)"},
    {"_ZN4llvm7CCState11HandleByValEjNS_3MVTES1_NS_11CCValAssign7LocInfoEiiNS_3ISD10ArgFlagsTyE", "llvm::CCState::HandleByVal(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, int, int, llvm::ISD::ArgFlagsTy)"},
    {"_ZN4llvm7CCState13MarkAllocatedEj", "llvm::CCState::MarkAllocated(unsigned int)"},
    {"_ZN4llvm7CCState22AnalyzeFormalArgumentsERKNS_15SmallVectorImplINS_3ISD8InputArgEEEPFbjNS_3MVTES7_NS_11CCValAssign7LocInfoENS2_10ArgFlagsTyERS0_E", "llvm::CCState::AnalyzeFormalArguments(llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState11CheckReturnERKNS_15SmallVectorImplINS_3ISD9OutputArgEEEPFbjNS_3MVTES7_NS_11CCValAssign7LocInfoENS2_10ArgFlagsTyERS0_E", "llvm::CCState::CheckReturn(llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState13AnalyzeReturnERKNS_15SmallVectorImplINS_3ISD9OutputArgEEEPFbjNS_3MVTES7_NS_11CCValAssign7LocInfoENS2_10ArgFlagsTyERS0_E", "llvm::CCState::AnalyzeReturn(llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState19AnalyzeCallOperandsERKNS_15SmallVectorImplINS_3ISD9OutputArgEEEPFbjNS_3MVTES7_NS_11CCValAssign7LocInfoENS2_10ArgFlagsTyERS0_E", "llvm::CCState::AnalyzeCallOperands(llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState19AnalyzeCallOperandsERNS_15SmallVectorImplINS_3MVTEEERNS1_INS_3ISD10ArgFlagsTyEEEPFbjS2_S2_NS_11CCValAssign7LocInfoES6_RS0_E", "llvm::CCState::AnalyzeCallOperands(llvm::SmallVectorImpl<llvm::MVT>&, llvm::SmallVectorImpl<llvm::ISD::ArgFlagsTy>&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState17AnalyzeCallResultERKNS_15SmallVectorImplINS_3ISD8InputArgEEEPFbjNS_3MVTES7_NS_11CCValAssign7LocInfoENS2_10ArgFlagsTyERS0_E", "llvm::CCState::AnalyzeCallResult(llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm7CCState17AnalyzeCallResultENS_3MVTEPFbjS1_S1_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERS0_E", "llvm::CCState::AnalyzeCallResult(llvm::MVT, bool (*)(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&))"},
    {"_ZN4llvm26createCodePlacementOptPassEv", "llvm::createCodePlacementOptPass()"},
    {"_ZN12_GLOBAL__N_116CodePlacementOptD1Ev", "(anonymous namespace)::CodePlacementOpt::~CodePlacementOpt()"},
    {"_ZN12_GLOBAL__N_116CodePlacementOptD0Ev", "(anonymous namespace)::CodePlacementOpt::~CodePlacementOpt()"},
    {"_ZNK12_GLOBAL__N_116CodePlacementOpt11getPassNameEv", "(anonymous namespace)::CodePlacementOpt::getPassName() const"},
    {"_ZNK12_GLOBAL__N_116CodePlacementOpt16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::CodePlacementOpt::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_116CodePlacementOpt20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::CodePlacementOpt::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_116CodePlacementOpt9AlignLoopERN4llvm15MachineFunctionEPNS1_11MachineLoopEj", "(anonymous namespace)::CodePlacementOpt::AlignLoop(llvm::MachineFunction&, llvm::MachineLoop*, unsigned int)"},
    {"_ZN12_GLOBAL__N_116CodePlacementOpt32OptimizeIntraLoopEdgesInLoopNestERN4llvm15MachineFunctionEPNS1_11MachineLoopE", "(anonymous namespace)::CodePlacementOpt::OptimizeIntraLoopEdgesInLoopNest(llvm::MachineFunction&, llvm::MachineLoop*)"},
    {"_ZN12_GLOBAL__N_116CodePlacementOpt23HasAnalyzableTerminatorEPN4llvm17MachineBasicBlockE", "(anonymous namespace)::CodePlacementOpt::HasAnalyzableTerminator(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm22CriticalAntiDepBreakerC1ERNS_15MachineFunctionE", "llvm::CriticalAntiDepBreaker::CriticalAntiDepBreaker(llvm::MachineFunction&)"},
    {"_ZN4llvm22CriticalAntiDepBreakerC2ERNS_15MachineFunctionE", "llvm::CriticalAntiDepBreaker::CriticalAntiDepBreaker(llvm::MachineFunction&)"},
    {"_ZN4llvm22CriticalAntiDepBreakerD0Ev", "llvm::CriticalAntiDepBreaker::~CriticalAntiDepBreaker()"},
    {"_ZN4llvm22CriticalAntiDepBreakerD1Ev", "llvm::CriticalAntiDepBreaker::~CriticalAntiDepBreaker()"},
    {"_ZN4llvm22CriticalAntiDepBreakerD2Ev", "llvm::CriticalAntiDepBreaker::~CriticalAntiDepBreaker()"},
    {"_ZN4llvm22CriticalAntiDepBreaker10StartBlockEPNS_17MachineBasicBlockE", "llvm::CriticalAntiDepBreaker::StartBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm22CriticalAntiDepBreaker11FinishBlockEv", "llvm::CriticalAntiDepBreaker::FinishBlock()"},
    {"_ZN4llvm22CriticalAntiDepBreaker7ObserveEPNS_12MachineInstrEjj", "llvm::CriticalAntiDepBreaker::Observe(llvm::MachineInstr*, unsigned int, unsigned int)"},
    {"_ZN4llvm22CriticalAntiDepBreaker18PrescanInstructionEPNS_12MachineInstrE", "llvm::CriticalAntiDepBreaker::PrescanInstruction(llvm::MachineInstr*)"},
    {"_ZN4llvm22CriticalAntiDepBreaker15ScanInstructionEPNS_12MachineInstrEj", "llvm::CriticalAntiDepBreaker::ScanInstruction(llvm::MachineInstr*, unsigned int)"},
    {"_ZN4llvm22CriticalAntiDepBreaker23isNewRegClobberedByRefsESt23_Rb_tree_const_iteratorISt4pairIKjPNS_14MachineOperandEEES7_j", "llvm::CriticalAntiDepBreaker::isNewRegClobberedByRefs(std::_Rb_tree_const_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >, std::_Rb_tree_const_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >, unsigned int)"},
    {"_ZN4llvm22CriticalAntiDepBreaker24findSuitableFreeRegisterESt23_Rb_tree_const_iteratorISt4pairIKjPNS_14MachineOperandEEES7_jjPKNS_19TargetRegisterClassE", "llvm::CriticalAntiDepBreaker::findSuitableFreeRegister(std::_Rb_tree_const_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >, std::_Rb_tree_const_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >, unsigned int, unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm22CriticalAntiDepBreaker21BreakAntiDependenciesERKSt6vectorINS_5SUnitESaIS2_EENS_14ilist_iteratorINS_12MachineInstrEEES9_j", "llvm::CriticalAntiDepBreaker::BreakAntiDependencies(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> > const&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPKNS_5SUnitENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::MachineInstr*, llvm::SUnit const*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::SUnit const*> >::InsertIntoBucket(llvm::MachineInstr* const&, llvm::SUnit const* const&, std::pair<llvm::MachineInstr*, llvm::SUnit const*>*)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPKNS_5SUnitENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr*, llvm::SUnit const*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::SUnit const*> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN4llvm14MachineOperandEESt10_Select1stIS5_ESt4lessIjESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::MachineOperand*>, std::_Select1st<std::pair<unsigned int const, llvm::MachineOperand*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::MachineOperand*> > >::erase(std::_Rb_tree_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >, std::_Rb_tree_iterator<std::pair<unsigned int const, llvm::MachineOperand*> >)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjPN4llvm14MachineOperandEESt10_Select1stIS5_ESt4lessIjESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::MachineOperand*>, std::_Select1st<std::pair<unsigned int const, llvm::MachineOperand*> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::MachineOperand*> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, llvm::MachineOperand*> >*)"},
    {"_ZN4llvm40initializeDeadMachineInstructionElimPassERNS_12PassRegistryE", "llvm::initializeDeadMachineInstructionElimPass(llvm::PassRegistry&)"},
    {"_ZN4llvm36createDeadMachineInstructionElimPassEv", "llvm::createDeadMachineInstructionElimPass()"},
    {"_ZN12_GLOBAL__N_126DeadMachineInstructionElimD1Ev", "(anonymous namespace)::DeadMachineInstructionElim::~DeadMachineInstructionElim()"},
    {"_ZN12_GLOBAL__N_126DeadMachineInstructionElimD0Ev", "(anonymous namespace)::DeadMachineInstructionElim::~DeadMachineInstructionElim()"},
    {"_ZN12_GLOBAL__N_126DeadMachineInstructionElim20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::DeadMachineInstructionElim::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_126DeadMachineInstructionElimEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::DeadMachineInstructionElim>()"},
    {"_ZN4llvm17createDwarfEHPassEPKNS_13TargetMachineE", "llvm::createDwarfEHPass(llvm::TargetMachine const*)"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepareD1Ev", "(anonymous namespace)::DwarfEHPrepare::~DwarfEHPrepare()"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepareD0Ev", "(anonymous namespace)::DwarfEHPrepare::~DwarfEHPrepare()"},
    {"_ZNK12_GLOBAL__N_114DwarfEHPrepare11getPassNameEv", "(anonymous namespace)::DwarfEHPrepare::getPassName() const"},
    {"_ZNK12_GLOBAL__N_114DwarfEHPrepare16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::DwarfEHPrepare::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepare13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::DwarfEHPrepare::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepare16CleanupSelectorsERN4llvm11SmallPtrSetIPNS1_13IntrinsicInstELj32EEE", "(anonymous namespace)::DwarfEHPrepare::CleanupSelectors(llvm::SmallPtrSet<llvm::IntrinsicInst*, 32u>&)"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepare19FindSelectorAndURoREPN4llvm11InstructionERbRNS1_11SmallPtrSetIPNS1_13IntrinsicInstELj8EEE", "(anonymous namespace)::DwarfEHPrepare::FindSelectorAndURoR(llvm::Instruction*, bool&, llvm::SmallPtrSet<llvm::IntrinsicInst*, 8u>&)"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepare24CreateExceptionValueCallEPN4llvm10BasicBlockE", "(anonymous namespace)::DwarfEHPrepare::CreateExceptionValueCall(llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE5SplitINS_7InverseIPS1_EENS_11GraphTraitsIS6_EEEEvRNS0_INT0_8NodeTypeEEEPSA_", "void llvm::DominatorTreeBase<llvm::BasicBlock>::Split<llvm::Inverse<llvm::BasicBlock*>, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE5SplitIPS1_NS_11GraphTraitsIS4_EEEEvRNS0_INT0_8NodeTypeEEEPS8_", "void llvm::DominatorTreeBase<llvm::BasicBlock>::Split<llvm::BasicBlock*, llvm::GraphTraits<llvm::BasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::BasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::BasicBlock*>::NodeType*)"},
    {"_ZNSt6vectorIPN4llvm15DomTreeNodeBaseINS0_10BasicBlockEEESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::BasicBlock>* const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm15DomTreeNodeBaseINS2_10BasicBlockEEESt6vectorIS6_SaIS6_EEEES6_ET_SC_SC_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::BasicBlock>*>(__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::BasicBlock>* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::DomTreeNodeBase<llvm::BasicBlock>* const&, std::pair<llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock* const&)"},
    {"_ZN4llvm25initializeEdgeBundlesPassERNS_12PassRegistryE", "llvm::initializeEdgeBundlesPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm11EdgeBundles16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::EdgeBundles::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm11EdgeBundles20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::EdgeBundles::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNK4llvm11EdgeBundles4viewEv", "llvm::EdgeBundles::view() const"},
    {"_ZN4llvm10WriteGraphERNS_11raw_ostreamERKNS_11EdgeBundlesEbRKSs", "llvm::WriteGraph(llvm::raw_ostream&, llvm::EdgeBundles const&, bool, std::string const&)"},
    {"_ZN4llvm11EdgeBundlesD1Ev", "llvm::EdgeBundles::~EdgeBundles()"},
    {"_ZN4llvm11EdgeBundlesD0Ev", "llvm::EdgeBundles::~EdgeBundles()"},
    {"_ZN4llvm10WriteGraphINS_11EdgeBundlesEEENS_3sys4PathERKT_RKSsbS8_", "llvm::sys::Path llvm::WriteGraph<llvm::EdgeBundles>(llvm::EdgeBundles const&, std::string const&, bool, std::string const&)"},
    {"_ZN4llvm15callDefaultCtorINS_11EdgeBundlesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::EdgeBundles>()"},
    {"_ZN4llvm31initializeExpandISelPseudosPassERNS_12PassRegistryE", "llvm::initializeExpandISelPseudosPass(llvm::PassRegistry&)"},
    {"_ZN4llvm27createExpandISelPseudosPassEv", "llvm::createExpandISelPseudosPass()"},
    {"_ZN12_GLOBAL__N_117ExpandISelPseudosD1Ev", "(anonymous namespace)::ExpandISelPseudos::~ExpandISelPseudos()"},
    {"_ZN12_GLOBAL__N_117ExpandISelPseudosD0Ev", "(anonymous namespace)::ExpandISelPseudos::~ExpandISelPseudos()"},
    {"_ZNK12_GLOBAL__N_117ExpandISelPseudos11getPassNameEv", "(anonymous namespace)::ExpandISelPseudos::getPassName() const"},
    {"_ZNK12_GLOBAL__N_117ExpandISelPseudos16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::ExpandISelPseudos::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_117ExpandISelPseudos20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::ExpandISelPseudos::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_117ExpandISelPseudosEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::ExpandISelPseudos>()"},
    {"_ZN4llvm26initializeGCModuleInfoPassERNS_12PassRegistryE", "llvm::initializeGCModuleInfoPass(llvm::PassRegistry&)"},
    {"_ZN4llvm14GCFunctionInfoC1ERKNS_8FunctionERNS_10GCStrategyE", "llvm::GCFunctionInfo::GCFunctionInfo(llvm::Function const&, llvm::GCStrategy&)"},
    {"_ZN4llvm14GCFunctionInfoC2ERKNS_8FunctionERNS_10GCStrategyE", "llvm::GCFunctionInfo::GCFunctionInfo(llvm::Function const&, llvm::GCStrategy&)"},
    {"_ZN4llvm14GCFunctionInfoD1Ev", "llvm::GCFunctionInfo::~GCFunctionInfo()"},
    {"_ZN4llvm14GCFunctionInfoD2Ev", "llvm::GCFunctionInfo::~GCFunctionInfo()"},
    {"_ZN4llvm12GCModuleInfoC1Ev", "llvm::GCModuleInfo::GCModuleInfo()"},
    {"_ZN4llvm12GCModuleInfoC2Ev", "llvm::GCModuleInfo::GCModuleInfo()"},
    {"_ZN4llvm12GCModuleInfoD0Ev", "llvm::GCModuleInfo::~GCModuleInfo()"},
    {"_ZN4llvm12GCModuleInfoD1Ev", "llvm::GCModuleInfo::~GCModuleInfo()"},
    {"_ZN4llvm12GCModuleInfoD2Ev", "llvm::GCModuleInfo::~GCModuleInfo()"},
    {"_ZN4llvm12GCModuleInfo5clearEv", "llvm::GCModuleInfo::clear()"},
    {"_ZN4llvm12GCModuleInfo19getOrCreateStrategyEPKNS_6ModuleERKSs", "llvm::GCModuleInfo::getOrCreateStrategy(llvm::Module const*, std::string const&)"},
    {"_ZN4llvm12GCModuleInfo15getFunctionInfoERKNS_8FunctionE", "llvm::GCModuleInfo::getFunctionInfo(llvm::Function const&)"},
    {"_ZN4llvm19createGCInfoPrinterERNS_11raw_ostreamE", "llvm::createGCInfoPrinter(llvm::raw_ostream&)"},
    {"_ZN4llvm19createGCInfoDeleterEv", "llvm::createGCInfoDeleter()"},
    {"_ZN12_GLOBAL__N_17DeleterD1Ev", "(anonymous namespace)::Deleter::~Deleter()"},
    {"_ZN12_GLOBAL__N_17DeleterD0Ev", "(anonymous namespace)::Deleter::~Deleter()"},
    {"_ZNK12_GLOBAL__N_17Deleter11getPassNameEv", "(anonymous namespace)::Deleter::getPassName() const"},
    {"_ZNK12_GLOBAL__N_17Deleter16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::Deleter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_17Deleter13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::Deleter::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_17Deleter14doFinalizationERN4llvm6ModuleE", "(anonymous namespace)::Deleter::doFinalization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_17PrinterD1Ev", "(anonymous namespace)::Printer::~Printer()"},
    {"_ZN12_GLOBAL__N_17PrinterD0Ev", "(anonymous namespace)::Printer::~Printer()"},
    {"_ZNK12_GLOBAL__N_17Printer11getPassNameEv", "(anonymous namespace)::Printer::getPassName() const"},
    {"_ZNK12_GLOBAL__N_17Printer16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::Printer::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_17Printer13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::Printer::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionEPNS_14GCFunctionInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Function const*, llvm::GCFunctionInfo*, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::GCFunctionInfo*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionEPNS_14GCFunctionInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::Function const*, llvm::GCFunctionInfo*, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::GCFunctionInfo*> >::InsertIntoBucket(llvm::Function const* const&, llvm::GCFunctionInfo* const&, std::pair<llvm::Function const*, llvm::GCFunctionInfo*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionEPNS_14GCFunctionInfoENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::Function const*, llvm::GCFunctionInfo*, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::GCFunctionInfo*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm10GCStrategyESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::GCStrategy*, std::allocator<llvm::GCStrategy*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GCStrategy**, std::vector<llvm::GCStrategy*, std::allocator<llvm::GCStrategy*> > >, llvm::GCStrategy* const&)"},
    {"_ZN4llvm9StringMapIPNS_10GCStrategyENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::GCStrategy*>& llvm::StringMap<llvm::GCStrategy*, llvm::MallocAllocator>::GetOrCreateValue<llvm::GCStrategy*>(llvm::StringRef, llvm::GCStrategy*)"},
    {"_ZN4llvm15callDefaultCtorINS_12GCModuleInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::GCModuleInfo>()"},
    {"_ZN4llvm10GCStrategyC1Ev", "llvm::GCStrategy::GCStrategy()"},
    {"_ZN4llvm10GCStrategyC2Ev", "llvm::GCStrategy::GCStrategy()"},
    {"_ZN4llvm10GCStrategyD0Ev", "llvm::GCStrategy::~GCStrategy()"},
    {"_ZN4llvm10GCStrategyD1Ev", "llvm::GCStrategy::~GCStrategy()"},
    {"_ZN4llvm10GCStrategyD2Ev", "llvm::GCStrategy::~GCStrategy()"},
    {"_ZN4llvm10GCStrategy24initializeCustomLoweringERNS_6ModuleE", "llvm::GCStrategy::initializeCustomLowering(llvm::Module&)"},
    {"_ZN4llvm10GCStrategy21performCustomLoweringERNS_8FunctionE", "llvm::GCStrategy::performCustomLowering(llvm::Function&)"},
    {"_ZN4llvm10GCStrategy18insertFunctionInfoERKNS_8FunctionE", "llvm::GCStrategy::insertFunctionInfo(llvm::Function const&)"},
    {"_ZN4llvm29initializeLowerIntrinsicsPassERNS_12PassRegistryE", "llvm::initializeLowerIntrinsicsPass(llvm::PassRegistry&)"},
    {"_ZN4llvm20createGCLoweringPassEv", "llvm::createGCLoweringPass()"},
    {"_ZN4llvm31createGCMachineCodeAnalysisPassEv", "llvm::createGCMachineCodeAnalysisPass()"},
    {"_ZN12_GLOBAL__N_119MachineCodeAnalysisD1Ev", "(anonymous namespace)::MachineCodeAnalysis::~MachineCodeAnalysis()"},
    {"_ZN12_GLOBAL__N_119MachineCodeAnalysisD0Ev", "(anonymous namespace)::MachineCodeAnalysis::~MachineCodeAnalysis()"},
    {"_ZNK12_GLOBAL__N_119MachineCodeAnalysis11getPassNameEv", "(anonymous namespace)::MachineCodeAnalysis::getPassName() const"},
    {"_ZNK12_GLOBAL__N_119MachineCodeAnalysis16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineCodeAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_119MachineCodeAnalysis20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineCodeAnalysis::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNK12_GLOBAL__N_119MachineCodeAnalysis11InsertLabelERN4llvm17MachineBasicBlockENS1_14ilist_iteratorINS1_12MachineInstrEEENS1_8DebugLocE", "(anonymous namespace)::MachineCodeAnalysis::InsertLabel(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::DebugLoc) const"},
    {"_ZN12_GLOBAL__N_115LowerIntrinsicsD1Ev", "(anonymous namespace)::LowerIntrinsics::~LowerIntrinsics()"},
    {"_ZN12_GLOBAL__N_115LowerIntrinsicsD0Ev", "(anonymous namespace)::LowerIntrinsics::~LowerIntrinsics()"},
    {"_ZNK12_GLOBAL__N_115LowerIntrinsics11getPassNameEv", "(anonymous namespace)::LowerIntrinsics::getPassName() const"},
    {"_ZNK12_GLOBAL__N_115LowerIntrinsics16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LowerIntrinsics::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_115LowerIntrinsics16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::LowerIntrinsics::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_115LowerIntrinsics13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::LowerIntrinsics::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_115LowerIntrinsicsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LowerIntrinsics>()"},
    {"_ZNSt6vectorIPN4llvm14GCFunctionInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::GCFunctionInfo*, std::allocator<llvm::GCFunctionInfo*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GCFunctionInfo**, std::vector<llvm::GCFunctionInfo*, std::allocator<llvm::GCFunctionInfo*> > >, llvm::GCFunctionInfo* const&)"},
    {"_ZNSt6vectorIN4llvm7GCPointESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::GCPoint, std::allocator<llvm::GCPoint> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GCPoint*, std::vector<llvm::GCPoint, std::allocator<llvm::GCPoint> > >, llvm::GCPoint const&)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE11recalculateINS_8FunctionEEEvRT_", "void llvm::DominatorTreeBase<llvm::BasicBlock>::recalculate<llvm::Function>(llvm::Function&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::BasicBlock* const&, std::pair<llvm::BasicBlock*, llvm::BasicBlock*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockES2_NS_12DenseMapInfoIS2_EES4_E16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >::shrink_and_clear()"},
    {"_ZN4llvm25initializeIfConverterPassERNS_12PassRegistryE", "llvm::initializeIfConverterPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createIfConverterPassEv", "llvm::createIfConverterPass()"},
    {"_ZN12_GLOBAL__N_111IfConverterD1Ev", "(anonymous namespace)::IfConverter::~IfConverter()"},
    {"_ZN12_GLOBAL__N_111IfConverterD0Ev", "(anonymous namespace)::IfConverter::~IfConverter()"},
    {"_ZNK12_GLOBAL__N_111IfConverter11getPassNameEv", "(anonymous namespace)::IfConverter::getPassName() const"},
    {"_ZNK12_GLOBAL__N_111IfConverter16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::IfConverter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111IfConverter20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::IfConverter::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_111IfConverter14PredicateBlockERNS0_6BBInfoEN4llvm14ilist_iteratorINS3_12MachineInstrEEERNS3_15SmallVectorImplINS3_14MachineOperandEEERNS3_8SmallSetIjLj4EEE", "(anonymous namespace)::IfConverter::PredicateBlock((anonymous namespace)::IfConverter::BBInfo&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::SmallVectorImpl<llvm::MachineOperand>&, llvm::SmallSet<unsigned int, 4u>&)"},
    {"_ZN12_GLOBAL__N_111IfConverter11MergeBlocksERNS0_6BBInfoES2_b", "(anonymous namespace)::IfConverter::MergeBlocks((anonymous namespace)::IfConverter::BBInfo&, (anonymous namespace)::IfConverter::BBInfo&, bool)"},
    {"_ZN12_GLOBAL__N_111IfConverter6BBInfoC1ERKS1_", "(anonymous namespace)::IfConverter::BBInfo::BBInfo((anonymous namespace)::IfConverter::BBInfo const&)"},
    {"_ZN12_GLOBAL__N_111IfConverter16RemoveExtraEdgesERNS0_6BBInfoE", "(anonymous namespace)::IfConverter::RemoveExtraEdges((anonymous namespace)::IfConverter::BBInfo&)"},
    {"_Z16UpdatePredRedefsPN4llvm12MachineInstrERNS_8SmallSetIjLj4EEEPKNS_18TargetRegisterInfoEb", "UpdatePredRedefs(llvm::MachineInstr*, llvm::SmallSet<unsigned int, 4u>&, llvm::TargetRegisterInfo const*, bool)"},
    {"_ZN12_GLOBAL__N_111IfConverter21CopyAndPredicateBlockERNS0_6BBInfoES2_RN4llvm15SmallVectorImplINS3_14MachineOperandEEERNS3_8SmallSetIjLj4EEEb", "(anonymous namespace)::IfConverter::CopyAndPredicateBlock((anonymous namespace)::IfConverter::BBInfo&, (anonymous namespace)::IfConverter::BBInfo&, llvm::SmallVectorImpl<llvm::MachineOperand>&, llvm::SmallSet<unsigned int, 4u>&, bool)"},
    {"_ZN12_GLOBAL__N_111IfConverter12AnalyzeBlockEPN4llvm17MachineBasicBlockERSt6vectorIPNS0_10IfcvtTokenESaIS6_EE", "(anonymous namespace)::IfConverter::AnalyzeBlock(llvm::MachineBasicBlock*, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> >&)"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEEPFbS5_S5_EEvT_SD_T0_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEES6_lPFbS5_S5_EEvT_SD_T0_T1_T2_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, long, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, long, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEES6_PFbS5_S5_EEvT_SD_T0_T1_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEElS6_PFbS5_S5_EEvT_SD_SD_T0_SE_T1_SE_T2_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, long, (anonymous namespace)::IfConverter::IfcvtToken**, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, long, long, (anonymous namespace)::IfConverter::IfcvtToken**, long, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt11lower_boundIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEES5_PFbS5_S5_EET_SD_SD_RKT0_T1_", "__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > > std::lower_bound<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken*, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken* const&, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt11upper_boundIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEES5_PFbS5_S5_EET_SD_SD_RKT0_T1_", "__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > > std::upper_bound<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken*, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken* const&, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt6rotateIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEEEvT_SB_SB_", "void std::rotate<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > > >(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >)"},
    {"_ZSt5mergeIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenES4_N9__gnu_cxx17__normal_iteratorIS4_St6vectorIS3_SaIS3_EEEEPFbS3_S3_EET1_T_SE_T0_SF_SD_T2_", "__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > > std::merge<(anonymous namespace)::IfConverter::IfcvtToken**, (anonymous namespace)::IfConverter::IfcvtToken**, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>((anonymous namespace)::IfConverter::IfcvtToken**, (anonymous namespace)::IfConverter::IfcvtToken**, (anonymous namespace)::IfConverter::IfcvtToken**, (anonymous namespace)::IfConverter::IfcvtToken**, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt5mergeIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEESA_S6_PFbS5_S5_EET1_T_SE_T0_SF_SD_T2_", "(anonymous namespace)::IfConverter::IfcvtToken** std::merge<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, (anonymous namespace)::IfConverter::IfcvtToken**, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEEPFbS5_S5_EEvT_SD_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPPN12_GLOBAL__N_111IfConverter10IfcvtTokenESt6vectorIS5_SaIS5_EEEElPFbS5_S5_EEvT_SD_SD_T0_SE_T1_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, long, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*)>(__gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::IfConverter::IfcvtToken**, std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> > >, long, long, bool (*)((anonymous namespace)::IfConverter::IfcvtToken*, (anonymous namespace)::IfConverter::IfcvtToken*))"},
    {"_ZN12_GLOBAL__N_111IfConverter19FeasibilityAnalysisERNS0_6BBInfoERN4llvm15SmallVectorImplINS3_14MachineOperandEEEbb", "(anonymous namespace)::IfConverter::FeasibilityAnalysis((anonymous namespace)::IfConverter::BBInfo&, llvm::SmallVectorImpl<llvm::MachineOperand>&, bool, bool)"},
    {"_ZNSt6vectorIPN12_GLOBAL__N_111IfConverter10IfcvtTokenESaIS3_EE9push_backERKS3_", "std::vector<(anonymous namespace)::IfConverter::IfcvtToken*, std::allocator<(anonymous namespace)::IfConverter::IfcvtToken*> >::push_back((anonymous namespace)::IfConverter::IfcvtToken* const&)"},
    {"_ZNK12_GLOBAL__N_111IfConverter13ValidTriangleERNS0_6BBInfoES2_bRjff", "(anonymous namespace)::IfConverter::ValidTriangle((anonymous namespace)::IfConverter::BBInfo&, (anonymous namespace)::IfConverter::BBInfo&, bool, unsigned int&, float, float) const"},
    {"_ZSt4fillIPN12_GLOBAL__N_111IfConverter6BBInfoES2_EvT_S4_RKT0_", "void std::fill<(anonymous namespace)::IfConverter::BBInfo*, (anonymous namespace)::IfConverter::BBInfo>((anonymous namespace)::IfConverter::BBInfo*, (anonymous namespace)::IfConverter::BBInfo*, (anonymous namespace)::IfConverter::BBInfo const&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111IfConverterEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::IfConverter>()"},
    {"_ZN4llvm15SmallVectorImplINS_14MachineOperandEEaSERKS2_", "llvm::SmallVectorImpl<llvm::MachineOperand>::operator=(llvm::SmallVectorImpl<llvm::MachineOperand> const&)"},
    {"_ZN4llvm8SmallSetIjLj4EE5eraseERKj", "llvm::SmallSet<unsigned int, 4u>::erase(unsigned int const&)"},
    {"_ZNSt8_Rb_treeIjjSt9_IdentityIjESt4lessIjESaIjEE5eraseESt17_Rb_tree_iteratorIjES7_", "std::_Rb_tree<unsigned int, unsigned int, std::_Identity<unsigned int>, std::less<unsigned int>, std::allocator<unsigned int> >::erase(std::_Rb_tree_iterator<unsigned int>, std::_Rb_tree_iterator<unsigned int>)"},
    {"_ZN4llvm13idf_ext_beginIPNS_17MachineBasicBlockESt3setIS2_St4lessIS2_ESaIS2_EEEENS_16idf_ext_iteratorIT_T0_EERKS9_RSA_", "llvm::idf_ext_iterator<llvm::MachineBasicBlock*, std::set<llvm::MachineBasicBlock*, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> > > llvm::idf_ext_begin<llvm::MachineBasicBlock*, std::set<llvm::MachineBasicBlock*, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> > >(llvm::MachineBasicBlock* const&, std::set<llvm::MachineBasicBlock*, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> >&)"},
    {"_ZNSt8_Rb_treeIPN4llvm17MachineBasicBlockES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, std::_Identity<llvm::MachineBasicBlock*>, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> >::_M_erase(std::_Rb_tree_node<llvm::MachineBasicBlock*>*)"},
    {"_ZN4llvm11df_iteratorINS_7InverseIPNS_17MachineBasicBlockEEESt3setIS3_St4lessIS3_ESaIS3_EELb1ENS_11GraphTraitsIS4_EEE6toNextEv", "llvm::df_iterator<llvm::Inverse<llvm::MachineBasicBlock*>, std::set<llvm::MachineBasicBlock*, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> >, true, llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> > >::toNext()"},
    {"_ZNSt8_Rb_treeIPN4llvm17MachineBasicBlockES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, std::_Identity<llvm::MachineBasicBlock*>, std::less<llvm::MachineBasicBlock*>, std::allocator<llvm::MachineBasicBlock*> >::_M_insert_unique(llvm::MachineBasicBlock* const&)"},
    {"_ZN4llvm19createInlineSpillerERNS_19MachineFunctionPassERNS_15MachineFunctionERNS_10VirtRegMapE", "llvm::createInlineSpiller(llvm::MachineFunctionPass&, llvm::MachineFunction&, llvm::VirtRegMap&)"},
    {"_ZN12_GLOBAL__N_113InlineSpillerD1Ev", "(anonymous namespace)::InlineSpiller::~InlineSpiller()"},
    {"_ZN12_GLOBAL__N_113InlineSpillerD0Ev", "(anonymous namespace)::InlineSpiller::~InlineSpiller()"},
    {"_ZN12_GLOBAL__N_113InlineSpiller5spillEPN4llvm12LiveIntervalERNS1_15SmallVectorImplIS3_EERKS5_", "(anonymous namespace)::InlineSpiller::spill(llvm::LiveInterval*, llvm::SmallVectorImpl<llvm::LiveInterval*>&, llvm::SmallVectorImpl<llvm::LiveInterval*> const&)"},
    {"_ZN12_GLOBAL__N_113InlineSpiller17foldMemoryOperandEN4llvm14ilist_iteratorINS1_12MachineInstrEEERKNS1_15SmallVectorImplIjEEPS3_", "(anonymous namespace)::InlineSpiller::foldMemoryOperand(llvm::ilist_iterator<llvm::MachineInstr>, llvm::SmallVectorImpl<unsigned int> const&, llvm::MachineInstr*)"},
    {"_ZN4llvm17IntrinsicLowering13AddPrototypesERNS_6ModuleE", "llvm::IntrinsicLowering::AddPrototypes(llvm::Module&)"},
    {"_Z20EnsureFunctionExistsIN4llvm14ilist_iteratorINS0_8ArgumentEEEEvRNS0_6ModuleEPKcT_S8_PKNS0_4TypeE", "void EnsureFunctionExists<llvm::ilist_iterator<llvm::Argument> >(llvm::Module&, char const*, llvm::ilist_iterator<llvm::Argument>, llvm::ilist_iterator<llvm::Argument>, llvm::Type const*)"},
    {"_Z23EnsureFPIntrinsicsExistRN4llvm6ModuleEPNS_8FunctionEPKcS5_S5_", "EnsureFPIntrinsicsExist(llvm::Module&, llvm::Function*, char const*, char const*, char const*)"},
    {"_ZN4llvm17IntrinsicLowering18LowerIntrinsicCallEPNS_8CallInstE", "llvm::IntrinsicLowering::LowerIntrinsicCall(llvm::CallInst*)"},
    {"_Z15ReplaceCallWithIPN4llvm3UseEEPNS0_8CallInstEPKcS4_T_S7_PKNS0_4TypeE", "llvm::CallInst* ReplaceCallWith<llvm::Use*>(char const*, llvm::CallInst*, llvm::Use*, llvm::Use*, llvm::Type const*)"},
    {"_Z10LowerCTPOPRN4llvm11LLVMContextEPNS_5ValueEPNS_11InstructionE", "LowerCTPOP(llvm::LLVMContext&, llvm::Value*, llvm::Instruction*)"},
    {"_Z15ReplaceCallWithIPPN4llvm5ValueEEPNS0_8CallInstEPKcS5_T_S8_PKNS0_4TypeE", "llvm::CallInst* ReplaceCallWith<llvm::Value**>(char const*, llvm::CallInst*, llvm::Value**, llvm::Value**, llvm::Type const*)"},
    {"_Z26ReplaceFPIntrinsicWithCallPN4llvm8CallInstEPKcS3_S3_", "ReplaceFPIntrinsicWithCall(llvm::CallInst*, char const*, char const*, char const*)"},
    {"_ZN4llvm17IntrinsicLowering15LowerToByteSwapEPNS_8CallInstE", "llvm::IntrinsicLowering::LowerToByteSwap(llvm::CallInst*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateNotEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateNot(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateSubEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateAndEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateAnd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE13CreateIntCastEPNS_5ValueEPKNS_4TypeEbRKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateIntCast(llvm::Value*, llvm::Type const*, bool, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateCallIPPNS_5ValueEEEPNS_8CallInstES7_T_SB_RKNS_5TwineE", "llvm::CallInst* llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCall<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateAddEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm17LLVMTargetMachineC1ERKNS_6TargetERKSs", "llvm::LLVMTargetMachine::LLVMTargetMachine(llvm::Target const&, std::string const&)"},
    {"_ZN4llvm17LLVMTargetMachineC2ERKNS_6TargetERKSs", "llvm::LLVMTargetMachine::LLVMTargetMachine(llvm::Target const&, std::string const&)"},
    {"_ZN4llvm17LLVMTargetMachine18setCodeModelForJITEv", "llvm::LLVMTargetMachine::setCodeModelForJIT()"},
    {"_ZN4llvm17LLVMTargetMachine21setCodeModelForStaticEv", "llvm::LLVMTargetMachine::setCodeModelForStatic()"},
    {"_ZN4llvm17LLVMTargetMachine19addPassesToEmitFileERNS_15PassManagerBaseERNS_21formatted_raw_ostreamENS_13TargetMachine15CodeGenFileTypeENS_10CodeGenOpt5LevelEb", "llvm::LLVMTargetMachine::addPassesToEmitFile(llvm::PassManagerBase&, llvm::formatted_raw_ostream&, llvm::TargetMachine::CodeGenFileType, llvm::CodeGenOpt::Level, bool)"},
    {"_ZN4llvm17LLVMTargetMachine22addCommonCodeGenPassesERNS_15PassManagerBaseENS_10CodeGenOpt5LevelEbRPNS_9MCContextE", "llvm::LLVMTargetMachine::addCommonCodeGenPasses(llvm::PassManagerBase&, llvm::CodeGenOpt::Level, bool, llvm::MCContext*&)"},
    {"_ZN4llvm17LLVMTargetMachine26addPassesToEmitMachineCodeERNS_15PassManagerBaseERNS_14JITCodeEmitterENS_10CodeGenOpt5LevelEb", "llvm::LLVMTargetMachine::addPassesToEmitMachineCode(llvm::PassManagerBase&, llvm::JITCodeEmitter&, llvm::CodeGenOpt::Level, bool)"},
    {"_ZN4llvm17LLVMTargetMachine17addPassesToEmitMCERNS_15PassManagerBaseERPNS_9MCContextENS_10CodeGenOpt5LevelEb", "llvm::LLVMTargetMachine::addPassesToEmitMC(llvm::PassManagerBase&, llvm::MCContext*&, llvm::CodeGenOpt::Level, bool)"},
    {"_Z14printAndVerifyRN4llvm15PassManagerBaseEPKc", "printAndVerify(llvm::PassManagerBase&, char const*)"},
    {"_Z13printNoVerifyRN4llvm15PassManagerBaseEPKc", "printNoVerify(llvm::PassManagerBase&, char const*)"},
    {"_ZN4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEED1Ev", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::~opt()"},
    {"_ZN4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEE16handleOccurrenceEjNS_9StringRefES6_", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEE14getOptionWidthEv", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEE15printOptionInfoEm", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEED0Ev", "llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >::~opt()"},
    {"_ZNK4llvm12latency_sortclEPKNS_5SUnitES3_", "llvm::latency_sort::operator()(llvm::SUnit const*, llvm::SUnit const*) const"},
    {"_ZN4llvm20LatencyPriorityQueue24getSingleUnscheduledPredEPNS_5SUnitE", "llvm::LatencyPriorityQueue::getSingleUnscheduledPred(llvm::SUnit*)"},
    {"_ZN4llvm20LatencyPriorityQueue4pushEPNS_5SUnitE", "llvm::LatencyPriorityQueue::push(llvm::SUnit*)"},
    {"_ZN4llvm20LatencyPriorityQueue13ScheduledNodeEPNS_5SUnitE", "llvm::LatencyPriorityQueue::ScheduledNode(llvm::SUnit*)"},
    {"_ZN4llvm20LatencyPriorityQueue32AdjustPriorityOfUnscheduledPredsEPNS_5SUnitE", "llvm::LatencyPriorityQueue::AdjustPriorityOfUnscheduledPreds(llvm::SUnit*)"},
    {"_ZN4llvm20LatencyPriorityQueue3popEv", "llvm::LatencyPriorityQueue::pop()"},
    {"_ZN4llvm20LatencyPriorityQueue6removeEPNS_5SUnitE", "llvm::LatencyPriorityQueue::remove(llvm::SUnit*)"},
    {"_ZNK4llvm20LatencyPriorityQueue4dumpEPNS_11ScheduleDAGE", "llvm::LatencyPriorityQueue::dump(llvm::ScheduleDAG*) const"},
    {"_ZN4llvm32initializeLiveDebugVariablesPassERNS_12PassRegistryE", "llvm::initializeLiveDebugVariablesPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm18LiveDebugVariables16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LiveDebugVariables::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm18LiveDebugVariablesC1Ev", "llvm::LiveDebugVariables::LiveDebugVariables()"},
    {"_ZN4llvm18LiveDebugVariablesC2Ev", "llvm::LiveDebugVariables::LiveDebugVariables()"},
    {"_ZN4llvm18LiveDebugVariables20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::LiveDebugVariables::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm18LiveDebugVariables13releaseMemoryEv", "llvm::LiveDebugVariables::releaseMemory()"},
    {"_ZN12_GLOBAL__N_17LDVImpl5clearEv", "(anonymous namespace)::LDVImpl::clear()"},
    {"_ZN4llvm18LiveDebugVariablesD0Ev", "llvm::LiveDebugVariables::~LiveDebugVariables()"},
    {"_ZN4llvm18LiveDebugVariablesD1Ev", "llvm::LiveDebugVariables::~LiveDebugVariables()"},
    {"_ZN4llvm18LiveDebugVariablesD2Ev", "llvm::LiveDebugVariables::~LiveDebugVariables()"},
    {"_ZN4llvm18LiveDebugVariables14renameRegisterEjjj", "llvm::LiveDebugVariables::renameRegister(unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm18LiveDebugVariables15emitDebugValuesEPNS_10VirtRegMapE", "llvm::LiveDebugVariables::emitDebugValues(llvm::VirtRegMap*)"},
    {"_ZN12_GLOBAL__N_19UserValue16insertDebugValueEPN4llvm17MachineBasicBlockENS1_9SlotIndexEjRNS1_13LiveIntervalsERKNS1_15TargetInstrInfoE", "(anonymous namespace)::UserValue::insertDebugValue(llvm::MachineBasicBlock*, llvm::SlotIndex, unsigned int, llvm::LiveIntervals&, llvm::TargetInstrInfo const&)"},
    {"_Z18findInsertLocationPN4llvm17MachineBasicBlockENS_9SlotIndexERNS_13LiveIntervalsE", "findInsertLocation(llvm::MachineBasicBlock*, llvm::SlotIndex, llvm::LiveIntervals&)"},
    {"_ZN12_GLOBAL__N_19UserValue16coalesceLocationEj", "(anonymous namespace)::UserValue::coalesceLocation(unsigned int)"},
    {"_ZN12_GLOBAL__N_17LDVImpl10mapVirtRegEjPNS_9UserValueE", "(anonymous namespace)::LDVImpl::mapVirtReg(unsigned int, (anonymous namespace)::UserValue*)"},
    {"_ZN12_GLOBAL__N_19UserValue9extendDefEN4llvm9SlotIndexEjPNS1_12LiveIntervalEPKNS1_6VNInfoERNS1_13LiveIntervalsERNS1_20MachineDominatorTreeE", "(anonymous namespace)::UserValue::extendDef(llvm::SlotIndex, unsigned int, llvm::LiveInterval*, llvm::VNInfo const*, llvm::LiveIntervals&, llvm::MachineDominatorTree&)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE14const_iterator7setRootEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::const_iterator::setRoot(unsigned int)"},
    {"_ZN4llvm15IntervalMapImpl4Path8fillLeftEj", "llvm::IntervalMapImpl::Path::fillLeft(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15IntervalMapImpl4Path5EntryELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::IntervalMapImpl::Path::Entry, false>::grow(unsigned long)"},
    {"_ZN4llvm15SmallVectorImplINS_15IntervalMapImpl4Path5EntryEEaSERKS4_", "llvm::SmallVectorImpl<llvm::IntervalMapImpl::Path::Entry>::operator=(llvm::SmallVectorImpl<llvm::IntervalMapImpl::Path::Entry> const&)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator8setValueEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::setValue(unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator16canCoalesceRightES1_j", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::canCoalesceRight(llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator15canCoalesceLeftES1_j", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::canCoalesceLeft(llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator9treeEraseEb", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::treeErase(bool)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator9eraseNodeEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::eraseNode(unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE16switchRootToLeafEv", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::switchRootToLeaf()"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE10visitNodesEMS4_FvNS_15IntervalMapImpl7NodeRefEjE", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::visitNodes(void (llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::*)(llvm::IntervalMapImpl::NodeRef, unsigned int))"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE10deleteNodeENS_15IntervalMapImpl7NodeRefEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::deleteNode(llvm::IntervalMapImpl::NodeRef, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplINS_15IntervalMapImpl7NodeRefEE4swapERS3_", "llvm::SmallVectorImpl<llvm::IntervalMapImpl::NodeRef>::swap(llvm::SmallVectorImpl<llvm::IntervalMapImpl::NodeRef>&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15IntervalMapImpl7NodeRefELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::IntervalMapImpl::NodeRef, false>::grow(unsigned long)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator6insertES1_S1_j", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::insert(llvm::SlotIndex, llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator10treeInsertES1_S1_j", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::treeInsert(llvm::SlotIndex, llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm15IntervalMapImpl8LeafNodeINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS2_EEE10insertFromERjjS2_S2_j", "llvm::IntervalMapImpl::LeafNode<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::insertFrom(unsigned int&, unsigned int, llvm::SlotIndex, llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE10branchRootEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::branchRoot(unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE18switchRootToBranchEv", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::switchRootToBranch()"},
    {"_ZN4llvm15IntervalMapImpl8NodeBaseISt4pairINS_9SlotIndexES3_EjLj9EEC2Ev", "llvm::IntervalMapImpl::NodeBase<std::pair<llvm::SlotIndex, llvm::SlotIndex>, unsigned int, 9u>::NodeBase()"},
    {"_ZN4llvm15IntervalMapImpl8LeafNodeINS_9SlotIndexEjLj9ENS_15IntervalMapInfoIS2_EEE10insertFromERjjS2_S2_j", "llvm::IntervalMapImpl::LeafNode<llvm::SlotIndex, unsigned int, 9u, llvm::IntervalMapInfo<llvm::SlotIndex> >::insertFrom(unsigned int&, unsigned int, llvm::SlotIndex, llvm::SlotIndex, unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator8overflowINS_15IntervalMapImpl8LeafNodeIS1_jLj9ES3_EEEEbj", "bool llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::overflow<llvm::IntervalMapImpl::LeafNode<llvm::SlotIndex, unsigned int, 9u, llvm::IntervalMapInfo<llvm::SlotIndex> > >(unsigned int)"},
    {"_ZN4llvm15IntervalMapImpl18adjustSiblingSizesINS0_8LeafNodeINS_9SlotIndexEjLj9ENS_15IntervalMapInfoIS3_EEEEEEvPPT_jPjPKj", "void llvm::IntervalMapImpl::adjustSiblingSizes<llvm::IntervalMapImpl::LeafNode<llvm::SlotIndex, unsigned int, 9u, llvm::IntervalMapInfo<llvm::SlotIndex> > >(llvm::IntervalMapImpl::LeafNode<llvm::SlotIndex, unsigned int, 9u, llvm::IntervalMapInfo<llvm::SlotIndex> >**, unsigned int, unsigned int*, unsigned int const*)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator10insertNodeEjNS_15IntervalMapImpl7NodeRefES1_", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::insertNode(unsigned int, llvm::IntervalMapImpl::NodeRef, llvm::SlotIndex)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE9splitRootEj", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::splitRoot(unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE8iterator8overflowINS_15IntervalMapImpl10BranchNodeIS1_jLj12ES3_EEEEbj", "bool llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::iterator::overflow<llvm::IntervalMapImpl::BranchNode<llvm::SlotIndex, unsigned int, 12u, llvm::IntervalMapInfo<llvm::SlotIndex> > >(unsigned int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE7newNodeINS_15IntervalMapImpl10BranchNodeIS1_jLj12ES3_EEEEPT_v", "llvm::IntervalMapImpl::BranchNode<llvm::SlotIndex, unsigned int, 12u, llvm::IntervalMapInfo<llvm::SlotIndex> >* llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::newNode<llvm::IntervalMapImpl::BranchNode<llvm::SlotIndex, unsigned int, 12u, llvm::IntervalMapInfo<llvm::SlotIndex> > >()"},
    {"_ZN4llvm15IntervalMapImpl18adjustSiblingSizesINS0_10BranchNodeINS_9SlotIndexEjLj12ENS_15IntervalMapInfoIS3_EEEEEEvPPT_jPjPKj", "void llvm::IntervalMapImpl::adjustSiblingSizes<llvm::IntervalMapImpl::BranchNode<llvm::SlotIndex, unsigned int, 12u, llvm::IntervalMapInfo<llvm::SlotIndex> > >(llvm::IntervalMapImpl::BranchNode<llvm::SlotIndex, unsigned int, 12u, llvm::IntervalMapInfo<llvm::SlotIndex> >**, unsigned int, unsigned int*, unsigned int const*)"},
    {"_ZN4llvm15IntervalMapImpl8NodeBaseINS0_7NodeRefENS_9SlotIndexELj12EE17adjustFromLeftSibEjRS4_ji", "llvm::IntervalMapImpl::NodeBase<llvm::IntervalMapImpl::NodeRef, llvm::SlotIndex, 12u>::adjustFromLeftSib(unsigned int, llvm::IntervalMapImpl::NodeBase<llvm::IntervalMapImpl::NodeRef, llvm::SlotIndex, 12u>&, unsigned int, int)"},
    {"_ZN4llvm15IntervalMapImpl8NodeBaseISt4pairINS_9SlotIndexES3_EjLj9EE17adjustFromLeftSibEjRS5_ji", "llvm::IntervalMapImpl::NodeBase<std::pair<llvm::SlotIndex, llvm::SlotIndex>, unsigned int, 9u>::adjustFromLeftSib(unsigned int, llvm::IntervalMapImpl::NodeBase<std::pair<llvm::SlotIndex, llvm::SlotIndex>, unsigned int, 9u>&, unsigned int, int)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE14const_iterator4findES1_", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::const_iterator::find(llvm::SlotIndex)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE14const_iterator8treeFindES1_", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::const_iterator::treeFind(llvm::SlotIndex)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEE14const_iterator12pathFillFindES1_", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::const_iterator::pathFillFind(llvm::SlotIndex)"},
    {"_ZN4llvm11IntervalMapINS_9SlotIndexEjLj4ENS_15IntervalMapInfoIS1_EEEC2ERNS_18RecyclingAllocatorINS_16BumpPtrAllocatorEcLm192ELm64EEE", "llvm::IntervalMap<llvm::SlotIndex, unsigned int, 4u, llvm::IntervalMapInfo<llvm::SlotIndex> >::IntervalMap(llvm::RecyclingAllocator<llvm::BumpPtrAllocator, char, 192ul, 64ul>&)"},
    {"_ZN4llvm15callDefaultCtorINS_18LiveDebugVariablesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LiveDebugVariables>()"},
    {"_ZN4llvm12LiveInterval4findENS_9SlotIndexE", "llvm::LiveInterval::find(llvm::SlotIndex)"},
    {"_ZNK4llvm12LiveInterval13killedInRangeENS_9SlotIndexES1_", "llvm::LiveInterval::killedInRange(llvm::SlotIndex, llvm::SlotIndex) const"},
    {"_ZNK4llvm12LiveInterval12overlapsFromERKS0_PKNS_9LiveRangeE", "llvm::LiveInterval::overlapsFrom(llvm::LiveInterval const&, llvm::LiveRange const*) const"},
    {"_ZNK4llvm12LiveInterval8overlapsENS_9SlotIndexES1_", "llvm::LiveInterval::overlaps(llvm::SlotIndex, llvm::SlotIndex) const"},
    {"_ZN4llvm12LiveInterval20markValNoForDeletionEPNS_6VNInfoE", "llvm::LiveInterval::markValNoForDeletion(llvm::VNInfo*)"},
    {"_ZN4llvm12LiveInterval14RenumberValuesERNS_13LiveIntervalsE", "llvm::LiveInterval::RenumberValues(llvm::LiveIntervals&)"},
    {"_ZN4llvm12LiveInterval19extendIntervalEndToEPNS_9LiveRangeENS_9SlotIndexE", "llvm::LiveInterval::extendIntervalEndTo(llvm::LiveRange*, llvm::SlotIndex)"},
    {"_ZN4llvm12LiveInterval21extendIntervalStartToEPNS_9LiveRangeENS_9SlotIndexE", "llvm::LiveInterval::extendIntervalStartTo(llvm::LiveRange*, llvm::SlotIndex)"},
    {"_ZN4llvm12LiveInterval12addRangeFromENS_9LiveRangeEPS1_", "llvm::LiveInterval::addRangeFrom(llvm::LiveRange, llvm::LiveRange*)"},
    {"_ZN4llvm12LiveInterval11removeRangeENS_9SlotIndexES1_b", "llvm::LiveInterval::removeRange(llvm::SlotIndex, llvm::SlotIndex, bool)"},
    {"_ZN4llvm12LiveInterval11removeValNoEPNS_6VNInfoE", "llvm::LiveInterval::removeValNo(llvm::VNInfo*)"},
    {"_ZNK4llvm12LiveInterval26findDefinedVNInfoForRegIntENS_9SlotIndexE", "llvm::LiveInterval::findDefinedVNInfoForRegInt(llvm::SlotIndex) const"},
    {"_ZN4llvm12LiveInterval4joinERS0_PKiS3_RNS_11SmallVectorIPNS_6VNInfoELj16EEEPNS_19MachineRegisterInfoE", "llvm::LiveInterval::join(llvm::LiveInterval&, int const*, int const*, llvm::SmallVector<llvm::VNInfo*, 16u>&, llvm::MachineRegisterInfo*)"},
    {"_ZN4llvm12LiveInterval19ComputeJoinedWeightERKS0_", "llvm::LiveInterval::ComputeJoinedWeight(llvm::LiveInterval const&)"},
    {"_ZN4llvm12LiveInterval20MergeRangesInAsValueERKS0_PNS_6VNInfoE", "llvm::LiveInterval::MergeRangesInAsValue(llvm::LiveInterval const&, llvm::VNInfo*)"},
    {"_ZN4llvm12LiveInterval19MergeValueInAsValueERKS0_PKNS_6VNInfoEPS3_", "llvm::LiveInterval::MergeValueInAsValue(llvm::LiveInterval const&, llvm::VNInfo const*, llvm::VNInfo*)"},
    {"_ZN4llvm12LiveInterval20MergeValueNumberIntoEPNS_6VNInfoES2_", "llvm::LiveInterval::MergeValueNumberInto(llvm::VNInfo*, llvm::VNInfo*)"},
    {"_ZN4llvm12LiveInterval4CopyERKS0_PNS_19MachineRegisterInfoERNS_16BumpPtrAllocatorE", "llvm::LiveInterval::Copy(llvm::LiveInterval const&, llvm::MachineRegisterInfo*, llvm::BumpPtrAllocator&)"},
    {"_ZNK4llvm12LiveInterval7getSizeEv", "llvm::LiveInterval::getSize() const"},
    {"_ZN4llvmlsERNS_11raw_ostreamERKNS_9LiveRangeE", "llvm::operator<<(llvm::raw_ostream&, llvm::LiveRange const&)"},
    {"_ZNK4llvm9LiveRange4dumpEv", "llvm::LiveRange::dump() const"},
    {"_ZNK4llvm12LiveInterval5printERNS_11raw_ostreamEPKNS_18TargetRegisterInfoE", "llvm::LiveInterval::print(llvm::raw_ostream&, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm12LiveInterval4dumpEv", "llvm::LiveInterval::dump() const"},
    {"_ZNK4llvm9LiveRange5printERNS_11raw_ostreamE", "llvm::LiveRange::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm24ConnectedVNInfoEqClasses8ClassifyEPKNS_12LiveIntervalE", "llvm::ConnectedVNInfoEqClasses::Classify(llvm::LiveInterval const*)"},
    {"_ZN4llvm24ConnectedVNInfoEqClasses10DistributeEPPNS_12LiveIntervalE", "llvm::ConnectedVNInfoEqClasses::Distribute(llvm::LiveInterval**)"},
    {"_ZN4llvm15SmallVectorImplINS_9LiveRangeEE6insertEPS1_RKS1_", "llvm::SmallVectorImpl<llvm::LiveRange>::insert(llvm::LiveRange*, llvm::LiveRange const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_6VNInfoEE6resizeEj", "llvm::SmallVectorImpl<llvm::VNInfo*>::resize(unsigned int)"},
    {"_ZNK4llvm12LiveInterval23FindLiveRangeContainingENS_9SlotIndexE", "llvm::LiveInterval::FindLiveRangeContaining(llvm::SlotIndex) const"},
    {"_ZN4llvm27initializeLiveIntervalsPassERNS_12PassRegistryE", "llvm::initializeLiveIntervalsPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm13LiveIntervals16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LiveIntervals::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13LiveIntervals13releaseMemoryEv", "llvm::LiveIntervals::releaseMemory()"},
    {"_ZN4llvm13LiveIntervals20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::LiveIntervals::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm13LiveIntervals16computeIntervalsEv", "llvm::LiveIntervals::computeIntervals()"},
    {"_ZNK4llvm13LiveIntervals5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::LiveIntervals::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm13LiveIntervals11printInstrsERNS_11raw_ostreamE", "llvm::LiveIntervals::printInstrs(llvm::raw_ostream&) const"},
    {"_ZNK4llvm13LiveIntervals10dumpInstrsEv", "llvm::LiveIntervals::dumpInstrs() const"},
    {"_ZN4llvm13LiveIntervals20conflictsWithPhysRegERKNS_12LiveIntervalERNS_10VirtRegMapEj", "llvm::LiveIntervals::conflictsWithPhysReg(llvm::LiveInterval const&, llvm::VirtRegMap&, unsigned int)"},
    {"_ZN4llvm13LiveIntervals21conflictsWithAliasRefERNS_12LiveIntervalEjRNS_11SmallPtrSetIPNS_12MachineInstrELj32EEE", "llvm::LiveIntervals::conflictsWithAliasRef(llvm::LiveInterval&, unsigned int, llvm::SmallPtrSet<llvm::MachineInstr*, 32u>&)"},
    {"_ZN4llvm13LiveIntervals14isPartialRedefENS_9SlotIndexERNS_14MachineOperandERNS_12LiveIntervalE", "llvm::LiveIntervals::isPartialRedef(llvm::SlotIndex, llvm::MachineOperand&, llvm::LiveInterval&)"},
    {"_ZN4llvm13LiveIntervals24handleVirtualRegisterDefEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_9SlotIndexERNS_14MachineOperandEjRNS_12LiveIntervalE", "llvm::LiveIntervals::handleVirtualRegisterDef(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>, llvm::SlotIndex, llvm::MachineOperand&, unsigned int, llvm::LiveInterval&)"},
    {"_ZN4llvm13LiveIntervals25handlePhysicalRegisterDefEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_9SlotIndexERNS_14MachineOperandERNS_12LiveIntervalEPS4_", "llvm::LiveIntervals::handlePhysicalRegisterDef(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>, llvm::SlotIndex, llvm::MachineOperand&, llvm::LiveInterval&, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveIntervals17handleRegisterDefEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEENS_9SlotIndexERNS_14MachineOperandEj", "llvm::LiveIntervals::handleRegisterDef(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>, llvm::SlotIndex, llvm::MachineOperand&, unsigned int)"},
    {"_ZN4llvm13LiveIntervals20handleLiveInRegisterEPNS_17MachineBasicBlockENS_9SlotIndexERNS_12LiveIntervalEb", "llvm::LiveIntervals::handleLiveInRegister(llvm::MachineBasicBlock*, llvm::SlotIndex, llvm::LiveInterval&, bool)"},
    {"_ZN4llvm13LiveIntervals14createIntervalEj", "llvm::LiveIntervals::createInterval(unsigned int)"},
    {"_ZN4llvm13LiveIntervals11dupIntervalEPNS_12LiveIntervalE", "llvm::LiveIntervals::dupInterval(llvm::LiveInterval*)"},
    {"_ZN4llvm13LiveIntervals12shrinkToUsesEPNS_12LiveIntervalE", "llvm::LiveIntervals::shrinkToUses(llvm::LiveInterval*)"},
    {"_ZNK4llvm13LiveIntervals17getLastSplitPointERKNS_12LiveIntervalEPNS_17MachineBasicBlockE", "llvm::LiveIntervals::getLastSplitPoint(llvm::LiveInterval const&, llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm13LiveIntervals12addKillFlagsEv", "llvm::LiveIntervals::addKillFlags()"},
    {"_ZNK4llvm13LiveIntervals19getReMatImplicitUseERKNS_12LiveIntervalEPNS_12MachineInstrE", "llvm::LiveIntervals::getReMatImplicitUse(llvm::LiveInterval const&, llvm::MachineInstr*) const"},
    {"_ZNK4llvm13LiveIntervals18isValNoAvailableAtERKNS_12LiveIntervalEPNS_12MachineInstrENS_9SlotIndexE", "llvm::LiveIntervals::isValNoAvailableAt(llvm::LiveInterval const&, llvm::MachineInstr*, llvm::SlotIndex) const"},
    {"_ZN4llvm13LiveIntervals18isReMaterializableERKNS_12LiveIntervalEPKNS_6VNInfoEPNS_12MachineInstrERKNS_15SmallVectorImplIPS1_EERb", "llvm::LiveIntervals::isReMaterializable(llvm::LiveInterval const&, llvm::VNInfo const*, llvm::MachineInstr*, llvm::SmallVectorImpl<llvm::LiveInterval*> const&, bool&)"},
    {"_ZN4llvm13LiveIntervals18isReMaterializableERKNS_12LiveIntervalEPKNS_6VNInfoEPNS_12MachineInstrE", "llvm::LiveIntervals::isReMaterializable(llvm::LiveInterval const&, llvm::VNInfo const*, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveIntervals18isReMaterializableERKNS_12LiveIntervalERKNS_15SmallVectorImplIPS1_EERb", "llvm::LiveIntervals::isReMaterializable(llvm::LiveInterval const&, llvm::SmallVectorImpl<llvm::LiveInterval*> const&, bool&)"},
    {"_ZN4llvm13LiveIntervals20tryFoldMemoryOperandERPNS_12MachineInstrERNS_10VirtRegMapES2_NS_9SlotIndexERNS_11SmallVectorIjLj2EEEbij", "llvm::LiveIntervals::tryFoldMemoryOperand(llvm::MachineInstr*&, llvm::VirtRegMap&, llvm::MachineInstr*, llvm::SlotIndex, llvm::SmallVector<unsigned int, 2u>&, bool, int, unsigned int)"},
    {"_Z15FilterFoldedOpsPN4llvm12MachineInstrERNS_11SmallVectorIjLj2EEERjS4_", "FilterFoldedOps(llvm::MachineInstr*, llvm::SmallVector<unsigned int, 2u>&, unsigned int&, llvm::SmallVector<unsigned int, 2u>&)"},
    {"_ZNK4llvm13LiveIntervals20canFoldMemoryOperandEPNS_12MachineInstrERNS_11SmallVectorIjLj2EEEb", "llvm::LiveIntervals::canFoldMemoryOperand(llvm::MachineInstr*, llvm::SmallVector<unsigned int, 2u>&, bool) const"},
    {"_ZNK4llvm13LiveIntervals18intervalIsInOneMBBERKNS_12LiveIntervalE", "llvm::LiveIntervals::intervalIsInOneMBB(llvm::LiveInterval const&) const"},
    {"_ZN4llvm13LiveIntervals18rewriteImplicitOpsERKNS_12LiveIntervalEPNS_12MachineInstrEjRNS_10VirtRegMapE", "llvm::LiveIntervals::rewriteImplicitOps(llvm::LiveInterval const&, llvm::MachineInstr*, unsigned int, llvm::VirtRegMap&)"},
    {"_ZN4llvm13LiveIntervals27rewriteInstructionForSpillsERKNS_12LiveIntervalEPKNS_6VNInfoEbNS_9SlotIndexES7_PNS_12MachineInstrES9_S9_jibbbbRNS_10VirtRegMapEPKNS_19TargetRegisterClassERNS_11SmallVectorIiLj4EEEPKNS_15MachineLoopInfoERjjRbSM_RNS_8DenseMapIjjNS_12DenseMapInfoIjEESP_EERSt6vectorIPS1_SaIST_EE", "llvm::LiveIntervals::rewriteInstructionForSpills(llvm::LiveInterval const&, llvm::VNInfo const*, bool, llvm::SlotIndex, llvm::SlotIndex, llvm::MachineInstr*, llvm::MachineInstr*, llvm::MachineInstr*, unsigned int, int, bool, bool, bool, bool, llvm::VirtRegMap&, llvm::TargetRegisterClass const*, llvm::SmallVector<int, 4u>&, llvm::MachineLoopInfo const*, unsigned int&, unsigned int, bool&, bool&, llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >&, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> >&)"},
    {"_ZNK4llvm13LiveIntervals20anyKillInMBBAfterIdxERKNS_12LiveIntervalEPKNS_6VNInfoEPNS_17MachineBasicBlockENS_9SlotIndexE", "llvm::LiveIntervals::anyKillInMBBAfterIdx(llvm::LiveInterval const&, llvm::VNInfo const*, llvm::MachineBasicBlock*, llvm::SlotIndex) const"},
    {"_ZN4llvm13LiveIntervals28rewriteInstructionsForSpillsERKNS_12LiveIntervalEbRPKNS_9LiveRangeEPNS_12MachineInstrES9_jibbbbRNS_10VirtRegMapEPKNS_19TargetRegisterClassERNS_11SmallVectorIiLj4EEEPKNS_15MachineLoopInfoERNS_9BitVectorERNS_8DenseMapIjSt6vectorINS0_6SRInfoESaISP_EENS_12DenseMapInfoIjEENSS_ISR_EEEESM_SW_RNSN_IjjST_ST_EERSO_IPS1_SaISZ_EE", "llvm::LiveIntervals::rewriteInstructionsForSpills(llvm::LiveInterval const&, bool, llvm::LiveRange const*&, llvm::MachineInstr*, llvm::MachineInstr*, unsigned int, int, bool, bool, bool, bool, llvm::VirtRegMap&, llvm::TargetRegisterClass const*, llvm::SmallVector<int, 4u>&, llvm::MachineLoopInfo const*, llvm::BitVector&, llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >&, llvm::BitVector&, llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >&, llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >&, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> >&)"},
    {"_ZN4llvm13LiveIntervals14getSpillWeightEbbj", "llvm::LiveIntervals::getSpillWeight(bool, bool, unsigned int)"},
    {"_ZN4llvm13LiveIntervals16alsoFoldARestoreEiNS_9SlotIndexEjRNS_9BitVectorERNS_8DenseMapIjSt6vectorINS0_6SRInfoESaIS6_EENS_12DenseMapInfoIjEENS9_IS8_EEEE", "llvm::LiveIntervals::alsoFoldARestore(int, llvm::SlotIndex, unsigned int, llvm::BitVector&, llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >&)"},
    {"_ZN4llvm13LiveIntervals16eraseRestoreInfoEiNS_9SlotIndexEjRNS_9BitVectorERNS_8DenseMapIjSt6vectorINS0_6SRInfoESaIS6_EENS_12DenseMapInfoIjEENS9_IS8_EEEE", "llvm::LiveIntervals::eraseRestoreInfo(int, llvm::SlotIndex, unsigned int, llvm::BitVector&, llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >&)"},
    {"_ZN4llvm13LiveIntervals20handleSpilledImpDefsERKNS_12LiveIntervalERNS_10VirtRegMapEPKNS_19TargetRegisterClassERSt6vectorIPS1_SaISA_EE", "llvm::LiveIntervals::handleSpilledImpDefs(llvm::LiveInterval const&, llvm::VirtRegMap&, llvm::TargetRegisterClass const*, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> >&)"},
    {"_ZN4llvm13LiveIntervals21normalizeSpillWeightsERSt6vectorIPNS_12LiveIntervalESaIS3_EE", "llvm::LiveIntervals::normalizeSpillWeights(std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> >&)"},
    {"_ZN4llvm13LiveIntervals21addIntervalsForSpillsERKNS_12LiveIntervalERKNS_15SmallVectorImplIPS1_EEPKNS_15MachineLoopInfoERNS_10VirtRegMapE", "llvm::LiveIntervals::addIntervalsForSpills(llvm::LiveInterval const&, llvm::SmallVectorImpl<llvm::LiveInterval*> const&, llvm::MachineLoopInfo const*, llvm::VirtRegMap&)"},
    {"_ZNK4llvm13LiveIntervals22hasAllocatableSuperRegEj", "llvm::LiveIntervals::hasAllocatableSuperReg(unsigned int) const"},
    {"_ZNK4llvm13LiveIntervals20getRepresentativeRegEj", "llvm::LiveIntervals::getRepresentativeReg(unsigned int) const"},
    {"_ZNK4llvm13LiveIntervals26getNumConflictsWithPhysRegERKNS_12LiveIntervalEj", "llvm::LiveIntervals::getNumConflictsWithPhysReg(llvm::LiveInterval const&, unsigned int) const"},
    {"_ZN4llvm13LiveIntervals29spillPhysRegAroundRegDefsUsesERKNS_12LiveIntervalEjRNS_10VirtRegMapE", "llvm::LiveIntervals::spillPhysRegAroundRegDefsUses(llvm::LiveInterval const&, unsigned int, llvm::VirtRegMap&)"},
    {"_ZN4llvm13LiveIntervals24addLiveRangeToEndOfBlockEjPNS_12MachineInstrE", "llvm::LiveIntervals::addLiveRangeToEndOfBlock(unsigned int, llvm::MachineInstr*)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111RewriteInfoESt6vectorIS3_SaIS3_EEEElNS2_18RewriteInfoCompareEEvT_SA_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, long, (anonymous namespace)::RewriteInfoCompare>(__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, long, (anonymous namespace)::RewriteInfoCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111RewriteInfoESt6vectorIS3_SaIS3_EEEENS2_18RewriteInfoCompareEEvT_SA_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, (anonymous namespace)::RewriteInfoCompare>(__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, (anonymous namespace)::RewriteInfoCompare)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111RewriteInfoESt6vectorIS3_SaIS3_EEEElS3_NS2_18RewriteInfoCompareEEvT_T0_SB_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, long, (anonymous namespace)::RewriteInfo, (anonymous namespace)::RewriteInfoCompare>(__gnu_cxx::__normal_iterator<(anonymous namespace)::RewriteInfo*, std::vector<(anonymous namespace)::RewriteInfo, std::allocator<(anonymous namespace)::RewriteInfo> > >, long, long, (anonymous namespace)::RewriteInfo, (anonymous namespace)::RewriteInfoCompare)"},
    {"_ZN4llvm13LiveIntervals19getOrCreateIntervalEj", "llvm::LiveIntervals::getOrCreateInterval(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplINS_9LiveRangeEE4swapERS2_", "llvm::SmallVectorImpl<llvm::LiveRange>::swap(llvm::SmallVectorImpl<llvm::LiveRange>&)"},
    {"_ZNK4llvm13LiveIntervals13isLiveInToMBBERKNS_12LiveIntervalEPKNS_17MachineBasicBlockE", "llvm::LiveIntervals::isLiveInToMBB(llvm::LiveInterval const&, llvm::MachineBasicBlock const*) const"},
    {"_ZN4llvm10VirtRegMap16transferSpillPtsEPNS_12MachineInstrES2_", "llvm::VirtRegMap::transferSpillPts(llvm::MachineInstr*, llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap18transferRestorePtsEPNS_12MachineInstrES2_", "llvm::VirtRegMap::transferRestorePts(llvm::MachineInstr*, llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap23transferEmergencySpillsEPNS_12MachineInstrES2_", "llvm::VirtRegMap::transferEmergencySpills(llvm::MachineInstr*, llvm::MachineInstr*)"},
    {"_ZNK4llvm11SlotIndexes19getMBBCoveringRangeENS_9SlotIndexES1_", "llvm::SlotIndexes::getMBBCoveringRange(llvm::SlotIndex, llvm::SlotIndex) const"},
    {"_ZN4llvm8DenseMapIjSt6vectorINS_13LiveIntervals6SRInfoESaIS3_EENS_12DenseMapInfoIjEENS6_IS5_EEE6insertERKSt4pairIjS5_E", "llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >::insert(std::pair<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_12MachineInstrEE6resizeEjRKS2_", "llvm::SmallVectorImpl<llvm::MachineInstr*>::resize(unsigned int, llvm::MachineInstr* const&)"},
    {"_ZN4llvm15SmallVectorImplIiE6resizeEjRKi", "llvm::SmallVectorImpl<int>::resize(unsigned int, int const&)"},
    {"_ZN4llvm10VirtRegMap13addSpillPointEjbPNS_12MachineInstrE", "llvm::VirtRegMap::addSpillPoint(unsigned int, bool, llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap15addRestorePointEjPNS_12MachineInstrE", "llvm::VirtRegMap::addRestorePoint(unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap17addEmergencySpillEjPNS_12MachineInstrE", "llvm::VirtRegMap::addEmergencySpill(unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveIntervalsD1Ev", "llvm::LiveIntervals::~LiveIntervals()"},
    {"_ZN4llvm13LiveIntervalsD0Ev", "llvm::LiveIntervals::~LiveIntervals()"},
    {"_ZN4llvm8DenseMapIjSt6vectorINS_13LiveIntervals6SRInfoESaIS3_EENS_12DenseMapInfoIjEENS6_IS5_EEE16FindAndConstructERKj", "llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >::FindAndConstruct(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIjSt6vectorINS_13LiveIntervals6SRInfoESaIS3_EENS_12DenseMapInfoIjEENS6_IS5_EEE16InsertIntoBucketERKjRKS5_PSt4pairIjS5_E", "llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >::InsertIntoBucket(unsigned int const&, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > const&, std::pair<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > >*)"},
    {"_ZN4llvm8DenseMapIjSt6vectorINS_13LiveIntervals6SRInfoESaIS3_EENS_12DenseMapInfoIjEENS6_IS5_EEE4growEj", "llvm::DenseMap<unsigned int, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > > >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm13LiveIntervals6SRInfoESaIS2_EEC2ERKS4_", "std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >::vector(std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > const&)"},
    {"_ZNSt6vectorIN4llvm13LiveIntervals6SRInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LiveIntervals::SRInfo*, std::vector<llvm::LiveIntervals::SRInfo, std::allocator<llvm::LiveIntervals::SRInfo> > >, llvm::LiveIntervals::SRInfo const&)"},
    {"_ZN4llvm8DenseMapIjPNS_12LiveIntervalENS_12DenseMapInfoIjEENS3_IS2_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, llvm::LiveInterval*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::LiveInterval*> >::shrink_and_clear()"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIjSaIjEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > > >::_M_insert_unique(std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIjSaIjEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE14_M_create_nodeERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > > >::_M_create_node(std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIjSaIjEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > const&)"},
    {"_ZNSt6vectorISt4pairIjbESaIS1_EEC2ERKS3_", "std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > >::vector(std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIS3_IjbESaIS6_EEESt10_Select1stIS9_ESt4lessIS2_ESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > > >::_M_insert_unique(std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > const&)"},
    {"_ZNSt6vectorISt4pairIjbESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, bool>*, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > >, std::pair<unsigned int, bool> const&)"},
    {"_ZN4llvm11SlotIndexes25replaceMachineInstrInMapsEPNS_12MachineInstrES2_", "llvm::SlotIndexes::replaceMachineInstrInMaps(llvm::MachineInstr*, llvm::MachineInstr*)"},
    {"_ZN4llvm8DenseMapIjPNS_12LiveIntervalENS_12DenseMapInfoIjEENS3_IS2_EEE6insertERKSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::LiveInterval*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::LiveInterval*> >::insert(std::pair<unsigned int, llvm::LiveInterval*> const&)"},
    {"_ZN4llvm8DenseMapIjPNS_12LiveIntervalENS_12DenseMapInfoIjEENS3_IS2_EEE16InsertIntoBucketERKjRKS2_PSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::LiveInterval*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::LiveInterval*> >::InsertIntoBucket(unsigned int const&, llvm::LiveInterval* const&, std::pair<unsigned int, llvm::LiveInterval*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_12LiveIntervalENS_12DenseMapInfoIjEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::LiveInterval*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::LiveInterval*> >::grow(unsigned int)"},
    {"_ZN4llvm15callDefaultCtorINS_13LiveIntervalsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LiveIntervals>()"},
    {"_ZN4llvm13LiveIntervalsC2Ev", "llvm::LiveIntervals::LiveIntervals()"},
    {"_ZN4llvm13LiveRangeEdit6createERNS_19MachineRegisterInfoERNS_13LiveIntervalsERNS_10VirtRegMapE", "llvm::LiveRangeEdit::create(llvm::MachineRegisterInfo&, llvm::LiveIntervals&, llvm::VirtRegMap&)"},
    {"_ZN4llvm13LiveRangeEdit14scanRemattableERNS_13LiveIntervalsERKNS_15TargetInstrInfoEPNS_13AliasAnalysisE", "llvm::LiveRangeEdit::scanRemattable(llvm::LiveIntervals&, llvm::TargetInstrInfo const&, llvm::AliasAnalysis*)"},
    {"_ZN4llvm13LiveRangeEdit19anyRematerializableERNS_13LiveIntervalsERKNS_15TargetInstrInfoEPNS_13AliasAnalysisE", "llvm::LiveRangeEdit::anyRematerializable(llvm::LiveIntervals&, llvm::TargetInstrInfo const&, llvm::AliasAnalysis*)"},
    {"_ZN4llvm13LiveRangeEdit18allUsesAvailableAtEPKNS_12MachineInstrENS_9SlotIndexES4_RNS_13LiveIntervalsE", "llvm::LiveRangeEdit::allUsesAvailableAt(llvm::MachineInstr const*, llvm::SlotIndex, llvm::SlotIndex, llvm::LiveIntervals&)"},
    {"_ZN4llvm13LiveRangeEdit18canRematerializeAtERNS0_5RematENS_9SlotIndexEbRNS_13LiveIntervalsE", "llvm::LiveRangeEdit::canRematerializeAt(llvm::LiveRangeEdit::Remat&, llvm::SlotIndex, bool, llvm::LiveIntervals&)"},
    {"_ZN4llvm13LiveRangeEdit15rematerializeAtERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjRKNS0_5RematERNS_13LiveIntervalsERKNS_15TargetInstrInfoERKNS_18TargetRegisterInfoE", "llvm::LiveRangeEdit::rematerializeAt(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, llvm::LiveRangeEdit::Remat const&, llvm::LiveIntervals&, llvm::TargetInstrInfo const&, llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm24initializeLiveStacksPassERNS_12PassRegistryE", "llvm::initializeLiveStacksPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm10LiveStacks16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LiveStacks::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm10LiveStacks13releaseMemoryEv", "llvm::LiveStacks::releaseMemory()"},
    {"_ZN4llvm10LiveStacks20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::LiveStacks::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm10LiveStacks19getOrCreateIntervalEiPKNS_19TargetRegisterClassE", "llvm::LiveStacks::getOrCreateInterval(int, llvm::TargetRegisterClass const*)"},
    {"_ZNK4llvm10LiveStacks5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::LiveStacks::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZN4llvm10LiveStacksD1Ev", "llvm::LiveStacks::~LiveStacks()"},
    {"_ZN4llvm10LiveStacksD0Ev", "llvm::LiveStacks::~LiveStacks()"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm19TargetRegisterClassEESt10_Select1stIS6_ESt4lessIiESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<int, std::pair<int const, llvm::TargetRegisterClass const*>, std::_Select1st<std::pair<int const, llvm::TargetRegisterClass const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::TargetRegisterClass const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<int const, llvm::TargetRegisterClass const*> >, std::pair<int const, llvm::TargetRegisterClass const*> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm19TargetRegisterClassEESt10_Select1stIS6_ESt4lessIiESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<int, std::pair<int const, llvm::TargetRegisterClass const*>, std::_Select1st<std::pair<int const, llvm::TargetRegisterClass const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::TargetRegisterClass const*> > >::_M_insert_unique(std::pair<int const, llvm::TargetRegisterClass const*> const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_6VNInfoEEaSERKS3_", "llvm::SmallVectorImpl<llvm::VNInfo*>::operator=(llvm::SmallVectorImpl<llvm::VNInfo*> const&)"},
    {"_ZN4llvm15SmallVectorImplINS_9LiveRangeEEaSERKS2_", "llvm::SmallVectorImpl<llvm::LiveRange>::operator=(llvm::SmallVectorImpl<llvm::LiveRange> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiN4llvm12LiveIntervalEESt10_Select1stIS4_ESt4lessIiESaIS4_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS4_ERKS4_", "std::_Rb_tree<int, std::pair<int const, llvm::LiveInterval>, std::_Select1st<std::pair<int const, llvm::LiveInterval> >, std::less<int>, std::allocator<std::pair<int const, llvm::LiveInterval> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<int const, llvm::LiveInterval> >, std::pair<int const, llvm::LiveInterval> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiN4llvm12LiveIntervalEESt10_Select1stIS4_ESt4lessIiESaIS4_EE16_M_insert_uniqueERKS4_", "std::_Rb_tree<int, std::pair<int const, llvm::LiveInterval>, std::_Select1st<std::pair<int const, llvm::LiveInterval> >, std::less<int>, std::allocator<std::pair<int const, llvm::LiveInterval> > >::_M_insert_unique(std::pair<int const, llvm::LiveInterval> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiN4llvm12LiveIntervalEESt10_Select1stIS4_ESt4lessIiESaIS4_EE14_M_create_nodeERKS4_", "std::_Rb_tree<int, std::pair<int const, llvm::LiveInterval>, std::_Select1st<std::pair<int const, llvm::LiveInterval> >, std::less<int>, std::allocator<std::pair<int const, llvm::LiveInterval> > >::_M_create_node(std::pair<int const, llvm::LiveInterval> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm19TargetRegisterClassEESt10_Select1stIS6_ESt4lessIiESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<int, std::pair<int const, llvm::TargetRegisterClass const*>, std::_Select1st<std::pair<int const, llvm::TargetRegisterClass const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::TargetRegisterClass const*> > >::_M_erase(std::_Rb_tree_node<std::pair<int const, llvm::TargetRegisterClass const*> >*)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiN4llvm12LiveIntervalEESt10_Select1stIS4_ESt4lessIiESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<int, std::pair<int const, llvm::LiveInterval>, std::_Select1st<std::pair<int const, llvm::LiveInterval> >, std::less<int>, std::allocator<std::pair<int const, llvm::LiveInterval> > >::_M_erase(std::_Rb_tree_node<std::pair<int const, llvm::LiveInterval> >*)"},
    {"_ZN4llvm15callDefaultCtorINS_10LiveStacksEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LiveStacks>()"},
    {"_ZN4llvm27initializeLiveVariablesPassERNS_12PassRegistryE", "llvm::initializeLiveVariablesPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm13LiveVariables16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LiveVariables::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm13LiveVariables7VarInfo8findKillEPKNS_17MachineBasicBlockE", "llvm::LiveVariables::VarInfo::findKill(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm13LiveVariables7VarInfo4dumpEv", "llvm::LiveVariables::VarInfo::dump() const"},
    {"_ZN4llvm13LiveVariables10getVarInfoEj", "llvm::LiveVariables::getVarInfo(unsigned int)"},
    {"_ZN4llvm13LiveVariables23MarkVirtRegAliveInBlockERNS0_7VarInfoEPNS_17MachineBasicBlockES4_RSt6vectorIS4_SaIS4_EE", "llvm::LiveVariables::MarkVirtRegAliveInBlock(llvm::LiveVariables::VarInfo&, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> >&)"},
    {"_ZN4llvm13LiveVariables23MarkVirtRegAliveInBlockERNS0_7VarInfoEPNS_17MachineBasicBlockES4_", "llvm::LiveVariables::MarkVirtRegAliveInBlock(llvm::LiveVariables::VarInfo&, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm13LiveVariables16HandleVirtRegUseEjPNS_17MachineBasicBlockEPNS_12MachineInstrE", "llvm::LiveVariables::HandleVirtRegUse(unsigned int, llvm::MachineBasicBlock*, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables16HandleVirtRegDefEjPNS_12MachineInstrE", "llvm::LiveVariables::HandleVirtRegDef(unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables18FindLastPartialDefEjRNS_8SmallSetIjLj4EEE", "llvm::LiveVariables::FindLastPartialDef(unsigned int, llvm::SmallSet<unsigned int, 4u>&)"},
    {"_ZN4llvm13LiveVariables16HandlePhysRegUseEjPNS_12MachineInstrE", "llvm::LiveVariables::HandlePhysRegUse(unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables20FindLastRefOrPartRefEj", "llvm::LiveVariables::FindLastRefOrPartRef(unsigned int)"},
    {"_ZN4llvm13LiveVariables17HandlePhysRegKillEjPNS_12MachineInstrE", "llvm::LiveVariables::HandlePhysRegKill(unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables16HandlePhysRegDefEjPNS_12MachineInstrERNS_11SmallVectorIjLj4EEE", "llvm::LiveVariables::HandlePhysRegDef(unsigned int, llvm::MachineInstr*, llvm::SmallVector<unsigned int, 4u>&)"},
    {"_ZN4llvm13LiveVariables17UpdatePhysRegDefsEPNS_12MachineInstrERNS_11SmallVectorIjLj4EEE", "llvm::LiveVariables::UpdatePhysRegDefs(llvm::MachineInstr*, llvm::SmallVector<unsigned int, 4u>&)"},
    {"_ZN4llvm13LiveVariables20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::LiveVariables::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm13LiveVariables15analyzePHINodesERKNS_15MachineFunctionE", "llvm::LiveVariables::analyzePHINodes(llvm::MachineFunction const&)"},
    {"_ZN4llvm13LiveVariables22replaceKillInstructionEjPNS_12MachineInstrES2_", "llvm::LiveVariables::replaceKillInstruction(unsigned int, llvm::MachineInstr*, llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables28removeVirtualRegistersKilledEPNS_12MachineInstrE", "llvm::LiveVariables::removeVirtualRegistersKilled(llvm::MachineInstr*)"},
    {"_ZN4llvm13LiveVariables7VarInfo8isLiveInERKNS_17MachineBasicBlockEjRNS_19MachineRegisterInfoE", "llvm::LiveVariables::VarInfo::isLiveIn(llvm::MachineBasicBlock const&, unsigned int, llvm::MachineRegisterInfo&)"},
    {"_ZN4llvm13LiveVariables9isLiveOutEjRKNS_17MachineBasicBlockE", "llvm::LiveVariables::isLiveOut(unsigned int, llvm::MachineBasicBlock const&)"},
    {"_ZN4llvm13LiveVariables11addNewBlockEPNS_17MachineBasicBlockES2_S2_", "llvm::LiveVariables::addNewBlock(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm15SparseBitVectorILj128EE4testEj", "llvm::SparseBitVector<128u>::test(unsigned int)"},
    {"_ZN4llvm15SparseBitVectorILj128EE3setEj", "llvm::SparseBitVector<128u>::set(unsigned int)"},
    {"_ZN4llvm8SmallSetIjLj8EE5eraseERKj", "llvm::SmallSet<unsigned int, 8u>::erase(unsigned int const&)"},
    {"_ZN4llvm8SmallSetIjLj32EE6insertERKj", "llvm::SmallSet<unsigned int, 32u>::insert(unsigned int const&)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_12DenseMapInfoIS2_EENS3_IjEEE6insertERKSt4pairIS2_jE", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<llvm::MachineInstr*, unsigned int> const&)"},
    {"_ZN4llvm13LiveVariablesD1Ev", "llvm::LiveVariables::~LiveVariables()"},
    {"_ZN4llvm13LiveVariablesD0Ev", "llvm::LiveVariables::~LiveVariables()"},
    {"_ZN4llvm13LiveVariables13releaseMemoryEv", "llvm::LiveVariables::releaseMemory()"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEElEvT_SA_T0_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, long>(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, long)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEEEvT_SA_SA_", "void std::__heap_select<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZSt9sort_heapIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::sort_heap<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZSt9make_heapIN9__gnu_cxx17__normal_iteratorIPPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::make_heap<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm10IndexedMapINS_13LiveVariables7VarInfoENS_20VirtReg2IndexFunctorEE6resizeEm", "llvm::IndexedMap<llvm::LiveVariables::VarInfo, llvm::VirtReg2IndexFunctor>::resize(unsigned long)"},
    {"_ZNSt6vectorIN4llvm13LiveVariables7VarInfoESaIS2_EE6resizeEmS2_", "std::vector<llvm::LiveVariables::VarInfo, std::allocator<llvm::LiveVariables::VarInfo> >::resize(unsigned long, llvm::LiveVariables::VarInfo)"},
    {"_ZN4llvm6iplistINS_22SparseBitVectorElementILj128EEENS_12ilist_traitsIS2_EEED2Ev", "llvm::iplist<llvm::SparseBitVectorElement<128u>, llvm::ilist_traits<llvm::SparseBitVectorElement<128u> > >::~iplist()"},
    {"_ZN4llvm15SparseBitVectorILj128EEC2ERKS1_", "llvm::SparseBitVector<128u>::SparseBitVector(llvm::SparseBitVector<128u> const&)"},
    {"_ZN4llvm5ilistINS_22SparseBitVectorElementILj128EEEE9push_backERKS2_", "llvm::ilist<llvm::SparseBitVectorElement<128u> >::push_back(llvm::SparseBitVectorElement<128u> const&)"},
    {"_ZNSt6vectorIN4llvm13LiveVariables7VarInfoESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::LiveVariables::VarInfo, std::allocator<llvm::LiveVariables::VarInfo> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::LiveVariables::VarInfo*, std::vector<llvm::LiveVariables::VarInfo, std::allocator<llvm::LiveVariables::VarInfo> > >, unsigned long, llvm::LiveVariables::VarInfo const&)"},
    {"_ZSt26__uninitialized_fill_n_auxIPN4llvm13LiveVariables7VarInfoEmS2_EvT_T0_RKT1_St12__false_type", "void std::__uninitialized_fill_n_aux<llvm::LiveVariables::VarInfo*, unsigned long, llvm::LiveVariables::VarInfo>(llvm::LiveVariables::VarInfo*, unsigned long, llvm::LiveVariables::VarInfo const&, std::__false_type)"},
    {"_ZN4llvm15SparseBitVectorILj128EEaSERKS1_", "llvm::SparseBitVector<128u>::operator=(llvm::SparseBitVector<128u> const&)"},
    {"_ZNSt6vectorIPN4llvm12MachineInstrESaIS2_EEaSERKS4_", "std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >::operator=(std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > const&)"},
    {"_ZN4llvm15SparseBitVectorILj128EE23SparseBitVectorIterator20AdvanceToNextNonZeroEv", "llvm::SparseBitVector<128u>::SparseBitVectorIterator::AdvanceToNextNonZero()"},
    {"_ZN4llvm15SparseBitVectorILj128EE23SparseBitVectorIteratorC2EPKS1_b", "llvm::SparseBitVector<128u>::SparseBitVectorIterator::SparseBitVectorIterator(llvm::SparseBitVector<128u> const*, bool)"},
    {"_ZN4llvm15SparseBitVectorILj128EE14FindLowerBoundEj", "llvm::SparseBitVector<128u>::FindLowerBound(unsigned int)"},
    {"_ZN4llvm13LiveVariablesD2Ev", "llvm::LiveVariables::~LiveVariables()"},
    {"_ZN4llvm15callDefaultCtorINS_13LiveVariablesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LiveVariables>()"},
    {"_ZN4llvm13LiveVariablesC2Ev", "llvm::LiveVariables::LiveVariables()"},
    {"_ZN4llvm34createLocalStackSlotAllocationPassEv", "llvm::createLocalStackSlotAllocationPass()"},
    {"_ZN12_GLOBAL__N_118LocalStackSlotPassD1Ev", "(anonymous namespace)::LocalStackSlotPass::~LocalStackSlotPass()"},
    {"_ZN12_GLOBAL__N_118LocalStackSlotPassD0Ev", "(anonymous namespace)::LocalStackSlotPass::~LocalStackSlotPass()"},
    {"_ZNK12_GLOBAL__N_118LocalStackSlotPass11getPassNameEv", "(anonymous namespace)::LocalStackSlotPass::getPassName() const"},
    {"_ZNK12_GLOBAL__N_118LocalStackSlotPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LocalStackSlotPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118LocalStackSlotPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::LocalStackSlotPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_18FrameRefEEEiPKvS4_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::FrameRef>(void const*, void const*)"},
    {"_ZN12_GLOBAL__N_118LocalStackSlotPass17AdjustStackOffsetEPN4llvm16MachineFrameInfoEiRxbRj", "(anonymous namespace)::LocalStackSlotPass::AdjustStackOffset(llvm::MachineFrameInfo*, int, long long&, bool, unsigned int&)"},
    {"_ZN4llvm15SmallVectorImplIxE6resizeEj", "llvm::SmallVectorImpl<long long>::resize(unsigned int)"},
    {"_ZN4llvm22createLowerSubregsPassEv", "llvm::createLowerSubregsPass()"},
    {"_ZN12_GLOBAL__N_127LowerSubregsInstructionPassD1Ev", "(anonymous namespace)::LowerSubregsInstructionPass::~LowerSubregsInstructionPass()"},
    {"_ZN12_GLOBAL__N_127LowerSubregsInstructionPassD0Ev", "(anonymous namespace)::LowerSubregsInstructionPass::~LowerSubregsInstructionPass()"},
    {"_ZNK12_GLOBAL__N_127LowerSubregsInstructionPass11getPassNameEv", "(anonymous namespace)::LowerSubregsInstructionPass::getPassName() const"},
    {"_ZNK12_GLOBAL__N_127LowerSubregsInstructionPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LowerSubregsInstructionPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_127LowerSubregsInstructionPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::LowerSubregsInstructionPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm17MachineBasicBlockC1ERNS_15MachineFunctionEPKNS_10BasicBlockE", "llvm::MachineBasicBlock::MachineBasicBlock(llvm::MachineFunction&, llvm::BasicBlock const*)"},
    {"_ZN4llvm17MachineBasicBlockC2ERNS_15MachineFunctionEPKNS_10BasicBlockE", "llvm::MachineBasicBlock::MachineBasicBlock(llvm::MachineFunction&, llvm::BasicBlock const*)"},
    {"_ZN4llvm17MachineBasicBlockD1Ev", "llvm::MachineBasicBlock::~MachineBasicBlock()"},
    {"_ZN4llvm17MachineBasicBlockD2Ev", "llvm::MachineBasicBlock::~MachineBasicBlock()"},
    {"_ZNK4llvm17MachineBasicBlock9getSymbolEv", "llvm::MachineBasicBlock::getSymbol() const"},
    {"_ZN4llvmlsERNS_11raw_ostreamERKNS_17MachineBasicBlockE", "llvm::operator<<(llvm::raw_ostream&, llvm::MachineBasicBlock const&)"},
    {"_ZNK4llvm17MachineBasicBlock5printERNS_11raw_ostreamEPNS_11SlotIndexesE", "llvm::MachineBasicBlock::print(llvm::raw_ostream&, llvm::SlotIndexes*) const"},
    {"_ZN4llvm12ilist_traitsINS_17MachineBasicBlockEE13addNodeToListEPS1_", "llvm::ilist_traits<llvm::MachineBasicBlock>::addNodeToList(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12ilist_traitsINS_17MachineBasicBlockEE18removeNodeFromListEPS1_", "llvm::ilist_traits<llvm::MachineBasicBlock>::removeNodeFromList(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12ilist_traitsINS_12MachineInstrEE13addNodeToListEPS1_", "llvm::ilist_traits<llvm::MachineInstr>::addNodeToList(llvm::MachineInstr*)"},
    {"_ZN4llvm12ilist_traitsINS_12MachineInstrEE18removeNodeFromListEPS1_", "llvm::ilist_traits<llvm::MachineInstr>::removeNodeFromList(llvm::MachineInstr*)"},
    {"_ZN4llvm12ilist_traitsINS_12MachineInstrEE21transferNodesFromListERS2_NS_14ilist_iteratorIS1_EES5_", "llvm::ilist_traits<llvm::MachineInstr>::transferNodesFromList(llvm::ilist_traits<llvm::MachineInstr>&, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm12ilist_traitsINS_12MachineInstrEE10deleteNodeEPS1_", "llvm::ilist_traits<llvm::MachineInstr>::deleteNode(llvm::MachineInstr*)"},
    {"_ZN4llvm17MachineBasicBlock14getFirstNonPHIEv", "llvm::MachineBasicBlock::getFirstNonPHI()"},
    {"_ZN4llvm17MachineBasicBlock17SkipPHIsAndLabelsENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::MachineBasicBlock::SkipPHIsAndLabels(llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm17MachineBasicBlock18getFirstTerminatorEv", "llvm::MachineBasicBlock::getFirstTerminator()"},
    {"_ZN4llvm17MachineBasicBlock20getLastNonDebugInstrEv", "llvm::MachineBasicBlock::getLastNonDebugInstr()"},
    {"_ZNK4llvm17MachineBasicBlock22getLandingPadSuccessorEv", "llvm::MachineBasicBlock::getLandingPadSuccessor() const"},
    {"_ZNK4llvm17MachineBasicBlock4dumpEv", "llvm::MachineBasicBlock::dump() const"},
    {"_ZNK4llvm17MachineBasicBlock7getNameEv", "llvm::MachineBasicBlock::getName() const"},
    {"_ZN4llvm17MachineBasicBlock12removeLiveInEj", "llvm::MachineBasicBlock::removeLiveIn(unsigned int)"},
    {"_ZNK4llvm17MachineBasicBlock8isLiveInEj", "llvm::MachineBasicBlock::isLiveIn(unsigned int) const"},
    {"_ZN4llvm17MachineBasicBlock10moveBeforeEPS0_", "llvm::MachineBasicBlock::moveBefore(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock9moveAfterEPS0_", "llvm::MachineBasicBlock::moveAfter(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock16updateTerminatorEv", "llvm::MachineBasicBlock::updateTerminator()"},
    {"_ZNK4llvm17MachineBasicBlock17isLayoutSuccessorEPKS0_", "llvm::MachineBasicBlock::isLayoutSuccessor(llvm::MachineBasicBlock const*) const"},
    {"_ZN4llvm17MachineBasicBlock12addSuccessorEPS0_", "llvm::MachineBasicBlock::addSuccessor(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock14addPredecessorEPS0_", "llvm::MachineBasicBlock::addPredecessor(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock15removeSuccessorEPS0_", "llvm::MachineBasicBlock::removeSuccessor(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock17removePredecessorEPS0_", "llvm::MachineBasicBlock::removePredecessor(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock15removeSuccessorEN9__gnu_cxx17__normal_iteratorIPPS0_St6vectorIS3_SaIS3_EEEE", "llvm::MachineBasicBlock::removeSuccessor(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZN4llvm17MachineBasicBlock18transferSuccessorsEPS0_", "llvm::MachineBasicBlock::transferSuccessors(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock31transferSuccessorsAndUpdatePHIsEPS0_", "llvm::MachineBasicBlock::transferSuccessorsAndUpdatePHIs(llvm::MachineBasicBlock*)"},
    {"_ZNK4llvm17MachineBasicBlock11isSuccessorEPKS0_", "llvm::MachineBasicBlock::isSuccessor(llvm::MachineBasicBlock const*) const"},
    {"_ZN4llvm17MachineBasicBlock14canFallThroughEv", "llvm::MachineBasicBlock::canFallThrough()"},
    {"_ZN4llvm17MachineBasicBlock17SplitCriticalEdgeEPS0_PNS_4PassE", "llvm::MachineBasicBlock::SplitCriticalEdge(llvm::MachineBasicBlock*, llvm::Pass*)"},
    {"_ZN4llvm17MachineBasicBlock22ReplaceUsesOfBlockWithEPS0_S1_", "llvm::MachineBasicBlock::ReplaceUsesOfBlockWith(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineBasicBlock16removeFromParentEv", "llvm::MachineBasicBlock::removeFromParent()"},
    {"_ZN4llvm17MachineBasicBlock15eraseFromParentEv", "llvm::MachineBasicBlock::eraseFromParent()"},
    {"_ZN4llvm17MachineBasicBlock20CorrectExtraCFGEdgesEPS0_S1_b", "llvm::MachineBasicBlock::CorrectExtraCFGEdges(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, bool)"},
    {"_ZN4llvm17MachineBasicBlock12findDebugLocERNS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::MachineBasicBlock::findDebugLoc(llvm::ilist_iterator<llvm::MachineInstr>&)"},
    {"_ZN4llvm14WriteAsOperandERNS_11raw_ostreamEPKNS_17MachineBasicBlockEb", "llvm::WriteAsOperand(llvm::raw_ostream&, llvm::MachineBasicBlock const*, bool)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE19addBasicBlockToLoopEPS1_RNS_12LoopInfoBaseIS1_S2_EE", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::addBasicBlockToLoop(llvm::MachineBasicBlock*, llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_11MachineLoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineLoop*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineLoop*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::MachineLoop* const&, std::pair<llvm::MachineBasicBlock*, llvm::MachineLoop*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_11MachineLoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineLoop*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineLoop*> >::grow(unsigned int)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPKPN4llvm17MachineBasicBlockESt6vectorIS4_SaIS4_EEEEPKS3_ET_SD_SD_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock* const*, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock* const*, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, llvm::MachineBasicBlock const*>(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock* const*, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock* const*, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, llvm::MachineBasicBlock const* const&, std::random_access_iterator_tag)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPKjSt6vectorIjSaIjEEEEjET_S8_S8_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<unsigned int const*, std::vector<unsigned int, std::allocator<unsigned int> > > std::__find<__gnu_cxx::__normal_iterator<unsigned int const*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned int>(__gnu_cxx::__normal_iterator<unsigned int const*, std::vector<unsigned int, std::allocator<unsigned int> > >, __gnu_cxx::__normal_iterator<unsigned int const*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned int const&, std::random_access_iterator_tag)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPjSt6vectorIjSaIjEEEEjET_S7_S7_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > > std::__find<__gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned int>(__gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > >, __gnu_cxx::__normal_iterator<unsigned int*, std::vector<unsigned int, std::allocator<unsigned int> > >, unsigned int const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm15DomTreeNodeBaseINS0_17MachineBasicBlockEEESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm15DomTreeNodeBaseINS2_17MachineBasicBlockEEESt6vectorIS6_SaIS6_EEEES6_ET_SC_SC_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>(__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > >, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&, std::pair<llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEEN9__gnu_cxx17__normal_iteratorIPS5_St6vectorIS5_SaIS5_EEEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > >, false>::grow(unsigned long)"},
    {"_ZN4llvm6iplistINS_17MachineBasicBlockENS_12ilist_traitsIS1_EEE6insertENS_14ilist_iteratorIS1_EEPS1_", "llvm::iplist<llvm::MachineBasicBlock, llvm::ilist_traits<llvm::MachineBasicBlock> >::insert(llvm::ilist_iterator<llvm::MachineBasicBlock>, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm24initializeMachineCSEPassERNS_12PassRegistryE", "llvm::initializeMachineCSEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm20createMachineCSEPassEv", "llvm::createMachineCSEPass()"},
    {"_ZN12_GLOBAL__N_110MachineCSEC1Ev", "(anonymous namespace)::MachineCSE::MachineCSE()"},
    {"_ZN12_GLOBAL__N_110MachineCSED1Ev", "(anonymous namespace)::MachineCSE::~MachineCSE()"},
    {"_ZN12_GLOBAL__N_110MachineCSED0Ev", "(anonymous namespace)::MachineCSE::~MachineCSE()"},
    {"_ZNK12_GLOBAL__N_110MachineCSE16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineCSE::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_110MachineCSE13releaseMemoryEv", "(anonymous namespace)::MachineCSE::releaseMemory()"},
    {"_ZN12_GLOBAL__N_110MachineCSE20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineCSE::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_110MachineCSE9ExitScopeEPN4llvm17MachineBasicBlockE", "(anonymous namespace)::MachineCSE::ExitScope(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_110MachineCSEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::MachineCSE>()"},
    {"_ZN4llvm20ScopedHashTableScopeIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_jEELm32ELm8EEEED2Ev", "llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >::~ScopedHashTableScope()"},
    {"_ZNK4llvm8DenseMapIPNS_12MachineInstrEPNS_18ScopedHashTableValIS2_jEENS_27MachineInstrExpressionTraitENS_12DenseMapInfoIS5_EEE15LookupBucketForERKS2_RPSt4pairIS2_S5_E", "llvm::DenseMap<llvm::MachineInstr*, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*> >::LookupBucketFor(llvm::MachineInstr* const&, std::pair<llvm::MachineInstr*, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*>*&) const"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPNS_18ScopedHashTableValIS2_jEENS_27MachineInstrExpressionTraitENS_12DenseMapInfoIS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::MachineInstr*, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*> >::InsertIntoBucket(llvm::MachineInstr* const&, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>* const&, std::pair<llvm::MachineInstr*, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*>*)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPNS_18ScopedHashTableValIS2_jEENS_27MachineInstrExpressionTraitENS_12DenseMapInfoIS5_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr*, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>*> >::grow(unsigned int)"},
    {"_ZN4llvm15ScopedHashTableIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_jEELm32ELm8EEEE6insertERKS2_RKj", "llvm::ScopedHashTable<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >::insert(llvm::MachineInstr* const&, unsigned int const&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_20ScopedHashTableScopeIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS5_jEELm32ELm8EEEEENS_12DenseMapInfoIS2_EENSE_ISD_EEE16InsertIntoBucketERKS2_RKSD_PSt4pairIS2_SD_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >* const&, std::pair<llvm::MachineBasicBlock*, llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_20ScopedHashTableScopeIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS5_jEELm32ELm8EEEEENS_12DenseMapInfoIS2_EENSE_ISD_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEES4_NS_12DenseMapInfoIS4_EES6_E16InsertIntoBucketERKS4_S9_PSt4pairIS4_S4_E", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::InsertIntoBucket(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&, std::pair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>*)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEES4_NS_12DenseMapInfoIS4_EES6_E4growEj", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, unsigned int, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>* const&, unsigned int const&, std::pair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, unsigned int, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_20ScopedHashTableScopeIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS5_jEELm32ELm8EEEEENS_12DenseMapInfoIS2_EENSE_ISD_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::ScopedHashTableScope<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >*> >::shrink_and_clear()"},
    {"_ZN4llvm15ScopedHashTableIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_jEELm32ELm8EEEED2Ev", "llvm::ScopedHashTable<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::MachineInstr*, unsigned int>, 32ul, 8ul> >::~ScopedHashTable()"},
    {"_ZN4llvm12PrintDomTreeINS_17MachineBasicBlockEEEvPKNS_15DomTreeNodeBaseIT_EERNS_11raw_ostreamEj", "void llvm::PrintDomTree<llvm::MachineBasicBlock>(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*, llvm::raw_ostream&, unsigned int)"},
    {"_ZN4llvm34initializeMachineDominatorTreePassERNS_12PassRegistryE", "llvm::initializeMachineDominatorTreePass(llvm::PassRegistry&)"},
    {"_ZNK4llvm20MachineDominatorTree16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MachineDominatorTree::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm20MachineDominatorTree20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::MachineDominatorTree::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm20MachineDominatorTreeC1Ev", "llvm::MachineDominatorTree::MachineDominatorTree()"},
    {"_ZN4llvm20MachineDominatorTreeC2Ev", "llvm::MachineDominatorTree::MachineDominatorTree()"},
    {"_ZN4llvm20MachineDominatorTreeD0Ev", "llvm::MachineDominatorTree::~MachineDominatorTree()"},
    {"_ZN4llvm20MachineDominatorTreeD1Ev", "llvm::MachineDominatorTree::~MachineDominatorTree()"},
    {"_ZN4llvm20MachineDominatorTreeD2Ev", "llvm::MachineDominatorTree::~MachineDominatorTree()"},
    {"_ZN4llvm20MachineDominatorTree13releaseMemoryEv", "llvm::MachineDominatorTree::releaseMemory()"},
    {"_ZNK4llvm20MachineDominatorTree5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::MachineDominatorTree::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE5beginEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::begin()"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE3endEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::end()"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE5beginEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::begin() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE3endEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::end() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE8getBlockEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getBlock() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE7getIDomEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getIDom() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE11getChildrenEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getChildren() const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEEC1EPS1_PS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::DomTreeNodeBase(llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEEC2EPS1_PS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::DomTreeNodeBase(llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE8addChildEPS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::addChild(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE14getNumChildrenEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getNumChildren() const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE16clearAllChildrenEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::clearAllChildren()"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE7compareEPS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::compare(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE7setIDomEPS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::setIDom(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE11getDFSNumInEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getDFSNumIn() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE12getDFSNumOutEv", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::getDFSNumOut() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_17MachineBasicBlockEE11DominatedByEPKS2_", "llvm::DomTreeNodeBase<llvm::MachineBasicBlock>::DominatedBy(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7InfoRecC1Ev", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec::InfoRec()"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7InfoRecC2Ev", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec::InfoRec()"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE5resetEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::reset()"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEEC1Eb", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::DominatorTreeBase(bool)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEEC2Eb", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::DominatorTreeBase(bool)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEED0Ev", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::~DominatorTreeBase()"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEED1Ev", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::~DominatorTreeBase()"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEED2Ev", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::~DominatorTreeBase()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7compareERS2_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::compare(llvm::DominatorTreeBase<llvm::MachineBasicBlock>&) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE13releaseMemoryEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::releaseMemory()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7getNodeEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getNode(llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE11getRootNodeEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getRootNode()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE11getRootNodeEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getRootNode() const"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE17properlyDominatesEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::properlyDominates(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*) const"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE23dominatedBySlowTreeWalkEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::dominatedBySlowTreeWalk(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE17properlyDominatesEPKS1_S4_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::properlyDominates(llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE20isReachableFromEntryEPKS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::isReachableFromEntry(llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE9dominatesEPKS1_S4_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::dominates(llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE9dominatesEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::dominates(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE16updateDFSNumbersEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::updateDFSNumbers()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7getRootEv", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getRoot() const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE26findNearestCommonDominatorEPS1_S3_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::findNearestCommonDominator(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE26findNearestCommonDominatorEPKS1_S4_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::findNearestCommonDominator(llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE11addNewBlockEPS1_S3_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::addNewBlock(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE24changeImmediateDominatorEPNS_15DomTreeNodeBaseIS1_EES5_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::changeImmediateDominator(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE24changeImmediateDominatorEPS1_S3_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::changeImmediateDominator(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE9eraseNodeEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::eraseNode(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE10removeNodeEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::removeNode(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE10splitBlockEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::splitBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE5SplitINS_7InverseIPS1_EENS_11GraphTraitsIS6_EEEEvRNS0_INT0_8NodeTypeEEEPSA_", "void llvm::DominatorTreeBase<llvm::MachineBasicBlock>::Split<llvm::Inverse<llvm::MachineBasicBlock*>, llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE5SplitIPS1_NS_11GraphTraitsIS4_EEEEvRNS0_INT0_8NodeTypeEEEPS8_", "void llvm::DominatorTreeBase<llvm::MachineBasicBlock>::Split<llvm::MachineBasicBlock*, llvm::GraphTraits<llvm::MachineBasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType*)"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE5printERNS_11raw_ostreamE", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE15getNodeForBlockEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getNodeForBlock(llvm::MachineBasicBlock*)"},
    {"_ZNK4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE7getIDomEPS1_", "llvm::DominatorTreeBase<llvm::MachineBasicBlock>::getIDom(llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEE11recalculateINS_15MachineFunctionEEEvRT_", "void llvm::DominatorTreeBase<llvm::MachineBasicBlock>::recalculate<llvm::MachineFunction>(llvm::MachineFunction&)"},
    {"_ZN4llvm9CalculateINS_15MachineFunctionEPNS_17MachineBasicBlockEEEvRNS_17DominatorTreeBaseINS_11GraphTraitsIT0_E8NodeTypeEEERT_", "void llvm::Calculate<llvm::MachineFunction, llvm::MachineBasicBlock*>(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType>&, llvm::MachineFunction&)"},
    {"_ZN4llvm9CalculateINS_15MachineFunctionENS_7InverseIPNS_17MachineBasicBlockEEEEEvRNS_17DominatorTreeBaseINS_11GraphTraitsIT0_E8NodeTypeEEERT_", "void llvm::Calculate<llvm::MachineFunction, llvm::Inverse<llvm::MachineBasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType>&, llvm::MachineFunction&)"},
    {"_ZN4llvm7DFSPassINS_11GraphTraitsINS_7InverseIPNS_17MachineBasicBlockEEEEEEEjRNS_17DominatorTreeBaseINT_8NodeTypeEEEPS9_j", "unsigned int llvm::DFSPass<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType*, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIjE6resizeEj", "llvm::SmallVectorImpl<unsigned int>::resize(unsigned int)"},
    {"_ZN4llvm4EvalINS_11GraphTraitsINS_7InverseIPNS_17MachineBasicBlockEEEEEEEPNT_8NodeTypeERNS_17DominatorTreeBaseIS8_EES9_j", "llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType* llvm::Eval<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::MachineBasicBlock*> >::NodeType*, unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec> >::shrink_and_clear()"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPNS_17MachineBasicBlockEN9__gnu_cxx17__normal_iteratorIPS3_St6vectorIS3_SaIS3_EEEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::MachineBasicBlock*, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec const&, std::pair<llvm::MachineBasicBlock*, llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::MachineBasicBlock>::InfoRec> >::grow(unsigned int)"},
    {"_ZN4llvm7DFSPassINS_11GraphTraitsIPNS_17MachineBasicBlockEEEEEjRNS_17DominatorTreeBaseINT_8NodeTypeEEEPS7_j", "unsigned int llvm::DFSPass<llvm::GraphTraits<llvm::MachineBasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType*, unsigned int)"},
    {"_ZN4llvm4EvalINS_11GraphTraitsIPNS_17MachineBasicBlockEEEEEPNT_8NodeTypeERNS_17DominatorTreeBaseIS6_EES7_j", "llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType* llvm::Eval<llvm::GraphTraits<llvm::MachineBasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::MachineBasicBlock*>::NodeType*, unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::MachineBasicBlock* const&, std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockES2_NS_12DenseMapInfoIS2_EES4_E16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineBasicBlock*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_15DomTreeNodeBaseIS1_EENS_12DenseMapInfoIS2_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::shrink_and_clear()"},
    {"_ZN4llvm15callDefaultCtorINS_20MachineDominatorTreeEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::MachineDominatorTree>()"},
    {"_ZN4llvm19MachineFunctionInfoD0Ev", "llvm::MachineFunctionInfo::~MachineFunctionInfo()"},
    {"_ZN4llvm19MachineFunctionInfoD1Ev", "llvm::MachineFunctionInfo::~MachineFunctionInfo()"},
    {"_ZN4llvm19MachineFunctionInfoD2Ev", "llvm::MachineFunctionInfo::~MachineFunctionInfo()"},
    {"_ZN4llvm12ilist_traitsINS_17MachineBasicBlockEE10deleteNodeEPS1_", "llvm::ilist_traits<llvm::MachineBasicBlock>::deleteNode(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm15MachineFunction23DeleteMachineBasicBlockEPNS_17MachineBasicBlockE", "llvm::MachineFunction::DeleteMachineBasicBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm15MachineFunctionC1EPKNS_8FunctionERKNS_13TargetMachineEjRNS_17MachineModuleInfoEPNS_12GCModuleInfoE", "llvm::MachineFunction::MachineFunction(llvm::Function const*, llvm::TargetMachine const&, unsigned int, llvm::MachineModuleInfo&, llvm::GCModuleInfo*)"},
    {"_ZN4llvm15MachineFunctionC2EPKNS_8FunctionERKNS_13TargetMachineEjRNS_17MachineModuleInfoEPNS_12GCModuleInfoE", "llvm::MachineFunction::MachineFunction(llvm::Function const*, llvm::TargetMachine const&, unsigned int, llvm::MachineModuleInfo&, llvm::GCModuleInfo*)"},
    {"_ZN4llvm15MachineFunctionD1Ev", "llvm::MachineFunction::~MachineFunction()"},
    {"_ZN4llvm15MachineFunctionD2Ev", "llvm::MachineFunction::~MachineFunction()"},
    {"_ZN4llvm19MachineConstantPoolD1Ev", "llvm::MachineConstantPool::~MachineConstantPool()"},
    {"_ZN4llvm15MachineFunction24getOrCreateJumpTableInfoEj", "llvm::MachineFunction::getOrCreateJumpTableInfo(unsigned int)"},
    {"_ZN4llvm15MachineFunction14RenumberBlocksEPNS_17MachineBasicBlockE", "llvm::MachineFunction::RenumberBlocks(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm15MachineFunction18CreateMachineInstrERKNS_15TargetInstrDescENS_8DebugLocEb", "llvm::MachineFunction::CreateMachineInstr(llvm::TargetInstrDesc const&, llvm::DebugLoc, bool)"},
    {"_ZN4llvm15MachineFunction17CloneMachineInstrEPKNS_12MachineInstrE", "llvm::MachineFunction::CloneMachineInstr(llvm::MachineInstr const*)"},
    {"_ZN4llvm15MachineFunction18DeleteMachineInstrEPNS_12MachineInstrE", "llvm::MachineFunction::DeleteMachineInstr(llvm::MachineInstr*)"},
    {"_ZN4llvm15MachineFunction23CreateMachineBasicBlockEPKNS_10BasicBlockE", "llvm::MachineFunction::CreateMachineBasicBlock(llvm::BasicBlock const*)"},
    {"_ZN4llvm15MachineFunction20getMachineMemOperandENS_18MachinePointerInfoEjyjPKNS_6MDNodeE", "llvm::MachineFunction::getMachineMemOperand(llvm::MachinePointerInfo, unsigned int, unsigned long long, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm15MachineFunction20getMachineMemOperandEPKNS_17MachineMemOperandExy", "llvm::MachineFunction::getMachineMemOperand(llvm::MachineMemOperand const*, long long, unsigned long long)"},
    {"_ZN4llvm15MachineFunction20allocateMemRefsArrayEm", "llvm::MachineFunction::allocateMemRefsArray(unsigned long)"},
    {"_ZN4llvm15MachineFunction18extractLoadMemRefsEPPNS_17MachineMemOperandES3_", "llvm::MachineFunction::extractLoadMemRefs(llvm::MachineMemOperand**, llvm::MachineMemOperand**)"},
    {"_ZN4llvm15MachineFunction19extractStoreMemRefsEPPNS_17MachineMemOperandES3_", "llvm::MachineFunction::extractStoreMemRefs(llvm::MachineMemOperand**, llvm::MachineMemOperand**)"},
    {"_ZNK4llvm15MachineFunction4dumpEv", "llvm::MachineFunction::dump() const"},
    {"_ZNK4llvm15MachineFunction5printERNS_11raw_ostreamEPNS_11SlotIndexesE", "llvm::MachineFunction::print(llvm::raw_ostream&, llvm::SlotIndexes*) const"},
    {"_ZNK4llvm16MachineFrameInfo5printERKNS_15MachineFunctionERNS_11raw_ostreamE", "llvm::MachineFrameInfo::print(llvm::MachineFunction const&, llvm::raw_ostream&) const"},
    {"_ZNK4llvm20MachineJumpTableInfo5printERNS_11raw_ostreamE", "llvm::MachineJumpTableInfo::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm19MachineConstantPool5printERNS_11raw_ostreamE", "llvm::MachineConstantPool::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm15MachineFunction7viewCFGEv", "llvm::MachineFunction::viewCFG() const"},
    {"_ZNK4llvm15MachineFunction11viewCFGOnlyEv", "llvm::MachineFunction::viewCFGOnly() const"},
    {"_ZN4llvm15MachineFunction9addLiveInEjPKNS_19TargetRegisterClassENS_8DebugLocE", "llvm::MachineFunction::addLiveIn(unsigned int, llvm::TargetRegisterClass const*, llvm::DebugLoc)"},
    {"_ZNK4llvm15MachineFunction12getJTISymbolEjRNS_9MCContextEb", "llvm::MachineFunction::getJTISymbol(unsigned int, llvm::MCContext&, bool) const"},
    {"_ZNK4llvm15MachineFunction16getPICBaseSymbolEv", "llvm::MachineFunction::getPICBaseSymbol() const"},
    {"_ZN4llvm16MachineFrameInfo17CreateFixedObjectEyxb", "llvm::MachineFrameInfo::CreateFixedObject(unsigned long long, long long, bool)"},
    {"_ZNK4llvm16MachineFrameInfo15getPristineRegsEPKNS_17MachineBasicBlockE", "llvm::MachineFrameInfo::getPristineRegs(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm16MachineFrameInfo4dumpERKNS_15MachineFunctionE", "llvm::MachineFrameInfo::dump(llvm::MachineFunction const&) const"},
    {"_ZNK4llvm20MachineJumpTableInfo12getEntrySizeERKNS_10DataLayoutE", "llvm::MachineJumpTableInfo::getEntrySize(llvm::DataLayout const&) const"},
    {"_ZNK4llvm20MachineJumpTableInfo17getEntryAlignmentERKNS_10DataLayoutE", "llvm::MachineJumpTableInfo::getEntryAlignment(llvm::DataLayout const&) const"},
    {"_ZN4llvm20MachineJumpTableInfo20createJumpTableIndexERKSt6vectorIPNS_17MachineBasicBlockESaIS3_EE", "llvm::MachineJumpTableInfo::createJumpTableIndex(std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > const&)"},
    {"_ZN4llvm20MachineJumpTableInfo22ReplaceMBBInJumpTablesEPNS_17MachineBasicBlockES2_", "llvm::MachineJumpTableInfo::ReplaceMBBInJumpTables(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm20MachineJumpTableInfo21ReplaceMBBInJumpTableEjPNS_17MachineBasicBlockES2_", "llvm::MachineJumpTableInfo::ReplaceMBBInJumpTable(unsigned int, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*)"},
    {"_ZNK4llvm20MachineJumpTableInfo4dumpEv", "llvm::MachineJumpTableInfo::dump() const"},
    {"_ZNK4llvm24MachineConstantPoolEntry7getTypeEv", "llvm::MachineConstantPoolEntry::getType() const"},
    {"_ZNK4llvm24MachineConstantPoolEntry17getRelocationInfoEv", "llvm::MachineConstantPoolEntry::getRelocationInfo() const"},
    {"_ZN4llvm19MachineConstantPoolD2Ev", "llvm::MachineConstantPool::~MachineConstantPool()"},
    {"_ZN4llvm19MachineConstantPool20getConstantPoolIndexEPKNS_8ConstantEj", "llvm::MachineConstantPool::getConstantPoolIndex(llvm::Constant const*, unsigned int)"},
    {"_Z25CanShareConstantPoolEntryPKN4llvm8ConstantES2_PKNS_10DataLayoutE", "CanShareConstantPoolEntry(llvm::Constant const*, llvm::Constant const*, llvm::DataLayout const*)"},
    {"_ZN4llvm19MachineConstantPool20getConstantPoolIndexEPNS_24MachineConstantPoolValueEj", "llvm::MachineConstantPool::getConstantPoolIndex(llvm::MachineConstantPoolValue*, unsigned int)"},
    {"_ZNK4llvm19MachineConstantPool4dumpEv", "llvm::MachineConstantPool::dump() const"},
    {"_ZN4llvm6iplistINS_17MachineBasicBlockENS_12ilist_traitsIS1_EEE5clearEv", "llvm::iplist<llvm::MachineBasicBlock, llvm::ilist_traits<llvm::MachineBasicBlock> >::clear()"},
    {"_ZNSt6vectorIN4llvm21MachineJumpTableEntryESaIS1_EE9push_backERKS1_", "std::vector<llvm::MachineJumpTableEntry, std::allocator<llvm::MachineJumpTableEntry> >::push_back(llvm::MachineJumpTableEntry const&)"},
    {"_ZNSt6vectorIN4llvm24MachineConstantPoolEntryESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MachineConstantPoolEntry, std::allocator<llvm::MachineConstantPoolEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineConstantPoolEntry*, std::vector<llvm::MachineConstantPoolEntry, std::allocator<llvm::MachineConstantPoolEntry> > >, llvm::MachineConstantPoolEntry const&)"},
    {"_ZNSt6vectorIN4llvm21MachineJumpTableEntryESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MachineJumpTableEntry, std::allocator<llvm::MachineJumpTableEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineJumpTableEntry*, std::vector<llvm::MachineJumpTableEntry, std::allocator<llvm::MachineJumpTableEntry> > >, llvm::MachineJumpTableEntry const&)"},
    {"_ZNSt6vectorIPN4llvm17MachineBasicBlockESaIS2_EEaSERKS4_", "std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> >::operator=(std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > const&)"},
    {"_ZNSt6vectorIPN4llvm17MachineBasicBlockESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, unsigned long, llvm::MachineBasicBlock* const&)"},
    {"_ZN4llvm8DenseMapIjNS_8DebugLocENS_12DenseMapInfoIjEENS2_IS1_EEE16InsertIntoBucketERKjRKS1_PSt4pairIjS1_E", "llvm::DenseMap<unsigned int, llvm::DebugLoc, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::DebugLoc> >::InsertIntoBucket(unsigned int const&, llvm::DebugLoc const&, std::pair<unsigned int, llvm::DebugLoc>*)"},
    {"_ZN4llvm8DenseMapIjNS_8DebugLocENS_12DenseMapInfoIjEENS2_IS1_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::DebugLoc, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::DebugLoc> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIjjESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<unsigned int, unsigned int>, std::allocator<std::pair<unsigned int, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, unsigned int>*, std::vector<std::pair<unsigned int, unsigned int>, std::allocator<std::pair<unsigned int, unsigned int> > > >, std::pair<unsigned int, unsigned int> const&)"},
    {"_ZN4llvm23MachineFunctionAnalysisC1ERKNS_13TargetMachineENS_10CodeGenOpt5LevelE", "llvm::MachineFunctionAnalysis::MachineFunctionAnalysis(llvm::TargetMachine const&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm23MachineFunctionAnalysisC2ERKNS_13TargetMachineENS_10CodeGenOpt5LevelE", "llvm::MachineFunctionAnalysis::MachineFunctionAnalysis(llvm::TargetMachine const&, llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm23MachineFunctionAnalysisD0Ev", "llvm::MachineFunctionAnalysis::~MachineFunctionAnalysis()"},
    {"_ZN4llvm23MachineFunctionAnalysisD1Ev", "llvm::MachineFunctionAnalysis::~MachineFunctionAnalysis()"},
    {"_ZN4llvm23MachineFunctionAnalysisD2Ev", "llvm::MachineFunctionAnalysis::~MachineFunctionAnalysis()"},
    {"_ZNK4llvm23MachineFunctionAnalysis16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MachineFunctionAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm23MachineFunctionAnalysis16doInitializationERNS_6ModuleE", "llvm::MachineFunctionAnalysis::doInitialization(llvm::Module&)"},
    {"_ZN4llvm23MachineFunctionAnalysis13runOnFunctionERNS_8FunctionE", "llvm::MachineFunctionAnalysis::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm23MachineFunctionAnalysis13releaseMemoryEv", "llvm::MachineFunctionAnalysis::releaseMemory()"},
    {"_ZNK4llvm23MachineFunctionAnalysis11getPassNameEv", "llvm::MachineFunctionAnalysis::getPassName() const"},
    {"_ZNK4llvm19MachineFunctionPass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::MachineFunctionPass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZN4llvm19MachineFunctionPass13runOnFunctionERNS_8FunctionE", "llvm::MachineFunctionPass::runOnFunction(llvm::Function&)"},
    {"_ZNK4llvm19MachineFunctionPass16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MachineFunctionPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm32createMachineFunctionPrinterPassERNS_11raw_ostreamERKSs", "llvm::createMachineFunctionPrinterPass(llvm::raw_ostream&, std::string const&)"},
    {"_ZN12_GLOBAL__N_126MachineFunctionPrinterPassD1Ev", "(anonymous namespace)::MachineFunctionPrinterPass::~MachineFunctionPrinterPass()"},
    {"_ZN12_GLOBAL__N_126MachineFunctionPrinterPassD0Ev", "(anonymous namespace)::MachineFunctionPrinterPass::~MachineFunctionPrinterPass()"},
    {"_ZNK12_GLOBAL__N_126MachineFunctionPrinterPass11getPassNameEv", "(anonymous namespace)::MachineFunctionPrinterPass::getPassName() const"},
    {"_ZNK12_GLOBAL__N_126MachineFunctionPrinterPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineFunctionPrinterPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_126MachineFunctionPrinterPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineFunctionPrinterPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm14MachineOperand22AddRegOperandToRegInfoEPNS_19MachineRegisterInfoE", "llvm::MachineOperand::AddRegOperandToRegInfo(llvm::MachineRegisterInfo*)"},
    {"_ZN4llvm14MachineOperand27RemoveRegOperandFromRegInfoEv", "llvm::MachineOperand::RemoveRegOperandFromRegInfo()"},
    {"_ZN4llvm14MachineOperand6setRegEj", "llvm::MachineOperand::setReg(unsigned int)"},
    {"_ZN4llvm14MachineOperand12substVirtRegEjjRKNS_18TargetRegisterInfoE", "llvm::MachineOperand::substVirtReg(unsigned int, unsigned int, llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm14MachineOperand12substPhysRegEjRKNS_18TargetRegisterInfoE", "llvm::MachineOperand::substPhysReg(unsigned int, llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm14MachineOperand17ChangeToImmediateEx", "llvm::MachineOperand::ChangeToImmediate(long long)"},
    {"_ZN4llvm14MachineOperand16ChangeToRegisterEjbbbbbb", "llvm::MachineOperand::ChangeToRegister(unsigned int, bool, bool, bool, bool, bool, bool)"},
    {"_ZNK4llvm14MachineOperand13isIdenticalToERKS0_", "llvm::MachineOperand::isIdenticalTo(llvm::MachineOperand const&) const"},
    {"_ZNK4llvm14MachineOperand5printERNS_11raw_ostreamEPKNS_13TargetMachineE", "llvm::MachineOperand::print(llvm::raw_ostream&, llvm::TargetMachine const*) const"},
    {"_ZNK4llvm18MachinePointerInfo12getAddrSpaceEv", "llvm::MachinePointerInfo::getAddrSpace() const"},
    {"_ZN4llvm18MachinePointerInfo15getConstantPoolEv", "llvm::MachinePointerInfo::getConstantPool()"},
    {"_ZN4llvm18MachinePointerInfo13getFixedStackEix", "llvm::MachinePointerInfo::getFixedStack(int, long long)"},
    {"_ZN4llvm18MachinePointerInfo12getJumpTableEv", "llvm::MachinePointerInfo::getJumpTable()"},
    {"_ZN4llvm18MachinePointerInfo6getGOTEv", "llvm::MachinePointerInfo::getGOT()"},
    {"_ZN4llvm18MachinePointerInfo8getStackEx", "llvm::MachinePointerInfo::getStack(long long)"},
    {"_ZN4llvm17MachineMemOperandC1ENS_18MachinePointerInfoEjyjPKNS_6MDNodeE", "llvm::MachineMemOperand::MachineMemOperand(llvm::MachinePointerInfo, unsigned int, unsigned long long, unsigned int, llvm::MDNode const*)"},
    {"_ZN4llvm17MachineMemOperandC2ENS_18MachinePointerInfoEjyjPKNS_6MDNodeE", "llvm::MachineMemOperand::MachineMemOperand(llvm::MachinePointerInfo, unsigned int, unsigned long long, unsigned int, llvm::MDNode const*)"},
    {"_ZNK4llvm17MachineMemOperand7ProfileERNS_16FoldingSetNodeIDE", "llvm::MachineMemOperand::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm17MachineMemOperand15refineAlignmentEPKS0_", "llvm::MachineMemOperand::refineAlignment(llvm::MachineMemOperand const*)"},
    {"_ZNK4llvm17MachineMemOperand12getAlignmentEv", "llvm::MachineMemOperand::getAlignment() const"},
    {"_ZN4llvmlsERNS_11raw_ostreamERKNS_17MachineMemOperandE", "llvm::operator<<(llvm::raw_ostream&, llvm::MachineMemOperand const&)"},
    {"_ZN4llvm12MachineInstrC1Ev", "llvm::MachineInstr::MachineInstr()"},
    {"_ZN4llvm12MachineInstrC2Ev", "llvm::MachineInstr::MachineInstr()"},
    {"_ZN4llvm12MachineInstr25addImplicitDefUseOperandsEv", "llvm::MachineInstr::addImplicitDefUseOperands()"},
    {"_ZN4llvm12MachineInstr10addOperandERKNS_14MachineOperandE", "llvm::MachineInstr::addOperand(llvm::MachineOperand const&)"},
    {"_ZN4llvm12MachineInstrC1ERKNS_15TargetInstrDescEb", "llvm::MachineInstr::MachineInstr(llvm::TargetInstrDesc const&, bool)"},
    {"_ZN4llvm12MachineInstrC2ERKNS_15TargetInstrDescEb", "llvm::MachineInstr::MachineInstr(llvm::TargetInstrDesc const&, bool)"},
    {"_ZN4llvm12MachineInstrC1ERKNS_15TargetInstrDescENS_8DebugLocEb", "llvm::MachineInstr::MachineInstr(llvm::TargetInstrDesc const&, llvm::DebugLoc, bool)"},
    {"_ZN4llvm12MachineInstrC2ERKNS_15TargetInstrDescENS_8DebugLocEb", "llvm::MachineInstr::MachineInstr(llvm::TargetInstrDesc const&, llvm::DebugLoc, bool)"},
    {"_ZN4llvm12MachineInstrC1EPNS_17MachineBasicBlockERKNS_15TargetInstrDescE", "llvm::MachineInstr::MachineInstr(llvm::MachineBasicBlock*, llvm::TargetInstrDesc const&)"},
    {"_ZN4llvm12MachineInstrC2EPNS_17MachineBasicBlockERKNS_15TargetInstrDescE", "llvm::MachineInstr::MachineInstr(llvm::MachineBasicBlock*, llvm::TargetInstrDesc const&)"},
    {"_ZN4llvm12MachineInstrC1EPNS_17MachineBasicBlockENS_8DebugLocERKNS_15TargetInstrDescE", "llvm::MachineInstr::MachineInstr(llvm::MachineBasicBlock*, llvm::DebugLoc, llvm::TargetInstrDesc const&)"},
    {"_ZN4llvm12MachineInstrC2EPNS_17MachineBasicBlockENS_8DebugLocERKNS_15TargetInstrDescE", "llvm::MachineInstr::MachineInstr(llvm::MachineBasicBlock*, llvm::DebugLoc, llvm::TargetInstrDesc const&)"},
    {"_ZN4llvm12MachineInstrC1ERNS_15MachineFunctionERKS0_", "llvm::MachineInstr::MachineInstr(llvm::MachineFunction&, llvm::MachineInstr const&)"},
    {"_ZN4llvm12MachineInstrC2ERNS_15MachineFunctionERKS0_", "llvm::MachineInstr::MachineInstr(llvm::MachineFunction&, llvm::MachineInstr const&)"},
    {"_ZN4llvm12MachineInstrD1Ev", "llvm::MachineInstr::~MachineInstr()"},
    {"_ZN4llvm12MachineInstrD2Ev", "llvm::MachineInstr::~MachineInstr()"},
    {"_ZN4llvm12MachineInstr10getRegInfoEv", "llvm::MachineInstr::getRegInfo()"},
    {"_ZN4llvm12MachineInstr29RemoveRegOperandsFromUseListsEv", "llvm::MachineInstr::RemoveRegOperandsFromUseLists()"},
    {"_ZN4llvm12MachineInstr24AddRegOperandsToUseListsERNS_19MachineRegisterInfoE", "llvm::MachineInstr::AddRegOperandsToUseLists(llvm::MachineRegisterInfo&)"},
    {"_ZN4llvm12MachineInstr13RemoveOperandEj", "llvm::MachineInstr::RemoveOperand(unsigned int)"},
    {"_ZN4llvm12MachineInstr13addMemOperandERNS_15MachineFunctionEPNS_17MachineMemOperandE", "llvm::MachineInstr::addMemOperand(llvm::MachineFunction&, llvm::MachineMemOperand*)"},
    {"_ZNK4llvm12MachineInstr13isIdenticalToEPKS0_NS0_11MICheckTypeE", "llvm::MachineInstr::isIdenticalTo(llvm::MachineInstr const*, llvm::MachineInstr::MICheckType) const"},
    {"_ZN4llvm12MachineInstr16removeFromParentEv", "llvm::MachineInstr::removeFromParent()"},
    {"_ZN4llvm12MachineInstr15eraseFromParentEv", "llvm::MachineInstr::eraseFromParent()"},
    {"_ZNK4llvm12MachineInstr16OperandsCompleteEv", "llvm::MachineInstr::OperandsComplete() const"},
    {"_ZNK4llvm12MachineInstr22getNumExplicitOperandsEv", "llvm::MachineInstr::getNumExplicitOperands() const"},
    {"_ZNK4llvm12MachineInstr24isStackAligningInlineAsmEv", "llvm::MachineInstr::isStackAligningInlineAsm() const"},
    {"_ZNK4llvm12MachineInstr25findRegisterUseOperandIdxEjbPKNS_18TargetRegisterInfoE", "llvm::MachineInstr::findRegisterUseOperandIdx(unsigned int, bool, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm12MachineInstr26readsWritesVirtualRegisterEjPNS_15SmallVectorImplIjEE", "llvm::MachineInstr::readsWritesVirtualRegister(unsigned int, llvm::SmallVectorImpl<unsigned int>*) const"},
    {"_ZNK4llvm12MachineInstr25findRegisterDefOperandIdxEjbbPKNS_18TargetRegisterInfoE", "llvm::MachineInstr::findRegisterDefOperandIdx(unsigned int, bool, bool, llvm::TargetRegisterInfo const*) const"},
    {"_ZNK4llvm12MachineInstr23findFirstPredOperandIdxEv", "llvm::MachineInstr::findFirstPredOperandIdx() const"},
    {"_ZNK4llvm12MachineInstr21isRegTiedToUseOperandEjPj", "llvm::MachineInstr::isRegTiedToUseOperand(unsigned int, unsigned int*) const"},
    {"_ZNK4llvm12MachineInstr21isRegTiedToDefOperandEjPj", "llvm::MachineInstr::isRegTiedToDefOperand(unsigned int, unsigned int*) const"},
    {"_ZN4llvm12MachineInstr13clearKillInfoEv", "llvm::MachineInstr::clearKillInfo()"},
    {"_ZN4llvm12MachineInstr16copyKillDeadInfoEPKS0_", "llvm::MachineInstr::copyKillDeadInfo(llvm::MachineInstr const*)"},
    {"_ZN4llvm12MachineInstr14copyPredicatesEPKS0_", "llvm::MachineInstr::copyPredicates(llvm::MachineInstr const*)"},
    {"_ZN4llvm12MachineInstr18substituteRegisterEjjjRKNS_18TargetRegisterInfoE", "llvm::MachineInstr::substituteRegister(unsigned int, unsigned int, unsigned int, llvm::TargetRegisterInfo const&)"},
    {"_ZNK4llvm12MachineInstr12isSafeToMoveEPKNS_15TargetInstrInfoEPNS_13AliasAnalysisERb", "llvm::MachineInstr::isSafeToMove(llvm::TargetInstrInfo const*, llvm::AliasAnalysis*, bool&) const"},
    {"_ZNK4llvm12MachineInstr23hasUnmodeledSideEffectsEv", "llvm::MachineInstr::hasUnmodeledSideEffects() const"},
    {"_ZNK4llvm12MachineInstr15isInvariantLoadEPNS_13AliasAnalysisE", "llvm::MachineInstr::isInvariantLoad(llvm::AliasAnalysis*) const"},
    {"_ZNK4llvm12MachineInstr20hasVolatileMemoryRefEv", "llvm::MachineInstr::hasVolatileMemoryRef() const"},
    {"_ZNK4llvm12MachineInstr13isSafeToReMatEPKNS_15TargetInstrInfoEPNS_13AliasAnalysisEj", "llvm::MachineInstr::isSafeToReMat(llvm::TargetInstrInfo const*, llvm::AliasAnalysis*, unsigned int) const"},
    {"_ZNK4llvm12MachineInstr18isConstantValuePHIEv", "llvm::MachineInstr::isConstantValuePHI() const"},
    {"_ZNK4llvm12MachineInstr14allDefsAreDeadEv", "llvm::MachineInstr::allDefsAreDead() const"},
    {"_ZN4llvm12MachineInstr15copyImplicitOpsEPKS0_", "llvm::MachineInstr::copyImplicitOps(llvm::MachineInstr const*)"},
    {"_ZNK4llvm12MachineInstr4dumpEv", "llvm::MachineInstr::dump() const"},
    {"_ZNK4llvm12MachineInstr5printERNS_11raw_ostreamEPKNS_13TargetMachineE", "llvm::MachineInstr::print(llvm::raw_ostream&, llvm::TargetMachine const*) const"},
    {"_Z13printDebugLocN4llvm8DebugLocEPKNS_15MachineFunctionERNS_11raw_ostreamE", "printDebugLoc(llvm::DebugLoc, llvm::MachineFunction const*, llvm::raw_ostream&)"},
    {"_ZN4llvm12MachineInstr17addRegisterKilledEjPKNS_18TargetRegisterInfoEb", "llvm::MachineInstr::addRegisterKilled(unsigned int, llvm::TargetRegisterInfo const*, bool)"},
    {"_ZN4llvm12MachineInstr15addRegisterDeadEjPKNS_18TargetRegisterInfoEb", "llvm::MachineInstr::addRegisterDead(unsigned int, llvm::TargetRegisterInfo const*, bool)"},
    {"_ZN4llvm12MachineInstr18addRegisterDefinedEjPKNS_18TargetRegisterInfoE", "llvm::MachineInstr::addRegisterDefined(unsigned int, llvm::TargetRegisterInfo const*)"},
    {"_ZN4llvm12MachineInstr21setPhysRegsDeadExceptERKNS_15SmallVectorImplIjEERKNS_18TargetRegisterInfoE", "llvm::MachineInstr::setPhysRegsDeadExcept(llvm::SmallVectorImpl<unsigned int> const&, llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm27MachineInstrExpressionTrait12getHashValueERKPKNS_12MachineInstrE", "llvm::MachineInstrExpressionTrait::getHashValue(llvm::MachineInstr const* const&)"},
    {"_ZNSt6vectorIN4llvm14MachineOperandESaIS1_EE7reserveEm", "std::vector<llvm::MachineOperand, std::allocator<llvm::MachineOperand> >::reserve(unsigned long)"},
    {"_ZN4llvm25initializeMachineLICMPassERNS_12PassRegistryE", "llvm::initializeMachineLICMPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createMachineLICMPassEb", "llvm::createMachineLICMPass(bool)"},
    {"_ZN12_GLOBAL__N_111MachineLICMD1Ev", "(anonymous namespace)::MachineLICM::~MachineLICM()"},
    {"_ZN12_GLOBAL__N_111MachineLICMD0Ev", "(anonymous namespace)::MachineLICM::~MachineLICM()"},
    {"_ZNK12_GLOBAL__N_111MachineLICM11getPassNameEv", "(anonymous namespace)::MachineLICM::getPassName() const"},
    {"_ZNK12_GLOBAL__N_111MachineLICM16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineLICM::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111MachineLICM13releaseMemoryEv", "(anonymous namespace)::MachineLICM::releaseMemory()"},
    {"_ZN12_GLOBAL__N_111MachineLICM20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineLICM::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_111MachineLICM11HoistRegionEPN4llvm15DomTreeNodeBaseINS1_17MachineBasicBlockEEEb", "(anonymous namespace)::MachineLICM::HoistRegion(llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, bool)"},
    {"_ZN12_GLOBAL__N_111MachineLICM15getCurPreheaderEv", "(anonymous namespace)::MachineLICM::getCurPreheader()"},
    {"_ZN12_GLOBAL__N_111MachineLICM15InitRegPressureEPN4llvm17MachineBasicBlockE", "(anonymous namespace)::MachineLICM::InitRegPressure(llvm::MachineBasicBlock*)"},
    {"_ZN12_GLOBAL__N_111MachineLICM17UpdateRegPressureEPKN4llvm12MachineInstrE", "(anonymous namespace)::MachineLICM::UpdateRegPressure(llvm::MachineInstr const*)"},
    {"_ZN12_GLOBAL__N_111MachineLICM19IsLoopInvariantInstERN4llvm12MachineInstrE", "(anonymous namespace)::MachineLICM::IsLoopInvariantInst(llvm::MachineInstr&)"},
    {"_ZN12_GLOBAL__N_111MachineLICM19IsProfitableToHoistERN4llvm12MachineInstrE", "(anonymous namespace)::MachineLICM::IsProfitableToHoist(llvm::MachineInstr&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111MachineLICMEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::MachineLICM>()"},
    {"_ZN4llvm8DenseMapIjSt6vectorIPKNS_12MachineInstrESaIS4_EENS_12DenseMapInfoIjEENS7_IS6_EEE5clearEv", "llvm::DenseMap<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > >::clear()"},
    {"_ZN4llvm8DenseMapIjSt6vectorIPKNS_12MachineInstrESaIS4_EENS_12DenseMapInfoIjEENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIjSt6vectorIPKNS_12MachineInstrESaIS4_EENS_12DenseMapInfoIjEENS7_IS6_EEE6insertERKSt4pairIjS6_E", "llvm::DenseMap<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > >::insert(std::pair<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > const&)"},
    {"_ZN4llvm8DenseMapIjSt6vectorIPKNS_12MachineInstrESaIS4_EENS_12DenseMapInfoIjEENS7_IS6_EEE16InsertIntoBucketERKjRKS6_PSt4pairIjS6_E", "llvm::DenseMap<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > >::InsertIntoBucket(unsigned int const&, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > const&, std::pair<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > >*)"},
    {"_ZN4llvm8DenseMapIjSt6vectorIPKNS_12MachineInstrESaIS4_EENS_12DenseMapInfoIjEENS7_IS6_EEE4growEj", "llvm::DenseMap<unsigned int, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > > >::grow(unsigned int)"},
    {"_ZNSt6vectorIPKN4llvm12MachineInstrESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineInstr const**, std::vector<llvm::MachineInstr const*, std::allocator<llvm::MachineInstr const*> > >, llvm::MachineInstr const* const&)"},
    {"_ZN4llvm8DenseMapIjiNS_12DenseMapInfoIjEENS1_IiEEE6insertERKSt4pairIjiE", "llvm::DenseMap<unsigned int, int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<int> >::insert(std::pair<unsigned int, int> const&)"},
    {"_ZN4llvm8DenseMapIjiNS_12DenseMapInfoIjEENS1_IiEEE16InsertIntoBucketERKjRKiPSt4pairIjiE", "llvm::DenseMap<unsigned int, int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<int> >::InsertIntoBucket(unsigned int const&, int const&, std::pair<unsigned int, int>*)"},
    {"_ZN4llvm8DenseMapIjiNS_12DenseMapInfoIjEENS1_IiEEE4growEj", "llvm::DenseMap<unsigned int, int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIjLj8EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<unsigned int, 8u>, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIiiSt9_IdentityIiESt4lessIiESaIiEE8_M_eraseEPSt13_Rb_tree_nodeIiE", "std::_Rb_tree<int, int, std::_Identity<int>, std::less<int>, std::allocator<int> >::_M_erase(std::_Rb_tree_node<int>*)"},
    {"_ZN4llvm8SmallSetIiLj32EE6insertERKi", "llvm::SmallSet<int, 32u>::insert(int const&)"},
    {"_ZNSt8_Rb_treeIiiSt9_IdentityIiESt4lessIiESaIiEE16_M_insert_uniqueERKi", "std::_Rb_tree<int, int, std::_Identity<int>, std::less<int>, std::allocator<int> >::_M_insert_unique(int const&)"},
    {"_ZN4llvm29initializeMachineLoopInfoPassERNS_12PassRegistryE", "llvm::initializeMachineLoopInfoPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15MachineLoopInfo20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::MachineLoopInfo::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNK4llvm15MachineLoopInfo16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MachineLoopInfo::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm11MachineLoop11getTopBlockEv", "llvm::MachineLoop::getTopBlock()"},
    {"_ZN4llvm11MachineLoop14getBottomBlockEv", "llvm::MachineLoop::getBottomBlock()"},
    {"_ZNK4llvm11MachineLoop4dumpEv", "llvm::MachineLoop::dump() const"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEEC1Ev", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopBase()"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEEC2Ev", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopBase()"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEED1Ev", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::~LoopBase()"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEED2Ev", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::~LoopBase()"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE12getLoopDepthEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopDepth() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE9getHeaderEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getHeader() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE13getParentLoopEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getParentLoop() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE8containsEPKS2_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::contains(llvm::MachineLoop const*) const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE8containsEPKS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::contains(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE11block_beginEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::block_begin() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE9block_endEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::block_end() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE11getSubLoopsEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getSubLoops() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE5beginEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::begin() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE3endEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::end() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE5emptyEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::empty() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE9getBlocksEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getBlocks() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE13isLoopExitingEPKS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::isLoopExiting(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE15getNumBackEdgesEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getNumBackEdges() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE16getExitingBlocksERNS_15SmallVectorImplIPS1_EE", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getExitingBlocks(llvm::SmallVectorImpl<llvm::MachineBasicBlock*>&) const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE15getExitingBlockEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getExitingBlock() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE13getExitBlocksERNS_15SmallVectorImplIPS1_EE", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getExitBlocks(llvm::SmallVectorImpl<llvm::MachineBasicBlock*>&) const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE12getExitBlockEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getExitBlock() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE16getLoopPreheaderEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopPreheader() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE18getLoopPredecessorEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopPredecessor() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE12getLoopLatchEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopLatch() const"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE20replaceChildLoopWithEPS2_S4_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::replaceChildLoopWith(llvm::MachineLoop*, llvm::MachineLoop*)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE12addChildLoopEPS2_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::addChildLoop(llvm::MachineLoop*)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE15removeChildLoopEN9__gnu_cxx17__normal_iteratorIPKPS2_St6vectorIS6_SaIS6_EEEE", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::removeChildLoop(__gnu_cxx::__normal_iterator<llvm::MachineLoop* const*, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE13addBlockEntryEPS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::addBlockEntry(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE12moveToHeaderEPS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::moveToHeader(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE19removeBlockFromLoopEPS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::removeBlockFromLoop(llvm::MachineBasicBlock*)"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE10verifyLoopEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::verifyLoop() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE14verifyLoopNestEv", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::verifyLoopNest() const"},
    {"_ZNK4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEE5printERNS_11raw_ostreamEj", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::print(llvm::raw_ostream&, unsigned int) const"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEEC1EPS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopBase(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm8LoopBaseINS_17MachineBasicBlockENS_11MachineLoopEEC2EPS1_", "llvm::LoopBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopBase(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEEC1Ev", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopInfoBase()"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEEC2Ev", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::LoopInfoBase()"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEED1Ev", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::~LoopInfoBase()"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEED2Ev", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::~LoopInfoBase()"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE13releaseMemoryEv", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::releaseMemory()"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE5beginEv", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::begin() const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE3endEv", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::end() const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE5emptyEv", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::empty() const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE10getLoopForEPKS1_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopFor(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEEixEPKS1_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::operator[](llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE12getLoopDepthEPKS1_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::getLoopDepth(llvm::MachineBasicBlock const*) const"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE12isLoopHeaderEPS1_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::isLoopHeader(llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE10removeLoopEN9__gnu_cxx17__normal_iteratorIPKPS2_St6vectorIS6_SaIS6_EEEE", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::removeLoop(__gnu_cxx::__normal_iterator<llvm::MachineLoop* const*, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE13changeLoopForEPS1_PS2_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::changeLoopFor(llvm::MachineBasicBlock*, llvm::MachineLoop*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE18changeTopLevelLoopEPS2_S4_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::changeTopLevelLoop(llvm::MachineLoop*, llvm::MachineLoop*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE15addTopLevelLoopEPS2_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::addTopLevelLoop(llvm::MachineLoop*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE11removeBlockEPS1_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::removeBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE23isNotAlreadyContainedInEPKS2_S5_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::isNotAlreadyContainedIn(llvm::MachineLoop const*, llvm::MachineLoop const*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE9CalculateERNS_17DominatorTreeBaseIS1_EE", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::Calculate(llvm::DominatorTreeBase<llvm::MachineBasicBlock>&)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE15ConsiderForLoopEPS1_RNS_17DominatorTreeBaseIS1_EE", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::ConsiderForLoop(llvm::MachineBasicBlock*, llvm::DominatorTreeBase<llvm::MachineBasicBlock>&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_11MachineLoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineLoop*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineLoop*> >::insert(std::pair<llvm::MachineBasicBlock*, llvm::MachineLoop*> const&)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE19MoveSiblingLoopIntoEPS2_S4_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::MoveSiblingLoopInto(llvm::MachineLoop*, llvm::MachineLoop*)"},
    {"_ZN4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE14InsertLoopIntoEPS2_S4_", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::InsertLoopInto(llvm::MachineLoop*, llvm::MachineLoop*)"},
    {"_ZNK4llvm12LoopInfoBaseINS_17MachineBasicBlockENS_11MachineLoopEE5printERNS_11raw_ostreamE", "llvm::LoopInfoBase<llvm::MachineBasicBlock, llvm::MachineLoop>::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm15MachineLoopInfoD1Ev", "llvm::MachineLoopInfo::~MachineLoopInfo()"},
    {"_ZN4llvm15MachineLoopInfoD0Ev", "llvm::MachineLoopInfo::~MachineLoopInfo()"},
    {"_ZN4llvm15MachineLoopInfo13releaseMemoryEv", "llvm::MachineLoopInfo::releaseMemory()"},
    {"_ZNSt8_Rb_treeIPN4llvm17MachineBasicBlockESt4pairIKS2_PNS0_11MachineLoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<llvm::MachineBasicBlock*, std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*>, std::_Select1st<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> >, std::less<llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> >, std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm17MachineBasicBlockESt4pairIKS2_PNS0_11MachineLoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<llvm::MachineBasicBlock*, std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*>, std::_Select1st<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> >, std::less<llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> > >::_M_insert_unique(std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> const&)"},
    {"_ZNSt6vectorIPN4llvm17MachineBasicBlockESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS2_S4_EEEEvS9_T_SA_St20forward_iterator_tag", "void std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, std::forward_iterator_tag)"},
    {"_ZN4llvm11df_iteratorIPNS_17MachineBasicBlockENS_11SmallPtrSetIS2_Lj8EEELb0ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>, false, llvm::GraphTraits<llvm::MachineBasicBlock*> >::toNext()"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_11MachineLoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::MachineLoop*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::MachineLoop*> >::shrink_and_clear()"},
    {"_ZNSt6vectorIPN4llvm11MachineLoopESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MachineLoop**, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >, llvm::MachineLoop* const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm11MachineLoopESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::MachineLoop**, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::MachineLoop**, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >, llvm::MachineLoop*>(__gnu_cxx::__normal_iterator<llvm::MachineLoop**, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >, __gnu_cxx::__normal_iterator<llvm::MachineLoop**, std::vector<llvm::MachineLoop*, std::allocator<llvm::MachineLoop*> > >, llvm::MachineLoop* const&, std::random_access_iterator_tag)"},
    {"_ZSt16__introsort_loopIPPN4llvm17MachineBasicBlockElEvT_S4_T0_", "void std::__introsort_loop<llvm::MachineBasicBlock**, long>(llvm::MachineBasicBlock**, llvm::MachineBasicBlock**, long)"},
    {"_ZSt22__final_insertion_sortIPPN4llvm17MachineBasicBlockEEvT_S4_", "void std::__final_insertion_sort<llvm::MachineBasicBlock**>(llvm::MachineBasicBlock**, llvm::MachineBasicBlock**)"},
    {"_ZSt13__heap_selectIPPN4llvm17MachineBasicBlockEEvT_S4_S4_", "void std::__heap_select<llvm::MachineBasicBlock**>(llvm::MachineBasicBlock**, llvm::MachineBasicBlock**, llvm::MachineBasicBlock**)"},
    {"_ZSt9sort_heapIPPN4llvm17MachineBasicBlockEEvT_S4_", "void std::sort_heap<llvm::MachineBasicBlock**>(llvm::MachineBasicBlock**, llvm::MachineBasicBlock**)"},
    {"_ZSt9make_heapIPPN4llvm17MachineBasicBlockEEvT_S4_", "void std::make_heap<llvm::MachineBasicBlock**>(llvm::MachineBasicBlock**, llvm::MachineBasicBlock**)"},
    {"_ZN4llvm15callDefaultCtorINS_15MachineLoopInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::MachineLoopInfo>()"},
    {"_ZN4llvm15MachineLoopInfoC2Ev", "llvm::MachineLoopInfo::MachineLoopInfo()"},
    {"_ZNSt8_Rb_treeIPN4llvm17MachineBasicBlockESt4pairIKS2_PNS0_11MachineLoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::MachineBasicBlock*, std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*>, std::_Select1st<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> >, std::less<llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineBasicBlock* const, llvm::MachineLoop*> >*)"},
    {"_ZN4llvm31initializeMachineModuleInfoPassERNS_12PassRegistryE", "llvm::initializeMachineModuleInfoPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21MachineModuleInfoImplD0Ev", "llvm::MachineModuleInfoImpl::~MachineModuleInfoImpl()"},
    {"_ZN4llvm21MachineModuleInfoImplD1Ev", "llvm::MachineModuleInfoImpl::~MachineModuleInfoImpl()"},
    {"_ZN4llvm21MachineModuleInfoImplD2Ev", "llvm::MachineModuleInfoImpl::~MachineModuleInfoImpl()"},
    {"_ZN4llvm15MMIAddrLabelMap18getAddrLabelSymbolEPNS_10BasicBlockE", "llvm::MMIAddrLabelMap::getAddrLabelSymbol(llvm::BasicBlock*)"},
    {"_ZN4llvm15MMIAddrLabelMap24getAddrLabelSymbolToEmitEPNS_10BasicBlockE", "llvm::MMIAddrLabelMap::getAddrLabelSymbolToEmit(llvm::BasicBlock*)"},
    {"_ZN4llvm15MMIAddrLabelMap29takeDeletedSymbolsForFunctionEPNS_8FunctionERSt6vectorIPNS_8MCSymbolESaIS5_EE", "llvm::MMIAddrLabelMap::takeDeletedSymbolsForFunction(llvm::Function*, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >&)"},
    {"_ZN4llvm15MMIAddrLabelMap21UpdateForDeletedBlockEPNS_10BasicBlockE", "llvm::MMIAddrLabelMap::UpdateForDeletedBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm15MMIAddrLabelMap18UpdateForRAUWBlockEPNS_10BasicBlockES2_", "llvm::MMIAddrLabelMap::UpdateForRAUWBlock(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm26MMIAddrLabelMapCallbackPtr7deletedEv", "llvm::MMIAddrLabelMapCallbackPtr::deleted()"},
    {"_ZN4llvm26MMIAddrLabelMapCallbackPtr19allUsesReplacedWithEPNS_5ValueE", "llvm::MMIAddrLabelMapCallbackPtr::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm17MachineModuleInfoC1ERKNS_9MCAsmInfoEPKNS_13TargetAsmInfoE", "llvm::MachineModuleInfo::MachineModuleInfo(llvm::MCAsmInfo const&, llvm::TargetAsmInfo const*)"},
    {"_ZN4llvm17MachineModuleInfoC2ERKNS_9MCAsmInfoEPKNS_13TargetAsmInfoE", "llvm::MachineModuleInfo::MachineModuleInfo(llvm::MCAsmInfo const&, llvm::TargetAsmInfo const*)"},
    {"_ZN4llvm17MachineModuleInfoC1Ev", "llvm::MachineModuleInfo::MachineModuleInfo()"},
    {"_ZN4llvm17MachineModuleInfoC2Ev", "llvm::MachineModuleInfo::MachineModuleInfo()"},
    {"_ZN4llvm17MachineModuleInfoD0Ev", "llvm::MachineModuleInfo::~MachineModuleInfo()"},
    {"_ZN4llvm17MachineModuleInfoD1Ev", "llvm::MachineModuleInfo::~MachineModuleInfo()"},
    {"_ZN4llvm17MachineModuleInfoD2Ev", "llvm::MachineModuleInfo::~MachineModuleInfo()"},
    {"_ZN4llvm17MachineModuleInfo16doInitializationEv", "llvm::MachineModuleInfo::doInitialization()"},
    {"_ZN4llvm17MachineModuleInfo14doFinalizationEv", "llvm::MachineModuleInfo::doFinalization()"},
    {"_ZN4llvm17MachineModuleInfo11EndFunctionEv", "llvm::MachineModuleInfo::EndFunction()"},
    {"_ZN4llvm17MachineModuleInfo13AnalyzeModuleERKNS_6ModuleE", "llvm::MachineModuleInfo::AnalyzeModule(llvm::Module const&)"},
    {"_ZN4llvm17MachineModuleInfo18getAddrLabelSymbolEPKNS_10BasicBlockE", "llvm::MachineModuleInfo::getAddrLabelSymbol(llvm::BasicBlock const*)"},
    {"_ZN4llvm17MachineModuleInfo24getAddrLabelSymbolToEmitEPKNS_10BasicBlockE", "llvm::MachineModuleInfo::getAddrLabelSymbolToEmit(llvm::BasicBlock const*)"},
    {"_ZN4llvm17MachineModuleInfo29takeDeletedSymbolsForFunctionEPKNS_8FunctionERSt6vectorIPNS_8MCSymbolESaIS6_EE", "llvm::MachineModuleInfo::takeDeletedSymbolsForFunction(llvm::Function const*, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >&)"},
    {"_ZN4llvm17MachineModuleInfo25getOrCreateLandingPadInfoEPNS_17MachineBasicBlockE", "llvm::MachineModuleInfo::getOrCreateLandingPadInfo(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineModuleInfo9addInvokeEPNS_17MachineBasicBlockEPNS_8MCSymbolES4_", "llvm::MachineModuleInfo::addInvoke(llvm::MachineBasicBlock*, llvm::MCSymbol*, llvm::MCSymbol*)"},
    {"_ZN4llvm17MachineModuleInfo13addLandingPadEPNS_17MachineBasicBlockE", "llvm::MachineModuleInfo::addLandingPad(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineModuleInfo14addPersonalityEPNS_17MachineBasicBlockEPKNS_8FunctionE", "llvm::MachineModuleInfo::addPersonality(llvm::MachineBasicBlock*, llvm::Function const*)"},
    {"_ZN4llvm17MachineModuleInfo16addCatchTypeInfoEPNS_17MachineBasicBlockERSt6vectorIPKNS_14GlobalVariableESaIS6_EE", "llvm::MachineModuleInfo::addCatchTypeInfo(llvm::MachineBasicBlock*, std::vector<llvm::GlobalVariable const*, std::allocator<llvm::GlobalVariable const*> >&)"},
    {"_ZN4llvm17MachineModuleInfo12getTypeIDForEPKNS_14GlobalVariableE", "llvm::MachineModuleInfo::getTypeIDFor(llvm::GlobalVariable const*)"},
    {"_ZN4llvm17MachineModuleInfo17addFilterTypeInfoEPNS_17MachineBasicBlockERSt6vectorIPKNS_14GlobalVariableESaIS6_EE", "llvm::MachineModuleInfo::addFilterTypeInfo(llvm::MachineBasicBlock*, std::vector<llvm::GlobalVariable const*, std::allocator<llvm::GlobalVariable const*> >&)"},
    {"_ZN4llvm17MachineModuleInfo14getFilterIDForERSt6vectorIjSaIjEE", "llvm::MachineModuleInfo::getFilterIDFor(std::vector<unsigned int, std::allocator<unsigned int> >&)"},
    {"_ZN4llvm17MachineModuleInfo10addCleanupEPNS_17MachineBasicBlockE", "llvm::MachineModuleInfo::addCleanup(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineModuleInfo15TidyLandingPadsEPNS_8DenseMapIPNS_8MCSymbolEmNS_12DenseMapInfoIS3_EENS4_ImEEEE", "llvm::MachineModuleInfo::TidyLandingPads(llvm::DenseMap<llvm::MCSymbol*, unsigned long, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned long> >*)"},
    {"_ZNK4llvm17MachineModuleInfo14getPersonalityEv", "llvm::MachineModuleInfo::getPersonality() const"},
    {"_ZNK4llvm17MachineModuleInfo19getPersonalityIndexEv", "llvm::MachineModuleInfo::getPersonalityIndex() const"},
    {"_ZN4llvm26MMIAddrLabelMapCallbackPtrD1Ev", "llvm::MMIAddrLabelMapCallbackPtr::~MMIAddrLabelMapCallbackPtr()"},
    {"_ZNSt6vectorIPN4llvm8MCSymbolESaIS2_EEaSERKS4_", "std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >::operator=(std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > const&)"},
    {"_ZNSt6vectorIN4llvm14LandingPadInfoESaIS1_EE5clearEv", "std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> >::clear()"},
    {"_ZNSt6vectorIN4llvm14LandingPadInfoESaIS1_EE5eraseEN9__gnu_cxx17__normal_iteratorIPS1_S3_EE", "std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> >::erase(__gnu_cxx::__normal_iterator<llvm::LandingPadInfo*, std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> > >)"},
    {"_ZN4llvm26MMIAddrLabelMapCallbackPtrD0Ev", "llvm::MMIAddrLabelMapCallbackPtr::~MMIAddrLabelMapCallbackPtr()"},
    {"_ZNSt6vectorIPKN4llvm14GlobalVariableESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::GlobalVariable const*, std::allocator<llvm::GlobalVariable const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::GlobalVariable const**, std::vector<llvm::GlobalVariable const*, std::allocator<llvm::GlobalVariable const*> > >, llvm::GlobalVariable const* const&)"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPN4llvm14LandingPadInfoES5_EET0_T_S7_S6_", "llvm::LandingPadInfo* std::__copy<false, std::random_access_iterator_tag>::copy<llvm::LandingPadInfo*, llvm::LandingPadInfo*>(llvm::LandingPadInfo*, llvm::LandingPadInfo*, llvm::LandingPadInfo*)"},
    {"_ZNSt6vectorIiSaIiEEaSERKS1_", "std::vector<int, std::allocator<int> >::operator=(std::vector<int, std::allocator<int> > const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_8MCSymbolEEaSERKS3_", "llvm::SmallVectorImpl<llvm::MCSymbol*>::operator=(llvm::SmallVectorImpl<llvm::MCSymbol*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolEmNS_12DenseMapInfoIS2_EENS3_ImEEE16InsertIntoBucketERKS2_RKmPSt4pairIS2_mE", "llvm::DenseMap<llvm::MCSymbol*, unsigned long, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned long> >::InsertIntoBucket(llvm::MCSymbol* const&, unsigned long const&, std::pair<llvm::MCSymbol*, unsigned long>*)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolEmNS_12DenseMapInfoIS2_EENS3_ImEEE4growEj", "llvm::DenseMap<llvm::MCSymbol*, unsigned long, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned long> >::grow(unsigned int)"},
    {"_ZNSt6vectorIiSaIiEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPiS1_EERKi", "std::vector<int, std::allocator<int> >::_M_insert_aux(__gnu_cxx::__normal_iterator<int*, std::vector<int, std::allocator<int> > >, int const&)"},
    {"_ZNSt6vectorIN4llvm14LandingPadInfoESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LandingPadInfo*, std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> > >, llvm::LandingPadInfo const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm14LandingPadInfoES5_EET0_T_S7_S6_", "llvm::LandingPadInfo* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::LandingPadInfo*, llvm::LandingPadInfo*>(llvm::LandingPadInfo*, llvm::LandingPadInfo*, llvm::LandingPadInfo*)"},
    {"_ZN4llvm14LandingPadInfoC2ERKS0_", "llvm::LandingPadInfo::LandingPadInfo(llvm::LandingPadInfo const&)"},
    {"_ZN4llvm8DenseMapIPNS_8MCSymbolEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MCSymbol*, unsigned int, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZNSt6vectorIPKN4llvm8FunctionESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::Function const*, std::allocator<llvm::Function const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Function const**, std::vector<llvm::Function const*, std::allocator<llvm::Function const*> > >, llvm::Function const* const&)"},
    {"_ZNSt6vectorIN4llvm14LandingPadInfoESaIS1_EED2Ev", "std::vector<llvm::LandingPadInfo, std::allocator<llvm::LandingPadInfo> >::~vector()"},
    {"_ZNSt6vectorIPN4llvm8MCSymbolESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS2_S4_EEEEvS9_T_SA_St20forward_iterator_tag", "void std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::MCSymbol**, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > > >(__gnu_cxx::__normal_iterator<llvm::MCSymbol**, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > >, __gnu_cxx::__normal_iterator<llvm::MCSymbol**, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > >, __gnu_cxx::__normal_iterator<llvm::MCSymbol**, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > >, std::forward_iterator_tag)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_8FunctionEEESt6vectorIPNS_8MCSymbolESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::AssertingVH<llvm::Function>, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Function> >, llvm::DenseMapInfo<std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > > >::FindAndConstruct(llvm::AssertingVH<llvm::Function> const&)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_8FunctionEEESt6vectorIPNS_8MCSymbolESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE16InsertIntoBucketERKS3_RKS8_PSt4pairIS3_S8_E", "llvm::DenseMap<llvm::AssertingVH<llvm::Function>, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Function> >, llvm::DenseMapInfo<std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > > >::InsertIntoBucket(llvm::AssertingVH<llvm::Function> const&, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > const&, std::pair<llvm::AssertingVH<llvm::Function>, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > >*)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_8FunctionEEESt6vectorIPNS_8MCSymbolESaIS6_EENS_12DenseMapInfoIS3_EENS9_IS8_EEE4growEj", "llvm::DenseMap<llvm::AssertingVH<llvm::Function>, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Function> >, llvm::DenseMapInfo<std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > > >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm8MCSymbolESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCSymbol**, std::vector<llvm::MCSymbol*, std::allocator<llvm::MCSymbol*> > >, llvm::MCSymbol* const&)"},
    {"_ZNSt6vectorIN4llvm26MMIAddrLabelMapCallbackPtrESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MMIAddrLabelMapCallbackPtr, std::allocator<llvm::MMIAddrLabelMapCallbackPtr> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MMIAddrLabelMapCallbackPtr*, std::vector<llvm::MMIAddrLabelMapCallbackPtr, std::allocator<llvm::MMIAddrLabelMapCallbackPtr> > >, llvm::MMIAddrLabelMapCallbackPtr const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm26MMIAddrLabelMapCallbackPtrES5_EET0_T_S7_S6_", "llvm::MMIAddrLabelMapCallbackPtr* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::MMIAddrLabelMapCallbackPtr*, llvm::MMIAddrLabelMapCallbackPtr*>(llvm::MMIAddrLabelMapCallbackPtr*, llvm::MMIAddrLabelMapCallbackPtr*, llvm::MMIAddrLabelMapCallbackPtr*)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_10BasicBlockEEENS_15MMIAddrLabelMap17AddrLabelSymEntryENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::AssertingVH<llvm::BasicBlock>, llvm::MMIAddrLabelMap::AddrLabelSymEntry, llvm::DenseMapInfo<llvm::AssertingVH<llvm::BasicBlock> >, llvm::DenseMapInfo<llvm::MMIAddrLabelMap::AddrLabelSymEntry> >::InsertIntoBucket(llvm::AssertingVH<llvm::BasicBlock> const&, llvm::MMIAddrLabelMap::AddrLabelSymEntry const&, std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::MMIAddrLabelMap::AddrLabelSymEntry>*)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_10BasicBlockEEENS_15MMIAddrLabelMap17AddrLabelSymEntryENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::AssertingVH<llvm::BasicBlock>, llvm::MMIAddrLabelMap::AddrLabelSymEntry, llvm::DenseMapInfo<llvm::AssertingVH<llvm::BasicBlock> >, llvm::DenseMapInfo<llvm::MMIAddrLabelMap::AddrLabelSymEntry> >::grow(unsigned int)"},
    {"_ZN4llvm15MMIAddrLabelMapC2ERNS_9MCContextE", "llvm::MMIAddrLabelMap::MMIAddrLabelMap(llvm::MCContext&)"},
    {"_ZN4llvm15MMIAddrLabelMapD2Ev", "llvm::MMIAddrLabelMap::~MMIAddrLabelMap()"},
    {"_ZN4llvm15callDefaultCtorINS_17MachineModuleInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::MachineModuleInfo>()"},
    {"_ZN4llvm22MachineModuleInfoMachO6AnchorEv", "llvm::MachineModuleInfoMachO::Anchor()"},
    {"_ZN4llvm20MachineModuleInfoELF6AnchorEv", "llvm::MachineModuleInfoELF::Anchor()"},
    {"_ZN4llvm21MachineModuleInfoImpl14GetSortedStubsERKNS_8DenseMapIPNS_8MCSymbolENS_14PointerIntPairIS3_Lj1EbNS_21PointerLikeTypeTraitsIS3_EEEENS_12DenseMapInfoIS3_EENS8_IS7_EEEE", "llvm::MachineModuleInfoImpl::GetSortedStubs(llvm::DenseMap<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > > const&)"},
    {"_Z14SortSymbolPairPKvS0_", "SortSymbolPair(void const*, void const*)"},
    {"_ZN4llvm22MachineModuleInfoMachOD1Ev", "llvm::MachineModuleInfoMachO::~MachineModuleInfoMachO()"},
    {"_ZN4llvm22MachineModuleInfoMachOD0Ev", "llvm::MachineModuleInfoMachO::~MachineModuleInfoMachO()"},
    {"_ZN4llvm20MachineModuleInfoELFD1Ev", "llvm::MachineModuleInfoELF::~MachineModuleInfoELF()"},
    {"_ZN4llvm20MachineModuleInfoELFD0Ev", "llvm::MachineModuleInfoELF::~MachineModuleInfoELF()"},
    {"_ZNSt6vectorISt4pairIPN4llvm8MCSymbolENS1_14PointerIntPairIS3_Lj1EbNS1_21PointerLikeTypeTraitsIS3_EEEEESaIS8_EE19_M_range_initializeINS1_16DenseMapIteratorIS3_S7_NS1_12DenseMapInfoIS3_EENSD_IS7_EELb1EEEEEvT_SH_St20forward_iterator_tag", "void std::vector<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, std::allocator<std::pair<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > > > >::_M_range_initialize<llvm::DenseMapIterator<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, true> >(llvm::DenseMapIterator<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, true>, llvm::DenseMapIterator<llvm::MCSymbol*, llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> >, llvm::DenseMapInfo<llvm::MCSymbol*>, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::MCSymbol*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::MCSymbol*> > >, true>, std::forward_iterator_tag)"},
    {"_ZN4llvm19MachinePassRegistry6RemoveEPNS_23MachinePassRegistryNodeE", "llvm::MachinePassRegistry::Remove(llvm::MachinePassRegistryNode*)"},
    {"_ZN4llvm19MachineRegisterInfoC1ERKNS_18TargetRegisterInfoE", "llvm::MachineRegisterInfo::MachineRegisterInfo(llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm19MachineRegisterInfoC2ERKNS_18TargetRegisterInfoE", "llvm::MachineRegisterInfo::MachineRegisterInfo(llvm::TargetRegisterInfo const&)"},
    {"_ZN4llvm19MachineRegisterInfoD1Ev", "llvm::MachineRegisterInfo::~MachineRegisterInfo()"},
    {"_ZN4llvm19MachineRegisterInfoD2Ev", "llvm::MachineRegisterInfo::~MachineRegisterInfo()"},
    {"_ZN4llvm19MachineRegisterInfo11setRegClassEjPKNS_19TargetRegisterClassE", "llvm::MachineRegisterInfo::setRegClass(unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm19MachineRegisterInfo17constrainRegClassEjPKNS_19TargetRegisterClassE", "llvm::MachineRegisterInfo::constrainRegClass(unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm19MachineRegisterInfo21createVirtualRegisterEPKNS_19TargetRegisterClassE", "llvm::MachineRegisterInfo::createVirtualRegister(llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm19MachineRegisterInfo26HandleVRegListReallocationEv", "llvm::MachineRegisterInfo::HandleVRegListReallocation()"},
    {"_ZN4llvm19MachineRegisterInfo14replaceRegWithEjj", "llvm::MachineRegisterInfo::replaceRegWith(unsigned int, unsigned int)"},
    {"_ZNK4llvm19MachineRegisterInfo10getVRegDefEj", "llvm::MachineRegisterInfo::getVRegDef(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo9hasOneUseEj", "llvm::MachineRegisterInfo::hasOneUse(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo15hasOneNonDBGUseEj", "llvm::MachineRegisterInfo::hasOneNonDBGUse(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo14clearKillFlagsEj", "llvm::MachineRegisterInfo::clearKillFlags(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo8isLiveInEj", "llvm::MachineRegisterInfo::isLiveIn(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo9isLiveOutEj", "llvm::MachineRegisterInfo::isLiveOut(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo16getLiveInPhysRegEj", "llvm::MachineRegisterInfo::getLiveInPhysReg(unsigned int) const"},
    {"_ZNK4llvm19MachineRegisterInfo16getLiveInVirtRegEj", "llvm::MachineRegisterInfo::getLiveInVirtReg(unsigned int) const"},
    {"_ZN4llvm19MachineRegisterInfo16EmitLiveInCopiesEPNS_17MachineBasicBlockERKNS_18TargetRegisterInfoERKNS_15TargetInstrInfoE", "llvm::MachineRegisterInfo::EmitLiveInCopies(llvm::MachineBasicBlock*, llvm::TargetRegisterInfo const&, llvm::TargetInstrInfo const&)"},
    {"_ZN4llvm19MachineRegisterInfo17closePhysRegsUsedERKNS_18TargetRegisterInfoE", "llvm::MachineRegisterInfo::closePhysRegsUsed(llvm::TargetRegisterInfo const&)"},
    {"_ZNSt6vectorISt4pairIjjESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<std::pair<unsigned int, unsigned int>, std::allocator<std::pair<unsigned int, unsigned int> > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::pair<unsigned int, unsigned int>*, std::vector<std::pair<unsigned int, unsigned int>, std::allocator<std::pair<unsigned int, unsigned int> > > >, unsigned long, std::pair<unsigned int, unsigned int> const&)"},
    {"_ZNSt6vectorISt4pairIPKN4llvm19TargetRegisterClassEPNS1_14MachineOperandEESaIS7_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS7_S9_EEmRKS7_", "std::vector<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*>, std::allocator<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*> > >::_M_fill_insert(__gnu_cxx::__normal_iterator<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*>*, std::vector<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*>, std::allocator<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*> > > >, unsigned long, std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*> const&)"},
    {"_ZNSt6vectorISt4pairIjjESaIS1_EE7reserveEm", "std::vector<std::pair<unsigned int, unsigned int>, std::allocator<std::pair<unsigned int, unsigned int> > >::reserve(unsigned long)"},
    {"_ZNSt6vectorISt4pairIPKN4llvm19TargetRegisterClassEPNS1_14MachineOperandEESaIS7_EE7reserveEm", "std::vector<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*>, std::allocator<std::pair<llvm::TargetRegisterClass const*, llvm::MachineOperand*> > >::reserve(unsigned long)"},
    {"_ZN4llvm17MachineSSAUpdaterC1ERNS_15MachineFunctionEPNS_15SmallVectorImplIPNS_12MachineInstrEEE", "llvm::MachineSSAUpdater::MachineSSAUpdater(llvm::MachineFunction&, llvm::SmallVectorImpl<llvm::MachineInstr*>*)"},
    {"_ZN4llvm17MachineSSAUpdaterC2ERNS_15MachineFunctionEPNS_15SmallVectorImplIPNS_12MachineInstrEEE", "llvm::MachineSSAUpdater::MachineSSAUpdater(llvm::MachineFunction&, llvm::SmallVectorImpl<llvm::MachineInstr*>*)"},
    {"_ZN4llvm17MachineSSAUpdaterD1Ev", "llvm::MachineSSAUpdater::~MachineSSAUpdater()"},
    {"_ZN4llvm17MachineSSAUpdaterD2Ev", "llvm::MachineSSAUpdater::~MachineSSAUpdater()"},
    {"_ZN4llvm17MachineSSAUpdater10InitializeEj", "llvm::MachineSSAUpdater::Initialize(unsigned int)"},
    {"_ZNK4llvm17MachineSSAUpdater16HasValueForBlockEPNS_17MachineBasicBlockE", "llvm::MachineSSAUpdater::HasValueForBlock(llvm::MachineBasicBlock*) const"},
    {"_ZN4llvm17MachineSSAUpdater17AddAvailableValueEPNS_17MachineBasicBlockEj", "llvm::MachineSSAUpdater::AddAvailableValue(llvm::MachineBasicBlock*, unsigned int)"},
    {"_ZN4llvm17MachineSSAUpdater20GetValueAtEndOfBlockEPNS_17MachineBasicBlockE", "llvm::MachineSSAUpdater::GetValueAtEndOfBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineSSAUpdater28GetValueAtEndOfBlockInternalEPNS_17MachineBasicBlockE", "llvm::MachineSSAUpdater::GetValueAtEndOfBlockInternal(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17MachineSSAUpdater23GetValueInMiddleOfBlockEPNS_17MachineBasicBlockE", "llvm::MachineSSAUpdater::GetValueInMiddleOfBlock(llvm::MachineBasicBlock*)"},
    {"_Z12InsertNewDefjPN4llvm17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEPKNS_19TargetRegisterClassEPNS_19MachineRegisterInfoEPKNS_15TargetInstrInfoE", "InsertNewDef(unsigned int, llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>, llvm::TargetRegisterClass const*, llvm::MachineRegisterInfo*, llvm::TargetInstrInfo const*)"},
    {"_ZN4llvm17MachineSSAUpdater10RewriteUseERNS_14MachineOperandE", "llvm::MachineSSAUpdater::RewriteUse(llvm::MachineOperand&)"},
    {"_ZN4llvm17MachineSSAUpdater14ReplaceRegWithEjj", "llvm::MachineSSAUpdater::ReplaceRegWith(unsigned int, unsigned int)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE8GetValueEPNS_17MachineBasicBlockE", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::GetValue(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE14BuildBlockListEPNS_17MachineBasicBlockEPNS_15SmallVectorImplIPNS2_6BBInfoEEE", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BuildBlockList(llvm::MachineBasicBlock*, llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE14FindDominatorsEPNS_15SmallVectorImplIPNS2_6BBInfoEEES5_", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::FindDominators(llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*>*, llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE17FindAvailableValsEPNS_15SmallVectorImplIPNS2_6BBInfoEEE", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::FindAvailableVals(llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_14SSAUpdaterImplINS_17MachineSSAUpdaterEE6BBInfoENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo* const&, std::pair<llvm::MachineBasicBlock*, llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_14SSAUpdaterImplINS_17MachineSSAUpdaterEE6BBInfoENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::BBInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE17CheckIfPHIMatchesEPNS_12MachineInstrE", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::CheckIfPHIMatches(llvm::MachineInstr*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_17MachineSSAUpdaterEE17RecordMatchingPHIEPNS_12MachineInstrE", "llvm::SSAUpdaterImpl<llvm::MachineSSAUpdater>::RecordMatchingPHI(llvm::MachineInstr*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::MachineBasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, unsigned int const&, std::pair<llvm::MachineBasicBlock*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm28initializeMachineSinkingPassERNS_12PassRegistryE", "llvm::initializeMachineSinkingPass(llvm::PassRegistry&)"},
    {"_ZN4llvm24createMachineSinkingPassEv", "llvm::createMachineSinkingPass()"},
    {"_ZN12_GLOBAL__N_114MachineSinkingD1Ev", "(anonymous namespace)::MachineSinking::~MachineSinking()"},
    {"_ZN12_GLOBAL__N_114MachineSinkingD0Ev", "(anonymous namespace)::MachineSinking::~MachineSinking()"},
    {"_ZNK12_GLOBAL__N_114MachineSinking16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineSinking::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114MachineSinking13releaseMemoryEv", "(anonymous namespace)::MachineSinking::releaseMemory()"},
    {"_ZN12_GLOBAL__N_114MachineSinking20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineSinking::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNK12_GLOBAL__N_114MachineSinking23AllUsesDominatedByBlockEjPN4llvm17MachineBasicBlockES3_RbS4_", "(anonymous namespace)::MachineSinking::AllUsesDominatedByBlock(unsigned int, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, bool&, bool&) const"},
    {"_ZN12_GLOBAL__N_114MachineSinking17SplitCriticalEdgeEPN4llvm12MachineInstrEPNS1_17MachineBasicBlockES5_b", "(anonymous namespace)::MachineSinking::SplitCriticalEdge(llvm::MachineInstr*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, bool)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114MachineSinkingEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::MachineSinking>()"},
    {"_ZN4llvm8SmallSetISt4pairIPNS_17MachineBasicBlockES3_ELj8EE6insertERKS4_", "llvm::SmallSet<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>, 8u>::insert(std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPN4llvm17MachineBasicBlockES3_ES4_St9_IdentityIS4_ESt4lessIS4_ESaIS4_EE16_M_insert_uniqueERKS4_", "std::_Rb_tree<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>, std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>, std::_Identity<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> >, std::less<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> >, std::allocator<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> > >::_M_insert_unique(std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPN4llvm17MachineBasicBlockES3_ES4_St9_IdentityIS4_ESt4lessIS4_ESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>, std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*>, std::_Identity<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> >, std::less<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> >, std::allocator<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineBasicBlock*, llvm::MachineBasicBlock*> >*)"},
    {"_ZN4llvm33initializeMachineVerifierPassPassERNS_12PassRegistryE", "llvm::initializeMachineVerifierPassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm25createMachineVerifierPassEPKc", "llvm::createMachineVerifierPass(char const*)"},
    {"_ZNK4llvm15MachineFunction6verifyEPNS_4PassEPKc", "llvm::MachineFunction::verify(llvm::Pass*, char const*) const"},
    {"_ZN12_GLOBAL__N_115MachineVerifier20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineVerifier::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier6reportEPKcPKN4llvm17MachineBasicBlockE", "(anonymous namespace)::MachineVerifier::report(char const*, llvm::MachineBasicBlock const*)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier25visitMachineFunctionAfterEv", "(anonymous namespace)::MachineVerifier::visitMachineFunctionAfter()"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockEN12_GLOBAL__N_115MachineVerifier6BBInfoENS_12DenseMapInfoIS3_EENS7_IS6_EEEixERKS3_", "llvm::DenseMap<llvm::MachineBasicBlock const*, (anonymous namespace)::MachineVerifier::BBInfo, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<(anonymous namespace)::MachineVerifier::BBInfo> >::operator[](llvm::MachineBasicBlock const* const&)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier6reportEPKcPKN4llvm15MachineFunctionE", "(anonymous namespace)::MachineVerifier::report(char const*, llvm::MachineFunction const*)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier6reportEPKcPKN4llvm12MachineInstrE", "(anonymous namespace)::MachineVerifier::report(char const*, llvm::MachineInstr const*)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier6BBInfo9addPassedERKN4llvm8DenseSetIjNS2_12DenseMapInfoIjEEEE", "(anonymous namespace)::MachineVerifier::BBInfo::addPassed(llvm::DenseSet<unsigned int, llvm::DenseMapInfo<unsigned int> > const&)"},
    {"_ZN12_GLOBAL__N_115MachineVerifier13markReachableEPKN4llvm17MachineBasicBlockE", "(anonymous namespace)::MachineVerifier::markReachable(llvm::MachineBasicBlock const*)"},
    {"_ZN12_GLOBAL__N_119MachineVerifierPassD1Ev", "(anonymous namespace)::MachineVerifierPass::~MachineVerifierPass()"},
    {"_ZN12_GLOBAL__N_119MachineVerifierPassD0Ev", "(anonymous namespace)::MachineVerifierPass::~MachineVerifierPass()"},
    {"_ZNK12_GLOBAL__N_119MachineVerifierPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MachineVerifierPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_119MachineVerifierPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::MachineVerifierPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_119MachineVerifierPassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::MachineVerifierPass>()"},
    {"_ZN4llvm8DenseMapIjcNS_12DenseMapInfoIjEENS1_IcEEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> >::shrink_and_clear()"},
    {"_ZNK4llvm11SlotIndexes15getMBBFromIndexENS_9SlotIndexE", "llvm::SlotIndexes::getMBBFromIndex(llvm::SlotIndex) const"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<llvm::MachineBasicBlock const*, char, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::MachineBasicBlock const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<llvm::MachineBasicBlock const*, char, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::MachineBasicBlock const* const&, char const&, std::pair<llvm::MachineBasicBlock const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock const*, char, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjcNS_12DenseMapInfoIjEENS1_IcEEE8CopyFromERKS4_", "llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> >::CopyFrom(llvm::DenseMap<unsigned int, char, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<char> > const&)"},
    {"_ZN4llvm8DenseMapIjPKNS_12MachineInstrENS_12DenseMapInfoIjEENS4_IS3_EEE8CopyFromERKS7_", "llvm::DenseMap<unsigned int, llvm::MachineInstr const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::CopyFrom(llvm::DenseMap<unsigned int, llvm::MachineInstr const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr const*> > const&)"},
    {"_ZN4llvm8DenseMapIjPKNS_12MachineInstrENS_12DenseMapInfoIjEENS4_IS3_EEE6insertERKSt4pairIjS3_E", "llvm::DenseMap<unsigned int, llvm::MachineInstr const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr const*> >::insert(std::pair<unsigned int, llvm::MachineInstr const*> const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm12MachineInstrESt6vectorIS4_SaIS4_EEEEPKS3_ET_SC_SC_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, llvm::MachineInstr const*>(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, __gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, llvm::MachineInstr const* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm26initializeOptimizePHIsPassERNS_12PassRegistryE", "llvm::initializeOptimizePHIsPass(llvm::PassRegistry&)"},
    {"_ZN4llvm22createOptimizePHIsPassEv", "llvm::createOptimizePHIsPass()"},
    {"_ZN12_GLOBAL__N_112OptimizePHIsD1Ev", "(anonymous namespace)::OptimizePHIs::~OptimizePHIs()"},
    {"_ZN12_GLOBAL__N_112OptimizePHIsD0Ev", "(anonymous namespace)::OptimizePHIs::~OptimizePHIs()"},
    {"_ZNK12_GLOBAL__N_112OptimizePHIs16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::OptimizePHIs::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_112OptimizePHIs20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::OptimizePHIs::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_112OptimizePHIs21IsSingleValuePHICycleEPN4llvm12MachineInstrERjRNS1_11SmallPtrSetIS3_Lj16EEE", "(anonymous namespace)::OptimizePHIs::IsSingleValuePHICycle(llvm::MachineInstr*, unsigned int&, llvm::SmallPtrSet<llvm::MachineInstr*, 16u>&)"},
    {"_ZN12_GLOBAL__N_112OptimizePHIs14IsDeadPHICycleEPN4llvm12MachineInstrERNS1_11SmallPtrSetIS3_Lj16EEE", "(anonymous namespace)::OptimizePHIs::IsDeadPHICycle(llvm::MachineInstr*, llvm::SmallPtrSet<llvm::MachineInstr*, 16u>&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112OptimizePHIsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::OptimizePHIs>()"},
    {"_ZN4llvm28initializePHIEliminationPassERNS_12PassRegistryE", "llvm::initializePHIEliminationPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114PHIEliminationEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PHIElimination>()"},
    {"_ZN12_GLOBAL__N_114PHIEliminationD1Ev", "(anonymous namespace)::PHIElimination::~PHIElimination()"},
    {"_ZN12_GLOBAL__N_114PHIEliminationD0Ev", "(anonymous namespace)::PHIElimination::~PHIElimination()"},
    {"_ZNK12_GLOBAL__N_114PHIElimination16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PHIElimination::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114PHIElimination20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::PHIElimination::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm8DenseMapISt4pairIjjEjNS_12DenseMapInfoIS2_EENS3_IjEEE5clearEv", "llvm::DenseMap<std::pair<unsigned int, unsigned int>, unsigned int, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> >, llvm::DenseMapInfo<unsigned int> >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairIjjEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<std::pair<unsigned int, unsigned int>, unsigned int, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> >, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_12DenseMapInfoIjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm15SparseBitVectorILj128EE5resetEj", "llvm::SparseBitVector<128u>::reset(unsigned int)"},
    {"_ZNK4llvm8DenseMapISt4pairIjjEjNS_12DenseMapInfoIS2_EENS3_IjEEE15LookupBucketForERKS2_RPS1_IS2_jE", "llvm::DenseMap<std::pair<unsigned int, unsigned int>, unsigned int, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> >, llvm::DenseMapInfo<unsigned int> >::LookupBucketFor(std::pair<unsigned int, unsigned int> const&, std::pair<std::pair<unsigned int, unsigned int>, unsigned int>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIjjEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPS1_IS2_jE", "llvm::DenseMap<std::pair<unsigned int, unsigned int>, unsigned int, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> >, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(std::pair<unsigned int, unsigned int> const&, unsigned int const&, std::pair<std::pair<unsigned int, unsigned int>, unsigned int>*)"},
    {"_ZN4llvm8DenseMapISt4pairIjjEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<std::pair<unsigned int, unsigned int>, unsigned int, llvm::DenseMapInfo<std::pair<unsigned int, unsigned int> >, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_12DenseMapInfoIjEEE15LookupBucketForERKS2_RPSt4pairIS2_jE", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<unsigned int> >::LookupBucketFor(llvm::MachineInstr* const&, std::pair<llvm::MachineInstr*, unsigned int>*&) const"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_12DenseMapInfoIjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MachineInstr* const&, unsigned int const&, std::pair<llvm::MachineInstr*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEjNS_27MachineInstrExpressionTraitENS_12DenseMapInfoIjEEE4growEj", "llvm::DenseMap<llvm::MachineInstr*, unsigned int, llvm::MachineInstrExpressionTrait, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm22findPHICopyInsertPointEPNS_17MachineBasicBlockES1_j", "llvm::findPHICopyInsertPoint(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, unsigned int)"},
    {"_Z30createDefaultRegisterAllocatorv", "createDefaultRegisterAllocator()"},
    {"_ZN4llvm23createRegisterAllocatorENS_10CodeGenOpt5LevelE", "llvm::createRegisterAllocator(llvm::CodeGenOpt::Level)"},
    {"_ZN4llvm16RegisterRegAllocD1Ev", "llvm::RegisterRegAlloc::~RegisterRegAlloc()"},
    {"_ZN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEED1Ev", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::~opt()"},
    {"_ZN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE4doneEv", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::done()"},
    {"_ZN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE16handleOccurrenceEjNS_9StringRefESA_", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE14getOptionWidthEv", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE15printOptionInfoEm", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEED0Ev", "llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >::~opt()"},
    {"_ZN4llvm2cl6parserIPFPNS_12FunctionPassEvEE5parseERNS0_6OptionENS_9StringRefES9_RS5_", "llvm::cl::parser<llvm::FunctionPass* (*)()>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::FunctionPass* (*&)())"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserIPFPNS_12FunctionPassEvEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::FunctionPass* (*)()>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm18RegisterPassParserINS_16RegisterRegAllocEED1Ev", "llvm::RegisterPassParser<llvm::RegisterRegAlloc>::~RegisterPassParser()"},
    {"_ZN4llvm18RegisterPassParserINS_16RegisterRegAllocEED0Ev", "llvm::RegisterPassParser<llvm::RegisterRegAlloc>::~RegisterPassParser()"},
    {"_ZN4llvm18RegisterPassParserINS_16RegisterRegAllocEE9NotifyAddEPKcPFPvvES4_", "llvm::RegisterPassParser<llvm::RegisterRegAlloc>::NotifyAdd(char const*, void* (*)(), char const*)"},
    {"_ZN4llvm18RegisterPassParserINS_16RegisterRegAllocEE12NotifyRemoveEPKc", "llvm::RegisterPassParser<llvm::RegisterRegAlloc>::NotifyRemove(char const*)"},
    {"_ZThn16_N4llvm18RegisterPassParserINS_16RegisterRegAllocEED1Ev", "non-virtual thunk to llvm::RegisterPassParser<llvm::RegisterRegAlloc>::~RegisterPassParser()"},
    {"_ZThn16_N4llvm18RegisterPassParserINS_16RegisterRegAllocEED0Ev", "non-virtual thunk to llvm::RegisterPassParser<llvm::RegisterRegAlloc>::~RegisterPassParser()"},
    {"_ZNK4llvm2cl6parserIPFPNS_12FunctionPassEvEE13getNumOptionsEv", "llvm::cl::parser<llvm::FunctionPass* (*)()>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserIPFPNS_12FunctionPassEvEE9getOptionEj", "llvm::cl::parser<llvm::FunctionPass* (*)()>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserIPFPNS_12FunctionPassEvEE14getDescriptionEj", "llvm::cl::parser<llvm::FunctionPass* (*)()>::getDescription(unsigned int) const"},
    {"_ZN4llvm2cl6parserIPFPNS_12FunctionPassEvEED1Ev", "llvm::cl::parser<llvm::FunctionPass* (*)()>::~parser()"},
    {"_ZN4llvm2cl6parserIPFPNS_12FunctionPassEvEED0Ev", "llvm::cl::parser<llvm::FunctionPass* (*)()>::~parser()"},
    {"_ZN4llvm31initializePeepholeOptimizerPassERNS_12PassRegistryE", "llvm::initializePeepholeOptimizerPass(llvm::PassRegistry&)"},
    {"_ZN4llvm27createPeepholeOptimizerPassEv", "llvm::createPeepholeOptimizerPass()"},
    {"_ZN12_GLOBAL__N_117PeepholeOptimizerD1Ev", "(anonymous namespace)::PeepholeOptimizer::~PeepholeOptimizer()"},
    {"_ZN12_GLOBAL__N_117PeepholeOptimizerD0Ev", "(anonymous namespace)::PeepholeOptimizer::~PeepholeOptimizer()"},
    {"_ZNK12_GLOBAL__N_117PeepholeOptimizer16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PeepholeOptimizer::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_117PeepholeOptimizer20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::PeepholeOptimizer::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_117PeepholeOptimizerEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PeepholeOptimizer>()"},
    {"_ZN4llvm8DenseMapIjPNS_12MachineInstrENS_12DenseMapInfoIjEENS3_IS2_EEE6insertERKSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::MachineInstr*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr*> >::insert(std::pair<unsigned int, llvm::MachineInstr*> const&)"},
    {"_ZN4llvm8DenseMapIjPNS_12MachineInstrENS_12DenseMapInfoIjEENS3_IS2_EEE16InsertIntoBucketERKjRKS2_PSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::MachineInstr*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr*> >::InsertIntoBucket(unsigned int const&, llvm::MachineInstr* const&, std::pair<unsigned int, llvm::MachineInstr*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_12MachineInstrENS_12DenseMapInfoIjEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::MachineInstr*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjPNS_12MachineInstrENS_12DenseMapInfoIjEENS3_IS2_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, llvm::MachineInstr*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MachineInstr*> >::shrink_and_clear()"},
    {"_ZN4llvm14AntiDepBreakerD0Ev", "llvm::AntiDepBreaker::~AntiDepBreaker()"},
    {"_ZN4llvm14AntiDepBreakerD1Ev", "llvm::AntiDepBreaker::~AntiDepBreaker()"},
    {"_ZN4llvm14AntiDepBreakerD2Ev", "llvm::AntiDepBreaker::~AntiDepBreaker()"},
    {"_ZN4llvm21createPostRASchedulerENS_10CodeGenOpt5LevelE", "llvm::createPostRAScheduler(llvm::CodeGenOpt::Level)"},
    {"_ZN12_GLOBAL__N_115PostRASchedulerD1Ev", "(anonymous namespace)::PostRAScheduler::~PostRAScheduler()"},
    {"_ZN12_GLOBAL__N_115PostRASchedulerD0Ev", "(anonymous namespace)::PostRAScheduler::~PostRAScheduler()"},
    {"_ZNK12_GLOBAL__N_115PostRAScheduler11getPassNameEv", "(anonymous namespace)::PostRAScheduler::getPassName() const"},
    {"_ZNK12_GLOBAL__N_115PostRAScheduler16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PostRAScheduler::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_115PostRAScheduler20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::PostRAScheduler::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDList10StartBlockEPN4llvm17MachineBasicBlockE", "(anonymous namespace)::SchedulePostRATDList::StartBlock(llvm::MachineBasicBlock*)"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDList11FinishBlockEv", "(anonymous namespace)::SchedulePostRATDList::FinishBlock()"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDListD1Ev", "(anonymous namespace)::SchedulePostRATDList::~SchedulePostRATDList()"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDListD0Ev", "(anonymous namespace)::SchedulePostRATDList::~SchedulePostRATDList()"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDList8ScheduleEv", "(anonymous namespace)::SchedulePostRATDList::Schedule()"},
    {"_ZN12_GLOBAL__N_120SchedulePostRATDList17ReleaseSuccessorsEPN4llvm5SUnitE", "(anonymous namespace)::SchedulePostRATDList::ReleaseSuccessors(llvm::SUnit*)"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEEC2IA24_cNS0_4descENS0_11initializerIA5_cEENS0_12OptionHiddenEEERKT_RKT0_RKT1_RKT2_", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::opt<char [24], llvm::cl::desc, llvm::cl::initializer<char [5]>, llvm::cl::OptionHidden>(char const (&) [24], llvm::cl::desc const&, llvm::cl::initializer<char [5]> const&, llvm::cl::OptionHidden const&)"},
    {"_ZN4llvm20LatencyPriorityQueueD1Ev", "llvm::LatencyPriorityQueue::~LatencyPriorityQueue()"},
    {"_ZN4llvm17ScheduleDAGInstrsD2Ev", "llvm::ScheduleDAGInstrs::~ScheduleDAGInstrs()"},
    {"_ZNK4llvm11ScheduleDAG22addCustomGraphFeaturesERNS_11GraphWriterIPS0_EE", "llvm::ScheduleDAG::addCustomGraphFeatures(llvm::GraphWriter<llvm::ScheduleDAG*>&) const"},
    {"_ZNK4llvm11ScheduleDAG18ForceUnitLatenciesEv", "llvm::ScheduleDAG::ForceUnitLatencies() const"},
    {"_ZNSt6vectorIPN4llvm5SUnitESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SUnit**, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, llvm::SUnit* const&)"},
    {"_ZN4llvm15SmallVectorImplINS_4SDepEEaSERKS2_", "llvm::SmallVectorImpl<llvm::SDep>::operator=(llvm::SmallVectorImpl<llvm::SDep> const&)"},
    {"_ZNSt6vectorIN4llvm5SUnitESaIS1_EE15_M_erase_at_endEPS1_", "std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >::_M_erase_at_end(llvm::SUnit*)"},
    {"_ZN4llvm17ScheduleDAGInstrsD1Ev", "llvm::ScheduleDAGInstrs::~ScheduleDAGInstrs()"},
    {"_ZN4llvm17ScheduleDAGInstrsD0Ev", "llvm::ScheduleDAGInstrs::~ScheduleDAGInstrs()"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPKN4llvm14MachineOperandEjEESt10_Select1stIS7_ESt4lessIjESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > >*)"},
    {"_ZN4llvm20LatencyPriorityQueueD0Ev", "llvm::LatencyPriorityQueue::~LatencyPriorityQueue()"},
    {"_ZNK4llvm20LatencyPriorityQueue10isBottomUpEv", "llvm::LatencyPriorityQueue::isBottomUp() const"},
    {"_ZN4llvm20LatencyPriorityQueue9initNodesERSt6vectorINS_5SUnitESaIS2_EE", "llvm::LatencyPriorityQueue::initNodes(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >&)"},
    {"_ZN4llvm20LatencyPriorityQueue7addNodeEPKNS_5SUnitE", "llvm::LatencyPriorityQueue::addNode(llvm::SUnit const*)"},
    {"_ZN4llvm20LatencyPriorityQueue10updateNodeEPKNS_5SUnitE", "llvm::LatencyPriorityQueue::updateNode(llvm::SUnit const*)"},
    {"_ZN4llvm20LatencyPriorityQueue12releaseStateEv", "llvm::LatencyPriorityQueue::releaseState()"},
    {"_ZNK4llvm20LatencyPriorityQueue5emptyEv", "llvm::LatencyPriorityQueue::empty() const"},
    {"_ZNK4llvm23SchedulingPriorityQueue17tracksRegPressureEv", "llvm::SchedulingPriorityQueue::tracksRegPressure() const"},
    {"_ZNK4llvm23SchedulingPriorityQueue7isReadyEPNS_5SUnitE", "llvm::SchedulingPriorityQueue::isReady(llvm::SUnit*) const"},
    {"_ZN4llvm23SchedulingPriorityQueue15UnscheduledNodeEPNS_5SUnitE", "llvm::SchedulingPriorityQueue::UnscheduledNode(llvm::SUnit*)"},
    {"_ZN4llvm23SchedulingPriorityQueueD1Ev", "llvm::SchedulingPriorityQueue::~SchedulingPriorityQueue()"},
    {"_ZN4llvm23SchedulingPriorityQueueD0Ev", "llvm::SchedulingPriorityQueue::~SchedulingPriorityQueue()"},
    {"_ZNK4llvm23SchedulingPriorityQueue4dumpEPNS_11ScheduleDAGE", "llvm::SchedulingPriorityQueue::dump(llvm::ScheduleDAG*) const"},
    {"_ZN4llvm23SchedulingPriorityQueue13ScheduledNodeEPNS_5SUnitE", "llvm::SchedulingPriorityQueue::ScheduledNode(llvm::SUnit*)"},
    {"_ZN4llvm31initializePreAllocSplittingPassERNS_12PassRegistryE", "llvm::initializePreAllocSplittingPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_117PreAllocSplittingEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PreAllocSplitting>()"},
    {"_ZN12_GLOBAL__N_117PreAllocSplittingD1Ev", "(anonymous namespace)::PreAllocSplitting::~PreAllocSplitting()"},
    {"_ZN12_GLOBAL__N_117PreAllocSplittingD0Ev", "(anonymous namespace)::PreAllocSplitting::~PreAllocSplitting()"},
    {"_ZNK12_GLOBAL__N_117PreAllocSplitting11getPassNameEv", "(anonymous namespace)::PreAllocSplitting::getPassName() const"},
    {"_ZNK12_GLOBAL__N_117PreAllocSplitting5printERN4llvm11raw_ostreamEPKNS1_6ModuleE", "(anonymous namespace)::PreAllocSplitting::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK12_GLOBAL__N_117PreAllocSplitting16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PreAllocSplitting::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting13releaseMemoryEv", "(anonymous namespace)::PreAllocSplitting::releaseMemory()"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::PreAllocSplitting::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting9FoldSpillEjPKN4llvm19TargetRegisterClassEPNS1_12MachineInstrES6_PNS1_17MachineBasicBlockERiRNS1_11SmallPtrSetIS6_Lj4EEE", "(anonymous namespace)::PreAllocSplitting::FoldSpill(unsigned int, llvm::TargetRegisterClass const*, llvm::MachineInstr*, llvm::MachineInstr*, llvm::MachineBasicBlock*, int&, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>&)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting14findSpillPointEPN4llvm17MachineBasicBlockEPNS1_12MachineInstrES5_RNS1_11SmallPtrSetIS5_Lj4EEE", "(anonymous namespace)::PreAllocSplitting::findSpillPoint(llvm::MachineBasicBlock*, llvm::MachineInstr*, llvm::MachineInstr*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>&)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting20CreateSpillStackSlotEjPKN4llvm19TargetRegisterClassE", "(anonymous namespace)::PreAllocSplitting::CreateSpillStackSlot(unsigned int, llvm::TargetRegisterClass const*)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting23UpdateSpillSlotIntervalEPN4llvm6VNInfoENS1_9SlotIndexES4_", "(anonymous namespace)::PreAllocSplitting::UpdateSpillSlotInterval(llvm::VNInfo*, llvm::SlotIndex, llvm::SlotIndex)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting23ReconstructLiveIntervalEPN4llvm12LiveIntervalE", "(anonymous namespace)::PreAllocSplitting::ReconstructLiveInterval(llvm::LiveInterval*)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting13RenumberValnoEPN4llvm6VNInfoE", "(anonymous namespace)::PreAllocSplitting::RenumberValno(llvm::VNInfo*)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting22PerformPHIConstructionEN4llvm14ilist_iteratorINS1_12MachineInstrEEEPNS1_17MachineBasicBlockEPNS1_12LiveIntervalERNS1_11SmallPtrSetIPS3_Lj4EEERNS1_8DenseMapIS6_NS9_ISA_Lj2EEENS1_12DenseMapInfoIS6_EENSF_ISE_EEEESJ_RNSD_ISA_PNS1_6VNInfoENSF_ISA_EENSF_ISL_EEEERNSD_IS6_SL_SG_SN_EESR_bb", "(anonymous namespace)::PreAllocSplitting::PerformPHIConstruction(llvm::ilist_iterator<llvm::MachineInstr>, llvm::MachineBasicBlock*, llvm::LiveInterval*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >&, llvm::DenseMap<llvm::MachineInstr*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, bool, bool)"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting30PerformPHIConstructionFallBackEN4llvm14ilist_iteratorINS1_12MachineInstrEEEPNS1_17MachineBasicBlockEPNS1_12LiveIntervalERNS1_11SmallPtrSetIPS3_Lj4EEERNS1_8DenseMapIS6_NS9_ISA_Lj2EEENS1_12DenseMapInfoIS6_EENSF_ISE_EEEESJ_RNSD_ISA_PNS1_6VNInfoENSF_ISA_EENSF_ISL_EEEERNSD_IS6_SL_SG_SN_EESR_bb", "(anonymous namespace)::PreAllocSplitting::PerformPHIConstructionFallBack(llvm::ilist_iterator<llvm::MachineInstr>, llvm::MachineBasicBlock*, llvm::LiveInterval*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >&, llvm::DenseMap<llvm::MachineInstr*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, bool, bool)"},
    {"_ZN4llvm8DenseMapIPNS_6VNInfoENS_11SmallPtrSetIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::VNInfo*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 4u> > >::FindAndConstruct(llvm::VNInfo* const&)"},
    {"_ZN4llvm8DenseMapIPNS_6VNInfoENS_11SmallPtrSetIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE16InsertIntoBucketERKS2_RKS6_PSt4pairIS2_S6_E", "llvm::DenseMap<llvm::VNInfo*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 4u> > >::InsertIntoBucket(llvm::VNInfo* const&, llvm::SmallPtrSet<llvm::MachineInstr*, 4u> const&, std::pair<llvm::VNInfo*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_6VNInfoENS_11SmallPtrSetIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::VNInfo*, llvm::SmallPtrSet<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 4u> > >::grow(unsigned int)"},
    {"_ZSt6__findIPPN4llvm6VNInfoES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "llvm::VNInfo** std::__find<llvm::VNInfo**, llvm::VNInfo*>(llvm::VNInfo**, llvm::VNInfo**, llvm::VNInfo* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_6VNInfoENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::VNInfo* const&, std::pair<llvm::MachineBasicBlock*, llvm::VNInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockEPNS_6VNInfoENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::VNInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPNS_6VNInfoENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MachineInstr*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::VNInfo*> >::InsertIntoBucket(llvm::MachineInstr* const&, llvm::VNInfo* const&, std::pair<llvm::MachineInstr*, llvm::VNInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_12MachineInstrEPNS_6VNInfoENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::MachineInstr*>, llvm::DenseMapInfo<llvm::VNInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_11SmallPtrSetIPNS_12MachineInstrELj2EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >::FindAndConstruct(llvm::MachineBasicBlock* const&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_11SmallPtrSetIPNS_12MachineInstrELj2EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE16InsertIntoBucketERKS2_RKS6_PSt4pairIS2_S6_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::SmallPtrSet<llvm::MachineInstr*, 2u> const&, std::pair<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_11SmallPtrSetIPNS_12MachineInstrELj2EEENS_12DenseMapInfoIS2_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineInstr*, 2u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::MachineInstr*, 2u> > >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E15LookupBucketForERKS1_RPSt4pairIS1_S1_E", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::LookupBucketFor(llvm::SlotIndex const&, std::pair<llvm::SlotIndex, llvm::SlotIndex>*&) const"},
    {"_ZN4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E16InsertIntoBucketERKS1_S6_PSt4pairIS1_S1_E", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::InsertIntoBucket(llvm::SlotIndex const&, llvm::SlotIndex const&, std::pair<llvm::SlotIndex, llvm::SlotIndex>*)"},
    {"_ZN4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E4growEj", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E4findERKS1_", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::find(llvm::SlotIndex const&) const"},
    {"_ZN4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E5clearEv", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::clear()"},
    {"_ZN4llvm8DenseMapINS_9SlotIndexES1_NS_12DenseMapInfoIS1_EES3_E16shrink_and_clearEv", "llvm::DenseMap<llvm::SlotIndex, llvm::SlotIndex, llvm::DenseMapInfo<llvm::SlotIndex>, llvm::DenseMapInfo<llvm::SlotIndex> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIjiNS_12DenseMapInfoIjEENS1_IiEEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<int> >::shrink_and_clear()"},
    {"_ZN4llvm33initializeProcessImplicitDefsPassERNS_12PassRegistryE", "llvm::initializeProcessImplicitDefsPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm19ProcessImplicitDefs16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::ProcessImplicitDefs::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm19ProcessImplicitDefs22CanTurnIntoImplicitDefEPNS_12MachineInstrEjjPKNS_15TargetInstrInfoERNS_8SmallSetIjLj8EEE", "llvm::ProcessImplicitDefs::CanTurnIntoImplicitDef(llvm::MachineInstr*, unsigned int, unsigned int, llvm::TargetInstrInfo const*, llvm::SmallSet<unsigned int, 8u>&)"},
    {"_ZN4llvm19ProcessImplicitDefs20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::ProcessImplicitDefs::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm19ProcessImplicitDefsD1Ev", "llvm::ProcessImplicitDefs::~ProcessImplicitDefs()"},
    {"_ZN4llvm19ProcessImplicitDefsD0Ev", "llvm::ProcessImplicitDefs::~ProcessImplicitDefs()"},
    {"_ZN4llvm15callDefaultCtorINS_19ProcessImplicitDefsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::ProcessImplicitDefs>()"},
    {"_ZN4llvm17initializePEIPassERNS_12PassRegistryE", "llvm::initializePEIPass(llvm::PassRegistry&)"},
    {"_ZN4llvm30createPrologEpilogCodeInserterEv", "llvm::createPrologEpilogCodeInserter()"},
    {"_ZN4llvm3PEI20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::PEI::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI25calculateCallsInformationERNS_15MachineFunctionE", "llvm::PEI::calculateCallsInformation(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI29calculateCalleeSavedRegistersERNS_15MachineFunctionE", "llvm::PEI::calculateCalleeSavedRegisters(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI26insertCSRSpillsAndRestoresERNS_15MachineFunctionE", "llvm::PEI::insertCSRSpillsAndRestores(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI27calculateFrameObjectOffsetsERNS_15MachineFunctionE", "llvm::PEI::calculateFrameObjectOffsets(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI22insertPrologEpilogCodeERNS_15MachineFunctionE", "llvm::PEI::insertPrologEpilogCode(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI19replaceFrameIndicesERNS_15MachineFunctionE", "llvm::PEI::replaceFrameIndices(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI24scavengeFrameVirtualRegsERNS_15MachineFunctionE", "llvm::PEI::scavengeFrameVirtualRegs(llvm::MachineFunction&)"},
    {"_ZN4llvm8SmallSetIiLj16EE6insertERKi", "llvm::SmallSet<int, 16u>::insert(int const&)"},
    {"_ZNSt6vectorIN4llvm15CalleeSavedInfoESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::CalleeSavedInfo*, std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > >, llvm::CalleeSavedInfo const&)"},
    {"_ZNSt6vectorIN4llvm14ilist_iteratorINS0_12MachineInstrEEESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::ilist_iterator<llvm::MachineInstr>, std::allocator<llvm::ilist_iterator<llvm::MachineInstr> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ilist_iterator<llvm::MachineInstr>*, std::vector<llvm::ilist_iterator<llvm::MachineInstr>, std::allocator<llvm::ilist_iterator<llvm::MachineInstr> > > >, llvm::ilist_iterator<llvm::MachineInstr> const&)"},
    {"_ZNSt6vectorIN4llvm15CalleeSavedInfoESaIS1_EEaSERKS3_", "std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> >::operator=(std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&)"},
    {"_ZN4llvm3PEIC2Ev", "llvm::PEI::PEI()"},
    {"_ZNK4llvm3PEI11getPassNameEv", "llvm::PEI::getPassName() const"},
    {"_ZN4llvm15callDefaultCtorINS_3PEIEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::PEI>()"},
    {"_ZN4llvm17PseudoSourceValue8getStackEv", "llvm::PseudoSourceValue::getStack()"},
    {"_ZN4llvm17PseudoSourceValue6getGOTEv", "llvm::PseudoSourceValue::getGOT()"},
    {"_ZN4llvm17PseudoSourceValue12getJumpTableEv", "llvm::PseudoSourceValue::getJumpTable()"},
    {"_ZN4llvm17PseudoSourceValue15getConstantPoolEv", "llvm::PseudoSourceValue::getConstantPool()"},
    {"_ZN4llvm17PseudoSourceValueC1ENS_5Value7ValueTyE", "llvm::PseudoSourceValue::PseudoSourceValue(llvm::Value::ValueTy)"},
    {"_ZN4llvm17PseudoSourceValueC2ENS_5Value7ValueTyE", "llvm::PseudoSourceValue::PseudoSourceValue(llvm::Value::ValueTy)"},
    {"_ZNK4llvm17PseudoSourceValue11printCustomERNS_11raw_ostreamE", "llvm::PseudoSourceValue::printCustom(llvm::raw_ostream&) const"},
    {"_ZN4llvm17PseudoSourceValue13getFixedStackEi", "llvm::PseudoSourceValue::getFixedStack(int)"},
    {"_ZNK4llvm17PseudoSourceValue10isConstantEPKNS_16MachineFrameInfoE", "llvm::PseudoSourceValue::isConstant(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm17PseudoSourceValue9isAliasedEPKNS_16MachineFrameInfoE", "llvm::PseudoSourceValue::isAliased(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm17PseudoSourceValue8mayAliasEPKNS_16MachineFrameInfoE", "llvm::PseudoSourceValue::mayAlias(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm27FixedStackPseudoSourceValue10isConstantEPKNS_16MachineFrameInfoE", "llvm::FixedStackPseudoSourceValue::isConstant(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm27FixedStackPseudoSourceValue9isAliasedEPKNS_16MachineFrameInfoE", "llvm::FixedStackPseudoSourceValue::isAliased(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm27FixedStackPseudoSourceValue8mayAliasEPKNS_16MachineFrameInfoE", "llvm::FixedStackPseudoSourceValue::mayAlias(llvm::MachineFrameInfo const*) const"},
    {"_ZNK4llvm27FixedStackPseudoSourceValue11printCustomERNS_11raw_ostreamE", "llvm::FixedStackPseudoSourceValue::printCustom(llvm::raw_ostream&) const"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_112PSVGlobalsTyEEEPvv", "void* llvm::object_creator<(anonymous namespace)::PSVGlobalsTy>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_112PSVGlobalsTyEE4callEPv", "llvm::object_deleter<(anonymous namespace)::PSVGlobalsTy>::call(void*)"},
    {"_ZN4llvm17PseudoSourceValueD1Ev", "llvm::PseudoSourceValue::~PseudoSourceValue()"},
    {"_ZN4llvm17PseudoSourceValueD0Ev", "llvm::PseudoSourceValue::~PseudoSourceValue()"},
    {"_ZN4llvm27FixedStackPseudoSourceValueD1Ev", "llvm::FixedStackPseudoSourceValue::~FixedStackPseudoSourceValue()"},
    {"_ZN4llvm27FixedStackPseudoSourceValueD0Ev", "llvm::FixedStackPseudoSourceValue::~FixedStackPseudoSourceValue()"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm17PseudoSourceValueEESt10_Select1stIS6_ESt4lessIiESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<int, std::pair<int const, llvm::PseudoSourceValue const*>, std::_Select1st<std::pair<int const, llvm::PseudoSourceValue const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::PseudoSourceValue const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<int const, llvm::PseudoSourceValue const*> >, std::pair<int const, llvm::PseudoSourceValue const*> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm17PseudoSourceValueEESt10_Select1stIS6_ESt4lessIiESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<int, std::pair<int const, llvm::PseudoSourceValue const*>, std::_Select1st<std::pair<int const, llvm::PseudoSourceValue const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::PseudoSourceValue const*> > >::_M_insert_unique(std::pair<int const, llvm::PseudoSourceValue const*> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKiPKN4llvm17PseudoSourceValueEESt10_Select1stIS6_ESt4lessIiESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<int, std::pair<int const, llvm::PseudoSourceValue const*>, std::_Select1st<std::pair<int const, llvm::PseudoSourceValue const*> >, std::less<int>, std::allocator<std::pair<int const, llvm::PseudoSourceValue const*> > >::_M_erase(std::_Rb_tree_node<std::pair<int const, llvm::PseudoSourceValue const*> >*)"},
    {"_ZN4llvm27createFastRegisterAllocatorEv", "llvm::createFastRegisterAllocator()"},
    {"_ZN12_GLOBAL__N_16RAFastD1Ev", "(anonymous namespace)::RAFast::~RAFast()"},
    {"_ZN12_GLOBAL__N_16RAFastD0Ev", "(anonymous namespace)::RAFast::~RAFast()"},
    {"_ZNK12_GLOBAL__N_16RAFast11getPassNameEv", "(anonymous namespace)::RAFast::getPassName() const"},
    {"_ZNK12_GLOBAL__N_16RAFast16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::RAFast::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_16RAFast20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::RAFast::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_16RAFast13definePhysRegEPN4llvm12MachineInstrEjNS0_8RegStateE", "(anonymous namespace)::RAFast::definePhysReg(llvm::MachineInstr*, unsigned int, (anonymous namespace)::RAFast::RegState)"},
    {"_ZN12_GLOBAL__N_16RAFast10setPhysRegEPN4llvm12MachineInstrEjj", "(anonymous namespace)::RAFast::setPhysReg(llvm::MachineInstr*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_16RAFast13reloadVirtRegEPN4llvm12MachineInstrEjjj", "(anonymous namespace)::RAFast::reloadVirtReg(llvm::MachineInstr*, unsigned int, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_16RAFast8spillAllEPN4llvm12MachineInstrE", "(anonymous namespace)::RAFast::spillAll(llvm::MachineInstr*)"},
    {"_ZN12_GLOBAL__N_16RAFast13defineVirtRegEPN4llvm12MachineInstrEjjj", "(anonymous namespace)::RAFast::defineVirtReg(llvm::MachineInstr*, unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm8DenseMapIjN12_GLOBAL__N_16RAFast7LiveRegENS_12DenseMapInfoIjEENS4_IS3_EEE6insertERKSt4pairIjS3_E", "llvm::DenseMap<unsigned int, (anonymous namespace)::RAFast::LiveReg, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<(anonymous namespace)::RAFast::LiveReg> >::insert(std::pair<unsigned int, (anonymous namespace)::RAFast::LiveReg> const&)"},
    {"_ZN12_GLOBAL__N_16RAFast12allocVirtRegEPN4llvm12MachineInstrERSt4pairIjNS0_7LiveRegEEj", "(anonymous namespace)::RAFast::allocVirtReg(llvm::MachineInstr*, std::pair<unsigned int, (anonymous namespace)::RAFast::LiveReg>&, unsigned int)"},
    {"_ZNK12_GLOBAL__N_16RAFast13calcSpillCostEj", "(anonymous namespace)::RAFast::calcSpillCost(unsigned int) const"},
    {"_ZN12_GLOBAL__N_16RAFast12spillVirtRegEN4llvm14ilist_iteratorINS1_12MachineInstrEEENS1_16DenseMapIteratorIjNS0_7LiveRegENS1_12DenseMapInfoIjEENS7_IS6_EELb0EEE", "(anonymous namespace)::RAFast::spillVirtReg(llvm::ilist_iterator<llvm::MachineInstr>, llvm::DenseMapIterator<unsigned int, (anonymous namespace)::RAFast::LiveReg, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<(anonymous namespace)::RAFast::LiveReg>, false>)"},
    {"_ZN4llvm33createLinearScanRegisterAllocatorEv", "llvm::createLinearScanRegisterAllocator()"},
    {"_ZN4llvm23initializeRALinScanPassERNS_12PassRegistryE", "llvm::initializeRALinScanPass(llvm::PassRegistry&)"},
    {"_ZN12_GLOBAL__N_19RALinScanC1Ev", "(anonymous namespace)::RALinScan::RALinScan()"},
    {"_ZN12_GLOBAL__N_19RALinScanD1Ev", "(anonymous namespace)::RALinScan::~RALinScan()"},
    {"_ZN12_GLOBAL__N_19RALinScanD0Ev", "(anonymous namespace)::RALinScan::~RALinScan()"},
    {"_ZNK12_GLOBAL__N_19RALinScan11getPassNameEv", "(anonymous namespace)::RALinScan::getPassName() const"},
    {"_ZNK12_GLOBAL__N_19RALinScan16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::RALinScan::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_19RALinScan20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::RALinScan::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_19RALinScan30assignRegOrStackSlotAtIntervalEPN4llvm12LiveIntervalE", "(anonymous namespace)::RALinScan::assignRegOrStackSlotAtInterval(llvm::LiveInterval*)"},
    {"_ZN12_GLOBAL__N_19RALinScan14getFreePhysRegEPN4llvm12LiveIntervalE", "(anonymous namespace)::RALinScan::getFreePhysReg(llvm::LiveInterval*)"},
    {"_ZN12_GLOBAL__N_19RALinScan18updateSpillWeightsERSt6vectorIfSaIfEEjfPKN4llvm19TargetRegisterClassE", "(anonymous namespace)::RALinScan::updateSpillWeights(std::vector<float, std::allocator<float> >&, unsigned int, float, llvm::TargetRegisterClass const*)"},
    {"_ZSt4sortIPPN4llvm12LiveIntervalEN12_GLOBAL__N_18LISorterEEvT_S6_T0_", "void std::sort<llvm::LiveInterval**, (anonymous namespace)::LISorter>(llvm::LiveInterval**, llvm::LiveInterval**, (anonymous namespace)::LISorter)"},
    {"_Z17getConflictWeightPN4llvm12LiveIntervalEjPNS_13LiveIntervalsEPNS_19MachineRegisterInfoEPNS_15MachineLoopInfoE", "getConflictWeight(llvm::LiveInterval*, unsigned int, llvm::LiveIntervals*, llvm::MachineRegisterInfo*, llvm::MachineLoopInfo*)"},
    {"_ZSt16__introsort_loopIPPN4llvm12LiveIntervalElN12_GLOBAL__N_18LISorterEEvT_S6_T0_T1_", "void std::__introsort_loop<llvm::LiveInterval**, long, (anonymous namespace)::LISorter>(llvm::LiveInterval**, llvm::LiveInterval**, long, (anonymous namespace)::LISorter)"},
    {"_ZSt16__insertion_sortIPPN4llvm12LiveIntervalEN12_GLOBAL__N_18LISorterEEvT_S6_T0_", "void std::__insertion_sort<llvm::LiveInterval**, (anonymous namespace)::LISorter>(llvm::LiveInterval**, llvm::LiveInterval**, (anonymous namespace)::LISorter)"},
    {"_ZSt25__unguarded_linear_insertIPPN4llvm12LiveIntervalES2_N12_GLOBAL__N_18LISorterEEvT_T0_T1_", "void std::__unguarded_linear_insert<llvm::LiveInterval**, llvm::LiveInterval*, (anonymous namespace)::LISorter>(llvm::LiveInterval**, llvm::LiveInterval*, (anonymous namespace)::LISorter)"},
    {"_ZSt13__adjust_heapIPPN4llvm12LiveIntervalElS2_N12_GLOBAL__N_18LISorterEEvT_T0_S7_T1_T2_", "void std::__adjust_heap<llvm::LiveInterval**, long, llvm::LiveInterval*, (anonymous namespace)::LISorter>(llvm::LiveInterval**, long, long, llvm::LiveInterval*, (anonymous namespace)::LISorter)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIjfESt6vectorIS3_SaIS3_EEEElN12_GLOBAL__N_113WeightCompareEEvT_SB_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, long, (anonymous namespace)::WeightCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, __gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, long, (anonymous namespace)::WeightCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIjfESt6vectorIS3_SaIS3_EEEEN12_GLOBAL__N_113WeightCompareEEvT_SB_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, (anonymous namespace)::WeightCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, __gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, (anonymous namespace)::WeightCompare)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIjfESt6vectorIS3_SaIS3_EEEElS3_N12_GLOBAL__N_113WeightCompareEEvT_T0_SC_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, long, std::pair<unsigned int, float>, (anonymous namespace)::WeightCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, long, long, std::pair<unsigned int, float>, (anonymous namespace)::WeightCompare)"},
    {"_ZN12_GLOBAL__N_19RALinScan14getFreePhysRegEPN4llvm12LiveIntervalEPKNS1_19TargetRegisterClassEjRNS1_11SmallVectorIjLj256EEEb", "(anonymous namespace)::RALinScan::getFreePhysReg(llvm::LiveInterval*, llvm::TargetRegisterClass const*, unsigned int, llvm::SmallVector<unsigned int, 256u>&, bool)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_19RALinScanEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::RALinScan>()"},
    {"_ZN4llvm15SmallVectorImplIjE6resizeEjRKj", "llvm::SmallVectorImpl<unsigned int>::resize(unsigned int, unsigned int const&)"},
    {"_ZNK4llvm11SlotIndexes14findLiveInMBBsENS_9SlotIndexES1_RNS_15SmallVectorImplIPNS_17MachineBasicBlockEEE", "llvm::SlotIndexes::findLiveInMBBs(llvm::SlotIndex, llvm::SlotIndex, llvm::SmallVectorImpl<llvm::MachineBasicBlock*>&) const"},
    {"_ZNSt14priority_queueIPN4llvm12LiveIntervalENS0_11SmallVectorIS2_Lj64EEENS0_11greater_ptrIS1_EEE4pushERKS2_", "std::priority_queue<llvm::LiveInterval*, llvm::SmallVector<llvm::LiveInterval*, 64u>, llvm::greater_ptr<llvm::LiveInterval> >::push(llvm::LiveInterval* const&)"},
    {"_ZSt6__findIPKjjET_S2_S2_RKT0_St26random_access_iterator_tag", "unsigned int const* std::__find<unsigned int const*, unsigned int>(unsigned int const*, unsigned int const*, unsigned int const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIjPKNS_19TargetRegisterClassENS_12DenseMapInfoIjEENS4_IS3_EEE16InsertIntoBucketERKjRKS3_PSt4pairIjS3_E", "llvm::DenseMap<unsigned int, llvm::TargetRegisterClass const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::TargetRegisterClass const*> >::InsertIntoBucket(unsigned int const&, llvm::TargetRegisterClass const* const&, std::pair<unsigned int, llvm::TargetRegisterClass const*>*)"},
    {"_ZN4llvm8DenseMapIjPKNS_19TargetRegisterClassENS_12DenseMapInfoIjEENS4_IS3_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::TargetRegisterClass const*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::TargetRegisterClass const*> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIjfESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, float>*, std::vector<std::pair<unsigned int, float>, std::allocator<std::pair<unsigned int, float> > > >, std::pair<unsigned int, float> const&)"},
    {"_ZNK4llvm18EquivalenceClassesIPKNS_19TargetRegisterClassEE7ECValue9getLeaderEv", "llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue::getLeader() const"},
    {"_ZSt13__adjust_heapIPPN4llvm12LiveIntervalElS2_NS0_11greater_ptrIS1_EEEvT_T0_S7_T1_T2_", "void std::__adjust_heap<llvm::LiveInterval**, long, llvm::LiveInterval*, llvm::greater_ptr<llvm::LiveInterval> >(llvm::LiveInterval**, long, long, llvm::LiveInterval*, llvm::greater_ptr<llvm::LiveInterval>)"},
    {"_ZN4llvm18EquivalenceClassesIPKNS_19TargetRegisterClassEE9unionSetsERKS3_S6_", "llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::unionSets(llvm::TargetRegisterClass const* const&, llvm::TargetRegisterClass const* const&)"},
    {"_ZNSt8_Rb_treeIN4llvm18EquivalenceClassesIPKNS0_19TargetRegisterClassEE7ECValueES6_St9_IdentityIS6_ESt4lessIS6_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue, llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue, std::_Identity<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue>, std::less<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue>, std::allocator<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue> >::_M_insert_unique(llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue const&)"},
    {"_ZNSt8_Rb_treeIN4llvm18EquivalenceClassesIPKNS0_19TargetRegisterClassEE7ECValueES6_St9_IdentityIS6_ESt4lessIS6_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue, llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue, std::_Identity<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue>, std::less<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue>, std::allocator<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue> >::_M_erase(std::_Rb_tree_node<llvm::EquivalenceClasses<llvm::TargetRegisterClass const*>::ECValue>*)"},
    {"_ZNSt14priority_queueIPN4llvm12LiveIntervalENS0_11SmallVectorIS2_Lj64EEENS0_11greater_ptrIS1_EEEC2ERKS6_RKS4_", "std::priority_queue<llvm::LiveInterval*, llvm::SmallVector<llvm::LiveInterval*, 64u>, llvm::greater_ptr<llvm::LiveInterval> >::priority_queue(llvm::greater_ptr<llvm::LiveInterval> const&, llvm::SmallVector<llvm::LiveInterval*, 64u> const&)"},
    {"_ZN4llvm40initializeRegisterCoalescerAnalysisGroupERNS_12PassRegistryE", "llvm::initializeRegisterCoalescerAnalysisGroup(llvm::PassRegistry&)"},
    {"_ZN4llvm17RegisterCoalescerD0Ev", "llvm::RegisterCoalescer::~RegisterCoalescer()"},
    {"_ZN4llvm17RegisterCoalescerD1Ev", "llvm::RegisterCoalescer::~RegisterCoalescer()"},
    {"_ZN4llvm17RegisterCoalescerD2Ev", "llvm::RegisterCoalescer::~RegisterCoalescer()"},
    {"_ZNK4llvm13CoalescerPair7composeEjj", "llvm::CoalescerPair::compose(unsigned int, unsigned int) const"},
    {"_ZNK4llvm13CoalescerPair11isMoveInstrEPKNS_12MachineInstrERjS4_S4_S4_", "llvm::CoalescerPair::isMoveInstr(llvm::MachineInstr const*, unsigned int&, unsigned int&, unsigned int&, unsigned int&) const"},
    {"_ZN4llvm13CoalescerPair12setRegistersEPKNS_12MachineInstrE", "llvm::CoalescerPair::setRegisters(llvm::MachineInstr const*)"},
    {"_ZN4llvm13CoalescerPair4flipEv", "llvm::CoalescerPair::flip()"},
    {"_ZNK4llvm13CoalescerPair13isCoalescableEPKNS_12MachineInstrE", "llvm::CoalescerPair::isCoalescable(llvm::MachineInstr const*) const"},
    {"_ZN4llvm12RegScavenger7setUsedEj", "llvm::RegScavenger::setUsed(unsigned int)"},
    {"_ZNK4llvm12RegScavenger11isAliasUsedEj", "llvm::RegScavenger::isAliasUsed(unsigned int) const"},
    {"_ZN4llvm12RegScavenger12initRegStateEv", "llvm::RegScavenger::initRegState()"},
    {"_ZN4llvm12RegScavenger15enterBasicBlockEPNS_17MachineBasicBlockE", "llvm::RegScavenger::enterBasicBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm12RegScavenger17addRegWithSubRegsERNS_9BitVectorEj", "llvm::RegScavenger::addRegWithSubRegs(llvm::BitVector&, unsigned int)"},
    {"_ZN4llvm12RegScavenger17addRegWithAliasesERNS_9BitVectorEj", "llvm::RegScavenger::addRegWithAliases(llvm::BitVector&, unsigned int)"},
    {"_ZN4llvm12RegScavenger7forwardEv", "llvm::RegScavenger::forward()"},
    {"_ZN4llvm12RegScavenger11getRegsUsedERNS_9BitVectorEb", "llvm::RegScavenger::getRegsUsed(llvm::BitVector&, bool)"},
    {"_ZNK4llvm12RegScavenger13FindUnusedRegEPKNS_19TargetRegisterClassE", "llvm::RegScavenger::FindUnusedReg(llvm::TargetRegisterClass const*) const"},
    {"_ZN4llvm12RegScavenger16getRegsAvailableEPKNS_19TargetRegisterClassERNS_9BitVectorE", "llvm::RegScavenger::getRegsAvailable(llvm::TargetRegisterClass const*, llvm::BitVector&)"},
    {"_ZN4llvm12RegScavenger15findSurvivorRegENS_14ilist_iteratorINS_12MachineInstrEEERNS_9BitVectorEjRS3_", "llvm::RegScavenger::findSurvivorReg(llvm::ilist_iterator<llvm::MachineInstr>, llvm::BitVector&, unsigned int, llvm::ilist_iterator<llvm::MachineInstr>&)"},
    {"_ZN4llvm12RegScavenger16scavengeRegisterEPKNS_19TargetRegisterClassENS_14ilist_iteratorINS_12MachineInstrEEEi", "llvm::RegScavenger::scavengeRegister(llvm::TargetRegisterClass const*, llvm::ilist_iterator<llvm::MachineInstr>, int)"},
    {"_ZN4llvm9BitVectoraSERKS0_", "llvm::BitVector::operator=(llvm::BitVector const&)"},
    {"_ZN4llvm12RegScavenger7setUsedERNS_9BitVectorE", "llvm::RegScavenger::setUsed(llvm::BitVector&)"},
    {"_ZNK4llvm9BitVectorcoEv", "llvm::BitVector::operator~() const"},
    {"_ZN4llvm11ScheduleDAGC1ERNS_15MachineFunctionE", "llvm::ScheduleDAG::ScheduleDAG(llvm::MachineFunction&)"},
    {"_ZN4llvm11ScheduleDAGC2ERNS_15MachineFunctionE", "llvm::ScheduleDAG::ScheduleDAG(llvm::MachineFunction&)"},
    {"_ZN4llvm11ScheduleDAGD0Ev", "llvm::ScheduleDAG::~ScheduleDAG()"},
    {"_ZN4llvm11ScheduleDAGD1Ev", "llvm::ScheduleDAG::~ScheduleDAG()"},
    {"_ZN4llvm11ScheduleDAGD2Ev", "llvm::ScheduleDAG::~ScheduleDAG()"},
    {"_ZNK4llvm11ScheduleDAG11getNodeDescEPKNS_6SDNodeE", "llvm::ScheduleDAG::getNodeDesc(llvm::SDNode const*) const"},
    {"_ZNK4llvm11ScheduleDAG12dumpScheduleEv", "llvm::ScheduleDAG::dumpSchedule() const"},
    {"_ZNK4llvm5SUnit4dumpEPKNS_11ScheduleDAGE", "llvm::SUnit::dump(llvm::ScheduleDAG const*) const"},
    {"_ZN4llvm11ScheduleDAG3RunEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::ScheduleDAG::Run(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>)"},
    {"_ZN4llvm5SUnit7addPredERKNS_4SDepE", "llvm::SUnit::addPred(llvm::SDep const&)"},
    {"_ZN4llvm5SUnit13setDepthDirtyEv", "llvm::SUnit::setDepthDirty()"},
    {"_ZN4llvm5SUnit14setHeightDirtyEv", "llvm::SUnit::setHeightDirty()"},
    {"_ZN4llvm5SUnit10removePredERKNS_4SDepE", "llvm::SUnit::removePred(llvm::SDep const&)"},
    {"_ZN4llvm5SUnit17setDepthToAtLeastEj", "llvm::SUnit::setDepthToAtLeast(unsigned int)"},
    {"_ZN4llvm5SUnit18setHeightToAtLeastEj", "llvm::SUnit::setHeightToAtLeast(unsigned int)"},
    {"_ZN4llvm5SUnit12ComputeDepthEv", "llvm::SUnit::ComputeDepth()"},
    {"_ZN4llvm5SUnit13ComputeHeightEv", "llvm::SUnit::ComputeHeight()"},
    {"_ZNK4llvm5SUnit7dumpAllEPKNS_11ScheduleDAGE", "llvm::SUnit::dumpAll(llvm::ScheduleDAG const*) const"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort25InitDAGTopologicalSortingEv", "llvm::ScheduleDAGTopologicalSort::InitDAGTopologicalSorting()"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort8AllocateEii", "llvm::ScheduleDAGTopologicalSort::Allocate(int, int)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort7AddPredEPNS_5SUnitES2_", "llvm::ScheduleDAGTopologicalSort::AddPred(llvm::SUnit*, llvm::SUnit*)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort3DFSEPKNS_5SUnitEiRb", "llvm::ScheduleDAGTopologicalSort::DFS(llvm::SUnit const*, int, bool&)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort5ShiftERNS_9BitVectorEii", "llvm::ScheduleDAGTopologicalSort::Shift(llvm::BitVector&, int, int)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort10RemovePredEPNS_5SUnitES2_", "llvm::ScheduleDAGTopologicalSort::RemovePred(llvm::SUnit*, llvm::SUnit*)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort15WillCreateCycleEPNS_5SUnitES2_", "llvm::ScheduleDAGTopologicalSort::WillCreateCycle(llvm::SUnit*, llvm::SUnit*)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSort11IsReachableEPKNS_5SUnitES3_", "llvm::ScheduleDAGTopologicalSort::IsReachable(llvm::SUnit const*, llvm::SUnit const*)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSortC1ERSt6vectorINS_5SUnitESaIS2_EE", "llvm::ScheduleDAGTopologicalSort::ScheduleDAGTopologicalSort(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >&)"},
    {"_ZN4llvm26ScheduleDAGTopologicalSortC2ERSt6vectorINS_5SUnitESaIS2_EE", "llvm::ScheduleDAGTopologicalSort::ScheduleDAGTopologicalSort(std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >&)"},
    {"_ZN4llvm24ScheduleHazardRecognizerD0Ev", "llvm::ScheduleHazardRecognizer::~ScheduleHazardRecognizer()"},
    {"_ZN4llvm24ScheduleHazardRecognizerD1Ev", "llvm::ScheduleHazardRecognizer::~ScheduleHazardRecognizer()"},
    {"_ZN4llvm24ScheduleHazardRecognizerD2Ev", "llvm::ScheduleHazardRecognizer::~ScheduleHazardRecognizer()"},
    {"_ZNK4llvm11ScheduleDAG21ComputeOperandLatencyEPNS_5SUnitES2_RNS_4SDepE", "llvm::ScheduleDAG::ComputeOperandLatency(llvm::SUnit*, llvm::SUnit*, llvm::SDep&) const"},
    {"_ZNSt6vectorIPKN4llvm5SUnitESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::SUnit const*, std::allocator<llvm::SUnit const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SUnit const**, std::vector<llvm::SUnit const*, std::allocator<llvm::SUnit const*> > >, llvm::SUnit const* const&)"},
    {"_ZNSt6vectorIiSaIiEE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPiS1_EEmRKi", "std::vector<int, std::allocator<int> >::_M_fill_insert(__gnu_cxx::__normal_iterator<int*, std::vector<int, std::allocator<int> > >, unsigned long, int const&)"},
    {"_ZNSt6vectorIN4llvm5SUnitESaIS1_EED2Ev", "std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >::~vector()"},
    {"_ZN4llvm11ScheduleDAG8EmitNoopEv", "llvm::ScheduleDAG::EmitNoop()"},
    {"_ZN4llvm11ScheduleDAG15EmitPhysRegCopyEPNS_5SUnitERNS_8DenseMapIS2_jNS_12DenseMapInfoIS2_EENS4_IjEEEE", "llvm::ScheduleDAG::EmitPhysRegCopy(llvm::SUnit*, llvm::DenseMap<llvm::SUnit*, unsigned int, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitEjNS_12DenseMapInfoIS2_EENS3_IjEEE6insertERKSt4pairIS2_jE", "llvm::DenseMap<llvm::SUnit*, unsigned int, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<llvm::SUnit*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::SUnit*, unsigned int, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::SUnit* const&, unsigned int const&, std::pair<llvm::SUnit*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_5SUnitEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::SUnit*, unsigned int, llvm::DenseMapInfo<llvm::SUnit*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm17ScheduleDAGInstrsC1ERNS_15MachineFunctionERKNS_15MachineLoopInfoERKNS_20MachineDominatorTreeE", "llvm::ScheduleDAGInstrs::ScheduleDAGInstrs(llvm::MachineFunction&, llvm::MachineLoopInfo const&, llvm::MachineDominatorTree const&)"},
    {"_ZN4llvm17ScheduleDAGInstrsC2ERNS_15MachineFunctionERKNS_15MachineLoopInfoERKNS_20MachineDominatorTreeE", "llvm::ScheduleDAGInstrs::ScheduleDAGInstrs(llvm::MachineFunction&, llvm::MachineLoopInfo const&, llvm::MachineDominatorTree const&)"},
    {"_ZN4llvm17ScheduleDAGInstrs3RunEPNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEES5_j", "llvm::ScheduleDAGInstrs::Run(llvm::MachineBasicBlock*, llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int)"},
    {"_ZN4llvm17ScheduleDAGInstrs10StartBlockEPNS_17MachineBasicBlockE", "llvm::ScheduleDAGInstrs::StartBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm17ScheduleDAGInstrs19AddSchedBarrierDepsEv", "llvm::ScheduleDAGInstrs::AddSchedBarrierDeps()"},
    {"_ZN4llvm17ScheduleDAGInstrs15BuildSchedGraphEPNS_13AliasAnalysisE", "llvm::ScheduleDAGInstrs::BuildSchedGraph(llvm::AliasAnalysis*)"},
    {"_Z27getUnderlyingObjectForInstrPKN4llvm12MachineInstrEPKNS_16MachineFrameInfoERb", "getUnderlyingObjectForInstr(llvm::MachineInstr const*, llvm::MachineFrameInfo const*, bool&)"},
    {"_ZN4llvm17ScheduleDAGInstrs11FinishBlockEv", "llvm::ScheduleDAGInstrs::FinishBlock()"},
    {"_ZN4llvm17ScheduleDAGInstrs14ComputeLatencyEPNS_5SUnitE", "llvm::ScheduleDAGInstrs::ComputeLatency(llvm::SUnit*)"},
    {"_ZNK4llvm17ScheduleDAGInstrs21ComputeOperandLatencyEPNS_5SUnitES2_RNS_4SDepE", "llvm::ScheduleDAGInstrs::ComputeOperandLatency(llvm::SUnit*, llvm::SUnit*, llvm::SDep&) const"},
    {"_ZNK4llvm17ScheduleDAGInstrs8dumpNodeEPKNS_5SUnitE", "llvm::ScheduleDAGInstrs::dumpNode(llvm::SUnit const*) const"},
    {"_ZNK4llvm17ScheduleDAGInstrs17getGraphNodeLabelEPKNS_5SUnitE", "llvm::ScheduleDAGInstrs::getGraphNodeLabel(llvm::SUnit const*) const"},
    {"_ZN4llvm17ScheduleDAGInstrs12EmitScheduleEv", "llvm::ScheduleDAGInstrs::EmitSchedule()"},
    {"_ZN4llvm16LoopDependencies9VisitLoopEPKNS_11MachineLoopE", "llvm::LoopDependencies::VisitLoop(llvm::MachineLoop const*)"},
    {"_ZNSt6vectorIN4llvm5SUnitESaIS1_EE7reserveEm", "std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >::reserve(unsigned long)"},
    {"_ZN4llvm17ScheduleDAGInstrs8NewSUnitEPNS_12MachineInstrE", "llvm::ScheduleDAGInstrs::NewSUnit(llvm::MachineInstr*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_St6vectorIPNS0_5SUnitESaIS8_EEESt10_Select1stISB_ESt4lessIS3_ESaISB_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISB_ERKSB_", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, std::_Select1st<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >, std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_St6vectorIPNS0_5SUnitESaIS8_EEESt10_Select1stISB_ESt4lessIS3_ESaISB_EE16_M_insert_uniqueERKSB_", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, std::_Select1st<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > > >::_M_insert_unique(std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_St6vectorIPNS0_5SUnitESaIS8_EEESt10_Select1stISB_ESt4lessIS3_ESaISB_EE14_M_create_nodeERKSB_", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, std::_Select1st<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > > >::_M_create_node(std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_PNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, llvm::SUnit*>, std::_Select1st<std::pair<llvm::Value const* const, llvm::SUnit*> >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, llvm::SUnit*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Value const* const, llvm::SUnit*> >, std::pair<llvm::Value const* const, llvm::SUnit*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_PNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, llvm::SUnit*>, std::_Select1st<std::pair<llvm::Value const* const, llvm::SUnit*> >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, llvm::SUnit*> > >::_M_insert_unique(std::pair<llvm::Value const* const, llvm::SUnit*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_St6vectorIPNS0_5SUnitESaIS8_EEESt10_Select1stISB_ESt4lessIS3_ESaISB_EE8_M_eraseEPSt13_Rb_tree_nodeISB_E", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >, std::_Select1st<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Value const* const, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm5ValueESt4pairIKS3_PNS0_5SUnitEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::Value const*, std::pair<llvm::Value const* const, llvm::SUnit*>, std::_Select1st<std::pair<llvm::Value const* const, llvm::SUnit*> >, std::less<llvm::Value const*>, std::allocator<std::pair<llvm::Value const* const, llvm::SUnit*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Value const* const, llvm::SUnit*> >*)"},
    {"_ZN4llvm5SUnitC2ERKS0_", "llvm::SUnit::SUnit(llvm::SUnit const&)"},
    {"_ZSt26__uninitialized_fill_n_auxIPSt6vectorIPN4llvm5SUnitESaIS3_EEmS5_EvT_T0_RKT1_St12__false_type", "void std::__uninitialized_fill_n_aux<std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >*, unsigned long, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > >(std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> >*, unsigned long, std::vector<llvm::SUnit*, std::allocator<llvm::SUnit*> > const&, std::__false_type)"},
    {"_ZNSt6vectorIN4llvm5SUnitESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::SUnit, std::allocator<llvm::SUnit> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::SUnit*, std::vector<llvm::SUnit, std::allocator<llvm::SUnit> > >, llvm::SUnit const&)"},
    {"_ZN4llvm5SUnitaSERKS0_", "llvm::SUnit::operator=(llvm::SUnit const&)"},
    {"_ZN4llvm16LoopDependencies11VisitRegionEPKNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEEPKS2_PKNS_11MachineLoopERKNS_8SmallSetIjLj8EEE", "llvm::LoopDependencies::VisitRegion(llvm::DomTreeNodeBase<llvm::MachineBasicBlock> const*, llvm::MachineBasicBlock const*, llvm::MachineLoop const*, llvm::SmallSet<unsigned int, 8u> const&)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjS0_IPKN4llvm14MachineOperandEjEESt10_Select1stIS7_ESt4lessIjESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> >, std::_Select1st<std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > > >::_M_insert_unique(std::pair<unsigned int const, std::pair<llvm::MachineOperand const*, unsigned int> > const&)"},
    {"_ZN4llvm26ScoreboardHazardRecognizerC1EPKNS_18InstrItineraryDataEPKNS_11ScheduleDAGEPKc", "llvm::ScoreboardHazardRecognizer::ScoreboardHazardRecognizer(llvm::InstrItineraryData const*, llvm::ScheduleDAG const*, char const*)"},
    {"_ZN4llvm26ScoreboardHazardRecognizerC2EPKNS_18InstrItineraryDataEPKNS_11ScheduleDAGEPKc", "llvm::ScoreboardHazardRecognizer::ScoreboardHazardRecognizer(llvm::InstrItineraryData const*, llvm::ScheduleDAG const*, char const*)"},
    {"_ZN4llvm26ScoreboardHazardRecognizer5ResetEv", "llvm::ScoreboardHazardRecognizer::Reset()"},
    {"_ZNK4llvm26ScoreboardHazardRecognizer10Scoreboard4dumpEv", "llvm::ScoreboardHazardRecognizer::Scoreboard::dump() const"},
    {"_ZNK4llvm26ScoreboardHazardRecognizer12atIssueLimitEv", "llvm::ScoreboardHazardRecognizer::atIssueLimit() const"},
    {"_ZN4llvm26ScoreboardHazardRecognizer13getHazardTypeEPNS_5SUnitEi", "llvm::ScoreboardHazardRecognizer::getHazardType(llvm::SUnit*, int)"},
    {"_ZN4llvm26ScoreboardHazardRecognizer15EmitInstructionEPNS_5SUnitE", "llvm::ScoreboardHazardRecognizer::EmitInstruction(llvm::SUnit*)"},
    {"_ZN4llvm26ScoreboardHazardRecognizer12AdvanceCycleEv", "llvm::ScoreboardHazardRecognizer::AdvanceCycle()"},
    {"_ZN4llvm26ScoreboardHazardRecognizer11RecedeCycleEv", "llvm::ScoreboardHazardRecognizer::RecedeCycle()"},
    {"_ZN4llvm26ScoreboardHazardRecognizerD1Ev", "llvm::ScoreboardHazardRecognizer::~ScoreboardHazardRecognizer()"},
    {"_ZN4llvm26ScoreboardHazardRecognizerD0Ev", "llvm::ScoreboardHazardRecognizer::~ScoreboardHazardRecognizer()"},
    {"_ZNK4llvm3PEI16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::PEI::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm3PEI24getTopLevelLoopPreheaderEPNS_11MachineLoopE", "llvm::PEI::getTopLevelLoopPreheader(llvm::MachineLoop*)"},
    {"_ZN4llvm3PEI21getTopLevelLoopParentEPNS_11MachineLoopE", "llvm::PEI::getTopLevelLoopParent(llvm::MachineLoop*)"},
    {"_ZN4llvm3PEI13isReturnBlockEPNS_17MachineBasicBlockE", "llvm::PEI::isReturnBlock(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm3PEI19clearAnticAvailSetsEv", "llvm::PEI::clearAnticAvailSets()"},
    {"_ZN4llvm3PEI12clearAllSetsEv", "llvm::PEI::clearAllSets()"},
    {"_ZN4llvm3PEI22initShrinkWrappingInfoEv", "llvm::PEI::initShrinkWrappingInfo()"},
    {"_ZN4llvm3PEI25placeCSRSpillsAndRestoresERNS_15MachineFunctionE", "llvm::PEI::placeCSRSpillsAndRestores(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI13calculateSetsERNS_15MachineFunctionE", "llvm::PEI::calculateSets(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI22placeSpillsAndRestoresERNS_15MachineFunctionE", "llvm::PEI::placeSpillsAndRestores(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI14calcAnticInOutEPNS_17MachineBasicBlockE", "llvm::PEI::calcAnticInOut(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm3PEI14calcAvailInOutEPNS_17MachineBasicBlockE", "llvm::PEI::calcAvailInOut(llvm::MachineBasicBlock*)"},
    {"_ZN4llvm3PEI19calculateAnticAvailERNS_15MachineFunctionE", "llvm::PEI::calculateAnticAvail(llvm::MachineFunction&)"},
    {"_ZN4llvm3PEI23propagateUsesAroundLoopEPNS_17MachineBasicBlockEPNS_11MachineLoopE", "llvm::PEI::propagateUsesAroundLoop(llvm::MachineBasicBlock*, llvm::MachineLoop*)"},
    {"_ZN4llvm3PEI20addUsesForMEMERegionEPNS_17MachineBasicBlockERNS_11SmallVectorIS2_Lj4EEE", "llvm::PEI::addUsesForMEMERegion(llvm::MachineBasicBlock*, llvm::SmallVector<llvm::MachineBasicBlock*, 4u>&)"},
    {"_ZN4llvm3PEI23addUsesForTopLevelLoopsERNS_11SmallVectorIPNS_17MachineBasicBlockELj4EEE", "llvm::PEI::addUsesForTopLevelLoops(llvm::SmallVector<llvm::MachineBasicBlock*, 4u>&)"},
    {"_ZN4llvm3PEI19calcSpillPlacementsEPNS_17MachineBasicBlockERNS_11SmallVectorIS2_Lj4EEERNS_8DenseMapIS2_NS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS9_IS8_EEEE", "llvm::PEI::calcSpillPlacements(llvm::MachineBasicBlock*, llvm::SmallVector<llvm::MachineBasicBlock*, 4u>&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >&)"},
    {"_ZN4llvm3PEI21calcRestorePlacementsEPNS_17MachineBasicBlockERNS_11SmallVectorIS2_Lj4EEERNS_8DenseMapIS2_NS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS9_IS8_EEEE", "llvm::PEI::calcRestorePlacements(llvm::MachineBasicBlock*, llvm::SmallVector<llvm::MachineBasicBlock*, 4u>&, llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >&)"},
    {"_ZN4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEED1Ev", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::~opt()"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE5clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >::clear()"},
    {"_ZN4llvm15SparseBitVectorILj128EEaNERKS1_", "llvm::SparseBitVector<128u>::operator&=(llvm::SparseBitVector<128u> const&)"},
    {"_ZN4llvm15SparseBitVectorILj128EEoRERKS1_", "llvm::SparseBitVector<128u>::operator|=(llvm::SparseBitVector<128u> const&)"},
    {"_ZNK4llvm15SparseBitVectorILj128EEeqERKS1_", "llvm::SparseBitVector<128u>::operator==(llvm::SparseBitVector<128u> const&) const"},
    {"_ZNK4llvm15SparseBitVectorILj128EE10intersectsERKS1_", "llvm::SparseBitVector<128u>::intersects(llvm::SparseBitVector<128u> const&) const"},
    {"_ZNK4llvm15SparseBitVectorILj128EE5countEv", "llvm::SparseBitVector<128u>::count() const"},
    {"_ZN4llvm3PEID1Ev", "llvm::PEI::~PEI()"},
    {"_ZN4llvm3PEID0Ev", "llvm::PEI::~PEI()"},
    {"_ZN4llvm15SmallVectorImplIPNS_17MachineBasicBlockEEaSERKS3_", "llvm::SmallVectorImpl<llvm::MachineBasicBlock*>::operator=(llvm::SmallVectorImpl<llvm::MachineBasicBlock*> const&)"},
    {"_ZN4llvm15SparseBitVectorILj128EE23intersectWithComplementERKS1_S3_", "llvm::SparseBitVector<128u>::intersectWithComplement(llvm::SparseBitVector<128u> const&, llvm::SparseBitVector<128u> const&)"},
    {"_ZNSt6vectorIN4llvm15CalleeSavedInfoESaIS1_EEC2ERKS3_", "std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> >::vector(std::vector<llvm::CalleeSavedInfo, std::allocator<llvm::CalleeSavedInfo> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >::FindAndConstruct(llvm::MachineBasicBlock* const&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >::InsertIntoBucket(llvm::MachineBasicBlock* const&, llvm::SparseBitVector<128u> const&, std::pair<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockENS_15SparseBitVectorILj128EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, llvm::SparseBitVector<128u>, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<llvm::SparseBitVector<128u> > >::shrink_and_clear()"},
    {"_ZN4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEE16handleOccurrenceEjNS_9StringRefES6_", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEE14getOptionWidthEv", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEE15printOptionInfoEm", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEED0Ev", "llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >::~opt()"},
    {"_ZN4llvm2cl6parserI20ShrinkWrapDebugLevelE5parseERNS0_6OptionENS_9StringRefES6_RS2_", "llvm::cl::parser<ShrinkWrapDebugLevel>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, ShrinkWrapDebugLevel&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS5_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> > >(llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserI20ShrinkWrapDebugLevelE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<ShrinkWrapDebugLevel>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserI20ShrinkWrapDebugLevelED1Ev", "llvm::cl::parser<ShrinkWrapDebugLevel>::~parser()"},
    {"_ZN4llvm2cl6parserI20ShrinkWrapDebugLevelED0Ev", "llvm::cl::parser<ShrinkWrapDebugLevel>::~parser()"},
    {"_ZNK4llvm2cl6parserI20ShrinkWrapDebugLevelE13getNumOptionsEv", "llvm::cl::parser<ShrinkWrapDebugLevel>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserI20ShrinkWrapDebugLevelE9getOptionEj", "llvm::cl::parser<ShrinkWrapDebugLevel>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserI20ShrinkWrapDebugLevelE14getDescriptionEj", "llvm::cl::parser<ShrinkWrapDebugLevel>::getDescription(unsigned int) const"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEEC2IA17_cNS0_12OptionHiddenENS0_4descENS0_10value_descENS0_11initializerIA1_cEEEERKT_RKT0_RKT1_RKT2_RKT3_", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::opt<char [17], llvm::cl::OptionHidden, llvm::cl::desc, llvm::cl::value_desc, llvm::cl::initializer<char [1]> >(char const (&) [17], llvm::cl::OptionHidden const&, llvm::cl::desc const&, llvm::cl::value_desc const&, llvm::cl::initializer<char [1]> const&)"},
    {"_ZN4llvm3PEID2Ev", "llvm::PEI::~PEI()"},
    {"_ZN4llvm38initializeSimpleRegisterCoalescingPassERNS_12PassRegistryE", "llvm::initializeSimpleRegisterCoalescingPass(llvm::PassRegistry&)"},
    {"_ZThn32_NK4llvm24SimpleRegisterCoalescing16getAnalysisUsageERNS_13AnalysisUsageE", "non-virtual thunk to llvm::SimpleRegisterCoalescing::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm24SimpleRegisterCoalescing16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::SimpleRegisterCoalescing::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm24SimpleRegisterCoalescing20AdjustCopiesBackFromERKNS_13CoalescerPairEPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::AdjustCopiesBackFrom(llvm::CoalescerPair const&, llvm::MachineInstr*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing25TrimLiveIntervalToLastUseENS_9SlotIndexEPNS_17MachineBasicBlockERNS_12LiveIntervalEPKNS_9LiveRangeE", "llvm::SimpleRegisterCoalescing::TrimLiveIntervalToLastUse(llvm::SlotIndex, llvm::MachineBasicBlock*, llvm::LiveInterval&, llvm::LiveRange const*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing20HasOtherReachingDefsERNS_12LiveIntervalES2_PNS_6VNInfoES4_", "llvm::SimpleRegisterCoalescing::HasOtherReachingDefs(llvm::LiveInterval&, llvm::LiveInterval&, llvm::VNInfo*, llvm::VNInfo*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing24RemoveCopyByCommutingDefERKNS_13CoalescerPairEPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::RemoveCopyByCommutingDef(llvm::CoalescerPair const&, llvm::MachineInstr*)"},
    {"_ZNK4llvm24SimpleRegisterCoalescing15lastRegisterUseENS_9SlotIndexES1_jRS1_", "llvm::SimpleRegisterCoalescing::lastRegisterUse(llvm::SlotIndex, llvm::SlotIndex, unsigned int, llvm::SlotIndex&) const"},
    {"_Z21isSameOrFallThroughBBPN4llvm17MachineBasicBlockES1_PKNS_15TargetInstrInfoE", "isSameOrFallThroughBB(llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::TargetInstrInfo const*)"},
    {"_Z11removeRangeRN4llvm12LiveIntervalENS_9SlotIndexES2_PNS_13LiveIntervalsEPKNS_18TargetRegisterInfoE", "removeRange(llvm::LiveInterval&, llvm::SlotIndex, llvm::SlotIndex, llvm::LiveIntervals*, llvm::TargetRegisterInfo const*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing23ReMaterializeTrivialDefERNS_12LiveIntervalEbjjPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::ReMaterializeTrivialDef(llvm::LiveInterval&, bool, unsigned int, unsigned int, llvm::MachineInstr*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing14RemoveCopyFlagEjPKNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::RemoveCopyFlag(unsigned int, llvm::MachineInstr const*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing17UpdateRegDefsUsesERKNS_13CoalescerPairE", "llvm::SimpleRegisterCoalescing::UpdateRegDefsUses(llvm::CoalescerPair const&)"},
    {"_ZN4llvm24SimpleRegisterCoalescing24ShortenDeadCopyLiveRangeERNS_12LiveIntervalEPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::ShortenDeadCopyLiveRange(llvm::LiveInterval&, llvm::MachineInstr*)"},
    {"_Z21removeIntervalIfEmptyRN4llvm12LiveIntervalEPNS_13LiveIntervalsEPKNS_18TargetRegisterInfoE", "removeIntervalIfEmpty(llvm::LiveInterval&, llvm::LiveIntervals*, llvm::TargetRegisterInfo const*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing13RemoveDeadDefERNS_12LiveIntervalEPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::RemoveDeadDef(llvm::LiveInterval&, llvm::MachineInstr*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing27ShortenDeadCopySrcLiveRangeERNS_12LiveIntervalEPNS_12MachineInstrE", "llvm::SimpleRegisterCoalescing::ShortenDeadCopySrcLiveRange(llvm::LiveInterval&, llvm::MachineInstr*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing21isWinToJoinCrossClassEjjPKNS_19TargetRegisterClassES3_S3_", "llvm::SimpleRegisterCoalescing::isWinToJoinCrossClass(unsigned int, unsigned int, llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm24SimpleRegisterCoalescing8JoinCopyERNS_7CopyRecERb", "llvm::SimpleRegisterCoalescing::JoinCopy(llvm::CopyRec&, bool&)"},
    {"_ZN4llvm24SimpleRegisterCoalescing13JoinIntervalsERNS_13CoalescerPairE", "llvm::SimpleRegisterCoalescing::JoinIntervals(llvm::CoalescerPair&)"},
    {"_Z17ComputeUltimateVNPN4llvm6VNInfoERNS_11SmallVectorIS1_Lj16EEERNS_8DenseMapIS1_S1_NS_12DenseMapInfoIS1_EES7_EES9_RNS2_IiLj16EEESB_", "ComputeUltimateVN(llvm::VNInfo*, llvm::SmallVector<llvm::VNInfo*, 16u>&, llvm::DenseMap<llvm::VNInfo*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::DenseMap<llvm::VNInfo*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::VNInfo*> >&, llvm::SmallVector<int, 16u>&, llvm::SmallVector<int, 16u>&)"},
    {"_ZN4llvm24SimpleRegisterCoalescing17CopyCoalesceInMBBEPNS_17MachineBasicBlockERSt6vectorINS_7CopyRecESaIS4_EE", "llvm::SimpleRegisterCoalescing::CopyCoalesceInMBB(llvm::MachineBasicBlock*, std::vector<llvm::CopyRec, std::allocator<llvm::CopyRec> >&)"},
    {"_ZN4llvm24SimpleRegisterCoalescing13joinIntervalsEv", "llvm::SimpleRegisterCoalescing::joinIntervals()"},
    {"_ZNK4llvm24SimpleRegisterCoalescing24differingRegisterClassesEjj", "llvm::SimpleRegisterCoalescing::differingRegisterClasses(unsigned int, unsigned int) const"},
    {"_ZN4llvm24SimpleRegisterCoalescing13releaseMemoryEv", "llvm::SimpleRegisterCoalescing::releaseMemory()"},
    {"_ZN4llvm24SimpleRegisterCoalescing20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::SimpleRegisterCoalescing::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZNK4llvm24SimpleRegisterCoalescing5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::SimpleRegisterCoalescing::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZN4llvm29createSimpleRegisterCoalescerEv", "llvm::createSimpleRegisterCoalescer()"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIjPN4llvm17MachineBasicBlockEESt6vectorIS6_SaIS6_EEEElN12_GLOBAL__N_115DepthMBBCompareEEvT_SE_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, long, (anonymous namespace)::DepthMBBCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, long, (anonymous namespace)::DepthMBBCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIjPN4llvm17MachineBasicBlockEESt6vectorIS6_SaIS6_EEEEN12_GLOBAL__N_115DepthMBBCompareEEvT_SE_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, (anonymous namespace)::DepthMBBCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, (anonymous namespace)::DepthMBBCompare)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIjPN4llvm17MachineBasicBlockEESt6vectorIS6_SaIS6_EEEElS6_N12_GLOBAL__N_115DepthMBBCompareEEvT_T0_SF_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, long, std::pair<unsigned int, llvm::MachineBasicBlock*>, (anonymous namespace)::DepthMBBCompare>(__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, long, long, std::pair<unsigned int, llvm::MachineBasicBlock*>, (anonymous namespace)::DepthMBBCompare)"},
    {"_ZN4llvm13LiveIntervals14removeIntervalEj", "llvm::LiveIntervals::removeInterval(unsigned int)"},
    {"_ZN4llvm24SimpleRegisterCoalescingD1Ev", "llvm::SimpleRegisterCoalescing::~SimpleRegisterCoalescing()"},
    {"_ZN4llvm24SimpleRegisterCoalescingD0Ev", "llvm::SimpleRegisterCoalescing::~SimpleRegisterCoalescing()"},
    {"_ZN4llvm24SimpleRegisterCoalescing16coalesceFunctionERNS_15MachineFunctionERNS_13RegallocQueryE", "llvm::SimpleRegisterCoalescing::coalesceFunction(llvm::MachineFunction&, llvm::RegallocQuery&)"},
    {"_ZThn32_N4llvm24SimpleRegisterCoalescingD1Ev", "non-virtual thunk to llvm::SimpleRegisterCoalescing::~SimpleRegisterCoalescing()"},
    {"_ZThn32_N4llvm24SimpleRegisterCoalescingD0Ev", "non-virtual thunk to llvm::SimpleRegisterCoalescing::~SimpleRegisterCoalescing()"},
    {"_ZThn32_N4llvm24SimpleRegisterCoalescing16coalesceFunctionERNS_15MachineFunctionERNS_13RegallocQueryE", "non-virtual thunk to llvm::SimpleRegisterCoalescing::coalesceFunction(llvm::MachineFunction&, llvm::RegallocQuery&)"},
    {"_ZN4llvm17RegisterCoalescer5resetERNS_15MachineFunctionE", "llvm::RegisterCoalescer::reset(llvm::MachineFunction&)"},
    {"_ZNSt6vectorISt4pairIjPN4llvm17MachineBasicBlockEESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<unsigned int, llvm::MachineBasicBlock*>*, std::vector<std::pair<unsigned int, llvm::MachineBasicBlock*>, std::allocator<std::pair<unsigned int, llvm::MachineBasicBlock*> > > >, std::pair<unsigned int, llvm::MachineBasicBlock*> const&)"},
    {"_ZNSt6vectorIN4llvm7CopyRecESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::CopyRec, std::allocator<llvm::CopyRec> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::CopyRec*, std::vector<llvm::CopyRec, std::allocator<llvm::CopyRec> > >, llvm::CopyRec const&)"},
    {"_ZN4llvm8DenseMapIPNS_6VNInfoES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::VNInfo*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::VNInfo*> >::InsertIntoBucket(llvm::VNInfo* const&, llvm::VNInfo* const&, std::pair<llvm::VNInfo*, llvm::VNInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_6VNInfoES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::VNInfo*, llvm::VNInfo*, llvm::DenseMapInfo<llvm::VNInfo*>, llvm::DenseMapInfo<llvm::VNInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm24SimpleRegisterCoalescingD2Ev", "llvm::SimpleRegisterCoalescing::~SimpleRegisterCoalescing()"},
    {"_ZN4llvm24SimpleRegisterCoalescingC2Ev", "llvm::SimpleRegisterCoalescing::SimpleRegisterCoalescing()"},
    {"_ZNK4llvm17RegisterCoalescer16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::RegisterCoalescer::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm15callDefaultCtorINS_24SimpleRegisterCoalescingEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::SimpleRegisterCoalescing>()"},
    {"_ZN4llvm16createSjLjEHPassEPKNS_14TargetLoweringE", "llvm::createSjLjEHPass(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_110SjLjEHPassD1Ev", "(anonymous namespace)::SjLjEHPass::~SjLjEHPass()"},
    {"_ZN12_GLOBAL__N_110SjLjEHPassD0Ev", "(anonymous namespace)::SjLjEHPass::~SjLjEHPass()"},
    {"_ZNK12_GLOBAL__N_110SjLjEHPass11getPassNameEv", "(anonymous namespace)::SjLjEHPass::getPassName() const"},
    {"_ZNK12_GLOBAL__N_110SjLjEHPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::SjLjEHPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_110SjLjEHPass16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::SjLjEHPass::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_110SjLjEHPass13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::SjLjEHPass::runOnFunction(llvm::Function&)"},
    {"_Z16MarkBlocksLiveInPN4llvm10BasicBlockERSt3setIS1_St4lessIS1_ESaIS1_EE", "MarkBlocksLiveIn(llvm::BasicBlock*, std::set<llvm::BasicBlock*, std::less<llvm::BasicBlock*>, std::allocator<llvm::BasicBlock*> >&)"},
    {"_ZN4llvm8ICmpInstC2EPNS_11InstructionENS_7CmpInst9PredicateEPNS_5ValueES6_RKNS_5TwineE", "llvm::ICmpInst::ICmpInst(llvm::Instruction*, llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm17GetElementPtrInstC2IPPNS_5ValueEEES3_T_S5_jRKNS_5TwineEPNS_11InstructionE", "llvm::GetElementPtrInst::GetElementPtrInst<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::BasicBlock*, llvm::BasicBlock*, std::_Identity<llvm::BasicBlock*>, std::less<llvm::BasicBlock*>, std::allocator<llvm::BasicBlock*> >::_M_erase(std::_Rb_tree_node<llvm::BasicBlock*>*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<llvm::BasicBlock*, llvm::BasicBlock*, std::_Identity<llvm::BasicBlock*>, std::less<llvm::BasicBlock*>, std::allocator<llvm::BasicBlock*> >::_M_insert_unique(llvm::BasicBlock* const&)"},
    {"_ZN4llvm16ExtractValueInstC2IPjEEPNS_5ValueET_S5_RKNS_5TwineEPNS_11InstructionE", "llvm::ExtractValueInst::ExtractValueInst<unsigned int*>(llvm::Value*, unsigned int*, unsigned int*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm25initializeSlotIndexesPassERNS_12PassRegistryE", "llvm::initializeSlotIndexesPass(llvm::PassRegistry&)"},
    {"_ZN4llvm14IndexListEntry16getEmptyKeyEntryEv", "llvm::IndexListEntry::getEmptyKeyEntry()"},
    {"_ZN4llvm14IndexListEntry20getTombstoneKeyEntryEv", "llvm::IndexListEntry::getTombstoneKeyEntry()"},
    {"_ZNK4llvm11SlotIndexes16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::SlotIndexes::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm11SlotIndexes13releaseMemoryEv", "llvm::SlotIndexes::releaseMemory()"},
    {"_ZN4llvm11SlotIndexes20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::SlotIndexes::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm11SlotIndexes15renumberIndexesEv", "llvm::SlotIndexes::renumberIndexes()"},
    {"_ZNK4llvm11SlotIndexes4dumpEv", "llvm::SlotIndexes::dump() const"},
    {"_ZNK4llvm9SlotIndex5printERNS_11raw_ostreamE", "llvm::SlotIndex::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm9SlotIndex4dumpEv", "llvm::SlotIndex::dump() const"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_123TombstoneIndexListEntryEEEPvv", "void* llvm::object_creator<(anonymous namespace)::TombstoneIndexListEntry>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_123TombstoneIndexListEntryEE4callEPv", "llvm::object_deleter<(anonymous namespace)::TombstoneIndexListEntry>::call(void*)"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_119EmptyIndexListEntryEEEPvv", "void* llvm::object_creator<(anonymous namespace)::EmptyIndexListEntry>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_119EmptyIndexListEntryEE4callEPv", "llvm::object_deleter<(anonymous namespace)::EmptyIndexListEntry>::call(void*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrENS_9SlotIndexENS_12DenseMapInfoIS3_EENS5_IS4_EEE6insertERKSt4pairIS3_S4_E", "llvm::DenseMap<llvm::MachineInstr const*, llvm::SlotIndex, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::SlotIndex> >::insert(std::pair<llvm::MachineInstr const*, llvm::SlotIndex> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockESt4pairINS_9SlotIndexES5_ENS_12DenseMapInfoIS3_EENS7_IS6_EEE6insertERKS4_IS3_S6_E", "llvm::DenseMap<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex>, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<std::pair<llvm::SlotIndex, llvm::SlotIndex> > >::insert(std::pair<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex> > const&)"},
    {"_ZN4llvm11SlotIndexesD1Ev", "llvm::SlotIndexes::~SlotIndexes()"},
    {"_ZN4llvm11SlotIndexesD0Ev", "llvm::SlotIndexes::~SlotIndexes()"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockESt4pairINS_9SlotIndexES5_ENS_12DenseMapInfoIS3_EENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex>, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<std::pair<llvm::SlotIndex, llvm::SlotIndex> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrENS_9SlotIndexENS_12DenseMapInfoIS3_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineInstr const*, llvm::SlotIndex, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::SlotIndex> >::shrink_and_clear()"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEElNS3_14Idx2MBBCompareEEvT_SE_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, long, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, long, llvm::Idx2MBBCompare)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEENS3_14Idx2MBBCompareEEvT_SE_T0_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEENS3_14Idx2MBBCompareEEvT_SE_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare)"},
    {"_ZSt21__unguarded_partitionIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEES7_NS3_14Idx2MBBCompareEET_SE_SE_T0_T1_", "__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > > std::__unguarded_partition<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, llvm::Idx2MBBCompare)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEENS3_14Idx2MBBCompareEEvT_SE_SE_T0_", "void std::__heap_select<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, llvm::Idx2MBBCompare)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm9SlotIndexEPNS3_17MachineBasicBlockEESt6vectorIS7_SaIS7_EEEElS7_NS3_14Idx2MBBCompareEEvT_T0_SF_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, long, std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, llvm::Idx2MBBCompare>(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, long, long, std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, llvm::Idx2MBBCompare)"},
    {"_ZNSt6vectorISt4pairIN4llvm9SlotIndexEPNS1_17MachineBasicBlockEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>*, std::vector<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*>, std::allocator<std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> > > >, std::pair<llvm::SlotIndex, llvm::MachineBasicBlock*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockESt4pairINS_9SlotIndexES5_ENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PS4_IS3_S6_E", "llvm::DenseMap<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex>, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<std::pair<llvm::SlotIndex, llvm::SlotIndex> > >::InsertIntoBucket(llvm::MachineBasicBlock const* const&, std::pair<llvm::SlotIndex, llvm::SlotIndex> const&, std::pair<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_17MachineBasicBlockESt4pairINS_9SlotIndexES5_ENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock const*, std::pair<llvm::SlotIndex, llvm::SlotIndex>, llvm::DenseMapInfo<llvm::MachineBasicBlock const*>, llvm::DenseMapInfo<std::pair<llvm::SlotIndex, llvm::SlotIndex> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrENS_9SlotIndexENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::MachineInstr const*, llvm::SlotIndex, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::SlotIndex> >::InsertIntoBucket(llvm::MachineInstr const* const&, llvm::SlotIndex const&, std::pair<llvm::MachineInstr const*, llvm::SlotIndex>*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MachineInstrENS_9SlotIndexENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::MachineInstr const*, llvm::SlotIndex, llvm::DenseMapInfo<llvm::MachineInstr const*>, llvm::DenseMapInfo<llvm::SlotIndex> >::grow(unsigned int)"},
    {"_ZN4llvm15callDefaultCtorINS_11SlotIndexesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::SlotIndexes>()"},
    {"_ZN4llvm11SlotIndexesC2Ev", "llvm::SlotIndexes::SlotIndexes()"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEED1Ev", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::~opt()"},
    {"_ZN4llvm7SpillerD0Ev", "llvm::Spiller::~Spiller()"},
    {"_ZN4llvm7SpillerD1Ev", "llvm::Spiller::~Spiller()"},
    {"_ZN4llvm7SpillerD2Ev", "llvm::Spiller::~Spiller()"},
    {"_ZN4llvm13createSpillerERNS_19MachineFunctionPassERNS_15MachineFunctionERNS_10VirtRegMapE", "llvm::createSpiller(llvm::MachineFunctionPass&, llvm::MachineFunction&, llvm::VirtRegMap&)"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEE14getOptionWidthEv", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEE15printOptionInfoEm", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEED0Ev", "llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >::~opt()"},
    {"_ZN4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEED1Ev", "llvm::cl::parser<(anonymous namespace)::SpillerName>::~parser()"},
    {"_ZN4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEED0Ev", "llvm::cl::parser<(anonymous namespace)::SpillerName>::~parser()"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEE13getNumOptionsEv", "llvm::cl::parser<(anonymous namespace)::SpillerName>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEE9getOptionEj", "llvm::cl::parser<(anonymous namespace)::SpillerName>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEE14getDescriptionEj", "llvm::cl::parser<(anonymous namespace)::SpillerName>::getDescription(unsigned int) const"},
    {"_ZN12_GLOBAL__N_115StandardSpillerD1Ev", "(anonymous namespace)::StandardSpiller::~StandardSpiller()"},
    {"_ZN12_GLOBAL__N_115StandardSpillerD0Ev", "(anonymous namespace)::StandardSpiller::~StandardSpiller()"},
    {"_ZN12_GLOBAL__N_115StandardSpiller5spillEPN4llvm12LiveIntervalERNS1_15SmallVectorImplIS3_EERKS5_", "(anonymous namespace)::StandardSpiller::spill(llvm::LiveInterval*, llvm::SmallVectorImpl<llvm::LiveInterval*>&, llvm::SmallVectorImpl<llvm::LiveInterval*> const&)"},
    {"_ZN12_GLOBAL__N_114TrivialSpillerD1Ev", "(anonymous namespace)::TrivialSpiller::~TrivialSpiller()"},
    {"_ZN12_GLOBAL__N_114TrivialSpillerD0Ev", "(anonymous namespace)::TrivialSpiller::~TrivialSpiller()"},
    {"_ZN12_GLOBAL__N_114TrivialSpiller5spillEPN4llvm12LiveIntervalERNS1_15SmallVectorImplIS3_EERKS5_", "(anonymous namespace)::TrivialSpiller::spill(llvm::LiveInterval*, llvm::SmallVectorImpl<llvm::LiveInterval*>&, llvm::SmallVectorImpl<llvm::LiveInterval*> const&)"},
    {"_ZN12_GLOBAL__N_111SpillerBaseD1Ev", "(anonymous namespace)::SpillerBase::~SpillerBase()"},
    {"_ZN12_GLOBAL__N_111SpillerBaseD0Ev", "(anonymous namespace)::SpillerBase::~SpillerBase()"},
    {"_ZN4llvm15SmallVectorImplIPNS_12LiveIntervalEE6insertIN9__gnu_cxx17__normal_iteratorIPS2_St6vectorIS2_SaIS2_EEEEEES7_S7_T_SC_", "llvm::LiveInterval** llvm::SmallVectorImpl<llvm::LiveInterval*>::insert<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > > >(llvm::LiveInterval**, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >)"},
    {"_ZN4llvm11SlotIndexes24insertMachineInstrInMapsEPNS_12MachineInstrEPb", "llvm::SlotIndexes::insertMachineInstrInMaps(llvm::MachineInstr*, bool*)"},
    {"_ZN4llvm28initializeStackProtectorPassERNS_12PassRegistryE", "llvm::initializeStackProtectorPass(llvm::PassRegistry&)"},
    {"_ZN4llvm24createStackProtectorPassEPKNS_14TargetLoweringE", "llvm::createStackProtectorPass(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_114StackProtectorD1Ev", "(anonymous namespace)::StackProtector::~StackProtector()"},
    {"_ZN12_GLOBAL__N_114StackProtectorD0Ev", "(anonymous namespace)::StackProtector::~StackProtector()"},
    {"_ZNK12_GLOBAL__N_114StackProtector16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::StackProtector::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114StackProtector13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::StackProtector::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114StackProtectorEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::StackProtector>()"},
    {"_ZN4llvm8ICmpInstC2ERNS_10BasicBlockENS_7CmpInst9PredicateEPNS_5ValueES6_RKNS_5TwineE", "llvm::ICmpInst::ICmpInst(llvm::BasicBlock&, llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm31initializeStackSlotColoringPassERNS_12PassRegistryE", "llvm::initializeStackSlotColoringPass(llvm::PassRegistry&)"},
    {"_ZN4llvm27createStackSlotColoringPassEb", "llvm::createStackSlotColoringPass(bool)"},
    {"_ZN12_GLOBAL__N_117StackSlotColoringD1Ev", "(anonymous namespace)::StackSlotColoring::~StackSlotColoring()"},
    {"_ZN12_GLOBAL__N_117StackSlotColoringD0Ev", "(anonymous namespace)::StackSlotColoring::~StackSlotColoring()"},
    {"_ZNK12_GLOBAL__N_117StackSlotColoring11getPassNameEv", "(anonymous namespace)::StackSlotColoring::getPassName() const"},
    {"_ZNK12_GLOBAL__N_117StackSlotColoring16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::StackSlotColoring::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_117StackSlotColoring20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::StackSlotColoring::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEEN12_GLOBAL__N_114IntervalSorterEEvT_SC_T0_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, (anonymous namespace)::IntervalSorter>(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, (anonymous namespace)::IntervalSorter)"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEES5_lN12_GLOBAL__N_114IntervalSorterEEvT_SC_T0_T1_T2_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, llvm::LiveInterval**, long, (anonymous namespace)::IntervalSorter>(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, llvm::LiveInterval**, long, (anonymous namespace)::IntervalSorter)"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEES5_N12_GLOBAL__N_114IntervalSorterEEvT_SC_T0_T1_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, llvm::LiveInterval**, (anonymous namespace)::IntervalSorter>(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, llvm::LiveInterval**, (anonymous namespace)::IntervalSorter)"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEElS5_N12_GLOBAL__N_114IntervalSorterEEvT_SC_SC_T0_SD_T1_SD_T2_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, long, llvm::LiveInterval**, (anonymous namespace)::IntervalSorter>(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, long, long, llvm::LiveInterval**, long, (anonymous namespace)::IntervalSorter)"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEElN12_GLOBAL__N_114IntervalSorterEEvT_SC_SC_T0_SD_T1_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, long, (anonymous namespace)::IntervalSorter>(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, long, long, (anonymous namespace)::IntervalSorter)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_117StackSlotColoringEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::StackSlotColoring>()"},
    {"_ZN4llvm15SmallVectorImplINS_8SmallSetIjLj4EEEED2Ev", "llvm::SmallVectorImpl<llvm::SmallSet<unsigned int, 4u> >::~SmallVectorImpl()"},
    {"_ZN4llvm15SmallVectorImplINS_8SmallSetIjLj4EEEE9push_backERKS2_", "llvm::SmallVectorImpl<llvm::SmallSet<unsigned int, 4u> >::push_back(llvm::SmallSet<unsigned int, 4u> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_8SmallSetIjLj4EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallSet<unsigned int, 4u>, false>::grow(unsigned long)"},
    {"_ZSt24__uninitialized_copy_auxIPN4llvm8SmallSetIjLj4EEES3_ET0_T_S5_S4_St12__false_type", "llvm::SmallSet<unsigned int, 4u>* std::__uninitialized_copy_aux<llvm::SmallSet<unsigned int, 4u>*, llvm::SmallSet<unsigned int, 4u>*>(llvm::SmallSet<unsigned int, 4u>*, llvm::SmallSet<unsigned int, 4u>*, llvm::SmallSet<unsigned int, 4u>*, std::__false_type)"},
    {"_ZNSt8_Rb_treeIjjSt9_IdentityIjESt4lessIjESaIjEE7_M_copyEPKSt13_Rb_tree_nodeIjEPS7_", "std::_Rb_tree<unsigned int, unsigned int, std::_Identity<unsigned int>, std::less<unsigned int>, std::allocator<unsigned int> >::_M_copy(std::_Rb_tree_node<unsigned int> const*, std::_Rb_tree_node<unsigned int>*)"},
    {"_ZN4llvm10VirtRegMap22getFirstUnusedRegisterEPKNS_19TargetRegisterClassE", "llvm::VirtRegMap::getFirstUnusedRegister(llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::FindAndConstruct(llvm::TargetRegisterClass const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::InsertIntoBucket(llvm::TargetRegisterClass const* const&, llvm::BitVector const&, std::pair<llvm::TargetRegisterClass const*, llvm::BitVector>*)"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::grow(unsigned int)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPPN4llvm12LiveIntervalESt6vectorIS4_SaIS4_EEEEEvT_SA_SA_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > > >(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, __gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, std::random_access_iterator_tag)"},
    {"_ZN4llvm11SmallVectorINS0_IiLj4EEELj16EEC2EjRKS1_", "llvm::SmallVector<llvm::SmallVector<int, 4u>, 16u>::SmallVector(unsigned int, llvm::SmallVector<int, 4u> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIiLj4EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<int, 4u>, false>::grow(unsigned long)"},
    {"_ZN4llvm15SmallVectorImplIiEaSERKS1_", "llvm::SmallVectorImpl<int>::operator=(llvm::SmallVectorImpl<int> const&)"},
    {"_ZN4llvm11SmallVectorIfLj16EEC2EjRKf", "llvm::SmallVector<float, 16u>::SmallVector(unsigned int, float const&)"},
    {"_ZN4llvm11SmallVectorIiLj16EEC2EjRKi", "llvm::SmallVector<int, 16u>::SmallVector(unsigned int, int const&)"},
    {"_ZN4llvm15SmallVectorImplINS_11SmallVectorIPNS_12LiveIntervalELj4EEEE6resizeEj", "llvm::SmallVectorImpl<llvm::SmallVector<llvm::LiveInterval*, 4u> >::resize(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm12LiveIntervalESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::LiveInterval**, std::vector<llvm::LiveInterval*, std::allocator<llvm::LiveInterval*> > >, llvm::LiveInterval* const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIPNS_12LiveIntervalELj4EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<llvm::LiveInterval*, 4u>, false>::grow(unsigned long)"},
    {"_ZN4llvm15SmallVectorImplIPNS_12LiveIntervalEEaSERKS3_", "llvm::SmallVectorImpl<llvm::LiveInterval*>::operator=(llvm::SmallVectorImpl<llvm::LiveInterval*> const&)"},
    {"_ZN4llvm15SmallVectorImplINS_11SmallVectorIPNS_12MachineInstrELj8EEEE6resizeEj", "llvm::SmallVectorImpl<llvm::SmallVector<llvm::MachineInstr*, 8u> >::resize(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallVectorIPNS_12MachineInstrELj8EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallVector<llvm::MachineInstr*, 8u>, false>::grow(unsigned long)"},
    {"_ZN4llvm34initializeStrongPHIEliminationPassERNS_12PassRegistryE", "llvm::initializeStrongPHIEliminationPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_120StrongPHIEliminationEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::StrongPHIElimination>()"},
    {"_ZN12_GLOBAL__N_120StrongPHIEliminationD1Ev", "(anonymous namespace)::StrongPHIElimination::~StrongPHIElimination()"},
    {"_ZN12_GLOBAL__N_120StrongPHIEliminationD0Ev", "(anonymous namespace)::StrongPHIElimination::~StrongPHIElimination()"},
    {"_ZNK12_GLOBAL__N_120StrongPHIElimination16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::StrongPHIElimination::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_120StrongPHIElimination20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::StrongPHIElimination::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_120StrongPHIElimination9unionRegsEjj", "(anonymous namespace)::StrongPHIElimination::unionRegs(unsigned int, unsigned int)"},
    {"_ZN4llvm8DenseMapIjPN12_GLOBAL__N_120StrongPHIElimination4NodeENS_12DenseMapInfoIjEENS5_IS4_EEE5clearEv", "llvm::DenseMap<unsigned int, (anonymous namespace)::StrongPHIElimination::Node*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<(anonymous namespace)::StrongPHIElimination::Node*> >::clear()"},
    {"_ZN12_GLOBAL__N_120StrongPHIElimination11getRegColorEj", "(anonymous namespace)::StrongPHIElimination::getRegColor(unsigned int)"},
    {"_ZN12_GLOBAL__N_120StrongPHIElimination17MergeLIsAndRenameEjj", "(anonymous namespace)::StrongPHIElimination::MergeLIsAndRename(unsigned int, unsigned int)"},
    {"_ZN4llvm8DenseMapIjPN12_GLOBAL__N_120StrongPHIElimination4NodeENS_12DenseMapInfoIjEENS5_IS4_EEEixERKj", "llvm::DenseMap<unsigned int, (anonymous namespace)::StrongPHIElimination::Node*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<(anonymous namespace)::StrongPHIElimination::Node*> >::operator[](unsigned int const&)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPPN4llvm12MachineInstrESt6vectorIS4_SaIS4_EEEElN12_GLOBAL__N_114MIIndexCompareEEvT_SC_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, long, (anonymous namespace)::MIIndexCompare>(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, __gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, long, (anonymous namespace)::MIIndexCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPPN4llvm12MachineInstrESt6vectorIS4_SaIS4_EEEEN12_GLOBAL__N_114MIIndexCompareEEvT_SC_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, (anonymous namespace)::MIIndexCompare>(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, __gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, (anonymous namespace)::MIIndexCompare)"},
    {"_ZNK12_GLOBAL__N_114MIIndexCompareclEPKN4llvm12MachineInstrES4_", "(anonymous namespace)::MIIndexCompare::operator()(llvm::MachineInstr const*, llvm::MachineInstr const*) const"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPPN4llvm12MachineInstrESt6vectorIS4_SaIS4_EEEElS4_N12_GLOBAL__N_114MIIndexCompareEEvT_T0_SD_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, long, llvm::MachineInstr*, (anonymous namespace)::MIIndexCompare>(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, long, long, llvm::MachineInstr*, (anonymous namespace)::MIIndexCompare)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockESt6vectorIPNS_12MachineInstrESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE5clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEPNS_12MachineInstrENS_12DenseMapInfoIS4_EENS7_IS6_EEE5clearEv", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::MachineInstr*> >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEPNS_12MachineInstrENS_12DenseMapInfoIS4_EENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::MachineInstr*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE5clearEv", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE16shrink_and_clearEv", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockESt6vectorIPNS_12MachineInstrESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > >::shrink_and_clear()"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEPNS_12MachineInstrENS_12DenseMapInfoIS4_EENS7_IS6_EEE15LookupBucketForERKS4_RPS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::MachineInstr*> >::LookupBucketFor(std::pair<llvm::MachineBasicBlock*, unsigned int> const&, std::pair<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEPNS_12MachineInstrENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::MachineInstr*> >::InsertIntoBucket(std::pair<llvm::MachineBasicBlock*, unsigned int> const&, llvm::MachineInstr* const&, std::pair<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEPNS_12MachineInstrENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, llvm::MachineInstr*, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::MachineInstr*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE6insertERKS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<llvm::MachineBasicBlock*, unsigned int>, char> const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE15LookupBucketForERKS4_RPS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::LookupBucketFor(std::pair<llvm::MachineBasicBlock*, unsigned int> const&, std::pair<std::pair<llvm::MachineBasicBlock*, unsigned int>, char>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE16InsertIntoBucketERKS4_RKcPS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<llvm::MachineBasicBlock*, unsigned int> const&, char const&, std::pair<std::pair<llvm::MachineBasicBlock*, unsigned int>, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_17MachineBasicBlockEjEcNS_12DenseMapInfoIS4_EENS5_IcEEE4growEj", "llvm::DenseMap<std::pair<llvm::MachineBasicBlock*, unsigned int>, char, llvm::DenseMapInfo<std::pair<llvm::MachineBasicBlock*, unsigned int> >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm11df_iteratorIPNS_15DomTreeNodeBaseINS_17MachineBasicBlockEEENS_11SmallPtrSetIS4_Lj8EEELb0ENS_11GraphTraitsIS4_EEE6toNextEv", "llvm::df_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, llvm::SmallPtrSet<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 8u>, false, llvm::GraphTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_15DomTreeNodeBaseINS1_17MachineBasicBlockEEELj1EjNS1_21PointerLikeTypeTraitsIS6_EEEEN9__gnu_cxx17__normal_iteratorIPS6_S_IS6_SaIS6_EEEEESaISG_EE13_M_insert_auxENSB_IPSG_SI_EERKSG_", "std::vector<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > >*, std::vector<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > > > > >, std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::MachineBasicBlock>*> > > > const&)"},
    {"_ZNK4llvm13LiveIntervals14isLiveOutOfMBBERKNS_12LiveIntervalEPKNS_17MachineBasicBlockE", "llvm::LiveIntervals::isLiveOutOfMBB(llvm::LiveInterval const&, llvm::MachineBasicBlock const*) const"},
    {"_ZN4llvm8DenseMapIjSt4pairIPNS_12MachineInstrEjENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PS1_IjS4_E", "llvm::DenseMap<unsigned int, std::pair<llvm::MachineInstr*, unsigned int>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<llvm::MachineInstr*, unsigned int> > >::InsertIntoBucket(unsigned int const&, std::pair<llvm::MachineInstr*, unsigned int> const&, std::pair<unsigned int, std::pair<llvm::MachineInstr*, unsigned int> >*)"},
    {"_ZN4llvm8DenseMapIjSt4pairIPNS_12MachineInstrEjENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, std::pair<llvm::MachineInstr*, unsigned int>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<llvm::MachineInstr*, unsigned int> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIjSt4pairIPNS_12MachineInstrEjENS_12DenseMapInfoIjEENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, std::pair<llvm::MachineInstr*, unsigned int>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::pair<llvm::MachineInstr*, unsigned int> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockESt6vectorIPNS_12MachineInstrESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > >::FindAndConstruct(llvm::MachineBasicBlock* const&)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockESt6vectorIPNS_12MachineInstrESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > >::InsertIntoBucket(llvm::MachineBasicBlock* const&, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > const&, std::pair<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >*)"},
    {"_ZN4llvm8DenseMapIPNS_17MachineBasicBlockESt6vectorIPNS_12MachineInstrESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::MachineBasicBlock*, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >, llvm::DenseMapInfo<llvm::MachineBasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > > >::grow(unsigned int)"},
    {"_ZN4llvm23createTailDuplicatePassEb", "llvm::createTailDuplicatePass(bool)"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePassD1Ev", "(anonymous namespace)::TailDuplicatePass::~TailDuplicatePass()"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePassD0Ev", "(anonymous namespace)::TailDuplicatePass::~TailDuplicatePass()"},
    {"_ZNK12_GLOBAL__N_117TailDuplicatePass11getPassNameEv", "(anonymous namespace)::TailDuplicatePass::getPassName() const"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::TailDuplicatePass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_Z10VerifyPHIsRN4llvm15MachineFunctionEb", "VerifyPHIs(llvm::MachineFunction&, bool)"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePass10ProcessPHIEPN4llvm12MachineInstrEPNS1_17MachineBasicBlockES5_RNS1_8DenseMapIjjNS1_12DenseMapInfoIjEES8_EERNS1_11SmallVectorISt4pairIjjELj4EEE", "(anonymous namespace)::TailDuplicatePass::ProcessPHI(llvm::MachineInstr*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >&, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>&)"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePass20DuplicateInstructionEPN4llvm12MachineInstrEPNS1_17MachineBasicBlockES5_RNS1_15MachineFunctionERNS1_8DenseMapIjjNS1_12DenseMapInfoIjEESA_EE", "(anonymous namespace)::TailDuplicatePass::DuplicateInstruction(llvm::MachineInstr*, llvm::MachineBasicBlock*, llvm::MachineBasicBlock*, llvm::MachineFunction&, llvm::DenseMap<unsigned int, unsigned int, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePass17AddSSAUpdateEntryEjjPN4llvm17MachineBasicBlockE", "(anonymous namespace)::TailDuplicatePass::AddSSAUpdateEntry(unsigned int, unsigned int, llvm::MachineBasicBlock*)"},
    {"_ZN4llvm8DenseMapIjSt6vectorISt4pairIPNS_17MachineBasicBlockEjESaIS5_EENS_12DenseMapInfoIjEENS8_IS7_EEE5clearEv", "llvm::DenseMap<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > >::clear()"},
    {"_ZN4llvm8DenseMapIjSt6vectorISt4pairIPNS_17MachineBasicBlockEjESaIS5_EENS_12DenseMapInfoIjEENS8_IS7_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIjSt6vectorISt4pairIPNS_17MachineBasicBlockEjESaIS5_EENS_12DenseMapInfoIjEENS8_IS7_EEE6insertERKS2_IjS7_E", "llvm::DenseMap<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > >::insert(std::pair<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm17MachineBasicBlockEjESaIS4_EEC2ERKS6_", "std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >::vector(std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > const&)"},
    {"_ZN4llvm8DenseMapIjSt6vectorISt4pairIPNS_17MachineBasicBlockEjESaIS5_EENS_12DenseMapInfoIjEENS8_IS7_EEE16InsertIntoBucketERKjRKS7_PS2_IjS7_E", "llvm::DenseMap<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > >::InsertIntoBucket(unsigned int const&, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > const&, std::pair<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > >*)"},
    {"_ZN4llvm8DenseMapIjSt6vectorISt4pairIPNS_17MachineBasicBlockEjESaIS5_EENS_12DenseMapInfoIjEENS8_IS7_EEE4growEj", "llvm::DenseMap<unsigned int, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > > >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIPN4llvm17MachineBasicBlockEjESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::MachineBasicBlock*, unsigned int>*, std::vector<std::pair<llvm::MachineBasicBlock*, unsigned int>, std::allocator<std::pair<llvm::MachineBasicBlock*, unsigned int> > > >, std::pair<llvm::MachineBasicBlock*, unsigned int> const&)"},
    {"_ZN4llvm14SmallSetVectorIPNS_17MachineBasicBlockELj8EEC2IN9__gnu_cxx17__normal_iteratorIPS2_St6vectorIS2_SaIS2_EEEEEET_SC_", "llvm::SmallSetVector<llvm::MachineBasicBlock*, 8u>::SmallSetVector<__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::MachineBasicBlock**, std::vector<llvm::MachineBasicBlock*, std::allocator<llvm::MachineBasicBlock*> > >)"},
    {"_ZNK4llvm19TargetInstrInfoImpl23ReplaceTailWithBranchToENS_14ilist_iteratorINS_12MachineInstrEEEPNS_17MachineBasicBlockE", "llvm::TargetInstrInfoImpl::ReplaceTailWithBranchTo(llvm::ilist_iterator<llvm::MachineInstr>, llvm::MachineBasicBlock*) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl18commuteInstructionEPNS_12MachineInstrEb", "llvm::TargetInstrInfoImpl::commuteInstruction(llvm::MachineInstr*, bool) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl21findCommutedOpIndicesEPNS_12MachineInstrERjS3_", "llvm::TargetInstrInfoImpl::findCommutedOpIndices(llvm::MachineInstr*, unsigned int&, unsigned int&) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl20PredicateInstructionEPNS_12MachineInstrERKNS_15SmallVectorImplINS_14MachineOperandEEE", "llvm::TargetInstrInfoImpl::PredicateInstruction(llvm::MachineInstr*, llvm::SmallVectorImpl<llvm::MachineOperand> const&) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl13reMaterializeERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEEjjPKS4_RKNS_18TargetRegisterInfoE", "llvm::TargetInstrInfoImpl::reMaterialize(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, unsigned int, llvm::MachineInstr const*, llvm::TargetRegisterInfo const&) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl16produceSameValueEPKNS_12MachineInstrES3_PKNS_19MachineRegisterInfoE", "llvm::TargetInstrInfoImpl::produceSameValue(llvm::MachineInstr const*, llvm::MachineInstr const*, llvm::MachineRegisterInfo const*) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl9duplicateEPNS_12MachineInstrERNS_15MachineFunctionE", "llvm::TargetInstrInfoImpl::duplicate(llvm::MachineInstr*, llvm::MachineFunction&) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl20canFoldMemoryOperandEPKNS_12MachineInstrERKNS_15SmallVectorImplIjEE", "llvm::TargetInstrInfoImpl::canFoldMemoryOperand(llvm::MachineInstr const*, llvm::SmallVectorImpl<unsigned int> const&) const"},
    {"_Z11canFoldCopyPKN4llvm12MachineInstrEj", "canFoldCopy(llvm::MachineInstr const*, unsigned int)"},
    {"_ZNK4llvm15TargetInstrInfo17foldMemoryOperandENS_14ilist_iteratorINS_12MachineInstrEEERKNS_15SmallVectorImplIjEEi", "llvm::TargetInstrInfo::foldMemoryOperand(llvm::ilist_iterator<llvm::MachineInstr>, llvm::SmallVectorImpl<unsigned int> const&, int) const"},
    {"_ZNK4llvm15TargetInstrInfo17foldMemoryOperandENS_14ilist_iteratorINS_12MachineInstrEEERKNS_15SmallVectorImplIjEEPS2_", "llvm::TargetInstrInfo::foldMemoryOperand(llvm::ilist_iterator<llvm::MachineInstr>, llvm::SmallVectorImpl<unsigned int> const&, llvm::MachineInstr*) const"},
    {"_ZNK4llvm15TargetInstrInfo40isReallyTriviallyReMaterializableGenericEPKNS_12MachineInstrEPNS_13AliasAnalysisE", "llvm::TargetInstrInfo::isReallyTriviallyReMaterializableGeneric(llvm::MachineInstr const*, llvm::AliasAnalysis*) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl20isSchedulingBoundaryEPKNS_12MachineInstrEPKNS_17MachineBasicBlockERKNS_15MachineFunctionE", "llvm::TargetInstrInfoImpl::isSchedulingBoundary(llvm::MachineInstr const*, llvm::MachineBasicBlock const*, llvm::MachineFunction const&) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl24usePreRAHazardRecognizerEv", "llvm::TargetInstrInfoImpl::usePreRAHazardRecognizer() const"},
    {"_ZNK4llvm19TargetInstrInfoImpl28CreateTargetHazardRecognizerEPKNS_13TargetMachineEPKNS_11ScheduleDAGE", "llvm::TargetInstrInfoImpl::CreateTargetHazardRecognizer(llvm::TargetMachine const*, llvm::ScheduleDAG const*) const"},
    {"_ZNK4llvm19TargetInstrInfoImpl34CreateTargetPostRAHazardRecognizerEPKNS_18InstrItineraryDataEPKNS_11ScheduleDAGE", "llvm::TargetInstrInfoImpl::CreateTargetPostRAHazardRecognizer(llvm::InstrItineraryData const*, llvm::ScheduleDAG const*) const"},
    {"_ZNK4llvm24ScheduleHazardRecognizer12atIssueLimitEv", "llvm::ScheduleHazardRecognizer::atIssueLimit() const"},
    {"_ZN4llvm24ScheduleHazardRecognizer13getHazardTypeEPNS_5SUnitEi", "llvm::ScheduleHazardRecognizer::getHazardType(llvm::SUnit*, int)"},
    {"_ZN4llvm24ScheduleHazardRecognizer5ResetEv", "llvm::ScheduleHazardRecognizer::Reset()"},
    {"_ZN4llvm24ScheduleHazardRecognizer15EmitInstructionEPNS_5SUnitE", "llvm::ScheduleHazardRecognizer::EmitInstruction(llvm::SUnit*)"},
    {"_ZN4llvm24ScheduleHazardRecognizer12AdvanceCycleEv", "llvm::ScheduleHazardRecognizer::AdvanceCycle()"},
    {"_ZN4llvm24ScheduleHazardRecognizer11RecedeCycleEv", "llvm::ScheduleHazardRecognizer::RecedeCycle()"},
    {"_ZN4llvm27TargetLoweringObjectFileELF10InitializeERNS_9MCContextERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileELF::Initialize(llvm::MCContext&, llvm::TargetMachine const&)"},
    {"_ZNK4llvm27TargetLoweringObjectFileELF17getEHFrameSectionEv", "llvm::TargetLoweringObjectFileELF::getEHFrameSection() const"},
    {"_ZNK4llvm27TargetLoweringObjectFileELF24getExplicitSectionGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileELF::getExplicitSectionGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm27TargetLoweringObjectFileELF22SelectSectionForGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileELF::SelectSectionForGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm27TargetLoweringObjectFileELF21getSectionForConstantENS_11SectionKindE", "llvm::TargetLoweringObjectFileELF::getSectionForConstant(llvm::SectionKind) const"},
    {"_ZNK4llvm27TargetLoweringObjectFileELF30getExprForDwarfGlobalReferenceEPKNS_11GlobalValueEPNS_7ManglerEPNS_17MachineModuleInfoEjRNS_10MCStreamerE", "llvm::TargetLoweringObjectFileELF::getExprForDwarfGlobalReference(llvm::GlobalValue const*, llvm::Mangler*, llvm::MachineModuleInfo*, unsigned int, llvm::MCStreamer&) const"},
    {"_ZN4llvm29TargetLoweringObjectFileMachO10InitializeERNS_9MCContextERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileMachO::Initialize(llvm::MCContext&, llvm::TargetMachine const&)"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO17getEHFrameSectionEv", "llvm::TargetLoweringObjectFileMachO::getEHFrameSection() const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO24getExplicitSectionGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileMachO::getExplicitSectionGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO22SelectSectionForGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileMachO::SelectSectionForGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO21getSectionForConstantENS_11SectionKindE", "llvm::TargetLoweringObjectFileMachO::getSectionForConstant(llvm::SectionKind) const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO26shouldEmitUsedDirectiveForEPKNS_11GlobalValueEPNS_7ManglerE", "llvm::TargetLoweringObjectFileMachO::shouldEmitUsedDirectiveFor(llvm::GlobalValue const*, llvm::Mangler*) const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO30getExprForDwarfGlobalReferenceEPKNS_11GlobalValueEPNS_7ManglerEPNS_17MachineModuleInfoEjRNS_10MCStreamerE", "llvm::TargetLoweringObjectFileMachO::getExprForDwarfGlobalReference(llvm::GlobalValue const*, llvm::Mangler*, llvm::MachineModuleInfo*, unsigned int, llvm::MCStreamer&) const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO22getPersonalityEncodingEv", "llvm::TargetLoweringObjectFileMachO::getPersonalityEncoding() const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO15getLSDAEncodingEv", "llvm::TargetLoweringObjectFileMachO::getLSDAEncoding() const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO14getFDEEncodingEv", "llvm::TargetLoweringObjectFileMachO::getFDEEncoding() const"},
    {"_ZNK4llvm29TargetLoweringObjectFileMachO16getTTypeEncodingEv", "llvm::TargetLoweringObjectFileMachO::getTTypeEncoding() const"},
    {"_ZN4llvm28TargetLoweringObjectFileCOFF10InitializeERNS_9MCContextERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileCOFF::Initialize(llvm::MCContext&, llvm::TargetMachine const&)"},
    {"_ZNK4llvm28TargetLoweringObjectFileCOFF17getEHFrameSectionEv", "llvm::TargetLoweringObjectFileCOFF::getEHFrameSection() const"},
    {"_ZNK4llvm28TargetLoweringObjectFileCOFF24getExplicitSectionGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileCOFF::getExplicitSectionGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm28TargetLoweringObjectFileCOFF22SelectSectionForGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFileCOFF::SelectSectionForGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZN4llvm39initializeTwoAddressInstructionPassPassERNS_12PassRegistryE", "llvm::initializeTwoAddressInstructionPassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_125TwoAddressInstructionPassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::TwoAddressInstructionPass>()"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPassD1Ev", "(anonymous namespace)::TwoAddressInstructionPass::~TwoAddressInstructionPass()"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPassD0Ev", "(anonymous namespace)::TwoAddressInstructionPass::~TwoAddressInstructionPass()"},
    {"_ZNK12_GLOBAL__N_125TwoAddressInstructionPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::TwoAddressInstructionPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPass20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::TwoAddressInstructionPass::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPass23TryInstructionTransformERN4llvm14ilist_iteratorINS1_12MachineInstrEEES5_RNS2_INS1_17MachineBasicBlockEEEjjj", "(anonymous namespace)::TwoAddressInstructionPass::TryInstructionTransform(llvm::ilist_iterator<llvm::MachineInstr>&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::ilist_iterator<llvm::MachineBasicBlock>&, unsigned int, unsigned int, unsigned int)"},
    {"_Z8isKilledRN4llvm12MachineInstrEjPKNS_19MachineRegisterInfoEPKNS_15TargetInstrInfoE", "isKilled(llvm::MachineInstr&, unsigned int, llvm::MachineRegisterInfo const*, llvm::TargetInstrInfo const*)"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPass17NoUseAfterLastDefEjPN4llvm17MachineBasicBlockEjRj", "(anonymous namespace)::TwoAddressInstructionPass::NoUseAfterLastDef(unsigned int, llvm::MachineBasicBlock*, unsigned int, unsigned int&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIjjELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE6insertERKS2_IjS4_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > >::insert(std::pair<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIjjELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE5clearEv", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > >::clear()"},
    {"_ZSt16__introsort_loopIPjlEvT_S1_T0_", "void std::__introsort_loop<unsigned int*, long>(unsigned int*, unsigned int*, long)"},
    {"_ZSt22__final_insertion_sortIPjEvT_S1_", "void std::__final_insertion_sort<unsigned int*>(unsigned int*, unsigned int*)"},
    {"_ZSt13__heap_selectIPjEvT_S1_S1_", "void std::__heap_select<unsigned int*>(unsigned int*, unsigned int*, unsigned int*)"},
    {"_ZSt9sort_heapIPjEvT_S1_", "void std::sort_heap<unsigned int*>(unsigned int*, unsigned int*)"},
    {"_ZSt9make_heapIPjEvT_S1_", "void std::make_heap<unsigned int*>(unsigned int*, unsigned int*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIjjELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > >::shrink_and_clear()"},
    {"_ZN4llvm15SmallVectorImplISt4pairIjjEEaSERKS3_", "llvm::SmallVectorImpl<std::pair<unsigned int, unsigned int> >::operator=(llvm::SmallVectorImpl<std::pair<unsigned int, unsigned int> > const&)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIjjELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE16InsertIntoBucketERKjRKS4_PS2_IjS4_E", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > >::InsertIntoBucket(unsigned int const&, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> const&, std::pair<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> >*)"},
    {"_ZN4llvm8DenseMapIjNS_11SmallVectorISt4pairIjjELj4EEENS_12DenseMapInfoIjEENS5_IS4_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u>, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, unsigned int>, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm34initializeUnreachableBlockElimPassERNS_12PassRegistryE", "llvm::initializeUnreachableBlockElimPass(llvm::PassRegistry&)"},
    {"_ZN4llvm37createUnreachableBlockEliminationPassEv", "llvm::createUnreachableBlockEliminationPass()"},
    {"_ZN4llvm41initializeUnreachableMachineBlockElimPassERNS_12PassRegistryE", "llvm::initializeUnreachableMachineBlockElimPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_127UnreachableMachineBlockElimEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::UnreachableMachineBlockElim>()"},
    {"_ZN12_GLOBAL__N_127UnreachableMachineBlockElimD1Ev", "(anonymous namespace)::UnreachableMachineBlockElim::~UnreachableMachineBlockElim()"},
    {"_ZN12_GLOBAL__N_127UnreachableMachineBlockElimD0Ev", "(anonymous namespace)::UnreachableMachineBlockElim::~UnreachableMachineBlockElim()"},
    {"_ZNK12_GLOBAL__N_127UnreachableMachineBlockElim16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::UnreachableMachineBlockElim::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_127UnreachableMachineBlockElim20runOnMachineFunctionERN4llvm15MachineFunctionE", "(anonymous namespace)::UnreachableMachineBlockElim::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN12_GLOBAL__N_120UnreachableBlockElimD1Ev", "(anonymous namespace)::UnreachableBlockElim::~UnreachableBlockElim()"},
    {"_ZN12_GLOBAL__N_120UnreachableBlockElimD0Ev", "(anonymous namespace)::UnreachableBlockElim::~UnreachableBlockElim()"},
    {"_ZNK12_GLOBAL__N_120UnreachableBlockElim16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::UnreachableBlockElim::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_120UnreachableBlockElim13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::UnreachableBlockElim::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_120UnreachableBlockElimEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::UnreachableBlockElim>()"},
    {"_ZN4llvm12df_ext_beginIPNS_15MachineFunctionENS_11SmallPtrSetIPNS_17MachineBasicBlockELj8EEEEENS_15df_ext_iteratorIT_T0_EERKS8_RS9_", "llvm::df_ext_iterator<llvm::MachineFunction*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u> > llvm::df_ext_begin<llvm::MachineFunction*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u> >(llvm::MachineFunction* const&, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>&)"},
    {"_ZN4llvm11df_iteratorIPNS_15MachineFunctionENS_11SmallPtrSetIPNS_17MachineBasicBlockELj8EEELb1ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::MachineFunction*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>, true, llvm::GraphTraits<llvm::MachineFunction*> >::toNext()"},
    {"_ZN4llvm11df_iteratorIPNS_15MachineFunctionENS_11SmallPtrSetIPNS_17MachineBasicBlockELj8EEELb1ENS_11GraphTraitsIS2_EEEC2ES5_RS6_", "llvm::df_iterator<llvm::MachineFunction*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>, true, llvm::GraphTraits<llvm::MachineFunction*> >::df_iterator(llvm::MachineBasicBlock*, llvm::SmallPtrSet<llvm::MachineBasicBlock*, 8u>&)"},
    {"_ZN4llvm12df_ext_beginIPNS_8FunctionENS_11SmallPtrSetIPNS_10BasicBlockELj8EEEEENS_15df_ext_iteratorIT_T0_EERKS8_RS9_", "llvm::df_ext_iterator<llvm::Function*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u> > llvm::df_ext_begin<llvm::Function*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u> >(llvm::Function* const&, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>&)"},
    {"_ZN4llvm11df_iteratorIPNS_8FunctionENS_11SmallPtrSetIPNS_10BasicBlockELj8EEELb1ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::Function*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, true, llvm::GraphTraits<llvm::Function*> >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_10BasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEENS1_12SuccIteratorIPNS1_14TerminatorInstES3_EEESaISC_EEC2ERKSE_", "std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > >::vector(std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > > const&)"},
    {"_ZN4llvm11df_iteratorIPNS_8FunctionENS_11SmallPtrSetIPNS_10BasicBlockELj8EEELb1ENS_11GraphTraitsIS2_EEEC2ES5_RS6_", "llvm::df_iterator<llvm::Function*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, true, llvm::GraphTraits<llvm::Function*> >::df_iterator(llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>&)"},
    {"_ZN4llvm24initializeVirtRegMapPassERNS_12PassRegistryE", "llvm::initializeVirtRegMapPass(llvm::PassRegistry&)"},
    {"_ZN4llvm10VirtRegMap20runOnMachineFunctionERNS_15MachineFunctionE", "llvm::VirtRegMap::runOnMachineFunction(llvm::MachineFunction&)"},
    {"_ZN4llvm10VirtRegMap4growEv", "llvm::VirtRegMap::grow()"},
    {"_ZN4llvm10VirtRegMap15createSpillSlotEPKNS_19TargetRegisterClassE", "llvm::VirtRegMap::createSpillSlot(llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm10VirtRegMap15getRegAllocPrefEj", "llvm::VirtRegMap::getRegAllocPref(unsigned int)"},
    {"_ZN4llvm10VirtRegMap20assignVirt2StackSlotEj", "llvm::VirtRegMap::assignVirt2StackSlot(unsigned int)"},
    {"_ZN4llvm10VirtRegMap20assignVirt2StackSlotEji", "llvm::VirtRegMap::assignVirt2StackSlot(unsigned int, int)"},
    {"_ZN4llvm10VirtRegMap17assignVirtReMatIdEj", "llvm::VirtRegMap::assignVirtReMatId(unsigned int)"},
    {"_ZN4llvm10VirtRegMap17assignVirtReMatIdEji", "llvm::VirtRegMap::assignVirtReMatId(unsigned int, int)"},
    {"_ZN4llvm10VirtRegMap21getEmergencySpillSlotEPKNS_19TargetRegisterClassE", "llvm::VirtRegMap::getEmergencySpillSlot(llvm::TargetRegisterClass const*)"},
    {"_ZN4llvm10VirtRegMap15addSpillSlotUseEiPNS_12MachineInstrE", "llvm::VirtRegMap::addSpillSlotUse(int, llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap10virtFoldedEjPNS_12MachineInstrES2_NS0_6ModRefE", "llvm::VirtRegMap::virtFolded(unsigned int, llvm::MachineInstr*, llvm::MachineInstr*, llvm::VirtRegMap::ModRef)"},
    {"_ZN4llvm10VirtRegMap10virtFoldedEjPNS_12MachineInstrENS0_6ModRefE", "llvm::VirtRegMap::virtFolded(unsigned int, llvm::MachineInstr*, llvm::VirtRegMap::ModRef)"},
    {"_ZN4llvm10VirtRegMap26RemoveMachineInstrFromMapsEPNS_12MachineInstrE", "llvm::VirtRegMap::RemoveMachineInstrFromMaps(llvm::MachineInstr*)"},
    {"_ZN4llvm10VirtRegMap19FindUnusedRegistersEPNS_13LiveIntervalsE", "llvm::VirtRegMap::FindUnusedRegisters(llvm::LiveIntervals*)"},
    {"_ZNK4llvm10VirtRegMap5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::VirtRegMap::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm10VirtRegMap4dumpEv", "llvm::VirtRegMap::dump() const"},
    {"_ZN4llvm15SmallVectorImplINS_11SmallPtrSetIPNS_12MachineInstrELj4EEEE6resizeEj", "llvm::SmallVectorImpl<llvm::SmallPtrSet<llvm::MachineInstr*, 4u> >::resize(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE5clearEv", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::clear()"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE6insertERKSt4pairIS3_S4_E", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::insert(std::pair<llvm::TargetRegisterClass const*, llvm::BitVector> const&)"},
    {"_ZN4llvm10VirtRegMapD1Ev", "llvm::VirtRegMap::~VirtRegMap()"},
    {"_ZN4llvm10VirtRegMapD0Ev", "llvm::VirtRegMap::~VirtRegMap()"},
    {"_ZNK4llvm10VirtRegMap16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::VirtRegMap::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIjSaIjEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE5eraseESt17_Rb_tree_iteratorIS8_ESG_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIjSaIjEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineInstr* const, std::vector<unsigned int, std::allocator<unsigned int> > > >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIS3_IjbESaIS6_EEESt10_Select1stIS9_ESt4lessIS2_ESaIS9_EE5eraseESt17_Rb_tree_iteratorIS9_ESH_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIS3_IjbESaIS6_EEESt10_Select1stIS9_ESt4lessIS2_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_S3_IjNS0_10VirtRegMap6ModRefEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE5eraseESt17_Rb_tree_iteratorIS8_ESG_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >, std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_S3_IjNS0_10VirtRegMap6ModRefEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_S3_IjNS0_10VirtRegMap6ModRefEEESt10_Select1stIS8_ESt4lessIS2_ESaIS8_EE15_M_insert_equalESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > > >::_M_insert_equal(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > >, std::pair<llvm::MachineInstr* const, std::pair<unsigned int, llvm::VirtRegMap::ModRef> > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_iESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, int>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, int> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::TargetRegisterClass const* const, int> >, std::pair<llvm::TargetRegisterClass const* const, int> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_iESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, int>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, int> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, int> > >::_M_insert_unique(std::pair<llvm::TargetRegisterClass const* const, int> const&)"},
    {"_ZNSt6vectorIPN4llvm12MachineInstrESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::MachineInstr**, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> > >, unsigned long, llvm::MachineInstr* const&)"},
    {"_ZNSt6vectorIN4llvm9SlotIndexESaIS1_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS1_S3_EEmRKS1_", "std::vector<llvm::SlotIndex, std::allocator<llvm::SlotIndex> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::SlotIndex*, std::vector<llvm::SlotIndex, std::allocator<llvm::SlotIndex> > >, unsigned long, llvm::SlotIndex const&)"},
    {"_ZN4llvm8DenseMapIPKNS_19TargetRegisterClassENS_9BitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::TargetRegisterClass const*, llvm::BitVector, llvm::DenseMapInfo<llvm::TargetRegisterClass const*>, llvm::DenseMapInfo<llvm::BitVector> >::shrink_and_clear()"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_11SmallPtrSetIPNS_12MachineInstrELj4EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::SmallPtrSet<llvm::MachineInstr*, 4u>, false>::grow(unsigned long)"},
    {"_ZNSt8_Rb_treeIPKN4llvm19TargetRegisterClassESt4pairIKS3_iESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::TargetRegisterClass const*, std::pair<llvm::TargetRegisterClass const* const, int>, std::_Select1st<std::pair<llvm::TargetRegisterClass const* const, int> >, std::less<llvm::TargetRegisterClass const*>, std::allocator<std::pair<llvm::TargetRegisterClass const* const, int> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::TargetRegisterClass const* const, int> >*)"},
    {"_ZN4llvm10VirtRegMapD2Ev", "llvm::VirtRegMap::~VirtRegMap()"},
    {"_ZN4llvm15callDefaultCtorINS_10VirtRegMapEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::VirtRegMap>()"},
    {"_ZN4llvm10VirtRegMapC2Ev", "llvm::VirtRegMap::VirtRegMap()"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEED1Ev", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::~opt()"},
    {"_ZN4llvm15VirtRegRewriterD0Ev", "llvm::VirtRegRewriter::~VirtRegRewriter()"},
    {"_ZN4llvm15VirtRegRewriterD1Ev", "llvm::VirtRegRewriter::~VirtRegRewriter()"},
    {"_ZN4llvm15VirtRegRewriterD2Ev", "llvm::VirtRegRewriter::~VirtRegRewriter()"},
    {"_ZN4llvm21createVirtRegRewriterEv", "llvm::createVirtRegRewriter()"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEE14getOptionWidthEv", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEE15printOptionInfoEm", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEED0Ev", "llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >::~opt()"},
    {"_ZN4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEED1Ev", "llvm::cl::parser<(anonymous namespace)::RewriterName>::~parser()"},
    {"_ZN4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEED0Ev", "llvm::cl::parser<(anonymous namespace)::RewriterName>::~parser()"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEE13getNumOptionsEv", "llvm::cl::parser<(anonymous namespace)::RewriterName>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEE9getOptionEj", "llvm::cl::parser<(anonymous namespace)::RewriterName>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEE14getDescriptionEj", "llvm::cl::parser<(anonymous namespace)::RewriterName>::getDescription(unsigned int) const"},
    {"_ZN12_GLOBAL__N_115TrivialRewriterD1Ev", "(anonymous namespace)::TrivialRewriter::~TrivialRewriter()"},
    {"_ZN12_GLOBAL__N_115TrivialRewriterD0Ev", "(anonymous namespace)::TrivialRewriter::~TrivialRewriter()"},
    {"_ZN12_GLOBAL__N_115TrivialRewriter20runOnMachineFunctionERN4llvm15MachineFunctionERNS1_10VirtRegMapEPNS1_13LiveIntervalsE", "(anonymous namespace)::TrivialRewriter::runOnMachineFunction(llvm::MachineFunction&, llvm::VirtRegMap&, llvm::LiveIntervals*)"},
    {"_Z17substitutePhysRegRN4llvm14MachineOperandEjRKNS_18TargetRegisterInfoE", "substitutePhysReg(llvm::MachineOperand&, unsigned int, llvm::TargetRegisterInfo const&)"},
    {"_ZN12_GLOBAL__N_113LocalRewriterD1Ev", "(anonymous namespace)::LocalRewriter::~LocalRewriter()"},
    {"_ZN12_GLOBAL__N_113LocalRewriterD0Ev", "(anonymous namespace)::LocalRewriter::~LocalRewriter()"},
    {"_ZN12_GLOBAL__N_113LocalRewriter20runOnMachineFunctionERN4llvm15MachineFunctionERNS1_10VirtRegMapEPNS1_13LiveIntervalsE", "(anonymous namespace)::LocalRewriter::runOnMachineFunction(llvm::MachineFunction&, llvm::VirtRegMap&, llvm::LiveIntervals*)"},
    {"_ZN12_GLOBAL__N_113LocalRewriter10RewriteMBBEPN4llvm13LiveIntervalsERNS_15AvailableSpillsERNS1_9BitVectorERSt6vectorIPNS1_14MachineOperandESaISA_EE", "(anonymous namespace)::LocalRewriter::RewriteMBB(llvm::LiveIntervals*, (anonymous namespace)::AvailableSpills&, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&)"},
    {"_ZN12_GLOBAL__N_113LocalRewriter10EraseInstrEPN4llvm12MachineInstrE", "(anonymous namespace)::LocalRewriter::EraseInstr(llvm::MachineInstr*)"},
    {"_Z22ResurrectConfirmedKilljPKN4llvm18TargetRegisterInfoERNS_9BitVectorERSt6vectorIPNS_14MachineOperandESaIS7_EE", "ResurrectConfirmedKill(unsigned int, llvm::TargetRegisterInfo const*, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&)"},
    {"_ZN12_GLOBAL__N_115AvailableSpills18ClobberPhysRegOnlyEj", "(anonymous namespace)::AvailableSpills::ClobberPhysRegOnly(unsigned int)"},
    {"_Z15InvalidateKillsRN4llvm12MachineInstrEPKNS_18TargetRegisterInfoERNS_9BitVectorERSt6vectorIPNS_14MachineOperandESaIS9_EEPNS_11SmallVectorIjLj2EEE", "InvalidateKills(llvm::MachineInstr&, llvm::TargetRegisterInfo const*, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&, llvm::SmallVector<unsigned int, 2u>*)"},
    {"_ZN12_GLOBAL__N_115AvailableSpills22ModifyStackSlotOrReMatEi", "(anonymous namespace)::AvailableSpills::ModifyStackSlotOrReMat(int)"},
    {"_ZN12_GLOBAL__N_115AvailableSpills12addAvailableEijb", "(anonymous namespace)::AvailableSpills::addAvailable(int, unsigned int, bool)"},
    {"_Z11UpdateKillsRN4llvm12MachineInstrEPKNS_18TargetRegisterInfoERNS_9BitVectorERSt6vectorIPNS_14MachineOperandESaIS9_EE", "UpdateKills(llvm::MachineInstr&, llvm::TargetRegisterInfo const*, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&)"},
    {"_Z14isSafeToDeleteRN4llvm12MachineInstrE", "isSafeToDelete(llvm::MachineInstr&)"},
    {"_Z13ResurrectKillRN4llvm12MachineInstrEjPKNS_18TargetRegisterInfoERNS_9BitVectorERSt6vectorIPNS_14MachineOperandESaIS9_EE", "ResurrectKill(llvm::MachineInstr&, unsigned int, llvm::TargetRegisterInfo const*, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&)"},
    {"_ZN12_GLOBAL__N_19ReuseInfo15GetRegForReloadEPKN4llvm19TargetRegisterClassEjRNS1_15MachineFunctionEPNS1_12MachineInstrERNS_15AvailableSpillsERSt6vectorIS8_SaIS8_EERNS1_8SmallSetIjLj8EEERNS1_9BitVectorERSB_IPNS1_14MachineOperandESaISL_EERNS1_10VirtRegMapE", "(anonymous namespace)::ReuseInfo::GetRegForReload(llvm::TargetRegisterClass const*, unsigned int, llvm::MachineFunction&, llvm::MachineInstr*, (anonymous namespace)::AvailableSpills&, std::vector<llvm::MachineInstr*, std::allocator<llvm::MachineInstr*> >&, llvm::SmallSet<unsigned int, 8u>&, llvm::BitVector&, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >&, llvm::VirtRegMap&)"},
    {"_Z16ComputeReloadLocN4llvm14ilist_iteratorINS_12MachineInstrEEES2_jPKNS_18TargetRegisterInfoEbiPKNS_15TargetInstrInfoERKNS_15MachineFunctionE", "ComputeReloadLoc(llvm::ilist_iterator<llvm::MachineInstr>, llvm::ilist_iterator<llvm::MachineInstr>, unsigned int, llvm::TargetRegisterInfo const*, bool, int, llvm::TargetInstrInfo const*, llvm::MachineFunction const&)"},
    {"_Z13ReMaterializeRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEEjjPKNS_15TargetInstrInfoEPKNS_18TargetRegisterInfoERNS_10VirtRegMapE", "ReMaterialize(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, unsigned int, unsigned int, llvm::TargetInstrInfo const*, llvm::TargetRegisterInfo const*, llvm::VirtRegMap&)"},
    {"_ZSt16__introsort_loopIPSt4pairIPN4llvm12MachineInstrEiElN12_GLOBAL__N_19RefSorterEEvT_S8_T0_T1_", "void std::__introsort_loop<std::pair<llvm::MachineInstr*, int>*, long, (anonymous namespace)::RefSorter>(std::pair<llvm::MachineInstr*, int>*, std::pair<llvm::MachineInstr*, int>*, long, (anonymous namespace)::RefSorter)"},
    {"_ZSt16__insertion_sortIPSt4pairIPN4llvm12MachineInstrEiEN12_GLOBAL__N_19RefSorterEEvT_S8_T0_", "void std::__insertion_sort<std::pair<llvm::MachineInstr*, int>*, (anonymous namespace)::RefSorter>(std::pair<llvm::MachineInstr*, int>*, std::pair<llvm::MachineInstr*, int>*, (anonymous namespace)::RefSorter)"},
    {"_ZN12_GLOBAL__N_115AvailableSpills26disallowClobberPhysRegOnlyEj", "(anonymous namespace)::AvailableSpills::disallowClobberPhysRegOnly(unsigned int)"},
    {"_Z20FoldsStackSlotModRefRN4llvm12MachineInstrEijPKNS_15TargetInstrInfoEPKNS_18TargetRegisterInfoERNS_10VirtRegMapE", "FoldsStackSlotModRef(llvm::MachineInstr&, int, unsigned int, llvm::TargetInstrInfo const*, llvm::TargetRegisterInfo const*, llvm::VirtRegMap&)"},
    {"_ZN4llvm8DenseMapIiNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIiEENS5_IS4_EEE5clearEv", "llvm::DenseMap<int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::clear()"},
    {"_ZNSt8_Rb_treeIiSt4pairIKijESt10_Select1stIS2_ESt4lessIiESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<int, std::pair<int const, unsigned int>, std::_Select1st<std::pair<int const, unsigned int> >, std::less<int>, std::allocator<std::pair<int const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<int const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjiESt10_Select1stIS2_ESt4lessIjESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, int>, std::_Select1st<std::pair<unsigned int const, int> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, int> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, int> >*)"},
    {"_ZN4llvm8DenseMapIiNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIiEENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIiNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIiEENS5_IS4_EEE16FindAndConstructERKi", "llvm::DenseMap<int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::FindAndConstruct(int const&)"},
    {"_ZN4llvm8DenseMapIiNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIiEENS5_IS4_EEE16InsertIntoBucketERKiRKS4_PSt4pairIiS4_E", "llvm::DenseMap<int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::InsertIntoBucket(int const&, llvm::SmallVector<llvm::MachineInstr*, 4u> const&, std::pair<int, llvm::SmallVector<llvm::MachineInstr*, 4u> >*)"},
    {"_ZN4llvm8DenseMapIiNS_11SmallVectorIPNS_12MachineInstrELj4EEENS_12DenseMapInfoIiEENS5_IS4_EEE4growEj", "llvm::DenseMap<int, llvm::SmallVector<llvm::MachineInstr*, 4u>, llvm::DenseMapInfo<int>, llvm::DenseMapInfo<llvm::SmallVector<llvm::MachineInstr*, 4u> > >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKijESt10_Select1stIS2_ESt4lessIiESaIS2_EE5eraseESt17_Rb_tree_iteratorIS2_ESA_", "std::_Rb_tree<int, std::pair<int const, unsigned int>, std::_Select1st<std::pair<int const, unsigned int> >, std::less<int>, std::allocator<std::pair<int const, unsigned int> > >::erase(std::_Rb_tree_iterator<std::pair<int const, unsigned int> >, std::_Rb_tree_iterator<std::pair<int const, unsigned int> >)"},
    {"_ZN4llvm8SmallSetIiLj2EE6insertERKi", "llvm::SmallSet<int, 2u>::insert(int const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKijESt10_Select1stIS2_ESt4lessIiESaIS2_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS2_ERKS2_", "std::_Rb_tree<int, std::pair<int const, unsigned int>, std::_Select1st<std::pair<int const, unsigned int> >, std::less<int>, std::allocator<std::pair<int const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<int const, unsigned int> >, std::pair<int const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIiSt4pairIKijESt10_Select1stIS2_ESt4lessIiESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<int, std::pair<int const, unsigned int>, std::_Select1st<std::pair<int const, unsigned int> >, std::less<int>, std::allocator<std::pair<int const, unsigned int> > >::_M_insert_unique(std::pair<int const, unsigned int> const&)"},
    {"_ZN4llvm8SmallSetIjLj2EE6insertERKj", "llvm::SmallSet<unsigned int, 2u>::insert(unsigned int const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm12MachineInstrESt4pairIKS2_St6vectorIS3_IjbESaIS6_EEESt10_Select1stIS9_ESt4lessIS2_ESaIS9_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS9_ERKS9_", "std::_Rb_tree<llvm::MachineInstr*, std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > >, std::_Select1st<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::less<llvm::MachineInstr*>, std::allocator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > >, std::pair<llvm::MachineInstr* const, std::vector<std::pair<unsigned int, bool>, std::allocator<std::pair<unsigned int, bool> > > > const&)"},
    {"_ZN4llvm8SmallSetIiLj4EE6insertERKi", "llvm::SmallSet<int, 4u>::insert(int const&)"},
    {"_ZNSt6vectorIPN4llvm14MachineOperandESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::MachineOperand**, std::vector<llvm::MachineOperand*, std::allocator<llvm::MachineOperand*> > >, unsigned long, llvm::MachineOperand* const&)"},
    {"_ZN4llvm18initializeADCEPassERNS_12PassRegistryE", "llvm::initializeADCEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createAggressiveDCEPassEv", "llvm::createAggressiveDCEPass()"},
    {"_ZN12_GLOBAL__N_14ADCED1Ev", "(anonymous namespace)::ADCE::~ADCE()"},
    {"_ZN12_GLOBAL__N_14ADCED0Ev", "(anonymous namespace)::ADCE::~ADCE()"},
    {"_ZNK12_GLOBAL__N_14ADCE16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::ADCE::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14ADCE13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::ADCE::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_14ADCEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::ADCE>()"},
    {"_ZN4llvm28initializeCodeGenPreparePassERNS_12PassRegistryE", "llvm::initializeCodeGenPreparePass(llvm::PassRegistry&)"},
    {"_ZN4llvm24createCodeGenPreparePassEPKNS_14TargetLoweringE", "llvm::createCodeGenPreparePass(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepareC1EPKN4llvm14TargetLoweringE", "(anonymous namespace)::CodeGenPrepare::CodeGenPrepare(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepareD1Ev", "(anonymous namespace)::CodeGenPrepare::~CodeGenPrepare()"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepareD0Ev", "(anonymous namespace)::CodeGenPrepare::~CodeGenPrepare()"},
    {"_ZNK12_GLOBAL__N_114CodeGenPrepare16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::CodeGenPrepare::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepare13releaseMemoryEv", "(anonymous namespace)::CodeGenPrepare::releaseMemory()"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepare13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::CodeGenPrepare::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepare12OptimizeInstEPN4llvm11InstructionE", "(anonymous namespace)::CodeGenPrepare::OptimizeInst(llvm::Instruction*)"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepare18OptimizeMemoryInstEPN4llvm11InstructionEPNS1_5ValueEPKNS1_4TypeE", "(anonymous namespace)::CodeGenPrepare::OptimizeMemoryInst(llvm::Instruction*, llvm::Value*, llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_131CodeGenPrepareFortifiedLibCallsD1Ev", "(anonymous namespace)::CodeGenPrepareFortifiedLibCalls::~CodeGenPrepareFortifiedLibCalls()"},
    {"_ZN12_GLOBAL__N_131CodeGenPrepareFortifiedLibCalls11replaceCallEPN4llvm5ValueE", "(anonymous namespace)::CodeGenPrepareFortifiedLibCalls::replaceCall(llvm::Value*)"},
    {"_ZNK12_GLOBAL__N_131CodeGenPrepareFortifiedLibCalls10isFoldableEjjb", "(anonymous namespace)::CodeGenPrepareFortifiedLibCalls::isFoldable(unsigned int, unsigned int, bool) const"},
    {"_ZN12_GLOBAL__N_131CodeGenPrepareFortifiedLibCallsD0Ev", "(anonymous namespace)::CodeGenPrepareFortifiedLibCalls::~CodeGenPrepareFortifiedLibCalls()"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114CodeGenPrepareEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::CodeGenPrepare>()"},
    {"_ZN4llvm8DenseMapIPNS_5ValueES2_NS_12DenseMapInfoIS2_EES4_E16shrink_and_clearEv", "llvm::DenseMap<llvm::Value*, llvm::Value*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::Value*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_5ValueES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::Value*, llvm::Value*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(llvm::Value* const&, llvm::Value* const&, std::pair<llvm::Value*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::Value*, llvm::Value*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPNS_11InstructionEEaSERKS3_", "llvm::SmallVectorImpl<llvm::Instruction*>::operator=(llvm::SmallVectorImpl<llvm::Instruction*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_7CmpInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::CmpInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::CmpInst*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::CmpInst* const&, std::pair<llvm::BasicBlock*, llvm::CmpInst*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_7CmpInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::CmpInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::CmpInst*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_11InstructionENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::Instruction*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Instruction*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::Instruction* const&, std::pair<llvm::BasicBlock*, llvm::Instruction*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_11InstructionENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::Instruction*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Instruction*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_8CastInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::CastInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::CastInst*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::CastInst* const&, std::pair<llvm::BasicBlock*, llvm::CastInst*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_8CastInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::CastInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::CastInst*> >::grow(unsigned int)"},
    {"_ZNK4llvm8SmallSetISt4pairIPKNS_10BasicBlockES4_ELj8EE5countERKS5_", "llvm::SmallSet<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, 8u>::count(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&) const"},
    {"_ZN4llvm8SmallSetISt4pairIPKNS_10BasicBlockES4_ELj8EE6insertERKS5_", "llvm::SmallSet<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, 8u>::insert(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES5_St9_IdentityIS5_ESt4lessIS5_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::_Identity<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> > >::_M_insert_unique(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES5_St9_IdentityIS5_ESt4lessIS5_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::_Identity<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >*)"},
    {"_ZN4llvm40initializeCorrelatedValuePropagationPassERNS_12PassRegistryE", "llvm::initializeCorrelatedValuePropagationPass(llvm::PassRegistry&)"},
    {"_ZN4llvm36createCorrelatedValuePropagationPassEv", "llvm::createCorrelatedValuePropagationPass()"},
    {"_ZN12_GLOBAL__N_126CorrelatedValuePropagationD1Ev", "(anonymous namespace)::CorrelatedValuePropagation::~CorrelatedValuePropagation()"},
    {"_ZN12_GLOBAL__N_126CorrelatedValuePropagationD0Ev", "(anonymous namespace)::CorrelatedValuePropagation::~CorrelatedValuePropagation()"},
    {"_ZNK12_GLOBAL__N_126CorrelatedValuePropagation16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::CorrelatedValuePropagation::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_126CorrelatedValuePropagation13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::CorrelatedValuePropagation::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_126CorrelatedValuePropagationEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::CorrelatedValuePropagation>()"},
    {"_ZN4llvm17initializeDSEPassERNS_12PassRegistryE", "llvm::initializeDSEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm30createDeadStoreEliminationPassEv", "llvm::createDeadStoreEliminationPass()"},
    {"_ZN12_GLOBAL__N_13DSED1Ev", "(anonymous namespace)::DSE::~DSE()"},
    {"_ZN12_GLOBAL__N_13DSED0Ev", "(anonymous namespace)::DSE::~DSE()"},
    {"_ZNK12_GLOBAL__N_13DSE16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::DSE::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_13DSE13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::DSE::runOnFunction(llvm::Function&)"},
    {"_Z21DeleteDeadInstructionPN4llvm11InstructionERNS_24MemoryDependenceAnalysisEPNS_11SmallPtrSetIPNS_5ValueELj16EEE", "DeleteDeadInstruction(llvm::Instruction*, llvm::MemoryDependenceAnalysis&, llvm::SmallPtrSet<llvm::Value*, 16u>*)"},
    {"_Z14getLocForWritePN4llvm11InstructionERNS_13AliasAnalysisE", "getLocForWrite(llvm::Instruction*, llvm::AliasAnalysis&)"},
    {"_Z14getPointerSizePN4llvm5ValueERNS_13AliasAnalysisE", "getPointerSize(llvm::Value*, llvm::AliasAnalysis&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_13DSEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::DSE>()"},
    {"_ZN4llvm18createEarlyCSEPassEv", "llvm::createEarlyCSEPass()"},
    {"_ZN4llvm22initializeEarlyCSEPassERNS_12PassRegistryE", "llvm::initializeEarlyCSEPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_18EarlyCSEEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::EarlyCSE>()"},
    {"_ZN12_GLOBAL__N_18EarlyCSED1Ev", "(anonymous namespace)::EarlyCSE::~EarlyCSE()"},
    {"_ZN12_GLOBAL__N_18EarlyCSED0Ev", "(anonymous namespace)::EarlyCSE::~EarlyCSE()"},
    {"_ZNK12_GLOBAL__N_18EarlyCSE16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::EarlyCSE::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_18EarlyCSE13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::EarlyCSE::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_18EarlyCSE11processNodeEPN4llvm15DomTreeNodeBaseINS1_10BasicBlockEEE", "(anonymous namespace)::EarlyCSE::processNode(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_111SimpleValueEPNS_18ScopedHashTableValIS2_PNS_5ValueEEENS_12DenseMapInfoIS2_EENS8_IS7_EEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::SimpleValue, llvm::ScopedHashTableVal<(anonymous namespace)::SimpleValue, llvm::Value*>*, llvm::DenseMapInfo<(anonymous namespace)::SimpleValue>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<(anonymous namespace)::SimpleValue, llvm::Value*>*> >::operator[]((anonymous namespace)::SimpleValue const&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_111SimpleValueEPNS_18ScopedHashTableValIS2_PNS_5ValueEEENS_12DenseMapInfoIS2_EENS8_IS7_EEE15LookupBucketForERKS2_RPSt4pairIS2_S7_E", "llvm::DenseMap<(anonymous namespace)::SimpleValue, llvm::ScopedHashTableVal<(anonymous namespace)::SimpleValue, llvm::Value*>*, llvm::DenseMapInfo<(anonymous namespace)::SimpleValue>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<(anonymous namespace)::SimpleValue, llvm::Value*>*> >::LookupBucketFor((anonymous namespace)::SimpleValue const&, std::pair<(anonymous namespace)::SimpleValue, llvm::ScopedHashTableVal<(anonymous namespace)::SimpleValue, llvm::Value*>*>*&) const"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_19CallValueEPNS_18ScopedHashTableValIS2_St4pairIPNS_5ValueEjEEENS_12DenseMapInfoIS2_EENSA_IS9_EEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::CallValue, llvm::ScopedHashTableVal<(anonymous namespace)::CallValue, std::pair<llvm::Value*, unsigned int> >*, llvm::DenseMapInfo<(anonymous namespace)::CallValue>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<(anonymous namespace)::CallValue, std::pair<llvm::Value*, unsigned int> >*> >::operator[]((anonymous namespace)::CallValue const&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_19CallValueEPNS_18ScopedHashTableValIS2_St4pairIPNS_5ValueEjEEENS_12DenseMapInfoIS2_EENSA_IS9_EEE15LookupBucketForERKS2_RPS4_IS2_S9_E", "llvm::DenseMap<(anonymous namespace)::CallValue, llvm::ScopedHashTableVal<(anonymous namespace)::CallValue, std::pair<llvm::Value*, unsigned int> >*, llvm::DenseMapInfo<(anonymous namespace)::CallValue>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<(anonymous namespace)::CallValue, std::pair<llvm::Value*, unsigned int> >*> >::LookupBucketFor((anonymous namespace)::CallValue const&, std::pair<(anonymous namespace)::CallValue, llvm::ScopedHashTableVal<(anonymous namespace)::CallValue, std::pair<llvm::Value*, unsigned int> >*>*&) const"},
    {"_ZN4llvm15ScopedHashTableIPNS_5ValueESt4pairIS2_jENS_12DenseMapInfoIS2_EENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_S4_EELm40ELm8EEEED2Ev", "llvm::ScopedHashTable<llvm::Value*, std::pair<llvm::Value*, unsigned int>, llvm::DenseMapInfo<llvm::Value*>, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >, 40ul, 8ul> >::~ScopedHashTable()"},
    {"_ZN4llvm15ScopedHashTableIPNS_5ValueESt4pairIS2_jENS_12DenseMapInfoIS2_EENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_S4_EELm40ELm8EEEE6insertERKS2_RKS4_", "llvm::ScopedHashTable<llvm::Value*, std::pair<llvm::Value*, unsigned int>, llvm::DenseMapInfo<llvm::Value*>, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >, 40ul, 8ul> >::insert(llvm::Value* const&, std::pair<llvm::Value*, unsigned int> const&)"},
    {"_ZN4llvm20ScopedHashTableScopeIPNS_5ValueESt4pairIS2_jENS_12DenseMapInfoIS2_EENS_18RecyclingAllocatorINS_16BumpPtrAllocatorENS_18ScopedHashTableValIS2_S4_EELm40ELm8EEEED2Ev", "llvm::ScopedHashTableScope<llvm::Value*, std::pair<llvm::Value*, unsigned int>, llvm::DenseMapInfo<llvm::Value*>, llvm::RecyclingAllocator<llvm::BumpPtrAllocator, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >, 40ul, 8ul> >::~ScopedHashTableScope()"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_18ScopedHashTableValIS2_St4pairIS2_jEEENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PS4_IS2_S7_E", "llvm::DenseMap<llvm::Value*, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >*> >::InsertIntoBucket(llvm::Value* const&, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >* const&, std::pair<llvm::Value*, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >*>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_18ScopedHashTableValIS2_St4pairIS2_jEEENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::Value*, llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::ScopedHashTableVal<llvm::Value*, std::pair<llvm::Value*, unsigned int> >*> >::grow(unsigned int)"},
    {"_ZN4llvm25initializeGEPSplitterPassERNS_12PassRegistryE", "llvm::initializeGEPSplitterPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createGEPSplitterPassEv", "llvm::createGEPSplitterPass()"},
    {"_ZN12_GLOBAL__N_111GEPSplitterD1Ev", "(anonymous namespace)::GEPSplitter::~GEPSplitter()"},
    {"_ZN12_GLOBAL__N_111GEPSplitterD0Ev", "(anonymous namespace)::GEPSplitter::~GEPSplitter()"},
    {"_ZNK12_GLOBAL__N_111GEPSplitter16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::GEPSplitter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111GEPSplitter13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::GEPSplitter::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111GEPSplitterEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::GEPSplitter>()"},
    {"_ZN4llvm13createGVNPassEb", "llvm::createGVNPass(bool)"},
    {"_ZN12_GLOBAL__N_13GVNC1Eb", "(anonymous namespace)::GVN::GVN(bool)"},
    {"_ZN4llvm17initializeGVNPassERNS_12PassRegistryE", "llvm::initializeGVNPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_13GVNEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::GVN>()"},
    {"_ZN12_GLOBAL__N_13GVND1Ev", "(anonymous namespace)::GVN::~GVN()"},
    {"_ZN12_GLOBAL__N_13GVND0Ev", "(anonymous namespace)::GVN::~GVN()"},
    {"_ZNK12_GLOBAL__N_13GVN16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::GVN::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_13GVN13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::GVN::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_13GVN17cleanupGlobalSetsEv", "(anonymous namespace)::GVN::cleanupGlobalSets()"},
    {"_ZN12_GLOBAL__N_13GVN10findLeaderEPN4llvm10BasicBlockEj", "(anonymous namespace)::GVN::findLeader(llvm::BasicBlock*, unsigned int)"},
    {"_ZN4llvm8DenseMapIjN12_GLOBAL__N_13GVN16LeaderTableEntryENS_12DenseMapInfoIjEENS4_IS3_EEEixERKj", "llvm::DenseMap<unsigned int, (anonymous namespace)::GVN::LeaderTableEntry, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<(anonymous namespace)::GVN::LeaderTableEntry> >::operator[](unsigned int const&)"},
    {"_ZN12_GLOBAL__N_13GVN18processInstructionEPN4llvm11InstructionERNS1_15SmallVectorImplIS3_EE", "(anonymous namespace)::GVN::processInstruction(llvm::Instruction*, llvm::SmallVectorImpl<llvm::Instruction*>&)"},
    {"_ZN12_GLOBAL__N_110ValueTable13lookup_or_addEPN4llvm5ValueE", "(anonymous namespace)::ValueTable::lookup_or_add(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_110ValueTable17create_expressionEPN4llvm11InstructionE", "(anonymous namespace)::ValueTable::create_expression(llvm::Instruction*)"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_110ExpressionEjNS_12DenseMapInfoIS2_EENS3_IjEEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::Expression, unsigned int, llvm::DenseMapInfo<(anonymous namespace)::Expression>, llvm::DenseMapInfo<unsigned int> >::operator[]((anonymous namespace)::Expression const&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_110ExpressionEjNS_12DenseMapInfoIS2_EENS3_IjEEE15LookupBucketForERKS2_RPSt4pairIS2_jE", "llvm::DenseMap<(anonymous namespace)::Expression, unsigned int, llvm::DenseMapInfo<(anonymous namespace)::Expression>, llvm::DenseMapInfo<unsigned int> >::LookupBucketFor((anonymous namespace)::Expression const&, std::pair<(anonymous namespace)::Expression, unsigned int>*&) const"},
    {"_Z30AnalyzeLoadFromClobberingStorePKN4llvm4TypeEPNS_5ValueEPNS_9StoreInstERKNS_10DataLayoutE", "AnalyzeLoadFromClobberingStore(llvm::Type const*, llvm::Value*, llvm::StoreInst*, llvm::DataLayout const&)"},
    {"_Z20GetStoreValueForLoadPN4llvm5ValueEjPKNS_4TypeEPNS_11InstructionERKNS_10DataLayoutE", "GetStoreValueForLoad(llvm::Value*, unsigned int, llvm::Type const*, llvm::Instruction*, llvm::DataLayout const&)"},
    {"_Z32AnalyzeLoadFromClobberingMemInstPKN4llvm4TypeEPNS_5ValueEPNS_12MemIntrinsicERKNS_10DataLayoutE", "AnalyzeLoadFromClobberingMemInst(llvm::Type const*, llvm::Value*, llvm::MemIntrinsic*, llvm::DataLayout const&)"},
    {"_Z22GetMemInstValueForLoadPN4llvm12MemIntrinsicEjPKNS_4TypeEPNS_11InstructionERKNS_10DataLayoutE", "GetMemInstValueForLoad(llvm::MemIntrinsic*, unsigned int, llvm::Type const*, llvm::Instruction*, llvm::DataLayout const&)"},
    {"_Z30CoerceAvailableValueToLoadTypePN4llvm5ValueEPKNS_4TypeEPNS_11InstructionERKNS_10DataLayoutE", "CoerceAvailableValueToLoadType(llvm::Value*, llvm::Type const*, llvm::Instruction*, llvm::DataLayout const&)"},
    {"_Z31CanCoerceMustAliasedValueToLoadPN4llvm5ValueEPKNS_4TypeERKNS_10DataLayoutE", "CanCoerceMustAliasedValueToLoad(llvm::Value*, llvm::Type const*, llvm::DataLayout const&)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_121AvailableValueInBlockEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::AvailableValueInBlock>::push_back((anonymous namespace)::AvailableValueInBlock const&)"},
    {"_Z22ConstructSSAForLoadSetPN4llvm8LoadInstERNS_15SmallVectorImplIN12_GLOBAL__N_121AvailableValueInBlockEEEPKNS_10DataLayoutERKNS_13DominatorTreeEPNS_13AliasAnalysisE", "ConstructSSAForLoadSet(llvm::LoadInst*, llvm::SmallVectorImpl<(anonymous namespace)::AvailableValueInBlock>&, llvm::DataLayout const*, llvm::DominatorTree const&, llvm::AliasAnalysis*)"},
    {"_Z28IsValueFullyAvailableInBlockPN4llvm10BasicBlockERNS_8DenseMapIS1_cNS_12DenseMapInfoIS1_EENS3_IcEEEE", "IsValueFullyAvailableInBlock(llvm::BasicBlock*, llvm::DenseMap<llvm::BasicBlock*, char, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<char> >&)"},
    {"_ZNK12_GLOBAL__N_121AvailableValueInBlock24MaterializeAdjustedValueEPKN4llvm4TypeEPKNS1_10DataLayoutE", "(anonymous namespace)::AvailableValueInBlock::MaterializeAdjustedValue(llvm::Type const*, llvm::DataLayout const*) const"},
    {"_Z30AnalyzeLoadFromClobberingWritePKN4llvm4TypeEPNS_5ValueES4_yRKNS_10DataLayoutE", "AnalyzeLoadFromClobberingWrite(llvm::Type const*, llvm::Value*, llvm::Value*, unsigned long long, llvm::DataLayout const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Value*, unsigned int, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm7PHINode11addIncomingEPNS_5ValueEPNS_10BasicBlockE", "llvm::PHINode::addIncoming(llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm11df_iteratorIPNS_10BasicBlockENS_11SmallPtrSetIS2_Lj8EEELb0ENS_11GraphTraitsIS2_EEE6toNextEv", "llvm::df_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_10BasicBlockELj1EjNS1_21PointerLikeTypeTraitsIS4_EEEENS1_12SuccIteratorIPNS1_14TerminatorInstES3_EEESaISC_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPSC_SE_EERKSC_", "std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >*, std::vector<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > > >, std::pair<llvm::PointerIntPair<llvm::BasicBlock*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::BasicBlock*> >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEjNS_12DenseMapInfoIS2_EENS3_IjEEE6insertERKSt4pairIS2_jE", "llvm::DenseMap<llvm::Value*, unsigned int, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<llvm::Value*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::Value*, unsigned int, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Value* const&, unsigned int const&, std::pair<llvm::Value*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::Value*, unsigned int, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::Value*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::Value* const&, std::pair<llvm::BasicBlock*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::Value*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPNS_15DomTreeNodeBaseINS_10BasicBlockEEEN9__gnu_cxx17__normal_iteratorIPS5_St6vectorIS5_SaIS5_EEEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > >, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::Value*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Value*> >::shrink_and_clear()"},
    {"_ZN4llvm11df_iteratorIPNS_15DomTreeNodeBaseINS_10BasicBlockEEENS_11SmallPtrSetIS4_Lj8EEELb0ENS_11GraphTraitsIS4_EEE6toNextEv", "llvm::df_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>*, llvm::SmallPtrSet<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 8u>, false, llvm::GraphTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >::toNext()"},
    {"_ZNSt6vectorISt4pairIN4llvm14PointerIntPairIPNS1_15DomTreeNodeBaseINS1_10BasicBlockEEELj1EjNS1_21PointerLikeTypeTraitsIS6_EEEEN9__gnu_cxx17__normal_iteratorIPS6_S_IS6_SaIS6_EEEEESaISG_EE13_M_insert_auxENSB_IPSG_SI_EERKSG_", "std::vector<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > >*, std::vector<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > >, std::allocator<std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > > > > >, std::pair<llvm::PointerIntPair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, 1u, unsigned int, llvm::PointerLikeTypeTraits<llvm::DomTreeNodeBase<llvm::BasicBlock>*> >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > > const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEcNS_12DenseMapInfoIS2_EENS3_IcEEE6insertERKSt4pairIS2_cE", "llvm::DenseMap<llvm::BasicBlock*, char, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::BasicBlock*, char> const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEcNS_12DenseMapInfoIS2_EENS3_IcEEE16InsertIntoBucketERKS2_RKcPSt4pairIS2_cE", "llvm::DenseMap<llvm::BasicBlock*, char, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::BasicBlock* const&, char const&, std::pair<llvm::BasicBlock*, char>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEcNS_12DenseMapInfoIS2_EENS3_IcEEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, char, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE8CreateOrEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateOr(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateShlEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateShl(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateCastENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCast(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateLShrEPNS_5ValueES6_RKNS_5TwineEb", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateLShr(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm28initializeIndVarSimplifyPassERNS_12PassRegistryE", "llvm::initializeIndVarSimplifyPass(llvm::PassRegistry&)"},
    {"_ZN4llvm24createIndVarSimplifyPassEv", "llvm::createIndVarSimplifyPass()"},
    {"_ZN12_GLOBAL__N_114IndVarSimplifyD1Ev", "(anonymous namespace)::IndVarSimplify::~IndVarSimplify()"},
    {"_ZN12_GLOBAL__N_114IndVarSimplifyD0Ev", "(anonymous namespace)::IndVarSimplify::~IndVarSimplify()"},
    {"_ZNK12_GLOBAL__N_114IndVarSimplify16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::IndVarSimplify::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114IndVarSimplify9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::IndVarSimplify::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_Z6isSafePKN4llvm4SCEVEPKNS_4LoopEPNS_15ScalarEvolutionE", "isSafe(llvm::SCEV const*, llvm::Loop const*, llvm::ScalarEvolution*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114IndVarSimplifyEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::IndVarSimplify>()"},
    {"_ZN4llvm27initializeJumpThreadingPassERNS_12PassRegistryE", "llvm::initializeJumpThreadingPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createJumpThreadingPassEv", "llvm::createJumpThreadingPass()"},
    {"_ZN12_GLOBAL__N_113JumpThreadingC1Ev", "(anonymous namespace)::JumpThreading::JumpThreading()"},
    {"_ZN12_GLOBAL__N_113JumpThreadingD1Ev", "(anonymous namespace)::JumpThreading::~JumpThreading()"},
    {"_ZN12_GLOBAL__N_113JumpThreadingD0Ev", "(anonymous namespace)::JumpThreading::~JumpThreading()"},
    {"_ZNK12_GLOBAL__N_113JumpThreading16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::JumpThreading::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_113JumpThreading13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::JumpThreading::runOnFunction(llvm::Function&)"},
    {"_Z25GetBestDestForJumpOnUndefPN4llvm10BasicBlockE", "GetBestDestForJumpOnUndef(llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_113JumpThreading22ProcessThreadableEdgesEPN4llvm5ValueEPNS1_10BasicBlockENS_18ConstantPreferenceE", "(anonymous namespace)::JumpThreading::ProcessThreadableEdges(llvm::Value*, llvm::BasicBlock*, (anonymous namespace)::ConstantPreference)"},
    {"_ZN12_GLOBAL__N_113JumpThreading31ComputeValueKnownInPredecessorsEPN4llvm5ValueEPNS1_10BasicBlockERNS1_15SmallVectorImplISt4pairIPNS1_8ConstantES5_EEENS_18ConstantPreferenceE", "(anonymous namespace)::JumpThreading::ComputeValueKnownInPredecessors(llvm::Value*, llvm::BasicBlock*, llvm::SmallVectorImpl<std::pair<llvm::Constant*, llvm::BasicBlock*> >&, (anonymous namespace)::ConstantPreference)"},
    {"_ZN12_GLOBAL__N_113JumpThreading32DuplicateCondBranchOnPHIIntoPredEPN4llvm10BasicBlockERKNS1_15SmallVectorImplIS3_EE", "(anonymous namespace)::JumpThreading::DuplicateCondBranchOnPHIIntoPred(llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::BasicBlock*> const&)"},
    {"_Z28getJumpThreadDuplicationCostPKN4llvm10BasicBlockE", "getJumpThreadDuplicationCost(llvm::BasicBlock const*)"},
    {"_Z31AddPHINodeEntriesForMappedBlockPN4llvm10BasicBlockES1_S1_RNS_8DenseMapIPNS_11InstructionEPNS_5ValueENS_12DenseMapInfoIS4_EENS7_IS6_EEEE", "AddPHINodeEntriesForMappedBlock(llvm::BasicBlock*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::DenseMap<llvm::Instruction*, llvm::Value*, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::Value*> >&)"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIPNS_10BasicBlockEPNS_5ValueEEEEiPKvS8_", "int llvm::array_pod_sort_comparator<std::pair<llvm::BasicBlock*, llvm::Value*> >(void const*, void const*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113JumpThreadingEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::JumpThreading>()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Instruction*, llvm::Value*, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::Value*> >::InsertIntoBucket(llvm::Instruction* const&, llvm::Value* const&, std::pair<llvm::Instruction*, llvm::Value*>*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, llvm::Value*, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::Value*> >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_5ValueEPNS_10BasicBlockEEcNS_12DenseMapInfoIS6_EENS7_IcEEE15LookupBucketForERKS6_RPS1_IS6_cE", "llvm::DenseMap<std::pair<llvm::Value*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::LookupBucketFor(std::pair<llvm::Value*, llvm::BasicBlock*> const&, std::pair<std::pair<llvm::Value*, llvm::BasicBlock*>, char>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_5ValueEPNS_10BasicBlockEEcNS_12DenseMapInfoIS6_EENS7_IcEEE6insertERKS1_IS6_cE", "llvm::DenseMap<std::pair<llvm::Value*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<llvm::Value*, llvm::BasicBlock*>, char> const&)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_5ValueEPNS_10BasicBlockEEcNS_12DenseMapInfoIS6_EENS7_IcEEE16InsertIntoBucketERKS6_RKcPS1_IS6_cE", "llvm::DenseMap<std::pair<llvm::Value*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<llvm::Value*, llvm::BasicBlock*> const&, char const&, std::pair<std::pair<llvm::Value*, llvm::BasicBlock*>, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_5ValueEPNS_10BasicBlockEEcNS_12DenseMapInfoIS6_EENS7_IcEEE4growEj", "llvm::DenseMap<std::pair<llvm::Value*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::Value*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPNS_10BasicBlockEE6resizeEj", "llvm::SmallVectorImpl<llvm::BasicBlock*>::resize(unsigned int)"},
    {"_ZSt6__findIPPN4llvm10BasicBlockES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "llvm::BasicBlock** std::__find<llvm::BasicBlock**, llvm::BasicBlock*>(llvm::BasicBlock**, llvm::BasicBlock**, llvm::BasicBlock* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm18initializeLICMPassERNS_12PassRegistryE", "llvm::initializeLICMPass(llvm::PassRegistry&)"},
    {"_ZN4llvm14createLICMPassEv", "llvm::createLICMPass()"},
    {"_ZN12_GLOBAL__N_14LICMD1Ev", "(anonymous namespace)::LICM::~LICM()"},
    {"_ZN12_GLOBAL__N_14LICMD0Ev", "(anonymous namespace)::LICM::~LICM()"},
    {"_ZNK12_GLOBAL__N_14LICM16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LICM::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14LICM9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LICM::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN12_GLOBAL__N_14LICM14doFinalizationEv", "(anonymous namespace)::LICM::doFinalization()"},
    {"_ZN12_GLOBAL__N_14LICM23cloneBasicBlockAnalysisEPN4llvm10BasicBlockES3_PNS1_4LoopE", "(anonymous namespace)::LICM::cloneBasicBlockAnalysis(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Loop*)"},
    {"_ZN12_GLOBAL__N_14LICM19deleteAnalysisValueEPN4llvm5ValueEPNS1_4LoopE", "(anonymous namespace)::LICM::deleteAnalysisValue(llvm::Value*, llvm::Loop*)"},
    {"_ZN12_GLOBAL__N_14LICM10SinkRegionEPN4llvm15DomTreeNodeBaseINS1_10BasicBlockEEE", "(anonymous namespace)::LICM::SinkRegion(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN12_GLOBAL__N_14LICM11HoistRegionEPN4llvm15DomTreeNodeBaseINS1_10BasicBlockEEE", "(anonymous namespace)::LICM::HoistRegion(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN12_GLOBAL__N_14LICM30isSafeToExecuteUnconditionallyERN4llvm11InstructionE", "(anonymous namespace)::LICM::isSafeToExecuteUnconditionally(llvm::Instruction&)"},
    {"_ZN12_GLOBAL__N_112LoopPromoterD1Ev", "(anonymous namespace)::LoopPromoter::~LoopPromoter()"},
    {"_ZN12_GLOBAL__N_112LoopPromoterD0Ev", "(anonymous namespace)::LoopPromoter::~LoopPromoter()"},
    {"_ZNK12_GLOBAL__N_112LoopPromoter12isInstInListEPN4llvm11InstructionERKNS1_15SmallVectorImplIS3_EE", "(anonymous namespace)::LoopPromoter::isInstInList(llvm::Instruction*, llvm::SmallVectorImpl<llvm::Instruction*> const&) const"},
    {"_ZNK12_GLOBAL__N_112LoopPromoter34doExtraRewritesBeforeFinalDeletionEv", "(anonymous namespace)::LoopPromoter::doExtraRewritesBeforeFinalDeletion() const"},
    {"_ZNK12_GLOBAL__N_112LoopPromoter20replaceLoadWithValueEPN4llvm8LoadInstEPNS1_5ValueE", "(anonymous namespace)::LoopPromoter::replaceLoadWithValue(llvm::LoadInst*, llvm::Value*) const"},
    {"_ZNK12_GLOBAL__N_112LoopPromoter18instructionDeletedEPN4llvm11InstructionE", "(anonymous namespace)::LoopPromoter::instructionDeleted(llvm::Instruction*) const"},
    {"_ZN12_GLOBAL__N_14LICM18canSinkOrHoistInstERN4llvm11InstructionE", "(anonymous namespace)::LICM::canSinkOrHoistInst(llvm::Instruction&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_14LICMEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LICM>()"},
    {"_ZN4llvm6iplistINS_8AliasSetENS_12ilist_traitsIS1_EEE3endEv", "llvm::iplist<llvm::AliasSet, llvm::ilist_traits<llvm::AliasSet> >::end()"},
    {"_ZN4llvm6iplistINS_8AliasSetENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::AliasSet, llvm::ilist_traits<llvm::AliasSet> >::~iplist()"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEED2Ev", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::~DenseMap()"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVHD1Ev", "llvm::AliasSetTracker::ASTCallbackVH::~ASTCallbackVH()"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVHD0Ev", "llvm::AliasSetTracker::ASTCallbackVH::~ASTCallbackVH()"},
    {"_ZN4llvm8DenseMapIPNS_4LoopEPNS_15AliasSetTrackerENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Loop*, llvm::AliasSetTracker*, llvm::DenseMapInfo<llvm::Loop*>, llvm::DenseMapInfo<llvm::AliasSetTracker*> >::InsertIntoBucket(llvm::Loop* const&, llvm::AliasSetTracker* const&, std::pair<llvm::Loop*, llvm::AliasSetTracker*>*)"},
    {"_ZN4llvm8DenseMapIPNS_4LoopEPNS_15AliasSetTrackerENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Loop*, llvm::AliasSetTracker*, llvm::DenseMapInfo<llvm::Loop*>, llvm::DenseMapInfo<llvm::AliasSetTracker*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE4initEj", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::init(unsigned int)"},
    {"_ZN4llvm26initializeLoopDeletionPassERNS_12PassRegistryE", "llvm::initializeLoopDeletionPass(llvm::PassRegistry&)"},
    {"_ZN4llvm22createLoopDeletionPassEv", "llvm::createLoopDeletionPass()"},
    {"_ZN12_GLOBAL__N_112LoopDeletionD1Ev", "(anonymous namespace)::LoopDeletion::~LoopDeletion()"},
    {"_ZN12_GLOBAL__N_112LoopDeletionD0Ev", "(anonymous namespace)::LoopDeletion::~LoopDeletion()"},
    {"_ZNK12_GLOBAL__N_112LoopDeletion16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopDeletion::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_112LoopDeletion9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopDeletion::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112LoopDeletionEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopDeletion>()"},
    {"_ZN4llvm15SmallVectorImplIPNS_15DomTreeNodeBaseINS_10BasicBlockEEEE6insertIN9__gnu_cxx17__normal_iteratorIPS4_St6vectorIS4_SaIS4_EEEEEES9_S9_T_SE_", "llvm::DomTreeNodeBase<llvm::BasicBlock>** llvm::SmallVectorImpl<llvm::DomTreeNodeBase<llvm::BasicBlock>*>::insert<__gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > > >(llvm::DomTreeNodeBase<llvm::BasicBlock>**, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >, __gnu_cxx::__normal_iterator<llvm::DomTreeNodeBase<llvm::BasicBlock>**, std::vector<llvm::DomTreeNodeBase<llvm::BasicBlock>*, std::allocator<llvm::DomTreeNodeBase<llvm::BasicBlock>*> > >)"},
    {"_ZN4llvm32initializeLoopIdiomRecognizePassERNS_12PassRegistryE", "llvm::initializeLoopIdiomRecognizePass(llvm::PassRegistry&)"},
    {"_ZN4llvm19createLoopIdiomPassEv", "llvm::createLoopIdiomPass()"},
    {"_ZN12_GLOBAL__N_118LoopIdiomRecognizeD1Ev", "(anonymous namespace)::LoopIdiomRecognize::~LoopIdiomRecognize()"},
    {"_ZN12_GLOBAL__N_118LoopIdiomRecognizeD0Ev", "(anonymous namespace)::LoopIdiomRecognize::~LoopIdiomRecognize()"},
    {"_ZNK12_GLOBAL__N_118LoopIdiomRecognize16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopIdiomRecognize::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118LoopIdiomRecognize9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopIdiomRecognize::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN12_GLOBAL__N_118LoopIdiomRecognize28processLoopStoreOfSplatValueEPN4llvm5ValueEjjS3_PNS1_11InstructionEPKNS1_14SCEVAddRecExprEPKNS1_4SCEVE", "(anonymous namespace)::LoopIdiomRecognize::processLoopStoreOfSplatValue(llvm::Value*, unsigned int, unsigned int, llvm::Value*, llvm::Instruction*, llvm::SCEVAddRecExpr const*, llvm::SCEV const*)"},
    {"_Z21mayLoopAccessLocationPN4llvm5ValueENS_13AliasAnalysis12ModRefResultEPNS_4LoopEPKNS_4SCEVEjRS2_PNS_11InstructionE", "mayLoopAccessLocation(llvm::Value*, llvm::AliasAnalysis::ModRefResult, llvm::Loop*, llvm::SCEV const*, unsigned int, llvm::AliasAnalysis&, llvm::Instruction*)"},
    {"_Z21DeleteDeadInstructionPN4llvm11InstructionERNS_15ScalarEvolutionE", "DeleteDeadInstruction(llvm::Instruction*, llvm::ScalarEvolution&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_118LoopIdiomRecognizeEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopIdiomRecognize>()"},
    {"_ZN4llvm24initializeLoopRotatePassERNS_12PassRegistryE", "llvm::initializeLoopRotatePass(llvm::PassRegistry&)"},
    {"_ZN4llvm20createLoopRotatePassEv", "llvm::createLoopRotatePass()"},
    {"_ZN12_GLOBAL__N_110LoopRotateD1Ev", "(anonymous namespace)::LoopRotate::~LoopRotate()"},
    {"_ZN12_GLOBAL__N_110LoopRotateD0Ev", "(anonymous namespace)::LoopRotate::~LoopRotate()"},
    {"_ZNK12_GLOBAL__N_110LoopRotate16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopRotate::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_110LoopRotate9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopRotate::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_110LoopRotateEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopRotate>()"},
    {"_ZN4llvm8ValueMapIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEEixERKS3_", "llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::operator[](llvm::Value const* const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_ED2Ev", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::~DenseMap()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEED1Ev", "llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::~ValueMapCallbackVH()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEED0Ev", "llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::~ValueMapCallbackVH()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE7deletedEv", "llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::deleted()"},
    {"_ZN4llvm18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE19allUsesReplacedWithEPS1_", "llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4findERKSB_", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::find(llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5eraseENS_16DenseMapIteratorISB_S6_SC_SA_Lb0EEE", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::erase(llvm::DenseMapIterator<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> >, false>)"},
    {"_ZN4llvm8ValueMapIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEE6insertERKSt4pairIS3_S5_E", "llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::insert(std::pair<llvm::Value const*, llvm::TrackingVH<llvm::Value> > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E6insertERKSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::insert(std::pair<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value> > const&)"},
    {"_ZNK4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E15LookupBucketForERKSB_RPSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::LookupBucketFor(llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > const&, std::pair<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value> >*&) const"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E16InsertIntoBucketERKSB_RKS6_PSt4pairISB_S6_E", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::InsertIntoBucket(llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > const&, llvm::TrackingVH<llvm::Value> const&, std::pair<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value> >*)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4growEj", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5eraseERKSB_", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::erase(llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > const&)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E4initEj", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::init(unsigned int)"},
    {"_ZN4llvm32initializeLoopStrengthReducePassERNS_12PassRegistryE", "llvm::initializeLoopStrengthReducePass(llvm::PassRegistry&)"},
    {"_ZN4llvm28createLoopStrengthReducePassEPKNS_14TargetLoweringE", "llvm::createLoopStrengthReducePass(llvm::TargetLowering const*)"},
    {"_ZN12_GLOBAL__N_118LoopStrengthReduceD1Ev", "(anonymous namespace)::LoopStrengthReduce::~LoopStrengthReduce()"},
    {"_ZN12_GLOBAL__N_118LoopStrengthReduceD0Ev", "(anonymous namespace)::LoopStrengthReduce::~LoopStrengthReduce()"},
    {"_ZNK12_GLOBAL__N_118LoopStrengthReduce16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopStrengthReduce::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118LoopStrengthReduce9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopStrengthReduce::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN12_GLOBAL__N_16LSRUseD1Ev", "(anonymous namespace)::LSRUse::~LSRUse()"},
    {"_ZN12_GLOBAL__N_111LSRInstanceC2EPKN4llvm14TargetLoweringEPNS1_4LoopEPNS1_4PassE", "(anonymous namespace)::LSRInstance::LSRInstance(llvm::TargetLowering const*, llvm::Loop*, llvm::Pass*)"},
    {"_ZN12_GLOBAL__N_111LSRInstance24GenerateAllReuseFormulaeEv", "(anonymous namespace)::LSRInstance::GenerateAllReuseFormulae()"},
    {"_ZN12_GLOBAL__N_111LSRInstance38FilterOutUndesirableDedicatedRegistersEv", "(anonymous namespace)::LSRInstance::FilterOutUndesirableDedicatedRegisters()"},
    {"_ZNK12_GLOBAL__N_111LSRInstance6ExpandERKNS_8LSRFixupERKNS_7FormulaEN4llvm14ilist_iteratorINS7_11InstructionEEERNS7_12SCEVExpanderERNS7_15SmallVectorImplINS7_6WeakVHEEE", "(anonymous namespace)::LSRInstance::Expand((anonymous namespace)::LSRFixup const&, (anonymous namespace)::Formula const&, llvm::ilist_iterator<llvm::Instruction>, llvm::SCEVExpander&, llvm::SmallVectorImpl<llvm::WeakVH>&) const"},
    {"_ZNK12_GLOBAL__N_18LSRFixup21isUseFullyOutsideLoopEPKN4llvm4LoopE", "(anonymous namespace)::LSRFixup::isUseFullyOutsideLoop(llvm::Loop const*) const"},
    {"_ZNK12_GLOBAL__N_111LSRInstance12SolveRecurseERN4llvm15SmallVectorImplIPKNS_7FormulaEEERNS_4CostES7_RKS8_RKNS1_11SmallPtrSetIPKNS1_4SCEVELj16EEERNS1_8DenseSetISF_NS1_12DenseMapInfoISF_EEEE", "(anonymous namespace)::LSRInstance::SolveRecurse(llvm::SmallVectorImpl<(anonymous namespace)::Formula const*>&, (anonymous namespace)::Cost&, llvm::SmallVectorImpl<(anonymous namespace)::Formula const*>&, (anonymous namespace)::Cost const&, llvm::SmallPtrSet<llvm::SCEV const*, 16u> const&, llvm::DenseSet<llvm::SCEV const*, llvm::DenseMapInfo<llvm::SCEV const*> >&) const"},
    {"_ZN12_GLOBAL__N_14Cost11RateFormulaERKNS_7FormulaERN4llvm11SmallPtrSetIPKNS4_4SCEVELj16EEERKNS4_8DenseSetIS8_NS4_12DenseMapInfoIS8_EEEEPKNS4_4LoopERKNS4_15SmallVectorImplIxEERNS4_15ScalarEvolutionERNS4_13DominatorTreeE", "(anonymous namespace)::Cost::RateFormula((anonymous namespace)::Formula const&, llvm::SmallPtrSet<llvm::SCEV const*, 16u>&, llvm::DenseSet<llvm::SCEV const*, llvm::DenseMapInfo<llvm::SCEV const*> > const&, llvm::Loop const*, llvm::SmallVectorImpl<long long> const&, llvm::ScalarEvolution&, llvm::DominatorTree&)"},
    {"_ZN12_GLOBAL__N_14Cost12RateRegisterEPKN4llvm4SCEVERNS1_11SmallPtrSetIS4_Lj16EEEPKNS1_4LoopERNS1_15ScalarEvolutionERNS1_13DominatorTreeE", "(anonymous namespace)::Cost::RateRegister(llvm::SCEV const*, llvm::SmallPtrSet<llvm::SCEV const*, 16u>&, llvm::Loop const*, llvm::ScalarEvolution&, llvm::DominatorTree&)"},
    {"_ZN12_GLOBAL__N_16LSRUse13RecomputeRegsEmRNS_13RegUseTrackerE", "(anonymous namespace)::LSRUse::RecomputeRegs(unsigned long, (anonymous namespace)::RegUseTracker&)"},
    {"_ZN12_GLOBAL__N_111LSRInstance18reconcileNewOffsetERNS_6LSRUseExbNS1_8KindTypeEPKN4llvm4TypeE", "(anonymous namespace)::LSRInstance::reconcileNewOffset((anonymous namespace)::LSRUse&, long long, bool, (anonymous namespace)::LSRUse::KindType, llvm::Type const*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_17FormulaEEaSERKS3_", "llvm::SmallVectorImpl<(anonymous namespace)::Formula>::operator=(llvm::SmallVectorImpl<(anonymous namespace)::Formula> const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseIN12_GLOBAL__N_17FormulaELb0EE4growEm", "llvm::SmallVectorTemplateBase<(anonymous namespace)::Formula, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapINS_11SmallVectorIPKNS_4SCEVELj2EEEcN12_GLOBAL__N_122UniquifierDenseMapInfoENS_12DenseMapInfoIcEEE8CopyFromERKSA_", "llvm::DenseMap<llvm::SmallVector<llvm::SCEV const*, 2u>, char, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<char> >::CopyFrom(llvm::DenseMap<llvm::SmallVector<llvm::SCEV const*, 2u>, char, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<char> > const&)"},
    {"_Z10isLegalUseRKN4llvm14TargetLowering8AddrModeEN12_GLOBAL__N_16LSRUse8KindTypeEPKNS_4TypeEPKS0_", "isLegalUse(llvm::TargetLowering::AddrMode const&, (anonymous namespace)::LSRUse::KindType, llvm::Type const*, llvm::TargetLowering const*)"},
    {"_ZNK12_GLOBAL__N_16LSRUse22HasFormulaWithSameRegsERKNS_7FormulaE", "(anonymous namespace)::LSRUse::HasFormulaWithSameRegs((anonymous namespace)::Formula const&) const"},
    {"_ZNK4llvm8DenseMapINS_11SmallVectorIPKNS_4SCEVELj2EEEcN12_GLOBAL__N_122UniquifierDenseMapInfoENS_12DenseMapInfoIcEEE15LookupBucketForERKS5_RPSt4pairIS5_cE", "llvm::DenseMap<llvm::SmallVector<llvm::SCEV const*, 2u>, char, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<char> >::LookupBucketFor(llvm::SmallVector<llvm::SCEV const*, 2u> const&, std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, char>*&) const"},
    {"_ZNK12_GLOBAL__N_113RegUseTracker24isRegUsedByUsesOtherThanEPKN4llvm4SCEVEm", "(anonymous namespace)::RegUseTracker::isRegUsedByUsesOtherThan(llvm::SCEV const*, unsigned long) const"},
    {"_ZNK4llvm8DenseMapINS_11SmallVectorIPKNS_4SCEVELj2EEEmN12_GLOBAL__N_122UniquifierDenseMapInfoENS_12DenseMapInfoImEEE15LookupBucketForERKS5_RPSt4pairIS5_mE", "llvm::DenseMap<llvm::SmallVector<llvm::SCEV const*, 2u>, unsigned long, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<unsigned long> >::LookupBucketFor(llvm::SmallVector<llvm::SCEV const*, 2u> const&, std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, unsigned long>*&) const"},
    {"_ZN4llvm16DenseMapIteratorINS_11SmallVectorIPKNS_4SCEVELj2EEEmN12_GLOBAL__N_122UniquifierDenseMapInfoENS_12DenseMapInfoImEELb0EEC1EPSt4pairIS5_mESD_", "llvm::DenseMapIterator<llvm::SmallVector<llvm::SCEV const*, 2u>, unsigned long, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<unsigned long>, false>::DenseMapIterator(std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, unsigned long>*, std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, unsigned long>*)"},
    {"_ZN12_GLOBAL__N_111LSRInstance22GenerateReassociationsERNS_6LSRUseEjNS_7FormulaEj", "(anonymous namespace)::LSRInstance::GenerateReassociations((anonymous namespace)::LSRUse&, unsigned int, (anonymous namespace)::Formula, unsigned int)"},
    {"_Z16ExtractImmediateRPKN4llvm4SCEVERNS_15ScalarEvolutionE", "ExtractImmediate(llvm::SCEV const*&, llvm::ScalarEvolution&)"},
    {"_ZN12_GLOBAL__N_111LSRInstance13InsertFormulaERNS_6LSRUseEjRKNS_7FormulaE", "(anonymous namespace)::LSRInstance::InsertFormula((anonymous namespace)::LSRUse&, unsigned int, (anonymous namespace)::Formula const&)"},
    {"_ZN12_GLOBAL__N_113RegUseTracker13CountRegisterEPKN4llvm4SCEVEm", "(anonymous namespace)::RegUseTracker::CountRegister(llvm::SCEV const*, unsigned long)"},
    {"_ZN4llvm16DenseMapIteratorINS_11SmallVectorIPKNS_4SCEVELj2EEEcN12_GLOBAL__N_122UniquifierDenseMapInfoENS_12DenseMapInfoIcEELb0EEC1EPSt4pairIS5_cESD_", "llvm::DenseMapIterator<llvm::SmallVector<llvm::SCEV const*, 2u>, char, (anonymous namespace)::UniquifierDenseMapInfo, llvm::DenseMapInfo<char>, false>::DenseMapIterator(std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, char>*, std::pair<llvm::SmallVector<llvm::SCEV const*, 2u>, char>*)"},
    {"_Z12getExactSDivPKN4llvm4SCEVES2_RNS_15ScalarEvolutionEb", "getExactSDiv(llvm::SCEV const*, llvm::SCEV const*, llvm::ScalarEvolution&, bool)"},
    {"_Z13ExtractSymbolRPKN4llvm4SCEVERNS_15ScalarEvolutionE", "ExtractSymbol(llvm::SCEV const*&, llvm::ScalarEvolution&)"},
    {"_Z15CollectSubexprsPKN4llvm4SCEVEPKNS_12SCEVConstantERNS_15SmallVectorImplIS2_EEPKNS_4LoopERNS_15ScalarEvolutionE", "CollectSubexprs(llvm::SCEV const*, llvm::SCEVConstant const*, llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::Loop const*, llvm::ScalarEvolution&)"},
    {"_Z16isAlwaysFoldablePKN4llvm4SCEVExxbN12_GLOBAL__N_16LSRUse8KindTypeEPKNS_4TypeEPKNS_14TargetLoweringERNS_15ScalarEvolutionE", "isAlwaysFoldable(llvm::SCEV const*, long long, long long, bool, (anonymous namespace)::LSRUse::KindType, llvm::Type const*, llvm::TargetLowering const*, llvm::ScalarEvolution&)"},
    {"_ZN12_GLOBAL__N_111LSRInstance11getNewFixupEv", "(anonymous namespace)::LSRInstance::getNewFixup()"},
    {"_ZN12_GLOBAL__N_111LSRInstance6getUseERPKN4llvm4SCEVENS_6LSRUse8KindTypeEPKNS1_4TypeE", "(anonymous namespace)::LSRInstance::getUse(llvm::SCEV const*&, (anonymous namespace)::LSRUse::KindType, llvm::Type const*)"},
    {"_Z13getAccessTypePKN4llvm11InstructionE", "getAccessType(llvm::Instruction const*)"},
    {"_Z14DoInitialMatchPKN4llvm4SCEVEPNS_4LoopERNS_15SmallVectorImplIS2_EES7_RNS_15ScalarEvolutionE", "DoInitialMatch(llvm::SCEV const*, llvm::Loop*, llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::ScalarEvolution&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_118LoopStrengthReduceEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopStrengthReduce>()"},
    {"_ZNK4llvm8LoopPass27getPotentialPassManagerTypeEv", "llvm::LoopPass::getPotentialPassManagerType() const"},
    {"_ZN4llvm8LoopPass16doInitializationEPNS_4LoopERNS_13LPPassManagerE", "llvm::LoopPass::doInitialization(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN4llvm8LoopPass14doFinalizationEv", "llvm::LoopPass::doFinalization()"},
    {"_ZN4llvm8LoopPass23cloneBasicBlockAnalysisEPNS_10BasicBlockES2_PNS_4LoopE", "llvm::LoopPass::cloneBasicBlockAnalysis(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Loop*)"},
    {"_ZN4llvm8LoopPass19deleteAnalysisValueEPNS_5ValueEPNS_4LoopE", "llvm::LoopPass::deleteAnalysisValue(llvm::Value*, llvm::Loop*)"},
    {"_ZNSt8_Rb_treeIxxSt9_IdentityIxESt4lessIxESaIxEE8_M_eraseEPSt13_Rb_tree_nodeIxE", "std::_Rb_tree<long long, long long, std::_Identity<long long>, std::less<long long>, std::allocator<long long> >::_M_erase(std::_Rb_tree_node<long long>*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4SCEVEPNS1_11InstructionEES0_IKS7_NS1_11AssertingVHINS1_5ValueEEEESt10_Select1stISC_ESt4lessIS7_ESaISC_EE8_M_eraseEPSt13_Rb_tree_nodeISC_E", "std::_Rb_tree<std::pair<llvm::SCEV const*, llvm::Instruction*>, std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> >, std::_Select1st<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > >, std::less<std::pair<llvm::SCEV const*, llvm::Instruction*> >, std::allocator<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > >*)"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_5ValueEEES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE8_M_eraseEPSt13_Rb_tree_nodeIS3_E", "std::_Rb_tree<llvm::AssertingVH<llvm::Value>, llvm::AssertingVH<llvm::Value>, std::_Identity<llvm::AssertingVH<llvm::Value> >, std::less<llvm::AssertingVH<llvm::Value> >, std::allocator<llvm::AssertingVH<llvm::Value> > >::_M_erase(std::_Rb_tree_node<llvm::AssertingVH<llvm::Value> >*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_6WeakVHELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::WeakVH, false>::grow(unsigned long)"},
    {"_ZN4llvm15ScalarEvolution10getMulExprEPKNS_4SCEVES3_bb", "llvm::ScalarEvolution::getMulExpr(llvm::SCEV const*, llvm::SCEV const*, bool, bool)"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE12getLoopLatchEv", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::getLoopLatch() const"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE16getExitingBlocksERNS_15SmallVectorImplIPS1_EE", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::getExitingBlocks(llvm::SmallVectorImpl<llvm::BasicBlock*>&) const"},
    {"_ZSt16__introsort_loopIPPN4llvm10BasicBlockElEvT_S4_T0_", "void std::__introsort_loop<llvm::BasicBlock**, long>(llvm::BasicBlock**, llvm::BasicBlock**, long)"},
    {"_ZSt22__final_insertion_sortIPPN4llvm10BasicBlockEEvT_S4_", "void std::__final_insertion_sort<llvm::BasicBlock**>(llvm::BasicBlock**, llvm::BasicBlock**)"},
    {"_ZSt13__heap_selectIPPN4llvm10BasicBlockEEvT_S4_S4_", "void std::__heap_select<llvm::BasicBlock**>(llvm::BasicBlock**, llvm::BasicBlock**, llvm::BasicBlock**)"},
    {"_ZSt9sort_heapIPPN4llvm10BasicBlockEEvT_S4_", "void std::sort_heap<llvm::BasicBlock**>(llvm::BasicBlock**, llvm::BasicBlock**)"},
    {"_ZSt9make_heapIPPN4llvm10BasicBlockEEvT_S4_", "void std::make_heap<llvm::BasicBlock**>(llvm::BasicBlock**, llvm::BasicBlock**)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPKPN4llvm10BasicBlockESt6vectorIS4_SaIS4_EEEEPKS3_ET_SD_SD_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock const*>(__gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock const* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_5ValueENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::Value*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Value*> >::insert(std::pair<llvm::BasicBlock*, llvm::Value*> const&)"},
    {"_ZN4llvm12SCEVExpanderC2ERNS_15ScalarEvolutionE", "llvm::SCEVExpander::SCEVExpander(llvm::ScalarEvolution&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<llvm::SCEV const*, char, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::SCEV const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<llvm::SCEV const*, char, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::SCEV const* const&, char const&, std::pair<llvm::SCEV const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<llvm::SCEV const*, char, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZSt6__findIPKPKN4llvm4SCEVES3_ET_S6_S6_RKT0_St26random_access_iterator_tag", "llvm::SCEV const* const* std::__find<llvm::SCEV const* const*, llvm::SCEV const*>(llvm::SCEV const* const*, llvm::SCEV const* const*, llvm::SCEV const* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm15SmallVectorImplIPKNS_4SCEVEEaSERKS4_", "llvm::SmallVectorImpl<llvm::SCEV const*>::operator=(llvm::SmallVectorImpl<llvm::SCEV const*> const&)"},
    {"_ZN4llvm14SmallBitVector9referenceaSEb", "llvm::SmallBitVector::reference::operator=(bool)"},
    {"_ZN4llvm14SmallBitVector6resizeEjb", "llvm::SmallBitVector::resize(unsigned int, bool)"},
    {"_ZN4llvm15SmallVectorImplIxEaSERKS1_", "llvm::SmallVectorImpl<long long>::operator=(llvm::SmallVectorImpl<long long> const&)"},
    {"_ZSt16__introsort_loopIPPKN4llvm4SCEVElEvT_S5_T0_", "void std::__introsort_loop<llvm::SCEV const**, long>(llvm::SCEV const**, llvm::SCEV const**, long)"},
    {"_ZSt22__final_insertion_sortIPPKN4llvm4SCEVEEvT_S5_", "void std::__final_insertion_sort<llvm::SCEV const**>(llvm::SCEV const**, llvm::SCEV const**)"},
    {"_ZSt13__heap_selectIPPKN4llvm4SCEVEEvT_S5_S5_", "void std::__heap_select<llvm::SCEV const**>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**)"},
    {"_ZSt9sort_heapIPPKN4llvm4SCEVEEvT_S5_", "void std::sort_heap<llvm::SCEV const**>(llvm::SCEV const**, llvm::SCEV const**)"},
    {"_ZSt9make_heapIPPKN4llvm4SCEVEEvT_S5_", "void std::make_heap<llvm::SCEV const**>(llvm::SCEV const**, llvm::SCEV const**)"},
    {"_ZNK4llvm14SmallBitVector9find_nextEj", "llvm::SmallBitVector::find_next(unsigned int) const"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVESt3mapIxS3_St4lessIxESaISt4pairIKxS3_EEENS_12DenseMapInfoIS3_EENSC_ISB_EEE6insertERKS7_IS3_SB_E", "llvm::DenseMap<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > >::insert(std::pair<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > const&)"},
    {"_ZN4llvm14SmallBitVectoroRERKS0_", "llvm::SmallBitVector::operator|=(llvm::SmallBitVector const&)"},
    {"_ZN4llvm8SmallSetISt4pairImxELj32EE6insertERKS2_", "llvm::SmallSet<std::pair<unsigned long, long long>, 32u>::insert(std::pair<unsigned long, long long> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVESt3mapIxS3_St4lessIxESaISt4pairIKxS3_EEENS_12DenseMapInfoIS3_EENSC_ISB_EEE5clearEv", "llvm::DenseMap<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > >::clear()"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_14SmallBitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE5clearEv", "llvm::DenseMap<llvm::SCEV const*, llvm::SmallBitVector, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::SmallBitVector> >::clear()"},
    {"_ZN4llvm15ScalarEvolution10getAddExprEPKNS_4SCEVES3_bb", "llvm::ScalarEvolution::getAddExpr(llvm::SCEV const*, llvm::SCEV const*, bool, bool)"},
    {"_ZNK4llvm5APInt3uleEy", "llvm::APInt::ule(unsigned long long) const"},
    {"_ZNSt8_Rb_treeISt4pairImxES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE8_M_eraseEPSt13_Rb_tree_nodeIS1_E", "std::_Rb_tree<std::pair<unsigned long, long long>, std::pair<unsigned long, long long>, std::_Identity<std::pair<unsigned long, long long> >, std::less<std::pair<unsigned long, long long> >, std::allocator<std::pair<unsigned long, long long> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned long, long long> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_14SmallBitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::SCEV const*, llvm::SmallBitVector, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::SmallBitVector> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVESt3mapIxS3_St4lessIxESaISt4pairIKxS3_EEENS_12DenseMapInfoIS3_EENSC_ISB_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > >::shrink_and_clear()"},
    {"_ZNSt8_Rb_treeISt4pairImxES1_St9_IdentityIS1_ESt4lessIS1_ESaIS1_EE16_M_insert_uniqueERKS1_", "std::_Rb_tree<std::pair<unsigned long, long long>, std::pair<unsigned long, long long>, std::_Identity<std::pair<unsigned long, long long> >, std::less<std::pair<unsigned long, long long> >, std::allocator<std::pair<unsigned long, long long> > >::_M_insert_unique(std::pair<unsigned long, long long> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_14SmallBitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::SCEV const*, llvm::SmallBitVector, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::SmallBitVector> >::FindAndConstruct(llvm::SCEV const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_14SmallBitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::SCEV const*, llvm::SmallBitVector, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::SmallBitVector> >::InsertIntoBucket(llvm::SCEV const* const&, llvm::SmallBitVector const&, std::pair<llvm::SCEV const*, llvm::SmallBitVector>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_14SmallBitVectorENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::SCEV const*, llvm::SmallBitVector, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::SmallBitVector> >::grow(unsigned int)"},
    {"_ZNSt8_Rb_treeIxSt4pairIKxPKN4llvm4SCEVEESt10_Select1stIS6_ESt4lessIxESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<long long, std::pair<long long const, llvm::SCEV const*>, std::_Select1st<std::pair<long long const, llvm::SCEV const*> >, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >::_M_insert_unique(std::pair<long long const, llvm::SCEV const*> const&)"},
    {"_ZNSt8_Rb_treeIxSt4pairIKxPKN4llvm4SCEVEESt10_Select1stIS6_ESt4lessIxESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<long long, std::pair<long long const, llvm::SCEV const*>, std::_Select1st<std::pair<long long const, llvm::SCEV const*> >, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >::_M_erase(std::_Rb_tree_node<std::pair<long long const, llvm::SCEV const*> >*)"},
    {"_ZNSt8_Rb_treeIxSt4pairIKxPKN4llvm4SCEVEESt10_Select1stIS6_ESt4lessIxESaIS6_EE7_M_copyEPKSt13_Rb_tree_nodeIS6_EPSE_", "std::_Rb_tree<long long, std::pair<long long const, llvm::SCEV const*>, std::_Select1st<std::pair<long long const, llvm::SCEV const*> >, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >::_M_copy(std::_Rb_tree_node<std::pair<long long const, llvm::SCEV const*> > const*, std::_Rb_tree_node<std::pair<long long const, llvm::SCEV const*> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVESt3mapIxS3_St4lessIxESaISt4pairIKxS3_EEENS_12DenseMapInfoIS3_EENSC_ISB_EEE16InsertIntoBucketERKS3_RKSB_PS7_IS3_SB_E", "llvm::DenseMap<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > >::InsertIntoBucket(llvm::SCEV const* const&, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > const&, std::pair<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > >*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVESt3mapIxS3_St4lessIxESaISt4pairIKxS3_EEENS_12DenseMapInfoIS3_EENSC_ISB_EEE4growEj", "llvm::DenseMap<llvm::SCEV const*, std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > >, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<std::map<long long, llvm::SCEV const*, std::less<long long>, std::allocator<std::pair<long long const, llvm::SCEV const*> > > > >::grow(unsigned int)"},
    {"_ZNK4llvm14SCEVAddRecExpr17getStepRecurrenceERNS_15ScalarEvolutionE", "llvm::SCEVAddRecExpr::getStepRecurrence(llvm::ScalarEvolution&) const"},
    {"_ZN4llvm15ScalarEvolution13getAddRecExprERKNS_15SmallVectorImplIPKNS_4SCEVEEEPKNS_4LoopEbb", "llvm::ScalarEvolution::getAddRecExpr(llvm::SmallVectorImpl<llvm::SCEV const*> const&, llvm::Loop const*, bool, bool)"},
    {"_ZN4llvm8SmallSetIxLj8EE6insertERKx", "llvm::SmallSet<long long, 8u>::insert(long long const&)"},
    {"_ZNSt8_Rb_treeIxxSt9_IdentityIxESt4lessIxESaIxEE16_M_insert_uniqueERKx", "std::_Rb_tree<long long, long long, std::_Identity<long long>, std::less<long long>, std::allocator<long long> >::_M_insert_unique(long long const&)"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE18getLoopPredecessorEv", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::getLoopPredecessor() const"},
    {"_ZN4llvm8LoopPassD1Ev", "llvm::LoopPass::~LoopPass()"},
    {"_ZN4llvm8LoopPassD0Ev", "llvm::LoopPass::~LoopPass()"},
    {"_ZN4llvm24initializeLoopUnrollPassERNS_12PassRegistryE", "llvm::initializeLoopUnrollPass(llvm::PassRegistry&)"},
    {"_ZN4llvm20createLoopUnrollPassEv", "llvm::createLoopUnrollPass()"},
    {"_ZN12_GLOBAL__N_110LoopUnrollD1Ev", "(anonymous namespace)::LoopUnroll::~LoopUnroll()"},
    {"_ZN12_GLOBAL__N_110LoopUnrollD0Ev", "(anonymous namespace)::LoopUnroll::~LoopUnroll()"},
    {"_ZNK12_GLOBAL__N_110LoopUnroll16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopUnroll::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_110LoopUnroll9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopUnroll::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_110LoopUnrollEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopUnroll>()"},
    {"_ZN4llvm26initializeLoopUnswitchPassERNS_12PassRegistryE", "llvm::initializeLoopUnswitchPass(llvm::PassRegistry&)"},
    {"_ZN4llvm22createLoopUnswitchPassEb", "llvm::createLoopUnswitchPass(bool)"},
    {"_ZN12_GLOBAL__N_112LoopUnswitchD1Ev", "(anonymous namespace)::LoopUnswitch::~LoopUnswitch()"},
    {"_ZN12_GLOBAL__N_112LoopUnswitchD0Ev", "(anonymous namespace)::LoopUnswitch::~LoopUnswitch()"},
    {"_ZNK12_GLOBAL__N_112LoopUnswitch16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopUnswitch::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch13releaseMemoryEv", "(anonymous namespace)::LoopUnswitch::releaseMemory()"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopUnswitch::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_Z20FindLIVLoopConditionPN4llvm5ValueEPNS_4LoopERb", "FindLIVLoopCondition(llvm::Value*, llvm::Loop*, bool&)"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch20UnswitchIfProfitableEPN4llvm5ValueEPNS1_8ConstantE", "(anonymous namespace)::LoopUnswitch::UnswitchIfProfitable(llvm::Value*, llvm::Constant*)"},
    {"_Z9CloneLoopPN4llvm4LoopES1_RNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS3_EENS_14ValueMapConfigIS5_EENS_12DenseMapInfoIS7_EEEEPNS_8LoopInfoEPNS_13LPPassManagerE", "CloneLoop(llvm::Loop*, llvm::Loop*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, llvm::LoopInfo*, llvm::LPPassManager*)"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch30EmitPreheaderBranchOnConditionEPN4llvm5ValueEPNS1_8ConstantEPNS1_10BasicBlockES7_PNS1_11InstructionE", "(anonymous namespace)::LoopUnswitch::EmitPreheaderBranchOnCondition(llvm::Value*, llvm::Constant*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Instruction*)"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch36RewriteLoopBodyWithConditionConstantEPN4llvm4LoopEPNS1_5ValueEPNS1_8ConstantEb", "(anonymous namespace)::LoopUnswitch::RewriteLoopBodyWithConditionConstant(llvm::Loop*, llvm::Value*, llvm::Constant*, bool)"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch12SimplifyCodeERSt6vectorIPN4llvm11InstructionESaIS4_EEPNS2_4LoopE", "(anonymous namespace)::LoopUnswitch::SimplifyCode(std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >&, llvm::Loop*)"},
    {"_Z18RemoveFromWorklistPN4llvm11InstructionERSt6vectorIS1_SaIS1_EE", "RemoveFromWorklist(llvm::Instruction*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >&)"},
    {"_Z17ReplaceUsesOfWithPN4llvm11InstructionEPNS_5ValueERSt6vectorIS1_SaIS1_EEPNS_4LoopEPNS_13LPPassManagerE", "ReplaceUsesOfWith(llvm::Instruction*, llvm::Value*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >&, llvm::Loop*, llvm::LPPassManager*)"},
    {"_Z28isTrivialLoopExitBlockHelperPN4llvm4LoopEPNS_10BasicBlockERS3_RSt3setIS3_St4lessIS3_ESaIS3_EE", "isTrivialLoopExitBlockHelper(llvm::Loop*, llvm::BasicBlock*, llvm::BasicBlock*&, std::set<llvm::BasicBlock*, std::less<llvm::BasicBlock*>, std::allocator<llvm::BasicBlock*> >&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112LoopUnswitchEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopUnswitch>()"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm11InstructionESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::Instruction**, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::Instruction**, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > >, llvm::Instruction*>(__gnu_cxx::__normal_iterator<llvm::Instruction**, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > >, __gnu_cxx::__normal_iterator<llvm::Instruction**, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > >, llvm::Instruction* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm4UserESaIS2_EEC2INS0_18value_use_iteratorIS1_EEEET_S8_RKS3_", "std::vector<llvm::User*, std::allocator<llvm::User*> >::vector<llvm::value_use_iterator<llvm::User> >(llvm::value_use_iterator<llvm::User>, llvm::value_use_iterator<llvm::User>, std::allocator<llvm::User*> const&)"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE15_M_range_insertIPS2_EEvN9__gnu_cxx17__normal_iteratorIS6_S4_EET_SA_St20forward_iterator_tag", "void std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_range_insert<llvm::BasicBlock**>(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock**, llvm::BasicBlock**, std::forward_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPKS2_S4_EEEEvNS7_IPS2_S4_EET_SD_St20forward_iterator_tag", "void std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > >(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::BasicBlock* const*, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, std::forward_iterator_tag)"},
    {"_ZN4llvm19createMemCpyOptPassEv", "llvm::createMemCpyOptPass()"},
    {"_ZN4llvm23initializeMemCpyOptPassERNS_12PassRegistryE", "llvm::initializeMemCpyOptPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_19MemCpyOptEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::MemCpyOpt>()"},
    {"_ZN12_GLOBAL__N_19MemCpyOptD1Ev", "(anonymous namespace)::MemCpyOpt::~MemCpyOpt()"},
    {"_ZN12_GLOBAL__N_19MemCpyOptD0Ev", "(anonymous namespace)::MemCpyOpt::~MemCpyOpt()"},
    {"_ZNK12_GLOBAL__N_19MemCpyOpt16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::MemCpyOpt::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_19MemCpyOpt13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::MemCpyOpt::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_19MemCpyOpt20performCallSlotOptznEPN4llvm11InstructionEPNS1_5ValueES5_yPNS1_8CallInstE", "(anonymous namespace)::MemCpyOpt::performCallSlotOptzn(llvm::Instruction*, llvm::Value*, llvm::Value*, unsigned long long, llvm::CallInst*)"},
    {"_ZN12_GLOBAL__N_19MemCpyOpt20tryMergingIntoMemsetEPN4llvm11InstructionEPNS1_5ValueES5_", "(anonymous namespace)::MemCpyOpt::tryMergingIntoMemset(llvm::Instruction*, llvm::Value*, llvm::Value*)"},
    {"_Z15IsPointerOffsetPN4llvm5ValueES1_RxRKNS_10DataLayoutE", "IsPointerOffset(llvm::Value*, llvm::Value*, long long&, llvm::DataLayout const&)"},
    {"_ZN12_GLOBAL__N_112MemsetRanges8addStoreExPN4llvm9StoreInstE", "(anonymous namespace)::MemsetRanges::addStore(long long, llvm::StoreInst*)"},
    {"_ZN12_GLOBAL__N_112MemsetRanges8addRangeExxPN4llvm5ValueEjPNS1_11InstructionE", "(anonymous namespace)::MemsetRanges::addRange(long long, long long, llvm::Value*, unsigned int, llvm::Instruction*)"},
    {"_Z18GetOffsetFromIndexPKN4llvm17GetElementPtrInstEjRbRKNS_10DataLayoutE", "GetOffsetFromIndex(llvm::GetElementPtrInst const*, unsigned int, bool&, llvm::DataLayout const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_4UserEE6appendINS_18value_use_iteratorIS1_EEEEvT_S7_", "void llvm::SmallVectorImpl<llvm::User*>::append<llvm::value_use_iterator<llvm::User> >(llvm::value_use_iterator<llvm::User>, llvm::value_use_iterator<llvm::User>)"},
    {"_ZN4llvm25initializeReassociatePassERNS_12PassRegistryE", "llvm::initializeReassociatePass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createReassociatePassEv", "llvm::createReassociatePass()"},
    {"_ZN12_GLOBAL__N_111ReassociateC1Ev", "(anonymous namespace)::Reassociate::Reassociate()"},
    {"_ZN12_GLOBAL__N_111ReassociateD1Ev", "(anonymous namespace)::Reassociate::~Reassociate()"},
    {"_ZN12_GLOBAL__N_111ReassociateD0Ev", "(anonymous namespace)::Reassociate::~Reassociate()"},
    {"_ZNK12_GLOBAL__N_111Reassociate16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::Reassociate::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111Reassociate13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::Reassociate::runOnFunction(llvm::Function&)"},
    {"_Z21LowerNegateToMultiplyPN4llvm11InstructionERNS_8DenseMapINS_11AssertingVHINS_5ValueEEEjNS_12DenseMapInfoIS5_EENS6_IjEEEE", "LowerNegateToMultiply(llvm::Instruction*, llvm::DenseMap<llvm::AssertingVH<llvm::Value>, unsigned int, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Value> >, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN12_GLOBAL__N_111Reassociate21ReassociateExpressionEPN4llvm14BinaryOperatorE", "(anonymous namespace)::Reassociate::ReassociateExpression(llvm::BinaryOperator*)"},
    {"_ZN12_GLOBAL__N_111Reassociate17LinearizeExprTreeEPN4llvm14BinaryOperatorERNS1_15SmallVectorImplINS_10ValueEntryEEE", "(anonymous namespace)::Reassociate::LinearizeExprTree(llvm::BinaryOperator*, llvm::SmallVectorImpl<(anonymous namespace)::ValueEntry>&)"},
    {"_ZN12_GLOBAL__N_111Reassociate18OptimizeExpressionEPN4llvm14BinaryOperatorERNS1_15SmallVectorImplINS_10ValueEntryEEE", "(anonymous namespace)::Reassociate::OptimizeExpression(llvm::BinaryOperator*, llvm::SmallVectorImpl<(anonymous namespace)::ValueEntry>&)"},
    {"_ZN12_GLOBAL__N_111Reassociate18RemoveDeadBinaryOpEPN4llvm5ValueE", "(anonymous namespace)::Reassociate::RemoveDeadBinaryOp(llvm::Value*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_110ValueEntryEE6insertEPS2_RKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::ValueEntry>::insert((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry const&)"},
    {"_ZN12_GLOBAL__N_111Reassociate15RewriteExprTreeEPN4llvm14BinaryOperatorERNS1_15SmallVectorImplINS_10ValueEntryEEEj", "(anonymous namespace)::Reassociate::RewriteExprTree(llvm::BinaryOperator*, llvm::SmallVectorImpl<(anonymous namespace)::ValueEntry>&, unsigned int)"},
    {"_ZN12_GLOBAL__N_111Reassociate7getRankEPN4llvm5ValueE", "(anonymous namespace)::Reassociate::getRank(llvm::Value*)"},
    {"_Z28FindSingleUseMultiplyFactorsPN4llvm5ValueERNS_15SmallVectorImplIS1_EERKNS2_IN12_GLOBAL__N_110ValueEntryEEEb", "FindSingleUseMultiplyFactors(llvm::Value*, llvm::SmallVectorImpl<llvm::Value*>&, llvm::SmallVectorImpl<(anonymous namespace)::ValueEntry> const&, bool)"},
    {"_Z19EmitAddTreeOfValuesPN4llvm11InstructionERNS_15SmallVectorImplIPNS_5ValueEEE", "EmitAddTreeOfValues(llvm::Instruction*, llvm::SmallVectorImpl<llvm::Value*>&)"},
    {"_ZSt21__inplace_stable_sortIPN12_GLOBAL__N_110ValueEntryEEvT_S3_", "void std::__inplace_stable_sort<(anonymous namespace)::ValueEntry*>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*)"},
    {"_ZSt22__stable_sort_adaptiveIPN12_GLOBAL__N_110ValueEntryES2_lEvT_S3_T0_T1_", "void std::__stable_sort_adaptive<(anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, long>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, long)"},
    {"_ZSt24__merge_sort_with_bufferIPN12_GLOBAL__N_110ValueEntryES2_EvT_S3_T0_", "void std::__merge_sort_with_buffer<(anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*)"},
    {"_ZSt16__merge_adaptiveIPN12_GLOBAL__N_110ValueEntryElS2_EvT_S3_S3_T0_S4_T1_S4_", "void std::__merge_adaptive<(anonymous namespace)::ValueEntry*, long, (anonymous namespace)::ValueEntry*>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, long, long, (anonymous namespace)::ValueEntry*, long)"},
    {"_ZSt6rotateIPN12_GLOBAL__N_110ValueEntryEEvT_S3_S3_", "void std::rotate<(anonymous namespace)::ValueEntry*>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*)"},
    {"_ZSt16__insertion_sortIPN12_GLOBAL__N_110ValueEntryEEvT_S3_", "void std::__insertion_sort<(anonymous namespace)::ValueEntry*>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*)"},
    {"_ZSt22__merge_without_bufferIPN12_GLOBAL__N_110ValueEntryElEvT_S3_S3_T0_S4_", "void std::__merge_without_buffer<(anonymous namespace)::ValueEntry*, long>((anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, (anonymous namespace)::ValueEntry*, long, long)"},
    {"_Z11NegateValuePN4llvm5ValueEPNS_11InstructionE", "NegateValue(llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111ReassociateEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::Reassociate>()"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_5ValueEEEjNS_12DenseMapInfoIS3_EENS4_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::AssertingVH<llvm::Value>, unsigned int, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Value> >, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_5ValueEEEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::AssertingVH<llvm::Value>, unsigned int, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Value> >, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::AssertingVH<llvm::Value> const&, unsigned int const&, std::pair<llvm::AssertingVH<llvm::Value>, unsigned int>*)"},
    {"_ZN4llvm8DenseMapINS_11AssertingVHINS_5ValueEEEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::AssertingVH<llvm::Value>, unsigned int, llvm::DenseMapInfo<llvm::AssertingVH<llvm::Value> >, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm25ReversePostOrderTraversalIPNS_8FunctionENS_11GraphTraitsIS2_EEEC2ES2_", "llvm::ReversePostOrderTraversal<llvm::Function*, llvm::GraphTraits<llvm::Function*> >::ReversePostOrderTraversal(llvm::Function*)"},
    {"_ZSt4copyIN4llvm11po_iteratorIPNS0_10BasicBlockENS0_11SmallPtrSetIS3_Lj8EEELb0ENS0_11GraphTraitsIS3_EEEESt20back_insert_iteratorISt6vectorIS3_SaIS3_EEEET0_T_SF_SE_", "std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::copy<llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > >(llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >)"},
    {"_ZN4llvm8po_beginIPNS_10BasicBlockEEENS_11po_iteratorIT_NS_11SmallPtrSetIPNS_11GraphTraitsIS4_E8NodeTypeELj8EEELb0ES7_EES4_", "llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::GraphTraits<llvm::BasicBlock*>::NodeType*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> > llvm::po_begin<llvm::BasicBlock*>(llvm::BasicBlock*)"},
    {"_ZN4llvm11po_iteratorIPNS_10BasicBlockENS_11SmallPtrSetIS2_Lj8EEELb0ENS_11GraphTraitsIS2_EEE13traverseChildEv", "llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >::traverseChild()"},
    {"_ZNSt6vectorISt4pairIPN4llvm10BasicBlockENS1_12SuccIteratorIPNS1_14TerminatorInstES2_EEESaIS8_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS8_SA_EERKS8_", "std::vector<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >*, std::vector<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > > >, std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > const&)"},
    {"_ZNSt13__copy_normalILb0ELb0EE8__copy_nIN4llvm11po_iteratorIPNS2_10BasicBlockENS2_11SmallPtrSetIS5_Lj8EEELb0ENS2_11GraphTraitsIS5_EEEESt20back_insert_iteratorISt6vectorIS5_SaIS5_EEEEET0_T_SH_SG_", "std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::__copy_normal<false, false>::__copy_n<llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > >(llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >)"},
    {"_ZNSt6vectorISt4pairIPN4llvm10BasicBlockENS1_12SuccIteratorIPNS1_14TerminatorInstES2_EEESaIS8_EEC2ERKSA_", "std::vector<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > >::vector(std::vector<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, std::allocator<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> > > > const&)"},
    {"_ZSt10__copy_auxIN4llvm11po_iteratorIPNS0_10BasicBlockENS0_11SmallPtrSetIS3_Lj8EEELb0ENS0_11GraphTraitsIS3_EEEESt20back_insert_iteratorISt6vectorIS3_SaIS3_EEEET0_T_SF_SE_", "std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::__copy_aux<llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > >(llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >)"},
    {"_ZNSt6__copyILb0ESt20forward_iterator_tagE4copyIN4llvm11po_iteratorIPNS3_10BasicBlockENS3_11SmallPtrSetIS6_Lj8EEELb0ENS3_11GraphTraitsIS6_EEEESt20back_insert_iteratorISt6vectorIS6_SaIS6_EEEEET0_T_SI_SH_", "std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::__copy<false, std::forward_iterator_tag>::copy<llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > >(llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, llvm::po_iterator<llvm::BasicBlock*, llvm::SmallPtrSet<llvm::BasicBlock*, 8u>, false, llvm::GraphTraits<llvm::BasicBlock*> >, std::back_insert_iterator<std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >)"},
    {"_ZN4llvm18initializeSCCPPassERNS_12PassRegistryE", "llvm::initializeSCCPPass(llvm::PassRegistry&)"},
    {"_ZN4llvm14createSCCPPassEv", "llvm::createSCCPPass()"},
    {"_ZN4llvm20initializeIPSCCPPassERNS_12PassRegistryE", "llvm::initializeIPSCCPPass(llvm::PassRegistry&)"},
    {"_ZN4llvm16createIPSCCPPassEv", "llvm::createIPSCCPPass()"},
    {"_ZN12_GLOBAL__N_16IPSCCPD1Ev", "(anonymous namespace)::IPSCCP::~IPSCCP()"},
    {"_ZN12_GLOBAL__N_16IPSCCPD0Ev", "(anonymous namespace)::IPSCCP::~IPSCCP()"},
    {"_ZN12_GLOBAL__N_16IPSCCP11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::IPSCCP::runOnModule(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolverC1EPKN4llvm10DataLayoutE", "(anonymous namespace)::SCCPSolver::SCCPSolver(llvm::DataLayout const*)"},
    {"_Z14AddressIsTakenPKN4llvm11GlobalValueE", "AddressIsTaken(llvm::GlobalValue const*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver23markAnythingOverdefinedEPN4llvm5ValueE", "(anonymous namespace)::SCCPSolver::markAnythingOverdefined(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver5SolveEv", "(anonymous namespace)::SCCPSolver::Solve()"},
    {"_ZN12_GLOBAL__N_110SCCPSolver16ResolvedUndefsInERN4llvm8FunctionE", "(anonymous namespace)::SCCPSolver::ResolvedUndefsIn(llvm::Function&)"},
    {"_Z24DeleteInstructionInBlockPN4llvm10BasicBlockE", "DeleteInstructionInBlock(llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolverD1Ev", "(anonymous namespace)::SCCPSolver::~SCCPSolver()"},
    {"_ZN12_GLOBAL__N_110SCCPSolver19getStructValueStateEPN4llvm5ValueEj", "(anonymous namespace)::SCCPSolver::getStructValueState(llvm::Value*, unsigned int)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver13getValueStateEPN4llvm5ValueE", "(anonymous namespace)::SCCPSolver::getValueState(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver18markForcedConstantEPN4llvm5ValueEPNS1_8ConstantE", "(anonymous namespace)::SCCPSolver::markForcedConstant(llvm::Value*, llvm::Constant*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver15markOverdefinedEPN4llvm5ValueE", "(anonymous namespace)::SCCPSolver::markOverdefined(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver18markEdgeExecutableEPN4llvm10BasicBlockES3_", "(anonymous namespace)::SCCPSolver::markEdgeExecutable(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver12visitPHINodeERN4llvm7PHINodeE", "(anonymous namespace)::SCCPSolver::visitPHINode(llvm::PHINode&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver12markConstantEPN4llvm5ValueEPNS1_8ConstantE", "(anonymous namespace)::SCCPSolver::markConstant(llvm::Value*, llvm::Constant*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEN12_GLOBAL__N_110LatticeValENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Value*, (anonymous namespace)::LatticeVal, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<(anonymous namespace)::LatticeVal> >::InsertIntoBucket(llvm::Value* const&, (anonymous namespace)::LatticeVal const&, std::pair<llvm::Value*, (anonymous namespace)::LatticeVal>*)"},
    {"_ZN4llvm11InstVisitorIN12_GLOBAL__N_110SCCPSolverEvE5visitERNS_11InstructionE", "llvm::InstVisitor<(anonymous namespace)::SCCPSolver, void>::visit(llvm::Instruction&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver12mergeInValueERNS_10LatticeValEPN4llvm5ValueES1_", "(anonymous namespace)::SCCPSolver::mergeInValue((anonymous namespace)::LatticeVal&, llvm::Value*, (anonymous namespace)::LatticeVal)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver12mergeInValueEPN4llvm5ValueENS_10LatticeValE", "(anonymous namespace)::SCCPSolver::mergeInValue(llvm::Value*, (anonymous namespace)::LatticeVal)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver13visitCallSiteEN4llvm8CallSiteE", "(anonymous namespace)::SCCPSolver::visitCallSite(llvm::CallSite)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_8FunctionEjEN12_GLOBAL__N_110LatticeValENS_12DenseMapInfoIS4_EENS7_IS6_EEE15LookupBucketForERKS4_RPS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::Function*, unsigned int>, (anonymous namespace)::LatticeVal, llvm::DenseMapInfo<std::pair<llvm::Function*, unsigned int> >, llvm::DenseMapInfo<(anonymous namespace)::LatticeVal> >::LookupBucketFor(std::pair<llvm::Function*, unsigned int> const&, std::pair<std::pair<llvm::Function*, unsigned int>, (anonymous namespace)::LatticeVal>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_8FunctionEjEN12_GLOBAL__N_110LatticeValENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::Function*, unsigned int>, (anonymous namespace)::LatticeVal, llvm::DenseMapInfo<std::pair<llvm::Function*, unsigned int> >, llvm::DenseMapInfo<(anonymous namespace)::LatticeVal> >::InsertIntoBucket(std::pair<llvm::Function*, unsigned int> const&, (anonymous namespace)::LatticeVal const&, std::pair<std::pair<llvm::Function*, unsigned int>, (anonymous namespace)::LatticeVal>*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver12visitCmpInstERN4llvm7CmpInstE", "(anonymous namespace)::SCCPSolver::visitCmpInst(llvm::CmpInst&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver23InsertInOverdefinedPHIsEPN4llvm11InstructionEPNS1_7PHINodeE", "(anonymous namespace)::SCCPSolver::InsertInOverdefinedPHIs(llvm::Instruction*, llvm::PHINode*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver25RemoveFromOverdefinedPHIsEPN4llvm11InstructionEPNS1_7PHINodeE", "(anonymous namespace)::SCCPSolver::RemoveFromOverdefinedPHIs(llvm::Instruction*, llvm::PHINode*)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver13visitCastInstERN4llvm8CastInstE", "(anonymous namespace)::SCCPSolver::visitCastInst(llvm::CastInst&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver19visitBinaryOperatorERN4llvm11InstructionE", "(anonymous namespace)::SCCPSolver::visitBinaryOperator(llvm::Instruction&)"},
    {"_ZN12_GLOBAL__N_110SCCPSolver19visitTerminatorInstERN4llvm14TerminatorInstE", "(anonymous namespace)::SCCPSolver::visitTerminatorInst(llvm::TerminatorInst&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_5ValueEjEN12_GLOBAL__N_110LatticeValENS_12DenseMapInfoIS4_EENS7_IS6_EEE15LookupBucketForERKS4_RPS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::Value*, unsigned int>, (anonymous namespace)::LatticeVal, llvm::DenseMapInfo<std::pair<llvm::Value*, unsigned int> >, llvm::DenseMapInfo<(anonymous namespace)::LatticeVal> >::LookupBucketFor(std::pair<llvm::Value*, unsigned int> const&, std::pair<std::pair<llvm::Value*, unsigned int>, (anonymous namespace)::LatticeVal>*&) const"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_16IPSCCPEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::IPSCCP>()"},
    {"_ZN12_GLOBAL__N_14SCCPD1Ev", "(anonymous namespace)::SCCP::~SCCP()"},
    {"_ZN12_GLOBAL__N_14SCCPD0Ev", "(anonymous namespace)::SCCP::~SCCP()"},
    {"_ZN12_GLOBAL__N_14SCCP13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::SCCP::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_14SCCPEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::SCCP>()"},
    {"_ZNSt8_Rb_treeIPN4llvm7PHINodeESt4pairIKS2_PNS0_11InstructionEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::PHINode*, std::pair<llvm::PHINode* const, llvm::Instruction*>, std::_Select1st<std::pair<llvm::PHINode* const, llvm::Instruction*> >, std::less<llvm::PHINode*>, std::allocator<std::pair<llvm::PHINode* const, llvm::Instruction*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::PHINode* const, llvm::Instruction*> >*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockES3_EcNS_12DenseMapInfoIS4_EENS5_IcEEE6insertERKS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char> const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_10BasicBlockES3_EcNS_12DenseMapInfoIS4_EENS5_IcEEE15LookupBucketForERKS4_RPS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::LookupBucketFor(std::pair<llvm::BasicBlock*, llvm::BasicBlock*> const&, std::pair<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockES3_EcNS_12DenseMapInfoIS4_EENS5_IcEEE16InsertIntoBucketERKS4_RKcPS1_IS4_cE", "llvm::DenseMap<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<llvm::BasicBlock*, llvm::BasicBlock*> const&, char const&, std::pair<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockES3_EcNS_12DenseMapInfoIS4_EENS5_IcEEE4growEj", "llvm::DenseMap<std::pair<llvm::BasicBlock*, llvm::BasicBlock*>, char, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, llvm::BasicBlock*> >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm21initializeSROA_DTPassERNS_12PassRegistryE", "llvm::initializeSROA_DTPass(llvm::PassRegistry&)"},
    {"_ZN4llvm24initializeSROA_SSAUpPassERNS_12PassRegistryE", "llvm::initializeSROA_SSAUpPass(llvm::PassRegistry&)"},
    {"_ZN4llvm30createScalarReplAggregatesPassEib", "llvm::createScalarReplAggregatesPass(int, bool)"},
    {"_ZN12_GLOBAL__N_110SROA_SSAUpD1Ev", "(anonymous namespace)::SROA_SSAUp::~SROA_SSAUp()"},
    {"_ZN12_GLOBAL__N_110SROA_SSAUpD0Ev", "(anonymous namespace)::SROA_SSAUp::~SROA_SSAUp()"},
    {"_ZNK12_GLOBAL__N_110SROA_SSAUp16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::SROA_SSAUp::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14SROA13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::SROA::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_14SROA16performPromotionERN4llvm8FunctionE", "(anonymous namespace)::SROA::performPromotion(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_119ConvertToScalarInfo18CanConvertToScalarEPN4llvm5ValueEy", "(anonymous namespace)::ConvertToScalarInfo::CanConvertToScalar(llvm::Value*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_119ConvertToScalarInfo19ConvertUsesToScalarEPN4llvm5ValueEPNS1_10AllocaInstEy", "(anonymous namespace)::ConvertToScalarInfo::ConvertUsesToScalar(llvm::Value*, llvm::AllocaInst*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_119ConvertToScalarInfo26ConvertScalar_ExtractValueEPN4llvm5ValueEPKNS1_4TypeEyRNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::ConvertToScalarInfo::ConvertScalar_ExtractValue(llvm::Value*, llvm::Type const*, unsigned long long, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_119ConvertToScalarInfo25ConvertScalar_InsertValueEPN4llvm5ValueES3_yRNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::ConvertToScalarInfo::ConvertScalar_InsertValue(llvm::Value*, llvm::Value*, unsigned long long, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_119ConvertToScalarInfo11MergeInTypeEPKN4llvm4TypeEy", "(anonymous namespace)::ConvertToScalarInfo::MergeInType(llvm::Type const*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_14SROA20RewriteForScalarReplEPN4llvm11InstructionEPNS1_10AllocaInstEyRNS1_11SmallVectorIS5_Lj32EEE", "(anonymous namespace)::SROA::RewriteForScalarRepl(llvm::Instruction*, llvm::AllocaInst*, unsigned long long, llvm::SmallVector<llvm::AllocaInst*, 32u>&)"},
    {"_ZN12_GLOBAL__N_14SROA20FindElementAndOffsetERPKN4llvm4TypeERyS5_", "(anonymous namespace)::SROA::FindElementAndOffset(llvm::Type const*&, unsigned long long&, llvm::Type const*&)"},
    {"_ZN12_GLOBAL__N_14SROA19isSafeForScalarReplEPN4llvm11InstructionEyRNS0_10AllocaInfoE", "(anonymous namespace)::SROA::isSafeForScalarRepl(llvm::Instruction*, unsigned long long, (anonymous namespace)::SROA::AllocaInfo&)"},
    {"_ZN12_GLOBAL__N_14SROA15isSafeMemAccessEyyPKN4llvm4TypeEbRNS0_10AllocaInfoEPNS1_11InstructionEb", "(anonymous namespace)::SROA::isSafeMemAccess(unsigned long long, unsigned long long, llvm::Type const*, bool, (anonymous namespace)::SROA::AllocaInfo&, llvm::Instruction*, bool)"},
    {"_ZN12_GLOBAL__N_14SROA31isSafePHISelectUseForScalarReplEPN4llvm11InstructionEyRNS0_10AllocaInfoE", "(anonymous namespace)::SROA::isSafePHISelectUseForScalarRepl(llvm::Instruction*, unsigned long long, (anonymous namespace)::SROA::AllocaInfo&)"},
    {"_ZN12_GLOBAL__N_14SROA16TypeHasComponentEPKN4llvm4TypeEyy", "(anonymous namespace)::SROA::TypeHasComponent(llvm::Type const*, unsigned long long, unsigned long long)"},
    {"_Z30isOnlyCopiedFromConstantGlobalPN4llvm5ValueERPNS_15MemTransferInstEb", "isOnlyCopiedFromConstantGlobal(llvm::Value*, llvm::MemTransferInst*&, bool)"},
    {"_Z27tryToMakeAllocaBePromotablePN4llvm10AllocaInstEPKNS_10DataLayoutE", "tryToMakeAllocaBePromotable(llvm::AllocaInst*, llvm::DataLayout const*)"},
    {"_ZN12_GLOBAL__N_114AllocaPromoterD1Ev", "(anonymous namespace)::AllocaPromoter::~AllocaPromoter()"},
    {"_ZN12_GLOBAL__N_114AllocaPromoterD0Ev", "(anonymous namespace)::AllocaPromoter::~AllocaPromoter()"},
    {"_ZNK12_GLOBAL__N_114AllocaPromoter12isInstInListEPN4llvm11InstructionERKNS1_15SmallVectorImplIS3_EE", "(anonymous namespace)::AllocaPromoter::isInstInList(llvm::Instruction*, llvm::SmallVectorImpl<llvm::Instruction*> const&) const"},
    {"_ZN12_GLOBAL__N_14SROAD1Ev", "(anonymous namespace)::SROA::~SROA()"},
    {"_ZN12_GLOBAL__N_14SROAD0Ev", "(anonymous namespace)::SROA::~SROA()"},
    {"_ZN12_GLOBAL__N_17SROA_DTD1Ev", "(anonymous namespace)::SROA_DT::~SROA_DT()"},
    {"_ZN12_GLOBAL__N_17SROA_DTD0Ev", "(anonymous namespace)::SROA_DT::~SROA_DT()"},
    {"_ZNK12_GLOBAL__N_17SROA_DT16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::SROA_DT::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_110SROA_SSAUpEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::SROA_SSAUp>()"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_17SROA_DTEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::SROA_DT>()"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateLoadEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateLoad(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE19CreateInsertElementEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateInsertElement(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE18CreateExtractValueEPNS_5ValueEjRKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateExtractValue(llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE20CreateExtractElementEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateExtractElement(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE17CreateInsertValueEPNS_5ValueES6_jRKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateInsertValue(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm11SmallVectorIPNS_8ConstantELj16EEC2EjRKS2_", "llvm::SmallVector<llvm::Constant*, 16u>::SmallVector(unsigned int, llvm::Constant* const&)"},
    {"_ZNSt6vectorIPN4llvm10AllocaInstESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AllocaInst**, std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > >, llvm::AllocaInst* const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_8LoadInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::LoadInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::LoadInst*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::LoadInst* const&, std::pair<llvm::BasicBlock*, llvm::LoadInst*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_8LoadInstENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::LoadInst*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::LoadInst*> >::grow(unsigned int)"},
    {"_ZN4llvm29initializeCFGSimplifyPassPassERNS_12PassRegistryE", "llvm::initializeCFGSimplifyPassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm27createCFGSimplificationPassEv", "llvm::createCFGSimplificationPass()"},
    {"_ZN12_GLOBAL__N_115CFGSimplifyPassD1Ev", "(anonymous namespace)::CFGSimplifyPass::~CFGSimplifyPass()"},
    {"_ZN12_GLOBAL__N_115CFGSimplifyPassD0Ev", "(anonymous namespace)::CFGSimplifyPass::~CFGSimplifyPass()"},
    {"_ZN12_GLOBAL__N_115CFGSimplifyPass13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::CFGSimplifyPass::runOnFunction(llvm::Function&)"},
    {"_Z29RemoveUnreachableBlocksFromFnRN4llvm8FunctionE", "RemoveUnreachableBlocksFromFn(llvm::Function&)"},
    {"_Z20IterativeSimplifyCFGRN4llvm8FunctionEPKNS_10DataLayoutE", "IterativeSimplifyCFG(llvm::Function&, llvm::DataLayout const*)"},
    {"_Z19ChangeToUnreachablePN4llvm11InstructionEb", "ChangeToUnreachable(llvm::Instruction*, bool)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_115CFGSimplifyPassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::CFGSimplifyPass>()"},
    {"_ZN4llvm30initializeSimplifyLibCallsPassERNS_12PassRegistryE", "llvm::initializeSimplifyLibCallsPass(llvm::PassRegistry&)"},
    {"_ZN4llvm26createSimplifyLibCallsPassEv", "llvm::createSimplifyLibCallsPass()"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCallsD1Ev", "(anonymous namespace)::SimplifyLibCalls::~SimplifyLibCalls()"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCallsD0Ev", "(anonymous namespace)::SimplifyLibCalls::~SimplifyLibCalls()"},
    {"_ZNK12_GLOBAL__N_116SimplifyLibCalls16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::SimplifyLibCalls::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::SimplifyLibCalls::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::SimplifyLibCalls::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm9StringMapIPN12_GLOBAL__N_119LibCallOptimizationENS_15MallocAllocatorEEixENS_9StringRefE", "llvm::StringMap<(anonymous namespace)::LibCallOptimization*, llvm::MallocAllocator>::operator[](llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls15setDoesNotThrowERN4llvm8FunctionE", "(anonymous namespace)::SimplifyLibCalls::setDoesNotThrow(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls15setDoesNotAliasERN4llvm8FunctionEj", "(anonymous namespace)::SimplifyLibCalls::setDoesNotAlias(llvm::Function&, unsigned int)"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls22setDoesNotAccessMemoryERN4llvm8FunctionE", "(anonymous namespace)::SimplifyLibCalls::setDoesNotAccessMemory(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_17PutsOptD1Ev", "(anonymous namespace)::PutsOpt::~PutsOpt()"},
    {"_ZN12_GLOBAL__N_110FPrintFOptD1Ev", "(anonymous namespace)::FPrintFOpt::~FPrintFOpt()"},
    {"_ZN12_GLOBAL__N_18FPutsOptD1Ev", "(anonymous namespace)::FPutsOpt::~FPutsOpt()"},
    {"_ZN12_GLOBAL__N_19FWriteOptD1Ev", "(anonymous namespace)::FWriteOpt::~FWriteOpt()"},
    {"_ZN12_GLOBAL__N_19PrintFOptD1Ev", "(anonymous namespace)::PrintFOpt::~PrintFOpt()"},
    {"_ZN12_GLOBAL__N_110SPrintFOptD1Ev", "(anonymous namespace)::SPrintFOpt::~SPrintFOpt()"},
    {"_ZN12_GLOBAL__N_110ToAsciiOptD1Ev", "(anonymous namespace)::ToAsciiOpt::~ToAsciiOpt()"},
    {"_ZN12_GLOBAL__N_110IsAsciiOptD1Ev", "(anonymous namespace)::IsAsciiOpt::~IsAsciiOpt()"},
    {"_ZN12_GLOBAL__N_110IsDigitOptD1Ev", "(anonymous namespace)::IsDigitOpt::~IsDigitOpt()"},
    {"_ZN12_GLOBAL__N_16AbsOptD1Ev", "(anonymous namespace)::AbsOpt::~AbsOpt()"},
    {"_ZN12_GLOBAL__N_16FFSOptD1Ev", "(anonymous namespace)::FFSOpt::~FFSOpt()"},
    {"_ZN12_GLOBAL__N_116UnaryDoubleFPOptD1Ev", "(anonymous namespace)::UnaryDoubleFPOpt::~UnaryDoubleFPOpt()"},
    {"_ZN12_GLOBAL__N_17Exp2OptD1Ev", "(anonymous namespace)::Exp2Opt::~Exp2Opt()"},
    {"_ZN12_GLOBAL__N_16PowOptD1Ev", "(anonymous namespace)::PowOpt::~PowOpt()"},
    {"_ZN12_GLOBAL__N_19MemSetOptD1Ev", "(anonymous namespace)::MemSetOpt::~MemSetOpt()"},
    {"_ZN12_GLOBAL__N_110MemMoveOptD1Ev", "(anonymous namespace)::MemMoveOpt::~MemMoveOpt()"},
    {"_ZN12_GLOBAL__N_19MemCpyOptD1Ev", "(anonymous namespace)::MemCpyOpt::~MemCpyOpt()"},
    {"_ZN12_GLOBAL__N_19MemCmpOptD1Ev", "(anonymous namespace)::MemCmpOpt::~MemCmpOpt()"},
    {"_ZN12_GLOBAL__N_19StrStrOptD1Ev", "(anonymous namespace)::StrStrOpt::~StrStrOpt()"},
    {"_ZN12_GLOBAL__N_110StrCSpnOptD1Ev", "(anonymous namespace)::StrCSpnOpt::~StrCSpnOpt()"},
    {"_ZN12_GLOBAL__N_19StrSpnOptD1Ev", "(anonymous namespace)::StrSpnOpt::~StrSpnOpt()"},
    {"_ZN12_GLOBAL__N_18StrToOptD1Ev", "(anonymous namespace)::StrToOpt::~StrToOpt()"},
    {"_ZN12_GLOBAL__N_110StrPBrkOptD1Ev", "(anonymous namespace)::StrPBrkOpt::~StrPBrkOpt()"},
    {"_ZN12_GLOBAL__N_19StrLenOptD1Ev", "(anonymous namespace)::StrLenOpt::~StrLenOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCpyOptD1Ev", "(anonymous namespace)::StrNCpyOpt::~StrNCpyOpt()"},
    {"_ZN12_GLOBAL__N_19StrCpyOptD1Ev", "(anonymous namespace)::StrCpyOpt::~StrCpyOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCmpOptD1Ev", "(anonymous namespace)::StrNCmpOpt::~StrNCmpOpt()"},
    {"_ZN12_GLOBAL__N_19StrCmpOptD1Ev", "(anonymous namespace)::StrCmpOpt::~StrCmpOpt()"},
    {"_ZN12_GLOBAL__N_110StrRChrOptD1Ev", "(anonymous namespace)::StrRChrOpt::~StrRChrOpt()"},
    {"_ZN12_GLOBAL__N_19StrChrOptD1Ev", "(anonymous namespace)::StrChrOpt::~StrChrOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCatOptD1Ev", "(anonymous namespace)::StrNCatOpt::~StrNCatOpt()"},
    {"_ZN12_GLOBAL__N_19StrCatOptD1Ev", "(anonymous namespace)::StrCatOpt::~StrCatOpt()"},
    {"_ZN12_GLOBAL__N_19StrCatOptD0Ev", "(anonymous namespace)::StrCatOpt::~StrCatOpt()"},
    {"_ZN12_GLOBAL__N_19StrCatOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrCatOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_119LibCallOptimizationD1Ev", "(anonymous namespace)::LibCallOptimization::~LibCallOptimization()"},
    {"_ZN12_GLOBAL__N_119LibCallOptimizationD0Ev", "(anonymous namespace)::LibCallOptimization::~LibCallOptimization()"},
    {"_ZN12_GLOBAL__N_110StrNCatOptD0Ev", "(anonymous namespace)::StrNCatOpt::~StrNCatOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCatOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrNCatOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrChrOptD0Ev", "(anonymous namespace)::StrChrOpt::~StrChrOpt()"},
    {"_ZN12_GLOBAL__N_19StrChrOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrChrOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110StrRChrOptD0Ev", "(anonymous namespace)::StrRChrOpt::~StrRChrOpt()"},
    {"_ZN12_GLOBAL__N_110StrRChrOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrRChrOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrCmpOptD0Ev", "(anonymous namespace)::StrCmpOpt::~StrCmpOpt()"},
    {"_ZN12_GLOBAL__N_19StrCmpOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrCmpOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110StrNCmpOptD0Ev", "(anonymous namespace)::StrNCmpOpt::~StrNCmpOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCmpOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrNCmpOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrCpyOptD0Ev", "(anonymous namespace)::StrCpyOpt::~StrCpyOpt()"},
    {"_ZN12_GLOBAL__N_19StrCpyOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrCpyOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110StrNCpyOptD0Ev", "(anonymous namespace)::StrNCpyOpt::~StrNCpyOpt()"},
    {"_ZN12_GLOBAL__N_110StrNCpyOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrNCpyOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrLenOptD0Ev", "(anonymous namespace)::StrLenOpt::~StrLenOpt()"},
    {"_ZN12_GLOBAL__N_19StrLenOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrLenOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110StrPBrkOptD0Ev", "(anonymous namespace)::StrPBrkOpt::~StrPBrkOpt()"},
    {"_ZN12_GLOBAL__N_110StrPBrkOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrPBrkOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_18StrToOptD0Ev", "(anonymous namespace)::StrToOpt::~StrToOpt()"},
    {"_ZN12_GLOBAL__N_18StrToOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrToOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrSpnOptD0Ev", "(anonymous namespace)::StrSpnOpt::~StrSpnOpt()"},
    {"_ZN12_GLOBAL__N_19StrSpnOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrSpnOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110StrCSpnOptD0Ev", "(anonymous namespace)::StrCSpnOpt::~StrCSpnOpt()"},
    {"_ZN12_GLOBAL__N_110StrCSpnOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrCSpnOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19StrStrOptD0Ev", "(anonymous namespace)::StrStrOpt::~StrStrOpt()"},
    {"_ZN12_GLOBAL__N_19StrStrOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::StrStrOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19MemCmpOptD0Ev", "(anonymous namespace)::MemCmpOpt::~MemCmpOpt()"},
    {"_ZN12_GLOBAL__N_19MemCmpOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::MemCmpOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19MemCpyOptD0Ev", "(anonymous namespace)::MemCpyOpt::~MemCpyOpt()"},
    {"_ZN12_GLOBAL__N_19MemCpyOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::MemCpyOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110MemMoveOptD0Ev", "(anonymous namespace)::MemMoveOpt::~MemMoveOpt()"},
    {"_ZN12_GLOBAL__N_110MemMoveOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::MemMoveOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19MemSetOptD0Ev", "(anonymous namespace)::MemSetOpt::~MemSetOpt()"},
    {"_ZN12_GLOBAL__N_19MemSetOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::MemSetOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_16PowOptD0Ev", "(anonymous namespace)::PowOpt::~PowOpt()"},
    {"_ZN12_GLOBAL__N_16PowOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::PowOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_17Exp2OptD0Ev", "(anonymous namespace)::Exp2Opt::~Exp2Opt()"},
    {"_ZN12_GLOBAL__N_17Exp2Opt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::Exp2Opt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_116UnaryDoubleFPOptD0Ev", "(anonymous namespace)::UnaryDoubleFPOpt::~UnaryDoubleFPOpt()"},
    {"_ZN12_GLOBAL__N_116UnaryDoubleFPOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::UnaryDoubleFPOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_16FFSOptD0Ev", "(anonymous namespace)::FFSOpt::~FFSOpt()"},
    {"_ZN12_GLOBAL__N_16FFSOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::FFSOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_16AbsOptD0Ev", "(anonymous namespace)::AbsOpt::~AbsOpt()"},
    {"_ZN12_GLOBAL__N_16AbsOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::AbsOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110IsDigitOptD0Ev", "(anonymous namespace)::IsDigitOpt::~IsDigitOpt()"},
    {"_ZN12_GLOBAL__N_110IsDigitOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::IsDigitOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110IsAsciiOptD0Ev", "(anonymous namespace)::IsAsciiOpt::~IsAsciiOpt()"},
    {"_ZN12_GLOBAL__N_110IsAsciiOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::IsAsciiOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110ToAsciiOptD0Ev", "(anonymous namespace)::ToAsciiOpt::~ToAsciiOpt()"},
    {"_ZN12_GLOBAL__N_110ToAsciiOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::ToAsciiOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110SPrintFOptD0Ev", "(anonymous namespace)::SPrintFOpt::~SPrintFOpt()"},
    {"_ZN12_GLOBAL__N_110SPrintFOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::SPrintFOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19PrintFOptD0Ev", "(anonymous namespace)::PrintFOpt::~PrintFOpt()"},
    {"_ZN12_GLOBAL__N_19PrintFOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::PrintFOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_19FWriteOptD0Ev", "(anonymous namespace)::FWriteOpt::~FWriteOpt()"},
    {"_ZN12_GLOBAL__N_19FWriteOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::FWriteOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_18FPutsOptD0Ev", "(anonymous namespace)::FPutsOpt::~FPutsOpt()"},
    {"_ZN12_GLOBAL__N_18FPutsOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::FPutsOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_110FPrintFOptD0Ev", "(anonymous namespace)::FPrintFOpt::~FPrintFOpt()"},
    {"_ZN12_GLOBAL__N_110FPrintFOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::FPrintFOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN12_GLOBAL__N_17PutsOptD0Ev", "(anonymous namespace)::PutsOpt::~PutsOpt()"},
    {"_ZN12_GLOBAL__N_17PutsOpt13CallOptimizerEPN4llvm8FunctionEPNS1_8CallInstERNS1_9IRBuilderILb1ENS1_14ConstantFolderENS1_24IRBuilderDefaultInserterILb1EEEEE", "(anonymous namespace)::PutsOpt::CallOptimizer(llvm::Function*, llvm::CallInst*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_116SimplifyLibCallsEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::SimplifyLibCalls>()"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateGEPEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateGEP(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateLoadEPNS_5ValueEPKc", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateLoad(llvm::Value*, char const*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateICmpENS_7CmpInst9PredicateEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateICmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE26CreateConstInBoundsGEP1_64EPNS_5ValueEyRKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateConstInBoundsGEP1_64(llvm::Value*, unsigned long long, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE12CreateSelectEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateSelect(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateFMulEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateFMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateFDivEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateFDiv(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateFCmpENS_7CmpInst9PredicateEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateFCmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE11CreateCall2EPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCall2(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateCallEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCall(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateNegEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateNeg(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE11CreateStoreEPNS_5ValueES6_b", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateStore(llvm::Value*, llvm::Value*, bool)"},
    {"_ZN4llvm26initializeTailCallElimPassERNS_12PassRegistryE", "llvm::initializeTailCallElimPass(llvm::PassRegistry&)"},
    {"_ZN4llvm29createTailCallEliminationPassEv", "llvm::createTailCallEliminationPass()"},
    {"_ZN12_GLOBAL__N_112TailCallElimD1Ev", "(anonymous namespace)::TailCallElim::~TailCallElim()"},
    {"_ZN12_GLOBAL__N_112TailCallElimD0Ev", "(anonymous namespace)::TailCallElim::~TailCallElim()"},
    {"_ZN12_GLOBAL__N_112TailCallElim13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::TailCallElim::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_112TailCallElim16FindTRECandidateEPN4llvm11InstructionEb", "(anonymous namespace)::TailCallElim::FindTRECandidate(llvm::Instruction*, bool)"},
    {"_ZN12_GLOBAL__N_112TailCallElim26EliminateRecursiveTailCallEPN4llvm8CallInstEPNS1_10ReturnInstERPNS1_10BasicBlockERbRNS1_11SmallVectorIPNS1_7PHINodeELj8EEEb", "(anonymous namespace)::TailCallElim::EliminateRecursiveTailCall(llvm::CallInst*, llvm::ReturnInst*, llvm::BasicBlock*&, bool&, llvm::SmallVector<llvm::PHINode*, 8u>&, bool)"},
    {"_Z17isDynamicConstantPN4llvm5ValueEPNS_8CallInstEPNS_10ReturnInstE", "isDynamicConstant(llvm::Value*, llvm::CallInst*, llvm::ReturnInst*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112TailCallElimEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::TailCallElim>()"},
    {"_ZN4llvm12InstCombiner24WillNotOverflowSignedAddEPNS_5ValueES2_", "llvm::InstCombiner::WillNotOverflowSignedAdd(llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner8visitAddERNS_14BinaryOperatorE", "llvm::InstCombiner::visitAdd(llvm::BinaryOperator&)"},
    {"_Z19dyn_castFoldableMulPN4llvm5ValueERPNS_11ConstantIntE", "dyn_castFoldableMul(llvm::Value*, llvm::ConstantInt*&)"},
    {"_ZN4llvm12InstCombiner9visitFAddERNS_14BinaryOperatorE", "llvm::InstCombiner::visitFAdd(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner13EmitGEPOffsetEPNS_4UserE", "llvm::InstCombiner::EmitGEPOffset(llvm::User*)"},
    {"_ZN4llvm12InstCombiner25OptimizePointerDifferenceEPNS_5ValueES2_PKNS_4TypeE", "llvm::InstCombiner::OptimizePointerDifference(llvm::Value*, llvm::Value*, llvm::Type const*)"},
    {"_ZN4llvm12InstCombiner8visitSubERNS_14BinaryOperatorE", "llvm::InstCombiner::visitSub(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitFSubERNS_14BinaryOperatorE", "llvm::InstCombiner::visitFSub(llvm::BinaryOperator&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE12CreateNSWAddEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateNSWAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm12PatternMatch15CastClass_matchINS1_INS0_7bind_tyINS_5ValueEEELj39EEELj30EE5matchIS3_EEbPT_", "bool llvm::PatternMatch::CastClass_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 39u>, 30u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_9neg_matchINS0_7bind_tyINS_5ValueEEEEES5_Lj12EE5matchINS_14BinaryOperatorEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::neg_match<llvm::PatternMatch::bind_ty<llvm::Value> >, llvm::PatternMatch::bind_ty<llvm::Value>, 12u>::match<llvm::BinaryOperator>(llvm::BinaryOperator*)"},
    {"_ZN4llvm12PatternMatch9neg_matchINS0_7bind_tyINS_5ValueEEEE5matchIS3_EEbPT_", "bool llvm::PatternMatch::neg_match<llvm::PatternMatch::bind_ty<llvm::Value> >::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch9neg_matchINS0_7bind_tyINS_5ValueEEEE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::neg_match<llvm::PatternMatch::bind_ty<llvm::Value> >::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_7bind_tyINS_5ValueEEENS0_9neg_matchIS4_EELj12EE5matchINS_14BinaryOperatorEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::bind_ty<llvm::Value>, llvm::PatternMatch::neg_match<llvm::PatternMatch::bind_ty<llvm::Value> >, 12u>::match<llvm::BinaryOperator>(llvm::BinaryOperator*)"},
    {"_ZN4llvm12InstCombiner8OptAndOpEPNS_11InstructionEPNS_11ConstantIntES4_RNS_14BinaryOperatorE", "llvm::InstCombiner::OptAndOp(llvm::Instruction*, llvm::ConstantInt*, llvm::ConstantInt*, llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner15InsertRangeTestEPNS_5ValueEPNS_8ConstantES4_bb", "llvm::InstCombiner::InsertRangeTest(llvm::Value*, llvm::Constant*, llvm::Constant*, bool, bool)"},
    {"_ZN4llvm12InstCombiner18FoldLogicalPlusAndEPNS_5ValueES2_PNS_11ConstantIntEbRNS_11InstructionE", "llvm::InstCombiner::FoldLogicalPlusAnd(llvm::Value*, llvm::Value*, llvm::ConstantInt*, bool, llvm::Instruction&)"},
    {"_ZN4llvm12InstCombiner14FoldAndOfICmpsEPNS_8ICmpInstES2_", "llvm::InstCombiner::FoldAndOfICmps(llvm::ICmpInst*, llvm::ICmpInst*)"},
    {"_Z12getICmpValuebjPN4llvm5ValueES1_PNS_9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEEE", "getICmpValue(bool, unsigned int, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>*)"},
    {"_Z22foldLogOpOfMaskedICmpsPN4llvm8ICmpInstES1_NS_7CmpInst9PredicateEPNS_9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEEE", "foldLogOpOfMaskedICmps(llvm::ICmpInst*, llvm::ICmpInst*, llvm::CmpInst::Predicate, llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>*)"},
    {"_ZN4llvm12InstCombiner14FoldAndOfFCmpsEPNS_8FCmpInstES2_", "llvm::InstCombiner::FoldAndOfFCmps(llvm::FCmpInst*, llvm::FCmpInst*)"},
    {"_ZN4llvm12InstCombiner8visitAndERNS_14BinaryOperatorE", "llvm::InstCombiner::visitAnd(llvm::BinaryOperator&)"},
    {"_Z14dyn_castNotValPN4llvm5ValueE", "dyn_castNotVal(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner10MatchBSwapERNS_14BinaryOperatorE", "llvm::InstCombiner::MatchBSwap(llvm::BinaryOperator&)"},
    {"_Z17CollectBSwapPartsPN4llvm5ValueEijRNS_11SmallVectorIS1_Lj8EEE", "CollectBSwapParts(llvm::Value*, int, unsigned int, llvm::SmallVector<llvm::Value*, 8u>&)"},
    {"_ZN4llvm12InstCombiner13FoldOrOfICmpsEPNS_8ICmpInstES2_", "llvm::InstCombiner::FoldOrOfICmps(llvm::ICmpInst*, llvm::ICmpInst*)"},
    {"_ZN4llvm12InstCombiner13FoldOrOfFCmpsEPNS_8FCmpInstES2_", "llvm::InstCombiner::FoldOrOfFCmps(llvm::FCmpInst*, llvm::FCmpInst*)"},
    {"_ZN4llvm12InstCombiner19FoldOrWithConstantsERNS_14BinaryOperatorEPNS_5ValueES4_S4_S4_", "llvm::InstCombiner::FoldOrWithConstants(llvm::BinaryOperator&, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner7visitOrERNS_14BinaryOperatorE", "llvm::InstCombiner::visitOr(llvm::BinaryOperator&)"},
    {"_Z20MatchSelectFromAndOrPN4llvm5ValueES1_S1_S1_", "MatchSelectFromAndOr(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner8visitXorERNS_14BinaryOperatorE", "llvm::InstCombiner::visitXor(llvm::BinaryOperator&)"},
    {"_Z19getTypeOfMaskedICmpPN4llvm5ValueES1_S1_NS_7CmpInst9PredicateE", "getTypeOfMaskedICmp(llvm::Value*, llvm::Value*, llvm::Value*, llvm::CmpInst::Predicate)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE8CreateOrEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateOr(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateNegEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateNeg(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE20CreateTruncOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateTruncOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_5ValueEE6resizeEj", "llvm::SmallVectorImpl<llvm::Value*>::resize(unsigned int)"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_14specificval_tyEE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::not_match<llvm::PatternMatch::specificval_ty>::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_14BinaryOp_matchINS0_7bind_tyINS_5ValueEEES5_Lj23EEEE10matchIfNotEPS4_S8_", "llvm::PatternMatch::not_match<llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::bind_ty<llvm::Value>, llvm::PatternMatch::bind_ty<llvm::Value>, 23u> >::matchIfNot(llvm::Value*, llvm::Value*)"},
    {"_ZNK4llvm12TargetFolder20CreateTruncOrBitCastEPNS_8ConstantEPKNS_4TypeE", "llvm::TargetFolder::CreateTruncOrBitCast(llvm::Constant*, llvm::Type const*) const"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_15CastClass_matchINS0_14specificval_tyELj32EEEE10matchIfNotEPNS_5ValueES7_", "llvm::PatternMatch::not_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::specificval_ty, 32u> >::matchIfNot(llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm8APIntOps13isShiftedMaskEjRKNS_5APIntE", "llvm::APIntOps::isShiftedMask(unsigned int, llvm::APInt const&)"},
    {"_ZN4llvm12InstCombiner19SimplifyMemTransferEPNS_12MemIntrinsicE", "llvm::InstCombiner::SimplifyMemTransfer(llvm::MemIntrinsic*)"},
    {"_ZN4llvm12InstCombiner14SimplifyMemSetEPNS_10MemSetInstE", "llvm::InstCombiner::SimplifyMemSet(llvm::MemSetInst*)"},
    {"_ZN4llvm12InstCombiner13visitCallInstERNS_8CallInstE", "llvm::InstCombiner::visitCallInst(llvm::CallInst&)"},
    {"_ZN4llvm12InstCombiner13visitCallSiteENS_8CallSiteE", "llvm::InstCombiner::visitCallSite(llvm::CallSite)"},
    {"_ZN4llvm12InstCombiner15visitInvokeInstERNS_10InvokeInstE", "llvm::InstCombiner::visitInvokeInst(llvm::InvokeInst&)"},
    {"_ZN4llvm12InstCombiner15tryOptimizeCallEPNS_8CallInstEPKNS_10DataLayoutE", "llvm::InstCombiner::tryOptimizeCall(llvm::CallInst*, llvm::DataLayout const*)"},
    {"_ZN12_GLOBAL__N_128InstCombineFortifiedLibCallsD1Ev", "(anonymous namespace)::InstCombineFortifiedLibCalls::~InstCombineFortifiedLibCalls()"},
    {"_ZN4llvm12InstCombiner26transformConstExprCastCallENS_8CallSiteE", "llvm::InstCombiner::transformConstExprCastCall(llvm::CallSite)"},
    {"_ZN4llvm12InstCombiner30transformCallThroughTrampolineENS_8CallSiteE", "llvm::InstCombiner::transformCallThroughTrampoline(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_128InstCombineFortifiedLibCalls11replaceCallEPN4llvm5ValueE", "(anonymous namespace)::InstCombineFortifiedLibCalls::replaceCall(llvm::Value*)"},
    {"_ZNK12_GLOBAL__N_128InstCombineFortifiedLibCalls10isFoldableEjjb", "(anonymous namespace)::InstCombineFortifiedLibCalls::isFoldable(unsigned int, unsigned int, bool) const"},
    {"_ZN12_GLOBAL__N_128InstCombineFortifiedLibCallsD0Ev", "(anonymous namespace)::InstCombineFortifiedLibCalls::~InstCombineFortifiedLibCalls()"},
    {"_ZN4llvm12InstCombiner23PromoteCastOfAllocationERNS_11BitCastInstERNS_10AllocaInstE", "llvm::InstCombiner::PromoteCastOfAllocation(llvm::BitCastInst&, llvm::AllocaInst&)"},
    {"_Z25DecomposeSimpleLinearExprPN4llvm5ValueERjRy", "DecomposeSimpleLinearExpr(llvm::Value*, unsigned int&, unsigned long long&)"},
    {"_ZN4llvm12InstCombiner23EvaluateInDifferentTypeEPNS_5ValueEPKNS_4TypeEb", "llvm::InstCombiner::EvaluateInDifferentType(llvm::Value*, llvm::Type const*, bool)"},
    {"_ZN4llvm12InstCombiner18ShouldOptimizeCastENS_11Instruction7CastOpsEPKNS_5ValueEPKNS_4TypeE", "llvm::InstCombiner::ShouldOptimizeCast(llvm::Instruction::CastOps, llvm::Value const*, llvm::Type const*)"},
    {"_Z20isEliminableCastPairPKN4llvm8CastInstEjPKNS_4TypeEPNS_10DataLayoutE", "isEliminableCastPair(llvm::CastInst const*, unsigned int, llvm::Type const*, llvm::DataLayout*)"},
    {"_ZN4llvm12InstCombiner20commonCastTransformsERNS_8CastInstE", "llvm::InstCombiner::commonCastTransforms(llvm::CastInst&)"},
    {"_ZN4llvm12InstCombiner10visitTruncERNS_9TruncInstE", "llvm::InstCombiner::visitTrunc(llvm::TruncInst&)"},
    {"_Z20CanEvaluateTruncatedPN4llvm5ValueEPKNS_4TypeE", "CanEvaluateTruncated(llvm::Value*, llvm::Type const*)"},
    {"_ZN4llvm12InstCombiner17transformZExtICmpEPNS_8ICmpInstERNS_11InstructionEb", "llvm::InstCombiner::transformZExtICmp(llvm::ICmpInst*, llvm::Instruction&, bool)"},
    {"_ZN4llvm12InstCombiner9visitZExtERNS_8ZExtInstE", "llvm::InstCombiner::visitZExt(llvm::ZExtInst&)"},
    {"_Z16CanEvaluateZExtdPN4llvm5ValueEPKNS_4TypeERj", "CanEvaluateZExtd(llvm::Value*, llvm::Type const*, unsigned int&)"},
    {"_ZN4llvm12InstCombiner9visitSExtERNS_8SExtInstE", "llvm::InstCombiner::visitSExt(llvm::SExtInst&)"},
    {"_Z16CanEvaluateSExtdPN4llvm5ValueEPKNS_4TypeE", "CanEvaluateSExtd(llvm::Value*, llvm::Type const*)"},
    {"_ZN4llvm12InstCombiner12visitFPTruncERNS_11FPTruncInstE", "llvm::InstCombiner::visitFPTrunc(llvm::FPTruncInst&)"},
    {"_Z23LookThroughFPExtensionsPN4llvm5ValueE", "LookThroughFPExtensions(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner10visitFPExtERNS_8CastInstE", "llvm::InstCombiner::visitFPExt(llvm::CastInst&)"},
    {"_ZN4llvm12InstCombiner11visitFPToUIERNS_10FPToUIInstE", "llvm::InstCombiner::visitFPToUI(llvm::FPToUIInst&)"},
    {"_ZN4llvm12InstCombiner11visitFPToSIERNS_10FPToSIInstE", "llvm::InstCombiner::visitFPToSI(llvm::FPToSIInst&)"},
    {"_ZN4llvm12InstCombiner11visitUIToFPERNS_8CastInstE", "llvm::InstCombiner::visitUIToFP(llvm::CastInst&)"},
    {"_ZN4llvm12InstCombiner11visitSIToFPERNS_8CastInstE", "llvm::InstCombiner::visitSIToFP(llvm::CastInst&)"},
    {"_ZN4llvm12InstCombiner13visitIntToPtrERNS_12IntToPtrInstE", "llvm::InstCombiner::visitIntToPtr(llvm::IntToPtrInst&)"},
    {"_ZN4llvm12InstCombiner27commonPointerCastTransformsERNS_8CastInstE", "llvm::InstCombiner::commonPointerCastTransforms(llvm::CastInst&)"},
    {"_ZN4llvm12InstCombiner13visitPtrToIntERNS_12PtrToIntInstE", "llvm::InstCombiner::visitPtrToInt(llvm::PtrToIntInst&)"},
    {"_ZN4llvm12InstCombiner12visitBitCastERNS_11BitCastInstE", "llvm::InstCombiner::visitBitCast(llvm::BitCastInst&)"},
    {"_Z24CollectInsertionElementsPN4llvm5ValueEjRNS_15SmallVectorImplIS1_EEPKNS_4TypeE", "CollectInsertionElements(llvm::Value*, unsigned int, llvm::SmallVectorImpl<llvm::Value*>&, llvm::Type const*)"},
    {"_Z12FitsInFPTypePN4llvm10ConstantFPERKNS_12fltSemanticsE", "FitsInFPType(llvm::ConstantFP*, llvm::fltSemantics const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateAndEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateAnd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateLShrEPNS_5ValueES5_RKNS_5TwineEb", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateLShr(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateXorEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateXor(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateShlEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateShl(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateAShrEPNS_5ValueES5_RKNS_5TwineEb", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateAShr(llvm::Value*, llvm::Value*, llvm::Twine const&, bool)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateNotEPNS_5ValueERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateNot(llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE20CreateExtractElementEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateExtractElement(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm11SmallVectorIPNS_5ValueELj8EEC2EjRKS2_", "llvm::SmallVector<llvm::Value*, 8u>::SmallVector(unsigned int, llvm::Value* const&)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS1_INS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj30EEENS3_INS_11ConstantIntEEELj20EEES8_Lj22EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 30u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 20u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 22u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj30EEENS3_INS_11ConstantIntEEELj20EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 30u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 20u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj30EEENS3_INS_11ConstantIntEEELj20EE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 30u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 20u>::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_7bind_tyINS_5ValueEEEE5matchINS_14BinaryOperatorEEEbPT_", "bool llvm::PatternMatch::not_match<llvm::PatternMatch::bind_ty<llvm::Value> >::match<llvm::BinaryOperator>(llvm::BinaryOperator*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj31EEENS3_INS_11ConstantIntEEELj21EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 31u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 21u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE19CreateInsertElementEPNS_5ValueES5_S5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateInsertElement(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj41EEENS3_INS_11ConstantIntEEELj21EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 41u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 21u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj41EEENS3_INS_11ConstantIntEEELj21EE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 41u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 21u>::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch15CastClass_matchINS1_INS0_7bind_tyINS_5ValueEEELj41EEELj30EE5matchIS3_EEbPT_", "bool llvm::PatternMatch::CastClass_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 41u>, 30u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner28FoldCmpLoadFromIndexedGlobalEPNS_17GetElementPtrInstEPNS_14GlobalVariableERNS_7CmpInstEPNS_11ConstantIntE", "llvm::InstCombiner::FoldCmpLoadFromIndexedGlobal(llvm::GetElementPtrInst*, llvm::GlobalVariable*, llvm::CmpInst&, llvm::ConstantInt*)"},
    {"_ZN4llvm12InstCombiner11FoldGEPICmpEPNS_11GEPOperatorEPNS_5ValueENS_7CmpInst9PredicateERNS_11InstructionE", "llvm::InstCombiner::FoldGEPICmp(llvm::GEPOperator*, llvm::Value*, llvm::CmpInst::Predicate, llvm::Instruction&)"},
    {"_ZN4llvm12InstCombiner16FoldICmpAddOpCstERNS_8ICmpInstEPNS_5ValueEPNS_11ConstantIntENS_7CmpInst9PredicateES4_", "llvm::InstCombiner::FoldICmpAddOpCst(llvm::ICmpInst&, llvm::Value*, llvm::ConstantInt*, llvm::CmpInst::Predicate, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner14FoldICmpDivCstERNS_8ICmpInstEPNS_14BinaryOperatorEPNS_11ConstantIntE", "llvm::InstCombiner::FoldICmpDivCst(llvm::ICmpInst&, llvm::BinaryOperator*, llvm::ConstantInt*)"},
    {"_Z15AddWithOverflowRPN4llvm8ConstantES1_S1_b", "AddWithOverflow(llvm::Constant*&, llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm12InstCombiner30visitICmpInstWithInstAndIntCstERNS_8ICmpInstEPNS_11InstructionEPNS_11ConstantIntE", "llvm::InstCombiner::visitICmpInstWithInstAndIntCst(llvm::ICmpInst&, llvm::Instruction*, llvm::ConstantInt*)"},
    {"_Z14isSignBitCheckN4llvm7CmpInst9PredicateEPNS_11ConstantIntERb", "isSignBitCheck(llvm::CmpInst::Predicate, llvm::ConstantInt*, bool&)"},
    {"_ZN4llvm12InstCombiner28visitICmpInstWithCastAndCastERNS_8ICmpInstE", "llvm::InstCombiner::visitICmpInstWithCastAndCast(llvm::ICmpInst&)"},
    {"_ZN4llvm12InstCombiner13visitICmpInstERNS_8ICmpInstE", "llvm::InstCombiner::visitICmpInst(llvm::ICmpInst&)"},
    {"_Z38ComputeSignedMinMaxValuesFromKnownBitsRKN4llvm5APIntES2_RS0_S3_", "ComputeSignedMinMaxValuesFromKnownBits(llvm::APInt const&, llvm::APInt const&, llvm::APInt&, llvm::APInt&)"},
    {"_Z16ProcessUAddIdiomRN4llvm11InstructionEPNS_5ValueERNS_12InstCombinerE", "ProcessUAddIdiom(llvm::Instruction&, llvm::Value*, llvm::InstCombiner&)"},
    {"_ZN4llvm12InstCombiner20FoldFCmp_IntToFP_CstERNS_8FCmpInstEPNS_11InstructionEPNS_8ConstantE", "llvm::InstCombiner::FoldFCmp_IntToFP_Cst(llvm::FCmpInst&, llvm::Instruction*, llvm::Constant*)"},
    {"_ZN4llvm12InstCombiner13visitFCmpInstERNS_8FCmpInstE", "llvm::InstCombiner::visitFCmpInst(llvm::FCmpInst&)"},
    {"_Z14HasAddOverflowPN4llvm11ConstantIntES1_S1_b", "HasAddOverflow(llvm::ConstantInt*, llvm::ConstantInt*, llvm::ConstantInt*, bool)"},
    {"_ZNK4llvm5APInt3sgtEy", "llvm::APInt::sgt(unsigned long long) const"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS1_INS0_7bind_tyINS_5ValueEEES4_Lj8EEENS2_INS_11ConstantIntEEELj8EE5matchIS3_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::bind_ty<llvm::Value>, llvm::PatternMatch::bind_ty<llvm::Value>, 8u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 8u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_15CastClass_matchINS0_7bind_tyINS_5ValueEEELj39EEES6_Lj24EE5matchINS_11InstructionEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 39u>, llvm::PatternMatch::CastClass_match<llvm::PatternMatch::bind_ty<llvm::Value>, 39u>, 24u>::match<llvm::Instruction>(llvm::Instruction*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE18CreateExtractValueEPNS_5ValueEjRKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateExtractValue(llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm12InstCombiner15visitAllocaInstERNS_10AllocaInstE", "llvm::InstCombiner::visitAllocaInst(llvm::AllocaInst&)"},
    {"_ZN4llvm12InstCombiner13visitLoadInstERNS_8LoadInstE", "llvm::InstCombiner::visitLoadInst(llvm::LoadInst&)"},
    {"_Z19InstCombineLoadCastRN4llvm12InstCombinerERNS_8LoadInstEPKNS_10DataLayoutE", "InstCombineLoadCast(llvm::InstCombiner&, llvm::LoadInst&, llvm::DataLayout const*)"},
    {"_ZN4llvm12InstCombiner20hasOneUsePlusDeclareEPNS_5ValueE", "llvm::InstCombiner::hasOneUsePlusDeclare(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner14visitStoreInstERNS_9StoreInstE", "llvm::InstCombiner::visitStoreInst(llvm::StoreInst&)"},
    {"_Z22InstCombineStoreToCastRN4llvm12InstCombinerERNS_9StoreInstE", "InstCombineStoreToCast(llvm::InstCombiner&, llvm::StoreInst&)"},
    {"_ZN4llvm12InstCombiner25SimplifyStoreAtEndOfBlockERNS_9StoreInstE", "llvm::InstCombiner::SimplifyStoreAtEndOfBlock(llvm::StoreInst&)"},
    {"_ZN4llvm12InstCombiner8visitMulERNS_14BinaryOperatorE", "llvm::InstCombiner::visitMul(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitFMulERNS_14BinaryOperatorE", "llvm::InstCombiner::visitFMul(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner22SimplifyDivRemOfSelectERNS_14BinaryOperatorE", "llvm::InstCombiner::SimplifyDivRemOfSelect(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner20commonIDivTransformsERNS_14BinaryOperatorE", "llvm::InstCombiner::commonIDivTransforms(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitUDivERNS_14BinaryOperatorE", "llvm::InstCombiner::visitUDiv(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitSDivERNS_14BinaryOperatorE", "llvm::InstCombiner::visitSDiv(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitFDivERNS_14BinaryOperatorE", "llvm::InstCombiner::visitFDiv(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner19commonRemTransformsERNS_14BinaryOperatorE", "llvm::InstCombiner::commonRemTransforms(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner20commonIRemTransformsERNS_14BinaryOperatorE", "llvm::InstCombiner::commonIRemTransforms(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitURemERNS_14BinaryOperatorE", "llvm::InstCombiner::visitURem(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitSRemERNS_14BinaryOperatorE", "llvm::InstCombiner::visitSRem(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitFRemERNS_14BinaryOperatorE", "llvm::InstCombiner::visitFRem(llvm::BinaryOperator&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateURemEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateURem(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateSRemEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateSRem(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateSubEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm19InstCombineWorklist8AddValueEPNS_5ValueE", "llvm::InstCombineWorklist::AddValue(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_11cst_pred_tyINS0_6is_oneEEENS0_7bind_tyINS_5ValueEEELj20EE5matchIS6_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_one>, llvm::PatternMatch::bind_ty<llvm::Value>, 20u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner22FoldPHIArgBinOpIntoPHIERNS_7PHINodeE", "llvm::InstCombiner::FoldPHIArgBinOpIntoPHI(llvm::PHINode&)"},
    {"_ZN4llvm12InstCombiner20FoldPHIArgGEPIntoPHIERNS_7PHINodeE", "llvm::InstCombiner::FoldPHIArgGEPIntoPHI(llvm::PHINode&)"},
    {"_ZN4llvm12InstCombiner21FoldPHIArgLoadIntoPHIERNS_7PHINodeE", "llvm::InstCombiner::FoldPHIArgLoadIntoPHI(llvm::PHINode&)"},
    {"_Z29isSafeAndProfitableToSinkLoadPN4llvm8LoadInstE", "isSafeAndProfitableToSinkLoad(llvm::LoadInst*)"},
    {"_ZN4llvm12InstCombiner19FoldPHIArgOpIntoPHIERNS_7PHINodeE", "llvm::InstCombiner::FoldPHIArgOpIntoPHI(llvm::PHINode&)"},
    {"_ZN4llvm12InstCombiner24SliceUpIllegalIntegerPHIERNS_7PHINodeE", "llvm::InstCombiner::SliceUpIllegalIntegerPHI(llvm::PHINode&)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_114PHIUsageRecordEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::PHIUsageRecord>::push_back((anonymous namespace)::PHIUsageRecord const&)"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_116LoweredPHIRecordEPNS_7PHINodeENS_12DenseMapInfoIS2_EENS5_IS4_EEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::LoweredPHIRecord, llvm::PHINode*, llvm::DenseMapInfo<(anonymous namespace)::LoweredPHIRecord>, llvm::DenseMapInfo<llvm::PHINode*> >::operator[]((anonymous namespace)::LoweredPHIRecord const&)"},
    {"_ZN4llvm12InstCombiner12visitPHINodeERNS_7PHINodeE", "llvm::InstCombiner::visitPHINode(llvm::PHINode&)"},
    {"_Z14PHIsEqualValuePN4llvm7PHINodeEPNS_5ValueERNS_11SmallPtrSetIS1_Lj16EEE", "PHIsEqualValue(llvm::PHINode*, llvm::Value*, llvm::SmallPtrSet<llvm::PHINode*, 16u>&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_116LoweredPHIRecordEPNS_7PHINodeENS_12DenseMapInfoIS2_EENS5_IS4_EEE15LookupBucketForERKS2_RPSt4pairIS2_S4_E", "llvm::DenseMap<(anonymous namespace)::LoweredPHIRecord, llvm::PHINode*, llvm::DenseMapInfo<(anonymous namespace)::LoweredPHIRecord>, llvm::DenseMapInfo<llvm::PHINode*> >::LookupBucketFor((anonymous namespace)::LoweredPHIRecord const&, std::pair<(anonymous namespace)::LoweredPHIRecord, llvm::PHINode*>*&) const"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_114PHIUsageRecordEEEiPKvS4_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::PHIUsageRecord>(void const*, void const*)"},
    {"_ZSt6__findIPPN4llvm7PHINodeES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "llvm::PHINode** std::__find<llvm::PHINode**, llvm::PHINode*>(llvm::PHINode**, llvm::PHINode**, llvm::PHINode* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm11SmallVectorIPNS_7PHINodeELj16EEC2EjRKS2_", "llvm::SmallVector<llvm::PHINode*, 16u>::SmallVector(unsigned int, llvm::PHINode* const&)"},
    {"_ZN4llvm12InstCombiner14FoldSelectOpOpERNS_10SelectInstEPNS_11InstructionES4_", "llvm::InstCombiner::FoldSelectOpOp(llvm::SelectInst&, llvm::Instruction*, llvm::Instruction*)"},
    {"_ZN4llvm12InstCombiner16FoldSelectIntoOpERNS_10SelectInstEPNS_5ValueES4_", "llvm::InstCombiner::FoldSelectIntoOp(llvm::SelectInst&, llvm::Value*, llvm::Value*)"},
    {"_Z25GetSelectFoldableConstantPN4llvm11InstructionE", "GetSelectFoldableConstant(llvm::Instruction*)"},
    {"_Z10isSelect01PN4llvm8ConstantES1_", "isSelect01(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12InstCombiner23visitSelectInstWithICmpERNS_10SelectInstEPNS_8ICmpInstE", "llvm::InstCombiner::visitSelectInstWithICmp(llvm::SelectInst&, llvm::ICmpInst*)"},
    {"_ZN4llvm12InstCombiner12FoldSPFofSPFEPNS_11InstructionENS_19SelectPatternFlavorEPNS_5ValueES5_RS1_S3_S5_", "llvm::InstCombiner::FoldSPFofSPF(llvm::Instruction*, llvm::SelectPatternFlavor, llvm::Value*, llvm::Value*, llvm::Instruction&, llvm::SelectPatternFlavor, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner15visitSelectInstERNS_10SelectInstE", "llvm::InstCombiner::visitSelectInst(llvm::SelectInst&)"},
    {"_ZN4llvm12InstCombiner21commonShiftTransformsERNS_14BinaryOperatorE", "llvm::InstCombiner::commonShiftTransforms(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner19FoldShiftByConstantEPNS_5ValueEPNS_11ConstantIntERNS_14BinaryOperatorE", "llvm::InstCombiner::FoldShiftByConstant(llvm::Value*, llvm::ConstantInt*, llvm::BinaryOperator&)"},
    {"_Z18CanEvaluateShiftedPN4llvm5ValueEjbRNS_12InstCombinerE", "CanEvaluateShifted(llvm::Value*, unsigned int, bool, llvm::InstCombiner&)"},
    {"_Z15GetShiftedValuePN4llvm5ValueEjbRNS_12InstCombinerE", "GetShiftedValue(llvm::Value*, unsigned int, bool, llvm::InstCombiner&)"},
    {"_ZN4llvm12InstCombiner8visitShlERNS_14BinaryOperatorE", "llvm::InstCombiner::visitShl(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitLShrERNS_14BinaryOperatorE", "llvm::InstCombiner::visitLShr(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner9visitAShrERNS_14BinaryOperatorE", "llvm::InstCombiner::visitAShr(llvm::BinaryOperator&)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_12BinOp2_matchINS0_7bind_tyINS_5ValueEEES5_Lj21ELj22EEENS3_INS_11ConstantIntEEELj23EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::BinOp2_match<llvm::PatternMatch::bind_ty<llvm::Value>, llvm::PatternMatch::bind_ty<llvm::Value>, 21u, 22u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 23u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_12BinOp2_matchINS0_7bind_tyINS_5ValueEEENS0_14specificval_tyELj21ELj22EEENS3_INS_11ConstantIntEEELj23EE5matchIS4_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::BinOp2_match<llvm::PatternMatch::bind_ty<llvm::Value>, llvm::PatternMatch::specificval_ty, 21u, 22u>, llvm::PatternMatch::bind_ty<llvm::ConstantInt>, 23u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12InstCombiner31SimplifyDemandedInstructionBitsERNS_11InstructionE", "llvm::InstCombiner::SimplifyDemandedInstructionBits(llvm::Instruction&)"},
    {"_ZN4llvm12InstCombiner23SimplifyDemandedUseBitsEPNS_5ValueENS_5APIntERS3_S4_j", "llvm::InstCombiner::SimplifyDemandedUseBits(llvm::Value*, llvm::APInt, llvm::APInt&, llvm::APInt&, unsigned int)"},
    {"_ZN4llvm12InstCombiner20SimplifyDemandedBitsERNS_3UseENS_5APIntERS3_S4_j", "llvm::InstCombiner::SimplifyDemandedBits(llvm::Use&, llvm::APInt, llvm::APInt&, llvm::APInt&, unsigned int)"},
    {"_Z22ShrinkDemandedConstantPN4llvm11InstructionEjNS_5APIntE", "ShrinkDemandedConstant(llvm::Instruction*, unsigned int, llvm::APInt)"},
    {"_ZN4llvm12InstCombiner26SimplifyDemandedVectorEltsEPNS_5ValueENS_5APIntERS3_j", "llvm::InstCombiner::SimplifyDemandedVectorElts(llvm::Value*, llvm::APInt, llvm::APInt&, unsigned int)"},
    {"_ZN4llvm12InstCombiner23visitExtractElementInstERNS_18ExtractElementInstE", "llvm::InstCombiner::visitExtractElementInst(llvm::ExtractElementInst&)"},
    {"_Z17FindScalarElementPN4llvm5ValueEj", "FindScalarElement(llvm::Value*, unsigned int)"},
    {"_Z16CheapToScalarizePN4llvm5ValueEb", "CheapToScalarize(llvm::Value*, bool)"},
    {"_Z14getShuffleMaskPKN4llvm17ShuffleVectorInstE", "getShuffleMask(llvm::ShuffleVectorInst const*)"},
    {"_ZN4llvm12InstCombiner22visitInsertElementInstERNS_17InsertElementInstE", "llvm::InstCombiner::visitInsertElementInst(llvm::InsertElementInst&)"},
    {"_Z22CollectShuffleElementsPN4llvm5ValueERSt6vectorIPNS_8ConstantESaIS4_EERS1_", "CollectShuffleElements(llvm::Value*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >&, llvm::Value*&)"},
    {"_ZN4llvm12InstCombiner22visitShuffleVectorInstERNS_17ShuffleVectorInstE", "llvm::InstCombiner::visitShuffleVectorInst(llvm::ShuffleVectorInst&)"},
    {"_Z28CollectSingleShuffleElementsPN4llvm5ValueES1_S1_RSt6vectorIPNS_8ConstantESaIS4_EE", "CollectSingleShuffleElements(llvm::Value*, llvm::Value*, llvm::Value*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >&)"},
    {"_ZN4llvm21initializeInstCombineERNS_12PassRegistryE", "llvm::initializeInstCombine(llvm::PassRegistry&)"},
    {"_ZN4llvm26initializeInstCombinerPassERNS_12PassRegistryE", "llvm::initializeInstCombinerPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm12InstCombiner16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::InstCombiner::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm12InstCombiner16ShouldChangeTypeEPKNS_4TypeES3_", "llvm::InstCombiner::ShouldChangeType(llvm::Type const*, llvm::Type const*) const"},
    {"_ZN4llvm12InstCombiner32SimplifyAssociativeOrCommutativeERNS_14BinaryOperatorE", "llvm::InstCombiner::SimplifyAssociativeOrCommutative(llvm::BinaryOperator&)"},
    {"_ZN4llvm12InstCombiner29SimplifyUsingDistributiveLawsERNS_14BinaryOperatorE", "llvm::InstCombiner::SimplifyUsingDistributiveLaws(llvm::BinaryOperator&)"},
    {"_ZNK4llvm12InstCombiner14dyn_castNegValEPNS_5ValueE", "llvm::InstCombiner::dyn_castNegVal(llvm::Value*) const"},
    {"_ZNK4llvm12InstCombiner15dyn_castFNegValEPNS_5ValueE", "llvm::InstCombiner::dyn_castFNegVal(llvm::Value*) const"},
    {"_ZN4llvm12InstCombiner16FoldOpIntoSelectERNS_11InstructionEPNS_10SelectInstE", "llvm::InstCombiner::FoldOpIntoSelect(llvm::Instruction&, llvm::SelectInst*)"},
    {"_Z30FoldOperationIntoSelectOperandRN4llvm11InstructionEPNS_5ValueEPNS_12InstCombinerE", "FoldOperationIntoSelectOperand(llvm::Instruction&, llvm::Value*, llvm::InstCombiner*)"},
    {"_ZN4llvm12InstCombiner13FoldOpIntoPhiERNS_11InstructionE", "llvm::InstCombiner::FoldOpIntoPhi(llvm::Instruction&)"},
    {"_ZN4llvm12InstCombiner19FindElementAtOffsetEPKNS_4TypeExRNS_15SmallVectorImplIPNS_5ValueEEE", "llvm::InstCombiner::FindElementAtOffset(llvm::Type const*, long long, llvm::SmallVectorImpl<llvm::Value*>&)"},
    {"_ZN4llvm12InstCombiner22visitGetElementPtrInstERNS_17GetElementPtrInstE", "llvm::InstCombiner::visitGetElementPtrInst(llvm::GetElementPtrInst&)"},
    {"_ZN4llvm12InstCombiner11visitMallocERNS_11InstructionE", "llvm::InstCombiner::visitMalloc(llvm::Instruction&)"},
    {"_ZN4llvm12InstCombiner9visitFreeERNS_8CallInstE", "llvm::InstCombiner::visitFree(llvm::CallInst&)"},
    {"_ZN4llvm12InstCombiner15visitBranchInstERNS_10BranchInstE", "llvm::InstCombiner::visitBranchInst(llvm::BranchInst&)"},
    {"_ZN4llvm12InstCombiner15visitSwitchInstERNS_10SwitchInstE", "llvm::InstCombiner::visitSwitchInst(llvm::SwitchInst&)"},
    {"_ZN4llvm12InstCombiner21visitExtractValueInstERNS_16ExtractValueInstE", "llvm::InstCombiner::visitExtractValueInst(llvm::ExtractValueInst&)"},
    {"_ZN4llvm12InstCombiner14DoOneIterationERNS_8FunctionEj", "llvm::InstCombiner::DoOneIteration(llvm::Function&, unsigned int)"},
    {"_ZN4llvm12InstCombiner13runOnFunctionERNS_8FunctionE", "llvm::InstCombiner::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm30createInstructionCombiningPassEv", "llvm::createInstructionCombiningPass()"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE11CreateBinOpENS_11Instruction9BinaryOpsEPNS_5ValueES7_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateBinOp(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm12InstCombiner19InsertNewInstBeforeEPNS_11InstructionERS1_", "llvm::InstCombiner::InsertNewInstBefore(llvm::Instruction*, llvm::Instruction&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE12CreateSelectEPNS_5ValueES5_S5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateSelect(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateICmpENS_7CmpInst9PredicateEPNS_5ValueES7_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateICmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateFCmpENS_7CmpInst9PredicateEPNS_5ValueES7_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateFCmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE10CreateCastENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateCast(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm12InstCombiner19ReplaceInstUsesWithERNS_11InstructionEPNS_5ValueE", "llvm::InstCombiner::ReplaceInstUsesWith(llvm::Instruction&, llvm::Value*)"},
    {"_ZN4llvm12InstCombiner21EraseInstFromFunctionERNS_11InstructionE", "llvm::InstCombiner::EraseInstFromFunction(llvm::Instruction&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE13CreateIntCastEPNS_5ValueEPKNS_4TypeEbRKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateIntCast(llvm::Value*, llvm::Type const*, bool, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateAddEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE17CreateInBoundsGEPIPPNS_5ValueEEES6_S6_T_S8_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateInBoundsGEP<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateGEPIPPNS_5ValueEEES6_S6_T_S8_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateGEP<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE9CreateMulEPNS_5ValueES5_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_21InstCombineIRInserterEE18CreateExtractValueIPKjEEPNS_5ValueES8_T_S9_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<true, llvm::TargetFolder, llvm::InstCombineIRInserter>::CreateExtractValue<unsigned int const*>(llvm::Value*, unsigned int const*, unsigned int const*, llvm::Twine const&)"},
    {"_ZN4llvm11InstVisitorINS_12InstCombinerEPNS_11InstructionEE5visitERS2_", "llvm::InstVisitor<llvm::InstCombiner, llvm::Instruction*>::visit(llvm::Instruction&)"},
    {"_ZN4llvm19InstCombineWorklist18AddUsersToWorkListERNS_11InstructionE", "llvm::InstCombineWorklist::AddUsersToWorkList(llvm::Instruction&)"},
    {"_ZN4llvm12InstCombinerD1Ev", "llvm::InstCombiner::~InstCombiner()"},
    {"_ZN4llvm12InstCombinerD0Ev", "llvm::InstCombiner::~InstCombiner()"},
    {"_ZNK4llvm21InstCombineIRInserter12InsertHelperEPNS_11InstructionERKNS_5TwineEPNS_10BasicBlockENS_14ilist_iteratorIS1_EE", "llvm::InstCombineIRInserter::InsertHelper(llvm::Instruction*, llvm::Twine const&, llvm::BasicBlock*, llvm::ilist_iterator<llvm::Instruction>) const"},
    {"_ZN4llvm15InsertValueInstC2IPKjEEPNS_5ValueES5_T_S6_RKNS_5TwineEPNS_11InstructionE", "llvm::InsertValueInst::InsertValueInst<unsigned int const*>(llvm::Value*, llvm::Value*, unsigned int const*, unsigned int const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm16ExtractValueInstC2IPKjEEPNS_5ValueET_S6_RKNS_5TwineEPNS_11InstructionE", "llvm::ExtractValueInst::ExtractValueInst<unsigned int const*>(llvm::Value*, unsigned int const*, unsigned int const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_7bind_tyINS_5ValueEEEE5matchIS3_EEbPT_", "bool llvm::PatternMatch::not_match<llvm::PatternMatch::bind_ty<llvm::Value> >::match<llvm::Value>(llvm::Value*)"},
    {"_ZNK4llvm12TargetFolder13CreateIntCastEPNS_8ConstantEPKNS_4TypeEb", "llvm::TargetFolder::CreateIntCast(llvm::Constant*, llvm::Type const*, bool) const"},
    {"_ZN4llvm12InstCombinerC2Ev", "llvm::InstCombiner::InstCombiner()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, unsigned int, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm19InstCombineWorklist15AddInitialGroupEPKPNS_11InstructionEj", "llvm::InstCombineWorklist::AddInitialGroup(llvm::Instruction* const*, unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEjNS_12DenseMapInfoIS2_EENS3_IjEEE6insertERKSt4pairIS2_jE", "llvm::DenseMap<llvm::Instruction*, unsigned int, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<unsigned int> >::insert(std::pair<llvm::Instruction*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::Instruction*, unsigned int, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Instruction* const&, unsigned int const&, std::pair<llvm::Instruction*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::Instruction*, unsigned int, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm15callDefaultCtorINS_12InstCombinerEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::InstCombiner>()"},
    {"_ZNK4llvm11ExtAddrMode5printERNS_11raw_ostreamE", "llvm::ExtAddrMode::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm11ExtAddrMode4dumpEv", "llvm::ExtAddrMode::dump() const"},
    {"_ZN4llvm21AddressingModeMatcher16MatchScaledValueEPNS_5ValueExj", "llvm::AddressingModeMatcher::MatchScaledValue(llvm::Value*, long long, unsigned int)"},
    {"_ZN4llvm21AddressingModeMatcher9MatchAddrEPNS_5ValueEj", "llvm::AddressingModeMatcher::MatchAddr(llvm::Value*, unsigned int)"},
    {"_ZN4llvm21AddressingModeMatcher18MatchOperationAddrEPNS_4UserEjj", "llvm::AddressingModeMatcher::MatchOperationAddr(llvm::User*, unsigned int, unsigned int)"},
    {"_ZN4llvm21AddressingModeMatcher36IsProfitableToFoldIntoAddressingModeEPNS_11InstructionERNS_11ExtAddrModeES4_", "llvm::AddressingModeMatcher::IsProfitableToFoldIntoAddressingMode(llvm::Instruction*, llvm::ExtAddrMode&, llvm::ExtAddrMode&)"},
    {"_ZN4llvm21AddressingModeMatcher22ValueAlreadyLiveAtInstEPNS_5ValueES2_S2_", "llvm::AddressingModeMatcher::ValueAlreadyLiveAtInst(llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_Z17FindAllMemoryUsesPN4llvm11InstructionERNS_15SmallVectorImplISt4pairIS1_jEEERNS_11SmallPtrSetIS1_Lj16EEERKNS_14TargetLoweringE", "FindAllMemoryUses(llvm::Instruction*, llvm::SmallVectorImpl<std::pair<llvm::Instruction*, unsigned int> >&, llvm::SmallPtrSet<llvm::Instruction*, 16u>&, llvm::TargetLowering const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_11InstructionEE6resizeEj", "llvm::SmallVectorImpl<llvm::Instruction*>::resize(unsigned int)"},
    {"_ZSt6__findIPPN4llvm11InstructionES2_ET_S4_S4_RKT0_St26random_access_iterator_tag", "llvm::Instruction** std::__find<llvm::Instruction**, llvm::Instruction*>(llvm::Instruction**, llvm::Instruction**, llvm::Instruction* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm15DeleteDeadBlockEPNS_10BasicBlockE", "llvm::DeleteDeadBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm23FoldSingleEntryPHINodesEPNS_10BasicBlockEPNS_4PassE", "llvm::FoldSingleEntryPHINodes(llvm::BasicBlock*, llvm::Pass*)"},
    {"_ZN4llvm14DeleteDeadPHIsEPNS_10BasicBlockE", "llvm::DeleteDeadPHIs(llvm::BasicBlock*)"},
    {"_ZN4llvm25MergeBlockIntoPredecessorEPNS_10BasicBlockEPNS_4PassE", "llvm::MergeBlockIntoPredecessor(llvm::BasicBlock*, llvm::Pass*)"},
    {"_ZN4llvm20ReplaceInstWithValueERNS_6iplistINS_11InstructionENS_12ilist_traitsIS1_EEEERNS_14ilist_iteratorIS1_EEPNS_5ValueE", "llvm::ReplaceInstWithValue(llvm::iplist<llvm::Instruction, llvm::ilist_traits<llvm::Instruction> >&, llvm::ilist_iterator<llvm::Instruction>&, llvm::Value*)"},
    {"_ZN4llvm19ReplaceInstWithInstERNS_6iplistINS_11InstructionENS_12ilist_traitsIS1_EEEERNS_14ilist_iteratorIS1_EEPS1_", "llvm::ReplaceInstWithInst(llvm::iplist<llvm::Instruction, llvm::ilist_traits<llvm::Instruction> >&, llvm::ilist_iterator<llvm::Instruction>&, llvm::Instruction*)"},
    {"_ZN4llvm19ReplaceInstWithInstEPNS_11InstructionES1_", "llvm::ReplaceInstWithInst(llvm::Instruction*, llvm::Instruction*)"},
    {"_ZN4llvm18GetSuccessorNumberEPNS_10BasicBlockES1_", "llvm::GetSuccessorNumber(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm9SplitEdgeEPNS_10BasicBlockES1_PNS_4PassE", "llvm::SplitEdge(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Pass*)"},
    {"_ZN4llvm10SplitBlockEPNS_10BasicBlockEPNS_11InstructionEPNS_4PassE", "llvm::SplitBlock(llvm::BasicBlock*, llvm::Instruction*, llvm::Pass*)"},
    {"_ZN4llvm22SplitBlockPredecessorsEPNS_10BasicBlockEPKS1_jPKcPNS_4PassE", "llvm::SplitBlockPredecessors(llvm::BasicBlock*, llvm::BasicBlock* const*, unsigned int, char const*, llvm::Pass*)"},
    {"_ZN4llvm21FindFunctionBackedgesERKNS_8FunctionERNS_15SmallVectorImplISt4pairIPKNS_10BasicBlockES7_EEE", "llvm::FindFunctionBackedges(llvm::Function const&, llvm::SmallVectorImpl<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >&)"},
    {"_ZN4llvm26FoldReturnIntoUncondBranchEPNS_10ReturnInstEPNS_10BasicBlockES3_", "llvm::FoldReturnIntoUncondBranch(llvm::ReturnInst*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE19addBasicBlockToLoopEPS1_RNS_12LoopInfoBaseIS1_S2_EE", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::addBasicBlockToLoop(llvm::BasicBlock*, llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPKNS_10BasicBlockENS_12SuccIteratorIPKNS_14TerminatorInstES3_EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::BasicBlock const*, llvm::SuccIterator<llvm::TerminatorInst const*, llvm::BasicBlock const> >, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_4LoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::Loop*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Loop*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::Loop* const&, std::pair<llvm::BasicBlock*, llvm::Loop*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_4LoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::Loop*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Loop*> >::grow(unsigned int)"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE11removeBlockEPS1_", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::removeBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE19removeBlockFromLoopEPS1_", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::removeBlockFromLoop(llvm::BasicBlock*)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm10BasicBlockESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock*>(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, __gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::BasicBlock* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm32initializeBreakCriticalEdgesPassERNS_12PassRegistryE", "llvm::initializeBreakCriticalEdgesPass(llvm::PassRegistry&)"},
    {"_ZN4llvm28createBreakCriticalEdgesPassEv", "llvm::createBreakCriticalEdgesPass()"},
    {"_ZN4llvm14isCriticalEdgeEPKNS_14TerminatorInstEjb", "llvm::isCriticalEdge(llvm::TerminatorInst const*, unsigned int, bool)"},
    {"_ZN4llvm17SplitCriticalEdgeEPNS_14TerminatorInstEjPNS_4PassEb", "llvm::SplitCriticalEdge(llvm::TerminatorInst*, unsigned int, llvm::Pass*, bool)"},
    {"_Z26CreatePHIsForSplitLoopExitRN4llvm15SmallVectorImplIPNS_10BasicBlockEEES2_S2_", "CreatePHIsForSplitLoopExit(llvm::SmallVectorImpl<llvm::BasicBlock*>&, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_118BreakCriticalEdgesD1Ev", "(anonymous namespace)::BreakCriticalEdges::~BreakCriticalEdges()"},
    {"_ZN12_GLOBAL__N_118BreakCriticalEdgesD0Ev", "(anonymous namespace)::BreakCriticalEdges::~BreakCriticalEdges()"},
    {"_ZNK12_GLOBAL__N_118BreakCriticalEdges16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::BreakCriticalEdges::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118BreakCriticalEdges13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::BreakCriticalEdges::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_118BreakCriticalEdgesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::BreakCriticalEdges>()"},
    {"_ZN4llvm10CastToCStrEPNS_5ValueERNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEE", "llvm::CastToCStr(llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&)"},
    {"_ZN4llvm10EmitStrLenEPNS_5ValueERNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitStrLen(llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm10EmitStrChrEPNS_5ValueEcRNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitStrChr(llvm::Value*, char, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm11EmitStrNCmpEPNS_5ValueES1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitStrNCmp(llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm10EmitStrCpyEPNS_5ValueES1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutENS_9StringRefE", "llvm::EmitStrCpy(llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*, llvm::StringRef)"},
    {"_ZN4llvm11EmitStrNCpyEPNS_5ValueES1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutENS_9StringRefE", "llvm::EmitStrNCpy(llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*, llvm::StringRef)"},
    {"_ZN4llvm13EmitMemCpyChkEPNS_5ValueES1_S1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitMemCpyChk(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm10EmitMemChrEPNS_5ValueES1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitMemChr(llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm10EmitMemCmpEPNS_5ValueES1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitMemCmp(llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm20EmitUnaryFloatFnCallEPNS_5ValueEPKcRNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEERKNS_11AttrListPtrE", "llvm::EmitUnaryFloatFnCall(llvm::Value*, char const*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::AttrListPtr const&)"},
    {"_ZN4llvm11EmitPutCharEPNS_5ValueERNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitPutChar(llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm8EmitPutSEPNS_5ValueERNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitPutS(llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm9EmitFPutCEPNS_5ValueES1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitFPutC(llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm9EmitFPutSEPNS_5ValueES1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitFPutS(llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm10EmitFWriteEPNS_5ValueES1_S1_RNS_9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEEEPKNS_10DataLayoutE", "llvm::EmitFWrite(llvm::Value*, llvm::Value*, llvm::Value*, llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >&, llvm::DataLayout const*)"},
    {"_ZN4llvm25SimplifyFortifiedLibCallsD0Ev", "llvm::SimplifyFortifiedLibCalls::~SimplifyFortifiedLibCalls()"},
    {"_ZN4llvm25SimplifyFortifiedLibCallsD1Ev", "llvm::SimplifyFortifiedLibCalls::~SimplifyFortifiedLibCalls()"},
    {"_ZN4llvm25SimplifyFortifiedLibCallsD2Ev", "llvm::SimplifyFortifiedLibCalls::~SimplifyFortifiedLibCalls()"},
    {"_ZN4llvm25SimplifyFortifiedLibCalls4foldEPNS_8CallInstEPKNS_10DataLayoutE", "llvm::SimplifyFortifiedLibCalls::fold(llvm::CallInst*, llvm::DataLayout const*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE11CreateCall3EPNS_5ValueES6_S6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCall3(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE11CreateCall4EPNS_5ValueES6_S6_S6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCall4(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm15CloneBasicBlockEPKNS_10BasicBlockERNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS4_EENS_14ValueMapConfigIS6_EENS_12DenseMapInfoIS8_EEEERKNS_5TwineEPNS_8FunctionEPNS_14ClonedCodeInfoE", "llvm::CloneBasicBlock(llvm::BasicBlock const*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, llvm::Twine const&, llvm::Function*, llvm::ClonedCodeInfo*)"},
    {"_ZN4llvm17CloneFunctionIntoEPNS_8FunctionEPKS0_RNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS5_EENS_14ValueMapConfigIS7_EENS_12DenseMapInfoIS9_EEEEbRNS_15SmallVectorImplIPNS_10ReturnInstEEEPKcPNS_14ClonedCodeInfoE", "llvm::CloneFunctionInto(llvm::Function*, llvm::Function const*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, bool, llvm::SmallVectorImpl<llvm::ReturnInst*>&, char const*, llvm::ClonedCodeInfo*)"},
    {"_ZN4llvm13CloneFunctionEPKNS_8FunctionERNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS4_EENS_14ValueMapConfigIS6_EENS_12DenseMapInfoIS8_EEEEbPNS_14ClonedCodeInfoE", "llvm::CloneFunction(llvm::Function const*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, bool, llvm::ClonedCodeInfo*)"},
    {"_ZN4llvm25CloneAndPruneFunctionIntoEPNS_8FunctionEPKS0_RNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS5_EENS_14ValueMapConfigIS7_EENS_12DenseMapInfoIS9_EEEEbRNS_15SmallVectorImplIPNS_10ReturnInstEEEPKcPNS_14ClonedCodeInfoEPKNS_10DataLayoutEPNS_11InstructionE", "llvm::CloneAndPruneFunctionInto(llvm::Function*, llvm::Function const*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, bool, llvm::SmallVectorImpl<llvm::ReturnInst*>&, char const*, llvm::ClonedCodeInfo*, llvm::DataLayout const*, llvm::Instruction*)"},
    {"_Z19UpdateInlinedAtInfoRKN4llvm8DebugLocES2_RNS_11LLVMContextE", "UpdateInlinedAtInfo(llvm::DebugLoc const&, llvm::DebugLoc const&, llvm::LLVMContext&)"},
    {"_ZNSt6vectorIPKN4llvm10BasicBlockESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::BasicBlock const**, std::vector<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> > >, llvm::BasicBlock const* const&)"},
    {"_ZN4llvm16DemoteRegToStackERNS_11InstructionEbPS0_", "llvm::DemoteRegToStack(llvm::Instruction&, bool, llvm::Instruction*)"},
    {"_ZN4llvm16DemotePHIToStackEPNS_7PHINodeEPNS_11InstructionE", "llvm::DemotePHIToStack(llvm::PHINode*, llvm::Instruction*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_5ValueEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Value*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Value*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Value*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock* const, llvm::Value*> >, std::pair<llvm::BasicBlock* const, llvm::Value*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_5ValueEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Value*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Value*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Value*> > >::_M_insert_unique(std::pair<llvm::BasicBlock* const, llvm::Value*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_5ValueEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Value*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Value*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Value*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock* const, llvm::Value*> >*)"},
    {"_ZN4llvm14InlineFunctionEPNS_8CallInstERNS_18InlineFunctionInfoE", "llvm::InlineFunction(llvm::CallInst*, llvm::InlineFunctionInfo&)"},
    {"_ZN4llvm14InlineFunctionENS_8CallSiteERNS_18InlineFunctionInfoE", "llvm::InlineFunction(llvm::CallSite, llvm::InlineFunctionInfo&)"},
    {"_ZN4llvm14InlineFunctionEPNS_10InvokeInstERNS_18InlineFunctionInfoE", "llvm::InlineFunction(llvm::InvokeInst*, llvm::InlineFunctionInfo&)"},
    {"_ZNSt6vectorISt4pairIN4llvm6WeakVHEPNS1_13CallGraphNodeEESaIS5_EE13_M_assign_auxIN9__gnu_cxx17__normal_iteratorIPS5_S7_EEEEvT_SD_St20forward_iterator_tag", "void std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > >::_M_assign_aux<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::forward_iterator_tag)"},
    {"_ZNSt6vectorISt4pairIN4llvm6WeakVHEPNS1_13CallGraphNodeEESaIS5_EE20_M_allocate_and_copyIN9__gnu_cxx17__normal_iteratorIPS5_S7_EEEESB_mT_SD_", "std::pair<llvm::WeakVH, llvm::CallGraphNode*>* std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > >::_M_allocate_and_copy<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > > >(unsigned long, __gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >)"},
    {"_ZN4llvm19initializeLCSSAPassERNS_12PassRegistryE", "llvm::initializeLCSSAPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15createLCSSAPassEv", "llvm::createLCSSAPass()"},
    {"_ZN12_GLOBAL__N_15LCSSAC1Ev", "(anonymous namespace)::LCSSA::LCSSA()"},
    {"_ZN12_GLOBAL__N_15LCSSAD1Ev", "(anonymous namespace)::LCSSA::~LCSSA()"},
    {"_ZN12_GLOBAL__N_15LCSSAD0Ev", "(anonymous namespace)::LCSSA::~LCSSA()"},
    {"_ZNK12_GLOBAL__N_15LCSSA16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LCSSA::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK12_GLOBAL__N_15LCSSA14verifyAnalysisEv", "(anonymous namespace)::LCSSA::verifyAnalysis() const"},
    {"_ZN12_GLOBAL__N_15LCSSA9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LCSSA::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN4llvm25array_pod_sort_comparatorIPNS_10BasicBlockEEEiPKvS4_", "int llvm::array_pod_sort_comparator<llvm::BasicBlock*>(void const*, void const*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_15LCSSAEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LCSSA>()"},
    {"_ZN4llvm17PredIteratorCache11GetNumPredsEPNS_10BasicBlockE", "llvm::PredIteratorCache::GetNumPreds(llvm::BasicBlock*)"},
    {"_ZN4llvm22ConstantFoldTerminatorEPNS_10BasicBlockE", "llvm::ConstantFoldTerminator(llvm::BasicBlock*)"},
    {"_ZN4llvm26isInstructionTriviallyDeadEPNS_11InstructionE", "llvm::isInstructionTriviallyDead(llvm::Instruction*)"},
    {"_ZN4llvm42RecursivelyDeleteTriviallyDeadInstructionsEPNS_5ValueE", "llvm::RecursivelyDeleteTriviallyDeadInstructions(llvm::Value*)"},
    {"_ZN4llvm28RecursivelyDeleteDeadPHINodeEPNS_7PHINodeE", "llvm::RecursivelyDeleteDeadPHINode(llvm::PHINode*)"},
    {"_ZN4llvm27SimplifyInstructionsInBlockEPNS_10BasicBlockEPKNS_10DataLayoutE", "llvm::SimplifyInstructionsInBlock(llvm::BasicBlock*, llvm::DataLayout const*)"},
    {"_ZN4llvm28RemovePredecessorAndSimplifyEPNS_10BasicBlockES1_PNS_10DataLayoutE", "llvm::RemovePredecessorAndSimplify(llvm::BasicBlock*, llvm::BasicBlock*, llvm::DataLayout*)"},
    {"_ZN4llvm27MergeBasicBlockIntoOnlyPredEPNS_10BasicBlockEPNS_4PassE", "llvm::MergeBasicBlockIntoOnlyPred(llvm::BasicBlock*, llvm::Pass*)"},
    {"_ZN4llvm39TryToSimplifyUncondBranchFromEmptyBlockEPNS_10BasicBlockE", "llvm::TryToSimplifyUncondBranchFromEmptyBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm26EliminateDuplicatePHINodesEPNS_10BasicBlockE", "llvm::EliminateDuplicatePHINodes(llvm::BasicBlock*)"},
    {"_ZN4llvm26getOrEnforceKnownAlignmentEPNS_5ValueEjPKNS_10DataLayoutE", "llvm::getOrEnforceKnownAlignment(llvm::Value*, unsigned int, llvm::DataLayout const*)"},
    {"_ZN4llvm8DenseMapImPNS_7PHINodeENS_12DenseMapInfoImEENS3_IS2_EEE6insertERKSt4pairImS2_E", "llvm::DenseMap<unsigned long, llvm::PHINode*, llvm::DenseMapInfo<unsigned long>, llvm::DenseMapInfo<llvm::PHINode*> >::insert(std::pair<unsigned long, llvm::PHINode*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_7PHINodeES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::PHINode*, llvm::PHINode*, llvm::DenseMapInfo<llvm::PHINode*>, llvm::DenseMapInfo<llvm::PHINode*> >::InsertIntoBucket(llvm::PHINode* const&, llvm::PHINode* const&, std::pair<llvm::PHINode*, llvm::PHINode*>*)"},
    {"_ZN4llvm8DenseMapIPNS_7PHINodeES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::PHINode*, llvm::PHINode*, llvm::DenseMapInfo<llvm::PHINode*>, llvm::DenseMapInfo<llvm::PHINode*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapImPNS_7PHINodeENS_12DenseMapInfoImEENS3_IS2_EEE16InsertIntoBucketERKmRKS2_PSt4pairImS2_E", "llvm::DenseMap<unsigned long, llvm::PHINode*, llvm::DenseMapInfo<unsigned long>, llvm::DenseMapInfo<llvm::PHINode*> >::InsertIntoBucket(unsigned long const&, llvm::PHINode* const&, std::pair<unsigned long, llvm::PHINode*>*)"},
    {"_ZN4llvm8DenseMapImPNS_7PHINodeENS_12DenseMapInfoImEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned long, llvm::PHINode*, llvm::DenseMapInfo<unsigned long>, llvm::DenseMapInfo<llvm::PHINode*> >::grow(unsigned int)"},
    {"_ZN4llvm11SmallPtrSetIPNS_10BasicBlockELj16EEC2INS_12PredIteratorIS1_NS_18value_use_iteratorINS_4UserEEEEEEET_SA_", "llvm::SmallPtrSet<llvm::BasicBlock*, 16u>::SmallPtrSet<llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >(llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >)"},
    {"_ZN4llvm26initializeLoopSimplifyPassERNS_12PassRegistryE", "llvm::initializeLoopSimplifyPass(llvm::PassRegistry&)"},
    {"_ZN4llvm22createLoopSimplifyPassEv", "llvm::createLoopSimplifyPass()"},
    {"_ZN12_GLOBAL__N_112LoopSimplifyD1Ev", "(anonymous namespace)::LoopSimplify::~LoopSimplify()"},
    {"_ZN12_GLOBAL__N_112LoopSimplifyD0Ev", "(anonymous namespace)::LoopSimplify::~LoopSimplify()"},
    {"_ZNK12_GLOBAL__N_112LoopSimplify16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LoopSimplify::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK12_GLOBAL__N_112LoopSimplify14verifyAnalysisEv", "(anonymous namespace)::LoopSimplify::verifyAnalysis() const"},
    {"_ZN12_GLOBAL__N_112LoopSimplify9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::LoopSimplify::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZN12_GLOBAL__N_112LoopSimplify24PlaceSplitBlockCarefullyEPN4llvm10BasicBlockERNS1_15SmallVectorImplIS3_EEPNS1_4LoopE", "(anonymous namespace)::LoopSimplify::PlaceSplitBlockCarefully(llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::BasicBlock*>&, llvm::Loop*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_112LoopSimplifyEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LoopSimplify>()"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE15getNumBackEdgesEv", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::getNumBackEdges() const"},
    {"_ZN4llvm14SmallSetVectorIPNS_10BasicBlockELj8EEC2IPS2_EET_S6_", "llvm::SmallSetVector<llvm::BasicBlock*, 8u>::SmallSetVector<llvm::BasicBlock**>(llvm::BasicBlock**, llvm::BasicBlock**)"},
    {"_ZN4llvm10UnrollLoopEPNS_4LoopEjPNS_8LoopInfoEPNS_13LPPassManagerE", "llvm::UnrollLoop(llvm::Loop*, unsigned int, llvm::LoopInfo*, llvm::LPPassManager*)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS2_EENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5beginEv", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >, llvm::TrackingVH<llvm::Value>, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > > >, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >::begin()"},
    {"_ZNSt6vectorIPN4llvm7PHINodeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::PHINode*, std::allocator<llvm::PHINode*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::PHINode**, std::vector<llvm::PHINode*, std::allocator<llvm::PHINode*> > >, llvm::PHINode* const&)"},
    {"_ZN4llvm25initializeLowerInvokePassERNS_12PassRegistryE", "llvm::initializeLowerInvokePass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createLowerInvokePassEPKNS_14TargetLoweringE", "llvm::createLowerInvokePass(llvm::TargetLowering const*)"},
    {"_ZN4llvm21createLowerInvokePassEPKNS_14TargetLoweringEb", "llvm::createLowerInvokePass(llvm::TargetLowering const*, bool)"},
    {"_ZN12_GLOBAL__N_111LowerInvokeD1Ev", "(anonymous namespace)::LowerInvoke::~LowerInvoke()"},
    {"_ZN12_GLOBAL__N_111LowerInvokeD0Ev", "(anonymous namespace)::LowerInvoke::~LowerInvoke()"},
    {"_ZNK12_GLOBAL__N_111LowerInvoke16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LowerInvoke::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111LowerInvoke16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::LowerInvoke::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_111LowerInvoke13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::LowerInvoke::runOnFunction(llvm::Function&)"},
    {"_Z16MarkBlocksLiveInPN4llvm10BasicBlockERSt3setIS1_St4lessIS1_ESaIS1_EE", "MarkBlocksLiveIn(llvm::BasicBlock*, std::set<llvm::BasicBlock*, std::less<llvm::BasicBlock*>, std::allocator<llvm::BasicBlock*> >&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111LowerInvokeEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LowerInvoke>()"},
    {"_ZN4llvm17GetElementPtrInstC2IPPNS_5ValueEEES3_T_S5_jRKNS_5TwineEPNS_10BasicBlockE", "llvm::GetElementPtrInst::GetElementPtrInst<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, unsigned int, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10InvokeInstES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::InvokeInst*, llvm::InvokeInst*, std::_Identity<llvm::InvokeInst*>, std::less<llvm::InvokeInst*>, std::allocator<llvm::InvokeInst*> >::_M_erase(std::_Rb_tree_node<llvm::InvokeInst*>*)"},
    {"_ZN4llvm25initializeLowerSwitchPassERNS_12PassRegistryE", "llvm::initializeLowerSwitchPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createLowerSwitchPassEv", "llvm::createLowerSwitchPass()"},
    {"_ZlsRN4llvm11raw_ostreamERKSt6vectorIN12_GLOBAL__N_111LowerSwitch9CaseRangeESaIS5_EE", "operator<<(llvm::raw_ostream&, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > const&)"},
    {"_ZN12_GLOBAL__N_111LowerSwitchD1Ev", "(anonymous namespace)::LowerSwitch::~LowerSwitch()"},
    {"_ZN12_GLOBAL__N_111LowerSwitchD0Ev", "(anonymous namespace)::LowerSwitch::~LowerSwitch()"},
    {"_ZNK12_GLOBAL__N_111LowerSwitch16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::LowerSwitch::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111LowerSwitch13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::LowerSwitch::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_111LowerSwitch13switchConvertEN9__gnu_cxx17__normal_iteratorIPNS0_9CaseRangeESt6vectorIS3_SaIS3_EEEES8_PN4llvm5ValueEPNS9_10BasicBlockESD_", "(anonymous namespace)::LowerSwitch::switchConvert(__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111LowerSwitch9CaseRangeESt6vectorIS4_SaIS4_EEEElNS2_7CaseCmpEEvT_SB_T0_T1_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, long, (anonymous namespace)::CaseCmp>(__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, long, (anonymous namespace)::CaseCmp)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111LowerSwitch9CaseRangeESt6vectorIS4_SaIS4_EEEENS2_7CaseCmpEEvT_SB_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, (anonymous namespace)::CaseCmp>(__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, (anonymous namespace)::CaseCmp)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_111LowerSwitch9CaseRangeESt6vectorIS4_SaIS4_EEEElS4_NS2_7CaseCmpEEvT_T0_SC_T1_T2_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, long, (anonymous namespace)::LowerSwitch::CaseRange, (anonymous namespace)::CaseCmp>(__gnu_cxx::__normal_iterator<(anonymous namespace)::LowerSwitch::CaseRange*, std::vector<(anonymous namespace)::LowerSwitch::CaseRange, std::allocator<(anonymous namespace)::LowerSwitch::CaseRange> > >, long, long, (anonymous namespace)::LowerSwitch::CaseRange, (anonymous namespace)::CaseCmp)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111LowerSwitchEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::LowerSwitch>()"},
    {"_ZN4llvm18isAllocaPromotableEPKNS_10AllocaInstE", "llvm::isAllocaPromotable(llvm::AllocaInst const*)"},
    {"_ZN4llvm15PromoteMemToRegERKSt6vectorIPNS_10AllocaInstESaIS2_EERNS_13DominatorTreeEPNS_15AliasSetTrackerE", "llvm::PromoteMemToReg(std::vector<llvm::AllocaInst*, std::allocator<llvm::AllocaInst*> > const&, llvm::DominatorTree&, llvm::AliasSetTracker*)"},
    {"_ZN12_GLOBAL__N_114PromoteMem2Reg3runEv", "(anonymous namespace)::PromoteMem2Reg::run()"},
    {"_ZN12_GLOBAL__N_114PromoteMem2Reg31ConvertDebugDeclareToDebugValueEPN4llvm14DbgDeclareInstEPNS1_9StoreInstE", "(anonymous namespace)::PromoteMem2Reg::ConvertDebugDeclareToDebugValue(llvm::DbgDeclareInst*, llvm::StoreInst*)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_114RenamePassDataESaIS1_EE9push_backERKS1_", "std::vector<(anonymous namespace)::RenamePassData, std::allocator<(anonymous namespace)::RenamePassData> >::push_back((anonymous namespace)::RenamePassData const&)"},
    {"_ZN12_GLOBAL__N_114PromoteMem2Reg11getNumPredsEPKN4llvm10BasicBlockE", "(anonymous namespace)::PromoteMem2Reg::getNumPreds(llvm::BasicBlock const*)"},
    {"_ZN12_GLOBAL__N_114LargeBlockInfo19getInstructionIndexEPKN4llvm11InstructionE", "(anonymous namespace)::LargeBlockInfo::getInstructionIndex(llvm::Instruction const*)"},
    {"_ZN4llvm15SmallVectorImplIPNS_14DbgDeclareInstEE6resizeEj", "llvm::SmallVectorImpl<llvm::DbgDeclareInst*>::resize(unsigned int)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockEjEPNS_7PHINodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE5clearEv", "llvm::DenseMap<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::PHINode*> >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockEjEPNS_7PHINodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE16shrink_and_clearEv", "llvm::DenseMap<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::PHINode*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_7PHINodeEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::PHINode*, unsigned int, llvm::DenseMapInfo<llvm::PHINode*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::PHINode* const&, unsigned int const&, std::pair<llvm::PHINode*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_7PHINodeEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::PHINode*, unsigned int, llvm::DenseMapInfo<llvm::PHINode*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionEjNS_12DenseMapInfoIS3_EENS4_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction const*, unsigned int, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockEjEPNS_7PHINodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE16FindAndConstructERKS4_", "llvm::DenseMap<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::PHINode*> >::FindAndConstruct(std::pair<llvm::BasicBlock*, unsigned int> const&)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockEjEPNS_7PHINodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE16InsertIntoBucketERKS4_RKS6_PS1_IS4_S6_E", "llvm::DenseMap<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::PHINode*> >::InsertIntoBucket(std::pair<llvm::BasicBlock*, unsigned int> const&, llvm::PHINode* const&, std::pair<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_10BasicBlockEjEPNS_7PHINodeENS_12DenseMapInfoIS4_EENS7_IS6_EEE4growEj", "llvm::DenseMap<std::pair<llvm::BasicBlock*, unsigned int>, llvm::PHINode*, llvm::DenseMapInfo<std::pair<llvm::BasicBlock*, unsigned int> >, llvm::DenseMapInfo<llvm::PHINode*> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIPSt4pairIjPN4llvm10BasicBlockEElEvT_S6_T0_", "void std::__introsort_loop<std::pair<unsigned int, llvm::BasicBlock*>*, long>(std::pair<unsigned int, llvm::BasicBlock*>*, std::pair<unsigned int, llvm::BasicBlock*>*, long)"},
    {"_ZSt22__final_insertion_sortIPSt4pairIjPN4llvm10BasicBlockEEEvT_S6_", "void std::__final_insertion_sort<std::pair<unsigned int, llvm::BasicBlock*>*>(std::pair<unsigned int, llvm::BasicBlock*>*, std::pair<unsigned int, llvm::BasicBlock*>*)"},
    {"_ZSt16__insertion_sortIPSt4pairIjPN4llvm10BasicBlockEEEvT_S6_", "void std::__insertion_sort<std::pair<unsigned int, llvm::BasicBlock*>*>(std::pair<unsigned int, llvm::BasicBlock*>*, std::pair<unsigned int, llvm::BasicBlock*>*)"},
    {"_ZSt13__heap_selectIPSt4pairIjPN4llvm10BasicBlockEEEvT_S6_S6_", "void std::__heap_select<std::pair<unsigned int, llvm::BasicBlock*>*>(std::pair<unsigned int, llvm::BasicBlock*>*, std::pair<unsigned int, llvm::BasicBlock*>*, std::pair<unsigned int, llvm::BasicBlock*>*)"},
    {"_ZSt13__adjust_heapIPSt4pairIjPN4llvm10BasicBlockEElS4_EvT_T0_S7_T1_", "void std::__adjust_heap<std::pair<unsigned int, llvm::BasicBlock*>*, long, std::pair<unsigned int, llvm::BasicBlock*> >(std::pair<unsigned int, llvm::BasicBlock*>*, long, long, std::pair<unsigned int, llvm::BasicBlock*>)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIPNS_15DomTreeNodeBaseINS_10BasicBlockEEEjEEaSERKS7_", "llvm::SmallVectorImpl<std::pair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, unsigned int> >::operator=(llvm::SmallVectorImpl<std::pair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, unsigned int> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_10AllocaInstEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::AllocaInst*, unsigned int, llvm::DenseMapInfo<llvm::AllocaInst*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::AllocaInst* const&, unsigned int const&, std::pair<llvm::AllocaInst*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_10AllocaInstEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::AllocaInst*, unsigned int, llvm::DenseMapInfo<llvm::AllocaInst*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_10BasicBlockEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE16InsertIntoBucketERKS4_RKjPSt4pairIS4_jE", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::BasicBlock>*, unsigned int, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::BasicBlock>*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::DomTreeNodeBase<llvm::BasicBlock>* const&, unsigned int const&, std::pair<llvm::DomTreeNodeBase<llvm::BasicBlock>*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_15DomTreeNodeBaseINS_10BasicBlockEEEjNS_12DenseMapInfoIS4_EENS5_IjEEE4growEj", "llvm::DenseMap<llvm::DomTreeNodeBase<llvm::BasicBlock>*, unsigned int, llvm::DenseMapInfo<llvm::DomTreeNodeBase<llvm::BasicBlock>*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZSt16__introsort_loopIPSt4pairIjPN4llvm9StoreInstEElEvT_S6_T0_", "void std::__introsort_loop<std::pair<unsigned int, llvm::StoreInst*>*, long>(std::pair<unsigned int, llvm::StoreInst*>*, std::pair<unsigned int, llvm::StoreInst*>*, long)"},
    {"_ZSt22__final_insertion_sortIPSt4pairIjPN4llvm9StoreInstEEEvT_S6_", "void std::__final_insertion_sort<std::pair<unsigned int, llvm::StoreInst*>*>(std::pair<unsigned int, llvm::StoreInst*>*, std::pair<unsigned int, llvm::StoreInst*>*)"},
    {"_ZSt16__insertion_sortIPSt4pairIjPN4llvm9StoreInstEEEvT_S6_", "void std::__insertion_sort<std::pair<unsigned int, llvm::StoreInst*>*>(std::pair<unsigned int, llvm::StoreInst*>*, std::pair<unsigned int, llvm::StoreInst*>*)"},
    {"_ZSt13__heap_selectIPSt4pairIjPN4llvm9StoreInstEEEvT_S6_S6_", "void std::__heap_select<std::pair<unsigned int, llvm::StoreInst*>*>(std::pair<unsigned int, llvm::StoreInst*>*, std::pair<unsigned int, llvm::StoreInst*>*, std::pair<unsigned int, llvm::StoreInst*>*)"},
    {"_ZSt13__adjust_heapIPSt4pairIjPN4llvm9StoreInstEElS4_EvT_T0_S7_T1_", "void std::__adjust_heap<std::pair<unsigned int, llvm::StoreInst*>*, long, std::pair<unsigned int, llvm::StoreInst*> >(std::pair<unsigned int, llvm::StoreInst*>*, long, long, std::pair<unsigned int, llvm::StoreInst*>)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::Instruction const*, unsigned int, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Instruction const* const&, unsigned int const&, std::pair<llvm::Instruction const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::Instruction const*, unsigned int, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm10SSAUpdaterC1EPNS_15SmallVectorImplIPNS_7PHINodeEEE", "llvm::SSAUpdater::SSAUpdater(llvm::SmallVectorImpl<llvm::PHINode*>*)"},
    {"_ZN4llvm10SSAUpdaterC2EPNS_15SmallVectorImplIPNS_7PHINodeEEE", "llvm::SSAUpdater::SSAUpdater(llvm::SmallVectorImpl<llvm::PHINode*>*)"},
    {"_ZN4llvm10SSAUpdaterD1Ev", "llvm::SSAUpdater::~SSAUpdater()"},
    {"_ZN4llvm10SSAUpdaterD2Ev", "llvm::SSAUpdater::~SSAUpdater()"},
    {"_ZN4llvm10SSAUpdater10InitializeEPKNS_4TypeENS_9StringRefE", "llvm::SSAUpdater::Initialize(llvm::Type const*, llvm::StringRef)"},
    {"_ZNK4llvm10SSAUpdater16HasValueForBlockEPNS_10BasicBlockE", "llvm::SSAUpdater::HasValueForBlock(llvm::BasicBlock*) const"},
    {"_ZN4llvm10SSAUpdater17AddAvailableValueEPNS_10BasicBlockEPNS_5ValueE", "llvm::SSAUpdater::AddAvailableValue(llvm::BasicBlock*, llvm::Value*)"},
    {"_ZN4llvm10SSAUpdater20GetValueAtEndOfBlockEPNS_10BasicBlockE", "llvm::SSAUpdater::GetValueAtEndOfBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm10SSAUpdater28GetValueAtEndOfBlockInternalEPNS_10BasicBlockE", "llvm::SSAUpdater::GetValueAtEndOfBlockInternal(llvm::BasicBlock*)"},
    {"_ZN4llvm10SSAUpdater23GetValueInMiddleOfBlockEPNS_10BasicBlockE", "llvm::SSAUpdater::GetValueInMiddleOfBlock(llvm::BasicBlock*)"},
    {"_ZN4llvm10SSAUpdater10RewriteUseERNS_3UseE", "llvm::SSAUpdater::RewriteUse(llvm::Use&)"},
    {"_ZN4llvm10SSAUpdater25RewriteUseAfterInsertionsERNS_3UseE", "llvm::SSAUpdater::RewriteUseAfterInsertions(llvm::Use&)"},
    {"_ZN4llvm20LoadAndStorePromoterC1ERKNS_15SmallVectorImplIPNS_11InstructionEEERNS_10SSAUpdaterENS_9StringRefE", "llvm::LoadAndStorePromoter::LoadAndStorePromoter(llvm::SmallVectorImpl<llvm::Instruction*> const&, llvm::SSAUpdater&, llvm::StringRef)"},
    {"_ZN4llvm20LoadAndStorePromoterC2ERKNS_15SmallVectorImplIPNS_11InstructionEEERNS_10SSAUpdaterENS_9StringRefE", "llvm::LoadAndStorePromoter::LoadAndStorePromoter(llvm::SmallVectorImpl<llvm::Instruction*> const&, llvm::SSAUpdater&, llvm::StringRef)"},
    {"_ZNK4llvm20LoadAndStorePromoter3runERKNS_15SmallVectorImplIPNS_11InstructionEEE", "llvm::LoadAndStorePromoter::run(llvm::SmallVectorImpl<llvm::Instruction*> const&) const"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE8GetValueEPNS_10BasicBlockE", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::GetValue(llvm::BasicBlock*)"},
    {"_ZN4llvm20LoadAndStorePromoterD1Ev", "llvm::LoadAndStorePromoter::~LoadAndStorePromoter()"},
    {"_ZN4llvm20LoadAndStorePromoterD0Ev", "llvm::LoadAndStorePromoter::~LoadAndStorePromoter()"},
    {"_ZNK4llvm20LoadAndStorePromoter12isInstInListEPNS_11InstructionERKNS_15SmallVectorImplIS2_EE", "llvm::LoadAndStorePromoter::isInstInList(llvm::Instruction*, llvm::SmallVectorImpl<llvm::Instruction*> const&) const"},
    {"_ZNK4llvm20LoadAndStorePromoter34doExtraRewritesBeforeFinalDeletionEv", "llvm::LoadAndStorePromoter::doExtraRewritesBeforeFinalDeletion() const"},
    {"_ZNK4llvm20LoadAndStorePromoter20replaceLoadWithValueEPNS_8LoadInstEPNS_5ValueE", "llvm::LoadAndStorePromoter::replaceLoadWithValue(llvm::LoadInst*, llvm::Value*) const"},
    {"_ZNK4llvm20LoadAndStorePromoter18instructionDeletedEPNS_11InstructionE", "llvm::LoadAndStorePromoter::instructionDeleted(llvm::Instruction*) const"},
    {"_ZNSt6vectorIPN4llvm11InstructionESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Instruction**, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > >, llvm::Instruction* const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockESt6vectorIPNS_11InstructionESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::BasicBlock*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > > >::FindAndConstruct(llvm::BasicBlock* const&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockESt6vectorIPNS_11InstructionESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<llvm::BasicBlock*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > > >::InsertIntoBucket(llvm::BasicBlock* const&, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > const&, std::pair<llvm::BasicBlock*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > >*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockESt6vectorIPNS_11InstructionESaIS5_EENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> >, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<std::vector<llvm::Instruction*, std::allocator<llvm::Instruction*> > > >::grow(unsigned int)"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE14BuildBlockListEPNS_10BasicBlockEPNS_15SmallVectorImplIPNS2_6BBInfoEEE", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BuildBlockList(llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE14FindDominatorsEPNS_15SmallVectorImplIPNS2_6BBInfoEEES5_", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::FindDominators(llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*>*, llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE17FindAvailableValsEPNS_15SmallVectorImplIPNS2_6BBInfoEEE", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::FindAvailableVals(llvm::SmallVectorImpl<llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_14SSAUpdaterImplINS_10SSAUpdaterEE6BBInfoENS_12DenseMapInfoIS2_EENS8_IS7_EEE16InsertIntoBucketERKS2_RKS7_PSt4pairIS2_S7_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo* const&, std::pair<llvm::BasicBlock*, llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_14SSAUpdaterImplINS_10SSAUpdaterEE6BBInfoENS_12DenseMapInfoIS2_EENS8_IS7_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::SSAUpdaterImpl<llvm::SSAUpdater>::BBInfo*> >::grow(unsigned int)"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE17CheckIfPHIMatchesEPNS_7PHINodeE", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::CheckIfPHIMatches(llvm::PHINode*)"},
    {"_ZN4llvm14SSAUpdaterImplINS_10SSAUpdaterEE17RecordMatchingPHIEPNS_7PHINodeE", "llvm::SSAUpdaterImpl<llvm::SSAUpdater>::RecordMatchingPHI(llvm::PHINode*)"},
    {"_ZN4llvm16SSAUpdaterTraitsINS_10SSAUpdaterEE21FindPredecessorBlocksEPNS_10BasicBlockEPNS_15SmallVectorImplIS4_EE", "llvm::SSAUpdaterTraits<llvm::SSAUpdater>::FindPredecessorBlocks(llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::BasicBlock*>*)"},
    {"_ZN4llvm22FoldBranchToCommonDestEPNS_10BranchInstE", "llvm::FoldBranchToCommonDest(llvm::BranchInst*)"},
    {"_Z22SafeToMergeTerminatorsPN4llvm14TerminatorInstES1_", "SafeToMergeTerminators(llvm::TerminatorInst*, llvm::TerminatorInst*)"},
    {"_ZN4llvm11SimplifyCFGEPNS_10BasicBlockEPKNS_10DataLayoutE", "llvm::SimplifyCFG(llvm::BasicBlock*, llvm::DataLayout const*)"},
    {"_ZN12_GLOBAL__N_114SimplifyCFGOpt3runEPN4llvm10BasicBlockE", "(anonymous namespace)::SimplifyCFGOpt::run(llvm::BasicBlock*)"},
    {"_Z29EraseTerminatorInstAndDCECondPN4llvm14TerminatorInstE", "EraseTerminatorInstAndDCECond(llvm::TerminatorInst*)"},
    {"_ZN12_GLOBAL__N_114SimplifyCFGOpt25isValueEqualityComparisonEPN4llvm14TerminatorInstE", "(anonymous namespace)::SimplifyCFGOpt::isValueEqualityComparison(llvm::TerminatorInst*)"},
    {"_ZN12_GLOBAL__N_114SimplifyCFGOpt45SimplifyEqualityComparisonWithOnlyPredecessorEPN4llvm14TerminatorInstEPNS1_10BasicBlockE", "(anonymous namespace)::SimplifyCFGOpt::SimplifyEqualityComparisonWithOnlyPredecessor(llvm::TerminatorInst*, llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_114SimplifyCFGOpt35FoldValueComparisonIntoPredecessorsEPN4llvm14TerminatorInstE", "(anonymous namespace)::SimplifyCFGOpt::FoldValueComparisonIntoPredecessors(llvm::TerminatorInst*)"},
    {"_Z24ConstantIntSortPredicatePKvS0_", "ConstantIntSortPredicate(void const*, void const*)"},
    {"_ZN12_GLOBAL__N_114SimplifyCFGOpt31GetValueEqualityComparisonCasesEPN4llvm14TerminatorInstERSt6vectorISt4pairIPNS1_11ConstantIntEPNS1_10BasicBlockEESaISA_EE", "(anonymous namespace)::SimplifyCFGOpt::GetValueEqualityComparisonCases(llvm::TerminatorInst*, std::vector<std::pair<llvm::ConstantInt*, llvm::BasicBlock*>, std::allocator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*> > >&)"},
    {"_ZNSt3setIPN4llvm11ConstantIntEN12_GLOBAL__N_119ConstantIntOrderingESaIS2_EE6insertERKS2_", "std::set<llvm::ConstantInt*, (anonymous namespace)::ConstantIntOrdering, std::allocator<llvm::ConstantInt*> >::insert(llvm::ConstantInt* const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm11ConstantIntES2_St9_IdentityIS2_EN12_GLOBAL__N_119ConstantIntOrderingESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::ConstantInt*, llvm::ConstantInt*, std::_Identity<llvm::ConstantInt*>, (anonymous namespace)::ConstantIntOrdering, std::allocator<llvm::ConstantInt*> >::_M_erase(std::_Rb_tree_node<llvm::ConstantInt*>*)"},
    {"_Z14GetConstantIntPN4llvm5ValueEPKNS_10DataLayoutE", "GetConstantInt(llvm::Value*, llvm::DataLayout const*)"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIPNS_11ConstantIntEPNS_10BasicBlockEEEEiPKvS8_", "int llvm::array_pod_sort_comparator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*> >(void const*, void const*)"},
    {"_Z22SpeculativelyExecuteBBPN4llvm10BranchInstEPNS_10BasicBlockE", "SpeculativelyExecuteBB(llvm::BranchInst*, llvm::BasicBlock*)"},
    {"_Z19FoldCondBranchOnPHIPN4llvm10BranchInstEPKNS_10DataLayoutE", "FoldCondBranchOnPHI(llvm::BranchInst*, llvm::DataLayout const*)"},
    {"_Z34BlockIsSimpleEnoughToThreadThroughPN4llvm10BasicBlockE", "BlockIsSimpleEnoughToThreadThrough(llvm::BasicBlock*)"},
    {"_Z19isSafeToHoistInvokePN4llvm10BasicBlockES1_PNS_11InstructionES3_", "isSafeToHoistInvoke(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Instruction*, llvm::Instruction*)"},
    {"_Z22GatherConstantComparesPN4llvm5ValueERSt6vectorIPNS_11ConstantIntESaIS4_EERS1_PKNS_10DataLayoutEbRj", "GatherConstantCompares(llvm::Value*, std::vector<llvm::ConstantInt*, std::allocator<llvm::ConstantInt*> >&, llvm::Value*&, llvm::DataLayout const*, bool, unsigned int&)"},
    {"_Z19DominatesMergePointPN4llvm5ValueEPNS_10BasicBlockEPNS_11SmallPtrSetIPNS_11InstructionELj4EEE", "DominatesMergePoint(llvm::Value*, llvm::BasicBlock*, llvm::SmallPtrSet<llvm::Instruction*, 4u>*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, unsigned int>, std::_Select1st<std::pair<llvm::BasicBlock* const, unsigned int> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, unsigned int> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock* const, unsigned int> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS5_ERKS5_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, unsigned int>, std::_Select1st<std::pair<llvm::BasicBlock* const, unsigned int> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, unsigned int> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock* const, unsigned int> >, std::pair<llvm::BasicBlock* const, unsigned int> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_jESt10_Select1stIS5_ESt4lessIS2_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, unsigned int>, std::_Select1st<std::pair<llvm::BasicBlock* const, unsigned int> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, unsigned int> > >::_M_insert_unique(std::pair<llvm::BasicBlock* const, unsigned int> const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm11ConstantIntEPNS1_10BasicBlockEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<llvm::ConstantInt*, llvm::BasicBlock*>, std::allocator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*>*, std::vector<std::pair<llvm::ConstantInt*, llvm::BasicBlock*>, std::allocator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*> > > >, std::pair<llvm::ConstantInt*, llvm::BasicBlock*> const&)"},
    {"_ZNSt6vectorISt4pairIPN4llvm11ConstantIntEPNS1_10BasicBlockEESaIS6_EE7reserveEm", "std::vector<std::pair<llvm::ConstantInt*, llvm::BasicBlock*>, std::allocator<std::pair<llvm::ConstantInt*, llvm::BasicBlock*> > >::reserve(unsigned long)"},
    {"_ZNSt8_Rb_treeISt4pairIPN4llvm5ValueES3_ES0_IKS4_PNS1_10SelectInstEESt10_Select1stIS8_ESt4lessIS4_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<std::pair<llvm::Value*, llvm::Value*>, std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*>, std::_Select1st<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> >, std::less<std::pair<llvm::Value*, llvm::Value*> >, std::allocator<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPN4llvm5ValueES3_ES0_IKS4_PNS1_10SelectInstEESt10_Select1stIS8_ESt4lessIS4_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<std::pair<llvm::Value*, llvm::Value*>, std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*>, std::_Select1st<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> >, std::less<std::pair<llvm::Value*, llvm::Value*> >, std::allocator<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> >, std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPN4llvm5ValueES3_ES0_IKS4_PNS1_10SelectInstEESt10_Select1stIS8_ESt4lessIS4_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<std::pair<llvm::Value*, llvm::Value*>, std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*>, std::_Select1st<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> >, std::less<std::pair<llvm::Value*, llvm::Value*> >, std::allocator<std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> > >::_M_insert_unique(std::pair<std::pair<llvm::Value*, llvm::Value*> const, llvm::SelectInst*> const&)"},
    {"_ZNSt6vectorIPN4llvm11ConstantIntESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::ConstantInt*, std::allocator<llvm::ConstantInt*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::ConstantInt**, std::vector<llvm::ConstantInt*, std::allocator<llvm::ConstantInt*> > >, unsigned long, llvm::ConstantInt* const&)"},
    {"_ZN4llvm36initializeUnifyFunctionExitNodesPassERNS_12PassRegistryE", "llvm::initializeUnifyFunctionExitNodesPass(llvm::PassRegistry&)"},
    {"_ZN4llvm32createUnifyFunctionExitNodesPassEv", "llvm::createUnifyFunctionExitNodesPass()"},
    {"_ZNK4llvm22UnifyFunctionExitNodes16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::UnifyFunctionExitNodes::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm22UnifyFunctionExitNodes13runOnFunctionERNS_8FunctionE", "llvm::UnifyFunctionExitNodes::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm22UnifyFunctionExitNodesD1Ev", "llvm::UnifyFunctionExitNodes::~UnifyFunctionExitNodes()"},
    {"_ZN4llvm22UnifyFunctionExitNodesD0Ev", "llvm::UnifyFunctionExitNodes::~UnifyFunctionExitNodes()"},
    {"_ZN4llvm15callDefaultCtorINS_22UnifyFunctionExitNodesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::UnifyFunctionExitNodes>()"},
    {"_ZN4llvm8MapValueEPKNS_5ValueERNS_8ValueMapIS2_NS_10TrackingVHIS0_EENS_14ValueMapConfigIS2_EENS_12DenseMapInfoIS5_EEEENS_10RemapFlagsE", "llvm::MapValue(llvm::Value const*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, llvm::RemapFlags)"},
    {"_ZN4llvm16RemapInstructionEPNS_11InstructionERNS_8ValueMapIPKNS_5ValueENS_10TrackingVHIS3_EENS_14ValueMapConfigIS5_EENS_12DenseMapInfoIS7_EEEENS_10RemapFlagsE", "llvm::RemapInstruction(llvm::Instruction*, llvm::ValueMap<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >&, llvm::RemapFlags)"},
    {"_ZN4llvm32initializeCallGraphAnalysisGroupERNS_12PassRegistryE", "llvm::initializeCallGraphAnalysisGroup(llvm::PassRegistry&)"},
    {"_ZN4llvm28initializeBasicCallGraphPassERNS_12PassRegistryE", "llvm::initializeBasicCallGraphPass(llvm::PassRegistry&)"},
    {"_ZN4llvm9CallGraph10initializeERNS_6ModuleE", "llvm::CallGraph::initialize(llvm::Module&)"},
    {"_ZN4llvm9CallGraph7destroyEv", "llvm::CallGraph::destroy()"},
    {"_ZNK4llvm9CallGraph5printERNS_11raw_ostreamEPNS_6ModuleE", "llvm::CallGraph::print(llvm::raw_ostream&, llvm::Module*) const"},
    {"_ZNK4llvm13CallGraphNode5printERNS_11raw_ostreamE", "llvm::CallGraphNode::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm9CallGraph4dumpEv", "llvm::CallGraph::dump() const"},
    {"_ZN4llvm9CallGraph24removeFunctionFromModuleEPNS_13CallGraphNodeE", "llvm::CallGraph::removeFunctionFromModule(llvm::CallGraphNode*)"},
    {"_ZN4llvm9CallGraph14spliceFunctionEPKNS_8FunctionES3_", "llvm::CallGraph::spliceFunction(llvm::Function const*, llvm::Function const*)"},
    {"_ZN4llvm9CallGraph19getOrInsertFunctionEPKNS_8FunctionE", "llvm::CallGraph::getOrInsertFunction(llvm::Function const*)"},
    {"_ZNK4llvm13CallGraphNode4dumpEv", "llvm::CallGraphNode::dump() const"},
    {"_ZN4llvm13CallGraphNode17removeCallEdgeForENS_8CallSiteE", "llvm::CallGraphNode::removeCallEdgeFor(llvm::CallSite)"},
    {"_ZN4llvm13CallGraphNode19removeAnyCallEdgeToEPS0_", "llvm::CallGraphNode::removeAnyCallEdgeTo(llvm::CallGraphNode*)"},
    {"_ZN4llvm13CallGraphNode23removeOneAbstractEdgeToEPS0_", "llvm::CallGraphNode::removeOneAbstractEdgeTo(llvm::CallGraphNode*)"},
    {"_ZN4llvm13CallGraphNode15replaceCallEdgeENS_8CallSiteES1_PS0_", "llvm::CallGraphNode::replaceCallEdge(llvm::CallSite, llvm::CallSite, llvm::CallGraphNode*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_114BasicCallGraphEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::BasicCallGraph>()"},
    {"_ZN12_GLOBAL__N_114BasicCallGraphD1Ev", "(anonymous namespace)::BasicCallGraph::~BasicCallGraph()"},
    {"_ZN12_GLOBAL__N_114BasicCallGraphD0Ev", "(anonymous namespace)::BasicCallGraph::~BasicCallGraph()"},
    {"_ZNK12_GLOBAL__N_114BasicCallGraph5printERN4llvm11raw_ostreamEPKNS1_6ModuleE", "(anonymous namespace)::BasicCallGraph::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK12_GLOBAL__N_114BasicCallGraph16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::BasicCallGraph::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph13releaseMemoryEv", "(anonymous namespace)::BasicCallGraph::releaseMemory()"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph26getAdjustedAnalysisPointerEPKv", "(anonymous namespace)::BasicCallGraph::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::BasicCallGraph::runOnModule(llvm::Module&)"},
    {"_ZNK12_GLOBAL__N_114BasicCallGraph22getExternalCallingNodeEv", "(anonymous namespace)::BasicCallGraph::getExternalCallingNode() const"},
    {"_ZNK12_GLOBAL__N_114BasicCallGraph20getCallsExternalNodeEv", "(anonymous namespace)::BasicCallGraph::getCallsExternalNode() const"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph7getRootEv", "(anonymous namespace)::BasicCallGraph::getRoot()"},
    {"_ZNK12_GLOBAL__N_114BasicCallGraph7getRootEv", "(anonymous namespace)::BasicCallGraph::getRoot() const"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph7destroyEv", "(anonymous namespace)::BasicCallGraph::destroy()"},
    {"_ZThn32_NK12_GLOBAL__N_114BasicCallGraph22getExternalCallingNodeEv", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::getExternalCallingNode() const"},
    {"_ZThn32_NK12_GLOBAL__N_114BasicCallGraph20getCallsExternalNodeEv", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::getCallsExternalNode() const"},
    {"_ZThn32_N12_GLOBAL__N_114BasicCallGraph7getRootEv", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::getRoot()"},
    {"_ZThn32_NK12_GLOBAL__N_114BasicCallGraph7getRootEv", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::getRoot() const"},
    {"_ZThn32_N12_GLOBAL__N_114BasicCallGraphD1Ev", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::~BasicCallGraph()"},
    {"_ZThn32_N12_GLOBAL__N_114BasicCallGraphD0Ev", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::~BasicCallGraph()"},
    {"_ZThn32_N12_GLOBAL__N_114BasicCallGraph7destroyEv", "non-virtual thunk to (anonymous namespace)::BasicCallGraph::destroy()"},
    {"_ZN4llvm13CallGraphNodeD1Ev", "llvm::CallGraphNode::~CallGraphNode()"},
    {"_ZNK4llvm9CallGraph22getExternalCallingNodeEv", "llvm::CallGraph::getExternalCallingNode() const"},
    {"_ZNK4llvm9CallGraph20getCallsExternalNodeEv", "llvm::CallGraph::getCallsExternalNode() const"},
    {"_ZN4llvm9CallGraph7getRootEv", "llvm::CallGraph::getRoot()"},
    {"_ZNK4llvm9CallGraph7getRootEv", "llvm::CallGraph::getRoot() const"},
    {"_ZN4llvm9CallGraphD1Ev", "llvm::CallGraph::~CallGraph()"},
    {"_ZN4llvm9CallGraphD0Ev", "llvm::CallGraph::~CallGraph()"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_PNS0_13CallGraphNodeEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, llvm::CallGraphNode*>, std::_Select1st<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::pair<llvm::Function const* const, llvm::CallGraphNode*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_PNS0_13CallGraphNodeEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, llvm::CallGraphNode*>, std::_Select1st<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> > >::_M_insert_unique(std::pair<llvm::Function const* const, llvm::CallGraphNode*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_PNS0_13CallGraphNodeEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE5eraseESt17_Rb_tree_iteratorIS8_ESG_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, llvm::CallGraphNode*>, std::_Select1st<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::_Rb_tree_iterator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_PNS0_13CallGraphNodeEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, llvm::CallGraphNode*>, std::_Select1st<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, llvm::CallGraphNode*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Function const* const, llvm::CallGraphNode*> >*)"},
    {"_ZN4llvm13CallGraphNode17addCalledFunctionENS_8CallSiteEPS0_", "llvm::CallGraphNode::addCalledFunction(llvm::CallSite, llvm::CallGraphNode*)"},
    {"_ZNSt6vectorISt4pairIN4llvm6WeakVHEPNS1_13CallGraphNodeEESaIS5_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS5_S7_EERKS5_", "std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pair<llvm::WeakVH, llvm::CallGraphNode*> const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm6WeakVHEPNS4_13CallGraphNodeEES9_EET0_T_SB_SA_", "std::pair<llvm::WeakVH, llvm::CallGraphNode*>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::pair<llvm::WeakVH, llvm::CallGraphNode*>*>(std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::pair<llvm::WeakVH, llvm::CallGraphNode*>*)"},
    {"_ZN4llvm12CallGraphSCC11ReplaceNodeEPNS_13CallGraphNodeES2_", "llvm::CallGraphSCC::ReplaceNode(llvm::CallGraphNode*, llvm::CallGraphNode*)"},
    {"_ZN4llvm16CallGraphSCCPass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::CallGraphSCCPass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZNK4llvm16CallGraphSCCPass16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::CallGraphSCCPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm16CallGraphSCCPass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::CallGraphSCCPass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZN12_GLOBAL__N_118PrintCallGraphPassD1Ev", "(anonymous namespace)::PrintCallGraphPass::~PrintCallGraphPass()"},
    {"_ZN12_GLOBAL__N_118PrintCallGraphPassD0Ev", "(anonymous namespace)::PrintCallGraphPass::~PrintCallGraphPass()"},
    {"_ZNK12_GLOBAL__N_118PrintCallGraphPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PrintCallGraphPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118PrintCallGraphPass8runOnSCCERN4llvm12CallGraphSCCE", "(anonymous namespace)::PrintCallGraphPass::runOnSCC(llvm::CallGraphSCC&)"},
    {"_ZN12_GLOBAL__N_113CGPassManagerD1Ev", "(anonymous namespace)::CGPassManager::~CGPassManager()"},
    {"_ZN12_GLOBAL__N_113CGPassManagerD0Ev", "(anonymous namespace)::CGPassManager::~CGPassManager()"},
    {"_ZNK12_GLOBAL__N_113CGPassManager11getPassNameEv", "(anonymous namespace)::CGPassManager::getPassName() const"},
    {"_ZNK12_GLOBAL__N_113CGPassManager16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::CGPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_113CGPassManager18getAsPMDataManagerEv", "(anonymous namespace)::CGPassManager::getAsPMDataManager()"},
    {"_ZN12_GLOBAL__N_113CGPassManager17dumpPassStructureEj", "(anonymous namespace)::CGPassManager::dumpPassStructure(unsigned int)"},
    {"_ZN12_GLOBAL__N_113CGPassManager11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::CGPassManager::runOnModule(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_113CGPassManager9getAsPassEv", "(anonymous namespace)::CGPassManager::getAsPass()"},
    {"_ZNK12_GLOBAL__N_113CGPassManager18getPassManagerTypeEv", "(anonymous namespace)::CGPassManager::getPassManagerType() const"},
    {"_ZThn32_N12_GLOBAL__N_113CGPassManagerD1Ev", "non-virtual thunk to (anonymous namespace)::CGPassManager::~CGPassManager()"},
    {"_ZThn32_N12_GLOBAL__N_113CGPassManagerD0Ev", "non-virtual thunk to (anonymous namespace)::CGPassManager::~CGPassManager()"},
    {"_ZThn32_N12_GLOBAL__N_113CGPassManager9getAsPassEv", "non-virtual thunk to (anonymous namespace)::CGPassManager::getAsPass()"},
    {"_ZThn32_NK12_GLOBAL__N_113CGPassManager18getPassManagerTypeEv", "non-virtual thunk to (anonymous namespace)::CGPassManager::getPassManagerType() const"},
    {"_ZN12_GLOBAL__N_113CGPassManager16RefreshCallGraphERN4llvm12CallGraphSCCERNS1_9CallGraphEb", "(anonymous namespace)::CGPassManager::RefreshCallGraph(llvm::CallGraphSCC&, llvm::CallGraph&, bool)"},
    {"_ZN4llvm12scc_iteratorIPNS_9CallGraphENS_11GraphTraitsIS2_EEE11ReplaceNodeEPNS_13CallGraphNodeES7_", "llvm::scc_iterator<llvm::CallGraph*, llvm::GraphTraits<llvm::CallGraph*> >::ReplaceNode(llvm::CallGraphNode*, llvm::CallGraphNode*)"},
    {"_ZN4llvm8DenseMapIPNS_13CallGraphNodeEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::CallGraphNode*, unsigned int, llvm::DenseMapInfo<llvm::CallGraphNode*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::CallGraphNode* const&, unsigned int const&, std::pair<llvm::CallGraphNode*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_13CallGraphNodeEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::CallGraphNode*, unsigned int, llvm::DenseMapInfo<llvm::CallGraphNode*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm9scc_beginIPNS_9CallGraphEEENS_12scc_iteratorIT_NS_11GraphTraitsIS4_EEEERKS4_", "llvm::scc_iterator<llvm::CallGraph*, llvm::GraphTraits<llvm::CallGraph*> > llvm::scc_begin<llvm::CallGraph*>(llvm::CallGraph* const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_13CallGraphNodeENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Value*, llvm::CallGraphNode*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::CallGraphNode*> >::insert(std::pair<llvm::Value*, llvm::CallGraphNode*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_13CallGraphNodeENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Value*, llvm::CallGraphNode*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::CallGraphNode*> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_13CallGraphNodeENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Value*, llvm::CallGraphNode*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::CallGraphNode*> >::InsertIntoBucket(llvm::Value* const&, llvm::CallGraphNode* const&, std::pair<llvm::Value*, llvm::CallGraphNode*>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_13CallGraphNodeENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Value*, llvm::CallGraphNode*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::CallGraphNode*> >::grow(unsigned int)"},
    {"_ZN4llvm12scc_iteratorIPNS_9CallGraphENS_11GraphTraitsIS2_EEE10GetNextSCCEv", "llvm::scc_iterator<llvm::CallGraph*, llvm::GraphTraits<llvm::CallGraph*> >::GetNextSCC()"},
    {"_ZN4llvm12scc_iteratorIPNS_9CallGraphENS_11GraphTraitsIS2_EEE16DFSVisitChildrenEv", "llvm::scc_iterator<llvm::CallGraph*, llvm::GraphTraits<llvm::CallGraph*> >::DFSVisitChildren()"},
    {"_ZNSt6vectorIPN4llvm13CallGraphNodeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::CallGraphNode*, std::allocator<llvm::CallGraphNode*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::CallGraphNode**, std::vector<llvm::CallGraphNode*, std::allocator<llvm::CallGraphNode*> > >, llvm::CallGraphNode* const&)"},
    {"_ZN4llvm12scc_iteratorIPNS_9CallGraphENS_11GraphTraitsIS2_EEE11DFSVisitOneEPNS_13CallGraphNodeE", "llvm::scc_iterator<llvm::CallGraph*, llvm::GraphTraits<llvm::CallGraph*> >::DFSVisitOne(llvm::CallGraphNode*)"},
    {"_ZN4llvm11GraphTraitsIPNS_13CallGraphNodeEE8CGNDerefESt4pairINS_6WeakVHES2_E", "llvm::GraphTraits<llvm::CallGraphNode*>::CGNDeref(std::pair<llvm::WeakVH, llvm::CallGraphNode*>)"},
    {"_ZNSt6vectorISt4pairIPN4llvm13CallGraphNodeENS1_15mapped_iteratorIN9__gnu_cxx17__normal_iteratorIPS0_INS1_6WeakVHES3_ES_IS8_SaIS8_EEEESt25pointer_to_unary_functionIS8_S3_EEEESaISG_EE13_M_insert_auxENS6_IPSG_SI_EERKSG_", "std::vector<std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > >, std::allocator<std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > > > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > >*, std::vector<std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > >, std::allocator<std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > > > > >, std::pair<llvm::CallGraphNode*, llvm::mapped_iterator<__gnu_cxx::__normal_iterator<std::pair<llvm::WeakVH, llvm::CallGraphNode*>*, std::vector<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, std::allocator<std::pair<llvm::WeakVH, llvm::CallGraphNode*> > > >, std::pointer_to_unary_function<std::pair<llvm::WeakVH, llvm::CallGraphNode*>, llvm::CallGraphNode*> > > const&)"},
    {"_ZNSt6vectorIPN4llvm13CallGraphNodeESaIS2_EE13_M_assign_auxIPKS2_EEvT_S8_St20forward_iterator_tag", "void std::vector<llvm::CallGraphNode*, std::allocator<llvm::CallGraphNode*> >::_M_assign_aux<llvm::CallGraphNode* const*>(llvm::CallGraphNode* const*, llvm::CallGraphNode* const*, std::forward_iterator_tag)"},
    {"_ZN4llvm27initializeFindUsedTypesPassERNS_12PassRegistryE", "llvm::initializeFindUsedTypesPass(llvm::PassRegistry&)"},
    {"_ZN4llvm13FindUsedTypes15IncorporateTypeEPKNS_4TypeE", "llvm::FindUsedTypes::IncorporateType(llvm::Type const*)"},
    {"_ZN4llvm13FindUsedTypes16IncorporateValueEPKNS_5ValueE", "llvm::FindUsedTypes::IncorporateValue(llvm::Value const*)"},
    {"_ZN4llvm13FindUsedTypes11runOnModuleERNS_6ModuleE", "llvm::FindUsedTypes::runOnModule(llvm::Module&)"},
    {"_ZNK4llvm13FindUsedTypes5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::FindUsedTypes::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZN4llvm13FindUsedTypesD1Ev", "llvm::FindUsedTypes::~FindUsedTypes()"},
    {"_ZN4llvm13FindUsedTypesD0Ev", "llvm::FindUsedTypes::~FindUsedTypes()"},
    {"_ZNK4llvm13FindUsedTypes16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::FindUsedTypes::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE16_M_insert_uniqueERKS3_", "std::_Rb_tree<llvm::Type const*, llvm::Type const*, std::_Identity<llvm::Type const*>, std::less<llvm::Type const*>, std::allocator<llvm::Type const*> >::_M_insert_unique(llvm::Type const* const&)"},
    {"_ZN4llvm15callDefaultCtorINS_13FindUsedTypesEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::FindUsedTypes>()"},
    {"_ZN4llvm36initializeAliasAnalysisAnalysisGroupERNS_12PassRegistryE", "llvm::initializeAliasAnalysisAnalysisGroup(llvm::PassRegistry&)"},
    {"_ZN4llvm13AliasAnalysis5aliasERKNS0_8LocationES3_", "llvm::AliasAnalysis::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysis22pointsToConstantMemoryERKNS0_8LocationEb", "llvm::AliasAnalysis::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZN4llvm13AliasAnalysis11deleteValueEPNS_5ValueE", "llvm::AliasAnalysis::deleteValue(llvm::Value*)"},
    {"_ZN4llvm13AliasAnalysis9copyValueEPNS_5ValueES2_", "llvm::AliasAnalysis::copyValue(llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm13AliasAnalysis14addEscapingUseERNS_3UseE", "llvm::AliasAnalysis::addEscapingUse(llvm::Use&)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoENS_17ImmutableCallSiteERKNS0_8LocationE", "llvm::AliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoENS_17ImmutableCallSiteES1_", "llvm::AliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZN4llvm13AliasAnalysis17getModRefBehaviorENS_17ImmutableCallSiteE", "llvm::AliasAnalysis::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZN4llvm13AliasAnalysis17getModRefBehaviorEPKNS_8FunctionE", "llvm::AliasAnalysis::getModRefBehavior(llvm::Function const*)"},
    {"_ZN4llvm13AliasAnalysis11getLocationEPKNS_8LoadInstE", "llvm::AliasAnalysis::getLocation(llvm::LoadInst const*)"},
    {"_ZN4llvm13AliasAnalysis16getTypeStoreSizeEPKNS_4TypeE", "llvm::AliasAnalysis::getTypeStoreSize(llvm::Type const*)"},
    {"_ZN4llvm13AliasAnalysis11getLocationEPKNS_9StoreInstE", "llvm::AliasAnalysis::getLocation(llvm::StoreInst const*)"},
    {"_ZN4llvm13AliasAnalysis11getLocationEPKNS_9VAArgInstE", "llvm::AliasAnalysis::getLocation(llvm::VAArgInst const*)"},
    {"_ZN4llvm13AliasAnalysis20getLocationForSourceEPKNS_15MemTransferInstE", "llvm::AliasAnalysis::getLocationForSource(llvm::MemTransferInst const*)"},
    {"_ZN4llvm13AliasAnalysis18getLocationForDestEPKNS_12MemIntrinsicE", "llvm::AliasAnalysis::getLocationForDest(llvm::MemIntrinsic const*)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoEPKNS_8LoadInstERKNS0_8LocationE", "llvm::AliasAnalysis::getModRefInfo(llvm::LoadInst const*, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoEPKNS_9StoreInstERKNS0_8LocationE", "llvm::AliasAnalysis::getModRefInfo(llvm::StoreInst const*, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoEPKNS_9VAArgInstERKNS0_8LocationE", "llvm::AliasAnalysis::getModRefInfo(llvm::VAArgInst const*, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysisD0Ev", "llvm::AliasAnalysis::~AliasAnalysis()"},
    {"_ZN4llvm13AliasAnalysisD1Ev", "llvm::AliasAnalysis::~AliasAnalysis()"},
    {"_ZN4llvm13AliasAnalysisD2Ev", "llvm::AliasAnalysis::~AliasAnalysis()"},
    {"_ZN4llvm13AliasAnalysis23InitializeAliasAnalysisEPNS_4PassE", "llvm::AliasAnalysis::InitializeAliasAnalysis(llvm::Pass*)"},
    {"_ZNK4llvm13AliasAnalysis16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::AliasAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13AliasAnalysis19canBasicBlockModifyERKNS_10BasicBlockERKNS0_8LocationE", "llvm::AliasAnalysis::canBasicBlockModify(llvm::BasicBlock const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13AliasAnalysis25canInstructionRangeModifyERKNS_11InstructionES3_RKNS0_8LocationE", "llvm::AliasAnalysis::canInstructionRangeModify(llvm::Instruction const&, llvm::Instruction const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm13isNoAliasCallEPKNS_5ValueE", "llvm::isNoAliasCall(llvm::Value const*)"},
    {"_ZN4llvm18isIdentifiedObjectEPKNS_5ValueE", "llvm::isIdentifiedObject(llvm::Value const*)"},
    {"_ZN4llvm13AliasAnalysis13getModRefInfoEPKNS_11InstructionERKNS0_8LocationE", "llvm::AliasAnalysis::getModRefInfo(llvm::Instruction const*, llvm::AliasAnalysis::Location const&)"},
    {"_ZN4llvm8AliasSet10mergeSetInERS0_RNS_15AliasSetTrackerE", "llvm::AliasSet::mergeSetIn(llvm::AliasSet&, llvm::AliasSetTracker&)"},
    {"_ZN4llvm15AliasSetTracker14removeAliasSetEPNS_8AliasSetE", "llvm::AliasSetTracker::removeAliasSet(llvm::AliasSet*)"},
    {"_ZN4llvm8AliasSet17removeFromTrackerERNS_15AliasSetTrackerE", "llvm::AliasSet::removeFromTracker(llvm::AliasSetTracker&)"},
    {"_ZN4llvm8AliasSet10addPointerERNS_15AliasSetTrackerERNS0_10PointerRecEyPKNS_6MDNodeEb", "llvm::AliasSet::addPointer(llvm::AliasSetTracker&, llvm::AliasSet::PointerRec&, unsigned long long, llvm::MDNode const*, bool)"},
    {"_ZN4llvm8AliasSet11addCallSiteENS_8CallSiteERNS_13AliasAnalysisE", "llvm::AliasSet::addCallSite(llvm::CallSite, llvm::AliasAnalysis&)"},
    {"_ZNK4llvm8AliasSet14aliasesPointerEPKNS_5ValueEyPKNS_6MDNodeERNS_13AliasAnalysisE", "llvm::AliasSet::aliasesPointer(llvm::Value const*, unsigned long long, llvm::MDNode const*, llvm::AliasAnalysis&) const"},
    {"_ZNK4llvm8AliasSet15aliasesCallSiteENS_8CallSiteERNS_13AliasAnalysisE", "llvm::AliasSet::aliasesCallSite(llvm::CallSite, llvm::AliasAnalysis&) const"},
    {"_ZN4llvm15AliasSetTracker5clearEv", "llvm::AliasSetTracker::clear()"},
    {"_ZN4llvm15AliasSetTracker22findAliasSetForPointerEPKNS_5ValueEyPKNS_6MDNodeE", "llvm::AliasSetTracker::findAliasSetForPointer(llvm::Value const*, unsigned long long, llvm::MDNode const*)"},
    {"_ZNK4llvm15AliasSetTracker15containsPointerEPNS_5ValueEyPKNS_6MDNodeE", "llvm::AliasSetTracker::containsPointer(llvm::Value*, unsigned long long, llvm::MDNode const*) const"},
    {"_ZN4llvm15AliasSetTracker23findAliasSetForCallSiteENS_8CallSiteE", "llvm::AliasSetTracker::findAliasSetForCallSite(llvm::CallSite)"},
    {"_ZN4llvm15AliasSetTracker21getAliasSetForPointerEPNS_5ValueEyPKNS_6MDNodeEPb", "llvm::AliasSetTracker::getAliasSetForPointer(llvm::Value*, unsigned long long, llvm::MDNode const*, bool*)"},
    {"_ZN4llvm15AliasSetTracker3addEPNS_5ValueEyPKNS_6MDNodeE", "llvm::AliasSetTracker::add(llvm::Value*, unsigned long long, llvm::MDNode const*)"},
    {"_ZN4llvm15AliasSetTracker3addEPNS_8LoadInstE", "llvm::AliasSetTracker::add(llvm::LoadInst*)"},
    {"_ZN4llvm15AliasSetTracker3addEPNS_9StoreInstE", "llvm::AliasSetTracker::add(llvm::StoreInst*)"},
    {"_ZN4llvm15AliasSetTracker3addEPNS_9VAArgInstE", "llvm::AliasSetTracker::add(llvm::VAArgInst*)"},
    {"_ZN4llvm15AliasSetTracker3addENS_8CallSiteE", "llvm::AliasSetTracker::add(llvm::CallSite)"},
    {"_ZN4llvm15AliasSetTracker3addEPNS_11InstructionE", "llvm::AliasSetTracker::add(llvm::Instruction*)"},
    {"_ZN4llvm15AliasSetTracker3addERNS_10BasicBlockE", "llvm::AliasSetTracker::add(llvm::BasicBlock&)"},
    {"_ZN4llvm15AliasSetTracker3addERKS0_", "llvm::AliasSetTracker::add(llvm::AliasSetTracker const&)"},
    {"_ZN4llvm15AliasSetTracker6removeERNS_8AliasSetE", "llvm::AliasSetTracker::remove(llvm::AliasSet&)"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVHC1EPNS_5ValueEPS0_", "llvm::AliasSetTracker::ASTCallbackVH::ASTCallbackVH(llvm::Value*, llvm::AliasSetTracker*)"},
    {"_ZN4llvm15AliasSetTracker6removeEPNS_5ValueEyPKNS_6MDNodeE", "llvm::AliasSetTracker::remove(llvm::Value*, unsigned long long, llvm::MDNode const*)"},
    {"_ZN4llvm15AliasSetTracker6removeEPNS_8LoadInstE", "llvm::AliasSetTracker::remove(llvm::LoadInst*)"},
    {"_ZN4llvm15AliasSetTracker6removeEPNS_9StoreInstE", "llvm::AliasSetTracker::remove(llvm::StoreInst*)"},
    {"_ZN4llvm15AliasSetTracker6removeEPNS_9VAArgInstE", "llvm::AliasSetTracker::remove(llvm::VAArgInst*)"},
    {"_ZN4llvm15AliasSetTracker6removeENS_8CallSiteE", "llvm::AliasSetTracker::remove(llvm::CallSite)"},
    {"_ZN4llvm15AliasSetTracker6removeEPNS_11InstructionE", "llvm::AliasSetTracker::remove(llvm::Instruction*)"},
    {"_ZN4llvm15AliasSetTracker11deleteValueEPNS_5ValueE", "llvm::AliasSetTracker::deleteValue(llvm::Value*)"},
    {"_ZN4llvm15AliasSetTracker9copyValueEPNS_5ValueES2_", "llvm::AliasSetTracker::copyValue(llvm::Value*, llvm::Value*)"},
    {"_ZNK4llvm8AliasSet5printERNS_11raw_ostreamE", "llvm::AliasSet::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm15AliasSetTracker5printERNS_11raw_ostreamE", "llvm::AliasSetTracker::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm8AliasSet4dumpEv", "llvm::AliasSet::dump() const"},
    {"_ZNK4llvm15AliasSetTracker4dumpEv", "llvm::AliasSetTracker::dump() const"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVH7deletedEv", "llvm::AliasSetTracker::ASTCallbackVH::deleted()"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVHC2EPNS_5ValueEPS0_", "llvm::AliasSetTracker::ASTCallbackVH::ASTCallbackVH(llvm::Value*, llvm::AliasSetTracker*)"},
    {"_ZN4llvm15AliasSetTracker13ASTCallbackVHaSEPNS_5ValueE", "llvm::AliasSetTracker::ASTCallbackVH::operator=(llvm::Value*)"},
    {"_ZN4llvm29initializeAliasSetPrinterPassERNS_12PassRegistryE", "llvm::initializeAliasSetPrinterPass(llvm::PassRegistry&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_115AliasSetPrinterEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::AliasSetPrinter>()"},
    {"_ZN12_GLOBAL__N_115AliasSetPrinterD1Ev", "(anonymous namespace)::AliasSetPrinter::~AliasSetPrinter()"},
    {"_ZN12_GLOBAL__N_115AliasSetPrinterD0Ev", "(anonymous namespace)::AliasSetPrinter::~AliasSetPrinter()"},
    {"_ZNK12_GLOBAL__N_115AliasSetPrinter16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::AliasSetPrinter::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_115AliasSetPrinter13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::AliasSetPrinter::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE5beginEv", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::begin()"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE5clearEv", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::clear()"},
    {"_ZN4llvm15AliasSetTracker11getEntryForEPNS_5ValueE", "llvm::AliasSetTracker::getEntryFor(llvm::Value*)"},
    {"_ZN4llvm8AliasSet18getForwardedTargetERNS_15AliasSetTrackerE", "llvm::AliasSet::getForwardedTarget(llvm::AliasSetTracker&)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE5eraseERKS2_", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::erase(llvm::AliasSetTracker::ASTCallbackVH const&)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE4findERKS2_", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::find(llvm::AliasSetTracker::ASTCallbackVH const&)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE5eraseENS_16DenseMapIteratorIS2_S5_S6_S8_Lb0EEE", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::erase(llvm::DenseMapIterator<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*>, false>)"},
    {"_ZNK4llvm6iplistINS_8AliasSetENS_12ilist_traitsIS1_EEE3endEv", "llvm::iplist<llvm::AliasSet, llvm::ilist_traits<llvm::AliasSet> >::end() const"},
    {"_ZNK4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE15LookupBucketForERKS2_RPSt4pairIS2_S5_E", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::LookupBucketFor(llvm::AliasSetTracker::ASTCallbackVH const&, std::pair<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*>*&) const"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::shrink_and_clear()"},
    {"_ZNSt6vectorIN4llvm11AssertingVHINS0_11InstructionEEESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AssertingVH<llvm::Instruction>*, std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > > >, llvm::AssertingVH<llvm::Instruction> const&)"},
    {"_ZNSt6vectorIN4llvm11AssertingVHINS0_11InstructionEEESaIS3_EE15_M_range_insertIN9__gnu_cxx17__normal_iteratorIPS3_S5_EEEEvSA_T_SB_St20forward_iterator_tag", "void std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > >::_M_range_insert<__gnu_cxx::__normal_iterator<llvm::AssertingVH<llvm::Instruction>*, std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > > > >(__gnu_cxx::__normal_iterator<llvm::AssertingVH<llvm::Instruction>*, std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > > >, __gnu_cxx::__normal_iterator<llvm::AssertingVH<llvm::Instruction>*, std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > > >, __gnu_cxx::__normal_iterator<llvm::AssertingVH<llvm::Instruction>*, std::vector<llvm::AssertingVH<llvm::Instruction>, std::allocator<llvm::AssertingVH<llvm::Instruction> > > >, std::forward_iterator_tag)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::InsertIntoBucket(llvm::AliasSetTracker::ASTCallbackVH const&, llvm::AliasSet::PointerRec* const&, std::pair<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*>*)"},
    {"_ZN4llvm8DenseMapINS_15AliasSetTracker13ASTCallbackVHEPNS_8AliasSet10PointerRecENS1_25ASTCallbackVHDenseMapInfoENS_12DenseMapInfoIS5_EEE4growEj", "llvm::DenseMap<llvm::AliasSetTracker::ASTCallbackVH, llvm::AliasSet::PointerRec*, llvm::AliasSetTracker::ASTCallbackVHDenseMapInfo, llvm::DenseMapInfo<llvm::AliasSet::PointerRec*> >::grow(unsigned int)"},
    {"_ZN4llvm32initializeBasicAliasAnalysisPassERNS_12PassRegistryE", "llvm::initializeBasicAliasAnalysisPass(llvm::PassRegistry&)"},
    {"_ZN4llvm28createBasicAliasAnalysisPassEv", "llvm::createBasicAliasAnalysisPass()"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysisD1Ev", "(anonymous namespace)::BasicAliasAnalysis::~BasicAliasAnalysis()"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysisD0Ev", "(anonymous namespace)::BasicAliasAnalysis::~BasicAliasAnalysis()"},
    {"_ZNK12_GLOBAL__N_118BasicAliasAnalysis16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::BasicAliasAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis26getAdjustedAnalysisPointerEPKv", "(anonymous namespace)::BasicAliasAnalysis::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis14initializePassEv", "(anonymous namespace)::BasicAliasAnalysis::initializePass()"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis5aliasERKN4llvm13AliasAnalysis8LocationES5_", "(anonymous namespace)::BasicAliasAnalysis::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "(anonymous namespace)::BasicAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "(anonymous namespace)::BasicAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "(anonymous namespace)::BasicAliasAnalysis::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "(anonymous namespace)::BasicAliasAnalysis::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis17getModRefBehaviorEPKN4llvm8FunctionE", "(anonymous namespace)::BasicAliasAnalysis::getModRefBehavior(llvm::Function const*)"},
    {"_ZThn32_NK12_GLOBAL__N_118BasicAliasAnalysis16getAnalysisUsageERN4llvm13AnalysisUsageE", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysisD1Ev", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::~BasicAliasAnalysis()"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysisD0Ev", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::~BasicAliasAnalysis()"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis5aliasERKN4llvm13AliasAnalysis8LocationES5_", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis17getModRefBehaviorEPKN4llvm8FunctionE", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::getModRefBehavior(llvm::Function const*)"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_118BasicAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "non-virtual thunk to (anonymous namespace)::BasicAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis10aliasCheckEPKN4llvm5ValueEyPKNS1_6MDNodeES4_yS7_", "(anonymous namespace)::BasicAliasAnalysis::aliasCheck(llvm::Value const*, unsigned long long, llvm::MDNode const*, llvm::Value const*, unsigned long long, llvm::MDNode const*)"},
    {"_Z24isNonEscapingLocalObjectPKN4llvm5ValueE", "isNonEscapingLocalObject(llvm::Value const*)"},
    {"_Z13getObjectSizePKN4llvm5ValueERKNS_10DataLayoutE", "getObjectSize(llvm::Value const*, llvm::DataLayout const&)"},
    {"_Z22DecomposeGEPExpressionPKN4llvm5ValueERxRNS_15SmallVectorImplIN12_GLOBAL__N_116VariableGEPIndexEEEPKNS_10DataLayoutE", "DecomposeGEPExpression(llvm::Value const*, long long&, llvm::SmallVectorImpl<(anonymous namespace)::VariableGEPIndex>&, llvm::DataLayout const*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_116VariableGEPIndexEE9push_backERKS2_", "llvm::SmallVectorImpl<(anonymous namespace)::VariableGEPIndex>::push_back((anonymous namespace)::VariableGEPIndex const&)"},
    {"_Z19GetLinearExpressionPN4llvm5ValueERNS_5APIntES3_RN12_GLOBAL__N_113ExtensionKindERKNS_10DataLayoutEj", "GetLinearExpression(llvm::Value*, llvm::APInt&, llvm::APInt&, (anonymous namespace)::ExtensionKind&, llvm::DataLayout const&, unsigned int)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_118BasicAliasAnalysisEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::BasicAliasAnalysis>()"},
    {"_ZN4llvm20PointerMayBeCapturedEPKNS_5ValueEbb", "llvm::PointerMayBeCaptured(llvm::Value const*, bool, bool)"},
    {"_ZN4llvm28ConstantFoldLoadFromConstPtrEPNS_8ConstantEPKNS_10DataLayoutE", "llvm::ConstantFoldLoadFromConstPtr(llvm::Constant*, llvm::DataLayout const*)"},
    {"_ZN4llvm38ConstantFoldLoadThroughGEPConstantExprEPNS_8ConstantEPNS_12ConstantExprE", "llvm::ConstantFoldLoadThroughGEPConstantExpr(llvm::Constant*, llvm::ConstantExpr*)"},
    {"_Z31FoldReinterpretLoadFromConstPtrPN4llvm8ConstantERKNS_10DataLayoutE", "FoldReinterpretLoadFromConstPtr(llvm::Constant*, llvm::DataLayout const&)"},
    {"_ZN4llvm23ConstantFoldInstructionEPNS_11InstructionEPKNS_10DataLayoutE", "llvm::ConstantFoldInstruction(llvm::Instruction*, llvm::DataLayout const*)"},
    {"_ZN4llvm31ConstantFoldCompareInstOperandsEjPNS_8ConstantES1_PKNS_10DataLayoutE", "llvm::ConstantFoldCompareInstOperands(unsigned int, llvm::Constant*, llvm::Constant*, llvm::DataLayout const*)"},
    {"_ZN4llvm24ConstantFoldInstOperandsEjPKNS_4TypeEPKPNS_8ConstantEjPKNS_10DataLayoutE", "llvm::ConstantFoldInstOperands(unsigned int, llvm::Type const*, llvm::Constant* const*, unsigned int, llvm::DataLayout const*)"},
    {"_ZN4llvm30ConstantFoldConstantExpressionEPKNS_12ConstantExprEPKNS_10DataLayoutE", "llvm::ConstantFoldConstantExpression(llvm::ConstantExpr const*, llvm::DataLayout const*)"},
    {"_ZN4llvm21canConstantFoldCallToEPKNS_8FunctionE", "llvm::canConstantFoldCallTo(llvm::Function const*)"},
    {"_ZN4llvm16ConstantFoldCallEPNS_8FunctionEPKPNS_8ConstantEj", "llvm::ConstantFoldCall(llvm::Function*, llvm::Constant* const*, unsigned int)"},
    {"_Z11FoldBitCastPN4llvm8ConstantEPKNS_4TypeERKNS_10DataLayoutE", "FoldBitCast(llvm::Constant*, llvm::Type const*, llvm::DataLayout const&)"},
    {"_Z14ConstantFoldFPPFddEdPKN4llvm4TypeE", "ConstantFoldFP(double (*)(double), double, llvm::Type const*)"},
    {"_Z20ConstantFoldBinaryFPPFdddEddPKN4llvm4TypeE", "ConstantFoldBinaryFP(double (*)(double, double), double, double, llvm::Type const*)"},
    {"_Z26IsConstantOffsetFromGlobalPN4llvm8ConstantERPNS_11GlobalValueERxRKNS_10DataLayoutE", "IsConstantOffsetFromGlobal(llvm::Constant*, llvm::GlobalValue*&, long long&, llvm::DataLayout const&)"},
    {"_Z18ReadDataFromGlobalPN4llvm8ConstantEyPhjRKNS_10DataLayoutE", "ReadDataFromGlobal(llvm::Constant*, unsigned long long, unsigned char*, unsigned int, llvm::DataLayout const&)"},
    {"_ZN4llvm9DIBuilderC1ERNS_6ModuleE", "llvm::DIBuilder::DIBuilder(llvm::Module&)"},
    {"_ZN4llvm9DIBuilderC2ERNS_6ModuleE", "llvm::DIBuilder::DIBuilder(llvm::Module&)"},
    {"_ZN4llvm9DIBuilder17CreateCompileUnitEjNS_9StringRefES1_S1_bS1_j", "llvm::DIBuilder::CreateCompileUnit(unsigned int, llvm::StringRef, llvm::StringRef, llvm::StringRef, bool, llvm::StringRef, unsigned int)"},
    {"_ZN4llvm9DIBuilder10CreateFileENS_9StringRefES1_", "llvm::DIBuilder::CreateFile(llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm9DIBuilder16CreateEnumeratorENS_9StringRefEy", "llvm::DIBuilder::CreateEnumerator(llvm::StringRef, unsigned long long)"},
    {"_ZN4llvm9DIBuilder15CreateBasicTypeENS_9StringRefEyyj", "llvm::DIBuilder::CreateBasicType(llvm::StringRef, unsigned long long, unsigned long long, unsigned int)"},
    {"_ZN4llvm9DIBuilder19CreateQualifiedTypeEjNS_6DITypeE", "llvm::DIBuilder::CreateQualifiedType(unsigned int, llvm::DIType)"},
    {"_ZN4llvm9DIBuilder17CreatePointerTypeENS_6DITypeEyyNS_9StringRefE", "llvm::DIBuilder::CreatePointerType(llvm::DIType, unsigned long long, unsigned long long, llvm::StringRef)"},
    {"_ZN4llvm9DIBuilder19CreateReferenceTypeENS_6DITypeE", "llvm::DIBuilder::CreateReferenceType(llvm::DIType)"},
    {"_ZN4llvm9DIBuilder13CreateTypedefENS_6DITypeENS_9StringRefENS_6DIFileEj", "llvm::DIBuilder::CreateTypedef(llvm::DIType, llvm::StringRef, llvm::DIFile, unsigned int)"},
    {"_ZN4llvm9DIBuilder12CreateFriendENS_6DITypeES1_", "llvm::DIBuilder::CreateFriend(llvm::DIType, llvm::DIType)"},
    {"_ZN4llvm9DIBuilder17CreateInheritanceENS_6DITypeES1_yj", "llvm::DIBuilder::CreateInheritance(llvm::DIType, llvm::DIType, unsigned long long, unsigned int)"},
    {"_ZN4llvm9DIBuilder16CreateMemberTypeENS_9StringRefENS_6DIFileEjyyyjNS_6DITypeE", "llvm::DIBuilder::CreateMemberType(llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned int, llvm::DIType)"},
    {"_ZN4llvm9DIBuilder15CreateClassTypeENS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyyjNS_6DITypeENS_7DIArrayEPNS_6MDNodeES7_", "llvm::DIBuilder::CreateClassType(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned int, llvm::DIType, llvm::DIArray, llvm::MDNode*, llvm::MDNode*)"},
    {"_ZN4llvm9DIBuilder27CreateTemplateTypeParameterENS_12DIDescriptorENS_9StringRefENS_6DITypeEPNS_6MDNodeEjj", "llvm::DIBuilder::CreateTemplateTypeParameter(llvm::DIDescriptor, llvm::StringRef, llvm::DIType, llvm::MDNode*, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIBuilder28CreateTemplateValueParameterENS_12DIDescriptorENS_9StringRefENS_6DITypeEyPNS_6MDNodeEjj", "llvm::DIBuilder::CreateTemplateValueParameter(llvm::DIDescriptor, llvm::StringRef, llvm::DIType, unsigned long long, llvm::MDNode*, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIBuilder16CreateStructTypeENS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyjNS_7DIArrayEj", "llvm::DIBuilder::CreateStructType(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned int, llvm::DIArray, unsigned int)"},
    {"_ZN4llvm9DIBuilder15CreateUnionTypeENS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyjNS_7DIArrayEj", "llvm::DIBuilder::CreateUnionType(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned int, llvm::DIArray, unsigned int)"},
    {"_ZN4llvm9DIBuilder20CreateSubroutineTypeENS_6DIFileENS_7DIArrayE", "llvm::DIBuilder::CreateSubroutineType(llvm::DIFile, llvm::DIArray)"},
    {"_ZN4llvm9DIBuilder21CreateEnumerationTypeENS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyNS_7DIArrayE", "llvm::DIBuilder::CreateEnumerationType(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, llvm::DIArray)"},
    {"_ZN4llvm9DIBuilder15CreateArrayTypeEyyNS_6DITypeENS_7DIArrayE", "llvm::DIBuilder::CreateArrayType(unsigned long long, unsigned long long, llvm::DIType, llvm::DIArray)"},
    {"_ZN4llvm9DIBuilder16CreateVectorTypeEyyNS_6DITypeENS_7DIArrayE", "llvm::DIBuilder::CreateVectorType(unsigned long long, unsigned long long, llvm::DIType, llvm::DIArray)"},
    {"_ZN4llvm9DIBuilder20CreateArtificialTypeENS_6DITypeE", "llvm::DIBuilder::CreateArtificialType(llvm::DIType)"},
    {"_ZN4llvm9DIBuilder10RetainTypeENS_6DITypeE", "llvm::DIBuilder::RetainType(llvm::DIType)"},
    {"_ZN4llvm9DIBuilder26CreateUnspecifiedParameterEv", "llvm::DIBuilder::CreateUnspecifiedParameter()"},
    {"_ZN4llvm9DIBuilder19CreateTemporaryTypeEv", "llvm::DIBuilder::CreateTemporaryType()"},
    {"_ZN4llvm9DIBuilder19CreateTemporaryTypeENS_6DIFileE", "llvm::DIBuilder::CreateTemporaryType(llvm::DIFile)"},
    {"_ZN4llvm9DIBuilder16GetOrCreateArrayEPKPNS_5ValueEj", "llvm::DIBuilder::GetOrCreateArray(llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm9DIBuilder19GetOrCreateSubrangeExx", "llvm::DIBuilder::GetOrCreateSubrange(long long, long long)"},
    {"_ZN4llvm9DIBuilder20CreateGlobalVariableENS_9StringRefENS_6DIFileEjNS_6DITypeEbPNS_5ValueE", "llvm::DIBuilder::CreateGlobalVariable(llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, llvm::Value*)"},
    {"_ZN4llvm9DIBuilder20CreateStaticVariableENS_12DIDescriptorENS_9StringRefES2_NS_6DIFileEjNS_6DITypeEbPNS_5ValueE", "llvm::DIBuilder::CreateStaticVariable(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, llvm::Value*)"},
    {"_ZN4llvm9DIBuilder19CreateLocalVariableEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjNS_6DITypeEbj", "llvm::DIBuilder::CreateLocalVariable(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, unsigned int)"},
    {"_ZN4llvm9DIBuilder21CreateComplexVariableEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjNS_6DITypeEPKPNS_5ValueEj", "llvm::DIBuilder::CreateComplexVariable(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm9DIBuilder14CreateFunctionENS_12DIDescriptorENS_9StringRefES2_NS_6DIFileEjNS_6DITypeEbbjbPNS_8FunctionE", "llvm::DIBuilder::CreateFunction(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, bool, unsigned int, bool, llvm::Function*)"},
    {"_ZN4llvm9DIBuilder12CreateMethodENS_12DIDescriptorENS_9StringRefES2_NS_6DIFileEjNS_6DITypeEbbjjPNS_6MDNodeEjbPNS_8FunctionE", "llvm::DIBuilder::CreateMethod(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, bool, unsigned int, unsigned int, llvm::MDNode*, unsigned int, bool, llvm::Function*)"},
    {"_ZN4llvm9DIBuilder15CreateNameSpaceENS_12DIDescriptorENS_9StringRefENS_6DIFileEj", "llvm::DIBuilder::CreateNameSpace(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int)"},
    {"_ZN4llvm9DIBuilder18CreateLexicalBlockENS_12DIDescriptorENS_6DIFileEjj", "llvm::DIBuilder::CreateLexicalBlock(llvm::DIDescriptor, llvm::DIFile, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIBuilder13InsertDeclareEPNS_5ValueENS_10DIVariableEPNS_11InstructionE", "llvm::DIBuilder::InsertDeclare(llvm::Value*, llvm::DIVariable, llvm::Instruction*)"},
    {"_ZN4llvm9DIBuilder13InsertDeclareEPNS_5ValueENS_10DIVariableEPNS_10BasicBlockE", "llvm::DIBuilder::InsertDeclare(llvm::Value*, llvm::DIVariable, llvm::BasicBlock*)"},
    {"_ZN4llvm9DIBuilder23InsertDbgValueIntrinsicEPNS_5ValueEyNS_10DIVariableEPNS_11InstructionE", "llvm::DIBuilder::InsertDbgValueIntrinsic(llvm::Value*, unsigned long long, llvm::DIVariable, llvm::Instruction*)"},
    {"_ZN4llvm9DIBuilder23InsertDbgValueIntrinsicEPNS_5ValueEyNS_10DIVariableEPNS_10BasicBlockE", "llvm::DIBuilder::InsertDbgValueIntrinsic(llvm::Value*, unsigned long long, llvm::DIVariable, llvm::BasicBlock*)"},
    {"_ZN4llvm12DIDescriptorC1ENS_6DIFileE", "llvm::DIDescriptor::DIDescriptor(llvm::DIFile)"},
    {"_ZN4llvm12DIDescriptorC2ENS_6DIFileE", "llvm::DIDescriptor::DIDescriptor(llvm::DIFile)"},
    {"_ZN4llvm12DIDescriptorC1ENS_12DISubprogramE", "llvm::DIDescriptor::DIDescriptor(llvm::DISubprogram)"},
    {"_ZN4llvm12DIDescriptorC2ENS_12DISubprogramE", "llvm::DIDescriptor::DIDescriptor(llvm::DISubprogram)"},
    {"_ZN4llvm12DIDescriptorC1ENS_14DILexicalBlockE", "llvm::DIDescriptor::DIDescriptor(llvm::DILexicalBlock)"},
    {"_ZN4llvm12DIDescriptorC2ENS_14DILexicalBlockE", "llvm::DIDescriptor::DIDescriptor(llvm::DILexicalBlock)"},
    {"_ZN4llvm12DIDescriptorC1ENS_10DIVariableE", "llvm::DIDescriptor::DIDescriptor(llvm::DIVariable)"},
    {"_ZN4llvm12DIDescriptorC2ENS_10DIVariableE", "llvm::DIDescriptor::DIDescriptor(llvm::DIVariable)"},
    {"_ZN4llvm12DIDescriptorC1ENS_6DITypeE", "llvm::DIDescriptor::DIDescriptor(llvm::DIType)"},
    {"_ZN4llvm12DIDescriptorC2ENS_6DITypeE", "llvm::DIDescriptor::DIDescriptor(llvm::DIType)"},
    {"_ZNK4llvm12DIDescriptor14getStringFieldEj", "llvm::DIDescriptor::getStringField(unsigned int) const"},
    {"_ZNK4llvm12DIDescriptor14getUInt64FieldEj", "llvm::DIDescriptor::getUInt64Field(unsigned int) const"},
    {"_ZNK4llvm12DIDescriptor18getDescriptorFieldEj", "llvm::DIDescriptor::getDescriptorField(unsigned int) const"},
    {"_ZNK4llvm12DIDescriptor22getGlobalVariableFieldEj", "llvm::DIDescriptor::getGlobalVariableField(unsigned int) const"},
    {"_ZNK4llvm12DIDescriptor16getConstantFieldEj", "llvm::DIDescriptor::getConstantField(unsigned int) const"},
    {"_ZNK4llvm12DIDescriptor16getFunctionFieldEj", "llvm::DIDescriptor::getFunctionField(unsigned int) const"},
    {"_ZNK4llvm10DIVariable18getNumAddrElementsEv", "llvm::DIVariable::getNumAddrElements() const"},
    {"_ZNK4llvm12DIDescriptor11isBasicTypeEv", "llvm::DIDescriptor::isBasicType() const"},
    {"_ZNK4llvm12DIDescriptor13isDerivedTypeEv", "llvm::DIDescriptor::isDerivedType() const"},
    {"_ZNK4llvm12DIDescriptor15isCompositeTypeEv", "llvm::DIDescriptor::isCompositeType() const"},
    {"_ZNK4llvm12DIDescriptor10isVariableEv", "llvm::DIDescriptor::isVariable() const"},
    {"_ZNK4llvm12DIDescriptor6isTypeEv", "llvm::DIDescriptor::isType() const"},
    {"_ZNK4llvm12DIDescriptor12isSubprogramEv", "llvm::DIDescriptor::isSubprogram() const"},
    {"_ZNK4llvm12DIDescriptor16isGlobalVariableEv", "llvm::DIDescriptor::isGlobalVariable() const"},
    {"_ZNK4llvm12DIDescriptor8isGlobalEv", "llvm::DIDescriptor::isGlobal() const"},
    {"_ZNK4llvm12DIDescriptor22isUnspecifiedParameterEv", "llvm::DIDescriptor::isUnspecifiedParameter() const"},
    {"_ZNK4llvm12DIDescriptor7isScopeEv", "llvm::DIDescriptor::isScope() const"},
    {"_ZNK4llvm12DIDescriptor23isTemplateTypeParameterEv", "llvm::DIDescriptor::isTemplateTypeParameter() const"},
    {"_ZNK4llvm12DIDescriptor24isTemplateValueParameterEv", "llvm::DIDescriptor::isTemplateValueParameter() const"},
    {"_ZNK4llvm12DIDescriptor13isCompileUnitEv", "llvm::DIDescriptor::isCompileUnit() const"},
    {"_ZNK4llvm12DIDescriptor6isFileEv", "llvm::DIDescriptor::isFile() const"},
    {"_ZNK4llvm12DIDescriptor11isNameSpaceEv", "llvm::DIDescriptor::isNameSpace() const"},
    {"_ZNK4llvm12DIDescriptor14isLexicalBlockEv", "llvm::DIDescriptor::isLexicalBlock() const"},
    {"_ZNK4llvm12DIDescriptor10isSubrangeEv", "llvm::DIDescriptor::isSubrange() const"},
    {"_ZNK4llvm12DIDescriptor12isEnumeratorEv", "llvm::DIDescriptor::isEnumerator() const"},
    {"_ZN4llvm6DITypeC1EPKNS_6MDNodeE", "llvm::DIType::DIType(llvm::MDNode const*)"},
    {"_ZN4llvm6DITypeC2EPKNS_6MDNodeE", "llvm::DIType::DIType(llvm::MDNode const*)"},
    {"_ZNK4llvm7DIArray14getNumElementsEv", "llvm::DIArray::getNumElements() const"},
    {"_ZN4llvm6DIType18replaceAllUsesWithERNS_12DIDescriptorE", "llvm::DIType::replaceAllUsesWith(llvm::DIDescriptor&)"},
    {"_ZN4llvm6DIType18replaceAllUsesWithEPNS_6MDNodeE", "llvm::DIType::replaceAllUsesWith(llvm::MDNode*)"},
    {"_ZNK4llvm13DICompileUnit6VerifyEv", "llvm::DICompileUnit::Verify() const"},
    {"_ZNK4llvm6DIType6VerifyEv", "llvm::DIType::Verify() const"},
    {"_ZNK4llvm11DIBasicType6VerifyEv", "llvm::DIBasicType::Verify() const"},
    {"_ZNK4llvm13DIDerivedType6VerifyEv", "llvm::DIDerivedType::Verify() const"},
    {"_ZNK4llvm15DICompositeType6VerifyEv", "llvm::DICompositeType::Verify() const"},
    {"_ZNK4llvm12DISubprogram6VerifyEv", "llvm::DISubprogram::Verify() const"},
    {"_ZNK4llvm16DIGlobalVariable6VerifyEv", "llvm::DIGlobalVariable::Verify() const"},
    {"_ZNK4llvm10DIVariable6VerifyEv", "llvm::DIVariable::Verify() const"},
    {"_ZNK4llvm10DILocation6VerifyEv", "llvm::DILocation::Verify() const"},
    {"_ZNK4llvm11DINameSpace6VerifyEv", "llvm::DINameSpace::Verify() const"},
    {"_ZNK4llvm13DIDerivedType19getOriginalTypeSizeEv", "llvm::DIDerivedType::getOriginalTypeSize() const"},
    {"_ZN4llvm10DIVariable19isInlinedFnArgumentEPKNS_8FunctionE", "llvm::DIVariable::isInlinedFnArgument(llvm::Function const*)"},
    {"_ZN4llvm12DISubprogram9describesEPKNS_8FunctionE", "llvm::DISubprogram::describes(llvm::Function const*)"},
    {"_ZNK4llvm12DISubprogram11isOptimizedEv", "llvm::DISubprogram::isOptimized() const"},
    {"_ZNK4llvm7DIScope11getFilenameEv", "llvm::DIScope::getFilename() const"},
    {"_ZNK4llvm7DIScope12getDirectoryEv", "llvm::DIScope::getDirectory() const"},
    {"_ZNK4llvm12DIDescriptor5printERNS_11raw_ostreamE", "llvm::DIDescriptor::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm13DICompileUnit5printERNS_11raw_ostreamE", "llvm::DICompileUnit::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm6DIType5printERNS_11raw_ostreamE", "llvm::DIType::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm11DIBasicType5printERNS_11raw_ostreamE", "llvm::DIBasicType::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm13DIDerivedType5printERNS_11raw_ostreamE", "llvm::DIDerivedType::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm15DICompositeType5printERNS_11raw_ostreamE", "llvm::DICompositeType::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm12DISubprogram5printERNS_11raw_ostreamE", "llvm::DISubprogram::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm16DIGlobalVariable5printERNS_11raw_ostreamE", "llvm::DIGlobalVariable::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm10DIVariable5printERNS_11raw_ostreamE", "llvm::DIVariable::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm12DIDescriptor4dumpEv", "llvm::DIDescriptor::dump() const"},
    {"_ZNK4llvm13DICompileUnit4dumpEv", "llvm::DICompileUnit::dump() const"},
    {"_ZNK4llvm6DIType4dumpEv", "llvm::DIType::dump() const"},
    {"_ZNK4llvm11DIBasicType4dumpEv", "llvm::DIBasicType::dump() const"},
    {"_ZNK4llvm13DIDerivedType4dumpEv", "llvm::DIDerivedType::dump() const"},
    {"_ZNK4llvm15DICompositeType4dumpEv", "llvm::DICompositeType::dump() const"},
    {"_ZNK4llvm12DISubprogram4dumpEv", "llvm::DISubprogram::dump() const"},
    {"_ZNK4llvm16DIGlobalVariable4dumpEv", "llvm::DIGlobalVariable::dump() const"},
    {"_ZNK4llvm10DIVariable4dumpEv", "llvm::DIVariable::dump() const"},
    {"_ZN4llvm9DIFactoryC1ERNS_6ModuleE", "llvm::DIFactory::DIFactory(llvm::Module&)"},
    {"_ZN4llvm9DIFactoryC2ERNS_6ModuleE", "llvm::DIFactory::DIFactory(llvm::Module&)"},
    {"_ZN4llvm9DIFactory14GetTagConstantEj", "llvm::DIFactory::GetTagConstant(unsigned int)"},
    {"_ZN4llvm9DIFactory16GetOrCreateArrayEPNS_12DIDescriptorEj", "llvm::DIFactory::GetOrCreateArray(llvm::DIDescriptor*, unsigned int)"},
    {"_ZN4llvm9DIFactory19GetOrCreateSubrangeExx", "llvm::DIFactory::GetOrCreateSubrange(long long, long long)"},
    {"_ZN4llvm9DIFactory26CreateUnspecifiedParameterEv", "llvm::DIFactory::CreateUnspecifiedParameter()"},
    {"_ZN4llvm9DIFactory17CreateCompileUnitEjNS_9StringRefES1_S1_bbS1_j", "llvm::DIFactory::CreateCompileUnit(unsigned int, llvm::StringRef, llvm::StringRef, llvm::StringRef, bool, bool, llvm::StringRef, unsigned int)"},
    {"_ZN4llvm9DIFactory10CreateFileENS_9StringRefES1_NS_13DICompileUnitE", "llvm::DIFactory::CreateFile(llvm::StringRef, llvm::StringRef, llvm::DICompileUnit)"},
    {"_ZN4llvm9DIFactory16CreateEnumeratorENS_9StringRefEy", "llvm::DIFactory::CreateEnumerator(llvm::StringRef, unsigned long long)"},
    {"_ZN4llvm9DIFactory15CreateBasicTypeENS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyyjj", "llvm::DIFactory::CreateBasicType(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIFactory17CreateBasicTypeExENS_12DIDescriptorENS_9StringRefENS_6DIFileEjPNS_8ConstantES5_S5_jj", "llvm::DIFactory::CreateBasicTypeEx(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::Constant*, llvm::Constant*, llvm::Constant*, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIFactory20CreateArtificialTypeENS_6DITypeE", "llvm::DIFactory::CreateArtificialType(llvm::DIType)"},
    {"_ZN4llvm9DIFactory17CreateDerivedTypeEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyyjNS_6DITypeE", "llvm::DIFactory::CreateDerivedType(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned int, llvm::DIType)"},
    {"_ZN4llvm9DIFactory19CreateDerivedTypeExEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjPNS_8ConstantES5_S5_jNS_6DITypeE", "llvm::DIFactory::CreateDerivedTypeEx(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::Constant*, llvm::Constant*, llvm::Constant*, unsigned int, llvm::DIType)"},
    {"_ZN4llvm9DIFactory19CreateCompositeTypeEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjyyyjNS_6DITypeENS_7DIArrayEjPNS_6MDNodeE", "llvm::DIFactory::CreateCompositeType(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned int, llvm::DIType, llvm::DIArray, unsigned int, llvm::MDNode*)"},
    {"_ZN4llvm9DIFactory19CreateTemporaryTypeEv", "llvm::DIFactory::CreateTemporaryType()"},
    {"_ZN4llvm9DIFactory19CreateTemporaryTypeENS_6DIFileE", "llvm::DIFactory::CreateTemporaryType(llvm::DIFile)"},
    {"_ZN4llvm9DIFactory21CreateCompositeTypeExEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjPNS_8ConstantES5_S5_jNS_6DITypeENS_7DIArrayEjPNS_6MDNodeE", "llvm::DIFactory::CreateCompositeTypeEx(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::Constant*, llvm::Constant*, llvm::Constant*, unsigned int, llvm::DIType, llvm::DIArray, unsigned int, llvm::MDNode*)"},
    {"_ZN4llvm9DIFactory16CreateSubprogramENS_12DIDescriptorENS_9StringRefES2_S2_NS_6DIFileEjNS_6DITypeEbbjjS4_jbPNS_8FunctionE", "llvm::DIFactory::CreateSubprogram(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, bool, unsigned int, unsigned int, llvm::DIType, unsigned int, bool, llvm::Function*)"},
    {"_ZN4llvm9DIFactory26CreateSubprogramDefinitionERNS_12DISubprogramE", "llvm::DIFactory::CreateSubprogramDefinition(llvm::DISubprogram&)"},
    {"_ZN4llvm9DIFactory20CreateGlobalVariableENS_12DIDescriptorENS_9StringRefES2_S2_NS_6DIFileEjNS_6DITypeEbbPNS_14GlobalVariableE", "llvm::DIFactory::CreateGlobalVariable(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, bool, llvm::GlobalVariable*)"},
    {"_ZN4llvm9DIFactory20CreateGlobalVariableENS_12DIDescriptorENS_9StringRefES2_S2_NS_6DIFileEjNS_6DITypeEbbPNS_8ConstantE", "llvm::DIFactory::CreateGlobalVariable(llvm::DIDescriptor, llvm::StringRef, llvm::StringRef, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, bool, llvm::Constant*)"},
    {"_ZN4llvm27getOrInsertFnSpecificMDNodeERNS_6ModuleENS_9StringRefE", "llvm::getOrInsertFnSpecificMDNode(llvm::Module&, llvm::StringRef)"},
    {"_ZN4llvm19getFnSpecificMDNodeERKNS_6ModuleENS_9StringRefE", "llvm::getFnSpecificMDNode(llvm::Module const&, llvm::StringRef)"},
    {"_ZN4llvm9DIFactory14CreateVariableEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjNS_6DITypeEbj", "llvm::DIFactory::CreateVariable(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, bool, unsigned int)"},
    {"_ZN4llvm15getDISubprogramEPKNS_6MDNodeE", "llvm::getDISubprogram(llvm::MDNode const*)"},
    {"_ZN4llvm9DIFactory21CreateComplexVariableEjNS_12DIDescriptorENS_9StringRefENS_6DIFileEjNS_6DITypeEPKPNS_5ValueEj", "llvm::DIFactory::CreateComplexVariable(unsigned int, llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int, llvm::DIType, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm9DIFactory18CreateLexicalBlockENS_12DIDescriptorENS_6DIFileEjj", "llvm::DIFactory::CreateLexicalBlock(llvm::DIDescriptor, llvm::DIFile, unsigned int, unsigned int)"},
    {"_ZN4llvm9DIFactory15CreateNameSpaceENS_12DIDescriptorENS_9StringRefENS_6DIFileEj", "llvm::DIFactory::CreateNameSpace(llvm::DIDescriptor, llvm::StringRef, llvm::DIFile, unsigned int)"},
    {"_ZN4llvm9DIFactory14CreateLocationEjjNS_7DIScopeENS_10DILocationE", "llvm::DIFactory::CreateLocation(unsigned int, unsigned int, llvm::DIScope, llvm::DILocation)"},
    {"_ZN4llvm9DIFactory13InsertDeclareEPNS_5ValueENS_10DIVariableEPNS_11InstructionE", "llvm::DIFactory::InsertDeclare(llvm::Value*, llvm::DIVariable, llvm::Instruction*)"},
    {"_ZN4llvm9DIFactory13InsertDeclareEPNS_5ValueENS_10DIVariableEPNS_10BasicBlockE", "llvm::DIFactory::InsertDeclare(llvm::Value*, llvm::DIVariable, llvm::BasicBlock*)"},
    {"_ZN4llvm9DIFactory23InsertDbgValueIntrinsicEPNS_5ValueEyNS_10DIVariableEPNS_11InstructionE", "llvm::DIFactory::InsertDbgValueIntrinsic(llvm::Value*, unsigned long long, llvm::DIVariable, llvm::Instruction*)"},
    {"_ZN4llvm9DIFactory23InsertDbgValueIntrinsicEPNS_5ValueEyNS_10DIVariableEPNS_10BasicBlockE", "llvm::DIFactory::InsertDbgValueIntrinsic(llvm::Value*, unsigned long long, llvm::DIVariable, llvm::BasicBlock*)"},
    {"_ZN4llvm9DIFactory10RecordTypeENS_6DITypeE", "llvm::DIFactory::RecordType(llvm::DIType)"},
    {"_ZN4llvm15DebugInfoFinder13processModuleERNS_6ModuleE", "llvm::DebugInfoFinder::processModule(llvm::Module&)"},
    {"_ZN4llvm15DebugInfoFinder14processDeclareEPNS_14DbgDeclareInstE", "llvm::DebugInfoFinder::processDeclare(llvm::DbgDeclareInst*)"},
    {"_ZN4llvm15DebugInfoFinder14addCompileUnitENS_13DICompileUnitE", "llvm::DebugInfoFinder::addCompileUnit(llvm::DICompileUnit)"},
    {"_ZN4llvm15DebugInfoFinder17processSubprogramENS_12DISubprogramE", "llvm::DebugInfoFinder::processSubprogram(llvm::DISubprogram)"},
    {"_ZN4llvm15DebugInfoFinder19processLexicalBlockENS_14DILexicalBlockE", "llvm::DebugInfoFinder::processLexicalBlock(llvm::DILexicalBlock)"},
    {"_ZN4llvm15DebugInfoFinder15processLocationENS_10DILocationE", "llvm::DebugInfoFinder::processLocation(llvm::DILocation)"},
    {"_ZN4llvm15DebugInfoFinder17addGlobalVariableENS_16DIGlobalVariableE", "llvm::DebugInfoFinder::addGlobalVariable(llvm::DIGlobalVariable)"},
    {"_ZN4llvm15DebugInfoFinder11processTypeENS_6DITypeE", "llvm::DebugInfoFinder::processType(llvm::DIType)"},
    {"_ZN4llvm15DebugInfoFinder7addTypeENS_6DITypeE", "llvm::DebugInfoFinder::addType(llvm::DIType)"},
    {"_ZN4llvm15DebugInfoFinder13addSubprogramENS_12DISubprogramE", "llvm::DebugInfoFinder::addSubprogram(llvm::DISubprogram)"},
    {"_ZN4llvm15getLocationInfoEPKNS_5ValueERSsS3_RjS3_S3_", "llvm::getLocationInfo(llvm::Value const*, std::string&, std::string&, unsigned int&, std::string&, std::string&)"},
    {"_ZN4llvm18getDICompositeTypeENS_6DITypeE", "llvm::getDICompositeType(llvm::DIType)"},
    {"_ZNK4llvm6DIType11getFilenameEv", "llvm::DIType::getFilename() const"},
    {"_ZNK4llvm6DIType14getCompileUnitEv", "llvm::DIType::getCompileUnit() const"},
    {"_ZN4llvm13DICompileUnitD1Ev", "llvm::DICompileUnit::~DICompileUnit()"},
    {"_ZNK4llvm12DISubprogram14getCompileUnitEv", "llvm::DISubprogram::getCompileUnit() const"},
    {"_ZN4llvm15DICompositeTypeD1Ev", "llvm::DICompositeType::~DICompositeType()"},
    {"_ZNK4llvm16DIGlobalVariable14getCompileUnitEv", "llvm::DIGlobalVariable::getCompileUnit() const"},
    {"_ZN4llvm6DITypeD1Ev", "llvm::DIType::~DIType()"},
    {"_ZNK4llvm10DIVariable14getCompileUnitEv", "llvm::DIVariable::getCompileUnit() const"},
    {"_ZNK4llvm11DINameSpace14getCompileUnitEv", "llvm::DINameSpace::getCompileUnit() const"},
    {"_ZNK4llvm6DIType7isValidEv", "llvm::DIType::isValid() const"},
    {"_ZN4llvm13DIDerivedTypeD1Ev", "llvm::DIDerivedType::~DIDerivedType()"},
    {"_ZNK4llvm14DILexicalBlock11getFilenameEv", "llvm::DILexicalBlock::getFilename() const"},
    {"_ZN4llvm14DILexicalBlockD1Ev", "llvm::DILexicalBlock::~DILexicalBlock()"},
    {"_ZNK4llvm12DISubprogram11getFilenameEv", "llvm::DISubprogram::getFilename() const"},
    {"_ZNK4llvm11DINameSpace11getFilenameEv", "llvm::DINameSpace::getFilename() const"},
    {"_ZN4llvm11DINameSpaceD1Ev", "llvm::DINameSpace::~DINameSpace()"},
    {"_ZN4llvm6DIFileD1Ev", "llvm::DIFile::~DIFile()"},
    {"_ZNK4llvm14DILexicalBlock12getDirectoryEv", "llvm::DILexicalBlock::getDirectory() const"},
    {"_ZNK4llvm12DISubprogram12getDirectoryEv", "llvm::DISubprogram::getDirectory() const"},
    {"_ZNK4llvm11DINameSpace12getDirectoryEv", "llvm::DINameSpace::getDirectory() const"},
    {"_ZNK4llvm6DIType12getDirectoryEv", "llvm::DIType::getDirectory() const"},
    {"_ZN4llvm11DIBasicTypeD1Ev", "llvm::DIBasicType::~DIBasicType()"},
    {"_ZN4llvm6DITypeD0Ev", "llvm::DIType::~DIType()"},
    {"_ZN4llvm8CallInstC2IPPNS_5ValueEEES3_T_S5_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CallInst::CallInst<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInstC2IPPNS_5ValueEEES3_T_S5_RKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm15DICompositeTypeD0Ev", "llvm::DICompositeType::~DICompositeType()"},
    {"_ZN4llvm13DIDerivedTypeD0Ev", "llvm::DIDerivedType::~DIDerivedType()"},
    {"_ZN4llvm11DIBasicTypeD0Ev", "llvm::DIBasicType::~DIBasicType()"},
    {"_ZN4llvm6DIFileD0Ev", "llvm::DIFile::~DIFile()"},
    {"_ZN4llvm11DINameSpaceD0Ev", "llvm::DINameSpace::~DINameSpace()"},
    {"_ZN4llvm13DICompileUnitD0Ev", "llvm::DICompileUnit::~DICompileUnit()"},
    {"_ZN4llvm14DILexicalBlockD0Ev", "llvm::DILexicalBlock::~DILexicalBlock()"},
    {"_ZNK4llvm12DIDescriptor10getFieldAsINS_15DICompositeTypeEEET_j", "llvm::DICompositeType llvm::DIDescriptor::getFieldAs<llvm::DICompositeType>(unsigned int) const"},
    {"_ZN4llvm21initializeIVUsersPassERNS_12PassRegistryE", "llvm::initializeIVUsersPass(llvm::PassRegistry&)"},
    {"_ZN4llvm17createIVUsersPassEv", "llvm::createIVUsersPass()"},
    {"_ZN4llvm7IVUsersC1Ev", "llvm::IVUsers::IVUsers()"},
    {"_ZN4llvm7IVUsers21AddUsersIfInterestingEPNS_11InstructionE", "llvm::IVUsers::AddUsersIfInteresting(llvm::Instruction*)"},
    {"_Z13isInterestingPKN4llvm4SCEVEPKNS_11InstructionEPKNS_4LoopEPNS_15ScalarEvolutionE", "isInteresting(llvm::SCEV const*, llvm::Instruction const*, llvm::Loop const*, llvm::ScalarEvolution*)"},
    {"_ZN4llvm7IVUsers7AddUserEPNS_11InstructionEPNS_5ValueE", "llvm::IVUsers::AddUser(llvm::Instruction*, llvm::Value*)"},
    {"_ZN4llvm7IVUsersC2Ev", "llvm::IVUsers::IVUsers()"},
    {"_ZNK4llvm7IVUsers16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::IVUsers::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm7IVUsers9runOnLoopEPNS_4LoopERNS_13LPPassManagerE", "llvm::IVUsers::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZNK4llvm7IVUsers5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::IVUsers::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm7IVUsers18getReplacementExprERKNS_11IVStrideUseE", "llvm::IVUsers::getReplacementExpr(llvm::IVStrideUse const&) const"},
    {"_ZNK4llvm7IVUsers4dumpEv", "llvm::IVUsers::dump() const"},
    {"_ZN4llvm7IVUsers13releaseMemoryEv", "llvm::IVUsers::releaseMemory()"},
    {"_ZNK4llvm7IVUsers7getExprERKNS_11IVStrideUseE", "llvm::IVUsers::getExpr(llvm::IVStrideUse const&) const"},
    {"_ZNK4llvm7IVUsers9getStrideERKNS_11IVStrideUseEPKNS_4LoopE", "llvm::IVUsers::getStride(llvm::IVStrideUse const&, llvm::Loop const*) const"},
    {"_Z17findAddRecForLoopPKN4llvm4SCEVEPKNS_4LoopE", "findAddRecForLoop(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm11IVStrideUse18transformToPostIncEPKNS_4LoopE", "llvm::IVStrideUse::transformToPostInc(llvm::Loop const*)"},
    {"_ZN4llvm11IVStrideUse7deletedEv", "llvm::IVStrideUse::deleted()"},
    {"_ZN4llvm11IVStrideUseD1Ev", "llvm::IVStrideUse::~IVStrideUse()"},
    {"_ZN4llvm11IVStrideUseD0Ev", "llvm::IVStrideUse::~IVStrideUse()"},
    {"_ZN4llvm7IVUsersD1Ev", "llvm::IVUsers::~IVUsers()"},
    {"_ZN4llvm7IVUsersD0Ev", "llvm::IVUsers::~IVUsers()"},
    {"_ZN4llvm15callDefaultCtorINS_7IVUsersEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::IVUsers>()"},
    {"_ZN4llvm11callIsSmallEPKNS_8FunctionE", "llvm::callIsSmall(llvm::Function const*)"},
    {"_ZN4llvm11CodeMetrics17analyzeBasicBlockEPKNS_10BasicBlockE", "llvm::CodeMetrics::analyzeBasicBlock(llvm::BasicBlock const*)"},
    {"_ZN4llvm11CodeMetrics29CountCodeReductionForConstantEPNS_5ValueE", "llvm::CodeMetrics::CountCodeReductionForConstant(llvm::Value*)"},
    {"_ZN4llvm11CodeMetrics27CountCodeReductionForAllocaEPNS_5ValueE", "llvm::CodeMetrics::CountCodeReductionForAlloca(llvm::Value*)"},
    {"_ZN4llvm11CodeMetrics15analyzeFunctionEPNS_8FunctionE", "llvm::CodeMetrics::analyzeFunction(llvm::Function*)"},
    {"_ZN4llvm18InlineCostAnalyzer12FunctionInfo15analyzeFunctionEPNS_8FunctionE", "llvm::InlineCostAnalyzer::FunctionInfo::analyzeFunction(llvm::Function*)"},
    {"_ZN4llvm18InlineCostAnalyzer12FunctionInfo11NeverInlineEv", "llvm::InlineCostAnalyzer::FunctionInfo::NeverInline()"},
    {"_ZN4llvm18InlineCostAnalyzer22getSpecializationBonusEPNS_8FunctionERNS_15SmallVectorImplIjEE", "llvm::InlineCostAnalyzer::getSpecializationBonus(llvm::Function*, llvm::SmallVectorImpl<unsigned int>&)"},
    {"_ZN4llvm18InlineCostAnalyzer21CountBonusForConstantEPNS_5ValueEPNS_8ConstantE", "llvm::InlineCostAnalyzer::CountBonusForConstant(llvm::Value*, llvm::Constant*)"},
    {"_ZN4llvm18InlineCostAnalyzer21ConstantFunctionBonusENS_8CallSiteEPNS_8ConstantE", "llvm::InlineCostAnalyzer::ConstantFunctionBonus(llvm::CallSite, llvm::Constant*)"},
    {"_ZN4llvm18InlineCostAnalyzer13getInlineSizeENS_8CallSiteEPNS_8FunctionE", "llvm::InlineCostAnalyzer::getInlineSize(llvm::CallSite, llvm::Function*)"},
    {"_ZN4llvm18InlineCostAnalyzer16getInlineBonusesENS_8CallSiteEPNS_8FunctionE", "llvm::InlineCostAnalyzer::getInlineBonuses(llvm::CallSite, llvm::Function*)"},
    {"_ZN4llvm18InlineCostAnalyzer13getInlineCostENS_8CallSiteERNS_11SmallPtrSetIPKNS_8FunctionELj16EEE", "llvm::InlineCostAnalyzer::getInlineCost(llvm::CallSite, llvm::SmallPtrSet<llvm::Function const*, 16u>&)"},
    {"_ZN4llvm18InlineCostAnalyzer13getInlineCostENS_8CallSiteEPNS_8FunctionERNS_11SmallPtrSetIPKS2_Lj16EEE", "llvm::InlineCostAnalyzer::getInlineCost(llvm::CallSite, llvm::Function*, llvm::SmallPtrSet<llvm::Function const*, 16u>&)"},
    {"_ZN4llvm18InlineCostAnalyzer21getSpecializationCostEPNS_8FunctionERNS_15SmallVectorImplIjEE", "llvm::InlineCostAnalyzer::getSpecializationCost(llvm::Function*, llvm::SmallVectorImpl<unsigned int>&)"},
    {"_ZN4llvm18InlineCostAnalyzer20getInlineFudgeFactorENS_8CallSiteE", "llvm::InlineCostAnalyzer::getInlineFudgeFactor(llvm::CallSite)"},
    {"_ZN4llvm18InlineCostAnalyzer18growCachedCostInfoEPNS_8FunctionES2_", "llvm::InlineCostAnalyzer::growCachedCostInfo(llvm::Function*, llvm::Function*)"},
    {"_ZN4llvm18InlineCostAnalyzer5clearEv", "llvm::InlineCostAnalyzer::clear()"},
    {"_ZNSt6vectorIN4llvm18InlineCostAnalyzer7ArgInfoESaIS2_EE7reserveEm", "std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> >::reserve(unsigned long)"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E5clearEv", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::clear()"},
    {"_ZN4llvm8DenseMapINS_18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS4_EENS_12DenseMapInfoIS6_EEEES6_NS9_ISB_EESA_E16shrink_and_clearEv", "llvm::DenseMap<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >, llvm::InlineCostAnalyzer::FunctionInfo, llvm::DenseMapInfo<llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> > >, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >::shrink_and_clear()"},
    {"_ZNSt6vectorIN4llvm18InlineCostAnalyzer7ArgInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::InlineCostAnalyzer::ArgInfo*, std::vector<llvm::InlineCostAnalyzer::ArgInfo, std::allocator<llvm::InlineCostAnalyzer::ArgInfo> > >, llvm::InlineCostAnalyzer::ArgInfo const&)"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::BasicBlock const*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::BasicBlock const* const&, unsigned int const&, std::pair<llvm::BasicBlock const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_10BasicBlockEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::BasicBlock const*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm15SimplifyAddInstEPNS_5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyAddInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifyAddInstPN4llvm5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyAddInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm15SimplifySubInstEPNS_5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifySubInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifySubInstPN4llvm5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifySubInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm15SimplifyMulInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyMulInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifyMulInstPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyMulInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm16SimplifySDivInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifySDivInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm16SimplifyUDivInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyUDivInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm16SimplifyFDivInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyFDivInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm15SimplifyShlInstEPNS_5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyShlInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifyShlInstPN4llvm5ValueES1_bbPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyShlInst(llvm::Value*, llvm::Value*, bool, bool, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm16SimplifyLShrInstEPNS_5ValueES1_bPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyLShrInst(llvm::Value*, llvm::Value*, bool, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z16SimplifyLShrInstPN4llvm5ValueES1_bPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyLShrInst(llvm::Value*, llvm::Value*, bool, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm16SimplifyAShrInstEPNS_5ValueES1_bPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyAShrInst(llvm::Value*, llvm::Value*, bool, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z16SimplifyAShrInstPN4llvm5ValueES1_bPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyAShrInst(llvm::Value*, llvm::Value*, bool, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm15SimplifyAndInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyAndInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifyAndInstPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyAndInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm14SimplifyOrInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyOrInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z14SimplifyOrInstPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyOrInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm15SimplifyXorInstEPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyXorInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z15SimplifyXorInstPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyXorInst(llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm16SimplifyICmpInstEjPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyICmpInst(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z16SimplifyICmpInstjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyICmpInst(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm16SimplifyFCmpInstEjPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyFCmpInst(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z16SimplifyFCmpInstjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyFCmpInst(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm18SimplifySelectInstEPNS_5ValueES1_S1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifySelectInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm15SimplifyGEPInstEPKPNS_5ValueEjPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyGEPInst(llvm::Value* const*, unsigned int, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm13SimplifyBinOpEjPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyBinOp(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z13SimplifyBinOpjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyBinOp(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm15SimplifyCmpInstEjPNS_5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyCmpInst(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm19SimplifyInstructionEPNS_11InstructionEPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::SimplifyInstruction(llvm::Instruction*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_ZN4llvm25ReplaceAndSimplifyAllUsesEPNS_11InstructionEPNS_5ValueEPKNS_10DataLayoutEPKNS_13DominatorTreeE", "llvm::ReplaceAndSimplifyAllUses(llvm::Instruction*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*)"},
    {"_Z24SimplifyAssociativeBinOpjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyAssociativeBinOp(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z21ThreadBinOpOverSelectjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "ThreadBinOpOverSelect(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z18ThreadBinOpOverPHIjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "ThreadBinOpOverPHI(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z19ThreadCmpOverSelectN4llvm7CmpInst9PredicateEPNS_5ValueES3_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "ThreadCmpOverSelect(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z16ThreadCmpOverPHIN4llvm7CmpInst9PredicateEPNS_5ValueES3_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "ThreadCmpOverPHI(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z14FactorizeBinOpjPN4llvm5ValueES1_jPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "FactorizeBinOp(unsigned int, llvm::Value*, llvm::Value*, unsigned int, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z11ExpandBinOpjPN4llvm5ValueES1_jPKNS_10DataLayoutEPKNS_13DominatorTreeEj", "ExpandBinOp(unsigned int, llvm::Value*, llvm::Value*, unsigned int, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z13SimplifyShiftjPN4llvm5ValueES1_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyShift(unsigned int, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_Z11SimplifyDivN4llvm11Instruction9BinaryOpsEPNS_5ValueES3_PKNS_10DataLayoutEPKNS_13DominatorTreeEj", "SimplifyDiv(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::DataLayout const*, llvm::DominatorTree const*, unsigned int)"},
    {"_ZN4llvm12PatternMatch9not_matchINS0_14specificval_tyEE5matchINS_5ValueEEEbPT_", "bool llvm::PatternMatch::not_match<llvm::PatternMatch::specificval_ty>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch11cst_pred_tyINS0_11is_all_onesEE5matchINS_5ValueEEEbPT_", "bool llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_all_ones>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_14specificval_tyENS0_11cst_pred_tyINS0_6is_oneEEELj20EE5matchINS_5ValueEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::specificval_ty, llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_one>, 20u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch11cst_pred_tyINS0_6is_oneEE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_one>::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_14specificval_tyENS0_17constantint_matchILx2EEELj12EE5matchINS_5ValueEEEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::specificval_ty, llvm::PatternMatch::constantint_match<2ll>, 12u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm27initializeLazyValueInfoPassERNS_12PassRegistryE", "llvm::initializeLazyValueInfoPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createLazyValueInfoPassEv", "llvm::createLazyValueInfoPass()"},
    {"_ZN4llvm13LazyValueInfo13runOnFunctionERNS_8FunctionE", "llvm::LazyValueInfo::runOnFunction(llvm::Function&)"},
    {"_Z8getCacheRPv", "getCache(void*&)"},
    {"_ZN4llvm13LazyValueInfo13releaseMemoryEv", "llvm::LazyValueInfo::releaseMemory()"},
    {"_ZN4llvm13LazyValueInfo11getConstantEPNS_5ValueEPNS_10BasicBlockE", "llvm::LazyValueInfo::getConstant(llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm13LazyValueInfo17getConstantOnEdgeEPNS_5ValueEPNS_10BasicBlockES4_", "llvm::LazyValueInfo::getConstantOnEdge(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm13LazyValueInfo18getPredicateOnEdgeEjPNS_5ValueEPNS_8ConstantEPNS_10BasicBlockES6_", "llvm::LazyValueInfo::getPredicateOnEdge(unsigned int, llvm::Value*, llvm::Constant*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm13LazyValueInfo10threadEdgeEPNS_10BasicBlockES2_S2_", "llvm::LazyValueInfo::threadEdge(llvm::BasicBlock*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm13LazyValueInfo10eraseBlockEPNS_10BasicBlockE", "llvm::LazyValueInfo::eraseBlock(llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_114LVIValueHandleD1Ev", "(anonymous namespace)::LVIValueHandle::~LVIValueHandle()"},
    {"_ZN12_GLOBAL__N_114LVIValueHandleD0Ev", "(anonymous namespace)::LVIValueHandle::~LVIValueHandle()"},
    {"_ZN12_GLOBAL__N_114LVIValueHandle7deletedEv", "(anonymous namespace)::LVIValueHandle::deleted()"},
    {"_ZN12_GLOBAL__N_114LVIValueHandle19allUsesReplacedWithEPN4llvm5ValueE", "(anonymous namespace)::LVIValueHandle::allUsesReplacedWith(llvm::Value*)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_114LVIValueHandleESt3mapINS_11AssertingVHINS_10BasicBlockEEENS1_13LVILatticeValESt4lessIS6_ESaISt4pairIKS6_S7_EEENS_12DenseMapInfoIS2_EENSF_ISE_EEE15LookupBucketForERKS2_RPSA_IS2_SE_E", "llvm::DenseMap<(anonymous namespace)::LVIValueHandle, std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >, llvm::DenseMapInfo<(anonymous namespace)::LVIValueHandle>, llvm::DenseMapInfo<std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > > > >::LookupBucketFor((anonymous namespace)::LVIValueHandle const&, std::pair<(anonymous namespace)::LVIValueHandle, std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > > >*&) const"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_10BasicBlockEEESt4pairIKS3_N12_GLOBAL__N_113LVILatticeValEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::AssertingVH<llvm::BasicBlock>, std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal>, std::_Select1st<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >*)"},
    {"_ZN4llvm8DenseMapIN12_GLOBAL__N_114LVIValueHandleESt3mapINS_11AssertingVHINS_10BasicBlockEEENS1_13LVILatticeValESt4lessIS6_ESaISt4pairIKS6_S7_EEENS_12DenseMapInfoIS2_EENSF_ISE_EEEixERKS2_", "llvm::DenseMap<(anonymous namespace)::LVIValueHandle, std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >, llvm::DenseMapInfo<(anonymous namespace)::LVIValueHandle>, llvm::DenseMapInfo<std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > > > >::operator[]((anonymous namespace)::LVIValueHandle const&)"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_10BasicBlockEEESt4pairIKS3_N12_GLOBAL__N_113LVILatticeValEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE7_M_copyEPKSt13_Rb_tree_nodeIS8_EPSG_", "std::_Rb_tree<llvm::AssertingVH<llvm::BasicBlock>, std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal>, std::_Select1st<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > const*, std::_Rb_tree_node<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_10BasicBlockEEESt4pairIKS3_N12_GLOBAL__N_113LVILatticeValEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE14_M_create_nodeERKS8_", "std::_Rb_tree<llvm::AssertingVH<llvm::BasicBlock>, std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal>, std::_Select1st<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >::_M_create_node(std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> const&)"},
    {"_ZN12_GLOBAL__N_118LazyValueInfoCache12getEdgeValueEPN4llvm5ValueEPNS1_10BasicBlockES5_RNS_13LVILatticeValE", "(anonymous namespace)::LazyValueInfoCache::getEdgeValue(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, (anonymous namespace)::LVILatticeVal&)"},
    {"_ZN12_GLOBAL__N_118LazyValueInfoCache5solveEv", "(anonymous namespace)::LazyValueInfoCache::solve()"},
    {"_ZNSt3mapIN4llvm11AssertingVHINS0_10BasicBlockEEEN12_GLOBAL__N_113LVILatticeValESt4lessIS3_ESaISt4pairIKS3_S5_EEEixERS9_", "std::map<llvm::AssertingVH<llvm::BasicBlock>, (anonymous namespace)::LVILatticeVal, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >::operator[](llvm::AssertingVH<llvm::BasicBlock> const&)"},
    {"_ZN12_GLOBAL__N_118LazyValueInfoCache13hasBlockValueEPN4llvm5ValueEPNS1_10BasicBlockE", "(anonymous namespace)::LazyValueInfoCache::hasBlockValue(llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_118LazyValueInfoCache13getBlockValueEPN4llvm5ValueEPNS1_10BasicBlockE", "(anonymous namespace)::LazyValueInfoCache::getBlockValue(llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN12_GLOBAL__N_113LVILatticeVal17markConstantRangeEN4llvm13ConstantRangeE", "(anonymous namespace)::LVILatticeVal::markConstantRange(llvm::ConstantRange)"},
    {"_ZN12_GLOBAL__N_113LVILatticeVal3getEPN4llvm8ConstantE", "(anonymous namespace)::LVILatticeVal::get(llvm::Constant*)"},
    {"_ZN12_GLOBAL__N_113LVILatticeVal15markNotConstantEPN4llvm8ConstantE", "(anonymous namespace)::LVILatticeVal::markNotConstant(llvm::Constant*)"},
    {"_ZN12_GLOBAL__N_113LVILatticeVal7mergeInERKS0_", "(anonymous namespace)::LVILatticeVal::mergeIn((anonymous namespace)::LVILatticeVal const&)"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_10BasicBlockEEESt4pairIKS3_N12_GLOBAL__N_113LVILatticeValEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::AssertingVH<llvm::BasicBlock>, std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal>, std::_Select1st<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> >, std::less<llvm::AssertingVH<llvm::BasicBlock> >, std::allocator<std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> > >::_M_insert_unique(std::pair<llvm::AssertingVH<llvm::BasicBlock> const, (anonymous namespace)::LVILatticeVal> const&)"},
    {"_ZN12_GLOBAL__N_113LVILatticeVal8getRangeEN4llvm13ConstantRangeE", "(anonymous namespace)::LVILatticeVal::getRange(llvm::ConstantRange)"},
    {"_ZN4llvm13LazyValueInfoD1Ev", "llvm::LazyValueInfo::~LazyValueInfo()"},
    {"_ZN4llvm13LazyValueInfoD0Ev", "llvm::LazyValueInfo::~LazyValueInfo()"},
    {"_ZNK4llvm13LazyValueInfo16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LazyValueInfo::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE15_M_range_insertINS0_12SuccIteratorIPNS0_14TerminatorInstES1_EEEEvN9__gnu_cxx17__normal_iteratorIPS2_S4_EET_SE_St20forward_iterator_tag", "void std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_range_insert<llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock>, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock>, std::forward_iterator_tag)"},
    {"_ZN4llvm8DenseMapISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEEcNS_12DenseMapInfoIS7_EENS8_IcEEE4findERKS7_", "llvm::DenseMap<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> >, llvm::DenseMapInfo<char> >::find(std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEcNS_12DenseMapInfoIS2_EENS3_IcEEE6insertERKSt4pairIS2_cE", "llvm::DenseMap<llvm::Value*, char, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::Value*, char> const&)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEcNS_12DenseMapInfoIS2_EENS3_IcEEE16InsertIntoBucketERKS2_RKcPSt4pairIS2_cE", "llvm::DenseMap<llvm::Value*, char, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::Value* const&, char const&, std::pair<llvm::Value*, char>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEcNS_12DenseMapInfoIS2_EENS3_IcEEE4growEj", "llvm::DenseMap<llvm::Value*, char, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZNSt5dequeISt4pairIPN4llvm10BasicBlockEPNS1_5ValueEESaIS6_EE9push_backERKS6_", "std::deque<std::pair<llvm::BasicBlock*, llvm::Value*>, std::allocator<std::pair<llvm::BasicBlock*, llvm::Value*> > >::push_back(std::pair<llvm::BasicBlock*, llvm::Value*> const&)"},
    {"_ZNSt5dequeISt4pairIPN4llvm10BasicBlockEPNS1_5ValueEESaIS6_EE17_M_reallocate_mapEmb", "std::deque<std::pair<llvm::BasicBlock*, llvm::Value*>, std::allocator<std::pair<llvm::BasicBlock*, llvm::Value*> > >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZN4llvm8DenseMapISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEEcNS_12DenseMapInfoIS7_EENS8_IcEEE6insertERKS1_IS7_cE", "llvm::DenseMap<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> >, llvm::DenseMapInfo<char> >::insert(std::pair<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char> const&)"},
    {"_ZN4llvm8DenseMapISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEEcNS_12DenseMapInfoIS7_EENS8_IcEEE16InsertIntoBucketERKS7_RKcPS1_IS7_cE", "llvm::DenseMap<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> >, llvm::DenseMapInfo<char> >::InsertIntoBucket(std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> const&, char const&, std::pair<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char>*)"},
    {"_ZN4llvm8DenseMapISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEEcNS_12DenseMapInfoIS7_EENS8_IcEEE4growEj", "llvm::DenseMap<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> >, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseSetISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEENS_12DenseMapInfoIS7_EEE5clearEv", "llvm::DenseSet<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> > >::clear()"},
    {"_ZN4llvm8DenseMapISt4pairINS_11AssertingVHINS_10BasicBlockEEEPNS_5ValueEEcNS_12DenseMapInfoIS7_EENS8_IcEEE16shrink_and_clearEv", "llvm::DenseMap<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*>, char, llvm::DenseMapInfo<std::pair<llvm::AssertingVH<llvm::BasicBlock>, llvm::Value*> >, llvm::DenseMapInfo<char> >::shrink_and_clear()"},
    {"_ZNSt11_Deque_baseISt4pairIPN4llvm10BasicBlockEPNS1_5ValueEESaIS6_EE17_M_initialize_mapEm", "std::_Deque_base<std::pair<llvm::BasicBlock*, llvm::Value*>, std::allocator<std::pair<llvm::BasicBlock*, llvm::Value*> > >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeISt4pairIPN4llvm10BasicBlockEPNS1_5ValueEESaIS6_EEC2ERKS8_", "std::deque<std::pair<llvm::BasicBlock*, llvm::Value*>, std::allocator<std::pair<llvm::BasicBlock*, llvm::Value*> > >::deque(std::deque<std::pair<llvm::BasicBlock*, llvm::Value*>, std::allocator<std::pair<llvm::BasicBlock*, llvm::Value*> > > const&)"},
    {"_ZN4llvm15callDefaultCtorINS_13LazyValueInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LazyValueInfo>()"},
    {"_ZN4llvm27isSafeToLoadUnconditionallyEPNS_5ValueEPNS_11InstructionEjPKNS_10DataLayoutE", "llvm::isSafeToLoadUnconditionally(llvm::Value*, llvm::Instruction*, unsigned int, llvm::DataLayout const*)"},
    {"_ZN4llvm24FindAvailableLoadedValueEPNS_5ValueEPNS_10BasicBlockERNS_14ilist_iteratorINS_11InstructionEEEjPNS_13AliasAnalysisE", "llvm::FindAvailableLoadedValue(llvm::Value*, llvm::BasicBlock*, llvm::ilist_iterator<llvm::Instruction>&, unsigned int, llvm::AliasAnalysis*)"},
    {"_ZN4llvm22initializeLoopInfoPassERNS_12PassRegistryE", "llvm::initializeLoopInfoPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm4Loop15isLoopInvariantEPNS_5ValueE", "llvm::Loop::isLoopInvariant(llvm::Value*) const"},
    {"_ZNK4llvm4Loop24hasLoopInvariantOperandsEPNS_11InstructionE", "llvm::Loop::hasLoopInvariantOperands(llvm::Instruction*) const"},
    {"_ZNK4llvm4Loop17makeLoopInvariantEPNS_5ValueERbPNS_11InstructionE", "llvm::Loop::makeLoopInvariant(llvm::Value*, bool&, llvm::Instruction*) const"},
    {"_ZNK4llvm4Loop17makeLoopInvariantEPNS_11InstructionERbS2_", "llvm::Loop::makeLoopInvariant(llvm::Instruction*, bool&, llvm::Instruction*) const"},
    {"_ZNK4llvm4Loop29getCanonicalInductionVariableEv", "llvm::Loop::getCanonicalInductionVariable() const"},
    {"_ZNK4llvm4Loop12getTripCountEv", "llvm::Loop::getTripCount() const"},
    {"_ZNK4llvm4Loop25getSmallConstantTripCountEv", "llvm::Loop::getSmallConstantTripCount() const"},
    {"_ZNK4llvm4Loop28getSmallConstantTripMultipleEv", "llvm::Loop::getSmallConstantTripMultiple() const"},
    {"_ZNK4llvm4Loop11isLCSSAFormERNS_13DominatorTreeE", "llvm::Loop::isLCSSAForm(llvm::DominatorTree&) const"},
    {"_ZNK4llvm4Loop18isLoopSimplifyFormEv", "llvm::Loop::isLoopSimplifyForm() const"},
    {"_ZNK4llvm4Loop17hasDedicatedExitsEv", "llvm::Loop::hasDedicatedExits() const"},
    {"_ZNK4llvm4Loop19getUniqueExitBlocksERNS_15SmallVectorImplIPNS_10BasicBlockEEE", "llvm::Loop::getUniqueExitBlocks(llvm::SmallVectorImpl<llvm::BasicBlock*>&) const"},
    {"_ZNK4llvm4Loop18getUniqueExitBlockEv", "llvm::Loop::getUniqueExitBlock() const"},
    {"_ZNK4llvm4Loop4dumpEv", "llvm::Loop::dump() const"},
    {"_ZN4llvm8LoopInfo13runOnFunctionERNS_8FunctionE", "llvm::LoopInfo::runOnFunction(llvm::Function&)"},
    {"_ZNK4llvm8LoopInfo14verifyAnalysisEv", "llvm::LoopInfo::verifyAnalysis() const"},
    {"_ZNK4llvm8LoopInfo16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LoopInfo::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZNK4llvm8LoopInfo5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::LoopInfo::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE5printERNS_11raw_ostreamEj", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::print(llvm::raw_ostream&, unsigned int) const"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE9CalculateERNS_17DominatorTreeBaseIS1_EE", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::Calculate(llvm::DominatorTreeBase<llvm::BasicBlock>&)"},
    {"_ZN4llvm8LoopInfoD1Ev", "llvm::LoopInfo::~LoopInfo()"},
    {"_ZN4llvm8LoopInfoD0Ev", "llvm::LoopInfo::~LoopInfo()"},
    {"_ZN4llvm8LoopInfo13releaseMemoryEv", "llvm::LoopInfo::releaseMemory()"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE15ConsiderForLoopEPS1_RNS_17DominatorTreeBaseIS1_EE", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::ConsiderForLoop(llvm::BasicBlock*, llvm::DominatorTreeBase<llvm::BasicBlock>&)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_4LoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE6insertERKSt4pairIS2_S4_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::Loop*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Loop*> >::insert(std::pair<llvm::BasicBlock*, llvm::Loop*> const&)"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE19MoveSiblingLoopIntoEPS2_S4_", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::MoveSiblingLoopInto(llvm::Loop*, llvm::Loop*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_4LoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Loop*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Loop*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Loop*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock* const, llvm::Loop*> >*)"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE14InsertLoopIntoEPS2_S4_", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::InsertLoopInto(llvm::Loop*, llvm::Loop*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_4LoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Loop*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Loop*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Loop*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock* const, llvm::Loop*> >, std::pair<llvm::BasicBlock* const, llvm::Loop*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm10BasicBlockESt4pairIKS2_PNS0_4LoopEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<llvm::BasicBlock*, std::pair<llvm::BasicBlock* const, llvm::Loop*>, std::_Select1st<std::pair<llvm::BasicBlock* const, llvm::Loop*> >, std::less<llvm::BasicBlock*>, std::allocator<std::pair<llvm::BasicBlock* const, llvm::Loop*> > >::_M_insert_unique(std::pair<llvm::BasicBlock* const, llvm::Loop*> const&)"},
    {"_ZNSt6vectorIPN4llvm10BasicBlockESaIS2_EE15_M_range_insertINS0_12PredIteratorIS1_NS0_18value_use_iteratorINS0_4UserEEEEEEEvN9__gnu_cxx17__normal_iteratorIPS2_S4_EET_SF_St20forward_iterator_tag", "void std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> >::_M_range_insert<llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >(__gnu_cxx::__normal_iterator<llvm::BasicBlock**, std::vector<llvm::BasicBlock*, std::allocator<llvm::BasicBlock*> > >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >, std::forward_iterator_tag)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm4LoopESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::Loop**, std::vector<llvm::Loop*, std::allocator<llvm::Loop*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::Loop**, std::vector<llvm::Loop*, std::allocator<llvm::Loop*> > >, llvm::Loop*>(__gnu_cxx::__normal_iterator<llvm::Loop**, std::vector<llvm::Loop*, std::allocator<llvm::Loop*> > >, __gnu_cxx::__normal_iterator<llvm::Loop**, std::vector<llvm::Loop*, std::allocator<llvm::Loop*> > >, llvm::Loop* const&, std::random_access_iterator_tag)"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE13isLoopExitingEPKS1_", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::isLoopExiting(llvm::BasicBlock const*) const"},
    {"_ZN4llvm12LoopInfoBaseINS_10BasicBlockENS_4LoopEE13releaseMemoryEv", "llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop>::releaseMemory()"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPNS_4LoopENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::Loop*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Loop*> >::shrink_and_clear()"},
    {"_ZN4llvm15callDefaultCtorINS_8LoopInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::LoopInfo>()"},
    {"_ZN4llvm8LoopInfoC2Ev", "llvm::LoopInfo::LoopInfo()"},
    {"_ZN4llvm13LPPassManagerC1Ei", "llvm::LPPassManager::LPPassManager(int)"},
    {"_ZN4llvm13LPPassManagerC2Ei", "llvm::LPPassManager::LPPassManager(int)"},
    {"_ZN4llvm13LPPassManager19deleteLoopFromQueueEPNS_4LoopE", "llvm::LPPassManager::deleteLoopFromQueue(llvm::Loop*)"},
    {"_ZN4llvm13LPPassManager10insertLoopEPNS_4LoopES2_", "llvm::LPPassManager::insertLoop(llvm::Loop*, llvm::Loop*)"},
    {"_ZN4llvm13LPPassManager19insertLoopIntoQueueEPNS_4LoopE", "llvm::LPPassManager::insertLoopIntoQueue(llvm::Loop*)"},
    {"_ZN4llvm13LPPassManager8redoLoopEPNS_4LoopE", "llvm::LPPassManager::redoLoop(llvm::Loop*)"},
    {"_ZN4llvm13LPPassManager29cloneBasicBlockSimpleAnalysisEPNS_10BasicBlockES2_PNS_4LoopE", "llvm::LPPassManager::cloneBasicBlockSimpleAnalysis(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Loop*)"},
    {"_ZN4llvm13LPPassManager25deleteSimpleAnalysisValueEPNS_5ValueEPNS_4LoopE", "llvm::LPPassManager::deleteSimpleAnalysisValue(llvm::Value*, llvm::Loop*)"},
    {"_ZNK4llvm13LPPassManager16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::LPPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13LPPassManager13runOnFunctionERNS_8FunctionE", "llvm::LPPassManager::runOnFunction(llvm::Function&)"},
    {"_Z16addLoopIntoQueuePN4llvm4LoopERSt5dequeIS1_SaIS1_EE", "addLoopIntoQueue(llvm::Loop*, std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >&)"},
    {"_ZN4llvm13LPPassManager17dumpPassStructureEj", "llvm::LPPassManager::dumpPassStructure(unsigned int)"},
    {"_ZNK4llvm8LoopPass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::LoopPass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZN4llvm8LoopPass18preparePassManagerERNS_7PMStackE", "llvm::LoopPass::preparePassManager(llvm::PMStack&)"},
    {"_ZN4llvm8LoopPass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::LoopPass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZN12_GLOBAL__N_113PrintLoopPassD1Ev", "(anonymous namespace)::PrintLoopPass::~PrintLoopPass()"},
    {"_ZN12_GLOBAL__N_113PrintLoopPassD0Ev", "(anonymous namespace)::PrintLoopPass::~PrintLoopPass()"},
    {"_ZNK12_GLOBAL__N_113PrintLoopPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PrintLoopPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_113PrintLoopPass9runOnLoopEPN4llvm4LoopERNS1_13LPPassManagerE", "(anonymous namespace)::PrintLoopPass::runOnLoop(llvm::Loop*, llvm::LPPassManager&)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE5eraseESt15_Deque_iteratorIS2_RS2_PS2_E", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::erase(std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE9push_backERKS2_", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::push_back(llvm::Loop* const&)"},
    {"_ZN4llvm13LPPassManagerD1Ev", "llvm::LPPassManager::~LPPassManager()"},
    {"_ZN4llvm13LPPassManagerD0Ev", "llvm::LPPassManager::~LPPassManager()"},
    {"_ZNK4llvm13LPPassManager11getPassNameEv", "llvm::LPPassManager::getPassName() const"},
    {"_ZN4llvm13LPPassManager18getAsPMDataManagerEv", "llvm::LPPassManager::getAsPMDataManager()"},
    {"_ZN4llvm13LPPassManager9getAsPassEv", "llvm::LPPassManager::getAsPass()"},
    {"_ZNK4llvm13LPPassManager18getPassManagerTypeEv", "llvm::LPPassManager::getPassManagerType() const"},
    {"_ZThn32_N4llvm13LPPassManagerD1Ev", "non-virtual thunk to llvm::LPPassManager::~LPPassManager()"},
    {"_ZThn32_N4llvm13LPPassManagerD0Ev", "non-virtual thunk to llvm::LPPassManager::~LPPassManager()"},
    {"_ZThn32_N4llvm13LPPassManager9getAsPassEv", "non-virtual thunk to llvm::LPPassManager::getAsPass()"},
    {"_ZThn32_NK4llvm13LPPassManager18getPassManagerTypeEv", "non-virtual thunk to llvm::LPPassManager::getPassManagerType() const"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE17_M_reallocate_mapEmb", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE14_M_fill_insertESt15_Deque_iteratorIS2_RS2_PS2_EmRKS2_", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_fill_insert(std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>, unsigned long, llvm::Loop* const&)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE13_M_insert_auxESt15_Deque_iteratorIS2_RS2_PS2_EmRKS2_", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_insert_aux(std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>, unsigned long, llvm::Loop* const&)"},
    {"_ZSt4fillIPN4llvm4LoopEEvRKSt15_Deque_iteratorIT_RS4_PS4_ES9_RKS4_", "void std::fill<llvm::Loop*>(std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**> const&, std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**> const&, llvm::Loop* const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bISt15_Deque_iteratorIPN4llvm4LoopERS6_PS6_ES9_EET0_T_SB_SA_", "std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**> std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>, std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**> >(std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>, std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>, std::_Deque_iterator<llvm::Loop*, llvm::Loop*&, llvm::Loop**>)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE23_M_new_elements_at_backEm", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_new_elements_at_back(unsigned long)"},
    {"_ZNSt5dequeIPN4llvm4LoopESaIS2_EE24_M_new_elements_at_frontEm", "std::deque<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_new_elements_at_front(unsigned long)"},
    {"_ZNSt6vectorIPN4llvm4LoopESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Loop**, std::vector<llvm::Loop*, std::allocator<llvm::Loop*> > >, llvm::Loop* const&)"},
    {"_ZNSt11_Deque_baseIPN4llvm4LoopESaIS2_EE17_M_initialize_mapEm", "std::_Deque_base<llvm::Loop*, std::allocator<llvm::Loop*> >::_M_initialize_map(unsigned long)"},
    {"_ZN4llvm8LoopBaseINS_10BasicBlockENS_4LoopEED2Ev", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::~LoopBase()"},
    {"_ZN4llvm8isMallocEPKNS_5ValueE", "llvm::isMalloc(llvm::Value const*)"},
    {"_ZN4llvm17extractMallocCallEPKNS_5ValueE", "llvm::extractMallocCall(llvm::Value const*)"},
    {"_ZN4llvm28extractMallocCallFromBitCastEPKNS_5ValueE", "llvm::extractMallocCallFromBitCast(llvm::Value const*)"},
    {"_Z12isMallocCallPKN4llvm8CallInstE", "isMallocCall(llvm::CallInst const*)"},
    {"_ZN4llvm17extractMallocCallEPNS_5ValueE", "llvm::extractMallocCall(llvm::Value*)"},
    {"_ZN4llvm28extractMallocCallFromBitCastEPNS_5ValueE", "llvm::extractMallocCallFromBitCast(llvm::Value*)"},
    {"_ZN4llvm13isArrayMallocEPKNS_5ValueEPKNS_10DataLayoutE", "llvm::isArrayMalloc(llvm::Value const*, llvm::DataLayout const*)"},
    {"_Z16computeArraySizePKN4llvm8CallInstEPKNS_10DataLayoutEb", "computeArraySize(llvm::CallInst const*, llvm::DataLayout const*, bool)"},
    {"_ZN4llvm13getMallocTypeEPKNS_8CallInstE", "llvm::getMallocType(llvm::CallInst const*)"},
    {"_ZN4llvm22getMallocAllocatedTypeEPKNS_8CallInstE", "llvm::getMallocAllocatedType(llvm::CallInst const*)"},
    {"_ZN4llvm18getMallocArraySizeEPNS_8CallInstEPKNS_10DataLayoutEb", "llvm::getMallocArraySize(llvm::CallInst*, llvm::DataLayout const*, bool)"},
    {"_ZN4llvm10isFreeCallEPKNS_5ValueE", "llvm::isFreeCall(llvm::Value const*)"},
    {"_ZN4llvm38initializeMemoryDependenceAnalysisPassERNS_12PassRegistryE", "llvm::initializeMemoryDependenceAnalysisPass(llvm::PassRegistry&)"},
    {"_ZN4llvm24MemoryDependenceAnalysisC1Ev", "llvm::MemoryDependenceAnalysis::MemoryDependenceAnalysis()"},
    {"_ZN4llvm24MemoryDependenceAnalysisC2Ev", "llvm::MemoryDependenceAnalysis::MemoryDependenceAnalysis()"},
    {"_ZN4llvm24MemoryDependenceAnalysisD0Ev", "llvm::MemoryDependenceAnalysis::~MemoryDependenceAnalysis()"},
    {"_ZN4llvm24MemoryDependenceAnalysisD1Ev", "llvm::MemoryDependenceAnalysis::~MemoryDependenceAnalysis()"},
    {"_ZN4llvm24MemoryDependenceAnalysisD2Ev", "llvm::MemoryDependenceAnalysis::~MemoryDependenceAnalysis()"},
    {"_ZN4llvm24MemoryDependenceAnalysis13releaseMemoryEv", "llvm::MemoryDependenceAnalysis::releaseMemory()"},
    {"_ZNK4llvm24MemoryDependenceAnalysis16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MemoryDependenceAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm24MemoryDependenceAnalysis13runOnFunctionERNS_8FunctionE", "llvm::MemoryDependenceAnalysis::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm24MemoryDependenceAnalysis25getCallSiteDependencyFromENS_8CallSiteEbNS_14ilist_iteratorINS_11InstructionEEEPNS_10BasicBlockE", "llvm::MemoryDependenceAnalysis::getCallSiteDependencyFrom(llvm::CallSite, bool, llvm::ilist_iterator<llvm::Instruction>, llvm::BasicBlock*)"},
    {"_Z11GetLocationPKN4llvm11InstructionERNS_13AliasAnalysis8LocationEPS3_", "GetLocation(llvm::Instruction const*, llvm::AliasAnalysis::Location&, llvm::AliasAnalysis*)"},
    {"_ZN4llvm24MemoryDependenceAnalysis24getPointerDependencyFromERKNS_13AliasAnalysis8LocationEbNS_14ilist_iteratorINS_11InstructionEEEPNS_10BasicBlockE", "llvm::MemoryDependenceAnalysis::getPointerDependencyFrom(llvm::AliasAnalysis::Location const&, bool, llvm::ilist_iterator<llvm::Instruction>, llvm::BasicBlock*)"},
    {"_ZN4llvm24MemoryDependenceAnalysis13getDependencyEPNS_11InstructionE", "llvm::MemoryDependenceAnalysis::getDependency(llvm::Instruction*)"},
    {"_Z20RemoveFromReverseMapIPN4llvm11InstructionEEvRNS0_8DenseMapIS2_NS0_11SmallPtrSetIT_Lj4EEENS0_12DenseMapInfoIS2_EENS7_IS6_EEEES2_S5_", "void RemoveFromReverseMap<llvm::Instruction*>(llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >&, llvm::Instruction*, llvm::Instruction*)"},
    {"_ZN4llvm24MemoryDependenceAnalysis25getNonLocalCallDependencyENS_8CallSiteE", "llvm::MemoryDependenceAnalysis::getNonLocalCallDependency(llvm::CallSite)"},
    {"_ZN4llvm24MemoryDependenceAnalysis28getNonLocalPointerDependencyERKNS_13AliasAnalysis8LocationEbPNS_10BasicBlockERNS_15SmallVectorImplINS_17NonLocalDepResultEEE", "llvm::MemoryDependenceAnalysis::getNonLocalPointerDependency(llvm::AliasAnalysis::Location const&, bool, llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::NonLocalDepResult>&)"},
    {"_ZN4llvm24MemoryDependenceAnalysis27getNonLocalPointerDepFromBBERKNS_12PHITransAddrERKNS_13AliasAnalysis8LocationEbPNS_10BasicBlockERNS_15SmallVectorImplINS_17NonLocalDepResultEEERNS_8DenseMapIS9_PNS_5ValueENS_12DenseMapInfoIS9_EENSH_ISG_EEEEb", "llvm::MemoryDependenceAnalysis::getNonLocalPointerDepFromBB(llvm::PHITransAddr const&, llvm::AliasAnalysis::Location const&, bool, llvm::BasicBlock*, llvm::SmallVectorImpl<llvm::NonLocalDepResult>&, llvm::DenseMap<llvm::BasicBlock*, llvm::Value*, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::Value*> >&, bool)"},
    {"_ZN4llvm24MemoryDependenceAnalysis23GetNonLocalInfoForBlockERKNS_13AliasAnalysis8LocationEbPNS_10BasicBlockEPSt6vectorINS_16NonLocalDepEntryESaIS8_EEj", "llvm::MemoryDependenceAnalysis::GetNonLocalInfoForBlock(llvm::AliasAnalysis::Location const&, bool, llvm::BasicBlock*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >*, unsigned int)"},
    {"_Z20RemoveFromReverseMapIN4llvm14PointerIntPairIPKNS0_5ValueELj1EbNS0_21PointerLikeTypeTraitsIS4_EEEEEvRNS0_8DenseMapIPNS0_11InstructionENS0_11SmallPtrSetIT_Lj4EEENS0_12DenseMapInfoISA_EENSE_ISD_EEEESA_SC_", "void RemoveFromReverseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >(llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >&, llvm::Instruction*, llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >)"},
    {"_Z24SortNonLocalDepInfoCacheRSt6vectorIN4llvm16NonLocalDepEntryESaIS1_EEj", "SortNonLocalDepInfoCache(std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >&, unsigned int)"},
    {"_ZN4llvm24MemoryDependenceAnalysis39RemoveCachedNonLocalPointerDependenciesENS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEE", "llvm::MemoryDependenceAnalysis::RemoveCachedNonLocalPointerDependencies(llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >)"},
    {"_ZN4llvm24MemoryDependenceAnalysis27invalidateCachedPointerInfoEPNS_5ValueE", "llvm::MemoryDependenceAnalysis::invalidateCachedPointerInfo(llvm::Value*)"},
    {"_ZN4llvm24MemoryDependenceAnalysis28invalidateCachedPredecessorsEv", "llvm::MemoryDependenceAnalysis::invalidateCachedPredecessors()"},
    {"_ZN4llvm24MemoryDependenceAnalysis17removeInstructionEPNS_11InstructionE", "llvm::MemoryDependenceAnalysis::removeInstruction(llvm::Instruction*)"},
    {"_ZNK4llvm24MemoryDependenceAnalysis13verifyRemovedEPNS_11InstructionE", "llvm::MemoryDependenceAnalysis::verifyRemoved(llvm::Instruction*) const"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt4pairISt6vectorINS_16NonLocalDepEntryESaIS5_EEbENS_12DenseMapInfoIS2_EENS9_IS8_EEE5clearEv", "llvm::DenseMap<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> > >::clear()"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE5clearEv", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::clear()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetIS2_Lj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE5clearEv", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >::clear()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS7_EEEELj4EEENS_12DenseMapInfoIS2_EENSC_ISB_EEE5clearEv", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >::clear()"},
    {"_ZN4llvm17PredIteratorCache5clearEv", "llvm::PredIteratorCache::clear()"},
    {"_ZN4llvm17PredIteratorCache8GetPredsEPNS_10BasicBlockE", "llvm::PredIteratorCache::GetPreds(llvm::BasicBlock*)"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE6insertERKSt4pairIS7_S9_E", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::insert(std::pair<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> const&)"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE16FindAndConstructERKS7_", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::FindAndConstruct(llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > const&)"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE16InsertIntoBucketERKS7_RKS9_PSt4pairIS7_S9_E", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::InsertIntoBucket(llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > const&, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo const&, std::pair<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo>*)"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE4growEj", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS7_EEEELj4EEENS_12DenseMapInfoIS2_EENSC_ISB_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >::FindAndConstruct(llvm::Instruction* const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS7_EEEELj4EEENS_12DenseMapInfoIS2_EENSC_ISB_EEE16InsertIntoBucketERKS2_RKSB_PSt4pairIS2_SB_E", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >::InsertIntoBucket(llvm::Instruction* const&, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> const&, std::pair<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS7_EEEELj4EEENS_12DenseMapInfoIS2_EENSC_ISB_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_17NonLocalDepResultELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::NonLocalDepResult, false>::grow(unsigned long)"},
    {"_ZNSt6vectorIN4llvm16NonLocalDepEntryESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, llvm::NonLocalDepEntry const&)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEElEvT_S9_T0_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, long>(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, long)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEEEvT_S9_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > > >(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEEEvT_S9_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > > >(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEEEvT_S9_S9_", "void std::__heap_select<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > > >(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >)"},
    {"_ZSt9sort_heapIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEEEvT_S9_", "void std::sort_heap<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > > >(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >)"},
    {"_ZSt9make_heapIN9__gnu_cxx17__normal_iteratorIPN4llvm16NonLocalDepEntryESt6vectorIS3_SaIS3_EEEEEvT_S9_", "void std::make_heap<__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > > >(__gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >, __gnu_cxx::__normal_iterator<llvm::NonLocalDepEntry*, std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > >)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt4pairISt6vectorINS_16NonLocalDepEntryESaIS5_EEbENS_12DenseMapInfoIS2_EENS9_IS8_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> > >::FindAndConstruct(llvm::Instruction* const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt4pairISt6vectorINS_16NonLocalDepEntryESaIS5_EEbENS_12DenseMapInfoIS2_EENS9_IS8_EEE16InsertIntoBucketERKS2_RKS8_PS3_IS2_S8_E", "llvm::DenseMap<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> > >::InsertIntoBucket(llvm::Instruction* const&, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> const&, std::pair<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> >*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt4pairISt6vectorINS_16NonLocalDepEntryESaIS5_EEbENS_12DenseMapInfoIS2_EENS9_IS8_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> > >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm16NonLocalDepEntryESaIS1_EEC2ERKS3_", "std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >::vector(std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> > const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetIS2_Lj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >::FindAndConstruct(llvm::Instruction* const&)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetIS2_Lj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >::InsertIntoBucket(llvm::Instruction* const&, llvm::SmallPtrSet<llvm::Instruction*, 4u> const&, std::pair<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetIS2_Lj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_12MemDepResultENS_12DenseMapInfoIS2_EENS4_IS3_EEE16InsertIntoBucketERKS2_RKS3_PSt4pairIS2_S3_E", "llvm::DenseMap<llvm::Instruction*, llvm::MemDepResult, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::MemDepResult> >::InsertIntoBucket(llvm::Instruction* const&, llvm::MemDepResult const&, std::pair<llvm::Instruction*, llvm::MemDepResult>*)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_12MemDepResultENS_12DenseMapInfoIS2_EENS4_IS3_EEE4growEj", "llvm::DenseMap<llvm::Instruction*, llvm::MemDepResult, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::MemDepResult> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS7_EEEELj4EEENS_12DenseMapInfoIS2_EENSC_ISB_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, 4u> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_11SmallPtrSetIS2_Lj4EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, llvm::SmallPtrSet<llvm::Instruction*, 4u>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Instruction*, 4u> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapINS_14PointerIntPairIPKNS_5ValueELj1EbNS_21PointerLikeTypeTraitsIS4_EEEENS_24MemoryDependenceAnalysis19NonLocalPointerInfoENS_12DenseMapInfoIS7_EENSA_IS9_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> >, llvm::MemoryDependenceAnalysis::NonLocalPointerInfo, llvm::DenseMapInfo<llvm::PointerIntPair<llvm::Value const*, 1u, bool, llvm::PointerLikeTypeTraits<llvm::Value const*> > >, llvm::DenseMapInfo<llvm::MemoryDependenceAnalysis::NonLocalPointerInfo> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionESt4pairISt6vectorINS_16NonLocalDepEntryESaIS5_EEbENS_12DenseMapInfoIS2_EENS9_IS8_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool>, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<std::pair<std::vector<llvm::NonLocalDepEntry, std::allocator<llvm::NonLocalDepEntry> >, bool> > >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_11InstructionENS_12MemDepResultENS_12DenseMapInfoIS2_EENS4_IS3_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Instruction*, llvm::MemDepResult, llvm::DenseMapInfo<llvm::Instruction*>, llvm::DenseMapInfo<llvm::MemDepResult> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::BasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::BasicBlock* const&, unsigned int const&, std::pair<llvm::BasicBlock*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPS2_NS_12DenseMapInfoIS2_EENS4_IS3_EEE16InsertIntoBucketERKS2_RKS3_PSt4pairIS2_S3_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock**, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock**> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::BasicBlock** const&, std::pair<llvm::BasicBlock*, llvm::BasicBlock**>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPS2_NS_12DenseMapInfoIS2_EENS4_IS3_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock**, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock**> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEjNS_12DenseMapInfoIS2_EENS3_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, unsigned int, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockEPS2_NS_12DenseMapInfoIS2_EENS4_IS3_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::BasicBlock**, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::BasicBlock**> >::shrink_and_clear()"},
    {"_ZN4llvm15callDefaultCtorINS_24MemoryDependenceAnalysisEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::MemoryDependenceAnalysis>()"},
    {"_ZN4llvm18initializeNoAAPassERNS_12PassRegistryE", "llvm::initializeNoAAPass(llvm::PassRegistry&)"},
    {"_ZN4llvm14createNoAAPassEv", "llvm::createNoAAPass()"},
    {"_ZN12_GLOBAL__N_14NoAAD1Ev", "(anonymous namespace)::NoAA::~NoAA()"},
    {"_ZN12_GLOBAL__N_14NoAAD0Ev", "(anonymous namespace)::NoAA::~NoAA()"},
    {"_ZNK12_GLOBAL__N_14NoAA16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::NoAA::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_14NoAA26getAdjustedAnalysisPointerEPKv", "(anonymous namespace)::NoAA::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN12_GLOBAL__N_14NoAA14initializePassEv", "(anonymous namespace)::NoAA::initializePass()"},
    {"_ZN12_GLOBAL__N_14NoAA5aliasERKN4llvm13AliasAnalysis8LocationES5_", "(anonymous namespace)::NoAA::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_14NoAA17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "(anonymous namespace)::NoAA::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_14NoAA17getModRefBehaviorEPKN4llvm8FunctionE", "(anonymous namespace)::NoAA::getModRefBehavior(llvm::Function const*)"},
    {"_ZN12_GLOBAL__N_14NoAA22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "(anonymous namespace)::NoAA::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZN12_GLOBAL__N_14NoAA13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "(anonymous namespace)::NoAA::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_14NoAA13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "(anonymous namespace)::NoAA::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_14NoAA11deleteValueEPN4llvm5ValueE", "(anonymous namespace)::NoAA::deleteValue(llvm::Value*)"},
    {"_ZN12_GLOBAL__N_14NoAA9copyValueEPN4llvm5ValueES3_", "(anonymous namespace)::NoAA::copyValue(llvm::Value*, llvm::Value*)"},
    {"_ZN12_GLOBAL__N_14NoAA14addEscapingUseERN4llvm3UseE", "(anonymous namespace)::NoAA::addEscapingUse(llvm::Use&)"},
    {"_ZThn32_NK12_GLOBAL__N_14NoAA16getAnalysisUsageERN4llvm13AnalysisUsageE", "non-virtual thunk to (anonymous namespace)::NoAA::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZThn32_N12_GLOBAL__N_14NoAAD1Ev", "non-virtual thunk to (anonymous namespace)::NoAA::~NoAA()"},
    {"_ZThn32_N12_GLOBAL__N_14NoAAD0Ev", "non-virtual thunk to (anonymous namespace)::NoAA::~NoAA()"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA5aliasERKN4llvm13AliasAnalysis8LocationES5_", "non-virtual thunk to (anonymous namespace)::NoAA::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "non-virtual thunk to (anonymous namespace)::NoAA::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "non-virtual thunk to (anonymous namespace)::NoAA::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA17getModRefBehaviorEPKN4llvm8FunctionE", "non-virtual thunk to (anonymous namespace)::NoAA::getModRefBehavior(llvm::Function const*)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "non-virtual thunk to (anonymous namespace)::NoAA::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "non-virtual thunk to (anonymous namespace)::NoAA::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA11deleteValueEPN4llvm5ValueE", "non-virtual thunk to (anonymous namespace)::NoAA::deleteValue(llvm::Value*)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA9copyValueEPN4llvm5ValueES3_", "non-virtual thunk to (anonymous namespace)::NoAA::copyValue(llvm::Value*, llvm::Value*)"},
    {"_ZThn32_N12_GLOBAL__N_14NoAA14addEscapingUseERN4llvm3UseE", "non-virtual thunk to (anonymous namespace)::NoAA::addEscapingUse(llvm::Use&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_14NoAAEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::NoAA>()"},
    {"_ZNK4llvm12PHITransAddr4dumpEv", "llvm::PHITransAddr::dump() const"},
    {"_ZNK4llvm12PHITransAddr6VerifyEv", "llvm::PHITransAddr::Verify() const"},
    {"_Z13VerifySubExprPN4llvm5ValueERNS_15SmallVectorImplIPNS_11InstructionEEE", "VerifySubExpr(llvm::Value*, llvm::SmallVectorImpl<llvm::Instruction*>&)"},
    {"_ZNK4llvm12PHITransAddr28IsPotentiallyPHITranslatableEv", "llvm::PHITransAddr::IsPotentiallyPHITranslatable() const"},
    {"_ZN4llvm12PHITransAddr19PHITranslateSubExprEPNS_5ValueEPNS_10BasicBlockES4_PKNS_13DominatorTreeE", "llvm::PHITransAddr::PHITranslateSubExpr(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::DominatorTree const*)"},
    {"_Z16RemoveInstInputsPN4llvm5ValueERNS_15SmallVectorImplIPNS_11InstructionEEE", "RemoveInstInputs(llvm::Value*, llvm::SmallVectorImpl<llvm::Instruction*>&)"},
    {"_ZN4llvm12PHITransAddr17PHITranslateValueEPNS_10BasicBlockES2_PKNS_13DominatorTreeE", "llvm::PHITransAddr::PHITranslateValue(llvm::BasicBlock*, llvm::BasicBlock*, llvm::DominatorTree const*)"},
    {"_ZN4llvm12PHITransAddr25PHITranslateWithInsertionEPNS_10BasicBlockES2_RKNS_13DominatorTreeERNS_15SmallVectorImplIPNS_11InstructionEEE", "llvm::PHITransAddr::PHITranslateWithInsertion(llvm::BasicBlock*, llvm::BasicBlock*, llvm::DominatorTree const&, llvm::SmallVectorImpl<llvm::Instruction*>&)"},
    {"_ZN4llvm12PHITransAddr26InsertPHITranslatedSubExprEPNS_5ValueEPNS_10BasicBlockES4_RKNS_13DominatorTreeERNS_15SmallVectorImplIPNS_11InstructionEEE", "llvm::PHITransAddr::InsertPHITranslatedSubExpr(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::DominatorTree const&, llvm::SmallVectorImpl<llvm::Instruction*>&)"},
    {"_ZN4llvm34initializeProfileInfoAnalysisGroupERNS_12PassRegistryE", "llvm::initializeProfileInfoAnalysisGroup(llvm::PassRegistry&)"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEEC1Ev", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEEC2Ev", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEED1Ev", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::~ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEED2Ev", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::~ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEEC1Ev", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEEC2Ev", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEED1Ev", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::~ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEED2Ev", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::~ProfileInfoT()"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE17getExecutionCountEPKS2_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::getExecutionCount(llvm::BasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEE17getExecutionCountEPKS2_", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::getExecutionCount(llvm::MachineBasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE17getExecutionCountEPKS1_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::getExecutionCount(llvm::Function const*)"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEE17getExecutionCountEPKS1_", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::getExecutionCount(llvm::MachineFunction const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE17setExecutionCountEPKS2_d", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::setExecutionCount(llvm::BasicBlock const*, double)"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEE17setExecutionCountEPKS2_d", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::setExecutionCount(llvm::MachineBasicBlock const*, double)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE13addEdgeWeightESt4pairIPKS2_S6_Ed", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::addEdgeWeight(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE17addExecutionCountEPKS2_d", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::addExecutionCount(llvm::BasicBlock const*, double)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE11removeBlockEPKS2_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::removeBlock(llvm::BasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE10removeEdgeESt4pairIPKS2_S6_E", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::removeEdge(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE11replaceEdgeERKSt4pairIPKS2_S6_ES9_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::replaceEdge(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE7GetPathEPKS2_S5_RSt3mapIS5_S5_St4lessIS5_ESaISt4pairIKS5_S5_EEEj", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::GetPath(llvm::BasicBlock const*, llvm::BasicBlock const*, std::map<llvm::BasicBlock const*, llvm::BasicBlock const*, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> > >&, unsigned int)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE10divertFlowERKSt4pairIPKS2_S6_ES9_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::divertFlow(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const&)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE14replaceAllUsesEPKS2_S5_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::replaceAllUses(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE9splitEdgeEPKS2_S5_S5_b", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::splitEdge(llvm::BasicBlock const*, llvm::BasicBlock const*, llvm::BasicBlock const*, bool)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE10splitBlockEPKS2_S5_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::splitBlock(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE10splitBlockEPKS2_S5_PKPS2_j", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::splitBlock(llvm::BasicBlock const*, llvm::BasicBlock const*, llvm::BasicBlock* const*, unsigned int)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE8transferEPKS1_S5_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::transfer(llvm::Function const*, llvm::Function const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE20CalculateMissingEdgeEPKS2_RSt4pairIS5_S5_Eb", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::CalculateMissingEdge(llvm::BasicBlock const*, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>&, bool)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE20EstimateMissingEdgesEPKS2_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::EstimateMissingEdges(llvm::BasicBlock const*)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE6repairEPKS1_", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::repair(llvm::Function const*)"},
    {"_ZN4llvmlsERNS_11raw_ostreamEPKNS_8FunctionE", "llvm::operator<<(llvm::raw_ostream&, llvm::Function const*)"},
    {"_ZN4llvmlsERNS_11raw_ostreamEPKNS_15MachineFunctionE", "llvm::operator<<(llvm::raw_ostream&, llvm::MachineFunction const*)"},
    {"_ZN4llvmlsERNS_11raw_ostreamEPKNS_10BasicBlockE", "llvm::operator<<(llvm::raw_ostream&, llvm::BasicBlock const*)"},
    {"_ZN4llvmlsERNS_11raw_ostreamEPKNS_17MachineBasicBlockE", "llvm::operator<<(llvm::raw_ostream&, llvm::MachineBasicBlock const*)"},
    {"_ZN4llvmlsERNS_11raw_ostreamESt4pairIPKNS_10BasicBlockES5_E", "llvm::operator<<(llvm::raw_ostream&, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>)"},
    {"_ZN4llvmlsERNS_11raw_ostreamESt4pairIPKNS_17MachineBasicBlockES5_E", "llvm::operator<<(llvm::raw_ostream&, std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>)"},
    {"_ZN4llvm27initializeNoProfileInfoPassERNS_12PassRegistryE", "llvm::initializeNoProfileInfoPass(llvm::PassRegistry&)"},
    {"_ZN4llvm23createNoProfileInfoPassEv", "llvm::createNoProfileInfoPass()"},
    {"_ZN12_GLOBAL__N_113NoProfileInfoD1Ev", "(anonymous namespace)::NoProfileInfo::~NoProfileInfo()"},
    {"_ZN12_GLOBAL__N_113NoProfileInfoD0Ev", "(anonymous namespace)::NoProfileInfo::~NoProfileInfo()"},
    {"_ZNK12_GLOBAL__N_113NoProfileInfo11getPassNameEv", "(anonymous namespace)::NoProfileInfo::getPassName() const"},
    {"_ZN12_GLOBAL__N_113NoProfileInfo26getAdjustedAnalysisPointerEPKv", "(anonymous namespace)::NoProfileInfo::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_113NoProfileInfoEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::NoProfileInfo>()"},
    {"_ZNK4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE13getEdgeWeightESt4pairIPKS2_S6_E", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::getEdgeWeight(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>) const"},
    {"_ZNSt3mapIPKN4llvm8FunctionES_IPKNS0_10BasicBlockEdSt4lessIS6_ESaISt4pairIKS6_dEEES7_IS3_ESaIS9_IKS3_SD_EEEixERSF_", "std::map<llvm::Function const*, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::operator[](llvm::Function const* const&)"},
    {"_ZNSt3mapIPKN4llvm15MachineFunctionES_IPKNS0_17MachineBasicBlockEdSt4lessIS6_ESaISt4pairIKS6_dEEES7_IS3_ESaIS9_IKS3_SD_EEEixERSF_", "std::map<llvm::MachineFunction const*, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > > >::operator[](llvm::MachineFunction const* const&)"},
    {"_ZNSt3mapIPKN4llvm8FunctionES_ISt4pairIPKNS0_10BasicBlockES7_EdSt4lessIS8_ESaIS4_IKS8_dEEES9_IS3_ESaIS4_IKS3_SE_EEEixERSG_", "std::map<llvm::Function const*, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::operator[](llvm::Function const* const&)"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE13setEdgeWeightESt4pairIPKS2_S6_Ed", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::setEdgeWeight(std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE7_M_copyEPKSt13_Rb_tree_nodeIS7_EPSF_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >::_M_copy(std::_Rb_tree_node<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > const*, std::_Rb_tree_node<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE8_M_eraseEPSt13_Rb_tree_nodeIS3_E", "std::_Rb_tree<llvm::BasicBlock const*, llvm::BasicBlock const*, std::_Identity<llvm::BasicBlock const*>, std::less<llvm::BasicBlock const*>, std::allocator<llvm::BasicBlock const*> >::_M_erase(std::_Rb_tree_node<llvm::BasicBlock const*>*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES5_St9_IdentityIS5_ESt4lessIS5_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::_Identity<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE5eraseESt17_Rb_tree_iteratorIS3_ESB_", "std::_Rb_tree<llvm::BasicBlock const*, llvm::BasicBlock const*, std::_Identity<llvm::BasicBlock const*>, std::less<llvm::BasicBlock const*>, std::allocator<llvm::BasicBlock const*> >::erase(std::_Rb_tree_iterator<llvm::BasicBlock const*>, std::_Rb_tree_iterator<llvm::BasicBlock const*>)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE5eraseESt17_Rb_tree_iteratorIS6_ESE_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, double>, std::_Select1st<std::pair<llvm::Function const* const, double> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, double> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, double> >, std::_Rb_tree_iterator<std::pair<llvm::Function const* const, double> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, double>, std::_Select1st<std::pair<llvm::Function const* const, double> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Function const* const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIPKNS0_10BasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE5eraseESt17_Rb_tree_iteratorISG_ESN_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, double>, std::_Select1st<std::pair<llvm::BasicBlock const* const, double> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIPKNS0_10BasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIS4_IPKNS0_10BasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE5eraseESt17_Rb_tree_iteratorISH_ESO_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIS4_IPKNS0_10BasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE8_M_eraseEPSt13_Rb_tree_nodeISH_E", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> >, std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> > >::_M_insert_unique(std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> const&)"},
    {"_ZNSt5dequeIPKN4llvm10BasicBlockESaIS3_EE9push_backERKS3_", "std::deque<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> >::push_back(llvm::BasicBlock const* const&)"},
    {"_ZNSt5dequeIPKN4llvm10BasicBlockESaIS3_EE17_M_reallocate_mapEmb", "std::deque<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> >::_M_reallocate_map(unsigned long, bool)"},
    {"_ZNSt11_Deque_baseIPKN4llvm10BasicBlockESaIS3_EE17_M_initialize_mapEm", "std::_Deque_base<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> >::_M_initialize_map(unsigned long)"},
    {"_ZNSt5dequeIPKN4llvm10BasicBlockESaIS3_EEC2ERKS5_", "std::deque<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> >::deque(std::deque<llvm::BasicBlock const*, std::allocator<llvm::BasicBlock const*> > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >::erase(std::_Rb_tree_iterator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::_Rb_tree_iterator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE5eraseESt17_Rb_tree_iteratorIS6_ESE_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, double>, std::_Select1st<std::pair<llvm::BasicBlock const* const, double> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const* const, double> >, std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const* const, double> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10BasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> >, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > >::_M_insert_unique(std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIS4_IPKNS0_10BasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISH_ERKSH_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIS4_IPKNS0_10BasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE9_M_insertEPSt18_Rb_tree_node_baseSO_RKSH_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIS4_IPKNS0_10BasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE16_M_insert_uniqueERKSH_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > > >::_M_insert_unique(std::pair<llvm::Function const* const, std::map<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*>, double, std::less<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::BasicBlock const*, llvm::BasicBlock const*> const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm17MachineBasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::MachineBasicBlock const*, std::pair<llvm::MachineBasicBlock const* const, double>, std::_Select1st<std::pair<llvm::MachineBasicBlock const* const, double> >, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineBasicBlock const* const, double> >, std::pair<llvm::MachineBasicBlock const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm17MachineBasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::MachineBasicBlock const*, std::pair<llvm::MachineBasicBlock const* const, double>, std::_Select1st<std::pair<llvm::MachineBasicBlock const* const, double> >, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > >::_M_insert_unique(std::pair<llvm::MachineBasicBlock const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm17MachineBasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::MachineBasicBlock const*, std::pair<llvm::MachineBasicBlock const* const, double>, std::_Select1st<std::pair<llvm::MachineBasicBlock const* const, double> >, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineBasicBlock const* const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm17MachineBasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE7_M_copyEPKSt13_Rb_tree_nodeIS6_EPSE_", "std::_Rb_tree<llvm::MachineBasicBlock const*, std::pair<llvm::MachineBasicBlock const* const, double>, std::_Select1st<std::pair<llvm::MachineBasicBlock const* const, double> >, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::MachineBasicBlock const* const, double> > const*, std::_Rb_tree_node<std::pair<llvm::MachineBasicBlock const* const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_St3mapIPKNS0_17MachineBasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISG_ERKSG_", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_St3mapIPKNS0_17MachineBasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE9_M_insertEPSt18_Rb_tree_node_baseSN_RKSG_", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_St3mapIPKNS0_17MachineBasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > > >::_M_insert_unique(std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, double>, std::_Select1st<std::pair<llvm::MachineFunction const* const, double> >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, double> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::MachineFunction const* const, double> >, std::pair<llvm::MachineFunction const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, double>, std::_Select1st<std::pair<llvm::MachineFunction const* const, double> >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, double> > >::_M_insert_unique(std::pair<llvm::MachineFunction const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, double>, std::_Select1st<std::pair<llvm::Function const* const, double> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, double> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, double> >, std::pair<llvm::Function const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, double>, std::_Select1st<std::pair<llvm::Function const* const, double> >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, double> > >::_M_insert_unique(std::pair<llvm::Function const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, double>, std::_Select1st<std::pair<llvm::BasicBlock const* const, double> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const* const, double> >, std::pair<llvm::BasicBlock const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, double>, std::_Select1st<std::pair<llvm::BasicBlock const* const, double> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >::_M_insert_unique(std::pair<llvm::BasicBlock const* const, double> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE7_M_copyEPKSt13_Rb_tree_nodeIS6_EPSE_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, double>, std::_Select1st<std::pair<llvm::BasicBlock const* const, double> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, double> > const*, std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIPKNS0_10BasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISG_ERKSG_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIPKNS0_10BasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE9_M_insertEPSt18_Rb_tree_node_baseSN_RKSG_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm8FunctionESt4pairIKS3_St3mapIPKNS0_10BasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::Function const*, std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > >, std::less<llvm::Function const*>, std::allocator<std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > > >::_M_insert_unique(std::pair<llvm::Function const* const, std::map<llvm::BasicBlock const*, double, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, double> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE16_M_insert_uniqueERKS3_", "std::_Rb_tree<llvm::BasicBlock const*, llvm::BasicBlock const*, std::_Identity<llvm::BasicBlock const*>, std::less<llvm::BasicBlock const*>, std::allocator<llvm::BasicBlock const*> >::_M_insert_unique(llvm::BasicBlock const* const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, llvm::BasicBlock const*> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_St3mapIS4_IPKNS0_17MachineBasicBlockES9_EdSt4lessISA_ESaIS4_IKSA_dEEEESt10_Select1stISH_ESB_IS3_ESaISH_EE8_M_eraseEPSt13_Rb_tree_nodeISH_E", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, std::map<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>, double, std::less<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> > > >, std::_Select1st<std::pair<llvm::MachineFunction const* const, std::map<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>, double, std::less<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> > > > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>, double, std::less<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineFunction const* const, std::map<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>, double, std::less<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> > > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm17MachineBasicBlockES4_ES0_IKS5_dESt10_Select1stIS7_ESt4lessIS5_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*>, std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double>, std::_Select1st<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> >, std::less<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> >, std::allocator<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::MachineBasicBlock const*, llvm::MachineBasicBlock const*> const, double> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_St3mapIPKNS0_17MachineBasicBlockEdSt4lessIS9_ESaIS4_IKS9_dEEEESt10_Select1stISG_ESA_IS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > >, std::_Select1st<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineFunction const* const, std::map<llvm::MachineBasicBlock const*, double, std::less<llvm::MachineBasicBlock const*>, std::allocator<std::pair<llvm::MachineBasicBlock const* const, double> > > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm15MachineFunctionESt4pairIKS3_dESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::MachineFunction const*, std::pair<llvm::MachineFunction const* const, double>, std::_Select1st<std::pair<llvm::MachineFunction const* const, double> >, std::less<llvm::MachineFunction const*>, std::allocator<std::pair<llvm::MachineFunction const* const, double> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::MachineFunction const* const, double> >*)"},
    {"_ZN4llvm29initializeScalarEvolutionPassERNS_12PassRegistryE", "llvm::initializeScalarEvolutionPass(llvm::PassRegistry&)"},
    {"_ZNK4llvm4SCEV4dumpEv", "llvm::SCEV::dump() const"},
    {"_ZNK4llvm4SCEV5printERNS_11raw_ostreamE", "llvm::SCEV::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm4SCEV7getTypeEv", "llvm::SCEV::getType() const"},
    {"_ZNK4llvm11SCEVUnknown8isSizeOfERPKNS_4TypeE", "llvm::SCEVUnknown::isSizeOf(llvm::Type const*&) const"},
    {"_ZNK4llvm11SCEVUnknown9isAlignOfERPKNS_4TypeE", "llvm::SCEVUnknown::isAlignOf(llvm::Type const*&) const"},
    {"_ZNK4llvm11SCEVUnknown10isOffsetOfERPKNS_4TypeERPNS_8ConstantE", "llvm::SCEVUnknown::isOffsetOf(llvm::Type const*&, llvm::Constant*&) const"},
    {"_ZNK4llvm4SCEV6isZeroEv", "llvm::SCEV::isZero() const"},
    {"_ZNK4llvm4SCEV5isOneEv", "llvm::SCEV::isOne() const"},
    {"_ZNK4llvm4SCEV14isAllOnesValueEv", "llvm::SCEV::isAllOnesValue() const"},
    {"_ZN4llvm19SCEVCouldNotComputeC1Ev", "llvm::SCEVCouldNotCompute::SCEVCouldNotCompute()"},
    {"_ZN4llvm19SCEVCouldNotComputeC2Ev", "llvm::SCEVCouldNotCompute::SCEVCouldNotCompute()"},
    {"_ZN4llvm19SCEVCouldNotCompute7classofEPKNS_4SCEVE", "llvm::SCEVCouldNotCompute::classof(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getConstantEPNS_11ConstantIntE", "llvm::ScalarEvolution::getConstant(llvm::ConstantInt*)"},
    {"_ZN4llvm15ScalarEvolution11getConstantERKNS_5APIntE", "llvm::ScalarEvolution::getConstant(llvm::APInt const&)"},
    {"_ZN4llvm15ScalarEvolution11getConstantEPKNS_4TypeEyb", "llvm::ScalarEvolution::getConstant(llvm::Type const*, unsigned long long, bool)"},
    {"_ZNK4llvm15ScalarEvolution20getEffectiveSCEVTypeEPKNS_4TypeE", "llvm::ScalarEvolution::getEffectiveSCEVType(llvm::Type const*) const"},
    {"_ZN4llvm12SCEVCastExprC1ENS_19FoldingSetNodeIDRefEjPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVCastExpr::SCEVCastExpr(llvm::FoldingSetNodeIDRef, unsigned int, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm12SCEVCastExprC2ENS_19FoldingSetNodeIDRefEjPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVCastExpr::SCEVCastExpr(llvm::FoldingSetNodeIDRef, unsigned int, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm16SCEVTruncateExprC1ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVTruncateExpr::SCEVTruncateExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm16SCEVTruncateExprC2ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVTruncateExpr::SCEVTruncateExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm18SCEVZeroExtendExprC1ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVZeroExtendExpr::SCEVZeroExtendExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm18SCEVZeroExtendExprC2ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVZeroExtendExpr::SCEVZeroExtendExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm18SCEVSignExtendExprC1ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVSignExtendExpr::SCEVSignExtendExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm18SCEVSignExtendExprC2ENS_19FoldingSetNodeIDRefEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVSignExtendExpr::SCEVSignExtendExpr(llvm::FoldingSetNodeIDRef, llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm11SCEVUnknown7deletedEv", "llvm::SCEVUnknown::deleted()"},
    {"_ZN4llvm15ScalarEvolution21forgetMemoizedResultsEPKNS_4SCEVE", "llvm::ScalarEvolution::forgetMemoizedResults(llvm::SCEV const*)"},
    {"_ZN4llvm11SCEVUnknown19allUsesReplacedWithEPNS_5ValueE", "llvm::SCEVUnknown::allUsesReplacedWith(llvm::Value*)"},
    {"_ZNK4llvm14SCEVAddRecExpr19evaluateAtIterationEPKNS_4SCEVERNS_15ScalarEvolutionE", "llvm::SCEVAddRecExpr::evaluateAtIteration(llvm::SCEV const*, llvm::ScalarEvolution&) const"},
    {"_ZN4llvm15ScalarEvolution15getTruncateExprEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getTruncateExpr(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution23getTruncateOrSignExtendEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getTruncateOrSignExtend(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution23getTruncateOrZeroExtendEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getTruncateOrZeroExtend(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution10getAddExprERNS_15SmallVectorImplIPKNS_4SCEVEEEbb", "llvm::ScalarEvolution::getAddExpr(llvm::SmallVectorImpl<llvm::SCEV const*>&, bool, bool)"},
    {"_ZN4llvm15ScalarEvolution10getMulExprERNS_15SmallVectorImplIPKNS_4SCEVEEEbb", "llvm::ScalarEvolution::getMulExpr(llvm::SmallVectorImpl<llvm::SCEV const*>&, bool, bool)"},
    {"_ZN4llvm15ScalarEvolution13getAddRecExprERNS_15SmallVectorImplIPKNS_4SCEVEEEPKNS_4LoopEbb", "llvm::ScalarEvolution::getAddRecExpr(llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::Loop const*, bool, bool)"},
    {"_ZN4llvm15ScalarEvolution7getSCEVEPNS_5ValueE", "llvm::ScalarEvolution::getSCEV(llvm::Value*)"},
    {"_ZN4llvm15ScalarEvolution17getZeroExtendExprEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getZeroExtendExpr(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution16getUnsignedRangeEPKNS_4SCEVE", "llvm::ScalarEvolution::getUnsignedRange(llvm::SCEV const*)"},
    {"_ZNK4llvm15ScalarEvolution17getTypeSizeInBitsEPKNS_4TypeE", "llvm::ScalarEvolution::getTypeSizeInBits(llvm::Type const*) const"},
    {"_ZN4llvm15ScalarEvolution13getAddRecExprEPKNS_4SCEVES3_PKNS_4LoopEbb", "llvm::ScalarEvolution::getAddRecExpr(llvm::SCEV const*, llvm::SCEV const*, llvm::Loop const*, bool, bool)"},
    {"_ZN4llvm15ScalarEvolution24getMaxBackedgeTakenCountEPKNS_4LoopE", "llvm::ScalarEvolution::getMaxBackedgeTakenCount(llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution17getSignExtendExprEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getSignExtendExpr(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution15isKnownPositiveEPKNS_4SCEVE", "llvm::ScalarEvolution::isKnownPositive(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution27isLoopBackedgeGuardedByCondEPKNS_4LoopENS_7CmpInst9PredicateEPKNS_4SCEVES8_", "llvm::ScalarEvolution::isLoopBackedgeGuardedByCond(llvm::Loop const*, llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution24isLoopEntryGuardedByCondEPKNS_4LoopENS_7CmpInst9PredicateEPKNS_4SCEVES8_", "llvm::ScalarEvolution::isLoopEntryGuardedByCond(llvm::Loop const*, llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution15isKnownNegativeEPKNS_4SCEVE", "llvm::ScalarEvolution::isKnownNegative(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution14getSignedRangeEPKNS_4SCEVE", "llvm::ScalarEvolution::getSignedRange(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution18isKnownNonNegativeEPKNS_4SCEVE", "llvm::ScalarEvolution::isKnownNonNegative(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution16getAnyExtendExprEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getAnyExtendExpr(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution17getTruncateOrNoopEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getTruncateOrNoop(llvm::SCEV const*, llvm::Type const*)"},
    {"_Z17GroupByComplexityRN4llvm15SmallVectorImplIPKNS_4SCEVEEEPNS_8LoopInfoE", "GroupByComplexity(llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::LoopInfo*)"},
    {"_Z28CollectAddOperandsWithScalesRN4llvm8DenseMapIPKNS_4SCEVENS_5APIntENS_12DenseMapInfoIS3_EENS5_IS4_EEEERNS_11SmallVectorIS3_Lj8EEERS4_PKS3_mRKS4_RNS_15ScalarEvolutionE", "CollectAddOperandsWithScales(llvm::DenseMap<llvm::SCEV const*, llvm::APInt, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::APInt> >&, llvm::SmallVector<llvm::SCEV const*, 8u>&, llvm::APInt&, llvm::SCEV const* const*, unsigned long, llvm::APInt const&, llvm::ScalarEvolution&)"},
    {"_ZN4llvm15ScalarEvolution15isLoopInvariantEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::isLoopInvariant(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution11getUDivExprEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getUDivExpr(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getSMaxExprEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getSMaxExpr(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getSMaxExprERNS_15SmallVectorImplIPKNS_4SCEVEEE", "llvm::ScalarEvolution::getSMaxExpr(llvm::SmallVectorImpl<llvm::SCEV const*>&)"},
    {"_ZN4llvm15ScalarEvolution16isKnownPredicateENS_7CmpInst9PredicateEPKNS_4SCEVES5_", "llvm::ScalarEvolution::isKnownPredicate(llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getUMaxExprEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getUMaxExpr(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getUMaxExprERNS_15SmallVectorImplIPKNS_4SCEVEEE", "llvm::ScalarEvolution::getUMaxExpr(llvm::SmallVectorImpl<llvm::SCEV const*>&)"},
    {"_ZN4llvm15ScalarEvolution11getSMinExprEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getSMinExpr(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution10getNotSCEVEPKNS_4SCEVE", "llvm::ScalarEvolution::getNotSCEV(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution11getUMinExprEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getUMinExpr(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution13getSizeOfExprEPKNS_4TypeE", "llvm::ScalarEvolution::getSizeOfExpr(llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution14getAlignOfExprEPKNS_4TypeE", "llvm::ScalarEvolution::getAlignOfExpr(llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution15getOffsetOfExprEPKNS_10StructTypeEj", "llvm::ScalarEvolution::getOffsetOfExpr(llvm::StructType const*, unsigned int)"},
    {"_ZN4llvm15ScalarEvolution15getOffsetOfExprEPKNS_4TypeEPNS_8ConstantE", "llvm::ScalarEvolution::getOffsetOfExpr(llvm::Type const*, llvm::Constant*)"},
    {"_ZN4llvm15ScalarEvolution10getUnknownEPNS_5ValueE", "llvm::ScalarEvolution::getUnknown(llvm::Value*)"},
    {"_ZNK4llvm15ScalarEvolution10isSCEVableEPKNS_4TypeE", "llvm::ScalarEvolution::isSCEVable(llvm::Type const*) const"},
    {"_ZN4llvm15ScalarEvolution18getCouldNotComputeEv", "llvm::ScalarEvolution::getCouldNotCompute()"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVHC1EPNS_5ValueEPS0_", "llvm::ScalarEvolution::SCEVCallbackVH::SCEVCallbackVH(llvm::Value*, llvm::ScalarEvolution*)"},
    {"_ZN4llvm15ScalarEvolution10createSCEVEPNS_5ValueE", "llvm::ScalarEvolution::createSCEV(llvm::Value*)"},
    {"_ZN4llvm15ScalarEvolution15getNegativeSCEVEPKNS_4SCEVE", "llvm::ScalarEvolution::getNegativeSCEV(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution12getMinusSCEVEPKNS_4SCEVES3_bb", "llvm::ScalarEvolution::getMinusSCEV(llvm::SCEV const*, llvm::SCEV const*, bool, bool)"},
    {"_ZN4llvm15ScalarEvolution19getNoopOrZeroExtendEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getNoopOrZeroExtend(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution19getNoopOrSignExtendEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getNoopOrSignExtend(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution18getNoopOrAnyExtendEPKNS_4SCEVEPKNS_4TypeE", "llvm::ScalarEvolution::getNoopOrAnyExtend(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm15ScalarEvolution26getUMaxFromMismatchedTypesEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getUMaxFromMismatchedTypes(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution26getUMinFromMismatchedTypesEPKNS_4SCEVES3_", "llvm::ScalarEvolution::getUMinFromMismatchedTypes(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution18ForgetSymbolicNameEPNS_11InstructionEPKNS_4SCEVE", "llvm::ScalarEvolution::ForgetSymbolicName(llvm::Instruction*, llvm::SCEV const*)"},
    {"_ZNK4llvm15ScalarEvolution10hasOperandEPKNS_4SCEVES3_", "llvm::ScalarEvolution::hasOperand(llvm::SCEV const*, llvm::SCEV const*) const"},
    {"_ZN4llvm15ScalarEvolution16createNodeForPHIEPNS_7PHINodeE", "llvm::ScalarEvolution::createNodeForPHI(llvm::PHINode*)"},
    {"_ZN4llvm15ScalarEvolution16createNodeForGEPEPNS_11GEPOperatorE", "llvm::ScalarEvolution::createNodeForGEP(llvm::GEPOperator*)"},
    {"_ZN4llvm15ScalarEvolution19GetMinTrailingZerosEPKNS_4SCEVE", "llvm::ScalarEvolution::GetMinTrailingZeros(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution18isKnownNonPositiveEPKNS_4SCEVE", "llvm::ScalarEvolution::isKnownNonPositive(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution21getBackedgeTakenCountEPKNS_4LoopE", "llvm::ScalarEvolution::getBackedgeTakenCount(llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution20getBackedgeTakenInfoEPKNS_4LoopE", "llvm::ScalarEvolution::getBackedgeTakenInfo(llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution25ComputeBackedgeTakenCountEPKNS_4LoopE", "llvm::ScalarEvolution::ComputeBackedgeTakenCount(llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution10forgetLoopEPKNS_4LoopE", "llvm::ScalarEvolution::forgetLoop(llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution11forgetValueEPNS_5ValueE", "llvm::ScalarEvolution::forgetValue(llvm::Value*)"},
    {"_ZN4llvm15ScalarEvolution33ComputeBackedgeTakenCountFromExitEPKNS_4LoopEPNS_10BasicBlockE", "llvm::ScalarEvolution::ComputeBackedgeTakenCountFromExit(llvm::Loop const*, llvm::BasicBlock*)"},
    {"_ZN4llvm15ScalarEvolution37ComputeBackedgeTakenCountFromExitCondEPKNS_4LoopEPNS_5ValueEPNS_10BasicBlockES7_", "llvm::ScalarEvolution::ComputeBackedgeTakenCountFromExitCond(llvm::Loop const*, llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm15ScalarEvolution41ComputeBackedgeTakenCountFromExitCondICmpEPKNS_4LoopEPNS_8ICmpInstEPNS_10BasicBlockES7_", "llvm::ScalarEvolution::ComputeBackedgeTakenCountFromExitCondICmp(llvm::Loop const*, llvm::ICmpInst*, llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm15ScalarEvolution37ComputeBackedgeTakenCountExhaustivelyEPKNS_4LoopEPNS_5ValueEb", "llvm::ScalarEvolution::ComputeBackedgeTakenCountExhaustively(llvm::Loop const*, llvm::Value*, bool)"},
    {"_ZN4llvm15ScalarEvolution44ComputeLoadConstantCompareBackedgeTakenCountEPNS_8LoadInstEPNS_8ConstantEPKNS_4LoopENS_7CmpInst9PredicateE", "llvm::ScalarEvolution::ComputeLoadConstantCompareBackedgeTakenCount(llvm::LoadInst*, llvm::Constant*, llvm::Loop const*, llvm::CmpInst::Predicate)"},
    {"_ZN4llvm15ScalarEvolution14getSCEVAtScopeEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::getSCEVAtScope(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution20SimplifyICmpOperandsERNS_7CmpInst9PredicateERPKNS_4SCEVES7_", "llvm::ScalarEvolution::SimplifyICmpOperands(llvm::CmpInst::Predicate&, llvm::SCEV const*&, llvm::SCEV const*&)"},
    {"_ZNK4llvm14SCEVAddRecExpr23getNumIterationsInRangeENS_13ConstantRangeERNS_15ScalarEvolutionE", "llvm::SCEVAddRecExpr::getNumIterationsInRange(llvm::ConstantRange, llvm::ScalarEvolution&) const"},
    {"_ZN4llvm15ScalarEvolution12HowFarToZeroEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::HowFarToZero(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution15HowFarToNonZeroEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::HowFarToNonZero(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution16HowManyLessThansEPKNS_4SCEVES3_PKNS_4LoopEb", "llvm::ScalarEvolution::HowManyLessThans(llvm::SCEV const*, llvm::SCEV const*, llvm::Loop const*, bool)"},
    {"_ZN4llvm15ScalarEvolution33getConstantEvolutionLoopExitValueEPNS_7PHINodeERKNS_5APIntEPKNS_4LoopE", "llvm::ScalarEvolution::getConstantEvolutionLoopExitValue(llvm::PHINode*, llvm::APInt const&, llvm::Loop const*)"},
    {"_Z22getConstantEvolvingPHIPN4llvm5ValueEPKNS_4LoopE", "getConstantEvolvingPHI(llvm::Value*, llvm::Loop const*)"},
    {"_Z18EvaluateExpressionPN4llvm5ValueEPNS_8ConstantEPKNS_10DataLayoutE", "EvaluateExpression(llvm::Value*, llvm::Constant*, llvm::DataLayout const*)"},
    {"_ZN4llvm15ScalarEvolution18computeSCEVAtScopeEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::computeSCEVAtScope(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution14getSCEVAtScopeEPNS_5ValueEPKNS_4LoopE", "llvm::ScalarEvolution::getSCEVAtScope(llvm::Value*, llvm::Loop const*)"},
    {"_Z22SolveQuadraticEquationPKN4llvm14SCEVAddRecExprERNS_15ScalarEvolutionE", "SolveQuadraticEquation(llvm::SCEVAddRecExpr const*, llvm::ScalarEvolution&)"},
    {"_ZN4llvm15ScalarEvolution38getPredecessorWithUniqueSuccessorForBBEPNS_10BasicBlockE", "llvm::ScalarEvolution::getPredecessorWithUniqueSuccessorForBB(llvm::BasicBlock*)"},
    {"_ZN4llvm15ScalarEvolution17properlyDominatesEPKNS_4SCEVEPKNS_10BasicBlockE", "llvm::ScalarEvolution::properlyDominates(llvm::SCEV const*, llvm::BasicBlock const*)"},
    {"_Z12HasSameValuePKN4llvm4SCEVES2_", "HasSameValue(llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution14isKnownNonZeroEPKNS_4SCEVE", "llvm::ScalarEvolution::isKnownNonZero(llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution26isKnownPredicateWithRangesENS_7CmpInst9PredicateEPKNS_4SCEVES5_", "llvm::ScalarEvolution::isKnownPredicateWithRanges(llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution13isImpliedCondENS_7CmpInst9PredicateEPKNS_4SCEVES5_PNS_5ValueEb", "llvm::ScalarEvolution::isImpliedCond(llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*, llvm::Value*, bool)"},
    {"_ZN4llvm15ScalarEvolution21isImpliedCondOperandsENS_7CmpInst9PredicateEPKNS_4SCEVES5_S5_S5_", "llvm::ScalarEvolution::isImpliedCondOperands(llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution27isImpliedCondOperandsHelperENS_7CmpInst9PredicateEPKNS_4SCEVES5_S5_S5_", "llvm::ScalarEvolution::isImpliedCondOperandsHelper(llvm::CmpInst::Predicate, llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*)"},
    {"_ZN4llvm15ScalarEvolution10getBECountEPKNS_4SCEVES3_S3_b", "llvm::ScalarEvolution::getBECount(llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*, bool)"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVH7deletedEv", "llvm::ScalarEvolution::SCEVCallbackVH::deleted()"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVH19allUsesReplacedWithEPNS_5ValueE", "llvm::ScalarEvolution::SCEVCallbackVH::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVHC2EPNS_5ValueEPS0_", "llvm::ScalarEvolution::SCEVCallbackVH::SCEVCallbackVH(llvm::Value*, llvm::ScalarEvolution*)"},
    {"_ZN4llvm15ScalarEvolutionC1Ev", "llvm::ScalarEvolution::ScalarEvolution()"},
    {"_ZN4llvm15ScalarEvolutionC2Ev", "llvm::ScalarEvolution::ScalarEvolution()"},
    {"_ZN4llvm15ScalarEvolution13runOnFunctionERNS_8FunctionE", "llvm::ScalarEvolution::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm15ScalarEvolution13releaseMemoryEv", "llvm::ScalarEvolution::releaseMemory()"},
    {"_ZNK4llvm15ScalarEvolution16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::ScalarEvolution::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm15ScalarEvolution34hasLoopInvariantBackedgeTakenCountEPKNS_4LoopE", "llvm::ScalarEvolution::hasLoopInvariantBackedgeTakenCount(llvm::Loop const*)"},
    {"_ZNK4llvm15ScalarEvolution5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::ScalarEvolution::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_Z13PrintLoopInfoRN4llvm11raw_ostreamEPNS_15ScalarEvolutionEPKNS_4LoopE", "PrintLoopInfo(llvm::raw_ostream&, llvm::ScalarEvolution*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution18getLoopDispositionEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::getLoopDisposition(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution22computeLoopDispositionEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::computeLoopDisposition(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution26hasComputableLoopEvolutionEPKNS_4SCEVEPKNS_4LoopE", "llvm::ScalarEvolution::hasComputableLoopEvolution(llvm::SCEV const*, llvm::Loop const*)"},
    {"_ZN4llvm15ScalarEvolution19getBlockDispositionEPKNS_4SCEVEPKNS_10BasicBlockE", "llvm::ScalarEvolution::getBlockDisposition(llvm::SCEV const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm15ScalarEvolution23computeBlockDispositionEPKNS_4SCEVEPKNS_10BasicBlockE", "llvm::ScalarEvolution::computeBlockDisposition(llvm::SCEV const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm15ScalarEvolution9dominatesEPKNS_4SCEVEPKNS_10BasicBlockE", "llvm::ScalarEvolution::dominates(llvm::SCEV const*, llvm::BasicBlock const*)"},
    {"_ZNSt8_Rb_treeIN4llvm5APIntESt4pairIKS1_NS0_11SmallVectorIPKNS0_4SCEVELj4EEEESt10_Select1stIS9_EN12_GLOBAL__N_112APIntCompareESaIS9_EE9_M_insertEPSt18_Rb_tree_node_baseSH_RKS9_", "std::_Rb_tree<llvm::APInt, std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> >, std::_Select1st<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > >, (anonymous namespace)::APIntCompare, std::allocator<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > const&)"},
    {"_ZNSt8_Rb_treeIN4llvm5APIntESt4pairIKS1_NS0_11SmallVectorIPKNS0_4SCEVELj4EEEESt10_Select1stIS9_EN12_GLOBAL__N_112APIntCompareESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<llvm::APInt, std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> >, std::_Select1st<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > >, (anonymous namespace)::APIntCompare, std::allocator<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > > >::_M_insert_unique(std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > const&)"},
    {"_ZNSt8_Rb_treeIN4llvm5APIntESt4pairIKS1_NS0_11SmallVectorIPKNS0_4SCEVELj4EEEESt10_Select1stIS9_EN12_GLOBAL__N_112APIntCompareESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<llvm::APInt, std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> >, std::_Select1st<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > >, (anonymous namespace)::APIntCompare, std::allocator<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::APInt const, llvm::SmallVector<llvm::SCEV const*, 4u> > >*)"},
    {"_ZSt21__inplace_stable_sortIPPKN4llvm4SCEVEN12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_T0_", "void std::__inplace_stable_sort<llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZSt22__stable_sort_adaptiveIPPKN4llvm4SCEVES4_lN12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_T0_T1_T2_", "void std::__stable_sort_adaptive<llvm::SCEV const**, llvm::SCEV const**, long, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**, long, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZSt24__merge_sort_with_bufferIPPKN4llvm4SCEVES4_N12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_T0_T1_", "void std::__merge_sort_with_buffer<llvm::SCEV const**, llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZSt16__merge_adaptiveIPPKN4llvm4SCEVElS4_N12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_S7_T0_S8_T1_S8_T2_", "void std::__merge_adaptive<llvm::SCEV const**, long, llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**, long, long, llvm::SCEV const**, long, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZSt16__insertion_sortIPPKN4llvm4SCEVEN12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_T0_", "void std::__insertion_sort<llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZSt22__merge_without_bufferIPPKN4llvm4SCEVElN12_GLOBAL__N_121SCEVComplexityCompareEEvT_S7_S7_T0_S8_T1_", "void std::__merge_without_buffer<llvm::SCEV const**, long, (anonymous namespace)::SCEVComplexityCompare>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**, long, long, (anonymous namespace)::SCEVComplexityCompare)"},
    {"_ZNK12_GLOBAL__N_121SCEVComplexityCompare7compareEPKN4llvm4SCEVES4_", "(anonymous namespace)::SCEVComplexityCompare::compare(llvm::SCEV const*, llvm::SCEV const*) const"},
    {"_ZN4llvm15ScalarEvolution10getAddExprEPKNS_4SCEVES3_S3_bb", "llvm::ScalarEvolution::getAddExpr(llvm::SCEV const*, llvm::SCEV const*, llvm::SCEV const*, bool, bool)"},
    {"_ZNK4llvm11ConstantInt10isMaxValueEb", "llvm::ConstantInt::isMaxValue(bool) const"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE4findERKS2_", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::find(llvm::ScalarEvolution::SCEVCallbackVH const&)"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVHD1Ev", "llvm::ScalarEvolution::SCEVCallbackVH::~SCEVCallbackVH()"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE6insertERKSt4pairIS2_S5_E", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::insert(std::pair<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*> const&)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE5eraseENS_16DenseMapIteratorIS2_S5_S9_SA_Lb0EEE", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::erase(llvm::DenseMapIterator<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*>, false>)"},
    {"_ZN4llvm8LoopInfo29replacementPreservesLCSSAFormEPNS_11InstructionEPNS_5ValueE", "llvm::LoopInfo::replacementPreservesLCSSAForm(llvm::Instruction*, llvm::Value*)"},
    {"_ZN4llvm15ScalarEvolution16setUnsignedRangeEPKNS_4SCEVERKNS_13ConstantRangeE", "llvm::ScalarEvolution::setUnsignedRange(llvm::SCEV const*, llvm::ConstantRange const&)"},
    {"_ZN4llvm15ScalarEvolution14setSignedRangeEPKNS_4SCEVERKNS_13ConstantRangeE", "llvm::ScalarEvolution::setSignedRange(llvm::SCEV const*, llvm::ConstantRange const&)"},
    {"_ZNK4llvm5APInt3ugtEy", "llvm::APInt::ugt(unsigned long long) const"},
    {"_ZNSt3mapIPKN4llvm4SCEVES_IPKNS0_4LoopES3_St4lessIS6_ESaISt4pairIKS6_S3_EEES7_IS3_ESaIS9_IKS3_SD_EEEixERSF_", "std::map<llvm::SCEV const*, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::operator[](llvm::SCEV const* const&)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE5eraseERKS2_", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::erase(llvm::ScalarEvolution::SCEVCallbackVH const&)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE5clearEv", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::clear()"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE5clearEv", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::clear()"},
    {"_ZNSt3mapIPKN4llvm4SCEVES_IPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS6_ESaISt4pairIKS6_S8_EEES9_IS3_ESaISB_IKS3_SF_EEEixERSH_", "std::map<llvm::SCEV const*, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::operator[](llvm::SCEV const* const&)"},
    {"_ZNSt3mapIPKN4llvm4SCEVES_IPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS6_ESaISt4pairIKS6_S8_EEES9_IS3_ESaISB_IKS3_SF_EEEixERSH_", "std::map<llvm::SCEV const*, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::operator[](llvm::SCEV const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE5eraseERKS3_", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::erase(llvm::SCEV const* const&)"},
    {"_ZN4llvm11SCEVUnknownD1Ev", "llvm::SCEVUnknown::~SCEVUnknown()"},
    {"_ZN4llvm11SCEVUnknownD0Ev", "llvm::SCEVUnknown::~SCEVUnknown()"},
    {"_ZN4llvm15ScalarEvolution14SCEVCallbackVHD0Ev", "llvm::ScalarEvolution::SCEVCallbackVH::~SCEVCallbackVH()"},
    {"_ZN4llvm15ScalarEvolutionD1Ev", "llvm::ScalarEvolution::~ScalarEvolution()"},
    {"_ZN4llvm15ScalarEvolutionD0Ev", "llvm::ScalarEvolution::~ScalarEvolution()"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE5eraseESt17_Rb_tree_iteratorISI_ESP_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_NS0_15ScalarEvolution16BlockDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE8_M_eraseEPSt13_Rb_tree_nodeISI_E", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE5eraseESt17_Rb_tree_iteratorISI_ESP_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution15LoopDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE8_M_eraseEPSt13_Rb_tree_nodeISI_E", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopES3_St4lessIS9_ESaIS4_IKS9_S3_EEEESt10_Select1stISG_ESA_IS3_ESaISG_EE5eraseESt17_Rb_tree_iteratorISG_ESN_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_PKNS0_4SCEVEESt10_Select1stIS9_ESt4lessIS3_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::SCEV const*>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::SCEV const*> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::SCEV const*> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopES3_St4lessIS9_ESaIS4_IKS9_S3_EEEESt10_Select1stISG_ESA_IS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_NS0_15ScalarEvolution16BlockDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >, std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_NS0_15ScalarEvolution16BlockDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > >::_M_insert_unique(std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm10BasicBlockESt4pairIKS3_NS0_15ScalarEvolution16BlockDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE7_M_copyEPKSt13_Rb_tree_nodeIS8_EPSG_", "std::_Rb_tree<llvm::BasicBlock const*, std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition>, std::_Select1st<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > const*, std::_Rb_tree_node<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISI_ERKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE9_M_insertEPSt18_Rb_tree_node_baseSP_RKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_10BasicBlockENS0_15ScalarEvolution16BlockDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE16_M_insert_uniqueERKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > > >::_M_insert_unique(std::pair<llvm::SCEV const* const, std::map<llvm::BasicBlock const*, llvm::ScalarEvolution::BlockDisposition, std::less<llvm::BasicBlock const*>, std::allocator<std::pair<llvm::BasicBlock const* const, llvm::ScalarEvolution::BlockDisposition> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution15LoopDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS8_ERKS8_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution15LoopDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > >::_M_insert_unique(std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution15LoopDispositionEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE7_M_copyEPKSt13_Rb_tree_nodeIS8_EPSG_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > const*, std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISI_ERKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE9_M_insertEPSt18_Rb_tree_node_baseSP_RKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopENS0_15ScalarEvolution15LoopDispositionESt4lessIS9_ESaIS4_IKS9_SB_EEEESt10_Select1stISI_ESC_IS3_ESaISI_EE16_M_insert_uniqueERKSI_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > > >::_M_insert_unique(std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::ScalarEvolution::LoopDisposition, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::LoopDisposition> > > > const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::shrink_and_clear()"},
    {"_ZNSt8_Rb_treeIPN4llvm7PHINodeESt4pairIKS2_PNS0_8ConstantEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::PHINode*, std::pair<llvm::PHINode* const, llvm::Constant*>, std::_Select1st<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::less<llvm::PHINode*>, std::allocator<std::pair<llvm::PHINode* const, llvm::Constant*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::PHINode* const, llvm::Constant*> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution17BackedgeTakenInfoEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE8_M_eraseEPSt13_Rb_tree_nodeIS8_E", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >*)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::shrink_and_clear()"},
    {"_ZN4llvm10FoldingSetINS_4SCEVEED1Ev", "llvm::FoldingSet<llvm::SCEV>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_4SCEVEED0Ev", "llvm::FoldingSet<llvm::SCEV>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_4SCEVEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::SCEV>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_4SCEVEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS6_", "llvm::FoldingSet<llvm::SCEV>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_4SCEVEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::SCEV>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE4initEj", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::init(unsigned int)"},
    {"_ZNK4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE15LookupBucketForERKS2_RPSt4pairIS2_S5_E", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::LookupBucketFor(llvm::ScalarEvolution::SCEVCallbackVH const&, std::pair<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*>*&) const"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_PKNS0_4SCEVEESt10_Select1stIS9_ESt4lessIS3_ESaIS9_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS9_ERKS9_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::SCEV const*>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::SCEV const*> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Loop const* const, llvm::SCEV const*> >, std::pair<llvm::Loop const* const, llvm::SCEV const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_PKNS0_4SCEVEESt10_Select1stIS9_ESt4lessIS3_ESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::SCEV const*>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::SCEV const*> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > >::_M_insert_unique(std::pair<llvm::Loop const* const, llvm::SCEV const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_PKNS0_4SCEVEESt10_Select1stIS9_ESt4lessIS3_ESaIS9_EE7_M_copyEPKSt13_Rb_tree_nodeIS9_EPSH_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::SCEV const*>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::SCEV const*> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > >::_M_copy(std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::SCEV const*> > const*, std::_Rb_tree_node<std::pair<llvm::Loop const* const, llvm::SCEV const*> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopES3_St4lessIS9_ESaIS4_IKS9_S3_EEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISG_ERKSG_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopES3_St4lessIS9_ESaIS4_IKS9_S3_EEEESt10_Select1stISG_ESA_IS3_ESaISG_EE9_M_insertEPSt18_Rb_tree_node_baseSN_RKSG_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4SCEVESt4pairIKS3_St3mapIPKNS0_4LoopES3_St4lessIS9_ESaIS4_IKS9_S3_EEEESt10_Select1stISG_ESA_IS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::SCEV const*, std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > >, std::_Select1st<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > >, std::less<llvm::SCEV const*>, std::allocator<std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > > >::_M_insert_unique(std::pair<llvm::SCEV const* const, std::map<llvm::Loop const*, llvm::SCEV const*, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::SCEV const*> > > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm7PHINodeESt4pairIKS2_PNS0_8ConstantEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<llvm::PHINode*, std::pair<llvm::PHINode* const, llvm::Constant*>, std::_Select1st<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::less<llvm::PHINode*>, std::allocator<std::pair<llvm::PHINode* const, llvm::Constant*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::pair<llvm::PHINode* const, llvm::Constant*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm7PHINodeESt4pairIKS2_PNS0_8ConstantEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<llvm::PHINode*, std::pair<llvm::PHINode* const, llvm::Constant*>, std::_Select1st<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::less<llvm::PHINode*>, std::allocator<std::pair<llvm::PHINode* const, llvm::Constant*> > >::_M_insert_unique(std::pair<llvm::PHINode* const, llvm::Constant*> const&)"},
    {"_ZNSt6vectorIPN4llvm11ConstantIntESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::ConstantInt*, std::allocator<llvm::ConstantInt*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::ConstantInt**, std::vector<llvm::ConstantInt*, std::allocator<llvm::ConstantInt*> > >, llvm::ConstantInt* const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution17BackedgeTakenInfoEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE5eraseESt17_Rb_tree_iteratorIS8_ESG_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >, std::_Rb_tree_iterator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >)"},
    {"_ZNSt8_Rb_treeIPN4llvm7PHINodeESt4pairIKS2_PNS0_8ConstantEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<llvm::PHINode*, std::pair<llvm::PHINode* const, llvm::Constant*>, std::_Select1st<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::less<llvm::PHINode*>, std::allocator<std::pair<llvm::PHINode* const, llvm::Constant*> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::PHINode* const, llvm::Constant*> >, std::_Rb_tree_iterator<std::pair<llvm::PHINode* const, llvm::Constant*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4LoopESt4pairIKS3_NS0_15ScalarEvolution17BackedgeTakenInfoEESt10_Select1stIS8_ESt4lessIS3_ESaIS8_EE16_M_insert_uniqueERKS8_", "std::_Rb_tree<llvm::Loop const*, std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo>, std::_Select1st<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> >, std::less<llvm::Loop const*>, std::allocator<std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> > >::_M_insert_unique(std::pair<llvm::Loop const* const, llvm::ScalarEvolution::BackedgeTakenInfo> const&)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::InsertIntoBucket(llvm::ScalarEvolution::SCEVCallbackVH const&, llvm::SCEV const* const&, std::pair<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*>*)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapINS_15ScalarEvolution14SCEVCallbackVHEPKNS_4SCEVENS_12DenseMapInfoIPNS_5ValueEEENS6_IS5_EEED2Ev", "llvm::DenseMap<llvm::ScalarEvolution::SCEVCallbackVH, llvm::SCEV const*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::SCEV const*> >::~DenseMap()"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEED2Ev", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::~DenseMap()"},
    {"_ZNK4llvm8LoopBaseINS_10BasicBlockENS_4LoopEE13getExitBlocksERNS_15SmallVectorImplIPS1_EE", "llvm::LoopBase<llvm::BasicBlock, llvm::Loop>::getExitBlocks(llvm::SmallVectorImpl<llvm::BasicBlock*>&) const"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE6insertERKSt4pairIS3_S4_E", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::insert(std::pair<llvm::SCEV const*, llvm::ConstantRange> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::InsertIntoBucket(llvm::SCEV const* const&, llvm::ConstantRange const&, std::pair<llvm::SCEV const*, llvm::ConstantRange>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_13ConstantRangeENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::SCEV const*, llvm::ConstantRange, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::ConstantRange> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_5APIntENS_12DenseMapInfoIS3_EENS5_IS4_EEE6insertERKSt4pairIS3_S4_E", "llvm::DenseMap<llvm::SCEV const*, llvm::APInt, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::APInt> >::insert(std::pair<llvm::SCEV const*, llvm::APInt> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_5APIntENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::SCEV const*, llvm::APInt, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::APInt> >::InsertIntoBucket(llvm::SCEV const* const&, llvm::APInt const&, std::pair<llvm::SCEV const*, llvm::APInt>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVENS_5APIntENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::SCEV const*, llvm::APInt, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::APInt> >::grow(unsigned int)"},
    {"_ZSt8__rotateIPPKN4llvm4SCEVEEvT_S5_S5_St26random_access_iterator_tag", "void std::__rotate<llvm::SCEV const**>(llvm::SCEV const**, llvm::SCEV const**, llvm::SCEV const**, std::random_access_iterator_tag)"},
    {"_ZN4llvm15callDefaultCtorINS_15ScalarEvolutionEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::ScalarEvolution>()"},
    {"_ZN4llvm12SCEVExpander17ReuseOrCreateCastEPNS_5ValueEPKNS_4TypeENS_11Instruction7CastOpsENS_14ilist_iteratorIS6_EE", "llvm::SCEVExpander::ReuseOrCreateCast(llvm::Value*, llvm::Type const*, llvm::Instruction::CastOps, llvm::ilist_iterator<llvm::Instruction>)"},
    {"_ZN4llvm12SCEVExpander19rememberInstructionEPNS_5ValueE", "llvm::SCEVExpander::rememberInstruction(llvm::Value*)"},
    {"_ZN4llvm12SCEVExpander18InsertNoopCastOfToEPNS_5ValueEPKNS_4TypeE", "llvm::SCEVExpander::InsertNoopCastOfTo(llvm::Value*, llvm::Type const*)"},
    {"_ZN4llvm12SCEVExpander11InsertBinopENS_11Instruction9BinaryOpsEPNS_5ValueES4_", "llvm::SCEVExpander::InsertBinop(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm12SCEVExpander18restoreInsertPointEPNS_10BasicBlockENS_14ilist_iteratorINS_11InstructionEEE", "llvm::SCEVExpander::restoreInsertPoint(llvm::BasicBlock*, llvm::ilist_iterator<llvm::Instruction>)"},
    {"_ZN4llvm12SCEVExpander14expandAddToGEPEPKPKNS_4SCEVES5_PKNS_11PointerTypeEPKNS_4TypeEPNS_5ValueE", "llvm::SCEVExpander::expandAddToGEP(llvm::SCEV const* const*, llvm::SCEV const* const*, llvm::PointerType const*, llvm::Type const*, llvm::Value*)"},
    {"_Z17FactorOutConstantRPKN4llvm4SCEVES3_S2_RNS_15ScalarEvolutionEPKNS_10DataLayoutE", "FactorOutConstant(llvm::SCEV const*&, llvm::SCEV const*&, llvm::SCEV const*, llvm::ScalarEvolution&, llvm::DataLayout const*)"},
    {"_Z19SimplifyAddOperandsRN4llvm15SmallVectorImplIPKNS_4SCEVEEEPKNS_4TypeERNS_15ScalarEvolutionE", "SimplifyAddOperands(llvm::SmallVectorImpl<llvm::SCEV const*>&, llvm::Type const*, llvm::ScalarEvolution&)"},
    {"_ZN4llvm12SCEVExpander13expandCodeForEPKNS_4SCEVEPKNS_4TypeE", "llvm::SCEVExpander::expandCodeFor(llvm::SCEV const*, llvm::Type const*)"},
    {"_ZN4llvm12SCEVExpander6expandEPKNS_4SCEVE", "llvm::SCEVExpander::expand(llvm::SCEV const*)"},
    {"_ZN4llvm12SCEVExpander15getRelevantLoopEPKNS_4SCEVE", "llvm::SCEVExpander::getRelevantLoop(llvm::SCEV const*)"},
    {"_ZN4llvm12SCEVExpander12visitAddExprEPKNS_11SCEVAddExprE", "llvm::SCEVExpander::visitAddExpr(llvm::SCEVAddExpr const*)"},
    {"_ZN4llvm12SCEVExpander12visitMulExprEPKNS_11SCEVMulExprE", "llvm::SCEVExpander::visitMulExpr(llvm::SCEVMulExpr const*)"},
    {"_ZN4llvm12SCEVExpander13visitUDivExprEPKNS_12SCEVUDivExprE", "llvm::SCEVExpander::visitUDivExpr(llvm::SCEVUDivExpr const*)"},
    {"_ZN4llvm12SCEVExpander25getAddRecExprPHILiterallyEPKNS_14SCEVAddRecExprEPKNS_4LoopEPKNS_4TypeES9_", "llvm::SCEVExpander::getAddRecExprPHILiterally(llvm::SCEVAddRecExpr const*, llvm::Loop const*, llvm::Type const*, llvm::Type const*)"},
    {"_ZN4llvm12SCEVExpander13expandCodeForEPKNS_4SCEVEPKNS_4TypeEPNS_11InstructionE", "llvm::SCEVExpander::expandCodeFor(llvm::SCEV const*, llvm::Type const*, llvm::Instruction*)"},
    {"_ZN4llvm12SCEVExpander25expandAddRecExprLiterallyEPKNS_14SCEVAddRecExprE", "llvm::SCEVExpander::expandAddRecExprLiterally(llvm::SCEVAddRecExpr const*)"},
    {"_ZN4llvm12SCEVExpander15visitAddRecExprEPKNS_14SCEVAddRecExprE", "llvm::SCEVExpander::visitAddRecExpr(llvm::SCEVAddRecExpr const*)"},
    {"_Z17ExposePointerBaseRPKN4llvm4SCEVES3_RNS_15ScalarEvolutionE", "ExposePointerBase(llvm::SCEV const*&, llvm::SCEV const*&, llvm::ScalarEvolution&)"},
    {"_ZN4llvm12SCEVExpander17visitTruncateExprEPKNS_16SCEVTruncateExprE", "llvm::SCEVExpander::visitTruncateExpr(llvm::SCEVTruncateExpr const*)"},
    {"_ZN4llvm12SCEVExpander19visitZeroExtendExprEPKNS_18SCEVZeroExtendExprE", "llvm::SCEVExpander::visitZeroExtendExpr(llvm::SCEVZeroExtendExpr const*)"},
    {"_ZN4llvm12SCEVExpander19visitSignExtendExprEPKNS_18SCEVSignExtendExprE", "llvm::SCEVExpander::visitSignExtendExpr(llvm::SCEVSignExtendExpr const*)"},
    {"_ZN4llvm12SCEVExpander13visitSMaxExprEPKNS_12SCEVSMaxExprE", "llvm::SCEVExpander::visitSMaxExpr(llvm::SCEVSMaxExpr const*)"},
    {"_ZN4llvm12SCEVExpander13visitUMaxExprEPKNS_12SCEVUMaxExprE", "llvm::SCEVExpander::visitUMaxExpr(llvm::SCEVUMaxExpr const*)"},
    {"_ZN4llvm12SCEVExpander37getOrInsertCanonicalInductionVariableEPKNS_4LoopEPKNS_4TypeE", "llvm::SCEVExpander::getOrInsertCanonicalInductionVariable(llvm::Loop const*, llvm::Type const*)"},
    {"_ZSt21__inplace_stable_sortIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEEN12_GLOBAL__N_111LoopCompareEEvT_SC_T0_", "void std::__inplace_stable_sort<std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare)"},
    {"_ZSt22__stable_sort_adaptiveIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEES9_lN12_GLOBAL__N_111LoopCompareEEvT_SC_T0_T1_T2_", "void std::__stable_sort_adaptive<std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, (anonymous namespace)::LoopCompare)"},
    {"_ZSt24__merge_sort_with_bufferIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEES9_N12_GLOBAL__N_111LoopCompareEEvT_SC_T0_T1_", "void std::__merge_sort_with_buffer<std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare)"},
    {"_ZSt16__merge_adaptiveIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEElS9_N12_GLOBAL__N_111LoopCompareEEvT_SC_SC_T0_SD_T1_SD_T2_", "void std::__merge_adaptive<std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, long, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, (anonymous namespace)::LoopCompare)"},
    {"_ZSt5mergeIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEES9_S9_N12_GLOBAL__N_111LoopCompareEET1_T_SD_T0_SE_SC_T2_", "std::pair<llvm::Loop const*, llvm::SCEV const*>* std::merge<std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare)"},
    {"_ZNK12_GLOBAL__N_111LoopCompareclESt4pairIPKN4llvm4LoopEPKNS2_4SCEVEES9_", "(anonymous namespace)::LoopCompare::operator()(std::pair<llvm::Loop const*, llvm::SCEV const*>, std::pair<llvm::Loop const*, llvm::SCEV const*>) const"},
    {"_ZSt16__insertion_sortIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEEN12_GLOBAL__N_111LoopCompareEEvT_SC_T0_", "void std::__insertion_sort<std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, (anonymous namespace)::LoopCompare)"},
    {"_ZSt22__merge_without_bufferIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEElN12_GLOBAL__N_111LoopCompareEEvT_SC_SC_T0_SD_T1_", "void std::__merge_without_buffer<std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, (anonymous namespace)::LoopCompare>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, long, (anonymous namespace)::LoopCompare)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE11CreateBinOpENS_11Instruction9BinaryOpsEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateBinOp(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateGEPEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateGEP(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateGEPIPPNS_5ValueEEES7_S7_T_S9_RKNS_5TwineE", "llvm::Value* llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateGEP<llvm::Value**>(llvm::Value*, llvm::Value**, llvm::Value**, llvm::Twine const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEPKNS_4LoopENS_12DenseMapInfoIS3_EENS7_IS6_EEE6insertERKSt4pairIS3_S6_E", "llvm::DenseMap<llvm::SCEV const*, llvm::Loop const*, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::Loop const*> >::insert(std::pair<llvm::SCEV const*, llvm::Loop const*> const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreatePHIEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreatePHI(llvm::Type const*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateSubEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateSub(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateAddEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateAdd(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE9CreateMulEPNS_5ValueES6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateMul(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE12CreateSelectEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateSelect(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm11SCEVVisitorINS_12SCEVExpanderEPNS_5ValueEE5visitEPKNS_4SCEVE", "llvm::SCEVVisitor<llvm::SCEVExpander, llvm::Value*>::visit(llvm::SCEV const*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4SCEVEPNS1_11InstructionEES0_IKS7_NS1_11AssertingVHINS1_5ValueEEEESt10_Select1stISC_ESt4lessIS7_ESaISC_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISC_ERKSC_", "std::_Rb_tree<std::pair<llvm::SCEV const*, llvm::Instruction*>, std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> >, std::_Select1st<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > >, std::less<std::pair<llvm::SCEV const*, llvm::Instruction*> >, std::allocator<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > >, std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4SCEVEPNS1_11InstructionEES0_IKS7_NS1_11AssertingVHINS1_5ValueEEEESt10_Select1stISC_ESt4lessIS7_ESaISC_EE16_M_insert_uniqueERKSC_", "std::_Rb_tree<std::pair<llvm::SCEV const*, llvm::Instruction*>, std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> >, std::_Select1st<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > >, std::less<std::pair<llvm::SCEV const*, llvm::Instruction*> >, std::allocator<std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > > >::_M_insert_unique(std::pair<std::pair<llvm::SCEV const*, llvm::Instruction*> const, llvm::AssertingVH<llvm::Value> > const&)"},
    {"_ZN4llvm11SCEVVisitorINS_12SCEVExpanderEPNS_5ValueEE20visitCouldNotComputeEPKNS_19SCEVCouldNotComputeE", "llvm::SCEVVisitor<llvm::SCEVExpander, llvm::Value*>::visitCouldNotCompute(llvm::SCEVCouldNotCompute const*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateICmpENS_7CmpInst9PredicateEPNS_5ValueES8_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateICmp(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZNK4llvm12TargetFolder12CreateSelectEPNS_8ConstantES2_S2_", "llvm::TargetFolder::CreateSelect(llvm::Constant*, llvm::Constant*, llvm::Constant*) const"},
    {"_ZN4llvm9IRBuilderILb1ENS_12TargetFolderENS_24IRBuilderDefaultInserterILb1EEEE10CreateCastENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineE", "llvm::IRBuilder<true, llvm::TargetFolder, llvm::IRBuilderDefaultInserter<true> >::CreateCast(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&)"},
    {"_ZNK4llvm12TargetFolder10CreateCastENS_11Instruction7CastOpsEPNS_8ConstantEPKNS_4TypeE", "llvm::TargetFolder::CreateCast(llvm::Instruction::CastOps, llvm::Constant*, llvm::Type const*) const"},
    {"_ZN4llvm11SmallVectorIPKNS_4SCEVELj4EEC2EjRKS3_", "llvm::SmallVector<llvm::SCEV const*, 4u>::SmallVector(unsigned int, llvm::SCEV const* const&)"},
    {"_ZNSt8_Rb_treeIN4llvm11AssertingVHINS0_5ValueEEES3_St9_IdentityIS3_ESt4lessIS3_ESaIS3_EE16_M_insert_uniqueERKS3_", "std::_Rb_tree<llvm::AssertingVH<llvm::Value>, llvm::AssertingVH<llvm::Value>, std::_Identity<llvm::AssertingVH<llvm::Value> >, std::less<llvm::AssertingVH<llvm::Value> >, std::allocator<llvm::AssertingVH<llvm::Value> > >::_M_insert_unique(llvm::AssertingVH<llvm::Value> const&)"},
    {"_ZSt4copyIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEES9_ET0_T_SB_SA_", "std::pair<llvm::Loop const*, llvm::SCEV const*>* std::copy<std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*)"},
    {"_ZSt17__rotate_adaptiveIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEES9_lET_SA_SA_SA_T1_SB_T0_SB_", "std::pair<llvm::Loop const*, llvm::SCEV const*>* std::__rotate_adaptive<std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long, long, std::pair<llvm::Loop const*, llvm::SCEV const*>*, long)"},
    {"_ZSt8__rotateIPSt4pairIPKN4llvm4LoopEPKNS1_4SCEVEEEvT_SA_SA_St26random_access_iterator_tag", "void std::__rotate<std::pair<llvm::Loop const*, llvm::SCEV const*>*>(std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::pair<llvm::Loop const*, llvm::SCEV const*>*, std::random_access_iterator_tag)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEPKNS_4LoopENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PSt4pairIS3_S6_E", "llvm::DenseMap<llvm::SCEV const*, llvm::Loop const*, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::Loop const*> >::InsertIntoBucket(llvm::SCEV const* const&, llvm::Loop const* const&, std::pair<llvm::SCEV const*, llvm::Loop const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4SCEVEPKNS_4LoopENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::SCEV const*, llvm::Loop const*, llvm::DenseMapInfo<llvm::SCEV const*>, llvm::DenseMapInfo<llvm::Loop const*> >::grow(unsigned int)"},
    {"_ZN4llvm22TransformForPostIncUseENS_13TransformKindEPKNS_4SCEVEPNS_11InstructionEPNS_5ValueERNS_11SmallPtrSetIPKNS_4LoopELj2EEERNS_15ScalarEvolutionERNS_13DominatorTreeE", "llvm::TransformForPostIncUse(llvm::TransformKind, llvm::SCEV const*, llvm::Instruction*, llvm::Value*, llvm::SmallPtrSet<llvm::Loop const*, 2u>&, llvm::ScalarEvolution&, llvm::DominatorTree&)"},
    {"_ZN4llvm36initializeTypeBasedAliasAnalysisPassERNS_12PassRegistryE", "llvm::initializeTypeBasedAliasAnalysisPass(llvm::PassRegistry&)"},
    {"_ZN4llvm32createTypeBasedAliasAnalysisPassEv", "llvm::createTypeBasedAliasAnalysisPass()"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysisD1Ev", "(anonymous namespace)::TypeBasedAliasAnalysis::~TypeBasedAliasAnalysis()"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysisD0Ev", "(anonymous namespace)::TypeBasedAliasAnalysis::~TypeBasedAliasAnalysis()"},
    {"_ZNK12_GLOBAL__N_122TypeBasedAliasAnalysis16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::TypeBasedAliasAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis26getAdjustedAnalysisPointerEPKv", "(anonymous namespace)::TypeBasedAliasAnalysis::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis14initializePassEv", "(anonymous namespace)::TypeBasedAliasAnalysis::initializePass()"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis5aliasERKN4llvm13AliasAnalysis8LocationES5_", "(anonymous namespace)::TypeBasedAliasAnalysis::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "(anonymous namespace)::TypeBasedAliasAnalysis::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "(anonymous namespace)::TypeBasedAliasAnalysis::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis17getModRefBehaviorEPKN4llvm8FunctionE", "(anonymous namespace)::TypeBasedAliasAnalysis::getModRefBehavior(llvm::Function const*)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "(anonymous namespace)::TypeBasedAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "(anonymous namespace)::TypeBasedAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZThn32_NK12_GLOBAL__N_122TypeBasedAliasAnalysis16getAnalysisUsageERN4llvm13AnalysisUsageE", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysisD1Ev", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::~TypeBasedAliasAnalysis()"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysisD0Ev", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::~TypeBasedAliasAnalysis()"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis5aliasERKN4llvm13AliasAnalysis8LocationES5_", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::alias(llvm::AliasAnalysis::Location const&, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis22pointsToConstantMemoryERKN4llvm13AliasAnalysis8LocationEb", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::pointsToConstantMemory(llvm::AliasAnalysis::Location const&, bool)"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis17getModRefBehaviorEN4llvm17ImmutableCallSiteE", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::getModRefBehavior(llvm::ImmutableCallSite)"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis17getModRefBehaviorEPKN4llvm8FunctionE", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::getModRefBehavior(llvm::Function const*)"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteERKNS1_13AliasAnalysis8LocationE", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::AliasAnalysis::Location const&)"},
    {"_ZThn32_N12_GLOBAL__N_122TypeBasedAliasAnalysis13getModRefInfoEN4llvm17ImmutableCallSiteES2_", "non-virtual thunk to (anonymous namespace)::TypeBasedAliasAnalysis::getModRefInfo(llvm::ImmutableCallSite, llvm::ImmutableCallSite)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_122TypeBasedAliasAnalysisEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::TypeBasedAliasAnalysis>()"},
    {"_ZN4llvm17ComputeMaskedBitsEPNS_5ValueERKNS_5APIntERS2_S5_PKNS_10DataLayoutEj", "llvm::ComputeMaskedBits(llvm::Value*, llvm::APInt const&, llvm::APInt&, llvm::APInt&, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm14ComputeSignBitEPNS_5ValueERbS2_PKNS_10DataLayoutEj", "llvm::ComputeSignBit(llvm::Value*, bool&, bool&, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm12isPowerOfTwoEPNS_5ValueEPKNS_10DataLayoutEj", "llvm::isPowerOfTwo(llvm::Value*, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm14isKnownNonZeroEPNS_5ValueEPKNS_10DataLayoutEj", "llvm::isKnownNonZero(llvm::Value*, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm17MaskedValueIsZeroEPNS_5ValueERKNS_5APIntEPKNS_10DataLayoutEj", "llvm::MaskedValueIsZero(llvm::Value*, llvm::APInt const&, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm18ComputeNumSignBitsEPNS_5ValueEPKNS_10DataLayoutEj", "llvm::ComputeNumSignBits(llvm::Value*, llvm::DataLayout const*, unsigned int)"},
    {"_ZN4llvm15ComputeMultipleEPNS_5ValueEjRS1_bj", "llvm::ComputeMultiple(llvm::Value*, unsigned int, llvm::Value*&, bool, unsigned int)"},
    {"_ZN4llvm20CannotBeNegativeZeroEPKNS_5ValueEj", "llvm::CannotBeNegativeZero(llvm::Value const*, unsigned int)"},
    {"_ZN4llvm15isBytewiseValueEPNS_5ValueE", "llvm::isBytewiseValue(llvm::Value*)"},
    {"_ZN4llvm17FindInsertedValueEPNS_5ValueEPKjS3_PNS_11InstructionE", "llvm::FindInsertedValue(llvm::Value*, unsigned int const*, unsigned int const*, llvm::Instruction*)"},
    {"_ZN4llvm32GetPointerBaseWithConstantOffsetEPNS_5ValueERxRKNS_10DataLayoutE", "llvm::GetPointerBaseWithConstantOffset(llvm::Value*, long long&, llvm::DataLayout const&)"},
    {"_ZN4llvm21GetConstantStringInfoEPKNS_5ValueERSsyb", "llvm::GetConstantStringInfo(llvm::Value const*, std::string&, unsigned long long, bool)"},
    {"_ZN4llvm15GetStringLengthEPNS_5ValueE", "llvm::GetStringLength(llvm::Value*)"},
    {"_Z16GetStringLengthHPN4llvm5ValueERNS_11SmallPtrSetIPNS_7PHINodeELj32EEE", "GetStringLengthH(llvm::Value*, llvm::SmallPtrSet<llvm::PHINode*, 32u>&)"},
    {"_ZN4llvm19GetUnderlyingObjectEPNS_5ValueEPKNS_10DataLayoutEj", "llvm::GetUnderlyingObject(llvm::Value*, llvm::DataLayout const*, unsigned int)"},
    {"_Z17BuildSubAggregatePN4llvm5ValueES1_PKNS_4TypeERNS_11SmallVectorIjLj10EEEjPNS_11InstructionE", "BuildSubAggregate(llvm::Value*, llvm::Value*, llvm::Type const*, llvm::SmallVector<unsigned int, 10u>&, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm12PatternMatch11cst_pred_tyINS0_11is_sign_bitEE5matchINS_5ValueEEEbPT_", "bool llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_sign_bit>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm12PatternMatch11cst_pred_tyINS0_11is_sign_bitEE5matchINS_8ConstantEEEbPT_", "bool llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_sign_bit>::match<llvm::Constant>(llvm::Constant*)"},
    {"_ZN4llvm12PatternMatch14BinaryOp_matchINS0_11cst_pred_tyINS0_6is_oneEEENS0_11class_matchINS_5ValueEEELj20EE5matchIS6_EEbPT_", "bool llvm::PatternMatch::BinaryOp_match<llvm::PatternMatch::cst_pred_ty<llvm::PatternMatch::is_one>, llvm::PatternMatch::class_match<llvm::Value>, 20u>::match<llvm::Value>(llvm::Value*)"},
    {"_ZN4llvm7Mangler17getNameWithPrefixERNS_15SmallVectorImplIcEERKNS_5TwineENS0_15ManglerPrefixTyE", "llvm::Mangler::getNameWithPrefix(llvm::SmallVectorImpl<char>&, llvm::Twine const&, llvm::Mangler::ManglerPrefixTy)"},
    {"_ZN4llvm7Mangler17getNameWithPrefixERNS_15SmallVectorImplIcEEPKNS_11GlobalValueEb", "llvm::Mangler::getNameWithPrefix(llvm::SmallVectorImpl<char>&, llvm::GlobalValue const*, bool)"},
    {"_ZN4llvm7Mangler17getNameWithPrefixEPKNS_11GlobalValueEb", "llvm::Mangler::getNameWithPrefix(llvm::GlobalValue const*, bool)"},
    {"_ZN4llvm7Mangler9getSymbolEPKNS_11GlobalValueE", "llvm::Mangler::getSymbol(llvm::GlobalValue const*)"},
    {"_Z12MangleLetterRN4llvm15SmallVectorImplIcEEh", "MangleLetter(llvm::SmallVectorImpl<char>&, unsigned char)"},
    {"_ZN4llvm15SmallVectorImplIcE6insertEPcRKc", "llvm::SmallVectorImpl<char>::insert(char*, char const&)"},
    {"_ZN4llvm8DenseMapIPKNS_11GlobalValueEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::GlobalValue const*, unsigned int, llvm::DenseMapInfo<llvm::GlobalValue const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::GlobalValue const* const&, unsigned int const&, std::pair<llvm::GlobalValue const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_11GlobalValueEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::GlobalValue const*, unsigned int, llvm::DenseMapInfo<llvm::GlobalValue const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm17SubtargetFeatures10AddFeatureERKSsb", "llvm::SubtargetFeatures::AddFeature(std::string const&, bool)"},
    {"_ZN4llvm15LowercaseStringERKSs", "llvm::LowercaseString(std::string const&)"},
    {"_ZN4llvm17SubtargetFeaturesC1ERKSs", "llvm::SubtargetFeatures::SubtargetFeatures(std::string const&)"},
    {"_ZN4llvm17SubtargetFeaturesC2ERKSs", "llvm::SubtargetFeatures::SubtargetFeatures(std::string const&)"},
    {"_Z5SplitRSt6vectorISsSaISsEERKSs", "Split(std::vector<std::string, std::allocator<std::string> >&, std::string const&)"},
    {"_ZNK4llvm17SubtargetFeatures9getStringEv", "llvm::SubtargetFeatures::getString() const"},
    {"_ZN4llvm17SubtargetFeatures9setStringERKSs", "llvm::SubtargetFeatures::setString(std::string const&)"},
    {"_ZN4llvm17SubtargetFeatures6setCPUERKSs", "llvm::SubtargetFeatures::setCPU(std::string const&)"},
    {"_ZN4llvm17SubtargetFeatures12setCPUIfNoneERKSs", "llvm::SubtargetFeatures::setCPUIfNone(std::string const&)"},
    {"_ZNK4llvm17SubtargetFeatures6getCPUEv", "llvm::SubtargetFeatures::getCPU() const"},
    {"_ZN4llvm17SubtargetFeatures7getBitsEPKNS_18SubtargetFeatureKVEmS3_m", "llvm::SubtargetFeatures::getBits(llvm::SubtargetFeatureKV const*, unsigned long, llvm::SubtargetFeatureKV const*, unsigned long)"},
    {"_Z4HelpPKN4llvm18SubtargetFeatureKVEmS2_m", "Help(llvm::SubtargetFeatureKV const*, unsigned long, llvm::SubtargetFeatureKV const*, unsigned long)"},
    {"_Z14SetImpliedBitsRjPKN4llvm18SubtargetFeatureKVES3_m", "SetImpliedBits(unsigned int&, llvm::SubtargetFeatureKV const*, llvm::SubtargetFeatureKV const*, unsigned long)"},
    {"_Z16ClearImpliedBitsRjPKN4llvm18SubtargetFeatureKVES3_m", "ClearImpliedBits(unsigned int&, llvm::SubtargetFeatureKV const*, llvm::SubtargetFeatureKV const*, unsigned long)"},
    {"_ZN4llvm17SubtargetFeatures7getInfoEPKNS_15SubtargetInfoKVEm", "llvm::SubtargetFeatures::getInfo(llvm::SubtargetInfoKV const*, unsigned long)"},
    {"_ZNK4llvm17SubtargetFeatures5printERNS_11raw_ostreamE", "llvm::SubtargetFeatures::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm17SubtargetFeatures4dumpEv", "llvm::SubtargetFeatures::dump() const"},
    {"_ZN4llvm17SubtargetFeatures27getDefaultSubtargetFeaturesERKSsRKNS_6TripleE", "llvm::SubtargetFeatures::getDefaultSubtargetFeatures(std::string const&, llvm::Triple const&)"},
    {"_Z4FindIN4llvm18SubtargetFeatureKVEEPKT_RKSsS4_m", "llvm::SubtargetFeatureKV const* Find<llvm::SubtargetFeatureKV>(std::string const&, llvm::SubtargetFeatureKV const*, unsigned long)"},
    {"_Z4FindIN4llvm15SubtargetInfoKVEEPKT_RKSsS4_m", "llvm::SubtargetInfoKV const* Find<llvm::SubtargetInfoKV>(std::string const&, llvm::SubtargetInfoKV const*, unsigned long)"},
    {"_ZN4llvm13TargetAsmInfoC1ERKNS_13TargetMachineE", "llvm::TargetAsmInfo::TargetAsmInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm13TargetAsmInfoC2ERKNS_13TargetMachineE", "llvm::TargetAsmInfo::TargetAsmInfo(llvm::TargetMachine const&)"},
    {"_ZN4llvm14TargetAsmLexerC1ERKNS_6TargetE", "llvm::TargetAsmLexer::TargetAsmLexer(llvm::Target const&)"},
    {"_ZN4llvm14TargetAsmLexerC2ERKNS_6TargetE", "llvm::TargetAsmLexer::TargetAsmLexer(llvm::Target const&)"},
    {"_ZN4llvm14TargetAsmLexerD0Ev", "llvm::TargetAsmLexer::~TargetAsmLexer()"},
    {"_ZN4llvm14TargetAsmLexerD1Ev", "llvm::TargetAsmLexer::~TargetAsmLexer()"},
    {"_ZN4llvm14TargetAsmLexerD2Ev", "llvm::TargetAsmLexer::~TargetAsmLexer()"},
    {"_ZN4llvm24initializeDataLayoutPassERNS_12PassRegistryE", "llvm::initializeDataLayoutPass(llvm::PassRegistry&)"},
    {"_ZN4llvm12StructLayoutC1EPKNS_10StructTypeERKNS_10DataLayoutE", "llvm::StructLayout::StructLayout(llvm::StructType const*, llvm::DataLayout const&)"},
    {"_ZN4llvm12StructLayoutC2EPKNS_10StructTypeERKNS_10DataLayoutE", "llvm::StructLayout::StructLayout(llvm::StructType const*, llvm::DataLayout const&)"},
    {"_ZNK4llvm10DataLayout19getABITypeAlignmentEPKNS_4TypeE", "llvm::DataLayout::getABITypeAlignment(llvm::Type const*) const"},
    {"_ZNK4llvm12StructLayout26getElementContainingOffsetEy", "llvm::StructLayout::getElementContainingOffset(unsigned long long) const"},
    {"_ZN4llvm15TargetAlignElem3getENS_13AlignTypeEnumEjjj", "llvm::TargetAlignElem::get(llvm::AlignTypeEnum, unsigned int, unsigned int, unsigned int)"},
    {"_ZNK4llvm15TargetAlignElemeqERKS0_", "llvm::TargetAlignElem::operator==(llvm::TargetAlignElem const&) const"},
    {"_ZN4llvm10DataLayout4initENS_9StringRefE", "llvm::DataLayout::init(llvm::StringRef)"},
    {"_ZN4llvm10DataLayout12setAlignmentENS_13AlignTypeEnumEjjj", "llvm::DataLayout::setAlignment(llvm::AlignTypeEnum, unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm10DataLayoutC1Ev", "llvm::DataLayout::DataLayout()"},
    {"_ZN4llvm10DataLayoutC2Ev", "llvm::DataLayout::DataLayout()"},
    {"_ZN4llvm10DataLayoutC1EPKNS_6ModuleE", "llvm::DataLayout::DataLayout(llvm::Module const*)"},
    {"_ZN4llvm10DataLayoutC2EPKNS_6ModuleE", "llvm::DataLayout::DataLayout(llvm::Module const*)"},
    {"_ZNK4llvm10DataLayout16getAlignmentInfoENS_13AlignTypeEnumEjbPKNS_4TypeE", "llvm::DataLayout::getAlignmentInfo(llvm::AlignTypeEnum, unsigned int, bool, llvm::Type const*) const"},
    {"_ZN4llvm10DataLayoutD0Ev", "llvm::DataLayout::~DataLayout()"},
    {"_ZN4llvm10DataLayoutD1Ev", "llvm::DataLayout::~DataLayout()"},
    {"_ZN4llvm10DataLayoutD2Ev", "llvm::DataLayout::~DataLayout()"},
    {"_ZNK4llvm10DataLayout15getStructLayoutEPKNS_10StructTypeE", "llvm::DataLayout::getStructLayout(llvm::StructType const*) const"},
    {"_ZNK4llvm10DataLayout26InvalidateStructLayoutInfoEPKNS_10StructTypeE", "llvm::DataLayout::InvalidateStructLayoutInfo(llvm::StructType const*) const"},
    {"_ZNK4llvm10DataLayout23getStringRepresentationEv", "llvm::DataLayout::getStringRepresentation() const"},
    {"_ZNK4llvm10DataLayout17getTypeSizeInBitsEPKNS_4TypeE", "llvm::DataLayout::getTypeSizeInBits(llvm::Type const*) const"},
    {"_ZNK4llvm10DataLayout12getAlignmentEPKNS_4TypeEb", "llvm::DataLayout::getAlignment(llvm::Type const*, bool) const"},
    {"_ZNK4llvm10DataLayout26getABIIntegerTypeAlignmentEj", "llvm::DataLayout::getABIIntegerTypeAlignment(unsigned int) const"},
    {"_ZNK4llvm10DataLayout25getCallFrameTypeAlignmentEPKNS_4TypeE", "llvm::DataLayout::getCallFrameTypeAlignment(llvm::Type const*) const"},
    {"_ZNK4llvm10DataLayout20getPrefTypeAlignmentEPKNS_4TypeE", "llvm::DataLayout::getPrefTypeAlignment(llvm::Type const*) const"},
    {"_ZNK4llvm10DataLayout30getPreferredTypeAlignmentShiftEPKNS_4TypeE", "llvm::DataLayout::getPreferredTypeAlignmentShift(llvm::Type const*) const"},
    {"_ZNK4llvm10DataLayout13getIntPtrTypeERNS_11LLVMContextE", "llvm::DataLayout::getIntPtrType(llvm::LLVMContext&) const"},
    {"_ZNK4llvm10DataLayout16getIndexedOffsetEPKNS_4TypeEPKPNS_5ValueEj", "llvm::DataLayout::getIndexedOffset(llvm::Type const*, llvm::Value* const*, unsigned int) const"},
    {"_ZNK4llvm10DataLayout21getPreferredAlignmentEPKNS_14GlobalVariableE", "llvm::DataLayout::getPreferredAlignment(llvm::GlobalVariable const*) const"},
    {"_ZNK4llvm10DataLayout24getPreferredAlignmentLogEPKNS_14GlobalVariableE", "llvm::DataLayout::getPreferredAlignmentLog(llvm::GlobalVariable const*) const"},
    {"_ZN12_GLOBAL__N_115StructLayoutMapD1Ev", "(anonymous namespace)::StructLayoutMap::~StructLayoutMap()"},
    {"_ZN12_GLOBAL__N_115StructLayoutMapD0Ev", "(anonymous namespace)::StructLayoutMap::~StructLayoutMap()"},
    {"_ZN12_GLOBAL__N_115StructLayoutMap18refineAbstractTypeEPKN4llvm11DerivedTypeEPKNS1_4TypeE", "(anonymous namespace)::StructLayoutMap::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_115StructLayoutMap18typeBecameConcreteEPKN4llvm11DerivedTypeE", "(anonymous namespace)::StructLayoutMap::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK12_GLOBAL__N_115StructLayoutMap4dumpEv", "(anonymous namespace)::StructLayoutMap::dump() const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_15TargetAlignElemELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::TargetAlignElem, false>::grow(unsigned long)"},
    {"_ZN4llvm12PATypeHolderaSEPKNS_4TypeE", "llvm::PATypeHolder::operator=(llvm::Type const*)"},
    {"_ZN4llvm8DenseMapIPKNS_10StructTypeEPNS_12StructLayoutENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::StructType const*, llvm::StructLayout*, llvm::DenseMapInfo<llvm::StructType const*>, llvm::DenseMapInfo<llvm::StructLayout*> >::InsertIntoBucket(llvm::StructType const* const&, llvm::StructLayout* const&, std::pair<llvm::StructType const*, llvm::StructLayout*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_10StructTypeEPNS_12StructLayoutENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::StructType const*, llvm::StructLayout*, llvm::DenseMapInfo<llvm::StructType const*>, llvm::DenseMapInfo<llvm::StructLayout*> >::grow(unsigned int)"},
    {"_ZN4llvm15callDefaultCtorINS_10DataLayoutEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::DataLayout>()"},
    {"_ZN4llvm19TargetELFWriterInfoC1Ebb", "llvm::TargetELFWriterInfo::TargetELFWriterInfo(bool, bool)"},
    {"_ZN4llvm19TargetELFWriterInfoC2Ebb", "llvm::TargetELFWriterInfo::TargetELFWriterInfo(bool, bool)"},
    {"_ZN4llvm19TargetELFWriterInfoD0Ev", "llvm::TargetELFWriterInfo::~TargetELFWriterInfo()"},
    {"_ZN4llvm19TargetELFWriterInfoD1Ev", "llvm::TargetELFWriterInfo::~TargetELFWriterInfo()"},
    {"_ZN4llvm19TargetELFWriterInfoD2Ev", "llvm::TargetELFWriterInfo::~TargetELFWriterInfo()"},
    {"_ZN4llvm19TargetFrameLoweringD0Ev", "llvm::TargetFrameLowering::~TargetFrameLowering()"},
    {"_ZN4llvm19TargetFrameLoweringD1Ev", "llvm::TargetFrameLowering::~TargetFrameLowering()"},
    {"_ZN4llvm19TargetFrameLoweringD2Ev", "llvm::TargetFrameLowering::~TargetFrameLowering()"},
    {"_ZNK4llvm19TargetFrameLowering20getInitialFrameStateERSt6vectorINS_11MachineMoveESaIS2_EE", "llvm::TargetFrameLowering::getInitialFrameState(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> >&) const"},
    {"_ZNK4llvm19TargetFrameLowering19getFrameIndexOffsetERKNS_15MachineFunctionEi", "llvm::TargetFrameLowering::getFrameIndexOffset(llvm::MachineFunction const&, int) const"},
    {"_ZNK4llvm19TargetFrameLowering22getFrameIndexReferenceERKNS_15MachineFunctionEiRj", "llvm::TargetFrameLowering::getFrameIndexReference(llvm::MachineFunction const&, int, unsigned int&) const"},
    {"_ZNK4llvm17TargetOperandInfo11getRegClassEPKNS_18TargetRegisterInfoE", "llvm::TargetOperandInfo::getRegClass(llvm::TargetRegisterInfo const*) const"},
    {"_ZN4llvm15TargetInstrInfoC1EPKNS_15TargetInstrDescEj", "llvm::TargetInstrInfo::TargetInstrInfo(llvm::TargetInstrDesc const*, unsigned int)"},
    {"_ZN4llvm15TargetInstrInfoC2EPKNS_15TargetInstrDescEj", "llvm::TargetInstrInfo::TargetInstrInfo(llvm::TargetInstrDesc const*, unsigned int)"},
    {"_ZN4llvm15TargetInstrInfoD0Ev", "llvm::TargetInstrInfo::~TargetInstrInfo()"},
    {"_ZN4llvm15TargetInstrInfoD1Ev", "llvm::TargetInstrInfo::~TargetInstrInfo()"},
    {"_ZN4llvm15TargetInstrInfoD2Ev", "llvm::TargetInstrInfo::~TargetInstrInfo()"},
    {"_ZNK4llvm15TargetInstrInfo14getNumMicroOpsEPKNS_18InstrItineraryDataEPKNS_12MachineInstrE", "llvm::TargetInstrInfo::getNumMicroOps(llvm::InstrItineraryData const*, llvm::MachineInstr const*) const"},
    {"_ZNK4llvm15TargetInstrInfo17getOperandLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEjS6_j", "llvm::TargetInstrInfo::getOperandLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNK4llvm15TargetInstrInfo17getOperandLatencyEPKNS_18InstrItineraryDataEPNS_6SDNodeEjS5_j", "llvm::TargetInstrInfo::getOperandLatency(llvm::InstrItineraryData const*, llvm::SDNode*, unsigned int, llvm::SDNode*, unsigned int) const"},
    {"_ZNK4llvm15TargetInstrInfo15getInstrLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEPj", "llvm::TargetInstrInfo::getInstrLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int*) const"},
    {"_ZNK4llvm15TargetInstrInfo15getInstrLatencyEPKNS_18InstrItineraryDataEPNS_6SDNodeE", "llvm::TargetInstrInfo::getInstrLatency(llvm::InstrItineraryData const*, llvm::SDNode*) const"},
    {"_ZNK4llvm15TargetInstrInfo16hasLowDefLatencyEPKNS_18InstrItineraryDataEPKNS_12MachineInstrEj", "llvm::TargetInstrInfo::hasLowDefLatency(llvm::InstrItineraryData const*, llvm::MachineInstr const*, unsigned int) const"},
    {"_ZNK4llvm15TargetInstrInfo10insertNoopERNS_17MachineBasicBlockENS_14ilist_iteratorINS_12MachineInstrEEE", "llvm::TargetInstrInfo::insertNoop(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>) const"},
    {"_ZNK4llvm15TargetInstrInfo24isUnpredicatedTerminatorEPKNS_12MachineInstrE", "llvm::TargetInstrInfo::isUnpredicatedTerminator(llvm::MachineInstr const*) const"},
    {"_ZNK4llvm15TargetInstrInfo18getInlineAsmLengthEPKcRKNS_9MCAsmInfoE", "llvm::TargetInstrInfo::getInlineAsmLength(char const*, llvm::MCAsmInfo const&) const"},
    {"_ZN4llvm24TargetLoweringObjectFileC1Ev", "llvm::TargetLoweringObjectFile::TargetLoweringObjectFile()"},
    {"_ZN4llvm24TargetLoweringObjectFileC2Ev", "llvm::TargetLoweringObjectFile::TargetLoweringObjectFile()"},
    {"_ZN4llvm24TargetLoweringObjectFileD0Ev", "llvm::TargetLoweringObjectFile::~TargetLoweringObjectFile()"},
    {"_ZN4llvm24TargetLoweringObjectFileD1Ev", "llvm::TargetLoweringObjectFile::~TargetLoweringObjectFile()"},
    {"_ZN4llvm24TargetLoweringObjectFileD2Ev", "llvm::TargetLoweringObjectFile::~TargetLoweringObjectFile()"},
    {"_ZN4llvm24TargetLoweringObjectFile16getKindForGlobalEPKNS_11GlobalValueERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFile::getKindForGlobal(llvm::GlobalValue const*, llvm::TargetMachine const&)"},
    {"_ZNK4llvm24TargetLoweringObjectFile16SectionForGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFile::SectionForGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile22SelectSectionForGlobalEPKNS_11GlobalValueENS_11SectionKindEPNS_7ManglerERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFile::SelectSectionForGlobal(llvm::GlobalValue const*, llvm::SectionKind, llvm::Mangler*, llvm::TargetMachine const&) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile21getSectionForConstantENS_11SectionKindE", "llvm::TargetLoweringObjectFile::getSectionForConstant(llvm::SectionKind) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile30getExprForDwarfGlobalReferenceEPKNS_11GlobalValueEPNS_7ManglerEPNS_17MachineModuleInfoEjRNS_10MCStreamerE", "llvm::TargetLoweringObjectFile::getExprForDwarfGlobalReference(llvm::GlobalValue const*, llvm::Mangler*, llvm::MachineModuleInfo*, unsigned int, llvm::MCStreamer&) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile24getExprForDwarfReferenceEPKNS_8MCSymbolEPNS_7ManglerEPNS_17MachineModuleInfoEjRNS_10MCStreamerE", "llvm::TargetLoweringObjectFile::getExprForDwarfReference(llvm::MCSymbol const*, llvm::Mangler*, llvm::MachineModuleInfo*, unsigned int, llvm::MCStreamer&) const"},
    {"_ZNK4llvm24TargetLoweringObjectFile22getPersonalityEncodingEv", "llvm::TargetLoweringObjectFile::getPersonalityEncoding() const"},
    {"_ZNK4llvm24TargetLoweringObjectFile15getLSDAEncodingEv", "llvm::TargetLoweringObjectFile::getLSDAEncoding() const"},
    {"_ZNK4llvm24TargetLoweringObjectFile14getFDEEncodingEv", "llvm::TargetLoweringObjectFile::getFDEEncoding() const"},
    {"_ZNK4llvm24TargetLoweringObjectFile16getTTypeEncodingEv", "llvm::TargetLoweringObjectFile::getTTypeEncoding() const"},
    {"_ZN4llvm24TargetLoweringObjectFile10InitializeERNS_9MCContextERKNS_13TargetMachineE", "llvm::TargetLoweringObjectFile::Initialize(llvm::MCContext&, llvm::TargetMachine const&)"},
    {"_ZN4llvm13TargetMachineC1ERKNS_6TargetE", "llvm::TargetMachine::TargetMachine(llvm::Target const&)"},
    {"_ZN4llvm13TargetMachineC2ERKNS_6TargetE", "llvm::TargetMachine::TargetMachine(llvm::Target const&)"},
    {"_ZN4llvm13TargetMachineD0Ev", "llvm::TargetMachine::~TargetMachine()"},
    {"_ZN4llvm13TargetMachineD1Ev", "llvm::TargetMachine::~TargetMachine()"},
    {"_ZN4llvm13TargetMachineD2Ev", "llvm::TargetMachine::~TargetMachine()"},
    {"_ZN4llvm13TargetMachine18getRelocationModelEv", "llvm::TargetMachine::getRelocationModel()"},
    {"_ZN4llvm13TargetMachine18setRelocationModelENS_5Reloc5ModelE", "llvm::TargetMachine::setRelocationModel(llvm::Reloc::Model)"},
    {"_ZN4llvm13TargetMachine12getCodeModelEv", "llvm::TargetMachine::getCodeModel()"},
    {"_ZN4llvm13TargetMachine12setCodeModelENS_9CodeModel5ModelE", "llvm::TargetMachine::setCodeModel(llvm::CodeModel::Model)"},
    {"_ZN4llvm13TargetMachine22getAsmVerbosityDefaultEv", "llvm::TargetMachine::getAsmVerbosityDefault()"},
    {"_ZN4llvm13TargetMachine22setAsmVerbosityDefaultEb", "llvm::TargetMachine::setAsmVerbosityDefault(bool)"},
    {"_ZN4llvm13TargetMachine19getFunctionSectionsEv", "llvm::TargetMachine::getFunctionSections()"},
    {"_ZN4llvm13TargetMachine15getDataSectionsEv", "llvm::TargetMachine::getDataSections()"},
    {"_ZN4llvm13TargetMachine19setFunctionSectionsEb", "llvm::TargetMachine::setFunctionSections(bool)"},
    {"_ZN4llvm13TargetMachine15setDataSectionsEb", "llvm::TargetMachine::setDataSections(bool)"},
    {"_ZN4llvm23DisableFramePointerElimERKNS_15MachineFunctionE", "llvm::DisableFramePointerElim(llvm::MachineFunction const&)"},
    {"_ZN4llvm16LessPreciseFPMADEv", "llvm::LessPreciseFPMAD()"},
    {"_ZN4llvm32HonorSignDependentRoundingFPMathEv", "llvm::HonorSignDependentRoundingFPMath()"},
    {"_ZN4llvm2cl3optIbLb1ENS0_6parserIbEEED1Ev", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEED1Ev", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::~opt()"},
    {"_ZN4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEED1Ev", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::~opt()"},
    {"_ZN4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEED1Ev", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::~opt()"},
    {"_ZN4llvm2cl3optIjLb1ENS0_6parserIjEEED1Ev", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::~opt()"},
    {"_ZNK4llvm13TargetMachine25getEnableTailMergeDefaultEv", "llvm::TargetMachine::getEnableTailMergeDefault() const"},
    {"_ZN4llvm13TargetMachine19addPassesToEmitFileERNS_15PassManagerBaseERNS_21formatted_raw_ostreamENS0_15CodeGenFileTypeENS_10CodeGenOpt5LevelEb", "llvm::TargetMachine::addPassesToEmitFile(llvm::PassManagerBase&, llvm::formatted_raw_ostream&, llvm::TargetMachine::CodeGenFileType, llvm::CodeGenOpt::Level, bool)"},
    {"_ZN4llvm13TargetMachine26addPassesToEmitMachineCodeERNS_15PassManagerBaseERNS_14JITCodeEmitterENS_10CodeGenOpt5LevelEb", "llvm::TargetMachine::addPassesToEmitMachineCode(llvm::PassManagerBase&, llvm::JITCodeEmitter&, llvm::CodeGenOpt::Level, bool)"},
    {"_ZN4llvm13TargetMachine17addPassesToEmitMCERNS_15PassManagerBaseERPNS_9MCContextENS_10CodeGenOpt5LevelEb", "llvm::TargetMachine::addPassesToEmitMC(llvm::PassManagerBase&, llvm::MCContext*&, llvm::CodeGenOpt::Level, bool)"},
    {"_ZN4llvm2cl3optIbLb1ENS0_6parserIbEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIbLb1ENS0_6parserIbEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIbLb1ENS0_6parserIbEEE14getOptionWidthEv", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIbLb1ENS0_6parserIbEEE15printOptionInfoEm", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIbLb1ENS0_6parserIbEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIbLb1ENS0_6parserIbEEED0Ev", "llvm::cl::opt<bool, true, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl3optIjLb1ENS0_6parserIjEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIjLb1ENS0_6parserIjEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIjLb1ENS0_6parserIjEEE14getOptionWidthEv", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIjLb1ENS0_6parserIjEEE15printOptionInfoEm", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIjLb1ENS0_6parserIjEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIjLb1ENS0_6parserIjEEED0Ev", "llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >::~opt()"},
    {"_ZN4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEE14getOptionWidthEv", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEE15printOptionInfoEm", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEED0Ev", "llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >::~opt()"},
    {"_ZN4llvm2cl19generic_parser_base19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::generic_parser_base::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl6parserINS_9CodeModel5ModelEE5parseERNS0_6OptionENS_9StringRefES7_RS3_", "llvm::cl::parser<llvm::CodeModel::Model>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::CodeModel::Model&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optINS_9CodeModel5ModelELb1ENS0_6parserIS6_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> > >(llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserINS_9CodeModel5ModelEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::CodeModel::Model>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserINS_9CodeModel5ModelEED1Ev", "llvm::cl::parser<llvm::CodeModel::Model>::~parser()"},
    {"_ZN4llvm2cl6parserINS_9CodeModel5ModelEED0Ev", "llvm::cl::parser<llvm::CodeModel::Model>::~parser()"},
    {"_ZNK4llvm2cl6parserINS_9CodeModel5ModelEE13getNumOptionsEv", "llvm::cl::parser<llvm::CodeModel::Model>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserINS_9CodeModel5ModelEE9getOptionEj", "llvm::cl::parser<llvm::CodeModel::Model>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserINS_9CodeModel5ModelEE14getDescriptionEj", "llvm::cl::parser<llvm::CodeModel::Model>::getDescription(unsigned int) const"},
    {"_ZN4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEE14getOptionWidthEv", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEE15printOptionInfoEm", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEED0Ev", "llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >::~opt()"},
    {"_ZN4llvm2cl6parserINS_5Reloc5ModelEE5parseERNS0_6OptionENS_9StringRefES7_RS3_", "llvm::cl::parser<llvm::Reloc::Model>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::Reloc::Model&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optINS_5Reloc5ModelELb1ENS0_6parserIS6_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> > >(llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserINS_5Reloc5ModelEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::Reloc::Model>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserINS_5Reloc5ModelEED1Ev", "llvm::cl::parser<llvm::Reloc::Model>::~parser()"},
    {"_ZN4llvm2cl6parserINS_5Reloc5ModelEED0Ev", "llvm::cl::parser<llvm::Reloc::Model>::~parser()"},
    {"_ZNK4llvm2cl6parserINS_5Reloc5ModelEE13getNumOptionsEv", "llvm::cl::parser<llvm::Reloc::Model>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserINS_5Reloc5ModelEE9getOptionEj", "llvm::cl::parser<llvm::Reloc::Model>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserINS_5Reloc5ModelEE14getDescriptionEj", "llvm::cl::parser<llvm::Reloc::Model>::getDescription(unsigned int) const"},
    {"_ZN4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEE14getOptionWidthEv", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEE15printOptionInfoEm", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEED0Ev", "llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >::~opt()"},
    {"_ZN4llvm2cl6parserINS_8FloatABI7ABITypeEE5parseERNS0_6OptionENS_9StringRefES7_RS3_", "llvm::cl::parser<llvm::FloatABI::ABIType>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::FloatABI::ABIType&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS6_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> > >(llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserINS_8FloatABI7ABITypeEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::FloatABI::ABIType>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserINS_8FloatABI7ABITypeEED1Ev", "llvm::cl::parser<llvm::FloatABI::ABIType>::~parser()"},
    {"_ZN4llvm2cl6parserINS_8FloatABI7ABITypeEED0Ev", "llvm::cl::parser<llvm::FloatABI::ABIType>::~parser()"},
    {"_ZNK4llvm2cl6parserINS_8FloatABI7ABITypeEE13getNumOptionsEv", "llvm::cl::parser<llvm::FloatABI::ABIType>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserINS_8FloatABI7ABITypeEE9getOptionEj", "llvm::cl::parser<llvm::FloatABI::ABIType>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserINS_8FloatABI7ABITypeEE14getDescriptionEj", "llvm::cl::parser<llvm::FloatABI::ABIType>::getDescription(unsigned int) const"},
    {"_ZN4llvm18TargetRegisterInfoC1EPKNS_18TargetRegisterDescEjPKPKNS_19TargetRegisterClassES8_PKPKciiPKjjSE_j", "llvm::TargetRegisterInfo::TargetRegisterInfo(llvm::TargetRegisterDesc const*, unsigned int, llvm::TargetRegisterClass const* const*, llvm::TargetRegisterClass const* const*, char const* const*, int, int, unsigned int const*, unsigned int, unsigned int const*, unsigned int)"},
    {"_ZN4llvm18TargetRegisterInfoC2EPKNS_18TargetRegisterDescEjPKPKNS_19TargetRegisterClassES8_PKPKciiPKjjSE_j", "llvm::TargetRegisterInfo::TargetRegisterInfo(llvm::TargetRegisterDesc const*, unsigned int, llvm::TargetRegisterClass const* const*, llvm::TargetRegisterClass const* const*, char const* const*, int, int, unsigned int const*, unsigned int, unsigned int const*, unsigned int)"},
    {"_ZN4llvm18TargetRegisterInfoD0Ev", "llvm::TargetRegisterInfo::~TargetRegisterInfo()"},
    {"_ZN4llvm18TargetRegisterInfoD1Ev", "llvm::TargetRegisterInfo::~TargetRegisterInfo()"},
    {"_ZN4llvm18TargetRegisterInfoD2Ev", "llvm::TargetRegisterInfo::~TargetRegisterInfo()"},
    {"_ZNK4llvm8PrintReg5printERNS_11raw_ostreamE", "llvm::PrintReg::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm18TargetRegisterInfo22getMinimalPhysRegClassEjNS_3EVTE", "llvm::TargetRegisterInfo::getMinimalPhysRegClass(unsigned int, llvm::EVT) const"},
    {"_ZNK4llvm18TargetRegisterInfo17getAllocatableSetERKNS_15MachineFunctionEPKNS_19TargetRegisterClassE", "llvm::TargetRegisterInfo::getAllocatableSet(llvm::MachineFunction const&, llvm::TargetRegisterClass const*) const"},
    {"_ZN4llvm17getCommonSubClassEPKNS_19TargetRegisterClassES2_", "llvm::getCommonSubClass(llvm::TargetRegisterClass const*, llvm::TargetRegisterClass const*)"},
    {"_ZNK4llvm18TargetRegisterInfo20composeSubRegIndicesEjj", "llvm::TargetRegisterInfo::composeSubRegIndices(unsigned int, unsigned int) const"},
    {"_ZNK4llvm18TargetRegisterInfo21needsStackRealignmentERKNS_15MachineFunctionE", "llvm::TargetRegisterInfo::needsStackRealignment(llvm::MachineFunction const&) const"},
    {"_ZN4llvm15TargetSubtargetC1Ev", "llvm::TargetSubtarget::TargetSubtarget()"},
    {"_ZN4llvm15TargetSubtargetC2Ev", "llvm::TargetSubtarget::TargetSubtarget()"},
    {"_ZN4llvm15TargetSubtargetD0Ev", "llvm::TargetSubtarget::~TargetSubtarget()"},
    {"_ZN4llvm15TargetSubtargetD1Ev", "llvm::TargetSubtarget::~TargetSubtarget()"},
    {"_ZN4llvm15TargetSubtargetD2Ev", "llvm::TargetSubtarget::~TargetSubtarget()"},
    {"_ZNK4llvm15TargetSubtarget21enablePostRASchedulerENS_10CodeGenOpt5LevelERNS0_16AntiDepBreakModeERNS_15SmallVectorImplIPNS_19TargetRegisterClassEEE", "llvm::TargetSubtarget::enablePostRAScheduler(llvm::CodeGenOpt::Level, llvm::TargetSubtarget::AntiDepBreakMode&, llvm::SmallVectorImpl<llvm::TargetRegisterClass*>&) const"},
    {"_ZN4llvm24AssemblyAnnotationWriterD0Ev", "llvm::AssemblyAnnotationWriter::~AssemblyAnnotationWriter()"},
    {"_ZN4llvm24AssemblyAnnotationWriterD1Ev", "llvm::AssemblyAnnotationWriter::~AssemblyAnnotationWriter()"},
    {"_ZN4llvm24AssemblyAnnotationWriterD2Ev", "llvm::AssemblyAnnotationWriter::~AssemblyAnnotationWriter()"},
    {"_ZN4llvm12TypePrinting5clearEv", "llvm::TypePrinting::clear()"},
    {"_ZNK4llvm12TypePrinting11hasTypeNameEPKNS_4TypeE", "llvm::TypePrinting::hasTypeName(llvm::Type const*) const"},
    {"_ZN4llvm12TypePrinting11addTypeNameEPKNS_4TypeERKSs", "llvm::TypePrinting::addTypeName(llvm::Type const*, std::string const&)"},
    {"_ZN4llvm12TypePrintingC1Ev", "llvm::TypePrinting::TypePrinting()"},
    {"_ZN4llvm12TypePrintingC2Ev", "llvm::TypePrinting::TypePrinting()"},
    {"_ZN4llvm12TypePrintingD1Ev", "llvm::TypePrinting::~TypePrinting()"},
    {"_ZN4llvm12TypePrintingD2Ev", "llvm::TypePrinting::~TypePrinting()"},
    {"_ZN4llvm12TypePrinting12CalcTypeNameEPKNS_4TypeERNS_15SmallVectorImplIS3_EERNS_11raw_ostreamEb", "llvm::TypePrinting::CalcTypeName(llvm::Type const*, llvm::SmallVectorImpl<llvm::Type const*>&, llvm::raw_ostream&, bool)"},
    {"_ZN4llvm12TypePrinting5printEPKNS_4TypeERNS_11raw_ostreamEb", "llvm::TypePrinting::print(llvm::Type const*, llvm::raw_ostream&, bool)"},
    {"_ZN4llvm17WriteTypeSymbolicERNS_11raw_ostreamEPKNS_4TypeEPKNS_6ModuleE", "llvm::WriteTypeSymbolic(llvm::raw_ostream&, llvm::Type const*, llvm::Module const*)"},
    {"_Z23AddModuleTypesToPrinterRN4llvm12TypePrintingERSt6vectorIPKNS_4TypeESaIS5_EEPKNS_6ModuleE", "AddModuleTypesToPrinter(llvm::TypePrinting&, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> >&, llvm::Module const*)"},
    {"_ZN4llvm14WriteAsOperandERNS_11raw_ostreamEPKNS_5ValueEbPKNS_6ModuleE", "llvm::WriteAsOperand(llvm::raw_ostream&, llvm::Value const*, bool, llvm::Module const*)"},
    {"_Z22WriteAsOperandInternalRN4llvm11raw_ostreamEPKNS_5ValueEPNS_12TypePrintingEPN12_GLOBAL__N_111SlotTrackerEPKNS_6ModuleE", "WriteAsOperandInternal(llvm::raw_ostream&, llvm::Value const*, llvm::TypePrinting*, (anonymous namespace)::SlotTracker*, llvm::Module const*)"},
    {"_ZNK4llvm6Module5printERNS_11raw_ostreamEPNS_24AssemblyAnnotationWriterE", "llvm::Module::print(llvm::raw_ostream&, llvm::AssemblyAnnotationWriter*) const"},
    {"_ZN12_GLOBAL__N_111SlotTrackerC1EPKN4llvm6ModuleE", "(anonymous namespace)::SlotTracker::SlotTracker(llvm::Module const*)"},
    {"_ZNK4llvm11NamedMDNode5printERNS_11raw_ostreamEPNS_24AssemblyAnnotationWriterE", "llvm::NamedMDNode::print(llvm::raw_ostream&, llvm::AssemblyAnnotationWriter*) const"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter16printNamedMDNodeEPKN4llvm11NamedMDNodeE", "(anonymous namespace)::AssemblyWriter::printNamedMDNode(llvm::NamedMDNode const*)"},
    {"_ZNK4llvm4Type5printERNS_11raw_ostreamE", "llvm::Type::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm5Value5printERNS_11raw_ostreamEPNS_24AssemblyAnnotationWriterE", "llvm::Value::print(llvm::raw_ostream&, llvm::AssemblyAnnotationWriter*) const"},
    {"_ZN12_GLOBAL__N_111SlotTrackerC1EPKN4llvm8FunctionE", "(anonymous namespace)::SlotTracker::SlotTracker(llvm::Function const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter16printInstructionERKN4llvm11InstructionE", "(anonymous namespace)::AssemblyWriter::printInstruction(llvm::Instruction const&)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter15printBasicBlockEPKN4llvm10BasicBlockE", "(anonymous namespace)::AssemblyWriter::printBasicBlock(llvm::BasicBlock const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter11printGlobalEPKN4llvm14GlobalVariableE", "(anonymous namespace)::AssemblyWriter::printGlobal(llvm::GlobalVariable const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter13printFunctionEPKN4llvm8FunctionE", "(anonymous namespace)::AssemblyWriter::printFunction(llvm::Function const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter10printAliasEPKN4llvm11GlobalAliasE", "(anonymous namespace)::AssemblyWriter::printAlias(llvm::GlobalAlias const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter15printMDNodeBodyEPKN4llvm6MDNodeE", "(anonymous namespace)::AssemblyWriter::printMDNodeBody(llvm::MDNode const*)"},
    {"_Z21WriteConstantInternalRN4llvm11raw_ostreamEPKNS_8ConstantERNS_12TypePrintingEPN12_GLOBAL__N_111SlotTrackerEPKNS_6ModuleE", "WriteConstantInternal(llvm::raw_ostream&, llvm::Constant const*, llvm::TypePrinting&, (anonymous namespace)::SlotTracker*, llvm::Module const*)"},
    {"_ZNK4llvm5Value11printCustomERNS_11raw_ostreamE", "llvm::Value::printCustom(llvm::raw_ostream&) const"},
    {"_ZNK4llvm5Value4dumpEv", "llvm::Value::dump() const"},
    {"_ZNK4llvm4Type4dumpEPKNS_6ModuleE", "llvm::Type::dump(llvm::Module const*) const"},
    {"_ZNK4llvm4Type4dumpEv", "llvm::Type::dump() const"},
    {"_ZNK4llvm6Module4dumpEv", "llvm::Module::dump() const"},
    {"_Z18PrintEscapedStringN4llvm9StringRefERNS_11raw_ostreamE", "PrintEscapedString(llvm::StringRef, llvm::raw_ostream&)"},
    {"_Z21WriteOptimizationInfoRN4llvm11raw_ostreamEPKNS_4UserE", "WriteOptimizationInfo(llvm::raw_ostream&, llvm::User const*)"},
    {"_Z23WriteMDNodeBodyInternalRN4llvm11raw_ostreamEPKNS_6MDNodeEPNS_12TypePrintingEPN12_GLOBAL__N_111SlotTrackerEPKNS_6ModuleE", "WriteMDNodeBodyInternal(llvm::raw_ostream&, llvm::MDNode const*, llvm::TypePrinting*, (anonymous namespace)::SlotTracker*, llvm::Module const*)"},
    {"_Z12PrintLinkageN4llvm11GlobalValue12LinkageTypesERNS_21formatted_raw_ostreamE", "PrintLinkage(llvm::GlobalValue::LinkageTypes, llvm::formatted_raw_ostream&)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter12writeOperandEPKN4llvm5ValueEb", "(anonymous namespace)::AssemblyWriter::writeOperand(llvm::Value const*, bool)"},
    {"_Z13PrintLLVMNameRN4llvm11raw_ostreamENS_9StringRefE10PrefixType", "PrintLLVMName(llvm::raw_ostream&, llvm::StringRef, PrefixType)"},
    {"_ZN12_GLOBAL__N_111SlotTracker10initializeEv", "(anonymous namespace)::SlotTracker::initialize()"},
    {"_ZN12_GLOBAL__N_111SlotTracker18CreateMetadataSlotEPKN4llvm6MDNodeE", "(anonymous namespace)::SlotTracker::CreateMetadataSlot(llvm::MDNode const*)"},
    {"_ZN12_GLOBAL__N_114AssemblyWriter17writeParamOperandEPKN4llvm5ValueEj", "(anonymous namespace)::AssemblyWriter::writeParamOperand(llvm::Value const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_110TypeFinder15IncorporateTypeEPKN4llvm4TypeE", "(anonymous namespace)::TypeFinder::IncorporateType(llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_110TypeFinder16IncorporateValueEPKN4llvm5ValueE", "(anonymous namespace)::TypeFinder::IncorporateValue(llvm::Value const*)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeESsNS_12DenseMapInfoIS3_EENS4_ISsEEE5clearEv", "llvm::DenseMap<llvm::Type const*, std::string, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<std::string> >::clear()"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeESsNS_12DenseMapInfoIS3_EENS4_ISsEEE6insertERKSt4pairIS3_SsE", "llvm::DenseMap<llvm::Type const*, std::string, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<std::string> >::insert(std::pair<llvm::Type const*, std::string> const&)"},
    {"_ZN4llvm24AssemblyAnnotationWriter17emitFunctionAnnotEPKNS_8FunctionERNS_21formatted_raw_ostreamE", "llvm::AssemblyAnnotationWriter::emitFunctionAnnot(llvm::Function const*, llvm::formatted_raw_ostream&)"},
    {"_ZN4llvm24AssemblyAnnotationWriter24emitBasicBlockStartAnnotEPKNS_10BasicBlockERNS_21formatted_raw_ostreamE", "llvm::AssemblyAnnotationWriter::emitBasicBlockStartAnnot(llvm::BasicBlock const*, llvm::formatted_raw_ostream&)"},
    {"_ZN4llvm24AssemblyAnnotationWriter22emitBasicBlockEndAnnotEPKNS_10BasicBlockERNS_21formatted_raw_ostreamE", "llvm::AssemblyAnnotationWriter::emitBasicBlockEndAnnot(llvm::BasicBlock const*, llvm::formatted_raw_ostream&)"},
    {"_ZN4llvm24AssemblyAnnotationWriter20emitInstructionAnnotEPKNS_11InstructionERNS_21formatted_raw_ostreamE", "llvm::AssemblyAnnotationWriter::emitInstructionAnnot(llvm::Instruction const*, llvm::formatted_raw_ostream&)"},
    {"_ZN4llvm24AssemblyAnnotationWriter16printInfoCommentERKNS_5ValueERNS_21formatted_raw_ostreamE", "llvm::AssemblyAnnotationWriter::printInfoComment(llvm::Value const&, llvm::formatted_raw_ostream&)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16InsertIntoBucketERKS3_RKSsPSt4pairIS3_SsE", "llvm::DenseMap<llvm::Type const*, std::string, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<std::string> >::InsertIntoBucket(llvm::Type const* const&, std::string const&, std::pair<llvm::Type const*, std::string>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeESsNS_12DenseMapInfoIS3_EENS4_ISsEEE4growEj", "llvm::DenseMap<llvm::Type const*, std::string, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<std::string> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeESsNS_12DenseMapInfoIS3_EENS4_ISsEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Type const*, std::string, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<std::string> >::shrink_and_clear()"},
    {"_ZNK4llvm5APInt3ultEy", "llvm::APInt::ult(unsigned long long) const"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEjNS_12DenseMapInfoIS3_EENS4_IjEEE16shrink_and_clearEv", "llvm::DenseMap<llvm::Value const*, unsigned int, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<unsigned int> >::shrink_and_clear()"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::MDNode const*, unsigned int, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MDNode const* const&, unsigned int const&, std::pair<llvm::MDNode const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_6MDNodeEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::MDNode const*, unsigned int, llvm::DenseMapInfo<llvm::MDNode const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplIPKNS_6MDNodeEE6resizeEj", "llvm::SmallVectorImpl<llvm::MDNode const*>::resize(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<llvm::Value const*, char, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::Value const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<llvm::Value const*, char, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::Value const* const&, char const&, std::pair<llvm::Value const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKNS_5ValueEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<llvm::Value const*, char, llvm::DenseMapInfo<llvm::Value const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeEcNS_12DenseMapInfoIS3_EENS4_IcEEE6insertERKSt4pairIS3_cE", "llvm::DenseMap<llvm::Type const*, char, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<char> >::insert(std::pair<llvm::Type const*, char> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeEcNS_12DenseMapInfoIS3_EENS4_IcEEE16InsertIntoBucketERKS3_RKcPSt4pairIS3_cE", "llvm::DenseMap<llvm::Type const*, char, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<char> >::InsertIntoBucket(llvm::Type const* const&, char const&, std::pair<llvm::Type const*, char>*)"},
    {"_ZN4llvm8DenseMapIPKNS_4TypeEcNS_12DenseMapInfoIS3_EENS4_IcEEE4growEj", "llvm::DenseMap<llvm::Type const*, char, llvm::DenseMapInfo<llvm::Type const*>, llvm::DenseMapInfo<char> >::grow(unsigned int)"},
    {"_ZN4llvm9Attribute11getAsStringEj", "llvm::Attribute::getAsString(unsigned int)"},
    {"_ZN4llvm9Attribute16typeIncompatibleEPKNS_4TypeE", "llvm::Attribute::typeIncompatible(llvm::Type const*)"},
    {"_ZN4llvm17AttributeListImplD1Ev", "llvm::AttributeListImpl::~AttributeListImpl()"},
    {"_ZN4llvm17AttributeListImplD2Ev", "llvm::AttributeListImpl::~AttributeListImpl()"},
    {"_ZN4llvm11AttrListPtr3getEPKNS_18AttributeWithIndexEj", "llvm::AttrListPtr::get(llvm::AttributeWithIndex const*, unsigned int)"},
    {"_ZN4llvm11AttrListPtrC1EPNS_17AttributeListImplE", "llvm::AttrListPtr::AttrListPtr(llvm::AttributeListImpl*)"},
    {"_ZN4llvm11AttrListPtrC2EPNS_17AttributeListImplE", "llvm::AttrListPtr::AttrListPtr(llvm::AttributeListImpl*)"},
    {"_ZN4llvm11AttrListPtrC1ERKS0_", "llvm::AttrListPtr::AttrListPtr(llvm::AttrListPtr const&)"},
    {"_ZN4llvm11AttrListPtrC2ERKS0_", "llvm::AttrListPtr::AttrListPtr(llvm::AttrListPtr const&)"},
    {"_ZN4llvm11AttrListPtraSERKS0_", "llvm::AttrListPtr::operator=(llvm::AttrListPtr const&)"},
    {"_ZN4llvm11AttrListPtrD1Ev", "llvm::AttrListPtr::~AttrListPtr()"},
    {"_ZN4llvm11AttrListPtrD2Ev", "llvm::AttrListPtr::~AttrListPtr()"},
    {"_ZNK4llvm11AttrListPtr11getNumSlotsEv", "llvm::AttrListPtr::getNumSlots() const"},
    {"_ZNK4llvm11AttrListPtr7getSlotEj", "llvm::AttrListPtr::getSlot(unsigned int) const"},
    {"_ZNK4llvm11AttrListPtr13getAttributesEj", "llvm::AttrListPtr::getAttributes(unsigned int) const"},
    {"_ZNK4llvm11AttrListPtr16hasAttrSomewhereEj", "llvm::AttrListPtr::hasAttrSomewhere(unsigned int) const"},
    {"_ZNK4llvm11AttrListPtr7addAttrEjj", "llvm::AttrListPtr::addAttr(unsigned int, unsigned int) const"},
    {"_ZNK4llvm11AttrListPtr10removeAttrEjj", "llvm::AttrListPtr::removeAttr(unsigned int, unsigned int) const"},
    {"_ZNK4llvm11AttrListPtr4dumpEv", "llvm::AttrListPtr::dump() const"},
    {"_ZN4llvm17AttributeListImpl6AddRefEv", "llvm::AttributeListImpl::AddRef()"},
    {"_ZN4llvm17AttributeListImpl7DropRefEv", "llvm::AttributeListImpl::DropRef()"},
    {"_ZN4llvm15SmallVectorImplINS_18AttributeWithIndexEE6insertIPKS1_EEPS1_S6_T_S7_", "llvm::AttributeWithIndex* llvm::SmallVectorImpl<llvm::AttributeWithIndex>::insert<llvm::AttributeWithIndex const*>(llvm::AttributeWithIndex*, llvm::AttributeWithIndex const*, llvm::AttributeWithIndex const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_18AttributeWithIndexELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::AttributeWithIndex, false>::grow(unsigned long)"},
    {"_ZN4llvm14object_creatorINS_10FoldingSetINS_17AttributeListImplEEEEEPvv", "void* llvm::object_creator<llvm::FoldingSet<llvm::AttributeListImpl> >()"},
    {"_ZN4llvm14object_deleterINS_10FoldingSetINS_17AttributeListImplEEEE4callEPv", "llvm::object_deleter<llvm::FoldingSet<llvm::AttributeListImpl> >::call(void*)"},
    {"_ZN4llvm10FoldingSetINS_17AttributeListImplEED1Ev", "llvm::FoldingSet<llvm::AttributeListImpl>::~FoldingSet()"},
    {"_ZN4llvm10FoldingSetINS_17AttributeListImplEED0Ev", "llvm::FoldingSet<llvm::AttributeListImpl>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_17AttributeListImplEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::AttributeListImpl>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17AttributeListImplEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS6_", "llvm::FoldingSet<llvm::AttributeListImpl>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_17AttributeListImplEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::AttributeListImpl>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm24UpgradeIntrinsicFunctionEPNS_8FunctionERS1_", "llvm::UpgradeIntrinsicFunction(llvm::Function*, llvm::Function*&)"},
    {"_Z25UpgradeIntrinsicFunction1PN4llvm8FunctionERS1_", "UpgradeIntrinsicFunction1(llvm::Function*, llvm::Function*&)"},
    {"_ZN4llvm21UpgradeGlobalVariableEPNS_14GlobalVariableE", "llvm::UpgradeGlobalVariable(llvm::GlobalVariable*)"},
    {"_ZN4llvm20UpgradeIntrinsicCallEPNS_8CallInstEPNS_8FunctionE", "llvm::UpgradeIntrinsicCall(llvm::CallInst*, llvm::Function*)"},
    {"_Z14ExtendNEONArgsPN4llvm8CallInstEPNS_5ValueES3_RS3_S4_", "ExtendNEONArgs(llvm::CallInst*, llvm::Value*, llvm::Value*, llvm::Value*&, llvm::Value*&)"},
    {"_Z8CallVABDPN4llvm8CallInstEPNS_5ValueES3_", "CallVABD(llvm::CallInst*, llvm::Value*, llvm::Value*)"},
    {"_Z20ConstructNewCallInstPN4llvm8FunctionEPNS_8CallInstEPPNS_5ValueEjb", "ConstructNewCallInst(llvm::Function*, llvm::CallInst*, llvm::Value**, unsigned int, bool)"},
    {"_ZN4llvm23UpgradeCallsToIntrinsicEPNS_8FunctionE", "llvm::UpgradeCallsToIntrinsic(llvm::Function*)"},
    {"_ZN4llvm24CheckDebugInfoIntrinsicsEPNS_6ModuleE", "llvm::CheckDebugInfoIntrinsics(llvm::Module*)"},
    {"_ZN4llvm9IRBuilderILb1ENS_14ConstantFolderENS_24IRBuilderDefaultInserterILb1EEEE19CreateShuffleVectorEPNS_5ValueES6_S6_RKNS_5TwineE", "llvm::IRBuilder<true, llvm::ConstantFolder, llvm::IRBuilderDefaultInserter<true> >::CreateShuffleVector(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm10BasicBlock19getValueSymbolTableEv", "llvm::BasicBlock::getValueSymbolTable()"},
    {"_ZNK4llvm10BasicBlock10getContextEv", "llvm::BasicBlock::getContext() const"},
    {"_ZN4llvm10BasicBlockC1ERNS_11LLVMContextERKNS_5TwineEPNS_8FunctionEPS0_", "llvm::BasicBlock::BasicBlock(llvm::LLVMContext&, llvm::Twine const&, llvm::Function*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BasicBlockC2ERNS_11LLVMContextERKNS_5TwineEPNS_8FunctionEPS0_", "llvm::BasicBlock::BasicBlock(llvm::LLVMContext&, llvm::Twine const&, llvm::Function*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BasicBlockD0Ev", "llvm::BasicBlock::~BasicBlock()"},
    {"_ZN4llvm10BasicBlockD1Ev", "llvm::BasicBlock::~BasicBlock()"},
    {"_ZN4llvm10BasicBlockD2Ev", "llvm::BasicBlock::~BasicBlock()"},
    {"_ZN4llvm10BasicBlock17dropAllReferencesEv", "llvm::BasicBlock::dropAllReferences()"},
    {"_ZN4llvm10BasicBlock9setParentEPNS_8FunctionE", "llvm::BasicBlock::setParent(llvm::Function*)"},
    {"_ZN4llvm10BasicBlock16removeFromParentEv", "llvm::BasicBlock::removeFromParent()"},
    {"_ZN4llvm10BasicBlock15eraseFromParentEv", "llvm::BasicBlock::eraseFromParent()"},
    {"_ZN4llvm10BasicBlock10moveBeforeEPS0_", "llvm::BasicBlock::moveBefore(llvm::BasicBlock*)"},
    {"_ZN4llvm10BasicBlock9moveAfterEPS0_", "llvm::BasicBlock::moveAfter(llvm::BasicBlock*)"},
    {"_ZN4llvm10BasicBlock13getTerminatorEv", "llvm::BasicBlock::getTerminator()"},
    {"_ZNK4llvm10BasicBlock13getTerminatorEv", "llvm::BasicBlock::getTerminator() const"},
    {"_ZN4llvm10BasicBlock14getFirstNonPHIEv", "llvm::BasicBlock::getFirstNonPHI()"},
    {"_ZN4llvm10BasicBlock19getFirstNonPHIOrDbgEv", "llvm::BasicBlock::getFirstNonPHIOrDbg()"},
    {"_ZN4llvm10BasicBlock20getSinglePredecessorEv", "llvm::BasicBlock::getSinglePredecessor()"},
    {"_ZN4llvm10BasicBlock20getUniquePredecessorEv", "llvm::BasicBlock::getUniquePredecessor()"},
    {"_ZN4llvm10BasicBlock17removePredecessorEPS0_b", "llvm::BasicBlock::removePredecessor(llvm::BasicBlock*, bool)"},
    {"_ZN4llvm10BasicBlock15splitBasicBlockENS_14ilist_iteratorINS_11InstructionEEERKNS_5TwineE", "llvm::BasicBlock::splitBasicBlock(llvm::ilist_iterator<llvm::Instruction>, llvm::Twine const&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEEC1Ev", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEEC2Ev", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::getList(llvm::BasicBlock*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::getSymTab(llvm::BasicBlock*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::addNodeToList(llvm::Instruction*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::removeNodeFromList(llvm::Instruction*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::transferNodesFromList(llvm::ilist_traits<llvm::Instruction>&, llvm::ilist_iterator<llvm::Instruction>, llvm::ilist_iterator<llvm::Instruction>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11InstructionENS_10BasicBlockEE15setSymTabObjectIPNS_8FunctionEEEvPT_S7_", "void llvm::SymbolTableListTraits<llvm::Instruction, llvm::BasicBlock>::setSymTabObject<llvm::Function*>(llvm::Function**, llvm::Function*)"},
    {"_ZN4llvm6iplistINS_11InstructionENS_12ilist_traitsIS1_EEE5eraseENS_14ilist_iteratorIS1_EE", "llvm::iplist<llvm::Instruction, llvm::ilist_traits<llvm::Instruction> >::erase(llvm::ilist_iterator<llvm::Instruction>)"},
    {"_ZN4llvm27ConstantFoldCastInstructionEjPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantFoldCastInstruction(unsigned int, llvm::Constant*, llvm::Type const*)"},
    {"_Z15getFoldedSizeOfPKN4llvm4TypeES2_b", "getFoldedSizeOf(llvm::Type const*, llvm::Type const*, bool)"},
    {"_Z16getFoldedAlignOfPKN4llvm4TypeES2_b", "getFoldedAlignOf(llvm::Type const*, llvm::Type const*, bool)"},
    {"_Z20ExtractConstantBytesPN4llvm8ConstantEjj", "ExtractConstantBytes(llvm::Constant*, unsigned int, unsigned int)"},
    {"_ZN4llvm29ConstantFoldSelectInstructionEPNS_8ConstantES1_S1_", "llvm::ConstantFoldSelectInstruction(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm37ConstantFoldExtractElementInstructionEPNS_8ConstantES1_", "llvm::ConstantFoldExtractElementInstruction(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm36ConstantFoldInsertElementInstructionEPNS_8ConstantES1_S1_", "llvm::ConstantFoldInsertElementInstruction(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm36ConstantFoldShuffleVectorInstructionEPNS_8ConstantES1_S1_", "llvm::ConstantFoldShuffleVectorInstruction(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_Z16GetVectorElementPN4llvm8ConstantEj", "GetVectorElement(llvm::Constant*, unsigned int)"},
    {"_ZN4llvm35ConstantFoldExtractValueInstructionEPNS_8ConstantEPKjj", "llvm::ConstantFoldExtractValueInstruction(llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm34ConstantFoldInsertValueInstructionEPNS_8ConstantES1_PKjj", "llvm::ConstantFoldInsertValueInstruction(llvm::Constant*, llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm29ConstantFoldBinaryInstructionEjPNS_8ConstantES1_", "llvm::ConstantFoldBinaryInstruction(unsigned int, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm30ConstantFoldCompareInstructionEtPNS_8ConstantES1_", "llvm::ConstantFoldCompareInstruction(unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_Z20evaluateFCmpRelationPN4llvm8ConstantES1_", "evaluateFCmpRelation(llvm::Constant*, llvm::Constant*)"},
    {"_Z20evaluateICmpRelationPN4llvm8ConstantES1_b", "evaluateICmpRelation(llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm25ConstantFoldGetElementPtrEPNS_8ConstantEbPKS1_j", "llvm::ConstantFoldGetElementPtr(llvm::Constant*, bool, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm25ConstantFoldGetElementPtrEPNS_8ConstantEbPKPNS_5ValueEj", "llvm::ConstantFoldGetElementPtr(llvm::Constant*, bool, llvm::Value* const*, unsigned int)"},
    {"_Z20isMaybeZeroSizedTypePKN4llvm4TypeE", "isMaybeZeroSizedType(llvm::Type const*)"},
    {"_ZN4llvm8Constant12getNullValueEPKNS_4TypeE", "llvm::Constant::getNullValue(llvm::Type const*)"},
    {"_ZN4llvm11ConstantInt3getEPKNS_4TypeEyb", "llvm::ConstantInt::get(llvm::Type const*, unsigned long long, bool)"},
    {"_ZN4llvm10ConstantFP3getERNS_11LLVMContextERKNS_7APFloatE", "llvm::ConstantFP::get(llvm::LLVMContext&, llvm::APFloat const&)"},
    {"_ZN4llvm19ConstantPointerNull3getEPKNS_11PointerTypeE", "llvm::ConstantPointerNull::get(llvm::PointerType const*)"},
    {"_ZN4llvm21ConstantAggregateZero3getEPKNS_4TypeE", "llvm::ConstantAggregateZero::get(llvm::Type const*)"},
    {"_ZN4llvm8Constant15getIntegerValueEPKNS_4TypeERKNS_5APIntE", "llvm::Constant::getIntegerValue(llvm::Type const*, llvm::APInt const&)"},
    {"_ZN4llvm11ConstantInt3getERNS_11LLVMContextERKNS_5APIntE", "llvm::ConstantInt::get(llvm::LLVMContext&, llvm::APInt const&)"},
    {"_ZN4llvm12ConstantExpr11getIntToPtrEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getIntToPtr(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm14ConstantVector3getERKSt6vectorIPNS_8ConstantESaIS3_EE", "llvm::ConstantVector::get(std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm8Constant15getAllOnesValueEPKNS_4TypeE", "llvm::Constant::getAllOnesValue(llvm::Type const*)"},
    {"_ZN4llvm8Constant19destroyConstantImplEv", "llvm::Constant::destroyConstantImpl()"},
    {"_ZNK4llvm8Constant7canTrapEv", "llvm::Constant::canTrap() const"},
    {"_ZNK4llvm8Constant14isConstantUsedEv", "llvm::Constant::isConstantUsed() const"},
    {"_ZNK4llvm8Constant17getRelocationInfoEv", "llvm::Constant::getRelocationInfo() const"},
    {"_ZNK4llvm8Constant17getVectorElementsERNS_15SmallVectorImplIPS0_EE", "llvm::Constant::getVectorElements(llvm::SmallVectorImpl<llvm::Constant*>&) const"},
    {"_ZN4llvm10UndefValue3getEPKNS_4TypeE", "llvm::UndefValue::get(llvm::Type const*)"},
    {"_ZN4llvm11ConstantIntC1EPKNS_11IntegerTypeERKNS_5APIntE", "llvm::ConstantInt::ConstantInt(llvm::IntegerType const*, llvm::APInt const&)"},
    {"_ZN4llvm11ConstantIntC2EPKNS_11IntegerTypeERKNS_5APIntE", "llvm::ConstantInt::ConstantInt(llvm::IntegerType const*, llvm::APInt const&)"},
    {"_ZN4llvm11ConstantInt7getTrueERNS_11LLVMContextE", "llvm::ConstantInt::getTrue(llvm::LLVMContext&)"},
    {"_ZN4llvm11ConstantInt3getEPKNS_11IntegerTypeEyb", "llvm::ConstantInt::get(llvm::IntegerType const*, unsigned long long, bool)"},
    {"_ZN4llvm11ConstantInt8getFalseERNS_11LLVMContextE", "llvm::ConstantInt::getFalse(llvm::LLVMContext&)"},
    {"_ZN4llvm11ConstantInt9getSignedEPKNS_11IntegerTypeEx", "llvm::ConstantInt::getSigned(llvm::IntegerType const*, long long)"},
    {"_ZN4llvm11ConstantInt9getSignedEPKNS_4TypeEx", "llvm::ConstantInt::getSigned(llvm::Type const*, long long)"},
    {"_ZN4llvm11ConstantInt3getEPKNS_4TypeERKNS_5APIntE", "llvm::ConstantInt::get(llvm::Type const*, llvm::APInt const&)"},
    {"_ZN4llvm11ConstantInt3getEPKNS_11IntegerTypeENS_9StringRefEh", "llvm::ConstantInt::get(llvm::IntegerType const*, llvm::StringRef, unsigned char)"},
    {"_ZN4llvm10ConstantFP3getEPKNS_4TypeEd", "llvm::ConstantFP::get(llvm::Type const*, double)"},
    {"_ZN4llvm10ConstantFP3getEPKNS_4TypeENS_9StringRefE", "llvm::ConstantFP::get(llvm::Type const*, llvm::StringRef)"},
    {"_ZN4llvm10ConstantFP15getNegativeZeroEPKNS_4TypeE", "llvm::ConstantFP::getNegativeZero(llvm::Type const*)"},
    {"_ZN4llvm10ConstantFP23getZeroValueForNegationEPKNS_4TypeE", "llvm::ConstantFP::getZeroValueForNegation(llvm::Type const*)"},
    {"_ZN4llvm14ConstantVector3getEPKNS_10VectorTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantVector::get(llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm10ConstantFPC1EPKNS_4TypeERKNS_7APFloatE", "llvm::ConstantFP::ConstantFP(llvm::Type const*, llvm::APFloat const&)"},
    {"_ZN4llvm10ConstantFP11getInfinityEPKNS_4TypeEb", "llvm::ConstantFP::getInfinity(llvm::Type const*, bool)"},
    {"_ZN4llvm10ConstantFPC2EPKNS_4TypeERKNS_7APFloatE", "llvm::ConstantFP::ConstantFP(llvm::Type const*, llvm::APFloat const&)"},
    {"_ZNK4llvm10ConstantFP11isNullValueEv", "llvm::ConstantFP::isNullValue() const"},
    {"_ZNK4llvm10ConstantFP14isExactlyValueERKNS_7APFloatE", "llvm::ConstantFP::isExactlyValue(llvm::APFloat const&) const"},
    {"_ZN4llvm13ConstantArrayC1EPKNS_9ArrayTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantArray::ConstantArray(llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm13ConstantArrayC2EPKNS_9ArrayTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantArray::ConstantArray(llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm13ConstantArray3getEPKNS_9ArrayTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantArray::get(llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm13ConstantArray3getEPKNS_9ArrayTypeEPKPNS_8ConstantEj", "llvm::ConstantArray::get(llvm::ArrayType const*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm13ConstantArray3getERNS_11LLVMContextENS_9StringRefEb", "llvm::ConstantArray::get(llvm::LLVMContext&, llvm::StringRef, bool)"},
    {"_ZN4llvm14ConstantStructC1EPKNS_10StructTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantStruct::ConstantStruct(llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm14ConstantStructC2EPKNS_10StructTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantStruct::ConstantStruct(llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm14ConstantStruct3getEPKNS_10StructTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantStruct::get(llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm14ConstantStruct3getERNS_11LLVMContextERKSt6vectorIPNS_8ConstantESaIS5_EEb", "llvm::ConstantStruct::get(llvm::LLVMContext&, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, bool)"},
    {"_ZN4llvm14ConstantStruct3getERNS_11LLVMContextEPKPNS_8ConstantEjb", "llvm::ConstantStruct::get(llvm::LLVMContext&, llvm::Constant* const*, unsigned int, bool)"},
    {"_ZN4llvm14ConstantVectorC1EPKNS_10VectorTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantVector::ConstantVector(llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm14ConstantVectorC2EPKNS_10VectorTypeERKSt6vectorIPNS_8ConstantESaIS6_EE", "llvm::ConstantVector::ConstantVector(llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm14ConstantVector3getEPKPNS_8ConstantEj", "llvm::ConstantVector::get(llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr9getNSWNegEPNS_8ConstantE", "llvm::ConstantExpr::getNSWNeg(llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNSWSubEPNS_8ConstantES2_", "llvm::ConstantExpr::getNSWSub(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNUWNegEPNS_8ConstantE", "llvm::ConstantExpr::getNUWNeg(llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNUWSubEPNS_8ConstantES2_", "llvm::ConstantExpr::getNUWSub(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNSWAddEPNS_8ConstantES2_", "llvm::ConstantExpr::getNSWAdd(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr5getTyEPKNS_4TypeEjPNS_8ConstantES5_j", "llvm::ConstantExpr::getTy(llvm::Type const*, unsigned int, llvm::Constant*, llvm::Constant*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr9getNUWAddEPNS_8ConstantES2_", "llvm::ConstantExpr::getNUWAdd(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNSWMulEPNS_8ConstantES2_", "llvm::ConstantExpr::getNSWMul(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNUWMulEPNS_8ConstantES2_", "llvm::ConstantExpr::getNUWMul(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNSWShlEPNS_8ConstantES2_", "llvm::ConstantExpr::getNSWShl(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getNUWShlEPNS_8ConstantES2_", "llvm::ConstantExpr::getNUWShl(llvm::Constant*, llvm::Constant*)"},
    {"_ZNK4llvm12ConstantExpr6isCastEv", "llvm::ConstantExpr::isCast() const"},
    {"_ZNK4llvm12ConstantExpr9isCompareEv", "llvm::ConstantExpr::isCompare() const"},
    {"_ZNK4llvm12ConstantExpr31isGEPWithNoNotionalOverIndexingEv", "llvm::ConstantExpr::isGEPWithNoNotionalOverIndexing() const"},
    {"_ZNK4llvm12ConstantExpr10hasIndicesEv", "llvm::ConstantExpr::hasIndices() const"},
    {"_ZNK4llvm12ConstantExpr10getIndicesEv", "llvm::ConstantExpr::getIndices() const"},
    {"_ZNK4llvm12ConstantExpr12getPredicateEv", "llvm::ConstantExpr::getPredicate() const"},
    {"_ZNK4llvm12ConstantExpr22getWithOperandReplacedEjPNS_8ConstantE", "llvm::ConstantExpr::getWithOperandReplaced(unsigned int, llvm::Constant*) const"},
    {"_ZN4llvm12ConstantExpr7getCastEjPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getCast(unsigned int, llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr16getInsertElementEPNS_8ConstantES2_S2_", "llvm::ConstantExpr::getInsertElement(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr17getExtractElementEPNS_8ConstantES2_", "llvm::ConstantExpr::getExtractElement(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr16getShuffleVectorEPNS_8ConstantES2_S2_", "llvm::ConstantExpr::getShuffleVector(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr24getInBoundsGetElementPtrEPNS_8ConstantEPKS2_j", "llvm::ConstantExpr::getInBoundsGetElementPtr(llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr16getGetElementPtrEPNS_8ConstantEPKS2_j", "llvm::ConstantExpr::getGetElementPtr(llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr3getEjPNS_8ConstantES2_j", "llvm::ConstantExpr::get(unsigned int, llvm::Constant*, llvm::Constant*, unsigned int)"},
    {"_ZNK4llvm12ConstantExpr15getWithOperandsEPKPNS_8ConstantEj", "llvm::ConstantExpr::getWithOperands(llvm::Constant* const*, unsigned int) const"},
    {"_ZN4llvm12ConstantExpr10getCompareEtPNS_8ConstantES2_", "llvm::ConstantExpr::getCompare(unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm11ConstantInt19isValueValidForTypeEPKNS_4TypeEy", "llvm::ConstantInt::isValueValidForType(llvm::Type const*, unsigned long long)"},
    {"_ZN4llvm11ConstantInt19isValueValidForTypeEPKNS_4TypeEx", "llvm::ConstantInt::isValueValidForType(llvm::Type const*, long long)"},
    {"_ZN4llvm10ConstantFP19isValueValidForTypeEPKNS_4TypeERKNS_7APFloatE", "llvm::ConstantFP::isValueValidForType(llvm::Type const*, llvm::APFloat const&)"},
    {"_ZN4llvm21ConstantAggregateZero15destroyConstantEv", "llvm::ConstantAggregateZero::destroyConstant()"},
    {"_ZN4llvm13ConstantArray15destroyConstantEv", "llvm::ConstantArray::destroyConstant()"},
    {"_ZNK4llvm13ConstantArray8isStringEv", "llvm::ConstantArray::isString() const"},
    {"_ZNK4llvm13ConstantArray9isCStringEv", "llvm::ConstantArray::isCString() const"},
    {"_ZNK4llvm13ConstantArray11getAsStringEv", "llvm::ConstantArray::getAsString() const"},
    {"_ZN4llvm14ConstantStruct15destroyConstantEv", "llvm::ConstantStruct::destroyConstant()"},
    {"_ZN4llvm14ConstantVector15destroyConstantEv", "llvm::ConstantVector::destroyConstant()"},
    {"_ZNK4llvm14ConstantVector14isAllOnesValueEv", "llvm::ConstantVector::isAllOnesValue() const"},
    {"_ZNK4llvm14ConstantVector13getSplatValueEv", "llvm::ConstantVector::getSplatValue() const"},
    {"_ZN4llvm19ConstantPointerNull15destroyConstantEv", "llvm::ConstantPointerNull::destroyConstant()"},
    {"_ZN4llvm10UndefValue15destroyConstantEv", "llvm::UndefValue::destroyConstant()"},
    {"_ZN4llvm12BlockAddress3getEPNS_10BasicBlockE", "llvm::BlockAddress::get(llvm::BasicBlock*)"},
    {"_ZN4llvm12BlockAddress3getEPNS_8FunctionEPNS_10BasicBlockE", "llvm::BlockAddress::get(llvm::Function*, llvm::BasicBlock*)"},
    {"_ZN4llvm12BlockAddressC1EPNS_8FunctionEPNS_10BasicBlockE", "llvm::BlockAddress::BlockAddress(llvm::Function*, llvm::BasicBlock*)"},
    {"_ZN4llvm12BlockAddressC2EPNS_8FunctionEPNS_10BasicBlockE", "llvm::BlockAddress::BlockAddress(llvm::Function*, llvm::BasicBlock*)"},
    {"_ZN4llvm12BlockAddress15destroyConstantEv", "llvm::BlockAddress::destroyConstant()"},
    {"_ZN4llvm12BlockAddress27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::BlockAddress::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm12ConstantExpr8getTruncEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getTrunc(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr7getZExtEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getZExt(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr7getSExtEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getSExt(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr10getFPTruncEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getFPTrunc(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr11getFPExtendEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getFPExtend(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr9getUIToFPEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getUIToFP(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr9getSIToFPEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getSIToFP(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr9getFPToUIEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getFPToUI(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr9getFPToSIEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getFPToSI(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr11getPtrToIntEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getPtrToInt(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr10getBitCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getBitCast(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr16getZExtOrBitCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getZExtOrBitCast(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr16getSExtOrBitCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getSExtOrBitCast(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr17getTruncOrBitCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getTruncOrBitCast(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr14getPointerCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getPointerCast(llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr14getIntegerCastEPNS_8ConstantEPKNS_4TypeEb", "llvm::ConstantExpr::getIntegerCast(llvm::Constant*, llvm::Type const*, bool)"},
    {"_ZN4llvm12ConstantExpr9getFPCastEPNS_8ConstantEPKNS_4TypeE", "llvm::ConstantExpr::getFPCast(llvm::Constant*, llvm::Type const*)"},
    {"_Z13getFoldedCastN4llvm11Instruction7CastOpsEPNS_8ConstantEPKNS_4TypeE", "getFoldedCast(llvm::Instruction::CastOps, llvm::Constant*, llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr12getCompareTyEtPNS_8ConstantES2_", "llvm::ConstantExpr::getCompareTy(unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFCmpEtPNS_8ConstantES2_", "llvm::ConstantExpr::getFCmp(unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getICmpEtPNS_8ConstantES2_", "llvm::ConstantExpr::getICmp(unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr9getSizeOfEPKNS_4TypeE", "llvm::ConstantExpr::getSizeOf(llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr10getAlignOfEPKNS_4TypeE", "llvm::ConstantExpr::getAlignOf(llvm::Type const*)"},
    {"_ZN4llvm12ConstantExpr11getOffsetOfEPKNS_10StructTypeEj", "llvm::ConstantExpr::getOffsetOf(llvm::StructType const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr11getOffsetOfEPKNS_4TypeEPNS_8ConstantE", "llvm::ConstantExpr::getOffsetOf(llvm::Type const*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr11getSelectTyEPKNS_4TypeEPNS_8ConstantES5_S5_", "llvm::ConstantExpr::getSelectTy(llvm::Type const*, llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr16getGetElementPtrEPNS_8ConstantEPKPNS_5ValueEj", "llvm::ConstantExpr::getGetElementPtr(llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr24getInBoundsGetElementPtrEPNS_8ConstantEPKPNS_5ValueEj", "llvm::ConstantExpr::getInBoundsGetElementPtr(llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr19getExtractElementTyEPKNS_4TypeEPNS_8ConstantES5_", "llvm::ConstantExpr::getExtractElementTy(llvm::Type const*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr18getInsertElementTyEPKNS_4TypeEPNS_8ConstantES5_S5_", "llvm::ConstantExpr::getInsertElementTy(llvm::Type const*, llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr18getShuffleVectorTyEPKNS_4TypeEPNS_8ConstantES5_S5_", "llvm::ConstantExpr::getShuffleVectorTy(llvm::Type const*, llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr16getInsertValueTyEPKNS_4TypeEPNS_8ConstantES5_PKjj", "llvm::ConstantExpr::getInsertValueTy(llvm::Type const*, llvm::Constant*, llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr14getInsertValueEPNS_8ConstantES2_PKjj", "llvm::ConstantExpr::getInsertValue(llvm::Constant*, llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr17getExtractValueTyEPKNS_4TypeEPNS_8ConstantEPKjj", "llvm::ConstantExpr::getExtractValueTy(llvm::Type const*, llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr15getExtractValueEPNS_8ConstantEPKjj", "llvm::ConstantExpr::getExtractValue(llvm::Constant*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr6getNegEPNS_8ConstantE", "llvm::ConstantExpr::getNeg(llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFNegEPNS_8ConstantE", "llvm::ConstantExpr::getFNeg(llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getNotEPNS_8ConstantE", "llvm::ConstantExpr::getNot(llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getAddEPNS_8ConstantES2_", "llvm::ConstantExpr::getAdd(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFAddEPNS_8ConstantES2_", "llvm::ConstantExpr::getFAdd(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getSubEPNS_8ConstantES2_", "llvm::ConstantExpr::getSub(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFSubEPNS_8ConstantES2_", "llvm::ConstantExpr::getFSub(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getMulEPNS_8ConstantES2_", "llvm::ConstantExpr::getMul(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFMulEPNS_8ConstantES2_", "llvm::ConstantExpr::getFMul(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getUDivEPNS_8ConstantES2_b", "llvm::ConstantExpr::getUDiv(llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm12ConstantExpr7getSDivEPNS_8ConstantES2_b", "llvm::ConstantExpr::getSDiv(llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm12ConstantExpr7getFDivEPNS_8ConstantES2_", "llvm::ConstantExpr::getFDiv(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getURemEPNS_8ConstantES2_", "llvm::ConstantExpr::getURem(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getSRemEPNS_8ConstantES2_", "llvm::ConstantExpr::getSRem(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getFRemEPNS_8ConstantES2_", "llvm::ConstantExpr::getFRem(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getAndEPNS_8ConstantES2_", "llvm::ConstantExpr::getAnd(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr5getOrEPNS_8ConstantES2_", "llvm::ConstantExpr::getOr(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getXorEPNS_8ConstantES2_", "llvm::ConstantExpr::getXor(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr6getShlEPNS_8ConstantES2_", "llvm::ConstantExpr::getShl(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm12ConstantExpr7getLShrEPNS_8ConstantES2_b", "llvm::ConstantExpr::getLShr(llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm12ConstantExpr7getAShrEPNS_8ConstantES2_b", "llvm::ConstantExpr::getAShr(llvm::Constant*, llvm::Constant*, bool)"},
    {"_ZN4llvm12ConstantExpr15destroyConstantEv", "llvm::ConstantExpr::destroyConstant()"},
    {"_ZNK4llvm12ConstantExpr13getOpcodeNameEv", "llvm::ConstantExpr::getOpcodeName() const"},
    {"_ZN4llvm25GetElementPtrConstantExprC1EPNS_8ConstantERKSt6vectorIS2_SaIS2_EEPKNS_4TypeE", "llvm::GetElementPtrConstantExpr::GetElementPtrConstantExpr(llvm::Constant*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, llvm::Type const*)"},
    {"_ZN4llvm25GetElementPtrConstantExprC2EPNS_8ConstantERKSt6vectorIS2_SaIS2_EEPKNS_4TypeE", "llvm::GetElementPtrConstantExpr::GetElementPtrConstantExpr(llvm::Constant*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, llvm::Type const*)"},
    {"_ZN4llvm13ConstantArray27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::ConstantArray::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm14ConstantStruct27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::ConstantStruct::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm14ConstantVector27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::ConstantVector::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm12ConstantExpr27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::ConstantExpr::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE11getOrCreateEPKS6_RKS5_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::getOrCreate(llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE11getOrCreateEPKS6_RKS5_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::getOrCreate(llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE11getOrCreateEPKS6_RKS5_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::getOrCreate(llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&)"},
    {"_ZN4llvm15SmallVectorImplIPNS_8ConstantEE6resizeEj", "llvm::SmallVectorImpl<llvm::Constant*>::resize(unsigned int)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE6removeEPS2_", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::remove(llvm::ConstantAggregateZero*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE6removeEPS7_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::remove(llvm::ConstantArray*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE6removeEPS7_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::remove(llvm::ConstantStruct*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE6removeEPS7_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::remove(llvm::ConstantVector*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE6removeEPS2_", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::remove(llvm::ConstantPointerNull*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE6removeEPS2_", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::remove(llvm::UndefValue*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE11getOrCreateEPKS2_RKS1_", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::getOrCreate(llvm::Type const*, llvm::ExprMapKeyType const&)"},
    {"_ZN4llvm12ConstantExpr20getGetElementPtrImplIPNS_5ValueEEEPNS_8ConstantES5_PKT_j", "llvm::Constant* llvm::ConstantExpr::getGetElementPtrImpl<llvm::Value*>(llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr20getGetElementPtrImplIPNS_8ConstantEEES3_S3_PKT_j", "llvm::Constant* llvm::ConstantExpr::getGetElementPtrImpl<llvm::Constant*>(llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr28getInBoundsGetElementPtrImplIPNS_5ValueEEEPNS_8ConstantES5_PKT_j", "llvm::Constant* llvm::ConstantExpr::getInBoundsGetElementPtrImpl<llvm::Value*>(llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr28getInBoundsGetElementPtrImplIPNS_8ConstantEEES3_S3_PKT_j", "llvm::Constant* llvm::ConstantExpr::getInBoundsGetElementPtrImpl<llvm::Constant*>(llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE6removeEPS3_", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::remove(llvm::ConstantExpr*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE15InsertOrGetItemERSt4pairIS9_IPKS6_S5_EPS7_ERb", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::InsertOrGetItem(std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, llvm::ConstantArray*>&, bool&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE21MoveConstantToNewSlotEPS7_St17_Rb_tree_iteratorISt4pairIKSB_IPKS6_S5_ES9_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::MoveConstantToNewSlot(llvm::ConstantArray*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE15InsertOrGetItemERSt4pairIS9_IPKS6_S5_EPS7_ERb", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::InsertOrGetItem(std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, llvm::ConstantStruct*>&, bool&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE21MoveConstantToNewSlotEPS7_St17_Rb_tree_iteratorISt4pairIKSB_IPKS6_S5_ES9_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::MoveConstantToNewSlot(llvm::ConstantStruct*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >)"},
    {"_ZN4llvm21ConstantAggregateZeroD1Ev", "llvm::ConstantAggregateZero::~ConstantAggregateZero()"},
    {"_ZN4llvm21ConstantAggregateZeroD0Ev", "llvm::ConstantAggregateZero::~ConstantAggregateZero()"},
    {"_ZNK4llvm21ConstantAggregateZero11isNullValueEv", "llvm::ConstantAggregateZero::isNullValue() const"},
    {"_ZNK4llvm8Constant19isNegativeZeroValueEv", "llvm::Constant::isNegativeZeroValue() const"},
    {"_ZN4llvm8Constant27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::Constant::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm19ConstantPointerNullD1Ev", "llvm::ConstantPointerNull::~ConstantPointerNull()"},
    {"_ZN4llvm19ConstantPointerNullD0Ev", "llvm::ConstantPointerNull::~ConstantPointerNull()"},
    {"_ZNK4llvm19ConstantPointerNull11isNullValueEv", "llvm::ConstantPointerNull::isNullValue() const"},
    {"_ZN4llvm12ConstantExprD1Ev", "llvm::ConstantExpr::~ConstantExpr()"},
    {"_ZN4llvm12ConstantExprD0Ev", "llvm::ConstantExpr::~ConstantExpr()"},
    {"_ZNK4llvm12ConstantExpr11isNullValueEv", "llvm::ConstantExpr::isNullValue() const"},
    {"_ZN4llvm10UndefValueD1Ev", "llvm::UndefValue::~UndefValue()"},
    {"_ZN4llvm10UndefValueD0Ev", "llvm::UndefValue::~UndefValue()"},
    {"_ZNK4llvm10UndefValue11isNullValueEv", "llvm::UndefValue::isNullValue() const"},
    {"_ZN4llvm10ConstantFPD1Ev", "llvm::ConstantFP::~ConstantFP()"},
    {"_ZN4llvm10ConstantFPD0Ev", "llvm::ConstantFP::~ConstantFP()"},
    {"_ZNK4llvm10ConstantFP19isNegativeZeroValueEv", "llvm::ConstantFP::isNegativeZeroValue() const"},
    {"_ZN4llvm8Constant15destroyConstantEv", "llvm::Constant::destroyConstant()"},
    {"_ZN4llvm13ConstantArrayD1Ev", "llvm::ConstantArray::~ConstantArray()"},
    {"_ZN4llvm13ConstantArrayD0Ev", "llvm::ConstantArray::~ConstantArray()"},
    {"_ZNK4llvm13ConstantArray11isNullValueEv", "llvm::ConstantArray::isNullValue() const"},
    {"_ZN4llvm14ConstantStructD1Ev", "llvm::ConstantStruct::~ConstantStruct()"},
    {"_ZN4llvm14ConstantStructD0Ev", "llvm::ConstantStruct::~ConstantStruct()"},
    {"_ZNK4llvm14ConstantStruct11isNullValueEv", "llvm::ConstantStruct::isNullValue() const"},
    {"_ZN4llvm14ConstantVectorD1Ev", "llvm::ConstantVector::~ConstantVector()"},
    {"_ZN4llvm14ConstantVectorD0Ev", "llvm::ConstantVector::~ConstantVector()"},
    {"_ZNK4llvm14ConstantVector11isNullValueEv", "llvm::ConstantVector::isNullValue() const"},
    {"_ZN4llvm12BlockAddressD1Ev", "llvm::BlockAddress::~BlockAddress()"},
    {"_ZN4llvm12BlockAddressD0Ev", "llvm::BlockAddress::~BlockAddress()"},
    {"_ZNK4llvm12BlockAddress11isNullValueEv", "llvm::BlockAddress::isNullValue() const"},
    {"_ZN4llvm25GetElementPtrConstantExprD1Ev", "llvm::GetElementPtrConstantExpr::~GetElementPtrConstantExpr()"},
    {"_ZN4llvm25GetElementPtrConstantExprD0Ev", "llvm::GetElementPtrConstantExpr::~GetElementPtrConstantExpr()"},
    {"_ZN4llvm11ConstantIntD1Ev", "llvm::ConstantInt::~ConstantInt()"},
    {"_ZN4llvm11ConstantIntD0Ev", "llvm::ConstantInt::~ConstantInt()"},
    {"_ZNK4llvm11ConstantInt11isNullValueEv", "llvm::ConstantInt::isNullValue() const"},
    {"_ZN4llvm8ConstantD1Ev", "llvm::Constant::~Constant()"},
    {"_ZN4llvm8ConstantD0Ev", "llvm::Constant::~Constant()"},
    {"_ZN4llvm4UserD1Ev", "llvm::User::~User()"},
    {"_ZN4llvm4UserD0Ev", "llvm::User::~User()"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE19FindExistingElementEPS3_", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::FindExistingElement(llvm::ConstantExpr*)"},
    {"_ZN4llvm12ConstantExpr26getInBoundsGetElementPtrTyIPNS_8ConstantEEES3_PKNS_4TypeES3_PKT_j", "llvm::Constant* llvm::ConstantExpr::getInBoundsGetElementPtrTy<llvm::Constant*>(llvm::Type const*, llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr26getInBoundsGetElementPtrTyIPNS_5ValueEEEPNS_8ConstantEPKNS_4TypeES5_PKT_j", "llvm::Constant* llvm::ConstantExpr::getInBoundsGetElementPtrTy<llvm::Value*>(llvm::Type const*, llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr18getGetElementPtrTyIPNS_8ConstantEEES3_PKNS_4TypeES3_PKT_j", "llvm::Constant* llvm::ConstantExpr::getGetElementPtrTy<llvm::Constant*>(llvm::Type const*, llvm::Constant*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm12ConstantExpr18getGetElementPtrTyIPNS_5ValueEEEPNS_8ConstantEPKNS_4TypeES5_PKT_j", "llvm::Constant* llvm::ConstantExpr::getGetElementPtrTy<llvm::Value*>(llvm::Type const*, llvm::Constant*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE6CreateEPKS2_RKS1_St17_Rb_tree_iteratorISt4pairIKSA_IS6_S1_EPS3_EE", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::Create(llvm::Type const*, llvm::ExprMapKeyType const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >)"},
    {"_ZN4llvm15ConstantCreatorINS_12ConstantExprENS_4TypeENS_14ExprMapKeyTypeEE6createEPKS2_RKS3_t", "llvm::ConstantCreator<llvm::ConstantExpr, llvm::Type, llvm::ExprMapKeyType>::create(llvm::Type const*, llvm::ExprMapKeyType const&, unsigned short)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeENS1_14ExprMapKeyTypeEES0_IKS6_PNS1_12ConstantExprEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >, std::less<std::pair<llvm::Type const*, llvm::ExprMapKeyType> >, std::allocator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >, std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> const&)"},
    {"_ZN4llvm19CompareConstantExprC2EPKNS_4TypeENS_11Instruction8OtherOpsEtPNS_8ConstantES7_", "llvm::CompareConstantExpr::CompareConstantExpr(llvm::Type const*, llvm::Instruction::OtherOps, unsigned short, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm19CompareConstantExprD1Ev", "llvm::CompareConstantExpr::~CompareConstantExpr()"},
    {"_ZN4llvm19CompareConstantExprD0Ev", "llvm::CompareConstantExpr::~CompareConstantExpr()"},
    {"_ZN4llvm24ExtractValueConstantExprC2EPNS_8ConstantERKNS_11SmallVectorIjLj4EEEPKNS_4TypeE", "llvm::ExtractValueConstantExpr::ExtractValueConstantExpr(llvm::Constant*, llvm::SmallVector<unsigned int, 4u> const&, llvm::Type const*)"},
    {"_ZN4llvm24ExtractValueConstantExprD1Ev", "llvm::ExtractValueConstantExpr::~ExtractValueConstantExpr()"},
    {"_ZN4llvm24ExtractValueConstantExprD0Ev", "llvm::ExtractValueConstantExpr::~ExtractValueConstantExpr()"},
    {"_ZN4llvm23InsertValueConstantExprC2EPNS_8ConstantES2_RKNS_11SmallVectorIjLj4EEEPKNS_4TypeE", "llvm::InsertValueConstantExpr::InsertValueConstantExpr(llvm::Constant*, llvm::Constant*, llvm::SmallVector<unsigned int, 4u> const&, llvm::Type const*)"},
    {"_ZN4llvm23InsertValueConstantExprD1Ev", "llvm::InsertValueConstantExpr::~InsertValueConstantExpr()"},
    {"_ZN4llvm23InsertValueConstantExprD0Ev", "llvm::InsertValueConstantExpr::~InsertValueConstantExpr()"},
    {"_ZN4llvm25ShuffleVectorConstantExprC2EPNS_8ConstantES2_S2_", "llvm::ShuffleVectorConstantExpr::ShuffleVectorConstantExpr(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm25ShuffleVectorConstantExprD1Ev", "llvm::ShuffleVectorConstantExpr::~ShuffleVectorConstantExpr()"},
    {"_ZN4llvm25ShuffleVectorConstantExprD0Ev", "llvm::ShuffleVectorConstantExpr::~ShuffleVectorConstantExpr()"},
    {"_ZN4llvm25InsertElementConstantExprC2EPNS_8ConstantES2_S2_", "llvm::InsertElementConstantExpr::InsertElementConstantExpr(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm25InsertElementConstantExprD1Ev", "llvm::InsertElementConstantExpr::~InsertElementConstantExpr()"},
    {"_ZN4llvm25InsertElementConstantExprD0Ev", "llvm::InsertElementConstantExpr::~InsertElementConstantExpr()"},
    {"_ZN4llvm26ExtractElementConstantExprC2EPNS_8ConstantES2_", "llvm::ExtractElementConstantExpr::ExtractElementConstantExpr(llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm26ExtractElementConstantExprD1Ev", "llvm::ExtractElementConstantExpr::~ExtractElementConstantExpr()"},
    {"_ZN4llvm26ExtractElementConstantExprD0Ev", "llvm::ExtractElementConstantExpr::~ExtractElementConstantExpr()"},
    {"_ZN4llvm18SelectConstantExprC2EPNS_8ConstantES2_S2_", "llvm::SelectConstantExpr::SelectConstantExpr(llvm::Constant*, llvm::Constant*, llvm::Constant*)"},
    {"_ZN4llvm18SelectConstantExprD1Ev", "llvm::SelectConstantExpr::~SelectConstantExpr()"},
    {"_ZN4llvm18SelectConstantExprD0Ev", "llvm::SelectConstantExpr::~SelectConstantExpr()"},
    {"_ZN4llvm18BinaryConstantExprC2EjPNS_8ConstantES2_j", "llvm::BinaryConstantExpr::BinaryConstantExpr(unsigned int, llvm::Constant*, llvm::Constant*, unsigned int)"},
    {"_ZN4llvm18BinaryConstantExprD1Ev", "llvm::BinaryConstantExpr::~BinaryConstantExpr()"},
    {"_ZN4llvm18BinaryConstantExprD0Ev", "llvm::BinaryConstantExpr::~BinaryConstantExpr()"},
    {"_ZN4llvm17UnaryConstantExprD1Ev", "llvm::UnaryConstantExpr::~UnaryConstantExpr()"},
    {"_ZN4llvm17UnaryConstantExprD0Ev", "llvm::UnaryConstantExpr::~UnaryConstantExpr()"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_8FunctionEPNS_10BasicBlockEEPNS_12BlockAddressENS_12DenseMapInfoIS6_EENS9_IS8_EEE15LookupBucketForERKS6_RPS1_IS6_S8_E", "llvm::DenseMap<std::pair<llvm::Function*, llvm::BasicBlock*>, llvm::BlockAddress*, llvm::DenseMapInfo<std::pair<llvm::Function*, llvm::BasicBlock*> >, llvm::DenseMapInfo<llvm::BlockAddress*> >::LookupBucketFor(std::pair<llvm::Function*, llvm::BasicBlock*> const&, std::pair<std::pair<llvm::Function*, llvm::BasicBlock*>, llvm::BlockAddress*>*&) const"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_8FunctionEPNS_10BasicBlockEEPNS_12BlockAddressENS_12DenseMapInfoIS6_EENS9_IS8_EEE16InsertIntoBucketERKS6_RKS8_PS1_IS6_S8_E", "llvm::DenseMap<std::pair<llvm::Function*, llvm::BasicBlock*>, llvm::BlockAddress*, llvm::DenseMapInfo<std::pair<llvm::Function*, llvm::BasicBlock*> >, llvm::DenseMapInfo<llvm::BlockAddress*> >::InsertIntoBucket(std::pair<llvm::Function*, llvm::BasicBlock*> const&, llvm::BlockAddress* const&, std::pair<std::pair<llvm::Function*, llvm::BasicBlock*>, llvm::BlockAddress*>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_8FunctionEPNS_10BasicBlockEEPNS_12BlockAddressENS_12DenseMapInfoIS6_EENS9_IS8_EEE4growEj", "llvm::DenseMap<std::pair<llvm::Function*, llvm::BasicBlock*>, llvm::BlockAddress*, llvm::DenseMapInfo<std::pair<llvm::Function*, llvm::BasicBlock*> >, llvm::DenseMapInfo<llvm::BlockAddress*> >::grow(unsigned int)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE6CreateEPKS1_RKcSt17_Rb_tree_iteratorISt4pairIKS9_IS5_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::Create(llvm::Type const*, char const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_10UndefValueEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS9_ERKS9_", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >, std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> const&)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE6CreateEPKS1_RKcSt17_Rb_tree_iteratorISt4pairIKS9_IS5_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::Create(llvm::PointerType const*, char const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeEcES0_IKS5_PNS1_19ConstantPointerNullEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS9_ERKS9_", "std::_Rb_tree<std::pair<llvm::PointerType const*, char>, std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >, std::less<std::pair<llvm::PointerType const*, char> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >, std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE19FindExistingElementEPS7_", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::FindExistingElement(llvm::ConstantVector*)"},
    {"_ZNSt8_Rb_treeIPN4llvm14ConstantStructESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE5eraseES5_ISI_ESP_", "std::_Rb_tree<llvm::ConstantStruct*, std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::ConstantStruct*>, std::allocator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::_Rb_tree_iterator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >)"},
    {"_ZNSt8_Rb_treeIPN4llvm13ConstantArrayESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE5eraseES5_ISI_ESP_", "std::_Rb_tree<llvm::ConstantArray*, std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::ConstantArray*>, std::allocator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::_Rb_tree_iterator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE6CreateEPKS1_RKcSt17_Rb_tree_iteratorISt4pairIKS9_IS5_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::Create(llvm::Type const*, char const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_21ConstantAggregateZeroEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS9_ERKS9_", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >, std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE6CreateEPKS6_RKS5_St17_Rb_tree_iteratorISt4pairIKSE_ISA_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::Create(llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISE_ERKSE_", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE6CreateEPKS6_RKS5_St17_Rb_tree_iteratorISt4pairIKSE_ISA_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::Create(llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISE_ERKSE_", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> const&)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE6CreateEPKS6_RKS5_St17_Rb_tree_iteratorISt4pairIKSE_ISA_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::Create(llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISE_ERKSE_", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> const&)"},
    {"_ZNK4llvm8DenseMapINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EEE15LookupBucketForERKS2_RPSt4pairIS2_S4_E", "llvm::DenseMap<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*> >::LookupBucketFor(llvm::DenseMapAPFloatKeyInfo::KeyTy const&, std::pair<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*>*&) const"},
    {"_ZN4llvm8DenseMapINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*> >::InsertIntoBucket(llvm::DenseMapAPFloatKeyInfo::KeyTy const&, llvm::ConstantFP* const&, std::pair<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*>*)"},
    {"_ZN4llvm8DenseMapINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EEE4growEj", "llvm::DenseMap<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*> >::grow(unsigned int)"},
    {"_ZNK4llvm8DenseMapINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EEE15LookupBucketForERKS2_RPSt4pairIS2_S4_E", "llvm::DenseMap<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*> >::LookupBucketFor(llvm::DenseMapAPIntKeyInfo::KeyTy const&, std::pair<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*>*&) const"},
    {"_ZN4llvm8DenseMapINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*> >::InsertIntoBucket(llvm::DenseMapAPIntKeyInfo::KeyTy const&, llvm::ConstantInt* const&, std::pair<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*>*)"},
    {"_ZN4llvm8DenseMapINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EEE4growEj", "llvm::DenseMap<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPN4llvm8ConstantESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, unsigned long, llvm::Constant* const&)"},
    {"_ZNK4llvm8DebugLoc8getScopeERKNS_11LLVMContextE", "llvm::DebugLoc::getScope(llvm::LLVMContext const&) const"},
    {"_ZNK4llvm8DebugLoc12getInlinedAtERKNS_11LLVMContextE", "llvm::DebugLoc::getInlinedAt(llvm::LLVMContext const&) const"},
    {"_ZNK4llvm8DebugLoc20getScopeAndInlinedAtERPNS_6MDNodeES3_RKNS_11LLVMContextE", "llvm::DebugLoc::getScopeAndInlinedAt(llvm::MDNode*&, llvm::MDNode*&, llvm::LLVMContext const&) const"},
    {"_ZN4llvm8DebugLoc3getEjjPNS_6MDNodeES2_", "llvm::DebugLoc::get(unsigned int, unsigned int, llvm::MDNode*, llvm::MDNode*)"},
    {"_ZN4llvm15LLVMContextImpl27getOrAddScopeRecordIdxEntryEPNS_6MDNodeEi", "llvm::LLVMContextImpl::getOrAddScopeRecordIdxEntry(llvm::MDNode*, int)"},
    {"_ZN4llvm15LLVMContextImpl30getOrAddScopeInlinedAtIdxEntryEPNS_6MDNodeES2_i", "llvm::LLVMContextImpl::getOrAddScopeInlinedAtIdxEntry(llvm::MDNode*, llvm::MDNode*, int)"},
    {"_ZNK4llvm8DebugLoc11getAsMDNodeERKNS_11LLVMContextE", "llvm::DebugLoc::getAsMDNode(llvm::LLVMContext const&) const"},
    {"_ZN4llvm8DebugLoc17getFromDILocationEPNS_6MDNodeE", "llvm::DebugLoc::getFromDILocation(llvm::MDNode*)"},
    {"_ZN4llvm10DebugRecVH7deletedEv", "llvm::DebugRecVH::deleted()"},
    {"_ZN4llvm10DebugRecVH19allUsesReplacedWithEPNS_5ValueE", "llvm::DebugRecVH::allUsesReplacedWith(llvm::Value*)"},
    {"_ZNSt6vectorIN4llvm10DebugRecVHESaIS1_EE7reserveEm", "std::vector<llvm::DebugRecVH, std::allocator<llvm::DebugRecVH> >::reserve(unsigned long)"},
    {"_ZNSt6vectorISt4pairIN4llvm10DebugRecVHES2_ESaIS3_EE7reserveEm", "std::vector<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>, std::allocator<std::pair<llvm::DebugRecVH, llvm::DebugRecVH> > >::reserve(unsigned long)"},
    {"_ZNSt6vectorISt4pairIN4llvm10DebugRecVHES2_ESaIS3_EE9push_backERKS3_", "std::vector<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>, std::allocator<std::pair<llvm::DebugRecVH, llvm::DebugRecVH> > >::push_back(std::pair<llvm::DebugRecVH, llvm::DebugRecVH> const&)"},
    {"_ZNK4llvm8DenseMapISt4pairIPNS_6MDNodeES3_EiNS_12DenseMapInfoIS4_EENS5_IiEEE15LookupBucketForERKS4_RPS1_IS4_iE", "llvm::DenseMap<std::pair<llvm::MDNode*, llvm::MDNode*>, int, llvm::DenseMapInfo<std::pair<llvm::MDNode*, llvm::MDNode*> >, llvm::DenseMapInfo<int> >::LookupBucketFor(std::pair<llvm::MDNode*, llvm::MDNode*> const&, std::pair<std::pair<llvm::MDNode*, llvm::MDNode*>, int>*&) const"},
    {"_ZNSt6vectorISt4pairIN4llvm10DebugRecVHES2_ESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>, std::allocator<std::pair<llvm::DebugRecVH, llvm::DebugRecVH> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::vector<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>, std::allocator<std::pair<llvm::DebugRecVH, llvm::DebugRecVH> > > >, std::pair<llvm::DebugRecVH, llvm::DebugRecVH> const&)"},
    {"_ZSt24__uninitialized_copy_auxIPSt4pairIN4llvm10DebugRecVHES2_ES4_ET0_T_S6_S5_St12__false_type", "std::pair<llvm::DebugRecVH, llvm::DebugRecVH>* std::__uninitialized_copy_aux<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*>(std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::__false_type)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPSt4pairIN4llvm10DebugRecVHES5_ES7_EET0_T_S9_S8_", "std::pair<llvm::DebugRecVH, llvm::DebugRecVH>* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*>(std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*, std::pair<llvm::DebugRecVH, llvm::DebugRecVH>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_6MDNodeES3_EiNS_12DenseMapInfoIS4_EENS5_IiEEE16InsertIntoBucketERKS4_RKiPS1_IS4_iE", "llvm::DenseMap<std::pair<llvm::MDNode*, llvm::MDNode*>, int, llvm::DenseMapInfo<std::pair<llvm::MDNode*, llvm::MDNode*> >, llvm::DenseMapInfo<int> >::InsertIntoBucket(std::pair<llvm::MDNode*, llvm::MDNode*> const&, int const&, std::pair<std::pair<llvm::MDNode*, llvm::MDNode*>, int>*)"},
    {"_ZN4llvm8DenseMapISt4pairIPNS_6MDNodeES3_EiNS_12DenseMapInfoIS4_EENS5_IiEEE4growEj", "llvm::DenseMap<std::pair<llvm::MDNode*, llvm::MDNode*>, int, llvm::DenseMapInfo<std::pair<llvm::MDNode*, llvm::MDNode*> >, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm10DebugRecVHESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::DebugRecVH, std::allocator<llvm::DebugRecVH> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::DebugRecVH*, std::vector<llvm::DebugRecVH, std::allocator<llvm::DebugRecVH> > >, llvm::DebugRecVH const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm10DebugRecVHES5_EET0_T_S7_S6_", "llvm::DebugRecVH* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::DebugRecVH*, llvm::DebugRecVH*>(llvm::DebugRecVH*, llvm::DebugRecVH*, llvm::DebugRecVH*)"},
    {"_ZNSt6vectorIN4llvm10DebugRecVHESaIS1_EE20_M_allocate_and_copyIPS1_EES5_mT_S6_", "llvm::DebugRecVH* std::vector<llvm::DebugRecVH, std::allocator<llvm::DebugRecVH> >::_M_allocate_and_copy<llvm::DebugRecVH*>(unsigned long, llvm::DebugRecVH*, llvm::DebugRecVH*)"},
    {"_ZN4llvm8DenseMapIPNS_6MDNodeEiNS_12DenseMapInfoIS2_EENS3_IiEEE16InsertIntoBucketERKS2_RKiPSt4pairIS2_iE", "llvm::DenseMap<llvm::MDNode*, int, llvm::DenseMapInfo<llvm::MDNode*>, llvm::DenseMapInfo<int> >::InsertIntoBucket(llvm::MDNode* const&, int const&, std::pair<llvm::MDNode*, int>*)"},
    {"_ZN4llvm8DenseMapIPNS_6MDNodeEiNS_12DenseMapInfoIS2_EENS3_IiEEE4growEj", "llvm::DenseMap<llvm::MDNode*, int, llvm::DenseMapInfo<llvm::MDNode*>, llvm::DenseMapInfo<int> >::grow(unsigned int)"},
    {"_ZN4llvm12PrintDomTreeINS_10BasicBlockEEEvPKNS_15DomTreeNodeBaseIT_EERNS_11raw_ostreamEj", "void llvm::PrintDomTree<llvm::BasicBlock>(llvm::DomTreeNodeBase<llvm::BasicBlock> const*, llvm::raw_ostream&, unsigned int)"},
    {"_ZN4llvm27initializeDominatorTreePassERNS_12PassRegistryE", "llvm::initializeDominatorTreePass(llvm::PassRegistry&)"},
    {"_ZN4llvm13DominatorTree13runOnFunctionERNS_8FunctionE", "llvm::DominatorTree::runOnFunction(llvm::Function&)"},
    {"_ZNK4llvm13DominatorTree14verifyAnalysisEv", "llvm::DominatorTree::verifyAnalysis() const"},
    {"_ZNK4llvm13DominatorTree5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::DominatorTree::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm13DominatorTree9dominatesEPKNS_11InstructionES3_", "llvm::DominatorTree::dominates(llvm::Instruction const*, llvm::Instruction const*) const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE5beginEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::begin()"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE3endEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::end()"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE5beginEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::begin() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE3endEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::end() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE8getBlockEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getBlock() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE7getIDomEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getIDom() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE11getChildrenEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getChildren() const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEEC1EPS1_PS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::DomTreeNodeBase(llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEEC2EPS1_PS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::DomTreeNodeBase(llvm::BasicBlock*, llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE8addChildEPS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::addChild(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE14getNumChildrenEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getNumChildren() const"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE16clearAllChildrenEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::clearAllChildren()"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE7compareEPS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::compare(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN4llvm15DomTreeNodeBaseINS_10BasicBlockEE7setIDomEPS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::setIDom(llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE11getDFSNumInEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getDFSNumIn() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE12getDFSNumOutEv", "llvm::DomTreeNodeBase<llvm::BasicBlock>::getDFSNumOut() const"},
    {"_ZNK4llvm15DomTreeNodeBaseINS_10BasicBlockEE11DominatedByEPKS2_", "llvm::DomTreeNodeBase<llvm::BasicBlock>::DominatedBy(llvm::DomTreeNodeBase<llvm::BasicBlock> const*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE7InfoRecC1Ev", "llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec::InfoRec()"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE7InfoRecC2Ev", "llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec::InfoRec()"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE5resetEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::reset()"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEEC1Eb", "llvm::DominatorTreeBase<llvm::BasicBlock>::DominatorTreeBase(bool)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEEC2Eb", "llvm::DominatorTreeBase<llvm::BasicBlock>::DominatorTreeBase(bool)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEED0Ev", "llvm::DominatorTreeBase<llvm::BasicBlock>::~DominatorTreeBase()"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEED1Ev", "llvm::DominatorTreeBase<llvm::BasicBlock>::~DominatorTreeBase()"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEED2Ev", "llvm::DominatorTreeBase<llvm::BasicBlock>::~DominatorTreeBase()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE7compareERS2_", "llvm::DominatorTreeBase<llvm::BasicBlock>::compare(llvm::DominatorTreeBase<llvm::BasicBlock>&) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE13releaseMemoryEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::releaseMemory()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE7getNodeEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::getNode(llvm::BasicBlock*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE11getRootNodeEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::getRootNode()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE11getRootNodeEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::getRootNode() const"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE17properlyDominatesEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::BasicBlock>::properlyDominates(llvm::DomTreeNodeBase<llvm::BasicBlock> const*, llvm::DomTreeNodeBase<llvm::BasicBlock> const*) const"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE23dominatedBySlowTreeWalkEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::BasicBlock>::dominatedBySlowTreeWalk(llvm::DomTreeNodeBase<llvm::BasicBlock> const*, llvm::DomTreeNodeBase<llvm::BasicBlock> const*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE17properlyDominatesEPKS1_S4_", "llvm::DominatorTreeBase<llvm::BasicBlock>::properlyDominates(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE20isReachableFromEntryEPKS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::isReachableFromEntry(llvm::BasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE9dominatesEPKS1_S4_", "llvm::DominatorTreeBase<llvm::BasicBlock>::dominates(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE9dominatesEPKNS_15DomTreeNodeBaseIS1_EES6_", "llvm::DominatorTreeBase<llvm::BasicBlock>::dominates(llvm::DomTreeNodeBase<llvm::BasicBlock> const*, llvm::DomTreeNodeBase<llvm::BasicBlock> const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE16updateDFSNumbersEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::updateDFSNumbers()"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE7getRootEv", "llvm::DominatorTreeBase<llvm::BasicBlock>::getRoot() const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE26findNearestCommonDominatorEPS1_S3_", "llvm::DominatorTreeBase<llvm::BasicBlock>::findNearestCommonDominator(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE26findNearestCommonDominatorEPKS1_S4_", "llvm::DominatorTreeBase<llvm::BasicBlock>::findNearestCommonDominator(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE11addNewBlockEPS1_S3_", "llvm::DominatorTreeBase<llvm::BasicBlock>::addNewBlock(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE24changeImmediateDominatorEPNS_15DomTreeNodeBaseIS1_EES5_", "llvm::DominatorTreeBase<llvm::BasicBlock>::changeImmediateDominator(llvm::DomTreeNodeBase<llvm::BasicBlock>*, llvm::DomTreeNodeBase<llvm::BasicBlock>*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE24changeImmediateDominatorEPS1_S3_", "llvm::DominatorTreeBase<llvm::BasicBlock>::changeImmediateDominator(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE9eraseNodeEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::eraseNode(llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE10removeNodeEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::removeNode(llvm::BasicBlock*)"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE10splitBlockEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::splitBlock(llvm::BasicBlock*)"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE5printERNS_11raw_ostreamE", "llvm::DominatorTreeBase<llvm::BasicBlock>::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE15getNodeForBlockEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::getNodeForBlock(llvm::BasicBlock*)"},
    {"_ZNK4llvm17DominatorTreeBaseINS_10BasicBlockEE7getIDomEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::getIDom(llvm::BasicBlock*) const"},
    {"_ZN4llvm17DominatorTreeBaseINS_10BasicBlockEE7addRootEPS1_", "llvm::DominatorTreeBase<llvm::BasicBlock>::addRoot(llvm::BasicBlock*)"},
    {"_ZN4llvm13DominatorTreeD1Ev", "llvm::DominatorTree::~DominatorTree()"},
    {"_ZN4llvm13DominatorTreeD0Ev", "llvm::DominatorTree::~DominatorTree()"},
    {"_ZNK4llvm13DominatorTree16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::DominatorTree::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13DominatorTree13releaseMemoryEv", "llvm::DominatorTree::releaseMemory()"},
    {"_ZN4llvm9CalculateINS_8FunctionEPNS_10BasicBlockEEEvRNS_17DominatorTreeBaseINS_11GraphTraitsIT0_E8NodeTypeEEERT_", "void llvm::Calculate<llvm::Function, llvm::BasicBlock*>(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::BasicBlock*>::NodeType>&, llvm::Function&)"},
    {"_ZN4llvm9CalculateINS_8FunctionENS_7InverseIPNS_10BasicBlockEEEEEvRNS_17DominatorTreeBaseINS_11GraphTraitsIT0_E8NodeTypeEEERT_", "void llvm::Calculate<llvm::Function, llvm::Inverse<llvm::BasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType>&, llvm::Function&)"},
    {"_ZN4llvm7DFSPassINS_11GraphTraitsINS_7InverseIPNS_10BasicBlockEEEEEEEjRNS_17DominatorTreeBaseINT_8NodeTypeEEEPS9_j", "unsigned int llvm::DFSPass<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType*, unsigned int)"},
    {"_ZN4llvm4EvalINS_11GraphTraitsINS_7InverseIPNS_10BasicBlockEEEEEEEPNT_8NodeTypeERNS_17DominatorTreeBaseIS8_EES9_j", "llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType* llvm::Eval<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> > >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType>&, llvm::GraphTraits<llvm::Inverse<llvm::BasicBlock*> >::NodeType*, unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE16shrink_and_clearEv", "llvm::DenseMap<llvm::BasicBlock*, llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec> >::shrink_and_clear()"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPNS_10BasicBlockENS_12PredIteratorIS2_NS_18value_use_iteratorINS_4UserEEEEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::BasicBlock*, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<llvm::BasicBlock*, llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec> >::InsertIntoBucket(llvm::BasicBlock* const&, llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec const&, std::pair<llvm::BasicBlock*, llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec>*)"},
    {"_ZN4llvm8DenseMapIPNS_10BasicBlockENS_17DominatorTreeBaseIS1_E7InfoRecENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::BasicBlock*, llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec, llvm::DenseMapInfo<llvm::BasicBlock*>, llvm::DenseMapInfo<llvm::DominatorTreeBase<llvm::BasicBlock>::InfoRec> >::grow(unsigned int)"},
    {"_ZN4llvm7DFSPassINS_11GraphTraitsIPNS_10BasicBlockEEEEEjRNS_17DominatorTreeBaseINT_8NodeTypeEEEPS7_j", "unsigned int llvm::DFSPass<llvm::GraphTraits<llvm::BasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::BasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::BasicBlock*>::NodeType*, unsigned int)"},
    {"_ZN4llvm4EvalINS_11GraphTraitsIPNS_10BasicBlockEEEEEPNT_8NodeTypeERNS_17DominatorTreeBaseIS6_EES7_j", "llvm::GraphTraits<llvm::BasicBlock*>::NodeType* llvm::Eval<llvm::GraphTraits<llvm::BasicBlock*> >(llvm::DominatorTreeBase<llvm::GraphTraits<llvm::BasicBlock*>::NodeType>&, llvm::GraphTraits<llvm::BasicBlock*>::NodeType*, unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIPNS_10BasicBlockENS_12SuccIteratorIPNS_14TerminatorInstES2_EEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<llvm::BasicBlock*, llvm::SuccIterator<llvm::TerminatorInst*, llvm::BasicBlock> >, false>::grow(unsigned long)"},
    {"_ZN4llvm15callDefaultCtorINS_13DominatorTreeEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<llvm::DominatorTree>()"},
    {"_ZN4llvm8Argument9setParentEPNS_8FunctionE", "llvm::Argument::setParent(llvm::Function*)"},
    {"_ZN4llvm8ArgumentC1EPKNS_4TypeERKNS_5TwineEPNS_8FunctionE", "llvm::Argument::Argument(llvm::Type const*, llvm::Twine const&, llvm::Function*)"},
    {"_ZN4llvm8ArgumentC2EPKNS_4TypeERKNS_5TwineEPNS_8FunctionE", "llvm::Argument::Argument(llvm::Type const*, llvm::Twine const&, llvm::Function*)"},
    {"_ZNK4llvm8Argument8getArgNoEv", "llvm::Argument::getArgNo() const"},
    {"_ZNK4llvm8Argument12hasByValAttrEv", "llvm::Argument::hasByValAttr() const"},
    {"_ZNK4llvm8Argument11hasNestAttrEv", "llvm::Argument::hasNestAttr() const"},
    {"_ZNK4llvm8Argument14hasNoAliasAttrEv", "llvm::Argument::hasNoAliasAttr() const"},
    {"_ZNK4llvm8Argument16hasNoCaptureAttrEv", "llvm::Argument::hasNoCaptureAttr() const"},
    {"_ZNK4llvm8Argument16hasStructRetAttrEv", "llvm::Argument::hasStructRetAttr() const"},
    {"_ZN4llvm8Argument7addAttrEj", "llvm::Argument::addAttr(unsigned int)"},
    {"_ZN4llvm8Function12addAttributeEjj", "llvm::Function::addAttribute(unsigned int, unsigned int)"},
    {"_ZN4llvm8Argument10removeAttrEj", "llvm::Argument::removeAttr(unsigned int)"},
    {"_ZN4llvm8Function15removeAttributeEjj", "llvm::Function::removeAttribute(unsigned int, unsigned int)"},
    {"_ZNK4llvm8Function10getContextEv", "llvm::Function::getContext() const"},
    {"_ZNK4llvm8Function15getFunctionTypeEv", "llvm::Function::getFunctionType() const"},
    {"_ZNK4llvm8Function8isVarArgEv", "llvm::Function::isVarArg() const"},
    {"_ZNK4llvm8Function13getReturnTypeEv", "llvm::Function::getReturnType() const"},
    {"_ZN4llvm8Function16removeFromParentEv", "llvm::Function::removeFromParent()"},
    {"_ZN4llvm8Function15eraseFromParentEv", "llvm::Function::eraseFromParent()"},
    {"_ZN4llvm8FunctionC1EPKNS_12FunctionTypeENS_11GlobalValue12LinkageTypesERKNS_5TwineEPNS_6ModuleE", "llvm::Function::Function(llvm::FunctionType const*, llvm::GlobalValue::LinkageTypes, llvm::Twine const&, llvm::Module*)"},
    {"_ZN4llvm8FunctionC2EPKNS_12FunctionTypeENS_11GlobalValue12LinkageTypesERKNS_5TwineEPNS_6ModuleE", "llvm::Function::Function(llvm::FunctionType const*, llvm::GlobalValue::LinkageTypes, llvm::Twine const&, llvm::Module*)"},
    {"_ZNK4llvm8Function14getIntrinsicIDEv", "llvm::Function::getIntrinsicID() const"},
    {"_ZN4llvm9Intrinsic13getAttributesENS0_2IDE", "llvm::Intrinsic::getAttributes(llvm::Intrinsic::ID)"},
    {"_ZN4llvm8FunctionD0Ev", "llvm::Function::~Function()"},
    {"_ZN4llvm8FunctionD1Ev", "llvm::Function::~Function()"},
    {"_ZN4llvm8FunctionD2Ev", "llvm::Function::~Function()"},
    {"_ZN4llvm8Function17dropAllReferencesEv", "llvm::Function::dropAllReferences()"},
    {"_ZN4llvm8Function7clearGCEv", "llvm::Function::clearGC()"},
    {"_ZNK4llvm8Function18BuildLazyArgumentsEv", "llvm::Function::BuildLazyArguments() const"},
    {"_ZNK4llvm8Function8arg_sizeEv", "llvm::Function::arg_size() const"},
    {"_ZNK4llvm8Function9arg_emptyEv", "llvm::Function::arg_empty() const"},
    {"_ZN4llvm8Function9setParentEPNS_6ModuleE", "llvm::Function::setParent(llvm::Module*)"},
    {"_ZNK4llvm8Function5hasGCEv", "llvm::Function::hasGC() const"},
    {"_ZNK4llvm8Function5getGCEv", "llvm::Function::getGC() const"},
    {"_ZN4llvm8Function5setGCEPKc", "llvm::Function::setGC(char const*)"},
    {"_ZN4llvm8Function18copyAttributesFromEPKNS_11GlobalValueE", "llvm::Function::copyAttributesFrom(llvm::GlobalValue const*)"},
    {"_ZN4llvm9Intrinsic7getNameENS0_2IDEPPKNS_4TypeEj", "llvm::Intrinsic::getName(llvm::Intrinsic::ID, llvm::Type const**, unsigned int)"},
    {"_ZN4llvm9Intrinsic7getTypeERNS_11LLVMContextENS0_2IDEPPKNS_4TypeEj", "llvm::Intrinsic::getType(llvm::LLVMContext&, llvm::Intrinsic::ID, llvm::Type const**, unsigned int)"},
    {"_ZN4llvm9Intrinsic12isOverloadedENS0_2IDE", "llvm::Intrinsic::isOverloaded(llvm::Intrinsic::ID)"},
    {"_ZN4llvm9Intrinsic14getDeclarationEPNS_6ModuleENS0_2IDEPPKNS_4TypeEj", "llvm::Intrinsic::getDeclaration(llvm::Module*, llvm::Intrinsic::ID, llvm::Type const**, unsigned int)"},
    {"_ZN4llvm9Intrinsic25getIntrinsicForGCCBuiltinEPKcS2_", "llvm::Intrinsic::getIntrinsicForGCCBuiltin(char const*, char const*)"},
    {"_ZNK4llvm8Function15hasAddressTakenEPPKNS_4UserE", "llvm::Function::hasAddressTaken(llvm::User const**) const"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEEC1Ev", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEEC2Ev", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::getList(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::getSymTab(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::addNodeToList(llvm::Argument*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::removeNodeFromList(llvm::Argument*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::transferNodesFromList(llvm::ilist_traits<llvm::Argument>&, llvm::ilist_iterator<llvm::Argument>, llvm::ilist_iterator<llvm::Argument>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8ArgumentENS_8FunctionEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Argument, llvm::Function>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEEC1Ev", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEEC2Ev", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::getList(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::getSymTab(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::addNodeToList(llvm::BasicBlock*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::removeNodeFromList(llvm::BasicBlock*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::transferNodesFromList(llvm::ilist_traits<llvm::BasicBlock>&, llvm::ilist_iterator<llvm::BasicBlock>, llvm::ilist_iterator<llvm::BasicBlock>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_10BasicBlockENS_8FunctionEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::BasicBlock, llvm::Function>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm6iplistINS_8ArgumentENS_12ilist_traitsIS1_EEE5clearEv", "llvm::iplist<llvm::Argument, llvm::ilist_traits<llvm::Argument> >::clear()"},
    {"_ZN4llvm11GlobalValueD2Ev", "llvm::GlobalValue::~GlobalValue()"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionENS_15PooledStringPtrENS_12DenseMapInfoIS3_EENS5_IS4_EEE5eraseERKS3_", "llvm::DenseMap<llvm::Function const*, llvm::PooledStringPtr, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::PooledStringPtr> >::erase(llvm::Function const* const&)"},
    {"_ZNK4llvm11GlobalValue11isNullValueEv", "llvm::GlobalValue::isNullValue() const"},
    {"_ZNK4llvm8Function13isDeclarationEv", "llvm::Function::isDeclaration() const"},
    {"_ZN4llvm8ArgumentD1Ev", "llvm::Argument::~Argument()"},
    {"_ZN4llvm8ArgumentD0Ev", "llvm::Argument::~Argument()"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionENS_15PooledStringPtrENS_12DenseMapInfoIS3_EENS5_IS4_EEED2Ev", "llvm::DenseMap<llvm::Function const*, llvm::PooledStringPtr, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::PooledStringPtr> >::~DenseMap()"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionENS_15PooledStringPtrENS_12DenseMapInfoIS3_EENS5_IS4_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::Function const*, llvm::PooledStringPtr, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::PooledStringPtr> >::FindAndConstruct(llvm::Function const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionENS_15PooledStringPtrENS_12DenseMapInfoIS3_EENS5_IS4_EEE16InsertIntoBucketERKS3_RKS4_PSt4pairIS3_S4_E", "llvm::DenseMap<llvm::Function const*, llvm::PooledStringPtr, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::PooledStringPtr> >::InsertIntoBucket(llvm::Function const* const&, llvm::PooledStringPtr const&, std::pair<llvm::Function const*, llvm::PooledStringPtr>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8FunctionENS_15PooledStringPtrENS_12DenseMapInfoIS3_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Function const*, llvm::PooledStringPtr, llvm::DenseMapInfo<llvm::Function const*>, llvm::DenseMapInfo<llvm::PooledStringPtr> >::grow(unsigned int)"},
    {"_ZN4llvm14object_creatorINS_3sys12SmartRWMutexILb1EEEEEPvv", "void* llvm::object_creator<llvm::sys::SmartRWMutex<true> >()"},
    {"_ZN4llvm14object_deleterINS_3sys12SmartRWMutexILb1EEEE4callEPv", "llvm::object_deleter<llvm::sys::SmartRWMutex<true> >::call(void*)"},
    {"_ZN4llvm6iplistINS_10BasicBlockENS_12ilist_traitsIS1_EEE5eraseENS_14ilist_iteratorIS1_EE", "llvm::iplist<llvm::BasicBlock, llvm::ilist_traits<llvm::BasicBlock> >::erase(llvm::ilist_iterator<llvm::BasicBlock>)"},
    {"_ZN4llvm11GlobalValueD1Ev", "llvm::GlobalValue::~GlobalValue()"},
    {"_ZN4llvm11GlobalValueD0Ev", "llvm::GlobalValue::~GlobalValue()"},
    {"_ZN4llvm14GVMaterializerD0Ev", "llvm::GVMaterializer::~GVMaterializer()"},
    {"_ZN4llvm14GVMaterializerD1Ev", "llvm::GVMaterializer::~GVMaterializer()"},
    {"_ZN4llvm14GVMaterializerD2Ev", "llvm::GVMaterializer::~GVMaterializer()"},
    {"_ZNK4llvm11GlobalValue16isMaterializableEv", "llvm::GlobalValue::isMaterializable() const"},
    {"_ZNK4llvm11GlobalValue18isDematerializableEv", "llvm::GlobalValue::isDematerializable() const"},
    {"_ZN4llvm11GlobalValue11MaterializeEPSs", "llvm::GlobalValue::Materialize(std::string*)"},
    {"_ZN4llvm11GlobalValue13DematerializeEv", "llvm::GlobalValue::Dematerialize()"},
    {"_ZNK4llvm11GlobalValue23removeDeadConstantUsersEv", "llvm::GlobalValue::removeDeadConstantUsers() const"},
    {"_Z25removeDeadUsersOfConstantPKN4llvm8ConstantE", "removeDeadUsersOfConstant(llvm::Constant const*)"},
    {"_ZN4llvm11GlobalValue15destroyConstantEv", "llvm::GlobalValue::destroyConstant()"},
    {"_ZN4llvm11GlobalValue18copyAttributesFromEPKS0_", "llvm::GlobalValue::copyAttributesFrom(llvm::GlobalValue const*)"},
    {"_ZN4llvm11GlobalValue12setAlignmentEj", "llvm::GlobalValue::setAlignment(unsigned int)"},
    {"_ZN4llvm14GlobalVariableC1EPKNS_4TypeEbNS_11GlobalValue12LinkageTypesEPNS_8ConstantERKNS_5TwineEbj", "llvm::GlobalVariable::GlobalVariable(llvm::Type const*, bool, llvm::GlobalValue::LinkageTypes, llvm::Constant*, llvm::Twine const&, bool, unsigned int)"},
    {"_ZN4llvm14GlobalVariableC2EPKNS_4TypeEbNS_11GlobalValue12LinkageTypesEPNS_8ConstantERKNS_5TwineEbj", "llvm::GlobalVariable::GlobalVariable(llvm::Type const*, bool, llvm::GlobalValue::LinkageTypes, llvm::Constant*, llvm::Twine const&, bool, unsigned int)"},
    {"_ZN4llvm14GlobalVariableC1ERNS_6ModuleEPKNS_4TypeEbNS_11GlobalValue12LinkageTypesEPNS_8ConstantERKNS_5TwineEPS0_bj", "llvm::GlobalVariable::GlobalVariable(llvm::Module&, llvm::Type const*, bool, llvm::GlobalValue::LinkageTypes, llvm::Constant*, llvm::Twine const&, llvm::GlobalVariable*, bool, unsigned int)"},
    {"_ZN4llvm14GlobalVariableC2ERNS_6ModuleEPKNS_4TypeEbNS_11GlobalValue12LinkageTypesEPNS_8ConstantERKNS_5TwineEPS0_bj", "llvm::GlobalVariable::GlobalVariable(llvm::Module&, llvm::Type const*, bool, llvm::GlobalValue::LinkageTypes, llvm::Constant*, llvm::Twine const&, llvm::GlobalVariable*, bool, unsigned int)"},
    {"_ZN4llvm14GlobalVariable9setParentEPNS_6ModuleE", "llvm::GlobalVariable::setParent(llvm::Module*)"},
    {"_ZN4llvm14GlobalVariable16removeFromParentEv", "llvm::GlobalVariable::removeFromParent()"},
    {"_ZN4llvm14GlobalVariable15eraseFromParentEv", "llvm::GlobalVariable::eraseFromParent()"},
    {"_ZN4llvm14GlobalVariable27replaceUsesOfWithOnConstantEPNS_5ValueES2_PNS_3UseE", "llvm::GlobalVariable::replaceUsesOfWithOnConstant(llvm::Value*, llvm::Value*, llvm::Use*)"},
    {"_ZN4llvm14GlobalVariable14setInitializerEPNS_8ConstantE", "llvm::GlobalVariable::setInitializer(llvm::Constant*)"},
    {"_ZN4llvm14GlobalVariable18copyAttributesFromEPKNS_11GlobalValueE", "llvm::GlobalVariable::copyAttributesFrom(llvm::GlobalValue const*)"},
    {"_ZN4llvm11GlobalAliasC1EPKNS_4TypeENS_11GlobalValue12LinkageTypesERKNS_5TwineEPNS_8ConstantEPNS_6ModuleE", "llvm::GlobalAlias::GlobalAlias(llvm::Type const*, llvm::GlobalValue::LinkageTypes, llvm::Twine const&, llvm::Constant*, llvm::Module*)"},
    {"_ZN4llvm11GlobalAliasC2EPKNS_4TypeENS_11GlobalValue12LinkageTypesERKNS_5TwineEPNS_8ConstantEPNS_6ModuleE", "llvm::GlobalAlias::GlobalAlias(llvm::Type const*, llvm::GlobalValue::LinkageTypes, llvm::Twine const&, llvm::Constant*, llvm::Module*)"},
    {"_ZN4llvm11GlobalAlias9setParentEPNS_6ModuleE", "llvm::GlobalAlias::setParent(llvm::Module*)"},
    {"_ZN4llvm11GlobalAlias16removeFromParentEv", "llvm::GlobalAlias::removeFromParent()"},
    {"_ZN4llvm11GlobalAlias15eraseFromParentEv", "llvm::GlobalAlias::eraseFromParent()"},
    {"_ZNK4llvm11GlobalAlias13isDeclarationEv", "llvm::GlobalAlias::isDeclaration() const"},
    {"_ZNK4llvm11GlobalAlias16getAliasedGlobalEv", "llvm::GlobalAlias::getAliasedGlobal() const"},
    {"_ZN4llvm11GlobalAlias10setAliaseeEPNS_8ConstantE", "llvm::GlobalAlias::setAliasee(llvm::Constant*)"},
    {"_ZNK4llvm11GlobalAlias20resolveAliasedGlobalEb", "llvm::GlobalAlias::resolveAliasedGlobal(bool) const"},
    {"_ZN4llvm6iplistINS_14GlobalVariableENS_12ilist_traitsIS1_EEE9push_backEPS1_", "llvm::iplist<llvm::GlobalVariable, llvm::ilist_traits<llvm::GlobalVariable> >::push_back(llvm::GlobalVariable*)"},
    {"_ZN4llvm6iplistINS_11GlobalAliasENS_12ilist_traitsIS1_EEE9push_backEPS1_", "llvm::iplist<llvm::GlobalAlias, llvm::ilist_traits<llvm::GlobalAlias> >::push_back(llvm::GlobalAlias*)"},
    {"_ZN4llvm14GlobalVariableD1Ev", "llvm::GlobalVariable::~GlobalVariable()"},
    {"_ZN4llvm14GlobalVariableD0Ev", "llvm::GlobalVariable::~GlobalVariable()"},
    {"_ZNK4llvm14GlobalVariable13isDeclarationEv", "llvm::GlobalVariable::isDeclaration() const"},
    {"_ZN4llvm11GlobalAliasD1Ev", "llvm::GlobalAlias::~GlobalAlias()"},
    {"_ZN4llvm11GlobalAliasD0Ev", "llvm::GlobalAlias::~GlobalAlias()"},
    {"_ZN4llvm13IRBuilderBase18CreateGlobalStringEPKcRKNS_5TwineE", "llvm::IRBuilderBase::CreateGlobalString(char const*, llvm::Twine const&)"},
    {"_ZNK4llvm13IRBuilderBase28getCurrentFunctionReturnTypeEv", "llvm::IRBuilderBase::getCurrentFunctionReturnType() const"},
    {"_ZN4llvm13IRBuilderBase21getCastedInt8PtrValueEPNS_5ValueE", "llvm::IRBuilderBase::getCastedInt8PtrValue(llvm::Value*)"},
    {"_ZN4llvm13IRBuilderBase12CreateMemSetEPNS_5ValueES2_S2_jbPNS_6MDNodeE", "llvm::IRBuilderBase::CreateMemSet(llvm::Value*, llvm::Value*, llvm::Value*, unsigned int, bool, llvm::MDNode*)"},
    {"_Z16createCallHelperPN4llvm5ValueEPKS1_jPNS_13IRBuilderBaseE", "createCallHelper(llvm::Value*, llvm::Value* const*, unsigned int, llvm::IRBuilderBase*)"},
    {"_ZN4llvm13IRBuilderBase12CreateMemCpyEPNS_5ValueES2_S2_jbPNS_6MDNodeE", "llvm::IRBuilderBase::CreateMemCpy(llvm::Value*, llvm::Value*, llvm::Value*, unsigned int, bool, llvm::MDNode*)"},
    {"_ZN4llvm13IRBuilderBase13CreateMemMoveEPNS_5ValueES2_S2_jbPNS_6MDNodeE", "llvm::IRBuilderBase::CreateMemMove(llvm::Value*, llvm::Value*, llvm::Value*, unsigned int, bool, llvm::MDNode*)"},
    {"_ZN4llvm9InlineAsmD0Ev", "llvm::InlineAsm::~InlineAsm()"},
    {"_ZN4llvm9InlineAsmD1Ev", "llvm::InlineAsm::~InlineAsm()"},
    {"_ZN4llvm9InlineAsmD2Ev", "llvm::InlineAsm::~InlineAsm()"},
    {"_ZN4llvm9InlineAsm3getEPKNS_12FunctionTypeENS_9StringRefES4_bb", "llvm::InlineAsm::get(llvm::FunctionType const*, llvm::StringRef, llvm::StringRef, bool, bool)"},
    {"_ZN4llvm9InlineAsmC1EPKNS_11PointerTypeERKSsS5_bb", "llvm::InlineAsm::InlineAsm(llvm::PointerType const*, std::string const&, std::string const&, bool, bool)"},
    {"_ZN4llvm9InlineAsmC2EPKNS_11PointerTypeERKSsS5_bb", "llvm::InlineAsm::InlineAsm(llvm::PointerType const*, std::string const&, std::string const&, bool, bool)"},
    {"_ZN4llvm9InlineAsm15destroyConstantEv", "llvm::InlineAsm::destroyConstant()"},
    {"_ZNK4llvm9InlineAsm15getFunctionTypeEv", "llvm::InlineAsm::getFunctionType() const"},
    {"_ZN4llvm9InlineAsm14ConstraintInfoC1Ev", "llvm::InlineAsm::ConstraintInfo::ConstraintInfo()"},
    {"_ZN4llvm9InlineAsm14ConstraintInfoC2Ev", "llvm::InlineAsm::ConstraintInfo::ConstraintInfo()"},
    {"_ZN4llvm9InlineAsm14ConstraintInfoC1ERKS1_", "llvm::InlineAsm::ConstraintInfo::ConstraintInfo(llvm::InlineAsm::ConstraintInfo const&)"},
    {"_ZN4llvm9InlineAsm14ConstraintInfoC2ERKS1_", "llvm::InlineAsm::ConstraintInfo::ConstraintInfo(llvm::InlineAsm::ConstraintInfo const&)"},
    {"_ZN4llvm9InlineAsm14ConstraintInfo5ParseENS_9StringRefERSt6vectorIS1_SaIS1_EE", "llvm::InlineAsm::ConstraintInfo::Parse(llvm::StringRef, std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >&)"},
    {"_ZN4llvm9InlineAsm14ConstraintInfo17selectAlternativeEj", "llvm::InlineAsm::ConstraintInfo::selectAlternative(unsigned int)"},
    {"_ZN4llvm9InlineAsm16ParseConstraintsENS_9StringRefE", "llvm::InlineAsm::ParseConstraints(llvm::StringRef)"},
    {"_ZN4llvm9InlineAsm6VerifyEPKNS_12FunctionTypeENS_9StringRefE", "llvm::InlineAsm::Verify(llvm::FunctionType const*, llvm::StringRef)"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE11getOrCreateEPKS2_RKS1_", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::getOrCreate(llvm::PointerType const*, llvm::InlineAsmKeyType const&)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm14ConstraintInfoESaIS2_EE9push_backERKS2_", "std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >::push_back(llvm::InlineAsm::ConstraintInfo const&)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm14ConstraintInfoESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::InlineAsm::ConstraintInfo*, std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> > >, llvm::InlineAsm::ConstraintInfo const&)"},
    {"_ZNSt15__copy_backwardILb0ESt26random_access_iterator_tagE8__copy_bIPN4llvm9InlineAsm14ConstraintInfoES6_EET0_T_S8_S7_", "llvm::InlineAsm::ConstraintInfo* std::__copy_backward<false, std::random_access_iterator_tag>::__copy_b<llvm::InlineAsm::ConstraintInfo*, llvm::InlineAsm::ConstraintInfo*>(llvm::InlineAsm::ConstraintInfo*, llvm::InlineAsm::ConstraintInfo*, llvm::InlineAsm::ConstraintInfo*)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm14ConstraintInfoESaIS2_EE15_M_erase_at_endEPS2_", "std::vector<llvm::InlineAsm::ConstraintInfo, std::allocator<llvm::InlineAsm::ConstraintInfo> >::_M_erase_at_end(llvm::InlineAsm::ConstraintInfo*)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm17SubConstraintInfoESaIS2_EE15_M_erase_at_endEPS2_", "std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> >::_M_erase_at_end(llvm::InlineAsm::SubConstraintInfo*)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm17SubConstraintInfoESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::InlineAsm::SubConstraintInfo*, std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> > >, unsigned long, llvm::InlineAsm::SubConstraintInfo const&)"},
    {"_ZNSt6vectorIN4llvm9InlineAsm17SubConstraintInfoESaIS2_EEC2ERKS4_", "std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> >::vector(std::vector<llvm::InlineAsm::SubConstraintInfo, std::allocator<llvm::InlineAsm::SubConstraintInfo> > const&)"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE19FindExistingElementEPS3_", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::FindExistingElement(llvm::InlineAsm*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE6CreateEPKS2_RKS1_St17_Rb_tree_iteratorISt4pairIKSA_IS6_S1_EPS3_EE", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::Create(llvm::PointerType const*, llvm::InlineAsmKeyType const&, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeENS1_16InlineAsmKeyTypeEES0_IKS6_PNS1_9InlineAsmEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE16_M_insert_uniqueESt17_Rb_tree_iteratorISA_ERKSA_", "std::_Rb_tree<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType>, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::less<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> const&)"},
    {"_ZN4llvm11InstructionC1EPKNS_4TypeEjPNS_3UseEjPS0_", "llvm::Instruction::Instruction(llvm::Type const*, unsigned int, llvm::Use*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm11InstructionC2EPKNS_4TypeEjPNS_3UseEjPS0_", "llvm::Instruction::Instruction(llvm::Type const*, unsigned int, llvm::Use*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm11InstructionC1EPKNS_4TypeEjPNS_3UseEjPNS_10BasicBlockE", "llvm::Instruction::Instruction(llvm::Type const*, unsigned int, llvm::Use*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm11InstructionC2EPKNS_4TypeEjPNS_3UseEjPNS_10BasicBlockE", "llvm::Instruction::Instruction(llvm::Type const*, unsigned int, llvm::Use*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm11InstructionD0Ev", "llvm::Instruction::~Instruction()"},
    {"_ZN4llvm11InstructionD1Ev", "llvm::Instruction::~Instruction()"},
    {"_ZN4llvm11InstructionD2Ev", "llvm::Instruction::~Instruction()"},
    {"_ZN4llvm11Instruction9setParentEPNS_10BasicBlockE", "llvm::Instruction::setParent(llvm::BasicBlock*)"},
    {"_ZN4llvm11Instruction16removeFromParentEv", "llvm::Instruction::removeFromParent()"},
    {"_ZN4llvm11Instruction15eraseFromParentEv", "llvm::Instruction::eraseFromParent()"},
    {"_ZN4llvm11Instruction12insertBeforeEPS0_", "llvm::Instruction::insertBefore(llvm::Instruction*)"},
    {"_ZN4llvm11Instruction11insertAfterEPS0_", "llvm::Instruction::insertAfter(llvm::Instruction*)"},
    {"_ZN4llvm11Instruction10moveBeforeEPS0_", "llvm::Instruction::moveBefore(llvm::Instruction*)"},
    {"_ZN4llvm11Instruction13getOpcodeNameEj", "llvm::Instruction::getOpcodeName(unsigned int)"},
    {"_ZNK4llvm11Instruction13isIdenticalToEPKS0_", "llvm::Instruction::isIdenticalTo(llvm::Instruction const*) const"},
    {"_ZNK4llvm11Instruction24isIdenticalToWhenDefinedEPKS0_", "llvm::Instruction::isIdenticalToWhenDefined(llvm::Instruction const*) const"},
    {"_ZNK4llvm11Instruction17isSameOperationAsEPKS0_", "llvm::Instruction::isSameOperationAs(llvm::Instruction const*) const"},
    {"_ZNK4llvm11Instruction20isUsedOutsideOfBlockEPKNS_10BasicBlockE", "llvm::Instruction::isUsedOutsideOfBlock(llvm::BasicBlock const*) const"},
    {"_ZNK4llvm11Instruction17mayReadFromMemoryEv", "llvm::Instruction::mayReadFromMemory() const"},
    {"_ZNK4llvm11Instruction16mayWriteToMemoryEv", "llvm::Instruction::mayWriteToMemory() const"},
    {"_ZNK4llvm11Instruction8mayThrowEv", "llvm::Instruction::mayThrow() const"},
    {"_ZN4llvm11Instruction13isAssociativeEj", "llvm::Instruction::isAssociative(unsigned int)"},
    {"_ZN4llvm11Instruction13isCommutativeEj", "llvm::Instruction::isCommutative(unsigned int)"},
    {"_ZNK4llvm11Instruction28isSafeToSpeculativelyExecuteEv", "llvm::Instruction::isSafeToSpeculativelyExecute() const"},
    {"_ZNK4llvm11Instruction5cloneEv", "llvm::Instruction::clone() const"},
    {"_ZNK4llvm8CallSite9getCalleeEv", "llvm::CallSite::getCallee() const"},
    {"_ZN4llvm14TerminatorInstD0Ev", "llvm::TerminatorInst::~TerminatorInst()"},
    {"_ZN4llvm14TerminatorInstD1Ev", "llvm::TerminatorInst::~TerminatorInst()"},
    {"_ZN4llvm14TerminatorInstD2Ev", "llvm::TerminatorInst::~TerminatorInst()"},
    {"_ZN4llvm16UnaryInstructionD0Ev", "llvm::UnaryInstruction::~UnaryInstruction()"},
    {"_ZN4llvm16UnaryInstructionD1Ev", "llvm::UnaryInstruction::~UnaryInstruction()"},
    {"_ZN4llvm16UnaryInstructionD2Ev", "llvm::UnaryInstruction::~UnaryInstruction()"},
    {"_ZN4llvm10SelectInst18areInvalidOperandsEPNS_5ValueES2_S2_", "llvm::SelectInst::areInvalidOperands(llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm7PHINodeC1ERKS0_", "llvm::PHINode::PHINode(llvm::PHINode const&)"},
    {"_ZN4llvm7PHINodeC2ERKS0_", "llvm::PHINode::PHINode(llvm::PHINode const&)"},
    {"_ZN4llvm7PHINodeD0Ev", "llvm::PHINode::~PHINode()"},
    {"_ZN4llvm7PHINodeD1Ev", "llvm::PHINode::~PHINode()"},
    {"_ZN4llvm7PHINodeD2Ev", "llvm::PHINode::~PHINode()"},
    {"_ZN4llvm7PHINode19removeIncomingValueEjb", "llvm::PHINode::removeIncomingValue(unsigned int, bool)"},
    {"_ZN4llvm7PHINode14resizeOperandsEj", "llvm::PHINode::resizeOperands(unsigned int)"},
    {"_ZNK4llvm7PHINode16hasConstantValueEv", "llvm::PHINode::hasConstantValue() const"},
    {"_ZN4llvm8CallInstD0Ev", "llvm::CallInst::~CallInst()"},
    {"_ZN4llvm8CallInstD1Ev", "llvm::CallInst::~CallInst()"},
    {"_ZN4llvm8CallInstD2Ev", "llvm::CallInst::~CallInst()"},
    {"_ZN4llvm8CallInst4initEPNS_5ValueEPKS2_j", "llvm::CallInst::init(llvm::Value*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm8CallInst4initEPNS_5ValueES2_S2_", "llvm::CallInst::init(llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm8CallInst4initEPNS_5ValueES2_", "llvm::CallInst::init(llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm8CallInst4initEPNS_5ValueE", "llvm::CallInst::init(llvm::Value*)"},
    {"_ZN4llvm8CallInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CallInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CallInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInstC1EPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CallInstC2EPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CallInstC1EPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInstC2EPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::CallInst::CallInst(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInstC1ERKS0_", "llvm::CallInst::CallInst(llvm::CallInst const&)"},
    {"_ZN4llvm8CallInstC2ERKS0_", "llvm::CallInst::CallInst(llvm::CallInst const&)"},
    {"_ZN4llvm8CallInst12addAttributeEjj", "llvm::CallInst::addAttribute(unsigned int, unsigned int)"},
    {"_ZN4llvm8CallInst15removeAttributeEjj", "llvm::CallInst::removeAttribute(unsigned int, unsigned int)"},
    {"_ZNK4llvm8CallInst12paramHasAttrEjj", "llvm::CallInst::paramHasAttr(unsigned int, unsigned int) const"},
    {"_ZN4llvm8CallInst12CreateMallocEPNS_11InstructionEPKNS_4TypeES5_PNS_5ValueES7_PNS_8FunctionERKNS_5TwineE", "llvm::CallInst::CreateMalloc(llvm::Instruction*, llvm::Type const*, llvm::Type const*, llvm::Value*, llvm::Value*, llvm::Function*, llvm::Twine const&)"},
    {"_Z12createMallocPN4llvm11InstructionEPNS_10BasicBlockEPKNS_4TypeES6_PNS_5ValueES8_PNS_8FunctionERKNS_5TwineE", "createMalloc(llvm::Instruction*, llvm::BasicBlock*, llvm::Type const*, llvm::Type const*, llvm::Value*, llvm::Value*, llvm::Function*, llvm::Twine const&)"},
    {"_ZN4llvm8CallInst12CreateMallocEPNS_10BasicBlockEPKNS_4TypeES5_PNS_5ValueES7_PNS_8FunctionERKNS_5TwineE", "llvm::CallInst::CreateMalloc(llvm::BasicBlock*, llvm::Type const*, llvm::Type const*, llvm::Value*, llvm::Value*, llvm::Function*, llvm::Twine const&)"},
    {"_ZN4llvm8CallInst10CreateFreeEPNS_5ValueEPNS_11InstructionE", "llvm::CallInst::CreateFree(llvm::Value*, llvm::Instruction*)"},
    {"_Z10createFreePN4llvm5ValueEPNS_11InstructionEPNS_10BasicBlockE", "createFree(llvm::Value*, llvm::Instruction*, llvm::BasicBlock*)"},
    {"_ZN4llvm8CallInst10CreateFreeEPNS_5ValueEPNS_10BasicBlockE", "llvm::CallInst::CreateFree(llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm10InvokeInst4initEPNS_5ValueEPNS_10BasicBlockES4_PKS2_j", "llvm::InvokeInst::init(llvm::Value*, llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm10InvokeInstC1ERKS0_", "llvm::InvokeInst::InvokeInst(llvm::InvokeInst const&)"},
    {"_ZN4llvm10InvokeInstC2ERKS0_", "llvm::InvokeInst::InvokeInst(llvm::InvokeInst const&)"},
    {"_ZNK4llvm10InvokeInst13getSuccessorVEj", "llvm::InvokeInst::getSuccessorV(unsigned int) const"},
    {"_ZNK4llvm10InvokeInst17getNumSuccessorsVEv", "llvm::InvokeInst::getNumSuccessorsV() const"},
    {"_ZN4llvm10InvokeInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::InvokeInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZNK4llvm10InvokeInst12paramHasAttrEjj", "llvm::InvokeInst::paramHasAttr(unsigned int, unsigned int) const"},
    {"_ZN4llvm10InvokeInst12addAttributeEjj", "llvm::InvokeInst::addAttribute(unsigned int, unsigned int)"},
    {"_ZN4llvm10InvokeInst15removeAttributeEjj", "llvm::InvokeInst::removeAttribute(unsigned int, unsigned int)"},
    {"_ZN4llvm10ReturnInstC1ERKS0_", "llvm::ReturnInst::ReturnInst(llvm::ReturnInst const&)"},
    {"_ZN4llvm10ReturnInstC2ERKS0_", "llvm::ReturnInst::ReturnInst(llvm::ReturnInst const&)"},
    {"_ZN4llvm10ReturnInstC1ERNS_11LLVMContextEPNS_5ValueEPNS_11InstructionE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm10ReturnInstC2ERNS_11LLVMContextEPNS_5ValueEPNS_11InstructionE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm10ReturnInstC1ERNS_11LLVMContextEPNS_5ValueEPNS_10BasicBlockE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm10ReturnInstC2ERNS_11LLVMContextEPNS_5ValueEPNS_10BasicBlockE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm10ReturnInstC1ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZN4llvm10ReturnInstC2ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::ReturnInst::ReturnInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZNK4llvm10ReturnInst17getNumSuccessorsVEv", "llvm::ReturnInst::getNumSuccessorsV() const"},
    {"_ZN4llvm10ReturnInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::ReturnInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZNK4llvm10ReturnInst13getSuccessorVEj", "llvm::ReturnInst::getSuccessorV(unsigned int) const"},
    {"_ZN4llvm10ReturnInstD0Ev", "llvm::ReturnInst::~ReturnInst()"},
    {"_ZN4llvm10ReturnInstD1Ev", "llvm::ReturnInst::~ReturnInst()"},
    {"_ZN4llvm10ReturnInstD2Ev", "llvm::ReturnInst::~ReturnInst()"},
    {"_ZN4llvm10UnwindInstC1ERNS_11LLVMContextEPNS_11InstructionE", "llvm::UnwindInst::UnwindInst(llvm::LLVMContext&, llvm::Instruction*)"},
    {"_ZN4llvm10UnwindInstC2ERNS_11LLVMContextEPNS_11InstructionE", "llvm::UnwindInst::UnwindInst(llvm::LLVMContext&, llvm::Instruction*)"},
    {"_ZN4llvm10UnwindInstC1ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::UnwindInst::UnwindInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZN4llvm10UnwindInstC2ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::UnwindInst::UnwindInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZNK4llvm10UnwindInst17getNumSuccessorsVEv", "llvm::UnwindInst::getNumSuccessorsV() const"},
    {"_ZN4llvm10UnwindInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::UnwindInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZNK4llvm10UnwindInst13getSuccessorVEj", "llvm::UnwindInst::getSuccessorV(unsigned int) const"},
    {"_ZN4llvm15UnreachableInstC1ERNS_11LLVMContextEPNS_11InstructionE", "llvm::UnreachableInst::UnreachableInst(llvm::LLVMContext&, llvm::Instruction*)"},
    {"_ZN4llvm15UnreachableInstC2ERNS_11LLVMContextEPNS_11InstructionE", "llvm::UnreachableInst::UnreachableInst(llvm::LLVMContext&, llvm::Instruction*)"},
    {"_ZN4llvm15UnreachableInstC1ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::UnreachableInst::UnreachableInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZN4llvm15UnreachableInstC2ERNS_11LLVMContextEPNS_10BasicBlockE", "llvm::UnreachableInst::UnreachableInst(llvm::LLVMContext&, llvm::BasicBlock*)"},
    {"_ZNK4llvm15UnreachableInst17getNumSuccessorsVEv", "llvm::UnreachableInst::getNumSuccessorsV() const"},
    {"_ZN4llvm15UnreachableInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::UnreachableInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZNK4llvm15UnreachableInst13getSuccessorVEj", "llvm::UnreachableInst::getSuccessorV(unsigned int) const"},
    {"_ZN4llvm10BranchInst8AssertOKEv", "llvm::BranchInst::AssertOK()"},
    {"_ZN4llvm10BranchInstC1EPNS_10BasicBlockEPNS_11InstructionE", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::Instruction*)"},
    {"_ZN4llvm10BranchInstC2EPNS_10BasicBlockEPNS_11InstructionE", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::Instruction*)"},
    {"_ZN4llvm10BranchInstC1EPNS_10BasicBlockES2_PNS_5ValueEPNS_11InstructionE", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm10BranchInstC2EPNS_10BasicBlockES2_PNS_5ValueEPNS_11InstructionE", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm10BranchInstC1EPNS_10BasicBlockES2_", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BranchInstC2EPNS_10BasicBlockES2_", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BranchInstC1EPNS_10BasicBlockES2_PNS_5ValueES2_", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BranchInstC2EPNS_10BasicBlockES2_PNS_5ValueES2_", "llvm::BranchInst::BranchInst(llvm::BasicBlock*, llvm::BasicBlock*, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm10BranchInstC1ERKS0_", "llvm::BranchInst::BranchInst(llvm::BranchInst const&)"},
    {"_ZN4llvm10BranchInstC2ERKS0_", "llvm::BranchInst::BranchInst(llvm::BranchInst const&)"},
    {"_ZNK4llvm10BranchInst13getSuccessorVEj", "llvm::BranchInst::getSuccessorV(unsigned int) const"},
    {"_ZNK4llvm10BranchInst17getNumSuccessorsVEv", "llvm::BranchInst::getNumSuccessorsV() const"},
    {"_ZN4llvm10BranchInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::BranchInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInst12setAlignmentEj", "llvm::AllocaInst::setAlignment(unsigned int)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeEPNS_5ValueEjRKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeEPNS_5ValueEjRKNS_5TwineEPNS_11InstructionE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10AllocaInstC1EPKNS_4TypeEPNS_5ValueEjRKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstC2EPKNS_4TypeEPNS_5ValueEjRKNS_5TwineEPNS_10BasicBlockE", "llvm::AllocaInst::AllocaInst(llvm::Type const*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10AllocaInstD0Ev", "llvm::AllocaInst::~AllocaInst()"},
    {"_ZN4llvm10AllocaInstD1Ev", "llvm::AllocaInst::~AllocaInst()"},
    {"_ZN4llvm10AllocaInstD2Ev", "llvm::AllocaInst::~AllocaInst()"},
    {"_ZNK4llvm10AllocaInst17isArrayAllocationEv", "llvm::AllocaInst::isArrayAllocation() const"},
    {"_ZNK4llvm10AllocaInst16getAllocatedTypeEv", "llvm::AllocaInst::getAllocatedType() const"},
    {"_ZNK4llvm10AllocaInst14isStaticAllocaEv", "llvm::AllocaInst::isStaticAlloca() const"},
    {"_ZN4llvm8LoadInst8AssertOKEv", "llvm::LoadInst::AssertOK()"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInst12setAlignmentEj", "llvm::LoadInst::setAlignment(unsigned int)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEbPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEbPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEbjPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEbjPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEbjPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEbjPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueERKNS_5TwineEbPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueERKNS_5TwineEbPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, llvm::Twine const&, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueEPKcPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueEPKcPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueEPKcPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueEPKcPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueEPKcbPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, bool, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueEPKcbPNS_11InstructionE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, bool, llvm::Instruction*)"},
    {"_ZN4llvm8LoadInstC1EPNS_5ValueEPKcbPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm8LoadInstC2EPNS_5ValueEPKcbPNS_10BasicBlockE", "llvm::LoadInst::LoadInst(llvm::Value*, char const*, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInst8AssertOKEv", "llvm::StoreInst::AssertOK()"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_PNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_PNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInst12setAlignmentEj", "llvm::StoreInst::setAlignment(unsigned int)"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_PNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_PNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_bPNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_bPNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_bjPNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_bjPNS_11InstructionE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_bjPNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_bjPNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInstC1EPNS_5ValueES2_bPNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm9StoreInstC2EPNS_5ValueES2_bPNS_10BasicBlockE", "llvm::StoreInst::StoreInst(llvm::Value*, llvm::Value*, bool, llvm::BasicBlock*)"},
    {"_ZN4llvm17GetElementPtrInst4initEPNS_5ValueEPKS2_jRKNS_5TwineE", "llvm::GetElementPtrInst::init(llvm::Value*, llvm::Value* const*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm17GetElementPtrInst4initEPNS_5ValueES2_RKNS_5TwineE", "llvm::GetElementPtrInst::init(llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm17GetElementPtrInstC1ERKS0_", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::GetElementPtrInst const&)"},
    {"_ZN4llvm17GetElementPtrInstC2ERKS0_", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::GetElementPtrInst const&)"},
    {"_ZN4llvm17GetElementPtrInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17GetElementPtrInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17GetElementPtrInst14getIndexedTypeEPKNS_4TypeEPNS_5ValueE", "llvm::GetElementPtrInst::getIndexedType(llvm::Type const*, llvm::Value*)"},
    {"_ZN4llvm17GetElementPtrInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17GetElementPtrInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::GetElementPtrInst::GetElementPtrInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17GetElementPtrInst14getIndexedTypeEPKNS_4TypeEPKPNS_5ValueEj", "llvm::GetElementPtrInst::getIndexedType(llvm::Type const*, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm17GetElementPtrInst14getIndexedTypeEPKNS_4TypeEPKPNS_8ConstantEj", "llvm::GetElementPtrInst::getIndexedType(llvm::Type const*, llvm::Constant* const*, unsigned int)"},
    {"_ZN4llvm17GetElementPtrInst14getIndexedTypeEPKNS_4TypeEPKyj", "llvm::GetElementPtrInst::getIndexedType(llvm::Type const*, unsigned long long const*, unsigned int)"},
    {"_ZNK4llvm17GetElementPtrInst17hasAllZeroIndicesEv", "llvm::GetElementPtrInst::hasAllZeroIndices() const"},
    {"_ZNK4llvm17GetElementPtrInst21hasAllConstantIndicesEv", "llvm::GetElementPtrInst::hasAllConstantIndices() const"},
    {"_ZN4llvm17GetElementPtrInst13setIsInBoundsEb", "llvm::GetElementPtrInst::setIsInBounds(bool)"},
    {"_ZNK4llvm17GetElementPtrInst10isInBoundsEv", "llvm::GetElementPtrInst::isInBounds() const"},
    {"_ZN4llvm18ExtractElementInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::ExtractElementInst::ExtractElementInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm18ExtractElementInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_11InstructionE", "llvm::ExtractElementInst::ExtractElementInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm18ExtractElementInstC1EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::ExtractElementInst::ExtractElementInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm18ExtractElementInstC2EPNS_5ValueES2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::ExtractElementInst::ExtractElementInst(llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm18ExtractElementInst15isValidOperandsEPKNS_5ValueES3_", "llvm::ExtractElementInst::isValidOperands(llvm::Value const*, llvm::Value const*)"},
    {"_ZN4llvm17InsertElementInstC1EPNS_5ValueES2_S2_RKNS_5TwineEPNS_11InstructionE", "llvm::InsertElementInst::InsertElementInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17InsertElementInstC2EPNS_5ValueES2_S2_RKNS_5TwineEPNS_11InstructionE", "llvm::InsertElementInst::InsertElementInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17InsertElementInstC1EPNS_5ValueES2_S2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::InsertElementInst::InsertElementInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17InsertElementInstC2EPNS_5ValueES2_S2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::InsertElementInst::InsertElementInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17InsertElementInst15isValidOperandsEPKNS_5ValueES3_S3_", "llvm::InsertElementInst::isValidOperands(llvm::Value const*, llvm::Value const*, llvm::Value const*)"},
    {"_ZN4llvm17ShuffleVectorInstC1EPNS_5ValueES2_S2_RKNS_5TwineEPNS_11InstructionE", "llvm::ShuffleVectorInst::ShuffleVectorInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17ShuffleVectorInstC2EPNS_5ValueES2_S2_RKNS_5TwineEPNS_11InstructionE", "llvm::ShuffleVectorInst::ShuffleVectorInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm17ShuffleVectorInstC1EPNS_5ValueES2_S2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::ShuffleVectorInst::ShuffleVectorInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17ShuffleVectorInstC2EPNS_5ValueES2_S2_RKNS_5TwineEPNS_10BasicBlockE", "llvm::ShuffleVectorInst::ShuffleVectorInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17ShuffleVectorInst15isValidOperandsEPKNS_5ValueES3_S3_", "llvm::ShuffleVectorInst::isValidOperands(llvm::Value const*, llvm::Value const*, llvm::Value const*)"},
    {"_ZNK4llvm17ShuffleVectorInst12getMaskValueEj", "llvm::ShuffleVectorInst::getMaskValue(unsigned int) const"},
    {"_ZN4llvm15InsertValueInst4initEPNS_5ValueES2_PKjjRKNS_5TwineE", "llvm::InsertValueInst::init(llvm::Value*, llvm::Value*, unsigned int const*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm15InsertValueInst4initEPNS_5ValueES2_jRKNS_5TwineE", "llvm::InsertValueInst::init(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm15InsertValueInstC1ERKS0_", "llvm::InsertValueInst::InsertValueInst(llvm::InsertValueInst const&)"},
    {"_ZN4llvm15InsertValueInstC2ERKS0_", "llvm::InsertValueInst::InsertValueInst(llvm::InsertValueInst const&)"},
    {"_ZN4llvm15InsertValueInstC1EPNS_5ValueES2_jRKNS_5TwineEPNS_11InstructionE", "llvm::InsertValueInst::InsertValueInst(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm15InsertValueInstC2EPNS_5ValueES2_jRKNS_5TwineEPNS_11InstructionE", "llvm::InsertValueInst::InsertValueInst(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm15InsertValueInstC1EPNS_5ValueES2_jRKNS_5TwineEPNS_10BasicBlockE", "llvm::InsertValueInst::InsertValueInst(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm15InsertValueInstC2EPNS_5ValueES2_jRKNS_5TwineEPNS_10BasicBlockE", "llvm::InsertValueInst::InsertValueInst(llvm::Value*, llvm::Value*, unsigned int, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm16ExtractValueInst4initEPKjjRKNS_5TwineE", "llvm::ExtractValueInst::init(unsigned int const*, unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm16ExtractValueInst4initEjRKNS_5TwineE", "llvm::ExtractValueInst::init(unsigned int, llvm::Twine const&)"},
    {"_ZN4llvm16ExtractValueInstC1ERKS0_", "llvm::ExtractValueInst::ExtractValueInst(llvm::ExtractValueInst const&)"},
    {"_ZN4llvm16ExtractValueInstC2ERKS0_", "llvm::ExtractValueInst::ExtractValueInst(llvm::ExtractValueInst const&)"},
    {"_ZN4llvm16ExtractValueInst14getIndexedTypeEPKNS_4TypeEPKjj", "llvm::ExtractValueInst::getIndexedType(llvm::Type const*, unsigned int const*, unsigned int)"},
    {"_ZN4llvm16ExtractValueInst14getIndexedTypeEPKNS_4TypeEj", "llvm::ExtractValueInst::getIndexedType(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm14BinaryOperatorC1ENS_11Instruction9BinaryOpsEPNS_5ValueES4_PKNS_4TypeERKNS_5TwineEPS1_", "llvm::BinaryOperator::BinaryOperator(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperatorC2ENS_11Instruction9BinaryOpsEPNS_5ValueES4_PKNS_4TypeERKNS_5TwineEPS1_", "llvm::BinaryOperator::BinaryOperator(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator4initENS_11Instruction9BinaryOpsE", "llvm::BinaryOperator::init(llvm::Instruction::BinaryOps)"},
    {"_ZN4llvm14BinaryOperatorC1ENS_11Instruction9BinaryOpsEPNS_5ValueES4_PKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::BinaryOperator(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperatorC2ENS_11Instruction9BinaryOpsEPNS_5ValueES4_PKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::BinaryOperator(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator6CreateENS_11Instruction9BinaryOpsEPNS_5ValueES4_RKNS_5TwineEPS1_", "llvm::BinaryOperator::Create(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator6CreateENS_11Instruction9BinaryOpsEPNS_5ValueES4_RKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::Create(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator9CreateNegEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::BinaryOperator::CreateNeg(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator9CreateNegEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::CreateNeg(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator12CreateNSWNegEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::BinaryOperator::CreateNSWNeg(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator12CreateNSWNegEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::CreateNSWNeg(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator12CreateNUWNegEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::BinaryOperator::CreateNUWNeg(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator12CreateNUWNegEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::CreateNUWNeg(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator10CreateFNegEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::BinaryOperator::CreateFNeg(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator10CreateFNegEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::CreateFNeg(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator9CreateNotEPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::BinaryOperator::CreateNot(llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator9CreateNotEPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::BinaryOperator::CreateNot(llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm14BinaryOperator5isNegEPKNS_5ValueE", "llvm::BinaryOperator::isNeg(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator6isFNegEPKNS_5ValueE", "llvm::BinaryOperator::isFNeg(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator5isNotEPKNS_5ValueE", "llvm::BinaryOperator::isNot(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator14getNegArgumentEPNS_5ValueE", "llvm::BinaryOperator::getNegArgument(llvm::Value*)"},
    {"_ZN4llvm14BinaryOperator14getNegArgumentEPKNS_5ValueE", "llvm::BinaryOperator::getNegArgument(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator15getFNegArgumentEPNS_5ValueE", "llvm::BinaryOperator::getFNegArgument(llvm::Value*)"},
    {"_ZN4llvm14BinaryOperator15getFNegArgumentEPKNS_5ValueE", "llvm::BinaryOperator::getFNegArgument(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator14getNotArgumentEPNS_5ValueE", "llvm::BinaryOperator::getNotArgument(llvm::Value*)"},
    {"_ZN4llvm14BinaryOperator14getNotArgumentEPKNS_5ValueE", "llvm::BinaryOperator::getNotArgument(llvm::Value const*)"},
    {"_ZN4llvm14BinaryOperator12swapOperandsEv", "llvm::BinaryOperator::swapOperands()"},
    {"_ZN4llvm14BinaryOperator20setHasNoUnsignedWrapEb", "llvm::BinaryOperator::setHasNoUnsignedWrap(bool)"},
    {"_ZN4llvm14BinaryOperator18setHasNoSignedWrapEb", "llvm::BinaryOperator::setHasNoSignedWrap(bool)"},
    {"_ZN4llvm14BinaryOperator10setIsExactEb", "llvm::BinaryOperator::setIsExact(bool)"},
    {"_ZNK4llvm14BinaryOperator17hasNoUnsignedWrapEv", "llvm::BinaryOperator::hasNoUnsignedWrap() const"},
    {"_ZNK4llvm14BinaryOperator15hasNoSignedWrapEv", "llvm::BinaryOperator::hasNoSignedWrap() const"},
    {"_ZNK4llvm14BinaryOperator7isExactEv", "llvm::BinaryOperator::isExact() const"},
    {"_ZNK4llvm8CastInst13isIntegerCastEv", "llvm::CastInst::isIntegerCast() const"},
    {"_ZNK4llvm8CastInst14isLosslessCastEv", "llvm::CastInst::isLosslessCast() const"},
    {"_ZN4llvm8CastInst10isNoopCastENS_11Instruction7CastOpsEPKNS_4TypeES5_S5_", "llvm::CastInst::isNoopCast(llvm::Instruction::CastOps, llvm::Type const*, llvm::Type const*, llvm::Type const*)"},
    {"_ZNK4llvm8CastInst10isNoopCastEPKNS_4TypeE", "llvm::CastInst::isNoopCast(llvm::Type const*) const"},
    {"_ZN4llvm8CastInst20isEliminableCastPairENS_11Instruction7CastOpsES2_PKNS_4TypeES5_S5_S5_", "llvm::CastInst::isEliminableCastPair(llvm::Instruction::CastOps, llvm::Instruction::CastOps, llvm::Type const*, llvm::Type const*, llvm::Type const*, llvm::Type const*)"},
    {"_ZN4llvm8CastInst6CreateENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPS1_", "llvm::CastInst::Create(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm9TruncInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::TruncInst::TruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8ZExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::ZExtInst::ZExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8SExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::SExtInst::SExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm11FPTruncInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPTruncInst::FPTruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm9FPExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPExtInst::FPExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10UIToFPInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::UIToFPInst::UIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10SIToFPInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::SIToFPInst::SIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10FPToUIInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPToUIInst::FPToUIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10FPToSIInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPToSIInst::FPToSIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm12PtrToIntInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::PtrToIntInst::PtrToIntInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm12IntToPtrInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::IntToPtrInst::IntToPtrInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm11BitCastInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::BitCastInst::BitCastInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst6CreateENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::Create(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm9TruncInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::TruncInst::TruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8ZExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::ZExtInst::ZExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8SExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::SExtInst::SExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm11FPTruncInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPTruncInst::FPTruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm9FPExtInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPExtInst::FPExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10UIToFPInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::UIToFPInst::UIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10SIToFPInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::SIToFPInst::SIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10FPToUIInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPToUIInst::FPToUIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10FPToSIInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPToSIInst::FPToSIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm12PtrToIntInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::PtrToIntInst::PtrToIntInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm12IntToPtrInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::IntToPtrInst::IntToPtrInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm11BitCastInstC1EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::BitCastInst::BitCastInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst19CreateZExtOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreateZExtOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst19CreateZExtOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreateZExtOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst19CreateSExtOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreateSExtOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst19CreateSExtOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreateSExtOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst20CreateTruncOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreateTruncOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst20CreateTruncOrBitCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreateTruncOrBitCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst17CreatePointerCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreatePointerCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst17CreatePointerCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreatePointerCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst17CreateIntegerCastEPNS_5ValueEPKNS_4TypeEbRKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreateIntegerCast(llvm::Value*, llvm::Type const*, bool, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst17CreateIntegerCastEPNS_5ValueEPKNS_4TypeEbRKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreateIntegerCast(llvm::Value*, llvm::Type const*, bool, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst12CreateFPCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CreateFPCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInst12CreateFPCastEPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CreateFPCast(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8CastInst10isCastableEPKNS_4TypeES3_", "llvm::CastInst::isCastable(llvm::Type const*, llvm::Type const*)"},
    {"_ZN4llvm8CastInst13getCastOpcodeEPKNS_5ValueEbPKNS_4TypeEb", "llvm::CastInst::getCastOpcode(llvm::Value const*, bool, llvm::Type const*, bool)"},
    {"_ZN4llvm8CastInst11castIsValidENS_11Instruction7CastOpsEPNS_5ValueEPKNS_4TypeE", "llvm::CastInst::castIsValid(llvm::Instruction::CastOps, llvm::Value*, llvm::Type const*)"},
    {"_ZN4llvm9TruncInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::TruncInst::TruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm9TruncInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::TruncInst::TruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8ZExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::ZExtInst::ZExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8ZExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::ZExtInst::ZExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm8SExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::SExtInst::SExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8SExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::SExtInst::SExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm11FPTruncInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPTruncInst::FPTruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm11FPTruncInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPTruncInst::FPTruncInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm9FPExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPExtInst::FPExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm9FPExtInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPExtInst::FPExtInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10UIToFPInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::UIToFPInst::UIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10UIToFPInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::UIToFPInst::UIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10SIToFPInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::SIToFPInst::SIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10SIToFPInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::SIToFPInst::SIToFPInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10FPToUIInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPToUIInst::FPToUIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10FPToUIInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPToUIInst::FPToUIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm10FPToSIInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::FPToSIInst::FPToSIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10FPToSIInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::FPToSIInst::FPToSIInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm12PtrToIntInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::PtrToIntInst::PtrToIntInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm12PtrToIntInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::PtrToIntInst::PtrToIntInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm12IntToPtrInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::IntToPtrInst::IntToPtrInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm12IntToPtrInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::IntToPtrInst::IntToPtrInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm11BitCastInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::BitCastInst::BitCastInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm11BitCastInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_10BasicBlockE", "llvm::BitCastInst::BitCastInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZNK4llvm7CmpInst6AnchorEv", "llvm::CmpInst::Anchor() const"},
    {"_ZN4llvm7CmpInstC1EPKNS_4TypeENS_11Instruction8OtherOpsEtPNS_5ValueES7_RKNS_5TwineEPS4_", "llvm::CmpInst::CmpInst(llvm::Type const*, llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm7CmpInstC2EPKNS_4TypeENS_11Instruction8OtherOpsEtPNS_5ValueES7_RKNS_5TwineEPS4_", "llvm::CmpInst::CmpInst(llvm::Type const*, llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm7CmpInstC1EPKNS_4TypeENS_11Instruction8OtherOpsEtPNS_5ValueES7_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CmpInst::CmpInst(llvm::Type const*, llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm7CmpInstC2EPKNS_4TypeENS_11Instruction8OtherOpsEtPNS_5ValueES7_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CmpInst::CmpInst(llvm::Type const*, llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm7CmpInst6CreateENS_11Instruction8OtherOpsEtPNS_5ValueES4_RKNS_5TwineEPS1_", "llvm::CmpInst::Create(llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm7CmpInst6CreateENS_11Instruction8OtherOpsEtPNS_5ValueES4_RKNS_5TwineEPNS_10BasicBlockE", "llvm::CmpInst::Create(llvm::Instruction::OtherOps, unsigned short, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm7CmpInst12swapOperandsEv", "llvm::CmpInst::swapOperands()"},
    {"_ZNK4llvm7CmpInst13isCommutativeEv", "llvm::CmpInst::isCommutative() const"},
    {"_ZNK4llvm7CmpInst10isEqualityEv", "llvm::CmpInst::isEquality() const"},
    {"_ZN4llvm7CmpInst19getInversePredicateENS0_9PredicateE", "llvm::CmpInst::getInversePredicate(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm8ICmpInst18getSignedPredicateENS_7CmpInst9PredicateE", "llvm::ICmpInst::getSignedPredicate(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm8ICmpInst20getUnsignedPredicateENS_7CmpInst9PredicateE", "llvm::ICmpInst::getUnsignedPredicate(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm8ICmpInst17makeConstantRangeENS_7CmpInst9PredicateERKNS_5APIntE", "llvm::ICmpInst::makeConstantRange(llvm::CmpInst::Predicate, llvm::APInt const&)"},
    {"_ZN4llvm7CmpInst19getSwappedPredicateENS0_9PredicateE", "llvm::CmpInst::getSwappedPredicate(llvm::CmpInst::Predicate)"},
    {"_ZN4llvm7CmpInst10isUnsignedEt", "llvm::CmpInst::isUnsigned(unsigned short)"},
    {"_ZN4llvm7CmpInst8isSignedEt", "llvm::CmpInst::isSigned(unsigned short)"},
    {"_ZN4llvm7CmpInst9isOrderedEt", "llvm::CmpInst::isOrdered(unsigned short)"},
    {"_ZN4llvm7CmpInst11isUnorderedEt", "llvm::CmpInst::isUnordered(unsigned short)"},
    {"_ZN4llvm7CmpInst15isTrueWhenEqualEt", "llvm::CmpInst::isTrueWhenEqual(unsigned short)"},
    {"_ZN4llvm7CmpInst16isFalseWhenEqualEt", "llvm::CmpInst::isFalseWhenEqual(unsigned short)"},
    {"_ZN4llvm10SwitchInst4initEPNS_5ValueEPNS_10BasicBlockEj", "llvm::SwitchInst::init(llvm::Value*, llvm::BasicBlock*, unsigned int)"},
    {"_ZN4llvm10SwitchInstC1EPNS_5ValueEPNS_10BasicBlockEjPNS_11InstructionE", "llvm::SwitchInst::SwitchInst(llvm::Value*, llvm::BasicBlock*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm10SwitchInstC2EPNS_5ValueEPNS_10BasicBlockEjPNS_11InstructionE", "llvm::SwitchInst::SwitchInst(llvm::Value*, llvm::BasicBlock*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm10SwitchInstC1EPNS_5ValueEPNS_10BasicBlockEjS4_", "llvm::SwitchInst::SwitchInst(llvm::Value*, llvm::BasicBlock*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm10SwitchInstC2EPNS_5ValueEPNS_10BasicBlockEjS4_", "llvm::SwitchInst::SwitchInst(llvm::Value*, llvm::BasicBlock*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm10SwitchInstC1ERKS0_", "llvm::SwitchInst::SwitchInst(llvm::SwitchInst const&)"},
    {"_ZN4llvm10SwitchInstC2ERKS0_", "llvm::SwitchInst::SwitchInst(llvm::SwitchInst const&)"},
    {"_ZN4llvm10SwitchInstD0Ev", "llvm::SwitchInst::~SwitchInst()"},
    {"_ZN4llvm10SwitchInstD1Ev", "llvm::SwitchInst::~SwitchInst()"},
    {"_ZN4llvm10SwitchInstD2Ev", "llvm::SwitchInst::~SwitchInst()"},
    {"_ZN4llvm10SwitchInst7addCaseEPNS_11ConstantIntEPNS_10BasicBlockE", "llvm::SwitchInst::addCase(llvm::ConstantInt*, llvm::BasicBlock*)"},
    {"_ZN4llvm10SwitchInst14resizeOperandsEj", "llvm::SwitchInst::resizeOperands(unsigned int)"},
    {"_ZN4llvm10SwitchInst10removeCaseEj", "llvm::SwitchInst::removeCase(unsigned int)"},
    {"_ZNK4llvm10SwitchInst13getSuccessorVEj", "llvm::SwitchInst::getSuccessorV(unsigned int) const"},
    {"_ZNK4llvm10SwitchInst17getNumSuccessorsVEv", "llvm::SwitchInst::getNumSuccessorsV() const"},
    {"_ZN4llvm10SwitchInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::SwitchInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm14IndirectBrInst4initEPNS_5ValueEj", "llvm::IndirectBrInst::init(llvm::Value*, unsigned int)"},
    {"_ZN4llvm14IndirectBrInst14resizeOperandsEj", "llvm::IndirectBrInst::resizeOperands(unsigned int)"},
    {"_ZN4llvm14IndirectBrInstC1EPNS_5ValueEjPNS_11InstructionE", "llvm::IndirectBrInst::IndirectBrInst(llvm::Value*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm14IndirectBrInstC2EPNS_5ValueEjPNS_11InstructionE", "llvm::IndirectBrInst::IndirectBrInst(llvm::Value*, unsigned int, llvm::Instruction*)"},
    {"_ZN4llvm14IndirectBrInstC1EPNS_5ValueEjPNS_10BasicBlockE", "llvm::IndirectBrInst::IndirectBrInst(llvm::Value*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm14IndirectBrInstC2EPNS_5ValueEjPNS_10BasicBlockE", "llvm::IndirectBrInst::IndirectBrInst(llvm::Value*, unsigned int, llvm::BasicBlock*)"},
    {"_ZN4llvm14IndirectBrInstC1ERKS0_", "llvm::IndirectBrInst::IndirectBrInst(llvm::IndirectBrInst const&)"},
    {"_ZN4llvm14IndirectBrInstC2ERKS0_", "llvm::IndirectBrInst::IndirectBrInst(llvm::IndirectBrInst const&)"},
    {"_ZN4llvm14IndirectBrInstD0Ev", "llvm::IndirectBrInst::~IndirectBrInst()"},
    {"_ZN4llvm14IndirectBrInstD1Ev", "llvm::IndirectBrInst::~IndirectBrInst()"},
    {"_ZN4llvm14IndirectBrInstD2Ev", "llvm::IndirectBrInst::~IndirectBrInst()"},
    {"_ZN4llvm14IndirectBrInst14addDestinationEPNS_10BasicBlockE", "llvm::IndirectBrInst::addDestination(llvm::BasicBlock*)"},
    {"_ZN4llvm14IndirectBrInst17removeDestinationEj", "llvm::IndirectBrInst::removeDestination(unsigned int)"},
    {"_ZNK4llvm14IndirectBrInst13getSuccessorVEj", "llvm::IndirectBrInst::getSuccessorV(unsigned int) const"},
    {"_ZNK4llvm14IndirectBrInst17getNumSuccessorsVEv", "llvm::IndirectBrInst::getNumSuccessorsV() const"},
    {"_ZN4llvm14IndirectBrInst13setSuccessorVEjPNS_10BasicBlockE", "llvm::IndirectBrInst::setSuccessorV(unsigned int, llvm::BasicBlock*)"},
    {"_ZNK4llvm17GetElementPtrInst10clone_implEv", "llvm::GetElementPtrInst::clone_impl() const"},
    {"_ZNK4llvm14BinaryOperator10clone_implEv", "llvm::BinaryOperator::clone_impl() const"},
    {"_ZNK4llvm8FCmpInst10clone_implEv", "llvm::FCmpInst::clone_impl() const"},
    {"_ZNK4llvm8ICmpInst10clone_implEv", "llvm::ICmpInst::clone_impl() const"},
    {"_ZNK4llvm16ExtractValueInst10clone_implEv", "llvm::ExtractValueInst::clone_impl() const"},
    {"_ZNK4llvm15InsertValueInst10clone_implEv", "llvm::InsertValueInst::clone_impl() const"},
    {"_ZNK4llvm10AllocaInst10clone_implEv", "llvm::AllocaInst::clone_impl() const"},
    {"_ZNK4llvm8LoadInst10clone_implEv", "llvm::LoadInst::clone_impl() const"},
    {"_ZNK4llvm9StoreInst10clone_implEv", "llvm::StoreInst::clone_impl() const"},
    {"_ZNK4llvm9TruncInst10clone_implEv", "llvm::TruncInst::clone_impl() const"},
    {"_ZNK4llvm8ZExtInst10clone_implEv", "llvm::ZExtInst::clone_impl() const"},
    {"_ZNK4llvm8SExtInst10clone_implEv", "llvm::SExtInst::clone_impl() const"},
    {"_ZNK4llvm11FPTruncInst10clone_implEv", "llvm::FPTruncInst::clone_impl() const"},
    {"_ZNK4llvm9FPExtInst10clone_implEv", "llvm::FPExtInst::clone_impl() const"},
    {"_ZNK4llvm10UIToFPInst10clone_implEv", "llvm::UIToFPInst::clone_impl() const"},
    {"_ZNK4llvm10SIToFPInst10clone_implEv", "llvm::SIToFPInst::clone_impl() const"},
    {"_ZNK4llvm10FPToUIInst10clone_implEv", "llvm::FPToUIInst::clone_impl() const"},
    {"_ZNK4llvm10FPToSIInst10clone_implEv", "llvm::FPToSIInst::clone_impl() const"},
    {"_ZNK4llvm12PtrToIntInst10clone_implEv", "llvm::PtrToIntInst::clone_impl() const"},
    {"_ZNK4llvm12IntToPtrInst10clone_implEv", "llvm::IntToPtrInst::clone_impl() const"},
    {"_ZNK4llvm11BitCastInst10clone_implEv", "llvm::BitCastInst::clone_impl() const"},
    {"_ZNK4llvm8CallInst10clone_implEv", "llvm::CallInst::clone_impl() const"},
    {"_ZNK4llvm10SelectInst10clone_implEv", "llvm::SelectInst::clone_impl() const"},
    {"_ZNK4llvm9VAArgInst10clone_implEv", "llvm::VAArgInst::clone_impl() const"},
    {"_ZNK4llvm18ExtractElementInst10clone_implEv", "llvm::ExtractElementInst::clone_impl() const"},
    {"_ZNK4llvm17InsertElementInst10clone_implEv", "llvm::InsertElementInst::clone_impl() const"},
    {"_ZNK4llvm17ShuffleVectorInst10clone_implEv", "llvm::ShuffleVectorInst::clone_impl() const"},
    {"_ZNK4llvm7PHINode10clone_implEv", "llvm::PHINode::clone_impl() const"},
    {"_ZNK4llvm10ReturnInst10clone_implEv", "llvm::ReturnInst::clone_impl() const"},
    {"_ZNK4llvm10BranchInst10clone_implEv", "llvm::BranchInst::clone_impl() const"},
    {"_ZNK4llvm10SwitchInst10clone_implEv", "llvm::SwitchInst::clone_impl() const"},
    {"_ZNK4llvm14IndirectBrInst10clone_implEv", "llvm::IndirectBrInst::clone_impl() const"},
    {"_ZNK4llvm10InvokeInst10clone_implEv", "llvm::InvokeInst::clone_impl() const"},
    {"_ZNK4llvm10UnwindInst10clone_implEv", "llvm::UnwindInst::clone_impl() const"},
    {"_ZNK4llvm15UnreachableInst10clone_implEv", "llvm::UnreachableInst::clone_impl() const"},
    {"_ZN4llvm8CastInstC2EPKNS_4TypeEjPNS_5ValueERKNS_5TwineEPNS_11InstructionE", "llvm::CastInst::CastInst(llvm::Type const*, unsigned int, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm8CastInstC2EPKNS_4TypeEjPNS_5ValueERKNS_5TwineEPNS_10BasicBlockE", "llvm::CastInst::CastInst(llvm::Type const*, unsigned int, llvm::Value*, llvm::Twine const&, llvm::BasicBlock*)"},
    {"_ZN4llvm17GetElementPtrInstD1Ev", "llvm::GetElementPtrInst::~GetElementPtrInst()"},
    {"_ZN4llvm17GetElementPtrInstD0Ev", "llvm::GetElementPtrInst::~GetElementPtrInst()"},
    {"_ZN4llvm7CmpInstD1Ev", "llvm::CmpInst::~CmpInst()"},
    {"_ZN4llvm7CmpInstD0Ev", "llvm::CmpInst::~CmpInst()"},
    {"_ZN4llvm8ICmpInstD1Ev", "llvm::ICmpInst::~ICmpInst()"},
    {"_ZN4llvm8ICmpInstD0Ev", "llvm::ICmpInst::~ICmpInst()"},
    {"_ZN4llvm8FCmpInstD1Ev", "llvm::FCmpInst::~FCmpInst()"},
    {"_ZN4llvm8FCmpInstD0Ev", "llvm::FCmpInst::~FCmpInst()"},
    {"_ZN4llvm10SelectInstD1Ev", "llvm::SelectInst::~SelectInst()"},
    {"_ZN4llvm10SelectInstD0Ev", "llvm::SelectInst::~SelectInst()"},
    {"_ZN4llvm9VAArgInstD1Ev", "llvm::VAArgInst::~VAArgInst()"},
    {"_ZN4llvm9VAArgInstD0Ev", "llvm::VAArgInst::~VAArgInst()"},
    {"_ZN4llvm18ExtractElementInstD1Ev", "llvm::ExtractElementInst::~ExtractElementInst()"},
    {"_ZN4llvm18ExtractElementInstD0Ev", "llvm::ExtractElementInst::~ExtractElementInst()"},
    {"_ZN4llvm17InsertElementInstD1Ev", "llvm::InsertElementInst::~InsertElementInst()"},
    {"_ZN4llvm17InsertElementInstD0Ev", "llvm::InsertElementInst::~InsertElementInst()"},
    {"_ZN4llvm16ExtractValueInstD1Ev", "llvm::ExtractValueInst::~ExtractValueInst()"},
    {"_ZN4llvm16ExtractValueInstD0Ev", "llvm::ExtractValueInst::~ExtractValueInst()"},
    {"_ZN4llvm15InsertValueInstD1Ev", "llvm::InsertValueInst::~InsertValueInst()"},
    {"_ZN4llvm15InsertValueInstD0Ev", "llvm::InsertValueInst::~InsertValueInst()"},
    {"_ZN4llvm10BranchInstD1Ev", "llvm::BranchInst::~BranchInst()"},
    {"_ZN4llvm10BranchInstD0Ev", "llvm::BranchInst::~BranchInst()"},
    {"_ZN4llvm11BitCastInstD1Ev", "llvm::BitCastInst::~BitCastInst()"},
    {"_ZN4llvm11BitCastInstD0Ev", "llvm::BitCastInst::~BitCastInst()"},
    {"_ZN4llvm10InvokeInstD1Ev", "llvm::InvokeInst::~InvokeInst()"},
    {"_ZN4llvm10InvokeInstD0Ev", "llvm::InvokeInst::~InvokeInst()"},
    {"_ZN4llvm10UnwindInstD1Ev", "llvm::UnwindInst::~UnwindInst()"},
    {"_ZN4llvm10UnwindInstD0Ev", "llvm::UnwindInst::~UnwindInst()"},
    {"_ZN4llvm15UnreachableInstD1Ev", "llvm::UnreachableInst::~UnreachableInst()"},
    {"_ZN4llvm15UnreachableInstD0Ev", "llvm::UnreachableInst::~UnreachableInst()"},
    {"_ZN4llvm8LoadInstD1Ev", "llvm::LoadInst::~LoadInst()"},
    {"_ZN4llvm8LoadInstD0Ev", "llvm::LoadInst::~LoadInst()"},
    {"_ZN4llvm9StoreInstD1Ev", "llvm::StoreInst::~StoreInst()"},
    {"_ZN4llvm9StoreInstD0Ev", "llvm::StoreInst::~StoreInst()"},
    {"_ZN4llvm17ShuffleVectorInstD1Ev", "llvm::ShuffleVectorInst::~ShuffleVectorInst()"},
    {"_ZN4llvm17ShuffleVectorInstD0Ev", "llvm::ShuffleVectorInst::~ShuffleVectorInst()"},
    {"_ZN4llvm14BinaryOperatorD1Ev", "llvm::BinaryOperator::~BinaryOperator()"},
    {"_ZN4llvm14BinaryOperatorD0Ev", "llvm::BinaryOperator::~BinaryOperator()"},
    {"_ZN4llvm9TruncInstD1Ev", "llvm::TruncInst::~TruncInst()"},
    {"_ZN4llvm9TruncInstD0Ev", "llvm::TruncInst::~TruncInst()"},
    {"_ZN4llvm8ZExtInstD1Ev", "llvm::ZExtInst::~ZExtInst()"},
    {"_ZN4llvm8ZExtInstD0Ev", "llvm::ZExtInst::~ZExtInst()"},
    {"_ZN4llvm8SExtInstD1Ev", "llvm::SExtInst::~SExtInst()"},
    {"_ZN4llvm8SExtInstD0Ev", "llvm::SExtInst::~SExtInst()"},
    {"_ZN4llvm11FPTruncInstD1Ev", "llvm::FPTruncInst::~FPTruncInst()"},
    {"_ZN4llvm11FPTruncInstD0Ev", "llvm::FPTruncInst::~FPTruncInst()"},
    {"_ZN4llvm9FPExtInstD1Ev", "llvm::FPExtInst::~FPExtInst()"},
    {"_ZN4llvm9FPExtInstD0Ev", "llvm::FPExtInst::~FPExtInst()"},
    {"_ZN4llvm10UIToFPInstD1Ev", "llvm::UIToFPInst::~UIToFPInst()"},
    {"_ZN4llvm10UIToFPInstD0Ev", "llvm::UIToFPInst::~UIToFPInst()"},
    {"_ZN4llvm10SIToFPInstD1Ev", "llvm::SIToFPInst::~SIToFPInst()"},
    {"_ZN4llvm10SIToFPInstD0Ev", "llvm::SIToFPInst::~SIToFPInst()"},
    {"_ZN4llvm10FPToUIInstD1Ev", "llvm::FPToUIInst::~FPToUIInst()"},
    {"_ZN4llvm10FPToUIInstD0Ev", "llvm::FPToUIInst::~FPToUIInst()"},
    {"_ZN4llvm10FPToSIInstD1Ev", "llvm::FPToSIInst::~FPToSIInst()"},
    {"_ZN4llvm10FPToSIInstD0Ev", "llvm::FPToSIInst::~FPToSIInst()"},
    {"_ZN4llvm12PtrToIntInstD1Ev", "llvm::PtrToIntInst::~PtrToIntInst()"},
    {"_ZN4llvm12PtrToIntInstD0Ev", "llvm::PtrToIntInst::~PtrToIntInst()"},
    {"_ZN4llvm12IntToPtrInstD1Ev", "llvm::IntToPtrInst::~IntToPtrInst()"},
    {"_ZN4llvm12IntToPtrInstD0Ev", "llvm::IntToPtrInst::~IntToPtrInst()"},
    {"_ZNSt6__copyILb0ESt26random_access_iterator_tagE4copyIPN4llvm3UseES5_EET0_T_S7_S6_", "llvm::Use* std::__copy<false, std::random_access_iterator_tag>::copy<llvm::Use*, llvm::Use*>(llvm::Use*, llvm::Use*, llvm::Use*)"},
    {"_ZN4llvm8CastInstD1Ev", "llvm::CastInst::~CastInst()"},
    {"_ZN4llvm8CastInstD0Ev", "llvm::CastInst::~CastInst()"},
    {"_ZN4llvm9VAArgInstC2EPNS_5ValueEPKNS_4TypeERKNS_5TwineEPNS_11InstructionE", "llvm::VAArgInst::VAArgInst(llvm::Value*, llvm::Type const*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10SelectInstC2EPNS_5ValueES2_S2_RKNS_5TwineEPNS_11InstructionE", "llvm::SelectInst::SelectInst(llvm::Value*, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm10SelectInst4initEPNS_5ValueES2_S2_", "llvm::SelectInst::init(llvm::Value*, llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm8FCmpInstC2ERNS_10BasicBlockENS_7CmpInst9PredicateEPNS_5ValueES6_RKNS_5TwineE", "llvm::FCmpInst::FCmpInst(llvm::BasicBlock&, llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm8FCmpInstC2ENS_7CmpInst9PredicateEPNS_5ValueES4_RKNS_5TwineE", "llvm::FCmpInst::FCmpInst(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm8FCmpInstC2EPNS_11InstructionENS_7CmpInst9PredicateEPNS_5ValueES6_RKNS_5TwineE", "llvm::FCmpInst::FCmpInst(llvm::Instruction*, llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm8ICmpInstC2ENS_7CmpInst9PredicateEPNS_5ValueES4_RKNS_5TwineE", "llvm::ICmpInst::ICmpInst(llvm::CmpInst::Predicate, llvm::Value*, llvm::Value*, llvm::Twine const&)"},
    {"_ZN4llvm14BinaryOperator9CreateNUWENS_11Instruction9BinaryOpsEPNS_5ValueES4_RKNS_5TwineEPS1_", "llvm::BinaryOperator::CreateNUW(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm14BinaryOperator9CreateNSWENS_11Instruction9BinaryOpsEPNS_5ValueES4_RKNS_5TwineEPS1_", "llvm::BinaryOperator::CreateNSW(llvm::Instruction::BinaryOps, llvm::Value*, llvm::Value*, llvm::Twine const&, llvm::Instruction*)"},
    {"_ZN4llvm16DbgInfoIntrinsic9StripCastEPNS_5ValueE", "llvm::DbgInfoIntrinsic::StripCast(llvm::Value*)"},
    {"_ZNK4llvm14DbgDeclareInst10getAddressEv", "llvm::DbgDeclareInst::getAddress() const"},
    {"_ZNK4llvm12DbgValueInst8getValueEv", "llvm::DbgValueInst::getValue() const"},
    {"_ZN4llvm12DbgValueInst8getValueEv", "llvm::DbgValueInst::getValue()"},
    {"_ZN4llvm16getGlobalContextEv", "llvm::getGlobalContext()"},
    {"_ZN4llvm11LLVMContextC1Ev", "llvm::LLVMContext::LLVMContext()"},
    {"_ZN4llvm11LLVMContextC2Ev", "llvm::LLVMContext::LLVMContext()"},
    {"_ZNK4llvm11LLVMContext11getMDKindIDENS_9StringRefE", "llvm::LLVMContext::getMDKindID(llvm::StringRef) const"},
    {"_ZN4llvm11LLVMContextD1Ev", "llvm::LLVMContext::~LLVMContext()"},
    {"_ZN4llvm11LLVMContextD2Ev", "llvm::LLVMContext::~LLVMContext()"},
    {"_ZN4llvm11LLVMContext9addModuleEPNS_6ModuleE", "llvm::LLVMContext::addModule(llvm::Module*)"},
    {"_ZN4llvm11LLVMContext12removeModuleEPNS_6ModuleE", "llvm::LLVMContext::removeModule(llvm::Module*)"},
    {"_ZN4llvm11LLVMContext29setInlineAsmDiagnosticHandlerEPFvRKNS_12SMDiagnosticEPvjES4_", "llvm::LLVMContext::setInlineAsmDiagnosticHandler(void (*)(llvm::SMDiagnostic const&, void*, unsigned int), void*)"},
    {"_ZNK4llvm11LLVMContext29getInlineAsmDiagnosticHandlerEv", "llvm::LLVMContext::getInlineAsmDiagnosticHandler() const"},
    {"_ZNK4llvm11LLVMContext29getInlineAsmDiagnosticContextEv", "llvm::LLVMContext::getInlineAsmDiagnosticContext() const"},
    {"_ZN4llvm11LLVMContext9emitErrorENS_9StringRefE", "llvm::LLVMContext::emitError(llvm::StringRef)"},
    {"_ZN4llvm11LLVMContext9emitErrorEjNS_9StringRefE", "llvm::LLVMContext::emitError(unsigned int, llvm::StringRef)"},
    {"_ZN4llvm11LLVMContext9emitErrorEPKNS_11InstructionENS_9StringRefE", "llvm::LLVMContext::emitError(llvm::Instruction const*, llvm::StringRef)"},
    {"_ZNK4llvm11LLVMContext14getMDKindNamesERNS_15SmallVectorImplINS_9StringRefEEE", "llvm::LLVMContext::getMDKindNames(llvm::SmallVectorImpl<llvm::StringRef>&) const"},
    {"_ZN4llvm9StringMapIjNS_15MallocAllocatorEE16GetOrCreateValueIjEERNS_14StringMapEntryIjEENS_9StringRefET_", "llvm::StringMapEntry<unsigned int>& llvm::StringMap<unsigned int, llvm::MallocAllocator>::GetOrCreateValue<unsigned int>(llvm::StringRef, unsigned int)"},
    {"_ZN4llvm15SmallVectorImplINS_9StringRefEE6resizeEj", "llvm::SmallVectorImpl<llvm::StringRef>::resize(unsigned int)"},
    {"_ZN4llvm14object_creatorINS_11LLVMContextEEEPvv", "void* llvm::object_creator<llvm::LLVMContext>()"},
    {"_ZN4llvm14object_deleterINS_11LLVMContextEE4callEPv", "llvm::object_deleter<llvm::LLVMContext>::call(void*)"},
    {"_ZN4llvm15LLVMContextImplC1ERNS_11LLVMContextE", "llvm::LLVMContextImpl::LLVMContextImpl(llvm::LLVMContext&)"},
    {"_ZN4llvm15LLVMContextImplC2ERNS_11LLVMContextE", "llvm::LLVMContextImpl::LLVMContextImpl(llvm::LLVMContext&)"},
    {"_ZN4llvm15LLVMContextImplD1Ev", "llvm::LLVMContextImpl::~LLVMContextImpl()"},
    {"_ZN4llvm15LLVMContextImplD2Ev", "llvm::LLVMContextImpl::~LLVMContextImpl()"},
    {"_ZN4llvm15SmallVectorImplIPNS_6MDNodeEE6appendINS_19SmallPtrSetIteratorIS2_EEEEvT_S7_", "void llvm::SmallVectorImpl<llvm::MDNode*>::append<llvm::SmallPtrSetIterator<llvm::MDNode*> >(llvm::SmallPtrSetIterator<llvm::MDNode*>, llvm::SmallPtrSetIterator<llvm::MDNode*>)"},
    {"_ZN4llvm11IntegerTypeD1Ev", "llvm::IntegerType::~IntegerType()"},
    {"_ZN4llvm4TypeD1Ev", "llvm::Type::~Type()"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EED1Ev", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EED1Ev", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EED1Ev", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EED1Ev", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EED1Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EED1Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EED1Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EED1Ev", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm10FoldingSetINS_6MDNodeEED1Ev", "llvm::FoldingSet<llvm::MDNode>::~FoldingSet()"},
    {"_ZN4llvm16DenseMapIteratorINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EELb0EE23AdvancePastEmptyBucketsEv", "llvm::DenseMapIterator<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*>, false>::AdvancePastEmptyBuckets()"},
    {"_ZN4llvm16DenseMapIteratorINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EELb0EE23AdvancePastEmptyBucketsEv", "llvm::DenseMapIterator<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*>, false>::AdvancePastEmptyBuckets()"},
    {"_ZNSt6vectorIPN4llvm6ModuleESaIS2_EE19_M_range_initializeINS0_19SmallPtrSetIteratorIS2_EEEEvT_S8_St20forward_iterator_tag", "void std::vector<llvm::Module*, std::allocator<llvm::Module*> >::_M_range_initialize<llvm::SmallPtrSetIterator<llvm::Module*> >(llvm::SmallPtrSetIterator<llvm::Module*>, llvm::SmallPtrSetIterator<llvm::Module*>, std::forward_iterator_tag)"},
    {"_ZN4llvm10DebugRecVHD1Ev", "llvm::DebugRecVH::~DebugRecVH()"},
    {"_ZN4llvm10DebugRecVHD0Ev", "llvm::DebugRecVH::~DebugRecVH()"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionENS_11SmallVectorISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELj2EEENS_12DenseMapInfoIS3_EENSB_ISA_EEED2Ev", "llvm::DenseMap<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u>, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> > >::~DenseMap()"},
    {"_ZN4llvm8DenseMapINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EEED2Ev", "llvm::DenseMap<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*> >::~DenseMap()"},
    {"_ZN4llvm8DenseMapINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EEED2Ev", "llvm::DenseMap<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*> >::~DenseMap()"},
    {"_ZN4llvm10FoldingSetINS_6MDNodeEED0Ev", "llvm::FoldingSet<llvm::MDNode>::~FoldingSet()"},
    {"_ZNK4llvm10FoldingSetINS_6MDNodeEE14GetNodeProfileEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::MDNode>::GetNodeProfile(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_6MDNodeEE10NodeEqualsEPNS_14FoldingSetImpl4NodeERKNS_16FoldingSetNodeIDERS6_", "llvm::FoldingSet<llvm::MDNode>::NodeEquals(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID const&, llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm10FoldingSetINS_6MDNodeEE15ComputeNodeHashEPNS_14FoldingSetImpl4NodeERNS_16FoldingSetNodeIDE", "llvm::FoldingSet<llvm::MDNode>::ComputeNodeHash(llvm::FoldingSetImpl::Node*, llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm8DenseMapINS_22DenseMapAPFloatKeyInfo5KeyTyEPNS_10ConstantFPES1_NS_12DenseMapInfoIS4_EEE4initEj", "llvm::DenseMap<llvm::DenseMapAPFloatKeyInfo::KeyTy, llvm::ConstantFP*, llvm::DenseMapAPFloatKeyInfo, llvm::DenseMapInfo<llvm::ConstantFP*> >::init(unsigned int)"},
    {"_ZN4llvm8DenseMapINS_20DenseMapAPIntKeyInfo5KeyTyEPNS_11ConstantIntES1_NS_12DenseMapInfoIS4_EEE4initEj", "llvm::DenseMap<llvm::DenseMapAPIntKeyInfo::KeyTy, llvm::ConstantInt*, llvm::DenseMapAPIntKeyInfo, llvm::DenseMapInfo<llvm::ConstantInt*> >::init(unsigned int)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EED0Ev", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKS1_", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE4dumpEv", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::dump() const"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKS8_IPKS1_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_21ConstantAggregateZeroEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueES6_ISG_ERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_21ConstantAggregateZeroEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_21ConstantAggregateZeroEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE5eraseES6_ISG_ESN_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_21ConstantAggregateZeroEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_21ConstantAggregateZeroEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >::_M_insert_unique(std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_21ConstantAggregateZeroEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm21ConstantAggregateZeroESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_4TypeEcES2_EEESt10_Select1stISD_ESt4lessIS2_ESaISD_EE8_M_eraseEPSt13_Rb_tree_nodeISD_E", "std::_Rb_tree<llvm::ConstantAggregateZero*, std::pair<llvm::ConstantAggregateZero* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > >, std::_Select1st<std::pair<llvm::ConstantAggregateZero* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >, std::less<llvm::ConstantAggregateZero*>, std::allocator<std::pair<llvm::ConstantAggregateZero* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantAggregateZero* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::ConstantAggregateZero*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EED0Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE4dumpEv", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::dump() const"},
    {"_ZN4llvm15ConstantKeyDataINS_13ConstantArrayEE10getValTypeEPS1_", "llvm::ConstantKeyData<llvm::ConstantArray>::getValType(llvm::ConstantArray*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKSD_IPKS6_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_13ConstantArrayEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueES6_ISL_ERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_13ConstantArrayEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm13ConstantArrayESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE16_M_insert_uniqueES5_ISI_ERKSI_", "std::_Rb_tree<llvm::ConstantArray*, std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::ConstantArray*>, std::allocator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm13ConstantArrayESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE16_M_insert_uniqueERKSI_", "std::_Rb_tree<llvm::ConstantArray*, std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::ConstantArray*>, std::allocator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_insert_unique(std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_13ConstantArrayEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE5eraseES6_ISL_ESS_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_13ConstantArrayEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE8_M_eraseEPSt13_Rb_tree_nodeISL_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE4findERSB_", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::find(std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueERKSE_", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::_M_insert_unique(std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE9_M_insertEPSt18_Rb_tree_node_baseSM_RKSE_", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE14_M_create_nodeERKSE_", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::_M_create_node(std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> const&)"},
    {"_ZNSt6vectorIPN4llvm8ConstantESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Constant**, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, llvm::Constant* const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm9ArrayTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_13ConstantArrayEESt10_Select1stISE_ESt4lessISA_ESaISE_EE8_M_eraseEPSt13_Rb_tree_nodeISE_E", "std::_Rb_tree<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*>, std::_Select1st<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >, std::less<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm13ConstantArrayESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_9ArrayTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE8_M_eraseEPSt13_Rb_tree_nodeISI_E", "std::_Rb_tree<llvm::ConstantArray*, std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > >, std::_Select1st<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >, std::less<llvm::ConstantArray*>, std::allocator<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantArray* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::ArrayType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantArray*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EED0Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE4dumpEv", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::dump() const"},
    {"_ZN4llvm15ConstantKeyDataINS_14ConstantStructEE10getValTypeEPS1_", "llvm::ConstantKeyData<llvm::ConstantStruct>::getValType(llvm::ConstantStruct*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKSD_IPKS6_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantStructEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueES6_ISL_ERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantStructEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm14ConstantStructESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE16_M_insert_uniqueES5_ISI_ERKSI_", "std::_Rb_tree<llvm::ConstantStruct*, std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::ConstantStruct*>, std::allocator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm14ConstantStructESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE16_M_insert_uniqueERKSI_", "std::_Rb_tree<llvm::ConstantStruct*, std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::ConstantStruct*>, std::allocator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_insert_unique(std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantStructEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE5eraseES6_ISL_ESS_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantStructEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE8_M_eraseEPSt13_Rb_tree_nodeISL_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE4findERSB_", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::find(std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueERKSE_", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::_M_insert_unique(std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE9_M_insertEPSt18_Rb_tree_node_baseSM_RKSE_", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE14_M_create_nodeERKSE_", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::_M_create_node(std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10StructTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantStructEESt10_Select1stISE_ESt4lessISA_ESaISE_EE8_M_eraseEPSt13_Rb_tree_nodeISE_E", "std::_Rb_tree<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*>, std::_Select1st<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >, std::less<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm14ConstantStructESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_10StructTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE8_M_eraseEPSt13_Rb_tree_nodeISI_E", "std::_Rb_tree<llvm::ConstantStruct*, std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > >, std::_Select1st<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >, std::less<llvm::ConstantStruct*>, std::allocator<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantStruct* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::StructType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantStruct*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EED0Ev", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE4dumpEv", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::dump() const"},
    {"_ZN4llvm15ConstantKeyDataINS_14ConstantVectorEE10getValTypeEPS1_", "llvm::ConstantKeyData<llvm::ConstantVector>::getValType(llvm::ConstantVector*)"},
    {"_ZN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKSD_IPKS6_S5_EPS7_EE", "llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10VectorTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantVectorEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueES6_ISL_ERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10VectorTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantVectorEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE16_M_insert_uniqueERKSL_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10VectorTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantVectorEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE5eraseES6_ISL_ESS_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_10VectorTypeESt6vectorIPNS0_8ConstantESaISC_EEEPNS0_14ConstantVectorEEEESt10_Select1stISL_ESt4lessIS3_ESaISL_EE8_M_eraseEPSt13_Rb_tree_nodeISL_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE4findERSB_", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::find(std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE16_M_insert_uniqueERKSE_", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::_M_insert_unique(std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE9_M_insertEPSt18_Rb_tree_node_baseSM_RKSE_", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE14_M_create_nodeERKSE_", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::_M_create_node(std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm10VectorTypeESt6vectorIPNS1_8ConstantESaIS7_EEES0_IKSA_PNS1_14ConstantVectorEESt10_Select1stISE_ESt4lessISA_ESaISE_EE8_M_eraseEPSt13_Rb_tree_nodeISE_E", "std::_Rb_tree<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > >, std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*>, std::_Select1st<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >, std::less<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > >, std::allocator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm14ConstantVectorESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_10VectorTypeESt6vectorIPNS0_8ConstantESaISB_EEES2_EEESt10_Select1stISI_ESt4lessIS2_ESaISI_EE8_M_eraseEPSt13_Rb_tree_nodeISI_E", "std::_Rb_tree<llvm::ConstantVector*, std::pair<llvm::ConstantVector* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > >, std::_Select1st<std::pair<llvm::ConstantVector* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >, std::less<llvm::ConstantVector*>, std::allocator<std::pair<llvm::ConstantVector* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantVector* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::VectorType const*, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > > const, llvm::ConstantVector*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EED0Ev", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE4dumpEv", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::dump() const"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKS8_IPKS1_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeEcEPNS0_19ConstantPointerNullEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueES6_ISG_ERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeEcEPNS0_19ConstantPointerNullEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeEcEPNS0_19ConstantPointerNullEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE5eraseES6_ISG_ESN_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeEcEPNS0_19ConstantPointerNullEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeEcES0_IKS5_PNS1_19ConstantPointerNullEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<std::pair<llvm::PointerType const*, char>, std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >, std::less<std::pair<llvm::PointerType const*, char> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >::_M_insert_unique(std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeEcES0_IKS5_PNS1_19ConstantPointerNullEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<std::pair<llvm::PointerType const*, char>, std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >, std::less<std::pair<llvm::PointerType const*, char> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm19ConstantPointerNullESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_11PointerTypeEcES2_EEESt10_Select1stISD_ESt4lessIS2_ESaISD_EE8_M_eraseEPSt13_Rb_tree_nodeISD_E", "std::_Rb_tree<llvm::ConstantPointerNull*, std::pair<llvm::ConstantPointerNull* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > >, std::_Select1st<std::pair<llvm::ConstantPointerNull* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >, std::less<llvm::ConstantPointerNull*>, std::allocator<std::pair<llvm::ConstantPointerNull* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantPointerNull* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, char> const, llvm::ConstantPointerNull*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EED0Ev", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKS1_", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE4dumpEv", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::dump() const"},
    {"_ZN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKS8_IPKS1_cEPS2_EE", "llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_10UndefValueEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueES6_ISG_ERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_10UndefValueEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE16_M_insert_uniqueERKSG_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_10UndefValueEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE5eraseES6_ISG_ESN_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeEcEPNS0_10UndefValueEEEESt10_Select1stISG_ESt4lessIS3_ESaISG_EE8_M_eraseEPSt13_Rb_tree_nodeISG_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >*)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_10UndefValueEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE16_M_insert_uniqueERKS9_", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >::_M_insert_unique(std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeEcES0_IKS5_PNS1_10UndefValueEESt10_Select1stIS9_ESt4lessIS5_ESaIS9_EE8_M_eraseEPSt13_Rb_tree_nodeIS9_E", "std::_Rb_tree<std::pair<llvm::Type const*, char>, std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >, std::less<std::pair<llvm::Type const*, char> >, std::allocator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm10UndefValueESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_4TypeEcES2_EEESt10_Select1stISD_ESt4lessIS2_ESaISD_EE8_M_eraseEPSt13_Rb_tree_nodeISD_E", "std::_Rb_tree<llvm::UndefValue*, std::pair<llvm::UndefValue* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > >, std::_Select1st<std::pair<llvm::UndefValue* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >, std::less<llvm::UndefValue*>, std::allocator<std::pair<llvm::UndefValue* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::UndefValue* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, char> const, llvm::UndefValue*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EED0Ev", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKS2_", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE4dumpEv", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::dump() const"},
    {"_ZN4llvm15ConstantKeyDataINS_12ConstantExprEE10getValTypeEPS1_", "llvm::ConstantKeyData<llvm::ConstantExpr>::getValType(llvm::ConstantExpr*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKS9_IPKS2_S1_EPS3_EE", "llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE16_M_insert_uniqueES6_ISH_ERKSH_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE16_M_insert_uniqueERKSH_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE5eraseES6_ISH_ESO_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE8_M_eraseEPSt13_Rb_tree_nodeISH_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >*)"},
    {"_ZNK4llvm14ExprMapKeyTypeltERKS0_", "llvm::ExprMapKeyType::operator<(llvm::ExprMapKeyType const&) const"},
    {"_ZNSt4pairIS_IPKN4llvm4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEC2ERKS5_RKS7_", "std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, llvm::ConstantExpr*>::pair(std::pair<llvm::Type const*, llvm::ExprMapKeyType> const&, llvm::ConstantExpr* const&)"},
    {"_ZNSt4pairIKS_IPKN4llvm4TypeENS0_14ExprMapKeyTypeEEPNS0_12ConstantExprEEC2IS5_S8_EERKS_IT_T0_E", "std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*>::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, llvm::ConstantExpr*>(std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, llvm::ConstantExpr*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeENS1_14ExprMapKeyTypeEES0_IKS6_PNS1_12ConstantExprEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >, std::less<std::pair<llvm::Type const*, llvm::ExprMapKeyType> >, std::allocator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >::_M_insert_unique(std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> const&)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIKS1_IPKN4llvm4TypeENS2_14ExprMapKeyTypeEEPNS2_12ConstantExprEEE9constructEPSB_RKSB_", "__gnu_cxx::new_allocator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >::construct(std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*>*, std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> const&)"},
    {"_ZN4llvm14ExprMapKeyTypeC2EjRKSt6vectorIPNS_8ConstantESaIS3_EEttRKNS_11SmallVectorIjLj4EEE", "llvm::ExprMapKeyType::ExprMapKeyType(unsigned int, std::vector<llvm::Constant*, std::allocator<llvm::Constant*> > const&, unsigned short, unsigned short, llvm::SmallVector<unsigned int, 4u> const&)"},
    {"_ZNSt4pairIPKN4llvm4TypeENS0_14ExprMapKeyTypeEEC2ERKS3_RKS4_", "std::pair<llvm::Type const*, llvm::ExprMapKeyType>::pair(llvm::Type const* const&, llvm::ExprMapKeyType const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm4TypeENS1_14ExprMapKeyTypeEES0_IKS6_PNS1_12ConstantExprEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<std::pair<llvm::Type const*, llvm::ExprMapKeyType>, std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*>, std::_Select1st<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >, std::less<std::pair<llvm::Type const*, llvm::ExprMapKeyType> >, std::allocator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm12ConstantExprESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_4TypeENS0_14ExprMapKeyTypeEES2_EEESt10_Select1stISE_ESt4lessIS2_ESaISE_EE8_M_eraseEPSt13_Rb_tree_nodeISE_E", "std::_Rb_tree<llvm::ConstantExpr*, std::pair<llvm::ConstantExpr* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > >, std::_Select1st<std::pair<llvm::ConstantExpr* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >, std::less<llvm::ConstantExpr*>, std::allocator<std::pair<llvm::ConstantExpr* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ConstantExpr* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::Type const*, llvm::ExprMapKeyType> const, llvm::ConstantExpr*> > > >*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EED0Ev", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::~ConstantUniqueMap()"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE4dumpEv", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::dump() const"},
    {"_ZN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EE21UpdateAbstractTypeMapEPKNS_11DerivedTypeESt17_Rb_tree_iteratorISt4pairIKS9_IPKS2_S1_EPS3_EE", "llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>::UpdateAbstractTypeMap(llvm::DerivedType const*, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeENS0_16InlineAsmKeyTypeEEPNS0_9InlineAsmEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE16_M_insert_uniqueES6_ISH_ERKSH_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeENS0_16InlineAsmKeyTypeEEPNS0_9InlineAsmEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE16_M_insert_uniqueERKSH_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > > >::_M_insert_unique(std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeENS1_16InlineAsmKeyTypeEES0_IKS6_PNS1_9InlineAsmEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE5eraseESt17_Rb_tree_iteratorISA_E", "std::_Rb_tree<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType>, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::less<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >::erase(std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeENS0_16InlineAsmKeyTypeEEPNS0_9InlineAsmEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE5eraseES6_ISH_ESO_", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > > >::erase(std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::_Rb_tree_iterator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >)"},
    {"_ZNSt8_Rb_treeIPKN4llvm11DerivedTypeESt4pairIKS3_St17_Rb_tree_iteratorIS4_IKS4_IPKNS0_11PointerTypeENS0_16InlineAsmKeyTypeEEPNS0_9InlineAsmEEEESt10_Select1stISH_ESt4lessIS3_ESaISH_EE8_M_eraseEPSt13_Rb_tree_nodeISH_E", "std::_Rb_tree<llvm::DerivedType const*, std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >, std::_Select1st<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::less<llvm::DerivedType const*>, std::allocator<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::DerivedType const* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >*)"},
    {"_ZNK4llvm16InlineAsmKeyTypeltERKS0_", "llvm::InlineAsmKeyType::operator<(llvm::InlineAsmKeyType const&) const"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeENS1_16InlineAsmKeyTypeEES0_IKS6_PNS1_9InlineAsmEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE16_M_insert_uniqueERKSA_", "std::_Rb_tree<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType>, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::less<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >::_M_insert_unique(std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeENS1_16InlineAsmKeyTypeEES0_IKS6_PNS1_9InlineAsmEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE9_M_insertEPSt18_Rb_tree_node_baseSI_RKSA_", "std::_Rb_tree<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType>, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::less<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> const&)"},
    {"_ZNSt8_Rb_treeISt4pairIPKN4llvm11PointerTypeENS1_16InlineAsmKeyTypeEES0_IKS6_PNS1_9InlineAsmEESt10_Select1stISA_ESt4lessIS6_ESaISA_EE8_M_eraseEPSt13_Rb_tree_nodeISA_E", "std::_Rb_tree<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType>, std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*>, std::_Select1st<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >, std::less<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> >, std::allocator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> >*)"},
    {"_ZNSt8_Rb_treeIPN4llvm9InlineAsmESt4pairIKS2_St17_Rb_tree_iteratorIS3_IKS3_IPKNS0_11PointerTypeENS0_16InlineAsmKeyTypeEES2_EEESt10_Select1stISE_ESt4lessIS2_ESaISE_EE8_M_eraseEPSt13_Rb_tree_nodeISE_E", "std::_Rb_tree<llvm::InlineAsm*, std::pair<llvm::InlineAsm* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > >, std::_Select1st<std::pair<llvm::InlineAsm* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >, std::less<llvm::InlineAsm*>, std::allocator<std::pair<llvm::InlineAsm* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::InlineAsm* const, std::_Rb_tree_iterator<std::pair<std::pair<llvm::PointerType const*, llvm::InlineAsmKeyType> const, llvm::InlineAsm*> > > >*)"},
    {"_ZN4llvm4TypeD0Ev", "llvm::Type::~Type()"},
    {"_ZN4llvm11IntegerTypeD0Ev", "llvm::IntegerType::~IntegerType()"},
    {"_ZNK4llvm11DerivedType4dumpEv", "llvm::DerivedType::dump() const"},
    {"_ZN4llvm11DerivedTypeD1Ev", "llvm::DerivedType::~DerivedType()"},
    {"_ZN4llvm11DerivedTypeD0Ev", "llvm::DerivedType::~DerivedType()"},
    {"_ZN4llvm11TypeMapBaseD2Ev", "llvm::TypeMapBase::~TypeMapBase()"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm12PATypeHolderEESt10_Select1stIS4_ESt4lessIjESaIS4_EE8_M_eraseEPSt13_Rb_tree_nodeIS4_E", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::PATypeHolder>, std::_Select1st<std::pair<unsigned int const, llvm::PATypeHolder> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<unsigned int const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm12ArrayValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::ArrayValType, std::pair<llvm::ArrayValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >, std::less<llvm::ArrayValType>, std::allocator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm13VectorValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::VectorValType, std::pair<llvm::VectorValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >, std::less<llvm::VectorValType>, std::allocator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm14PointerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::PointerValType, std::pair<llvm::PointerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >, std::less<llvm::PointerValType>, std::allocator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::StructValType const, llvm::PATypeHolder> >*)"},
    {"_ZNSt8_Rb_treeIN4llvm14IntegerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE8_M_eraseEPSt13_Rb_tree_nodeIS5_E", "std::_Rb_tree<llvm::IntegerValType, std::pair<llvm::IntegerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::IntegerValType const, llvm::PATypeHolder> >, std::less<llvm::IntegerValType>, std::allocator<std::pair<llvm::IntegerValType const, llvm::PATypeHolder> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::IntegerValType const, llvm::PATypeHolder> >*)"},
    {"_ZN4llvm8MDStringC1ERNS_11LLVMContextENS_9StringRefE", "llvm::MDString::MDString(llvm::LLVMContext&, llvm::StringRef)"},
    {"_ZN4llvm8MDStringC2ERNS_11LLVMContextENS_9StringRefE", "llvm::MDString::MDString(llvm::LLVMContext&, llvm::StringRef)"},
    {"_ZN4llvm8MDString3getERNS_11LLVMContextENS_9StringRefE", "llvm::MDString::get(llvm::LLVMContext&, llvm::StringRef)"},
    {"_ZN4llvm13MDNodeOperand7deletedEv", "llvm::MDNodeOperand::deleted()"},
    {"_ZN4llvm6MDNode14replaceOperandEPNS_13MDNodeOperandEPNS_5ValueE", "llvm::MDNode::replaceOperand(llvm::MDNodeOperand*, llvm::Value*)"},
    {"_ZN4llvm13MDNodeOperand19allUsesReplacedWithEPNS_5ValueE", "llvm::MDNodeOperand::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm6MDNodeC1ERNS_11LLVMContextEPKPNS_5ValueEjb", "llvm::MDNode::MDNode(llvm::LLVMContext&, llvm::Value* const*, unsigned int, bool)"},
    {"_ZN4llvm6MDNodeC2ERNS_11LLVMContextEPKPNS_5ValueEjb", "llvm::MDNode::MDNode(llvm::LLVMContext&, llvm::Value* const*, unsigned int, bool)"},
    {"_ZN4llvm6MDNodeD0Ev", "llvm::MDNode::~MDNode()"},
    {"_ZN4llvm6MDNodeD1Ev", "llvm::MDNode::~MDNode()"},
    {"_ZN4llvm6MDNodeD2Ev", "llvm::MDNode::~MDNode()"},
    {"_ZNK4llvm6MDNode11getFunctionEv", "llvm::MDNode::getFunction() const"},
    {"_Z19getFunctionForValuePN4llvm5ValueE", "getFunctionForValue(llvm::Value*)"},
    {"_ZNK4llvm6MDNode10getOperandEj", "llvm::MDNode::getOperand(unsigned int) const"},
    {"_ZN4llvm6MDNode7destroyEv", "llvm::MDNode::destroy()"},
    {"_ZN4llvm6MDNode9getMDNodeERNS_11LLVMContextEPKPNS_5ValueEjNS0_17FunctionLocalnessEb", "llvm::MDNode::getMDNode(llvm::LLVMContext&, llvm::Value* const*, unsigned int, llvm::MDNode::FunctionLocalness, bool)"},
    {"_ZN4llvm6MDNode3getERNS_11LLVMContextEPKPNS_5ValueEj", "llvm::MDNode::get(llvm::LLVMContext&, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm6MDNode21getWhenValsUnresolvedERNS_11LLVMContextEPKPNS_5ValueEjb", "llvm::MDNode::getWhenValsUnresolved(llvm::LLVMContext&, llvm::Value* const*, unsigned int, bool)"},
    {"_ZN4llvm6MDNode11getIfExistsERNS_11LLVMContextEPKPNS_5ValueEj", "llvm::MDNode::getIfExists(llvm::LLVMContext&, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm6MDNode12getTemporaryERNS_11LLVMContextEPKPNS_5ValueEj", "llvm::MDNode::getTemporary(llvm::LLVMContext&, llvm::Value* const*, unsigned int)"},
    {"_ZN4llvm6MDNode15deleteTemporaryEPS0_", "llvm::MDNode::deleteTemporary(llvm::MDNode*)"},
    {"_ZNK4llvm6MDNode7ProfileERNS_16FoldingSetNodeIDE", "llvm::MDNode::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm6MDNode15setIsNotUniquedEv", "llvm::MDNode::setIsNotUniqued()"},
    {"_ZN4llvm11NamedMDNodeC1ERKNS_5TwineE", "llvm::NamedMDNode::NamedMDNode(llvm::Twine const&)"},
    {"_ZN4llvm11NamedMDNodeC2ERKNS_5TwineE", "llvm::NamedMDNode::NamedMDNode(llvm::Twine const&)"},
    {"_ZN4llvm11NamedMDNodeD1Ev", "llvm::NamedMDNode::~NamedMDNode()"},
    {"_ZN4llvm11NamedMDNodeD2Ev", "llvm::NamedMDNode::~NamedMDNode()"},
    {"_ZN4llvm11NamedMDNode17dropAllReferencesEv", "llvm::NamedMDNode::dropAllReferences()"},
    {"_ZNK4llvm11NamedMDNode14getNumOperandsEv", "llvm::NamedMDNode::getNumOperands() const"},
    {"_ZNK4llvm11NamedMDNode10getOperandEj", "llvm::NamedMDNode::getOperand(unsigned int) const"},
    {"_ZN4llvm11NamedMDNode10addOperandEPNS_6MDNodeE", "llvm::NamedMDNode::addOperand(llvm::MDNode*)"},
    {"_ZN4llvm11NamedMDNode15eraseFromParentEv", "llvm::NamedMDNode::eraseFromParent()"},
    {"_ZNK4llvm11NamedMDNode7getNameEv", "llvm::NamedMDNode::getName() const"},
    {"_ZN4llvm11Instruction11setMetadataEPKcPNS_6MDNodeE", "llvm::Instruction::setMetadata(char const*, llvm::MDNode*)"},
    {"_ZN4llvm11Instruction11setMetadataEjPNS_6MDNodeE", "llvm::Instruction::setMetadata(unsigned int, llvm::MDNode*)"},
    {"_ZNK4llvm11Instruction15getMetadataImplEPKc", "llvm::Instruction::getMetadataImpl(char const*) const"},
    {"_ZNK4llvm11Instruction15getMetadataImplEj", "llvm::Instruction::getMetadataImpl(unsigned int) const"},
    {"_ZNK4llvm11Instruction18getAllMetadataImplERNS_15SmallVectorImplISt4pairIjPNS_6MDNodeEEEE", "llvm::Instruction::getAllMetadataImpl(llvm::SmallVectorImpl<std::pair<unsigned int, llvm::MDNode*> >&) const"},
    {"_ZNK4llvm11Instruction35getAllMetadataOtherThanDebugLocImplERNS_15SmallVectorImplISt4pairIjPNS_6MDNodeEEEE", "llvm::Instruction::getAllMetadataOtherThanDebugLocImpl(llvm::SmallVectorImpl<std::pair<unsigned int, llvm::MDNode*> >&) const"},
    {"_ZN4llvm11Instruction24clearMetadataHashEntriesEv", "llvm::Instruction::clearMetadataHashEntries()"},
    {"_ZN4llvm25array_pod_sort_comparatorISt4pairIjPNS_6MDNodeEEEEiPKvS6_", "int llvm::array_pod_sort_comparator<std::pair<unsigned int, llvm::MDNode*> >(void const*, void const*)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionENS_11SmallVectorISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELj2EEENS_12DenseMapInfoIS3_EENSB_ISA_EEE5eraseERKS3_", "llvm::DenseMap<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u>, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> > >::erase(llvm::Instruction const* const&)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIjPNS_6MDNodeEEE6appendIPKS1_IjNS_10TrackingVHIS2_EEEEEvT_SC_", "void llvm::SmallVectorImpl<std::pair<unsigned int, llvm::MDNode*> >::append<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> > const*>(std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> > const*, std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> > const*)"},
    {"_ZN4llvm13MDNodeOperandD1Ev", "llvm::MDNodeOperand::~MDNodeOperand()"},
    {"_ZN4llvm13MDNodeOperandD0Ev", "llvm::MDNodeOperand::~MDNodeOperand()"},
    {"_ZN4llvm8MDStringD1Ev", "llvm::MDString::~MDString()"},
    {"_ZN4llvm8MDStringD0Ev", "llvm::MDString::~MDString()"},
    {"_ZN4llvm23SmallVectorTemplateBaseISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, false>::grow(unsigned long)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionENS_11SmallVectorISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELj2EEENS_12DenseMapInfoIS3_EENSB_ISA_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u>, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> > >::FindAndConstruct(llvm::Instruction const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionENS_11SmallVectorISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELj2EEENS_12DenseMapInfoIS3_EENSB_ISA_EEE16InsertIntoBucketERKS3_RKSA_PS5_IS3_SA_E", "llvm::DenseMap<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u>, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> > >::InsertIntoBucket(llvm::Instruction const* const&, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> const&, std::pair<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> >*)"},
    {"_ZN4llvm8DenseMapIPKNS_11InstructionENS_11SmallVectorISt4pairIjNS_10TrackingVHINS_6MDNodeEEEELj2EEENS_12DenseMapInfoIS3_EENSB_ISA_EEE4growEj", "llvm::DenseMap<llvm::Instruction const*, llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u>, llvm::DenseMapInfo<llvm::Instruction const*>, llvm::DenseMapInfo<llvm::SmallVector<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> >, 2u> > >::grow(unsigned int)"},
    {"_ZN4llvm15SmallVectorImplISt4pairIjNS_10TrackingVHINS_6MDNodeEEEEEaSERKS6_", "llvm::SmallVectorImpl<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> > >::operator=(llvm::SmallVectorImpl<std::pair<unsigned int, llvm::TrackingVH<llvm::MDNode> > > const&)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_10TrackingVHINS_6MDNodeEEELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::TrackingVH<llvm::MDNode>, false>::grow(unsigned long)"},
    {"_ZN4llvm9StringMapIPNS_8MDStringENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::MDString*>& llvm::StringMap<llvm::MDString*, llvm::MallocAllocator>::GetOrCreateValue<llvm::MDString*>(llvm::StringRef, llvm::MDString*)"},
    {"_ZN4llvm10CallbackVH7deletedEv", "llvm::CallbackVH::deleted()"},
    {"_ZN4llvm10CallbackVH19allUsesReplacedWithEPNS_5ValueE", "llvm::CallbackVH::allUsesReplacedWith(llvm::Value*)"},
    {"_ZN4llvm12ilist_traitsINS_14GlobalVariableEE14createSentinelEv", "llvm::ilist_traits<llvm::GlobalVariable>::createSentinel()"},
    {"_ZN4llvm12ilist_traitsINS_11GlobalAliasEE14createSentinelEv", "llvm::ilist_traits<llvm::GlobalAlias>::createSentinel()"},
    {"_ZN4llvm6ModuleC1ENS_9StringRefERNS_11LLVMContextE", "llvm::Module::Module(llvm::StringRef, llvm::LLVMContext&)"},
    {"_ZN4llvm6ModuleC2ENS_9StringRefERNS_11LLVMContextE", "llvm::Module::Module(llvm::StringRef, llvm::LLVMContext&)"},
    {"_ZN4llvm6ModuleD1Ev", "llvm::Module::~Module()"},
    {"_ZN4llvm6ModuleD2Ev", "llvm::Module::~Module()"},
    {"_ZN4llvm6Module17dropAllReferencesEv", "llvm::Module::dropAllReferences()"},
    {"_ZNK4llvm6Module13getEndiannessEv", "llvm::Module::getEndianness() const"},
    {"_ZNK4llvm6Module14getPointerSizeEv", "llvm::Module::getPointerSize() const"},
    {"_ZNK4llvm6Module13getNamedValueENS_9StringRefE", "llvm::Module::getNamedValue(llvm::StringRef) const"},
    {"_ZNK4llvm6Module11getMDKindIDENS_9StringRefE", "llvm::Module::getMDKindID(llvm::StringRef) const"},
    {"_ZNK4llvm6Module14getMDKindNamesERNS_15SmallVectorImplINS_9StringRefEEE", "llvm::Module::getMDKindNames(llvm::SmallVectorImpl<llvm::StringRef>&) const"},
    {"_ZN4llvm6Module19getOrInsertFunctionENS_9StringRefEPKNS_12FunctionTypeENS_11AttrListPtrE", "llvm::Module::getOrInsertFunction(llvm::StringRef, llvm::FunctionType const*, llvm::AttrListPtr)"},
    {"_ZN4llvm6Module19getOrInsertFunctionENS_9StringRefEPKNS_12FunctionTypeE", "llvm::Module::getOrInsertFunction(llvm::StringRef, llvm::FunctionType const*)"},
    {"_ZN4llvm6Module26getOrInsertTargetIntrinsicENS_9StringRefEPKNS_12FunctionTypeENS_11AttrListPtrE", "llvm::Module::getOrInsertTargetIntrinsic(llvm::StringRef, llvm::FunctionType const*, llvm::AttrListPtr)"},
    {"_ZN4llvm6Module19getOrInsertFunctionENS_9StringRefENS_11AttrListPtrEPKNS_4TypeEz", "llvm::Module::getOrInsertFunction(llvm::StringRef, llvm::AttrListPtr, llvm::Type const*, ...)"},
    {"_ZN4llvm6Module19getOrInsertFunctionENS_9StringRefEPKNS_4TypeEz", "llvm::Module::getOrInsertFunction(llvm::StringRef, llvm::Type const*, ...)"},
    {"_ZNK4llvm6Module11getFunctionENS_9StringRefE", "llvm::Module::getFunction(llvm::StringRef) const"},
    {"_ZNK4llvm6Module17getGlobalVariableENS_9StringRefEb", "llvm::Module::getGlobalVariable(llvm::StringRef, bool) const"},
    {"_ZN4llvm6Module17getOrInsertGlobalENS_9StringRefEPKNS_4TypeE", "llvm::Module::getOrInsertGlobal(llvm::StringRef, llvm::Type const*)"},
    {"_ZNK4llvm6Module13getNamedAliasENS_9StringRefE", "llvm::Module::getNamedAlias(llvm::StringRef) const"},
    {"_ZNK4llvm6Module16getNamedMetadataERKNS_5TwineE", "llvm::Module::getNamedMetadata(llvm::Twine const&) const"},
    {"_ZN4llvm6Module24getOrInsertNamedMetadataENS_9StringRefE", "llvm::Module::getOrInsertNamedMetadata(llvm::StringRef)"},
    {"_ZN4llvm6Module18eraseNamedMetadataEPNS_11NamedMDNodeE", "llvm::Module::eraseNamedMetadata(llvm::NamedMDNode*)"},
    {"_ZN4llvm6Module11addTypeNameENS_9StringRefEPKNS_4TypeE", "llvm::Module::addTypeName(llvm::StringRef, llvm::Type const*)"},
    {"_ZNK4llvm6Module13getTypeByNameENS_9StringRefE", "llvm::Module::getTypeByName(llvm::StringRef) const"},
    {"_ZNK4llvm6Module11getTypeNameEPKNS_4TypeE", "llvm::Module::getTypeName(llvm::Type const*) const"},
    {"_ZN4llvm6Module15setMaterializerEPNS_14GVMaterializerE", "llvm::Module::setMaterializer(llvm::GVMaterializer*)"},
    {"_ZNK4llvm6Module16isMaterializableEPKNS_11GlobalValueE", "llvm::Module::isMaterializable(llvm::GlobalValue const*) const"},
    {"_ZNK4llvm6Module18isDematerializableEPKNS_11GlobalValueE", "llvm::Module::isDematerializable(llvm::GlobalValue const*) const"},
    {"_ZN4llvm6Module11MaterializeEPNS_11GlobalValueEPSs", "llvm::Module::Materialize(llvm::GlobalValue*, std::string*)"},
    {"_ZN4llvm6Module13DematerializeEPNS_11GlobalValueE", "llvm::Module::Dematerialize(llvm::GlobalValue*)"},
    {"_ZN4llvm6Module14MaterializeAllEPSs", "llvm::Module::MaterializeAll(std::string*)"},
    {"_ZN4llvm6Module25MaterializeAllPermanentlyEPSs", "llvm::Module::MaterializeAllPermanently(std::string*)"},
    {"_ZN4llvm6Module10addLibraryENS_9StringRefE", "llvm::Module::addLibrary(llvm::StringRef)"},
    {"_ZN4llvm6Module13removeLibraryENS_9StringRefE", "llvm::Module::removeLibrary(llvm::StringRef)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEEC1Ev", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEEC2Ev", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::getList(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::getSymTab(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::addNodeToList(llvm::GlobalVariable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::removeNodeFromList(llvm::GlobalVariable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::transferNodesFromList(llvm::ilist_traits<llvm::GlobalVariable>&, llvm::ilist_iterator<llvm::GlobalVariable>, llvm::ilist_iterator<llvm::GlobalVariable>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_14GlobalVariableENS_6ModuleEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::GlobalVariable, llvm::Module>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEEC1Ev", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEEC2Ev", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::getList(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::getSymTab(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::addNodeToList(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::removeNodeFromList(llvm::Function*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::transferNodesFromList(llvm::ilist_traits<llvm::Function>&, llvm::ilist_iterator<llvm::Function>, llvm::ilist_iterator<llvm::Function>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_8FunctionENS_6ModuleEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::Function, llvm::Module>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEEC1Ev", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEEC2Ev", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::SymbolTableListTraits()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE12getListOwnerEv", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::getListOwner()"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE7getListEPS2_", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::getList(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE9getSymTabEPS2_", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::getSymTab(llvm::Module*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE5toPtrERNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::toPtr(llvm::ValueSymbolTable&)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE13addNodeToListEPS1_", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::addNodeToList(llvm::GlobalAlias*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE18removeNodeFromListEPS1_", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::removeNodeFromList(llvm::GlobalAlias*)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE21transferNodesFromListERNS_12ilist_traitsIS1_EENS_14ilist_iteratorIS1_EES8_", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::transferNodesFromList(llvm::ilist_traits<llvm::GlobalAlias>&, llvm::ilist_iterator<llvm::GlobalAlias>, llvm::ilist_iterator<llvm::GlobalAlias>)"},
    {"_ZN4llvm21SymbolTableListTraitsINS_11GlobalAliasENS_6ModuleEE5toPtrEPNS_16ValueSymbolTableE", "llvm::SymbolTableListTraits<llvm::GlobalAlias, llvm::Module>::toPtr(llvm::ValueSymbolTable*)"},
    {"_ZN4llvm9StringMapIPNS_11NamedMDNodeENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::NamedMDNode*>& llvm::StringMap<llvm::NamedMDNode*, llvm::MallocAllocator>::GetOrCreateValue<llvm::NamedMDNode*>(llvm::StringRef, llvm::NamedMDNode*)"},
    {"_ZNSt6vectorIPKN4llvm4TypeESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::Type const*, std::allocator<llvm::Type const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Type const**, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > >, llvm::Type const* const&)"},
    {"_ZN4llvm21ilist_sentinel_traitsINS_11GlobalAliasEE10ensureHeadERPS1_", "llvm::ilist_sentinel_traits<llvm::GlobalAlias>::ensureHead(llvm::GlobalAlias*&)"},
    {"_ZN4llvm6iplistINS_11GlobalAliasENS_12ilist_traitsIS1_EEE5eraseENS_14ilist_iteratorIS1_EE", "llvm::iplist<llvm::GlobalAlias, llvm::ilist_traits<llvm::GlobalAlias> >::erase(llvm::ilist_iterator<llvm::GlobalAlias>)"},
    {"_ZN4llvm6iplistINS_8FunctionENS_12ilist_traitsIS1_EEE5eraseENS_14ilist_iteratorIS1_EE", "llvm::iplist<llvm::Function, llvm::ilist_traits<llvm::Function> >::erase(llvm::ilist_iterator<llvm::Function>)"},
    {"_ZN4llvm21ilist_sentinel_traitsINS_14GlobalVariableEE10ensureHeadERPS1_", "llvm::ilist_sentinel_traits<llvm::GlobalVariable>::ensureHead(llvm::GlobalVariable*&)"},
    {"_ZN4llvm6iplistINS_14GlobalVariableENS_12ilist_traitsIS1_EEE5eraseENS_14ilist_iteratorIS1_EE", "llvm::iplist<llvm::GlobalVariable, llvm::ilist_traits<llvm::GlobalVariable> >::erase(llvm::ilist_iterator<llvm::GlobalVariable>)"},
    {"_ZN4llvm6iplistINS_11GlobalAliasENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::GlobalAlias, llvm::ilist_traits<llvm::GlobalAlias> >::~iplist()"},
    {"_ZN4llvm6iplistINS_14GlobalVariableENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::GlobalVariable, llvm::ilist_traits<llvm::GlobalVariable> >::~iplist()"},
    {"_ZN4llvm4PassC1ENS_8PassKindERc", "llvm::Pass::Pass(llvm::PassKind, char&)"},
    {"_ZN4llvm4PassC2ENS_8PassKindERc", "llvm::Pass::Pass(llvm::PassKind, char&)"},
    {"_ZN4llvm4PassD0Ev", "llvm::Pass::~Pass()"},
    {"_ZN4llvm4PassD1Ev", "llvm::Pass::~Pass()"},
    {"_ZN4llvm4PassD2Ev", "llvm::Pass::~Pass()"},
    {"_ZN4llvm10ModulePassD0Ev", "llvm::ModulePass::~ModulePass()"},
    {"_ZN4llvm10ModulePassD1Ev", "llvm::ModulePass::~ModulePass()"},
    {"_ZN4llvm10ModulePassD2Ev", "llvm::ModulePass::~ModulePass()"},
    {"_ZNK4llvm10ModulePass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::ModulePass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZNK4llvm10ModulePass27getPotentialPassManagerTypeEv", "llvm::ModulePass::getPotentialPassManagerType() const"},
    {"_ZNK4llvm4Pass22mustPreserveAnalysisIDERc", "llvm::Pass::mustPreserveAnalysisID(char&) const"},
    {"_ZN4llvm4Pass17dumpPassStructureEj", "llvm::Pass::dumpPassStructure(unsigned int)"},
    {"_ZNK4llvm4Pass11getPassNameEv", "llvm::Pass::getPassName() const"},
    {"_ZN4llvm4Pass18preparePassManagerERNS_7PMStackE", "llvm::Pass::preparePassManager(llvm::PMStack&)"},
    {"_ZNK4llvm4Pass27getPotentialPassManagerTypeEv", "llvm::Pass::getPotentialPassManagerType() const"},
    {"_ZNK4llvm4Pass16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::Pass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm4Pass13releaseMemoryEv", "llvm::Pass::releaseMemory()"},
    {"_ZNK4llvm4Pass14verifyAnalysisEv", "llvm::Pass::verifyAnalysis() const"},
    {"_ZN4llvm4Pass26getAdjustedAnalysisPointerEPKv", "llvm::Pass::getAdjustedAnalysisPointer(void const*)"},
    {"_ZN4llvm4Pass18getAsImmutablePassEv", "llvm::Pass::getAsImmutablePass()"},
    {"_ZN4llvm4Pass18getAsPMDataManagerEv", "llvm::Pass::getAsPMDataManager()"},
    {"_ZN4llvm4Pass11setResolverEPNS_16AnalysisResolverE", "llvm::Pass::setResolver(llvm::AnalysisResolver*)"},
    {"_ZNK4llvm4Pass5printERNS_11raw_ostreamEPKNS_6ModuleE", "llvm::Pass::print(llvm::raw_ostream&, llvm::Module const*) const"},
    {"_ZNK4llvm4Pass4dumpEv", "llvm::Pass::dump() const"},
    {"_ZN4llvm13ImmutablePassD0Ev", "llvm::ImmutablePass::~ImmutablePass()"},
    {"_ZN4llvm13ImmutablePassD1Ev", "llvm::ImmutablePass::~ImmutablePass()"},
    {"_ZN4llvm13ImmutablePassD2Ev", "llvm::ImmutablePass::~ImmutablePass()"},
    {"_ZN4llvm13ImmutablePass14initializePassEv", "llvm::ImmutablePass::initializePass()"},
    {"_ZNK4llvm12FunctionPass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::FunctionPass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZN4llvm12FunctionPass16doInitializationERNS_6ModuleE", "llvm::FunctionPass::doInitialization(llvm::Module&)"},
    {"_ZN4llvm12FunctionPass14doFinalizationERNS_6ModuleE", "llvm::FunctionPass::doFinalization(llvm::Module&)"},
    {"_ZNK4llvm12FunctionPass27getPotentialPassManagerTypeEv", "llvm::FunctionPass::getPotentialPassManagerType() const"},
    {"_ZNK4llvm14BasicBlockPass17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::BasicBlockPass::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZN4llvm14BasicBlockPass16doInitializationERNS_6ModuleE", "llvm::BasicBlockPass::doInitialization(llvm::Module&)"},
    {"_ZN4llvm14BasicBlockPass16doInitializationERNS_8FunctionE", "llvm::BasicBlockPass::doInitialization(llvm::Function&)"},
    {"_ZN4llvm14BasicBlockPass14doFinalizationERNS_8FunctionE", "llvm::BasicBlockPass::doFinalization(llvm::Function&)"},
    {"_ZN4llvm14BasicBlockPass14doFinalizationERNS_6ModuleE", "llvm::BasicBlockPass::doFinalization(llvm::Module&)"},
    {"_ZNK4llvm14BasicBlockPass27getPotentialPassManagerTypeEv", "llvm::BasicBlockPass::getPotentialPassManagerType() const"},
    {"_ZN4llvm4Pass14lookupPassInfoEPKv", "llvm::Pass::lookupPassInfo(void const*)"},
    {"_ZN4llvm4Pass14lookupPassInfoENS_9StringRefE", "llvm::Pass::lookupPassInfo(llvm::StringRef)"},
    {"_ZNK4llvm8PassInfo10createPassEv", "llvm::PassInfo::createPass() const"},
    {"_ZN4llvm14RegisterAGBaseC1EPKcPKvS4_b", "llvm::RegisterAGBase::RegisterAGBase(char const*, void const*, void const*, bool)"},
    {"_ZN4llvm14RegisterAGBaseC2EPKcPKvS4_b", "llvm::RegisterAGBase::RegisterAGBase(char const*, void const*, void const*, bool)"},
    {"_ZN4llvm24PassRegistrationListenerC1Ev", "llvm::PassRegistrationListener::PassRegistrationListener()"},
    {"_ZN4llvm24PassRegistrationListenerD0Ev", "llvm::PassRegistrationListener::~PassRegistrationListener()"},
    {"_ZN4llvm24PassRegistrationListenerD1Ev", "llvm::PassRegistrationListener::~PassRegistrationListener()"},
    {"_ZN4llvm24PassRegistrationListenerD2Ev", "llvm::PassRegistrationListener::~PassRegistrationListener()"},
    {"_ZThn16_N4llvm14PassNameParserD0Ev", "non-virtual thunk to llvm::PassNameParser::~PassNameParser()"},
    {"_ZN4llvm14PassNameParserD0Ev", "llvm::PassNameParser::~PassNameParser()"},
    {"_ZN4llvm14PassNameParserD1Ev", "llvm::PassNameParser::~PassNameParser()"},
    {"_ZThn16_N4llvm14PassNameParserD1Ev", "non-virtual thunk to llvm::PassNameParser::~PassNameParser()"},
    {"_ZN4llvm14PassNameParserD2Ev", "llvm::PassNameParser::~PassNameParser()"},
    {"_ZN4llvm13AnalysisUsage15setPreservesCFGEv", "llvm::AnalysisUsage::setPreservesCFG()"},
    {"_ZN12_GLOBAL__N_116GetCFGOnlyPassesD1Ev", "(anonymous namespace)::GetCFGOnlyPasses::~GetCFGOnlyPasses()"},
    {"_ZN4llvm13AnalysisUsage12addPreservedENS_9StringRefE", "llvm::AnalysisUsage::addPreserved(llvm::StringRef)"},
    {"_ZN4llvm13AnalysisUsage13addRequiredIDEPKv", "llvm::AnalysisUsage::addRequiredID(void const*)"},
    {"_ZN4llvm13AnalysisUsage13addRequiredIDERc", "llvm::AnalysisUsage::addRequiredID(char&)"},
    {"_ZN4llvm13AnalysisUsage23addRequiredTransitiveIDERc", "llvm::AnalysisUsage::addRequiredTransitiveID(char&)"},
    {"_ZN12_GLOBAL__N_116GetCFGOnlyPassesD0Ev", "(anonymous namespace)::GetCFGOnlyPasses::~GetCFGOnlyPasses()"},
    {"_ZN12_GLOBAL__N_116GetCFGOnlyPasses13passEnumerateEPKN4llvm8PassInfoE", "(anonymous namespace)::GetCFGOnlyPasses::passEnumerate(llvm::PassInfo const*)"},
    {"_ZN4llvm4Pass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::Pass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZN4llvm14BasicBlockPassD1Ev", "llvm::BasicBlockPass::~BasicBlockPass()"},
    {"_ZN4llvm14BasicBlockPassD0Ev", "llvm::BasicBlockPass::~BasicBlockPass()"},
    {"_ZN4llvm24PassRegistrationListener14passRegisteredEPKNS_8PassInfoE", "llvm::PassRegistrationListener::passRegistered(llvm::PassInfo const*)"},
    {"_ZN4llvm24PassRegistrationListener13passEnumerateEPKNS_8PassInfoE", "llvm::PassRegistrationListener::passEnumerate(llvm::PassInfo const*)"},
    {"_ZN4llvm14PassNameParser14passRegisteredEPKNS_8PassInfoE", "llvm::PassNameParser::passRegistered(llvm::PassInfo const*)"},
    {"_ZN4llvm14PassNameParser13passEnumerateEPKNS_8PassInfoE", "llvm::PassNameParser::passEnumerate(llvm::PassInfo const*)"},
    {"_ZNK4llvm14PassNameParser17ignorablePassImplEPKNS_8PassInfoE", "llvm::PassNameParser::ignorablePassImpl(llvm::PassInfo const*) const"},
    {"_ZNK4llvm14PassNameParser15printOptionInfoERKNS_2cl6OptionEm", "llvm::PassNameParser::printOptionInfo(llvm::cl::Option const&, unsigned long) const"},
    {"_ZNK4llvm2cl6parserIPKNS_8PassInfoEE13getNumOptionsEv", "llvm::cl::parser<llvm::PassInfo const*>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserIPKNS_8PassInfoEE9getOptionEj", "llvm::cl::parser<llvm::PassInfo const*>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserIPKNS_8PassInfoEE14getDescriptionEj", "llvm::cl::parser<llvm::PassInfo const*>::getDescription(unsigned int) const"},
    {"_ZThn16_NK4llvm14PassNameParser15printOptionInfoERKNS_2cl6OptionEm", "non-virtual thunk to llvm::PassNameParser::printOptionInfo(llvm::cl::Option const&, unsigned long) const"},
    {"_ZN4llvm14PassNameParser11ValLessThanEPKvS2_", "llvm::PassNameParser::ValLessThan(void const*, void const*)"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserIPKNS_8PassInfoEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::PassInfo const*>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserIPKNS_8PassInfoEED1Ev", "llvm::cl::parser<llvm::PassInfo const*>::~parser()"},
    {"_ZN4llvm2cl6parserIPKNS_8PassInfoEED0Ev", "llvm::cl::parser<llvm::PassInfo const*>::~parser()"},
    {"_ZNK4llvm13PMDataManager31isPassDebuggingExecutionsOrMoreEv", "llvm::PMDataManager::isPassDebuggingExecutionsOrMore() const"},
    {"_ZNK4llvm27PassManagerPrettyStackEntry5printERNS_11raw_ostreamE", "llvm::PassManagerPrettyStackEntry::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm17PMTopLevelManagerC1EPNS_13PMDataManagerE", "llvm::PMTopLevelManager::PMTopLevelManager(llvm::PMDataManager*)"},
    {"_ZN4llvm17PMTopLevelManagerC2EPNS_13PMDataManagerE", "llvm::PMTopLevelManager::PMTopLevelManager(llvm::PMDataManager*)"},
    {"_ZN4llvm7PMStack4pushEPNS_13PMDataManagerE", "llvm::PMStack::push(llvm::PMDataManager*)"},
    {"_ZN4llvm17PMTopLevelManager11setLastUserERKNS_15SmallVectorImplIPNS_4PassEEES3_", "llvm::PMTopLevelManager::setLastUser(llvm::SmallVectorImpl<llvm::Pass*> const&, llvm::Pass*)"},
    {"_ZN4llvm17PMTopLevelManager17findAnalysisUsageEPNS_4PassE", "llvm::PMTopLevelManager::findAnalysisUsage(llvm::Pass*)"},
    {"_ZN4llvm17PMTopLevelManager16findAnalysisPassEPKv", "llvm::PMTopLevelManager::findAnalysisPass(void const*)"},
    {"_ZN4llvm17PMTopLevelManager15collectLastUsesERNS_15SmallVectorImplIPNS_4PassEEES3_", "llvm::PMTopLevelManager::collectLastUses(llvm::SmallVectorImpl<llvm::Pass*>&, llvm::Pass*)"},
    {"_ZN4llvm17PMTopLevelManager12schedulePassEPNS_4PassE", "llvm::PMTopLevelManager::schedulePass(llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager16findAnalysisPassEPKvb", "llvm::PMDataManager::findAnalysisPass(void const*, bool)"},
    {"_ZNK4llvm17PMTopLevelManager10dumpPassesEv", "llvm::PMTopLevelManager::dumpPasses() const"},
    {"_ZNK4llvm17PMTopLevelManager13dumpArgumentsEv", "llvm::PMTopLevelManager::dumpArguments() const"},
    {"_ZNK4llvm13PMDataManager17dumpPassArgumentsEv", "llvm::PMDataManager::dumpPassArguments() const"},
    {"_ZN4llvm17PMTopLevelManager25initializeAllAnalysisInfoEv", "llvm::PMTopLevelManager::initializeAllAnalysisInfo()"},
    {"_ZN4llvm17PMTopLevelManagerD0Ev", "llvm::PMTopLevelManager::~PMTopLevelManager()"},
    {"_ZN4llvm17PMTopLevelManagerD1Ev", "llvm::PMTopLevelManager::~PMTopLevelManager()"},
    {"_ZN4llvm17PMTopLevelManagerD2Ev", "llvm::PMTopLevelManager::~PMTopLevelManager()"},
    {"_ZN4llvm13PMDataManager23recordAvailableAnalysisEPNS_4PassE", "llvm::PMDataManager::recordAvailableAnalysis(llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager27preserveHigherLevelAnalysisEPNS_4PassE", "llvm::PMDataManager::preserveHigherLevelAnalysis(llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager23verifyPreservedAnalysisEPNS_4PassE", "llvm::PMDataManager::verifyPreservedAnalysis(llvm::Pass*)"},
    {"_ZN4llvm12getPassTimerEPNS_4PassE", "llvm::getPassTimer(llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager26removeNotPreservedAnalysisEPNS_4PassE", "llvm::PMDataManager::removeNotPreservedAnalysis(llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager16removeDeadPassesEPNS_4PassENS_9StringRefENS_19PassDebuggingStringE", "llvm::PMDataManager::removeDeadPasses(llvm::Pass*, llvm::StringRef, llvm::PassDebuggingString)"},
    {"_ZN4llvm13PMDataManager8freePassEPNS_4PassENS_9StringRefENS_19PassDebuggingStringE", "llvm::PMDataManager::freePass(llvm::Pass*, llvm::StringRef, llvm::PassDebuggingString)"},
    {"_ZN4llvm13PMDataManager12dumpPassInfoEPNS_4PassENS_19PassDebuggingStringES3_NS_9StringRefE", "llvm::PMDataManager::dumpPassInfo(llvm::Pass*, llvm::PassDebuggingString, llvm::PassDebuggingString, llvm::StringRef)"},
    {"_ZN4llvm13PMDataManager3addEPNS_4PassEb", "llvm::PMDataManager::add(llvm::Pass*, bool)"},
    {"_ZN4llvm13PMDataManager23collectRequiredAnalysisERNS_15SmallVectorImplIPNS_4PassEEERNS1_IPKvEES3_", "llvm::PMDataManager::collectRequiredAnalysis(llvm::SmallVectorImpl<llvm::Pass*>&, llvm::SmallVectorImpl<void const*>&, llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager22initializeAnalysisImplEPNS_4PassE", "llvm::PMDataManager::initializeAnalysisImpl(llvm::Pass*)"},
    {"_ZNK4llvm13PMDataManager12dumpLastUsesEPNS_4PassEj", "llvm::PMDataManager::dumpLastUses(llvm::Pass*, unsigned int) const"},
    {"_ZNK4llvm13PMDataManager15dumpRequiredSetEPKNS_4PassE", "llvm::PMDataManager::dumpRequiredSet(llvm::Pass const*) const"},
    {"_ZNK4llvm13PMDataManager17dumpAnalysisUsageENS_9StringRefEPKNS_4PassERKNS_11SmallVectorIPKvLj32EEE", "llvm::PMDataManager::dumpAnalysisUsage(llvm::StringRef, llvm::Pass const*, llvm::SmallVector<void const*, 32u> const&) const"},
    {"_ZNK4llvm13PMDataManager16dumpPreservedSetEPKNS_4PassE", "llvm::PMDataManager::dumpPreservedSet(llvm::Pass const*) const"},
    {"_ZN4llvm13PMDataManager25addLowerLevelRequiredPassEPNS_4PassES2_", "llvm::PMDataManager::addLowerLevelRequiredPass(llvm::Pass*, llvm::Pass*)"},
    {"_ZN4llvm13PMDataManager15getOnTheFlyPassEPNS_4PassEPKvRNS_8FunctionE", "llvm::PMDataManager::getOnTheFlyPass(llvm::Pass*, void const*, llvm::Function&)"},
    {"_ZN4llvm13PMDataManagerD0Ev", "llvm::PMDataManager::~PMDataManager()"},
    {"_ZN4llvm13PMDataManagerD1Ev", "llvm::PMDataManager::~PMDataManager()"},
    {"_ZN4llvm13PMDataManagerD2Ev", "llvm::PMDataManager::~PMDataManager()"},
    {"_ZNK4llvm16AnalysisResolver22getAnalysisIfAvailableEPKvb", "llvm::AnalysisResolver::getAnalysisIfAvailable(void const*, bool) const"},
    {"_ZN4llvm16AnalysisResolver12findImplPassEPNS_4PassEPKvRNS_8FunctionE", "llvm::AnalysisResolver::findImplPass(llvm::Pass*, void const*, llvm::Function&)"},
    {"_ZN4llvm19FunctionPassManagerC1EPNS_6ModuleE", "llvm::FunctionPassManager::FunctionPassManager(llvm::Module*)"},
    {"_ZN4llvm19FunctionPassManagerC2EPNS_6ModuleE", "llvm::FunctionPassManager::FunctionPassManager(llvm::Module*)"},
    {"_ZN4llvm19FunctionPassManagerD0Ev", "llvm::FunctionPassManager::~FunctionPassManager()"},
    {"_ZN4llvm19FunctionPassManagerD1Ev", "llvm::FunctionPassManager::~FunctionPassManager()"},
    {"_ZN4llvm19FunctionPassManagerD2Ev", "llvm::FunctionPassManager::~FunctionPassManager()"},
    {"_ZN4llvm15PassManagerBaseD2Ev", "llvm::PassManagerBase::~PassManagerBase()"},
    {"_ZN4llvm19FunctionPassManager7addImplEPNS_4PassE", "llvm::FunctionPassManager::addImpl(llvm::Pass*)"},
    {"_ZN4llvm19FunctionPassManager3addEPNS_4PassE", "llvm::FunctionPassManager::add(llvm::Pass*)"},
    {"_ZN4llvm19FunctionPassManager3runERNS_8FunctionE", "llvm::FunctionPassManager::run(llvm::Function&)"},
    {"_ZN4llvm23FunctionPassManagerImpl3runERNS_8FunctionE", "llvm::FunctionPassManagerImpl::run(llvm::Function&)"},
    {"_ZN4llvm19FunctionPassManager16doInitializationEv", "llvm::FunctionPassManager::doInitialization()"},
    {"_ZN4llvm23FunctionPassManagerImpl16doInitializationERNS_6ModuleE", "llvm::FunctionPassManagerImpl::doInitialization(llvm::Module&)"},
    {"_ZN4llvm19FunctionPassManager14doFinalizationEv", "llvm::FunctionPassManager::doFinalization()"},
    {"_ZN4llvm23FunctionPassManagerImpl14doFinalizationERNS_6ModuleE", "llvm::FunctionPassManagerImpl::doFinalization(llvm::Module&)"},
    {"_ZN4llvm13FPPassManager16doInitializationERNS_6ModuleE", "llvm::FPPassManager::doInitialization(llvm::Module&)"},
    {"_ZN4llvm13FPPassManager14doFinalizationERNS_6ModuleE", "llvm::FPPassManager::doFinalization(llvm::Module&)"},
    {"_ZN4llvm13FPPassManager7cleanupEv", "llvm::FPPassManager::cleanup()"},
    {"_ZN4llvm23FunctionPassManagerImpl21releaseMemoryOnTheFlyEv", "llvm::FunctionPassManagerImpl::releaseMemoryOnTheFly()"},
    {"_ZN4llvm13FPPassManager13runOnFunctionERNS_8FunctionE", "llvm::FPPassManager::runOnFunction(llvm::Function&)"},
    {"_ZN4llvm13FPPassManager17dumpPassStructureEj", "llvm::FPPassManager::dumpPassStructure(unsigned int)"},
    {"_ZN4llvm13FPPassManager11runOnModuleERNS_6ModuleE", "llvm::FPPassManager::runOnModule(llvm::Module&)"},
    {"_ZN4llvm13MPPassManager11runOnModuleERNS_6ModuleE", "llvm::MPPassManager::runOnModule(llvm::Module&)"},
    {"_ZThn32_N4llvm13MPPassManager25addLowerLevelRequiredPassEPNS_4PassES2_", "non-virtual thunk to llvm::MPPassManager::addLowerLevelRequiredPass(llvm::Pass*, llvm::Pass*)"},
    {"_ZN4llvm13MPPassManager25addLowerLevelRequiredPassEPNS_4PassES2_", "llvm::MPPassManager::addLowerLevelRequiredPass(llvm::Pass*, llvm::Pass*)"},
    {"_ZThn32_N4llvm13MPPassManager15getOnTheFlyPassEPNS_4PassEPKvRNS_8FunctionE", "non-virtual thunk to llvm::MPPassManager::getOnTheFlyPass(llvm::Pass*, void const*, llvm::Function&)"},
    {"_ZN4llvm13MPPassManager15getOnTheFlyPassEPNS_4PassEPKvRNS_8FunctionE", "llvm::MPPassManager::getOnTheFlyPass(llvm::Pass*, void const*, llvm::Function&)"},
    {"_ZN4llvm15PassManagerImpl3runERNS_6ModuleE", "llvm::PassManagerImpl::run(llvm::Module&)"},
    {"_ZN4llvm11PassManagerC1Ev", "llvm::PassManager::PassManager()"},
    {"_ZN4llvm11PassManagerC2Ev", "llvm::PassManager::PassManager()"},
    {"_ZN4llvm11PassManagerD0Ev", "llvm::PassManager::~PassManager()"},
    {"_ZN4llvm11PassManagerD1Ev", "llvm::PassManager::~PassManager()"},
    {"_ZN4llvm11PassManagerD2Ev", "llvm::PassManager::~PassManager()"},
    {"_ZN4llvm11PassManager7addImplEPNS_4PassE", "llvm::PassManager::addImpl(llvm::Pass*)"},
    {"_ZN4llvm11PassManager3addEPNS_4PassE", "llvm::PassManager::add(llvm::Pass*)"},
    {"_ZN4llvm11PassManager3runERNS_6ModuleE", "llvm::PassManager::run(llvm::Module&)"},
    {"_ZN4llvm7PMStack3popEv", "llvm::PMStack::pop()"},
    {"_ZNK4llvm7PMStack4dumpEv", "llvm::PMStack::dump() const"},
    {"_ZN4llvm10ModulePass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::ModulePass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZN4llvm12FunctionPass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::FunctionPass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZN4llvm14BasicBlockPass17assignPassManagerERNS_7PMStackENS_15PassManagerTypeE", "llvm::BasicBlockPass::assignPassManager(llvm::PMStack&, llvm::PassManagerType)"},
    {"_ZN4llvm15PassManagerBaseD0Ev", "llvm::PassManagerBase::~PassManagerBase()"},
    {"_ZN4llvm15PassManagerBaseD1Ev", "llvm::PassManagerBase::~PassManagerBase()"},
    {"_ZN12_GLOBAL__N_113BBPassManagerD1Ev", "(anonymous namespace)::BBPassManager::~BBPassManager()"},
    {"_ZN12_GLOBAL__N_113BBPassManagerD0Ev", "(anonymous namespace)::BBPassManager::~BBPassManager()"},
    {"_ZN12_GLOBAL__N_113BBPassManager9getAsPassEv", "(anonymous namespace)::BBPassManager::getAsPass()"},
    {"_ZNK12_GLOBAL__N_113BBPassManager18getPassManagerTypeEv", "(anonymous namespace)::BBPassManager::getPassManagerType() const"},
    {"_ZN12_GLOBAL__N_113BBPassManager13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::BBPassManager::runOnFunction(llvm::Function&)"},
    {"_ZNK12_GLOBAL__N_113BBPassManager16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::BBPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_113BBPassManager16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::BBPassManager::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_113BBPassManager14doFinalizationERN4llvm6ModuleE", "(anonymous namespace)::BBPassManager::doFinalization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_113BBPassManager18getAsPMDataManagerEv", "(anonymous namespace)::BBPassManager::getAsPMDataManager()"},
    {"_ZNK12_GLOBAL__N_113BBPassManager11getPassNameEv", "(anonymous namespace)::BBPassManager::getPassName() const"},
    {"_ZN12_GLOBAL__N_113BBPassManager17dumpPassStructureEj", "(anonymous namespace)::BBPassManager::dumpPassStructure(unsigned int)"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManagerD1Ev", "non-virtual thunk to (anonymous namespace)::BBPassManager::~BBPassManager()"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManagerD0Ev", "non-virtual thunk to (anonymous namespace)::BBPassManager::~BBPassManager()"},
    {"_ZThn392_NK12_GLOBAL__N_113BBPassManager11getPassNameEv", "non-virtual thunk to (anonymous namespace)::BBPassManager::getPassName() const"},
    {"_ZThn392_NK12_GLOBAL__N_113BBPassManager16getAnalysisUsageERN4llvm13AnalysisUsageE", "non-virtual thunk to (anonymous namespace)::BBPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManager18getAsPMDataManagerEv", "non-virtual thunk to (anonymous namespace)::BBPassManager::getAsPMDataManager()"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManager17dumpPassStructureEj", "non-virtual thunk to (anonymous namespace)::BBPassManager::dumpPassStructure(unsigned int)"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManager16doInitializationERN4llvm6ModuleE", "non-virtual thunk to (anonymous namespace)::BBPassManager::doInitialization(llvm::Module&)"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManager13runOnFunctionERN4llvm8FunctionE", "non-virtual thunk to (anonymous namespace)::BBPassManager::runOnFunction(llvm::Function&)"},
    {"_ZThn392_N12_GLOBAL__N_113BBPassManager14doFinalizationERN4llvm6ModuleE", "non-virtual thunk to (anonymous namespace)::BBPassManager::doFinalization(llvm::Module&)"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_110TimingInfoEEEPvv", "void* llvm::object_creator<(anonymous namespace)::TimingInfo>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_110TimingInfoEE4callEPv", "llvm::object_deleter<(anonymous namespace)::TimingInfo>::call(void*)"},
    {"_ZN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEED1Ev", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::~opt()"},
    {"_ZN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEED1Ev", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::~list()"},
    {"_ZN4llvm27PassManagerPrettyStackEntryD1Ev", "llvm::PassManagerPrettyStackEntry::~PassManagerPrettyStackEntry()"},
    {"_ZN4llvm27PassManagerPrettyStackEntryD0Ev", "llvm::PassManagerPrettyStackEntry::~PassManagerPrettyStackEntry()"},
    {"_ZNK4llvm13PMDataManager21getNumContainedPassesEv", "llvm::PMDataManager::getNumContainedPasses() const"},
    {"_ZNK4llvm13PMDataManager18getPassManagerTypeEv", "llvm::PMDataManager::getPassManagerType() const"},
    {"_ZN4llvm13FPPassManagerD1Ev", "llvm::FPPassManager::~FPPassManager()"},
    {"_ZN4llvm13FPPassManagerD0Ev", "llvm::FPPassManager::~FPPassManager()"},
    {"_ZNK4llvm13FPPassManager11getPassNameEv", "llvm::FPPassManager::getPassName() const"},
    {"_ZNK4llvm13FPPassManager16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::FPPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13FPPassManager18getAsPMDataManagerEv", "llvm::FPPassManager::getAsPMDataManager()"},
    {"_ZN4llvm13FPPassManager9getAsPassEv", "llvm::FPPassManager::getAsPass()"},
    {"_ZNK4llvm13FPPassManager18getPassManagerTypeEv", "llvm::FPPassManager::getPassManagerType() const"},
    {"_ZThn32_N4llvm13FPPassManagerD1Ev", "non-virtual thunk to llvm::FPPassManager::~FPPassManager()"},
    {"_ZThn32_N4llvm13FPPassManagerD0Ev", "non-virtual thunk to llvm::FPPassManager::~FPPassManager()"},
    {"_ZThn32_N4llvm13FPPassManager9getAsPassEv", "non-virtual thunk to llvm::FPPassManager::getAsPass()"},
    {"_ZThn32_NK4llvm13FPPassManager18getPassManagerTypeEv", "non-virtual thunk to llvm::FPPassManager::getPassManagerType() const"},
    {"_ZNK4llvm17PMTopLevelManager23getNumContainedManagersEv", "llvm::PMTopLevelManager::getNumContainedManagers() const"},
    {"_ZN4llvm13MPPassManagerD1Ev", "llvm::MPPassManager::~MPPassManager()"},
    {"_ZN4llvm13MPPassManagerD0Ev", "llvm::MPPassManager::~MPPassManager()"},
    {"_ZNK4llvm13MPPassManager11getPassNameEv", "llvm::MPPassManager::getPassName() const"},
    {"_ZNK4llvm13MPPassManager17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::MPPassManager::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZNK4llvm13MPPassManager16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::MPPassManager::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm13MPPassManager18getAsPMDataManagerEv", "llvm::MPPassManager::getAsPMDataManager()"},
    {"_ZN4llvm13MPPassManager17dumpPassStructureEj", "llvm::MPPassManager::dumpPassStructure(unsigned int)"},
    {"_ZN4llvm13MPPassManager9getAsPassEv", "llvm::MPPassManager::getAsPass()"},
    {"_ZNK4llvm13MPPassManager18getPassManagerTypeEv", "llvm::MPPassManager::getPassManagerType() const"},
    {"_ZThn32_N4llvm13MPPassManagerD1Ev", "non-virtual thunk to llvm::MPPassManager::~MPPassManager()"},
    {"_ZThn32_N4llvm13MPPassManagerD0Ev", "non-virtual thunk to llvm::MPPassManager::~MPPassManager()"},
    {"_ZThn32_N4llvm13MPPassManager9getAsPassEv", "non-virtual thunk to llvm::MPPassManager::getAsPass()"},
    {"_ZThn32_NK4llvm13MPPassManager18getPassManagerTypeEv", "non-virtual thunk to llvm::MPPassManager::getPassManagerType() const"},
    {"_ZNSt6vectorIPN4llvm13PMDataManagerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::PMDataManager*, std::allocator<llvm::PMDataManager*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::PMDataManager**, std::vector<llvm::PMDataManager*, std::allocator<llvm::PMDataManager*> > >, llvm::PMDataManager* const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm4PassESt4pairIKS2_PNS0_23FunctionPassManagerImplEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<llvm::Pass*, std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*>, std::_Select1st<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> >, std::less<llvm::Pass*>, std::allocator<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> >, std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> const&)"},
    {"_ZNSt8_Rb_treeIPN4llvm4PassESt4pairIKS2_PNS0_23FunctionPassManagerImplEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<llvm::Pass*, std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*>, std::_Select1st<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> >, std::less<llvm::Pass*>, std::allocator<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> > >::_M_insert_unique(std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> const&)"},
    {"_ZNSt8_Rb_treeIPKvSt4pairIKS1_PN4llvm4PassEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE5eraseESt17_Rb_tree_iteratorIS7_ESF_", "std::_Rb_tree<void const*, std::pair<void const* const, llvm::Pass*>, std::_Select1st<std::pair<void const* const, llvm::Pass*> >, std::less<void const*>, std::allocator<std::pair<void const* const, llvm::Pass*> > >::erase(std::_Rb_tree_iterator<std::pair<void const* const, llvm::Pass*> >, std::_Rb_tree_iterator<std::pair<void const* const, llvm::Pass*> >)"},
    {"_ZNSt8_Rb_treeIPKvSt4pairIKS1_PN4llvm4PassEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<void const*, std::pair<void const* const, llvm::Pass*>, std::_Select1st<std::pair<void const* const, llvm::Pass*> >, std::less<void const*>, std::allocator<std::pair<void const* const, llvm::Pass*> > >::_M_erase(std::_Rb_tree_node<std::pair<void const* const, llvm::Pass*> >*)"},
    {"_ZSt6__findIPKPKvS1_ET_S4_S4_RKT0_St26random_access_iterator_tag", "void const* const* std::__find<void const* const*, void const*>(void const* const*, void const* const*, void const* const&, std::random_access_iterator_tag)"},
    {"_ZNSt8_Rb_treeIPKvSt4pairIKS1_PN4llvm4PassEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS7_ERKS7_", "std::_Rb_tree<void const*, std::pair<void const* const, llvm::Pass*>, std::_Select1st<std::pair<void const* const, llvm::Pass*> >, std::less<void const*>, std::allocator<std::pair<void const* const, llvm::Pass*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<void const* const, llvm::Pass*> >, std::pair<void const* const, llvm::Pass*> const&)"},
    {"_ZNSt8_Rb_treeIPKvSt4pairIKS1_PN4llvm4PassEESt10_Select1stIS7_ESt4lessIS1_ESaIS7_EE16_M_insert_uniqueERKS7_", "std::_Rb_tree<void const*, std::pair<void const* const, llvm::Pass*>, std::_Select1st<std::pair<void const* const, llvm::Pass*> >, std::less<void const*>, std::allocator<std::pair<void const* const, llvm::Pass*> > >::_M_insert_unique(std::pair<void const* const, llvm::Pass*> const&)"},
    {"_ZN4llvm8DenseMapIPNS_4PassENS_11SmallPtrSetIS2_Lj8EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16FindAndConstructERKS2_", "llvm::DenseMap<llvm::Pass*, llvm::SmallPtrSet<llvm::Pass*, 8u>, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Pass*, 8u> > >::FindAndConstruct(llvm::Pass* const&)"},
    {"_ZN4llvm8DenseMapIPNS_4PassENS_11SmallPtrSetIS2_Lj8EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Pass*, llvm::SmallPtrSet<llvm::Pass*, 8u>, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Pass*, 8u> > >::InsertIntoBucket(llvm::Pass* const&, llvm::SmallPtrSet<llvm::Pass*, 8u> const&, std::pair<llvm::Pass*, llvm::SmallPtrSet<llvm::Pass*, 8u> >*)"},
    {"_ZN4llvm8DenseMapIPNS_4PassENS_11SmallPtrSetIS2_Lj8EEENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Pass*, llvm::SmallPtrSet<llvm::Pass*, 8u>, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::SmallPtrSet<llvm::Pass*, 8u> > >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_4PassEPNS_13AnalysisUsageENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Pass*, llvm::AnalysisUsage*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::AnalysisUsage*> >::InsertIntoBucket(llvm::Pass* const&, llvm::AnalysisUsage* const&, std::pair<llvm::Pass*, llvm::AnalysisUsage*>*)"},
    {"_ZN4llvm8DenseMapIPNS_4PassEPNS_13AnalysisUsageENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Pass*, llvm::AnalysisUsage*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::AnalysisUsage*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPNS_4PassES2_NS_12DenseMapInfoIS2_EES4_E16InsertIntoBucketERKS2_S7_PSt4pairIS2_S2_E", "llvm::DenseMap<llvm::Pass*, llvm::Pass*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::Pass*> >::InsertIntoBucket(llvm::Pass* const&, llvm::Pass* const&, std::pair<llvm::Pass*, llvm::Pass*>*)"},
    {"_ZN4llvm8DenseMapIPNS_4PassES2_NS_12DenseMapInfoIS2_EES4_E4growEj", "llvm::DenseMap<llvm::Pass*, llvm::Pass*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::Pass*> >::grow(unsigned int)"},
    {"_ZN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEE27getValueExpectedFlagDefaultEv", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEE14getOptionWidthEv", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::getOptionWidth() const"},
    {"_ZNK4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEE15printOptionInfoEm", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEED0Ev", "llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>::~list()"},
    {"_ZN4llvm2cl6parserIPKNS_8PassInfoEE5parseERNS0_6OptionENS_9StringRefES8_RS4_", "llvm::cl::parser<llvm::PassInfo const*>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::PassInfo const*&)"},
    {"_ZNSt6vectorIPKN4llvm8PassInfoESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::PassInfo const**, std::vector<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> > >, llvm::PassInfo const* const&)"},
    {"_ZN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEE16handleOccurrenceEjNS_9StringRefES6_", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEE14getOptionWidthEv", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEE15printOptionInfoEm", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEED0Ev", "llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >::~opt()"},
    {"_ZN4llvm2cl6parserINS_14PassDebugLevelEE5parseERNS0_6OptionENS_9StringRefES6_RS2_", "llvm::cl::parser<llvm::PassDebugLevel>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::PassDebugLevel&)"},
    {"_ZNK4llvm2cl11ValuesClassIiE5applyINS0_3optINS_14PassDebugLevelELb0ENS0_6parserIS5_EEEEEEvRT_", "void llvm::cl::ValuesClass<int>::apply<llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> > >(llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >&) const"},
    {"_ZN4llvm23SmallVectorTemplateBaseINS_2cl6parserINS_14PassDebugLevelEE10OptionInfoELb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm::cl::parser<llvm::PassDebugLevel>::OptionInfo, false>::grow(unsigned long)"},
    {"_ZN4llvm2cl6parserINS_14PassDebugLevelEED1Ev", "llvm::cl::parser<llvm::PassDebugLevel>::~parser()"},
    {"_ZN4llvm2cl6parserINS_14PassDebugLevelEED0Ev", "llvm::cl::parser<llvm::PassDebugLevel>::~parser()"},
    {"_ZNK4llvm2cl6parserINS_14PassDebugLevelEE13getNumOptionsEv", "llvm::cl::parser<llvm::PassDebugLevel>::getNumOptions() const"},
    {"_ZNK4llvm2cl6parserINS_14PassDebugLevelEE9getOptionEj", "llvm::cl::parser<llvm::PassDebugLevel>::getOption(unsigned int) const"},
    {"_ZNK4llvm2cl6parserINS_14PassDebugLevelEE14getDescriptionEj", "llvm::cl::parser<llvm::PassDebugLevel>::getDescription(unsigned int) const"},
    {"_ZN4llvm13MPPassManagerD2Ev", "llvm::MPPassManager::~MPPassManager()"},
    {"_ZNSt8_Rb_treeIPN4llvm4PassESt4pairIKS2_PNS0_23FunctionPassManagerImplEESt10_Select1stIS7_ESt4lessIS2_ESaIS7_EE8_M_eraseEPSt13_Rb_tree_nodeIS7_E", "std::_Rb_tree<llvm::Pass*, std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*>, std::_Select1st<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> >, std::less<llvm::Pass*>, std::allocator<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Pass* const, llvm::FunctionPassManagerImpl*> >*)"},
    {"_ZN4llvm8DenseMapIPNS_4PassEPNS_5TimerENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Pass*, llvm::Timer*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::Timer*> >::InsertIntoBucket(llvm::Pass* const&, llvm::Timer* const&, std::pair<llvm::Pass*, llvm::Timer*>*)"},
    {"_ZN4llvm8DenseMapIPNS_4PassEPNS_5TimerENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Pass*, llvm::Timer*, llvm::DenseMapInfo<llvm::Pass*>, llvm::DenseMapInfo<llvm::Timer*> >::grow(unsigned int)"},
    {"_ZN4llvm15PassManagerImplC2Ei", "llvm::PassManagerImpl::PassManagerImpl(int)"},
    {"_ZN4llvm15PassManagerImplD1Ev", "llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZN4llvm15PassManagerImplD0Ev", "llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZNK4llvm15PassManagerImpl17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::PassManagerImpl::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZNK4llvm15PassManagerImpl16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::PassManagerImpl::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm15PassManagerImpl18getAsPMDataManagerEv", "llvm::PassManagerImpl::getAsPMDataManager()"},
    {"_ZN4llvm15PassManagerImpl15addTopLevelPassEPNS_4PassE", "llvm::PassManagerImpl::addTopLevelPass(llvm::Pass*)"},
    {"_ZN4llvm15PassManagerImpl9getAsPassEv", "llvm::PassManagerImpl::getAsPass()"},
    {"_ZThn32_N4llvm15PassManagerImplD1Ev", "non-virtual thunk to llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZThn32_N4llvm15PassManagerImplD0Ev", "non-virtual thunk to llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZThn32_N4llvm15PassManagerImpl9getAsPassEv", "non-virtual thunk to llvm::PassManagerImpl::getAsPass()"},
    {"_ZThn432_N4llvm15PassManagerImpl15addTopLevelPassEPNS_4PassE", "non-virtual thunk to llvm::PassManagerImpl::addTopLevelPass(llvm::Pass*)"},
    {"_ZThn432_N4llvm15PassManagerImplD1Ev", "non-virtual thunk to llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZThn432_N4llvm15PassManagerImplD0Ev", "non-virtual thunk to llvm::PassManagerImpl::~PassManagerImpl()"},
    {"_ZN4llvm23FunctionPassManagerImplC2Ei", "llvm::FunctionPassManagerImpl::FunctionPassManagerImpl(int)"},
    {"_ZN4llvm23FunctionPassManagerImplD1Ev", "llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZN4llvm23FunctionPassManagerImplD0Ev", "llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZNK4llvm23FunctionPassManagerImpl17createPrinterPassERNS_11raw_ostreamERKSs", "llvm::FunctionPassManagerImpl::createPrinterPass(llvm::raw_ostream&, std::string const&) const"},
    {"_ZNK4llvm23FunctionPassManagerImpl16getAnalysisUsageERNS_13AnalysisUsageE", "llvm::FunctionPassManagerImpl::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN4llvm23FunctionPassManagerImpl18getAsPMDataManagerEv", "llvm::FunctionPassManagerImpl::getAsPMDataManager()"},
    {"_ZN4llvm23FunctionPassManagerImpl9getAsPassEv", "llvm::FunctionPassManagerImpl::getAsPass()"},
    {"_ZN4llvm23FunctionPassManagerImpl15addTopLevelPassEPNS_4PassE", "llvm::FunctionPassManagerImpl::addTopLevelPass(llvm::Pass*)"},
    {"_ZThn32_N4llvm23FunctionPassManagerImplD1Ev", "non-virtual thunk to llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZThn32_N4llvm23FunctionPassManagerImplD0Ev", "non-virtual thunk to llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZThn32_N4llvm23FunctionPassManagerImpl9getAsPassEv", "non-virtual thunk to llvm::FunctionPassManagerImpl::getAsPass()"},
    {"_ZThn432_N4llvm23FunctionPassManagerImpl15addTopLevelPassEPNS_4PassE", "non-virtual thunk to llvm::FunctionPassManagerImpl::addTopLevelPass(llvm::Pass*)"},
    {"_ZThn432_N4llvm23FunctionPassManagerImplD1Ev", "non-virtual thunk to llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZThn432_N4llvm23FunctionPassManagerImplD0Ev", "non-virtual thunk to llvm::FunctionPassManagerImpl::~FunctionPassManagerImpl()"},
    {"_ZNSt6vectorISt4pairIPKvPN4llvm4PassEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<void const*, llvm::Pass*>, std::allocator<std::pair<void const*, llvm::Pass*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<void const*, llvm::Pass*>*, std::vector<std::pair<void const*, llvm::Pass*>, std::allocator<std::pair<void const*, llvm::Pass*> > > >, std::pair<void const*, llvm::Pass*> const&)"},
    {"_ZN4llvm12PassRegistryD1Ev", "llvm::PassRegistry::~PassRegistry()"},
    {"_ZN4llvm12PassRegistryD2Ev", "llvm::PassRegistry::~PassRegistry()"},
    {"_ZNK4llvm12PassRegistry11getPassInfoEPKv", "llvm::PassRegistry::getPassInfo(void const*) const"},
    {"_ZNK4llvm12PassRegistry11getPassInfoENS_9StringRefE", "llvm::PassRegistry::getPassInfo(llvm::StringRef) const"},
    {"_ZN4llvm12PassRegistry12registerPassERKNS_8PassInfoEb", "llvm::PassRegistry::registerPass(llvm::PassInfo const&, bool)"},
    {"_ZN4llvm12PassRegistry14unregisterPassERKNS_8PassInfoE", "llvm::PassRegistry::unregisterPass(llvm::PassInfo const&)"},
    {"_ZN4llvm12PassRegistry21registerAnalysisGroupEPKvS2_RNS_8PassInfoEbb", "llvm::PassRegistry::registerAnalysisGroup(void const*, void const*, llvm::PassInfo&, bool, bool)"},
    {"_ZN4llvm12PassRegistry26removeRegistrationListenerEPNS_24PassRegistrationListenerE", "llvm::PassRegistry::removeRegistrationListener(llvm::PassRegistrationListener*)"},
    {"_ZN4llvm8DenseMapIPKvPKNS_8PassInfoENS_12DenseMapInfoIS2_EENS6_IS5_EEE6insertERKSt4pairIS2_S5_E", "llvm::DenseMap<void const*, llvm::PassInfo const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<llvm::PassInfo const*> >::insert(std::pair<void const*, llvm::PassInfo const*> const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm24PassRegistrationListenerESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::PassRegistrationListener**, std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::PassRegistrationListener**, std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> > >, llvm::PassRegistrationListener*>(__gnu_cxx::__normal_iterator<llvm::PassRegistrationListener**, std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> > >, __gnu_cxx::__normal_iterator<llvm::PassRegistrationListener**, std::vector<llvm::PassRegistrationListener*, std::allocator<llvm::PassRegistrationListener*> > >, llvm::PassRegistrationListener* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm9StringMapIPKNS_8PassInfoENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::PassInfo const*>& llvm::StringMap<llvm::PassInfo const*, llvm::MallocAllocator>::GetOrCreateValue<llvm::PassInfo const*>(llvm::StringRef, llvm::PassInfo const*)"},
    {"_ZN4llvm8DenseMapIPKvPKNS_8PassInfoENS_12DenseMapInfoIS2_EENS6_IS5_EEE16InsertIntoBucketERKS2_RKS5_PSt4pairIS2_S5_E", "llvm::DenseMap<void const*, llvm::PassInfo const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<llvm::PassInfo const*> >::InsertIntoBucket(void const* const&, llvm::PassInfo const* const&, std::pair<void const*, llvm::PassInfo const*>*)"},
    {"_ZN4llvm8DenseMapIPKvPKNS_8PassInfoENS_12DenseMapInfoIS2_EENS6_IS5_EEE4growEj", "llvm::DenseMap<void const*, llvm::PassInfo const*, llvm::DenseMapInfo<void const*>, llvm::DenseMapInfo<llvm::PassInfo const*> >::grow(unsigned int)"},
    {"_ZN4llvm14object_creatorINS_12PassRegistryEEEPvv", "void* llvm::object_creator<llvm::PassRegistry>()"},
    {"_ZN4llvm14object_deleterINS_12PassRegistryEE4callEPv", "llvm::object_deleter<llvm::PassRegistry>::call(void*)"},
    {"_ZN4llvm29initializePrintModulePassPassERNS_12PassRegistryE", "llvm::initializePrintModulePassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm31initializePrintFunctionPassPassERNS_12PassRegistryE", "llvm::initializePrintFunctionPassPass(llvm::PassRegistry&)"},
    {"_ZN4llvm21createPrintModulePassEPNS_11raw_ostreamEbRKSs", "llvm::createPrintModulePass(llvm::raw_ostream*, bool, std::string const&)"},
    {"_ZN4llvm23createPrintFunctionPassERKSsPNS_11raw_ostreamEb", "llvm::createPrintFunctionPass(std::string const&, llvm::raw_ostream*, bool)"},
    {"_ZN12_GLOBAL__N_117PrintFunctionPassD1Ev", "(anonymous namespace)::PrintFunctionPass::~PrintFunctionPass()"},
    {"_ZN12_GLOBAL__N_117PrintFunctionPassD0Ev", "(anonymous namespace)::PrintFunctionPass::~PrintFunctionPass()"},
    {"_ZNK12_GLOBAL__N_117PrintFunctionPass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PrintFunctionPass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_117PrintFunctionPass13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::PrintFunctionPass::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_115PrintModulePassD1Ev", "(anonymous namespace)::PrintModulePass::~PrintModulePass()"},
    {"_ZN12_GLOBAL__N_115PrintModulePassD0Ev", "(anonymous namespace)::PrintModulePass::~PrintModulePass()"},
    {"_ZNK12_GLOBAL__N_115PrintModulePass16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PrintModulePass::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_115PrintModulePass11runOnModuleERN4llvm6ModuleE", "(anonymous namespace)::PrintModulePass::runOnModule(llvm::Module&)"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_117PrintFunctionPassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PrintFunctionPass>()"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_115PrintModulePassEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PrintModulePass>()"},
    {"_ZN4llvm16AbstractTypeUserD0Ev", "llvm::AbstractTypeUser::~AbstractTypeUser()"},
    {"_ZN4llvm16AbstractTypeUserD1Ev", "llvm::AbstractTypeUser::~AbstractTypeUser()"},
    {"_ZN4llvm16AbstractTypeUserD2Ev", "llvm::AbstractTypeUser::~AbstractTypeUser()"},
    {"_ZN4llvm16AbstractTypeUser7setTypeEPNS_5ValueEPKNS_4TypeE", "llvm::AbstractTypeUser::setType(llvm::Value*, llvm::Type const*)"},
    {"_ZNK4llvm4Type7destroyEv", "llvm::Type::destroy() const"},
    {"_ZN4llvm4Type16getPrimitiveTypeERNS_11LLVMContextENS0_6TypeIDE", "llvm::Type::getPrimitiveType(llvm::LLVMContext&, llvm::Type::TypeID)"},
    {"_ZN4llvm4Type9getVoidTyERNS_11LLVMContextE", "llvm::Type::getVoidTy(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type10getFloatTyERNS_11LLVMContextE", "llvm::Type::getFloatTy(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type11getDoubleTyERNS_11LLVMContextE", "llvm::Type::getDoubleTy(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type13getX86_FP80TyERNS_11LLVMContextE", "llvm::Type::getX86_FP80Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type10getFP128TyERNS_11LLVMContextE", "llvm::Type::getFP128Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type14getPPC_FP128TyERNS_11LLVMContextE", "llvm::Type::getPPC_FP128Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type10getLabelTyERNS_11LLVMContextE", "llvm::Type::getLabelTy(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type13getMetadataTyERNS_11LLVMContextE", "llvm::Type::getMetadataTy(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type12getX86_MMXTyERNS_11LLVMContextE", "llvm::Type::getX86_MMXTy(llvm::LLVMContext&)"},
    {"_ZNK4llvm4Type21getVAArgsPromotedTypeERNS_11LLVMContextE", "llvm::Type::getVAArgsPromotedType(llvm::LLVMContext&) const"},
    {"_ZN4llvm4Type10getInt32TyERNS_11LLVMContextE", "llvm::Type::getInt32Ty(llvm::LLVMContext&)"},
    {"_ZNK4llvm4Type13getScalarTypeEv", "llvm::Type::getScalarType() const"},
    {"_ZNK4llvm4Type11isIntegerTyEj", "llvm::Type::isIntegerTy(unsigned int) const"},
    {"_ZNK4llvm4Type18isIntOrIntVectorTyEv", "llvm::Type::isIntOrIntVectorTy() const"},
    {"_ZNK4llvm4Type16isFPOrFPVectorTyEv", "llvm::Type::isFPOrFPVectorTy() const"},
    {"_ZNK4llvm4Type22canLosslesslyBitCastToEPKS0_", "llvm::Type::canLosslesslyBitCastTo(llvm::Type const*) const"},
    {"_ZNK4llvm4Type22getPrimitiveSizeInBitsEv", "llvm::Type::getPrimitiveSizeInBits() const"},
    {"_ZNK4llvm4Type19getScalarSizeInBitsEv", "llvm::Type::getScalarSizeInBits() const"},
    {"_ZNK4llvm4Type18getFPMantissaWidthEv", "llvm::Type::getFPMantissaWidth() const"},
    {"_ZNK4llvm4Type18isSizedDerivedTypeEv", "llvm::Type::isSizedDerivedType() const"},
    {"_ZNK4llvm4Type24getForwardedTypeInternalEv", "llvm::Type::getForwardedTypeInternal() const"},
    {"_ZN4llvm4Type18refineAbstractTypeEPKNS_11DerivedTypeEPKS0_", "llvm::Type::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm4Type18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::Type::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm4Type14getDescriptionEv", "llvm::Type::getDescription() const"},
    {"_ZNK4llvm10StructType10indexValidEPKNS_5ValueE", "llvm::StructType::indexValid(llvm::Value const*) const"},
    {"_ZNK4llvm10StructType10indexValidEj", "llvm::StructType::indexValid(unsigned int) const"},
    {"_ZNK4llvm10StructType14getTypeAtIndexEPKNS_5ValueE", "llvm::StructType::getTypeAtIndex(llvm::Value const*) const"},
    {"_ZNK4llvm10StructType14getTypeAtIndexEj", "llvm::StructType::getTypeAtIndex(unsigned int) const"},
    {"_ZN4llvm4Type9getIntNTyERNS_11LLVMContextEj", "llvm::Type::getIntNTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm11IntegerType3getERNS_11LLVMContextEj", "llvm::IntegerType::get(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type9getInt1TyERNS_11LLVMContextE", "llvm::Type::getInt1Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type9getInt8TyERNS_11LLVMContextE", "llvm::Type::getInt8Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type10getInt16TyERNS_11LLVMContextE", "llvm::Type::getInt16Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type10getInt64TyERNS_11LLVMContextE", "llvm::Type::getInt64Ty(llvm::LLVMContext&)"},
    {"_ZN4llvm4Type13getFloatPtrTyERNS_11LLVMContextEj", "llvm::Type::getFloatPtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZNK4llvm4Type12getPointerToEj", "llvm::Type::getPointerTo(unsigned int) const"},
    {"_ZN4llvm4Type14getDoublePtrTyERNS_11LLVMContextEj", "llvm::Type::getDoublePtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type16getX86_FP80PtrTyERNS_11LLVMContextEj", "llvm::Type::getX86_FP80PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type13getFP128PtrTyERNS_11LLVMContextEj", "llvm::Type::getFP128PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type17getPPC_FP128PtrTyERNS_11LLVMContextEj", "llvm::Type::getPPC_FP128PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type15getX86_MMXPtrTyERNS_11LLVMContextEj", "llvm::Type::getX86_MMXPtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type12getIntNPtrTyERNS_11LLVMContextEjj", "llvm::Type::getIntNPtrTy(llvm::LLVMContext&, unsigned int, unsigned int)"},
    {"_ZN4llvm4Type12getInt1PtrTyERNS_11LLVMContextEj", "llvm::Type::getInt1PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type12getInt8PtrTyERNS_11LLVMContextEj", "llvm::Type::getInt8PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type13getInt16PtrTyERNS_11LLVMContextEj", "llvm::Type::getInt16PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type13getInt32PtrTyERNS_11LLVMContextEj", "llvm::Type::getInt32PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm4Type13getInt64PtrTyERNS_11LLVMContextEj", "llvm::Type::getInt64PtrTy(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm12FunctionType17isValidReturnTypeEPKNS_4TypeE", "llvm::FunctionType::isValidReturnType(llvm::Type const*)"},
    {"_ZN4llvm12FunctionType19isValidArgumentTypeEPKNS_4TypeE", "llvm::FunctionType::isValidArgumentType(llvm::Type const*)"},
    {"_ZN4llvm12FunctionTypeC1EPKNS_4TypeERKSt6vectorIS3_SaIS3_EEb", "llvm::FunctionType::FunctionType(llvm::Type const*, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm12FunctionTypeC2EPKNS_4TypeERKSt6vectorIS3_SaIS3_EEb", "llvm::FunctionType::FunctionType(llvm::Type const*, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm10StructTypeC1ERNS_11LLVMContextERKSt6vectorIPKNS_4TypeESaIS6_EEb", "llvm::StructType::StructType(llvm::LLVMContext&, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm10StructTypeC2ERNS_11LLVMContextERKSt6vectorIPKNS_4TypeESaIS6_EEb", "llvm::StructType::StructType(llvm::LLVMContext&, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm9ArrayTypeC1EPKNS_4TypeEy", "llvm::ArrayType::ArrayType(llvm::Type const*, unsigned long long)"},
    {"_ZN4llvm9ArrayTypeC2EPKNS_4TypeEy", "llvm::ArrayType::ArrayType(llvm::Type const*, unsigned long long)"},
    {"_ZN4llvm10VectorTypeC1EPKNS_4TypeEj", "llvm::VectorType::VectorType(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm10VectorTypeC2EPKNS_4TypeEj", "llvm::VectorType::VectorType(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm11PointerTypeC1EPKNS_4TypeEj", "llvm::PointerType::PointerType(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm11PointerTypeC2EPKNS_4TypeEj", "llvm::PointerType::PointerType(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm10OpaqueTypeC1ERNS_11LLVMContextE", "llvm::OpaqueType::OpaqueType(llvm::LLVMContext&)"},
    {"_ZN4llvm10OpaqueTypeC2ERNS_11LLVMContextE", "llvm::OpaqueType::OpaqueType(llvm::LLVMContext&)"},
    {"_ZN4llvm12PATypeHolder7destroyEv", "llvm::PATypeHolder::destroy()"},
    {"_ZN4llvm11DerivedType15dropAllTypeUsesEv", "llvm::DerivedType::dropAllTypeUses()"},
    {"_ZN4llvm4Type25PromoteAbstractToConcreteEv", "llvm::Type::PromoteAbstractToConcrete()"},
    {"_ZN4llvm11DerivedType32notifyUsesThatTypeBecameConcreteEv", "llvm::DerivedType::notifyUsesThatTypeBecameConcrete()"},
    {"_ZNK4llvm11IntegerType19isPowerOf2ByteWidthEv", "llvm::IntegerType::isPowerOf2ByteWidth() const"},
    {"_ZNK4llvm11IntegerType7getMaskEv", "llvm::IntegerType::getMask() const"},
    {"_ZN4llvm15FunctionValType3getEPKNS_12FunctionTypeE", "llvm::FunctionValType::get(llvm::FunctionType const*)"},
    {"_ZN4llvm12FunctionType3getEPKNS_4TypeERKSt6vectorIS3_SaIS3_EEb", "llvm::FunctionType::get(llvm::Type const*, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm9ArrayType3getEPKNS_4TypeEy", "llvm::ArrayType::get(llvm::Type const*, unsigned long long)"},
    {"_ZN4llvm9ArrayType18isValidElementTypeEPKNS_4TypeE", "llvm::ArrayType::isValidElementType(llvm::Type const*)"},
    {"_ZN4llvm10VectorType3getEPKNS_4TypeEj", "llvm::VectorType::get(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm10VectorType18isValidElementTypeEPKNS_4TypeE", "llvm::VectorType::isValidElementType(llvm::Type const*)"},
    {"_ZN4llvm10StructType3getERNS_11LLVMContextERKSt6vectorIPKNS_4TypeESaIS6_EEb", "llvm::StructType::get(llvm::LLVMContext&, std::vector<llvm::Type const*, std::allocator<llvm::Type const*> > const&, bool)"},
    {"_ZN4llvm10StructType3getERNS_11LLVMContextEPKNS_4TypeEz", "llvm::StructType::get(llvm::LLVMContext&, llvm::Type const*, ...)"},
    {"_ZN4llvm10StructType18isValidElementTypeEPKNS_4TypeE", "llvm::StructType::isValidElementType(llvm::Type const*)"},
    {"_ZN4llvm11PointerType3getEPKNS_4TypeEj", "llvm::PointerType::get(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm11PointerType18isValidElementTypeEPKNS_4TypeE", "llvm::PointerType::isValidElementType(llvm::Type const*)"},
    {"_ZN4llvm10OpaqueType3getERNS_11LLVMContextE", "llvm::OpaqueType::get(llvm::LLVMContext&)"},
    {"_ZNK4llvm4Type19addAbstractTypeUserEPNS_16AbstractTypeUserE", "llvm::Type::addAbstractTypeUser(llvm::AbstractTypeUser*) const"},
    {"_ZNK4llvm4Type22removeAbstractTypeUserEPNS_16AbstractTypeUserE", "llvm::Type::removeAbstractTypeUser(llvm::AbstractTypeUser*) const"},
    {"_ZN4llvm11DerivedType20refineAbstractTypeToEPKNS_4TypeE", "llvm::DerivedType::refineAbstractTypeTo(llvm::Type const*)"},
    {"_ZN4llvm12FunctionType18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::FunctionType::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm12FunctionType18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::FunctionType::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZN4llvm9ArrayType18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::ArrayType::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm9ArrayType18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::ArrayType::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZN4llvm10VectorType18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::VectorType::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm10VectorType18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::VectorType::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZN4llvm10StructType18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::StructType::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm10StructType18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::StructType::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZN4llvm11PointerType18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::PointerType::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm11PointerType18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::PointerType::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm14SequentialType10indexValidEPKNS_5ValueE", "llvm::SequentialType::indexValid(llvm::Value const*) const"},
    {"_ZN4llvmlsERNS_11raw_ostreamERKNS_4TypeE", "llvm::operator<<(llvm::raw_ostream&, llvm::Type const&)"},
    {"_ZN4llvm25TypeHasCycleThroughItselfEPKNS_4TypeE", "llvm::TypeHasCycleThroughItself(llvm::Type const*)"},
    {"_Z10TypesEqualPKN4llvm4TypeES2_RSt3mapIS2_S2_St4lessIS2_ESaISt4pairIKS2_S2_EEE", "TypesEqual(llvm::Type const*, llvm::Type const*, std::map<llvm::Type const*, llvm::Type const*, std::less<llvm::Type const*>, std::allocator<std::pair<llvm::Type const* const, llvm::Type const*> > >&)"},
    {"_Z27AbstractTypeHasCycleThroughPKN4llvm4TypeES2_RNS_11SmallPtrSetIS2_Lj128EEE", "AbstractTypeHasCycleThrough(llvm::Type const*, llvm::Type const*, llvm::SmallPtrSet<llvm::Type const*, 128u>&)"},
    {"_Z27ConcreteTypeHasCycleThroughPKN4llvm4TypeES2_RNS_11SmallPtrSetIS2_Lj128EEE", "ConcreteTypeHasCycleThrough(llvm::Type const*, llvm::Type const*, llvm::SmallPtrSet<llvm::Type const*, 128u>&)"},
    {"_ZN4llvm12scc_iteratorIN12_GLOBAL__N_118TypePromotionGraphENS_11GraphTraitsIS2_EEE10GetNextSCCEv", "llvm::scc_iterator<(anonymous namespace)::TypePromotionGraph, llvm::GraphTraits<(anonymous namespace)::TypePromotionGraph> >::GetNextSCC()"},
    {"_ZN4llvm12scc_iteratorIN12_GLOBAL__N_118TypePromotionGraphENS_11GraphTraitsIS2_EEE11DFSVisitOneEPNS_4TypeE", "llvm::scc_iterator<(anonymous namespace)::TypePromotionGraph, llvm::GraphTraits<(anonymous namespace)::TypePromotionGraph> >::DFSVisitOne(llvm::Type*)"},
    {"_ZN4llvm12FunctionTypeD1Ev", "llvm::FunctionType::~FunctionType()"},
    {"_ZN4llvm10StructTypeD1Ev", "llvm::StructType::~StructType()"},
    {"_ZNK4llvm10VectorType11getBitWidthEv", "llvm::VectorType::getBitWidth() const"},
    {"_ZNK4llvm4Type16getForwardedTypeEv", "llvm::Type::getForwardedType() const"},
    {"_ZN4llvm14SequentialTypeC2ENS_4Type6TypeIDEPKS1_", "llvm::SequentialType::SequentialType(llvm::Type::TypeID, llvm::Type const*)"},
    {"_ZN4llvm12PATypeHandleaSEPKNS_4TypeE", "llvm::PATypeHandle::operator=(llvm::Type const*)"},
    {"_ZN4llvm7TypeMapINS_14IntegerValTypeENS_11IntegerTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::IntegerValType, llvm::IntegerType>::add(llvm::IntegerValType const&, llvm::IntegerType*)"},
    {"_ZN4llvm7TypeMapINS_15FunctionValTypeENS_12FunctionTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::FunctionValType, llvm::FunctionType>::add(llvm::FunctionValType const&, llvm::FunctionType*)"},
    {"_ZN4llvm7TypeMapINS_12ArrayValTypeENS_9ArrayTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::ArrayValType, llvm::ArrayType>::add(llvm::ArrayValType const&, llvm::ArrayType*)"},
    {"_ZN4llvm7TypeMapINS_13VectorValTypeENS_10VectorTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::VectorValType, llvm::VectorType>::add(llvm::VectorValType const&, llvm::VectorType*)"},
    {"_ZN4llvm7TypeMapINS_13StructValTypeENS_10StructTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::StructValType, llvm::StructType>::add(llvm::StructValType const&, llvm::StructType*)"},
    {"_ZN4llvm7TypeMapINS_14PointerValTypeENS_11PointerTypeEE3addERKS1_PS2_", "llvm::TypeMap<llvm::PointerValType, llvm::PointerType>::add(llvm::PointerValType const&, llvm::PointerType*)"},
    {"_ZN4llvm7TypeMapINS_15FunctionValTypeENS_12FunctionTypeEE18RefineAbstractTypeEPS2_PKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeMap<llvm::FunctionValType, llvm::FunctionType>::RefineAbstractType(llvm::FunctionType*, llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm11TypeMapBase18TypeBecameConcreteEPNS_11DerivedTypeEPKS1_", "llvm::TypeMapBase::TypeBecameConcrete(llvm::DerivedType*, llvm::DerivedType const*)"},
    {"_ZN4llvm7TypeMapINS_12ArrayValTypeENS_9ArrayTypeEE18RefineAbstractTypeEPS2_PKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeMap<llvm::ArrayValType, llvm::ArrayType>::RefineAbstractType(llvm::ArrayType*, llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm7TypeMapINS_13VectorValTypeENS_10VectorTypeEE18RefineAbstractTypeEPS2_PKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeMap<llvm::VectorValType, llvm::VectorType>::RefineAbstractType(llvm::VectorType*, llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm7TypeMapINS_13StructValTypeENS_10StructTypeEE18RefineAbstractTypeEPS2_PKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeMap<llvm::StructValType, llvm::StructType>::RefineAbstractType(llvm::StructType*, llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm7TypeMapINS_14PointerValTypeENS_11PointerTypeEE18RefineAbstractTypeEPS2_PKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeMap<llvm::PointerValType, llvm::PointerType>::RefineAbstractType(llvm::PointerType*, llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm14SequentialTypeD1Ev", "llvm::SequentialType::~SequentialType()"},
    {"_ZN4llvm14SequentialTypeD0Ev", "llvm::SequentialType::~SequentialType()"},
    {"_ZNK4llvm14SequentialType14getTypeAtIndexEPKNS_5ValueE", "llvm::SequentialType::getTypeAtIndex(llvm::Value const*) const"},
    {"_ZNK4llvm14SequentialType14getTypeAtIndexEj", "llvm::SequentialType::getTypeAtIndex(unsigned int) const"},
    {"_ZNK4llvm14SequentialType10indexValidEj", "llvm::SequentialType::indexValid(unsigned int) const"},
    {"_ZN4llvm12FunctionTypeD0Ev", "llvm::FunctionType::~FunctionType()"},
    {"_ZN4llvm10StructTypeD0Ev", "llvm::StructType::~StructType()"},
    {"_ZN4llvm9ArrayTypeD1Ev", "llvm::ArrayType::~ArrayType()"},
    {"_ZN4llvm9ArrayTypeD0Ev", "llvm::ArrayType::~ArrayType()"},
    {"_ZN4llvm10VectorTypeD1Ev", "llvm::VectorType::~VectorType()"},
    {"_ZN4llvm10VectorTypeD0Ev", "llvm::VectorType::~VectorType()"},
    {"_ZN4llvm11PointerTypeD1Ev", "llvm::PointerType::~PointerType()"},
    {"_ZN4llvm11PointerTypeD0Ev", "llvm::PointerType::~PointerType()"},
    {"_ZN4llvm10OpaqueTypeD1Ev", "llvm::OpaqueType::~OpaqueType()"},
    {"_ZN4llvm10OpaqueTypeD0Ev", "llvm::OpaqueType::~OpaqueType()"},
    {"_ZN4llvm11TypeMapBase21RemoveFromTypesByHashEjPKNS_4TypeE", "llvm::TypeMapBase::RemoveFromTypesByHash(unsigned int, llvm::Type const*)"},
    {"_ZNSt8_Rb_treeIjSt4pairIKjN4llvm12PATypeHolderEESt10_Select1stIS4_ESt4lessIjESaIS4_EE15_M_insert_equalERKS4_", "std::_Rb_tree<unsigned int, std::pair<unsigned int const, llvm::PATypeHolder>, std::_Select1st<std::pair<unsigned int const, llvm::PATypeHolder> >, std::less<unsigned int>, std::allocator<std::pair<unsigned int const, llvm::PATypeHolder> > >::_M_insert_equal(std::pair<unsigned int const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<llvm::Type const*, std::pair<llvm::Type const* const, llvm::Type const*>, std::_Select1st<std::pair<llvm::Type const* const, llvm::Type const*> >, std::less<llvm::Type const*>, std::allocator<std::pair<llvm::Type const* const, llvm::Type const*> > >::_M_erase(std::_Rb_tree_node<std::pair<llvm::Type const* const, llvm::Type const*> >*)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS6_ERKS6_", "std::_Rb_tree<llvm::Type const*, std::pair<llvm::Type const* const, llvm::Type const*>, std::_Select1st<std::pair<llvm::Type const* const, llvm::Type const*> >, std::less<llvm::Type const*>, std::allocator<std::pair<llvm::Type const* const, llvm::Type const*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<llvm::Type const* const, llvm::Type const*> >, std::pair<llvm::Type const* const, llvm::Type const*> const&)"},
    {"_ZNSt8_Rb_treeIPKN4llvm4TypeESt4pairIKS3_S3_ESt10_Select1stIS6_ESt4lessIS3_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<llvm::Type const*, std::pair<llvm::Type const* const, llvm::Type const*>, std::_Select1st<std::pair<llvm::Type const* const, llvm::Type const*> >, std::less<llvm::Type const*>, std::allocator<std::pair<llvm::Type const* const, llvm::Type const*> > >::_M_insert_unique(std::pair<llvm::Type const* const, llvm::Type const*> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm14PointerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::PointerValType, std::pair<llvm::PointerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >, std::less<llvm::PointerValType>, std::allocator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::PointerValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm14PointerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<llvm::PointerValType, std::pair<llvm::PointerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >, std::less<llvm::PointerValType>, std::allocator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::PointerValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm14PointerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<llvm::PointerValType, std::pair<llvm::PointerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >, std::less<llvm::PointerValType>, std::allocator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >, std::_Rb_tree_iterator<std::pair<llvm::PointerValType const, llvm::PATypeHolder> >)"},
    {"_ZN4llvm13StructValType3getEPKNS_10StructTypeE", "llvm::StructValType::get(llvm::StructType const*)"},
    {"_ZNSt4pairIKN4llvm13StructValTypeENS0_12PATypeHolderEEC2IS1_PNS0_10StructTypeEEERKS_IT_T0_E", "std::pair<llvm::StructValType const, llvm::PATypeHolder>::pair<llvm::StructValType, llvm::StructType*>(std::pair<llvm::StructValType, llvm::StructType*> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::StructValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::StructValType const, llvm::PATypeHolder> const&)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIKN4llvm13StructValTypeENS2_12PATypeHolderEEE9constructEPS6_RKS6_", "__gnu_cxx::new_allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> >::construct(std::pair<llvm::StructValType const, llvm::PATypeHolder>*, std::pair<llvm::StructValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseERS3_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::erase(llvm::StructValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_E", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::StructValType const, llvm::PATypeHolder> >)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE11lower_boundERS3_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::lower_bound(llvm::StructValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE11upper_boundERS3_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::upper_bound(llvm::StructValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13VectorValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::VectorValType, std::pair<llvm::VectorValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >, std::less<llvm::VectorValType>, std::allocator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::VectorValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13VectorValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<llvm::VectorValType, std::pair<llvm::VectorValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >, std::less<llvm::VectorValType>, std::allocator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::VectorValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13VectorValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<llvm::VectorValType, std::pair<llvm::VectorValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >, std::less<llvm::VectorValType>, std::allocator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >, std::_Rb_tree_iterator<std::pair<llvm::VectorValType const, llvm::PATypeHolder> >)"},
    {"_ZNSt8_Rb_treeIN4llvm12ArrayValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::ArrayValType, std::pair<llvm::ArrayValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >, std::less<llvm::ArrayValType>, std::allocator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::ArrayValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm12ArrayValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<llvm::ArrayValType, std::pair<llvm::ArrayValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >, std::less<llvm::ArrayValType>, std::allocator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::ArrayValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm12ArrayValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_ESD_", "std::_Rb_tree<llvm::ArrayValType, std::pair<llvm::ArrayValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >, std::less<llvm::ArrayValType>, std::allocator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >, std::_Rb_tree_iterator<std::pair<llvm::ArrayValType const, llvm::PATypeHolder> >)"},
    {"_ZNSt4pairIKN4llvm15FunctionValTypeENS0_12PATypeHolderEEC2IS1_PNS0_12FunctionTypeEEERKS_IT_T0_E", "std::pair<llvm::FunctionValType const, llvm::PATypeHolder>::pair<llvm::FunctionValType, llvm::FunctionType*>(std::pair<llvm::FunctionValType, llvm::FunctionType*> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::FunctionValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE9_M_insertEPSt18_Rb_tree_node_baseSD_RKS5_", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<llvm::FunctionValType const, llvm::PATypeHolder> const&)"},
    {"_ZN9__gnu_cxx13new_allocatorISt4pairIKN4llvm15FunctionValTypeENS2_12PATypeHolderEEE9constructEPS6_RKS6_", "__gnu_cxx::new_allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >::construct(std::pair<llvm::FunctionValType const, llvm::PATypeHolder>*, std::pair<llvm::FunctionValType const, llvm::PATypeHolder> const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseERS3_", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::erase(llvm::FunctionValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE11equal_rangeERS3_", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::equal_range(llvm::FunctionValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE5eraseESt17_Rb_tree_iteratorIS5_E", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::erase(std::_Rb_tree_iterator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >)"},
    {"_ZNSt6vectorIPN4llvm16AbstractTypeUserESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::AbstractTypeUser*, std::allocator<llvm::AbstractTypeUser*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::AbstractTypeUser**, std::vector<llvm::AbstractTypeUser*, std::allocator<llvm::AbstractTypeUser*> > >, llvm::AbstractTypeUser* const&)"},
    {"_ZNSt8_Rb_treeIN4llvm13StructValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE4findERS3_", "std::_Rb_tree<llvm::StructValType, std::pair<llvm::StructValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::StructValType const, llvm::PATypeHolder> >, std::less<llvm::StructValType>, std::allocator<std::pair<llvm::StructValType const, llvm::PATypeHolder> > >::find(llvm::StructValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm15FunctionValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE4findERS3_", "std::_Rb_tree<llvm::FunctionValType, std::pair<llvm::FunctionValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> >, std::less<llvm::FunctionValType>, std::allocator<std::pair<llvm::FunctionValType const, llvm::PATypeHolder> > >::find(llvm::FunctionValType const&)"},
    {"_ZNSt8_Rb_treeIN4llvm14IntegerValTypeESt4pairIKS1_NS0_12PATypeHolderEESt10_Select1stIS5_ESt4lessIS1_ESaIS5_EE16_M_insert_uniqueERKS5_", "std::_Rb_tree<llvm::IntegerValType, std::pair<llvm::IntegerValType const, llvm::PATypeHolder>, std::_Select1st<std::pair<llvm::IntegerValType const, llvm::PATypeHolder> >, std::less<llvm::IntegerValType>, std::allocator<std::pair<llvm::IntegerValType const, llvm::PATypeHolder> > >::_M_insert_unique(std::pair<llvm::IntegerValType const, llvm::PATypeHolder> const&)"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm4TypeESt6vectorIS4_SaIS4_EEEENS2_12PATypeHandleEET_SB_SB_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::Type**, std::vector<llvm::Type*, std::allocator<llvm::Type*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::Type**, std::vector<llvm::Type*, std::allocator<llvm::Type*> > >, llvm::PATypeHandle>(__gnu_cxx::__normal_iterator<llvm::Type**, std::vector<llvm::Type*, std::allocator<llvm::Type*> > >, __gnu_cxx::__normal_iterator<llvm::Type**, std::vector<llvm::Type*, std::allocator<llvm::Type*> > >, llvm::PATypeHandle const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm4TypeESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Type*, std::allocator<llvm::Type*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Type**, std::vector<llvm::Type*, std::allocator<llvm::Type*> > >, llvm::Type* const&)"},
    {"_ZN4llvm8DenseMapIPNS_4TypeEjNS_12DenseMapInfoIS2_EENS3_IjEEE16InsertIntoBucketERKS2_RKjPSt4pairIS2_jE", "llvm::DenseMap<llvm::Type*, unsigned int, llvm::DenseMapInfo<llvm::Type*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::Type* const&, unsigned int const&, std::pair<llvm::Type*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPNS_4TypeEjNS_12DenseMapInfoIS2_EENS3_IjEEE4growEj", "llvm::DenseMap<llvm::Type*, unsigned int, llvm::DenseMapInfo<llvm::Type*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorISt4pairIPN4llvm4TypeEPNS1_12PATypeHandleEESaIS6_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS6_S8_EERKS6_", "std::vector<std::pair<llvm::Type*, llvm::PATypeHandle*>, std::allocator<std::pair<llvm::Type*, llvm::PATypeHandle*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::Type*, llvm::PATypeHandle*>*, std::vector<std::pair<llvm::Type*, llvm::PATypeHandle*>, std::allocator<std::pair<llvm::Type*, llvm::PATypeHandle*> > > >, std::pair<llvm::Type*, llvm::PATypeHandle*> const&)"},
    {"_ZN4llvm13CompositeTypeD1Ev", "llvm::CompositeType::~CompositeType()"},
    {"_ZN4llvm13CompositeTypeD0Ev", "llvm::CompositeType::~CompositeType()"},
    {"_ZN4llvm15TypeSymbolTableD0Ev", "llvm::TypeSymbolTable::~TypeSymbolTable()"},
    {"_ZN4llvm15TypeSymbolTableD1Ev", "llvm::TypeSymbolTable::~TypeSymbolTable()"},
    {"_ZN4llvm15TypeSymbolTableD2Ev", "llvm::TypeSymbolTable::~TypeSymbolTable()"},
    {"_ZNK4llvm15TypeSymbolTable13getUniqueNameENS_9StringRefE", "llvm::TypeSymbolTable::getUniqueName(llvm::StringRef) const"},
    {"_ZNK4llvm15TypeSymbolTable6lookupENS_9StringRefE", "llvm::TypeSymbolTable::lookup(llvm::StringRef) const"},
    {"_ZN4llvm15TypeSymbolTable6removeESt17_Rb_tree_iteratorISt4pairIKSsPKNS_4TypeEEE", "llvm::TypeSymbolTable::remove(std::_Rb_tree_iterator<std::pair<std::string const, llvm::Type const*> >)"},
    {"_ZN4llvm15TypeSymbolTable6insertENS_9StringRefEPKNS_4TypeE", "llvm::TypeSymbolTable::insert(llvm::StringRef, llvm::Type const*)"},
    {"_ZN4llvm15TypeSymbolTable18refineAbstractTypeEPKNS_11DerivedTypeEPKNS_4TypeE", "llvm::TypeSymbolTable::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN4llvm15TypeSymbolTable18typeBecameConcreteEPKNS_11DerivedTypeE", "llvm::TypeSymbolTable::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK4llvm15TypeSymbolTable4dumpEv", "llvm::TypeSymbolTable::dump() const"},
    {"_ZNSt8_Rb_treeIKSsSt4pairIS0_PKN4llvm4TypeEESt10_Select1stIS6_ESt4lessIS0_ESaIS6_EE16_M_insert_uniqueERKS6_", "std::_Rb_tree<std::string const, std::pair<std::string const, llvm::Type const*>, std::_Select1st<std::pair<std::string const, llvm::Type const*> >, std::less<std::string const>, std::allocator<std::pair<std::string const, llvm::Type const*> > >::_M_insert_unique(std::pair<std::string const, llvm::Type const*> const&)"},
    {"_ZNSt8_Rb_treeIKSsSt4pairIS0_PKN4llvm4TypeEESt10_Select1stIS6_ESt4lessIS0_ESaIS6_EE9_M_insertEPSt18_Rb_tree_node_baseSE_RKS6_", "std::_Rb_tree<std::string const, std::pair<std::string const, llvm::Type const*>, std::_Select1st<std::pair<std::string const, llvm::Type const*> >, std::less<std::string const>, std::allocator<std::pair<std::string const, llvm::Type const*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, llvm::Type const*> const&)"},
    {"_ZNKSt8_Rb_treeIKSsSt4pairIS0_PKN4llvm4TypeEESt10_Select1stIS6_ESt4lessIS0_ESaIS6_EE4findERS0_", "std::_Rb_tree<std::string const, std::pair<std::string const, llvm::Type const*>, std::_Select1st<std::pair<std::string const, llvm::Type const*> >, std::less<std::string const>, std::allocator<std::pair<std::string const, llvm::Type const*> > >::find(std::string const&) const"},
    {"_ZNSt8_Rb_treeIKSsSt4pairIS0_PKN4llvm4TypeEESt10_Select1stIS6_ESt4lessIS0_ESaIS6_EE8_M_eraseEPSt13_Rb_tree_nodeIS6_E", "std::_Rb_tree<std::string const, std::pair<std::string const, llvm::Type const*>, std::_Select1st<std::pair<std::string const, llvm::Type const*> >, std::less<std::string const>, std::allocator<std::pair<std::string const, llvm::Type const*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, llvm::Type const*> >*)"},
    {"_ZN4llvm3Use4swapERS0_", "llvm::Use::swap(llvm::Use&)"},
    {"_ZNK4llvm3Use14getImpliedUserEv", "llvm::Use::getImpliedUser() const"},
    {"_ZN4llvm3Use8initTagsEPS0_S1_", "llvm::Use::initTags(llvm::Use*, llvm::Use*)"},
    {"_ZN4llvm3Use3zapEPS0_PKS0_b", "llvm::Use::zap(llvm::Use*, llvm::Use const*, bool)"},
    {"_ZNK4llvm3Use7getUserEv", "llvm::Use::getUser() const"},
    {"_ZN4llvm4User17replaceUsesOfWithEPNS_5ValueES2_", "llvm::User::replaceUsesOfWith(llvm::Value*, llvm::Value*)"},
    {"_ZNK4llvm4User16allocHungoffUsesEj", "llvm::User::allocHungoffUses(unsigned int) const"},
    {"_ZN4llvm4UsernwEmj", "llvm::User::operator new(unsigned long, unsigned int)"},
    {"_ZN4llvm4UserdlEPv", "llvm::User::operator delete(void*)"},
    {"_ZN4llvm5ValueC1EPKNS_4TypeEj", "llvm::Value::Value(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm5ValueC2EPKNS_4TypeEj", "llvm::Value::Value(llvm::Type const*, unsigned int)"},
    {"_ZN4llvm5ValueD0Ev", "llvm::Value::~Value()"},
    {"_ZN4llvm5ValueD1Ev", "llvm::Value::~Value()"},
    {"_ZN4llvm5ValueD2Ev", "llvm::Value::~Value()"},
    {"_ZN4llvm15ValueHandleBase14ValueIsDeletedEPNS_5ValueE", "llvm::ValueHandleBase::ValueIsDeleted(llvm::Value*)"},
    {"_ZNK4llvm5Value8hasNUsesEj", "llvm::Value::hasNUses(unsigned int) const"},
    {"_ZNK4llvm5Value14hasNUsesOrMoreEj", "llvm::Value::hasNUsesOrMore(unsigned int) const"},
    {"_ZNK4llvm5Value18isUsedInBasicBlockEPKNS_10BasicBlockE", "llvm::Value::isUsedInBasicBlock(llvm::BasicBlock const*) const"},
    {"_ZNK4llvm5Value10getNumUsesEv", "llvm::Value::getNumUses() const"},
    {"_ZNK4llvm5Value7getNameEv", "llvm::Value::getName() const"},
    {"_ZNK4llvm5Value10getNameStrEv", "llvm::Value::getNameStr() const"},
    {"_ZN4llvm5Value7setNameERKNS_5TwineE", "llvm::Value::setName(llvm::Twine const&)"},
    {"_ZN4llvm5Value8takeNameEPS0_", "llvm::Value::takeName(llvm::Value*)"},
    {"_ZN4llvm5Value27uncheckedReplaceAllUsesWithEPS0_", "llvm::Value::uncheckedReplaceAllUsesWith(llvm::Value*)"},
    {"_ZN4llvm15ValueHandleBase12ValueIsRAUWdEPNS_5ValueES2_", "llvm::ValueHandleBase::ValueIsRAUWd(llvm::Value*, llvm::Value*)"},
    {"_ZN4llvm5Value18replaceAllUsesWithEPS0_", "llvm::Value::replaceAllUsesWith(llvm::Value*)"},
    {"_ZN4llvm5Value17stripPointerCastsEv", "llvm::Value::stripPointerCasts()"},
    {"_ZNK4llvm5Value24isDereferenceablePointerEv", "llvm::Value::isDereferenceablePointer() const"},
    {"_ZN4llvm5Value16DoPHITranslationEPKNS_10BasicBlockES3_", "llvm::Value::DoPHITranslation(llvm::BasicBlock const*, llvm::BasicBlock const*)"},
    {"_ZNK4llvm5Value10getContextEv", "llvm::Value::getContext() const"},
    {"_ZN4llvm15ValueHandleBase20AddToExistingUseListEPPS0_", "llvm::ValueHandleBase::AddToExistingUseList(llvm::ValueHandleBase**)"},
    {"_ZN4llvm15ValueHandleBase25AddToExistingUseListAfterEPS0_", "llvm::ValueHandleBase::AddToExistingUseListAfter(llvm::ValueHandleBase*)"},
    {"_ZN4llvm15ValueHandleBase12AddToUseListEv", "llvm::ValueHandleBase::AddToUseList()"},
    {"_ZN4llvm15ValueHandleBase17RemoveFromUseListEv", "llvm::ValueHandleBase::RemoveFromUseList()"},
    {"_ZN4llvm10CallbackVHD0Ev", "llvm::CallbackVH::~CallbackVH()"},
    {"_ZN4llvm10CallbackVHD1Ev", "llvm::CallbackVH::~CallbackVH()"},
    {"_ZN4llvm10CallbackVHD2Ev", "llvm::CallbackVH::~CallbackVH()"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_15ValueHandleBaseENS_12DenseMapInfoIS2_EENS5_IS4_EEE16InsertIntoBucketERKS2_RKS4_PSt4pairIS2_S4_E", "llvm::DenseMap<llvm::Value*, llvm::ValueHandleBase*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::ValueHandleBase*> >::InsertIntoBucket(llvm::Value* const&, llvm::ValueHandleBase* const&, std::pair<llvm::Value*, llvm::ValueHandleBase*>*)"},
    {"_ZN4llvm8DenseMapIPNS_5ValueEPNS_15ValueHandleBaseENS_12DenseMapInfoIS2_EENS5_IS4_EEE4growEj", "llvm::DenseMap<llvm::Value*, llvm::ValueHandleBase*, llvm::DenseMapInfo<llvm::Value*>, llvm::DenseMapInfo<llvm::ValueHandleBase*> >::grow(unsigned int)"},
    {"_ZN4llvm16ValueSymbolTableD1Ev", "llvm::ValueSymbolTable::~ValueSymbolTable()"},
    {"_ZN4llvm16ValueSymbolTableD2Ev", "llvm::ValueSymbolTable::~ValueSymbolTable()"},
    {"_ZN4llvm16ValueSymbolTable13reinsertValueEPNS_5ValueE", "llvm::ValueSymbolTable::reinsertValue(llvm::Value*)"},
    {"_ZN4llvm16ValueSymbolTable15removeValueNameEPNS_14StringMapEntryIPNS_5ValueEEE", "llvm::ValueSymbolTable::removeValueName(llvm::StringMapEntry<llvm::Value*>*)"},
    {"_ZN4llvm16ValueSymbolTable15createValueNameENS_9StringRefEPNS_5ValueE", "llvm::ValueSymbolTable::createValueName(llvm::StringRef, llvm::Value*)"},
    {"_ZNK4llvm16ValueSymbolTable4dumpEv", "llvm::ValueSymbolTable::dump() const"},
    {"_ZN4llvm9StringMapIPNS_5ValueENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::Value*>& llvm::StringMap<llvm::Value*, llvm::MallocAllocator>::GetOrCreateValue<llvm::Value*>(llvm::StringRef, llvm::Value*)"},
    {"_ZN4llvm3EVT20getExtendedIntegerVTERNS_11LLVMContextEj", "llvm::EVT::getExtendedIntegerVT(llvm::LLVMContext&, unsigned int)"},
    {"_ZN4llvm3EVT19getExtendedVectorVTERNS_11LLVMContextES0_j", "llvm::EVT::getExtendedVectorVT(llvm::LLVMContext&, llvm::EVT, unsigned int)"},
    {"_ZNK4llvm3EVT13getTypeForEVTERNS_11LLVMContextE", "llvm::EVT::getTypeForEVT(llvm::LLVMContext&) const"},
    {"_ZNK4llvm3EVT23isExtendedFloatingPointEv", "llvm::EVT::isExtendedFloatingPoint() const"},
    {"_ZNK4llvm3EVT17isExtendedIntegerEv", "llvm::EVT::isExtendedInteger() const"},
    {"_ZNK4llvm3EVT16isExtendedVectorEv", "llvm::EVT::isExtendedVector() const"},
    {"_ZNK4llvm3EVT21isExtended64BitVectorEv", "llvm::EVT::isExtended64BitVector() const"},
    {"_ZNK4llvm3EVT22isExtended128BitVectorEv", "llvm::EVT::isExtended128BitVector() const"},
    {"_ZNK4llvm3EVT22isExtended256BitVectorEv", "llvm::EVT::isExtended256BitVector() const"},
    {"_ZNK4llvm3EVT22isExtended512BitVectorEv", "llvm::EVT::isExtended512BitVector() const"},
    {"_ZNK4llvm3EVT28getExtendedVectorElementTypeEv", "llvm::EVT::getExtendedVectorElementType() const"},
    {"_ZN4llvm3EVT6getEVTEPKNS_4TypeEb", "llvm::EVT::getEVT(llvm::Type const*, bool)"},
    {"_ZNK4llvm3EVT28getExtendedVectorNumElementsEv", "llvm::EVT::getExtendedVectorNumElements() const"},
    {"_ZNK4llvm3EVT21getExtendedSizeInBitsEv", "llvm::EVT::getExtendedSizeInBits() const"},
    {"_ZNK4llvm3EVT12getEVTStringEv", "llvm::EVT::getEVTString() const"},
    {"_ZN4llvm25initializePreVerifierPassERNS_12PassRegistryE", "llvm::initializePreVerifierPass(llvm::PassRegistry&)"},
    {"_ZN4llvm22initializeVerifierPassERNS_12PassRegistryE", "llvm::initializeVerifierPass(llvm::PassRegistry&)"},
    {"_ZN4llvm18createVerifierPassENS_21VerifierFailureActionE", "llvm::createVerifierPass(llvm::VerifierFailureAction)"},
    {"_ZN4llvm14verifyFunctionERKNS_8FunctionENS_21VerifierFailureActionE", "llvm::verifyFunction(llvm::Function const&, llvm::VerifierFailureAction)"},
    {"_ZN4llvm12verifyModuleERKNS_6ModuleENS_21VerifierFailureActionEPSs", "llvm::verifyModule(llvm::Module const&, llvm::VerifierFailureAction, std::string*)"},
    {"_ZN12_GLOBAL__N_18VerifierD1Ev", "(anonymous namespace)::Verifier::~Verifier()"},
    {"_ZN12_GLOBAL__N_18VerifierD0Ev", "(anonymous namespace)::Verifier::~Verifier()"},
    {"_ZNK12_GLOBAL__N_18Verifier16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::Verifier::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_18Verifier16doInitializationERN4llvm6ModuleE", "(anonymous namespace)::Verifier::doInitialization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_18Verifier13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::Verifier::runOnFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_18Verifier14doFinalizationERN4llvm6ModuleE", "(anonymous namespace)::Verifier::doFinalization(llvm::Module&)"},
    {"_ZN12_GLOBAL__N_18Verifier16visitGlobalValueERN4llvm11GlobalValueE", "(anonymous namespace)::Verifier::visitGlobalValue(llvm::GlobalValue&)"},
    {"_ZN12_GLOBAL__N_18Verifier13visitFunctionERN4llvm8FunctionE", "(anonymous namespace)::Verifier::visitFunction(llvm::Function&)"},
    {"_ZN12_GLOBAL__N_18Verifier13abortIfBrokenEv", "(anonymous namespace)::Verifier::abortIfBroken()"},
    {"_ZN12_GLOBAL__N_18Verifier11CheckFailedERKN4llvm5TwineEPKNS1_5ValueES7_S7_S7_", "(anonymous namespace)::Verifier::CheckFailed(llvm::Twine const&, llvm::Value const*, llvm::Value const*, llvm::Value const*, llvm::Value const*)"},
    {"_ZN12_GLOBAL__N_18Verifier11visitMDNodeERN4llvm6MDNodeEPNS1_8FunctionE", "(anonymous namespace)::Verifier::visitMDNode(llvm::MDNode&, llvm::Function*)"},
    {"_ZN12_GLOBAL__N_18Verifier11CheckFailedERKN4llvm5TwineEPKNS1_5ValueEPKNS1_4TypeES7_", "(anonymous namespace)::Verifier::CheckFailed(llvm::Twine const&, llvm::Value const*, llvm::Type const*, llvm::Value const*)"},
    {"_ZN12_GLOBAL__N_18Verifier19VerifyFunctionAttrsEPKN4llvm12FunctionTypeERKNS1_11AttrListPtrEPKNS1_5ValueE", "(anonymous namespace)::Verifier::VerifyFunctionAttrs(llvm::FunctionType const*, llvm::AttrListPtr const&, llvm::Value const*)"},
    {"_ZN12_GLOBAL__N_18Verifier20VerifyParameterAttrsEjPKN4llvm4TypeEbPKNS1_5ValueE", "(anonymous namespace)::Verifier::VerifyParameterAttrs(unsigned int, llvm::Type const*, bool, llvm::Value const*)"},
    {"_ZN4llvm11InstVisitorIN12_GLOBAL__N_18VerifierEvE5visitERNS_11InstructionE", "llvm::InstVisitor<(anonymous namespace)::Verifier, void>::visit(llvm::Instruction&)"},
    {"_ZN12_GLOBAL__N_18Verifier16visitInstructionERN4llvm11InstructionE", "(anonymous namespace)::Verifier::visitInstruction(llvm::Instruction&)"},
    {"_ZN12_GLOBAL__N_18Verifier10VerifyTypeEPKN4llvm4TypeE", "(anonymous namespace)::Verifier::VerifyType(llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_18Verifier11CheckFailedERKN4llvm5TwineEPKNS1_4TypeES7_S7_", "(anonymous namespace)::Verifier::CheckFailed(llvm::Twine const&, llvm::Type const*, llvm::Type const*, llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_18Verifier14VerifyCallSiteEN4llvm8CallSiteE", "(anonymous namespace)::Verifier::VerifyCallSite(llvm::CallSite)"},
    {"_ZN12_GLOBAL__N_18Verifier26visitIntrinsicFunctionCallEN4llvm9Intrinsic2IDERNS1_8CallInstE", "(anonymous namespace)::Verifier::visitIntrinsicFunctionCall(llvm::Intrinsic::ID, llvm::CallInst&)"},
    {"_ZN12_GLOBAL__N_18Verifier24VerifyIntrinsicPrototypeEN4llvm9Intrinsic2IDEPNS1_8FunctionEjjz", "(anonymous namespace)::Verifier::VerifyIntrinsicPrototype(llvm::Intrinsic::ID, llvm::Function*, unsigned int, unsigned int, ...)"},
    {"_ZN12_GLOBAL__N_18Verifier16PerformTypeCheckEN4llvm9Intrinsic2IDEPNS1_8FunctionEPKNS1_4TypeEijRSs", "(anonymous namespace)::Verifier::PerformTypeCheck(llvm::Intrinsic::ID, llvm::Function*, llvm::Type const*, int, unsigned int, std::string&)"},
    {"_Z14IntrinsicParamjj", "IntrinsicParam(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_18Verifier19visitBinaryOperatorERN4llvm14BinaryOperatorE", "(anonymous namespace)::Verifier::visitBinaryOperator(llvm::BinaryOperator&)"},
    {"_ZN12_GLOBAL__N_17TypeSetD1Ev", "(anonymous namespace)::TypeSet::~TypeSet()"},
    {"_ZN12_GLOBAL__N_17TypeSetD0Ev", "(anonymous namespace)::TypeSet::~TypeSet()"},
    {"_ZN12_GLOBAL__N_17TypeSet18refineAbstractTypeEPKN4llvm11DerivedTypeEPKNS1_4TypeE", "(anonymous namespace)::TypeSet::refineAbstractType(llvm::DerivedType const*, llvm::Type const*)"},
    {"_ZN12_GLOBAL__N_17TypeSet18typeBecameConcreteEPKN4llvm11DerivedTypeE", "(anonymous namespace)::TypeSet::typeBecameConcrete(llvm::DerivedType const*)"},
    {"_ZNK12_GLOBAL__N_17TypeSet4dumpEv", "(anonymous namespace)::TypeSet::dump() const"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_18VerifierEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::Verifier>()"},
    {"_ZN4llvm15callDefaultCtorIN12_GLOBAL__N_111PreVerifierEEEPNS_4PassEv", "llvm::Pass* llvm::callDefaultCtor<(anonymous namespace)::PreVerifier>()"},
    {"_ZN12_GLOBAL__N_111PreVerifierD1Ev", "(anonymous namespace)::PreVerifier::~PreVerifier()"},
    {"_ZN12_GLOBAL__N_111PreVerifierD0Ev", "(anonymous namespace)::PreVerifier::~PreVerifier()"},
    {"_ZNK12_GLOBAL__N_111PreVerifier16getAnalysisUsageERN4llvm13AnalysisUsageE", "(anonymous namespace)::PreVerifier::getAnalysisUsage(llvm::AnalysisUsage&) const"},
    {"_ZN12_GLOBAL__N_111PreVerifier13runOnFunctionERN4llvm8FunctionE", "(anonymous namespace)::PreVerifier::runOnFunction(llvm::Function&)"},
    {"_ZSt16__introsort_loopIPSt4pairIPN4llvm10BasicBlockEPNS1_5ValueEElEvT_S8_T0_", "void std::__introsort_loop<std::pair<llvm::BasicBlock*, llvm::Value*>*, long>(std::pair<llvm::BasicBlock*, llvm::Value*>*, std::pair<llvm::BasicBlock*, llvm::Value*>*, long)"},
    {"_ZSt22__final_insertion_sortIPSt4pairIPN4llvm10BasicBlockEPNS1_5ValueEEEvT_S8_", "void std::__final_insertion_sort<std::pair<llvm::BasicBlock*, llvm::Value*>*>(std::pair<llvm::BasicBlock*, llvm::Value*>*, std::pair<llvm::BasicBlock*, llvm::Value*>*)"},
    {"_ZSt16__insertion_sortIPSt4pairIPN4llvm10BasicBlockEPNS1_5ValueEEEvT_S8_", "void std::__insertion_sort<std::pair<llvm::BasicBlock*, llvm::Value*>*>(std::pair<llvm::BasicBlock*, llvm::Value*>*, std::pair<llvm::BasicBlock*, llvm::Value*>*)"},
    {"_ZSt13__heap_selectIPSt4pairIPN4llvm10BasicBlockEPNS1_5ValueEEEvT_S8_S8_", "void std::__heap_select<std::pair<llvm::BasicBlock*, llvm::Value*>*>(std::pair<llvm::BasicBlock*, llvm::Value*>*, std::pair<llvm::BasicBlock*, llvm::Value*>*, std::pair<llvm::BasicBlock*, llvm::Value*>*)"},
    {"_ZSt13__adjust_heapIPSt4pairIPN4llvm10BasicBlockEPNS1_5ValueEElS6_EvT_T0_S9_T1_", "void std::__adjust_heap<std::pair<llvm::BasicBlock*, llvm::Value*>*, long, std::pair<llvm::BasicBlock*, llvm::Value*> >(std::pair<llvm::BasicBlock*, llvm::Value*>*, long, long, std::pair<llvm::BasicBlock*, llvm::Value*>)"},
    {"_ZN4llvm15SmallVectorImplIPNS_10BasicBlockEE6appendINS_12PredIteratorIS1_NS_18value_use_iteratorINS_4UserEEEEEEEvT_SA_", "void llvm::SmallVectorImpl<llvm::BasicBlock*>::append<llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> > >(llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >, llvm::PredIterator<llvm::BasicBlock, llvm::value_use_iterator<llvm::User> >)"},
    {"_ZSt6__findIPPKN4llvm4TypeES3_ET_S5_S5_RKT0_St26random_access_iterator_tag", "llvm::Type const** std::__find<llvm::Type const**, llvm::Type const*>(llvm::Type const**, llvm::Type const**, llvm::Type const* const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm14ARMInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::ARMInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter12printOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter24printSBitModifierOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printSBitModifierOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printPredicateOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printPredicateOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter17printSORegOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printSORegOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter15printPImmediateEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printPImmediate(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printCPSOptionOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printCPSOptionOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter15printMemBOptionEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printMemBOption(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter25printAddrModeImm12OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrModeImm12Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printAddrMode2OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode2Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printLdStmModeOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printLdStmModeOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter18printSetendOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printSetendOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter16printThumbITMaskEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbITMask(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter30printMandatoryPredicateOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printMandatoryPredicateOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter15printCImmediateEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printCImmediate(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter20printNoHashImmediateEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printNoHashImmediate(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter19printMSRMaskOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printMSRMaskOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter22printThumbS4ImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbS4ImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter17printSOImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printSOImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter22printNEONModImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printNEONModImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printAddrMode5OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode5Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter26printT2AddrModeImm8OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2AddrModeImm8Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printT2AddrModeSoRegOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2AddrModeSoRegOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter17printRegisterListEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printRegisterList(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter30printBitfieldInvMaskImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printBitfieldInvMaskImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printAddrMode3OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode3Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter16printT2SOOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2SOOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter31printThumbAddrModeImm5S1OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbAddrModeImm5S1Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printThumbAddrModeRROperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbAddrModeRROperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter31printThumbAddrModeImm5S2OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbAddrModeImm5S2Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter31printThumbAddrModeImm5S4OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbAddrModeImm5S4Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printThumbAddrModeSPOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printThumbAddrModeSPOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter28printT2AddrModeImm8s4OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2AddrModeImm8s4Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printVFPf64ImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printVFPf64ImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printVFPf32ImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printVFPf32ImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printAddrMode2OffsetOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode2OffsetOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printAddrMode3OffsetOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode3OffsetOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter32printT2AddrModeImm8OffsetOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2AddrModeImm8OffsetOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter20printShiftImmOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printShiftImmOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter21printAddrMode6OperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode6Operand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter27printAddrMode6OffsetOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printAddrMode6OffsetOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter34printT2AddrModeImm8s4OffsetOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printT2AddrModeImm8s4OffsetOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter15getRegisterNameEj", "llvm::ARMInstPrinter::getRegisterName(unsigned int)"},
    {"_ZN4llvm14ARMInstPrinter18getInstructionNameEj", "llvm::ARMInstPrinter::getInstructionName(unsigned int)"},
    {"_ZNK4llvm14ARMInstPrinter13getOpcodeNameEj", "llvm::ARMInstPrinter::getOpcodeName(unsigned int) const"},
    {"_ZN4llvm14ARMInstPrinter9printInstEPKNS_6MCInstERNS_11raw_ostreamE", "llvm::ARMInstPrinter::printInst(llvm::MCInst const*, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter19printNegZeroOperandEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printNegZeroOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter12printPCLabelEPKNS_6MCInstEjRNS_11raw_ostreamE", "llvm::ARMInstPrinter::printPCLabel(llvm::MCInst const*, unsigned int, llvm::raw_ostream&)"},
    {"_ZN4llvm14ARMInstPrinter30printThumbAddrModeImm5SOperandEPKNS_6MCInstEjRNS_11raw_ostreamEj", "llvm::ARMInstPrinter::printThumbAddrModeImm5SOperand(llvm::MCInst const*, unsigned int, llvm::raw_ostream&, unsigned int)"},
    {"_ZN4llvm14ARMInstPrinterD1Ev", "llvm::ARMInstPrinter::~ARMInstPrinter()"},
    {"_ZN4llvm14ARMInstPrinterD0Ev", "llvm::ARMInstPrinter::~ARMInstPrinter()"},
    {"_ZN4llvm14RegisterTargetILNS_6Triple8ArchTypeE14ELb1EE21getTripleMatchQualityERKSs", "llvm::RegisterTarget<(llvm::Triple::ArchType)14, true>::getTripleMatchQuality(std::string const&)"},
    {"_ZN4llvm14RegisterTargetILNS_6Triple8ArchTypeE2ELb1EE21getTripleMatchQualityERKSs", "llvm::RegisterTarget<(llvm::Triple::ArchType)2, true>::getTripleMatchQuality(std::string const&)"},
    {"_ZN4llvm21createELFObjectWriterEPNS_23MCELFObjectTargetWriterERNS_11raw_ostreamEb", "llvm::createELFObjectWriter(llvm::MCELFObjectTargetWriter*, llvm::raw_ostream&, bool)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriterC2EPN4llvm23MCELFObjectTargetWriterERNS1_11raw_ostreamEb", "(anonymous namespace)::ELFObjectWriter::ELFObjectWriter(llvm::MCELFObjectTargetWriter*, llvm::raw_ostream&, bool)"},
    {"_ZN12_GLOBAL__N_118X86ELFObjectWriterD1Ev", "(anonymous namespace)::X86ELFObjectWriter::~X86ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_118X86ELFObjectWriterD0Ev", "(anonymous namespace)::X86ELFObjectWriter::~X86ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter24ExecutePostLayoutBindingERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::ELFObjectWriter::ExecutePostLayoutBinding(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter16RecordRelocationERKN4llvm11MCAssemblerERKNS1_11MCAsmLayoutEPKNS1_10MCFragmentERKNS1_7MCFixupENS1_7MCValueERy", "(anonymous namespace)::ELFObjectWriter::RecordRelocation(llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::MCFragment const*, llvm::MCFixup const&, llvm::MCValue, unsigned long long&)"},
    {"_ZNK12_GLOBAL__N_115ELFObjectWriter38IsSymbolRefDifferenceFullyResolvedImplERKN4llvm11MCAssemblerERKNS1_12MCSymbolDataERKNS1_10MCFragmentEbb", "(anonymous namespace)::ELFObjectWriter::IsSymbolRefDifferenceFullyResolvedImpl(llvm::MCAssembler const&, llvm::MCSymbolData const&, llvm::MCFragment const&, bool, bool) const"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter11WriteObjectERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::ELFObjectWriter::WriteObject(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZNK12_GLOBAL__N_115ELFObjectWriter14ExplicitRelSymERKN4llvm11MCAssemblerERKNS1_7MCValueERKNS1_10MCFragmentEb", "(anonymous namespace)::ELFObjectWriter::ExplicitRelSym(llvm::MCAssembler const&, llvm::MCValue const&, llvm::MCFragment const&, bool) const"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter11WriteHeaderEyj", "(anonymous namespace)::ELFObjectWriter::WriteHeader(unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter11WriteEFlagsEv", "(anonymous namespace)::ELFObjectWriter::WriteEFlags()"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter16WriteSymbolEntryEPN4llvm14MCDataFragmentES3_yhyyhjb", "(anonymous namespace)::ELFObjectWriter::WriteSymbolEntry(llvm::MCDataFragment*, llvm::MCDataFragment*, unsigned long long, unsigned char, unsigned long long, unsigned long long, unsigned char, unsigned int, bool)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter11WriteSymbolEPN4llvm14MCDataFragmentES3_RNS0_13ELFSymbolDataERKNS1_11MCAsmLayoutE", "(anonymous namespace)::ELFObjectWriter::WriteSymbol(llvm::MCDataFragment*, llvm::MCDataFragment*, (anonymous namespace)::ELFObjectWriter::ELFSymbolData&, llvm::MCAsmLayout const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter16WriteSymbolTableEPN4llvm14MCDataFragmentES3_RKNS1_11MCAssemblerERKNS1_11MCAsmLayoutERKNS1_8DenseMapIPKNS1_12MCSectionELFEjNS1_12DenseMapInfoISD_EENSE_IjEEEE", "(anonymous namespace)::ELFObjectWriter::WriteSymbolTable(llvm::MCDataFragment*, llvm::MCDataFragment*, llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> > const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter27getSymbolIndexInSymbolTableERKN4llvm11MCAssemblerEPKNS1_8MCSymbolE", "(anonymous namespace)::ELFObjectWriter::getSymbolIndexInSymbolTable(llvm::MCAssembler const&, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter18ComputeSymbolTableERN4llvm11MCAssemblerERKNS1_8DenseMapIPKNS1_12MCSectionELFEjNS1_12DenseMapInfoIS7_EENS8_IjEEEENS4_IPKNS1_8MCSymbolES7_NS8_ISG_EES9_EE", "(anonymous namespace)::ELFObjectWriter::ComputeSymbolTable(llvm::MCAssembler&, llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> > const&, llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> >)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter15ComputeIndexMapERN4llvm11MCAssemblerERNS1_8DenseMapIPKNS1_12MCSectionELFEjNS1_12DenseMapInfoIS7_EENS8_IjEEEE", "(anonymous namespace)::ELFObjectWriter::ComputeIndexMap(llvm::MCAssembler&, llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> >&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter15WriteRelocationERN4llvm11MCAssemblerERNS1_11MCAsmLayoutERKNS1_13MCSectionDataE", "(anonymous namespace)::ELFObjectWriter::WriteRelocation(llvm::MCAssembler&, llvm::MCAsmLayout&, llvm::MCSectionData const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter16WriteRelocationsERN4llvm11MCAssemblerERNS1_11MCAsmLayoutE", "(anonymous namespace)::ELFObjectWriter::WriteRelocations(llvm::MCAssembler&, llvm::MCAsmLayout&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter22CreateMetadataSectionsERN4llvm11MCAssemblerERNS1_11MCAsmLayoutERKNS1_8DenseMapIPKNS1_12MCSectionELFEjNS1_12DenseMapInfoIS9_EENSA_IjEEEE", "(anonymous namespace)::ELFObjectWriter::CreateMetadataSections(llvm::MCAssembler&, llvm::MCAsmLayout&, llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> > const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter21CreateIndexedSectionsERN4llvm11MCAssemblerERNS1_11MCAsmLayoutERNS1_8DenseMapIPKNS1_12MCSectionELFEPKNS1_8MCSymbolENS1_12DenseMapInfoIS9_EENSD_ISC_EEEERNS6_ISC_S9_SF_SE_EE", "(anonymous namespace)::ELFObjectWriter::CreateIndexedSections(llvm::MCAssembler&, llvm::MCAsmLayout&, llvm::DenseMap<llvm::MCSectionELF const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >&, llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> >&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter16WriteSecHdrEntryEjjyyyyjjyy", "(anonymous namespace)::ELFObjectWriter::WriteSecHdrEntry(unsigned int, unsigned int, unsigned long long, unsigned long long, unsigned long long, unsigned long long, unsigned int, unsigned int, unsigned long long, unsigned long long)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter24WriteRelocationsFragmentERKN4llvm11MCAssemblerEPNS1_14MCDataFragmentEPKNS1_13MCSectionDataE", "(anonymous namespace)::ELFObjectWriter::WriteRelocationsFragment(llvm::MCAssembler const&, llvm::MCDataFragment*, llvm::MCSectionData const*)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter12WriteSectionERN4llvm11MCAssemblerERKNS1_8DenseMapIPKNS1_12MCSectionELFEjNS1_12DenseMapInfoIS7_EENS8_IjEEEEjyyyRS6_", "(anonymous namespace)::ELFObjectWriter::WriteSection(llvm::MCAssembler&, llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> > const&, unsigned int, unsigned long long, unsigned long long, unsigned long long, llvm::MCSectionELF const&)"},
    {"_ZN12_GLOBAL__N_118X86ELFObjectWriter12GetRelocTypeERKN4llvm7MCValueERKNS1_7MCFixupEbbx", "(anonymous namespace)::X86ELFObjectWriter::GetRelocType(llvm::MCValue const&, llvm::MCFixup const&, bool, bool, long long)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataESt6vectorIN12_GLOBAL__N_115ELFObjectWriter18ELFRelocationEntryESaIS7_EENS_12DenseMapInfoIS3_EENSA_IS9_EEEixERKS3_", "llvm::DenseMap<llvm::MCSectionData const*, std::vector<(anonymous namespace)::ELFObjectWriter::ELFRelocationEntry, std::allocator<(anonymous namespace)::ELFObjectWriter::ELFRelocationEntry> >, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<std::vector<(anonymous namespace)::ELFObjectWriter::ELFRelocationEntry, std::allocator<(anonymous namespace)::ELFObjectWriter::ELFRelocationEntry> > > >::operator[](llvm::MCSectionData const* const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriter8String64ERN4llvm14MCDataFragmentEy", "(anonymous namespace)::ELFObjectWriter::String64(llvm::MCDataFragment&, unsigned long long)"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_115ELFObjectWriter18ELFRelocationEntryEEEiPKvS5_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::ELFObjectWriter::ELFRelocationEntry>(void const*, void const*)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_115ELFObjectWriter13ELFSymbolDataESaIS2_EE9push_backERKS2_", "std::vector<(anonymous namespace)::ELFObjectWriter::ELFSymbolData, std::allocator<(anonymous namespace)::ELFObjectWriter::ELFSymbolData> >::push_back((anonymous namespace)::ELFObjectWriter::ELFSymbolData const&)"},
    {"_ZN4llvm25array_pod_sort_comparatorIN12_GLOBAL__N_115ELFObjectWriter13ELFSymbolDataEEEiPKvS5_", "int llvm::array_pod_sort_comparator<(anonymous namespace)::ELFObjectWriter::ELFSymbolData>(void const*, void const*)"},
    {"_Z15DataSectionSizeRKN4llvm13MCSectionDataE", "DataSectionSize(llvm::MCSectionData const&)"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriterD2Ev", "(anonymous namespace)::ELFObjectWriter::~ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriterD1Ev", "(anonymous namespace)::ELFObjectWriter::~ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_115ELFObjectWriterD0Ev", "(anonymous namespace)::ELFObjectWriter::~ELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_121MBlazeELFObjectWriterD1Ev", "(anonymous namespace)::MBlazeELFObjectWriter::~MBlazeELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_121MBlazeELFObjectWriterD0Ev", "(anonymous namespace)::MBlazeELFObjectWriter::~MBlazeELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_121MBlazeELFObjectWriter12GetRelocTypeERKN4llvm7MCValueERKNS1_7MCFixupEbbx", "(anonymous namespace)::MBlazeELFObjectWriter::GetRelocType(llvm::MCValue const&, llvm::MCFixup const&, bool, bool, long long)"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriterD1Ev", "(anonymous namespace)::ARMELFObjectWriter::~ARMELFObjectWriter()"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriterD0Ev", "(anonymous namespace)::ARMELFObjectWriter::~ARMELFObjectWriter()"},
    {"_ZNK12_GLOBAL__N_118ARMELFObjectWriter14ExplicitRelSymERKN4llvm11MCAssemblerERKNS1_7MCValueERKNS1_10MCFragmentEb", "(anonymous namespace)::ARMELFObjectWriter::ExplicitRelSym(llvm::MCAssembler const&, llvm::MCValue const&, llvm::MCFragment const&, bool) const"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriter11WriteEFlagsEv", "(anonymous namespace)::ARMELFObjectWriter::WriteEFlags()"},
    {"_ZN12_GLOBAL__N_118ARMELFObjectWriter12GetRelocTypeERKN4llvm7MCValueERKNS1_7MCFixupEbbx", "(anonymous namespace)::ARMELFObjectWriter::GetRelocType(llvm::MCValue const&, llvm::MCFixup const&, bool, bool, long long)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEyNS_12DenseMapInfoIS3_EENS4_IyEEE16InsertIntoBucketERKS3_RKyPSt4pairIS3_yE", "llvm::DenseMap<llvm::MCSection const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(llvm::MCSection const* const&, unsigned long long const&, std::pair<llvm::MCSection const*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEyNS_12DenseMapInfoIS3_EENS4_IyEEE4growEj", "llvm::DenseMap<llvm::MCSection const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_12MCSectionELFEPKNS_8MCSymbolENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PSt4pairIS3_S6_E", "llvm::DenseMap<llvm::MCSectionELF const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::InsertIntoBucket(llvm::MCSectionELF const* const&, llvm::MCSymbol const* const&, std::pair<llvm::MCSectionELF const*, llvm::MCSymbol const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MCSectionELFEPKNS_8MCSymbolENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::MCSectionELF const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPKNS_12MCSectionELFENS_12DenseMapInfoIS3_EENS7_IS6_EEE16InsertIntoBucketERKS3_RKS6_PSt4pairIS3_S6_E", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> >::InsertIntoBucket(llvm::MCSymbol const* const&, llvm::MCSectionELF const* const&, std::pair<llvm::MCSymbol const*, llvm::MCSectionELF const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPKNS_12MCSectionELFENS_12DenseMapInfoIS3_EENS7_IS6_EEE4growEj", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_12MCSectionELFEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MCSectionELF const* const&, unsigned int const&, std::pair<llvm::MCSectionELF const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_12MCSectionELFEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::MCSectionELF const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionELF const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZNSt6vectorIPKN4llvm12MCSectionELFESaIS3_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS3_S5_EEmRKS3_", "std::vector<llvm::MCSectionELF const*, std::allocator<llvm::MCSectionELF const*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::MCSectionELF const**, std::vector<llvm::MCSectionELF const*, std::allocator<llvm::MCSectionELF const*> > >, unsigned long, llvm::MCSectionELF const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPKNS_12MCSectionELFENS_12DenseMapInfoIS3_EENS7_IS6_EEE8CopyFromERKSA_", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> >::CopyFrom(llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSectionELF const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSectionELF const*> > const&)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolES3_NS_12DenseMapInfoIS3_EES5_E6insertERKSt4pairIS3_S3_E", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::insert(std::pair<llvm::MCSymbol const*, llvm::MCSymbol const*> const&)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolES3_NS_12DenseMapInfoIS3_EES5_E16InsertIntoBucketERKS3_S8_PSt4pairIS3_S3_E", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::InsertIntoBucket(llvm::MCSymbol const* const&, llvm::MCSymbol const* const&, std::pair<llvm::MCSymbol const*, llvm::MCSymbol const*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolES3_NS_12DenseMapInfoIS3_EES5_E4growEj", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSymbol const*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::grow(unsigned int)"},
    {"_ZN4llvm9MCAsmInfoC1Ev", "llvm::MCAsmInfo::MCAsmInfo()"},
    {"_ZN4llvm9MCAsmInfoC2Ev", "llvm::MCAsmInfo::MCAsmInfo()"},
    {"_ZN4llvm9MCAsmInfoD0Ev", "llvm::MCAsmInfo::~MCAsmInfo()"},
    {"_ZN4llvm9MCAsmInfoD1Ev", "llvm::MCAsmInfo::~MCAsmInfo()"},
    {"_ZN4llvm9MCAsmInfoD2Ev", "llvm::MCAsmInfo::~MCAsmInfo()"},
    {"_ZN4llvm9MCAsmInfo14getULEB128SizeEj", "llvm::MCAsmInfo::getULEB128Size(unsigned int)"},
    {"_ZN4llvm9MCAsmInfo14getSLEB128SizeEi", "llvm::MCAsmInfo::getSLEB128Size(int)"},
    {"_ZN4llvm13MCAsmInfoCOFFC1Ev", "llvm::MCAsmInfoCOFF::MCAsmInfoCOFF()"},
    {"_ZN4llvm13MCAsmInfoCOFFC2Ev", "llvm::MCAsmInfoCOFF::MCAsmInfoCOFF()"},
    {"_ZN4llvm15MCAsmInfoDarwinC1Ev", "llvm::MCAsmInfoDarwin::MCAsmInfoDarwin()"},
    {"_ZN4llvm15MCAsmInfoDarwinC2Ev", "llvm::MCAsmInfoDarwin::MCAsmInfoDarwin()"},
    {"_ZN4llvm17createAsmStreamerERNS_9MCContextERNS_21formatted_raw_ostreamEbbPNS_13MCInstPrinterEPNS_13MCCodeEmitterEPNS_16TargetAsmBackendEb", "llvm::createAsmStreamer(llvm::MCContext&, llvm::formatted_raw_ostream&, bool, bool, llvm::MCInstPrinter*, llvm::MCCodeEmitter*, llvm::TargetAsmBackend*, bool)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamerD1Ev", "(anonymous namespace)::MCAsmStreamer::~MCAsmStreamer()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamerD0Ev", "(anonymous namespace)::MCAsmStreamer::~MCAsmStreamer()"},
    {"_ZNK12_GLOBAL__N_113MCAsmStreamer12isVerboseAsmEv", "(anonymous namespace)::MCAsmStreamer::isVerboseAsm() const"},
    {"_ZNK12_GLOBAL__N_113MCAsmStreamer17hasRawTextSupportEv", "(anonymous namespace)::MCAsmStreamer::hasRawTextSupport() const"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer10AddCommentERKN4llvm5TwineE", "(anonymous namespace)::MCAsmStreamer::AddComment(llvm::Twine const&)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer12GetCommentOSEv", "(anonymous namespace)::MCAsmStreamer::GetCommentOS()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer12AddBlankLineEv", "(anonymous namespace)::MCAsmStreamer::AddBlankLine()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer13SwitchSectionEPKN4llvm9MCSectionE", "(anonymous namespace)::MCAsmStreamer::SwitchSection(llvm::MCSection const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer12InitSectionsEv", "(anonymous namespace)::MCAsmStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::MCAsmStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::MCAsmStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::MCAsmStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCAsmStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer17EmitWeakReferenceEPN4llvm8MCSymbolEPKS2_", "(anonymous namespace)::MCAsmStreamer::EmitWeakReference(llvm::MCSymbol*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::MCAsmStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::MCAsmStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::MCAsmStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::MCAsmStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::MCAsmStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::MCAsmStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCAsmStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::MCAsmStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::MCAsmStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::MCAsmStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::MCAsmStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::MCAsmStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer13EmitValueImplEPKN4llvm6MCExprEjbj", "(anonymous namespace)::MCAsmStreamer::EmitValueImpl(llvm::MCExpr const*, unsigned int, bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer12EmitIntValueEyjj", "(anonymous namespace)::MCAsmStreamer::EmitIntValue(unsigned long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EmitULEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCAsmStreamer::EmitULEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EmitSLEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCAsmStreamer::EmitSLEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EmitGPRel32ValueEPKN4llvm6MCExprE", "(anonymous namespace)::MCAsmStreamer::EmitGPRel32Value(llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer8EmitFillEyhj", "(anonymous namespace)::MCAsmStreamer::EmitFill(unsigned long long, unsigned char, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::MCAsmStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::MCAsmStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer17EmitValueToOffsetEPKN4llvm6MCExprEh", "(anonymous namespace)::MCAsmStreamer::EmitValueToOffset(llvm::MCExpr const*, unsigned char)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::MCAsmStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer22EmitDwarfFileDirectiveEjN4llvm9StringRefE", "(anonymous namespace)::MCAsmStreamer::EmitDwarfFileDirective(unsigned int, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer21EmitDwarfLocDirectiveEjjjjjj", "(anonymous namespace)::MCAsmStreamer::EmitDwarfLocDirective(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer24EmitDwarfAdvanceLineAddrExPKN4llvm8MCSymbolES4_", "(anonymous namespace)::MCAsmStreamer::EmitDwarfAdvanceLineAddr(long long, llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer16EmitCFIStartProcEv", "(anonymous namespace)::MCAsmStreamer::EmitCFIStartProc()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer14EmitCFIEndProcEv", "(anonymous namespace)::MCAsmStreamer::EmitCFIEndProc()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer19EmitCFIDefCfaOffsetEx", "(anonymous namespace)::MCAsmStreamer::EmitCFIDefCfaOffset(long long)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer21EmitCFIDefCfaRegisterEx", "(anonymous namespace)::MCAsmStreamer::EmitCFIDefCfaRegister(long long)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer13EmitCFIOffsetExx", "(anonymous namespace)::MCAsmStreamer::EmitCFIOffset(long long, long long)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer18EmitCFIPersonalityEPKN4llvm8MCSymbolEj", "(anonymous namespace)::MCAsmStreamer::EmitCFIPersonality(llvm::MCSymbol const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer11EmitCFILsdaEPKN4llvm8MCSymbolEj", "(anonymous namespace)::MCAsmStreamer::EmitCFILsda(llvm::MCSymbol const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer15EmitInstructionERKN4llvm6MCInstE", "(anonymous namespace)::MCAsmStreamer::EmitInstruction(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer11EmitRawTextEN4llvm9StringRefE", "(anonymous namespace)::MCAsmStreamer::EmitRawText(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer6FinishEv", "(anonymous namespace)::MCAsmStreamer::Finish()"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer18AddEncodingCommentERKN4llvm6MCInstE", "(anonymous namespace)::MCAsmStreamer::AddEncodingComment(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_113MCAsmStreamer7EmitEOLEv", "(anonymous namespace)::MCAsmStreamer::EmitEOL()"},
    {"_Z17PrintQuotedStringN4llvm9StringRefERNS_11raw_ostreamE", "PrintQuotedString(llvm::StringRef, llvm::raw_ostream&)"},
    {"_ZN4llvm14format_object1IhED1Ev", "llvm::format_object1<unsigned char>::~format_object1()"},
    {"_ZNK4llvm14format_object1IhE7snprintEPcj", "llvm::format_object1<unsigned char>::snprint(char*, unsigned int) const"},
    {"_ZN4llvm14format_object1IhED0Ev", "llvm::format_object1<unsigned char>::~format_object1()"},
    {"_ZN4llvm11MCAsmLayoutC1ERNS_11MCAssemblerE", "llvm::MCAsmLayout::MCAsmLayout(llvm::MCAssembler&)"},
    {"_ZN4llvm11MCAsmLayoutC2ERNS_11MCAssemblerE", "llvm::MCAsmLayout::MCAsmLayout(llvm::MCAssembler&)"},
    {"_ZNK4llvm11MCAsmLayout18isFragmentUpToDateEPKNS_10MCFragmentE", "llvm::MCAsmLayout::isFragmentUpToDate(llvm::MCFragment const*) const"},
    {"_ZN4llvm11MCAsmLayout10InvalidateEPNS_10MCFragmentE", "llvm::MCAsmLayout::Invalidate(llvm::MCFragment*)"},
    {"_ZNK4llvm11MCAsmLayout11EnsureValidEPKNS_10MCFragmentE", "llvm::MCAsmLayout::EnsureValid(llvm::MCFragment const*) const"},
    {"_ZN4llvm11MCAsmLayout14LayoutFragmentEPNS_10MCFragmentE", "llvm::MCAsmLayout::LayoutFragment(llvm::MCFragment*)"},
    {"_ZNK4llvm11MCAsmLayout17getFragmentOffsetEPKNS_10MCFragmentE", "llvm::MCAsmLayout::getFragmentOffset(llvm::MCFragment const*) const"},
    {"_ZNK4llvm11MCAsmLayout15getSymbolOffsetEPKNS_12MCSymbolDataE", "llvm::MCAsmLayout::getSymbolOffset(llvm::MCSymbolData const*) const"},
    {"_ZNK4llvm11MCAsmLayout21getSectionAddressSizeEPKNS_13MCSectionDataE", "llvm::MCAsmLayout::getSectionAddressSize(llvm::MCSectionData const*) const"},
    {"_ZNK4llvm11MCAssembler19ComputeFragmentSizeERKNS_11MCAsmLayoutERKNS_10MCFragmentE", "llvm::MCAssembler::ComputeFragmentSize(llvm::MCAsmLayout const&, llvm::MCFragment const&) const"},
    {"_ZNK4llvm11MCAsmLayout18getSectionFileSizeEPKNS_13MCSectionDataE", "llvm::MCAsmLayout::getSectionFileSize(llvm::MCSectionData const*) const"},
    {"_ZN4llvm10MCFragmentC1Ev", "llvm::MCFragment::MCFragment()"},
    {"_ZN4llvm10MCFragmentC2Ev", "llvm::MCFragment::MCFragment()"},
    {"_ZN4llvm10MCFragmentD0Ev", "llvm::MCFragment::~MCFragment()"},
    {"_ZN4llvm10MCFragmentD1Ev", "llvm::MCFragment::~MCFragment()"},
    {"_ZN4llvm10MCFragmentD2Ev", "llvm::MCFragment::~MCFragment()"},
    {"_ZN4llvm10MCFragmentC1ENS0_12FragmentTypeEPNS_13MCSectionDataE", "llvm::MCFragment::MCFragment(llvm::MCFragment::FragmentType, llvm::MCSectionData*)"},
    {"_ZN4llvm10MCFragmentC2ENS0_12FragmentTypeEPNS_13MCSectionDataE", "llvm::MCFragment::MCFragment(llvm::MCFragment::FragmentType, llvm::MCSectionData*)"},
    {"_ZN4llvm13MCSectionDataC1Ev", "llvm::MCSectionData::MCSectionData()"},
    {"_ZN4llvm13MCSectionDataC2Ev", "llvm::MCSectionData::MCSectionData()"},
    {"_ZN4llvm13MCSectionDataC1ERKNS_9MCSectionEPNS_11MCAssemblerE", "llvm::MCSectionData::MCSectionData(llvm::MCSection const&, llvm::MCAssembler*)"},
    {"_ZN4llvm13MCSectionDataC2ERKNS_9MCSectionEPNS_11MCAssemblerE", "llvm::MCSectionData::MCSectionData(llvm::MCSection const&, llvm::MCAssembler*)"},
    {"_ZN4llvm12MCSymbolDataC1Ev", "llvm::MCSymbolData::MCSymbolData()"},
    {"_ZN4llvm12MCSymbolDataC2Ev", "llvm::MCSymbolData::MCSymbolData()"},
    {"_ZN4llvm12MCSymbolDataC1ERKNS_8MCSymbolEPNS_10MCFragmentEyPNS_11MCAssemblerE", "llvm::MCSymbolData::MCSymbolData(llvm::MCSymbol const&, llvm::MCFragment*, unsigned long long, llvm::MCAssembler*)"},
    {"_ZN4llvm12MCSymbolDataC2ERKNS_8MCSymbolEPNS_10MCFragmentEyPNS_11MCAssemblerE", "llvm::MCSymbolData::MCSymbolData(llvm::MCSymbol const&, llvm::MCFragment*, unsigned long long, llvm::MCAssembler*)"},
    {"_ZN4llvm11MCAssemblerC1ERNS_9MCContextERNS_16TargetAsmBackendERNS_13MCCodeEmitterERNS_14MCObjectWriterERNS_11raw_ostreamE", "llvm::MCAssembler::MCAssembler(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::MCCodeEmitter&, llvm::MCObjectWriter&, llvm::raw_ostream&)"},
    {"_ZN4llvm11MCAssemblerC2ERNS_9MCContextERNS_16TargetAsmBackendERNS_13MCCodeEmitterERNS_14MCObjectWriterERNS_11raw_ostreamE", "llvm::MCAssembler::MCAssembler(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::MCCodeEmitter&, llvm::MCObjectWriter&, llvm::raw_ostream&)"},
    {"_ZN4llvm11MCAssemblerD1Ev", "llvm::MCAssembler::~MCAssembler()"},
    {"_ZN4llvm11MCAssemblerD2Ev", "llvm::MCAssembler::~MCAssembler()"},
    {"_ZNK4llvm11MCAssembler21isSymbolLinkerVisibleERKNS_8MCSymbolE", "llvm::MCAssembler::isSymbolLinkerVisible(llvm::MCSymbol const&) const"},
    {"_ZNK4llvm11MCAssembler7getAtomEPKNS_12MCSymbolDataE", "llvm::MCAssembler::getAtom(llvm::MCSymbolData const*) const"},
    {"_ZNK4llvm11MCAssembler13EvaluateFixupERKNS_11MCAsmLayoutERKNS_7MCFixupEPKNS_10MCFragmentERNS_7MCValueERy", "llvm::MCAssembler::EvaluateFixup(llvm::MCAsmLayout const&, llvm::MCFixup const&, llvm::MCFragment const*, llvm::MCValue&, unsigned long long&) const"},
    {"_ZNK4llvm11MCAssembler16WriteSectionDataEPKNS_13MCSectionDataERKNS_11MCAsmLayoutE", "llvm::MCAssembler::WriteSectionData(llvm::MCSectionData const*, llvm::MCAsmLayout const&) const"},
    {"_ZN4llvm11MCAssembler11HandleFixupERKNS_11MCAsmLayoutERNS_10MCFragmentERKNS_7MCFixupE", "llvm::MCAssembler::HandleFixup(llvm::MCAsmLayout const&, llvm::MCFragment&, llvm::MCFixup const&)"},
    {"_ZN4llvm11MCAssembler6FinishEv", "llvm::MCAssembler::Finish()"},
    {"_ZN4llvm11MCAssembler10LayoutOnceERNS_11MCAsmLayoutE", "llvm::MCAssembler::LayoutOnce(llvm::MCAsmLayout&)"},
    {"_ZN4llvm11MCAssembler12FinishLayoutERNS_11MCAsmLayoutE", "llvm::MCAssembler::FinishLayout(llvm::MCAsmLayout&)"},
    {"_ZNK4llvm11MCAssembler20FixupNeedsRelaxationERKNS_7MCFixupEPKNS_10MCFragmentERKNS_11MCAsmLayoutE", "llvm::MCAssembler::FixupNeedsRelaxation(llvm::MCFixup const&, llvm::MCFragment const*, llvm::MCAsmLayout const&) const"},
    {"_ZNK4llvm11MCAssembler23FragmentNeedsRelaxationEPKNS_14MCInstFragmentERKNS_11MCAsmLayoutE", "llvm::MCAssembler::FragmentNeedsRelaxation(llvm::MCInstFragment const*, llvm::MCAsmLayout const&) const"},
    {"_ZN4llvm11MCAssembler16RelaxInstructionERNS_11MCAsmLayoutERNS_14MCInstFragmentE", "llvm::MCAssembler::RelaxInstruction(llvm::MCAsmLayout&, llvm::MCInstFragment&)"},
    {"_ZN4llvm11MCAssembler8RelaxLEBERNS_11MCAsmLayoutERNS_13MCLEBFragmentE", "llvm::MCAssembler::RelaxLEB(llvm::MCAsmLayout&, llvm::MCLEBFragment&)"},
    {"_ZN4llvm11MCAssembler18RelaxDwarfLineAddrERNS_11MCAsmLayoutERNS_23MCDwarfLineAddrFragmentE", "llvm::MCAssembler::RelaxDwarfLineAddr(llvm::MCAsmLayout&, llvm::MCDwarfLineAddrFragment&)"},
    {"_ZN4llvm11MCAssembler27RelaxDwarfCallFrameFragmentERNS_11MCAsmLayoutERNS_24MCDwarfCallFrameFragmentE", "llvm::MCAssembler::RelaxDwarfCallFrameFragment(llvm::MCAsmLayout&, llvm::MCDwarfCallFrameFragment&)"},
    {"_ZN4llvm11MCAssembler17LayoutSectionOnceERNS_11MCAsmLayoutERNS_13MCSectionDataE", "llvm::MCAssembler::LayoutSectionOnce(llvm::MCAsmLayout&, llvm::MCSectionData&)"},
    {"_ZN4llvmlsERNS_11raw_ostreamERKNS_7MCFixupE", "llvm::operator<<(llvm::raw_ostream&, llvm::MCFixup const&)"},
    {"_ZN4llvm10MCFragment4dumpEv", "llvm::MCFragment::dump()"},
    {"_ZN4llvm13MCSectionData4dumpEv", "llvm::MCSectionData::dump()"},
    {"_ZN4llvm12MCSymbolData4dumpEv", "llvm::MCSymbolData::dump()"},
    {"_ZN4llvm11MCAssembler4dumpEv", "llvm::MCAssembler::dump()"},
    {"_ZN4llvm6iplistINS_12MCSymbolDataENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::MCSymbolData, llvm::ilist_traits<llvm::MCSymbolData> >::~iplist()"},
    {"_ZN4llvm6iplistINS_13MCSectionDataENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::MCSectionData, llvm::ilist_traits<llvm::MCSectionData> >::~iplist()"},
    {"_ZN4llvm6iplistINS_10MCFragmentENS_12ilist_traitsIS1_EEED2Ev", "llvm::iplist<llvm::MCFragment, llvm::ilist_traits<llvm::MCFragment> >::~iplist()"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEPNS_10MCFragmentENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MCSectionData const*, llvm::MCFragment*, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<llvm::MCFragment*> >::InsertIntoBucket(llvm::MCSectionData const* const&, llvm::MCFragment* const&, std::pair<llvm::MCSectionData const*, llvm::MCFragment*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEPNS_10MCFragmentENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MCSectionData const*, llvm::MCFragment*, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<llvm::MCFragment*> >::grow(unsigned int)"},
    {"_ZN4llvm14MCObjectWriter10WriteZerosEj", "llvm::MCObjectWriter::WriteZeros(unsigned int)"},
    {"_ZN4llvm13MCCodeEmitterC1Ev", "llvm::MCCodeEmitter::MCCodeEmitter()"},
    {"_ZN4llvm13MCCodeEmitterC2Ev", "llvm::MCCodeEmitter::MCCodeEmitter()"},
    {"_ZN4llvm13MCCodeEmitterD0Ev", "llvm::MCCodeEmitter::~MCCodeEmitter()"},
    {"_ZN4llvm13MCCodeEmitterD1Ev", "llvm::MCCodeEmitter::~MCCodeEmitter()"},
    {"_ZN4llvm13MCCodeEmitterD2Ev", "llvm::MCCodeEmitter::~MCCodeEmitter()"},
    {"_ZN4llvm9MCContextC1ERKNS_9MCAsmInfoEPKNS_13TargetAsmInfoE", "llvm::MCContext::MCContext(llvm::MCAsmInfo const&, llvm::TargetAsmInfo const*)"},
    {"_ZN4llvm9MCContextC2ERKNS_9MCAsmInfoEPKNS_13TargetAsmInfoE", "llvm::MCContext::MCContext(llvm::MCAsmInfo const&, llvm::TargetAsmInfo const*)"},
    {"_ZN4llvm9MCContextD1Ev", "llvm::MCContext::~MCContext()"},
    {"_ZN4llvm9MCContextD2Ev", "llvm::MCContext::~MCContext()"},
    {"_ZN4llvm9MCContext17GetOrCreateSymbolENS_9StringRefE", "llvm::MCContext::GetOrCreateSymbol(llvm::StringRef)"},
    {"_ZN4llvm9MCContext12CreateSymbolENS_9StringRefE", "llvm::MCContext::CreateSymbol(llvm::StringRef)"},
    {"_ZN4llvm9MCContext17GetOrCreateSymbolERKNS_5TwineE", "llvm::MCContext::GetOrCreateSymbol(llvm::Twine const&)"},
    {"_ZN4llvm9MCContext16CreateTempSymbolEv", "llvm::MCContext::CreateTempSymbol()"},
    {"_ZN4llvm9MCContext12NextInstanceEx", "llvm::MCContext::NextInstance(long long)"},
    {"_ZN4llvm9MCContext11GetInstanceEx", "llvm::MCContext::GetInstance(long long)"},
    {"_ZN4llvm9MCContext28CreateDirectionalLocalSymbolEx", "llvm::MCContext::CreateDirectionalLocalSymbol(long long)"},
    {"_ZN4llvm9MCContext25GetDirectionalLocalSymbolExi", "llvm::MCContext::GetDirectionalLocalSymbol(long long, int)"},
    {"_ZNK4llvm9MCContext12LookupSymbolENS_9StringRefE", "llvm::MCContext::LookupSymbol(llvm::StringRef) const"},
    {"_ZN4llvm9MCContext15getMachOSectionENS_9StringRefES1_jjNS_11SectionKindE", "llvm::MCContext::getMachOSection(llvm::StringRef, llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind)"},
    {"_ZN4llvm9MCContext13getELFSectionENS_9StringRefEjjNS_11SectionKindE", "llvm::MCContext::getELFSection(llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind)"},
    {"_ZN4llvm9MCContext13getELFSectionENS_9StringRefEjjNS_11SectionKindEjS1_", "llvm::MCContext::getELFSection(llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind, unsigned int, llvm::StringRef)"},
    {"_ZN4llvm9MCContext21CreateELFGroupSectionEv", "llvm::MCContext::CreateELFGroupSection()"},
    {"_ZN4llvm9MCContext14getCOFFSectionENS_9StringRefEjiNS_11SectionKindE", "llvm::MCContext::getCOFFSection(llvm::StringRef, unsigned int, int, llvm::SectionKind)"},
    {"_ZN4llvm9MCContext12GetDwarfFileENS_9StringRefEj", "llvm::MCContext::GetDwarfFile(llvm::StringRef, unsigned int)"},
    {"_ZN4llvm9MCContext22isValidDwarfFileNumberEj", "llvm::MCContext::isValidDwarfFileNumber(unsigned int)"},
    {"_ZNSt6vectorIN4llvm9StringRefESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::StringRef, std::allocator<llvm::StringRef> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::StringRef*, std::vector<llvm::StringRef, std::allocator<llvm::StringRef> > >, llvm::StringRef const&)"},
    {"_ZNSt6vectorIPN4llvm11MCDwarfFileESaIS2_EE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPS2_S4_EEmRKS2_", "std::vector<llvm::MCDwarfFile*, std::allocator<llvm::MCDwarfFile*> >::_M_fill_insert(__gnu_cxx::__normal_iterator<llvm::MCDwarfFile**, std::vector<llvm::MCDwarfFile*, std::allocator<llvm::MCDwarfFile*> > >, unsigned long, llvm::MCDwarfFile* const&)"},
    {"_ZN4llvm9StringMapIPKNS_13MCSectionCOFFENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::MCSectionCOFF const*>& llvm::StringMap<llvm::MCSectionCOFF const*, llvm::MallocAllocator>::GetOrCreateValue<llvm::MCSectionCOFF const*>(llvm::StringRef, llvm::MCSectionCOFF const*)"},
    {"_ZN4llvm9StringMapIPKNS_12MCSectionELFENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::MCSectionELF const*>& llvm::StringMap<llvm::MCSectionELF const*, llvm::MallocAllocator>::GetOrCreateValue<llvm::MCSectionELF const*>(llvm::StringRef, llvm::MCSectionELF const*)"},
    {"_ZN4llvm9StringMapIPKNS_14MCSectionMachOENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::MCSectionMachO const*>& llvm::StringMap<llvm::MCSectionMachO const*, llvm::MallocAllocator>::GetOrCreateValue<llvm::MCSectionMachO const*>(llvm::StringRef, llvm::MCSectionMachO const*)"},
    {"_ZN4llvm8DenseMapIjPNS_7MCLabelENS_12DenseMapInfoIjEENS3_IS2_EEE16InsertIntoBucketERKjRKS2_PSt4pairIjS2_E", "llvm::DenseMap<unsigned int, llvm::MCLabel*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MCLabel*> >::InsertIntoBucket(unsigned int const&, llvm::MCLabel* const&, std::pair<unsigned int, llvm::MCLabel*>*)"},
    {"_ZN4llvm8DenseMapIjPNS_7MCLabelENS_12DenseMapInfoIjEENS3_IS2_EEE4growEj", "llvm::DenseMap<unsigned int, llvm::MCLabel*, llvm::DenseMapInfo<unsigned int>, llvm::DenseMapInfo<llvm::MCLabel*> >::grow(unsigned int)"},
    {"_ZN4llvm9StringMapIbNS_15MallocAllocatorEE16GetOrCreateValueIbEERNS_14StringMapEntryIbEENS_9StringRefET_", "llvm::StringMapEntry<bool>& llvm::StringMap<bool, llvm::MallocAllocator>::GetOrCreateValue<bool>(llvm::StringRef, bool)"},
    {"_ZN4llvm9StringMapIPNS_8MCSymbolENS_15MallocAllocatorEE16GetOrCreateValueIS2_EERNS_14StringMapEntryIS2_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::MCSymbol*>& llvm::StringMap<llvm::MCSymbol*, llvm::MallocAllocator>::GetOrCreateValue<llvm::MCSymbol*>(llvm::StringRef, llvm::MCSymbol*)"},
    {"_ZNK4llvm9MCSection22isBaseAddressKnownZeroEv", "llvm::MCSection::isBaseAddressKnownZero() const"},
    {"_ZNK4llvm12MCSectionELF22isBaseAddressKnownZeroEv", "llvm::MCSectionELF::isBaseAddressKnownZero() const"},
    {"_ZN4llvm11MCLineEntry4MakeEPNS_10MCStreamerEPKNS_9MCSectionE", "llvm::MCLineEntry::Make(llvm::MCStreamer*, llvm::MCSection const*)"},
    {"_ZN4llvm16MCDwarfFileTable4EmitEPNS_10MCStreamerE", "llvm::MCDwarfFileTable::Emit(llvm::MCStreamer*)"},
    {"_ZN4llvm15MCDwarfLineAddr4EmitEPNS_10MCStreamerExy", "llvm::MCDwarfLineAddr::Emit(llvm::MCStreamer*, long long, unsigned long long)"},
    {"_ZN4llvm15MCDwarfLineAddr5WriteEPNS_14MCObjectWriterExy", "llvm::MCDwarfLineAddr::Write(llvm::MCObjectWriter*, long long, unsigned long long)"},
    {"_ZN4llvm15MCDwarfLineAddr6EncodeExyRNS_11raw_ostreamE", "llvm::MCDwarfLineAddr::Encode(long long, unsigned long long, llvm::raw_ostream&)"},
    {"_ZNK4llvm11MCDwarfFile5printERNS_11raw_ostreamE", "llvm::MCDwarfFile::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm11MCDwarfFile4dumpEv", "llvm::MCDwarfFile::dump() const"},
    {"_ZN4llvm19MCDwarfFrameEmitter4EmitERNS_10MCStreamerE", "llvm::MCDwarfFrameEmitter::Emit(llvm::MCStreamer&)"},
    {"_ZN4llvm19MCDwarfFrameEmitter14EmitAdvanceLocERNS_10MCStreamerEy", "llvm::MCDwarfFrameEmitter::EmitAdvanceLoc(llvm::MCStreamer&, unsigned long long)"},
    {"_ZN4llvm19MCDwarfFrameEmitter16EncodeAdvanceLocEyRNS_11raw_ostreamE", "llvm::MCDwarfFrameEmitter::EncodeAdvanceLoc(unsigned long long, llvm::raw_ostream&)"},
    {"_ZNK4llvm8DenseMapIN12_GLOBAL__N_16CIEKeyEPKNS_8MCSymbolENS_12DenseMapInfoIS2_EENS6_IS5_EEE15LookupBucketForERKS2_RPSt4pairIS2_S5_E", "llvm::DenseMap<(anonymous namespace)::CIEKey, llvm::MCSymbol const*, llvm::DenseMapInfo<(anonymous namespace)::CIEKey>, llvm::DenseMapInfo<llvm::MCSymbol const*> >::LookupBucketFor((anonymous namespace)::CIEKey const&, std::pair<(anonymous namespace)::CIEKey, llvm::MCSymbol const*>*&) const"},
    {"_Z19EmitCFIInstructionsRN4llvm10MCStreamerERKSt6vectorINS_16MCCFIInstructionESaIS3_EEPNS_8MCSymbolE", "EmitCFIInstructions(llvm::MCStreamer&, std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> > const&, llvm::MCSymbol*)"},
    {"_ZN4llvm9MCContext16addMCLineSectionEPKNS_9MCSectionEPNS_13MCLineSectionE", "llvm::MCContext::addMCLineSection(llvm::MCSection const*, llvm::MCLineSection*)"},
    {"_ZNSt6vectorIN4llvm11MachineMoveESaIS1_EEC2ERKS3_", "std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> >::vector(std::vector<llvm::MachineMove, std::allocator<llvm::MachineMove> > const&)"},
    {"_ZNSt6vectorIN4llvm11MCLineEntryESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MCLineEntry, std::allocator<llvm::MCLineEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCLineEntry*, std::vector<llvm::MCLineEntry, std::allocator<llvm::MCLineEntry> > >, llvm::MCLineEntry const&)"},
    {"_ZNSt6vectorIPKN4llvm9MCSectionESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::MCSection const*, std::allocator<llvm::MCSection const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCSection const**, std::vector<llvm::MCSection const*, std::allocator<llvm::MCSection const*> > >, llvm::MCSection const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEPNS_13MCLineSectionENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MCSection const*, llvm::MCLineSection*, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<llvm::MCLineSection*> >::InsertIntoBucket(llvm::MCSection const* const&, llvm::MCLineSection* const&, std::pair<llvm::MCSection const*, llvm::MCLineSection*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEPNS_13MCLineSectionENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MCSection const*, llvm::MCLineSection*, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<llvm::MCLineSection*> >::grow(unsigned int)"},
    {"_ZN4llvm23MCELFObjectTargetWriterC1EbNS_6Triple6OSTypeEtb", "llvm::MCELFObjectTargetWriter::MCELFObjectTargetWriter(bool, llvm::Triple::OSType, unsigned short, bool)"},
    {"_ZN4llvm23MCELFObjectTargetWriterC2EbNS_6Triple6OSTypeEtb", "llvm::MCELFObjectTargetWriter::MCELFObjectTargetWriter(bool, llvm::Triple::OSType, unsigned short, bool)"},
    {"_ZN4llvm23MCELFObjectTargetWriterD0Ev", "llvm::MCELFObjectTargetWriter::~MCELFObjectTargetWriter()"},
    {"_ZN4llvm23MCELFObjectTargetWriterD1Ev", "llvm::MCELFObjectTargetWriter::~MCELFObjectTargetWriter()"},
    {"_ZN4llvm23MCELFObjectTargetWriterD2Ev", "llvm::MCELFObjectTargetWriter::~MCELFObjectTargetWriter()"},
    {"_ZN4llvm17createELFStreamerERNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterEbb", "llvm::createELFStreamer(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*, bool, bool)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamerD1Ev", "(anonymous namespace)::MCELFStreamer::~MCELFStreamer()"},
    {"_ZN12_GLOBAL__N_113MCELFStreamerD0Ev", "(anonymous namespace)::MCELFStreamer::~MCELFStreamer()"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer13SwitchSectionEPKN4llvm9MCSectionE", "(anonymous namespace)::MCELFStreamer::SwitchSection(llvm::MCSection const*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer12InitSectionsEv", "(anonymous namespace)::MCELFStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::MCELFStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::MCELFStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::MCELFStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCELFStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer17EmitWeakReferenceEPN4llvm8MCSymbolEPKS2_", "(anonymous namespace)::MCELFStreamer::EmitWeakReference(llvm::MCSymbol*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::MCELFStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::MCELFStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::MCELFStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::MCELFStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::MCELFStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::MCELFStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCELFStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::MCELFStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::MCELFStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::MCELFStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::MCELFStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::MCELFStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::MCELFStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::MCELFStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::MCELFStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer6FinishEv", "(anonymous namespace)::MCELFStreamer::Finish()"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer14EmitInstToDataERKN4llvm6MCInstE", "(anonymous namespace)::MCELFStreamer::EmitInstToData(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer18EmitInstToFragmentERKN4llvm6MCInstE", "(anonymous namespace)::MCELFStreamer::EmitInstToFragment(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_113MCELFStreamer21fixSymbolsInTLSFixupsEPKN4llvm6MCExprE", "(anonymous namespace)::MCELFStreamer::fixSymbolsInTLSFixups(llvm::MCExpr const*)"},
    {"_ZNK4llvm6MCExpr5printERNS_11raw_ostreamE", "llvm::MCExpr::print(llvm::raw_ostream&) const"},
    {"_ZN4llvm15MCSymbolRefExpr18getVariantKindNameENS0_11VariantKindE", "llvm::MCSymbolRefExpr::getVariantKindName(llvm::MCSymbolRefExpr::VariantKind)"},
    {"_ZNK4llvm6MCExpr4dumpEv", "llvm::MCExpr::dump() const"},
    {"_ZN4llvm12MCBinaryExpr6CreateENS0_6OpcodeEPKNS_6MCExprES4_RNS_9MCContextE", "llvm::MCBinaryExpr::Create(llvm::MCBinaryExpr::Opcode, llvm::MCExpr const*, llvm::MCExpr const*, llvm::MCContext&)"},
    {"_ZN4llvm11MCUnaryExpr6CreateENS0_6OpcodeEPKNS_6MCExprERNS_9MCContextE", "llvm::MCUnaryExpr::Create(llvm::MCUnaryExpr::Opcode, llvm::MCExpr const*, llvm::MCContext&)"},
    {"_ZN4llvm14MCConstantExpr6CreateExRNS_9MCContextE", "llvm::MCConstantExpr::Create(long long, llvm::MCContext&)"},
    {"_ZN4llvm15MCSymbolRefExpr6CreateEPKNS_8MCSymbolENS0_11VariantKindERNS_9MCContextE", "llvm::MCSymbolRefExpr::Create(llvm::MCSymbol const*, llvm::MCSymbolRefExpr::VariantKind, llvm::MCContext&)"},
    {"_ZN4llvm15MCSymbolRefExpr6CreateENS_9StringRefENS0_11VariantKindERNS_9MCContextE", "llvm::MCSymbolRefExpr::Create(llvm::StringRef, llvm::MCSymbolRefExpr::VariantKind, llvm::MCContext&)"},
    {"_ZN4llvm15MCSymbolRefExpr21getVariantKindForNameENS_9StringRefE", "llvm::MCSymbolRefExpr::getVariantKindForName(llvm::StringRef)"},
    {"_ZN4llvm12MCTargetExpr6AnchorEv", "llvm::MCTargetExpr::Anchor()"},
    {"_ZNK4llvm6MCExpr18EvaluateAsAbsoluteERx", "llvm::MCExpr::EvaluateAsAbsolute(long long&) const"},
    {"_ZNK4llvm6MCExpr18EvaluateAsAbsoluteERxPKNS_11MCAssemblerEPKNS_11MCAsmLayoutEPKNS_8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoISB_EENSC_IyEEEE", "llvm::MCExpr::EvaluateAsAbsolute(long long&, llvm::MCAssembler const*, llvm::MCAsmLayout const*, llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> > const*) const"},
    {"_ZNK4llvm6MCExpr18EvaluateAsAbsoluteERxRKNS_11MCAsmLayoutE", "llvm::MCExpr::EvaluateAsAbsolute(long long&, llvm::MCAsmLayout const&) const"},
    {"_ZNK4llvm6MCExpr18EvaluateAsAbsoluteERxRKNS_11MCAsmLayoutERKNS_8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoIS8_EENS9_IyEEEE", "llvm::MCExpr::EvaluateAsAbsolute(long long&, llvm::MCAsmLayout const&, llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> > const&) const"},
    {"_ZNK4llvm6MCExpr18EvaluateAsAbsoluteERxRKNS_11MCAssemblerE", "llvm::MCExpr::EvaluateAsAbsolute(long long&, llvm::MCAssembler const&) const"},
    {"_ZNK4llvm6MCExpr25EvaluateAsRelocatableImplERNS_7MCValueEPKNS_11MCAssemblerEPKNS_11MCAsmLayoutEPKNS_8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoISC_EENSD_IyEEEEb", "llvm::MCExpr::EvaluateAsRelocatableImpl(llvm::MCValue&, llvm::MCAssembler const*, llvm::MCAsmLayout const*, llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> > const*, bool) const"},
    {"_ZNK4llvm6MCExpr21EvaluateAsRelocatableERNS_7MCValueERKNS_11MCAsmLayoutE", "llvm::MCExpr::EvaluateAsRelocatable(llvm::MCValue&, llvm::MCAsmLayout const&) const"},
    {"_Z19EvaluateSymbolicAddPKN4llvm11MCAssemblerEPKNS_11MCAsmLayoutEPKNS_8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoIS9_EENSA_IyEEEEbRKNS_7MCValueEPKNS_15MCSymbolRefExprESL_xRSG_", "EvaluateSymbolicAdd(llvm::MCAssembler const*, llvm::MCAsmLayout const*, llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> > const*, bool, llvm::MCValue const&, llvm::MCSymbolRefExpr const*, llvm::MCSymbolRefExpr const*, long long, llvm::MCValue&)"},
    {"_Z35AttemptToFoldSymbolOffsetDifferencePKN4llvm11MCAssemblerEPKNS_11MCAsmLayoutEPKNS_8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoIS9_EENSA_IyEEEEbRPKNS_15MCSymbolRefExprESJ_Rx", "AttemptToFoldSymbolOffsetDifference(llvm::MCAssembler const*, llvm::MCAsmLayout const*, llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> > const*, bool, llvm::MCSymbolRefExpr const*&, llvm::MCSymbolRefExpr const*&, long long&)"},
    {"_ZN4llvm12MCTargetExprD1Ev", "llvm::MCTargetExpr::~MCTargetExpr()"},
    {"_ZN4llvm12MCTargetExprD0Ev", "llvm::MCTargetExpr::~MCTargetExpr()"},
    {"_ZNK4llvm9MCOperand5printERNS_11raw_ostreamEPKNS_9MCAsmInfoE", "llvm::MCOperand::print(llvm::raw_ostream&, llvm::MCAsmInfo const*) const"},
    {"_ZNK4llvm9MCOperand4dumpEv", "llvm::MCOperand::dump() const"},
    {"_ZNK4llvm6MCInst5printERNS_11raw_ostreamEPKNS_9MCAsmInfoE", "llvm::MCInst::print(llvm::raw_ostream&, llvm::MCAsmInfo const*) const"},
    {"_ZNK4llvm6MCInst11dump_prettyERNS_11raw_ostreamEPKNS_9MCAsmInfoEPKNS_13MCInstPrinterENS_9StringRefE", "llvm::MCInst::dump_pretty(llvm::raw_ostream&, llvm::MCAsmInfo const*, llvm::MCInstPrinter const*, llvm::StringRef) const"},
    {"_ZNK4llvm6MCInst4dumpEv", "llvm::MCInst::dump() const"},
    {"_ZN4llvm13MCInstPrinterD0Ev", "llvm::MCInstPrinter::~MCInstPrinter()"},
    {"_ZN4llvm13MCInstPrinterD1Ev", "llvm::MCInstPrinter::~MCInstPrinter()"},
    {"_ZN4llvm13MCInstPrinterD2Ev", "llvm::MCInstPrinter::~MCInstPrinter()"},
    {"_ZNK4llvm13MCInstPrinter13getOpcodeNameEj", "llvm::MCInstPrinter::getOpcodeName(unsigned int) const"},
    {"_ZN4llvm21createLoggingStreamerEPNS_10MCStreamerERNS_11raw_ostreamE", "llvm::createLoggingStreamer(llvm::MCStreamer*, llvm::raw_ostream&)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamerD1Ev", "(anonymous namespace)::MCLoggingStreamer::~MCLoggingStreamer()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamerD0Ev", "(anonymous namespace)::MCLoggingStreamer::~MCLoggingStreamer()"},
    {"_ZNK12_GLOBAL__N_117MCLoggingStreamer12isVerboseAsmEv", "(anonymous namespace)::MCLoggingStreamer::isVerboseAsm() const"},
    {"_ZNK12_GLOBAL__N_117MCLoggingStreamer17hasRawTextSupportEv", "(anonymous namespace)::MCLoggingStreamer::hasRawTextSupport() const"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer10AddCommentERKN4llvm5TwineE", "(anonymous namespace)::MCLoggingStreamer::AddComment(llvm::Twine const&)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer12GetCommentOSEv", "(anonymous namespace)::MCLoggingStreamer::GetCommentOS()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer12AddBlankLineEv", "(anonymous namespace)::MCLoggingStreamer::AddBlankLine()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer13SwitchSectionEPKN4llvm9MCSectionE", "(anonymous namespace)::MCLoggingStreamer::SwitchSection(llvm::MCSection const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer12InitSectionsEv", "(anonymous namespace)::MCLoggingStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::MCLoggingStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::MCLoggingStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::MCLoggingStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCLoggingStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer17EmitWeakReferenceEPN4llvm8MCSymbolEPKS2_", "(anonymous namespace)::MCLoggingStreamer::EmitWeakReference(llvm::MCSymbol*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::MCLoggingStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::MCLoggingStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::MCLoggingStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::MCLoggingStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::MCLoggingStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::MCLoggingStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCLoggingStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::MCLoggingStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::MCLoggingStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::MCLoggingStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::MCLoggingStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::MCLoggingStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer13EmitValueImplEPKN4llvm6MCExprEjbj", "(anonymous namespace)::MCLoggingStreamer::EmitValueImpl(llvm::MCExpr const*, unsigned int, bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer16EmitULEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCLoggingStreamer::EmitULEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer16EmitSLEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCLoggingStreamer::EmitSLEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer16EmitGPRel32ValueEPKN4llvm6MCExprE", "(anonymous namespace)::MCLoggingStreamer::EmitGPRel32Value(llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer8EmitFillEyhj", "(anonymous namespace)::MCLoggingStreamer::EmitFill(unsigned long long, unsigned char, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::MCLoggingStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::MCLoggingStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer17EmitValueToOffsetEPKN4llvm6MCExprEh", "(anonymous namespace)::MCLoggingStreamer::EmitValueToOffset(llvm::MCExpr const*, unsigned char)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::MCLoggingStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer22EmitDwarfFileDirectiveEjN4llvm9StringRefE", "(anonymous namespace)::MCLoggingStreamer::EmitDwarfFileDirective(unsigned int, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer21EmitDwarfLocDirectiveEjjjjjj", "(anonymous namespace)::MCLoggingStreamer::EmitDwarfLocDirective(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer24EmitDwarfAdvanceLineAddrExPKN4llvm8MCSymbolES4_", "(anonymous namespace)::MCLoggingStreamer::EmitDwarfAdvanceLineAddr(long long, llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer15EmitInstructionERKN4llvm6MCInstE", "(anonymous namespace)::MCLoggingStreamer::EmitInstruction(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer11EmitRawTextEN4llvm9StringRefE", "(anonymous namespace)::MCLoggingStreamer::EmitRawText(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer6FinishEv", "(anonymous namespace)::MCLoggingStreamer::Finish()"},
    {"_ZN12_GLOBAL__N_117MCLoggingStreamer7LogCallEPKcRKN4llvm5TwineE", "(anonymous namespace)::MCLoggingStreamer::LogCall(char const*, llvm::Twine const&)"},
    {"_ZN4llvm19createMachOStreamerERNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterEb", "llvm::createMachOStreamer(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*, bool)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamerD1Ev", "(anonymous namespace)::MCMachOStreamer::~MCMachOStreamer()"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamerD0Ev", "(anonymous namespace)::MCMachOStreamer::~MCMachOStreamer()"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer12InitSectionsEv", "(anonymous namespace)::MCMachOStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::MCMachOStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::MCMachOStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::MCMachOStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCMachOStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::MCMachOStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::MCMachOStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::MCMachOStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::MCMachOStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::MCMachOStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::MCMachOStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCMachOStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::MCMachOStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::MCMachOStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::MCMachOStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::MCMachOStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::MCMachOStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::MCMachOStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::MCMachOStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::MCMachOStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer6FinishEv", "(anonymous namespace)::MCMachOStreamer::Finish()"},
    {"_ZN12_GLOBAL__N_115MCMachOStreamer14EmitInstToDataERKN4llvm6MCInstE", "(anonymous namespace)::MCMachOStreamer::EmitInstToData(llvm::MCInst const&)"},
    {"_ZNSt6vectorIN4llvm7MCFixupESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MCFixup, std::allocator<llvm::MCFixup> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCFixup*, std::vector<llvm::MCFixup, std::allocator<llvm::MCFixup> > >, llvm::MCFixup const&)"},
    {"_ZN4llvm8DenseMapIPKNS_10MCFragmentEPNS_12MCSymbolDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MCFragment const*, llvm::MCSymbolData*, llvm::DenseMapInfo<llvm::MCFragment const*>, llvm::DenseMapInfo<llvm::MCSymbolData*> >::InsertIntoBucket(llvm::MCFragment const* const&, llvm::MCSymbolData* const&, std::pair<llvm::MCFragment const*, llvm::MCSymbolData*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_10MCFragmentEPNS_12MCSymbolDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MCFragment const*, llvm::MCSymbolData*, llvm::DenseMapInfo<llvm::MCFragment const*>, llvm::DenseMapInfo<llvm::MCSymbolData*> >::grow(unsigned int)"},
    {"_ZN4llvm15MCAlignFragmentD1Ev", "llvm::MCAlignFragment::~MCAlignFragment()"},
    {"_ZN4llvm15MCAlignFragmentD0Ev", "llvm::MCAlignFragment::~MCAlignFragment()"},
    {"_ZN4llvm11MCAssembler22getOrCreateSectionDataERKNS_9MCSectionEPb", "llvm::MCAssembler::getOrCreateSectionData(llvm::MCSection const&, bool*)"},
    {"_ZN4llvm11MCAssembler21getOrCreateSymbolDataERKNS_8MCSymbolEPb", "llvm::MCAssembler::getOrCreateSymbolData(llvm::MCSymbol const&, bool*)"},
    {"_ZN4llvm14MCFillFragmentD1Ev", "llvm::MCFillFragment::~MCFillFragment()"},
    {"_ZN4llvm14MCFillFragmentD0Ev", "llvm::MCFillFragment::~MCFillFragment()"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPNS_12MCSymbolDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSymbolData*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSymbolData*> >::InsertIntoBucket(llvm::MCSymbol const* const&, llvm::MCSymbolData* const&, std::pair<llvm::MCSymbol const*, llvm::MCSymbolData*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPNS_12MCSymbolDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MCSymbol const*, llvm::MCSymbolData*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<llvm::MCSymbolData*> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEPNS_13MCSectionDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE16InsertIntoBucketERKS3_RKS5_PSt4pairIS3_S5_E", "llvm::DenseMap<llvm::MCSection const*, llvm::MCSectionData*, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<llvm::MCSectionData*> >::InsertIntoBucket(llvm::MCSection const* const&, llvm::MCSectionData* const&, std::pair<llvm::MCSection const*, llvm::MCSectionData*>*)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEPNS_13MCSectionDataENS_12DenseMapInfoIS3_EENS6_IS5_EEE4growEj", "llvm::DenseMap<llvm::MCSection const*, llvm::MCSectionData*, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<llvm::MCSectionData*> >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm18IndirectSymbolDataESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::IndirectSymbolData, std::allocator<llvm::IndirectSymbolData> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::IndirectSymbolData*, std::vector<llvm::IndirectSymbolData, std::allocator<llvm::IndirectSymbolData> > >, llvm::IndirectSymbolData const&)"},
    {"_ZN4llvm14MCDataFragmentD1Ev", "llvm::MCDataFragment::~MCDataFragment()"},
    {"_ZN4llvm14MCDataFragmentD0Ev", "llvm::MCDataFragment::~MCDataFragment()"},
    {"_ZN4llvm24MCMachObjectTargetWriterC1Ebjjb", "llvm::MCMachObjectTargetWriter::MCMachObjectTargetWriter(bool, unsigned int, unsigned int, bool)"},
    {"_ZN4llvm24MCMachObjectTargetWriterC2Ebjjb", "llvm::MCMachObjectTargetWriter::MCMachObjectTargetWriter(bool, unsigned int, unsigned int, bool)"},
    {"_ZN4llvm24MCMachObjectTargetWriterD0Ev", "llvm::MCMachObjectTargetWriter::~MCMachObjectTargetWriter()"},
    {"_ZN4llvm24MCMachObjectTargetWriterD1Ev", "llvm::MCMachObjectTargetWriter::~MCMachObjectTargetWriter()"},
    {"_ZN4llvm24MCMachObjectTargetWriterD2Ev", "llvm::MCMachObjectTargetWriter::~MCMachObjectTargetWriter()"},
    {"_ZN4llvm18createNullStreamerERNS_9MCContextE", "llvm::createNullStreamer(llvm::MCContext&)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamerD1Ev", "(anonymous namespace)::MCNullStreamer::~MCNullStreamer()"},
    {"_ZN12_GLOBAL__N_114MCNullStreamerD0Ev", "(anonymous namespace)::MCNullStreamer::~MCNullStreamer()"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer13SwitchSectionEPKN4llvm9MCSectionE", "(anonymous namespace)::MCNullStreamer::SwitchSection(llvm::MCSection const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer12InitSectionsEv", "(anonymous namespace)::MCNullStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::MCNullStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::MCNullStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::MCNullStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCNullStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer17EmitWeakReferenceEPN4llvm8MCSymbolEPKS2_", "(anonymous namespace)::MCNullStreamer::EmitWeakReference(llvm::MCSymbol*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::MCNullStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::MCNullStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::MCNullStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::MCNullStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::MCNullStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::MCNullStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::MCNullStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::MCNullStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::MCNullStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::MCNullStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::MCNullStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::MCNullStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer13EmitValueImplEPKN4llvm6MCExprEjbj", "(anonymous namespace)::MCNullStreamer::EmitValueImpl(llvm::MCExpr const*, unsigned int, bool, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer16EmitULEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCNullStreamer::EmitULEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer16EmitSLEB128ValueEPKN4llvm6MCExprEj", "(anonymous namespace)::MCNullStreamer::EmitSLEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer16EmitGPRel32ValueEPKN4llvm6MCExprE", "(anonymous namespace)::MCNullStreamer::EmitGPRel32Value(llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::MCNullStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::MCNullStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer17EmitValueToOffsetEPKN4llvm6MCExprEh", "(anonymous namespace)::MCNullStreamer::EmitValueToOffset(llvm::MCExpr const*, unsigned char)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::MCNullStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer22EmitDwarfFileDirectiveEjN4llvm9StringRefE", "(anonymous namespace)::MCNullStreamer::EmitDwarfFileDirective(unsigned int, llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer21EmitDwarfLocDirectiveEjjjjjj", "(anonymous namespace)::MCNullStreamer::EmitDwarfLocDirective(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer24EmitDwarfAdvanceLineAddrExPKN4llvm8MCSymbolES4_", "(anonymous namespace)::MCNullStreamer::EmitDwarfAdvanceLineAddr(long long, llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer15EmitInstructionERKN4llvm6MCInstE", "(anonymous namespace)::MCNullStreamer::EmitInstruction(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_114MCNullStreamer6FinishEv", "(anonymous namespace)::MCNullStreamer::Finish()"},
    {"_ZNK4llvm10MCStreamer12isVerboseAsmEv", "llvm::MCStreamer::isVerboseAsm() const"},
    {"_ZNK4llvm10MCStreamer17hasRawTextSupportEv", "llvm::MCStreamer::hasRawTextSupport() const"},
    {"_ZN4llvm10MCStreamer10AddCommentERKNS_5TwineE", "llvm::MCStreamer::AddComment(llvm::Twine const&)"},
    {"_ZN4llvm10MCStreamer12AddBlankLineEv", "llvm::MCStreamer::AddBlankLine()"},
    {"_ZN4llvm10MCStreamer25EmitDwarfAdvanceFrameAddrEPKNS_8MCSymbolES3_", "llvm::MCStreamer::EmitDwarfAdvanceFrameAddr(llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN4llvm16MCObjectStreamerC1ERNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterE", "llvm::MCObjectStreamer::MCObjectStreamer(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*)"},
    {"_ZN4llvm16MCObjectStreamerC2ERNS_9MCContextERNS_16TargetAsmBackendERNS_11raw_ostreamEPNS_13MCCodeEmitterE", "llvm::MCObjectStreamer::MCObjectStreamer(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::raw_ostream&, llvm::MCCodeEmitter*)"},
    {"_ZN4llvm16MCObjectStreamerD0Ev", "llvm::MCObjectStreamer::~MCObjectStreamer()"},
    {"_ZN4llvm16MCObjectStreamerD1Ev", "llvm::MCObjectStreamer::~MCObjectStreamer()"},
    {"_ZN4llvm16MCObjectStreamerD2Ev", "llvm::MCObjectStreamer::~MCObjectStreamer()"},
    {"_ZNK4llvm16MCObjectStreamer18getCurrentFragmentEv", "llvm::MCObjectStreamer::getCurrentFragment() const"},
    {"_ZNK4llvm16MCObjectStreamer23getOrCreateDataFragmentEv", "llvm::MCObjectStreamer::getOrCreateDataFragment() const"},
    {"_ZN4llvm16MCObjectStreamer15AddValueSymbolsEPKNS_6MCExprE", "llvm::MCObjectStreamer::AddValueSymbols(llvm::MCExpr const*)"},
    {"_ZN4llvm16MCObjectStreamer13EmitValueImplEPKNS_6MCExprEjbj", "llvm::MCObjectStreamer::EmitValueImpl(llvm::MCExpr const*, unsigned int, bool, unsigned int)"},
    {"_ZN4llvm16MCObjectStreamer9EmitLabelEPNS_8MCSymbolE", "llvm::MCObjectStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN4llvm16MCObjectStreamer16EmitULEB128ValueEPKNS_6MCExprEj", "llvm::MCObjectStreamer::EmitULEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN4llvm16MCObjectStreamer16EmitSLEB128ValueEPKNS_6MCExprEj", "llvm::MCObjectStreamer::EmitSLEB128Value(llvm::MCExpr const*, unsigned int)"},
    {"_ZN4llvm16MCObjectStreamer17EmitWeakReferenceEPNS_8MCSymbolEPKS1_", "llvm::MCObjectStreamer::EmitWeakReference(llvm::MCSymbol*, llvm::MCSymbol const*)"},
    {"_ZN4llvm16MCObjectStreamer13SwitchSectionEPKNS_9MCSectionE", "llvm::MCObjectStreamer::SwitchSection(llvm::MCSection const*)"},
    {"_ZN4llvm16MCObjectStreamer15EmitInstructionERKNS_6MCInstE", "llvm::MCObjectStreamer::EmitInstruction(llvm::MCInst const&)"},
    {"_ZN4llvm16MCObjectStreamer18EmitInstToFragmentERKNS_6MCInstE", "llvm::MCObjectStreamer::EmitInstToFragment(llvm::MCInst const&)"},
    {"_ZN4llvm16MCObjectStreamer24EmitDwarfAdvanceLineAddrExPKNS_8MCSymbolES3_", "llvm::MCObjectStreamer::EmitDwarfAdvanceLineAddr(long long, llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN4llvm16MCObjectStreamer25EmitDwarfAdvanceFrameAddrEPKNS_8MCSymbolES3_", "llvm::MCObjectStreamer::EmitDwarfAdvanceFrameAddr(llvm::MCSymbol const*, llvm::MCSymbol const*)"},
    {"_ZN4llvm16MCObjectStreamer17EmitValueToOffsetEPKNS_6MCExprEh", "llvm::MCObjectStreamer::EmitValueToOffset(llvm::MCExpr const*, unsigned char)"},
    {"_ZN4llvm16MCObjectStreamer6FinishEv", "llvm::MCObjectStreamer::Finish()"},
    {"_ZN4llvm13MCOrgFragmentD1Ev", "llvm::MCOrgFragment::~MCOrgFragment()"},
    {"_ZN4llvm13MCOrgFragmentD0Ev", "llvm::MCOrgFragment::~MCOrgFragment()"},
    {"_ZN4llvm24MCDwarfCallFrameFragmentD1Ev", "llvm::MCDwarfCallFrameFragment::~MCDwarfCallFrameFragment()"},
    {"_ZN4llvm24MCDwarfCallFrameFragmentD0Ev", "llvm::MCDwarfCallFrameFragment::~MCDwarfCallFrameFragment()"},
    {"_ZN4llvm23MCDwarfLineAddrFragmentD1Ev", "llvm::MCDwarfLineAddrFragment::~MCDwarfLineAddrFragment()"},
    {"_ZN4llvm23MCDwarfLineAddrFragmentD0Ev", "llvm::MCDwarfLineAddrFragment::~MCDwarfLineAddrFragment()"},
    {"_ZN4llvm13MCLEBFragmentD1Ev", "llvm::MCLEBFragment::~MCLEBFragment()"},
    {"_ZN4llvm13MCLEBFragmentD0Ev", "llvm::MCLEBFragment::~MCLEBFragment()"},
    {"_ZN4llvm14MCObjectWriterD0Ev", "llvm::MCObjectWriter::~MCObjectWriter()"},
    {"_ZN4llvm14MCObjectWriterD1Ev", "llvm::MCObjectWriter::~MCObjectWriter()"},
    {"_ZN4llvm14MCObjectWriterD2Ev", "llvm::MCObjectWriter::~MCObjectWriter()"},
    {"_ZN4llvm14MCObjectWriter13EncodeSLEB128ExRNS_11raw_ostreamE", "llvm::MCObjectWriter::EncodeSLEB128(long long, llvm::raw_ostream&)"},
    {"_ZN4llvm14MCObjectWriter13EncodeULEB128EyRNS_11raw_ostreamE", "llvm::MCObjectWriter::EncodeULEB128(unsigned long long, llvm::raw_ostream&)"},
    {"_ZNK4llvm14MCObjectWriter34IsSymbolRefDifferenceFullyResolvedERKNS_11MCAssemblerEPKNS_15MCSymbolRefExprES6_b", "llvm::MCObjectWriter::IsSymbolRefDifferenceFullyResolved(llvm::MCAssembler const&, llvm::MCSymbolRefExpr const*, llvm::MCSymbolRefExpr const*, bool) const"},
    {"_ZN4llvm9MCSectionD0Ev", "llvm::MCSection::~MCSection()"},
    {"_ZN4llvm9MCSectionD1Ev", "llvm::MCSection::~MCSection()"},
    {"_ZN4llvm9MCSectionD2Ev", "llvm::MCSection::~MCSection()"},
    {"_ZN4llvm13MCSectionCOFFD0Ev", "llvm::MCSectionCOFF::~MCSectionCOFF()"},
    {"_ZN4llvm13MCSectionCOFFD1Ev", "llvm::MCSectionCOFF::~MCSectionCOFF()"},
    {"_ZN4llvm13MCSectionCOFFD2Ev", "llvm::MCSectionCOFF::~MCSectionCOFF()"},
    {"_ZNK4llvm13MCSectionCOFF26ShouldOmitSectionDirectiveENS_9StringRefERKNS_9MCAsmInfoE", "llvm::MCSectionCOFF::ShouldOmitSectionDirective(llvm::StringRef, llvm::MCAsmInfo const&) const"},
    {"_ZNK4llvm13MCSectionCOFF20PrintSwitchToSectionERKNS_9MCAsmInfoERNS_11raw_ostreamE", "llvm::MCSectionCOFF::PrintSwitchToSection(llvm::MCAsmInfo const&, llvm::raw_ostream&) const"},
    {"_ZNK4llvm13MCSectionCOFF12UseCodeAlignEv", "llvm::MCSectionCOFF::UseCodeAlign() const"},
    {"_ZNK4llvm13MCSectionCOFF16isVirtualSectionEv", "llvm::MCSectionCOFF::isVirtualSection() const"},
    {"_ZN4llvm12MCSectionELFD0Ev", "llvm::MCSectionELF::~MCSectionELF()"},
    {"_ZN4llvm12MCSectionELFD1Ev", "llvm::MCSectionELF::~MCSectionELF()"},
    {"_ZN4llvm12MCSectionELFD2Ev", "llvm::MCSectionELF::~MCSectionELF()"},
    {"_ZNK4llvm12MCSectionELF26ShouldOmitSectionDirectiveENS_9StringRefERKNS_9MCAsmInfoE", "llvm::MCSectionELF::ShouldOmitSectionDirective(llvm::StringRef, llvm::MCAsmInfo const&) const"},
    {"_ZNK4llvm12MCSectionELF20PrintSwitchToSectionERKNS_9MCAsmInfoERNS_11raw_ostreamE", "llvm::MCSectionELF::PrintSwitchToSection(llvm::MCAsmInfo const&, llvm::raw_ostream&) const"},
    {"_ZNK4llvm12MCSectionELF12UseCodeAlignEv", "llvm::MCSectionELF::UseCodeAlign() const"},
    {"_ZNK4llvm12MCSectionELF16isVirtualSectionEv", "llvm::MCSectionELF::isVirtualSection() const"},
    {"_ZNK4llvm12MCSectionELF16HasCommonSymbolsEv", "llvm::MCSectionELF::HasCommonSymbols() const"},
    {"_ZN4llvm12MCSectionELF18DetermineEntrySizeENS_11SectionKindE", "llvm::MCSectionELF::DetermineEntrySize(llvm::SectionKind)"},
    {"_ZN4llvm14MCSectionMachOC1ENS_9StringRefES1_jjNS_11SectionKindE", "llvm::MCSectionMachO::MCSectionMachO(llvm::StringRef, llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind)"},
    {"_ZN4llvm14MCSectionMachOC2ENS_9StringRefES1_jjNS_11SectionKindE", "llvm::MCSectionMachO::MCSectionMachO(llvm::StringRef, llvm::StringRef, unsigned int, unsigned int, llvm::SectionKind)"},
    {"_ZNK4llvm14MCSectionMachO20PrintSwitchToSectionERKNS_9MCAsmInfoERNS_11raw_ostreamE", "llvm::MCSectionMachO::PrintSwitchToSection(llvm::MCAsmInfo const&, llvm::raw_ostream&) const"},
    {"_ZNK4llvm14MCSectionMachO12UseCodeAlignEv", "llvm::MCSectionMachO::UseCodeAlign() const"},
    {"_ZNK4llvm14MCSectionMachO16isVirtualSectionEv", "llvm::MCSectionMachO::isVirtualSection() const"},
    {"_ZN4llvm14MCSectionMachO21ParseSectionSpecifierENS_9StringRefERS1_S2_RjS3_", "llvm::MCSectionMachO::ParseSectionSpecifier(llvm::StringRef, llvm::StringRef&, llvm::StringRef&, unsigned int&, unsigned int&)"},
    {"_Z11StripSpacesRN4llvm9StringRefE", "StripSpaces(llvm::StringRef&)"},
    {"_ZN4llvm14MCSectionMachOD1Ev", "llvm::MCSectionMachO::~MCSectionMachO()"},
    {"_ZN4llvm14MCSectionMachOD0Ev", "llvm::MCSectionMachO::~MCSectionMachO()"},
    {"_ZN4llvm10MCStreamerC1ERNS_9MCContextE", "llvm::MCStreamer::MCStreamer(llvm::MCContext&)"},
    {"_ZN4llvm10MCStreamerC2ERNS_9MCContextE", "llvm::MCStreamer::MCStreamer(llvm::MCContext&)"},
    {"_ZN4llvm10MCStreamerD0Ev", "llvm::MCStreamer::~MCStreamer()"},
    {"_ZN4llvm10MCStreamerD1Ev", "llvm::MCStreamer::~MCStreamer()"},
    {"_ZN4llvm10MCStreamerD2Ev", "llvm::MCStreamer::~MCStreamer()"},
    {"_ZN4llvm10MCStreamer12GetCommentOSEv", "llvm::MCStreamer::GetCommentOS()"},
    {"_ZN4llvm10MCStreamer20EmitDwarfSetLineAddrExPKNS_8MCSymbolEi", "llvm::MCStreamer::EmitDwarfSetLineAddr(long long, llvm::MCSymbol const*, int)"},
    {"_ZN4llvm10MCStreamer19EmitULEB128IntValueEyj", "llvm::MCStreamer::EmitULEB128IntValue(unsigned long long, unsigned int)"},
    {"_ZN4llvm10MCStreamer15EmitSymbolValueEPKNS_8MCSymbolEjj", "llvm::MCStreamer::EmitSymbolValue(llvm::MCSymbol const*, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer12EmitIntValueEyjj", "llvm::MCStreamer::EmitIntValue(unsigned long long, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer19EmitSLEB128IntValueExj", "llvm::MCStreamer::EmitSLEB128IntValue(long long, unsigned int)"},
    {"_ZN4llvm10MCStreamer12EmitAbsValueEPKNS_6MCExprEjj", "llvm::MCStreamer::EmitAbsValue(llvm::MCExpr const*, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer9EmitValueEPKNS_6MCExprEjj", "llvm::MCStreamer::EmitValue(llvm::MCExpr const*, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer14EmitPCRelValueEPKNS_6MCExprEjj", "llvm::MCStreamer::EmitPCRelValue(llvm::MCExpr const*, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer15EmitSymbolValueEPKNS_8MCSymbolEjbj", "llvm::MCStreamer::EmitSymbolValue(llvm::MCSymbol const*, unsigned int, bool, unsigned int)"},
    {"_ZN4llvm10MCStreamer20EmitPCRelSymbolValueEPKNS_8MCSymbolEjj", "llvm::MCStreamer::EmitPCRelSymbolValue(llvm::MCSymbol const*, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer16EmitGPRel32ValueEPKNS_6MCExprE", "llvm::MCStreamer::EmitGPRel32Value(llvm::MCExpr const*)"},
    {"_ZN4llvm10MCStreamer8EmitFillEyhj", "llvm::MCStreamer::EmitFill(unsigned long long, unsigned char, unsigned int)"},
    {"_ZN4llvm10MCStreamer22EmitDwarfFileDirectiveEjNS_9StringRefE", "llvm::MCStreamer::EmitDwarfFileDirective(unsigned int, llvm::StringRef)"},
    {"_ZN4llvm10MCStreamer21EmitDwarfLocDirectiveEjjjjjj", "llvm::MCStreamer::EmitDwarfLocDirective(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int)"},
    {"_ZN4llvm10MCStreamer19getCurrentFrameInfoEv", "llvm::MCStreamer::getCurrentFrameInfo()"},
    {"_ZN4llvm10MCStreamer16EnsureValidFrameEv", "llvm::MCStreamer::EnsureValidFrame()"},
    {"_ZN4llvm10MCStreamer16EmitCFIStartProcEv", "llvm::MCStreamer::EmitCFIStartProc()"},
    {"_ZN4llvm10MCStreamer14EmitCFIEndProcEv", "llvm::MCStreamer::EmitCFIEndProc()"},
    {"_ZN4llvm10MCStreamer13EmitCFIDefCfaExx", "llvm::MCStreamer::EmitCFIDefCfa(long long, long long)"},
    {"_ZN4llvm10MCStreamer19EmitCFIDefCfaOffsetEx", "llvm::MCStreamer::EmitCFIDefCfaOffset(long long)"},
    {"_ZN4llvm10MCStreamer21EmitCFIDefCfaRegisterEx", "llvm::MCStreamer::EmitCFIDefCfaRegister(long long)"},
    {"_ZN4llvm10MCStreamer13EmitCFIOffsetExx", "llvm::MCStreamer::EmitCFIOffset(long long, long long)"},
    {"_ZN4llvm10MCStreamer18EmitCFIPersonalityEPKNS_8MCSymbolEj", "llvm::MCStreamer::EmitCFIPersonality(llvm::MCSymbol const*, unsigned int)"},
    {"_ZN4llvm10MCStreamer11EmitCFILsdaEPKNS_8MCSymbolEj", "llvm::MCStreamer::EmitCFILsda(llvm::MCSymbol const*, unsigned int)"},
    {"_ZN4llvm10MCStreamer20EmitCFIRememberStateEv", "llvm::MCStreamer::EmitCFIRememberState()"},
    {"_ZN4llvm10MCStreamer19EmitCFIRestoreStateEv", "llvm::MCStreamer::EmitCFIRestoreState()"},
    {"_ZN4llvm10MCStreamer11EmitRawTextENS_9StringRefE", "llvm::MCStreamer::EmitRawText(llvm::StringRef)"},
    {"_ZN4llvm10MCStreamer11EmitRawTextERKNS_5TwineE", "llvm::MCStreamer::EmitRawText(llvm::Twine const&)"},
    {"_ZNSt6vectorIN4llvm16MCCFIInstructionESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCCFIInstruction*, std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> > >, llvm::MCCFIInstruction const&)"},
    {"_ZNSt6vectorIN4llvm16MCDwarfFrameInfoESaIS1_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS1_S3_EERKS1_", "std::vector<llvm::MCDwarfFrameInfo, std::allocator<llvm::MCDwarfFrameInfo> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::MCDwarfFrameInfo*, std::vector<llvm::MCDwarfFrameInfo, std::allocator<llvm::MCDwarfFrameInfo> > >, llvm::MCDwarfFrameInfo const&)"},
    {"_ZNSt6vectorIN4llvm16MCCFIInstructionESaIS1_EEaSERKS3_", "std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> >::operator=(std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> > const&)"},
    {"_ZNSt6vectorIN4llvm16MCCFIInstructionESaIS1_EEC2ERKS3_", "std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> >::vector(std::vector<llvm::MCCFIInstruction, std::allocator<llvm::MCCFIInstruction> > const&)"},
    {"_ZNK4llvm8MCSymbol13AliasedSymbolEv", "llvm::MCSymbol::AliasedSymbol() const"},
    {"_ZN4llvm8MCSymbol16setVariableValueEPKNS_6MCExprE", "llvm::MCSymbol::setVariableValue(llvm::MCExpr const*)"},
    {"_ZNK4llvm8MCSymbol5printERNS_11raw_ostreamE", "llvm::MCSymbol::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm8MCSymbol4dumpEv", "llvm::MCSymbol::dump() const"},
    {"_ZN4llvm22createMachObjectWriterEPNS_24MCMachObjectTargetWriterERNS_11raw_ostreamEb", "llvm::createMachObjectWriter(llvm::MCMachObjectTargetWriter*, llvm::raw_ostream&, bool)"},
    {"_ZN12_GLOBAL__N_116MachObjectWriterD1Ev", "(anonymous namespace)::MachObjectWriter::~MachObjectWriter()"},
    {"_ZN12_GLOBAL__N_116MachObjectWriterD0Ev", "(anonymous namespace)::MachObjectWriter::~MachObjectWriter()"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter24ExecutePostLayoutBindingERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::MachObjectWriter::ExecutePostLayoutBinding(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter16RecordRelocationERKN4llvm11MCAssemblerERKNS1_11MCAsmLayoutEPKNS1_10MCFragmentERKNS1_7MCFixupENS1_7MCValueERy", "(anonymous namespace)::MachObjectWriter::RecordRelocation(llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::MCFragment const*, llvm::MCFixup const&, llvm::MCValue, unsigned long long&)"},
    {"_ZNK12_GLOBAL__N_116MachObjectWriter38IsSymbolRefDifferenceFullyResolvedImplERKN4llvm11MCAssemblerERKNS1_12MCSymbolDataERKNS1_10MCFragmentEbb", "(anonymous namespace)::MachObjectWriter::IsSymbolRefDifferenceFullyResolvedImpl(llvm::MCAssembler const&, llvm::MCSymbolData const&, llvm::MCFragment const&, bool, bool) const"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter11WriteObjectERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::MachObjectWriter::WriteObject(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZNK12_GLOBAL__N_116MachObjectWriter14getPaddingSizeEPKN4llvm13MCSectionDataERKNS1_11MCAsmLayoutE", "(anonymous namespace)::MachObjectWriter::getPaddingSize(llvm::MCSectionData const*, llvm::MCAsmLayout const&) const"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter10WriteNlistERNS0_14MachSymbolDataERKN4llvm11MCAsmLayoutE", "(anonymous namespace)::MachObjectWriter::WriteNlist((anonymous namespace)::MachObjectWriter::MachSymbolData&, llvm::MCAsmLayout const&)"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter25RecordScatteredRelocationERKN4llvm11MCAssemblerERKNS1_11MCAsmLayoutEPKNS1_10MCFragmentERKNS1_7MCFixupENS1_7MCValueEjRy", "(anonymous namespace)::MachObjectWriter::RecordScatteredRelocation(llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::MCFragment const*, llvm::MCFixup const&, llvm::MCValue, unsigned int, unsigned long long&)"},
    {"_ZN12_GLOBAL__N_116MachObjectWriter28RecordARMScatteredRelocationERKN4llvm11MCAssemblerERKNS1_11MCAsmLayoutEPKNS1_10MCFragmentERKNS1_7MCFixupENS1_7MCValueEjRy", "(anonymous namespace)::MachObjectWriter::RecordARMScatteredRelocation(llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::MCFragment const*, llvm::MCFixup const&, llvm::MCValue, unsigned int, unsigned long long&)"},
    {"_ZNSt6vectorIN12_GLOBAL__N_116MachObjectWriter14MachSymbolDataESaIS2_EE9push_backERKS2_", "std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> >::push_back((anonymous namespace)::MachObjectWriter::MachSymbolData const&)"},
    {"_ZSt4sortIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_116MachObjectWriter14MachSymbolDataESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > > >(__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >)"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_116MachObjectWriter14MachSymbolDataESt6vectorIS4_SaIS4_EEEElEvT_SA_T0_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, long>(__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, long)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_116MachObjectWriter14MachSymbolDataESt6vectorIS4_SaIS4_EEEEEvT_SA_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > > >(__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, __gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPN12_GLOBAL__N_116MachObjectWriter14MachSymbolDataESt6vectorIS4_SaIS4_EEEElS4_EvT_T0_SB_T1_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, long, (anonymous namespace)::MachObjectWriter::MachSymbolData>(__gnu_cxx::__normal_iterator<(anonymous namespace)::MachObjectWriter::MachSymbolData*, std::vector<(anonymous namespace)::MachObjectWriter::MachSymbolData, std::allocator<(anonymous namespace)::MachObjectWriter::MachSymbolData> > >, long, long, (anonymous namespace)::MachObjectWriter::MachSymbolData)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataESt6vectorINS_6object5macho15RelocationEntryESaIS7_EENS_12DenseMapInfoIS3_EENSA_IS9_EEE16FindAndConstructERKS3_", "llvm::DenseMap<llvm::MCSectionData const*, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> >, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > > >::FindAndConstruct(llvm::MCSectionData const* const&)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataESt6vectorINS_6object5macho15RelocationEntryESaIS7_EENS_12DenseMapInfoIS3_EENSA_IS9_EEE16InsertIntoBucketERKS3_RKS9_PSt4pairIS3_S9_E", "llvm::DenseMap<llvm::MCSectionData const*, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> >, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > > >::InsertIntoBucket(llvm::MCSectionData const* const&, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > const&, std::pair<llvm::MCSectionData const*, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > >*)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataESt6vectorINS_6object5macho15RelocationEntryESaIS7_EENS_12DenseMapInfoIS3_EENSA_IS9_EEE4growEj", "llvm::DenseMap<llvm::MCSectionData const*, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> >, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > > >::grow(unsigned int)"},
    {"_ZNSt6vectorIN4llvm6object5macho15RelocationEntryESaIS3_EEC2ERKS5_", "std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> >::vector(std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > const&)"},
    {"_ZNSt6vectorIN4llvm6object5macho15RelocationEntryESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::object::macho::RelocationEntry*, std::vector<llvm::object::macho::RelocationEntry, std::allocator<llvm::object::macho::RelocationEntry> > >, llvm::object::macho::RelocationEntry const&)"},
    {"_ZN4llvm9StringMapIyNS_15MallocAllocatorEE16GetOrCreateValueIyEERNS_14StringMapEntryIyEENS_9StringRefET_", "llvm::StringMapEntry<unsigned long long>& llvm::StringMap<unsigned long long, llvm::MallocAllocator>::GetOrCreateValue<unsigned long long>(llvm::StringRef, unsigned long long)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEhNS_12DenseMapInfoIS3_EENS4_IhEEE16InsertIntoBucketERKS3_RKhPSt4pairIS3_hE", "llvm::DenseMap<llvm::MCSection const*, unsigned char, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<unsigned char> >::InsertIntoBucket(llvm::MCSection const* const&, unsigned char const&, std::pair<llvm::MCSection const*, unsigned char>*)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEhNS_12DenseMapInfoIS3_EENS4_IhEEE4growEj", "llvm::DenseMap<llvm::MCSection const*, unsigned char, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<unsigned char> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEjNS_12DenseMapInfoIS3_EENS4_IjEEE16InsertIntoBucketERKS3_RKjPSt4pairIS3_jE", "llvm::DenseMap<llvm::MCSectionData const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned int> >::InsertIntoBucket(llvm::MCSectionData const* const&, unsigned int const&, std::pair<llvm::MCSectionData const*, unsigned int>*)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEjNS_12DenseMapInfoIS3_EENS4_IjEEE4growEj", "llvm::DenseMap<llvm::MCSectionData const*, unsigned int, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned int> >::grow(unsigned int)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoIS3_EENS4_IyEEE16InsertIntoBucketERKS3_RKyPSt4pairIS3_yE", "llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> >::InsertIntoBucket(llvm::MCSectionData const* const&, unsigned long long const&, std::pair<llvm::MCSectionData const*, unsigned long long>*)"},
    {"_ZN4llvm8DenseMapIPKNS_13MCSectionDataEyNS_12DenseMapInfoIS3_EENS4_IyEEE4growEj", "llvm::DenseMap<llvm::MCSectionData const*, unsigned long long, llvm::DenseMapInfo<llvm::MCSectionData const*>, llvm::DenseMapInfo<unsigned long long> >::grow(unsigned int)"},
    {"_ZN4llvm16TargetAsmBackendC1Ev", "llvm::TargetAsmBackend::TargetAsmBackend()"},
    {"_ZN4llvm16TargetAsmBackendC2Ev", "llvm::TargetAsmBackend::TargetAsmBackend()"},
    {"_ZN4llvm16TargetAsmBackendD0Ev", "llvm::TargetAsmBackend::~TargetAsmBackend()"},
    {"_ZN4llvm16TargetAsmBackendD1Ev", "llvm::TargetAsmBackend::~TargetAsmBackend()"},
    {"_ZN4llvm16TargetAsmBackendD2Ev", "llvm::TargetAsmBackend::~TargetAsmBackend()"},
    {"_ZNK4llvm16TargetAsmBackend16getFixupKindInfoENS_11MCFixupKindE", "llvm::TargetAsmBackend::getFixupKindInfo(llvm::MCFixupKind) const"},
    {"_ZN4llvm25createWinCOFFObjectWriterERNS_11raw_ostreamEb", "llvm::createWinCOFFObjectWriter(llvm::raw_ostream&, bool)"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriterD1Ev", "(anonymous namespace)::WinCOFFObjectWriter::~WinCOFFObjectWriter()"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriterD0Ev", "(anonymous namespace)::WinCOFFObjectWriter::~WinCOFFObjectWriter()"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriter24ExecutePostLayoutBindingERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::WinCOFFObjectWriter::ExecutePostLayoutBinding(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriter16RecordRelocationERKN4llvm11MCAssemblerERKNS1_11MCAsmLayoutEPKNS1_10MCFragmentERKNS1_7MCFixupENS1_7MCValueERy", "(anonymous namespace)::WinCOFFObjectWriter::RecordRelocation(llvm::MCAssembler const&, llvm::MCAsmLayout const&, llvm::MCFragment const*, llvm::MCFixup const&, llvm::MCValue, unsigned long long&)"},
    {"_ZNK12_GLOBAL__N_119WinCOFFObjectWriter38IsSymbolRefDifferenceFullyResolvedImplERKN4llvm11MCAssemblerERKNS1_12MCSymbolDataERKNS1_10MCFragmentEbb", "(anonymous namespace)::WinCOFFObjectWriter::IsSymbolRefDifferenceFullyResolvedImpl(llvm::MCAssembler const&, llvm::MCSymbolData const&, llvm::MCFragment const&, bool, bool) const"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriter11WriteObjectERN4llvm11MCAssemblerERKNS1_11MCAsmLayoutE", "(anonymous namespace)::WinCOFFObjectWriter::WriteObject(llvm::MCAssembler&, llvm::MCAsmLayout const&)"},
    {"_ZN4llvm8DenseMapIPKNS_9MCSectionEPN12_GLOBAL__N_111COFFSectionENS_12DenseMapInfoIS3_EENS7_IS6_EEEixERKS3_", "llvm::DenseMap<llvm::MCSection const*, (anonymous namespace)::COFFSection*, llvm::DenseMapInfo<llvm::MCSection const*>, llvm::DenseMapInfo<(anonymous namespace)::COFFSection*> >::operator[](llvm::MCSection const* const&)"},
    {"_ZN12_GLOBAL__N_111StringTable6insertEN4llvm9StringRefE", "(anonymous namespace)::StringTable::insert(llvm::StringRef)"},
    {"_ZN4llvm8DenseMapIPKNS_8MCSymbolEPN12_GLOBAL__N_110COFFSymbolENS_12DenseMapInfoIS3_EENS7_IS6_EEEixERKS3_", "llvm::DenseMap<llvm::MCSymbol const*, (anonymous namespace)::COFFSymbol*, llvm::DenseMapInfo<llvm::MCSymbol const*>, llvm::DenseMapInfo<(anonymous namespace)::COFFSymbol*> >::operator[](llvm::MCSymbol const* const&)"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriter21GetOrCreateCOFFSymbolEPKN4llvm8MCSymbolE", "(anonymous namespace)::WinCOFFObjectWriter::GetOrCreateCOFFSymbol(llvm::MCSymbol const*)"},
    {"_ZN4llvm15SmallVectorImplIN12_GLOBAL__N_19AuxSymbolEE6resizeEj", "llvm::SmallVectorImpl<(anonymous namespace)::AuxSymbol>::resize(unsigned int)"},
    {"_ZN12_GLOBAL__N_119WinCOFFObjectWriter16createCOFFEntityINS_10COFFSymbolESt6vectorIPS2_SaIS4_EEEEPT_N4llvm9StringRefERT0_", "(anonymous namespace)::COFFSymbol* (anonymous namespace)::WinCOFFObjectWriter::createCOFFEntity<(anonymous namespace)::COFFSymbol, std::vector<(anonymous namespace)::COFFSymbol*, std::allocator<(anonymous namespace)::COFFSymbol*> > >(llvm::StringRef, std::vector<(anonymous namespace)::COFFSymbol*, std::allocator<(anonymous namespace)::COFFSymbol*> >&)"},
    {"_ZN4llvm9StringMapImNS_15MallocAllocatorEE16GetOrCreateValueImEERNS_14StringMapEntryImEENS_9StringRefET_", "llvm::StringMapEntry<unsigned long>& llvm::StringMap<unsigned long, llvm::MallocAllocator>::GetOrCreateValue<unsigned long>(llvm::StringRef, unsigned long)"},
    {"_ZNSt6vectorIcSaIcEE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPcS1_EERKc", "std::vector<char, std::allocator<char> >::_M_insert_aux(__gnu_cxx::__normal_iterator<char*, std::vector<char, std::allocator<char> > >, char const&)"},
    {"_ZNSt6vectorIcSaIcEE15_M_range_insertIPKcEEvN9__gnu_cxx17__normal_iteratorIPcS1_EET_S9_St20forward_iterator_tag", "void std::vector<char, std::allocator<char> >::_M_range_insert<char const*>(__gnu_cxx::__normal_iterator<char*, std::vector<char, std::allocator<char> > >, char const*, char const*, std::forward_iterator_tag)"},
    {"_ZN4llvm21createWinCOFFStreamerERNS_9MCContextERNS_16TargetAsmBackendERNS_13MCCodeEmitterERNS_11raw_ostreamEb", "llvm::createWinCOFFStreamer(llvm::MCContext&, llvm::TargetAsmBackend&, llvm::MCCodeEmitter&, llvm::raw_ostream&, bool)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamerD1Ev", "(anonymous namespace)::WinCOFFStreamer::~WinCOFFStreamer()"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamerD0Ev", "(anonymous namespace)::WinCOFFStreamer::~WinCOFFStreamer()"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer12InitSectionsEv", "(anonymous namespace)::WinCOFFStreamer::InitSections()"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer9EmitLabelEPN4llvm8MCSymbolE", "(anonymous namespace)::WinCOFFStreamer::EmitLabel(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer17EmitAssemblerFlagEN4llvm15MCAssemblerFlagE", "(anonymous namespace)::WinCOFFStreamer::EmitAssemblerFlag(llvm::MCAssemblerFlag)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer13EmitThumbFuncEPN4llvm8MCSymbolE", "(anonymous namespace)::WinCOFFStreamer::EmitThumbFunc(llvm::MCSymbol*)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer14EmitAssignmentEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::WinCOFFStreamer::EmitAssignment(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer19EmitSymbolAttributeEPN4llvm8MCSymbolENS1_12MCSymbolAttrE", "(anonymous namespace)::WinCOFFStreamer::EmitSymbolAttribute(llvm::MCSymbol*, llvm::MCSymbolAttr)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer14EmitSymbolDescEPN4llvm8MCSymbolEj", "(anonymous namespace)::WinCOFFStreamer::EmitSymbolDesc(llvm::MCSymbol*, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer18BeginCOFFSymbolDefEPKN4llvm8MCSymbolE", "(anonymous namespace)::WinCOFFStreamer::BeginCOFFSymbolDef(llvm::MCSymbol const*)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer26EmitCOFFSymbolStorageClassEi", "(anonymous namespace)::WinCOFFStreamer::EmitCOFFSymbolStorageClass(int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer18EmitCOFFSymbolTypeEi", "(anonymous namespace)::WinCOFFStreamer::EmitCOFFSymbolType(int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer16EndCOFFSymbolDefEv", "(anonymous namespace)::WinCOFFStreamer::EndCOFFSymbolDef()"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer11EmitELFSizeEPN4llvm8MCSymbolEPKNS1_6MCExprE", "(anonymous namespace)::WinCOFFStreamer::EmitELFSize(llvm::MCSymbol*, llvm::MCExpr const*)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer16EmitCommonSymbolEPN4llvm8MCSymbolEyj", "(anonymous namespace)::WinCOFFStreamer::EmitCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer21EmitLocalCommonSymbolEPN4llvm8MCSymbolEy", "(anonymous namespace)::WinCOFFStreamer::EmitLocalCommonSymbol(llvm::MCSymbol*, unsigned long long)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer12EmitZerofillEPKN4llvm9MCSectionEPNS1_8MCSymbolEjj", "(anonymous namespace)::WinCOFFStreamer::EmitZerofill(llvm::MCSection const*, llvm::MCSymbol*, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer14EmitTBSSSymbolEPKN4llvm9MCSectionEPNS1_8MCSymbolEyj", "(anonymous namespace)::WinCOFFStreamer::EmitTBSSSymbol(llvm::MCSection const*, llvm::MCSymbol*, unsigned long long, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer9EmitBytesEN4llvm9StringRefEj", "(anonymous namespace)::WinCOFFStreamer::EmitBytes(llvm::StringRef, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer20EmitValueToAlignmentEjxjj", "(anonymous namespace)::WinCOFFStreamer::EmitValueToAlignment(unsigned int, long long, unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer17EmitCodeAlignmentEjj", "(anonymous namespace)::WinCOFFStreamer::EmitCodeAlignment(unsigned int, unsigned int)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer17EmitFileDirectiveEN4llvm9StringRefE", "(anonymous namespace)::WinCOFFStreamer::EmitFileDirective(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer15EmitInstructionERKN4llvm6MCInstE", "(anonymous namespace)::WinCOFFStreamer::EmitInstruction(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer6FinishEv", "(anonymous namespace)::WinCOFFStreamer::Finish()"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer14EmitInstToDataERKN4llvm6MCInstE", "(anonymous namespace)::WinCOFFStreamer::EmitInstToData(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer18EmitInstToFragmentERKN4llvm6MCInstE", "(anonymous namespace)::WinCOFFStreamer::EmitInstToFragment(llvm::MCInst const&)"},
    {"_ZN12_GLOBAL__N_115WinCOFFStreamer15AddCommonSymbolEPN4llvm8MCSymbolEyjb", "(anonymous namespace)::WinCOFFStreamer::AddCommonSymbol(llvm::MCSymbol*, unsigned long long, unsigned int, bool)"},
    {"_ZN4llvm14MCInstFragmentD1Ev", "llvm::MCInstFragment::~MCInstFragment()"},
    {"_ZN4llvm14MCInstFragmentD0Ev", "llvm::MCInstFragment::~MCInstFragment()"},
    {"_ZN4llvm7APFloat10initializeEPKNS_12fltSemanticsE", "llvm::APFloat::initialize(llvm::fltSemantics const*)"},
    {"_ZNK4llvm7APFloat9partCountEv", "llvm::APFloat::partCount() const"},
    {"_ZN4llvm7APFloat15freeSignificandEv", "llvm::APFloat::freeSignificand()"},
    {"_ZN4llvm7APFloat6assignERKS0_", "llvm::APFloat::assign(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat15copySignificandERKS0_", "llvm::APFloat::copySignificand(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat16significandPartsEv", "llvm::APFloat::significandParts()"},
    {"_ZNK4llvm7APFloat16significandPartsEv", "llvm::APFloat::significandParts() const"},
    {"_ZN4llvm7APFloat7makeNaNEbbPKNS_5APIntE", "llvm::APFloat::makeNaN(bool, bool, llvm::APInt const*)"},
    {"_ZN4llvm7APFloat7makeNaNERKNS_12fltSemanticsEbbPKNS_5APIntE", "llvm::APFloat::makeNaN(llvm::fltSemantics const&, bool, bool, llvm::APInt const*)"},
    {"_ZN4llvm7APFloatC1ERKNS_12fltSemanticsENS0_16uninitializedTagE", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::APFloat::uninitializedTag)"},
    {"_ZN4llvm7APFloatD1Ev", "llvm::APFloat::~APFloat()"},
    {"_ZN4llvm7APFloataSERKS0_", "llvm::APFloat::operator=(llvm::APFloat const&)"},
    {"_ZNK4llvm7APFloat14bitwiseIsEqualERKS0_", "llvm::APFloat::bitwiseIsEqual(llvm::APFloat const&) const"},
    {"_ZN4llvm7APFloatC1ERKNS_12fltSemanticsEy", "llvm::APFloat::APFloat(llvm::fltSemantics const&, unsigned long long)"},
    {"_ZN4llvm7APFloatC2ERKNS_12fltSemanticsEy", "llvm::APFloat::APFloat(llvm::fltSemantics const&, unsigned long long)"},
    {"_ZN4llvm7APFloat15zeroSignificandEv", "llvm::APFloat::zeroSignificand()"},
    {"_ZN4llvm7APFloat9normalizeENS0_12roundingModeENS_12lostFractionE", "llvm::APFloat::normalize(llvm::APFloat::roundingMode, llvm::lostFraction)"},
    {"_ZN4llvm7APFloatC1ERKNS_12fltSemanticsE", "llvm::APFloat::APFloat(llvm::fltSemantics const&)"},
    {"_ZN4llvm7APFloatC2ERKNS_12fltSemanticsE", "llvm::APFloat::APFloat(llvm::fltSemantics const&)"},
    {"_ZN4llvm7APFloatC2ERKNS_12fltSemanticsENS0_16uninitializedTagE", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::APFloat::uninitializedTag)"},
    {"_ZN4llvm7APFloatC1ERKNS_12fltSemanticsENS0_11fltCategoryEb", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::APFloat::fltCategory, bool)"},
    {"_ZN4llvm7APFloatC2ERKNS_12fltSemanticsENS0_11fltCategoryEb", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::APFloat::fltCategory, bool)"},
    {"_ZN4llvm7APFloatC1ERKNS_12fltSemanticsENS_9StringRefE", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::StringRef)"},
    {"_ZN4llvm7APFloatC2ERKNS_12fltSemanticsENS_9StringRefE", "llvm::APFloat::APFloat(llvm::fltSemantics const&, llvm::StringRef)"},
    {"_ZN4llvm7APFloat17convertFromStringENS_9StringRefENS0_12roundingModeE", "llvm::APFloat::convertFromString(llvm::StringRef, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloatC1ERKS0_", "llvm::APFloat::APFloat(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloatC2ERKS0_", "llvm::APFloat::APFloat(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloatD2Ev", "llvm::APFloat::~APFloat()"},
    {"_ZNK4llvm7APFloat7ProfileERNS_16FoldingSetNodeIDE", "llvm::APFloat::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZNK4llvm7APFloat14bitcastToAPIntEv", "llvm::APFloat::bitcastToAPInt() const"},
    {"_ZN4llvm7APFloat18semanticsPrecisionERKNS_12fltSemanticsE", "llvm::APFloat::semanticsPrecision(llvm::fltSemantics const&)"},
    {"_ZN4llvm7APFloat20incrementSignificandEv", "llvm::APFloat::incrementSignificand()"},
    {"_ZN4llvm7APFloat14addSignificandERKS0_", "llvm::APFloat::addSignificand(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat19subtractSignificandERKS0_y", "llvm::APFloat::subtractSignificand(llvm::APFloat const&, unsigned long long)"},
    {"_ZN4llvm7APFloat19multiplySignificandERKS0_PS1_", "llvm::APFloat::multiplySignificand(llvm::APFloat const&, llvm::APFloat const*)"},
    {"_ZN4llvm7APFloat7convertERKNS_12fltSemanticsENS0_12roundingModeEPb", "llvm::APFloat::convert(llvm::fltSemantics const&, llvm::APFloat::roundingMode, bool*)"},
    {"_ZN4llvm7APFloat24addOrSubtractSignificandERKS0_b", "llvm::APFloat::addOrSubtractSignificand(llvm::APFloat const&, bool)"},
    {"_ZN4llvm7APFloat17divideSignificandERKS0_", "llvm::APFloat::divideSignificand(llvm::APFloat const&)"},
    {"_ZNK4llvm7APFloat14significandMSBEv", "llvm::APFloat::significandMSB() const"},
    {"_ZNK4llvm7APFloat14significandLSBEv", "llvm::APFloat::significandLSB() const"},
    {"_ZN4llvm7APFloat21shiftSignificandRightEj", "llvm::APFloat::shiftSignificandRight(unsigned int)"},
    {"_ZN4llvm7APFloat20shiftSignificandLeftEj", "llvm::APFloat::shiftSignificandLeft(unsigned int)"},
    {"_ZNK4llvm7APFloat20compareAbsoluteValueERKS0_", "llvm::APFloat::compareAbsoluteValue(llvm::APFloat const&) const"},
    {"_ZN4llvm7APFloat14handleOverflowENS0_12roundingModeE", "llvm::APFloat::handleOverflow(llvm::APFloat::roundingMode)"},
    {"_ZNK4llvm7APFloat17roundAwayFromZeroENS0_12roundingModeENS_12lostFractionEj", "llvm::APFloat::roundAwayFromZero(llvm::APFloat::roundingMode, llvm::lostFraction, unsigned int) const"},
    {"_ZN4llvm7APFloat21addOrSubtractSpecialsERKS0_b", "llvm::APFloat::addOrSubtractSpecials(llvm::APFloat const&, bool)"},
    {"_ZN4llvm7APFloat16multiplySpecialsERKS0_", "llvm::APFloat::multiplySpecials(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat14divideSpecialsERKS0_", "llvm::APFloat::divideSpecials(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat11modSpecialsERKS0_", "llvm::APFloat::modSpecials(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat10changeSignEv", "llvm::APFloat::changeSign()"},
    {"_ZN4llvm7APFloat9clearSignEv", "llvm::APFloat::clearSign()"},
    {"_ZN4llvm7APFloat8copySignERKS0_", "llvm::APFloat::copySign(llvm::APFloat const&)"},
    {"_ZN4llvm7APFloat13addOrSubtractERKS0_NS0_12roundingModeEb", "llvm::APFloat::addOrSubtract(llvm::APFloat const&, llvm::APFloat::roundingMode, bool)"},
    {"_ZN4llvm7APFloat3addERKS0_NS0_12roundingModeE", "llvm::APFloat::add(llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat8subtractERKS0_NS0_12roundingModeE", "llvm::APFloat::subtract(llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat8multiplyERKS0_NS0_12roundingModeE", "llvm::APFloat::multiply(llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat6divideERKS0_NS0_12roundingModeE", "llvm::APFloat::divide(llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat9remainderERKS0_", "llvm::APFloat::remainder(llvm::APFloat const&)"},
    {"_ZNK4llvm7APFloat16convertToIntegerEPyjbNS0_12roundingModeEPb", "llvm::APFloat::convertToInteger(unsigned long long*, unsigned int, bool, llvm::APFloat::roundingMode, bool*) const"},
    {"_ZN4llvm7APFloat30convertFromZeroExtendedIntegerEPKyjbNS0_12roundingModeE", "llvm::APFloat::convertFromZeroExtendedInteger(unsigned long long const*, unsigned int, bool, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat3modERKS0_NS0_12roundingModeE", "llvm::APFloat::mod(llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat16fusedMultiplyAddERKS0_S2_NS0_12roundingModeE", "llvm::APFloat::fusedMultiplyAdd(llvm::APFloat const&, llvm::APFloat const&, llvm::APFloat::roundingMode)"},
    {"_ZNK4llvm7APFloat7compareERKS0_", "llvm::APFloat::compare(llvm::APFloat const&) const"},
    {"_ZNK4llvm7APFloat28convertToSignExtendedIntegerEPyjbNS0_12roundingModeEPb", "llvm::APFloat::convertToSignExtendedInteger(unsigned long long*, unsigned int, bool, llvm::APFloat::roundingMode, bool*) const"},
    {"_ZN4llvm7APFloat24convertFromUnsignedPartsEPKyjNS0_12roundingModeE", "llvm::APFloat::convertFromUnsignedParts(unsigned long long const*, unsigned int, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat16convertFromAPIntERKNS_5APIntEbNS0_12roundingModeE", "llvm::APFloat::convertFromAPInt(llvm::APInt const&, bool, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat30convertFromSignExtendedIntegerEPKyjbNS0_12roundingModeE", "llvm::APFloat::convertFromSignExtendedInteger(unsigned long long const*, unsigned int, bool, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat28convertFromHexadecimalStringENS_9StringRefENS0_12roundingModeE", "llvm::APFloat::convertFromHexadecimalString(llvm::StringRef, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat28roundSignificandWithExponentEPKyjiNS0_12roundingModeE", "llvm::APFloat::roundSignificandWithExponent(unsigned long long const*, unsigned int, int, llvm::APFloat::roundingMode)"},
    {"_ZN4llvm7APFloat24convertFromDecimalStringENS_9StringRefENS0_12roundingModeE", "llvm::APFloat::convertFromDecimalString(llvm::StringRef, llvm::APFloat::roundingMode)"},
    {"_ZNK4llvm7APFloat18convertToHexStringEPcjbNS0_12roundingModeE", "llvm::APFloat::convertToHexString(char*, unsigned int, bool, llvm::APFloat::roundingMode) const"},
    {"_ZNK4llvm7APFloat24convertNormalToHexStringEPcjbNS0_12roundingModeE", "llvm::APFloat::convertNormalToHexString(char*, unsigned int, bool, llvm::APFloat::roundingMode) const"},
    {"_ZNK4llvm7APFloat12getHashValueEv", "llvm::APFloat::getHashValue() const"},
    {"_ZNK4llvm7APFloat34convertF80LongDoubleAPFloatToAPIntEv", "llvm::APFloat::convertF80LongDoubleAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat36convertPPCDoubleDoubleAPFloatToAPIntEv", "llvm::APFloat::convertPPCDoubleDoubleAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat30convertQuadrupleAPFloatToAPIntEv", "llvm::APFloat::convertQuadrupleAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat27convertDoubleAPFloatToAPIntEv", "llvm::APFloat::convertDoubleAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat26convertFloatAPFloatToAPIntEv", "llvm::APFloat::convertFloatAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat25convertHalfAPFloatToAPIntEv", "llvm::APFloat::convertHalfAPFloatToAPInt() const"},
    {"_ZNK4llvm7APFloat14convertToFloatEv", "llvm::APFloat::convertToFloat() const"},
    {"_ZNK4llvm7APFloat15convertToDoubleEv", "llvm::APFloat::convertToDouble() const"},
    {"_ZN4llvm7APFloat26initFromF80LongDoubleAPIntERKNS_5APIntE", "llvm::APFloat::initFromF80LongDoubleAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat28initFromPPCDoubleDoubleAPIntERKNS_5APIntE", "llvm::APFloat::initFromPPCDoubleDoubleAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat22initFromQuadrupleAPIntERKNS_5APIntE", "llvm::APFloat::initFromQuadrupleAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat19initFromDoubleAPIntERKNS_5APIntE", "llvm::APFloat::initFromDoubleAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat18initFromFloatAPIntERKNS_5APIntE", "llvm::APFloat::initFromFloatAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat17initFromHalfAPIntERKNS_5APIntE", "llvm::APFloat::initFromHalfAPInt(llvm::APInt const&)"},
    {"_ZN4llvm7APFloat13initFromAPIntERKNS_5APIntEb", "llvm::APFloat::initFromAPInt(llvm::APInt const&, bool)"},
    {"_ZN4llvm7APFloat10getLargestERKNS_12fltSemanticsEb", "llvm::APFloat::getLargest(llvm::fltSemantics const&, bool)"},
    {"_ZN4llvm7APFloat11getSmallestERKNS_12fltSemanticsEb", "llvm::APFloat::getSmallest(llvm::fltSemantics const&, bool)"},
    {"_ZN4llvm7APFloat21getSmallestNormalizedERKNS_12fltSemanticsEb", "llvm::APFloat::getSmallestNormalized(llvm::fltSemantics const&, bool)"},
    {"_ZN4llvm7APFloatC1ERKNS_5APIntEb", "llvm::APFloat::APFloat(llvm::APInt const&, bool)"},
    {"_ZN4llvm7APFloatC2ERKNS_5APIntEb", "llvm::APFloat::APFloat(llvm::APInt const&, bool)"},
    {"_ZN4llvm7APFloatC1Ef", "llvm::APFloat::APFloat(float)"},
    {"_ZN4llvm7APFloatC2Ef", "llvm::APFloat::APFloat(float)"},
    {"_ZN4llvm7APFloatC1Ed", "llvm::APFloat::APFloat(double)"},
    {"_ZN4llvm7APFloatC2Ed", "llvm::APFloat::APFloat(double)"},
    {"_ZNK4llvm7APFloat8toStringERNS_15SmallVectorImplIcEEjj", "llvm::APFloat::toString(llvm::SmallVectorImpl<char>&, unsigned int, unsigned int) const"},
    {"_ZNK4llvm5APIntngEv", "llvm::APInt::operator-() const"},
    {"_ZN4llvm5APIntlSEj", "llvm::APInt::operator<<=(unsigned int)"},
    {"_ZN4llvm5APInt12initSlowCaseEjyb", "llvm::APInt::initSlowCase(unsigned int, unsigned long long, bool)"},
    {"_ZN4llvm5APInt12initSlowCaseERKS0_", "llvm::APInt::initSlowCase(llvm::APInt const&)"},
    {"_ZN4llvm5APIntC1EjjPKy", "llvm::APInt::APInt(unsigned int, unsigned int, unsigned long long const*)"},
    {"_ZN4llvm5APIntC2EjjPKy", "llvm::APInt::APInt(unsigned int, unsigned int, unsigned long long const*)"},
    {"_ZN4llvm5APIntC1EjNS_9StringRefEh", "llvm::APInt::APInt(unsigned int, llvm::StringRef, unsigned char)"},
    {"_ZN4llvm5APIntC2EjNS_9StringRefEh", "llvm::APInt::APInt(unsigned int, llvm::StringRef, unsigned char)"},
    {"_ZN4llvm5APInt10fromStringEjNS_9StringRefEh", "llvm::APInt::fromString(unsigned int, llvm::StringRef, unsigned char)"},
    {"_ZN4llvm5APInt14AssignSlowCaseERKS0_", "llvm::APInt::AssignSlowCase(llvm::APInt const&)"},
    {"_ZN4llvm5APIntaSEy", "llvm::APInt::operator=(unsigned long long)"},
    {"_ZNK4llvm5APInt7ProfileERNS_16FoldingSetNodeIDE", "llvm::APInt::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm5APIntppEv", "llvm::APInt::operator++()"},
    {"_ZN4llvm5APIntmmEv", "llvm::APInt::operator--()"},
    {"_ZN4llvm5APIntpLERKS0_", "llvm::APInt::operator+=(llvm::APInt const&)"},
    {"_ZN4llvm5APIntmIERKS0_", "llvm::APInt::operator-=(llvm::APInt const&)"},
    {"_ZN4llvm5APIntmLERKS0_", "llvm::APInt::operator*=(llvm::APInt const&)"},
    {"_ZN4llvm5APIntaNERKS0_", "llvm::APInt::operator&=(llvm::APInt const&)"},
    {"_ZN4llvm5APIntoRERKS0_", "llvm::APInt::operator|=(llvm::APInt const&)"},
    {"_ZN4llvm5APInteOERKS0_", "llvm::APInt::operator^=(llvm::APInt const&)"},
    {"_ZNK4llvm5APInt11AndSlowCaseERKS0_", "llvm::APInt::AndSlowCase(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt10OrSlowCaseERKS0_", "llvm::APInt::OrSlowCase(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt11XorSlowCaseERKS0_", "llvm::APInt::XorSlowCase(llvm::APInt const&) const"},
    {"_ZNK4llvm5APIntntEv", "llvm::APInt::operator!() const"},
    {"_ZNK4llvm5APIntmlERKS0_", "llvm::APInt::operator*(llvm::APInt const&) const"},
    {"_ZNK4llvm5APIntplERKS0_", "llvm::APInt::operator+(llvm::APInt const&) const"},
    {"_ZNK4llvm5APIntmiERKS0_", "llvm::APInt::operator-(llvm::APInt const&) const"},
    {"_ZNK4llvm5APIntixEj", "llvm::APInt::operator[](unsigned int) const"},
    {"_ZNK4llvm5APInt13EqualSlowCaseERKS0_", "llvm::APInt::EqualSlowCase(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt13EqualSlowCaseEy", "llvm::APInt::EqualSlowCase(unsigned long long) const"},
    {"_ZNK4llvm5APInt3ultERKS0_", "llvm::APInt::ult(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt3sltERKS0_", "llvm::APInt::slt(llvm::APInt const&) const"},
    {"_ZN4llvm5APInt6setBitEj", "llvm::APInt::setBit(unsigned int)"},
    {"_ZN4llvm5APInt8clearBitEj", "llvm::APInt::clearBit(unsigned int)"},
    {"_ZN4llvm5APInt7flipBitEj", "llvm::APInt::flipBit(unsigned int)"},
    {"_ZN4llvm5APInt13getBitsNeededENS_9StringRefEh", "llvm::APInt::getBitsNeeded(llvm::StringRef, unsigned char)"},
    {"_ZNK4llvm5APInt12getHashValueEv", "llvm::APInt::getHashValue() const"},
    {"_ZNK4llvm5APInt9getHiBitsEj", "llvm::APInt::getHiBits(unsigned int) const"},
    {"_ZNK4llvm5APInt9getLoBitsEj", "llvm::APInt::getLoBits(unsigned int) const"},
    {"_ZNK4llvm5APInt25countLeadingZerosSlowCaseEv", "llvm::APInt::countLeadingZerosSlowCase() const"},
    {"_ZNK4llvm5APInt16countLeadingOnesEv", "llvm::APInt::countLeadingOnes() const"},
    {"_ZNK4llvm5APInt18countTrailingZerosEv", "llvm::APInt::countTrailingZeros() const"},
    {"_ZNK4llvm5APInt25countTrailingOnesSlowCaseEv", "llvm::APInt::countTrailingOnesSlowCase() const"},
    {"_ZNK4llvm5APInt23countPopulationSlowCaseEv", "llvm::APInt::countPopulationSlowCase() const"},
    {"_ZNK4llvm5APInt8byteSwapEv", "llvm::APInt::byteSwap() const"},
    {"_ZN4llvm8APIntOps21GreatestCommonDivisorERKNS_5APIntES3_", "llvm::APIntOps::GreatestCommonDivisor(llvm::APInt const&, llvm::APInt const&)"},
    {"_ZN4llvm8APIntOps18RoundDoubleToAPIntEdj", "llvm::APIntOps::RoundDoubleToAPInt(double, unsigned int)"},
    {"_ZNK4llvm5APInt13roundToDoubleEb", "llvm::APInt::roundToDouble(bool) const"},
    {"_ZNK4llvm5APInt5truncEj", "llvm::APInt::trunc(unsigned int) const"},
    {"_ZNK4llvm5APInt4sextEj", "llvm::APInt::sext(unsigned int) const"},
    {"_ZNK4llvm5APInt4zextEj", "llvm::APInt::zext(unsigned int) const"},
    {"_ZNK4llvm5APInt11zextOrTruncEj", "llvm::APInt::zextOrTrunc(unsigned int) const"},
    {"_ZNK4llvm5APInt11sextOrTruncEj", "llvm::APInt::sextOrTrunc(unsigned int) const"},
    {"_ZNK4llvm5APInt4ashrERKS0_", "llvm::APInt::ashr(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt4ashrEj", "llvm::APInt::ashr(unsigned int) const"},
    {"_ZNK4llvm5APInt4lshrERKS0_", "llvm::APInt::lshr(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt4lshrEj", "llvm::APInt::lshr(unsigned int) const"},
    {"_ZNK4llvm5APInt3shlERKS0_", "llvm::APInt::shl(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt11shlSlowCaseEj", "llvm::APInt::shlSlowCase(unsigned int) const"},
    {"_ZNK4llvm5APInt4rotlERKS0_", "llvm::APInt::rotl(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt4rotlEj", "llvm::APInt::rotl(unsigned int) const"},
    {"_ZNK4llvm5APInt4rotrERKS0_", "llvm::APInt::rotr(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt4rotrEj", "llvm::APInt::rotr(unsigned int) const"},
    {"_ZNK4llvm5APInt4sqrtEv", "llvm::APInt::sqrt() const"},
    {"_ZNK4llvm5APInt4udivERKS0_", "llvm::APInt::udiv(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt21multiplicativeInverseERKS0_", "llvm::APInt::multiplicativeInverse(llvm::APInt const&) const"},
    {"_ZN4llvm5APInt7udivremERKS0_S2_RS0_S3_", "llvm::APInt::udivrem(llvm::APInt const&, llvm::APInt const&, llvm::APInt&, llvm::APInt&)"},
    {"_ZNK4llvm5APInt5magicEv", "llvm::APInt::magic() const"},
    {"_ZNK4llvm5APInt4uremERKS0_", "llvm::APInt::urem(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt6magicuEv", "llvm::APInt::magicu() const"},
    {"_ZN4llvm5APInt6divideES0_jRKS0_jPS0_S3_", "llvm::APInt::divide(llvm::APInt, unsigned int, llvm::APInt const&, unsigned int, llvm::APInt*, llvm::APInt*)"},
    {"_ZNK4llvm5APInt7sadd_ovERKS0_Rb", "llvm::APInt::sadd_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7uadd_ovERKS0_Rb", "llvm::APInt::uadd_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7ssub_ovERKS0_Rb", "llvm::APInt::ssub_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7usub_ovERKS0_Rb", "llvm::APInt::usub_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7sdiv_ovERKS0_Rb", "llvm::APInt::sdiv_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7smul_ovERKS0_Rb", "llvm::APInt::smul_ov(llvm::APInt const&, bool&) const"},
    {"_ZNK4llvm5APInt7sshl_ovEjRb", "llvm::APInt::sshl_ov(unsigned int, bool&) const"},
    {"_ZNK4llvm5APInt8toStringERNS_15SmallVectorImplIcEEjb", "llvm::APInt::toString(llvm::SmallVectorImpl<char>&, unsigned int, bool) const"},
    {"_ZNK4llvm5APInt8toStringEjb", "llvm::APInt::toString(unsigned int, bool) const"},
    {"_ZNK4llvm5APInt4dumpEv", "llvm::APInt::dump() const"},
    {"_ZNK4llvm5APInt5printERNS_11raw_ostreamEb", "llvm::APInt::print(llvm::raw_ostream&, bool) const"},
    {"_ZN4llvm5APInt5tcSetEPyyj", "llvm::APInt::tcSet(unsigned long long*, unsigned long long, unsigned int)"},
    {"_ZN4llvm5APInt8tcAssignEPyPKyj", "llvm::APInt::tcAssign(unsigned long long*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt8tcIsZeroEPKyj", "llvm::APInt::tcIsZero(unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt12tcExtractBitEPKyj", "llvm::APInt::tcExtractBit(unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt8tcSetBitEPyj", "llvm::APInt::tcSetBit(unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt10tcClearBitEPyj", "llvm::APInt::tcClearBit(unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt5tcLSBEPKyj", "llvm::APInt::tcLSB(unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt5tcMSBEPKyj", "llvm::APInt::tcMSB(unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt9tcExtractEPyjPKyjj", "llvm::APInt::tcExtract(unsigned long long*, unsigned int, unsigned long long const*, unsigned int, unsigned int)"},
    {"_ZN4llvm5APInt12tcShiftRightEPyjj", "llvm::APInt::tcShiftRight(unsigned long long*, unsigned int, unsigned int)"},
    {"_ZN4llvm5APInt5tcAddEPyPKyyj", "llvm::APInt::tcAdd(unsigned long long*, unsigned long long const*, unsigned long long, unsigned int)"},
    {"_ZN4llvm5APInt10tcSubtractEPyPKyyj", "llvm::APInt::tcSubtract(unsigned long long*, unsigned long long const*, unsigned long long, unsigned int)"},
    {"_ZN4llvm5APInt8tcNegateEPyj", "llvm::APInt::tcNegate(unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt12tcComplementEPyj", "llvm::APInt::tcComplement(unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt11tcIncrementEPyj", "llvm::APInt::tcIncrement(unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt14tcMultiplyPartEPyPKyyyjjb", "llvm::APInt::tcMultiplyPart(unsigned long long*, unsigned long long const*, unsigned long long, unsigned long long, unsigned int, unsigned int, bool)"},
    {"_ZN4llvm5APInt10tcMultiplyEPyPKyS3_j", "llvm::APInt::tcMultiply(unsigned long long*, unsigned long long const*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt14tcFullMultiplyEPyPKyS3_jj", "llvm::APInt::tcFullMultiply(unsigned long long*, unsigned long long const*, unsigned long long const*, unsigned int, unsigned int)"},
    {"_ZN4llvm5APInt8tcDivideEPyPKyS1_S1_j", "llvm::APInt::tcDivide(unsigned long long*, unsigned long long const*, unsigned long long*, unsigned long long*, unsigned int)"},
    {"_ZN4llvm5APInt11tcShiftLeftEPyjj", "llvm::APInt::tcShiftLeft(unsigned long long*, unsigned int, unsigned int)"},
    {"_ZN4llvm5APInt9tcCompareEPKyS2_j", "llvm::APInt::tcCompare(unsigned long long const*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt5tcAndEPyPKyj", "llvm::APInt::tcAnd(unsigned long long*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt4tcOrEPyPKyj", "llvm::APInt::tcOr(unsigned long long*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt5tcXorEPyPKyj", "llvm::APInt::tcXor(unsigned long long*, unsigned long long const*, unsigned int)"},
    {"_ZN4llvm5APInt25tcSetLeastSignificantBitsEPyjj", "llvm::APInt::tcSetLeastSignificantBits(unsigned long long*, unsigned int, unsigned int)"},
    {"_ZN4llvm5APIntppEi", "llvm::APInt::operator++(int)"},
    {"_ZNK4llvm5APInt3absEv", "llvm::APInt::abs() const"},
    {"_ZNK4llvm5APInteqERKS0_", "llvm::APInt::operator==(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt16isMinSignedValueEv", "llvm::APInt::isMinSignedValue() const"},
    {"_ZNK4llvm5APInt4sdivERKS0_", "llvm::APInt::sdiv(llvm::APInt const&) const"},
    {"_ZNK4llvm5APInt17countLeadingZerosEv", "llvm::APInt::countLeadingZeros() const"},
    {"_ZN4llvm5APIntmmEi", "llvm::APInt::operator--(int)"},
    {"_ZN4llvm5APIntC2Ejyb", "llvm::APInt::APInt(unsigned int, unsigned long long, bool)"},
    {"_ZNK4llvm6APSInt7ProfileERNS_16FoldingSetNodeIDE", "llvm::APSInt::Profile(llvm::FoldingSetNodeID&) const"},
    {"_ZN4llvm16BumpPtrAllocatorC1EmmRNS_13SlabAllocatorE", "llvm::BumpPtrAllocator::BumpPtrAllocator(unsigned long, unsigned long, llvm::SlabAllocator&)"},
    {"_ZN4llvm16BumpPtrAllocatorC2EmmRNS_13SlabAllocatorE", "llvm::BumpPtrAllocator::BumpPtrAllocator(unsigned long, unsigned long, llvm::SlabAllocator&)"},
    {"_ZN4llvm16BumpPtrAllocatorD1Ev", "llvm::BumpPtrAllocator::~BumpPtrAllocator()"},
    {"_ZN4llvm16BumpPtrAllocatorD2Ev", "llvm::BumpPtrAllocator::~BumpPtrAllocator()"},
    {"_ZN4llvm16BumpPtrAllocator15DeallocateSlabsEPNS_7MemSlabE", "llvm::BumpPtrAllocator::DeallocateSlabs(llvm::MemSlab*)"},
    {"_ZN4llvm16BumpPtrAllocator8AlignPtrEPcm", "llvm::BumpPtrAllocator::AlignPtr(char*, unsigned long)"},
    {"_ZN4llvm16BumpPtrAllocator12StartNewSlabEv", "llvm::BumpPtrAllocator::StartNewSlab()"},
    {"_ZN4llvm16BumpPtrAllocator5ResetEv", "llvm::BumpPtrAllocator::Reset()"},
    {"_ZN4llvm16BumpPtrAllocator8AllocateEmm", "llvm::BumpPtrAllocator::Allocate(unsigned long, unsigned long)"},
    {"_ZNK4llvm16BumpPtrAllocator11GetNumSlabsEv", "llvm::BumpPtrAllocator::GetNumSlabs() const"},
    {"_ZNK4llvm16BumpPtrAllocator10PrintStatsEv", "llvm::BumpPtrAllocator::PrintStats() const"},
    {"_ZN4llvm19MallocSlabAllocatorD1Ev", "llvm::MallocSlabAllocator::~MallocSlabAllocator()"},
    {"_ZN4llvm13SlabAllocatorD0Ev", "llvm::SlabAllocator::~SlabAllocator()"},
    {"_ZN4llvm13SlabAllocatorD1Ev", "llvm::SlabAllocator::~SlabAllocator()"},
    {"_ZN4llvm13SlabAllocatorD2Ev", "llvm::SlabAllocator::~SlabAllocator()"},
    {"_ZN4llvm19MallocSlabAllocatorD0Ev", "llvm::MallocSlabAllocator::~MallocSlabAllocator()"},
    {"_ZN4llvm19MallocSlabAllocatorD2Ev", "llvm::MallocSlabAllocator::~MallocSlabAllocator()"},
    {"_ZN4llvm19MallocSlabAllocator8AllocateEm", "llvm::MallocSlabAllocator::Allocate(unsigned long)"},
    {"_ZN4llvm19MallocSlabAllocator10DeallocateEPNS_7MemSlabE", "llvm::MallocSlabAllocator::Deallocate(llvm::MemSlab*)"},
    {"_ZN4llvm18PrintRecyclerStatsEmmm", "llvm::PrintRecyclerStats(unsigned long, unsigned long, unsigned long)"},
    {"_ZN4llvm3sys14CompareAndSwapEPVjjj", "llvm::sys::CompareAndSwap(unsigned int volatile*, unsigned int, unsigned int)"},
    {"_ZN4llvm3sys15AtomicIncrementEPVj", "llvm::sys::AtomicIncrement(unsigned int volatile*)"},
    {"_ZN4llvm3sys15AtomicDecrementEPVj", "llvm::sys::AtomicDecrement(unsigned int volatile*)"},
    {"_ZN4llvm3sys9AtomicAddEPVjj", "llvm::sys::AtomicAdd(unsigned int volatile*, unsigned int)"},
    {"_ZN4llvm3sys9AtomicMulEPVjj", "llvm::sys::AtomicMul(unsigned int volatile*, unsigned int)"},
    {"_ZN4llvm3sys9AtomicDivEPVjj", "llvm::sys::AtomicDiv(unsigned int volatile*, unsigned int)"},
    {"_ZN4llvm2cl6parserIjE5parseERNS0_6OptionENS_9StringRefES5_Rj", "llvm::cl::parser<unsigned int>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, unsigned int&)"},
    {"_ZNK4llvm2cl17basic_parser_impl14getOptionWidthERKNS0_6OptionE", "llvm::cl::basic_parser_impl::getOptionWidth(llvm::cl::Option const&) const"},
    {"_ZNK4llvm2cl17basic_parser_impl15printOptionInfoERKNS0_6OptionEm", "llvm::cl::basic_parser_impl::printOptionInfo(llvm::cl::Option const&, unsigned long) const"},
    {"_ZN4llvm2cl6parserIiE5parseERNS0_6OptionENS_9StringRefES5_Ri", "llvm::cl::parser<int>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, int&)"},
    {"_ZN4llvm2cl6parserIbE5parseERNS0_6OptionENS_9StringRefES5_Rb", "llvm::cl::parser<bool>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, bool&)"},
    {"_ZN4llvm2cl6Option6anchorEv", "llvm::cl::Option::anchor()"},
    {"_ZN4llvm2cl17basic_parser_impl6anchorEv", "llvm::cl::basic_parser_impl::anchor()"},
    {"_ZN4llvm2cl6parserIbE6anchorEv", "llvm::cl::parser<bool>::anchor()"},
    {"_ZN4llvm2cl6parserINS0_13boolOrDefaultEE6anchorEv", "llvm::cl::parser<llvm::cl::boolOrDefault>::anchor()"},
    {"_ZN4llvm2cl6parserIiE6anchorEv", "llvm::cl::parser<int>::anchor()"},
    {"_ZN4llvm2cl6parserIjE6anchorEv", "llvm::cl::parser<unsigned int>::anchor()"},
    {"_ZN4llvm2cl6parserIdE6anchorEv", "llvm::cl::parser<double>::anchor()"},
    {"_ZN4llvm2cl6parserIfE6anchorEv", "llvm::cl::parser<float>::anchor()"},
    {"_ZN4llvm2cl6parserISsE6anchorEv", "llvm::cl::parser<std::string>::anchor()"},
    {"_ZN4llvm2cl6parserIcE6anchorEv", "llvm::cl::parser<char>::anchor()"},
    {"_ZN4llvm2cl9extrahelpC1EPKc", "llvm::cl::extrahelp::extrahelp(char const*)"},
    {"_ZN4llvm2cl9extrahelpC2EPKc", "llvm::cl::extrahelp::extrahelp(char const*)"},
    {"_ZN4llvm2cl23ParseEnvironmentOptionsEPKcS2_S2_b", "llvm::cl::ParseEnvironmentOptions(char const*, char const*, char const*, bool)"},
    {"_Z18ParseCStringVectorRSt6vectorIPcSaIS0_EEPKc", "ParseCStringVector(std::vector<char*, std::allocator<char*> >&, char const*)"},
    {"_ZN4llvm2cl23ParseCommandLineOptionsEiPPcPKcb", "llvm::cl::ParseCommandLineOptions(int, char**, char const*, bool)"},
    {"_Z13GetOptionInfoRN4llvm15SmallVectorImplIPNS_2cl6OptionEEES5_RNS_9StringMapIS3_NS_15MallocAllocatorEEE", "GetOptionInfo(llvm::SmallVectorImpl<llvm::cl::Option*>&, llvm::SmallVectorImpl<llvm::cl::Option*>&, llvm::StringMap<llvm::cl::Option*, llvm::MallocAllocator>&)"},
    {"_ZN4llvm2cl6Option5errorERKNS_5TwineENS_9StringRefE", "llvm::cl::Option::error(llvm::Twine const&, llvm::StringRef)"},
    {"_Z12LookupOptionRN4llvm9StringRefES1_RKNS_9StringMapIPNS_2cl6OptionENS_15MallocAllocatorEEE", "LookupOption(llvm::StringRef&, llvm::StringRef&, llvm::StringMap<llvm::cl::Option*, llvm::MallocAllocator> const&)"},
    {"_ZN4llvm2cl6Option13addOccurrenceEjNS_9StringRefES2_b", "llvm::cl::Option::addOccurrence(unsigned int, llvm::StringRef, llvm::StringRef, bool)"},
    {"_Z13ProvideOptionPN4llvm2cl6OptionENS_9StringRefES3_iPPcRi", "ProvideOption(llvm::cl::Option*, llvm::StringRef, llvm::StringRef, int, char**, int&)"},
    {"_ZNK4llvm2cl5alias14getOptionWidthEv", "llvm::cl::alias::getOptionWidth() const"},
    {"_ZNK4llvm2cl5alias15printOptionInfoEm", "llvm::cl::alias::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl6parserINS0_13boolOrDefaultEE5parseERNS0_6OptionENS_9StringRefES6_RS2_", "llvm::cl::parser<llvm::cl::boolOrDefault>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, llvm::cl::boolOrDefault&)"},
    {"_ZN4llvm2cl6parserIdE5parseERNS0_6OptionENS_9StringRefES5_Rd", "llvm::cl::parser<double>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, double&)"},
    {"_Z11parseDoubleRN4llvm2cl6OptionENS_9StringRefERd", "parseDouble(llvm::cl::Option&, llvm::StringRef, double&)"},
    {"_ZN4llvm2cl6parserIfE5parseERNS0_6OptionENS_9StringRefES5_Rf", "llvm::cl::parser<float>::parse(llvm::cl::Option&, llvm::StringRef, llvm::StringRef, float&)"},
    {"_ZN4llvm2cl19generic_parser_base10findOptionEPKc", "llvm::cl::generic_parser_base::findOption(char const*)"},
    {"_ZNK4llvm2cl19generic_parser_base14getOptionWidthERKNS0_6OptionE", "llvm::cl::generic_parser_base::getOptionWidth(llvm::cl::Option const&) const"},
    {"_ZNK4llvm2cl19generic_parser_base15printOptionInfoERKNS0_6OptionEm", "llvm::cl::generic_parser_base::printOptionInfo(llvm::cl::Option const&, unsigned long) const"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEED1Ev", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEED1Ev", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl16PrintHelpMessageEv", "llvm::cl::PrintHelpMessage()"},
    {"_ZN12_GLOBAL__N_111HelpPrinteraSEb", "(anonymous namespace)::HelpPrinter::operator=(bool)"},
    {"_ZN4llvm2cl19PrintVersionMessageEv", "llvm::cl::PrintVersionMessage()"},
    {"_ZN12_GLOBAL__N_114VersionPrinter5printEv", "(anonymous namespace)::VersionPrinter::print()"},
    {"_ZN4llvm2cl17SetVersionPrinterEPFvvE", "llvm::cl::SetVersionPrinter(void (*)())"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEE14getOptionWidthEv", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEE15printOptionInfoEm", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEED0Ev", "llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEE16handleOccurrenceEjNS_9StringRefES7_", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEE14getOptionWidthEv", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEE15printOptionInfoEm", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEED0Ev", "llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >::~opt()"},
    {"_Z17TargetArraySortFnPKvS0_", "TargetArraySortFn(void const*, void const*)"},
    {"_Z14OptNameComparePKvS0_", "OptNameCompare(void const*, void const*)"},
    {"_Z28CommaSeparateAndAddOccurencePN4llvm2cl6OptionEjNS_9StringRefES3_b", "CommaSeparateAndAddOccurence(llvm::cl::Option*, unsigned int, llvm::StringRef, llvm::StringRef, bool)"},
    {"_Z13getOptionPredN4llvm9StringRefERmPFbPKNS_2cl6OptionEERKNS_9StringMapIPS3_NS_15MallocAllocatorEEE", "getOptionPred(llvm::StringRef, unsigned long&, bool (*)(llvm::cl::Option const*), llvm::StringMap<llvm::cl::Option*, llvm::MallocAllocator> const&)"},
    {"_Z20isPrefixedOrGroupingPKN4llvm2cl6OptionE", "isPrefixedOrGrouping(llvm::cl::Option const*)"},
    {"_Z10isGroupingPKN4llvm2cl6OptionE", "isGrouping(llvm::cl::Option const*)"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIjLb0ENS0_6parserIjEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZNK4llvm2cl3optIjLb0ENS0_6parserIjEEE14getOptionWidthEv", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIjLb0ENS0_6parserIjEEE15printOptionInfoEm", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEE4doneEv", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::done()"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEE15setInitialValueERKj", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::setInitialValue(unsigned int const&)"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEE9getParserEv", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::getParser()"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIiLb0ENS0_6parserIiEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZNK4llvm2cl3optIiLb0ENS0_6parserIiEEE14getOptionWidthEv", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIiLb0ENS0_6parserIiEEE15printOptionInfoEm", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEE4doneEv", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::done()"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEE15setInitialValueERKi", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::setInitialValue(int const&)"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEE9getParserEv", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::getParser()"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optISsLb0ENS0_6parserISsEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZNK4llvm2cl3optISsLb0ENS0_6parserISsEEE14getOptionWidthEv", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optISsLb0ENS0_6parserISsEEE15printOptionInfoEm", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEE4doneEv", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::done()"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEE15setInitialValueERKSs", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::setInitialValue(std::string const&)"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEE9getParserEv", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::getParser()"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIcLb0ENS0_6parserIcEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZNK4llvm2cl3optIcLb0ENS0_6parserIcEEE14getOptionWidthEv", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIcLb0ENS0_6parserIcEEE15printOptionInfoEm", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEE4doneEv", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::done()"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEE15setInitialValueERKc", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::setInitialValue(char const&)"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEE9getParserEv", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::getParser()"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optIbLb0ENS0_6parserIbEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZNK4llvm2cl3optIbLb0ENS0_6parserIbEEE14getOptionWidthEv", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optIbLb0ENS0_6parserIbEEE15printOptionInfoEm", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEE4doneEv", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::done()"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEE15setInitialValueERKb", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::setInitialValue(bool const&)"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEE9getParserEv", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::getParser()"},
    {"_ZN4llvm2cl19generic_parser_baseD1Ev", "llvm::cl::generic_parser_base::~generic_parser_base()"},
    {"_ZN4llvm2cl19generic_parser_baseD0Ev", "llvm::cl::generic_parser_base::~generic_parser_base()"},
    {"_ZN4llvm2cl12basic_parserIbED1Ev", "llvm::cl::basic_parser<bool>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIbED0Ev", "llvm::cl::basic_parser<bool>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserINS0_13boolOrDefaultEED1Ev", "llvm::cl::basic_parser<llvm::cl::boolOrDefault>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserINS0_13boolOrDefaultEED0Ev", "llvm::cl::basic_parser<llvm::cl::boolOrDefault>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIiED1Ev", "llvm::cl::basic_parser<int>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIiED0Ev", "llvm::cl::basic_parser<int>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIjED1Ev", "llvm::cl::basic_parser<unsigned int>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIjED0Ev", "llvm::cl::basic_parser<unsigned int>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIdED1Ev", "llvm::cl::basic_parser<double>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIdED0Ev", "llvm::cl::basic_parser<double>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIfED1Ev", "llvm::cl::basic_parser<float>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIfED0Ev", "llvm::cl::basic_parser<float>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserISsED1Ev", "llvm::cl::basic_parser<std::string>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserISsED0Ev", "llvm::cl::basic_parser<std::string>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIcED1Ev", "llvm::cl::basic_parser<char>::~basic_parser()"},
    {"_ZN4llvm2cl12basic_parserIcED0Ev", "llvm::cl::basic_parser<char>::~basic_parser()"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEED1Ev", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::~opt()"},
    {"_ZN4llvm2cl3optIjLb0ENS0_6parserIjEEED0Ev", "llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >::~opt()"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEED1Ev", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::~opt()"},
    {"_ZN4llvm2cl3optIiLb0ENS0_6parserIiEEED0Ev", "llvm::cl::opt<int, false, llvm::cl::parser<int> >::~opt()"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEED1Ev", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::~opt()"},
    {"_ZN4llvm2cl3optISsLb0ENS0_6parserISsEEED0Ev", "llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >::~opt()"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEED1Ev", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::~opt()"},
    {"_ZN4llvm2cl3optIcLb0ENS0_6parserIcEEED0Ev", "llvm::cl::opt<char, false, llvm::cl::parser<char> >::~opt()"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEED0Ev", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl6parserINS0_13boolOrDefaultEED1Ev", "llvm::cl::parser<llvm::cl::boolOrDefault>::~parser()"},
    {"_ZN4llvm2cl6parserINS0_13boolOrDefaultEED0Ev", "llvm::cl::parser<llvm::cl::boolOrDefault>::~parser()"},
    {"_ZNK4llvm2cl6parserINS0_13boolOrDefaultEE12getValueNameEv", "llvm::cl::parser<llvm::cl::boolOrDefault>::getValueName() const"},
    {"_ZN4llvm2cl6parserIiED1Ev", "llvm::cl::parser<int>::~parser()"},
    {"_ZN4llvm2cl6parserIiED0Ev", "llvm::cl::parser<int>::~parser()"},
    {"_ZNK4llvm2cl6parserIiE12getValueNameEv", "llvm::cl::parser<int>::getValueName() const"},
    {"_ZN4llvm2cl6parserIjED1Ev", "llvm::cl::parser<unsigned int>::~parser()"},
    {"_ZN4llvm2cl6parserIjED0Ev", "llvm::cl::parser<unsigned int>::~parser()"},
    {"_ZNK4llvm2cl6parserIjE12getValueNameEv", "llvm::cl::parser<unsigned int>::getValueName() const"},
    {"_ZN4llvm2cl6parserIdED1Ev", "llvm::cl::parser<double>::~parser()"},
    {"_ZN4llvm2cl6parserIdED0Ev", "llvm::cl::parser<double>::~parser()"},
    {"_ZNK4llvm2cl6parserIdE12getValueNameEv", "llvm::cl::parser<double>::getValueName() const"},
    {"_ZN4llvm2cl6parserIfED1Ev", "llvm::cl::parser<float>::~parser()"},
    {"_ZN4llvm2cl6parserIfED0Ev", "llvm::cl::parser<float>::~parser()"},
    {"_ZNK4llvm2cl6parserIfE12getValueNameEv", "llvm::cl::parser<float>::getValueName() const"},
    {"_ZN4llvm2cl6parserIcED1Ev", "llvm::cl::parser<char>::~parser()"},
    {"_ZN4llvm2cl6parserIcED0Ev", "llvm::cl::parser<char>::~parser()"},
    {"_ZNK4llvm2cl6parserIcE12getValueNameEv", "llvm::cl::parser<char>::getValueName() const"},
    {"_ZN4llvm2cl5alias16handleOccurrenceEjNS_9StringRefES2_", "llvm::cl::alias::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm2cl5aliasD1Ev", "llvm::cl::alias::~alias()"},
    {"_ZN4llvm2cl5aliasD0Ev", "llvm::cl::alias::~alias()"},
    {"_ZNSt6vectorIPcSaIS0_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS0_S2_EERKS0_", "std::vector<char*, std::allocator<char*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<char**, std::vector<char*, std::allocator<char*> > >, char* const&)"},
    {"_ZN4llvm14object_creatorISt6vectorIPKcSaIS3_EEEEPvv", "void* llvm::object_creator<std::vector<char const*, std::allocator<char const*> > >()"},
    {"_ZN4llvm14object_deleterISt6vectorIPKcSaIS3_EEE4callEPv", "llvm::object_deleter<std::vector<char const*, std::allocator<char const*> > >::call(void*)"},
    {"_ZNSt6vectorISt4pairIPKcPKN4llvm6TargetEESaIS7_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS7_S9_EERKS7_", "std::vector<std::pair<char const*, llvm::Target const*>, std::allocator<std::pair<char const*, llvm::Target const*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<char const*, llvm::Target const*>*, std::vector<std::pair<char const*, llvm::Target const*>, std::allocator<std::pair<char const*, llvm::Target const*> > > >, std::pair<char const*, llvm::Target const*> const&)"},
    {"_ZN4llvm9StringMapIPNS_2cl6OptionENS_15MallocAllocatorEE16GetOrCreateValueIS3_EERNS_14StringMapEntryIS3_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::cl::Option*>& llvm::StringMap<llvm::cl::Option*, llvm::MallocAllocator>::GetOrCreateValue<llvm::cl::Option*>(llvm::StringRef, llvm::cl::Option*)"},
    {"_ZN4llvm13ConstantRangeC1Ejb", "llvm::ConstantRange::ConstantRange(unsigned int, bool)"},
    {"_ZN4llvm13ConstantRangeC2Ejb", "llvm::ConstantRange::ConstantRange(unsigned int, bool)"},
    {"_ZN4llvm13ConstantRangeC1ERKNS_5APIntE", "llvm::ConstantRange::ConstantRange(llvm::APInt const&)"},
    {"_ZN4llvm13ConstantRangeC2ERKNS_5APIntE", "llvm::ConstantRange::ConstantRange(llvm::APInt const&)"},
    {"_ZN4llvm13ConstantRangeC1ERKNS_5APIntES3_", "llvm::ConstantRange::ConstantRange(llvm::APInt const&, llvm::APInt const&)"},
    {"_ZN4llvm13ConstantRangeC2ERKNS_5APIntES3_", "llvm::ConstantRange::ConstantRange(llvm::APInt const&, llvm::APInt const&)"},
    {"_ZN4llvm13ConstantRange14makeICmpRegionEjRKS0_", "llvm::ConstantRange::makeICmpRegion(unsigned int, llvm::ConstantRange const&)"},
    {"_ZNK4llvm13ConstantRange10isEmptySetEv", "llvm::ConstantRange::isEmptySet() const"},
    {"_ZNK4llvm13ConstantRange14getUnsignedMaxEv", "llvm::ConstantRange::getUnsignedMax() const"},
    {"_ZNK4llvm13ConstantRange12getSignedMaxEv", "llvm::ConstantRange::getSignedMax() const"},
    {"_ZNK4llvm13ConstantRange14getUnsignedMinEv", "llvm::ConstantRange::getUnsignedMin() const"},
    {"_ZNK4llvm13ConstantRange12getSignedMinEv", "llvm::ConstantRange::getSignedMin() const"},
    {"_ZNK4llvm13ConstantRange9isFullSetEv", "llvm::ConstantRange::isFullSet() const"},
    {"_ZNK4llvm13ConstantRange12isWrappedSetEv", "llvm::ConstantRange::isWrappedSet() const"},
    {"_ZNK4llvm13ConstantRange16isSignWrappedSetEv", "llvm::ConstantRange::isSignWrappedSet() const"},
    {"_ZNK4llvm13ConstantRange8containsERKNS_5APIntE", "llvm::ConstantRange::contains(llvm::APInt const&) const"},
    {"_ZNK4llvm13ConstantRange10getSetSizeEv", "llvm::ConstantRange::getSetSize() const"},
    {"_ZNK4llvm13ConstantRange8containsERKS0_", "llvm::ConstantRange::contains(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange8subtractERKNS_5APIntE", "llvm::ConstantRange::subtract(llvm::APInt const&) const"},
    {"_ZNK4llvm13ConstantRange13intersectWithERKS0_", "llvm::ConstantRange::intersectWith(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange9unionWithERKS0_", "llvm::ConstantRange::unionWith(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange10zeroExtendEj", "llvm::ConstantRange::zeroExtend(unsigned int) const"},
    {"_ZNK4llvm13ConstantRange10signExtendEj", "llvm::ConstantRange::signExtend(unsigned int) const"},
    {"_ZNK4llvm13ConstantRange8truncateEj", "llvm::ConstantRange::truncate(unsigned int) const"},
    {"_ZNK4llvm13ConstantRange11zextOrTruncEj", "llvm::ConstantRange::zextOrTrunc(unsigned int) const"},
    {"_ZNK4llvm13ConstantRange11sextOrTruncEj", "llvm::ConstantRange::sextOrTrunc(unsigned int) const"},
    {"_ZNK4llvm13ConstantRange3addERKS0_", "llvm::ConstantRange::add(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange3subERKS0_", "llvm::ConstantRange::sub(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange8multiplyERKS0_", "llvm::ConstantRange::multiply(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange4smaxERKS0_", "llvm::ConstantRange::smax(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange4umaxERKS0_", "llvm::ConstantRange::umax(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange4udivERKS0_", "llvm::ConstantRange::udiv(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange9binaryAndERKS0_", "llvm::ConstantRange::binaryAnd(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange8binaryOrERKS0_", "llvm::ConstantRange::binaryOr(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange3shlERKS0_", "llvm::ConstantRange::shl(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange4lshrERKS0_", "llvm::ConstantRange::lshr(llvm::ConstantRange const&) const"},
    {"_ZNK4llvm13ConstantRange7inverseEv", "llvm::ConstantRange::inverse() const"},
    {"_ZNK4llvm13ConstantRange5printERNS_11raw_ostreamE", "llvm::ConstantRange::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm13ConstantRange4dumpEv", "llvm::ConstantRange::dump() const"},
    {"_ZN4llvm3sys11ThreadLocalIKN12_GLOBAL__N_124CrashRecoveryContextImplEED1Ev", "llvm::sys::ThreadLocal<(anonymous namespace)::CrashRecoveryContextImpl const>::~ThreadLocal()"},
    {"_ZN4llvm20CrashRecoveryContextD1Ev", "llvm::CrashRecoveryContext::~CrashRecoveryContext()"},
    {"_ZN4llvm20CrashRecoveryContextD2Ev", "llvm::CrashRecoveryContext::~CrashRecoveryContext()"},
    {"_ZN4llvm20CrashRecoveryContext10GetCurrentEv", "llvm::CrashRecoveryContext::GetCurrent()"},
    {"_ZN4llvm20CrashRecoveryContext6EnableEv", "llvm::CrashRecoveryContext::Enable()"},
    {"_Z26CrashRecoverySignalHandleri", "CrashRecoverySignalHandler(int)"},
    {"_ZN4llvm20CrashRecoveryContext7DisableEv", "llvm::CrashRecoveryContext::Disable()"},
    {"_ZN4llvm20CrashRecoveryContext9RunSafelyEPFvPvES1_", "llvm::CrashRecoveryContext::RunSafely(void (*)(void*), void*)"},
    {"_ZN4llvm20CrashRecoveryContext11HandleCrashEv", "llvm::CrashRecoveryContext::HandleCrash()"},
    {"_ZNK4llvm20CrashRecoveryContext12getBacktraceEv", "llvm::CrashRecoveryContext::getBacktrace() const"},
    {"_ZN4llvm20CrashRecoveryContext17RunSafelyOnThreadEPFvPvES1_j", "llvm::CrashRecoveryContext::RunSafelyOnThread(void (*)(void*), void*, unsigned int)"},
    {"_Z26RunSafelyOnThread_DispatchPv", "RunSafelyOnThread_Dispatch(void*)"},
    {"_ZN4llvm3sys11ThreadLocalIKN12_GLOBAL__N_124CrashRecoveryContextImplEED0Ev", "llvm::sys::ThreadLocal<(anonymous namespace)::CrashRecoveryContextImpl const>::~ThreadLocal()"},
    {"_ZN4llvm3sys10SmartMutexILb0EED1Ev", "llvm::sys::SmartMutex<false>::~SmartMutex()"},
    {"_ZN4llvm4dbgsEv", "llvm::dbgs()"},
    {"_ZN4llvm5dwarf9TagStringEj", "llvm::dwarf::TagString(unsigned int)"},
    {"_ZN4llvm5dwarf14ChildrenStringEj", "llvm::dwarf::ChildrenString(unsigned int)"},
    {"_ZN4llvm5dwarf15AttributeStringEj", "llvm::dwarf::AttributeString(unsigned int)"},
    {"_ZN4llvm5dwarf18FormEncodingStringEj", "llvm::dwarf::FormEncodingString(unsigned int)"},
    {"_ZN4llvm5dwarf23OperationEncodingStringEj", "llvm::dwarf::OperationEncodingString(unsigned int)"},
    {"_ZN4llvm5dwarf23AttributeEncodingStringEj", "llvm::dwarf::AttributeEncodingString(unsigned int)"},
    {"_ZN4llvm5dwarf17DecimalSignStringEj", "llvm::dwarf::DecimalSignString(unsigned int)"},
    {"_ZN4llvm5dwarf15EndianityStringEj", "llvm::dwarf::EndianityString(unsigned int)"},
    {"_ZN4llvm5dwarf19AccessibilityStringEj", "llvm::dwarf::AccessibilityString(unsigned int)"},
    {"_ZN4llvm5dwarf16VisibilityStringEj", "llvm::dwarf::VisibilityString(unsigned int)"},
    {"_ZN4llvm5dwarf16VirtualityStringEj", "llvm::dwarf::VirtualityString(unsigned int)"},
    {"_ZN4llvm5dwarf14LanguageStringEj", "llvm::dwarf::LanguageString(unsigned int)"},
    {"_ZN4llvm5dwarf10CaseStringEj", "llvm::dwarf::CaseString(unsigned int)"},
    {"_ZN4llvm5dwarf16ConventionStringEj", "llvm::dwarf::ConventionString(unsigned int)"},
    {"_ZN4llvm5dwarf16InlineCodeStringEj", "llvm::dwarf::InlineCodeString(unsigned int)"},
    {"_ZN4llvm5dwarf16ArrayOrderStringEj", "llvm::dwarf::ArrayOrderString(unsigned int)"},
    {"_ZN4llvm5dwarf18DiscriminantStringEj", "llvm::dwarf::DiscriminantString(unsigned int)"},
    {"_ZN4llvm5dwarf16LNStandardStringEj", "llvm::dwarf::LNStandardString(unsigned int)"},
    {"_ZN4llvm5dwarf16LNExtendedStringEj", "llvm::dwarf::LNExtendedString(unsigned int)"},
    {"_ZN4llvm5dwarf13MacinfoStringEj", "llvm::dwarf::MacinfoString(unsigned int)"},
    {"_ZN4llvm5dwarf15CallFrameStringEj", "llvm::dwarf::CallFrameString(unsigned int)"},
    {"_ZN12_GLOBAL__N_122ExplicitSymbolsDeleterD1Ev", "(anonymous namespace)::ExplicitSymbolsDeleter::~ExplicitSymbolsDeleter()"},
    {"_ZN4llvm3sys14DynamicLibrary9AddSymbolEPKcPv", "llvm::sys::DynamicLibrary::AddSymbol(char const*, void*)"},
    {"_ZN4llvm3sys14DynamicLibrary22LoadLibraryPermanentlyEPKcPSs", "llvm::sys::DynamicLibrary::LoadLibraryPermanently(char const*, std::string*)"},
    {"_ZN4llvm3sys14DynamicLibrary24SearchForAddressOfSymbolEPKc", "llvm::sys::DynamicLibrary::SearchForAddressOfSymbol(char const*)"},
    {"_ZNSt3mapISsPvSt4lessISsESaISt4pairIKSsS0_EEEixERS4_", "std::map<std::string, void*, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::operator[](std::string const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPvESt10_Select1stIS3_ESt4lessISsESaIS3_EE4findERS1_", "std::_Rb_tree<std::string, std::pair<std::string const, void*>, std::_Select1st<std::pair<std::string const, void*> >, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::find(std::string const&)"},
    {"_ZNSt6vectorIPvSaIS0_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS0_S2_EERKS0_", "std::vector<void*, std::allocator<void*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<void**, std::vector<void*, std::allocator<void*> > >, void* const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPvESt10_Select1stIS3_ESt4lessISsESaIS3_EE16_M_insert_uniqueESt17_Rb_tree_iteratorIS3_ERKS3_", "std::_Rb_tree<std::string, std::pair<std::string const, void*>, std::_Select1st<std::pair<std::string const, void*> >, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::_M_insert_unique(std::_Rb_tree_iterator<std::pair<std::string const, void*> >, std::pair<std::string const, void*> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPvESt10_Select1stIS3_ESt4lessISsESaIS3_EE9_M_insertEPSt18_Rb_tree_node_baseSB_RKS3_", "std::_Rb_tree<std::string, std::pair<std::string const, void*>, std::_Select1st<std::pair<std::string const, void*> >, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::pair<std::string const, void*> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPvESt10_Select1stIS3_ESt4lessISsESaIS3_EE16_M_insert_uniqueERKS3_", "std::_Rb_tree<std::string, std::pair<std::string const, void*>, std::_Select1st<std::pair<std::string const, void*> >, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::_M_insert_unique(std::pair<std::string const, void*> const&)"},
    {"_ZNSt8_Rb_treeISsSt4pairIKSsPvESt10_Select1stIS3_ESt4lessISsESaIS3_EE8_M_eraseEPSt13_Rb_tree_nodeIS3_E", "std::_Rb_tree<std::string, std::pair<std::string const, void*>, std::_Select1st<std::pair<std::string const, void*> >, std::less<std::string>, std::allocator<std::pair<std::string const, void*> > >::_M_erase(std::_Rb_tree_node<std::pair<std::string const, void*> >*)"},
    {"_ZN4llvm3sys8StrErrorEv", "llvm::sys::StrError()"},
    {"_ZN4llvm3sys8StrErrorEi", "llvm::sys::StrError(int)"},
    {"_ZN4llvm27install_fatal_error_handlerEPFvPvRKSsES0_", "llvm::install_fatal_error_handler(void (*)(void*, std::string const&), void*)"},
    {"_ZN4llvm26remove_fatal_error_handlerEv", "llvm::remove_fatal_error_handler()"},
    {"_ZN4llvm18report_fatal_errorEPKc", "llvm::report_fatal_error(char const*)"},
    {"_ZN4llvm18report_fatal_errorERKNS_5TwineE", "llvm::report_fatal_error(llvm::Twine const&)"},
    {"_ZN4llvm18report_fatal_errorERKSs", "llvm::report_fatal_error(std::string const&)"},
    {"_ZN4llvm18report_fatal_errorENS_9StringRefE", "llvm::report_fatal_error(llvm::StringRef)"},
    {"_ZN4llvm25llvm_unreachable_internalEPKcS1_j", "llvm::llvm_unreachable_internal(char const*, char const*, unsigned int)"},
    {"_ZNK4llvm19FoldingSetNodeIDRef11ComputeHashEv", "llvm::FoldingSetNodeIDRef::ComputeHash() const"},
    {"_ZNK4llvm19FoldingSetNodeIDRefeqES0_", "llvm::FoldingSetNodeIDRef::operator==(llvm::FoldingSetNodeIDRef) const"},
    {"_ZN4llvm16FoldingSetNodeID10AddPointerEPKv", "llvm::FoldingSetNodeID::AddPointer(void const*)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEi", "llvm::FoldingSetNodeID::AddInteger(int)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEj", "llvm::FoldingSetNodeID::AddInteger(unsigned int)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEl", "llvm::FoldingSetNodeID::AddInteger(long)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEm", "llvm::FoldingSetNodeID::AddInteger(unsigned long)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEy", "llvm::FoldingSetNodeID::AddInteger(unsigned long long)"},
    {"_ZN4llvm16FoldingSetNodeID10AddIntegerEx", "llvm::FoldingSetNodeID::AddInteger(long long)"},
    {"_ZN4llvm16FoldingSetNodeID9AddStringENS_9StringRefE", "llvm::FoldingSetNodeID::AddString(llvm::StringRef)"},
    {"_ZNK4llvm16FoldingSetNodeID11ComputeHashEv", "llvm::FoldingSetNodeID::ComputeHash() const"},
    {"_ZNK4llvm16FoldingSetNodeIDeqERKS0_", "llvm::FoldingSetNodeID::operator==(llvm::FoldingSetNodeID const&) const"},
    {"_ZNK4llvm16FoldingSetNodeIDeqENS_19FoldingSetNodeIDRefE", "llvm::FoldingSetNodeID::operator==(llvm::FoldingSetNodeIDRef) const"},
    {"_ZNK4llvm16FoldingSetNodeID6InternERNS_16BumpPtrAllocatorE", "llvm::FoldingSetNodeID::Intern(llvm::BumpPtrAllocator&) const"},
    {"_ZN4llvm14FoldingSetImplC1Ej", "llvm::FoldingSetImpl::FoldingSetImpl(unsigned int)"},
    {"_ZN4llvm14FoldingSetImplC2Ej", "llvm::FoldingSetImpl::FoldingSetImpl(unsigned int)"},
    {"_ZN4llvm14FoldingSetImplD0Ev", "llvm::FoldingSetImpl::~FoldingSetImpl()"},
    {"_ZN4llvm14FoldingSetImplD1Ev", "llvm::FoldingSetImpl::~FoldingSetImpl()"},
    {"_ZN4llvm14FoldingSetImplD2Ev", "llvm::FoldingSetImpl::~FoldingSetImpl()"},
    {"_ZN4llvm14FoldingSetImpl5clearEv", "llvm::FoldingSetImpl::clear()"},
    {"_ZN4llvm14FoldingSetImpl13GrowHashTableEv", "llvm::FoldingSetImpl::GrowHashTable()"},
    {"_ZN4llvm14FoldingSetImpl10InsertNodeEPNS0_4NodeEPv", "llvm::FoldingSetImpl::InsertNode(llvm::FoldingSetImpl::Node*, void*)"},
    {"_ZN4llvm14FoldingSetImpl19FindNodeOrInsertPosERKNS_16FoldingSetNodeIDERPv", "llvm::FoldingSetImpl::FindNodeOrInsertPos(llvm::FoldingSetNodeID const&, void*&)"},
    {"_ZN4llvm14FoldingSetImpl10RemoveNodeEPNS0_4NodeE", "llvm::FoldingSetImpl::RemoveNode(llvm::FoldingSetImpl::Node*)"},
    {"_ZN4llvm14FoldingSetImpl15GetOrInsertNodeEPNS0_4NodeE", "llvm::FoldingSetImpl::GetOrInsertNode(llvm::FoldingSetImpl::Node*)"},
    {"_ZN4llvm22FoldingSetIteratorImplC1EPPv", "llvm::FoldingSetIteratorImpl::FoldingSetIteratorImpl(void**)"},
    {"_ZN4llvm22FoldingSetIteratorImplC2EPPv", "llvm::FoldingSetIteratorImpl::FoldingSetIteratorImpl(void**)"},
    {"_ZN4llvm22FoldingSetIteratorImpl7advanceEv", "llvm::FoldingSetIteratorImpl::advance()"},
    {"_ZN4llvm28FoldingSetBucketIteratorImplC1EPPv", "llvm::FoldingSetBucketIteratorImpl::FoldingSetBucketIteratorImpl(void**)"},
    {"_ZN4llvm28FoldingSetBucketIteratorImplC2EPPv", "llvm::FoldingSetBucketIteratorImpl::FoldingSetBucketIteratorImpl(void**)"},
    {"_ZN4llvm21formatted_raw_ostream13ComputeColumnEPKcm", "llvm::formatted_raw_ostream::ComputeColumn(char const*, unsigned long)"},
    {"_ZN4llvm21formatted_raw_ostream11PadToColumnEj", "llvm::formatted_raw_ostream::PadToColumn(unsigned int)"},
    {"_ZN4llvm21formatted_raw_ostream10write_implEPKcm", "llvm::formatted_raw_ostream::write_impl(char const*, unsigned long)"},
    {"_ZN4llvm5foutsEv", "llvm::fouts()"},
    {"_ZN4llvm5ferrsEv", "llvm::ferrs()"},
    {"_ZN4llvm5fdbgsEv", "llvm::fdbgs()"},
    {"_ZN4llvm3DOT12EscapeStringERKSs", "llvm::DOT::EscapeString(std::string const&)"},
    {"_ZN4llvm12DisplayGraphERKNS_3sys4PathEbNS_12GraphProgram4NameE", "llvm::DisplayGraph(llvm::sys::Path const&, bool, llvm::GraphProgram::Name)"},
    {"_ZN4llvm3sys13getHostTripleEv", "llvm::sys::getHostTriple()"},
    {"_ZN4llvm3sys14getHostCPUNameEv", "llvm::sys::getHostCPUName()"},
    {"_ZN4llvm3sys18getHostCPUFeaturesERNS_9StringMapIbNS_15MallocAllocatorEEE", "llvm::sys::getHostCPUFeatures(llvm::StringMap<bool, llvm::MallocAllocator>&)"},
    {"_ZN4llvm12IntEqClasses4growEj", "llvm::IntEqClasses::grow(unsigned int)"},
    {"_ZN4llvm12IntEqClasses4joinEjj", "llvm::IntEqClasses::join(unsigned int, unsigned int)"},
    {"_ZNK4llvm12IntEqClasses10findLeaderEj", "llvm::IntEqClasses::findLeader(unsigned int) const"},
    {"_ZN4llvm12IntEqClasses8compressEv", "llvm::IntEqClasses::compress()"},
    {"_ZN4llvm12IntEqClasses10uncompressEv", "llvm::IntEqClasses::uncompress()"},
    {"_ZN4llvm15IntervalMapImpl4Path11replaceRootEPvjSt4pairIjjE", "llvm::IntervalMapImpl::Path::replaceRoot(void*, unsigned int, std::pair<unsigned int, unsigned int>)"},
    {"_ZNK4llvm15IntervalMapImpl4Path14getLeftSiblingEj", "llvm::IntervalMapImpl::Path::getLeftSibling(unsigned int) const"},
    {"_ZN4llvm15IntervalMapImpl4Path8moveLeftEj", "llvm::IntervalMapImpl::Path::moveLeft(unsigned int)"},
    {"_ZNK4llvm15IntervalMapImpl4Path15getRightSiblingEj", "llvm::IntervalMapImpl::Path::getRightSibling(unsigned int) const"},
    {"_ZN4llvm15IntervalMapImpl4Path9moveRightEj", "llvm::IntervalMapImpl::Path::moveRight(unsigned int)"},
    {"_ZN4llvm15IntervalMapImpl10distributeEjjjPKjPjjb", "llvm::IntervalMapImpl::distribute(unsigned int, unsigned int, unsigned int, unsigned int const*, unsigned int*, unsigned int, bool)"},
    {"_ZN4llvm15SmallVectorImplINS_15IntervalMapImpl4Path5EntryEE6insertEPS3_RKS3_", "llvm::SmallVectorImpl<llvm::IntervalMapImpl::Path::Entry>::insert(llvm::IntervalMapImpl::Path::Entry*, llvm::IntervalMapImpl::Path::Entry const&)"},
    {"_ZN4llvm15SmallVectorImplINS_15IntervalMapImpl4Path5EntryEE6resizeEjRKS3_", "llvm::SmallVectorImpl<llvm::IntervalMapImpl::Path::Entry>::resize(unsigned int, llvm::IntervalMapImpl::Path::Entry const&)"},
    {"_ZNK4llvm17ManagedStaticBase7destroyEv", "llvm::ManagedStaticBase::destroy() const"},
    {"_ZN4llvm13llvm_shutdownEv", "llvm::llvm_shutdown()"},
    {"_ZN4llvm3sys6Memory11AllocateRWXEmPKNS0_11MemoryBlockEPSs", "llvm::sys::Memory::AllocateRWX(unsigned long, llvm::sys::MemoryBlock const*, std::string*)"},
    {"_Z10MakeErrMsgPSsRKSsi", "MakeErrMsg(std::string*, std::string const&, int)"},
    {"_ZN4llvm3sys6Memory10ReleaseRWXERNS0_11MemoryBlockEPSs", "llvm::sys::Memory::ReleaseRWX(llvm::sys::MemoryBlock&, std::string*)"},
    {"_ZN4llvm3sys6Memory11setWritableERNS0_11MemoryBlockEPSs", "llvm::sys::Memory::setWritable(llvm::sys::MemoryBlock&, std::string*)"},
    {"_ZN4llvm3sys6Memory13setExecutableERNS0_11MemoryBlockEPSs", "llvm::sys::Memory::setExecutable(llvm::sys::MemoryBlock&, std::string*)"},
    {"_ZN4llvm3sys6Memory16setRangeWritableEPKvm", "llvm::sys::Memory::setRangeWritable(void const*, unsigned long)"},
    {"_ZN4llvm3sys6Memory18setRangeExecutableEPKvm", "llvm::sys::Memory::setRangeExecutable(void const*, unsigned long)"},
    {"_ZN4llvm3sys6Memory26InvalidateInstructionCacheEPKvm", "llvm::sys::Memory::InvalidateInstructionCache(void const*, unsigned long)"},
    {"_ZN4llvm12MemoryBufferD0Ev", "llvm::MemoryBuffer::~MemoryBuffer()"},
    {"_ZN4llvm12MemoryBufferD1Ev", "llvm::MemoryBuffer::~MemoryBuffer()"},
    {"_ZN4llvm12MemoryBufferD2Ev", "llvm::MemoryBuffer::~MemoryBuffer()"},
    {"_ZN4llvm12MemoryBuffer4initEPKcS2_", "llvm::MemoryBuffer::init(char const*, char const*)"},
    {"_ZN4llvm12MemoryBuffer12getMemBufferENS_9StringRefES1_", "llvm::MemoryBuffer::getMemBuffer(llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm12MemoryBuffer16getMemBufferCopyENS_9StringRefES1_", "llvm::MemoryBuffer::getMemBufferCopy(llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm12MemoryBuffer21getNewUninitMemBufferEmNS_9StringRefE", "llvm::MemoryBuffer::getNewUninitMemBuffer(unsigned long, llvm::StringRef)"},
    {"_ZN4llvm12MemoryBuffer15getNewMemBufferEmNS_9StringRefE", "llvm::MemoryBuffer::getNewMemBuffer(unsigned long, llvm::StringRef)"},
    {"_ZN4llvm12MemoryBuffer14getFileOrSTDINENS_9StringRefERNS_9OwningPtrIS0_EEx", "llvm::MemoryBuffer::getFileOrSTDIN(llvm::StringRef, llvm::OwningPtr<llvm::MemoryBuffer>&, long long)"},
    {"_ZN4llvm12MemoryBuffer8getSTDINERNS_9OwningPtrIS0_EE", "llvm::MemoryBuffer::getSTDIN(llvm::OwningPtr<llvm::MemoryBuffer>&)"},
    {"_ZN4llvm12MemoryBuffer7getFileENS_9StringRefERNS_9OwningPtrIS0_EEx", "llvm::MemoryBuffer::getFile(llvm::StringRef, llvm::OwningPtr<llvm::MemoryBuffer>&, long long)"},
    {"_ZN4llvm12MemoryBuffer14getFileOrSTDINEPKcRNS_9OwningPtrIS0_EEx", "llvm::MemoryBuffer::getFileOrSTDIN(char const*, llvm::OwningPtr<llvm::MemoryBuffer>&, long long)"},
    {"_ZN4llvm12MemoryBuffer7getFileEPKcRNS_9OwningPtrIS0_EEx", "llvm::MemoryBuffer::getFile(char const*, llvm::OwningPtr<llvm::MemoryBuffer>&, long long)"},
    {"_ZN4llvm12MemoryBuffer11getOpenFileEiPKcRNS_9OwningPtrIS0_EEx", "llvm::MemoryBuffer::getOpenFile(int, char const*, llvm::OwningPtr<llvm::MemoryBuffer>&, long long)"},
    {"_ZN12_GLOBAL__N_120MemoryBufferMMapFileD1Ev", "(anonymous namespace)::MemoryBufferMMapFile::~MemoryBufferMMapFile()"},
    {"_ZN12_GLOBAL__N_120MemoryBufferMMapFileD0Ev", "(anonymous namespace)::MemoryBufferMMapFile::~MemoryBufferMMapFile()"},
    {"_ZNK12_GLOBAL__N_115MemoryBufferMem19getBufferIdentifierEv", "(anonymous namespace)::MemoryBufferMem::getBufferIdentifier() const"},
    {"_ZN12_GLOBAL__N_115MemoryBufferMemD1Ev", "(anonymous namespace)::MemoryBufferMem::~MemoryBufferMem()"},
    {"_ZN12_GLOBAL__N_115MemoryBufferMemD0Ev", "(anonymous namespace)::MemoryBufferMem::~MemoryBufferMem()"},
    {"_ZNK4llvm12MemoryBuffer19getBufferIdentifierEv", "llvm::MemoryBuffer::getBufferIdentifier() const"},
    {"_ZN4llvm3sys9MutexImplC1Eb", "llvm::sys::MutexImpl::MutexImpl(bool)"},
    {"_ZN4llvm3sys9MutexImplD1Ev", "llvm::sys::MutexImpl::~MutexImpl()"},
    {"_ZN4llvm3sys9MutexImplD2Ev", "llvm::sys::MutexImpl::~MutexImpl()"},
    {"_ZN4llvm3sys9MutexImpl10tryacquireEv", "llvm::sys::MutexImpl::tryacquire()"},
    {"_ZNK4llvm3sys4PatheqERKS1_", "llvm::sys::Path::operator==(llvm::sys::Path const&) const"},
    {"_ZNK4llvm3sys4PathltERKS1_", "llvm::sys::Path::operator<(llvm::sys::Path const&) const"},
    {"_ZN4llvm3sys4Path16GetLLVMConfigDirEv", "llvm::sys::Path::GetLLVMConfigDir()"},
    {"_ZN4llvm3sys4Path3setENS_9StringRefE", "llvm::sys::Path::set(llvm::StringRef)"},
    {"_ZN4llvm3sys4Path23GetLLVMDefaultConfigDirEv", "llvm::sys::Path::GetLLVMDefaultConfigDir()"},
    {"_ZN4llvm3sys16IdentifyFileTypeEPKcj", "llvm::sys::IdentifyFileType(char const*, unsigned int)"},
    {"_ZNK4llvm3sys4Path9isArchiveEv", "llvm::sys::Path::isArchive() const"},
    {"_ZNK4llvm3sys4Path16isDynamicLibraryEv", "llvm::sys::Path::isDynamicLibrary() const"},
    {"_ZNK4llvm3sys4Path12isObjectFileEv", "llvm::sys::Path::isObjectFile() const"},
    {"_ZN4llvm3sys4Path11FindLibraryERSs", "llvm::sys::Path::FindLibrary(std::string&)"},
    {"_ZN4llvm3sys4Path21GetSystemLibraryPathsERSt6vectorIS1_SaIS1_EE", "llvm::sys::Path::GetSystemLibraryPaths(std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> >&)"},
    {"_ZN4llvm3sys4Path15appendComponentENS_9StringRefE", "llvm::sys::Path::appendComponent(llvm::StringRef)"},
    {"_ZN4llvm3sys4Path11eraseSuffixEv", "llvm::sys::Path::eraseSuffix()"},
    {"_ZN4llvm3sys4Path12appendSuffixENS_9StringRefE", "llvm::sys::Path::appendSuffix(llvm::StringRef)"},
    {"_ZN4llvm3sys4Path12GetDLLSuffixEv", "llvm::sys::Path::GetDLLSuffix()"},
    {"_ZNK4llvm3sys4Path13isBitcodeFileEv", "llvm::sys::Path::isBitcodeFile() const"},
    {"_ZNK4llvm3sys4Path14hasMagicNumberENS_9StringRefE", "llvm::sys::Path::hasMagicNumber(llvm::StringRef) const"},
    {"_ZNK4llvm3sys4Path14getMagicNumberERSsj", "llvm::sys::Path::getMagicNumber(std::string&, unsigned int) const"},
    {"_ZN4llvm3sys4Path12GetEXESuffixEv", "llvm::sys::Path::GetEXESuffix()"},
    {"_ZN4llvm3sys4PathC1ENS_9StringRefE", "llvm::sys::Path::Path(llvm::StringRef)"},
    {"_ZN4llvm3sys4PathC2ENS_9StringRefE", "llvm::sys::Path::Path(llvm::StringRef)"},
    {"_ZN4llvm3sys4PathC1EPKcj", "llvm::sys::Path::Path(char const*, unsigned int)"},
    {"_ZN4llvm3sys4PathC2EPKcj", "llvm::sys::Path::Path(char const*, unsigned int)"},
    {"_ZN4llvm3sys4PathaSENS_9StringRefE", "llvm::sys::Path::operator=(llvm::StringRef)"},
    {"_ZNK4llvm3sys4Path7isValidEv", "llvm::sys::Path::isValid() const"},
    {"_ZN4llvm3sys4Path10isAbsoluteEPKcj", "llvm::sys::Path::isAbsolute(char const*, unsigned int)"},
    {"_ZNK4llvm3sys4Path10isAbsoluteEv", "llvm::sys::Path::isAbsolute() const"},
    {"_ZN4llvm3sys4Path16GetRootDirectoryEv", "llvm::sys::Path::GetRootDirectory()"},
    {"_ZN4llvm3sys4Path21GetTemporaryDirectoryEPSs", "llvm::sys::Path::GetTemporaryDirectory(std::string*)"},
    {"_Z10MakeErrMsgPSsRKSsi", "MakeErrMsg(std::string*, std::string const&, int)"},
    {"_Z11getPathListPKcRSt6vectorIN4llvm3sys4PathESaIS4_EE", "getPathList(char const*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> >&)"},
    {"_ZN4llvm3sys4Path22GetBitcodeLibraryPathsERSt6vectorIS1_SaIS1_EE", "llvm::sys::Path::GetBitcodeLibraryPaths(std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> >&)"},
    {"_ZNK4llvm3sys4Path7canReadEv", "llvm::sys::Path::canRead() const"},
    {"_ZN4llvm3sys4Path20GetUserHomeDirectoryEv", "llvm::sys::Path::GetUserHomeDirectory()"},
    {"_ZN4llvm3sys4Path19GetCurrentDirectoryEv", "llvm::sys::Path::GetCurrentDirectory()"},
    {"_ZN4llvm3sys4Path17GetMainExecutableEPKcPv", "llvm::sys::Path::GetMainExecutable(char const*, void*)"},
    {"_ZNK4llvm3sys4Path10getDirnameEv", "llvm::sys::Path::getDirname() const"},
    {"_ZNK4llvm3sys4Path11getBasenameEv", "llvm::sys::Path::getBasename() const"},
    {"_ZNK4llvm3sys4Path9getSuffixEv", "llvm::sys::Path::getSuffix() const"},
    {"_ZNK4llvm3sys4Path6existsEv", "llvm::sys::Path::exists() const"},
    {"_ZNK4llvm3sys4Path11isDirectoryEv", "llvm::sys::Path::isDirectory() const"},
    {"_ZNK4llvm3sys4Path9isSymLinkEv", "llvm::sys::Path::isSymLink() const"},
    {"_ZNK4llvm3sys4Path8canWriteEv", "llvm::sys::Path::canWrite() const"},
    {"_ZNK4llvm3sys4Path13isRegularFileEv", "llvm::sys::Path::isRegularFile() const"},
    {"_ZNK4llvm3sys4Path10canExecuteEv", "llvm::sys::Path::canExecute() const"},
    {"_ZNK4llvm3sys4Path7getLastEv", "llvm::sys::Path::getLast() const"},
    {"_ZNK4llvm3sys14PathWithStatus13getFileStatusEbPSs", "llvm::sys::PathWithStatus::getFileStatus(bool, std::string*) const"},
    {"_ZN4llvm3sys4Path18makeReadableOnDiskEPSs", "llvm::sys::Path::makeReadableOnDisk(std::string*)"},
    {"_ZN4llvm3sys4Path19makeWriteableOnDiskEPSs", "llvm::sys::Path::makeWriteableOnDisk(std::string*)"},
    {"_ZN4llvm3sys4Path20makeExecutableOnDiskEPSs", "llvm::sys::Path::makeExecutableOnDisk(std::string*)"},
    {"_ZNK4llvm3sys4Path20getDirectoryContentsERSt3setIS1_St4lessIS1_ESaIS1_EEPSs", "llvm::sys::Path::getDirectoryContents(std::set<llvm::sys::Path, std::less<llvm::sys::Path>, std::allocator<llvm::sys::Path> >&, std::string*) const"},
    {"_ZN4llvm3sys4Path14eraseComponentEv", "llvm::sys::Path::eraseComponent()"},
    {"_ZN4llvm3sys4Path21createDirectoryOnDiskEbPSs", "llvm::sys::Path::createDirectoryOnDisk(bool, std::string*)"},
    {"_ZN4llvm21createDirectoryHelperEPcS0_b", "llvm::createDirectoryHelper(char*, char*, bool)"},
    {"_ZN4llvm3sys4Path16createFileOnDiskEPSs", "llvm::sys::Path::createFileOnDisk(std::string*)"},
    {"_ZN4llvm3sys4Path25createTemporaryFileOnDiskEbPSs", "llvm::sys::Path::createTemporaryFileOnDisk(bool, std::string*)"},
    {"_ZN4llvm3sys4Path10makeUniqueEbPSs", "llvm::sys::Path::makeUnique(bool, std::string*)"},
    {"_ZNK4llvm3sys4Path13eraseFromDiskEbPSs", "llvm::sys::Path::eraseFromDisk(bool, std::string*) const"},
    {"_ZN4llvm3sys4Path16renamePathOnDiskERKS1_PSs", "llvm::sys::Path::renamePathOnDisk(llvm::sys::Path const&, std::string*)"},
    {"_ZNK4llvm3sys4Path19setStatusInfoOnDiskERKNS0_10FileStatusEPSs", "llvm::sys::Path::setStatusInfoOnDisk(llvm::sys::FileStatus const&, std::string*) const"},
    {"_ZN4llvm3sys8CopyFileERKNS0_4PathES3_PSs", "llvm::sys::CopyFile(llvm::sys::Path const&, llvm::sys::Path const&, std::string*)"},
    {"_ZN4llvm3sys4Path14MapInFilePagesEiy", "llvm::sys::Path::MapInFilePages(int, unsigned long long)"},
    {"_ZN4llvm3sys4Path14UnMapFilePagesEPKcy", "llvm::sys::Path::UnMapFilePages(char const*, unsigned long long)"},
    {"_ZNSt6vectorIcSaIcEE14_M_fill_insertEN9__gnu_cxx17__normal_iteratorIPcS1_EEmRKc", "std::vector<char, std::allocator<char> >::_M_fill_insert(__gnu_cxx::__normal_iterator<char*, std::vector<char, std::allocator<char> > >, unsigned long, char const&)"},
    {"_ZNSt8_Rb_treeIN4llvm3sys4PathES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE16_M_insert_uniqueERKS2_", "std::_Rb_tree<llvm::sys::Path, llvm::sys::Path, std::_Identity<llvm::sys::Path>, std::less<llvm::sys::Path>, std::allocator<llvm::sys::Path> >::_M_insert_unique(llvm::sys::Path const&)"},
    {"_ZNSt8_Rb_treeIN4llvm3sys4PathES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE9_M_insertEPSt18_Rb_tree_node_baseSA_RKS2_", "std::_Rb_tree<llvm::sys::Path, llvm::sys::Path, std::_Identity<llvm::sys::Path>, std::less<llvm::sys::Path>, std::allocator<llvm::sys::Path> >::_M_insert(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, llvm::sys::Path const&)"},
    {"_ZNSt8_Rb_treeIN4llvm3sys4PathES2_St9_IdentityIS2_ESt4lessIS2_ESaIS2_EE8_M_eraseEPSt13_Rb_tree_nodeIS2_E", "std::_Rb_tree<llvm::sys::Path, llvm::sys::Path, std::_Identity<llvm::sys::Path>, std::less<llvm::sys::Path>, std::allocator<llvm::sys::Path> >::_M_erase(std::_Rb_tree_node<llvm::sys::Path>*)"},
    {"_ZN4llvm3sys4path5beginENS_9StringRefE", "llvm::sys::path::begin(llvm::StringRef)"},
    {"_ZN4llvm3sys4path3endENS_9StringRefE", "llvm::sys::path::end(llvm::StringRef)"},
    {"_ZN4llvm3sys4path14const_iteratorppEv", "llvm::sys::path::const_iterator::operator++()"},
    {"_ZN4llvm3sys4path14const_iteratormmEv", "llvm::sys::path::const_iterator::operator--()"},
    {"_ZNK4llvm3sys4path14const_iteratoreqERKS2_", "llvm::sys::path::const_iterator::operator==(llvm::sys::path::const_iterator const&) const"},
    {"_ZNK4llvm3sys4path14const_iteratorneERKS2_", "llvm::sys::path::const_iterator::operator!=(llvm::sys::path::const_iterator const&) const"},
    {"_ZNK4llvm3sys4path14const_iteratormiERKS2_", "llvm::sys::path::const_iterator::operator-(llvm::sys::path::const_iterator const&) const"},
    {"_ZN4llvm3sys4path9root_pathENS_9StringRefE", "llvm::sys::path::root_path(llvm::StringRef)"},
    {"_ZN4llvm3sys4path9root_nameENS_9StringRefE", "llvm::sys::path::root_name(llvm::StringRef)"},
    {"_ZN4llvm3sys4path14root_directoryENS_9StringRefE", "llvm::sys::path::root_directory(llvm::StringRef)"},
    {"_ZN4llvm3sys4path13relative_pathENS_9StringRefE", "llvm::sys::path::relative_path(llvm::StringRef)"},
    {"_ZN4llvm3sys4path6appendERNS_15SmallVectorImplIcEERKNS_5TwineES7_S7_S7_", "llvm::sys::path::append(llvm::SmallVectorImpl<char>&, llvm::Twine const&, llvm::Twine const&, llvm::Twine const&, llvm::Twine const&)"},
    {"_ZN4llvm3sys4path13has_root_nameERKNS_5TwineE", "llvm::sys::path::has_root_name(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path11parent_pathENS_9StringRefE", "llvm::sys::path::parent_path(llvm::StringRef)"},
    {"_ZN12_GLOBAL__N_115parent_path_endEN4llvm9StringRefE", "(anonymous namespace)::parent_path_end(llvm::StringRef)"},
    {"_ZN4llvm3sys4path15remove_filenameERNS_15SmallVectorImplIcEE", "llvm::sys::path::remove_filename(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys4path17replace_extensionERNS_15SmallVectorImplIcEERKNS_5TwineE", "llvm::sys::path::replace_extension(llvm::SmallVectorImpl<char>&, llvm::Twine const&)"},
    {"_ZN4llvm3sys4path6nativeERKNS_5TwineERNS_15SmallVectorImplIcEE", "llvm::sys::path::native(llvm::Twine const&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys4path8filenameENS_9StringRefE", "llvm::sys::path::filename(llvm::StringRef)"},
    {"_ZN4llvm3sys4path4stemENS_9StringRefE", "llvm::sys::path::stem(llvm::StringRef)"},
    {"_ZN4llvm3sys4path9extensionENS_9StringRefE", "llvm::sys::path::extension(llvm::StringRef)"},
    {"_ZN4llvm3sys4path18has_root_directoryERKNS_5TwineE", "llvm::sys::path::has_root_directory(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path13has_root_pathERKNS_5TwineE", "llvm::sys::path::has_root_path(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path17has_relative_pathERKNS_5TwineE", "llvm::sys::path::has_relative_path(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path12has_filenameERKNS_5TwineE", "llvm::sys::path::has_filename(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path15has_parent_pathERKNS_5TwineE", "llvm::sys::path::has_parent_path(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path8has_stemERKNS_5TwineE", "llvm::sys::path::has_stem(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path13has_extensionERKNS_5TwineE", "llvm::sys::path::has_extension(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path11is_absoluteERKNS_5TwineE", "llvm::sys::path::is_absolute(llvm::Twine const&)"},
    {"_ZN4llvm3sys4path11is_relativeERKNS_5TwineE", "llvm::sys::path::is_relative(llvm::Twine const&)"},
    {"_ZN4llvm3sys2fs13make_absoluteERNS_15SmallVectorImplIcEE", "llvm::sys::fs::make_absolute(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys2fs12current_pathERNS_15SmallVectorImplIcEE", "llvm::sys::fs::current_path(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys2fs18create_directoriesERKNS_5TwineERb", "llvm::sys::fs::create_directories(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs6existsERKNS_5TwineERb", "llvm::sys::fs::exists(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs16create_directoryERKNS_5TwineERb", "llvm::sys::fs::create_directory(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs6existsENS1_11file_statusE", "llvm::sys::fs::exists(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs12status_knownENS1_11file_statusE", "llvm::sys::fs::status_known(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs12is_directoryENS1_11file_statusE", "llvm::sys::fs::is_directory(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs12is_directoryERKNS_5TwineERb", "llvm::sys::fs::is_directory(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs6statusERKNS_5TwineERNS1_11file_statusE", "llvm::sys::fs::status(llvm::Twine const&, llvm::sys::fs::file_status&)"},
    {"_ZN4llvm3sys2fs15is_regular_fileENS1_11file_statusE", "llvm::sys::fs::is_regular_file(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs15is_regular_fileERKNS_5TwineERb", "llvm::sys::fs::is_regular_file(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs10is_symlinkENS1_11file_statusE", "llvm::sys::fs::is_symlink(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs10is_symlinkERKNS_5TwineERb", "llvm::sys::fs::is_symlink(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs8is_otherENS1_11file_statusE", "llvm::sys::fs::is_other(llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs15directory_entry16replace_filenameERKNS_5TwineENS1_11file_statusES6_", "llvm::sys::fs::directory_entry::replace_filename(llvm::Twine const&, llvm::sys::fs::file_status, llvm::sys::fs::file_status)"},
    {"_ZN4llvm3sys2fs9has_magicERKNS_5TwineES4_Rb", "llvm::sys::fs::has_magic(llvm::Twine const&, llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs9get_magicERKNS_5TwineEjRNS_15SmallVectorImplIcEE", "llvm::sys::fs::get_magic(llvm::Twine const&, unsigned int, llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys2fs14identify_magicERKNS_5TwineERNS0_12LLVMFileTypeE", "llvm::sys::fs::identify_magic(llvm::Twine const&, llvm::sys::LLVMFileType&)"},
    {"_ZN4llvm3sys2fs10remove_allERKNS_5TwineERj", "llvm::sys::fs::remove_all(llvm::Twine const&, unsigned int&)"},
    {"_ZN4llvm3sys2fs12_GLOBAL__N_112remove_all_rENS_9StringRefENS1_9file_typeERj", "llvm::sys::fs::(anonymous namespace)::remove_all_r(llvm::StringRef, llvm::sys::fs::file_type, unsigned int&)"},
    {"_ZNK4llvm3sys2fs15directory_entry6statusERNS1_11file_statusE", "llvm::sys::fs::directory_entry::status(llvm::sys::fs::file_status&) const"},
    {"_ZN4llvm3sys2fs9copy_fileERKNS_5TwineES4_NS1_11copy_optionE", "llvm::sys::fs::copy_file(llvm::Twine const&, llvm::Twine const&, llvm::sys::fs::copy_option)"},
    {"_ZN4llvm3sys2fs16create_hard_linkERKNS_5TwineES4_", "llvm::sys::fs::create_hard_link(llvm::Twine const&, llvm::Twine const&)"},
    {"_ZN4llvm3sys2fs14create_symlinkERKNS_5TwineES4_", "llvm::sys::fs::create_symlink(llvm::Twine const&, llvm::Twine const&)"},
    {"_ZN4llvm3sys2fs6removeERKNS_5TwineERb", "llvm::sys::fs::remove(llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs6renameERKNS_5TwineES4_", "llvm::sys::fs::rename(llvm::Twine const&, llvm::Twine const&)"},
    {"_ZN4llvm3sys2fs11resize_fileERKNS_5TwineEy", "llvm::sys::fs::resize_file(llvm::Twine const&, unsigned long long)"},
    {"_ZN4llvm3sys2fs10equivalentERKNS_5TwineES4_Rb", "llvm::sys::fs::equivalent(llvm::Twine const&, llvm::Twine const&, bool&)"},
    {"_ZN4llvm3sys2fs9file_sizeERKNS_5TwineERy", "llvm::sys::fs::file_size(llvm::Twine const&, unsigned long long&)"},
    {"_ZN4llvm3sys2fs11unique_fileERKNS_5TwineERiRNS_15SmallVectorImplIcEE", "llvm::sys::fs::unique_file(llvm::Twine const&, int&, llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm3sys2fs28directory_iterator_constructERNS1_18directory_iteratorENS_9StringRefE", "llvm::sys::fs::directory_iterator_construct(llvm::sys::fs::directory_iterator&, llvm::StringRef)"},
    {"_ZN4llvm3sys2fs28directory_iterator_incrementERNS1_18directory_iteratorE", "llvm::sys::fs::directory_iterator_increment(llvm::sys::fs::directory_iterator&)"},
    {"_ZN4llvm3sys2fs27directory_iterator_destructERNS1_18directory_iteratorE", "llvm::sys::fs::directory_iterator_destruct(llvm::sys::fs::directory_iterator&)"},
    {"_ZN4llvm3sys4path9canonicalEPKcRNS_15SmallVectorImplIcEE", "llvm::sys::path::canonical(char const*, llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm15SmallVectorImplIcE4swapERS1_", "llvm::SmallVectorImpl<char>::swap(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm21PrettyStackTraceEntryC1Ev", "llvm::PrettyStackTraceEntry::PrettyStackTraceEntry()"},
    {"_ZN4llvm21PrettyStackTraceEntryC2Ev", "llvm::PrettyStackTraceEntry::PrettyStackTraceEntry()"},
    {"_ZN4llvm21PrettyStackTraceEntryD0Ev", "llvm::PrettyStackTraceEntry::~PrettyStackTraceEntry()"},
    {"_ZN4llvm21PrettyStackTraceEntryD1Ev", "llvm::PrettyStackTraceEntry::~PrettyStackTraceEntry()"},
    {"_ZN4llvm21PrettyStackTraceEntryD2Ev", "llvm::PrettyStackTraceEntry::~PrettyStackTraceEntry()"},
    {"_ZNK4llvm22PrettyStackTraceString5printERNS_11raw_ostreamE", "llvm::PrettyStackTraceString::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm23PrettyStackTraceProgram5printERNS_11raw_ostreamE", "llvm::PrettyStackTraceProgram::print(llvm::raw_ostream&) const"},
    {"_Z12CrashHandlerPv", "CrashHandler(void*)"},
    {"_Z10PrintStackPKN4llvm21PrettyStackTraceEntryERNS_11raw_ostreamE", "PrintStack(llvm::PrettyStackTraceEntry const*, llvm::raw_ostream&)"},
    {"_ZN4llvm3sys11ThreadLocalIKNS_21PrettyStackTraceEntryEED1Ev", "llvm::sys::ThreadLocal<llvm::PrettyStackTraceEntry const>::~ThreadLocal()"},
    {"_ZN4llvm3sys11ThreadLocalIKNS_21PrettyStackTraceEntryEED0Ev", "llvm::sys::ThreadLocal<llvm::PrettyStackTraceEntry const>::~ThreadLocal()"},
    {"_ZN4llvm3sys7Process11GetPageSizeEv", "llvm::sys::Process::GetPageSize()"},
    {"_ZN4llvm3sys7Process14GetMallocUsageEv", "llvm::sys::Process::GetMallocUsage()"},
    {"_ZN4llvm3sys7Process19GetTotalMemoryUsageEv", "llvm::sys::Process::GetTotalMemoryUsage()"},
    {"_ZN4llvm3sys7Process12GetTimeUsageERNS0_9TimeValueES3_S3_", "llvm::sys::Process::GetTimeUsage(llvm::sys::TimeValue&, llvm::sys::TimeValue&, llvm::sys::TimeValue&)"},
    {"_ZN4llvm3sys7Process16GetCurrentUserIdEv", "llvm::sys::Process::GetCurrentUserId()"},
    {"_ZN4llvm3sys7Process17GetCurrentGroupIdEv", "llvm::sys::Process::GetCurrentGroupId()"},
    {"_ZN4llvm3sys7Process16PreventCoreFilesEv", "llvm::sys::Process::PreventCoreFiles()"},
    {"_ZN4llvm3sys7Process21StandardInIsUserInputEv", "llvm::sys::Process::StandardInIsUserInput()"},
    {"_ZN4llvm3sys7Process25FileDescriptorIsDisplayedEi", "llvm::sys::Process::FileDescriptorIsDisplayed(int)"},
    {"_ZN4llvm3sys7Process22StandardOutIsDisplayedEv", "llvm::sys::Process::StandardOutIsDisplayed()"},
    {"_ZN4llvm3sys7Process22StandardErrIsDisplayedEv", "llvm::sys::Process::StandardErrIsDisplayed()"},
    {"_ZN4llvm3sys7Process18StandardOutColumnsEv", "llvm::sys::Process::StandardOutColumns()"},
    {"_ZN4llvm3sys7Process18StandardErrColumnsEv", "llvm::sys::Process::StandardErrColumns()"},
    {"_ZN4llvm3sys7Process20StandardOutHasColorsEv", "llvm::sys::Process::StandardOutHasColors()"},
    {"_ZN4llvm3sys7Process20StandardErrHasColorsEv", "llvm::sys::Process::StandardErrHasColors()"},
    {"_ZN4llvm3sys7Process15ColorNeedsFlushEv", "llvm::sys::Process::ColorNeedsFlush()"},
    {"_ZN4llvm3sys7Process11OutputColorEcbb", "llvm::sys::Process::OutputColor(char, bool, bool)"},
    {"_ZN4llvm3sys7Process10OutputBoldEb", "llvm::sys::Process::OutputBold(bool)"},
    {"_ZN4llvm3sys7Process10ResetColorEv", "llvm::sys::Process::ResetColor()"},
    {"_ZN4llvm3sys7Program14ExecuteAndWaitERKNS0_4PathEPPKcS7_PPS3_jjPSs", "llvm::sys::Program::ExecuteAndWait(llvm::sys::Path const&, char const**, char const**, llvm::sys::Path const**, unsigned int, unsigned int, std::string*)"},
    {"_ZN4llvm3sys7ProgramC1Ev", "llvm::sys::Program::Program()"},
    {"_ZN4llvm3sys7Program7ExecuteERKNS0_4PathEPPKcS7_PPS3_jPSs", "llvm::sys::Program::Execute(llvm::sys::Path const&, char const**, char const**, llvm::sys::Path const**, unsigned int, std::string*)"},
    {"_ZN4llvm3sys7Program4WaitERKNS0_4PathEjPSs", "llvm::sys::Program::Wait(llvm::sys::Path const&, unsigned int, std::string*)"},
    {"_ZN4llvm3sys7ProgramD1Ev", "llvm::sys::Program::~Program()"},
    {"_ZN4llvm3sys7Program13ExecuteNoWaitERKNS0_4PathEPPKcS7_PPS3_jPSs", "llvm::sys::Program::ExecuteNoWait(llvm::sys::Path const&, char const**, char const**, llvm::sys::Path const**, unsigned int, std::string*)"},
    {"_ZN4llvm3sys7ProgramC2Ev", "llvm::sys::Program::Program()"},
    {"_ZN4llvm3sys7ProgramD2Ev", "llvm::sys::Program::~Program()"},
    {"_ZNK4llvm3sys7Program6GetPidEv", "llvm::sys::Program::GetPid() const"},
    {"_ZN4llvm3sys7Program17FindProgramByNameERKSs", "llvm::sys::Program::FindProgramByName(std::string const&)"},
    {"_ZN4llvm13RedirectIO_PSEPKNS_3sys4PathEiPSsRPv", "llvm::RedirectIO_PS(llvm::sys::Path const*, int, std::string*, void*&)"},
    {"_Z10MakeErrMsgPSsRKSsi", "MakeErrMsg(std::string*, std::string const&, int)"},
    {"_ZN4llvm10RedirectIOEPKNS_3sys4PathEiPSs", "llvm::RedirectIO(llvm::sys::Path const*, int, std::string*)"},
    {"_ZN4llvm14TimeOutHandlerEi", "llvm::TimeOutHandler(int)"},
    {"_ZN4llvm3sys7Program4KillEPSs", "llvm::sys::Program::Kill(std::string*)"},
    {"_ZN4llvm3sys7Program19ChangeStdinToBinaryEv", "llvm::sys::Program::ChangeStdinToBinary()"},
    {"_ZN4llvm3sys7Program20ChangeStdoutToBinaryEv", "llvm::sys::Program::ChangeStdoutToBinary()"},
    {"_ZN4llvm3sys7Program20ChangeStderrToBinaryEv", "llvm::sys::Program::ChangeStderrToBinary()"},
    {"_ZN4llvm3sys11RWMutexImplC1Ev", "llvm::sys::RWMutexImpl::RWMutexImpl()"},
    {"_ZN4llvm3sys11RWMutexImplC2Ev", "llvm::sys::RWMutexImpl::RWMutexImpl()"},
    {"_ZN4llvm3sys11RWMutexImplD1Ev", "llvm::sys::RWMutexImpl::~RWMutexImpl()"},
    {"_ZN4llvm3sys11RWMutexImplD2Ev", "llvm::sys::RWMutexImpl::~RWMutexImpl()"},
    {"_ZN4llvm3sys11RWMutexImpl14reader_acquireEv", "llvm::sys::RWMutexImpl::reader_acquire()"},
    {"_ZN4llvm3sys11RWMutexImpl14reader_releaseEv", "llvm::sys::RWMutexImpl::reader_release()"},
    {"_ZN4llvm3sys11RWMutexImpl14writer_acquireEv", "llvm::sys::RWMutexImpl::writer_acquire()"},
    {"_ZN4llvm3sys11RWMutexImpl14writer_releaseEv", "llvm::sys::RWMutexImpl::writer_release()"},
    {"_ZN4llvm5RegexC1ENS_9StringRefEj", "llvm::Regex::Regex(llvm::StringRef, unsigned int)"},
    {"_ZN4llvm5RegexC2ENS_9StringRefEj", "llvm::Regex::Regex(llvm::StringRef, unsigned int)"},
    {"_ZN4llvm5RegexD1Ev", "llvm::Regex::~Regex()"},
    {"_ZN4llvm5RegexD2Ev", "llvm::Regex::~Regex()"},
    {"_ZN4llvm5Regex7isValidERSs", "llvm::Regex::isValid(std::string&)"},
    {"_ZNK4llvm5Regex13getNumMatchesEv", "llvm::Regex::getNumMatches() const"},
    {"_ZN4llvm5Regex5matchENS_9StringRefEPNS_15SmallVectorImplIS1_EE", "llvm::Regex::match(llvm::StringRef, llvm::SmallVectorImpl<llvm::StringRef>*)"},
    {"_ZN4llvm5Regex3subENS_9StringRefES1_PSs", "llvm::Regex::sub(llvm::StringRef, llvm::StringRef, std::string*)"},
    {"_ZN4llvm15SmallVectorImplI15llvm_regmatch_tE6resizeEj", "llvm::SmallVectorImpl<llvm_regmatch_t>::resize(unsigned int)"},
    {"_ZN4llvm23SmallVectorTemplateBaseI15llvm_regmatch_tLb0EE4growEm", "llvm::SmallVectorTemplateBase<llvm_regmatch_t, false>::grow(unsigned long)"},
    {"_ZN4llvm31SearchForAddressOfSpecialSymbolEPKc", "llvm::SearchForAddressOfSpecialSymbol(char const*)"},
    {"_ZN4llvm3sys20RunInterruptHandlersEv", "llvm::sys::RunInterruptHandlers()"},
    {"_ZN4llvm3sys20SetInterruptFunctionEPFvvE", "llvm::sys::SetInterruptFunction(void (*)())"},
    {"_ZN4llvm3sys18RemoveFileOnSignalERKNS0_4PathEPSs", "llvm::sys::RemoveFileOnSignal(llvm::sys::Path const&, std::string*)"},
    {"_ZN4llvm3sys22DontRemoveFileOnSignalERKNS0_4PathE", "llvm::sys::DontRemoveFileOnSignal(llvm::sys::Path const&)"},
    {"_ZN4llvm3sys16AddSignalHandlerEPFvPvES1_", "llvm::sys::AddSignalHandler(void (*)(void*), void*)"},
    {"_ZN4llvm3sys28PrintStackTraceOnErrorSignalEv", "llvm::sys::PrintStackTraceOnErrorSignal()"},
    {"_Z15PrintStackTracePv", "PrintStackTrace(void*)"},
    {"_Z12__assert_rtnPKcS0_iS0_", "__assert_rtn(char const*, char const*, int, char const*)"},
    {"_Z13SignalHandleri", "SignalHandler(int)"},
    {"_ZN4llvm3sys10SmartMutexILb1EED1Ev", "llvm::sys::SmartMutex<true>::~SmartMutex()"},
    {"_ZNSt6vectorIN4llvm3sys4PathESaIS2_EED1Ev", "std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> >::~vector()"},
    {"_ZNSt6vectorISt4pairIPFvPvES1_ESaIS4_EED1Ev", "std::vector<std::pair<void (*)(void*), void*>, std::allocator<std::pair<void (*)(void*), void*> > >::~vector()"},
    {"_ZNSt6vectorISt4pairIPFvPvES1_ESaIS4_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS4_S6_EERKS4_", "std::vector<std::pair<void (*)(void*), void*>, std::allocator<std::pair<void (*)(void*), void*> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<void (*)(void*), void*>*, std::vector<std::pair<void (*)(void*), void*>, std::allocator<std::pair<void (*)(void*), void*> > > >, std::pair<void (*)(void*), void*> const&)"},
    {"_ZSt6__findISt16reverse_iteratorIN9__gnu_cxx17__normal_iteratorIPN4llvm3sys4PathESt6vectorIS5_SaIS5_EEEEES5_ET_SC_SC_RKT0_St26random_access_iterator_tag", "std::reverse_iterator<__gnu_cxx::__normal_iterator<llvm::sys::Path*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> > > > std::__find<std::reverse_iterator<__gnu_cxx::__normal_iterator<llvm::sys::Path*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> > > >, llvm::sys::Path>(std::reverse_iterator<__gnu_cxx::__normal_iterator<llvm::sys::Path*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> > > >, std::reverse_iterator<__gnu_cxx::__normal_iterator<llvm::sys::Path*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> > > >, llvm::sys::Path const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIN4llvm3sys4PathESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::sys::Path*, std::vector<llvm::sys::Path, std::allocator<llvm::sys::Path> > >, llvm::sys::Path const&)"},
    {"_ZSt6__findIPKiiET_S2_S2_RKT0_St26random_access_iterator_tag", "int const* std::__find<int const*, int>(int const*, int const*, int const&, std::random_access_iterator_tag)"},
    {"_ZN4llvm15SmallPtrSetImpl16shrink_and_clearEv", "llvm::SmallPtrSetImpl::shrink_and_clear()"},
    {"_ZN4llvm15SmallPtrSetImpl10insert_impEPKv", "llvm::SmallPtrSetImpl::insert_imp(void const*)"},
    {"_ZN4llvm15SmallPtrSetImpl4GrowEv", "llvm::SmallPtrSetImpl::Grow()"},
    {"_ZNK4llvm15SmallPtrSetImpl13FindBucketForEPKv", "llvm::SmallPtrSetImpl::FindBucketFor(void const*) const"},
    {"_ZN4llvm15SmallPtrSetImpl9erase_impEPKv", "llvm::SmallPtrSetImpl::erase_imp(void const*)"},
    {"_ZN4llvm15SmallPtrSetImplC1EPPKvRKS0_", "llvm::SmallPtrSetImpl::SmallPtrSetImpl(void const**, llvm::SmallPtrSetImpl const&)"},
    {"_ZN4llvm15SmallPtrSetImplC2EPPKvRKS0_", "llvm::SmallPtrSetImpl::SmallPtrSetImpl(void const**, llvm::SmallPtrSetImpl const&)"},
    {"_ZN4llvm15SmallPtrSetImpl8CopyFromERKS0_", "llvm::SmallPtrSetImpl::CopyFrom(llvm::SmallPtrSetImpl const&)"},
    {"_ZN4llvm15SmallPtrSetImplD1Ev", "llvm::SmallPtrSetImpl::~SmallPtrSetImpl()"},
    {"_ZN4llvm15SmallPtrSetImplD2Ev", "llvm::SmallPtrSetImpl::~SmallPtrSetImpl()"},
    {"_ZN4llvm9SourceMgrD1Ev", "llvm::SourceMgr::~SourceMgr()"},
    {"_ZN4llvm9SourceMgrD2Ev", "llvm::SourceMgr::~SourceMgr()"},
    {"_ZN4llvm9SourceMgr14AddIncludeFileERKSsNS_5SMLocE", "llvm::SourceMgr::AddIncludeFile(std::string const&, llvm::SMLoc)"},
    {"_ZNK4llvm9SourceMgr23FindBufferContainingLocENS_5SMLocE", "llvm::SourceMgr::FindBufferContainingLoc(llvm::SMLoc) const"},
    {"_ZNK4llvm9SourceMgr14FindLineNumberENS_5SMLocEi", "llvm::SourceMgr::FindLineNumber(llvm::SMLoc, int) const"},
    {"_ZNK4llvm9SourceMgr17PrintIncludeStackENS_5SMLocERNS_11raw_ostreamE", "llvm::SourceMgr::PrintIncludeStack(llvm::SMLoc, llvm::raw_ostream&) const"},
    {"_ZNK4llvm9SourceMgr10GetMessageENS_5SMLocERKNS_5TwineEPKcb", "llvm::SourceMgr::GetMessage(llvm::SMLoc, llvm::Twine const&, char const*, bool) const"},
    {"_ZNK4llvm9SourceMgr12PrintMessageENS_5SMLocERKNS_5TwineEPKcb", "llvm::SourceMgr::PrintMessage(llvm::SMLoc, llvm::Twine const&, char const*, bool) const"},
    {"_ZNK4llvm12SMDiagnostic5PrintEPKcRNS_11raw_ostreamE", "llvm::SMDiagnostic::Print(char const*, llvm::raw_ostream&) const"},
    {"_ZN4llvm9Statistic17RegisterStatisticEv", "llvm::Statistic::RegisterStatistic()"},
    {"_ZN4llvm16EnableStatisticsEv", "llvm::EnableStatistics()"},
    {"_ZN4llvm15PrintStatisticsERNS_11raw_ostreamE", "llvm::PrintStatistics(llvm::raw_ostream&)"},
    {"_ZN4llvm15PrintStatisticsEv", "llvm::PrintStatistics()"},
    {"_ZSt21__inplace_stable_sortIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEEN12_GLOBAL__N_111NameCompareEEvT_SD_T0_", "void std::__inplace_stable_sort<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare)"},
    {"_ZSt22__stable_sort_adaptiveIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEES6_lN12_GLOBAL__N_111NameCompareEEvT_SD_T0_T1_T2_", "void std::__stable_sort_adaptive<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, long, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, long, (anonymous namespace)::NameCompare)"},
    {"_ZSt24__merge_sort_with_bufferIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEES6_N12_GLOBAL__N_111NameCompareEEvT_SD_T0_T1_", "void std::__merge_sort_with_buffer<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, (anonymous namespace)::NameCompare)"},
    {"_ZSt16__merge_adaptiveIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEElS6_N12_GLOBAL__N_111NameCompareEEvT_SD_SD_T0_SE_T1_SE_T2_", "void std::__merge_adaptive<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, long, llvm::Statistic const**, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, long, long, llvm::Statistic const**, long, (anonymous namespace)::NameCompare)"},
    {"_ZSt5mergeIPPKN4llvm9StatisticES4_N9__gnu_cxx17__normal_iteratorIS4_St6vectorIS3_SaIS3_EEEEN12_GLOBAL__N_111NameCompareEET1_T_SE_T0_SF_SD_T2_", "__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > > std::merge<llvm::Statistic const**, llvm::Statistic const**, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare>(llvm::Statistic const**, llvm::Statistic const**, llvm::Statistic const**, llvm::Statistic const**, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare)"},
    {"_ZSt5mergeIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEESA_S6_N12_GLOBAL__N_111NameCompareEET1_T_SE_T0_SF_SD_T2_", "llvm::Statistic const** std::merge<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const**, (anonymous namespace)::NameCompare)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEEN12_GLOBAL__N_111NameCompareEEvT_SD_T0_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, (anonymous namespace)::NameCompare)"},
    {"_ZSt22__merge_without_bufferIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEElN12_GLOBAL__N_111NameCompareEEvT_SD_SD_T0_SE_T1_", "void std::__merge_without_buffer<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, long, (anonymous namespace)::NameCompare>(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, long, long, (anonymous namespace)::NameCompare)"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_113StatisticInfoEEEPvv", "void* llvm::object_creator<(anonymous namespace)::StatisticInfo>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_113StatisticInfoEE4callEPv", "llvm::object_deleter<(anonymous namespace)::StatisticInfo>::call(void*)"},
    {"_ZSt8__rotateIN9__gnu_cxx17__normal_iteratorIPPKN4llvm9StatisticESt6vectorIS5_SaIS5_EEEEEvT_SB_SB_St26random_access_iterator_tag", "void std::__rotate<__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > > >(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, __gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPKN4llvm9StatisticESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Statistic const**, std::vector<llvm::Statistic const*, std::allocator<llvm::Statistic const*> > >, llvm::Statistic const* const&)"},
    {"_ZN4llvm14StrInStrNoCaseENS_9StringRefES0_", "llvm::StrInStrNoCase(llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm8getTokenENS_9StringRefES0_", "llvm::getToken(llvm::StringRef, llvm::StringRef)"},
    {"_ZN4llvm11SplitStringENS_9StringRefERNS_15SmallVectorImplIS0_EES0_", "llvm::SplitString(llvm::StringRef, llvm::SmallVectorImpl<llvm::StringRef>&, llvm::StringRef)"},
    {"_ZNK4llvm9StringRef5splitERNS_15SmallVectorImplIS0_EES0_ib", "llvm::StringRef::split(llvm::SmallVectorImpl<llvm::StringRef>&, llvm::StringRef, int, bool) const"},
    {"_ZN4llvm13StringMapImplC1Ejj", "llvm::StringMapImpl::StringMapImpl(unsigned int, unsigned int)"},
    {"_ZN4llvm13StringMapImplC2Ejj", "llvm::StringMapImpl::StringMapImpl(unsigned int, unsigned int)"},
    {"_ZN4llvm13StringMapImpl4initEj", "llvm::StringMapImpl::init(unsigned int)"},
    {"_ZN4llvm13StringMapImpl15LookupBucketForENS_9StringRefE", "llvm::StringMapImpl::LookupBucketFor(llvm::StringRef)"},
    {"_ZNK4llvm13StringMapImpl7FindKeyENS_9StringRefE", "llvm::StringMapImpl::FindKey(llvm::StringRef) const"},
    {"_ZN4llvm13StringMapImpl9RemoveKeyEPNS_18StringMapEntryBaseE", "llvm::StringMapImpl::RemoveKey(llvm::StringMapEntryBase*)"},
    {"_ZN4llvm13StringMapImpl9RemoveKeyENS_9StringRefE", "llvm::StringMapImpl::RemoveKey(llvm::StringRef)"},
    {"_ZN4llvm13StringMapImpl11RehashTableEv", "llvm::StringMapImpl::RehashTable()"},
    {"_ZN4llvm10StringPoolC1Ev", "llvm::StringPool::StringPool()"},
    {"_ZN4llvm10StringPoolC2Ev", "llvm::StringPool::StringPool()"},
    {"_ZN4llvm10StringPoolD1Ev", "llvm::StringPool::~StringPool()"},
    {"_ZN4llvm10StringPoolD2Ev", "llvm::StringPool::~StringPool()"},
    {"_ZN4llvm10StringPool6internENS_9StringRefE", "llvm::StringPool::intern(llvm::StringRef)"},
    {"_ZNK4llvm9StringRef13compare_lowerES0_", "llvm::StringRef::compare_lower(llvm::StringRef) const"},
    {"_ZNK4llvm9StringRef15compare_numericES0_", "llvm::StringRef::compare_numeric(llvm::StringRef) const"},
    {"_ZN4llvm9StringRef13edit_distanceES0_bj", "llvm::StringRef::edit_distance(llvm::StringRef, bool, unsigned int)"},
    {"_ZNK4llvm9StringRef4findES0_m", "llvm::StringRef::find(llvm::StringRef, unsigned long) const"},
    {"_ZNK4llvm9StringRef5rfindES0_", "llvm::StringRef::rfind(llvm::StringRef) const"},
    {"_ZNK4llvm9StringRef13find_first_ofES0_m", "llvm::StringRef::find_first_of(llvm::StringRef, unsigned long) const"},
    {"_ZNK4llvm9StringRef17find_first_not_ofEcm", "llvm::StringRef::find_first_not_of(char, unsigned long) const"},
    {"_ZNK4llvm9StringRef17find_first_not_ofES0_m", "llvm::StringRef::find_first_not_of(llvm::StringRef, unsigned long) const"},
    {"_ZNK4llvm9StringRef12find_last_ofES0_m", "llvm::StringRef::find_last_of(llvm::StringRef, unsigned long) const"},
    {"_ZNK4llvm9StringRef5countES0_", "llvm::StringRef::count(llvm::StringRef) const"},
    {"_ZNK4llvm9StringRef12getAsIntegerEjRy", "llvm::StringRef::getAsInteger(unsigned int, unsigned long long&) const"},
    {"_Z20GetAsUnsignedIntegerN4llvm9StringRefEjRy", "GetAsUnsignedInteger(llvm::StringRef, unsigned int, unsigned long long&)"},
    {"_ZNK4llvm9StringRef12getAsIntegerEjRx", "llvm::StringRef::getAsInteger(unsigned int, long long&) const"},
    {"_ZNK4llvm9StringRef12getAsIntegerEjRi", "llvm::StringRef::getAsInteger(unsigned int, int&) const"},
    {"_ZNK4llvm9StringRef12getAsIntegerEjRj", "llvm::StringRef::getAsInteger(unsigned int, unsigned int&) const"},
    {"_ZNK4llvm9StringRef12getAsIntegerEjRNS_5APIntE", "llvm::StringRef::getAsInteger(unsigned int, llvm::APInt&) const"},
    {"_ZN4llvm14TargetRegistry5beginEv", "llvm::TargetRegistry::begin()"},
    {"_ZN4llvm14TargetRegistry12lookupTargetERKSsRSs", "llvm::TargetRegistry::lookupTarget(std::string const&, std::string&)"},
    {"_ZN4llvm14TargetRegistry14RegisterTargetERNS_6TargetEPKcS4_PFjRKSsEb", "llvm::TargetRegistry::RegisterTarget(llvm::Target&, char const*, char const*, unsigned int (*)(std::string const&), bool)"},
    {"_ZN4llvm14TargetRegistry22getClosestTargetForJITERSs", "llvm::TargetRegistry::getClosestTargetForJIT(std::string&)"},
    {"_ZN4llvm3sys15ThreadLocalImplC1Ev", "llvm::sys::ThreadLocalImpl::ThreadLocalImpl()"},
    {"_ZN4llvm3sys15ThreadLocalImplD0Ev", "llvm::sys::ThreadLocalImpl::~ThreadLocalImpl()"},
    {"_ZN4llvm3sys15ThreadLocalImplD1Ev", "llvm::sys::ThreadLocalImpl::~ThreadLocalImpl()"},
    {"_ZN4llvm3sys15ThreadLocalImplD2Ev", "llvm::sys::ThreadLocalImpl::~ThreadLocalImpl()"},
    {"_ZN4llvm3sys15ThreadLocalImpl11setInstanceEPKv", "llvm::sys::ThreadLocalImpl::setInstance(void const*)"},
    {"_ZN4llvm3sys15ThreadLocalImpl11getInstanceEv", "llvm::sys::ThreadLocalImpl::getInstance()"},
    {"_ZN4llvm3sys15ThreadLocalImpl14removeInstanceEv", "llvm::sys::ThreadLocalImpl::removeInstance()"},
    {"_ZN4llvm24llvm_start_multithreadedEv", "llvm::llvm_start_multithreaded()"},
    {"_ZN4llvm23llvm_stop_multithreadedEv", "llvm::llvm_stop_multithreaded()"},
    {"_ZN4llvm22llvm_execute_on_threadEPFvPvES0_j", "llvm::llvm_execute_on_thread(void (*)(void*), void*, unsigned int)"},
    {"_Z24ExecuteOnThread_DispatchPv", "ExecuteOnThread_Dispatch(void*)"},
    {"_ZN4llvm3sys9TimeValue9normalizeEv", "llvm::sys::TimeValue::normalize()"},
    {"_ZNK4llvm3sys9TimeValue3strEv", "llvm::sys::TimeValue::str() const"},
    {"_ZN4llvm3sys9TimeValue3nowEv", "llvm::sys::TimeValue::now()"},
    {"_ZN4llvm20CreateInfoOutputFileEv", "llvm::CreateInfoOutputFile()"},
    {"_ZN4llvm5Timer4initENS_9StringRefE", "llvm::Timer::init(llvm::StringRef)"},
    {"_ZN4llvm10TimerGroup8addTimerERNS_5TimerE", "llvm::TimerGroup::addTimer(llvm::Timer&)"},
    {"_ZN4llvm5Timer4initENS_9StringRefERNS_10TimerGroupE", "llvm::Timer::init(llvm::StringRef, llvm::TimerGroup&)"},
    {"_ZN4llvm5TimerD1Ev", "llvm::Timer::~Timer()"},
    {"_ZN4llvm5TimerD2Ev", "llvm::Timer::~Timer()"},
    {"_ZN4llvm10TimerGroup11removeTimerERNS_5TimerE", "llvm::TimerGroup::removeTimer(llvm::Timer&)"},
    {"_ZN4llvm10TimeRecord14getCurrentTimeEb", "llvm::TimeRecord::getCurrentTime(bool)"},
    {"_ZN4llvm5Timer10startTimerEv", "llvm::Timer::startTimer()"},
    {"_ZN4llvm5Timer9stopTimerEv", "llvm::Timer::stopTimer()"},
    {"_ZNK4llvm10TimeRecord5printERKS0_RNS_11raw_ostreamE", "llvm::TimeRecord::print(llvm::TimeRecord const&, llvm::raw_ostream&) const"},
    {"_Z8printValddRN4llvm11raw_ostreamE", "printVal(double, double, llvm::raw_ostream&)"},
    {"_ZN4llvm16NamedRegionTimerC1ENS_9StringRefEb", "llvm::NamedRegionTimer::NamedRegionTimer(llvm::StringRef, bool)"},
    {"_ZN4llvm16NamedRegionTimerC2ENS_9StringRefEb", "llvm::NamedRegionTimer::NamedRegionTimer(llvm::StringRef, bool)"},
    {"_ZN4llvm16NamedRegionTimerC1ENS_9StringRefES1_b", "llvm::NamedRegionTimer::NamedRegionTimer(llvm::StringRef, llvm::StringRef, bool)"},
    {"_ZN4llvm16NamedRegionTimerC2ENS_9StringRefES1_b", "llvm::NamedRegionTimer::NamedRegionTimer(llvm::StringRef, llvm::StringRef, bool)"},
    {"_ZN4llvm10TimerGroupC1ENS_9StringRefE", "llvm::TimerGroup::TimerGroup(llvm::StringRef)"},
    {"_ZN4llvm10TimerGroupC2ENS_9StringRefE", "llvm::TimerGroup::TimerGroup(llvm::StringRef)"},
    {"_ZN4llvm10TimerGroupD1Ev", "llvm::TimerGroup::~TimerGroup()"},
    {"_ZN4llvm10TimerGroupD2Ev", "llvm::TimerGroup::~TimerGroup()"},
    {"_ZN4llvm10TimerGroup17PrintQueuedTimersERNS_11raw_ostreamE", "llvm::TimerGroup::PrintQueuedTimers(llvm::raw_ostream&)"},
    {"_ZN4llvm10TimerGroup5printERNS_11raw_ostreamE", "llvm::TimerGroup::print(llvm::raw_ostream&)"},
    {"_ZN4llvm10TimerGroup8printAllERNS_11raw_ostreamE", "llvm::TimerGroup::printAll(llvm::raw_ostream&)"},
    {"_ZN4llvm14object_creatorIN12_GLOBAL__N_112Name2PairMapEEEPvv", "void* llvm::object_creator<(anonymous namespace)::Name2PairMap>()"},
    {"_ZN4llvm14object_deleterIN12_GLOBAL__N_112Name2PairMapEE4callEPv", "llvm::object_deleter<(anonymous namespace)::Name2PairMap>::call(void*)"},
    {"_ZN4llvm2cl3optIbLb0ENS0_6parserIbEEED1Ev", "llvm::cl::opt<bool, false, llvm::cl::parser<bool> >::~opt()"},
    {"_ZN4llvm2cl3optISsLb1ENS0_6parserISsEEED1Ev", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::~opt()"},
    {"_ZN4llvm14format_object1IxED1Ev", "llvm::format_object1<long long>::~format_object1()"},
    {"_ZN4llvm14format_object1IdED1Ev", "llvm::format_object1<double>::~format_object1()"},
    {"_ZSt16__introsort_loopIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEElEvT_SB_T0_", "void std::__introsort_loop<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long>(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long)"},
    {"_ZSt22__final_insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEEEvT_SB_", "void std::__final_insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >)"},
    {"_ZSt16__insertion_sortIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEEEvT_SB_", "void std::__insertion_sort<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >)"},
    {"_ZSt25__unguarded_linear_insertIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEES5_EvT_T0_", "void std::__unguarded_linear_insert<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, std::pair<llvm::TimeRecord, std::string> >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, std::pair<llvm::TimeRecord, std::string>)"},
    {"_ZSt21__unguarded_partitionIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEES5_ET_SB_SB_T0_", "__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > std::__unguarded_partition<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, std::pair<llvm::TimeRecord, std::string> >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, std::pair<llvm::TimeRecord, std::string>)"},
    {"_ZSt8__medianISt4pairIN4llvm10TimeRecordESsEERKT_S6_S6_S6_", "std::pair<llvm::TimeRecord, std::string> const& std::__median<std::pair<llvm::TimeRecord, std::string> >(std::pair<llvm::TimeRecord, std::string> const&, std::pair<llvm::TimeRecord, std::string> const&, std::pair<llvm::TimeRecord, std::string> const&)"},
    {"_ZSt13__heap_selectIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEEEvT_SB_SB_", "void std::__heap_select<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >)"},
    {"_ZSt8pop_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEEEvT_SB_", "void std::pop_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >)"},
    {"_ZSt13__adjust_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEElS5_EvT_T0_SC_T1_", "void std::__adjust_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long, std::pair<llvm::TimeRecord, std::string> >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long, long, std::pair<llvm::TimeRecord, std::string>)"},
    {"_ZSt11__push_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEElS5_EvT_T0_SC_T1_", "void std::__push_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long, std::pair<llvm::TimeRecord, std::string> >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, long, long, std::pair<llvm::TimeRecord, std::string>)"},
    {"_ZSt9make_heapIN9__gnu_cxx17__normal_iteratorIPSt4pairIN4llvm10TimeRecordESsESt6vectorIS5_SaIS5_EEEEEvT_SB_", "void std::make_heap<__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > > >(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, __gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >)"},
    {"_ZNSt6vectorISt4pairIN4llvm10TimeRecordESsESaIS3_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS3_S5_EERKS3_", "std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > >::_M_insert_aux(__gnu_cxx::__normal_iterator<std::pair<llvm::TimeRecord, std::string>*, std::vector<std::pair<llvm::TimeRecord, std::string>, std::allocator<std::pair<llvm::TimeRecord, std::string> > > >, std::pair<llvm::TimeRecord, std::string> const&)"},
    {"_ZN4llvm9StringMapISt4pairIPNS_10TimerGroupENS0_INS_5TimerENS_15MallocAllocatorEEEES5_ED2Ev", "llvm::StringMap<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >, llvm::MallocAllocator>::~StringMap()"},
    {"_ZN4llvm9StringMapINS_5TimerENS_15MallocAllocatorEE5clearEv", "llvm::StringMap<llvm::Timer, llvm::MallocAllocator>::clear()"},
    {"_ZN4llvm14object_creatorINS_3sys10SmartMutexILb1EEEEEPvv", "void* llvm::object_creator<llvm::sys::SmartMutex<true> >()"},
    {"_ZN4llvm14object_deleterINS_3sys10SmartMutexILb1EEEE4callEPv", "llvm::object_deleter<llvm::sys::SmartMutex<true> >::call(void*)"},
    {"_ZNK4llvm14format_object1IxE7snprintEPcj", "llvm::format_object1<long long>::snprint(char*, unsigned int) const"},
    {"_ZN4llvm14format_object1IxED0Ev", "llvm::format_object1<long long>::~format_object1()"},
    {"_ZN4llvm18format_object_baseD1Ev", "llvm::format_object_base::~format_object_base()"},
    {"_ZN4llvm18format_object_baseD0Ev", "llvm::format_object_base::~format_object_base()"},
    {"_ZNK4llvm14format_object1IdE7snprintEPcj", "llvm::format_object1<double>::snprint(char*, unsigned int) const"},
    {"_ZN4llvm14format_object1IdED0Ev", "llvm::format_object1<double>::~format_object1()"},
    {"_ZSt6__findIN9__gnu_cxx17__normal_iteratorIPPN4llvm5TimerESt6vectorIS4_SaIS4_EEEES4_ET_SA_SA_RKT0_St26random_access_iterator_tag", "__gnu_cxx::__normal_iterator<llvm::Timer**, std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > > std::__find<__gnu_cxx::__normal_iterator<llvm::Timer**, std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >, llvm::Timer*>(__gnu_cxx::__normal_iterator<llvm::Timer**, std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >, __gnu_cxx::__normal_iterator<llvm::Timer**, std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >, llvm::Timer* const&, std::random_access_iterator_tag)"},
    {"_ZNSt6vectorIPN4llvm5TimerESaIS2_EE13_M_insert_auxEN9__gnu_cxx17__normal_iteratorIPS2_S4_EERKS2_", "std::vector<llvm::Timer*, std::allocator<llvm::Timer*> >::_M_insert_aux(__gnu_cxx::__normal_iterator<llvm::Timer**, std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >, llvm::Timer* const&)"},
    {"_ZN4llvm14object_creatorISt6vectorIPNS_5TimerESaIS3_EEEEPvv", "void* llvm::object_creator<std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >()"},
    {"_ZN4llvm14object_deleterISt6vectorIPNS_5TimerESaIS3_EEE4callEPv", "llvm::object_deleter<std::vector<llvm::Timer*, std::allocator<llvm::Timer*> > >::call(void*)"},
    {"_ZN4llvm2cl3optISsLb1ENS0_6parserISsEEE16handleOccurrenceEjNS_9StringRefES5_", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::handleOccurrence(unsigned int, llvm::StringRef, llvm::StringRef)"},
    {"_ZNK4llvm2cl3optISsLb1ENS0_6parserISsEEE27getValueExpectedFlagDefaultEv", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::getValueExpectedFlagDefault() const"},
    {"_ZNK4llvm2cl3optISsLb1ENS0_6parserISsEEE14getOptionWidthEv", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::getOptionWidth() const"},
    {"_ZNK4llvm2cl3optISsLb1ENS0_6parserISsEEE15printOptionInfoEm", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::printOptionInfo(unsigned long) const"},
    {"_ZN4llvm2cl3optISsLb1ENS0_6parserISsEEE19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl3optISsLb1ENS0_6parserISsEEED0Ev", "llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >::~opt()"},
    {"_ZN4llvm2cl6parserISsED1Ev", "llvm::cl::parser<std::string>::~parser()"},
    {"_ZN4llvm2cl6parserISsED0Ev", "llvm::cl::parser<std::string>::~parser()"},
    {"_ZNK4llvm2cl6parserISsE12getValueNameEv", "llvm::cl::parser<std::string>::getValueName() const"},
    {"_ZN4llvm2cl17basic_parser_implD1Ev", "llvm::cl::basic_parser_impl::~basic_parser_impl()"},
    {"_ZN4llvm2cl17basic_parser_implD0Ev", "llvm::cl::basic_parser_impl::~basic_parser_impl()"},
    {"_ZNK4llvm2cl17basic_parser_impl12getValueNameEv", "llvm::cl::basic_parser_impl::getValueName() const"},
    {"_ZNK4llvm2cl6Option27getValueExpectedFlagDefaultEv", "llvm::cl::Option::getValueExpectedFlagDefault() const"},
    {"_ZN4llvm2cl6Option19getExtraOptionNamesERNS_15SmallVectorImplIPKcEE", "llvm::cl::Option::getExtraOptionNames(llvm::SmallVectorImpl<char const*>&)"},
    {"_ZN4llvm2cl6OptionD1Ev", "llvm::cl::Option::~Option()"},
    {"_ZN4llvm2cl6OptionD0Ev", "llvm::cl::Option::~Option()"},
    {"_ZN4llvm2cl6parserIbED1Ev", "llvm::cl::parser<bool>::~parser()"},
    {"_ZN4llvm2cl6parserIbED0Ev", "llvm::cl::parser<bool>::~parser()"},
    {"_ZNK4llvm2cl6parserIbE12getValueNameEv", "llvm::cl::parser<bool>::getValueName() const"},
    {"_ZN4llvm9StringMapINS_5TimerENS_15MallocAllocatorEE16GetOrCreateValueIS1_EERNS_14StringMapEntryIS1_EENS_9StringRefET_", "llvm::StringMapEntry<llvm::Timer>& llvm::StringMap<llvm::Timer, llvm::MallocAllocator>::GetOrCreateValue<llvm::Timer>(llvm::StringRef, llvm::Timer)"},
    {"_ZN4llvm9StringMapISt4pairIPNS_10TimerGroupENS0_INS_5TimerENS_15MallocAllocatorEEEES5_E16GetOrCreateValueIS7_EERNS_14StringMapEntryIS7_EENS_9StringRefET_", "llvm::StringMapEntry<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> > >& llvm::StringMap<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >, llvm::MallocAllocator>::GetOrCreateValue<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> > >(llvm::StringRef, std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >)"},
    {"_ZN4llvm14StringMapEntryISt4pairIPNS_10TimerGroupENS_9StringMapINS_5TimerENS_15MallocAllocatorEEEEE6CreateIS6_S8_EEPS9_PKcSD_RT_T0_", "llvm::StringMapEntry<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> > >* llvm::StringMapEntry<std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> > >::Create<llvm::MallocAllocator, std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> > >(char const*, char const*, llvm::MallocAllocator&, std::pair<llvm::TimerGroup*, llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >)"},
    {"_ZN4llvm14object_creatorINS_9StringMapINS_5TimerENS_15MallocAllocatorEEEEEPvv", "void* llvm::object_creator<llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >()"},
    {"_ZN4llvm14object_deleterINS_9StringMapINS_5TimerENS_15MallocAllocatorEEEE4callEPv", "llvm::object_deleter<llvm::StringMap<llvm::Timer, llvm::MallocAllocator> >::call(void*)"},
    {"_ZN4llvm14object_creatorISsEEPvv", "void* llvm::object_creator<std::string>()"},
    {"_ZN4llvm14object_deleterISsE4callEPv", "llvm::object_deleter<std::string>::call(void*)"},
    {"_ZN4llvm6Triple15getArchTypeNameENS0_8ArchTypeE", "llvm::Triple::getArchTypeName(llvm::Triple::ArchType)"},
    {"_ZN4llvm6Triple17getArchTypePrefixENS0_8ArchTypeE", "llvm::Triple::getArchTypePrefix(llvm::Triple::ArchType)"},
    {"_ZN4llvm6Triple17getVendorTypeNameENS0_10VendorTypeE", "llvm::Triple::getVendorTypeName(llvm::Triple::VendorType)"},
    {"_ZN4llvm6Triple13getOSTypeNameENS0_6OSTypeE", "llvm::Triple::getOSTypeName(llvm::Triple::OSType)"},
    {"_ZN4llvm6Triple22getEnvironmentTypeNameENS0_15EnvironmentTypeE", "llvm::Triple::getEnvironmentTypeName(llvm::Triple::EnvironmentType)"},
    {"_ZN4llvm6Triple22getArchTypeForLLVMNameENS_9StringRefE", "llvm::Triple::getArchTypeForLLVMName(llvm::StringRef)"},
    {"_ZN4llvm6Triple28getArchTypeForDarwinArchNameENS_9StringRefE", "llvm::Triple::getArchTypeForDarwinArchName(llvm::StringRef)"},
    {"_ZN4llvm6Triple23getArchNameForAssemblerEv", "llvm::Triple::getArchNameForAssembler()"},
    {"_ZNK4llvm6Triple11getArchNameEv", "llvm::Triple::getArchName() const"},
    {"_ZN4llvm6Triple9ParseArchENS_9StringRefE", "llvm::Triple::ParseArch(llvm::StringRef)"},
    {"_ZN4llvm6Triple11ParseVendorENS_9StringRefE", "llvm::Triple::ParseVendor(llvm::StringRef)"},
    {"_ZN4llvm6Triple7ParseOSENS_9StringRefE", "llvm::Triple::ParseOS(llvm::StringRef)"},
    {"_ZN4llvm6Triple16ParseEnvironmentENS_9StringRefE", "llvm::Triple::ParseEnvironment(llvm::StringRef)"},
    {"_ZNK4llvm6Triple5ParseEv", "llvm::Triple::Parse() const"},
    {"_ZNK4llvm6Triple13getVendorNameEv", "llvm::Triple::getVendorName() const"},
    {"_ZNK4llvm6Triple9getOSNameEv", "llvm::Triple::getOSName() const"},
    {"_ZNK4llvm6Triple18getEnvironmentNameEv", "llvm::Triple::getEnvironmentName() const"},
    {"_ZN4llvm6Triple9normalizeENS_9StringRefE", "llvm::Triple::normalize(llvm::StringRef)"},
    {"_ZNK4llvm6Triple23getOSAndEnvironmentNameEv", "llvm::Triple::getOSAndEnvironmentName() const"},
    {"_ZNK4llvm6Triple15getDarwinNumberERjS1_S1_", "llvm::Triple::getDarwinNumber(unsigned int&, unsigned int&, unsigned int&) const"},
    {"_ZN4llvm6Triple9setTripleERKNS_5TwineE", "llvm::Triple::setTriple(llvm::Twine const&)"},
    {"_ZN4llvm6Triple7setArchENS0_8ArchTypeE", "llvm::Triple::setArch(llvm::Triple::ArchType)"},
    {"_ZN4llvm6Triple11setArchNameENS_9StringRefE", "llvm::Triple::setArchName(llvm::StringRef)"},
    {"_ZN4llvm6Triple9setVendorENS0_10VendorTypeE", "llvm::Triple::setVendor(llvm::Triple::VendorType)"},
    {"_ZN4llvm6Triple13setVendorNameENS_9StringRefE", "llvm::Triple::setVendorName(llvm::StringRef)"},
    {"_ZN4llvm6Triple5setOSENS0_6OSTypeE", "llvm::Triple::setOS(llvm::Triple::OSType)"},
    {"_ZN4llvm6Triple9setOSNameENS_9StringRefE", "llvm::Triple::setOSName(llvm::StringRef)"},
    {"_ZN4llvm6Triple14setEnvironmentENS0_15EnvironmentTypeE", "llvm::Triple::setEnvironment(llvm::Triple::EnvironmentType)"},
    {"_ZN4llvm6Triple18setEnvironmentNameENS_9StringRefE", "llvm::Triple::setEnvironmentName(llvm::StringRef)"},
    {"_ZN4llvm6Triple23setOSAndEnvironmentNameENS_9StringRefE", "llvm::Triple::setOSAndEnvironmentName(llvm::StringRef)"},
    {"_ZNK4llvm5Twine3strEv", "llvm::Twine::str() const"},
    {"_ZNK4llvm5Twine11toStringRefERNS_15SmallVectorImplIcEE", "llvm::Twine::toStringRef(llvm::SmallVectorImpl<char>&) const"},
    {"_ZNK4llvm5Twine8toVectorERNS_15SmallVectorImplIcEE", "llvm::Twine::toVector(llvm::SmallVectorImpl<char>&) const"},
    {"_ZNK4llvm5Twine5printERNS_11raw_ostreamE", "llvm::Twine::print(llvm::raw_ostream&) const"},
    {"_ZNK4llvm5Twine25toNullTerminatedStringRefERNS_15SmallVectorImplIcEE", "llvm::Twine::toNullTerminatedStringRef(llvm::SmallVectorImpl<char>&) const"},
    {"_ZNK4llvm5Twine13printOneChildERNS_11raw_ostreamEPKvNS0_8NodeKindE", "llvm::Twine::printOneChild(llvm::raw_ostream&, void const*, llvm::Twine::NodeKind) const"},
    {"_ZNK4llvm5Twine17printOneChildReprERNS_11raw_ostreamEPKvNS0_8NodeKindE", "llvm::Twine::printOneChildRepr(llvm::raw_ostream&, void const*, llvm::Twine::NodeKind) const"},
    {"_ZNK4llvm5Twine9printReprERNS_11raw_ostreamE", "llvm::Twine::printRepr(llvm::raw_ostream&) const"},
    {"_ZNK4llvm5Twine4dumpEv", "llvm::Twine::dump() const"},
    {"_ZNK4llvm5Twine8dumpReprEv", "llvm::Twine::dumpRepr() const"},
    {"_ZN4llvm3sys17RunningOnValgrindEv", "llvm::sys::RunningOnValgrind()"},
    {"_ZN4llvm3sys27ValgrindDiscardTranslationsEPKvm", "llvm::sys::ValgrindDiscardTranslations(void const*, unsigned long)"},
    {"_ZN4llvm11raw_ostreamD0Ev", "llvm::raw_ostream::~raw_ostream()"},
    {"_ZN4llvm11raw_ostreamD1Ev", "llvm::raw_ostream::~raw_ostream()"},
    {"_ZN4llvm11raw_ostreamD2Ev", "llvm::raw_ostream::~raw_ostream()"},
    {"_ZN4llvm11raw_ostream6handleEv", "llvm::raw_ostream::handle()"},
    {"_ZNK4llvm11raw_ostream21preferred_buffer_sizeEv", "llvm::raw_ostream::preferred_buffer_size() const"},
    {"_ZN4llvm11raw_ostream11SetBufferedEv", "llvm::raw_ostream::SetBuffered()"},
    {"_ZN4llvm11raw_ostream16SetBufferAndModeEPcmNS0_10BufferKindE", "llvm::raw_ostream::SetBufferAndMode(char*, unsigned long, llvm::raw_ostream::BufferKind)"},
    {"_ZN4llvm11raw_ostreamlsEm", "llvm::raw_ostream::operator<<(unsigned long)"},
    {"_ZN4llvm11raw_ostream5writeEPKcm", "llvm::raw_ostream::write(char const*, unsigned long)"},
    {"_ZN4llvm11raw_ostreamlsEl", "llvm::raw_ostream::operator<<(long)"},
    {"_ZN4llvm11raw_ostreamlsEy", "llvm::raw_ostream::operator<<(unsigned long long)"},
    {"_ZN4llvm11raw_ostreamlsEx", "llvm::raw_ostream::operator<<(long long)"},
    {"_ZN4llvm11raw_ostream9write_hexEy", "llvm::raw_ostream::write_hex(unsigned long long)"},
    {"_ZN4llvm11raw_ostream13write_escapedENS_9StringRefEb", "llvm::raw_ostream::write_escaped(llvm::StringRef, bool)"},
    {"_ZN4llvm11raw_ostreamlsEPKv", "llvm::raw_ostream::operator<<(void const*)"},
    {"_ZN4llvm11raw_ostreamlsEd", "llvm::raw_ostream::operator<<(double)"},
    {"_ZN4llvm11raw_ostreamlsERKNS_18format_object_baseE", "llvm::raw_ostream::operator<<(llvm::format_object_base const&)"},
    {"_ZN4llvm11raw_ostream14flush_nonemptyEv", "llvm::raw_ostream::flush_nonempty()"},
    {"_ZN4llvm11raw_ostream5writeEh", "llvm::raw_ostream::write(unsigned char)"},
    {"_ZN4llvm11raw_ostream14copy_to_bufferEPKcm", "llvm::raw_ostream::copy_to_buffer(char const*, unsigned long)"},
    {"_ZN4llvm11raw_ostream6indentEj", "llvm::raw_ostream::indent(unsigned int)"},
    {"_ZN4llvm18format_object_base4homeEv", "llvm::format_object_base::home()"},
    {"_ZN4llvm14raw_fd_ostreamC1EPKcRSsj", "llvm::raw_fd_ostream::raw_fd_ostream(char const*, std::string&, unsigned int)"},
    {"_ZN4llvm14raw_fd_ostreamC2EPKcRSsj", "llvm::raw_fd_ostream::raw_fd_ostream(char const*, std::string&, unsigned int)"},
    {"_ZN4llvm14raw_fd_ostreamC1Eibb", "llvm::raw_fd_ostream::raw_fd_ostream(int, bool, bool)"},
    {"_ZN4llvm14raw_fd_ostreamC2Eibb", "llvm::raw_fd_ostream::raw_fd_ostream(int, bool, bool)"},
    {"_ZN4llvm14raw_fd_ostreamD0Ev", "llvm::raw_fd_ostream::~raw_fd_ostream()"},
    {"_ZN4llvm14raw_fd_ostreamD1Ev", "llvm::raw_fd_ostream::~raw_fd_ostream()"},
    {"_ZN4llvm14raw_fd_ostreamD2Ev", "llvm::raw_fd_ostream::~raw_fd_ostream()"},
    {"_ZN4llvm14raw_fd_ostream10write_implEPKcm", "llvm::raw_fd_ostream::write_impl(char const*, unsigned long)"},
    {"_ZN4llvm14raw_fd_ostream5closeEv", "llvm::raw_fd_ostream::close()"},
    {"_ZN4llvm14raw_fd_ostream4seekEy", "llvm::raw_fd_ostream::seek(unsigned long long)"},
    {"_ZNK4llvm14raw_fd_ostream21preferred_buffer_sizeEv", "llvm::raw_fd_ostream::preferred_buffer_size() const"},
    {"_ZN4llvm14raw_fd_ostream11changeColorENS_11raw_ostream6ColorsEbb", "llvm::raw_fd_ostream::changeColor(llvm::raw_ostream::Colors, bool, bool)"},
    {"_ZN4llvm14raw_fd_ostream10resetColorEv", "llvm::raw_fd_ostream::resetColor()"},
    {"_ZNK4llvm14raw_fd_ostream12is_displayedEv", "llvm::raw_fd_ostream::is_displayed() const"},
    {"_ZN4llvm4outsEv", "llvm::outs()"},
    {"_ZN4llvm4errsEv", "llvm::errs()"},
    {"_ZN4llvm5nullsEv", "llvm::nulls()"},
    {"_ZN4llvm16raw_null_ostreamD1Ev", "llvm::raw_null_ostream::~raw_null_ostream()"},
    {"_ZN4llvm18raw_string_ostreamD0Ev", "llvm::raw_string_ostream::~raw_string_ostream()"},
    {"_ZN4llvm18raw_string_ostreamD1Ev", "llvm::raw_string_ostream::~raw_string_ostream()"},
    {"_ZN4llvm18raw_string_ostreamD2Ev", "llvm::raw_string_ostream::~raw_string_ostream()"},
    {"_ZN4llvm18raw_string_ostream10write_implEPKcm", "llvm::raw_string_ostream::write_impl(char const*, unsigned long)"},
    {"_ZN4llvm19raw_svector_ostreamC1ERNS_15SmallVectorImplIcEE", "llvm::raw_svector_ostream::raw_svector_ostream(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm19raw_svector_ostreamC2ERNS_15SmallVectorImplIcEE", "llvm::raw_svector_ostream::raw_svector_ostream(llvm::SmallVectorImpl<char>&)"},
    {"_ZN4llvm19raw_svector_ostreamD0Ev", "llvm::raw_svector_ostream::~raw_svector_ostream()"},
    {"_ZN4llvm19raw_svector_ostreamD1Ev", "llvm::raw_svector_ostream::~raw_svector_ostream()"},
    {"_ZN4llvm19raw_svector_ostreamD2Ev", "llvm::raw_svector_ostream::~raw_svector_ostream()"},
    {"_ZN4llvm19raw_svector_ostream6resyncEv", "llvm::raw_svector_ostream::resync()"},
    {"_ZN4llvm19raw_svector_ostream10write_implEPKcm", "llvm::raw_svector_ostream::write_impl(char const*, unsigned long)"},
    {"_ZNK4llvm19raw_svector_ostream11current_posEv", "llvm::raw_svector_ostream::current_pos() const"},
    {"_ZN4llvm19raw_svector_ostream3strEv", "llvm::raw_svector_ostream::str()"},
    {"_ZN4llvm16raw_null_ostreamD0Ev", "llvm::raw_null_ostream::~raw_null_ostream()"},
    {"_ZN4llvm16raw_null_ostreamD2Ev", "llvm::raw_null_ostream::~raw_null_ostream()"},
    {"_ZN4llvm16raw_null_ostream10write_implEPKcm", "llvm::raw_null_ostream::write_impl(char const*, unsigned long)"},
    {"_ZNK4llvm16raw_null_ostream11current_posEv", "llvm::raw_null_ostream::current_pos() const"},
    {"_ZNK4llvm14raw_fd_ostream11current_posEv", "llvm::raw_fd_ostream::current_pos() const"},
    {"_ZN4llvm14error_categoryC1Ev", "llvm::error_category::error_category()"},
    {"_ZN4llvm14error_categoryC2Ev", "llvm::error_category::error_category()"},
    {"_ZN4llvm14error_categoryD0Ev", "llvm::error_category::~error_category()"},
    {"_ZN4llvm14error_categoryD1Ev", "llvm::error_category::~error_category()"},
    {"_ZN4llvm14error_categoryD2Ev", "llvm::error_category::~error_category()"},
    {"_ZNK4llvm14error_category23default_error_conditionEi", "llvm::error_category::default_error_condition(int) const"},
    {"_ZNK4llvm14error_category10equivalentEiRKNS_15error_conditionE", "llvm::error_category::equivalent(int, llvm::error_condition const&) const"},
    {"_ZNK4llvm14error_category10equivalentERKNS_10error_codeEi", "llvm::error_category::equivalent(llvm::error_code const&, int) const"},
    {"_ZNK4llvm11_do_message7messageEi", "llvm::_do_message::message(int) const"},
    {"_ZNK4llvm23_generic_error_category4nameEv", "llvm::_generic_error_category::name() const"},
    {"_ZNK4llvm23_generic_error_category7messageEi", "llvm::_generic_error_category::message(int) const"},
    {"_ZN4llvm16generic_categoryEv", "llvm::generic_category()"},
    {"_ZNK4llvm22_system_error_category4nameEv", "llvm::_system_error_category::name() const"},
    {"_ZN4llvm14posix_categoryEv", "llvm::posix_category()"},
    {"_ZNK4llvm15error_condition7messageEv", "llvm::error_condition::message() const"},
    {"_ZNK4llvm10error_code7messageEv", "llvm::error_code::message() const"},
    {"_ZNK4llvm22_system_error_category7messageEi", "llvm::_system_error_category::message(int) const"},
    {"_ZNK4llvm22_system_error_category23default_error_conditionEi", "llvm::_system_error_category::default_error_condition(int) const"},
    {"_ZN4llvm23_generic_error_categoryD1Ev", "llvm::_generic_error_category::~_generic_error_category()"},
    {"_ZN4llvm22_system_error_categoryD1Ev", "llvm::_system_error_category::~_system_error_category()"},
    {"_ZN4llvm11_do_messageD1Ev", "llvm::_do_message::~_do_message()"},
    {"_ZN4llvm11_do_messageD0Ev", "llvm::_do_message::~_do_message()"},
    {"_ZN4llvm23_generic_error_categoryD0Ev", "llvm::_generic_error_category::~_generic_error_category()"},
    {"_ZN4llvm22_system_error_categoryD0Ev", "llvm::_system_error_category::~_system_error_category()"},
    {"_ZZN4llvm3sys4Path21GetTemporaryDirectoryEPSsE8pathname", "llvm::sys::Path::GetTemporaryDirectory(std::string*)::pathname"},
    {"_ZZN5clang6Parser30ParseTemplateTemplateParameterEjjE7EndToks", "clang::Parser::ParseTemplateTemplateParameter(unsigned int, unsigned int)::EndToks"},
    {"_ZZN5clang4Sema27SemaBuiltinAtomicOverloadedENS_12ActionResultIPNS_4ExprELb1EEEE14BuiltinIndices", "clang::Sema::SemaBuiltinAtomicOverloaded(clang::ActionResult<clang::Expr*, true>)::BuiltinIndices"},
    {"_ZZN5clang21GetConversionCategoryENS_22ImplicitConversionKindEE8Category", "clang::GetConversionCategory(clang::ImplicitConversionKind)::Category"},
    {"_ZZN5clang17GetConversionRankENS_22ImplicitConversionKindEE4Rank", "clang::GetConversionRank(clang::ImplicitConversionKind)::Rank"},
    {"_ZZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder17getArithmeticTypeEjE15ArithmeticTypes", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::getArithmeticType(unsigned int)::ArithmeticTypes"},
    {"_ZZN5clang12_GLOBAL__N_130BuiltinOperatorOverloadBuilder29getUsualArithmeticConversionsEjjE16ConversionsTable", "clang::(anonymous namespace)::BuiltinOperatorOverloadBuilder::getUsualArithmeticConversions(unsigned int, unsigned int)::ConversionsTable"},
    {"_ZZ15GetCFNumberSizeRN5clang10ASTContextEyE9FixedSize", "GetCFNumberSize(clang::ASTContext&, unsigned long long)::FixedSize"},
    {"_ZZN5clang15DeclarationName21getUsingDirectiveNameEvE9UDirExtra", "clang::DeclarationName::getUsingDirectiveName()::UDirExtra"},
    {"_ZZN5clang14BinaryOperator21getOverloadedOperatorENS_18BinaryOperatorKindEE7OverOps", "clang::BinaryOperator::getOverloadedOperator(clang::BinaryOperatorKind)::OverOps"},
    {"_ZZ15EncodeUCNEscapeRPKcS0_RPcRbN5clang13FullSourceLocEbPNS5_10DiagnosticERKNS5_11LangOptionsEE13firstByteMark", "EncodeUCNEscape(char const*&, char const*, char*&, bool&, clang::FullSourceLoc, bool, clang::Diagnostic*, clang::LangOptions const&)::firstByteMark"},
    {"_ZZN5clang10TargetInfo9setCXXABIERKSsE7Unknown", "clang::TargetInfo::setCXXABI(std::string const&)::Unknown"},
    {"_ZZNK12_GLOBAL__N_113X86AsmBackend12WriteNopDataEyPN4llvm14MCObjectWriterEE4Nops", "(anonymous namespace)::X86AsmBackend::WriteNopData(unsigned long long, llvm::MCObjectWriter*) const::Nops"},
    {"_ZZN4llvm13X86AsmPrinter16EmitEndOfAsmFileERNS_6ModuleEE8HltInsts", "llvm::X86AsmPrinter::EmitEndOfAsmFile(llvm::Module&)::HltInsts"},
    {"_ZZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE16emitMemModRMByteERKNS1_12MachineInstrEjjlE7SSTable", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::emitMemModRMByte(llvm::MachineInstr const&, unsigned int, unsigned int, long)::SSTable"},
    {"_ZZ14RetCC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList7", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList7"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ11CC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList7", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList7"},
    {"_ZZ13CC_X86_32_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_FastCCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_FastCC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_FastCCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_32_FastCC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ18CC_X86_32_FastCalljN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_FastCall(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ13CC_X86_64_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_64_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ13CC_X86_64_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_64_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ22findDeadCallerSavedRegRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEERKNS_18TargetRegisterInfoEbE20CallerSavedRegs32Bit", "findDeadCallerSavedReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::TargetRegisterInfo const&, bool)::CallerSavedRegs32Bit"},
    {"_ZZ22findDeadCallerSavedRegRN4llvm17MachineBasicBlockERNS_14ilist_iteratorINS_12MachineInstrEEERKNS_18TargetRegisterInfoEbE20CallerSavedRegs64Bit", "findDeadCallerSavedReg(llvm::MachineBasicBlock&, llvm::ilist_iterator<llvm::MachineInstr>&, llvm::TargetRegisterInfo const&, bool)::CallerSavedRegs64Bit"},
    {"_ZZN12_GLOBAL__N_115X86DAGToDAGISel10SelectCodeEPN4llvm6SDNodeEE12MatcherTable", "(anonymous namespace)::X86DAGToDAGISel::SelectCode(llvm::SDNode*)::MatcherTable"},
    {"_ZZNK4llvm17X86TargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EEE17GPR64ArgRegsWin64", "llvm::X86TargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const::GPR64ArgRegsWin64"},
    {"_ZZNK4llvm17X86TargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EEE17GPR64ArgRegs64Bit", "llvm::X86TargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const::GPR64ArgRegs64Bit"},
    {"_ZZNK4llvm17X86TargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EEE15XMMArgRegs64Bit", "llvm::X86TargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const::XMMArgRegs64Bit"},
    {"_ZZNK4llvm17X86TargetLowering9LowerCallENS_7SDValueES1_NS_11CallingConv2IDEbRbRKNS_15SmallVectorImplINS_3ISD9OutputArgEEERKNS5_IS1_EERKNS5_INS6_8InputArgEEENS_8DebugLocERNS_12SelectionDAGERSB_E10XMMArgRegs", "llvm::X86TargetLowering::LowerCall(llvm::SDValue, llvm::SDValue, llvm::CallingConv::ID, bool, bool&, llvm::SmallVectorImpl<llvm::ISD::OutputArg> const&, llvm::SmallVectorImpl<llvm::SDValue> const&, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const::XMMArgRegs"},
    {"_ZZ11CC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ16CC_X86_32_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList7", "CC_X86_32_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList7"},
    {"_ZZ13CC_X86_32_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_FastCCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_FastCC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_X86_32_FastCCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_32_FastCC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ18CC_X86_32_FastCalljN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_32_FastCall(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ11CC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "CC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ14CC_X86_Win64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "CC_X86_Win64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ13CC_X86_64_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_X86_64_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ13CC_X86_64_GHCjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_X86_64_GHC(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14RetCC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_X86_32_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_32_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList6", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList6"},
    {"_ZZ15RetCC_X86CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList7", "RetCC_X86Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList7"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList4", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList4"},
    {"_ZZ17RetCC_X86_32_FastjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList5", "RetCC_X86_32_Fast(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList5"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14RetCC_X86_64_CjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_X86_64_C(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZN4llvm12X86InstrInfoC1ERNS_16X86TargetMachineEE10OpTbl2Addr", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)::OpTbl2Addr"},
    {"_ZZN4llvm12X86InstrInfoC1ERNS_16X86TargetMachineEE6OpTbl0", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)::OpTbl0"},
    {"_ZZN4llvm12X86InstrInfoC1ERNS_16X86TargetMachineEE6OpTbl1", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)::OpTbl1"},
    {"_ZZN4llvm12X86InstrInfoC1ERNS_16X86TargetMachineEE6OpTbl2", "llvm::X86InstrInfo::X86InstrInfo(llvm::X86TargetMachine&)::OpTbl2"},
    {"_ZZNK12_GLOBAL__N_116X86MCCodeEmitter16EmitMemModRMByteERKN4llvm6MCInstEjjyRjRNS1_11raw_ostreamERNS1_15SmallVectorImplINS1_7MCFixupEEEE7SSTable", "(anonymous namespace)::X86MCCodeEmitter::EmitMemModRMByte(llvm::MCInst const&, unsigned int, unsigned int, unsigned long long, unsigned int&, llvm::raw_ostream&, llvm::SmallVectorImpl<llvm::MCFixup>&) const::SSTable"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE18GhcCalleeSavedRegs", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::GhcCalleeSavedRegs"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE20CalleeSavedRegs32Bit", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegs32Bit"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE22CalleeSavedRegs32EHRet", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegs32EHRet"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE20CalleeSavedRegs64Bit", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegs64Bit"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE22CalleeSavedRegs64EHRet", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegs64EHRet"},
    {"_ZZNK4llvm15X86RegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE20CalleeSavedRegsWin64", "llvm::X86RegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegsWin64"},
    {"_ZZN4llvm17X86ATTInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamEE6OpInfo", "llvm::X86ATTInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)::OpInfo"},
    {"_ZZN4llvm17X86ATTInstPrinter15getRegisterNameEjE12RegAsmOffset", "llvm::X86ATTInstPrinter::getRegisterName(unsigned int)::RegAsmOffset"},
    {"_ZZN4llvm17X86ATTInstPrinter18getInstructionNameEjE13InstAsmOffset", "llvm::X86ATTInstPrinter::getInstructionName(unsigned int)::InstAsmOffset"},
    {"_ZZN4llvm19X86IntelInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamEE6OpInfo", "llvm::X86IntelInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)::OpInfo"},
    {"_ZZN4llvm19X86IntelInstPrinter15getRegisterNameEjE12RegAsmOffset", "llvm::X86IntelInstPrinter::getRegisterName(unsigned int)::RegAsmOffset"},
    {"_ZZN4llvm19X86IntelInstPrinter18getInstructionNameEjE13InstAsmOffset", "llvm::X86IntelInstPrinter::getInstructionName(unsigned int)::InstAsmOffset"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE15CalleeSavedRegs", "llvm::ARMBaseRegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::CalleeSavedRegs"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getCalleeSavedRegsEPKNS_15MachineFunctionEE21DarwinCalleeSavedRegs", "llvm::ARMBaseRegisterInfo::getCalleeSavedRegs(llvm::MachineFunction const*) const::DarwinCalleeSavedRegs"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven1", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven1"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd1", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd1"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven2", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven2"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd2", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd2"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven3", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven3"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd3", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd3"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven4", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven4"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd4", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd4"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven5", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven5"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd5", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd5"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE8GPREven6", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPREven6"},
    {"_ZZNK4llvm19ARMBaseRegisterInfo18getAllocationOrderEPKNS_19TargetRegisterClassEjjRKNS_15MachineFunctionEE7GPROdd6", "llvm::ARMBaseRegisterInfo::getAllocationOrder(llvm::TargetRegisterClass const*, unsigned int, unsigned int, llvm::MachineFunction const&) const::GPROdd6"},
    {"_ZZNK12_GLOBAL__N_114ARMCodeEmitter21getBinaryCodeForInstrERKN4llvm12MachineInstrEE8InstBits", "(anonymous namespace)::ARMCodeEmitter::getBinaryCodeForInstr(llvm::MachineInstr const&) const::InstBits"},
    {"_ZZ11CC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZN4llvm13f64AssignAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE7RegList", "llvm::f64AssignAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::RegList"},
    {"_ZZ14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEE9HiRegList", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)::HiRegList"},
    {"_ZZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEE9LoRegList", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)::LoRegList"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE9HiRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::HiRegList"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE9LoRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::LoRegList"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE13ShadowRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::ShadowRegList"},
    {"_ZZ22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes1", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel10SelectCodeEPN4llvm6SDNodeEE12MatcherTable", "(anonymous namespace)::ARMDAGToDAGISel::SelectCode(llvm::SDNode*)::MatcherTable"},
    {"_ZZNK4llvm17ARMTargetLowering20LowerFormalArgumentsENS_7SDValueENS_11CallingConv2IDEbRKNS_15SmallVectorImplINS_3ISD8InputArgEEENS_8DebugLocERNS_12SelectionDAGERNS4_IS1_EEE10GPRArgRegs", "llvm::ARMTargetLowering::LowerFormalArguments(llvm::SDValue, llvm::CallingConv::ID, bool, llvm::SmallVectorImpl<llvm::ISD::InputArg> const&, llvm::DebugLoc, llvm::SelectionDAG&, llvm::SmallVectorImpl<llvm::SDValue>&) const::GPRArgRegs"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ19CC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE9HiRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::HiRegList"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE9LoRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::LoRegList"},
    {"_ZZN4llvm14f64AssignAAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE13ShadowRegList", "llvm::f64AssignAAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::ShadowRegList"},
    {"_ZZ22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ22RetCC_ARM_AAPCS_CommonjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_AAPCS_Common(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEE9HiRegList", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)::HiRegList"},
    {"_ZZN4llvm12f64RetAssignERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEE9LoRegList", "llvm::f64RetAssign(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&)::LoRegList"},
    {"_ZZ11CC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZN4llvm13f64AssignAPCSERjRNS_3MVTES2_RNS_11CCValAssign7LocInfoERNS_7CCStateEbE7RegList", "llvm::f64AssignAPCS(unsigned int&, llvm::MVT&, llvm::MVT&, llvm::CCValAssign::LocInfo&, llvm::CCState&, bool)::RegList"},
    {"_ZZ14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ14RetCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ16CC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "CC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ19RetCC_ARM_AAPCS_VFPjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetCC_ARM_AAPCS_VFP(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ15FastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "FastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ15FastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "FastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ15FastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "FastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZ18RetFastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList1", "RetFastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList1"},
    {"_ZZ18RetFastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList2", "RetFastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList2"},
    {"_ZZ18RetFastCC_ARM_APCSjN4llvm3MVTES0_NS_11CCValAssign7LocInfoENS_3ISD10ArgFlagsTyERNS_7CCStateEE8RegList3", "RetFastCC_ARM_APCS(unsigned int, llvm::MVT, llvm::MVT, llvm::CCValAssign::LocInfo, llvm::ISD::ArgFlagsTy, llvm::CCState&)::RegList3"},
    {"_ZZNK12_GLOBAL__N_116ARMMCCodeEmitter21getBinaryCodeForInstrERKN4llvm6MCInstERNS1_15SmallVectorImplINS1_7MCFixupEEEE8InstBits", "(anonymous namespace)::ARMMCCodeEmitter::getBinaryCodeForInstr(llvm::MCInst const&, llvm::SmallVectorImpl<llvm::MCFixup>&) const::InstBits"},
    {"_ZZN12_GLOBAL__N_116Thumb2SizeReduce13ReduceSpecialERN4llvm17MachineBasicBlockEPNS1_12MachineInstrERKNS_11ReduceEntryEbE11NarrowEntry", "(anonymous namespace)::Thumb2SizeReduce::ReduceSpecial(llvm::MachineBasicBlock&, llvm::MachineInstr*, (anonymous namespace)::ReduceEntry const&, bool)::NarrowEntry"},
    {"_ZZN12_GLOBAL__N_120SelectionDAGLegalize10ExpandNodeEPN4llvm6SDNodeERNS1_15SmallVectorImplINS1_7SDValueEEEE3Ops", "(anonymous namespace)::SelectionDAGLegalize::ExpandNode(llvm::SDNode*, llvm::SmallVectorImpl<llvm::SDValue>&)::Ops"},
    {"_ZZN4llvm16DAGTypeLegalizer25ExpandFloatRes_XINT_TO_FPEPNS_6SDNodeERNS_7SDValueES4_E6TwoE32", "llvm::DAGTypeLegalizer::ExpandFloatRes_XINT_TO_FP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)::TwoE32"},
    {"_ZZN4llvm16DAGTypeLegalizer25ExpandFloatRes_XINT_TO_FPEPNS_6SDNodeERNS_7SDValueES4_E6TwoE64", "llvm::DAGTypeLegalizer::ExpandFloatRes_XINT_TO_FP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)::TwoE64"},
    {"_ZZN4llvm16DAGTypeLegalizer25ExpandFloatRes_XINT_TO_FPEPNS_6SDNodeERNS_7SDValueES4_E7TwoE128", "llvm::DAGTypeLegalizer::ExpandFloatRes_XINT_TO_FP(llvm::SDNode*, llvm::SDValue&, llvm::SDValue&)::TwoE128"},
    {"_ZZN4llvm16DAGTypeLegalizer24ExpandFloatOp_FP_TO_UINTEPNS_6SDNodeEE6TwoE31", "llvm::DAGTypeLegalizer::ExpandFloatOp_FP_TO_UINT(llvm::SDNode*)::TwoE31"},
    {"_ZZ10LowerCTPOPRN4llvm11LLVMContextEPNS_5ValueEPNS_11InstructionEE10MaskValues", "LowerCTPOP(llvm::LLVMContext&, llvm::Value*, llvm::Instruction*)::MaskValues"},
    {"_ZN4llvm24RegisterCoalescerLinkVarE", "llvm::RegisterCoalescerLinkVar"},
    {"_ZN4llvm31SimpleRegisterCoalescingLinkVarE", "llvm::SimpleRegisterCoalescingLinkVar"},
    {"_ZN4llvm16CallGraphLinkVarE", "llvm::CallGraphLinkVar"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE12MissingValueE", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::MissingValue"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEE12MissingValueE", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::MissingValue"},
    {"_ZTSN4llvm5ValueE", "typeinfo name for llvm::Value"},
    {"_ZTSN4llvm4TypeE", "typeinfo name for llvm::Type"},
    {"_ZTSN4llvm24AssemblyAnnotationWriterE", "typeinfo name for llvm::AssemblyAnnotationWriter"},
    {"_ZTSN4llvm10FoldingSetINS_17AttributeListImplEEE", "typeinfo name for llvm::FoldingSet<llvm::AttributeListImpl>"},
    {"_ZTSN4llvm10BasicBlockE", "typeinfo name for llvm::BasicBlock"},
    {"_ZTSN4llvm10ilist_nodeINS_10BasicBlockEEE", "typeinfo name for llvm::ilist_node<llvm::BasicBlock>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_10BasicBlockEEE", "typeinfo name for llvm::ilist_half_node<llvm::BasicBlock>"},
    {"_ZTSN4llvm21ConstantAggregateZeroE", "typeinfo name for llvm::ConstantAggregateZero"},
    {"_ZTSN4llvm19ConstantPointerNullE", "typeinfo name for llvm::ConstantPointerNull"},
    {"_ZTSN4llvm12ConstantExprE", "typeinfo name for llvm::ConstantExpr"},
    {"_ZTSN4llvm10UndefValueE", "typeinfo name for llvm::UndefValue"},
    {"_ZTSN4llvm10ConstantFPE", "typeinfo name for llvm::ConstantFP"},
    {"_ZTSN4llvm13ConstantArrayE", "typeinfo name for llvm::ConstantArray"},
    {"_ZTSN4llvm14ConstantStructE", "typeinfo name for llvm::ConstantStruct"},
    {"_ZTSN4llvm14ConstantVectorE", "typeinfo name for llvm::ConstantVector"},
    {"_ZTSN4llvm12BlockAddressE", "typeinfo name for llvm::BlockAddress"},
    {"_ZTSN4llvm8ConstantE", "typeinfo name for llvm::Constant"},
    {"_ZTSN4llvm25GetElementPtrConstantExprE", "typeinfo name for llvm::GetElementPtrConstantExpr"},
    {"_ZTSN4llvm11ConstantIntE", "typeinfo name for llvm::ConstantInt"},
    {"_ZTSN4llvm19CompareConstantExprE", "typeinfo name for llvm::CompareConstantExpr"},
    {"_ZTSN4llvm24ExtractValueConstantExprE", "typeinfo name for llvm::ExtractValueConstantExpr"},
    {"_ZTSN4llvm23InsertValueConstantExprE", "typeinfo name for llvm::InsertValueConstantExpr"},
    {"_ZTSN4llvm25ShuffleVectorConstantExprE", "typeinfo name for llvm::ShuffleVectorConstantExpr"},
    {"_ZTSN4llvm25InsertElementConstantExprE", "typeinfo name for llvm::InsertElementConstantExpr"},
    {"_ZTSN4llvm26ExtractElementConstantExprE", "typeinfo name for llvm::ExtractElementConstantExpr"},
    {"_ZTSN4llvm18SelectConstantExprE", "typeinfo name for llvm::SelectConstantExpr"},
    {"_ZTSN4llvm18BinaryConstantExprE", "typeinfo name for llvm::BinaryConstantExpr"},
    {"_ZTSN4llvm17UnaryConstantExprE", "typeinfo name for llvm::UnaryConstantExpr"},
    {"_ZTSN4llvm10DebugRecVHE", "typeinfo name for llvm::DebugRecVH"},
    {"_ZTSN4llvm17DominatorTreeBaseINS_10BasicBlockEEE", "typeinfo name for llvm::DominatorTreeBase<llvm::BasicBlock>"},
    {"_ZTSN4llvm13DominatorBaseINS_10BasicBlockEEE", "typeinfo name for llvm::DominatorBase<llvm::BasicBlock>"},
    {"_ZTSN4llvm13DominatorTreeE", "typeinfo name for llvm::DominatorTree"},
    {"_ZZN4llvm9Intrinsic12isOverloadedENS0_2IDEE6OTable", "llvm::Intrinsic::isOverloaded(llvm::Intrinsic::ID)::OTable"},
    {"_ZTSN4llvm8FunctionE", "typeinfo name for llvm::Function"},
    {"_ZTSN4llvm10ilist_nodeINS_8FunctionEEE", "typeinfo name for llvm::ilist_node<llvm::Function>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_8FunctionEEE", "typeinfo name for llvm::ilist_half_node<llvm::Function>"},
    {"_ZTSN4llvm8ArgumentE", "typeinfo name for llvm::Argument"},
    {"_ZTSN4llvm10ilist_nodeINS_8ArgumentEEE", "typeinfo name for llvm::ilist_node<llvm::Argument>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_8ArgumentEEE", "typeinfo name for llvm::ilist_half_node<llvm::Argument>"},
    {"_ZTSN4llvm14GVMaterializerE", "typeinfo name for llvm::GVMaterializer"},
    {"_ZTSN4llvm11GlobalValueE", "typeinfo name for llvm::GlobalValue"},
    {"_ZTSN4llvm14GlobalVariableE", "typeinfo name for llvm::GlobalVariable"},
    {"_ZTSN4llvm11GlobalAliasE", "typeinfo name for llvm::GlobalAlias"},
    {"_ZTSN4llvm10ilist_nodeINS_14GlobalVariableEEE", "typeinfo name for llvm::ilist_node<llvm::GlobalVariable>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_14GlobalVariableEEE", "typeinfo name for llvm::ilist_half_node<llvm::GlobalVariable>"},
    {"_ZTSN4llvm10ilist_nodeINS_11GlobalAliasEEE", "typeinfo name for llvm::ilist_node<llvm::GlobalAlias>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_11GlobalAliasEEE", "typeinfo name for llvm::ilist_half_node<llvm::GlobalAlias>"},
    {"_ZTSN4llvm9InlineAsmE", "typeinfo name for llvm::InlineAsm"},
    {"_ZTSN4llvm11InstructionE", "typeinfo name for llvm::Instruction"},
    {"_ZTSN4llvm4UserE", "typeinfo name for llvm::User"},
    {"_ZTSN4llvm10ilist_nodeINS_11InstructionEEE", "typeinfo name for llvm::ilist_node<llvm::Instruction>"},
    {"_ZTSN4llvm15ilist_half_nodeINS_11InstructionEEE", "typeinfo name for llvm::ilist_half_node<llvm::Instruction>"},
    {"_ZZN4llvm8CastInst20isEliminableCastPairENS_11Instruction7CastOpsES2_PKNS_4TypeES5_S5_S5_E11CastResults", "llvm::CastInst::isEliminableCastPair(llvm::Instruction::CastOps, llvm::Instruction::CastOps, llvm::Type const*, llvm::Type const*, llvm::Type const*, llvm::Type const*)::CastResults"},
    {"_ZTSN4llvm14TerminatorInstE", "typeinfo name for llvm::TerminatorInst"},
    {"_ZTSN4llvm16UnaryInstructionE", "typeinfo name for llvm::UnaryInstruction"},
    {"_ZTSN4llvm17GetElementPtrInstE", "typeinfo name for llvm::GetElementPtrInst"},
    {"_ZTSN4llvm7CmpInstE", "typeinfo name for llvm::CmpInst"},
    {"_ZTSN4llvm8ICmpInstE", "typeinfo name for llvm::ICmpInst"},
    {"_ZTSN4llvm8FCmpInstE", "typeinfo name for llvm::FCmpInst"},
    {"_ZTSN4llvm8CallInstE", "typeinfo name for llvm::CallInst"},
    {"_ZTSN4llvm10SelectInstE", "typeinfo name for llvm::SelectInst"},
    {"_ZTSN4llvm9VAArgInstE", "typeinfo name for llvm::VAArgInst"},
    {"_ZTSN4llvm18ExtractElementInstE", "typeinfo name for llvm::ExtractElementInst"},
    {"_ZTSN4llvm17InsertElementInstE", "typeinfo name for llvm::InsertElementInst"},
    {"_ZTSN4llvm16ExtractValueInstE", "typeinfo name for llvm::ExtractValueInst"},
    {"_ZTSN4llvm15InsertValueInstE", "typeinfo name for llvm::InsertValueInst"},
    {"_ZTSN4llvm7PHINodeE", "typeinfo name for llvm::PHINode"},
    {"_ZTSN4llvm10ReturnInstE", "typeinfo name for llvm::ReturnInst"},
    {"_ZTSN4llvm10BranchInstE", "typeinfo name for llvm::BranchInst"},
    {"_ZTSN4llvm10SwitchInstE", "typeinfo name for llvm::SwitchInst"},
    {"_ZTSN4llvm14IndirectBrInstE", "typeinfo name for llvm::IndirectBrInst"},
    {"_ZTSN4llvm11BitCastInstE", "typeinfo name for llvm::BitCastInst"},
    {"_ZTSN4llvm10InvokeInstE", "typeinfo name for llvm::InvokeInst"},
    {"_ZTSN4llvm10UnwindInstE", "typeinfo name for llvm::UnwindInst"},
    {"_ZTSN4llvm15UnreachableInstE", "typeinfo name for llvm::UnreachableInst"},
    {"_ZTSN4llvm10AllocaInstE", "typeinfo name for llvm::AllocaInst"},
    {"_ZTSN4llvm8LoadInstE", "typeinfo name for llvm::LoadInst"},
    {"_ZTSN4llvm9StoreInstE", "typeinfo name for llvm::StoreInst"},
    {"_ZTSN4llvm17ShuffleVectorInstE", "typeinfo name for llvm::ShuffleVectorInst"},
    {"_ZTSN4llvm14BinaryOperatorE", "typeinfo name for llvm::BinaryOperator"},
    {"_ZTSN4llvm9TruncInstE", "typeinfo name for llvm::TruncInst"},
    {"_ZTSN4llvm8ZExtInstE", "typeinfo name for llvm::ZExtInst"},
    {"_ZTSN4llvm8SExtInstE", "typeinfo name for llvm::SExtInst"},
    {"_ZTSN4llvm11FPTruncInstE", "typeinfo name for llvm::FPTruncInst"},
    {"_ZTSN4llvm9FPExtInstE", "typeinfo name for llvm::FPExtInst"},
    {"_ZTSN4llvm10UIToFPInstE", "typeinfo name for llvm::UIToFPInst"},
    {"_ZTSN4llvm10SIToFPInstE", "typeinfo name for llvm::SIToFPInst"},
    {"_ZTSN4llvm10FPToUIInstE", "typeinfo name for llvm::FPToUIInst"},
    {"_ZTSN4llvm10FPToSIInstE", "typeinfo name for llvm::FPToSIInst"},
    {"_ZTSN4llvm12PtrToIntInstE", "typeinfo name for llvm::PtrToIntInst"},
    {"_ZTSN4llvm12IntToPtrInstE", "typeinfo name for llvm::IntToPtrInst"},
    {"_ZTSN4llvm8CastInstE", "typeinfo name for llvm::CastInst"},
    {"_ZTSN4llvm10FoldingSetINS_6MDNodeEEE", "typeinfo name for llvm::FoldingSet<llvm::MDNode>"},
    {"_ZTSN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>"},
    {"_ZTSN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EEE", "typeinfo name for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>"},
    {"_ZTSN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EEE", "typeinfo name for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>"},
    {"_ZTSN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>"},
    {"_ZTSN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>"},
    {"_ZTSN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>"},
    {"_ZTSN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>"},
    {"_ZTSN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EEE", "typeinfo name for llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>"},
    {"_ZTSN4llvm11IntegerTypeE", "typeinfo name for llvm::IntegerType"},
    {"_ZTSN4llvm11DerivedTypeE", "typeinfo name for llvm::DerivedType"},
    {"_ZTSN4llvm13MDNodeOperandE", "typeinfo name for llvm::MDNodeOperand"},
    {"_ZTSN4llvm6MDNodeE", "typeinfo name for llvm::MDNode"},
    {"_ZTSN4llvm15ValueHandleBaseE", "typeinfo name for llvm::ValueHandleBase"},
    {"_ZTSN4llvm14FoldingSetImpl4NodeE", "typeinfo name for llvm::FoldingSetImpl::Node"},
    {"_ZTSN4llvm8MDStringE", "typeinfo name for llvm::MDString"},
    {"_ZTSN4llvm4PassE", "typeinfo name for llvm::Pass"},
    {"_ZTSN4llvm10ModulePassE", "typeinfo name for llvm::ModulePass"},
    {"_ZTSN4llvm13ImmutablePassE", "typeinfo name for llvm::ImmutablePass"},
    {"_ZTSN4llvm12FunctionPassE", "typeinfo name for llvm::FunctionPass"},
    {"_ZTSN4llvm14BasicBlockPassE", "typeinfo name for llvm::BasicBlockPass"},
    {"_ZTSN4llvm24PassRegistrationListenerE", "typeinfo name for llvm::PassRegistrationListener"},
    {"_ZTSN4llvm14PassNameParserE", "typeinfo name for llvm::PassNameParser"},
    {"_ZTSN12_GLOBAL__N_116GetCFGOnlyPassesE", "typeinfo name for (anonymous namespace)::GetCFGOnlyPasses"},
    {"_ZTSN4llvm2cl6parserIPKNS_8PassInfoEEE", "typeinfo name for llvm::cl::parser<llvm::PassInfo const*>"},
    {"_ZTSN4llvm27PassManagerPrettyStackEntryE", "typeinfo name for llvm::PassManagerPrettyStackEntry"},
    {"_ZTSN4llvm13PMDataManagerE", "typeinfo name for llvm::PMDataManager"},
    {"_ZTSN4llvm13FPPassManagerE", "typeinfo name for llvm::FPPassManager"},
    {"_ZTSN4llvm17PMTopLevelManagerE", "typeinfo name for llvm::PMTopLevelManager"},
    {"_ZTSN4llvm13MPPassManagerE", "typeinfo name for llvm::MPPassManager"},
    {"_ZTSN4llvm15PassManagerBaseE", "typeinfo name for llvm::PassManagerBase"},
    {"_ZTSN4llvm19FunctionPassManagerE", "typeinfo name for llvm::FunctionPassManager"},
    {"_ZTSN4llvm11PassManagerE", "typeinfo name for llvm::PassManager"},
    {"_ZTSN12_GLOBAL__N_113BBPassManagerE", "typeinfo name for (anonymous namespace)::BBPassManager"},
    {"_ZTSN4llvm2cl3optIbLb1ENS0_6parserIbEEEE", "typeinfo name for llvm::cl::opt<bool, true, llvm::cl::parser<bool> >"},
    {"_ZTSN4llvm2cl11opt_storageIbLb1ELb0EEE", "typeinfo name for llvm::cl::opt_storage<bool, true, false>"},
    {"_ZTSN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEEE", "typeinfo name for llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>"},
    {"_ZTSN4llvm2cl12list_storageIPKNS_8PassInfoEbEE", "typeinfo name for llvm::cl::list_storage<llvm::PassInfo const*, bool>"},
    {"_ZTSSt6vectorIPKN4llvm8PassInfoESaIS3_EE", "typeinfo name for std::vector<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> >"},
    {"_ZTSSt12_Vector_baseIPKN4llvm8PassInfoESaIS3_EE", "typeinfo name for std::_Vector_base<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> >"},
    {"_ZTSN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEEE", "typeinfo name for llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >"},
    {"_ZTSN4llvm2cl11opt_storageINS_14PassDebugLevelELb0ELb0EEE", "typeinfo name for llvm::cl::opt_storage<llvm::PassDebugLevel, false, false>"},
    {"_ZTSN4llvm2cl6parserINS_14PassDebugLevelEEE", "typeinfo name for llvm::cl::parser<llvm::PassDebugLevel>"},
    {"_ZTSN4llvm15PassManagerImplE", "typeinfo name for llvm::PassManagerImpl"},
    {"_ZTSN4llvm23FunctionPassManagerImplE", "typeinfo name for llvm::FunctionPassManagerImpl"},
    {"_ZTSN12_GLOBAL__N_117PrintFunctionPassE", "typeinfo name for (anonymous namespace)::PrintFunctionPass"},
    {"_ZTSN12_GLOBAL__N_115PrintModulePassE", "typeinfo name for (anonymous namespace)::PrintModulePass"},
    {"_ZTSN4llvm16AbstractTypeUserE", "typeinfo name for llvm::AbstractTypeUser"},
    {"_ZTSN4llvm14SequentialTypeE", "typeinfo name for llvm::SequentialType"},
    {"_ZTSN4llvm12FunctionTypeE", "typeinfo name for llvm::FunctionType"},
    {"_ZTSN4llvm10StructTypeE", "typeinfo name for llvm::StructType"},
    {"_ZTSN4llvm9ArrayTypeE", "typeinfo name for llvm::ArrayType"},
    {"_ZTSN4llvm10VectorTypeE", "typeinfo name for llvm::VectorType"},
    {"_ZTSN4llvm11PointerTypeE", "typeinfo name for llvm::PointerType"},
    {"_ZTSN4llvm13CompositeTypeE", "typeinfo name for llvm::CompositeType"},
    {"_ZTSN4llvm10OpaqueTypeE", "typeinfo name for llvm::OpaqueType"},
    {"_ZTSN4llvm15TypeSymbolTableE", "typeinfo name for llvm::TypeSymbolTable"},
    {"_ZZN4llvm3Use8initTagsEPS0_S1_E4tags", "llvm::Use::initTags(llvm::Use*, llvm::Use*)::tags"},
    {"_ZTSN4llvm10CallbackVHE", "typeinfo name for llvm::CallbackVH"},
    {"_ZTSN12_GLOBAL__N_18VerifierE", "typeinfo name for (anonymous namespace)::Verifier"},
    {"_ZTSN4llvm11InstVisitorIN12_GLOBAL__N_18VerifierEvEE", "typeinfo name for llvm::InstVisitor<(anonymous namespace)::Verifier, void>"},
    {"_ZTSN12_GLOBAL__N_17TypeSetE", "typeinfo name for (anonymous namespace)::TypeSet"},
    {"_ZTSN12_GLOBAL__N_111PreVerifierE", "typeinfo name for (anonymous namespace)::PreVerifier"},
    {"_ZZN4llvm14ARMInstPrinter16printInstructionEPKNS_6MCInstERNS_11raw_ostreamEE6OpInfo", "llvm::ARMInstPrinter::printInstruction(llvm::MCInst const*, llvm::raw_ostream&)::OpInfo"},
    {"_ZZN4llvm14ARMInstPrinter15getRegisterNameEjE12RegAsmOffset", "llvm::ARMInstPrinter::getRegisterName(unsigned int)::RegAsmOffset"},
    {"_ZZN4llvm14ARMInstPrinter18getInstructionNameEjE13InstAsmOffset", "llvm::ARMInstPrinter::getInstructionName(unsigned int)::InstAsmOffset"},
    {"_ZZN4llvm14MCObjectWriter10WriteZerosEjE5Zeros", "llvm::MCObjectWriter::WriteZeros(unsigned int)::Zeros"},
    {"_ZZN4llvm14MCObjectWriter10WriteZerosEjE5Zeros", "llvm::MCObjectWriter::WriteZeros(unsigned int)::Zeros"},
    {"_ZZN4llvm14MCObjectWriter10WriteZerosEjE5Zeros", "llvm::MCObjectWriter::WriteZeros(unsigned int)::Zeros"},
    {"_ZZN4llvm14MCObjectWriter10WriteZerosEjE5Zeros", "llvm::MCObjectWriter::WriteZeros(unsigned int)::Zeros"},
    {"_ZZN4llvm14MCObjectWriter10WriteZerosEjE5Zeros", "llvm::MCObjectWriter::WriteZeros(unsigned int)::Zeros"},
    {"_ZZN4llvm7APFloat28roundSignificandWithExponentEPKyjiNS0_12roundingModeEE13calcSemantics", "llvm::APFloat::roundSignificandWithExponent(unsigned long long const*, unsigned int, int, llvm::APFloat::roundingMode)::calcSemantics"},
    {"_ZZ8powerOf5PyjE16firstEightPowers", "powerOf5(unsigned long long*, unsigned int)::firstEightPowers"},
    {"_ZZNK4llvm5APInt4sqrtEvE7results", "llvm::APInt::sqrt() const::results"},
    {"_ZZNK4llvm5APInt8toStringERNS_15SmallVectorImplIcEEjbE6Digits", "llvm::APInt::toString(llvm::SmallVectorImpl<char>&, unsigned int, bool) const::Digits"},
    {"_ZTSN4llvm13SlabAllocatorE", "typeinfo name for llvm::SlabAllocator"},
    {"_ZTSN4llvm19MallocSlabAllocatorE", "typeinfo name for llvm::MallocSlabAllocator"},
    {"_ZTSN4llvm2cl6OptionE", "typeinfo name for llvm::cl::Option"},
    {"_ZTSN4llvm2cl19generic_parser_baseE", "typeinfo name for llvm::cl::generic_parser_base"},
    {"_ZTSN4llvm2cl17basic_parser_implE", "typeinfo name for llvm::cl::basic_parser_impl"},
    {"_ZTSN4llvm2cl6parserIbEE", "typeinfo name for llvm::cl::parser<bool>"},
    {"_ZTSN4llvm2cl6parserINS0_13boolOrDefaultEEE", "typeinfo name for llvm::cl::parser<llvm::cl::boolOrDefault>"},
    {"_ZTSN4llvm2cl6parserIiEE", "typeinfo name for llvm::cl::parser<int>"},
    {"_ZTSN4llvm2cl6parserIjEE", "typeinfo name for llvm::cl::parser<unsigned int>"},
    {"_ZTSN4llvm2cl6parserIdEE", "typeinfo name for llvm::cl::parser<double>"},
    {"_ZTSN4llvm2cl6parserIfEE", "typeinfo name for llvm::cl::parser<float>"},
    {"_ZTSN4llvm2cl6parserISsEE", "typeinfo name for llvm::cl::parser<std::string>"},
    {"_ZTSN4llvm2cl6parserIcEE", "typeinfo name for llvm::cl::parser<char>"},
    {"_ZTSN4llvm2cl5aliasE", "typeinfo name for llvm::cl::alias"},
    {"_ZTSN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEEE", "typeinfo name for llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTSN4llvm2cl11opt_storageIN12_GLOBAL__N_114VersionPrinterELb1ELb1EEE", "typeinfo name for llvm::cl::opt_storage<(anonymous namespace)::VersionPrinter, true, true>"},
    {"_ZTSN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEEE", "typeinfo name for llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTSN4llvm2cl11opt_storageIN12_GLOBAL__N_111HelpPrinterELb1ELb1EEE", "typeinfo name for llvm::cl::opt_storage<(anonymous namespace)::HelpPrinter, true, true>"},
    {"_ZTSN4llvm2cl12basic_parserIbEE", "typeinfo name for llvm::cl::basic_parser<bool>"},
    {"_ZTSN4llvm2cl12basic_parserINS0_13boolOrDefaultEEE", "typeinfo name for llvm::cl::basic_parser<llvm::cl::boolOrDefault>"},
    {"_ZTSN4llvm2cl12basic_parserIiEE", "typeinfo name for llvm::cl::basic_parser<int>"},
    {"_ZTSN4llvm2cl12basic_parserIjEE", "typeinfo name for llvm::cl::basic_parser<unsigned int>"},
    {"_ZTSN4llvm2cl12basic_parserIdEE", "typeinfo name for llvm::cl::basic_parser<double>"},
    {"_ZTSN4llvm2cl12basic_parserIfEE", "typeinfo name for llvm::cl::basic_parser<float>"},
    {"_ZTSN4llvm2cl12basic_parserISsEE", "typeinfo name for llvm::cl::basic_parser<std::string>"},
    {"_ZTSN4llvm2cl12basic_parserIcEE", "typeinfo name for llvm::cl::basic_parser<char>"},
    {"_ZTSN4llvm2cl3optIjLb0ENS0_6parserIjEEEE", "typeinfo name for llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >"},
    {"_ZTSN4llvm2cl11opt_storageIjLb0ELb0EEE", "typeinfo name for llvm::cl::opt_storage<unsigned int, false, false>"},
    {"_ZTSN4llvm2cl3optIiLb0ENS0_6parserIiEEEE", "typeinfo name for llvm::cl::opt<int, false, llvm::cl::parser<int> >"},
    {"_ZTSN4llvm2cl11opt_storageIiLb0ELb0EEE", "typeinfo name for llvm::cl::opt_storage<int, false, false>"},
    {"_ZTSN4llvm2cl3optISsLb0ENS0_6parserISsEEEE", "typeinfo name for llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >"},
    {"_ZTSN4llvm2cl11opt_storageISsLb0ELb1EEE", "typeinfo name for llvm::cl::opt_storage<std::string, false, true>"},
    {"_ZTSSs", "typeinfo name for std::string"},
    {"_ZTSN4llvm2cl3optIcLb0ENS0_6parserIcEEEE", "typeinfo name for llvm::cl::opt<char, false, llvm::cl::parser<char> >"},
    {"_ZTSN4llvm2cl11opt_storageIcLb0ELb0EEE", "typeinfo name for llvm::cl::opt_storage<char, false, false>"},
    {"_ZTSN4llvm2cl3optIbLb0ENS0_6parserIbEEEE", "typeinfo name for llvm::cl::opt<bool, false, llvm::cl::parser<bool> >"},
    {"_ZTSN4llvm2cl11opt_storageIbLb0ELb0EEE", "typeinfo name for llvm::cl::opt_storage<bool, false, false>"},
    {"_ZTSN4llvm3sys11ThreadLocalIKN12_GLOBAL__N_124CrashRecoveryContextImplEEE", "typeinfo name for llvm::sys::ThreadLocal<(anonymous namespace)::CrashRecoveryContextImpl const>"},
    {"_ZTSN4llvm14FoldingSetImplE", "typeinfo name for llvm::FoldingSetImpl"},
    {"_ZTSN4llvm21formatted_raw_ostreamE", "typeinfo name for llvm::formatted_raw_ostream"},
    {"_ZTSN4llvm12MemoryBufferE", "typeinfo name for llvm::MemoryBuffer"},
    {"_ZTSN12_GLOBAL__N_120MemoryBufferMMapFileE", "typeinfo name for (anonymous namespace)::MemoryBufferMMapFile"},
    {"_ZTSN12_GLOBAL__N_115MemoryBufferMemE", "typeinfo name for (anonymous namespace)::MemoryBufferMem"},
    {"_ZN4llvm3sys13PathSeparatorE", "llvm::sys::PathSeparator"},
    {"_ZTSN4llvm21PrettyStackTraceEntryE", "typeinfo name for llvm::PrettyStackTraceEntry"},
    {"_ZTSN4llvm22PrettyStackTraceStringE", "typeinfo name for llvm::PrettyStackTraceString"},
    {"_ZTSN4llvm23PrettyStackTraceProgramE", "typeinfo name for llvm::PrettyStackTraceProgram"},
    {"_ZTSN4llvm3sys11ThreadLocalIKNS_21PrettyStackTraceEntryEEE", "typeinfo name for llvm::sys::ThreadLocal<llvm::PrettyStackTraceEntry const>"},
    {"_ZN4llvm9StringRef4nposE", "llvm::StringRef::npos"},
    {"_ZTSN4llvm3sys15ThreadLocalImplE", "typeinfo name for llvm::sys::ThreadLocalImpl"},
    {"_ZTSN4llvm14format_object1IxEE", "typeinfo name for llvm::format_object1<long long>"},
    {"_ZTSN4llvm14format_object1IdEE", "typeinfo name for llvm::format_object1<double>"},
    {"_ZTSN4llvm2cl3optISsLb1ENS0_6parserISsEEEE", "typeinfo name for llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >"},
    {"_ZTSN4llvm2cl11opt_storageISsLb1ELb1EEE", "typeinfo name for llvm::cl::opt_storage<std::string, true, true>"},
    {"_ZZN4llvm11raw_ostream6indentEjE6Spaces", "llvm::raw_ostream::indent(unsigned int)::Spaces"},
    {"_ZTSN4llvm11raw_ostreamE", "typeinfo name for llvm::raw_ostream"},
    {"_ZTSN4llvm18raw_string_ostreamE", "typeinfo name for llvm::raw_string_ostream"},
    {"_ZTSN4llvm16raw_null_ostreamE", "typeinfo name for llvm::raw_null_ostream"},
    {"_ZTSN4llvm18format_object_baseE", "typeinfo name for llvm::format_object_base"},
    {"_ZTSN4llvm14raw_fd_ostreamE", "typeinfo name for llvm::raw_fd_ostream"},
    {"_ZTSN4llvm19raw_svector_ostreamE", "typeinfo name for llvm::raw_svector_ostream"},
    {"_ZTSN4llvm14error_categoryE", "typeinfo name for llvm::error_category"},
    {"_ZTSN4llvm11_do_messageE", "typeinfo name for llvm::_do_message"},
    {"_ZTSN4llvm23_generic_error_categoryE", "typeinfo name for llvm::_generic_error_category"},
    {"_ZTSN4llvm22_system_error_categoryE", "typeinfo name for llvm::_system_error_category"},
    {"_ZTVN12_GLOBAL__N_110ASTDumpXMLE", "vtable for (anonymous namespace)::ASTDumpXML"},
    {"_ZTVN12_GLOBAL__N_117InheritanceViewerE", "vtable for (anonymous namespace)::InheritanceViewer"},
    {"_ZTVN12_GLOBAL__N_118DeclContextPrinterE", "vtable for (anonymous namespace)::DeclContextPrinter"},
    {"_ZTVN12_GLOBAL__N_19ASTViewerE", "vtable for (anonymous namespace)::ASTViewer"},
    {"_ZTVN12_GLOBAL__N_113ASTPrinterXMLE", "vtable for (anonymous namespace)::ASTPrinterXML"},
    {"_ZTVN12_GLOBAL__N_110ASTPrinterE", "vtable for (anonymous namespace)::ASTPrinter"},
    {"_ZTVN5clang14ASTMergeActionE", "vtable for clang::ASTMergeAction"},
    {"_ZTVN12_GLOBAL__N_129AugmentedCodeCompleteConsumerE", "vtable for (anonymous namespace)::AugmentedCodeCompleteConsumer"},
    {"_ZTVN12_GLOBAL__N_122StoredDiagnosticClientE", "vtable for (anonymous namespace)::StoredDiagnosticClient"},
    {"_ZTVN12_GLOBAL__N_124PrecompilePreambleActionE", "vtable for (anonymous namespace)::PrecompilePreambleAction"},
    {"_ZTVN12_GLOBAL__N_126PrecompilePreambleConsumerE", "vtable for (anonymous namespace)::PrecompilePreambleConsumer"},
    {"_ZTVN12_GLOBAL__N_125TopLevelDeclTrackerActionE", "vtable for (anonymous namespace)::TopLevelDeclTrackerAction"},
    {"_ZTVN12_GLOBAL__N_127TopLevelDeclTrackerConsumerE", "vtable for (anonymous namespace)::TopLevelDeclTrackerConsumer"},
    {"_ZTVN12_GLOBAL__N_116ASTInfoCollectorE", "vtable for (anonymous namespace)::ASTInfoCollector"},
    {"_ZTVN5clang14BoostConActionE", "vtable for clang::BoostConAction"},
    {"_ZTVN12_GLOBAL__N_119BoostConASTConsumerE", "vtable for (anonymous namespace)::BoostConASTConsumer"},
    {"_ZTVN12_GLOBAL__N_112StatListenerE", "vtable for (anonymous namespace)::StatListener"},
    {"_ZTVN12_GLOBAL__N_122DependencyFileCallbackE", "vtable for (anonymous namespace)::DependencyFileCallback"},
    {"_ZTVN5clang14FrontendActionE", "vtable for clang::FrontendAction"},
    {"_ZTVN5clang17ASTFrontendActionE", "vtable for clang::ASTFrontendAction"},
    {"_ZTVN5clang26PreprocessorFrontendActionE", "vtable for clang::PreprocessorFrontendAction"},
    {"_ZTVN12_GLOBAL__N_124DeserializedDeclsCheckerE", "vtable for (anonymous namespace)::DeserializedDeclsChecker"},
    {"_ZTVN12_GLOBAL__N_123DeserializedDeclsDumperE", "vtable for (anonymous namespace)::DeserializedDeclsDumper"},
    {"_ZTVN5clang14InitOnlyActionE", "vtable for clang::InitOnlyAction"},
    {"_ZTVN5clang14ASTPrintActionE", "vtable for clang::ASTPrintAction"},
    {"_ZTVN5clang17ASTPrintXMLActionE", "vtable for clang::ASTPrintXMLAction"},
    {"_ZTVN5clang13ASTDumpActionE", "vtable for clang::ASTDumpAction"},
    {"_ZTVN5clang16ASTDumpXMLActionE", "vtable for clang::ASTDumpXMLAction"},
    {"_ZTVN5clang13ASTViewActionE", "vtable for clang::ASTViewAction"},
    {"_ZTVN5clang22DeclContextPrintActionE", "vtable for clang::DeclContextPrintAction"},
    {"_ZTVN5clang17GeneratePCHActionE", "vtable for clang::GeneratePCHAction"},
    {"_ZTVN5clang21InheritanceViewActionE", "vtable for clang::InheritanceViewAction"},
    {"_ZTVN5clang16SyntaxOnlyActionE", "vtable for clang::SyntaxOnlyAction"},
    {"_ZTVN5clang19PrintPreambleActionE", "vtable for clang::PrintPreambleAction"},
    {"_ZTVN5clang19DumpRawTokensActionE", "vtable for clang::DumpRawTokensAction"},
    {"_ZTVN5clang16DumpTokensActionE", "vtable for clang::DumpTokensAction"},
    {"_ZTVN5clang17GeneratePTHActionE", "vtable for clang::GeneratePTHAction"},
    {"_ZTVN5clang20PreprocessOnlyActionE", "vtable for clang::PreprocessOnlyAction"},
    {"_ZTVN5clang23PrintPreprocessedActionE", "vtable for clang::PrintPreprocessedAction"},
    {"_ZTVN12_GLOBAL__N_122HeaderIncludesCallbackE", "vtable for (anonymous namespace)::HeaderIncludesCallback"},
    {"_ZTVN5clang35MultiplexASTDeserializationListenerE", "vtable for clang::MultiplexASTDeserializationListener"},
    {"_ZTVN5clang28MultiplexASTMutationListenerE", "vtable for clang::MultiplexASTMutationListener"},
    {"_ZTVN5clang17MultiplexConsumerE", "vtable for clang::MultiplexConsumer"},
    {"_ZTVN12_GLOBAL__N_120UnknownPragmaHandlerE", "vtable for (anonymous namespace)::UnknownPragmaHandler"},
    {"_ZTVN12_GLOBAL__N_124PrintPPOutputPPCallbacksE", "vtable for (anonymous namespace)::PrintPPOutputPPCallbacks"},
    {"_ZTVN5clang20TextDiagnosticBufferE", "vtable for clang::TextDiagnosticBuffer"},
    {"_ZTVN5clang21TextDiagnosticPrinterE", "vtable for clang::TextDiagnosticPrinter"},
    {"_ZTVN5clang23VerifyDiagnosticsClientE", "vtable for clang::VerifyDiagnosticsClient"},
    {"_ZTVN12_GLOBAL__N_117StandardDirectiveE", "vtable for (anonymous namespace)::StandardDirective"},
    {"_ZTVN12_GLOBAL__N_19DirectiveE", "vtable for (anonymous namespace)::Directive"},
    {"_ZTVN12_GLOBAL__N_114RegexDirectiveE", "vtable for (anonymous namespace)::RegexDirective"},
    {"_ZTVN5clang6driver6ActionE", "vtable for clang::driver::Action"},
    {"_ZTVN5clang6driver7ArgListE", "vtable for clang::driver::ArgList"},
    {"_ZTVN5clang6driver12InputArgListE", "vtable for clang::driver::InputArgList"},
    {"_ZTVN5clang6driver14DerivedArgListE", "vtable for clang::driver::DerivedArgList"},
    {"_ZTVN5clang6driver8HostInfoE", "vtable for clang::driver::HostInfo"},
    {"_ZTVN12_GLOBAL__N_115UnknownHostInfoE", "vtable for (anonymous namespace)::UnknownHostInfo"},
    {"_ZTVN12_GLOBAL__N_113MinGWHostInfoE", "vtable for (anonymous namespace)::MinGWHostInfo"},
    {"_ZTVN12_GLOBAL__N_115WindowsHostInfoE", "vtable for (anonymous namespace)::WindowsHostInfo"},
    {"_ZTVN12_GLOBAL__N_111TCEHostInfoE", "vtable for (anonymous namespace)::TCEHostInfo"},
    {"_ZTVN12_GLOBAL__N_113LinuxHostInfoE", "vtable for (anonymous namespace)::LinuxHostInfo"},
    {"_ZTVN12_GLOBAL__N_117DragonFlyHostInfoE", "vtable for (anonymous namespace)::DragonFlyHostInfo"},
    {"_ZTVN12_GLOBAL__N_113MinixHostInfoE", "vtable for (anonymous namespace)::MinixHostInfo"},
    {"_ZTVN12_GLOBAL__N_114NetBSDHostInfoE", "vtable for (anonymous namespace)::NetBSDHostInfo"},
    {"_ZTVN12_GLOBAL__N_115FreeBSDHostInfoE", "vtable for (anonymous namespace)::FreeBSDHostInfo"},
    {"_ZTVN12_GLOBAL__N_115OpenBSDHostInfoE", "vtable for (anonymous namespace)::OpenBSDHostInfo"},
    {"_ZTVN12_GLOBAL__N_114DarwinHostInfoE", "vtable for (anonymous namespace)::DarwinHostInfo"},
    {"_ZTVN12_GLOBAL__N_116AuroraUXHostInfoE", "vtable for (anonymous namespace)::AuroraUXHostInfo"},
    {"_ZTVN5clang6driver3JobE", "vtable for clang::driver::Job"},
    {"_ZTVN5clang6driver7JobListE", "vtable for clang::driver::JobList"},
    {"_ZTVN5clang6driver6OptionE", "vtable for clang::driver::Option"},
    {"_ZTVN5clang6driver11OptionGroupE", "vtable for clang::driver::OptionGroup"},
    {"_ZTVN5clang6driver11InputOptionE", "vtable for clang::driver::InputOption"},
    {"_ZTVN5clang6driver13UnknownOptionE", "vtable for clang::driver::UnknownOption"},
    {"_ZTVN5clang6driver10FlagOptionE", "vtable for clang::driver::FlagOption"},
    {"_ZTVN5clang6driver12JoinedOptionE", "vtable for clang::driver::JoinedOption"},
    {"_ZTVN5clang6driver17CommaJoinedOptionE", "vtable for clang::driver::CommaJoinedOption"},
    {"_ZTVN5clang6driver14SeparateOptionE", "vtable for clang::driver::SeparateOption"},
    {"_ZTVN5clang6driver14MultiArgOptionE", "vtable for clang::driver::MultiArgOption"},
    {"_ZTVN5clang6driver22JoinedOrSeparateOptionE", "vtable for clang::driver::JoinedOrSeparateOption"},
    {"_ZTVN5clang6driver23JoinedAndSeparateOptionE", "vtable for clang::driver::JoinedAndSeparateOption"},
    {"_ZTVN5clang6driver4ToolE", "vtable for clang::driver::Tool"},
    {"_ZTVN5clang6driver9ToolChainE", "vtable for clang::driver::ToolChain"},
    {"_ZTVN5clang6driver10toolchains6DarwinE", "vtable for clang::driver::toolchains::Darwin"},
    {"_ZTVN5clang6driver10toolchains9DarwinGCCE", "vtable for clang::driver::toolchains::DarwinGCC"},
    {"_ZTVN5clang6driver10toolchains11DarwinClangE", "vtable for clang::driver::toolchains::DarwinClang"},
    {"_ZTVN5clang6driver10toolchains11Generic_GCCE", "vtable for clang::driver::toolchains::Generic_GCC"},
    {"_ZTVN5clang6driver10toolchains12TCEToolChainE", "vtable for clang::driver::toolchains::TCEToolChain"},
    {"_ZTVN5clang6driver10toolchains7OpenBSDE", "vtable for clang::driver::toolchains::OpenBSD"},
    {"_ZTVN5clang6driver10toolchains7FreeBSDE", "vtable for clang::driver::toolchains::FreeBSD"},
    {"_ZTVN5clang6driver10toolchains6NetBSDE", "vtable for clang::driver::toolchains::NetBSD"},
    {"_ZTVN5clang6driver10toolchains5MinixE", "vtable for clang::driver::toolchains::Minix"},
    {"_ZTVN5clang6driver10toolchains8AuroraUXE", "vtable for clang::driver::toolchains::AuroraUX"},
    {"_ZTVN5clang6driver10toolchains5LinuxE", "vtable for clang::driver::toolchains::Linux"},
    {"_ZZN5clang6driver10toolchains5LinuxC1ERKNS0_8HostInfoERKN4llvm6TripleEE11GccVersions", "clang::driver::toolchains::Linux::Linux(clang::driver::HostInfo const&, llvm::Triple const&)::GccVersions"},
    {"_ZTVN5clang6driver10toolchains9DragonFlyE", "vtable for clang::driver::toolchains::DragonFly"},
    {"_ZTVN5clang6driver10toolchains7WindowsE", "vtable for clang::driver::toolchains::Windows"},
    {"_ZTVN5clang6driver10toolchains18Darwin_Generic_GCCE", "vtable for clang::driver::toolchains::Darwin_Generic_GCC"},
    {"_ZTVN5clang6driver5tools5ClangE", "vtable for clang::driver::tools::Clang"},
    {"_ZTVN5clang6driver5tools7ClangAsE", "vtable for clang::driver::tools::ClangAs"},
    {"_ZTVN5clang6driver5tools3gcc6CommonE", "vtable for clang::driver::tools::gcc::Common"},
    {"_ZTVN5clang6driver5tools3gcc10PreprocessE", "vtable for clang::driver::tools::gcc::Preprocess"},
    {"_ZTVN5clang6driver5tools3gcc10PrecompileE", "vtable for clang::driver::tools::gcc::Precompile"},
    {"_ZTVN5clang6driver5tools3gcc7CompileE", "vtable for clang::driver::tools::gcc::Compile"},
    {"_ZTVN5clang6driver5tools3gcc8AssembleE", "vtable for clang::driver::tools::gcc::Assemble"},
    {"_ZTVN5clang6driver5tools3gcc4LinkE", "vtable for clang::driver::tools::gcc::Link"},
    {"_ZTVN5clang6driver5tools6darwin10PreprocessE", "vtable for clang::driver::tools::darwin::Preprocess"},
    {"_ZTVN5clang6driver5tools6darwin7CompileE", "vtable for clang::driver::tools::darwin::Compile"},
    {"_ZTVN5clang6driver5tools6darwin8AssembleE", "vtable for clang::driver::tools::darwin::Assemble"},
    {"_ZTVN5clang6driver5tools6darwin4LinkE", "vtable for clang::driver::tools::darwin::Link"},
    {"_ZTVN5clang6driver5tools6darwin4LipoE", "vtable for clang::driver::tools::darwin::Lipo"},
    {"_ZTVN5clang6driver5tools6darwin8DsymutilE", "vtable for clang::driver::tools::darwin::Dsymutil"},
    {"_ZTVN5clang6driver5tools7openbsd8AssembleE", "vtable for clang::driver::tools::openbsd::Assemble"},
    {"_ZTVN5clang6driver5tools7openbsd4LinkE", "vtable for clang::driver::tools::openbsd::Link"},
    {"_ZTVN5clang6driver5tools7freebsd8AssembleE", "vtable for clang::driver::tools::freebsd::Assemble"},
    {"_ZTVN5clang6driver5tools7freebsd4LinkE", "vtable for clang::driver::tools::freebsd::Link"},
    {"_ZTVN5clang6driver5tools6netbsd8AssembleE", "vtable for clang::driver::tools::netbsd::Assemble"},
    {"_ZTVN5clang6driver5tools6netbsd4LinkE", "vtable for clang::driver::tools::netbsd::Link"},
    {"_ZTVN5clang6driver5tools10linuxtools8AssembleE", "vtable for clang::driver::tools::linuxtools::Assemble"},
    {"_ZTVN5clang6driver5tools10linuxtools4LinkE", "vtable for clang::driver::tools::linuxtools::Link"},
    {"_ZTVN5clang6driver5tools5minix8AssembleE", "vtable for clang::driver::tools::minix::Assemble"},
    {"_ZTVN5clang6driver5tools5minix4LinkE", "vtable for clang::driver::tools::minix::Link"},
    {"_ZTVN5clang6driver5tools8auroraux8AssembleE", "vtable for clang::driver::tools::auroraux::Assemble"},
    {"_ZTVN5clang6driver5tools8auroraux4LinkE", "vtable for clang::driver::tools::auroraux::Link"},
    {"_ZTVN5clang6driver5tools9dragonfly8AssembleE", "vtable for clang::driver::tools::dragonfly::Assemble"},
    {"_ZTVN5clang6driver5tools9dragonfly4LinkE", "vtable for clang::driver::tools::dragonfly::Link"},
    {"_ZTVN5clang6driver5tools12visualstudio4LinkE", "vtable for clang::driver::tools::visualstudio::Link"},
    {"_ZTVN5clang26ASTDeserializationListenerE", "vtable for clang::ASTDeserializationListener"},
    {"_ZTVN5clang17ASTReaderListenerE", "vtable for clang::ASTReaderListener"},
    {"_ZTVN5clang21ASTIdentifierIteratorE", "vtable for clang::ASTIdentifierIterator"},
    {"_ZTVN5clang9ASTReaderE", "vtable for clang::ASTReader"},
    {"_ZTVN5clang12PCHValidatorE", "vtable for clang::PCHValidator"},
    {"_ZTVN12_GLOBAL__N_112ASTStatCacheE", "vtable for (anonymous namespace)::ASTStatCache"},
    {"_ZTVN5clang9ASTWriterE", "vtable for clang::ASTWriter"},
    {"_ZTVN5clang24ASTSerializationListenerE", "vtable for clang::ASTSerializationListener"},
    {"_ZTVN5clang12PCHGeneratorE", "vtable for clang::PCHGenerator"},
    {"_ZTVN5clang7CodeGen8CGCXXABIE", "vtable for clang::CodeGen::CGCXXABI"},
    {"_ZTVN12_GLOBAL__N_113CallLocalDtorE", "vtable for (anonymous namespace)::CallLocalDtor"},
    {"_ZTVN12_GLOBAL__N_113CallFieldDtorE", "vtable for (anonymous namespace)::CallFieldDtor"},
    {"_ZTVN12_GLOBAL__N_118CallArrayFieldDtorE", "vtable for (anonymous namespace)::CallArrayFieldDtor"},
    {"_ZTVN12_GLOBAL__N_114CallDtorDeleteE", "vtable for (anonymous namespace)::CallDtorDelete"},
    {"_ZTVN12_GLOBAL__N_112CallBaseDtorE", "vtable for (anonymous namespace)::CallBaseDtor"},
    {"_ZTVN12_GLOBAL__N_114CallMemberDtorE", "vtable for (anonymous namespace)::CallMemberDtor"},
    {"_ZTVN5clang7CodeGen12EHScopeStack7CleanupE", "vtable for clang::CodeGen::EHScopeStack::Cleanup"},
    {"_ZTVN12_GLOBAL__N_116CallBlockReleaseE", "vtable for (anonymous namespace)::CallBlockRelease"},
    {"_ZTVN12_GLOBAL__N_119CallCleanupFunctionE", "vtable for (anonymous namespace)::CallCleanupFunction"},
    {"_ZTVN12_GLOBAL__N_111CallVarDtorE", "vtable for (anonymous namespace)::CallVarDtor"},
    {"_ZTVN12_GLOBAL__N_113CallArrayDtorE", "vtable for (anonymous namespace)::CallArrayDtor"},
    {"_ZTVN12_GLOBAL__N_116CallStackRestoreE", "vtable for (anonymous namespace)::CallStackRestore"},
    {"_ZTVN12_GLOBAL__N_114PerformFinallyE", "vtable for (anonymous namespace)::PerformFinally"},
    {"_ZTVN12_GLOBAL__N_122CallEndCatchForFinallyE", "vtable for (anonymous namespace)::CallEndCatchForFinally"},
    {"_ZTVN12_GLOBAL__N_111CallRethrowE", "vtable for (anonymous namespace)::CallRethrow"},
    {"_ZTVN12_GLOBAL__N_112CallEndCatchE", "vtable for (anonymous namespace)::CallEndCatch"},
    {"_ZTVN12_GLOBAL__N_116CallObjectDeleteE", "vtable for (anonymous namespace)::CallObjectDelete"},
    {"_ZTVN12_GLOBAL__N_115CallArrayDeleteE", "vtable for (anonymous namespace)::CallArrayDelete"},
    {"_ZTVN12_GLOBAL__N_130CallDeleteDuringConditionalNewE", "vtable for (anonymous namespace)::CallDeleteDuringConditionalNew"},
    {"_ZTVN12_GLOBAL__N_119CallDeleteDuringNewE", "vtable for (anonymous namespace)::CallDeleteDuringNew"},
    {"_ZTVN5clang7CodeGen13CGObjCRuntimeE", "vtable for clang::CodeGen::CGObjCRuntime"},
    {"_ZTVN12_GLOBAL__N_19CGObjCGNUE", "vtable for (anonymous namespace)::CGObjCGNU"},
    {"_ZTVN12_GLOBAL__N_112CallSyncExitE", "vtable for (anonymous namespace)::CallSyncExit"},
    {"_ZTVN12_GLOBAL__N_122CGObjCNonFragileABIMacE", "vtable for (anonymous namespace)::CGObjCNonFragileABIMac"},
    {"_ZTVN12_GLOBAL__N_116CallObjCEndCatchE", "vtable for (anonymous namespace)::CallObjCEndCatch"},
    {"_ZTVN12_GLOBAL__N_112CallSyncExitE", "vtable for (anonymous namespace)::CallSyncExit"},
    {"_ZTVN12_GLOBAL__N_115CGObjCCommonMacE", "vtable for (anonymous namespace)::CGObjCCommonMac"},
    {"_ZTVN12_GLOBAL__N_19CGObjCMacE", "vtable for (anonymous namespace)::CGObjCMac"},
    {"_ZTVN12_GLOBAL__N_121PerformFragileFinallyE", "vtable for (anonymous namespace)::PerformFragileFinally"},
    {"_ZTVN5clang13CodeGenActionE", "vtable for clang::CodeGenAction"},
    {"_ZTVN12_GLOBAL__N_19ARMCXXABIE", "vtable for (anonymous namespace)::ARMCXXABI"},
    {"_ZTVN12_GLOBAL__N_114CallGuardAbortE", "vtable for (anonymous namespace)::CallGuardAbort"},
    {"_ZTVN12_GLOBAL__N_113ItaniumCXXABIE", "vtable for (anonymous namespace)::ItaniumCXXABI"},
    {"_ZTVN12_GLOBAL__N_115MicrosoftCXXABIE", "vtable for (anonymous namespace)::MicrosoftCXXABI"},
    {"_ZTVN12_GLOBAL__N_117CodeGeneratorImplE", "vtable for (anonymous namespace)::CodeGeneratorImpl"},
    {"_ZTVN5clang7ABIInfoE", "vtable for clang::ABIInfo"},
    {"_ZTVN5clang17TargetCodeGenInfoE", "vtable for clang::TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_123X86_64TargetCodeGenInfoE", "vtable for (anonymous namespace)::X86_64TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_113X86_64ABIInfoE", "vtable for (anonymous namespace)::X86_64ABIInfo"},
    {"_ZTVN12_GLOBAL__N_126WinX86_64TargetCodeGenInfoE", "vtable for (anonymous namespace)::WinX86_64TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_116WinX86_64ABIInfoE", "vtable for (anonymous namespace)::WinX86_64ABIInfo"},
    {"_ZTVN12_GLOBAL__N_123X86_32TargetCodeGenInfoE", "vtable for (anonymous namespace)::X86_32TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_113X86_32ABIInfoE", "vtable for (anonymous namespace)::X86_32ABIInfo"},
    {"_ZTVN12_GLOBAL__N_123MSP430TargetCodeGenInfoE", "vtable for (anonymous namespace)::MSP430TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_114DefaultABIInfoE", "vtable for (anonymous namespace)::DefaultABIInfo"},
    {"_ZTVN12_GLOBAL__N_123MBlazeTargetCodeGenInfoE", "vtable for (anonymous namespace)::MBlazeTargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_113MBlazeABIInfoE", "vtable for (anonymous namespace)::MBlazeABIInfo"},
    {"_ZTVN12_GLOBAL__N_124SystemZTargetCodeGenInfoE", "vtable for (anonymous namespace)::SystemZTargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_114SystemZABIInfoE", "vtable for (anonymous namespace)::SystemZABIInfo"},
    {"_ZTVN12_GLOBAL__N_122PPC32TargetCodeGenInfoE", "vtable for (anonymous namespace)::PPC32TargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_124DefaultTargetCodeGenInfoE", "vtable for (anonymous namespace)::DefaultTargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_120ARMTargetCodeGenInfoE", "vtable for (anonymous namespace)::ARMTargetCodeGenInfo"},
    {"_ZTVN12_GLOBAL__N_110ARMABIInfoE", "vtable for (anonymous namespace)::ARMABIInfo"},
    {"_ZTVN12_GLOBAL__N_121MIPSTargetCodeGenInfoE", "vtable for (anonymous namespace)::MIPSTargetCodeGenInfo"},
    {"_ZTVN5clang6Parser21LateParsedDeclarationE", "vtable for clang::Parser::LateParsedDeclaration"},
    {"_ZTVN5clang6Parser15LateParsedClassE", "vtable for clang::Parser::LateParsedClass"},
    {"_ZTVN5clang6Parser11LexedMethodE", "vtable for clang::Parser::LexedMethod"},
    {"_ZTVN5clang6Parser27LateParsedMethodDeclarationE", "vtable for clang::Parser::LateParsedMethodDeclaration"},
    {"_ZTVZN5clang6Parser20ParseStructUnionBodyENS_14SourceLocationEjPNS_4DeclEE14CFieldCallback", "vtable for clang::Parser::ParseStructUnionBody(clang::SourceLocation, unsigned int, clang::Decl*)::CFieldCallback"},
    {"_ZTVZN5clang6Parser31ParseObjCClassInstanceVariablesEPNS_4DeclENS_3tok15ObjCKeywordKindENS_14SourceLocationEE16ObjCIvarCallback", "vtable for clang::Parser::ParseObjCClassInstanceVariables(clang::Decl*, clang::tok::ObjCKeywordKind, clang::SourceLocation)::ObjCIvarCallback"},
    {"_ZTVN5clang18PragmaAlignHandlerE", "vtable for clang::PragmaAlignHandler"},
    {"_ZTVN5clang26PragmaGCCVisibilityHandlerE", "vtable for clang::PragmaGCCVisibilityHandler"},
    {"_ZTVN5clang20PragmaOptionsHandlerE", "vtable for clang::PragmaOptionsHandler"},
    {"_ZTVN5clang17PragmaPackHandlerE", "vtable for clang::PragmaPackHandler"},
    {"_ZTVN5clang19PragmaUnusedHandlerE", "vtable for clang::PragmaUnusedHandler"},
    {"_ZTVN5clang17PragmaWeakHandlerE", "vtable for clang::PragmaWeakHandler"},
    {"_ZTVN5clang6ParserE", "vtable for clang::Parser"},
    {"_ZTVN5clang27PrettyStackTraceParserEntryE", "vtable for clang::PrettyStackTraceParserEntry"},
    {"_ZTVN5clang6Parser13FieldCallbackE", "vtable for clang::Parser::FieldCallback"},
    {"_ZTVN12_GLOBAL__N_122UninitValsDiagReporterE", "vtable for (anonymous namespace)::UninitValsDiagReporter"},
    {"_ZTVN12_GLOBAL__N_122UnreachableCodeHandlerE", "vtable for (anonymous namespace)::UnreachableCodeHandler"},
    {"_ZTVN5clang20CodeCompleteConsumerE", "vtable for clang::CodeCompleteConsumer"},
    {"_ZTVN5clang28PrintingCodeCompleteConsumerE", "vtable for clang::PrintingCodeCompleteConsumer"},
    {"_ZTVN5clang4sema17FunctionScopeInfoE", "vtable for clang::sema::FunctionScopeInfo"},
    {"_ZTVN5clang4sema14BlockScopeInfoE", "vtable for clang::sema::BlockScopeInfo"},
    {"_ZTVN5clang18ExternalSemaSourceE", "vtable for clang::ExternalSemaSource"},
    {"_ZTVN5clang25PrettyDeclStackTraceEntryE", "vtable for clang::PrettyDeclStackTraceEntry"},
    {"_ZTVN12_GLOBAL__N_117CheckScanfHandlerE", "vtable for (anonymous namespace)::CheckScanfHandler"},
    {"_ZTVN12_GLOBAL__N_118CheckFormatHandlerE", "vtable for (anonymous namespace)::CheckFormatHandler"},
    {"_ZTVN12_GLOBAL__N_118CheckPrintfHandlerE", "vtable for (anonymous namespace)::CheckPrintfHandler"},
    {"_ZTVN12_GLOBAL__N_126CodeCompletionDeclConsumerE", "vtable for (anonymous namespace)::CodeCompletionDeclConsumer"},
    {"_ZTVN5clang19VisibleDeclConsumerE", "vtable for clang::VisibleDeclConsumer"},
    {"_ZZN5clang4Sema11CorrectTypoERNS_12LookupResultEPNS_5ScopeEPNS_12CXXScopeSpecEPNS_11DeclContextEbNS0_18CorrectTypoContextEPKNS_21ObjCObjectPointerTypeEE10CTypeSpecs", "clang::Sema::CorrectTypo(clang::LookupResult&, clang::Scope*, clang::CXXScopeSpec*, clang::DeclContext*, bool, clang::Sema::CorrectTypoContext, clang::ObjCObjectPointerType const*)::CTypeSpecs"},
    {"_ZZN5clang4Sema11CorrectTypoERNS_12LookupResultEPNS_5ScopeEPNS_12CXXScopeSpecEPNS_11DeclContextEbNS0_18CorrectTypoContextEPKNS_21ObjCObjectPointerTypeEE6CStmts", "clang::Sema::CorrectTypo(clang::LookupResult&, clang::Scope*, clang::CXXScopeSpec*, clang::DeclContext*, bool, clang::Sema::CorrectTypoContext, clang::ObjCObjectPointerType const*)::CStmts"},
    {"_ZTVN12_GLOBAL__N_122TypoCorrectionConsumerE", "vtable for (anonymous namespace)::TypoCorrectionConsumer"},
    {"_ZZN5clang25GetImplicitConversionNameENS_22ImplicitConversionKindEE4Name", "clang::GetImplicitConversionName(clang::ImplicitConversionKind)::Name"},
    {"_ZTVN5clang20TargetAttributesSemaE", "vtable for clang::TargetAttributesSema"},
    {"_ZTVN12_GLOBAL__N_117X86AttributesSemaE", "vtable for (anonymous namespace)::X86AttributesSema"},
    {"_ZTVN12_GLOBAL__N_120MBlazeAttributesSemaE", "vtable for (anonymous namespace)::MBlazeAttributesSema"},
    {"_ZTVN12_GLOBAL__N_120MSP430AttributesSemaE", "vtable for (anonymous namespace)::MSP430AttributesSema"},
    {"_ZTVN12_GLOBAL__N_126AdjustedReturnValueCheckerE", "vtable for (anonymous namespace)::AdjustedReturnValueChecker"},
    {"_ZTVN12_GLOBAL__N_111UbigraphVizE", "vtable for (anonymous namespace)::UbigraphViz"},
    {"_ZTVN12_GLOBAL__N_116AnalysisConsumerE", "vtable for (anonymous namespace)::AnalysisConsumer"},
    {"_ZTVN12_GLOBAL__N_120AnalyzerStatsCheckerE", "vtable for (anonymous namespace)::AnalyzerStatsChecker"},
    {"_ZTVN12_GLOBAL__N_117ArrayBoundCheckerE", "vtable for (anonymous namespace)::ArrayBoundChecker"},
    {"_ZTVN12_GLOBAL__N_119ArrayBoundCheckerV2E", "vtable for (anonymous namespace)::ArrayBoundCheckerV2"},
    {"_ZTVN12_GLOBAL__N_118AttrNonNullCheckerE", "vtable for (anonymous namespace)::AttrNonNullChecker"},
    {"_ZTVN12_GLOBAL__N_119ClassReleaseCheckerE", "vtable for (anonymous namespace)::ClassReleaseChecker"},
    {"_ZTVN12_GLOBAL__N_19APIMisuseE", "vtable for (anonymous namespace)::APIMisuse"},
    {"_ZTVN12_GLOBAL__N_122CFRetainReleaseCheckerE", "vtable for (anonymous namespace)::CFRetainReleaseChecker"},
    {"_ZTVN12_GLOBAL__N_121CFNumberCreateCheckerE", "vtable for (anonymous namespace)::CFNumberCreateChecker"},
    {"_ZTVN12_GLOBAL__N_113NilArgCheckerE", "vtable for (anonymous namespace)::NilArgChecker"},
    {"_ZTVN12_GLOBAL__N_122BuiltinFunctionCheckerE", "vtable for (anonymous namespace)::BuiltinFunctionChecker"},
    {"_ZTVN12_GLOBAL__N_114CStringCheckerE", "vtable for (anonymous namespace)::CStringChecker"},
    {"_ZTVN12_GLOBAL__N_121CallAndMessageCheckerE", "vtable for (anonymous namespace)::CallAndMessageChecker"},
    {"_ZTVN12_GLOBAL__N_115CastSizeCheckerE", "vtable for (anonymous namespace)::CastSizeChecker"},
    {"_ZTVN12_GLOBAL__N_119CastToStructCheckerE", "vtable for (anonymous namespace)::CastToStructChecker"},
    {"_ZZN12_GLOBAL__N_17WalkAST14CheckCall_randEPKN5clang8CallExprEPKNS1_12FunctionDeclEE11identifiers", "(anonymous namespace)::WalkAST::CheckCall_rand(clang::CallExpr const*, clang::FunctionDecl const*)::identifiers"},
    {"_ZTVN12_GLOBAL__N_113ChrootCheckerE", "vtable for (anonymous namespace)::ChrootChecker"},
    {"_ZTVN12_GLOBAL__N_112DeadStoreObsE", "vtable for (anonymous namespace)::DeadStoreObs"},
    {"_ZTVN12_GLOBAL__N_118DereferenceCheckerE", "vtable for (anonymous namespace)::DereferenceChecker"},
    {"_ZTVN12_GLOBAL__N_114DivZeroCheckerE", "vtable for (anonymous namespace)::DivZeroChecker"},
    {"_ZTVN5clang4ento10ExprEngineE", "vtable for clang::ento::ExprEngine"},
    {"_ZTVN12_GLOBAL__N_119FixedAddressCheckerE", "vtable for (anonymous namespace)::FixedAddressChecker"},
    {"_ZTVN5clang4ento14AnalysisActionE", "vtable for clang::ento::AnalysisAction"},
    {"_ZTVN12_GLOBAL__N_126IdempotentOperationCheckerE", "vtable for (anonymous namespace)::IdempotentOperationChecker"},
    {"_ZTVZN12_GLOBAL__N_126IdempotentOperationChecker25PathWasCompletelyAnalyzedEPKN5clang3CFGEPKNS1_8CFGBlockEPKNS1_10CFGStmtMapERKNS1_4ento10CoreEngineEE7VisitWL", "vtable for (anonymous namespace)::IdempotentOperationChecker::PathWasCompletelyAnalyzed(clang::CFG const*, clang::CFGBlock const*, clang::CFGStmtMap const*, clang::ento::CoreEngine const&)::VisitWL"},
    {"_ZTVN12_GLOBAL__N_116MacOSXAPICheckerE", "vtable for (anonymous namespace)::MacOSXAPIChecker"},
    {"_ZTVN12_GLOBAL__N_113MallocCheckerE", "vtable for (anonymous namespace)::MallocChecker"},
    {"_ZTVN12_GLOBAL__N_124NSAutoreleasePoolCheckerE", "vtable for (anonymous namespace)::NSAutoreleasePoolChecker"},
    {"_ZTVN12_GLOBAL__N_114NSErrorCheckerE", "vtable for (anonymous namespace)::NSErrorChecker"},
    {"_ZTVN12_GLOBAL__N_123NoReturnFunctionCheckerE", "vtable for (anonymous namespace)::NoReturnFunctionChecker"},
    {"_ZTVN12_GLOBAL__N_115OSAtomicCheckerE", "vtable for (anonymous namespace)::OSAtomicChecker"},
    {"_ZTVN12_GLOBAL__N_117ObjCAtSyncCheckerE", "vtable for (anonymous namespace)::ObjCAtSyncChecker"},
    {"_ZTVN12_GLOBAL__N_119ObjCSelfInitCheckerE", "vtable for (anonymous namespace)::ObjCSelfInitChecker"},
    {"_ZTVN12_GLOBAL__N_111InitSelfBugE", "vtable for (anonymous namespace)::InitSelfBug"},
    {"_ZTVN12_GLOBAL__N_119PointerArithCheckerE", "vtable for (anonymous namespace)::PointerArithChecker"},
    {"_ZTVN12_GLOBAL__N_117PointerSubCheckerE", "vtable for (anonymous namespace)::PointerSubChecker"},
    {"_ZTVN12_GLOBAL__N_118PthreadLockCheckerE", "vtable for (anonymous namespace)::PthreadLockChecker"},
    {"_ZTVN12_GLOBAL__N_125ReturnPointerRangeCheckerE", "vtable for (anonymous namespace)::ReturnPointerRangeChecker"},
    {"_ZTVN12_GLOBAL__N_118ReturnUndefCheckerE", "vtable for (anonymous namespace)::ReturnUndefChecker"},
    {"_ZTVN12_GLOBAL__N_120StackAddrLeakCheckerE", "vtable for (anonymous namespace)::StackAddrLeakChecker"},
    {"_ZTVZN12_GLOBAL__N_120StackAddrLeakChecker11evalEndPathERN5clang4ento24EndOfFunctionNodeBuilderEPvRNS2_10ExprEngineEE8CallBack", "vtable for (anonymous namespace)::StackAddrLeakChecker::evalEndPath(clang::ento::EndOfFunctionNodeBuilder&, void*, clang::ento::ExprEngine&)::CallBack"},
    {"_ZTVN12_GLOBAL__N_113StreamCheckerE", "vtable for (anonymous namespace)::StreamChecker"},
    {"_ZTVN12_GLOBAL__N_118UndefBranchCheckerE", "vtable for (anonymous namespace)::UndefBranchChecker"},
    {"_ZTVN12_GLOBAL__N_128UndefCapturedBlockVarCheckerE", "vtable for (anonymous namespace)::UndefCapturedBlockVarChecker"},
    {"_ZTVN12_GLOBAL__N_118UndefResultCheckerE", "vtable for (anonymous namespace)::UndefResultChecker"},
    {"_ZTVN12_GLOBAL__N_130UndefinedArraySubscriptCheckerE", "vtable for (anonymous namespace)::UndefinedArraySubscriptChecker"},
    {"_ZTVN12_GLOBAL__N_126UndefinedAssignmentCheckerE", "vtable for (anonymous namespace)::UndefinedAssignmentChecker"},
    {"_ZTVN12_GLOBAL__N_114UnixAPICheckerE", "vtable for (anonymous namespace)::UnixAPIChecker"},
    {"_ZTVN12_GLOBAL__N_122UnreachableCodeCheckerE", "vtable for (anonymous namespace)::UnreachableCodeChecker"},
    {"_ZTVN12_GLOBAL__N_114VLASizeCheckerE", "vtable for (anonymous namespace)::VLASizeChecker"},
    {"_ZTVN12_GLOBAL__N_122BasicConstraintManagerE", "vtable for (anonymous namespace)::BasicConstraintManager"},
    {"_ZTVN5clang4ento12StoreManager15BindingsHandlerE", "vtable for clang::ento::StoreManager::BindingsHandler"},
    {"_ZTVN12_GLOBAL__N_117BasicStoreManagerE", "vtable for (anonymous namespace)::BasicStoreManager"},
    {"_ZTVN12_GLOBAL__N_122BasicStoreSubRegionMapE", "vtable for (anonymous namespace)::BasicStoreSubRegionMap"},
    {"_ZTVN5clang4ento18BugReporterVisitorE", "vtable for clang::ento::BugReporterVisitor"},
    {"_ZTVN5clang4ento18BugReporterContextE", "vtable for clang::ento::BugReporterContext"},
    {"_ZTVN5clang4ento7BugTypeE", "vtable for clang::ento::BugType"},
    {"_ZTVN5clang4ento9BugReportE", "vtable for clang::ento::BugReport"},
    {"_ZTVN5clang4ento15RangedBugReportE", "vtable for clang::ento::RangedBugReport"},
    {"_ZTVN5clang4ento13GRBugReporterE", "vtable for clang::ento::GRBugReporter"},
    {"_ZTVN5clang4ento15BugReporterDataE", "vtable for clang::ento::BugReporterData"},
    {"_ZTVN5clang4ento11BugReporterE", "vtable for clang::ento::BugReporter"},
    {"_ZTVN12_GLOBAL__N_121PathDiagnosticBuilderE", "vtable for (anonymous namespace)::PathDiagnosticBuilder"},
    {"_ZTVN12_GLOBAL__N_114NodeMapClosureE", "vtable for (anonymous namespace)::NodeMapClosure"},
    {"_ZTVN12_GLOBAL__N_118ScanNotableSymbolsE", "vtable for (anonymous namespace)::ScanNotableSymbols"},
    {"_ZTVN12_GLOBAL__N_120NotableSymbolHandlerE", "vtable for (anonymous namespace)::NotableSymbolHandler"},
    {"_ZTVN12_GLOBAL__N_118NilReceiverVisitorE", "vtable for (anonymous namespace)::NilReceiverVisitor"},
    {"_ZTVN12_GLOBAL__N_122FindLastStoreBRVisitorE", "vtable for (anonymous namespace)::FindLastStoreBRVisitor"},
    {"_ZTVN12_GLOBAL__N_124TrackConstraintBRVisitorE", "vtable for (anonymous namespace)::TrackConstraintBRVisitor"},
    {"_ZTVN12_GLOBAL__N_110CFRefCountE", "vtable for (anonymous namespace)::CFRefCount"},
    {"_ZTVN12_GLOBAL__N_111CFRefReportE", "vtable for (anonymous namespace)::CFRefReport"},
    {"_ZTVN12_GLOBAL__N_115CFRefLeakReportE", "vtable for (anonymous namespace)::CFRefLeakReport"},
    {"_ZTVN12_GLOBAL__N_117FindUniqueBindingE", "vtable for (anonymous namespace)::FindUniqueBinding"},
    {"_ZTVN12_GLOBAL__N_120StopTrackingCallbackE", "vtable for (anonymous namespace)::StopTrackingCallback"},
    {"_ZTVN12_GLOBAL__N_120RetainReleaseCheckerE", "vtable for (anonymous namespace)::RetainReleaseChecker"},
    {"_ZTVN12_GLOBAL__N_118LeakWithinFunctionE", "vtable for (anonymous namespace)::LeakWithinFunction"},
    {"_ZTVN12_GLOBAL__N_18CFRefBugE", "vtable for (anonymous namespace)::CFRefBug"},
    {"_ZTVN12_GLOBAL__N_112LeakAtReturnE", "vtable for (anonymous namespace)::LeakAtReturn"},
    {"_ZTVN12_GLOBAL__N_124ReturnedNotOwnedForOwnedE", "vtable for (anonymous namespace)::ReturnedNotOwnedForOwned"},
    {"_ZTVN12_GLOBAL__N_115OverAutoreleaseE", "vtable for (anonymous namespace)::OverAutorelease"},
    {"_ZTVN12_GLOBAL__N_115DeallocNotOwnedE", "vtable for (anonymous namespace)::DeallocNotOwned"},
    {"_ZTVN12_GLOBAL__N_19DeallocGCE", "vtable for (anonymous namespace)::DeallocGC"},
    {"_ZTVN12_GLOBAL__N_110BadReleaseE", "vtable for (anonymous namespace)::BadRelease"},
    {"_ZTVN12_GLOBAL__N_115UseAfterReleaseE", "vtable for (anonymous namespace)::UseAfterRelease"},
    {"_ZTVN12_GLOBAL__N_110CFRefCount15BindingsPrinterE", "vtable for (anonymous namespace)::CFRefCount::BindingsPrinter"},
    {"_ZTVN5clang4ento7CheckerE", "vtable for clang::ento::Checker"},
    {"_ZTVN5clang4ento8WorkList7VisitorE", "vtable for clang::ento::WorkList::Visitor"},
    {"_ZTVN5clang4ento8WorkListE", "vtable for clang::ento::WorkList"},
    {"_ZTVN12_GLOBAL__N_119BFSBlockDFSContentsE", "vtable for (anonymous namespace)::BFSBlockDFSContents"},
    {"_ZTVN12_GLOBAL__N_13BFSE", "vtable for (anonymous namespace)::BFS"},
    {"_ZTVN12_GLOBAL__N_13DFSE", "vtable for (anonymous namespace)::DFS"},
    {"_ZTVN12_GLOBAL__N_116MarkLiveCallbackE", "vtable for (anonymous namespace)::MarkLiveCallback"},
    {"_ZTVN5clang4ento12ExplodedNode7AuditorE", "vtable for clang::ento::ExplodedNode::Auditor"},
    {"_ZTVN12_GLOBAL__N_116FlatStoreManagerE", "vtable for (anonymous namespace)::FlatStoreManager"},
    {"_ZTVN5clang4ento17ConstraintManagerE", "vtable for clang::ento::ConstraintManager"},
    {"_ZTVN12_GLOBAL__N_120ScanReachableSymbolsE", "vtable for (anonymous namespace)::ScanReachableSymbols"},
    {"_ZTVN12_GLOBAL__N_115HTMLDiagnosticsE", "vtable for (anonymous namespace)::HTMLDiagnostics"},
    {"_ZTVN5clang4ento9MemRegionE", "vtable for clang::ento::MemRegion"},
    {"_ZTVN5clang4ento14MemSpaceRegionE", "vtable for clang::ento::MemSpaceRegion"},
    {"_ZTVN5clang4ento23StaticGlobalSpaceRegionE", "vtable for clang::ento::StaticGlobalSpaceRegion"},
    {"_ZTVN5clang4ento26NonStaticGlobalSpaceRegionE", "vtable for clang::ento::NonStaticGlobalSpaceRegion"},
    {"_ZTVN5clang4ento16StackSpaceRegionE", "vtable for clang::ento::StackSpaceRegion"},
    {"_ZTVN5clang4ento9SubRegionE", "vtable for clang::ento::SubRegion"},
    {"_ZTVN5clang4ento12AllocaRegionE", "vtable for clang::ento::AllocaRegion"},
    {"_ZTVN5clang4ento18FunctionTextRegionE", "vtable for clang::ento::FunctionTextRegion"},
    {"_ZTVN5clang4ento15BlockTextRegionE", "vtable for clang::ento::BlockTextRegion"},
    {"_ZTVN5clang4ento15BlockDataRegionE", "vtable for clang::ento::BlockDataRegion"},
    {"_ZTVN5clang4ento14SymbolicRegionE", "vtable for clang::ento::SymbolicRegion"},
    {"_ZTVN5clang4ento12StringRegionE", "vtable for clang::ento::StringRegion"},
    {"_ZTVN5clang4ento21CompoundLiteralRegionE", "vtable for clang::ento::CompoundLiteralRegion"},
    {"_ZTVN5clang4ento10DeclRegionE", "vtable for clang::ento::DeclRegion"},
    {"_ZTVN5clang4ento9VarRegionE", "vtable for clang::ento::VarRegion"},
    {"_ZTVN5clang4ento13CXXThisRegionE", "vtable for clang::ento::CXXThisRegion"},
    {"_ZTVN5clang4ento11FieldRegionE", "vtable for clang::ento::FieldRegion"},
    {"_ZTVN5clang4ento14ObjCIvarRegionE", "vtable for clang::ento::ObjCIvarRegion"},
    {"_ZTVN5clang4ento13ElementRegionE", "vtable for clang::ento::ElementRegion"},
    {"_ZTVN5clang4ento19CXXTempObjectRegionE", "vtable for clang::ento::CXXTempObjectRegion"},
    {"_ZTVN5clang4ento19CXXBaseObjectRegionE", "vtable for clang::ento::CXXBaseObjectRegion"},
    {"_ZTVN5clang4ento19PathDiagnosticPieceE", "vtable for clang::ento::PathDiagnosticPiece"},
    {"_ZTVN5clang4ento24PathDiagnosticEventPieceE", "vtable for clang::ento::PathDiagnosticEventPiece"},
    {"_ZTVN5clang4ento30PathDiagnosticControlFlowPieceE", "vtable for clang::ento::PathDiagnosticControlFlowPiece"},
    {"_ZTVN5clang4ento24PathDiagnosticMacroPieceE", "vtable for clang::ento::PathDiagnosticMacroPiece"},
    {"_ZTVN5clang4ento20PathDiagnosticClientE", "vtable for clang::ento::PathDiagnosticClient"},
    {"_ZTVN5clang4ento23PathDiagnosticSpotPieceE", "vtable for clang::ento::PathDiagnosticSpotPiece"},
    {"_ZTVN12_GLOBAL__N_116PlistDiagnosticsE", "vtable for (anonymous namespace)::PlistDiagnostics"},
    {"_ZTVN12_GLOBAL__N_122RangeConstraintManagerE", "vtable for (anonymous namespace)::RangeConstraintManager"},
    {"_ZTVN12_GLOBAL__N_118RegionStoreManagerE", "vtable for (anonymous namespace)::RegionStoreManager"},
    {"_ZTVN12_GLOBAL__N_123RegionStoreSubRegionMapE", "vtable for (anonymous namespace)::RegionStoreSubRegionMap"},
    {"_ZTVN5clang4ento23SimpleConstraintManagerE", "vtable for clang::ento::SimpleConstraintManager"},
    {"_ZTVN12_GLOBAL__N_117SimpleSValBuilderE", "vtable for (anonymous namespace)::SimpleSValBuilder"},
    {"_ZTVN5clang4ento12StoreManagerE", "vtable for clang::ento::StoreManager"},
    {"_ZTVN5clang4ento13SymbolVisitorE", "vtable for clang::ento::SymbolVisitor"},
    {"_ZTVN5clang4ento17SymbolRegionValueE", "vtable for clang::ento::SymbolRegionValue"},
    {"_ZTVN5clang4ento14SymbolConjuredE", "vtable for clang::ento::SymbolConjured"},
    {"_ZTVN5clang4ento13SymbolDerivedE", "vtable for clang::ento::SymbolDerived"},
    {"_ZTVN5clang4ento12SymbolExtentE", "vtable for clang::ento::SymbolExtent"},
    {"_ZTVN5clang4ento14SymbolMetadataE", "vtable for clang::ento::SymbolMetadata"},
    {"_ZTVN5clang4ento10SymIntExprE", "vtable for clang::ento::SymIntExpr"},
    {"_ZTVN5clang4ento10SymSymExprE", "vtable for clang::ento::SymSymExpr"},
    {"_ZTVN12_GLOBAL__N_119TextPathDiagnosticsE", "vtable for (anonymous namespace)::TextPathDiagnostics"},
    {"_ZTVN5clang15LocationContextE", "vtable for clang::LocationContext"},
    {"_ZTVN5clang17StackFrameContextE", "vtable for clang::StackFrameContext"},
    {"_ZTVN5clang12ScopeContextE", "vtable for clang::ScopeContext"},
    {"_ZTVN5clang22BlockInvocationContextE", "vtable for clang::BlockInvocationContext"},
    {"_ZTVN12_GLOBAL__N_117StmtPrinterHelperE", "vtable for (anonymous namespace)::StmtPrinterHelper"},
    {"_ZTVN5clang21analyze_format_string19FormatStringHandlerE", "vtable for clang::analyze_format_string::FormatStringHandler"},
    {"_ZTVN5clang30UninitializedValues_ValueTypes10ObserverTyE", "vtable for clang::UninitializedValues_ValueTypes::ObserverTy"},
    {"_ZTVN12_GLOBAL__N_126UninitializedValuesCheckerE", "vtable for (anonymous namespace)::UninitializedValuesChecker"},
    {"_ZTVN5clang22UninitVariablesHandlerE", "vtable for clang::UninitVariablesHandler"},
    {"_ZTVN5clang3idx13EntityHandlerE", "vtable for clang::idx::EntityHandler"},
    {"_ZTVN5clang3idx22TranslationUnitHandlerE", "vtable for clang::idx::TranslationUnitHandler"},
    {"_ZTVN5clang3idx17TULocationHandlerE", "vtable for clang::idx::TULocationHandler"},
    {"_ZTVN5clang3idx13IndexProviderE", "vtable for clang::idx::IndexProvider"},
    {"_ZTVN5clang3idx7IndexerE", "vtable for clang::idx::Indexer"},
    {"_ZTVN12_GLOBAL__N_113EntityIndexerE", "vtable for (anonymous namespace)::EntityIndexer"},
    {"_ZTVN5clang3idx15TranslationUnitE", "vtable for clang::idx::TranslationUnit"},
    {"_ZTVN5clang13FixItRewriterE", "vtable for clang::FixItRewriter"},
    {"_ZTVN5clang12FixItOptionsE", "vtable for clang::FixItOptions"},
    {"_ZTVN5clang11FixItActionE", "vtable for clang::FixItAction"},
    {"_ZTVN5clang15HTMLPrintActionE", "vtable for clang::HTMLPrintAction"},
    {"_ZTVN5clang17RewriteObjCActionE", "vtable for clang::RewriteObjCAction"},
    {"_ZTVN5clang19RewriteMacrosActionE", "vtable for clang::RewriteMacrosAction"},
    {"_ZTVN5clang17RewriteTestActionE", "vtable for clang::RewriteTestAction"},
    {"_ZTVN12_GLOBAL__N_119FixItRewriteInPlaceE", "vtable for (anonymous namespace)::FixItRewriteInPlace"},
    {"_ZTVN12_GLOBAL__N_125FixItActionSuffixInserterE", "vtable for (anonymous namespace)::FixItActionSuffixInserter"},
    {"_ZTVN12_GLOBAL__N_111HTMLPrinterE", "vtable for (anonymous namespace)::HTMLPrinter"},
    {"_ZTVN12_GLOBAL__N_118IgnoringDiagClientE", "vtable for (anonymous namespace)::IgnoringDiagClient"},
    {"_ZTVN12_GLOBAL__N_111RewriteObjCE", "vtable for (anonymous namespace)::RewriteObjC"},
    {"_ZTVN5clang11ASTConsumerE", "vtable for clang::ASTConsumer"},
    {"_ZTVN5clang17ExternalASTSourceE", "vtable for clang::ExternalASTSource"},
    {"_ZTVN5clang19ASTMutationListenerE", "vtable for clang::ASTMutationListener"},
    {"_ZTVN5clang6CXXABIE", "vtable for clang::CXXABI"},
    {"_ZTVN5clang11ASTImporterE", "vtable for clang::ASTImporter"},
    {"_ZTVN5clang4AttrE", "vtable for clang::Attr"},
    {"_ZTVN5clang9AliasAttrE", "vtable for clang::AliasAttr"},
    {"_ZTVN5clang15AlignMac68kAttrE", "vtable for clang::AlignMac68kAttr"},
    {"_ZTVN5clang11AlignedAttrE", "vtable for clang::AlignedAttr"},
    {"_ZTVN5clang16AlwaysInlineAttrE", "vtable for clang::AlwaysInlineAttr"},
    {"_ZTVN5clang20AnalyzerNoReturnAttrE", "vtable for clang::AnalyzerNoReturnAttr"},
    {"_ZTVN5clang12AnnotateAttrE", "vtable for clang::AnnotateAttr"},
    {"_ZTVN5clang12AsmLabelAttrE", "vtable for clang::AsmLabelAttr"},
    {"_ZTVN5clang10BlocksAttrE", "vtable for clang::BlocksAttr"},
    {"_ZTVN5clang9CDeclAttrE", "vtable for clang::CDeclAttr"},
    {"_ZTVN5clang14CFConsumedAttrE", "vtable for clang::CFConsumedAttr"},
    {"_ZTVN5clang24CFReturnsNotRetainedAttrE", "vtable for clang::CFReturnsNotRetainedAttr"},
    {"_ZTVN5clang21CFReturnsRetainedAttrE", "vtable for clang::CFReturnsRetainedAttr"},
    {"_ZTVN5clang16CUDAConstantAttrE", "vtable for clang::CUDAConstantAttr"},
    {"_ZTVN5clang14CUDADeviceAttrE", "vtable for clang::CUDADeviceAttr"},
    {"_ZTVN5clang14CUDAGlobalAttrE", "vtable for clang::CUDAGlobalAttr"},
    {"_ZTVN5clang12CUDAHostAttrE", "vtable for clang::CUDAHostAttr"},
    {"_ZTVN5clang20CUDALaunchBoundsAttrE", "vtable for clang::CUDALaunchBoundsAttr"},
    {"_ZTVN5clang14CUDASharedAttrE", "vtable for clang::CUDASharedAttr"},
    {"_ZTVN5clang21CarriesDependencyAttrE", "vtable for clang::CarriesDependencyAttr"},
    {"_ZTVN5clang11CleanupAttrE", "vtable for clang::CleanupAttr"},
    {"_ZTVN5clang10CommonAttrE", "vtable for clang::CommonAttr"},
    {"_ZTVN5clang9ConstAttrE", "vtable for clang::ConstAttr"},
    {"_ZTVN5clang15ConstructorAttrE", "vtable for clang::ConstructorAttr"},
    {"_ZTVN5clang13DLLExportAttrE", "vtable for clang::DLLExportAttr"},
    {"_ZTVN5clang13DLLImportAttrE", "vtable for clang::DLLImportAttr"},
    {"_ZTVN5clang14DeprecatedAttrE", "vtable for clang::DeprecatedAttr"},
    {"_ZTVN5clang14DestructorAttrE", "vtable for clang::DestructorAttr"},
    {"_ZTVN5clang12ExplicitAttrE", "vtable for clang::ExplicitAttr"},
    {"_ZTVN5clang12FastCallAttrE", "vtable for clang::FastCallAttr"},
    {"_ZTVN5clang9FinalAttrE", "vtable for clang::FinalAttr"},
    {"_ZTVN5clang10FormatAttrE", "vtable for clang::FormatAttr"},
    {"_ZTVN5clang13FormatArgAttrE", "vtable for clang::FormatArgAttr"},
    {"_ZTVN5clang13GNUInlineAttrE", "vtable for clang::GNUInlineAttr"},
    {"_ZTVN5clang12IBActionAttrE", "vtable for clang::IBActionAttr"},
    {"_ZTVN5clang12IBOutletAttrE", "vtable for clang::IBOutletAttr"},
    {"_ZTVN5clang22IBOutletCollectionAttrE", "vtable for clang::IBOutletCollectionAttr"},
    {"_ZTVN5clang16InitPriorityAttrE", "vtable for clang::InitPriorityAttr"},
    {"_ZTVN5clang26MBlazeInterruptHandlerAttrE", "vtable for clang::MBlazeInterruptHandlerAttr"},
    {"_ZTVN5clang23MBlazeSaveVolatilesAttrE", "vtable for clang::MBlazeSaveVolatilesAttr"},
    {"_ZTVN5clang19MSP430InterruptAttrE", "vtable for clang::MSP430InterruptAttr"},
    {"_ZTVN5clang10MallocAttrE", "vtable for clang::MallocAttr"},
    {"_ZTVN5clang21MaxFieldAlignmentAttrE", "vtable for clang::MaxFieldAlignmentAttr"},
    {"_ZTVN5clang12MayAliasAttrE", "vtable for clang::MayAliasAttr"},
    {"_ZTVN5clang14NSConsumedAttrE", "vtable for clang::NSConsumedAttr"},
    {"_ZTVN5clang18NSConsumesSelfAttrE", "vtable for clang::NSConsumesSelfAttr"},
    {"_ZTVN5clang25NSReturnsAutoreleasedAttrE", "vtable for clang::NSReturnsAutoreleasedAttr"},
    {"_ZTVN5clang24NSReturnsNotRetainedAttrE", "vtable for clang::NSReturnsNotRetainedAttr"},
    {"_ZTVN5clang21NSReturnsRetainedAttrE", "vtable for clang::NSReturnsRetainedAttr"},
    {"_ZTVN5clang9NakedAttrE", "vtable for clang::NakedAttr"},
    {"_ZTVN5clang12NoCommonAttrE", "vtable for clang::NoCommonAttr"},
    {"_ZTVN5clang11NoDebugAttrE", "vtable for clang::NoDebugAttr"},
    {"_ZTVN5clang12NoInlineAttrE", "vtable for clang::NoInlineAttr"},
    {"_ZTVN5clang24NoInstrumentFunctionAttrE", "vtable for clang::NoInstrumentFunctionAttr"},
    {"_ZTVN5clang12NoReturnAttrE", "vtable for clang::NoReturnAttr"},
    {"_ZTVN5clang11NoThrowAttrE", "vtable for clang::NoThrowAttr"},
    {"_ZTVN5clang11NonNullAttrE", "vtable for clang::NonNullAttr"},
    {"_ZTVN5clang17ObjCExceptionAttrE", "vtable for clang::ObjCExceptionAttr"},
    {"_ZTVN5clang16ObjCNSObjectAttrE", "vtable for clang::ObjCNSObjectAttr"},
    {"_ZTVN5clang16OverloadableAttrE", "vtable for clang::OverloadableAttr"},
    {"_ZTVN5clang12OverrideAttrE", "vtable for clang::OverrideAttr"},
    {"_ZTVN5clang13OwnershipAttrE", "vtable for clang::OwnershipAttr"},
    {"_ZTVN5clang10PackedAttrE", "vtable for clang::PackedAttr"},
    {"_ZTVN5clang10PascalAttrE", "vtable for clang::PascalAttr"},
    {"_ZTVN5clang8PureAttrE", "vtable for clang::PureAttr"},
    {"_ZTVN5clang11RegparmAttrE", "vtable for clang::RegparmAttr"},
    {"_ZTVN5clang21ReqdWorkGroupSizeAttrE", "vtable for clang::ReqdWorkGroupSizeAttr"},
    {"_ZTVN5clang11SectionAttrE", "vtable for clang::SectionAttr"},
    {"_ZTVN5clang12SentinelAttrE", "vtable for clang::SentinelAttr"},
    {"_ZTVN5clang11StdCallAttrE", "vtable for clang::StdCallAttr"},
    {"_ZTVN5clang12ThisCallAttrE", "vtable for clang::ThisCallAttr"},
    {"_ZTVN5clang20TransparentUnionAttrE", "vtable for clang::TransparentUnionAttr"},
    {"_ZTVN5clang15UnavailableAttrE", "vtable for clang::UnavailableAttr"},
    {"_ZTVN5clang10UnusedAttrE", "vtable for clang::UnusedAttr"},
    {"_ZTVN5clang8UsedAttrE", "vtable for clang::UsedAttr"},
    {"_ZTVN5clang8UuidAttrE", "vtable for clang::UuidAttr"},
    {"_ZTVN5clang13VecReturnAttrE", "vtable for clang::VecReturnAttr"},
    {"_ZTVN5clang14VisibilityAttrE", "vtable for clang::VisibilityAttr"},
    {"_ZTVN5clang20WarnUnusedResultAttrE", "vtable for clang::WarnUnusedResultAttr"},
    {"_ZTVN5clang8WeakAttrE", "vtable for clang::WeakAttr"},
    {"_ZTVN5clang14WeakImportAttrE", "vtable for clang::WeakImportAttr"},
    {"_ZTVN5clang11WeakRefAttrE", "vtable for clang::WeakRefAttr"},
    {"_ZTVN5clang27X86ForceAlignArgPointerAttrE", "vtable for clang::X86ForceAlignArgPointerAttr"},
    {"_ZTVN5clang10RecordDeclE", "vtable for clang::RecordDecl"},
    {"_ZTVN5clang7VarDeclE", "vtable for clang::VarDecl"},
    {"_ZTVN5clang12FunctionDeclE", "vtable for clang::FunctionDecl"},
    {"_ZTVN5clang16EnumConstantDeclE", "vtable for clang::EnumConstantDecl"},
    {"_ZTVN5clang7TagDeclE", "vtable for clang::TagDecl"},
    {"_ZTVN5clang9BlockDeclE", "vtable for clang::BlockDecl"},
    {"_ZTVN5clang4DeclE", "vtable for clang::Decl"},
    {"_ZTVN5clang20PrettyStackTraceDeclE", "vtable for clang::PrettyStackTraceDecl"},
    {"_ZTVN5clang13CXXRecordDeclE", "vtable for clang::CXXRecordDecl"},
    {"_ZTVN5clang13ObjCClassDeclE", "vtable for clang::ObjCClassDecl"},
    {"_ZTVN5clang14ObjCMethodDeclE", "vtable for clang::ObjCMethodDecl"},
    {"_ZTVN5clang20ObjCPropertyImplDeclE", "vtable for clang::ObjCPropertyImplDecl"},
    {"_ZTVN5clang23NonTypeTemplateParmDeclE", "vtable for clang::NonTypeTemplateParmDecl"},
    {"_ZTVN5clang31ClassTemplateSpecializationDeclE", "vtable for clang::ClassTemplateSpecializationDecl"},
    {"_ZTVN5clang24RedeclarableTemplateDeclE", "vtable for clang::RedeclarableTemplateDecl"},
    {"_ZTVN5clang20FunctionTemplateDeclE", "vtable for clang::FunctionTemplateDecl"},
    {"_ZTVN5clang17ClassTemplateDeclE", "vtable for clang::ClassTemplateDecl"},
    {"_ZZNK5clang15DeclarationName9printNameERN4llvm11raw_ostreamEE13OperatorNames", "clang::DeclarationName::printName(llvm::raw_ostream&) const::OperatorNames"},
    {"_ZTVN12_GLOBAL__N_19ARMCXXABIE", "vtable for (anonymous namespace)::ARMCXXABI"},
    {"_ZTVN12_GLOBAL__N_113ItaniumCXXABIE", "vtable for (anonymous namespace)::ItaniumCXXABI"},
    {"_ZTVN12_GLOBAL__N_120ItaniumMangleContextE", "vtable for (anonymous namespace)::ItaniumMangleContext"},
    {"_ZTVN12_GLOBAL__N_115MicrosoftCXXABIE", "vtable for (anonymous namespace)::MicrosoftCXXABI"},
    {"_ZTVN12_GLOBAL__N_122MicrosoftMangleContextE", "vtable for (anonymous namespace)::MicrosoftMangleContext"},
    {"_ZTVN12_GLOBAL__N_119RecordLayoutBuilderE", "vtable for (anonymous namespace)::RecordLayoutBuilder"},
    {"_ZTVN12_GLOBAL__N_121MSRecordLayoutBuilderE", "vtable for (anonymous namespace)::MSRecordLayoutBuilder"},
    {"_ZTVN5clang13PrinterHelperE", "vtable for clang::PrinterHelper"},
    {"_ZZN12_GLOBAL__N_111StmtPrinter24VisitCXXOperatorCallExprEPN5clang19CXXOperatorCallExprEE9OpStrings", "(anonymous namespace)::StmtPrinter::VisitCXXOperatorCallExpr(clang::CXXOperatorCallExpr*)::OpStrings"},
    {"_ZTVN5clang11PPCallbacksE", "vtable for clang::PPCallbacks"},
    {"_ZZ16ComputeDATE_TIMERN5clang14SourceLocationES1_RNS_12PreprocessorEE6Months", "ComputeDATE_TIME(clang::SourceLocation&, clang::SourceLocation&, clang::Preprocessor&)::Months"},
    {"_ZTVN5clang8PTHLexerE", "vtable for clang::PTHLexer"},
    {"_ZTVN5clang10PTHManagerE", "vtable for clang::PTHManager"},
    {"_ZTVN12_GLOBAL__N_112PTHStatCacheE", "vtable for (anonymous namespace)::PTHStatCache"},
    {"_ZTVN5clang13PragmaHandlerE", "vtable for clang::PragmaHandler"},
    {"_ZTVN5clang18EmptyPragmaHandlerE", "vtable for clang::EmptyPragmaHandler"},
    {"_ZTVN5clang15PragmaNamespaceE", "vtable for clang::PragmaNamespace"},
    {"_ZTVN12_GLOBAL__N_120PragmaCommentHandlerE", "vtable for (anonymous namespace)::PragmaCommentHandler"},
    {"_ZTVN12_GLOBAL__N_125PragmaSTDC_UnknownHandlerE", "vtable for (anonymous namespace)::PragmaSTDC_UnknownHandler"},
    {"_ZTVN12_GLOBAL__N_134PragmaSTDC_CX_LIMITED_RANGEHandlerE", "vtable for (anonymous namespace)::PragmaSTDC_CX_LIMITED_RANGEHandler"},
    {"_ZTVN12_GLOBAL__N_129PragmaSTDC_FENV_ACCESSHandlerE", "vtable for (anonymous namespace)::PragmaSTDC_FENV_ACCESSHandler"},
    {"_ZTVN12_GLOBAL__N_129PragmaSTDC_FP_CONTRACTHandlerE", "vtable for (anonymous namespace)::PragmaSTDC_FP_CONTRACTHandler"},
    {"_ZTVN12_GLOBAL__N_118PragmaDebugHandlerE", "vtable for (anonymous namespace)::PragmaDebugHandler"},
    {"_ZTVN12_GLOBAL__N_123PragmaDiagnosticHandlerE", "vtable for (anonymous namespace)::PragmaDiagnosticHandler"},
    {"_ZTVN12_GLOBAL__N_123PragmaDependencyHandlerE", "vtable for (anonymous namespace)::PragmaDependencyHandler"},
    {"_ZTVN12_GLOBAL__N_125PragmaSystemHeaderHandlerE", "vtable for (anonymous namespace)::PragmaSystemHeaderHandler"},
    {"_ZTVN12_GLOBAL__N_119PragmaPoisonHandlerE", "vtable for (anonymous namespace)::PragmaPoisonHandler"},
    {"_ZTVN12_GLOBAL__N_120PragmaMessageHandlerE", "vtable for (anonymous namespace)::PragmaMessageHandler"},
    {"_ZTVN12_GLOBAL__N_121PragmaPopMacroHandlerE", "vtable for (anonymous namespace)::PragmaPopMacroHandler"},
    {"_ZTVN12_GLOBAL__N_122PragmaPushMacroHandlerE", "vtable for (anonymous namespace)::PragmaPushMacroHandler"},
    {"_ZTVN12_GLOBAL__N_117PragmaMarkHandlerE", "vtable for (anonymous namespace)::PragmaMarkHandler"},
    {"_ZTVN12_GLOBAL__N_117PragmaOnceHandlerE", "vtable for (anonymous namespace)::PragmaOnceHandler"},
    {"_ZTVN5clang33ExternalPreprocessingRecordSourceE", "vtable for clang::ExternalPreprocessingRecordSource"},
    {"_ZTVN5clang19PreprocessingRecordE", "vtable for clang::PreprocessingRecord"},
    {"_ZTVN5clang26ExternalPreprocessorSourceE", "vtable for clang::ExternalPreprocessorSource"},
    {"_ZTVN5clang14CommentHandlerE", "vtable for clang::CommentHandler"},
    {"_ZTVN5clang21CodeCompletionHandlerE", "vtable for clang::CodeCompletionHandler"},
    {"_ZTVN5clang16DiagnosticClientE", "vtable for clang::DiagnosticClient"},
    {"_ZZN5clang13DiagnosticIDs21getCategoryNameFromIDEjE17CategoryNameTable", "clang::DiagnosticIDs::getCategoryNameFromID(unsigned int)::CategoryNameTable"},
    {"_ZTVN5clang17MemorizeStatCallsE", "vtable for clang::MemorizeStatCalls"},
    {"_ZTVN5clang18IdentifierIteratorE", "vtable for clang::IdentifierIterator"},
    {"_ZTVN5clang20IdentifierInfoLookupE", "vtable for clang::IdentifierInfoLookup"},
    {"_ZTVN5clang24ExternalIdentifierLookupE", "vtable for clang::ExternalIdentifierLookup"},
    {"_ZTVN12_GLOBAL__N_119EmptyLookupIteratorE", "vtable for (anonymous namespace)::EmptyLookupIterator"},
    {"_ZTVN5clang19PrettyStackTraceLocE", "vtable for clang::PrettyStackTraceLoc"},
    {"_ZTVN5clang23ExternalSLocEntrySourceE", "vtable for clang::ExternalSLocEntrySource"},
    {"_ZTVN5clang10TargetInfoE", "vtable for clang::TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116X86_64TargetInfoE", "vtable for (anonymous namespace)::X86_64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_113X86TargetInfoE", "vtable for (anonymous namespace)::X86TargetInfo"},
    {"_ZTVN12_GLOBAL__N_135VisualStudioWindowsX86_64TargetInfoE", "vtable for (anonymous namespace)::VisualStudioWindowsX86_64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_121MinGWX86_64TargetInfoE", "vtable for (anonymous namespace)::MinGWX86_64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_123OpenBSDX86_64TargetInfoE", "vtable for (anonymous namespace)::OpenBSDX86_64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_122DarwinX86_64TargetInfoE", "vtable for (anonymous namespace)::DarwinX86_64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_64TargetInfoEEE", "vtable for (anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_116X86_32TargetInfoE", "vtable for (anonymous namespace)::X86_32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_121HaikuX86_32TargetInfoE", "vtable for (anonymous namespace)::HaikuX86_32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_135VisualStudioWindowsX86_32TargetInfoE", "vtable for (anonymous namespace)::VisualStudioWindowsX86_32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_121MinGWX86_32TargetInfoE", "vtable for (anonymous namespace)::MinGWX86_32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_122CygwinX86_32TargetInfoE", "vtable for (anonymous namespace)::CygwinX86_32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_117SolarisTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::SolarisTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115MinixTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::MinixTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_117FreeBSDTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_121OpenBSDI386TargetInfoE", "vtable for (anonymous namespace)::OpenBSDI386TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116NetBSDTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::NetBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_122DragonFlyBSDTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::DragonFlyBSDTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115LinuxTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_120DarwinI386TargetInfoE", "vtable for (anonymous namespace)::DarwinI386TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116DarwinTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_118AuroraUXTargetInfoINS_16X86_32TargetInfoEEE", "vtable for (anonymous namespace)::AuroraUXTargetInfo<(anonymous namespace)::X86_32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_113TCETargetInfoE", "vtable for (anonymous namespace)::TCETargetInfo"},
    {"_ZTVN12_GLOBAL__N_117SystemZTargetInfoE", "vtable for (anonymous namespace)::SystemZTargetInfo"},
    {"_ZN12_GLOBAL__N_117SystemZTargetInfo11GCCRegNamesE", "(anonymous namespace)::SystemZTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_116PS3SPUTargetInfoINS_15PPC64TargetInfoEEE", "vtable for (anonymous namespace)::PS3SPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>"},
    {"_ZN12_GLOBAL__N_113PPCTargetInfo11GCCRegNamesE", "(anonymous namespace)::PPCTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_115PPC64TargetInfoE", "vtable for (anonymous namespace)::PPC64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_113PPCTargetInfoE", "vtable for (anonymous namespace)::PPCTargetInfo"},
    {"_ZTVN12_GLOBAL__N_117SparcV8TargetInfoE", "vtable for (anonymous namespace)::SparcV8TargetInfo"},
    {"_ZN12_GLOBAL__N_117SparcV8TargetInfo11GCCRegNamesE", "(anonymous namespace)::SparcV8TargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_124SolarisSparcV8TargetInfoE", "vtable for (anonymous namespace)::SolarisSparcV8TargetInfo"},
    {"_ZTVN12_GLOBAL__N_125AuroraUXSparcV8TargetInfoE", "vtable for (anonymous namespace)::AuroraUXSparcV8TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116MBlazeTargetInfoE", "vtable for (anonymous namespace)::MBlazeTargetInfo"},
    {"_ZN12_GLOBAL__N_116MBlazeTargetInfo11GCCRegNamesE", "(anonymous namespace)::MBlazeTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC64TargetInfoEEE", "vtable for (anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_116PS3PPUTargetInfoINS_15PPC64TargetInfoEEE", "vtable for (anonymous namespace)::PS3PPUTargetInfo<(anonymous namespace)::PPC64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_121DarwinPPC64TargetInfoE", "vtable for (anonymous namespace)::DarwinPPC64TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC64TargetInfoEEE", "vtable for (anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC64TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115PPC32TargetInfoE", "vtable for (anonymous namespace)::PPC32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_117FreeBSDTargetInfoINS_15PPC32TargetInfoEEE", "vtable for (anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::PPC32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_121DarwinPPC32TargetInfoE", "vtable for (anonymous namespace)::DarwinPPC32TargetInfo"},
    {"_ZTVN12_GLOBAL__N_116DarwinTargetInfoINS_15PPC32TargetInfoEEE", "vtable for (anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::PPC32TargetInfo>"},
    {"_ZTVN12_GLOBAL__N_116MipselTargetInfoE", "vtable for (anonymous namespace)::MipselTargetInfo"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfo11GCCRegNamesE", "(anonymous namespace)::MipsTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_114MipsTargetInfoE", "vtable for (anonymous namespace)::MipsTargetInfo"},
    {"_ZTVN12_GLOBAL__N_115LinuxTargetInfoINS_16MipselTargetInfoEEE", "vtable for (anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipselTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_113PSPTargetInfoINS_16MipselTargetInfoEEE", "vtable for (anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipselTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115LinuxTargetInfoINS_14MipsTargetInfoEEE", "vtable for (anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::MipsTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_113PSPTargetInfoINS_14MipsTargetInfoEEE", "vtable for (anonymous namespace)::PSPTargetInfo<(anonymous namespace)::MipsTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_116MSP430TargetInfoE", "vtable for (anonymous namespace)::MSP430TargetInfo"},
    {"_ZN12_GLOBAL__N_116MSP430TargetInfo11GCCRegNamesE", "(anonymous namespace)::MSP430TargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_118BlackfinTargetInfoE", "vtable for (anonymous namespace)::BlackfinTargetInfo"},
    {"_ZN12_GLOBAL__N_118BlackfinTargetInfo11GCCRegNamesE", "(anonymous namespace)::BlackfinTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_113ARMTargetInfoE", "vtable for (anonymous namespace)::ARMTargetInfo"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo11GCCRegNamesE", "(anonymous namespace)::ARMTargetInfo::GCCRegNames"},
    {"_ZTVN12_GLOBAL__N_117FreeBSDTargetInfoINS_13ARMTargetInfoEEE", "vtable for (anonymous namespace)::FreeBSDTargetInfo<(anonymous namespace)::ARMTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_119DarwinARMTargetInfoE", "vtable for (anonymous namespace)::DarwinARMTargetInfo"},
    {"_ZTVN12_GLOBAL__N_116DarwinTargetInfoINS_13ARMTargetInfoEEE", "vtable for (anonymous namespace)::DarwinTargetInfo<(anonymous namespace)::ARMTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_115LinuxTargetInfoINS_13ARMTargetInfoEEE", "vtable for (anonymous namespace)::LinuxTargetInfo<(anonymous namespace)::ARMTargetInfo>"},
    {"_ZTVN12_GLOBAL__N_112ArgPromotionE", "vtable for (anonymous namespace)::ArgPromotion"},
    {"_ZTVN12_GLOBAL__N_113ConstantMergeE", "vtable for (anonymous namespace)::ConstantMerge"},
    {"_ZTVN12_GLOBAL__N_13DAHE", "vtable for (anonymous namespace)::DAH"},
    {"_ZTVN12_GLOBAL__N_13DAEE", "vtable for (anonymous namespace)::DAE"},
    {"_ZTVN12_GLOBAL__N_13DTEE", "vtable for (anonymous namespace)::DTE"},
    {"_ZTVN12_GLOBAL__N_113FunctionAttrsE", "vtable for (anonymous namespace)::FunctionAttrs"},
    {"_ZTVN12_GLOBAL__N_19GlobalDCEE", "vtable for (anonymous namespace)::GlobalDCE"},
    {"_ZTVN12_GLOBAL__N_19GlobalOptE", "vtable for (anonymous namespace)::GlobalOpt"},
    {"_ZTVN12_GLOBAL__N_113AlwaysInlinerE", "vtable for (anonymous namespace)::AlwaysInliner"},
    {"_ZTVN12_GLOBAL__N_113SimpleInlinerE", "vtable for (anonymous namespace)::SimpleInliner"},
    {"_ZTVN4llvm7InlinerE", "vtable for llvm::Inliner"},
    {"_ZTVN12_GLOBAL__N_17PruneEHE", "vtable for (anonymous namespace)::PruneEH"},
    {"_ZTVN12_GLOBAL__N_123StripDeadPrototypesPassE", "vtable for (anonymous namespace)::StripDeadPrototypesPass"},
    {"_ZTVN12_GLOBAL__N_116WriteBitcodePassE", "vtable for (anonymous namespace)::WriteBitcodePass"},
    {"_ZTVN4llvm13BitcodeReaderE", "vtable for llvm::BitcodeReader"},
    {"_ZTVN4llvm12_GLOBAL__N_119ConstantPlaceHolderE", "vtable for llvm::(anonymous namespace)::ConstantPlaceHolder"},
    {"_ZTVN12_GLOBAL__N_111X86AsmLexerE", "vtable for (anonymous namespace)::X86AsmLexer"},
    {"_ZTVN12_GLOBAL__N_118X86_64ATTAsmParserE", "vtable for (anonymous namespace)::X86_64ATTAsmParser"},
    {"_ZTVN12_GLOBAL__N_110X86OperandE", "vtable for (anonymous namespace)::X86Operand"},
    {"_ZTVN12_GLOBAL__N_115X86ATTAsmParserE", "vtable for (anonymous namespace)::X86ATTAsmParser"},
    {"_ZTVN12_GLOBAL__N_118X86_32ATTAsmParserE", "vtable for (anonymous namespace)::X86_32ATTAsmParser"},
    {"_ZTVN12_GLOBAL__N_116SSEDomainFixPassE", "vtable for (anonymous namespace)::SSEDomainFixPass"},
    {"_ZTVN12_GLOBAL__N_119ELFX86_64AsmBackendE", "vtable for (anonymous namespace)::ELFX86_64AsmBackend"},
    {"_ZZNK12_GLOBAL__N_113X86AsmBackend16getFixupKindInfoEN4llvm11MCFixupKindEE5Infos", "(anonymous namespace)::X86AsmBackend::getFixupKindInfo(llvm::MCFixupKind) const::Infos"},
    {"_ZTVN12_GLOBAL__N_118X86ELFObjectWriterE", "vtable for (anonymous namespace)::X86ELFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_113X86AsmBackendE", "vtable for (anonymous namespace)::X86AsmBackend"},
    {"_ZTVN12_GLOBAL__N_122DarwinX86_64AsmBackendE", "vtable for (anonymous namespace)::DarwinX86_64AsmBackend"},
    {"_ZTVN12_GLOBAL__N_119ELFX86_32AsmBackendE", "vtable for (anonymous namespace)::ELFX86_32AsmBackend"},
    {"_ZTVN12_GLOBAL__N_120WindowsX86AsmBackendE", "vtable for (anonymous namespace)::WindowsX86AsmBackend"},
    {"_ZTVN12_GLOBAL__N_122DarwinX86_32AsmBackendE", "vtable for (anonymous namespace)::DarwinX86_32AsmBackend"},
    {"_ZTVN4llvm13X86AsmPrinterE", "vtable for llvm::X86AsmPrinter"},
    {"_ZTVN4llvm24X86COFFMachineModuleInfoE", "vtable for llvm::X86COFFMachineModuleInfo"},
    {"_ZTVN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEEE", "vtable for (anonymous namespace)::Emitter<llvm::JITCodeEmitter>"},
    {"_ZTVN4llvm16X86ELFWriterInfoE", "vtable for llvm::X86ELFWriterInfo"},
    {"_ZTVN12_GLOBAL__N_111X86FastISelE", "vtable for (anonymous namespace)::X86FastISel"},
    {"_ZTVN12_GLOBAL__N_13FPSE", "vtable for (anonymous namespace)::FPS"},
    {"_ZTVN4llvm16X86FrameLoweringE", "vtable for llvm::X86FrameLowering"},
    {"_ZTVN12_GLOBAL__N_115X86DAGToDAGISelE", "vtable for (anonymous namespace)::X86DAGToDAGISel"},
    {"_ZTVN4llvm17X86TargetLoweringE", "vtable for llvm::X86TargetLowering"},
    {"_ZTVN4llvm12X86InstrInfoE", "vtable for llvm::X86InstrInfo"},
    {"_ZTVN12_GLOBAL__N_14CGBRE", "vtable for (anonymous namespace)::CGBR"},
    {"_ZTVN4llvm10X86JITInfoE", "vtable for llvm::X86JITInfo"},
    {"_ZTVN4llvm15X86ELFMCAsmInfoE", "vtable for llvm::X86ELFMCAsmInfo"},
    {"_ZTVN12_GLOBAL__N_116X86MCCodeEmitterE", "vtable for (anonymous namespace)::X86MCCodeEmitter"},
    {"_ZTVN12_GLOBAL__N_119X86MachObjectWriterE", "vtable for (anonymous namespace)::X86MachObjectWriter"},
    {"_ZTVN4llvm15X86RegisterInfoE", "vtable for llvm::X86RegisterInfo"},
    {"_ZTVN4llvm3X868CCRClassE", "vtable for llvm::X86::CCRClass"},
    {"_ZTVN4llvm3X869FR32ClassE", "vtable for llvm::X86::FR32Class"},
    {"_ZTVN4llvm3X869FR64ClassE", "vtable for llvm::X86::FR64Class"},
    {"_ZTVN4llvm3X869GR16ClassE", "vtable for llvm::X86::GR16Class"},
    {"_ZTVN4llvm3X8615GR16_NOREXClassE", "vtable for llvm::X86::GR16_NOREXClass"},
    {"_ZTVN4llvm3X869GR32ClassE", "vtable for llvm::X86::GR32Class"},
    {"_ZTVN4llvm3X8615GR32_NOREXClassE", "vtable for llvm::X86::GR32_NOREXClass"},
    {"_ZTVN4llvm3X8614GR32_NOSPClassE", "vtable for llvm::X86::GR32_NOSPClass"},
    {"_ZTVN4llvm3X869GR64ClassE", "vtable for llvm::X86::GR64Class"},
    {"_ZTVN4llvm3X8615GR64_NOREXClassE", "vtable for llvm::X86::GR64_NOREXClass"},
    {"_ZTVN4llvm3X8620GR64_NOREX_NOSPClassE", "vtable for llvm::X86::GR64_NOREX_NOSPClass"},
    {"_ZTVN4llvm3X8614GR64_NOSPClassE", "vtable for llvm::X86::GR64_NOSPClass"},
    {"_ZTVN4llvm3X868GR8ClassE", "vtable for llvm::X86::GR8Class"},
    {"_ZTVN4llvm3X8614GR8_NOREXClassE", "vtable for llvm::X86::GR8_NOREXClass"},
    {"_ZTVN4llvm3X868RSTClassE", "vtable for llvm::X86::RSTClass"},
    {"_ZTVN4llvm3X8610VR128ClassE", "vtable for llvm::X86::VR128Class"},
    {"_ZTVN4llvm3X8610VR256ClassE", "vtable for llvm::X86::VR256Class"},
    {"_ZTVN4llvm18X86GenRegisterInfoE", "vtable for llvm::X86GenRegisterInfo"},
    {"_ZTVN12_GLOBAL__N_14MSAHE", "vtable for (anonymous namespace)::MSAH"},
    {"_ZTVN4llvm19X86SelectionDAGInfoE", "vtable for llvm::X86SelectionDAGInfo"},
    {"_ZTVN4llvm12X86SubtargetE", "vtable for llvm::X86Subtarget"},
    {"_ZTVN4llvm16X86TargetMachineE", "vtable for llvm::X86TargetMachine"},
    {"_ZTVN4llvm25X8632_ELFTargetObjectFileE", "vtable for llvm::X8632_ELFTargetObjectFile"},
    {"_ZTVN4llvm25X8664_ELFTargetObjectFileE", "vtable for llvm::X8664_ELFTargetObjectFile"},
    {"_ZTVN4llvm27X8664_MachoTargetObjectFileE", "vtable for llvm::X8664_MachoTargetObjectFile"},
    {"_ZTVN4llvm17X86ATTInstPrinterE", "vtable for llvm::X86ATTInstPrinter"},
    {"_ZTVN4llvm19X86IntelInstPrinterE", "vtable for llvm::X86IntelInstPrinter"},
    {"_ZTVN12_GLOBAL__N_113ThumbAsmLexerE", "vtable for (anonymous namespace)::ThumbAsmLexer"},
    {"_ZTVN12_GLOBAL__N_115ARMBaseAsmLexerE", "vtable for (anonymous namespace)::ARMBaseAsmLexer"},
    {"_ZTVN12_GLOBAL__N_111ARMAsmLexerE", "vtable for (anonymous namespace)::ARMAsmLexer"},
    {"_ZTVN12_GLOBAL__N_112ARMAsmParserE", "vtable for (anonymous namespace)::ARMAsmParser"},
    {"_ZTVN12_GLOBAL__N_110ARMOperandE", "vtable for (anonymous namespace)::ARMOperand"},
    {"_ZTVN12_GLOBAL__N_116ELFARMAsmBackendE", "vtable for (anonymous namespace)::ELFARMAsmBackend"},
    {"_ZZNK12_GLOBAL__N_113ARMAsmBackend16getFixupKindInfoEN4llvm11MCFixupKindEE5Infos", "(anonymous namespace)::ARMAsmBackend::getFixupKindInfo(llvm::MCFixupKind) const::Infos"},
    {"_ZTVN12_GLOBAL__N_118ARMELFObjectWriterE", "vtable for (anonymous namespace)::ARMELFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_113ARMAsmBackendE", "vtable for (anonymous namespace)::ARMAsmBackend"},
    {"_ZTVN12_GLOBAL__N_119DarwinARMAsmBackendE", "vtable for (anonymous namespace)::DarwinARMAsmBackend"},
    {"_ZTVN12_GLOBAL__N_119ARMMachObjectWriterE", "vtable for (anonymous namespace)::ARMMachObjectWriter"},
    {"_ZTVN4llvm13ARMAsmPrinterE", "vtable for llvm::ARMAsmPrinter"},
    {"_ZTVN12_GLOBAL__N_122ObjectAttributeEmitterE", "vtable for (anonymous namespace)::ObjectAttributeEmitter"},
    {"_ZTVN12_GLOBAL__N_116AttributeEmitterE", "vtable for (anonymous namespace)::AttributeEmitter"},
    {"_ZTVN12_GLOBAL__N_119AsmAttributeEmitterE", "vtable for (anonymous namespace)::AsmAttributeEmitter"},
    {"_ZTVN4llvm16ARMBaseInstrInfoE", "vtable for llvm::ARMBaseInstrInfo"},
    {"_ZTVN4llvm19ARMBaseRegisterInfoE", "vtable for llvm::ARMBaseRegisterInfo"},
    {"_ZTVN4llvm3ARM8DPRClassE", "vtable for llvm::ARM::DPRClass"},
    {"_ZTVN4llvm3ARM8GPRClassE", "vtable for llvm::ARM::GPRClass"},
    {"_ZTVN4llvm3ARM8QPRClassE", "vtable for llvm::ARM::QPRClass"},
    {"_ZTVN4llvm3ARM9QQPRClassE", "vtable for llvm::ARM::QQPRClass"},
    {"_ZTVN4llvm3ARM11QQQQPRClassE", "vtable for llvm::ARM::QQQQPRClass"},
    {"_ZTVN4llvm3ARM9rGPRClassE", "vtable for llvm::ARM::rGPRClass"},
    {"_ZTVN4llvm3ARM10tcGPRClassE", "vtable for llvm::ARM::tcGPRClass"},
    {"_ZTVN4llvm18ARMGenRegisterInfoE", "vtable for llvm::ARMGenRegisterInfo"},
    {"_ZTVN12_GLOBAL__N_114ARMCodeEmitterE", "vtable for (anonymous namespace)::ARMCodeEmitter"},
    {"_ZTVN12_GLOBAL__N_118ARMConstantIslandsE", "vtable for (anonymous namespace)::ARMConstantIslands"},
    {"_ZTVN4llvm20ARMConstantPoolValueE", "vtable for llvm::ARMConstantPoolValue"},
    {"_ZTVN4llvm16ARMELFWriterInfoE", "vtable for llvm::ARMELFWriterInfo"},
    {"_ZTVN12_GLOBAL__N_115ARMExpandPseudoE", "vtable for (anonymous namespace)::ARMExpandPseudo"},
    {"_ZTVN12_GLOBAL__N_111ARMFastISelE", "vtable for (anonymous namespace)::ARMFastISel"},
    {"_ZTVN4llvm16ARMFrameLoweringE", "vtable for llvm::ARMFrameLowering"},
    {"_ZTVN12_GLOBAL__N_114ARMGlobalMergeE", "vtable for (anonymous namespace)::ARMGlobalMerge"},
    {"_ZTVN4llvm19ARMHazardRecognizerE", "vtable for llvm::ARMHazardRecognizer"},
    {"_ZTVN12_GLOBAL__N_115ARMDAGToDAGISelE", "vtable for (anonymous namespace)::ARMDAGToDAGISel"},
    {"_ZTVN4llvm17ARMTargetLoweringE", "vtable for llvm::ARMTargetLowering"},
    {"_ZTVN4llvm12ARMInstrInfoE", "vtable for llvm::ARMInstrInfo"},
    {"_ZTVN4llvm10ARMJITInfoE", "vtable for llvm::ARMJITInfo"},
    {"_ZTVN12_GLOBAL__N_115ARMLoadStoreOptE", "vtable for (anonymous namespace)::ARMLoadStoreOpt"},
    {"_ZTVN12_GLOBAL__N_123ARMPreAllocLoadStoreOptE", "vtable for (anonymous namespace)::ARMPreAllocLoadStoreOpt"},
    {"_ZTVN12_GLOBAL__N_116ARMMCCodeEmitterE", "vtable for (anonymous namespace)::ARMMCCodeEmitter"},
    {"_ZTVN4llvm9ARMMCExprE", "vtable for llvm::ARMMCExpr"},
    {"_ZTVN4llvm19ARMSelectionDAGInfoE", "vtable for llvm::ARMSelectionDAGInfo"},
    {"_ZTVN4llvm12ARMSubtargetE", "vtable for llvm::ARMSubtarget"},
    {"_ZTVN4llvm20ARMBaseTargetMachineE", "vtable for llvm::ARMBaseTargetMachine"},
    {"_ZTVN4llvm22ARMElfTargetObjectFileE", "vtable for llvm::ARMElfTargetObjectFile"},
    {"_ZTVN12_GLOBAL__N_112MLxExpansionE", "vtable for (anonymous namespace)::MLxExpansion"},
    {"_ZTVN12_GLOBAL__N_115NEONMoveFixPassE", "vtable for (anonymous namespace)::NEONMoveFixPass"},
    {"_ZTVN4llvm19Thumb1FrameLoweringE", "vtable for llvm::Thumb1FrameLowering"},
    {"_ZTVN4llvm15Thumb1InstrInfoE", "vtable for llvm::Thumb1InstrInfo"},
    {"_ZTVN4llvm18Thumb1RegisterInfoE", "vtable for llvm::Thumb1RegisterInfo"},
    {"_ZTVN12_GLOBAL__N_117Thumb2ITBlockPassE", "vtable for (anonymous namespace)::Thumb2ITBlockPass"},
    {"_ZTVN4llvm15Thumb2InstrInfoE", "vtable for llvm::Thumb2InstrInfo"},
    {"_ZTVN4llvm18Thumb2RegisterInfoE", "vtable for llvm::Thumb2RegisterInfo"},
    {"_ZTVN12_GLOBAL__N_116Thumb2SizeReduceE", "vtable for (anonymous namespace)::Thumb2SizeReduce"},
    {"_ZTVN12_GLOBAL__N_115WorkListRemoverE", "vtable for (anonymous namespace)::WorkListRemover"},
    {"_ZTVN4llvm8FastISelE", "vtable for llvm::FastISel"},
    {"_ZTVN12_GLOBAL__N_118NodeUpdateListenerE", "vtable for (anonymous namespace)::NodeUpdateListener"},
    {"_ZTVN12_GLOBAL__N_115ScheduleDAGListE", "vtable for (anonymous namespace)::ScheduleDAGList"},
    {"_ZTVN12_GLOBAL__N_125RegReductionPriorityQueueINS_14ilp_ls_rr_sortEEE", "vtable for (anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::ilp_ls_rr_sort>"},
    {"_ZTVN12_GLOBAL__N_118RegReductionPQBaseE", "vtable for (anonymous namespace)::RegReductionPQBase"},
    {"_ZTVN12_GLOBAL__N_125RegReductionPriorityQueueINS_17hybrid_ls_rr_sortEEE", "vtable for (anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::hybrid_ls_rr_sort>"},
    {"_ZTVN12_GLOBAL__N_125RegReductionPriorityQueueINS_14src_ls_rr_sortEEE", "vtable for (anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::src_ls_rr_sort>"},
    {"_ZTVN12_GLOBAL__N_125RegReductionPriorityQueueINS_13td_ls_rr_sortEEE", "vtable for (anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::td_ls_rr_sort>"},
    {"_ZTVN12_GLOBAL__N_125RegReductionPriorityQueueINS_13bu_ls_rr_sortEEE", "vtable for (anonymous namespace)::RegReductionPriorityQueue<(anonymous namespace)::bu_ls_rr_sort>"},
    {"_ZTVN12_GLOBAL__N_117ScheduleDAGRRListE", "vtable for (anonymous namespace)::ScheduleDAGRRList"},
    {"_ZTVN4llvm18ScheduleDAGSDNodesE", "vtable for llvm::ScheduleDAGSDNodes"},
    {"_ZTVN4llvm12SelectionDAG17DAGUpdateListenerE", "vtable for llvm::SelectionDAG::DAGUpdateListener"},
    {"_ZTVN12_GLOBAL__N_118RAUWUpdateListenerE", "vtable for (anonymous namespace)::RAUWUpdateListener"},
    {"_ZTVN4llvm16SelectionDAGISelE", "vtable for llvm::SelectionDAGISel"},
    {"_ZZ19FunctionCallsSetJmpPKN4llvm8FunctionEE15ReturnsTwiceFns", "FunctionCallsSetJmp(llvm::Function const*)::ReturnsTwiceFns"},
    {"_ZTVN4llvm14TargetLoweringE", "vtable for llvm::TargetLowering"},
    {"_ZTVN4llvm22TargetSelectionDAGInfoE", "vtable for llvm::TargetSelectionDAGInfo"},
    {"_ZTVN4llvm10AsmPrinterE", "vtable for llvm::AsmPrinter"},
    {"_ZTVN4llvm3DIEE", "vtable for llvm::DIE"},
    {"_ZTVN4llvm10DIEIntegerE", "vtable for llvm::DIEInteger"},
    {"_ZTVN4llvm9DIEStringE", "vtable for llvm::DIEString"},
    {"_ZTVN4llvm8DIELabelE", "vtable for llvm::DIELabel"},
    {"_ZTVN4llvm8DIEDeltaE", "vtable for llvm::DIEDelta"},
    {"_ZTVN4llvm8DIEEntryE", "vtable for llvm::DIEEntry"},
    {"_ZTVN4llvm8DIEBlockE", "vtable for llvm::DIEBlock"},
    {"_ZTVN4llvm17DwarfCFIExceptionE", "vtable for llvm::DwarfCFIException"},
    {"_ZTVN4llvm8DbgScopeE", "vtable for llvm::DbgScope"},
    {"_ZTVN4llvm14DwarfExceptionE", "vtable for llvm::DwarfException"},
    {"_ZTVN4llvm19DwarfTableExceptionE", "vtable for llvm::DwarfTableException"},
    {"_ZTVN4llvm8AsmLexerE", "vtable for llvm::AsmLexer"},
    {"_ZTVN12_GLOBAL__N_19AsmParserE", "vtable for (anonymous namespace)::AsmParser"},
    {"_ZTVN12_GLOBAL__N_116GenericAsmParserE", "vtable for (anonymous namespace)::GenericAsmParser"},
    {"_ZTVN12_GLOBAL__N_113COFFAsmParserE", "vtable for (anonymous namespace)::COFFAsmParser"},
    {"_ZTVN12_GLOBAL__N_115DarwinAsmParserE", "vtable for (anonymous namespace)::DarwinAsmParser"},
    {"_ZTVN12_GLOBAL__N_112ELFAsmParserE", "vtable for (anonymous namespace)::ELFAsmParser"},
    {"_ZTVN4llvm10MCAsmLexerE", "vtable for llvm::MCAsmLexer"},
    {"_ZTVN4llvm11MCAsmParserE", "vtable for llvm::MCAsmParser"},
    {"_ZTVN4llvm20MCAsmParserExtensionE", "vtable for llvm::MCAsmParserExtension"},
    {"_ZTVN4llvm15TargetAsmParserE", "vtable for llvm::TargetAsmParser"},
    {"_ZTVN4llvm24AggressiveAntiDepBreakerE", "vtable for llvm::AggressiveAntiDepBreaker"},
    {"_ZTVN12_GLOBAL__N_116BranchFolderPassE", "vtable for (anonymous namespace)::BranchFolderPass"},
    {"_ZTVN4llvm21CalculateSpillWeightsE", "vtable for llvm::CalculateSpillWeights"},
    {"_ZTVN12_GLOBAL__N_116CodePlacementOptE", "vtable for (anonymous namespace)::CodePlacementOpt"},
    {"_ZTVN4llvm22CriticalAntiDepBreakerE", "vtable for llvm::CriticalAntiDepBreaker"},
    {"_ZTVN12_GLOBAL__N_126DeadMachineInstructionElimE", "vtable for (anonymous namespace)::DeadMachineInstructionElim"},
    {"_ZTVN12_GLOBAL__N_114DwarfEHPrepareE", "vtable for (anonymous namespace)::DwarfEHPrepare"},
    {"_ZN4llvm13EdgeBundlesIDE", "llvm::EdgeBundlesID"},
    {"_ZTVN4llvm11EdgeBundlesE", "vtable for llvm::EdgeBundles"},
    {"_ZTVN12_GLOBAL__N_117ExpandISelPseudosE", "vtable for (anonymous namespace)::ExpandISelPseudos"},
    {"_ZTVN4llvm12GCModuleInfoE", "vtable for llvm::GCModuleInfo"},
    {"_ZTVN12_GLOBAL__N_17DeleterE", "vtable for (anonymous namespace)::Deleter"},
    {"_ZTVN12_GLOBAL__N_17PrinterE", "vtable for (anonymous namespace)::Printer"},
    {"_ZTVN4llvm10GCStrategyE", "vtable for llvm::GCStrategy"},
    {"_ZTVN12_GLOBAL__N_119MachineCodeAnalysisE", "vtable for (anonymous namespace)::MachineCodeAnalysis"},
    {"_ZTVN12_GLOBAL__N_115LowerIntrinsicsE", "vtable for (anonymous namespace)::LowerIntrinsics"},
    {"_ZTVN12_GLOBAL__N_111IfConverterE", "vtable for (anonymous namespace)::IfConverter"},
    {"_ZTVN12_GLOBAL__N_113InlineSpillerE", "vtable for (anonymous namespace)::InlineSpiller"},
    {"_ZTVN4llvm17LLVMTargetMachineE", "vtable for llvm::LLVMTargetMachine"},
    {"_ZTVN4llvm20LatencyPriorityQueueE", "vtable for llvm::LatencyPriorityQueue"},
    {"_ZTVN4llvm18LiveDebugVariablesE", "vtable for llvm::LiveDebugVariables"},
    {"_ZTVN4llvm13LiveIntervalsE", "vtable for llvm::LiveIntervals"},
    {"_ZN4llvm12LiveStacksIDE", "llvm::LiveStacksID"},
    {"_ZTVN4llvm10LiveStacksE", "vtable for llvm::LiveStacks"},
    {"_ZTVN4llvm13LiveVariablesE", "vtable for llvm::LiveVariables"},
    {"_ZTVN12_GLOBAL__N_118LocalStackSlotPassE", "vtable for (anonymous namespace)::LocalStackSlotPass"},
    {"_ZTVN12_GLOBAL__N_127LowerSubregsInstructionPassE", "vtable for (anonymous namespace)::LowerSubregsInstructionPass"},
    {"_ZTVN12_GLOBAL__N_110MachineCSEE", "vtable for (anonymous namespace)::MachineCSE"},
    {"_ZN4llvm19MachineDominatorsIDE", "llvm::MachineDominatorsID"},
    {"_ZTVN4llvm20MachineDominatorTreeE", "vtable for llvm::MachineDominatorTree"},
    {"_ZTVN4llvm19MachineFunctionInfoE", "vtable for llvm::MachineFunctionInfo"},
    {"_ZTVN4llvm23MachineFunctionAnalysisE", "vtable for llvm::MachineFunctionAnalysis"},
    {"_ZTVN4llvm19MachineFunctionPassE", "vtable for llvm::MachineFunctionPass"},
    {"_ZTVN12_GLOBAL__N_126MachineFunctionPrinterPassE", "vtable for (anonymous namespace)::MachineFunctionPrinterPass"},
    {"_ZTVN12_GLOBAL__N_111MachineLICME", "vtable for (anonymous namespace)::MachineLICM"},
    {"_ZN4llvm17MachineLoopInfoIDE", "llvm::MachineLoopInfoID"},
    {"_ZTVN4llvm15MachineLoopInfoE", "vtable for llvm::MachineLoopInfo"},
    {"_ZTVN4llvm21MachineModuleInfoImplE", "vtable for llvm::MachineModuleInfoImpl"},
    {"_ZTVN4llvm17MachineModuleInfoE", "vtable for llvm::MachineModuleInfo"},
    {"_ZTVN4llvm26MMIAddrLabelMapCallbackPtrE", "vtable for llvm::MMIAddrLabelMapCallbackPtr"},
    {"_ZTVN4llvm22MachineModuleInfoMachOE", "vtable for llvm::MachineModuleInfoMachO"},
    {"_ZTVN4llvm20MachineModuleInfoELFE", "vtable for llvm::MachineModuleInfoELF"},
    {"_ZTVN12_GLOBAL__N_114MachineSinkingE", "vtable for (anonymous namespace)::MachineSinking"},
    {"_ZTVN12_GLOBAL__N_119MachineVerifierPassE", "vtable for (anonymous namespace)::MachineVerifierPass"},
    {"_ZTVN12_GLOBAL__N_112OptimizePHIsE", "vtable for (anonymous namespace)::OptimizePHIs"},
    {"_ZN4llvm16PHIEliminationIDE", "llvm::PHIEliminationID"},
    {"_ZTVN12_GLOBAL__N_114PHIEliminationE", "vtable for (anonymous namespace)::PHIElimination"},
    {"_ZTVN12_GLOBAL__N_117PeepholeOptimizerE", "vtable for (anonymous namespace)::PeepholeOptimizer"},
    {"_ZTVN4llvm14AntiDepBreakerE", "vtable for llvm::AntiDepBreaker"},
    {"_ZTVN12_GLOBAL__N_115PostRASchedulerE", "vtable for (anonymous namespace)::PostRAScheduler"},
    {"_ZTVN12_GLOBAL__N_120SchedulePostRATDListE", "vtable for (anonymous namespace)::SchedulePostRATDList"},
    {"_ZN4llvm19PreAllocSplittingIDE", "llvm::PreAllocSplittingID"},
    {"_ZTVN12_GLOBAL__N_117PreAllocSplittingE", "vtable for (anonymous namespace)::PreAllocSplitting"},
    {"_ZTVN4llvm19ProcessImplicitDefsE", "vtable for llvm::ProcessImplicitDefs"},
    {"_ZTVN4llvm17PseudoSourceValueE", "vtable for llvm::PseudoSourceValue"},
    {"_ZTVN4llvm27FixedStackPseudoSourceValueE", "vtable for llvm::FixedStackPseudoSourceValue"},
    {"_ZTVN12_GLOBAL__N_16RAFastE", "vtable for (anonymous namespace)::RAFast"},
    {"_ZTVN12_GLOBAL__N_19RALinScanE", "vtable for (anonymous namespace)::RALinScan"},
    {"_ZTVN4llvm17RegisterCoalescerE", "vtable for llvm::RegisterCoalescer"},
    {"_ZTVN4llvm11ScheduleDAGE", "vtable for llvm::ScheduleDAG"},
    {"_ZTVN4llvm24ScheduleHazardRecognizerE", "vtable for llvm::ScheduleHazardRecognizer"},
    {"_ZTVN4llvm17ScheduleDAGInstrsE", "vtable for llvm::ScheduleDAGInstrs"},
    {"_ZTVN4llvm26ScoreboardHazardRecognizerE", "vtable for llvm::ScoreboardHazardRecognizer"},
    {"_ZTVN4llvm3PEIE", "vtable for llvm::PEI"},
    {"_ZN4llvm26SimpleRegisterCoalescingIDE", "llvm::SimpleRegisterCoalescingID"},
    {"_ZTVN4llvm24SimpleRegisterCoalescingE", "vtable for llvm::SimpleRegisterCoalescing"},
    {"_ZTVN12_GLOBAL__N_110SjLjEHPassE", "vtable for (anonymous namespace)::SjLjEHPass"},
    {"_ZTVN4llvm11SlotIndexesE", "vtable for llvm::SlotIndexes"},
    {"_ZTVN12_GLOBAL__N_115StandardSpillerE", "vtable for (anonymous namespace)::StandardSpiller"},
    {"_ZTVN12_GLOBAL__N_114TrivialSpillerE", "vtable for (anonymous namespace)::TrivialSpiller"},
    {"_ZTVN12_GLOBAL__N_111SpillerBaseE", "vtable for (anonymous namespace)::SpillerBase"},
    {"_ZTVN12_GLOBAL__N_114StackProtectorE", "vtable for (anonymous namespace)::StackProtector"},
    {"_ZTVN12_GLOBAL__N_117StackSlotColoringE", "vtable for (anonymous namespace)::StackSlotColoring"},
    {"_ZN4llvm22StrongPHIEliminationIDE", "llvm::StrongPHIEliminationID"},
    {"_ZTVN12_GLOBAL__N_120StrongPHIEliminationE", "vtable for (anonymous namespace)::StrongPHIElimination"},
    {"_ZTVN12_GLOBAL__N_117TailDuplicatePassE", "vtable for (anonymous namespace)::TailDuplicatePass"},
    {"_ZTVN4llvm19TargetInstrInfoImplE", "vtable for llvm::TargetInstrInfoImpl"},
    {"_ZTVN4llvm27TargetLoweringObjectFileELFE", "vtable for llvm::TargetLoweringObjectFileELF"},
    {"_ZTVN4llvm29TargetLoweringObjectFileMachOE", "vtable for llvm::TargetLoweringObjectFileMachO"},
    {"_ZTVN4llvm28TargetLoweringObjectFileCOFFE", "vtable for llvm::TargetLoweringObjectFileCOFF"},
    {"_ZN4llvm27TwoAddressInstructionPassIDE", "llvm::TwoAddressInstructionPassID"},
    {"_ZTVN12_GLOBAL__N_125TwoAddressInstructionPassE", "vtable for (anonymous namespace)::TwoAddressInstructionPass"},
    {"_ZN4llvm29UnreachableMachineBlockElimIDE", "llvm::UnreachableMachineBlockElimID"},
    {"_ZTVN12_GLOBAL__N_127UnreachableMachineBlockElimE", "vtable for (anonymous namespace)::UnreachableMachineBlockElim"},
    {"_ZTVN12_GLOBAL__N_120UnreachableBlockElimE", "vtable for (anonymous namespace)::UnreachableBlockElim"},
    {"_ZTVN4llvm10VirtRegMapE", "vtable for llvm::VirtRegMap"},
    {"_ZTVN4llvm15VirtRegRewriterE", "vtable for llvm::VirtRegRewriter"},
    {"_ZTVN12_GLOBAL__N_115TrivialRewriterE", "vtable for (anonymous namespace)::TrivialRewriter"},
    {"_ZTVN12_GLOBAL__N_113LocalRewriterE", "vtable for (anonymous namespace)::LocalRewriter"},
    {"_ZTVN12_GLOBAL__N_14ADCEE", "vtable for (anonymous namespace)::ADCE"},
    {"_ZTVN12_GLOBAL__N_114CodeGenPrepareE", "vtable for (anonymous namespace)::CodeGenPrepare"},
    {"_ZTVN12_GLOBAL__N_131CodeGenPrepareFortifiedLibCallsE", "vtable for (anonymous namespace)::CodeGenPrepareFortifiedLibCalls"},
    {"_ZTVN12_GLOBAL__N_126CorrelatedValuePropagationE", "vtable for (anonymous namespace)::CorrelatedValuePropagation"},
    {"_ZTVN12_GLOBAL__N_13DSEE", "vtable for (anonymous namespace)::DSE"},
    {"_ZTVN12_GLOBAL__N_18EarlyCSEE", "vtable for (anonymous namespace)::EarlyCSE"},
    {"_ZTVN12_GLOBAL__N_111GEPSplitterE", "vtable for (anonymous namespace)::GEPSplitter"},
    {"_ZTVN12_GLOBAL__N_13GVNE", "vtable for (anonymous namespace)::GVN"},
    {"_ZTVN12_GLOBAL__N_114IndVarSimplifyE", "vtable for (anonymous namespace)::IndVarSimplify"},
    {"_ZTVN12_GLOBAL__N_113JumpThreadingE", "vtable for (anonymous namespace)::JumpThreading"},
    {"_ZTVN12_GLOBAL__N_14LICME", "vtable for (anonymous namespace)::LICM"},
    {"_ZTVN12_GLOBAL__N_112LoopPromoterE", "vtable for (anonymous namespace)::LoopPromoter"},
    {"_ZTVN12_GLOBAL__N_112LoopDeletionE", "vtable for (anonymous namespace)::LoopDeletion"},
    {"_ZTVN12_GLOBAL__N_118LoopIdiomRecognizeE", "vtable for (anonymous namespace)::LoopIdiomRecognize"},
    {"_ZTVN12_GLOBAL__N_110LoopRotateE", "vtable for (anonymous namespace)::LoopRotate"},
    {"_ZTVN12_GLOBAL__N_118LoopStrengthReduceE", "vtable for (anonymous namespace)::LoopStrengthReduce"},
    {"_ZTVN12_GLOBAL__N_110LoopUnrollE", "vtable for (anonymous namespace)::LoopUnroll"},
    {"_ZTVN12_GLOBAL__N_112LoopUnswitchE", "vtable for (anonymous namespace)::LoopUnswitch"},
    {"_ZTVN12_GLOBAL__N_19MemCpyOptE", "vtable for (anonymous namespace)::MemCpyOpt"},
    {"_ZTVN12_GLOBAL__N_111ReassociateE", "vtable for (anonymous namespace)::Reassociate"},
    {"_ZTVN12_GLOBAL__N_16IPSCCPE", "vtable for (anonymous namespace)::IPSCCP"},
    {"_ZTVN12_GLOBAL__N_14SCCPE", "vtable for (anonymous namespace)::SCCP"},
    {"_ZTVN12_GLOBAL__N_110SROA_SSAUpE", "vtable for (anonymous namespace)::SROA_SSAUp"},
    {"_ZTVN12_GLOBAL__N_114AllocaPromoterE", "vtable for (anonymous namespace)::AllocaPromoter"},
    {"_ZTVN12_GLOBAL__N_14SROAE", "vtable for (anonymous namespace)::SROA"},
    {"_ZTVN12_GLOBAL__N_17SROA_DTE", "vtable for (anonymous namespace)::SROA_DT"},
    {"_ZTVN12_GLOBAL__N_115CFGSimplifyPassE", "vtable for (anonymous namespace)::CFGSimplifyPass"},
    {"_ZTVN12_GLOBAL__N_116SimplifyLibCallsE", "vtable for (anonymous namespace)::SimplifyLibCalls"},
    {"_ZTVN12_GLOBAL__N_19StrCatOptE", "vtable for (anonymous namespace)::StrCatOpt"},
    {"_ZTVN12_GLOBAL__N_119LibCallOptimizationE", "vtable for (anonymous namespace)::LibCallOptimization"},
    {"_ZTVN12_GLOBAL__N_110StrNCatOptE", "vtable for (anonymous namespace)::StrNCatOpt"},
    {"_ZTVN12_GLOBAL__N_19StrChrOptE", "vtable for (anonymous namespace)::StrChrOpt"},
    {"_ZTVN12_GLOBAL__N_110StrRChrOptE", "vtable for (anonymous namespace)::StrRChrOpt"},
    {"_ZTVN12_GLOBAL__N_19StrCmpOptE", "vtable for (anonymous namespace)::StrCmpOpt"},
    {"_ZTVN12_GLOBAL__N_110StrNCmpOptE", "vtable for (anonymous namespace)::StrNCmpOpt"},
    {"_ZTVN12_GLOBAL__N_19StrCpyOptE", "vtable for (anonymous namespace)::StrCpyOpt"},
    {"_ZTVN12_GLOBAL__N_110StrNCpyOptE", "vtable for (anonymous namespace)::StrNCpyOpt"},
    {"_ZTVN12_GLOBAL__N_19StrLenOptE", "vtable for (anonymous namespace)::StrLenOpt"},
    {"_ZTVN12_GLOBAL__N_110StrPBrkOptE", "vtable for (anonymous namespace)::StrPBrkOpt"},
    {"_ZTVN12_GLOBAL__N_18StrToOptE", "vtable for (anonymous namespace)::StrToOpt"},
    {"_ZTVN12_GLOBAL__N_19StrSpnOptE", "vtable for (anonymous namespace)::StrSpnOpt"},
    {"_ZTVN12_GLOBAL__N_110StrCSpnOptE", "vtable for (anonymous namespace)::StrCSpnOpt"},
    {"_ZTVN12_GLOBAL__N_19StrStrOptE", "vtable for (anonymous namespace)::StrStrOpt"},
    {"_ZTVN12_GLOBAL__N_19MemCmpOptE", "vtable for (anonymous namespace)::MemCmpOpt"},
    {"_ZTVN12_GLOBAL__N_19MemCpyOptE", "vtable for (anonymous namespace)::MemCpyOpt"},
    {"_ZTVN12_GLOBAL__N_110MemMoveOptE", "vtable for (anonymous namespace)::MemMoveOpt"},
    {"_ZTVN12_GLOBAL__N_19MemSetOptE", "vtable for (anonymous namespace)::MemSetOpt"},
    {"_ZTVN12_GLOBAL__N_16PowOptE", "vtable for (anonymous namespace)::PowOpt"},
    {"_ZTVN12_GLOBAL__N_17Exp2OptE", "vtable for (anonymous namespace)::Exp2Opt"},
    {"_ZTVN12_GLOBAL__N_116UnaryDoubleFPOptE", "vtable for (anonymous namespace)::UnaryDoubleFPOpt"},
    {"_ZTVN12_GLOBAL__N_16FFSOptE", "vtable for (anonymous namespace)::FFSOpt"},
    {"_ZTVN12_GLOBAL__N_16AbsOptE", "vtable for (anonymous namespace)::AbsOpt"},
    {"_ZTVN12_GLOBAL__N_110IsDigitOptE", "vtable for (anonymous namespace)::IsDigitOpt"},
    {"_ZTVN12_GLOBAL__N_110IsAsciiOptE", "vtable for (anonymous namespace)::IsAsciiOpt"},
    {"_ZTVN12_GLOBAL__N_110ToAsciiOptE", "vtable for (anonymous namespace)::ToAsciiOpt"},
    {"_ZTVN12_GLOBAL__N_110SPrintFOptE", "vtable for (anonymous namespace)::SPrintFOpt"},
    {"_ZTVN12_GLOBAL__N_19PrintFOptE", "vtable for (anonymous namespace)::PrintFOpt"},
    {"_ZTVN12_GLOBAL__N_19FWriteOptE", "vtable for (anonymous namespace)::FWriteOpt"},
    {"_ZTVN12_GLOBAL__N_18FPutsOptE", "vtable for (anonymous namespace)::FPutsOpt"},
    {"_ZTVN12_GLOBAL__N_110FPrintFOptE", "vtable for (anonymous namespace)::FPrintFOpt"},
    {"_ZTVN12_GLOBAL__N_17PutsOptE", "vtable for (anonymous namespace)::PutsOpt"},
    {"_ZTVN12_GLOBAL__N_112TailCallElimE", "vtable for (anonymous namespace)::TailCallElim"},
    {"_ZTVN12_GLOBAL__N_128InstCombineFortifiedLibCallsE", "vtable for (anonymous namespace)::InstCombineFortifiedLibCalls"},
    {"_ZTVN4llvm12InstCombinerE", "vtable for llvm::InstCombiner"},
    {"_ZN4llvm20BreakCriticalEdgesIDE", "llvm::BreakCriticalEdgesID"},
    {"_ZTVN12_GLOBAL__N_118BreakCriticalEdgesE", "vtable for (anonymous namespace)::BreakCriticalEdges"},
    {"_ZTVN4llvm25SimplifyFortifiedLibCallsE", "vtable for llvm::SimplifyFortifiedLibCalls"},
    {"_ZN4llvm7LCSSAIDE", "llvm::LCSSAID"},
    {"_ZTVN12_GLOBAL__N_15LCSSAE", "vtable for (anonymous namespace)::LCSSA"},
    {"_ZN4llvm14LoopSimplifyIDE", "llvm::LoopSimplifyID"},
    {"_ZTVN12_GLOBAL__N_112LoopSimplifyE", "vtable for (anonymous namespace)::LoopSimplify"},
    {"_ZN4llvm17LowerInvokePassIDE", "llvm::LowerInvokePassID"},
    {"_ZTVN12_GLOBAL__N_111LowerInvokeE", "vtable for (anonymous namespace)::LowerInvoke"},
    {"_ZN4llvm13LowerSwitchIDE", "llvm::LowerSwitchID"},
    {"_ZTVN12_GLOBAL__N_111LowerSwitchE", "vtable for (anonymous namespace)::LowerSwitch"},
    {"_ZTVN4llvm22UnifyFunctionExitNodesE", "vtable for llvm::UnifyFunctionExitNodes"},
    {"_ZTVN4llvm9CallGraphE", "vtable for llvm::CallGraph"},
    {"_ZTVN12_GLOBAL__N_114BasicCallGraphE", "vtable for (anonymous namespace)::BasicCallGraph"},
    {"_ZTVN4llvm16CallGraphSCCPassE", "vtable for llvm::CallGraphSCCPass"},
    {"_ZTVN12_GLOBAL__N_118PrintCallGraphPassE", "vtable for (anonymous namespace)::PrintCallGraphPass"},
    {"_ZTVN12_GLOBAL__N_113CGPassManagerE", "vtable for (anonymous namespace)::CGPassManager"},
    {"_ZTVN4llvm13FindUsedTypesE", "vtable for llvm::FindUsedTypes"},
    {"_ZTVN4llvm13AliasAnalysisE", "vtable for llvm::AliasAnalysis"},
    {"_ZTVN4llvm15AliasSetTracker13ASTCallbackVHE", "vtable for llvm::AliasSetTracker::ASTCallbackVH"},
    {"_ZTVN12_GLOBAL__N_115AliasSetPrinterE", "vtable for (anonymous namespace)::AliasSetPrinter"},
    {"_ZTVN12_GLOBAL__N_118BasicAliasAnalysisE", "vtable for (anonymous namespace)::BasicAliasAnalysis"},
    {"_ZTVN4llvm7IVUsersE", "vtable for llvm::IVUsers"},
    {"_ZTVN4llvm11IVStrideUseE", "vtable for llvm::IVStrideUse"},
    {"_ZTVN4llvm13LazyValueInfoE", "vtable for llvm::LazyValueInfo"},
    {"_ZTVN12_GLOBAL__N_114LVIValueHandleE", "vtable for (anonymous namespace)::LVIValueHandle"},
    {"_ZTVN4llvm8LoopInfoE", "vtable for llvm::LoopInfo"},
    {"_ZTVN4llvm13LPPassManagerE", "vtable for llvm::LPPassManager"},
    {"_ZTVN4llvm8LoopPassE", "vtable for llvm::LoopPass"},
    {"_ZTVN12_GLOBAL__N_113PrintLoopPassE", "vtable for (anonymous namespace)::PrintLoopPass"},
    {"_ZTVN4llvm24MemoryDependenceAnalysisE", "vtable for llvm::MemoryDependenceAnalysis"},
    {"_ZTVN12_GLOBAL__N_14NoAAE", "vtable for (anonymous namespace)::NoAA"},
    {"_ZTVN12_GLOBAL__N_113NoProfileInfoE", "vtable for (anonymous namespace)::NoProfileInfo"},
    {"_ZTVN4llvm15ScalarEvolution14SCEVCallbackVHE", "vtable for llvm::ScalarEvolution::SCEVCallbackVH"},
    {"_ZTVN4llvm15ScalarEvolutionE", "vtable for llvm::ScalarEvolution"},
    {"_ZTVN4llvm11SCEVUnknownE", "vtable for llvm::SCEVUnknown"},
    {"_ZTVN12_GLOBAL__N_122TypeBasedAliasAnalysisE", "vtable for (anonymous namespace)::TypeBasedAliasAnalysis"},
    {"_ZTVN4llvm14TargetAsmLexerE", "vtable for llvm::TargetAsmLexer"},
    {"_ZTVN4llvm10DataLayoutE", "vtable for llvm::DataLayout"},
    {"_ZTVN12_GLOBAL__N_115StructLayoutMapE", "vtable for (anonymous namespace)::StructLayoutMap"},
    {"_ZTVN4llvm19TargetELFWriterInfoE", "vtable for llvm::TargetELFWriterInfo"},
    {"_ZTVN4llvm19TargetFrameLoweringE", "vtable for llvm::TargetFrameLowering"},
    {"_ZTVN4llvm15TargetInstrInfoE", "vtable for llvm::TargetInstrInfo"},
    {"_ZTVN4llvm24TargetLoweringObjectFileE", "vtable for llvm::TargetLoweringObjectFile"},
    {"_ZTVN4llvm13TargetMachineE", "vtable for llvm::TargetMachine"},
    {"_ZTVN4llvm18TargetRegisterInfoE", "vtable for llvm::TargetRegisterInfo"},
    {"_ZTVN4llvm15TargetSubtargetE", "vtable for llvm::TargetSubtarget"},
    {"_ZTVN4llvm24AssemblyAnnotationWriterE", "vtable for llvm::AssemblyAnnotationWriter"},
    {"_ZTVN4llvm5ValueE", "vtable for llvm::Value"},
    {"_ZTIN4llvm5ValueE", "typeinfo for llvm::Value"},
    {"_ZTVN4llvm4TypeE", "vtable for llvm::Type"},
    {"_ZTIN4llvm4TypeE", "typeinfo for llvm::Type"},
    {"_ZTIN4llvm24AssemblyAnnotationWriterE", "typeinfo for llvm::AssemblyAnnotationWriter"},
    {"_ZTVN4llvm10BasicBlockE", "vtable for llvm::BasicBlock"},
    {"_ZTIN4llvm10BasicBlockE", "typeinfo for llvm::BasicBlock"},
    {"_ZTVN4llvm10ConstantFPE", "vtable for llvm::ConstantFP"},
    {"_ZTVN4llvm13ConstantArrayE", "vtable for llvm::ConstantArray"},
    {"_ZTVN4llvm14ConstantStructE", "vtable for llvm::ConstantStruct"},
    {"_ZTVN4llvm14ConstantVectorE", "vtable for llvm::ConstantVector"},
    {"_ZTVN4llvm12BlockAddressE", "vtable for llvm::BlockAddress"},
    {"_ZTVN4llvm21ConstantAggregateZeroE", "vtable for llvm::ConstantAggregateZero"},
    {"_ZTIN4llvm21ConstantAggregateZeroE", "typeinfo for llvm::ConstantAggregateZero"},
    {"_ZTVN4llvm19ConstantPointerNullE", "vtable for llvm::ConstantPointerNull"},
    {"_ZTIN4llvm19ConstantPointerNullE", "typeinfo for llvm::ConstantPointerNull"},
    {"_ZTVN4llvm12ConstantExprE", "vtable for llvm::ConstantExpr"},
    {"_ZTIN4llvm12ConstantExprE", "typeinfo for llvm::ConstantExpr"},
    {"_ZTVN4llvm10UndefValueE", "vtable for llvm::UndefValue"},
    {"_ZTIN4llvm10UndefValueE", "typeinfo for llvm::UndefValue"},
    {"_ZTIN4llvm10ConstantFPE", "typeinfo for llvm::ConstantFP"},
    {"_ZTIN4llvm13ConstantArrayE", "typeinfo for llvm::ConstantArray"},
    {"_ZTIN4llvm14ConstantStructE", "typeinfo for llvm::ConstantStruct"},
    {"_ZTIN4llvm14ConstantVectorE", "typeinfo for llvm::ConstantVector"},
    {"_ZTIN4llvm12BlockAddressE", "typeinfo for llvm::BlockAddress"},
    {"_ZTVN4llvm10DebugRecVHE", "vtable for llvm::DebugRecVH"},
    {"_ZTIN4llvm10DebugRecVHE", "typeinfo for llvm::DebugRecVH"},
    {"_ZTVN4llvm13DominatorTreeE", "vtable for llvm::DominatorTree"},
    {"_ZTIN4llvm13DominatorTreeE", "typeinfo for llvm::DominatorTree"},
    {"_ZTVN4llvm8FunctionE", "vtable for llvm::Function"},
    {"_ZZN4llvm9Intrinsic7getNameENS0_2IDEPPKNS_4TypeEjE5Table", "llvm::Intrinsic::getName(llvm::Intrinsic::ID, llvm::Type const**, unsigned int)::Table"},
    {"_ZTIN4llvm8FunctionE", "typeinfo for llvm::Function"},
    {"_ZTVN4llvm14GVMaterializerE", "vtable for llvm::GVMaterializer"},
    {"_ZTIN4llvm14GVMaterializerE", "typeinfo for llvm::GVMaterializer"},
    {"_ZTVN4llvm14GlobalVariableE", "vtable for llvm::GlobalVariable"},
    {"_ZTVN4llvm11GlobalAliasE", "vtable for llvm::GlobalAlias"},
    {"_ZTVN4llvm11GlobalValueE", "vtable for llvm::GlobalValue"},
    {"_ZTIN4llvm11GlobalValueE", "typeinfo for llvm::GlobalValue"},
    {"_ZTIN4llvm14GlobalVariableE", "typeinfo for llvm::GlobalVariable"},
    {"_ZTIN4llvm11GlobalAliasE", "typeinfo for llvm::GlobalAlias"},
    {"_ZTVN4llvm9InlineAsmE", "vtable for llvm::InlineAsm"},
    {"_ZTIN4llvm9InlineAsmE", "typeinfo for llvm::InlineAsm"},
    {"_ZTVN4llvm11InstructionE", "vtable for llvm::Instruction"},
    {"_ZTIN4llvm11InstructionE", "typeinfo for llvm::Instruction"},
    {"_ZTVN4llvm14TerminatorInstE", "vtable for llvm::TerminatorInst"},
    {"_ZTVN4llvm16UnaryInstructionE", "vtable for llvm::UnaryInstruction"},
    {"_ZTVN4llvm7PHINodeE", "vtable for llvm::PHINode"},
    {"_ZTVN4llvm8CallInstE", "vtable for llvm::CallInst"},
    {"_ZTVN4llvm10InvokeInstE", "vtable for llvm::InvokeInst"},
    {"_ZTVN4llvm10ReturnInstE", "vtable for llvm::ReturnInst"},
    {"_ZTVN4llvm10UnwindInstE", "vtable for llvm::UnwindInst"},
    {"_ZTVN4llvm15UnreachableInstE", "vtable for llvm::UnreachableInst"},
    {"_ZTVN4llvm10BranchInstE", "vtable for llvm::BranchInst"},
    {"_ZTVN4llvm10AllocaInstE", "vtable for llvm::AllocaInst"},
    {"_ZTVN4llvm8LoadInstE", "vtable for llvm::LoadInst"},
    {"_ZTVN4llvm9StoreInstE", "vtable for llvm::StoreInst"},
    {"_ZTVN4llvm17GetElementPtrInstE", "vtable for llvm::GetElementPtrInst"},
    {"_ZTVN4llvm18ExtractElementInstE", "vtable for llvm::ExtractElementInst"},
    {"_ZTVN4llvm17InsertElementInstE", "vtable for llvm::InsertElementInst"},
    {"_ZTVN4llvm17ShuffleVectorInstE", "vtable for llvm::ShuffleVectorInst"},
    {"_ZTVN4llvm15InsertValueInstE", "vtable for llvm::InsertValueInst"},
    {"_ZTVN4llvm16ExtractValueInstE", "vtable for llvm::ExtractValueInst"},
    {"_ZTVN4llvm14BinaryOperatorE", "vtable for llvm::BinaryOperator"},
    {"_ZTVN4llvm9TruncInstE", "vtable for llvm::TruncInst"},
    {"_ZTVN4llvm8ZExtInstE", "vtable for llvm::ZExtInst"},
    {"_ZTVN4llvm8SExtInstE", "vtable for llvm::SExtInst"},
    {"_ZTVN4llvm11FPTruncInstE", "vtable for llvm::FPTruncInst"},
    {"_ZTVN4llvm9FPExtInstE", "vtable for llvm::FPExtInst"},
    {"_ZTVN4llvm10UIToFPInstE", "vtable for llvm::UIToFPInst"},
    {"_ZTVN4llvm10SIToFPInstE", "vtable for llvm::SIToFPInst"},
    {"_ZTVN4llvm10FPToUIInstE", "vtable for llvm::FPToUIInst"},
    {"_ZTVN4llvm10FPToSIInstE", "vtable for llvm::FPToSIInst"},
    {"_ZTVN4llvm12PtrToIntInstE", "vtable for llvm::PtrToIntInst"},
    {"_ZTVN4llvm12IntToPtrInstE", "vtable for llvm::IntToPtrInst"},
    {"_ZTVN4llvm11BitCastInstE", "vtable for llvm::BitCastInst"},
    {"_ZTVN4llvm7CmpInstE", "vtable for llvm::CmpInst"},
    {"_ZTVN4llvm10SwitchInstE", "vtable for llvm::SwitchInst"},
    {"_ZTVN4llvm14IndirectBrInstE", "vtable for llvm::IndirectBrInst"},
    {"_ZTIN4llvm14TerminatorInstE", "typeinfo for llvm::TerminatorInst"},
    {"_ZTIN4llvm16UnaryInstructionE", "typeinfo for llvm::UnaryInstruction"},
    {"_ZTIN4llvm17GetElementPtrInstE", "typeinfo for llvm::GetElementPtrInst"},
    {"_ZTIN4llvm7CmpInstE", "typeinfo for llvm::CmpInst"},
    {"_ZTVN4llvm8ICmpInstE", "vtable for llvm::ICmpInst"},
    {"_ZTIN4llvm8ICmpInstE", "typeinfo for llvm::ICmpInst"},
    {"_ZTVN4llvm8FCmpInstE", "vtable for llvm::FCmpInst"},
    {"_ZTIN4llvm8FCmpInstE", "typeinfo for llvm::FCmpInst"},
    {"_ZTIN4llvm8CallInstE", "typeinfo for llvm::CallInst"},
    {"_ZTVN4llvm10SelectInstE", "vtable for llvm::SelectInst"},
    {"_ZTIN4llvm10SelectInstE", "typeinfo for llvm::SelectInst"},
    {"_ZTVN4llvm9VAArgInstE", "vtable for llvm::VAArgInst"},
    {"_ZTIN4llvm9VAArgInstE", "typeinfo for llvm::VAArgInst"},
    {"_ZTIN4llvm18ExtractElementInstE", "typeinfo for llvm::ExtractElementInst"},
    {"_ZTIN4llvm17InsertElementInstE", "typeinfo for llvm::InsertElementInst"},
    {"_ZTIN4llvm16ExtractValueInstE", "typeinfo for llvm::ExtractValueInst"},
    {"_ZTIN4llvm15InsertValueInstE", "typeinfo for llvm::InsertValueInst"},
    {"_ZTIN4llvm7PHINodeE", "typeinfo for llvm::PHINode"},
    {"_ZTIN4llvm10ReturnInstE", "typeinfo for llvm::ReturnInst"},
    {"_ZTIN4llvm10BranchInstE", "typeinfo for llvm::BranchInst"},
    {"_ZTIN4llvm10SwitchInstE", "typeinfo for llvm::SwitchInst"},
    {"_ZTIN4llvm14IndirectBrInstE", "typeinfo for llvm::IndirectBrInst"},
    {"_ZTIN4llvm11BitCastInstE", "typeinfo for llvm::BitCastInst"},
    {"_ZTIN4llvm10InvokeInstE", "typeinfo for llvm::InvokeInst"},
    {"_ZTIN4llvm10UnwindInstE", "typeinfo for llvm::UnwindInst"},
    {"_ZTIN4llvm15UnreachableInstE", "typeinfo for llvm::UnreachableInst"},
    {"_ZTIN4llvm10AllocaInstE", "typeinfo for llvm::AllocaInst"},
    {"_ZTIN4llvm8LoadInstE", "typeinfo for llvm::LoadInst"},
    {"_ZTIN4llvm9StoreInstE", "typeinfo for llvm::StoreInst"},
    {"_ZTIN4llvm17ShuffleVectorInstE", "typeinfo for llvm::ShuffleVectorInst"},
    {"_ZTIN4llvm14BinaryOperatorE", "typeinfo for llvm::BinaryOperator"},
    {"_ZTIN4llvm9TruncInstE", "typeinfo for llvm::TruncInst"},
    {"_ZTIN4llvm8ZExtInstE", "typeinfo for llvm::ZExtInst"},
    {"_ZTIN4llvm8SExtInstE", "typeinfo for llvm::SExtInst"},
    {"_ZTIN4llvm11FPTruncInstE", "typeinfo for llvm::FPTruncInst"},
    {"_ZTIN4llvm9FPExtInstE", "typeinfo for llvm::FPExtInst"},
    {"_ZTIN4llvm10UIToFPInstE", "typeinfo for llvm::UIToFPInst"},
    {"_ZTIN4llvm10SIToFPInstE", "typeinfo for llvm::SIToFPInst"},
    {"_ZTIN4llvm10FPToUIInstE", "typeinfo for llvm::FPToUIInst"},
    {"_ZTIN4llvm10FPToSIInstE", "typeinfo for llvm::FPToSIInst"},
    {"_ZTIN4llvm12PtrToIntInstE", "typeinfo for llvm::PtrToIntInst"},
    {"_ZTIN4llvm12IntToPtrInstE", "typeinfo for llvm::IntToPtrInst"},
    {"_ZTVN4llvm6MDNodeE", "vtable for llvm::MDNode"},
    {"_ZTVN4llvm13MDNodeOperandE", "vtable for llvm::MDNodeOperand"},
    {"_ZTIN4llvm13MDNodeOperandE", "typeinfo for llvm::MDNodeOperand"},
    {"_ZTIN4llvm6MDNodeE", "typeinfo for llvm::MDNode"},
    {"_ZTVN4llvm4PassE", "vtable for llvm::Pass"},
    {"_ZTVN4llvm10ModulePassE", "vtable for llvm::ModulePass"},
    {"_ZTVN4llvm13ImmutablePassE", "vtable for llvm::ImmutablePass"},
    {"_ZTVN4llvm24PassRegistrationListenerE", "vtable for llvm::PassRegistrationListener"},
    {"_ZTVN4llvm14PassNameParserE", "vtable for llvm::PassNameParser"},
    {"_ZTIN4llvm4PassE", "typeinfo for llvm::Pass"},
    {"_ZTIN4llvm10ModulePassE", "typeinfo for llvm::ModulePass"},
    {"_ZTIN4llvm13ImmutablePassE", "typeinfo for llvm::ImmutablePass"},
    {"_ZTVN4llvm12FunctionPassE", "vtable for llvm::FunctionPass"},
    {"_ZTIN4llvm12FunctionPassE", "typeinfo for llvm::FunctionPass"},
    {"_ZTVN4llvm14BasicBlockPassE", "vtable for llvm::BasicBlockPass"},
    {"_ZTIN4llvm14BasicBlockPassE", "typeinfo for llvm::BasicBlockPass"},
    {"_ZTIN4llvm24PassRegistrationListenerE", "typeinfo for llvm::PassRegistrationListener"},
    {"_ZTIN4llvm14PassNameParserE", "typeinfo for llvm::PassNameParser"},
    {"_ZTVN12_GLOBAL__N_116GetCFGOnlyPassesE", "vtable for (anonymous namespace)::GetCFGOnlyPasses"},
    {"_ZTIN12_GLOBAL__N_116GetCFGOnlyPassesE", "typeinfo for (anonymous namespace)::GetCFGOnlyPasses"},
    {"_ZTVN4llvm17PMTopLevelManagerE", "vtable for llvm::PMTopLevelManager"},
    {"_ZTVN4llvm13PMDataManagerE", "vtable for llvm::PMDataManager"},
    {"_ZTVN4llvm19FunctionPassManagerE", "vtable for llvm::FunctionPassManager"},
    {"_ZTVN4llvm11PassManagerE", "vtable for llvm::PassManager"},
    {"_ZTVN4llvm15PassManagerBaseE", "vtable for llvm::PassManagerBase"},
    {"_ZTVN4llvm27PassManagerPrettyStackEntryE", "vtable for llvm::PassManagerPrettyStackEntry"},
    {"_ZTIN4llvm27PassManagerPrettyStackEntryE", "typeinfo for llvm::PassManagerPrettyStackEntry"},
    {"_ZTIN4llvm13PMDataManagerE", "typeinfo for llvm::PMDataManager"},
    {"_ZTVN4llvm13FPPassManagerE", "vtable for llvm::FPPassManager"},
    {"_ZTIN4llvm13FPPassManagerE", "typeinfo for llvm::FPPassManager"},
    {"_ZTIN4llvm17PMTopLevelManagerE", "typeinfo for llvm::PMTopLevelManager"},
    {"_ZTVN4llvm13MPPassManagerE", "vtable for llvm::MPPassManager"},
    {"_ZTIN4llvm13MPPassManagerE", "typeinfo for llvm::MPPassManager"},
    {"_ZTIN4llvm15PassManagerBaseE", "typeinfo for llvm::PassManagerBase"},
    {"_ZTIN4llvm19FunctionPassManagerE", "typeinfo for llvm::FunctionPassManager"},
    {"_ZTIN4llvm11PassManagerE", "typeinfo for llvm::PassManager"},
    {"_ZTVN12_GLOBAL__N_113BBPassManagerE", "vtable for (anonymous namespace)::BBPassManager"},
    {"_ZTIN12_GLOBAL__N_113BBPassManagerE", "typeinfo for (anonymous namespace)::BBPassManager"},
    {"_ZTVN12_GLOBAL__N_117PrintFunctionPassE", "vtable for (anonymous namespace)::PrintFunctionPass"},
    {"_ZTIN12_GLOBAL__N_117PrintFunctionPassE", "typeinfo for (anonymous namespace)::PrintFunctionPass"},
    {"_ZTVN12_GLOBAL__N_115PrintModulePassE", "vtable for (anonymous namespace)::PrintModulePass"},
    {"_ZTIN12_GLOBAL__N_115PrintModulePassE", "typeinfo for (anonymous namespace)::PrintModulePass"},
    {"_ZTVN4llvm16AbstractTypeUserE", "vtable for llvm::AbstractTypeUser"},
    {"_ZTVN4llvm12FunctionTypeE", "vtable for llvm::FunctionType"},
    {"_ZTVN4llvm10StructTypeE", "vtable for llvm::StructType"},
    {"_ZTVN4llvm9ArrayTypeE", "vtable for llvm::ArrayType"},
    {"_ZTVN4llvm10VectorTypeE", "vtable for llvm::VectorType"},
    {"_ZTVN4llvm11PointerTypeE", "vtable for llvm::PointerType"},
    {"_ZTIN4llvm16AbstractTypeUserE", "typeinfo for llvm::AbstractTypeUser"},
    {"_ZTVN4llvm14SequentialTypeE", "vtable for llvm::SequentialType"},
    {"_ZTIN4llvm14SequentialTypeE", "typeinfo for llvm::SequentialType"},
    {"_ZTIN4llvm12FunctionTypeE", "typeinfo for llvm::FunctionType"},
    {"_ZTIN4llvm10StructTypeE", "typeinfo for llvm::StructType"},
    {"_ZTIN4llvm9ArrayTypeE", "typeinfo for llvm::ArrayType"},
    {"_ZTIN4llvm10VectorTypeE", "typeinfo for llvm::VectorType"},
    {"_ZTIN4llvm11PointerTypeE", "typeinfo for llvm::PointerType"},
    {"_ZTVN4llvm15TypeSymbolTableE", "vtable for llvm::TypeSymbolTable"},
    {"_ZTIN4llvm15TypeSymbolTableE", "typeinfo for llvm::TypeSymbolTable"},
    {"_ZTVN4llvm10CallbackVHE", "vtable for llvm::CallbackVH"},
    {"_ZTIN4llvm10CallbackVHE", "typeinfo for llvm::CallbackVH"},
    {"_ZTVN12_GLOBAL__N_18VerifierE", "vtable for (anonymous namespace)::Verifier"},
    {"_ZTIN4llvm11InstVisitorIN12_GLOBAL__N_18VerifierEvEE", "typeinfo for llvm::InstVisitor<(anonymous namespace)::Verifier, void>"},
    {"_ZTIN12_GLOBAL__N_18VerifierE", "typeinfo for (anonymous namespace)::Verifier"},
    {"_ZTVN12_GLOBAL__N_17TypeSetE", "vtable for (anonymous namespace)::TypeSet"},
    {"_ZTIN12_GLOBAL__N_17TypeSetE", "typeinfo for (anonymous namespace)::TypeSet"},
    {"_ZTVN12_GLOBAL__N_111PreVerifierE", "vtable for (anonymous namespace)::PreVerifier"},
    {"_ZTIN12_GLOBAL__N_111PreVerifierE", "typeinfo for (anonymous namespace)::PreVerifier"},
    {"_ZTVN4llvm14ARMInstPrinterE", "vtable for llvm::ARMInstPrinter"},
    {"_ZTVN12_GLOBAL__N_118X86ELFObjectWriterE", "vtable for (anonymous namespace)::X86ELFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_115ELFObjectWriterE", "vtable for (anonymous namespace)::ELFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_121MBlazeELFObjectWriterE", "vtable for (anonymous namespace)::MBlazeELFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_118ARMELFObjectWriterE", "vtable for (anonymous namespace)::ARMELFObjectWriter"},
    {"_ZTVN4llvm9MCAsmInfoE", "vtable for llvm::MCAsmInfo"},
    {"_ZTVN12_GLOBAL__N_113MCAsmStreamerE", "vtable for (anonymous namespace)::MCAsmStreamer"},
    {"_ZTVN4llvm10MCFragmentE", "vtable for llvm::MCFragment"},
    {"_ZTVN4llvm13MCCodeEmitterE", "vtable for llvm::MCCodeEmitter"},
    {"_ZTVN4llvm23MCELFObjectTargetWriterE", "vtable for llvm::MCELFObjectTargetWriter"},
    {"_ZTVN12_GLOBAL__N_113MCELFStreamerE", "vtable for (anonymous namespace)::MCELFStreamer"},
    {"_ZTVN4llvm12MCTargetExprE", "vtable for llvm::MCTargetExpr"},
    {"_ZTVN4llvm13MCInstPrinterE", "vtable for llvm::MCInstPrinter"},
    {"_ZTVN12_GLOBAL__N_117MCLoggingStreamerE", "vtable for (anonymous namespace)::MCLoggingStreamer"},
    {"_ZTVN12_GLOBAL__N_115MCMachOStreamerE", "vtable for (anonymous namespace)::MCMachOStreamer"},
    {"_ZTVN4llvm24MCMachObjectTargetWriterE", "vtable for llvm::MCMachObjectTargetWriter"},
    {"_ZTVN12_GLOBAL__N_114MCNullStreamerE", "vtable for (anonymous namespace)::MCNullStreamer"},
    {"_ZTVN4llvm16MCObjectStreamerE", "vtable for llvm::MCObjectStreamer"},
    {"_ZTVN4llvm14MCObjectWriterE", "vtable for llvm::MCObjectWriter"},
    {"_ZTVN4llvm9MCSectionE", "vtable for llvm::MCSection"},
    {"_ZTVN4llvm13MCSectionCOFFE", "vtable for llvm::MCSectionCOFF"},
    {"_ZTVN4llvm12MCSectionELFE", "vtable for llvm::MCSectionELF"},
    {"_ZTVN4llvm14MCSectionMachOE", "vtable for llvm::MCSectionMachO"},
    {"_ZTVN4llvm10MCStreamerE", "vtable for llvm::MCStreamer"},
    {"_ZTVN12_GLOBAL__N_116MachObjectWriterE", "vtable for (anonymous namespace)::MachObjectWriter"},
    {"_ZTVN4llvm16TargetAsmBackendE", "vtable for llvm::TargetAsmBackend"},
    {"_ZZNK4llvm16TargetAsmBackend16getFixupKindInfoENS_11MCFixupKindEE8Builtins", "llvm::TargetAsmBackend::getFixupKindInfo(llvm::MCFixupKind) const::Builtins"},
    {"_ZTVN12_GLOBAL__N_119WinCOFFObjectWriterE", "vtable for (anonymous namespace)::WinCOFFObjectWriter"},
    {"_ZTVN12_GLOBAL__N_115WinCOFFStreamerE", "vtable for (anonymous namespace)::WinCOFFStreamer"},
    {"_ZTVN4llvm13SlabAllocatorE", "vtable for llvm::SlabAllocator"},
    {"_ZTVN4llvm19MallocSlabAllocatorE", "vtable for llvm::MallocSlabAllocator"},
    {"_ZTIN4llvm13SlabAllocatorE", "typeinfo for llvm::SlabAllocator"},
    {"_ZTIN4llvm19MallocSlabAllocatorE", "typeinfo for llvm::MallocSlabAllocator"},
    {"_ZTVN4llvm2cl6OptionE", "vtable for llvm::cl::Option"},
    {"_ZTIN4llvm2cl6OptionE", "typeinfo for llvm::cl::Option"},
    {"_ZTVN4llvm2cl19generic_parser_baseE", "vtable for llvm::cl::generic_parser_base"},
    {"_ZTIN4llvm2cl19generic_parser_baseE", "typeinfo for llvm::cl::generic_parser_base"},
    {"_ZTIN4llvm2cl17basic_parser_implE", "typeinfo for llvm::cl::basic_parser_impl"},
    {"_ZTVN4llvm2cl17basic_parser_implE", "vtable for llvm::cl::basic_parser_impl"},
    {"_ZTVN4llvm2cl6parserIbEE", "vtable for llvm::cl::parser<bool>"},
    {"_ZTIN4llvm2cl6parserIbEE", "typeinfo for llvm::cl::parser<bool>"},
    {"_ZTVN4llvm2cl6parserINS0_13boolOrDefaultEEE", "vtable for llvm::cl::parser<llvm::cl::boolOrDefault>"},
    {"_ZTIN4llvm2cl6parserINS0_13boolOrDefaultEEE", "typeinfo for llvm::cl::parser<llvm::cl::boolOrDefault>"},
    {"_ZTVN4llvm2cl6parserIiEE", "vtable for llvm::cl::parser<int>"},
    {"_ZTIN4llvm2cl6parserIiEE", "typeinfo for llvm::cl::parser<int>"},
    {"_ZTVN4llvm2cl6parserIjEE", "vtable for llvm::cl::parser<unsigned int>"},
    {"_ZTIN4llvm2cl6parserIjEE", "typeinfo for llvm::cl::parser<unsigned int>"},
    {"_ZTVN4llvm2cl6parserIdEE", "vtable for llvm::cl::parser<double>"},
    {"_ZTIN4llvm2cl6parserIdEE", "typeinfo for llvm::cl::parser<double>"},
    {"_ZTVN4llvm2cl6parserIfEE", "vtable for llvm::cl::parser<float>"},
    {"_ZTIN4llvm2cl6parserIfEE", "typeinfo for llvm::cl::parser<float>"},
    {"_ZTVN4llvm2cl6parserISsEE", "vtable for llvm::cl::parser<std::string>"},
    {"_ZTIN4llvm2cl6parserISsEE", "typeinfo for llvm::cl::parser<std::string>"},
    {"_ZTVN4llvm2cl6parserIcEE", "vtable for llvm::cl::parser<char>"},
    {"_ZTIN4llvm2cl6parserIcEE", "typeinfo for llvm::cl::parser<char>"},
    {"_ZTVN4llvm2cl5aliasE", "vtable for llvm::cl::alias"},
    {"_ZTIN4llvm2cl5aliasE", "typeinfo for llvm::cl::alias"},
    {"_ZTIN4llvm2cl11opt_storageIN12_GLOBAL__N_114VersionPrinterELb1ELb1EEE", "typeinfo for llvm::cl::opt_storage<(anonymous namespace)::VersionPrinter, true, true>"},
    {"_ZTIN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEEE", "typeinfo for llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTIN4llvm2cl11opt_storageIN12_GLOBAL__N_111HelpPrinterELb1ELb1EEE", "typeinfo for llvm::cl::opt_storage<(anonymous namespace)::HelpPrinter, true, true>"},
    {"_ZTIN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEEE", "typeinfo for llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTIN4llvm3sys11ThreadLocalIKN12_GLOBAL__N_124CrashRecoveryContextImplEEE", "typeinfo for llvm::sys::ThreadLocal<(anonymous namespace)::CrashRecoveryContextImpl const>"},
    {"_ZTVN4llvm14FoldingSetImplE", "vtable for llvm::FoldingSetImpl"},
    {"_ZTIN4llvm14FoldingSetImplE", "typeinfo for llvm::FoldingSetImpl"},
    {"_ZTVN4llvm21formatted_raw_ostreamE", "vtable for llvm::formatted_raw_ostream"},
    {"_ZTIN4llvm21formatted_raw_ostreamE", "typeinfo for llvm::formatted_raw_ostream"},
    {"_ZTVN4llvm12MemoryBufferE", "vtable for llvm::MemoryBuffer"},
    {"_ZTIN4llvm12MemoryBufferE", "typeinfo for llvm::MemoryBuffer"},
    {"_ZTVN12_GLOBAL__N_120MemoryBufferMMapFileE", "vtable for (anonymous namespace)::MemoryBufferMMapFile"},
    {"_ZTIN12_GLOBAL__N_115MemoryBufferMemE", "typeinfo for (anonymous namespace)::MemoryBufferMem"},
    {"_ZTIN12_GLOBAL__N_120MemoryBufferMMapFileE", "typeinfo for (anonymous namespace)::MemoryBufferMMapFile"},
    {"_ZTVN12_GLOBAL__N_115MemoryBufferMemE", "vtable for (anonymous namespace)::MemoryBufferMem"},
    {"_ZTVN4llvm21PrettyStackTraceEntryE", "vtable for llvm::PrettyStackTraceEntry"},
    {"_ZTIN4llvm21PrettyStackTraceEntryE", "typeinfo for llvm::PrettyStackTraceEntry"},
    {"_ZTVN4llvm22PrettyStackTraceStringE", "vtable for llvm::PrettyStackTraceString"},
    {"_ZTIN4llvm22PrettyStackTraceStringE", "typeinfo for llvm::PrettyStackTraceString"},
    {"_ZTVN4llvm23PrettyStackTraceProgramE", "vtable for llvm::PrettyStackTraceProgram"},
    {"_ZTIN4llvm23PrettyStackTraceProgramE", "typeinfo for llvm::PrettyStackTraceProgram"},
    {"_ZTVN4llvm3sys15ThreadLocalImplE", "vtable for llvm::sys::ThreadLocalImpl"},
    {"_ZTIN4llvm3sys15ThreadLocalImplE", "typeinfo for llvm::sys::ThreadLocalImpl"},
    {"_ZTVN4llvm11raw_ostreamE", "vtable for llvm::raw_ostream"},
    {"_ZTVN4llvm14raw_fd_ostreamE", "vtable for llvm::raw_fd_ostream"},
    {"_ZTVN4llvm18raw_string_ostreamE", "vtable for llvm::raw_string_ostream"},
    {"_ZTVN4llvm19raw_svector_ostreamE", "vtable for llvm::raw_svector_ostream"},
    {"_ZTVN4llvm16raw_null_ostreamE", "vtable for llvm::raw_null_ostream"},
    {"_ZTIN4llvm11raw_ostreamE", "typeinfo for llvm::raw_ostream"},
    {"_ZTIN4llvm18raw_string_ostreamE", "typeinfo for llvm::raw_string_ostream"},
    {"_ZTIN4llvm16raw_null_ostreamE", "typeinfo for llvm::raw_null_ostream"},
    {"_ZTVN4llvm18format_object_baseE", "vtable for llvm::format_object_base"},
    {"_ZTIN4llvm18format_object_baseE", "typeinfo for llvm::format_object_base"},
    {"_ZTIN4llvm14raw_fd_ostreamE", "typeinfo for llvm::raw_fd_ostream"},
    {"_ZTIN4llvm19raw_svector_ostreamE", "typeinfo for llvm::raw_svector_ostream"},
    {"_ZTVN4llvm14error_categoryE", "vtable for llvm::error_category"},
    {"_ZTIN4llvm14error_categoryE", "typeinfo for llvm::error_category"},
    {"_ZTVN4llvm11_do_messageE", "vtable for llvm::_do_message"},
    {"_ZTIN4llvm11_do_messageE", "typeinfo for llvm::_do_message"},
    {"_ZTVN4llvm23_generic_error_categoryE", "vtable for llvm::_generic_error_category"},
    {"_ZTIN4llvm23_generic_error_categoryE", "typeinfo for llvm::_generic_error_category"},
    {"_ZTVN4llvm22_system_error_categoryE", "vtable for llvm::_system_error_category"},
    {"_ZTIN4llvm22_system_error_categoryE", "typeinfo for llvm::_system_error_category"},
    {"_ZTVN5clang17PreprocessorLexerE", "vtable for clang::PreprocessorLexer"},
    {"_ZTVN5clang6driver5tools6darwin10DarwinToolE", "vtable for clang::driver::tools::darwin::DarwinTool"},
    {"_ZTVN5clang6driver5tools6darwin3CC1E", "vtable for clang::driver::tools::darwin::CC1"},
    {"_ZTVN5clang19FileSystemStatCacheE", "vtable for clang::FileSystemStatCache"},
    {"_ZTVN5clang13CodeGeneratorE", "vtable for clang::CodeGenerator"},
    {"_ZTVN5clang14reachable_code8CallbackE", "vtable for clang::reachable_code::Callback"},
    {"_ZTVN5clang15InheritableAttrE", "vtable for clang::InheritableAttr"},
    {"_ZTVN5clang4ento9SubEngineE", "vtable for clang::ento::SubEngine"},
    {"_ZTVN5clang4ento12SubRegionMapE", "vtable for clang::ento::SubRegionMap"},
    {"_ZTVN5clang4ento9BugReport12NodeResolverE", "vtable for clang::ento::BugReport::NodeResolver"},
    {"_ZTVN5clang4ento7GRState7PrinterE", "vtable for clang::ento::GRState::Printer"},
    {"_ZTVN5clang4ento12SubRegionMap7VisitorE", "vtable for clang::ento::SubRegionMap::Visitor"},
    {"_ZTVN5clang4ento11TypedRegionE", "vtable for clang::ento::TypedRegion"},
    {"_ZTVN5clang4ento14CodeTextRegionE", "vtable for clang::ento::CodeTextRegion"},
    {"_ZTVN5clang4ento11SValBuilderE", "vtable for clang::ento::SValBuilder"},
    {"_ZTVN5clang4ento7SymExprE", "vtable for clang::ento::SymExpr"},
    {"_ZTVN5clang4ento10SymbolDataE", "vtable for clang::ento::SymbolData"},
    {"_ZTVN5clang13MangleContextE", "vtable for clang::MangleContext"},
    {"_ZTVN4llvm18MCParsedAsmOperandE", "vtable for llvm::MCParsedAsmOperand"},
    {"_ZTVN4llvm24MachineConstantPoolValueE", "vtable for llvm::MachineConstantPoolValue"},
    {"_ZTVN4llvm13TargetJITInfoE", "vtable for llvm::TargetJITInfo"},
    {"_ZTVN4llvm27MachinePassRegistryListenerE", "vtable for llvm::MachinePassRegistryListener"},
    {"_ZTVN4llvm8DIEValueE", "vtable for llvm::DIEValue"},
    {"_ZTVN4llvm23SchedulingPriorityQueueE", "vtable for llvm::SchedulingPriorityQueue"},
    {"_ZTVN4llvm7SpillerE", "vtable for llvm::Spiller"},
    {"_ZTIN4llvm10FoldingSetINS_17AttributeListImplEEE", "typeinfo for llvm::FoldingSet<llvm::AttributeListImpl>"},
    {"_ZTIN4llvm15ilist_half_nodeINS_10BasicBlockEEE", "typeinfo for llvm::ilist_half_node<llvm::BasicBlock>"},
    {"_ZTIN4llvm10ilist_nodeINS_10BasicBlockEEE", "typeinfo for llvm::ilist_node<llvm::BasicBlock>"},
    {"_ZTIN4llvm4UserE", "typeinfo for llvm::User"},
    {"_ZTIN4llvm8ConstantE", "typeinfo for llvm::Constant"},
    {"_ZTIN4llvm25GetElementPtrConstantExprE", "typeinfo for llvm::GetElementPtrConstantExpr"},
    {"_ZTIN4llvm11ConstantIntE", "typeinfo for llvm::ConstantInt"},
    {"_ZTVN4llvm8ConstantE", "vtable for llvm::Constant"},
    {"_ZTIN4llvm19CompareConstantExprE", "typeinfo for llvm::CompareConstantExpr"},
    {"_ZTIN4llvm24ExtractValueConstantExprE", "typeinfo for llvm::ExtractValueConstantExpr"},
    {"_ZTIN4llvm23InsertValueConstantExprE", "typeinfo for llvm::InsertValueConstantExpr"},
    {"_ZTIN4llvm25ShuffleVectorConstantExprE", "typeinfo for llvm::ShuffleVectorConstantExpr"},
    {"_ZTIN4llvm25InsertElementConstantExprE", "typeinfo for llvm::InsertElementConstantExpr"},
    {"_ZTIN4llvm26ExtractElementConstantExprE", "typeinfo for llvm::ExtractElementConstantExpr"},
    {"_ZTIN4llvm18SelectConstantExprE", "typeinfo for llvm::SelectConstantExpr"},
    {"_ZTIN4llvm18BinaryConstantExprE", "typeinfo for llvm::BinaryConstantExpr"},
    {"_ZTIN4llvm17UnaryConstantExprE", "typeinfo for llvm::UnaryConstantExpr"},
    {"_ZTIN4llvm13DominatorBaseINS_10BasicBlockEEE", "typeinfo for llvm::DominatorBase<llvm::BasicBlock>"},
    {"_ZTIN4llvm17DominatorTreeBaseINS_10BasicBlockEEE", "typeinfo for llvm::DominatorTreeBase<llvm::BasicBlock>"},
    {"_ZTIN4llvm15ilist_half_nodeINS_8FunctionEEE", "typeinfo for llvm::ilist_half_node<llvm::Function>"},
    {"_ZTIN4llvm10ilist_nodeINS_8FunctionEEE", "typeinfo for llvm::ilist_node<llvm::Function>"},
    {"_ZTIN4llvm15ilist_half_nodeINS_8ArgumentEEE", "typeinfo for llvm::ilist_half_node<llvm::Argument>"},
    {"_ZTIN4llvm10ilist_nodeINS_8ArgumentEEE", "typeinfo for llvm::ilist_node<llvm::Argument>"},
    {"_ZTIN4llvm8ArgumentE", "typeinfo for llvm::Argument"},
    {"_ZTIN4llvm15ilist_half_nodeINS_14GlobalVariableEEE", "typeinfo for llvm::ilist_half_node<llvm::GlobalVariable>"},
    {"_ZTIN4llvm10ilist_nodeINS_14GlobalVariableEEE", "typeinfo for llvm::ilist_node<llvm::GlobalVariable>"},
    {"_ZTIN4llvm15ilist_half_nodeINS_11GlobalAliasEEE", "typeinfo for llvm::ilist_half_node<llvm::GlobalAlias>"},
    {"_ZTIN4llvm10ilist_nodeINS_11GlobalAliasEEE", "typeinfo for llvm::ilist_node<llvm::GlobalAlias>"},
    {"_ZTIN4llvm15ilist_half_nodeINS_11InstructionEEE", "typeinfo for llvm::ilist_half_node<llvm::Instruction>"},
    {"_ZTIN4llvm10ilist_nodeINS_11InstructionEEE", "typeinfo for llvm::ilist_node<llvm::Instruction>"},
    {"_ZTIN4llvm8CastInstE", "typeinfo for llvm::CastInst"},
    {"_ZTVN4llvm8CastInstE", "vtable for llvm::CastInst"},
    {"_ZTIN4llvm15ValueHandleBaseE", "typeinfo for llvm::ValueHandleBase"},
    {"_ZTIN4llvm10FoldingSetINS_6MDNodeEEE", "typeinfo for llvm::FoldingSet<llvm::MDNode>"},
    {"_ZTIN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EEE", "typeinfo for llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>"},
    {"_ZTIN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EEE", "typeinfo for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>"},
    {"_ZTIN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EEE", "typeinfo for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>"},
    {"_ZTIN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EEE", "typeinfo for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>"},
    {"_ZTIN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EEE", "typeinfo for llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>"},
    {"_ZTIN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EEE", "typeinfo for llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>"},
    {"_ZTIN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EEE", "typeinfo for llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>"},
    {"_ZTIN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EEE", "typeinfo for llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>"},
    {"_ZTIN4llvm11DerivedTypeE", "typeinfo for llvm::DerivedType"},
    {"_ZTIN4llvm11IntegerTypeE", "typeinfo for llvm::IntegerType"},
    {"_ZTIN4llvm14FoldingSetImpl4NodeE", "typeinfo for llvm::FoldingSetImpl::Node"},
    {"_ZTIN4llvm8MDStringE", "typeinfo for llvm::MDString"},
    {"_ZTIN4llvm2cl6parserIPKNS_8PassInfoEEE", "typeinfo for llvm::cl::parser<llvm::PassInfo const*>"},
    {"_ZTIN4llvm2cl11opt_storageIbLb1ELb0EEE", "typeinfo for llvm::cl::opt_storage<bool, true, false>"},
    {"_ZTIN4llvm2cl3optIbLb1ENS0_6parserIbEEEE", "typeinfo for llvm::cl::opt<bool, true, llvm::cl::parser<bool> >"},
    {"_ZTISt12_Vector_baseIPKN4llvm8PassInfoESaIS3_EE", "typeinfo for std::_Vector_base<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> >"},
    {"_ZTISt6vectorIPKN4llvm8PassInfoESaIS3_EE", "typeinfo for std::vector<llvm::PassInfo const*, std::allocator<llvm::PassInfo const*> >"},
    {"_ZTIN4llvm2cl12list_storageIPKNS_8PassInfoEbEE", "typeinfo for llvm::cl::list_storage<llvm::PassInfo const*, bool>"},
    {"_ZTIN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEEE", "typeinfo for llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>"},
    {"_ZTIN4llvm2cl11opt_storageINS_14PassDebugLevelELb0ELb0EEE", "typeinfo for llvm::cl::opt_storage<llvm::PassDebugLevel, false, false>"},
    {"_ZTIN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEEE", "typeinfo for llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >"},
    {"_ZTIN4llvm2cl6parserINS_14PassDebugLevelEEE", "typeinfo for llvm::cl::parser<llvm::PassDebugLevel>"},
    {"_ZTIN4llvm15PassManagerImplE", "typeinfo for llvm::PassManagerImpl"},
    {"_ZTIN4llvm23FunctionPassManagerImplE", "typeinfo for llvm::FunctionPassManagerImpl"},
    {"_ZTIN4llvm13CompositeTypeE", "typeinfo for llvm::CompositeType"},
    {"_ZTIN4llvm10OpaqueTypeE", "typeinfo for llvm::OpaqueType"},
    {"_ZTVN4llvm13CompositeTypeE", "vtable for llvm::CompositeType"},
    {"_ZTIN4llvm2cl12basic_parserIbEE", "typeinfo for llvm::cl::basic_parser<bool>"},
    {"_ZTIN4llvm2cl12basic_parserINS0_13boolOrDefaultEEE", "typeinfo for llvm::cl::basic_parser<llvm::cl::boolOrDefault>"},
    {"_ZTIN4llvm2cl12basic_parserIiEE", "typeinfo for llvm::cl::basic_parser<int>"},
    {"_ZTIN4llvm2cl12basic_parserIjEE", "typeinfo for llvm::cl::basic_parser<unsigned int>"},
    {"_ZTIN4llvm2cl12basic_parserIdEE", "typeinfo for llvm::cl::basic_parser<double>"},
    {"_ZTIN4llvm2cl12basic_parserIfEE", "typeinfo for llvm::cl::basic_parser<float>"},
    {"_ZTIN4llvm2cl12basic_parserISsEE", "typeinfo for llvm::cl::basic_parser<std::string>"},
    {"_ZTIN4llvm2cl12basic_parserIcEE", "typeinfo for llvm::cl::basic_parser<char>"},
    {"_ZTIN4llvm2cl11opt_storageIjLb0ELb0EEE", "typeinfo for llvm::cl::opt_storage<unsigned int, false, false>"},
    {"_ZTIN4llvm2cl3optIjLb0ENS0_6parserIjEEEE", "typeinfo for llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >"},
    {"_ZTIN4llvm2cl11opt_storageIiLb0ELb0EEE", "typeinfo for llvm::cl::opt_storage<int, false, false>"},
    {"_ZTIN4llvm2cl3optIiLb0ENS0_6parserIiEEEE", "typeinfo for llvm::cl::opt<int, false, llvm::cl::parser<int> >"},
    {"_ZTISs", "typeinfo for std::string"},
    {"_ZTIN4llvm2cl11opt_storageISsLb0ELb1EEE", "typeinfo for llvm::cl::opt_storage<std::string, false, true>"},
    {"_ZTIN4llvm2cl3optISsLb0ENS0_6parserISsEEEE", "typeinfo for llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >"},
    {"_ZTIN4llvm2cl11opt_storageIcLb0ELb0EEE", "typeinfo for llvm::cl::opt_storage<char, false, false>"},
    {"_ZTIN4llvm2cl3optIcLb0ENS0_6parserIcEEEE", "typeinfo for llvm::cl::opt<char, false, llvm::cl::parser<char> >"},
    {"_ZTIN4llvm2cl11opt_storageIbLb0ELb0EEE", "typeinfo for llvm::cl::opt_storage<bool, false, false>"},
    {"_ZTIN4llvm2cl3optIbLb0ENS0_6parserIbEEEE", "typeinfo for llvm::cl::opt<bool, false, llvm::cl::parser<bool> >"},
    {"_ZTIN4llvm3sys11ThreadLocalIKNS_21PrettyStackTraceEntryEEE", "typeinfo for llvm::sys::ThreadLocal<llvm::PrettyStackTraceEntry const>"},
    {"_ZTIN4llvm14format_object1IxEE", "typeinfo for llvm::format_object1<long long>"},
    {"_ZTIN4llvm14format_object1IdEE", "typeinfo for llvm::format_object1<double>"},
    {"_ZTIN4llvm2cl11opt_storageISsLb1ELb1EEE", "typeinfo for llvm::cl::opt_storage<std::string, true, true>"},
    {"_ZTIN4llvm2cl3optISsLb1ENS0_6parserISsEEEE", "typeinfo for llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >"},
    {"_ZN4llvm8RegistryIN5clang15PluginASTActionENS_14RegistryTraitsIS2_EEE4HeadE", "llvm::Registry<clang::PluginASTAction, llvm::RegistryTraits<clang::PluginASTAction> >::Head"},
    {"_ZTVN5clang23ChainedDiagnosticClientE", "vtable for clang::ChainedDiagnosticClient"},
    {"_ZTVN5clang5LexerE", "vtable for clang::Lexer"},
    {"_ZTVN5clang12SemaConsumerE", "vtable for clang::SemaConsumer"},
    {"_ZTVN5clang6driver11InputActionE", "vtable for clang::driver::InputAction"},
    {"_ZTVN5clang6driver14BindArchActionE", "vtable for clang::driver::BindArchAction"},
    {"_ZTVN5clang6driver9JobActionE", "vtable for clang::driver::JobAction"},
    {"_ZTVN5clang6driver19PreprocessJobActionE", "vtable for clang::driver::PreprocessJobAction"},
    {"_ZTVN5clang6driver19PrecompileJobActionE", "vtable for clang::driver::PrecompileJobAction"},
    {"_ZTVN5clang6driver16AnalyzeJobActionE", "vtable for clang::driver::AnalyzeJobAction"},
    {"_ZTVN5clang6driver16CompileJobActionE", "vtable for clang::driver::CompileJobAction"},
    {"_ZTVN5clang6driver17AssembleJobActionE", "vtable for clang::driver::AssembleJobAction"},
    {"_ZTVN5clang6driver13LinkJobActionE", "vtable for clang::driver::LinkJobAction"},
    {"_ZTVN5clang6driver13LipoJobActionE", "vtable for clang::driver::LipoJobAction"},
    {"_ZTVN5clang6driver17DsymutilJobActionE", "vtable for clang::driver::DsymutilJobAction"},
    {"_ZTVN5clang6driver7CommandE", "vtable for clang::driver::Command"},
    {"_ZTVN5clang6driver10toolchains11Generic_ELFE", "vtable for clang::driver::toolchains::Generic_ELF"},
    {"_ZTVN5clang7CodeGen12EHScopeStack19ConditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEEE", "vtable for clang::CodeGen::EHScopeStack::ConditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>"},
    {"_ZTVN5clang7CodeGen12EHScopeStack21UnconditionalCleanup1IN12_GLOBAL__N_113FreeExceptionEPN4llvm5ValueEEE", "vtable for clang::CodeGen::EHScopeStack::UnconditionalCleanup1<(anonymous namespace)::FreeException, llvm::Value*>"},
    {"_ZTVN5clang7CodeGen12EHScopeStack19ConditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEEE", "vtable for clang::CodeGen::EHScopeStack::ConditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>"},
    {"_ZTVN5clang7CodeGen12EHScopeStack21UnconditionalCleanup2IN12_GLOBAL__N_116DestroyTemporaryEPKNS_17CXXDestructorDeclEPN4llvm5ValueEEE", "vtable for clang::CodeGen::EHScopeStack::UnconditionalCleanup2<(anonymous namespace)::DestroyTemporary, clang::CXXDestructorDecl const*, llvm::Value*>"},
    {"_ZTVN5clang18EmitAssemblyActionE", "vtable for clang::EmitAssemblyAction"},
    {"_ZTVN5clang12EmitBCActionE", "vtable for clang::EmitBCAction"},
    {"_ZTVN5clang14EmitLLVMActionE", "vtable for clang::EmitLLVMAction"},
    {"_ZTVN5clang18EmitLLVMOnlyActionE", "vtable for clang::EmitLLVMOnlyAction"},
    {"_ZTVN5clang21EmitCodeGenOnlyActionE", "vtable for clang::EmitCodeGenOnlyAction"},
    {"_ZTVN5clang13EmitObjActionE", "vtable for clang::EmitObjAction"},
    {"_ZTVN5clang15BackendConsumerE", "vtable for clang::BackendConsumer"},
    {"_ZTVN4llvm10FoldingSetIN5clang7CodeGen14CGFunctionInfoEEE", "vtable for llvm::FoldingSet<clang::CodeGen::CGFunctionInfo>"},
    {"_ZTVN5clang6Parser20ObjCPropertyCallbackE", "vtable for clang::Parser::ObjCPropertyCallback"},
    {"_ZTVN5clang15UsingShadowDeclE", "vtable for clang::UsingShadowDecl"},
    {"_ZTVN5clang14AccessSpecDeclE", "vtable for clang::AccessSpecDecl"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126AdjustedReturnValueCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::AdjustedReturnValueChecker>"},
    {"_ZTVN5clang4ento15AnalysisManagerE", "vtable for clang::ento::AnalysisManager"},
    {"_ZTVN4llvm10FoldingSetIN5clang15LocationContextEEE", "vtable for llvm::FoldingSet<clang::LocationContext>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120AnalyzerStatsCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::AnalyzerStatsChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ArrayBoundCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundChecker>"},
    {"_ZTVN5clang4ento10BuiltinBugE", "vtable for clang::ento::BuiltinBug"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ArrayBoundCheckerV2EEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ArrayBoundCheckerV2>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118AttrNonNullCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::AttrNonNullChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ClassReleaseCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ClassReleaseChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_122CFRetainReleaseCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CFRetainReleaseChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CFNumberCreateCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CFNumberCreateChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113NilArgCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::NilArgChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114CStringCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CStringChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_121CallAndMessageCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CallAndMessageChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_115CastSizeCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CastSizeChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119CastToStructCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::CastToStructChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113ChrootCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ChrootChecker>"},
    {"_ZTVN5clang24LiveVariables_ValueTypes10ObserverTyE", "vtable for clang::LiveVariables_ValueTypes::ObserverTy"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114DivZeroCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::DivZeroChecker>"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento7GRStateEEE", "vtable for llvm::FoldingSet<clang::ento::GRState>"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento12ExplodedNodeEEE", "vtable for llvm::FoldingSet<clang::ento::ExplodedNode>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119FixedAddressCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::FixedAddressChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126IdempotentOperationCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::IdempotentOperationChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_116MacOSXAPICheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::MacOSXAPIChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113MallocCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::MallocChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_124NSAutoreleasePoolCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::NSAutoreleasePoolChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_123NoReturnFunctionCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::NoReturnFunctionChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117ObjCAtSyncCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ObjCAtSyncChecker>"},
    {"_ZZN5clang4ento12GRStateTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEjNS2_16ImutKeyValueInfoIS6_jEEEEE8GDMIndexEvE5index", "clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, unsigned int, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, unsigned int> > >::GDMIndex()::index"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento19BugReportEquivClassEEE", "vtable for llvm::FoldingSet<clang::ento::BugReportEquivClass>"},
    {"_ZTVN5clang4ento17EnhancedBugReportE", "vtable for clang::ento::EnhancedBugReport"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119ObjCSelfInitCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ObjCSelfInitChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_119PointerArithCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::PointerArithChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_117PointerSubCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::PointerSubChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118PthreadLockCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::PthreadLockChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_125ReturnPointerRangeCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ReturnPointerRangeChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118ReturnUndefCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::ReturnUndefChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120StackAddrLeakCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::StackAddrLeakChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_113StreamCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::StreamChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_128UndefCapturedBlockVarCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::UndefCapturedBlockVarChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_118UndefResultCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::UndefResultChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_130UndefinedArraySubscriptCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedArraySubscriptChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_126UndefinedAssignmentCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::UndefinedAssignmentChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114UnixAPICheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::UnixAPIChecker>"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_114VLASizeCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::VLASizeChecker>"},
    {"_ZTVN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES4_EEE", "vtable for llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> >"},
    {"_ZTVN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValES5_EEEEE", "vtable for llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, clang::ento::SVal> > >"},
    {"_ZTVN4llvm21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEE", "vtable for llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> >"},
    {"_ZTVN4llvm10FoldingSetINS_21FoldingSetNodeWrapperISt4pairIN5clang4ento4SValEmEEEEE", "vtable for llvm::FoldingSet<llvm::FoldingSetNodeWrapper<std::pair<clang::ento::SVal, unsigned long> > >"},
    {"_ZTVN4llvm21FoldingSetNodeWrapperINS_6APSIntEEE", "vtable for llvm::FoldingSetNodeWrapper<llvm::APSInt>"},
    {"_ZTVN4llvm10FoldingSetINS_17ImmutableListImplIN5clang4ento4SValEEEEE", "vtable for llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SVal> >"},
    {"_ZTVN4llvm10FoldingSetINS_21FoldingSetNodeWrapperINS_6APSIntEEEEE", "vtable for llvm::FoldingSet<llvm::FoldingSetNodeWrapper<llvm::APSInt> >"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento15CompoundValDataEEE", "vtable for llvm::FoldingSet<clang::ento::CompoundValData>"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento19LazyCompoundValDataEEE", "vtable for llvm::FoldingSet<clang::ento::LazyCompoundValData>"},
    {"_ZTVN4llvm10FoldingSetINS_17ImmutableListImplIPN5clang4ento18BugReporterVisitorEEEEE", "vtable for llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::BugReporterVisitor*> >"},
    {"_ZTVN5clang4ento13DiagBugReportE", "vtable for clang::ento::DiagBugReport"},
    {"_ZTVN4llvm10FoldingSetIN12_GLOBAL__N_113DiagCacheItemEEE", "vtable for llvm::FoldingSet<(anonymous namespace)::DiagCacheItem>"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento18BugReporterVisitorEEE", "vtable for llvm::FoldingSet<clang::ento::BugReporterVisitor>"},
    {"_ZTVN4llvm10FoldingSetINS_17ImmutableListImplIPKN5clang4ento10SymbolDataEEEEE", "vtable for llvm::FoldingSet<llvm::ImmutableListImpl<clang::ento::SymbolData const*> >"},
    {"_ZTVN5clang4ento14CheckerVisitorIN12_GLOBAL__N_120RetainReleaseCheckerEEE", "vtable for clang::ento::CheckerVisitor<(anonymous namespace)::RetainReleaseChecker>"},
    {"_ZTVN5clang4ento13TransferFuncsE", "vtable for clang::ento::TransferFuncs"},
    {"_ZTVN5clang4ento21InterExplodedGraphMapE", "vtable for clang::ento::InterExplodedGraphMap"},
    {"_ZTVN5clang4ento18UnknownSpaceRegionE", "vtable for clang::ento::UnknownSpaceRegion"},
    {"_ZTVN5clang4ento15HeapSpaceRegionE", "vtable for clang::ento::HeapSpaceRegion"},
    {"_ZTVN5clang4ento18GlobalsSpaceRegionE", "vtable for clang::ento::GlobalsSpaceRegion"},
    {"_ZTVN5clang4ento25StackArgumentsSpaceRegionE", "vtable for clang::ento::StackArgumentsSpaceRegion"},
    {"_ZTVN5clang4ento22StackLocalsSpaceRegionE", "vtable for clang::ento::StackLocalsSpaceRegion"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento9MemRegionEEE", "vtable for llvm::FoldingSet<clang::ento::MemRegion>"},
    {"_ZTVN4llvm10FoldingSetIN5clang4ento7SymExprEEE", "vtable for llvm::FoldingSet<clang::ento::SymExpr>"},
    {"_ZTVN5clang24LiveVariables_ValueTypes14AnalysisDataTyE", "vtable for clang::LiveVariables_ValueTypes::AnalysisDataTy"},
    {"_ZTVN5clang23StmtDeclBitVector_Types14AnalysisDataTyE", "vtable for clang::StmtDeclBitVector_Types::AnalysisDataTy"},
    {"_ZTVN5clang19DeclBitVector_Types14AnalysisDataTyE", "vtable for clang::DeclBitVector_Types::AnalysisDataTy"},
    {"_ZTVN4llvm14format_object1IjEE", "vtable for llvm::format_object1<unsigned int>"},
    {"_ZTVN5clang30UninitializedValues_ValueTypes14AnalysisDataTyE", "vtable for clang::UninitializedValues_ValueTypes::AnalysisDataTy"},
    {"_ZTVN4llvm10FoldingSetIN5clang3idx10EntityImplEEE", "vtable for llvm::FoldingSet<clang::idx::EntityImpl>"},
    {"_ZTVN4llvm10FoldingSetIN5clang10ASTContext29CanonicalTemplateTemplateParmEEE", "vtable for llvm::FoldingSet<clang::ASTContext::CanonicalTemplateTemplateParm>"},
    {"_ZTVN4llvm10FoldingSetIN5clang19NestedNameSpecifierEEE", "vtable for llvm::FoldingSet<clang::NestedNameSpecifier>"},
    {"_ZTVN4llvm10FoldingSetIN5clang36SubstTemplateTemplateParmPackStorageEEE", "vtable for llvm::FoldingSet<clang::SubstTemplateTemplateParmPackStorage>"},
    {"_ZTVN4llvm10FoldingSetIN5clang21DependentTemplateNameEEE", "vtable for llvm::FoldingSet<clang::DependentTemplateName>"},
    {"_ZTVN4llvm10FoldingSetIN5clang21QualifiedTemplateNameEEE", "vtable for llvm::FoldingSet<clang::QualifiedTemplateName>"},
    {"_ZTVN4llvm10FoldingSetIN5clang14AttributedTypeEEE", "vtable for llvm::FoldingSet<clang::AttributedType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang21ObjCObjectPointerTypeEEE", "vtable for llvm::FoldingSet<clang::ObjCObjectPointerType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang18ObjCObjectTypeImplEEE", "vtable for llvm::FoldingSet<clang::ObjCObjectTypeImpl>"},
    {"_ZTVN4llvm10FoldingSetIN5clang17PackExpansionTypeEEE", "vtable for llvm::FoldingSet<clang::PackExpansionType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang17DependentNameTypeEEE", "vtable for llvm::FoldingSet<clang::DependentNameType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang14ElaboratedTypeEEE", "vtable for llvm::FoldingSet<clang::ElaboratedType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang9ParenTypeEEE", "vtable for llvm::FoldingSet<clang::ParenType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang29SubstTemplateTypeParmPackTypeEEE", "vtable for llvm::FoldingSet<clang::SubstTemplateTypeParmPackType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang25SubstTemplateTypeParmTypeEEE", "vtable for llvm::FoldingSet<clang::SubstTemplateTypeParmType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang20TemplateTypeParmTypeEEE", "vtable for llvm::FoldingSet<clang::TemplateTypeParmType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang21DependentDecltypeTypeEEE", "vtable for llvm::FoldingSet<clang::DependentDecltypeType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang23DependentTypeOfExprTypeEEE", "vtable for llvm::FoldingSet<clang::DependentTypeOfExprType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang17FunctionProtoTypeEEE", "vtable for llvm::FoldingSet<clang::FunctionProtoType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang19FunctionNoProtoTypeEEE", "vtable for llvm::FoldingSet<clang::FunctionNoProtoType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang10VectorTypeEEE", "vtable for llvm::FoldingSet<clang::VectorType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang27DependentSizedExtVectorTypeEEE", "vtable for llvm::FoldingSet<clang::DependentSizedExtVectorType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang23DependentSizedArrayTypeEEE", "vtable for llvm::FoldingSet<clang::DependentSizedArrayType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang19IncompleteArrayTypeEEE", "vtable for llvm::FoldingSet<clang::IncompleteArrayType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang17ConstantArrayTypeEEE", "vtable for llvm::FoldingSet<clang::ConstantArrayType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang17MemberPointerTypeEEE", "vtable for llvm::FoldingSet<clang::MemberPointerType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang19RValueReferenceTypeEEE", "vtable for llvm::FoldingSet<clang::RValueReferenceType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang19LValueReferenceTypeEEE", "vtable for llvm::FoldingSet<clang::LValueReferenceType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang16BlockPointerTypeEEE", "vtable for llvm::FoldingSet<clang::BlockPointerType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang11PointerTypeEEE", "vtable for llvm::FoldingSet<clang::PointerType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang11ComplexTypeEEE", "vtable for llvm::FoldingSet<clang::ComplexType>"},
    {"_ZTVN4llvm10FoldingSetIN5clang8ExtQualsEEE", "vtable for llvm::FoldingSet<clang::ExtQuals>"},
    {"_ZTVN4llvm20ContextualFoldingSetIN5clang35DependentTemplateSpecializationTypeERNS1_10ASTContextEEE", "vtable for llvm::ContextualFoldingSet<clang::DependentTemplateSpecializationType, clang::ASTContext&>"},
    {"_ZTVN4llvm20ContextualFoldingSetIN5clang26TemplateSpecializationTypeERNS1_10ASTContextEEE", "vtable for llvm::ContextualFoldingSet<clang::TemplateSpecializationType, clang::ASTContext&>"},
    {"_ZTVN5clang8TypeDeclE", "vtable for clang::TypeDecl"},
    {"_ZTVN5clang9NamedDeclE", "vtable for clang::NamedDecl"},
    {"_ZTVN5clang9ValueDeclE", "vtable for clang::ValueDecl"},
    {"_ZTVN5clang14DeclaratorDeclE", "vtable for clang::DeclaratorDecl"},
    {"_ZTVN5clang16FileScopeAsmDeclE", "vtable for clang::FileScopeAsmDecl"},
    {"_ZTVN5clang11TypedefDeclE", "vtable for clang::TypedefDecl"},
    {"_ZTVN5clang17IndirectFieldDeclE", "vtable for clang::IndirectFieldDecl"},
    {"_ZTVN5clang17ImplicitParamDeclE", "vtable for clang::ImplicitParamDecl"},
    {"_ZTVN5clang13NamespaceDeclE", "vtable for clang::NamespaceDecl"},
    {"_ZTVN5clang19TranslationUnitDeclE", "vtable for clang::TranslationUnitDecl"},
    {"_ZTVN5clang8EnumDeclE", "vtable for clang::EnumDecl"},
    {"_ZTVN5clang9FieldDeclE", "vtable for clang::FieldDecl"},
    {"_ZTVN5clang11ParmVarDeclE", "vtable for clang::ParmVarDecl"},
    {"_ZTVN5clang16StaticAssertDeclE", "vtable for clang::StaticAssertDecl"},
    {"_ZTVN5clang27UnresolvedUsingTypenameDeclE", "vtable for clang::UnresolvedUsingTypenameDecl"},
    {"_ZTVN5clang24UnresolvedUsingValueDeclE", "vtable for clang::UnresolvedUsingValueDecl"},
    {"_ZTVN5clang9UsingDeclE", "vtable for clang::UsingDecl"},
    {"_ZTVN5clang18NamespaceAliasDeclE", "vtable for clang::NamespaceAliasDecl"},
    {"_ZTVN5clang18UsingDirectiveDeclE", "vtable for clang::UsingDirectiveDecl"},
    {"_ZTVN5clang15LinkageSpecDeclE", "vtable for clang::LinkageSpecDecl"},
    {"_ZTVN5clang17CXXConversionDeclE", "vtable for clang::CXXConversionDecl"},
    {"_ZTVN5clang13CXXMethodDeclE", "vtable for clang::CXXMethodDecl"},
    {"_ZTVN5clang17CXXDestructorDeclE", "vtable for clang::CXXDestructorDecl"},
    {"_ZTVN5clang18CXXConstructorDeclE", "vtable for clang::CXXConstructorDecl"},
    {"_ZTVN5clang10FriendDeclE", "vtable for clang::FriendDecl"},
    {"_ZTVN5clang17ObjCInterfaceDeclE", "vtable for clang::ObjCInterfaceDecl"},
    {"_ZTVN5clang23ObjCForwardProtocolDeclE", "vtable for clang::ObjCForwardProtocolDecl"},
    {"_ZTVN5clang17ObjCContainerDeclE", "vtable for clang::ObjCContainerDecl"},
    {"_ZTVN5clang16ObjCPropertyDeclE", "vtable for clang::ObjCPropertyDecl"},
    {"_ZTVN5clang23ObjCCompatibleAliasDeclE", "vtable for clang::ObjCCompatibleAliasDecl"},
    {"_ZTVN5clang22ObjCImplementationDeclE", "vtable for clang::ObjCImplementationDecl"},
    {"_ZTVN5clang12ObjCImplDeclE", "vtable for clang::ObjCImplDecl"},
    {"_ZTVN5clang20ObjCCategoryImplDeclE", "vtable for clang::ObjCCategoryImplDecl"},
    {"_ZTVN5clang16ObjCCategoryDeclE", "vtable for clang::ObjCCategoryDecl"},
    {"_ZTVN5clang16ObjCProtocolDeclE", "vtable for clang::ObjCProtocolDecl"},
    {"_ZTVN5clang19ObjCAtDefsFieldDeclE", "vtable for clang::ObjCAtDefsFieldDecl"},
    {"_ZTVN5clang12ObjCIvarDeclE", "vtable for clang::ObjCIvarDecl"},
    {"_ZTVN5clang12TemplateDeclE", "vtable for clang::TemplateDecl"},
    {"_ZTVN5clang18FriendTemplateDeclE", "vtable for clang::FriendTemplateDecl"},
    {"_ZTVN5clang38ClassTemplatePartialSpecializationDeclE", "vtable for clang::ClassTemplatePartialSpecializationDecl"},
    {"_ZTVN5clang24TemplateTemplateParmDeclE", "vtable for clang::TemplateTemplateParmDecl"},
    {"_ZTVN5clang20TemplateTypeParmDeclE", "vtable for clang::TemplateTypeParmDecl"},
    {"_ZTVN4llvm10FoldingSetIN5clang38ClassTemplatePartialSpecializationDeclEEE", "vtable for llvm::FoldingSet<clang::ClassTemplatePartialSpecializationDecl>"},
    {"_ZTVN4llvm10FoldingSetIN5clang31ClassTemplateSpecializationDeclEEE", "vtable for llvm::FoldingSet<clang::ClassTemplateSpecializationDecl>"},
    {"_ZTVN4llvm10FoldingSetIN5clang34FunctionTemplateSpecializationInfoEEE", "vtable for llvm::FoldingSet<clang::FunctionTemplateSpecializationInfo>"},
    {"_ZTVN4llvm10FoldingSetIN5clang24CXXLiteralOperatorIdNameEEE", "vtable for llvm::FoldingSet<clang::CXXLiteralOperatorIdName>"},
    {"_ZTVN4llvm10FoldingSetIN5clang14CXXSpecialNameEEE", "vtable for llvm::FoldingSet<clang::CXXSpecialName>"},
    {"_ZTVN5clang18PPChainedCallbacksE", "vtable for clang::PPChainedCallbacks"},
    {"_ZTVN4llvm10FoldingSetIN5clang20MultiKeywordSelectorEEE", "vtable for llvm::FoldingSet<clang::MultiKeywordSelector>"},
    {"_ZN12_GLOBAL__N_113PPCTargetInfo11BuiltinInfoE", "(anonymous namespace)::PPCTargetInfo::BuiltinInfo"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo11BuiltinInfoE", "(anonymous namespace)::ARMTargetInfo::BuiltinInfo"},
    {"_ZN12_GLOBAL__N_113PPCTargetInfo13GCCRegAliasesE", "(anonymous namespace)::PPCTargetInfo::GCCRegAliases"},
    {"_ZN12_GLOBAL__N_117SparcV8TargetInfo13GCCRegAliasesE", "(anonymous namespace)::SparcV8TargetInfo::GCCRegAliases"},
    {"_ZN12_GLOBAL__N_116MBlazeTargetInfo13GCCRegAliasesE", "(anonymous namespace)::MBlazeTargetInfo::GCCRegAliases"},
    {"_ZN12_GLOBAL__N_114MipsTargetInfo13GCCRegAliasesE", "(anonymous namespace)::MipsTargetInfo::GCCRegAliases"},
    {"_ZN12_GLOBAL__N_113ARMTargetInfo13GCCRegAliasesE", "(anonymous namespace)::ARMTargetInfo::GCCRegAliases"},
    {"_ZTVN4llvm18ValueMapCallbackVHIPKNS_8FunctionENS_18InlineCostAnalyzer12FunctionInfoENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEEE", "vtable for llvm::ValueMapCallbackVH<llvm::Function const*, llvm::InlineCostAnalyzer::FunctionInfo, llvm::ValueMapConfig<llvm::Function const*>, llvm::DenseMapInfo<llvm::InlineCostAnalyzer::FunctionInfo> >"},
    {"_ZTVN4llvm12DISubprogramE", "vtable for llvm::DISubprogram"},
    {"_ZTVN4llvm7DIScopeE", "vtable for llvm::DIScope"},
    {"_ZTVN4llvm22X86MachineFunctionInfoE", "vtable for llvm::X86MachineFunctionInfo"},
    {"_ZTVN4llvm18X86MCAsmInfoDarwinE", "vtable for llvm::X86MCAsmInfoDarwin"},
    {"_ZTVN4llvm16X86MCAsmInfoCOFFE", "vtable for llvm::X86MCAsmInfoCOFF"},
    {"_ZTVN4llvm15MCAsmInfoDarwinE", "vtable for llvm::MCAsmInfoDarwin"},
    {"_ZTVN4llvm13MCAsmInfoCOFFE", "vtable for llvm::MCAsmInfoCOFF"},
    {"_ZTVN4llvm2cl3optI17AsmWriterFlavorTyLb0ENS0_6parserIS2_EEEE", "vtable for llvm::cl::opt<AsmWriterFlavorTy, false, llvm::cl::parser<AsmWriterFlavorTy> >"},
    {"_ZTVN4llvm2cl6parserI17AsmWriterFlavorTyEE", "vtable for llvm::cl::parser<AsmWriterFlavorTy>"},
    {"_ZTVN4llvm3X8616CONTROL_REGClassE", "vtable for llvm::X86::CONTROL_REGClass"},
    {"_ZTVN4llvm3X8614DEBUG_REGClassE", "vtable for llvm::X86::DEBUG_REGClass"},
    {"_ZTVN4llvm3X8614GR16_ABCDClassE", "vtable for llvm::X86::GR16_ABCDClass"},
    {"_ZTVN4llvm3X8614GR32_ABCDClassE", "vtable for llvm::X86::GR32_ABCDClass"},
    {"_ZTVN4llvm3X8612GR32_ADClassE", "vtable for llvm::X86::GR32_ADClass"},
    {"_ZTVN4llvm3X8612GR32_TCClassE", "vtable for llvm::X86::GR32_TCClass"},
    {"_ZTVN4llvm3X8614GR64_ABCDClassE", "vtable for llvm::X86::GR64_ABCDClass"},
    {"_ZTVN4llvm3X8612GR64_TCClassE", "vtable for llvm::X86::GR64_TCClass"},
    {"_ZTVN4llvm3X8615GR64_TCW64ClassE", "vtable for llvm::X86::GR64_TCW64Class"},
    {"_ZTVN4llvm3X8615GR8_ABCD_HClassE", "vtable for llvm::X86::GR8_ABCD_HClass"},
    {"_ZTVN4llvm3X8615GR8_ABCD_LClassE", "vtable for llvm::X86::GR8_ABCD_LClass"},
    {"_ZTVN4llvm3X8610RFP32ClassE", "vtable for llvm::X86::RFP32Class"},
    {"_ZTVN4llvm3X8610RFP64ClassE", "vtable for llvm::X86::RFP64Class"},
    {"_ZTVN4llvm3X8610RFP80ClassE", "vtable for llvm::X86::RFP80Class"},
    {"_ZTVN4llvm3X8616SEGMENT_REGClassE", "vtable for llvm::X86::SEGMENT_REGClass"},
    {"_ZTVN4llvm3X869VR64ClassE", "vtable for llvm::X86::VR64Class"},
    {"_ZTVN4llvm19X86_32TargetMachineE", "vtable for llvm::X86_32TargetMachine"},
    {"_ZTVN4llvm19X86_64TargetMachineE", "vtable for llvm::X86_64TargetMachine"},
    {"_ZN4llvm15TheX86_32TargetE", "llvm::TheX86_32Target"},
    {"_ZN4llvm15TheX86_64TargetE", "llvm::TheX86_64Target"},
    {"_ZTVN4llvm15ARMFunctionInfoE", "vtable for llvm::ARMFunctionInfo"},
    {"_ZTVN4llvm3ARM8CCRClassE", "vtable for llvm::ARM::CCRClass"},
    {"_ZTVN4llvm3ARM10DPR_8ClassE", "vtable for llvm::ARM::DPR_8Class"},
    {"_ZTVN4llvm3ARM13DPR_VFP2ClassE", "vtable for llvm::ARM::DPR_VFP2Class"},
    {"_ZTVN4llvm3ARM10QPR_8ClassE", "vtable for llvm::ARM::QPR_8Class"},
    {"_ZTVN4llvm3ARM13QPR_VFP2ClassE", "vtable for llvm::ARM::QPR_VFP2Class"},
    {"_ZTVN4llvm3ARM14QQPR_VFP2ClassE", "vtable for llvm::ARM::QQPR_VFP2Class"},
    {"_ZTVN4llvm3ARM8SPRClassE", "vtable for llvm::ARM::SPRClass"},
    {"_ZTVN4llvm3ARM10SPR_8ClassE", "vtable for llvm::ARM::SPR_8Class"},
    {"_ZTVN4llvm3ARM9tGPRClassE", "vtable for llvm::ARM::tGPRClass"},
    {"_ZTVN4llvm19TargetRegisterClassE", "vtable for llvm::TargetRegisterClass"},
    {"_ZTVN4llvm16SelectionDAGISel11ISelUpdaterE", "vtable for llvm::SelectionDAGISel::ISelUpdater"},
    {"_ZTVN4llvm18ARMMCAsmInfoDarwinE", "vtable for llvm::ARMMCAsmInfoDarwin"},
    {"_ZTVN4llvm15ARMELFMCAsmInfoE", "vtable for llvm::ARMELFMCAsmInfo"},
    {"_ZTVN4llvm16ARMTargetMachineE", "vtable for llvm::ARMTargetMachine"},
    {"_ZTVN4llvm18ThumbTargetMachineE", "vtable for llvm::ThumbTargetMachine"},
    {"_ZTVN4llvm15ARMRegisterInfoE", "vtable for llvm::ARMRegisterInfo"},
    {"_ZTVN4llvm10FoldingSetINS_6SDNodeEEE", "vtable for llvm::FoldingSet<llvm::SDNode>"},
    {"_ZTVN4llvm2cl3optIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEELb0ENS_18RegisterPassParserINS_17RegisterSchedulerEEEEE", "vtable for llvm::cl::opt<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level), false, llvm::RegisterPassParser<llvm::RegisterScheduler> >"},
    {"_ZTVN4llvm18RegisterPassParserINS_17RegisterSchedulerEEE", "vtable for llvm::RegisterPassParser<llvm::RegisterScheduler>"},
    {"_ZTVN4llvm2cl6parserIPFPNS_18ScheduleDAGSDNodesEPNS_16SelectionDAGISelENS_10CodeGenOpt5LevelEEEE", "vtable for llvm::cl::parser<llvm::ScheduleDAGSDNodes* (*)(llvm::SelectionDAGISel*, llvm::CodeGenOpt::Level)>"},
    {"_ZN4llvm8RegistryINS_17GCMetadataPrinterENS_14RegistryTraitsIS1_EEE4HeadE", "llvm::Registry<llvm::GCMetadataPrinter, llvm::RegistryTraits<llvm::GCMetadataPrinter> >::Head"},
    {"_ZTVN4llvm14format_object1IyEE", "vtable for llvm::format_object1<unsigned long long>"},
    {"_ZTVN4llvm10FoldingSetINS_9DIEAbbrevEEE", "vtable for llvm::FoldingSet<llvm::DIEAbbrev>"},
    {"_ZTVN4llvm9DIEAbbrevE", "vtable for llvm::DIEAbbrev"},
    {"_ZN4llvm8RegistryINS_10GCStrategyENS_14RegistryTraitsIS1_EEE4HeadE", "llvm::Registry<llvm::GCStrategy, llvm::RegistryTraits<llvm::GCStrategy> >::Head"},
    {"_ZTVN4llvm2cl3optINS0_13boolOrDefaultELb0ENS0_6parserIS2_EEEE", "vtable for llvm::cl::opt<llvm::cl::boolOrDefault, false, llvm::cl::parser<llvm::cl::boolOrDefault> >"},
    {"_ZTVN4llvm17DominatorTreeBaseINS_17MachineBasicBlockEEE", "vtable for llvm::DominatorTreeBase<llvm::MachineBasicBlock>"},
    {"_ZTVN4llvm2cl3optIPFPNS_12FunctionPassEvELb0ENS_18RegisterPassParserINS_16RegisterRegAllocEEEEE", "vtable for llvm::cl::opt<llvm::FunctionPass* (*)(), false, llvm::RegisterPassParser<llvm::RegisterRegAlloc> >"},
    {"_ZTVN4llvm18RegisterPassParserINS_16RegisterRegAllocEEE", "vtable for llvm::RegisterPassParser<llvm::RegisterRegAlloc>"},
    {"_ZTVN4llvm2cl6parserIPFPNS_12FunctionPassEvEEE", "vtable for llvm::cl::parser<llvm::FunctionPass* (*)()>"},
    {"_ZTVN4llvm2cl3optI20ShrinkWrapDebugLevelLb0ENS0_6parserIS2_EEEE", "vtable for llvm::cl::opt<ShrinkWrapDebugLevel, false, llvm::cl::parser<ShrinkWrapDebugLevel> >"},
    {"_ZTVN4llvm2cl6parserI20ShrinkWrapDebugLevelEE", "vtable for llvm::cl::parser<ShrinkWrapDebugLevel>"},
    {"_ZTVN4llvm2cl3optIN12_GLOBAL__N_111SpillerNameELb0ENS0_6parserIS3_EEEE", "vtable for llvm::cl::opt<(anonymous namespace)::SpillerName, false, llvm::cl::parser<(anonymous namespace)::SpillerName> >"},
    {"_ZTVN4llvm2cl6parserIN12_GLOBAL__N_111SpillerNameEEE", "vtable for llvm::cl::parser<(anonymous namespace)::SpillerName>"},
    {"_ZTVN4llvm2cl3optIN12_GLOBAL__N_112RewriterNameELb0ENS0_6parserIS3_EEEE", "vtable for llvm::cl::opt<(anonymous namespace)::RewriterName, false, llvm::cl::parser<(anonymous namespace)::RewriterName> >"},
    {"_ZTVN4llvm2cl6parserIN12_GLOBAL__N_112RewriterNameEEE", "vtable for llvm::cl::parser<(anonymous namespace)::RewriterName>"},
    {"_ZTVN4llvm18ValueMapCallbackVHIPKNS_5ValueENS_10TrackingVHIS1_EENS_14ValueMapConfigIS3_EENS_12DenseMapInfoIS5_EEEE", "vtable for llvm::ValueMapCallbackVH<llvm::Value const*, llvm::TrackingVH<llvm::Value>, llvm::ValueMapConfig<llvm::Value const*>, llvm::DenseMapInfo<llvm::TrackingVH<llvm::Value> > >"},
    {"_ZTVN4llvm20LoadAndStorePromoterE", "vtable for llvm::LoadAndStorePromoter"},
    {"_ZTVN4llvm6DITypeE", "vtable for llvm::DIType"},
    {"_ZTVN4llvm15DICompositeTypeE", "vtable for llvm::DICompositeType"},
    {"_ZTVN4llvm13DIDerivedTypeE", "vtable for llvm::DIDerivedType"},
    {"_ZTVN4llvm11DIBasicTypeE", "vtable for llvm::DIBasicType"},
    {"_ZTVN4llvm6DIFileE", "vtable for llvm::DIFile"},
    {"_ZTVN4llvm11DINameSpaceE", "vtable for llvm::DINameSpace"},
    {"_ZTVN4llvm13DICompileUnitE", "vtable for llvm::DICompileUnit"},
    {"_ZTVN4llvm14DILexicalBlockE", "vtable for llvm::DILexicalBlock"},
    {"_ZTVN4llvm10FoldingSetINS_4SCEVEEE", "vtable for llvm::FoldingSet<llvm::SCEV>"},
    {"_ZTVN4llvm2cl3optIbLb1ENS0_6parserIbEEEE", "vtable for llvm::cl::opt<bool, true, llvm::cl::parser<bool> >"},
    {"_ZTVN4llvm2cl3optIjLb1ENS0_6parserIjEEEE", "vtable for llvm::cl::opt<unsigned int, true, llvm::cl::parser<unsigned int> >"},
    {"_ZTVN4llvm2cl3optINS_9CodeModel5ModelELb1ENS0_6parserIS3_EEEE", "vtable for llvm::cl::opt<llvm::CodeModel::Model, true, llvm::cl::parser<llvm::CodeModel::Model> >"},
    {"_ZTVN4llvm2cl6parserINS_9CodeModel5ModelEEE", "vtable for llvm::cl::parser<llvm::CodeModel::Model>"},
    {"_ZTVN4llvm2cl3optINS_5Reloc5ModelELb1ENS0_6parserIS3_EEEE", "vtable for llvm::cl::opt<llvm::Reloc::Model, true, llvm::cl::parser<llvm::Reloc::Model> >"},
    {"_ZTVN4llvm2cl6parserINS_5Reloc5ModelEEE", "vtable for llvm::cl::parser<llvm::Reloc::Model>"},
    {"_ZTVN4llvm2cl3optINS_8FloatABI7ABITypeELb1ENS0_6parserIS3_EEEE", "vtable for llvm::cl::opt<llvm::FloatABI::ABIType, true, llvm::cl::parser<llvm::FloatABI::ABIType> >"},
    {"_ZTVN4llvm2cl6parserINS_8FloatABI7ABITypeEEE", "vtable for llvm::cl::parser<llvm::FloatABI::ABIType>"},
    {"_ZTVN4llvm10FoldingSetINS_17AttributeListImplEEE", "vtable for llvm::FoldingSet<llvm::AttributeListImpl>"},
    {"_ZTVN4llvm11ConstantIntE", "vtable for llvm::ConstantInt"},
    {"_ZTVN4llvm25GetElementPtrConstantExprE", "vtable for llvm::GetElementPtrConstantExpr"},
    {"_ZTVN4llvm4UserE", "vtable for llvm::User"},
    {"_ZTVN4llvm19CompareConstantExprE", "vtable for llvm::CompareConstantExpr"},
    {"_ZTVN4llvm24ExtractValueConstantExprE", "vtable for llvm::ExtractValueConstantExpr"},
    {"_ZTVN4llvm23InsertValueConstantExprE", "vtable for llvm::InsertValueConstantExpr"},
    {"_ZTVN4llvm25ShuffleVectorConstantExprE", "vtable for llvm::ShuffleVectorConstantExpr"},
    {"_ZTVN4llvm25InsertElementConstantExprE", "vtable for llvm::InsertElementConstantExpr"},
    {"_ZTVN4llvm26ExtractElementConstantExprE", "vtable for llvm::ExtractElementConstantExpr"},
    {"_ZTVN4llvm18SelectConstantExprE", "vtable for llvm::SelectConstantExpr"},
    {"_ZTVN4llvm18BinaryConstantExprE", "vtable for llvm::BinaryConstantExpr"},
    {"_ZTVN4llvm17UnaryConstantExprE", "vtable for llvm::UnaryConstantExpr"},
    {"_ZTVN4llvm17DominatorTreeBaseINS_10BasicBlockEEE", "vtable for llvm::DominatorTreeBase<llvm::BasicBlock>"},
    {"_ZTVN4llvm8ArgumentE", "vtable for llvm::Argument"},
    {"_ZTVN4llvm10FoldingSetINS_6MDNodeEEE", "vtable for llvm::FoldingSet<llvm::MDNode>"},
    {"_ZTVN4llvm17ConstantUniqueMapIcNS_4TypeENS_21ConstantAggregateZeroELb0EEE", "vtable for llvm::ConstantUniqueMap<char, llvm::Type, llvm::ConstantAggregateZero, false>"},
    {"_ZTVN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_9ArrayTypeENS_13ConstantArrayELb1EEE", "vtable for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::ArrayType, llvm::ConstantArray, true>"},
    {"_ZTVN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10StructTypeENS_14ConstantStructELb1EEE", "vtable for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::StructType, llvm::ConstantStruct, true>"},
    {"_ZTVN4llvm17ConstantUniqueMapISt6vectorIPNS_8ConstantESaIS3_EENS_10VectorTypeENS_14ConstantVectorELb0EEE", "vtable for llvm::ConstantUniqueMap<std::vector<llvm::Constant*, std::allocator<llvm::Constant*> >, llvm::VectorType, llvm::ConstantVector, false>"},
    {"_ZTVN4llvm17ConstantUniqueMapIcNS_11PointerTypeENS_19ConstantPointerNullELb0EEE", "vtable for llvm::ConstantUniqueMap<char, llvm::PointerType, llvm::ConstantPointerNull, false>"},
    {"_ZTVN4llvm17ConstantUniqueMapIcNS_4TypeENS_10UndefValueELb0EEE", "vtable for llvm::ConstantUniqueMap<char, llvm::Type, llvm::UndefValue, false>"},
    {"_ZTVN4llvm17ConstantUniqueMapINS_14ExprMapKeyTypeENS_4TypeENS_12ConstantExprELb0EEE", "vtable for llvm::ConstantUniqueMap<llvm::ExprMapKeyType, llvm::Type, llvm::ConstantExpr, false>"},
    {"_ZTVN4llvm17ConstantUniqueMapINS_16InlineAsmKeyTypeENS_11PointerTypeENS_9InlineAsmELb0EEE", "vtable for llvm::ConstantUniqueMap<llvm::InlineAsmKeyType, llvm::PointerType, llvm::InlineAsm, false>"},
    {"_ZTVN4llvm11IntegerTypeE", "vtable for llvm::IntegerType"},
    {"_ZTVN4llvm11DerivedTypeE", "vtable for llvm::DerivedType"},
    {"_ZTVN4llvm8MDStringE", "vtable for llvm::MDString"},
    {"_ZTVN4llvm2cl6parserIPKNS_8PassInfoEEE", "vtable for llvm::cl::parser<llvm::PassInfo const*>"},
    {"_ZTVN4llvm2cl4listIPKNS_8PassInfoEbNS_14PassNameParserEEE", "vtable for llvm::cl::list<llvm::PassInfo const*, bool, llvm::PassNameParser>"},
    {"_ZTVN4llvm2cl3optINS_14PassDebugLevelELb0ENS0_6parserIS2_EEEE", "vtable for llvm::cl::opt<llvm::PassDebugLevel, false, llvm::cl::parser<llvm::PassDebugLevel> >"},
    {"_ZTVN4llvm2cl6parserINS_14PassDebugLevelEEE", "vtable for llvm::cl::parser<llvm::PassDebugLevel>"},
    {"_ZTVN4llvm15PassManagerImplE", "vtable for llvm::PassManagerImpl"},
    {"_ZTVN4llvm23FunctionPassManagerImplE", "vtable for llvm::FunctionPassManagerImpl"},
    {"_ZTVN4llvm10OpaqueTypeE", "vtable for llvm::OpaqueType"},
    {"_ZN4llvm12TheARMTargetE", "llvm::TheARMTarget"},
    {"_ZN4llvm14TheThumbTargetE", "llvm::TheThumbTarget"},
    {"_ZTVN4llvm14format_object1IhEE", "vtable for llvm::format_object1<unsigned char>"},
    {"_ZTVN4llvm15MCAlignFragmentE", "vtable for llvm::MCAlignFragment"},
    {"_ZTVN4llvm14MCFillFragmentE", "vtable for llvm::MCFillFragment"},
    {"_ZTVN4llvm14MCDataFragmentE", "vtable for llvm::MCDataFragment"},
    {"_ZTVN4llvm13MCOrgFragmentE", "vtable for llvm::MCOrgFragment"},
    {"_ZTVN4llvm24MCDwarfCallFrameFragmentE", "vtable for llvm::MCDwarfCallFrameFragment"},
    {"_ZTVN4llvm23MCDwarfLineAddrFragmentE", "vtable for llvm::MCDwarfLineAddrFragment"},
    {"_ZTVN4llvm13MCLEBFragmentE", "vtable for llvm::MCLEBFragment"},
    {"_ZN4llvm8MCSymbol21AbsolutePseudoSectionE", "llvm::MCSymbol::AbsolutePseudoSection"},
    {"_ZTVN4llvm14MCInstFragmentE", "vtable for llvm::MCInstFragment"},
    {"_ZN4llvm7APFloat8IEEEhalfE", "llvm::APFloat::IEEEhalf"},
    {"_ZN4llvm7APFloat10IEEEsingleE", "llvm::APFloat::IEEEsingle"},
    {"_ZN4llvm7APFloat10IEEEdoubleE", "llvm::APFloat::IEEEdouble"},
    {"_ZN4llvm7APFloat8IEEEquadE", "llvm::APFloat::IEEEquad"},
    {"_ZN4llvm7APFloat17x87DoubleExtendedE", "llvm::APFloat::x87DoubleExtended"},
    {"_ZN4llvm7APFloat5BogusE", "llvm::APFloat::Bogus"},
    {"_ZN4llvm7APFloat15PPCDoubleDoubleE", "llvm::APFloat::PPCDoubleDouble"},
    {"_ZTVN4llvm2cl12basic_parserIbEE", "vtable for llvm::cl::basic_parser<bool>"},
    {"_ZTVN4llvm2cl12basic_parserINS0_13boolOrDefaultEEE", "vtable for llvm::cl::basic_parser<llvm::cl::boolOrDefault>"},
    {"_ZTVN4llvm2cl12basic_parserIiEE", "vtable for llvm::cl::basic_parser<int>"},
    {"_ZTVN4llvm2cl12basic_parserIjEE", "vtable for llvm::cl::basic_parser<unsigned int>"},
    {"_ZTVN4llvm2cl12basic_parserIdEE", "vtable for llvm::cl::basic_parser<double>"},
    {"_ZTVN4llvm2cl12basic_parserIfEE", "vtable for llvm::cl::basic_parser<float>"},
    {"_ZTVN4llvm2cl12basic_parserISsEE", "vtable for llvm::cl::basic_parser<std::string>"},
    {"_ZTVN4llvm2cl12basic_parserIcEE", "vtable for llvm::cl::basic_parser<char>"},
    {"_ZTVN4llvm2cl3optIjLb0ENS0_6parserIjEEEE", "vtable for llvm::cl::opt<unsigned int, false, llvm::cl::parser<unsigned int> >"},
    {"_ZTVN4llvm2cl3optIiLb0ENS0_6parserIiEEEE", "vtable for llvm::cl::opt<int, false, llvm::cl::parser<int> >"},
    {"_ZTVN4llvm2cl3optISsLb0ENS0_6parserISsEEEE", "vtable for llvm::cl::opt<std::string, false, llvm::cl::parser<std::string> >"},
    {"_ZTVN4llvm2cl3optIcLb0ENS0_6parserIcEEEE", "vtable for llvm::cl::opt<char, false, llvm::cl::parser<char> >"},
    {"_ZTVN4llvm2cl3optIbLb0ENS0_6parserIbEEEE", "vtable for llvm::cl::opt<bool, false, llvm::cl::parser<bool> >"},
    {"_ZTVN4llvm2cl3optIN12_GLOBAL__N_114VersionPrinterELb1ENS0_6parserIbEEEE", "vtable for llvm::cl::opt<(anonymous namespace)::VersionPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTVN4llvm2cl3optIN12_GLOBAL__N_111HelpPrinterELb1ENS0_6parserIbEEEE", "vtable for llvm::cl::opt<(anonymous namespace)::HelpPrinter, true, llvm::cl::parser<bool> >"},
    {"_ZTVN4llvm3sys11ThreadLocalIKN12_GLOBAL__N_124CrashRecoveryContextImplEEE", "vtable for llvm::sys::ThreadLocal<(anonymous namespace)::CrashRecoveryContextImpl const>"},
    {"_ZTVN4llvm3sys11ThreadLocalIKNS_21PrettyStackTraceEntryEEE", "vtable for llvm::sys::ThreadLocal<llvm::PrettyStackTraceEntry const>"},
    {"_ZN4llvm3sys9TimeValue7MinTimeE", "llvm::sys::TimeValue::MinTime"},
    {"_ZN4llvm3sys9TimeValue7MaxTimeE", "llvm::sys::TimeValue::MaxTime"},
    {"_ZN4llvm3sys9TimeValue13PosixZeroTimeE", "llvm::sys::TimeValue::PosixZeroTime"},
    {"_ZN4llvm3sys9TimeValue13Win32ZeroTimeE", "llvm::sys::TimeValue::Win32ZeroTime"},
    {"_ZTVN4llvm14format_object1IxEE", "vtable for llvm::format_object1<long long>"},
    {"_ZTVN4llvm14format_object1IdEE", "vtable for llvm::format_object1<double>"},
    {"_ZTVN4llvm2cl3optISsLb1ENS0_6parserISsEEEE", "vtable for llvm::cl::opt<std::string, true, llvm::cl::parser<std::string> >"},
    {"_ZZN5clang12_GLOBAL__N_18getNewIdENS0_7tIdTypeEE8idCounts", "clang::(anonymous namespace)::getNewId(clang::(anonymous namespace)::tIdType)::idCounts"},
    {"_ZZN12_GLOBAL__N_126AdjustedReturnValueChecker6getTagEvE1x", "(anonymous namespace)::AdjustedReturnValueChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_120AnalyzerStatsChecker6getTagEvE1x", "(anonymous namespace)::AnalyzerStatsChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_117ArrayBoundChecker6getTagEvE1x", "(anonymous namespace)::ArrayBoundChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_119ArrayBoundCheckerV26getTagEvE1x", "(anonymous namespace)::ArrayBoundCheckerV2::getTag()::x"},
    {"_ZZN12_GLOBAL__N_118AttrNonNullChecker6getTagEvE1x", "(anonymous namespace)::AttrNonNullChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_119ClassReleaseChecker6getTagEvE1x", "(anonymous namespace)::ClassReleaseChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_122CFRetainReleaseChecker6getTagEvE1x", "(anonymous namespace)::CFRetainReleaseChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_121CFNumberCreateChecker6getTagEvE1x", "(anonymous namespace)::CFNumberCreateChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_113NilArgChecker6getTagEvE1x", "(anonymous namespace)::NilArgChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_122BuiltinFunctionChecker6getTagEvE3tag", "(anonymous namespace)::BuiltinFunctionChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_114CStringChecker6getTagEvE3tag", "(anonymous namespace)::CStringChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_121CallAndMessageChecker6getTagEvE1x", "(anonymous namespace)::CallAndMessageChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_115CastSizeChecker6getTagEvE1x", "(anonymous namespace)::CastSizeChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_119CastToStructChecker6getTagEvE1x", "(anonymous namespace)::CastToStructChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_113ChrootChecker6getTagEvE1x", "(anonymous namespace)::ChrootChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_118DereferenceChecker6getTagEvE3tag", "(anonymous namespace)::DereferenceChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_114DivZeroChecker6getTagEvE1x", "(anonymous namespace)::DivZeroChecker::getTag()::x"},
    {"_ZZN5clang4ento10ExprEngine23processCFGBlockEntranceERNS0_15ExplodedNodeSetERNS0_18GenericNodeBuilderINS_13BlockEntranceEEEE3tag", "clang::ento::ExprEngine::processCFGBlockEntrance(clang::ento::ExplodedNodeSet&, clang::ento::GenericNodeBuilder<clang::BlockEntrance>&)::tag"},
    {"_ZZN5clang4ento10ExprEngine8evalLoadERNS0_15ExplodedNodeSetEPKNS_4ExprEPNS0_12ExplodedNodeEPKNS0_7GRStateENS0_4SValEPKvNS_8QualTypeEE16loadReferenceTag", "clang::ento::ExprEngine::evalLoad(clang::ento::ExplodedNodeSet&, clang::Expr const*, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SVal, void const*, clang::QualType)::loadReferenceTag"},
    {"_ZZN5clang4ento10ExprEngine15VisitReturnStmtEPKNS_10ReturnStmtEPNS0_12ExplodedNodeERNS0_15ExplodedNodeSetEE3tag", "clang::ento::ExprEngine::VisitReturnStmt(clang::ReturnStmt const*, clang::ento::ExplodedNode*, clang::ento::ExplodedNodeSet&)::tag"},
    {"_ZN12_GLOBAL__N_110ReturnExpr6TagIntE", "(anonymous namespace)::ReturnExpr::TagInt"},
    {"_ZZN12_GLOBAL__N_119FixedAddressChecker6getTagEvE1x", "(anonymous namespace)::FixedAddressChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_126IdempotentOperationChecker6getTagEvE1x", "(anonymous namespace)::IdempotentOperationChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_116MacOSXAPIChecker6getTagEvE3tag", "(anonymous namespace)::MacOSXAPIChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_113MallocChecker6getTagEvE1x", "(anonymous namespace)::MallocChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_124NSAutoreleasePoolChecker6getTagEvE1x", "(anonymous namespace)::NSAutoreleasePoolChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_123NoReturnFunctionChecker6getTagEvE3tag", "(anonymous namespace)::NoReturnFunctionChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_115OSAtomicChecker6getTagEvE3tag", "(anonymous namespace)::OSAtomicChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_115OSAtomicChecker26evalOSAtomicCompareAndSwapERN5clang4ento14CheckerContextEPKNS1_8CallExprEE10magic_load", "(anonymous namespace)::OSAtomicChecker::evalOSAtomicCompareAndSwap(clang::ento::CheckerContext&, clang::CallExpr const*)::magic_load"},
    {"_ZZN12_GLOBAL__N_115OSAtomicChecker26evalOSAtomicCompareAndSwapERN5clang4ento14CheckerContextEPKNS1_8CallExprEE11magic_store", "(anonymous namespace)::OSAtomicChecker::evalOSAtomicCompareAndSwap(clang::ento::CheckerContext&, clang::CallExpr const*)::magic_store"},
    {"_ZZN12_GLOBAL__N_117ObjCAtSyncChecker6getTagEvE3tag", "(anonymous namespace)::ObjCAtSyncChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_119ObjCSelfInitChecker6getTagEvE3tag", "(anonymous namespace)::ObjCSelfInitChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_119PointerArithChecker6getTagEvE1x", "(anonymous namespace)::PointerArithChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_117PointerSubChecker6getTagEvE1x", "(anonymous namespace)::PointerSubChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_118PthreadLockChecker6getTagEvE1x", "(anonymous namespace)::PthreadLockChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_125ReturnPointerRangeChecker6getTagEvE1x", "(anonymous namespace)::ReturnPointerRangeChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_118ReturnUndefChecker6getTagEvE1x", "(anonymous namespace)::ReturnUndefChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_120StackAddrLeakChecker6getTagEvE1x", "(anonymous namespace)::StackAddrLeakChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_113StreamChecker6getTagEvE1x", "(anonymous namespace)::StreamChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_118UndefBranchChecker6getTagEvE1x", "(anonymous namespace)::UndefBranchChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_128UndefCapturedBlockVarChecker6getTagEvE3tag", "(anonymous namespace)::UndefCapturedBlockVarChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_118UndefResultChecker6getTagEvE3tag", "(anonymous namespace)::UndefResultChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_130UndefinedArraySubscriptChecker6getTagEvE1x", "(anonymous namespace)::UndefinedArraySubscriptChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_126UndefinedAssignmentChecker6getTagEvE1x", "(anonymous namespace)::UndefinedAssignmentChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_114UnixAPIChecker6getTagEvE3tag", "(anonymous namespace)::UnixAPIChecker::getTag()::tag"},
    {"_ZZN12_GLOBAL__N_122UnreachableCodeChecker6getTagEvE1x", "(anonymous namespace)::UnreachableCodeChecker::getTag()::x"},
    {"_ZZN12_GLOBAL__N_114VLASizeChecker6getTagEvE3tag", "(anonymous namespace)::VLASizeChecker::getTag()::tag"},
    {"_ZZ18IsCachedDiagnosticPN5clang4ento9BugReportEPNS0_14PathDiagnosticEE2DC", "IsCachedDiagnostic(clang::ento::BugReport*, clang::ento::PathDiagnostic*)::DC"},
    {"_ZGVZ18IsCachedDiagnosticPN5clang4ento9BugReportEPNS0_14PathDiagnosticEE2DC", "guard variable for IsCachedDiagnostic(clang::ento::BugReport*, clang::ento::PathDiagnostic*)::DC"},
    {"_ZZNK12_GLOBAL__N_118NilReceiverVisitor7ProfileERN4llvm16FoldingSetNodeIDEE1x", "(anonymous namespace)::NilReceiverVisitor::Profile(llvm::FoldingSetNodeID&) const::x"},
    {"_ZZNK12_GLOBAL__N_122FindLastStoreBRVisitor7ProfileERN4llvm16FoldingSetNodeIDEE3tag", "(anonymous namespace)::FindLastStoreBRVisitor::Profile(llvm::FoldingSetNodeID&) const::tag"},
    {"_ZZNK12_GLOBAL__N_124TrackConstraintBRVisitor7ProfileERN4llvm16FoldingSetNodeIDEE3tag", "(anonymous namespace)::TrackConstraintBRVisitor::Profile(llvm::FoldingSetNodeID&) const::tag"},
    {"_ZZN5clang4ento12GRStateTraitIN4llvm12ImmutableMapIPKNS0_10SymbolDataEN12_GLOBAL__N_16RefValENS2_16ImutKeyValueInfoIS6_S8_EEEEE8GDMIndexEvE9RefBIndex", "clang::ento::GRStateTrait<llvm::ImmutableMap<clang::ento::SymbolData const*, (anonymous namespace)::RefVal, llvm::ImutKeyValueInfo<clang::ento::SymbolData const*, (anonymous namespace)::RefVal> > >::GDMIndex()::RefBIndex"},
    {"_ZZN12_GLOBAL__N_110CFRefCount10evalReturnERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_10ReturnStmtEPNS2_12ExplodedNodeEE14autoreleasetag", "(anonymous namespace)::CFRefCount::evalReturn(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ReturnStmt const*, clang::ento::ExplodedNode*)::autoreleasetag"},
    {"_ZZN12_GLOBAL__N_110CFRefCount10evalReturnERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_10ReturnStmtEPNS2_12ExplodedNodeEE16ReturnOwnLeakTag", "(anonymous namespace)::CFRefCount::evalReturn(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ReturnStmt const*, clang::ento::ExplodedNode*)::ReturnOwnLeakTag"},
    {"_ZZN12_GLOBAL__N_110CFRefCount10evalReturnERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPKNS1_10ReturnStmtEPNS2_12ExplodedNodeEE25ReturnNotOwnedForOwnedTag", "(anonymous namespace)::CFRefCount::evalReturn(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ReturnStmt const*, clang::ento::ExplodedNode*)::ReturnNotOwnedForOwnedTag"},
    {"_ZZN12_GLOBAL__N_110CFRefCount15evalDeadSymbolsERN5clang4ento15ExplodedNodeSetERNS2_10ExprEngineERNS2_15StmtNodeBuilderEPNS2_12ExplodedNodeEPKNS2_7GRStateERNS2_12SymbolReaperEE9LeakPPTag", "(anonymous namespace)::CFRefCount::evalDeadSymbols(clang::ento::ExplodedNodeSet&, clang::ento::ExprEngine&, clang::ento::StmtNodeBuilder&, clang::ento::ExplodedNode*, clang::ento::GRState const*, clang::ento::SymbolReaper&)::LeakPPTag"},
    {"_ZZN12_GLOBAL__N_120RetainReleaseChecker6getTagEvE1x", "(anonymous namespace)::RetainReleaseChecker::getTag()::x"},
    {"_ZZN5clang4ento14CheckerContextD1EvE17autoTransitionTag", "clang::ento::CheckerContext::~CheckerContext()::autoTransitionTag"},
    {"_ZZN12_GLOBAL__N_111RewriteObjC38RewriteFunctionBodyOrGlobalInitializerEPN5clang4StmtEE20uniqueByrefDeclCount", "(anonymous namespace)::RewriteObjC::RewriteFunctionBodyOrGlobalInitializer(clang::Stmt*)::uniqueByrefDeclCount"},
    {"_ZZN5clang22getClangRepositoryPathEvE13SVNRepository", "clang::getClangRepositoryPath()::SVNRepository"},
    {"_ZGVZN5clang22getClangRepositoryPathEvE13SVNRepository", "guard variable for clang::getClangRepositoryPath()::SVNRepository"},
    {"_ZZN4llvm26initializeArgPromotionPassERNS_12PassRegistryEE11initialized", "llvm::initializeArgPromotionPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112ArgPromotion2IDE", "(anonymous namespace)::ArgPromotion::ID"},
    {"_ZZN4llvm27initializeConstantMergePassERNS_12PassRegistryEE11initialized", "llvm::initializeConstantMergePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113ConstantMerge2IDE", "(anonymous namespace)::ConstantMerge::ID"},
    {"_ZZN4llvm17initializeDAEPassERNS_12PassRegistryEE11initialized", "llvm::initializeDAEPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm17initializeDAHPassERNS_12PassRegistryEE11initialized", "llvm::initializeDAHPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_13DAH2IDE", "(anonymous namespace)::DAH::ID"},
    {"_ZN12_GLOBAL__N_13DAE2IDE", "(anonymous namespace)::DAE::ID"},
    {"_ZZN4llvm17initializeDTEPassERNS_12PassRegistryEE11initialized", "llvm::initializeDTEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_13DTE2IDE", "(anonymous namespace)::DTE::ID"},
    {"_ZZN4llvm27initializeFunctionAttrsPassERNS_12PassRegistryEE11initialized", "llvm::initializeFunctionAttrsPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113FunctionAttrs2IDE", "(anonymous namespace)::FunctionAttrs::ID"},
    {"_ZZN4llvm23initializeGlobalDCEPassERNS_12PassRegistryEE11initialized", "llvm::initializeGlobalDCEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_19GlobalDCE2IDE", "(anonymous namespace)::GlobalDCE::ID"},
    {"_ZZN4llvm23initializeGlobalOptPassERNS_12PassRegistryEE11initialized", "llvm::initializeGlobalOptPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_19GlobalOpt2IDE", "(anonymous namespace)::GlobalOpt::ID"},
    {"_ZZN4llvm27initializeAlwaysInlinerPassERNS_12PassRegistryEE11initialized", "llvm::initializeAlwaysInlinerPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113AlwaysInliner2IDE", "(anonymous namespace)::AlwaysInliner::ID"},
    {"_ZZN4llvm27initializeSimpleInlinerPassERNS_12PassRegistryEE11initialized", "llvm::initializeSimpleInlinerPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113SimpleInliner2IDE", "(anonymous namespace)::SimpleInliner::ID"},
    {"_ZZN4llvm21initializePruneEHPassERNS_12PassRegistryEE11initialized", "llvm::initializePruneEHPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_17PruneEH2IDE", "(anonymous namespace)::PruneEH::ID"},
    {"_ZZN4llvm37initializeStripDeadPrototypesPassPassERNS_12PassRegistryEE11initialized", "llvm::initializeStripDeadPrototypesPassPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_123StripDeadPrototypesPass2IDE", "(anonymous namespace)::StripDeadPrototypesPass::ID"},
    {"_ZN12_GLOBAL__N_116WriteBitcodePass2IDE", "(anonymous namespace)::WriteBitcodePass::ID"},
    {"_ZN12_GLOBAL__N_116SSEDomainFixPass2IDE", "(anonymous namespace)::SSEDomainFixPass::ID"},
    {"_ZN12_GLOBAL__N_17EmitterIN4llvm14JITCodeEmitterEE2IDE", "(anonymous namespace)::Emitter<llvm::JITCodeEmitter>::ID"},
    {"_ZN12_GLOBAL__N_13FPS2IDE", "(anonymous namespace)::FPS::ID"},
    {"_ZZN4llvm17X86TargetLoweringC1ERNS_16X86TargetMachineEE6IntVTs", "llvm::X86TargetLowering::X86TargetLowering(llvm::X86TargetMachine&)::IntVTs"},
    {"_ZGVZN4llvm17X86TargetLoweringC1ERNS_16X86TargetMachineEE6IntVTs", "guard variable for llvm::X86TargetLowering::X86TargetLowering(llvm::X86TargetMachine&)::IntVTs"},
    {"_ZN12_GLOBAL__N_14CGBR2IDE", "(anonymous namespace)::CGBR::ID"},
    {"_ZN12_GLOBAL__N_14MSAH2IDE", "(anonymous namespace)::MSAH::ID"},
    {"_ZN12_GLOBAL__N_114ARMCodeEmitter2IDE", "(anonymous namespace)::ARMCodeEmitter::ID"},
    {"_ZN12_GLOBAL__N_118ARMConstantIslands2IDE", "(anonymous namespace)::ARMConstantIslands::ID"},
    {"_ZN12_GLOBAL__N_115ARMExpandPseudo2IDE", "(anonymous namespace)::ARMExpandPseudo::ID"},
    {"_ZN12_GLOBAL__N_114ARMGlobalMerge2IDE", "(anonymous namespace)::ARMGlobalMerge::ID"},
    {"_ZN12_GLOBAL__N_115ARMLoadStoreOpt2IDE", "(anonymous namespace)::ARMLoadStoreOpt::ID"},
    {"_ZN12_GLOBAL__N_123ARMPreAllocLoadStoreOpt2IDE", "(anonymous namespace)::ARMPreAllocLoadStoreOpt::ID"},
    {"_ZN12_GLOBAL__N_112MLxExpansion2IDE", "(anonymous namespace)::MLxExpansion::ID"},
    {"_ZN12_GLOBAL__N_115NEONMoveFixPass2IDE", "(anonymous namespace)::NEONMoveFixPass::ID"},
    {"_ZN12_GLOBAL__N_117Thumb2ITBlockPass2IDE", "(anonymous namespace)::Thumb2ITBlockPass::ID"},
    {"_ZN12_GLOBAL__N_116Thumb2SizeReduce2IDE", "(anonymous namespace)::Thumb2SizeReduce::ID"},
    {"_ZZN12_GLOBAL__N_120SelectionDAGLegalize10LegalizeOpEN4llvm7SDValueEE5depth", "(anonymous namespace)::SelectionDAGLegalize::LegalizeOp(llvm::SDValue)::depth"},
    {"_ZZN4llvm10DwarfDebug21computeSizeAndOffsetsEvE6Offset", "llvm::DwarfDebug::computeSizeAndOffsets()::Offset"},
    {"_ZGVZN4llvm10DwarfDebug21computeSizeAndOffsetsEvE6Offset", "guard variable for llvm::DwarfDebug::computeSizeAndOffsets()::Offset"},
    {"_ZN12_GLOBAL__N_116BranchFolderPass2IDE", "(anonymous namespace)::BranchFolderPass::ID"},
    {"_ZZN4llvm35initializeCalculateSpillWeightsPassERNS_12PassRegistryEE11initialized", "llvm::initializeCalculateSpillWeightsPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_116CodePlacementOpt2IDE", "(anonymous namespace)::CodePlacementOpt::ID"},
    {"_ZZN4llvm40initializeDeadMachineInstructionElimPassERNS_12PassRegistryEE11initialized", "llvm::initializeDeadMachineInstructionElimPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_126DeadMachineInstructionElim2IDE", "(anonymous namespace)::DeadMachineInstructionElim::ID"},
    {"_ZN12_GLOBAL__N_114DwarfEHPrepare2IDE", "(anonymous namespace)::DwarfEHPrepare::ID"},
    {"_ZZN4llvm25initializeEdgeBundlesPassERNS_12PassRegistryEE11initialized", "llvm::initializeEdgeBundlesPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm31initializeExpandISelPseudosPassERNS_12PassRegistryEE11initialized", "llvm::initializeExpandISelPseudosPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_117ExpandISelPseudos2IDE", "(anonymous namespace)::ExpandISelPseudos::ID"},
    {"_ZZN4llvm26initializeGCModuleInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeGCModuleInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_17Deleter2IDE", "(anonymous namespace)::Deleter::ID"},
    {"_ZN12_GLOBAL__N_17Printer2IDE", "(anonymous namespace)::Printer::ID"},
    {"_ZZN4llvm29initializeLowerIntrinsicsPassERNS_12PassRegistryEE11initialized", "llvm::initializeLowerIntrinsicsPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_119MachineCodeAnalysis2IDE", "(anonymous namespace)::MachineCodeAnalysis::ID"},
    {"_ZN12_GLOBAL__N_115LowerIntrinsics2IDE", "(anonymous namespace)::LowerIntrinsics::ID"},
    {"_ZZN4llvm25initializeIfConverterPassERNS_12PassRegistryEE11initialized", "llvm::initializeIfConverterPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111IfConverter2IDE", "(anonymous namespace)::IfConverter::ID"},
    {"_ZZN4llvm32initializeLiveDebugVariablesPassERNS_12PassRegistryEE11initialized", "llvm::initializeLiveDebugVariablesPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm27initializeLiveIntervalsPassERNS_12PassRegistryEE11initialized", "llvm::initializeLiveIntervalsPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm24initializeLiveStacksPassERNS_12PassRegistryEE11initialized", "llvm::initializeLiveStacksPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm27initializeLiveVariablesPassERNS_12PassRegistryEE11initialized", "llvm::initializeLiveVariablesPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_118LocalStackSlotPass2IDE", "(anonymous namespace)::LocalStackSlotPass::ID"},
    {"_ZN12_GLOBAL__N_127LowerSubregsInstructionPass2IDE", "(anonymous namespace)::LowerSubregsInstructionPass::ID"},
    {"_ZZN4llvm24initializeMachineCSEPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineCSEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_110MachineCSE2IDE", "(anonymous namespace)::MachineCSE::ID"},
    {"_ZZN4llvm34initializeMachineDominatorTreePassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineDominatorTreePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_126MachineFunctionPrinterPass2IDE", "(anonymous namespace)::MachineFunctionPrinterPass::ID"},
    {"_ZZN4llvm25initializeMachineLICMPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineLICMPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111MachineLICM2IDE", "(anonymous namespace)::MachineLICM::ID"},
    {"_ZZN4llvm29initializeMachineLoopInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineLoopInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm31initializeMachineModuleInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineModuleInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm28initializeMachineSinkingPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineSinkingPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114MachineSinking2IDE", "(anonymous namespace)::MachineSinking::ID"},
    {"_ZZN4llvm33initializeMachineVerifierPassPassERNS_12PassRegistryEE11initialized", "llvm::initializeMachineVerifierPassPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_119MachineVerifierPass2IDE", "(anonymous namespace)::MachineVerifierPass::ID"},
    {"_ZZN4llvm26initializeOptimizePHIsPassERNS_12PassRegistryEE11initialized", "llvm::initializeOptimizePHIsPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112OptimizePHIs2IDE", "(anonymous namespace)::OptimizePHIs::ID"},
    {"_ZZN4llvm28initializePHIEliminationPassERNS_12PassRegistryEE11initialized", "llvm::initializePHIEliminationPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114PHIElimination2IDE", "(anonymous namespace)::PHIElimination::ID"},
    {"_ZZN4llvm31initializePeepholeOptimizerPassERNS_12PassRegistryEE11initialized", "llvm::initializePeepholeOptimizerPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_117PeepholeOptimizer2IDE", "(anonymous namespace)::PeepholeOptimizer::ID"},
    {"_ZN12_GLOBAL__N_115PostRAScheduler2IDE", "(anonymous namespace)::PostRAScheduler::ID"},
    {"_ZZN4llvm31initializePreAllocSplittingPassERNS_12PassRegistryEE11initialized", "llvm::initializePreAllocSplittingPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_117PreAllocSplitting2IDE", "(anonymous namespace)::PreAllocSplitting::ID"},
    {"_ZZN4llvm33initializeProcessImplicitDefsPassERNS_12PassRegistryEE11initialized", "llvm::initializeProcessImplicitDefsPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm17initializePEIPassERNS_12PassRegistryEE11initialized", "llvm::initializePEIPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_16RAFast2IDE", "(anonymous namespace)::RAFast::ID"},
    {"_ZZN4llvm23initializeRALinScanPassERNS_12PassRegistryEE11initialized", "llvm::initializeRALinScanPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_19RALinScan2IDE", "(anonymous namespace)::RALinScan::ID"},
    {"_ZZN4llvm40initializeRegisterCoalescerAnalysisGroupERNS_12PassRegistryEE11initialized", "llvm::initializeRegisterCoalescerAnalysisGroup(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm38initializeSimpleRegisterCoalescingPassERNS_12PassRegistryEE11initialized", "llvm::initializeSimpleRegisterCoalescingPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_110SjLjEHPass2IDE", "(anonymous namespace)::SjLjEHPass::ID"},
    {"_ZN12_GLOBAL__N_122IndexListEntryEmptyKeyE", "(anonymous namespace)::IndexListEntryEmptyKey"},
    {"_ZN12_GLOBAL__N_126IndexListEntryTombstoneKeyE", "(anonymous namespace)::IndexListEntryTombstoneKey"},
    {"_ZZN4llvm25initializeSlotIndexesPassERNS_12PassRegistryEE11initialized", "llvm::initializeSlotIndexesPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm28initializeStackProtectorPassERNS_12PassRegistryEE11initialized", "llvm::initializeStackProtectorPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114StackProtector2IDE", "(anonymous namespace)::StackProtector::ID"},
    {"_ZZN4llvm31initializeStackSlotColoringPassERNS_12PassRegistryEE11initialized", "llvm::initializeStackSlotColoringPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_117StackSlotColoring2IDE", "(anonymous namespace)::StackSlotColoring::ID"},
    {"_ZZN4llvm34initializeStrongPHIEliminationPassERNS_12PassRegistryEE11initialized", "llvm::initializeStrongPHIEliminationPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_120StrongPHIElimination2IDE", "(anonymous namespace)::StrongPHIElimination::ID"},
    {"_ZN12_GLOBAL__N_117TailDuplicatePass2IDE", "(anonymous namespace)::TailDuplicatePass::ID"},
    {"_ZZN4llvm39initializeTwoAddressInstructionPassPassERNS_12PassRegistryEE11initialized", "llvm::initializeTwoAddressInstructionPassPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_125TwoAddressInstructionPass2IDE", "(anonymous namespace)::TwoAddressInstructionPass::ID"},
    {"_ZZN4llvm34initializeUnreachableBlockElimPassERNS_12PassRegistryEE11initialized", "llvm::initializeUnreachableBlockElimPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm41initializeUnreachableMachineBlockElimPassERNS_12PassRegistryEE11initialized", "llvm::initializeUnreachableMachineBlockElimPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_127UnreachableMachineBlockElim2IDE", "(anonymous namespace)::UnreachableMachineBlockElim::ID"},
    {"_ZN12_GLOBAL__N_120UnreachableBlockElim2IDE", "(anonymous namespace)::UnreachableBlockElim::ID"},
    {"_ZZN4llvm24initializeVirtRegMapPassERNS_12PassRegistryEE11initialized", "llvm::initializeVirtRegMapPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm18initializeADCEPassERNS_12PassRegistryEE11initialized", "llvm::initializeADCEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_14ADCE2IDE", "(anonymous namespace)::ADCE::ID"},
    {"_ZZN4llvm28initializeCodeGenPreparePassERNS_12PassRegistryEE11initialized", "llvm::initializeCodeGenPreparePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114CodeGenPrepare2IDE", "(anonymous namespace)::CodeGenPrepare::ID"},
    {"_ZZN4llvm40initializeCorrelatedValuePropagationPassERNS_12PassRegistryEE11initialized", "llvm::initializeCorrelatedValuePropagationPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_126CorrelatedValuePropagation2IDE", "(anonymous namespace)::CorrelatedValuePropagation::ID"},
    {"_ZZN4llvm17initializeDSEPassERNS_12PassRegistryEE11initialized", "llvm::initializeDSEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_13DSE2IDE", "(anonymous namespace)::DSE::ID"},
    {"_ZZN4llvm22initializeEarlyCSEPassERNS_12PassRegistryEE11initialized", "llvm::initializeEarlyCSEPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_18EarlyCSE2IDE", "(anonymous namespace)::EarlyCSE::ID"},
    {"_ZZN4llvm25initializeGEPSplitterPassERNS_12PassRegistryEE11initialized", "llvm::initializeGEPSplitterPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111GEPSplitter2IDE", "(anonymous namespace)::GEPSplitter::ID"},
    {"_ZZN4llvm17initializeGVNPassERNS_12PassRegistryEE11initialized", "llvm::initializeGVNPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_13GVN2IDE", "(anonymous namespace)::GVN::ID"},
    {"_ZZN4llvm28initializeIndVarSimplifyPassERNS_12PassRegistryEE11initialized", "llvm::initializeIndVarSimplifyPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114IndVarSimplify2IDE", "(anonymous namespace)::IndVarSimplify::ID"},
    {"_ZZN4llvm27initializeJumpThreadingPassERNS_12PassRegistryEE11initialized", "llvm::initializeJumpThreadingPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113JumpThreading2IDE", "(anonymous namespace)::JumpThreading::ID"},
    {"_ZZN4llvm18initializeLICMPassERNS_12PassRegistryEE11initialized", "llvm::initializeLICMPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_14LICM2IDE", "(anonymous namespace)::LICM::ID"},
    {"_ZZN4llvm26initializeLoopDeletionPassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopDeletionPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112LoopDeletion2IDE", "(anonymous namespace)::LoopDeletion::ID"},
    {"_ZZN4llvm32initializeLoopIdiomRecognizePassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopIdiomRecognizePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_118LoopIdiomRecognize2IDE", "(anonymous namespace)::LoopIdiomRecognize::ID"},
    {"_ZZN4llvm24initializeLoopRotatePassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopRotatePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_110LoopRotate2IDE", "(anonymous namespace)::LoopRotate::ID"},
    {"_ZZN4llvm32initializeLoopStrengthReducePassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopStrengthReducePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_118LoopStrengthReduce2IDE", "(anonymous namespace)::LoopStrengthReduce::ID"},
    {"_ZZN4llvm24initializeLoopUnrollPassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopUnrollPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_110LoopUnroll2IDE", "(anonymous namespace)::LoopUnroll::ID"},
    {"_ZZN4llvm26initializeLoopUnswitchPassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopUnswitchPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112LoopUnswitch2IDE", "(anonymous namespace)::LoopUnswitch::ID"},
    {"_ZZN4llvm23initializeMemCpyOptPassERNS_12PassRegistryEE11initialized", "llvm::initializeMemCpyOptPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_19MemCpyOpt2IDE", "(anonymous namespace)::MemCpyOpt::ID"},
    {"_ZZN4llvm25initializeReassociatePassERNS_12PassRegistryEE11initialized", "llvm::initializeReassociatePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111Reassociate2IDE", "(anonymous namespace)::Reassociate::ID"},
    {"_ZZN4llvm18initializeSCCPPassERNS_12PassRegistryEE11initialized", "llvm::initializeSCCPPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm20initializeIPSCCPPassERNS_12PassRegistryEE11initialized", "llvm::initializeIPSCCPPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_16IPSCCP2IDE", "(anonymous namespace)::IPSCCP::ID"},
    {"_ZN12_GLOBAL__N_14SCCP2IDE", "(anonymous namespace)::SCCP::ID"},
    {"_ZZN4llvm21initializeSROA_DTPassERNS_12PassRegistryEE11initialized", "llvm::initializeSROA_DTPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm24initializeSROA_SSAUpPassERNS_12PassRegistryEE11initialized", "llvm::initializeSROA_SSAUpPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_110SROA_SSAUp2IDE", "(anonymous namespace)::SROA_SSAUp::ID"},
    {"_ZN12_GLOBAL__N_17SROA_DT2IDE", "(anonymous namespace)::SROA_DT::ID"},
    {"_ZZN4llvm29initializeCFGSimplifyPassPassERNS_12PassRegistryEE11initialized", "llvm::initializeCFGSimplifyPassPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_115CFGSimplifyPass2IDE", "(anonymous namespace)::CFGSimplifyPass::ID"},
    {"_ZZN4llvm30initializeSimplifyLibCallsPassERNS_12PassRegistryEE11initialized", "llvm::initializeSimplifyLibCallsPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_116SimplifyLibCalls2IDE", "(anonymous namespace)::SimplifyLibCalls::ID"},
    {"_ZZN4llvm26initializeTailCallElimPassERNS_12PassRegistryEE11initialized", "llvm::initializeTailCallElimPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112TailCallElim2IDE", "(anonymous namespace)::TailCallElim::ID"},
    {"_ZZN4llvm26initializeInstCombinerPassERNS_12PassRegistryEE11initialized", "llvm::initializeInstCombinerPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm32initializeBreakCriticalEdgesPassERNS_12PassRegistryEE11initialized", "llvm::initializeBreakCriticalEdgesPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_118BreakCriticalEdges2IDE", "(anonymous namespace)::BreakCriticalEdges::ID"},
    {"_ZZN4llvm19initializeLCSSAPassERNS_12PassRegistryEE11initialized", "llvm::initializeLCSSAPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_15LCSSA2IDE", "(anonymous namespace)::LCSSA::ID"},
    {"_ZZN4llvm26initializeLoopSimplifyPassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopSimplifyPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_112LoopSimplify2IDE", "(anonymous namespace)::LoopSimplify::ID"},
    {"_ZZN4llvm25initializeLowerInvokePassERNS_12PassRegistryEE11initialized", "llvm::initializeLowerInvokePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111LowerInvoke2IDE", "(anonymous namespace)::LowerInvoke::ID"},
    {"_ZZN4llvm25initializeLowerSwitchPassERNS_12PassRegistryEE11initialized", "llvm::initializeLowerSwitchPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_111LowerSwitch2IDE", "(anonymous namespace)::LowerSwitch::ID"},
    {"_ZZN4llvm36initializeUnifyFunctionExitNodesPassERNS_12PassRegistryEE11initialized", "llvm::initializeUnifyFunctionExitNodesPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm32initializeCallGraphAnalysisGroupERNS_12PassRegistryEE11initialized", "llvm::initializeCallGraphAnalysisGroup(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm28initializeBasicCallGraphPassERNS_12PassRegistryEE11initialized", "llvm::initializeBasicCallGraphPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_114BasicCallGraph2IDE", "(anonymous namespace)::BasicCallGraph::ID"},
    {"_ZN12_GLOBAL__N_118PrintCallGraphPass2IDE", "(anonymous namespace)::PrintCallGraphPass::ID"},
    {"_ZN12_GLOBAL__N_113CGPassManager2IDE", "(anonymous namespace)::CGPassManager::ID"},
    {"_ZZN4llvm27initializeFindUsedTypesPassERNS_12PassRegistryEE11initialized", "llvm::initializeFindUsedTypesPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm36initializeAliasAnalysisAnalysisGroupERNS_12PassRegistryEE11initialized", "llvm::initializeAliasAnalysisAnalysisGroup(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm29initializeAliasSetPrinterPassERNS_12PassRegistryEE11initialized", "llvm::initializeAliasSetPrinterPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_115AliasSetPrinter2IDE", "(anonymous namespace)::AliasSetPrinter::ID"},
    {"_ZZN4llvm32initializeBasicAliasAnalysisPassERNS_12PassRegistryEE11initialized", "llvm::initializeBasicAliasAnalysisPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_118BasicAliasAnalysis2IDE", "(anonymous namespace)::BasicAliasAnalysis::ID"},
    {"_ZZN4llvm9DIBuilder18CreateLexicalBlockENS_12DIDescriptorENS_6DIFileEjjE9unique_id", "llvm::DIBuilder::CreateLexicalBlock(llvm::DIDescriptor, llvm::DIFile, unsigned int, unsigned int)::unique_id"},
    {"_ZZN4llvm9DIFactory18CreateLexicalBlockENS_12DIDescriptorENS_6DIFileEjjE9unique_id", "llvm::DIFactory::CreateLexicalBlock(llvm::DIDescriptor, llvm::DIFile, unsigned int, unsigned int)::unique_id"},
    {"_ZZN4llvm21initializeIVUsersPassERNS_12PassRegistryEE11initialized", "llvm::initializeIVUsersPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm27initializeLazyValueInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeLazyValueInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm22initializeLoopInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeLoopInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113PrintLoopPass2IDE", "(anonymous namespace)::PrintLoopPass::ID"},
    {"_ZZN4llvm38initializeMemoryDependenceAnalysisPassERNS_12PassRegistryEE11initialized", "llvm::initializeMemoryDependenceAnalysisPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm18initializeNoAAPassERNS_12PassRegistryEE11initialized", "llvm::initializeNoAAPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_14NoAA2IDE", "(anonymous namespace)::NoAA::ID"},
    {"_ZZN4llvm34initializeProfileInfoAnalysisGroupERNS_12PassRegistryEE11initialized", "llvm::initializeProfileInfoAnalysisGroup(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm27initializeNoProfileInfoPassERNS_12PassRegistryEE11initialized", "llvm::initializeNoProfileInfoPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113NoProfileInfo2IDE", "(anonymous namespace)::NoProfileInfo::ID"},
    {"_ZZN4llvm29initializeScalarEvolutionPassERNS_12PassRegistryEE11initialized", "llvm::initializeScalarEvolutionPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm36initializeTypeBasedAliasAnalysisPassERNS_12PassRegistryEE11initialized", "llvm::initializeTypeBasedAliasAnalysisPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_122TypeBasedAliasAnalysis2IDE", "(anonymous namespace)::TypeBasedAliasAnalysis::ID"},
    {"_ZZN4llvm24initializeDataLayoutPassERNS_12PassRegistryEE11initialized", "llvm::initializeDataLayoutPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm27initializeDominatorTreePassERNS_12PassRegistryEE11initialized", "llvm::initializeDominatorTreePass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_113BBPassManager2IDE", "(anonymous namespace)::BBPassManager::ID"},
    {"_ZZN12_GLOBAL__N_110TimingInfo17createTheTimeInfoEvE3TTI", "(anonymous namespace)::TimingInfo::createTheTimeInfo()::TTI"},
    {"_ZZN4llvm29initializePrintModulePassPassERNS_12PassRegistryEE11initialized", "llvm::initializePrintModulePassPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm31initializePrintFunctionPassPassERNS_12PassRegistryEE11initialized", "llvm::initializePrintFunctionPassPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_117PrintFunctionPass2IDE", "(anonymous namespace)::PrintFunctionPass::ID"},
    {"_ZN12_GLOBAL__N_115PrintModulePass2IDE", "(anonymous namespace)::PrintModulePass::ID"},
    {"_ZZN4llvm25initializePreVerifierPassERNS_12PassRegistryEE11initialized", "llvm::initializePreVerifierPass(llvm::PassRegistry&)::initialized"},
    {"_ZZN4llvm22initializeVerifierPassERNS_12PassRegistryEE11initialized", "llvm::initializeVerifierPass(llvm::PassRegistry&)::initialized"},
    {"_ZN12_GLOBAL__N_18Verifier2IDE", "(anonymous namespace)::Verifier::ID"},
    {"_ZN12_GLOBAL__N_111PreVerifier2IDE", "(anonymous namespace)::PreVerifier::ID"},
    {"_ZZ8getMutexvE12HandlesMutex", "getMutex()::HandlesMutex"},
    {"_ZGVZ8getMutexvE12HandlesMutex", "guard variable for getMutex()::HandlesMutex"},
    {"_ZZN4llvm5foutsEvE1S", "llvm::fouts()::S"},
    {"_ZGVZN4llvm5foutsEvE1S", "guard variable for llvm::fouts()::S"},
    {"_ZZN4llvm5ferrsEvE1S", "llvm::ferrs()::S"},
    {"_ZGVZN4llvm5ferrsEvE1S", "guard variable for llvm::ferrs()::S"},
    {"_ZZN4llvm5fdbgsEvE1S", "llvm::fdbgs()::S"},
    {"_ZGVZN4llvm5fdbgsEvE1S", "guard variable for llvm::fdbgs()::S"},
    {"_ZGVZN4llvm21PrettyStackTraceEntryC1EvE17HandlerRegistered", "guard variable for llvm::PrettyStackTraceEntry::PrettyStackTraceEntry()::HandlerRegistered"},
    {"_ZZ15PrintStackTracePvE10StackTrace", "PrintStackTrace(void*)::StackTrace"},
    {"_ZZN4llvm4outsEvE1S", "llvm::outs()::S"},
    {"_ZGVZN4llvm4outsEvE1S", "guard variable for llvm::outs()::S"},
    {"_ZZN4llvm4errsEvE1S", "llvm::errs()::S"},
    {"_ZGVZN4llvm4errsEvE1S", "guard variable for llvm::errs()::S"},
    {"_ZZN4llvm5nullsEvE1S", "llvm::nulls()::S"},
    {"_ZGVZN4llvm5nullsEvE1S", "guard variable for llvm::nulls()::S"},
    {"_ZZN4llvm16generic_categoryEvE1s", "llvm::generic_category()::s"},
    {"_ZGVZN4llvm16generic_categoryEvE1s", "guard variable for llvm::generic_category()::s"},
    {"_ZZN4llvm15system_categoryEvE1s", "llvm::system_category()::s"},
    {"_ZGVZN4llvm15system_categoryEvE1s", "guard variable for llvm::system_category()::s"},
    {"_ZN5clang7CodeGen13EHPersonality5GNU_CE", "clang::CodeGen::EHPersonality::GNU_C"},
    {"_ZN5clang7CodeGen13EHPersonality10GNU_C_SJLJE", "clang::CodeGen::EHPersonality::GNU_C_SJLJ"},
    {"_ZN5clang7CodeGen13EHPersonality9NeXT_ObjCE", "clang::CodeGen::EHPersonality::NeXT_ObjC"},
    {"_ZN5clang7CodeGen13EHPersonality13GNU_CPlusPlusE", "clang::CodeGen::EHPersonality::GNU_CPlusPlus"},
    {"_ZN5clang7CodeGen13EHPersonality18GNU_CPlusPlus_SJLJE", "clang::CodeGen::EHPersonality::GNU_CPlusPlus_SJLJ"},
    {"_ZN5clang7CodeGen13EHPersonality8GNU_ObjCE", "clang::CodeGen::EHPersonality::GNU_ObjC"},
    {"_ZN5clang4ento15ManagerRegistry15StoreMgrCreatorE", "clang::ento::ManagerRegistry::StoreMgrCreator"},
    {"_ZN5clang4ento15ManagerRegistry20ConstraintMgrCreatorE", "clang::ento::ManagerRegistry::ConstraintMgrCreator"},
    {"_ZN5clang10ASTContext30NumImplicitDefaultConstructorsE", "clang::ASTContext::NumImplicitDefaultConstructors"},
    {"_ZN5clang10ASTContext38NumImplicitDefaultConstructorsDeclaredE", "clang::ASTContext::NumImplicitDefaultConstructorsDeclared"},
    {"_ZN5clang10ASTContext27NumImplicitCopyConstructorsE", "clang::ASTContext::NumImplicitCopyConstructors"},
    {"_ZN5clang10ASTContext35NumImplicitCopyConstructorsDeclaredE", "clang::ASTContext::NumImplicitCopyConstructorsDeclared"},
    {"_ZN5clang10ASTContext34NumImplicitCopyAssignmentOperatorsE", "clang::ASTContext::NumImplicitCopyAssignmentOperators"},
    {"_ZN5clang10ASTContext42NumImplicitCopyAssignmentOperatorsDeclaredE", "clang::ASTContext::NumImplicitCopyAssignmentOperatorsDeclared"},
    {"_ZN5clang10ASTContext22NumImplicitDestructorsE", "clang::ASTContext::NumImplicitDestructors"},
    {"_ZN5clang10ASTContext30NumImplicitDestructorsDeclaredE", "clang::ASTContext::NumImplicitDestructorsDeclared"},
    {"_ZN4llvm3X8611GR8RegClassE", "llvm::X86::GR8RegClass"},
    {"_ZN4llvm3X8618GR8_ABCD_LRegClassE", "llvm::X86::GR8_ABCD_LRegClass"},
    {"_ZN4llvm3X8618GR8_ABCD_HRegClassE", "llvm::X86::GR8_ABCD_HRegClass"},
    {"_ZN4llvm3X8612GR64RegClassE", "llvm::X86::GR64RegClass"},
    {"_ZN4llvm3X8617GR64_ABCDRegClassE", "llvm::X86::GR64_ABCDRegClass"},
    {"_ZN4llvm3X8618GR64_NOREXRegClassE", "llvm::X86::GR64_NOREXRegClass"},
    {"_ZN4llvm3X8617GR64_NOSPRegClassE", "llvm::X86::GR64_NOSPRegClass"},
    {"_ZN4llvm3X8623GR64_NOREX_NOSPRegClassE", "llvm::X86::GR64_NOREX_NOSPRegClass"},
    {"_ZN4llvm3X8612GR32RegClassE", "llvm::X86::GR32RegClass"},
    {"_ZN4llvm3X8617GR32_ABCDRegClassE", "llvm::X86::GR32_ABCDRegClass"},
    {"_ZN4llvm3X8618GR32_NOREXRegClassE", "llvm::X86::GR32_NOREXRegClass"},
    {"_ZN4llvm3X8617GR32_NOSPRegClassE", "llvm::X86::GR32_NOSPRegClass"},
    {"_ZN4llvm3X8612GR16RegClassE", "llvm::X86::GR16RegClass"},
    {"_ZN4llvm3X8617GR16_ABCDRegClassE", "llvm::X86::GR16_ABCDRegClass"},
    {"_ZN4llvm3X8618GR16_NOREXRegClassE", "llvm::X86::GR16_NOREXRegClass"},
    {"_ZN4llvm3X8617GR8_NOREXRegClassE", "llvm::X86::GR8_NOREXRegClass"},
    {"_ZN4llvm3X8612FR32RegClassE", "llvm::X86::FR32RegClass"},
    {"_ZN4llvm3X8612FR64RegClassE", "llvm::X86::FR64RegClass"},
    {"_ZN4llvm3X8613VR128RegClassE", "llvm::X86::VR128RegClass"},
    {"_ZN4llvm3X8618GR64_TCW64RegClassE", "llvm::X86::GR64_TCW64RegClass"},
    {"_ZN4llvm3X8615GR64_TCRegClassE", "llvm::X86::GR64_TCRegClass"},
    {"_ZN4llvm3X8615GR32_TCRegClassE", "llvm::X86::GR32_TCRegClass"},
    {"_ZN4llvm3X8611CCRRegClassE", "llvm::X86::CCRRegClass"},
    {"_ZN4llvm3X8619CONTROL_REGRegClassE", "llvm::X86::CONTROL_REGRegClass"},
    {"_ZN4llvm3X8617DEBUG_REGRegClassE", "llvm::X86::DEBUG_REGRegClass"},
    {"_ZN4llvm3X8615GR32_ADRegClassE", "llvm::X86::GR32_ADRegClass"},
    {"_ZN4llvm3X8613RFP32RegClassE", "llvm::X86::RFP32RegClass"},
    {"_ZN4llvm3X8613RFP64RegClassE", "llvm::X86::RFP64RegClass"},
    {"_ZN4llvm3X8613RFP80RegClassE", "llvm::X86::RFP80RegClass"},
    {"_ZN4llvm3X8611RSTRegClassE", "llvm::X86::RSTRegClass"},
    {"_ZN4llvm3X8619SEGMENT_REGRegClassE", "llvm::X86::SEGMENT_REGRegClass"},
    {"_ZN4llvm3X8613VR256RegClassE", "llvm::X86::VR256RegClass"},
    {"_ZN4llvm3X8612VR64RegClassE", "llvm::X86::VR64RegClass"},
    {"_ZN4llvm3ARM13SPR_8RegClassE", "llvm::ARM::SPR_8RegClass"},
    {"_ZN4llvm3ARM13DPR_8RegClassE", "llvm::ARM::DPR_8RegClass"},
    {"_ZN4llvm3ARM16DPR_VFP2RegClassE", "llvm::ARM::DPR_VFP2RegClass"},
    {"_ZN4llvm3ARM13QPR_8RegClassE", "llvm::ARM::QPR_8RegClass"},
    {"_ZN4llvm3ARM16QPR_VFP2RegClassE", "llvm::ARM::QPR_VFP2RegClass"},
    {"_ZN4llvm3ARM17QQPR_VFP2RegClassE", "llvm::ARM::QQPR_VFP2RegClass"},
    {"_ZN4llvm3ARM11DPRRegClassE", "llvm::ARM::DPRRegClass"},
    {"_ZN4llvm3ARM11QPRRegClassE", "llvm::ARM::QPRRegClass"},
    {"_ZN4llvm3ARM11CCRRegClassE", "llvm::ARM::CCRRegClass"},
    {"_ZN4llvm3ARM11GPRRegClassE", "llvm::ARM::GPRRegClass"},
    {"_ZN4llvm3ARM12QQPRRegClassE", "llvm::ARM::QQPRRegClass"},
    {"_ZN4llvm3ARM14QQQQPRRegClassE", "llvm::ARM::QQQQPRRegClass"},
    {"_ZN4llvm3ARM11SPRRegClassE", "llvm::ARM::SPRRegClass"},
    {"_ZN4llvm3ARM12rGPRRegClassE", "llvm::ARM::rGPRRegClass"},
    {"_ZN4llvm3ARM12tGPRRegClassE", "llvm::ARM::tGPRRegClass"},
    {"_ZN4llvm3ARM13tcGPRRegClassE", "llvm::ARM::tcGPRRegClass"},
    {"_ZN4llvm17RegisterScheduler8RegistryE", "llvm::RegisterScheduler::Registry"},
    {"_ZN4llvm16SelectionDAGISel2IDE", "llvm::SelectionDAGISel::ID"},
    {"_ZN4llvm10AsmPrinter2IDE", "llvm::AsmPrinter::ID"},
    {"_ZN4llvm21CalculateSpillWeights2IDE", "llvm::CalculateSpillWeights::ID"},
    {"_ZN4llvm11EdgeBundles2IDE", "llvm::EdgeBundles::ID"},
    {"_ZN4llvm12GCModuleInfo2IDE", "llvm::GCModuleInfo::ID"},
    {"_ZN4llvm14EnableFastISelE", "llvm::EnableFastISel"},
    {"_ZN4llvm18LiveDebugVariables2IDE", "llvm::LiveDebugVariables::ID"},
    {"_ZN4llvm13LiveIntervals2IDE", "llvm::LiveIntervals::ID"},
    {"_ZN4llvm10LiveStacks2IDE", "llvm::LiveStacks::ID"},
    {"_ZN4llvm13LiveVariables2IDE", "llvm::LiveVariables::ID"},
    {"_ZN4llvm20MachineDominatorTree2IDE", "llvm::MachineDominatorTree::ID"},
    {"_ZN4llvm23MachineFunctionAnalysis2IDE", "llvm::MachineFunctionAnalysis::ID"},
    {"_ZN4llvm15MachineLoopInfo2IDE", "llvm::MachineLoopInfo::ID"},
    {"_ZN4llvm17MachineModuleInfo2IDE", "llvm::MachineModuleInfo::ID"},
    {"_ZN4llvm16RegisterRegAlloc8RegistryE", "llvm::RegisterRegAlloc::Registry"},
    {"_ZN4llvm19ProcessImplicitDefs2IDE", "llvm::ProcessImplicitDefs::ID"},
    {"_ZN4llvm3PEI2IDE", "llvm::PEI::ID"},
    {"_ZN4llvm17RegisterCoalescer2IDE", "llvm::RegisterCoalescer::ID"},
    {"_ZN4llvm24SimpleRegisterCoalescing2IDE", "llvm::SimpleRegisterCoalescing::ID"},
    {"_ZN4llvm11SlotIndexes2IDE", "llvm::SlotIndexes::ID"},
    {"_ZN4llvm10VirtRegMap2IDE", "llvm::VirtRegMap::ID"},
    {"_ZN4llvm12InstCombiner2IDE", "llvm::InstCombiner::ID"},
    {"_ZN4llvm22UnifyFunctionExitNodes2IDE", "llvm::UnifyFunctionExitNodes::ID"},
    {"_ZN4llvm9CallGraph2IDE", "llvm::CallGraph::ID"},
    {"_ZN4llvm13FindUsedTypes2IDE", "llvm::FindUsedTypes::ID"},
    {"_ZN4llvm13AliasAnalysis2IDE", "llvm::AliasAnalysis::ID"},
    {"_ZN4llvm7IVUsers2IDE", "llvm::IVUsers::ID"},
    {"_ZN4llvm13LazyValueInfo2IDE", "llvm::LazyValueInfo::ID"},
    {"_ZN4llvm8LoopInfo2IDE", "llvm::LoopInfo::ID"},
    {"_ZN4llvm13LPPassManager2IDE", "llvm::LPPassManager::ID"},
    {"_ZN4llvm24MemoryDependenceAnalysis2IDE", "llvm::MemoryDependenceAnalysis::ID"},
    {"_ZN4llvm12ProfileInfoTINS_8FunctionENS_10BasicBlockEE2IDE", "llvm::ProfileInfoT<llvm::Function, llvm::BasicBlock>::ID"},
    {"_ZN4llvm12ProfileInfoTINS_15MachineFunctionENS_17MachineBasicBlockEE2IDE", "llvm::ProfileInfoT<llvm::MachineFunction, llvm::MachineBasicBlock>::ID"},
    {"_ZN4llvm15ScalarEvolution2IDE", "llvm::ScalarEvolution::ID"},
    {"_ZN4llvm10DataLayout2IDE", "llvm::DataLayout::ID"},
    {"_ZN4llvm10DataLayout20InvalidAlignmentElemE", "llvm::DataLayout::InvalidAlignmentElem"},
    {"_ZN4llvm22LessPreciseFPMADOptionE", "llvm::LessPreciseFPMADOption"},
    {"_ZN4llvm16PrintMachineCodeE", "llvm::PrintMachineCode"},
    {"_ZN4llvm18NoFramePointerElimE", "llvm::NoFramePointerElim"},
    {"_ZN4llvm25NoFramePointerElimNonLeafE", "llvm::NoFramePointerElimNonLeaf"},
    {"_ZN4llvm19NoExcessFPPrecisionE", "llvm::NoExcessFPPrecision"},
    {"_ZN4llvm12UnsafeFPMathE", "llvm::UnsafeFPMath"},
    {"_ZN4llvm12NoInfsFPMathE", "llvm::NoInfsFPMath"},
    {"_ZN4llvm12NoNaNsFPMathE", "llvm::NoNaNsFPMath"},
    {"_ZN4llvm38HonorSignDependentRoundingFPMathOptionE", "llvm::HonorSignDependentRoundingFPMathOption"},
    {"_ZN4llvm12UseSoftFloatE", "llvm::UseSoftFloat"},
    {"_ZN4llvm12FloatABITypeE", "llvm::FloatABIType"},
    {"_ZN4llvm15NoImplicitFloatE", "llvm::NoImplicitFloat"},
    {"_ZN4llvm12NoZerosInBSSE", "llvm::NoZerosInBSS"},
    {"_ZN4llvm20JITExceptionHandlingE", "llvm::JITExceptionHandling"},
    {"_ZN4llvm16JITEmitDebugInfoE", "llvm::JITEmitDebugInfo"},
    {"_ZN4llvm22JITEmitDebugInfoToDiskE", "llvm::JITEmitDebugInfoToDisk"},
    {"_ZN4llvm21UnwindTablesMandatoryE", "llvm::UnwindTablesMandatory"},
    {"_ZN4llvm15RelocationModelE", "llvm::RelocationModel"},
    {"_ZN4llvm7CMModelE", "llvm::CMModel"},
    {"_ZN4llvm21GuaranteedTailCallOptE", "llvm::GuaranteedTailCallOpt"},
    {"_ZN4llvm14StackAlignmentE", "llvm::StackAlignment"},
    {"_ZN4llvm12RealignStackE", "llvm::RealignStack"},
    {"_ZN4llvm17DisableJumpTablesE", "llvm::DisableJumpTables"},
    {"_ZN4llvm13StrongPHIElimE", "llvm::StrongPHIElim"},
    {"_ZN4llvm19AsmVerbosityDefaultE", "llvm::AsmVerbosityDefault"},
    {"_ZN4llvm13DominatorTree2IDE", "llvm::DominatorTree::ID"},
    {"_ZN4llvm23FunctionPassManagerImpl2IDE", "llvm::FunctionPassManagerImpl::ID"},
    {"_ZN4llvm13MPPassManager2IDE", "llvm::MPPassManager::ID"},
    {"_ZN4llvm15PassManagerImpl2IDE", "llvm::PassManagerImpl::ID"},
    {"_ZN4llvm13FPPassManager2IDE", "llvm::FPPassManager::ID"},
    {"_ZN4llvm19TimePassesIsEnabledE", "llvm::TimePassesIsEnabled"},
    {"_ZN4llvm16BumpPtrAllocator20DefaultSlabAllocatorE", "llvm::BumpPtrAllocator::DefaultSlabAllocator"},
    {"_ZN4llvm20EnableDebugBufferingE", "llvm::EnableDebugBuffering"},
    {"_ZN4llvm23DisablePrettyStackTraceE", "llvm::DisablePrettyStackTrace"},
    {"_ZN4llvm3sys9TimeValue8ZeroTimeE", "llvm::sys::TimeValue::ZeroTime"},
    {"_ZNKSs13find_first_ofEPKcmm", "std::string::find_first_of(char const*, unsigned long, unsigned long) const"},
    {"_ZNKSs4copyEPcmm", "std::string::copy(char*, unsigned long, unsigned long) const"},
    {"_ZNKSs4findEPKcmm", "std::string::find(char const*, unsigned long, unsigned long) const"},
    {"_ZNKSs4findEcm", "std::string::find(char, unsigned long) const"},
    {"_ZNKSs5rfindEPKcmm", "std::string::rfind(char const*, unsigned long, unsigned long) const"},
    {"_ZNKSs5rfindEcm", "std::string::rfind(char, unsigned long) const"},
    {"_ZNKSs7compareEPKc", "std::string::compare(char const*) const"},
    {"_ZNKSs7compareEmmPKc", "std::string::compare(unsigned long, unsigned long, char const*) const"},
    {"_ZNKSs7compareEmmPKcm", "std::string::compare(unsigned long, unsigned long, char const*, unsigned long) const"},
    {"_ZNKSt9basic_iosIcSt11char_traitsIcEE5widenEc", "std::basic_ios<char, std::char_traits<char> >::widen(char) const"},
    {"_ZNSo3putEc", "std::ostream::put(char)"},
    {"_ZNSo5flushEv", "std::ostream::flush()"},
    {"_ZNSs12_M_leak_hardEv", "std::string::_M_leak_hard()"},
    {"_ZNSs14_M_replace_auxEmmmc", "std::string::_M_replace_aux(unsigned long, unsigned long, unsigned long, char)"},
    {"_ZNSs4_Rep10_M_destroyERKSaIcE", "std::string::_Rep::_M_destroy(std::allocator<char> const&)"},
    {"_ZNSs4_Rep11_S_terminalE", "std::string::_Rep::_S_terminal"},
    {"_ZNSs4_Rep20_S_empty_rep_storageE", "std::string::_Rep::_S_empty_rep_storage"},
    {"_ZNSs4_Rep9_S_createEmmRKSaIcE", "std::string::_Rep::_S_create(unsigned long, unsigned long, std::allocator<char> const&)"},
    {"_ZNSs4swapERSs", "std::string::swap(std::string&)"},
    {"_ZNSs6appendEPKcm", "std::string::append(char const*, unsigned long)"},
    {"_ZNSs6appendERKSs", "std::string::append(std::string const&)"},
    {"_ZNSs6appendEmc", "std::string::append(unsigned long, char)"},
    {"_ZNSs6assignEPKcm", "std::string::assign(char const*, unsigned long)"},
    {"_ZNSs6assignERKSs", "std::string::assign(std::string const&)"},
    {"_ZNSs6resizeEmc", "std::string::resize(unsigned long, char)"},
    {"_ZNSs7replaceEmmPKcm", "std::string::replace(unsigned long, unsigned long, char const*, unsigned long)"},
    {"_ZNSs7reserveEm", "std::string::reserve(unsigned long)"},
    {"_ZNSs9_M_mutateEmmm", "std::string::_M_mutate(unsigned long, unsigned long, unsigned long)"},
    {"_ZNSsC1EPKcRKSaIcE", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(char const*, std::allocator<char> const&)"},
    {"_ZNSsC1EPKcmRKSaIcE", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(char const*, unsigned long, std::allocator<char> const&)"},
    {"_ZNSsC1ERKSs", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(std::string const&)"},
    {"_ZNSsC1ERKSsmm", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(std::string const&, unsigned long, unsigned long)"},
    {"_ZNSsC1EmcRKSaIcE", "std::basic_string<char, std::char_traits<char>, std::allocator<char> >::basic_string(unsigned long, char, std::allocator<char> const&)"},
    {"_ZNSsixEm", "std::string::operator[](unsigned long)"},
    {"_ZNSspLEPKc", "std::string::operator+=(char const*)"},
    {"_ZNSspLEc", "std::string::operator+=(char)"},
    {"_ZNSt15_List_node_base4hookEPS_", "std::_List_node_base::hook(std::_List_node_base*)"},
    {"_ZNSt15_List_node_base4swapERS_S0_", "std::_List_node_base::swap(std::_List_node_base&, std::_List_node_base&)"},
    {"_ZNSt15_List_node_base6unhookEv", "std::_List_node_base::unhook()"},
    {"_ZNSt8ios_base4InitC1Ev", "std::ios_base::Init::Init()"},
    {"_ZNSt8ios_base4InitD1Ev", "std::ios_base::Init::~Init()"},
    {"_ZSt16__ostream_insertIcSt11char_traitsIcEERSt13basic_ostreamIT_T0_ES6_PKS3_l", "std::basic_ostream<char, std::char_traits<char> >& std::__ostream_insert<char, std::char_traits<char> >(std::basic_ostream<char, std::char_traits<char> >&, char const*, long)"},
    {"_ZSt17__throw_bad_allocv", "std::__throw_bad_alloc()"},
    {"_ZSt18_Rb_tree_decrementPKSt18_Rb_tree_node_base", "std::_Rb_tree_decrement(std::_Rb_tree_node_base const*)"},
    {"_ZSt18_Rb_tree_decrementPSt18_Rb_tree_node_base", "std::_Rb_tree_decrement(std::_Rb_tree_node_base*)"},
    {"_ZSt18_Rb_tree_incrementPKSt18_Rb_tree_node_base", "std::_Rb_tree_increment(std::_Rb_tree_node_base const*)"},
    {"_ZSt18_Rb_tree_incrementPSt18_Rb_tree_node_base", "std::_Rb_tree_increment(std::_Rb_tree_node_base*)"},
    {"_ZSt19__throw_logic_errorPKc", "std::__throw_logic_error(char const*)"},
    {"_ZSt20__throw_length_errorPKc", "std::__throw_length_error(char const*)"},
    {"_ZSt20__throw_out_of_rangePKc", "std::__throw_out_of_range(char const*)"},
    {"_ZSt28_Rb_tree_rebalance_for_erasePSt18_Rb_tree_node_baseRS_", "std::_Rb_tree_rebalance_for_erase(std::_Rb_tree_node_base*, std::_Rb_tree_node_base&)"},
    {"_ZSt29_Rb_tree_insert_and_rebalancebPSt18_Rb_tree_node_baseS0_RS_", "std::_Rb_tree_insert_and_rebalance(bool, std::_Rb_tree_node_base*, std::_Rb_tree_node_base*, std::_Rb_tree_node_base&)"},
    {"_ZSt4cout", "std::cout"},
    {"_ZSt7nothrow", "std::nothrow"},
    {"_ZTVN10__cxxabiv117__class_type_infoE", "vtable for __cxxabiv1::__class_type_info"},
    {"_ZTVN10__cxxabiv120__si_class_type_infoE", "vtable for __cxxabiv1::__si_class_type_info"},
    {"_ZTVN10__cxxabiv121__vmi_class_type_infoE", "vtable for __cxxabiv1::__vmi_class_type_info"},
    {"_ZdaPv", "operator delete[](void*)"},
    {"_ZdlPv", "operator delete(void*)"},
    {"_ZdlPvRKSt9nothrow_t", "operator delete(void*, std::nothrow_t const&)"},
    {"_Znam", "operator new[](unsigned long)"},
    {"_Znwm", "operator new(unsigned long)"},
    {"_ZnwmRKSt9nothrow_t", "operator new(unsigned long, std::nothrow_t const&)"},

    // The following pairs are not supported by gcc's __cxa_demangle

    {"_ZStL8__ioinit", "std::__ioinit"},
    {"_ZL4NaNU", "NaNU"},
    {"_ZL4NaNL", "NaNL"},
    {"_ZL8CharInfo", "CharInfo"},
    {"_ZL13DiagSubGroup0", "DiagSubGroup0"},
    {"_ZL10DiagArray1", "DiagArray1"},
    {"_ZL10DiagArray2", "DiagArray2"},
    {"_ZL10DiagArray5", "DiagArray5"},
    {"_ZL13DiagSubGroup7", "DiagSubGroup7"},
    {"_ZL10DiagArray8", "DiagArray8"},
    {"_ZL13DiagSubGroup9", "DiagSubGroup9"},
    {"_ZL11DiagArray12", "DiagArray12"},
    {"_ZL11DiagArray13", "DiagArray13"},
    {"_ZL11DiagArray14", "DiagArray14"},
    {"_ZL11DiagArray15", "DiagArray15"},
    {"_ZL11DiagArray16", "DiagArray16"},
    {"_ZL11DiagArray17", "DiagArray17"},
    {"_ZL14DiagSubGroup17", "DiagSubGroup17"},
    {"_ZL11DiagArray18", "DiagArray18"},
    {"_ZL11DiagArray19", "DiagArray19"},
    {"_ZL11DiagArray22", "DiagArray22"},
    {"_ZL11DiagArray23", "DiagArray23"},
    {"_ZL14DiagSubGroup24", "DiagSubGroup24"},
    {"_ZL11DiagArray25", "DiagArray25"},
    {"_ZL11DiagArray26", "DiagArray26"},
    {"_ZL11DiagArray27", "DiagArray27"},
    {"_ZL14DiagSubGroup27", "DiagSubGroup27"},
    {"_ZL11DiagArray29", "DiagArray29"},
    {"_ZL11DiagArray30", "DiagArray30"},
    {"_ZL11DiagArray31", "DiagArray31"},
    {"_ZL14DiagSubGroup31", "DiagSubGroup31"},
    {"_ZL11DiagArray32", "DiagArray32"},
    {"_ZL11DiagArray33", "DiagArray33"},
    {"_ZL11DiagArray38", "DiagArray38"},
    {"_ZL14DiagSubGroup39", "DiagSubGroup39"},
    {"_ZL14DiagSubGroup40", "DiagSubGroup40"},
    {"_ZL11DiagArray41", "DiagArray41"},
    {"_ZL11DiagArray42", "DiagArray42"},
    {"_ZL11DiagArray43", "DiagArray43"},
    {"_ZL14DiagSubGroup43", "DiagSubGroup43"},
    {"_ZL11DiagArray44", "DiagArray44"},
    {"_ZL11DiagArray45", "DiagArray45"},
    {"_ZL14DiagSubGroup45", "DiagSubGroup45"},
    {"_ZL11DiagArray46", "DiagArray46"},
    {"_ZL14DiagSubGroup46", "DiagSubGroup46"},
    {"_ZL14DiagSubGroup47", "DiagSubGroup47"},
    {"_ZL11DiagArray48", "DiagArray48"},
    {"_ZL14DiagSubGroup49", "DiagSubGroup49"},
    {"_ZL11DiagArray50", "DiagArray50"},
    {"_ZL11DiagArray51", "DiagArray51"},
    {"_ZL11DiagArray52", "DiagArray52"},
    {"_ZL14DiagSubGroup52", "DiagSubGroup52"},
    {"_ZL11DiagArray53", "DiagArray53"},
    {"_ZL11DiagArray54", "DiagArray54"},
    {"_ZL11DiagArray55", "DiagArray55"},
    {"_ZL14DiagSubGroup56", "DiagSubGroup56"},
    {"_ZL11DiagArray57", "DiagArray57"},
    {"_ZL11DiagArray58", "DiagArray58"},
    {"_ZL11DiagArray59", "DiagArray59"},
    {"_ZL11DiagArray62", "DiagArray62"},
    {"_ZL11DiagArray65", "DiagArray65"},
    {"_ZL11DiagArray66", "DiagArray66"},
    {"_ZL11DiagArray68", "DiagArray68"},
    {"_ZL11DiagArray69", "DiagArray69"},
    {"_ZL11DiagArray70", "DiagArray70"},
    {"_ZL11DiagArray71", "DiagArray71"},
    {"_ZL11DiagArray72", "DiagArray72"},
    {"_ZL11DiagArray73", "DiagArray73"},
    {"_ZL11DiagArray75", "DiagArray75"},
    {"_ZL11DiagArray76", "DiagArray76"},
    {"_ZL11DiagArray77", "DiagArray77"},
    {"_ZL11DiagArray78", "DiagArray78"},
    {"_ZL11DiagArray79", "DiagArray79"},
    {"_ZL11DiagArray80", "DiagArray80"},
    {"_ZL11DiagArray83", "DiagArray83"},
    {"_ZL11DiagArray84", "DiagArray84"},
    {"_ZL14DiagSubGroup85", "DiagSubGroup85"},
    {"_ZL11DiagArray86", "DiagArray86"},
    {"_ZL14DiagSubGroup89", "DiagSubGroup89"},
    {"_ZL11DiagArray90", "DiagArray90"},
    {"_ZL11DiagArray91", "DiagArray91"},
    {"_ZL11DiagArray92", "DiagArray92"},
    {"_ZL11DiagArray93", "DiagArray93"},
    {"_ZL11DiagArray95", "DiagArray95"},
    {"_ZL11DiagArray98", "DiagArray98"},
    {"_ZL12DiagArray100", "DiagArray100"},
    {"_ZL12DiagArray101", "DiagArray101"},
    {"_ZL12DiagArray102", "DiagArray102"},
    {"_ZL12DiagArray103", "DiagArray103"},
    {"_ZL12DiagArray104", "DiagArray104"},
    {"_ZL15DiagSubGroup104", "DiagSubGroup104"},
    {"_ZL12DiagArray105", "DiagArray105"},
    {"_ZL12DiagArray106", "DiagArray106"},
    {"_ZL12DiagArray108", "DiagArray108"},
    {"_ZL12DiagArray109", "DiagArray109"},
    {"_ZL12DiagArray111", "DiagArray111"},
    {"_ZL12DiagArray112", "DiagArray112"},
    {"_ZL12DiagArray113", "DiagArray113"},
    {"_ZL12DiagArray114", "DiagArray114"},
    {"_ZL12DiagArray115", "DiagArray115"},
    {"_ZL12DiagArray117", "DiagArray117"},
    {"_ZL12DiagArray118", "DiagArray118"},
    {"_ZL12DiagArray119", "DiagArray119"},
    {"_ZL12DiagArray134", "DiagArray134"},
    {"_ZL12DiagArray135", "DiagArray135"},
    {"_ZL12DiagArray136", "DiagArray136"},
    {"_ZL15DiagSubGroup136", "DiagSubGroup136"},
    {"_ZL12DiagArray138", "DiagArray138"},
    {"_ZL12DiagArray140", "DiagArray140"},
    {"_ZL12DiagArray141", "DiagArray141"},
    {"_ZL12DiagArray143", "DiagArray143"},
    {"_ZL12DiagArray144", "DiagArray144"},
    {"_ZL12DiagArray145", "DiagArray145"},
    {"_ZL12DiagArray146", "DiagArray146"},
    {"_ZL12DiagArray147", "DiagArray147"},
    {"_ZL12DiagArray148", "DiagArray148"},
    {"_ZL12DiagArray149", "DiagArray149"},
    {"_ZL12DiagArray150", "DiagArray150"},
    {"_ZL12DiagArray151", "DiagArray151"},
    {"_ZL12DiagArray152", "DiagArray152"},
    {"_ZL15DiagSubGroup153", "DiagSubGroup153"},
    {"_ZL12DiagArray155", "DiagArray155"},
    {"_ZL12DiagArray156", "DiagArray156"},
    {"_ZL12DiagArray157", "DiagArray157"},
    {"_ZL12DiagArray158", "DiagArray158"},
    {"_ZL12DiagArray159", "DiagArray159"},
    {"_ZL12DiagArray160", "DiagArray160"},
    {"_ZL12DiagArray161", "DiagArray161"},
    {"_ZL12DiagArray162", "DiagArray162"},
    {"_ZL12DiagArray163", "DiagArray163"},
    {"_ZL12DiagArray164", "DiagArray164"},
    {"_ZL12DiagArray165", "DiagArray165"},
    {"_ZL12DiagArray166", "DiagArray166"},
    {"_ZL12DiagArray168", "DiagArray168"},
    {"_ZN4llvmL13ImplicitList1E", "llvm::ImplicitList1"},
    {"_ZN4llvmL13ImplicitList2E", "llvm::ImplicitList2"},
    {"_ZN4llvmL13ImplicitList3E", "llvm::ImplicitList3"},
    {"_ZN4llvmL13ImplicitList4E", "llvm::ImplicitList4"},
    {"_ZN4llvmL13ImplicitList5E", "llvm::ImplicitList5"},
    {"_ZN4llvmL13ImplicitList6E", "llvm::ImplicitList6"},
    {"_ZN4llvmL13ImplicitList7E", "llvm::ImplicitList7"},
    {"_ZN4llvmL13ImplicitList8E", "llvm::ImplicitList8"},
    {"_ZN4llvmL13ImplicitList9E", "llvm::ImplicitList9"},
    {"_ZN4llvmL14ImplicitList10E", "llvm::ImplicitList10"},
    {"_ZN4llvmL14ImplicitList11E", "llvm::ImplicitList11"},
    {"_ZN4llvmL14ImplicitList12E", "llvm::ImplicitList12"},
    {"_ZN4llvmL14ImplicitList13E", "llvm::ImplicitList13"},
    {"_ZN4llvmL14ImplicitList14E", "llvm::ImplicitList14"},
    {"_ZN4llvmL14ImplicitList15E", "llvm::ImplicitList15"},
    {"_ZN4llvmL14ImplicitList16E", "llvm::ImplicitList16"},
    {"_ZN4llvmL14ImplicitList17E", "llvm::ImplicitList17"},
    {"_ZN4llvmL14ImplicitList18E", "llvm::ImplicitList18"},
    {"_ZN4llvmL14ImplicitList19E", "llvm::ImplicitList19"},
    {"_ZN4llvmL14ImplicitList20E", "llvm::ImplicitList20"},
    {"_ZN4llvmL14ImplicitList21E", "llvm::ImplicitList21"},
    {"_ZN4llvmL14ImplicitList22E", "llvm::ImplicitList22"},
    {"_ZN4llvmL14ImplicitList23E", "llvm::ImplicitList23"},
    {"_ZN4llvmL14ImplicitList24E", "llvm::ImplicitList24"},
    {"_ZN4llvmL14ImplicitList25E", "llvm::ImplicitList25"},
    {"_ZN4llvmL14ImplicitList26E", "llvm::ImplicitList26"},
    {"_ZN4llvmL14ImplicitList27E", "llvm::ImplicitList27"},
    {"_ZN4llvmL14ImplicitList28E", "llvm::ImplicitList28"},
    {"_ZN4llvmL14ImplicitList29E", "llvm::ImplicitList29"},
    {"_ZN4llvmL14ImplicitList30E", "llvm::ImplicitList30"},
    {"_ZN4llvmL14ImplicitList31E", "llvm::ImplicitList31"},
    {"_ZN4llvmL14ImplicitList32E", "llvm::ImplicitList32"},
    {"_ZN4llvmL14ImplicitList33E", "llvm::ImplicitList33"},
    {"_ZN4llvmL14ImplicitList34E", "llvm::ImplicitList34"},
    {"_ZN4llvmL14ImplicitList35E", "llvm::ImplicitList35"},
    {"_ZN4llvmL14ImplicitList36E", "llvm::ImplicitList36"},
    {"_ZN4llvmL14ImplicitList37E", "llvm::ImplicitList37"},
    {"_ZN4llvmL14ImplicitList38E", "llvm::ImplicitList38"},
    {"_ZN4llvmL14ImplicitList39E", "llvm::ImplicitList39"},
    {"_ZN4llvmL14ImplicitList40E", "llvm::ImplicitList40"},
    {"_ZN4llvmL14ImplicitList41E", "llvm::ImplicitList41"},
    {"_ZN4llvmL14ImplicitList42E", "llvm::ImplicitList42"},
    {"_ZN4llvmL14ImplicitList43E", "llvm::ImplicitList43"},
    {"_ZN4llvmL14ImplicitList44E", "llvm::ImplicitList44"},
    {"_ZN4llvmL14ImplicitList45E", "llvm::ImplicitList45"},
    {"_ZN4llvmL14ImplicitList46E", "llvm::ImplicitList46"},
    {"_ZN4llvmL14ImplicitList47E", "llvm::ImplicitList47"},
    {"_ZN4llvmL14ImplicitList48E", "llvm::ImplicitList48"},
    {"_ZN4llvmL14ImplicitList49E", "llvm::ImplicitList49"},
    {"_ZN4llvmL14ImplicitList50E", "llvm::ImplicitList50"},
    {"_ZN4llvmL14ImplicitList51E", "llvm::ImplicitList51"},
    {"_ZN4llvmL14ImplicitList52E", "llvm::ImplicitList52"},
    {"_ZN4llvmL14ImplicitList53E", "llvm::ImplicitList53"},
    {"_ZN4llvmL14ImplicitList54E", "llvm::ImplicitList54"},
    {"_ZN4llvmL14ImplicitList55E", "llvm::ImplicitList55"},
    {"_ZN4llvmL14ImplicitList56E", "llvm::ImplicitList56"},
    {"_ZN4llvmL14ImplicitList57E", "llvm::ImplicitList57"},
    {"_ZN4llvmL14ImplicitList58E", "llvm::ImplicitList58"},
    {"_ZN4llvmL14ImplicitList59E", "llvm::ImplicitList59"},
    {"_ZN4llvmL14ImplicitList60E", "llvm::ImplicitList60"},
    {"_ZN4llvmL14ImplicitList61E", "llvm::ImplicitList61"},
    {"_ZN4llvmL14ImplicitList62E", "llvm::ImplicitList62"},
    {"_ZN4llvmL14ImplicitList63E", "llvm::ImplicitList63"},
    {"_ZN4llvmL14ImplicitList64E", "llvm::ImplicitList64"},
    {"_ZN4llvmL14ImplicitList65E", "llvm::ImplicitList65"},
    {"_ZL17ReplaceableInstrs", "ReplaceableInstrs"},
    {"_ZN4llvm3X86L18CCRSuperRegClassesE", "llvm::X86::CCRSuperRegClasses"},
    {"_ZN4llvm3X86L26CONTROL_REGSuperRegClassesE", "llvm::X86::CONTROL_REGSuperRegClasses"},
    {"_ZN4llvm3X86L24DEBUG_REGSuperRegClassesE", "llvm::X86::DEBUG_REGSuperRegClasses"},
    {"_ZN4llvm3X86L22GR32_ADSuperRegClassesE", "llvm::X86::GR32_ADSuperRegClasses"},
    {"_ZN4llvm3X86L19GR64SuperRegClassesE", "llvm::X86::GR64SuperRegClasses"},
    {"_ZN4llvm3X86L24GR64_ABCDSuperRegClassesE", "llvm::X86::GR64_ABCDSuperRegClasses"},
    {"_ZN4llvm3X86L25GR64_NOREXSuperRegClassesE", "llvm::X86::GR64_NOREXSuperRegClasses"},
    {"_ZN4llvm3X86L30GR64_NOREX_NOSPSuperRegClassesE", "llvm::X86::GR64_NOREX_NOSPSuperRegClasses"},
    {"_ZN4llvm3X86L24GR64_NOSPSuperRegClassesE", "llvm::X86::GR64_NOSPSuperRegClasses"},
    {"_ZN4llvm3X86L22GR64_TCSuperRegClassesE", "llvm::X86::GR64_TCSuperRegClasses"},
    {"_ZN4llvm3X86L25GR64_TCW64SuperRegClassesE", "llvm::X86::GR64_TCW64SuperRegClasses"},
    {"_ZN4llvm3X86L20RFP32SuperRegClassesE", "llvm::X86::RFP32SuperRegClasses"},
    {"_ZN4llvm3X86L20RFP64SuperRegClassesE", "llvm::X86::RFP64SuperRegClasses"},
    {"_ZN4llvm3X86L20RFP80SuperRegClassesE", "llvm::X86::RFP80SuperRegClasses"},
    {"_ZN4llvm3X86L18RSTSuperRegClassesE", "llvm::X86::RSTSuperRegClasses"},
    {"_ZN4llvm3X86L26SEGMENT_REGSuperRegClassesE", "llvm::X86::SEGMENT_REGSuperRegClasses"},
    {"_ZN4llvm3X86L20VR256SuperRegClassesE", "llvm::X86::VR256SuperRegClasses"},
    {"_ZN4llvm3X86L19VR64SuperRegClassesE", "llvm::X86::VR64SuperRegClasses"},
    {"_ZN4llvm3X86L13CCRSubclassesE", "llvm::X86::CCRSubclasses"},
    {"_ZN4llvm3X86L21CONTROL_REGSubclassesE", "llvm::X86::CONTROL_REGSubclasses"},
    {"_ZN4llvm3X86L19DEBUG_REGSubclassesE", "llvm::X86::DEBUG_REGSubclasses"},
    {"_ZN4llvm3X86L19GR16_ABCDSubclassesE", "llvm::X86::GR16_ABCDSubclasses"},
    {"_ZN4llvm3X86L17GR32_ADSubclassesE", "llvm::X86::GR32_ADSubclasses"},
    {"_ZN4llvm3X86L19GR64_ABCDSubclassesE", "llvm::X86::GR64_ABCDSubclasses"},
    {"_ZN4llvm3X86L20GR64_TCW64SubclassesE", "llvm::X86::GR64_TCW64Subclasses"},
    {"_ZN4llvm3X86L20GR8_ABCD_HSubclassesE", "llvm::X86::GR8_ABCD_HSubclasses"},
    {"_ZN4llvm3X86L20GR8_ABCD_LSubclassesE", "llvm::X86::GR8_ABCD_LSubclasses"},
    {"_ZN4llvm3X86L15RFP80SubclassesE", "llvm::X86::RFP80Subclasses"},
    {"_ZN4llvm3X86L13RSTSubclassesE", "llvm::X86::RSTSubclasses"},
    {"_ZN4llvm3X86L21SEGMENT_REGSubclassesE", "llvm::X86::SEGMENT_REGSubclasses"},
    {"_ZN4llvm3X86L15VR128SubclassesE", "llvm::X86::VR128Subclasses"},
    {"_ZN4llvm3X86L15VR256SubclassesE", "llvm::X86::VR256Subclasses"},
    {"_ZN4llvm3X86L14VR64SubclassesE", "llvm::X86::VR64Subclasses"},
    {"_ZN4llvm3X86L15CCRSuperclassesE", "llvm::X86::CCRSuperclasses"},
    {"_ZN4llvm3X86L23CONTROL_REGSuperclassesE", "llvm::X86::CONTROL_REGSuperclasses"},
    {"_ZN4llvm3X86L21DEBUG_REGSuperclassesE", "llvm::X86::DEBUG_REGSuperclasses"},
    {"_ZN4llvm3X86L16FR32SuperclassesE", "llvm::X86::FR32Superclasses"},
    {"_ZN4llvm3X86L16GR16SuperclassesE", "llvm::X86::GR16Superclasses"},
    {"_ZN4llvm3X86L16GR32SuperclassesE", "llvm::X86::GR32Superclasses"},
    {"_ZN4llvm3X86L16GR64SuperclassesE", "llvm::X86::GR64Superclasses"},
    {"_ZN4llvm3X86L15GR8SuperclassesE", "llvm::X86::GR8Superclasses"},
    {"_ZN4llvm3X86L17RFP32SuperclassesE", "llvm::X86::RFP32Superclasses"},
    {"_ZN4llvm3X86L15RSTSuperclassesE", "llvm::X86::RSTSuperclasses"},
    {"_ZN4llvm3X86L23SEGMENT_REGSuperclassesE", "llvm::X86::SEGMENT_REGSuperclasses"},
    {"_ZN4llvm3X86L17VR256SuperclassesE", "llvm::X86::VR256Superclasses"},
    {"_ZN4llvm3X86L16VR64SuperclassesE", "llvm::X86::VR64Superclasses"},
    {"_ZN4llvm3X86L16CCRSubRegClassesE", "llvm::X86::CCRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3CCRE", "llvm::(anonymous namespace)::CCR"},
    {"_ZN4llvm3X86L24CONTROL_REGSubRegClassesE", "llvm::X86::CONTROL_REGSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L11CONTROL_REGE", "llvm::(anonymous namespace)::CONTROL_REG"},
    {"_ZN4llvm3X86L22DEBUG_REGSubRegClassesE", "llvm::X86::DEBUG_REGSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L9DEBUG_REGE", "llvm::(anonymous namespace)::DEBUG_REG"},
    {"_ZN4llvm3X86L17FR32SubRegClassesE", "llvm::X86::FR32SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L4FR32E", "llvm::(anonymous namespace)::FR32"},
    {"_ZN4llvm3X86L17FR64SubRegClassesE", "llvm::X86::FR64SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L4FR64E", "llvm::(anonymous namespace)::FR64"},
    {"_ZN4llvm3X86L14X86_GR16_AO_64E", "llvm::X86::X86_GR16_AO_64"},
    {"_ZN4llvm12_GLOBAL__N_1L4GR16E", "llvm::(anonymous namespace)::GR16"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR16_ABCDE", "llvm::(anonymous namespace)::GR16_ABCD"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR16_NOREXE", "llvm::(anonymous namespace)::GR16_NOREX"},
    {"_ZN4llvm3X86L14X86_GR32_AO_64E", "llvm::X86::X86_GR32_AO_64"},
    {"_ZN4llvm12_GLOBAL__N_1L4GR32E", "llvm::(anonymous namespace)::GR32"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR32_ABCDE", "llvm::(anonymous namespace)::GR32_ABCD"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR32_ADE", "llvm::(anonymous namespace)::GR32_AD"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR32_NOREXE", "llvm::(anonymous namespace)::GR32_NOREX"},
    {"_ZN4llvm3X86L19X86_GR32_NOSP_AO_64E", "llvm::X86::X86_GR32_NOSP_AO_64"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR32_NOSPE", "llvm::(anonymous namespace)::GR32_NOSP"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR32_TCE", "llvm::(anonymous namespace)::GR32_TC"},
    {"_ZN4llvm12_GLOBAL__N_1L4GR64E", "llvm::(anonymous namespace)::GR64"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR64_ABCDE", "llvm::(anonymous namespace)::GR64_ABCD"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR64_NOREXE", "llvm::(anonymous namespace)::GR64_NOREX"},
    {"_ZN4llvm12_GLOBAL__N_1L15GR64_NOREX_NOSPE", "llvm::(anonymous namespace)::GR64_NOREX_NOSP"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR64_NOSPE", "llvm::(anonymous namespace)::GR64_NOSP"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR64_TCE", "llvm::(anonymous namespace)::GR64_TC"},
    {"_ZN4llvm3X86L23GR64_TCW64SubRegClassesE", "llvm::X86::GR64_TCW64SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR64_TCW64E", "llvm::(anonymous namespace)::GR64_TCW64"},
    {"_ZN4llvm3X86L13X86_GR8_AO_64E", "llvm::X86::X86_GR8_AO_64"},
    {"_ZN4llvm3X86L16GR8SubRegClassesE", "llvm::X86::GR8SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3GR8E", "llvm::(anonymous namespace)::GR8"},
    {"_ZN4llvm3X86L23GR8_ABCD_HSubRegClassesE", "llvm::X86::GR8_ABCD_HSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR8_ABCD_HE", "llvm::(anonymous namespace)::GR8_ABCD_H"},
    {"_ZN4llvm3X86L23GR8_ABCD_LSubRegClassesE", "llvm::X86::GR8_ABCD_LSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR8_ABCD_LE", "llvm::(anonymous namespace)::GR8_ABCD_L"},
    {"_ZN4llvm3X86L19X86_GR8_NOREX_AO_64E", "llvm::X86::X86_GR8_NOREX_AO_64"},
    {"_ZN4llvm3X86L22GR8_NOREXSubRegClassesE", "llvm::X86::GR8_NOREXSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L9GR8_NOREXE", "llvm::(anonymous namespace)::GR8_NOREX"},
    {"_ZN4llvm3X86L18RFP32SubRegClassesE", "llvm::X86::RFP32SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L5RFP32E", "llvm::(anonymous namespace)::RFP32"},
    {"_ZN4llvm3X86L18RFP64SubRegClassesE", "llvm::X86::RFP64SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L5RFP64E", "llvm::(anonymous namespace)::RFP64"},
    {"_ZN4llvm3X86L18RFP80SubRegClassesE", "llvm::X86::RFP80SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L5RFP80E", "llvm::(anonymous namespace)::RFP80"},
    {"_ZN4llvm3X86L16RSTSubRegClassesE", "llvm::X86::RSTSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3RSTE", "llvm::(anonymous namespace)::RST"},
    {"_ZN4llvm3X86L24SEGMENT_REGSubRegClassesE", "llvm::X86::SEGMENT_REGSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L11SEGMENT_REGE", "llvm::(anonymous namespace)::SEGMENT_REG"},
    {"_ZN4llvm12_GLOBAL__N_1L5VR128E", "llvm::(anonymous namespace)::VR128"},
    {"_ZN4llvm12_GLOBAL__N_1L5VR256E", "llvm::(anonymous namespace)::VR256"},
    {"_ZN4llvm3X86L17VR64SubRegClassesE", "llvm::X86::VR64SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L4VR64E", "llvm::(anonymous namespace)::VR64"},
    {"_ZN4llvm12_GLOBAL__N_1L15SubregHashTableE", "llvm::(anonymous namespace)::SubregHashTable"},
    {"_ZN4llvm12_GLOBAL__N_1L16AliasesHashTableE", "llvm::(anonymous namespace)::AliasesHashTable"},
    {"_ZN4llvm12_GLOBAL__N_1L11AH_OverlapsE", "llvm::(anonymous namespace)::AH_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16Empty_SubRegsSetE", "llvm::(anonymous namespace)::Empty_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15AH_SuperRegsSetE", "llvm::(anonymous namespace)::AH_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11AL_OverlapsE", "llvm::(anonymous namespace)::AL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15AL_SuperRegsSetE", "llvm::(anonymous namespace)::AL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11AX_OverlapsE", "llvm::(anonymous namespace)::AX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13AX_SubRegsSetE", "llvm::(anonymous namespace)::AX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15AX_SuperRegsSetE", "llvm::(anonymous namespace)::AX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11BH_OverlapsE", "llvm::(anonymous namespace)::BH_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15BH_SuperRegsSetE", "llvm::(anonymous namespace)::BH_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11BL_OverlapsE", "llvm::(anonymous namespace)::BL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15BL_SuperRegsSetE", "llvm::(anonymous namespace)::BL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11BP_OverlapsE", "llvm::(anonymous namespace)::BP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13BP_SubRegsSetE", "llvm::(anonymous namespace)::BP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15BP_SuperRegsSetE", "llvm::(anonymous namespace)::BP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12BPL_OverlapsE", "llvm::(anonymous namespace)::BPL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16BPL_SuperRegsSetE", "llvm::(anonymous namespace)::BPL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11BX_OverlapsE", "llvm::(anonymous namespace)::BX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13BX_SubRegsSetE", "llvm::(anonymous namespace)::BX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15BX_SuperRegsSetE", "llvm::(anonymous namespace)::BX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11CH_OverlapsE", "llvm::(anonymous namespace)::CH_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15CH_SuperRegsSetE", "llvm::(anonymous namespace)::CH_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11CL_OverlapsE", "llvm::(anonymous namespace)::CL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15CL_SuperRegsSetE", "llvm::(anonymous namespace)::CL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR0_OverlapsE", "llvm::(anonymous namespace)::CR0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18Empty_SuperRegsSetE", "llvm::(anonymous namespace)::Empty_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR1_OverlapsE", "llvm::(anonymous namespace)::CR1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR2_OverlapsE", "llvm::(anonymous namespace)::CR2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR3_OverlapsE", "llvm::(anonymous namespace)::CR3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR4_OverlapsE", "llvm::(anonymous namespace)::CR4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR5_OverlapsE", "llvm::(anonymous namespace)::CR5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR6_OverlapsE", "llvm::(anonymous namespace)::CR6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR7_OverlapsE", "llvm::(anonymous namespace)::CR7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR8_OverlapsE", "llvm::(anonymous namespace)::CR8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12CR9_OverlapsE", "llvm::(anonymous namespace)::CR9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR10_OverlapsE", "llvm::(anonymous namespace)::CR10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR11_OverlapsE", "llvm::(anonymous namespace)::CR11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR12_OverlapsE", "llvm::(anonymous namespace)::CR12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR13_OverlapsE", "llvm::(anonymous namespace)::CR13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR14_OverlapsE", "llvm::(anonymous namespace)::CR14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CR15_OverlapsE", "llvm::(anonymous namespace)::CR15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11CS_OverlapsE", "llvm::(anonymous namespace)::CS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11CX_OverlapsE", "llvm::(anonymous namespace)::CX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13CX_SubRegsSetE", "llvm::(anonymous namespace)::CX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15CX_SuperRegsSetE", "llvm::(anonymous namespace)::CX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11DH_OverlapsE", "llvm::(anonymous namespace)::DH_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15DH_SuperRegsSetE", "llvm::(anonymous namespace)::DH_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11DI_OverlapsE", "llvm::(anonymous namespace)::DI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13DI_SubRegsSetE", "llvm::(anonymous namespace)::DI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15DI_SuperRegsSetE", "llvm::(anonymous namespace)::DI_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12DIL_OverlapsE", "llvm::(anonymous namespace)::DIL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16DIL_SuperRegsSetE", "llvm::(anonymous namespace)::DIL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11DL_OverlapsE", "llvm::(anonymous namespace)::DL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15DL_SuperRegsSetE", "llvm::(anonymous namespace)::DL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR0_OverlapsE", "llvm::(anonymous namespace)::DR0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR1_OverlapsE", "llvm::(anonymous namespace)::DR1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR2_OverlapsE", "llvm::(anonymous namespace)::DR2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR3_OverlapsE", "llvm::(anonymous namespace)::DR3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR4_OverlapsE", "llvm::(anonymous namespace)::DR4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR5_OverlapsE", "llvm::(anonymous namespace)::DR5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR6_OverlapsE", "llvm::(anonymous namespace)::DR6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12DR7_OverlapsE", "llvm::(anonymous namespace)::DR7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11DS_OverlapsE", "llvm::(anonymous namespace)::DS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11DX_OverlapsE", "llvm::(anonymous namespace)::DX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13DX_SubRegsSetE", "llvm::(anonymous namespace)::DX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15DX_SuperRegsSetE", "llvm::(anonymous namespace)::DX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EAX_OverlapsE", "llvm::(anonymous namespace)::EAX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EAX_SubRegsSetE", "llvm::(anonymous namespace)::EAX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EAX_SuperRegsSetE", "llvm::(anonymous namespace)::EAX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EBP_OverlapsE", "llvm::(anonymous namespace)::EBP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EBP_SubRegsSetE", "llvm::(anonymous namespace)::EBP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EBP_SuperRegsSetE", "llvm::(anonymous namespace)::EBP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EBX_OverlapsE", "llvm::(anonymous namespace)::EBX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EBX_SubRegsSetE", "llvm::(anonymous namespace)::EBX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EBX_SuperRegsSetE", "llvm::(anonymous namespace)::EBX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12ECX_OverlapsE", "llvm::(anonymous namespace)::ECX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14ECX_SubRegsSetE", "llvm::(anonymous namespace)::ECX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16ECX_SuperRegsSetE", "llvm::(anonymous namespace)::ECX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EDI_OverlapsE", "llvm::(anonymous namespace)::EDI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EDI_SubRegsSetE", "llvm::(anonymous namespace)::EDI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EDI_SuperRegsSetE", "llvm::(anonymous namespace)::EDI_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EDX_OverlapsE", "llvm::(anonymous namespace)::EDX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EDX_SubRegsSetE", "llvm::(anonymous namespace)::EDX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EDX_SuperRegsSetE", "llvm::(anonymous namespace)::EDX_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15EFLAGS_OverlapsE", "llvm::(anonymous namespace)::EFLAGS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12EIP_OverlapsE", "llvm::(anonymous namespace)::EIP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14EIP_SubRegsSetE", "llvm::(anonymous namespace)::EIP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16EIP_SuperRegsSetE", "llvm::(anonymous namespace)::EIP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12EIZ_OverlapsE", "llvm::(anonymous namespace)::EIZ_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11ES_OverlapsE", "llvm::(anonymous namespace)::ES_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ESI_OverlapsE", "llvm::(anonymous namespace)::ESI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14ESI_SubRegsSetE", "llvm::(anonymous namespace)::ESI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16ESI_SuperRegsSetE", "llvm::(anonymous namespace)::ESI_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12ESP_OverlapsE", "llvm::(anonymous namespace)::ESP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14ESP_SubRegsSetE", "llvm::(anonymous namespace)::ESP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16ESP_SuperRegsSetE", "llvm::(anonymous namespace)::ESP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP0_OverlapsE", "llvm::(anonymous namespace)::FP0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP1_OverlapsE", "llvm::(anonymous namespace)::FP1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP2_OverlapsE", "llvm::(anonymous namespace)::FP2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP3_OverlapsE", "llvm::(anonymous namespace)::FP3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP4_OverlapsE", "llvm::(anonymous namespace)::FP4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP5_OverlapsE", "llvm::(anonymous namespace)::FP5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12FP6_OverlapsE", "llvm::(anonymous namespace)::FP6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11FS_OverlapsE", "llvm::(anonymous namespace)::FS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11GS_OverlapsE", "llvm::(anonymous namespace)::GS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11IP_OverlapsE", "llvm::(anonymous namespace)::IP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15IP_SuperRegsSetE", "llvm::(anonymous namespace)::IP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM0_OverlapsE", "llvm::(anonymous namespace)::MM0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM1_OverlapsE", "llvm::(anonymous namespace)::MM1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM2_OverlapsE", "llvm::(anonymous namespace)::MM2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM3_OverlapsE", "llvm::(anonymous namespace)::MM3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM4_OverlapsE", "llvm::(anonymous namespace)::MM4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM5_OverlapsE", "llvm::(anonymous namespace)::MM5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM6_OverlapsE", "llvm::(anonymous namespace)::MM6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12MM7_OverlapsE", "llvm::(anonymous namespace)::MM7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R8_OverlapsE", "llvm::(anonymous namespace)::R8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13R8_SubRegsSetE", "llvm::(anonymous namespace)::R8_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R8B_OverlapsE", "llvm::(anonymous namespace)::R8B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16R8B_SuperRegsSetE", "llvm::(anonymous namespace)::R8B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R8D_OverlapsE", "llvm::(anonymous namespace)::R8D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R8D_SubRegsSetE", "llvm::(anonymous namespace)::R8D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16R8D_SuperRegsSetE", "llvm::(anonymous namespace)::R8D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R8W_OverlapsE", "llvm::(anonymous namespace)::R8W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R8W_SubRegsSetE", "llvm::(anonymous namespace)::R8W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16R8W_SuperRegsSetE", "llvm::(anonymous namespace)::R8W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11R9_OverlapsE", "llvm::(anonymous namespace)::R9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13R9_SubRegsSetE", "llvm::(anonymous namespace)::R9_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R9B_OverlapsE", "llvm::(anonymous namespace)::R9B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16R9B_SuperRegsSetE", "llvm::(anonymous namespace)::R9B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R9D_OverlapsE", "llvm::(anonymous namespace)::R9D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R9D_SubRegsSetE", "llvm::(anonymous namespace)::R9D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16R9D_SuperRegsSetE", "llvm::(anonymous namespace)::R9D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R9W_OverlapsE", "llvm::(anonymous namespace)::R9W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R9W_SubRegsSetE", "llvm::(anonymous namespace)::R9W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16R9W_SuperRegsSetE", "llvm::(anonymous namespace)::R9W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R10_OverlapsE", "llvm::(anonymous namespace)::R10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R10_SubRegsSetE", "llvm::(anonymous namespace)::R10_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R10B_OverlapsE", "llvm::(anonymous namespace)::R10B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R10B_SuperRegsSetE", "llvm::(anonymous namespace)::R10B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R10D_OverlapsE", "llvm::(anonymous namespace)::R10D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R10D_SubRegsSetE", "llvm::(anonymous namespace)::R10D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R10D_SuperRegsSetE", "llvm::(anonymous namespace)::R10D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R10W_OverlapsE", "llvm::(anonymous namespace)::R10W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R10W_SubRegsSetE", "llvm::(anonymous namespace)::R10W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R10W_SuperRegsSetE", "llvm::(anonymous namespace)::R10W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R11_OverlapsE", "llvm::(anonymous namespace)::R11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R11_SubRegsSetE", "llvm::(anonymous namespace)::R11_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R11B_OverlapsE", "llvm::(anonymous namespace)::R11B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R11B_SuperRegsSetE", "llvm::(anonymous namespace)::R11B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R11D_OverlapsE", "llvm::(anonymous namespace)::R11D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R11D_SubRegsSetE", "llvm::(anonymous namespace)::R11D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R11D_SuperRegsSetE", "llvm::(anonymous namespace)::R11D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R11W_OverlapsE", "llvm::(anonymous namespace)::R11W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R11W_SubRegsSetE", "llvm::(anonymous namespace)::R11W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R11W_SuperRegsSetE", "llvm::(anonymous namespace)::R11W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R12_OverlapsE", "llvm::(anonymous namespace)::R12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R12_SubRegsSetE", "llvm::(anonymous namespace)::R12_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R12B_OverlapsE", "llvm::(anonymous namespace)::R12B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R12B_SuperRegsSetE", "llvm::(anonymous namespace)::R12B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R12D_OverlapsE", "llvm::(anonymous namespace)::R12D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R12D_SubRegsSetE", "llvm::(anonymous namespace)::R12D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R12D_SuperRegsSetE", "llvm::(anonymous namespace)::R12D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R12W_OverlapsE", "llvm::(anonymous namespace)::R12W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R12W_SubRegsSetE", "llvm::(anonymous namespace)::R12W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R12W_SuperRegsSetE", "llvm::(anonymous namespace)::R12W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R13_OverlapsE", "llvm::(anonymous namespace)::R13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R13_SubRegsSetE", "llvm::(anonymous namespace)::R13_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R13B_OverlapsE", "llvm::(anonymous namespace)::R13B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R13B_SuperRegsSetE", "llvm::(anonymous namespace)::R13B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R13D_OverlapsE", "llvm::(anonymous namespace)::R13D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R13D_SubRegsSetE", "llvm::(anonymous namespace)::R13D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R13D_SuperRegsSetE", "llvm::(anonymous namespace)::R13D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R13W_OverlapsE", "llvm::(anonymous namespace)::R13W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R13W_SubRegsSetE", "llvm::(anonymous namespace)::R13W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R13W_SuperRegsSetE", "llvm::(anonymous namespace)::R13W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R14_OverlapsE", "llvm::(anonymous namespace)::R14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R14_SubRegsSetE", "llvm::(anonymous namespace)::R14_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R14B_OverlapsE", "llvm::(anonymous namespace)::R14B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R14B_SuperRegsSetE", "llvm::(anonymous namespace)::R14B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R14D_OverlapsE", "llvm::(anonymous namespace)::R14D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R14D_SubRegsSetE", "llvm::(anonymous namespace)::R14D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R14D_SuperRegsSetE", "llvm::(anonymous namespace)::R14D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R14W_OverlapsE", "llvm::(anonymous namespace)::R14W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R14W_SubRegsSetE", "llvm::(anonymous namespace)::R14W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R14W_SuperRegsSetE", "llvm::(anonymous namespace)::R14W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12R15_OverlapsE", "llvm::(anonymous namespace)::R15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14R15_SubRegsSetE", "llvm::(anonymous namespace)::R15_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R15B_OverlapsE", "llvm::(anonymous namespace)::R15B_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17R15B_SuperRegsSetE", "llvm::(anonymous namespace)::R15B_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R15D_OverlapsE", "llvm::(anonymous namespace)::R15D_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R15D_SubRegsSetE", "llvm::(anonymous namespace)::R15D_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R15D_SuperRegsSetE", "llvm::(anonymous namespace)::R15D_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13R15W_OverlapsE", "llvm::(anonymous namespace)::R15W_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15R15W_SubRegsSetE", "llvm::(anonymous namespace)::R15W_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L17R15W_SuperRegsSetE", "llvm::(anonymous namespace)::R15W_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RAX_OverlapsE", "llvm::(anonymous namespace)::RAX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RAX_SubRegsSetE", "llvm::(anonymous namespace)::RAX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RBP_OverlapsE", "llvm::(anonymous namespace)::RBP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RBP_SubRegsSetE", "llvm::(anonymous namespace)::RBP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RBX_OverlapsE", "llvm::(anonymous namespace)::RBX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RBX_SubRegsSetE", "llvm::(anonymous namespace)::RBX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RCX_OverlapsE", "llvm::(anonymous namespace)::RCX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RCX_SubRegsSetE", "llvm::(anonymous namespace)::RCX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RDI_OverlapsE", "llvm::(anonymous namespace)::RDI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RDI_SubRegsSetE", "llvm::(anonymous namespace)::RDI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RDX_OverlapsE", "llvm::(anonymous namespace)::RDX_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RDX_SubRegsSetE", "llvm::(anonymous namespace)::RDX_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RIP_OverlapsE", "llvm::(anonymous namespace)::RIP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RIP_SubRegsSetE", "llvm::(anonymous namespace)::RIP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RIZ_OverlapsE", "llvm::(anonymous namespace)::RIZ_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12RSI_OverlapsE", "llvm::(anonymous namespace)::RSI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RSI_SubRegsSetE", "llvm::(anonymous namespace)::RSI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12RSP_OverlapsE", "llvm::(anonymous namespace)::RSP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14RSP_SubRegsSetE", "llvm::(anonymous namespace)::RSP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11SI_OverlapsE", "llvm::(anonymous namespace)::SI_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13SI_SubRegsSetE", "llvm::(anonymous namespace)::SI_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15SI_SuperRegsSetE", "llvm::(anonymous namespace)::SI_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12SIL_OverlapsE", "llvm::(anonymous namespace)::SIL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16SIL_SuperRegsSetE", "llvm::(anonymous namespace)::SIL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11SP_OverlapsE", "llvm::(anonymous namespace)::SP_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13SP_SubRegsSetE", "llvm::(anonymous namespace)::SP_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15SP_SuperRegsSetE", "llvm::(anonymous namespace)::SP_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12SPL_OverlapsE", "llvm::(anonymous namespace)::SPL_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16SPL_SuperRegsSetE", "llvm::(anonymous namespace)::SPL_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11SS_OverlapsE", "llvm::(anonymous namespace)::SS_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST0_OverlapsE", "llvm::(anonymous namespace)::ST0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST1_OverlapsE", "llvm::(anonymous namespace)::ST1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST2_OverlapsE", "llvm::(anonymous namespace)::ST2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST3_OverlapsE", "llvm::(anonymous namespace)::ST3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST4_OverlapsE", "llvm::(anonymous namespace)::ST4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST5_OverlapsE", "llvm::(anonymous namespace)::ST5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST6_OverlapsE", "llvm::(anonymous namespace)::ST6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12ST7_OverlapsE", "llvm::(anonymous namespace)::ST7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM0_OverlapsE", "llvm::(anonymous namespace)::XMM0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM0_SuperRegsSetE", "llvm::(anonymous namespace)::XMM0_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM1_OverlapsE", "llvm::(anonymous namespace)::XMM1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM1_SuperRegsSetE", "llvm::(anonymous namespace)::XMM1_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM2_OverlapsE", "llvm::(anonymous namespace)::XMM2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM2_SuperRegsSetE", "llvm::(anonymous namespace)::XMM2_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM3_OverlapsE", "llvm::(anonymous namespace)::XMM3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM3_SuperRegsSetE", "llvm::(anonymous namespace)::XMM3_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM4_OverlapsE", "llvm::(anonymous namespace)::XMM4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM4_SuperRegsSetE", "llvm::(anonymous namespace)::XMM4_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM5_OverlapsE", "llvm::(anonymous namespace)::XMM5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM5_SuperRegsSetE", "llvm::(anonymous namespace)::XMM5_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM6_OverlapsE", "llvm::(anonymous namespace)::XMM6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM6_SuperRegsSetE", "llvm::(anonymous namespace)::XMM6_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM7_OverlapsE", "llvm::(anonymous namespace)::XMM7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM7_SuperRegsSetE", "llvm::(anonymous namespace)::XMM7_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM8_OverlapsE", "llvm::(anonymous namespace)::XMM8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM8_SuperRegsSetE", "llvm::(anonymous namespace)::XMM8_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13XMM9_OverlapsE", "llvm::(anonymous namespace)::XMM9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L17XMM9_SuperRegsSetE", "llvm::(anonymous namespace)::XMM9_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM10_OverlapsE", "llvm::(anonymous namespace)::XMM10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM10_SuperRegsSetE", "llvm::(anonymous namespace)::XMM10_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM11_OverlapsE", "llvm::(anonymous namespace)::XMM11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM11_SuperRegsSetE", "llvm::(anonymous namespace)::XMM11_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM12_OverlapsE", "llvm::(anonymous namespace)::XMM12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM12_SuperRegsSetE", "llvm::(anonymous namespace)::XMM12_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM13_OverlapsE", "llvm::(anonymous namespace)::XMM13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM13_SuperRegsSetE", "llvm::(anonymous namespace)::XMM13_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM14_OverlapsE", "llvm::(anonymous namespace)::XMM14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM14_SuperRegsSetE", "llvm::(anonymous namespace)::XMM14_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14XMM15_OverlapsE", "llvm::(anonymous namespace)::XMM15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L18XMM15_SuperRegsSetE", "llvm::(anonymous namespace)::XMM15_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM0_OverlapsE", "llvm::(anonymous namespace)::YMM0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM0_SubRegsSetE", "llvm::(anonymous namespace)::YMM0_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM1_OverlapsE", "llvm::(anonymous namespace)::YMM1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM1_SubRegsSetE", "llvm::(anonymous namespace)::YMM1_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM2_OverlapsE", "llvm::(anonymous namespace)::YMM2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM2_SubRegsSetE", "llvm::(anonymous namespace)::YMM2_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM3_OverlapsE", "llvm::(anonymous namespace)::YMM3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM3_SubRegsSetE", "llvm::(anonymous namespace)::YMM3_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM4_OverlapsE", "llvm::(anonymous namespace)::YMM4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM4_SubRegsSetE", "llvm::(anonymous namespace)::YMM4_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM5_OverlapsE", "llvm::(anonymous namespace)::YMM5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM5_SubRegsSetE", "llvm::(anonymous namespace)::YMM5_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM6_OverlapsE", "llvm::(anonymous namespace)::YMM6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM6_SubRegsSetE", "llvm::(anonymous namespace)::YMM6_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM7_OverlapsE", "llvm::(anonymous namespace)::YMM7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM7_SubRegsSetE", "llvm::(anonymous namespace)::YMM7_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM8_OverlapsE", "llvm::(anonymous namespace)::YMM8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM8_SubRegsSetE", "llvm::(anonymous namespace)::YMM8_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L13YMM9_OverlapsE", "llvm::(anonymous namespace)::YMM9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15YMM9_SubRegsSetE", "llvm::(anonymous namespace)::YMM9_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM10_OverlapsE", "llvm::(anonymous namespace)::YMM10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM10_SubRegsSetE", "llvm::(anonymous namespace)::YMM10_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM11_OverlapsE", "llvm::(anonymous namespace)::YMM11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM11_SubRegsSetE", "llvm::(anonymous namespace)::YMM11_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM12_OverlapsE", "llvm::(anonymous namespace)::YMM12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM12_SubRegsSetE", "llvm::(anonymous namespace)::YMM12_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM13_OverlapsE", "llvm::(anonymous namespace)::YMM13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM13_SubRegsSetE", "llvm::(anonymous namespace)::YMM13_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM14_OverlapsE", "llvm::(anonymous namespace)::YMM14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM14_SubRegsSetE", "llvm::(anonymous namespace)::YMM14_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14YMM15_OverlapsE", "llvm::(anonymous namespace)::YMM15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16YMM15_SubRegsSetE", "llvm::(anonymous namespace)::YMM15_SubRegsSet"},
    {"_ZN4llvmL13ImplicitList1E", "llvm::ImplicitList1"},
    {"_ZN4llvmL13ImplicitList2E", "llvm::ImplicitList2"},
    {"_ZN4llvmL13ImplicitList3E", "llvm::ImplicitList3"},
    {"_ZN4llvmL13ImplicitList4E", "llvm::ImplicitList4"},
    {"_ZN4llvmL13ImplicitList5E", "llvm::ImplicitList5"},
    {"_ZN4llvmL13ImplicitList6E", "llvm::ImplicitList6"},
    {"_ZN4llvmL13ImplicitList7E", "llvm::ImplicitList7"},
    {"_ZN4llvmL13ImplicitList8E", "llvm::ImplicitList8"},
    {"_ZN4llvmL13ImplicitList9E", "llvm::ImplicitList9"},
    {"_ZN4llvmL14ImplicitList10E", "llvm::ImplicitList10"},
    {"_ZN4llvmL14ImplicitList11E", "llvm::ImplicitList11"},
    {"_ZN4llvmL14ImplicitList12E", "llvm::ImplicitList12"},
    {"_ZN4llvmL14ImplicitList13E", "llvm::ImplicitList13"},
    {"_ZN4llvmL14ImplicitList14E", "llvm::ImplicitList14"},
    {"_ZN4llvmL14ImplicitList15E", "llvm::ImplicitList15"},
    {"_ZN4llvmL14ImplicitList16E", "llvm::ImplicitList16"},
    {"_ZN4llvm3ARML18CCRSuperRegClassesE", "llvm::ARM::CCRSuperRegClasses"},
    {"_ZN4llvm3ARML18GPRSuperRegClassesE", "llvm::ARM::GPRSuperRegClasses"},
    {"_ZN4llvm3ARML20QPR_8SuperRegClassesE", "llvm::ARM::QPR_8SuperRegClasses"},
    {"_ZN4llvm3ARML19QQPRSuperRegClassesE", "llvm::ARM::QQPRSuperRegClasses"},
    {"_ZN4llvm3ARML24QQPR_VFP2SuperRegClassesE", "llvm::ARM::QQPR_VFP2SuperRegClasses"},
    {"_ZN4llvm3ARML21QQQQPRSuperRegClassesE", "llvm::ARM::QQQQPRSuperRegClasses"},
    {"_ZN4llvm3ARML19rGPRSuperRegClassesE", "llvm::ARM::rGPRSuperRegClasses"},
    {"_ZN4llvm3ARML19tGPRSuperRegClassesE", "llvm::ARM::tGPRSuperRegClasses"},
    {"_ZN4llvm3ARML20tcGPRSuperRegClassesE", "llvm::ARM::tcGPRSuperRegClasses"},
    {"_ZN4llvm3ARML13CCRSubclassesE", "llvm::ARM::CCRSubclasses"},
    {"_ZN4llvm3ARML15DPR_8SubclassesE", "llvm::ARM::DPR_8Subclasses"},
    {"_ZN4llvm3ARML15QPR_8SubclassesE", "llvm::ARM::QPR_8Subclasses"},
    {"_ZN4llvm3ARML19QQPR_VFP2SubclassesE", "llvm::ARM::QQPR_VFP2Subclasses"},
    {"_ZN4llvm3ARML16QQQQPRSubclassesE", "llvm::ARM::QQQQPRSubclasses"},
    {"_ZN4llvm3ARML15SPR_8SubclassesE", "llvm::ARM::SPR_8Subclasses"},
    {"_ZN4llvm3ARML14tGPRSubclassesE", "llvm::ARM::tGPRSubclasses"},
    {"_ZN4llvm3ARML15tcGPRSubclassesE", "llvm::ARM::tcGPRSubclasses"},
    {"_ZN4llvm3ARML15CCRSuperclassesE", "llvm::ARM::CCRSuperclasses"},
    {"_ZN4llvm3ARML15DPRSuperclassesE", "llvm::ARM::DPRSuperclasses"},
    {"_ZN4llvm3ARML15GPRSuperclassesE", "llvm::ARM::GPRSuperclasses"},
    {"_ZN4llvm3ARML15QPRSuperclassesE", "llvm::ARM::QPRSuperclasses"},
    {"_ZN4llvm3ARML16QQPRSuperclassesE", "llvm::ARM::QQPRSuperclasses"},
    {"_ZN4llvm3ARML18QQQQPRSuperclassesE", "llvm::ARM::QQQQPRSuperclasses"},
    {"_ZN4llvm3ARML15SPRSuperclassesE", "llvm::ARM::SPRSuperclasses"},
    {"_ZN4llvm3ARML16CCRSubRegClassesE", "llvm::ARM::CCRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3CCRE", "llvm::(anonymous namespace)::CCR"},
    {"_ZN4llvm3ARML12ARM_DPR_VFP3E", "llvm::ARM::ARM_DPR_VFP3"},
    {"_ZN4llvm3ARML12ARM_DPR_VFP2E", "llvm::ARM::ARM_DPR_VFP2"},
    {"_ZN4llvm3ARML16DPRSubRegClassesE", "llvm::ARM::DPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3DPRE", "llvm::(anonymous namespace)::DPR"},
    {"_ZN4llvm12_GLOBAL__N_1L5DPR_8E", "llvm::(anonymous namespace)::DPR_8"},
    {"_ZN4llvm12_GLOBAL__N_1L8DPR_VFP2E", "llvm::(anonymous namespace)::DPR_VFP2"},
    {"_ZN4llvm3ARML12THUMB_GPR_AOE", "llvm::ARM::THUMB_GPR_AO"},
    {"_ZN4llvm3ARML10ARM_GPR_AOE", "llvm::ARM::ARM_GPR_AO"},
    {"_ZN4llvm3ARML16GPRSubRegClassesE", "llvm::ARM::GPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3GPRE", "llvm::(anonymous namespace)::GPR"},
    {"_ZN4llvm3ARML7ARM_QPRE", "llvm::ARM::ARM_QPR"},
    {"_ZN4llvm12_GLOBAL__N_1L3QPRE", "llvm::(anonymous namespace)::QPR"},
    {"_ZN4llvm12_GLOBAL__N_1L5QPR_8E", "llvm::(anonymous namespace)::QPR_8"},
    {"_ZN4llvm12_GLOBAL__N_1L8QPR_VFP2E", "llvm::(anonymous namespace)::QPR_VFP2"},
    {"_ZN4llvm3ARML8ARM_QQPRE", "llvm::ARM::ARM_QQPR"},
    {"_ZN4llvm12_GLOBAL__N_1L4QQPRE", "llvm::(anonymous namespace)::QQPR"},
    {"_ZN4llvm12_GLOBAL__N_1L9QQPR_VFP2E", "llvm::(anonymous namespace)::QQPR_VFP2"},
    {"_ZN4llvm3ARML10ARM_QQQQPRE", "llvm::ARM::ARM_QQQQPR"},
    {"_ZN4llvm12_GLOBAL__N_1L6QQQQPRE", "llvm::(anonymous namespace)::QQQQPR"},
    {"_ZN4llvm3ARML16SPRSubRegClassesE", "llvm::ARM::SPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L3SPRE", "llvm::(anonymous namespace)::SPR"},
    {"_ZN4llvm3ARML18SPR_8SubRegClassesE", "llvm::ARM::SPR_8SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L5SPR_8E", "llvm::(anonymous namespace)::SPR_8"},
    {"_ZN4llvm3ARML13THUMB_rGPR_AOE", "llvm::ARM::THUMB_rGPR_AO"},
    {"_ZN4llvm3ARML11ARM_rGPR_AOE", "llvm::ARM::ARM_rGPR_AO"},
    {"_ZN4llvm3ARML17rGPRSubRegClassesE", "llvm::ARM::rGPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L4rGPRE", "llvm::(anonymous namespace)::rGPR"},
    {"_ZN4llvm3ARML17tGPRSubRegClassesE", "llvm::ARM::tGPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L4tGPRE", "llvm::(anonymous namespace)::tGPR"},
    {"_ZN4llvm3ARML15THUMB_GPR_AO_TCE", "llvm::ARM::THUMB_GPR_AO_TC"},
    {"_ZN4llvm3ARML13ARM_GPR_R9_TCE", "llvm::ARM::ARM_GPR_R9_TC"},
    {"_ZN4llvm3ARML15ARM_GPR_NOR9_TCE", "llvm::ARM::ARM_GPR_NOR9_TC"},
    {"_ZN4llvm3ARML18tcGPRSubRegClassesE", "llvm::ARM::tcGPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L5tcGPRE", "llvm::(anonymous namespace)::tcGPR"},
    {"_ZN4llvm12_GLOBAL__N_1L15SubregHashTableE", "llvm::(anonymous namespace)::SubregHashTable"},
    {"_ZN4llvm12_GLOBAL__N_1L16AliasesHashTableE", "llvm::(anonymous namespace)::AliasesHashTable"},
    {"_ZN4llvm12_GLOBAL__N_1L13CPSR_OverlapsE", "llvm::(anonymous namespace)::CPSR_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16Empty_SubRegsSetE", "llvm::(anonymous namespace)::Empty_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L18Empty_SuperRegsSetE", "llvm::(anonymous namespace)::Empty_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D0_OverlapsE", "llvm::(anonymous namespace)::D0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D0_SubRegsSetE", "llvm::(anonymous namespace)::D0_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D0_SuperRegsSetE", "llvm::(anonymous namespace)::D0_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D1_OverlapsE", "llvm::(anonymous namespace)::D1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D1_SubRegsSetE", "llvm::(anonymous namespace)::D1_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D1_SuperRegsSetE", "llvm::(anonymous namespace)::D1_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D2_OverlapsE", "llvm::(anonymous namespace)::D2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D2_SubRegsSetE", "llvm::(anonymous namespace)::D2_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D2_SuperRegsSetE", "llvm::(anonymous namespace)::D2_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D3_OverlapsE", "llvm::(anonymous namespace)::D3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D3_SubRegsSetE", "llvm::(anonymous namespace)::D3_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D3_SuperRegsSetE", "llvm::(anonymous namespace)::D3_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D4_OverlapsE", "llvm::(anonymous namespace)::D4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D4_SubRegsSetE", "llvm::(anonymous namespace)::D4_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D4_SuperRegsSetE", "llvm::(anonymous namespace)::D4_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D5_OverlapsE", "llvm::(anonymous namespace)::D5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D5_SubRegsSetE", "llvm::(anonymous namespace)::D5_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D5_SuperRegsSetE", "llvm::(anonymous namespace)::D5_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D6_OverlapsE", "llvm::(anonymous namespace)::D6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D6_SubRegsSetE", "llvm::(anonymous namespace)::D6_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D6_SuperRegsSetE", "llvm::(anonymous namespace)::D6_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D7_OverlapsE", "llvm::(anonymous namespace)::D7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D7_SubRegsSetE", "llvm::(anonymous namespace)::D7_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D7_SuperRegsSetE", "llvm::(anonymous namespace)::D7_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D8_OverlapsE", "llvm::(anonymous namespace)::D8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D8_SubRegsSetE", "llvm::(anonymous namespace)::D8_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D8_SuperRegsSetE", "llvm::(anonymous namespace)::D8_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11D9_OverlapsE", "llvm::(anonymous namespace)::D9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13D9_SubRegsSetE", "llvm::(anonymous namespace)::D9_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15D9_SuperRegsSetE", "llvm::(anonymous namespace)::D9_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D10_OverlapsE", "llvm::(anonymous namespace)::D10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D10_SubRegsSetE", "llvm::(anonymous namespace)::D10_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D10_SuperRegsSetE", "llvm::(anonymous namespace)::D10_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D11_OverlapsE", "llvm::(anonymous namespace)::D11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D11_SubRegsSetE", "llvm::(anonymous namespace)::D11_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D11_SuperRegsSetE", "llvm::(anonymous namespace)::D11_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D12_OverlapsE", "llvm::(anonymous namespace)::D12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D12_SubRegsSetE", "llvm::(anonymous namespace)::D12_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D12_SuperRegsSetE", "llvm::(anonymous namespace)::D12_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D13_OverlapsE", "llvm::(anonymous namespace)::D13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D13_SubRegsSetE", "llvm::(anonymous namespace)::D13_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D13_SuperRegsSetE", "llvm::(anonymous namespace)::D13_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D14_OverlapsE", "llvm::(anonymous namespace)::D14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D14_SubRegsSetE", "llvm::(anonymous namespace)::D14_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D14_SuperRegsSetE", "llvm::(anonymous namespace)::D14_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D15_OverlapsE", "llvm::(anonymous namespace)::D15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14D15_SubRegsSetE", "llvm::(anonymous namespace)::D15_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16D15_SuperRegsSetE", "llvm::(anonymous namespace)::D15_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D16_OverlapsE", "llvm::(anonymous namespace)::D16_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D16_SuperRegsSetE", "llvm::(anonymous namespace)::D16_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D17_OverlapsE", "llvm::(anonymous namespace)::D17_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D17_SuperRegsSetE", "llvm::(anonymous namespace)::D17_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D18_OverlapsE", "llvm::(anonymous namespace)::D18_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D18_SuperRegsSetE", "llvm::(anonymous namespace)::D18_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D19_OverlapsE", "llvm::(anonymous namespace)::D19_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D19_SuperRegsSetE", "llvm::(anonymous namespace)::D19_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D20_OverlapsE", "llvm::(anonymous namespace)::D20_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D20_SuperRegsSetE", "llvm::(anonymous namespace)::D20_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D21_OverlapsE", "llvm::(anonymous namespace)::D21_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D21_SuperRegsSetE", "llvm::(anonymous namespace)::D21_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D22_OverlapsE", "llvm::(anonymous namespace)::D22_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D22_SuperRegsSetE", "llvm::(anonymous namespace)::D22_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D23_OverlapsE", "llvm::(anonymous namespace)::D23_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D23_SuperRegsSetE", "llvm::(anonymous namespace)::D23_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D24_OverlapsE", "llvm::(anonymous namespace)::D24_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D24_SuperRegsSetE", "llvm::(anonymous namespace)::D24_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D25_OverlapsE", "llvm::(anonymous namespace)::D25_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D25_SuperRegsSetE", "llvm::(anonymous namespace)::D25_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D26_OverlapsE", "llvm::(anonymous namespace)::D26_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D26_SuperRegsSetE", "llvm::(anonymous namespace)::D26_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D27_OverlapsE", "llvm::(anonymous namespace)::D27_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D27_SuperRegsSetE", "llvm::(anonymous namespace)::D27_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D28_OverlapsE", "llvm::(anonymous namespace)::D28_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D28_SuperRegsSetE", "llvm::(anonymous namespace)::D28_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D29_OverlapsE", "llvm::(anonymous namespace)::D29_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D29_SuperRegsSetE", "llvm::(anonymous namespace)::D29_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D30_OverlapsE", "llvm::(anonymous namespace)::D30_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D30_SuperRegsSetE", "llvm::(anonymous namespace)::D30_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12D31_OverlapsE", "llvm::(anonymous namespace)::D31_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16D31_SuperRegsSetE", "llvm::(anonymous namespace)::D31_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14FPEXC_OverlapsE", "llvm::(anonymous namespace)::FPEXC_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14FPSCR_OverlapsE", "llvm::(anonymous namespace)::FPSCR_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14FPSID_OverlapsE", "llvm::(anonymous namespace)::FPSID_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16ITSTATE_OverlapsE", "llvm::(anonymous namespace)::ITSTATE_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11LR_OverlapsE", "llvm::(anonymous namespace)::LR_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11PC_OverlapsE", "llvm::(anonymous namespace)::PC_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q0_OverlapsE", "llvm::(anonymous namespace)::Q0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q0_SubRegsSetE", "llvm::(anonymous namespace)::Q0_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q0_SuperRegsSetE", "llvm::(anonymous namespace)::Q0_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q1_OverlapsE", "llvm::(anonymous namespace)::Q1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q1_SubRegsSetE", "llvm::(anonymous namespace)::Q1_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q1_SuperRegsSetE", "llvm::(anonymous namespace)::Q1_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q2_OverlapsE", "llvm::(anonymous namespace)::Q2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q2_SubRegsSetE", "llvm::(anonymous namespace)::Q2_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q2_SuperRegsSetE", "llvm::(anonymous namespace)::Q2_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q3_OverlapsE", "llvm::(anonymous namespace)::Q3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q3_SubRegsSetE", "llvm::(anonymous namespace)::Q3_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q3_SuperRegsSetE", "llvm::(anonymous namespace)::Q3_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q4_OverlapsE", "llvm::(anonymous namespace)::Q4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q4_SubRegsSetE", "llvm::(anonymous namespace)::Q4_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q4_SuperRegsSetE", "llvm::(anonymous namespace)::Q4_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q5_OverlapsE", "llvm::(anonymous namespace)::Q5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q5_SubRegsSetE", "llvm::(anonymous namespace)::Q5_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q5_SuperRegsSetE", "llvm::(anonymous namespace)::Q5_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q6_OverlapsE", "llvm::(anonymous namespace)::Q6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q6_SubRegsSetE", "llvm::(anonymous namespace)::Q6_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q6_SuperRegsSetE", "llvm::(anonymous namespace)::Q6_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q7_OverlapsE", "llvm::(anonymous namespace)::Q7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q7_SubRegsSetE", "llvm::(anonymous namespace)::Q7_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q7_SuperRegsSetE", "llvm::(anonymous namespace)::Q7_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q8_OverlapsE", "llvm::(anonymous namespace)::Q8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q8_SubRegsSetE", "llvm::(anonymous namespace)::Q8_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q8_SuperRegsSetE", "llvm::(anonymous namespace)::Q8_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11Q9_OverlapsE", "llvm::(anonymous namespace)::Q9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L13Q9_SubRegsSetE", "llvm::(anonymous namespace)::Q9_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L15Q9_SuperRegsSetE", "llvm::(anonymous namespace)::Q9_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q10_OverlapsE", "llvm::(anonymous namespace)::Q10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q10_SubRegsSetE", "llvm::(anonymous namespace)::Q10_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q10_SuperRegsSetE", "llvm::(anonymous namespace)::Q10_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q11_OverlapsE", "llvm::(anonymous namespace)::Q11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q11_SubRegsSetE", "llvm::(anonymous namespace)::Q11_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q11_SuperRegsSetE", "llvm::(anonymous namespace)::Q11_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q12_OverlapsE", "llvm::(anonymous namespace)::Q12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q12_SubRegsSetE", "llvm::(anonymous namespace)::Q12_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q12_SuperRegsSetE", "llvm::(anonymous namespace)::Q12_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q13_OverlapsE", "llvm::(anonymous namespace)::Q13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q13_SubRegsSetE", "llvm::(anonymous namespace)::Q13_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q13_SuperRegsSetE", "llvm::(anonymous namespace)::Q13_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q14_OverlapsE", "llvm::(anonymous namespace)::Q14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q14_SubRegsSetE", "llvm::(anonymous namespace)::Q14_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q14_SuperRegsSetE", "llvm::(anonymous namespace)::Q14_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12Q15_OverlapsE", "llvm::(anonymous namespace)::Q15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14Q15_SubRegsSetE", "llvm::(anonymous namespace)::Q15_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16Q15_SuperRegsSetE", "llvm::(anonymous namespace)::Q15_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ0_OverlapsE", "llvm::(anonymous namespace)::QQ0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ0_SubRegsSetE", "llvm::(anonymous namespace)::QQ0_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ0_SuperRegsSetE", "llvm::(anonymous namespace)::QQ0_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ1_OverlapsE", "llvm::(anonymous namespace)::QQ1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ1_SubRegsSetE", "llvm::(anonymous namespace)::QQ1_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ1_SuperRegsSetE", "llvm::(anonymous namespace)::QQ1_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ2_OverlapsE", "llvm::(anonymous namespace)::QQ2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ2_SubRegsSetE", "llvm::(anonymous namespace)::QQ2_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ2_SuperRegsSetE", "llvm::(anonymous namespace)::QQ2_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ3_OverlapsE", "llvm::(anonymous namespace)::QQ3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ3_SubRegsSetE", "llvm::(anonymous namespace)::QQ3_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ3_SuperRegsSetE", "llvm::(anonymous namespace)::QQ3_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ4_OverlapsE", "llvm::(anonymous namespace)::QQ4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ4_SubRegsSetE", "llvm::(anonymous namespace)::QQ4_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ4_SuperRegsSetE", "llvm::(anonymous namespace)::QQ4_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ5_OverlapsE", "llvm::(anonymous namespace)::QQ5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ5_SubRegsSetE", "llvm::(anonymous namespace)::QQ5_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ5_SuperRegsSetE", "llvm::(anonymous namespace)::QQ5_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ6_OverlapsE", "llvm::(anonymous namespace)::QQ6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ6_SubRegsSetE", "llvm::(anonymous namespace)::QQ6_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ6_SuperRegsSetE", "llvm::(anonymous namespace)::QQ6_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQ7_OverlapsE", "llvm::(anonymous namespace)::QQ7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQ7_SubRegsSetE", "llvm::(anonymous namespace)::QQ7_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQ7_SuperRegsSetE", "llvm::(anonymous namespace)::QQ7_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQQQ0_OverlapsE", "llvm::(anonymous namespace)::QQQQ0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQQQ0_SubRegsSetE", "llvm::(anonymous namespace)::QQQQ0_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQQQ1_OverlapsE", "llvm::(anonymous namespace)::QQQQ1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQQQ1_SubRegsSetE", "llvm::(anonymous namespace)::QQQQ1_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQQQ2_OverlapsE", "llvm::(anonymous namespace)::QQQQ2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQQQ2_SubRegsSetE", "llvm::(anonymous namespace)::QQQQ2_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L14QQQQ3_OverlapsE", "llvm::(anonymous namespace)::QQQQ3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16QQQQ3_SubRegsSetE", "llvm::(anonymous namespace)::QQQQ3_SubRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11R0_OverlapsE", "llvm::(anonymous namespace)::R0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R1_OverlapsE", "llvm::(anonymous namespace)::R1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R2_OverlapsE", "llvm::(anonymous namespace)::R2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R3_OverlapsE", "llvm::(anonymous namespace)::R3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R4_OverlapsE", "llvm::(anonymous namespace)::R4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R5_OverlapsE", "llvm::(anonymous namespace)::R5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R6_OverlapsE", "llvm::(anonymous namespace)::R6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R7_OverlapsE", "llvm::(anonymous namespace)::R7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R8_OverlapsE", "llvm::(anonymous namespace)::R8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11R9_OverlapsE", "llvm::(anonymous namespace)::R9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12R10_OverlapsE", "llvm::(anonymous namespace)::R10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12R11_OverlapsE", "llvm::(anonymous namespace)::R11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L12R12_OverlapsE", "llvm::(anonymous namespace)::R12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L11S0_OverlapsE", "llvm::(anonymous namespace)::S0_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S0_SuperRegsSetE", "llvm::(anonymous namespace)::S0_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S1_OverlapsE", "llvm::(anonymous namespace)::S1_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S1_SuperRegsSetE", "llvm::(anonymous namespace)::S1_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S2_OverlapsE", "llvm::(anonymous namespace)::S2_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S2_SuperRegsSetE", "llvm::(anonymous namespace)::S2_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S3_OverlapsE", "llvm::(anonymous namespace)::S3_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S3_SuperRegsSetE", "llvm::(anonymous namespace)::S3_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S4_OverlapsE", "llvm::(anonymous namespace)::S4_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S4_SuperRegsSetE", "llvm::(anonymous namespace)::S4_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S5_OverlapsE", "llvm::(anonymous namespace)::S5_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S5_SuperRegsSetE", "llvm::(anonymous namespace)::S5_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S6_OverlapsE", "llvm::(anonymous namespace)::S6_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S6_SuperRegsSetE", "llvm::(anonymous namespace)::S6_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S7_OverlapsE", "llvm::(anonymous namespace)::S7_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S7_SuperRegsSetE", "llvm::(anonymous namespace)::S7_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S8_OverlapsE", "llvm::(anonymous namespace)::S8_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S8_SuperRegsSetE", "llvm::(anonymous namespace)::S8_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11S9_OverlapsE", "llvm::(anonymous namespace)::S9_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L15S9_SuperRegsSetE", "llvm::(anonymous namespace)::S9_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S10_OverlapsE", "llvm::(anonymous namespace)::S10_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S10_SuperRegsSetE", "llvm::(anonymous namespace)::S10_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S11_OverlapsE", "llvm::(anonymous namespace)::S11_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S11_SuperRegsSetE", "llvm::(anonymous namespace)::S11_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S12_OverlapsE", "llvm::(anonymous namespace)::S12_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S12_SuperRegsSetE", "llvm::(anonymous namespace)::S12_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S13_OverlapsE", "llvm::(anonymous namespace)::S13_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S13_SuperRegsSetE", "llvm::(anonymous namespace)::S13_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S14_OverlapsE", "llvm::(anonymous namespace)::S14_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S14_SuperRegsSetE", "llvm::(anonymous namespace)::S14_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S15_OverlapsE", "llvm::(anonymous namespace)::S15_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S15_SuperRegsSetE", "llvm::(anonymous namespace)::S15_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S16_OverlapsE", "llvm::(anonymous namespace)::S16_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S16_SuperRegsSetE", "llvm::(anonymous namespace)::S16_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S17_OverlapsE", "llvm::(anonymous namespace)::S17_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S17_SuperRegsSetE", "llvm::(anonymous namespace)::S17_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S18_OverlapsE", "llvm::(anonymous namespace)::S18_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S18_SuperRegsSetE", "llvm::(anonymous namespace)::S18_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S19_OverlapsE", "llvm::(anonymous namespace)::S19_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S19_SuperRegsSetE", "llvm::(anonymous namespace)::S19_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S20_OverlapsE", "llvm::(anonymous namespace)::S20_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S20_SuperRegsSetE", "llvm::(anonymous namespace)::S20_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S21_OverlapsE", "llvm::(anonymous namespace)::S21_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S21_SuperRegsSetE", "llvm::(anonymous namespace)::S21_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S22_OverlapsE", "llvm::(anonymous namespace)::S22_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S22_SuperRegsSetE", "llvm::(anonymous namespace)::S22_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S23_OverlapsE", "llvm::(anonymous namespace)::S23_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S23_SuperRegsSetE", "llvm::(anonymous namespace)::S23_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S24_OverlapsE", "llvm::(anonymous namespace)::S24_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S24_SuperRegsSetE", "llvm::(anonymous namespace)::S24_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S25_OverlapsE", "llvm::(anonymous namespace)::S25_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S25_SuperRegsSetE", "llvm::(anonymous namespace)::S25_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S26_OverlapsE", "llvm::(anonymous namespace)::S26_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S26_SuperRegsSetE", "llvm::(anonymous namespace)::S26_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S27_OverlapsE", "llvm::(anonymous namespace)::S27_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S27_SuperRegsSetE", "llvm::(anonymous namespace)::S27_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S28_OverlapsE", "llvm::(anonymous namespace)::S28_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S28_SuperRegsSetE", "llvm::(anonymous namespace)::S28_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S29_OverlapsE", "llvm::(anonymous namespace)::S29_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S29_SuperRegsSetE", "llvm::(anonymous namespace)::S29_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S30_OverlapsE", "llvm::(anonymous namespace)::S30_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S30_SuperRegsSetE", "llvm::(anonymous namespace)::S30_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L12S31_OverlapsE", "llvm::(anonymous namespace)::S31_Overlaps"},
    {"_ZN4llvm12_GLOBAL__N_1L16S31_SuperRegsSetE", "llvm::(anonymous namespace)::S31_SuperRegsSet"},
    {"_ZN4llvm12_GLOBAL__N_1L11SP_OverlapsE", "llvm::(anonymous namespace)::SP_Overlaps"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes8", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes9", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes10", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes11", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE7Opcodes12", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::Opcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes13", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes14", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes15", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes16", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes118", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes19", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes21", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes23", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes25", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes26", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes27", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes28", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes29", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes131", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes32", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes134", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes35", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes37", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes39", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes41", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes42", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes43", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes44", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes45", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes046", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes0"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes147", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes48", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes049", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes0"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes150", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes51", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes53", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes55", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes57", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes58", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes59", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8QOpcodes60", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes61", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes062", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes0"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes163", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes64", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes065", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes0"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE9QOpcodes166", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::QOpcodes1"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes67", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes69", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZZN12_GLOBAL__N_115ARMDAGToDAGISel6SelectEPN4llvm6SDNodeEE8DOpcodes71", "(anonymous namespace)::ARMDAGToDAGISel::Select(llvm::SDNode*)::DOpcodes"},
    {"_ZL19PerfectShuffleTable", "PerfectShuffleTable"},
    {"_ZL13OperandCycles", "OperandCycles"},
    {"_ZL16ForwardingPathes", "ForwardingPathes"},
    {"_ZZN12_GLOBAL__N_116Thumb2SizeReduce13ReduceSpecialERN4llvm17MachineBasicBlockEPNS1_12MachineInstrERKNS_11ReduceEntryEbE11NarrowEntry11", "(anonymous namespace)::Thumb2SizeReduce::ReduceSpecial(llvm::MachineBasicBlock&, llvm::MachineInstr*, (anonymous namespace)::ReduceEntry const&, bool)::NarrowEntry"},
    {"_ZN4llvm9AttributeL20MutuallyIncompatibleE", "llvm::Attribute::MutuallyIncompatible"},
    {"_ZL14hexDigitsUpper", "hexDigitsUpper"},
    {"_ZL14hexDigitsLower", "hexDigitsLower"},
    {"_ZL10colorcodes", "colorcodes"},
    {"_ZL7IntSigs", "IntSigs"},
    {"_ZL8KillSigs", "KillSigs"},
    {"_ZN12_GLOBAL__N_1L11GCCRegNamesE", "(anonymous namespace)::GCCRegNames"},
    {"_ZL8TokNames", "TokNames"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZL13x86_asm_table", "x86_asm_table"},
    {"_ZN4llvm3X86L19FR32SuperRegClassesE", "llvm::X86::FR32SuperRegClasses"},
    {"_ZN4llvm3X86L19FR64SuperRegClassesE", "llvm::X86::FR64SuperRegClasses"},
    {"_ZN4llvm3X86L19GR16SuperRegClassesE", "llvm::X86::GR16SuperRegClasses"},
    {"_ZN4llvm3X86L24GR16_ABCDSuperRegClassesE", "llvm::X86::GR16_ABCDSuperRegClasses"},
    {"_ZN4llvm3X86L25GR16_NOREXSuperRegClassesE", "llvm::X86::GR16_NOREXSuperRegClasses"},
    {"_ZN4llvm3X86L19GR32SuperRegClassesE", "llvm::X86::GR32SuperRegClasses"},
    {"_ZN4llvm3X86L24GR32_ABCDSuperRegClassesE", "llvm::X86::GR32_ABCDSuperRegClasses"},
    {"_ZN4llvm3X86L25GR32_NOREXSuperRegClassesE", "llvm::X86::GR32_NOREXSuperRegClasses"},
    {"_ZN4llvm3X86L24GR32_NOSPSuperRegClassesE", "llvm::X86::GR32_NOSPSuperRegClasses"},
    {"_ZN4llvm3X86L22GR32_TCSuperRegClassesE", "llvm::X86::GR32_TCSuperRegClasses"},
    {"_ZN4llvm3X86L18GR8SuperRegClassesE", "llvm::X86::GR8SuperRegClasses"},
    {"_ZN4llvm3X86L25GR8_ABCD_HSuperRegClassesE", "llvm::X86::GR8_ABCD_HSuperRegClasses"},
    {"_ZN4llvm3X86L25GR8_ABCD_LSuperRegClassesE", "llvm::X86::GR8_ABCD_LSuperRegClasses"},
    {"_ZN4llvm3X86L24GR8_NOREXSuperRegClassesE", "llvm::X86::GR8_NOREXSuperRegClasses"},
    {"_ZN4llvm3X86L20VR128SuperRegClassesE", "llvm::X86::VR128SuperRegClasses"},
    {"_ZN4llvm3X86L14FR32SubclassesE", "llvm::X86::FR32Subclasses"},
    {"_ZN4llvm3X86L14FR64SubclassesE", "llvm::X86::FR64Subclasses"},
    {"_ZN4llvm3X86L14GR16SubclassesE", "llvm::X86::GR16Subclasses"},
    {"_ZN4llvm3X86L20GR16_NOREXSubclassesE", "llvm::X86::GR16_NOREXSubclasses"},
    {"_ZN4llvm3X86L14GR32SubclassesE", "llvm::X86::GR32Subclasses"},
    {"_ZN4llvm3X86L19GR32_ABCDSubclassesE", "llvm::X86::GR32_ABCDSubclasses"},
    {"_ZN4llvm3X86L20GR32_NOREXSubclassesE", "llvm::X86::GR32_NOREXSubclasses"},
    {"_ZN4llvm3X86L19GR32_NOSPSubclassesE", "llvm::X86::GR32_NOSPSubclasses"},
    {"_ZN4llvm3X86L17GR32_TCSubclassesE", "llvm::X86::GR32_TCSubclasses"},
    {"_ZN4llvm3X86L14GR64SubclassesE", "llvm::X86::GR64Subclasses"},
    {"_ZN4llvm3X86L20GR64_NOREXSubclassesE", "llvm::X86::GR64_NOREXSubclasses"},
    {"_ZN4llvm3X86L25GR64_NOREX_NOSPSubclassesE", "llvm::X86::GR64_NOREX_NOSPSubclasses"},
    {"_ZN4llvm3X86L19GR64_NOSPSubclassesE", "llvm::X86::GR64_NOSPSubclasses"},
    {"_ZN4llvm3X86L17GR64_TCSubclassesE", "llvm::X86::GR64_TCSubclasses"},
    {"_ZN4llvm3X86L13GR8SubclassesE", "llvm::X86::GR8Subclasses"},
    {"_ZN4llvm3X86L19GR8_NOREXSubclassesE", "llvm::X86::GR8_NOREXSubclasses"},
    {"_ZN4llvm3X86L15RFP32SubclassesE", "llvm::X86::RFP32Subclasses"},
    {"_ZN4llvm3X86L15RFP64SubclassesE", "llvm::X86::RFP64Subclasses"},
    {"_ZN4llvm3X86L16FR64SuperclassesE", "llvm::X86::FR64Superclasses"},
    {"_ZN4llvm3X86L21GR16_ABCDSuperclassesE", "llvm::X86::GR16_ABCDSuperclasses"},
    {"_ZN4llvm3X86L22GR16_NOREXSuperclassesE", "llvm::X86::GR16_NOREXSuperclasses"},
    {"_ZN4llvm3X86L21GR32_ABCDSuperclassesE", "llvm::X86::GR32_ABCDSuperclasses"},
    {"_ZN4llvm3X86L19GR32_ADSuperclassesE", "llvm::X86::GR32_ADSuperclasses"},
    {"_ZN4llvm3X86L22GR32_NOREXSuperclassesE", "llvm::X86::GR32_NOREXSuperclasses"},
    {"_ZN4llvm3X86L21GR32_NOSPSuperclassesE", "llvm::X86::GR32_NOSPSuperclasses"},
    {"_ZN4llvm3X86L19GR32_TCSuperclassesE", "llvm::X86::GR32_TCSuperclasses"},
    {"_ZN4llvm3X86L21GR64_ABCDSuperclassesE", "llvm::X86::GR64_ABCDSuperclasses"},
    {"_ZN4llvm3X86L22GR64_NOREXSuperclassesE", "llvm::X86::GR64_NOREXSuperclasses"},
    {"_ZN4llvm3X86L27GR64_NOREX_NOSPSuperclassesE", "llvm::X86::GR64_NOREX_NOSPSuperclasses"},
    {"_ZN4llvm3X86L21GR64_NOSPSuperclassesE", "llvm::X86::GR64_NOSPSuperclasses"},
    {"_ZN4llvm3X86L19GR64_TCSuperclassesE", "llvm::X86::GR64_TCSuperclasses"},
    {"_ZN4llvm3X86L22GR64_TCW64SuperclassesE", "llvm::X86::GR64_TCW64Superclasses"},
    {"_ZN4llvm3X86L22GR8_ABCD_HSuperclassesE", "llvm::X86::GR8_ABCD_HSuperclasses"},
    {"_ZN4llvm3X86L22GR8_ABCD_LSuperclassesE", "llvm::X86::GR8_ABCD_LSuperclasses"},
    {"_ZN4llvm3X86L21GR8_NOREXSuperclassesE", "llvm::X86::GR8_NOREXSuperclasses"},
    {"_ZN4llvm3X86L17RFP64SuperclassesE", "llvm::X86::RFP64Superclasses"},
    {"_ZN4llvm3X86L17RFP80SuperclassesE", "llvm::X86::RFP80Superclasses"},
    {"_ZN4llvm3X86L17VR128SuperclassesE", "llvm::X86::VR128Superclasses"},
    {"_ZN4llvm3X86L17GR16SubRegClassesE", "llvm::X86::GR16SubRegClasses"},
    {"_ZN4llvm3X86L22GR16_ABCDSubRegClassesE", "llvm::X86::GR16_ABCDSubRegClasses"},
    {"_ZN4llvm3X86L23GR16_NOREXSubRegClassesE", "llvm::X86::GR16_NOREXSubRegClasses"},
    {"_ZN4llvm3X86L17GR32SubRegClassesE", "llvm::X86::GR32SubRegClasses"},
    {"_ZN4llvm3X86L22GR32_ABCDSubRegClassesE", "llvm::X86::GR32_ABCDSubRegClasses"},
    {"_ZN4llvm3X86L20GR32_ADSubRegClassesE", "llvm::X86::GR32_ADSubRegClasses"},
    {"_ZN4llvm3X86L23GR32_NOREXSubRegClassesE", "llvm::X86::GR32_NOREXSubRegClasses"},
    {"_ZN4llvm3X86L22GR32_NOSPSubRegClassesE", "llvm::X86::GR32_NOSPSubRegClasses"},
    {"_ZN4llvm3X86L20GR32_TCSubRegClassesE", "llvm::X86::GR32_TCSubRegClasses"},
    {"_ZN4llvm3X86L17GR64SubRegClassesE", "llvm::X86::GR64SubRegClasses"},
    {"_ZN4llvm3X86L22GR64_ABCDSubRegClassesE", "llvm::X86::GR64_ABCDSubRegClasses"},
    {"_ZN4llvm3X86L23GR64_NOREXSubRegClassesE", "llvm::X86::GR64_NOREXSubRegClasses"},
    {"_ZN4llvm3X86L28GR64_NOREX_NOSPSubRegClassesE", "llvm::X86::GR64_NOREX_NOSPSubRegClasses"},
    {"_ZN4llvm3X86L22GR64_NOSPSubRegClassesE", "llvm::X86::GR64_NOSPSubRegClasses"},
    {"_ZN4llvm3X86L20GR64_TCSubRegClassesE", "llvm::X86::GR64_TCSubRegClasses"},
    {"_ZN4llvm3X86L18VR128SubRegClassesE", "llvm::X86::VR128SubRegClasses"},
    {"_ZN4llvm3X86L18VR256SubRegClassesE", "llvm::X86::VR256SubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L15RegisterClassesE", "llvm::(anonymous namespace)::RegisterClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L16SubRegIndexTableE", "llvm::(anonymous namespace)::SubRegIndexTable"},
    {"_ZN4llvm3ARML18DPRSuperRegClassesE", "llvm::ARM::DPRSuperRegClasses"},
    {"_ZN4llvm3ARML20DPR_8SuperRegClassesE", "llvm::ARM::DPR_8SuperRegClasses"},
    {"_ZN4llvm3ARML23DPR_VFP2SuperRegClassesE", "llvm::ARM::DPR_VFP2SuperRegClasses"},
    {"_ZN4llvm3ARML18QPRSuperRegClassesE", "llvm::ARM::QPRSuperRegClasses"},
    {"_ZN4llvm3ARML23QPR_VFP2SuperRegClassesE", "llvm::ARM::QPR_VFP2SuperRegClasses"},
    {"_ZN4llvm3ARML18SPRSuperRegClassesE", "llvm::ARM::SPRSuperRegClasses"},
    {"_ZN4llvm3ARML20SPR_8SuperRegClassesE", "llvm::ARM::SPR_8SuperRegClasses"},
    {"_ZN4llvm3ARML13DPRSubclassesE", "llvm::ARM::DPRSubclasses"},
    {"_ZN4llvm3ARML18DPR_VFP2SubclassesE", "llvm::ARM::DPR_VFP2Subclasses"},
    {"_ZN4llvm3ARML13GPRSubclassesE", "llvm::ARM::GPRSubclasses"},
    {"_ZN4llvm3ARML13QPRSubclassesE", "llvm::ARM::QPRSubclasses"},
    {"_ZN4llvm3ARML18QPR_VFP2SubclassesE", "llvm::ARM::QPR_VFP2Subclasses"},
    {"_ZN4llvm3ARML14QQPRSubclassesE", "llvm::ARM::QQPRSubclasses"},
    {"_ZN4llvm3ARML13SPRSubclassesE", "llvm::ARM::SPRSubclasses"},
    {"_ZN4llvm3ARML14rGPRSubclassesE", "llvm::ARM::rGPRSubclasses"},
    {"_ZN4llvm3ARML17DPR_8SuperclassesE", "llvm::ARM::DPR_8Superclasses"},
    {"_ZN4llvm3ARML20DPR_VFP2SuperclassesE", "llvm::ARM::DPR_VFP2Superclasses"},
    {"_ZN4llvm3ARML17QPR_8SuperclassesE", "llvm::ARM::QPR_8Superclasses"},
    {"_ZN4llvm3ARML20QPR_VFP2SuperclassesE", "llvm::ARM::QPR_VFP2Superclasses"},
    {"_ZN4llvm3ARML21QQPR_VFP2SuperclassesE", "llvm::ARM::QQPR_VFP2Superclasses"},
    {"_ZN4llvm3ARML17SPR_8SuperclassesE", "llvm::ARM::SPR_8Superclasses"},
    {"_ZN4llvm3ARML16rGPRSuperclassesE", "llvm::ARM::rGPRSuperclasses"},
    {"_ZN4llvm3ARML16tGPRSuperclassesE", "llvm::ARM::tGPRSuperclasses"},
    {"_ZN4llvm3ARML17tcGPRSuperclassesE", "llvm::ARM::tcGPRSuperclasses"},
    {"_ZN4llvm3ARML18DPR_8SubRegClassesE", "llvm::ARM::DPR_8SubRegClasses"},
    {"_ZN4llvm3ARML21DPR_VFP2SubRegClassesE", "llvm::ARM::DPR_VFP2SubRegClasses"},
    {"_ZN4llvm3ARML16QPRSubRegClassesE", "llvm::ARM::QPRSubRegClasses"},
    {"_ZN4llvm3ARML18QPR_8SubRegClassesE", "llvm::ARM::QPR_8SubRegClasses"},
    {"_ZN4llvm3ARML21QPR_VFP2SubRegClassesE", "llvm::ARM::QPR_VFP2SubRegClasses"},
    {"_ZN4llvm3ARML17QQPRSubRegClassesE", "llvm::ARM::QQPRSubRegClasses"},
    {"_ZN4llvm3ARML22QQPR_VFP2SubRegClassesE", "llvm::ARM::QQPR_VFP2SubRegClasses"},
    {"_ZN4llvm3ARML19QQQQPRSubRegClassesE", "llvm::ARM::QQQQPRSubRegClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L15RegisterClassesE", "llvm::(anonymous namespace)::RegisterClasses"},
    {"_ZN4llvm12_GLOBAL__N_1L16SubRegIndexTableE", "llvm::(anonymous namespace)::SubRegIndexTable"},
    {"_ZL13arm_asm_table", "arm_asm_table"},
    {"_ZN4llvmL24RegisterCoalescerLinkObjE", "llvm::RegisterCoalescerLinkObj"},
    {"_ZN4llvmL31SimpleRegisterCoalescingLinkObjE", "llvm::SimpleRegisterCoalescingLinkObj"},
    {"_ZL8PSVNames", "PSVNames"},
    {"_ZN4llvmL24RegisterCoalescerLinkObjE", "llvm::RegisterCoalescerLinkObj"},
    {"_ZN4llvmL31SimpleRegisterCoalescingLinkObjE", "llvm::SimpleRegisterCoalescingLinkObj"},
    {"_ZN4llvmL24RegisterCoalescerLinkObjE", "llvm::RegisterCoalescerLinkObj"},
    {"_ZN4llvmL31SimpleRegisterCoalescingLinkObjE", "llvm::SimpleRegisterCoalescingLinkObj"},
    {"_ZN4llvmL24RegisterCoalescerLinkObjE", "llvm::RegisterCoalescerLinkObj"},
    {"_ZN4llvmL31SimpleRegisterCoalescingLinkObjE", "llvm::SimpleRegisterCoalescingLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZN4llvmL16CallGraphLinkObjE", "llvm::CallGraphLinkObj"},
    {"_ZL22SectionTypeDescriptors", "SectionTypeDescriptors"},
    {"_ZL22SectionAttrDescriptors", "SectionAttrDescriptors"},
    {"_ZL9Lang_cuda", "Lang_cuda"},
    {"_ZL11Lang_opencl", "Lang_opencl"},
    {"_ZL13Lang_gnucxx0x", "Lang_gnucxx0x"},
    {"_ZL10Lang_cxx0x", "Lang_cxx0x"},
    {"_ZL13Lang_gnucxx98", "Lang_gnucxx98"},
    {"_ZL10Lang_cxx98", "Lang_cxx98"},
    {"_ZL10Lang_gnu9x", "Lang_gnu9x"},
    {"_ZL10Lang_gnu99", "Lang_gnu99"},
    {"_ZL17Lang_iso9899_199x", "Lang_iso9899_199x"},
    {"_ZL17Lang_iso9899_1999", "Lang_iso9899_1999"},
    {"_ZL8Lang_c9x", "Lang_c9x"},
    {"_ZL8Lang_c99", "Lang_c99"},
    {"_ZL10Lang_gnu89", "Lang_gnu89"},
    {"_ZL8Lang_c94", "Lang_c94"},
    {"_ZL17Lang_iso9899_1990", "Lang_iso9899_1990"},
    {"_ZL8Lang_c90", "Lang_c90"},
    {"_ZL8Lang_c89", "Lang_c89"},
    {"_ZL14CC1AsInfoTable", "CC1AsInfoTable"},
    {"_ZL12CC1InfoTable", "CC1InfoTable"},
    {"_ZL9InfoTable", "InfoTable"},
    {"_ZL9TypeInfos", "TypeInfos"},
    {"_ZL16EagerlyAssumeTag", "EagerlyAssumeTag"},
    {"_ZL4Msgs", "Msgs"},
    {"_ZL11BuiltinInfo", "BuiltinInfo"},
    {"_ZL14StaticDiagInfo", "StaticDiagInfo"},
    {"_ZL11OptionTable", "OptionTable"},
    {"_ZN12_GLOBAL__N_1L11BuiltinInfoE", "(anonymous namespace)::BuiltinInfo"},
    {"_ZN12_GLOBAL__N_1L13GCCRegAliasesE", "(anonymous namespace)::GCCRegAliases"},
    {"_ZL21NumAggregatesPromoted", "NumAggregatesPromoted"},
    {"_ZL20NumArgumentsPromoted", "NumArgumentsPromoted"},
    {"_ZL16NumArgumentsDead", "NumArgumentsDead"},
    {"_ZL20NumByValArgsPromoted", "NumByValArgsPromoted"},
    {"_ZL9NumMerged", "NumMerged"},
    {"_ZL29NumArgumentsReplacedWithUndef", "NumArgumentsReplacedWithUndef"},
    {"_ZL22NumArgumentsEliminated", "NumArgumentsEliminated"},
    {"_ZL20NumRetValsEliminated", "NumRetValsEliminated"},
    {"_ZL9NumKilled", "NumKilled"},
    {"_ZL10NumNoAlias", "NumNoAlias"},
    {"_ZL12NumNoCapture", "NumNoCapture"},
    {"_ZL11NumReadNone", "NumReadNone"},
    {"_ZL11NumReadOnly", "NumReadOnly"},
    {"_ZL10NumAliases", "NumAliases"},
    {"_ZL12NumVariables", "NumVariables"},
    {"_ZL12NumFunctions", "NumFunctions"},
    {"_ZL17NumAliasesRemoved", "NumAliasesRemoved"},
    {"_ZL18NumAliasesResolved", "NumAliasesResolved"},
    {"_ZL15NumShrunkToBool", "NumShrunkToBool"},
    {"_ZL10NumHeapSRA", "NumHeapSRA"},
    {"_ZL11NumGlobUses", "NumGlobUses"},
    {"_ZL13NumSubstitute", "NumSubstitute"},
    {"_ZL6NumSRA", "NumSRA"},
    {"_ZL9NumMarked", "NumMarked"},
    {"_ZL12NumLocalized", "NumLocalized"},
    {"_ZL10NumUnnamed", "NumUnnamed"},
    {"_ZL10NumDeleted", "NumDeleted"},
    {"_ZL17NumCtorsEvaluated", "NumCtorsEvaluated"},
    {"_ZL14NumNestRemoved", "NumNestRemoved"},
    {"_ZL14NumFastCallFns", "NumFastCallFns"},
    {"_ZL12NumFnDeleted", "NumFnDeleted"},
    {"_ZL10NumDeleted", "NumDeleted"},
    {"_ZL10NumInlined", "NumInlined"},
    {"_ZL16NumMergedAllocas", "NumMergedAllocas"},
    {"_ZL15NumCallsDeleted", "NumCallsDeleted"},
    {"_ZL10NumUnreach", "NumUnreach"},
    {"_ZL10NumRemoved", "NumRemoved"},
    {"_ZL17NumDeadPrototypes", "NumDeadPrototypes"},
    {"_ZL10MatchTable", "MatchTable"},
    {"_ZL10NumEmitted", "NumEmitted"},
    {"_ZL7NumFXCH", "NumFXCH"},
    {"_ZL8PopTable", "PopTable"},
    {"_ZL11OpcodeTable", "OpcodeTable"},
    {"_ZL15ForwardST0Table", "ForwardST0Table"},
    {"_ZL15ReverseST0Table", "ReverseST0Table"},
    {"_ZL15ForwardSTiTable", "ForwardSTiTable"},
    {"_ZL15ReverseSTiTable", "ReverseSTiTable"},
    {"_ZL5NumFP", "NumFP"},
    {"_ZL12NumLoadMoved", "NumLoadMoved"},
    {"_ZL12NumTailCalls", "NumTailCalls"},
    {"_ZN4llvmL9Barriers1E", "llvm::Barriers1"},
    {"_ZN4llvmL9Barriers2E", "llvm::Barriers2"},
    {"_ZN4llvmL9Barriers3E", "llvm::Barriers3"},
    {"_ZN4llvmL9Barriers4E", "llvm::Barriers4"},
    {"_ZN4llvmL9Barriers5E", "llvm::Barriers5"},
    {"_ZN4llvmL9Barriers6E", "llvm::Barriers6"},
    {"_ZN4llvmL9Barriers7E", "llvm::Barriers7"},
    {"_ZN4llvmL9Barriers8E", "llvm::Barriers8"},
    {"_ZN4llvmL12OperandInfo2E", "llvm::OperandInfo2"},
    {"_ZN4llvmL12OperandInfo3E", "llvm::OperandInfo3"},
    {"_ZN4llvmL12OperandInfo4E", "llvm::OperandInfo4"},
    {"_ZN4llvmL12OperandInfo5E", "llvm::OperandInfo5"},
    {"_ZN4llvmL12OperandInfo6E", "llvm::OperandInfo6"},
    {"_ZN4llvmL12OperandInfo7E", "llvm::OperandInfo7"},
    {"_ZN4llvmL12OperandInfo8E", "llvm::OperandInfo8"},
    {"_ZN4llvmL12OperandInfo9E", "llvm::OperandInfo9"},
    {"_ZN4llvmL13OperandInfo10E", "llvm::OperandInfo10"},
    {"_ZN4llvmL13OperandInfo11E", "llvm::OperandInfo11"},
    {"_ZN4llvmL13OperandInfo12E", "llvm::OperandInfo12"},
    {"_ZN4llvmL13OperandInfo13E", "llvm::OperandInfo13"},
    {"_ZN4llvmL13OperandInfo14E", "llvm::OperandInfo14"},
    {"_ZN4llvmL13OperandInfo15E", "llvm::OperandInfo15"},
    {"_ZN4llvmL13OperandInfo16E", "llvm::OperandInfo16"},
    {"_ZN4llvmL13OperandInfo17E", "llvm::OperandInfo17"},
    {"_ZN4llvmL13OperandInfo18E", "llvm::OperandInfo18"},
    {"_ZN4llvmL13OperandInfo19E", "llvm::OperandInfo19"},
    {"_ZN4llvmL13OperandInfo20E", "llvm::OperandInfo20"},
    {"_ZN4llvmL13OperandInfo21E", "llvm::OperandInfo21"},
    {"_ZN4llvmL13OperandInfo22E", "llvm::OperandInfo22"},
    {"_ZN4llvmL13OperandInfo23E", "llvm::OperandInfo23"},
    {"_ZN4llvmL13OperandInfo24E", "llvm::OperandInfo24"},
    {"_ZN4llvmL13OperandInfo25E", "llvm::OperandInfo25"},
    {"_ZN4llvmL13OperandInfo26E", "llvm::OperandInfo26"},
    {"_ZN4llvmL13OperandInfo27E", "llvm::OperandInfo27"},
    {"_ZN4llvmL13OperandInfo28E", "llvm::OperandInfo28"},
    {"_ZN4llvmL13OperandInfo29E", "llvm::OperandInfo29"},
    {"_ZN4llvmL13OperandInfo30E", "llvm::OperandInfo30"},
    {"_ZN4llvmL13OperandInfo31E", "llvm::OperandInfo31"},
    {"_ZN4llvmL13OperandInfo32E", "llvm::OperandInfo32"},
    {"_ZN4llvmL13OperandInfo33E", "llvm::OperandInfo33"},
    {"_ZN4llvmL13OperandInfo34E", "llvm::OperandInfo34"},
    {"_ZN4llvmL13OperandInfo35E", "llvm::OperandInfo35"},
    {"_ZN4llvmL13OperandInfo36E", "llvm::OperandInfo36"},
    {"_ZN4llvmL13OperandInfo37E", "llvm::OperandInfo37"},
    {"_ZN4llvmL13OperandInfo38E", "llvm::OperandInfo38"},
    {"_ZN4llvmL13OperandInfo39E", "llvm::OperandInfo39"},
    {"_ZN4llvmL13OperandInfo40E", "llvm::OperandInfo40"},
    {"_ZN4llvmL13OperandInfo41E", "llvm::OperandInfo41"},
    {"_ZN4llvmL13OperandInfo42E", "llvm::OperandInfo42"},
    {"_ZN4llvmL13OperandInfo43E", "llvm::OperandInfo43"},
    {"_ZN4llvmL13OperandInfo44E", "llvm::OperandInfo44"},
    {"_ZN4llvmL13OperandInfo45E", "llvm::OperandInfo45"},
    {"_ZN4llvmL13OperandInfo46E", "llvm::OperandInfo46"},
    {"_ZN4llvmL13OperandInfo47E", "llvm::OperandInfo47"},
    {"_ZN4llvmL13OperandInfo48E", "llvm::OperandInfo48"},
    {"_ZN4llvmL13OperandInfo49E", "llvm::OperandInfo49"},
    {"_ZN4llvmL13OperandInfo50E", "llvm::OperandInfo50"},
    {"_ZN4llvmL13OperandInfo51E", "llvm::OperandInfo51"},
    {"_ZN4llvmL13OperandInfo52E", "llvm::OperandInfo52"},
    {"_ZN4llvmL13OperandInfo53E", "llvm::OperandInfo53"},
    {"_ZN4llvmL13OperandInfo54E", "llvm::OperandInfo54"},
    {"_ZN4llvmL13OperandInfo55E", "llvm::OperandInfo55"},
    {"_ZN4llvmL13OperandInfo56E", "llvm::OperandInfo56"},
    {"_ZN4llvmL13OperandInfo57E", "llvm::OperandInfo57"},
    {"_ZN4llvmL13OperandInfo58E", "llvm::OperandInfo58"},
    {"_ZN4llvmL13OperandInfo59E", "llvm::OperandInfo59"},
    {"_ZN4llvmL13OperandInfo60E", "llvm::OperandInfo60"},
    {"_ZN4llvmL13OperandInfo61E", "llvm::OperandInfo61"},
    {"_ZN4llvmL13OperandInfo62E", "llvm::OperandInfo62"},
    {"_ZN4llvmL13OperandInfo63E", "llvm::OperandInfo63"},
    {"_ZN4llvmL13OperandInfo64E", "llvm::OperandInfo64"},
    {"_ZN4llvmL13OperandInfo65E", "llvm::OperandInfo65"},
    {"_ZN4llvmL13OperandInfo66E", "llvm::OperandInfo66"},
    {"_ZN4llvmL13OperandInfo67E", "llvm::OperandInfo67"},
    {"_ZN4llvmL13OperandInfo68E", "llvm::OperandInfo68"},
    {"_ZN4llvmL13OperandInfo69E", "llvm::OperandInfo69"},
    {"_ZN4llvmL13OperandInfo70E", "llvm::OperandInfo70"},
    {"_ZN4llvmL13OperandInfo71E", "llvm::OperandInfo71"},
    {"_ZN4llvmL13OperandInfo72E", "llvm::OperandInfo72"},
    {"_ZN4llvmL13OperandInfo73E", "llvm::OperandInfo73"},
    {"_ZN4llvmL13OperandInfo74E", "llvm::OperandInfo74"},
    {"_ZN4llvmL13OperandInfo75E", "llvm::OperandInfo75"},
    {"_ZN4llvmL13OperandInfo76E", "llvm::OperandInfo76"},
    {"_ZN4llvmL13OperandInfo77E", "llvm::OperandInfo77"},
    {"_ZN4llvmL13OperandInfo78E", "llvm::OperandInfo78"},
    {"_ZN4llvmL13OperandInfo79E", "llvm::OperandInfo79"},
    {"_ZN4llvmL13OperandInfo80E", "llvm::OperandInfo80"},
    {"_ZN4llvmL13OperandInfo81E", "llvm::OperandInfo81"},
    {"_ZN4llvmL13OperandInfo82E", "llvm::OperandInfo82"},
    {"_ZN4llvmL13OperandInfo83E", "llvm::OperandInfo83"},
    {"_ZN4llvmL13OperandInfo84E", "llvm::OperandInfo84"},
    {"_ZN4llvmL13OperandInfo85E", "llvm::OperandInfo85"},
    {"_ZN4llvmL13OperandInfo86E", "llvm::OperandInfo86"},
    {"_ZN4llvmL13OperandInfo87E", "llvm::OperandInfo87"},
    {"_ZN4llvmL13OperandInfo88E", "llvm::OperandInfo88"},
    {"_ZN4llvmL13OperandInfo89E", "llvm::OperandInfo89"},
    {"_ZN4llvmL13OperandInfo90E", "llvm::OperandInfo90"},
    {"_ZN4llvmL13OperandInfo91E", "llvm::OperandInfo91"},
    {"_ZN4llvmL13OperandInfo92E", "llvm::OperandInfo92"},
    {"_ZN4llvmL13OperandInfo93E", "llvm::OperandInfo93"},
    {"_ZN4llvmL13OperandInfo94E", "llvm::OperandInfo94"},
    {"_ZN4llvmL13OperandInfo95E", "llvm::OperandInfo95"},
    {"_ZN4llvmL13OperandInfo96E", "llvm::OperandInfo96"},
    {"_ZN4llvmL13OperandInfo97E", "llvm::OperandInfo97"},
    {"_ZN4llvmL13OperandInfo98E", "llvm::OperandInfo98"},
    {"_ZN4llvmL13OperandInfo99E", "llvm::OperandInfo99"},
    {"_ZN4llvmL14OperandInfo100E", "llvm::OperandInfo100"},
    {"_ZN4llvmL14OperandInfo101E", "llvm::OperandInfo101"},
    {"_ZN4llvmL14OperandInfo102E", "llvm::OperandInfo102"},
    {"_ZN4llvmL14OperandInfo103E", "llvm::OperandInfo103"},
    {"_ZN4llvmL14OperandInfo104E", "llvm::OperandInfo104"},
    {"_ZN4llvmL14OperandInfo105E", "llvm::OperandInfo105"},
    {"_ZN4llvmL14OperandInfo106E", "llvm::OperandInfo106"},
    {"_ZN4llvmL14OperandInfo107E", "llvm::OperandInfo107"},
    {"_ZN4llvmL14OperandInfo108E", "llvm::OperandInfo108"},
    {"_ZN4llvmL14OperandInfo109E", "llvm::OperandInfo109"},
    {"_ZN4llvmL14OperandInfo110E", "llvm::OperandInfo110"},
    {"_ZN4llvmL14OperandInfo111E", "llvm::OperandInfo111"},
    {"_ZN4llvmL14OperandInfo112E", "llvm::OperandInfo112"},
    {"_ZN4llvmL14OperandInfo113E", "llvm::OperandInfo113"},
    {"_ZN4llvmL14OperandInfo114E", "llvm::OperandInfo114"},
    {"_ZN4llvmL14OperandInfo115E", "llvm::OperandInfo115"},
    {"_ZN4llvmL14OperandInfo116E", "llvm::OperandInfo116"},
    {"_ZN4llvmL14OperandInfo117E", "llvm::OperandInfo117"},
    {"_ZN4llvmL14OperandInfo118E", "llvm::OperandInfo118"},
    {"_ZN4llvmL14OperandInfo119E", "llvm::OperandInfo119"},
    {"_ZN4llvmL14OperandInfo120E", "llvm::OperandInfo120"},
    {"_ZN4llvmL14OperandInfo121E", "llvm::OperandInfo121"},
    {"_ZN4llvmL14OperandInfo122E", "llvm::OperandInfo122"},
    {"_ZN4llvmL14OperandInfo123E", "llvm::OperandInfo123"},
    {"_ZN4llvmL14OperandInfo124E", "llvm::OperandInfo124"},
    {"_ZN4llvmL14OperandInfo125E", "llvm::OperandInfo125"},
    {"_ZN4llvmL14OperandInfo126E", "llvm::OperandInfo126"},
    {"_ZN4llvmL14OperandInfo127E", "llvm::OperandInfo127"},
    {"_ZN4llvmL14OperandInfo128E", "llvm::OperandInfo128"},
    {"_ZN4llvmL14OperandInfo129E", "llvm::OperandInfo129"},
    {"_ZN4llvmL14OperandInfo130E", "llvm::OperandInfo130"},
    {"_ZN4llvmL14OperandInfo131E", "llvm::OperandInfo131"},
    {"_ZN4llvmL14OperandInfo132E", "llvm::OperandInfo132"},
    {"_ZN4llvmL14OperandInfo133E", "llvm::OperandInfo133"},
    {"_ZN4llvmL14OperandInfo134E", "llvm::OperandInfo134"},
    {"_ZN4llvmL14OperandInfo135E", "llvm::OperandInfo135"},
    {"_ZN4llvmL14OperandInfo136E", "llvm::OperandInfo136"},
    {"_ZN4llvmL14OperandInfo137E", "llvm::OperandInfo137"},
    {"_ZN4llvmL14OperandInfo138E", "llvm::OperandInfo138"},
    {"_ZN4llvmL14OperandInfo139E", "llvm::OperandInfo139"},
    {"_ZN4llvmL14OperandInfo140E", "llvm::OperandInfo140"},
    {"_ZN4llvmL14OperandInfo141E", "llvm::OperandInfo141"},
    {"_ZN4llvmL14OperandInfo142E", "llvm::OperandInfo142"},
    {"_ZN4llvmL14OperandInfo143E", "llvm::OperandInfo143"},
    {"_ZN4llvmL14OperandInfo144E", "llvm::OperandInfo144"},
    {"_ZN4llvmL14OperandInfo145E", "llvm::OperandInfo145"},
    {"_ZN4llvmL14OperandInfo146E", "llvm::OperandInfo146"},
    {"_ZN4llvmL14OperandInfo147E", "llvm::OperandInfo147"},
    {"_ZN4llvmL14OperandInfo148E", "llvm::OperandInfo148"},
    {"_ZN4llvmL14OperandInfo149E", "llvm::OperandInfo149"},
    {"_ZN4llvmL14OperandInfo150E", "llvm::OperandInfo150"},
    {"_ZN4llvmL14OperandInfo151E", "llvm::OperandInfo151"},
    {"_ZN4llvmL14OperandInfo152E", "llvm::OperandInfo152"},
    {"_ZN4llvmL14OperandInfo153E", "llvm::OperandInfo153"},
    {"_ZN4llvmL14OperandInfo154E", "llvm::OperandInfo154"},
    {"_ZN4llvmL14OperandInfo155E", "llvm::OperandInfo155"},
    {"_ZN4llvmL14OperandInfo156E", "llvm::OperandInfo156"},
    {"_ZN4llvmL14OperandInfo157E", "llvm::OperandInfo157"},
    {"_ZN4llvmL14OperandInfo158E", "llvm::OperandInfo158"},
    {"_ZN4llvmL14OperandInfo159E", "llvm::OperandInfo159"},
    {"_ZN4llvmL14OperandInfo160E", "llvm::OperandInfo160"},
    {"_ZN4llvmL14OperandInfo161E", "llvm::OperandInfo161"},
    {"_ZN4llvmL14OperandInfo162E", "llvm::OperandInfo162"},
    {"_ZN4llvmL14OperandInfo163E", "llvm::OperandInfo163"},
    {"_ZN4llvmL14OperandInfo164E", "llvm::OperandInfo164"},
    {"_ZN4llvmL14OperandInfo165E", "llvm::OperandInfo165"},
    {"_ZN4llvmL14OperandInfo166E", "llvm::OperandInfo166"},
    {"_ZN4llvmL14OperandInfo167E", "llvm::OperandInfo167"},
    {"_ZN4llvmL14OperandInfo168E", "llvm::OperandInfo168"},
    {"_ZN4llvmL14OperandInfo169E", "llvm::OperandInfo169"},
    {"_ZN4llvmL14OperandInfo170E", "llvm::OperandInfo170"},
    {"_ZN4llvmL14OperandInfo171E", "llvm::OperandInfo171"},
    {"_ZN4llvmL14OperandInfo172E", "llvm::OperandInfo172"},
    {"_ZN4llvmL14OperandInfo173E", "llvm::OperandInfo173"},
    {"_ZN4llvmL14OperandInfo174E", "llvm::OperandInfo174"},
    {"_ZN4llvmL14OperandInfo175E", "llvm::OperandInfo175"},
    {"_ZN4llvmL14OperandInfo176E", "llvm::OperandInfo176"},
    {"_ZN4llvmL14OperandInfo177E", "llvm::OperandInfo177"},
    {"_ZN4llvmL14OperandInfo178E", "llvm::OperandInfo178"},
    {"_ZN4llvmL14OperandInfo179E", "llvm::OperandInfo179"},
    {"_ZN4llvmL14OperandInfo180E", "llvm::OperandInfo180"},
    {"_ZN4llvmL14OperandInfo181E", "llvm::OperandInfo181"},
    {"_ZN4llvmL14OperandInfo182E", "llvm::OperandInfo182"},
    {"_ZN4llvmL14OperandInfo183E", "llvm::OperandInfo183"},
    {"_ZN4llvmL14OperandInfo184E", "llvm::OperandInfo184"},
    {"_ZN4llvmL14OperandInfo185E", "llvm::OperandInfo185"},
    {"_ZN4llvmL14OperandInfo186E", "llvm::OperandInfo186"},
    {"_ZN4llvmL14OperandInfo187E", "llvm::OperandInfo187"},
    {"_ZN4llvmL14OperandInfo188E", "llvm::OperandInfo188"},
    {"_ZN4llvmL14OperandInfo189E", "llvm::OperandInfo189"},
    {"_ZN4llvmL14OperandInfo190E", "llvm::OperandInfo190"},
    {"_ZN4llvmL14OperandInfo191E", "llvm::OperandInfo191"},
    {"_ZN4llvmL14OperandInfo192E", "llvm::OperandInfo192"},
    {"_ZN4llvmL14OperandInfo193E", "llvm::OperandInfo193"},
    {"_ZN4llvmL14OperandInfo194E", "llvm::OperandInfo194"},
    {"_ZN4llvmL14OperandInfo195E", "llvm::OperandInfo195"},
    {"_ZN4llvmL14OperandInfo196E", "llvm::OperandInfo196"},
    {"_ZN4llvmL14OperandInfo197E", "llvm::OperandInfo197"},
    {"_ZN4llvmL14OperandInfo198E", "llvm::OperandInfo198"},
    {"_ZN4llvmL14OperandInfo199E", "llvm::OperandInfo199"},
    {"_ZN4llvmL14OperandInfo200E", "llvm::OperandInfo200"},
    {"_ZN4llvmL14OperandInfo201E", "llvm::OperandInfo201"},
    {"_ZN4llvmL14OperandInfo202E", "llvm::OperandInfo202"},
    {"_ZN4llvmL14OperandInfo203E", "llvm::OperandInfo203"},
    {"_ZN4llvmL14OperandInfo204E", "llvm::OperandInfo204"},
    {"_ZN4llvmL14OperandInfo205E", "llvm::OperandInfo205"},
    {"_ZN4llvmL14OperandInfo206E", "llvm::OperandInfo206"},
    {"_ZN4llvmL14OperandInfo207E", "llvm::OperandInfo207"},
    {"_ZN4llvmL14OperandInfo208E", "llvm::OperandInfo208"},
    {"_ZN4llvmL14OperandInfo209E", "llvm::OperandInfo209"},
    {"_ZN4llvmL14OperandInfo210E", "llvm::OperandInfo210"},
    {"_ZN4llvmL14OperandInfo211E", "llvm::OperandInfo211"},
    {"_ZN4llvmL14OperandInfo212E", "llvm::OperandInfo212"},
    {"_ZN4llvmL14OperandInfo213E", "llvm::OperandInfo213"},
    {"_ZN4llvmL14OperandInfo214E", "llvm::OperandInfo214"},
    {"_ZN4llvmL14OperandInfo215E", "llvm::OperandInfo215"},
    {"_ZN4llvmL14OperandInfo216E", "llvm::OperandInfo216"},
    {"_ZN4llvmL14OperandInfo217E", "llvm::OperandInfo217"},
    {"_ZN4llvmL14OperandInfo218E", "llvm::OperandInfo218"},
    {"_ZN4llvmL14OperandInfo219E", "llvm::OperandInfo219"},
    {"_ZN4llvmL14OperandInfo220E", "llvm::OperandInfo220"},
    {"_ZN4llvmL14OperandInfo221E", "llvm::OperandInfo221"},
    {"_ZN4llvmL14OperandInfo222E", "llvm::OperandInfo222"},
    {"_ZN4llvmL14OperandInfo223E", "llvm::OperandInfo223"},
    {"_ZN4llvmL14OperandInfo224E", "llvm::OperandInfo224"},
    {"_ZN4llvmL14OperandInfo225E", "llvm::OperandInfo225"},
    {"_ZN4llvmL14OperandInfo226E", "llvm::OperandInfo226"},
    {"_ZN4llvmL14OperandInfo227E", "llvm::OperandInfo227"},
    {"_ZN4llvmL14OperandInfo228E", "llvm::OperandInfo228"},
    {"_ZN4llvmL14OperandInfo229E", "llvm::OperandInfo229"},
    {"_ZN4llvmL14OperandInfo230E", "llvm::OperandInfo230"},
    {"_ZN4llvmL14OperandInfo231E", "llvm::OperandInfo231"},
    {"_ZN4llvmL14OperandInfo232E", "llvm::OperandInfo232"},
    {"_ZN4llvmL14OperandInfo233E", "llvm::OperandInfo233"},
    {"_ZN4llvmL14OperandInfo234E", "llvm::OperandInfo234"},
    {"_ZN4llvmL14OperandInfo235E", "llvm::OperandInfo235"},
    {"_ZN4llvmL14OperandInfo236E", "llvm::OperandInfo236"},
    {"_ZN4llvmL14OperandInfo237E", "llvm::OperandInfo237"},
    {"_ZN4llvmL14OperandInfo238E", "llvm::OperandInfo238"},
    {"_ZN4llvmL14OperandInfo239E", "llvm::OperandInfo239"},
    {"_ZN4llvmL14OperandInfo240E", "llvm::OperandInfo240"},
    {"_ZN4llvmL14OperandInfo241E", "llvm::OperandInfo241"},
    {"_ZN4llvmL14OperandInfo242E", "llvm::OperandInfo242"},
    {"_ZN4llvmL14OperandInfo243E", "llvm::OperandInfo243"},
    {"_ZN4llvmL14OperandInfo244E", "llvm::OperandInfo244"},
    {"_ZN4llvmL14OperandInfo245E", "llvm::OperandInfo245"},
    {"_ZN4llvmL14OperandInfo246E", "llvm::OperandInfo246"},
    {"_ZN4llvmL14OperandInfo247E", "llvm::OperandInfo247"},
    {"_ZN4llvmL14OperandInfo248E", "llvm::OperandInfo248"},
    {"_ZN4llvmL14OperandInfo249E", "llvm::OperandInfo249"},
    {"_ZN4llvmL14OperandInfo250E", "llvm::OperandInfo250"},
    {"_ZN4llvmL14OperandInfo251E", "llvm::OperandInfo251"},
    {"_ZN4llvmL14OperandInfo252E", "llvm::OperandInfo252"},
    {"_ZN4llvmL14OperandInfo253E", "llvm::OperandInfo253"},
    {"_ZN4llvmL14OperandInfo254E", "llvm::OperandInfo254"},
    {"_ZN4llvmL14OperandInfo255E", "llvm::OperandInfo255"},
    {"_ZN4llvmL14OperandInfo256E", "llvm::OperandInfo256"},
    {"_ZN4llvmL14OperandInfo257E", "llvm::OperandInfo257"},
    {"_ZN4llvmL14OperandInfo258E", "llvm::OperandInfo258"},
    {"_ZN4llvmL14OperandInfo259E", "llvm::OperandInfo259"},
    {"_ZN4llvmL14OperandInfo260E", "llvm::OperandInfo260"},
    {"_ZN4llvmL14OperandInfo261E", "llvm::OperandInfo261"},
    {"_ZN4llvmL8X86InstsE", "llvm::X86Insts"},
    {"_ZN4llvm12_GLOBAL__N_1L19RegisterDescriptorsE", "llvm::(anonymous namespace)::RegisterDescriptors"},
    {"_ZL9SubTypeKV", "SubTypeKV"},
    {"_ZL9FeatureKV", "FeatureKV"},
    {"_ZL10MatchTable", "MatchTable"},
    {"_ZL17OperandMatchTable", "OperandMatchTable"},
    {"_ZN4llvmL9Barriers1E", "llvm::Barriers1"},
    {"_ZN4llvmL9Barriers2E", "llvm::Barriers2"},
    {"_ZN4llvmL9Barriers3E", "llvm::Barriers3"},
    {"_ZN4llvmL9Barriers4E", "llvm::Barriers4"},
    {"_ZN4llvmL9Barriers5E", "llvm::Barriers5"},
    {"_ZN4llvmL9Barriers6E", "llvm::Barriers6"},
    {"_ZN4llvmL9Barriers7E", "llvm::Barriers7"},
    {"_ZN4llvmL9Barriers8E", "llvm::Barriers8"},
    {"_ZN4llvmL12OperandInfo2E", "llvm::OperandInfo2"},
    {"_ZN4llvmL12OperandInfo3E", "llvm::OperandInfo3"},
    {"_ZN4llvmL12OperandInfo4E", "llvm::OperandInfo4"},
    {"_ZN4llvmL12OperandInfo5E", "llvm::OperandInfo5"},
    {"_ZN4llvmL12OperandInfo6E", "llvm::OperandInfo6"},
    {"_ZN4llvmL12OperandInfo7E", "llvm::OperandInfo7"},
    {"_ZN4llvmL12OperandInfo8E", "llvm::OperandInfo8"},
    {"_ZN4llvmL12OperandInfo9E", "llvm::OperandInfo9"},
    {"_ZN4llvmL13OperandInfo10E", "llvm::OperandInfo10"},
    {"_ZN4llvmL13OperandInfo11E", "llvm::OperandInfo11"},
    {"_ZN4llvmL13OperandInfo12E", "llvm::OperandInfo12"},
    {"_ZN4llvmL13OperandInfo13E", "llvm::OperandInfo13"},
    {"_ZN4llvmL13OperandInfo14E", "llvm::OperandInfo14"},
    {"_ZN4llvmL13OperandInfo15E", "llvm::OperandInfo15"},
    {"_ZN4llvmL13OperandInfo16E", "llvm::OperandInfo16"},
    {"_ZN4llvmL13OperandInfo17E", "llvm::OperandInfo17"},
    {"_ZN4llvmL13OperandInfo18E", "llvm::OperandInfo18"},
    {"_ZN4llvmL13OperandInfo19E", "llvm::OperandInfo19"},
    {"_ZN4llvmL13OperandInfo20E", "llvm::OperandInfo20"},
    {"_ZN4llvmL13OperandInfo21E", "llvm::OperandInfo21"},
    {"_ZN4llvmL13OperandInfo22E", "llvm::OperandInfo22"},
    {"_ZN4llvmL13OperandInfo23E", "llvm::OperandInfo23"},
    {"_ZN4llvmL13OperandInfo24E", "llvm::OperandInfo24"},
    {"_ZN4llvmL13OperandInfo25E", "llvm::OperandInfo25"},
    {"_ZN4llvmL13OperandInfo26E", "llvm::OperandInfo26"},
    {"_ZN4llvmL13OperandInfo27E", "llvm::OperandInfo27"},
    {"_ZN4llvmL13OperandInfo28E", "llvm::OperandInfo28"},
    {"_ZN4llvmL13OperandInfo29E", "llvm::OperandInfo29"},
    {"_ZN4llvmL13OperandInfo30E", "llvm::OperandInfo30"},
    {"_ZN4llvmL13OperandInfo31E", "llvm::OperandInfo31"},
    {"_ZN4llvmL13OperandInfo32E", "llvm::OperandInfo32"},
    {"_ZN4llvmL13OperandInfo33E", "llvm::OperandInfo33"},
    {"_ZN4llvmL13OperandInfo34E", "llvm::OperandInfo34"},
    {"_ZN4llvmL13OperandInfo35E", "llvm::OperandInfo35"},
    {"_ZN4llvmL13OperandInfo36E", "llvm::OperandInfo36"},
    {"_ZN4llvmL13OperandInfo37E", "llvm::OperandInfo37"},
    {"_ZN4llvmL13OperandInfo38E", "llvm::OperandInfo38"},
    {"_ZN4llvmL13OperandInfo39E", "llvm::OperandInfo39"},
    {"_ZN4llvmL13OperandInfo40E", "llvm::OperandInfo40"},
    {"_ZN4llvmL13OperandInfo41E", "llvm::OperandInfo41"},
    {"_ZN4llvmL13OperandInfo42E", "llvm::OperandInfo42"},
    {"_ZN4llvmL13OperandInfo43E", "llvm::OperandInfo43"},
    {"_ZN4llvmL13OperandInfo44E", "llvm::OperandInfo44"},
    {"_ZN4llvmL13OperandInfo45E", "llvm::OperandInfo45"},
    {"_ZN4llvmL13OperandInfo46E", "llvm::OperandInfo46"},
    {"_ZN4llvmL13OperandInfo47E", "llvm::OperandInfo47"},
    {"_ZN4llvmL13OperandInfo48E", "llvm::OperandInfo48"},
    {"_ZN4llvmL13OperandInfo49E", "llvm::OperandInfo49"},
    {"_ZN4llvmL13OperandInfo50E", "llvm::OperandInfo50"},
    {"_ZN4llvmL13OperandInfo51E", "llvm::OperandInfo51"},
    {"_ZN4llvmL13OperandInfo52E", "llvm::OperandInfo52"},
    {"_ZN4llvmL13OperandInfo53E", "llvm::OperandInfo53"},
    {"_ZN4llvmL13OperandInfo54E", "llvm::OperandInfo54"},
    {"_ZN4llvmL13OperandInfo55E", "llvm::OperandInfo55"},
    {"_ZN4llvmL13OperandInfo56E", "llvm::OperandInfo56"},
    {"_ZN4llvmL13OperandInfo57E", "llvm::OperandInfo57"},
    {"_ZN4llvmL13OperandInfo58E", "llvm::OperandInfo58"},
    {"_ZN4llvmL13OperandInfo59E", "llvm::OperandInfo59"},
    {"_ZN4llvmL13OperandInfo60E", "llvm::OperandInfo60"},
    {"_ZN4llvmL13OperandInfo61E", "llvm::OperandInfo61"},
    {"_ZN4llvmL13OperandInfo62E", "llvm::OperandInfo62"},
    {"_ZN4llvmL13OperandInfo63E", "llvm::OperandInfo63"},
    {"_ZN4llvmL13OperandInfo64E", "llvm::OperandInfo64"},
    {"_ZN4llvmL13OperandInfo65E", "llvm::OperandInfo65"},
    {"_ZN4llvmL13OperandInfo66E", "llvm::OperandInfo66"},
    {"_ZN4llvmL13OperandInfo67E", "llvm::OperandInfo67"},
    {"_ZN4llvmL13OperandInfo68E", "llvm::OperandInfo68"},
    {"_ZN4llvmL13OperandInfo69E", "llvm::OperandInfo69"},
    {"_ZN4llvmL13OperandInfo70E", "llvm::OperandInfo70"},
    {"_ZN4llvmL13OperandInfo71E", "llvm::OperandInfo71"},
    {"_ZN4llvmL13OperandInfo72E", "llvm::OperandInfo72"},
    {"_ZN4llvmL13OperandInfo73E", "llvm::OperandInfo73"},
    {"_ZN4llvmL13OperandInfo74E", "llvm::OperandInfo74"},
    {"_ZN4llvmL13OperandInfo75E", "llvm::OperandInfo75"},
    {"_ZN4llvmL13OperandInfo76E", "llvm::OperandInfo76"},
    {"_ZN4llvmL13OperandInfo77E", "llvm::OperandInfo77"},
    {"_ZN4llvmL13OperandInfo78E", "llvm::OperandInfo78"},
    {"_ZN4llvmL13OperandInfo79E", "llvm::OperandInfo79"},
    {"_ZN4llvmL13OperandInfo80E", "llvm::OperandInfo80"},
    {"_ZN4llvmL13OperandInfo81E", "llvm::OperandInfo81"},
    {"_ZN4llvmL13OperandInfo82E", "llvm::OperandInfo82"},
    {"_ZN4llvmL13OperandInfo83E", "llvm::OperandInfo83"},
    {"_ZN4llvmL13OperandInfo84E", "llvm::OperandInfo84"},
    {"_ZN4llvmL13OperandInfo85E", "llvm::OperandInfo85"},
    {"_ZN4llvmL13OperandInfo86E", "llvm::OperandInfo86"},
    {"_ZN4llvmL13OperandInfo87E", "llvm::OperandInfo87"},
    {"_ZN4llvmL13OperandInfo88E", "llvm::OperandInfo88"},
    {"_ZN4llvmL13OperandInfo89E", "llvm::OperandInfo89"},
    {"_ZN4llvmL13OperandInfo90E", "llvm::OperandInfo90"},
    {"_ZN4llvmL13OperandInfo91E", "llvm::OperandInfo91"},
    {"_ZN4llvmL13OperandInfo92E", "llvm::OperandInfo92"},
    {"_ZN4llvmL13OperandInfo93E", "llvm::OperandInfo93"},
    {"_ZN4llvmL13OperandInfo94E", "llvm::OperandInfo94"},
    {"_ZN4llvmL13OperandInfo95E", "llvm::OperandInfo95"},
    {"_ZN4llvmL13OperandInfo96E", "llvm::OperandInfo96"},
    {"_ZN4llvmL13OperandInfo97E", "llvm::OperandInfo97"},
    {"_ZN4llvmL13OperandInfo98E", "llvm::OperandInfo98"},
    {"_ZN4llvmL13OperandInfo99E", "llvm::OperandInfo99"},
    {"_ZN4llvmL14OperandInfo100E", "llvm::OperandInfo100"},
    {"_ZN4llvmL14OperandInfo101E", "llvm::OperandInfo101"},
    {"_ZN4llvmL14OperandInfo102E", "llvm::OperandInfo102"},
    {"_ZN4llvmL14OperandInfo103E", "llvm::OperandInfo103"},
    {"_ZN4llvmL14OperandInfo104E", "llvm::OperandInfo104"},
    {"_ZN4llvmL14OperandInfo105E", "llvm::OperandInfo105"},
    {"_ZN4llvmL14OperandInfo106E", "llvm::OperandInfo106"},
    {"_ZN4llvmL14OperandInfo107E", "llvm::OperandInfo107"},
    {"_ZN4llvmL14OperandInfo108E", "llvm::OperandInfo108"},
    {"_ZN4llvmL14OperandInfo109E", "llvm::OperandInfo109"},
    {"_ZN4llvmL14OperandInfo110E", "llvm::OperandInfo110"},
    {"_ZN4llvmL14OperandInfo111E", "llvm::OperandInfo111"},
    {"_ZN4llvmL14OperandInfo112E", "llvm::OperandInfo112"},
    {"_ZN4llvmL14OperandInfo113E", "llvm::OperandInfo113"},
    {"_ZN4llvmL14OperandInfo114E", "llvm::OperandInfo114"},
    {"_ZN4llvmL14OperandInfo115E", "llvm::OperandInfo115"},
    {"_ZN4llvmL14OperandInfo116E", "llvm::OperandInfo116"},
    {"_ZN4llvmL14OperandInfo117E", "llvm::OperandInfo117"},
    {"_ZN4llvmL14OperandInfo118E", "llvm::OperandInfo118"},
    {"_ZN4llvmL14OperandInfo119E", "llvm::OperandInfo119"},
    {"_ZN4llvmL14OperandInfo120E", "llvm::OperandInfo120"},
    {"_ZN4llvmL14OperandInfo121E", "llvm::OperandInfo121"},
    {"_ZN4llvmL14OperandInfo122E", "llvm::OperandInfo122"},
    {"_ZN4llvmL14OperandInfo123E", "llvm::OperandInfo123"},
    {"_ZN4llvmL14OperandInfo124E", "llvm::OperandInfo124"},
    {"_ZN4llvmL14OperandInfo125E", "llvm::OperandInfo125"},
    {"_ZN4llvmL14OperandInfo126E", "llvm::OperandInfo126"},
    {"_ZN4llvmL14OperandInfo127E", "llvm::OperandInfo127"},
    {"_ZN4llvmL14OperandInfo128E", "llvm::OperandInfo128"},
    {"_ZN4llvmL14OperandInfo129E", "llvm::OperandInfo129"},
    {"_ZN4llvmL14OperandInfo130E", "llvm::OperandInfo130"},
    {"_ZN4llvmL14OperandInfo131E", "llvm::OperandInfo131"},
    {"_ZN4llvmL14OperandInfo132E", "llvm::OperandInfo132"},
    {"_ZN4llvmL14OperandInfo133E", "llvm::OperandInfo133"},
    {"_ZN4llvmL14OperandInfo134E", "llvm::OperandInfo134"},
    {"_ZN4llvmL14OperandInfo135E", "llvm::OperandInfo135"},
    {"_ZN4llvmL14OperandInfo136E", "llvm::OperandInfo136"},
    {"_ZN4llvmL14OperandInfo137E", "llvm::OperandInfo137"},
    {"_ZN4llvmL14OperandInfo138E", "llvm::OperandInfo138"},
    {"_ZN4llvmL14OperandInfo139E", "llvm::OperandInfo139"},
    {"_ZN4llvmL14OperandInfo140E", "llvm::OperandInfo140"},
    {"_ZN4llvmL14OperandInfo141E", "llvm::OperandInfo141"},
    {"_ZN4llvmL14OperandInfo142E", "llvm::OperandInfo142"},
    {"_ZN4llvmL14OperandInfo143E", "llvm::OperandInfo143"},
    {"_ZN4llvmL14OperandInfo144E", "llvm::OperandInfo144"},
    {"_ZN4llvmL14OperandInfo145E", "llvm::OperandInfo145"},
    {"_ZN4llvmL14OperandInfo146E", "llvm::OperandInfo146"},
    {"_ZN4llvmL14OperandInfo147E", "llvm::OperandInfo147"},
    {"_ZN4llvmL14OperandInfo148E", "llvm::OperandInfo148"},
    {"_ZN4llvmL14OperandInfo149E", "llvm::OperandInfo149"},
    {"_ZN4llvmL14OperandInfo150E", "llvm::OperandInfo150"},
    {"_ZN4llvmL14OperandInfo151E", "llvm::OperandInfo151"},
    {"_ZN4llvmL14OperandInfo152E", "llvm::OperandInfo152"},
    {"_ZN4llvmL14OperandInfo153E", "llvm::OperandInfo153"},
    {"_ZN4llvmL14OperandInfo154E", "llvm::OperandInfo154"},
    {"_ZN4llvmL14OperandInfo155E", "llvm::OperandInfo155"},
    {"_ZN4llvmL14OperandInfo156E", "llvm::OperandInfo156"},
    {"_ZN4llvmL14OperandInfo157E", "llvm::OperandInfo157"},
    {"_ZN4llvmL14OperandInfo158E", "llvm::OperandInfo158"},
    {"_ZN4llvmL14OperandInfo159E", "llvm::OperandInfo159"},
    {"_ZN4llvmL14OperandInfo160E", "llvm::OperandInfo160"},
    {"_ZN4llvmL14OperandInfo161E", "llvm::OperandInfo161"},
    {"_ZN4llvmL14OperandInfo162E", "llvm::OperandInfo162"},
    {"_ZN4llvmL14OperandInfo163E", "llvm::OperandInfo163"},
    {"_ZN4llvmL14OperandInfo164E", "llvm::OperandInfo164"},
    {"_ZN4llvmL14OperandInfo165E", "llvm::OperandInfo165"},
    {"_ZN4llvmL14OperandInfo166E", "llvm::OperandInfo166"},
    {"_ZN4llvmL14OperandInfo167E", "llvm::OperandInfo167"},
    {"_ZN4llvmL14OperandInfo168E", "llvm::OperandInfo168"},
    {"_ZN4llvmL14OperandInfo169E", "llvm::OperandInfo169"},
    {"_ZN4llvmL14OperandInfo170E", "llvm::OperandInfo170"},
    {"_ZN4llvmL14OperandInfo171E", "llvm::OperandInfo171"},
    {"_ZN4llvmL14OperandInfo172E", "llvm::OperandInfo172"},
    {"_ZN4llvmL14OperandInfo173E", "llvm::OperandInfo173"},
    {"_ZN4llvmL14OperandInfo174E", "llvm::OperandInfo174"},
    {"_ZN4llvmL14OperandInfo175E", "llvm::OperandInfo175"},
    {"_ZN4llvmL14OperandInfo176E", "llvm::OperandInfo176"},
    {"_ZN4llvmL14OperandInfo177E", "llvm::OperandInfo177"},
    {"_ZN4llvmL14OperandInfo178E", "llvm::OperandInfo178"},
    {"_ZN4llvmL14OperandInfo179E", "llvm::OperandInfo179"},
    {"_ZN4llvmL14OperandInfo180E", "llvm::OperandInfo180"},
    {"_ZN4llvmL14OperandInfo181E", "llvm::OperandInfo181"},
    {"_ZN4llvmL14OperandInfo182E", "llvm::OperandInfo182"},
    {"_ZN4llvmL14OperandInfo183E", "llvm::OperandInfo183"},
    {"_ZN4llvmL14OperandInfo184E", "llvm::OperandInfo184"},
    {"_ZN4llvmL14OperandInfo185E", "llvm::OperandInfo185"},
    {"_ZN4llvmL14OperandInfo186E", "llvm::OperandInfo186"},
    {"_ZN4llvmL14OperandInfo187E", "llvm::OperandInfo187"},
    {"_ZN4llvmL14OperandInfo188E", "llvm::OperandInfo188"},
    {"_ZN4llvmL14OperandInfo189E", "llvm::OperandInfo189"},
    {"_ZN4llvmL14OperandInfo190E", "llvm::OperandInfo190"},
    {"_ZN4llvmL14OperandInfo191E", "llvm::OperandInfo191"},
    {"_ZN4llvmL14OperandInfo192E", "llvm::OperandInfo192"},
    {"_ZN4llvmL14OperandInfo193E", "llvm::OperandInfo193"},
    {"_ZN4llvmL14OperandInfo194E", "llvm::OperandInfo194"},
    {"_ZN4llvmL14OperandInfo195E", "llvm::OperandInfo195"},
    {"_ZN4llvmL14OperandInfo196E", "llvm::OperandInfo196"},
    {"_ZN4llvmL14OperandInfo197E", "llvm::OperandInfo197"},
    {"_ZN4llvmL14OperandInfo198E", "llvm::OperandInfo198"},
    {"_ZN4llvmL14OperandInfo199E", "llvm::OperandInfo199"},
    {"_ZN4llvmL14OperandInfo200E", "llvm::OperandInfo200"},
    {"_ZN4llvmL14OperandInfo201E", "llvm::OperandInfo201"},
    {"_ZN4llvmL14OperandInfo202E", "llvm::OperandInfo202"},
    {"_ZN4llvmL14OperandInfo203E", "llvm::OperandInfo203"},
    {"_ZN4llvmL14OperandInfo204E", "llvm::OperandInfo204"},
    {"_ZN4llvmL14OperandInfo205E", "llvm::OperandInfo205"},
    {"_ZN4llvmL14OperandInfo206E", "llvm::OperandInfo206"},
    {"_ZN4llvmL14OperandInfo207E", "llvm::OperandInfo207"},
    {"_ZN4llvmL14OperandInfo208E", "llvm::OperandInfo208"},
    {"_ZN4llvmL14OperandInfo209E", "llvm::OperandInfo209"},
    {"_ZN4llvmL14OperandInfo210E", "llvm::OperandInfo210"},
    {"_ZN4llvmL14OperandInfo211E", "llvm::OperandInfo211"},
    {"_ZN4llvmL14OperandInfo212E", "llvm::OperandInfo212"},
    {"_ZN4llvmL14OperandInfo213E", "llvm::OperandInfo213"},
    {"_ZN4llvmL14OperandInfo214E", "llvm::OperandInfo214"},
    {"_ZN4llvmL14OperandInfo215E", "llvm::OperandInfo215"},
    {"_ZN4llvmL14OperandInfo216E", "llvm::OperandInfo216"},
    {"_ZN4llvmL14OperandInfo217E", "llvm::OperandInfo217"},
    {"_ZN4llvmL14OperandInfo218E", "llvm::OperandInfo218"},
    {"_ZN4llvmL14OperandInfo219E", "llvm::OperandInfo219"},
    {"_ZN4llvmL14OperandInfo220E", "llvm::OperandInfo220"},
    {"_ZN4llvmL14OperandInfo221E", "llvm::OperandInfo221"},
    {"_ZN4llvmL14OperandInfo222E", "llvm::OperandInfo222"},
    {"_ZN4llvmL14OperandInfo223E", "llvm::OperandInfo223"},
    {"_ZN4llvmL14OperandInfo224E", "llvm::OperandInfo224"},
    {"_ZN4llvmL14OperandInfo225E", "llvm::OperandInfo225"},
    {"_ZN4llvmL14OperandInfo226E", "llvm::OperandInfo226"},
    {"_ZN4llvmL14OperandInfo227E", "llvm::OperandInfo227"},
    {"_ZN4llvmL14OperandInfo228E", "llvm::OperandInfo228"},
    {"_ZN4llvmL14OperandInfo229E", "llvm::OperandInfo229"},
    {"_ZN4llvmL14OperandInfo230E", "llvm::OperandInfo230"},
    {"_ZN4llvmL14OperandInfo231E", "llvm::OperandInfo231"},
    {"_ZN4llvmL14OperandInfo232E", "llvm::OperandInfo232"},
    {"_ZN4llvmL14OperandInfo233E", "llvm::OperandInfo233"},
    {"_ZN4llvmL14OperandInfo234E", "llvm::OperandInfo234"},
    {"_ZN4llvmL14OperandInfo235E", "llvm::OperandInfo235"},
    {"_ZN4llvmL14OperandInfo236E", "llvm::OperandInfo236"},
    {"_ZN4llvmL14OperandInfo237E", "llvm::OperandInfo237"},
    {"_ZN4llvmL14OperandInfo238E", "llvm::OperandInfo238"},
    {"_ZN4llvmL14OperandInfo239E", "llvm::OperandInfo239"},
    {"_ZN4llvmL14OperandInfo240E", "llvm::OperandInfo240"},
    {"_ZN4llvmL14OperandInfo241E", "llvm::OperandInfo241"},
    {"_ZN4llvmL14OperandInfo242E", "llvm::OperandInfo242"},
    {"_ZN4llvmL14OperandInfo243E", "llvm::OperandInfo243"},
    {"_ZN4llvmL14OperandInfo244E", "llvm::OperandInfo244"},
    {"_ZN4llvmL14OperandInfo245E", "llvm::OperandInfo245"},
    {"_ZN4llvmL14OperandInfo246E", "llvm::OperandInfo246"},
    {"_ZN4llvmL14OperandInfo247E", "llvm::OperandInfo247"},
    {"_ZN4llvmL14OperandInfo248E", "llvm::OperandInfo248"},
    {"_ZN4llvmL14OperandInfo249E", "llvm::OperandInfo249"},
    {"_ZN4llvmL14OperandInfo250E", "llvm::OperandInfo250"},
    {"_ZN4llvmL14OperandInfo251E", "llvm::OperandInfo251"},
    {"_ZN4llvmL14OperandInfo252E", "llvm::OperandInfo252"},
    {"_ZN4llvmL14OperandInfo253E", "llvm::OperandInfo253"},
    {"_ZN4llvmL14OperandInfo254E", "llvm::OperandInfo254"},
    {"_ZN4llvmL14OperandInfo255E", "llvm::OperandInfo255"},
    {"_ZN4llvmL14OperandInfo256E", "llvm::OperandInfo256"},
    {"_ZN4llvmL14OperandInfo257E", "llvm::OperandInfo257"},
    {"_ZN4llvmL14OperandInfo258E", "llvm::OperandInfo258"},
    {"_ZN4llvmL14OperandInfo259E", "llvm::OperandInfo259"},
    {"_ZN4llvmL14OperandInfo260E", "llvm::OperandInfo260"},
    {"_ZN4llvmL8ARMInstsE", "llvm::ARMInsts"},
    {"_ZL12ARM_MLxTable", "ARM_MLxTable"},
    {"_ZN4llvm12_GLOBAL__N_1L19RegisterDescriptorsE", "llvm::(anonymous namespace)::RegisterDescriptors"},
    {"_ZL10NumEmitted", "NumEmitted"},
    {"_ZL6NumTBs", "NumTBs"},
    {"_ZL6NumCBZ", "NumCBZ"},
    {"_ZL13NumT2BrShrunk", "NumT2BrShrunk"},
    {"_ZL13NumT2CPShrunk", "NumT2CPShrunk"},
    {"_ZL8NumSplit", "NumSplit"},
    {"_ZL11NumCBrFixed", "NumCBrFixed"},
    {"_ZL11NumUBrFixed", "NumUBrFixed"},
    {"_ZL7NumCPEs", "NumCPEs"},
    {"_ZL13NumJTInserted", "NumJTInserted"},
    {"_ZL10NumJTMoved", "NumJTMoved"},
    {"_ZL13NEONLdStTable", "NEONLdStTable"},
    {"_ZL11NumMovwMovt", "NumMovwMovt"},
    {"_ZL12NumTailCalls", "NumTailCalls"},
    {"_ZL12NumVSTMGened", "NumVSTMGened"},
    {"_ZL12NumVLDMGened", "NumVLDMGened"},
    {"_ZL11NumSTMGened", "NumSTMGened"},
    {"_ZL11NumLDMGened", "NumLDMGened"},
    {"_ZL11NumSTRD2STR", "NumSTRD2STR"},
    {"_ZL11NumLDRD2LDR", "NumLDRD2LDR"},
    {"_ZL11NumSTRD2STM", "NumSTRD2STM"},
    {"_ZL11NumLDRD2LDM", "NumLDRD2LDM"},
    {"_ZL12NumLdStMoved", "NumLdStMoved"},
    {"_ZL13NumSTRDFormed", "NumSTRDFormed"},
    {"_ZL13NumLDRDFormed", "NumLDRDFormed"},
    {"_ZL12MCNumEmitted", "MCNumEmitted"},
    {"_ZL18MCNumCPRelocations", "MCNumCPRelocations"},
    {"_ZL9SubTypeKV", "SubTypeKV"},
    {"_ZL9FeatureKV", "FeatureKV"},
    {"_ZL10ProcItinKV", "ProcItinKV"},
    {"_ZL6Stages", "Stages"},
    {"_ZL18GenericItineraries", "GenericItineraries"},
    {"_ZL16ARMV6Itineraries", "ARMV6Itineraries"},
    {"_ZL19CortexA8Itineraries", "CortexA8Itineraries"},
    {"_ZL19CortexA9Itineraries", "CortexA9Itineraries"},
    {"_ZL9NumExpand", "NumExpand"},
    {"_ZL8NumVMovs", "NumVMovs"},
    {"_ZL6NumITs", "NumITs"},
    {"_ZL13NumMovedInsts", "NumMovedInsts"},
    {"_ZL10NumNarrows", "NumNarrows"},
    {"_ZL9Num2Addrs", "Num2Addrs"},
    {"_ZL8NumLdSts", "NumLdSts"},
    {"_ZN12_GLOBAL__N_1L11ReduceTableE", "(anonymous namespace)::ReduceTable"},
    {"_ZL13NodesCombined", "NodesCombined"},
    {"_ZL11OpsNarrowed", "OpsNarrowed"},
    {"_ZL16PostIndexedNodes", "PostIndexedNodes"},
    {"_ZL15PreIndexedNodes", "PreIndexedNodes"},
    {"_ZL10LdStFP2Int", "LdStFP2Int"},
    {"_ZL8NumNoops", "NumNoops"},
    {"_ZL9NumStalls", "NumStalls"},
    {"_ZL11NumPRCopies", "NumPRCopies"},
    {"_ZL7NumDups", "NumDups"},
    {"_ZL10NumUnfolds", "NumUnfolds"},
    {"_ZL13NumBacktracks", "NumBacktracks"},
    {"_ZL14LoadsClustered", "LoadsClustered"},
    {"_ZL17NumDAGIselRetries", "NumDAGIselRetries"},
    {"_ZL17NumFastIselBlocks", "NumFastIselBlocks"},
    {"_ZL12NumDAGBlocks", "NumDAGBlocks"},
    {"_ZL19NumFastIselFailures", "NumFastIselFailures"},
    {"_ZL12EmittedInsts", "EmittedInsts"},
    {"_ZL13NumBranchOpts", "NumBranchOpts"},
    {"_ZL13NumDeadBlocks", "NumDeadBlocks"},
    {"_ZL12NumTailMerge", "NumTailMerge"},
    {"_ZL15NumLoopsAligned", "NumLoopsAligned"},
    {"_ZL13NumIntraMoved", "NumIntraMoved"},
    {"_ZL12NumIntraElim", "NumIntraElim"},
    {"_ZL10NumDeletes", "NumDeletes"},
    {"_ZL23NumExceptionValuesMoved", "NumExceptionValuesMoved"},
    {"_ZL17NumUnwindsLowered", "NumUnwindsLowered"},
    {"_ZL19NumLandingPadsSplit", "NumLandingPadsSplit"},
    {"_ZL12NumIfConvBBs", "NumIfConvBBs"},
    {"_ZL9NumDupBBs", "NumDupBBs"},
    {"_ZL11NumDiamonds", "NumDiamonds"},
    {"_ZL15NumTriangleFRev", "NumTriangleFRev"},
    {"_ZL16NumTriangleFalse", "NumTriangleFalse"},
    {"_ZL14NumTriangleRev", "NumTriangleRev"},
    {"_ZL11NumTriangle", "NumTriangle"},
    {"_ZL14NumSimpleFalse", "NumSimpleFalse"},
    {"_ZL9NumSimple", "NumSimple"},
    {"_ZL9numSplits", "numSplits"},
    {"_ZL8numFolds", "numFolds"},
    {"_ZL12numIntervals", "numIntervals"},
    {"_ZL15NumReplacements", "NumReplacements"},
    {"_ZL16NumBaseRegisters", "NumBaseRegisters"},
    {"_ZL14NumAllocations", "NumAllocations"},
    {"_ZL11NumCommutes", "NumCommutes"},
    {"_ZL11NumPhysCSEs", "NumPhysCSEs"},
    {"_ZL7NumCSEs", "NumCSEs"},
    {"_ZL12NumCoalesces", "NumCoalesces"},
    {"_ZL10NumHoisted", "NumHoisted"},
    {"_ZL8NumCSEed", "NumCSEed"},
    {"_ZL8NumLowRP", "NumLowRP"},
    {"_ZL14NumHighLatency", "NumHighLatency"},
    {"_ZL16NumPostRAHoisted", "NumPostRAHoisted"},
    {"_ZL7NumSunk", "NumSunk"},
    {"_ZL8NumSplit", "NumSplit"},
    {"_ZL12NumCoalesces", "NumCoalesces"},
    {"_ZL16NumDeadPHICycles", "NumDeadPHICycles"},
    {"_ZL12NumPHICycles", "NumPHICycles"},
    {"_ZL9NumReused", "NumReused"},
    {"_ZL9NumAtomic", "NumAtomic"},
    {"_ZL10NumImmFold", "NumImmFold"},
    {"_ZL8NumReuse", "NumReuse"},
    {"_ZL13NumEliminated", "NumEliminated"},
    {"_ZL8NumNoops", "NumNoops"},
    {"_ZL9NumStalls", "NumStalls"},
    {"_ZL12NumFixedAnti", "NumFixedAnti"},
    {"_ZL13NumDeadSpills", "NumDeadSpills"},
    {"_ZL12NumRenumbers", "NumRenumbers"},
    {"_ZL15NumRestoreFolds", "NumRestoreFolds"},
    {"_ZL8NumFolds", "NumFolds"},
    {"_ZL9NumRemats", "NumRemats"},
    {"_ZL9NumSplits", "NumSplits"},
    {"_ZL16NumScavengedRegs", "NumScavengedRegs"},
    {"_ZL19NumVirtualFrameRegs", "NumVirtualFrameRegs"},
    {"_ZL9NumCopies", "NumCopies"},
    {"_ZL9NumStores", "NumStores"},
    {"_ZL8NumLoads", "NumLoads"},
    {"_ZL11NumCoalesce", "NumCoalesce"},
    {"_ZL13NumBacktracks", "NumBacktracks"},
    {"_ZL12NumDowngrade", "NumDowngrade"},
    {"_ZL8NumIters", "NumIters"},
    {"_ZL12numSRReduced", "numSRReduced"},
    {"_ZL7numPeep", "numPeep"},
    {"_ZL8numJoins", "numJoins"},
    {"_ZL11numCrossRCs", "numCrossRCs"},
    {"_ZL9numAborts", "numAborts"},
    {"_ZL12numDeadValNo", "numDeadValNo"},
    {"_ZL9NumReMats", "NumReMats"},
    {"_ZL11numCommutes", "numCommutes"},
    {"_ZL10numExtends", "numExtends"},
    {"_ZL10NumSpilled", "NumSpilled"},
    {"_ZL10NumUnwinds", "NumUnwinds"},
    {"_ZL10NumInvokes", "NumInvokes"},
    {"_ZL7NumDead", "NumDead"},
    {"_ZL12NumStoreElim", "NumStoreElim"},
    {"_ZL10NumRegRepl", "NumRegRepl"},
    {"_ZL11NumLoadElim", "NumLoadElim"},
    {"_ZL13NumEliminated", "NumEliminated"},
    {"_ZL13NumDeadBlocks", "NumDeadBlocks"},
    {"_ZL12NumInstrDups", "NumInstrDups"},
    {"_ZL11NumTailDups", "NumTailDups"},
    {"_ZL8NumTails", "NumTails"},
    {"_ZL9NumReMats", "NumReMats"},
    {"_ZL19NumConvertedTo3Addr", "NumConvertedTo3Addr"},
    {"_ZL12Num3AddrSunk", "Num3AddrSunk"},
    {"_ZL15NumAggrCommuted", "NumAggrCommuted"},
    {"_ZL11NumCommuted", "NumCommuted"},
    {"_ZL10NumDeletes", "NumDeletes"},
    {"_ZL19NumTwoAddressInstrs", "NumTwoAddressInstrs"},
    {"_ZL9NumSpills", "NumSpills"},
    {"_ZL6NumDSS", "NumDSS"},
    {"_ZL9NumStores", "NumStores"},
    {"_ZL6NumDRM", "NumDRM"},
    {"_ZL9NumReused", "NumReused"},
    {"_ZL8NumLoads", "NumLoads"},
    {"_ZL9NumReMats", "NumReMats"},
    {"_ZL6NumDCE", "NumDCE"},
    {"_ZL11NumCommutes", "NumCommutes"},
    {"_ZL6NumDSE", "NumDSE"},
    {"_ZL10NumSUnfold", "NumSUnfold"},
    {"_ZL10NumAvoided", "NumAvoided"},
    {"_ZL11NumCopified", "NumCopified"},
    {"_ZL10NumOmitted", "NumOmitted"},
    {"_ZL10NumPSpills", "NumPSpills"},
    {"_ZL15NumModRefUnfold", "NumModRefUnfold"},
    {"_ZL10NumRemoved", "NumRemoved"},
    {"_ZL11NumGEPsElim", "NumGEPsElim"},
    {"_ZL14NumMemoryInsts", "NumMemoryInsts"},
    {"_ZL10NumCmpUses", "NumCmpUses"},
    {"_ZL10NumExtUses", "NumExtUses"},
    {"_ZL12NumExtsMoved", "NumExtsMoved"},
    {"_ZL11NumCastUses", "NumCastUses"},
    {"_ZL11NumPHIsElim", "NumPHIsElim"},
    {"_ZL13NumBlocksElim", "NumBlocksElim"},
    {"_ZL12NumMemAccess", "NumMemAccess"},
    {"_ZL7NumCmps", "NumCmps"},
    {"_ZL7NumPhis", "NumPhis"},
    {"_ZL10NumSelects", "NumSelects"},
    {"_ZL12NumFastOther", "NumFastOther"},
    {"_ZL13NumFastStores", "NumFastStores"},
    {"_ZL6NumDSE", "NumDSE"},
    {"_ZL10NumCSECall", "NumCSECall"},
    {"_ZL10NumCSELoad", "NumCSELoad"},
    {"_ZL6NumCSE", "NumCSE"},
    {"_ZL11NumSimplify", "NumSimplify"},
    {"_ZL9NumGVNPRE", "NumGVNPRE"},
    {"_ZL11NumGVNInstr", "NumGVNInstr"},
    {"_ZL10NumPRELoad", "NumPRELoad"},
    {"_ZL10NumGVNLoad", "NumGVNLoad"},
    {"_ZL12NumGVNBlocks", "NumGVNBlocks"},
    {"_ZL10NumRemoved", "NumRemoved"},
    {"_ZL7NumLFTR", "NumLFTR"},
    {"_ZL11NumInserted", "NumInserted"},
    {"_ZL11NumReplaced", "NumReplaced"},
    {"_ZL8NumDupes", "NumDupes"},
    {"_ZL10NumThreads", "NumThreads"},
    {"_ZL8NumFolds", "NumFolds"},
    {"_ZL11NumPromoted", "NumPromoted"},
    {"_ZL10NumHoisted", "NumHoisted"},
    {"_ZL13NumMovedCalls", "NumMovedCalls"},
    {"_ZL13NumMovedLoads", "NumMovedLoads"},
    {"_ZL7NumSunk", "NumSunk"},
    {"_ZL10NumDeleted", "NumDeleted"},
    {"_ZL9NumMemSet", "NumMemSet"},
    {"_ZL9NumMemCpy", "NumMemCpy"},
    {"_ZL10NumRotated", "NumRotated"},
    {"_ZL10NumSelects", "NumSelects"},
    {"_ZL11NumSwitches", "NumSwitches"},
    {"_ZL11NumBranches", "NumBranches"},
    {"_ZL11NumSimplify", "NumSimplify"},
    {"_ZL10NumTrivial", "NumTrivial"},
    {"_ZL14NumMemCpyInstr", "NumMemCpyInstr"},
    {"_ZL12NumMoveToCpy", "NumMoveToCpy"},
    {"_ZL11NumCpyToSet", "NumCpyToSet"},
    {"_ZL14NumMemSetInfer", "NumMemSetInfer"},
    {"_ZL10NumChanged", "NumChanged"},
    {"_ZL10NumAnnihil", "NumAnnihil"},
    {"_ZL9NumFactor", "NumFactor"},
    {"_ZL9NumLinear", "NumLinear"},
    {"_ZL16IPNumGlobalConst", "IPNumGlobalConst"},
    {"_ZL16IPNumInstRemoved", "IPNumInstRemoved"},
    {"_ZL14NumInstRemoved", "NumInstRemoved"},
    {"_ZL13NumDeadBlocks", "NumDeadBlocks"},
    {"_ZL15IPNumArgsElimed", "IPNumArgsElimed"},
    {"_ZL12NumConverted", "NumConverted"},
    {"_ZL11NumReplaced", "NumReplaced"},
    {"_ZL10NumGlobals", "NumGlobals"},
    {"_ZL11NumPromoted", "NumPromoted"},
    {"_ZL11NumAdjusted", "NumAdjusted"},
    {"_ZL8NumSimpl", "NumSimpl"},
    {"_ZL13NumSimplified", "NumSimplified"},
    {"_ZL12NumAnnotated", "NumAnnotated"},
    {"_ZL11NumRetDuped", "NumRetDuped"},
    {"_ZL13NumEliminated", "NumEliminated"},
    {"_ZL13NumAccumAdded", "NumAccumAdded"},
    {"_ZL12NumDeadStore", "NumDeadStore"},
    {"_ZL11NumCombined", "NumCombined"},
    {"_ZL11NumSunkInst", "NumSunkInst"},
    {"_ZL12NumConstProp", "NumConstProp"},
    {"_ZL11NumDeadInst", "NumDeadInst"},
    {"_ZL9NumExpand", "NumExpand"},
    {"_ZL9NumFactor", "NumFactor"},
    {"_ZL10NumReassoc", "NumReassoc"},
    {"_ZL9NumBroken", "NumBroken"},
    {"_ZL8NumLCSSA", "NumLCSSA"},
    {"_ZL9NumNested", "NumNested"},
    {"_ZL11NumInserted", "NumInserted"},
    {"_ZL11NumUnrolled", "NumUnrolled"},
    {"_ZL21NumCompletelyUnrolled", "NumCompletelyUnrolled"},
    {"_ZL10NumUnwinds", "NumUnwinds"},
    {"_ZL10NumInvokes", "NumInvokes"},
    {"_ZL10NumSpilled", "NumSpilled"},
    {"_ZL12NumPHIInsert", "NumPHIInsert"},
    {"_ZL16NumLocalPromoted", "NumLocalPromoted"},
    {"_ZL14NumSingleStore", "NumSingleStore"},
    {"_ZL13NumDeadAlloca", "NumDeadAlloca"},
    {"_ZL15NumSpeculations", "NumSpeculations"},
    {"_ZL16MaxSCCIterations", "MaxSCCIterations"},
    {"_ZL10NumReassoc", "NumReassoc"},
    {"_ZL9NumFactor", "NumFactor"},
    {"_ZL9NumExpand", "NumExpand"},
    {"_ZL27NumCacheCompleteNonLocalPtr", "NumCacheCompleteNonLocalPtr"},
    {"_ZL21NumUncacheNonLocalPtr", "NumUncacheNonLocalPtr"},
    {"_ZL24NumCacheDirtyNonLocalPtr", "NumCacheDirtyNonLocalPtr"},
    {"_ZL19NumCacheNonLocalPtr", "NumCacheNonLocalPtr"},
    {"_ZL18NumUncacheNonLocal", "NumUncacheNonLocal"},
    {"_ZL21NumCacheDirtyNonLocal", "NumCacheDirtyNonLocal"},
    {"_ZL16NumCacheNonLocal", "NumCacheNonLocal"},
    {"_ZL31NumBruteForceTripCountsComputed", "NumBruteForceTripCountsComputed"},
    {"_ZL19NumArrayLenItCounts", "NumArrayLenItCounts"},
    {"_ZL24NumTripCountsNotComputed", "NumTripCountsNotComputed"},
    {"_ZL21NumTripCountsComputed", "NumTripCountsComputed"},
    {"_ZN12_GLOBAL__N_15statsL15RelaxationStepsE", "(anonymous namespace)::stats::RelaxationSteps"},
    {"_ZN12_GLOBAL__N_15statsL19RelaxedInstructionsE", "(anonymous namespace)::stats::RelaxedInstructions"},
    {"_ZN12_GLOBAL__N_15statsL11ObjectBytesE", "(anonymous namespace)::stats::ObjectBytes"},
    {"_ZN12_GLOBAL__N_15statsL16EmittedFragmentsE", "(anonymous namespace)::stats::EmittedFragments"},
    {"_ZN12_GLOBAL__N_15statsL15FragmentLayoutsE", "(anonymous namespace)::stats::FragmentLayouts"},
    {"_ZN12_GLOBAL__N_15statsL13EvaluateFixupE", "(anonymous namespace)::stats::EvaluateFixup"},
    {"_ZN12_GLOBAL__N_15statsL14MCExprEvaluateE", "(anonymous namespace)::stats::MCExprEvaluate"},
    {"_ZL11ProgramName", "ProgramName"},
    {"_ZL20ActiveASTUnitObjects", "ActiveASTUnitObjects"},
    {"_ZL12ConstEqIndex", "ConstEqIndex"},
    {"_ZL15ConstNotEqIndex", "ConstNotEqIndex"},
    {"_ZL11AutoRCIndex", "AutoRCIndex"},
    {"_ZL11AutoRBIndex", "AutoRBIndex"},
    {"_ZL20ConstraintRangeIndex", "ConstraintRangeIndex"},
    {"_ZL12nAccessSpecs", "nAccessSpecs"},
    {"_ZL7nBlocks", "nBlocks"},
    {"_ZL14nFileScopeAsms", "nFileScopeAsms"},
    {"_ZL8nFriends", "nFriends"},
    {"_ZL16nFriendTemplates", "nFriendTemplates"},
    {"_ZL13nLinkageSpecs", "nLinkageSpecs"},
    {"_ZL11nNamespaces", "nNamespaces"},
    {"_ZL16nNamespaceAliass", "nNamespaceAliass"},
    {"_ZL21nObjCCompatibleAliass", "nObjCCompatibleAliass"},
    {"_ZL14nObjCCategorys", "nObjCCategorys"},
    {"_ZL18nObjCCategoryImpls", "nObjCCategoryImpls"},
    {"_ZL20nObjCImplementations", "nObjCImplementations"},
    {"_ZL15nObjCInterfaces", "nObjCInterfaces"},
    {"_ZL14nObjCProtocols", "nObjCProtocols"},
    {"_ZL12nObjCMethods", "nObjCMethods"},
    {"_ZL14nObjCPropertys", "nObjCPropertys"},
    {"_ZL15nClassTemplates", "nClassTemplates"},
    {"_ZL18nFunctionTemplates", "nFunctionTemplates"},
    {"_ZL22nTemplateTemplateParms", "nTemplateTemplateParms"},
    {"_ZL6nEnums", "nEnums"},
    {"_ZL8nRecords", "nRecords"},
    {"_ZL11nCXXRecords", "nCXXRecords"},
    {"_ZL29nClassTemplateSpecializations", "nClassTemplateSpecializations"},
    {"_ZL36nClassTemplatePartialSpecializations", "nClassTemplatePartialSpecializations"},
    {"_ZL18nTemplateTypeParms", "nTemplateTypeParms"},
    {"_ZL9nTypedefs", "nTypedefs"},
    {"_ZL25nUnresolvedUsingTypenames", "nUnresolvedUsingTypenames"},
    {"_ZL7nUsings", "nUsings"},
    {"_ZL16nUsingDirectives", "nUsingDirectives"},
    {"_ZL13nUsingShadows", "nUsingShadows"},
    {"_ZL7nFields", "nFields"},
    {"_ZL17nObjCAtDefsFields", "nObjCAtDefsFields"},
    {"_ZL10nObjCIvars", "nObjCIvars"},
    {"_ZL10nFunctions", "nFunctions"},
    {"_ZL11nCXXMethods", "nCXXMethods"},
    {"_ZL16nCXXConstructors", "nCXXConstructors"},
    {"_ZL15nCXXConversions", "nCXXConversions"},
    {"_ZL15nCXXDestructors", "nCXXDestructors"},
    {"_ZL21nNonTypeTemplateParms", "nNonTypeTemplateParms"},
    {"_ZL5nVars", "nVars"},
    {"_ZL15nImplicitParams", "nImplicitParams"},
    {"_ZL9nParmVars", "nParmVars"},
    {"_ZL14nEnumConstants", "nEnumConstants"},
    {"_ZL15nIndirectFields", "nIndirectFields"},
    {"_ZL22nUnresolvedUsingValues", "nUnresolvedUsingValues"},
    {"_ZL11nObjCClasss", "nObjCClasss"},
    {"_ZL21nObjCForwardProtocols", "nObjCForwardProtocols"},
    {"_ZL18nObjCPropertyImpls", "nObjCPropertyImpls"},
    {"_ZL14nStaticAsserts", "nStaticAsserts"},
    {"_ZL17nTranslationUnits", "nTranslationUnits"},
    {"_ZL13StmtClassInfo", "StmtClassInfo"},
    {"_ZL11InlineLimit", "InlineLimit"},
    {"_ZL13HintThreshold", "HintThreshold"},
    {"_ZL13Disable256Bit", "Disable256Bit"},
    {"_ZL8NoFusing", "NoFusing"},
    {"_ZL17PrintFailedFusing", "PrintFailedFusing"},
    {"_ZL16ReMatPICStubLoad", "ReMatPICStubLoad"},
    {"_ZL19JITCompilerFunction", "JITCompilerFunction"},
    {"_ZL15AsmWriterFlavor", "AsmWriterFlavor"},
    {"_ZN4llvm12_GLOBAL__N_1L6CCRVTsE", "llvm::(anonymous namespace)::CCRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L14CONTROL_REGVTsE", "llvm::(anonymous namespace)::CONTROL_REGVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12DEBUG_REGVTsE", "llvm::(anonymous namespace)::DEBUG_REGVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7FR32VTsE", "llvm::(anonymous namespace)::FR32VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7FR64VTsE", "llvm::(anonymous namespace)::FR64VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR16VTsE", "llvm::(anonymous namespace)::GR16VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR16_ABCDVTsE", "llvm::(anonymous namespace)::GR16_ABCDVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR16_NOREXVTsE", "llvm::(anonymous namespace)::GR16_NOREXVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR32VTsE", "llvm::(anonymous namespace)::GR32VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR32_ABCDVTsE", "llvm::(anonymous namespace)::GR32_ABCDVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR32_ADVTsE", "llvm::(anonymous namespace)::GR32_ADVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR32_NOREXVTsE", "llvm::(anonymous namespace)::GR32_NOREXVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR32_NOSPVTsE", "llvm::(anonymous namespace)::GR32_NOSPVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR32_TCVTsE", "llvm::(anonymous namespace)::GR32_TCVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7GR64VTsE", "llvm::(anonymous namespace)::GR64VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR64_ABCDVTsE", "llvm::(anonymous namespace)::GR64_ABCDVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR64_NOREXVTsE", "llvm::(anonymous namespace)::GR64_NOREXVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L18GR64_NOREX_NOSPVTsE", "llvm::(anonymous namespace)::GR64_NOREX_NOSPVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR64_NOSPVTsE", "llvm::(anonymous namespace)::GR64_NOSPVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L10GR64_TCVTsE", "llvm::(anonymous namespace)::GR64_TCVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR64_TCW64VTsE", "llvm::(anonymous namespace)::GR64_TCW64VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6GR8VTsE", "llvm::(anonymous namespace)::GR8VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR8_ABCD_HVTsE", "llvm::(anonymous namespace)::GR8_ABCD_HVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L13GR8_ABCD_LVTsE", "llvm::(anonymous namespace)::GR8_ABCD_LVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12GR8_NOREXVTsE", "llvm::(anonymous namespace)::GR8_NOREXVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8RFP32VTsE", "llvm::(anonymous namespace)::RFP32VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8RFP64VTsE", "llvm::(anonymous namespace)::RFP64VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8RFP80VTsE", "llvm::(anonymous namespace)::RFP80VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6RSTVTsE", "llvm::(anonymous namespace)::RSTVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L14SEGMENT_REGVTsE", "llvm::(anonymous namespace)::SEGMENT_REGVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8VR128VTsE", "llvm::(anonymous namespace)::VR128VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8VR256VTsE", "llvm::(anonymous namespace)::VR256VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7VR64VTsE", "llvm::(anonymous namespace)::VR64VTs"},
    {"_ZL14EnableARM3Addr", "EnableARM3Addr"},
    {"_ZL20ForceAllBaseRegAlloc", "ForceAllBaseRegAlloc"},
    {"_ZL21EnableLocalStackAlloc", "EnableLocalStackAlloc"},
    {"_ZL17EnableBasePointer", "EnableBasePointer"},
    {"_ZN4llvm12_GLOBAL__N_1L6CCRVTsE", "llvm::(anonymous namespace)::CCRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6DPRVTsE", "llvm::(anonymous namespace)::DPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8DPR_8VTsE", "llvm::(anonymous namespace)::DPR_8VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L11DPR_VFP2VTsE", "llvm::(anonymous namespace)::DPR_VFP2VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6GPRVTsE", "llvm::(anonymous namespace)::GPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6QPRVTsE", "llvm::(anonymous namespace)::QPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8QPR_8VTsE", "llvm::(anonymous namespace)::QPR_8VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L11QPR_VFP2VTsE", "llvm::(anonymous namespace)::QPR_VFP2VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7QQPRVTsE", "llvm::(anonymous namespace)::QQPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L12QQPR_VFP2VTsE", "llvm::(anonymous namespace)::QQPR_VFP2VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L9QQQQPRVTsE", "llvm::(anonymous namespace)::QQQQPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L6SPRVTsE", "llvm::(anonymous namespace)::SPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8SPR_8VTsE", "llvm::(anonymous namespace)::SPR_8VTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7rGPRVTsE", "llvm::(anonymous namespace)::rGPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L7tGPRVTsE", "llvm::(anonymous namespace)::tGPRVTs"},
    {"_ZN4llvm12_GLOBAL__N_1L8tcGPRVTsE", "llvm::(anonymous namespace)::tcGPRVTs"},
    {"_ZL21AdjustJumpTableBlocks", "AdjustJumpTableBlocks"},
    {"_ZL18DisableARMFastISel", "DisableARMFastISel"},
    {"_ZL16DisableShifterOp", "DisableShifterOp"},
    {"_ZL15CheckVMLxHazard", "CheckVMLxHazard"},
    {"_ZL18EnableARMTailCalls", "EnableARMTailCalls"},
    {"_ZL15ARMInterworking", "ARMInterworking"},
    {"_ZL19JITCompilerFunction", "JITCompilerFunction"},
    {"_ZL9ReserveR9", "ReserveR9"},
    {"_ZL13DarwinUseMOVT", "DarwinUseMOVT"},
    {"_ZL11StrictAlign", "StrictAlign"},
    {"_ZL9ExpandMLx", "ExpandMLx"},
    {"_ZL11ForceExapnd", "ForceExapnd"},
    {"_ZL11ExpandLimit", "ExpandLimit"},
    {"_ZL10OldT2IfCvt", "OldT2IfCvt"},
    {"_ZL11ReduceLimit", "ReduceLimit"},
    {"_ZL16ReduceLimit2Addr", "ReduceLimit2Addr"},
    {"_ZL15ReduceLimitLdSt", "ReduceLimitLdSt"},
    {"_ZN12_GLOBAL__N_1L10CombinerAAE", "(anonymous namespace)::CombinerAA"},
    {"_ZN12_GLOBAL__N_1L16CombinerGlobalAAE", "(anonymous namespace)::CombinerGlobalAA"},
    {"_ZL21EnableExpensiveChecks", "EnableExpensiveChecks"},
    {"_ZL18tdListDAGScheduler", "tdListDAGScheduler"},
    {"_ZL20burrListDAGScheduler", "burrListDAGScheduler"},
    {"_ZL20tdrListrDAGScheduler", "tdrListrDAGScheduler"},
    {"_ZL22sourceListDAGScheduler", "sourceListDAGScheduler"},
    {"_ZL22hybridListDAGScheduler", "hybridListDAGScheduler"},
    {"_ZL19ILPListDAGScheduler", "ILPListDAGScheduler"},
    {"_ZL18DisableSchedCycles", "DisableSchedCycles"},
    {"_ZL4EVTs", "EVTs"},
    {"_ZL13SimpleVTArray", "SimpleVTArray"},
    {"_ZL7VTMutex", "VTMutex"},
    {"_ZL16LimitFPPrecision", "LimitFPPrecision"},
    {"_ZL19LimitFloatPrecision", "LimitFloatPrecision"},
    {"_ZL17MaxParallelChains", "MaxParallelChains"},
    {"_ZL21EnableFastISelVerbose", "EnableFastISelVerbose"},
    {"_ZL19EnableFastISelAbort", "EnableFastISelAbort"},
    {"_ZL11ISHeuristic", "ISHeuristic"},
    {"_ZL23defaultListDAGScheduler", "defaultListDAGScheduler"},
    {"_ZL13PrintDbgScope", "PrintDbgScope"},
    {"_ZL24DisableDebugInfoPrinting", "DisableDebugInfoPrinting"},
    {"_ZL16UnknownLocations", "UnknownLocations"},
    {"_ZL8DebugDiv", "DebugDiv"},
    {"_ZL8DebugMod", "DebugMod"},
    {"_ZL19FlagEnableTailMerge", "FlagEnableTailMerge"},
    {"_ZL18TailMergeThreshold", "TailMergeThreshold"},
    {"_ZL13TailMergeSize", "TailMergeSize"},
    {"_ZL15ViewEdgeBundles", "ViewEdgeBundles"},
    {"_ZL12IfCvtFnStart", "IfCvtFnStart"},
    {"_ZL11IfCvtFnStop", "IfCvtFnStop"},
    {"_ZL10IfCvtLimit", "IfCvtLimit"},
    {"_ZL13DisableSimple", "DisableSimple"},
    {"_ZL14DisableSimpleF", "DisableSimpleF"},
    {"_ZL15DisableTriangle", "DisableTriangle"},
    {"_ZL16DisableTriangleR", "DisableTriangleR"},
    {"_ZL16DisableTriangleF", "DisableTriangleF"},
    {"_ZL17DisableTriangleFR", "DisableTriangleFR"},
    {"_ZL14DisableDiamond", "DisableDiamond"},
    {"_ZL15IfCvtBranchFold", "IfCvtBranchFold"},
    {"_ZL12VerifySpills", "VerifySpills"},
    {"_ZL13DisablePostRA", "DisablePostRA"},
    {"_ZL17DisableBranchFold", "DisableBranchFold"},
    {"_ZL20DisableTailDuplicate", "DisableTailDuplicate"},
    {"_ZL19DisableEarlyTailDup", "DisableEarlyTailDup"},
    {"_ZL16DisableCodePlace", "DisableCodePlace"},
    {"_ZL10DisableSSC", "DisableSSC"},
    {"_ZL18DisableMachineLICM", "DisableMachineLICM"},
    {"_ZL24DisablePostRAMachineLICM", "DisablePostRAMachineLICM"},
    {"_ZL18DisableMachineSink", "DisableMachineSink"},
    {"_ZL10DisableLSR", "DisableLSR"},
    {"_ZL10DisableCGP", "DisableCGP"},
    {"_ZL8PrintLSR", "PrintLSR"},
    {"_ZL14PrintISelInput", "PrintISelInput"},
    {"_ZL11PrintGCInfo", "PrintGCInfo"},
    {"_ZL14ShowMCEncoding", "ShowMCEncoding"},
    {"_ZL10ShowMCInst", "ShowMCInst"},
    {"_ZL15EnableMCLogging", "EnableMCLogging"},
    {"_ZL17VerifyMachineCode", "VerifyMachineCode"},
    {"_ZL10AsmVerbose", "AsmVerbose"},
    {"_ZL20EnableFastISelOption", "EnableFastISelOption"},
    {"_ZL17EnableSplitGEPGVN", "EnableSplitGEPGVN"},
    {"_ZL9EnableLDV", "EnableLDV"},
    {"_ZL12DisableReMat", "DisableReMat"},
    {"_ZL10SplitEdges", "SplitEdges"},
    {"_ZL15defaultRegAlloc", "defaultRegAlloc"},
    {"_ZL8RegAlloc", "RegAlloc"},
    {"_ZL10Aggressive", "Aggressive"},
    {"_ZL15DisablePeephole", "DisablePeephole"},
    {"_ZL21EnablePostRAScheduler", "EnablePostRAScheduler"},
    {"_ZL21EnableAntiDepBreaking", "EnableAntiDepBreaking"},
    {"_ZL8DebugDiv", "DebugDiv"},
    {"_ZL8DebugMod", "DebugMod"},
    {"_ZL13PreSplitLimit", "PreSplitLimit"},
    {"_ZL14DeadSplitLimit", "DeadSplitLimit"},
    {"_ZL16RestoreFoldLimit", "RestoreFoldLimit"},
    {"_ZN12_GLOBAL__N_1L10PSVGlobalsE", "(anonymous namespace)::PSVGlobals"},
    {"_ZL12fastRegAlloc", "fastRegAlloc"},
    {"_ZL12NewHeuristic", "NewHeuristic"},
    {"_ZL17PreSplitIntervals", "PreSplitIntervals"},
    {"_ZL16TrivCoalesceEnds", "TrivCoalesceEnds"},
    {"_ZL18linearscanRegAlloc", "linearscanRegAlloc"},
    {"_ZN12_GLOBAL__N_1L19NumRecentlyUsedRegsE", "(anonymous namespace)::NumRecentlyUsedRegs"},
    {"_ZL14ShrinkWrapping", "ShrinkWrapping"},
    {"_ZL14ShrinkWrapFunc", "ShrinkWrapFunc"},
    {"_ZL19ShrinkWrapDebugging", "ShrinkWrapDebugging"},
    {"_ZL13EnableJoining", "EnableJoining"},
    {"_ZL21DisableCrossClassJoin", "DisableCrossClassJoin"},
    {"_ZL19DisablePhysicalJoin", "DisablePhysicalJoin"},
    {"_ZL16VerifyCoalescing", "VerifyCoalescing"},
    {"_ZL10spillerOpt", "spillerOpt"},
    {"_ZL13SSPBufferSize", "SSPBufferSize"},
    {"_ZL14DisableSharing", "DisableSharing"},
    {"_ZL16ColorWithRegsOpt", "ColorWithRegsOpt"},
    {"_ZL8DCELimit", "DCELimit"},
    {"_ZL17TailDuplicateSize", "TailDuplicateSize"},
    {"_ZL13TailDupVerify", "TailDupVerify"},
    {"_ZL12TailDupLimit", "TailDupLimit"},
    {"_ZL23DisableHazardRecognizer", "DisableHazardRecognizer"},
    {"_ZL11RewriterOpt", "RewriterOpt"},
    {"_ZL14ScheduleSpills", "ScheduleSpills"},
    {"_ZL17CriticalEdgeSplit", "CriticalEdgeSplit"},
    {"_ZL9EnablePRE", "EnablePRE"},
    {"_ZL13EnableLoadPRE", "EnableLoadPRE"},
    {"_ZL9Threshold", "Threshold"},
    {"_ZL16DisablePromotion", "DisablePromotion"},
    {"_ZL15UnrollThreshold", "UnrollThreshold"},
    {"_ZL11UnrollCount", "UnrollCount"},
    {"_ZL18UnrollAllowPartial", "UnrollAllowPartial"},
    {"_ZL9Threshold", "Threshold"},
    {"_ZL18ExpensiveEHSupport", "ExpensiveEHSupport"},
    {"_ZL6DupRet", "DupRet"},
    {"_ZL13MaxIterations", "MaxIterations"},
    {"_ZL14VerifyLoopInfo", "VerifyLoopInfo"},
    {"_ZL15VerifyLoopInfoX", "VerifyLoopInfoX"},
    {"_ZL23MaxBruteForceIterations", "MaxBruteForceIterations"},
    {"_ZL10EnableTBAA", "EnableTBAA"},
    {"_ZL9PrintCode", "PrintCode"},
    {"_ZL13DisableFPElim", "DisableFPElim"},
    {"_ZL20DisableFPElimNonLeaf", "DisableFPElimNonLeaf"},
    {"_ZL22DisableExcessPrecision", "DisableExcessPrecision"},
    {"_ZL11EnableFPMAD", "EnableFPMAD"},
    {"_ZL18EnableUnsafeFPMath", "EnableUnsafeFPMath"},
    {"_ZL18EnableNoInfsFPMath", "EnableNoInfsFPMath"},
    {"_ZL18EnableNoNaNsFPMath", "EnableNoNaNsFPMath"},
    {"_ZL38EnableHonorSignDependentRoundingFPMath", "EnableHonorSignDependentRoundingFPMath"},
    {"_ZL22GenerateSoftFloatCalls", "GenerateSoftFloatCalls"},
    {"_ZL16FloatABIForCalls", "FloatABIForCalls"},
    {"_ZL19DontPlaceZerosInBSS", "DontPlaceZerosInBSS"},
    {"_ZL26EnableJITExceptionHandling", "EnableJITExceptionHandling"},
    {"_ZL16EmitJitDebugInfo", "EmitJitDebugInfo"},
    {"_ZL22EmitJitDebugInfoToDisk", "EmitJitDebugInfoToDisk"},
    {"_ZL18EnableUnwindTables", "EnableUnwindTables"},
    {"_ZL18DefRelocationModel", "DefRelocationModel"},
    {"_ZL12DefCodeModel", "DefCodeModel"},
    {"_ZL27EnableGuaranteedTailCallOpt", "EnableGuaranteedTailCallOpt"},
    {"_ZL22OverrideStackAlignment", "OverrideStackAlignment"},
    {"_ZL18EnableRealignStack", "EnableRealignStack"},
    {"_ZL19DisableSwitchTables", "DisableSwitchTables"},
    {"_ZL19EnableStrongPHIElim", "EnableStrongPHIElim"},
    {"_ZL12DataSections", "DataSections"},
    {"_ZL16FunctionSections", "FunctionSections"},
    {"_ZL15AttributesLists", "AttributesLists"},
    {"_ZN4llvmL7ALMutexE", "llvm::ALMutex"},
    {"_ZL13VerifyDomInfo", "VerifyDomInfo"},
    {"_ZL14VerifyDomInfoX", "VerifyDomInfoX"},
    {"_ZL6GCLock", "GCLock"},
    {"_ZL7GCNames", "GCNames"},
    {"_ZL10GCNamePool", "GCNamePool"},
    {"_ZL13GlobalContext", "GlobalContext"},
    {"_ZN4llvmL13PassDebuggingE", "llvm::PassDebugging"},
    {"_ZN4llvmL11PrintBeforeE", "llvm::PrintBefore"},
    {"_ZN4llvmL10PrintAfterE", "llvm::PrintAfter"},
    {"_ZN4llvmL14PrintBeforeAllE", "llvm::PrintBeforeAll"},
    {"_ZN4llvmL13PrintAfterAllE", "llvm::PrintAfterAll"},
    {"_ZN12_GLOBAL__N_1L15TimingInfoMutexE", "(anonymous namespace)::TimingInfoMutex"},
    {"_ZL12EnableTiming", "EnableTiming"},
    {"_ZL11TheTimeInfo", "TheTimeInfo"},
    {"_ZL15PassRegistryObj", "PassRegistryObj"},
    {"_ZL4Lock", "Lock"},
    {"_ZL8MoreHelp", "MoreHelp"},
    {"_ZL20RegisteredOptionList", "RegisteredOptionList"},
    {"_ZL15ProgramOverview", "ProgramOverview"},
    {"_ZL13NormalPrinter", "NormalPrinter"},
    {"_ZL13HiddenPrinter", "HiddenPrinter"},
    {"_ZL3HOp", "HOp"},
    {"_ZL4HHOp", "HHOp"},
    {"_ZL6VersOp", "VersOp"},
    {"_ZL22VersionPrinterInstance", "VersionPrinterInstance"},
    {"_ZL22OverrideVersionPrinter", "OverrideVersionPrinter"},
    {"_ZN12_GLOBAL__N_1L14CurrentContextE", "(anonymous namespace)::CurrentContext"},
    {"_ZL25gCrashRecoveryContexMutex", "gCrashRecoveryContexMutex"},
    {"_ZL11PrevActions", "PrevActions"},
    {"_ZL5Dummy", "Dummy"},
    {"_ZL15ExplicitSymbols", "ExplicitSymbols"},
    {"_ZL13OpenedHandles", "OpenedHandles"},
    {"_ZL12ErrorHandler", "ErrorHandler"},
    {"_ZL20ErrorHandlerUserData", "ErrorHandlerUserData"},
    {"_ZL10StaticList", "StaticList"},
    {"_ZN12_GLOBAL__N_1L7successE", "(anonymous namespace)::success"},
    {"_ZN12_GLOBAL__N_1L10separatorsE", "(anonymous namespace)::separators"},
    {"_ZN12_GLOBAL__N_1L7successE", "(anonymous namespace)::success"},
    {"_ZL20PrettyStackTraceHead", "PrettyStackTraceHead"},
    {"_ZL12SignalsMutex", "SignalsMutex"},
    {"_ZL13FilesToRemove", "FilesToRemove"},
    {"_ZL14CallBacksToRun", "CallBacksToRun"},
    {"_ZL17InterruptFunction", "InterruptFunction"},
    {"_ZL20NumRegisteredSignals", "NumRegisteredSignals"},
    {"_ZL20RegisteredSignalInfo", "RegisteredSignalInfo"},
    {"_ZL7Enabled", "Enabled"},
    {"_ZL8StatInfo", "StatInfo"},
    {"_ZL8StatLock", "StatLock"},
    {"_ZL11FirstTarget", "FirstTarget"},
    {"_ZL11global_lock", "global_lock"},
    {"_ZL28LibSupportInfoOutputFilename", "LibSupportInfoOutputFilename"},
    {"_ZL9TimerLock", "TimerLock"},
    {"_ZN12_GLOBAL__N_1L10TrackSpaceE", "(anonymous namespace)::TrackSpace"},
    {"_ZN12_GLOBAL__N_1L18InfoOutputFilenameE", "(anonymous namespace)::InfoOutputFilename"},
    {"_ZL12ActiveTimers", "ActiveTimers"},
    {"_ZL11NamedTimers", "NamedTimers"},
    {"_ZL18NamedGroupedTimers", "NamedGroupedTimers"},
    {"_ZL14TimerGroupList", "TimerGroupList"},
    {"_ZL17DefaultTimerGroup", "DefaultTimerGroup"},
    {"_ZL16NotUnderValgrind", "NotUnderValgrind"},
    {"_ZN8Blizzard6Memory12voidp_returncvPT_IcEEv", "Blizzard::Memory::voidp_return::operator char*<char>()"},
    {"_Z2f0PU3AS1c", "f0(char AS1*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXleT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) <= (4), void>::type*)"},
    {"_ZN5Casts6cstyleILj4EEEvPN9enable_ifIXleT_cvjLi4EEvE4typeE", "void Casts::cstyle<4u>(enable_if<(4u) <= ((unsigned int)(4)), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXaaT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) && (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXaNT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) &= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXaST_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) = (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXstT_EvE4typeE", "void Casts::implicit<4u>(enable_if<sizeof (4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXszT_EvE4typeE", "void Casts::implicit<4u>(enable_if<sizeof (4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXatT_EvE4typeE", "void Casts::implicit<4u>(enable_if<alignof (4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXazT_EvE4typeE", "void Casts::implicit<4u>(enable_if<alignof (4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXcmT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) , (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXcoT_EvE4typeE", "void Casts::implicit<4u>(enable_if<~(4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXdeT_EvE4typeE", "void Casts::implicit<4u>(enable_if<*(4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXdvT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) / (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXdVT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) /= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXeoT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) ^ (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXeOT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) ^= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXeqT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) == (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXgeT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) >= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXgtT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<((4u) > (4)), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXlsT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) << (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXlST_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) <<= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXltT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) < (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXmiT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) - (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXmIT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) -= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXmlT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) * (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXmLT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) *= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXmmT_EvE4typeE", "void Casts::implicit<4u>(enable_if<(4u)--, void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXneT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) != (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXngT_EvE4typeE", "void Casts::implicit<4u>(enable_if<-(4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXntT_EvE4typeE", "void Casts::implicit<4u>(enable_if<!(4u), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXooT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) || (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXorT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) | (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXoRT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) |= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXpmT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) ->* (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXplT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) + (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXpLT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) += (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXppT_EvE4typeE", "void Casts::implicit<4u>(enable_if<(4u)++, void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXpsT_EvE4typeE", "void Casts::implicit<4u>(enable_if<+(4u), void>::type*)"},
//    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXptT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) -> (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXquT_Li4ELi5EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) ? (4) : (5), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXrmT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) % (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXrMT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) %= (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXrsT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) >> (4), void>::type*)"},
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXrST_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) >>= (4), void>::type*)"},
    {"_Z1fPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP1XS13_S12_S11_S10_SZ_SY_SX_SW_SV_SU_ST_SS_SR_SQ_SP_SO_SN_SM_SL_SK_SJ_SI_SH_SG_SF_SE_SD_SC_SB_SA_S9_S8_S7_S6_S5_S4_S3_S2_S1_S0_S_", "f(X****************************************, X****************************************, X***************************************, X**************************************, X*************************************, X************************************, X***********************************, X**********************************, X*********************************, X********************************, X*******************************, X******************************, X*****************************, X****************************, X***************************, X**************************, X*************************, X************************, X***********************, X**********************, X*********************, X********************, X*******************, X******************, X*****************, X****************, X***************, X**************, X*************, X************, X***********, X**********, X*********, X********, X*******, X******, X*****, X****, X***, X**, X*, X)"},
    {"_ZZN1J1KEvENK1C1FEv", "J::K()::C::F() const"},
    {"_ZZNVK1J1KEvENK1C1FEv", "J::K() const volatile::C::F() const"},
    {"_Z1fM1AKFvvE", "f(void (A::*)() const)"},
    {"_ZNR1X1fEv", "X::f() &"},
    {"_ZNKO1X1hEv", "X::h() const &&"},
//    {"_Z1fM1XVKFivEMS_VFivEMS_KOFivE", "f(int (X::*)() const volatile, int (X::*)() volatile, int (X::*)() const &&)"},
//    {"_Z1fM1XRFivEMS_OFivEMS_KOFivE", "f(int (X::*)() &, int (X::*)() &&, int (X::*)() const &&)"},
    {"_ZN5test12f0ENS_1TILZNS_1xEEEE", "test1::f0(test1::T<test1::x>)"},
    {"_ZN5test12f1ENS_2t1ILZNS_2f0EfEEE", "test1::f1(test1::t1<test1::f0(float)>)"},
    {"_ZN5test22f1ENS_2t1IXadL_ZNS_2f0EfEEEE", "test2::f1(test2::t1<&(test2::f0(float))>)"},
    {"_ZN5test32f1ENS_2t1ILZ8test3_f0EEE", "test3::f1(test3::t1<test3_f0>)"},
    {"_ZN5test42f1ENS_2t1IXadL_Z8test4_f0EEEE", "test4::f1(test4::t1<&(test4_f0)>)"},
    {"_ZN5test52f1ENS_2t1ILZ8test5_f0EEE", "test5::f1(test5::t1<test5_f0>)"},
    {"_ZN5test52f2ENS_2t2ILZ4mainEEE", "test5::f2(test5::t2<main>)"},
    {"_ZN5test52f2ENS_2t2ILZ4mainEEE", "test5::f2(test5::t2<main>)"},
    {"_ZN5test62f0ENS_1TIXadL_ZNS_1A3im0EfEEEE", "test6::f0(test6::T<&(test6::A::im0(float))>)"},
    {"_ZN5test71XIiEC1IdEEPT_PNS_5int_cIXplL_ZNS_4metaIiE5valueEEsrNS6_IS3_EE5valueEE4typeE", "test7::X<int>::X<double>(double*, test7::int_c<(test7::meta<int>::value) + (test7::meta<double>::value)>::type*)"},
    {"_ZN5test81fIiEEvNS_5int_cIXsrNS_4metaIT_E4typeE5valueEEE", "void test8::f<int>(test8::int_c<test8::meta<int>::type::value>)"},
    {"_ZN5test91fIiNS_1XEEENS_9supermetaIT_E5applyIT0_E4typeEv", "test9::supermeta<int>::apply<test9::X>::type test9::f<int, test9::X>()"},
    {"_ZN6test101fIidEENS_1XIT_E10definitionIT0_EES2_S5_", "test10::X<int>::definition<double> test10::f<int, double>(int, double)"},
    {"_ZZ2f5vE1a", "f5()::a"},
    {"_ZZ2f6vE1b", "f6()::b"},
    {"_ZNV3$_35test9Ev", "$_3::test9() volatile"},
    {"_ZN5Test8I3$_2EC1ES0_", "Test8<$_2>::Test8($_2)"},
    {"_Z3fooIZN3BarC1EvE3$_0EvT_", "void foo<Bar::Bar()::$_0>(Bar::Bar()::$_0)"},
    {"_ZGVZN1N1gEvE1a", "guard variable for N::g()::a"},
    {"_ZplRK1YRA100_P1X", "operator+(Y const&, X* (&) [100])"},
    {"_Z1fno", "f(__int128, unsigned __int128)"},
    {"_Z1fM1SKFvvE", "f(void (S::*)() const)"},
    {"_Z3ft7IiEN11__enable_ifIXsr16__is_scalar_typeIT_EE7__valueEvE6__typeEv", "__enable_if<__is_scalar_type<int>::__value, void>::__type ft7<int>()"},
    {"_Z3ft7IPvEN11__enable_ifIXsr16__is_scalar_typeIT_EE7__valueEvE6__typeEv", "__enable_if<__is_scalar_type<void*>::__value, void>::__type ft7<void*>()"},
    {"_ZN6PR57968__fill_aIiEENS_11__enable_ifIXntsrNS_11__is_scalarIT_EE7__valueEvE6__typeEv", "PR5796::__enable_if<!(PR5796::__is_scalar<int>::__value), void>::__type PR5796::__fill_a<int>()"},
    {"_ZN11Expressions2f1ILi1EEEvPAplngT_Li2E_i", "void Expressions::f1<1>(int (*) [(-(1)) + (2)])"},
    {"_ZN11Expressions2f2ILi1EEEvPApsT__i", "void Expressions::f2<1>(int (*) [+(1)])"},
    {"_ZN11Expressions2f3ILi1EEEvPAplT_T__i", "void Expressions::f3<1>(int (*) [(1) + (1)])"},
    {"_ZN11Expressions2f4ILi1EEEvPAplplLi2ET_T__i", "void Expressions::f4<1>(int (*) [((2) + (1)) + (1)])"},
    {"_ZN3OpsplERKS_", "Ops::operator+(Ops const&)"},
    {"_ZN6PR58615AllocIcNS_6PolicyINS_1PELb1EEEE8allocateEiPKv", "PR5861::Alloc<char, PR5861::Policy<PR5861::P, true> >::allocate(int, void const*)"},
    {"_ZN5test01fIdEEvT_RAszcl3ovlcvS1__EE_c", "void test0::f<double>(double, char (&) [sizeof (ovl((double)()))])"},
    {"_ZN5test01fIiEEvT_RAszcl3ovlcvS1__EE_c", "void test0::f<int>(int, char (&) [sizeof (ovl((int)()))])"},
    {"_ZN5test01jINS_1AEEEvRAszdtcvT__E6buffer_c", "void test0::j<test0::A>(char (&) [sizeof ((test0::A)().buffer)])"},
    {"_ZN5test11fINS_1XEiEEvT_IT0_E", "void test1::f<test1::X, int>(test1::X<int>)"},
    {"_ZN5test211read_memberINS_1AEEEDtptcvPT_Li0E6memberERS2_", "decltype((test2::A*)(0)->member) test2::read_member<test2::A>(test2::A&)"},
    {"_ZN5test37get_p_1INS_7DerivedEEEDtptcvPT_Li0EsrNS_5Path1E1pERS2_", "decltype((test3::Derived*)(0)->test3::Path1::p) test3::get_p_1<test3::Derived>(test3::Derived&)"},
    {"_ZN5test37get_p_1INS_7DerivedEEEDtptcvPT_Li0EsrNS_5Path2E1pERS2_", "decltype((test3::Derived*)(0)->test3::Path2::p) test3::get_p_1<test3::Derived>(test3::Derived&)"},
    {"_ZN5test41gEPNS_3zedIXadL_ZNS_3foo3barEEEEE", "test4::g(test4::zed<&(test4::foo::bar)>*)"},
    {"_ZN5test51gEPNS_3zedIXadL_ZNS_3foo3barEEEEE", "test5::g(test5::zed<&(test5::foo::bar)>*)"},
    {"_ZN5test71gEPNS_3zedIXadL_ZNS_3foo3barEvEEEE", "test7::g(test7::zed<&(test7::foo::bar())>*)"},
    {"_ZN5test91fIiNS_3barEEEvRKNT0_3baz1XE", "void test9::f<int, test9::bar>(test9::bar::baz::X const&)"},
    {"_ZN6test101fILc3EEEvNS_1SIXquLb0ELc97ET_EEE", "void test10::f<(char)3>(test10::S<(false) ? ((char)97) : ((char)3)>)"},
    {"_ZN6test111fEz", "test11::f(...)"},
    {"_ZN6test121fENS_1AILt33000EEE", "test12::f(test12::A<(unsigned short)33000>)"},
    {"_ZN6test151fILi7EEEvNS_1SIXplT_LNS_1EE3EEEE", "void test15::f<7>(test15::S<(7) + ((test15::E)3)>)"},
    {"_ZN6test174funcINS_1BEEENS_1AIXszclsrT_3fooEEEEv", "test17::A<sizeof (test17::B::foo())> test17::func<test17::B>()"},
    {"_ZN6test181fINS_1AEEEvNS_1SIXadsrT_plEEE", "void test18::f<test18::A>(test18::S<&(test18::A::operator+)>)"},
    {"_ZN6test181fINS_1AEEEvNS_1SIXadsrT_anEEE", "void test18::f<test18::A>(test18::S<&(test18::A::operator&)>)"},
    {"_ZN6test191gINS_1AEEEvNS_1SIXadsrT_1fIiEEEE", "void test19::g<test19::A>(test19::S<&(test19::A::f<int>)>)"},
    {"_ZN6test191gINS_1AEEEvNS_1SIXadsrT_plEEE", "void test19::g<test19::A>(test19::S<&(test19::A::operator+)>)"},
    {"_ZN6test191gINS_1AEEEvNS_1SIXadsrT_cviEEE", "void test19::g<test19::A>(test19::S<&(test19::A::operator int)>)"},
    {"_ZN6test191gINS_1AEEEvNS_1SIXadsrT_miIdEEEE", "void test19::g<test19::A>(test19::S<&(test19::A::operator-<double>)>)"},
    {"_ZN6test205test0IiEEvDTcl1fIPT_ELi0EEE", "void test20::test0<int>(decltype(f<int*>(0)))"},
    {"_ZN6test2112vla_arg_funcEiPA_i", "test21::vla_arg_func(int, int (*) [])"},
    {"_ZN6test221fEDn", "test22::f(std::nullptr_t)"},
    {"_ZN6test231fERA10_KPv", "test23::f(void* const (&) [10])"},
    {"_ZN6test231fERA10_A5_VKPv", "test23::f(void* const volatile (&) [10][5])"},
    {"_Z4testIcLj5ELj10ELj15EEvRAT0__AT1__AT2__T_", "void test<char, 5u, 10u, 15u>(char (&) [5u][10u][15u])"},
    {"_ZN6test205test1IiEEvDTcl1fIEcvT__EEE", "void test20::test1<int>(decltype(f<>((int)())))"},
    {"_Z2f0IJEEv1XIXsZT_EJDpRT_EE", "void f0<>(X<sizeof...()>)"},
    {"_Z2f0IJifdEEv1XIXsZT_EJDpRT_EE", "void f0<int, float, double>(X<sizeof...(int, float, double), int&, float&, double&>)"},
    {"_Z2f1IJEEvv", "void f1<>()"},
    {"_Z2f1IJiEEvv", "void f1<int>()"},
    {"_Z2f1IJifEEvv", "void f1<int, float>()"},
    {"_Z2f2IJEEvDpT_", "void f2<>()"},
    {"_Z2f2IJiEEvDpT_", "void f2<int>(int)"},
    {"_Z2f2IJifEEvDpT_", "void f2<int, float>(int, float)"},
    {"_Z2f3IJEEvDpPKT_", "void f3<>()"},
    {"_Z2f3IJiEEvDpPKT_", "void f3<int>(int const*)"},
    {"_Z2f3IJifEEvDpPKT_", "void f3<int, float>(int const*, float const*)"},
    {"_Z2f4IJifdEE5tupleIJDpT_EEv", "tuple<int, float, double> f4<int, float, double>()"},
    {"_Z2f5IiJifdEE8identityIFT_DpT0_EEv", "identity<int (int, float, double)> f5<int, int, float, double>()"},
    {"_Z2f6IJLi1ELi2ELi3EEE9int_tupleIJXspT_EEEv", "int_tuple<1, 2, 3> f6<1, 2, 3>()"},
    {"_Z2f7IJ8identity13add_referenceEE14template_tupleIJDpT_EEv", "template_tuple<identity, add_reference> f7<identity, add_reference>()"},
    {"_ZNK10__cxxabiv111__libcxxabi5__sub20first_demangled_nameEPc.eh", "__cxxabiv1::__libcxxabi::__sub::first_demangled_name(char*) const (.eh)"},
    {"_ZSt13copy_backwardIN9__gnu_cxx17__normal_iteratorIPNS0_19_Hashtable_iteratorISt4pairIK16CSCppSymbolOwnerP20CSCppSymbolOwnerDataES4_27CSCppSymbolOwnerHashFunctorSt10_Select1stIS8_E29CSCppSymbolOwnerEqualsFunctorSaIS7_EEESt6vectorISE_SaISE_EEEESJ_ET0_T_SL_SK_", "__gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > > std::copy_backward<__gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > >, __gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > > >(__gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > >, __gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > >, __gnu_cxx::__normal_iterator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >*, std::vector<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> >, std::allocator<__gnu_cxx::_Hashtable_iterator<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*>, CSCppSymbolOwner, CSCppSymbolOwnerHashFunctor, std::_Select1st<std::pair<CSCppSymbolOwner const, CSCppSymbolOwnerData*> >, CSCppSymbolOwnerEqualsFunctor, std::allocator<CSCppSymbolOwnerData*> > > > >)"},
    {"_ZTC14SSDatabaseImpl0_N8Security10CssmClient6DbImplE", "construction vtable for Security::CssmClient::DbImpl-in-SSDatabaseImpl"},
    {"_ZGRZN1N1gEvE1a", "reference temporary for N::g()::a"},
    {"_ZN5boost4bindIvN10libtorrent15peer_connectionEiRKNS1_11disk_io_jobENS1_12peer_requestENS_10shared_ptrINS1_7torrentEEENS_13intrusive_ptrIS2_EENS_3argILi1EEENSC_ILi2EEES6_S9_EENS_3_bi6bind_tIT_NS_4_mfi3mf4ISH_T0_T1_T2_T3_T4_EENSF_9list_av_5IT5_T6_T7_T8_T9_E4typeEEEMSK_FSH_SL_SM_SN_SO_ESR_SS_ST_SU_SV_", "boost::_bi::bind_t<void, boost::_mfi::mf4<void, libtorrent::peer_connection, int, libtorrent::disk_io_job const&, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent> >, boost::_bi::list_av_5<boost::intrusive_ptr<libtorrent::peer_connection>, boost::arg<1>, boost::arg<2>, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent> >::type> boost::bind<void, libtorrent::peer_connection, int, libtorrent::disk_io_job const&, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent>, boost::intrusive_ptr<libtorrent::peer_connection>, boost::arg<1>, boost::arg<2>, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent> >(void (libtorrent::peer_connection::*)(int, libtorrent::disk_io_job const&, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent>), boost::intrusive_ptr<libtorrent::peer_connection>, boost::arg<1>, boost::arg<2>, libtorrent::peer_request, boost::shared_ptr<libtorrent::torrent>)"},
    {"_ZN7WebCore20createFileThreadTaskINS_15FileStreamProxyEN3WTF6StringERKS3_ddEENS2_10PassOwnPtrINS_10FileThread4TaskEEEPT_MSA_FvT1_T3_ERKT0_RKT2_", "WTF::PassOwnPtr<WebCore::FileThread::Task> WebCore::createFileThreadTask<WebCore::FileStreamProxy, WTF::String, WTF::String const&, double, double>(WebCore::FileStreamProxy*, void (WebCore::FileStreamProxy::*)(WTF::String const&, double), WTF::String const&, double const&)"},
    {"_ZN3WTF15deleteAllValuesIPN7WebCore5XPath4Step8NodeTestEKNS_9HashTableIS5_S5_NS_17IdentityExtractorENS_7PtrHashIS5_EENS_10HashTraitsIS5_EESB_EEEEvRT0_", "void WTF::deleteAllValues<WebCore::XPath::Step::NodeTest*, WTF::HashTable<WebCore::XPath::Step::NodeTest*, WebCore::XPath::Step::NodeTest*, WTF::IdentityExtractor, WTF::PtrHash<WebCore::XPath::Step::NodeTest*>, WTF::HashTraits<WebCore::XPath::Step::NodeTest*>, WTF::HashTraits<WebCore::XPath::Step::NodeTest*> > const>(WTF::HashTable<WebCore::XPath::Step::NodeTest*, WebCore::XPath::Step::NodeTest*, WTF::IdentityExtractor, WTF::PtrHash<WebCore::XPath::Step::NodeTest*>, WTF::HashTraits<WebCore::XPath::Step::NodeTest*>, WTF::HashTraits<WebCore::XPath::Step::NodeTest*> > const&)"},
    {"_Z13JVTLib_103270ILi1EEvPsDv2_xS1_", "void JVTLib_103270<1>(short*, long long vector[2], long long vector[2])"},
    {"_ZN8platform20split_string_convertIcPFiRKSsEiSaIiESt6vectorEEjPKT_S9_S7_T0_RT3_IT1_T2_E", "unsigned int platform::split_string_convert<char, int (*)(std::string const&), int, std::allocator<int>, std::vector>(char const*, char const*, char, int (*)(std::string const&), std::vector<int, std::allocator<int> >&)"},
    {"_ZN2MF12_GLOBAL__N_114WeakCallHelperINS0_15DecodeQueueImplEEEvRKN5boost8functionIFvvEEERKNS3_8weak_ptrIT_EE", "void MF::(anonymous namespace)::WeakCallHelper<MF::(anonymous namespace)::DecodeQueueImpl>(boost::function<void ()> const&, boost::weak_ptr<MF::(anonymous namespace)::DecodeQueueImpl> const&)"},
    {"_ZZN4NIds4NStr14TCStrAggregateINS0_13TCTCStrTraitsINS0_11TCStrTraitsIcNS0_17CDefaultStrParamsEEENS0_14TCStrImp_FixedIS5_Lx256EEEEEE21f_AddFromIteratorUTF8INS0_16CStrIteratorUTF8EEEvRxRKT_ENKSA_ISB_EUlmE0_clEm", "void NIds::NStr::TCStrAggregate<NIds::NStr::TCTCStrTraits<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, NIds::NStr::TCStrImp_Fixed<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, 256ll> > >::f_AddFromIteratorUTF8<NIds::NStr::CStrIteratorUTF8>(long long&, NIds::NStr::CStrIteratorUTF8 const&)::NIds::NStr::TCStrAggregate<NIds::NStr::TCTCStrTraits<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, NIds::NStr::TCStrImp_Fixed<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, 256ll> > >::f_AddFromIteratorUTF8<NIds::NStr::CStrIteratorUTF8>::'lambda0'(unsigned long)::operator()(unsigned long) const"},
    {"_ZZN4NIds4NStr14TCStrAggregateINS0_13TCTCStrTraitsINS0_11TCStrTraitsIcNS0_17CDefaultStrParamsEEENS0_14TCStrImp_FixedIS5_Lx256EEEEEE21f_AddFromIteratorUTF8INS0_16CStrIteratorUTF8EEEvRxRKT_ENKSA_ISB_EUt0_clEm", "void NIds::NStr::TCStrAggregate<NIds::NStr::TCTCStrTraits<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, NIds::NStr::TCStrImp_Fixed<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, 256ll> > >::f_AddFromIteratorUTF8<NIds::NStr::CStrIteratorUTF8>(long long&, NIds::NStr::CStrIteratorUTF8 const&)::NIds::NStr::TCStrAggregate<NIds::NStr::TCTCStrTraits<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, NIds::NStr::TCStrImp_Fixed<NIds::NStr::TCStrTraits<char, NIds::NStr::CDefaultStrParams>, 256ll> > >::f_AddFromIteratorUTF8<NIds::NStr::CStrIteratorUTF8>::'unnamed0'::operator()(unsigned long) const"},
    {"_ZNK3com9markzware2js11cJSArgumentcvRKT_I8cMyClassEEv", "com::markzware::js::cJSArgument::operator cMyClass const&<cMyClass>() const"},
    {"_ZNKSt3__110__function6__funcIZN4DLCL8DLFutureIP15AnalysenManagerE3setINS_8functionIFS5_vEEEJEEEvT_DpOT0_EUlvE_NS_9allocatorISF_EEFvvEE7__cloneEv", "std::__1::__function::__func<void DLCL::DLFuture<AnalysenManager*>::set<std::__1::function<AnalysenManager* ()> >(std::__1::function<AnalysenManager* ()>)::'lambda'(), std::__1::allocator<void DLCL::DLFuture<AnalysenManager*>::set<std::__1::function<AnalysenManager* ()> >(std::__1::function<AnalysenManager* ()>)::'lambda'()>, void ()>::__clone() const"},
    {"___ZN19URLConnectionClient33_clientInterface_cancelConnectionEP16dispatch_queue_sU13block_pointerFvvE_block_invoke14", "invocation function for block in URLConnectionClient::_clientInterface_cancelConnection(dispatch_queue_s*, void () block_pointer)"},
    {"_Z1fIJicdEEPFvDpT_EPFvDpRPS0_ES8_S1_DpS4_S6_", "void (*f<int, char, double>(void (*)(int*&, char*&, double*&), void (*)(int*&, char*&, double*&), int, char, double, int*, char*, double*, int*&, char*&, double*&))(int, char, double)"},
    {"_ZSteqIcEN9__gnu_cxx11__enable_ifIXsr9__is_charIT_EE7__valueEbE6__typeERKSbIS2_St11char_traitsIS2_ESaIS2_EESA_", "__gnu_cxx::__enable_if<__is_char<char>::__value, bool>::__type std::operator==<char>(std::basic_string<char, std::char_traits<char>, std::allocator<char> > const&, std::basic_string<char, std::char_traits<char>, std::allocator<char> > const&)"},
    {"_ZZ10+[Foo bar]E3Baz", "+[Foo bar]::Baz"},
    {"_ZN9__gnu_cxx17__normal_iteratorIPKSt4pairISsbESt6vectorIS2_SaIS2_EEEC5ERKS4_", "__gnu_cxx::__normal_iterator<std::pair<std::string, bool> const*, std::vector<std::pair<std::string, bool>, std::allocator<std::pair<std::string, bool> > > >::__normal_iterator(std::pair<std::string, bool> const* const&)"},
    {"_Z1fIiEDTeqfp_LDnEEPT_", "decltype((fp) == (nullptr)) f<int>(int*)"},
    {"_Z1fIiEDTeqfp1_LDnEEicPT_", "decltype((fp1) == (nullptr)) f<int>(int, char, int*)"},
    {"_ZZN1S1fEiiEd0_NKUlvE_clEv", "S::f(int, int)::'lambda'()::operator()() const"},
    {"_Z3fooPM2ABi", "foo(int AB::**)"},
    {"_Z1rM1GFivEMS_KFivES_M1HFivES1_4whatIKS_E5what2IS8_ES3_", "r(int (G::*)(), int (G::*)() const, G, int (H::*)(), int (G::*)(), what<G const>, what2<G const>, int (G::*)() const)"},
    {"_Z1fPU11objcproto1A11objc_object", "f(id<A>)"},
    {"_Z1fPKU11objcproto1A7NSArray", "f(NSArray<A> const*)"},
    {"_ZNK1AIJ1Z1Y1XEEcv1BIJDpPT_EEIJS2_S1_S0_EEEv", "A<Z, Y, X>::operator B<X*, Y*, Z*><X, Y, Z>() const"},
    {"_ZNK3Ncr6Silver7Utility6detail12CallOnThreadIZ53-[DeploymentSetupController handleManualServerEntry:]E3$_5EclIJEEEDTclclL_ZNS2_4getTIS4_EERT_vEEspclsr3stdE7forwardIT_Efp_EEEDpOSA_", "decltype(-[DeploymentSetupController handleManualServerEntry:]::$_5& Ncr::Silver::Utility::detail::getT<-[DeploymentSetupController handleManualServerEntry:]::$_5>()()()) Ncr::Silver::Utility::detail::CallOnThread<-[DeploymentSetupController handleManualServerEntry:]::$_5>::operator()<>() const"},
    {"_Zli2_xy", "operator\"\" _x(unsigned long long)"},
    {"_Z1fIiEDcT_", "decltype(auto) f<int>(int)"},
    {"_ZZ4testvEN1g3fooE5Point", "test()::g::foo(Point)"},
    {"_ZThn12_NSt9strstreamD0Ev",   "non-virtual thunk to std::strstream::~strstream()"},
    {"_ZTv0_n12_NSt9strstreamD0Ev",     "virtual thunk to std::strstream::~strstream()"},

    // NOTE: disable this test since it is a negative test case, you cannot demangle a non-mangled symbol
    // {"\x6D", nullptr}, 	// This use to crash with ASAN
    {"_ZTIU4_farrVKPi", "typeinfo for int* const volatile restrict _far"},

    // mangled names can include type manglings too, which don't start with _Z:
    {"i", "int"},

    {"PKFvRiE", "void (*)(int&) const"},
    {"PVFvRmOE", "void (*)(unsigned long&) volatile &&"},
    {"PFvRmOE", "void (*)(unsigned long&) &&"},
    {"_ZTW1x", "thread-local wrapper routine for x"},
    {"_ZTHN3fooE", "thread-local initialization routine for foo"},

    // attribute abi_tag
    {"_Z1fB3foov", "f[abi:foo]()"},
    {"_Z1fB3fooB3barv", "f[abi:foo][abi:bar]()"},
    {"_ZN1SB5outer1fB5innerEv", "S[abi:outer]::f[abi:inner]()"},
    {"_ZN1SC2B8ctor_tagEv", "S::S[abi:ctor_tag]()"},
    {"_ZplB4MERP1SS_", "operator+[abi:MERP](S, S)"},

    {"_Z1fIJifcEEvDp5unaryIT_E", "void f<int, float, char>(unary<int>, unary<float>, unary<char>)"},
    {"_Z1fIJEJiEEvDpT_DpT0_", "void f<int>(int)"},
    {"_Z1fIJicEEvDp7MuncherIAstT__S1_E", "void f<int, char>(Muncher<int [sizeof (int)]>, Muncher<char [sizeof (char)]>)"},
    {"_ZN1SIJifcEE1fIJdjEEEiDp4MerpIJifcT_EE", "int S<int, float, char>::f<double, unsigned int>(Merp<int, float, char, double>, Merp<int, float, char, unsigned int>)"},
    {"_Z1pIJicEEiDp4MerpIXsZT_EJT_EE", "int p<int, char>(Merp<sizeof...(int, char), int>, Merp<sizeof...(int, char), char>)"},

    // Some expression symbols found in clang's test/CodeGenCXX/mangle-exprs.cpp
    {"_ZN5Casts8implicitILj4EEEvPN9enable_ifIXleT_Li4EEvE4typeE", "void Casts::implicit<4u>(enable_if<(4u) <= (4), void>::type*)"},
    {"_ZN5Casts6cstyleILj4EEEvPN9enable_ifIXleT_cvjLi4EEvE4typeE", "void Casts::cstyle<4u>(enable_if<(4u) <= ((unsigned int)(4)), void>::type*)"},
    {"_ZN5Casts10functionalILj4EEEvPN9enable_ifIXleT_cvjLi4EEvE4typeE", "void Casts::functional<4u>(enable_if<(4u) <= ((unsigned int)(4)), void>::type*)"},
    {"_ZN5Casts7static_ILj4EEEvPN9enable_ifIXleT_scjLi4EEvE4typeE", "void Casts::static_<4u>(enable_if<(4u) <= (static_cast<unsigned int>(4)), void>::type*)"},
    {"_ZN5Casts12reinterpret_ILj4EiEEvPN9enable_ifIXleT_szrcPT0_Li0EEvE4typeE", "void Casts::reinterpret_<4u, int>(enable_if<(4u) <= (sizeof (reinterpret_cast<int*>(0))), void>::type*)"},
    {"_ZN5Casts6const_IiXadL_ZNS_1iEEEEEvPN9enable_ifIXleLi0EszccPT_T0_EvE4typeE", "void Casts::const_<int, &(Casts::i)>(enable_if<(0) <= (sizeof (const_cast<int*>(&(Casts::i)))), void>::type*)"},
    {"_ZN5Casts8dynamic_INS_1SEXadL_ZNS_1sEEEEEvPN9enable_ifIXleLi0EszdcPT_T0_EvE4typeE", "void Casts::dynamic_<Casts::S, &(Casts::s)>(enable_if<(0) <= (sizeof (dynamic_cast<Casts::S*>(&(Casts::s)))), void>::type*)"},
    {"_ZN5Casts1fILi6EEENS_1TIXT_EEEv", "Casts::T<6> Casts::f<6>()"},
    {"_ZN5Casts5auto_IiEEvDTnw_DapicvT__EEE", "void Casts::auto_<int>(decltype(new auto((int)())))"},
    {"_ZN5Casts7scalar_IiEEvDTcmcvT__Ecvi_EE", "void Casts::scalar_<int>(decltype(((int)()) , ((int)())))"},
    {"_ZN5test11aIsEEDTcl3foocvT__EEES1_", "decltype(foo((short)())) test1::a<short>(short)"},
    {"_ZN5test21aIPFfvEEEvT_DTclfL0p_EE", "void test2::a<float (*)()>(float (*)(), decltype(fp()))"},
    {"_ZN5test21bIPFfvEEEDTclfp_EET_", "decltype(fp()) test2::b<float (*)()>(float (*)())"},
    {"_ZN5test21cIPFfvEEEvT_PFvDTclfL1p_EEE", "void test2::c<float (*)()>(float (*)(), void (*)(decltype(fp())))"},
    {"_ZN5test21dIPFfvEEEvT_PFDTclfL0p_EEvE", "void test2::d<float (*)()>(float (*)(), decltype(fp()) (*)())"},
    {"_ZN5test21eIPFfvEEEvPFDTclfp_EET_E", "void test2::e<float (*)()>(decltype(fp()) (*)(float (*)()))"},
    {"_ZN5test21fIPFfvEEEvPFvT_DTclfL0p_EEE", "void test2::f<float (*)()>(void (*)(float (*)(), decltype(fp())))"},
    {"_ZN5test21gIPFfvEEEvT_DTclfL0p_EE", "void test2::g<float (*)()>(float (*)(), decltype(fp()))"},
    {"_ZN5test21hIPFfvEEEvT_DTcvPFDTclfL0p_EEvELi0EE", "void test2::h<float (*)()>(float (*)(), decltype((decltype(fp()) (*)())(0)))"},
    {"_ZN5test21iIPFfvEEEvDTcvPFDTclfp_EET_ELi0EE", "void test2::i<float (*)()>(decltype((decltype(fp()) (*)(float (*)()))(0)))"},
    {"_ZZN5test21gIPFfvEEEvT_DTclfL0p_EEE8variable", "void test2::g<float (*)()>(float (*)(), decltype(fp()))::variable"},
    {"_ZN5test31aINS_1XEMS1_PiEEvT_T0_DTdsfL0p_fL0p0_E", "void test3::a<test3::X, int* test3::X::*>(test3::X, int* test3::X::*, decltype(fp.*fp0))"},
    {"_ZN5test43tf1INS_1XEEEvDTnw_T_piLi1EEE", "void test4::tf1<test4::X>(decltype(new test4::X(1)))"},
    {"_ZN5test51aIiEEvDTnxcvT__EE", "void test5::a<int>(decltype(noexcept ((int)())))"},
    {"_ZN5test62f1IiEEvDTcvT_dtdtL_ZNS_1zEE2ua1iE", "void test6::f1<int>(decltype((int)(test6::z.ua.i)))"},
    {"_ZN5test62f2IiEEvDTcvT_dtdtL_ZNS_1zEE2ub1iE", "void test6::f2<int>(decltype((int)(test6::z.ub.i)))"},
    {"_ZN5test62f3IiEEvDTcvT_dtdtL_ZNS_1zEE1s1iE", "void test6::f3<int>(decltype((int)(test6::z.s.i)))"},
    {"_ZN5test62f4IiEEvDTcvT_dtdtL_ZNS_1zEE4uuss1iE", "void test6::f4<int>(decltype((int)(test6::z.uuss.i)))"},
    {"_ZN5test62f5IiEEvDTcvT_dtptL_ZNS_2zpEE2ua1iE", "void test6::f5<int>(decltype((int)(test6::zp->ua.i)))"},
    {"_ZN5test62f6IiEEvDTcvT_dtptL_ZNS_2zpEE2ub1iE", "void test6::f6<int>(decltype((int)(test6::zp->ub.i)))"},
    {"_ZN5test62f7IiEEvDTcvT_dtptL_ZNS_2zpEE1s1iE", "void test6::f7<int>(decltype((int)(test6::zp->s.i)))"},
    {"_ZN5test62f8IiEEvDTcvT_dtptL_ZNS_2zpEE4uuss1iE", "void test6::f8<int>(decltype((int)(test6::zp->uuss.i)))"},
    {"_ZN5test73fD2IiEEDTcmcvNS_1DEL_ZNS_1bEEcvT__EES2_", "decltype(((test7::D)(test7::b)) , ((int)())) test7::fD2<int>(int)"},
    {"_ZN5test73fT2IiEEDTcvT__EES1_", "decltype((int)()) test7::fT2<int>(int)"},
    {"_ZN5test73fT4IiEEDTcvT_Li1EES1_", "decltype((int)(1)) test7::fT4<int>(int)"},
    {"_ZN5test73fT6INS_1BEEEDTcvT__Li1ELi2EEES2_", "decltype((test7::B)(1, 2)) test7::fT6<test7::B>(test7::B)"},
    {"_ZNK5test81XIiE3barIiEEDTcl3fooIT_EEEv", "decltype(foo<int>()) test8::X<int>::bar<int>() const"},

    // Multiple qualifiers on the same type should all get the same entry in the substitution table.
    {"_Z1fPU3AS1KiS0_", "f(int const AS1*, int const AS1*)"},

    {"_ZN6test471fINS_1SEEEvPTsNT_1cE", "void test47::f<test47::S>(struct test47::S::c*)"},
    {"_ZN6test481fINS_1SEEEvPTuNT_1uE", "void test48::f<test48::S>(union test48::S::u*)"},
    {"_ZN6test451fINS_1SEEEvPTeNT_1eE", "void test45::f<test45::S>(enum test45::S::e*)"},

    // String literals
    // FIXME: We need to encode the string contents in order to avoid symbol collisions.
    {"_Z1fIcEvDTcv3StrIT_ELA6_KcEE", "void f<char>(decltype((Str<char>)(\"<char const [6]>\")))"},

    // Initializer list expressions
    {"_ZN5test43tf2INS_1XEEEvDTnw_T_piilLi1EEEE", "void test4::tf2<test4::X>(decltype(new test4::X({1})))"},
    {"_ZN5test73fA1IiEEDTcmtlNS_1AELi1ELi2EEcvT__EES2_", "decltype((test7::A{1, 2}) , ((int)())) test7::fA1<int>(int)"},
    {"_ZN5test73fA2IiEEDTcmcvNS_1AEilLi1ELi2EEcvT__EES2_", "decltype(((test7::A)({1, 2})) , ((int)())) test7::fA2<int>(int)"},
    {"_ZN5test73fB1IiEEDTcmtlNS_1BELi1ELi2EEcvT__EES2_", "decltype((test7::B{1, 2}) , ((int)())) test7::fB1<int>(int)"},
    {"_ZN5test73fB2IiEEDTcmcvNS_1BEilLi1ELi2EEcvT__EES2_", "decltype(((test7::B)({1, 2})) , ((int)())) test7::fB2<int>(int)"},
    {"_ZN5test73fC1IiEEDTcmtlNS_1CEilLi1ELi2EEEcvT__EES2_", "decltype((test7::C{{1, 2}}) , ((int)())) test7::fC1<int>(int)"},
    {"_ZN5test73fC2IiEEDTcmcvNS_1CEilLi1ELi2EEcvT__EES2_", "decltype(((test7::C)({1, 2})) , ((int)())) test7::fC2<int>(int)"},
    {"_ZN5test73fD1IiEEDTcmtlNS_1DEL_ZNS_1bEEEcvT__EES2_", "decltype((test7::D{test7::b}) , ((int)())) test7::fD1<int>(int)"},
    {"_ZN5test73fE1IiEEDTcmtlNS_1EELi1ELi2EEcvT__EES2_", "decltype((test7::E{1, 2}) , ((int)())) test7::fE1<int>(int)"},
    {"_ZN5test73fE2IiEEDTcmcvNS_1EEilLi1ELi2EEcvT__EES2_", "decltype(((test7::E)({1, 2})) , ((int)())) test7::fE2<int>(int)"},
    {"_ZN5test73fF1IiEEDTcmtlNS_1FEilLi1ELi2EEEcvT__EES2_", "decltype((test7::F{{1, 2}}) , ((int)())) test7::fF1<int>(int)"},
    {"_ZN5test73fF2IiEEDTcmcvNS_1FEilLi1ELi2EEcvT__EES2_", "decltype(((test7::F)({1, 2})) , ((int)())) test7::fF2<int>(int)"},
    {"_ZN5test73fT1IiEEDTtlT_EES1_", "decltype(int{}) test7::fT1<int>(int)"},
    {"_ZN5test73fT3IiEEDTtlT_Li1EEES1_", "decltype(int{1}) test7::fT3<int>(int)"},
    {"_ZN5test73fT5INS_1BEEEDTtlT_Li1ELi2EEES2_", "decltype(test7::B{1, 2}) test7::fT5<test7::B>(test7::B)"},
    {"_ZN5test73fT7INS_1AEEEDTtlT_ilEEES2_", "decltype(test7::A{{}}) test7::fT7<test7::A>(test7::A)"},
    {"_ZN5test73fT8INS_1AEEEDTcvT_ilEES2_", "decltype((test7::A)({})) test7::fT8<test7::A>(test7::A)"},
    {"_ZN5test73fT9INS_1AEEEDTtlT_ilLi1EEEES2_", "decltype(test7::A{{1}}) test7::fT9<test7::A>(test7::A)"},
    {"_ZN5test73fTAINS_1AEEEDTcvT_ilLi1EEES2_", "decltype((test7::A)({1})) test7::fTA<test7::A>(test7::A)"},
    {"_ZN5test73fTBINS_1CEEEDTtlT_ilLi1ELi2EEEES2_", "decltype(test7::C{{1, 2}}) test7::fTB<test7::C>(test7::C)"},
    {"_ZN5test73fTCINS_1CEEEDTcvT_ilLi1ELi2EEES2_", "decltype((test7::C)({1, 2})) test7::fTC<test7::C>(test7::C)"},

    // Designated init expressions
    {"_ZN15designated_init1fINS_1AEEEvDTtlT_di1adi1bdxLi3EdXLi1ELi4ELi9EEE", "void designated_init::f<designated_init::A>(decltype(designated_init::A{.a.b[3][1 ... 4] = 9}))"},
    {"_Z1fIXtl1Xdi1adi1bdxLi3ELi1EEEE", "f<X{.a.b[3] = 1}>"},

    // Inheriting constructors:
    {"_ZN1BCI21AEi", "B::B(int)"},
    {"_ZN1DCI21CIiEET_", "D::D(int)"},

    // Exception specifiers:
    {"_Z1bPDoFivE", "b(int (*)() noexcept)"},
    {"_Z1fILb0EEvPDOT_EFivE", "void f<false>(int (*)() noexcept(false))"},
    {"_Z1fILb1EEvPDOT_EFivE", "void f<true>(int (*)() noexcept(true))"},
    {"_Z1gIJEEvPDwDpT_EFivE", "void g<>(int (*)() throw())"},
    {"_Z1gIJfEEvPDwDpT_EFivE", "void g<float>(int (*)() throw(float))"},
    {"_Z1hIJEEPDwDpT_iEFivEPDwiS1_EFivE", "int (*h<>(int (*)() throw(int)))() throw(int)"},
    {"_Z1hIJfEEPDwDpT_iEFivEPDwiS1_EFivE", "int (*h<float>(int (*)() throw(int, float)))() throw(float, int)"},
    {"_Z1iIJEEPDwiDpT_EFivEPS2_", "int (*i<>(int (*)() throw(int)))() throw(int)"},
    {"_Z1iIJEEPDwiDpT_EFivES3_", "int (*i<>(int (*)() throw(int)))() throw(int)"},
    {"_Z1iIJfEEPDwiDpT_EFivEPS2_", "int (*i<float>(int (*)() throw(int, float)))() throw(int, float)"},
    {"_Z1iIJfEEPDwiDpT_EFivES3_", "int (*i<float>(int (*)() throw(int, float)))() throw(int, float)"},
    {"_Z1pILb1EEiM1SKDOT_EFivRE", "int p<true>(int (S::*)() const & noexcept(true))"},
    {"_Z1pIJicfEEiM1SVKDwDpT_EFivOE", "int p<int, char, float>(int (S::*)() const volatile && throw(int, char, float))"},
    {"_Z1pM1SDoFivE", "p(int (S::*)() noexcept)"},

    {"_ZNKR4llvm8OptionalINS_11MCFixupKindEEdeEv", "llvm::Optional<llvm::MCFixupKind>::operator*() const &"},
    {"_ZZL23isValidCoroutineContextRN5clang4SemaENS_14SourceLocationEN4llvm9StringRefEENK3$_4clEZL23isValidCoroutineContextS1_S2_S4_E15InvalidFuncDiag", "isValidCoroutineContext(clang::Sema&, clang::SourceLocation, llvm::StringRef)::$_4::operator()(isValidCoroutineContext(clang::Sema&, clang::SourceLocation, llvm::StringRef)::InvalidFuncDiag) const"},

    // ABI tags can apply to built-in substitutions.
    {"_Z1fSsB1XS_", "f(std::string[abi:X], std::string[abi:X])"},

    {"___Z10blocksNRVOv_block_invoke", "invocation function for block in blocksNRVO()"},

    // Structured bindings:
    {"_ZDC2a12a2E", "[a1, a2]"},
    {"_ZN2NSDC1x1yEE", "NS::[x, y]"},

    {"_ZGRDC1x1yE_", "reference temporary for [x, y]"},
    {"_ZGR1bIvE2_", "reference temporary for b<void>"},

    {"_ZZ18test_assign_throwsI20small_throws_on_copyLb0EEvvENKUlRNSt3__13anyEOT_E_clIRS0_EEDaS3_S5_", "auto void test_assign_throws<small_throws_on_copy, false>()::'lambda'(std::__1::any&, auto&&)::operator()<small_throws_on_copy&>(std::__1::any&, auto&&) const"},

    // enable_if attributes:
    {"_Z1fUa9enable_ifIXLi1EEEv", "f() [enable_if:1]"},
    {"_ZN5test4IdE1fEUa9enable_ifIXeqfL0p_Li1EEXeqfL0p0_Li2EEEi", "test4<double>::f(int) [enable_if:(fp) == (1), (fp0) == (2)]"},
    {"_Z3quxUa9enable_ifIXLi1EEXL_Z9TRUEFACTSEEEi", "qux(int) [enable_if:1, TRUEFACTS]"},

    // Conversion operators:
    {"_ZN5OuterI4MarpEcv7MuncherIJT_T0_DpT1_EEI4MerpS0_JicfEEEv", "Outer<Marp>::operator Muncher<Merp, Marp, int, char, float><Merp, Marp, int, char, float>()"},
    {"_ZN5OuterI4MarpEcvT_I4MerpEEv", "Outer<Marp>::operator Merp<Merp>()"},
    {"_ZZN5OuterI4MarpEcv7MuncherIJT_T0_DpT1_EEI4MerpS0_JicfEEEvEN1ScvS9_Ev", "Outer<Marp>::operator Muncher<Merp, Marp, int, char, float><Merp, Marp, int, char, float>()::S::operator Merp()"},
    {"_ZN1Scv7MuncherIJDpPT_EEIJFivEA_iEEEv", "S::operator Muncher<int (*)(), int (*) []><int (), int []>()"},

    {"_Z2f8IiJ8identityIiES0_IfEEEvRAsPiDpT0_T_DpNS3_4typeEE_i", "void f8<int, identity<int>, identity<float> >(int (&) [sizeof... (int, identity<int>, identity<float>, int, identity<int>::type, identity<float>::type)])"},

    {"_ZNK13StaticMembersIfE1xMUlvE_clEv", "StaticMembers<float>::x::'lambda'()::operator()() const"},
    {"_ZNK10inline_varMUlvE_clEv", "inline_var::'lambda'()::operator()() const"},
    // C++1z fold expressions:
    {"_Z6foldl1IJLi1ELi2ELi3EEEv1AIXfLplLi1ET_EE", "void foldl1<1, 2, 3>(A<(1 + ... + (1, 2, 3))>)"},
    {"_Z6foldr1IJLi1ELi2ELi3EEEv1AIXfRplT_Li1EEE", "void foldr1<1, 2, 3>(A<((1, 2, 3) + ... + 1)>)"},
    {"_Z5foldlIJLi1ELi2ELi3EEEv1AIXflplT_EE", "void foldl<1, 2, 3>(A<(... + (1, 2, 3))>)"},
    {"_Z5foldrIJLi1ELi2ELi3EEEv1AIXfrplT_EE", "void foldr<1, 2, 3>(A<((1, 2, 3) + ...)>)"},
    {"_ZN7PartialIJLi1ELi2EEE5foldrIJLi3ELi4EEEEv1AIXplLi1EplLi2EfRplT_plLi1EplLi2EfrplT_EE", "void Partial<1, 2>::foldr<3, 4>(A<(1) + ((2) + (((3, 4) + ... + (1) + ((2) + (((3, 4) + ...))))))>)"},

    // reference collapsing:
    {"_Z1fIR1SEiOT_", "int f<S&>(S&)"},
    {"_Z1fIJR1SS0_EEiDpOT_", "int f<S&, S>(S&, S&&)"},

    {"___Z3foo_block_invoke.25", "invocation function for block in foo"},

    // Darwin adds leading underscores to symbols, just demangle them anyways.
    {"____Z3foo_block_invoke.25", "invocation function for block in foo"},
    {"__Z1fv", "f()"},

    {"_Z2tfIZUb_E1SEiv", "int tf<'block-literal'::S>()"},

    // Vendor extension types are substitution candidates.
    {"_Z1fu3fooS_", "f(foo, foo)"},

    // alignof with type and expression, and __alignof__ with the same.
    {"_Z2f1IiEvDTatT_E", "void f1<int>(decltype(alignof (int)))"},
    {"_Z2f3IiEvDTazcvT_Li0EE", "void f3<int>(decltype(alignof ((int)(0))))"},
    {"_Z2f2IiEvDTu11__alignof__T_EE", "void f2<int>(decltype(__alignof__(int)))"},
    {"_Z2f4IiEvDTu11__alignof__XcvT_Li0EEEE", "void f4<int>(decltype(__alignof__((int)(0))))"},

    // Legacy nonstandard mangling for __uuidof.
    {"_Z15test_uuidofTypeI10TestStructEvDTu8__uuidoftT_E", "void test_uuidofType<TestStruct>(decltype(__uuidof(TestStruct)))"},
    {"_Z15test_uuidofExprI9HasMemberEvDTu8__uuidofXsrT_6memberEEE", "void test_uuidofExpr<HasMember>(decltype(__uuidof(HasMember::member)))"},
    // Current __uuidof mangling using vendor extended expression.
    {"_Z15test_uuidofTypeI10TestStructEvDTu8__uuidofT_EE", "void test_uuidofType<TestStruct>(decltype(__uuidof(TestStruct)))"},
    {"_Z15test_uuidofExprI9HasMemberEvDTu8__uuidofXsrT_6memberEEE", "void test_uuidofExpr<HasMember>(decltype(__uuidof(HasMember::member)))"},

    // C++2a char8_t:
    {"_ZTSPDu", "typeinfo name for char8_t*"},

    // C++2a lambda-expressions:
    {"_ZNK1xMUlTyT_E_clIiEEDaS_", "auto x::'lambda'<typename $T>($T)::operator()<int>(x) const"},
    {"_ZNK1xMUlTnPA3_ivE_clILS0_0EEEDav", "auto x::'lambda'<int (*$N) [3]>()::operator()<(int [3])0>() const"},
    {"_ZNK1xMUlTyTtTyTnT_TpTnPA3_TL0__ETpTyvE_clIi1XJfEEEDav", "auto x::'lambda'<typename $T, template<typename $T0, $T $N, $T0 (*...$N0) [3]> typename $TT, typename ...$T1>()::operator()<int, X, float>() const"},
    {"_ZN1AIiE1fIfEEvDTLUlTyTtTyTnTL1__ETL0_1_T_TL0__E_EE", "void A<int>::f<float>(decltype([]<typename $T, template<typename $T0, $T0 $N> typename $TT>(auto, float, $T){...}))"},
    {"_ZN1S1fILb1EEEv1XILUlvE_EE", "void S::f<true>(X<[](){...}>)"},
    {"_ZN1S1fILb1EEEv1XILUlvE0_EE", "void S::f<true>(X<[](){...}>)"},
    // FIXME: Consider special-casing the call operator of a lambda and
    // producing something like
    //   "auto inline_func()::'lambda'<int, int>(int, int) const"
    {"_ZZ11inline_funcvENKUlTyTyT_T0_E_clIiiEEDaS_S0_", "auto inline_func()::'lambda'<typename $T, typename $T0>($T, $T0)::operator()<int, int>($T, $T0) const"},
    {"_ZZ11inline_funcvENKUlTyTyT_T1_T0_E_clIiiiEEDaS_S0_S1_", "auto inline_func()::'lambda'<typename $T, typename $T0>($T, auto, $T0)::operator()<int, int, int>($T, auto, $T0) const"},
    {"_ZN1XIZ1fIiEvOT_EUlOT_DpT0_E_EclIJEEEvDpT_", "void X<void f<int>(int&&)::'lambda'(auto&&, auto...)>::operator()<>()"},
    // FIXME: This is wrong, should demangle to the same as the previous entry.
    // See https://github.com/itanium-cxx-abi/cxx-abi/issues/106.
    {"_ZN1XIZ1fIiEvOT_EUlS2_DpT0_E_EclIJEEEvDpT_", "void X<void f<int>(int&&)::'lambda'(int&&, auto...)>::operator()<>()"},
    {"_ZZZZN6abcdef9abcdefghi29abcdefabcdefabcdefabcefabcdef27xxxxxxxxxxxxxxxxxxxxxxxxxxxEN4absl8DurationERKNSt3__u12basic_stringIcNS4_11char_traitsIcEENS4_9allocatorIcEEEEPNS1_19yyyyyyyyyyyyyyyyyyyEENK3$_5clEvENKUlvE_clEvE6zzzzzz", "abcdef::abcdefghi::abcdefabcdefabcdefabcefabcdef::xxxxxxxxxxxxxxxxxxxxxxxxxxx(absl::Duration, std::__u::basic_string<char, std::__u::char_traits<char>, std::__u::allocator<char> > const&, abcdef::abcdefghi::abcdefabcdefabcdefabcefabcdef::yyyyyyyyyyyyyyyyyyy*)::$_5::operator()() const::'lambda'()::operator()() const::zzzzzz"},

    // C++2a class type non-type template parameters:
    {"_Z1fIXtl1BLPi0ELi1EEEEvv", "void f<B{(int*)0, 1}>()"},
    {"_Z1fIXtl1BLPi32EEEEvv", "void f<B{(int*)32}>()"},
    {"_Z1fIXtl1BrcPiLi0EEEEvv", "void f<B{reinterpret_cast<int*>(0)}>()"},
    {"_Z1fIXtl1BadsoiL_Z6nestedE_EEEEvv", "void f<B{&(nested.<int at offset 0>)}>()"},
    {"_Z1fIXtl1BadsoiL_Z6nestedE16_0pEEEEvv", "void f<B{&(nested.<int at offset 16>)}>()"},
    {"_Z1fIXtl2BRsoiL_Z7derivedE4EEEEvv", "void f<BR{derived.<int at offset 4>}>()"},
    {"_Z1fIXtl1BcvPiplcvPcadL_Z7derivedELl16EEEEvv", "void f<B{(int*)(((char*)(&(derived))) + (16l))}>()"},
    {"_Z1fIXtl1CadsoKiL_Z7derivedE4EEEEvv", "void f<C{&(derived.<int const at offset 4>)}>()"},
    {"_Z1fIXtl1DLM7DerivedKi0ELi1EEEEvv", "void f<D{(int const Derived::*)0, 1}>()"},
    // FIXME: This is not valid pointer-to-member syntax.
    {"_Z1fIXtl1DmcM7DerivedKiadL_ZN11MoreDerived1zEEn8EEEEvv", "void f<D{(int const Derived::*)(&(MoreDerived::z))}>()"},
    {"_Z1fIXtl1Edi1nLi42EEEEvv", "void f<E{.n = 42}>()"},
    {"_ZTAXtl1StlA32_cLc104ELc101ELc108ELc108ELc111ELc32ELc119ELc111ELc114ELc108ELc100EEEE", "template parameter object for S{char [32]{(char)104, (char)101, (char)108, (char)108, (char)111, (char)32, (char)119, (char)111, (char)114, (char)108, (char)100}}"},

    // FIXME: This is wrong; the S2_ backref should expand to OT_ and then to
    // "double&&". But we can't cope with a substitution that represents a
    // different type the node it is a substitute for.
    // See https://github.com/itanium-cxx-abi/cxx-abi/issues/106.
    {"_Z1h1XIJZ1fIiEDaOT_E1AZ1gIdEDaS2_E1BEE", "h(X<auto f<int>(int&&)::A, auto g<double>(int&&)::B>)"},

    {"_Z1fIL4Enumn1EEvv", "void f<(Enum)-1>()"},

    {"_ZN1A1gIiEEDTcldtptfpT1b1fIT_EEEv", "decltype(this->b.f<int>()) A::g<int>()"},

    // Optional template-args for vendor extended type qualifier.
    // See https://bugs.llvm.org/show_bug.cgi?id=48009.
    {"_Z3fooILi79EEbU7_ExtIntIXT_EEi", "bool foo<79>(int _ExtInt<79>)"},

    {"_ZN3xxx3yyyIvNS_1AILm0EEEZNS_2bb2cc2ddILNS_1eE1EEEvRKNS_1fERKNS_1g1hINS_1iEEERKNS_1jEfRKNS_1kEiPhEUlvE_JEEEvT1_DpT2_", "void xxx::yyy<void, xxx::A<0ul>, void xxx::bb::cc::dd<(xxx::e)1>(xxx::f const&, xxx::g::h<xxx::i> const&, xxx::j const&, float, xxx::k const&, int, unsigned char*)::'lambda'()>(void xxx::bb::cc::dd<(xxx::e)1>(xxx::f const&, xxx::g::h<xxx::i> const&, xxx::j const&, float, xxx::k const&, int, unsigned char*)::'lambda'())"},

};

const unsigned N = sizeof(cases) / sizeof(cases[0]);

struct FPLiteralCase {
    const char *mangled;
    // There are four possible demanglings of a given float.
    std::string expecting[4];
} fp_literal_cases[] =
{
    {"_ZN5test01gIfEEvRAszplcvT__ELf40a00000E_c", {
        "void test0::g<float>(char (&) [sizeof (((float)()) + (0x1.4p+2f))])",
        "void test0::g<float>(char (&) [sizeof (((float)()) + (0x2.8p+1f))])",
        "void test0::g<float>(char (&) [sizeof (((float)()) + (0x5p+0f))])",
        "void test0::g<float>(char (&) [sizeof (((float)()) + (0xap-1f))])",
    }},
    {"_ZN5test01hIfEEvRAszplcvT__ELd4014000000000000E_c", {
        "void test0::h<float>(char (&) [sizeof (((float)()) + (0x1.4p+2))])",
        "void test0::h<float>(char (&) [sizeof (((float)()) + (0x2.8p+1))])",
        "void test0::h<float>(char (&) [sizeof (((float)()) + (0x5p+0))])",
        "void test0::h<float>(char (&) [sizeof (((float)()) + (0xap-1))])",
    }},
#if LDBL_FP80
    {"_ZN5test01hIfEEvRAcvjplstT_Le4001a000000000000000E_c", {
        "void test0::h<float>(char (&) [(unsigned int)((sizeof (float)) + (0x1.4p+2L))])",
        "void test0::h<float>(char (&) [(unsigned int)((sizeof (float)) + (0x2.8p+1L))])",
        "void test0::h<float>(char (&) [(unsigned int)((sizeof (float)) + (0x5p+0L))])",
        "void test0::h<float>(char (&) [(unsigned int)((sizeof (float)) + (0xap-1L))])",
    }},
#endif
#if LDBL_FP128
    // This was found by libFuzzer+HWASan on aarch64 Android.
    {"1\006ILeeeEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE",
     {"\x6<-0x1.cecececececececececececececep+11983L>"}},
#endif
};
const unsigned NF = sizeof(fp_literal_cases) / sizeof(fp_literal_cases[0]);
const unsigned NEF = sizeof(fp_literal_cases[0].expecting) / sizeof(fp_literal_cases[0].expecting[0]);


const char* invalid_cases[] =
{
    "_ZIPPreEncode",
    "Agentt",
    "NSoERj5E=Y1[uM:ga",
    "Aon_PmKVPDk7?fg4XP5smMUL6;<WsI_mgbf23cCgsHbT<l8EE\0uVRkNOoXDrgdA4[8IU>Vl<>IL8ayHpiVDDDXTY;^o9;i",
    "_ZNSt16allocator_traitsISaIN4llvm3sys2fs18directory_iteratorEEE9constructIS3_IS3_EEEDTcl12_S_constructfp_fp0_spcl7forwardIT0_Efp1_EEERS4_PT_DpOS7_",
#if !LDBL_FP80 && __LDBL_MANT_DIG__ < 64
    "_ZN5test01hIfEEvRAcvjplstT_Le4001a000000000000000E_c",
#endif
	// The following test cases were found by libFuzzer+ASAN
    "\x44\x74\x70\x74\x71\x75\x34\x43\x41\x72\x4D\x6E\x65\x34\x9F\xC1\x43\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\x6C\x53\xF9\x5F\x70\x74\x70\x69\x45\x34\xD3\x73\x9E\x2A\x37",
    "\x4D\x41\x72\x63\x4E\x39\x44\x76\x72\x4D\x34\x44\x53\x4B\x6F\x44\x54\x6E\x61\x37\x47\x77\x78\x38\x43\x27\x41\x5F\x73\x70\x69\x45*",
    "\x41\x64\x6E\x32*",
    "\x43\x46\x41\x67\x73*",
    "\x72\x3A\x4E\x53\x64\x45\x39\x4F\x52\x4E\x1F\x43\x34\x64\x54\x5F\x49\x31\x41\x63\x6C\x37\x2A\x4D\x41\x67\x73\x76\x43\x54\x35\x5F\x49\x4B\x4C\x55\x6C\x73\x4C\x38\x64\x43\x41\x47\x4C\x5A\x28\x4F\x41\x6E\x77\x5F\x53\x6F\x70\x69\x45\x5F\x63\x47\x61\x4C\x31\x4F\x4C\x33\x3E\x41\x4C\x4B\x4C\x55\x6C\x73\x4C\x38\x64\x43\x66\x41\x47\x4C\x5A\x28\x4F\x41\x6E\x77\x5F\x53\x6F\x70\x69\x45\x5F\x37\x41*",
    "\x2D\x5F\x63\x47\x4F\x63\xD3",
    "\x44\x74\x70\x74\x71\x75\x32\x43\x41\x38\x65\x6E\x9B\x72\x4D\xC1\x43\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\xC3\x53\xF9\x5F\x70\x74\x70\x69\x45\x38\xD3\x73\x9E\x2A\x37",
    "\x4C\x5A\x4C\x55\x6C\x4D\x41\x5F\x41\x67\x74\x71\x75\x34\x4D\x41\x64\x73\x4C\x44\x76\x72\x4D\x34\x44\x4B\x44\x54\x6E\x61\x37\x47\x77\x78\x38\x43\x27\x41\x5F\x73\x70\x69\x45\x6D\x73\x72\x53\x41\x6F\x41\x7B",
    "\x44\x74\x70\x74\x71\x75\x32\x43\x41\x38\x65\x6E\x9B\x72\x4D\xC1\x43\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\x2C\x53\xF9\x5F\x70\x74\x70\x69\x45\xB4\xD3\x73\x9F\x2A\x37",
    "\x4C\x5A\x4C\x55\x6C\x69\x4D\x73\x72\x53\x6F\x7A\x41\x5F\x41\x67\x74\x71\x75\x32\x4D\x41\x64\x73\x39\x28\x76\x72\x4D\x34\x44\x4B\x45\x54\x6E\x61\x37\x47\x77\x78\x38\x43\x27\x41\x5F\x73\x70\x69\x45\x6F\x45\x49\x6D\x1A\x4C\x53\x38\x6A\x7A\x5A",
    "\x44\x74\x63*",
    "\x44\x74\x71\x75\x35\x2A\xDF\x74\x44\x61\x73\x63\x35\x2A\x3B\x41\x72\x4D\x6E\x65\x34\x9F\xC1\x63\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\x6C\x53\xF9\x5F\x70\x74\x70\x69\x45\x33\x44\x76\x35",
    "\x44\x74\x70\x74\x71\x75\x32\x43\x41\x38\x65\x6E\x9B\x72\x4D\xC1\x43\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\x6C\x53\xF9\x5F\x70\x74\x70\x69\x45\x38\xD3\x73\x9E\x2A\x37",
    "\x46\x44\x74\x70\x74\x71\x75\x32\x43\x41\x72\x4D\x6E\x65\x34\x9F\xC1\x43\x41\x72\x4D\x6E\x77\x38\x9A\x8E\x44\x6F\x64\x6C\x53\xF9\x5F\x70\x74\x70\x69\x45\x34\xD3\x73\x9E\x2A\x37\x72\x33\x8E\x3A\x29\x8E\x44\x35",
    "_ZcvCiIJEEDvT__FFFFT_vT_v",
    "Z1JIJ1_T_EE3o00EUlT_E0",
    "___Z2i_D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D",
    "ZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_ZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_ZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_Dv_Dv_Dv_Dv_dZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_ZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_ZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIZcvSdIDv_Dv_Dv_Dv_Dv_d",
    "Z1 Z1 IJEEAcvZcvT_EcvT_T_",
    "T_IZaaIJEEAnaaaT_T__",
    "PT_IJPNT_IJEET_T_T_T_)J)JKE",
    "1 IJEVNT_T_T_EE",
    "AT__ZSiIJEEAnwscT_T__",
    "FSiIJEENT_IoE ",
    "ZTVSiIZTVSiIZTVSiIZTVSiINIJEET_T_T_T_T_ ",
    "Ana_T_E_T_IJEffffffffffffffersfffffrsrsffffffbgE",
};

const unsigned NI = sizeof(invalid_cases) / sizeof(invalid_cases[0]);

void test()
{
    std::size_t len = 0;
    char* buf = nullptr;
    bool failed = false;
    for (unsigned i = 0; i < N; ++i)
    {
        int status;
        char* demang = __cxxabiv1::__cxa_demangle(cases[i][0], buf, &len, &status);
        if (demang == 0 || std::strcmp(demang, cases[i][1]) != 0)
        {
            std::printf("ERROR demangling %s\nexpected: %s\n", cases[i][0], cases[i][1]);
            if (demang)
            {
                std::printf(" reality: %s\n", demang);
                buf = demang;
                failed = true;
            }
            else
            {
                std::printf("Got instead: NULL, %d\n", status);
                failed = true;
            }
        }
        else
        {
            buf = demang;
        }
    }
    assert(!failed);
    free(buf);
}

void test_invalid_cases()
{
    std::size_t len = 0;
    char* buf = nullptr;
    for (unsigned i = 0; i < NI; ++i)
    {
        int status;
        char* demang = __cxxabiv1::__cxa_demangle(invalid_cases[i], buf, &len, &status);
        if (status != -2)
        {
            std::printf("%s should be invalid but is not\n", invalid_cases[i]);
            std::printf("Got status %d\n", status);
            assert(status == -2);
        }
        else
        {
            buf = demang;
        }
    }
    free(buf);
}

const char *xfail_cases[] = {
    "_ZW6FooBarE2f3v", // C++ modules TS

    // FIXME: Why does clang generate the "cp" expr?
    "_ZN5test11bIsEEDTcp3foocvT__EEES1_",
};

const size_t num_xfails = sizeof(xfail_cases) / sizeof(xfail_cases[0]);

void test_xfail_cases()
{
    std::size_t len = 0;
    char* buf = nullptr;
    for (std::size_t i = 0; i < num_xfails; ++i)
    {
        int status;
        char* demang = __cxxabiv1::__cxa_demangle(xfail_cases[i], buf, &len, &status);
        if (status != -2)
        {
            std::printf("%s was documented as xfail but passed\n", xfail_cases[i]);
            std::printf("Got status = %d\n", status);
            assert(status == -2);
        }
        else
        {
            buf = demang;
        }
    }
    free(buf);
}

void testFPLiterals()
{
    std::size_t len = 0;
    char* buf = nullptr;
    for (unsigned i = 0; i < NF; ++i)
    {
        FPLiteralCase *fpCase = fp_literal_cases+i;
        int status;
        char* demang = __cxxabiv1::__cxa_demangle(fpCase->mangled, buf, &len, &status);
        if (demang == 0)
        {
            std::printf("%s -> %s\n", fpCase->mangled, fpCase->expecting[0].c_str());
            std::printf("Got instead: NULL, %d\n", status);
            assert(false);
            continue;
        }
        std::string *e_beg = fpCase->expecting;
        std::string *e_end = fpCase->expecting + NEF;
        if (std::find(e_beg, e_end, demang) == e_end)
        {
            std::printf("%s -> %s\n", fpCase->mangled, fpCase->expecting[0].c_str());
            std::printf("Got instead: %s\n", demang);
            assert(false);
            continue;
        }
        buf = demang;
    }
    free(buf);
}

int main(int, char**)
{
    std::printf("Testing %d symbols.\n", N);
    {
        timer t;
        test();
        test_invalid_cases();
        test_xfail_cases();
        testFPLiterals();
    }
#if 0
    std::string input;
    while (std::cin)
    {
        std::getline(std::cin, input);
        if (std::cin.fail())
            break;
        std::size_t len = 0;
        int status;
        len = 0;
        char* demang = abi::__cxa_demangle(input.c_str(), 0, &len, &status);
        switch (status)
        {
        case -3:
            std::cout << "Invalid arguments\n";
            break;
        case -2:
            std::cout << "Invalid mangled name\n";
            break;
        case -1:
            std::cout << "memory allocation failure\n";
            break;
        case 0:
            std::cout << "len = " << len << '\n';
            std::cout << demang << '\n';
            std::free(demang);
            break;
        case 1:
            std::cout << "not implemented\n";
            break;
        }
        std::cout << '\n';
    }
#endif

    return 0;
}
